<p><-- Begin file 19 of 26:  Letter S (Version 0.48) 
        
           This file is part 19 of the GNU version of
     The Collaborative International Dictionary of English
               Also referred to as GCIDE
  * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * 

GCIDE is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2, or (at your option)
any later version.

GCIDE is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this copy of GCIDE; see the file COPYING.  If not, write 
to the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
Boston, MA 02111-1307, USA.
           * * * * * * * * * * * * * * * * * * * * * 

           This dictionary was derived from the
         Webster's Revised Unabridged Dictionary
                 Version published 1913
               by the  C. & G. Merriam Co.
                   Springfield, Mass.
                 Under the direction of
                Noah Porter, D.D., LL.D.

                        and from
           WordNet, a semantic network created by
              the Cognitive Science Department
                 of Princeton University
                  under the direction of
                   Prof. George Miller

             and is being updated and supplemented by
         an open coalition of volunteer collaborators from
                       around the world.

     This electronic dictionary is the starting point for an 
ongoing project to develop a modern on-line comprehensive encyclopedic
dictionary, by the efforts of all individuals willing to help build a
large and freely available knowledge base.  Contributions of data,
time, and effort are requested from any person willing to assist creation
of a comprehensive and organized knowledge base for free access on the
internet.  Anyone willing to assist in any way in constructing such a
knowledge base should contact:

     Patrick Cassidy          pc@worldsoul.org
     735 Belvidere Ave.       Office: (908)668-5252
     Plainfield, NJ 07062
     (908) 561-3416

    Last edit October 6, 2002.

 --></p>

<p><centered><point26>S.</point26></centered></p>

<p><ent>S</ent><br/
<hw>S</hw> <pr>(<ecr/s)</pr>, <def>the nineteenth letter of the English alphabet, is a consonant, and is often called a sibilant, in allusion to its hissing sound. It has two principal sounds; one a mere hissing, as in <xex>sack</xex>, <xex>this</xex>; the other a vocal hissing (the same as that of <xex>z</xex>), as in <xex>is</xex>, <xex>wise</xex>. Besides these it sometimes has the sounds of <xex>sh</xex> and <xex>zh</xex>, as in <xex>sure</xex>, <xex>measure</xex>. It generally has its hissing sound at the beginning of words, but in the middle and at the end of words its sound is determined by usage. In a few words it is silent, as in <xex>isle</xex>, <xex>d<eacute/bris</xex>.  With the letter <xex>h</xex> it forms the digraph <xex>sh</xex>.  See <xex>Guide to pronunciation</xex>, <sect/<sect/ 255-261.</def><br/
[<source>1913 Webster</source>]</p>

<p><note>   Both the form and the name of the letter S are derived from the Latin, which got the letter through the Greek from the Phoenician. The ultimate origin is Egyptian. S is etymologically most nearly related to <xex>c</xex>, <xex>z</xex>, <xex>t</xex>, and <xex>r</xex>; as, in i<xex>c</xex>e, OE. i<xex>s</xex>; E. hen<xex>c</xex>e, OE. henne<xex>s</xex>; E. ra<xex>s</xex>e, ra<xex>z</xex>e; era<xex>s</xex>e, ra<xex>z</xex>or; tha<xex>t</xex>, G. da<xex>s</xex>; E. rea<xex>s</xex>on, F. rai<xex>s</xex>on,  L. ra<xex>t</xex>io; E. wa<xex>s</xex>, we<xex>r</xex>e; chai<xex>r</xex>, chai<xex>s</xex>e (see C, Z, T, and R.).</note><br/
[<source>1913 Webster</source>]</p>

<p><ent>-s</ent><br/
<hw>-s</hw>. <sn>1.</sn> <ety>[OE. <ets>es</ets>, AS. <ets>as</ets>.]</ety> <def>The suffix used to form the plural of most words; as in road<xex>s</xex>, elf<xex>s</xex>, side<xex>s</xex>, account<xex>s</xex>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <ety>[OE. <ets>-s</ets>, for older <ets>-th</ets>, AS. <ets>-<edh/</ets>.]</ety> <def>The suffix used to form the third person singular indicative of English verbs; as in fall<xex>s</xex>, tell<xex>s</xex>, send<xex>s</xex>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>An adverbial suffix; as in toward<xex>s</xex>, need<xex>s</xex>, alway<xex>s</xex>, -- originally the genitive, possesive, ending. See <er>-'s</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>-'s</ent><br/
<hw>-'s</hw> <ety>[OE. <ets>-es</ets>, AS. <ets>-es</ets>.]</ety> The suffix used to form the possessive singular of nouns; <as>as, boy<ex>'s</ex>; man<ex>'s</ex></as>.<br/
[<source>1913 Webster</source>]</p>

<p><ent>'s</ent><br/
<hw>'s</hw>. <def>A contraction for <xex>is</xex> or (colloquially) for <xex>has</xex>.</def> <ldquo/My heart<xex>'s</xex> subdued.<rdquo/ <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Saadh</ent><br/
<hw>Sa"adh</hw> <pr>(s<aum/"<adot/d)</pr>, <pos>n.</pos> <def>See <er>Sadh</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Saan</ent><br/
<hw>Saan</hw> <pr>(s<aum/n)</pr>, <pos>n. pl.</pos> <fld>(Ethnol.)</fld> <def>Same as <er>Bushmen</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sabadilla</ent><br/
<hw>Sab`a*dil"la</hw> <pr>(s<acr/b`<adot/*d<icr/l"l<adot/)</pr>, <pos>n.</pos> <ety>[Sp. <ets>cebadilla</ets>.]</ety> <fld>(Bot.)</fld> <def>A Mexican liliaceous plant (<spn>Schoenocaulon officinale</spn>); also, its seeds, which contain the alkaloid <prod>veratrine</prod>.  It was formerly used in medicine as an emetic and purgative.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sabaean</ent><br/
<hw>Sa*bae"an</hw> <pr>(?)</pr>, <pos>a. & n.</pos> <def>Same as <er>Sabian</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sabaeanism</ent><br/
<hw>Sa*bae"an*ism</hw> <pr>(?)</pr>, <pos>n.</pos> <def>Same as <er>Sabianism</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sabaism</ent><br/
<ent>Sabaeism</ent><br/
<mhw>{ <hw>Sa"bae*ism</hw> <pr>(?)</pr>, <hw>Sa"ba*ism</hw> <pr>(?)</pr>, }</mhw> <pos>n.</pos> <def>See <er>Sabianism</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sabal</ent><br/
<hw>Sa"bal</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Bot.)</fld> <def>A genus of palm trees including the palmetto of the Southern United States.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sabaoth</ent><br/
<hw>Sab"a*oth</hw> <pr>(s<acr/b"<asl/*<ocr/th <it>or</it> s<adot/"b<amac/*<ocr/th; 277)</pr>, <pos>n. pl.</pos> <ety>[Heb. <ets>tseb<amac/'<omac/th</ets>, pl. of <ets>ts<amac/b<amac/'</ets>, an army or host, fr. <ets>ts<amac/b<amac/'</ets>, to go forth to war.]</ety> <sn>1.</sn> <def>Armies; hosts.</def> <note>[Used twice in the English Bible, in the phrase <ldquo/The Lord of Sabaoth.<rdquo/]</note><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Incorrectly, the Sabbath.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sabbat</ent><br/
<hw>Sab"bat</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[See <er>Sabbath</er>.]</ety> <def>In mediaeval demonology, the nocturnal assembly in which demons and sorcerers were thought to celebrate their orgies.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sabbatarian</ent><br/
<hw>Sab`ba*ta"ri*an</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>Sabbatarius</ets>: cf. F. <ets>sabbataire</ets>. See <er>Sabbath</er>.]</ety> <sn>1.</sn> <def>One who regards and keeps the seventh day of the week as holy, agreeably to the letter of the fourth commandment in the Decalogue.</def><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ There were Christians in the early church who held this opinion, and certain Christians, esp. the <stype>Seventh-day Baptists</stype>, hold it now.</note><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A strict observer of the Sabbath.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sabbatarian</ent><br/
<hw>Sab`ba*ta"ri*an</hw>, <pos>a.</pos> <def>Of or pertaining to the Sabbath, or the tenets of Sabbatarians.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sabbatarianism</ent><br/
<hw>Sab`ba*ta"ri*an*ism</hw> <pr>(?)</pr>, <pos>n.</pos> <def>The tenets of Sabbatarians.</def>  <rj><au>Bp. Ward (1673).</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sabbath</ent><br/
<hw>Sab"bath</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[OE. <ets>sabat</ets>, <ets>sabbat</ets>, F. <ets>sabbat</ets>, L. <ets>sabbatum</ets>, Gr. <grk>sa`bbaton</grk>, fr. Heb. <ets>shabb<amac/th</ets>, fr. <ets>sh<amac/bath</ets> to rest from labor.  Cf. <er>Sabbat</er>.]</ety> <sn>1.</sn> <def>A season or day of rest; one day in seven appointed for rest or worship, the observance of which was enjoined upon the Jews in the Decalogue, and has been continued by the Christian church with a transference of the day observed from the last to the first day of the week, which is called also <altname>Lord's Day</altname>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Remember the <qex>sabbath</qex> day, to keep it holy.</q> <rj><qau>Ex. xx. 8.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>The seventh year, observed among the Israelites as one of rest and festival.</def>  <rj><au>Lev. xxv. 4.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>Fig.: A time of rest or repose; intermission of pain, effort, sorrow, or the like.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Peaceful sleep out the <qex>sabbath</qex> of the tomb.</q> <rj><qau>Pope.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Sabbath breaker</b></col>, <cd>one who violates the law of the Sabbath.</cd> -- <col><b>Sabbath breaking</b></col>, <cd>the violation of the law of the Sabbath.</cd> -- <col><b>Sabbath-day's journey</b></col>, <cd>a distance of about a mile, which, under Rabbinical law, the Jews were allowed to travel on the Sabbath.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><syn><b>Syn.</b> -- <er>Sabbath</er>, <er>Sunday</er>.</syn> <usage> <xex>Sabbath</xex> is not strictly synonymous with <xex>Sunday</xex>. <xex>Sabbath</xex> denotes the institution; <xex>Sunday</xex> is the name of the first day of the week. The <xex>Sabbath</xex> of the Jews is on <xex>Saturday</xex>, and the <xex>Sabbath</xex> of most Christians on <xex>Sunday</xex>. In New England, the first day of the week has been called <ldquo/<xex>the Sabbath</xex>,<rdquo/ to mark it as holy time; <xex>Sunday</xex> is the word more commonly used, at present, in all parts of the United States, as it is in England. <ldquo/So if we will be the children of our heavenly Father, we must be careful to keep the Christian <xex>Sabbath</xex> day, which is the <xex>Sunday</xex>.<rdquo/ <au>Homilies.</au></usage><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sabbathless</ent><br/
<hw>Sab"bath*less</hw>, <pos>a.</pos> <def>Without Sabbath, or intermission of labor; hence, without respite or rest.</def>  <rj><au>Bacon.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sabbatical</ent><br/
<ent>Sabbatic</ent><br/
<mhw>{ <hw>Sab*bat"ic</hw> <pr>(?)</pr>, <hw>Sab*bat"ic*al</hw> <pr>(?)</pr>, }</mhw> <pos>a.</pos> <ety>[Gr. <?/: cf. F. <ets>sabbatique</ets>.]</ety> <def>Of or pertaining to the Sabbath; resembling the Sabbath; enjoying or bringing an intermission of labor.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Sabbatical year</b></col> <fld>(Jewish Antiq.)</fld>, <cd>every seventh year, in which the Israelites were commanded to suffer their fields and vineyards to rest, or lie without tillage.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sabbatism</ent><br/
<hw>Sab"ba*tism</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>sabbatismus</ets>, Gr. <?/, fr. <?/ to keep the Sabbath: cf. F. <ets>sabbatisme</ets>. See <er>Sabbath</er>.]</ety> <def>Intermission of labor, as upon the Sabbath; rest.</def> <rj><au>Dr. H. More.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sabbaton</ent><br/
<hw>Sab"ba*ton</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Cf. Sp. <ets>zapaton</ets>, a large shoe, F. <ets>sabot</ets> a wooden shoe.]</ety> <def>A round-toed, armed covering for the feet, worn during a part of the sixteenth century in both military and civil dress.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sabean</ent><br/
<hw>Sa*be"an</hw> <pr>(?)</pr>, <pos>a. & n.</pos> <def>Same as <er>Sabian</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sabeism</ent><br/
<hw>Sa"be*ism</hw> <pr>(?)</pr>, <pos>n.</pos> <def>Same as <er>Sabianism</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sabella</ent><br/
||<hw>Sa*bel"la</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[NL., fr. L. <ets>sabulum</ets> gravel.]</ety> <fld>(Zool.)</fld> <def>A genus of tubicolous annelids having a circle of plumose gills around the head.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sabellian</ent><br/
<hw>Sa*bel"li*an</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Pertaining to the doctrines or tenets of Sabellius. See <er>Sabellian</er>, <pos>n.</pos></def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sabellian</ent><br/
<hw>Sa*bel"li*an</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Eccl. Hist.)</fld> <def>A follower of <etsep>Sabellius</etsep>, a presbyter of Ptolemais in the third century, who maintained that there is but one person in the Godhead, and that the Son and Holy Spirit are only different powers, operations, or offices of the one God the Father.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sabellianism</ent><br/
<hw>Sa*bel"li*an*ism</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Eccl.)</fld> <def>The doctrines or tenets of Sabellius. See <er>Sabellian</er>, <pos>n.</pos></def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sabelloid</ent><br/
<hw>Sa*bel"loid</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[<ets>Sabella</ets> + <ets>-oid</ets>.]</ety> <fld>(Zool.)</fld> <def>Like, or related to, the genus Sabella.</def> -- <wordforms><wf>Sa*bel"loid</wf>, <pos>n.</pos></wordforms><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sabre</ent><br/
<ent>Saber</ent><br/
<mhw>{ <hw>Sa"ber</hw>, <hw>Sa"bre</hw> }</mhw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F. <ets>sabre</ets>, G. <ets>s<aum/bel</ets>; of uncertain origin; cf. Hung. <ets>sz<aacute/blya</ets>, Pol. <ets>szabla</ets>, Russ. <ets>sabla</ets>, and L. Gr. <grk>zabo`s</grk> crooked, curved.]</ety> <def>A sword with a broad and heavy blade, thick at the back, and usually more or less curved like a scimiter; a cavalry sword.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><mcol><col><b>Saber fish</b></col>, <it>or</it> <col><b>Sabre fish</b></col></mcol> <fld>(Zool.)</fld>, <cd>the cutlass fish.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sabre</ent><br/
<ent>Saber</ent><br/
<mhw>{ <hw>Sa"ber</hw>, <hw>Sa"bre</hw>  }</mhw>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Sabered</conjf> <pr>(?)</pr> or <conjf>Sabred</conjf> <pr>(<?/)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Sabering</conjf> or <conjf>Sabring</conjf> <pr>(<?/)</pr>.]</vmorph> <ety>[Cf. F. <ets>sabrer</ets>.]</ety> <def>To strike, cut, or kill with a saber; to cut down, as with a saber.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>You send troops to <qex>saber</qex> and bayonet us into submission.</q> <rj><qau>Burke.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sabrebill</ent><br/
<ent>Saberbill</ent><br/
<mhw>{ <hw>Sa"ber*bill`</hw>, <hw>Sa"bre*bill`</hw>  }</mhw>, <pos>n.</pos> <fld>(Zool.)</fld> <def>The curlew.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sabian</ent><br/
<hw>Sa"bi*an</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>Sabaeus</ets>.]</ety> <altsp>[Written also <asp>Sabean</asp>, and <asp>Sabaean</asp>.]</altsp> <sn>1.</sn> <def>Of or pertaining to Saba in Arabia, celebrated for producing aromatic plants.</def><-- #err An error in the original dictionary here: it says "Written also ... Sabaeanism"
  PCP --><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Relating to the religion of Saba, or to the worship of the heavenly bodies.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sabian</ent><br/
<hw>Sa"bi*an</hw>, <pos>n.</pos> <def>An adherent of the Sabian religion; a worshiper of the heavenly bodies.</def> <altsp>[Written also <asp>Sabaean</asp>, and <asp>Sabean</asp>.]</altsp><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sabianism</ent><br/
<hw>Sa"bi*an*ism</hw> <pr>(?)</pr>, <pos>n.</pos> <def>The doctrine of the Sabians; the Sabian religion; that species of idolatry which consists in worshiping the sun, moon, and stars; heliolatry.</def> <altsp>[Written also <asp>Sabaeanism</asp>.]</altsp><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sabicu</ent><br/
||<hw>Sab"i*cu</hw> <pr>(?)</pr>, <pos>n.</pos> <def>The very hard wood of a leguminous West Indian tree (<spn>Lysiloma Sabicu</spn>), valued for shipbuilding.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sabine</ent><br/
<hw>Sa"bine</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>Sabinus</ets>.]</ety> <def>Of or pertaining to the ancient Sabines, a people of Italy.</def> -- <def2><pos>n.</pos> <def>One of the Sabine people.</def></def2><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sabine</ent><br/
<hw>Sab"ine</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F., fr. L. <ets>Sabina herba</ets>, fr. <ets>Sabini</ets> the Sabines.  Cf. <er>Savin</er>.]</ety> <fld>(Bot.)</fld> <def>See <er>Savin</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sable</ent><br/
<hw>Sa"ble</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[OF. <ets>sable</ets>, F. <ets>zibeline</ets> sable (in sense 4), LL. <ets>sabellum</ets>; cf. D. <ets>sabel</ets>, Dan. <ets>sabel</ets>, <ets>zobel</ets>, Sw. <ets>sabel</ets>, <ets>sobel</ets>, G. <ets>zobel</ets>; all fr. Russ. <ets>s<oacute/bole</ets>.]</ety> <sn>1.</sn> <fld>(Zool.)</fld> <def>A carnivorous animal of the Weasel family (<spn>Mustela zibellina</spn>) native of the northern latitudes of Europe, Asia, and America, -- noted for its fine, soft, and valuable fur.</def><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ The sable resembles the marten, but has a longer head and ears. Its fur consists of a soft under wool, with a dense coat of hair, overtopped by another still longer. It varies greatly in color and quality according to the locality and the season of the year.  The darkest and most valuable furs are taken in autumn and winter in the colder parts of Siberia, Russia, and British North America.</note><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ The American sable, or marten, was formerly considered a distinct species (<spn>Mustela Americana</spn>), but it differs very little from the Asiatic sable, and is now considered only a geographical variety.</note><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>The fur of the sable.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>A mourning garment; a funeral robe; -- generally in the plural.</def> <ldquo/<xex>Sables</xex> wove by destiny.<rdquo/  <rj><au>Young.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <fld>(Her.)</fld> <def>The tincture black; -- represented by vertical and horizontal lines crossing each other.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sable</ent><br/
<hw>Sa"ble</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Of the color of the sable's fur; dark; black; -- used chiefly in poetry.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Night, <qex>sable</qex> goddess! from her ebon throne,<br/
In rayless majesty, now stretches forth<br/
Her leaden scepter o'er a slumbering world.</q> <rj><qau>Young.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Sable antelope</b></col> <fld>(Zool.)</fld>, <cd>a large South African antelope (<spn>Hippotragus niger</spn>). Both sexes have long, sharp horns. The adult male is black; the female is dark chestnut above, white beneath.</cd> -- <col><b>Sable iron</b></col>, <cd>a superior quality of Russia iron; -- so called because originally stamped with the figure of a sable.</cd> -- <col><b>Sable mouse</b></col> <fld>(Zool.)</fld>, <cd>the lemming.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sable</ent><br/
<hw>Sa"ble</hw>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Sabled</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Sabling</conjf> <pr>(?)</pr>.]</vmorph> <def>To render sable or dark; to drape darkly or in black.</def><br/
[<source>1913 Webster</source>]</p>

<p><q><qex>Sabled</qex> all in black the shady sky.</q> <rj><qau>G. Fletcher.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sabot</ent><br/
||<hw>Sa`bot"</hw> <pr>(s<adot/`b<omac/")</pr>, <pos>n.</pos> <ety>[F.]</ety> <sn>1.</sn> <def>A kind of wooden shoe worn by the peasantry in France, Belgium, Sweden, and some other European countries.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Mil.)</fld> <def>A thick, circular disk of wood, to which the cartridge bag and projectile are attached, in fixed ammunition for cannon; also, a piece of soft metal attached to a projectile to take the groove of the rifling.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sabotage</ent><br/
||<hw>Sa`bo`tage"</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F.]</ety> <sn>1.</sn> <sd>(a)</sd> <def>Scamped work.</def> <sd>(b)</sd> <def>Malicious waste or destruction of an employer's property or injury to his interests by workmen during labor troubles.</def></p>

<p><sn>2.</sn> <def>any surreptitious destruction of property or obstruction of activity by persons not known to be hostile; -- in war, such actions carried out behind enemy lines by agents or local sympathisers of the hostile power.</def><br/
[<source>PJC</source>]</p>

<p><ent>Sabotiere</ent><br/
||<hw>Sa`bo"ti<egrave/re</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F.]</ety> <def>A kind of freezer for ices.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sabre</ent><br/
<hw>Sa"bre</hw> <pr>(?)</pr>, <pos>n. & v.</pos> <def>See <er>Saber</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sabretasche</ent><br/
||<hw>Sa"bre*tasche`</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F. <ets>sabretache</ets>, G. <ets>s<aum/beltasche</ets>; <ets>s<aum/bel</ets> saber + <ets>tasche</ets> a pocket.]</ety> <fld>(Mil.)</fld> <def>A leather case or pocket worn by cavalry at the left side, suspended from the sword belt.</def>  <rj><au>Campbell (Dict. Mil. Sci.).</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sabrina work</ent><br/
<hw>Sa*bri"na work`</hw> <pr>(?)</pr>. <def>A variety of appliqu<eacute/ work for quilts, table covers, etc.</def>  <rj><au>Caulfeild & S. (Dict. of Needlework).</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sabulose</ent><br/
<hw>Sab"u*lose</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>sabulosus</ets>, from <ets>sabulum</ets>, <ets>sabulo</ets>, sand.]</ety> <fld>(Bot.)</fld> <def>Growing in sandy places.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sabulosity</ent><br/
<hw>Sab`u*los"i*ty</hw> <pr>(?)</pr>, <pos>n.</pos> <def>The quality of being sabulous; sandiness; grittiness.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sabulous</ent><br/
<hw>Sab"u*lous</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>sabulosus</ets>.]</ety> <def>Sandy; gritty.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sac</ent><br/
<hw>Sac</hw> <pr>(s<add/k)</pr>, <pos>n.</pos> <fld>(Ethnol.)</fld> <def>See <er>Sacs</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sac</ent><br/
<hw>Sac</hw>, <pos>n.</pos> <ety>[See <er>Sake</er>, <er>Soc</er>.]</ety> <fld>(O.Eng. Law)</fld> <def>The privilege formerly enjoyed by the lord of a manor, of holding courts, trying causes, and imposing fines.</def>  <rj><au>Cowell.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sac</ent><br/
<hw>Sac</hw> <pr>(s<acr/k)</pr>, <pos>n.</pos> <ety>[F., fr. L. <ets>saccus</ets> a sack. See <er>Sack</er> a bag.]</ety> <sn>1.</sn> <def>See 2d <er>Sack</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Biol.)</fld> <def>A cavity, bag, or receptacle, usually containing fluid, and either closed, or opening into another cavity to the exterior; a sack.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sacalait</ent><br/
<hw>Sac"a*lait</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Zool.)</fld> <def>A kind of fresh-water bass; the crappie.</def> <mark>[Southern U.S.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sacar</ent><br/
<hw>Sa"car</hw> <pr>(?)</pr>, <pos>n.</pos> <def>See <er>Saker</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Saccade</ent><br/
<hw>Sac*cade"</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F.]</ety> <fld>(Man.)</fld> <def>A sudden, violent check of a horse by drawing or twitching the reins on a sudden and with one pull.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Saccate</ent><br/
<hw>Sac"cate</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[NL. <ets>saccatus</ets>, fr. L. <ets>saccus</ets> a sack, bag.]</ety> <sn>1.</sn> <fld>(Biol.)</fld> <def>Having the form of a sack or pouch; furnished with a sack or pouch, as a petal.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Zool.)</fld> <def>Of or pertaining to the <ord>Saccata</ord>, a suborder of ctenophores having two pouches into which the long tentacles can be retracted.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Saccharate</ent><br/
<hw>Sac"cha*rate</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Chem.)</fld> <sd>(a)</sd> <def>A salt of saccharic acid.</def> <sd>(b)</sd> <def>In a wider sense, a compound of saccharose, or any similar carbohydrate, with such bases as the oxides of calcium, barium, or lead; a sucrate.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Saccharic</ent><br/
<hw>Sac*char"ic</hw> <pr>(?)</pr>, <pos>a.</pos> <fld>(Chem.)</fld> <def>Of, pertaining to, or obtained from, saccharine substances; specifically, designating an acid obtained, as a white amorphous gummy mass, by the oxidation of mannite, glucose, sucrose, etc.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sacchariferous</ent><br/
<hw>Sac`cha*rif"er*ous</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>saccharon</ets> sugar + <ets>-ferous</ets>.]</ety> <def>Producing sugar; <as>as, <ex>sacchariferous</ex> canes</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Saccharify</ent><br/
<hw>Sac*char"i*fy</hw> <pr>(?)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Saccharified</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Saccharifying</conjf> <pr>(?)</pr>.]</vmorph> <ety>[L. <ets>saccharon</ets> sugar + <ets>-fy</ets>: cf. F. <ets>saccharifier</ets>.]</ety> <def>To convert into, or to impregnate with, sugar.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Saccharilla</ent><br/
<hw>Sac`cha*ril"la</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A kind of muslin.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Saccharimeter</ent><br/
<hw>Sac`cha*rim"e*ter</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>saccharon</ets> sugar + <ets>-meter</ets>: cf. F. <ets>saccharim<egrave/tre</ets>.]</ety> <def>An instrument for ascertaining the quantity of saccharine matter in any solution, as the juice of a plant, or brewers' and distillers' worts.</def> <altsp>[Written also <asp>saccharometer</asp>.]</altsp><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ The <xex>common saccharimeter</xex> of the brewer is an hydrometer adapted by its scale to point out the proportion of saccharine matter in a solution of any specific gravity. The <xex>polarizing saccharimeter</xex> of the chemist is a complex optical apparatus, in which polarized light is transmitted through the saccharine solution, and the proportion of sugar indicated by the relative deviation of the plane of polarization.</note><br/
[<source>1913 Webster</source>]</p>

<p><ent>Saccharimetrical</ent><br/
<hw>Sac`cha*ri*met"ric*al</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Of or pertaining to saccharimetry; obtained by saccharimetry.</def><br/
[<source>1913 Webster</source>]</p>

<p><-- p. 1265 pr=PCP --></p>

<p><ent>Saccharimetry</ent><br/
<hw>Sac`cha*rim"e*try</hw> <pr>(s<acr/k`k<adot/*r<icr/m"<esl/*tr<ycr/)</pr>, <pos>n.</pos> <def>The act, process or method of determining the amount and kind of sugar present in sirup, molasses, and the like, especially by the employment of polarizing apparatus.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Saccharin</ent><br/
<hw>Sac"cha*rin</hw> <pr>(s<acr/k"k<adot/*r<icr/n)</pr>, <pos>n.</pos> <ety>[F., from L. <ets>saccharon</ets> sugar.]</ety> <fld>(Chem.)</fld> <def>A bitter white crystalline substance obtained from the saccharinates and regarded as the lactone of saccharinic acid; -- so called because formerly supposed to be isomeric with cane sugar (<xex>saccharose</xex>).</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Saccharinate</ent><br/
<hw>Sac"cha*ri*nate</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Chem.)</fld> <sd>(a)</sd> <def>A salt of saccharinic acid.</def> <sd>(b)</sd> <def>A salt of saccharine.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Saccharine</ent><br/
<hw>Sac"cha*rine</hw> <pr>(? <or/ ?)</pr>, <pos>a.</pos> <ety>[F. <ets>saccharin</ets>, fr. L. <ets>saccharon</ets> sugar, Gr. <?/, <?/, <?/, Skr. <ets><cced/arkara</ets>. Cf. <er>Sugar</er>.]</ety> <def>Of or pertaining to sugar; having the qualities of sugar; producing sugar; sweet; <as>as, a <ex>saccharine</ex> taste; <ex>saccharine</ex> matter</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Saccharine</ent><br/
<hw>Sac"cha*rine</hw> <pr>(? <or/ ?)</pr>, <pos>n.</pos> <fld>(Chem.)</fld> <def>A trade name for benzoic sulphinide.</def> <altsp>[Written also <asp>saccharin</asp>.]</altsp> <-- A synthetic sweetening agent used (in the form of the sodium salt) as a non-caloric sweetening agent, to avoid gaining weight or for medical purposes. Benzoic sulfimide, <chform>C7H5NO3S</chform>. --><br/
[<source>1913 Webster</source>]</p>

<p><ent>Saccharinic</ent><br/
<hw>Sac"cha*rin"ic</hw> <pr>(?)</pr>, <pos>a.</pos> <fld>(Chem.)</fld> <def>Of, pertaining to, or derived from, saccharin; specifically, designating a complex acid not known in the free state but well known in its salts, which are obtained by boiling dextrose and levulose (invert sugar) with milk of lime.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Saccharize</ent><br/
<hw>Sac"cha*rize</hw> <pr>(?)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Saccharized</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Saccharizing</conjf> <pr>(?)</pr>.]</vmorph> <def>To convert into, or to impregnate with, sugar.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Saccharoidal</ent><br/
<ent>Saccharoid</ent><br/
<mhw>{ <hw>Sac"cha*roid</hw> <pr>(?)</pr>, <hw>Sac`cha*roid"al</hw> <pr>(?)</pr>, }</mhw> <pos>a.</pos> <ety>[L. <ets>saccharon</ets> sugar + <ets>-oid</ets>: cf. F. <ets>saccharo<ium/de</ets>.]</ety> <def>Resembling sugar, as in taste, appearance, consistency, or composition; <as>as, <ex>saccharoidal</ex> limestone</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Saccharometer</ent><br/
<hw>Sac`cha*rom"e*ter</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A saccharimeter.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Saccharomyces</ent><br/
||<hw>Sac`cha*ro*my"ces</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[NL., fr. Gr. <?/ sugar + <?/, <?/, a fungus.]</ety> <fld>(Biol.)</fld> <def>A genus of budding fungi, the various species of which have the power, to a greater or less extent, or splitting up sugar into alcohol and carbonic acid. They are the active agents in producing fermentation of wine, beer, etc. <spn>Saccharomyces cerevisiae</spn> is the yeast of sedimentary beer. Also called <altname>Torula</altname>.</def><-- Brewers' yeast is Saccharomyces cerevisiae. --><br/
[<source>1913 Webster</source>]</p>

<p><ent>Saccharomycetes</ent><br/
||<hw>Sac`cha*ro*my*ce"tes</hw> <pr>(?)</pr>, <pos>n. pl.</pos> <fld>(Biol.)</fld> <def>A family of fungi consisting of the one genus Saccharomyces.</def> <br/
[<source>1913 Webster</source>]</p>

<p><ent>Saccharonate</ent><br/
<hw>Sac"cha*ro*nate</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Chem.)</fld> <def>A salt of saccharonic acid.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Saccharone</ent><br/
<hw>Sac"cha*rone</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[<ets>Sacchar</ets>in + lact<ets>one</ets>.]</ety> <fld>(Chem.)</fld> <sd>(a)</sd> <def>A white crystalline substance, <chform>C6H8O6</chform>, obtained by the oxidation of saccharin, and regarded as the lactone of saccharonic acid.</def> <sd>(b)</sd> <def>An oily liquid, <chform>C6H10O2</chform>, obtained by the reduction of saccharin.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Saccharonic</ent><br/
<hw>Sac`cha*ron"ic</hw> <pr>(?)</pr>, <pos>a.</pos> <fld>(Chem.)</fld> <def>Of, pertaining to, or derived from, saccharone; specifically, designating an unstable acid which is obtained from saccharone <sd>(a)</sd> by hydration, and forms a well-known series of salts.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Saccharose</ent><br/
<hw>Sac"cha*rose`</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Chem.)</fld> <def>Cane sugar; sucrose; also, in general, any one of the group of which saccharose, or sucrose proper, is the type. See <er>Sucrose</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Saccharous</ent><br/
<hw>Sac"cha*rous</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Saccharine.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Saccharum</ent><br/
||<hw>Sac"cha*rum</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[NL. See <er>Saccharine</er>.]</ety> <fld>(Bot.)</fld> <def>A genus of tall tropical grasses including the sugar cane.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Saccholactate</ent><br/
<hw>Sac`cho*lac"tate</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[See <er>Saccholactic</er>.]</ety> <fld>(Chem.)</fld> <def>A salt of saccholactic acid; -- formerly called also <altname>saccholate</altname>.</def> <mark>[Obs.]</mark> <see>See <er>Mucate</er>.</see><br/
[<source>1913 Webster</source>]</p>

<p><ent>Saccholactic</ent><br/
<hw>Sac`cho*lac"tic</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>saccharon</ets> sugar + <ets>lac</ets>, <ets>lactis</ets>, milk.]</ety> <fld>(Chem.)</fld> <def>Of, pertaining to, or designating, an acid now called <xex>mucic acid</xex>; saccholic.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Saccholic</ent><br/
<hw>Sac*chol"ic</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Saccholactic.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sacchulmate</ent><br/
<hw>Sac*chul"mate</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Chem.)</fld> <def>A salt of sacchulmic acid.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sacchulmic</ent><br/
<hw>Sac*chul"mic</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[<ets>Sacch</ets>arine + <ets>ulmic</ets>.]</ety> <fld>(Chem.)</fld> <def>Of, pertaining to, or designating, an acid obtained as a dark amorphous substance by the long-continued boiling of sucrose with very dilute sulphuric acid. It resembles humic acid.</def> <altsp>[Written also <asp>sacculmic</asp>.]</altsp><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sacchulmin</ent><br/
<hw>Sac*chul"min</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Chem.)</fld> <def>An amorphous huminlike substance resembling sacchulmic acid, and produced together with it.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sacciferous</ent><br/
<hw>Sac*cif"er*ous</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>saccus</ets> a sack + <ets>-ferous</ets>.]</ety> <fld>(Biol.)</fld> <def>Bearing a sac.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sacciform</ent><br/
<hw>Sac"ci*form</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>saccus</ets> a sack + <ets>-form</ets>.]</ety> <fld>(Biol.)</fld> <def>Having the general form of a sac.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Saccoglossa</ent><br/
||<hw>Sac`co*glos"sa</hw> <pr>(?)</pr>, <pos>n. pl.</pos> <ety>[NL., fr. L. <ets>saccus</ets> a sack + Gr. <?/ a tongue.]</ety> <fld>(Zool.)</fld> <def>Same as <er>Pellibranchiata</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Saccular</ent><br/
<hw>Sac"cu*lar</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Like a sac; sacciform.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sacculated</ent><br/
<hw>Sac"cu*la`ted</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Furnished with little sacs.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Saccule</ent><br/
<hw>Sac"cule</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>sacculus</ets>, dim. of <ets>saccus</ets> sack.]</ety> <def>A little sac; specifically, the sacculus of the ear.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sacculo-cochlear</ent><br/
<hw>Sac`cu*lo-coch"le*ar</hw> <pr>(?)</pr>, <pos>a.</pos> <fld>(Anat.)</fld> <def>Pertaining to the sacculus and cochlea of the ear.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sacculo-utricular</ent><br/
<hw>Sac`cu*lo-u*tric"u*lar</hw> <pr>(?)</pr>, <pos>a.</pos> <fld>(Anat.)</fld> <def>Pertaining to the sacculus and utriculus of the ear.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sacculus</ent><br/
||<hw>Sac"cu*lus</hw> <pr>(?)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Sacculi</plw> <pr>(#)</pr>.</plu> <ety>[L., little sack.]</ety> <fld>(Anat.)</fld> <def>A little sac; esp., a part of the membranous labyrinth of the ear.</def> <see>See the Note under <er>Ear</er>.</see> <br/
[<source>1913 Webster</source>]</p>

<p><ent>Saccus</ent><br/
||<hw>Sac"cus</hw> <pr>(?)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Sacci</plw> <pr>(#)</pr>.</plu> <ety>[L., a sack.]</ety> <fld>(Biol.)</fld> <def>A sac.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sacellum</ent><br/
<hw>Sa*cel"lum</hw> <pr>(?)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Sacella</plw> <pr>(#)</pr>.</plu> <ety>[L., dim. of <ets>sacrum</ets> a sacred place.]</ety> <sd>(a)</sd> <fld>(Rom. Antiq.)</fld> <def>An unroofed space consecrated to a divinity.</def> <sd>(b)</sd> <fld>(Eccl.)</fld> <def>A small monumental chapel in a church.</def> <rj><au>Shipley.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sacerdotal</ent><br/
<hw>Sac`er*do"tal</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>sacerdotalis</ets>, fr. <ets>sacerdos</ets>, <ets>-otis</ets>, a priest, fr. <ets>sacer</ets> holy, sacred: cf. F. <ets>sacerdotal</ets>.]</ety> <def>Of or pertaining to priests, or to the order of priests; relating to the priesthood; priesty; <as>as, <ex>sacerdotal</ex> dignity; <ex>sacerdotal</ex> functions</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The ascendency of the <qex>sacerdotal</qex> order was long the ascendency which naturally and properly belongs to intellectual superiority.</q> <rj><qau>Macaulay.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sacerdotalism</ent><br/
<hw>Sac`er*do"tal*ism</hw> <pr>(?)</pr>, <pos>n.</pos> <def>The system, style, spirit, or character, of a priesthood, or sacerdotal order; devotion to the interests of the sacerdotal order.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sacerdotally</ent><br/
<hw>Sac`er*do"tal*ly</hw>, <pos>adv.</pos> <def>In a sacerdotal manner.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sachel</ent><br/
<hw>Sach"el</hw> <pr>(s<acr/ch"<ecr/l)</pr>, <pos>n.</pos> <def>A small bag.</def> <see>See <er>Satchel</er>.</see><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sachem</ent><br/
<hw>Sa"chem</hw> <pr>(s<acr/"ch<eit/m)</pr>, <pos>n.</pos> <def>A chief of a tribe of the American Indians; a sagamore.</def> <see>See <er>Sagamore</er>.</see><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sachemdom</ent><br/
<hw>Sa"chem*dom</hw> <pr>(-d<ucr/m)</pr>, <pos>n.</pos> <def>The government or jurisdiction of a sachem.</def> <rj><au>Dr. T. Dwight.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sachemship</ent><br/
<hw>Sa"chem*ship</hw>, <pos>n.</pos> <def>Office or condition of a sachem.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sachet</ent><br/
||<hw>Sa`chet"</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F., dim. of <ets>sac</ets>. See <er>Sac</er>.]</ety> <def>A scent bag, or perfume cushion, to be laid among handkerchiefs, garments, etc., to perfume them.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Saciety</ent><br/
<hw>Sa*ci"e*ty</hw> <pr>(?)</pr>, <pos>n.</pos> <def>Satiety.</def> <mark>[Obs.]</mark> <rj><au>Bacon.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sack</ent><br/
<hw>Sack</hw> <pr>(s<acr/k)</pr>, <pos>n.</pos> <ety>[OE. <ets>seck</ets>, F. <ets>sec</ets> dry (cf. Sp. <ets>seco</ets>, It. <ets>secco</ets>), from L. <ets>siccus</ets> dry, harsh; perhaps akin to Gr. <grk>'ischno`s</grk>, Skr. <ets>sikata</ets> sand, Ir. <ets>sesc</ets> dry, W. <ets>hysp</ets>. Cf. <er>Desiccate</er>.]</ety> <def>A name formerly given to various dry Spanish wines.</def> <ldquo/Sherris <xex>sack</xex>.<rdquo/ <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Sack posset</b></col>, <cd>a posset made of sack, and some other ingredients.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sack</ent><br/
<hw>Sack</hw>, <pos>n.</pos> <ety>[OE. <ets>sak</ets>, <ets>sek</ets>, AS. <ets>sacc</ets>, <ets>saecc</ets>, L. <ets>saccus</ets>, Gr. <grk>sa`kkos</grk> from Heb. <ets>sak</ets>; cf. F. <ets>sac</ets>, from the Latin. Cf. <er>Sac</er>, <er>Satchel</er>, <er>Sack</er> to plunder.]</ety> <sn>1.</sn> <def>A bag for holding and carrying goods of any kind; a receptacle made of some kind of pliable material, as cloth, leather, and the like; a large pouch.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A measure of varying capacity, according to local usage and the substance. The American <xex>sack</xex> of salt is 215 pounds; the <xex>sack</xex> of wheat, two bushels.</def> <rj><au>McElrath.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <ety>[Perhaps a different word.]</ety> <def>Originally, a loosely hanging garment for women, worn like a cloak about the shoulders, and serving as a decorative appendage to the gown; now, an outer garment with sleeves, worn by women; <as>as, a dressing <ex>sack</ex></as>.</def> <altsp>[Written also <asp>sacque</asp>.]</altsp><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>A sack coat; a kind of coat worn by men, and extending from top to bottom without a cross seam.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <fld>(Biol.)</fld> <def>See 2d <er>Sac</er>, 2.</def><br/
[<source>1913 Webster</source>]</p>

<p><-- <sn>6.</sn> <def>Bed.</def> <mark>[Colloq.]</mark> --><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Sack bearer</b></col> <fld>(Zool.)</fld>. <cd>See <cref>Basket worm</cref>, under <er>Basket</er>.</cd> -- <col><b>Sack tree</b></col> <fld>(Bot.)</fld>, <cd>an East Indian tree (<spn>Antiaris saccidora</spn>) which is cut into lengths, and made into sacks by turning the bark inside out, and leaving a slice of the wood for a bottom.</cd> -- <mcol><col><b>To give the sack to</b></col> <it>or</it> <col><b>get the sack</b></col></mcol>, <cd>to discharge, or be discharged, from employment; to jilt, or be jilted.</cd> <mark>[Slang]</mark>  --  <col><b>To hit the sack</b></col>, <cd>to go to bed.</cd> <mark>[Slang]</mark></cs> <br/
[<source>1913 Webster</source> <source>+PJC</source>]</p>

<p><ent>Sack</ent><br/
<hw>Sack</hw>, <pos>v. t.</pos> <sn>1.</sn> <def>To put in a sack; to bag; <as>as, to <ex>sack</ex> corn</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Bolsters <qex>sacked</qex> in cloth, blue and crimson.</q> <rj><qau>L. Wallace.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To bear or carry in a sack upon the back or the shoulders.</def> <mark>[Colloq.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sack</ent><br/
<hw>Sack</hw>, <pos>n.</pos> <ety>[F. <ets>sac</ets> plunder, pillage, originally, a pack, packet, booty packed up, fr. L. <ets>saccus</ets>. See <er>Sack</er> a bag.]</ety> <def>The pillage or plunder, as of a town or city; the storm and plunder of a town; devastation; ravage.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The town was stormed, and delivered up to <qex>sack</qex>, -- by which phrase is to be understood the perpetration of all those outrages which the ruthless code of war allowed, in that age, on the persons and property of the defenseless inhabitants, without regard to sex or age.</q> <rj><qau>Prescott.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sack</ent><br/
<hw>Sack</hw>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Sacked</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Sacking</conjf>.]</vmorph> <ety>[See <er>Sack</er> pillage.]</ety> <def>To plunder or pillage, as a town or city; to devastate; to ravage.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The Romans lay under the apprehensions of seeing their city <qex>sacked</qex> by a barbarous enemy.</q> <rj><qau>Addison.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sackage</ent><br/
<hw>Sack"age</hw> <pr>(?; 48)</pr>, <pos>n.</pos> <def>The act of taking by storm and pillaging; sack.</def> <mark>[R.]</mark> <rj><au>H. Roscoe.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sackbut</ent><br/
<hw>Sack"but</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F. <ets>saquebute</ets>, OF. <ets>saqueboute</ets> a sackbut, earlier, a sort of hook attached to the end of a lance used by foot soldiers to unhorse cavalrymen; prop. meaning, pull and push; fr. <ets>saquier</ets>, <ets>sachier</ets>, to pull, draw (perhaps originally, to put into a bag or take out from a bag; see <er>Sack</er> a bag) + <ets>bouter</ets> to push (see <er>Butt</er> to thrust). The name was given to the musical instrument from its being lengthened and shortened.]</ety> <fld>(Mus.)</fld> <def>A brass wind instrument, like a bass trumpet, so contrived that it can be lengthened or shortened according to the tone required; -- said to be the same as the trombone.</def> <altsp>[Written also <asp>sagbut</asp>.]</altsp> <rj><au>Moore (Encyc. of Music).</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ The <xex>sackbut</xex> of the Scriptures is supposed to have been a stringed instrument.</note><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sackcloth</ent><br/
<hw>Sack"cloth`</hw> <pr>(?; 115)</pr>, <pos>n.</pos> <def>Linen or cotton cloth such as sacks are made of; coarse cloth; anciently, a cloth or garment worn in mourning, distress, mortification, or penitence.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Gird you with <qex>sackcloth</qex>, and mourn before Abner.</q> <rj><qau>2 Sam. iii. 31.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Thus with <qex>sackcloth</qex> I invest my woe.</q> <rj><qau>Sandys.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sackclothed</ent><br/
<hw>Sack"clothed`</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Clothed in sackcloth.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sacker</ent><br/
<hw>Sack"er</hw> <pr>(?)</pr>, <pos>n.</pos> <def>One who sacks; one who takes part in the storm and pillage of a town.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sackful</ent><br/
<hw>Sack"ful</hw> <pr>(?)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Sackfuls</plw> <pr>(<?/)</pr>.</plu> <def>As much as a sack will hold.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sackful</ent><br/
<hw>Sack"ful</hw>, <pos>a.</pos> <def>Bent on plunder.</def> <mark>[Obs.]</mark> <rj><au>Chapman.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sacking</ent><br/
<hw>Sack"ing</hw>, <pos>n.</pos> <ety>[AS. <ets>saeccing</ets>, from <ets>saecc</ets> sack, bag.]</ety> <def>Stout, coarse cloth of which sacks, bags, etc., are made.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sackless</ent><br/
<hw>Sack"less</hw>, <pos>a.</pos> <ety>[AS. <ets>sacle<aacute/s</ets>; <ets>sacu</ets> contention + <ets>le<aacute/s</ets> loose, free from.]</ety> <def>Quiet; peaceable; harmless; innocent.</def> <mark>[Obs. or Prov. Eng.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sack-winged</ent><br/
<hw>Sack"-winged`</hw> <pr>(?)</pr>, <pos>a.</pos> <fld>(Zool.)</fld> <def>Having a peculiar pouch developed near the front edge of the wing; -- said of certain bats of the genus <gen>Saccopteryx</gen>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sacque</ent><br/
<hw>Sacque</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Formed after the analogy of the French. See 2d <er>Sack</er>.]</ety> <def>Same as 2d <er>Sack</er>, 3.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sacral</ent><br/
<hw>Sa"cral</hw> <pr>(?)</pr>, <pos>a.</pos> <fld>(Anat.)</fld> <def>Of or pertaining to the sacrum; in the region of the sacrum.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sacrament</ent><br/
<hw>Sac"ra*ment</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>sacramentum</ets> an oath, a sacred thing, a mystery, a sacrament, fr. <ets>sacrare</ets> to declare as sacred, <ets>sacer</ets> sacred: cf. F. <ets>sacrement</ets>. See <er>Sacred</er>.]</ety> <sn>1.</sn> <def>The oath of allegiance taken by Roman soldiers; hence, a sacred ceremony used to impress an obligation; a solemn oath-taking; an oath.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>I'll take the <qex>sacrament</qex> on't.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>The pledge or token of an oath or solemn covenant; a sacred thing; a mystery.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>God sometimes sent a light of fire, and pillar of a cloud . . . and the <qex>sacrament</qex> of a rainbow, to guide his people through their portion of sorrows.</q> <rj><qau>Jer. Taylor.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <fld>(Theol.)</fld> <def>One of the solemn religious ordinances enjoined by Christ, the head of the Christian church, to be observed by his followers; hence, specifically, the eucharist; the Lord's Supper.</def><br/
[<source>1913 Webster</source>]</p>

<p><syn><b>Syn.</b> -- <er>Sacrament</er>, <er>Eucharist</er>.</syn> -- <usage>Protestants apply the term <xex>sacrament</xex> to baptism and the Lord's Supper, especially the latter. The R. Cath. and Greek churches have five other sacraments, viz., confirmation, penance, holy orders, matrimony, and extreme unction. As <xex>sacrament</xex> denotes an oath or vow, the word has been applied by way of emphasis to the Lord's Supper, where the most sacred vows are renewed by the Christian in commemorating the death of his Redeemer. <xex>Eucharist</xex> denotes the <xex>giving of thanks</xex>; and this term also has been applied to the same ordinance, as expressing the grateful remembrance of Christ's sufferings and death. <ldquo/Some receive the <xex>sacrament</xex> as a means to procure great graces and blessings; others as an <xex>eucharist</xex> and an office of thanksgiving for what they have received.<rdquo/</usage> <rj><au>Jer. Taylor.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sacrament</ent><br/
<hw>Sac"ra*ment</hw> <pr>(?)</pr>, <pos>v. t.</pos> <def>To bind by an oath.</def> <mark>[Obs.]</mark> <rj><au>Laud.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sacramental</ent><br/
<hw>Sac`ra*men"tal</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>sacramentalis</ets>: cf. F. <ets>sacramental</ets>, <ets>sacramentel</ets>.]</ety> <sn>1.</sn> <def>Of or pertaining to a sacrament or the sacraments; of the nature of a sacrament; sacredly or solemnly binding; <as>as, <ex>sacramental</ex> rites or elements</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Bound by a sacrament.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The <qex>sacramental</qex> host of God's elect.</q> <rj><qau>Cowper.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sacramental</ent><br/
<hw>Sac`ra*men"tal</hw>, <pos>n.</pos> <def>That which relates to a sacrament.</def> <rj><au>Bp. Morton.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sacramentalism</ent><br/
<hw>Sac`ra*men"tal*ism</hw> <pr>(?)</pr>, <pos>n.</pos> <def>The doctrine and use of sacraments; attachment of excessive importance to sacraments.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sacramentalist</ent><br/
<hw>Sac`ra*men"tal*ist</hw>, <pos>n.</pos> <def>One who holds the doctrine of the real objective presence of Christ's body and blood in the holy eucharist.</def> <rj><au>Shipley.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sacramentally</ent><br/
<hw>Sac`ra*men"tal*ly</hw>, <pos>adv.</pos> <def>In a sacramental manner.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sacramentarian</ent><br/
<hw>Sac`ra*men*ta"ri*an</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[LL. <ets>sacramentarius</ets>: cf. F. <ets>sacramentaire</ets>.]</ety> <sn>1.</sn> <fld>(Eccl.)</fld> <def>A name given in the sixteenth century to those German reformers who rejected both the Roman and the Lutheran doctrine of the holy eucharist.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>One who holds extreme opinions regarding the efficacy of sacraments.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sacramentarian</ent><br/
<hw>Sac`ra*men*ta"ri*an</hw>, <pos>a.</pos> <sn>1.</sn> <def>Of or pertaining a sacrament, or to the sacramentals; sacramental.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Of or pertaining to the Sacramentarians.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sacramentary</ent><br/
<hw>Sac`ra*men"ta*ry</hw> <pr>(?)</pr>, <pos>a.</pos> <sn>1.</sn> <def>Of or pertaining to a sacrament or the sacraments; sacramental.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Of or pertaining to the Sacramentarians.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sacramentary</ent><br/
<hw>Sac`ra*men"ta*ry</hw>, <pos>n.</pos>; <plu><it>pl.</it> <plw>-ries</plw> <pr>(#)</pr>.</plu> <ety>[LL. <ets>sacramentarium</ets>: cf. F. <ets>sacramentaire</ets>.]</ety> <sn>1.</sn> <def>An ancient book of the Roman Catholic Church, written by Pope Gelasius, and revised, corrected, and abridged by St. Gregory, in which were contained the rites for Mass, the sacraments, the dedication of churches, and other ceremonies. There are several ancient books of the same kind in France and Germany.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Same as <er>Sacramentarian</er>, <pos>n.</pos>, 1.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Papists, Anabaptists, and <qex>Sacramentaries</qex>.</q> <rj><qau>Jer. Taylor.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sacramentize</ent><br/
<hw>Sac"ra*ment*ize</hw> <pr>(?)</pr>, <pos>v. i.</pos> <def>To administer the sacraments.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>Both to preach and <qex>sacramentize</qex>.</q> <rj><qau>Fuller.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sacrarium</ent><br/
||<hw>Sa*cra"ri*um</hw> <pr>(?)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>-ria</plw> <pr>(#)</pr>.</plu> <ety>[L., fr. <ets>sacer</ets> sacred.]</ety> <sn>1.</sn> <def>A sort of family chapel in the houses of the Romans, devoted to a special divinity.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>The adytum of a temple.</def> <rj><au>Gwilt.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>In a Christian church, the sanctuary.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sacrate</ent><br/
<hw>Sa"crate</hw> <pr>(?)</pr>, <pos>v. t.</pos> <ety>[L. <ets>sacratus</ets>, p. p. of <ets>sacrare</ets>. See <er>Sacred</er>.]</ety> <def>To consecrate.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sacration</ent><br/
<hw>Sa*cra"tion</hw> <pr>(?)</pr>, <pos>n.</pos> <def>Consecration.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sacre</ent><br/
<hw>Sa"cre</hw> <pr>(?)</pr>, <pos>n.</pos> <def>See <er>Saker</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sacre</ent><br/
<hw>Sa"cre</hw>, <pos>v. t.</pos> <ety>[F. <ets>sacrer</ets>. See <er>Sacred</er>.]</ety> <def>To consecrate; to make sacred.</def> <mark>[Obs.]</mark> <rj><au>Holland.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sacred</ent><br/
<hw>Sa"cred</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[Originally p. p. of OE. <ets>sacren</ets> to consecrate, F. <ets>sacrer</ets>, fr. L. <ets>sacrare</ets>, fr. <ets>sacer</ets> sacred, holy, cursed. Cf. <er>Consecrate</er>, <er>Execrate</er>, <er>Saint</er>, <er>Sexton</er>.]</ety> <sn>1.</sn> <def>Set apart by solemn religious ceremony; especially, in a good sense, made holy; set apart to religious use; consecrated; not profane or common; <as>as, a <ex>sacred</ex> place; a <ex>sacred</ex> day; <ex>sacred</ex> service</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Relating to religion, or to the services of religion; not secular; religious; <as>as, <ex>sacred</ex> history</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Smit with the love of <qex>sacred</qex> song.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>Designated or exalted by a divine sanction; possessing the highest title to obedience, honor, reverence, or veneration; entitled to extreme reverence; venerable.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Such neighbor nearness to our <qex>sacred</qex> [royal] blood<br/
Should nothing privilege him.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Poet and saint to thee alone were given,<br/
The two most <qex>sacred</qex> names of earth and heaven.</q> <rj><qau>Cowley.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>Hence, not to be profaned or violated; inviolable.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Secrets of marriage still are <qex>sacred</qex> held.</q> <rj><qau>Dryden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <def>Consecrated; dedicated; devoted; -- with <xex>to</xex>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>A temple, <qex>sacred</qex> to the queen of love.</q> <rj><qau>Dryden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>6.</sn> <def>Solemnly devoted, in a bad sense, as to evil, vengeance, curse, or the like; accursed; baleful.</def> <mark>[Archaic]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>But, to destruction <qex>sacred</qex> and devote.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><-- p. 1266 pr=PCP --></p>

<p><cs><col><b>Society of the Sacred Heart</b></col> <fld>(R.C. Ch.)</fld>, <cd>a religious order of women, founded in France in 1800, and approved in 1826. It was introduced into America in 1817. The members of the order devote themselves to the higher branches of female education.</cd> -- <col><b>Sacred baboon</b></col>. <fld>(Zool.)</fld> <cd>See <er>Hamadryas</er>.</cd> -- <col><b>Sacred bean</b></col> <fld>(Bot.)</fld>, <cd>a seed of the Oriental lotus (<spn>Nelumbo speciosa</spn> or <spn>Nelumbium speciosum</spn>), a plant resembling a water lily; also, the plant itself. See <er>Lotus</er>.</cd> -- <col><b>Sacred beetle</b></col> <fld>(Zool.)</fld> <cd>See <er>Scarab</er>.</cd> -- <col><b>Sacred canon</b></col>. <cd>See <er>Canon</er>, <pos>n.</pos>, 3.</cd> -- <col><b>Sacred fish</b></col> <fld>(Zool.)</fld>, <cd>any one of numerous species of fresh-water African fishes of the family <fam>Mormyridae</fam>. Several large species inhabit the Nile and were considered sacred by the ancient Egyptians; especially <spn>Mormyrus oxyrhynchus</spn>.</cd> -- <col><b>Sacred ibis</b></col>. <cd>See <er>Ibis</er>.</cd> -- <col><b>Sacred monkey</b></col>. <fld>(Zool.)</fld> <sd>(a)</sd> <cd>Any Asiatic monkey of the genus <gen>Semnopithecus</gen>, regarded as sacred by the Hindoos; especially, the entellus. See <er>Entellus</er>.</cd> <sd>(b)</sd> <cd>The sacred baboon.</cd> See <er>Hamadryas</er>. <sd>(c)</sd> <cd>The bhunder, or rhesus monkey.</cd> -- <col><b>Sacred place</b></col> <fld>(Civil Law)</fld>, <cd>the place where a deceased person is buried.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><syn><b>Syn.</b> -- Holy; divine; hallowed; consecrated; dedicated; devoted; religious; venerable; reverend.</syn><br/
[<source>1913 Webster</source>]</p>

<p> -- <wordforms><wf>Sa"cred*ly</wf> <pr>(#)</pr>, <pos>adv.</pos> -- <wf>Sa"cred*ness</wf>, <pos>n.</pos></wordforms><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sacrifical</ent><br/
<ent>Sacrific</ent><br/
<mhw>{ <hw>Sacrif"ic</hw> <pr>(?)</pr>, <hw>Sa*crif"ic*al</hw> <pr>(?)</pr>, }</mhw> <pos>a.</pos> <ety>[L. <ets>sacrificus</ets>, <ets>sacrificalis</ets>. See <er>Sacrifice</er>.]</ety> <def>Employed in sacrifice.</def> <mark>[R.]</mark> <rj><au>Johnson.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sacrificable</ent><br/
<hw>Sa*crif"ic*a*ble</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Capable of being offered in sacrifice.</def> <mark>[R.]</mark> <rj><au>Sir T. Browne.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sacrificant</ent><br/
<hw>Sa*crif"ic*ant</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>sacrificans</ets>, p. pr. See <er>Sacrifice</er>.]</ety> <def>One who offers a sacrifice.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sacrificator</ent><br/
<hw>Sac"ri*fi*ca`tor</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L.]</ety> <def>A sacrificer; one who offers a sacrifice.</def> <mark>[R.]</mark> <rj><au>Sir T. Browne.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sacrificatory</ent><br/
<hw>Sa*crif"ic*a*to*ry</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Cf. F. <ets>sacrificatoire</ets>.]</ety> <def>Offering sacrifice.</def> <mark>[R.]</mark> <rj><au>Sherwood.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sacrifice</ent><br/
<hw>Sac"ri*fice</hw> <pr>(?; 277)</pr>, <pos>n.</pos> <ety>[OE. <ets>sacrifise</ets>, <ets>sacrifice</ets>, F. <ets>sacrifice</ets>, fr. L. <ets>sacrificium</ets>; <ets>sacer</ets> sacred + <ets>facere</ets> to make. See <er>Sacred</er>, and <er>Fact</er>.]</ety> <sn>1.</sn> <def>The offering of anything to God, or to a god; consecratory rite.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Great pomp, and <qex>sacrifice</qex>, and praises loud,<br/
To Dagon.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Anything consecrated and offered to God, or to a divinity; an immolated victim, or an offering of any kind, laid upon an altar, or otherwise presented in the way of religious thanksgiving, atonement, or conciliation.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Moloch, horrid king, besmeared with blood<br/
Of human <qex>sacrifice</qex>.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>My life, if thou preserv'st my life,<br/
Thy <qex>sacrifice</qex> shall be.</q> <rj><qau>Addison.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>Destruction or surrender of anything for the sake of something else; devotion of some desirable object in behalf of a higher object, or to a claim deemed more pressing; hence, also, the thing so devoted or given up; <as>as, the <ex>sacrifice</ex> of interest to pleasure, or of pleasure to interest</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>A sale at a price less than the cost or the actual value.</def> <mark>[Tradesmen's Cant]</mark><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Burnt sacrifice</b></col>. <cd>See <cref>Burnt offering</cref>, under <er>Burnt</er>.</cd> -- <col><b>Sacrifice hit</b></col> <fld>(Baseball)</fld>, <cd>in batting, a hit of such a kind that the batter loses his chance of tallying, but enables one or more who are on bases to get home or gain a base.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sacrifice</ent><br/
<hw>Sac"ri*fice</hw> <pr>(?; 277)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Sacrificed</conjf> <pr>(<?/)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Sacrificing</conjf> <pr>(<?/)</pr>.]</vmorph> <ety>[From <er>Sacrifice</er>, <pos>n.</pos>: cf. F. <ets>sacrifier</ets>, L. <ets>sacrificare</ets>; <ets>sacer</ets> sacred, holy + <ets>-ficare</ets> (only in comp.) to make. See <er>-fy</er>.]</ety> <sn>1.</sn> <def>To make an offering of; to consecrate or present to a divinity by way of expiation or propitiation, or as a token acknowledgment or thanksgiving; to immolate on the altar of God, in order to atone for sin, to procure favor, or to express thankfulness; <as>as, to <ex>sacrifice</ex> an ox or a sheep</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Oft <qex>sacrificing</qex> bullock, lamb, or kid.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Hence, to destroy, surrender, or suffer to be lost, for the sake of obtaining something; to give up in favor of a higher or more imperative object or duty; to devote, with loss or suffering.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Condemned to <qex>sacrifice</qex> his childish years<br/
To babbling ignorance, and to empty fears.</q> <rj><qau>Prior.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>The Baronet had <qex>sacrificed</qex> a large sum . . . for the sake of . . . making this boy his heir.</q> <rj><qau>G. Eliot.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>To destroy; to kill.</def> <rj><au>Johnson.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>To sell at a price less than the cost or the actual value.</def> <mark>[Tradesmen's Cant]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sacrifice</ent><br/
<hw>Sac"ri*fice</hw>, <pos>v. i.</pos> <def>To make offerings to God, or to a deity, of things consumed on the altar; to offer sacrifice.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>O teacher, some great mischief hath befallen<br/
To that meek man, who well had <qex>sacrificed</qex>.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sacrificer</ent><br/
<hw>Sac"ri*fi`cer</hw> <pr>(?)</pr>, <pos>n.</pos> <def>One who sacrifices.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sacrificial</ent><br/
<hw>Sac`ri*fi"cial</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Of or pertaining to sacrifice or sacrifices; consisting in sacrifice; performing sacrifice.</def> <ldquo/<xex>Sacrificial</xex> rites.<rdquo/ <rj><au>Jer. Taylor.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sacrilege</ent><br/
<hw>Sac"ri*lege</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F. <ets>sacril<egrave/ge</ets>, L. <ets>sacrilegium</ets>, from <ets>sacrilegus</ets> that steals, properly, gathers or picks up, sacred things; <ets>sacer</ets> sacred + <ets>legere</ets> to gather, pick up. See <er>Sacred</er>, and <er>Legend</er>.]</ety> <def>The sin or crime of violating or profaning sacred things; the alienating to laymen, or to common purposes, what has been appropriated or consecrated to religious persons or uses.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>And the hid treasures in her sacred tomb<br/
With <qex>sacrilege</qex> to dig.</q> <rj><qau>Spenser.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Families raised upon the ruins of churches, and enriched with the spoils of <qex>sacrilege</qex>.</q> <rj><qau>South.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sacrilegious</ent><br/
<hw>Sac`ri*le"gious</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[From <ets>sacrilege</ets>: cf. L. <ets>sacrilegus</ets>.]</ety> <def>Violating sacred things; polluted with sacrilege; involving sacrilege; profane; impious.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Above the reach of <qex>sacrilegious</qex> hands.</q> <rj><qau>Pope.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p>-- <wordforms><wf>Sac`ri*le"gious*ly</wf>, <pos>adv.</pos> -- <wf>Sac`ri*le"gious*ness</wf>, <pos>n.</pos></wordforms><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sacrilegist</ent><br/
<hw>Sac"ri*le`gist</hw> <pr>(?)</pr>, <pos>n.</pos> <def>One guilty of sacrilege.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sacring</ent><br/
<hw>Sac"ring</hw> <pr>(?)</pr>, <def><pos>a. & n.</pos> from <er>Sacre</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Sacring bell</b></col>. <cd>See <cref>Sanctus bell</cref>, under <er>Sanctus</er>.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sacrist</ent><br/
<hw>Sa"crist</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[LL. <ets>sacrista</ets>. See <er>Sacristan</er>.]</ety> <def>A sacristan; also, a person retained in a cathedral to copy out music for the choir, and take care of the books.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sacristan</ent><br/
<hw>Sac"ris*tan</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F. <ets>sacristain</ets>, LL. <ets>sacrista</ets>, fr. L. <ets>sacer</ets>. See <er>Sacred</er>, and cf. <er>Sexton</er>.]</ety> <def>An officer of the church who has the care of the utensils or movables, and of the church in general; a sexton.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sacristy</ent><br/
<hw>Sac"ris*ty</hw> <pr>(?)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Sacristies</plw> <pr>(#)</pr>.</plu> <ety>[F. <ets>sacristie</ets>, LL. <ets>sacristia</ets>, fr. L. <ets>sacer</ets>. See <er>Sacred</er>.]</ety> <def>An apartment in a church where the sacred utensils, vestments, etc., are kept; a vestry.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sacro-</ent><br/
<hw>Sa"cro-</hw> <pr>(<?/)</pr>. <fld>(Anat.)</fld> <def>A combining form denoting <xex>connection with</xex>, or <xex>relation to</xex>, <xex>the sacrum</xex>, as in <xex>sacro</xex>-coccygeal, <xex>sacro</xex>-iliac, <xex>sacro</xex>sciatic.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sacrosanct</ent><br/
<hw>Sac"ro*sanct</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>sucrosanctus</ets>.]</ety> <def>Sacred; inviolable.</def> <mark>[R.]</mark> <rj><au>Dr. H. More.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sacrosciatic</ent><br/
<hw>Sa`cro*sci*at"ic</hw> <pr>(?)</pr>, <pos>a.</pos> <fld>(Anat.)</fld> <def>Of or pertaining to both the sacrum and the hip; <as>as, the <ex>sacrosciatic</ex> foramina formed by the <ex>sacrosciatic</ex> ligaments which connect the sacrum and the hip bone</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sacrovertebral</ent><br/
<hw>Sa`cro*ver"te*bral</hw> <pr>(?)</pr>, <pos>a.</pos> <fld>(Anat.)</fld> <def>Of or pertaining to the sacrum and that part of the vertebral column immediately anterior to it; <as>as, the <ex>sacrovertebral</ex> angle</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sacrum</ent><br/
||<hw>Sa"crum</hw> <pr>(?)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>sacra</plw> <pr>(<?/)</pr>.</plu> <ety>[NL., fr. L. <ets>sacer</ets> sacred, <ets>os sacrum</ets> the lowest bone of the spine.]</ety> <fld>(Anat.)</fld> <def>That part of the vertebral column which is directly connected with, or forms a part of, the pelvis.</def><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ It may consist of a single vertebra or of several more or less consolidated. In man it forms the dorsal, or posterior, wall of the pelvis, and consists of five united vertebrae, which diminish in size very rapidly to the posterior extremity, which bears the coccyx.</note><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sacs</ent><br/
<hw>Sacs</hw> <pr>(s<add/ks)</pr>, <pos>n. pl.</pos>; <sing>sing. <singw>Sac</singw> <pr>(<?/)</pr></sing>. <fld>(Ethnol.)</fld> <def>A tribe of Indians, which, together with the Foxes, formerly occupied the region about Green Bay, Wisconsin.</def> <altsp>[Written also <asp>Sauks</asp>.]</altsp><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sad</ent><br/
<hw>Sad</hw> <pr>(s<acr/d)</pr>, <pos>a.</pos> <amorph>[<pos>Compar.</pos> <adjf>Sadder</adjf> <pr>(s<acr/d"d<etil/r)</pr>; <pos>superl.</pos> <adjf>Saddest</adjf>.]</amorph> <ety>[OE. <ets>sad</ets> sated, tired, satisfied, firm, steadfast, AS. <ets>saed</ets> satisfied, sated; akin to D. <ets>zat</ets>, OS. <ets>sad</ets>, G. <ets>satt</ets>, OHG. <ets>sat</ets>, Icel. <ets>sa<edh/r</ets>, <ets>saddr</ets>, Goth. <ets>sa<thorn/s</ets>, Lith. <ets>sotus</ets>, L. <ets>sat</ets>, <ets>satis</ets>, enough, <ets>satur</ets> sated, Gr. <grk>'a`menai</grk> to satiate, <grk>'a`dnh</grk> enough. Cf. <er>Assets</er>, <er>Sate</er>, <er>Satiate</er>, <er>Satisfy</er>, <er>Satire</er>.]</ety> <sn>1.</sn> <def>Sated; satisfied; weary; tired.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>Yet of that art they can not waxen <qex>sad</qex>,<br/
For unto them it is a bitter sweet.</q> <rj><qau>Chaucer.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Heavy; weighty; ponderous; close; hard.</def> <mark>[Obs., except in a few phrases; <as>as, <ex>sad</ex> bread</as>.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>His hand, more <qex>sad</qex> than lump of lead.</q> <rj><qau>Spenser.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Chalky lands are naturally cold and <qex>sad</qex>.</q> <rj><qau>Mortimer.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>Dull; grave; dark; somber; -- said of colors.</def> <ldquo/<xex>Sad</xex>-colored clothes.<rdquo/ <rj><au>Walton.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Woad, or wade, is used by the dyers to lay the foundation of all <qex>sad</qex> colors.</q> <rj><qau>Mortimer.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>Serious; grave; sober; steadfast; not light or frivolous.</def> <mark>[Obs.]</mark> <ldquo/Ripe and <xex>sad</xex> courage.<rdquo/ <rj><au>Chaucer.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Lady Catharine, a <qex>sad</qex> and religious woman.</q> <rj><au>Bacon.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Which treaty was wisely handled by <qex>sad</qex> and discrete counsel of both parties.</q> <rj><qau>Ld. Berners.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <def>Affected with grief or unhappiness; cast down with affliction; downcast; gloomy; mournful.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>First were we <qex>sad</qex>, fearing you would not come;<br/
Now <qex>sadder</qex>, that you come so unprovided.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>The angelic guards ascended, mute and <qex>sad</qex>.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>6.</sn> <def>Afflictive; calamitous; causing sorrow; <as>as, a <ex>sad</ex> accident; a <ex>sad</ex> misfortune</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>7.</sn> <def>Hence, bad; naughty; troublesome; wicked.</def> <mark>[Colloq.]</mark> <ldquo/<xex>Sad</xex> tipsy fellows, both of them.<rdquo/ <rj><au>I. Taylor.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ <ex>Sad</ex> is sometimes used in the formation of self-explaining compounds; <as>as, <ex>sad</ex>-colored, <ex>sad</ex>-eyed, <ex>sad</ex>-hearted, <ex>sad</ex>-looking, and the like</as>.</note><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Sad bread</b></col>, <cd>heavy bread.</cd> <mark>[Scot. & Local, U.S.]</mark> <rj><au>Bartlett.</au></rj></cs><br/
[<source>1913 Webster</source>]</p>

<p><syn><b>Syn.</b> -- Sorrowful; mournful; gloomy; dejected; depressed; cheerless; downcast; sedate; serious; grave; grievous; afflictive; calamitous.</syn><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sad</ent><br/
<hw>Sad</hw>, <pos>v. t.</pos> <def>To make sorrowful; to sadden.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>How it <qex>sadded</qex> the minister's spirits!</q> <rj><qau>H. Peters.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>SAD</ent><br/
<hw>SAD</hw> <pr>(?)</pr>, <pos>n.</pos> <def>Seasonal affective disorder.</def> <mark>[Acron.]</mark><br/
[<source>PJC</source>]</p>

<p><ent>Sadda</ent><br/
||<hw>Sad"da</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Per. <ets>sad-dar</ets> the hundred gates or ways; <ets>sad</ets> a hundred + <ets>dar</ets> door, way.]</ety> <def>A work in the Persian tongue, being a summary of the Zend-Avesta, or sacred books.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sadden</ent><br/
<hw>Sad"den</hw> <pr>(?)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Saddened</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Saddening</conjf>.]</vmorph> <def>To make sad.</def> Specifically: <sd>(a)</sd> <def>To render heavy or cohesive.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>Marl is binding, and <qex>saddening</qex> of land is the great prejudice it doth to clay lands.</q> <rj><qau>Mortimer.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sd>(b)</sd> <def>To make dull- or sad-colored, as cloth.</def> <sd>(c)</sd> <def>To make grave or serious; to make melancholy or sorrowful.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Her gloomy presence <qex>saddens</qex> all the scene.</q> <rj><qau>Pope.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sadden</ent><br/
<hw>Sad"den</hw>, <pos>v. i.</pos> <def>To become, or be made, sad.</def> <rj><au>Tennyson.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sadder</ent><br/
<hw>Sad"der</hw> <pr>(?)</pr>, <pos>n.</pos> <def>Same as <er>Sadda</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Saddle</ent><br/
<hw>Sad"dle</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[OE. <ets>sadel</ets>, AS. <ets>sadol</ets>; akin to D. <ets>zadel</ets>, G. <ets>sattel</ets>, OHG. <ets>satal</ets>, <ets>satul</ets>, Icel. <ets>s<oum/<edh/ull</ets>, Dan. & Sw. <ets>sadel</ets>; cf. Russ. <ets>siedlo</ets>; all perh. ultimately from the root of E. <ets>sit</ets>.]</ety> <sn>1.</sn> <def>A seat for a rider, -- usually made of leather, padded to span comfortably a horse's back, furnished with stirrups for the rider's feet to rest in, and fastened in place with a girth; also, a seat for the rider on a bicycle or tricycle.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A padded part of a harness which is worn on a horse's back, being fastened in place with a girth. It serves various purposes, as to keep the breeching in place, carry guides for the reins, etc.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>A piece of meat containing a part of the backbone of an animal with the ribs on each side; <as>as, a <ex>saddle</ex> of mutton, of venison, etc.</as></def><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <fld>(Naut.)</fld> <def>A block of wood, usually fastened to some spar, and shaped to receive the end of another spar.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <fld>(Mach.)</fld> <def>A part, as a flange, which is hollowed out to fit upon a convex surface and serve as a means of attachment or support.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>6.</sn> <fld>(Zool.)</fld> <def>The clitellus of an earthworm.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>7.</sn> <fld>(Arch.)</fld> <def>The threshold of a door, when a separate piece from the floor or landing; -- so called because it spans and covers the joint between two floors.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>8.</sn> <fld>(Phys. Geog.)</fld> <def>A ridge connected two higher elevations; a low point in the crest line of a ridge; a col.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><sn>9.</sn>  <fld>(Mining)</fld> <def>A formation of gold-bearing quartz occurring along the crest of an anticlinal fold, esp. in Australia.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><cs><col><b>Saddle bar</b></col> <fld>(Arch.)</fld>, <cd>one the small iron bars to which the lead panels of a glazed window are secured.</cd> <au>Oxf. Gloss.</au> -- <col><b>Saddle gall</b></col> <fld>(Far.)</fld>, <cd>a sore or gall upon a horse's back, made by the saddle.</cd> -- <col><b>Saddle girth</b></col>, <cd>a band passing round the body of a horse to hold the saddle in its place.</cd> -- <col><b>saddle horse</b></col>, <cd>a horse suitable or trained for riding with a saddle.</cd> -- <col><b>Saddle joint</b></col>, <cd>in sheet-metal roofing, a joint formed by bending up the edge of a sheet and folding it downward over the turned-up edge of the next sheet.</cd> -- <col><b>Saddle roof</b></col>, <fld>(Arch.)</fld>, <cd>a roof having two gables and one ridge; -- said of such a roof when used in places where a different form is more common; <as>as, a tower surmounted by a <ex>saddle roof</ex></as>. Called also <altname>saddleback roof</altname>.</cd> -- <col><b>Saddle shell</b></col> <fld>(Zool.)</fld>, <cd>any thin plicated bivalve shell of the genera <gen>Placuna</gen> and <gen>Anomia</gen>; -- so called from its shape. Called also <altname>saddle oyster</altname>.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Saddle</ent><br/
<hw>Sad"dle</hw> <pr>(?)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Saddled</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Saddling</conjf> <pr>(?)</pr>.]</vmorph> <ety>[AS. <ets>sadelian</ets>.]</ety> <sn>1.</sn> <def>To put a saddle upon; to equip (a beast) for riding.</def> <ldquo/<xex>saddle</xex> my horse.<rdquo/ <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Abraham rose up early, . . . and <qex>saddled</qex> his ass.</q> <rj><qau>Gen. xxii. 3.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Hence: To fix as a charge or burden upon; to load; to encumber; <as>as, to <ex>saddle</ex> a town with the expense of bridges and highways</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Saddleback</ent><br/
<hw>Sad"dle*back`</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Same as <er>Saddle-backed</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Saddleback roof</b></col>. <fld>(Arch.)</fld> <cd>See <cref>Saddle roof</cref>, under <er>Saddle</er>.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Saddleback</ent><br/
<hw>Sad"dle*back`</hw>, <pos>n.</pos> <sn>1.</sn> <def>Anything saddle-backed; esp., a hill or ridge having a concave outline at the top.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Zool.)</fld> <sd>(a)</sd> <def>The harp seal.</def> <sd>(b)</sd> <def>The great blackbacked gull (<spn>Larus marinus</spn>).</def> <sd>(c)</sd> <def>The larva of a bombycid moth (<spn>Empretia stimulea</spn>) which has a large, bright green, saddle-shaped patch of color on the back.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Saddle-backed</ent><br/
<hw>Sad"dle-backed`</hw> <pr>(?)</pr>, <pos>a.</pos> <sn>1.</sn> <def>Having the outline of the upper part concave like the seat of a saddle.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Having a low back and high neck, as a horse.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Saddlebags</ent><br/
<hw>Sad"dle*bags</hw> <pr>(?)</pr>, <pos>n. pl.</pos> <def>Bags, usually of leather, united by straps or a band, formerly much used by horseback riders to carry small articles, one bag hanging on each side.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Saddlebow</ent><br/
<hw>Sad"dle*bow`</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[AS. <ets>sadelboga</ets>.]</ety> <def>The bow or arch in the front part of a saddle, or the pieces which form the front.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Saddlecloth</ent><br/
<hw>Sad"dle*cloth`</hw> <pr>(?; 115)</pr>, <pos>n.</pos> <def>A cloth under a saddle, and extending out behind; a housing.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Saddled</ent><br/
<hw>Sad"dled</hw> <pr>(?)</pr>, <pos>a.</pos> <fld>(Zool.)</fld> <def>Having a broad patch of color across the back, like a saddle; saddle-backed.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Saddler</ent><br/
<hw>Sad"dler</hw> <pr>(?)</pr>, <pos>n.</pos> <def>One who makes saddles.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Zool.)</fld> <def>A harp seal.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Saddlery</ent><br/
<hw>Sad"dler*y</hw> <pr>(?)</pr>, <pos>n.</pos> <sn>1.</sn> <def>The materials for making saddles and harnesses; the articles usually offered for sale in a saddler's shop.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>The trade or employment of a saddler.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Saddle-shaped</ent><br/
<hw>Sad"dle-shaped`</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Shaped like a saddle.</def> Specifically: <sd>(a)</sd> <fld>(Bot.)</fld> <def>Bent down at the sides so as to give the upper part a rounded form.</def> <rj><au>Henslow.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sd>(b)</sd> <fld>(Geol.)</fld> <def>Bent on each side of a mountain or ridge, without being broken at top; -- said of strata.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Saddletree</ent><br/
<hw>Sad"dle*tree`</hw> <pr>(?)</pr>, <pos>n.</pos> <def>The frame of a saddle.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>For <qex>saddletree</qex> scarce reached had he,<br/
His journey to begin.</q> <rj><qau>Cowper.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sadducaic</ent><br/
<hw>Sad`du*ca"ic</hw> <pr>(?; 135)</pr>, <pos>a.</pos> <def>Pertaining to, or like, the Sadducees; <as>as, <ex>Sadducaic</ex> reasonings</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sadducee</ent><br/
<hw>Sad"du*cee</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>Sadducaei</ets>, p., Gr. <?/, Heb. <ets>Tsadd<umac/k<imac/m</ets>; -- so called from <ets>Ts<amac/d<omac/k</ets>, the founder of the sect.]</ety> <def>One of a sect among the ancient Jews, who denied the resurrection, a future state, and the existence of angels.</def> -- <wordforms><wf>Sad`du*ce"an</wf> <pr>(#)</pr>, <pos>a.</pos></wordforms><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sadducism</ent><br/
<ent>Sadduceeism</ent><br/
<mhw>{ <hw>Sad"du*cee`ism</hw> <pr>(?)</pr>, <hw>Sad"du*cism</hw> <pr>(?)</pr>, }</mhw> <pos>n.</pos> <def>The tenets of the Sadducees.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sadducize</ent><br/
<hw>Sad"du*cize</hw> <pr>(?)</pr>, <pos>v. i.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Sadducized</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Sadducizing</conjf> <pr>(?)</pr>.]</vmorph> <def>To adopt the principles of the Sadducees.</def> <rj><au>Atterbury.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sadh</ent><br/
<hw>Sadh</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Skr. <ets>s<amac/dhu</ets> perfect, pure.]</ety> <def>A member of a monotheistic sect of Hindoos. Sadhs resemble the Quakers in many respects.</def> <rj><au>Balfour (Cyc. of India).</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sadiron</ent><br/
<hw>Sad"i`ron</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Probably <ets>sad</ets> heavy + <ets>iron</ets>.]</ety> <def>An iron for smoothing clothes; a flatiron.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sadly</ent><br/
<hw>Sad"ly</hw>, <pos>adv.</pos> <sn>1.</sn> <def>Wearily; heavily; firmly.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>In go the spears full <qex>sadly</qex> in arest.</q> <rj><qau>Chaucer.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Seriously; soberly; gravely.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>To tell thee <qex>sadly</qex>, shepherd, without blame<br/
Or our neglect, we lost her as we came.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>Grievously; deeply; sorrowfully; miserably.</def> <ldquo/He <xex>sadly</xex> suffers in their grief.<rdquo/ <rj><au>Dryden.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sadness</ent><br/
<hw>Sad"ness</hw>, <pos>n.</pos> <sn>1.</sn> <def>Heaviness; firmness.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Seriousness; gravity; discretion.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>Her <qex>sadness</qex> and her benignity.</q> <rj><qau>Chaucer.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>Quality of being sad, or unhappy; gloominess; sorrowfulness; dejection.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Dim <qex>sadness</qex> did not spare<br/
That time celestial visages.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><syn><b>Syn.</b> -- Sorrow; heaviness; dejection. See <er>Grief</er>.</syn><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sadr</ent><br/
||<hw>Sadr</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Bot.)</fld> <def>A plant of the genus <gen>Ziziphus</gen> (<spn>Ziziphus lotus</spn>); -- so called by the Arabs of Barbary, who use its berries for food. See <er>Lotus</er> <sd>(b)</sd>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Saengerbund</ent><br/
||<hw>Saeng"er*bund`</hw> <pr>(?)</pr>, <pos>n.</pos>; <plu><it>G. pl.</it> <plw>Saengerb<uum/nde</plw> <pr>(#)</pr></plu>. <ety>[G. <ets>s<aum/ngerbund</ets>.]</ety> <fld>(Music)</fld> <def>A singers' union; an association of singers or singing clubs, esp. German.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Saengerfest</ent><br/
||<hw>Saeng"er*fest</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[G. <ets>s<aum/ngerfest</ets>.]</ety> <def>A festival of singers; a German singing festival.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Safe</ent><br/
<hw>Safe</hw> <pr>(?)</pr>, <pos>a.</pos> <amorph>[<pos>Compar.</pos> <adjf>Safer</adjf> <pr>(?)</pr>; <pos>superl.</pos> <adjf>Safest</adjf>.]</amorph> <ety>[OE. <ets>sauf</ets>, F. <ets>sauf</ets>, fr. L. <ets>salvus</ets>, akin to <ets>salus</ets> health, welfare, safety. Cf. <er>Salute</er>, <er>Salvation</er>, <er>Sage</er> a plant, <er>Save</er>, <er>Salvo</er> an exception.]</ety> <sn>1.</sn> <def>Free from harm, injury, or risk; untouched or unthreatened by danger or injury; unharmed; unhurt; secure; whole; <as>as, <ex>safe</ex> from disease; <ex>safe</ex> from storms; <ex>safe</ex> from foes</as>.</def> <ldquo/And ye dwelled <xex>safe</xex>.<rdquo/ <rj><au>1 Sam. xii. 11.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>They escaped all <qex>safe</qex> to land.</q> <rj><qau>Acts xxvii. 44.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Established in a <qex>safe</qex>, unenvied throne.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Conferring safety; securing from harm; not exposing to danger; confining securely; to be relied upon; not dangerous; <as>as, a <ex>safe</ex> harbor; a <ex>safe</ex> bridge, etc.</as></def> <ldquo/The man of <xex>safe</xex> discretion.<rdquo/ <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>The King of heaven hath doomed<br/
This place our dungeon, not our <qex>safe</qex> retreat.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><-- p. 1267 pr=PCP --></p>

<p><sn>3.</sn> <def>Incapable of doing harm; no longer dangerous; in secure care or custody; <as>as, the prisoner is <ex>safe</ex></as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>But Banquo's <qex>safe</qex>?<br/
Ay, my good lord, <qex>safe</qex> in a ditch he bides.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Safe hit</b></col> <fld>(Baseball)</fld>, <cd>a hit which enables the batter to get to first base even if no error is made by the other side.</cd> <-- <col><b>safe house</b></col>, <cd>a residence where a person in hiding from the authorities or other persons may stay without being discovered.</cd> --></cs><br/
[<source>1913 Webster</source>]</p>

<p><syn><b>Syn.</b> -- Secure; unendangered; sure.</syn><br/
[<source>1913 Webster</source>]</p>

<p><ent>Safe</ent><br/
<hw>Safe</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A place for keeping things in safety.</def> <specif>Specifically:</specif> <sd>(a)</sd> <def>A strong and fireproof receptacle (as a movable chest of steel, etc., or a closet or vault of brickwork) for containing money, valuable papers, or the like.</def> <sd>(b)</sd> <def>A ventilated or refrigerated chest or closet for securing provisions from noxious animals or insects.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Safe</ent><br/
<hw>Safe</hw>, <pos>v. t.</pos> <def>To render safe; to make right.</def> <mark>[Obs.]</mark> <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Safe-conduct</ent><br/
<hw>Safe"-con"duct</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[<ets>Safe</ets> + <ets>conduct</ets>: cf. F. <ets>sauf-conduit</ets>.]</ety> <def>That which gives a safe passage</def>; either <sd>(a)</sd> <def>a convoy or guard to protect a person in an enemy's country or a foreign country</def>, or <sd>(b)</sd> <def>a writing, pass, or warrant of security, given to a person to enable him to travel with safety.</def> <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Safe-conduct</ent><br/
<hw>Safe`-con*duct"</hw> <pr>(?)</pr>, <pos>v. t.</pos> <def>To conduct safely; to give safe-conduct to.</def> <mark>[Poetic]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>He him by all the bonds of love besought<br/
To <qex>safe-conduct</qex> his love.</q> <rj><qau>Spenser.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Safeguard</ent><br/
<hw>Safe"guard`</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Safe = <ets>guard</ets>: cf. F. <ets>sauvegarde</ets>.]</ety> <sn>1.</sn> <def>One who, or that which, defends or protects; defense; protection.</def> <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Thy sword, the <qex>safeguard</qex> of thy brother's throne.</q> <rj><qau>Granville.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A convoy or guard to protect a traveler or property.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>A pass; a passport; a safe-conduct.</def> <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Safeguard</ent><br/
<hw>Safe"guard`</hw>, <pos>v. t.</pos> <def>To guard; to protect.</def> <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Safe-keeping</ent><br/
<hw>Safe"-keep"ing</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[<ets>Safe</ets> + <ets>keep</ets>.]</ety> <def>The act of keeping or preserving in safety from injury or from escape; care; custody.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Safely</ent><br/
<hw>Safe"ly</hw>, <pos>adv.</pos> <def>In a safe manner; danger, injury, loss, or evil consequences.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Safeness</ent><br/
<hw>Safe"ness</hw>, <pos>n.</pos> <def>The quality or state of being safe; freedom from hazard, danger, harm, or loss; safety; security; as the <xex>safeness</xex> of an experiment, of a journey, or of a possession.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Safe-pledge</ent><br/
<hw>Safe"-pledge"</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Law)</fld> <def>A surety for the appearance of a person at a given time.</def> <rj><au>Bracton.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Safety</ent><br/
<hw>Safe"ty</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Cf. F. <ets>sauvet<eacute/</ets>.]</ety> <sn>1.</sn> <def>The condition or state of being safe; freedom from danger or hazard; exemption from hurt, injury, or loss.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Up led by thee,<br/
Into the heaven I have presumed,<br/
An earthly guest . . . With like <qex>safety</qex> guided down,<br/
Return me to my native element.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Freedom from whatever exposes one to danger or from liability to cause danger or harm; safeness; hence, the quality of making safe or secure, or of giving confidence, justifying trust, insuring against harm or loss, etc.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Would there were any <qex>safety</qex> in thy sex,<br/
That I might put a thousand sorrows off,<br/
And credit thy repentance!</q> <rj><qau>Beau. & Fl.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>Preservation from escape; close custody.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Imprison him, . . . <br/
Deliver him to <qex>safety</qex>; and return.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <fld>(Amer. Football)</fld> <def>the act or result of a ball-carrier on the offensive team being tackled behind his own goal line, or the downing of a ball behind the offensive team's own goal line when it had been carried or propelled behind that goal line by a player on the offensive tream; such a play causes a score of two points to be awarded to the defensive team; -- it is distinguished from <contr>touchback</contr>, when the ball is downed behind the goal after being propelled there or last touched by a player of the defending team. See <er>Touchdown</er>. Same as <cref>Safety touchdown</cref>, below.</def><br/
[<source>Webster 1913 Suppl.</source> <source>+PJC</source>]</p>

<p><sn>5.</sn> <def>Short for <er>Safety bicycle</er>.</def> <mark>[archaic]</mark><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><sn>6.</sn> <def>a switch on a firearm that locks the trigger and prevents the firearm from being discharged unintentionally; -- also called <altname>safety catch</altname>, <altname>safety lock</altname>, or <altname>lock</altname>.</def> <mark>[archaic]</mark><br/
[<source>PJC</source>]</p>

<p><ent>Safety bicycle</ent><br/
<hw>Safety bicycle</hw>. <def>A bicycle with equal or nearly equal wheels, usually about 28 inches diameter, driven by pedals connected to the rear (driving) wheel by a multiplying gear.  Since the 1930's this has been the most common type of bicycle, now simply called <altname>bicycle</altname>.  The older high-wheelers are often referred to as <contr>bone-rattlers</contr>.</def><br/
[<source>Webster 1913 Suppl.</source> <source>+PJC</source>]</p>

<p><ent>Safety chain</ent><br/
<hw>Safety chain</hw>. <sd>(a)</sd> <fld>(Railroads)</fld> <def>A normally slack chain for preventing excessive movement between a truck and a car body in sluing.</def> <sd>(b)</sd> <def>An auxiliary watch chain, secured to the clothes, usually out of sight, to prevent stealing of the watch.</def> <sd>(c)</sd> <def>A chain of sheet metal links with an elongated hole through each broad end, made up by doubling the first link on itself, slipping the next link through and doubling, and so on.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><cs><col><b>Safety arch</b></col> <fld>(Arch.)</fld>, <cd>a discharging arch.</cd> See under <er>Discharge</er>, <pos>v. t.</pos> -- <col><b>Safety belt</b></col>, <cd>a belt made of some buoyant material, or which is capable of being inflated, so as to enable a person to float in water; a life preserver.</cd> -- <col><b>Safety buoy</b></col>, <cd>a buoy to enable a person to float in water; a safety belt.</cd> -- <col><b>Safety cage</b></col> <fld>(Mach.)</fld>, <cd>a cage for an elevator or mine lift, having appliances to prevent it from dropping if the lifting rope should break.</cd> -- <col><b>Safety lamp</b></col>. <fld>(Mining)</fld> <cd>See under <er>Lamp</er>.</cd> -- <col><b>Safety match</b></col>, <cd>a match which can be ignited only on a surface specially prepared for the purpose.</cd> -- <col><b>Safety pin</b></col>, <cd>a pin made in the form of a clasp, with a guard covering its point so that it will not prick the wearer.</cd> -- <col><b>Safety plug</b></col>. <cd>See <cref>Fusible plug</cref>, under <er>Fusible</er>.</cd> -- <col><b>Safety switch</b></col>. <cd>See <er>Switch</er>.</cd> -- <col><b>Safety touchdown</b></col> <fld>(Football)</fld>, <cd>the act or result of a player's touching to the ground behind his own goal line a ball which received its last impulse from a man on his own side; -- distinguished from <xex>touchback</xex>. See <er>Touchdown</er>.  Same as <er>safety</er></cd>  -- <col><b>Safety tube</b></col> <fld>(Chem.)</fld>, <cd>a tube to prevent explosion, or to control delivery of gases by an automatic valvular connection with the outer air; especially, a bent funnel tube with bulbs for adding those reagents which produce unpleasant fumes or violent effervescence.</cd> -- <col><b>Safety valve</b></col>, <cd>a valve which is held shut by a spring or weight and opens automatically to permit the escape of steam, or confined gas, water, etc., from a boiler, or other vessel, when the pressure becomes too great for safety; also, sometimes, a similar valve opening inward to admit air to a vessel in which the pressure is less than that of the atmosphere, to prevent collapse.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Safflow</ent><br/
<hw>Saf"flow</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Bot.)</fld> <def>The safflower.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Safflower</ent><br/
<hw>Saf"flow`er</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F. <ets>safleur</ets>, <ets>saflor</ets>, for <ets>safran</ets>, influenced by <ets>fleur</ets> flower. See <er>Saffron</er>, and <er>Flower</er>.]</ety> <sn>1.</sn> <fld>(Bot.)</fld> <def>An annual composite plant (<spn>Carthamus tinctorius</spn>), the flowers of which are used as a dyestuff and in making rouge; bastard, or false, saffron.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>The dried flowers of the <spn>Carthamus tinctorius</spn>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>A dyestuff from these flowers. See <er>Safranin</er> <sd>(b)</sd>.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Oil of safflower</b></col>, <cd>a purgative oil expressed from the seeds of the safflower.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Saffron</ent><br/
<hw>Saf"fron</hw> <pr>(?; 277)</pr>, <pos>n.</pos> <ety>[OE. <ets>saffran</ets>, F. <ets>safran</ets>; cf. It. <ets>zafferano</ets>, Sp. <ets>azafran</ets>, Pg. <ets>a<cced/afr<atil/o</ets>; all fr. Ar. & Per. <ets>za' far<amac/n</ets>.]</ety> <sn>1.</sn> <fld>(Bot.)</fld> <def>A bulbous iridaceous plant (<spn>Crocus sativus</spn>) having blue flowers with large yellow stigmas. See <er>Crocus</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>The aromatic, pungent, dried stigmas, usually with part of the stile, of the <spn>Crocus sativus</spn>. Saffron is used in cookery, and in coloring confectionery, liquors, varnishes, etc., and was formerly much used in medicine.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>An orange or deep yellow color, like that of the stigmas of the <spn>Crocus sativus</spn>.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><mcol><col><b>Bastard saffron</b></col>, <col><b>Dyer's saffron</b></col></mcol>. <fld>(Bot.)</fld> <cd>See <er>Safflower</er>.</cd> -- <col><b>Meadow saffron</b></col> <fld>(Bot.)</fld>, <cd>a bulbous plant (<spn>Colchichum autumnale</spn>) of Europe, resembling saffron.</cd> -- <col><b>Saffron wood</b></col> <fld>(Bot.)</fld>, <cd>the yellowish wood of a South African tree (<spn>Elaeodendron croceum</spn>); also, the tree itself.</cd> -- <col><b>Saffron yellow</b></col>, <cd>a shade of yellow like that obtained from the stigmas of the true saffron (<spn>Crocus sativus</spn>).</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Saffron</ent><br/
<hw>Saf"fron</hw> <pr>(?; 277)</pr>, <pos>a.</pos> <def>Having the color of the stigmas of saffron flowers; deep orange-yellow; <as>as, a <ex>saffron</ex> face; a <ex>saffron</ex> streamer</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Saffron</ent><br/
<hw>Saf"fron</hw>, <pos>v. t.</pos> <def>To give color and flavor to, as by means of saffron; to spice.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>And in Latyn I speak a wordes few,<br/
To <qex>saffron</qex> with my predication.</q> <rj><qau>Chaucer.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Saffrony</ent><br/
<hw>Saf"fron*y</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Having a color somewhat like saffron; yellowish.</def> <rj><au>Lord (1630).</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Safranin</ent><br/
<hw>Saf"ra*nin</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Chem.)</fld> <sd>(a)</sd> <def>An orange-red dyestuff extracted from the saffron.</def> <mark>[R.]</mark> <sd>(b)</sd> <def>A red dyestuff extracted from the safflower, and formerly used in dyeing wool, silk, and cotton pink and scarlet; -- called also <altname>Spanish red</altname>, <altname>China lake</altname>, and <altname>carthamin</altname>.</def> <sd>(c)</sd> <def>An orange-red dyestuff prepared from certain nitro compounds of creosol, and used as a substitute for the safflower dye.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Safranine</ent><br/
<hw>Saf"ra*nine</hw> <pr>(? <or/ ?)</pr>, <pos>n.</pos> <ety>[So called because used as a substitute for safranin.]</ety> <fld>(Chem.)</fld> <def>An orange-red nitrogenous dyestuff produced artificially by oxidizing certain aniline derivatives, and used in dyeing silk and wool; also, any one of the series of which safranine proper is the type.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sag</ent><br/
<hw>Sag</hw> <pr>(s<acr/g)</pr>, <pos>v. i.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Sagged</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Sagging</conjf> <pr>(?)</pr>.]</vmorph> <ety>[Akin to Sw. <ets>sacka</ets> to settle, sink down, LG. <ets>sacken</ets>, D. <ets>zakken</ets>. Cf. <er>Sink</er>, <pos>v. i.</pos>]</ety> <sn>1.</sn> <def>To sink, in the middle, by its weight or under applied pressure, below a horizontal line or plane; <as>as, a line or cable supported by its ends <ex>sags</ex>, though tightly drawn; the floor of a room <ex>sags</ex></as>; hence, to lean, give way, or settle from a vertical position; <as>as, a building may <ex>sag</ex> one way or another; a door <ex>sags</ex> on its hinges</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Fig.: To lose firmness or elasticity; to sink; to droop; to flag; to bend; to yield, as the mind or spirits, under the pressure of care, trouble, doubt, or the like; to be unsettled or unbalanced.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>The mind I sway by, and the heart I bear,<br/
Shall never <qex>sag</qex> with doubt nor shake with fear.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>To loiter in walking; to idle along; to drag or droop heavily.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>To sag to leeward</b></col> <fld>(Naut.)</fld>, <cd>to make much leeway by reason of the wind, sea, or current; to drift to leeward; -- said of a vessel.</cd> <rj><au>Totten.</au></rj></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sag</ent><br/
<hw>Sag</hw>, <pos>v. t.</pos> <def>To cause to bend or give way; to load.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sag</ent><br/
<hw>Sag</hw>, <pos>n.</pos> <def>State of sinking or bending; sagging.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Saga</ent><br/
<hw>Sa"ga</hw> <pr>(s<amac/"g<adot/)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Sagas</plw> <pr>(-g<adot/z)</pr>.</plu> <ety>[Icel., akin to E. <ets>saw</ets> a saying. See <er>Say</er>, and cf. <er>Saw</er>.]</ety> <def>A Scandinavian legend, or heroic or mythic tradition, among the Norsemen and kindred people; a northern European popular historical or religious tale of olden time.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>And then the blue-eyed Norseman told<br/
A <qex>saga</qex> of the days of old.</q> <rj><qau>Longfellow.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sagacious</ent><br/
<hw>Sa*ga"cious</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>sagax</ets>, <ets>sagacis</ets>, akin to <ets>sagire</ets> to perceive quickly or keenly, and probably to E. <ets>seek</ets>. See <er>Seek</er>, and cf. <er>Presage</er>.]</ety> <sn>1.</sn> <def>Of quick sense perceptions; keen-scented; skilled in following a trail.</def><br/
[<source>1913 Webster</source>]</p>

<p><q><qex>Sagacious</qex> of his quarry from so far.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Hence, of quick intellectual perceptions; of keen penetration and judgment; discerning and judicious; knowing; far-sighted; shrewd; sage; wise; <as>as, a <ex>sagacious</ex> man; a <ex>sagacious</ex> remark</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Instinct . . . makes them, many times, <qex>sagacious</qex> above our apprehension.</q> <rj><qau>Dr. H. More.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Only <qex>sagacious</qex> heads light on these observations, and reduce them into general propositions.</q> <rj><qau>Locke.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><syn><b>Syn.</b> -- See <er>Shrewd</er>.</syn><br/
[<source>1913 Webster</source>]</p>

<p>-- <wordforms><wf>Sa*ga"cious*ly</wf>, <pos>adv.</pos> -- <wf>Sa*ga"cious*ness</wf>, <pos>n.</pos></wordforms><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sagacity</ent><br/
<hw>Sa*gac"i*ty</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>sagacitas</ets>. See <er>Sagacious</er>.]</ety> <def>The quality of being sagacious; quickness or acuteness of sense perceptions; keenness of discernment or penetration with soundness of judgment; shrewdness.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Some [brutes] show that nice <qex>sagacity</qex> of smell.</q> <rj><qau>Cowper.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Natural <qex>sagacity</qex> improved by generous education.</q> <rj><qau>V. Knox.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><syn><b>Syn.</b> -- Penetration; shrewdness; judiciousness.</syn> -- <usage><er>Sagacity</er>, <er>Penetration</er>. <xex>Penetration</xex> enables us to enter into the depths of an abstruse subject, to detect motives, plans, etc. <xex>Sagacity</xex> adds to penetration a keen, practical judgment, which enables one to guard against the designs of others, and to turn everything to the best possible advantage.</usage><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sagamore</ent><br/
<hw>Sag"a*more</hw> <pr>(?)</pr>, <pos>n.</pos> <sn>1.</sn> <ety>[Cf. <er>Sachem</er>.]</ety> <def>The head of a tribe among the American Indians; a chief; -- generally used as synonymous with <xex>sachem</xex>, but some writters distinguished between them, making the <xex>sachem</xex> a chief of the first rank, and a <xex>sagamore</xex> one of the second rank.</def> <ldquo/Be it <xex>sagamore</xex>, sachem, or powwow.<rdquo/ <rj><au>Longfellow.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A juice used in medicine.</def> <mark>[Obs.]</mark> <rj><au>Johnson.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sagapen</ent><br/
<hw>Sag"a*pen</hw> <pr>(?)</pr>, <pos>n.</pos> <def>Sagapenum.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sagapenum</ent><br/
||<hw>Sag`a*pe"num</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>sagapenon</ets>, <ets>sacopenium</ets>, Gr. <?/: cf. F. <ets>sagapin</ets>, gomme <ets>sagapin</ets>, sagap<eacute/num, Ar. <ets>sikb<imac/naj</ets>, Per. <ets>sakb<imac/nah</ets>, <ets>sikb<imac/nah</ets>.]</ety> <fld>(Med.)</fld> <def>A fetid gum resin obtained from a species of <gen>Ferula</gen>. It has been used in hysteria, etc., but is now seldom met with.  See also <er>asafetida</er>.</def> <rj><au>U. S. Disp.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sagathy</ent><br/
<hw>Sag"a*thy</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F. <ets>sagatis</ets>: cf. Sp. <ets>sagat<iacute/</ets>, <ets>saet<iacute/</ets>.]</ety> <def>A mixed woven fabric of silk and cotton, or silk and wool; sayette; also, a light woolen fabric.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sage</ent><br/
<hw>Sage</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[OE. <ets>sauge</ets>, F. <ets>sauge</ets>, L. <ets>salvia</ets>, from <ets>salvus</ets> saved, in allusion to its reputed healing virtues. See <er>Safe</er>.]</ety> <fld>(Bot.)</fld> <sd>(a)</sd> <def>A suffruticose labiate plant (<spn>Salvia officinalis</spn>) with grayish green foliage, much used in flavoring meats, etc. The name is often extended to the whole genus, of which many species are cultivated for ornament, as the scarlet sage, and Mexican red and blue sage.</def> <sd>(b)</sd> <def>The sagebrush.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Meadow sage</b></col> <fld>(Bot.)</fld>, <cd>a blue-flowered species of <gen>Salvia</gen> (<spn>Salvia pratensis</spn>) growing in meadows in Europe.</cd> -- <col><b>Sage cheese</b></col>, <cd>cheese flavored with sage, and colored green by the juice of leaves of spinach and other plants which are added to the milk.</cd> -- <col><b>Sage cock</b></col> <fld>(Zool.)</fld>, <cd>the male of the sage grouse; in a more general sense, the specific name of the sage grouse.</cd> -- <col><b>Sage green</b></col>, <cd>of a dull grayish green color, like the leaves of garden sage.</cd> -- <col><b>Sage grouse</b></col> <fld>(Zool.)</fld>, <cd>a very large American grouse (<spn>Centrocercus urophasianus</spn>), native of the dry sagebrush plains of Western North America. Called also <altname>cock of the plains</altname>. The male is called <stype>sage cock</stype>, and the female <stype>sage hen</stype>.</cd> -- <mcol><col><b>Sage hare</b></col>, <it>or</it> <col><b>Sage rabbit</b></col></mcol> <fld>(Zool.)</fld>, <cd>a species of hare (<spn>Lepus Nuttalli</spn> syn. <spn>Lepus artemisia</spn>) which inhabits the arid regions of <geog>Western North America</geog> and lives among sagebrush.  By recent writers it is considered to be merely a variety of the common cottontail, or wood rabbit.</cd> -- <col><b>Sage hen</b></col> <fld>(Zool.)</fld>, <cd>the female of the sage grouse.</cd> -- <col><b>Sage sparrow</b></col> <fld>(Zool.)</fld>, <cd>a small sparrow (<spn>Amphispiza Belli</spn>, var. <varn>Nevadensis</varn>) which inhabits the dry plains of the Rocky Mountain region, living among sagebrush.</cd> -- <col><b>Sage thrasher</b></col> <fld>(Zool.)</fld>, <cd>a singing bird (<spn>Oroscoptes montanus</spn>) which inhabits the sagebrush plains of <geog>Western North America</geog>.</cd> -- <col><b>Sage willow</b></col> <fld>(Bot.)</fld>, <cd>a species of willow (<spn>Salix tristis</spn>) forming a low bush with nearly sessile grayish green leaves.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sage</ent><br/
<hw>Sage</hw> <pr>(?)</pr>, <pos>a.</pos> <amorph>[<pos>Compar.</pos> <adjf>Sager</adjf> <pr>(?)</pr>; <pos>superl.</pos> <adjf>Sagest</adjf>.]</amorph> <ety>[F., fr. L. <ets>sapius</ets> (only in <ets>nesapius</ets> unwise, foolish), fr. <ets>sapere</ets> to be wise; perhaps akin to E. <ets>sap</ets>. Cf. <er>Savor</er>, <er>Sapient</er>, <er>Insipid</er>.]</ety> <sn>1.</sn> <def>Having nice discernment and powers of judging; prudent; grave; sagacious.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>All you <qex>sage</qex> counselors, hence!</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Proceeding from wisdom; well judged; shrewd; well adapted to the purpose.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Commanders, who, cloaking their fear under show of <qex>sage</qex> advice, counseled the general to retreat.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>Grave; serious; solemn.</def> <mark>[R.]</mark> <ldquo/[Great bards] in <xex>sage</xex> and solemn tunes have sung.<rdquo/ <rj><au>Milton.</au></rj> <-- the "great bards" was moved inside the quote for consistency. --><br/
[<source>1913 Webster</source>]</p>

<p><syn><b>Syn.</b> -- Wise; sagacious; sapient; grave; prudent; judicious.</syn><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sage</ent><br/
<hw>Sage</hw>, <pos>n.</pos> <def>A wise man; a man of gravity and wisdom; especially, a man venerable for years, and of sound judgment and prudence; a grave philosopher.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>At his birth a star,<br/
Unseen before in heaven, proclaims him come,<br/
And guides the Eastern <qex>sages</qex>.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sagebrush</ent><br/
<hw>Sage"brush`</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A low irregular shrub (<spn>Artemisia tridentata</spn>), of the order <ord>Compositae</ord>, covering vast tracts of the dry alkaline regions of the American plains; -- called also <altname>sagebush</altname>, and <altname>wild sage</altname>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sagebrush State</ent><br/
<hw>Sagebrush State</hw>. <def>Nevada; -- a nickname.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Sagely</ent><br/
<hw>Sage"ly</hw>, <pos>adv.</pos> <def>In a sage manner; wisely.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sagene</ent><br/
<hw>Sa*gene"</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Russ. <ets>sajene</ets>.]</ety> <def>A Russian measure of length equal to about seven English feet.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sageness</ent><br/
<hw>Sage"ness</hw> <pr>(?)</pr>, <pos>n.</pos> <def>The quality or state of being sage; wisdom; sagacity; prudence; gravity.</def> <rj><au>Ascham.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sagenite</ent><br/
<hw>Sag"e*nite</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F. <ets>sag<eacute/nite</ets>, fr. L. <ets>sagena</ets> a large net. See <er>Seine</er>.]</ety> <fld>(Min.)</fld> <def>Acicular rutile occurring in reticulated forms imbedded in quartz.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sagenitic</ent><br/
<hw>Sag`e*nit"ic</hw> <pr>(?)</pr>, <pos>a.</pos> <fld>(Min.)</fld> <def>Resembling sagenite; -- applied to quartz when containing acicular crystals of other minerals, most commonly rutile, also tourmaline, actinolite, and the like.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sagger</ent><br/
<hw>Sag"ger</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[See <er>Seggar</er>.]</ety> <sn>1.</sn> <def>A pot or case of fire clay, in which fine stoneware is inclosed while baking in the kiln; a seggar.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>The clay of which such pots or cases are made.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sagging</ent><br/
<hw>Sag"ging</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A bending or sinking between the ends of a thing, in consequence of its own, or an imposed, weight; an arching downward in the middle, as of a ship after straining. Cf. <er>Hogging</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Saginate</ent><br/
<hw>Sag"i*nate</hw> <pr>(?)</pr>, <pos>v. t.</pos> <ety>[L. <ets>saginatus</ets>, p. p. of <ets>saginare</ets> to fat, fr. <ets>sagina</ets> stuffing.]</ety> <def>To make fat; to pamper.</def> <mark>[R.]</mark> <ldquo/Many a <xex>saginated</xex> boar.<rdquo/ <rj><au>Cowper.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sagination</ent><br/
<hw>Sag`i*na"tion</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>saginatio</ets>.]</ety> <def>The act of fattening or pampering.</def> <mark>[R.]</mark> <rj><au>Topsell.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sagitta</ent><br/
||<hw>Sa*git"ta</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L., an arrow.]</ety> <sn>1.</sn> <fld>(Astron.)</fld> <def>A small constellation north of Aquila; the Arrow.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Arch.)</fld> <def>The keystone of an arch.</def> <mark>[R.]</mark> <rj><au>Gwilt.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <fld>(Geom.)</fld> <def>The distance from a point in a curve to the chord; also, the versed sine of an arc; -- so called from its resemblance to an arrow resting on the bow and string.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <fld>(Anat.)</fld> <def>The larger of the two otoliths, or ear bones, found in most fishes.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <fld>(Zool.)</fld> <def>A genus of transparent, free-swimming marine worms having lateral and caudal fins, and capable of swimming rapidly. It is the type of the class Chaetognatha.</def><br/
[<source>1913 Webster</source>]</p>

<p><-- p. 1268 pr=PCP --></p>

<p><ent>Sagittal</ent><br/
<hw>Sag"it*tal</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>sagitta</ets> an arrow: cf. F. <ets>sagittal</ets>.]</ety> <sn>1.</sn> <def>Of or pertaining to an arrow; resembling an arrow; furnished with an arrowlike appendage.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Anat.)</fld> <sd>(a)</sd> <def>Of or pertaining to the sagittal suture; in the region of the sagittal suture; rabdoidal; <as>as, the <ex>sagittal</ex> furrow, or groove, on the inner surface of the roof of the skull</as>.</def> <sd>(b)</sd> <def>In the mesial plane; mesial; <as>as, a <ex>sagittal</ex> section of an animal</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Sagittal suture</b></col> <fld>(Anat.)</fld>, <cd>the suture between the two parietal bones in the top of the skull; -- called also <altname>rabdoidal suture</altname>, and <altname>interparietal suture</altname>.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sagittarius</ent><br/
||<hw>Sag`it*ta"ri*us</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L., literally, an archer, fr. <ets>sagittarius</ets> belonging to an arrow, fr. <ets>sagitta</ets> an arrow.]</ety> <fld>(Astron.)</fld> <sd>(a)</sd> <def>The ninth of the twelve signs of the zodiac, which the sun enters about November 22, marked thus [<sagittarius/] in almanacs; the Archer.</def> <sd>(b)</sd> <def>A zodiacal constellation, represented on maps and globes as a centaur shooting an arrow.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sagittary</ent><br/
<hw>Sag"it*ta"ry</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[See <er>Sagittarius</er>.]</ety> <sn>1.</sn> <fld>(Myth.)</fld> <def>A centaur; a fabulous being, half man, half horse, armed with a bow and quiver.</def> <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>The Arsenal in Venice; -- so called from having a figure of an archer over the door.</def> <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sagittary</ent><br/
<hw>Sag"it*ta*ry</hw>, <pos>a.</pos> <ety>[L. <ets>sagittarius</ets>.]</ety> <def>Pertaining to, or resembling, an arrow.</def> <rj><au>Sir T. Browne.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sagittate</ent><br/
<hw>Sag"it*tate</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[NL. <ets>sagittatus</ets>, fr. L. <ets>sagitta</ets> an arrow.]</ety> <def>Shaped like an arrowhead; triangular, with the two basal angles prolonged downward.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sagittated</ent><br/
<hw>Sag"it*ta`ted</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Sagittal; sagittate.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sagittocyst</ent><br/
<hw>Sag"it*to*cyst</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[See <er>Sagitta</er>, and <er>Cyst</er>.]</ety> <fld>(Zool.)</fld> <def>A defensive cell containing a minute rodlike structure which may be expelled. Such cells are found in certain Turbellaria.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sago</ent><br/
<hw>Sa"go</hw> <pr>(s<amac/"g<osl/)</pr>, <pos>n.</pos> <ety>[Malay. <ets>s<amac/gu</ets>.]</ety> <def>A dry granulated starch imported from the East Indies, much used for making puddings and as an article of diet for the sick; also, as starch, for stiffening textile fabrics. It is prepared from the stems of several East Indian and Malayan palm trees, but chiefly from the <spn>Metroxylon Sagu</spn>; also from several cycadaceous plants (<spn>Cycas revoluta</spn>, <spn>Zamia integrifolia</spn>, etc.).</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Portland sago</b></col>, <cd>a kind of sago prepared from the corms of the cuckoopint (<spn>Arum maculatum</spn>).</cd> -- <col><b>Sago palm</b></col>. <fld>(Bot.)</fld> <sd>(a)</sd> <cd>A palm tree which yields sago</cd>. <sd>(b)</sd> <cd>A species of Cycas (<spn>Cycas revoluta</spn>).</cd> -- <col><b>Sago spleen</b></col> <fld>(Med.)</fld>, <cd>a morbid condition of the spleen, produced by amyloid degeneration of the organ, in which a cross section shows scattered gray translucent bodies looking like grains of sago.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sagoin</ent><br/
<hw>Sa*goin"</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F. <ets>sagouin</ets>(formed from the native South American name).]</ety> <fld>(Zool.)</fld> <def>A marmoset; -- called also <altname>sagouin</altname>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sagum</ent><br/
||<hw>Sa"gum</hw> <pr>(?)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Saga</plw> <pr>(#)</pr>.</plu> <ety>[L. <ets>sagum</ets>, <ets>sagus</ets>; cf. Gr. <?/. Cf. <er>Say</er> a kind of serge.]</ety> <fld>(Rom. Antiq.)</fld> <def>The military cloak of the Roman soldiers.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sagus</ent><br/
||<hw>Sa"gus</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[NL. See <er>Sago</er>.]</ety> <fld>(Bot.)</fld> <def>A genus of palms from which sago is obtained.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sagy</ent><br/
<hw>Sa"gy</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Full of sage; seasoned with sage.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Saheb</ent><br/
<ent>Sahib</ent><br/
<mhw>||<hw>Sa"hib</hw> <pr>(?)</pr>, ||<hw>Sa"heb</hw> <pr>(<?/)</pr></mhw>, <pos>n.</pos> <ety>[Ar. <ets><cced/<amac/hib</ets> master, lord, fem. <ets><cced/<amac/hibah</ets>.]</ety> <def>A respectful title or appellation given to Europeans of rank.</def> <mark>[India]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sahibah</ent><br/
||<hw>Sa"hi*bah</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[See <er>Sahib</er>.]</ety> <def>A lady; mistress.</def> <mark>[India]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sahidic</ent><br/
<hw>Sa*hid"ic</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Same as <er>Thebaic</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sahlite</ent><br/
<hw>Sah"lite</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Min.)</fld> <def>See <er>Salite</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sahui</ent><br/
||<hw>Sa*hui"</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Zool.)</fld> <def>A marmoset.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sai</ent><br/
||<hw>Sa"i</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Cf. Pg. <ets>sahi</ets>.]</ety> <fld>(Zool.)</fld> <def>See <er>Capuchin</er>, 3 <sd>(a)</sd>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Saibling</ent><br/
||<hw>Sai"bling</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Dial. G.]</ety> <fld>(Zool.)</fld> <def>A European mountain trout (<spn>Salvelinus alpinus</spn>); -- called also <altname>Bavarian charr</altname>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Saic</ent><br/
<hw>Sa"ic</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F. <ets>sa<ium/que</ets>, Turk. <ets>sha<ium/ka</ets>.]</ety> <fld>(Naut.)</fld> <def>A kind of ketch very common in the Levant, which has neither topgallant sail nor mizzen topsail.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Said</ent><br/
<hw>Said</hw> <pr>(?)</pr>, <def><pos>imp. & p. p.</pos> of <er>Say</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Said</ent><br/
<hw>Said</hw>, <pos>a.</pos> <def>Before-mentioned; already spoken of or specified; aforesaid; -- used chiefly in legal style.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Saiga</ent><br/
||<hw>Sai"ga</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Russ. <ets>saika</ets>.]</ety> <fld>(Zool.)</fld> <def>An antelope (<spn>Saiga Tartarica</spn>) native of the plains of Siberia and Eastern Russia. The male has erect annulated horns, and tufts of long hair beneath the eyes and ears.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Saikyr</ent><br/
<hw>Sai"kyr</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Mil.)</fld> <def>Same as <er>Saker</er>.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sail</ent><br/
<hw>Sail</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[OE. <ets>seil</ets>, AS. <ets>segel</ets>, <ets>segl</ets>; akin to D. <ets>zeil</ets>, OHG. <ets>segal</ets>, G. & Sw. <ets>segel</ets>, Icel. <ets>segl</ets>, Dan. <ets>seil</ets>. <root/ 153.]</ety> <sn>1.</sn> <def>An extent of canvas or other fabric by means of which the wind is made serviceable as a power for propelling vessels through the water.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Behoves him now both <qex>sail</qex> and oar.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Anything resembling a sail, or regarded as a sail.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>A wing; a van.</def> <mark>[Poetic]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>Like an eagle soaring<br/
To weather his broad <qex>sails</qex>.</q> <rj><qau>Spenser.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>The extended surface of the arm of a windmill.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <def>A sailing vessel; a vessel of any kind; a craft.</def><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ In this sense, the plural has usually the same form as the singular; <as>as, twenty <ex>sail</ex> were in sight</as>.</note><br/
[<source>1913 Webster</source>]</p>

<p><sn>6.</sn> <def>A passage by a sailing vessel; a journey or excursion upon the water.</def><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ Sails are of two general kinds, <stype>fore-and-aft sails</stype>, and <stype>square sails</stype>. Square sails are always bent to yards, with their foot lying across the line of the vessel. Fore-and-aft sails are set upon stays or gaffs with their foot in line with the keel. A fore-and-aft sail is triangular, or quadrilateral with the after leech longer than the fore leech. Square sails are quadrilateral, but not necessarily square. See Phrases under <er>Fore</er>, <pos>a.</pos>, and <er>Square</er>, <pos>a.</pos>; also, <er>Bark</er>, <er>Brig</er>, <er>Schooner</er>, <er>Ship</er>, <er>Stay</er>.</note><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Sail burton</b></col> <fld>(Naut.)</fld>, <cd>a purchase for hoisting sails aloft for bending.</cd> -- <col><b>Sail fluke</b></col> <fld>(Zool.)</fld>, <cd>the whiff.</cd> -- <col><b>Sail hook</b></col>, <cd>a small hook used in making sails, to hold the seams square.</cd> -- <col><b>Sail loft</b></col>, <cd>a loft or room where sails are cut out and made.</cd> -- <col><b>Sail room</b></col> <fld>(Naut.)</fld>, <cd>a room in a vessel where sails are stowed when not in use.</cd> -- <col><b>Sail yard</b></col> <fld>(Naut.)</fld>, <cd>the yard or spar on which a sail is extended.</cd> -- <col><b>Shoulder-of-mutton sail</b></col> <fld>(Naut.)</fld>, <cd>a triangular sail of peculiar form. It is chiefly used to set on a boat's mast.</cd> -- <col><b>To crowd sail</b></col>. <fld>(Naut.)</fld> <cd>See under <er>Crowd</er>.</cd> -- <col><b>To loose sails</b></col> <fld>(Naut.)</fld>, <cd>to unfurl or spread sails.</cd> -- <col><b>To make sail</b></col> <fld>(Naut.)</fld>, <cd>to extend an additional quantity of sail.</cd> -- <col><b>To set a sail</b></col> <fld>(Naut.)</fld>, <cd>to extend or spread a sail to the wind.</cd> -- <col><b>To set sail</b></col> <fld>(Naut.)</fld>, <cd>to unfurl or spread the sails; hence, to begin a voyage.</cd> -- <col><b>To shorten sail</b></col> <fld>(Naut.)</fld>, <cd>to reduce the extent of sail, or take in a part.</cd> -- <col><b>To strike sail</b></col> <fld>(Naut.)</fld>, <cd>to lower the sails suddenly, as in saluting, or in sudden gusts of wind; hence, to acknowledge inferiority; to abate pretension.</cd> -- <col><b>Under sail</b></col>, <cd>having the sails spread.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sail</ent><br/
<hw>Sail</hw> <pr>(?)</pr>, <pos>v. i.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Sailed</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Sailing</conjf>.]</vmorph> <ety>[AS. <ets>segelian</ets>, <ets>seglian</ets>. See <er>Sail</er>, <pos>n.</pos>]</ety> <sn>1.</sn> <def>To be impelled or driven forward by the action of wind upon sails, as a ship on water; to be impelled on a body of water by the action of steam or other power.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To move through or on the water; to swim, as a fish or a water fowl.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>To be conveyed in a vessel on water; to pass by water; <as>as, they <ex>sailed</ex> from London to Canton</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>To set sail; to begin a voyage.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <def>To move smoothly through the air; to glide through the air without apparent exertion, as a bird.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>As is a winged messenger of heaven, . . .<br/
When he bestrides the lazy pacing clouds,<br/
And <qex>sails</qex> upon the bosom of the air.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sail</ent><br/
<hw>Sail</hw>, <pos>v. t.</pos> <sn>1.</sn> <def>To pass or move upon, as in a ship, by means of sails; hence, to move or journey upon (the water) by means of steam or other force.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>A thousand ships were manned to <qex>sail</qex> the sea.</q> <rj><qau>Dryden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To fly through; to glide or move smoothly through.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Sublime she <qex>sails</qex><br/
The aerial space, and mounts the wing<egrave/d gales.</q> <rj><qau>Pope.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>To direct or manage the motion of, as a vessel; <as>as, to <ex>sail</ex> one's own ship</as>.</def> <rj><au>Totten.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sailable</ent><br/
<hw>Sail"a*ble</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Capable of being sailed over; navigable; <as>as, a <ex>sailable</ex> river</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sailboat</ent><br/
<hw>Sail"boat`</hw>, <pos>n.</pos> <def>A boat propelled by a sail or sails.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sailcloth</ent><br/
<hw>Sail"cloth`</hw> <pr>(?)</pr>, <pos>n.</pos> <def>Duck or canvas used in making sails.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sailer</ent><br/
<hw>Sail"er</hw> <pr>(?)</pr>, <pos>n.</pos> <sn>1.</sn> <def>A sailor.</def> <mark>[R.]</mark> <rj><au>Sir P. Sidney.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A ship or other vessel; -- with qualifying words descriptive of speed or manner of sailing; <as>as, a heavy <ex>sailer</ex>; a fast <ex>sailer</ex></as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sailfish</ent><br/
<hw>Sail"fish</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Zool.)</fld> <sd>(a)</sd> <def>The banner fish, or spikefish (<gen>Histiophorus</gen>.)</def> <sd>(b)</sd> <def>The basking, or liver, shark.</def> <sd>(c)</sd> <def>The quillback.</def> <-- Illust. of Sailfish (Histiophorus Americanus) --><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sailing</ent><br/
<hw>Sail"ing</hw> <pr>(?)</pr>, <pos>n.</pos> <sn>1.</sn> <def>The act of one who, or that which, sails; the motion of a vessel on water, impelled by wind or steam; the act of starting on a voyage.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Naut.)</fld> <def>The art of managing a vessel; seamanship; navigation; <as>as, globular <ex>sailing</ex>; oblique <ex>sailing</ex></as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ For the several methods of sailing, see under <er>Circular</er>, <er>Globular</er>, <er>Oblique</er>, <er>Parallel</er>, etc.</note><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Sailing master</b></col> <fld>(U. S. Navy)</fld>, <cd>formerly, a warrant officer, ranking next below a lieutenant, whose duties were to navigate the vessel; and under the direction of the executive officer, to attend to the stowage of the hold, to the cables, rigging, etc. The grade was merged in that of <xex>master</xex> in 1862.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sailless</ent><br/
<hw>Sail"less</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Destitute of sails.</def> <rj><au>Pollok.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sailmaker</ent><br/
<hw>Sail"mak`er</hw> <pr>(?)</pr>, <pos>n.</pos> <def>One whose occupation is to make or repair sails.</def> -- <wordforms><wf>Sail"mak`ing</wf>, <pos>n.</pos></wordforms><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sailor</ent><br/
<hw>Sail"or</hw> <pr>(?)</pr>, <pos>n.</pos> <def>One who follows the business of navigating ships or other vessels; one who understands the practical management of ships; one of the crew of a vessel; a mariner; a common seaman.</def><br/
[<source>1913 Webster</source>]</p>

<p><syn><b>Syn.</b> -- Mariner; seaman; seafarer.</syn><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Sailor's choice</b></col>. <fld>(Zool.)</fld> <sd>(a)</sd> <cd>An excellent marine food fish (<spn>Diplodus rhomboides</spn>, syn. <spn>Lagodon rhomboides</spn>) of the <geog>Southern United States</geog>; -- called also <altname>porgy</altname>, <altname>squirrel fish</altname>, <altname>yellowtail</altname>, and <altname>salt-water bream</altname>.</cd> <sd>(b)</sd> <cd>A species of grunt (<spn>Orthopristis chrysopterus</spn> syn. <spn>Pomadasys chrysopterus</spn>), an excellent food fish common on the southern coasts of the United States; -- called also <altname>hogfish</altname>, and <altname>pigfish</altname>.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Saily</ent><br/
<hw>Sail"y</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Like a sail.</def> <mark>[R.]</mark> <rj><au>Drayton.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Saim</ent><br/
<hw>Saim</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[OF. <ets>sain</ets>, LL. <ets>saginum</ets>, fr. L. <ets>sagina</ets> a fattening.]</ety> <def>Lard; grease.</def> <mark>[Scot. & Prov. Eng.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Saimir</ent><br/
||<hw>Sai*mir"</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Zool.)</fld> <def>The squirrel monkey.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sain</ent><br/
<hw>Sain</hw> <pr>(?)</pr>, <mark>obs.</mark> <pos>p. p.</pos> <mord>of <er>Say</er>, for <ets>sayen</ets></mord>. <def>Said.</def> <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sain</ent><br/
<hw>Sain</hw>, <pos>v. t.</pos> <ety>[Cf. <ets>Saint</ets>, <ets>Sane</ets>.]</ety> <def>To sanctify; to bless so as to protect from evil influence.</def> <mark>[R.]</mark> <rj><au>Sir W. Scott.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sainfoin</ent><br/
<hw>Sain"foin</hw> <pr>(?; 277)</pr>, <pos>n.</pos> <ety>[F., fr. <ets>sain</ets> wholesome (L. <ets>sanus</ets>; see <er>Sane</er>.) + <ets>foin</ets> hay (L. <ets>faenum</ets>); or perh. fr. <ets>saint</ets> sacred (L. <ets>sanctus</ets>; see <er>Saint</er>) + <ets>foin</ets> hay.]</ety> <fld>(Bot.)</fld> <sd>(a)</sd> <def>A leguminous plant (<spn>Onobrychis sativa</spn>) cultivated for fodder.</def> <altsp>[Written also <asp>saintfoin</asp>.]</altsp> <sd>(b)</sd> <def>A kind of tick trefoil (<spn>Desmodium Canadense</spn>).</def> <mark>[Canada]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Saint</ent><br/
<hw>Saint</hw> <pr>(s<amac/nt)</pr>, <pos>n.</pos> <ety>[F., fr. L. <ets>sanctus</ets> sacred, properly p. p. of <ets>sancire</ets> to render sacred by a religious act, to appoint as sacred; akin to <ets>sacer</ets> sacred. Cf. <er>Sacred</er>, <er>Sanctity</er>, <er>Sanctum</er>, <er>Sanctus</er>.]</ety> <sn>1.</sn> <def>A person sanctified; a holy or godly person; one eminent for piety and virtue; any true Christian, as being redeemed and consecrated to God.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Them that are sanctified in Christ Jesus, called to be <qex>saints</qex>.</q> <rj><qau>1 Cor. i. 2.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>One of the blessed in heaven.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Then shall thy <qex>saints</qex>, unmixed, and from the impure<br/
Far separate, circling thy holy mount,<br/
Unfeigned hallelujahs to thee sing.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <fld>(Eccl.)</fld> <def>One canonized by the church.</def> <altsp>[Abbrev. <abbr>St.</abbr>]</altsp><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Saint Andrew's cross</b></col>. <sd>(a)</sd> <cd>A cross shaped like the letter X. See <xex>Illust.</xex> 4, under <er>Cross</er>.</cd> <sd>(b)</sd> <fld>(Bot.)</fld> <cd>A low North American shrub (<spn>Ascyrum Crux-Andreae</spn>, the petals of which have the form of a Saint Andrew's cross.</cd> <au>Gray.</au> -- <col><b>Saint Anthony's cross</b></col>, <cd>a <universbold>T</universbold>-shaped cross. See <xex>Illust.</xex> 6, under <er>Cross</er>.</cd> -- <col><b>Saint Anthony's fire</b></col>, <cd>the erysipelas; -- popularly so called because it was supposed to have been cured by the intercession of Saint Anthony.</cd> -- <col><b>Saint Anthony's nut</b></col> <fld>(Bot.)</fld>, <cd>the groundnut (<spn>Bunium flexuosum</spn>); -- so called because swine feed on it, and St. Anthony was once a swineherd.</cd> <au>Dr. Prior.</au> -- <col><b>Saint Anthony's turnip</b></col> <fld>(Bot.)</fld>, <cd>the bulbous crowfoot, a favorite food of swine.</cd> <au>Dr. Prior.</au> -- <col><b>Saint Barnaby's thistle</b></col> <fld>(Bot.)</fld>, <cd>a kind of knapweed (<spn>Centaurea solstitialis</spn>) flowering on St. Barnabas's Day, June 11th.</cd> <au>Dr. Prior.</au> -- <col><b>Saint Bernard</b></col> <fld>(Zool.)</fld>, <cd>a breed of large, handsome dogs celebrated for strength and sagacity, formerly bred chiefly at the Hospice of St. Bernard in Switzerland, but now common in Europe and America. There are two races, the smooth-haired and the rough-haired. See <xex>Illust.</xex> under <er>Dog</er>.</cd> -- <col><b>Saint Catharine's flower</b></col> <fld>(Bot.)</fld>, <cd>the plant love-in-a-mist. See under <er>Love</er>.</cd> -- <col><b>Saint Cuthbert's beads</b></col> <fld>(Paleon.)</fld>, <cd>the fossil joints of crinoid stems.</cd> -- <col><b>Saint Dabeoc's heath</b></col> <fld>(Bot.)</fld>, <cd>a heatherlike plant (<spn>Daboecia polifolia</spn>), named from an Irish saint.</cd> -- <col><b>Saint Distaff's Day</b></col>. <cd>See under <er>Distaff</er>.</cd> -- <col><b>Saint Elmo's fire</b></col>, <cd>a luminous, flamelike appearance, sometimes seen in dark, tempestuous nights, at some prominent point on a ship, particularly at the masthead and the yardarms. It has also been observed on land, and is due to the discharge of electricity from elevated or pointed objects. A single flame is called a <stype>Helena</stype>, or a <stype>Corposant</stype>; a double, or twin, flame is called a <stype>Castor and Pollux</stype>, or a <stype>double Corposant</stype>. It takes its name from St. Elmo, the patron saint of sailors.</cd> -- <col><b>Saint George's cross</b></col> <fld>(Her.)</fld>, <cd>a Greek cross gules upon a field argent, the field being represented by a narrow fimbriation in the ensign, or union jack, of Great Britain.</cd> -- <col><b>Saint George's ensign</b></col>, <cd>a red cross on a white field with a union jack in the upper corner next the mast. It is the distinguishing badge of ships of the royal navy of England; -- called also <altname>the white ensign</altname>.</cd> <au>Brande & C.</au> -- <col><b>Saint George's flag</b></col>, <cd>a smaller flag resembling the ensign, but without the union jack; used as the sign of the presence and command of an admiral.</cd> <mark>[Eng.]</mark> <au>Brande & C.</au> -- <col><b>Saint Gobain glass</b></col> <fld>(Chem.)</fld>, <cd>a fine variety of soda-lime plate glass, so called from St. Gobain in France, where it was manufactured.</cd> -- <col><b>Saint Ignatius's bean</b></col> <fld>(Bot.)</fld>, <cd>the seed of a tree of the Philippines (<spn>Strychnos Ignatia</spn>), of properties similar to the nux vomica.</cd> -- <col><b>Saint James's shell</b></col> <fld>(Zool.)</fld>, <cd>a pecten (<spn>Vola Jacobaeus</spn>) worn by pilgrims to the Holy Land. See <xex>Illust.</xex> under <er>Scallop</er>.</cd> -- <col><b>Saint James's-wort</b></col> <fld>(Bot.)</fld>, <cd>a kind of ragwort (<spn>Senecio Jacobaea</spn>).</cd> -- <col><b>Saint John's bread</b></col>. <fld>(Bot.)</fld> <cd>See <er>Carob</er>.</cd> -- <col><b>Saint John's-wort</b></col> <fld>(Bot.)</fld>, <cd>any plant of the genus <gen>Hypericum</gen>, most species of which have yellow flowers; -- called also <altname>John's-wort</altname>.</cd> -- <col><b>Saint Leger</b></col>, <cd>the name of a race for three-year-old horses run annually in September at Doncaster, England; -- instituted in 1776 by Col. St. Leger.</cd> -- <col><b>Saint Martin's herb</b></col> <fld>(Bot.)</fld>, <cd>a small tropical American violaceous plant (<spn>Sauvagesia erecta</spn>). It is very mucilaginous and is used in medicine.</cd> <-- p. 1269 pr=PCP --> -- <col><b>Saint Martin's summer</b></col>, <cd>a season of mild, damp weather frequently prevailing during late autumn in England and the Mediterranean countries; -- so called from St. Martin's Festival, occurring on November 11. It corresponds to the Indian summer in America.</cd> <au>Shak.</au> <au>Whittier.</au> -- <col><b>Saint Patrick's cross</b></col>. <cd>See <xex>Illust.</xex> 4, under <er>Cross</er>.</cd> -- <col><b>Saint Patrick's Day</b></col>, <cd>the 17th of March, anniversary of the death (about 466) of St. Patrick, the apostle and patron saint of Ireland.</cd> -- <col><b>Saint Peter's fish</b></col>. <fld>(Zool.)</fld> <cd>See <cref>John Dory</cref>, under <er>John</er>.</cd> -- <col><b>Saint Peter's-wort</b></col> <fld>(Bot.)</fld>, <cd>a name of several plants, as <spn>Hypericum Ascyron</spn>, <spn>Hypericum quadrangulum</spn>, <spn>Ascyrum stans</spn>, etc.</cd> -- <col><b>Saint Peter's wreath</b></col> <fld>(Bot.)</fld>, <cd>a shrubby kind of <gen>Spiraea</gen> (<spn>Spiraea hypericifolia</spn>), having long slender branches covered with clusters of small white blossoms in spring.</cd> -- <col><b>Saint's bell</b></col>. <cd>See <cref>Sanctus bell</cref>, under <er>Sanctus</er>.</cd> -- <col><b>Saint Vitus's dance</b></col> <fld>(Med.)</fld>, <cd>chorea; -- so called from the supposed cures wrought on intercession to this saint.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Saint</ent><br/
<hw>Saint</hw> <pr>(s<amac/nt)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Sainted</conjf>; <pos>p. pr. & vb. n.</pos> <conjf>Sainting</conjf>.]</vmorph> <def>To make a saint of; to enroll among the saints by an offical act, as of the pope; to canonize; to give the title or reputation of a saint to (some one).</def><br/
[<source>1913 Webster</source>]</p>

<p><q>A large hospital, erected by a shoemaker who has been beatified, though never <qex>sainted</qex>.</q> <rj><qau>Addison.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>To saint it</b></col>, <cd>to act as a saint, or with a show of piety.</cd><br/
[<source>1913 Webster</source>]</p>

<p><q>Whether the charmer sinner it or <qex>saint it</qex>.</q> <rj><qau>Pope.</qau></rj></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Saint</ent><br/
<hw>Saint</hw>, <pos>v. i.</pos> <def>To act or live as a saint.</def> <mark>[R.]</mark> <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Saintdom</ent><br/
<hw>Saint"dom</hw> <pr>(-d<ucr/m)</pr>, <pos>n.</pos> <def>The state or character of a saint.</def> <mark>[R.]</mark> <rj><au>Tennyson.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sainted</ent><br/
<hw>Saint"ed</hw>, <pos>a.</pos> <sn>1.</sn> <def>Consecrated; sacred; holy; pious.</def> <ldquo/A most <xex>sainted</xex> king.<rdquo/ <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Amongst the enthroned gods on <qex>sainted</qex> seats.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Entered into heaven; -- a euphemism for <altname>dead</altname>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Saintess</ent><br/
<hw>Saint"ess</hw>, <pos>n.</pos> <def>A female saint.</def> <mark>[R.]</mark> <rj><au>Bp. Fisher.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sainthood</ent><br/
<hw>Saint"hood</hw> <pr>(?)</pr>, <pos>n.</pos> <sn>1.</sn> <def>The state of being a saint; the condition of a saint.</def> <rj><au>Walpole.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>The order, or united body, of saints; saints, considered collectively.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>It was supposed he felt no call to any expedition that might endanger the reign of the military <qex>sainthood</qex>.</q> <rj><qau>Sir W. Scott.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Saintish</ent><br/
<hw>Saint"ish</hw>, <pos>a.</pos> <def>Somewhat saintlike; -- used ironically.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Saintism</ent><br/
<hw>Saint"ism</hw> <pr>(?)</pr>, <pos>n.</pos> <def>The character or quality of saints; also, hypocritical pretense of holiness.</def> <rj><au>Wood.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Saintlike</ent><br/
<hw>Saint"like`</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Resembling a saint; suiting a saint; becoming a saint; saintly.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Glossed over only with a <qex>saintlike</qex> show.</q> <rj><qau>Dryden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Saintliness</ent><br/
<hw>Saint"li*ness</hw> <pr>(?)</pr>, <pos>n.</pos> <def>Quality of being saintly.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Saintly</ent><br/
<hw>Saint"ly</hw>, <pos>a.</pos> <amorph>[<pos>Compar.</pos> <adjf>Saintlier</adjf> <pr>(?)</pr>; <pos>superl.</pos> <adjf>Saintliest</adjf>.]</amorph> <def>Like a saint; becoming a holy person.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>So dear to Heaven is <qex>saintly</qex> chastity.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>St. Nicholas</ent><br/
<ent>Saint Nicholas</ent><br/
<mhw><hw>Saint Nicholas</hw>, <hw>St. Nicholas</hw></mhw> <pr>(?)</pr>, <pos>n.</pos> <def>A Dutch saint, who was reputed to bring gifts to children on Christmas even, giving rise to the modern legend of <altname>Santa Claus</altname>.</def><br/
[<source>PJC</source>]</p>

<p><cs><col><b>A Visit from St. Nicholas</b></col> <cd>The original name for a poem by <person>Clement Clarke Moore</person>, popularly called titled <altname>The Night Before Christmans</altname>.  It is a popular poem with the theme of <person>St. Nicholas</person> (Santa Claus) coming to bring gifts to children on Christmans eve.  See <er>Night Before Christmas</er> in the vocabulary.</cd></cs><br/
[<source>PJC</source>]</p>

<p><ent>Saintologist</ent><br/
<hw>Saint*ol"o*gist</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[<ets>Saint</ets> + <ets>-logy</ets> + <ets>-ist</ets>.]</ety> <fld>(Theol.)</fld> <def>One who writes the lives of saints.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Saintship</ent><br/
<hw>Saint"ship</hw>, <pos>n.</pos> <def>The character or qualities of a saint.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Saint-Simonian</ent><br/
<hw>Saint`-Si*mo"ni*an</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A follower of the <person>Count de <etsep>St. Simon</etsep></person>, who died in 1825, and who maintained that the principle of property held in common, and the just division of the fruits of common labor among the members of society, are the true remedy for the social evils which exist.</def> <rj><au>Brande & C.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Saint-Simonianism</ent><br/
<hw>Saint`-Si*mo"ni*an*ism</hw> <pr>(?)</pr>, <pos>n.</pos> <def>The principles, doctrines, or practice of the Saint-Simonians; -- called also <altname>Saint- Simonism</altname>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Saint-Simonism</ent><br/
<hw>Saint-Si"mon*ism</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A system of socialism in which the state owns all the property and the laborer is entitled to share according to the quality and  amount of his work, founded by <etsep>Saint Simon</etsep> (1760-1825); -- called also <altname>Saint- Simonianism</altname>.</def><br/
[<source>Webster 1913 Suppl.</source> <source>+PJC</source>]</p>

<p><ent>Saith</ent><br/
<hw>Saith</hw> <pr>(?)</pr>, <def><pos>3d pers. sing. pres.</pos> of <er>Say</er>.</def> <mark>[Archaic]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Saithe</ent><br/
<hw>Saithe</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Gael. <ets>saoidheam</ets>.]</ety> <fld>(Zool.)</fld> <def>The pollock, or coalfish; -- called also <altname>sillock</altname>.</def> <mark>[Scot.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Saiva</ent><br/
<hw>Sai"va</hw> <pr>(? <or/ ?)</pr>, <pos>n.</pos> <ety>[Skr. <ets><cced/aiva</ets> devoted to <ets>Siva</ets>.]</ety> <def>One of an important religious sect in India which regards Siva with peculiar veneration.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Saivism</ent><br/
<hw>Sai"vism</hw> <pr>(?)</pr>, <pos>n.</pos> <def>The worship of Siva.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sajene</ent><br/
<hw>Sa*jene"</hw> <pr>(?)</pr>, <pos>n.</pos> <def>Same as <er>Sagene</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sajou</ent><br/
<hw>Sa"jou</hw> <pr>(?; F. <?/)</pr>, <pos>n.</pos> <fld>(Zool.)</fld> <def>Same as <er>Sapajou</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sake</ent><br/
<hw>Sake</hw> <pr>(s<amac/k)</pr>, <pos>n.</pos> <ety>[OE. <ets>sake</ets> cause, also, lawsuit, fault, AS. <ets>sacu</ets> strife, a cause or suit at law; akin to D. <ets>zaak</ets> cause, thing, affair, G. <ets>sache</ets> thing, cause in law, OHG. <ets>sahha</ets>, Icel. <ets>s<oum/k</ets>, Sw. <ets>sak</ets>, Dan. <ets>sag</ets>, Goth. <ets>sakj<omac/</ets> strife, AS. <ets>sacan</ets> to contend, strive, Goth. <ets>sakam</ets>, Icel. <ets>saka</ets> to contend, strive, blame, OHG. <ets>sahhan</ets>, MHG. <ets>sachen</ets>, to contend, strive, defend one's right, accuse, charge in a lawsuit, and also to E. <ets>seek</ets>. Cf. <er>Seek</er>.]</ety> <def>Final cause; end; purpose of obtaining; cause; motive; reason; interest; concern; account; regard or respect; -- used chiefly in such phrases as, <xex>for the sake of</xex>, <xex>for his sake</xex>, <xex>for man's sake</xex>, <xex>for mercy's sake</xex>, and the like; <as>as, to commit crime for the <ex>sake</ex> of gain; to go abroad for the <ex>sake</ex> of one's health</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Moved with wrath and shame and ladies' <qex>sake</qex>.</q> <rj><qau>Spenser.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>I will not again curse the ground any more for man's <qex>sake</qex>.</q> <rj><qau>Gen. viii. 21.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Will he draw out,<br/
For anger's <qex>sake</qex>, finite to infinite?</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Knowledge is for the <qex>sake</qex> of man, and not man for the <qex>sake</qex> of knowledge.</q> <rj><qau>Sir W. Hamilton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ The <xex>-s</xex> of the possessive case preceding <xex>sake</xex> is sometimes omitted for euphony; as, for goodness <xex>sake</xex>. <ldquo/<xex>For conscience sake</xex>.<rdquo/ <au>1 Cor. x. 28.</au> The plural <xex>sakes</xex> is often used with a possessive plural. <ldquo/For both our <xex>sakes</xex>.<rdquo/ <au>Shak.</au></note><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sake</ent><br/
<hw>Sa"ke</hw> <pr>(s<aum/"k<esl/)</pr>, <pos>n.</pos> <def>a traditional alcoholic drink of Japan. It is made from rice.</def> <altsp>[Also spelled <asp>saki</asp>.]</altsp><br/
[<source>1913 Webster</source> <source>+PJC</source>]</p>

<p><ent>Saker</ent><br/
<hw>Sa"ker</hw> <pr>(s<amac/"k<etil/r)</pr>, <pos>n.</pos> <ety>[F. <ets>sacre</ets> (cf. It. <ets>sagro</ets>, Sp. & Pg. <ets>sacre</ets>), either fr. L. <ets>sacer</ets> sacred, holy, as a translation of Gr. <grk>"ie`rax</grk> falcon, from <grk>"iero`s</grk> holy, or more probably from Ar. <ets><cced/aqr</ets> hawk.]</ety> <altsp>[Written also <asp>sacar</asp>, <asp>sacre</asp>.]</altsp> <sn>1.</sn> <fld>(Zool.)</fld> <sd>(a)</sd> <def>A falcon (<spn>Falco sacer</spn>) native of Southern Europe and Asia, closely resembling the lanner.</def><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ The female is called <stype>chargh</stype>, and the male <stype>charghela</stype>, or <stype>sakeret</stype>.</note><br/
[<source>1913 Webster</source>]</p>

<p><sd>(b)</sd> <def>The peregrine falcon.</def> <mark>[Prov. Eng.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Mil.)</fld> <def>A small piece of artillery.</def> <rj><au>Wilhelm.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>On the bastions were planted culverins and <qex>sakers</qex>.</q> <rj><qau>Macaulay.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>The culverins and <qex>sakers</qex> showing their deadly muzzles over the rampart.</q> <rj><qau>Hawthorne.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sakeret</ent><br/
<hw>Sa"ker*et</hw> <pr>(s<amac/"k<etil/r*<ecr/t)</pr>, <pos>n.</pos> <ety>[F. <ets>sacret</ets>. See <er>Saker</er>.]</ety> <fld>(Zool.)</fld> <def>The male of the saker <sd>(a)</sd>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Saki</ent><br/
<hw>Sa"ki</hw> <pr>(s<amac/"k<icr/)</pr>, <pos>n.</pos> <ety>[Cf. F. & Pg. <ets>saki</ets>; probably from the native name.]</ety> <fld>(Zool.)</fld> <def>Any one of several species of South American monkeys of the genus <gen>Pithecia</gen>. They have large ears, and a long hairy tail which is not prehensile.</def><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ The black saki (<spn>Pithecia satanas</spn>), the white-headed (<spn>Pithecia leucocephala</spn>), and the red-backed, or hand-drinking, saki (<spn>Pithecia chiropotes</spn>), are among the best-known.</note><br/
[<source>1913 Webster</source>]</p>

<p><ent>Saki</ent><br/
<hw>Sa"ki</hw> <pr>(s<aum/"k<esl/)</pr>, <pos>n.</pos> <def>The alcoholic drink of Japan. It is made from rice; it is usually spelled <asp>sake</asp>.</def><br/
[<source>1913 Webster</source> <source>+PJC</source>]</p>

<p><ent>Sakiyeh</ent><br/
<ent>Sakieh</ent><br/
<mhw>{ <hw>Sak"i*eh</hw> <pr>(?)</pr>, <hw>Sak"i*yeh</hw> <pr>(?)</pr>, }</mhw> <pos>n.</pos>} <ety>[Ar. <ets>s<amac/q<imac/ah</ets> canal, trench.]</ety> <def>A kind of water wheel used in Egypt for raising water, from wells or pits, in buckets attached to its periphery or to an endless rope.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Sakti</ent><br/
<hw>Sak"ti</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Skr.]</ety> <fld>(Hind. Myth.)</fld> <def>The divine energy, personified as the wife of a deity (Brahma, Vishnu, Siva, etc.); the female principle.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sal</ent><br/
||<hw>Sal</hw> <pr>(s<add/l)</pr>, <pos>n.</pos> <ety>[Hind. <ets>s<amac/l</ets>, Skr. <ets><cced/<amac/la</ets>.]</ety> <fld>(Bot.)</fld> <def>An East Indian timber tree (<spn>Shorea robusta</spn>), much used for building purposes. It is of a light brown color, close-grained, heavy, and durable.</def> <altsp>[Written also <asp>saul</asp>.]</altsp><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sal</ent><br/
<hw>Sal</hw> <pr>(s<acr/l)</pr>, <pos>n.</pos> <ety>[L. See <er>Salt</er>.]</ety> <fld>(Chem. & Pharm.)</fld> <def>Salt.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Sal absinthii</b></col> <ety>[NL.]</ety> <fld>(Old Chem.)</fld>, <cd>an impure potassium carbonate obtained from the ashes of wormwood (<spn>Artemisia Absinthium</spn>).</cd> -- <col><b>Sal acetosellae</b></col> <ety>[NL.]</ety> <fld>(Old Chem.)</fld>, <cd>salt of sorrel.</cd> -- <col><b>Sal alembroth</b></col>. <fld>(Old Chem.)</fld> <cd>See <er>Alembroth</er>.</cd> -- <col><b>Sal ammoniac</b></col> <fld>(Chem.)</fld>, <cd>ammonium chloride, <chform>NH4Cl</chform>, a white crystalline volatile substance having a sharp salty taste, obtained from gas works, from nitrogenous matter, etc. It is largely employed as a source of ammonia, as a reagent, and as an expectorant in bronchitis. So called because originally made from the soot from camel's dung at the temple of Jupiter <etsep>Ammon</etsep> in Africa. Called also <altname>muriate of ammonia</altname>.</cd> -- <col><b>Sal catharticus</b></col> <ety>[NL.]</ety> <fld>(Old Med. Chem.)</fld>, <cd>Epsom salts.</cd> -- <col><b>Sal culinarius</b></col> <ety>[L.]</ety> <fld>(Old Chem.)</fld>, <cd>common salt, or sodium chloride.</cd> -- <col><b>Sal Cyrenaicus</b></col>. <ety>[NL.]</ety> <fld>(Old Chem.)</fld> <cd>See <cref>Sal ammoniac</cref> above.</cd> -- <mcol><col><b>Sal de duobus</b></col>, <col><b>Sal duplicatum</b></col></mcol> <ety>[NL.]</ety> <fld>(Old Chem.)</fld>, <cd>potassium sulphate; -- so called because erroneously supposed to be composed of two salts, one acid and one alkaline.</cd> -- <col><b>Sal diureticus</b></col> <ety>[NL.]</ety> <fld>(Old Med. Chem.)</fld>, <cd>potassium acetate.</cd> -- <col><b>Sal enixum</b></col> <ety>[NL.]</ety> <fld>(Old Chem.)</fld>, <cd>acid potassium sulphate.</cd> -- <col><b>Sal gemmae</b></col> <ety>[NL.]</ety> <fld>(Old Min.)</fld>, <cd>common salt occuring native.</cd> -- <col><b>Sal Jovis</b></col> <ety>[NL.]</ety> <fld>(Old Chem.)</fld>, <cd>salt tin, or stannic chloride; -- the alchemical name of tin being <etsep>Jove</etsep>.</cd> -- <col><b>Sal Martis</b></col> <ety>[NL.]</ety> <fld>(Old Chem.)</fld>, <cd>green vitriol, or ferrous sulphate; -- the alchemical name of iron being <etsep>Mars</etsep>.</cd> -- <col><b>Sal microcosmicum</b></col> <ety>[NL.]</ety> <fld>(Old Chem.)</fld> <cd>See <cref>Microcosmic salt</cref>, under <er>Microcosmic</er>.</cd> -- <col><b>Sal plumbi</b></col> <ety>[NL.]</ety> <fld>(Old Chem.)</fld>, <cd>sugar of lead.</cd> -- <col><b>Sal prunella</b></col>. <fld>(Old Chem.)</fld> <cd>See <cref>Prunella salt</cref>, under 1st <er>Prunella</er>.</cd> -- <col><b>Sal Saturni</b></col> <ety>[NL.]</ety> <fld>(Old Chem.)</fld>, <cd>sugar of lead, or lead acetate; -- the alchemical name of lead being <etsep>Saturn</etsep>.</cd> -- <col><b>Sal sedativus</b></col> <ety>[NL.]</ety> <fld>(Old Chem.)</fld>, <cd>sedative salt, or boric acid.</cd> -- <col><b>Sal Seignette</b></col> <ety>[F. <ets>seignette</ets>, <ets>sel de seignette</ets>]</ety> <fld>(Chem.)</fld>, <cd>Rochelle salt.</cd> -- <col><b>Sal soda</b></col> <fld>(Chem.)</fld>, <cd>sodium carbonate. See under <er>Sodium</er>.</cd> -- <col><b>Sal vitrioli</b></col> <ety>[NL.]</ety> <fld>(Old Chem.)</fld>, <cd>white vitriol; zinc sulphate.</cd> -- <col><b>Sal volatile</b></col>. <ety>[NL.]</ety> <sd>(a)</sd> <fld>(Chem.)</fld> <cd>See <cref>Sal ammoniac</cref>, above.</cd> <sd>(b)</sd> <cd>Spirits of ammonia.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Salaam</ent><br/
||<hw>Sa*laam"</hw> <pr>(s<adot/*l<aum/m")</pr>, <pos>n.</pos> <def>Same as <er>Salam</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Finally, Josiah might have made his <qex>salaam</qex> to the exciseman just as he was folding up that letter.</q> <rj><qau>Prof. Wilson.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Salaam</ent><br/
<hw>Sa*laam"</hw>, <pos>v. i.</pos> <def>To make or perform a salam.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>I have <qex>salaamed</qex> and kowtowed to him.</q> <rj><qau>H. James.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Salability</ent><br/
<hw>Sal`a*bil"i*ty</hw> <pr>(?)</pr>, <pos>n.</pos> <def>The quality or condition of being salable; salableness.</def> <rj><au>Duke of Argyll.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Salable</ent><br/
<hw>Sal"a*ble</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[From <er>Sale</er>.]</ety> <def>Capable of being sold; fit to be sold; finding a ready market.</def> -- <wordforms><wf>Sal"a*ble*ness</wf>, <pos>n.</pos> -- <wf>Sal"a*bly</wf>, <pos>adv.</pos></wordforms><br/
[<source>1913 Webster</source>]</p>

<p><ent>Salacious</ent><br/
<hw>Sa*la"cious</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>salax</ets>, <ets>-acis</ets>, fond of leaping, lustful, fr. <ets>salire</ets> to leap. See <er>Salient</er>.]</ety> <def>Having a propensity to venery; lustful; lecherous.</def> <rj><au>Dryden.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p>-- <wordforms><wf>Sa*la"cious*ly</wf>, <pos>adv.</pos> -- <wf>Sa*la"cious*ness</wf>, <pos>n.</pos></wordforms><br/
[<source>1913 Webster</source>]</p>

<p><ent>Salacity</ent><br/
<hw>Sa*lac"i*ty</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>salacitas</ets>: cf. F. <ets>salacit<eacute/</ets>]</ety> <def>Strong propensity to venery; lust; lecherousness.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Salad</ent><br/
<hw>Sal"ad</hw> <pr>(s<acr/l"<ait/d)</pr>, <pos>n.</pos> <ety>[F. <ets>salade</ets>, OIt. <ets>salata</ets>, It. <ets>insalata</ets>, fr. <ets>salare</ets> to salt, fr. L. <ets>sal</ets> salt. See <er>Salt</er>, and cf. <er>Slaw</er>.]</ety> <sn>1.</sn> <def>A preparation of vegetables, as lettuce, celery, water cress, onions, etc., usually dressed with salt, vinegar, oil, and spice, and eaten for giving a relish to other food; <as>as, lettuce <ex>salad</ex>; tomato <ex>salad</ex>, etc.</as></def><br/
[<source>1913 Webster</source>]</p>

<p><q>Leaves eaten raw are termed <qex>salad</qex>.</q> <rj><qau>I. Watts.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A dish composed of chopped meat or fish, esp. chicken or lobster, mixed with lettuce or other vegetables, and seasoned with oil, vinegar, mustard, and other condiments; <as>as, chicken <ex>salad</ex>; lobster <ex>salad</ex></as>.</def><-- mention mayonnaise --><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Salad burnet</b></col> <fld>(Bot.)</fld>, <cd>the common burnet (<spn>Poterium Sanguisorba</spn>), sometimes eaten as a salad in Italy.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Salad days</ent><br/
<hw>Sal"ad days`</hw> <pr>(?)</pr>, <pos>n. pl.</pos> <def>a period when a person is young and inexperienced.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Salade</ent><br/
<hw>Sal"ade</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A helmet. See <er>Sallet</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Salading</ent><br/
<hw>Sal"ad*ing</hw> <pr>(?)</pr>, <pos>n.</pos> <def>Vegetables for salad.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Salaeratus</ent><br/
<hw>Sal`ae*ra"tus</hw> <pr>(?)</pr>, <pos>n.</pos> <def>See <er>Saleratus</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Salagane</ent><br/
<hw>Sal"a*gane</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[From the Chinese name.]</ety> <fld>(Zool.)</fld> <def>The esculent swallow. See under <er>Esculent</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Salal-berry</ent><br/
<hw>Sal"al-ber`ry</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Probably of American Indian origin.]</ety> <fld>(Bot.)</fld> <def>The edible fruit of the <spn>Gaultheria Shallon</spn>, an ericaceous shrub found from California northwards. The berries are about the size of a common grape and of a dark purple color.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Salam</ent><br/
||<hw>Sa*lam</hw> <pr>(s<adot/*l<aum/m")</pr>, <pos>n.</pos> <ety>[Ar. <ets>sal<amac/m</ets> peace, safety.]</ety> <def>A salutation or compliment of ceremony in the east by word or act; an obeisance, performed by bowing very low and placing the right palm on the forehead.</def> <altsp>[Written also <asp>salaam</asp>.]</altsp><br/
[<source>1913 Webster</source>]</p>

<p><ent>Salamander</ent><br/
<hw>Sal"a*man`der</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F. <ets>salamandre</ets>, L. <ets>salamandra</ets>, Gr. <?/; cf. Per. <ets>samander</ets>, <ets>samandel</ets>.]</ety> <sn>1.</sn> <fld>(Zool.)</fld> <def>Any one of numerous species of Urodela, belonging to <gen>Salamandra</gen>, <gen>Amblystoma</gen>, <gen>Plethodon</gen>, and various allied genera, especially those that are more or less terrestrial in their habits.</def><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ The salamanders have, like lizards, an elongated body, four feet, and a long tail, but are destitute of scales. They are true Amphibia, related to the frogs. Formerly, it was a superstition that the salamander could live in fire without harm, and even extinguish it by the natural coldness of its body.</note><br/
[<source>1913 Webster</source>]</p>

<p><q>I have maintained that <qex>salamander</qex> of yours with fire any time this two and thirty years.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Whereas it is commonly said that a <qex>salamander</qex> extinguisheth fire, we have found by experience that on hot coals, it dieth immediately.</q> <rj><qau>Sir T. Browne.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Zool.)</fld> <def>The pouched gopher (<spn>Geomys tuza</spn>) of the Southern United States.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>A culinary utensil of metal with a plate or disk which is heated, and held over pastry, etc., to brown it.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>A large poker.</def> <mark>[Prov. Eng.]</mark> <rj><au>Halliwell.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <fld>(Metal.)</fld> <def>Solidified material in a furnace hearth.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Giant salamander</b></col>. <fld>(Zool.)</fld> <cd>See under <er>Giant</er>.</cd> -- <mcol><col><b>Salamander's hair</b></col> <it>or</it> <col><b>Salamander's wool</b></col></mcol> <fld>(Min.)</fld>, <cd>a species of asbestos or mineral flax.</cd> <mark>[Obs.]</mark> <au>Bacon.</au></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Salamandrina</ent><br/
||<hw>Sal`a*man*dri"na</hw> <pr>(?)</pr>, <pos>n.</pos>; <plu>pl.</plu> <ety>[NL.]</ety> <fld>(Zool.)</fld> <def>A suborder of Urodela, comprising salamanders.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Salamandrine</ent><br/
<hw>Sal`a*man"drine</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Of, pertaining to, or resembling, a salamander; enduring fire.</def> <rj><au>Addison.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Salamandroid</ent><br/
<hw>Sal`a*man"droid</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[<ets>Salamander</ets> + <ets>-oid</ets>.]</ety> <fld>(Zool.)</fld> <def>Like or pertaining to the salamanders.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Salamandroidea</ent><br/
||<hw>Sal`a*man*droi"de*a</hw> <pr>(?)</pr>, <pos>n. pl.</pos> <ety>[NL.]</ety> <fld>(Zool.)</fld> <def>A division of Amphibia including the Salamanders and allied groups; the Urodela.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Salamstone</ent><br/
<hw>Sal"am*stone`</hw> <pr>(? <or/ ?)</pr>, <pos>n.</pos> <fld>(Min.)</fld> <def>A kind of blue sapphire brought from Ceylon.</def> <rj><au>Dana.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Salangana</ent><br/
<hw>Sa*lan"ga*na</hw> <pr>(?)</pr>, <pos>n.</pos> <def>The salagane.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Salaried</ent><br/
<hw>Sal"a*ried</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Receiving a salary; paid by a salary; having a salary attached; <as>as, a <ex>salaried</ex> officer; a <ex>salaried</ex> office</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Salary</ent><br/
<hw>Sal"a*ry</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>salarius</ets>.]</ety> <def>Saline</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Salary</ent><br/
<hw>Sal"a*ry</hw> <pr>(?)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Salaries</plw> <pr>(#)</pr>.</plu> <ety>[F. <ets>salaire</ets>, L. <ets>salarium</ets>, originally, salt money, the money given to the Roman soldiers for salt, which was a part of their pay, fr. <ets>salarius</ets> belonging to salt, fr. <ets>sal</ets> salt. See <er>Salt</er>.]</ety> <def>The recompense or consideration paid, or stipulated to be paid, to a person at regular intervals for services; fixed wages, as by the year, quarter, or month; stipend; hire.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>This is hire and <qex>salary</qex>, not revenge.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ Recompense for services paid at, or reckoned by, short intervals, as a day or week, is usually called <xex>wages</xex>.</note><br/
[<source>1913 Webster</source>]</p>

<p><syn><b>Syn.</b> -- Stipend; pay; wages; hire; allowance.</syn><br/
[<source>1913 Webster</source>]</p>

<p><ent>Salary</ent><br/
<hw>Sal"a*ry</hw> <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Salaried</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Salarying</conjf> <pr>(?)</pr>.]</vmorph> <def>To pay, or agree to pay, a salary to; to attach salary to; <as>as, to <ex>salary</ex> a clerk; to <ex>salary</ex> a position</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sale</ent><br/
<hw>Sale</hw> <pr>(?)</pr>, <pos>n.</pos> <def>See 1st <er>Sallow</er>.</def> <mark>[Obs.]</mark> <rj><au>Spenser.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sale</ent><br/
<hw>Sale</hw>, <pos>n.</pos> <ety>[Icel. <ets>sala</ets>, <ets>sal</ets>, akin to E. <ets>sell</ets>. See <er>Sell</er>, <pos>v. t.</pos>]</ety> <sn>1.</sn> <def>The act of selling; the transfer of property, or a contract to transfer the ownership of property, from one person to another for a valuable consideration, or for a price in money.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Opportunity of selling; demand; market.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>They shall have ready <qex>sale</qex> for them.</q> <rj><qau>Spenser.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>Public disposal to the highest bidder, or exposure of goods in market; auction.</def> <rj><au>Sir W. Temple.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Bill of sale</b></col>. <cd>See under <er>Bill</er>.</cd> -- <mcol><col><b>Of sale</b></col>, <col><b>On sale</b></col>, <col><b>For sale</b></col></mcol>, <cd>to be bought or sold; offered to purchasers; in the market.</cd> -- <col><b>To set to sale</b></col>, <cd>to offer for sale; to put up for purchase; to make merchandise of.</cd> <mark>[Obs.]</mark> <rj><au>Milton.</au></rj></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Saleably</ent><br/
<ent>Saleable</ent><br/
<mhw><hw>Sale"a*ble</hw> <pr>(?)</pr>, <pos>a.</pos>, <hw>Sale"a*bly</hw>, <pos>adv.</pos>, etc.</mhw> <def>See <er>Salable</er>, <er>Salably</er>, etc.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Saleb</ent><br/
<hw>Sal"eb</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Med.)</fld> <def>See <er>Salep</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Salebrosity</ent><br/
<hw>Sal`e*bros"i*ty</hw> <pr>(?)</pr>, <pos>n.</pos> <def>Roughness or ruggedness.</def> <mark>[Obs.]</mark> <rj><au>Feltham.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Salebrous</ent><br/
<hw>Sal"e*brous</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>salebrosus</ets>, fr. <ets>salebra</ets> a rugged road, fr. <ets>salire</ets> to leap.]</ety> <def>Rough; rugged.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Salep</ent><br/
<hw>Sal"ep</hw> <pr>(s<acr/l"<ecr/p)</pr>, <pos>n.</pos> <ety>[Ar. <ets>sahleb</ets>, perhaps a corruption of an Arabic word for fox, one Ar. name of the orchis signifying literally, fox's testicles: cf. F. <ets>salep</ets>.]</ety> <altsp>[Written also <asp>saleb</asp>, <asp>salop</asp>, and <asp>saloop</asp>.]</altsp> <def>The dried tubers of various species of <gen>Orchis</gen>, and <gen>Eulophia</gen>. It is used to make a nutritious beverage by treating the powdered preparation with hot water.</def> <rj><au>U. S. Disp.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Saleratus</ent><br/
<hw>Sal`e*ra"tus</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[NL. <ets>sal a<eum/ratus</ets>; -- so called because it is a source of fixed air (carbon dioxide). See <er>Sal</er>, and and <er>Aerated</er>.]</ety> <fld>(Old Chem.)</fld> <def>Aerated salt; a white crystalline substance having an alkaline taste and reaction, consisting of sodium bicarbonate (see under <er>Sodium</er>.) It is largely used in cooking, with sour milk (lactic acid) or cream of tartar as a substitute for yeast. It is also an ingredient of most baking powders, and is used in the preparation of effervescing drinks.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Salesman</ent><br/
<hw>Sales"man</hw> <pr>(s<amac/lz"m<ait/n)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Salesmen</plw> <pr>(-m<eit/n)</pr>.</plu> <ety>[<ets>Sale</ets> + <ets>man</ets>.]</ety> <def>One who sells anything; one whose occupation is to sell goods or merchandise.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sales tax</ent><br/
<hw>Sales" tax</hw> <pr>(s<amac/lz"t<acr/ks)</pr>, <pos>n.</pos> <ety>[<ets>Sales</ets> + <ets>tax</ets>.]</ety> <def>a tax imposed upon the retail sale of goods or the sale of services, usually collected by the seller at the time of purchase; -- it is typically calculated as a percentage of the price of the object sold, being commonly from 3% to 7% of the <er>base price</er>.</def><br/
[<source>PJC</source>]</p>

<p><ent>Saleswoman</ent><br/
<hw>Sales"wom`an</hw> <pr>(?)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Saleswomen</plw> <pr>(<?/)</pr>.</plu> <def>A woman whose occupation is to sell goods or merchandise.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Salework</ent><br/
<hw>Sale"work`</hw> <pr>(?)</pr>, <pos>n.</pos> <def>Work or things made for sale; hence, work done carelessly or slightingly.</def> <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Salian</ent><br/
<hw>Sa"lian</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Denoting a tribe of Franks who established themselves early in the fourth century on the river Sala [now Yssel]; Salic.</def> -- <def2><pos>n.</pos> <def>A Salian Frank.</def></def2><br/
[<source>1913 Webster</source>]</p>

<p><ent>Saliant</ent><br/
<hw>Sa"li*ant</hw> <pr>(?)</pr>, <pos>a.</pos> <fld>(Her.)</fld> <def>Same as <er>Salient</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Saliaunce</ent><br/
<hw>Sal"i*aunce</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[See <er>Sally</er>.]</ety> <def>Salience; onslaught.</def> <mark>[Obs.]</mark> <ldquo/So fierce <xex>saliaunce</xex>.<rdquo/ <rj><au>Spenser.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><-- p. 1270 pr=PCP --></p>

<p><ent>Salic</ent><br/
<hw>Sal"ic</hw> <pr>(s<acr/l"<icr/k)</pr>, <pos>a.</pos> <ety>[F. <ets>salique</ets>, fr. the <ets>Salian</ets> Franks, who, in the fifth century, formed a body of laws called in Latin <ets>leges Salicae</ets>.]</ety> <def>Of or pertaining to the Salian Franks, or to the Salic law so called.</def> <altsp>[Also <asp>salique</asp>.]</altsp><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Salic law</b></col>. <sd>(a)</sd> <cd>A code of laws formed by the Salian Franks in the fifth century. By one provision of this code women were excluded from the inheritance of landed property.</cd> <sd>(b)</sd> <cd>Specifically, in modern times, a law supposed to be a special application of the above-mentioned provision, in accordance with which males alone can inherit the throne. This law has obtained in France, and at times in other countries of Europe, as Spain.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Salicaceous</ent><br/
<hw>Sal`i*ca"ceous</hw> <pr>(s<acr/l`<icr/*k<amac/"sh<ucr/s)</pr>, <pos>a.</pos> <ety>[L. <ets>salix</ets>, <ets>-icis</ets>, the willow.]</ety> <def>Belonging or relating to the willow.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Salicin</ent><br/
<hw>Sal"i*cin</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>salix</ets>, <ets>-icis</ets>, a willow: cf. F. <ets>salicine</ets>. See <er>Sallow</er> the tree.]</ety> <fld>(Chem.)</fld> <def>A glucoside found in the bark and leaves of several species of willow (<gen>Salix</gen>) and poplar, and extracted as a bitter white crystalline substance.</def> <-- <altname>salicyl alcohol glucoside</altname>, <altname>salicyl alcohol <beta/-D-glucopyranoside</altname>, <altname>saligenin <beta/-D-glucopyranoside</altname>, <chform>C13H18O7</chform>. It is used in biochemistry as a standard substrate for evaluating the potency of <beta/-glucosidase in enzymatic preparations. It is also an analgesic. --><br/
[<source>1913 Webster</source>]</p>

<p><ent>Salicyl</ent><br/
<hw>Sal"i*cyl</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[<ets>Salic</ets>in + <ets>-yl</ets>.]</ety> <fld>(Chem.)</fld> <def>The hypothetical radical of salicylic acid and of certain related compounds.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Salicylal</ent><br/
<hw>Sal"i*cyl`al</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[<ets>Salicyl</ets>ic + <ets>al</ets>dehide.]</ety> <fld>(Chem.)</fld> <def>A thin, fragrant, colorless oil, <chform>HO.C6H4.CHO</chform>, found in the flowers of meadow sweet (<gen>Spiraea</gen>), and also obtained by oxidation of salicin, saligenin, etc. It reddens on exposure. Called also <altname>salicylol</altname>, <altname>salicylic aldehyde</altname>, and formerly <altname>salicylous acid</altname> <it>or</it> <altname>spiroylous acid</altname>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Salicylate</ent><br/
<hw>Sal"i*cyl`ate</hw> <pr>(-<asl/t)</pr>, <pos>n.</pos> <fld>(Chem.)</fld> <def>A salt of salicylic acid.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Salicylic</ent><br/
<hw>Sal`i*cyl"ic</hw> <pr>(?)</pr>, <pos>a.</pos> <fld>(Chem.)</fld> <def>Pertaining to, derived from, or designating, an acid formerly obtained by fusing salicin with potassium hydroxide, and now made in large quantities from phenol (carbolic acid) by the action of carbon dioxide on heated sodium phenolate. It is a white crystalline substance. It is used as an antiseptic, and in its salts in the treatment of rheumatism. Called also <altname>hydroxybenzoic</altname> <it>acid</it>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Salicylide</ent><br/
<hw>Sal"i*cyl`ide</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[<ets>Salicyl</ets>ic + anhydr<ets>ide</ets>.]</ety> <fld>(Chem.)</fld> <def>A white crystalline substance obtained by dehydration of salicylic acid.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Salicylite</ent><br/
<hw>Sal"i*cyl`ite</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Chem.)</fld> <def>A compound of salicylal; -- named after the analogy of a salt.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Salicylol</ent><br/
<hw>Sal"i*cyl`ol</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[<ets>Salicyl</ets>ic + L. <ets>ol</ets>eum oil.]</ety> <fld>(Chem.)</fld> <def>Same as <er>Salicylal</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Salicylous</ent><br/
<hw>Sa*lic"y*lous</hw> <pr>(? <or/ ?)</pr>, <pos>a.</pos> <fld>(Chem.)</fld> <def>Pertaining to, or designating, a substance formerly called <xex>salicylous acid</xex>, and now <xex>salicylal</xex>.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Salience</ent><br/
<hw>Sa"li*ence</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[See <er>Salient</er>.]</ety> <sn>1.</sn> <def>The quality or condition of being salient; a leaping; a springing forward; an assaulting.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>The quality or state of projecting, or being projected; projection; protrusion.</def> <rj><au>Sir W. Hamilton.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Saliency</ent><br/
<hw>Sa"li*en*cy</hw> <pr>(?)</pr>, <pos>n.</pos> <def>Quality of being salient; hence, vigor.</def> <ldquo/A fatal lack of poetic <xex>saliency</xex>.<rdquo/ <rj><au>J. Morley.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Salient</ent><br/
<hw>Sa"li*ent</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>saliens</ets>, <ets>-entis</ets>, p. pr. of <ets>salire</ets> to leap; cf. F. <ets>saillant</ets>. See <er>Sally</er>, <pos>n.</pos> & <pos>v. i.</pos>.]</ety> <sn>1.</sn> <def>Moving by leaps or springs; leaping; bounding; jumping.</def> <ldquo/Frogs and <xex>salient</xex> animals.<rdquo/ <rj><au>Sir T. Browne.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Shooting out or up; springing; projecting.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>He had in himself a <qex>salient</qex>, living spring of generous and manly action.</q> <rj><qau>Burke.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>Hence, figuratively, forcing itself on the attention; prominent; conspicuous; noticeable.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>He [Grenville] had neither <qex>salient</qex> traits, nor general comprehensiveness of mind.</q> <rj><qau>Bancroft.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <fld>(Math. & Fort.)</fld> <def>Projecting outwardly; <as>as, a <ex>salient</ex> angle</as>; -- opposed to <contr>reentering</contr>. See <xex>Illust.</xex> of <er>Bastion</er>.</def><-- convex? --><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <fld>(Her.)</fld> <def>Represented in a leaping position; <as>as, a lion <ex>salient</ex></as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Salient angle</b></col>. <cd>See <er>Salient</er>, <pos>a.</pos>, 4.</cd> -- <col><b>Salient polygon</b></col> <fld>(Geom.)</fld>, <cd>a polygon all of whose angles are salient.</cd> -- <col><b>Salient polyhedron</b></col> <fld>(Geom.)</fld>, <cd>a polyhedron all of whose solid angles are salient.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Salient</ent><br/
<hw>Sa"li*ent</hw>, <pos>a.</pos> <fld>(Fort.)</fld> <def>A salient angle or part; a projection.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Saliently</ent><br/
<hw>Sa"li*ent*ly</hw>, <pos>adv.</pos> <def>In a salient manner.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Saliferous</ent><br/
<hw>Sa*lif"er*ous</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>sal</ets> salt + <ets>-ferous</ets>.]</ety> <def>Producing, or impregnated with, salt.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Saliferous rocks</b></col> <fld>(Geol.)</fld>, <cd>the New Red Sandstone system of some geologists; -- so called because, in Europe, this formation contains beds of salt. The saliferous beds of New York State belong largely to the Salina period of the Upper Silurian. See the <xex>Chart</xex> of <er>Geology</er>.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Salifiable</ent><br/
<hw>Sal"i*fi`a*ble</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[Cf. F. <ets>salifiable</ets>. See <er>Salify</er>.]</ety> <fld>(Chem.)</fld> <def>Capable of neutralizing an acid to form a salt; -- said of bases; thus, ammonia is <xex>salifiable</xex>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Salification</ent><br/
<hw>Sal`i*fi*ca"tion</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Cf. F. <ets>salification</ets>.]</ety> <fld>(Chem.)</fld> <def>The act, process, or result of salifying; the state of being salified.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Salify</ent><br/
<hw>Sal"i*fy</hw> <pr>(?)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Salified</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Salifying</conjf> <pr>(?)</pr>.]</vmorph> <ety>[F. <ets>salifier</ets>; from L. <ets>sal</ets> salt + <ets>-ficare</ets> (only in comp.) to make. See <er>-fy</er>.]</ety> <fld>(Chem.)</fld> <sd>(a)</sd> <def>To combine or impregnate with a salt.</def> <sd>(b)</sd> <def>To form a salt with; to convert into a salt; <as>as, to <ex>salify</ex> a base or an acid</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Saligenin</ent><br/
<hw>Sa*lig"e*nin</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[<ets>Sal</ets>icin + <ets>-gen</ets>.]</ety> <fld>(Chem.)</fld> <def>A phenol alcohol obtained, by the decomposition of salicin, as a white crystalline substance; -- called also <altname>hydroxy-benzyl alcohol</altname>.</def> <-- ortho-hydroxybenzyl alcohol, saligenol, salicyl alcohol. <chform>HOCH2.C6H4.OH</chform> --><br/
[<source>1913 Webster</source>]</p>

<p><ent>Saligot</ent><br/
<hw>Sal"i*got</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F.]</ety> <fld>(Bot.)</fld> <def>The water chestnut (<spn>Trapa natans</spn>).</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Salimeter</ent><br/
<hw>Sal*im"e*ter</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>sal</ets> salt + <ets>-meter</ets>.]</ety> <def>An instrument for measuring the amount of salt present in any given solution.</def> <altsp>[Written also <asp>salometer</asp>.]</altsp><br/
[<source>1913 Webster</source>]</p>

<p><ent>Salimetry</ent><br/
<hw>Sal*im"e*try</hw> <pr>(?)</pr>, <pos>n.</pos> <def>The art or process of measuring the amount of salt in a substance.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Salina</ent><br/
<hw>Sa*li"na</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Cf. L. <ets>salinae</ets>, pl., salt works, from <ets>sal</ets> salt. See <er>Saline</er>, <pos>a.</pos>]</ety> <sn>1.</sn> <def>A salt marsh, or salt pond, inclosed from the sea.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Salt works.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Salina period</ent><br/
<hw>Sa*li"na pe"ri*od</hw> <pr>(?)</pr>. <ety>[So called from <ets>Salina</ets>, a town in New York.]</ety> <fld>(Geol.)</fld> <def>The period in which the American Upper Silurian system, containing the brine-producing rocks of central New York, was formed. See the <xex>Chart</xex> of <er>Geology</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Salination</ent><br/
<hw>Sal`i*na"tion</hw> <pr>(?)</pr>, <pos>n.</pos> <def>The act of washing with salt water.</def> <mark>[R. & Obs.]</mark> <rj><au>Greenhill.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Saline</ent><br/
<hw>Sa"line</hw> <pr>(? <or/ ?; 277)</pr>, <pos>a.</pos> <ety>[F. <ets>salin</ets>, fr. L. <ets>sal</ets> salt: cf. L. <ets>salinae</ets> salt works, <ets>salinum</ets> saltcellar. See <er>Salt</er>.]</ety> <sn>1.</sn> <def>Consisting of salt, or containing salt; <as>as, <ex>saline</ex> particles; <ex>saline</ex> substances; a <ex>saline</ex> cathartic</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Of the quality of salt; salty; <as>as, a <ex>saline</ex> taste</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Saline</ent><br/
<hw>Sa"line</hw> <pr>(? <or/ ?; 277)</pr>, <pos>n.</pos> <ety>[Cf. F. <ets>saline</ets>. See <er>Saline</er>, <pos>a.</pos>]</ety> <def>A salt spring; a place where salt water is collected in the earth.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Saline</ent><br/
<hw>Sal"ine</hw> <pr>(?)</pr>, <pos>n.</pos> <sn>1.</sn> <fld>(Chem.)</fld> <def>A crude potash obtained from beet-root residues and other similar sources.</def> <altsp>[Written also <asp>salin</asp>.]</altsp><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Med. Chem.)</fld> <def>A metallic salt; esp., a salt of potassium, sodium, lithium, or magnesium, used in medicine.</def><br/
[<source>1913 Webster</source>]</p>

<p><-- 3. (Med., Biochemistry) A saline solution, esp. normal saline, or isotonic saline, used for infusion, to maintain blood pressure. --><br/
[<source>1913 Webster</source>]</p>

<p><ent>Salineness</ent><br/
<hw>Sa*line"ness</hw> <pr>(?)</pr>, <pos>n.</pos> <def>The quality or state of being salt; saltness.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Saliniferous</ent><br/
<hw>Sal`i*nif"er*ous</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[<ets>Saline</ets> + <ets>-ferous</ets>.]</ety> <def>Same as <er>Saliferous</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Saliniform</ent><br/
<hw>Sa*lin"i*form</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Having the form or the qualities of a salt, especially of common salt.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Salinity</ent><br/
<hw>Sa*lin"i*ty</hw> <pr>(?)</pr>, <pos>n.</pos> <def>Salineness.</def> <rj><au>Carpenter.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Salinometer</ent><br/
<hw>Sal`i*nom"e*ter</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[<ets>Saline</ets> + <ets>-meter</ets>.]</ety> <def>A salimeter.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Salinous</ent><br/
<hw>Sa*lin"ous</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Saline.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Salique</ent><br/
<hw>Sal"ique</hw> <pr>(? <or/ ?)</pr>, <pos>a.</pos> <ety>[F.]</ety> <def>Salic.</def> <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>She fulmined out her scorn of laws <qex>salique</qex>.</q> <rj><qau>Tennyson.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Saliretin</ent><br/
<hw>Sal`i*re"tin</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[<ets>Sali</ets>genin + Gr. <?/ resin.]</ety> <fld>(Chem.)</fld> <def>A yellow amorphous resinoid substance obtained by the action of dilute acids on saligenin.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Salisburia</ent><br/
||<hw>Sal`is*bu"ri*a</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Named after R. A. <ets>Salisbury</ets>, an English botanist.]</ety> <fld>(Bot.)</fld> <def>The ginkgo tree (<spn>Ginkgo biloba</spn>, or <spn>Salisburia adiantifolia</spn>).</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Salite</ent><br/
<hw>Sal"ite</hw> <pr>(?)</pr>, <pos>v. t.</pos> <ety>[L. <ets>salitus</ets>, p. p. of <ets>salire</ets> to salt, fr. <ets>sal</ets> salt.]</ety> <def>To season with salt; to salt.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Salite</ent><br/
<hw>Sa"lite</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[So called from <ets>Sala</ets>, a town in Sweden.]</ety> <fld>(Min.)</fld> <def>A massive lamellar variety of pyroxene, of a dingy green color.</def> <altsp>[Written also <asp>sahlite</asp>.]</altsp><br/
[<source>1913 Webster</source>]</p>

<p><ent>Saliva</ent><br/
<hw>Sa*li"va</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L.; cf. Gr. <?/.]</ety> <fld>(Physiol.)</fld> <def>The secretion from the salivary glands.</def><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ In man the saliva is a more or less turbid and slighty viscid fluid, generally of an alkaline reaction, and is secreted by the parotid, submaxillary, and sublingual glands. In the mouth the saliva is mixed with the secretion from the buccal glands. The secretions from the individual salivary glands have their own special characteristics, and these are not the same in all animals. In man and many animals <xex>mixed saliva</xex>, i.e., saliva composed of the secretions of all three of the salivary glands, is an important digestive fluid on account of the presence of the peculiar enzyme, ptyalin.</note><br/
[<source>1913 Webster</source>]</p>

<p><ent>Salival</ent><br/
<hw>Sa*li"val</hw> <pr>(?; 277)</pr>, <pos>a.</pos> <def>Salivary.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Salivant</ent><br/
<hw>Sal"i*vant</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>salivans</ets>, p. pr. of <ets>salivare</ets>. See <er>Salivate</er>.]</ety> <def>Producing salivation.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Salivant</ent><br/
<hw>Sal"i*vant</hw>, <pos>n.</pos> <def>That which produces salivation.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Salivary</ent><br/
<hw>Sal"i*va*ry</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>salivarius</ets> slimy, clammy: cf. F. <ets>salivaire</ets>.]</ety> <fld>(Physiol.)</fld> <def>Of or pertaining to saliva; producing or carrying saliva; <as>as, the <ex>salivary</ex> ferment; the <ex>salivary</ex> glands; the <ex>salivary</ex> ducts, etc.</as></def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Salivate</ent><br/
<hw>Sal"i*vate</hw> <pr>(?)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Salivated</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Salivating</conjf>.]</vmorph> <ety>[L. <ets>salivatus</ets>, p. p. of <ets>salivare</ets> to salivate. See <er>Saliva</er>.]</ety> <def>To produce an abnormal flow of saliva in; to produce salivation or ptyalism in, as by the use of mercury.</def><br/
[<source>1913 Webster</source>]</p>

<p><-- <pos>v. i.</pos> <def>To produce saliva, esp. in excess.</def>   2. To drool.   3. (Fig.) To anticipate keenly with pleasure, as though salivating at the expectation of a delicious meal. Used often with <it>over.</it>; as, <ex>salivate over</ex> the prospects of high profits from an enterprise.<br/
[<source>1913 Webster</source>]</p>

<p><note>Probably influenced by the experiments of Pavlov, who trained dogs to salivate at the sound of a bell, by previously ringing the bell immediately prior to feeding them.</note> --><br/
[<source>1913 Webster</source>]</p>

<p><ent>Salivation</ent><br/
<hw>Sal`i*va"tion</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>salivatio</ets>: cf. F. <ets>salivation</ets>.]</ety> <fld>(Physiol.)</fld> <def>The act or process of salivating; an excessive secretion of saliva, often accompanied with soreness of the mouth and gums; ptyalism.</def><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ It may be induced by direct chemical or mechanical stimulation, as in mastication of some tasteless substance like rubber, or indirectly by some agent which affects the whole system, as mercury compounds.</note><br/
[<source>1913 Webster</source>]</p>

<p><ent>Salivous</ent><br/
<hw>Sa*li"vous</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>salivosus</ets>: cf. F. <ets>saliveux</ets>.]</ety> <def>Pertaining to saliva; of the nature of saliva.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Salix</ent><br/
||<hw>Sa"lix</hw> <pr>(?)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Salices</plw> <pr>(#)</pr>.</plu> <ety>[L., the willow.]</ety> <fld>(Bot.)</fld> <sd>(a)</sd> <def>A genus of trees or shrubs including the willow, osier, and the like, growing usually in wet grounds.</def> <sd>(b)</sd> <def>A tree or shrub of any kind of willow.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sallenders</ent><br/
<hw>Sal"len*ders</hw> <pr>(?)</pr>, <pos>n. pl.</pos> <ety>[F. <ets>solandres</ets>, <ets>solandre</ets>.]</ety> <fld>(Far.)</fld> <def>An eruption on the hind leg of a horse.</def> <altsp>[Written also <asp>sellanders</asp>, and <asp>sellenders</asp>.]</altsp><br/
[<source>1913 Webster</source>]</p>

<p><q>On the inside of the hock, or a little below it, as well as at the bend of the knee, there is occasionally a scurfy eruption called <ldquo/mallenders<rdquo/ in the fore leg, and <ldquo/<qex>sallenders</qex><rdquo/ in the hind leg.</q> <rj><qau>Youatt.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sallet</ent><br/
<hw>Sal"let</hw> <pr>(s<acr/l"l<ecr/t)</pr>, <pos>n.</pos> <ety>[F. <ets>salade</ets>, Sp. <ets>celada</ets>, or It. <ets>celata</ets>, fr. L. (cassis) <ets>caelata</ets>, fr. <ets>caelare</ets>, <ets>caelatum</ets>, to engrave in relief. So called from the figures engraved upon it.]</ety> <def>A light kind of helmet, with or without a visor, introduced during the 15th century.</def> <altsp>[Written also <asp>salade</asp>.]</altsp><br/
[<source>1913 Webster</source>]</p>

<p><q>Then he must have a <qex>sallet</qex> wherewith his head may be saved.</q> <rj><qau>Latimer.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Salleting</ent><br/
<ent>Sallet</ent><br/
<mhw>{ <hw>Sal"let</hw>, <hw>Sal"let*ing</hw>, }</mhw> <pos>n.</pos> <def>Salad.</def> <mark>[Obs.]</mark> <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Salliance</ent><br/
<hw>Sal"li*ance</hw> <pr>(?)</pr>, <pos>n.</pos> <def>Salience.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sallow</ent><br/
<hw>Sal"low</hw> <pr>(s<acr/l"l<osl/)</pr>, <pos>n.</pos> <ety>[OE. <ets>salwe</ets>, AS. <ets>sealh</ets>; akin to OHG. <ets>salaha</ets>, G. <ets>salweide</ets>, Icel. <ets>selja</ets>, L. <ets>salix</ets>, Ir. <ets>sail</ets>, <ets>saileach</ets>, Gael. <ets>seileach</ets>, W. <ets>helyg</ets>, Gr. <grk>"eli`kh</grk>.]</ety> <sn>1.</sn> <def>The willow; willow twigs.</def> <mark>[Poetic]</mark> <rj><au>Tennyson.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>And bend the pliant <qex>sallow</qex> to a shield.</q> <rj><qau>Fawkes.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>The <qex>sallow</qex> knows the basketmaker's thumb.</q> <rj><qau>Emerson.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Bot.)</fld> <def>A name given to certain species of willow, especially those which do not have flexible shoots, as <spn>Salix caprea</spn>, <spn>Salix cinerea</spn>, etc.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Sallow thorn</b></col> <fld>(Bot.)</fld>, <cd>a European thorny shrub (<spn>Hippophae rhamnoides</spn>) much like an Elaeagnus. The yellow berries are sometimes used for making jelly, and the plant affords a yellow dye.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sallow</ent><br/
<hw>Sal"low</hw>, <pos>a.</pos> <amorph>[<pos>Compar.</pos> <adjf>Sallower</adjf> <pr>(?)</pr>; <pos>superl.</pos> <adjf>Sallowest</adjf>.]</amorph> <ety>[AS. <ets>salu</ets>; akin to D. <ets>zaluw</ets>, OHG. <ets>salo</ets>, Icel. <ets>s<oum/lr</ets> yellow.]</ety> <def>Having a yellowish color; of a pale, sickly color, tinged with yellow; <as>as, a <ex>sallow</ex> skin</as>.</def> <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sallow</ent><br/
<hw>Sal"low</hw>, <pos>v. t.</pos> <def>To tinge with sallowness.</def> <mark>[Poetic]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>July breathes hot, <qex>sallows</qex> the crispy fields.</q> <rj><qau>Lowell.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sallowish</ent><br/
<hw>Sal"low*ish</hw>, <pos>a.</pos> <def>Somewhat sallow.</def> <rj><au>Dickens.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sallowness</ent><br/
<hw>Sal"low*ness</hw> <pr>(?)</pr>, <pos>n.</pos> <def>The quality or condition of being sallow.</def> <rj><au>Addison.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sally</ent><br/
<hw>Sal"ly</hw> <pr>(s<acr/l"l<ycr/)</pr>, <pos>v. i.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Sallied</conjf> <pr>(-l<icr/d)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Sallying</conjf>.]</vmorph> <ety>[F. <ets>saillir</ets>, fr. L. <ets>salire</ets> to leap, spring, akin to Gr. <grk>"a`llesqai</grk>; cf. Skr. <ets>s<rsdot/</ets> to go, to flow. Cf. <er>Salient</er>, <er>Assail</er>, <er>Assault</er>, <er>Exult</er>, <er>Insult</er>, <er>Saltation</er>, <er>Saltire</er>.]</ety> <def>To leap or rush out; to burst forth; to issue suddenly; as a body of troops from a fortified place to attack besiegers; to make a sally.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>They break the truce, and <qex>sally</qex> out by night.</q> <rj><qau>Dryden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>The foe retires, -- she heads the <qex>sallying</qex> host.</q> <rj><qau>Byron.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sally</ent><br/
<hw>Sal"ly</hw>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Sallies</plw> <pr>(#)</pr>.</plu> <ety>[F. <ets>saillie</ets>, fr. <ets>saillir</ets>. See <ets>Sally</ets>, <pos>v.</pos>]</ety> <sn>1.</sn> <def>A leaping forth; a darting; a spring.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A rushing or bursting forth; a quick issue; a sudden eruption; specifically, an issuing of troops from a place besieged to attack the besiegers; a sortie.</def><br/
[<source>1913 Webster</source>]</p>

<p><q><qex>Sallies</qex> were made by the Spaniards, but they were beaten in with loss.</q> <rj><qau>Bacon.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>An excursion from the usual track; range; digression; deviation.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Every one shall know a country better that makes often <qex>sallies</qex> into it, and traverses it up and down, than he that . . . goes still round in the same track.</q> <rj><qau>Locke.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>A flight of fancy, liveliness, wit, or the like; a flashing forth of a quick and active mind.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The unaffected mirth with which she enjoyed his <qex>sallies</qex>.</q> <rj><qau>Sir W. Scott.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <def>Transgression of the limits of soberness or steadiness; act of levity; wild gayety; frolic; escapade.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The excursion was esteemed but a <qex>sally</qex> of youth.</q> <rj><qau>Sir H. Wotton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Sally port</b></col>. <sd>(a)</sd> <fld>(Fort.)</fld> <cd>A postern gate, or a passage underground, from the inner to the outer works, to afford free egress for troops in a sortie</cd>. <sd>(b)</sd> <fld>(Naval)</fld> <cd>A large port on each quarter of a fireship, for the escape of the men into boats when the train is fired; a large port in an old-fashioned three-decker or a large modern ironclad.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sally Lunn</ent><br/
<hw>Sal"ly Lunn"</hw> <pr>(?)</pr>. <ety>[From a woman, <ets>Sally Lunn</ets>, who is said to have first made the cakes, and sold them in the streets of Bath, Eng.]</ety> <def>A tea cake slighty sweetened, and raised with yeast, baked in the form of biscuits or in a thin loaf, and eaten hot with butter.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sallyman</ent><br/
<hw>Sal"ly*man</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Zool.)</fld> <def>The velella; -- called also <altname>saleeman</altname>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Salm</ent><br/
<hw>Salm</hw> <pr>(?)</pr>, <pos>n.</pos> <def>Psalm.</def> <mark>[Obs2E]</mark> <rj><au>Piers Plowman.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Salmagundi</ent><br/
<hw>Sal`ma*gun"di</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F. <ets>salmigondis</ets>, of uncertain origin; perhaps from L. <ets>salgama condita</ets>, pl.; <ets>salgama</ets> pickles + <ets>condita</ets> preserved (see <er>Condite</er>); or from the Countess <ets>Salmagondi</ets>, lady of honor to Maria de Medici, who is said to have invented it; or cf. It. <ets>salame</ets> salt meat, and F. <ets>salmis</ets> a ragout.]</ety> <sn>1.</sn> <def>A mixture of chopped meat and pickled herring, with oil, vinegar, pepper, and onions.</def> <rj><au>Johnson.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Hence, a mixture of various ingredients; an olio or medley; a potpourri; a miscellany.</def> <rj><au>W. Irving.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Salmi</ent><br/
<hw>Sal"mi</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Cookery)</fld> <def>Same as <er>Salmis</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Salmiac</ent><br/
<hw>Sal"mi*ac</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Cf. F. <ets>salmiac</ets>, G. <ets>salmiak</ets>.]</ety> <fld>(Old Chem.)</fld> <def>Sal ammoniac. See under <er>Sal</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Salmis</ent><br/
||<hw>Sal`mis"</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F.]</ety> <fld>(Cookery)</fld> <def>A ragout of partly roasted game stewed with sauce, wine, bread, and condiments suited to provoke appetite.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Salmon</ent><br/
<hw>Salm"on</hw> <pr>(s<acr/m"<ucr/n)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Salmons</plw> <pr>(-<ucr/nz)</pr> <it>or</it> (<it>collectively</it>) <plw>Salmon</plw>.</plu> <ety>[OE. <ets>saumoun</ets>, <ets>salmon</ets>, F. <ets>saumon</ets>, fr. L. <ets>salmo</ets>, <ets>salmonis</ets>, perhaps from <ets>salire</ets> to leap. Cf. <er>Sally</er>, <pos>v.</pos>]</ety> <sn>1.</sn> <fld>(Zool.)</fld> <def>Any one of several species of fishes of the genus <gen>Salmo</gen> and allied genera. The common salmon (<spn>Salmo salar</spn>) of Northern Europe and Eastern North America, and the California salmon, or quinnat, are the most important species. They are extensively preserved for food. See <er>Quinnat</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><-- In the following paragraph I have taken the collocation section out of the note section, left it in the same paragraph, and added the definition tags. PCP --><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ The salmons ascend rivers and penetrate to their head streams to spawn. They are remarkably strong fishes, and will even leap over considerable falls which lie in the way of their progress. The common salmon has been known to grow to the weight of seventy-five pounds; more generally it is from fifteen to twenty-five pounds. Young salmon are called <xex>parr</xex>, <xex>peal</xex>, <xex>smolt</xex>, and <xex>grilse</xex>.</note> <cs>Among the true salmons are: <mcol><col><b>Black salmon</b></col>, <it>or</it> <col><b>Lake salmon</b></col></mcol>, <cd>the namaycush.</cd> -- <col><b>Dog salmon</b></col>, <cd>a salmon of Western North America (<spn>Oncorhynchus keta</spn>).</cd> -- <col><b>Humpbacked salmon</b></col>, <cd>a Pacific-coast salmon (<spn>Oncorhynchus gorbuscha</spn>).</cd> -- <col><b>King salmon</b></col>, <cd>the quinnat.</cd> -- <col><b>Landlocked salmon</b></col>, <cd>a variety of the common salmon (var. <varn>Sebago</varn>), long confined in certain lakes in consequence of obstructions that prevented it from returning to the sea. This last is called also <altname>dwarf salmon</altname>.</cd></cs> <br/
[<source>1913 Webster</source>]</p>

<p><note>  Among fishes of other families which are locally and erroneously called <ex>salmon</ex> are: the pike perch, called <stype>jack salmon</stype>; the spotted, or southern, squeteague; the cabrilla, called <stype>kelp salmon</stype>; young pollock, called <stype>sea salmon</stype>; and the California yellowtail.</note><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A reddish yellow or orange color, like the flesh of the salmon.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Salmon berry</b></col> <fld>(Bot.)</fld>, <cd>a large red raspberry growing from Alaska to California, the fruit of the <spn>Rubus Nutkanus</spn>.</cd> -- <col><b>Salmon killer</b></col> <fld>(Zool.)</fld>, <cd>a stickleback (<spn>Gasterosteus cataphractus</spn>) of Western North America and Northern Asia.</cd> -- <mcol><col><b>Salmon ladder</b></col>, <col><b>Salmon stair</b></col></mcol>. <cd>See <cref>Fish ladder</cref>, under <er>Fish</er>.</cd> -- <col><b>Salmon peel</b></col>, <cd>a young salmon.</cd> -- <col><b>Salmon pipe</b></col>, <cd>a certain device for catching salmon.</cd> <au>Crabb.</au> -- <col><b>Salmon trout</b></col>. <fld>(Zool.)</fld> <sd>(a)</sd> <cd>The European sea trout (<spn>Salmo trutta</spn>). It resembles the salmon, but is smaller, and has smaller and more numerous scales.</cd> <sd>(b)</sd> <cd>The American namaycush</cd>. <sd>(c)</sd> <cd>A name that is also applied locally to the adult black spotted trout (<spn>Salmo purpuratus</spn>), and to the steel head and other large trout of the Pacific coast.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Salmon</ent><br/
<hw>Salm"on</hw>, <pos>a.</pos> <def>Of a reddish yellow or orange color, like that of the flesh of the salmon.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Salmonella</ent><br/
<hw>Sal`mo*nel"la</hw>, <pos>prop. n.</pos> <ety>[After <person>Daniel E. <etsep>Salmon</etsep></person>, a U. S. pathologist (1850-1914).]</ety> <def>A genus of gram-negative bacteria that may be motile or non-motile; they are typically rod-shaped and may be aerobic or facultatively aerobic.  They may be pathogenic for humans and other animals.  Their metabolism is fermentative, and they produce acid and usually gas from glucose, but they do not metabolize lactose.  The type species is <spn>Salmonella cholerae-suis</spn>, which is found in pigs.  Other species, pathogenic in man, are <spn>Salmonella typhi</spn> (<spn>Salmonella typhosa</spn>), <spn>Salmonella typhimurium</spn>, and <spn>Salmonella schotmulleri</spn>, whih cause <causes>typhoid fever</causes>, <causes>food poisoning</causes>, and <causes>enteric fever</causes>, respectively.</def>  <au>Stedman.</au><br/
[<source>PJC</source>]</p>

<p><ent>salmonellosis</ent><br/
<hw>sal`mo*nel*lo"sis</hw>, <pos>n.</pos> <def>Infection with bacteria of the genus <gen>Salmonella</gen>.</def>  <au>Stedman.</au><br/
[<source>PJC</source>]</p>

<p><-- p. 1271 pr=PCP --></p>

<p><ent>Salmonet</ent><br/
<hw>Salm"on*et</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Cf. <ets>Samlet</ets>.]</ety> <fld>(Zool.)</fld> <def>A salmon of small size; a samlet.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Salmonoid</ent><br/
<hw>Sal"mon*oid</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[<ets>Salmon</ets> + <ets>-oid</ets>.]</ety> <fld>(Zool.)</fld> <def>Like, or pertaining to, the <fam>Salmonidae</fam>, a family of fishes including the trout and salmon.</def> -- <def2><pos>n.</pos> <def>Any fish of the family <fam>Salmonidae</fam>.</def></def2><br/
[<source>1913 Webster</source>]</p>

<p><ent>Salogen</ent><br/
<hw>Sal"o*gen</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>sal</ets> salt + <ets>-gen</ets>.]</ety> <fld>(Chem.)</fld> <def>A halogen.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Salol</ent><br/
<hw>Sal"ol</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[<ets>Salicylic</ets> + <ets>-ol</ets>.]</ety> <fld>(Chem.)</fld> <def>A white crystalline substance consisting of phenol salicylate.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>salometer</ent><br/
<hw>sa*lom"e*ter</hw> <pr>(?)</pr>, <pos>n.</pos> <def>See <er>Salimeter</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Salometry</ent><br/
<hw>Sa*lom"e*try</hw> <pr>(?)</pr>, <pos>n.</pos> <def>Salimetry.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Salon</ent><br/
||<hw>Sa`lon"</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F. See <er>Saloon</er>.]</ety> <sn>1.</sn> <def>An apartment for the reception of company; hence, in the plural, fashionable parties; circles of fashionable society.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>An apartment for the reception and exhibition of works of art; hence, an annual exhibition of paintings, sculptures, etc., held in Paris by the Society of French Artists; -- sometimes called the <altname><col><b>Old Salon</b></col></altname>. <col><b>New Salon</b></col> is a popular name for an annual exhibition of paintings, sculptures, etc., held in Paris at the Champs de Mars, by the Soci<eacute/t<eacute/ Nationale des Beaux-Arts (National Society of Fine Arts), a body of artists who, in 1890, seceded from the Soci<eacute/t<eacute/ des Artistes Fran<cced/ais (Society of French Artists).</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Saloon</ent><br/
<hw>Sa*loon"</hw> <pr>(s<adot/*l<oomac/n")</pr>, <pos>n.</pos> <ety>[F. <ets>salon</ets> (cf. It. <ets>salone</ets>), fr. F. <ets>salle</ets> a large room, a hall, of German or Dutch origin; cf. OHG. <ets>sal</ets> house, hall, G. <ets>saal</ets>; akin to AS. <ets>sael</ets>, <ets>sele</ets>, D. <ets>zaal</ets>, Icel. <ets>salr</ets>, Goth. <ets>saljan</ets> to dwell, and probably to L. <ets>solum</ets> ground. Cf. <er>Sole</er> of the foot, <er>Soil</er> ground, earth.]</ety> <sn>1.</sn> <def>A spacious and elegant apartment for the reception of company or for works of art; a hall of reception, esp. a hall for public entertainments or amusements; a large room or parlor; <as>as, the <ex>saloon</ex> of a steamboat</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The gilden <qex>saloons</qex> in which the first magnates of the realm . . . gave banquets and balls.</q> <rj><qau>Macaulay.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Popularly, a public room for specific uses; esp., a barroom or grogshop; <as>as, a drinking <ex>saloon</ex>; an eating <ex>saloon</ex>; a dancing <ex>saloon</ex></as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>We hear of no hells, or low music halls, or low dancing <qex>saloons</qex> [at Athens.]</q> <rj><qau>J. P. Mahaffy.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Saloop</ent><br/
<hw>Sa*loop"</hw> <pr>(s<adot/*l<oomac/p")</pr>, <pos>n.</pos> <def>An aromatic drink prepared from sassafras bark and other ingredients, at one time much used in London.</def> <rj><au>J. Smith (Dict. Econ. Plants).</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Saloop bush</b></col> <fld>(Bot.)</fld>, <cd>an Australian shrub (<spn>Rhagodia hastata</spn>) of the Goosefoot family, used for fodder.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Salp</ent><br/
<hw>Salp</hw> <pr>(s<acr/lp)</pr>, <pos>n.</pos> <fld>(Zool.)</fld> <def>Any species of <gen>Salpa</gen>, or of the family <fam>Salpidae</fam>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Salpa</ent><br/
||<hw>Sal"pa</hw> <pr>(s<acr/l"p<adot/)</pr>, <pos>n.</pos>; <plu><it>pl.</it> L. <plw>Salpae</plw> <pr>(s<acr/l"p<emac/)</pr>, E. <plw>Salpas</plw> <pr>(s<acr/l"p<adot/z)</pr>.</plu> <ety>[NL.: cf. L. <ets>salpa</ets> a kind of stockfish.]</ety> <fld>(Zool.)</fld> <def>A genus of transparent, tubular, free-swimming oceanic tunicates found abundantly in all the warmer latitudes. See <xex>Illustration</xex> in Appendix.</def><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ Each species exists in two distinct forms, one of which lives solitary, and produces, by budding from an internal organ, a series of the other kind. These are united together, side by side, so as to form a chain, or cluster, often of large size. Each of the individuals composing the chain carries a single egg, which develops into the solitary kind.</note><br/
[<source>1913 Webster</source>]</p>

<p><ent>Salpid</ent><br/
<ent>Salpian</ent><br/
<mhw>{ <hw>Sal"pi*an</hw> <pr>(?)</pr>, <hw>Sal"pid</hw> <pr>(?)</pr>, }</mhw> <pos>n.</pos> <fld>(Zool.)</fld> <def>A salpa.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Salpicon</ent><br/
<hw>Sal"pi*con</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F. <ets>salpicon</ets>, Sp. <ets>salpicon</ets>.]</ety> <def>Chopped meat, bread, etc., used to stuff legs of veal or other joints; stuffing; farce.</def> <rj><au>Bacon.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Salpingitis</ent><br/
||<hw>Sal`pin*gi"tis</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[NL. See <er>Salpinx</er>, and <er>-itis</er>.]</ety> <fld>(Med.)</fld> <def>Inflammation of the salpinx.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Salpinx</ent><br/
||<hw>Sal"pinx</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[NL., from Gr. <?/, <?/, a trumpet.]</ety> <fld>(Old Anat.)</fld> <def>The Eustachian tube, or the Fallopian tube.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Salsafy</ent><br/
<hw>Sal"sa*fy</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Bot.)</fld> <def>See <er>Salsify</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Salsamentarious</ent><br/
<hw>Sal`sa*men*ta"ri*ous</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>salsamentarius</ets>, fr. <ets>salsamentum</ets> brine, pickled fish, fr. <ets>salsus</ets> salted, p. p. of <ets>salire</ets> to salt.]</ety> <def>Salt; salted; saline.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Salse</ent><br/
<hw>Salse</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F.]</ety> <def>A mud volcano, the water of which is often impregnated with salts, whence the name.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Salsify</ent><br/
<hw>Sal"si*fy</hw> <pr>(?; 277)</pr>, <pos>n.</pos> <ety>[F. <ets>salsifis</ets>.]</ety> <fld>(Bot.)</fld> <def>See <cref>Oyster plant</cref> <sd>(a)</sd>, under <er>Oyster</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Salso-acid</ent><br/
<hw>Sal"so-ac`id</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>salsus</ets> salted, salt + <ets>acidus</ets> acid.]</ety> <def>Having a taste compounded of saltness and acidity; both salt and acid.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Salsoda</ent><br/
<hw>Sal`so"da</hw> <pr>(?)</pr>, <pos>n.</pos> <def>See <cref>Sal soda</cref>, under <er>Sal</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Salsola</ent><br/
||<hw>Sal"so*la</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[NL., fr. L. <ets>salsus</ets> salt, because they contain alkaline salts.]</ety> <fld>(Bot.)</fld> <def>A genus of plants including the glasswort. See <er>Glasswort</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>salsuginous</ent><br/
<hw>sal*su"gi*nous</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>salsugo</ets>, <ets>-ginis</ets>, saltness, from <ets>salsus</ets> salted, salt: cf. F. <ets>salsugineux</ets>.]</ety> <fld>(Bot.)</fld> <def>Growing in brackish places or in salt marshes.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Salt</ent><br/
<hw>Salt</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[AS. <ets>sealt</ets>; akin to OS. & OFries. <ets>salt</ets>, D. <ets>zout</ets>, G. <ets>salz</ets>, Icel., Sw., & Dan. <ets>salt</ets>, L. <ets>sal</ets>, Gr. <?/, Russ. <ets>sole</ets>, Ir. & Gael. <ets>salann</ets>, W. <ets>halen</ets>, of unknown origin. Cf. <er>Sal</er>, <er>Salad</er>, <er>Salary</er>, <er>Saline</er>, <er>Sauce</er>, <er>Sausage</er>.]</ety> <sn>1.</sn> <def>The chloride of sodium, a substance used for seasoning food, for the preservation of meat, etc. It is found native in the earth, and is also produced, by evaporation and crystallization, from sea water and other water impregnated with saline particles.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Hence, flavor; taste; savor; smack; seasoning.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Though we are justices and doctors and churchmen . . . we have some <qex>salt</qex> of our youth in us.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>Hence, also, piquancy; wit; sense; <as>as, Attic <ex>salt</ex></as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>A dish for salt at table; a saltcellar.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>I out and bought some things; among others, a dozen of silver <qex>salts</qex>.</q> <rj><qau>Pepys.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <def>A sailor; -- usually qualified by <xex>old</xex>.</def> <mark>[Colloq.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>Around the door are generally to be seen, laughing and gossiping, clusters of old <qex>salts</qex>.</q> <rj><qau>Hawthorne.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>6.</sn> <fld>(Chem.)</fld> <def>The neutral compound formed by the union of an acid and a base; <as>thus, sulphuric acid and iron form the <ex>salt</ex> sulphate of iron or green vitriol</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ Except in case of ammonium salts, accurately speaking, it is the acid radical which unites with the base or basic radical, with the elimination of hydrogen, of water, or of analogous compounds as side products. In the case of diacid and triacid bases, and of dibasic and tribasic acids, the mutual neutralization may vary in degree, producing respectively <xex>basic</xex>, <xex>neutral</xex>, or <xex>acid</xex> salts. See Phrases below.</note><br/
[<source>1913 Webster</source>]</p>

<p><sn>7.</sn> <def><markp>Fig.</markp>: That which preserves from corruption or error; that which purifies; a corrective; an antiseptic; also, an allowance or deduction; <as>as, his statements must be taken with a grain of <ex>salt</ex></as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Ye are the <qex>salt</qex> of the earth.</q> <rj><qau>Matt. v. 13.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>8.</sn> <pluf>pl.</pluf> <def>Any mineral salt used as an aperient or cathartic, especially Epsom salts, Rochelle salt, or Glauber's salt.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>9.</sn> <pluf>pl.</pluf> <def>Marshes flooded by the tide.</def> <mark>[Prov. Eng.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><cs><mcol><col><b>Above the salt</b></col>, <col><b>Below the salt</b></col></mcol>, <cd>phrases which have survived the old custom, in the houses of people of rank, of placing a large saltcellar near the middle of a long table, the places above which were assigned to the guests of distinction, and those below to dependents, inferiors, and poor relations. See <er>Saltfoot</er>.</cd><br/
[<source>1913 Webster</source>]</p>

<p><q>His fashion is not to take knowledge of him that is beneath him in clothes. He never drinks <qex>below the salt</qex>.</q> <rj><qau>B. Jonson.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p>-- <col><b>Acid salt</b></col> <fld>(Chem.)</fld> <sd>(a)</sd> <cd>A salt derived from an acid which has several replaceable hydrogen atoms which are only partially exchanged for metallic atoms or basic radicals; <as>as, acid potassium sulphate is an <ex>acid salt</ex></as>.</cd> <sd>(b)</sd> <cd>A salt, whatever its constitution, which merely gives an acid reaction; <as>thus, copper sulphate, which is composed of a strong acid united with a weak base, is an <ex>acid salt</ex> in this sense, though theoretically it is a neutral salt</as>.</cd> -- <col><b>Alkaline salt</b></col> <fld>(Chem.)</fld>, <cd>a salt which gives an alkaline reaction, as sodium carbonate.</cd> -- <col><b>Amphid salt</b></col> <fld>(Old Chem.)</fld>, <cd>a salt of the oxy type, formerly regarded as composed of two oxides, an acid and a basic oxide.</cd> <mark>[Obsolescent]</mark> -- <col><b>Basic salt</b></col> <fld>(Chem.)</fld> <sd>(a)</sd> <cd>A salt which contains more of the basic constituent than is required to neutralize the acid</cd>. <sd>(b)</sd> <cd>An alkaline salt.</cd> -- <col><b>Binary salt</b></col> <fld>(Chem.)</fld>, <cd>a salt of the oxy type conveniently regarded as composed of two ingredients (analogously to a haloid salt), viz., a metal and an acid radical.</cd> -- <col><b>Double salt</b></col> <fld>(Chem.)</fld>, <cd>a salt regarded as formed by the union of two distinct salts, as common alum, potassium aluminium sulphate. See under <er>Double</er>.</cd> -- <col><b>Epsom salts</b></col>. <cd>See in the Vocabulary.</cd> -- <col><b>Essential salt</b></col> <fld>(Old Chem.)</fld>, <cd>a salt obtained by crystallizing plant juices.</cd> -- <col><b>Ethereal salt</b></col>. <fld>(Chem.)</fld> <cd>See under <er>Ethereal</er>.</cd> -- <mcol><col><b>Glauber's salt</b></col> <it>or</it> <col><b>Glauber's salts</b></col></mcol>. <cd>See in Vocabulary.</cd> -- <col><b>Haloid salt</b></col> <fld>(Chem.)</fld>, <cd>a simple salt of a halogen acid, as sodium chloride.</cd> -- <col><b>Microcosmic salt</b></col>. <fld>(Chem.)</fld>. <cd>See under <er>Microcosmic</er>.</cd> -- <col><b>Neutral salt</b></col>. <fld>(Chem.)</fld> <sd>(a)</sd> <cd>A salt in which the acid and base (in theory) neutralize each other</cd>. <sd>(b)</sd> <cd>A salt which gives a neutral reaction.</cd> -- <col><b>Oxy salt</b></col> <fld>(Chem.)</fld>, <cd>a salt derived from an oxygen acid.</cd> -- <col><b>Per salt</b></col> <fld>(Old Chem.)</fld>, <cd>a salt supposed to be derived from a peroxide base or analogous compound.</cd> <mark>[Obs.]</mark> -- <col><b>Permanent salt</b></col>, <cd>a salt which undergoes no change on exposure to the air.</cd> -- <col><b>Proto salt</b></col> <fld>(Chem.)</fld>, <cd>a salt derived from a protoxide base or analogous compound.</cd> -- <col><b>Rochelle salt</b></col>. <cd>See under <er>Rochelle</er>.</cd> -- <col><b>Salt of amber</b></col> <fld>(Old Chem.)</fld>, <cd>succinic acid.</cd> -- <col><b>Salt of colcothar</b></col> <fld>(Old Chem.)</fld>, <cd>green vitriol, or sulphate of iron.</cd> -- <col><b>Salt of hartshorn</b></col>. <fld>(Old Chem.)</fld> <sd>(a)</sd> <cd>Sal ammoniac, or ammonium chloride</cd>. <sd>(b)</sd> <cd>Ammonium carbonate. Cf. <cref>Spirit of hartshorn</cref>, under <er>Hartshorn</er>.</cd> -- <col><b>Salt of lemons</b></col>. <fld>(Chem.)</fld> <cd>See <cref>Salt of sorrel</cref>, below.</cd> -- <col><b>Salt of Saturn</b></col> <fld>(Old Chem.)</fld>, <cd>sugar of lead; lead acetate; -- the alchemical name of lead being <xex>Saturn</xex>.</cd> -- <col><b>Salt of Seignette</b></col>. <cd>Same as <er>Rochelle salt</er>.</cd> -- <col><b>Salt of soda</b></col> <fld>(Old Chem.)</fld>, <cd>sodium carbonate.</cd> -- <col><b>Salt of sorrel</b></col> <fld>(Old Chem.)</fld>, <cd>acid potassium oxalate, or potassium quadroxalate, used as a solvent for ink stains; -- so called because found in the sorrel, or Oxalis. Also sometimes inaccurately called <altname>salt of lemon</altname>.</cd> -- <col><b>Salt of tartar</b></col> <fld>(Old Chem.)</fld>, <cd>potassium carbonate; -- so called because formerly made by heating cream of tartar, or potassium tartrate.</cd> <mark>[Obs.]</mark> -- <col><b>Salt of Venus</b></col> <fld>(Old Chem.)</fld>, <cd>blue vitriol; copper sulphate; -- the alchemical name of copper being <xex>Venus</xex>.</cd> -- <col><b>Salt of wisdom</b></col>. <cd>See <er>Alembroth</er>.</cd> -- <col><b>Sedative salt</b></col> <fld>(Old Med. Chem.)</fld>, <cd>boric acid.</cd> -- <col><b>Sesqui salt</b></col> <fld>(Chem.)</fld>, <cd>a salt derived from a sesquioxide base or analogous compound.</cd> -- <col><b>Spirit of salt</b></col>. <fld>(Chem.)</fld> <cd>See under <er>Spirit</er>.</cd> -- <col><b>Sulpho salt</b></col> <fld>(Chem.)</fld>, <cd>a salt analogous to an oxy salt, but containing sulphur in place of oxygen.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Salt</ent><br/
<hw>Salt</hw> <pr>(?)</pr>, <pos>a.</pos> <amorph>[<pos>Compar.</pos> <adjf>Salter</adjf> <pr>(?)</pr>; <pos>superl.</pos> <adjf>Saltest</adjf>.]</amorph> <ety>[AS. <ets>sealt</ets>, <ets>salt</ets>. See <er>Salt</er>, <pos>n.</pos>]</ety> <sn>1.</sn> <def>Of or relating to salt; abounding in, or containing, salt; prepared or preserved with, or tasting of, salt; salted; <as>as, <ex>salt</ex> beef; <ex>salt</ex> water</as>.</def> <ldquo/<xex>Salt</xex> tears.<rdquo/ <rj><au>Chaucer.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Overflowed with, or growing in, salt water; <as>as, a <ex>salt</ex> marsh; <ex>salt</ex> grass</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def><markp>Fig.</markp>: Bitter; sharp; pungent.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>I have a <qex>salt</qex> and sorry rheum offends me.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def><markp>Fig.</markp>: Salacious; lecherous; lustful.</def> <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Salt</ent><br/
<-- Alphabetically, <col><b>Salt rising</b></col> should come after <col><b>Salt raker</b></col> within <hw>Salt</hw> <pos>a.</pos>; not before. PCP --><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Salt acid</b></col> <fld>(Chem.)</fld>, <cd>hydrochloric acid.</cd> -- <col><b>Salt block</b></col>, <cd>an apparatus for evaporating brine; a salt factory.</cd> <au>Knight.</au> -- <col><b>Salt bottom</b></col>, <cd>a flat piece of ground covered with saline efflorescences.</cd> <mark>[Western U.S.]</mark> <au>Bartlett.</au> -- <col><b>Salt cake</b></col> <fld>(Chem.)</fld>, <cd>the white caked mass, consisting of sodium sulphate, which is obtained as the product of the first stage in the manufacture of soda, according to Leblanc's process.</cd> -- <col><b>Salt fish</b></col>. <sd>(a)</sd> <cd>Salted fish, especially cod, haddock, and similar fishes that have been salted and dried for food.</cd> <sd>(b)</sd> <cd>A marine fish.</cd> -- <col><b>Salt garden</b></col>, <cd>an arrangement for the natural evaporation of sea water for the production of salt, employing large shallow basins excavated near the seashore.</cd> -- <col><b>Salt gauge</b></col>, <cd>an instrument used to test the strength of brine; a salimeter.</cd> -- <col><b>Salt horse</b></col>, <cd>salted beef.</cd> <mark>[Slang]</mark> -- <col><b>Salt junk</b></col>, <cd>hard salt beef for use at sea.</cd> <mark>[Slang]</mark> -- <col><b>Salt lick</b></col>. <cd>See <er>Lick</er>, <pos>n.</pos></cd> -- <col><b>Salt marsh</b></col>, <cd>grass land subject to the overflow of salt water.</cd> -- <col><b>Salt-marsh caterpillar</b></col> <fld>(Zool.)</fld>, <cd>an American bombycid moth (<spn>Spilosoma acraea</spn> which is very destructive to the salt-marsh grasses and to other crops. Called also <altname>woolly bear</altname>. See Illust. under <er>Moth</er>, <er>Pupa</er>, and <cref>Woolly bear</cref>, under <er>Woolly</er>.</cd> -- <col><b>Salt-marsh fleabane</b></col> <fld>(Bot.)</fld>, <cd>a strong-scented composite herb (<spn>Pluchea camphorata</spn>) with rayless purplish heads, growing in salt marshes.</cd> -- <col><b>Salt-marsh hen</b></col> <fld>(Zool.)</fld>, <cd>the clapper rail. See under <er>Rail</er>.</cd> -- <col><b>Salt-marsh terrapin</b></col> <fld>(Zool.)</fld>, <cd>the diamond-back.</cd> -- <col><b>Salt mine</b></col>, <cd>a mine where rock salt is obtained.</cd> -- <col><b>Salt pan</b></col>. <sd>(a)</sd> <cd>A large pan used for making salt by evaporation; also, a shallow basin in the ground where salt water is evaporated by the heat of the sun.</cd> <sd>(b)</sd> <pluf>pl.</pluf> <cd>Salt works.</cd> -- <col><b>Salt pit</b></col>, <cd>a pit where salt is obtained or made.</cd> -- <col><b>Salt rising</b></col>, <cd>a kind of yeast in which common salt is a principal ingredient.</cd> <mark>[U.S.]</mark> -- <col><b>Salt raker</b></col>, <cd>one who collects salt in natural salt ponds, or inclosures from the sea.</cd> -- <col><b>Salt sedative</b></col> <fld>(Chem.)</fld>, <cd>boracic acid.</cd> <mark>[Obs.]</mark> -- <col><b>Salt spring</b></col>, <cd>a spring of salt water.</cd> -- <col><b>Salt tree</b></col> <fld>(Bot.)</fld>, <cd>a small leguminous tree (<spn>Halimodendron argenteum</spn>) growing in the salt plains of the Caspian region and in Siberia.</cd> -- <col><b>Salt water</b></col>, <cd>water impregnated with salt, as that of the ocean and of certain seas and lakes; sometimes, also, tears.</cd><br/
[<source>1913 Webster</source>]</p>

<p><q>Mine eyes are full of tears, I can not see;<br/
And yet <qex>salt water</qex> blinds them not so much<br/
But they can see a sort of traitors here.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p>-- <col><b>Salt-water sailor</b></col>, <cd>an ocean mariner.</cd> -- <col><b>Salt-water tailor</b></col>. <fld>(Zool.)</fld> <cd>See <er>Bluefish</er>.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Salt</ent><br/
<hw>Salt</hw>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Salted</conjf>; <pos>p. pr. & vb. n.</pos> <conjf>Salting</conjf>.]</vmorph> <sn>1.</sn> <def>To sprinkle, impregnate, or season with salt; to preserve with salt or in brine; to supply with salt; <as>as, to <ex>salt</ex> fish, beef, or pork; to <ex>salt</ex> cattle</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To fill with salt between the timbers and planks, as a ship, for the preservation of the timber.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>To salt a mine</b></col>, <cd>to artfully deposit minerals in a mine in order to deceive purchasers regarding its value.</cd> <mark>[Cant]</mark> -- <mcol><col><b>To salt away</b></col>, <col><b>To salt down</b></col></mcol>, <cd>to prepare with, or pack in, salt for preserving, as meat, eggs, etc.; hence, colloquially, to save, lay up, or invest sagely, as money.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Salt</ent><br/
<hw>Salt</hw> <pr>(?)</pr>, <pos>v. i.</pos> <def>To deposit salt as a saline solution; <as>as, the brine begins to <ex>salt</ex></as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Salt</ent><br/
<hw>Salt</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>saltus</ets>, fr. <ets>salire</ets> to leap.]</ety> <def>The act of leaping or jumping; a leap.</def> <mark>[Obs.]</mark> <rj><au>B. Jonson.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Saltant</ent><br/
<hw>Sal"tant</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>saltans</ets>, p. pr. of <ets>saltare</ets> to dance, v. intens. fr. <ets>salire</ets> to leap: cf. F. <ets>sautant</ets>. See <er>Sally</er>, <pos>v.</pos>]</ety> <sn>1.</sn> <def>Leaping; jumping; dancing.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Her.)</fld> <def>In a leaping position; springing forward; -- applied especially to the squirrel, weasel, and rat, also to the cat, greyhound, monkey, etc.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Saltarella</ent><br/
||<hw>Sal`ta*rel"la</hw> <pr>(?)</pr>, <pos>n.</pos> <def>See <er>Saltarello</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Saltarello</ent><br/
||<hw>Sal`ta*rel"lo</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[It., fr. L. <ets>saltare</ets> to jump.]</ety> <def>A popular Italian dance in quick 3-4 or 6-8 time, running mostly in triplets, but with a hop step at the beginning of each measure. See <er>Tarantella</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Saltate</ent><br/
<hw>Sal"tate</hw> <pr>(?)</pr>, <pos>v. i.</pos> <ety>[See <er>Saltant</er>.]</ety> <def>To leap or dance.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Saltation</ent><br/
<hw>Sal*ta"tion</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>saltatio</ets>: cf. F. <ets>saltation</ets>.]</ety> <sn>1.</sn> <def>A leaping or jumping.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Continued his <qex>saltation</qex> without pause.</q> <rj><qau>Sir W. Scott.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Beating or palpitation; <as>as, the <ex>saltation</ex> of the great artery</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <fld>(Biol.)</fld> <def>An abrupt and marked variation in the condition or appearance of a species; a sudden modification which may give rise to new races.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>We greatly suspect that nature does make considerable jumps in the way of variation now and then, and that these <qex>saltations</qex> give rise to some of the gaps which appear to exist in the series of known forms.</q> <rj><qau>Huxley.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Saltatoria</ent><br/
||<hw>Sal`ta*to"ri*a</hw> <pr>(?)</pr>, <pos>n. pl.</pos> <ety>[NL.]</ety> <fld>(Zool.)</fld> <def>A division of Orthoptera including grasshoppers, locusts, and crickets.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Saltatorial</ent><br/
<hw>Sal`ta*to"ri*al</hw> <pr>(?)</pr>, <pos>a.</pos> <sn>1.</sn> <def>Relating to leaping; saltatory; <as>as, <ex>saltatorial</ex> exercises</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Zool.)</fld> <sd>(a)</sd> <def>Same as <er>Saltatorious</er>.</def> <sd>(b)</sd> <def>Of or pertaining to the Saltatoria.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Saltatorious</ent><br/
<hw>Sal`ta*to"ri*ous</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Capable of leaping; formed for leaping; saltatory; <as>as, a <ex>saltatorious</ex> insect or leg</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Saltatory</ent><br/
<hw>Sal"ta*to"ry</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>saltatorius</ets>. See <er>Saltant</er>, and cf. <er>Saltire</er>.]</ety> <def>Leaping or dancing; having the power of, or used in, leaping or dancing.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Saltatory evolution</b></col> <fld>(Biol.)</fld>, <cd>a theory of evolution which holds that the transmutation of species is not always gradual, but that there may come sudden and marked variations. See <er>Saltation</er>.</cd><-- recently revived as "punctuated equilibrium" --> -- <col><b>Saltatory spasm</b></col> <fld>(Med.)</fld>, <cd>an affection in which pressure of the foot on a floor causes the patient to spring into the air, so as to make repeated involuntary motions of hopping and jumping.</cd> <rj><au>J. Ross.</au></rj></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Saltbush</ent><br/
<hw>Salt"bush`</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Bot.)</fld> <def>An Australian plant (<spn>Atriplex nummularia</spn>) of the Goosefoot family.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Saltcat</ent><br/
<hw>Salt"cat`</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A mixture of salt, coarse meal, lime, etc., attractive to pigeons.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Saltcellar</ent><br/
<hw>Salt"cel*lar</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[OE. <ets>saltsaler</ets>; <ets>salt</ets> + F. <ets>sali<egrave/re</ets> saltcellar, from L. <ets>sal</ets> salt. See <er>Salt</er>, and cf. <er>Salary</er>.]</ety> <def>Formerly a large vessel, now a small vessel of glass or other material, used for holding salt on the table.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Salter</ent><br/
<hw>Salt"er</hw> <pr>(?)</pr>, <pos>n.</pos> <def>One who makes, sells, or applies salt; one who salts meat or fish.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Saltern</ent><br/
<hw>Salt"ern</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A building or place where salt is made by boiling or by evaporation; salt works.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Saltfoot</ent><br/
<hw>Salt"foot`</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A large saltcellar formerly placed near the center of the table. The superior guests were seated above the saltfoot.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Salt-green</ent><br/
<hw>Salt"-green</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Sea-green in color.</def> <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Saltie</ent><br/
<hw>Salt"ie</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Zool.)</fld> <def>The European dab.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Saltier</ent><br/
<hw>Sal"tier</hw> <pr>(?)</pr>, <pos>n.</pos> <def>See <er>Saltire</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Saltigradae</ent><br/
||<hw>Sal`ti*gra"dae</hw> <pr>(?)</pr>, <pos>n. pl.</pos> <ety>[NL. See <er>Saltigrade</er>.]</ety> <fld>(Zool.)</fld> <def>A tribe of spiders including those which lie in wait and leap upon their prey; the leaping spiders; called also <altname>Salticidae</altname>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Saltigrade</ent><br/
<hw>Sal"ti*grade</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>saltus</ets> a leap + <ets>gradi</ets> to walk, go: cf. F. <ets>saltigrade</ets>.]</ety> <fld>(Zool.)</fld> <def>Having feet or legs formed for leaping.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Saltigrade</ent><br/
<hw>Sal"ti*grade</hw>, <pos>n.</pos> <fld>(Zool.)</fld> <def>One of the <fam>Saltigradae</fam>, a tribe of spiders which leap to seize their prey.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Saltimbanco</ent><br/
<hw>Sal`tim*ban"co</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[It., literally, one who leaps or mounts upon a bench; <ets>saltare</ets> to leap + <ets>in</ets> in, upon + <ets>banco</ets> a bench.]</ety> <def>A mountebank; a quack.</def> <mark>[Obs.]</mark> <altsp>[Written also <asp>santinbanco</asp>.]</altsp><br/
[<source>1913 Webster</source>]</p>

<p><q><qex>Saltimbancos</qex>, quacksalvers, and charlatans.</q> <rj><qau>Sir T. Browne.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>saltiness</ent><br/
<hw>salt"i*ness</hw> <pr>(s<add/lt"<esl/*n<ecr/s)</pr>, <pos>n.</pos> <sn>1.</sn> <def>The quality or state of containing salt; salt taste; <as>as, the <ex>saltiness</ex> of sea water</as>.</def><br/
<syn><b>Syn. --</b> saltiness.</syn><br/
[<source>1913 Webster</source> + <source>PJC</source>]</p>

<p><ent>Salting</ent><br/
<hw>Salt"ing</hw> <pr>(?)</pr>, <pos>n.</pos> <sn>1.</sn> <def>The act of sprinkling, impregnating, or furnishing, with salt.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A salt marsh.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Saltire</ent><br/
<hw>Sal"tire</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F. <ets>sautoir</ets>, fr. LL. <ets>saltatorium</ets> a sort of stirrup, fr. L. <ets>saltatorius</ets> saltatory. See <er>Saltatory</er>, <er>Sally</er>, <pos>v.</pos>]</ety> <fld>(Her.)</fld> <def>A St. Andrew's cross, or cross in the form of an <universbold>X</universbold>, -- one of the honorable ordinaries.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Saltirewise</ent><br/
<hw>Sal"tire*wise`</hw> <pr>(?)</pr>, <pos>adv.</pos> <fld>(Her.)</fld> <def>In the manner of a saltire; -- said especially of the blazoning of a shield divided by two lines drawn in the direction of a bend and a bend sinister, and crossing at the center.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Saltish</ent><br/
<hw>Salt"ish</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Somewhat salt.</def> -- <wordforms><wf>Salt"ish*ly</wf>, adv. -- <wf>Salt"ish*ness</wf>, <pos>n.</pos></wordforms><br/
[<source>1913 Webster</source>]</p>

<p><ent>Saltless</ent><br/
<hw>Salt"less</hw>, <pos>a.</pos> <def>Destitute of salt; insipid.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Saltly</ent><br/
<hw>Salt"ly</hw>, <pos>adv.</pos> <def>With taste of salt; in a salt manner.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Saltmouth</ent><br/
<hw>Salt"mouth`</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A wide-mouthed bottle with glass stopper for holding chemicals, especially crystallized salts.</def><br/
[<source>1913 Webster</source>]</p>

<p><-- p. 1272 pr=PCP --></p>

<p><ent>Saltness</ent><br/
<hw>Salt"ness</hw> <pr>(s<add/lt"n<ecr/s)</pr>, <pos>n.</pos> <def>The quality or state of being salt, or impregnated with salt; salt taste; <as>as, the <ex>saltness</ex> of sea water</as>.  In the sense of having salt content, <altname>saltiness</altname> is more commonly used.</def><br/
<syn><b>Syn. --</b> saltiness.</syn><br/
[<source>1913 Webster</source> + <source>PJC</source>]</p>

<p><ent>Saltpetre</ent><br/
<ent>Saltpeter</ent><br/
<mhw>{ <hw>Salt`pe"ter</hw>, <hw>Salt`pe"tre</hw>, }</mhw> <pr>(s<add/lt`p<emac/"t<etil/r)</pr>, <pos>n.</pos> <ety>[F. <ets>salp<ecir/tre</ets>, NL. <ets>sal petrae</ets>, literally, rock salt, or stone salt; so called because it exudes from rocks or walls. See <er>Salt</er>, and <er>Petrify</er>.]</ety> <fld>(Chem.)</fld> <def>Potassium nitrate; niter; a white crystalline substance, <chform>KNO3</chform>, having a cooling saline taste, obtained by leaching from certain soils in which it is produced by the process of nitrification (see <er>Nitrification</er>, 2). It is a strong oxidizer, is the chief constituent of gunpowder, and is also used as an antiseptic in curing meat, and in medicine as a diuretic, diaphoretic, and refrigerant.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Chili salpeter</b></col> <fld>(Chem.)</fld>, <cd>sodium nitrate (distinguished from potassium nitrate, or true salpeter), a white crystalline substance, <chform>NaNO3</chform>, having a cooling, saline, slightly bitter taste. It is obtained by leaching the soil of the rainless districts of Chili and Peru. It is deliquescent and cannot be used in gunpowder, but is employed in the production of nitric acid. Called also <altname>cubic niter</altname>.</cd> -- <col><b>Saltpeter acid</b></col> <fld>(Chem.)</fld>, <cd>nitric acid; -- sometimes so called because made from saltpeter.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Saltpetrous</ent><br/
<hw>Salt`pe"trous</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[Cf. F. <ets>salp<ecir/treux</ets>.]</ety> <def>Pertaining to saltpeter, or partaking of its qualities; impregnated with saltpeter.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Salt rheum</ent><br/
<hw>Salt" rheum</hw> <pr>(?)</pr>. <fld>(Med.)</fld> <def>A popular name, esp. in the United States, for various cutaneous eruptions, particularly for those of eczema. See <er>Eczema</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Saltwort</ent><br/
<hw>Salt"wort`</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Bot.)</fld> <def>A name given to several plants which grow on the seashore, as the <spn>Batis maritima</spn>, and the glasswort. See <er>Glasswort</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Black saltwort</b></col>, <cd>the sea milkwort.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Salty</ent><br/
<hw>Salt"y</hw> <pr>(s<add/lt"<ycr/)</pr>, <pos>a.</pos> <sn>1.</sn> <def>Containing salt; tasting of salt; saltish; <as>as, the <ex>salty</ex> sea; the potatoes are too <ex>salty</ex></as>.</def><br/
[<source>1913 Webster</source> + <source>PJC</source>]</p>

<p><sn>2.</sn> <def>Interesting and witty; piquant; -- of discourse.</def><br/
[<source>PJC</source>]</p>

<p><sn>3.</sn> <def>Racy; sexually suggestive; -- of discourse.</def><br/
[<source>PJC</source>]</p>

<p><ent>Salubrious</ent><br/
<hw>Sa*lu"bri*ous</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>salubris</ets>, or <ets>saluber</ets>, fr. <ets>salus</ets> health; akin to <ets>salvus</ets> safe, sound, well. See <er>Safe</er>.]</ety> <def>Favorable to health; healthful; promoting health; <as>as, <ex>salubrious</ex> air, water, or climate</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><syn><b>Syn.</b> -- Healthful; wholesome; healthy; salutary.</syn><br/
[<source>1913 Webster</source>]</p>

<p>-- <wordforms><wf>Sa-lu"bri*ous*ly</wf>, <pos>adv.</pos> -- <wf>Sa*lu"bri*ous*ness</wf>, <pos>n.</pos></wordforms><br/
[<source>1913 Webster</source>]</p>

<p><ent>Salubrity</ent><br/
<hw>Sa*lu"bri*ty</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>salubritas</ets>: cf. F. <ets>salubrit<eacute/</ets> See <er>Salubrious</er>.]</ety> <def>The quality of being salubrious; favorableness to the preservation of health; salubriousness; wholesomeness; healthfulness; <as>as, the <ex>salubrity</ex> of the air, of a country, or a climate</as>.</def> <ldquo/A sweet, dry smell of <xex>salubrity</xex>.<rdquo/ <rj><au>G. W. Cable.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Salue</ent><br/
<hw>Sa*lue"</hw> <pr>(?)</pr>, <pos>v. t.</pos> <ety>[F. <ets>saluer</ets>. See <er>Salute</er>.]</ety> <def>To salute.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>There was no <ldquo/good day<rdquo/ and no <qex>saluyng</qex>.</q> <rj><qau>Chaucer.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Salutary</ent><br/
<hw>Sal"u*ta*ry</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>salutaris</ets>, from <ets>salus</ets>, <ets>-utis</ets>, health, safety: cf. F. <ets>salutaire</ets>. See <er>Salubrious</er>.]</ety> <sn>1.</sn> <def>Wholesome; healthful; promoting health; <as>as, <ex>salutary</ex> exercise</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Promotive of, or contributing to, some beneficial purpose; beneficial; advantageous; <as>as, a <ex>salutary</ex> design</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><syn><b>Syn.</b> -- Wholesome; healthful; salubrious; beneficial; useful; advantageous; profitable.</syn><br/
[<source>1913 Webster</source>]</p>

<p>-- <wordforms><wf>Sal"u*ta*ri*ly</wf> <pr>(#)</pr>, <pos>adv.</pos> -- <wf>Sal"u*ta*ri*ness</wf>, <pos>n.</pos></wordforms><br/
[<source>1913 Webster</source>]</p>

<p><ent>Salutation</ent><br/
<hw>Sal`u*ta"tion</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>salutatio</ets>: cf. F. <ets>salutation</ets>. See <er>Salute</er>.]</ety> <def>The act of saluting, or paying respect or reverence, by the customary words or actions; the act of greeting, or expressing good will or courtesy; also, that which is uttered or done in saluting or greeting.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>In all public meetings or private addresses, use those forms of <qex>salutation</qex>, reverence, and decency usual amongst the most sober persons.</q> <rj><qau>Jer. Taylor.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><syn><b>Syn.</b> -- Greeting; salute; address.</syn> -- <usage><er>Salutation</er>, <er>Greeting</er>, <er>Salute</er>. <xex>Greeting</xex> is the general word for all manner of expressions of recognition, agreeable or otherwise, made when persons meet or communicate with each other. A <xex>greeting</xex> may be hearty and loving, chilling and offensive, or merely formal, as in the opening sentence of legal documents. <xex>Salutation</xex> more definitely implies a wishing well, and is used of expressions at parting as well as at meeting. It is used especially of uttered expressions of good will. <xex>Salute</xex>, while formerly and sometimes still in the sense of either <xex>greeting</xex> or <xex>salutation</xex>, is now used specifically to denote a conventional demonstration not expressed in words. The guests received a <xex>greeting</xex> which relieved their embarrassment, offered their <xex>salutations</xex> in well-chosen terms, and when they retired, as when they entered, made a deferential <xex>salute</xex>.</usage><br/
[<source>1913 Webster</source>]</p>

<p><q>Woe unto you, Pharisees! for ye love the uppermost seats in the synagogues, and <qex>greetings</qex> in the markets.</q> <rj><qau>Luke xi. 43.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>When Elisabeth heard the <qex>salutation</qex> of Mary, the babe leaped in her womb.</q> <rj><qau>Luke i. 41.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>I shall not trouble my reader with the first <qex>salutes</qex> of our three friends.</q> <rj><qau>Addison.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Salutatorian</ent><br/
<hw>Sa*lu`ta*to"ri*an</hw> <pr>(?)</pr>, <pos>n.</pos> <def>The student who pronounces the salutatory oration at the annual Commencement or like exercises of a college, -- an honor commonly assigned to that member of the graduating class who ranks second in scholarship.</def> <mark>[U.S.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Salutatorily</ent><br/
<hw>Sa*lu"ta*to*ri*ly</hw> <pr>(?)</pr>, <pos>adv.</pos> <def>By way of salutation.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Salutatory</ent><br/
<hw>Sa*lu"ta*to*ry</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>salutatorius</ets>. See <er>Salute</er>.]</ety> <def>Containing or expressing salutations; speaking a welcome; greeting; -- applied especially to the oration which introduces the exercises of the Commencements, or similar public exhibitions, in American colleges.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Salutatory</ent><br/
<hw>Sa*lu"ta*to*ry</hw>, <pos>n.</pos> <sn>1.</sn> <def>A place for saluting or greeting; a vestibule; a porch.</def> <mark>[Obs.]</mark> <rj><au>Milton.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(American Colleges)</fld> <def>The salutatory oration.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Salute</ent><br/
<hw>Sa*lute"</hw> <pr>(?)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Saluted</conjf>; <pos>p. pr. & vb. n.</pos> <conjf>Saluting</conjf>.]</vmorph> <ety>[L. <ets>salutare</ets>, <ets>salutatum</ets>, from <ets>salus</ets>, <ets>-utis</ets>, health, safety. See <er>Salubrious</er>.]</ety> <sn>1.</sn> <def>To address, as with expressions of kind wishes and courtesy; to greet; to hail.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>I <qex>salute</qex> you with this kingly title.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Hence, to give a sign of good will; to compliment by an act or ceremony, as a kiss, a bow, etc.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>You have the prettiest tip of a finger . . . I must take the freedom to <qex>salute</qex> it.</q> <rj><qau>Addison.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <fld>(Mil. & Naval)</fld> <def>To honor, as some day, person, or nation, by a discharge of cannon or small arms, by dipping colors, by cheers, etc.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>To promote the welfare and safety of; to benefit; to gratify.</def> <mark>[Obs.]</mark> <ldquo/If this <xex>salute</xex> my blood a jot.<rdquo/ <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Salute</ent><br/
<hw>Sa*lute"</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Cf. F. <ets>salut</ets>. See <er>Salute</er>, <pos>v.</pos>]</ety> <sn>1.</sn> <def>The act of saluting, or expressing kind wishes or respect; salutation; greeting.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A sign, token, or ceremony, expressing good will, compliment, or respect, as a kiss, a bow, etc.</def> <rj><au>Tennyson.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <fld>(Mil. & Naval)</fld> <def>A token of respect or honor for some distinguished or official personage, for a foreign vessel or flag, or for some festival or event, as by presenting arms, by a discharge of cannon, volleys of small arms, dipping the colors or the topsails, etc.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Saluter</ent><br/
<hw>Sa*lut"er</hw> <pr>(?)</pr>, <pos>n.</pos> <def>One who salutes.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Salutiferous</ent><br/
<hw>Sal`u*tif"er*ous</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>salutifer</ets>; <ets>salus</ets>, <ets>-utis</ets>, health + <ets>ferre</ets> to bring.]</ety> <def>Bringing health; healthy; salutary; beneficial; <as>as, <ex>salutiferous</ex> air</as>.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>Innumerable powers, all of them <qex>salutiferous</qex>.</q> <rj><qau>Cudworth.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><syn><b>Syn.</b> -- Healthful; healthy; salutary; salubrious.</syn><br/
[<source>1913 Webster</source>]</p>

<p><ent>Salutiferously</ent><br/
<hw>Sal`u*tif"er*ous*ly</hw>, <pos>adv.</pos> <def>Salutarily.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Salvability</ent><br/
<hw>Sal`va*bil"i*ty</hw> <pr>(?)</pr>, <pos>n.</pos> <def>The quality or condition of being salvable; salvableness.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>In the Latin scheme of redemption, <qex>salvability</qex> was not possible outside the communion of the visible organization.</q> <rj><qau>A. V. G. Allen.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Salvable</ent><br/
<hw>Sal"va*ble</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>salvare</ets> to save, from <ets>salvus</ets> safe. Cf. <er>Savable</er>.]</ety> <def>Capable of being saved; admitting of salvation.</def> <rj><au>Dr. H. More.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p>-- <wordforms><wf>Sal"va*ble*ness</wf>, <pos>n.</pos> -- <wf>Sal"va*bly</wf>, <pos>adv.</pos></wordforms><br/
[<source>1913 Webster</source>]</p>

<p><ent>Salvage</ent><br/
<hw>Sal"vage</hw> <pr>(?; 48)</pr>, <pos>n.</pos> <ety>[F. <ets>salvage</ets>, OF. <ets>salver</ets> to save, F. <ets>sauver</ets>, fr. L. <ets>salvare</ets>. See <er>Save</er>.]</ety> <sn>1.</sn> <def>The act of saving a vessel, goods, or life, from perils of the sea.</def><br/
[<source>1913 Webster</source>]</p>

<p><q><qex>Salvage</qex> of life from a British ship, or a foreign ship in British waters, ranks before <qex>salvage</qex> of goods.</q> <rj><qau>Encyc. Brit.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Maritime Law)</fld> <sd>(a)</sd> <def>The compensation allowed to persons who voluntarily assist in saving a ship or her cargo from peril.</def> <sd>(b)</sd> <def>That part of the property that survives the peril and is saved.</def> <rj><au>Kent. Abbot.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Salvage</ent><br/
<hw>Sal"vage</hw>, <pos>a. & n.</pos> <def>Savage.</def> <mark>[Obs.]</mark> <rj><au>Spenser.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Salvation</ent><br/
<hw>Sal*va"tion</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[OE. <ets>salvacioun</ets>, <ets>sauvacion</ets>, F. <ets>salvation</ets>, fr. L. <ets>salvatio</ets>, fr. <ets>salvare</ets> to save. See <er>Save</er>.]</ety> <sn>1.</sn> <def>The act of saving; preservation or deliverance from destruction, danger, or great calamity.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Theol.)</fld> <def>The redemption of man from the bondage of sin and liability to eternal death, and the conferring on him of everlasting happiness.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>To earn <qex>salvation</qex> for the sons of men.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Godly sorrow worketh repentance to <qex>salvation</qex>.</q> <rj><qau>2. Cor. vii. 10.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>Saving power; that which saves.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Fear ye not; stand still, and see the <qex>salvation</qex> of the Lord, which he will show to you to-day.</q> <rj><qau>Ex. xiv. 13.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Salvation Army</b></col>, <cd>an organization for prosecuting the work of Christian evangelization, especially among the degraded populations of cities. It is virtually a new sect founded in London in 1861 by William Booth. The evangelists, male and female, have military titles according to rank, that of the chief being <ldquo/General.<rdquo/ They wear a uniform, and in their phraseology and mode of work adopt a quasi military style.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Salvationist</ent><br/
<hw>Sal*va"tion*ist</hw>, <pos>n.</pos> <def>An evangelist, a member, or a recruit, of the Salvation Army.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Salvatory</ent><br/
<hw>Sal"va*to*ry</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[LL. <ets>salvatorium</ets>, fr. <ets>salvare</ets> to save.]</ety> <def>A place where things are preserved; a repository.</def> <mark>[R.]</mark> <rj><au>Sir M. Hale.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Salve</ent><br/
||<hw>Sal"ve</hw> <pr>(?)</pr>, <pos>interj.</pos> <ety>[L., hail, God save you, imperat. of <ets>salvere</ets> to be well. Cf. <er>Salvo</er> a volley.]</ety> <def>Hail!</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Salve</ent><br/
<hw>Sal"ve</hw> <pr>(? <or/ ?)</pr>, <pos>v. t.</pos> <def>To say <ldquo/<xex>Salve</xex><rdquo/ to; to greet; to salute.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>By this that stranger knight in presence came,<br/
And goodly <qex>salved</qex> them.</q> <rj><qau>Spenser.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Salve</ent><br/
<hw>Salve</hw> <pr>(?; 277)</pr>, <pos>n.</pos> <ety>[AS. <ets>sealf</ets> ointment; akin to LG. <ets>salwe</ets>, D. <ets>zalve</ets>, <ets>zalf</ets>, OHG. <ets>salba</ets>, Dan. <ets>salve</ets>, Sw. <ets>salfva</ets>, Goth. <ets>salb<omac/n</ets> to anoint, and probably to Gr. (Hesychius) <?/ oil, <?/ butter, Skr. <ets>sarpis</ets> clarified butter. <root/155, 291.]</ety> <sn>1.</sn> <def>An adhesive composition or substance to be applied to wounds or sores; a healing ointment.</def> <rj><au>Chaucer.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A soothing remedy or antidote.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Counsel or consolation we may bring.<br/
<qex>Salve</qex> to thy sores.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Salve bug</b></col> <fld>(Zool.)</fld>, <cd>a large, stout isopod crustacean (<spn>Aega psora</spn>), parasitic on the halibut and codfish, -- used by fishermen in the preparation of a salve. It becomes about two inches in length.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Salve</ent><br/
<hw>Salve</hw>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Salved</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Salving</conjf>.]</vmorph> <ety>[AS. <ets>sealfian</ets> to anoint. See <er>Salve</er>, <pos>n.</pos>]</ety> <sn>1.</sn> <def>To heal by applications or medicaments; to cure by remedial treatment; to apply salve to; <as>as, to <ex>salve</ex> a wound</as>.</def> <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To heal; to remedy; to cure; to make good; to soothe, as with an ointment, especially by some device, trick, or quibble; to gloss over.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>But Ebranck <qex>salved</qex> both their infamies<br/
With noble deeds.</q> <rj><qau>Spenser.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>What may we do, then, to <qex>salve</qex> this seeming inconsistence?</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><-- salve one's conscience. salve one's wounded pride --><br/
[<source>1913 Webster</source>]</p>

<p><ent>Salve</ent><br/
<hw>Salve</hw> <pr>(?)</pr>, <pos>v. t. & i.</pos> <ety>[See <er>Salvage</er>]</ety> <def>To save, as a ship or goods, from the perils of the sea.</def> <mark>[Recent]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Salver</ent><br/
<hw>Salv"er</hw> <pr>(?)</pr>, <pos>n.</pos> <def>One who salves, or uses salve as a remedy; hence, a quacksalver, or quack.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Salver</ent><br/
<hw>Sal"ver</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Cf. <er>Salvage</er>.]</ety> <def>A salvor.</def> <rj><au>Skeat.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Salver</ent><br/
<hw>Sal"ver</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Sp. <ets>salva</ets> pregustation, the tasting of viands before they are served, salver, fr. <ets>salvar</ets> to save, to taste, to prove the food or drink of nobles, from L. <ets>salvare</ets> to save. See <er>Save</er>.]</ety> <def>A tray or waiter on which anything is presented.</def> <-- now used mostly in compounds; e.g. tea salver --><br/
[<source>1913 Webster</source>]</p>

<p><ent>Salver-shaped</ent><br/
<hw>Sal"ver-shaped`</hw> <pr>(?)</pr>, <pos>a.</pos> <fld>(Bot.)</fld> <def>Tubular, with a spreading border. See <er>Hypocraterimorphous</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Salvia</ent><br/
||<hw>Sal"vi*a</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L., sage.]</ety> <fld>(Bot.)</fld> <def>A genus of plants including the sage. See <er>Sage</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Salvific</ent><br/
<hw>Sal*vif"ic</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>salficus</ets> saving; <ets>salvus</ets> saved, safe + <ets>facere</ets> to make.]</ety> <def>Tending to save or secure safety.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Salvo</ent><br/
<hw>Sal"vo</hw> <pr>(?)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Salvos</plw> <pr>(#)</pr>.</plu> <ety>[L. <ets>salvo jure</ets>, literally, the right being reserved. See <er>Safe.</er>]</ety> <def>An exception; a reservation; an excuse.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>They admit many <qex>salvos</qex>, cautions, and reservations.</q> <rj><qau>Eikon Basilike.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Salvo</ent><br/
<hw>Sal"vo</hw>, <pos>n.</pos> <ety>[F. <ets>salve</ets> a discharge of heavy cannon, a volley, L. <ets>salve</ets> hail, imperat. of <ets>salvere</ets> to be well, akin to <ets>salvus</ets> well. See <er>Safe</er>.]</ety> <sn>1.</sn> <fld>(Mil.)</fld> <def>A concentrated fire from pieces of artillery, as in endeavoring to make a break in a fortification; a volley.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A salute paid by a simultaneous, or nearly simultaneous, firing of a number of cannon.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Salvor</ent><br/
<hw>Sal"vor</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[See <er>Salvation</er>, <er>Save</er>]</ety> <fld>(Law)</fld> <def>One who assists in saving a ship or goods at sea, without being under special obligation to do so.</def> <rj><au>Wheaton.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sam</ent><br/
<hw>Sam</hw> <pr>(?)</pr>, <pos>adv.</pos> <ety>[AS. <ets>same</ets>. See <er>Same</er>, <pos>a.</pos>]</ety> <def>Together.</def> <mark>[Obs.]</mark> <ldquo/All in that city <xex>sam</xex>.<rdquo/ <rj><au>Spenser.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>SAM</ent><br/
<hw>SAM</hw> <pr>(s<acr/m)</pr>, <pos>n.</pos> <ety>[acronym]</ety> <fld>(Mil.)</fld> <def>a <er>Surface to Air Missile</er>.</def><br/
[<source>PJC</source>]</p>

<p><ent>Samaj</ent><br/
||<hw>Sa*maj"</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Hind. <ets>sam<amac/j</ets> meeting, assembly, fr. Skr. <ets>sam<amac/ja</ets> a community.]</ety> <def>A society or congregation; a church or religious body.</def> <mark>[India]</mark><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Samara</ent><br/
<hw>Sa*ma"ra</hw> <pr>(? <or/ ?)</pr>, <pos>n.</pos> <ety>[L. <ets>samara</ets>, <ets>samera</ets>, the seed of the elm.]</ety> <fld>(Bot.)</fld> <def>A dry, indehiscent, usually one-seeded, winged fruit, as that of the ash, maple, and elm; a key or key fruit.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Samare</ent><br/
<hw>Sam"are</hw> <pr>(?)</pr>, <pos>n.</pos> <def>See <er>Simar</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Samaritan</ent><br/
<hw>Sa*mar"i*tan</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>Samaritanus</ets>.]</ety> <def>Of or pertaining to Samaria, in Palestine.</def> -- <def2><pos>n.</pos> <def>A native or inhabitant of Samaria; also, the language of Samaria.</def></def2><br/
[<source>1913 Webster</source>]</p>

<p><ent>Samarium</ent><br/
<hw>Sa*ma"ri*um</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[NL., fr. E. <ets>samar</ets>skite.]</ety> <fld>(Chem.)</fld> <def>A rare metallic element of doubtful identity.</def><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ Samarium was discovered, by means of spectrum analysis, in certain minerals (<xex>samarskite</xex>, <xex>cerite</xex>, etc.), in which it is associated with other elements of the earthy group. It has been confounded with the doubtful elements <xex>decipium</xex>, <xex>philippium</xex>, etc., and is possibly a complex mixture of elements not as yet clearly identified. Symbol Sm. Provisional atomic weight 150.2.</note> <-- a true element. Symb. Sa or Sm; At. No. 62; At. wt. 150.43. Valence 2 or 3. --><br/
[<source>1913 Webster</source>]</p>

<p><ent>Samaroid</ent><br/
<hw>Sam"a*roid</hw> <pr>(?; 277)</pr>, <pos>a.</pos> <ety>[<ets>Samara</ets> + <ets>-oid</ets>.]</ety> <fld>(Bot.)</fld> <def>Resembling a samara, or winged seed vessel.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Samarra</ent><br/
<hw>Sa*mar"ra</hw> <pr>(?)</pr>, <pos>n.</pos> <def>See <er>Simar</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Samarskite</ent><br/
<hw>Sa*mar"skite</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[After <ets>Samarski</ets>, a Russian.]</ety> <fld>(Min.)</fld> <def>A rare mineral having a velvet-black color and submetallic luster. It is a niobate of uranium, iron, and the yttrium and cerium metals.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sambo</ent><br/
<hw>Sam"bo</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Sp. <ets>zambo</ets> bandy-legged, the child of a negro and an Indian; prob. of African origin.]</ety> <sn>1.</sn> <def>A negro; sometimes, the offspring of a black person and a mulatto; -- formerly used colloquially or with humorous intent, but now considered offensive or racist by African-Americans.</def> <mark>[Derogatory and offensive]</mark> <note>A children's book named <it>Little Black Sambo</it> was at one time popular in elementary schools, but the book has been removed from reading lists in many American schools due to the development of a reduced acceptance of patronizing depictions of negros, as well as the negative associations of the word itself.</note><br/
[<source>Webster 1913 Suppl.</source> <source>+PJC</source>]</p>

<p><sn>2.</sn>  <def>In Central America, an Indian and negro half-breed, or mixed blood.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Sambo</ent><br/
<hw>Sam"bo</hw>, <pos>n.</pos> <ety>[Sp. <ets>zambo</ets>, <ets>sambo</ets>.]</ety> <def>A colloquial or humorous appellation for a negro; sometimes, the offspring of a black person and a mulatto; a zambo.</def> <-- deprecatory and impolite --><br/
[<source>1913 Webster</source>]</p>

<p><ent>Samboo</ent><br/
<hw>Sam"boo</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Zool.)</fld> <def>Same as <er>Sambur</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sambucus</ent><br/
||<hw>Sam*bu"cus</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L., an elder tree.]</ety> <fld>(Bot.)</fld> <def>A genus of shrubs and trees; the elder.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sambuke</ent><br/
<hw>Sam"buke</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>sambuca</ets>, Gr. <?/.]</ety> <fld>(Mus.)</fld> <def>An ancient stringed instrument used by the Greeks, the particular construction of which is unknown.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sambur</ent><br/
<hw>Sam"bur</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Hind. <ets>s<amac/mbar</ets>, <ets>s<amac/bar</ets>.]</ety> <fld>(Zool.)</fld> <def>An East Indian deer (<spn>Rusa Aristotelis</spn>) having a mane on its neck. Its antlers have but three prongs. Called also <altname>gerow</altname>. The name is applied to other species of the genus <gen>Rusa</gen>, as the Bornean sambur (<spn>Rusa equina</spn>).</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Same</ent><br/
<hw>Same</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[AS. <ets>same</ets>, adv.; akin to OS. <ets>sama</ets>, <ets>samo</ets>, adv., OHG. <ets>sam</ets>, a., <ets>sama</ets>, adv., Icel. <ets>samr</ets>, a., Sw. <ets>samme</ets>, <ets>samma</ets>, Dan. <ets>samme</ets>, Goth. <ets>sama</ets>, Russ. <ets>samuii</ets>, Gr. <?/, Skr. <ets>sama</ets>, Gr. <?/ like, L. <ets>simul</ets> at the same time, <ets>similis</ets> like, and E. <ets>some</ets>, a., <ets>-some</ets>. <root/191. Cf. <er>Anomalous</er>, <er>Assemble</er>, <er>Homeopathy</er>, <er>Homily</er>, <er>Seem</er>, <pos>v. i.</pos>, <er>Semi-</er>, <er>Similar</er>, <er>Some</er>.]</ety> <sn>1.</sn> <def>Not different or other; not another or others; identical; unchanged.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Thou art the <qex>same</qex>, and thy years shall have no end.</q> <rj><qau>Ps. cii. 27.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Of like kind, species, sort, dimensions, or the like; not differing in character or in the quality or qualities compared; corresponding; not discordant; similar; like.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The ethereal vigor is in all the <qex>same</qex>.</q> <rj><qau>Dryden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>Just mentioned, or just about to be mentioned.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>What ye know, the <qex>same</qex> do I know.</q> <rj><qau>Job. xiii. 2.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Do but think how well the <qex>same</qex> he spends,<br/
Who spends his blood his country to relieve.</q> <rj><qau>Daniel.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ <xex>Same</xex> is commonly preceded by <xex>the</xex>, <xex>this</xex>, or <xex>that</xex> and is often used substantively as in the citations above. In a comparative use it is followed by <xex>as</xex> or <xex>with</xex>.</note><br/
[<source>1913 Webster</source>]</p>

<p><q>Bees like the <qex>same</qex> odors as we do.</q> <rj><qau>Lubbock.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>[He] held the <qex>same</qex> political opinions with his illustrious friend.</q> <rj><qau>Macaulay.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sameliness</ent><br/
<hw>Same"li*ness</hw> <pr>(?)</pr>, <pos>n.</pos> <def>Sameness, 2.</def> <mark>[R.]</mark> <rj><au>Bayne.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sameness</ent><br/
<hw>Same"ness</hw>, <pos>n.</pos> <sn>1.</sn> <def>The state of being the same; identity; absence of difference; near resemblance; correspondence; similarity; <as>as, a <ex>sameness</ex> of person, of manner, of sound, of appearance, and the like</as>.</def> <ldquo/A <xex>sameness</xex> of the terms.<rdquo/ <rj><au>Bp. Horsley.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Hence, want of variety; tedious monotony.</def><br/
[<source>1913 Webster</source>]</p>

<p><syn><b>Syn.</b> -- Identity; identicalness; oneness.</syn><br/
[<source>1913 Webster</source>]</p>

<p><ent>Samette</ent><br/
<hw>Sa*mette"</hw> <pr>(?)</pr>, <pos>n.</pos> <def>See <er>Samite</er>.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Samian</ent><br/
<hw>Sa"mi*an</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>Samius</ets>.]</ety> <def>Of or pertaining to the island of Samos.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Fill high the cup with <qex>Samian</qex> wine.</q> <rj><qau>Byron.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Samian earth</b></col>, <cd>a species of clay from Samos, formerly used in medicine as an astringent.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Samian</ent><br/
<hw>Sa"mi*an</hw>, <pos>n.</pos> <def>A native or inhabitant of Samos.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Samiel</ent><br/
<hw>Sa"mi*el</hw> <pr>(?; 277)</pr>, <pos>n.</pos> <ety>[Turk. <ets>sam-yeli</ets>; Ar. <ets>samm</ets> poison + Turk. <ets>yel</ets> wind. Cf. <er>Simoom</er>.]</ety> <def>A hot and destructive wind that sometimes blows, in Turkey, from the desert. It is identical with the <altname>simoom</altname> of Arabia and the <altname>kamsin</altname> of Syria.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Samiot</ent><br/
<hw>Sa"mi*ot</hw> <pr>(?)</pr>, <pos>a. & n.</pos> <ety>[Cf. F. <ets>samiote</ets>.]</ety> <def>Samian.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Samisen</ent><br/
||<hw>Sam"i*sen</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Jap.]</ety> <fld>(Mus.)</fld> <def>A Japanese musical instrument with three strings, resembling a guitar or banjo.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><-- p. 1273 pr=PCP --></p>

<p><ent>Samite</ent><br/
<hw>Sa"mite</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[OF. <ets>samit</ets>, LL. <ets>samitum</ets>, <ets>examitum</ets>, from LGr. <?/, <?/ woven with six threads; Gr. <?/ six + <?/ a thread. See <er>Six</er>, and cf. <er>Dimity</er>.]</ety> <def>A species of silk stuff, or taffeta, generally interwoven with gold.</def> <rj><au>Tennyson.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>In silken <qex>samite</qex> she was light arrayed.</q> <rj><qau>Spenser.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Samlet</ent><br/
<hw>Sam"let</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Cf. <er>Salmonet</er>.]</ety> <def>The parr.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sammier</ent><br/
<hw>Sam"mi*er</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A machine for pressing the water from skins in tanning.</def> <rj><au>Knight.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Samoan</ent><br/
<hw>Sa*mo"an</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Of or pertaining to the Samoan Islands (formerly called Navigators' Islands) in the South Pacific Ocean, or their inhabitants.</def> -- <def2><pos>n.</pos> <def>An inhabitant of the Samoan Islands.</def></def2><br/
[<source>1913 Webster</source>]</p>

<p><ent>Samovar</ent><br/
||<hw>Sa"mo*var</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Russ. <ets>samovar'</ets>.]</ety> <def>A metal urn used in Russia for making tea. It is filled with water, which is heated by charcoal placed in a pipe, with chimney attached, which passes through the urn.</def><br/
[<source>1913 Webster</source>]</p>

<p><-- Samoyed. 1. A breed of medium-sized sled dogs, originating in Siberia, of white or cream color. 2. A Uralic language spoken by the Samoyed people. --><br/
[<source>1913 Webster</source>]</p>

<p><ent>Samoyedes</ent><br/
<hw>Sam`oy*edes"</hw> <pr>(?)</pr>, <pos>n. pl.</pos>; <sing>sing. <singw>Samoyede</singw> <pr>(<?/)</pr></sing>. <fld>(Ethnol.)</fld> <def>An ignorant and degraded Turanian tribe which occupies a portion of Northern Russia and a part of Siberia.</def><-- also <altname>Samoyeds.</altname> --><br/
[<source>1913 Webster</source>]</p>

<p><ent>Samp</ent><br/
<hw>Samp</hw> <pr>(s<acr/mp)</pr>, <pos>n.</pos> <ety>[Massachusetts Indian <ets>nas<agrave/ump</ets> unparched meal porridge.]</ety> <def>An article of food consisting of maize broken or bruised, which is cooked by boiling, and usually eaten with milk; coarse hominy.</def> <mark>[U. S.]</mark><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Sampan</ent><br/
||<hw>Sam"pan</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Naut.)</fld> <def>A Chinese boat from twelve to fifteen feet long, covered with a house, and sometimes used as a permanent habitation on the inland waters.</def> <altsp>[Written also <asp>sanpan</asp>.]</altsp><br/
[<source>1913 Webster</source>]</p>

<p><ent>Samphire</ent><br/
<hw>Sam"phire</hw> <pr>(? <or/ ?; 277)</pr>, <pos>n.</pos> <ety>[F. l'herbe de <ets>Saint Pierre</ets>. See <er>Saint</er>, and <er>Petrel</er>.]</ety> <fld>(Bot.)</fld> <sd>(a)</sd> <def>A fleshy, suffrutescent, umbelliferous European plant (<spn>Crithmum maritimum</spn>). It grows among rocks and on cliffs along the seacoast, and is used for pickles.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Hangs one that gathers <qex>samphire</qex>, dreadful trade!</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sd>(b)</sd> <def>The species of glasswort (<spn>Salicornia herbacea</spn>); -- called in England <altname>marsh samphire</altname>.</def> <sd>(c)</sd> <def>A seashore shrub (<spn>Borrichia arborescens</spn>) of the West Indies.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Golden samphire</b></col>. <cd>See under <er>Golden</er>.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sample</ent><br/
<hw>Sam"ple</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[OE. <ets>sample</ets>, <ets>asaumple</ets>, OF. <ets>essample</ets>, <ets>example</ets>, fr. L. <ets>exemplum</ets>. See <er>Example</er>, and cf. <er>Ensample</er>, <er>Sampler</er>.]</ety> <sn>1.</sn> <def>Example; pattern.</def> <mark>[Obs.]</mark> <au>Spenser.</au> <ldquo/A <xex>sample</xex> to the youngest.<rdquo/ <au>Shak.</au><br/
[<source>1913 Webster</source>]</p>

<p><q>Thus he concludes, and every hardy knight<br/
His <qex>sample</qex> followed.</q> <rj><qau>Fairfax.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A part of anything presented for inspection, or shown as evidence of the quality of the whole; a specimen; <as>as, goods are often purchased by <ex>samples</ex></as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>I design this but for a <qex>sample</qex> of what I hope more fully to discuss.</q> <rj><qau>Woodward.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><syn><b>Syn.</b> -- Specimen; example. See <er>Specimen</er>.</syn><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sample</ent><br/
<hw>Sam"ple</hw>, <pos>v. t.</pos> <sn>1.</sn> <def>To make or show something similar to; to match.</def> <rj><au>Bp. Hall.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To take or to test a sample or samples of; <as>as, to <ex>sample</ex> sugar, teas, wools, cloths</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sampler</ent><br/
<hw>Sam"pler</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[See <er>Exampler</er>, <er>Exemplar</er>.]</ety> <sn>1.</sn> <def>One who makes up samples for inspection; one who examines samples, or by samples; <as>as, a wool <ex>sampler</ex></as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A pattern; a specimen; especially, a collection of needlework patterns, as letters, borders, etc., to be used as samples, or to display the skill of the worker.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Susie dear, bring your <qex>sampler</qex> and Mrs. Schumann will show you how to make that W you bothered over.</q> <rj><qau>E. E. Hale.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Samshu</ent><br/
<ent>Samshoo</ent><br/
<mhw>||<hw>Sam"shoo</hw>, <hw>Sam"shu</hw> <pr>(<?/)</pr></mhw>, <pos>n.</pos> <ety>[Chinese <ets>san-shao</ets> thrice fired.]</ety> <def>A spirituous liquor distilled by the Chinese from the yeasty liquor in which boiled rice has fermented under pressure.</def> <rj><au>S. W. Williams.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Samson</ent><br/
<hw>Sam"son</hw> <pr>(?)</pr>, <pos>n.</pos> <def>An Israelite of Bible record (see <au>Judges xiii.</au>), distinguished for his great strength; hence, a man of extraordinary physical strength.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Samson post</b></col>. <sd>(a)</sd> <fld>(Naut.)</fld> <cd>A strong post resting on the keelson, and supporting a beam of the deck; also, a temporary or movable pillar carrying a leading block or pulley for various purposes.</cd> <au>Brande & C.</au> <sd>(b)</sd> <cd>In deep-well boring, the post which supports the walking beam of the apparatus.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Samurai</ent><br/
||<hw>Sa"mu*rai`</hw> <pr>(?)</pr>, <pos>n. pl. & sing.</pos> <ety>[Jap.]</ety> <def>In the former feudal system of Japan, the class or a member of the class, of military retainers of the daimios, constituting the gentry or lesser nobility. They possessed power of life and death over the commoners, and wore two swords as their distinguishing mark. Their special rights and privileges were abolished with the fall of feudalism in 1871.  They were referred to as <ldquo/a cross between a knight and a gentleman<rdquo/.</def><br/
[<source>Webster 1913 Suppl.</source> <source>+PJC</source>]</p>

<p><ent>Sanability</ent><br/
<hw>San`a*bil"i*ty</hw> <pr>(?)</pr>, <pos>n.</pos> <def>The quality or state of being sanable; sanableness; curableness.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sanable</ent><br/
<hw>San"a*ble</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>sanabilis</ets>, fr. <ets>sanare</ets> to heal, fr. <ets>sanus</ets> sound, healthy. See <er>Sane</er>.]</ety> <def>Capable of being healed or cured; susceptible of remedy.</def><br/
[<source>1913 Webster</source>]</p>

<p><syn><b>Syn.</b> -- Remediable; curable; healable.</syn><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sanableness</ent><br/
<hw>San"a*ble*ness</hw>, <pos>n.</pos> <def>The quality of being sanable.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sanation</ent><br/
<hw>Sa*na"tion</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>sanatio</ets>. See <er>Sanable</er>.]</ety> <def>The act of healing or curing.</def> <mark>[Obs.]</mark> <rj><au>Wiseman.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sanative</ent><br/
<hw>San"a*tive</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[LL. <ets>sanativus</ets>.]</ety> <def>Having the power to cure or heal; healing; tending to heal; sanatory.</def> -- <wordforms><wf>San"a*tive*ness</wf>, <pos>n.</pos></wordforms><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sanatorium</ent><br/
<hw>San`a*to"ri*um</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[NL. See <er>Sanatory</er>.]</ety> <def>An establishment for the treatment of the sick; a resort for invalids. See <er>Sanitarium</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sanatory</ent><br/
<hw>San"a*to*ry</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[LL. <ets>sanatorius</ets>, fr. L. <ets>sanare</ets> to heal. See <er>Sanable</er>.]</ety> <def>Conducive to health; tending to cure; healing; curative; sanative.</def><br/
[<source>1913 Webster</source>]</p>

<p><q><qex>Sanatory</qex> ordinances for the protection of public health, such as quarantine, fever hospitals, draining, etc.</q> <rj><qau>De Quincey.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ <xex>Sanatory</xex> and <xex>sanitary</xex> should not be confounded. <xex>Sanatory</xex> signifies <xex>conducive to health</xex>, while <xex>sanitary</xex> has the more general meaning of <xex>pertaining to health</xex>.</note><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sanbenito</ent><br/
||<hw>San`be*ni"to</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Sp. & Pg. <ets>sambenito</ets>, contr. from L. <ets>saccus</ets> sack + <ets>benedictus</ets> blessed.]</ety> <sn>1.</sn> <def>Anciently, a sackcloth coat worn by penitents on being reconciled to the church.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A garnment or cap, or sometimes both, painted with flames, figures, etc., and worn by persons who had been examined by the Inquisition and were brought forth for punishment at the auto-da-f<eacute/.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sancte bell</ent><br/
<ent>Sance-bell</ent><br/
<mhw>{ <hw>Sance"-bell`</hw> <pr>(?)</pr>, <hw>Sanc"te bell`</hw> <pr>(?)</pr>, }</mhw> <pos>n.</pos> <def>See <cref>Sanctus bell</cref>, under <er>Sanctus</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sancho</ent><br/
<hw>San"cho</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Sp., a proper name.]</ety> <fld>(Card Playing)</fld> <def>The nine of trumps in sancho pedro.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Sancho pedro</ent><br/
<hw>Sancho pedro</hw>. <ety>[Sp. <ets>Pedro</ets> Peter.]</ety> <fld>(Card Playing)</fld> <def>A variety of auction pitch in which the nine (<xex>sancho</xex>) and five (<xex>pedro</xex>) of trumps are added as counting cards at their pip value, and the ten of trumps counts game.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Sanctificate</ent><br/
<hw>Sanc"ti*fi*cate</hw> <pr>(?)</pr>, <pos>v. t.</pos> <ety>[L. <ets>sanctificatus</ets>, p. p. of <ets>sanctificare</ets>.]</ety> <def>To sanctify.</def> <mark>[Obs.]</mark> <rj><au>Barrow.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sanctification</ent><br/
<hw>Sanc`ti*fi*ca"tion</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>sanctificatio</ets>: cf. F. <ets>sanctification</ets>.]</ety> <sn>1.</sn> <def>The act of sanctifying or making holy; the state of being sanctified or made holy;</def> <specif>esp.</specif> <fld>(Theol.)</fld>, <def>the act of God's grace by which the affections of men are purified, or alienated from sin and the world, and exalted to a supreme love to God; also, the state of being thus purified or sanctified.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>God hath from the beginning chosen you to salvation through <qex>sanctification</qex> of the Spirit and belief of the truth.</q> <rj><qau>2 Thess. ii. 13.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>The act of consecrating, or of setting apart for a sacred purpose; consecration.</def> <rj><au>Bp. Burnet.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sanctified</ent><br/
<hw>Sanc"ti*fied</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Made holy; also, made to have the air of sanctity; sanctimonious.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sanctifier</ent><br/
<hw>Sanc"ti*fi`er</hw> <pr>(?)</pr>, <pos>n.</pos> <def>One who sanctifies, or makes holy; specifically, the Holy Spirit.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sanctify</ent><br/
<hw>Sanc"ti*fy</hw> <pr>(?)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Sanctified</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Sanctifying</conjf> <pr>(?)</pr>.]</vmorph> <ety>[F. <ets>sanctifier</ets>, L. <ets>sanctificare</ets>; <ets>sanctus</ets> holy + <ets>-ficare</ets> (in comp.) to make. See <er>Saint</er>, and <er>-fy</er>.]</ety> <sn>1.</sn> <def>To make sacred or holy; to set apart to a holy or religious use; to consecrate by appropriate rites; to hallow.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>God blessed the seventh day and <qex>sanctified</qex> it.</q> <rj><qau>Gen. ii. 3.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Moses . . . <qex>sanctified</qex> Aaron and his garments.</q> <rj><qau>Lev. viii. 30.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To make free from sin; to cleanse from moral corruption and pollution; to purify.</def><br/
[<source>1913 Webster</source>]</p>

<p><q><qex>Sanctify</qex> them through thy truth.</q> <rj><qau>John xvii. 17.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>To make efficient as the means of holiness; to render productive of holiness or piety.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>A means which his mercy hath <qex>sanctified</qex> so to me as to make me repent of that unjust act.</q> <rj><qau>Eikon Basilike.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>To impart or impute sacredness, venerableness, inviolability, title to reverence and respect, or the like, to; to secure from violation; to give sanction to.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The holy man, amazed at what he saw,<br/
Made haste to <qex>sanctify</qex> the bliss by law.</q> <rj><qau>Dryden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Truth guards the poet, <qex>sanctifies</qex> the line.</q> <rj><qau>Pope.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sanctifyingly</ent><br/
<hw>Sanc"ti*fy`ing*ly</hw> <pr>(?)</pr>, <pos>adv.</pos> <def>In a manner or degree tending to sanctify or make holy.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sanctiloquent</ent><br/
<hw>Sanc*til"o*quent</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>sanctus</ets> holy + <ets>loquens</ets>, p. pr. of <ets>loqui</ets> to speak.]</ety> <def>Discoursing on heavenly or holy things, or in a holy manner.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sanctimonial</ent><br/
<hw>Sanc`ti*mo"ni*al</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[Cf. LL. <ets>sanctimonialis</ets>. ]</ety> <def>Sanctimonious.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sanctimonious</ent><br/
<hw>Sanc`ti*mo"ni*ous</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[See <er>Sanctimony</er>.]</ety> <sn>1.</sn> <def>Possessing sanctimony; holy; sacred; saintly.</def> <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Making a show of sanctity; affecting saintliness; hypocritically devout or pious.</def> <ldquo/Like the <xex>sanctimonious</xex> pirate.<rdquo/ <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p>-- <wordforms><wf>Sanc`ti*mo"ni*ous*ly</wf>, <pos>adv.</pos> -- <wf>Sanc`ti*mo"ni*ous*ness</wf>, <pos>n.</pos></wordforms><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sanctimony</ent><br/
<hw>Sanc"ti*mo*ny</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>sanctimonia</ets>, fr. <ets>sanctus</ets> holy: cf. OF. <ets>sanctimonie</ets>. See <er>Saint</er>.]</ety> <def>Holiness; devoutness; scrupulous austerity; sanctity; especially, outward or artificial saintliness; assumed or pretended holiness; hypocritical devoutness.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Her pretense is a pilgrimage; . . . which holy undertaking with most austere <qex>sanctimony</qex> she accomplished.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sanction</ent><br/
<hw>Sanc"tion</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>sanctio</ets>, from <ets>sancire</ets>, <ets>sanctum</ets> to render sacred or inviolable, to fix unalterably: cf. F. <ets>sanction</ets>. See <er>Saint</er>.]</ety> <sn>1.</sn> <def>Solemn or ceremonious ratification; an official act of a superior by which he ratifies and gives validity to the act of some other person or body; establishment or furtherance of anything by giving authority to it; confirmation; approbation.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The strictest professors of reason have added the <qex>sanction</qex> of their testimony.</q> <rj><qau>I. Watts.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Anything done or said to enforce the will, law, or authority of another; <as>as, legal <ex>sanctions</ex></as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><syn><b>Syn.</b> -- Ratification; authorization; authority; countenance; support.</syn><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sanction</ent><br/
<hw>Sanc"tion</hw>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Sanctioned</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Sanctioning</conjf>.]</vmorph> <def>To give sanction to; to ratify; to confirm; to approve.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Would have counseled, or even <qex>sanctioned</qex>, such perilous experiments.</q> <rj><qau>De Quincey.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><syn><b>Syn.</b> -- To ratify; confirm; authorize; countenance.</syn><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sanctionary</ent><br/
<hw>Sanc"tion*a*ry</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Of, pertaining to, or giving, sanction.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sanctitude</ent><br/
<hw>Sanc"ti*tude</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>sanctitudo</ets>.]</ety> <def>Holiness; sacredness; sanctity.</def> <mark>[R.]</mark> <rj><au>Milton.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sanctity</ent><br/
<hw>Sanc"ti*ty</hw> <pr>(?)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Sanctities</plw> <pr>(#)</pr>.</plu> <ety>[L. <ets>sanctitas</ets>, from <ets>sanctus</ets> holy. See <er>Saint</er>.]</ety> <sn>1.</sn> <def>The state or quality of being sacred or holy; holiness; saintliness; moral purity; godliness.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>To <qex>sanctity</qex> she made no pretense, and, indeed, narrowly escaped the imputation of irreligion.</q> <rj><qau>Macaulay.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Sacredness; solemnity; inviolability; religious binding force; <as>as, the <ex>sanctity</ex> of an oath</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>A saint or holy being.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>About him all the <qex>sanctities</qex> of heaven.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><syn><b>Syn.</b> -- Holiness; godliness; piety; devotion; goodness; purity; religiousness; sacredness; solemnity. See the Note under <er>Religion</er>.</syn><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sanctuarize</ent><br/
<hw>Sanc"tu*a*rize</hw> <pr>(?)</pr>, <pos>v. t.</pos> <def>To shelter by means of a sanctuary or sacred privileges.</def> <mark>[Obs.]</mark> <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sanctuary</ent><br/
<hw>Sanc"tu*a*ry</hw> <pr>(?)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Sanctuaries</plw> <pr>(#)</pr>.</plu> <ety>[OE. <ets>seintuarie</ets>, OF. <ets>saintuaire</ets>, F. <ets>sanctuaire</ets>, fr. L. <ets>sanctuarium</ets>, from <ets>sanctus</ets> sacred, holy. See <er>Saint</er>.]</ety> <def>A sacred place; a consecrated spot; a holy and inviolable site.</def> Hence, specifically: <sd>(a)</sd> <def>The most retired part of the temple at Jerusalem, called the <xex>Holy of Holies</xex>, in which was kept the ark of the covenant, and into which no person was permitted to enter except the high priest, and he only once a year, to intercede for the people; also, the most sacred part of the tabernacle; also, the temple at Jerusalem.</def> <sd>(b)</sd> <fld>(Arch.)</fld> <def>The most sacred part of any religious building, esp. that part of a Christian church in which the altar is placed.</def> <sd>(c)</sd> <def>A house consecrated to the worship of God; a place where divine service is performed; a church, temple, or other place of worship.</def> <sd>(d)</sd> <def>A sacred and inviolable asylum; a place of refuge and protection; shelter; refuge; protection.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>These laws, whoever made them, bestowed on temples the privilege of <qex>sanctuary.</qex></q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>The admirable works of painting were made fuel for the fire; but some relics of it took <qex>sanctuary</qex> under ground, and escaped the common destiny.</q> <rj><qau>Dryden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><-- <col><b>Wildlife sanctuary</b></col>, <cd>a tract of land set aside by law for the preservation of wildlife, in which no hunting is permitted.</cd> --><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sanctum</ent><br/
<hw>Sanc"tum</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L., p. p. of <ets>sancire</ets> to consecrate.]</ety> <def>A sacred place; hence, a place of retreat; a room reserved for personal use; <as>as, an editor's <ex>sanctum</ex></as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs>||<col><b>Sanctum sanctorum</b></col> <ety>[L.]</ety>, <cd>the Holy of Holies; the most holy place, as in the Jewish temple.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sanctus</ent><br/
<hw>Sanc"tus</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>sanctus</ets>, p. p. of <ets>sancire</ets>.]</ety> <sn>1.</sn> <fld>(Eccl.)</fld> <def>A part of the Mass, or, in Protestant churches, a part of the communion service, of which the first words in Latin are <xex>Sanctus, sanctus, sanctus</xex> [Holy, holy, holy]; -- called also <altname>Tersanctus</altname>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Mus.)</fld> <def>An anthem composed for these words.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Sanctus bell</b></col>, <cd>a small bell usually suspended in a bell cot at the apex of the nave roof, over the chancel arch, in mediaeval churches, but a hand bell is now often used; -- so called because rung at the singing of the <xex>sanctus</xex>, at the conclusion of the ordinary of the Mass, and again at the elevation of the host. Called also <altname>Mass bell</altname>, <altname>sacring bell</altname>, <altname>saints' bell</altname>, <altname>sance-bell</altname>, <altname>sancte bell</altname>.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sand</ent><br/
<hw>Sand</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[AS. <ets>sand</ets>; akin to D. <ets>zand</ets>, G. <ets>sand</ets>, OHG. <ets>sant</ets>, Icel. <ets>sandr</ets>, Dan. & Sw. <ets>sand</ets>, Gr. <?/.]</ety> <sn>1.</sn> <def>Fine particles of stone, esp. of siliceous stone, but not reduced to dust; comminuted stone in the form of loose grains, which are not coherent when wet.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>That finer matter, called <qex>sand</qex>, is no other than very small pebbles.</q> <rj><qau>Woodward.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A single particle of such stone.</def> <mark>[R.]</mark> <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>The sand in the hourglass; hence, a moment or interval of time; the term or extent of one's life.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The <qex>sands</qex> are numbered that make up my life.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <pluf>pl.</pluf> <def>Tracts of land consisting of sand, like the deserts of Arabia and Africa; also, extensive tracts of sand exposed by the ebb of the tide.</def> <ldquo/The Libyan <xex>sands</xex>.<rdquo/ <au>Milton.</au> <ldquo/The <xex>sands</xex> o' Dee.<rdquo/ <au>C. Kingsley.</au><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <def>Courage; pluck; grit.</def> <mark>[Slang]</mark><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Sand badger</b></col> <fld>(Zool.)</fld>, <cd>the Japanese badger (<spn>Meles ankuma</spn>).</cd> -- <col><b>Sand bag</b></col>. <sd>(a)</sd> <cd>A bag filled with sand or earth, used for various purposes, as in fortification, for ballast, etc.</cd> <sd>(b)</sd> <cd>A long bag filled with sand, used as a club by assassins.</cd> -- <col><b>Sand ball</b></col>, <cd>soap mixed with sand, made into a ball for use at the toilet.</cd> -- <col><b>Sand bath</b></col>. <sd>(a)</sd> <fld>(Chem.)</fld> <cd>A vessel of hot sand in a laboratory, in which vessels that are to be heated are partially immersed</cd>. <sd>(b)</sd> <cd>A bath in which the body is immersed in hot sand.</cd> -- <col><b>Sand bed</b></col>, <cd>a thick layer of sand, whether deposited naturally or artificially; specifically, a thick layer of sand into which molten metal is run in casting, or from a reducing furnace.</cd> -- <col><b>Sand birds</b></col> <fld>(Zool.)</fld>, <cd>a collective name for numerous species of limicoline birds, such as the sandpipers, plovers, tattlers, and many others; -- called also <altname>shore birds</altname>.</cd> -- <col><b>Sand blast</b></col>, <cd>a process of engraving and cutting glass and other hard substances by driving sand against them by a steam jet or otherwise; also, the apparatus used in the process.</cd> -- <col><b>Sand box</b></col>. <sd>(a)</sd> <cd>A box with a perforated top or cover, for sprinkling paper with sand</cd>. <sd>(b)</sd> <cd>A box carried on locomotives, from which sand runs on the rails in front of the driving wheel, to prevent slipping.</cd> -- <col><b>Sand-box tree</b></col> <fld>(Bot.)</fld>, <cd>a tropical American tree (<spn>Hura crepitans</spn>). Its fruit is a depressed many-celled woody capsule which, when completely dry, bursts with a loud report and scatters the seeds. See <xex>Illust.</xex> of <er>Regma</er>.</cd> -- <col><b>Sand bug</b></col> <fld>(Zool.)</fld>, <cd>an American anomuran crustacean (<spn>Hippa talpoidea</spn>) which burrows in sandy seabeaches. It is often used as bait by fishermen. See <xex>Illust.</xex> under <er>Anomura</er>.</cd> -- <col><b>Sand canal</b></col> <fld>(Zool.)</fld>, <cd>a tubular vessel having a calcareous coating, and connecting the oral ambulacral ring with the madreporic tubercle. It appears to be excretory in function.</cd> -- <col><b>Sand cock</b></col> <fld>(Zool.)</fld>, <cd>the redshank.</cd> <mark>[Prov. Eng.]</mark> -- <col><b>Sand collar</b></col>. <fld>(Zool.)</fld> <cd>Same as <cref>Sand saucer</cref>, below.</cd> -- <col><b>Sand crab</b></col>. <fld>(Zool.)</fld> <sd>(a)</sd> <cd>The lady crab.</cd> <sd>(b)</sd> <cd>A land crab, or ocypodian.</cd> -- <col><b>Sand crack</b></col> <fld>(Far.)</fld>, <cd>a crack extending downward from the coronet, in the wall of a horse's hoof, which often causes lameness.</cd> -- <col><b>Sand cricket</b></col> <fld>(Zool.)</fld>, <cd>any one of several species of large terrestrial crickets of the genus <gen>Stenophelmatus</gen> and allied genera, native of the sandy plains of the Western United States.</cd> -- <col><b>Sand cusk</b></col> <fld>(Zool.)</fld>, <cd>any ophidioid fish. See <er>Illust.</er> under <er>Ophidioid</er>.</cd> -- <col><b>Sand dab</b></col> <fld>(Zool.)</fld>, <cd>a small American flounder (<spn>Limanda ferruginea</spn>); -- called also <altname>rusty dab</altname>. The name is also applied locally to other allied species.</cd> -- <col><b>Sand darter</b></col> <fld>(Zool.)</fld>, <cd>a small etheostomoid fish of the Ohio valley (<spn>Ammocrypta pellucida</spn>).</cd> -- <col><b>Sand dollar</b></col> <fld>(Zool.)</fld>, <cd>any one of several species of small flat circular sea urchins, which live on sandy bottoms, especially <spn>Echinarachnius parma</spn> of the American coast.</cd> -- <col><b>Sand drift</b></col>, <cd>drifting sand; also, a mound or bank of drifted sand.</cd> -- <col><b>Sand eel</b></col>. <fld>(Zool.)</fld> <sd>(a)</sd> <cd>A lant, or launce</cd>. <sd>(b)</sd> <cd>A slender Pacific Ocean fish of the genus <gen>Gonorhynchus</gen>, having barbels about the mouth.</cd> -- <col><b>Sand flag</b></col>, <cd>sandstone which splits up into flagstones.</cd> -- <col><b>Sand flea</b></col>. <fld>(Zool.)</fld> <sd>(a)</sd> <cd>Any species of flea which inhabits, or breeds in, sandy places, especially the common dog flea.</cd> <sd>(b)</sd> <cd>The chigoe</cd>. <sd>(c)</sd> <cd>Any leaping amphipod crustacean; a beach flea, or orchestian. See <cref>Beach flea</cref>, under <er>Beach</er>.</cd> -- <col><b>Sand flood</b></col>, <cd>a vast body of sand borne along by the wind.</cd> <au>James Bruce.</au> -- <col><b>Sand fluke</b></col>. <fld>(Zool.)</fld> <sd>(a)</sd> <cd>The sandnecker</cd>. <sd>(b)</sd> <cd>The European smooth dab (<spn>Pleuronectes microcephalus</spn>); -- called also <altname>kitt</altname>, <altname>marysole</altname>, <altname>smear dab</altname>, <altname>town dab</altname>.</cd> -- <col><b>Sand fly</b></col> <fld>(Zool.)</fld>, <cd>any one of several species of small dipterous flies of the genus <gen>Simulium</gen>, abounding on sandy shores, especially <spn>Simulium nocivum</spn> of the United States. They are very troublesome on account of their biting habits. Called also <altname>no-see-um</altname>, <altname>punky</altname>, and <altname>midge</altname>.</cd> -- <col><b>Sand gall</b></col>. <fld>(Geol.)</fld> <cd>See <cref>Sand pipe</cref>, below.</cd> -- <col><b>Sand grass</b></col> <fld>(Bot.)</fld>, <cd>any species of grass which grows in sand; especially, a tufted grass (<spn>Triplasis purpurea</spn>) with numerous bearded joints, and acid awl-shaped leaves, growing on the Atlantic coast.</cd> <-- p. 1274 pr=PCP --> -- <col><b>Sand grouse</b></col> <fld>(Zool.)</fld>, <cd>any one of many species of Old World birds belonging to the suborder Pterocletes, and resembling both grouse and pigeons. Called also <altname>rock grouse</altname>, <altname>rock pigeon</altname>, and <altname>ganga</altname>. They mostly belong to the genus <gen>Pterocles</gen>, as the common Indian species (<spn>Pterocles exustus</spn>). The large sand grouse (<spn>Pterocles arenarius</spn>), the painted sand grouse (<spn>Pterocles fasciatus</spn>), and the pintail sand grouse (<spn>Pterocles alchata</spn>) are also found in India. See <xex>Illust.</xex> under <er>Pterocletes</er>.</cd> -- <col><b>Sand hill</b></col>, <cd>a hill of sand; a dune.</cd> -- <col><b>Sand-hill crane</b></col> <fld>(Zool.)</fld>, <cd>the American brown crane (<spn>Grus Mexicana</spn>).</cd> -- <col><b>Sand hopper</b></col> <fld>(Zool.)</fld>, <cd>a beach flea; an orchestian.</cd> -- <col><b>Sand hornet</b></col> <fld>(Zool.)</fld>, <cd>a sand wasp.</cd> -- <col><b>Sand lark</b></col>. <fld>(Zool.)</fld> <sd>(a)</sd> <cd>A small lark (<spn>Alaudala raytal</spn>), native of India</cd>. <sd>(b)</sd> <cd>A small sandpiper, or plover, as the ringneck, the sanderling, and the common European sandpiper</cd>. <sd>(c)</sd> <cd>The Australian red-capped dotterel (<spn>Aegialophilus ruficapillus</spn>); -- called also <altname>red-necked plover</altname>.</cd> -- <col><b>Sand launce</b></col> <fld>(Zool.)</fld>, <cd>a lant, or launce.</cd> -- <col><b>Sand lizard</b></col> <fld>(Zool.)</fld>, <cd>a common European lizard (<spn>Lacerta agilis</spn>).</cd> -- <col><b>Sand martin</b></col> <fld>(Zool.)</fld>, <cd>the bank swallow.</cd> -- <col><b>Sand mole</b></col> <fld>(Zool.)</fld>, <cd>the coast rat.</cd> -- <col><b>Sand monitor</b></col> <fld>(Zool.)</fld>, <cd>a large Egyptian lizard (<spn>Monitor arenarius</spn>) which inhabits dry localities.</cd> -- <col><b>Sand mouse</b></col> <fld>(Zool.)</fld>, <cd>the dunlin.</cd> <mark>[Prov. Eng.]</mark> -- <col><b>Sand myrtle</b></col>. <fld>(Bot.)</fld> <cd>See under <er>Myrtle</er>.</cd> -- <col><b>Sand partridge</b></col> <fld>(Zool.)</fld>, <cd>either of two small Asiatic partridges of the genus <gen>Ammoperdix</gen>. The wings are long and the tarsus is spurless. One species (<spn>Ammoperdix Heeji</spn>) inhabits Palestine and Arabia. The other species (<spn>Ammoperdix Bonhami</spn>), inhabiting Central Asia, is called also <altname>seesee partridge</altname>, and <altname>teehoo</altname>.</cd> -- <col><b>Sand picture</b></col>, <cd>a picture made by putting sand of different colors on an adhesive surface.</cd> -- <col><b>Sand pike</b></col>. <fld>(Zool.)</fld> <sd>(a)</sd> <cd>The sauger</cd>. <sd>(b)</sd> <cd>The lizard fish.</cd> -- <col><b>Sand pillar</b></col>, <cd>a sand storm which takes the form of a whirling pillar in its progress in desert tracts like those of the Sahara and Mongolia.</cd> -- <col><b>Sand pipe</b></col> <fld>(Geol.)</fld>, <cd>a tubular cavity, from a few inches to several feet in depth, occurring especially in calcareous rocks, and often filled with gravel, sand, etc.; -- called also <altname>sand gall</altname>.</cd> -- <col><b>Sand pride</b></col> <fld>(Zool.)</fld>, <cd>a small British lamprey now considered to be the young of larger species; -- called also <altname>sand prey</altname>.</cd> -- <col><b>Sand pump</b></col>, <cd>in artesian well boring, a long, slender bucket with a valve at the bottom for raising sand from the well.</cd> -- <col><b>Sand rat</b></col> <fld>(Zool.)</fld>, <cd>the pocket gopher.</cd> -- <col><b>Sand rock</b></col>, <cd>a rock made of cemented sand.</cd> -- <col><b>Sand runner</b></col> <fld>(Zool.)</fld>, <cd>the turnstone.</cd> -- <col><b>Sand saucer</b></col> <fld>(Zool.)</fld>, <cd>the mass of egg capsules, or oothecae, of any mollusk of the genus <gen>Natica</gen> and allied genera. It has the shape of a bottomless saucer, and is coated with fine sand; -- called also <altname>sand collar</altname>.</cd> -- <col><b>Sand screw</b></col> <fld>(Zool.)</fld>, <cd>an amphipod crustacean (<spn>Lepidactylis arenarius</spn>), which burrows in the sandy seabeaches of Europe and America.</cd> -- <col><b>Sand shark</b></col> <fld>(Zool.)</fld>, <cd>an American shark (<spn>Odontaspis littoralis</spn>) found on the sandy coasts of the Eastern United States; -- called also <altname>gray shark</altname>, and <altname>dogfish shark</altname>. See <xex>Illust.</xex> under <er>Remora</er>.</cd> -- <col><b>Sand skink</b></col> <fld>(Zool.)</fld>, <cd>any one of several species of Old World lizards belonging to the genus <gen>Seps</gen>; <as>as, the ocellated <ex>sand skink</ex> (<spn>Seps ocellatus</spn>) of Southern Europe</as>.</cd> -- <col><b>Sand skipper</b></col> <fld>(Zool.)</fld>, <cd>a beach flea, or orchestian.</cd> -- <col><b>Sand smelt</b></col> <fld>(Zool.)</fld>, <cd>a silverside.</cd> -- <col><b>Sand snake</b></col>. <fld>(Zool.)</fld> <sd>(a)</sd> <cd>Any one of several species of harmless burrowing snakes of the genus <gen>Eryx</gen>, native of Southern Europe, Africa, and Asia, especially <spn>Eryx jaculus</spn> of India and <spn>Eryx Johnii</spn>, used by snake charmers.</cd> <sd>(b)</sd> <cd>Any innocuous South African snake of the genus <gen>Psammophis</gen>, especially <spn>Psammophis sibilans</spn>.</cd> -- <col><b>Sand snipe</b></col> <fld>(Zool.)</fld>, <cd>the sandpiper.</cd> -- <col><b>Sand star</b></col> <fld>(Zool.)</fld>, <cd>an ophiurioid starfish living on sandy sea bottoms; a brittle star.</cd> -- <col><b>Sand storm</b></col>, <cd>a cloud of sand driven violently by the wind.</cd> -- <col><b>Sand sucker</b></col>, <cd>the sandnecker.</cd> -- <col><b>Sand swallow</b></col> <fld>(Zool.)</fld>, <cd>the bank swallow. See under <er>Bank</er>.</cd> -- <col><b>Sand trap</b></col>, <fld>(Golf)</fld> <cd>a shallow pit on a golf course having a layer of sand in it, usually located near a green, and designed to function as a hazard, due to the difficulty of hitting balls effectively from such a position.</cd> -- <col><b>Sand tube</b></col>, <cd>a tube made of sand.</cd> Especially: <sd>(a)</sd> <cd>A tube of vitrified sand, produced by a stroke of lightning; a fulgurite</cd>. <sd>(b)</sd> <fld>(Zool.)</fld> <cd>Any tube made of cemented sand.</cd> <sd>(c)</sd> <fld>(Zool.)</fld> <cd>In starfishes, a tube having calcareous particles in its wall, which connects the oral water tube with the madreporic plate.</cd> -- <col><b>Sand viper</b></col>. <fld>(Zool.)</fld> <cd>See <er>Hognose snake</er>.</cd> -- <col><b>Sand wasp</b></col> <fld>(Zool.)</fld>, <cd>any one of numerous species of hymenopterous insects belonging to the families <fam>Pompilidae</fam> and <fam>Spheridae</fam>, which dig burrows in sand. The female provisions the nest with insects or spiders which she paralyzes by stinging, and which serve as food for her young.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sand</ent><br/
<hw>Sand</hw> <pr>(?)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Sanded</conjf>; <pos>p. pr. & vb. n.</pos> <conjf>Sanding</conjf>.]</vmorph> <sn>1.</sn> <def>To sprinkle or cover with sand.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To drive upon the sand.</def> <mark>[Obs.]</mark> <rj><au>Burton.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>To bury (oysters) beneath drifting sand or mud.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>To mix with sand for purposes of fraud; <as>as, to <ex>sand</ex> sugar</as>.</def> <mark>[Colloq.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><-- 5. To grind down or make smooth by rubbing with an abrasive object, esp. with sandpaper; to sand down --> <br/
[<source>1913 Webster</source>]</p>

<p><ent>Sandal</ent><br/
<hw>San"dal</hw> <pr>(?)</pr>, <pos>n.</pos> <def>Same as <er>Sendal</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Sails of silk and ropes of <qex>sandal</qex>.</q> <rj><qau>Longfellow.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sandal</ent><br/
<hw>San"dal</hw>, <pos>n.</pos> <def>Sandalwood.</def> <ldquo/Fans of <xex>sandal</xex>.<rdquo/ <rj><au>Tennyson.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sandal</ent><br/
<hw>San"dal</hw>, <pos>n.</pos> <ety>[F. <ets>sandale</ets>, L. <ets>sandalium</ets>, Gr. <?/, dim. of <?/, probably from Per. <ets>sandal</ets>.]</ety> <sd>(a)</sd> <def>A kind of shoe consisting of a sole strapped to the foot; a protection for the foot, covering its lower surface, but not its upper.</def> <sd>(b)</sd> <def>A kind of slipper.</def> <sd>(c)</sd> <def>An overshoe with parallel openings across the instep.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sandaled</ent><br/
<hw>San"daled</hw> <pr>(?)</pr>, <pos>a.</pos> <sn>1.</sn> <def>Wearing sandals.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The measured footfalls of his <qex>sandaled</qex> feet.</q> <rj><qau>Longfellow.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Made like a sandal.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sandaliform</ent><br/
<hw>San*dal"i*form</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[<ets>Sandal</ets> + <ets>-form</ets>.]</ety> <fld>(Bot.)</fld> <def>Shaped like a sandal or slipper.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sandalwood</ent><br/
<hw>San"dal*wood</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F. <ets>sandal</ets>, <ets>santal</ets>, fr. Ar. <ets><cced/andal</ets>, or Gr. <grk>sa`ntalon</grk>; both ultimately fr. Skr. <ets>candana</ets>. Cf. <er>Sanders</er>.]</ety> <fld>(Bot.)</fld> <sd>(a)</sd> <def>The highly perfumed yellowish heartwood of an East Indian and Polynesian tree (<spn>Santalum album</spn>), and of several other trees of the same genus, as the Hawaiian <spn>Santalum Freycinetianum</spn> and <spn>Santalum pyrularium</spn>, the Australian <spn>Santalum latifolium</spn>, etc. The name is extended to several other kinds of fragrant wood.</def> <sd>(b)</sd> <def>Any tree of the genus <gen>Santalum</gen>, or a tree which yields sandalwood.</def> <sd>(c)</sd> <def>The red wood of a kind of buckthorn, used in Russia for dyeing leather (<spn>Rhamnus Dahuricus</spn>).</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>False sandalwood</b></col>, <cd>the fragrant wood of several trees not of the genus <gen>Santalum</gen>, as <spn>Ximenia Americana</spn>, <spn>Myoporum tenuifolium</spn> of Tahiti.</cd> -- <col><b>Red sandalwood</b></col>, <cd>a heavy, dark red dyewood, being the heartwood of two leguminous trees of India (<spn>Pterocarpus santalinus</spn>, and <spn>Adenanthera pavonina</spn>); -- called also <altname>red sanderswood</altname>, <altname>sanders</altname> or <altname>saunders</altname>, and <altname>rubywood</altname>.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sandarac</ent><br/
<ent>Sandarach</ent><br/
<mhw>{ <hw>San"da*rach</hw>, <hw>San"da*rac</hw>, }</mhw> <pr>(<?/)</pr>, <pos>n.</pos> <ety>[L. <ets>sandaraca</ets>, Gr. <?/.]</ety> <sn>1.</sn> <fld>(Min.)</fld> <def>Realgar; red sulphide of arsenic.</def> <mark>[Archaic]</mark><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Bot. Chem.)</fld> <def>A white or yellow resin obtained from a Barbary tree (<spn>Callitris quadrivalvis</spn> or <spn>Thuya articulata</spn>), and pulverized for pounce; -- probably so called from a resemblance to the mineral.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>sandbag</ent><br/
<hw>sand"bag`</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A bag filled with sand; small sandbags may be used as a weapon, or larger ones to build walls or as ballast; <as>as, they kept the flooding river from the area by buiding a temmporary dike out of <ex>sandbags</ex></as>.</def><br/
[<source>WordNet 1.6</source>]</p>

<p><ent>sandbag</ent><br/
<hw>sand"bag`</hw> <pr>(?)</pr>, <pos>v.</pos> <def>To treat harshly or unfairly.</def> <wns>[wns=1]</wns><br/
[<source>WordNet 1.6</source>]</p>

<p><sn>2.</sn> <def>To hit something or somebody with or as if with a sandbag.</def> <wns>[wns=2]</wns><br/
[<source>WordNet 1.6</source>]</p>

<p><sn>3.</sn> <def>To protect or strengthen with sandbags; stop up; <as>as, the residents <ex>sandbagged</ex> the beach front</as>.</def> <wns>[wns=3]</wns><br/
[<source>WordNet 1.6</source>]</p>

<p><sn>4.</sn> <def>To thwart (another person's plans) by surreptitious maneuvers; <as>as, he <ex>sandbagged</ex> my proposal by talking in private with other members of the committee</as>.</def> <mark>[informal]</mark><br/
[<source>PJC</source>]</p>

<p><sn>5.</sn> <def>To intimidate or coerce, especially by crude methods.</def> <mark>[Informal]</mark><br/
[<source>PJC</source>]</p>

<p><sn>6.</sn> <def>To deceive and take advantage of (a person) by misrepresenting one's true intentions.</def> <mark>[Informal]</mark><br/
[<source>PJC</source>]</p>

<p><sn>7.</sn> <specif>Hence:</specif> <fld>(Poker)</fld> <def>To encourage opponents into betting more by first refraining from betting while having a strong hand, and only later raising the stakes.  In informal games, certain types of <ex>sandbagging</ex> are forbidden.</def><br/
[<source>PJC</source>]</p>

<p><ent>Sandbagger</ent><br/
<hw>Sand"bag`ger</hw> <pr>(?)</pr>, <pos>n.</pos> <def>An assaulter whose weapon is a sand bag. See <cref>Sand bag</cref>, under <er>Sand</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sand-blind</ent><br/
<hw>Sand"-blind`</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[For <ets>sam blind</ets> half blind; AS. <ets>s<amac/m-</ets> half (akin to <ets>semi-</ets>) + <ets>blind</ets>.]</ety> <def>Having defective sight; dim-sighted; purblind.</def> <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sanded</ent><br/
<hw>Sand"ed</hw>, <pos>a.</pos> <sn>1.</sn> <def>Covered or sprinkled with sand; sandy; barren.</def> <rj><au>Thomson.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Marked with small spots; variegated with spots; speckled; of a sandy color, as a hound.</def> <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>Short-sighted.</def> <mark>[Prov. Eng.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sandemanian</ent><br/
<hw>San`de*ma"ni*an</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Eccl. Hist.)</fld> <def>A follower of Robert <xex>Sandeman</xex>, a Scotch sectary of the eighteenth century. See <er>Glassite</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sandemanianism</ent><br/
<hw>San`de*ma"ni*an*ism</hw> <pr>(?)</pr>, <pos>n.</pos> <def>The faith or system of the Sandemanians.</def> <rj><au>A. Fuller.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sanderling</ent><br/
<hw>San"der*ling</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[<ets>Sand</ets> + <ets>-ling</ets>. So called because it obtains its food by searching the moist sands of the seashore.]</ety> <fld>(Zool.)</fld> <def>A small gray and brown sandpiper (<spn>Calidris arenaria</spn>) very common on sandy beaches in America, Europe, and Asia. Called also <altname>curwillet</altname>, <altname>sand lark</altname>, <altname>stint</altname>, and <altname>ruddy plover</altname>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sanders</ent><br/
<hw>San"ders</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[See <er>Sandal</er>.]</ety> <def>An old name of sandalwood, now applied only to the red sandalwood. See under <er>Sandalwood</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sanders-blue</ent><br/
<hw>San"ders-blue"</hw> <pr>(?)</pr>, <pos>n.</pos> <def>See <er>Saunders-blue</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sandever</ent><br/
<hw>San"de*ver</hw> <pr>(?)</pr>, <pos>n.</pos> <def>See <er>Sandiver</er>.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sandfish</ent><br/
<hw>Sand"fish`</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Zool.)</fld> <def>A small marine fish of the Pacific coast of North America (<spn>Trichodon trichodon</spn>) which buries itself in the sand.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sandglass</ent><br/
<hw>Sand"glass`</hw> <pr>(?)</pr>, <pos>n.</pos> <def>An instrument for measuring time by the running of sand. See <er>Hourglass</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sandhiller</ent><br/
<hw>Sand"hill`er</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A nickname given to any <ldquo/poor white<rdquo/ living in the pine woods which cover the sandy hills in Georgia and South Carolina.</def> <mark>[U.S.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sandiness</ent><br/
<hw>Sand"i*ness</hw> <pr>(?)</pr>, <pos>n.</pos> <def>The quality or state of being sandy, or of being of a sandy color.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sandish</ent><br/
<hw>Sand"ish</hw>, <pos>a.</pos> <def>Approaching the nature of sand; loose; not compact.</def> <mark>[Obs.]</mark> <rj><au>Evelyn.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sandiver</ent><br/
<hw>San"di*ver</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Perh. fr. OF. <ets>sa<ium/n</ets> grease, fat + <ets>de</ets> of + <ets>verre</ets> glass (cf. <er>Saim</er>), or fr. F. <ets>sel de verre</ets> sandiver.]</ety> <def>A whitish substance which is cast up, as a scum, from the materials of glass in fusion, and, floating on the top, is skimmed off; -- called also <altname>glass gall</altname>.</def> <altsp>[Formerly written also <asp>sandever</asp>.]</altsp><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sandix</ent><br/
||<hw>San"dix</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>sandix</ets>, <ets>sandyx</ets>, vermilion, or a color like vermilion, Gr. <?/, <?/.]</ety> <def>A kind of minium, or red lead, made by calcining carbonate of lead, but inferior to true minium.</def> <altsp>[Written also <asp>sandyx</asp>.]</altsp> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sand-lot</ent><br/
<ent>sandlot</ent><br/
<mhw><hw>sand"lot</hw>, <hw>Sand"-lot`</hw></mhw>, <pos>a.</pos> <sn>1.</sn> <def>Lit., of or pert. to a lot or piece of sandy ground, -- hence, pert. to, or characteristic of, the policy or practices of the socialistic or communistic followers of the Irish agitator Denis Kearney, who delivered many of his speeches in the open sand lots about San Francisco; <as>as, the <col><b>sand-lot constitution</b></col> of California, framed in 1879, under the influence of <ex>sand-lot</ex> agitation</as>.</def></p>

<p><sn>2.</sn> <def> of or pertaining to a sandlot; -- used especially in reference to informal games played by children; <as>as, <ex>sandlot</ex> baseball.</as></def><br/
[<source>PJC</source>]</p>

<p><ent>sandlot</ent><br/
<hw>sand"lot</hw> <pr>(s<acr/nd"l<ocr/t)</pr>, <pos>n.</pos> <def>a vacant lot, especially one where children play games.</def><br/
[<source>PJC</source>]</p>

<p><ent>Sandman</ent><br/
<hw>Sand"man`</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A mythical person who makes children sleepy, so that they rub their eyes as if there were sand in them.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sandnecker</ent><br/
<hw>Sand"neck`er</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Zool.)</fld> <def>A European flounder (<spn>Hippoglossoides limandoides</spn>); -- called also <altname>rough dab</altname>, <altname>long fluke</altname>, <altname>sand fluke</altname>, and <altname>sand sucker</altname>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sandpaper</ent><br/
<hw>Sand"pa`per</hw> <pr>(?)</pr>, <pos>n.</pos> <def>Paper covered on one side with sand glued fast, -- used for smoothing and polishing.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sandpaper</ent><br/
<hw>Sand"pa`per</hw>, <pos>v. t.</pos> <def>To smooth or polish with sandpaper; <as>as, to <ex>sandpaper</ex> a door</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sandpiper</ent><br/
<hw>Sand"pi`per</hw> <pr>(?)</pr>, <pos>n.</pos> <sn>1.</sn> <fld>(Zool.)</fld> <def>Any one of numerous species of small limicoline game birds belonging to <gen>Tringa</gen>, <gen>Actodromas</gen>, <gen>Ereunetes</gen>, and various allied genera of the family <fam>Tringidae</fam>.</def><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ The most important North American species are the pectoral sandpiper (<spn>Tringa maculata</spn>), called also <altname>brownback</altname>, <altname>grass snipe</altname>, and <altname>jacksnipe</altname>; the red-backed, or black-breasted, sandpiper, or dunlin (<spn>Tringa alpina</spn>); the purple sandpiper (<spn>Tringa maritima</spn>: the red-breasted sandpiper, or knot (<spn>Tringa canutus</spn>); the semipalmated sandpiper (<spn>Ereunetes pusillus</spn>); the spotted sandpiper, or teeter-tail (<spn>Actitis macularia</spn>); the buff-breasted sandpiper (<spn>Tryngites subruficollis</spn>), and the Bartramian sandpiper, or upland plover. See under <er>Upland</er>. Among the European species are the dunlin, the knot, the ruff, the sanderling, and the common sandpiper (<spn>Actitis hypoleucus</spn> syn. <spn>Tringoides hypoleucus</spn>), called also <altname>fiddler</altname>, <altname>peeper</altname>, <altname>pleeps</altname>, <altname>weet-weet</altname>, and <altname>summer snipe</altname>.  Some of the small plovers and tattlers are also called <ex>sandpipers</ex>.</note><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Zool.)</fld> <def>A small lamprey eel; the pride.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Curlew sandpiper</b></col>. <cd>See under <er>Curlew</er>.</cd> -- <col><b>Stilt sandpiper</b></col>. <cd>See under <er>Stilt</er>.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sandpit</ent><br/
<hw>Sand"pit`</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A pit or excavation from which sand is or has been taken.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sandre</ent><br/
<hw>San"dre</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Zool.)</fld> <def>A Russian fish (<spn>Lucioperca sandre</spn>) which yields a valuable oil, called <xex>sandre oil</xex>, used in the preparation of caviar.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sandstone</ent><br/
<hw>Sand"stone`</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A rock made of sand more or less firmly united. Common or siliceous sandstone consists mainly of quartz sand.</def><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ Different names are applied to the various kinds of sandstone according to their composition; as, <xex>granitic</xex>, <xex>argillaceous</xex>, <xex>micaceous</xex>, etc.</note><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Flexible sandstone</b></col> <fld>(Min.)</fld>, <cd>the finer-grained variety of itacolumite, which on account of the scales of mica in the lamination is quite flexible.</cd> -- <col><b>Red sandstone</b></col>, <cd>a name given to two extensive series of British rocks in which red sandstones predominate, one below, and the other above, the coal measures. These were formerly known as the <xex>Old</xex> and the <xex>New Red Sandstone</xex> respectively, and the former name is still retained for the group preceding the Coal and referred to the Devonian age, but the term <xex>New Red Sandstone</xex> is now little used, some of the strata being regarded as Permian and the remained as Triassic. See the <xex>Chart</xex> of <er>Geology</er>.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sandwich</ent><br/
<hw>Sand"wich</hw> <pr>(?; 277)</pr>, <pos>n.</pos> <ety>[Named from the Earl of <ets>Sandwich</ets>.]</ety> <def>Two pieces of bread and butter with a thin slice of meat, cheese, or the like, between them.</def><br/
[<source>1913 Webster</source>]</p>

<p><-- 2. Any food composed of two pieces of bread with another food in between. 3. Any object composed of two layers of one subtance on either side of a second substance. --><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sandwich</ent><br/
<hw>Sand"wich</hw>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Sandwiched</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Sandwiching</conjf>.]</vmorph> <def>To make into a sandwich; also, figuratively, to insert between portions of something dissimilar; to form of alternate parts or things, or alternating layers of a different nature; to interlard.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sandworm</ent><br/
<hw>Sand"worm`</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Zool.)</fld> <sd>(a)</sd> <def>Any one of numerous species of annelids which burrow in the sand of the seashore.</def> <sd>(b)</sd> <def>Any species of annelids of the genus <gen>Sabellaria</gen>. They construct firm tubes of agglutinated sand on rocks and shells, and are sometimes destructive to oysters.</def> <sd>(c)</sd> <def>The chigoe, a species of flea.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sandwort</ent><br/
<hw>Sand"wort`</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Bot.)</fld> <def>Any plant of the genus <gen>Arenaria</gen>, low, tufted herbs (order <ord>Caryophyllaceae</ord>.)</def> <-- "of" missing? --><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sandy</ent><br/
<hw>Sand"y</hw> <pr>(?)</pr>, <pos>a.</pos> <amorph>[<pos>Compar.</pos> <adjf>Sandier</adjf> <pr>(?)</pr>; <pos>superl.</pos> <adjf>Sandiest</adjf>.]</amorph> <ety>[AS. <ets>sandig</ets>.]</ety> <sn>1.</sn> <def>Consisting of, abounding with, or resembling, sand; full of sand; covered or sprinkled with sand; <as>as, a <ex>sandy</ex> desert, road, or soil</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Of the color of sand; of a light yellowish red color; <as>as, <ex>sandy</ex> hair</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sandyx</ent><br/
||<hw>San"dyx</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L.]</ety> <def>See <er>Sandix</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sane</ent><br/
<hw>Sane</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>sanus</ets>; cf. Gr. <?/, <?/, safe, sound. Cf. <er>Sound</er>, <pos>a.</pos>]</ety> <sn>1.</sn> <def>Being in a healthy condition; not deranged; acting rationally; -- said of the mind.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Mentally sound; possessing a rational mind; having the mental faculties in such condition as to be able to anticipate and judge of the effect of one's actions in an ordinary maner; -- said of persons.</def><br/
[<source>1913 Webster</source>]</p>

<p><syn><b>Syn.</b> -- Sound; healthy; underanged; unbroken.</syn><br/
[<source>1913 Webster</source>]</p>

<p><ent>Saneness</ent><br/
<hw>Sane"ness</hw>, <pos>n.</pos> <def>The state of being sane; sanity.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sang</ent><br/
<hw>Sang</hw> <pr>(?)</pr>, <def><pos>imp.</pos> of <er>Sing</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sangu</ent><br/
<ent>Sanga</ent><br/
<mhw>{ ||<hw>San"ga</hw> <pr>(?)</pr>, <hw>San"gu</hw> <pr>(?)</pr>, }</mhw> <pos>n.</pos> <fld>(Zool.)</fld> <def>The Abyssinian ox (<spn>Bos Africanus</spn> syn. <spn>Bibos Africanus</spn>), noted for the great length of its horns. It has a hump on its back.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sangaree</ent><br/
<hw>San`ga*ree"</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Sp. <ets>sangria</ets>, lit., bleeding, from <ets>sangre</ets>, blood, L. <ets>sanguis</ets>.]</ety> <def>Wine and water sweetened and spiced, -- a favorite West Indian drink.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sang-froid</ent><br/
||<hw>Sang`-froid"</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F., cold blood.]</ety> <def>Freedom from agitation or excitement of mind; coolness in trying circumstances; indifference; calmness.</def> <rj><au>Burke.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sangiac</ent><br/
<hw>San"gi*ac</hw> <pr>(?)</pr>, <pos>n.</pos> <def>See <er>Sanjak</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sangreal</ent><br/
<ent>Sangraal</ent><br/
<mhw>{ <hw>San`graal"</hw> <pr>(?)</pr>, <hw>San"gre*al</hw> <pr>(?)</pr>, }</mhw> <pos>n.</pos> <ety>[See <er>Saint</er>, and <er>Grail</er>.]</ety> <def>See <cref>Holy Grail</cref>, under <er>Grail</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sanguiferous</ent><br/
<hw>San*guif"er*ous</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>sanguis</ets> blood + <ets>-ferous</ets>.]</ety> <fld>(Physiol.)</fld> <def>Conveying blood; <as>as, <ex>sanguiferous</ex> vessels, <it>i. e.</it>, the arteries, veins, capillaries</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sanguification</ent><br/
<hw>San`gui*fi*ca"tion</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Cf. F. <ets>sanguification</ets>. See <er>Sanguify</er>.]</ety> <fld>(Physiol.)</fld> <def>The production of blood; the conversion of the products of digestion into blood; hematosis.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sanguifier</ent><br/
<hw>San"gui*fi`er</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A producer of blood.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sanguifluous</ent><br/
<hw>San*guif"lu*ous</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>sanguis</ets> blood + <ets>fluere</ets> to flow.]</ety> <def>Flowing or running with blood.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sanguify</ent><br/
<hw>San"gui*fy</hw> <pr>(?)</pr>, <pos>v. t.</pos> <ety>[L. <ets>sanguis</ets> blood + <ets>-fy</ets>: cf. F. <ets>sanguifier</ets>.]</ety> <def>To produce blood from.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sanguigenous</ent><br/
<hw>San*guig"e*nous</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>sanguis</ets> + <ets>-genous</ets>.]</ety> <def>Producing blood; <as>as, <ex>sanguigenous</ex> food</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sanguinaceous</ent><br/
<hw>San`gui*na"ceous</hw> <pr>(?)</pr>, <pos>n.</pos> <def>Of a blood-red color; sanguine.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sanguinaria</ent><br/
||<hw>San`gui*na"ri*a</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[NL. See <er>Sanguinary</er>, <pos>a.</pos> & <pos>n.</pos>]</ety> <sn>1.</sn> <fld>(Bot.)</fld> <def>A genus of plants of the Poppy family.</def><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ <spn>Sanguinaria Canadensis</spn>, or bloodroot, is the only species. It has a perennial rootstock, which sends up a few roundish lobed leaves and solitary white blossoms in early spring. See <er>Bloodroot</er>.</note><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>The rootstock of the bloodroot, used in medicine as an emetic, etc.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sanguinarily</ent><br/
<hw>San"gui*na*ri*ly</hw> <pr>(?)</pr>, <pos>adv.</pos> <def>In a sanguinary manner.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sanguinariness</ent><br/
<hw>San"gui*na*ri*ness</hw>, <pos>n.</pos> <def>The quality or state of being sanguinary.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sanguinary</ent><br/
<hw>San"gui*na*ry</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>sanguinarius</ets>, fr. <ets>sanguis</ets> blood: cf. F. <ets>sanguinaire</ets>.]</ety> <sn>1.</sn> <def>Attended with much bloodshed; bloody; murderous; <as>as, a <ex>sanguinary</ex> war, contest, or battle</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>We may not propagate religion by wars, or by <qex>sanguinary</qex> persecutions to force consciences.</q> <rj><qau>Bacon.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Bloodthirsty; cruel; eager to shed blood.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Passion . . . makes us brutal and <qex>sanguinary</qex>.</q> <rj><qau>Broome.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><syn><b>Syn.</b> -- Bloody; murderous; bloodthirsty; cruel.</syn><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sanguinary</ent><br/
<hw>San"gui*na*ry</hw>, <pos>n.</pos> <ety>[L. <ets>herba sanguinaria</ets> an herb that stanches blood: cf. F. <ets>sanguinaire</ets>. See <er>Sanguinary</er>, <pos>a.</pos>]</ety> <fld>(Bot.)</fld> <sd>(a)</sd> <def>The yarrow.</def> <sd>(b)</sd> <def>The Sanguinaria.</def><br/
[<source>1913 Webster</source>]</p>

<p><-- p. 1275 pr=PCP --></p>

<p><ent>Sanguine</ent><br/
<hw>San"guine</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[F. <ets>sanguin</ets>, L. <ets>sanguineus</ets>, fr. <ets>sanguis</ets> blood. Cf. <er>Sanguineous</er>.]</ety> <sn>1.</sn> <def>Having the color of blood; red.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Of his complexion he was <qex>sanguine</qex>.</q> <rj><qau>Chaucer.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Like to that <qex>sanguine</qex> flower inscribed with woe.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Characterized by abundance and active circulation of blood; <as>as, a <ex>sanguine</ex> bodily temperament</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>Warm; ardent; <as>as, a <ex>sanguine</ex> temper</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>Anticipating the best; cheerfully optimistic; not desponding; confident; full of hope; <as>as, <ex>sanguine</ex> of success; a <ex>sanguine</ex> disposition</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><syn><b>Syn.</b> -- Warm; ardent; lively; confident; hopeful; optimistic.</syn><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sanguine</ent><br/
<hw>San"guine</hw>, <pos>n.</pos> <sn>1.</sn> <def>Blood color; red.</def> <rj><au>Spenser.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Anything of a blood-red color, as cloth.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>In <qex>sanguine</qex> and in pes he clad was all.</q> <rj><qau>Chaucer.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <fld>(Min.)</fld> <def>Bloodstone.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>Red crayon. See the Note under <er>Crayon</er>, 1.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sanguine</ent><br/
<hw>San"guine</hw>, <pos>v. t.</pos> <def>To stain with blood; to impart the color of blood to; to ensanguine.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sanguineless</ent><br/
<hw>San"guine*less</hw>, <pos>a.</pos> <def>Destitute of blood; pale.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sanguinely</ent><br/
<hw>San"guine*ly</hw>, <pos>adv.</pos> <def>In a sanguine manner.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>I can not speculate quite so <qex>sanguinely</qex> as he does.</q> <rj><qau>Burke.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sanguineness</ent><br/
<hw>San"guine*ness</hw>, <pos>n.</pos> <def>The quality of being sanguine.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sanguineous</ent><br/
<hw>San*guin"e*ous</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>sanguineus</ets>. See <er>Sanguine</er>.]</ety> <sn>1.</sn> <def>Abounding with blood; sanguine.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Of or pertaining to blood; bloody; constituting blood.</def> <rj><au>Sir T. Browne.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>Blood-red; crimson.</def> <rj><au>Keats.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>sanguinity</ent><br/
<hw>san*guin"i*ty</hw>, <pos>n.</pos> <def>The quality of being sanguine; sanguineness.</def> <rj><au>Swift.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sanguinivorous</ent><br/
<hw>San"gui*niv"o*rous</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>sanguis + vorare</ets> to devour.]</ety> <def>Subsisting on blood.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sanguinolency</ent><br/
<hw>San*guin"o*len*cy</hw> <pr>(?)</pr>, <pos>n.</pos> <def>The state of being sanguinolent, or bloody.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sanguinolent</ent><br/
<hw>San*guin"o*lent</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>sanguinolentus</ets>, from <ets>sanguis</ets> blood: cf. F. <ets>sanguinolent</ets>.]</ety> <def>Tinged or mingled with blood; bloody; <as>as, <ex>sanguinolent</ex> sputa</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sanguisuge</ent><br/
<hw>San"gui*suge</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>sanguisuga</ets>; <ets>sanguis</ets> blood + <ets>sugere</ets> to suck.]</ety> <fld>(Zool.)</fld> <def>A bloodsucker, or leech.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sanguivorous</ent><br/
<hw>San*guiv"o*rous</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>sanguis</ets> blood + <ets>vorare</ets> to devour.]</ety> <fld>(Zool.)</fld> <def>Subsisting upon blood; -- said of certain blood-sucking bats and other animals. See <er>Vampire</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sanhedrim</ent><br/
<ent>Sanhedrin</ent><br/
<mhw>{ <hw>San"he*drin</hw> <pr>(?)</pr>, <hw>San"he*drim</hw> <pr>(?)</pr>, }</mhw> <pos>n.</pos> <ety>[Heb. <ets>sanhedr<imac/n</ets>, fr. Gr. <?/; <?/ with + <?/ a seat, fr. <?/ to sit. See <er>Sit</er>.]</ety> <fld>(Jewish Antiq.)</fld> <def>the great council of the Jews, which consisted of seventy members, to whom the high priest was added. It had jurisdiction of religious matters.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sanhedrist</ent><br/
<hw>San"he*drist</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A member of the sanhedrin.</def> <rj><au>Schaeffer (Lange's Com.).</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sanhita</ent><br/
||<hw>San"hi*ta</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Skr. <ets>samhita</ets>, properly, combination.]</ety> <def>A collection of vedic hymns, songs, or verses, forming the first part of each Veda.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sanicle</ent><br/
<hw>San"i*cle</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F., from L. <ets>sanare</ets> to heal.]</ety> <fld>(Bot.)</fld> <def>Any plant of the umbelliferous genus <gen>Sanicula</gen>, reputed to have healing powers.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sanidine</ent><br/
<hw>San"i*dine</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Gr. <?/. <?/, a board. So called in allusion to the tabular crystals.]</ety> <fld>(Min.)</fld> <def>A variety of orthoclase feldspar common in certain eruptive rocks, as trachyte; -- called also <altname>glassy feldspar</altname>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sanies</ent><br/
||<hw>Sa"ni*es</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L.]</ety> <fld>(Med.)</fld> <def>A thin, serous fluid commonly discharged from ulcers or foul wounds.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sanious</ent><br/
<hw>Sa"ni*ous</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>saniosus</ets>, fr. <ets>sanies</ets>: cf. F. <ets>sanieux</ets>.]</ety> <sn>1.</sn> <fld>(Med.)</fld> <def>Pertaining to sanies, or partaking of its nature and appearance; thin and serous, with a slight bloody tinge; <as>as, the <ex>sanious</ex> matter of an ulcer</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(med.)</fld> <def>Discharging sanies; <as>as, a <ex>sanious</ex> ulcer</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sanitarian</ent><br/
<hw>San`i*ta"ri*an</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Of or pertaining to health, or the laws of health; sanitary.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sanitarian</ent><br/
<hw>San`i*ta"ri*an</hw>, <pos>n.</pos> <def>An advocate of sanitary measures; one especially interested or versed in sanitary measures.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sanitarist</ent><br/
<hw>San"i*ta*rist</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A sanitarian.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sanitarium</ent><br/
<hw>San`i*ta"ri*um</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[NL. See <er>Sanitary</er>.]</ety> <def>A health station or retreat; a sanatorium.</def> <ldquo/A <xex>sanitarium</xex> for troops.<rdquo/ <rj><au>L. Oliphant.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sanitary</ent><br/
<hw>San"i*ta*ry</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>sanitas</ets> health: cf. F. <ets>sanitaire</ets>. See <er>Sanity</er>.]</ety> <def>Of or pertaining to health; designed to secure or preserve health; relating to the preservation or restoration of health; hygienic; <as>as, <ex>sanitary</ex> regulations</as>. See the Note under <er>Sanatory</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Sanitary Commission</b></col>. <cd>See under <er>Commission</er>.</cd></cs> <-- sanitary napkin, an absorbant pad worn to absorb the menstrual flow of women. --><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sanitation</ent><br/
<hw>San`i*ta"tion</hw> <pr>(?)</pr>, <pos>n.</pos> <def>The act of rendering sanitary; the science of sanitary conditions; the preservation of health; the use of sanitary measures; hygiene.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>How much <qex>sanitation</qex> has advanced during the last half century.</q> <rj><qau>H. Hartshorne.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sanity</ent><br/
<hw>San"i*ty</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>sanitas</ets>, from <ets>sanus</ets> sound, healthy. See <er>Sane</er>.]</ety> <def>The condition or quality of being sane; soundness of health of body or mind, especially of the mind; saneness.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sanjak</ent><br/
<hw>San"jak</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Turk. <ets>sanj<amac/g</ets>.]</ety> <def>A district or a subvision of a vilayet.</def> <mark>[Turkey]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>San Jose scale</ent><br/
<hw>San Jo*s<eacute/" scale</hw> <pr>(?)</pr>. <def>A very destructive scale insect (<spn>Aspidiotus perniciosus</spn>) that infests the apple, pear, and other fruit trees.  So called because first introduced into the United States at <city><etsep>San Jos<eacute/</etsep></city>, California.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Sank</ent><br/
<hw>Sank</hw> <pr>(s<acr/<nsm/k)</pr>, <def><pos>imp.</pos> of <er>Sink</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sankha</ent><br/
||<hw>Sank"ha</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Skr. <ets><cced/ankha</ets> a shell.]</ety> <def>A chank shell (<spn>Turbinella pyrum</spn>); also, a shell bracelet or necklace made in India from the chank shell.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sankhya</ent><br/
||<hw>Sankh"ya</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A Hindu system of philosophy which refers all things to soul and a rootless germ called <xex>prakriti</xex>, consisting of three elements, goodness, passion, and darkness.</def> <rj><au>Whitworth.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sannop</ent><br/
<hw>San"nop</hw> <pr>(s<acr/n"n<ocr/p)</pr>, <pos>n.</pos> <def>Same as <er>Sannup</er>.</def> <rj><au>Bancroft.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sannup</ent><br/
<hw>San"nup</hw> <pr>(-n<ucr/p)</pr>, <pos>n.</pos> <def>A married male Indian; a brave; -- correlative of <inv>squaw</inv>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sanny</ent><br/
<hw>San"ny</hw> <pr>(?)</pr>, <pos>n.</pos> <def>The sandpiper.</def> <mark>[Prov. Eng.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sans</ent><br/
||<hw>Sans</hw> <pr>(s<aum/n; E. s<acr/nz)</pr>, <pos>prep.</pos> <ety>[F., from L. <ets>sine</ets> without.]</ety> <def>Without; deprived or destitute of. Rarely used as an English word.</def> <ldquo/<xex>Sans</xex> fail.<rdquo/ <rj><au>Chaucer.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q><qex>Sans</qex> teeth, <qex>sans</qex> eyes, <qex>sans</qex> taste, <qex>sans</qex> everything.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sanscrit</ent><br/
<hw>San"scrit</hw> <pr>(?)</pr>, <pos>n.</pos> <def>See <er>Sanskrit</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sans-culotte</ent><br/
||<hw>Sans`-cu`lotte"</hw> <pr>(F. <?/; E. <?/)</pr>, <pos>n.</pos> <ety>[F., without breeches.]</ety> <sn>1.</sn> <def>A fellow without breeches; a ragged fellow; -- a name of reproach given in the first French revolution to the extreme republican party, who rejected breeches as an emblem peculiar to the upper classes or aristocracy, and adopted pantaloons.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Hence, an extreme or radical republican; a violent revolutionist; a Jacobin.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sans-culottic</ent><br/
<hw>Sans`-cu*lot"tic</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Pertaining to, or involving, sans-culottism; radical; revolutionary; Jacobinical.</def> <rj><au>Carlyle.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sans-culottism</ent><br/
<hw>Sans`-cu*lot"tism</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F. <ets>sans-culottisme</ets>.]</ety> <def>Extreme republican principles; the principles or practice of the <xex>sans-culottes</xex>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sanskrit</ent><br/
<hw>San"skrit</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Skr. <ets>Samsk<rsdot/ta</ets> the Sanskrit language, literally, the perfect, polished, or classical language, fr. <ets>samsk<rsdot/ta</ets> prepared, wrought, made, excellent, perfect; <ets>sam</ets> together (akin to E. <ets>same</ets>) + <ets>k<rsdot/ta</ets> made. See <er>Same</er>, <er>Create</er>.]</ety> <altsp>[Written also <asp>Sanscrit</asp>.]</altsp> <def>The ancient language of the Hindoos, long since obsolete in vernacular use, but preserved to the present day as the literary and sacred dialect of India. It is nearly allied to the Persian, and to the principal languages of Europe, classical and modern, and by its more perfect preservation of the roots and forms of the primitive language from which they are all descended, is a most important assistance in determining their history and relations. Cf. <er>Prakrit</er>, and <er>Veda</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sanskrit</ent><br/
<hw>San"skrit</hw>, <pos>a.</pos> <def>Of or pertaining to Sanskrit; written in Sanskrit; <as>as, a <ex>Sanskrit</ex> dictionary or inscription</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sanskritic</ent><br/
<hw>San*skrit"ic</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Sanskrit.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sanskritist</ent><br/
<hw>San"skrit*ist</hw>, <pos>n.</pos> <def>One versed in Sanskrit.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sans-souci</ent><br/
||<hw>Sans`-sou`ci"</hw> <pr>(?)</pr>, <pos>adv.</pos> <ety>[F.]</ety> <def>Without care; free and easy.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Santal</ent><br/
<hw>San"tal</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[<ets>Sant</ets>alum + piperon<ets>al</ets>.]</ety> <fld>(Chem.)</fld> <def>A colorless crystalline substance, isomeric with piperonal, but having weak acid properties. It is extracted from sandalwood.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Santalaceous</ent><br/
<hw>San`ta*la"ceous</hw> <pr>(?)</pr>, <pos>a.</pos> <fld>(Bot.)</fld> <def>Of or pertaining to a natural order of plants (<ord>Santalaceae</ord>), of which the genus <gen>Santalum</gen> is the type, and which includes the buffalo nut and a few other North American plants, and many peculiar plants of the southern hemisphere.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Santalic</ent><br/
<hw>San*tal"ic</hw> <pr>(?)</pr>, <pos>a.</pos> <fld>(Chem.)</fld> <def>Of, pertaining to, or obtained from, sandalwood (<gen>Santalum</gen>); -- used specifically to designate an acid obtained as a resinous or red crystalline dyestuff, which is called also <xex>santalin</xex>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Santalin</ent><br/
<hw>San"ta*lin</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Cf. F. <ets>santaline</ets>.]</ety> <fld>(Chem.)</fld> <def>Santalic acid. See <er>Santalic</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Santalum</ent><br/
||<hw>San"ta*lum</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[NL. See <er>Sandalwood</er>.]</ety> <fld>(Bot.)</fld> <def>A genus of trees with entire opposite leaves and small apetalous flowers. There are less than a dozen species, occurring from India to Australia and the Pacific Islands. See <er>Sandalwood</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Santees</ent><br/
<hw>San`tees"</hw> <pr>(?)</pr>, <pos>n. pl.</pos>; <sing>sing. <singw>Santee</singw> <pr>(<?/)</pr></sing>. <fld>(Ethnol.)</fld> <def>One of the seven confederated tribes of Indians belonging to the Sioux, or Dakotas.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Santer</ent><br/
<hw>San"ter</hw> <pr>(?)</pr>, <pos>v. i.</pos> <def>See <er>Saunter</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Santon</ent><br/
||<hw>San"ton</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Sp. <ets>santon</ets>, augmented fr. <ets>santo</ets> holy, L. <ets>sanctus</ets>.]</ety> <def>A Turkish saint; a kind of dervish, regarded by the people as a saint: also, a hermit.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Santonate</ent><br/
<hw>San"to*nate</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Chem.)</fld> <def>A salt of santonic acid.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Santonic</ent><br/
<hw>San*ton"ic</hw> <pr>(?)</pr>, <pos>a.</pos> <fld>(Chem.)</fld> <def>Of, pertaining to, or designating, an acid (distinct from santoninic acid) obtained from santonin as a white crystalline substance.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Santonin</ent><br/
<hw>San"to*nin</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>herba santonica</ets>, a kind of plant, fr. <ets>Santoni</ets> a people of Aquitania; cf. Gr. <?/: cf. F. <ets>santonine</ets>.]</ety> <fld>(Chem.)</fld> <def>A white crystalline substance having a bitter taste, extracted from the buds of levant wormseed and used as an anthelmintic. It occassions a peculiar temporary color blindness, causing objects to appear as if seen through a yellow glass.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Santoninate</ent><br/
<hw>San"to*nin`ate</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Chem.)</fld> <def>A salt of santoninic acid.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Santoninic</ent><br/
<hw>San`to*nin"ic</hw> <pr>(?)</pr>, <pos>a.</pos> <fld>(Chem.)</fld> <def>Of or pertaining to santonin; -- used specifically to designate an acid not known in the free state, but obtained in its salts.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sao</ent><br/
||<hw>Sa"o</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Zool.)</fld> <def>Any marine annelid of the genus <gen>Hyalinaecia</gen>, especially <spn>Hyalinaecia tubicola</spn> of Europe, which inhabits a transparent movable tube resembling a quill in color and texture.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sap</ent><br/
<hw>Sap</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[AS. <ets>saep</ets>; akin to OHG. <ets>saf</ets>, G. <ets>saft</ets>, Icel. <ets>safi</ets>; of uncertain origin; possibly akin to L. <ets>sapere</ets> to taste, to be wise, <ets>sapa</ets> must or new wine boiled thick. Cf. <er>Sapid</er>, <er>Sapient</er>.]</ety> <sn>1.</sn> <def>The juice of plants of any kind, especially the ascending and descending juices or circulating fluid essential to nutrition.</def><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ The ascending is the <xex>crude</xex> sap, the assimilation of which takes place in the leaves, when it becomes the <xex>elaborated</xex> sap suited to the growth of the plant.</note><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>The sapwood, or alburnum, of a tree.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>A simpleton; a saphead; a milksop.</def> <mark>[Slang]</mark><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Sap ball</b></col> <fld>(Bot.)</fld>, <cd>any large fungus of the genus Polyporus. See <er>Polyporus</er>.</cd> -- <col><b>Sap green</b></col>, <cd>a dull light green pigment prepared from the juice of the ripe berries of the <spn>Rhamnus catharticus</spn>, or buckthorn. It is used especially by water-color artists.</cd> -- <col><b>Sap rot</b></col>, <cd>the dry rot. See under <er>Dry</er>.</cd> -- <col><b>Sap sucker</b></col> <fld>(Zool.)</fld>, <cd>any one of several species of small American woodpeckers of the genus <gen>Sphyrapicus</gen>, especially the yellow-bellied woodpecker (<spn>Sphyrapicus varius</spn>) of the Eastern United States. They are so named because they puncture the bark of trees and feed upon the sap. The name is loosely applied to other woodpeckers.</cd> -- <col><b>Sap tube</b></col> <fld>(Bot.)</fld>, <cd>a vessel that conveys sap.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sap</ent><br/
<hw>Sap</hw>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Sapped</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Sapping</conjf>.]</vmorph> <ety>[F. <ets>saper</ets> (cf. Sp. <ets>zapar</ets>, It. <ets>zapare</ets>), fr. <ets>sape</ets> a sort of scythe, LL. <ets>sappa</ets> a sort of mattock.]</ety> <sn>1.</sn> <def>To subvert by digging or wearing away; to mine; to undermine; to destroy the foundation of.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Nor safe their dwellings were, for <qex>sapped</qex> by floods,<br/
Their houses fell upon their household gods.</q> <rj><qau>Dryden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Mil.)</fld> <def>To pierce with saps.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>To make unstable or infirm; to unsettle; to weaken.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Ring out the grief that <qex>saps</qex> the mind.</q> <rj><qau>Tennyson.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sap</ent><br/
<hw>Sap</hw> <pr>(?)</pr>, <pos>v. i.</pos> <def>To proceed by mining, or by secretly undermining; to execute saps.</def> <rj><au>W. P. Craighill.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Both assaults are carried on by <qex>sapping</qex>.</q> <rj><qau>Tatler.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sap</ent><br/
<hw>Sap</hw>, <pos>n.</pos> <fld>(Mil.)</fld> <def>A narrow ditch or trench made from the foremost parallel toward the glacis or covert way of a besieged place by digging under cover of gabions, etc.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Sap fagot</b></col> <fld>(Mil.)</fld>, <cd>a fascine about three feet long, used in sapping, to close the crevices between the gabions before the parapet is made.</cd> -- <col><b>Sap roller</b></col> <fld>(Mil.)</fld>, <cd>a large gabion, six or seven feet long, filled with fascines, which the sapper sometimes rolls along before him for protection from the fire of an enemy.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sapadillo</ent><br/
<hw>Sap`a*dil"lo</hw> <pr>(?)</pr>, <pos>n.</pos> <def>See <er>Sapodila</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sapajo</ent><br/
<hw>Sap"a*jo</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Zool.)</fld> <def>The sapajou.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sapajou</ent><br/
<hw>Sap"a*jou</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F. <ets>sapajou</ets>, <ets>sajou</ets>, Braz. <ets>sajuassu</ets>.]</ety> <fld>(Zool.)</fld> <def>Any one of several species of South American monkeys of the genus <gen>Cebus</gen>, having long and prehensile tails. Some of the species are called also <stype>capuchins</stype>. The bonnet sapajou (<spn>Cebus subcristatus</spn>), the golden-handed sapajou (<spn>Cebus chrysopus</spn>), and the white-throated sapajou (<spn>Cebus hypoleucus</spn>) are well known species. See <er>Capuchin</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sapan wood</ent><br/
<hw>Sa*pan" wood</hw> <pr>(?)</pr>. <ety>[Malay <ets>sapang</ets>.]</ety> <fld>(Bot.)</fld> <def>A dyewood yielded by <spn>Caesalpinia Sappan</spn>, a thorny leguminous tree of Southern Asia and the neighboring islands. It is the original Brazil wood.</def> <altsp>[Written also <asp>sappan wood</asp>.]</altsp><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sapful</ent><br/
<hw>Sap"ful</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Abounding in sap; sappy.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Saphead</ent><br/
<hw>Sap"head`</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A weak-minded, stupid fellow; a milksop.</def> <mark>[Low]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Saphenous</ent><br/
<hw>Sa*phe"nous</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[Gr. <?/ manifest.]</ety> <fld>(Anat.)</fld> <sd>(a)</sd> <def>Manifest; -- applied to the two principal superficial veins of the lower limb of man.</def> <sd>(b)</sd> <def>Of, pertaining to, or in the region of, the saphenous veins; <as>as, the <ex>saphenous</ex> nerves; the <ex>saphenous</ex> opening, an opening in the broad fascia of the thigh through which the internal saphenous vein passes</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sapid</ent><br/
<hw>Sap"id</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>sapidus</ets>, fr. <ets>sapere</ets> to taste: cf. F. <ets>sapide</ets>. See <er>Sapient</er>, <er>Savor</er>.]</ety> <def>Having the power of affecting the organs of taste; possessing savor, or flavor.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Camels, to make the water <qex>sapid</qex>, do raise the mud with their feet.</q> <rj><qau>Sir T. Browne.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sapidity</ent><br/
<hw>Sa*pid"i*ty</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Cf. F. <ets>sapidit<eacute/</ets>.]</ety> <def>The quality or state of being sapid; taste; savor; savoriness.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Whether one kind of <qex>sapidity</qex> is more effective than another.</q> <rj><qau>M. S. Lamson.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sapidness</ent><br/
<hw>Sap"id*ness</hw>, <pos>n.</pos> <def>Quality of being sapid; sapidity.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>When the Israelites fancied the <qex>sapidness</qex> and relish of the fleshpots, they longed to taste and to return.</q> <rj><qau>Jer. Taylor.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sapience</ent><br/
<hw>Sa"pi*ence</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>sapientia</ets>: cf. F. <ets>sapience</ets>. See <er>Sapient</er>..]</ety> <def>The quality of being sapient; wisdom; sageness; knowledge.</def> <rj><au>Cowper.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Woman, if I might sit beside your feet,<br/
And glean your scattered <qex>sapience</qex>.</q> <rj><qau>Tennyson.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sapient</ent><br/
<hw>Sa"pi*ent</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>sapiens</ets>, <ets>-entis</ets>, p. pr. of <ets>sapere</ets> to taste, to have sense, to know. See <er>Sage</er>, <pos>a.</pos>]</ety> <def>Wise; sage; discerning; -- often in irony or contempt.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Where the <qex>sapient</qex> king<br/
Held dalliance with his fair Egyptian spouse.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><syn><b>Syn.</b> -- Sage; sagacious; knowing; wise; discerning.</syn><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sapiential</ent><br/
<hw>Sa`pi*en"tial</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>sapientialis</ets>.]</ety> <def>Having or affording wisdom.</def> -- <wordforms><wf>Sa`pi*en"tial*ly</wf>, <pos>adv.</pos></wordforms><br/
[<source>1913 Webster</source>]</p>

<p><q>The <qex>sapiential</qex> books of the Old [Testament].</q> <rj><qau>Jer. Taylor.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sapientious</ent><br/
<hw>Sa`pi*en"tious</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Sapiential.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sapientize</ent><br/
<hw>Sa"pi*ent*ize</hw>, <pos>v. t.</pos> <def>To make sapient.</def> <mark>[R.]</mark> <rj><au>Coleridge.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sapiently</ent><br/
<hw>Sa"pi*ent*ly</hw> <pr>(?)</pr>, <pos>adv.</pos> <def>In a sapient manner.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sapindaceous</ent><br/
<hw>Sap`in*da"ceous</hw> <pr>(?)</pr>, <pos>a.</pos> <fld>(Bot.)</fld> <def>Of or pertaining to an order of trees and shrubs (<ord>Sapindaceae</ord>), including the (typical) genus Sapindus, the maples, the margosa, and about seventy other genera.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sapindus</ent><br/
||<hw>Sa*pin"dus</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[NL., fr. L. <ets>sapo</ets> soap + <ets>Indicus</ets> Indian.]</ety> <fld>(Bot.)</fld> <def>A genus of tropical and subtropical trees with pinnate leaves and panicled flowers. The fruits of some species are used instead of soap, and their round black seeds are made into necklaces.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sapless</ent><br/
<hw>Sap"less</hw> <pr>(?)</pr>, <pos>a.</pos> <sn>1.</sn> <def>Destitute of sap; not juicy.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def><markp>Fig.</markp>: Dry; old; husky; withered; spiritless.</def> <ldquo/A somewhat <xex>sapless</xex> womanhood.<rdquo/ <rj><au>Lowell.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Now <qex>sapless</qex> on the verge of death he stands.</q> <rj><qau>Dryden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>sapling</ent><br/
<hw>sap"ling</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A young tree.</def> <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sapodilla</ent><br/
<hw>Sap`o*dil"la</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Sp. <ets>zapote</ets>, <ets>sapotillo</ets>, <ets>zapotillo</ets>, Mexican <ets>cochit-zapotl</ets>. Cf. <er>Sapota</er>.]</ety> <fld>(Bot.)</fld> <def>A tall, evergeen, tropical American tree (<spn>Achras Sapota</spn>); also, its edible fruit, the sapodilla plum.</def> <altsp>[Written also <asp>sapadillo</asp>, <asp>sappadillo</asp>, <asp>sappodilla</asp>, and <asp>zapotilla</asp>.]</altsp><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Sapodilla plum</b></col> <fld>(Bot.)</fld>, <cd>the fruit of <spn>Achras Sapota</spn>. It is about the size of an ordinary quince, having a rough, brittle, dull brown rind, the flesh being of a dirty yellowish white color, very soft, and deliciously sweet. Called also <altname>naseberry</altname>. It is eatable only when it begins to be spotted, and is much used in desserts.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sapogenin</ent><br/
<hw>Sa*pog"e*nin</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[<ets>Sapo</ets>nin + <ets>-gen</ets> + <ets>-in</ets>.]</ety> <fld>(Chem.)</fld> <def>A white crystalline substance obtained by the decomposition of saponin.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Saponaceous</ent><br/
<hw>Sap`o*na"ceous</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>sapo</ets>, <ets>-onis</ets>, soap, of Teutonic origin, and akin to E. <ets>soap</ets>. See <er>Soap</er>.]</ety> <def>Resembling soap; having the qualities of soap; soapy.</def><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ <xex>Saponaceous</xex> bodies are compounds of an acid and a base, and are in reality a kind of salt.</note><br/
[<source>1913 Webster</source>]</p>

<p><-- p. 1276 pr=PCP --></p>

<p><ent>Saponacity</ent><br/
<hw>Sap`o*nac"i*ty</hw> <pr>(?)</pr>, <pos>n.</pos> <def>The quality or state of being saponaceous.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Saponary</ent><br/
<hw>Sap"o*na*ry</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Saponaceous.</def> <rj><au>Boyle.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Saponifiable</ent><br/
<hw>Sa*pon*i*fi`a*ble</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Capable of conversion into soap; <as>as, a <ex>saponifiable</ex> substance</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Saponification</ent><br/
<hw>Sa*pon`i*fi*ca"tion</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Cf. F. <ets>saponification</ets>. See <er>Saponify</er>.]</ety> <def>The act, process, or result, of soap making; conversion into soap; specifically <fld>(Chem.)</fld>, the decomposition of fats and other ethereal salts by alkalies; <as>as, the <ex>saponification</ex> of ethyl acetate</as>.</def><-- "ethereal salt" = ester --><br/
[<source>1913 Webster</source>]</p>

<p><ent>Saponifier</ent><br/
<hw>Sa*pon"i*fi`er</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Chem.)</fld> <def>That which saponifies; any reagent used to cause saponification.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Saponify</ent><br/
<hw>Sa*pon"i*fy</hw> <pr>(?)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Saponified</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Saponifying</conjf> <pr>(?)</pr>.]</vmorph> <ety>[L. <ets>sapo</ets>, <ets>-onis</ets>, soap + <ets>-fy</ets>: cf. F. <ets>saponifier</ets>.]</ety> <def>To convert into soap, as tallow or any fat; hence <fld>(Chem.)</fld>, to subject to any similar process, as that which ethereal salts undergo in decomposition; <as>as, to <ex>saponify</ex> ethyl acetate</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Saponin</ent><br/
<hw>Sap"o*nin</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>sapo</ets>, <ets>-onis</ets> soap: cf. F. <ets>saponine</ets>.]</ety> <fld>(Chem.)</fld> <def>A poisonous glucoside found in many plants, as in the root of soapwort (<spn>Saponaria officinalis</spn>), in the bark of soap bark (<spn>Quillaja saponaria</spn>), etc.  It is extracted as a white amorphous powder, which produces a soapy lather in solution, and produces a local anaesthesia.  It is used as a detergent and for emulsifying oils.  Formerly called also <altname>struthiin</altname>, <altname>quillaiin</altname>, <altname>senegin</altname>, <altname>polygalic acid</altname>, etc. By extension, any one of a group of related bodies of which saponin proper is the type.</def><br/
[<source>1913 Webster</source> + <source>PJC</source>]</p>

<p><ent>Saponite</ent><br/
<hw>Sap"o*nite</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Sw. <ets>saponit</ets>, fr. L. <ets>sapo</ets>, <ets>-onis</ets>, soap.]</ety> <fld>(Min.)</fld> <def>A hydrous silicate of magnesia and alumina. It occurs in soft, soapy, amorphous masses, filling veins in serpentine and cavities in trap rock.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Saponul</ent><br/
<hw>Sap"o*nul</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F. <ets>saponule</ets>, fr. L. <ets>sapo</ets>, <ets>-onis</ets>, soap.]</ety> <fld>(Old Chem.)</fld> <def>A soapy mixture obtained by treating an essential oil with an alkali; hence, any similar compound of an essential oil.</def> <altsp>[Written also <asp>saponule</asp>.]</altsp> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sapor</ent><br/
||<hw>Sa"por</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. See <er>Savor</er>.]</ety> <def>Power of affecting the organs of taste; savor; flavor; taste.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>There is some <qex>sapor</qex> in all aliments.</q> <rj><qau>Sir T. Browne.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Saporific</ent><br/
<hw>Sap`o*rif"ic</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>sapor</ets> taste + <ets>facere</ets> to make.]</ety> <def>Having the power to produce the sensation of taste; producing taste, flavor, or relish.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Saporosity</ent><br/
<hw>Sap`o*ros"i*ty</hw> <pr>(?)</pr>, <pos>n.</pos> <def>The quality of a body by which it excites the sensation of taste.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Saporous</ent><br/
<hw>Sap"o*rous</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>saporus</ets> that relishes well, savory, fr. <ets>sapor</ets> taste.]</ety> <def>Having flavor or taste; yielding a taste.</def> <mark>[R.]</mark> <rj><au>Bailey.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sapota</ent><br/
<hw>Sa*po"ta</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[NL., from Sp. <ets>sapote</ets>, <ets>zapote</ets>. See <er>Sapodilla</er>.]</ety> <fld>(Bot.)</fld> <def>The sapodilla.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sapotaceous</ent><br/
<hw>Sap`o*ta"ceous</hw> <pr>(?)</pr>, <pos>a.</pos> <fld>(Bot.)</fld> <def>Of or pertaining to a natural order (<ord>Sapotaceae</ord>) of (mostly tropical) trees and shrubs, including the star apple, the Lucuma, or natural marmalade tree, the gutta-percha tree (<gen>Isonandra</gen>), and the India mahwa, as well as the sapodilla, or sapota, after which the order is named.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sappan wood</ent><br/
<hw>Sap*pan" wood"</hw> <pr>(?)</pr>. <def>Sapan wood.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sappare</ent><br/
<hw>Sap"pare</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F. <ets>sappare</ets>; -- so called by Saussure.]</ety> <fld>(Min.)</fld> <def>Kyanite.</def> <altsp>[Written also <asp>sappar</asp>.]</altsp><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sapper</ent><br/
<hw>Sap"per</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Cf. F. <ets>sapeur</ets>.]</ety> <def>One who saps; specifically <fld>(Mil.)</fld>, one who is employed in working at saps, building and repairing fortifications, and the like.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sapphic</ent><br/
<hw>Sap"phic</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>Sapphicus</ets>, Gr. <?/, fr. <?/ Sappho.]</ety> <sn>1.</sn> <def>Of or pertaining to Sappho, the Grecian poetess; <as>as, <ex>Sapphic</ex> odes; <ex>Sapphic</ex> verse</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Pros.)</fld> <def>Belonging to, or in the manner of, Sappho; -- said of a certain kind of verse reputed to have been invented by Sappho, consisting of five feet, of which the first, fourth, and fifth are trochees, the second is a spondee, and the third a dactyl.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sapphic</ent><br/
<hw>Sap"phic</hw>, <pos>n.</pos> <fld>(Pros.)</fld> <def>A Sapphic verse.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sapphire</ent><br/
<hw>Sap"phire</hw> <pr>(? <or/ ?; 277)</pr>, <pos>n.</pos> <ety>[OE. <ets>saphir</ets>, F. <ets>saphir</ets>, L. <ets>sapphirus</ets>, Gr. <?/, of Oriental origin; cf. Heb. <ets>sapp<imac/r</ets>.]</ety> <sn>1.</sn> <fld>(Min.)</fld> <def>Native alumina or aluminium sesquioxide, <chform>Al2O3</chform>; corundum; esp., the blue transparent variety of corundum, highly prized as a gem.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Of rubies, <qex>sapphires</qex>, and of pearl<eacute/s white.</q> <rj><qau>Chaucer.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ <xex>Sapphire</xex> occurs in hexagonal crystals and also in granular and massive forms. The name <xex>sapphire</xex> is usually restricted to the blue crystals, while the bright red crystals are called <xex>Oriental rubies</xex> (see under <er>Ruby</er>), the amethystine variety <xex>Oriental amethyst</xex> (see under <er>Amethyst</er>), and the dull massive varieties <xex>corundum</xex> (a name which is also used as a general term to include all varieties). See <er>Corundum</er>.</note><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>The color of the gem; bright blue.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <fld>(Zool.)</fld> <def>Any humming bird of the genus <gen>Hylocharis</gen>, native of South America. The throat and breast are usually bright blue.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><mcol><col><b>Star sapphire</b></col>, <it>or</it> <col><b>Asteriated sapphire</b></col></mcol> <fld>(Min.)</fld>, <cd>a kind of sapphire which exhibits asterism.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sapphire</ent><br/
<hw>Sap"phire</hw>, <pos>a.</pos> <def>Of or resembling sapphire; sapphirine; blue.</def> <ldquo/The <xex>sapphire</xex> blaze.<rdquo/ <rj><au>Gray.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sapphirine</ent><br/
<hw>Sap"phir*ine</hw> <pr>(?)</pr>, <pos>n.</pos> <def>Resembling sapphire; made of sapphire; having the color, or any quality of sapphire.</def> <ldquo/<xex>Sapphirine</xex> degree of hardness.<rdquo/ <rj><au>Boyle.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sappho</ent><br/
<hw>Sap"pho</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[See <er>Sapphic</er>.]</ety> <fld>(Zool.)</fld> <def>Any one of several species of brilliant South American humming birds of the genus <gen>Sappho</gen>, having very bright-colored and deeply forked tails; -- called also <altname>firetail</altname>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sappiness</ent><br/
<hw>Sap"pi*ness</hw> <pr>(?)</pr>, <pos>n.</pos> <def>The quality of being sappy; juiciness.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sappodilla</ent><br/
<hw>Sap`po*dil"la</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Bot.)</fld> <def>See <er>Sapodilla</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sappy</ent><br/
<hw>Sap"py</hw> <pr>(?)</pr>, <pos>a.</pos> <amorph>[<pos>Compar.</pos> <adjf>Sappier</adjf> <pr>(?)</pr>; <pos>superl.</pos> <adjf>Sappiest</adjf>.]</amorph> <ety>[From 1st <er>Sap</er>.]</ety><br/
[<source>1913 Webster</source>]</p>

<p><sn>1.</sn> <def>Abounding with sap; full of sap; juicy; succulent.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Hence, young, not firm; weak, feeble.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>When he had passed this weak and <qex>sappy</qex> age.</q> <rj><qau>Hayward.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>Weak in intellect.</def> <mark>[Low]</mark><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <fld>(Bot.)</fld> <def>Abounding in sap; resembling, or consisting largely of, sapwood.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sappy</ent><br/
<hw>Sap"py</hw> <pr>(?)</pr>, <pos>a.</pos> <altsp>[Written also <asp>sapy</asp>.]</altsp> <ety>[Cf. L. <ets>sapere</ets> to taste.]</ety> <def>Musty; tainted.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Saprophagan</ent><br/
<hw>Sa*proph"a*gan</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Gr. <grk>sapro`s</grk> rotten + <grk>fagei^n</grk> to eat: cf. F. <ets>saprophage</ets>.]</ety> <fld>(Zool.)</fld> <def>One of a tribe of beetles which feed upon decaying animal and vegetable substances; a carrion beetle.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Saprophagous</ent><br/
<hw>Sa*proph"a*gous</hw> <pr>(?)</pr>, <pos>a.</pos> <fld>(Zool.)</fld> <def>Feeding on carrion.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Saprophyte</ent><br/
<hw>Sap"ro*phyte</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Gr. <grk>sapro`s</grk> rotten + <grk>fyto`n</grk> a plant.]</ety> <fld>(Bot.)</fld> <def>Any plant growing on decayed animal or vegetable matter, as most fungi and some flowering plants with no green color, as the Indian pipe.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Saprophytic</ent><br/
<hw>Sap`ro*phyt"ic</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Feeding or growing upon decaying animal or vegetable matter; pertaining to a saprophyte or the saprophytes.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Saprophytism</ent><br/
<hw>Sap"ro*phyt*ism</hw> <pr>(?)</pr>, <pos>n.</pos> <def>State or fact of being saprophytic.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Sapsago</ent><br/
<hw>Sap"sa*go</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[G. <ets>schabzieger</ets>; <ets>schaben</ets> to shave, to scrape + <ets>zieger</ets> a sort of whey.]</ety> <def>A kind of Swiss cheese, of a greenish color, flavored with melilot.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sapskull</ent><br/
<hw>Sap"skull`</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A saphead.</def> <mark>[Low]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sapucaia</ent><br/
<hw>Sap`u*ca"ia</hw> <pr>(?; Pg. <?/)</pr>, <pos>n.</pos> <ety>[Pg. <ets>sapucaya</ets>.]</ety> <fld>(Bot.)</fld> <def>A Brazilian tree. See <er>Lecythis</er>, and <er>Monkey-pot</er>.</def> <altsp>[Written also <asp>sapucaya</asp>.]</altsp><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Sapucaia nut</b></col> <fld>(Bot.)</fld>, <cd>the seed of the sapucaia; -- called also <altname>paradise nut</altname>.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sapwood</ent><br/
<hw>Sap"wood`</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Bot.)</fld> <def>The alburnum, or part of the wood of any exogenous tree next to the bark, being that portion of the tree through which the sap flows most freely; -- distinguished from <contr>heartwood</contr>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sarabaite</ent><br/
<hw>Sar"a*ba*ite</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[LL. <ets>Saraba<ium/tae</ets>, pl.]</ety> <fld>(Eccl. Hist.)</fld> <def>One of certain vagrant or heretical Oriental monks in the early church.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Saraband</ent><br/
<hw>Sar"a*band</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F. <ets>sarabande</ets>, Sp. <ets>zarabanda</ets>, fr. Per. <ets>serbend</ets> a song.]</ety> <def>A slow Spanish dance of Saracenic origin, to an air in triple time; also, the air itself.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>She has brought us the newest <qex>saraband</qex> from the court of Queen Mab.</q> <rj><qau>Sir W. Scott.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Saracen</ent><br/
<hw>Sar"a*cen</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>Saracenus</ets> perhaps fr. Ar. <ets>sharqi</ets>, pl. <ets>sharqi<imac/n</ets>, Oriental, Eastern, fr. <ets>sharaqa</ets> to rise, said of the sun: cf. F. <ets>sarrasin</ets>. Cf. <er>Sarcenet</er>, <er>Sarrasin</er>, <er>Sirocco</er>.]</ety> <def>Anciently, an Arab; later, a Mussulman; in the Middle Ages, the common term among Christians in Europe for a Mohammedan hostile to the crusaders.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Saracens' consound</b></col> <fld>(Bot.)</fld>, <cd>a kind of ragwort (<spn>Senecio Saracenicus</spn>), anciently used to heal wounds.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Saracenical</ent><br/
<ent>Saracenic</ent><br/
<mhw>{ <hw>Sar`a*cen"ic</hw> <pr>(?)</pr>, <hw>Sar`a*cen"ic*al</hw> <pr>(?)</pr>, }</mhw> <pos>a.</pos> <def>Of or pertaining to the Saracens; <as>as, <ex>Saracenic</ex> architecture</as>.</def> <ldquo/<xex>Saracenic</xex> music.<rdquo/ <rj><au>Sir W. Scott.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sarasin</ent><br/
<hw>Sar"a*sin</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Arch.)</fld> <def>See <er>Sarrasin</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Saraswati</ent><br/
||<hw>Sa`ras*wa"ti</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Skr. <ets>Sarasvat<imac/</ets>.]</ety> <fld>(Hind. Myth.)</fld> <def>The sakti or wife of Brahma; the Hindoo goddess of learning, music, and poetry.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sarcasm</ent><br/
<hw>Sar"casm</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F. <ets>sarcasme</ets>, L. <ets>sarcasmus</ets>, Gr. <grk>sarkasmo`s</grk>, from <grk>sarka`zein</grk> to tear flesh like dogs, to bite the lips in rage, to speak bitterly, to sneer, fr. <grk>sa`rx</grk>, <grk>sa`rkos</grk>, flesh.]</ety> <def>A keen, reproachful expression; a satirical remark uttered with some degree of scorn or contempt; a taunt; a gibe; a cutting jest.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The <qex>sarcasms</qex> of those critics who imagine our art to be a matter of inspiration.</q> <rj><qau>Sir J. Reynolds.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><syn><b>Syn.</b> -- Satire; irony; ridicule; taunt; gibe.</syn><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sarcasmous</ent><br/
<hw>Sar*cas"mous</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Sarcastic.</def> <mark>[Obs.]</mark> <ldquo/<xex>Sarcasmous</xex> scandal.<rdquo/ <rj><au>Hubidras.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sarcastical</ent><br/
<ent>Sarcastic</ent><br/
<mhw>{ <hw>Sar*cas"tic</hw> <pr>(?)</pr>, <hw>Sar*cas"tic*al</hw> <pr>(?)</pr>, }</mhw> <pos>a.</pos> <def>Expressing, or expressed by, sarcasm; characterized by, or of the nature of, sarcasm; given to the use of sarcasm; bitterly satirical; scornfully severe; taunting.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>What a fierce and <qex>sarcastic</qex> reprehension would this have drawn from the friendship of the world!</q> <rj><qau>South.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sarcastically</ent><br/
<hw>Sar*cas"tic*al*ly</hw>, <pos>adv.</pos> <def>In a sarcastic manner.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sarcel</ent><br/
<hw>Sar"cel</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[OF. <ets>cercel</ets>, F. <ets>cerceau</ets>, L. <ets>circellus</ets>, dim. of <ets>circulus</ets>. See <er>Circle</er>.]</ety> <def>One of the outer pinions or feathers of the wing of a bird, esp. of a hawk.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sarceled</ent><br/
<hw>Sar"celed</hw> <pr>(?)</pr>, <pos>a.</pos> <fld>(her.)</fld> <def>Cut through the middle.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sarcelle</ent><br/
||<hw>Sar`celle"</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F., fr. L. <ets>querquedula</ets>.]</ety> <fld>(Zool.)</fld> <def>The old squaw, or long-tailed duck.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sarcenet</ent><br/
<hw>Sarce"net</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[OF. <ets>sarcenet</ets>; cf. LL. <ets>saracenicum</ets> cloth made by Saracens. See <er>Saracen</er>.]</ety> <def>A species of fine thin silk fabric, used for linings, etc.</def> <altsp>[Written also <asp>sarsenet</asp>.]</altsp><br/
[<source>1913 Webster</source>]</p>

<p><q>Thou green <qex>sarcenet</qex> flap for a sore eye.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sarcin</ent><br/
<hw>Sar"cin</hw> <pr>(?)</pr>, <pos>n.</pos> <def>Same as <er>Hypoxanthin</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sarcina</ent><br/
||<hw>Sar*ci"na</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[NL., fr. Gr. <?/ of flesh, fr. <grk>sa`rx</grk>, <grk>sa`rkos</grk>, flesh.]</ety> <fld>(Biol.)</fld> <def>A genus of bacteria found in various organic fluids, especially in those those of the stomach, associated with certain diseases. The individual organisms undergo division along two perpendicular partitions, so that multiplication takes place in two directions, giving groups of four cubical cells. Also used adjectively; <as>as, a <ex>sarcina</ex> micrococcus; a <ex>sarcina</ex> group</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Sarcina form</b></col> <fld>(Biol.)</fld>, <cd>the tetrad form seen in the division of a dumb-bell group of micrococci into four; -- applied particularly to bacteria. See <er>micrococcus</er>.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sarcle</ent><br/
<hw>Sar"cle</hw> <pr>(?)</pr>, <pos>v. t.</pos> <ety>[F. <ets>sarcler</ets> to weed, fr. L. <ets>sarculare</ets> to hoe, fr. <ets>sarculum</ets> hoe.]</ety> <def>To weed, or clear of weeds, with a hoe.</def> <mark>[Obs.]</mark> <rj><au>Ainsworth.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sarco-</ent><br/
<hw>Sar"co-</hw> <pr>(?)</pr>. <def>A combining form from Gr. <grk>sa`rx</grk>, <grk>sa`rkos</grk>, <xex>flesh</xex>; <as>as, <ex>sarco</ex>phagous, flesh-eating; <ex>sarco</ex>logy</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sarcobasis</ent><br/
||<hw>Sar*cob"a*sis</hw> <pr>(?)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Sarcobases</plw> <pr>(#)</pr>.</plu> <ety>[NL., fr. Gr. <grk>sa`rx</grk>, <grk>sa`rkos</grk>, flesh + <?/ base.]</ety> <fld>(Bot.)</fld> <def>A fruit consisting of many dry indehiscent cells, which contain but few seeds and cohere about a common style, as in the mallows.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sarcoblast</ent><br/
<hw>Sar"co*blast</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[<ets>Sarco-</ets> + <ets>-blast</ets>.]</ety> <fld>(Zool.)</fld> <def>A minute yellowish body present in the interior of certain rhizopods.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sarcocarp</ent><br/
<hw>Sar"co*carp</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[<ets>Sarco-</ets> + Gr. <?/ fruit: cf. F. <ets>sarcocarpe</ets>.]</ety> <fld>(Bot.)</fld> <def>The fleshy part of a stone fruit, situated between the skin, or epicarp, and the stone, or endocarp, as in a peach. See <xex>Illust.</xex> of <er>Endocarp</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ The term has also been used to denote any fruit which is fleshy throughout.</note> <rj><au>M. T. Masters.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sarcocele</ent><br/
<hw>Sar"co*cele</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Gr. <?/; <grk>sa`rx</grk>, <grk>sa`rkos</grk>, flesh + <?/ tumor: cf. F. <ets>sarcoc<egrave/le</ets>.]</ety> <fld>(Med.)</fld> <def>Any solid tumor of the testicle.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sarcocolla</ent><br/
<ent>Sarcocol</ent><br/
<mhw>{ <hw>Sar"co*col</hw> <pr>(?)</pr>, <hw>Sar`co*col"la</hw> <pr>(?)</pr>, }</mhw> <pos>n.</pos> <ety>[L. <ets>sarcocolla</ets>, from Gr. <?/; <grk>sa`rx</grk>, <grk>sa`rkos</grk>, flesh + <?/ glue: cf. F. <ets>sarcocolle</ets>.]</ety> <def>A gum resin obtained from certain shrubs of Africa (<gen>Penaea</gen>), -- formerly thought to cause healing of wounds and ulcers.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sarcode</ent><br/
<hw>Sar"code</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Gr. <?/ fleshy; <grk>sa`rx</grk>, flesh + <grk>e'i^dos</grk> form. Cf. <er>Sarcoid</er>.]</ety> <fld>(Biol.)</fld> <def>A name applied by Dujardin in 1835 to the gelatinous material forming the bodies of the lowest animals; protoplasm.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>sarcoderma</ent><br/
<ent>Sarcoderm</ent><br/
<mhw>{ <hw>Sar"co*derm</hw> <pr>(?)</pr>, ||<hw>sar`co*der"ma</hw> <pr>(?)</pr>, }</mhw> <pos>n.</pos> <ety>[NL. <ets>sarcoderma</ets>. See <er>Sarco-</er>, and <er>Derm</er>.]</ety> <fld>(Bot.)</fld> <sd>(a)</sd> <def>A fleshy covering of a seed, lying between the external and internal integuments.</def> <sd>(b)</sd> <def>A sarcocarp.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sarcodic</ent><br/
<hw>Sar*cod"ic</hw> <pr>(? <or/ ?)</pr>, <pos>a.</pos> <fld>(Biol.)</fld> <def>Of or pertaining to sarcode.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sarcoid</ent><br/
<hw>Sar"coid</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[Gr. <?/. See <er>Sarcode</er>.]</ety> <fld>(Biol.)</fld> <def>Resembling flesh, or muscle; composed of sarcode.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sarcolactic</ent><br/
<hw>Sar`co*lac"tic</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[<ets>Sarco-</ets> + <ets>lactic</ets>.]</ety> <fld>(Physiol. Chem.)</fld> <def>Relating to muscle and milk; <as>as, <ex>sarcolactic acid</ex></as>. See <cref>Lactic acid</cref>, under <er>Lactic</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sarcolemma</ent><br/
<hw>Sar`co*lem"ma</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[NL., from Gr. <grk>sa`rx</grk>, <grk>sa`rkos</grk>, flesh + <?/ rind, skin.]</ety> <fld>(Anat.)</fld> <def>The very thin transparent and apparently homogeneous sheath which incloses a striated muscular fiber; the myolemma.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sarcoline</ent><br/
<hw>Sar"co*line</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[Gr. <grk>sa`rx</grk>, <grk>sa`rkos</grk>, flesh.]</ety> <fld>(Min.)</fld> <def>Flesh-colored.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sarcological</ent><br/
<ent>Sarcologic</ent><br/
<mhw>{ <hw>Sar`co*log"ic</hw> <pr>(?)</pr>, <hw>Sar`co*log"ic*al</hw> <pr>(?)</pr>, }</mhw> <pos>a.</pos> <def>Of or pertaining to sarcology.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sarcology</ent><br/
<hw>Sar*col"o*gy</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[<ets>Sarco-</ets> + <ets>-logy</ets>: cf. F. <ets>sarcologie</ets>.]</ety> <def>That part of anatomy which treats of the soft parts. It includes myology, angiology, neurology, and splanchnology.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sarcoma</ent><br/
||<hw>Sar*co"ma</hw> <pr>(?)</pr>, <pos>n.</pos>; <plu><it>pl.</it> L. <plw>Sarcomata</plw> <pr>(# <or/ #)</pr>, E. <plw>sarcomas</plw> <pr>(#)</pr>.</plu> <ety>[NL., from Gr. <?/, from <grk>sa`rx</grk>, <grk>sa`rkos</grk>, flesh.]</ety> <fld>(Med.)</fld> <def>A tumor of fleshy consistence; -- formerly applied to many varieties of tumor, now restricted to a variety of malignant growth made up of cells resembling those of fetal development without any proper intercellular substance.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sarcomatous</ent><br/
<hw>Sar*com"a*tous</hw> <pr>(? <or/ ?)</pr>, <pos>a.</pos> <fld>(Med.)</fld> <def>Of or pertaining to sarcoma; resembling sarcoma.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sarcophaga</ent><br/
||<hw>Sar*coph"a*ga</hw> <pr>(?)</pr>, <pos>n. pl.</pos> <ety>[NL., neut. pl. See <er>Sarcophagus</er>.]</ety> <fld>(Zool.)</fld> <def>A suborder of carnivorous and insectivorous marsupials including the dasyures and the opossums.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sarcophaga</ent><br/
||<hw>Sar*coph"a*ga</hw>, <pos>n.</pos> <ety>[NL., fem. sing. See <er>Sarcophagus</er>.]</ety> <fld>(Zool.)</fld> <def>A genus of Diptera, including the flesh flies.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sarcophagan</ent><br/
<hw>Sar*coph"a*gan</hw> <pr>(?)</pr>, <pos>n.</pos> <sn>1.</sn> <fld>(Zool.)</fld> <def>Any animal which eats flesh, especially any carnivorous marsupial.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Zool.)</fld> <def>Any fly of the genus Sarcophaga.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sarcophagous</ent><br/
<hw>Sar*coph"a*gous</hw> <pr>(?)</pr>, <pos>a.</pos> <fld>(Zool.)</fld> <def>Feeding on flesh; flesh-eating; carnivorous.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sarcophagus</ent><br/
<hw>Sar*coph"a*gus</hw> <pr>(?)</pr>, <pos>n.</pos>; <plu><it>pl.</it> L. <plw>Sarcophagi</plw> <pr>(#)</pr>, E. <plw>Sarcophaguses</plw> <pr>(#)</pr>.</plu> <ety>[L., fr. Gr. <grk>sarkofa`gos</grk>, properly, eating flesh; <grk>sa`rx</grk>, <grk>sa`rkos</grk>, flesh + <grk>fagei^n</grk> to eat. Cf. <er>Sarcasm</er>.]</ety> <sn>1.</sn> <def>A species of limestone used among the Greeks for making coffins, which was so called because it consumed within a few weeks the flesh of bodies deposited in it. It is otherwise called <altname>lapis Assius</altname>, or <altname>Assian stone</altname>, and is said to have been found at Assos, a city of Lycia.</def> <rj><au>Holland.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A coffin or chest-shaped tomb of the kind of stone described above; hence, any stone coffin.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>A stone shaped like a sarcophagus and placed by a grave as a memorial.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sarcophagy</ent><br/
<hw>Sar*coph"a*gy</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Gr. <grk>sarkofagi`a</grk>. See <er>Sarcophagus</er>.]</ety> <def>The practice of eating flesh.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sarcophile</ent><br/
<hw>Sar"co*phile</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[<ets>Sacro-</ets> + Gr. <?/ a lover.]</ety> <fld>(Zool.)</fld> <def>A flesh-eating animal, especially any one of the carnivorous marsupials.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sarcoptes</ent><br/
||<hw>Sar*cop"tes</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[NL., from Gr. <grk>sa`rx</grk>, <grk>sa`rkos</grk>, flesh + <grk>ko`ptein</grk> to cut.]</ety> <fld>(Zool.)</fld> <def>A genus of parasitic mites including the itch mites.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sarcoptid</ent><br/
<hw>Sar*cop"tid</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Zool.)</fld> <def>Any species of the genus <gen>Sarcoptes</gen> and related genera of mites, comprising the itch mites and mange mites.</def> -- <def2><pos>a.</pos> <def>Of or pertaining to the itch mites.</def></def2><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sarcorhamphi</ent><br/
||<hw>Sar`co*rham"phi</hw> <pr>(?)</pr>, <pos>n. pl.</pos> <ety>[NL., fr. Gr. <grk>sa`rx</grk>, <grk>sa`rkos</grk>, flesh + <?/ beak.]</ety> <fld>(Zool.)</fld> <def>A division of raptorial birds comprising the vultures.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sarcoseptum</ent><br/
||<hw>Sar`co*sep"tum</hw> <pr>(?)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Sarcosepta</plw> <pr>(#)</pr>.</plu> <ety>[<ets>Sarco-</ets> + <ets>septum</ets>.]</ety> <fld>(Zool.)</fld> <def>One of the mesenteries of an anthozoan.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sarcosin</ent><br/
<hw>Sar"co*sin</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Physiol. Chem.)</fld> <def>A crystalline nitrogenous substance, formed in the decomposition of creatin (one of the constituents of muscle tissue). Chemically, it is methyl glycocoll.</def><-- N-Methylglycine, C3H7NO2. --><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sarcosis</ent><br/
||<hw>Sar*co"sis</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[NL., fr. Gr. <?/, fr. <grk>sa`rx</grk>, <grk>sa`rkos</grk>, flesh.]</ety> <fld>(Med.)</fld> <sd>(a)</sd> <def>Abnormal formation of flesh.</def> <sd>(b)</sd> <def>Sarcoma.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sarcotic</ent><br/
<hw>Sar*cot"ic</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[Gr. <?/: cf. F. <ets>sarcotique</ets>.]</ety> <fld>(Med.)</fld> <def>Producing or promoting the growth of flesh.</def> <mark>[R.]</mark> -- <def2><pos>n.</pos> <def>A sarcotic medicine.</def></def2> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sarcous</ent><br/
<hw>Sar"cous</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[Gr. <grk>sa`rx</grk>, <grk>sa`rkos</grk>, flesh.]</ety> <fld>(Anat.)</fld> <def>Fleshy; -- applied to the minute structural elements, called <xex>sarcous elements</xex>, or <xex>sarcous disks</xex>, of which striated muscular fiber is composed.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sarculation</ent><br/
<hw>Sar`cu*la"tion</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>sarculatio</ets>. See <er>Sarcle</er>.]</ety> <def>A weeding, as with a hoe or a rake.</def><br/
[<source>1913 Webster</source>]</p>

<p><-- p. 1277 pr=PCP --></p>

<p><ent>Sard</ent><br/
<hw>Sard</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>sarda</ets>, Gr. <?/, or <?/ (sc. <?/), i.e., Sardian stone, fr. <?/ Sardian, <?/ Sardes, the capital of Lydia: cf. F. <ets>sarde</ets>. Cf. <er>Sardius</er>.]</ety> <fld>(Min.)</fld> <def>A variety of carnelian, of a rich reddish yellow or brownish red color. See the Note under <xex>Chalcedony</xex>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sardachate</ent><br/
<hw>Sar"da*chate</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>sardachates</ets>: cf. F. <ets>sardachate</ets>. See <er>Sard</er>, and <er>Agate</er>.]</ety> <fld>(Min.)</fld> <def>A variety of agate containing sard.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sardel</ent><br/
<ent>Sardan</ent><br/
<mhw>{ <hw>Sar"dan</hw> <pr>(?)</pr>, <hw>Sar"del</hw> <pr>(?)</pr>, }</mhw> <pos>n.</pos> <ety>[It. <ets>sardella</ets>. See <er>Sardine</er> a fish.]</ety> <fld>(Zool.)</fld> <def>A sardine.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sardel</ent><br/
<hw>Sar"del</hw>, <pos>n.</pos> <def>A precious stone. See <er>Sardius</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sardine</ent><br/
<hw>Sar"dine</hw> <pr>(? <or/ ?; 277)</pr>, <pos>n.</pos> <ety>[F. <ets>sardine</ets> (cf. Sp. <ets>sardina</ets>, <ets>sarda</ets>, It. <ets>sardina</ets>, <ets>sardella</ets>), L. <ets>sardina</ets>, <ets>sarda</ets>; cf. Gr. <?/, <?/; so called from the island of <etsep>Sardinia</etsep>, Gr. <?/.]</ety> <fld>(Zool.)</fld> <def>Any one of several small species of herring which are commonly preserved in olive oil for food, especially the pilchard, or European sardine (<spn>Clupea pilchardus</spn>). The California sardine (<spn>Clupea sagax</spn>) is similar. The American sardines of the Atlantic coast are mostly the young of the common herring and of the menhaden.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sardine</ent><br/
<hw>Sar"dine</hw> <pr>(? <or/ ?; 277)</pr>, <pos>n.</pos> <def>See <er>Sardius</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sardinian</ent><br/
<hw>Sar*din"i*an</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>Sardinianus</ets>.]</ety> <def>Of or pertaining to the island, kingdom, or people of Sardinia.</def> -- <def2><pos>n.</pos> <def>A native or inhabitant of Sardinia.</def></def2><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sardius</ent><br/
<hw>Sar"di*us</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>sardius</ets>, <ets>lapis sardinus</ets>, Gr. <?/, <?/, <?/. See <er>Sard</er>.]</ety> <def>A precious stone, probably a carnelian, one of which was set in Aaron's breastplate.</def> <rj><au>Ex. xxviii. 17.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sardoin</ent><br/
<hw>Sar"doin</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Cf. F. <ets>sardoine</ets>.]</ety> <fld>(Min.)</fld> <def>Sard; carnelian.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sardonian</ent><br/
<hw>Sar*do"ni*an</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[Cf. F. <ets>sardonien</ets>.]</ety> <def>Sardonic.</def> <mark>[Obs.]</mark> <ldquo/With <xex>Sardonian</xex> smile.<rdquo/ <rj><au>Spenser.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sardonic</ent><br/
<hw>Sar*don"ic</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[F. <ets>sardonique</ets>, L. <ets>sardonius</ets>, Gr. <?/, <?/, perhaps fr. <?/ to grin like a dog, or from a certain plant of <ets>Sardinia</ets>, Gr. <?/, which was said to screw up the face of the eater.]</ety> <def>Forced; unnatural; insincere; hence, derisive, mocking, malignant, or bitterly sarcastic; -- applied only to a laugh, smile, or some facial semblance of gayety.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Where strained, <qex>sardonic</qex> smiles are glozing still,<br/
And grief is forced to laugh against her will.</q> <rj><qau>Sir H. Wotton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>The scornful, ferocious, <qex>sardonic</qex> grin of a bloody ruffian.</q> <rj><qau>Burke.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><mcol><col><b>Sardonic grin</b></col> <it>or</it> <col><b>Sardonic laugh</b></col></mcol>, <cd>an old medical term for a spasmodic affection of the muscles of the face, giving it an appearance of laughter.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sardonic</ent><br/
<hw>Sar*don"ic</hw>, <pos>a.</pos> <def>Of, pertaining to, or resembling, a kind of linen made at Colchis.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sardonyx</ent><br/
<hw>Sar"do*nyx</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L., fr. Gr. <?/. See <er>Sard</er>, and <er>Onyx</er>.]</ety> <fld>(Min.)</fld> <def>A variety of onyx consisting of sard and white chalcedony in alternate layers.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Saree</ent><br/
||<hw>Sa"ree</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Hind. <?/.]</ety> <def>The principal garment of a Hindoo woman. It consists of a long piece of cloth, which is wrapped round the middle of the body, a portion being arranged to hang down in front, and the remainder passed across the bosom over the left shoulder.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sargasso</ent><br/
<hw>Sar*gas"so</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Sp. <ets>sargazo</ets> seaweed.]</ety> <fld>(Bot.)</fld> <def>The gulf weed. See under <er>Gulf</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Sargasso Sea</b></col>, <cd>a large tract of the North Atlantic Ocean where sargasso in great abundance floats on the surface.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sargassum</ent><br/
||<hw>Sar*gas"sum</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[NL.]</ety> <def>A genus of algae including the gulf weed.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sargo</ent><br/
||<hw>Sar"go</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Sp. <ets>sargo</ets>, L. <ets>sargus</ets> a kind of fish.]</ety> <fld>(Zool.)</fld> <def>Any one of several species of sparoid fishes belonging to <gen>Sargus</gen>, <gen>Pomadasys</gen>, and related genera; -- called also <altname>sar</altname>, and <altname>saragu</altname>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sari</ent><br/
||<hw>Sa"ri</hw> <pr>(?)</pr>, <pos>n.</pos> <def>Same as <er>Saree</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sarigue</ent><br/
||<hw>Sa*rigue"</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F., from Braz. <ets><cced/arigueia</ets>, <ets><cced/arigueira</ets>.]</ety> <fld>(Zool.)</fld> <def>A small South American opossum (<spn>Didelphys opossum</spn>), having four white spots on the face.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sark</ent><br/
<hw>Sark</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[AS. <ets>serce</ets>, <ets>syrce</ets>, a shirt; akin to Icel. <ets>serkr</ets>, Sw. <ets>s<aum/rk</ets>.]</ety> <def>A shirt.</def> <mark>[Scot.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sark</ent><br/
<hw>Sark</hw>, <pos>v. t.</pos> <fld>(Carp.)</fld> <def>To cover with sarking, or thin boards.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sarkin</ent><br/
<hw>Sar"kin</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Gr. <grk>sa`rx</grk>, <grk>sa`rkos</grk>, flesh.]</ety> <fld>(Physiol. Chem.)</fld> <def>Same as <er>Hypoxanthin</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sarking</ent><br/
<hw>Sark"ing</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[From <er>Sark</er> shirt.]</ety> <fld>(Carp.)</fld> <def>Thin boards for sheathing, as above the rafters, and under the shingles or slates, and for similar purposes.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sarlyk</ent><br/
<ent>Sarlac</ent><br/
<mhw>{ <hw>Sar"lac</hw> <pr>(?)</pr>, <hw>Sar"lyk</hw> <pr>(?)</pr>, }</mhw> <pos>n.</pos> <ety>[Mongolian <ets>sarlyk</ets>.]</ety> <fld>(Zool.)</fld> <def>The yak.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sarmatic</ent><br/
<ent>Sarmatian</ent><br/
<mhw>{ <hw>Sar*ma"tian</hw> <pr>(?)</pr>, <hw>Sar*mat"ic</hw> <pr>(?)</pr>, }</mhw> <pos>a.</pos> <ety>[L. <ets>Sarmaticus</ets>.]</ety> <def>Of or pertaining to Sarmatia, or its inhabitants, the ancestors of the Russians and the Poles.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sarment</ent><br/
<hw>Sar"ment</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>sarmentum</ets> a twig, fr. <ets>sarpere</ets> to cut off, to trim: cf. F. <ets>sarment</ets>.]</ety> <fld>(Bot.)</fld> <def>A prostrate filiform stem or runner, as of the strawberry. See <er>Runner</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sarmentaceous</ent><br/
<hw>Sar`men*ta"ceous</hw> <pr>(?)</pr>, <pos>a.</pos> <fld>(Bot.)</fld> <def>Bearing sarments, or runners, as the strawberry.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sarmentose</ent><br/
<hw>Sar`men*tose"</hw> <pr>(? <or/ ?)</pr>, <pos>a.</pos> <ety>[L. <ets>sarmentosus</ets>: cf. F. <ets>sarmenteux</ets>. See <er>Sarment</er>.]</ety> <fld>(Bot.)</fld> <sd>(a)</sd> <def>Long and filiform, and almost naked, or having only leaves at the joints where it strikes root; <as>as, a <ex>sarmentose</ex> stem</as>.</def> <sd>(b)</sd> <def>Bearing sarments; sarmentaceous.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sarmentous</ent><br/
<hw>Sar*men"tous</hw> <pr>(?)</pr>, <pos>a.</pos> <fld>(Bot.)</fld> <def>Sarmentose.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sarn</ent><br/
<hw>Sarn</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[W. <ets>sarn</ets> a causeway, paving.]</ety> <def>A pavement or stepping-stone.</def> <mark>[Prov. Eng.]</mark> <rj><au>Johnson.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sarong</ent><br/
||<hw>Sa"rong</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Malay <ets>s<amac/rung</ets>.]</ety> <def>A sort of petticoat worn by both sexes in Java and the Malay Archipelago.</def> <rj><au>Balfour (Cyc. of India)</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Saros</ent><br/
<hw>Sa"ros</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[NL., fr. Gr. <?/]</ety> <fld>(Astron)</fld> <def>A Chaldean astronomical period or cycle, the length of which has been variously estimated from 3,600 years to 3,600 days, or a little short of 10 years.</def> <rj><au>Brande & C.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><-- A length of time (6535.82 days, or 18 years 11.32 days, assuming 4 leap years in that interval), after which the eclipses of the sun repeat their pattern, but are shifted 120<deg/ west.. --><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sarplar</ent><br/
<hw>Sar"plar</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Cf. LL. <ets>sarplare</ets>. See <er>Sarplier</er>.]</ety> <def>A large bale or package of wool, containing eighty tods, or 2,240 pounds, in weight.</def> <mark>[Eng.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sarplier</ent><br/
<hw>Sar"plier</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F. <ets>serpilli<egrave/re</ets>; cf. Pr. <ets>sarpelheira</ets>, LL. <ets>serpelleria</ets>, <ets>serpleria</ets>, Catalan <ets>sarpallera</ets>, Sp. <ets>arpillera</ets>.]</ety> <def>A coarse cloth made of hemp, and used for packing goods, etc.</def> <altsp>[Written also <asp>sarpelere</asp>.]</altsp> <rj><au>Tyrwhitt.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sarpo</ent><br/
<hw>Sar"po</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Corruption of Sp. <ets>sapo</ets> a toad.]</ety> <fld>(Zool.)</fld> <def>A large toadfish of the Southern United States and the Gulf of Mexico (<spn>Batrachus tau</spn>, var. <var>pardus</var>).</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sarracenia</ent><br/
||<hw>Sar`ra*ce"ni*a</hw> <pr>(?)</pr>, <pos>prop. n.</pos> <ety>[NL. So named after a Dr. <ets>Sarrazin</ets> of Quebec.]</ety> <fld>(Bot.)</fld> <def>A genus of American perennial herbs growing in bogs; the American pitcher plant.</def><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ They have hollow pitcher-shaped or tubular leaves, and solitary flowers with an umbrella-shaped style. <spn>Sarracenia purpurea</spn>, the sidesaddle flower, is common at the North; <spn>Sarracenia flava</spn>, <spn>Sarracenia rubra</spn>, <spn>Sarracenia Drummondii</spn>, <spn>Sarracenia variolaris</spn>, and <spn>Sarracenia psittacina</spn> are Southern species.  All are insectivorous, catching and drowning insects in their curious leaves. See <er>Illust</er>. of Sidesaddle flower, under <er>Sidesaddle</er>.</note><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sarrasine</ent><br/
<ent>Sarrasin</ent><br/
<mhw>{ <hw>Sar"ra*sin</hw>, <hw>Sar"ra*sine</hw> }</mhw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F. <ets>sarrasine</ets>, LL. <ets>saracina</ets>. See <er>Saracen</er>.]</ety> <fld>(Fort.)</fld> <def>A portcullis, or herse.</def> <altsp>[Written also <asp>sarasin</asp>.]</altsp><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sarsa</ent><br/
<hw>Sar"sa</hw> <pr>(?)</pr>, <pos>n.</pos> <def>Sarsaparilla.</def> <altsp>[Written also <asp>sarza</asp>.]</altsp><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sarsaparilla</ent><br/
<hw>Sar`sa*pa*ril"la</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Sp. <ets>zarzaparrilla</ets>; <ets>zarza</ets> a bramble (perhaps fr. Bisc. <ets>zartzia</ets>) + <ets>parra</ets> a vine, or <ets>Parillo</ets>, a physician said to have discovered it.]</ety> <fld>(Bot.)</fld> <sd>(a)</sd> <def>Any plant of several tropical American species of <gen>Smilax</gen>.</def> <sd>(b)</sd> <def>The bitter mucilaginous roots of such plants, used in medicine and in sirups for soda, etc.</def><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ The name is also applied to many other plants and their roots, especially to the <spn>Aralia nudicaulis</spn>, the wild sarsaparilla of the United States.</note><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sarsaparillin</ent><br/
<hw>Sar`sa*pa*ril"lin</hw> <pr>(?)</pr>, <pos>n.</pos> <def>See <er>Parillin</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sarse</ent><br/
<hw>Sarse</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F. <ets>sas</ets>, OF. <ets>saas</ets>, LL. <ets>setatium</ets>, fr. L. <ets>seta</ets> a stiff hair.]</ety> <def>A fine sieve; a searce.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sarse</ent><br/
<hw>Sarse</hw>, <pos>v. t.</pos> <def>To sift through a sarse.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sarsen</ent><br/
<hw>Sar"sen</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Etymol. uncertain; perhaps for <ets>saracen stone</ets>, i.e., a heathen or pagan stone or monument.]</ety> <def>One of the large sandstone blocks scattered over the English chalk downs; -- called also <altname>sarsen stone</altname>, and <altname>Druid stone</altname>.</def> <mark>[Eng.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sarsenet</ent><br/
<hw>Sarse"net</hw> <pr>(?)</pr>, <pos>n.</pos> <def>See <er>Sarcenet</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sart</ent><br/
<hw>Sart</hw> <pr>(?)</pr>, <pos>n.</pos> <def>An assart, or clearing.</def> <mark>[Obs.]</mark> <rj><au>Bailey.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sartorial</ent><br/
<hw>Sar*to"ri*al</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[See <er>Sartorius</er>.]</ety> <sn>1.</sn> <def>Of or pertaining to a tailor or his work.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Our legs skulked under the table as free from <qex>sartorial</qex> impertinences as those of the noblest savages.</q> <rj><qau>Lowell.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Anat.)</fld> <def>Of or pertaining to the sartorius muscle.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sartorius</ent><br/
||<hw>Sar*to"ri*us</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[NL., fr. L. <ets>sartor</ets> a patcher, tailor, fr. <ets>sarcire</ets>, <ets>sartum</ets>, to patch, mend.]</ety> <fld>(Anat.)</fld> <def>A muscle of the thigh, called the <xex>tailor's muscle</xex>, which arises from the hip bone and is inserted just below the knee. So named because its contraction was supposed to produce the position of the legs assumed by the tailor in sitting.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sarum use</ent><br/
<hw>Sa"rum use`</hw> <pr>(?)</pr>. <fld>(Ch. of Eng.)</fld> <def>A liturgy, or <xex>use</xex>, put forth about 1087 by St. Osmund, bishop of <xex>Sarum</xex>, based on Anglo-Saxon and Norman customs.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sash</ent><br/
<hw>Sash</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Pers. <ets>shast</ets> a sort of girdle.]</ety> <def>A scarf or band worn about the waist, over the shoulder, or otherwise; a belt; a girdle, -- worn by women and children as an ornament; also worn as a badge of distinction by military officers, members of societies, etc.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sash</ent><br/
<hw>Sash</hw>, <pos>v. t.</pos> <def>To adorn with a sash or scarf.</def> <rj><au>Burke.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sash</ent><br/
<hw>Sash</hw>, <pos>n.</pos> <ety>[F. <ets>ch<acir/ssis</ets> a frame, sash, fr. <ets>ch<acir/sse</ets> a shrine, reliquary, frame, L. <ets>capsa</ets>. See <er>Case</er> a box.]</ety> <sn>1.</sn> <def>The framing in which the panes of glass are set in a glazed window or door, including the narrow bars between the panes.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>In a sawmill, the rectangular frame in which the saw is strained and by which it is carried up and down with a reciprocating motion; -- also called <altname>gate</altname>.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>French sash</b></col>, <cd>a casement swinging on hinges; -- in distinction from a <xex>vertical sash</xex> sliding up and down.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sash</ent><br/
<hw>Sash</hw>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Sashed</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Sashing</conjf>.]</vmorph> <def>To furnish with a sash or sashes; <as>as, to <ex>sash</ex> a door or a window</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sashery</ent><br/
<hw>Sash"er*y</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[From 1st <er>Sash</er>.]</ety> <def>A collection of sashes; ornamentation by means of sashes.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>Distinguished by their <qex>sasheries</qex> and insignia.</q> <rj><qau>Carlyle.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sashoon</ent><br/
<hw>Sash"oon</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Etymology uncertain.]</ety> <def>A kind of pad worn on the leg under the boot.</def> <mark>[Obs.]</mark> <rj><au>Nares.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sasin</ent><br/
<hw>Sa"sin</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Zool.)</fld> <def>The Indian antelope (<spn>Antilope bezoartica</spn> syn.  <spn>Antilope cervicapra</spn>), noted for its beauty and swiftness. It has long, spiral, divergent horns.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sassabye</ent><br/
<ent>Sassaby</ent><br/
<mhw>{ <hw>Sas"sa*by</hw> <pr>(?)</pr>, <hw>Sas"sa*bye</hw> <pr>(?)</pr>, }</mhw> <pos>n.</pos> <fld>(Zool.)</fld> <def>A large African antelope (<spn>Alcelaphus lunata</spn>), similar to the hartbeest, but having its horns regularly curved.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sassafras</ent><br/
<hw>Sas"sa*fras</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F. <ets>sassafras</ets> (cf. It. <ets>sassafrasso</ets>, <ets>sassafras</ets>, Sp. <ets>sasafras</ets>, <ets>salsafras</ets>, <ets>salsifrax</ets>, <ets>salsifragia</ets>, <ets>saxifragia</ets>), fr. L. <ets>saxifraga</ets> saxifrage. See <er>Saxifrage</er>.]</ety> <fld>(Bot.)</fld> <def>An American tree of the Laurel family (<spn>Sassafras officinale</spn>); also, the bark of the roots, which has an aromatic smell and taste.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Australian sassafras</b></col>, <cd>a lofty tree (<spn>Doryophora Sassafras</spn>) with aromatic bark and leaves.</cd> -- <col><b>Chilian sassafras</b></col>, <cd>an aromatic tree (<spn>Laurelia sempervirens</spn>).</cd> -- <col><b>New Zealand sassafras</b></col>, <cd>a similar tree (<spn>Laurelia Novae Zelandiae</spn>).</cd> -- <col><b>Sassafras nut</b></col>. <cd>See <er>Pichurim bean</er>.</cd> -- <col><b>Swamp sassafras</b></col>, <cd>the sweet bay (<spn>Magnolia glauca</spn>). See <er>Magnolia</er>.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sassanage</ent><br/
<hw>Sas"sa*nage</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[See <er>Sarse</er> a sieve.]</ety> <def>Stones left after sifting.</def> <rj><au>Smart.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sassarara</ent><br/
<hw>Sas`sa*ra"ra</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Perh. a corruption of <ets>certiorari</ets>, the name of a writ.]</ety> <def>A word used to emphasize a statement.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>Out she shall pack, with a <qex>sassarara</qex>.</q> <rj><qau>Goldsmith.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sasse</ent><br/
<hw>Sasse</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[D. <ets>sas</ets>, fr. F. <ets>sas</ets> the basin of a waterfall.]</ety> <def>A sluice or lock, as in a river, to make it more navigable.</def> <mark>[Obs.]</mark> <rj><au>Pepys.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sassenach</ent><br/
<hw>Sas"sen*ach</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Gael. <ets>sasunnach</ets>.]</ety> <def>A Saxon; an Englishman; a Lowlander.</def> <mark>[Celtic]</mark> <rj><au>Sir W. Scott.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sassoline</ent><br/
<ent>Sassolin</ent><br/
<mhw>{ <hw>Sas"so*lin</hw> <pr>(?)</pr>, <hw>Sas"so*line</hw> <pr>(?)</pr>, }</mhw> <pos>n.</pos> <ety>[From <ets>Sasso</ets>, a town in Italy: cf. F. <ets>sassolin</ets>.]</ety> <fld>(Min.)</fld> <def>Native boric acid, found in saline incrustations on the borders of hot springs near Sasso, in the territory of Florence.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sassorolla</ent><br/
<ent>Sassorol</ent><br/
<mhw>{ <hw>Sas"so*rol</hw> <pr>(?)</pr>, <hw>Sas`so*rol"la</hw> <pr>(?)</pr>, }</mhw> <pos>n.</pos> <fld>(Zool.)</fld> <def>The rock pigeon. See under <er>Pigeon</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sassy bark</ent><br/
<hw>Sas"sy bark`</hw> <pr>(?)</pr>. <fld>(Bot.)</fld> <def>The bark of a West African leguminous tree (<spn>Erythrophlaeum Guineense</spn>, used by the natives as an ordeal poison, and also medicinally; -- called also <altname>mancona bark</altname>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sastra</ent><br/
||<hw>Sas"tra</hw> <pr>(?)</pr>, <pos>n.</pos> <def>Same as <er>Shaster</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sastrugi</ent><br/
<hw>Sas*tru"gi</hw> <pr>(?)</pr>. <def>Incorrect, but common, var. of <er>Zastrugi</er>.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Sat</ent><br/
<hw>Sat</hw> <pr>(s<acr/t)</pr>, <def><pos>imp.</pos> of <er>Sit</er>.</def> <altsp>[Written also <asp>sate</asp>.]</altsp><br/
[<source>1913 Webster</source>]</p>

<p><ent>Satan</ent><br/
<hw>Sa"tan</hw> <pr>(s<amac/"t<ait/n, s<acr/t"<ait/n <mark>obs</mark> )</pr>, <pos>n.</pos> <ety>[Heb. <ets>s<amac/t<amac/n</ets> an adversary, fr. <ets>s<amac/tan</ets> to be adverse, to persecute: cf. Gr. <grk>Sata^n</grk>, <grk>Satana^s</grk>, L. <ets>Satan</ets>, <ets>Satanas</ets>.]</ety> <def>The grand adversary of man; the Devil, or Prince of darkness; the chief of the fallen angels; the archfiend.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>I beheld <qex>Satan</qex> as lightning fall from heaven.</q> <rj><qau>Luke x. 18.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Satanical</ent><br/
<ent>Satanic</ent><br/
<mhw>{ <hw>Sa*tan"ic</hw> <pr>(?)</pr>, <hw>Sa*tan"ic*al</hw> <pr>(?)</pr>, }</mhw> <pos>a.</pos> <ety>[Cf. F. <ets>satanique</ets>, Gr. <?/.]</ety> <def>Of or pertaining to Satan; having the qualities of Satan; resembling Satan; extremely malicious or wicked; devilish; infernal.</def> <ldquo/<xex>Satanic</xex> strength.<rdquo/ <ldquo/<xex>Satanic</xex> host.<rdquo/ <rj><au>Milton.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Detest the slander which, with a <qex>Satanic</qex> smile, exults over the character it has ruined.</q> <rj><qau>Dr. T. Dwight.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p>-- <wordforms><wf>Sa*tan"ic*al*ly</wf>, <pos>adv.</pos> -- <wf>Sa*tan"ic*al*ness</wf>, <pos>n.</pos></wordforms><br/
[<source>1913 Webster</source>]</p>

<p><ent>Satanism</ent><br/
<hw>Sa"tan*ism</hw> <pr>(?)</pr>, <pos>n.</pos> <sn>1.</sn> <def>The evil and malicious disposition of Satan; a diabolical spirit.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>The worship of Satan.</def><br/
[<source>PJC</source>]</p>

<p><ent>Satanist</ent><br/
<hw>Sa"tan*ist</hw>, <pos>n.</pos> <sn>1.</sn> <def>A very wicked person.</def> <mark>[R.]</mark> <rj><au>Granger.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A worshiper of Satan.</def><br/
[<source>PJC</source>]</p>

<p><ent>Satanophany</ent><br/
<hw>Sa`tan*oph"a*ny</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[<ets>Satan</ets> + Gr. <?/ to appear.]</ety> <def>An incarnation of Satan; a being possessed by a demon.</def> <mark>[R.]</mark> <rj><au>O. A. Brownson.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Satchel</ent><br/
<hw>Satch"el</hw> <pr>(?)</pr> <pos>n.</pos> <ety>[OF. <ets>sachel</ets>, fr. L. <ets>saccellus</ets>, dim. of <ets>saccus</ets>. See <er>Sack</er> a bag.]</ety> <def>A little sack or bag for carrying papers, books, or small articles of wearing apparel; a hand bag.</def> <altsp>[Spelled also <asp>sachel</asp>.]</altsp><br/
[<source>1913 Webster</source>]</p>

<p><q>The whining schoolboy with his <qex>satchel</qex>.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sate</ent><br/
<hw>Sate</hw> <pr>(?)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Sated</conjf>; <pos>p. pr. & vb. n.</pos> <conjf>Sating</conjf>.]</vmorph> <ety>[Probably shortened fr. <ets>satiate</ets>: cf. L. <ets>satur</ets> full. See <er>Satiate</er>.]</ety> <def>To satisfy the desire or appetite of; to satiate; to glut; to surfeit.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Crowds of wanderers <qex>sated</qex> with the business and pleasure of great cities.</q> <rj><qau>Macaulay.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sate</ent><br/
<hw>Sate</hw> <pr>(?)</pr>, <def><pos>imp.</pos> of <er>Sit</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>But <qex>sate</qex> an equal guest at every board.</q> <rj><qau>Lowell.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sateen</ent><br/
<hw>Sat*een"</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Cf. <er>Satin</er>.]</ety> <def>A kind of dress goods made of cotton or woolen, with a glossy surface resembling satin.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sateless</ent><br/
<hw>Sate"less</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Insatiable.</def> <mark>[R.]</mark> <rj><au>Young.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Satellite</ent><br/
<hw>Sat"el*lite</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F., fr. L. <ets>satelles</ets>, <ets>-itis</ets>, an attendant.]</ety> <sn>1.</sn> <def>An attendant attached to a prince or other powerful person; hence, an obsequious dependent.</def> <ldquo/The <xex>satellites</xex> of power.<rdquo/ <rj><au>I. Disraeli.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Astron.)</fld> <def>A secondary planet which revolves about another planet; <as>as, the moon is a <ex>satellite</ex> of the earth</as>. See <cref>Solar system</cref>, under <er>Solar</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Satellite moth</b></col> <fld>(Zool.)</fld>, <cd>a handsome European noctuid moth (<spn>Scopelosoma satellitia</spn>).</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Satellite</ent><br/
<hw>Sat"el*lite</hw>, <pos>a.</pos> <fld>(Anat.)</fld> <def>Situated near; accompanying; <as>as, the <ex>satellite</ex> veins, those which accompany the arteries</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Satellitious</ent><br/
<hw>Sat`el*li"tious</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Pertaining to, or consisting of, satellites.</def> <mark>[R.]</mark> <rj><au>Cheyne.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sathanas</ent><br/
<hw>Sath"an*as</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>Satanas</ets>. See <er>Satan</er>]</ety> <def>Satan.</def> <mark>[Obs.]</mark> <rj><au>Chaucer. Wyclif.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Satiate</ent><br/
<hw>Sa"ti*ate</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>satiatus</ets>, p. p. of <ets>satiare</ets> to satisfy, from <ets>sat</ets>, <ets>satis</ets>, enough. See <er>Sad</er>, <pos>a.</pos>, and cf. <er>Sate</er>.]</ety> <def>Filled to satiety; glutted; sated; -- followed by <xex>with</xex> or <xex>of</xex>.</def> <ldquo/<xex>Satiate</xex> of applause.<rdquo/ <rj><au>Pope.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Satiate</ent><br/
<hw>Sa"ti*ate</hw> <pr>(?)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Satiated</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Satiating</conjf>.]</vmorph> <sn>1.</sn> <def>To satisfy the appetite or desire of; to feed to the full; to furnish enjoyment to, to the extent of desire; to sate; <as>as, to <ex>satiate</ex> appetite or sense</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>These [smells] rather woo the sense than <qex>satiate</qex> it.</q> <rj><qau>Bacon.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>I may yet survive the malice of my enemies, although they should be <qex>satiated</qex> with my blood.</q> <rj><qau>Eikon Basilike.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To full beyond natural desire; to gratify to repletion or loathing; to surfeit; to glut.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>To saturate.</def> <mark>[Obs.]</mark> <rj><au>Sir I. Newton.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><syn><b>Syn.</b> -- To satisfy; sate; suffice; cloy; gorge; overfill; surfeit; glut.</syn> -- <usage><er>Satiate</er>, <er>Satisfy</er>, <er>Content</er>. These words differ principally in degree. To <xex>content</xex> is to make contented, even though every desire or appetite is not fully gratified. To <xex>satisfy</xex> is to appease fully the longings of desire. To <xex>satiate</xex> is to fill so completely that it is not possible to receive or enjoy more; hence, to overfill; to cause disgust in.</usage><br/
[<source>1913 Webster</source>]</p>

<p><q><qex>Content</qex> with science in the vale of peace.</q> <rj><qau>Pope.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>His whole felicity is endless strife;<br/
No peace, no <qex>satisfaction</qex>, crowns his life.</q> <rj><qau>Beaumont.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>He may be <qex>satiated</qex>, but not <qex>satisfied</qex>.</q> <rj><qau>Norris.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Satiation</ent><br/
<hw>Sa`ti*a"tion</hw> <pr>(?)</pr>, <pos>n.</pos> <def>Satiety.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Satiety</ent><br/
<hw>Sa*ti"e*ty</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>satietas</ets>, from <ets>satis</ets>, <ets>sat</ets>, enough: cf. F. <ets>sati<eacute/t<eacute/</ets>.]</ety> <def>The state of being satiated or glutted; fullness of gratification, either of the appetite or of any sensual desire; fullness beyond desire; an excess of gratification which excites wearisomeness or loathing; repletion; satiation.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>In all pleasures there is <qex>satiety</qex>.</q> <rj><qau>Hakewill.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>But thy words, with grace divine<br/
Imbued, bring to their sweetness no <qex>satiety</qex>.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><syn><b>Syn.</b> -- Repletion; satiation; surfeit; cloyment.</syn><br/
[<source>1913 Webster</source>]</p>

<p><-- p. 1278 pr=PCP --></p>

<p><ent>Satin</ent><br/
<hw>Sat"in</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F. <ets>satin</ets> (cf. Pg. <ets>setim</ets>), fr. It. <ets>setino</ets>, from <ets>seta</ets> silk, L. <ets>saeta</ets>, <ets>seta</ets>, a thick, stiff hair, a bristle; or possibly ultimately of Chinese origin; cf. Chin. <ets>sz-t<uum/n</ets>, <ets>sz-twan</ets>. Cf. <er>Sateen</er>.]</ety> <def>A silk cloth, of a thick, close texture, and overshot woof, which has a glossy surface.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Cloths of gold and <qex>satins</qex> rich of hue.</q> <rj><qau>Chaucer.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Denmark satin</b></col>, <cd>a kind of lasting; a stout worsted stuff, woven with a satin twill, used for women's shoes.</cd> -- <col><b>Farmer's satin</b></col>. <cd>See under <er>Farmer</er>.</cd> -- <col><b>Satin bird</b></col> <fld>(Zool.)</fld>, <cd>an Australian bower bird. Called also <altname>satin grackle</altname>.</cd> -- <col><b>Satin flower</b></col> <fld>(Bot.)</fld> <cd>See <er>Honesty</er>, 4.</cd> -- <col><b>Satin spar</b></col>. <fld>(Min.)</fld> <sd>(a)</sd> <cd>A fine fibrous variety of calcite, having a pearly luster</cd>. <sd>(b)</sd> <cd>A similar variety of gypsum.</cd> -- <col><b>Satin sparrow</b></col> <fld>(Zool.)</fld>, <cd>the shining flycatcher (<spn>Myiagra nitida</spn>) of Tasmania and Australia. The upper surface of the male is rich blackish green with a metallic luster.</cd> -- <col><b>Satin stone</b></col>, <cd>satin spar.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Satinet</ent><br/
<hw>Sat`i*net"</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F., fr. <ets>satin</ets>. See <er>Satin</er>.]</ety> <sn>1.</sn> <def>A thin kind of satin.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A kind of cloth made of cotton warp and woolen filling, used chiefly for trousers.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Satinette</ent><br/
<hw>Sat`i*nette"</hw> <pr>(?)</pr>, <pos>n.</pos> <def>One of a breed of fancy frilled pigeons allied to the owls and turbits, having the body white, the shoulders tricolored, and the tail bluish black with a large white spot on each feather.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Satin weave</ent><br/
<hw>Sat"in weave</hw>. <def>A style of weaving producing smooth-faced fabric in which the warp interlaces with the filling at points distributed over the surface.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Satinwood</ent><br/
<hw>Sat"in*wood`</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Bot.)</fld> <def>The hard, lemon-colored, fragrant wood of an East Indian tree (<spn>Chloroxylon Swietenia</spn>). It takes a lustrous finish, and is used in cabinetwork. The name is also given to the wood of a species of prickly ash (<spn>Xanthoxylum Caribaeum</spn>) growing in Florida and the West Indies.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Satiny</ent><br/
<hw>Sat"in*y</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Like or composed of satin; glossy; <as>as, to have a <ex>satiny</ex> appearance; a <ex>satiny</ex> texture</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sation</ent><br/
<hw>Sa"tion</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>satio</ets>, fr. <ets>serere</ets>, <ets>satum</ets>, to sow.]</ety> <def>A sowing or planting.</def> <mark>[Obs.]</mark> <rj><au>Sir T. Browne.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Satire</ent><br/
<hw>Sat"ire</hw> <pr>(?; <xex>in Eng. often</xex> <?/; 277)</pr>, <pos>n.</pos> <ety>[L. <ets>satira</ets>, <ets>satura</ets>, fr. <ets>satura</ets> (sc. <ets>lanx</ets>) a dish filled with various kinds of fruits, food composed of various ingredients, a mixture, a medley, fr. <ets>satur</ets> full of food, sated, fr. <ets>sat</ets>, <ets>satis</ets>, enough: cf. F. <ets>satire</ets>. See <er>Sate</er>, <er>Sad</er>, <pos>a.</pos>, and cf. <er>Saturate</er>.]</ety> <sn>1.</sn> <def>A composition, generally poetical, holding up vice or folly to reprobation; a keen or severe exposure of what in public or private morals deserves rebuke; an invective poem; <as>as, the <ex>Satires</ex> of Juvenal</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Keeness and severity of remark; caustic exposure to reprobation; trenchant wit; sarcasm.</def><br/
[<source>1913 Webster</source>]</p>

<p><syn><b>Syn.</b> -- Lampoon; sarcasm; irony; ridicule; pasquinade; burlesque; wit; humor.</syn><br/
[<source>1913 Webster</source>]</p>

<p><ent>Satirical</ent><br/
<ent>Satiric</ent><br/
<mhw>{ <hw>Sa*tir"ic</hw> <pr>(?)</pr>, <hw>Sa*tir"ic*al</hw> <pr>(?)</pr>, }</mhw> <pos>a.</pos> <ety>[L. <ets>satiricus</ets>: cf. F. <ets>satirique</ets>.]</ety> <sn>1.</sn> <def>Of or pertaining to satire; of the nature of satire; <as>as, a <ex>satiric</ex> style</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Censorious; severe in language; sarcastic; insulting.</def> <ldquo/<xex>Satirical</xex> rogue.<rdquo/ <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><syn><b>Syn.</b> -- Cutting; caustic; poignant; sarcastic; ironical; bitter; reproachful; abusive.</syn><br/
[<source>1913 Webster</source>]</p>

<p>-- <wordforms><wf>Sa*tir"ic*al*ly</wf>, <pos>adv.</pos> -- <wf>Sa*tir"ic*al*ness</wf>, <pos>n.</pos></wordforms><br/
[<source>1913 Webster</source>]</p>

<p><ent>Satirist</ent><br/
<hw>Sat"ir*ist</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Cf. F. <ets>satiriste</ets>.]</ety> <def>One who satirizes; especially, one who writes satire.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The mighty <qex>satirist</qex>, who . . . had spread terror through the Whig ranks.</q> <rj><qau>Macaulay.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Satirize</ent><br/
<hw>Sat"ir*ize</hw> <pr>(?)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Satirized</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Satirizing</conjf> <pr>(?)</pr>.]</vmorph> <ety>[Cf. F. <ets>satiriser</ets>.]</ety> <def>To make the object of satire; to attack with satire; to censure with keenness or severe sarcasm.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>It is as hard to <qex>satirize</qex> well a man of distinguished vices, as to praise well a man of distinguished virtues.</q> <rj><qau>Swift.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Satisfaction</ent><br/
<hw>Sat`is*fac"tion</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[OE. <ets>satisfaccioun</ets>, F. <ets>satisfaction</ets>, fr. L. <ets>satisfactio</ets>, fr. <ets>satisfacere</ets> to satisfy. See <er>Satisfy</er>.]</ety> <sn>1.</sn> <def>The act of satisfying, or the state of being satisfied; gratification of desire; contentment in possession and enjoyment; repose of mind resulting from compliance with its desires or demands.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The mind having a power to suspend the execution and <qex>satisfaction</qex> of any of its desires.</q> <rj><qau>Locke.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Settlement of a claim, due, or demand; payment; indemnification; adequate compensation.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>We shall make full <qex>satisfaction</qex>.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>That which satisfies or gratifies; atonement.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Die he, or justice must; unless for him<br/
Some other, able, and as willing, pay<br/
The rigid <qex>satisfaction</qex>, death for death.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><syn><b>Syn.</b> -- Contentment; content; gratification; pleasure; recompense; compensation; amends; remuneration; indemnification; atonement.</syn><br/
[<source>1913 Webster</source>]</p>

<p><ent>Satisfactive</ent><br/
<hw>Sat`is*fac"tive</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Satisfactory.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q><qex>Satisfactive</qex> discernment of fish.</q> <rj><qau>Sir T. Browne.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Satisfactory</ent><br/
<hw>Sat`is*fac"to*ry</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[Cf. F. <ets>satisfactoire</ets>.]</ety> <sn>1.</sn> <def>Giving or producing satisfaction; yielding content; especially, relieving the mind from doubt or uncertainty, and enabling it to rest with confidence; sufficient; <as>as, a <ex>satisfactory</ex> account or explanation</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Making amends, indemnification, or recompense; causing to cease from claims and to rest content; compensating; atoning; <as>as, to make <ex>satisfactory</ex> compensation, or a <ex>satisfactory</ex> apology</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>A most wise and sufficient means of redemption and salvation, by the <qex>satisfactory</qex> and meritorious death and obedience of the incarnate Son of God, Jesus Christ.</q> <rj><qau>Bp. Sanderson.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p>-- <wordforms><wf>Sat`is*fac"to*ri*ly</wf> <pr>(#)</pr>, <pos>adv.</pos> -- <wf>Sat`is*fac"to*ri*ness</wf>, <pos>n.</pos></wordforms><br/
[<source>1913 Webster</source>]</p>

<p><ent>Satisfiable</ent><br/
<hw>Sat"is*fi`a*ble</hw>, <pos>a.</pos> <def>That may be satisfied.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Satisfier</ent><br/
<hw>Sat"is*fi`er</hw> <pr>(?)</pr>, <pos>n.</pos> <def>One who satisfies.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Satisfy</ent><br/
<hw>Sat"is*fy</hw> <pr>(?)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Satisfied</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Satisfying</conjf> <pr>(?)</pr>.]</vmorph> <ety>[OF. <ets>satisfier</ets>; L. <ets>satis</ets> enough + <ets>-ficare</ets> (in comp.) to make; cf. F. <ets>satisfaire</ets>, L. <ets>satisfacere</ets>. See <er>Sad</er>, <pos>a.</pos>, and <er>Fact</er>.]</ety> <sn>1.</sn> <def>In general, to fill up the measure of a want of (a person or a thing); hence, to grafity fully the desire of; to make content; to supply to the full, or so far as to give contentment with what is wished for.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Death shall . . . with us two<br/
Be forced to <qex>satisfy</qex> his ravenous maw.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To pay to the extent of claims or deserts; to give what is due to; <as>as, to <ex>satisfy</ex> a creditor</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>To answer or discharge, as a claim, debt, legal demand, or the like; to give compensation for; to pay off; to requite; <as>as, to <ex>satisfy</ex> a claim or an execution</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>To free from doubt, suspense, or uncertainty; to give assurance to; to set at rest the mind of; to convince; <as>as, to <ex>satisfy</ex> one's self by inquiry</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The standing evidences of the truth of the gospel are in themselves most firm, solid, and <qex>satisfying</qex>.</q> <rj><qau>Atterbury.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><syn><b>Syn.</b> -- To satiate; sate; content; grafity; compensate. See <er>Satiate</er>.</syn><br/
[<source>1913 Webster</source>]</p>

<p><ent>Satisfy</ent><br/
<hw>Sat"is*fy</hw> <pr>(?)</pr>, <pos>v. i.</pos> <sn>1.</sn> <def>To give satisfaction; to afford gratification; to leave nothing to be desired.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To make payment or atonement; to atone.</def> <rj><au>Milton.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Satisfyingly</ent><br/
<hw>Sat"is*fy`ing*ly</hw> <pr>(?)</pr>, <pos>adv.</pos> <def>So as to satisfy; satisfactorily.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sative</ent><br/
<hw>Sa"tive</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>sativus</ets>, fr. <ets>serere</ets>, satum, to sow.]</ety> <def>Sown; propagated by seed.</def> <mark>[Obs.]</mark> <rj><au>Evelyn.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Satle</ent><br/
<hw>Sa"tle</hw> <pr>(?)</pr>, <pos>v. t. & i.</pos> <def>To settle.</def> <mark>[Obs.]</mark> <rj><au>Chaucer.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Satrap</ent><br/
<hw>Sa"trap</hw> <pr>(? <or/ ?; 277)</pr>, <pos>n.</pos> <ety>[L. <ets>satrapes</ets>, Gr. <?/, fr. OPers. <ets>khshatrap<amac/van</ets> ruler: cf. F. <ets>satrape</ets>.]</ety> <def>The governor of a province in ancient Persia; hence, a petty autocrat despot.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Satrapal</ent><br/
<hw>Sa"trap*al</hw> <pr>(? <or/ ?)</pr>, <pos>a.</pos> <def>Of or pertaining to a satrap, or a satrapy.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Satrapess</ent><br/
<hw>Sa"trap*ess</hw> <pr>(? <or/ ?)</pr>, <pos>n.</pos> <def>A female satrap.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Satrapical</ent><br/
<hw>Sa*trap"ic*al</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Satrapal.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Satrapy</ent><br/
<hw>Sa"trap*y</hw> <pr>(?; 277)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Satrapies</plw> <pr>(#)</pr>.</plu> <ety>[L. <ets>satrapia</ets>, <ets>satrapea</ets>, Gr. <?/: cf. F. <ets>satrapie</ets>.]</ety> <def>The government or jurisdiction of a satrap; a principality.</def> <rj><au>Milton.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Satsuma ware</ent><br/
<hw>Sat"su*ma ware"</hw> <pr>(? <or/ ?)</pr>. <fld>(Fine Arts)</fld> <def>A kind of ornamental hard-glazed pottery made at Satsuma in Kiushu, one of the Japanese islands.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Saturable</ent><br/
<hw>Sat"u*ra*ble</hw> <pr>(?; 135)</pr>, <pos>a.</pos> <ety>[L. <ets>saturabilis</ets>: cf. F. <ets>saturable</ets>.]</ety> <def>Capable of being saturated; admitting of saturation.</def> -- <wordforms><wf>Sat`u*ra*bil"i*ty</wf> <pr>(#)</pr>, <pos>n.</pos></wordforms><br/
[<source>1913 Webster</source>]</p>

<p><ent>Saturant</ent><br/
<hw>Sat"u*rant</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>saturans</ets>, p. pr. See <er>Saturate</er>.]</ety> <def>Impregnating to the full; saturating.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Saturant</ent><br/
<hw>Sat"u*rant</hw>, <pos>n.</pos> <sn>1.</sn> <fld>(Chem.)</fld> <def>A substance used to neutralize or saturate the affinity of another substance.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Med.)</fld> <def>An antacid, as magnesia, used to correct acidity of the stomach.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Saturate</ent><br/
<hw>Sat"u*rate</hw> <pr>(?)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Saturated</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Saturating</conjf>.]</vmorph> <ety>[L. <ets>saturatus</ets>, p. p. of <ets>saturare</ets> to saturate, fr. <ets>satur</ets> full of food, sated. See <er>Satire</er>.]</ety> <sn>1.</sn> <def>To cause to become completely penetrated, impregnated, or soaked; to fill fully; to sate.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Innumerable flocks and herds covered that vast expanse of emerald meadow <qex>saturated</qex> with the moisture of the Atlantic.</q> <rj><qau>Macaulay.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Fill and <qex>saturate</qex> each kind<br/
With good according to its mind.</q> <rj><qau>Emerson.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Chem.)</fld> <def>To satisfy the affinity of; to cause to become inert by chemical combination with all that it can hold; <as>as, to <ex>saturate</ex> phosphorus with chlorine</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Saturate</ent><br/
<hw>Sat"u*rate</hw> <pr>(?)</pr>, <pos>p. a.</pos> <ety>[L. <ets>saturatus</ets>, p. p.]</ety> <def>Filled to repletion; saturated; soaked.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Dries his feathers <qex>saturate</qex> with dew.</q> <rj><qau>Cowper.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>The sand beneath our feet is <qex>saturate</qex><br/
With blood of martyrs.</q> <rj><qau>Longfellow.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Saturated</ent><br/
<hw>Sat"u*ra`ted</hw> <pr>(?)</pr>, <pos>a.</pos> <sn>1.</sn> <def>Filled to repletion; holding by absorption, or in solution, all that is possible; <as>as, <ex>saturated</ex> garments; a <ex>saturated</ex> solution of salt</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Chem.)</fld> <def>Having its affinity satisfied; combined with all it can hold; -- said of certain atoms, radicals, or compounds; <as>thus, methane is a <ex>saturated</ex> compound</as>. Contrasted with <contr>unsaturated</contr>.</def><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ A <ex>saturated</ex> compound may exchange certain ingredients for others, but can not take on more without such exchange.</note><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Saturated color</b></col> <fld>(Optics)</fld>, <cd>a color not diluted with white; a pure unmixed color, like those of the spectrum.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Saturation</ent><br/
<hw>Sat`u*ra"tion</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>saturatio</ets>: cf. F. <ets>saturation</ets>.]</ety> <sn>1.</sn> <def>The act of saturating, or the state of being saturating; complete penetration or impregnation.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Chem.)</fld> <def>The act, process, or result of saturating a substance, or of combining it to its fullest extent.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <fld>(Optics)</fld> <def>Freedom from mixture or dilution with white; purity; -- said of colors.</def><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ The degree of <xex>saturation</xex> of a color is its relative purity, or freedom from admixture with white.</note><br/
[<source>1913 Webster</source>]</p>

<p><ent>Saturator</ent><br/
<hw>Sat"u*ra`tor</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L.]</ety> <def>One who, or that which, saturates.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Saturday</ent><br/
<hw>Sat"ur*day</hw> <pr>(?; 48)</pr>, <pos>n.</pos> <ety>[OE. <ets>Saterday</ets>, AS. <ets>Saeterdaeg</ets>, <ets>Saeterndaeg</ets>, <ets>Saeternesdaeg</ets>, literally, Saturn's day, fr. L. <ets>Saturnus</ets> Saturn + AS. <ets>daeg</ets> day; cf. L. <ets>dies Saturni</ets>.]</ety> <def>The seventh or last day of the week; the day following Friday and preceding Sunday.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Saturity</ent><br/
<hw>Sa*tu"ri*ty</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>saturitas</ets>, fr. <ets>satur</ets> full of food, sated.]</ety> <def>The state of being saturated; fullness of supply.</def> <mark>[Obs.]</mark> <rj><au>Warner.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Saturn</ent><br/
<hw>Sa"turn</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>Saturnus</ets>, literally, the sower, fr. <ets>serere</ets>, <ets>satum</ets>, to sow. See <er>Season</er>.]</ety> <sn>1.</sn> <fld>(Roman Myth.)</fld> <def>One of the elder and principal deities, the son of Coelus and Terra (Heaven and Earth), and the father of Jupiter. The corresponding Greek divinity was <grk>Kro`nos</grk>, later <grk>CHro`nos</grk>, Time.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Astron.)</fld> <def>One of the planets of the solar system, next in magnitude to Jupiter, but more remote from the sun. Its diameter is seventy thousand miles, its mean distance from the sun nearly eight hundred and eighty millions of miles, and its year, or periodical revolution round the sun, nearly twenty-nine years and a half. It is surrounded by a remarkable system of rings, and has eight satellites.</def> <-- more satellites have been discovered. --><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <fld>(Alchem.)</fld> <def>The metal lead.</def> <mark>[Archaic]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Saturnalia</ent><br/
<hw>Sat`ur*na"li*a</hw> <pr>(?)</pr>, <pos>n. pl.</pos> <ety>[L. See <er>Saturn</er>.]</ety> <sn>1.</sn> <fld>(Rom. Antiq.)</fld> <def>The festival of Saturn, celebrated in December, originally during one day, but afterward during seven days, as a period of unrestrained license and merriment for all classes, extending even to the slaves.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Hence: A period or occasion of general license, in which the passions or vices have riotous indulgence.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Saturnalian</ent><br/
<hw>Sat`ur*na"li*an</hw> <pr>(?)</pr>, <pos>a.</pos> <sn>1.</sn> <def>Of or pertaining to the Saturnalia.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Of unrestrained and intemperate jollity; riotously merry; dissolute.</def> <ldquo/<xex>Saturnalian</xex> amusement.<rdquo/ <rj><au>Burke.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Saturnian</ent><br/
<hw>Sa*tur"ni*an</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>Saturnius</ets>.]</ety> <sn>1.</sn> <fld>(Roman Myth.)</fld> <def>Of or pertaining to Saturn, whose age or reign, from the mildness and wisdom of his government, is called the <xex>golden age</xex>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Hence: Resembling the golden age; distinguished for peacefulness, happiness, contentment.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Augustus, born to bring <qex>Saturnian</qex> times.</q> <rj><qau>Pope.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <fld>(Astron.)</fld> <def>Of or pertaining to the planet Saturn; <as>as, the <ex>Saturnian</ex> year</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Saturnian verse</b></col> <fld>(Pros.)</fld>, <cd>a meter employed by early Roman satirists, consisting of three iambics and an extra syllable followed by three trochees, as in the line: -- Th<ecr/ qu<emac/en | w<acr/s <imac/n | th<ecr/ k<imac/tch | <ecr/n || <emac/at<icr/ng | br<emac/ad <acr/nd | h<omac/n<ecr/y. </cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Saturnian</ent><br/
<hw>Sa*tur"ni*an</hw>, <pos>n.</pos> <fld>(Zool.)</fld> <def>Any one of numerous species of large handsome moths belonging to <gen>Saturnia</gen> and allied genera.  The luna moth, polyphemus, and promethea, are examples. They belong to the Silkworn family, and some are raised for their silk. See <er>Polyphemus</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Saturnicentric</ent><br/
<hw>Sat`urn*i*cen"tric</hw> <pr>(?)</pr>, <pos>a.</pos> <fld>(Astron.)</fld> <def>Appearing as if seen from the center of the planet Saturn; relating or referred to Saturn as a center.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Saturnine</ent><br/
<hw>Sat"ur*nine</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>Saturnus</ets> the god Saturn, also, the planet Saturn: cf. F. <ets>saturnin</ets> of or pertaining to lead (Saturn, in old chemistry, meaning lead), <ets>saturnien</ets> saturnine, saturnian. See <er>Saturn</er>.]</ety> <sn>1.</sn> <def>Born under, or influenced by, the planet Saturn.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Heavy; grave; gloomy; dull; -- the opposite of <ant>mercurial</ant>; <as>as, a <ex>saturnine</ex> person or temper</as>.</def> <rj><au>Addison.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <fld>(Old Chem.)</fld> <def>Of or pertaining to lead; characterized by, or resembling, lead, which was formerly called <xex>Saturn</xex>.</def> <mark>[Archaic]</mark><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Saturnine colic</b></col> <fld>(Med.)</fld>, <cd>lead colic.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Saturnism</ent><br/
<hw>Sat"ur*nism</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Med.)</fld> <def>Plumbism.</def> <rj><au>Quain.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Saturnist</ent><br/
<hw>Sat"ur*nist</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A person of a dull, grave, gloomy temperament.</def> <rj><au>W. Browne.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Satyr</ent><br/
<hw>Sa"tyr</hw> <pr>(?; 277)</pr>, <pos>n.</pos> <ety>[L. <ets>satyrus</ets>, Gr. <?/: cf. F. <ets>satyre</ets>.]</ety> <sn>1.</sn> <fld>(Class. Myth.)</fld> <def>A sylvan deity or demigod, represented as part man and part goat, and characterized by riotous merriment and lasciviousness.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Rough <qex>Satyrs</qex> danced; and Fauns, with cloven heel,<br/
From the glad sound would not be absent long.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Zool.)</fld> <def>Any one of many species of butterflies belonging to the family <fam>Nymphalidae</fam>. Their colors are commonly brown and gray, often with ocelli on the wings. Called also <altname>meadow browns</altname>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <fld>(Zool.)</fld> <def>The orang-outang.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Satyriasis</ent><br/
||<hw>Sat`y*ri"a*sis</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L., fr. Gr. <?/. See <er>Satyr</er>.]</ety> <def>Immoderate venereal appetite in the male.</def> <rj><au>Quain.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Satyrical</ent><br/
<ent>Satyric</ent><br/
<mhw>{ <hw>Sa*tyr"ic</hw> <pr>(?)</pr>, <hw>Sa*tyr"ic*al</hw> <pr>(?)</pr>, }</mhw> <pos>a.</pos> <ety>[L. <ets>satyricus</ets>, Gr. <grk>satyriko`s</grk>.]</ety> <def>Of or pertaining to satyrs; burlesque; <as>as, <ex>satyric</ex> tragedy</as>.</def> <rj><au>P. Cyc.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Satyrion</ent><br/
||<hw>Sa*tyr"i*on</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L., fr. Gr. <grk>saty`rion</grk>.]</ety> <fld>(Bot.)</fld> <def>Any one of several kinds of orchids.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sauba ant</ent><br/
<hw>Sau"ba ant`</hw> <pr>(?)</pr>. <fld>(Zool.)</fld> <def>A South American ant (<spn>Oecodoma cephalotes</spn>) remarkable for having two large kinds of workers besides the ordinary ones, and for the immense size of its formicaries. The sauba ant cuts off leaves of plants and carries them into its subterranean nests, and thus often does great damage by defoliating trees and cultivated plants.</def><-- a leaf-cutting ant --><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sauce</ent><br/
<hw>Sauce</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F., fr. OF. <ets>sausse</ets>, LL. <ets>salsa</ets>, properly, salt pickle, fr. L. <ets>salsus</ets> salted, salt, p. p. of <ets>salire</ets> to salt, fr. <ets>sal</ets> salt. See <er>Salt</er>, and cf. <er>Saucer</er>, <er>Souse</er> pickle, <er>Souse</er> to plunge.]</ety> <sn>1.</sn> <def>A composition of condiments and appetizing ingredients eaten with food as a relish; especially, a dressing for meat or fish or for puddings; <as>as, mint <ex>sauce</ex>; sweet <ex>sauce</ex>, etc.</as></def> <ldquo/Poignant <xex>sauce</xex>.<rdquo/ <rj><au>Chaucer.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>High <qex>sauces</qex> and rich spices fetched from the Indies.</q> <rj><qau>Sir S. Baker.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Any garden vegetables eaten with meat.</def> <mark>[Prov. Eng. & Colloq. U.S.]</mark> <rj><au>Forby. Bartlett.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Roots, herbs, vine fruits, and salad flowers . . . they dish up various ways, and find them very delicious <qex>sauce</qex> to their meats, both roasted and boiled, fresh and salt.</q> <rj><qau>Beverly.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>Stewed or preserved fruit eaten with other food as a relish; <as>as, apple <ex>sauce</ex>, cranberry <ex>sauce</ex>, etc.</as></def> <mark>[U.S.]</mark> <ldquo/Stewed apple <xex>sauce</xex>.<rdquo/ <rj><au>Mrs. Lincoln (Cook Book).</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>Sauciness; impertinence.</def> <mark>[Low.]</mark> <rj><au>Haliwell.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>To serve one the same sauce</b></col>, <cd>to retaliate in the same kind.</cd> <mark>[Vulgar]</mark></cs><br/
[<source>1913 Webster</source>]</p>

<p><-- p. 1279 pr=PCP --></p>

<p><ent>Sauce</ent><br/
<hw>Sauce</hw> <pr>(s<add/s)</pr>, <pos>v. t.</pos> <ety>[Cf. F. <ets>saucer</ets>.]</ety> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Sauced</conjf> <pr>(s<add/st)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Saucing</conjf> <pr>(s<add/"s<icr/ng)</pr>.]</vmorph> <sn>1.</sn> <def>To accompany with something intended to give a higher relish; to supply with appetizing condiments; to season; to flavor.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To cause to relish anything, as if with a sauce; to tickle or gratify, as the palate; to please; to stimulate; hence, to cover, mingle, or dress, as if with sauce; to make an application to.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>Earth, yield me roots;<br/
Who seeks for better of thee, <qex>sauce</qex> his palate<br/
With thy most operant poison!</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>To make poignant; to give zest, flavor or interest to; to set off; to vary and render attractive.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Then fell she to <qex>sauce</qex> her desires with threatenings.</q> <rj><qau>Sir P. Sidney.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Thou sayest his meat was <qex>sauced</qex> with thy upbraidings.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>To treat with bitter, pert, or tart language; to be impudent or saucy to.</def> <mark>[Colloq. or Low]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>I'll <qex>sauce</qex> her with bitter words.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sauce</ent><br/
||<hw>Sauce</hw> <pr>(s<omac/s)</pr>, <pos>n.</pos> <ety>[F.]</ety> <fld>(Fine Art)</fld> <def>A soft crayon for use in stump drawing or in shading with the stump.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sauce-alone</ent><br/
<hw>Sauce"-a*lone`</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Etymol. uncertain.]</ety> <fld>(Bot.)</fld> <def>Jack-by-the-hedge. See under <er>Jack</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Saucebox</ent><br/
<hw>Sauce"box`</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[See <er>Sauce</er>, and <er>Saucy</er>.]</ety> <def>A saucy, impudent person; especially, a pert child.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Saucebox, go, meddle with your lady's fan,<br/
And prate not here!</q> <rj><qau>A. Brewer.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Saucepan</ent><br/
<hw>Sauce"pan`</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A small pan with a handle, in which sauce is prepared over a fire; a stewpan.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Saucer</ent><br/
<hw>Sau"cer</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F. <ets>sauci<egrave/re</ets>, from <ets>sauce</ets>. See <er>Sauce</er>.]</ety> <sn>1.</sn> <def>A small pan or vessel in which sauce was set on a table.</def> <mark>[Obs.]</mark> <rj><au>Bacon.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A small dish, commonly deeper than a plate, in which a cup is set at table.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>Something resembling a saucer in shape.</def> Specifically: <sd>(a)</sd> <def>A flat, shallow caisson for raising sunken ships.</def> <sd>(b)</sd> <def>A shallow socket for the pivot of a capstan.</def><br/
[<source>1913 Webster</source>]</p>

<p><-- <col><b>Flying saucer</b></col>, <cd>a type of Unidentified Flying Object, having a biconvex discoid shape; such objects are occasionally reported to have been sighted, but no example of one has been reliably shown to exist.</cd> They are believed by ufologists to originate in outer space, but they are generally presumed to be misinterpretations of ordinary phenomena, illusions or imaginary objects. Fraudulent photographs purporting to show flying saucers are published from time to time. --><br/
[<source>1913 Webster</source>]</p>

<p><ent>Saucily</ent><br/
<hw>Sau"ci*ly</hw> <pr>(?)</pr>, <pos>adv.</pos> <def>In a saucy manner; impudently; with impertinent boldness.</def> <rj><au>Addison.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sauciness</ent><br/
<hw>Sau"ci*ness</hw>, <pos>n.</pos> <def>The quality or state of being saucy; that which is saucy; impertinent boldness; contempt of superiors; impudence.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Your <qex>sauciness</qex> will jest upon my love.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><syn><b>Syn.</b> -- Impudence; impertinence; rudeness; insolence. See <er>Impudence</er>.</syn><br/
[<source>1913 Webster</source>]</p>

<p><ent>Saucisse</ent><br/
<ent>Saucisson</ent><br/
<mhw>{ ||<hw>Sau`cis`son"</hw> <pr>(?)</pr>, <hw>Sau`cisse"</hw> <pr>(?)</pr>, }</mhw> <pos>n.</pos> <ety>[F., fr. <ets>saucisse</ets> sausage. See <er>Sausage</er>.]</ety> <sn>1.</sn> <fld>(Mining or Gun.)</fld> <def>A long and slender pipe or bag, made of cloth well pitched, or of leather, filled with powder, and used to communicate fire to mines, caissons, bomb chests, etc.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Fort.)</fld> <def>A fascine of more than ordinary length.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Saucy</ent><br/
<hw>Sau"cy</hw> <pr>(?)</pr>, <pos>a.</pos> <amorph>[<pos>Compar.</pos> <adjf>Saucier</adjf> <pr>(?)</pr>; <pos>superl.</pos> <adjf>Sauciest</adjf>.]</amorph> <ety>[From <er>Sauce</er>.]</ety> <sn>1.</sn> <def>Showing impertinent boldness or pertness; transgressing the rules of decorum; treating superiors with contempt; impudent; insolent; <as>as, a <ex>saucy</ex> fellow</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Am I not protector, <qex>saucy</qex> priest?</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Expressive of, or characterized by, impudence; impertinent; <as>as, a <ex>saucy</ex> eye; <ex>saucy</ex> looks</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>We then have done you bold and <qex>saucy</qex> wrongs.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><syn><b>Syn.</b> -- Impudent; insolent; impertinent; rude.</syn><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sauerkraut</ent><br/
<hw>Sauer"kraut`</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[G., fr. <ets>sauer</ets> sour + <ets>kraut</ets> herb, cabbage.]</ety> <def>Cabbage cut fine and allowed to ferment in a brine made of its own juice with salt, -- a German dish.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sauf</ent><br/
<hw>Sauf</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Safe.</def> <mark>[Obs.]</mark> <rj><au>Chaucer.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sauf</ent><br/
<hw>Sauf</hw>, <pos>conj. & prep.</pos> <def>Save; except.</def> <mark>[Obs.]</mark> <ldquo/<xex>Sauf</xex> I myself.<rdquo/ <rj><au>Chaucer.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Saufly</ent><br/
<hw>Sauf"ly</hw>, <pos>adv.</pos> <def>Safely.</def> <mark>[Obs.]</mark> <rj><au>Chaucer.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sauger</ent><br/
<hw>Sau"ger</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Zool.)</fld> <def>An American fresh-water food fish (<spn>Stizostedion Canadense</spn>); -- called also <altname>gray pike</altname>, <altname>blue pike</altname>, <altname>hornfish</altname>, <altname>land pike</altname>, <altname>sand pike</altname>, <altname>pickering</altname>, and <altname>pickerel</altname>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sauh</ent><br/
<ent>Saugh</ent><br/
<mhw>{ <hw>Saugh</hw>, <hw>Sauh</hw> <pr>(?)</pr>, }</mhw> <mark>obs.</mark> <def><pos>imp. sing.</pos> of <er>See</er>.</def> <rj><au>Chaucer.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sauks</ent><br/
<hw>Sauks</hw> <pr>(?)</pr>, <pos>n. pl.</pos> <fld>(Ethnol.)</fld> <def>Same as <er>Sacs</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Saul</ent><br/
<hw>Saul</hw> <pr>(?)</pr>, <pos>n.</pos> <def>Soul.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Saul</ent><br/
<hw>Saul</hw>, <pos>n.</pos> <def>Same as <er>Sal</er>, the tree.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Saulie</ent><br/
<hw>Sau"lie</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A hired mourner at a funeral.</def> <mark>[Scot.]</mark> <rj><au>Sir W. Scott.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sault</ent><br/
||<hw>Sault</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[OF., F. <ets>saut</ets>, fr. L. <ets>saltus</ets>. See <er>Salt</er> a leap.]</ety> <def>A rapid in some rivers; <as>as, the <ex>Sault</ex> Ste. Marie</as>.</def> <mark>[U.S.]</mark> <rj><au>Bartlett.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Saunders</ent><br/
<hw>Saun"ders</hw> <pr>(?)</pr>, <pos>n.</pos> <def>See <er>Sandress</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Saunders-blue</ent><br/
<hw>Saun"ders-blue`</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Corrupted fr. F. <ets>cendres bleues</ets> blue ashes.]</ety> <def>A kind of color prepared from calcined lapis lazuli; ultramarine; also, a blue prepared from carbonate of copper.</def> <altsp>[Written also <asp>sanders-blue</asp>.]</altsp><br/
[<source>1913 Webster</source>]</p>

<p><ent>Saunter</ent><br/
<hw>Saun"ter</hw> <pr>(?)</pr>, <pos>v. i.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Sauntered</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Sauntering</conjf>.]</vmorph> <ety>[Written also <ets>santer</ets>.]</ety> <ety>[Probably fr. F. <ets>s'aventurer</ets> to adventure (one's self), through a shortened form <ets>s'auntrer</ets>. See <er>Adventure</er>, <pos>n. & v.</pos>]</ety> <def>To wander or walk about idly and in a leisurely or lazy manner; to lounge; to stroll; to loiter.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>One could lie under elm trees in a lawn, or <qex>saunter</qex> in meadows by the side of a stream.</q> <rj><qau>Masson.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><syn><b>Syn.</b> -- To loiter; linger; stroll; wander.</syn><br/
[<source>1913 Webster</source>]</p>

<p><ent>Saunter</ent><br/
<hw>Saun"ter</hw>, <pos>n.</pos> <def>A sauntering, or a sauntering place.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>That wheel of fops, that <qex>saunter</qex> of the town.</q> <rj><qau>Young.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Saunterer</ent><br/
<hw>Saun"ter*er</hw> <pr>(?)</pr>, <pos>n.</pos> <def>One who saunters.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Saur</ent><br/
<hw>Saur</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Contracted from Gael. <ets>salachar</ets> filth, nastiness, fr. <ets>salach</ets> nasty, fr. <ets>sal</ets> filth, refuse.]</ety> <def>Soil; dirt; dirty water; urine from a cowhouse.</def> <mark>[Prov. Eng.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Saurel</ent><br/
<hw>Sau"rel</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Zool.)</fld> <def>Any carangoid fish of the genus <gen>Trachurus</gen>, especially <spn>Trachurus trachurus</spn>, or <spn>Trachurus saurus</spn>, of Europe and America, and <spn>Trachurus picturatus</spn> of California. Called also <altname>skipjack</altname>, and <altname>horse mackerel</altname>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sauria</ent><br/
||<hw>Sau"ri*a</hw> <pr>(?)</pr>, <pos>n. pl.</pos> <ety>[NL., from Gr. <?/ a lizard.]</ety> <fld>(Zool.)</fld> <def>A division of Reptilia formerly established to include the Lacertilia, Crocodilia, Dinosauria, and other groups. By some writers the name is restricted to the Lacertilia.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Saurian</ent><br/
<hw>Sau"ri*an</hw> <pr>(?)</pr>, <pos>a.</pos> <fld>(Zool.)</fld> <def>Of or pertaining to, or of the nature of, the Sauria.</def> -- <def2><pos>n.</pos> <def>One of the Sauria.</def></def2><br/
[<source>1913 Webster</source>]</p>

<p><ent>Saurioid</ent><br/
<hw>Sau"ri*oid</hw> <pr>(?)</pr>, <pos>a.</pos> <fld>(Zool.)</fld> <def>Same as <er>Sauroid</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Saurobatrachia</ent><br/
||<hw>Sau"ro*ba*tra"chi*a</hw> <pr>(?)</pr>, <pos>n. pl.</pos> <ety>[NL. See <er>Sauria</er>, and <er>Batrachia</er>.]</ety> <fld>(Zool.)</fld> <def>The Urodela.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Saurognathous</ent><br/
<hw>Sau*rog"na*thous</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[Gr. <?/ a lizard + <?/ the jaw.]</ety> <fld>(Zool.)</fld> <def>Having the bones of the palate arranged as in saurians, the vomer consisting of two lateral halves, as in the woodpeckers (<fam>Pici</fam>).</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sauroid</ent><br/
<hw>Sau"roid</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[Gr. <?/ a lizard + <ets>-oid</ets>: cf. Gr. <?/ lizardlike.]</ety> <fld>(Zool.)</fld> <sd>(a)</sd> <def>Like or pertaining to the saurians.</def> <sd>(b)</sd> <def>Resembling a saurian superficially; <as>as, a <ex>sauroid</ex> fish</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sauroidichnite</ent><br/
<hw>Sau`roid*ich"nite</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[See <er>Sauroid</er>, and <er>Ichnite</er>.]</ety> <fld>(Paleon.)</fld> <def>The fossil track of a saurian.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sauropoda</ent><br/
||<hw>Sau*rop"o*da</hw> <pr>(?)</pr>, <pos>n. pl.</pos> <ety>[NL., fr. Gr. <?/ a lizard + <ets>-poda</ets>.]</ety> <fld>(Paleon.)</fld> <def>An extinct order of herbivorous dinosaurs having the feet of a saurian type, instead of birdlike, as they are in many dinosaurs. It includes the largest known land animals, belonging to Brontosaurus, Camarasaurus, and allied genera. See <xex>Illustration</xex> in Appendix.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sauropsida</ent><br/
||<hw>Sau*rop"si*da</hw> <pr>(?)</pr>, <pos>n. pl.</pos> <ety>[NL., fr. Gr. <?/ a lizard + <?/ appearance.]</ety> <fld>(Zool.)</fld> <def>A comprehensive group of vertebrates, comprising the reptiles and birds.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sauropterygia</ent><br/
||<hw>Sau*rop`te*ryg"i*a</hw> <pr>(?)</pr>, <pos>n. pl.</pos> <ety>[NL., fr. Gr. <?/ a lizard + <?/, <?/, a wing.]</ety> <fld>(Paleon.)</fld> <def>Same as <er>Plesiosauria</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Saururae</ent><br/
||<hw>Sau*ru"rae</hw> <pr>(?)</pr>, <pos>n. pl.</pos> <ety>[NL., fr. Gr. <?/ a lizard + <?/ a tail.]</ety> <fld>(Paleon.)</fld> <def>An extinct order of birds having a long vertebrated tail with quills along each side of it. Archaeopteryx is the type. See <er>Archaeopteryx</er>, and <er>Odontornithes</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Saury</ent><br/
<hw>Sau"ry</hw> <pr>(?)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Sauries</plw> <pr>(#)</pr>.</plu> <ety>[Etymol. uncertain.]</ety> <fld>(Zool.)</fld> <def>A slender marine fish (<spn>Scomberesox saurus</spn>) of Europe and America. It has long, thin, beaklike jaws. Called also <altname>billfish</altname>, <altname>gowdnook</altname>, <altname>gawnook</altname>, <altname>skipper</altname>, <altname>skipjack</altname>, <altname>skopster</altname>, <altname>lizard fish</altname>, and <altname>Egypt herring</altname>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sausage</ent><br/
<hw>Sau"sage</hw> <pr>(?; 48)</pr>, <pos>n.</pos> <ety>[F. <ets>saucisse</ets>, LL. <ets>salcitia</ets>, <ets>salsicia</ets>, fr. <ets>salsa</ets>. See <er>Sauce</er>.]</ety> <sn>1.</sn> <def>An article of food consisting of meat (esp. pork) minced and highly seasoned, and inclosed in a cylindrical case or skin usually made of the prepared intestine of some animal.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A saucisson. See <er>Saucisson</er>.</def> <rj><au>Wilhelm.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sauseflem</ent><br/
<hw>Sau"se*flem</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[OF. <ets>saus</ets> salt (L. <ets>salsus</ets>) + <ets>flemme</ets> phlegm.]</ety> <def>Having a red, pimpled face.</def> <mark>[Obs.]</mark> <altsp>[Written also <asp>sawceflem</asp>.]</altsp> <rj><au>Chaucer.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Saussurite</ent><br/
<hw>Saus"sur*ite</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F. So called from M. <ets>Saussure</ets>.]</ety> <fld>(Min.)</fld> <def>A tough, compact mineral, of a white, greenish, or grayish color. It is near zoisite in composition, and in part, at least, has been produced by the alteration of feldspar.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Saute</ent><br/
<ent>Saut</ent><br/
<mhw>{ <hw>Saut</hw>, <hw>Saute</hw> <pr>(?)</pr>, }</mhw> <pos>n.</pos> <def>An assault.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Saute</ent><br/
||<hw>Sau`te"</hw> <pr>(?)</pr>, <def><pos>p. p.</pos> of <er>Sauter</er>.</def> <rj><au>C. Owen.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sauter</ent><br/
||<hw>Sau`ter"</hw> <pr>(?)</pr>, <pos>v. t.</pos> <ety>[F., properly, to jump.]</ety> <def>To fry lightly and quickly, as meat, by turning or tossing it over frequently in a hot pan greased with a little fat.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sauter</ent><br/
<hw>Sau"ter</hw> <pr>(?)</pr>, <pos>n.</pos> <def>Psalter.</def> <mark>[Obs.]</mark> <rj><au>Piers Plowman.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sauterelle</ent><br/
<hw>Sau`te*relle</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F.]</ety> <def>An instrument used by masons and others to trace and form angles.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sauterne</ent><br/
<hw>Sau`terne"</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F.]</ety> <def>A white wine made in the district of <etsep>Sauterne</etsep>, France.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sautrie</ent><br/
<hw>Sau"trie</hw> <pr>(?)</pr>, <pos>n.</pos> <def>Psaltery.</def> <mark>[Obs.]</mark> <rj><au>Chaucer.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sauvegarde</ent><br/
||<hw>Sau`ve*garde"</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F.]</ety> <fld>(Zool.)</fld> <def>The monitor.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Savable</ent><br/
<hw>Sav"a*ble</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[From <er>Save</er>. Cf. <er>Salvable</er>.]</ety> <def>Capable of, or admitting of, being saved.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>In the person prayed for there ought to be the great disposition of being in a <qex>savable</qex> condition.</q> <rj><qau>Jer. Taylor.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Savableness</ent><br/
<hw>Sav"a*ble*ness</hw>, <pos>n.</pos> <def>Capability of being saved.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Savacioun</ent><br/
<hw>Sa*va"ci*oun`</hw> <pr>(?)</pr>, <pos>n.</pos> <def>Salvation.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Savage</ent><br/
<hw>Sav"age</hw> <pr>(?; 48)</pr>, <pos>a.</pos> <ety>[F. <ets>sauvage</ets>, OF. <ets>salvage</ets>, fr. L. <ets>silvaticus</ets> belonging to a wood, wild, fr. <ets>silva</ets> a wood. See <er>Silvan</er>, and cf. <er>Sylvatic</er>.]</ety> <sn>1.</sn> <def>Of or pertaining to the forest; remote from human abodes and cultivation; in a state of nature; wild; <as>as, a <ex>savage</ex> wilderness</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Wild; untamed; uncultivated; <as>as, <ex>savage</ex> beasts</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Cornels, and <qex>savage</qex> berries of the wood.</q> <rj><qau>Dryden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>Uncivilized; untaught; unpolished; rude; <as>as, <ex>savage</ex> life; <ex>savage</ex> manners</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>What nation, since the commencement of the Christian era, ever rose from <qex>savage</qex> to civilized without Christianity?</q> <rj><qau>E. D. Griffin.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>Characterized by cruelty; barbarous; fierce; ferocious; inhuman; brutal; <as>as, a <ex>savage</ex> spirit</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><syn><b>Syn.</b> -- Ferocious; wild; uncultivated; untamed; untaught; uncivilized; unpolished; rude; brutish; brutal; heathenish; barbarous; cruel; inhuman; fierce; pitiless; merciless; unmerciful; atrocious. See <er>Ferocious</er>.</syn><br/
[<source>1913 Webster</source>]</p>

<p><ent>Savage</ent><br/
<hw>Sav"age</hw>, <pos>n.</pos> <sn>1.</sn> <def>A human being in his native state of rudeness; one who is untaught, uncivilized, or without cultivation of mind or manners.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A man of extreme, unfeeling, brutal cruelty; a barbarian.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Savage</ent><br/
<hw>Sav"age</hw> <pr>(?; 48)</pr>, <pos>v. t.</pos> <def>To make savage.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>Its bloodhounds, <qex>savaged</qex> by a cross of wolf.</q> <rj><qau>Southey.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Savagely</ent><br/
<hw>Sav"age*ly</hw>, <pos>adv.</pos> <def>In a savage manner.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Savageness</ent><br/
<hw>Sav"age*ness</hw>, <pos>n.</pos> <def>The state or quality of being savage.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Wolves and bears, they say,<br/
Casting their <qex>savageness</qex> aside have done<br/
Like offices of pity.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Savagery</ent><br/
<hw>Sav"age*ry</hw> <pr>(?; 277)</pr>, <pos>n.</pos> <ety>[F. <ets>sauvagerie</ets>.]</ety> <sn>1.</sn> <def>The state of being savage; savageness; savagism.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>A like work of primeval <qex>savagery</qex>.</q> <rj><qau>C. Kingsley.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>An act of cruelty; barbarity.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The wildest <qex>savagery</qex>, the vilest stroke,<br/
That ever wall-eyed wrath or staring rage<br/
Presented to the tears of soft remorse.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>Wild growth, as of plants.</def> <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Savagism</ent><br/
<hw>Sav"a*gism</hw> <pr>(?)</pr>, <pos>n.</pos> <def>The state of being savage; the state of rude, uncivilized men, or of men in their native wildness and rudeness.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Savanilla</ent><br/
<hw>Sav`a*nil"la</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Zool.)</fld> <def>The tarpum.</def> <mark>[Local, U.S.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Savanna</ent><br/
<hw>Sa*van"na</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Of American Indian origin; cf. Sp. <ets>sabana</ets>, F. <ets>savane</ets>.]</ety> <def>A tract of level land covered with the vegetable growth usually found in a damp soil and warm climate, -- as grass or reeds, -- but destitute of trees.</def> <altsp>[Spelt also <asp>savannah</asp>.]</altsp><br/
[<source>1913 Webster</source>]</p>

<p><q><qex>Savannahs</qex> are clear pieces of land without woods.</q> <rj><qau>Dampier.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Savanna flower</b></col> <fld>(Bot.)</fld>, <cd>a West Indian name for several climbing apocyneous plants of the genus <gen>Echites</gen>.</cd> -- <col><b>Savanna sparrow</b></col> <fld>(Zool.)</fld>, <cd>an American sparrow (<spn>Ammodramus sandwichensis</spn> or <spn>Passerculus savanna</spn>) of which several varieties are found on grassy plains from Alaska to the Eastern United States.</cd> -- <col><b>Savanna wattle</b></col> <fld>(Bot.)</fld>, <cd>a name of two West Indian trees of the genus <gen>Citharexylum</gen>.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Savant</ent><br/
||<hw>Sa`vant"</hw> <pr>(?)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Savants</plw> <pr>(F. <?/; E. <?/)</pr>.</plu> <ety>[F., fr. <ets>savoir</ets> to know, L. <ets>sapere</ets>. See <er>Sage</er>, <pos>a.</pos>]</ety> <def>A man of learning; one versed in literature or science; a person eminent for acquirements.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Save</ent><br/
<hw>Save</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[See <er>Sage</er> the herb.]</ety> <def>The herb sage, or salvia.</def> <mark>[Obs.]</mark> <rj><au>Chaucer.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Save</ent><br/
<hw>Save</hw> <pr>(?)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Saved</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Saving</conjf>.]</vmorph> <ety>[OE. <ets>saven</ets>, <ets>sauven</ets>, <ets>salven</ets>, OF. <ets>salver</ets>, <ets>sauver</ets>, F. <ets>sauver</ets>, L. <ets>salvare</ets>, fr. <ets>salvus</ets> saved, safe. See <er>Safe</er>, <pos>a.</pos>]</ety> <sn>1.</sn> <def>To make safe; to procure the safety of; to preserve from injury, destruction, or evil of any kind; to rescue from impending danger; <as>as, to <ex>save</ex> a house from the flames</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>God <qex>save</qex> all this fair company.</q> <rj><qau>Chaucer.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>He cried, saying, Lord, <qex>save</qex> me.</q> <rj><qau>Matt. xiv. 30.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Thou hast . . . quitted all to <qex>save</qex><br/
A world from utter loss.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Theol.)</fld> <def>Specifically, to deliver from sin and its penalty; to rescue from a state of condemnation and spiritual death, and bring into a state of spiritual life.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Christ Jesus came into the world to <qex>save</qex> sinners.</q> <rj><qau>1 Tim. i. 15.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>To keep from being spent or lost; to secure from waste or expenditure; to lay up; to reserve.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Now save a nation, and now <qex>save</qex> a groat.</q> <rj><qau>Pope.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>To rescue from something undesirable or hurtful; to prevent from doing something; to spare.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>I'll <qex>save</qex> you<br/
That labor, sir. All's now done.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <def>To hinder from doing, suffering, or happening; to obviate the necessity of; to prevent; to spare.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Will you not speak to <qex>save</qex> a lady's blush?</q> <rj><qau>Dryden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>6.</sn> <def>To hold possession or use of; to escape loss of.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Just <qex>saving</qex> the tide, and putting in a stock of merit.</q> <rj><qau>Swift.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>To save appearances</b></col>, <cd>to preserve a decent outside; to avoid exposure of a discreditable state of things.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><syn><b>Syn.</b> -- To preserve; rescue; deliver; protect; spare; reserve; prevent.</syn><br/
[<source>1913 Webster</source>]</p>

<p><ent>Save</ent><br/
<hw>Save</hw>, <pos>v. i.</pos> <def>To avoid unnecessary expense or expenditure; to prevent waste; to be economical.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Brass ordnance <qex>saveth</qex> in the quantity of the material.</q> <rj><qau>Bacon.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Save</ent><br/
<hw>Save</hw>, <pos>prep. <or/ conj.</pos> <ety>[F. <ets>sauf</ets>, properly adj., safe. See <er>Safe</er>, <pos>a.</pos>]</ety> <def>Except; excepting; not including; leaving out; deducting; reserving; saving.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Five times received I forty stripes <qex>save</qex> one.</q> <rj><qau>2 Cor. xi. 24.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><syn><b>Syn.</b> -- See <er>Except</er>.</syn><br/
[<source>1913 Webster</source>]</p>

<p><ent>Save</ent><br/
<hw>Save</hw>, <pos>conj.</pos> <def>Except; unless.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Saveable</ent><br/
<hw>Save"a*ble</hw> <pr>(?)</pr>, <pos>a.</pos> <def>See <er>Savable</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Save-all</ent><br/
<hw>Save"-all`</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[<ets>Save</ets> + <ets>all</ets>.]</ety> <def>Anything which saves fragments, or prevents waste or loss.</def> Specifically: <sd>(a)</sd> <def>A device in a candlestick to hold the ends of candles, so that they be burned.</def> <sd>(b)</sd> <fld>(Naut.)</fld> <def>A small sail sometimes set under the foot of another sail, to catch the wind that would pass under it.</def> <rj><au>Totten.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sd>(c)</sd> A trough to prevent waste in a paper-making machine.<br/
[<source>1913 Webster</source>]</p>

<p><ent>Saveloy</ent><br/
<hw>Sav"e*loy</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F. <ets>cervelas</ets>, It. <ets>cervellata</ets>, fr. <ets>cervello</ets> brain, L. <ets>cerebellum</ets>, dim. of <ets>cerebrum</ets> brain. See <er>Cerebral</er>.]</ety> <def>A kind of dried sausage.</def> <rj><au>McElrath.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Savely</ent><br/
<hw>Save"ly</hw> <pr>(?)</pr>, <pos>adv.</pos> <def>Safely.</def> <mark>[Obs.]</mark> <rj><au>Chaucer.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Savement</ent><br/
<hw>Save"ment</hw> <pr>(?)</pr>, <pos>n.</pos> <def>The act of saving.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Saver</ent><br/
<hw>Sav"er</hw> <pr>(?)</pr>, <pos>n.</pos> <def>One who saves.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Savine</ent><br/
<ent>Savin</ent><br/
<mhw>{ <hw>Sav"in</hw>, <hw>Sav"ine</hw> <pr>(?)</pr>, }</mhw> <pos>n.</pos> <ety>[OE. <ets>saveine</ets>, AS. <ets>safinae</ets>, <ets>savine</ets>, L. <ets>sabina herba</ets>. Cf. <er>Sabine</er>.]</ety> <altsp>[Written also <asp>sabine</asp>.]</altsp> <fld>(Bot.)</fld> <sd>(a)</sd> <def>A coniferous shrub (<spn>Juniperus Sabina</spn>) of Western Asia, occasionally found also in the northern parts of the United States and in British America. It is a compact bush, with dark-colored foliage, and produces small berries having a glaucous bloom. Its bitter, acrid tops are sometimes used in medicine for gout, amenorrhoea, etc.</def> <sd>(b)</sd> <def>The North American red cedar (<spn>Juniperus Virginiana</spn>.)</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Saving</ent><br/
<hw>Sav"ing</hw> <pr>(?)</pr>, <pos>a.</pos> <sn>1.</sn> <def>Preserving; rescuing.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>He is the <qex>saving</qex> strength of his anointed.</q> <rj><qau>Ps. xxviii. 8.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Avoiding unnecessary expense or waste; frugal; not lavish or wasteful; economical; <as>as, a <ex>saving</ex> cook</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>Bringing back in returns or in receipts the sum expended; incurring no loss, though not gainful; <as>as, a <ex>saving</ex> bargain; the ship has made a <ex>saving</ex> voyage</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>Making reservation or exception; <as>as, a <ex>saving</ex> clause</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ <xex>Saving</xex> is often used with a noun to form a compound adjective; <as>as, labor-<ex>saving</ex>, life-<ex>saving</ex>, etc.</as></note><br/
[<source>1913 Webster</source>]</p>

<p><-- p. 1280 pr=PCP --></p>

<p><ent>Saving</ent><br/
<hw>Sav"ing</hw> <pr>(s<amac/v"<icr/ng)</pr>, <pos>prep. <or/ conj.</pos>; but properly a <pos>participle</pos>. <def>With the exception of; except; excepting; also, without disrespect to.</def> <ldquo/<xex>Saving</xex> your reverence.<rdquo/ <au>Shak.</au> <ldquo/<xex>Saving</xex> your presence.<rdquo/ <au>Burns.</au><br/
[<source>1913 Webster</source>]</p>

<p><q>None of us put off our clothes, <qex>saving</qex> that every one put them off for washing.</q> <rj><qau>Neh. iv. 23.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>And in the stone a new name written, which no man knoweth <qex>saving</qex> he that receiveth it.</q> <rj><qau>Rev. ii. 17.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Saving</ent><br/
<hw>Sav"ing</hw>, <pos>n.</pos> <sn>1.</sn> <def>Something kept from being expended or lost; that which is saved or laid up; <as>as, the <ex>savings</ex> of years of economy</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Exception; reservation.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Contend not with those that are too strong for us, but still with a <qex>saving</qex> to honesty.</q> <rj><qau>L'Estrange.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Savings bank</b></col>, <cd>a bank in which savings or earnings are deposited and put at interest.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Savingly</ent><br/
<hw>Sav"ing*ly</hw>, <pos>adv.</pos> <sn>1.</sn> <def>In a saving manner; with frugality or parsimony.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>So as to be finally saved from eternal death.</def><br/
[<source>1913 Webster</source>]</p>

<p><q><qex>Savingly</qex> born of water and the Spirit.</q> <rj><qau>Waterland.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Savingness</ent><br/
<hw>Sav"ing*ness</hw>, <pos>n.</pos> <sn>1.</sn> <def>The quality of being saving; carefulness not to expend money uselessly; frugality; parsimony.</def> <rj><au>Mrs. H. H. Jackson.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Tendency to promote salvation.</def> <rj><au>Johnson.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Savior</ent><br/
<hw>Sav"ior</hw> <pr>(s<amac/v"y<etil/r)</pr>, <pos>n.</pos> <ety>[OE. <ets>saveour</ets>, OF. <ets>salveor</ets>, F. <ets>sauveur</ets>, fr. L. <ets>salvator</ets>, fr. <ets>salvare</ets> to save. See <er>Save</er>, <pos>v.</pos>]</ety> <altsp>[Written also <asp>saviour</asp>.]</altsp> <sn>1.</sn> <def>One who saves, preserves, or delivers from destruction or danger.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Specifically: <xex>The</xex> (or <xex>our</xex>, <xex>your</xex>, etc.) <xex>Savior</xex>, he who brings salvation to men; Jesus Christ, the Redeemer.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Savioress</ent><br/
<hw>Sav"ior*ess</hw>, <pos>n.</pos> <def>A female savior.</def> <mark>[Written also saviouress.]</mark> <mark>[R.]</mark> <rj><au>Bp. Hall.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Savor</ent><br/
<hw>Sa"vor</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[OE. <ets>savour</ets>, <ets>savor</ets>, <ets>savur</ets>, OF. <ets>savor</ets>, <ets>savour</ets>, F. <ets>saveur</ets>, fr. L. <ets>sapor</ets>, fr. <ets>sapere</ets> to taste, savor. See <er>Sage</er>, <pos>a.</pos>, and cf. <er>Sapid</er>, <er>Insipid</er>, <er>Sapor</er>.]</ety> <altsp>[Written also <asp>savour</asp>.]</altsp> <sn>1.</sn> <def>That property of a thing which affects the organs of taste or smell; taste and odor; flavor; relish; scent; <as>as, the <ex>savor</ex> of an orange or a rose; an ill <ex>savor</ex></as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>I smell sweet <qex>savors</qex> and I feel soft things.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Hence, specific flavor or quality; characteristic property; distinctive temper, tinge, taint, and the like.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Why is not my life a continual joy, and the <qex>savor</qex> of heaven perpetually upon my spirit?</q> <rj><qau>Baxter.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>Sense of smell; power to scent, or trace by scent.</def> <mark>[R.]</mark> <ldquo/Beyond my <xex>savor</xex>.<rdquo/ <rj><au>Herbert.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>Pleasure; delight; attractiveness.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>She shall no <qex>savor</qex> have therein but lite.</q> <rj><qau>Chaucer.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><syn><b>Syn.</b> -- Taste; flavor; relish; odor; scent; smell.</syn><br/
[<source>1913 Webster</source>]</p>

<p><ent>Savor</ent><br/
<hw>Sa"vor</hw>, <pos>v. i.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Savored</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Savoring</conjf>.]</vmorph> <ety>[Cf. OF. <ets>savorer</ets>, F. <ets>savourer</ets>. See <er>Savor</er>, <pos>n.</pos>]</ety> <altsp>[Written also <asp>savour</asp>.]</altsp> <sn>1.</sn> <def>To have a particular smell or taste; -- with <xex>of</xex>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To partake of the quality or nature; to indicate the presence or influence; to smack; -- with <xex>of</xex>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>This <qex>savors</qex> not much of distraction.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>I have rejected everything that <qex>savors</qex> of party.</q> <rj><qau>Addison.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>To use the sense of taste.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>By sight, hearing, smelling, tasting or <qex>savoring</qex>, and feeling.</q> <rj><qau>Chaucer.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Savor</ent><br/
<hw>Sa"vor</hw>, <pos>v. t.</pos> <sn>1.</sn> <def>To perceive by the smell or the taste; hence, to perceive; to note.</def> <mark>[Obs.]</mark> <rj><au>B. Jonson.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To have the flavor or quality of; to indicate the presence of.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>That cuts us off from hope, and <qex>savors</qex> only<br/
Rancor and pride, impatience and despite.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>To taste or smell with pleasure; to delight in; to relish; to like; to favor.</def> <mark>[R.]</mark> <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Savorily</ent><br/
<hw>Sa"vor*i*ly</hw> <pr>(?)</pr>, <pos>adv.</pos> <def>In a savory manner.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Savoriness</ent><br/
<hw>Sa"vor*i*ness</hw>, <pos>n.</pos> <def>The quality of being savory.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Savorless</ent><br/
<hw>Sa"vor*less</hw>, <pos>a.</pos> <def>Having no savor; destitute of smell or of taste; insipid.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Savorly</ent><br/
<hw>Sa"vor*ly</hw>, <pos>a.</pos> <def>Savory.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Savorly</ent><br/
<hw>Sa"vor*ly</hw>, <pos>adv.</pos> <def>In a savory manner.</def> <mark>[Obs.]</mark> <rj><au>Barrow.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Savorous</ent><br/
<hw>Sa"vor*ous</hw> <pr>(-<ucr/s)</pr>, <pos>a.</pos> <ety>[Cf. F. <ets>savoureux</ets>, OF. <ets>saveros</ets>, L. <ets>saporosus</ets>. Cf. <er>Saporous</er>, and see <er>Savor</er>, <pos>n.</pos>]</ety> <def>Having a savor; savory.</def> <mark>[Obs.]</mark> <rj><au>Rom. of R.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Savory</ent><br/
<hw>Sa"vor*y</hw> <pr>(-<ycr/)</pr>, <pos>a.</pos> <ety>[From <er>Savor</er>.]</ety> <def>Pleasing to the organs of taste or smell.</def> <altsp>[Written also <asp>savoury</asp>.]</altsp><br/
[<source>1913 Webster</source>]</p>

<p><q>The chewing flocks<br/
Had ta'en their supper on the <qex>savory</qex> herb.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Savory</ent><br/
<hw>Sa"vo*ry</hw> <pr>(s<amac/"v<osl/*r<ycr/)</pr>, <pos>n.</pos> <ety>[F. <ets>savor<eacute/e</ets>; cf. It. <ets>santoreggia</ets>, <ets>satureja</ets>, L. <ets>satureia</ets>,]</ety> <fld>(Bot.)</fld> <def>An aromatic labiate plant (<spn>Satureia hortensis</spn>), much used in cooking; -- also called <altname>summer savory</altname>.</def> <altsp>[Written also <asp>savoury</asp>.]</altsp><br/
[<source>1913 Webster</source>]</p>

<p><ent>Savoy</ent><br/
<hw>Sa*voy"</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F. <ets>chou de Savoie</ets> cabbage of Savoy.]</ety> <fld>(Bot.)</fld> <def>A variety of the common cabbage (<spn>Brassica oleracea major</spn>), having curled leaves, -- much cultivated for winter use.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Savoyard</ent><br/
<hw>Sav`oy*ard"</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F.]</ety> <def>A native or inhabitant of Savoy.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Savvey</ent><br/
<ent>Savvy</ent><br/
<mhw>{ <hw>Sav"vy</hw>, <hw>Sav"vey</hw>  }</mhw> <pr>(?)</pr>, <pos>v. t. & i.</pos> <altsp>[Written also <asp>savey</asp>.]</altsp> <ety>[Sp. <ets>saber</ets> to know, <ets>sabe usted</ets> do you know?]</ety> <def>To understand; to comprehend; know.</def> <mark>[Slang, U. S.]</mark><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Savvey</ent><br/
<ent>Savvy</ent><br/
<mhw>{ <hw>Sav"vy</hw>, <hw>Sav"vey</hw>  }</mhw>, <pos>n.</pos> <def>Comprehension; knowledge of affairs; mental grasp; also, practical know-how; common sense.</def> <mark>[Slang, U. S.]</mark> <def2>also <pos>adj.</pos> <def>knowledgeable; well-informed; clever; canny; wise.</def></def2><br/
[<source>Webster 1913 Suppl.</source> <source>+PJC</source>]</p>

<p><ent>Saw</ent><br/
<hw>Saw</hw> <pr>(s<add/)</pr>, <def><pos>imp.</pos> of <er>See</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Saw</ent><br/
<hw>Saw</hw>, <pos>n.</pos> <ety>[OE. <ets>sawe</ets>, AS. sagu; akin to <ets>secgan</ets> to say. See <er>Say</er>, <pos>v. t.</pos> and cf. <er>Saga</er>.]</ety><br/
[<source>1913 Webster</source>]</p>

<p><sn>1.</sn> <def>Something said; speech; discourse.</def> <mark>[Obs.]</mark> <ldquo/To hearken all his <xex>sawe</xex>.<rdquo/ <rj><au>Chaucer.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A saying; a proverb; a maxim.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>His champions are the prophets and apostles,<br/
His weapons holy <qex>saws</qex> of sacred writ.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>Dictate; command; decree.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>[Love] rules the creatures by his powerful <qex>saw</qex>.</q> <rj><qau>Spenser.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Saw</ent><br/
<hw>Saw</hw>, <pos>n.</pos> <ety>[OE. <ets>sawe</ets>, AS. <ets>sage</ets>; akin to D. <ets>zaag</ets>, G. <ets>s<aum/ge</ets>, OHG. <ets>sega</ets>, <ets>saga</ets>, Dan. <ets>sav</ets>, Sw. <ets>s<aring/g</ets>, Icel. <ets>s<oum/g</ets>, L. <ets>secare</ets> to cut, <ets>securis</ets> ax, <ets>secula</ets> sickle. Cf. <er>Scythe</er>, <er>Sickle</er>, <er>Section</er>, <er>Sedge</er>.]</ety> <def>An instrument for cutting or dividing substances, as wood, iron, etc., consisting of a thin blade, or plate, of steel, with a series of sharp teeth on the edge, which remove successive portions of the material by cutting and tearing.</def><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ <xex>Saw</xex> is frequently used adjectively, or as the first part of a compound.</note><br/
[<source>1913 Webster</source>]</p>

<p><cs><mcol><col><b>Band saw</b></col>, <col><b>Crosscut saw</b></col>, etc.</mcol> <cd>See under <er>Band</er>, <er>Crosscut</er>, etc.</cd> -- <col><b>Circular saw</b></col>, <cd>a disk of steel with saw teeth upon its periphery, and revolved on an arbor.</cd> -- <col><b>Saw bench</b></col>, <cd>a bench or table with a flat top for for sawing, especially with a circular saw which projects above the table.</cd> -- <col><b>Saw file</b></col>, <cd>a three-cornered file, such as is used for sharpening saw teeth.</cd> -- <col><b>Saw frame</b></col>, <cd>the frame or sash in a sawmill, in which the saw, or gang of saws, is held.</cd> -- <col><b>Saw gate</b></col>, <cd>a saw frame.</cd> -- <col><b>Saw gin</b></col>, <cd>the form of cotton gin invented by Eli Whitney, in which the cotton fibers are drawn, by the teeth of a set of revolving circular saws, through a wire grating which is too fine for the seeds to pass.</cd> -- <col><b>Saw grass</b></col> <fld>(Bot.)</fld>, <cd>any one of certain cyperaceous plants having the edges of the leaves set with minute sharp teeth, especially the <spn>Cladium Mariscus</spn> of Europe, and the <spn>Cladium effusum</spn> of the Southern United States. Cf. <cref>Razor grass</cref>, under <er>Razor</er>.</cd> -- <col><b>Saw log</b></col>, <cd>a log of suitable size for sawing into lumber.</cd> -- <col><b>Saw mandrel</b></col>, <cd>a mandrel on which a circular saw is fastened for running.</cd> -- <col><b>Saw pit</b></col>, <cd>a pit over which timbor is sawed by two men, one standing below the timber and the other above.</cd> <au>Mortimer.</au> -- <col><b>Saw sharpener</b></col> <fld>(Zool.)</fld>, <cd>the great titmouse; -- so named from its harsh call note.</cd> <mark>[Prov. Eng.]</mark> -- <col><b>Saw whetter</b></col> <fld>(Zool.)</fld>, <cd>the marsh titmouse (<spn>Parus palustris</spn>); -- so named from its call note.</cd> <mark>[Prov. Eng.]</mark><-- no relation to saw-whet, an owl also named for its note --> -- <col><b>Scroll saw</b></col>, <cd>a ribbon of steel with saw teeth upon one edge, stretched in a frame and adapted for sawing curved outlines; also, a machine in which such a saw is worked by foot or power.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Saw</ent><br/
<hw>Saw</hw> <pr>(?)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp.</pos> <conjf>Sawed</conjf> <pr>(?)</pr>; <pos>p. p.</pos> <conjf>Sawed</conjf> <it>or</it> <conjf>Sawn</conjf> <pr>(<?/)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Sawing</conjf>.]</vmorph> <sn>1.</sn> <def>To cut with a saw; to separate with a saw; <as>as, to <ex>saw</ex> timber or marble</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To form by cutting with a saw; <as>as, to <ex>saw</ex> boards or planks, that is, to saw logs or timber into boards or planks; to <ex>saw</ex> shingles; to <ex>saw</ex> out a panel</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>Also used figuratively; <as>as, to <ex>saw</ex> the air</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Saw</ent><br/
<hw>Saw</hw>, <pos>v. i.</pos> <sn>1.</sn> <def>To use a saw; to practice sawing; <as>as, a man <ex>saws</ex> well</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To cut, as a saw; <as>as, the saw or mill <ex>saws</ex> fast</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>To be cut with a saw; <as>as, the timber <ex>saws</ex> smoothly</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sawarra nut</ent><br/
<hw>Sa*war"ra nut`</hw> <pr>(?)</pr>. <def>See <er>Souari nut</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sawbelly</ent><br/
<hw>Saw"bel`ly</hw> <pr>(?)</pr>, <pos>n.</pos> <def>The alewife.</def> <mark>[Local, U.S.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sawbill</ent><br/
<hw>Saw"bill`</hw> <pr>(?)</pr>, <pos>n.</pos> <def>The merganser.</def> <mark>[Prov. Eng.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sawbones</ent><br/
<hw>Saw"bones`</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A nickname for a surgeon.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sawbuck</ent><br/
<hw>Saw"buck`</hw> <pr>(?)</pr>, <pos>n.</pos> <sn>1.</sn> <def>A sawhorse.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <ety>[Colloq., from the Roman X for ten, like the support of a <ets>sawbuck</ets>.]</ety> <def>a ten-dollar bill; also, <col><b>double sawbuck</b></col>, a twenty-dollar bill.</def><br/
[<source>PJC</source>]</p>

<p><ent>Sawceflem</ent><br/
<hw>Saw"ce*flem</hw> <pr>(?)</pr>, <pos>a.</pos> <def>See <er>Sauseflem</er>.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sawder</ent><br/
<hw>Saw"der</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A corrupt spelling and pronunciation of <er>solder</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Soft sawder</b></col>, <cd>seductive praise; flattery; blarney.</cd> <mark>[Slang]</mark></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sawdust</ent><br/
<hw>Saw"dust`</hw> <pr>(?)</pr>, <pos>n.</pos> <def>Dust or small fragments of wood (or of stone, etc.) made by the cutting of a saw.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sawer</ent><br/
<hw>Saw"er`</hw> <pr>(?)</pr>, <pos>n.</pos> <def>One who saws; a sawyer.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sawfish</ent><br/
<hw>Saw"fish`</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Zool.)</fld> <def>Any one of several species of elasmobranch fishes of the genus <gen>Pristis</gen>. They have a sharklike form, but are more nearly allied to the rays. The flattened and much elongated snout has a row of stout toothlike structures inserted along each edge, forming a sawlike organ with which it mutilates or kills its prey.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sawfly</ent><br/
<hw>Saw"fly`</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Zool.)</fld> <def>Any one of numerous species of hymenopterous insects belonging to the family <fam>Tenthredinidae</fam>. The female usually has an ovipositor containing a pair of sawlike organs with which she makes incisions in the leaves or stems of plants in which to lay the eggs. The larvae resemble those of Lepidoptera.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sawhorse</ent><br/
<hw>Saw"horse`</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A kind of rack, shaped like a double St. Andrew's cross, on which sticks of wood are laid for sawing by hand; -- called also <altname>buck</altname>, and <altname>sawbuck</altname>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sawmill</ent><br/
<hw>Saw"mill`</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A mill for sawing, especially one for sawing timber or lumber.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sawneb</ent><br/
<hw>Saw"neb`</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A merganser.</def> <mark>[Prov. Eng.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Saw palmetto</ent><br/
<hw>Saw" pal*met"to</hw>. <def>See under <er>Palmetto</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Saw-set</ent><br/
<hw>Saw"-set`</hw> <pr>(?)</pr>, <pos>n.</pos> <def>An instrument used to set or turn the teeth of a saw a little sidewise, that they may make a kerf somewhat wider than the thickness of the blade, to prevent friction; -- called also <altname>saw-wrest</altname>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sawtooth</ent><br/
<hw>Saw"tooth`</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Zool.)</fld> <def>An arctic seal (<spn>Lobodon carcinophaga</spn>), having the molars serrated; -- called also <altname>crab-eating seal</altname>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Saw-toothed</ent><br/
<hw>Saw"-toothed"</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Having a tooth or teeth like those of a saw; serrate.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sawtry</ent><br/
<hw>Saw"try</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A psaltery.</def> <mark>[Obs.]</mark> <rj><au>Dryden.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Saw-whet</ent><br/
<hw>Saw"-whet`</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Zool.)</fld> <def>A small North American owl (<spn>Nyctale Acadica</spn>), destitute of ear tufts and having feathered toes; -- called also <altname>Acadian owl</altname>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Saw-wort</ent><br/
<hw>Saw"-wort`</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Bot.)</fld> <def>Any plant of the composite genus <gen>Serratula</gen>; -- so named from the serrated leaves of most of the species.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Saw-wrest</ent><br/
<hw>Saw"-wrest`</hw> <pr>(?)</pr>, <pos>n.</pos> <def>See <er>Saw-set</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sawyer</ent><br/
<hw>Saw"yer</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[<ets>Saw</ets> + <ets>-yer</ets>, as in lawyer. Cf. <er>Sawer</er>.]</ety> <sn>1.</sn> <def>One whose occupation is to saw timber into planks or boards, or to saw wood for fuel; a sawer.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A tree which has fallen into a stream so that its branches project above the surface, rising and falling with a rocking or swaying motion in the current.</def> <mark>[U.S.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <fld>(Zool.)</fld> <def>The bowfin.</def> <mark>[Local, U.S.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sax</ent><br/
<hw>Sax</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[AS. <ets>seax</ets> a knife.]</ety> <def>A kind of chopping instrument for trimming the edges of roofing slates.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Saxatile</ent><br/
<hw>Sax"a*tile</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>saxatilis</ets>, fr. <ets>saxum</ets> a rock: cf. F. <ets>saxatile</ets>.]</ety> <def>Of or pertaining to rocks; living among rocks; <as>as, a <ex>saxatile</ex> plant</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Saxhorn</ent><br/
<hw>Sax"horn`</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Mus.)</fld> <def>A name given to a numerous family of brass wind instruments with valves, invented by Antoine Joseph Adolphe <etsep>Sax</etsep> (known as Adolphe Sax), of Belgium and Paris, and much used in military bands and in orchestras.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Saxicava</ent><br/
<hw>Sax`i*ca"va</hw> <pr>(?)</pr>, <pos>n.</pos>; <plu><it>pl.</it> E. <plw>saxicavas</plw> <pr>(#)</pr>, L. <plw>Saxicavae</plw> <pr>(#)</pr>.</plu> <ety>[NL. See <er>Saxicavous</er>.]</ety> <fld>(Zool.)</fld> <def>Any species of marine bivalve shells of the genus <gen>Saxicava</gen>. Some of the species are noted for their power of boring holes in limestone and similar rocks.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Saxicavid</ent><br/
<hw>Sax`i*ca"vid</hw> <pr>(?)</pr>, <pos>a.</pos> <fld>(Zool.)</fld> <def>Of or pertaining to the saxicavas.</def> -- <def2><pos>n.</pos> <def>A saxicava.</def></def2><br/
[<source>1913 Webster</source>]</p>

<p><ent>Saxicavous</ent><br/
<hw>Sax`i*ca"vous</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>saxum</ets> rock + <ets>cavare</ets> to make hollow, fr. <ets>cavus</ets> hollow: cf. F. <ets>saxicave</ets>.]</ety> <fld>(Zool.)</fld> <def>Boring, or hollowing out, rocks; -- said of certain mollusks which live in holes which they burrow in rocks. See <xex>Illust.</xex> of <er>Lithodomus</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Saxicoline</ent><br/
<hw>Sax*ic"o*line</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>saxum</ets> a rock + <ets>colere</ets> to inhabit.]</ety> <fld>(Zool.)</fld> <def>Stone-inhabiting; pertaining to, or having the characteristics of, the stonechats.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Saxicolous</ent><br/
<hw>Sax*ic"o*lous</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[See <er>Saxicoline</er>.]</ety> <fld>(Bot.)</fld> <def>Growing on rocks.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Saxifraga</ent><br/
||<hw>Sax*if"ra*ga</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L., saxifrage. See <er>Saxifrage</er>.]</ety> <fld>(Bot.)</fld> <def>A genus of exogenous polypetalous plants, embracing about one hundred and eighty species. See <er>Saxifrage</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Saxifragaceous</ent><br/
<hw>Sax`i*fra*ga"ceous</hw> <pr>(?)</pr>, <pos>a.</pos> <fld>(Bot.)</fld> <def>Of or pertaining to a natural order of plants (<ord>Saxifragaceae</ord>) of which saxifrage is the type. The order includes also the alum root, the hydrangeas, the mock orange, currants and gooseberries, and many other plants.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Saxifragant</ent><br/
<hw>Sax*if"ra*gant</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[See <er>Saxifrage</er>.]</ety> <def>Breaking or destroying stones; saxifragous.</def> <mark>[R.]</mark> -- <def2><pos>n.</pos> <def>That which breaks or destroys stones.</def></def2> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Saxifrage</ent><br/
<hw>Sax"i*frage</hw> <pr>(?; 48)</pr>, <pos>n.</pos> <ety>[L. <ets>saxifraga</ets>, from <ets>saxifragus</ets> stone-breaking; <ets>saxum</ets> rock + <ets>frangere</ets> to break: cf. F. <ets>saxifrage</ets>. See <er>Fracture</er>, and cf. <er>Sassafras</er>, <er>Saxon</er>.]</ety> <fld>(Bot.)</fld> <def>Any plant of the genus <gen>Saxifraga</gen>, mostly perennial herbs growing in crevices of rocks in mountainous regions.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Burnet saxifrage</b></col>, <cd>a European umbelliferous plant (<spn>Pimpinella Saxifraga</spn>).</cd> -- <col><b>Golden saxifrage</b></col>, <cd>a low half-succulent herb (<spn>Chrysosplenium oppositifolium</spn>) growing in rivulets in Europe; also, <spn>Chrysosplenium Americanum</spn>, common in the United States. See also under <er>Golden</er>.</cd> -- <mcol><col><b>Meadow saxifrage</b></col>, or <col><b>Pepper saxifrage</b></col></mcol>. <cd>See under <er>Meadow</er>.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Saxifragous</ent><br/
<hw>Sax*if"ra*gous</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>saxifragus</ets>: cf. F. <ets>saxifrage</ets>. See <er>Saxifrage</er>.]</ety> <def>Dissolving stone, especially dissolving stone in the bladder.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Saxon</ent><br/
<hw>Sax"on</hw> <pr>(s<acr/ks"<ucr/n <it>or</it> -'n)</pr>, <pos>n.</pos> <ety>[L. <ets>Saxo</ets>, pl. <ets>Saxones</ets>, from the Saxon national name; cf. AS. pl. <ets>Seaxe</ets>, <ets>Seaxan</ets>, fr. <ets>seax</ets> a knife, a short sword, a dagger (akin to OHG. <ets>sahs</ets>, and perhaps to L. <ets>saxum</ets> rock, stone, knives being originally made of stone); and cf. G. <ets>Sachse</ets>, pl. <ets>Sachsen</ets>. Cf. <er>Saxifrage</er>.]</ety> <sn>1.</sn> <sd>(a)</sd> <def>One of a nation or people who formerly dwelt in the northern part of Germany, and who, with other Teutonic tribes, invaded and conquered England in the fifth and sixth centuries.</def> <sd>(b)</sd> <def>Also used in the sense of <xex>Anglo-Saxon</xex>.</def> <sd>(c)</sd> <def>A native or inhabitant of modern Saxony.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>The language of the Saxons; Anglo-Saxon.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Old Saxon</b></col>, <cd>the Saxon of the continent of Europe in the old form of the language, as shown particularly in the <ldquo/Heliand<rdquo/, a metrical narration of the gospel history preserved in manuscripts of the 9th century.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Saxon</ent><br/
<hw>Sax"on</hw>, <pos>a.</pos> <def>Of or pertaining to the Saxons, their country, or their language.</def> <sd>(b)</sd> <def>Anglo-Saxon.</def> <sd>(c)</sd> <def>Of or pertaining to Saxony or its inhabitants.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Saxon blue</b></col> <fld>(Dyeing)</fld>, <cd>a deep blue liquid used in dyeing, and obtained by dissolving indigo in concentrated sulphuric acid.</cd> <au>Brande & C.</au> -- <col><b>Saxon green</b></col> <fld>(Dyeing)</fld>, <cd>a green color produced by dyeing with yellow upon a ground of Saxon blue.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Saxonic</ent><br/
<hw>Sax*on"ic</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Relating to the Saxons or Anglo- Saxons.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Saxonism</ent><br/
<hw>Sax"on*ism</hw> <pr>(?)</pr>, <pos>n.</pos> <def>An idiom of the Saxon or Anglo-Saxon language.</def> <rj><au>T. Warton.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Saxonist</ent><br/
<hw>Sax"on*ist</hw>, <pos>n.</pos> <def>One versed in the Saxon language.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Saxonite</ent><br/
<hw>Sax"on*ite</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Min.)</fld> <def>See <cref>Mountain soap</cref>, under <er>Mountain</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Saxony</ent><br/
<hw>Sax"o*ny</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[So named after the kingdom of <etsep>Saxony</etsep>, reputed to produce fine wool.]</ety> <sn>1.</sn> <def>A kind of glossy woolen cloth formerly much used.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><sn>2.</sn>  <def>Saxony yarn, or flannel made of it or similar yarn.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Saxony yarn</ent><br/
<hw>Saxony yarn</hw>. <def>A fine grade of woolen yarn twisted somewhat harder and smoother than zephyr yarn.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Saxophone</ent><br/
<hw>Sax"o*phone</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[A.A.J. <ets>Sax</ets>, the inventor (see <er>Saxhorn</er>) + Gr. <?/ tone.]</ety> <fld>(Mus.)</fld> <def>A wind instrument of brass, containing a reed, and partaking of the qualities both of a brass instrument and of a clarinet.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sax-tuba</ent><br/
<hw>Sax"-tu`ba</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[See <er>Saxhorn</er>, and <er>Tube</er>.]</ety> <fld>(Mus.)</fld> <def>A powerful instrument of brass, curved somewhat like the Roman <xex>buccina</xex>, or tuba.</def><br/
[<source>1913 Webster</source>]</p>

<p><-- p. 1281 pr=PCP --></p>

<p><ent>Say</ent><br/
<hw>Say</hw> <pr>(s<amac/)</pr>, <mark>obs.</mark> <pos>imp.</pos> <mord>of <er>See</er></mord>. <def>Saw.</def> <rj><au>Chaucer.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Say</ent><br/
<hw>Say</hw> <pr>(s<amac/)</pr>, <pos>n.</pos> <ety>[Aphetic form of <ets>assay</ets>.]</ety> <sn>1.</sn> <def>Trial by sample; assay; sample; specimen; smack.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>If those principal works of God . . . be but certain tastes and <qex>says</qex>, as it were, of that final benefit.</q> <rj><qau>Hooker.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Thy tongue some <qex>say</qex> of breeding breathes.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Tried quality; temper; proof.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>He found a sword of better <qex>say</qex>.</q> <rj><qau>Spenser.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>Essay; trial; attempt.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>To give a say at</b></col>, <cd>to attempt.</cd> <rj><au>B. Jonson.</au></rj></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Say</ent><br/
<hw>Say</hw>, <pos>v. t.</pos> <def>To try; to assay.</def> <mark>[Obs.]</mark> <rj><au>B. Jonson.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Say</ent><br/
<hw>Say</hw>, <pos>n.</pos> <ety>[OE. <ets>saie</ets>, F. <ets>saie</ets>, fr. L. <ets>saga</ets>, equiv. to <ets>sagum</ets>, <ets>sagus</ets>, a coarse woolen mantle; cf. Gr. <grk>sa`gos</grk>. See <er>Sagum</er>.]</ety> <sn>1.</sn> <def>A kind of silk or satin.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>Thou <qex>say</qex>, thou serge, nay, thou buckram lord!</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A delicate kind of serge, or woolen cloth.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>His garment neither was of silk nor <qex>say</qex>.</q> <rj><qau>Spenser.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Say</ent><br/
<hw>Say</hw>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Said</conjf> <pr>(s<ecr/d)</pr>, contracted from <xex>sayed</xex>; <pos>p. pr. & vb. n.</pos> <conjf>Saying</conjf>.]</vmorph> <ety>[OE. <ets>seggen</ets>, <ets>seyen</ets>, <ets>siggen</ets>, <ets>sayen</ets>, <ets>sayn</ets>, AS. <ets>secgan</ets>; akin to OS. <ets>seggian</ets>, D. <ets>zeggen</ets>, LG. <ets>seggen</ets>, OHG. <ets>sag<emac/n</ets>, G. <ets>sagen</ets>, Icel. <ets>segja</ets>, Sw. <ets>s<aum/ga</ets>, Dan. <ets>sige</ets>, Lith. <ets>sakyti</ets>; cf. OL. <ets>insece</ets> tell, relate, Gr. <grk>'e`nnepe</grk> (for <grk>'en-sepe</grk>), <grk>'e`spete</grk>. Cf. <er>Saga</er>, <er>Saw</er> a saying.]</ety> <sn>1.</sn> <def>To utter or express in words; to tell; to speak; to declare; <as>as, he <ex>said</ex> many wise things</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Arise, and <qex>say</qex> how thou camest here.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To repeat; to rehearse; to recite; to pronounce; <as>as, to <ex>say</ex> a lesson</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Of my instruction hast thou nothing bated<br/
In what thou hadst to <qex>say</qex>?</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>After which shall be <qex>said</qex> or sung the following hymn.</q> <rj><qau>Bk. of Com. Prayer.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>To announce as a decision or opinion; to state positively; to assert; hence, to form an opinion upon; to be sure about; to be determined in mind as to.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>But what it is, hard is to <qex>say</qex>.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>To mention or suggest as an estimate, hypothesis, or approximation; hence, to suppose; -- in the imperative, followed sometimes by the subjunctive; <as>as, he had, <ex>say</ex> fifty thousand dollars; the fox had run, <ex>say</ex> ten miles</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q><qex>Say</qex>, for nonpayment that the debt should double,<br/
Is twenty hundred kisses such a trouble?</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><mcol><col><b>It is said</b></col>, <it>or</it> <col><b>They say</b></col></mcol>, <cd>it is commonly reported; it is rumored; people assert or maintain.</cd> -- <col><b>That is to say</b></col>, <cd>that is; in other words; otherwise.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Say</ent><br/
<hw>Say</hw>, <pos>v. i.</pos> <def>To speak; to express an opinion; to make answer; to reply.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>You have <qex>said</qex>; but whether wisely or no, let the forest judge.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>To this argument we shall soon have <qex>said</qex>; for what concerns it us to hear a husband divulge his household privacies?</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Say</ent><br/
<hw>Say</hw>, <pos>n.</pos> <ety>[From <er>Say</er>, <pos>v. t.</pos>; cf. <er>Saw</er> a saying.]</ety> <def>A speech; something said; an expression of opinion; a current story; a maxim or proverb.</def> <mark>[Archaic or Colloq.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>He no sooner said out his <qex>say</qex>, but up rises a cunning snap.</q> <rj><qau>L'Estrange.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>That strange palmer's boding <qex>say</qex>,<br/
That fell so ominous and drear<br/
Full on the object of his fear.</q> <rj><qau>Sir W. Scott.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sayer</ent><br/
<hw>Say"er</hw> <pr>(?)</pr>, <pos>n.</pos> <def>One who says; an utterer.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Mr. Curran was something much better than a <qex>sayer</qex> of smart sayings.</q> <rj><qau>Jeffrey.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sayette</ent><br/
<hw>Sa*yette"</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F. Cf. <er>Say</er> a kind of serge.]</ety> <def>A mixed stuff, called also <altname>sagathy</altname>. See <er>Sagathy</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Saying</ent><br/
<hw>Say"ing</hw> <pr>(?)</pr>, <pos>n.</pos> <def>That which is said; a declaration; a statement, especially a proverbial one; an aphorism; a proverb.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Many are the <qex>sayings</qex> of the wise,<br/
In ancient and in modern books enrolled.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><syn><b>Syn.</b> -- Declaration; speech; adage; maxim; aphorism; apothegm; saw; proverb; byword.</syn><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sayman</ent><br/
<hw>Say"man</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[<ets>Say</ets> sample + <ets>man</ets>.]</ety> <def>One who assays.</def> <mark>[Obs.]</mark> <rj><au>Bacon.</au></rj> <br/
[<source>1913 Webster</source>]</p>

<p><ent>Saymaster</ent><br/
<hw>Say"mas`ter</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A master of assay; one who tries or proves.</def> <mark>[Obs.]</mark> <ldquo/Great <xex>saymaster</xex> of state.<rdquo/ <rj><au>B. Jonson.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Saynd</ent><br/
<-- I have amended "Saynd" to read more like the first "Say". PCP --><hw>Saynd</hw> <pr>(?)</pr>, <mark>obs.</mark> <def><pos>p. p.</pos> <mord>of <er>Senge</er></mord>, to singe.</def> <rj><au>Chaucer.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>'Sblood</ent><br/
<hw>'Sblood</hw> <pr>(?)</pr>, <pos>interj.</pos> <def>An abbreviation of <xex>God's blood</xex>; -- used as an oath.</def> <mark>[Obs.]</mark> <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scab</ent><br/
<hw>Scab</hw> <pr>(sk<acr/b)</pr>, <pos>n.</pos> <ety>[OE. <ets>scab</ets>, <ets>scabbe</ets>, <ets>shabbe</ets>; cf. AS. <ets>scaeb</ets>, <ets>sceabb</ets>, <ets>scebb</ets>, Dan. & Sw. <ets>skab</ets>, and also L. <ets>scabies</ets>, fr. <ets>scabere</ets> to scratch, akin to E. <ets>shave</ets>. See <er>Shave</er>, and cf. <er>Shab</er>, <er>Shabby</er>.]</ety> <sn>1.</sn> <def>An incrustation over a sore, wound, vesicle, or pustule, formed by the drying up of the discharge from the diseased part.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>The itch in man; also, the scurvy.</def> <mark>[Colloq. or Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>The mange, esp. when it appears on sheep.</def> <rj><au>Chaucer.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>A disease of potatoes producing pits in their surface, caused by a minute fungus (<spn>Tiburcinia Scabies</spn>).</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <fld>(Founding)</fld> <def>A slight irregular protuberance which defaces the surface of a casting, caused by the breaking away of a part of the mold.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>6.</sn> <def>A mean, dirty, paltry fellow.</def> <mark>[Low]</mark> <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>7.</sn> <def>A nickname for a workman who engages for lower wages than are fixed by the trades unions; also, for one who takes the place of a workman on a strike.</def> <mark>[Cant]</mark><br/
[<source>1913 Webster</source>]</p>

<p><sn>8.</sn> <fld>(Bot.)</fld> <def>Any one of various more or less destructive fungus diseases attacking cultivated plants, and usually forming dark-colored crustlike spots.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Scab</ent><br/
<hw>Scab</hw>, <pos>v. i.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Scabbed</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Scabbing</conjf>.]</vmorph> <sn>1.</sn> <def>To become covered with a scab; <as>as, the wound <ex>scabbed</ex> over</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>to take the place of a striking worker.</def><br/
[<source>PJC</source>]</p>

<p><ent>Scabbard</ent><br/
<hw>Scab"bard</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[OE. <ets>scaubert</ets>, <ets>scauberk</ets>, OF. <ets>escaubers</ets>, <ets>escauberz</ets>, pl., scabbards, probably of German or Scan. origin; cf. Icel. <ets>sk<amac/lpr</ets> scabbard, and G. <ets>bergen</ets> to conceal. Cf. <er>Hauberk</er>.]</ety> <def>The case in which the blade of a sword, dagger, etc., is kept; a sheath.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Nor in thy <qex>scabbard</qex> sheathe that famous blade.</q> <rj><qau>Fairfax.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Scabbard fish</b></col> <fld>(Zool.)</fld>, <cd>a long, compressed, silver-colored taenioid fish (<spn>Lepidopus argyreus</spn> syn. <spn>Lepidopus caudatus</spn>), found on the European coasts, and more abundantly about New Zealand, where it is called <altname>frostfish</altname> and considered an excellent food fish.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scabbard</ent><br/
<hw>Scab"bard</hw> <pr>(?)</pr>, <pos>v. t.</pos> <def>To put in a scabbard.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scabbard plane</ent><br/
<hw>Scab"bard plane`</hw> <pr>(?)</pr>. <def>See <cref>Scaleboard plane</cref>, under <er>Scaleboard</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scabbed</ent><br/
<hw>Scab"bed</hw> <pr>(? <or/ ?)</pr>, <pos>a.</pos> <sn>1.</sn> <def>Abounding with scabs; diseased with scabs.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Fig.: Mean; paltry; vile; worthless.</def> <rj><au>Bacon.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scabbedness</ent><br/
<hw>Scab"bed*ness</hw> <pr>(?)</pr>, <pos>n.</pos> <def>Scabbiness.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scabbily</ent><br/
<hw>Scab"bi*ly</hw> <pr>(?)</pr>, <pos>adv.</pos> <def>In a scabby manner.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scabbiness</ent><br/
<hw>Scab"bi*ness</hw>, <pos>n.</pos> <def>The quality or state of being scabby.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scabble</ent><br/
<hw>Scab"ble</hw> <pr>(?)</pr>, <pos>v. t.</pos> <def>See <er>Scapple</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scabby</ent><br/
<hw>Scab"by</hw> <pr>(?)</pr>, <pos>a.</pos> <amorph>[<pos>Compar.</pos> <adjf>Scabbier</adjf> <pr>(<?/)</pr>; <pos>superl.</pos> <adjf>Scabbiest</adjf>.]</amorph> <sn>1.</sn> <def>Affected with scabs; full of scabs.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Diseased with the scab, or mange; mangy.</def> <rj><au>Swift.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scabies</ent><br/
||<hw>Sca"bi*es</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Med.)</fld> <def>The itch.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scabious</ent><br/
<hw>Sca"bi*ous</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>scabiosus</ets>, from <ets>scabies</ets> the scab: cf. F. <ets>scabieux</ets>.]</ety> <def>Consisting of scabs; rough; itchy; leprous; <as>as, <ex>scabious</ex> eruptions</as>.</def> <rj><au>Arbuthnot.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scabious</ent><br/
<hw>Sca"bi*ous</hw>, <pos>n.</pos> <ety>[Cf. F. <ets>scabieuse</ets>. See <er>Scabious</er>, <pos>a.</pos>]</ety> <fld>(Bot.)</fld> <def>Any plant of the genus <gen>Scabiosa</gen>, several of the species of which are common in Europe. They resemble the <fam>Compositae</fam>, and have similar heads of flowers, but the anthers are not connected.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Sweet scabious</b></col>. <sd>(a)</sd> <cd>Mourning bride</cd>. <sd>(b)</sd> <cd>A daisylike plant (<spn>Erigeron annuus</spn>) having a stout branching stem.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scabling</ent><br/
<hw>Scab"ling</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[See <er>Scapple</er>.]</ety> <def>A fragment or chip of stone.</def> <altsp>[Written also <asp>scabline</asp>.]</altsp><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scabredity</ent><br/
<hw>Sca*bred"i*ty</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>scabredo</ets>, fr. <ets>scaber</ets> rough.]</ety> <def>Roughness; ruggedness.</def> <mark>[Obs.]</mark> <rj><au>Burton.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scabrous</ent><br/
<hw>Sca"brous</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>scabrosus</ets>, fr. <ets>scaber</ets> rough: cf. F. <ets>scabreux</ets>.]</ety> <sn>1.</sn> <def>Rough to the touch, like a file; having small raised dots, scales, or points; scabby; scurfy; scaly.</def> <rj><au>Arbuthnot.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Fig.: Harsh; unmusical.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>His verse is <qex>scabrous</qex> and hobbling.</q> <rj><qau>Dryden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scabrousness</ent><br/
<hw>Sca"brous*ness</hw>, <pos>n.</pos> <def>The quality of being scabrous.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scabwort</ent><br/
<hw>Scab"wort`</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Bot.)</fld> <def>Elecampane.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scad</ent><br/
<hw>Scad</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Gael. & Ir. <ets>sgadan</ets> a herring.]</ety> <fld>(Zool.)</fld> <sd>(a)</sd> <def>A small carangoid fish (<spn>Trachurus saurus</spn>) abundant on the European coast, and less common on the American. The name is applied also to several allied species.</def> <sd>(b)</sd> <def>The goggler; -- called also <altname>big-eyed scad</altname>. See <er>Goggler</er>.</def> <sd>(c)</sd> <def>The friar skate.</def> <mark>[Scot.]</mark> <sd>(d)</sd> <def>The cigar fish, or round robin.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scaffold</ent><br/
<hw>Scaf"fold</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[OF. <ets>eschafault</ets>, <ets>eschafaut</ets>, <ets>escafaut</ets>, <ets>escadafaut</ets>, F. <ets><eacute/chafaud</ets>; probably originally the same word as E. & F. <ets>catafalque</ets>, It. <ets>catafalco</ets>. See <er>Catafalque</er>.]</ety> <sn>1.</sn> <def>A temporary structure of timber, boards, etc., for various purposes, as for supporting workmen and materials in building, for exhibiting a spectacle upon, for holding the spectators at a show, etc.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Pardon, gentles all,<br/
The flat, unraised spirits that have dared<br/
On this unworthy <qex>scaffold</qex> to bring forth<br/
So great an object.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Specifically, a stage or elevated platform for the execution of a criminal; <as>as, to die on the <ex>scaffold</ex></as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>That a <qex>scaffold</qex> of execution should grow a scaffold of coronation.</q> <rj><qau>Sir P. Sidney.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <fld>(Metal.)</fld> <def>An accumulation of adherent, partly fused material forming a shelf, or dome-shaped obstruction, above the tuy<egrave/res in a blast furnace.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scaffold</ent><br/
<hw>Scaf"fold</hw>, <pos>v. t.</pos> <def>To furnish or uphold with a scaffold.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scaffoldage</ent><br/
<hw>Scaf"fold*age</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A scaffold.</def> <mark>[R.]</mark> <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scaffolding</ent><br/
<hw>Scaf"fold*ing</hw>, <pos>n.</pos> <sn>1.</sn> <def>A scaffold; a supporting framework; <as>as, the <ex>scaffolding</ex> of the body</as>.</def> <rj><au>Pope.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Materials for building scaffolds.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scaglia</ent><br/
<hw>Scagl"ia</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[It. <ets>scaglia</ets> a scale, a shell, a chip of marble.]</ety> <def>A reddish variety of limestone.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scagliola</ent><br/
<hw>Scagl*io"la</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[It. <ets>scagliuola</ets>, dim. of <ets>scaglia</ets>. See <er>Scaglia</er>.]</ety> <def>An imitation of any veined and ornamental stone, as marble, formed by a substratum of finely ground gypsum mixed with glue, the surface of which, while soft, is variegated with splinters of marble, spar, granite, etc., and subsequently colored and polished.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scala</ent><br/
||<hw>Sca"la</hw> <pr>(?)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Scalae</plw> <pr>(#)</pr>.</plu> <ety>[L., a ladder.]</ety> <sn>1.</sn> <fld>(Surg.)</fld> <def>A machine formerly employed for reducing dislocations of the humerus.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Anat.)</fld> <def>A term applied to any one of the three canals of the cochlea.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scalable</ent><br/
<hw>Scal"a*ble</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Capable of being scaled.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scalado</ent><br/
<ent>Scalade</ent><br/
<mhw>{ <hw>Sca*lade"</hw> <pr>(?)</pr>, <hw>Sca*la"do</hw> <pr>(?)</pr>, }</mhw> <pos>n.</pos> <fld>(Mil.)</fld> <def>See <er>Escalade</er>.</def> <rj><au>Fairfax.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scalar</ent><br/
<hw>Sca"lar</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Math.)</fld> <def>In the quaternion analysis, a quantity that has magnitude, but not direction; -- distinguished from a <xex>vector</xex>, which has both magnitude and direction.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scalaria</ent><br/
||<hw>Sca*la"ri*a</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L., flight of steps.]</ety> <fld>(Zool.)</fld> <def>Any one of numerous species of marine gastropods of the genus <gen>Scalaria</gen>, or family <fam>Scalaridae</fam>, having elongated spiral turreted shells, with rounded whorls, usually crossed by ribs or varices. The color is generally white or pale. Called also <altname>ladder shell</altname>, and <altname>wentletrap</altname>. See <er>Ptenoglossa</er>, and <er>Wentletrap</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scalariform</ent><br/
<hw>Sca*lar"i*form</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>scalare</ets>, <ets>scalaria</ets>, staircase, ladder + <ets>-form</ets>: cf. F. <ets>scalariforme</ets>.]</ety> <sn>1.</sn> <def>Resembling a ladder in form or appearance; having transverse bars or markings like the rounds of a ladder; <as>as, the <ex>scalariform</ex> cells and <ex>scalariform</ex> pits in some plants</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Zool.)</fld> <def>Like or pertaining to a scalaria.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scalary</ent><br/
<hw>Sca"la*ry</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>scalaris</ets>, fr. <ets>scalae</ets>, pl. <ets>scala</ets>, staircase, ladder.]</ety> <def>Resembling a ladder; formed with steps.</def> <mark>[Obs.]</mark> <rj><au>Sir T. Browne.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scalawag</ent><br/
<hw>Scal"a*wag</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A scamp; a scapegrace.</def> <altsp>[Spelt also <asp>scallawag</asp>.]</altsp> <mark>[Slang, U.S.]</mark> <rj><au>Bartlett.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scald</ent><br/
<hw>Scald</hw> <pr>(?)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Scalded</conjf>; <pos>p. pr. & vb. n.</pos> <conjf>Scalding</conjf>.]</vmorph> <ety>[OF. <ets>eschalder</ets>, <ets>eschauder</ets>, <ets>escauder</ets>, F. <ets><eacute/chauder</ets>, fr. L. <ets>excaldare</ets>; <ets>ex + caldus</ets>, <ets>calidus</ets>, warm, hot. See <er>Ex</er>, and <er>Caldron</er>.]</ety> <sn>1.</sn> <def>To burn with hot liquid or steam; to pain or injure by contact with, or immersion in, any hot fluid; <as>as, to <ex>scald</ex> the hand</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Mine own tears<br/
Do <qex>scald</qex> like molten lead.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Here the blue flames of <qex>scalding</qex> brimstone fall.</q> <rj><qau>Cowley.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To expose to a boiling or violent heat over a fire, or in hot water or other liquor; <as>as, to <ex>scald</ex> milk or meat</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scald</ent><br/
<hw>Scald</hw>, <pos>n.</pos> <def>A burn, or injury to the skin or flesh, by some hot liquid, or by steam.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scald</ent><br/
<hw>Scald</hw>, <pos>a.</pos> <ety>[For <ets>scalled</ets>. See <er>Scall</er>.]</ety> <sn>1.</sn> <def>Affected with the scab; scabby.</def> <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Scurvy; paltry; <as>as, <ex>scald</ex> rhymers</as>.</def> <mark>[Obs.]</mark> <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Scald crow</b></col> <fld>(Zool.)</fld>, <cd>the hooded crow.</cd> <mark>[Ireland]</mark> -- <col><b>Scald head</b></col> <fld>(Med.)</fld>, <cd>a name popularly given to several diseases of the scalp characterized by pustules (the dried discharge of which forms scales) and by falling out of the hair.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scald</ent><br/
<hw>Scald</hw>, <pos>n.</pos> <def>Scurf on the head. See <er>Scall</er>.</def> <rj><au>Spenser.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scald</ent><br/
<hw>Scald</hw> <pr>(sk<acr/ld <it>or</it> sk<add/ld; 277)</pr>, <pos>n.</pos> <ety>[Icel. <ets>sk<amac/ld</ets>.]</ety> <def>One of the ancient Scandinavian poets and historiographers; a reciter and singer of heroic poems, eulogies, etc., among the Norsemen; more rarely, a bard of any of the ancient Teutonic tribes.</def> <altsp>[Written also <asp>skald</asp>.]</altsp><br/
[<source>1913 Webster</source>]</p>

<p><q>A war song such as was of yore chanted on the field of battle by the <qex>scalds</qex> of the yet heathen Saxons.</q> <rj><qau>Sir W. Scott.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scalder</ent><br/
<hw>Scald"er</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A Scandinavian poet; a scald.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scaldfish</ent><br/
<hw>Scald"fish`</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[<ets>Scald</ets>, a. + <ets>fish</ets>.]</ety> <fld>(Zool.)</fld> <def>A European flounder (<spn>Arnoglossus laterna</spn>, or <spn>Psetta arnoglossa</spn>); -- called also <altname>megrim</altname>, and <altname>smooth sole</altname>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scaldic</ent><br/
<hw>Scald"ic</hw> <pr>(? <or/ ?)</pr>, <pos>a.</pos> <def>Of or pertaining to the scalds of the Norsemen; <as>as, <ex>scaldic</ex> poetry</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scale</ent><br/
<hw>Scale</hw> <pr>(sk<amac/l)</pr>, <pos>n.</pos> <ety>[AS. <ets>sc<amac/le</ets>; perhaps influenced by the kindred Icel. <ets>sk<amac/l</ets> balance, dish, akin also to D. <ets>schaal</ets> a scale, bowl, shell, G. <ets>schale</ets>, OHG. <ets>sc<amac/la</ets>, Dan. <ets>skaal</ets> drinking cup, bowl, dish, and perh. to E. <ets>scale</ets> of a fish. Cf. <er>Scale</er> of a fish, <er>Skull</er> the brain case.]</ety> <sn>1.</sn> <def>The dish of a balance; hence, the balance itself; an instrument or machine for weighing; <as>as, to turn the <ex>scale</ex></as>; -- chiefly used in the plural when applied to the whole instrument or apparatus for weighing. Also used figuratively.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Long time in even <qex>scale</qex><br/
The battle hung.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>The <qex>scales</qex> are turned; her kindness weighs no more<br/
Now than my vows.</q> <rj><qau>Waller.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <pluf>pl.</pluf> <fld>(Astron.)</fld> <def>The sign or constellation Libra.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Platform scale</b></col>. <cd>See under <er>Platform</er>.</cd></cs> <-- <col><b>tip the scales</b></col>, influence an action so as to change an outcome from one likely result to another. --> <br/
[<source>1913 Webster</source>]</p>

<p><ent>Scale</ent><br/
<hw>Scale</hw>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Scaled</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Scaling</conjf>.]</vmorph> <def>To weigh or measure according to a scale; to measure; also, to grade or vary according to a scale or system.</def><br/
[<source>1913 Webster</source>]</p>

<p><q><qex>Scaling</qex> his present bearing with his past.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><mcol><col><b>To scale a debt, wages, etc.</b></col> <it>or</it> <col><b>To scale down a debt, wages, etc.</b></col></mcol>, <cd>to reduce a debt, etc., according to a fixed ratio or scale.</cd> <mark>[U.S.]</mark></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scale</ent><br/
<hw>Scale</hw>, <pos>n.</pos> <ety>[Cf. AS. <ets>scealu</ets>, <ets>scalu</ets>, a shell, parings; akin to D. <ets>schaal</ets>, G. <ets>schale</ets>, OHG. <ets>scala</ets>, Dan. & Sw. <ets>skal</ets> a shell, Dan. <ets>skiael</ets> a fish scale, Goth. <ets>skalja</ets> tile, and E. <ets>shale</ets>, <ets>shell</ets>, and perhaps also to <ets>scale</ets> of a balance; but perhaps rather fr. OF. <ets>escale</ets>, <ets>escaile</ets>, F. <ets><eacute/caille</ets> scale of a fish, and <ets><eacute/cale</ets> shell of beans, pease, eggs, nuts, of German origin, and akin to Goth. <ets>skalja</ets>, G. <ets>schale</ets>. See <er>Shale</er>.]</ety> <sn>1.</sn> <fld>(Anat.)</fld> <def>One of the small, thin, membranous, bony or horny pieces which form the covering of many fishes and reptiles, and some mammals, belonging to the dermal part of the skeleton, or dermoskeleton. See <er>Cycloid</er>, <er>Ctenoid</er>, and <er>Ganoid</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Fish that, with their fins and shining <qex>scales</qex>,<br/
Glide under the green wave.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Hence, any layer or leaf of metal or other material, resembling in size and thinness the scale of a fish; <as>as, a <ex>scale</ex> of iron, of bone, etc.</as></def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <fld>(Zool.)</fld> <def>One of the small scalelike structures covering parts of some invertebrates, as those on the wings of Lepidoptera and on the body of Thysanura; the elytra of certain annelids. See <er>Lepidoptera</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <fld>(Zool.)</fld> <def>A scale insect. (See below.)</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <fld>(Bot.)</fld> <def>A small appendage like a rudimentary leaf, resembling the scales of a fish in form, and often in arrangement; <as>as, the <ex>scale</ex> of a bud, of a pine cone, and the like</as>. The name is also given to the chaff on the stems of ferns.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>6.</sn> <def>The thin metallic side plate of the handle of a pocketknife. See <xex>Illust.</xex> of <er>Pocketknife</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>7.</sn> <def>An incrustation deposit on the inside of a vessel in which water is heated, as a steam boiler.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>8.</sn> <fld>(Metal.)</fld> <def>The thin oxide which forms on the surface of iron forgings. It consists essentially of the magnetic oxide, <chform>Fe3O4</chform>. Also, a similar coating upon other metals.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Covering scale</b></col> <fld>(Zool.)</fld>, <cd>a hydrophyllium.</cd> -- <col><b>Ganoid scale</b></col>. <fld>(Zool.)</fld> <cd>See under <er>Ganoid</er>.</cd> -- <col><b>Scale armor</b></col> <fld>(Mil.)</fld>, <cd>armor made of small metallic scales overlapping, and fastened upon leather or cloth.</cd> -- <col><b>Scale beetle</b></col> <fld>(Zool.)</fld>, <cd>the tiger beetle.</cd> -- <col><b>Scale carp</b></col> <fld>(Zool.)</fld>, <cd>a carp having normal scales.</cd> -- <col><b>Scale insect</b></col> <fld>(Zool.)</fld>, <cd>any one of numerous species of small hemipterous insects belonging to the family <fam>Coccidae</fam>, in which the females, when adult, become more or less scalelike in form. They are found upon the leaves and twigs of various trees and shrubs, and often do great damage to fruit trees. See <er>Orange scale</er>,under <er>Orange</er>.</cd> -- <col><b>Scale moss</b></col> <fld>(Bot.)</fld>, <cd>any leafy-stemmed moss of the order <ord>Hepaticae</ord>; -- so called from the small imbricated scalelike leaves of most of the species. See <er>Hepatica</er>, 2, and <er>Jungermannia</er>.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><-- p. 1282 pr=PCP --></p>

<p><ent>Scale</ent><br/
<hw>Scale</hw> <pr>(?)</pr>, <pos>v. t.</pos> <sn>1.</sn> <def>To strip or clear of scale or scales; <as>as, to <ex>scale</ex> a fish; to <ex>scale</ex> the inside of a boiler</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To take off in thin layers or scales, as tartar from the teeth; to pare off, as a surface.</def> <ldquo/If all the mountains were <xex>scaled</xex>, and the earth made even.<rdquo/ <rj><au>T. Burnet.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>To scatter; to spread.</def> <mark>[Scot. & Prov. Eng.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <fld>(Gun.)</fld> <def>To clean, as the inside of a cannon, by the explosion of a small quantity of powder.</def> <rj><au>Totten.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scale</ent><br/
<hw>Scale</hw>, <pos>v. i.</pos> <sn>1.</sn> <def>To separate and come off in thin layers or laminae; <as>as, some sandstone <ex>scales</ex> by exposure</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Those that cast their shell are the lobster and crab; the old skins are found, but the old shells never; so it is likely that they <qex>scale</qex> off.</q> <rj><qau>Bacon.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To separate; to scatter.</def> <mark>[Scot. & Prov. Eng.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scale</ent><br/
<hw>Scale</hw>, <pos>n.</pos> <ety>[L. <ets>scalae</ets>, pl., <ets>scala</ets> staircase, ladder; akin to <ets>scandere</ets> to climb. See <er>Scan</er>; cf. <er>Escalade</er>.]</ety> <sn>1.</sn> <def>A ladder; a series of steps; a means of ascending.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Hence, anything graduated, especially when employed as a measure or rule, or marked by lines at regular intervals.</def> Specifically: <sd>(a)</sd> <def>A mathematical instrument, consisting of a slip of wood, ivory, or metal, with one or more sets of spaces graduated and numbered on its surface, for measuring or laying off distances, etc., as in drawing, plotting, and the like. See <er>Gunter's scale</er>.</def> <sd>(b)</sd> <def>A series of spaces marked by lines, and representing proportionately larger distances; <as>as, a <ex>scale</ex> of miles, yards, feet, etc., for a map or plan</as>.</def> <sd>(c)</sd> <def>A basis for a numeral system; <as>as, the decimal <ex>scale</ex>; the binary <ex>scale</ex>, etc.</as></def> <sd>(d)</sd> <fld>(Mus.)</fld> <def>The graduated series of all the tones, ascending or descending, from the keynote to its octave; -- called also the <altname>gamut</altname>. It may be repeated through any number of octaves. See <cref>Chromatic scale</cref>, <cref>Diatonic scale</cref>, <cref>Major scale</cref>, and <cref>Minor scale</cref>, under <er>Chromatic</er>, <er>Diatonic</er>, <er>Major</er>, and <er>Minor</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>Gradation; succession of ascending and descending steps and degrees; progressive series; scheme of comparative rank or order; <as>as, a <ex>scale</ex> of being</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>There is a certain <qex>scale</qex> of duties . . . which for want of studying in right order, all the world is in confusion.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>Relative dimensions, without difference in proportion of parts; size or degree of the parts or components in any complex thing, compared with other like things; especially, the relative proportion of the linear dimensions of the parts of a drawing, map, model, etc., to the dimensions of the corresponding parts of the object that is represented; <as>as, a map on a <ex>scale</ex> of an inch to a mile</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Scale of chords</b></col>, <cd>a graduated scale on which are given the lengths of the chords of arcs from 0<deg/ to 90<deg/ in a circle of given radius, -- used in measuring given angles and in plotting angles of given numbers of degrees.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scale</ent><br/
<hw>Scale</hw>, <pos>v. t.</pos> <ety>[Cf. It. <ets>scalare</ets>, fr. L. <ets>scalae</ets>, <ets>scala</ets>. See <er>Scale</er> a ladder.]</ety> <def>To climb by a ladder, or as if by a ladder; to ascend by steps or by climbing; to clamber up; <as>as, to <ex>scale</ex> the wall of a fort</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Oft have I <qex>scaled</qex> the craggy oak.</q> <rj><qau>Spenser.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scale</ent><br/
<hw>Scale</hw>, <pos>v. i.</pos> <def>To lead up by steps; to ascend.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>Satan from hence, now on the lower stair,<br/
That <qex>scaled</qex> by steps of gold to heaven-gate,<br/
Looks down with wonder.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scaleback</ent><br/
<hw>Scale"back`</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Zool.)</fld> <def>Any one of numerous species of marine annelids of the family <fam>Polynoidae</fam>, and allies, which have two rows of scales, or elytra, along the back. See <xex>Illust.</xex> under <er>Chaetopoda</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scalebeam</ent><br/
<hw>Scale"beam`</hw> <pr>(?)</pr>, <pos>n.</pos> <sn>1.</sn> <def>The lever or beam of a balance; the lever of a platform scale, to which the poise for weighing is applied.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A weighing apparatus with a sliding weight, resembling a steelyard.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scaleboard</ent><br/
<hw>Scale"board`</hw> <pr>(?; <it>commonly</it> <?/)</pr>, <pos>n.</pos> <ety>[3d <ets>scale</ets> + <ets>board</ets>.]</ety> <sn>1.</sn> <fld>(Print.)</fld> <def>A thin slip of wood used to justify a page.</def> <mark>[Obs.]</mark> <rj><au>Crabb.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A thin veneer of leaf of wood used for covering the surface of articles of furniture, and the like.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Scaleboard plane</b></col>, <cd>a plane for cutting from a board a wide shaving forming a scaleboard.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scaled</ent><br/
<hw>Scaled</hw> <pr>(?)</pr>, <pos>a.</pos> <sn>1.</sn> <def>Covered with scales, or scalelike structures; -- said of a fish, a reptile, a moth, etc.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Without scales, or with the scales removed; <as>as, <ex>scaled</ex> herring</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <fld>(Zool.)</fld> <def>Having feathers which in form, color, or arrangement somewhat resemble scales; <as>as, the <ex>scaled</ex> dove</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Scaled dove</b></col> <fld>(Zool.)</fld>, <cd>any American dove of the genus <gen>Scardafella</gen>. Its colored feather tips resemble scales.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scaleless</ent><br/
<hw>Scale"less</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Destitute of scales.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scalene</ent><br/
<hw>Sca*lene"</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>scalenus</ets>, Gr. <?/: cf. F. <ets>scal<egrave/ne</ets>.]</ety> <sn>1.</sn> <fld>(Geom.)</fld> <sd>(a)</sd> <def>Having the sides and angles unequal; -- said of a triangle.</def> <sd>(b)</sd> <def>Having the axis inclined to the base, as a cone.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Anat.)</fld> <sd>(a)</sd> <def>Designating several triangular muscles called <xex>scalene muscles</xex>.</def> <sd>(b)</sd> <def>Of or pertaining to the scalene muscles.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Scalene muscles</b></col> <fld>(Anat.)</fld>, <cd>a group of muscles, usually three on each side in man, extending from the cervical vertebrae to the first and second ribs.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scalene</ent><br/
<hw>Sca*lene"</hw>, <pos>n.</pos> <fld>(Geom.)</fld> <def>A triangle having its sides and angles unequal.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scalenohedral</ent><br/
<hw>Sca*le`no*he"dral</hw> <pr>(sk<adot/*l<emac/`n<osl/*h<emac/"dr<ait/l)</pr>, <pos>a.</pos> <fld>(Crystallog.)</fld> <def>Of or pertaining to a scalenohedron.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scalenohedron</ent><br/
<hw>Sca*le`no*he"dron</hw> <pr>(-dr<ocr/n)</pr>, <pos>n.</pos> <ety>[Gr. <grk>skalhno`s</grk> uneven + <grk>"e`dra</grk> seat, base.]</ety> <fld>(Crystallog.)</fld> <def>A pyramidal form under the rhombohedral system, inclosed by twelve faces, each a scalene triangle.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scaler</ent><br/
<hw>Scal"er</hw> <pr>(?)</pr>, <pos>n.</pos> <def>One who, or that which, scales; specifically, a dentist's instrument for removing tartar from the teeth.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scale-winged</ent><br/
<hw>Scale"-winged`</hw> <pr>(?)</pr>, <pos>a.</pos> <fld>(Zool.)</fld> <def>Having the wings covered with small scalelike structures, as the Lepidoptera; scaly-winged.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scaliness</ent><br/
<hw>Scal"i*ness</hw> <pr>(?)</pr>, <pos>n.</pos> <def>The state of being scaly; roughness.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scaling</ent><br/
<hw>Scal"ing</hw> <pr>(sk<amac/l"<icr/ng)</pr>, <pos>a.</pos> <sn>1.</sn> <def>Adapted for removing scales, as from a fish; <as>as, a <ex>scaling</ex> knife</as>; adapted for removing scale, as from the interior of a steam boiler; <as>as, a <ex>scaling</ex> hammer, bar, etc.</as></def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Serving as an aid in clambering; <as>as, a <ex>scaling</ex> ladder, used in assaulting a fortified place</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scaliola</ent><br/
<hw>Scal*io"la</hw> <pr>(?)</pr>, <pos>n.</pos> <def>Same as <er>Scagliola</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scall</ent><br/
<hw>Scall</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Icel. <ets>skalli</ets> a bald head. Cf. <er>Scald</er>, <pos>a.</pos>]</ety> <def>A scurf or scabby disease, especially of the scalp.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>It is a dry <qex>scall</qex>, even a leprosy upon the head.</q> <rj><qau>Lev. xiii. 30.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scall</ent><br/
<hw>Scall</hw>, <pos>a.</pos> <def>Scabby; scurfy.</def> <mark>[Obs.]</mark> <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scalled</ent><br/
<hw>Scalled</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Scabby; scurfy; scall.</def> <mark>[Obs.]</mark> <ldquo/With <xex>scalled</xex> brows black.<rdquo/ <rj><au>Chaucer.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Scalled head</b></col>. <fld>(Med.)</fld> <cd>See <cref>Scald head</cref>, under <er>Scald</er>, <pos>a.</pos></cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scallion</ent><br/
<hw>Scal"lion</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[OF. <ets>escalone</ets>, <ets>eschaloingne</ets>, L. <ets>caepa Ascalonia</ets> onion of Ascalon; <ets>caepa</ets> onion + <ets>Ascalonius</ets> of Ascalon,  fr. <ets>Ascalo</ets> Ascalon, a town in Palestine. Cf. <er>Shallot</er>.]</ety> <sn>1.</sn> <fld>(Bot.)</fld> <def>A kind of small onion (<spn>Allium Ascalonicum</spn>), native of Palestine; the eschalot, or shallot.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Any onion which does not <ldquo/bottom out,<rdquo/ but remains with a thick stem like a leek.</def> <rj><au>Amer. Cyc.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scallop</ent><br/
<hw>Scal"lop</hw> <pr>(?; 277)</pr>, <pos>n.</pos> <ety>[OF. <ets>escalope</ets> a shell, probably of German or Dutch origin, and akin to E. <ets>scale of a fish</ets>; cf. D. <ets>schelp</ets> shell. See <er>Scale</er> of a fish, and cf. <er>Escalop</er>.]</ety> <altsp>[Written also <asp>scollop</asp>.]</altsp> <sn>1.</sn> <fld>(Zool.)</fld> <def>Any one of numerous species of marine bivalve mollusks of the genus Pecten and allied genera of the family <fam>Pectinidae</fam>. The shell is usually radially ribbed, and the edge is therefore often undulated in a characteristic manner. The large adductor muscle of some the species is much used as food. One species (<spn>Vola Jacobaeus</spn>) occurs on the coast of Palestine, and its shell was formerly worn by pilgrims as a mark that they had been to the Holy Land. Called also <altname>fan shell</altname>. See <er>Pecten</er>, 2.</def><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ The common edible scallop of the <geog>Eastern United States</geog> is <spn>Pecten irradians</spn>; the large sea scallop, also used as food, is <spn>Pecten Clintonius</spn> syn. <spn>Pecten tenuicostatus</spn>.</note><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>One of series of segments of circles joined at their extremities, forming a border like the edge or surface of a scallop shell.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>One of the shells of a scallop; also, a dish resembling a scallop shell.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scallop</ent><br/
<hw>Scal"lop</hw>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Scalloped</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Scalloping</conjf>.]</vmorph> <sn>1.</sn> <def>To mark or cut the edge or border of into segments of circles, like the edge or surface of a scallop shell. See <er>Scallop</er>, <pos>n.</pos>, 2.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Cookery)</fld> <def>To bake in scallop shells or dishes; to prepare with crumbs of bread or cracker, and bake. See <cref>Scalloped oysters</cref>, below.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scalloped</ent><br/
<hw>Scal"loped</hw> <pr>(?)</pr>, <pos>a.</pos> <sn>1.</sn> <def>Furnished with a scallop; made or done with or in a scallop.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Having the edge or border cut or marked with segments of circles. See <er>Scallop</er>, <pos>n.</pos>, 2.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <fld>(Cookery)</fld> <def>Baked in a scallop; cooked with crumbs.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Scalloped oysters</b></col> <fld>(Cookery)</fld>, <cd>opened oysters baked in a deep dish with alternate layers of bread or cracker crumbs, seasoned with pepper, nutmeg, and butter. This was at first done in scallop shells.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scalloper</ent><br/
<hw>Scal"lop*er</hw> <pr>(?)</pr>, <pos>n.</pos> <def>One who fishes for scallops.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scalloping</ent><br/
<hw>Scal"lop*ing</hw>, <pos>n.</pos> <def>Fishing for scallops.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scalp</ent><br/
<hw>Scalp</hw> <pr>(sk<acr/lp)</pr>, <pos>n.</pos> <ety>[Cf. <er>Scallop</er>.]</ety> <def>A bed of oysters or mussels.</def> <mark>[Scot.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scalp</ent><br/
<hw>Scalp</hw>, <pos>n.</pos> <ety>[Perhaps akin to D. <ets>schelp</ets> shell. Cf. <er>Scallop</er>.]</ety> <sn>1.</sn> <def>That part of the integument of the head which is usually covered with hair.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>By the bare <qex>scalp</qex> of Robin Hodd's fat friar,<br/
This fellow were a king for our wild faction!</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A part of the skin of the head, with the hair attached, cut or torn off from an enemy by the Indian warriors of North America, as a token of victory.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>Fig.: The top; the summit.</def> <rj><au>Macaulay.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Scalp lock</b></col>, <cd>a long tuft of hair left on the crown of the head by the warriors of some tribes of American Indians.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scalp</ent><br/
<hw>Scalp</hw>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Scalped</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Scalping</conjf>.]</vmorph> <sn>1.</sn> <def>To deprive of the scalp; to cut or tear the scalp from the head of.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Surg.)</fld> <def>To remove the skin of.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>We must <qex>scalp</qex> the whole lid [of the eye].</q> <rj><qau>J. S. Wells.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <fld>(Milling)</fld> <def>To brush the hairs or fuzz from, as wheat grains, in the process of high milling.</def> <rj><au>Knight.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scalp</ent><br/
<hw>Scalp</hw>, <pos>v. i.</pos> <def>To make a small, quick profit by slight fluctuations of the market; -- said of brokers who operate in this way on their own account.</def> <mark>[Cant]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scalpel</ent><br/
<hw>Scal"pel</hw> <pr>(sk<acr/l"p<ecr/l)</pr>, <pos>n.</pos> <ety>[L. <ets>scalpellum</ets>, dim. of <ets>scalprum</ets> a knife, akin to <ets>scalpere</ets> to cut, carve, scrape: cf. F. <ets>scalpel</ets>.]</ety> <fld>(Surg.)</fld> <def>A small knife with a thin, keen blade, -- used by surgeons, and in dissecting.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scalper</ent><br/
<hw>Scalper</hw> <pr>(sk<acr/lp"<etil/r)</pr>, <pos>n.</pos> <sn>1.</sn> <def>One who, or that which, scalps.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Surg.)</fld> <def>Same as <cref>Scalping iron</cref>, under <er>Scalping</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>A broker who, dealing on his own account, tries to get a small and quick profit from slight fluctuations of the market.</def> <mark>[Cant]</mark><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>A person who buys and sells the unused parts of railroad tickets.</def> <mark>[Cant]</mark><br/
[<source>1913 Webster</source>]</p>

<p><--<sn>5.</sn> <def>A person who buys tickets for entertainment or sports events and sells them at a profit, often at a much higher price.  Also, <altname>ticket scalper</altname>.</def> <mark>[Cant]</mark>--><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scalping</ent><br/
<hw>Scalp"ing</hw> <pr>(sk<acr/lp"<icr/ng)</pr>, <def><pos>a. & n.</pos> from <er>Scalp</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Scalping iron</b></col> <fld>(Surg.)</fld>, <cd>an instrument used in scraping foul and carious bones; a raspatory.</cd> -- <col><b>Scalping knife</b></col>, <cd>a knife used by North American Indians in scalping.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scalpriform</ent><br/
<hw>Scal"pri*form</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>scalprum</ets> chisel, knife + <ets>-form</ets>.]</ety> <fld>(Anat.)</fld> <def>Shaped like a chisel; <as>as, the <ex>scalpriform</ex> incisors of rodents</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scaly</ent><br/
<hw>Scal"y</hw> <pr>(?)</pr>, <pos>a.</pos> <sn>1.</sn> <def>Covered or abounding with scales; <as>as, a <ex>scaly</ex> fish</as>.</def> <ldquo/<xex>Scaly</xex> crocodile.<rdquo/ <rj><au>Milton.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Resembling scales, laminae, or layers.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>Mean; low; <as>as, a <ex>scaly</ex> fellow</as>.</def> <mark>[Low]</mark><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <fld>(Bot.)</fld> <def>Composed of scales lying over each other; <as>as, a <ex>scaly</ex> bulb</as>; covered with scales; <as>as, a <ex>scaly</ex> stem</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Scaly ant-eater</b></col> <fld>(Zool.)</fld>, <cd>the pangolin.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scaly-winged</ent><br/
<hw>Scal"y-winged`</hw> <pr>(?)</pr>, <pos>a.</pos> <fld>(Zool.)</fld> <def>Scale-winged.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scamble</ent><br/
<hw>Scam"ble</hw> <pr>(?)</pr>, <pos>v. i.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Scambled</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Scambling</conjf>.]</vmorph> <ety>[Cf. OD. <ets>schampelen</ets> to deviate, to slip, <ets>schampen</ets> to go away, escape, slip, and E. <ets>scamper</ets>, <ets>shamble</ets>.]</ety> <sn>1.</sn> <def>To move awkwardly; to be shuffling, irregular, or unsteady; to sprawl; to shamble.</def> <ldquo/Some <xex>scambling</xex> shifts.<rdquo/ <au>Dr. H. More.</au> <ldquo/A fine old hall, but a <xex>scambling</xex> house.<rdquo/ <au>Evelyn.</au><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To move about pushing and jostling; to be rude and turbulent; to scramble.</def> <ldquo/The <xex>scambling</xex> and unquiet time did push it out of . . . question.<rdquo/ <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scamble</ent><br/
<hw>Scam"ble</hw>, <pos>v. t.</pos> <def>To mangle.</def> <mark>[Obs.]</mark> <rj><au>Mortimer.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scambler</ent><br/
<hw>Scam"bler</hw> <pr>(?)</pr>, <pos>n.</pos> <def>1. One who scambles.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A bold intruder upon the hospitality of others; a mealtime visitor.</def> <mark>[Scot.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scamblingly</ent><br/
<hw>Scam"bling*ly</hw> <pr>(?)</pr>, <pos>adv.</pos> <def>In a scambling manner; with turbulence and noise; with bold intrusiveness.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scammel</ent><br/
<ent>Scamell</ent><br/
<mhw>{ <hw>Scam"ell</hw> <pr>(?)</pr>, <it>or</it> <hw>Scam"mel</hw>, }</mhw> <pos>n.</pos> <fld>(Zool.)</fld> <def>The female bar-tailed godwit.</def> <mark>[Prov. Eng.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ Whether this is the <xex>scamel</xex> mentioned by Shakespeare [<ldquo/Tempest,<rdquo/ ii. 2] is not known.</note><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scamillus</ent><br/
||<hw>Sca*mil"lus</hw> <pr>(?)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Scamilli</plw> <pr>(#)</pr>.</plu> <ety>[L., originally, a little bench, dim. of <ets>scamnum</ets> bench, stool.]</ety> <fld>(Arch.)</fld> <def>A sort of second plinth or block, below the bases of Ionic and Corinthian columns, generally without moldings, and of smaller size horizontally than the pedestal.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scammoniate</ent><br/
<hw>Scam*mo"ni*ate</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Made from scammony; <as>as, a <ex>scammoniate</ex> aperient</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scammony</ent><br/
<hw>Scam"mo*ny</hw> <pr>(sk<acr/m"m<osl/*n<ycr/)</pr>, <pos>n.</pos> <ety>[F. <ets>scammon<eacute/e</ets>, L. <ets>scammonia</ets>, <ets>scammonea</ets>, Gr. <grk>skammwni`a</grk>.]</ety> <sn>1.</sn> <fld>(Bot.)</fld> <def>A species of bindweed or <gen>Convolvulus</gen> (<spn>Convolvulus Scammonia</spn>).</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>An inspissated sap obtained from the root of the <spn>Convolvulus Scammonia</spn>, of a blackish gray color, a nauseous smell like that of old cheese, and a somewhat acrid taste. It is used in medicine as a cathartic.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scamp</ent><br/
<hw>Scamp</hw> <pr>(sk<acr/mp)</pr>, <pos>n.</pos> <ety>[OF. <ets>escamper</ets> to run away, to make one's escape. Originally, one who runs away, a fugitive, a vagabond. See <er>Scamper</er>.]</ety> <def>A rascal; a swindler; a rogue.</def> <rj><au>De Quincey.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scamp</ent><br/
<hw>Scamp</hw>, <pos>v. t.</pos> <ety>[Cf. <er>Scamp</er>,<pos>n.</pos>, or <er>Scant</er>, <pos>a.</pos>, and <er>Skimp</er>.]</ety> <def>To perform in a hasty, neglectful, or imperfect manner; to do superficially.</def> <mark>[Colloq.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>A workman is said to <qex>scamp</qex> his work when he does it in a superficial, dishonest manner.</q> <rj><qau>Wedgwood.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Much of the <qex>scamping</qex> and dawdling complained of is that of men in establishments of good repute.</q> <rj><qau>T. Hughes.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scampavia</ent><br/
||<hw>Scam`pa*vi"a</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[It.]</ety> <def>A long, low war galley used by the Neapolitans and Sicilians in the early part of the nineteenth century.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scamper</ent><br/
<hw>Scam"per</hw> <pr>(?)</pr>, <pos>v. i.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Scampered</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Scampering</conjf>.]</vmorph> <ety>[OF. <ets>escamper</ets> to escape, to save one's self; L. <ets>ex</ets> from + <ets>campus</ets> the field (sc. of battle). See <er>Camp</er>, and cf. <er>Decamp</er>, <er>Scamp</er>, <pos>n.</pos>, <er>Shamble</er>, <pos>v. t.</pos>]</ety> <def>To run with speed; to run or move in a quick, hurried manner; to hasten away.</def> <rj><au>Macaulay.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>The lady, however, . . . could not help <qex>scampering</qex> about the room after a mouse.</q> <rj><qau>S. Sharpe.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scamper</ent><br/
<hw>Scam"per</hw>, <pos>n.</pos> <def>A scampering; a hasty flight.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scamperer</ent><br/
<hw>Scam"per*er</hw> <pr>(?)</pr>, <pos>n.</pos> <def>One who scampers.</def> <rj><au>Tyndell.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scampish</ent><br/
<hw>Scamp"ish</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Of or like a scamp; knavish; <as>as, <ex>scampish</ex> conduct</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scan</ent><br/
<hw>Scan</hw> <pr>(sk<acr/n)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Scanned</conjf> <pr>(sk<acr/nd)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Scanning</conjf>.]</vmorph> <ety>[L. <ets>scandere</ets>, <ets>scansum</ets>, to climb, to scan, akin to Skr. <ets>skand</ets> to spring, leap: cf. F. <ets>scander</ets>. Cf. <er>Ascend</er>, <er>Descend</er>, <er>Scale</er> a ladder.]</ety> <sn>1.</sn> <def>To mount by steps; to go through with step by step.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>Nor stayed till she the highest stage had <qex>scand</qex>.</q> <rj><qau>Spenser.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><-- Note that "scand" is not given as a hw, a conjf, or an asp. PCP.  It is presumably an older spelling of "scanned". --><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Specifically <fld>(Pros.)</fld>, to go through with, as a verse, marking and distinguishing the feet of which it is composed; to show, in reading, the metrical structure of; to recite metrically.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>To go over and examine point by point; to examine with care; to look closely at or into; to scrutinize.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The actions of men in high stations are all conspicuous, and liable to be <qex>scanned</qex> and sifted.</q> <rj><qau>Atterbury.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>To examine quickly, from point to point, in search of something specific; <as>as, to <ex>scan</ex> an article for mention of a particular person</as>.</def><br/
[<source>PJC</source>]</p>

<p><sn>5.</sn> <fld>(Electronics)</fld> <def>To form an image or an electronic representation of, by passing a beam of light or electrons over, and detecting and recording the reflected or transmitted signal.</def><br/
[<source>PJC</source>]</p>

<p><ent>Scandal</ent><br/
<hw>Scan"dal</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F. <ets>scandale</ets>, fr. L. <ets>scandalum</ets>, Gr. <?/, a snare laid for an enemy, a stumbling block, offense, scandal: cf. OE. <ets>scandle</ets>, OF. <ets>escandle</ets>. See <er>Slander</er>.]</ety> <sn>1.</sn> <def>Offense caused or experienced; reproach or reprobation called forth by what is regarded as wrong, criminal, heinous, or flagrant: opprobrium or disgrace.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>O, what a <qex>scandal</qex> is it to our crown,<br/
That two such noble peers as ye should jar!</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>[I] have brought <qex>scandal</qex><br/
To Israel, diffidence of God, and doubt<br/
In feeble hearts.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Reproachful aspersion; opprobrious censure; defamatory talk, uttered heedlessly or maliciously.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>You must not put another <qex>scandal</qex> on him.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>My known virtue is from <qex>scandal</qex> free.</q> <rj><qau>Dryden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <fld>(Equity)</fld> <def>Anything alleged in pleading which is impertinent, and is reproachful to any person, or which derogates from the dignity of the court, or is contrary to good manners.</def> <rj><au>Daniell.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><syn><b>Syn.</b> -- Defamation; detraction; slander; calumny; opprobrium; reproach; shame; disgrace.</syn><br/
[<source>1913 Webster</source>]</p>

<p><-- p. 1283 pr=PCP --></p>

<p><ent>Scandal</ent><br/
<hw>Scan"dal</hw> <pr>(?)</pr>, <pos>v. t.</pos> <sn>1.</sn> <def>To treat opprobriously; to defame; to asperse; to traduce; to slander.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>I do fawn on men and hug them hard<br/
And after <qex>scandal</qex> them.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To scandalize; to offend.</def> <mark>[Obs.]</mark> <rj><au>Bp. Story.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><syn><b>Syn.</b> -- To defame; traduce; reproach; slander; calumniate; asperse; vilify; disgrace.</syn><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scandalize</ent><br/
<hw>Scan"dal*ize</hw> <pr>(?)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Scandalized</conjf> <pr>(<?/)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Scandalizing</conjf> <pr>(<?/)</pr>.]</vmorph> <ety>[F. <ets>scandaliser</ets>, L. <ets>scandalizare</ets>, from Gr. <grk>skandali`zein</grk>.]</ety> <sn>1.</sn> <def>To offend the feelings or the conscience of (a person) by some action which is considered immoral or criminal; to bring shame, disgrace, or reproach upon.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>I demand who they are whom we <qex>scandalize</qex> by using harmless things.</q> <rj><qau>Hooker.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>The congregation looked on in silence, the better class <qex>scandalized</qex>, and the lower orders, some laughing, others backing the soldier or the minister, as their fancy dictated.</q> <rj><qau>Sir W. Scott.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To reproach; to libel; to defame; to slander.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>To tell his tale might be interpreted into <qex>scandalizing</qex> the order.</q> <rj><qau>Sir W. Scott.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scandalous</ent><br/
<hw>Scan"dal*ous</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[Cf. F. <ets>scandaleux</ets>.]</ety> <sn>1.</sn> <def>Giving offense to the conscience or moral feelings; exciting reprobation; calling out condemnation.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Nothing <qex>scandalous</qex> or offensive unto any.</q> <rj><qau>Hooker.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Disgraceful to reputation; bringing shame or infamy; opprobrious; <as>as, a <ex>scandalous</ex> crime or vice</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>Defamatory; libelous; <as>as, a <ex>scandalous</ex> story</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scandalously</ent><br/
<hw>Scan"dal*ous*ly</hw>, <pos>adv.</pos> <sn>1.</sn> <def>In a manner to give offense; shamefully.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>His discourse at table was <qex>scandalously</qex> unbecoming the dignity of his station.</q> <rj><qau>Swift.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>With a disposition to impute immorality or wrong.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Shun their fault, who, <qex>scandalously</qex> nice,<br/
Will needs mistake an author into vice.</q> <rj><qau>Pope.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scandalousness</ent><br/
<hw>Scan"dal*ous*ness</hw>, <pos>n.</pos> <def>Quality of being scandalous.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scandalum magnatum</ent><br/
||<hw>Scan"da*lum mag*na"tum`</hw> <pr>(?)</pr>. <ety>[L., scandal of magnates.]</ety> <fld>(Law)</fld> <def>A defamatory speech or writing published to the injury of a person of dignity; -- usually abbreviated <abbr>scan. mag.</abbr></def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scandent</ent><br/
<hw>Scan"dent</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>scandens</ets>, <ets>-entis</ets>, p. pr. of <ets>scandere</ets> to climb.]</ety> <def>Climbing.</def><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ Scandent plants may climb either by twining, as the hop, or by twisted leafstalks, as the clematis, or by tendrils, as the passion flower, or by rootlets, as the ivy.</note><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scandia</ent><br/
<hw>Scan"di*a</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[NL. See <er>Scandium</er>.]</ety> <fld>(Chem.)</fld> <def>A chemical earth, the oxide of scandium.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scandic</ent><br/
<hw>Scan"dic</hw> <pr>(?)</pr>, <pos>a.</pos> <fld>(Chem.)</fld> <def>Of or pertaining to scandium; derived from, or containing, scandium.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scandinavian</ent><br/
<hw>Scan`di*na"vi*an</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Of or pertaining to Scandinavia, that is, Sweden, Norway, and Denmark.</def> -- <def2><pos>n.</pos> <def>A native or inhabitant of Scandinavia.</def></def2><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scandium</ent><br/
<hw>Scan"di*um</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[NL. So called because found in <etsep>Scandinavian</etsep> minerals.]</ety> <fld>(Chem.)</fld> <def>A rare metallic element of the boron group, whose existence was predicted under the provisional name <altname>ekaboron</altname> by means of the periodic law, and subsequently discovered by spectrum analysis in certain rare Scandinavian minerals (<xex>euxenite</xex> and <xex>gadolinite</xex>). It has not yet been isolated. Symbol Sc. Atomic weight 44.</def><-- Atomic weight 44.96, at. no. 21; valence 3. --><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scansion</ent><br/
<hw>Scan"sion</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>scansio</ets>, fr. <ets>scandere</ets>, <ets>scansum</ets>, to climb. See <er>Scan</er>.]</ety> <fld>(Pros.)</fld> <def>The act of scanning; distinguishing the metrical feet of a verse by emphasis, pauses, or otherwise.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scansores</ent><br/
||<hw>Scan*so"res</hw> <pr>(?)</pr>, <pos>n. pl.</pos> <ety>[NL., fr. L. <ets>scandere</ets>, <ets>scansum</ets>, to climb.]</ety> <fld>(Zool.)</fld> <def>An artifical group of birds formerly regarded as an order. They are distributed among several orders by modern ornithologists.</def><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ The toes are in pairs, two before and two behind, by which they are enabled to cling to, and climb upon, trees, as the woodpeckers, parrots, cuckoos, and trogons. See <xex>Illust.</xex> under <er>Aves</er>.</note><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scansorial</ent><br/
<hw>Scan*so"ri*al</hw> <pr>(?)</pr>, <pos>a.</pos> <fld>(Zool.)</fld> <sd>(a)</sd> <def>Capable of climbing; <as>as, the woodpecker is a <ex>scansorial</ex> bird</as>; adapted for climbing; <as>as, a <ex>scansorial</ex> foot</as>.</def> <sd>(b)</sd> <def>Of or pertaining to the Scansores. See <xex>Illust.</xex> under <er>Aves</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Scansorial tail</b></col> <fld>(Zool.)</fld>, <cd>a tail in which the feathers are stiff and sharp at the tip, as in the woodpeckers.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scant</ent><br/
<hw>Scant</hw> <pr>(?)</pr>, <pos>a.</pos> <amorph>[<pos>Compar.</pos> <adjf>Scanter</adjf> <pr>(?)</pr>; <pos>superl.</pos> <adjf>Scantest</adjf>.]</amorph> <ety>[Icel. <ets>skamt</ets>, neuter of <ets>skamr</ets>, <ets>skammr</ets>, short; cf. <ets>skamta</ets> to dole out, to portion.]</ety> <sn>1.</sn> <def>Not full, large, or plentiful; scarcely sufficient; less than is wanted for the purpose; scanty; meager; not enough; <as>as, a <ex>scant</ex> allowance of provisions or water; a <ex>scant</ex> pattern of cloth for a garment</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>His sermon was <qex>scant</qex>, in all, a quarter of an hour.</q> <rj><qau>Ridley.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Sparing; parsimonious; chary.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Be somewhat <qex>scanter</qex> of your maiden presence.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><syn><b>Syn.</b> -- See under <er>Scanty</er>.</syn><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scant</ent><br/
<hw>Scant</hw>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Scanted</conjf>; <pos>p. pr. & vb. n.</pos> <conjf>Scanting</conjf>.]</vmorph> <sn>1.</sn> <def>To limit; to straiten; to treat illiberally; to stint; <as>as, to <ex>scant</ex> one in provisions; to <ex>scant</ex> ourselves in the use of necessaries</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Where a man hath a great living laid together and where he is <qex>scanted</qex>.</q> <rj><qau>Bacon.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>I am <qex>scanted</qex> in the pleasure of dwelling on your actions.</q> <rj><qau>Dryden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To cut short; to make small, narrow, or scanty; to curtail.</def> <ldquo/<xex>Scant</xex> not my cups.<rdquo/ <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scant</ent><br/
<hw>Scant</hw>, <pos>v. i.</pos> <def>To fail, or become less; to scantle; <as>as, the wind <ex>scants</ex></as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scant</ent><br/
<hw>Scant</hw>, <pos>adv.</pos> <def>In a scant manner; with difficulty; scarcely; hardly.</def> <mark>[Obs.]</mark> <rj><au>Bacon.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>So weak that he was <qex>scant</qex> able to go down the stairs.</q> <rj><qau>Fuller.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scant</ent><br/
<hw>Scant</hw>, <pos>n.</pos> <def>Scantness; scarcity.</def> <mark>[R.]</mark> <rj><au>T. Carew.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scantily</ent><br/
<hw>Scant"i*ly</hw> <pr>(?)</pr>, <pos>adv.</pos> <def>In a scanty manner; not fully; not plentifully; sparingly; parsimoniously.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>His mind was very <qex>scantily</qex> stored with materials.</q> <rj><qau>Macaulay.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><-- scantily clad, wearing almost no clothing. --><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scantiness</ent><br/
<hw>Scant"i*ness</hw>, <pos>n.</pos> <def>Quality or condition of being scanty.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scantle</ent><br/
<hw>Scan"tle</hw> <pr>(?)</pr>, <pos>v. i.</pos> <ety>[Dim. of <ets>scant</ets>, v.]</ety> <def>To be deficient; to fail.</def> <mark>[Obs.]</mark> <rj><au>Drayton.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scantle</ent><br/
<hw>Scan"tle</hw> <pr>(?)</pr>, <pos>v. t.</pos> <ety>[OF. <ets>escanteler</ets>, <ets>eschanteler</ets>, to break into contles; pref. <ets>es-</ets> (L. <ets>ex</ets>) + <ets>cantel</ets>, <ets>chantel</ets>, corner, side, piece. Confused with E. <ets>scant</ets>. See <er>Cantle</er>.]</ety> <def>To scant; to be niggard of; to divide into small pieces; to cut short or down.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>All their pay<br/
Must your discretion <qex>scantle</qex>; keep it back.</q> <rj><qau>J. Webster.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scantlet</ent><br/
<hw>Scant"let</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[OF. <ets>eschantelet</ets> corner.]</ety> <def>A small pattern; a small quantity.</def> <mark>[Obs.]</mark> <rj><au>Sir M. Hale.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scantling</ent><br/
<hw>Scant"ling</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[See <er>Scant</er>, <pos>a.</pos>]</ety> <def>Not plentiful; small; scanty.</def> <mark>[Obs.]</mark> <rj><au>Jer. Taylor.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scantling</ent><br/
<hw>Scant"ling</hw>, <pos>n.</pos> <ety>[Cf. OF. <ets>eschantillon</ets>, F. <ets><eacute/chantillon</ets>, a sample, pattern, example. In some senses confused with <ets>scant</ets> insufficient. See <er>Scantle</er>, <pos>v. t.</pos>]</ety> <sn>1.</sn> <def>A fragment; a bit; a little piece.</def> Specifically: <sd>(a)</sd> <def>A piece or quantity cut for a special purpose; a sample.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>Such as exceed not this <qex>scantling</qex>; -- to be solace to the sovereign and harmless to the people.</q> <rj><qau>Bacon.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>A pretty <qex>scantling</qex> of his knowledge may taken by his deferring to be baptized so many years.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sd>(b)</sd> <def>A small quantity; a little bit; not much.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>Reducing them to narrow <qex>scantlings</qex>.</q> <rj><qau>Jer. Taylor.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A piece of timber sawed or cut of a small size, as for studs, rails, etc.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>The dimensions of a piece of timber with regard to its breadth and thickness; hence, the measure or dimensions of anything.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>A rough draught; a rude sketch or outline.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <def>A frame for casks to lie upon; a trestle.</def> <rj><au>Knight.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scantly</ent><br/
<hw>Scant"ly</hw>, <pos>adv.</pos> <sn>1.</sn> <def>In a scant manner; not fully or sufficiently; narrowly; penuriously.</def> <rj><au>Dryden.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Scarcely; hardly; barely.</def><br/
[<source>1913 Webster</source>]</p>

<p><q><qex>Scantly</qex> they durst their feeble eyes dispread<br/
Upon that town.</q> <rj><qau>Fairfax.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>We hold a tourney here to-morrow morn,<br/
And there is <qex>scantly</qex> time for half the work.</q> <rj><qau>Tennyson.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scantness</ent><br/
<hw>Scant"ness</hw>, <pos>n.</pos> <def>The quality or condition of being scant; narrowness; smallness; insufficiency; scantiness.</def> <ldquo/<xex>Scantness</xex> of outward things.<rdquo/ <rj><au>Barrow.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scanty</ent><br/
<hw>Scant"y</hw> <pr>(?)</pr>, <pos>a.</pos> <amorph>[<pos>Compar.</pos> <adjf>Scantier</adjf> <pr>(?)</pr>; <pos>superl.</pos> <adjf>Scantiest</adjf>.]</amorph> <ety>[From <er>Scant</er>, <pos>a.</pos>]</ety> <sn>1.</sn> <def>Lacking amplitude or extent; narrow; small; not abundant.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>His dominions were very narrow and <qex>scanty</qex>.</q> <rj><qau>Locke.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Now <qex>scantier</qex> limits the proud arch confine.</q> <rj><qau>Pope.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Somewhat less than is needed; insufficient; scant; <as>as, a <ex>scanty</ex> supply of words; a <ex>scanty</ex> supply of bread</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>Sparing; niggardly; parsimonious.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>In illustrating a point of difficulty, be not too <qex>scanty</qex> of words.</q> <rj><qau>I. Watts.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><syn><b>Syn.</b> -- Scant; narrow; small; poor; deficient; meager; scarce; chary; sparing; parsimonious; penurious; niggardly; grudging.</syn><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scape</ent><br/
<hw>Scape</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>scapus</ets> shaft, stem, stalk; cf. Gr. <?/ a staff: cf. F. <ets>scape</ets>. Cf. <er>Scepter</er>.]</ety> <sn>1.</sn> <fld>(Bot.)</fld> <def>A peduncle rising from the ground or from a subterranean stem, as in the stemless violets, the bloodroot, and the like.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Zool.)</fld> <def>The long basal joint of the antennae of an insect.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <fld>(Arch.)</fld> <sd>(a)</sd> <def>The shaft of a column.</def> <sd>(b)</sd> <def>The apophyge of a shaft.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scape</ent><br/
<hw>Scape</hw>, <pos>v. t. & i.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Scaped</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Scaping</conjf>.]</vmorph> <ety>[Aphetic form of <ets>escape</ets>.]</ety> <def>To escape.</def> <mark>[Obs. or Poetic.]</mark> <rj><au>Milton.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Out of this prison help that we may <qex>scape</qex>.</q> <rj><qau>Chaucer.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scape</ent><br/
<hw>Scape</hw>, <pos>n.</pos> <sn>1.</sn> <def>An escape.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>I spake of most disastrous chances, . . .<br/
Of hairbreadth <qex>scapes</qex> in the imminent, deadly breach.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Means of escape; evasion.</def> <mark>[Obs.]</mark> <rj><au>Donne.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>A freak; a slip; a fault; an escapade.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>Not pardoning so much as the <qex>scapes</qex> of error and ignorance.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>Loose act of vice or lewdness.</def> <mark>[Obs.]</mark> <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scapegallows</ent><br/
<hw>Scape"gal`lows</hw> <pr>(?)</pr>, <pos>n.</pos> <def>One who has narrowly escaped the gallows for his crimes.</def> <mark>[Colloq.]</mark> <rj><au>Dickens.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scapegoat</ent><br/
<hw>Scape"goat`</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[<ets>Scape</ets> (for <ets>escape</ets>) + <ets>goat</ets>.]</ety> <sn>1.</sn> <fld>(Jewish Antiq.)</fld> <def>A goat upon whose head were symbolically placed the sins of the people, after which he was suffered to escape into the wilderness.</def> <rj><au>Lev. xvi. 10.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Hence, a person or thing that is made to bear blame for others.</def> <rj><au>Tennyson.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scapegrace</ent><br/
<hw>Scape"grace`</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A graceless, unprincipled person; one who is wild and reckless.</def> <rj><au>Beaconsfield.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scapeless</ent><br/
<hw>Scape"less</hw>, <pos>a.</pos> <fld>(Bot.)</fld> <def>Destitute of a scape.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scapement</ent><br/
<hw>Scape"ment</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[See <er>Scape</er>, <pos>v.</pos>, <er>Escapement</er>.]</ety> <def>Same as <er>Escapement</er>, 3.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scape-wheel</ent><br/
<hw>Scape"-wheel`</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Horol.)</fld> <def>The wheel in an escapement (as of a clock or a watch) into the teeth of which the pallets play.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scaphander</ent><br/
<hw>Sca*phan"der</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Gr. <?/, <?/, anything hollowed + <?/, <?/, a man: cf. F. <ets>scaphandre</ets>.]</ety> <def>The case, or impermeable apparel, in which a diver can work while under water.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scaphism</ent><br/
<hw>Scaph"ism</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Gr. <grk>ska`fh</grk> a trough.]</ety> <def>An ancient mode of punishing criminals among the Persians, by confining the victim in a trough, with his head and limbs smeared with honey or the like, and exposed to the sun and to insects until he died.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scaphite</ent><br/
<hw>Scaph"ite</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>scapha</ets> a boat, fr. Gr. <grk>ska`fh</grk> a boat, anything dug or scooped out, fr. <grk>ska`ptein</grk> to dig.]</ety> <fld>(Paleon.)</fld> <def>Any fossil cephalopod shell of the genus <gen>Scaphites</gen>, belonging to the Ammonite family and having a chambered boat-shaped shell. Scaphites are found in the Cretaceous formation.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scaphocephalic</ent><br/
<hw>Scaph`o*ce*phal"ic</hw> <pr>(?)</pr>, <pos>a.</pos> <fld>(Anat.)</fld> <def>Of, pertaining to, or affected with, scaphocephaly.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scaphocephaly</ent><br/
<hw>Scaph`o*ceph"a*ly</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Gr. <grk>ska`fh</grk> a boat + <grk>kefalh`</grk> head.]</ety> <fld>(Anat.)</fld> <def>A deformed condition of the skull, in which the vault is narrow, elongated, and more or less boat-shaped.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scaphocerite</ent><br/
<hw>Scaph`o*ce"rite</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Gr. <grk>ska`fh</grk> boat + E. <ets>cerite</ets>.]</ety> <fld>(Zool.)</fld> <def>A flattened plate or scale attached to the second joint of the antennae of many Crustacea.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scaphognathite</ent><br/
<hw>Sca*phog"na*thite</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Gr. <grk>ska`fh</grk> boat + <grk>gna`qos</grk> jaw.]</ety> <fld>(Zool.)</fld> <def>A thin leafike appendage (the exopodite) of the second maxilla of decapod crustaceans. It serves as a pumping organ to draw the water through the gill cavity.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scaphoid</ent><br/
<hw>Scaph"oid</hw> <pr>(?; 277)</pr>, <pos>a.</pos> <ety>[Gr. <grk>ska`fh</grk> a boat + <ets>-oid</ets>: cf. F. <ets>scapho<ium/de</ets>.]</ety> <fld>(Anat.)</fld> <def>Resembling a boat in form; boat-shaped.</def> -- <def2><pos>n.</pos> <def>The scaphoid bone.</def></def2><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Scaphoid bone</b></col> <sd>(a)</sd> <cd>One of the carpal bones, which articulates with the radius; the radiale</cd>. <sd>(b)</sd> <cd>One of the tarsal bones; the navicular bone. See under <er>Navicular</er>.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scapholunar</ent><br/
<hw>Scaph`o*lu"nar</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[<ets>Scapho</ets>id + <ets>lunar</ets>.]</ety> <fld>(Anat.)</fld> <def>Of or pertaining to the scaphoid and lunar bones of the carpus.</def> -- <def2><pos>n.</pos> <def>The scapholunar bone.</def></def2><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Scapholunar bone</b></col>, <cd>a bone formed by the coalescence of the scaphoid and lunar in the carpus of carnivora.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scaphopoda</ent><br/
||<hw>Sca*phop"o*da</hw> <pr>(?)</pr>, <pos>n. pl.</pos> <ety>[NL., from Gr. <grk>ska`fh</grk> a boat + <ets>-poda</ets>.]</ety> <fld>(Zool.)</fld> <def>A class of marine cephalate Mollusca having a tubular shell open at both ends, a pointed or spadelike foot for burrowing, and many long, slender, prehensile oral tentacles. It includes Dentalium, or the tooth shells, and other similar shells. Called also <altname>Prosopocephala</altname>, and <altname>Solenoconcha</altname>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scapiform</ent><br/
<hw>Sca"pi*form</hw> <pr>(?)</pr>, <pos>a.</pos> <fld>(Bot.)</fld> <def>Resembling a scape, or flower stem.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scapolite</ent><br/
<hw>Scap"o*lite</hw> <pr>(sk<acr/p"<osl/*l<imac/t)</pr>, <pos>n.</pos> <ety>[Gr. <?/ a staff, or L. <ets>scapus</ets> a stem, stalk + <ets>-lite</ets>: cf. F. <ets>scapolite</ets>.]</ety> <fld>(Mon.)</fld> <def>A grayish white mineral occuring in tetragonal crystals and in cleavable masses. It is essentially a silicate of alumina and soda.</def><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ The <xex>scapolite group</xex> includes scapolite proper, or wernerite, also meionite, dipyre, etc.</note><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scapple</ent><br/
<hw>Scap"ple</hw> <pr>(sk<acr/p"p'l)</pr>, <pos>v. t.</pos> <ety>[Cf. OF. <ets>eskapeler</ets>, <ets>eschapler</ets>, to cut, hew, LL. <ets>scapellare</ets>. Cf. <er>Scabble</er>.]</ety> <sd>(a)</sd> <def>To work roughly, or shape without finishing, as stone before leaving the quarry.</def> <sd>(b)</sd> <def>To dress in any way short of fine tooling or rubbing, as stone.</def> <rj><au>Gwilt.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scapula</ent><br/
<hw>Scap"u*la</hw> <pr>(sk<acr/p"<usl/*l<adot/)</pr>, <pos>n.</pos>; <plu><it>pl.</it> L. <plw>Scapulae</plw> <pr>(#)</pr>, E. <plw>Scapulas</plw> <pr>(#)</pr>.</plu> <ety>[L.]</ety> <sn>1.</sn> <fld>(Anat.)</fld> <def>The principal bone of the shoulder girdle in mammals; the shoulder blade.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Zool.)</fld> <def>One of the plates from which the arms of a crinoid arise.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scapular</ent><br/
<hw>Scap"u*lar</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[Cf. F. <ets>scapulaire</ets>. Cf. <er>Scapulary</er>.]</ety> <def>Of or pertaining to the scapula or the shoulder.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Scapular arch</b></col> <fld>(Anat.)</fld>, <cd>the pectoral arch. See under <er>pectoral</er>.</cd> -- <mcol><col><b>Scapular region</b></col>, <it>or</it> <col><b>Scapular tract</b></col></mcol> <fld>(Zool.)</fld>, <cd>a definite longitudinal area over the shoulder and along each side of the back of a bird, from which the scapular feathers arise.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scapular</ent><br/
<hw>Scap"u*lar</hw>, <pos>n.</pos> <fld>(Zool.)</fld> <def>One of a special group of feathers which arise from each of the scapular regions and lie along the sides of the back.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scapulary</ent><br/
<ent>Scapular</ent><br/
<mhw>{ <hw>Scap"u*lar</hw> <pr>(?)</pr>, <hw>Scap"u*la*ry</hw> <pr>(?)</pr>, }</mhw> <pos>n.</pos> <ety>[F. <ets>scapulaire</ets>, LL. <ets>scapularium</ets>, <ets>scapulare</ets>, fr. L. <ets>scapula</ets> shoulder blade.]</ety> <sn>1.</sn> <fld>(R. C. Ch.)</fld> <sd>(a)</sd> <def>A loose sleeveless vestment falling in front and behind, worn by certain religious orders and devout persons.</def> <sd>(b)</sd> <def>The name given to two pieces of cloth worn under the ordinary garb and over the shoulders as an act of devotion.</def> <rj><au>Addis & Arnold.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Surg.)</fld> <def>A bandage passing over the shoulder to support it, or to retain another bandage in place.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scapulary</ent><br/
<hw>Scap"u*la*ry</hw>, <pos>a.</pos> <def>Same as <er>Scapular</er>, <pos>a.</pos></def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scapulary</ent><br/
<hw>Scap"u*la*ry</hw>, <pos>n.</pos> <fld>(Zool.)</fld> <def>Same as 2d and 3d <er>Scapular</er>.</def><-- But 3d Scapular is (R. C. Ch.), not (Zool.)! PCP --><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scapulet</ent><br/
<hw>Scap"u*let</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Dim. of <ets>scapula</ets>.]</ety> <fld>(Zool.)</fld> <def>A secondary mouth fold developed at the base of each of the armlike lobes of the manubrium of many rhizostome medusae. See <xex>Illustration</xex> in Appendix.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scapulo-</ent><br/
<hw>Scap"u*lo-</hw> <pr>(<?/)</pr>. <def>A combining form used in anatomy to indicate <xex>connection with</xex>, or <xex>relation to</xex>, <xex>the scapula</xex> or <xex>the shoulder</xex>; <as>as, the <ex>scapulo</ex>-clavicular articulation, the articulation between the scapula and clavicle</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scapus</ent><br/
||<hw>Sca"pus</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L.]</ety> <def>See 1st <er>Scape</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scar</ent><br/
<hw>Scar</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[OF. <ets>escare</ets>, F. <ets>eschare</ets> an eschar, a dry slough (cf. It. & Sp. <ets>escara</ets>), L. <ets>eschara</ets>, fr. Gr. <?/ hearth, fireplace, scab, eschar. Cf. <er>Eschar</er>.]</ety> <sn>1.</sn> <def>A mark in the skin or flesh of an animal, made by a wound or ulcer, and remaining after the wound or ulcer is healed; a cicatrix; a mark left by a previous injury; a blemish; a disfigurement.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>This earth had the beauty of youth, . . . and not a wrinkle, <qex>scar</qex>, or fracture on all its body.</q> <rj><qau>T. Burnet.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Bot.)</fld> <def>A mark left upon a stem or branch by the fall of a leaf, leaflet, or frond, or upon a seed by the separation of its support. See <xex>Illust.</xex> under <er>Axillary</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scar</ent><br/
<hw>Scar</hw>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Scarred</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Scarring</conjf>.]</vmorph> <def>To mark with a scar or scars.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Yet I'll not shed her blood;<br/
Nor <qex>scar</qex> that whiter skin of hers than snow.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>His cheeks were deeply <qex>scarred</qex>.</q> <rj><qau>Macaulay.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scar</ent><br/
<hw>Scar</hw>, <pos>v. i.</pos> <def>To form a scar.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scar</ent><br/
<hw>Scar</hw>, <pos>n.</pos> <ety>[Scot. <ets>scar</ets>, <ets>scaur</ets>, Icel. <ets>sker</ets> a skerry, an isolated rock in the sea; akin to Dan. <ets>skiaer</ets>, Sw. <ets>sk<aum/r</ets>. Cf. <er>Skerry</er>.]</ety> <def>An isolated or protruding rock; a steep, rocky eminence; a bare place on the side of a mountain or steep bank of earth.</def> <altsp>[Written also <asp>scaur</asp>.]</altsp><br/
[<source>1913 Webster</source>]</p>

<p><q>O sweet and far, from cliff and <qex>scar</qex>,<br/
The horns of Elfland faintly blowing.</q> <rj><qau>Tennyson.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scar</ent><br/
<hw>Scar</hw>, <pos>n.</pos> <ety>[L. <ets>scarus</ets>, a kind of fish, Gr. <grk>ska`ros</grk>.]</ety> <fld>(Zool.)</fld> <def>A marine food fish, the scarus, or parrot fish.</def><br/
[<source>1913 Webster</source>]</p>

<p><-- p. 1284 pr=PCP --></p>

<p><ent>Scarabee</ent><br/
<ent>Scarab</ent><br/
<mhw>{ <hw>Scar"ab</hw> <pr>(?)</pr>, <hw>Scar"a*bee</hw> <pr>(?)</pr>, }</mhw> <pos>n.</pos> <ety>[L. <ets>scarabaeus</ets>; cf. F. <ets>scarab<eacute/e</ets>.]</ety> <sn>1.</sn> <fld>(Zool.)</fld> <def>Any one of numerous species of lamellicorn beetles of the genus <gen>Scarabaeus</gen>, or family <fam>Scarabaeidae</fam>, especially the sacred, or Egyptian, species (<spn>Scarabaeus sacer</spn>, and <spn>Scarabaeus Egyptiorum</spn>).</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Egyptian Archaeology, Jewelry)</fld> <def>A stylized representation of a scarab beetle carved in stone or faience, or made in baked clay, usually in a conventionalized form in which the beetle has its legs held closely at its sides, and commonly having an inscription on the flat underside; -- a symbol of resurrection, used by the ancient Egyptians as an ornament or a talisman,  and in modern times used in jewelry, usually by engraving the formalized scarab design on cabuchon stones.  Also used attributively; <as>as, a <ex>scarab</ex> bracelet [a bracelet containing <ex>scarabs</ex>]; a ring with a <ex>scarab</ex> [the carved stone itelf]</as>.</def><br/
[<source>1913 Webster</source> <source>+PJC</source>]</p>

<p><ent>Scarabaeus</ent><br/
||<hw>Scar`a*bae"us</hw> <pr>(?)</pr>, <pos>n.</pos> <def>Same as <er>Scarab</er> in both senses.</def></p>

<p><ent>Scaraboid</ent><br/
<hw>Scar"a*boid</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[<ets>Scarab</ets> + <ets>-oid</ets>.]</ety> <fld>(Zool.)</fld> <def>Of or pertaining to the family <fam>Scarabaeidae</fam>, an extensive group which includes the Egyptian scarab, the tumbledung, and many similar lamellicorn beetles.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scaraboid</ent><br/
<hw>Scar"a*boid</hw>, <pos>n.</pos> <fld>(Zool.)</fld> <def>A scaraboid beetle.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scaramouch</ent><br/
<hw>Scar"a*mouch`</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F. <ets>scaramouche</ets>, It. <ets>scaramuccio</ets>, <ets>scaramuccia</ets>, originally the name of a celebrated Italian comedian; cf. It. <ets>scaramuccia</ets>, <ets>scaramuccio</ets>, F. <ets>escarmouche</ets>, skirmish. Cf. <er>Skirmish</er>.]</ety> <def>A personage in the old Italian comedy (derived from Spain) characterized by great boastfulness and poltroonery; hence, a person of like characteristics; a buffoon.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scarce</ent><br/
<hw>Scarce</hw> <pr>(sk<acir/rs)</pr>, <pos>a.</pos> <amorph>[<pos>Compar.</pos> <adjf>Scarcer</adjf> <pr>(sk<acir/r"s<etil/r)</pr>; <pos>superl.</pos> <adjf>Scarcest</adjf>.]</amorph> <ety>[OE. <ets>scars</ets>, OF. <ets>escars</ets>, <ets>eschars</ets>, LL. <ets>scarpsus</ets>, <ets>excarpsus</ets>, for L. <ets>excerptus</ets>, p. p. of <ets>excerpere</ets> to pick out, and hence to contract, to shorten; <ets>ex</ets> (see <er>Ex-</er>) + <ets>carpere</ets>. See <er>Carpet</er>, and cf. <er>Excerp</er>.]</ety> <sn>1.</sn> <def>Not plentiful or abundant; in small quantity in proportion to the demand; not easily to be procured; rare; uncommon.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>You tell him silver is <qex>scarcer</qex> now in England, and therefore risen one fifth in value.</q> <rj><qau>Locke.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>The <qex>scarcest</qex> of all is a Pescennius Niger on a medallion well preserved.</q> <rj><qau>Addison.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Scantily supplied (with); deficient (in); -- with <xex>of</xex>.</def> <mark>[Obs.]</mark> <ldquo/A region <xex>scarce</xex> of prey.<rdquo/ <rj><au>Milton.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>Sparing; frugal; parsimonious; stingy.</def> <mark>[Obs.]</mark> <ldquo/Too <xex>scarce</xex> ne too sparing.<rdquo/ <rj><au>Chaucer.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>To make one's self scarce</b></col>, <cd>to decamp; to depart.</cd> <mark>[Slang]</mark></cs><br/
[<source>1913 Webster</source>]</p>

<p><syn><b>Syn.</b> -- Rare; infrequent; deficient. See <er>Rare</er>.</syn><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scarcely</ent><br/
<ent>Scarce</ent><br/
<mhw>{ <hw>Scarce</hw>, <hw>Scarce"ly</hw>, }</mhw> <pos>adv.</pos> <sn>1.</sn> <def>With difficulty; hardly; scantly; barely; but just.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>With a <qex>scarce</qex> well-lighted flame.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>The eldest <qex>scarcely</qex> five year was of age.</q> <rj><qau>Chaucer.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Slowly she sails, and <qex>scarcely</qex> stems the tides.</q> <rj><qau>Dryden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>He had <qex>scarcely</qex> finished, when the laborer arrived who had been sent for my ransom.</q> <rj><qau>W. Irving.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Frugally; penuriously.</def> <mark>[Obs.]</mark> <rj><au>Chaucer.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scarcement</ent><br/
<hw>Scarce"ment</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Arch. & Engin.)</fld> <def>An offset where a wall or bank of earth, etc., retreats, leaving a shelf or footing.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scarcity</ent><br/
<ent>Scarceness</ent><br/
<mhw>{ <hw>Scarce"ness</hw> <pr>(?)</pr>, <hw>Scar"ci*ty</hw> <pr>(?)</pr>, }</mhw> <pos>n.</pos> <def>The quality or condition of being scarce; smallness of quantity in proportion to the wants or demands; deficiency; lack of plenty; short supply; penury; <as>as, a <ex>scarcity</ex> of grain; a great <ex>scarcity</ex> of beauties</as>.</def> <rj><au>Chaucer.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>A <qex>scarcity</qex> of snow would raise a mutiny at Naples.</q> <rj><qau>Addison.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Praise . . . owes its value to its <qex>scarcity</qex>.</q> <rj><qau>Rambler.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>The value of an advantage is enhanced by its <qex>scarceness</qex>.</q> <rj><qau>Collier.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><syn><b>Syn.</b> -- Deficiency; lack; want; penury; dearth; rareness; rarity; infrequency.</syn><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scard</ent><br/
<hw>Scard</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A shard or fragment.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scare</ent><br/
<hw>Scare</hw> <pr>(?)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Scared</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Scaring</conjf>.]</vmorph> <ety>[OE. <ets>skerren</ets>, <ets>skeren</ets>, Icel. <ets>skirra</ets> to bar, prevent, <ets>skirrask</ets> to shun , shrink from; or fr. OE. <ets>skerre</ets>, adj., scared, Icel. <ets>skjarr</ets>; both perhaps akin to E. <ets>sheer</ets> to turn.]</ety> <def>To frighten; to strike with sudden fear; to alarm.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The noise of thy crossbow<br/
Will <qex>scare</qex> the herd, and so my shoot is lost.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>To scare away</b></col>, <cd>to drive away by frightening.</cd> -- <col><b>To scare up</b></col>, <cd>to find by search, as if by beating for game.</cd> <mark>[Slang]</mark></cs><br/
[<source>1913 Webster</source>]</p>

<p><syn><b>Syn.</b> -- To alarm; frighten; startle; affright; terrify.</syn><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scare</ent><br/
<hw>Scare</hw>, <pos>n.</pos> <def>Fright; esp., sudden fright produced by a trifling cause, or originating in mistake.</def> <mark>[Colloq.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scarecrow</ent><br/
<hw>Scare"crow`</hw> <pr>(?)</pr>, <pos>n.</pos> <sn>1.</sn> <def>Anything set up to frighten crows or other birds from cornfields; hence, anything terifying without danger.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>A <qex>scarecrow</qex> set to frighten fools away.</q> <rj><qau>Dryden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A person clad in rags and tatters.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>No eye hath seen such <qex>scarecrows</qex>. I'll not march with them through Coventry, that's flat.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <fld>(Zool.)</fld> <def>The black tern.</def> <mark>[Prov. Eng.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scarefire</ent><br/
<hw>Scare"fire`</hw> <pr>(?)</pr>, <pos>n.</pos> <sn>1.</sn> <def>An alarm of fire.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A fire causing alarm.</def> <mark>[Obs.]</mark> <rj><au>Fuller.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scarf</ent><br/
<hw>Scarf</hw> <pr>(sk<aum/rf)</pr>, <pos>n.</pos> <ety>[Icel. <ets>skarfr</ets>.]</ety> <def>A cormorant.</def> <mark>[Scot.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scarf</ent><br/
<hw>Scarf</hw>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Scarfs</plw>, rarely <plw>Scarves</plw> <pr>(sk<aum/rvz)</pr>.</plu> <ety>[Cf. OF. <ets>escharpe</ets> a pilgrim's scrip, or wallet (hanging about the neck), F. <ets><eacute/charpe</ets> sash, scarf; probably from OHG. <ets>scharpe</ets> pocket; also (from the French) Dan. <ets>skiaerf</ets>; Sw. <ets>sk<aum/rp</ets>, Prov. G. <ets>sch<aum/rfe</ets>, LG. <ets>scherf</ets>, G. <ets>sch<aum/rpe</ets>; and also AS. <ets>scearf</ets> a fragment; possibly akin to E. <ets>scrip</ets> a wallet. Cf. <er>Scarp</er> a scarf.]</ety> <def>An article of dress of a light and decorative character, worn loosely over the shoulders or about the neck or the waist; a light shawl or handkerchief for the neck; also, a cravat; a neckcloth.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Put on your hood and <qex>scarf</qex>.</q> <rj><qau>Swift.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>With care about the banners, <qex>scarves</qex>, and staves.</q> <rj><qau>R. Browning.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scarf</ent><br/
<hw>Scarf</hw>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Scarfed</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Scarfing</conjf>.]</vmorph> <sn>1.</sn> <def>To throw on loosely; to put on like a scarf.</def> <ldquo/My sea-gown <xex>scarfed</xex> about me.<rdquo/ <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To dress with a scarf, or as with a scarf; to cover with a loose wrapping.</def> <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scarf</ent><br/
<hw>Scarf</hw>, <pos>v. t.</pos> <ety>[Sw. <ets>skarfva</ets> to eke out, to join together, <ets>skarf</ets> a seam, joint; cf. Dan. <ets>skarre</ets> to joint, to unite timber, Icel. <ets>skara</ets> to clinch the planks of a boat, G. <ets>scharben</ets> to chop, to cut small.]</ety> <sd>(a)</sd> <def>To form a scarf on the end or edge of, as for a joint in timber, metal rods, etc.</def> <sd>(b)</sd> <def>To unite, as two pieces of timber or metal, by a scarf joint.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scarf</ent><br/
<hw>Scarf</hw> <pr>(?)</pr>, <pos>n.</pos> <sd>(a)</sd> <def>In a piece which is to be united to another by a scarf joint, the part of the end or edge that is tapered off, rabbeted, or notched so as to be thinner than the rest of the piece.</def> <sd>(b)</sd> <def>A scarf joint.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Scarf joint</b></col> <sd>(a)</sd> <cd>A joint made by overlapping and bolting or locking together the ends of two pieces of timber that are halved, notched, or cut away so that they will fit each other and form a lengthened beam of the same size at the junction as elsewhere</cd>. <sd>(b)</sd> <cd>A joint formed by welding, riveting, or brazing together the overlapping scarfed ends, or edges, of metal rods, sheets, etc.</cd> -- <col><b>Scarf weld</b></col>. <cd>See under <er>Weld</er>.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scarfskin</ent><br/
<hw>Scarf"skin`</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Anat.)</fld> <def>See <er>Epidermis</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scarification</ent><br/
<hw>Scar`i*fi*ca"tion</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>scarificatio</ets>: cf. F. <ets>scarification</ets>.]</ety> <def>The act of scarifying.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scarificator</ent><br/
<hw>Scar"i*fi*ca`tor</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Cf. F. <ets>scarificateur</ets>.]</ety> <fld>(Surg.)</fld> <def>An instrument, principally used in cupping, containing several lancets moved simultaneously by a spring, for making slight incisions.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scarifier</ent><br/
<hw>Scar"i*fi`er</hw> <pr>(?)</pr>, <pos>n.</pos> <sn>1.</sn> <def>One who scarifies.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Surg.)</fld> <def>The instrument used for scarifying.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <fld>(Agric.)</fld> <def>An implement for stripping and loosening the soil, without bringing up a fresh surface.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>You have your <qex>scarifiers</qex> to make the ground clean.</q> <rj><qau>Southey.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scarify</ent><br/
<hw>Scar"i*fy</hw> <pr>(?)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Scarified</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Scarifying</conjf> <pr>(?)</pr>.]</vmorph> <ety>[F. <ets>scarifier</ets>, L. <ets>scarificare</ets>, <ets>scarifare</ets>, fr. Gr. <?/ to scratch up, fr. <?/ a pointed instrument.]</ety> <sn>1.</sn> <def>To scratch or cut the skin of; esp. <fld>(Med.)</fld>, to make small incisions in, by means of a lancet or scarificator, so as to draw blood from the smaller vessels without opening a large vein.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Agric.)</fld> <def>To stir the surface soil of, as a field.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scarious</ent><br/
<ent>Scariose</ent><br/
<mhw>{ <hw>Sca"ri*ose</hw> <pr>(?)</pr>, <hw>Sca"ri*ous</hw> <pr>(?)</pr>, }</mhw> <pos>a.</pos> <ety>[F. <ets>scarieux</ets>, NL. <ets>scariosus</ets>. Cf. <er>Scary</er>.]</ety> <fld>(Bot.)</fld> <def>Thin, dry, membranous, and not green.</def> <rj><au>Gray.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scarlatina</ent><br/
<hw>Scar`la*ti"na</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[NL.: cf. F. <ets>scarlatine</ets>. See <er>Scarlet</er>.]</ety> <fld>(Med.)</fld> <def>Scarlet fever.</def> -- <wordforms><wf>Scar`la*ti"nal</wf> <pr>(#)</pr>, <pos>a.</pos> -- <wf>Scar*lat"i*nous</wf> <pr>(# <or/ #)</pr>, <pos>a.</pos></wordforms><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scarless</ent><br/
<hw>Scar"less</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Free from scar.</def> <rj><au>Drummond.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scarlet</ent><br/
<hw>Scar"let</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[OE. <ets>scarlat</ets>, <ets>scarlet</ets>, OF. <ets>escarlate</ets>, F. <ets><eacute/carlate</ets> (cf. Pr. <ets>escarlat</ets>, <ets>escarlata</ets>, Sp. & Pg. <ets>escarlata</ets>, It. <ets>scarlatto</ets>, LL. <ets>scarlatum</ets>), from Per. <ets>sakirl<amac/t</ets>.]</ety> <def>A deep bright red tinged with orange or yellow, -- of many tints and shades; a vivid or bright red color.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Cloth of a scarlet color.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>All her household are clothed with <qex>scarlet</qex>.</q> <rj><qau>Prov. xxxi. 21.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scarlet</ent><br/
<hw>Scar"let</hw>, <pos>a.</pos> <def>Of the color called scarlet; <as>as, a <ex>scarlet</ex> cloth or thread</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Scarlet admiral</b></col> <fld>(Zool.)</fld>, <cd>the red admiral. See under <er>Red</er>. -- Scarlet bean <fld>(Bot.)</fld>, a kind of bean (<spn>Phaseolus multiflorus</spn>) having scarlet flowers; scarlet runner.</cd> -- <col><b>Scarlet fever</b></col> <fld>(Med.)</fld>, <cd>a contagious febrile disease characterized by inflammation of the fauces and a scarlet rash, appearing usually on the second day, and ending in desquamation about the sixth or seventh day.</cd> -- <col><b>Scarlet fish</b></col> <fld>(Zool.)</fld>, <cd>the telescope fish; -- so called from its red color. See under <er>Telescope</er>.</cd> -- <col><b>Scarlet ibis</b></col> <fld>(Zool.)</fld> <cd>See under <er>Ibis</er>.</cd> -- <col><b>Scarlet maple</b></col> <fld>(Bot.)</fld>, <cd>the red maple. See <er>Maple</er>.</cd> -- <col><b>Scarlet mite</b></col> <fld>(Zool.)</fld>, <cd>any one of numerous species of bright red carnivorous mites found among grass and moss, especially <spn>Thombidium holosericeum</spn> and allied species. The young are parasitic upon spiders and insects.</cd> -- <col><b>Scarlet oak</b></col> <fld>(Bot.)</fld>, <cd>a species of oak (<spn>Quercus coccinea</spn>) of the United States; -- so called from the scarlet color of its leaves in autumn.</cd> -- <col><b>Scarlet runner</b></col> <fld>(Bot.)</fld>, <cd>the scarlet bean.</cd> -- <col><b>Scarlet tanager</b></col>. <fld>(Zool.)</fld> <cd>See under <er>Tanager</er>.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scarlet</ent><br/
<hw>Scar"let</hw>, <pos>v. t.</pos> <def>To dye or tinge with scarlet.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>The ashy paleness of my cheek<br/
Is <qex>scarleted</qex> in ruddy flakes of wrath.</q> <rj><qau>Ford.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scarmoge</ent><br/
<ent>Scarmage</ent><br/
<mhw>{ <hw>Scar"mage</hw> <pr>(?)</pr>, <hw>Scar"moge</hw> <pr>(?)</pr>, }</mhw> <pos>n.</pos> <def>A slight contest; a skirmish. See <er>Skirmish</er>.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>Such cruel game my <qex>scarmoges</qex> disarms.</q> <rj><qau>Spenser.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scarn</ent><br/
<hw>Scarn</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Icel. <ets>skarn</ets>; akin to AS. <ets>scearn</ets>. Cf. <er>Shearn</er>.]</ety> <def>Dung.</def> <mark>[Obs. or Prov. Eng.]</mark> <rj><au>Ray.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Scarn bee</b></col> <fld>(Zool.)</fld>, <cd>a dung beetle.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scaroid</ent><br/
<hw>Sca"roid</hw>, <pos>a.</pos> <ety>[<ets>Scarus</ets> + <ets>-oid</ets>.]</ety> <fld>(Zool.)</fld> <def>Of or pertaining to the <fam>Scaridae</fam>, a family of marine fishes including the parrot fishes.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scarp</ent><br/
<hw>Scarp</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[OF. <ets>escharpe</ets>. See 2d <er>Scarf</er>.]</ety> <fld>(Her.)</fld> <def>A band in the same position as the bend sinister, but only half as broad as the latter.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scarp</ent><br/
<hw>Scarp</hw>, <pos>n.</pos> <ety>[Aphetic form of <er>Escarp</er>.]</ety> <sn>1.</sn> <fld>(Fort.)</fld> <def>The slope of the ditch nearest the parapet; the escarp.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A steep descent or declivity.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scarp</ent><br/
<hw>Scarp</hw>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Scarped</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Scarping</conjf>.]</vmorph> <def>To cut down perpendicularly, or nearly so; <as>as, to <ex>scarp</ex> the face of a ditch or a rock</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>From <qex>scarped</qex> cliff and quarried stone.</q> <rj><qau>Tennyson.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Sweep ruins from the <qex>scarped</qex> mountain.</q> <rj><qau>Emerson.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scarring</ent><br/
<hw>Scar"ring</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A scar; a mark.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>We find upon the limestone rocks the <qex>scarrings</qex> of the ancient glacier which brought the bowlder here.</q> <rj><qau>Tyndall.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scarry</ent><br/
<hw>Scar"ry</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Bearing scars or marks of wounds.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scarry</ent><br/
<hw>Scar"ry</hw>, <pos>a.</pos> <ety>[See 4th <er>Scar</er>.]</ety> <def>Like a scar, or rocky eminence; containing scars.</def> <rj><au>Holinshed.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scarus</ent><br/
||<hw>Sca"rus</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. See <er>Scar</er> a kind of fish.]</ety> <fld>(Zool.)</fld> <def>A Mediterranean food fish (<spn>Sparisoma scarus</spn>) of excellent quality and highly valued by the Romans; -- called also <altname>parrot fish</altname>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scary</ent><br/
<hw>Sca"ry</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Prov. E. <ets>scare</ets> scraggy.]</ety> <def>Barren land having only a thin coat of grass.</def> <mark>[Prov. Eng.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scary</ent><br/
<hw>Scar"y</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[From <er>Scare</er>.]</ety> <sn>1.</sn> <def>Subject to sudden alarm.</def> <mark>[Colloq. U. S.]</mark> <rj><au>Whittier.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Causing fright; alarming.</def> <mark>[Colloq. U. S.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scasely</ent><br/
<hw>Scase"ly</hw> <pr>(?)</pr>, <pos>adv.</pos> <def>Scarcely; hardly.</def> <mark>[Obs. or Colloq.]</mark> <rj><au>Robynson (More's Utopia)</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scat</ent><br/
<hw>Scat</hw> <pr>(sk<acr/t)</pr>, <pos>interj.</pos> <def>Go away; begone; away; -- chiefly used in driving off a cat.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scatt</ent><br/
<ent>Scat</ent><br/
<mhw>{ <hw>Scat</hw>, <hw>Scatt</hw>, }</mhw> <pos>n.</pos> <ety>[Icel. <ets>skattr</ets>.]</ety> <def>Tribute.</def> <mark>[R.]</mark> <ldquo/Seizing <xex>scatt</xex> and treasure.<rdquo/ <rj><au>Longfellow.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scat</ent><br/
<hw>Scat</hw>, <pos>n.</pos> <def>A shower of rain.</def> <mark>[Prov. Eng.]</mark> <rj><au>Wright.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scatch</ent><br/
<hw>Scatch</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F. <ets>escache</ets>.]</ety> <def>A kind of bit for the bridle of a horse; -- called also <altname>scatchmouth</altname>.</def> <rj><au>Bailey.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scatches</ent><br/
<hw>Scatch"es</hw> <pr>(?)</pr>, <pos>n. pl.</pos> <ety>[OF. <ets>eschaces</ets>, F. <ets><eacute/chasses</ets>, fr. D. <ets>schaats</ets> a high-heeled shoe, a skate. See <er>Skate</er>, for the foot.]</ety> <def>Stilts.</def> <mark>[Prov. Eng.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scate</ent><br/
<hw>Scate</hw> <pr>(sk<amac/t)</pr>, <pos>n.</pos> <def>See <er>Skate</er>, for the foot.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scatebrous</ent><br/
<hw>Scat"e*brous</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>scatebra</ets> a gushing up of water, from <ets>scatere</ets> to bubble, gush.]</ety> <def>Abounding with springs.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scath</ent><br/
<hw>Scath</hw> <pr>(sk<acr/th; 277)</pr>, <pos>n.</pos> <ety>[Icel. <ets>ska<edh/i</ets>; akin to Dan. <ets>skade</ets>, Sw. <ets>skada</ets>, AS. <ets>scea<edh/a</ets>, <ets>sca<edh/a</ets>, foe, injurer, OS. <ets>ska<edh/o</ets>, D. <ets>schade</ets>, harm, injury, OHG. <ets>scade</ets>, G. <ets>schade</ets>, <ets>schaden</ets>; cf. Gr. <grk>'askhqh`s</grk> unharmed. Cf. <er>Scathe</er>, <pos>v.</pos>]</ety> <def>Harm; damage; injury; hurt; waste; misfortune.</def> <altsp>[Written also <asp>scathe</asp>.]</altsp><br/
[<source>1913 Webster</source>]</p>

<p><q>But she was somedeal deaf, and that was <qex>skathe</qex>.</q> <rj><qau>Chaucer.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Great mercy, sure, for to enlarge a thrall,<br/
Whose freedom shall thee turn to greatest <qex>scath</qex>.</q> <rj><qau>Spenser.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Wherein Rome hath done you any <qex>scath</qex>,<br/
Let him make treble satisfaction.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scath</ent><br/
<ent>Scathe</ent><br/
<mhw>{ <hw>Scathe</hw> <pr>(sk<amac/<th/; 277)</pr>, <hw>Scath</hw> <pr>(sk<acr/th; 277)</pr>, }</mhw> <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Scathed</conjf> <pr>(sk<amac/<th/d or sk<acr/tht)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Scathing</conjf> <pr>(sk<amac/<th/"<icr/ng or sk<acr/th"-)</pr>.]</vmorph> <ety>[Icel. <ets>ska<edh/a</ets>; akin to AS. <ets>scea<edh/an</ets>, <ets>sce<edh/<edh/an</ets>, Dan. <ets>skade</ets>, Sw. <ets>skada</ets>, D. & G. <ets>schaden</ets>, OHG. <ets>scad<omac/n</ets>, Goth. <ets>ska<thorn/jan</ets>.]</ety> <def>To do harm to; to injure; to damage; to waste; to destroy.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>As when heaven's fire<br/
Hath <qex>scathed</qex> the forest oaks or mountain pines.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Strokes of calamity that <qex>scathe</qex> and scorch the soul.</q> <rj><qau>W. Irving.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scathful</ent><br/
<hw>Scath"ful</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Harmful; doing damage; pernicious.</def> <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p>-- <wordforms><wf>Scath"ful*ness</wf>, <pos>n.</pos></wordforms><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scathless</ent><br/
<hw>Scath"less</hw>, <pos>a.</pos> <def>Unharmed.</def> <rj><au>R. L. Stevenson.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>He, too, . . . is to be dismissed scathless.</q> <rj><qau>Sir W. Scott.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scathly</ent><br/
<hw>Scath"ly</hw>, <pos>a.</pos> <def>Injurious; scathful.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scatter</ent><br/
<hw>Scat"ter</hw> <pr>(?)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Scattered</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Scattering</conjf>.]</vmorph> <ety>[OE. <ets>scateren</ets>. See <er>Shatter</er>.]</ety> <sn>1.</sn> <def>To strew about; to sprinkle around; to throw down loosely; to deposit or place here and there, esp. in an open or sparse order.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>And some are <qex>scattered</qex> all the floor about.</q> <rj><qau>Chaucer.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Why should my muse enlarge on Libyan swains,<br/
Their <qex>scattered</qex> cottages, and ample plains?</q> <rj><qau>Dryden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Teach the glad hours to <qex>scatter</qex>, as they fly,<br/
Soft quiet, gentle love, and endless joy.</q> <rj><qau>Prior.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To cause to separate in different directions; to reduce from a close or compact to a loose or broken order; to dissipate; to disperse.</def><br/
[<source>1913 Webster</source>]</p>

<p><q><qex>Scatter</qex> and disperse the giddy Goths.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>Hence, to frustrate, disappoint, and overthrow; <as>as, to <ex>scatter</ex> hopes, plans, or the like</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><syn><b>Syn.</b> -- To disperse; dissipate; spread; strew.</syn><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scatter</ent><br/
<hw>Scat"ter</hw>, <pos>v. i.</pos> <def>To be dispersed or dissipated; to disperse or separate; <as>as, clouds <ex>scatter</ex> after a storm</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scatter-brain</ent><br/
<hw>Scat"ter-brain`</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A giddy or thoughtless person; one incapable of concentration or attention.</def> <altsp>[Written also <asp>scatter-brains</asp>.]</altsp><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scatter-brained</ent><br/
<hw>Scat"ter-brained`</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Giddy; thoughtless.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scattered</ent><br/
<hw>Scat"tered</hw> <pr>(?)</pr>, <pos>a.</pos> <sn>1.</sn> <def>Dispersed; dissipated; sprinkled, or loosely spread.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Bot.)</fld> <def>Irregular in position; having no regular order; <as>as, <ex>scattered</ex> leaves</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p>-- <wordforms><wf>Scat"tered*ly</wf>, <pos>adv.</pos> -- <wf>Scat"tered*ness</wf>, <pos>n.</pos></wordforms><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scattergood</ent><br/
<hw>Scat"ter*good`</hw> <pr>(?)</pr>, <pos>n.</pos> <def>One who wastes; a spendthrift.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scattering</ent><br/
<hw>Scat"ter*ing</hw>, <pos>a.</pos> <def>Going or falling in various directions; not united or aggregated; divided among many; <as>as, <ex>scattering</ex> votes</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scattering</ent><br/
<hw>Scat"ter*ing</hw>, <pos>n.</pos> <def>Act of strewing about; something scattered.</def> <rj><au>South.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scatteringly</ent><br/
<hw>Scat"ter*ing*ly</hw>, <pos>adv.</pos> <def>In a scattering manner; dispersedly.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scatterling</ent><br/
<hw>Scat"ter*ling</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[<ets>Scatter</ets> + <ets>-ling</ets>.]</ety> <def>One who has no fixed habitation or residence; a vagabond.</def> <mark>[Obs.]</mark> <ldquo/Foreign <xex>scatterlings</xex>.<rdquo/ <rj><au>Spenser.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scaturient</ent><br/
<hw>Sca*tu"ri*ent</hw> <pr>(?)</pr>, <pos>a.</pos><ety>[L. <ets>scaturiens</ets>, p. pr. of <ets>scaturire</ets> gush out, from <ets>scatere</ets> to bubble, gush.]</ety> <def>Gushing forth; full to overflowing; effusive.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>A pen so <qex>scaturient</qex> and unretentive.</q> <rj><qau>Sir W. Scott.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scaturiginous</ent><br/
<hw>Scat`u*rig"i*nous</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>scaturiginosus</ets>, fr. <ets>scaturigo</ets> gushing water. See <er>Scaturient</er>.]</ety> <def>Abounding with springs.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scaup</ent><br/
<hw>Scaup</hw> <pr>(sk<add/p)</pr>, <pos>n.</pos> <ety>[See <er>Scalp</er> a bed of oysters or mussels.]</ety> <sn>1.</sn> <def>A bed or stratum of shellfish; scalp.</def> <mark>[Scot.]</mark><-- Websters omits the "[" before "See" !! PCP --><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Zool.)</fld> <def>A scaup duck. See below.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Scaup duck</b></col> <fld>(Zool.)</fld>, <cd>any one of several species of northern ducks of the genus <gen>Aythya</gen>, or <gen>Fuligula</gen>. The adult males are, in large part, black. The three North American species are: the greater scaup duck (<spn>Aythya marila</spn>, var. <var>nearctica</var>), called also <altname>broadbill</altname>, <altname>bluebill</altname>, <altname>blackhead</altname>, <altname>flock duck</altname>, <altname>flocking fowl</altname>, and <altname>raft duck</altname>; the lesser scaup duck (<spn>Aythya affinis</spn>), called also <altname>little bluebill</altname>, <altname>river broadbill</altname>, and <altname>shuffler</altname>; the tufted, or ring-necked, scaup duck (<spn>Aythya collaris</spn>), called also <altname>black jack</altname>, <altname>ringneck</altname>, <altname>ringbill</altname>, <altname>ringbill shuffler</altname>, etc. See <xex>Illust.</xex> of <cref>Ring-necked duck</cref>, under <er>Ring-necked</er>. The common European scaup, or mussel, duck (<spn>Aythya marila</spn>), closely resembles the American variety.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><-- p. 1285 pr=PCP --></p>

<p><ent>Scauper</ent><br/
<hw>Scaup"er</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Cf. <er>Scalper</er>.]</ety> <def>A tool with a semicircular edge, -- used by engravers to clear away the spaces between the lines of an engraving.</def>  <rj><au>Fairholt.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scaur</ent><br/
<hw>Scaur</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A precipitous bank or rock; a scar.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scavage</ent><br/
<hw>Scav"age</hw> <pr>(?; 48)</pr>, <pos>n.</pos> <ety>[LL. <ets>scavagium</ets>, fr. AS. <ets>sce<aacute/wian</ets> to look at, to inspect. See <er>Show</er>.]</ety> <fld>(O. Eng. Law)</fld> <def>A toll or duty formerly exacted of merchant strangers by mayors, sheriffs, etc., for goods shown or offered for sale within their precincts.</def> <rj><au>Cowell.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scavenge</ent><br/
<hw>Scav"enge</hw> <pr>(?)</pr>, <pos>v. t.</pos> <def>To cleanse, as streets, from filth.</def>  <rj><au>C. Kingsley.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>to salvage (usable items or material) from discarded or waste material.</def><br/
[<source>PJC</source>]</p>

<p><sn>3.</sn> <def>To remove (burned gases) from the cylinder after a working stroke.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Scavenge</ent><br/
<hw>Scav"enge</hw> <pr>(?)</pr>, <pos>v. i.</pos> <fld>(Internal-combustion Engines)</fld> <def>To remove the burned gases from the cylinder after a working stroke; <as>as, this engine does not <ex>scavenge</ex> well</as>.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Scavenger</ent><br/
<hw>Scav"en*ger</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[OE. <ets>scavager</ets> an officer with various duties, originally attending to <ets>scavage</ets>, fr. OE. & E. <ets>scavage</ets>. See <er>Scavage</er>, <er>Show</er>, <pos>v.</pos>]</ety> <def>A person whose employment is to clean the streets of a city, by scraping or sweeping, and carrying off the filth. The name is also applied to any animal which devours refuse, carrion, or anything injurious to health.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Scavenger beetle</b></col> <fld>(Zool.)</fld>, <cd>any beetle which feeds on decaying substances, as the carrion beetle.</cd> -- <col><b>Scavenger crab</b></col> <fld>(Zool.)</fld>, <cd>any crab which feeds on dead animals, as the spider crab.</cd> -- <col><b>Scavenger's daughter</b></col> <ety>[corrupt. of <ets>Skevington's daughter</ets>]</ety>, <cd>an instrument of torture invented by Sir W. <etsep>Skevington</etsep>, which so compressed the body as to force the blood to flow from the nostrils, and sometimes from the hands and feet.</cd>  <rj><au>Am. Cyc.</au></rj> </cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scavenger hunt</ent><br/
<hw>Scav"en*ger hunt</hw> <pr>(?)</pr>, <pos>n.</pos> <def>a game in which individuals or teams are given a list of items and must go out, gather them together without purchasing them, and bring them back; the first person or team to return with the complete list is the winner. The items are sometimes common but often of a humorous sort.</def><br/
[<source>PJC</source>]</p>

<p><ent>Scavenging</ent><br/
<hw>Scav"eng*ing</hw> <pr>(?)</pr>, <pos>p. pr. & vb. n.</pos> <mord>of <er>Scavenge</er>.</mord> Hence, <pos>n.</pos> <fld>(Internal-combustion Engines)</fld> <def>Act or process of expelling the exhaust gases from the cylinder by some special means, as, in many four-cycle engines, by utilizing the momentum of the exhaust gases in a long exhaust pipe.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Scazon</ent><br/
||<hw>Sca"zon</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L., fr. Gr. <grk>ska`zwn</grk>, fr. <grk>ska`zein</grk> to limp.]</ety> <fld>(Lat. Pros.)</fld> <def>A choliamb.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scelerat</ent><br/
<hw>Scel"er*at</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F. <ets>sc<eacute/l<eacute/rat</ets> from L. <ets>sceleratus</ets>, p. p. of <ets>scelerare</ets> to pollute, from <ets>scelus</ets>, <ets>sceleris</ets>, a crime.]</ety> <def>A villain; a criminal.</def> <mark>[Obs.]</mark>  <rj><au>Cheyne.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scelestic</ent><br/
<hw>Sce*les"tic</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>scelestus</ets>, from <ets>scelus</ets> wickedness.]</ety> <def>Evil; wicked; atrocious.</def> <mark>[Obs.]</mark> <ldquo/<xex>Scelestic</xex> villainies.<rdquo/ <rj><au>Feltham.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scelet</ent><br/
<hw>Scel"et</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[See <er>Skeleton</er>.]</ety> <def>A mummy; a skeleton.</def> <mark>[Obs.]</mark>  <rj><au>Holland.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scena</ent><br/
||<hw>Sce"na</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[It.]</ety> <fld>(Mus.)</fld> <sd>(a)</sd> <def>A scene in an opera.</def> <sd>(b)</sd> <def>An accompanied dramatic recitative, interspersed with passages of melody, or followed by a full aria.</def> <rj><au>Rockstro.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scenario</ent><br/
||<hw>Sce*na"ri*o</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[It.]</ety> <def>A preliminary sketch of the plot, or main incidents, of an opera.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scenary</ent><br/
<hw>Scen"a*ry</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Cf. L. <ets>scaenarius</ets> belonging to the stage.]</ety> <def>Scenery.</def> <mark>[Obs.]</mark> <rj><au>Dryden.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scene</ent><br/
<hw>Scene</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>scaena</ets>, <ets>scena</ets>, Gr. <grk>skhnh`</grk> a covered place, a tent, a stage.]</ety> <sn>1.</sn> <def>The structure on which a spectacle or play is exhibited; the part of a theater in which the acting is done, with its adjuncts and decorations; the stage.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>The decorations and fittings of a stage, representing the place in which the action is supposed to go on; one of the slides, or other devices, used to give an appearance of reality to the action of a play; <as>as, to paint <ex>scenes</ex>; to shift the <ex>scenes</ex>; to go behind the <ex>scenes</ex></as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>So much of a play as passes without change of locality or time, or important change of character; hence, a subdivision of an act; a separate portion of a play, subordinate to the act, but differently determined in different plays; <as>as, an act of four <ex>scenes</ex></as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>My dismal <qex>scene</qex> I needs must act alone.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>The place, time, circumstance, etc., in which anything occurs, or in which the action of a story, play, or the like, is laid; surroundings amid which anything is set before the imagination; place of occurrence, exhibition, or action.</def> <ldquo/In Troy, there lies the <xex>scene</xex>.<rdquo/  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>The world is a vast <qex>scene</qex> of strife.</q> <rj><qau>J. M. Mason.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <def>An assemblage of objects presented to the view at once; a series of actions and events exhibited in their connection; a spectacle; a show; an exhibition; a view.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Through what new <qex>scenes</qex> and changes must we pass!</q> <rj><qau>Addison.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>6.</sn> <def>A landscape, or part of a landscape; scenery.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>A sylvan <qex>scene</qex> with various greens was drawn,<br/
Shades on the sides, and in the midst a lawn.</q> <rj><qau>Dryden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>7.</sn> <def>An exhibition of passionate or strong feeling before others; often, an artifical or affected action, or course of action, done for effect; a theatrical display.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Probably no lover of <qex>scenes</qex> would have had very long to wait for some explosions between parties, both equally ready to take offense, and careless of giving it.</q> <rj><qau>De Quincey.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Behind the scenes</b></col>, <cd>behind the scenery of a theater; out of the view of the audience, but in sight of the actors, machinery, etc.; hence, conversant with the hidden motives and agencies of what appears to public view.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scene</ent><br/
<hw>Scene</hw>, <pos>v. t.</pos> <def>To exhibit as a scene; to make a scene of; to display.</def> <mark>[Obs.]</mark>  <rj><au>Abp. Sancroft.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sceneful</ent><br/
<hw>Scene"ful</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Having much scenery.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sceneman</ent><br/
<hw>Scene"man</hw> <pr>(?)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Scenemen</plw> <pr>(<?/)</pr>.</plu> <def>The man who manages the movable scenes in a theater.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scenery</ent><br/
<hw>Scen"er*y</hw> <pr>(?)</pr>, <pos>n.</pos> <sn>1.</sn> <def>Assemblage of scenes; the paintings and hangings representing the scenes of a play; the disposition and arrangement of the scenes in which the action of a play, poem, etc., is laid; representation of place of action or occurence.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Sum of scenes or views; general aspect, as regards variety and beauty or the reverse, in a landscape; combination of natural views, as woods, hills, etc.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Never need an American look beyond his own country for the sublime and beautiful of natural <qex>scenery</qex>.</q> <rj><qau>W. Irving.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sceneshifter</ent><br/
<hw>Scene"shift`er</hw> <pr>(?)</pr>, <pos>n.</pos> <def>One who moves the scenes in a theater; a sceneman.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scenical</ent><br/
<ent>Scenic</ent><br/
<mhw>{ <hw>Scen"ic</hw> <pr>(?)</pr>, <hw>Scen"ic*al</hw> <pr>(?)</pr>, }</mhw> <pos>a.</pos> <ety>[L. <ets>scaenicus</ets>, <ets>scenicus</ets>, Gr. <?/: cf. F. <ets>sc<eacute/nique</ets>. See <er>Scene</er>.]</ety> <def>Of or pertaining to scenery; of the nature of scenery; theatrical.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>All these situations communicate a <qex>scenical</qex> animation to the wild romance, if treated dramatically.</q> <rj><qau>De Quincey.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scenograph</ent><br/
<hw>Scen"o*graph</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[See <er>Scenography</er>.]</ety> <def>A perspective representation or general view of an object.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scenographical</ent><br/
<ent>Scenographic</ent><br/
<mhw>{ <hw>Scen`o*graph"ic</hw> <pr>(?)</pr>, <hw>Scen`o*graph"ic*al</hw> <pr>(?)</pr>, }</mhw> <pos>a.</pos> <ety>[Cf. F. <ets>sc<eacute/nographique</ets>, Gr. <?/.]</ety> <def>Of or pertaining to scenography; drawn in perspective.</def> -- <wordforms><wf>Scen`o*graph"ic*al*ly</wf>, <pos>adv.</pos></wordforms><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scenography</ent><br/
<hw>Sce*nog"ra*phy</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>scaenographia</ets>, Gr. <?/; <?/ scene, stage + <grk>gra`fein</grk> to write: cf. F. <ets>sc<eacute/nographie</ets>.]</ety> <def>The art or act of representing a body on a perspective plane; also, a representation or description of a body, in all its dimensions, as it appears to the eye.</def>  <rj><au>Greenhill.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scent</ent><br/
<hw>Scent</hw> <pr>(?)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Scented</conjf>; <pos>p. pr. & vb. n.</pos> <conjf>Scenting</conjf>.]</vmorph> <ety>[Originally <ets>sent</ets>, fr. F. <ets>sentir</ets> to feel, to smell. See <er>Sense</er>.]</ety> <sn>1.</sn> <def>To perceive by the olfactory organs; to smell; <as>as, to <ex>scent</ex> game, as a hound does</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Methinks I <qex>scent</qex> the morning air.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To imbue or fill with odor; to perfume.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Balm from a silver box distilled around,<br/
Shall all bedew the roots, and <qex>scent</qex> the sacred ground.</q> <rj><qau>Dryden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scent</ent><br/
<hw>Scent</hw>, <pos>v. i.</pos> <sn>1.</sn> <def>To have a smell.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>Thunderbolts . . . do <qex>scent</qex> strongly of brimstone.</q> <rj><qau>Holland.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To hunt animals by means of the sense of smell.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scent</ent><br/
<hw>Scent</hw>, <pos>n.</pos> <sn>1.</sn> <def>That which, issuing from a body, affects the olfactory organs of animals; odor; smell; <as>as, the <ex>scent</ex> of an orange, or of a rose; the <ex>scent</ex> of musk</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>With lavish hand diffuses <qex>scents</qex> ambrosial.</q> <rj><qau>Prior.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Specifically, the odor left by an animal on the ground in passing over it; <as>as, dogs find or lose the <ex>scent</ex></as>; hence, course of pursuit; track of discovery.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>He gained the observations of innumerable ages, and traveled upon the same <qex>scent</qex> into Ethiopia.</q> <rj><qau>Sir W. Temple.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>The power of smelling; the sense of smell; <as>as, a hound of nice <ex>scent</ex>; to divert the <ex>scent</ex></as>.</def>  <rj><au>I. Watts.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scentful</ent><br/
<hw>Scent"ful</hw> <pr>(?)</pr>, <pos>a.</pos> <sn>1.</sn> <def>Full of scent or odor; odorous.</def> <ldquo/A <xex>scentful</xex> nosegay.<rdquo/  <rj><au>W. Browne.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Of quick or keen smell.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The <qex>scentful</qex> osprey by the rock had fished.</q> <rj><qau>W. Browne.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scentingly</ent><br/
<hw>Scent"ing*ly</hw> <pr>(?)</pr>, <pos>adv.</pos> <def>By scent.</def> <mark>[R.]</mark>  <rj><au>Fuller.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scentless</ent><br/
<hw>Scent"less</hw>, <pos>a.</pos> <def>Having no scent.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The <qex>scentless</qex> and the scented rose.</q> <rj><qau>Cowper.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scepsis</ent><br/
||<hw>Scep"sis</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[NL., from Gr. <?/ doubt, fr. <?/ to consider: cf. G. <ets>skepsis</ets>. See <er>Skeptic</er>.]</ety> <def>Skepticism; skeptical philosophy.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>Among their products were the system of Locke, the <qex>scepsis</qex> of Hume, the critical philosophy of Kant.</q> <rj><qau>J. Martineau.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sceptre</ent><br/
<ent>Scepter</ent><br/
<mhw>{ <hw>Scep"ter</hw>, <hw>Scep"tre</hw>  }</mhw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F. <ets>sceptre</ets>, L. <ets>sceptrum</ets>, from Gr. <?/ a staff to lean upon, a scepter; probably akin to E. <ets>shaft</ets>. See <er>Shaft</er>, and cf. <er>Scape</er> a stem, shaft.]</ety> <sn>1.</sn> <def>A staff or baton borne by a sovereign, as a ceremonial badge or emblem of authority; a royal mace.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>And the king held out Esther the golden <qex>scepter</qex> that was in his hand.</q> <rj><qau>Esther v. 2.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Hence, royal or imperial power or authority; sovereignty; <as>as, to assume the <ex>scepter</ex></as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The <qex>scepter</qex> shall not depart from Judah, nor a lawgiver from between his feet, until Shiloh come.</q> <rj><qau>Gen. xlix. 10.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sceptre</ent><br/
<ent>Scepter</ent><br/
<mhw>{ <hw>Scep"ter</hw>, <hw>Scep"tre</hw>  }</mhw>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Sceptered</conjf> <pr>(?)</pr> or <conjf>Sceptred</conjf> (<?/); <pos>p. pr. & vb. n.</pos> <conjf>Sceptering</conjf> <pr>(?)</pr> or <conjf>Sceptring</conjf> <pr>(<?/)</pr>.]</vmorph> <def>To endow with the scepter, or emblem of authority; to invest with royal authority.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>To Britain's queen the <qex>sceptered</qex> suppliant bends.</q> <rj><qau>Tickell.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scepterellate</ent><br/
<hw>Scep`ter*el"late</hw> <pr>(?)</pr>, <pos>a.</pos> <fld>(Zool.)</fld> <def>Having a straight shaft with whorls of spines; -- said of certain sponge spicules. See <xex>Illust.</xex> under <er>Spicule</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sceptreless</ent><br/
<ent>Scepterless</ent><br/
<mhw>{ <hw>Scep"ter*less</hw>, <hw>Scep"tre*less</hw>  }</mhw>, <pos>a.</pos> <def>Having no scepter; without authority; powerless; <as>as, a <ex>scepterless</ex> king</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scepticism</ent><br/
<ent>Sceptical</ent><br/
<ent>Sceptic</ent><br/
<mhw>{ <hw>Scep"tic</hw> <pr>(?)</pr>, <hw>Scep"tic*al</hw>, <hw>Scep"ti*cism</hw>, etc. }</mhw> <def>See <er>Skeptic</er>, <er>Skeptical</er>, <er>Skepticism</er>, etc.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sceptral</ent><br/
<hw>Scep"tral</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Of or pertaining to a scepter; like a scepter.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scern</ent><br/
<hw>Scern</hw> <pr>(?)</pr>, <pos>v. t.</pos> <def>To discern; to perceive.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Schade</ent><br/
<hw>Schade</hw> <pr>(?)</pr>, <pos>n.</pos> <def>Shade; shadow.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ English words now beginning with <xex>sh</xex>, like <xex>shade</xex>, were formerly often spelled with a <xex>c</xex> between the <xex>s</xex> and <xex>h</xex>; <xex>as</xex>, <xex>schade</xex>; <xex>schame</xex>; <xex>schape</xex>; <xex>schort</xex>, etc.</note><br/
[<source>1913 Webster</source>]</p>

<p><ent>Schah</ent><br/
<hw>Schah</hw> <pr>(?)</pr>, <pos>n.</pos> <def>See <er>Shah</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Schappe</ent><br/
||<hw>Schap"pe</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[G. dial. (Swiss), waste, impurity.]</ety> <def>A silk yarn or fabric made out of carded spun silk.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Schatchen</ent><br/
<hw>Schat"chen</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Yiddish, fr. NHeb. <ets>shadkh<amac/n</ets>, fr. <ets>sh<amac/dakh</ets> to bring about a marriage, orig., to persuade.]</ety> <def>A person whose business is marriage brokage; a marriage broker, esp. among certain Jews.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Schediasm</ent><br/
<hw>Sche"di*asm</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Gr. <?/ an extempore, fr. <?/ to do offhand, <?/ sudden, fr. <?/ near.]</ety> <def>Cursory writing on a loose sheet.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Schedule</ent><br/
<hw>Sched"ule</hw> <pr>(?; <it>in England commonly</it> ?; 277)</pr>, <pos>n.</pos> <ety>[F. <ets>c<eacute/dule</ets>, formerly also spelt <ets>schedule</ets>, L. <ets>schedula</ets>, dim. of scheda, scida, a strip of papyrus bark, a leaf of paper; akin to (or perh. from) Gr. <?/ a tablet, leaf, and to L. <ets>scindere</ets> to cleave, Gr. <?/. See <er>Schism</er>, and cf. <er>Cedule</er>.]</ety> <def>A written or printed scroll or sheet of paper; a document; especially, a formal list or inventory; a list or catalogue annexed to a larger document, as to a will, a lease, a statute, etc.</def><br/
[<source>1913 Webster</source>]</p>

<p><-- 2. Timetable, esp. a list of times at which a conveyance is expected to arrive or leave.     3. Program, a list of items which will occur during an event, usu. with the expected time for each item.     4. Agenda. --><br/
[<source>1913 Webster</source>]</p>

<p><syn><b>Syn.</b> -- Catalogue; list; inventory. see <er>List</er>.</syn><br/
[<source>1913 Webster</source>]</p>

<p><ent>Schedule</ent><br/
<hw>Sched"ule</hw>, <pos>v. t.</pos> <def>To form into, or place in, a schedule.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scheele's green</ent><br/
<hw>Scheele's" green`</hw> <pr>(?)</pr>. <ety>[See <er>Scheelite</er>.]</ety> <fld>(Chem.)</fld> <def>See under <er>Green</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scheelin</ent><br/
<hw>Scheel"in</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Chem.)</fld> <def>Scheelium.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scheelite</ent><br/
<hw>Scheel"ite</hw> <pr>(<?/)</pr>, <pos>n.</pos> <ety>[From C.W. <ets>Scheele</ets>, a Swedish chemist.]</ety> <fld>(Min.)</fld> <def>Calcium tungstate, a mineral of a white or pale yellowish color and of the tetragonal system of crystallization.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scheelium</ent><br/
<hw>Schee"li*um</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[NL. From C.W. <ets>Scheele</ets>, who discovered it.]</ety> <fld>(Chem.)</fld> <def>The metal tungsten.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scheik</ent><br/
<hw>Scheik</hw> <pr>(sh<emac/k <it>or</it>  sh<amac/k)</pr>, <pos>n.</pos> <def>See <er>Sheik</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Schelly</ent><br/
<hw>Schel"ly</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Zool.)</fld> <def>The powan.</def> <mark>[Prov. Eng.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Schema</ent><br/
||<hw>Sche"ma</hw> <pr>(?)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Schemata</plw> <pr>(#)</pr>, E. <plw>Schemas</plw> <pr>(#)</pr>.</plu> <ety>[G. See <er>Scheme</er>.]</ety> <fld>(Kantian Philos.)</fld> <def>An outline or image universally applicable to a general conception, under which it is likely to be presented to the mind; <as>as, five dots in a line are a <ex>schema</ex> of the number five; a preceding and succeeding event are a <ex>schema</ex> of cause and effect</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Schematic</ent><br/
<hw>Sche*mat"ic</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[Cf. Gr. <?/ pretended.]</ety> <def>Of or pertaining to a scheme or a schema.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Schematism</ent><br/
<hw>Sche"ma*tism</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Cf. F. <ets>sch<eacute/matisme</ets> (cf. L. <ets>schematismos</ets> florid speech), fr. Gr. <?/, fr. <?/ to form. See <er>Scheme</er>.]</ety> <sn>1.</sn> <fld>(Astrol.)</fld> <def>Combination of the aspects of heavenly bodies.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Particular form or disposition of a thing; an exhibition in outline of any systematic arrangement.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Schematist</ent><br/
<hw>Sche"ma*tist</hw> <pr>(?)</pr>, <pos>n.</pos> <def>One given to forming schemes; a projector; a schemer.</def>  <rj><au>Swift.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Schematize</ent><br/
<hw>Sche"ma*tize</hw> <pr>(?)</pr>, <pos>v. i.</pos> <ety>[Cf. F. <ets>sch<eacute/matiser</ets>, Gr. <?/.]</ety> <def>To form a scheme or schemes.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scheme</ent><br/
<hw>Scheme</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>schema</ets> a rhetorical figure, a shape, figure, manner, Gr. <?/, <?/, form, shape, outline, plan, fr. <?/, <?/, to have or hold, to hold out, sustain, check, stop; cf. Skr. <ets>sah</ets> to be victorious, to endure, to hold out, AS. <ets>sige</ets> victory, G. <ets>sieg</ets>. <ets>Cf</ets>. <er>Epoch</er>, <er>Hectic</er>, <er>School</er>.]</ety> <sn>1.</sn> <def>A combination of things connected and adjusted by design; a system.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The appearance and outward <qex>scheme</qex> of things.</q> <rj><qau>Locke.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Such a <qex>scheme</qex> of things as shall at once take in time and eternity.</q> <rj><qau>Atterbury.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Arguments . . . sufficient to support and demonstrate a whole <qex>scheme</qex> of moral philosophy.</q> <rj><qau>J. Edwards.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>The Revolution came and changed his whole <qex>scheme</qex> of life.</q> <rj><qau>Macaulay.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A plan or theory something to be done; a design; a project; <as>as, to form a <ex>scheme</ex></as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The stoical <qex>scheme</qex> of supplying our wants by lopping off our desires, is like cutting off our feet when we want shoes.</q> <rj><qau>Swift.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>Any lineal or mathematical diagram; an outline.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>To draw an exact <qex>scheme</qex> of Constantinople, or a map of France.</q> <rj><qau>South.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <fld>(Astrol.)</fld> <def>A representation of the aspects of the celestial bodies for any moment or at a given event.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>A blue silk case, from which was drawn a <qex>scheme</qex> of nativity.</q> <rj><qau>Sir W. Scott.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><syn><b>Syn.</b> -- Plan; project; contrivance; purpose; device; plot.</syn>  -- <usage><er>Scheme</er>, <er>Plan</er>. <xex>Scheme</xex> and <xex>plan</xex> are subordinate to <xex>design</xex>; they propose modes of carrying our designs into effect. <xex>Scheme</xex> is the least definite of the two, and lies more in speculation. A <xex>plan</xex> is drawn out into details with a view to being carried into effect. As <xex>schemes</xex> are speculative, they often prove visionary; hence the opprobrious use of the words <xex>schemer</xex> and <xex>scheming</xex>. <xex>Plans</xex>, being more practical, are more frequently carried into effect.</usage><br/
[<source>1913 Webster</source>]</p>

<p><q>He forms the well-concerted <qex>scheme</qex> of mischief;<br/
'T is fixed, 't is done, and both are doomed to death.</q> <rj><qau>Rowe.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Artists and <qex>plans</qex> relieved my solemn hours;<br/
I founded palaces, and planted bowers.</q> <rj><qau>Prior.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scheme</ent><br/
<hw>Scheme</hw>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Schemed</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Scheming</conjf>.]</vmorph> <def>To make a scheme of; to plan; to design; to project; to plot.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>That wickedness which <qex>schemed</qex>, and executed, his destruction.</q> <rj><qau>G. Stuart.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scheme</ent><br/
<hw>Scheme</hw>, <pos>v. i.</pos> <def>To form a scheme or schemes.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Schemeful</ent><br/
<hw>Scheme"ful</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Full of schemes or plans.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Schemer</ent><br/
<hw>Schem"er</hw> <pr>(?)</pr>, <pos>n.</pos> <def>One who forms schemes; a projector; esp., a plotter; an intriguer.</def><br/
[<source>1913 Webster</source>]</p>

<p><q><qex>Schemers</qex> and confederates in guilt.</q> <rj><qau>Paley.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scheming</ent><br/
<hw>Schem"ing</hw>, <pos>a.</pos> <def>Given to forming schemes; artful; intriguing.</def> -- <wordforms><wf>Schem"ing*ly</wf>, <pos>adv.</pos></wordforms><br/
[<source>1913 Webster</source>]</p>

<p><ent>Schemist</ent><br/
<hw>Schem"ist</hw>, <pos>n.</pos> <def>A schemer.</def> <mark>[R.]</mark> <rj><au>Waterland.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Schene</ent><br/
<hw>Schene</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>schoenus</ets>, Gr. <?/ a rush, a reed, a land measure: cf. F. <ets>sch<egrave/ne</ets>.]</ety> <fld>(Antiq.)</fld> <def>An Egyptian or Persian measure of length, varying from thirty-two to sixty stadia.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Schenkbeer</ent><br/
<hw>Schenk"beer`</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[G. <ets>schenkbier</ets>; <ets>schenken</ets> to pour out + <ets>bier</ets> beer; -- so called because put on draught soon after it is made.]</ety> <def>A mild German beer.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scherbet</ent><br/
<hw>Scher"bet</hw> <pr>(?)</pr>, <pos>n.</pos> <def>See <er>Sherbet</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scherif</ent><br/
||<hw>Scher"if</hw> <pr>(? <or/ ?)</pr>, <pos>n.</pos> <def>See <er>Sherif</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scherzando</ent><br/
||<hw>Scher*zan"do</hw> <pr>(?)</pr>, <pos>adv.</pos> <ety>[It.]</ety> <fld>(Mus.)</fld> <def>In a playful or sportive manner.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scherzo</ent><br/
||<hw>Scher"zo</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[It.]</ety> <fld>(Mus.)</fld> <def>A playful, humorous movement, commonly in 3-4 measure, which often takes the place of the old minuet and trio in a sonata or a symphony.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Schesis</ent><br/
||<hw>Sche"sis</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[NL., fr. Gr. <?/, fr. <?/, <?/, to have or hold. See <er>Scheme</er>.]</ety> <sn>1.</sn> <def>General state or disposition of the body or mind, or of one thing with regard to other things; habitude.</def> <mark>[Obs.]</mark>  <rj><au>Norris.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Rhet.)</fld> <def>A figure of speech whereby the mental habitude of an adversary or opponent is feigned for the purpose of arguing against him.</def> <rj><au>Crabb.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Schetical</ent><br/
<ent>Schetic</ent><br/
<mhw>{ <hw>Schet"ic</hw> <pr>(?)</pr>, <hw>Schet"ic*al</hw> <pr>(?)</pr>, }</mhw> <pos>a.</pos> <ety>[Cf. Gr. <?/ holding back.]</ety> <def>Of or pertaining to the habit of the body; constitutional.</def> <mark>[Obs.]</mark>  <rj><au>Cudworth.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Schiedam</ent><br/
<hw>Schie*dam"</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Short for <ets>Schiedam schnapps</ets>.]</ety> <def>Holland gin made at Schiedam in the Netherlands.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Schiller</ent><br/
<hw>Schil"ler</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[G., play of colors.]</ety> <fld>(Min.)</fld> <def>The peculiar bronzelike luster observed in certain minerals, as hypersthene, schiller spar, etc. It is due to the presence of minute inclusions in parallel position, and is sometimes of secondary origin.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Schiller spar</b></col> <fld>(Min.)</fld>, <cd>an altered variety of enstatite, exhibiting, in certain positions, a bronzelike luster.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Schillerization</ent><br/
<hw>Schil`ler*i*za"tion</hw> <pr>(<?/)</pr>, <pos>n.</pos> <fld>(Min.)</fld> <def>The act or process of producing schiller in a mineral mass.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Schilling</ent><br/
<hw>Schil"ling</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[G. See <er>Shilling</er>.]</ety> <def>Any one of several small German and Dutch coins, worth from about one and a half cents to about five cents.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Schindylesis</ent><br/
||<hw>Schin`dy*le"sis</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[NL., from Gr. <?/ a splitting into fragments.]</ety> <fld>(Anat.)</fld> <def>A form of articulation in which one bone is received into a groove or slit in another.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Schirrhus</ent><br/
<hw>Schir"rhus</hw> <pr>(?)</pr>, <pos>n.</pos> <def>See <er>Scirrhus</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Schism</ent><br/
<hw>Schism</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[OE. <ets>scisme</ets>, OF. <ets>cisme</ets>, <ets>scisme</ets>, F. <ets>schisme</ets>, L. <ets>schisma</ets>, Gr. <grk>schi`sma</grk>, fr. <grk>schi`zein</grk> to split; akin to L. <ets>scindere</ets>, Skr. <ets>chid</ets>, and prob. to E. <ets>shed</ets>, v.t. (which see); cf. <er>Rescind</er>, <er>Schedule</er>, <er>Zest</er>.]</ety> <def>Division or separation;</def> <specif>specifically</specif> <fld>(Eccl.)</fld>, <def>permanent division or separation in the Christian church; breach of unity among people of the same religious faith; the offense of seeking to produce division in a church without justifiable cause.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Set bounds to our passions by reason, to our errors by truth, and to our <qex>schisms</qex> by charity.</q> <rj><qau>Eikon Basilike.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><-- p. 1286 pr=PCP --></p>

<p><cs><col><b>Greek schism</b></col> <fld>(Eccl.)</fld>, <cd>the separation of the Greek and Roman churches.</cd> -- <mcol><col><b>Great schism</b></col>, <it>or</it> <col><b>Western schism</b></col></mcol> <fld>(Eccl.)</fld> <cd>a schism in the Roman church in the latter part of the 14th century, on account of rival claimants to the papal throne.</cd> -- <col><b>Schism act</b></col> <fld>(Law)</fld>, <cd>an act of the English Parliament requiring all teachers to conform to the Established Church, -- passed in 1714, repealed in 1719.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Schisma</ent><br/
||<hw>Schis"ma</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L., a split, separation, Gr. <grk>schi`sma</grk>: cf. F. <ets>schisma</ets>. See <er>Schism</er>.]</ety> <fld>(Anc. Mus.)</fld> <def>An interval equal to half a comma.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Schismatic</ent><br/
<hw>Schis*mat"ic</hw> <pr>(s<icr/z*m<acr/t"<icr/k; <it>so nearly all orthoepists</it>)</pr>, <pos>a.</pos> <ety>[L. <ets>schismaticus</ets>, Gr. <?/: cf. F. <ets>schismatique</ets>.]</ety> <def>Of or pertaining to schism; implying schism; partaking of the nature of schism; tending to schism; <as>as, <ex>schismatic</ex> opinions or proposals</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Schismatic</ent><br/
<hw>Schis*mat"ic</hw> <pr>(?; 277)</pr>, <pos>n.</pos> <def>One who creates or takes part in schism; one who separates from an established church or religious communion on account of a difference of opinion.</def> <ldquo/They were popularly classed together as canting <xex>schismatics</xex>.<rdquo/  <rj><au>Macaulay.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><syn><b>Syn.</b> -- Heretic; partisan. See <er>Heretic</er>.</syn><br/
[<source>1913 Webster</source>]</p>

<p><ent>Schismatical</ent><br/
<hw>Schis*mat"ic*al</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Same as <er>Schismatic</er>.</def> -- <wordforms><wf>Schis*mat"ic*al*ly</wf>, <pos>adv.</pos> -- <wf>Schis*mat"ic*al*ness</wf>, <pos>n.</pos></wordforms><br/
[<source>1913 Webster</source>]</p>

<p><ent>Schismatize</ent><br/
<hw>Schis"ma*tize</hw> <pr>(?)</pr>, <pos>v. i.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Schismatized</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Schismatizing</conjf> <pr>(?)</pr>.]</vmorph> <ety>[Cf. F. <ets>schismatiser</ets>.]</ety> <def>To take part in schism; to make a breach of communion in the church.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Schismless</ent><br/
<hw>Schism"less</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Free from schism.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Schist</ent><br/
<hw>Schist</hw> <pr>(sh<icr/st)</pr>, <pos>n.</pos> <ety>[Gr. <?/ divided, divisible, fr. <?/ to divide: cf. F. <ets>schiste</ets>. See <er>Schism</er>.]</ety> <fld>(Geol.)</fld> <def>Any crystalline rock having a foliated structure (see <er>Foliation</er>) and hence admitting of ready division into slabs or slates. The common kinds are <xex>mica schist</xex>, and <xex>hornblendic schist</xex>, consisting chiefly of quartz with mica or hornblende and often feldspar.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Schistaceous</ent><br/
<hw>Schis*ta"ceous</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Of a slate color.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Schistic</ent><br/
<hw>Schist"ic</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Schistose.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Schistous</ent><br/
<ent>Schistose</ent><br/
<mhw>{ <hw>Schis*tose"</hw> <pr>(?; 277)</pr>, <hw>Schist*ous</hw> <pr>(?)</pr>, }</mhw> <pos>a.</pos> <ety>[Cf. F. <ets>schisteux</ets>.]</ety> <fld>(Geol.)</fld> <def>Of or pertaining to schist; having the structure of a schist.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Schistosity</ent><br/
<hw>Schis*tos"i*ty</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Cf. F. <ets>schistosit<eacute/</ets>.]</ety> <fld>(Geol.)</fld> <def>The quality or state of being schistose.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Schizo-</ent><br/
<hw>Schiz"o-</hw> <pr>(?)</pr>. <ety>[Gr. <?/ to split, cleave.]</ety> <def>A combining form denoting <xex>division</xex> or <xex>cleavage</xex>; <as>as, <ex>schizo</ex>genesis, reproduction by fission or cell division</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Schizocarp</ent><br/
<hw>Schiz"o*carp</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[<ets>Schizo-</ets> + Gr. <?/ fruit.]</ety> <fld>(Bot.)</fld> <def>A dry fruit which splits at maturity into several closed one-seeded portions.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Schizocoele</ent><br/
<hw>Schiz"o*coele</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[<ets>Schizo-</ets> + Gr. <?/ hollow.]</ety> <fld>(Anat.)</fld> <def>See <er>Enterocoele</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Schizocoelous</ent><br/
<hw>Schiz`o*coe"lous</hw> <pr>(?)</pr>, <pos>a.</pos> <fld>(Zool.)</fld> <def>Pertaining to, or of the nature of, a schizocoele.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Schizogenesis</ent><br/
<hw>Schiz`o*gen"e*sis</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[<ets>Schizo-</ets> + <ets>genesis</ets>.]</ety> <fld>(Biol.)</fld> <def>Reproduction by fission.</def> <rj><au>Haeckel.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Schizognath</ent><br/
<hw>Schiz"og*nath</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[See <er>Schizognathous</er>.]</ety> <fld>(Zool.)</fld> <def>Any bird with a schizognathous palate.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Schizognathae</ent><br/
||<hw>Schi*zog"na*thae</hw> <pr>(?)</pr>, <pos>n. pl.</pos> <ety>[NL.]</ety> <fld>(Zool.)</fld> <def>The schizognathous birds.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Schizognathism</ent><br/
<hw>Schi*zog"na*thism</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Zool.)</fld> <def>The condition of having a schizognathous palate.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Schizognathous</ent><br/
<hw>Schi*zog"na*thous</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[<ets>Schizo-</ets> + Gr. <?/ the jaw.]</ety> <fld>(Zool.)</fld> <def>Having the maxillo-palatine bones separate from each other and from the vomer, which is pointed in front, as in the gulls, snipes, grouse, and many other birds.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Schizomycetes</ent><br/
||<hw>Schiz`o*my*ce"tes</hw> <pr>(?)</pr>, <pos>n. pl.</pos>, <ety>[NL., fr. Gr. <?/ to split + <?/, -<?/, a fungus.]</ety> <fld>(Biol.)</fld> <def>An order of <phylum>Schizophyta</phylum>, including the so-called fission fungi, or bacteria. See <er>Schizophyta</er>, in the Supplement.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Schizonemertea</ent><br/
||<hw>Schiz`o*ne*mer"te*a</hw> <pr>(?)</pr>, <pos>n. pl.</pos> <ety>[NL. See <er>Schizo-</er>, and <er>Nemertes</er>.]</ety> <fld>(Zool.)</fld> <def>A group of nemerteans comprising those having a deep slit along each side of the head. See <xex>Illust.</xex> in Appendix.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Schizont</ent><br/
<hw>Schi"zont</hw> <pr>(sk<imac/"z<ocr/nt <it>or</it> sk<imac/z"<ocr/nt)</pr>, <pos>n.</pos> <ety>[Gr. <?/, <?/, p.pr., cleaving.]</ety> <fld>(Zool.)</fld> <def>In certain Sporozoa, a cell formed by the growth of a sporozoite or merozoite (in a cell or corpuscle of the host) which segment by superficial cleavage, without encystment or conjugation, into  merozoites.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Schizopelmous</ent><br/
<hw>Schiz`o*pel"mous</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[<ets>Schizo-</ets> + Gr. <grk>pe`lma</grk> the sole of the foot.]</ety> <fld>(Zool.)</fld> <def>Having the two flexor tendons of the toes entirely separate, and the <xex>flexor hallucis</xex> going to the first toe only.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Schizophyte</ent><br/
<hw>Schiz"o*phyte</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[<ets>Schizo-</ets> + Gr. <?/ a plant.]</ety> <fld>(Biol.)</fld> <def>One of a class of vegetable organisms, in the classification of Cohn, which includes all of the inferior forms that multiply by fission, whether they contain chlorophyll or not.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Schizopod</ent><br/
<hw>Schiz"o*pod</hw> <pr>(?; 277)</pr>, <pos>n.</pos> <fld>(Zool.)</fld> <def>one of the Schizopoda. Also used adjectively.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Schizopodous</ent><br/
<ent>Schizopod</ent><br/
<mhw>{ <hw>Schiz"o*pod</hw> <pr>(?; 277)</pr>, <hw>Schi*zop"o*dous</hw> <pr>(?)</pr>, }</mhw> <pos>a.</pos> <def>Of or pertaining to a schizopod, or the Schizopoda.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Schizopoda</ent><br/
||<hw>Schi*zop"o*da</hw> <pr>(?)</pr>, <pos>n. pl.</pos>, <ety>[NL. See <er>Schizo-</er>, and <er>-poda</er>.]</ety> <fld>(Zool.)</fld> <def>A division of shrimplike Thoracostraca in which each of the thoracic legs has a long fringed upper branch (exopodite) for swimming.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Schizorhinal</ent><br/
<hw>Schiz`o*rhi"nal</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[<ets>Schizo-</ets> + <ets>rhinal</ets>.]</ety> <sn>1.</sn> <fld>(Anat.)</fld> <def>Having the nasal bones separate.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Zool.)</fld> <def>Having the anterior nostrils prolonged backward in the form of a slit.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Schlich</ent><br/
||<hw>Schlich</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[G.; akin to LG. <ets>slick</ets> mud, D. <ets>slijk</ets>, MHG. <ets>sl<imac/ch</ets>.]</ety> <fld>(Metal.)</fld> <def>The finer portion of a crushed ore, as of gold, lead, or tin, separated by the water in certain wet processes.</def> <altsp>[Written also <asp>slich</asp>, <asp>slick</asp>.]</altsp><br/
[<source>1913 Webster</source>]</p>

<p><ent>Schmelze</ent><br/
<hw>Schmel"ze</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[G. <ets>schmelz</ets>, <ets>schmelzglas</ets>.]</ety> <def>A kind of glass of a red or ruby color, made in Bohemia.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Schnapps</ent><br/
<hw>Schnapps</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[G., a dram of spirits.]</ety> <def>Holland gin.</def> <mark>[U.S.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Schneiderian</ent><br/
<hw>Schnei*de"ri*an</hw> <pr>(<?/)</pr>, <pos>a.</pos> <fld>(Anat.)</fld> <def>Discovered or described by C. V. <xex>Schneider</xex>, a German anatomist of the seventeenth century.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Schneiderian membrane</b></col>, <cd>the mucous membrane which lines the nasal chambers; the pituitary membrane.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Schnorrer</ent><br/
||<hw>Schnor"rer</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Yiddish, fr. G. <ets>schnurrer</ets>, fr. <ets>schnurren</ets> to hum, whir, hence, from the sound of the musical instrument used by strolling beggars, to beg.]</ety> <def>Among the Jews, a beggar.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Schoharie grit</ent><br/
<hw>Scho*har"ie grit`</hw> <pr>(?)</pr>. <fld>(Geol.)</fld> <def>The formation belonging to the middle of the three subdivisions of the Corniferous period in the American Devonian system; -- so called from <etsep>Schoharie</etsep>, in New York, where it occurs. See the <xex>Chart</xex> of <er>Geology</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scholar</ent><br/
<hw>Schol"ar</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[OE. <ets>scoler</ets>, AS. <ets>sc<omac/lere</ets>, fr. L. <ets>scholaris</ets> belonging to a school, fr. <ets>schola</ets> a school. See <er>School</er>.]</ety> <sn>1.</sn> <def>One who attends a school; one who learns of a teacher; one under the tuition of a preceptor; a pupil; a disciple; a learner; a student.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>I am no breeching <qex>scholar</qex> in the schools.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>One engaged in the pursuits of learning; a learned person; one versed in any branch, or in many branches, of knowledge; a person of high literary or scientific attainments; a savant.</def>  <rj><au>Shak. Locke.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>A man of books.</def>  <rj><au>Bacon.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>In English universities, an undergraduate who belongs to the foundation of a college, and receives support in part from its revenues.</def><br/
[<source>1913 Webster</source>]</p>

<p><syn><b>Syn.</b> -- Pupil; learner; disciple.</syn>  -- <usage><er>Scholar</er>, <er>Pupil</er>. <xex>Scholar</xex> refers to the instruction, and <xex>pupil</xex> to the care and government, of a teacher. A <xex>scholar</xex> is one who is under instruction; a <xex>pupil</xex> is one who is under the immediate and personal care of an instructor; hence we speak of a bright <xex>scholar</xex>, and an obedient <xex>pupil</xex>.</usage><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scholarity</ent><br/
<hw>Scho*lar"i*ty</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[OF. <ets>scholarit<eacute/</ets>, or LL.  <ets>scholaritas</ets>.]</ety> <def>Scholarship.</def> <mark>[Obs.]</mark> <rj><au>B. Jonson.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scholarlike</ent><br/
<hw>Schol"ar*like`</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Scholarly.</def> <rj><au>Bacon.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scholarly</ent><br/
<hw>Schol"ar*ly</hw>, <pos>a.</pos> <def>Like a scholar, or learned person; showing the qualities of a scholar; <as>as, a <ex>scholarly</ex> essay or critique</as>.</def> -- <def2><pos>adv.</pos> <def>In a scholarly manner.</def></def2><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scholarship</ent><br/
<hw>Schol"ar*ship</hw>, <pos>n.</pos> <sn>1.</sn> <def>The character and qualities of a scholar; attainments in science or literature; erudition; learning.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>A man of my master's . . . great <qex>scholarship</qex>.</q> <rj><qau>Pope.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Literary education.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>Any other house of <qex>scholarship</qex>.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>Maintenance for a scholar; a foundation for the support of a student.</def>  <rj><au>T. Warton.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><syn><b>Syn.</b> -- Learning; erudition; knowledge.</syn><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scholastic</ent><br/
<hw>Scho*las"tic</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>scholasticus</ets>, Gr. <?/, fr. <?/ to have leisure, to give lectures, to keep a school, from <?/ leisure, a lecture, a school: cf. F. <ets>scholastique</ets>, <ets>scolastique</ets>. See <er>School</er>.]</ety> <sn>1.</sn> <def>Pertaining to, or suiting, a scholar, a school, or schools; scholarlike; <as>as, <ex>scholastic</ex> manners or pride; <ex>scholastic</ex> learning</as>.</def> <rj><au>Sir K. Digby.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Of or pertaining to the schoolmen and divines of the Middle Ages (see <er>Schoolman</er>); <as>as, <ex>scholastic</ex> divinity or theology; <ex>scholastic</ex> philosophy</as>.</def>  <rj><au>Locke.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>Hence, characterized by excessive subtilty, or needlessly minute subdivisions; pedantic; formal.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scholastic</ent><br/
<hw>Scho*las"tic</hw>, <pos>n.</pos> <sn>1.</sn> <def>One who adheres to the method or subtilties of the schools.</def>  <rj><au>Milton.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(R. C. Ch.)</fld> <def>See the Note under <er>Jesuit</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scholastical</ent><br/
<hw>Scho*las"tic*al</hw> <pr>(?)</pr>, <pos>a. & n.</pos> <def>Scholastic.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scholastically</ent><br/
<hw>Scho*las"tic*al*ly</hw>, <pos>adv.</pos> <def>In a scholastic manner.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scholasticism</ent><br/
<hw>Scho*las"ti*cism</hw> <pr>(?)</pr>, <pos>n.</pos> <def>The method or subtilties of the schools of philosophy; scholastic formality; scholastic doctrines or philosophy.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The spirit of the old <qex>scholasticism</qex> . . . spurned laborious investigation and slow induction.</q> <rj><qau>J. P. Smith.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scholia</ent><br/
<hw>Scho"li*a</hw> <pr>(?)</pr>, <pos>n. pl.</pos> <def>See <er>Scholium</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scholiast</ent><br/
<hw>Scho"li*ast</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Gr. <?/, fr. <?/ a scholium: cf. F. <ets>scoliate</ets>. See <er>Scholium</er>.]</ety> <def>A maker of scholia; a commentator or annotator.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>No . . . quotations from Talmudists and <qex>scholiasts</qex> . . . ever marred the effect of his grave temperate discourses.</q> <rj><qau>Macaulay.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scholiastic</ent><br/
<hw>Scho`li*as"tic</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Of or pertaining to a scholiast, or his pursuits.</def>  <rj><au>Swift.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scholiaze</ent><br/
<hw>Scho"li*aze</hw> <pr>(?)</pr>, <pos>v. i.</pos> <ety>[Cf. Gr. <?/.]</ety> <def>To write scholia.</def> <mark>[Obs.]</mark>  <rj><au>Milton.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scholical</ent><br/
<hw>Schol"ic*al</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>scholicus</ets>, Gr. <?/, fr. <?/. See <er>School</er>.]</ety> <def>Scholastic.</def> <mark>[Obs.]</mark> <rj><au>Hales.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scholion</ent><br/
||<hw>Scho"li*on</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[NL.]</ety> <def>A scholium.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>A judgment which follows immediately from another is sometimes called a corollary, or consectary . . . One which illustrates the science where it appears, but is not an integral part of it, is a <qex>scholion</qex>.</q> <rj><qau>Abp. Thomson (Laws of Thought).</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scholium</ent><br/
<hw>Scho"li*um</hw> <pr>(?)</pr>, <pos>n.</pos>; <plu><it>pl.</it> L. <plw>Scholia</plw> <pr>(#)</pr>, E. <plw>Scholiums</plw> <pr>(#)</pr>.</plu> <ety>[NL., fr. Gr. <?/, fr. <?/. See <er>School</er>.]</ety> <sn>1.</sn> <def>A marginal annotation; an explanatory remark or comment; specifically, an explanatory comment on the text of a classic author by an early grammarian.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A remark or observation subjoined to a demonstration or a train of reasoning.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scholy</ent><br/
<hw>Scho"ly</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A scholium.</def> <mark>[Obs.]</mark>  <rj><au>Hooker.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scholy</ent><br/
<hw>Scho"ly</hw> <pr>(?)</pr>, <pos>v. i. & t.</pos> <def>To write scholia; to annotate.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>School</ent><br/
<hw>School</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[For <ets>shoal</ets> a crowd; prob. confused with <ets>school</ets> for learning.]</ety> <def>A shoal; a multitude; <as>as, a <ex>school</ex> of fish</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>School</ent><br/
<hw>School</hw>, <pos>n.</pos> <ety>[OE. <ets>scole</ets>, AS. <ets>sc<?/lu</ets>, L. <ets>schola</ets>, Gr. <?/ leisure, that in which leisure is employed, disputation, lecture, a school, probably from the same root as <?/, the original sense being perhaps, a stopping, a resting. See <er>Scheme</er>.]</ety> <sn>1.</sn> <def>A place for learned intercourse and instruction; an institution for learning; an educational establishment; a place for acquiring knowledge and mental training; <as>as, the <ex>school</ex> of the prophets</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Disputing daily in the <qex>school</qex> of one Tyrannus.</q> <rj><qau>Acts xix. 9.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A place of primary instruction; an establishment for the instruction of children; <as>as, a primary <ex>school</ex>; a common <ex>school</ex>; a grammar <ex>school</ex></as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>As he sat in the <qex>school</qex> at his primer.</q> <rj><qau>Chaucer.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>A session of an institution of instruction.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>How now, Sir Hugh! No <qex>school</qex> to-day?</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>One of the seminaries for teaching logic, metaphysics, and theology, which were formed in the Middle Ages, and which were characterized by academical disputations and subtilties of reasoning.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>At Cambridge the philosophy of Descartes was still dominant in the <qex>schools</qex>.</q> <rj><qau>Macaulay.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <def>The room or hall in English universities where the examinations for degrees and honors are held.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>6.</sn> <def>An assemblage of scholars; those who attend upon instruction in a school of any kind; a body of pupils.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>What is the great community of Christians, but one of the innumerable <qex>schools</qex> in the vast plan which God has instituted for the education of various intelligences?</q> <rj><qau>Buckminster.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>7.</sn> <def>The disciples or followers of a teacher; those who hold a common doctrine, or accept the same teachings; a sect or denomination in philosophy, theology, science, medicine, politics, etc.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Let no man be less confident in his faith . . . by reason of any difference in the several <qex>schools</qex> of Christians.</q> <rj><qau>Jer. Taylor.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>8.</sn> <def>The canons, precepts, or body of opinion or practice, sanctioned by the authority of a particular class or age; <as>as, he was a gentleman of the old <ex>school</ex></as>.</def><br/
[<source>1913 Webster</source>]</p>  <p><q>His face pale but striking, though not handsome after the <qex>schools</qex>.</q> <rj><qau>A. S. Hardy.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>9.</sn> <def>Figuratively, any means of knowledge or discipline; <as>as, the <ex>school</ex> of experience</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><mcol><col><b>Boarding school</b></col>, <col><b>Common school</b></col>, <col><b>District school</b></col>, <col><b>Normal school</b></col></mcol>, <cd>etc. See under <er>Boarding</er>, <er>Common</er>, <er>District</er>, etc.</cd> -- <col><b>High school</b></col>, <cd>a free public school nearest the rank of a college.</cd> <mark>[U. S.]</mark> -- <col><b>School board</b></col>, <cd>a corporation established by law in every borough or parish in England, and elected by the burgesses or ratepayers, with the duty of providing public school accommodation for all children in their district.</cd> -- <mcol><col><b>School committee</b></col>, <col><b>School board</b></col></mcol>, <cd>an elected committee of citizens having charge and care of the public schools in any district, town, or city, and responsible for control of the money appropriated for school purposes.</cd> <mark>[U. S.]</mark><-- added "for" after "responsible", probably omitted inadvertently in original. --> -- <col><b>School days</b></col>, <cd>the period in which youth are sent to school.</cd> -- <col><b>School district</b></col>, <cd>a division of a town or city for establishing and conducting schools.</cd> <mark>[U.S.]</mark> -- <mcol><col><b>Sunday school</b></col>, <it>or</it> <col><b>Sabbath school</b></col></mcol>, <cd>a school held on Sunday for study of the Bible and for religious instruction; the pupils, or the teachers and pupils, of such a school, collectively.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>School</ent><br/
<hw>School</hw>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Schooled</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Schooling</conjf>.]</vmorph> <sn>1.</sn> <def>To train in an institution of learning; to educate at a school; to teach.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>He's gentle, never <qex>schooled</qex>, and yet learned.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To tutor; to chide and admonish; to reprove; to subject to systematic discipline; to train.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>It now remains for you to <qex>school</qex> your child,<br/
And ask why God's Anointed be reviled.</q> <rj><qau>Dryden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>The mother, while loving her child with the intensity of a sole affection, had <qex>schooled</qex> herself to hope for little other return than the waywardness of an April breeze.</q> <rj><qau>Hawthorne.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Schoolbook</ent><br/
<hw>School"book`</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A book used in schools for learning lessons.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Schoolboy</ent><br/
<hw>School"boy`</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A boy belonging to, or attending, a school.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Schooldame</ent><br/
<hw>School"dame`</hw> <pr>(?)</pr>. <pos>n.</pos> <def>A schoolmistress.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Schoolery</ent><br/
<hw>School"er*y</hw> <pr>(<?/)</pr>, <pos>n.</pos> <def>Something taught; precepts; schooling.</def> <mark>[Obs.]</mark>  <rj><au>Spenser.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Schoolfellow</ent><br/
<hw>School"fel`low</hw> <pr>(?)</pr>, <pos>n.</pos> <def>One bred at the same school; an associate in school.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Schoolgirl</ent><br/
<hw>School"girl`</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A girl belonging to, or attending, a school.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Schoolhouse</ent><br/
<hw>School"house`</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A house appropriated for the use of a school or schools, or for instruction.</def><-- a building used for schooling, esp. one used as an elementary school; usu. small, and usu. constructed specifically for that purpose.   --><br/
[<source>1913 Webster</source>]</p>

<p><ent>Schooling</ent><br/
<hw>School"ing</hw>, <pos>n.</pos> <sn>1.</sn> <def>Instruction in school; tuition; education in an institution of learning; act of teaching.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Discipline; reproof; reprimand; <as>as, he gave his son a good <ex>schooling</ex></as>.</def>  <rj><au>Sir W. Scott.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>Compensation for instruction; price or reward paid to an instructor for teaching pupils.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Schooling</ent><br/
<hw>School"ing</hw>, <pos>a.</pos> <ety>[See <er>School</er> a shoal.]</ety> <fld>(Zool.)</fld> <def>Collecting or running in schools or shoals.</def><-- used of fish --><br/
[<source>1913 Webster</source>]</p>

<p><q><qex>Schooling</qex> species like the herring and menhaden.</q> <rj><qau>G. B. Goode.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Schoolma'am</ent><br/
<hw>School"ma'am</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A schoolmistress.</def> <mark>[Colloq.U.S.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Schoolmaid</ent><br/
<hw>School"maid`</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A schoolgirl.</def> <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Schoolman</ent><br/
<hw>School"man`</hw> <pr>(?)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Schoolmen</plw> <pr>(<?/)</pr>.</plu> <def>One versed in the niceties of academical disputation or of school divinity.</def><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ The <xex>schoolmen</xex> were philosophers and divines of the Middle Ages, esp. from the 11th century to the Reformation, who spent much time on points of nice and abstract speculation. They were so called because they taught in the mediaeval universities and schools of divinity.</note><br/
[<source>1913 Webster</source>]</p>

<p><ent>Schoolmaster</ent><br/
<hw>School"mas`ter</hw> <pr>(?)</pr>, <pos>n.</pos> <sn>1.</sn> <def>The man who presides over and teaches a school; a male teacher of a school.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Let the soldier be abroad if he will; he can do nothing in this age. There is another personage abroad, -- a person less imposing, -- in the eyes of some, perhaps, insignificant. The <qex>schoolmaster</qex> is abroad; and I trust to him, armed with his primer, against the soldier in full military array.</q> <rj><qau>Brougham.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>One who, or that which, disciplines and directs.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The law was our <qex>schoolmaster</qex>, to bring us unto Christ.</q> <rj><qau>Gal. iii. 24.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><-- p. 1287 pr=PCP --></p>

<p><ent>Schoolmate</ent><br/
<hw>School"mate`</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A pupil who attends the same school as another.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Schoolmistress</ent><br/
<hw>School"mis`tress</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A woman who governs and teaches a school; a female school-teacher.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Schoolroom</ent><br/
<hw>School"room`</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A room in which pupils are taught.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Schoolship</ent><br/
<hw>School"ship`</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A vessel employed as a nautical training school, in which naval apprentices receive their education at the expense of the state, and are trained for service as sailors. Also, a vessel used as a reform school to which boys are committed by the courts to be disciplined, and instructed as mariners.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>School-teacher</ent><br/
<hw>School"-teach`er</hw> <pr>(?)</pr>, <pos>n.</pos> <def>One who teaches or instructs a school.</def> -- <wordforms><wf>School"-teach`ing</wf>, <pos>n.</pos></wordforms><br/
[<source>1913 Webster</source>]</p>

<p><ent>Schoolward</ent><br/
<hw>School"ward</hw> <pr>(?)</pr>, <pos>adv.</pos> <def>Toward school.</def> <rj><au>Chaucer.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Schooner</ent><br/
<hw>Schoon"er</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[See the Note below.  Cf. <er>Shun</er>.]</ety> <fld>(Naut.)</fld> <def>Originally, a small, sharp-built vessel, with two masts and fore-and-aft rig. Sometimes it carried square topsails on one or both masts and was called a <stype>topsail schooner</stype>. About 1840, longer vessels with three masts, fore-and-aft rigged, came into use, and since that time vessels with four masts and even with six masts, so rigged, are built. Schooners with more than two masts are designated <xex>three-masted schooners</xex>, <xex>four-masted schooners</xex>, etc. See <xex>Illustration</xex> in Appendix.</def><-- since early in the 20th century, almost all ocean commerce has been conducted on motorized ships, and such sailing ships have survived primarily as historical curiosities, or as pleasure boats. --><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ The first <xex>schooner</xex> ever constructed is said to have been built in Gloucester, Massachusetts, about the year 1713, by a Captain Andrew Robinson, and to have received its name from the following trivial circumstance: When the vessel went off the stocks into the water, a bystander cried out,<ldquo/O, how she <xex>scoons</xex>!<rdquo/ Robinson replied, <ldquo/ A <xex>scooner</xex> let her be;<rdquo/ and, from that time, vessels thus masted and rigged have gone by this name. The word <xex>scoon</xex> is popularly used in some parts of New England to denote the act of making stones skip along the surface of water. The Scottish <xex>scon</xex> means the same thing. Both words are probably allied to the Icel. <xex>skunda</xex>, <xex>skynda</xex>, to make haste, hurry, AS. <xex>scunian</xex> to avoid, shun, Prov. E. <xex>scun</xex>. In the New England records, the word appears to have been originally written <xex>scooner</xex>. Babson, in his <ldquo/History of Gloucester,<rdquo/ gives the following extract from a letter written in that place Sept. 25, 1721, by Dr. Moses Prince, brother of the Rev. Thomas Prince, the annalist of New England: <ldquo/This gentleman (Captain Robinson) was first contriver of <xex>schooners</xex>, and built the first of that sort about eight years since.<rdquo/</note><br/
[<source>1913 Webster</source>]</p>

<p><ent>Schooner</ent><br/
<hw>Schoon"er</hw>, <pos>n.</pos> <ety>[D.]</ety> <def>A large goblet or drinking glass, -- used for lager beer or ale.</def> <mark>[U.S.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Schorl</ent><br/
<hw>Schorl</hw> <pr>(sh<ocir/rl)</pr>, <pos>n.</pos> <ety>[G. <ets>sch<oum/rl</ets>; cf. Sw. <ets>sk<oum/rl</ets>.]</ety> <fld>(Min.)</fld> <def>Black tourmaline.</def> <altsp>[Written also <asp>shorl</asp>.]</altsp><br/
[<source>1913 Webster</source>]</p>

<p><ent>Schorlaceous</ent><br/
<hw>Schor*la"ceous</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Partaking of the nature and character of schorl; resembling schorl.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Schorlous</ent><br/
<hw>Schorl"ous</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Schorlaceous.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Schorly</ent><br/
<hw>Schorl"y</hw> <pr>(<?/)</pr>, <pos>a.</pos> <def>Pertaining to, or containing, schorl; <as>as, <ex>schorly</ex> granite</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Schottische</ent><br/
<ent>Schottish</ent><br/
<mhw>{ <hw>Schot"tish</hw>, <hw>Schot"tische</hw>  }</mhw>, <pr>(<?/)</pr>, <pos>n.</pos> <ety>[F. <ets>schottish</ets>, <ets>schotisch</ets> from G. <ets>schottisch</ets> Scottish, Scotch.]</ety> <def>A Scotch round dance in 2-4 time, similar to the polka, only slower; also, the music for such a dance; -- not to be confounded with the <xex><Eacute/cossaise</xex>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Schreibersite</ent><br/
<hw>Schrei"bers*ite</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Named after Carl von <ets>Schreibers</ets>, of Vienna.]</ety> <fld>(Min.)</fld> <def>A mineral occurring in steel-gray flexible folia. It contains iron, nickel, and phosphorus, and is found only in meteoric iron.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Schrode</ent><br/
<hw>Schrode</hw> <pr>(?)</pr>, <pos>n.</pos> <def>See <er>Scrod</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Schwann's sheath</ent><br/
<hw>Schwann's" sheath`</hw> <pr>(?)</pr>. <ety>[So called from Theodor <ets>Schwann</ets>, a German anatomist of the 19th century.]</ety> <fld>(Anat.)</fld> <def>The neurilemma.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Schwann's white substance</ent><br/
<hw>Schwann's white" sub"stance</hw> <pr>(?)</pr>. <fld>(Anat.)</fld> <def>The substance of the medullary sheath.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Schwanpan</ent><br/
<hw>Schwan"pan`</hw> <pr>(?)</pr>, <pos>n.</pos> <def>Chinese abacus.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Schweitzerkaese</ent><br/
||<hw>Schweit"zer*k<aum/"se</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[G. <ets>schweizerk<aum/se</ets> Swiss cheese.]</ety> <def>Gruy<egrave/re cheese.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Schwenkfeldian</ent><br/
<ent>Schwenkfelder</ent><br/
<mhw>{ <hw>Schwenk"feld`er</hw> <pr>(?)</pr>, <hw>Schwenk"feld`i*an</hw> <pr>(?)</pr>, }</mhw> <pos>n.</pos> <def>A member of a religious sect founded by Kaspar von Schwenkfeld, a Silesian reformer who disagreed with Luther, especially on the deification of the body of Christ.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sciaenoid</ent><br/
<hw>Sci*ae"noid</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>sciaena</ets> a kind of fish (fr. Gr. <?/) + <ets>-oid</ets>.]</ety> <fld>(Zool.)</fld> <def>Of or pertaining to the <fam>Sciaenidae</fam>, a family of carnivorous marine fishes which includes the meagre (<spn>Sciaena umbra</spn> or <spn>Sciaena aquila</spn>), and fish of the <stype>drum</stype> and <stype>croaker</stype> families.  The <stype>croaker</stype> is so called because it may make a croaking noise by use of its bladder; the <stype>Atlantic croaker</stype> (<spn>Micropogonias undulatus</spn>, formerly <spn>Micropogon undulatus</spn>) and the squeteague are a members of the croaker family, and the <stype>kingfish</stype> is a drum.</def><br/
[<source>1913 Webster</source> <source>+PJC</source>]</p>

<p><ent>Sciagraph</ent><br/
<hw>Sci"a*graph</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[See <er>Sciagraphy</er>.]</ety> <sn>1.</sn> <fld>(Arch.)</fld> <def>An old term for a vertical section of a building; -- called also <altname>sciagraphy</altname>. See <cref>Vertical section</cref>, under <er>Section</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Phys.)</fld> <def>A radiograph.</def> <altsp>[Written also <asp>skiagraph</asp>.]</altsp><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sciagraphical</ent><br/
<hw>Sci`a*graph"ic*al</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[Cf. F. <ets>sciagraphique</ets>, Gr. <?/.]</ety> <def>Pertaining to sciagraphy.</def> -- <wordforms><wf>Sci`a*graph"ic*al*ly</wf>, <pos>adv.</pos></wordforms><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sciagraphy</ent><br/
<hw>Sci*ag"ra*phy</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Gr. <?/, fr. <?/ drawing in light and shade; <?/ a shadow + <?/ to delineate, describe: cf. F. <ets>sciagraphie</ets>.]</ety> <sn>1.</sn> <def>The art or science of projecting or delineating shadows as they fall in nature.</def>  <rj><au>Gwilt.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Arch.)</fld> <def>Same as <er>Sciagraph</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <fld>(Physics)</fld> <def>Same as <er>Radiography</er>.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Sciamachy</ent><br/
<hw>Sci*am"a*chy</hw> <pr>(?)</pr>, <pos>n.</pos> <def>See <er>Sciomachy</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sciatherical</ent><br/
<ent>Sciatheric</ent><br/
<mhw>{ <hw>Sci`a*ther"ic</hw> <pr>(?)</pr>, <hw>Sci`a*ther"ic*al</hw> <pr>(?)</pr>, }</mhw> <pos>a.</pos> <ety>[Gr. <?/, fr. <?/ a sundial; <?/ a shadow + <?/ to hunt, to catch.]</ety> <def>Belonging to a sundial.</def> <mark>[Obs.]</mark>  <rj><au>Sir T. Browne.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p>-- <wordforms><wf>Sci`a*ther"ic*al*ly</wf>, <pos>adv.</pos> <mark>[Obs.]</mark></wordforms>  <rj><au>J. Gregory.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sciatic</ent><br/
<hw>Sci*at"ic</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[F. <ets>sciatique</ets>, LL. <ets>sciaticus</ets>, from L. <ets>ischiadicus</ets>, Gr. <?/. See <er>Ischiadic</er>.]</ety> <fld>(Anat.)</fld> <def>Of or pertaining to the hip; in the region of, or affecting, the hip; ischial; ischiatic; <as>as, the <ex>sciatic</ex> nerve, <ex>sciatic</ex> pains</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sciatic</ent><br/
<hw>Sci*at"ic</hw>, <pos>n.</pos> <ety>[Cf. F. <ets>sciatique</ets>.]</ety> <fld>(Med.)</fld> <def>Sciatica.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sciatica</ent><br/
<hw>Sci*at"i*ca</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[NL.]</ety> <fld>(Med.)</fld> <def>Neuralgia of the sciatic nerve, an affection characterized by paroxysmal attacks of pain in the buttock, back of the thigh, or in the leg or foot, following the course of the branches of the sciatic nerve. The name is also popularly applied to various painful affections of the hip and the parts adjoining it. See <cref>Ischiadic passion</cref>, under <er>Ischiadic</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sciatical</ent><br/
<hw>Sci*at"ic*al</hw> <pr>(?)</pr>, <pos>a.</pos> <fld>(Anat.)</fld> <def>Sciatic.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sciatically</ent><br/
<hw>Sci*at"ic*al*ly</hw>, <pos>adv.</pos> <def>With, or by means of, sciatica.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scibboleth</ent><br/
<hw>Scib"bo*leth</hw> <pr>(?)</pr>, <pos>n.</pos> <def>Shibboleth.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Science</ent><br/
<hw>Sci"ence</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F., fr. L. <ets>scientia</ets>, fr. <ets>sciens</ets>, <ets>-entis</ets>, p. pr. of <ets>scire</ets> to know.  Cf. <er>Conscience</er>, <er>Conscious</er>, <er>Nice</er>.]</ety> <sn>1.</sn> <def>Knowledge; knowledge of principles and causes; ascertained truth of facts.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>If we conceive God's sight or <qex>science</qex>, before the creation, to be extended to all and every part of the world, seeing everything as it is, . . . his <qex>science</qex> or sight from all eternity lays no necessity on anything to come to pass.</q> <rj><qau>Hammond.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Shakespeare's deep and accurate <qex>science</qex> in mental philosophy.</q> <rj><qau>Coleridge.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Accumulated and established knowledge, which has been systematized and formulated with reference to the discovery of general truths or the operation of general laws; knowledge classified and made available in work, life, or the search for truth; comprehensive, profound, or philosophical knowledge.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>All this new <qex>science</qex> that men lere [teach].</q> <rj><qau>Chaucer.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q><qex>Science</qex> is . . . a complement of cognitions, having, in point of form, the character of logical perfection, and in point of matter, the character of real truth.</q> <rj><qau>Sir W. Hamilton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>Especially, such knowledge when it relates to the physical world and its phenomena, the nature, constitution, and forces of matter, the qualities and functions of living tissues, etc.; -- called also <altname>natural science</altname>, and <altname>physical science</altname>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Voltaire hardly left a single corner of the field entirely unexplored in <qex>science</qex>, poetry, history, philosophy.</q> <rj><qau>J. Morley.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>Any branch or department of systematized knowledge considered as a distinct field of investigation or object of study; <as>as, the <ex>science</ex> of astronomy, of chemistry, or of mind</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ The ancients reckoned seven sciences, namely, grammar, rhetoric, logic, arithmetic, music, geometry, and astronomy; -- the first three being included in the <xex>Trivium</xex>, the remaining four in the <xex>Quadrivium</xex>.</note><br/
[<source>1913 Webster</source>]</p>

<p><q>Good sense, which only is the gift of Heaven,<br/
And though no <qex>science</qex>, fairly worth the seven.</q> <rj><qau>Pope.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <def>Art, skill, or expertness, regarded as the result of knowledge of laws and principles.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>His <qex>science</qex>, coolness, and great strength.</q> <rj><qau>G. A. Lawrence.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ <xex>Science</xex> is <xex>applied</xex> or <xex>pure</xex>. <xex>Applied science</xex> is a knowledge of facts, events, or phenomena, as explained, accounted for, or produced, by means of powers, causes, or laws. <xex>Pure science</xex> is the knowledge of these powers, causes, or laws, considered <xex>apart</xex>, or as pure from all applications. Both these terms have a similar and special signification when applied to the science of quantity; as, the <xex>applied</xex> and <xex>pure mathematics</xex>. <xex>Exact science</xex> is knowledge so systematized that prediction and verification, by measurement, experiment, observation, etc., are possible. The mathematical and physical sciences are called <xex>the exact sciences</xex>.</note><br/
[<source>1913 Webster</source>]</p>

<p><cs><mcol><col><b>Comparative sciences</b></col>, <col><b>Inductive sciences</b></col></mcol>. <cd>See under <er>Comparative</er>, and <er>Inductive</er>.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><syn><b>Syn.</b> -- Literature; art; knowledge.</syn>  -- <usage><er>Science</er>, <er>Literature</er>, <er>Art</er>. <xex>Science</xex> is literally <xex>knowledge</xex>, but more usually denotes a systematic and orderly arrangement of knowledge. In a more distinctive sense, <xex>science</xex> embraces those branches of knowledge of which the subject-matter is either ultimate principles, or facts as explained by principles or laws thus arranged in natural order. The term <xex>literature</xex> sometimes denotes all compositions not embraced under <xex>science</xex>, but usually confined to the <xex>belles-lettres</xex>. [See <er>Literature</er>.] <xex>Art</xex> is that which depends on practice and skill in performance. <ldquo/In <xex>science</xex>, <xex>scimus ut sciamus</xex>; in art, <xex>scimus ut producamus</xex>. And, therefore, <xex>science</xex> and <xex>art</xex> may be said to be investigations of truth; but one, <xex>science</xex>, inquires for the sake of knowledge; the other, <xex>art</xex>, for the sake of production; and hence <xex>science</xex> is more concerned with the higher truths, <xex>art</xex> with the lower; and <xex>science</xex> never is engaged, as <xex>art</xex> is, in productive application.  And the most perfect state of <xex>science</xex>, therefore, will be the most high and accurate inquiry; the perfection of <xex>art</xex> will be the most apt and efficient system of rules; art always throwing itself into the form of rules.<rdquo/</usage>  <rj><au>Karslake.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Science</ent><br/
<hw>Sci"ence</hw>, <pos>v. t.</pos> <def>To cause to become versed in science; to make skilled; to instruct.</def> <mark>[R.]</mark>  <rj><au>Francis.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>science fiction</ent><br/
<hw>sci"ence fic"tion</hw> <pr>(s<imac/"<eit/ns f<icr/k"sh<ucr/n)</pr>, <pos>n.</pos> <ety>[<ets>sci</ets>ence <ets>fi</ets>ction.]</ety> <def>A genre of fiction in which scientific and technological issues feature prominently, especially including scenarios in which speculative but unproven scientific advances are accepted as fact, and usually set at some time in the future, or in some distant region of the universe.</def><br/
[<source>PJC</source>]</p>

<p><ent>Scient</ent><br/
<hw>Sci"ent</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>sciens</ets>, <ets>-entis</ets>, p. pr.]</ety> <def>Knowing; skillful.</def> <mark>[Obs.]</mark> <rj><au>Cockeram.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scienter</ent><br/
||<hw>Sci*en"ter</hw> <pr>(?)</pr>, <pos>adv.</pos> <ety>[L.]</ety> <fld>(Law)</fld> <def>Knowingly; willfully.</def>  <rj><au>Bouvier.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sciential</ent><br/
<hw>Sci*en"tial</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[LL. <ets>scientialis</ets>, fr. L. <ets>scientia</ets>.]</ety> <def>Pertaining to, or producing, science.</def> <mark>[R.]</mark>  <rj><au>Milton.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scientific</ent><br/
<hw>Sci`en*tif"ic</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[F. <ets>scientifique</ets>; L. <ets>scientia</ets> science + <ets>facere</ets> to make.]</ety> <sn>1.</sn> <def>Of or pertaining to science; used in science; <as>as, <ex>scientific</ex> principles; <ex>scientific</ex> apparatus; <ex>scientific</ex> observations</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Agreeing with, or depending on, the rules or principles of science; <as>as, a <ex>scientific</ex> classification; a <ex>scientific</ex> arrangement of fossils</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>Having a knowledge of science, or of a science; evincing science or systematic knowledge; <as>as, a <ex>scientific</ex> chemist; a <ex>scientific</ex> reasoner; a <ex>scientific</ex> argument</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Bossuet is as <qex>scientific</qex> in the structure of his sentences.</q> <rj><qau>Landor.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Scientific method</b></col>, <cd>the method employed in exact science and consisting of: (<it>a</it>) Careful and abundant observation and experiment. (<it>b</it>) generalization of the results into formulated <ldquo/Laws<rdquo/ and statements.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scientifical</ent><br/
<hw>Sci`en*tif"ic*al</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Scientific.</def> <rj><au>Locke.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scientifically</ent><br/
<hw>Sci`en*tif"ic*al*ly</hw>, <pos>adv.</pos> <def>In a scientific manner; according to the rules or principles of science.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>It is easier to believe than to be <qex>scientifically</qex> instructed.</q> <rj><qau>Locke.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scientist</ent><br/
<hw>Sci"en*tist</hw> <pr>(?)</pr>, <pos>n.</pos> <def>One learned in science; a scientific investigator; one devoted to scientific study; a savant.</def> <mark>[Recent]</mark><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ Twenty years ago I ventured to propose one [a name for the class of men who give their lives to scientific study] which has been slowly finding its way to general adoption; and the word <xex>scientist</xex>, though scarcely euphonious, has gradually assumed its place in our vocabulary.</note>  <rj><au>B. A. Gould (Address, 1869).</au></rj> <-- MW10 gives 1834 as first ref. date. --><br/
[<source>1913 Webster</source>]</p>

<p><ent>scifi</ent><br/
<ent>Sci-Fi</ent><br/
<mhw><hw>Sci"-Fi"</hw>, <hw>sci"fi"</hw></mhw> <pr>(s<imac/"f<imac/")</pr>, <pos>n.</pos> <ety>[From <ets>sci</ets>ence <ets>fi</ets>ction.]</ety> <def>Science fiction; -- a common shortened form for the name of the literaray genre.  See <er>science fiction</er>.</def> <mark>[informal]</mark> <br/
[<source>PJC</source>]</p>

<p><ent>Scilicet</ent><br/
<hw>Scil"i*cet</hw> <pr>(?)</pr>, <pos>adv.</pos> <ety>[L., fr. <ets>scire licet</ets> you may know.]</ety> <def>To wit; namely; videlicet; -- often abbreviated to <abbr>sc.</abbr>, or <abbr>ss.</abbr></def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scillain</ent><br/
<hw>Scil"la*in</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Chem.)</fld> <def>A glucoside extracted from squill (<spn>Scilla maritima</spn>) as a light porous substance.</def><br/
[<source>1913 Webster</source> + <source>PJC</source>]</p>

<p><ent>Scillitin</ent><br/
<hw>Scil"li*tin</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Cf. F. <ets>scilitine</ets>.]</ety> <fld>(Chem.)</fld> <def>A bitter principle extracted from the bulbs of the squill (<spn>Scilla maritima</spn>), and probably consisting of a complex mixture of several substances.</def><br/
[<source>1913 Webster</source> + <source>PJC</source>]</p>

<p><ent>Scimitar</ent><br/
<ent>Scimiter</ent><br/
<mhw>{ <hw>Scim"i*ter</hw> , <hw>Scim"i*tar</hw>  }</mhw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F. <ets>cimeterre</ets>, cf. It. <ets>scimitarra</ets>, Sp. <ets>cimitarra</ets>; fr. Biscayan <ets>cimetarra</ets> with a sharp edge; or corrupted from Per. <ets>shimsh<imac/r</ets>.]</ety> <sn>1.</sn> <def>A saber with a much curved blade having the edge on the convex side, -- in use among Mohammedans, esp., the Arabs and persians.</def> <altsp>[Written also <asp>cimeter</asp>, and <asp>scymetar</asp>.]</altsp><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A long-handled billhook. See <er>Billhook</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Scimiter pods</b></col> <fld>(Bot.)</fld>, <cd>the immense curved woody pods of a leguminous woody climbing plant (<spn>Entada scandens</spn>) growing in tropical India and America. They contain hard round flattish seeds two inches in diameter, which are made into boxes.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scincoid</ent><br/
<hw>Scin"coid</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>scincus</ets> a kind of lizard (fr. Gr. <?/) + <ets>-oid</ets>.  Cf. <er>Skink</er>.]</ety> <fld>(Zool.)</fld> <def>Of or pertaining to the family <fam>Scincidae</fam>, or skinks.</def> -- <def2><pos>n.</pos> <def>A scincoidian.</def></def2><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scincoidea</ent><br/
||<hw>Scin*coi"de*a</hw> <pr>(?)</pr>, <pos>n. pl.</pos> <ety>[NL.]</ety> <fld>(Zool.)</fld> <def>A tribe of lizards including the skinks. See <er>Skink</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scincoidian</ent><br/
<hw>Scin*coid"i*an</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Zool.)</fld> <def>Any one of numerous species of lizards of the family <fam>Scincidae</fam> or tribe Scincoidea. The tongue is not extensile. The body and tail are covered with overlapping scales, and the toes are margined. See <xex>Illust.</xex> under <er>Skink</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sciniph</ent><br/
<hw>Scin"iph</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>scinifes</ets>, <ets>cinifes</ets>, or <ets>ciniphes</ets>, pl., Gr. <?/.]</ety> <def>Some kind of stinging or biting insect, as a flea, a gnat, a sandfly, or the like.</def>  <rj><au>Ex. viii. 17 (Douay version).</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scink</ent><br/
<hw>Scink</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Zool.)</fld> <def>A skink.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scink</ent><br/
<hw>Scink</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A slunk calf.</def> <mark>[Obs. or Prov. Eng.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scintilla</ent><br/
||<hw>Scin*til"la</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L.]</ety> <def>A spark; the least particle; an iota; a tittle.</def>  <rj><au>R. North.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scintillant</ent><br/
<hw>Scin"til*lant</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>scintillans</ets>, p. pr. of <ets>scintillare</ets> to sparkle. See <er>Scintillate</er>.]</ety> <def>Emitting sparks, or fine igneous particles; sparkling.</def>  <rj><au>M. Green.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scintillate</ent><br/
<hw>Scin"til*late</hw> <pr>(?)</pr>, <pos>v. i.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Scintillated</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Scintillating</conjf>.]</vmorph> <ety>[L. <ets>scintillare</ets>, <ets>scintillatum</ets>, from <ets>scintilla</ets> a spark.  Cf. <er>Stencil</er>.]</ety> <sn>1.</sn> <def>To emit sparks, or fine igneous particles.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>As the electrical globe only <qex>scintillates</qex> when rubbed against its cushion.</q> <rj><qau>Sir W. Scott.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To sparkle, as the fixed stars.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scintillation</ent><br/
<hw>Scin`til*la"tion</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>scintillatio</ets>: cf. F. <ets>scintillation</ets>.]</ety> <sn>1.</sn> <def>The act of scintillating.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A spark or flash emitted in scintillating.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>These <qex>scintillations</qex> are . . . the inflammable effluences discharged from the bodies collided.</q> <rj><qau>Sir T. Browne.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scintillous</ent><br/
<hw>Scin"til*lous</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Scintillant.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scintillously</ent><br/
<hw>Scin"til*lous*ly</hw>, <pos>adv.</pos> <def>In a scintillant manner.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sciography</ent><br/
<hw>Sci*og"ra*phy</hw> <pr>(?)</pr>, <pos>n.</pos> <def>See <er>Sciagraphy</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sciolism</ent><br/
<hw>Sci"o*lism</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[See <er>Sciolist</er>.]</ety> <def>The knowledge of a sciolist; superficial knowledge.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sciolist</ent><br/
<hw>Sci"o*list</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>sciolus</ets>. See <er>Sciolous</er>.]</ety> <def>One who knows many things superficially; a pretender to science; a smatterer.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>These passages in that book were enough to humble the presumption of our modern <qex>sciolists</qex>, if their pride were not as great as their ignorance.</q> <rj><qau>Sir W. Temple.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>A master were lauded and <qex>sciolists</qex> shent.</q> <rj><qau>R. Browning.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sciolistic</ent><br/
<hw>Sci`o*lis"tic</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Of or pertaining to sciolism, or a sciolist; partaking of sciolism; resembling a sciolist.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sciolous</ent><br/
<hw>Sci"o*lous</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>scilus</ets>, dim. of <ets>scius</ets> knowing, fr. <ets>scire</ets> to know. See <er>Science</er>.]</ety> <def>Knowing superficially or imperfectly.</def>  <rj><au>Howell.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sciomachy</ent><br/
<hw>Sci*om"a*chy</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Gr. <?/, <?/; <?/ a shadow + <?/ battle: cf. F. <ets>sciomachie</ets>, <ets>sciamachie</ets>.]</ety> <def>A fighting with a shadow; a mock contest; an imaginary or futile combat.</def> <altsp>[Written also <asp>scimachy</asp>.]</altsp>  <rj><au>Cowley.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sciomancy</ent><br/
<hw>Sci"o*man`cy</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Gr. <?/ a shadow + <ets>-mancy</ets>: cf. F. <ets>sciomance</ets>, <ets>sciamancie</ets>.]</ety> <def>Divination by means of shadows.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scion</ent><br/
<hw>Sci"on</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[OF. <ets>cion</ets>, F. <ets>scion</ets>, probably from <ets>scier</ets> to saw, fr. L. <ets>secare</ets> to cut. Cf. <er>Section</er>.]</ety> <sn>1.</sn> <fld>(Bot.)</fld> <sd>(a)</sd> <def>A shoot or sprout of a plant; a sucker.</def> <sd>(b)</sd> <def>A piece of a slender branch or twig cut for grafting.</def> <mark>[Formerly written also cion, and cyon.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Hence, a descendant; an heir; <as>as, a <ex>scion</ex> of a royal stock</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scioptic</ent><br/
<hw>Sci*op"tic</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[Gr. <?/ shadow + <?/ belonging to sight: cf. F. <ets>scioptique</ets>. See <er>Optic</er>.]</ety> <fld>(Opt.)</fld> <def>Of or pertaining to an optical arrangement for forming images in a darkened room, usually called <xex>scioptic ball</xex>.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Scioptic ball</b></col> <fld>(Opt.)</fld>, <cd>the lens of a camera obscura mounted in a wooden ball which fits a socket in a window shutter so as to be readily turned, like the eye, to different parts of the landscape.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sciopticon</ent><br/
<hw>Sci*op"ti*con</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[NL. See <er>Scioptic</er>.]</ety> <def>A kind of magic lantern.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scioptics</ent><br/
<hw>Sci*op"tics</hw> <pr>(?)</pr>, <pos>n.</pos> <def>The art or process of exhibiting luminous images, especially those of external objects, in a darkened room, by arrangements of lenses or mirrors.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scioptric</ent><br/
<hw>Sci*op"tric</hw> <pr>(?)</pr>, <pos>a.</pos> <fld>(Opt.)</fld> <def>Scioptic.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sciot</ent><br/
<hw>Sci"ot</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Of or pertaining to the island <etsep>Scio</etsep> (Chio or Chios).</def> -- <def2><pos>n.</pos> <def>A native or inhabitant of Scio.</def></def2> <altsp>[Written also <asp>Chiot</asp>.]</altsp><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sciotheric</ent><br/
<hw>Sci`o*ther"ic</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[Cf. L. <ets>sciothericon</ets> a sundial. See <er>Sciatheric</er>.]</ety> <def>Of or pertaining to a sundial.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Sciotheric telescope</b></col> <fld>(Dialing)</fld>, <cd>an instrument consisting of a horizontal dial, with a telescope attached to it, used for determining the time, whether of day or night.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scious</ent><br/
<hw>Sci"ous</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>scius</ets>.]</ety> <def>Knowing; having knowledge.</def> <ldquo/Brutes may be and are <xex>scious</xex>.<rdquo/ <rj><au>Coleridge.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><-- p. 1288 pr=PCP --></p>

<p><ent>Scire facias</ent><br/
||<hw>Sci`re fa"ci*as</hw> <pr>(s<imac/`r<esl/ f<amac/"sh<icr/*<acr/s)</pr>. <ety>[L., do you cause to know.]</ety> <fld>(Law)</fld> <def>A judicial writ, founded upon some record, and requiring the party proceeded against to show cause why the party bringing it should not have advantage of such record, or (as in the case of <xex>scire facias</xex> to repeal letters patent) why the record should not be annulled or vacated.</def>  <rj><au>Wharton. Bouvier.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scirrhoid</ent><br/
<hw>Scir"rhoid</hw> <pr>(sk<icr/r"roid)</pr>, <pos>a.</pos> <ety>[<ets>Scirrhus</ets> + <ets>-oid</ets>.]</ety> <def>Resembling scirrhus.</def>  <rj><au>Dunglison.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scirrhosity</ent><br/
<hw>Scir*rhos"i*ty</hw> <pr>(sk<icr/r*r<ocr/s"<icr/*t<ycr/)</pr>, <pos>n.</pos> <fld>(Med.)</fld> <def>A morbid induration, as of a gland; state of being scirrhous.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scirrhous</ent><br/
<hw>Scir"rhous</hw> <pr>(sk<icr/r"r<ucr/s)</pr>, <pos>a.</pos> <ety>[NL. <ets>scirrhosus</ets>.]</ety> <fld>(Med.)</fld> <def>Proceeding from scirrhus; of the nature of scirrhus; indurated; knotty; <as>as, <ex>scirrhous</ex> affections; <ex>scirrhous</ex> disease</as>.</def> <altsp>[Written also <asp>skirrhous</asp>.]</altsp><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scirrhus</ent><br/
<hw>Scir"rhus</hw> <pr>(?)</pr>, <pos>n.</pos>; <plu><it>pl.</it> L. <plw>Scirrhi</plw> <pr>(#)</pr>, E. <plw>Scirrhuses</plw> <pr>(#)</pr>.</plu> <ety>[NL., from L. <ets>scirros</ets>, Gr. <?/, <?/, fr. <?/, <?/, hard.]</ety> <fld>(Med.)</fld> <sd>(a)</sd> <def>An indurated organ or part; especially, an indurated gland.</def> <mark>[Obs.]</mark> <sd>(b)</sd> <def>A cancerous tumor which is hard, translucent, of a gray or bluish color, and emits a creaking sound when incised.</def> <altsp>[Sometimes incorrectly written <asp>schirrus</asp>; written also <asp>skirrhus</asp>.]</altsp><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sciscitation</ent><br/
<hw>Scis`ci*ta"tion</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>sciscitatio</ets>, fr. <ets>sciscitari</ets> to inquire, from <ets>sciscere</ets> to seek to know, v. incho. from <ets>scire</ets> to know.]</ety> <def>The act of inquiring; inquiry; demand.</def> <mark>[Obs.]</mark>  <rj><au>Bp. Hall.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scise</ent><br/
<hw>Scise</hw> <pr>(?)</pr>, <pos>v. i.</pos> <ety>[L. <ets>scindere</ets>, <ets>scissum</ets>, to cut, split.]</ety> <def>To cut; to penetrate.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>The wicked steel <qex>scised</qex> deep in his right side.</q> <rj><qau>Fairfax.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scissel</ent><br/
<hw>Scis"sel</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Cf. <er>Scissile</er>.]</ety> <sn>1.</sn> <def>The clippings of metals made in various mechanical operations.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>The slips or plates of metal out of which circular blanks have been cut for the purpose of coinage.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scissible</ent><br/
<hw>Scis"si*ble</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>scindere</ets>, <ets>scissum</ets>, to split.]</ety> <def>Capable of being cut or divided by a sharp instrument.</def> <mark>[R.]</mark>  <rj><au>Bacon.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scissil</ent><br/
<hw>Scis"sil</hw> <pr>(?)</pr>, <pos>n.</pos> <def>See <er>Scissel</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scissile</ent><br/
<hw>Scis"sile</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>scissilis</ets>, fr. <ets>scindere</ets>, <ets>scissum</ets>, to cut, to split: cf. F. <ets>scissile</ets>. See <er>Schism</er>.]</ety> <def>Capable of being cut smoothly; scissible.</def> <mark>[R.]</mark>  <rj><au>Arbuthnot.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scission</ent><br/
<hw>Scis"sion</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>scissio</ets>, fr. <ets>scindere</ets>, <ets>scissum</ets>, to cut, to split: cf. F. <ets>scission</ets>.]</ety> <def>The act of dividing with an instrument having a sharp edge.</def> <rj><au>Wiseman.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scissiparity</ent><br/
<hw>Scis`si*par"i*ty</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>scissus</ets> (p. p. of <ets>scindere</ets> to split) + <ets>parere</ets> to bring forth: cf. F. <ets>scissiparit<eacute/</ets>.]</ety> <fld>(Biol.)</fld> <def>Reproduction by fission.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scissor</ent><br/
<hw>Scis"sor</hw> <pr>(?)</pr>, <pos>v. t.</pos> <def>To cut with scissors or shears; to prepare with the aid of scissors.</def>  <rj><au>Massinger.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scissors</ent><br/
<hw>Scis"sors</hw> <pr>(?)</pr>, <pos>n. pl.</pos> <ety>[OE. <ets>sisoures</ets>, OF. <ets>cisoires</ets> (cf. F. <ets>ciseaux</ets>), probably fr. LL. <ets>cisorium</ets> a cutting instrument, fr. L. <ets>caedere</ets> to cut.  Cf. <er>Chisel</er>, <er>Concise</er>. The modern spelling is due to a mistaken derivation from L. <ets>scissor</ets> one who cleaves or divides, fr. <ets>scindere</ets>, <ets>scissum</ets>, to cut, split.]</ety> <def>A cutting instrument resembling shears, but smaller, consisting of two cutting blades with handles, movable on a pin in the center, by which they are held together. Often called a <altname>pair of scissors</altname>.</def> <altsp>[Formerly written also <asp>cisors</asp>, <asp>cizars</asp>, and <asp>scissars</asp>.]</altsp><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Scissors grinder</b></col> <fld>(Zool.)</fld>, <cd>the European goatsucker.</cd> <mark>[Prov. Eng.]</mark></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scissorsbill</ent><br/
<hw>Scis"sors*bill`</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Zool.)</fld> <def>See <er>Skimmer</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scissorstail</ent><br/
<hw>Scis"sors*tail`</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Zool.)</fld> <def>A tyrant flycatcher (<spn>Milvulus forficatus</spn>) of the Southern United States and Mexico, which has a deeply forked tail.  It is light gray above, white beneath, salmon on the flanks, and fiery red at the base of the crown feathers.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scissors-tailed</ent><br/
<hw>Scis"sors-tailed`</hw> <pr>(?)</pr>, <pos>a.</pos> <fld>(Zool.)</fld> <def>Having the outer feathers much the longest, the others decreasing regularly to the median ones.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scissure</ent><br/
<hw>Scis"sure</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>scissura</ets>, from <ets>scindere</ets>, <ets>scissum</ets>, to cut, split.]</ety> <def>A longitudinal opening in a body, made by cutting; a cleft; a fissure.</def>  <rj><au>Hammond.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scitamineous</ent><br/
<hw>Scit`a*min"e*ous</hw> <pr>(?; 277)</pr>, <pos>a.</pos> <ety>[NL. <ets>scitamineosus</ets>, fr. <ets>Scitamineae</ets>, fr. L. <ets>scitamentum</ets> a delicacy, dainty.]</ety> <fld>(Bot.)</fld> <def>Of or pertaining to a natural order of plants (<ord>Scitamineae</ord>), mostly tropical herbs, including the ginger, Indian shot, banana, and the plants producing turmeric and arrowroot.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sciurine</ent><br/
<hw>Sci"u*rine</hw> <pr>(?; 277)</pr>, <pos>a.</pos> <ety>[Cf. F. <ets>sciurien</ets>. See <er>Sciurus</er>.]</ety> <fld>(Zool.)</fld> <def>Of or pertaining to the Squirrel family.</def> -- <def2><pos>n.</pos> <def>A rodent of the Squirrel family.</def></def2><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sciuroid</ent><br/
<hw>Sci"u*roid</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[<ets>Sciurus</ets> + <ets>-oid</ets>.]</ety> <fld>(Bot.)</fld> <def>Resembling the tail of a squirrel; -- generally said of branches which are close and dense, or of spikes of grass like barley.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sciuromorpha</ent><br/
||<hw>Sci`u*ro*mor"pha</hw> <pr>(?)</pr>, <pos>n. pl.</pos> <ety>[NL., fr. L. <ets>sciurus</ets> squirrel + Gr. <grk>morfh`</grk> form.]</ety> <fld>(Zool.)</fld> <def>A tribe of rodents containing the squirrels and allied animals, such as the gophers, woodchucks, beavers, and others.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sciurus</ent><br/
||<hw>Sci*u"rus</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L., a squirrel, Gr. <?/. See <er>Squirrel</er>.]</ety> <fld>(Zool.)</fld> <def>A genus of rodents comprising the common squirrels.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sclaff</ent><br/
<hw>Sclaff</hw> <pr>(?)</pr>, <pos>v. i.</pos>  <vmorph>[<pos>imp. & p. p.</pos> <conjf>Sclaffed</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Sclaffing</conjf>.]</vmorph> <ety>[Orig. uncert.]</ety> <sn>1.</sn> <def>To scuff or shuffle along.</def> <mark>[Scot.]</mark><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><sn>2.</sn>  <fld>(Golf)</fld> <def>To scrape the ground with the sole of the club, before striking the ball, in making a stroke.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Sclaff</ent><br/
<hw>Sclaff</hw>, <pos>v. t.</pos> <fld>(Golf)</fld> <def>To scrape (the club) on the ground, in a stroke, before hitting the ball; also, to make (a stroke) in that way.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Sclaff</ent><br/
<hw>Sclaff</hw>, <pos>n.</pos>  <mark>[Scot.]</mark> <sn>1.</sn> <def>A slight blow; a slap; a soft fall; also, the accompanying noise.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><sn>2.</sn>  <fld>(Golf)</fld> <def>The stroke made by one who sclaffs.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><sn>3.</sn>  <def>A thin, solid substance, esp. a thin shoe or slipper.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Sclaundre</ent><br/
<hw>Sclaun"dre</hw> <pr>(?)</pr>, <pos>n.</pos> <def>Slander.</def> <mark>[Obs.]</mark>  <rj><au>Chaucer.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sclave</ent><br/
<ent>Sclav</ent><br/
<mhw>{ <hw>Sclav</hw> <pr>(?)</pr>, <hw>Sclave</hw>  }</mhw>, <pos>n.</pos> <def>Same as <er>Slav</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sclavic</ent><br/
<hw>Sclav"ic</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Same as <er>Slavic</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sclavism</ent><br/
<hw>Sclav"ism</hw> <pr>(?)</pr>, <pos>n.</pos> <def>Same as <er>Slavism</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sclavonian</ent><br/
<hw>Scla*vo"nian</hw> <pr>(?)</pr>, <pos>a. & n.</pos> <def>Same as <er>Slavonian</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sclavonic</ent><br/
<hw>Scla*von"ic</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Same as <er>Slavonic</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sclender</ent><br/
<hw>Sclen"der</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Slender.</def> <mark>[Obs.]</mark>  <rj><au>Chaucer.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scleragogy</ent><br/
<hw>Scler"a*go`gy</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Gr. <?/; <?/ hard + <?/ a leading or training.]</ety> <def>Severe discipline.</def> <mark>[Obs.]</mark>  <rj><au>Bp. Hacket.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sclerema</ent><br/
||<hw>Scle*re"ma</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[NL., fr. Gr. <grk>sklhro`s</grk> hard.]</ety> <fld>(Med.)</fld> <def>Induration of the cellular tissue.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Sclerema of adults</b></col>. <cd>See <er>Scleroderma</er>.</cd> -- ||<col><b>Sclerema neonatorum</b></col> <pr>(<?/)</pr> <ety>[NL., of the newborn]</ety>, <cd>an affection characterized by a peculiar hardening and rigidity of the cutaneous and subcutaneous tissues in the newly born. It is usually fatal. Called also <altname>skinbound disease</altname>.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sclerenchyma</ent><br/
<hw>Scle*ren"chy*ma</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[NL., from Gr. <grk>sklhro`s</grk> hard + <ets>-enchyma</ets> as in <ets>parenchyma</ets>.]</ety> <sn>1.</sn> <fld>(Bot.)</fld> <def>Vegetable tissue composed of short cells with thickened or hardened walls, as in nutshells and the gritty parts of a pear. See <er>Sclerotic</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ By recent German writers and their English translators, this term is used for <altname>liber cells</altname>.</note>  <rj><au>Goodale.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Zool.)</fld> <def>The hard calcareous deposit in the tissues of Anthozoa, constituting the stony corals.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sclerenchymatous</ent><br/
<hw>Scler`en*chym"a*tous</hw> <pr>(?)</pr>, <pos>a.</pos> <fld>(Bot. & Zool.)</fld> <def>Pertaining to, or composed of, sclerenchyma.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sclerenchyme</ent><br/
<hw>Scle*ren"chyme</hw> <pr>(?)</pr>, <pos>n.</pos> <def>Sclerenchyma.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scleriasis</ent><br/
||<hw>Scle*ri"a*sis</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[NL., fr. Gr. <?/.]</ety> <fld>(Med.)</fld> <sd>(a)</sd> <def>A morbid induration of the edge of the eyelid.</def> <sd>(b)</sd> <def>Induration of any part, including scleroderma.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sclerite</ent><br/
<hw>Scle"rite</hw> <pr>(skl<emac/"r<imac/t)</pr>, <pos>n.</pos> <fld>(Zool.)</fld> <def>A hard chitinous or calcareous process or corpuscle, especially a spicule of the Alcyonaria.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scleritis</ent><br/
||<hw>Scle*ri"tis</hw> <pr>(skl<esl/*r<imac/"t<icr/s)</pr>, <pos>n.</pos> <ety>[NL.]</ety> <def>See <er>Sclerotitis</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sclerobase</ent><br/
<hw>Scler"o*base</hw> <pr>(? <or/ ?)</pr>, <pos>n.</pos> <ety>[Gr. <grk>sklhro`s</grk> hard + <grk>ba`sis</grk> base.]</ety> <fld>(Zool.)</fld> <def>The calcareous or hornlike coral forming the central stem or axis of most compound alcyonarians; -- called also <altname>foot secretion</altname>. See <xex>Illust.</xex> under <er>Gorgoniacea</er>, and <er>Coenenchyma</er>.</def> -- <wordforms><wf>Scler`o*ba"sic</wf> <pr>(#)</pr>, <pos>a.</pos></wordforms><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scleroderm</ent><br/
<hw>Scler"o*derm</hw> <pr>(? <or/ ?; 277)</pr>, <pos>n.</pos> <ety>[Gr. <grk>sklhro`s</grk> hard + <grk>de`rma</grk> skin: cf. F. <ets>scl<eacute/roderme</ets>.]</ety> <sd>(a)</sd> <fld>(Zool.)</fld> <def>One of a tribe of plectognath fishes (<fam>Sclerodermi</fam>) having the skin covered with hard scales, or plates, as the cowfish and the trunkfish.</def> <sd>(b)</sd> <def>One of the Sclerodermata.</def> <sd>(c)</sd> <def>Hardened, or bony, integument of various animals.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scleroderma</ent><br/
||<hw>Scler`o*der"ma</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[NL.]</ety> <fld>(Med.)</fld> <def>A disease of adults, characterized by a diffuse rigidity and hardness of the skin.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sclerodermata</ent><br/
||<hw>Scler`o*der"ma*ta</hw> <pr>(?)</pr>, <pos>n. pl.</pos> <ety>[NL.]</ety> <fld>(Zool.)</fld> <def>The stony corals; the Madreporaria.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sclerodermous</ent><br/
<ent>Sclerodermic</ent><br/
<mhw>{ <hw>Scler`o*der"mic</hw> <pr>(?)</pr>, <hw>Scler`o*der"mous</hw> <pr>(?)</pr>, }</mhw> <fld>(Zool.)</fld> <sd>(a)</sd> <def>Having the integument, or skin, hard, or covered with hard plates.</def> <sd>(b)</sd> <def>Of or pertaining to the Sclerodermata.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sclerodermite</ent><br/
<hw>Scler`o*der"mite</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Zool.)</fld> <sd>(a)</sd> <def>The hard integument of Crustacea.</def> <sd>(b)</sd> <def>Sclerenchyma.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sclerogen</ent><br/
<hw>Scler"o*gen</hw> <pr>(? <or/ ?)</pr>, <pos>n.</pos> <ety>[Gr. <grk>sklhro`s</grk> hard + <ets>-gen</ets>.]</ety> <fld>(Bot.)</fld> <def>The thickening matter of woody cells; lignin.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sclerogenous</ent><br/
<hw>Scle*rog"e*nous</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[Gr. <grk>sklhro`s</grk> hard + <ets>-genous</ets>.]</ety> <fld>(Anat.)</fld> <def>Making or secreting a hard substance; becoming hard.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scleroid</ent><br/
<hw>Scle"roid</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[Gr. <?/; <grk>sklhro`s</grk> hard + <grk>e'i^dos</grk> form.]</ety> <fld>(Bot.)</fld> <def>Having a hard texture, as nutshells.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scleroma</ent><br/
||<hw>Scle*ro"ma</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[NL., fr. Gr. <grk>sklhro`s</grk> hard + <ets>-oma</ets>.]</ety> <fld>(Med.)</fld> <def>Induration of the tissues. See <er>Sclerema</er>, <er>Scleroderma</er>, and <er>Sclerosis</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sclerometer</ent><br/
<hw>Scle*rom"e*ter</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Gr. <grk>sklhro`s</grk> hard + <ets>-meter</ets>.]</ety> <def>An instrument for determining with accuracy the degree of hardness of a mineral.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sclerosed</ent><br/
<hw>Scle*rosed"</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Affected with sclerosis.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sclerosis</ent><br/
||<hw>Scle*ro"sis</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[NL., fr. Gr. (<?/<?/, fr. <grk>sklhro`s</grk> hard.]</ety> <sn>1.</sn> <fld>(Med.)</fld> <def>Induration; hardening; especially, that form of induration produced in an organ by increase of its interstitial connective tissue.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Bot.)</fld> <def>Hardening of the cell wall by lignification.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Cerebro-spinal sclerosis</b></col> <fld>(Med.)</fld>, <cd>an affection in which patches of hardening, produced by increase of the neuroglia and atrophy of the true nerve tissue, are found scattered throughout the brain and spinal cord. It is associated with complete or partial paralysis, a peculiar jerking tremor of the muscles, headache, and vertigo, and is usually fatal.  Formerly referred to as <altname>multiple sclerosis</altname>, <altname>disseminated sclerosis</altname>, <it>or</it> <altname>insular sclerosis</altname>, but now usually called only <altname>multiple sclerosis</altname>, or <altname>MS</altname>.</cd></cs><br/
[<source>1913 Webster</source> <source>+PJC</source>]</p>

<p><ent>Scleroskeleton</ent><br/
<hw>Scle`ro*skel"e*ton</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Gr. <grk>sklhro`s</grk> hard + E. <ets>skeleton</ets>.]</ety> <fld>(Anat.)</fld> <def>That part of the skeleton which is developed in tendons, ligaments, and aponeuroses.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sclerotal</ent><br/
<hw>Scle*ro"tal</hw> <pr>(?)</pr>, <pos>a.</pos> <fld>(Anat.)</fld> <def>Sclerotic.</def> -- <def2><pos>n.</pos> <def>The optic capsule; the sclerotic coat of the eye.</def></def2>  <rj><au>Owen.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sclerotic</ent><br/
<hw>Scle*rot"ic</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[Gr. <grk>sklhro`s</grk> hard: cf. F. <ets>scl<eacute/rotique</ets>.]</ety> <sn>1.</sn> <def>Hard; firm; indurated; -- applied especially in anatomy to the firm outer coat of the eyeball, which is often cartilaginous and sometimes bony.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Anat.)</fld> <def>Of or pertaining to the sclerotic coat of the eye; sclerotical.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <fld>(Med.)</fld> <def>Affected with sclerosis; sclerosed.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Sclerotic parenchyma</b></col> <fld>(Bot.)</fld>, <cd>sclerenchyma. By some writers a distinction is made, <xex>sclerotic parenchyma</xex> being applied to tissue composed of cells with the walls hardened but not thickened, and <xex>sclerenchyma</xex> to tissue composed of cells with the walls both hardened and thickened.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sclerotic</ent><br/
<hw>Scle*rot"ic</hw>, <pos>n.</pos> <ety>[Cf. F. <ets>scl<eacute/rotique</ets>.]</ety> <fld>(Anat.)</fld> <def>The sclerotic coat of the eye. See <xex>Illust.</xex> of <er>Eye</er> <sd>(d)</sd>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sclerotic</ent><br/
<hw>Scle*rot"ic</hw>, <pos>a.</pos> <fld>(Chem.)</fld> <def>Pertaining to, or designating, an acid obtained from ergot or the sclerotium of a fungus growing on rye.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sclerotical</ent><br/
<hw>Scle*rot"ic*al</hw> <pr>(?)</pr>, <pos>a.</pos> <fld>(Anat.)</fld> <def>Sclerotic.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sclerotitis</ent><br/
||<hw>Scler`o*ti"tis</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[NL. See <er>Sclerotic</er>, and <er>-itis</er>.]</ety> <fld>(Med.)</fld> <def>Inflammation of the sclerotic coat.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sclerotium</ent><br/
||<hw>Scle*ro"ti*um</hw> <pr>(?)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Sclerotia</plw> <pr>(#)</pr>.</plu> <ety>[NL., fr. Gr. <grk>sklhro`s</grk> hard.]</ety> <sn>1.</sn> <fld>(Bot.)</fld> <def>A hardened body formed by certain fungi, as by the <spn>Claviceps purpurea</spn>, which produces ergot.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Zool.)</fld> <def>The mature or resting stage of a plasmodium.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sclerotome</ent><br/
<hw>Scler"o*tome</hw> <pr>(skl<ecr/r"<osl/*t<omac/m <it>or</it> skl<emac/r"<osl/*t<omac/m)</pr>, <pos>n.</pos> <ety>[Gr. <grk>sklhro`s</grk> hard + <grk>te`mnein</grk> to cut.]</ety> <fld>(Zool.)</fld> <def>One of the bony, cartilaginous, or membranous partitions which separate the myotomes.</def> -- <wordforms><wf>Scler`o*tom"ic</wf> <pr>(#)</pr>, <pos>a.</pos></wordforms><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sclerous</ent><br/
<hw>Scle"rous</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[Gr. <?/.]</ety> <fld>(Anat.)</fld> <def>Hard; indurated; sclerotic.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scoat</ent><br/
<hw>Scoat</hw> <pr>(?)</pr>, <pos>v. t.</pos> <def>To prop; to scotch.</def> <mark>[Prov. Eng.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scobby</ent><br/
<hw>Scob"by</hw> <pr>(?)</pr>, <pos>n.</pos> <def>The chaffinch.</def> <mark>[Prov. Eng.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scobiform</ent><br/
<hw>Scob"i*form</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>scobs</ets>, or <ets>scobis</ets>, sawdust, scrapings + <ets>-form</ets>: cf. F. <ets>scobiforme</ets>.]</ety> <def>Having the form of, or resembling, sawdust or raspings.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scobs</ent><br/
<hw>Scobs</hw>, <pos>n. sing. & pl.</pos> <ety>[L. <ets>scobs</ets>, or <ets>scobis</ets>, fr. <ets>scabere</ets> to scrape.]</ety> <sn>1.</sn> <def>Raspings of ivory, hartshorn, metals, or other hard substance.</def>  <rj><au>Chambers.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>The dross of metals.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scoff</ent><br/
<hw>Scoff</hw> <pr>(?; 115)</pr>, <pos>n.</pos> <ety>[OE. <ets>scof</ets>; akin to OFries. <ets>schof</ets>, OHG. <ets>scoph</ets>, Icel. <ets>skaup</ets>, and perh. to E. <ets>shove</ets>.]</ety> <sn>1.</sn> <def>Derision; ridicule; mockery; derisive or mocking expression of scorn, contempt, or reproach.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>With <qex>scoffs</qex>, and scorns, and contumelious taunts.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>An object of scorn, mockery, or derision.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The <qex>scoff</qex> of withered age and beardless youth.</q> <rj><qau>Cowper.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scoff</ent><br/
<hw>Scoff</hw>, <pos>v. i.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Scoffed</conjf> <pr>(?; 115)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Scoffing</conjf>.]</vmorph> <ety>[Cf. Dan. <ets>skuffe</ets> to deceive, delude, Icel. <ets>skopa</ets> to scoff, OD. <ets>schoppen</ets>. See <er>Scoff</er>, <pos>n.</pos>]</ety> <def>To show insolent ridicule or mockery; to manifest contempt by derisive acts or language; -- often with <xex>at</xex>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Truth from his lips prevailed with double sway,<br/
And fools who came to <qex>scoff</qex>, remained to pray.</q> <rj><qau>Goldsmith.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>God's better gift they <qex>scoff</qex> at and refuse.</q> <rj><qau>Cowper.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><syn><b>Syn.</b> -- To sneer; mock; gibe; jeer. See <er>Sneer</er>.</syn><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scoff</ent><br/
<hw>Scoff</hw>, <pos>v. t.</pos> <def>To treat or address with derision; to assail scornfully; to mock at.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>To <qex>scoff</qex> religion is ridiculously proud and immodest.</q> <rj><qau>Glanvill.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scoffer</ent><br/
<hw>Scoff"er</hw> <pr>(?)</pr>, <pos>n.</pos> <def>One who scoffs.</def>  <rj><au>2 Pet. iii. 3.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scoffery</ent><br/
<hw>Scoff"er*y</hw> <pr>(?)</pr>, <pos>n.</pos> <def>The act of scoffing; scoffing conduct; mockery.</def>  <rj><au>Holinshed.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scoffingly</ent><br/
<hw>Scoff"ing*ly</hw>, <pos>adv.</pos> <def>In a scoffing manner.</def> <rj><au>Broome.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scoke</ent><br/
<hw>Scoke</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Bot.)</fld> <def>Poke (<spn>Phytolacca decandra</spn>).</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scolay</ent><br/
<hw>Sco*lay"</hw> <pr>(?)</pr>, <pos>v. i.</pos> <def>See <er>Scoley</er>.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scold</ent><br/
<hw>Scold</hw> <pr>(?)</pr>, <pos>v. i.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Scolded</conjf>; <pos>p. pr. & vb. n.</pos> <conjf>Scolding</conjf>.]</vmorph> <ety>[Akin to D. <ets>schelden</ets>, G. <ets>schelten</ets>, OHG. <ets>sceltan</ets>, Dan. <ets>skielde</ets>.]</ety> <def>To find fault or rail with rude clamor; to brawl; to utter harsh, rude, boisterous rebuke; to chide sharply or coarsely; -- often with <xex>at</xex>; <as>as, to <ex>scold</ex> at a servant</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Pardon me, lords, 't is the first time ever<br/
I was forced to <qex>scold</qex>.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scold</ent><br/
<hw>Scold</hw>, <pos>v. t.</pos> <def>To chide with rudeness and clamor; to rate; also, to rebuke or reprove with severity.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scold</ent><br/
<hw>Scold</hw>, <pos>n.</pos> <sn>1.</sn> <def>One who scolds, or makes a practice of scolding; esp., a rude, clamorous woman; a shrew.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>She is an irksome, brawling <qex>scold</qex>.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A scolding; a brawl.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scolder</ent><br/
<hw>Scold"er</hw> <pr>(?)</pr>, <pos>n.</pos> <sn>1.</sn> <def>One who scolds.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Zool.)</fld> <sd>(a)</sd> <def>The oyster catcher; -- so called from its shrill cries.</def> <sd>(b)</sd> <def>The old squaw.</def> <mark>[Local U.S.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scolding</ent><br/
<hw>Scold"ing</hw>, <def><pos>a. & n.</pos> from <er>Scold</er>, <pos>v.</pos></def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Scolding bridle</b></col>, <cd>an iron frame. See <er>Brank</er>, <pos>n.</pos>, 2.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scoldingly</ent><br/
<hw>Scold"ing*ly</hw>, <pos>adv.</pos> <def>In a scolding manner.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scole</ent><br/
<hw>Scole</hw> <pr>(?)</pr>, <pos>n.</pos> <def>School.</def> <mark>[Obs.]</mark> <rj><au>Chaucer.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scolecida</ent><br/
||<hw>Sco*le"ci*da</hw> <pr>(? <or/ ?)</pr>, <pos>n. pl.</pos> <ety>[NL. See <er>Scolex</er>.]</ety> <fld>(Zool.)</fld> <def>Same as <er>Helminthes</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scolecite</ent><br/
<hw>Scol"e*cite</hw> <pr>(? <or/ ?; 277)</pr>, <pos>n.</pos> <ety>[Gr. <grk>skw`lhx</grk>, <grk>-hkos</grk>, a worm, earthworm.]</ety> <fld>(Min.)</fld> <def>A zeolitic mineral occuring in delicate radiating groups of white crystals. It is a hydrous silicate of alumina and lime. Called also <altname>lime mesotype</altname>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scolecomorpha</ent><br/
||<hw>Sco*le`co*mor"pha</hw> <pr>(<?/)</pr>, <pos>n. pl.</pos> <ety>[NL. See <er>Scolex</er>, <er>-morphous</er>.]</ety> <fld>(Zool.)</fld> <def>Same as <er>Scolecida</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scolex</ent><br/
||<hw>Sco"lex</hw> <pr>(?)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Scoleces</plw> <pr>(#)</pr>.</plu> <ety>[NL., from Gr. <grk>skw`lhx</grk> worm, grub.]</ety> <fld>(Zool.)</fld> <sd>(a)</sd> <def>The embryo produced directly from the egg in a metagenetic series, especially the larva of a tapeworm or other parasitic worm. See <xex>Illust.</xex> of <er>Echinococcus</er>.</def> <sd>(b)</sd> <def>One of the Scolecida.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scoley</ent><br/
<hw>Sco*ley"</hw> <pr>(?)</pr>, <pos>v. i.</pos> <ety>[Cf. OF. <ets>escoler</ets> to teach. See <er>School</er>.]</ety> <def>To go to school; to study.</def> <mark>[Obs.]</mark>  <rj><au>Chaucer.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scoliosis</ent><br/
||<hw>Sco`li*o"sis</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[NL., fr. Gr. <grk>skolio`s</grk> crooked.]</ety> <fld>(Med.)</fld> <def>A lateral curvature of the spine.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scolithus</ent><br/
||<hw>Scol"i*thus</hw> <pr>(? <or/ ?)</pr>, <pos>n.</pos> <ety>[NL., fr. Gr. <grk>skw`lhx</grk> a worm + <grk>li`qos</grk> a stone.]</ety> <fld>(Paleon.)</fld> <def>A tubular structure found in Potsdam sandstone, and believed to be the fossil burrow of a marine worm.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scollop</ent><br/
<hw>Scol"lop</hw> <pr>(?)</pr>, <pos>n. & v.</pos> <def>See <er>Scallop</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scolopacine</ent><br/
<hw>Scol`o*pa"cine</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>scolopax</ets> a snipe, Gr. <?/.]</ety> <fld>(Zool.)</fld> <def>Of or pertaining to the <fam>Scolopacidae</fam>, or Snipe family.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scolopendra</ent><br/
||<hw>Scol`o*pen"dra</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L., a kind of multiped, fr. Gr. <?/.]</ety> <sn>1.</sn> <fld>(Zool.)</fld> <def>A genus of venomous myriapods including the centipeds. See <er>Centiped</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A sea fish.</def> <mark>[R.]</mark> <rj><au>Spenser.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scolopendrine</ent><br/
<hw>Scol`o*pen"drine</hw> <pr>(?)</pr>, <pos>a.</pos> <fld>(Zool.)</fld> <def>Like or pertaining to the Scolopendra.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scolytid</ent><br/
<hw>Scol"y*tid</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Gr. <grk>skoly`ptein</grk> to cut short.]</ety> <fld>(Zool.)</fld> <def>Any one of numerous species of small bark-boring beetles of the genus <gen>Scolytus</gen> and allied genera. Also used adjectively.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scomber</ent><br/
||<hw>Scom"ber</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L., a mackerel, Gr. <grk>sko`mbros</grk>.]</ety> <fld>(Zool.)</fld> <def>A genus of acanthopterygious fishes which includes the common mackerel.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scomberoid</ent><br/
<hw>Scom"ber*oid</hw> <pr>(?)</pr>, <pos>a. & n.</pos> <ety>[Cf. F. <ets>scomb<eacute/ro<ium/de</ets>.]</ety> <fld>(Zool.)</fld> <def>Same as <er>Scombroid</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><-- p. 1289 pr=PCP --></p>

<p><ent>Scombriformes</ent><br/
||<hw>Scom`bri*for"mes</hw> <pr>(sk<ocr/m`br<icr/*f<ocir/r"m<emac/z)</pr>, <pos>n. pl.</pos> <ety>[NL.]</ety> <fld>(Zool.)</fld> <def>A division of fishes including the mackerels, tunnies, and allied fishes.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scombroid</ent><br/
<hw>Scom"broid</hw> <pr>(sk<ocr/m"broid)</pr>, <pos>a.</pos> <ety>[<ets>Scomber</ets> + <ets>-oid</ets>.]</ety> <fld>(Zool.)</fld> <def>Like or pertaining to the Mackerel family.</def> -- <def2><pos>n.</pos> <def>Any fish of the family <fam>Scombridae</fam>, of which the mackerel (<gen>Scomber</gen>) is the type, and including the tuna (<gen>Thunnus</gen> and related genera).</def></def2><br/
[<source>1913 Webster</source> <source>+PJC</source>]</p>

<p><ent>Scomfish</ent><br/
<hw>Scom"fish</hw> <pr>(sk<ocr/m"f<icr/sh <it>or</it> sk<ucr/m"-)</pr>, <pos>v. t. & i.</pos> <def>To suffocate or stifle; to smother.</def> <mark>[Scot. & Prov. Eng.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scomfit</ent><br/
<hw>Scom"fit</hw> <pr>(sk<ucr/m"f<icr/y)</pr>, <pos>n. & v.</pos> <def>Discomfit.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scomm</ent><br/
<hw>Scomm</hw> <pr>(sk<ocr/m)</pr>, <pos>n.</pos> <ety>[L. <ets>scomma</ets> a taunt, jeer, scoff, Gr. <?/, fr. <?/ to mock, scoff at.]</ety> <sn>1.</sn> <def>A buffoon.</def> <mark>[Obs.]</mark> <rj><au>L'Estrange.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A flout; a jeer; a gibe; a taunt.</def> <mark>[Obs.]</mark> <rj><au>Fotherby.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sconce</ent><br/
<hw>Sconce</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[D. <ets>schans</ets>, OD. <ets>schantse</ets>, perhaps from OF. <ets>esconse</ets> a hiding place, akin to <ets>esconser</ets> to hide, L. <ets>absconsus</ets>, <pos>p. p.</pos> of <ets>abscondere</ets>. See <er>Abscond</er>, and cf. <er>Ensconce</er>, <er>Sconce</er> a candlestick.]</ety> <sn>1.</sn> <def>A fortification, or work for defense; a fort.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>No <qex>sconce</qex> or fortress of his raising was ever known either to have been forced, or yielded up, or quitted.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A hut for protection and shelter; a stall.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>One that . . . must raise a <qex>sconce</qex> by the highway and sell switches.</q> <rj><qau>Beau. & Fl.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>A piece of armor for the head; headpiece; helmet.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>I must get a <qex>sconce</qex> for my head.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>Fig.: The head; the skull; also, brains; sense; discretion.</def> <mark>[Colloq.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>To knock him about the <qex>sconce</qex> with a dirty shovel.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <def>A poll tax; a mulct or fine.</def> <rj><au>Johnson.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>6.</sn> <ety>[OF. <ets>esconse</ets> a dark lantern, properly, a hiding place. See Etymol. above.]</ety> <def>A protection for a light; a lantern or cased support for a candle; hence, a fixed hanging or projecting candlestick.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Tapers put into lanterns or <qex>sconces</qex> of several-colored, oiled paper, that the wind might not annoy them.</q> <rj><qau>Evelyn.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Golden <qex>sconces</qex> hang not on the walls.</q> <rj><qau>Dryden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>7.</sn> <def>Hence, the circular tube, with a brim, in a candlestick, into which the candle is inserted.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>8.</sn> <fld>(Arch.)</fld> <def>A squinch.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>9.</sn> <def>A fragment of a floe of ice.</def> <rj><au>Kane.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>10.</sn> <ety>[Perhaps a different word.]</ety> <def>A fixed seat or shelf.</def> <mark>[Prov. Eng.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sconce</ent><br/
<hw>Sconce</hw>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Sconced</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Sconcing</conjf>.]</vmorph> <sn>1.</sn> <def>To shut up in a sconce; to imprison; to insconce.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>Immure him, <qex>sconce</qex> him, barricade him in 't.</q> <rj><qau>Marston.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To mulct; to fine.</def> <mark>[Obs.]</mark> <rj><au>Milton.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sconcheon</ent><br/
<hw>Scon"cheon</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Arch.)</fld> <def>A squinch.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scone</ent><br/
<hw>Scone</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A cake, thinner than a bannock, made of wheat or barley or oat meal.</def> <altsp>[Written variously, <asp>scon</asp>, <asp>skone</asp>, <asp>skon</asp>, etc.]</altsp> <mark>[Scot.]</mark> <rj><au>Burns.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scoop</ent><br/
<hw>Scoop</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[OE. <ets>scope</ets>, of Scand. origin; cf. Sw. <ets>skopa</ets>, akin to D. <ets>schop</ets> a shovel, G. <ets>sch<uum/ppe</ets>, and also to E. <ets>shove</ets>. See <er>Shovel</er>.]</ety> <sn>1.</sn> <def>A large ladle; a vessel with a long handle, used for dipping liquids; a utensil for bailing boats.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A deep shovel, or any similar implement for digging out and dipping or shoveling up anything; <as>as, a flour <ex>scoop</ex>; the <ex>scoop</ex> of a dredging machine</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <fld>(Surg.)</fld> <def>A spoon-shaped instrument, used in extracting certain substances or foreign bodies.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>A place hollowed out; a basinlike cavity; a hollow.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Some had lain in the <qex>scoop</qex> of the rock.</q> <rj><qau>J. R. Drake.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <def>A sweep; a stroke; a swoop.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>6.</sn> <def>The act of scooping, or taking with a scoop or ladle; a motion with a scoop, as in dipping or shoveling.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>7.</sn> <def>a quantity sufficient to fill a scoop; -- used especially for ice cream, dispensed with an ice cream scoop; <as>as, an ice cream cone with two <ex>scoops</ex></as>.</def><br/
[<source>PJC</source>]</p>

<p><sn>8.</sn> <def>an act of reporting (news, research results) before a rival; also called a <altname>beat</altname>.</def> <mark>[Newspaper or laboratory cant]</mark><br/
[<source>Webster 1913 Suppl.</source> <source>+PJC</source>]</p>

<p><sn>9.</sn> <def>news or information; <as>as, what's the <ex>scoop</ex> on John's divorce?</as>.</def> <mark>[informal]</mark><br/
[<source>PJC</source>]</p>

<p><cs><col><b>Scoop net</b></col>, <cd>a kind of hand net, used in fishing; also, a net for sweeping the bottom of a river.</cd> -- <col><b>Scoop wheel</b></col>, <cd>a wheel for raising water, having scoops or buckets attached to its circumference; a tympanum.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scoop</ent><br/
<hw>Scoop</hw>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Scooped</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Scooping</conjf>.]</vmorph> <ety>[OE. <ets>scopen</ets>. See <er>Scoop</er>, <pos>n.</pos>]</ety> <sn>1.</sn> <def>To take out or up with, a scoop; to lade out.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>He <qex>scooped</qex> the water from the crystal flood.</q> <rj><qau>Dryden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To empty by lading; <as>as, to <ex>scoop</ex> a well dry</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>To make hollow, as a scoop or dish; to excavate; to dig out; to form by digging or excavation.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Those carbuncles the Indians will <qex>scoop</qex>, so as to hold above a pint.</q> <rj><qau>Arbuthnot.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scoop</ent><br/
<hw>Scoop</hw>, <pos>v. t.</pos> <def>to report a story first, before (a rival); to get a scoop, or a beat, on (a rival); -- used commonly in the passive; <as>as, we were <ex>scooped</ex></as>.  Also used in certain situations in scientific research, when one scientist or team of scientists reports their results before another who is working on the same problem.</def><br/
[<source>Webster 1913 Suppl.</source> <source>+PJC</source>]</p>

<p><ent>Scooper</ent><br/
<hw>Scoop"er</hw> <pr>(?)</pr>, <pos>n.</pos> <sn>1.</sn> <def>One who, or that which, scoops.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Zool.)</fld> <def>The avocet; -- so called because it scoops up the mud to obtain food.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scoot</ent><br/
<hw>Scoot</hw> <pr>(?)</pr>, <pos>v. i.</pos> <def>To walk fast; to go quickly; to run hastily away.</def> <mark>[Colloq. & Humorous, U. S.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scoparin</ent><br/
<hw>Sco"pa*rin</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Chem.)</fld> <def>A yellow gelatinous or crystalline substance found in broom (<spn>Cytisus scoparius</spn>) accompanying sparteine.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scopate</ent><br/
<hw>Sco"pate</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>scopae</ets>, <ets>scopa</ets>, a broom.]</ety> <fld>(Zool.)</fld> <def>Having the surface closely covered with hairs, like a brush.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>-scope</ent><br/
<hw>-scope</hw> <pr>(<?/)</pr>. <ety>[Gr. <grk>skopo`s</grk> a watcher, spy. See <er>Scope</er>.]</ety> <def>A combining form usually signifying <xex>an instrument for viewing</xex> (with the eye) or <xex>observing</xex> (in any way); <as>as in micro<ex>scope</ex>, tele<ex>scope</ex>, alto<ex>scope</ex>, anemo<ex>scope</ex></as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scope</ent><br/
<hw>Scope</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[It. <ets>scopo</ets>, L. <ets>scopos</ets> a mark, aim, Gr. <grk>skopo`s</grk>, a watcher, mark, aim; akin to <?/, <?/ to view, and perh. to E. <ets>spy</ets>. Cf. <er>Skeptic</er>, <er>Bishop</er>.]</ety> <sn>1.</sn> <def>That at which one aims; the thing or end to which the mind directs its view; that which is purposed to be reached or accomplished; hence, ultimate design, aim, or purpose; intention; drift; object.</def> <ldquo/Shooting wide, do miss the marked <xex>scope</xex>.<rdquo/ <rj><au>Spenser.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Your <qex>scope</qex> is as mine own,<br/
So to enforce or qualify the laws<br/
As to your soul seems good.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>The <qex>scope</qex> of all their pleading against man's authority, is to overthrow such laws and constitutions in the church.</q> <rj><qau>Hooker.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Room or opportunity for free outlook or aim; space for action; amplitude of opportunity; free course or vent; liberty; range of view, intent, or action.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Give him line and <qex>scope</qex>.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>In the fate and fortunes of the human race, <qex>scope</qex> is given to the operation of laws which man must always fail to discern the reasons of.</q> <rj><qau>I. Taylor.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Excuse me if I have given too much <qex>scope</qex> to the reflections which have arisen in my mind.</q> <rj><qau>Burke.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>An intellectual cultivation of no moderate depth or <qex>scope</qex>.</q> <rj><qau>Hawthorne.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>Extended area.</def> <mark>[Obs.]</mark> <ldquo/The <xex>scopes</xex> of land granted to the first adventurers.<rdquo/ <rj><au>Sir J. Davies.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>Length; extent; sweep; <as>as, <ex>scope</ex> of cable</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><-- Scope, <pos>v. t.</pos> <def>To look at for the purpose of evaluation; usually with <ptcl>out</ptcl>; <as>as, to <ex>scope out</ex> the area as a camping site</as>.</def> --><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scopeline</ent><br/
<hw>Sco"pe*line</hw> <pr>(?)</pr>, <pos>a.</pos> <fld>(Zool.)</fld> <def>Scopeloid.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scopeloid</ent><br/
<hw>Sco"pe*loid</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[NL. <ets>Scopelus</ets>, typical genus (fr. Gr. <?/ a headland) + <ets>-oid</ets>.]</ety> <fld>(Zool.)</fld> <def>Like or pertaining to fishes of the genus <gen>Scopelus</gen>, or family <fam>Scopelodae</fam>, which includes many small oceanic fishes, most of which are phosphorescent.</def> -- <def2><pos>n.</pos> <fld>(Zool.)</fld> <def>Any fish of the family <fam>Scopelidae</fam>.</def></def2><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scopiferous</ent><br/
<hw>Sco*pif"er*ous</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>scopae</ets>, <ets>scopa</ets> + <ets>-ferous</ets>.]</ety> <fld>(Zool.)</fld> <def>Bearing a tuft of brushlike hairs.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scopiform</ent><br/
<hw>Sco"pi*form</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>scopae</ets>, <ets>scopa</ets>, a broom + <ets>-form</ets>.]</ety> <def>Having the form of a broom or besom.</def> <ldquo/Zeolite, stelliform or <xex>scopiform</xex>.<rdquo/ <rj><au>Kirwan.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scopiped</ent><br/
<hw>Sco"pi*ped</hw> <pr>(?; 277)</pr>, <pos>n.</pos> <ety>[L. <ets>scopae</ets>, <ets>scopa</ets>, a broom + <ets>pes</ets>, <ets>pedis</ets>, a foot.]</ety> <fld>(Zool.)</fld> <def>Same as <er>Scopuliped</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scoppet</ent><br/
<hw>Scop"pet</hw> <pr>(?)</pr>, <pos>v. t.</pos> <ety>[From <er>Scoop</er>, <pos>v. t.</pos>]</ety> <def>To lade or dip out.</def> <mark>[Obs.]</mark> <rj><au>Bp. Hall.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scops owl</ent><br/
<hw>Scops" owl`</hw> <pr>(?)</pr>. <ety>[NL. <ets>scops</ets>, fr. Gr. <?/ the little horned owl.]</ety> <fld>(Zool.)</fld> <def>Any one of numerous species of small owls of the genus <gen>Scops</gen> having ear tufts like those of the horned owls, especially the European scops owl (<spn>Scops giu</spn>), and the American screech owl (<spn>Scops asio</spn>).</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scoptical</ent><br/
<ent>Scoptic</ent><br/
<mhw>{ <hw>Scop"tic</hw> <pr>(?)</pr>, <hw>Scop"tic*al</hw> <pr>(?)</pr>, }</mhw> <pos>a.</pos> <ety>[Gr. <grk>skwptiko`s</grk>, from <grk>skw`ptein</grk> to mock, to scoff at.]</ety> <def>Jesting; jeering; scoffing.</def> <mark>[Obs.]</mark> <rj><au>South.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p>-- <wordforms><wf>Scop"tic*al*ly</wf>, <pos>adv.</pos> <mark>[Obs.]</mark></wordforms><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scopula</ent><br/
||<hw>Scop"u*la</hw> <pr>(?)</pr>, <pos>n.</pos>; <plu><it>pl.</it> E. <plw>Scopulas</plw> <pr>(#)</pr>, L. <plw>Scopulae</plw> <pr>(#)</pr>.</plu> <ety>[L. <ets>scopulae</ets>, pl. a little broom.]</ety> <fld>(Zool.)</fld> <sd>(a)</sd> <def>A peculiar brushlike organ found on the foot of spiders and used in the construction of the web.</def> <sd>(b)</sd> <def>A special tuft of hairs on the leg of a bee.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scopuliped</ent><br/
<hw>Scop"u*li*ped</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>scopulae</ets>, pl., a little broom (fr. <ets>scopae</ets> a broom) + <ets>pes</ets>, <ets>pedis</ets>, foot.]</ety> <fld>(Zool.)</fld> <def>Any species of bee which has on the hind legs a brush of hairs used for collecting pollen, as the hive bees and bumblebees.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scopulous</ent><br/
<hw>Scop"u*lous</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>scopulosus</ets>, fr. <ets>scopulus</ets> a rock, Gr. <?/.]</ety> <def>Full of rocks; rocky.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scorbute</ent><br/
<hw>Scor"bute</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[LL. <ets>scorbutus</ets>: cf. F. <ets>scorbut</ets>. See <er>Scurvy</er>, <pos>n.</pos>]</ety> <def>Scurvy.</def> <mark>[Obs.]</mark> <rj><au>Purchas.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scorbutical</ent><br/
<ent>Scorbutic</ent><br/
<mhw>{ <hw>Scor*bu"tic</hw> <pr>(?)</pr>, <hw>Scor*bu"tic*al</hw> <pr>(?)</pr>, }</mhw> <pos>a.</pos> <ety>[Cf. F. <ets>scorbutique</ets>.]</ety> <fld>(Med.)</fld> <def>Of or pertaining to scurvy; of the nature of, or resembling, scurvy; diseased with scurvy; <as>as, a <ex>scorbutic</ex> person; <ex>scorbutic</ex> complaints or symptoms</as>.</def> -- <wordforms><wf>Scor*bu"tic*al*ly</wf>, <pos>adv.</pos></wordforms><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scorbutus</ent><br/
||<hw>Scor*bu"tus</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[LL. See <er>Scorbute</er>.]</ety> <fld>(Med.)</fld> <def>Scurvy.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scorce</ent><br/
<hw>Scorce</hw> <pr>(?)</pr>, <pos>n.</pos> <def>Barter.</def> <mark>[Obs.]</mark> <see>See <er>Scorse</er>.</see><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scorch</ent><br/
<hw>Scorch</hw> <pr>(sk<ocir/rch)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Scorched</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Scorching</conjf>.]</vmorph> <ety>[OE. <ets>scorchen</ets>, probably akin to <ets>scorcnen</ets>; cf. Norw. <ets>skrokken</ets> shrunk up, <ets>skrekka</ets>, <ets>skr<oum/kka</ets>, to shrink, to become wrinkled up, dial. Sw. <ets>skr<aring/kkla</ets> to wrinkle (see <er>Shrug</er>); but perhaps influenced by OF. <ets>escorchier</ets> to strip the bark from, to flay, to skin, F. <ets><eacute/corcher</ets>, LL. <ets>excorticare</ets>; L. <ets>ex</ets> from + <ets>cortex</ets>, <ets>-icis</ets>, bark (cf. <er>Cork</er>); because the skin falls off when scorched.]</ety> <sn>1.</sn> <def>To burn superficially; to parch, or shrivel, the surface of, by heat; to subject to so much heat as changes color and texture without consuming; <as>as, to <ex>scorch</ex> linen</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Summer drouth or sing<egrave/d air<br/
Never <qex>scorch</qex> thy tresses fair.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To affect painfully with heat, or as with heat; to dry up with heat; to affect as by heat.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Lashed by mad rage, and <qex>scorched</qex> by brutal fires.</q> <rj><qau>Prior.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>To burn; to destroy by, or as by, fire.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Power was given unto him to <qex>scorch</qex> men with fire.</q> <rj><qau>Rev. xvi. 8.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>The fire that <qex>scorches</qex> me to death.</q> <rj><qau>Dryden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scorch</ent><br/
<hw>Scorch</hw>, <pos>v. i.</pos> <sn>1.</sn> <def>To be burnt on the surface; to be parched; to be dried up.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Scatter a little mungy straw or fern amongst your seedlings, to prevent the roots from <qex>scorching</qex>.</q> <rj><qau>Mortimer.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To burn or be burnt.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>He laid his long forefinger on the scarlet letter, which forthwith seemed to <qex>scorch</qex> into Hester's breast, as if it had been red hot.</q> <rj><qau>Hawthorne.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>To ride or drive at great, usually at excessive, speed; -- applied chiefly to automobilists and bicyclists. <mark>[Colloq.]</mark></def> -- <wordforms><wf>Scorch"er</wf>, <pos>n.</pos> <mark>[Colloq.]</mark></wordforms></p>

<p><ent>scorcher</ent><br/
<hw>scorch"er</hw> <def>a very hot day.</def> <mark>[Informal]</mark><br/
[<source>PJC</source>]</p>

<p><ent>Scorching</ent><br/
<hw>Scorch"ing</hw>, <pos>a.</pos> <sn>1.</sn> <def>Burning; parching or shriveling with heat.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>sufficiently hot to cause scorching.</def><br/
[<source>PJC</source>]</p>

<p> -- <wordforms><wf>Scorch"ing*ly</wf>, <pos>adv.</pos> -- <wf>Scorch"ing*ness</wf>, <pos>n.</pos></wordforms><br/
[<source>1913 Webster</source>]</p>

<p><ent>Score</ent><br/
<hw>Score</hw> <pr>(sk<omac/r)</pr>, <pos>n.</pos> <ety>[AS. <ets>scor</ets> twenty, fr. <ets>sceran</ets>, <ets>scieran</ets>, to shear, cut, divide; or rather the kindred Icel. <ets>skor</ets> incision, twenty, akin to Dan. <ets>skure</ets> a notch, Sw. <ets>sk<aring/ra</ets>. See <er>Shear</er>.]</ety> <sn>1.</sn> <def>A notch or incision; especially, one that is made as a tally mark; hence, a mark, or line, made for the purpose of account.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Whereas, before, our forefathers had no other books but the <qex>score</qex> and the tally, thou hast caused printing to be used.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>An account or reckoning; account of dues; bill; hence, indebtedness.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>He parted well, and paid his <qex>score</qex>.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>Account; reason; motive; sake; behalf.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>But left the trade, as many more<br/
Have lately done on the same <qex>score</qex>.</q> <rj><qau>Hudibras.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>You act your kindness in Cydaria's <qex>score</qex>.</q> <rj><qau>Dryden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>The number twenty, as being marked off by a special score or tally; hence, in <pluf>pl.</pluf>, a large number.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Amongst three or four <qex>score</qex> hogsheads.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>At length the queen took upon herself to grant patents of monopoly by <qex>scores</qex>.</q> <rj><qau>Macaulay.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <def>A distance of twenty yards; -- a term used in ancient archery and gunnery.</def> <rj><au>Halliwell.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>6.</sn> <def>A weight of twenty pounds.</def> <mark>[Prov. Eng.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><sn>7.</sn> <def>The number of points gained by the contestants, or either of them, in any game, as in cards or cricket.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>8.</sn> <def>A line drawn; a groove or furrow.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>9.</sn> <fld>(Mus.)</fld> <def>The original and entire draught, or its transcript, of a composition, with the parts for all the different instruments or voices written on staves one above another, so that they can be read at a glance; -- so called from the bar, which, in its early use, was drawn through all the parts.</def> <rj><au>Moore (Encyc. of Music).</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>10.</sn> <def>the grade received on an examination, such as those given in school or as a qualifying examination for a job or admission to school; -- it may be expressed as a percentage of answers which are correct, or as a number or letter; <as>as, a <ex>score</ex> of 98 in a civil service exam</as>.</def><br/
[<source>PJC</source>]</p>

<p><cs><col><b>In score</b></col> <fld>(Mus.)</fld>, <cd>having all the parts arranged and placed in juxtaposition.</cd> <au>Smart.</au> -- <col><b>To quit scores</b></col>, <cd>to settle or balance accounts; to render an equivalent; to make compensation.</cd><br/
[<source>1913 Webster</source>]</p>

<p><q>Does not the earth <qex>quit scores</qex> with all the elements in the noble fruits that issue from it?</q> <rj><qau>South.</qau></rj></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Score</ent><br/
<hw>Score</hw> <pr>(sk<omac/r)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Scored</conjf> <pr>(sk<omac/rd)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Scoring</conjf>.]</vmorph> <sn>1.</sn> <def>To mark with lines, scratches, or notches; to cut notches or furrows in; to notch; to scratch; to furrow; <as>as, to <ex>score</ex> timber for hewing; to <ex>score</ex> the back with a lash</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Let us <qex>score</qex> their backs.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>A briar in that tangled wilderness<br/
Had <qex>scored</qex> her white right hand.</q> <rj><qau>M. Arnold.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Especially, to mark with significant lines or notches, for indicating or keeping account of something; <as>as, to <ex>score</ex> a tally</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>To mark or signify by lines or notches; to keep record or account of; to set down; to record; to charge.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Madam, I know when,<br/
Instead of five, you <qex>scored</qex> me ten.</q> <rj><qau>Swift.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Nor need I tallies thy dear love to <qex>score</qex>.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>To engrave, as upon a shield.</def> <mark>[R.]</mark> <rj><au>Spenser.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <def>To make a score of, as points, runs, etc., in a game.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>6.</sn> <fld>(Mus.)</fld> <def>To write down in proper order and arrangement; <as>as, to <ex>score</ex> an overture for an orchestra</as>. See <er>Score</er>, <pos>n.</pos>, 9.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>7.</sn> <fld>(Geol.)</fld> <def>To mark with parallel lines or scratches; <as>as, the rocks of New England and the Western States were <ex>scored</ex> in the drift epoch</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Score</ent><br/
<hw>Score</hw> <pr>(?)</pr>, <pos>v. i.</pos> <sn>1.</sn> <def>To keep the score in a game; to act as scorer.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><sn>2.</sn>  <def>To make or count a point or points, as in a game; to tally.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><sn>3.</sn>  <def>To run up a score, or account of dues.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><sn>4.</sn>  <def>To succeed in finding a partner for sexual intercourse; to make a sexual conquest.</def> <mark>[slang]</mark><br/
[<source>PJC</source>]</p>

<p><sn>5.</sn>  <def>To purchase drugs illegally.</def> <mark>[slang]</mark><br/
[<source>PJC</source>]</p>

<p><ent>Scorer</ent><br/
<hw>Scor"er</hw> <pr>(?)</pr>, <pos>n.</pos> <def>One who, or that which, scores.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scoria</ent><br/
||<hw>Sco"ri*a</hw> <pr>(?)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Scoriae</plw> <pr>(#)</pr>.</plu> <ety>[L., fr. Gr. <?/, fr. <?/ dung, ordure.]</ety> <sn>1.</sn> <def>The recrement of metals in fusion, or the slag rejected after the reduction of metallic ores; dross.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Cellular slaggy lava; volcanic cinders.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scoriac</ent><br/
<hw>Sco"ri*ac</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Scoriaceous.</def> <rj><au>E. A. Poe.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scoriaceous</ent><br/
<hw>Sco`ri*a"ceous</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[Cf. F. <ets>scoriac<eacute/</ets>.]</ety> <def>Of or pertaining to scoria; like scoria or the recrement of metals; partaking of the nature of scoria.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scorie</ent><br/
<hw>Sco"rie</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Zool.)</fld> <def>The young of any gull.</def> <altsp>[Written also <asp>scaurie</asp>.]</altsp> <mark>[Prov. Eng.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scorification</ent><br/
<hw>Sco`ri*fi*ca"tion</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Cf. F. <ets>scorification</ets>. See <er>Scorify</er>.]</ety> <fld>(Chem.)</fld> <def>The act, process, or result of scorifying, or reducing to a slag; hence, the separation from earthy matter by means of a slag; <as>as, the <ex>scorification</ex> of ores</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scorifier</ent><br/
<hw>Sco"ri*fi`er</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Chem.)</fld> <def>One who, or that which, scorifies; specifically, a small flat bowl-shaped cup used in the first heating in assaying, to remove the earth and gangue, and to concentrate the gold and silver in a lead button.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scoriform</ent><br/
<hw>Sco"ri*form</hw> <pr>(?)</pr>, <pos>a.</pos> <def>In the form of scoria.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scorify</ent><br/
<hw>Sco"ri*fy</hw> <pr>(?)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Scorified</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Scorifying</conjf> <pr>(?)</pr>.]</vmorph> <ety>[<ets>Scoria</ets> + <ets>-fy</ets>: cf. F. <ets>scorifier</ets>.]</ety> <fld>(Chem.)</fld> <def>To reduce to scoria or slag; specifically, in assaying, to fuse so as to separate the gangue and earthy material, with borax, lead, soda, etc., thus leaving the gold and silver in a lead button; hence, to separate from, or by means of, a slag.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scorious</ent><br/
<hw>Sco"ri*ous</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Scoriaceous.</def> <rj><au>Sir T. Browne.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scorn</ent><br/
<hw>Scorn</hw> <pr>(sk<ocir/rn)</pr>, <pos>n.</pos> <ety>[OE. <ets>scorn</ets>, <ets>scarn</ets>, <ets>scharn</ets>, OF. <ets>escarn</ets>, <ets>escharn</ets>, <ets>eschar</ets>, of German origin; cf. OHG. <ets>skern</ets> mockery, <ets>skern<omac/n</ets> to mock; but cf. also OF. <ets>escorner</ets> to mock.]</ety> <sn>1.</sn> <def>Extreme and lofty contempt; haughty disregard; that disdain which springs from the opinion of the utter meanness and unworthiness of an object.</def><br/
[<source>1913 Webster</source>]</p>

<p><q><qex>Scorn</qex> at first makes after love the more.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>And wandered backward as in <qex>scorn</qex>,<br/
To wait an aeon to be born.</q> <rj><qau>Emerson.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>An act or expression of extreme contempt.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Every sullen frown and bitter <qex>scorn</qex><br/
But fanned the fuel that too fast did burn.</q> <rj><qau>Dryden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>An object of extreme disdain, contempt, or derision.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Thou makest us a reproach to our neighbors, a <qex>scorn</qex> and a derision to them that are round about us.</q> <rj><qau>Ps. xliv. 13.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>To think scorn</b></col>, <cd>to regard as worthy of scorn or contempt; to disdain.</cd> <ldquo/He <xex>thought scorn</xex> to lay hands on Mordecai alone.<rdquo/ <au>Esther iii. 6.</au> -- <col><b>To laugh to scorn</b></col>, <cd>to deride; to make a mock of; to ridicule as contemptible.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><syn><b>Syn.</b> -- Contempt; disdain; derision; contumely; despite; slight; dishonor; mockery.</syn><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scorn</ent><br/
<hw>Scorn</hw>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Scorned</conjf> <pr>(sk<ocir/rnd)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Scoring</conjf>.]</vmorph> <ety>[OE. <ets>scornen</ets>, <ets>scarnen</ets>, <ets>schornen</ets>, OF. <ets>escarnir</ets>, <ets>escharnir</ets>. See <er>Scorn</er>, <pos>n.</pos>]</ety> <sn>1.</sn> <def>To hold in extreme contempt; to reject as unworthy of regard; to despise; to contemn; to disdain.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>I <qex>scorn</qex> thy meat; 't would choke me.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>This my long sufferance, and my day of grace,<br/
Those who neglect and <qex>scorn</qex> shall never taste.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>We <qex>scorn</qex> what is in itself contemptible or disgraceful.</q> <rj><qau>C. J. Smith.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To treat with extreme contempt; to make the object of insult; to mock; to scoff at; to deride.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>His fellow, that lay by his bed's side,<br/
Gan for to laugh, and <qex>scorned</qex> him full fast.</q> <rj><qau>Chaucer.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>To taunt and <qex>scorn</qex> you thus opprobriously.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><syn><b>Syn.</b> -- To contemn; despise; disdain. See <er>Contemn</er>.</syn><br/
[<source>1913 Webster</source>]</p>

<p><-- p. 1290 pr=PCP --></p>

<p><ent>Scorn</ent><br/
<hw>Scorn</hw> <pr>(sk<ocir/rn)</pr>, <pos>v. i.</pos> <def>To scoff; to mock; to show contumely, derision, or reproach; to act disdainfully.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>He said mine eyes were black and my hair black,<br/
And, now I am remembered, <qex>scorned</qex> at me.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scorner</ent><br/
<hw>Scorn"er</hw> <pr>(?)</pr>, <pos>n.</pos> <def>One who scorns; a despiser; a contemner; specifically, a scoffer at religion.</def> <ldquo/Great <xex>scorners</xex> of death.<rdquo/ <rj><au>Spenser.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Surely he scorneth the <qex>scorners</qex>: but he giveth grace unto the lowly.</q> <rj><qau>Prov. iii. 34.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scornful</ent><br/
<hw>Scorn"ful</hw> <pr>(?)</pr>, <pos>a.</pos> <sn>1.</sn> <def>Full of scorn or contempt; contemptuous; disdainful.</def><br/
[<source>1913 Webster</source>]</p>

<p><q><qex>Scornful</qex> of winter's frost and summer's sun.</q> <rj><qau>Prior.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Dart not <qex>scornful</qex> glances from those eyes.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Treated with scorn; exciting scorn.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>The <qex>scornful</qex> mark of every open eye.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><syn><b>Syn.</b> -- Contemptuous; disdainful; contumelious; reproachful; insolent.</syn><br/
[<source>1913 Webster</source>]</p>

<p>-- <wordforms><wf>Scorn"ful*ly</wf>, <pos>adv.</pos> -- <wf>Scorn"ful*ness</wf>, <pos>n.</pos></wordforms><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scorny</ent><br/
<hw>Scorn"y</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Deserving scorn; paltry.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scorodite</ent><br/
<hw>Scor"o*dite</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[G. <ets>scorodit</ets>; -- so called in allusion to its smell under the blowpipe, from Gr. <?/ garlic.]</ety> <fld>(Min.)</fld> <def>A leek-green or brownish mineral occurring in orthorhombic crystals. It is a hydrous arseniate of iron.</def> <altsp>[Written also <asp>skorodite</asp>.]</altsp><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scorpaenoid</ent><br/
<hw>Scor*pae"noid</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[NL. <ets>Scorpaena</ets>, a typical genus (see <er>Scorpene</er>) + <ets>-oid</ets>.]</ety> <fld>(Zool.)</fld> <def>Of or pertaining to the family <fam>Scorpaenidae</fam>, which includes the scorpene, the rosefish, the California rockfishes, and many other food fishes. <altsp>[Written also <asp>scorpaenid</asp>.]</altsp> See <xex>Illust.</xex> under <er>Rockfish</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scorpene</ent><br/
<hw>Scor"pene</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F. <ets>scorp<egrave/ne</ets>, fr. L. <ets>scorpaena</ets> a kind of fish, Gr. <?/.]</ety> <fld>(Zool.)</fld> <def>A marine food fish of the genus <gen>Scorpaena</gen>, as the European hogfish (<spn>Scorpaena scrofa</spn>), and the California species (<spn>Scorpaena guttata</spn>).</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scorper</ent><br/
<hw>Scor"per</hw> <pr>(?)</pr>, <pos>n.</pos> <def>Same as <er>Scauper</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scorpio</ent><br/
||<hw>Scor"pi*o</hw> <pr>(?)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Scorpiones</plw> <pr>(#)</pr>.</plu> <ety>[L.]</ety> <sn>1.</sn> <fld>(Zool.)</fld> <def>A scorpion.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Astron.)</fld> <sd>(a)</sd> <def>The eighth sign of the zodiac, which the sun enters about the twenty-third day of October, marked thus [<scorpio/] in almanacs.</def> <sd>(b)</sd> <def>A constellation of the zodiac containing the bright star Antares. It is drawn on the celestial globe in the figure of a scorpion.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scorpiodea</ent><br/
||<hw>Scor`pi*o"de*a</hw> <pr>(?)</pr>, <pos>n. pl.</pos> <ety>[NL.]</ety> <fld>(Zool.)</fld> <def>Same as <er>Scorpiones</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scorpioidal</ent><br/
<ent>Scorpioid</ent><br/
<mhw>{ <hw>Scor"pi*oid</hw> <pr>(?)</pr>, <hw>Scor`pi*oid"al</hw> <pr>(?)</pr>, }</mhw> <pos>a.</pos> <ety>[Gr. <?/; <?/ a scorpion + <grk>e'i^dos</grk> form.]</ety> <sn>1.</sn> <def>Having the inflorescence curved or circinate at the end, like a scorpion's tail.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scorpion</ent><br/
<hw>Scor"pi*on</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F., fr. L. <ets>scorpio</ets>, <ets>scorpius</ets>, Gr. <?/, perhaps akin to E. <ets>sharp</ets>.]</ety> <sn>1.</sn> <fld>(Zool.)</fld> <def>Any one of numerous species of pulmonate arachnids of the order <ord>Scorpiones</ord>, having a suctorial mouth, large claw-bearing palpi, and a caudal sting.</def><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ Scorpions have a flattened body, and a long, slender post-abdomen formed of six movable segments, the last of which terminates in a curved venomous sting. The venom causes great pain, but is unattended either with redness or swelling, except in the axillary or inguinal glands, when an extremity is affected. It is seldom if ever destructive of life. Scorpions are found widely dispersed in the warm climates of both the Old and New Worlds.</note><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Zool.)</fld> <def>The pine or gray lizard (<spn>Sceloporus undulatus</spn>).</def> <mark>[Local, U. S.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <fld>(Zool.)</fld> <def>The scorpene.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <fld>(Script.)</fld> <def>A painful scourge.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>My father hath chastised you with whips, but I will chastise you with <qex>scorpions</qex>.</q> <rj><qau>1 Kings xii. 11.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <fld>(Astron.)</fld> <def>A sign and constellation. See <er>Scorpio</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>6.</sn> <fld>(Antiq.)</fld> <def>An ancient military engine for hurling stones and other missiles.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Book scorpion</b></col>. <fld>(Zool.)</fld> <cd>See under <er>Book</er>.</cd> -- <col><b>False scorpion</b></col>. <fld>(Zool.)</fld> <cd>See under <er>False</er>, and <cref>Book scorpion</cref>.</cd> -- <mcol><col><b>Scorpion bug</b></col>, <it>or</it> <col><b>Water scorpion</b></col></mcol> <fld>(Zool.)</fld> <cd>See <er>Nepa</er>.</cd> -- <col><b>Scorpion fly</b></col> <fld>(Zool.)</fld>, <cd>a neuropterous insect of the genus <gen>Panorpa</gen>. See <er>Panorpid</er>.</cd> -- <col><b>Scorpion grass</b></col> <fld>(Bot.)</fld>, <cd>a plant of the genus <gen>Myosotis</gen>.  <spn>Myosotis palustris</spn> is the <stype>forget-me-not</stype>.</cd> -- <col><b>Scorpion senna</b></col> <fld>(Bot.)</fld>, <cd>a yellow-flowered leguminous shrub (<spn>Coronilla Emerus</spn>) having a slender joined pod, like a scorpion's tail. The leaves are said to yield a dye like indigo, and to be used sometimes to adulterate senna.</cd> -- <col><b>Scorpion shell</b></col> <fld>(Zool.)</fld>, <cd>any shell of the genus Pteroceras. See <er>Pteroceras</er>.</cd> -- <col><b>Scorpion spiders</b></col>. <fld>(Zool.)</fld>, <cd>any one of the Pedipalpi.</cd> -- <col><b>Scorpion's tail</b></col> <fld>(Bot.)</fld>, <cd>any plant of the leguminous genus <gen>Scorpiurus</gen>, herbs with a circinately coiled pod; -- also called <altname>caterpillar</altname>.</cd> -- <col><b>Scorpion's thorn</b></col> <fld>(Bot.)</fld>, <cd>a thorny leguminous plant (<spn>Genista Scorpius</spn>) of Southern Europe.</cd> -- <col><b>The Scorpion's Heart</b></col> <fld>(Astron.)</fld>, <cd>the star Antares in the constellation Scorpio.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scorpiones</ent><br/
||<hw>Scor`pi*o"nes</hw> <pr>(?)</pr>, <pos>n. pl.</pos> <ety>[NL.]</ety> <fld>(Zool.)</fld> <def>A division of arachnids comprising the scorpions.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scorpionidea</ent><br/
||<hw>Scor`pi*o*nid"e*a</hw> <pr>(?)</pr>, <pos>n. pl.</pos> <ety>[NL.]</ety> <fld>(Zool.)</fld> <def>Same as <er>Scorpiones</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scorpionwort</ent><br/
<hw>Scor"pi*on*wort`</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Bot.)</fld> <def>A leguminous plant (<spn>Ornithopus scorpioides</spn>) of Southern Europe, having slender curved pods.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scorse</ent><br/
<hw>Scorse</hw> <pr>(? <or/ ?)</pr>, <pos>n.</pos> <ety>[Cf. It. <ets>scorsa</ets> a course, and E. <ets>discourse</ets>.]</ety> <def>Barter; exchange; trade.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>And recompensed them with a better <qex>scorse</qex>.</q> <rj><qau>Spenser.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scorse</ent><br/
<hw>Scorse</hw>, <pos>v. t.</pos> <ety>[Written also <ets>scourse</ets>, and <ets>scoss</ets>.]</ety> <sn>1.</sn> <def>To barter or exchange.</def> <mark>[Obs.]</mark> <rj><au>Spenser.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To chase.</def> <mark>[Obs.]</mark> <rj><au>Spenser.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scorse</ent><br/
<hw>Scorse</hw>, <pos>v. i.</pos> <def>To deal for the purchase of anything; to practice barter.</def> <mark>[Obs.]</mark> <rj><au>B. Jonson.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scortatory</ent><br/
<hw>Scor"ta*to*ry</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>scortator</ets> a fornicator, from <ets>scortari</ets> to fornicate, <ets>scortum</ets> a prostitute.]</ety> <def>Pertaining to lewdness or fornication; lewd.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scot</ent><br/
<hw>Scot</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A name for a horse.</def> <mark>[Obs.]</mark> <rj><au>Chaucer.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scot</ent><br/
<hw>Scot</hw>, <pos>n.</pos> <ety>[Cf. L. <ets>Skoti</ets>, pl., AS. <ets>Scotta</ets>, pl. <ets>Skottas</ets>, <ets>Sceottas</ets>.]</ety> <def>A native or inhabitant of Scotland; a Scotsman, or Scotchman.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scot</ent><br/
<hw>Scot</hw>, <pos>n.</pos> <ety>[Icel. <ets>skot</ets>; or OF. <ets>escot</ets>, F. <ets><eacute/cot</ets>, LL. <ets>scottum</ets>, <ets>scotum</ets>, from a kindred German word; akin to AS. <ets>scot</ets>, and E. <ets>shot</ets>, <ets>shoot</ets>; cf. AS. <ets>sce<oacute/tan</ets> to shoot, to contribute. See <er>Shoot</er>, and cf. <er>Shot</er>.]</ety> <def>A portion of money assessed or paid; a tax or contribution; a mulct; a fine; a shot.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Scot and lot</b></col>, <cd>formerly, a parish assessment laid on subjects according to their ability.</cd> <mark>[Eng.]</mark> <au>Cowell.</au> <cd>Now, a phrase for obligations of every kind regarded collectivelly.</cd><br/
[<source>1913 Webster</source>]</p>

<p><q>Experienced men of the world know very well that it is best to pay <qex>scot and lot</qex> as they go along.</q> <rj><qau>Emerson.</qau></rj> </cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scotale</ent><br/
<ent>Scotal</ent><br/
<mhw>{ <hw>Scot"al</hw> <pr>(?)</pr>, <hw>Scot"ale</hw> <pr>(?)</pr>, }</mhw> <pos>n.</pos> <ety>[<ets>Scot</ets> + <ets>ale</ets>.]</ety> <fld>(O. Eng. Law)</fld> <def>The keeping of an alehouse by an officer of a forest, and drawing people to spend their money for liquor, for fear of his displeasure.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scotch</ent><br/
<hw>Scotch</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[Cf. <er>Scottish</er>.]</ety> <def>Of or pertaining to Scotland, its language, or its inhabitants; Scottish.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Scotch broom</b></col> <fld>(Bot.)</fld>, <cd>the <spn>Cytisus scoparius</spn>. See <er>Broom</er>.</cd> -- <mcol><col><b>Scotch dipper</b></col>, <it>or</it> <col><b>Scotch duck</b></col></mcol> <fld>(Zool.)</fld>, <cd>the bufflehead; -- called also <altname>Scotch teal</altname>, and <altname>Scotchman</altname>.</cd> -- <col><b>Scotch fiddle</b></col>, <cd>the itch.</cd> <mark>[Low]</mark> <au>Sir W. Scott.</au> -- <col><b>Scotch mist</b></col>, <cd>a coarse, dense mist, like fine rain.</cd> -- <col><b>Scotch nightingale</b></col> <fld>(Zool.)</fld>, <cd>the sedge warbler.</cd> <mark>[Prov. Eng.]</mark> -- <col><b>Scotch pebble</b></col>. <cd>See under <er>pebble</er>.</cd> -- <col><b>Scotch pine</b></col> <fld>(Bot.)</fld> <cd>See <er>Riga fir</er>.</cd> -- <col><b>Scotch thistle</b></col> <fld>(Bot.)</fld>, <cd>a species of thistle (<spn>Onopordon acanthium</spn>); -- so called from its being the national emblem of the Scotch.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scotch</ent><br/
<hw>Scotch</hw>, <pos>n.</pos> <sn>1.</sn> <def>The dialect or dialects of English spoken by the people of Scotland.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Collectively, the people of Scotland.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scotch</ent><br/
<hw>Scotch</hw>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Scotched</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Scotching</conjf>.]</vmorph> <ety>[Cf. Prov. E. <ets>scote</ets> a prop, and Walloon <ets>ascot</ets> a prop, <ets>ascoter</ets> to prop, F. <ets>accoter</ets>, also Armor. <ets>skoaz</ets> the shoulder, <ets>skoazia</ets> to shoulder up, to prop, to support, W. <ets>ysgwydd</ets> a shoulder, <ets>ysgwyddo</ets> to shoulder. Cf. <er>Scoat</er>.]</ety> <altsp>[Written also <asp>scoatch</asp>, <asp>scoat</asp>.]</altsp> <def>To shoulder up; to prop or block with a wedge, chock, etc., as a wheel, to prevent its rolling or slipping.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scotch</ent><br/
<hw>Scotch</hw>, <pos>n.</pos> <def>A chock, wedge, prop, or other support, to prevent slipping; <as>as, a <ex>scotch</ex> for a wheel or a log on inclined ground</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scotch</ent><br/
<hw>Scotch</hw>, <pos>v. t.</pos> <ety>[Probably the same word as <ets>scutch</ets>; cf. Norw. <ets>skoka</ets>, <ets>skoko</ets>, a swingle for flax; perhaps akin to E. <ets>shake</ets>.]</ety> <def>To cut superficially; to wound; to score.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>We have <qex>scotched</qex> the snake, not killed it.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Scotched collops</b></col> <fld>(Cookery)</fld>, <cd>a dish made of pieces of beef or veal cut thin, or minced, beaten flat, and stewed with onion and other condiments; -- called also <altname>Scotch collops</altname>.</cd> <altsp>[Written also <asp>scotcht collops</asp>.]</altsp></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scotch</ent><br/
<hw>Scotch</hw>, <pos>n.</pos> <def>A slight cut or incision; a score.</def> <rj><au>Walton.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scotch-hopper</ent><br/
<hw>Scotch"-hop`per</hw> <pr>(?)</pr>, <pos>n.</pos> <def>Hopscotch.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scotching</ent><br/
<hw>Scotch"ing</hw>, <pos>n.</pos> <fld>(Masonry)</fld> <def>Dressing stone with a pick or pointed instrument.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scotchman</ent><br/
<hw>Scotch"man</hw> <pr>(?)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Scotchmen</plw> <pr>(<?/)</pr>.</plu> <sn>1.</sn> <def>A native or inhabitant of Scotland; a Scot; a Scotsman.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Naut.)</fld> <def>A piece of wood or stiff hide placed over shrouds and other rigging to prevent chafe by the running gear.</def> <rj><au>Ham. Nav. Encyc.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scotch rite</ent><br/
<hw>Scotch rite</hw>. <fld>(Freemasonry)</fld> <def>The ceremonial observed by one of the <col><b>Masonic systems</b></col>, called in full the <altname>Ancient and Accepted Scotch Rite</altname>; also, the system itself, which confers thirty-three degrees, of which the first three are nearly identical with those of the York rite.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Scotch terrier</ent><br/
<hw>Scotch terrier</hw>. <fld>(Zool.)</fld> <def>One of a breed of small terriers with long, rough hair.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Scoter</ent><br/
<hw>Sco"ter</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Cf. Prov. E. <ets>scote</ets> to plow up.]</ety> <fld>(Zool.)</fld> <def>Any one of several species of northern sea ducks of the genus <gen>Oidemia</gen>.</def><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ The European scoters are <styp><spn>Oidemia nigra</spn></styp>, called also <altname>black duck</altname>, <altname>black diver</altname>, <altname>surf duck</altname>; and the velvet, or double, scoter (<styp><spn>Oidemia fusca</spn></styp>). The common American species are the velvet, or white-winged, scoter (<spn>Oidemia Deglandi</spn>), called also <altname>velvet duck</altname>, <altname>white-wing</altname>, <altname>bull coot</altname>, <altname>white-winged coot</altname>; the black scoter (<spn>Oidemia Americana</spn>), called also <altname>black coot</altname>, <altname>butterbill</altname>, <altname>coppernose</altname>; and the surf scoter, or surf duck (<spn>Oidemia perspicillata</spn>), called also <altname>baldpate</altname>, <altname>skunkhead</altname>, <altname>horsehead</altname>, <altname>patchhead</altname>, <altname>pishaug</altname>, and <altname>spectacled coot</altname>. These birds are collectively called also <altname>coots</altname>.  The females and young are called <stype>gray coots</stype>, and <stype>brown coots</stype>.</note><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scot-free</ent><br/
<hw>Scot"-free"</hw> <pr><mark>[?]</mark></pr>, <pos>a.</pos> <def>Free from payment of scot; untaxed; hence, unhurt; clear; safe.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Do as much for this purpose, and thou shalt pass <qex>scot-free</qex>.</q> <rj><qau>Sir W. Scott.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Then young Hay escaped <qex>scot-free</qex> to Holland.</q> <rj><qau>A. Lang.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scoth</ent><br/
<hw>Scoth</hw> <pr>(?)</pr>, <pos>v. t.</pos> <def>To clothe or cover up.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scotia</ent><br/
||<hw>Sco"ti*a</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L., fr. Gr. <grk>skoti`a</grk> darkness, a sunken molding in the base of a pillar, so called from the dark shadow it casts, from <grk>sko`tos</grk> darkness.]</ety> <fld>(Arch.)</fld> <def>A concave molding used especially in classical architecture.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scotia</ent><br/
<hw>Sco"ti*a</hw>, <pos>n.</pos> <ety>[L.]</ety> <def>Scotland</def> <mark>[Poetic]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>O <qex>Scotia</qex>! my dear, my native soil!</q> <rj><qau>Burns.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scotist</ent><br/
<hw>Sco"tist</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Eccl. Hist.)</fld> <def>A follower of (Joannes) Duns <etsep>Scotus</etsep>, the Franciscan scholastic (<it>d.</it> 1308), who maintained certain doctrines in philosophy and theology, in opposition to the <contr>Thomists</contr>, or followers of Thomas Aquinas, the Dominican scholastic.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scotograph</ent><br/
<hw>Scot"o*graph</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Gr. <grk>sko`tos</grk> darkness + <ets>-graph</ets>.]</ety> <def>An instrument for writing in the dark, or without seeing.</def> <rj><au>Maunder.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scotoma</ent><br/
||<hw>Sco*to"ma</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L.]</ety> <fld>(Med.)</fld> <def>Scotomy.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scotomy</ent><br/
<hw>Scot"o*my</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[NL. <ets>scotomia</ets>, from Gr. <?/ dizziness, fr. <?/ to darken, fr. <grk>sko`tos</grk> darkness: cf. F. <ets>scotomie</ets>.]</ety> <sn>1.</sn> <def>Dizziness with dimness of sight.</def> <mark>[Obs.]</mark> <rj><au>Massinger.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Med.)</fld> <def>Obscuration of the field of vision due to the appearance of a dark spot before the eye.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scotoscope</ent><br/
<hw>Sco"to*scope</hw> <pr>(? <or/ ?)</pr>, <pos>n.</pos> <ety>[Gr. <grk>sko`tos</grk> darkness + <ets>-scope</ets>.]</ety> <def>An instrument that discloses objects in the dark or in a faint light.</def> <mark>[Obs.]</mark> <rj><au>Pepys.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scots</ent><br/
<hw>Scots</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[For older <ets>Scottis</ets> Scottish. See <er>Scottish</er>.]</ety> <def>Of or pertaining to the Scotch; Scotch; Scottish; <as>as, <ex>Scots</ex> law; a pound <ex>Scots</ex> (1s. 8d.)</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scotsman</ent><br/
<hw>Scots"man</hw> <pr>(-m<it>a</it>n)</pr>, <pos>n.</pos> <def>See <er>Scotchman</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scottering</ent><br/
<hw>Scot"ter*ing</hw> <pr>(?)</pr>, <pos>n.</pos> <def>The burning of a wad of pease straw at the end of harvest.</def> <mark>[Prov. Eng.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scotticism</ent><br/
<hw>Scot"ti*cism</hw> <pr>(?)</pr>, <pos>n.</pos> <def>An idiom, or mode of expression, peculiar to Scotland or Scotchmen.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>That, in short, in which the <qex>Scotticism</qex> of Scotsmen most intimately consists, is the habit of emphasis.</q> <rj><qau>Masson.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scotticize</ent><br/
<hw>Scot"ti*cize</hw> <pr>(?)</pr>, <pos>v. t.</pos> <def>To cause to become like the Scotch; to make Scottish.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scottish</ent><br/
<hw>Scot"tish</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[From <er>Scot</er> a Scotchman: cf. AS. <ets>Scyttisc</ets>, and E. <er>Scotch</er>, <pos>a.</pos>, <er>Scots</er>, <pos>a.</pos>]</ety> <def>Of or pertaining to the inhabitants of Scotland, their country, or their language; <as>as, <ex>Scottish</ex> industry or economy; a <ex>Scottish</ex> chief; a <ex>Scottish</ex> dialect</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scottish terrier</ent><br/
<hw>Scot"tish ter"ri*er</hw>. <fld>(Zool.)</fld> <def>Same as <cref>Scotch terrier</cref>.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Scoundrel</ent><br/
<hw>Scoun"drel</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Probably from Prov. E. & Scotch <ets>scunner</ets>, <ets>scouner</ets>, to loathe, to disgust, akin to AS. <ets>scunian</ets> to shun. See <er>Shun</er>.]</ety> <def>A mean, worthless fellow; a rascal; a villain; a man without honor or virtue.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Go, if your ancient, but ignoble blood<br/
Has crept through <qex>scoundrels</qex> ever since the flood.</q> <rj><qau>Pope.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scoundrel</ent><br/
<hw>Scoun"drel</hw>, <pos>a.</pos> <def>Low; base; mean; unprincipled.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scoundreldom</ent><br/
<hw>Scoun"drel*dom</hw> <pr>(?)</pr>, <pos>n.</pos> <def>The domain or sphere of scoundrels; scoundrels, collectively; the state, ideas, or practices of scoundrels.</def> <rj><au>Carlyle.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scoundrelism</ent><br/
<hw>Scoun"drel*ism</hw> <pr>(?)</pr>, <pos>n.</pos> <def>The practices or conduct of a scoundrel; baseness; rascality.</def> <rj><au>Cotgrave.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scour</ent><br/
<hw>Scour</hw> <pr>(skour)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Scoured</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Scouring</conjf>.]</vmorph> <ety>[Akin to LG. <ets>sch<uum/ren</ets>, D. <ets>schuren</ets>, <ets>schueren</ets>, G. <ets>scheuern</ets>, Dan. <ets>skure</ets>; Sw. <ets>skura</ets>; all possibly fr. LL. <ets>escurare</ets>, fr. L. <ets>ex</ets> + <ets>curare</ets> to take care. Cf. <er>Cure</er>.]</ety> <sn>1.</sn> <def>To rub hard with something rough, as sand or Bristol brick, especially for the purpose of cleaning; to clean by friction; to make clean or bright; to cleanse from grease, dirt, etc., as articles of dress.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To purge; <as>as, to <ex>scour</ex> a horse</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>To remove by rubbing or cleansing; to sweep along or off; to carry away or remove, as by a current of water; -- often with <xex>off</xex> or <xex>away</xex>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>[I will] stain my favors in a bloody mask,<br/
Which, washed away, shall <qex>scour</qex> my shame with it.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <ety>[Perhaps a different word; cf. OF. <ets>escorre</ets>, <ets>escourre</ets>, It. <ets>scorrere</ets>, both fr. L. <ets>excurrere</ets> to run forth. Cf. <er>Excursion</er>.]</ety> <def>To pass swiftly over; to brush along; to traverse or search thoroughly; <as>as, to <ex>scour</ex> the coast</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Not so when swift Camilla <qex>scours</qex> the plain.</q> <rj><qau>Pope.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <def>To cleanse or clear, as by a current of water; to flush.</def></p>

<p><q>If my neighbor ought to <qex>scour</qex> a ditch.</q>  <rj><qau>Blackstone.</qau></rj><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><cs><col><b>Scouring barrel</b></col>, <cd>a tumbling barrel. See under <er>Tumbling</er>.</cd> -- <col><b>Scouring cinder</b></col> <fld>(Metal.)</fld>, <cd>a basic slag, which attacks the lining of a shaft furnace.</cd> <au>Raymond.</au> -- <col><b>Scouring rush</b></col>. <fld>(Bot.)</fld> <cd>See <cref>Dutch rush</cref>, under <er>Dutch</er>.</cd> -- <col><b>Scouring stock</b></col> <fld>(Woolen Manuf.)</fld>, <cd>a kind of fulling mill.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scour</ent><br/
<hw>Scour</hw>, <pos>v. i.</pos> <sn>1.</sn> <def>To clean anything by rubbing.</def> <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To cleanse anything.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Warm water is softer than cold, for it <qex>scoureth</qex> better.</q> <rj><qau>Bacon.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>To be purged freely; to have a diarrhoea.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>To run swiftly; to rove or range in pursuit or search of something; to scamper.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>So four fierce coursers, starting to the race,<br/
<qex>Scour</qex> through the plain, and lengthen every pace.</q> <rj><qau>Dryden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scour</ent><br/
<hw>Scour</hw>, <pos>n.</pos> <sn>1.</sn> <def>Diarrhoea or dysentery among cattle.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>The act of scouring.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><sn>3.</sn>  <def>A place scoured out by running water, as in the bed of a stream below a fall.</def></p>

<p><q>If you catch the two sole denizens [trout] of a particular <qex>scour</qex>, you will find another pair installed in their place to-morrow.</q>  <rj><qau>Grant Allen.</qau></rj><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Scourage</ent><br/
<hw>Scour"age</hw> <pr>(?; 48)</pr>, <pos>n.</pos> <def>Refuse water after scouring.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scourer</ent><br/
<hw>Scour"er</hw> <pr>(?)</pr>, <pos>n.</pos> <sn>1.</sn> <def>One who, or that which, scours.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A rover or footpad; a prowling robber.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>In those days of highwaymen and <qex>scourers</qex>.</q> <rj><qau>Macaulay.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scourge</ent><br/
<hw>Scourge</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F. <ets>escourg<eacute/e</ets>, fr. L. <ets>excoriata</ets> (sc. <ets>scutica</ets>) a stripped off (lash or whip), fr. <ets>excoriare</ets> to strip, to skin. See <er>Excoriate</er>.]</ety> <sn>1.</sn> <def>A lash; a strap or cord; especially, a lash used to inflict pain or punishment; an instrument of punishment or discipline; a whip.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Up to coach then goes<br/
The observed maid, takes both the <qex>scourge</qex> and reins.</q> <rj><qau>Chapman.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Hence, a means of inflicting punishment, vengeance, or suffering; an infliction of affliction; a punishment.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Sharp <qex>scourges</qex> of adversity.</q> <rj><qau>Chaucer.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>What <qex>scourge</qex> for perjury<br/
Can this dark monarchy afford false Clarence?</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scourge</ent><br/
<hw>Scourge</hw>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Scourged</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Scourging</conjf> <pr>(?)</pr>.]</vmorph> <ety>[From <er>Scourge</er>, <pos>n.</pos>: cf. OF. <ets>escorgier</ets>.]</ety> <sn>1.</sn> <def>To whip severely; to lash.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Is it lawful for you to <qex>scourge</qex> a . . . Roman?</q> <rj><qau>Acts xxii. 25.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To punish with severity; to chastise; to afflict, as for sins or faults, and with the purpose of correction.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Whom the Lord loveth he chasteneth, and <qex>scourgeth</qex> every son whom he receiveth.</q> <rj><qau>Heb. xii. 6.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>To harass or afflict severely.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>To <qex>scourge</qex> and impoverish the people.</q> <rj><qau>Brougham.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scourger</ent><br/
<hw>Scour"ger</hw> <pr>(?)</pr>, <pos>n.</pos> <def>One who scourges or punishes; one who afflicts severely.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The West must own the <qex>scourger</qex> of the world.</q> <rj><qau>Byron.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scourse</ent><br/
<hw>Scourse</hw> <pr>(sk<omac/rs)</pr>, <pos>v. t.</pos> <def>See <er>Scorse</er>.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scouse</ent><br/
<hw>Scouse</hw> <pr>(skous)</pr>, <pos>n.</pos> <fld>(Naut.)</fld> <def>A sailor's dish. Bread <ex>scouse</ex> contains no meat; lob<ex>scouse</ex> contains meat, etc. See <er>Lobscouse</er>.</def> <rj><au>Ham. Nav. Encyc.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scout</ent><br/
<hw>Scout</hw> <pr>(skout)</pr>, <pos>n.</pos> <ety>[Icel. <ets>sk<umac/ta</ets> a small craft or cutter.]</ety> <def>A swift sailing boat.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>So we took a <qex>scout</qex>, very much pleased with the manner and conversation of the passengers.</q> <rj><qau>Pepys.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scout</ent><br/
<hw>Scout</hw>, <pos>n.</pos> <ety>[Icel. <ets>sk<umac/ta</ets> to jut out. Cf. <er>Scout</er> to reject.]</ety> <def>A projecting rock.</def> <mark>[Prov. Eng.]</mark> <rj><au>Wright.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><-- p. 1291 pr=PCP --></p>

<p><ent>Scout</ent><br/
<hw>Scout</hw> <pr>(skout)</pr>, <pos>v. t.</pos> <ety>[Icel. <ets>sk<umac/ta</ets> a taunt; cf. Icel. <ets>sk<umac/ta</ets> to jut out, <ets>skota</ets> to shove, <ets>skj<omac/ta</ets> to shoot, to shove. See <er>Shoot</er>.]</ety> <def>To reject with contempt, as something absurd; to treat with ridicule; to flout; <as>as, to <ex>scout</ex> an idea or an apology</as>.</def> <ldquo/Flout 'em and <xex>scout</xex> 'em.<rdquo/ <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scout</ent><br/
<hw>Scout</hw>, <pos>n.</pos> <ety>[OF. <ets>escoute</ets> scout, spy, fr. <ets>escouter</ets>, <ets>escolter</ets>, to listen, to hear, F. <ets><eacute/couter</ets>, fr. L. <ets>auscultare</ets>, to hear with attention, to listen to. See <er>Auscultation</er>.]</ety> <sn>1.</sn> <def>A person sent out to gain and bring in tidings; especially, one employed in war to gain information of the movements and condition of an enemy.</def><br/
[<source>1913 Webster</source>]</p>

<p><q><qex>Scouts</qex> each coast light-arm<egrave/d scour,<br/
Each quarter, to descry the distant foe.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A college student's or undergraduate's servant; -- so called in Oxford, England; at Cambridge called a <xex>gyp</xex>; and at Dublin, a <xex>skip</xex>.</def> <mark>[Cant]</mark><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <fld>(Cricket)</fld> <def>A fielder in a game for practice.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>The act of scouting or reconnoitering.</def> <mark>[Colloq.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>While the rat is on the <qex>scout</qex>.</q> <rj><qau>Cowper.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <def>A boy scout or girl scout (which see, above).</def><br/
[<source>Webster 1913 Suppl.</source> <source>+PJC</source>]</p>

<p><syn><b>Syn.</b> -- <er>Scout</er>, <er>Spy</er>.</syn> -- <usage>In a military sense a <xex>scout</xex> is a soldier who does duty in his proper uniform, however hazardous his adventure. A <xex>spy</xex> is one who in disguise penetrates the enemies' lines, or lurks near them, to obtain information.</usage><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scout</ent><br/
<hw>Scout</hw>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Scouted</conjf>; <pos>p. pr. & vb. n.</pos> <conjf>Scouting</conjf>.]</vmorph> <sn>1.</sn> <def>To observe, watch, or look for, as a scout; to follow for the purpose of observation, as a scout.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Take more men,<br/
And <qex>scout</qex> him round.</q> <rj><qau>Beau. & Fl.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To pass over or through, as a scout; to reconnoiter; <as>as, to <ex>scout</ex> a country</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scout</ent><br/
<hw>Scout</hw>, <pos>v. i.</pos> <def>To go on the business of scouting, or watching the motions of an enemy; to act as a scout.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>With obscure wing<br/
<qex>Scout</qex> far and wide into the realm of night.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scovel</ent><br/
<hw>Scov"el</hw> <pr>(sk<ucr/v"'l)</pr>, <pos>n.</pos> <ety>[OF. <ets>escouve</ets>, <ets>escouvette</ets>, broom, L. <ets>scopae</ets>, or cf. W. <ets>ysgubell</ets>, dim. of <ets>ysgub</ets> a broom.]</ety> <def>A mop for sweeping ovens; a malkin.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scow</ent><br/
<hw>Scow</hw> <pr>(skou)</pr>, <pos>n.</pos> <ety>[D. <ets>schouw</ets>.]</ety> <fld>(Naut.)</fld> <def>A large flat-bottomed boat, having broad, square ends.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scow</ent><br/
<hw>Scow</hw>, <pos>v. t.</pos> <def>To transport in a scow.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scowl</ent><br/
<hw>Scowl</hw> <pr>(skoul)</pr>, <pos>v. i.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Scowled</conjf> <pr>(skould)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Scowling</conjf>.]</vmorph> <ety>[Akin to Dan. <ets>skule</ets>; cf. Icel. <ets>skolla</ets> to skulk, LG. <ets>schulen</ets> to hide one's self, D. <ets>schuilen</ets>, G. <ets>schielen</ets> to squint, Dan. <ets>skele</ets>, Sw. <ets>skela</ets>, AS. <ets>sceolh</ets> squinting. Cf. <er>Skulk</er>.]</ety> <sn>1.</sn> <def>To wrinkle the brows, as in frowning or displeasure; to put on a frowning look; to look sour, sullen, severe, or angry.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>She <qex>scowled</qex> and frowned with froward countenance.</q> <rj><qau>Spenser.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Hence, to look gloomy, dark, or threatening; to lower.</def> <ldquo/The <xex>scowling</xex> heavens.<rdquo/ <rj><au>Thomson.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scowl</ent><br/
<hw>Scowl</hw>, <pos>v. t.</pos> <sn>1.</sn> <def>To look at or repel with a scowl or a frown.</def> <rj><au>Milton.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To express by a scowl; <as>as, to <ex>scowl</ex> defiance</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scowl</ent><br/
<hw>Scowl</hw>, <pos>n.</pos> <sn>1.</sn> <def>The wrinkling of the brows or face in frowing; the expression of displeasure, sullenness, or discontent in the countenance; an angry frown.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>With solemn phiz, and critic <qex>scowl</qex>.</q> <rj><qau>Lloyd.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Hence, gloom; dark or threatening aspect.</def> <rj><au>Burns.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>A ruddy storm, whose <qex>scowl</qex><br/
Made heaven's radiant face look foul.</q> <rj><qau>Crashaw.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scowlingly</ent><br/
<hw>Scowl"ing*ly</hw>, <pos>adv.</pos> <def>In a scowling manner.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scrabbed eggs</ent><br/
<hw>Scrab"bed eggs`</hw> <pr>(?)</pr>. <ety>[CF. <er>Scramble</er>.]</ety> <def>A Lenten dish, composed of eggs boiled hard, chopped, and seasoned with butter, salt, and pepper.</def> <rj><au>Halliwell.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scrabble</ent><br/
<hw>Scrab"ble</hw> <pr>(skr<acr/b"b'l)</pr>, <pos>v. i.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Scrabbled</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Scrabbling</conjf> <pr>(?)</pr>.]</vmorph> <ety>[Freq. of <ets>scrape</ets>. Cf. <er>Scramble</er>, <er>Scrawl</er>, <pos>v. t.</pos>]</ety> <sn>1.</sn> <def>To scrape, paw, or scratch with the hands; to proceed by clawing with the hands and feet; to scramble; <as>as, to <ex>scrabble</ex> up a cliff or a tree</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Now after a while Little-faith came to himself, and getting up made shift to <qex>scrabble</qex> on his way.</q> <rj><qau>Bunyan.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To make irregular, crooked, or unmeaning marks; to scribble; to scrawl.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>David . . . <qex>scrabbled</qex> on the doors of the gate.</q> <rj><qau>1. Sam. xxi. 13.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scrabble</ent><br/
<hw>Scrab"ble</hw>, <pos>v. t.</pos> <def>To mark with irregular lines or letters; to scribble; <as>as, to <ex>scrabble</ex> paper</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scrabble</ent><br/
<hw>Scrab"ble</hw>, <pos>n.</pos> <def>The act of scrabbling; a moving upon the hands and knees; a scramble; also, a scribble.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scraber</ent><br/
<hw>Scra"ber</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Cf. <er>Scrabble</er>.]</ety> <fld>(Zool.)</fld> <sd>(a)</sd> <def>The Manx shearwater.</def> <sd>(b)</sd> <def>The black guillemot.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scraffle</ent><br/
<hw>Scraf"fle</hw> <pr>(skr<acr/f"f'l)</pr>, <pos>v. i.</pos> <ety>[See <er>Scramble</er>: cf. OD. <ets>schraeffelen</ets> to scrape.]</ety> <def>To scramble or struggle; to wrangle; also, to be industrious.</def> <mark>[Prov. Eng.]</mark> <rj><au>Halliwell.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scrag</ent><br/
<hw>Scrag</hw> <pr>(skr<acr/g)</pr>, <pos>n.</pos> <ety>[Cf. dial. Sw. <ets>skraka</ets> a great dry tree, a long, lean man, Gael. <ets>sgreagach</ets> dry, shriveled, rocky. See <er>Shrink</er>, and cf. <er>Scrog</er>, <er>Shrag</er>, <pos>n.</pos>]</ety> <sn>1.</sn> <def>Something thin, lean, or rough; a bony piece; especially, a bony neckpiece of meat; hence, humorously or in contempt, the neck.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Lady MacScrew, who . . . serves up a <qex>scrag</qex> of mutton on silver.</q> <rj><qau>Thackeray.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A rawboned person.</def> <mark>[Low]</mark> <rj><au>Halliwell.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>A ragged, stunted tree or branch.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Scrag whale</b></col> <fld>(Zool.)</fld>, <cd>a North Atlantic whalebone whale (<spn>Agaphelus gibbosus</spn>). By some it is considered the young of the right whale.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scrag</ent><br/
<hw>Scrag</hw> <pr>(?)</pr>, <pos>v. t.</pos> <ety>[Cf. <er>Scrag</er>.]</ety> <def>To seize, pull, or twist the neck of; specif., to hang by the neck; to kill by hanging.</def> <mark>[Colloq.]</mark></p>

<p><q>An enthusiastic mob will <qex>scrag</qex> me to a certainty the day war breaks out.</q>  <rj><qau>Pall Mall Mag.</qau></rj><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Scragged</ent><br/
<hw>Scrag"ged</hw> <pr>(?)</pr>, <pos>a.</pos> <sn>1.</sn> <def>Rough with irregular points, or a broken surface; scraggy; <as>as, a <ex>scragged</ex> backbone</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Lean and rough; scraggy.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scraggedness</ent><br/
<hw>Scrag"ged*ness</hw>, <pos>n.</pos> <def>Quality or state of being scragged.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scraggily</ent><br/
<hw>Scrag"gi*ly</hw> <pr>(?)</pr>, <pos>adv.</pos> <def>In a scraggy manner.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scragginess</ent><br/
<hw>Scrag"gi*ness</hw>, <pos>n.</pos> <def>The quality or state of being scraggy; scraggedness.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scraggy</ent><br/
<hw>Scrag"gy</hw> <pr>(?)</pr>, <pos>a.</pos> <amorph>[<pos>Compar.</pos> <adjf>Scragger</adjf> <pr>(?)</pr>; <pos>superl.</pos> <adjf>Scraggiest</adjf>.]</amorph> <sn>1.</sn> <def>Rough with irregular points; scragged.</def> <ldquo/A <xex>scraggy</xex> rock.<rdquo/ <rj><au>J. Philips.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Lean and rough; scragged.</def> <ldquo/His sinewy, <xex>scraggy</xex> neck.<rdquo/ <rj><au>Sir W. Scott.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scragly</ent><br/
<hw>Scrag"ly</hw>, <pos>a.</pos> <def>See <er>Scraggy</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scrag-necked</ent><br/
<hw>Scrag"-necked`</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Having a scraggy neck.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scram</ent><br/
<hw>Scram</hw> <pr>(skr<acr/m)</pr>, <pos>v. i.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Scrammed</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Scramming</conjf> <pr>(?)</pr>.]</vmorph> <def>to leave; to go away; used mostly as an impolite command to a person to go away from a specific location.</def> <mark>[informal]</mark><br/
[<source>PJC</source>]</p>

<p><ent>Scram</ent><br/
<hw>Scram</hw> <pr>(skr<acr/m)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Scrammed</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Scramming</conjf> <pr>(?)</pr>.]</vmorph> <def>to shut down (a nuclear reactor) quickly, as in an emergency.</def><br/
[<source>PJC</source>]</p>

<p><ent>Scram</ent><br/
<hw>Scram</hw> <pr>(skr<acr/m)</pr>, <pos>n.</pos> <def>the rapid shut down of a nuclear reactor, as in an emergency.</def><br/
[<source>PJC</source>]</p>

<p><ent>Scramble</ent><br/
<hw>Scram"ble</hw> <pr>(?)</pr>, <pos>v. i.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Scrambled</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Scrambling</conjf> <pr>(?)</pr>.]</vmorph> <ety>[Freq. of Prov. E. <ets>scramb</ets> to rake together with the hands, or of <ets>scramp</ets> to snatch at. cf. <er>Scrabble</er>.]</ety> <sn>1.</sn> <def>To clamber with hands and knees; to scrabble; <as>as, to <ex>scramble</ex> up a cliff; to <ex>scramble</ex> over the rocks</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To struggle eagerly with others for something thrown upon the ground; to go down upon all fours to seize something; to catch rudely at what is desired.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Of other care they little reckoning make,<br/
Than how to <qex>scramble</qex> at the shearer's feast.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scramble</ent><br/
<hw>Scram"ble</hw> <pr>(?)</pr>, <pos>v. t.</pos> <sn>1.</sn> <def>To collect by scrambling; <as>as, to <ex>scramble</ex> up wealth</as>.</def> <rj><au>Marlowe.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To prepare (eggs) as a dish for the table, by stirring the yolks and whites together while cooking.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scramble</ent><br/
<hw>Scram"ble</hw>, <pos>n.</pos> <sn>1.</sn> <def>The act of scrambling, climbing on all fours, or clambering.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>The act of jostling and pushing for something desired; eager and unceremonious struggle for what is thrown or held out; <as>as, a <ex>scramble</ex> for office</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Scarcity [of money] enhances its price, and increases the <qex>scramble</qex>.</q> <rj><qau>Locke.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scrambled eggs</ent><br/
<hw>Scram"bled eggs</hw> <pr>(?)</pr>. <def>Eggs of which the whites and yolks are stirred together while cooking, or eggs beaten slightly, often with a little milk, and stirred while cooking.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Scrambler</ent><br/
<hw>Scram"bler</hw> <pr>(?)</pr>, <pos>n.</pos> <sn>1.</sn> <def>One who scrambles; one who climbs on all fours.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A greedy and unceremonious contestant.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scrambling</ent><br/
<hw>Scram"bling</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Confused and irregular; awkward; scambling.</def> -- <wordforms><wf>Scram"bling*ly</wf>, <pos>adv.</pos></wordforms><br/
[<source>1913 Webster</source>]</p>

<p><q>A huge old <qex>scrambling</qex> bedroom.</q> <rj><qau>Sir W. Scott.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scranch</ent><br/
<hw>Scranch</hw> <pr>(?)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Scranched</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Scranching</conjf>.]</vmorph> <ety>[Cf. D. <ets>schransen</ets> to eat greedily, G. <ets>schranzen</ets>. Cf. <er>Crunch</er>, <er>Scrunch</er>.]</ety> <def>To grind with the teeth, and with a crackling sound; to craunch.</def> <mark>[Prov. Eng. & Colloq. U. S.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scranky</ent><br/
<hw>Scrank"y</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Thin; lean.</def> <mark>[Scot.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scrannel</ent><br/
<hw>Scran"nel</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[Cf. <er>Scrawny</er>.]</ety> <def>Slight; thin; lean; poor.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Grate on their <qex>scrannel</qex> pipes of wretched straw.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scranny</ent><br/
<hw>Scran"ny</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[See <er>Scrannel</er>.]</ety> <def>Thin; lean; meager; scrawny; scrannel.</def> <mark>[Prov. Eng. & Scot.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scrap</ent><br/
<hw>Scrap</hw> <pr>(skr<acr/p)</pr>, <pos>n.</pos> <ety>[OE. <ets>scrappe</ets>, fr. Icel. <ets>skrap</ets> trifle, cracking. See <er>Scrape</er>, <pos>v. t.</pos>]</ety> <sn>1.</sn> <def>Something scraped off; hence, a small piece; a bit; a fragment; a detached, incomplete portion.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>I have no materials -- not a <qex>scrap</qex>.</q> <rj><qau>De Quincey.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Specifically, a fragment of something written or printed; a brief excerpt; an unconnected extract.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <pluf>pl.</pluf> <def>The crisp substance that remains after drying out animal fat; <as>as, pork <ex>scraps</ex></as>.</def><-- original had "trying out". -- apparently a misprint in original --><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <pluf>pl.</pluf> <def>Same as <cref>Scrap iron</cref>, below.</def><br/
[<source>1913 Webster</source>]</p>

<p><-- 5. Manufactured objects or parts useful only for reprocessing, esp. metal objects. --><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Scrap forgings</b></col>, <cd>forgings made from wrought iron scrap.</cd> -- <col><b>Scrap iron</b></col>. <sd>(a)</sd> <cd>Cuttings and waste pieces of wrought iron from which bar iron or forgings can be made; -- called also <altname>wrought-iron scrap</altname>.</cd> <sd>(b)</sd> <cd>Fragments of cast iron or defective castings suitable for remelting in the foundry; -- called also <altname>foundry scrap</altname>, or <altname>cast scrap</altname>.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scrapbook</ent><br/
<hw>Scrap"book`</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A blank book in which extracts cut from books and papers may be pasted and kept.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scrape</ent><br/
<hw>Scrape</hw> <pr>(skr<amac/p)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Scraped</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Scraping</conjf>.]</vmorph> <ety>[Icel. <ets>skrapa</ets>; akin to Sw. <ets>skrapa</ets>, Dan. <ets>skrabe</ets>, D. <ets>schrapen</ets>, <ets>schrabben</ets>, G. <ets>schrappen</ets>, and prob. to E. <ets>sharp</ets>.]</ety> <sn>1.</sn> <def>To rub over the surface of (something) with a sharp or rough instrument; to rub over with something that roughens by removing portions of the surface; to grate harshly over; to abrade; to make even, or bring to a required condition or form, by moving the sharp edge of an instrument breadthwise over the surface with pressure, cutting away excesses and superfluous parts; to make smooth or clean; <as>as, to <ex>scrape</ex> a bone with a knife; to <ex>scrape</ex> a metal plate to an even surface</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To remove by rubbing or scraping (in the sense above).</def><br/
[<source>1913 Webster</source>]</p>

<p><q>I will also <qex>scrape</qex> her dust from her, and make her like the top of a rock.</q> <rj><qau>Ezek. xxvi. 4.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>To collect by, or as by, a process of scraping; to gather in small portions by laborious effort; hence, to acquire avariciously and save penuriously; -- often followed by <xex>together</xex> or <xex>up</xex>; <as>as, to <ex>scrape</ex> money together</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The prelatical party complained that, to swell a number the nonconformists did not choose, but <qex>scrape</qex>, subscribers.</q> <rj><qau>Fuller.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>To express disapprobation of, as a play, or to silence, as a speaker, by drawing the feet back and forth upon the floor; -- usually with <xex>down</xex>.</def> <rj><au>Macaulay.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>To scrape acquaintance</b></col>, <cd>to seek acquaintance otherwise than by an introduction.</cd> <rj><au>Farquhar.</au></rj></cs><br/
[<source>1913 Webster</source>]</p>

<p><q>He tried to <qex>scrape acquaintance</qex> with her, but failed ignominiously.</q> <rj><qau>G. W. Cable.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scrape</ent><br/
<hw>Scrape</hw>, <pos>v. i.</pos> <sn>1.</sn> <def>To rub over the surface of anything with something which roughens or removes it, or which smooths or cleans it; to rub harshly and noisily along.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To occupy one's self with getting laboriously; <as>as, he <ex>scraped</ex> and saved until he became rich</as>.</def> <ldquo/[Spend] their <xex>scraping</xex> fathers' gold.<rdquo/ <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>To play awkwardly and inharmoniously on a violin or like instrument.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>To draw back the right foot along the ground or floor when making a bow.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scrape</ent><br/
<hw>Scrape</hw>, <pos>n.</pos> <sn>1.</sn> <def>The act of scraping; also, the effect of scraping, as a scratch, or a harsh sound; <as>as, a noisy <ex>scrape</ex> on the floor; a <ex>scrape</ex> of a pen</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A drawing back of the right foot when bowing; also, a bow made with that accompaniment.</def> <rj><au>H. Spencer.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>A disagreeable and embarrassing predicament out of which one can not get without undergoing, as it were, a painful rubbing or scraping; a perplexity; a difficulty.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The too eager pursuit of this his old enemy through thick and thin has led him into many of these <qex>scrapes</qex>.</q> <rj><qau>Bp. Warburton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scrapepenny</ent><br/
<hw>Scrape"pen`ny</hw> <pr>(?)</pr>, <pos>n.</pos> <def>One who gathers and hoards money in trifling sums; a miser.</def><-- = a pinchpenny --><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scraper</ent><br/
<hw>Scrap"er</hw> <pr>(?)</pr>, <pos>n.</pos> <sn>1.</sn> <def>An instrument with which anything is scraped.</def> Specifically: <sd>(a)</sd> <def>An instrument by which the soles of shoes are cleaned from mud and the like, by drawing them across it.</def> <sd>(b)</sd> <def>An instrument drawn by oxen or horses, used for scraping up earth in making or repairing roads, digging cellars, canals etc.</def> <sd>(c)</sd> <fld>(Naut.)</fld> <def>An instrument having two or three sharp sides or edges, for cleaning the planks, masts, or decks of a ship.</def> <sd>(d)</sd> <fld>(Lithography)</fld> <def>In the printing press, a board, or blade, the edge of which is made to rub over the tympan sheet and thus produce the impression.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>One who scrapes.</def> Specifically: <sd>(a)</sd> <def>One who plays awkwardly on a violin.</def> <sd>(b)</sd> <def>One who acquires avariciously and saves penuriously.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scraping</ent><br/
<hw>Scrap"ing</hw> <pr>(?)</pr>, <pos>n.</pos> <sn>1.</sn> <def>The act of scraping; the act or process of making even, or reducing to the proper form, by means of a scraper.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Something scraped off; that which is separated from a substance, or is collected by scraping; <as>as, the <ex>scraping</ex> of the street</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scraping</ent><br/
<hw>Scrap"ing</hw>, <pos>a.</pos> <def>Resembling the act of, or the effect produced by, one who, or that which, scrapes; <as>as, a <ex>scraping</ex> noise; a <ex>scraping</ex> miser</as>.</def> -- <wordforms><wf>Scrap"ing*ly</wf>, <pos>adv.</pos></wordforms><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scrappily</ent><br/
<hw>Scrap"pi*ly</hw> <pr>(?)</pr>, <pos>adv.</pos> <def>In a scrappy manner; in scraps.</def> <rj><au>Mary Cowden Clarke.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scrapple</ent><br/
<hw>Scrap"ple</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Dim. of <ets>scrap</ets>.]</ety> <def>An article of food made by boiling together bits or scraps of meat, usually pork, and flour or Indian meal.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Scrappy</ent><br/
<hw>Scrap"py</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Consisting of scraps; fragmentary; lacking unity or consistency; <as>as, a <ex>scrappy</ex> lecture</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>A dreadfully <qex>scrappy</qex> dinner.</q> <rj><qau>Thackeray.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scrat</ent><br/
<hw>Scrat</hw> <pr>(?)</pr>, <pos>v. t.</pos> <ety>[OE. <ets>scratten</ets>. Cf. <er>Scratch</er>.]</ety> <def>To scratch.</def> <mark>[Obs.]</mark> <rj><au>Burton.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scrat</ent><br/
<hw>Scrat</hw>, <pos>v. i.</pos> <def>To rake; to search.</def> <mark>[Obs.]</mark> <rj><au>Mir. for Mag.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scrat</ent><br/
<hw>Scrat</hw>, <pos>n.</pos> <ety>[Cf. AS. <ets>scritta</ets> an hermaphrodite, Ir. <ets>scrut</ets> a scrub, a low, mean person, Gael. <ets>sgrut</ets>, <ets>sgruit</ets>, an old, shriveled person.]</ety> <def>An hermaphrodite.</def> <mark>[Obs.]</mark> <rj><au>Skinner.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scratch</ent><br/
<hw>Scratch</hw> <pr>(?)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Scratched</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Scratching</conjf>.]</vmorph> <ety>[OE. <ets>cracchen</ets> (perhaps influenced by OE. <ets>scratten</ets> to scratch); cf. OHG. <ets>chrazz<omac/n</ets>, G. <ets>kratzen</ets>, OD. <ets>kratsen</ets>, <ets>kretsen</ets>, D. <ets>krassen</ets>, Sw. <ets>kratsa</ets> to scrape, <ets>kratta</ets> to rake, to scratch, Dan. <ets>kradse</ets> to scratch, to scrape, Icel. <ets>krota</ets> to engrave. Cf. <er>Grate</er> to rub.]</ety> <sn>1.</sn> <def>To rub and tear or mark the surface of with something sharp or ragged; to scrape, roughen, or wound slightly by drawing something pointed or rough across, as the claws, the nails, a pin, or the like.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Small sand-colored stones, so hard as to <qex>scratch</qex> glass.</q> <rj><qau>Grew.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Be mindful, when invention fails,<br/
To <qex>scratch</qex> your head, and bite your nails.</q> <rj><qau>Swift.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To write or draw hastily or awkwardly.</def> <ldquo/<xex>Scratch</xex> out a pamphlet.<rdquo/ <rj><au>Swift.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>To cancel by drawing one or more lines through, as the name of a candidate upon a ballot, or of a horse in a list; hence, to erase; to efface; -- often with <xex>out</xex>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>To dig or excavate with the claws; <as>as, some animals <ex>scratch</ex> holes, in which they burrow</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>To scratch a ticket</b></col>, <cd>to cancel one or more names of candidates on a party ballot; to refuse to vote the party ticket in its entirety.</cd> <mark>[U. S.]</mark></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scratch</ent><br/
<hw>Scratch</hw>, <pos>v. i.</pos> <sn>1.</sn> <def>To use the claws or nails in tearing or in digging; to make scratches.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Dull, tame things, . . . that will neither bite nor <qex>scratch</qex>.</q> <rj><qau>Dr. H. More.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Billiards)</fld> <def>To score, not by skillful play but by some fortunate chance of the game.</def> <mark>[Cant, U. S.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scratch</ent><br/
<hw>Scratch</hw>, <pos>n.</pos> <sn>1.</sn> <def>A break in the surface of a thing made by scratching, or by rubbing with anything pointed or rough; a slight wound, mark, furrow, or incision.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The coarse file . . . makes deep <qex>scratches</qex> in the work.</q> <rj><qau>Moxon.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>These nails with <qex>scratches</qex> deform my breast.</q> <rj><qau>Prior.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>God forbid a shallow <qex>scratch</qex> should drive<br/
The prince of Wales from such a field as this.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Pugilistic Matches)</fld> <def>A line across the prize ring; up to which boxers are brought when they join fight; hence, test, trial, or proof of courage; <as>as, to bring to the <ex>scratch</ex>; to come up to the <ex>scratch</ex></as>.</def> <mark>[Cant]</mark> <rj><au>Grose.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <pluf>pl.</pluf> <fld>(Far.)</fld> <def>Minute, but tender and troublesome, excoriations, covered with scabs, upon the heels of horses which have been used where it is very wet or muddy.</def> <rj><au>Law (Farmer's Veter. Adviser).</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>A kind of wig covering only a portion of the head.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <fld>(Billiards)</fld> <sd>(a)</sd> <def>A shot which scores by chance and not as intended by the player; a fluke.</def> <mark>[Cant, U. S.]</mark> <sd>(b)</sd> <def>a shot which results in a penalty, such as dropping the cue ball in a pocket without hitting another ball.</def><br/
[<source>1913 Webster</source> <source>+PJC</source>]</p>

<p><sn>6.</sn> <def>In various sports, the line from which the start is made, except in the case of contestants receiving a distance handicap.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><cs><col><b>Scratch cradle</b></col>. <cd>See <cref>Cratch cradle</cref>, under <er>Cratch</er>.</cd> -- <col><b>Scratch grass</b></col> <fld>(Bot.)</fld>, <cd>a climbing knotweed (<spn>Polygonum sagittatum</spn>) with a square stem beset with fine recurved prickles along the angles.</cd> -- <col><b>Scratch wig</b></col>. <cd>Same as <er>Scratch</er>, 4, above.</cd> <rj><au>Thackeray.</au></rj> -- <col><b>start from scratch</b></col> <cd>to start (again) from the very beginning; also, to start without resources.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scratch</ent><br/
<hw>Scratch</hw>, <pos>a.</pos> <def>Made, done, or happening by chance; arranged with little or no preparation; determined by circumstances; haphazard; <as>as, a <ex>scratch</ex> team; a <ex>scratch</ex> crew for a boat race; a <ex>scratch</ex> shot in billiards</as>.</def> <mark>[Slang]</mark><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Scratch race</b></col>, <cd>one without restrictions regarding the entrance of competitors; also, one for which the competitors are chosen by lot.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scratchback</ent><br/
<hw>Scratch"back`</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A toy which imitates the sound of tearing cloth, -- used by drawing it across the back of unsuspecting persons.</def> <mark>[Eng.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scratchbrush</ent><br/
<hw>Scratch"brush`</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A stiff wire brush for cleaning iron castings and other metal.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scratch coat</ent><br/
<hw>Scratch" coat`</hw> <pr>(?)</pr>. <def>The first coat in plastering; -- called also <cref>scratchwork</cref>. See <er>Pricking-up</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scratcher</ent><br/
<hw>Scratch"er</hw> <pr>(?)</pr>, <pos>n.</pos> <def>One who, or that which, scratches; specifically <fld>(Zool.)</fld>, any rasorial bird.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scratch hit</ent><br/
<hw>Scratch" hit</hw>, <pos>n.</pos> <fld>(Baseball)</fld> <def>a base hit which was weakly batted, barely allowing the batter to reach first base safely.</def><br/
[<source>PJC</source>]</p>

<p><ent>Scratching</ent><br/
<hw>Scratch"ing</hw>, <pos>adv.</pos> <def>With the action of scratching.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scratch runner</ent><br/
<ent>Scratch player</ent><br/
<mhw>{ <hw>Scratch player</hw>, <hw>Scratch runner</hw>, etc. }</mhw> <def>One that starts from the scratch; hence, one of first-rate ability.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Scratchweed</ent><br/
<hw>Scratch"weed`</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Bot.)</fld> <def>Cleavers.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scratchwork</ent><br/
<hw>Scratch"work`</hw> <pr>(?)</pr>, <pos>n.</pos> <def>See <er>Scratch coat</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scratchy</ent><br/
<hw>Scratch"y</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Characterized by scratches.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scraw</ent><br/
<hw>Scraw</hw> <pr>(skr<add/)</pr>, <pos>n.</pos> <ety>[Ir. <ets>scrath</ets> a turf, <ets>sgraith</ets> a turf, green sod; akin to Gael. <ets>sgrath</ets>, <ets>sgroth</ets>, the outer skin of anything, a turf, a green sod.]</ety> <def>A turf.</def> <mark>[Obs.]</mark> <rj><au>Swift.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scrawl</ent><br/
<hw>Scrawl</hw> <pr>(?)</pr>, <pos>v. i.</pos> <def>See <er>Crawl</er>.</def> <mark>[Obs.]</mark> <rj><au>Latimer.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scrawl</ent><br/
<hw>Scrawl</hw>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Scrawled</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Scrawling</conjf>.]</vmorph> <ety>[Probably corrupted from <ets>scrabble</ets>.]</ety> <def>To draw or mark awkwardly and irregularly; to write hastily and carelessly; to scratch; to scribble; <as>as, to <ex>scrawl</ex> a letter</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>His name, <qex>scrawled</qex> by himself.</q> <rj><qau>Macaulay.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scrawl</ent><br/
<hw>Scrawl</hw>, <pos>v. i.</pos> <def>To write unskillfully and inelegantly.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Though with a golden pen you <qex>scrawl</qex>.</q> <rj><qau>Swift.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><-- p. 1292 pr=PCP --></p>

<p><ent>Scrawl</ent><br/
<hw>Scrawl</hw> <pr>(skr<add/l)</pr>, <pos>n.</pos> <def>Unskillful or inelegant writing; that which is unskillfully or inelegantly written.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The left hand will make such a <qex>scrawl</qex>, that it will not be legible.</q> <rj><qau>Arbuthnot.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>You bid me write no more than a <qex>scrawl</qex> to you.</q> <rj><qau>Gray.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scrawler</ent><br/
<hw>Scrawl"er</hw> <pr>(-<etil/r)</pr>, <pos>n.</pos> <def>One who scrawls; a hasty, awkward writer.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scrawny</ent><br/
<hw>Scraw"ny</hw> <pr>(skr<add/"n<ycr/)</pr>, <pos>a.</pos> <ety>[Cf. <er>Scrannel</er>.]</ety> <def>Meager; thin; rawboned; bony; scranny.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scray</ent><br/
<hw>Scray</hw> <pr>(skr<amac/)</pr>, <pos>n.</pos> <ety>[Cf. W. <ets>ysgr<aum/en</ets>, <ets>ysgr<aum/ell</ets>, a sea swallow, Armor. <ets>skrav</ets>.]</ety> <fld>(Zool.)</fld> <def>A tern; the sea swallow.</def> <mark>[Prov. Eng.]</mark> <altsp>[Written also <asp>scraye</asp>.]</altsp><br/
[<source>1913 Webster</source>]</p>

<p><ent>Screable</ent><br/
<hw>Scre"a*ble</hw> <pr>(skr<emac/"<adot/*b'l)</pr>, <pos>a.</pos> <ety>[L. <ets>screare</ets> to hawk, spit out.]</ety> <def>Capable of being spit out.</def> <mark>[Obs.]</mark> <rj><au>Bailey.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Screak</ent><br/
<hw>Screak</hw> <pr>(skr<emac/k)</pr>, <pos>v. i.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Screaked</conjf> <pr>(skr<emac/kt)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Screaking</conjf>.]</vmorph> <ety>[Cf. Icel. <ets>skraekja</ets> to screech. Cf. <er>Creak</er>, <pos>v.</pos>, <er>Screech</er>.]</ety> <def>To utter suddenly a sharp, shrill sound; to screech; to creak, as a door or wheel.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Screak</ent><br/
<hw>Screak</hw>, <pos>n.</pos> <def>A creaking; a screech; a shriek.</def> <rj><au>Bp. Bull.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scream</ent><br/
<hw>Scream</hw> <pr>(skr<emac/m)</pr>, <pos>v. i.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Screamed</conjf> <pr>(skr<emac/md)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Screaming</conjf>.]</vmorph> <ety>[Icel. <ets>skraema</ets> to scare, terrify; akin to Sw. <ets>skr<aum/ma</ets>, Dan. <ets>skraemme</ets>. Cf. <er>Screech</er>.]</ety> <def>To cry out with a shrill voice; to utter a sudden, sharp outcry, or shrill, loud cry, as in fright or extreme pain; to shriek; to screech.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>I heard the owl <qex>scream</qex> and the crickets cry.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>And <qex>scream</qex> thyself as none e'er <qex>screamed</qex> before.</q> <rj><qau>Pope.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scream</ent><br/
<hw>Scream</hw>, <pos>n.</pos> <def>A sharp, shrill cry, uttered suddenly, as in terror or in pain; a shriek; a screech.</def> <ldquo/<xex>Screams</xex> of horror.<rdquo/ <rj><au>Pope.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Screamer</ent><br/
<hw>Scream"er</hw> <pr>(skr<emac/m"<etil/r)</pr>, <pos>n.</pos> <fld>(Zool.)</fld> <def>Any one of three species of South American birds constituting the family <fam>Anhimidae</fam>, and the suborder <ord>Palamedeae</ord>. They have two spines on each wing, and the head is either crested or horned. They are easily tamed, and then serve as guardians for other poultry. The crested screamers, or chajas, belong to the genus <gen>Chauna</gen>. The horned screamer, or kamichi, is <spn>Palamedea cornuta</spn>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Something so remarkable as to provoke a scream, as of joy.</def> <mark>[Slang]</mark><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><sn>3.</sn>  <def>An exclamation mark.</def> <mark>[Printer's Slang]</mark><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Screaming</ent><br/
<hw>Scream"ing</hw>, <pos>a.</pos> <sn>1.</sn> <def>Uttering screams; shrieking.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Having the nature of a scream; like a scream; shrill; sharp.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The fearful matrons raise a <qex>screaming</qex> cry.</q> <rj><qau>Dryden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scree</ent><br/
<hw>Scree</hw> <pr>(skr<emac/)</pr>, <pos>n.</pos> <def>A pebble; a stone; also, a heap of stones or rocky d<eacute/bris.</def> <mark>[Prov. Eng.]</mark> <rj><au>Southey.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Screech</ent><br/
<hw>Screech</hw> <pr>(skr<emac/ch)</pr>, <pos>v. i.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Screeched</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Screeching</conjf>.]</vmorph> <ety>[Also formerly, <ets>scritch</ets>, OE. <ets>skriken</ets>, <ets>skrichen</ets>, <ets>schriken</ets>, of Scand. origin; cf. Icel. <ets>skr<ae/kja</ets> to shriek, to screech, <ets>skr<imac/kja</ets> to titter, Sw. <ets>skrika</ets> to shriek, Dan. <ets>skrige</ets>; also Gael. <ets>sgreach</ets>, <ets>sgreuch</ets>, W. <ets>ysgrechio</ets>, Skr. <ets>kharj</ets> to creak. Cf. <er>Shriek</er>, <pos>v.</pos>, <er>Scream</er>, <pos>v.</pos>]</ety> <def>To utter a harsh, shrill cry; to make a sharp outcry, as in terror or acute pain; to scream; to shriek.</def> <ldquo/The screech owl, <xex>screeching</xex> loud.<rdquo/ <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Screech</ent><br/
<hw>Screech</hw>, <pos>n.</pos> <def>A harsh, shrill cry, as of one in acute pain or in fright; a shriek; a scream.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><mcol><col><b>Screech bird</b></col>, <it>or</it> <col><b>Screech thrush</b></col></mcol> <fld>(Zool.)</fld>, <cd>the fieldfare; -- so called from its harsh cry before rain.</cd> -- <col><b>Screech rain</b></col>.<cd></cd> -- <col><b>Screech hawk</b></col> <fld>(Zool.)</fld>, <cd>the European goatsucker; -- so called from its note.</cd> <mark>[Prov. Eng.]</mark> -- <col><b>Screech owl</b></col>. <fld>(Zool.)</fld> <sd>(a)</sd> <cd>A small American owl (<spn>Scops asio</spn>), either gray or reddish in color</cd>. <sd>(b)</sd> <cd>The European barn owl. The name is applied also to other species.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Screechers</ent><br/
<hw>Screech"ers</hw> <pr>(?)</pr>, <pos>n. pl.</pos> <fld>(Zool.)</fld> <def>The picarian birds, as distinguished from the singing birds.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Screechy</ent><br/
<hw>Screech"y</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Like a screech; shrill and harsh.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Screed</ent><br/
<hw>Screed</hw> <pr>(skr<emac/d)</pr>, <pos>n.</pos> <ety>[Prov. E., a shred, the border of a cap. See <er>Shred</er>.]</ety> <sn>1.</sn> <fld>(Arch.)</fld> <sd>(a)</sd> <def>A strip of plaster of the thickness proposed for the coat, applied to the wall at intervals of four or five feet, as a guide.</def> <sd>(b)</sd> <def>A wooden straightedge used to lay across the plaster screed, as a limit for the thickness of the coat.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A fragment; a portion; a shred.</def> <mark>[Scot.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Screed</ent><br/
<hw>Screed</hw>, <pos>n.</pos> <ety>[See 1st <er>Screed</er>. For sense 2 cf. also Gael. <ets>sgread</ets> an outcry.]</ety> <sn>1.</sn> <def>A breach or rent; a breaking forth into a loud, shrill sound; <as>as, martial <ex>screeds</ex></as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>An harangue; a long tirade on any subject.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The old carl gae them a <qex>screed</qex> of doctrine; ye might have heard him a mile down the wind.</q> <rj><qau>Sir W. Scott.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Screen</ent><br/
<hw>Screen</hw> <pr>(skr<emac/n)</pr>, <pos>n.</pos> <ety>[OE. <ets>scren</ets>, OF. <ets>escrein</ets>, <ets>escran</ets>, F. <ets><eacute/cran</ets>, of uncertain origin; cf. G. <ets>schirm</ets> a screen, OHG. <ets>scirm</ets>, <ets>scerm</ets> a protection, shield, or G. <ets>schragen</ets> a trestle, a stack of wood, or G. <ets>schranne</ets> a railing.]</ety> <sn>1.</sn> <def>Anything that separates or cuts off inconvenience, injury, or danger; that which shelters or conceals from view; a shield or protection; <as>as, a fire <ex>screen</ex></as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Your leavy <qex>screens</qex> throw down.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Some ambitious men seem as <qex>screens</qex> to princes in matters of danger and envy.</q> <rj><qau>Bacon.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Arch.)</fld> <def>A dwarf wall or partition carried up to a certain height for separation and protection, as in a church, to separate the aisle from the choir, or the like.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>A surface, as that afforded by a curtain, sheet, wall, etc., upon which an image, as a picture, is thrown by a magic lantern, solar microscope, etc.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>A long, coarse riddle or sieve, sometimes a revolving perforated cylinder, used to separate the coarser from the finer parts, as of coal, sand, gravel, and the like.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <fld>(Cricket)</fld> <def>An erection of white canvas or wood placed on the boundary opposite a batsman to enable him to see ball better.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><sn>6.</sn> <def>a netting, usu. of metal, contained in a frame, used mostly in windows or doors to allow in fresh air while excluding insects.</def> -- <col><b>Screen door</b></col>, <def>a door of which half or more is composed of a screen.</def> -- <col><b>Screen window</b></col>, <def>a screen inside a frame, fitted for insertion into a window frame.</def><br/
[<source>PJC</source>]</p>

<p><sn>7.</sn>  <def>The surface of an electronic device, as a <partof>television set</partof> or <partof>computer monitor</partof>, on which a visible image is formed. The screen is frequently the surface of a cathode-ray tube containing phosphors excited by the electron beam, but other methods for causing an image to appear on the screen are also used, as in <partof>flat-panel displays</partof>.</def><br/
[<source>PJC</source>]</p>

<p><sn>8.</sn>  <def>The motion-picture industry; motion pictures.</def> <ldquo/A star of stage and <ex>screen</ex>.<rdquo/<br/
[<source>PJC</source>]</p>

<p><ent>Screen</ent><br/
<hw>Screen</hw> <pr>(skr<emac/n)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Screened</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Screening</conjf>.]</vmorph> <sn>1.</sn> <def>To provide with a shelter or means of concealment; to separate or cut off from inconvenience, injury, or danger; to shelter; to protect; to protect by hiding; to conceal; <as>as, fruits <ex>screened</ex> from cold winds by a forest or hill</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>They were encouraged and <qex>screened</qex> by some who were in high commands.</q> <rj><qau>Macaulay.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To pass, as coal, gravel, ashes, etc., through a screen in order to separate the coarse from the fine, or the worthless from the valuable; to sift.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>to examine a group of objects methodically, to separate them into groups or to select one or more for some purpose.</def> <specif>As:</specif> <sd>(a)</sd> <def>To inspect the qualifications of candidates for a job, to select one or more to be hired.</def> <sd>(b)</sd> <fld>(Biochem., Med.)</fld> <def>to test a large number of samples, in order to find those having specific desirable properties; <as>as, to <ex>screen</ex> plant extracts for anticancer agents</as>.</def><br/
[<source>PJC</source>]</p>

<p><ent>Screening</ent><br/
<hw>Screen"ing</hw> <pr>(skr<emac/n"<icr/ng)</pr>, <pos>n.</pos> <def>the process of examining or testing objects methodically to find those having desirable properties.  See <er>screen{3}</er>.</def>  <note>In the pharmaceutical industry, pharmaceutical <ex>screening</ex> involves testing a large number of samples of substances to find those having desirable pharmacological activity; those samples which have the property sought are called <it>active</it> or <it>positive</it> in the screen.  The substances tested may be pure compounds with known structure, mixtures of pure compounds, or complex mixtures obtained by extraction from living organisms.  There are often additional sets of test performed on <it>active</it> samples, called <styp><col><b>counterscreening</b></col></styp> to eliminate those samples that may also possess undesirable properties.  In the case of <ex>screening</ex> of mixtures from living organisms, a type of <cref>counterscreening</cref> called <er>dereplication</er> is usually performed, to determine if the active sample contains a known compound which has previously been studied.</note><br/
[<source>PJC</source>]</p>

<p><ent>Screenings</ent><br/
<hw>Screen"ings</hw> <pr>(?)</pr>, <pos>n. pl.</pos> <def>The refuse left after screening sand, coal, ashes, etc.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Screw</ent><br/
<hw>Screw</hw> <pr>(skr<udd/)</pr>, <pos>n.</pos> <ety>[OE. <ets>scrue</ets>, OF. <ets>escroue</ets>, <ets>escroe</ets>, female screw, F. <ets><eacute/crou</ets>, L. <ets>scrobis</ets> a ditch, trench, in LL., the hole made by swine in rooting; cf. D. <ets>schroef</ets> a screw, G. <ets>schraube</ets>, Icel. <ets>skr<umac/fa</ets>.]</ety> <sn>1.</sn> <def>A cylinder, or a cylindrical perforation, having a continuous rib, called the <xex>thread</xex>, winding round it spirally at a constant inclination, so as to leave a continuous spiral groove between one turn and the next, -- used chiefly for producing, when revolved, motion or pressure in the direction of its axis, by the sliding of the threads of the cylinder in the grooves between the threads of the perforation adapted to it, the former being distinguished as the <xex>external</xex>, or <xex>male screw</xex>, or, more usually the <xex>screw</xex>; the latter as the <xex>internal</xex>, or <xex>female screw</xex>, or, more usually, the <xex>nut</xex>.</def><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ The screw, as a mechanical power, is a modification of the inclined plane, and may be regarded as a right-angled triangle wrapped round a cylinder, the hypotenuse of the marking the spiral thread of the screw, its base equaling the circumference of the cylinder, and its height the <xex>pitch</xex> of the thread.</note><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Specifically, a kind of nail with a spiral thread and a head with a nick to receive the end of the screw-driver. Screws are much used to hold together pieces of wood or to fasten something; -- called also <altname>wood screws</altname>, and <altname>screw nails</altname>. See also <cref>Screw bolt</cref>, below.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>Anything shaped or acting like a screw; esp., a form of wheel for propelling steam vessels. It is placed at the stern, and furnished with blades having helicoidal surfaces to act against the water in the manner of a screw. See <cref>Screw propeller</cref>, below.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>A steam vesel propelled by a screw instead of wheels; a screw steamer; a propeller.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <def>An extortioner; a sharp bargainer; a skinflint; a niggard.</def> <rj><au>Thackeray.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>6.</sn> <def>An instructor who examines with great or unnecessary severity; also, a searching or strict examination of a student by an instructor.</def> <mark>[Cant, American Colleges]</mark><br/
[<source>1913 Webster</source>]</p>

<p><sn>7.</sn> <def>A small packet of tobacco.</def> <mark>[Slang]</mark> <rj><au>Mayhew.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>8.</sn> <def>An unsound or worn-out horse, useful as a hack, and commonly of good appearance.</def> <rj><au>Ld. Lytton.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>9.</sn> <fld>(Math.)</fld> <def>A straight line in space with which a definite linear magnitude termed the <xex>pitch</xex> is associated (cf. 5th <er>Pitch</er>, 10 <sd>(b)</sd>). It is used to express the displacement of a rigid body, which may always be made to consist of a rotation about an axis combined with a translation parallel to that axis.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>10.</sn> <fld>(Zool.)</fld> <def>An amphipod crustacean; <as>as, the skeleton <ex>screw</ex> (<gen>Caprella</gen>)</as>. See <cref>Sand screw</cref>, under <er>Sand</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><mcol><col><b>Archimedes screw</b></col>, <col><b>Compound screw</b></col>, <col><b>Foot screw</b></col>, etc.</mcol> <cd>See under <er>Archimedes</er>, <er>Compound</er>, <er>Foot</er>, etc.</cd> -- <col><b>A screw loose</b></col>, <cd>something out of order, so that work is not done smoothly; <as>as, there is <ex>a screw loose</ex> somewhere</as>.</cd> <au>H. Martineau.</au> -- <mcol><col><b>Endless screw</b></col>, <it>or</it> <col><b>perpetual screw</b></col></mcol>, <cd>a screw used to give motion to a toothed wheel by the action of its threads between the teeth of the wheel; -- called also a <altname>worm</altname>.</cd> -- <col><b>Lag screw</b></col>. <cd>See under <er>Lag</er>.</cd> -- <col><b>Micrometer screw</b></col>, <cd>a screw with fine threads, used for the measurement of very small spaces.</cd> -- <col><b>Right and left screw</b></col>, <cd>a screw having threads upon the opposite ends which wind in opposite directions.</cd> -- <col><b>Screw alley</b></col>. <cd>See <er>Shaft alley</er>, under <er>Shaft</er>.</cd> -- <col><b>Screw bean</b></col>. <fld>(Bot.)</fld> <sd>(a)</sd> <cd>The curious spirally coiled pod of a leguminous tree (<spn>Prosopis pubescens</spn>) growing from Texas to California. It is used for fodder, and ground into meal by the Indians.</cd> <sd>(b)</sd> <cd>The tree itself. Its heavy hard wood is used for fuel, for fencing, and for railroad ties.</cd> -- <col><b>Screw bolt</b></col>, <cd>a bolt having a screw thread on its shank, in distinction from a <contr>key bolt</contr>. See 1st <er>Bolt</er>, 3.</cd> -- <col><b>Screw box</b></col>, <cd>a device, resembling a die, for cutting the thread on a wooden screw.</cd> -- <col><b>Screw dock</b></col>. <cd>See under <er>Dock</er>.</cd> -- <col><b>Screw engine</b></col>, <cd>a marine engine for driving a screw propeller.</cd> -- <col><b>Screw gear</b></col>. <cd>See <cref>Spiral gear</cref>, under <er>Spiral</er>.</cd> -- <col><b>Screw jack</b></col>. <cd>Same as <er>Jackscrew</er>.</cd> -- <col><b>Screw key</b></col>, <cd>a wrench for turning a screw or nut; a spanner wrench.</cd> -- <col><b>Screw machine</b></col>. <sd>(a)</sd> <cd>One of a series of machines employed in the manufacture of wood screws.</cd> <sd>(b)</sd> <cd>A machine tool resembling a lathe, having a number of cutting tools that can be caused to act on the work successively, for making screws and other turned pieces from metal rods.</cd> -- <col><b>Screw pine</b></col> <fld>(Bot.)</fld>, <cd>any plant of the endogenous genus <gen>Pandanus</gen>, of which there are about fifty species, natives of tropical lands from Africa to Polynesia; -- named from the spiral arrangement of the pineapple-like leaves.</cd> -- <col><b>Screw plate</b></col>, <cd>a device for cutting threads on small screws, consisting of a thin steel plate having a series of perforations with internal screws forming dies.</cd> -- <col><b>Screw press</b></col>, <cd>a press in which pressure is exerted by means of a screw.</cd> -- <col><b>Screw propeller</b></col>, <cd>a screw or spiral bladed wheel, used in the propulsion of steam vessels; also, a steam vessel propelled by a screw.</cd> -- <col><b>Screw shell</b></col> <fld>(Zool.)</fld>, <cd>a long, slender, spiral gastropod shell, especially of the genus Turritella and allied genera. See <er>Turritella</er>.</cd> -- <col><b>Screw steamer</b></col>, <cd>a steamship propelled by a screw.</cd> -- <col><b>Screw thread</b></col>, <cd>the spiral rib which forms a screw.</cd> -- <col><b>Screw stone</b></col> <fld>(Paleon.)</fld>, <cd>the fossil stem of an encrinite.</cd> -- <col><b>Screw tree</b></col> <fld>(Bot.)</fld>, <cd>any plant of the genus <gen>Helicteres</gen>, consisting of about thirty species of tropical shrubs, with simple leaves and spirally twisted, five-celled capsules; -- also called <altname>twisted-horn</altname>, and <altname>twisty</altname>.</cd> -- <col><b>Screw valve</b></col>, <cd>a stop valve which is opened or closed by a screw.</cd> -- <col><b>Screw worm</b></col> <fld>(Zool.)</fld>, <cd>the larva of an American fly (<spn>Compsomyia macellaria</spn>), allied to the blowflies, which sometimes deposits its eggs in the nostrils, or about wounds, in man and other animals, with fatal results.</cd> -- <col><b>Screw wrench</b></col>. <sd>(a)</sd> <cd>A wrench for turning a screw.</cd> <sd>(b)</sd> <cd>A wrench with an adjustable jaw that is moved by a screw.</cd> -- <mcol><col><b>To put the screws on</b></col> <it>or</it> <col><b>To put the screw on</b></col></mcol>, <cd>to use pressure upon, as for the purpose of extortion; to coerce.</cd> -- <mcol><col><b>To put under the screw</b></col> <it>or</it> <col><b>To put under the screws</b></col></mcol>, <cd>to subject to pressure; to force.</cd> -- <col><b>Wood screw</b></col>, <cd>a metal screw with a sharp thread of coarse pitch, adapted to holding fast in wood. See <xex>Illust.</xex> of <cref>Wood screw</cref>, under <er>Wood</er>.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Screw</ent><br/
<hw>Screw</hw> <pr>(skr<udd/)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Screwed</conjf> <pr>(skr<udd/d)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Screwing</conjf>.]</vmorph> <sn>1.</sn> <def>To turn, as a screw; to apply a screw to; to press, fasten, or make firm, by means of a screw or screws; <as>as, to <ex>screw</ex> a lock on a door; to <ex>screw</ex> a press</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To force; to squeeze; to press, as by screws.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>But <qex>screw</qex> your courage to the sticking place,<br/
And we'll not fail.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>Hence: To practice extortion upon; to oppress by unreasonable or extortionate exactions.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Our country landlords, by unmeasurable <qex>screwing</qex> and racking their tenants, have already reduced the miserable people to a worse condition than the peasants in France.</q> <rj><qau>swift.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>To twist; to distort; <as>as, to <ex>screw</ex> his visage</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>He screwed his face into a hardened smile.</q> <rj><qau>Dryden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <def>To examine rigidly, as a student; to subject to a severe examination.</def> <mark>[Cant, American Colleges]</mark><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>To screw out</b></col>, <cd>to press out; to extort.</cd> -- <col><b>To screw up</b></col>, <sd>(a)</sd> <cd>to force; to bring by violent pressure.</cd> <au>Howell.</au>  <sd>(b)</sd> <cd>to damage by unskillful effort; to bungle; to botch; to mess up; <as>as, he <ex>screwed up</ex> the contract negotiations, and we lost the deal</as>.</cd> <sd>(c)</sd> <mark>[intrans.]</mark> <cd>to fail by unskillful effort, usually causing unpleasant consequences.</cd>  -- <col><b>To screw in</b></col>, <cd>to force in by turning or twisting.</cd></cs> <col><b>Screw around</b></col>, <sd>(a)</sd> <cd>to act aimlessly or unproductively.</cd>  <sd>(b)</sd> <cd>to commit adultery; to be sexually promiscuous.</cd> --  <col><b>Screw around with</b></col>, <cd>to operate or make changes on (a machine or device) without expert knowledge; to fiddle with.</cd> <mark>[Colloq.]</mark>  . --><br/
[<source>1913 Webster</source>]</p>

<p><ent>Screw</ent><br/
<hw>Screw</hw>, <pos>v. i.</pos> <sn>1.</sn> <def>To use violent means in making exactions; to be oppressive or exacting.</def> <rj><au>Howitt.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To turn one's self uneasily with a twisting motion; <as>as, he <ex>screws</ex> about in his chair</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>screwball</ent><br/
<hw>screw"ball</hw>, <pos>n.</pos> <sn>1.</sn> <def>An eccentric or crazy person; an oddball.</def><br/
[<source>PJC</source>]</p>

<p><sn>2.</sn> <fld>(Baseball)</fld>  <def>A baseball pitch that curves in the direction opposite to that of a curve ball.</def><br/
[<source>PJC</source>]</p>

<p><ent>screwball</ent><br/
<hw>screw"ball</hw>, <pos>adj.</pos> <sn>1.</sn> <def>Eccentric; zany; crazy; <as>as, <ex>screwball</ex> antics; a <ex>screwball</ex> comedy</as>.</def><br/
[<source>PJC</source>]</p>

<p><sn>2.</sn> <def>Ridiculously unsound; improbable or doomed to failure; -- of plans or ideas; <as>as, a <ex>screwball</ex> plan to raise cocoanuts in Alaska</as>.</def><br/
[<source>PJC</source>]</p>

<p><ent>Screw-cutting</ent><br/
<hw>Screw"-cut`ting</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Adapted for forming a screw by cutting; <as>as, a <ex>screw-cutting</ex> lathe</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Screw-driver</ent><br/
<ent>Screwdriver</ent><br/
<mhw><hw>Screw"driv`er</hw>, <hw>Screw"-driv`er</hw></mhw> <pr>(?)</pr>, <pos>n.</pos> <def>A tool for turning screws so as to drive them into their place. It has a thin end which enters the nick in the head of the screw.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Screwer</ent><br/
<hw>Screw"er</hw> <pr>(?)</pr>, <pos>n.</pos> <def>One who, or that which, screws.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Screwing</ent><br/
<hw>Screw"ing</hw>, <def><pos>a. & n.</pos> from <er>Screw</er>, <pos>v. t.</pos></def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Screwing machine</b></col>. <cd>See <cref>Screw machine</cref>, under <er>Screw</er>.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scribable</ent><br/
<hw>Scrib"a*ble</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[See <er>Scribe</er>.]</ety> <def>Capable of being written, or of being written upon.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scribatious</ent><br/
<hw>Scri*ba"tious</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[See <er>Scribe</er>.]</ety> <def>Skillful in, or fond of, writing.</def> <mark>[Obs.]</mark> <rj><au>Barrow.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scribbet</ent><br/
<hw>Scrib"bet</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A painter's pencil.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scribble</ent><br/
<hw>Scrib"ble</hw> <pr>(?)</pr>, <pos>v. t.</pos> <ety>[Cf. <er>Scrabble</er>.]</ety> <fld>(Woolen Manuf.)</fld> <def>To card coarsely; to run through the scribbling machine.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scribble</ent><br/
<hw>Scrib"ble</hw>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Scribbled</conjf> <pr>(-b'ld)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Scribbling</conjf> <pr>(-bl<icr/ng)</pr>.]</vmorph> <ety>[From <er>Scribe</er>.]</ety> <sn>1.</sn> <def>To write hastily or carelessly, without regard to correctness or elegance; <as>as, to <ex>scribble</ex> a letter</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To fill or cover with careless or worthless writing.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scribble</ent><br/
<hw>Scrib"ble</hw>, <pos>v. i.</pos> <def>To write without care, elegance, or value; to scrawl.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>If Maevius <qex>scribble</qex> in Apollo's spite.</q> <rj><qau>Pope.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scribble</ent><br/
<hw>Scrib"ble</hw>, <pos>n.</pos> <def>Hasty or careless writing; a writing of little value; a scrawl; <as>as, a hasty <ex>scribble</ex></as>.</def> <rj><au>Boyle.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Neither did I but vacant seasons spend<br/
In this my <qex>scribble</qex>.</q> <rj><qau>Bunyan.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scribblement</ent><br/
<hw>Scrib"ble*ment</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A scribble.</def> <mark>[R.]</mark> <rj><au>Foster.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scribbler</ent><br/
<hw>Scrib"bler</hw> <pr>(?)</pr>, <pos>n.</pos> <def>One who scribbles; a petty author; a writer of no reputation; a literary hack.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The <qex>scribbler</qex>, pinched with hunger, writes to dine.</q> <rj><qau>Granville.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scribbler</ent><br/
<hw>Scrib"bler</hw>, <pos>n.</pos> <def>A scribbling machine.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scribbling</ent><br/
<hw>Scrib"bling</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[See 1st <er>Scribble</er>.]</ety> <def>The act or process of carding coarsely.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Scribbling machine</b></col>, <cd>the machine used for the first carding of wool or other fiber; -- called also <altname>scribbler</altname>.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scribbling</ent><br/
<hw>Scrib"bling</hw>, <pos>a.</pos> <def>Writing hastily or poorly.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Ye newspaper witlings! ye pert <qex>scribbling</qex> folks!</q> <rj><qau>Goldsmith.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scribbling</ent><br/
<hw>Scrib"bling</hw>, <pos>n.</pos> <def>The act of writing hastily or idly.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scribblingly</ent><br/
<hw>Scrib"bling*ly</hw>, <pos>adv.</pos> <def>In a scribbling manner.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scribe</ent><br/
<hw>Scribe</hw> <pr>(skr<imac/b)</pr>, <pos>n.</pos> <ety>[L. <ets>scriba</ets>, fr. <ets>scribere</ets> to write; cf. Gr. <grk>ska`rifos</grk> a splinter, pencil, style (for writing), E. <ets>scarify</ets>. Cf. <er>Ascribe</er>, <er>Describe</er>, <er>Script</er>, <er>Scrivener</er>, <er>Scrutoire</er>.]</ety> <sn>1.</sn> <def>One who writes; a draughtsman; a writer for another; especially, an offical or public writer; an amanuensis or secretary; a notary; a copyist.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Jewish Hist.)</fld> <def>A writer and doctor of the law; one skilled in the law and traditions; one who read and explained the law to the people.</def><br/
[<source>1913 Webster</source>]</p>

<p><-- p. 1293 pr=PCP --></p>

<p><ent>Scribe</ent><br/
<hw>Scribe</hw> <pr>(skr<imac/b)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Scribed</conjf> <pr>(skr<imac/bd)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Scribing</conjf>.]</vmorph> <sn>1.</sn> <def>To write, engrave, or mark upon; to inscribe.</def>  <rj><au>Spenser.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Carp.)</fld> <def>To cut (anything) in such a way as to fit closely to a somewhat irregular surface, as a baseboard to a floor which is out of level, a board to the curves of a molding, or the like; -- so called because the workman marks, or <xex>scribes</xex>, with the compasses the line that he afterwards cuts.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>To score or mark with compasses or a scribing iron.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Scribing iron</b></col>, <cd>an iron-pointed instrument for scribing, or marking, casks and logs.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scribe</ent><br/
<hw>Scribe</hw>, <pos>v. i.</pos> <def>To make a mark.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>With the separated points of a pair of spring dividers <qex>scribe</qex> around the edge of the templet.</q> <rj><qau>A. M. Mayer.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scriber</ent><br/
<hw>Scrib"er</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A sharp-pointed tool, used by joiners for drawing lines on stuff; a marking awl.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scribism</ent><br/
<hw>Scrib"ism</hw> <pr>(?)</pr>, <pos>n.</pos> <def>The character and opinions of a Jewish scribe in the time of Christ.</def>  <rj><au>F. W. Robertson.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scrid</ent><br/
<hw>Scrid</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A screed; a shred; a fragment.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scriggle</ent><br/
<hw>Scrig"gle</hw> <pr>(?)</pr>, <pos>v. i.</pos> <def>To wriggle.</def> <mark>[Prov. Eng.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scrim</ent><br/
<hw>Scrim</hw> <pr>(?)</pr>, <pos>n.</pos> <sn>1.</sn> <def>A kind of light cotton or linen fabric, often woven in openwork patterns, -- used for curtains, etc,; -- called also <altname>India scrim</altname>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <pluf>pl.</pluf> <def>Thin canvas glued on the inside of panels to prevent shrinking, checking, etc.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scrimer</ent><br/
<hw>Scri"mer</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F. <ets>escrimeur</ets>. See <er>Skirmish</er>.]</ety> <def>A fencing master.</def> <mark>[Obs.]</mark>  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scrimmage</ent><br/
<hw>Scrim"mage</hw> <pr>(?; 48)</pr>, <pos>n.</pos> <ety>[A corruption of <ets>skirmish</ets>.  <ldquo/Sore <xex>scrymmishe</xex>.<rdquo/ <au>Ld. Berners.</au>]</ety> <altsp>[Written also <asp>scrummage</asp>.]</altsp> <sn>1.</sn> <def>Formerly, a skirmish; now, a general row or confused fight or struggle.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Football)</fld> <def>The struggle in the rush lines after the ball is put in play.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scrimp</ent><br/
<hw>Scrimp</hw> <pr>(?)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Scrimped</conjf> <pr>(?; 215)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Scrimping</conjf>.]</vmorph> <ety>[Cf. Dan. <ets>skrumpe</ets>, G. <ets>schrumpfen</ets>, D. <ets>krimpen</ets>.  Cf. <er>Shrimp</er>, <er>Shrink</er>.]</ety> <def>To make too small or short; to limit or straiten; to put on short allowance; to scant; to contract; to shorten; <as>as, to <ex>scrimp</ex> the pattern of a coat</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><-- To economize.
  Scrimp and save.  Economize and save the money not spent. --><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scrimp</ent><br/
<hw>Scrimp</hw>, <pos>a.</pos> <def>Short; scanty; curtailed.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scrimp</ent><br/
<hw>Scrimp</hw>, <pos>n.</pos> <def>A pinching miser; a niggard.</def> <mark>[U.S.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scrimping</ent><br/
<hw>Scrimp"ing</hw>, <def><pos>a. & n.</pos> from <er>Scrimp</er>, <pos>v. t.</pos></def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Scrimping bar</b></col>, <cd>a device used in connection with a calico printing machine for stretching the fabric breadthwise so that it may be smooth for printing.</cd>  <rj><au>Knight.</au></rj></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scrimpingly</ent><br/
<hw>Scrimp"ing*ly</hw>, <pos>adv.</pos> <def>In a scrimping manner.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scrimpness</ent><br/
<hw>Scrimp"ness</hw>, <pos>n.</pos> <def>The state of being scrimp.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scrimption</ent><br/
<hw>Scrimp"tion</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A small portion; a pittance; a little bit.</def> <mark>[Prov. Eng.]</mark>  <rj><au>Halliwell.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scrimshaw</ent><br/
<hw>Scrim"shaw`</hw> <pr>(?)</pr>, <pos>v. t.</pos> <def>To ornament, as shells, ivory, etc., by engraving, and (usually) rubbing pigments into the incised lines.</def> <mark>[Sailor's cant. U.S.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scrimshaw</ent><br/
<hw>Scrim"shaw`</hw>, <pos>n.</pos> <def>A shell, a whale's tooth, or the like, that is scrimshawed.</def> <mark>[Sailor's cant, U.S.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scrine</ent><br/
<hw>Scrine</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>scrinium</ets> a case for books, letters, etc.: cf. OF. <ets>escrin</ets>, F. <ets><eacute/crin</ets>. See <er>Shrine</er>.]</ety> <def>A chest, bookcase, or other place, where writings or curiosities are deposited; a shrine.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>But laid them up in immortal <qex>scrine</qex>.</q> <rj><qau>Spenser.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scringe</ent><br/
<hw>Scringe</hw> <pr>(?)</pr>, <pos>v. i.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Scringed</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Scringing</conjf> <pr>(?)</pr>.]</vmorph> <ety>[Cf. <er>Cringe</er>.]</ety> <def>To cringe.</def> <mark>[Prov. Eng. & Local, U.S.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scrip</ent><br/
<hw>Scrip</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[OE. <ets>scrippe</ets>, probably of Scand. origin; cf. Icel. & OSw. <ets>skreppa</ets>, and also LL. <ets>scrippum</ets>, OF. <ets>esquerpe</ets>, <ets>escrepe</ets>, F. <ets><eacute/charpe</ets> scarf.  Cf. <er>Scrap</er>, <er>Scarf</er> a piece of dress.]</ety> <def>A small bag; a wallet; a satchel.</def> <mark>[Archaic]</mark>  <rj><au>Chaucer.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>And in requital ope his leathern <qex>scrip</qex>.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scrip</ent><br/
<hw>Scrip</hw>, <pos>n.</pos> <ety>[From <ets>script</ets>.]</ety> <sn>1.</sn> <def>A small writing, certificate, or schedule; a piece of paper containing a writing.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Call them generally, man by man, according to the <qex>scrip</qex>.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Bills of exchange can not pay our debts abroad, till <qex>scrips</qex> of paper can be made current coin.</q> <rj><qau>Locke.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A preliminary certificate of a subscription to the capital of a bank, railroad, or other company, or for a share of other joint property, or a loan, stating the amount of the subscription and the date of the payment of the installments; <as>as, insurance <ex>scrip</ex>, consol <ex>scrip</ex>, etc.</as>  When all the installments are paid, the scrip is exchanged for a bond share certificate.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>Paper fractional currency.</def> <mark>[Colloq.U.S.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scrippage</ent><br/
<hw>Scrip"page</hw> <pr>(?; 48)</pr>, <pos>n.</pos> <def>The contents of a scrip, or wallet.</def> <mark>[Obs.]</mark>  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Script</ent><br/
<hw>Script</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[OE. <ets>scrit</ets>, L. <ets>scriptum</ets> something written, fr. <ets>scribere</ets>, <ets>scriptum</ets> to write: cf. OF. <ets>escript</ets>, <ets>escrit</ets>, F. <ets><eacute/crit</ets>. See <er>Scribe</er>, and cf. <er>Scrip</er> a writing.]</ety> <sn>1.</sn> <def>A writing; a written document.</def> <mark>[Obs.]</mark>  <rj><au>Chaucer.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Print.)</fld> <def>Type made in imitation of handwriting.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <fld>(Law)</fld> <def>An original instrument or document.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>Written characters; style of writing.</def><br/
[<source>1913 Webster</source>]</p>

<p><-- 5. The written document containing the dialogue and action for a drama; the text of a stage play, movie, or other performance.  Especially, the final form used for the performance itself. --><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scriptorium</ent><br/
<hw>Scrip*to"ri*um</hw> <pr>(?)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Scriptoria</plw> <pr>(#)</pr>.</plu> <ety>[LL. See <er>Scriptory</er>.]</ety> <def>In an abbey or monastery, the room set apart for writing or copying manuscripts; in general, a room devoted to writing.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Writing rooms, or <qex>scriptoria</qex>, where the chief works of Latin literature . . . were copied and illuminated.</q> <rj><qau>J. R. Green.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scriptory</ent><br/
<hw>Scrip"to*ry</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>scriptorius</ets>, fr. <ets>scribere</ets>, <ets>scriptum</ets> to write.]</ety> <def>Of or pertaining to writing; expressed in writing; used in writing; <as>as, <ex>scriptory</ex> wills; a <ex>scriptory</ex> reed</as>.</def> <mark>[R.]</mark>  <rj><au>Swift.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scriptural</ent><br/
<hw>Scrip"tur*al</hw> <pr>(?; 135)</pr>, <pos>a.</pos> <def>Contained in the Scriptures; according to the Scriptures, or sacred oracles; biblical; <as>as, a <ex>scriptural</ex> doctrine</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scripturalism</ent><br/
<hw>Scrip"tur*al*ism</hw> <pr>(?)</pr>, <pos>n.</pos> <def>The quality or state of being scriptural; literal adherence to the Scriptures.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scripturalist</ent><br/
<hw>Scrip"tur*al*ist</hw>, <pos>n.</pos> <def>One who adheres literally to the Scriptures.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scripturally</ent><br/
<hw>Scrip"tur*al*ly</hw>, <pos>adv.</pos> <def>In a scriptural manner.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scripturalness</ent><br/
<hw>Scrip"tur*al*ness</hw>, <pos>n.</pos> <def>Quality of being scriptural.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scripture</ent><br/
<hw>Scrip"ture</hw> <pr>(?; 135)</pr>, <pos>n.</pos> <ety>[L. <ets>scriptura</ets>, fr. <ets>scribere</ets>, <ets>scriptum</ets>, to write: cf. OF. <ets>escripture</ets>, <ets>escriture</ets>, F. <ets><eacute/criture</ets>. See <er>Scribe</er>.]</ety> <sn>1.</sn> <def>Anything written; a writing; a document; an inscription.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>I have put it in <qex>scripture</qex> and in remembrance.</q> <rj><qau>Chaucer.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Then the Lord of Manny read the <qex>scripture</qex> on the tomb, the which was in Latin.</q> <rj><qau>Ld. Berners.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>The books of the Old and the New Testament, or of either of them; the Bible; -- used by way of eminence or distinction, and chiefly in the plural.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>There is not any action a man ought to do, or to forbear, but the <qex>Scripture</qex> will give him a clear precept or prohibition for it.</q> <rj><qau>South.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Compared with the knowledge which the <qex>Scriptures</qex> contain, every other subject of human inquiry is vanity.</q> <rj><qau>Buckminster.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>A passage from the Bible; a text.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The devil can cite <qex>Scripture</qex> for his purpose.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Hanging by the twined thread of one doubtful <qex>Scripture</qex>.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scripturian</ent><br/
<hw>Scrip*tu"ri*an</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A Scripturist.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scripturist</ent><br/
<hw>Scrip"tur*ist</hw> <pr>(?; 135)</pr>, <pos>n.</pos> <def>One who is strongly attached to, or versed in, the Scriptures, or who endeavors to regulate his life by them.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The Puritan was a <qex>Scripturist</qex>, -- a <qex>Scripturist</qex> with all his heart, if as yet with imperfect intelligence . . . he cherished the scheme of looking to the Word of God as his sole and universal directory.</q> <rj><qau>Palfrey.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scrit</ent><br/
<hw>Scrit</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[See <er>Script</er>.]</ety> <def>Writing; document; scroll.</def> <mark>[Obs.]</mark> <ldquo/Of every <xex>scrit</xex> and bond.<rdquo/  <rj><au>Chaucer.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scritch</ent><br/
<hw>Scritch</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A screech.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>Perhaps it is the owlet's <qex>scritch</qex>.</q> <rj><qau>Coleridge.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scrivener</ent><br/
<hw>Scrive"ner</hw> <pr>(? <or/ ?)</pr>, <pos>n.</pos> <ety>[From older <ets>scrivein</ets>, OF. <ets>escrivain</ets>, F. <ets><eacute/crivain</ets>, LL. <ets>scribanus</ets>, from L. <ets>scribere</ets> to write. See <er>Scribe</er>.]</ety> <sn>1.</sn> <def>A professional writer; one whose occupation is to draw contracts or prepare writings.</def>  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>The writer better <qex>scrivener</qex> than clerk.</q> <rj><qau>Fuller.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>One whose business is to place money at interest; a broker.</def> <mark>[Obs.]</mark>  <rj><au>Dryden.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>A writing master.</def> <mark>[Prov. Eng.]</mark>  <rj><au>Halliwell.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Scrivener's palsy</b></col>. <cd>See <cref>Writer's cramp</cref>, under <er>Writer</er>.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scrobicula</ent><br/
||<hw>Scro*bic"u*la</hw> <pr>(?)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Scrobiculae</plw> <pr>(#)</pr>.</plu> <ety>[NL. See <er>Scrobiculate</er>.]</ety> <fld>(Zool.)</fld> <def>One of the smooth areas surrounding the tubercles of a sea urchin.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scrobicular</ent><br/
<hw>Scro*bic"u*lar</hw> <pr>(?)</pr>, <pos>a.</pos> <fld>(Zool.)</fld> <def>Pertaining to, or surrounding, scrobiculae; <as>as, <ex>scrobicular</ex> tubercles</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scrobiculated</ent><br/
<ent>Scrobiculate</ent><br/
<mhw>{ <hw>Scro*bic"u*late</hw> <pr>(?)</pr>, <hw>Scro*bic"u*la`ted</hw> <pr>(?)</pr>, }</mhw> <pos>a.</pos> <ety>[L. <ets>scrobiculus</ets>, dim. of <ets>scrobis</ets> a ditch or trench.]</ety> <fld>(Bot.)</fld> <def>Having numerous small, shallow depressions or hollows; pitted.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scrode</ent><br/
<ent>Scrod</ent><br/
<mhw>{ <hw>Scrod</hw> <pr>(?)</pr>, <hw>Scrode</hw> <pr>(?)</pr>, }</mhw> <pos>n.</pos> <def>A young codfish, especially when cut open on the back and dressed.</def> <altsp>[Written also <asp>escrod</asp>.]</altsp> <mark>[Local, U.S.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scroddled ware</ent><br/
<hw>Scrod"dled ware`</hw> <pr>(?)</pr>. <def>Mottled pottery made from scraps of differently colored clays.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scrofula</ent><br/
<hw>Scrof"u*la</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>scrofulae</ets>, fr. <ets>scrofa</ets> a breeding sow, because swine were supposed to be subject to such a complaint, or by a fanciful comparison of the glandular swellings to little pigs; perhaps akin to Gr. <?/ an old sow: cf. F. <ets>scrofules</ets>.  Cf. <er>Scroyle</er>.]</ety> <fld>(Med.)</fld> <def>A constitutional disease, generally hereditary, especially manifested by chronic enlargement and cheesy degeneration of the lymphatic glands, particularly those of the neck, and marked by a tendency to the development of chronic intractable inflammations of the skin, mucous membrane, bones, joints, and other parts, and by a diminution in the power of resistance to disease or injury and the capacity for recovery. Scrofula is now generally held to be tuberculous in character, and may develop into general or local tuberculosis (consumption).</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scrofulide</ent><br/
<hw>Scrof"u*lide</hw> <pr>(? <or/ ?)</pr>, <pos>n.</pos> <fld>(Med.)</fld> <def>Any affection of the skin dependent on scrofula.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scrofulous</ent><br/
<hw>Scrof"u*lous</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[Cf. F. <ets>scrofuleux</ets>.]</ety> <sn>1.</sn> <def>Pertaining to scrofula, or partaking of its nature; <as>as, <ex>scrofulous</ex> tumors; a <ex>scrofulous</ex> habit of body</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Diseased or affected with scrofula.</def><br/
[<source>1913 Webster</source>]</p>

<p><q><qex>Scrofulous</qex> persons can never be duly nourished.</q> <rj><qau>Arbuthnot.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p>-- <wordforms><wf>Scrof"u*lous*ly</wf>, <pos>adv.</pos> -- <wf>Scrof"u*lous*ness</wf>, <pos>n.</pos></wordforms><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scrog</ent><br/
<hw>Scrog</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Cf. <er>Scrag</er>, or Gael. <ets>sgrogag</ets> anything shriveled, from <ets>sgrog</ets> to compress, shrivel.]</ety> <def>A stunted shrub, bush, or branch.</def> <mark>[Prov. Eng. & Scot.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scroggy</ent><br/
<hw>Scrog"gy</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Abounding in scrog; also, twisted; stunted.</def> <mark>[Prov. Eng. & Scot.]</mark>  <rj><au>Halliwell.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scroll</ent><br/
<hw>Scroll</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[A dim. of OE. <ets>scroue</ets>, <ets>scrowe</ets> (whence E. <ets>escrow</ets>), OF. <ets>escroe</ets>, <ets>escroue</ets>, F. <ets><eacute/crou</ets> entry in the jail book, LL. <ets>scroa</ets> scroll, probably of Teutonic origin; cf. OD. <ets>schroode</ets> a strip, shred, slip of paper, akin to E. <ets>shred</ets>.  Cf. <er>Shred</er>, <er>Escrow</er>.]</ety> <sn>1.</sn> <def>A roll of paper or parchment; a writing formed into a roll; a schedule; a list.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The heavens shall be rolled together as a <qex>scroll</qex>.</q> <rj><qau>Isa. xxxiv. 4.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Here is the <qex>scroll</qex> of every man's name.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Arch.)</fld> <def>An ornament formed of undulations giving off spirals or sprays, usually suggestive of plant form. Roman architectural ornament is largely of some scroll pattern.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>A mark or flourish added to a person's signature, intended to represent a seal, and in some States allowed as a substitute for a seal.</def> <mark>[U.S.]</mark>  <rj><au>Burrill.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <fld>(Geom.)</fld> <def>Same as <cref>Skew surface</cref>. See under <er>Skew</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Linen scroll</b></col> <fld>(Arch.)</fld> <cd>See under <er>Linen</er>.</cd> -- <col><b>Scroll chuck</b></col> <fld>(Mach.)</fld>, <cd>an adjustable chuck, applicable to a lathe spindle, for centering and holding work, in which the jaws are adjusted and tightened simultaneously by turning a disk having in its face a spiral groove which is entered by teeth on the backs of the jaws.</cd> -- <col><b>Scroll saw</b></col>. <cd>See under <er>Saw</er>.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scrolled</ent><br/
<hw>Scrolled</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Formed like a scroll; contained in a scroll; adorned with scrolls; <as>as, <ex>scrolled</ex> work</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scrophularia</ent><br/
||<hw>Scroph`u*la"ri*a</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[NL. So called because it was reputed to be a remedy for <ets>scrofula</ets>.]</ety> <fld>(Bot.)</fld> <def>A genus of coarse herbs having small flowers in panicled cymes; figwort.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scrophulariaceous</ent><br/
<hw>Scroph`u*la`ri*a"ceous</hw> <pr>(?)</pr>, <pos>a.</pos> <fld>(Bot.)</fld> <def>Of or pertaining to a very large natural order of gamopetalous plants (<ord>Scrophulariaceae</ord>, or <ord>Scrophularineae</ord>), usually having irregular didynamous flowers and a two-celled pod. The order includes the mullein, foxglove, snapdragon, figwort, painted cup, yellow rattle, and some exotic trees, as the <xex>Paulownia</xex>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scrotal</ent><br/
<hw>Scro"tal</hw> <pr>(?)</pr>, <pos>a.</pos> <fld>(Anat.)</fld> <def>Of or pertaining to the scrotum; <as>as, <ex>scrotal</ex> hernia</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scrotiform</ent><br/
<hw>Scro"ti*form</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>scrotum</ets> scrotum + <ets>-form</ets>.]</ety> <def>Purse-shaped; pouch-shaped.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scrotocele</ent><br/
<hw>Scro"to*cele</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[<ets>Scrotum</ets> + Gr. <grk>kh`lh</grk> a tumor: cf. F. <ets>scrotoc<egrave/le</ets>.]</ety> <fld>(Med.)</fld> <def>A rupture or hernia in the scrotum; scrotal hernia.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scrotum</ent><br/
||<hw>Scro"tum</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L.]</ety> <fld>(Anat.)</fld> <def>The bag or pouch which contains the testicles; the cod.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scrouge</ent><br/
<hw>Scrouge</hw> <pr>(?)</pr>, <pos>v. t.</pos> <ety>[Etymol. uncertain.]</ety> <def>To crowd; to squeeze.</def> <mark>[Prov. Eng. & Colloq. U.S.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scrow</ent><br/
<hw>Scrow</hw> <pr>(? <or/ ?)</pr>, <pos>n.</pos> <ety>[See <er>Escrow</er>, <er>Scroll</er>.]</ety> <sn>1.</sn> <def>A scroll.</def> <mark>[Obs.]</mark>  <rj><au>Palsgrave.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A clipping from skins; a currier's cuttings.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scroyle</ent><br/
<hw>Scroyle</hw> <pr>(skroil)</pr>, <pos>n.</pos> <ety>[Cf. OF. <ets>escrouselle</ets> a kind of vermin, <ets>escrouelles</ets>, pl., scrofula, F. <ets><eacute/crouelles</ets>, fr. (assumed) LL. <ets>scrofellae</ets> for L. <ets>scrofulae</ets>. See <er>Scrofula</er>, and cf. <er>Cruels</er>.]</ety> <def>A mean fellow; a wretch.</def> <mark>[Obs.]</mark>  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scrub</ent><br/
<hw>Scrub</hw> <pr>(skr<ucr/b)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Scrubbed</conjf> <pr>(skr<ucr/bd)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Scrubbing</conjf>.]</vmorph> <ety>[OE. <ets>scrobben</ets>, probably of Dutch or Scand. origin; cf. Dan. <ets>skrubbe</ets>, Sw. <ets>skrubba</ets>, D. <ets>schrobben</ets>, LG. <ets>schrubben</ets>.]</ety> <def>To rub hard; to wash with rubbing; usually, to rub with a wet brush, or with something coarse or rough, for the purpose of cleaning or brightening; <as>as, to <ex>scrub</ex> a floor, a doorplate</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scrub</ent><br/
<hw>Scrub</hw> <pr>(skr<ucr/b)</pr>, <pos>v. i.</pos> <def>To rub anything hard, especially with a wet brush; to scour; hence, to be diligent and penurious; <as>as, to <ex>scrub</ex> hard for a living</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scrub</ent><br/
<hw>Scrub</hw> <pr>(skr<ucr/b)</pr>, <pos>n.</pos> <sn>1.</sn> <def>One who labors hard and lives meanly; a mean fellow.</def> <ldquo/A sorry <xex>scrub</xex>.<rdquo/  <rj><au>Bunyan.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>We should go there in as proper a manner as possible; nor altogether like the <qex>scrubs</qex> about us.</q> <rj><qau>Goldsmith.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Something small and mean.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>A worn-out brush.</def>  <rj><au>Ainsworth.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>A thicket or jungle, often specified by the name of the prevailing plant; <as>as, oak <ex>scrub</ex>, palmetto <ex>scrub</ex>, etc.</as></def><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <fld>(Stock Breeding)</fld> <def>One of the common live stock of a region of no particular breed or not of pure breed, esp. when inferior in size, etc.</def> <mark>[U.S.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><sn>6.</sn> <def>Vegetation of inferior quality, though sometimes thick and impenetrable, growing in poor soil or in sand; also, brush; -- called also <altname>scrub brush</altname>. See <er>Brush</er>, above.</def> <mark>[Australia & South Africa]</mark><br/
[<source>Webster 1913 Suppl.</source> <source>+PJC</source>]</p>

<p><sn>7.</sn>  <fld>(Forestry)</fld> <def>A low, straggling tree of inferior quality.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><cs><col><b>Scrub bird</b></col> <fld>(Zool.)</fld>, <cd>an Australian passerine bird of the family <fam>Atrichornithidae</fam>, as <spn>Atrichia clamosa</spn>; -- called also <altname>brush bird</altname>.</cd> -- <col><b>Scrub oak</b></col> <fld>(Bot.)</fld>, <cd>the popular name of several dwarfish species of oak. The scrub oak of New England and the Middle States is <spn>Quercus ilicifolia</spn>, a scraggy shrub; that of the Southern States is a small tree (<spn>Quercus Catesbaei</spn>); that of the Rocky Mountain region is <spn>Quercus undulata</spn>, var. <varn>Gambelii</varn>.</cd> -- <col><b>Scrub robin</b></col> <fld>(Zool.)</fld>, <cd>an Australian singing bird of the genus <gen>Drymodes</gen>.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scrub</ent><br/
<hw>Scrub</hw> <pr>(skr<ucr/b)</pr>, <pos>a.</pos> <def>Mean; dirty; contemptible; scrubby.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>How solitary, how <qex>scrub</qex>, does this town look!</q> <rj><qau>Walpole.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>No little <qex>scrub</qex> joint shall come on my board.</q> <rj><qau>Swift.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Scrub game</b></col>, <cd>a game, as of ball, by unpracticed players.</cd> -- <col><b>Scrub race</b></col>, <cd>a race between scrubs, or between untrained animals or contestants.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scrubbed</ent><br/
<hw>Scrub"bed</hw> <pr>(skr<ucr/b"b<ecr/d)</pr>, <pos>a.</pos> <def>Dwarfed or stunted; scrubby.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scrubber</ent><br/
<hw>Scrub"ber</hw> <pr>(skr<ucr/b"b<etil/r)</pr>, <pos>n.</pos> <sn>1.</sn> <def>One who, or that which, scrubs; esp., a brush or machine used in scrubbing.</def><br/
[<source>1913 Webster</source> <source>+PJC</source>]</p>

<p><sn>2.</sn> <fld>(Gas Manuf.)</fld> <def>A gas washer. See under <er>Gas</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <fld>(Manufacturing)</fld> <def>a device for removing pollutants from a gas stream, especially for removing sulfur oxides from processes burning coal or oil.</def><br/
[<source>PJC</source>]</p>

<p><ent>Scrubber</ent><br/
<hw>Scrub"ber</hw> <pr>(skr<ucr/b"b<etil/r)</pr>, <pos>n.</pos> <sn>1.</sn> <def>a stunted or emaciated steer.</def><br/
[<source>PJC</source>]</p>

<p><sn>2.</sn> <def>A person who lives in the bush.</def> <mark>[Australian]</mark><br/
[<source>PJC</source>]</p>

<p><sn>3.</sn> <def>A domesticated animal which has escaped and lives wild in the bush.</def> <mark>[Australian]</mark><br/
[<source>PJC</source>]</p>

<p><ent>Scrubboard</ent><br/
<hw>Scrub"board`</hw> <pr>(skr<ucr/b"b<omac/rd)</pr>, <pos>n.</pos> <def>A baseboard; a mopboard.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scrubby</ent><br/
<hw>Scrub"by</hw> <pr>(skr<ucr/b"b<ycr/)</pr>, <pos>a.</pos> <amorph>[<pos>Compar.</pos> <adjf>Scrubbier</adjf> <pr>(skr<ucr/b"b<icr/*<etil/r)</pr>; <pos>superl.</pos> <adjf>Scrubbiest</adjf>.]</amorph> <def>Of the nature of scrub; small and mean; stunted in growth; <as>as, a <ex>scrubby</ex> cur</as>.</def> <ldquo/Dense, <xex>scrubby</xex> woods.<rdquo/  <rj><au>Duke of Argyll.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scrubstone</ent><br/
<hw>Scrub"stone`</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A species of calciferous sandstone.</def> <mark>[Prov. Eng.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scruff</ent><br/
<hw>Scruff</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[See <er>Scurf</er>.]</ety> <def>Scurf.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scruff</ent><br/
<hw>Scruff</hw>, <pos>n.</pos> <ety>[Cf. <er>Scuff</er>.]</ety> <def>The nape of the neck; the loose outside skin, as of the back of the neck.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scrummage</ent><br/
<hw>Scrum"mage</hw> <pr>(?; 43)</pr>, <pos>n.</pos> <def>See <er>Scrimmage</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scrumptious</ent><br/
<hw>Scrump"tious</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Nice; particular; fastidious; excellent; fine.</def> <mark>[Slang]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scrunch</ent><br/
<hw>Scrunch</hw> <pr>(?)</pr>, <pos>v. t. & v. i.</pos> <ety>[Cf. <er>Scranch</er>, <er>Crunch</er>.]</ety> <def>To scranch; to crunch.</def>  <rj><au>Dickens.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scruple</ent><br/
<hw>Scru"ple</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>scrupulus</ets> a small sharp or pointed stone, the twenty-fourth part of an ounce, a scruple, uneasiness, doubt, dim. of <ets>scrupus</ets> a rough or sharp stone, anxiety, uneasiness; perh. akin to Gr. <?/ the chippings of stone, <?/ a razor, Skr. <ets>kshura</ets>: cf. F. <ets>scrupule</ets>.]</ety> <sn>1.</sn> <def>A weight of twenty grains; the third part of a dram.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Hence, a very small quantity; a particle.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>I will not bate thee a <qex>scruple</qex>.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>Hesitation as to action from the difficulty of determining what is right or expedient; unwillingness, doubt, or hesitation proceeding from motives of conscience.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>He was made miserable by the conflict between his tastes and his <qex>scruples</qex>.</q> <rj><qau>Macaulay.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>To make scruple</b></col>, <cd>to hesitate from conscientious motives; to scruple.</cd> <qau>Locke.</qau></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scruple</ent><br/
<hw>Scru"ple</hw>, <pos>v. i.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Scrupled</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Scrupling</conjf> <pr>(?)</pr>.]</vmorph> <def>To be reluctant or to hesitate, as regards an action, on account of considerations of conscience or expedience.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>We are often over-precise, scrupling to say or do those things which lawfully we may.</q> <rj><qau>Fuller.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Men <qex>scruple</qex> at the lawfulness of a set form of divine worship.</q> <rj><qau>South.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scruple</ent><br/
<hw>Scru"ple</hw>, <pos>v. t.</pos> <sn>1.</sn> <def>To regard with suspicion; to hesitate at; to question.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Others long before them . . . <qex>scrupled</qex> more the books of heretics than of gentiles.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To excite scruples in; to cause to scruple.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>Letters which did still <qex>scruple</qex> many of them.</q> <rj><qau>E. Symmons.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scrupler</ent><br/
<hw>Scru"pler</hw> <pr>(?)</pr>, <pos>n.</pos> <def>One who scruples.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scrupulist</ent><br/
<hw>Scru"pu*list</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A scrupler.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scrupulize</ent><br/
<hw>Scru"pu*lize</hw> <pr>(?)</pr>, <pos>v. t.</pos> <def>To perplex with scruples; to regard with scruples.</def> <mark>[Obs.]</mark>  <rj><au>Bp. Montagu.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><-- p. 1294 pr=PCP --></p>

<p><ent>Scrupulosity</ent><br/
<hw>Scru`pu*los"i*ty</hw> <pr>(skr<udd/`p<usl/*l<ocr/s"<icr/*t<ycr/)</pr>, <pos>n.</pos> <ety>[L. <ets>scrupulositas</ets>.]</ety> <def>The quality or state of being scrupulous; doubt; doubtfulness respecting decision or action; caution or tenderness from the fear of doing wrong or offending; nice regard to exactness and propriety; precision.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The first sacrilege is looked on with horror; but when they have made the breach, their <qex>scrupulosity</qex> soon retires.</q> <rj><qau>Dr. H. More.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Careful, even to <qex>scrupulosity</qex>, . . . to keep their Sabbath.</q> <rj><qau>South.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scrupulous</ent><br/
<hw>Scru"pu*lous</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>scrupulosus</ets>: cf. F. <ets>scrupuleux</ets>.]</ety> <sn>1.</sn> <def>Full of scruples; inclined to scruple; nicely doubtful; hesitating to determine or to act, from a fear of offending or of doing wrong.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Abusing their liberty, to the offense of their weak brethren which were <qex>scrupulous</qex>.</q> <rj><qau>Hooker.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Careful; cautious; exact; nice; <as>as, <ex>scrupulous</ex> abstinence from labor; <ex>scrupulous</ex> performance of duties</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>Given to making objections; captious.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>Equality of two domestic powers<br/
Breed <qex>scrupulous</qex> faction.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>Liable to be doubted; doubtful; nice.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>The justice of that cause ought to be evident; not obscure, not <qex>scrupulous</qex>.</q> <rj><qau>Bacon.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><syn><b>Syn.</b> -- Cautious; careful; conscientious; hesitating.</syn><br/
[<source>1913 Webster</source>]</p>

<p> -- <wordforms><wf>Scru"pu*lous*ly</wf>, <pos>adv.</pos> -- <wf>Scru"pu*lous*ness</wf>, <pos>n.</pos></wordforms><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scrutable</ent><br/
<hw>Scru"ta*ble</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Discoverable by scrutiny, inquiry, or critical examination.</def> <mark>[R.]</mark>  <rj><au>Dr. H. More.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scrutation</ent><br/
<hw>Scru*ta"tion</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>scrutatio</ets>.]</ety> <def>Search; scrutiny.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scrutator</ent><br/
||<hw>Scru*ta"tor</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L.]</ety> <def>One who scrutinizes; a close examiner or inquirer.</def>  <rj><au>Ayliffe.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scrutin de liste</ent><br/
||<hw>Scru`tin" de liste"</hw> <pr>(skr<usdot/`t<acr/N" d<etil/ l<emac/st)</pr>. <ety>[F., voting by list.]</ety> <def>Voting for a group of candidates for the same kind of office on one ticket or ballot, containing a list of them; -- the method, used in France, as from June, 1885, to Feb., 1889, in elections for the Chamber of Deputies, each elector voting for the candidates for the whole department in which he lived, as disting. from <col><b>scrutin d'arrondissement</b></col> <pr>(d<adot/`r<ocir/N`d<emac/s`m<aum/N")</pr>, or voting by each elector for the candidate or candidates for his own arrondissement only.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Scrutineer</ent><br/
<hw>Scru`ti*neer</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A scrutinizer; specifically, an examiner of votes, as at an election.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scrutinize</ent><br/
<hw>Scru"ti*nize</hw> <pr>(?)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Scrutinized</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Scrutinizing</conjf> <pr>(?)</pr>.]</vmorph> <ety>[From <er>Scrutiny</er>.]</ety> <def>To examine closely; to inspect or observe with critical attention; to regard narrowly; <as>as, to <ex>scrutinize</ex> the measures of administration; to <ex>scrutinize</ex> the conduct or motives of individuals</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Whose votes they were obliged to <qex>scrutinize</qex>.</q> <rj><qau>Ayliffe.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Those pronounced him youngest who <qex>scrutinized</qex> his face the closest.</q> <rj><qau>G. W. Cable.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scrutinize</ent><br/
<hw>Scru"ti*nize</hw>, <pos>v. i.</pos> <def>To make scrutiny.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scrutinizer</ent><br/
<hw>Scru"ti*ni`zer</hw> <pr>(?)</pr>, <pos>n.</pos> <def>One who scrutinizes.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scrutinous</ent><br/
<hw>Scru"ti*nous</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Closely examining, or inquiring; careful; strict.</def> -- <wordforms><wf>Scru"ti*nous*ly</wf>, <pos>adv.</pos></wordforms><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scrutiny</ent><br/
<hw>Scru"ti*ny</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>scrutinium</ets>, fr. <ets>scrutari</ets> to search carefully, originally, to search even to the rags, fr. <ets>scruta</ets> trash, trumpery; perhaps akin to E. <ets>shred</ets>: cf. AS. <ets>scrudnian</ets> to make scrutiny.]</ety> <sn>1.</sn> <def>Close examination; minute inspection; critical observation.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>They that have designed exactness and deep <qex>scrutiny</qex> have taken some one part of nature.</q> <rj><qau>Sir M. Hale.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Thenceforth I thought thee worth my nearer view<br/
And narrower <qex>scrutiny</qex>.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Anc. Church)</fld> <def>An examination of catechumens, in the last week of Lent, who were to receive baptism on Easter Day.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <fld>(Canon Law)</fld> <def>A ticket, or little paper billet, on which a vote is written.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <fld>(Parliamentary Practice)</fld> <def>An examination by a committee of the votes given at an election, for the purpose of correcting the poll.</def>  <rj><au>Brande & C.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scrutiny</ent><br/
<hw>Scru"ti*ny</hw>, <pos>v. t.</pos> <def>To scrutinize.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scrutoire</ent><br/
<hw>Scru*toire"</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[OF. <ets>escritoire</ets>. See <er>Escritoire</er>.]</ety> <def>A escritoire; a writing desk.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scruze</ent><br/
<hw>Scruze</hw> <pr>(?)</pr>, <pos>v. t.</pos> <ety>[Cf. <er>Excruciate</er>.]</ety> <def>To squeeze, compress, crush, or bruise.</def> <mark>[Obs. or Low]</mark>  <rj><au>Spenser.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scry</ent><br/
<hw>Scry</hw> <pr>(?)</pr>, <pos>v. t.</pos> <def>To descry.</def> <mark>[Obs.]</mark>  <rj><au>Spenser.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scry</ent><br/
<hw>Scry</hw>, <pos>n.</pos> <ety>[From <er>Scry</er>, <pos>v.</pos>]</ety> <def>A flock of wild fowl.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scry</ent><br/
<hw>Scry</hw>, <pos>n.</pos> <ety>[OE. <ets>ascrie</ets>, fr. <ets>ascrien</ets> to cry out, fr. OF. <ets>escrier</ets>, F. <ets>s'<eacute/crier</ets>. See <er>Ex-</er>, and <er>Cry</er>.]</ety> <def>A cry or shout.</def> <mark>[Obs.]</mark>  <rj><au>Ld. Berners.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scud</ent><br/
<hw>Scud</hw> <pr>(sk<ucr/d)</pr>, <pos>v. i.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Scudded</conjf>; <pos>p. pr. & vb. n.</pos> <conjf>Scudding</conjf>.]</vmorph> <ety>[Dan. <ets>skyde</ets> to shoot, shove, push, akin to <ets>skud</ets> shot, gunshot, a shoot, young bough, and to E. <ets>shoot</ets>. <root/159.  See <er>Shoot</er>.]</ety> <sn>1.</sn> <def>To move swiftly; especially, to move as if driven forward by something.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The first nautilus that <qex>scudded</qex> upon the glassy surface of warm primeval oceans.</q> <rj><qau>I. Taylor.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>The wind was high; the vast white clouds <qex>scudded</qex> over the blue heaven.</q> <rj><qau>Beaconsfield.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Naut.)</fld> <def>To be driven swiftly, or to run, before a gale, with little or no sail spread.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scud</ent><br/
<hw>Scud</hw>, <pos>v. t.</pos> <def>To pass over quickly.</def> <mark>[R.]</mark>  <rj><au>Shenstone.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scud</ent><br/
<hw>Scud</hw>, <pos>n.</pos> <sn>1.</sn> <def>The act of scudding; a driving along; a rushing with precipitation.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Loose, vapory clouds driven swiftly by the wind.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Borne on the <qex>scud</qex> of the sea.</q> <rj><qau>Longfellow.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>The <qex>scud</qex> was flying fast above us, throwing a veil over the moon.</q> <rj><qau>Sir S. Baker.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>A slight, sudden shower.</def> <mark>[Prov. Eng.]</mark>  <rj><au>Wright.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <fld>(Zool.)</fld> <def>A small flight of larks, or other birds, less than a flock.</def> <mark>[Prov. Eng.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <fld>(Zool.)</fld> <def>Any swimming amphipod crustacean.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Storm scud</b></col>. <cd>See the Note under <er>Cloud</er>.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scuddle</ent><br/
<hw>Scud"dle</hw> <pr>(?)</pr>, <pos>v. i.</pos> <ety>[Freq. of <ets>scud</ets>: cf. <er>Scuttle</er> to hurry.]</ety> <def>To run hastily; to hurry; to scuttle.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scudo</ent><br/
||<hw>Scu"do</hw> <pr>(?)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Scudi</plw> <pr>(#)</pr>.</plu> <ety>[It., a crown, a dollar, a shield, fr. L. <ets>scutum a shield</ets>.  Cf. <er>Scute</er>.]</ety> <fld>(Com.)</fld> <sd>(a)</sd> <def>A silver coin, and money of account, used in Italy and Sicily, varying in value, in different parts, but worth about 4 shillings sterling, or about 96 cents; also, a gold coin worth about the same.</def> <sd>(b)</sd> <def>A gold coin of Rome, worth 64 shillings 11 pence sterling, or about $ 15.70.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scuff</ent><br/
<hw>Scuff</hw> <pr>(sk<ucr/f)</pr>, <pos>n.</pos> <ety>[Cf. D. <ets>schoft</ets> shoulder, Goth. <ets>skuft</ets> hair of the head.  Cf. <er>Scruff</er>.]</ety> <def>The back part of the neck; the scruff.</def> <mark>[Prov. Eng.]</mark>  <rj><au>Ld. Lytton.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scuff</ent><br/
<hw>Scuff</hw>, <pos>v. i.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Scuffed</conjf> <pr>(sk<ucr/ft)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Scuffing</conjf>.]</vmorph> <ety>[See <er>Scuffle</er>.]</ety> <def>To walk without lifting the feet; to proceed with a scraping or dragging movement; to shuffle.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scuff</ent><br/
<hw>Scuff</hw>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Scuffed</conjf>; <pos>p. pr. & vb. n.</pos> <conjf>Scuffing</conjf>.]</vmorph>  <sn>1.</sn> <def>To cause a blemish on the surface of, by scraping against an object; <as>as, he <ex>scuffed</ex> his shoe on the ground</as>.</def><br/
[<source>PJC</source>]</p>

<p><sn>2.</sn> <def>To scrape with one's foot; <as>as, he <ex>scuffed</ex> the chair leg with his shoe</as>.</def><br/
[<source>PJC</source>]</p>

<p><ent>Scuffle</ent><br/
<hw>Scuf"fle</hw> <pr>(?)</pr>, <pos>v. i.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Scuffled</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Scuffling</conjf> <pr>(?)</pr>.]</vmorph> <ety>[Freq. of <ets>scuff</ets>, v.i.; cf. Sw. <ets>skuffa</ets> to push, shove, <ets>skuff</ets> a push, Dan. <ets>skuffe</ets> a drawer, a shovel, and E. <ets>shuffle</ets>, <ets>shove</ets>. See <er>Shove</er>, and cf. <er>Shuffle</er>.]</ety> <sn>1.</sn> <def>To strive or struggle with a close grapple; to wrestle in a rough fashion.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Hence, to strive or contend tumultuously; to struggle confusedly or at haphazard.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>A gallant man had rather fight to great disadvantage in the field, in an orderly way, than <qex>scuffle</qex> with an undisciplined rabble.</q> <rj><qau>Eikon Basilike.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scuffle</ent><br/
<hw>Scuf"fle</hw>, <pos>n.</pos> <sn>1.</sn> <def>A rough, haphazard struggle, or trial of strength; a disorderly wrestling at close quarters.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Hence, a confused contest; a tumultuous struggle for superiority; a fight.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The dog leaps upon the serpent, and tears it to pieces; but in the <qex>scuffle</qex> the cradle happened to be overturned.</q> <rj><qau>L'Estrange.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>A child's pinafore or bib.</def> <mark>[Prov. Eng.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>A garden hoe.</def> <mark>[Prov. Eng.]</mark>  <rj><au>Halliwell.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scuffler</ent><br/
<hw>Scuf"fler</hw> <pr>(?)</pr>, <pos>n.</pos> <sn>1.</sn> <def>One who scuffles.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>An agricultural implement resembling a scarifier, but usually lighter.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scug</ent><br/
<hw>Scug</hw> <pr>(sk<ucr/g)</pr>, <pos>v. i.</pos> <ety>[Cf. Dan. <ets>skygge</ets> to darken, a shade, SW. <ets>skugga</ets> to shade, a shade, Icel. <ets>skyggja</ets> to shade, <ets>skuggi</ets> a shade.]</ety> <def>To hide.</def> <mark>[Prov. Eng.]</mark>  <rj><au>Halliwell.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scug</ent><br/
<hw>Scug</hw>, <pos>n.</pos> <def>A place of shelter; the declivity of a hill.</def> <mark>[Prov. Eng.]</mark>  <rj><au>Halliwell.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sculker</ent><br/
<ent>Sculk</ent><br/
<mhw>{ <hw>Sculk</hw> <pr>(sk<ucr/lk)</pr>, <hw>Sculk"er</hw> <pr>(sk<ucr/lk"<etil/r)</pr> }</mhw>. <def>See <er>Skulk</er>, <er>Skulker</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scull</ent><br/
<hw>Scull</hw> <pr>(sk<ucr/l)</pr>, <pos>n.</pos> <fld>(Anat.)</fld> <def>The skull.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scull</ent><br/
<hw>Scull</hw>, <pos>n.</pos> <ety>[See 1st <er>School</er>.]</ety> <def>A shoal of fish.</def>  <rj><au>Milton.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scull</ent><br/
<hw>Scull</hw>, <pos>n.</pos> <ety>[Of uncertain origin; cf. Icel. <ets>skola</ets> to wash.]</ety> <sn>1.</sn> <fld>(Naut.)</fld> <sd>(a)</sd> <def>A boat; a cockboat. See <er>Sculler</er>.</def> <sd>(b)</sd> <def>One of a pair of short oars worked by one person.</def> <sd>(c)</sd> <def>A single oar used at the stern in propelling a boat.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Zool.)</fld> <def>The common skua gull.</def> <mark>[Prov. Eng.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scull</ent><br/
<hw>Scull</hw>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Sculled</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Sculling</conjf>.]</vmorph> <fld>(Naut.)</fld> <def>To impel (a boat) with a pair of sculls, or with a single scull or oar worked over the stern obliquely from side to side.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scull</ent><br/
<hw>Scull</hw>, <pos>v. i.</pos> <def>To impel a boat with a scull or sculls.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sculler</ent><br/
<hw>Scull"er</hw> <pr>(?)</pr>, <pos>n.</pos> <sn>1.</sn> <def>A boat rowed by one man with two sculls, or short oars.</def> <mark>[R.]</mark>  <rj><au>Dryden.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>One who sculls.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scullery</ent><br/
<hw>Scul"ler*y</hw> <pr>(sk<ucr/l"l<etil/r*<ycr/)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Sculleries</plw> <pr>(sk<ucr/l"l<etil/r*<icr/z)</pr>.</plu> <ety>[Probably originally, a place for washing dishes, and for <ets>swillery</ets>, fr. OE. <ets>swilen</ets> to wash, AS. <ets>swilian</ets> (see <er>Swill</er> to wash, to drink), but influenced either by Icel. <ets>skola</ets>, <ets>skyla</ets>, Dan. <ets>skylle</ets>, or by OF. <ets>escuelier</ets> a place for keeping dishes, fr. <ets>escuele</ets> a dish, F. <ets><eacute/cuelle</ets>, fr. L. <ets>scutella</ets> a salver, waiter (cf. <er>Scuttle</er> a basket); or perhaps the English word is immediately from the OF. <ets>escuelier</ets>; cf. OE. <ets>squyllare</ets> a dishwasher.]</ety> <sn>1.</sn> <def>A place where dishes, kettles, and culinary utensils, are cleaned and kept; also, a room attached to the kitchen, where the coarse work is done; a back kitchen.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Hence, refuse; filth; offal.</def> <mark>[Obs.]</mark>  <rj><au>Gauden.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scullion</ent><br/
<hw>Scul"lion</hw> <pr>(sk<ucr/l"y<ucr/n)</pr>, <pos>n.</pos> <fld>(Bot.)</fld> <def>A scallion.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scullion</ent><br/
<hw>Scul"lion</hw>, <pos>n.</pos> <ety>[OF. <ets>escouillon</ets> (Cot.) a dishclout, apparently for <ets>escouvillon</ets>, F. <ets><eacute/couvillon</ets> a swab; cf. also OF. <ets>souillon</ets> a servant employed for base offices.  Cf. <er>Scovel</er>.]</ety> <def>A servant who cleans pots and kettles, and does other menial services in the kitchen.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The meanest <qex>scullion</qex> that followed his camp.</q> <rj><qau>South.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scullionly</ent><br/
<hw>Scul"lion*ly</hw>, <pos>a.</pos> <def>Like a scullion; base.</def> <mark>[Obs.]</mark>  <rj><au>Milton.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sculp</ent><br/
<hw>Sculp</hw> <pr>(?)</pr>, <pos>v. t.</pos> <ety>[See <er>Sculptor</er>.]</ety> <def>To sculpture; to carve; to engrave.</def> <mark>[Obs. or Humorous.]</mark>  <rj><au>Sandys.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sculpin</ent><br/
<hw>Scul"pin</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Written also <ets>skulpin</ets>.]</ety> <fld>(Zool.)</fld> <sd>(a)</sd> <def>Any one of numerous species of marine cottoid fishes of the genus <gen>Cottus</gen>, or <gen>Acanthocottus</gen>, having a large head armed with several sharp spines, and a broad mouth. They are generally mottled with yellow, brown, and black. Several species are found on the Atlantic coasts of Europe and America.</def> <sd>(b)</sd> <def>A large cottoid market fish of California (<spn>Scorpaenichthys marmoratus</spn>); -- called also <altname>bighead</altname>, <altname>cabezon</altname>, <altname>scorpion</altname>, <altname>salpa</altname>.</def> <sd>(c)</sd> <def>The dragonet, or yellow sculpin, of Europe (<spn>Callionymus lyra</spn>).</def><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ The name is also applied to other related California species.</note><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Deep-water sculpin</b></col>, <cd>the sea raven.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sculptile</ent><br/
<hw>Sculp"tile</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>sculptilis</ets>. See <ets>Sculptor</ets>.]</ety> <def>Formed by carving; graven; <as>as, <ex>sculptile</ex> images</as>.</def> <mark>[Obs.]</mark>  <rj><au>Sir T. Browne.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sculptor</ent><br/
<hw>Sculp"tor</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>sculptor</ets>, fr. <ets>sculpere</ets>, <ets>sculptum</ets>, to carve; cf. <ets>scalpere</ets> to cut, carve, scratch, and Gr. <?/ to carve: cf. F. <ets>sculpteur</ets>.]</ety> <sn>1.</sn> <def>One who sculptures; one whose occupation is to carve statues, or works of sculpture.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Hence, an artist who designs works of sculpture, his first studies and his finished model being usually in a plastic material, from which model the marble is cut, or the bronze is cast.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sculptress</ent><br/
<hw>Sculp"tress</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A female sculptor.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sculptural</ent><br/
<hw>Sculp"tur*al</hw> <pr>(?; 135)</pr>, <pos>a.</pos> <def>Of or pertaining to sculpture.</def>  <rj><au>G. Eliot.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sculpture</ent><br/
<hw>Sculp"ture</hw> <pr>(?; 135)</pr>, <pos>n.</pos> <ety>[L. <ets>sculptura</ets>: cf. F. <ets>sculpture</ets>.]</ety> <sn>1.</sn> <def>The art of carving, cutting, or hewing wood, stone, metal, etc., into statues, ornaments, etc., or into figures, as of men, or other things; hence, the art of producing figures and groups, whether in plastic or hard materials.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Carved work modeled of, or cut upon, wood, stone, metal, etc.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>There, too, in living <qex>sculpture</qex>, might be seen<br/
The mad affection of the Cretan queen.</q> <rj><qau>Dryden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sculpture</ent><br/
<hw>Sculp"ture</hw> <pr>(?; 135)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Sculptured</conjf> <pr>(<?/)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Sculpturing</conjf>.]</vmorph> <def>To form with the chisel on, in, or from, wood, stone, or metal; to carve; to engrave.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Sculptured tortoise</b></col> <fld>(Zool.)</fld>, <cd>a common North American wood tortoise (<spn>Glyptemys insculpta</spn>). The shell is marked with strong grooving and ridges which resemble sculptured figures.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sculpturesque</ent><br/
<hw>Sculp`tur*esque"</hw> <pr>(?)</pr>, <pos>a.</pos> <def>After the manner of sculpture; resembling, or relating to, sculpture.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scum</ent><br/
<hw>Scum</hw> <pr>(sk<ucr/m)</pr>, <pos>n.</pos> <ety>[Of Scand. origin; cf. Dan. & Sw. <ets>skum</ets>, Icel. <ets>sk<umac/m</ets>, LG. <ets>schum</ets>, D. <ets>schuim</ets>, OHG. <ets>sc<umac/m</ets>, G. <ets>schaum</ets>; probably from a root meaning, to cover. <root/158.  Cf. <er>Hide</er> skin, <er>Meerschaum</er>, <er>Skim</er>, <pos>v.</pos>, <er>Sky</er>.]</ety><br/
[<source>1913 Webster</source>]</p>

<p><sn>1.</sn> <def>The extraneous matter or impurities which rise to the surface of liquids in boiling or fermentation, or which form on the surface by other means; also, the scoria of metals in a molten state; dross.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Some to remove the <qex>scum</qex> as it did rise.</q> <rj><qau>Spenser.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>refuse; recrement; anything vile or worthless.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The great and innocent are insulted by the <qex>scum</qex> and refuse of the people.</q> <rj><qau>Addison.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scum</ent><br/
<hw>Scum</hw>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Scummed</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Scumming</conjf> <pr>(?)</pr>.]</vmorph> <sn>1.</sn> <def>To take the scum from; to clear off the impure matter from the surface of; to skim.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>You that <qex>scum</qex> the molten lead.</q> <rj><qau>Dryden & Lee.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To sweep or range over the surface of.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>Wandering up and down without certain seat, they lived by <qex>scumming</qex> those seas and shores as pirates.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scum</ent><br/
<hw>Scum</hw>, <pos>v. i.</pos> <def>To form a scum; to become covered with scum. Also used figuratively.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Life, and the interest of life, have stagnated and <qex>scummed</qex> over.</q> <rj><qau>A. K. H. Boyd.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scumber</ent><br/
<hw>Scum"ber</hw> <pr>(?)</pr>, <pos>v. i.</pos> <ety>[Cf. <er>Discumber</er>.]</ety> <def>To void excrement.</def> <mark>[Obs. or Prov. Eng.]</mark>  <rj><au>Massinger.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scumber</ent><br/
<hw>Scum"ber</hw>, <pos>n.</pos> <def>Dung.</def> <mark>[Obs. or Prov. Eng.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scumble</ent><br/
<hw>Scum"ble</hw> <pr>(?)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Scumbled</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Scumbling</conjf> <pr>(?)</pr>.]</vmorph> <ety>[Freq. of <ets>scum</ets>. <root/ 158.]</ety> <fld>(Fine Arts)</fld> <def>To cover lighty, as a painting, or a drawing, with a thin wash of opaque color, or with color-crayon dust rubbed on with the stump, or to make any similar additions to the work, so as to produce a softened effect.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scumbling</ent><br/
<hw>Scum"bling</hw> <pr>(?)</pr>, <pos>n.</pos> <sn>1.</sn> <fld>(Fine Arts)</fld> <sd>(a)</sd> <def>A mode of obtaining a softened effect, in painting and drawing, by the application of a thin layer of opaque color to the surface of a painting, or part of the surface, which is too bright in color, or which requires harmonizing.</def> <sd>(b)</sd> <def>In crayon drawing, the use of the stump.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>The color so laid on. Also used figuratively.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Shining above the brown <qex>scumbling</qex> of leafless orchards.</q> <rj><qau>L. Wallace.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scummer</ent><br/
<hw>Scum"mer</hw> <pr>(?)</pr>, <pos>v. i.</pos> <def>To scumber.</def> <mark>[Obs.]</mark>  <rj><au>Holland.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scummer</ent><br/
<hw>Scum"mer</hw>, <pos>n.</pos> <def>Excrement; scumber.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scummer</ent><br/
<hw>Scum"mer</hw>, <pos>n.</pos> <ety>[Cf. OF. <ets>escumoire</ets>, F. <ets><eacute/cumoire</ets>. See <er>Scum</er>, and cf. <er>Skimmer</er>.]</ety> <def>An instrument for taking off scum; a skimmer.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scumming</ent><br/
<hw>Scum"ming</hw> <pr>(?)</pr>, <pos>n.</pos> <sd>(a)</sd> <def>The act of taking off scum.</def> <sd>(b)</sd> <def>That which is scummed off; skimmings; scum; -- used chiefly in the plural.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scummy</ent><br/
<hw>Scum"my</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Covered with scum; of the nature of scum.</def>  <rj><au>Sir P. Sidney.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scunner</ent><br/
<hw>Scun"ner</hw> <pr>(?)</pr>, <pos>v. t.</pos> <ety>[Cf. <er>Shun</er>.]</ety> <def>To cause to loathe, or feel disgust at.</def> <mark>[Scot. & Prov. Eng.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scunner</ent><br/
<hw>Scun"ner</hw>, <pos>v. i.</pos> <def>To have a feeling of loathing or disgust; hence, to have dislike, prejudice, or reluctance.</def> <mark>[Scot. & Prov. Eng.]</mark>  <rj><au>C. Kingsley.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scunner</ent><br/
<hw>Scun"ner</hw>, <pos>n.</pos> <def>A feeling of disgust or loathing; a strong prejudice; abhorrence; <as>as, to take a <ex>scunner</ex> against some one</as>.</def> <mark>[Scot. & Prov. Eng.]</mark>  <rj><au>Carlyle.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scup</ent><br/
<hw>Scup</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[D. <ets>schop</ets>.]</ety> <def>A swing.</def> <mark>[Local, U.S.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scup</ent><br/
<hw>Scup</hw>, <pos>n.</pos> <ety>[Contr. fr. American Indian <ets>mishc<ugrave/p</ets>, fr. <ets>mishe-kuppi</ets> large, thick-scaled.]</ety> <fld>(Zool.)</fld> <def>A marine sparoid food fish (<spn>Stenotomus chrysops</spn>, or <spn>Stenotomus argyrops</spn>), common on the Atlantic coast of the United States. It appears bright silvery when swimming in the daytime, but shows broad blackish transverse bands at night and when dead. Called also <altname>porgee</altname>, <altname>paugy</altname>, <altname>porgy</altname>, <altname>scuppaug</altname>.</def><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ The same names are also applied to a closely allied Southern species (<spn>Stenotomus Gardeni</spn>).</note><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scuppaug</ent><br/
<hw>Scup"paug</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Contr. fr. Amer. Indian <ets>mishcuppauog</ets>, pl. of <ets>mishcup</ets>.]</ety> <fld>(Zool.)</fld> <def>See 2d <er>Scup</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scupper</ent><br/
<hw>Scup"per</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[OF. <ets>escopir</ets>, <ets>escupir</ets>, to spit, perhaps for <ets>escospir</ets>, L. <ets>ex + conspuere</ets> to spit upon; pref. <ets>con-</ets> + <ets>spuere</ets> to spit.  Cf. <er>Spit</er>, <pos>v.</pos>]</ety> <fld>(Naut.)</fld> <def>An opening cut through the waterway and bulwarks of a ship, so that water falling on deck may flow overboard; -- called also <altname>scupper hole</altname>.</def><br/
[<source>1913 Webster</source>]</p>

<p><-- p. 1295 pr=PCP --></p>

<p><cs><col><b>Scupper hose</b></col> <fld>(Naut.)</fld>, <cd>a pipe of leather, canvas, etc., attached to the mouth of the scuppers, on the outside of a vessel, to prevent the water from entering.</cd> <au>Totten.</au> -- <col><b>Scupper nail</b></col> <fld>(Naut.)</fld>, <cd>a nail with a very broad head, for securing the edge of the hose to the scupper.</cd> -- <col><b>Scupper plug</b></col> <fld>(Naut.)</fld>, <cd>a plug to stop a scupper.</cd> <au>Totten.</au></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scuppernong</ent><br/
<hw>Scup"per*nong</hw> <pr>(sk<ucr/p"p<etil/r*n<ocr/ng)</pr>, <pos>n.</pos> <ety>[Probably of American Indian origin.]</ety> <fld>(Bot.)</fld> <def>An American grape, a form of <spn>Vitis vulpina</spn>, found in the Southern Atlantic States, and often cultivated.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scur</ent><br/
<hw>Scur</hw> <pr>(sk<ucir/r)</pr>, <pos>v. i.</pos> <ety>[Cf. <er>Scour</er> to run.]</ety> <def>To move hastily; to scour.</def> <mark>[Obs. or Prov. Eng.]</mark>  <rj><au>Halliwell.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scurf</ent><br/
<hw>Scurf</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[AS. <ets>scurf</ets>, <ets>sceorf</ets>, or from Scand.; cf. Sw. <ets>skorf</ets>, Dan. <ets>skurv</ets>, Icel. <ets>skurfur</ets>, D. <ets>schurft</ets>, G. <ets>schorf</ets>; all akin to AS. <ets>scurf</ets>, and to AS. <ets>sceorfan</ets> to scrape, to gnaw, G. <ets>sch<uum/rfen</ets> to scrape, and probably also to E. <ets>scrape</ets>.  Cf. <er>Scurvy</er>.]</ety> <sn>1.</sn> <def>Thin dry scales or scabs upon the body; especially, thin scales exfoliated from the cuticle, particularly of the scalp; dandruff.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Hence, the foul remains of anything adherent.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The <qex>scurf</qex> is worn away of each committed crime.</q> <rj><qau>Dryden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>Anything like flakes or scales adhering to a surface.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>There stood a hill not far, whose grisly top<br/
Belched fire and rolling smoke; the rest entire<br/
Shone with a glossy <qex>scurf</qex>.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <fld>(Bot.)</fld> <def>Minute membranous scales on the surface of some leaves, as in the goosefoot.</def>  <rj><au>Gray.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scurff</ent><br/
<hw>Scurff</hw> <pr>(?)</pr>, <pos>n.</pos> <def>The bull trout.</def> <mark>[Prov. Eng.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scurfiness</ent><br/
<hw>Scurf"i*ness</hw>, <pos>n.</pos> <sn>1.</sn> <def>Quality or state of being scurfy.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Bot.)</fld> <def>Scurf.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scurfy</ent><br/
<hw>Scurf"y</hw> <pr>(?)</pr>, <pos>a.</pos> <amorph>[<pos>Compar.</pos> <adjf>Scurfier</adjf> <pr>(?)</pr>; <pos>superl.</pos> <adjf>Scurfiest</adjf>.]</amorph> <def>Having or producing scurf; covered with scurf; resembling scurf.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scurrier</ent><br/
<hw>Scur"ri*er</hw> <pr>(?)</pr>, <pos>n.</pos> <def>One who scurries.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scurrile</ent><br/
<hw>Scur"rile</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>scurrilis</ets>, fr. <ets>scurra</ets> a bufoon, jester: cf. F. <ets>scurrile</ets>.]</ety> <def>Such as befits a buffoon or vulgar jester; grossly opprobrious or loudly jocose in language; scurrilous; <as>as, <ex>scurrile</ex> taunts</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The wretched affectation of <qex>scurrile</qex> laughter.</q> <rj><qau>Cowley.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>A <qex>scurrile</qex> or obscene jest will better advance you at the court of Charles than your father's ancient name.</q> <rj><qau>Sir W. Scott.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scurrility</ent><br/
<hw>Scur*ril"i*ty</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>scurrilitas</ets>: cf. F. <ets>scurrilit<eacute/</ets>.]</ety> <sn>1.</sn> <def>The quality or state of being scurrile or scurrilous; mean, vile, or obscene jocularity.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Your reasons . . . have been sharp and sententious, pleasant without <qex>scurrility</qex>.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>That which is scurrile or scurrilous; gross or obscene language; low buffoonery; vulgar abuse.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Interrupting prayers and sermons with clamor and <qex>scurrility</qex>.</q> <rj><qau>Macaulay.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><syn><b>Syn.</b> -- Scurrilousness; abuse; insolence; vulgarity; indecency.</syn><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scurrilous</ent><br/
<hw>Scur"ril*ous</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[See <ets>Scurrile</ets>.]</ety> <sn>1.</sn> <def>Using the low and indecent language of the meaner sort of people, or such as only the license of buffoons can warrant; <as>as, a <ex>scurrilous</ex> fellow</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Containing low indecency or abuse; mean; foul; vile; obscenely jocular; <as>as, <ex>scurrilous</ex> language</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The absurd and <qex>scurrilous</qex> sermon which had very unwisely been honored with impeachment.</q> <rj><qau>Macaulay.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><syn><b>Syn.</b> -- Opprobrious; abusive; reproachful; insulting; insolent; offensive; gross; vile; vulgar; low; foul; foul-mouthed; indecent; scurrile; mean.</syn><br/
[<source>1913 Webster</source>]</p>

<p>-- <wordforms><wf>Scur"ril*ous*ly</wf>, <pos>adv.</pos> -- <wf>Scur"ril*ous*ness</wf>, <pos>n.</pos></wordforms><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scurrit</ent><br/
<hw>Scur"rit</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Zool.)</fld> <def>The lesser tern (<spn>Sterna minuta</spn>).</def> <mark>[Prov. Eng.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scurry</ent><br/
<hw>Scur"ry</hw> <pr>(?)</pr>, <pos>v. i.</pos> <ety>[Cf. <er>Scur</er>, <er>Skirr</er>.]</ety> <def>To hasten away or along; to move rapidly; to hurry; <as>as, the rabbit <ex>scurried</ex> away</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scurry</ent><br/
<hw>Scur"ry</hw>, <pos>n.</pos> <def>Act of scurrying; hurried movement.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scurvily</ent><br/
<hw>Scur"vi*ly</hw> <pr>(?)</pr>, <pos>adv.</pos> <def>In a scurvy manner.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scurviness</ent><br/
<hw>Scur"vi*ness</hw> <pr>(?)</pr>, <pos>n.</pos> <def>The quality or state of being scurvy; vileness; meanness.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scurvy</ent><br/
<hw>Scur"vy</hw> <pr>(?)</pr>, <pos>a.</pos> <amorph>[<pos>Compar.</pos> <adjf>Scurvier</adjf> <pr>(?)</pr>; <pos>superl.</pos> <adjf>Scurviest</adjf>.]</amorph> <ety>[From <er>Scurf</er>; cf. <er>Scurvy</er>, <pos>n.</pos>]</ety> <sn>1.</sn> <def>Covered or affected with scurf or scabs; scabby; scurfy; specifically, diseased with the scurvy.</def> <ldquo/Whatsoever man . . . be <xex>scurvy</xex> or scabbed.<rdquo/  <rj><au>Lev. xxi. 18, 20.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Vile; mean; low; vulgar; contemptible.</def> <ldquo/A <xex>scurvy</xex> trick.<rdquo/  <rj><au>Ld. Lytton.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>That <qex>scurvy</qex> custom of taking tobacco.</q> <rj><qau>Swift.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>[He] spoke spoke such <qex>scurvy</qex> and provoking terms.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scurvy</ent><br/
<hw>Scur"vy</hw>, <pos>n.</pos> <ety>[Probably from the same source as <ets>scorbute</ets>, but influenced by <ets>scurf</ets>, <ets>scurfy</ets>, <ets>scurvy</ets>, adj.; cf. D. <ets>scheurbuik</ets> scurvy, G. <ets>scharbock</ets>, LL. <ets>scorbutus</ets>.  Cf. <er>Scorbute</er>.]</ety> <fld>(Med.)</fld> <def>A disease characterized by livid spots, especially about the thighs and legs, due to extravasation of blood, and by spongy gums, and bleeding from almost all the mucous membranes. It is accompanied by paleness, languor, depression, and general debility. It is occasioned by confinement, innutritious food, and hard labor, but especially by lack of fresh vegetable food, or confinement for a long time to a limited range of food, which is incapable of repairing the waste of the system. It was formerly prevalent among sailors and soldiers.</def><-- caused by lack of vitamin C --><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Scurvy grass</b></col> <ety>[<ets>Scurvy</ets> + <ets>grass</ets>; or cf. Icel. <ets>skarfa</ets>k<amac/l <ets>scurvy grass</ets>.]</ety> <fld>(Bot.)</fld> <cd>A kind of cress (<spn>Cochlearia officinalis</spn>) growing along the seacoast of Northern Europe and in arctic regions. It is a remedy for the scurvy, and has proved a valuable food to arctic explorers. The name is given also to other allied species of plants.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scut</ent><br/
<hw>Scut</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Cf. Icel. <ets>skott</ets> a fox's tail. <root/ 159.]</ety> <mark>[Obs.]</mark> <def>The tail of a hare, or of a deer, or other animal whose tail is short, esp. when carried erect; hence, sometimes, the animal itself.</def> <ldquo/He ran like a <xex>scut</xex>.<rdquo/  <rj><au>Skelton.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>How the Indian hare came to have a long tail, whereas that part in others attains no higher than a <qex>scut</qex>.</q> <rj><qau>Sir T. Browne.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>My doe with the black <qex>scut</qex>.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scuta</ent><br/
||<hw>Scu"ta</hw> <pr>(?)</pr>, <pos>n. pl.</pos> <def>See <er>Scutum</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scutage</ent><br/
<hw>Scu"tage</hw> <pr>(?; 48)</pr>, <pos>n.</pos> <ety>[LL. <ets>scutagium</ets>, from L. <ets>scutum</ets> a shield.]</ety> <fld>(Eng. Hist.)</fld> <def>Shield money; commutation of service for a sum of money. See <er>Escuage</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scutal</ent><br/
<hw>Scu"tal</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Of or pertaining to a shield.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>A good example of these <qex>scutal</qex> monstrosities.</q> <rj><qau>Cussans.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scutate</ent><br/
<hw>Scu"tate</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>scutatus</ets> armed with a shield, from <ets>scutum</ets> a shield.]</ety> <sn>1.</sn> <def>Buckler-shaped; round or nearly round.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Zool.)</fld> <def>Protected or covered by bony or horny plates, or large scales.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scutch</ent><br/
<hw>Scutch</hw> <pr>(?)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Scutched</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Scutching</conjf>.]</vmorph> <ety>[See <er>Scotch</er> to cut slightly.]</ety> <sn>1.</sn> <def>To beat or whip; to drub.</def> <mark>[Old or Prov. Eng. & Scot.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To separate the woody fiber from (flax, hemp, etc.) by beating; to swingle.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>To loosen and dress the fiber of (cotton or silk) by beating; to free (fibrous substances) from dust by beating and blowing.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Scutching machine</b></col>, <cd>a machine used to scutch cotton, silk, or flax; -- called also <altname>batting machine</altname>.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scutch</ent><br/
<hw>Scutch</hw>, <pos>n.</pos> <sn>1.</sn> <def>A wooden instrument used in scutching flax and hemp.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>The woody fiber of flax; the refuse of scutched flax.</def> <ldquo/The smoke of the burning <xex>scutch</xex>.<rdquo/  <rj><au>Cuthbert Bede.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scutcheon</ent><br/
<hw>Scutch"eon</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Aphetic form of <ets>escutcheon</ets>.]</ety> <sn>1.</sn> <def>An escutcheon; an emblazoned shield.</def>  <rj><au>Bacon.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>The corpse lay in state, with all the pomp of <qex>scutcheons</qex>, wax lights, black hangings, and mutes.</q> <rj><qau>Macaulay.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A small plate of metal, as the shield around a keyhole. See <er>Escutcheon</er>, 4.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scutcheoned</ent><br/
<hw>Scutch"eoned</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Emblazoned on or as a shield.</def><br/
[<source>1913 Webster</source>]</p>

<p><q><qex>Scutcheoned</qex> panes in cloisters old.</q> <rj><qau>Lowell.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scutcher</ent><br/
<hw>Scutch"er</hw> <pr>(?)</pr>, <pos>n.</pos> <sn>1.</sn> <def>One who scutches.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>An implement or machine for scutching hemp, flax, or cotton, etc.; a scutch; a scutching machine.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scutch grass</ent><br/
<hw>Scutch" grass`</hw> <pr>(?)</pr>. <fld>(Bot.)</fld> <def>A kind of pasture grass (<spn>Cynodon Dactylon</spn>). See <er>Bermuda grass</er>: also <xex>Illustration</xex> in Appendix.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scute</ent><br/
<hw>Scute</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>scutum</ets> a shield, a buckler. See <er>Scudo</er>.]</ety> <sn>1.</sn> <def>A small shield.</def> <mark>[Obs.]</mark>  <rj><au>Skelton.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>An old French gold coin of the value of 3s. 4d. sterling, or about 80 cents.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <fld>(Zool.)</fld> <def>A bony scale of a reptile or fish; a large horny scale on the leg of a bird, or on the belly of a snake.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scutella</ent><br/
||<hw>Scu*tel"la</hw> <pr>(?)</pr>, <pos>n. pl.</pos> <def>See <er>Scutellum</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scutella</ent><br/
||<hw>Scu*tel"la</hw>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Scutellae</plw> <pr>(#)</pr>.</plu> <ety>[NL., fem. dim. of L. <ets>scutum</ets>.]</ety> <fld>(Zool.)</fld> <def>See <er>Scutellum</er>, <pos>n.</pos>, 2.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scutellated</ent><br/
<ent>Scutellate</ent><br/
<mhw>{ <hw>Scu"tel*late</hw> <pr>(?)</pr>, <hw>Scu"tel*la`ted</hw> <pr>(?)</pr>, }</mhw> <pos>a.</pos> <ety>[L. <ets>scutella</ets> a dish, salver.  Cf. <er>Scuttle</er> a basket.]</ety> <sn>1.</sn> <fld>(Zool.)</fld> <def>Formed like a plate or salver; composed of platelike surfaces; <as>as, the <ex>scutellated</ex> bone of a sturgeon</as>.</def>  <rj><au>Woodward.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <ety>[See <er>Scutellum</er>.]</ety> <fld>(Zool.)</fld> <def>Having the tarsi covered with broad transverse scales, or scutella; -- said of certain birds.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scutellation</ent><br/
<hw>Scu`tel*la"tion</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Zool.)</fld> <def>The entire covering, or mode of arrangement, of scales, as on the legs and feet of a bird.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scutelliform</ent><br/
<hw>Scu*tel"li*form</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>scutella</ets> a dish + <ets>-form</ets>.]</ety> <sn>1.</sn> <def>Scutellate.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Bot.)</fld> <def>Having the form of a scutellum.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scutelliplantar</ent><br/
<hw>Scu*tel`li*plan"tar</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>scutellus</ets> a shield + <ets>planta</ets> foot.]</ety> <fld>(Zool.)</fld> <def>Having broad scutella on the front, and small scales on the posterior side, of the tarsus; -- said of certain birds.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scutellum</ent><br/
||<hw>Scu*tel"lum</hw> <pr>(?)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Scutella</plw> <pr>(#)</pr>.</plu> <ety>[NL., neut. dim. of L. <ets>scutum</ets> a shield.]</ety> <sn>1.</sn> <fld>(Bot.)</fld> <def>A rounded apothecium having an elevated rim formed of the proper thallus, the fructification of certain lichens.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Zool.)</fld> <sd>(a)</sd> <def>The third of the four pieces forming the upper part of a thoracic segment of an insect. It follows the scutum, and is followed by the small postscutellum; a scutella. See <er>Thorax</er>.</def> <sd>(b)</sd> <def>One of the transverse scales on the tarsi and toes of birds; a scutella.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scutibranch</ent><br/
<hw>Scu"ti*branch</hw> <pr>(?)</pr>, <pos>a.</pos> <fld>(Zool.)</fld> <def>Scutibranchiate.</def> -- <def2><pos>n.</pos> <def>One of the Scutibranchiata.</def></def2><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scutibranchia</ent><br/
||<hw>Scu`ti*bran"chi*a</hw> <pr>(?)</pr>, <pos>n. pl.</pos> <ety>[NL.]</ety> <fld>(Zool.)</fld> <def>Same as <er>Scutibranchiata</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scutibranchian</ent><br/
<hw>Scu`ti*bran"chi*an</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Zool.)</fld> <def>One of the Scutibranchiata.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scutibranchiata</ent><br/
||<hw>Scu`ti*bran`chi*a"ta</hw> <pr>(?)</pr>, <pos>n. pl.</pos> <ety>[NL. See <er>Scutum</er>, and <er>Branchia</er>.]</ety> <fld>(Zool.)</fld> <def>An order of gastropod Mollusca having a heart with two auricles and one ventricle. The shell may be either spiral or shieldlike.</def><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ It is now usually regarded as including only the Rhipidoglossa and the Docoglossa. When originally established, it included a heterogenous group of mollusks having shieldlike shells, such as Haliotis, Fissurella, Carinaria, etc.</note><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scutibranchiate</ent><br/
<hw>Scu`ti*bran"chi*ate</hw> <pr>(?)</pr>, <pos>a.</pos> <fld>(Zool.)</fld> <def>Having the gills protected by a shieldlike shell; of or pertaining to the Scutibranchiata.</def> -- <def2><pos>n.</pos> <def>One of the Scutibranchiata.</def></def2><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scutiferous</ent><br/
<hw>Scu*tif"er*ous</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>scutum</ets> shield + <ets>-ferous</ets>.]</ety> <def>Carrying a shield or buckler.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scutiform</ent><br/
<hw>Scu"ti*form</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>scutum</ets> shield + <ets>-form</ets>: cf. F. <ets>scutiforme</ets>.]</ety> <def>Shield-shaped; scutate.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scutiger</ent><br/
||<hw>Scu"ti*ger</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[NL., fr. L. <ets>scutum</ets> shield + <ets>gerere</ets> to bear.]</ety> <fld>(Zool.)</fld> <def>Any species of chilopod myriapods of the genus <gen>Scutigera</gen>. They sometimes enter buildings and prey upon insects.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scutiped</ent><br/
<hw>Scu"ti*ped</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>scutum</ets> a shield + <ets>pes</ets>, <ets>pedis</ets>, a foot: cf. F. <ets>scutip<egrave/de</ets>.]</ety> <fld>(Zool.)</fld> <def>Having the anterior surface of the tarsus covered with scutella, or transverse scales, in the form of incomplete bands terminating at a groove on each side; -- said of certain birds.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scutter</ent><br/
<hw>Scut"ter</hw> <pr>(?)</pr>, <pos>v. i.</pos> <ety>[Cf. <er>Scuttle</er>, <pos>v. i.</pos>]</ety> <def>To run quickly; to scurry; to scuttle.</def> <mark>[Prov. Eng.]</mark></p>

<p><q>A mangy little jackal . . . cocked up his ears and tail, and <qex>scuttered</qex> across the shallows.</q>  <rj><qau>Kipling.</qau></rj><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Scuttle</ent><br/
<hw>Scut"tle</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[AS. <ets>scutel</ets> a dish, platter; cf. Icel. <ets>skutill</ets>; both fr. L. <ets>scutella</ets>, dim. of <ets>scutra</ets>, <ets>scuta</ets>, a dish or platter; cf. <ets>scutum</ets> a shield.  Cf. <er>Skillet</er>.]</ety> <sn>1.</sn> <def>A broad, shallow basket.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A wide-mouthed vessel for holding coal: a coal hod.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scuttle</ent><br/
<hw>Scut"tle</hw>, <pos>v. i.</pos> <ety>[For <ets>scuddle</ets>, fr. <ets>scud</ets>.]</ety> <def>To run with affected precipitation; to hurry; to bustle; to scuddle.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>With the first dawn of day, old Janet was <qex>scuttling</qex> about the house to wake the baron.</q> <rj><qau>Sir W. Scott.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scuttle</ent><br/
<hw>Scut"tle</hw>, <pos>n.</pos> <def>A quick pace; a short run.</def>  <rj><au>Spectator.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scuttle</ent><br/
<hw>Scut"tle</hw> <pr>(sk<ucr/t"t'l)</pr>, <pos>n.</pos> <ety>[OF. <ets>escoutille</ets>, F. <ets><eacute/scoutille</ets>, cf. Sp. <ets>escotilla</ets>; probably akin to Sp. <ets>escotar</ets> to cut a thing so as to make it fit, to hollow a garment about the neck, perhaps originally, to cut a bosom-shaped piece out, and of Teutonic origin; cf. D. <ets>schoot</ets> lap, bosom, G. <ets>schoss</ets>, Goth. <ets>skauts</ets> the hem of a garnment.  Cf. <er>Sheet</er> an expanse.]</ety> <sn>1.</sn> <def>A small opening in an outside wall or covering, furnished with a lid.</def> <specif>Specifically:</specif> <sd>(a)</sd> <fld>(Naut.)</fld> <def>A small opening or hatchway in the deck of a ship, large enough to admit a man, and with a lid for covering it, also, a like hole in the side or bottom of a ship.</def> <sd>(b)</sd> <def>An opening in the roof of a house, with a lid.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>The lid or door which covers or closes an opening in a roof, wall, or the like.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><mcol><col><b>Scuttle butt</b></col>, <it>or</it> <col><b>Scuttle cask</b></col></mcol> <fld>(Naut.)</fld>, <cd>a butt or cask with a large hole in it, used to contain the fresh water for daily use in a ship.</cd></cs>  <rj><au>Totten.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scuttle</ent><br/
<hw>Scut"tle</hw>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Scuttled</conjf> <pr>(sk<ucr/t"t'ld)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Scuttling</conjf>.]</vmorph> <sn>1.</sn> <def>To cut a hole or holes through the bottom, deck, or sides of (as of a ship), for any purpose.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To sink by making holes through the bottom of; <as>as, to <ex>scuttle</ex> a ship</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <specif>Hence:</specif> <def>To defeat, frustrate, abandon, or cause to be abandoned; -- of plans, projects, actions, hopes; <as>as, the review committee <ex>scuttled</ex> the project due to lack of funds</as>.</def><br/
[<source>PJC</source>]</p>

<p><ent>scuttlebutt</ent><br/
<hw>scut"tle*butt`</hw> <pr>(sk<ucr/t"t'l*b<ucr/t`)</pr> , <pos>n.</pos> <sn>1.</sn> <def>See <cref>scuttle butt</cref>.</def><br/
[<source>PJC</source>]</p>

<p><sn>2.</sn> <def>A drinking fountain on board a ship or at a naval station.</def><br/
[<source>PJC</source>]</p>

<p><sn>3.</sn> <def>The latest gossip; rumors.</def><br/
[<source>PJC</source>]</p>

<p><ent>Scutum</ent><br/
||<hw>Scu"tum</hw> <pr>(?)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Scuta</plw> <pr>(#)</pr>.</plu> <ety>[L.]</ety> <sn>1.</sn> <fld>(Rom. Antiq.)</fld> <def>An oblong shield made of boards or wickerwork covered with leather, with sometimes an iron rim; -- carried chiefly by the heavy-armed infantry.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(O. Eng. Law)</fld> <def>A penthouse or awning.</def> <mark>[Obs.]</mark>  <rj><au>Burrill.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <fld>(Zool.)</fld> <sd>(a)</sd> <def>The second and largest of the four parts forming the upper surface of a thoracic segment of an insect. It is preceded by the prescutum and followed by the scutellum. See the <xex>Illust.</xex> under <er>Thorax</er>.</def> <sd>(b)</sd> <def>One of the two lower valves of the operculum of a barnacle.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scybala</ent><br/
||<hw>Scyb"a*la</hw> <pr>(?)</pr>, <pos>n. pl.</pos> <ety>[NL., fr. Gr. <grk>sky`balon</grk> dung.]</ety> <fld>(Med.)</fld> <def>Hardened masses of feces.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scye</ent><br/
<hw>Scye</hw> <pr>(s<imac/)</pr>, <pos>n.</pos> <def>Arm scye, a cutter's term for the armhole or part of the armhole of the waist of a garment.</def> <mark>[Cant]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scyle</ent><br/
<hw>Scyle</hw> <pr>(s<imac/l)</pr>, <pos>v. t.</pos> <ety>[AS. <ets>scylan</ets> to withdraw or remove.]</ety> <def>To hide; to secrete; to conceal.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scylla</ent><br/
<hw>Scyl"la</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A dangerous rock on the Italian coast opposite the whirpool Charybdis on the coast of Sicily, -- both personified in classical literature as ravenous monsters. The passage between them was formerly considered perilous; hence, the saying <ldquo/Between Scylla and Charybdis,<rdquo/ signifying a great peril on either hand.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scyllaea</ent><br/
||<hw>Scyl*lae"a</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[NL. See <er>Scylla</er>.]</ety> <fld>(Zool.)</fld> <def>A genus of oceanic nudibranchiate mollusks having the small branched gills situated on the upper side of four fleshy lateral lobes, and on the median caudal crest.</def><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ In color and form these mollusks closely imitate the fronds of sargassum and other floating seaweeds among which they live.</note><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scyllarian</ent><br/
<hw>Scyl*la"ri*an</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Zool.)</fld> <def>One of a family (<fam>Scyllaridae</fam>) of macruran Crustacea, remarkable for the depressed form of the body, and the broad, flat antennae. Also used adjectively.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scyllite</ent><br/
<hw>Scyl"lite</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Chem.)</fld> <def>A white crystalline substance of a sweetish taste, resembling inosite and metameric with dextrose. It is extracted from the kidney of the dogfish (of the genus <gen>Scyllium</gen>), the shark, and the skate.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scymetar</ent><br/
<hw>Scym"e*tar</hw> <pr>(?)</pr>, <pos>n.</pos> <def>See <er>Scimiter</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scypha</ent><br/
||<hw>Scy"pha</hw> <pr>(?)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Scyphae</plw> <pr>(#)</pr>.</plu> <ety>[NL.]</ety> <fld>(Bot.)</fld> <def>See <er>Scyphus</er>, 2 <sd>(b)</sd>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scyphiform</ent><br/
<hw>Scy"phi*form</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>scyphus</ets> a cup + <ets>-form</ets>.]</ety> <fld>(Bot.)</fld> <def>Cup-shaped.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scyphistoma</ent><br/
||<hw>Scy*phis"to*ma</hw> <pr>(?)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Scyphistomata</plw> <pr>(#)</pr>, <plw>Scyphistomae</plw> <pr>(#)</pr>.</plu> <ety>[NL., fr. Gr. <grk>sky`fos</grk> a cup + <grk>sto`ma</grk> the mouth.]</ety> <fld>(Zool.)</fld> <def>The young attached larva of Discophora in the stage when it resembles a hydroid, or actinian.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scyphobranchii</ent><br/
||<hw>Scy`pho*bran"chi*i</hw> <pr>(?)</pr>, <pos>n. pl.</pos> <ety>[NL., from Gr. <grk>sky`fos</grk> a cup + <grk>bra`gchion</grk> a gill.]</ety> <fld>(Zool.)</fld> <def>An order of fishes including the blennioid and gobioid fishes, and other related families.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scyphomedusae</ent><br/
||<hw>Scy`pho*me*du"sae</hw> <pr>(?)</pr>, <pos>n. pl.</pos> <ety>[NL., fr. Gr. <grk>sky`fos</grk> cup + NL. <ets>medusa</ets>.]</ety> <fld>(Zool.)</fld> <def>Same as <er>Acraspeda</er>, or <er>Discophora</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scyphophori</ent><br/
||<hw>Scy*phoph"o*ri</hw> <pr>(?)</pr>, <pos>n. pl.</pos> <ety>[NL., fr. Gr. <grk>sky`fos</grk> a cup + <grk>fe`rein</grk> to bear.]</ety> <fld>(Zool.)</fld> <def>An order of fresh-water fishes inhabiting tropical Africa. They have rudimentary electrical organs on each side of the tail.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scyphus</ent><br/
<hw>Scy"phus</hw> <pr>(?)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Scyphi</plw> <pr>(#)</pr>.</plu> <ety>[L., a cup, Gr. <grk>sky`fos</grk>.]</ety> <sn>1.</sn> <fld>(Antiq.)</fld> <def>A kind of large drinking cup, -- used by Greeks and Romans, esp. by poor folk.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Bot.)</fld> <sd>(a)</sd> <def>The cup of a narcissus, or a similar appendage to the corolla in other flowers.</def> <sd>(b)</sd> <def>A cup-shaped stem or podetium in lichens. Also called <altname>scypha</altname>. See <xex>Illust.</xex> of <cref>Cladonia pyxidata</cref>, under <er>Lichen</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scythe</ent><br/
<hw>Scythe</hw> <pr>(s<imac/th)</pr>, <pos>n.</pos> <ety>[OE. <ets>sithe</ets>, AS. s<imac/<edh/e, <ets>sig<edh/e</ets>; akin to Icel. <ets>sig<edh/r</ets> a sickle, LG. <ets>segd</ets>, <ets>seged</ets>, <ets>seed</ets>, <ets>seid</ets>, OHG. <ets>segansa</ets> sickle, scythe, G. <ets>sense</ets> scythe, and to E. <ets>saw</ets> a cutting instrument. See <er>Saw</er>.]</ety> <altsp>[Written also <asp>sithe</asp> and <asp>sythe</asp>.]</altsp> <sn>1.</sn> <def>An instrument for mowing grass, grain, or the like, by hand, composed of a long, curving blade, with a sharp edge, made fast to a long handle, called a <xex>snath</xex>, which is bent into a form convenient for use.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The sharp-edged <qex>scythe</qex> shears up the spiring grass.</q> <rj><qau>Drayton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Whatever thing<br/
The <qex>scythe</qex> of Time mows down.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Antiq.)</fld> <def>A scythe-shaped blade attached to ancient war chariots.</def><br/
[<source>1913 Webster</source>]</p>

<p><-- p. 1296 pr=PCP --></p>

<p><ent>Scythe</ent><br/
<hw>Scythe</hw> <pr>(?)</pr>, <pos>v. t.</pos> <def>To cut with a scythe; to cut off as with a scythe; to mow.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>Time had not <qex>scythed</qex> all that youth begun.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scythed</ent><br/
<hw>Scythed</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Armed with scythes, as a chariot.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Chariots <qex>scythed</qex>,<br/
On thundering axles rolled.</q> <rj><qau>Glover.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scytheman</ent><br/
<hw>Scythe"man</hw> <pr>(?)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Scythemen</plw> <pr>(<?/)</pr>.</plu> <def>One who uses a scythe; a mower.</def>  <rj><au>Macaulay.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scythestone</ent><br/
<hw>Scythe"stone`</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A stone for sharpening scythes; a whetstone.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scythewhet</ent><br/
<hw>Scythe"whet`</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Zool.)</fld> <def>Wilson's thrush; -- so called from its note.</def> <mark>[Local, U.S.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scythian</ent><br/
<hw>Scyth"i*an</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Of or pertaining to Scythia (a name given to the northern part of Asia, and Europe adjoining to Asia), or its language or inhabitants.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Scythian lamb</b></col>. <fld>(Bot.)</fld> <cd>See <er>Barometz</er>.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scythian</ent><br/
<hw>Scyth"i*an</hw>, <pos>n.</pos> <sn>1.</sn> <def>A native or inhabitant of Scythia; specifically <fld>(Ethnol.)</fld>, one of a Slavonic race which in early times occupied Eastern Europe.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>The language of the Scythians.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Scytodermata</ent><br/
||<hw>Scy`to*der"ma*ta</hw> <pr>(?)</pr>, <pos>n. pl.</pos> <ety>[NL., fr. Gr. <?/ a hide + <?/ a skin.]</ety> <fld>(Zool.)</fld> <def>Same as <er>Holothurioidea</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sdain</ent><br/
<hw>Sdain</hw> <pr>(?)</pr>, <pos>v. & n.</pos> <def>Disdain.</def> <mark>[Obs.]</mark>  <rj><au>Spenser.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>'Sdeath</ent><br/
<hw>'Sdeath</hw> <pr>(?)</pr>, <pos>interj.</pos> <ety>[Corrupted fr. <ets>God's death</ets>.]</ety> <def>An exclamation expressive of impatience or anger.</def>  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sdeign</ent><br/
<hw>Sdeign</hw> <pr>(?)</pr>, <pos>v. t.</pos> <def>To disdain.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>But either <qex>sdeigns</qex> with other to partake.</q> <rj><qau>Spenser.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sea</ent><br/
<hw>Sea</hw> <pr>(s<emac/)</pr>, <pos>n.</pos> <ety>[OE. <ets>see</ets>, AS. <ets>s<aemac/</ets>; akin to D. <ets>zee</ets>, OS. & OHG. <ets>s<emac/o</ets>, G. <ets>see</ets>, OFries. <ets>se</ets>, Dan. <ets>s<oum/</ets>, Sw. <ets>sj<oum/</ets>, Icel. <ets>saer</ets>, Goth. <ets>saiws</ets>, and perhaps to L. <ets>saevus</ets> fierce, savage. <root/151a.]</ety> <sn>1.</sn> <def>One of the larger bodies of salt water, less than an ocean, found on the earth's surface; a body of salt water of second rank, generally forming part of, or connecting with, an ocean or a larger sea; <as>as, the Mediterranean <ex>Sea</ex>; the <ex>Sea</ex> of Marmora; the North <ex>Sea</ex>; the Carribean <ex>Sea</ex></as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>An inland body of water, esp. if large or if salt or brackish; <as>as, the Caspian <ex>Sea</ex>; <ex>the Sea</ex> of Aral</as>; sometimes, a small fresh-water lake; <as>as, the <ex>Sea</ex> of Galilee</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>The ocean; the whole body of the salt water which covers a large part of the globe.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>I marvel how the fishes live in the <qex>sea</qex>.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Ambiguous between <qex>sea</qex> and land<br/
The river horse and scaly crocodile.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>The swell of the ocean or other body of water in a high wind; motion or agitation of the water's surface; also, a single wave; a billow; <as>as, there was a high <ex>sea</ex> after the storm; the vessel shipped a <ex>sea</ex></as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <fld>(Jewish Antiq.)</fld> <def>A great brazen laver in the temple at Jerusalem; -- so called from its size.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>He made a molten <qex>sea</qex> of ten cubits from brim to brim, round in compass, and five cubits the height thereof.</q> <rj><qau>2 Chron. iv. 2.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>6.</sn> <def>Fig.: Anything resembling the sea in vastness; <as>as, a <ex>sea</ex> of glory</as>.</def>  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>All the space . . . was one <qex>sea</qex> of heads.</q> <rj><qau>Macaulay.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ <xex>Sea</xex> is often used in the composition of words of obvious signification; as, <xex>sea</xex>-bathed, <xex>sea</xex>-beaten, <xex>sea</xex>-bound, <xex>sea</xex>-bred, <xex>sea</xex>-circled, <xex>sea</xex>like, <xex>sea</xex>-nursed, <xex>sea</xex>-tossed, <xex>sea</xex>-walled, <xex>sea</xex>-worn, and the like. It is also used either adjectively or in combination with substantives; as, <xex>sea</xex> bird, <xex>sea</xex>-bird, or <xex>sea</xex>bird, <xex>sea</xex> acorn, or <xex>sea</xex>-acorn.</note><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>At sea</b></col>, <cd>upon the ocean; away from land; figuratively, without landmarks for guidance; lost; at the mercy of circumstances.</cd> <ldquo/To say the old man was <xex>at sea</xex> would be too feeble an expression.<rdquo/ <au>G. W. Cable</au> -- <col><b>At full sea</b></col> <cd>at the height of flood tide; hence, at the height.</cd> <ldquo/But now God's mercy was <xex>at full sea</xex>.<rdquo/ <au>Jer. Taylor.</au> -- <mcol><col><b>Beyond seas</b></col>, <it>or</it> <col><b>Beyond the sea</b></col> <it>or</it> <col><b>Beyond the seas</b></col></mcol> <fld>(Law)</fld>, <cd>out of the state, territory, realm, or country.</cd> <au>Wharton.</au> -- <col><b>Half seas over</b></col>, <cd>half drunk.</cd> <mark>[Colloq.]</mark> <au>Spectator.</au> -- <col><b>Heavy sea</b></col>, <cd>a sea in which the waves run high.</cd> -- <col><b>Long sea</b></col>, <cd>a sea characterized by the uniform and steady motion of long and extensive waves.</cd> -- <col><b>Short sea</b></col>, <cd>a sea in which the waves are short, broken, and irregular, so as to produce a tumbling or jerking motion.</cd> -- <col><b>To go to sea</b></col>, <cd>to adopt the calling or occupation of a sailor.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sea acorn</ent><br/
<hw>Sea" a"corn</hw> <pr>(?)</pr>. <fld>(Zool.)</fld> <def>An acorn barnacle (<gen>Balanus</gen>).</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sea adder</ent><br/
<hw>Sea" ad"der</hw> <pr>(?)</pr>. <fld>(Zool.)</fld> <sd>(a)</sd> <def>The European fifteen-spined stickleback (<spn>Gasterosteus spinachia</spn>); -- called also <altname>bismore</altname>.</def> <sd>(b)</sd> <def>The European tanglefish, or pipefish (<spn>Syngnathus acus</spn>).</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sea anchor</ent><br/
<hw>Sea" an"chor</hw> <pr>(?)</pr>. <fld>(Naut.)</fld> <def>See <cref>Drag sail</cref>, under 4th <er>Drag</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sea anemone</ent><br/
<hw>Sea" a*nem"o*ne</hw> <pr>(?)</pr>. <fld>(Zool.)</fld> <def>Any one of numerous species of soft-bodied Anthozoa, belonging to the order <ord>Actinaria</ord>; an actinian.</def><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ They have the oral disk surrounded by one or more circles of simple tapering tentacles, which are often very numerous, and when expanded somewhat resemble the petals of flowers, with colors varied and often very beautiful.</note><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sea ape</ent><br/
<hw>Sea" ape`</hw> <pr>(?)</pr>. <fld>(Zool.)</fld> <sd>(a)</sd> <def>The thrasher shark.</def> <sd>(b)</sd> <def>The sea otter.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sea apple</ent><br/
<hw>Sea" ap"ple</hw> <pr>(?)</pr>. <fld>(Bot.)</fld> <def>The fruit of a West Indian palm (<spn>Manicaria Plukenetii</spn>), often found floating in the sea.</def>  <rj><au>A. Grisebach.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sea arrow</ent><br/
<hw>Sea" ar"row</hw> <pr>(?)</pr>. <fld>(Zool.)</fld> <def>A squid of the genus <gen>Ommastrephes</gen>. See <er>Squid</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sea bank</ent><br/
<hw>Sea" bank`</hw> <pr>(?)</pr>. <sn>1.</sn> <def>The seashore.</def>  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A bank or mole to defend against the sea.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sea-bar</ent><br/
<hw>Sea"-bar`</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Zool.)</fld> <def>A tern.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sea barrow</ent><br/
<hw>Sea" bar"row</hw> <pr>(?)</pr>. <fld>(Zool.)</fld> <def>A sea purse.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sea bass</ent><br/
<hw>Sea" bass`</hw>. <pr>(<?/)</pr>. <fld>(Zool.)</fld> <sd>(a)</sd> <def>A large marine food fish (<spn>Serranus atrarius</spn> syn. <spn>Centropristis atrarius</spn>) which abounds on the Atlantic coast of the <country>United States</country>.  It is dark bluish, with black bands, and more or less varied with small white spots and blotches.  Called also, locally, <altname>blue bass</altname>, <altname>black sea bass</altname>, <altname>blackfish</altname>, <altname>bluefish</altname>, and <altname>black perch</altname>.</def> <sd>(b)</sd> <def>A California food fish (<spn>Cynoscion nobile</spn>); -- called also <altname>white sea bass</altname>, and <altname>sea salmon</altname>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sea bat</ent><br/
<hw>Sea" bat`</hw> <pr>(?)</pr>. <fld>(Zool.)</fld> <def>See <er>Batfish</er> <sd>(a)</sd>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Seabeach</ent><br/
<hw>Sea"beach`</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A beach lying along the sea.</def> <ldquo/The bleak <xex>seabeach</xex>.<rdquo/  <rj><au>Longfellow.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sea bean</ent><br/
<hw>Sea" bean</hw> <pr>(?)</pr>. <fld>(Bot.)</fld> <def>Same as <er>Florida bean</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sea bear</ent><br/
<hw>Sea" bear`</hw> <pr>(?)</pr>. <fld>(Zool.)</fld> <sd>(a)</sd> <def>Any fur seal. See under <er>Fur</er>.</def> <sd>(b)</sd> <def>The white bear.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Seabeard</ent><br/
<hw>Sea"beard`</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Bot.)</fld> <def>A green seaweed (<spn>Cladophora rupestris</spn>) growing in dense tufts.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sea beast</ent><br/
<hw>Sea" beast`</hw> <pr>(?)</pr>. <fld>(Zool.)</fld> <def>Any large marine mammal, as a seal, walrus, or cetacean.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sea bird</ent><br/
<hw>Sea" bird`</hw> <pr>(?)</pr>. <fld>(Zool.)</fld> <def>Any swimming bird frequenting the sea; a sea fowl.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sea blite</ent><br/
<hw>Sea" blite`</hw> <pr>(?)</pr>. <fld>(Bot.)</fld> <def>A plant (<spn>Suaeda maritima</spn>) of the Goosefoot family, growing in salt marshes.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sea-blubber</ent><br/
<hw>Sea"-blub"ber</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Zool.)</fld> <def>A jellyfish.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Seaboard</ent><br/
<hw>Sea"board`</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[<ets>Sea</ets> + <ets>board</ets>, F. <ets>bord</ets> side.]</ety> <def>The seashore; seacoast.</def>  <rj><au>Ld. Berners.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Seaboard</ent><br/
<hw>Sea"board`</hw>, <pos>a.</pos> <def>Bordering upon, or being near, the sea; seaside; seacoast; <as>as, a <ex>seaboard</ex> town</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Seaboard</ent><br/
<hw>Sea"board`</hw>, <pos>adv.</pos> <def>Toward the sea.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Seaboat</ent><br/
<hw>Sea"boat`</hw> <pr>(?)</pr>. <ety>[AS. <ets>s<aemac/b<amac/t</ets>.]</ety> <sn>1.</sn> <def>A boat or vessel adapted to the open sea; hence, a vessel considered with reference to her power of resisting a storm, or maintaining herself in a heavy sea; <as>as, a good <ex>sea boat</ex></as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Zool.)</fld> <def>A chiton.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Seabord</ent><br/
<hw>Sea"bord`</hw> <pr>(?)</pr>, <pos>n. & a.</pos> <def>See <er>Seaboard</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sea-bordering</ent><br/
<hw>Sea"-bor"der*ing</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Bordering on the sea; situated beside the sea.</def>  <rj><au>Drayton.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sea-born</ent><br/
<hw>Sea"-born`</hw> <pr>(?)</pr>, <pos>a.</pos> <sn>1.</sn> <def>Born of the sea; produced by the sea.</def> <ldquo/Neptune and his <xex>sea-born</xex> niece.<rdquo/  <rj><au>Waller.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Born at sea.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Seabound</ent><br/
<hw>Sea"bound`</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Bounded by the sea.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sea bow</ent><br/
<hw>Sea" bow`</hw> <pr>(?)</pr>. <def>See <cref>Marine rainbow</cref>, under <er>Rainbow</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sea boy</ent><br/
<hw>Sea" boy`</hw> <pr>(?)</pr>. <def>A boy employed on shipboard.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sea breach</ent><br/
<hw>Sea" breach`</hw> <pr>(?)</pr>. <def>A breaking or overflow of a bank or a dike by the sea.</def>  <rj><au>L'Estrange.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sea bream</ent><br/
<hw>Sea" bream`</hw> <pr>(?)</pr>. <fld>(Zool.)</fld> <def>Any one of several species of sparoid fishes, especially the common European species (<spn>Pagellus centrodontus</spn>), the Spanish (<spn>Pagellus Oweni</spn>), and the black sea bream (<spn>Cantharus lineatus</spn>); -- called also <altname>old wife</altname>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sea brief</ent><br/
<hw>Sea" brief`</hw> <pr>(?)</pr>. <def>Same as <er>Sea letter</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sea bug</ent><br/
<hw>Sea" bug`</hw> <pr>(?)</pr>. <fld>(Zool.)</fld> <def>A chiton.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sea-built</ent><br/
<hw>Sea"-built`</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Built at, in, or by the sea.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sea butterfly</ent><br/
<hw>Sea" but"ter*fly`</hw> <pr>(?)</pr>. <fld>(Zool.)</fld> <def>A pteropod.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sea cabbage</ent><br/
<hw>Sea" cab"bage</hw> <pr>(?; 48)</pr>. <fld>(Bot.)</fld> <def>See <cref>Sea kale</cref>, under <er>Kale</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sea calf</ent><br/
<hw>Sea" calf`</hw> <pr>(?)</pr>. <fld>(Zool.)</fld> <def>The common seal.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sea canary</ent><br/
<hw>Sea" ca*na"ry</hw> <pr>(?)</pr>. <ety>[So called from a whistling sound which it makes.]</ety> <fld>(Zool.)</fld> <def>The beluga, or white whale.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sea captain</ent><br/
<hw>Sea" cap"tain</hw> <pr>(?)</pr>. <def>The captain of a vessel that sails upon the sea.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sea card</ent><br/
<hw>Sea" card`</hw> <pr>(?)</pr>. <def>Mariner's card, or compass.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sea cat</ent><br/
<ent>Sea catfish</ent><br/
<mhw>{ <hw>Sea" cat`fish</hw> <pr>(?)</pr>. <hw>Sea" cat`</hw> <pr>(?)</pr>. }</mhw> <fld>(Zool.)</fld> <sd>(a)</sd> <def>The wolf fish.</def> <sd>(b)</sd> <def>Any marine siluroid fish, as <spn>Aelurichthys marinus</spn>, and <spn>Arinus felis</spn>, of the eastern coast of the United States. Many species are found on the coasts of Central and South America.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sea chart</ent><br/
<hw>Sea" chart`</hw> <pr>(?)</pr>. <def>A chart or map on which the lines of the shore, islands, shoals, harbors, etc., are delineated.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sea chickweed</ent><br/
<hw>Sea" chick"weed`</hw> <pr>(?)</pr>. <fld>(Bot.)</fld> <def>A fleshy plant (<spn>Arenaria peploides</spn>) growing in large tufts in the sands of the northern Atlantic seacoast; -- called also <altname>sea sandwort</altname>, and <altname>sea purslane</altname>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sea clam</ent><br/
<hw>Sea" clam`</hw> <pr>(?)</pr>. <fld>(Zool.)</fld> <def>Any one of the large bivalve mollusks found on the open seacoast, especially those of the family <fam>Mactridae</fam>, as the common American species. (<spn>Mactra solidissima</spn> or <spn>Spisula solidissima</spn>); -- called also <altname>beach clam</altname>, and <altname>surf clam</altname>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sea coal</ent><br/
<hw>Sea" coal`</hw> <pr>(?)</pr>. <def>Coal brought by sea; -- a name by which mineral coal was formerly designated in the south of England, in distinction from <xex>charcoal</xex>, which was brought by land.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Sea-coal facing</b></col> <fld>(Founding)</fld>, <cd>facing consisting of pulverized bituminous coal.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Seacoast</ent><br/
<hw>Sea"coast`</hw> <pr>(?)</pr>, <pos>n.</pos> <def>The shore or border of the land adjacent to the sea or ocean. Also used adjectively.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sea cob</ent><br/
<hw>Sea" cob`</hw> <pr>(?)</pr>. <fld>(Zool.)</fld> <def>The black-backed gull.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sea cock</ent><br/
<hw>Sea" cock`</hw> <pr>(?)</pr>. <sn>1.</sn> <def>In a steamship, a cock or valve close to the vessel's side, for closing a pipe which communicates with the sea.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Zool.)</fld> <sd>(a)</sd> <def>The black-bellied plover.</def> <sd>(b)</sd> <def>A gurnard, as the European red gurnard (<spn>Trigla pini</spn>).</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sea cocoa</ent><br/
<hw>Sea" co"coa</hw> <pr>(?)</pr>. <fld>(Bot.)</fld> <def>A magnificent palm (<spn>Lodoicea Sechellarum</spn>) found only in the Seychelles Islands. The fruit is an immense two-lobed nut. It was found floating in the Indian Ocean before the tree was known, and called <xex>sea cocoanut</xex>, and <xex>double cocoanut</xex>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sea colander</ent><br/
<hw>Sea" col"an*der</hw> <pr>(?)</pr>. <fld>(Bot.)</fld> <def>A large blackfish seaweed (<spn>Agarum Turneri</spn>), the frond of which is punctured with many little holes.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sea colewort</ent><br/
<hw>Sea" cole"wort`</hw> <pr>(?)</pr>. <fld>(Bot.)</fld> <def>Sea cabbage.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sea compass</ent><br/
<hw>Sea" com"pass</hw> <pr>(?)</pr>. <def>The mariner's compass. See under <er>Compass</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sea coot</ent><br/
<hw>Sea" coot`</hw> <pr>(?)</pr>. <fld>(Zool.)</fld> <def>A scoter duck.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sea corn</ent><br/
<hw>Sea" corn`</hw> <pr>(?)</pr>. <fld>(Zool.)</fld> <def>A yellow cylindrical mass of egg capsules of certain species of whelks (<gen>Buccinum</gen>), which resembles an ear of maize.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sea cow</ent><br/
<hw>Sea" cow`</hw> <pr>(?)</pr>. <fld>(Zool.)</fld> <sd>(a)</sd> <def>The mantee.</def> <sd>(b)</sd> <def>The dugong.</def> <sd>(c)</sd> <def>The walrus.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sea crayfish</ent><br/
<ent>Sea crawfish</ent><br/
<mhw>{ <hw>Sea" craw"fish`</hw> <pr>(?)</pr>. <hw>Sea" cray"fish`</hw> <pr>(?)</pr>. }</mhw> <fld>(Zool.)</fld> <def>Any crustacean of the genus <gen>Palinurus</gen> and allied genera, as the European spiny lobster (<spn>Palinurus vulgaris</spn>), which is much used as an article of food. See <er>Lobster</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sea crow</ent><br/
<hw>Sea" crow`</hw> <pr>(?)</pr>. <fld>(Zool.)</fld> <sd>(a)</sd> <def>The chough.</def> <mark>[Ireland]</mark> <sd>(b)</sd> <def>The cormorant.</def> <sd>(c)</sd> <def>The blackheaded pewit, and other gulls.</def> <sd>(d)</sd> <def>The skua.</def> <sd>(e)</sd> <def>The razorbill.</def> <mark>[Orkney Islands]</mark> <sd>(f)</sd> <def>The coot.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sea cucumber</ent><br/
<hw>Sea" cu"cum*ber</hw> <pr>(?)</pr>. <fld>(Zool.)</fld> <def>Any large holothurian, especially one of those belonging to the genus <gen>Pentacta</gen>, or <gen>Cucumaria</gen>, as the common American and European species. (<spn>Pentacta frondosa</spn>).</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sea dace</ent><br/
<hw>Sea" dace`</hw> <pr>(?)</pr>. <fld>(Zool.)</fld> <def>The European sea perch.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sea daffodil</ent><br/
<hw>Sea" daf"fo*dil</hw> <pr>(?)</pr>. <fld>(Bot.)</fld> <def>A European amaryllidaceous plant (<spn>Pancratium maritimum</spn>).</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sea devil</ent><br/
<hw>Sea" dev`il</hw> <pr>(?)</pr> <fld>(Zool.)</fld> <sd>(a)</sd> <def>Any very large ray, especially any species of the genus <gen>Manta</gen> or <gen>Cephaloptera</gen>, some of which become more than twenty feet across and weigh several tons. See also <cref>Ox ray</cref>, under <er>Ox</er>.</def> <sd>(b)</sd> <def>Any large cephalopod, as a large Octopus, or a giant squid (<gen>Architeuthis</gen>). See <er>Devilfish</er>.</def> <sd>(c)</sd> <def>The angler.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sea dog</ent><br/
<hw>Sea" dog`</hw> <pr>(?)</pr>. <sn>1.</sn> <fld>(Zool.)</fld> <def>The dogfish.</def> <sd>(b)</sd> <def>The common seal.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>An old sailor; a salt.</def> <mark>[Colloq.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sea dotterel</ent><br/
<hw>Sea" dot"ter*el</hw> <pr>(?)</pr>. <fld>(Zool.)</fld> <def>The turnstone.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sea dove</ent><br/
<hw>Sea" dove`</hw> <pr>(?)</pr>. <fld>(Zool.)</fld> <def>The little auk, or rotche. See <xex>Illust.</xex> of <er>Rotche</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sea dragon</ent><br/
<hw>Sea" drag"on</hw> <pr>(?)</pr>. <fld>(Zool.)</fld> <sd>(a)</sd> <def>A dragonet, or sculpin.</def> <sd>(b)</sd> <def>The pegasus.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sea drake</ent><br/
<hw>Sea" drake`</hw> <pr>(?)</pr>. <fld>(Zool.)</fld> <def>The pewit gull.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sea duck</ent><br/
<hw>Sea" duck`</hw> <pr>(?)</pr>. <fld>(Zool.)</fld> <def>Any one of numerous species of ducks which frequent the seacoasts and feed mainly on fishes and mollusks. The scoters, eiders, old squaw, and ruddy duck are examples. They may be distinguished by the lobate hind toe.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sea eagle</ent><br/
<hw>Sea" ea"gle</hw> <pr>(?)</pr>. <sn>1.</sn> <fld>(Zool.)</fld> <def>Any one of several species of fish-eating eagles of the genus <gen>Haliaeetus</gen> and allied genera, as the North Pacific sea eagle (<spn>Haliaeetus pelagicus</spn>), which has white shoulders, head, rump, and tail; the European white-tailed eagle (<spn>Haliaeetus albicilla</spn>); and the Indian white-tailed sea eagle, or fishing eagle (<spn>Polioaetus ichthyaetus</spn>).  The bald eagle and the osprey are also sometimes classed as <ex>sea eagles</ex>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Zool.)</fld> <def>The eagle ray. See under <er>Ray</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sea-ear</ent><br/
<hw>Sea"-ear`</hw> <pr>(s<emac/"<emac/r`)</pr>, <pos>n.</pos> <fld>(Zool.)</fld> <def>Any species of ear-shaped shells of the genus <gen>Haliotis</gen>. See <er>Abalone</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sea eel</ent><br/
<hw>Sea" eel`</hw> <pr>(s<emac/" <emac/l`)</pr>. <fld>(Zool.)</fld> <def>The conger eel.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sea egg</ent><br/
<hw>Sea" egg`</hw> <pr>(s<emac/" <ecr/g`)</pr>. <fld>(Zool.)</fld> <def>A sea urchin.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sea elephant</ent><br/
<hw>Sea" el"e*phant</hw> <pr>(s<emac/" <emac/"<esl/*f<ait/nt)</pr>. <fld>(Zool.)</fld> <def>A very large seal (<spn>Macrorhinus proboscideus</spn>) of the Antarctic seas, much hunted for its oil. It sometimes attains a length of thirty feet, and is remarkable for the prolongation of the nose of the adult male into an erectile elastic proboscis, about a foot in length. Another species of smaller size (<spn>Macrorhinus angustirostris</spn>) occurs on the coast of Lower California, but is now nearly extinct.</def><br/
[<source>1913 Webster</source>]</p>

<p><-- p. 1297 pr=PCP --></p>

<p><ent>Sea fan</ent><br/
<hw>Sea" fan"</hw> <pr>(s<emac/" f<acr/n`)</pr>. <fld>(Zool.)</fld> <def>Any gorgonian which branches in a fanlike form, especially <spn>Gorgonia flabellum</spn> of Florida and the West Indies.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Seafarer</ent><br/
<hw>Sea"far`er</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Sea + <ets>fare</ets>.]</ety> <def>One who follows the sea as a business; a mariner; a sailor.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Seafaring</ent><br/
<hw>Sea"far`ing</hw>, <pos>a.</pos> <def>Following the business of a mariner; <as>as, a <ex>seafaring</ex> man</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sea feather</ent><br/
<hw>Sea" feath"er</hw> <pr>(?)</pr>. <fld>(Zool.)</fld> <def>Any gorgonian which branches in a plumelike form.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sea fennel</ent><br/
<hw>Sea" fen"nel</hw> <pr>(?)</pr>. <fld>(Bot.)</fld> <def>Samphire.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sea fern</ent><br/
<hw>Sea" fern"</hw> <pr>(?)</pr>. <fld>(Zool.)</fld> <def>Any gorgonian which branches like a fern.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sea fight</ent><br/
<hw>Sea" fight`</hw> <pr>(?)</pr>. <def>An engagement between ships at sea; a naval battle.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sea fir</ent><br/
<hw>Sea" fir`</hw> <pr>(?)</pr>. <fld>(Zool.)</fld> <def>A sertularian hydroid, especially <xex>Sertularia abietina</xex>, which branches like a miniature fir tree.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sea flower</ent><br/
<hw>Sea" flow"er</hw> <pr>(?)</pr>. <fld>(Zool.)</fld> <def>A sea anemone, or any related anthozoan.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sea foam</ent><br/
<hw>Sea" foam`</hw> <pr>(?)</pr>. <sn>1.</sn> <def>Foam of sea water.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Min.)</fld> <def>Meerschaum; -- called also <altname>sea froth</altname>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sea fowl</ent><br/
<hw>Sea" fowl`</hw> <pr>(?)</pr>. <fld>(Zool.)</fld> <def>Any bird which habitually frequents the sea, as an auk, gannet, gull, tern, or petrel; also, all such birds, collectively.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sea fox</ent><br/
<hw>Sea" fox`</hw> <pr>(?)</pr>. <fld>(Zool.)</fld> <def>The thrasher shark. See <er>Thrasher</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sea froth</ent><br/
<hw>Sea" froth`</hw> <pr>(?; 115)</pr>. <def>See <er>Sea foam</er>, 2.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sea-gait</ent><br/
<ent>Sea-gate</ent><br/
<mhw>{ <hw>Sea"-gate`</hw>, <hw>Sea"-gait`</hw>  }</mhw>, <pos>n.</pos> <def>A long, rolling swell of the sea.</def>  <rj><au>Ham. Nav. Encyc.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sea gauge</ent><br/
<hw>Sea" gauge`</hw> <pr>(?)</pr>. <def>See under <er>Gauge</er>, <pos>n.</pos></def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sea girkin</ent><br/
<ent>Sea gherkin</ent><br/
<mhw>{ <hw>Sea" gher`kin</hw> <pr>(?)</pr>, <it>or</it> <hw>Sea" gir"kin</hw> <pr>(?)</pr> }</mhw>. <fld>(Zool.)</fld> <def>Any small holothurian resembling in form a gherkin.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sea ginger</ent><br/
<hw>Sea" gin"ger</hw> <pr>(?)</pr>. <fld>(Zool.)</fld> <def>A hydroid coral of the genus <gen>Millepora</gen>, especially <spn>Millepora alcicornis</spn>, of the <geog>West Indies</geog> and <state>Florida</state>.  So called because it stings the tongue like ginger. See <xex>Illust.</xex> under <er>Millepore</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sea girdles</ent><br/
<hw>Sea" gir"dles</hw> <pr>(?)</pr>. <fld>(Bot.)</fld> <def>A kind of kelp (<spn>Laminaria digitata</spn>) with palmately cleft fronds; -- called also <altname>sea wand</altname>, <altname>seaware</altname>, and <xex>tangle</xex>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Seagirt</ent><br/
<hw>Sea"girt`</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Surrounded by the water of the sea or ocean; <as>as, a <ex>seagirt</ex> isle</as>.</def>  <rj><au>Milton.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sea god</ent><br/
<hw>Sea" god`</hw> <pr>(?)</pr>. <def>A marine deity; a fabulous being supposed to live in, or have dominion over, the sea, or some particular sea or part of the sea, as Neptune.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sea goddess</ent><br/
<hw>Sea" god"dess</hw> <pr>(?)</pr>. <def>A goddess supposed to live in or reign over the sea, or some part of the sea.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Seagoing</ent><br/
<hw>Sea"go`ing</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Going upon the sea; especially, sailing upon the deep sea; -- used in distinction from <xex>coasting</xex> or <xex>river</xex>, as applied to vessels.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sea goose</ent><br/
<hw>Sea" goose`</hw> <pr>(?)</pr>. <fld>(Zool.)</fld> <def>A phalarope.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sea gown</ent><br/
<hw>Sea" gown`</hw> <pr>(?)</pr>. <def>A gown or frock with short sleeves, formerly worn by mariners.</def>  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sea grape</ent><br/
<hw>Sea" grape`</hw> <pr>(?)</pr>. <sn>1.</sn> <fld>(Bot.)</fld> <sd>(a)</sd> <def>The gulf weed. See under <er>Gulf</er>.</def> <sd>(b)</sd> <def>A shrubby plant (<spn>Coccoloba uvifera</spn>) growing on the sandy shores of tropical America, somewhat resembling the grapevine.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <pluf>pl.</pluf> <fld>(Zool.)</fld> <def>The clusters of gelatinous egg capsules of a squid (<gen>Loligo</gen>).</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sea grass</ent><br/
<hw>Sea" grass`</hw> <pr>(?)</pr>. <fld>(Bot.)</fld> <def>Eelgrass.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sea green</ent><br/
<hw>Sea" green`</hw> <pr>(?)</pr>. <def>The green color of sea water.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sea-green</ent><br/
<hw>Sea"-green`</hw>, <pos>a.</pos> <def>Of a beautiful bluish green color, like sea water on soundings.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sea gudgeon</ent><br/
<hw>Sea" gud"geon</hw> <pr>(?)</pr>. <fld>(Zool.)</fld> <def>The European black goby (<spn>Gobius niger</spn>).</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sea gull</ent><br/
<hw>Sea" gull`</hw> <pr>(?)</pr>. <fld>(Zool.)</fld> <def>Any gull living on the seacoast.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Seah</ent><br/
||<hw>Se"ah</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A Jewish dry measure containing one third of an ephah.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sea hare</ent><br/
<hw>Sea" hare`</hw> <pr>(?)</pr>. <fld>(Zool.)</fld> <def>Any tectibranchiate mollusk of the genus <gen>Aplysia</gen>. See <er>Aplysia</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sea hawk</ent><br/
<hw>Sea" hawk`</hw> <pr>(?)</pr>. <fld>(Zool.)</fld> <def>A jager gull.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sea heath</ent><br/
<hw>Sea" heath`</hw> <pr>(?)</pr>. <fld>(Bot.)</fld> <def>A low perennial plant (<spn>Frankenia laevis</spn>) resembling heath, growing along the seashore in Europe.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sea hedgehog</ent><br/
<hw>Sea" hedge"hog`</hw> <pr>(?)</pr>. <fld>(Zool.)</fld> <def>A sea urchin.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sea hen</ent><br/
<hw>Sea" hen`</hw> <pr>(?)</pr>. <fld>(Zool.)</fld> <def>The common guillemot; -- applied also to various other sea birds.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sea hog</ent><br/
<hw>Sea" hog`</hw> <pr>(?)</pr>. <fld>(Zool.)</fld> <def>The porpoise.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sea holly</ent><br/
<hw>Sea" hol"ly</hw> <pr>(?)</pr>. <fld>(Bot.)</fld> <def>An evergeen seashore plant (<spn>Eryngium maritimum</spn>). See <er>Eryngium</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sea holm</ent><br/
<hw>Sea" holm`</hw> <pr>(?)</pr>. <def>A small uninhabited island.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sea holm</ent><br/
<hw>Sea" holm`</hw>. <fld>(Bot.)</fld> <def>Sea holly.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sea horse</ent><br/
<hw>Sea" horse`</hw> <pr>(?)</pr>. <sn>1.</sn> <def>A fabulous creature, half horse and half fish, represented in classic mythology as driven by sea dogs or ridden by the Nereids. It is also depicted in heraldry. See <er>Hippocampus</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Zool.)</fld> <sd>(a)</sd> <def>The walrus.</def> <sd>(b)</sd> <def>Any fish of the genus Hippocampus.</def><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ In a passage of Dryden's, the word is supposed to refer to the hippopotamus.</note><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sea hulver</ent><br/
<hw>Sea" hul"ver</hw> <pr>(?)</pr>. <fld>(Bot.)</fld> <def>Sea holly.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sea-island</ent><br/
<hw>Sea"-is`land</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Of or pertaining to certain islands along the coast of South Carolina and Georgia; <as>as, <ex>sea-island</ex> cotton, a superior cotton of long fiber produced on those islands</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sea jelly</ent><br/
<hw>Sea" jel"ly</hw> <pr>(?)</pr>. <fld>(Zool.)</fld> <def>A medusa, or jellyfish.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Seak</ent><br/
<hw>Seak</hw> <pr>(?)</pr>, <pos>n.</pos> <def>Soap prepared for use in milling cloth.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sea kale</ent><br/
<hw>Sea" kale"</hw> <pr>(?)</pr>. <fld>(Bot.)</fld> <def>See under <er>Kale</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sea king</ent><br/
<hw>Sea" king`</hw> <pr>(?)</pr>. <def>One of the leaders among the Norsemen who passed their lives in roving the seas in search of plunder and adventures; a Norse pirate chief. See the Note under <er>Viking</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Seal</ent><br/
<hw>Seal</hw> <pr>(s<emac/l)</pr>, <pos>n.</pos> <ety>[OE. <ets>sele</ets>, AS. <ets>seolh</ets>; akin to OHG. <ets>selah</ets>, Dan. <ets>sael</ets>, Sw. <ets>sj<aum/l</ets>, Icel. <ets>selr</ets>.]</ety> <fld>(Zool.)</fld> <def>Any aquatic carnivorous mammal of the families <fam>Phocidae</fam> and <fam>Otariidae</fam>.</def><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ Seals inhabit seacoasts, and are found principally in the higher latitudes of both hemispheres. There are numerous species, bearing such popular names as <stype>sea lion</stype>, <stype>sea leopard</stype>, <stype>sea bear</stype>, or <stype>ursine seal</stype>, <stype>fur seal</stype>, and <stype>sea elephant</stype>. The bearded seal (<spn>Erignathus barbatus</spn>), the hooded seal (<spn>Cystophora cristata</spn>), and the ringed seal (<spn>Phoca foetida</spn>), are northern species. See also <cref>Eared seal</cref>, <cref>Harp seal</cref>, <cref>Monk seal</cref>, and <cref>Fur seal</cref>, under <er>Eared</er>, <er>Harp</er>, <er>Monk</er>, and <er>Fur</er>. Seals are much hunted for their skins and fur, and also for their oil, which in some species is very abundant.</note><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Harbor seal</b></col> <fld>(Zool.)</fld>, <cd>the common seal (<spn>Phoca vitulina</spn>). It inhabits both the North Atlantic and the North Pacific Ocean, and often ascends rivers; -- called also <altname>marbled seal</altname>, <altname>native seal</altname>, <altname>river seal</altname>, <altname>bay seal</altname>, <altname>land seal</altname>, <altname>sea calf</altname>, <altname>sea cat</altname>, <altname>sea dog</altname>, <altname>dotard</altname>, <altname>ranger</altname>, <altname>selchie</altname>, <altname>tangfish</altname>.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Seal</ent><br/
<hw>Seal</hw>, <pos>n.</pos> <ety>[OE. <ets>seel</ets>, OF. <ets>seel</ets>, F. <ets>sceau</ets>, fr. L. <ets>sigillum</ets> a little figure or image, a seal, dim. of <ets>signum</ets> a mark, sign, figure, or image. See <er>Sign</er>, <pos>n.</pos>, and cf. <er>Sigil</er>.]</ety> <sn>1.</sn> <def>An engraved or inscribed stamp, used for marking an impression in wax or other soft substance, to be attached to a document, or otherwise used by way of authentication or security.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Wax, wafer, or other tenacious substance, set to an instrument, and impressed or stamped with a seal; <as>as, to give a deed under hand and <ex>seal</ex></as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Till thou canst rail the <qex>seal</qex> from off my bond<br/
Thou but offend'st thy lungs to speak so loud.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>That which seals or fastens; esp., the wax or wafer placed on a letter or other closed paper, etc., to fasten it.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>That which confirms, ratifies, or makes stable; that which authenticates; that which secures; assurance.</def> <ldquo/Under the <xex>seal</xex> of silence.<rdquo/  <rj><au>Milton.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Like a red <qex>seal</qex> is the setting sun<br/
On the good and the evil men have done.</q> <rj><qau>Longfellow.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <def>An arrangement for preventing the entrance or return of gas or air into a pipe, by which the open end of the pipe dips beneath the surface of water or other liquid, or a deep bend or sag in the pipe is filled with the liquid; a draintrap.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Great seal</b></col>. <cd>See under <er>Great</er>.</cd> -- <col><b>Privy seal</b></col>. <cd>See under <er>Privy</er>, <pos>a.</pos></cd> -- <col><b>Seal lock</b></col>, <cd>a lock in which the keyhole is covered by a seal in such a way that the lock can not be opened without rupturing the seal.</cd> -- <col><b>Seal manual</b></col>. <cd>See under <er>Manual</er>, <pos>a.</pos></cd> -- <col><b>Seal ring</b></col>, <cd>a ring having a seal engraved on it, or ornamented with a device resembling a seal; a signet ring.</cd>  <rj><au>Shak.</au></rj></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Seal</ent><br/
<hw>Seal</hw>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Sealed</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Sealing</conjf>.]</vmorph> <ety>[OE. <ets>selen</ets>; cf. OF. <ets>seeler</ets>, <ets>seieler</ets>, F. <ets>sceller</ets>, LL. <ets>sigillare</ets>. See <er>Seal</er> a stamp.]</ety> <sn>1.</sn> <def>To set or affix a seal to; hence, to authenticate; to confirm; to ratify; to establish; <as>as, to <ex>seal</ex> a deed</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>And with my hand I <qex>seal</qex> my true heart's love.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To mark with a stamp, as an evidence of standard exactness, legal size, or merchantable quality; <as>as, to <ex>seal</ex> weights and measures; to <ex>seal</ex> silverware</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>To fasten with a seal; to attach together with a wafer, wax, or other substance causing adhesion; <as>as, to <ex>seal</ex> a letter</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>Hence, to shut close; to keep close; to make fast; to keep secure or secret.</def><br/
[<source>1913 Webster</source>]</p>

<p><q><qex>Seal</qex> up your lips, and give no words but <ldquo/mum<rdquo/.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <def>To fix, as a piece of iron in a wall, with cement, plaster, or the like.</def>  <rj><au>Gwilt.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>6.</sn> <def>To close by means of a seal; <as>as, to <ex>seal</ex> a drainpipe with water</as>. See 2d <er>Seal</er>, 5.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>7.</sn> <def>Among the Mormons, to confirm or set apart as a second or additional wife.</def> <mark>[Utah, U.S.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>If a man once married desires a second helpmate . . . she is <qex>sealed</qex> to him under the solemn sanction of the church.</q> <rj><qau>H. Stansbury.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Seal</ent><br/
<hw>Seal</hw>, <pos>v. i.</pos> <def>To affix one's seal, or a seal.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>I will <qex>seal</qex> unto this bond.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sea laces</ent><br/
<hw>Sea" la"ces</hw> <pr>(?)</pr>. <fld>(Bot.)</fld> <def>A kind of seaweed (<spn>Chorda Filum</spn>) having blackish cordlike fronds, often many feet long.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sea lamprey</ent><br/
<hw>Sea" lam"prey</hw> <pr>(?)</pr>. <fld>(Zool.)</fld> <def>The common lamprey.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sea language</ent><br/
<hw>Sea" lan"guage</hw> <pr>(?)</pr>. <def>The peculiar language or phraseology of seamen; sailor's cant.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sea lark</ent><br/
<hw>Sea" lark`</hw> <pr>(?)</pr>. <fld>(Zool.)</fld> <sd>(a)</sd> <def>The rock pipit (<spn>Anthus obscurus</spn>).</def> <sd>(b)</sd> <def>Any one of several small sandpipers and plovers, as the ringed plover, the turnstone, the dunlin, and the sanderling.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sea lavender</ent><br/
<hw>Sea" lav"en*der</hw> <pr>(?)</pr>. <fld>(Bot.)</fld> <def>See <cref>Marsh rosemary</cref>, under <er>Marsh</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sea lawyer</ent><br/
<hw>Sea" law"yer</hw> <pr>(?)</pr>. <fld>(Zool.)</fld> <def>The gray snapper. See under <er>Snapper</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Seal-brown</ent><br/
<hw>Seal"-brown`</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Of a rich dark brown color, like the fur of the fur seal after it is dyed.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sea legs</ent><br/
<hw>Sea" legs`</hw> <pr>(?)</pr>. <def>Legs able to maintain their possessor upright in stormy weather at sea, that is, ability to stand or walk steadily on deck when a vessel is rolling or pitching in a rough sea.</def> <mark>[Sailor's Cant]</mark>  <rj><au>Totten.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sea lemon</ent><br/
<hw>Sea" lem"on</hw> <pr>(?)</pr>. <fld>(Zool.)</fld> <def>Any one of several species of nudibranchiate mollusks of the genus Doris and allied genera, having a smooth, thick, convex yellow body.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sea leopard</ent><br/
<hw>Sea" leop"ard</hw> <pr>(?)</pr>. <fld>(Zool.)</fld> <def>Any one of several species of spotted seals, especially <spn>Ogmorhinus leptonyx</spn>, and <spn>Leptonychotes Weddelli</spn>, of the Antarctic Ocean. The North Pacific sea leopard is the harbor seal.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sealer</ent><br/
<hw>Seal"er</hw> <pr>(?)</pr>, <pos>n.</pos> <def>One who seals; especially, an officer whose duty it is to seal writs or instruments, to stamp weights and measures, or the like.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sealer</ent><br/
<hw>Sealer</hw>, <pos>n.</pos> <def>A mariner or a vessel engaged in the business of capturing seals.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sea letter</ent><br/
<hw>Sea" let"ter</hw> <pr>(?)</pr>. <fld>(Mar. Law.)</fld> <def>The customary certificate of national character which neutral merchant vessels are bound to carry in time of war; a passport for a vessel and cargo.</def> <rj><au>Burrill.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sea lettuce</ent><br/
<hw>Sea" let"tuce</hw> <pr>(?)</pr>. <fld>(Bot.)</fld> <def>The green papery fronds of several seaweeds of the genus <gen>Ulva</gen>, sometimes used as food.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sea level</ent><br/
<hw>Sea" lev"el</hw> <pr>(?)</pr>. <def>The level of the surface of the sea; any surface on the same level with the sea.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Selch</ent><br/
<ent>Sealgh</ent><br/
<mhw>{ <hw>Sealgh</hw> <pr>(?)</pr>, <hw>Selch</hw>, <pos>n.</pos> }</mhw>. <fld>(Zool.)</fld> <def>A seal.</def> <mark>[Scotch]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sea lily</ent><br/
<hw>Sea" lil"y</hw> <pr>(?)</pr>. <fld>(Zool.)</fld> <def>A crinoid.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sealing wax</ent><br/
<hw>Seal"ing wax`</hw> <pr>(?)</pr>. <def>A compound of the resinous materials, pigments, etc., used as a material for seals, as for letters, documents, etc.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sea lion</ent><br/
<hw>Sea" li"on</hw> <pr>(?)</pr>. <fld>(Zool.)</fld> <def>Any one of several large species of seals of the family <fam>Otariidae</fam> native of the Pacific Ocean, especially the southern sea lion (<spn>Otaria jubata</spn>) of the South American coast; the northern sea lion (<spn>Eumetopias Stelleri</spn>) found from California to Japan; and the black, or California, sea lion (<spn>Zalophus Californianus</spn>), which is common on the rocks near San Francisco.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sea loach</ent><br/
<hw>Sea" loach"</hw> <pr>(?)</pr>. <fld>(Zool.)</fld> <def>The three-bearded rockling. See <er>Rockling</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sea louse</ent><br/
<hw>Sea" louse`</hw> <pr>(?)</pr>. <fld>(Zool.)</fld> <def>Any one of numerous species of isopod crustaceans of <gen>Cymothoa</gen>, <gen>Livoneca</gen>, and allied genera, mostly parasites on fishes.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sealskin</ent><br/
<hw>Seal"skin`</hw> <pr>(?)</pr>, <pos>n.</pos> <def>The skin of a seal; the pelt of a seal prepared for use, esp. of the fur seal; also, a garment made of this material.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Seam</ent><br/
<hw>Seam</hw> <pr>(s<emac/m)</pr>, <pos>n.</pos> <ety>[See <er>Saim</er>.]</ety> <def>Grease; tallow; lard.</def> <mark>[Obs. or Prov. Eng.]</mark>  <rj><au>Shak.</au> <au>Dryden.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Seam</ent><br/
<hw>Seam</hw>, <pos>n.</pos> <ety>[OE. <ets>seem</ets>, <ets>seam</ets>, AS. <ets>se<aacute/m</ets>; akin to D. <ets>zoom</ets>, OHG. <ets>soum</ets>, G. <ets>saum</ets>, LG. <ets>soom</ets>, Icel. <ets>saumr</ets>, Sw. & Dan. <ets>s<oum/m</ets>, and E. <ets>sew</ets>. <root/ 156. See <er>Sew</er> to fasten with thread.]</ety> <sn>1.</sn> <def>The fold or line formed by sewing together two pieces of cloth or leather.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Hence, a line of junction; a joint; a suture, as on a ship, a floor, or other structure; the line of union, or joint, of two boards, planks, metal plates, etc.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Precepts should be so finely wrought together . . . that no coarse <qex>seam</qex> may discover where they join.</q> <rj><qau>Addison.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <fld>(Geol. & Mining)</fld> <def>A thin layer or stratum; a narrow vein between two thicker strata; <as>as, a <ex>seam</ex> of coal</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>A line or depression left by a cut or wound; a scar; a cicatrix.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Seam blast</b></col>, <cd>a blast made by putting the powder into seams or cracks of rocks.</cd> -- <col><b>Seam lace</b></col>, <cd>a lace used by carriage makers to cover seams and edges; -- called also <altname>seaming lace</altname>.</cd> -- <col><b>Seam presser</b></col>. <fld>(Agric.)</fld> <sd>(a)</sd> <cd>A heavy roller to press down newly plowed furrows</cd>. <sd>(b)</sd> <cd>A tailor's sadiron for pressing seams.</cd> <au>Knight.</au> -- <col><b>Seam set</b></col>, <cd>a set for flattering the seams of metal sheets, leather work, etc.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Seam</ent><br/
<hw>Seam</hw>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Seamed</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Seaming</conjf>.]</vmorph> <sn>1.</sn> <def>To form a seam upon or of; to join by sewing together; to unite.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To mark with something resembling a seam; to line; to scar.</def><br/
[<source>1913 Webster</source>]</p>

<p><q><qex>Seamed</qex> o'er with wounds which his own saber gave.</q> <rj><qau>Pope.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>To make the appearance of a seam in, as in knitting a stocking; hence, to knit with a certain stitch, like that in such knitting.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Seam</ent><br/
<hw>Seam</hw>, <pos>v. i.</pos> <def>To become ridgy; to crack open.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Later their lips began to parch and <qex>seam</qex>.</q> <rj><qau>L. Wallace.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Seam</ent><br/
<hw>Seam</hw>, <pos>n.</pos> <ety>[AS. <ets>se<aacute/m</ets>, LL. <ets>sauma</ets>, L. <ets>sagma</ets> a packsaddle, fr. Gr. <?/. See <er>Sumpter</er>.]</ety> <def>A denomination of weight or measure.</def> Specifically: <sd>(a)</sd> <def>The quantity of eight bushels of grain.</def> <ldquo/A <xex>seam</xex> of oats.<rdquo/ <au>P. Plowman.</au> <sd>(b)</sd> <def>The quantity of 120 pounds of glass.</def> <mark>[Eng.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sea-maid</ent><br/
<hw>Sea"-maid`</hw> <pr>(?)</pr>, <pos>n.</pos> <sn>1.</sn> <def>The mermaid.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A sea nymph.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sea-mail</ent><br/
<hw>Sea"-mail`</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[<ets>Sea</ets> + (perhaps) <ets>Mall</ets> Mally, for <ets>Mary</ets>; hence, Prov. E. <ets>mally</ets> a hare.]</ety> <fld>(Zool.)</fld> <def>A gull; the mew.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Seaman</ent><br/
<hw>Sea"man</hw> <pr>(?)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Seamen</plw> <pr>(<?/)</pr>.</plu> <def>A merman; the male of the mermaid.</def> <mark>[R.]</mark> <ldquo/Not to mention mermaids or <xex>seamen</xex>.<rdquo/  <rj><au>Locke.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Seaman</ent><br/
<hw>Sea"man</hw> <pr>(?)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Seamen</plw> <pr>(#)</pr>.</plu> <ety>[AS. <ets>saeman</ets>.]</ety> <def>One whose occupation is to assist in the management of ships at sea; a mariner; a sailor; -- applied both to officers and common mariners, but especially to the latter. Opposed to <contr>landman</contr>, or <contr>landsman</contr>.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Able seaman</b></col>, <cd>a sailor who is practically conversant with all the duties of common seamanship.</cd> -- <col><b>Ordinary seaman</b></col>. <cd>See <er>Ordinary</er>.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Seamanlike</ent><br/
<hw>Sea"man*like`</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Having or showing the skill of a practical seaman.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Seamanship</ent><br/
<hw>Sea"man*ship</hw>, <pos>n.</pos> <def>The skill of a good seaman; the art, or skill in the art, of working a ship.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sea mantis</ent><br/
<hw>Sea" man"tis</hw> <pr>(?)</pr>. <fld>(Zool.)</fld> <def>A squilla.</def><br/
[<source>1913 Webster</source>]</p>

<p><-- p. 1298 pr=PCP --></p>

<p><ent>Sea marge</ent><br/
<hw>Sea" marge`</hw> <pr>(?)</pr>. <def>Land which borders on the sea; the seashore.</def>  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>You are near the <qex>sea marge</qex> of a land teeming with life.</q> <rj><qau>J. Burroughs.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Seamark</ent><br/
<hw>Sea"mark`</hw> <pr>(?)</pr>, <pos>n.</pos> <def>Any elevated object on land which serves as a guide to mariners; a beacon; a landmark visible from the sea, as a hill, a tree, a steeple, or the like.</def>  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sea mat</ent><br/
<hw>Sea" mat`</hw> <pr>(?)</pr>. <fld>(Zool.)</fld> <def>Any bryozoan of the genus <gen>Flustra</gen> or allied genera which form frondlike corals.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sea maw</ent><br/
<hw>Sea" maw`</hw> <pr>(?)</pr>. <fld>(Zool.)</fld> <def>The sea mew.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Seamed</ent><br/
<hw>Seamed</hw> <pr>(?)</pr>, <pos>a.</pos> <fld>(Falconry)</fld> <def>Out of condition; not in good condition; -- said of a hawk.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sea-mell</ent><br/
<hw>Sea"-mell`</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Zool.)</fld> <def>The sea mew.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sea mew</ent><br/
<hw>Sea" mew`</hw> <pr>(?)</pr>. <fld>(Zool.)</fld> <def>A gull; the mew.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sea mile</ent><br/
<hw>Sea" mile`</hw> <pr>(?)</pr>. <def>A geographical mile. See <er>Mile</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sea milkwort</ent><br/
<hw>Sea" milk"wort`</hw> <pr>(?)</pr>. <fld>(Bot.)</fld> <def>A low, fleshy perennial herb (<spn>Glaux maritima</spn>) found along northern seashores.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Seaming</ent><br/
<hw>Seam"ing</hw> <pr>(?)</pr>, <pos>n.</pos> <sn>1.</sn> <def>The act or process of forming a seam or joint.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Fishing)</fld> <def>The cord or rope at the margin of a seine, to which the meshes of the net are attached.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Seaming machine</b></col>, <cd>a machine for uniting the edges of sheet-metal plates by bending them and pinching them together.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Seamless</ent><br/
<hw>Seam"less</hw>, <pos>a.</pos> <def>Without a seam.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Christ's <qex>seamless</qex> coat, all of a piece.</q> <rj><qau>Jer. Taylor.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sea monk</ent><br/
<hw>Sea" monk`</hw> <pr>(?)</pr>. <fld>(Zool.)</fld> <def>See <er>Monk seal</er>, under <er>Monk</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sea monster</ent><br/
<hw>Sea" mon"ster</hw> <pr>(?)</pr>. <fld>(Zool.)</fld> <def>Any large sea animal.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sea moss</ent><br/
<hw>Sea" moss`</hw> <pr>(?; 115)</pr>. <fld>(Zool.)</fld> <def>Any branched marine bryozoan resembling moss.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sea mouse</ent><br/
<hw>Sea" mouse`</hw> <pr>(?)</pr>. <fld>(Zool.)</fld> <sd>(a)</sd> <def>A dorsibranchiate annelid, belonging to <gen>Aphrodite</gen> and allied genera, having long, slender, hairlike setae on the sides.</def> <sd>(b)</sd> <def>The dunlin.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Seamster</ent><br/
<hw>Seam"ster</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[See <er>Seamstress</er>.]</ety> <def>One who sews well, or whose occupation is to sew.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Seamstress</ent><br/
<hw>Seam"stress</hw> <pr>(?; 277)</pr>, <pos>n.</pos> <ety>[From older <ets>seamster</ets>, properly fem., AS. <ets>se<aacute/mestre</ets>. See <er>Seam</er>.]</ety> <def>A woman whose occupation is sewing; a needlewoman.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Seamstressy</ent><br/
<hw>Seam"stress*y</hw> <pr>(?)</pr>, <pos>n.</pos> <def>The business of a seamstress.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sea mud</ent><br/
<hw>Sea" mud`</hw> <pr>(?)</pr>. <def>A rich slimy deposit in salt marshes and along the seashore, sometimes used as a manure; -- called also <altname>sea ooze</altname>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Seamy</ent><br/
<hw>Seam"y</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Having a seam; containing seams, or showing them.</def> <ldquo/Many a <xex>seamy</xex> scar.<rdquo/  <rj><au>Burns.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Everything has its fair, as well as its <qex>seamy</qex>, side.</q> <rj><qau>Sir W. Scott.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sean</ent><br/
<hw>Sean</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A seine. See <er>Seine</er>.</def> <mark>[Prov. Eng.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Seance</ent><br/
||<hw>S<eacute/`ance"</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F., fr. L. <ets>sedens</ets>, <ets>-entis</ets>, p. pr. of <ets>sedere</ets> to sit. See <er>Sit</er>.]</ety> <def>A session, as of some public body; especially, a meeting of spiritualists to receive spirit communications, so called.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sea needle</ent><br/
<hw>Sea" nee"dle</hw> <pr>(?)</pr>. <fld>(Zool.)</fld> <def>See <er>Garfish</er> <sd>(a)</sd>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sea nettle</ent><br/
<hw>Sea" net`tle</hw> <pr>(?)</pr>. <def>A jellyfish, or medusa.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Seannachie</ent><br/
<hw>Sean"na*chie</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Gael. <ets>seanachaidh</ets>.]</ety> <def>A bard among the Highlanders of Scotland, who preserved and repeated the traditions of the tribes; also, a genealogist.</def> <altsp>[Written also <asp>sennachy</asp>.]</altsp> <mark>[Scot.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sea onion</ent><br/
<hw>Sea" on"ion</hw> <pr>(?)</pr>. <fld>(Bot.)</fld> <def>The officinal squill. See <er>Squill</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sea ooze</ent><br/
<hw>Sea" ooze`</hw> <pr>(?)</pr>. <def>Same as <cref>Sea mud</cref>.</def>  <rj><au>Mortimer.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sea orange</ent><br/
<hw>Sea" or"ange</hw> <pr>(?)</pr>. <fld>(Zool.)</fld> <def>A large American holothurian (<spn>Lophothuria Fabricii</spn>) having a bright orange convex body covered with finely granulated scales.  Its expanded tentacles are bright red.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sea-orb</ent><br/
<hw>Sea"-orb`</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Zool.)</fld> <def>A globefish.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sea otter</ent><br/
<hw>Sea" ot"ter</hw> <pr>(?)</pr>. <fld>(Zool.)</fld> <def>An aquatic carnivore (<spn>Enhydris lutris</spn> syn. <spn>Enhydris marina</spn>) found in the <geog>North Pacific Ocean</geog>.   Its fur is highly valued, especially by the Chinese.  It is allied to the common otter, but is larger, with feet more decidedly webbed.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Sea-otter's cabbage</b></col> <fld>(Bot.)</fld>, <cd>a gigantic kelp of the Pacific Ocean (<spn>Nereocystis Lutkeana</spn>). See <er>Nereocystis</er>.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sea owl</ent><br/
<hw>Sea" owl`</hw> <pr>(s<emac/" oul`)</pr>. <fld>(Zool.)</fld> <def>The lumpfish.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sea pad</ent><br/
<hw>Sea" pad`</hw> <pr>(s<emac/" p<acr/d`)</pr>. <fld>(Zool.)</fld> <def>A starfish.</def></p>

<p><ent>Sea parrot</ent><br/
<hw>Sea" par"rot</hw> <pr>(s<emac/" p<acr/r"r<ucr/t)</pr>. <fld>(Zool.)</fld> <def>The puffin.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sea partridge</ent><br/
<hw>Sea" par"tridge</hw> <pr>(?)</pr>. <fld>(Zool.)</fld> <def>The gilthead (<spn>Crenilabrus melops</spn>), a fish of the British coasts.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sea pass</ent><br/
<hw>Sea" pass`</hw> <pr>(?)</pr>. <def>A document carried by neutral merchant vessels in time of war, to show their nationality; a sea letter or passport. See <er>Passport</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sea peach</ent><br/
<hw>Sea" peach`</hw> <pr>(?)</pr>. <fld>(Zool.)</fld> <def>A beautiful American ascidian (<spn>Cynthia pyriformis</spn> syn. <spn>Halocynthia pyriformis</spn>) having the size, form, velvety surface, and color of a ripe peach.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sea pear</ent><br/
<hw>Sea" pear`</hw> <pr>(s<emac/" p<acir/r`)</pr>. <fld>(Zool.)</fld> <def>A pedunculated ascidian of the genus <gen>Boltonia</gen>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sea-pen</ent><br/
<hw>Sea"-pen"</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Zool.)</fld> <def>A pennatula.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sea perch</ent><br/
<hw>Sea" perch`</hw> <pr>(?)</pr>. <fld>(Zool.)</fld> <sd>(a)</sd> <def>The European bass (<spn>Roccus lupus</spn> syn. <spn>Labrax lupus</spn>); -- called also <altname>sea dace</altname>.</def> <sd>(b)</sd> <def>The cunner.</def> <sd>(c)</sd> <def>The sea bass.</def> <sd>(d)</sd> <def>The name is applied also to other species of fishes.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sea pheasant</ent><br/
<hw>Sea" pheas"ant</hw> <pr>(?)</pr>. <fld>(Zool.)</fld> <def>The pintail duck.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sea pie</ent><br/
<hw>Sea" pie</hw> <pr>(s<emac/" p<imac/`)</pr>. <fld>(Zool.)</fld> <def>The oyster catcher, a limicoline bird of the genus <gen>Haematopus</gen>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sea pie</ent><br/
<hw>Sea" pie`</hw>. <def>A dish of crust or pastry and meat or fish, etc., cooked together in alternate layers, -- a common food of sailors; <as>as, a three-decker <ex>sea pie</ex></as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Seapiece</ent><br/
<hw>Sea"piece`</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A picture representing a scene at sea; a marine picture.</def>  <rj><au>Addison.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sea piet</ent><br/
<hw>Sea" pi"et</hw> <pr>(?)</pr>. <fld>(Zool.)</fld> <def>See 1st <er>Sea pie</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sea pig</ent><br/
<hw>Sea" pig`</hw> <pr>(?)</pr>. <fld>(Zool.)</fld> <sd>(a)</sd> <def>A porpoise or dolphin.</def> <sd>(b)</sd> <def>A dugong.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sea pigeon</ent><br/
<hw>Sea" pi"geon</hw> <pr>(?)</pr>. <def>The common guillemot.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sea pike</ent><br/
<hw>Sea" pike`</hw> <pr>(?)</pr>. <fld>(Zool.)</fld> <sd>(a)</sd> <def>The garfish.</def> <sd>(b)</sd> <def>A large serranoid food fish (<spn>Centropomus undecimalis</spn>) found on both coasts of America; -- called also <altname>robalo</altname>.</def> <sd>(c)</sd> <def>The merluce.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sea pincushion</ent><br/
<hw>Sea" pin`cush`ion</hw> <pr>(?)</pr>. <fld>(Zool.)</fld> <sd>(a)</sd> <def>A sea purse.</def> <sd>(b)</sd> <def>A pentagonal starfish.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sea pink</ent><br/
<hw>Sea" pink`</hw> <pr>(?)</pr>. <fld>(Bot.)</fld> <def>See <er>Thrift</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sea plover</ent><br/
<hw>Sea" plov"er</hw> <pr>(?)</pr>. <def>The black-bellied plover.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sea poker</ent><br/
<ent>Sea poacher</ent><br/
<mhw>{ <hw>Sea" poach"er</hw> <pr>(s<emac/" p<omac/ch"<etil/r)</pr>. <hw>Sea" pok"er</hw> <pr>(s<emac/" p<omac/k"<etil/r)</pr>. }</mhw> <fld>(Zool.)</fld> <def>The lyrie.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sea pool</ent><br/
<hw>Sea" pool`</hw> <pr>(?)</pr>. <def>A pool of salt water.</def>  <rj><au>Spenser.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sea poppy</ent><br/
<hw>Sea" pop"py</hw> <pr>(?)</pr>. <fld>(Bot.)</fld> <def>The horn poppy. See under <er>Horn</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sea porcupine</ent><br/
<hw>Sea" por"cu*pine</hw> <pr>(?)</pr>. <fld>(Zool.)</fld> <def>Any fish of the genus <gen>Diodon</gen>, and allied genera, whose body is covered with spines. See <xex>Illust.</xex> under <er>Diodon</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sea pork</ent><br/
<hw>Sea" pork`</hw> <pr>(?)</pr>. <fld>(Zool.)</fld> <def>An American compound ascidian (<spn>Amoroecium stellatum</spn>) which forms large whitish masses resembling salt pork.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Seaport</ent><br/
<hw>Sea"port`</hw> <pr>(s<emac/"p<omac/rt`)</pr>, <pos>n.</pos> <def>A port on the seashore, or one accessible for seagoing vessels. Also used adjectively; <as>as, a <ex>seaport</ex> town</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Seapoy</ent><br/
<hw>Sea"poy</hw> <pr>(s<emac/"poy)</pr>, <pos>n.</pos> <def>See <er>Sepoy</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sea pudding</ent><br/
<hw>Sea" pud"ding</hw> <pr>(?)</pr>. <fld>(Zool.)</fld> <def>Any large holothurian.</def> <mark>[Prov. Eng.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sea purse</ent><br/
<hw>Sea" purse`</hw> <pr>(s<emac/" p<ucir/rs`)</pr>. <fld>(Zool.)</fld> <def>The horny egg case of a skate, and of certain sharks.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sea purslane</ent><br/
<hw>Sea" purs"lane</hw> <pr>(?)</pr>. <fld>(Bot.)</fld> <def>See under <er>Purslane</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sea pye</ent><br/
<hw>Sea" pye`</hw> <pr>(?)</pr>. <fld>(Zool.)</fld> <def>See 1st <er>Sea pie</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sea pyot</ent><br/
<hw>Sea" py"ot</hw> <pr>(?)</pr>. <fld>(Zool.)</fld> <def>See 1st <er>Sea pie</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sea quail</ent><br/
<hw>Sea" quail`</hw> <pr>(?)</pr>. <fld>(Zool.)</fld> <def>The turnstone.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Seaquake</ent><br/
<hw>Sea"quake`</hw> <pr>(s<emac/"kw<amac/k`)</pr>, <pos>n.</pos> <def>A quaking of the sea.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sere</ent><br/
<ent>Sear</ent><br/
<mhw>{ <hw>Sear</hw>, <hw>Sere</hw> <pr>(s<emac/r)</pr>, }</mhw> <pos>a.</pos> <ety>[OE. <ets>seer</ets>, AS. <ets>se<aacute/r</ets> (assumed) fr. <ets>se<aacute/rian</ets> to wither; akin to D. <ets>zoor</ets> dry, LG. <ets>soor</ets>, OHG. <ets>sor<emac/n</ets> to wither, Gr. <grk>a"y`ein</grk> to parch, to dry, Skr. <ets><cced/ush</ets> (for <ets>sush</ets>) to dry, to wither, Zend <ets>hush</ets> to dry.  <root/152.  Cf. <er>Austere</er>, <er>Sorrel</er>, <pos>a.</pos>]</ety> <def>Dry; withered; no longer green; -- applied to leaves.</def> <rj><au>Milton.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>I have lived long enough; my way of life<br/
Is fall'n into the <qex>sear</qex>, the yellow leaf.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sear</ent><br/
<hw>Sear</hw>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Seared</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Searing</conjf>.]</vmorph> <ety>[OE. <ets>seeren</ets>, AS. <ets>se<aacute/rian</ets>. See <er>Sear</er>, <pos>a.</pos>]</ety> <sn>1.</sn> <def>To wither; to dry up.</def>  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To burn (the surface of) to dryness and hardness; to cauterize; to expose to a degree of heat such as changes the color or the hardness and texture of the surface; to scorch; to make callous; <as>as, to <ex>sear</ex> the skin or flesh</as>. Also used figuratively.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>I'm <qex>seared</qex> with burning steel.</q> <rj><qau>Rowe.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>It was in vain that the amiable divine tried to give salutary pain to that <qex>seared</qex> conscience.</q> <rj><qau>Macaulay.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>The discipline of war, being a discipline in destruction of life, is a discipline in callousness. Whatever sympathies exist are <qex>seared</qex>.</q> <rj><qau>H. Spencer.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ <xex>Sear</xex> is allied to <xex>scorch</xex> in signification; but it is applied primarily to animal flesh, and has special reference to the effect of heat in marking the surface <xex>hard</xex>. <xex>Scorch</xex> is applied to flesh, cloth, or any other substance, and has no reference to the effect of hardness.</note><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>To sear up</b></col>, <cd>to close by searing.</cd> <ldquo/Cherish veins of good humor, and <xex>sear</xex> up those of ill.<rdquo/ <au>Sir W. Temple.</au></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sear</ent><br/
<hw>Sear</hw>, <pos>n.</pos> <ety>[F. <ets>serre</ets> a grasp, pressing, fr. L. <ets>sera</ets>. See <er>Serry</er>.]</ety> <def>The catch in a gunlock by which the hammer is held cocked or half cocked.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Sear spring</b></col>, <cd>the spring which causes the sear to catch in the notches by which the hammer is held.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sea rat</ent><br/
<hw>Sea" rat`</hw> <pr>(?)</pr>. <sn>1.</sn> <def>A pirate.</def> <mark>[R.]</mark>  <rj><au>Massinger.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Zool.)</fld> <def>The chimaera.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sea raven</ent><br/
<hw>Sea" ra"ven</hw> <pr>(?)</pr>. <fld>(Zool.)</fld> <sd>(a)</sd> <def>An American cottoid fish (<spn>Hemitripterus Americanus</spn>) allied to the sculpins, found on the northern Atlantic coasts.</def> <sd>(b)</sd> <def>The cormorant.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Searce</ent><br/
<hw>Searce</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[See <er>Sarse</er>.]</ety> <def>A fine sieve.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Searce</ent><br/
<hw>Searce</hw>, <pos>v. t.</pos> <def>To sift; to bolt.</def> <mark>[Obs.]</mark>  <rj><au>Mortimer.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Searcer</ent><br/
<hw>Sear"cer</hw> <pr>(?)</pr>, <pos>n.</pos> <sn>1.</sn> <def>One who sifts or bolts.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A searce, or sieve.</def> <mark>[Obs.]</mark>  <rj><au>Holland.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Search</ent><br/
<hw>Search</hw> <pr>(?)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Searched</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Searching</conjf>.]</vmorph> <ety>[OE. <ets>serchen</ets>, <ets>cerchen</ets>, OF. <ets>cerchier</ets>, F. <ets>chercher</ets>, L. <ets>circare</ets> to go about, fr. L. <ets>circum</ets>, <ets>circa</ets>, around. See <er>Circle</er>.]</ety> <sn>1.</sn> <def>To look over or through, for the purpose of finding something; to examine; to explore; <as>as, to <ex>search</ex> the city</as>.</def> <ldquo/<xex>Search</xex> the Scriptures.<rdquo/  <rj><au>John v. 39.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>They are come to <qex>search</qex> the house.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q><qex>Search</qex> me, O God, and know my heart.</q> <rj><qau>Ps. cxxxix. 23.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To inquire after; to look for; to seek.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>I will both <qex>search</qex> my sheep, and seek them out.</q> <rj><qau>Ezek. xxxiv. 11.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Enough is left besides to <qex>search</qex> and know.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>To examine or explore by feeling with an instrument; to probe; <as>as, to <ex>search</ex> a wound</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>To examine; to try; to put to the test.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>To search out</b></col>, <cd>to seek till found; to find by seeking; <as>as, to <ex>search out</ex> truth</as>.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><syn><b>Syn.</b> -- To explore; examine; scrutinize; seek; investigate; pry into; inquire.</syn><br/
[<source>1913 Webster</source>]</p>

<p><ent>Search</ent><br/
<hw>Search</hw>, <pos>v. i.</pos> <def>To seek; to look for something; to make inquiry, exploration, or examination; to hunt.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Once more <qex>search</qex> with me.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>It sufficeth that they have once with care sifted the matter, and <qex>searched</qex> into all the particulars.</q> <rj><qau>Locke.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Search</ent><br/
<hw>Search</hw>, <pos>n.</pos> <ety>[Cf. OF. <ets>cerche</ets>. See <er>Search</er>, <pos>v. t.</pos>]</ety> <def>The act of seeking or looking for something; quest; inquiry; pursuit for finding something; examination.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Thus the orb he roamed<br/
With narrow <qex>search</qex>, and with inspection deep<br/
Considered every creature.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Nor did my <qex>search</qex> of liberty begin<br/
Till my black hairs were changed upon my chin.</q> <rj><qau>Dryden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Right of search</b></col> <fld>(Mar. Law)</fld>, <cd>the right of the lawfully commissioned cruisers of belligerent nations to examine and search private merchant vessels on the high seas, for the enemy's property or for articles contraband of war.</cd> -- <col><b>Search warrant</b></col> <fld>(Law)</fld>, <cd>a warrant legally issued, authorizing an examination or search of a house, or other place, for goods stolen, secreted, or concealed.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><syn><b>Syn.</b> -- Scrutiny; examination; exploration; investigation; research; inquiry; quest; pursuit.</syn><br/
[<source>1913 Webster</source>]</p>

<p><ent>Searchable</ent><br/
<hw>Search"a*ble</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Capable of being searched.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Searchableness</ent><br/
<hw>Search"a*ble*ness</hw>, <pos>n.</pos> <def>Quality of being searchable.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Searcher</ent><br/
<hw>Search"er</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Cf. OF. <ets>cercheor</ets> inspector.]</ety> <def>One who, or that which, searches or examines; a seeker; an inquirer; an examiner; a trier.</def> Specifically: <sd>(a)</sd> <def>Formerly, an officer in London appointed to examine the bodies of the dead, and report the cause of death.</def> <au>Graunt.</au> <sd>(b)</sd> <def>An officer of the customs whose business it is to search ships, merchandise, luggage, etc.</def> <sd>(c)</sd> <def>An inspector of leather.</def> <mark>[Prov. Eng.]</mark> <sd>(d)</sd> <fld>(Gun.)</fld> <def>An instrument for examining the bore of a cannon, to detect cavities.</def> <sd>(e)</sd> <def>An implement for sampling butter; a butter trier.</def> <sd>(j)</sd> <fld>(Med.)</fld> <def>An instrument for feeling after calculi in the bladder, etc.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Searching</ent><br/
<hw>Search"ing</hw>, <pos>a.</pos> <def>Exploring thoroughly; scrutinizing; penetrating; trying; <as>as, a <ex>searching</ex> discourse; a <ex>searching</ex> eye</as>.</def> <ldquo/Piercing, <xex>searching</xex>, biting, cold.<rdquo/  <rj><au>Dickens.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p>-- <wordforms><wf>Search"ing*ly</wf>, <pos>adv.</pos> -- <wf>Search"ing*ness</wf>, <pos>n.</pos></wordforms><br/
[<source>1913 Webster</source>]</p>

<p><ent>Searchless</ent><br/
<hw>Search"less</hw>, <pos>a.</pos> <def>Impossible to be searched; inscrutable; impenetrable.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Searchlight</ent><br/
<hw>Search"light`</hw> <pr>(?)</pr>, <pos>n.</pos> <sd>(a)</sd> <def>An apparatus for projecting a powerful beam of light of approximately parallel rays, usually devised so that it can be swiveled about.</def> <sd>(b)</sd> <def>The beam of light projecting by this apparatus.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Searcloth</ent><br/
<hw>Sear"cloth`</hw> <pr>(?; 115)</pr>, <pos>n.</pos> <def>Cerecloth.</def>  <rj><au>Mortimer.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Searcloth</ent><br/
<hw>Sear"cloth</hw>, <pos>v. t.</pos> <def>To cover, as a sore, with cerecloth.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Seared</ent><br/
<hw>Seared</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Scorched; cauterized; hence, figuratively, insensible; not susceptible to moral influences.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>A <qex>seared</qex> conscience and a remorseless heart.</q> <rj><qau>Macaulay.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Searedness</ent><br/
<hw>Sear"ed*ness</hw> <pr>(?)</pr>, <pos>n.</pos> <def>The state of being seared or callous; insensibility.</def>  <rj><au>Bp. Hall.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sea reed</ent><br/
<hw>Sea" reed`</hw> <pr>(?)</pr>. <fld>(Bot.)</fld> <def>The sea-sand reed. See under <er>Reed</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sea risk</ent><br/
<hw>Sea" risk</hw> <pr>(?)</pr>. <def>Risk of injury, destruction, or loss by the sea, or while at sea.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sea robber</ent><br/
<hw>Sea" rob"ber</hw> <pr>(?)</pr>. <def>A pirate; a sea rover.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sea robin</ent><br/
<hw>Sea" rob"in</hw> <pr>(?)</pr>. <def>See under <er>Robin</er>, and <xex>Illustration</xex> in Appendix.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sea rocket</ent><br/
<hw>Sea" rock"et</hw> <pr>(?)</pr>.<fld>(Bot.)</fld> <def>See under <er>Rocket</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sea room</ent><br/
<hw>Sea" room`</hw> <pr>(?)</pr>. <fld>(Naut.)</fld> <def>Room or space at sea for a vessel to maneuver, drive, or scud, without peril of running ashore or aground.</def>  <rj><au>Totten.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sea rover</ent><br/
<hw>Sea" rov"er</hw> <pr>(?)</pr>. <def>One that cruises or roves the sea for plunder; a sea robber; a pirate; also, a piratical vessel.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sea-roving</ent><br/
<hw>Sea"-rov"ing</hw>, <pos>a.</pos> <def>Cruising at random on the ocean.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sea salmon</ent><br/
<hw>Sea" salm"on</hw> <pr>(?)</pr>. <fld>(Zool.)</fld> <sd>(a)</sd> <def>A young pollock.</def> <sd>(b)</sd> <def>The spotted squeteague.</def> <sd>(c)</sd> <def>See <er>Sea bass</er> <sd>(b)</sd>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sea salt</ent><br/
<hw>Sea" salt`</hw> <pr>(?)</pr>. <def>Common salt, obtained from sea water by evaporation.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sea sandpiper</ent><br/
<hw>Sea" sand"pi`per</hw> <pr>(?)</pr>. <fld>(Zool.)</fld> <def>The purple sandpiper.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sea sandwort</ent><br/
<hw>Sea" sand"wort`</hw> <pr>(?)</pr>. <fld>(Bot.)</fld> <def>See <er>Sea chickweed</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sea saurian</ent><br/
<hw>Sea" sau"ri*an</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Zool.)</fld> <def>Any marine saurian; esp. <fld>(Paleon.)</fld>, the large extinct species of <gen>Mosasaurus</gen>, <gen>Ichthyosaurus</gen>, <gen>Plesiosaurus</gen>, and related genera.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Seascape</ent><br/
<hw>Sea"scape</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Cf. <er>Landscape</er>.]</ety> <def>A picture representing a scene at sea.  Compare <er>landscape</er>.</def> <rj><au>Thackeray.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><-- p. 1299 pr=PCP --></p>

<p><ent>Sea scorpion</ent><br/
<hw>Sea" scor"pi*on</hw> <pr>(?)</pr>. <fld>(Zool.)</fld> <sd>(a)</sd> <def>A European sculpin (<spn>Cottus scorpius</spn>) having the head armed with short spines.</def> <sd>(b)</sd> <def>The scorpene.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sea scurf</ent><br/
<hw>Sea" scurf`</hw> <pr>(?)</pr>. <fld>(Zool.)</fld> <def>Any bryozoan which forms rounded or irregular patches of coral on stones, seaweeds, etc.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sea serpent</ent><br/
<hw>Sea" ser"pent</hw> <pr>(?)</pr>. <sn>1.</sn> <fld>(Zool.)</fld> <def>Any marine snake. See <er>Sea snake</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Zool.)</fld> <def>A large marine animal of unknown nature, often reported to have been seen at sea, but never yet captured.</def><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ Many accounts of sea serpents are imaginary or fictitious; others are greatly exaggerated and distorted by incompetent observers; but a number have been given by competent and trustworthy persons, which indicate that several diverse animals have been called sea serpents. Among these are, apparently, several large snakelike fishes, as the oar fish, or ribbon fish (<gen>Regalecus</gen>), and huge conger eels. Other accounts probably refer to the giant squids (<gen>Architeuthis</gen>). Some of the best accounts seem to describe a marine saurian, like the fossil Mosasauri, which were large serpentlike creatures with paddles.</note><br/
[<source>1913 Webster</source>]</p>

<p><ent>Seashell</ent><br/
<hw>Sea"shell`</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Zool.)</fld> <def>The shell of any marine mollusk.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Seashore</ent><br/
<hw>Sea"shore`</hw> <pr>(?)</pr>, <pos>n.</pos> <sn>1.</sn> <def>The coast of the sea; the land that lies adjacent to the sea or ocean.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Law)</fld> <def>All the ground between the ordinary high-water and low-water marks.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Seasick</ent><br/
<hw>Sea"sick`</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Affected with seasickness.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Seasickness</ent><br/
<hw>Sea"sick`ness</hw>, <pos>n.</pos> <def>The peculiar sickness, characterized by nausea and prostration, which is caused by the pitching or rolling of a vessel.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Seaside</ent><br/
<hw>Sea"side`</hw> <pr>(?)</pr>, <pos>n.</pos> <def>The land bordering on, or adjacent to, the sea; the seashore. Also used adjectively.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sea slater</ent><br/
<hw>Sea" slat"er</hw> <pr>(?)</pr>. <fld>(Zool.)</fld> <def>Any isopod crustacean of the genus <gen>Ligia</gen>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sea slug</ent><br/
<hw>Sea" slug`</hw> <pr>(?)</pr>. <fld>(Zool.)</fld> <sd>(a)</sd> <def>A holothurian.</def> <sd>(b)</sd> <def>A nudibranch mollusk.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sea snail</ent><br/
<hw>Sea" snail`</hw> <pr>(?)</pr>. <fld>(Zool.)</fld> <sd>(a)</sd> <def>A small fish of the genus <gen>Liparis</gen>, having a ventral sucker. It lives among stones and seaweeds.</def> <sd>(b)</sd> <def>Any small creeping marine gastropod, as the species of <gen>Littorina</gen>, <gen>Natica</gen>, etc.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sea snake</ent><br/
<hw>Sea" snake`</hw> <pr>(?)</pr>. <fld>(Zool.)</fld> <def>Any one of many species of venomous aquatic snakes of the family <fam>Hydrophidae</fam>, having a flattened tail and living entirely in the sea, especially in the warmer parts of the Indian and Pacific Oceans. They feed upon fishes, and are mostly of moderate size, but some species become eight or ten feet long and four inches broad.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sea snipe</ent><br/
<hw>Sea" snipe`</hw> <pr>(?)</pr>. <fld>(Zool.)</fld> <sd>(a)</sd> <def>A sandpiper, as the knot and dunlin.</def> <sd>(b)</sd> <def>The bellows fish.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Season</ent><br/
<hw>Sea"son</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[OE. <ets>sesoun</ets>, F. <ets>saison</ets>, properly, the sowing time, fr. L. <ets>satio</ets> a sowing, a planting, fr. <ets>serere</ets>, <ets>satum</ets>, to sow, plant; akin to E. <ets>sow</ets>, v., to scatter, as seed.]</ety> <sn>1.</sn> <def>One of the divisions of the year, marked by alterations in the length of day and night, or by distinct conditions of temperature, moisture, etc., caused mainly by the relative position of the earth with respect to the sun. In the north temperate zone, four seasons, namely, spring, summer, autumn, and winter, are generally recognized. Some parts of the world have three seasons, -- the dry, the rainy, and the cold; other parts have but two, -- the dry and the rainy.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The several <qex>seasons</qex> of the year in their beauty.</q> <rj><qau>Addison.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Hence, a period of time, especially as regards its fitness for anything contemplated or done; a suitable or convenient time; proper conjuncture; <as>as, the <ex>season</ex> for planting; the <ex>season</ex> for rest</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The <qex>season</qex>, prime for sweetest scents and airs.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>A period of time not very long; a while; a time.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Thou shalt be blind, not seeing the sun for a <qex>season</qex>.</q> <rj><qau>Acts xiii. 11.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>That which gives relish; seasoning.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>You lack the <qex>season</qex> of all natures, sleep.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>In season</b></col>, <cd>in good time, or sufficiently early for the purpose.</cd> -- <col><b>Out of season</b></col>, <cd>beyond or out of the proper time or the usual or appointed time.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Season</ent><br/
<hw>Sea"son</hw>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Seasoned</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Seasoning</conjf>.]</vmorph> <sn>1.</sn> <def>To render suitable or appropriate; to prepare; to fit.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>He is fit and <qex>seasoned</qex> for his passage.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To fit for any use by time or habit; to habituate; to accustom; to inure; to ripen; to mature; <as>as, to <ex>season</ex> one to a climate</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>Hence, to prepare by drying or hardening, or removal of natural juices; <as>as, to <ex>season</ex> timber</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>To fit for taste; to render palatable; to give zest or relish to; to spice; <as>as, to <ex>season</ex> food</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <def>Hence, to fit for enjoyment; to render agreeable.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>You <qex>season</qex> still with sports your serious hours.</q> <rj><qau>Dryden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>The proper use of wit is to <qex>season</qex> conversation.</q> <rj><qau>Tillotson.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>6.</sn> <def>To qualify by admixture; to moderate; to temper.</def> <ldquo/When mercy <xex>seasons</xex> justice.<rdquo/  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>7.</sn> <def>To imbue; to tinge or taint.</def> <ldquo/Who by his tutor being <xex>seasoned</xex> with the love of the truth.<rdquo/  <rj><au>Fuller.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q><qex>Season</qex> their younger years with prudent and pious principles.</q> <rj><qau>Jer. Taylor.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>8.</sn> <def>To copulate with; to impregnate.</def> <mark>[R.]</mark>  <rj><au>Holland.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Season</ent><br/
<hw>Sea"son</hw> <pr>(?)</pr>, <pos>v. i.</pos> <sn>1.</sn> <def>To become mature; to grow fit for use; to become adapted to a climate.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To become dry and hard, by the escape of the natural juices, or by being penetrated with other substance; <as>as, timber <ex>seasons</ex> in the sun</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>To give token; to savor.</def> <mark>[Obs.]</mark>  <rj><au>Beau. & Fl.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Seasonable</ent><br/
<hw>Sea"son*a*ble</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Occurring in good time, in due season, or in proper time for the purpose; suitable to the season; opportune; timely; <as>as, a <ex>seasonable</ex> supply of rain</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Mercy is <qex>seasonable</qex> in the time of affliction.</q> <rj><qau>Ecclus. xxxv. 20.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p>-- <wordforms><wf>Sea"son*a*ble*ness</wf>, <pos>n.</pos> -- <wf>Sea"son*a*bly</wf>, <pos>adv.</pos></wordforms><br/
[<source>1913 Webster</source>]</p>

<p><ent>Seasonage</ent><br/
<hw>Sea"son*age</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A seasoning.</def> <mark>[Obs.]</mark>  <rj><au>South.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Seasonal</ent><br/
<hw>Sea"son*al</hw> <pr>(?)</pr>, <pos>a.</pos> <sn>1.</sn> <def>Of or pertaining to the seasons.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Occurring or being used in a specific season; <as>as, <ex>seasonal</ex> items for sale</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Seasonal dimorphism</b></col> <fld>(Zool.)</fld>, <cd>the condition of having two distinct varieties which appear at different seasons, as certain species of butterflies in which the spring brood differs from the summer or autumnal brood.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>seasonal affective disorder</ent><br/
<hw>seasonal affective disorder</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A form of mild depression that occurs in winters, associated with reduction in the amount of sunlight.  It is characterized by oversleeping, irritability, and sometimes overeating.  It can be treated by light therapy and usually disappears with the arrival of spring.</def> <br/
[<source>PJC</source>]</p>

<p><ent>Seasoner</ent><br/
<hw>Sea"son*er</hw> <pr>(?)</pr>, <pos>n.</pos> <def>One who, or that which, seasons, or gives a relish; a seasoning.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Seasoning</ent><br/
<hw>Sea"son*ing</hw>, <pos>n.</pos> <sn>1.</sn> <def>The act or process by which anything is seasoned.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>That which is added to any species of food, to give it a higher relish, as salt, spices, etc.; a condiment.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>Hence, something added to enhance enjoyment or relieve dullness; <as>as, wit is the <ex>seasoning</ex> of conversation</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Political speculations are of so dry and austere a nature, that they will not go down with the public without frequent <qex>seasonings</qex>.</q> <rj><qau>Addison.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Seasoning tub</b></col> <fld>(Bakery)</fld>, <cd>a trough in which dough is set to rise.</cd>  <rj><au>Knight.</au></rj></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Seasonless</ent><br/
<hw>Sea"son*less</hw>, <pos>a.</pos> <def>Without succession of the seasons.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sea spider</ent><br/
<hw>Sea" spi"der</hw> <pr>(?)</pr>. <fld>(Zool.)</fld> <sd>(a)</sd> <def>Any maioid crab; a spider crab. See <er>Maioid</er>, and <cref>Spider crab</cref>, under <er>Spider</er>.</def> <sd>(b)</sd> <def>Any pycnogonid.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sea squirt</ent><br/
<hw>Sea" squirt`</hw> <pr>(?)</pr>. <fld>(Zool.)</fld> <def>An ascidian. See <xex>Illust.</xex> under <er>Tunicata</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sea star</ent><br/
<hw>Sea" star`</hw> <pr>(?)</pr>. <fld>(Zool.)</fld> <def>A starfish, or brittle star.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sea surgeon</ent><br/
<hw>Sea" sur"geon</hw> <pr>(?)</pr>. <fld>(Zool.)</fld> <def>A surgeon fish.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sea swallow</ent><br/
<hw>Sea" swal"low</hw> <pr>(?)</pr>. <sn>1.</sn> <fld>(Zool.)</fld> <sd>(a)</sd> <def>The common tern.</def> <sd>(b)</sd> <def>The storm petrel.</def> <sd>(c)</sd> <def>The gannet.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Her.)</fld> <def>See <cref>Cornish chough</cref>, under <er>Chough</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Seat</ent><br/
<hw>Seat</hw> <pr>(s<emac/t)</pr>, <pos>n.</pos> <ety>[OE. <ets>sete</ets>, Icel. <ets>saeti</ets>; akin to Sw. <ets>s<aum/te</ets>, Dan. <ets>saede</ets>, MHG. <ets>s<amac/ze</ets>, AS. <ets>set</ets>, <ets>setl</ets>, and E. <ets>sit</ets>. <root/154. See <er>Sit</er>, and cf. <er>Settle</er>, <pos>n.</pos>]</ety> <sn>1.</sn> <def>The place or thing upon which one sits; hence; anything made to be sat in or upon, as a chair, bench, stool, saddle, or the like.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>And Jesus . . . overthrew the tables of the money changers, and the <qex>seats</qex> of them that sold doves.</q> <rj><qau>Matt. xxi. 12.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>The place occupied by anything, or where any person or thing is situated, resides, or abides; a site; an abode, a station; a post; a situation.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Where thou dwellest, even where Satan's <qex>seat</qex> is.</q> <rj><qau>Rev. ii. 13.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>He that builds a fair house upon an ill <qex>seat</qex> committeth himself to prison.</q> <rj><qau>Bacon.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>A <qex>seat</qex> of plenty, content, and tranquillity.</q> <rj><qau>Macaulay.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>That part of a thing on which a person sits; <as>as, the <ex>seat</ex> of a chair or saddle; the <ex>seat</ex> of a pair of pantaloons</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>A sitting; a right to sit; regular or appropriate place of sitting; <as>as, a <ex>seat</ex> in a church; a <ex>seat</ex> for the season in the opera house</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <def>Posture, or way of sitting, on horseback.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>She had so good a <qex>seat</qex> and hand she might be trusted with any mount.</q> <rj><qau>G. Eliot.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>6.</sn> <fld>(Mach.)</fld> <def>A part or surface on which another part or surface rests; <as>as, a valve <ex>seat</ex></as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Seat worm</b></col> <fld>(Zool.)</fld>, <cd>the pinworm.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Seat</ent><br/
<hw>Seat</hw>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Seated</conjf>; <pos>p. pr. & vb. n.</pos> <conjf>Seating</conjf>.]</vmorph> <sn>1.</sn> <def>To place on a seat; to cause to sit down; <as>as, to <ex>seat</ex> one's self</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The guests were no sooner <qex>seated</qex> but they entered into a warm debate.</q> <rj><qau>Arbuthnot.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To cause to occupy a post, site, situation, or the like; to station; to establish; to fix; to settle.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Thus high . . . is King Richard <qex>seated</qex>.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>They had <qex>seated</qex> themselves in New Guiana.</q> <rj><qau>Sir W. Raleigh.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>To assign a seat to, or the seats of; to give a sitting to; <as>as, to <ex>seat</ex> a church, or persons in a church</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>To fix; to set firm.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>From their foundations, loosening to and fro,<br/
They plucked the <qex>seated</qex> hills.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <def>To settle; to plant with inhabitants; as to <xex>seat</xex> a country.</def> <mark>[Obs.]</mark>  <rj><au>W. Stith.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>6.</sn> <def>To put a seat or bottom in; <as>as, to <ex>seat</ex> a chair</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Seat</ent><br/
<hw>Seat</hw>, <pos>v. i.</pos> <def>To rest; to lie down.</def> <mark>[Obs.]</mark>  <rj><au>Spenser.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sea tang</ent><br/
<hw>Sea" tang`</hw> <pr>(?)</pr>. <fld>(Bot.)</fld> <def>A kind of seaweed; tang; tangle.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>To their nests of sedge and <qex>sea tang</qex>.</q> <rj><qau>Longfellow.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sea term</ent><br/
<hw>Sea" term`</hw> <pr>(?)</pr>. <def>A term used specifically by seamen; a nautical word or phrase.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sea thief</ent><br/
<hw>Sea" thief`</hw> <pr>(?)</pr>. <def>A pirate.</def>  <rj><au>Drayton.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sea thongs</ent><br/
<hw>Sea" thongs`</hw> <pr>(?; 115)</pr>. <fld>(Bot.)</fld> <def>A kind of blackish seaweed (<spn>Himanthalia lorea</spn>) found on the northern coasts of the Atlantic. It has a thonglike forking process rising from a top-shaped base.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Seating</ent><br/
<hw>Seat"ing</hw> <pr>(s<emac/t"<icr/ng)</pr>, <pos>n.</pos> <sn>1.</sn> <def>The act of providing with a seat or seats; <as>as, the <ex>seating</ex> of an audience</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>The act of making seats; also, the material for making seats; <as>as, cane <ex>seating</ex></as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sea titling</ent><br/
<hw>Sea" tit"ling</hw> <pr>(?)</pr>. <fld>(Zool.)</fld> <def>The rock pipit.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Seatless</ent><br/
<hw>Seat"less</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Having no seat.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sea toad</ent><br/
<hw>Sea" toad`</hw> <pr>(?)</pr>. <fld>(Zool.)</fld> <sd>(a)</sd> <def>A sculpin.</def> <sd>(b)</sd> <def>A toadfish.</def> <sd>(c)</sd> <def>The angler.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sea trout</ent><br/
<hw>Sea" trout`</hw> <pr>(?)</pr>. <fld>(Zool.)</fld> <sd>(a)</sd> <def>Any one of several species of true trouts which descend rivers and enter the sea after spawning, as the European bull trout and salmon trout, and the eastern American spotted trout.</def> <sd>(b)</sd> <def>The common squeteague, and the spotted squeteague.</def> <sd>(c)</sd> <def>A California fish of the family <fam>Chiridae</fam>, especially <spn>Hexagrammus decagrammus</spn>; -- called also <altname>spotted rock trout</altname>. See <cref>Rock trout</cref>, under <er>Rock</er>.</def> <sd>(d)</sd> <def>A California sciaenoid fish (<spn>Cynoscion nobilis</spn>); -- called also <altname>white sea bass</altname>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sea trumpet</ent><br/
<hw>Sea" trum"pet</hw> <pr>(?)</pr>. <sn>1.</sn> <fld>(Bot.)</fld> <def>A great blackish seaweed of the Southern Ocean, having a hollow and expanding stem and a pinnate frond, sometimes twenty feet long.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Zool.)</fld> <def>Any large marine univalve shell of the genus Triton. See <er>Triton</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sea turn</ent><br/
<hw>Sea" turn`</hw> <pr>(?)</pr>. <def>A breeze, gale, or mist from the sea.</def>  <rj><au>Ham. Nav. Encyc.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sea turtle</ent><br/
<hw>Sea" tur"tle</hw> <pr>(?)</pr>. <fld>(Zool.)</fld> <sd>(a)</sd> <def>Any one of several very large species of chelonians having the feet converted into paddles, as the green turtle, hawkbill, loggerhead, and leatherback. They inhabit all warm seas.</def> <sd>(b)</sd> <def>The sea pigeon, or guillemot.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sea unicorn</ent><br/
<hw>Sea" u"ni*corn</hw> <pr>(?)</pr>. <fld>(Zool.)</fld> <def>The narwhal.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sea urchin</ent><br/
<hw>Sea" ur"chin</hw> <pr>(?)</pr>. <fld>(Zool.)</fld> <def>Any one of numerous species of echinoderms of the order <ord>Echinoidea</ord>.</def>  <note>When living they are covered with movable spines which are often long and sharp.</note><br/
[<source>1913 Webster</source>]</p>

<p><ent>Seave</ent><br/
<hw>Seave</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Cf. Dan. <ets>siv</ets>, Sw. <ets>s<aum/f</ets>, Icel. <ets>sef</ets>.]</ety> <def>A rush.</def> <mark>[Prov. Eng.]</mark>  <rj><au>Halliwell.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Seavy</ent><br/
<hw>Seav`y</hw>, <pos>a.</pos> <def>Overgrown with rushes.</def> <mark>[Prov. Eng.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sea wall</ent><br/
<hw>Sea" wall`</hw> <pr>(?)</pr>. <ety>[AS. <ets>s<ae/weall</ets>.]</ety> <def>A wall, or embankment, to resist encroachments of the sea.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sea-walled</ent><br/
<hw>Sea"-walled`</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Surrounded, bounded, or protected by the sea, as if by a wall.</def>  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Seawant</ent><br/
<ent>Seawan</ent><br/
<mhw>{ <hw>Sea"wan</hw> <pr>(?)</pr>, <hw>Sea"want</hw> <pr>(?)</pr>, }</mhw> <pos>n.</pos> <def>The name used by the Algonquin Indians for the shell beads which passed among the Indians as money.</def><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ <ex>Seawan</ex> was of two kinds; <stype>wampum</stype>, white, and <stype>suckanhock</stype>, black or purple, -- the former having half the value of the latter. Many writers, however, use the terms <ex>seawan</ex> and <stype>wampum</stype> indiscriminately.</note> <rj><au>Bartlett.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Seawand</ent><br/
<hw>Sea"wand`</hw> . <fld>(Bot.)</fld> <def>See <er>Sea girdles</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Seaward</ent><br/
<hw>Sea"ward</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Directed or situated toward the sea.</def>  <rj><au>Donne.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Two still clouds . . . sparkled on their <qex>seaward</qex> edges like a frosted fleece.</q> <rj><qau>G. W. Cable.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Seaward</ent><br/
<hw>Sea"ward</hw>, <pos>adv.</pos> <def>Toward the sea.</def>  <rj><au>Drayton.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Seaware</ent><br/
<hw>Sea"ware`</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Cf. AS. <ets>s<aemac/w<amac/r</ets> seaweed.]</ety> <fld>(Bot.)</fld> <def>Seaweed; esp., coarse seaweed. See <er>Ware</er>, and <er>Sea girdles</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Seaweed</ent><br/
<hw>Sea"weed`</hw> <pr>(?)</pr>, <pos>n.</pos> <sn>1.</sn> <def>Popularly, any plant or plants growing in the sea.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Bot.)</fld> <def>Any marine plant of the class Algae, as kelp, dulse, <gen>Fucus</gen>, <gen>Ulva</gen>, etc.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sea whip</ent><br/
<hw>Sea" whip`</hw> <pr>(?)</pr>. <fld>(Zool.)</fld> <def>A gorgonian having a simple stem.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sea widgeon</ent><br/
<hw>Sea" wid"geon</hw> <pr>(?)</pr>. <fld>(Zool.)</fld> <sd>(a)</sd> <def>The scaup duck.</def> <sd>(b)</sd> <def>The pintail duck.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Seawife</ent><br/
<hw>Sea"wife`</hw> <pr>(?)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Seawives</plw> <pr>(<?/)</pr>.</plu> <fld>(Zool.)</fld> <def>A European wrasse (<spn>Labrus vetula</spn>).</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sea willow</ent><br/
<hw>Sea" wil"low</hw> <pr>(?)</pr>. <fld>(Zool.)</fld> <def>A gorgonian coral with long flexible branches.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sea wing</ent><br/
<hw>Sea" wing`</hw> <pr>(?)</pr>. <fld>(Zool.)</fld> <def>A wing shell (<gen>Avicula</gen>).</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sea withwind</ent><br/
<hw>Sea" with"wind`</hw> <pr>(?)</pr>. <fld>(Bot.)</fld> <def>A kind of bindweed (<spn>Convolvulus Soldanella</spn>) growing on the seacoast of Europe.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sea wolf</ent><br/
<hw>Sea" wolf`</hw> <pr>(?)</pr>. <fld>(Zool.)</fld> <sd>(a)</sd> <def>The wolf fish.</def> <sd>(b)</sd> <def>The European sea perch.</def> <sd>(c)</sd> <def>The sea elephant.</def> <sd>(d)</sd> <def>A sea lion.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sea woodcock</ent><br/
<hw>Sea" wood"cock`</hw> <pr>(?)</pr>. <fld>(Zool.)</fld> <def>The bar-tailed godwit.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sea wood louse</ent><br/
<hw>Sea" wood" louse`</hw> <pr>(?)</pr>. <fld>(Zool.)</fld> <def>A sea slater.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sea wormwood</ent><br/
<hw>Sea" worm"wood`</hw> <pr>(?)</pr>. <fld>(Bot.)</fld> <def>A European species of wormwood (<spn>Artemisia maritima</spn>) growing by the sea.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Seaworthiness</ent><br/
<hw>Sea"wor`thi*ness</hw> <pr>(?)</pr>, <pos>n.</pos> <def>The state or quality of being seaworthy, or able to resist the ordinary violence of wind and weather.</def>  <rj><au>Kent.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Seaworthy</ent><br/
<hw>Sea"wor`thy</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Fit for a voyage; worthy of being trusted to transport a cargo with safety; <as>as, a <ex>seaworthy</ex> ship</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sea wrack</ent><br/
<hw>Sea" wrack`</hw> <pr>(?)</pr>. <fld>(Bot.)</fld> <def>See <er>Wrack</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sebaceous</ent><br/
<hw>Se*ba"ceous</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[NL. <ets>sebaceus</ets>, from L. <ets>sebum</ets> tallow, grease.]</ety> <fld>(Physiol.)</fld> <def>Pertaining to, or secreting, fat; composed of fat; having the appearance of fat; <as>as, the <ex>sebaceous</ex> secretions of some plants, or the <ex>sebaceous</ex> humor of animals</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Sebaceous cyst</b></col> <fld>(Med.)</fld>, <cd>a cyst formed by distention of a sebaceous gland, due to obstruction of its excretory duct.</cd> -- <col><b>Sebaceous glands</b></col> <fld>(Anat.)</fld>, <cd>small subcutaneous glands, usually connected with hair follicles. They secrete an oily semifluid matter, composed in great part of fat, which softens and lubricates the hair and skin.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sebacic</ent><br/
<hw>Se*bac"ic</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>sebum</ets> tallow: cf. F. <ets>s<eacute/bacique</ets>.]</ety> <fld>(Chem.)</fld> <def>Of or pertaining to fat; derived from, or resembling, fat; specifically, designating an acid (formerly called also <altname>sebic</altname>, and <altname>pyroleic</altname>, acid), obtained by the distillation or saponification of certain oils (as castor oil) as a white crystalline substance.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sebat</ent><br/
<hw>Se"bat</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Heb. <ets>sh<ecr/b<amac/t</ets>.]</ety> <def>The eleventh month of the ancient Hebrew year, approximately corresponding with February.</def>  <rj><au>W. Smith (Bibl. Dict.).</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sebate</ent><br/
<hw>Se"bate</hw> <pr>(s<emac/"b<amac/t)</pr>, <pos>n.</pos> <fld>(Chem.)</fld> <def>A salt of sebacic acid.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sebesten</ent><br/
<hw>Se*bes"ten</hw> <pr>(s<esl/*b<ecr/s"t<ecr/n)</pr>, <pos>n.</pos> <ety>[Ar. <ets>sebest<amac/n</ets> the tree: cf. Sp. <ets>sebesten</ets>.]</ety> <fld>(Bot.)</fld> <def>The mucilaginous drupaceous fruit of two East Indian trees (<spn>Cordia Myxa</spn>, and <spn>Cordia latifolia</spn>), sometimes used medicinally in pectoral diseases.</def><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ In the West Indies the name is given to the similar fruit of <spn>Cordia Sebestana</spn>.</note><br/
[<source>1913 Webster</source>]</p>

<p><-- p. 1300 pr=PCP --></p>

<p><ent>Sebic</ent><br/
<hw>Se"bic</hw> <pr>(?)</pr>, <pos>a.</pos> <def>See <er>Sebacic</er>.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sebiferous</ent><br/
<hw>Se*bif"er*ous</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>sebum</ets> tallow + <ets>-ferous</ets>.]</ety> <sn>1.</sn> <fld>(Bot.)</fld> <def>Producing vegetable tallow.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Physiol.)</fld> <def>Producing fat; sebaceous; <as>as, the <ex>sebiferous</ex>, or sebaceous, glands</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sebiparous</ent><br/
<hw>Se*bip"a*rous</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>sebum</ets> tallow + <ets>parere</ets> to bring forth.]</ety> <fld>(Physiol.)</fld> <def>Same as <er>Sebiferous</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Seborrhea</ent><br/
||<hw>Seb"or*rhe*a</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[NL., fr. L. <ets>sebum</ets> tallow + Gr. <?/ to flow.]</ety> <fld>(Med.)</fld> <def>A morbidly increased discharge of sebaceous matter upon the skin; stearrhea.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Secale</ent><br/
||<hw>Se*ca"le</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L., a kind of grain.]</ety> <fld>(Bot.)</fld> <def>A genus of cereal grasses including rye.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Secancy</ent><br/
<hw>Se"can*cy</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[See <er>Secant</er>.]</ety> <def>A cutting; an intersection; <as>as, the point of <ex>secancy</ex> of one line by another</as>.</def> <mark>[R.]</mark>  <rj><au>Davies & Peck (Math. Dict.).</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Secant</ent><br/
<hw>Se"cant</hw> <pr>(s<emac/"k<acr/nt)</pr>, <pos>a.</pos> <ety>[L. <ets>secans</ets>, <ets>-antis</ets>, p. pr. of <ets>secare</ets> to cut. See <er>Section</er>.]</ety> <def>Cutting; dividing into two parts; <as>as, a <ex>secant</ex> line</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Secant</ent><br/
<hw>Secant</hw>, <pos>n.</pos> <ety>[Cf. F. <ets>s<eacute/cante</ets>. See <er>Secant</er>, <pos>a.</pos>]</ety> <sn>1.</sn> <fld>(Geom.)</fld> <def>A line that cuts another; especially, a straight line cutting a curve in two or more points.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Trig.)</fld> <def>A right line drawn from the center of a circle through one end of a circular arc, and terminated by a tangent drawn from the other end; the number expressing the ratio of this line to the radius of the circle. See <cref>Trigonometrical function</cref>, under <er>Function</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Secco</ent><br/
||<hw>Sec"co</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[It.]</ety> <def>Dry.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><mcol><col><b>Secco painting</b></col>, <it>or</it> <col><b>Painting in secco</b></col></mcol>, <cd>painting on dry plaster, as distinguished from <contr>fresco painting</contr>, which is on wet or fresh plaster.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Secede</ent><br/
<hw>Se"cede"</hw> <pr>(?)</pr>, <pos>v. i.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Seceded</conjf>; <pos>p. pr. & vb. n.</pos> <conjf>Seceding</conjf>.]</vmorph> <ety>[L. <ets>secedere</ets>, <ets>secessum</ets>; pref <ets>se-</ets> aside + <ets>cedere</ets> to go, move. See <er>Cede</er>.]</ety> <def>To withdraw from fellowship, communion, or association; to separate one's self by a solemn act; to draw off; to retire; especially, to withdraw from a political or religious body.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Seceder</ent><br/
<hw>Se*ced"er</hw> <pr>(?)</pr>, <pos>n.</pos> <sn>1.</sn> <def>One who secedes.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Eccl. Hist.)</fld> <def>One of a numerous body of Presbyterians in Scotland who seceded from the communion of the Established Church, about the year 1733, and formed the Secession Church, so called.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Secern</ent><br/
<hw>Se*cern"</hw> <pr>(?)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Secerned</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Secerning</conjf>.]</vmorph> <ety>[L. <ets>secernere</ets>. See <er>Secrete</er>.]</ety> <sn>1.</sn> <def>To separate; to distinguish.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Averroes <qex>secerns</qex> a sense of titillation, and a sense of hunger and thirst.</q> <rj><qau>Sir W. Hamilton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Physiol.)</fld> <def>To secrete; <as>as, mucus <ex>secerned</ex> in the nose</as>.</def>  <rj><au>Arbuthnot.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Secernent</ent><br/
<hw>Se*cern"ent</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>secernens</ets>, p. pr.]</ety> <fld>(Physiol.)</fld> <def>Secreting; secretory.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Secernent</ent><br/
<hw>Se*cern"ent</hw>, <pos>n.</pos> <sn>1.</sn> <def>That which promotes secretion.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Anat.)</fld> <def>A vessel in, or by means of, which the process of secretion takes place; a secreting vessel.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Secernment</ent><br/
<hw>Se*cern"ment</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Physiol.)</fld> <def>The act or process of secreting.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Secess</ent><br/
<hw>Se*cess"</hw> <pr>(s<esl/*s<ecr/s")</pr>, <pos>n.</pos> <ety>[L. <ets>secessus</ets>. See <er>Secede</er>.]</ety> <def>Retirement; retreat; secession.</def> <mark>[Obs.]</mark>  <rj><au>Dr. H. More.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Secession</ent><br/
<hw>Se*ces"sion</hw> <pr>(s<esl/*s<ecr/sh"<ucr/n)</pr>, <pos>n.</pos> <ety>[L. <ets>secessio</ets>: cf. F. <ets>s<eacute/cession</ets>. See <er>Secede</er>.]</ety> <sn>1.</sn> <def>The act of seceding; separation from fellowship or association with others, as in a religious or political organization; withdrawal.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(U.S. Hist.)</fld> <def>The withdrawal of a State from the national Union.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Secession Church</b></col><cd> (in Scotland). See <er>Seceder</er>.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Secessionism</ent><br/
<hw>Se*ces"sion*ism</hw> <pr>(?)</pr>, <pos>n.</pos> <def>The doctrine or policy of secession; the tenets of secession; the tenets of secessionists.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Secessionist</ent><br/
<hw>Se*ces"sion*ist</hw>, <pos>n.</pos> <sn>1.</sn> <def>One who upholds secession.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(U.S. Hist.)</fld> <def>One who holds to the belief that a State has the right to separate from the Union at its will.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Seche</ent><br/
<hw>Seche</hw> <pr>(?)</pr>, <pos>v. t. & i.</pos> <def>To seek.</def> <mark>[Obs.]</mark>  <rj><au>Chaucer.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sechium</ent><br/
<hw>Se"chi*um</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[NL.: cf. F. <ets>s<eacute/chion</ets>; perhaps formed fr. Gr. <?/ cucumber.]</ety> <fld>(Bot.)</fld> <def>The edible fruit of a West Indian plant (<spn>Sechium edule</spn>) of the Gourd family. It is soft, pear-shaped, and about four inches long, and contains a single large seed. The root of the plant resembles a yam, and is used for food.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Seck</ent><br/
<hw>Seck</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[F. <ets>sec</ets>, properly, dry, L. <ets>siccus</ets>.]</ety> <def>Barren; unprofitable. See <cref>Rent seck</cref>, under <er>Rent</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Seckel</ent><br/
<hw>Seck"el</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Bot.)</fld> <def>A small reddish brown sweet and juicy pear. It originated on a farm near Philadelphia, afterwards owned by a Mr. <etsep>Seckel</etsep>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Secle</ent><br/
<hw>Se"cle</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>saeculum</ets>: cf. F. <ets>si<egrave/cle</ets>. See <er>Secular</er>.]</ety> <def>A century.</def> <mark>[Obs.]</mark>  <rj><au>Hammond.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Seclude</ent><br/
<hw>Se*clude</hw> <pr>(?)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Secluded</conjf>; <pos>p. pr. & vb. n.</pos> <conjf>Secluding</conjf>.]</vmorph> <ety>[L. <ets>secludere</ets>, <ets>seclusum</ets>; pref. <ets>se-</ets> aside + <ets>claudere</ets> to shut. See <er>Close</er>, <pos>v. t.</pos>]</ety> <sn>1.</sn> <def>To shut up apart from others; to withdraw into, or place in, solitude; to separate from society or intercourse with others.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Let Eastern tyrants from the light of heaven<br/
<qex>Seclude</qex> their bosom slaves.</q> <rj><qau>Thomson.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To shut or keep out; to exclude.</def> <mark>[Obs.]</mark>  <rj><au>Evelyn.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p>-- <wordforms><wf>Se*clud"ed*ly</wf>, <pos>adv.</pos> -- <wf>Se*clud"ed*ness</wf>, <pos>n.</pos></wordforms><br/
[<source>1913 Webster</source>]</p>

<p><ent>Seclusion</ent><br/
<hw>Se*clu"sion</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[See <er>Seclude</er>.]</ety> <def>The act of secluding, or the state of being secluded; separation from society or connection; a withdrawing; privacy; <as>as, to live in <ex>seclusion</ex></as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>O blest <qex>seclusion</qex> from a jarring world, which he, thus occupied, enjoys!</q> <rj><qau>Cowper.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><syn><b>Syn.</b> -- Solitude; separation; withdrawment; retirement; privacy. See <er>Solitude</er>.</syn><br/
[<source>1913 Webster</source>]</p>

<p><ent>Seclusive</ent><br/
<hw>Se*clu"sive</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Tending to seclude; keeping in seclusion; secluding; sequestering.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Second</ent><br/
<hw>Sec"ond</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[F., fr. L. <ets>secundus</ets> second, properly, following, fr. <ets>sequi</ets> to follow. See <er>Sue</er> to follow, and cf. <er>Secund</er>.]</ety> <sn>1.</sn> <def>Immediately following the first; next to the first in order of place or time; hence, occurring again; another; other.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>And he slept and dreamed the <qex>second</qex> time.</q> <rj><qau>Gen. xli. 5.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Next to the first in value, power, excellence, dignity, or rank; secondary; subordinate; inferior.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>May the day when we become the <qex>second</qex> people upon earth . . . be the day of our utter extirpation.</q> <rj><qau>Landor.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>Being of the same kind as another that has preceded; another, like a prototype; <as>as, a <ex>second</ex> Cato; a <ex>second</ex> Troy; a <ex>second</ex> deluge</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>A Daniel, still say I, a <qex>second</qex> Daniel!</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Second Adventist</b></col>. <cd>See <er>Adventist</er>.</cd> -- <col><b>Second cousin</b></col>, <cd>the child of a cousin.</cd> -- <col><b>Second-cut file</b></col>. <cd>See under <er>File</er>.</cd> -- <col><b>Second distance</b></col> <fld>(Art)</fld>, <cd>that part of a picture between the foreground and the background; -- called also <altname>middle ground</altname>, or <altname>middle distance</altname>.</cd> <mark>[R.]</mark> -- <col><b>Second estate</b></col> <fld>(Eng.)</fld>, <cd>the House of Peers.</cd> -- <col><b>Second girl</b></col>, <cd>a female house-servant who does the lighter work, as chamber work or waiting on table.</cd> -- <col><b>Second intention</b></col>. <cd>See under <er>Intention</er>.</cd> -- <mcol><col><b>Second story</b></col>, <col><b>Second floor</b></col></mcol>, <cd>in America, the second range of rooms from the street level. This, in England, is called the <altname>first floor</altname>, the one beneath being the <contr>ground floor</contr>.</cd> -- <mcol><col><b>Second thought</b></col> <it>or</it> <col><b>Second thoughts</b></col></mcol>, <cd>consideration of a matter following a first impulse or impression; reconsideration.</cd><br/
[<source>1913 Webster</source>]</p>

<p><q>On <qex>second thoughts</qex>, gentlemen, I don't wish you had known him.</q> <rj><qau>Dickens.</qau></rj><br/
</cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Second</ent><br/
<hw>Sec"ond</hw> <pr>(?)</pr>, <pos>n.</pos> <sn>1.</sn> <def>One who, or that which, follows, or comes after; one next and inferior in place, time, rank, importance, excellence, or power.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Man<br/
An angel's <qex>second</qex>, nor his <qex>second</qex> long.</q> <rj><qau>Young.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>One who follows or attends another for his support and aid; a backer; an assistant; specifically, one who acts as another's aid in a duel.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Being sure enough of <qex>seconds</qex> after the first onset.</q> <rj><qau>Sir H. Wotton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>Aid; assistance; help.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>Give <qex>second</qex>, and my love<br/
Is everlasting thine.</q> <rj><qau>J. Fletcher.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <pluf>pl.</pluf> <def>An article of merchandise of a grade inferior to the best; esp., a coarse or inferior kind of flour.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <ety>[F. <ets>seconde</ets>. See <er>Second</er>, <pos>a.</pos>]</ety> <def>The sixtieth part of a minute of time or of a minute of space, that is, the <xex>second</xex> regular subdivision of the degree; <as>as, sound moves about 1,140 English feet in a <ex>second</ex>; five minutes and ten <ex>seconds</ex> north of this place.</as></def><br/
[<source>1913 Webster</source>]</p>

<p><sn>6.</sn> <def>In the duodecimal system of mensuration, the twelfth part of an inch or prime; a line. See <er>Inch</er>, and <er>Prime</er>, <pos>n.</pos>, 8.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>7.</sn> <fld>(Mus.)</fld> <sd>(a)</sd> <def>The interval between any tone and the tone which is represented on the degree of the staff next above it.</def> <sd>(b)</sd> <def>The second part in a concerted piece; -- often popularly applied to the alto.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>8.</sn> <fld>(Parliamentary Procedure)</fld> <def>A motion in support of another motion which has been moved in a deliberative body; <as> a motion without a <ex>second</ex> dies without discussion</as>.</def><br/
[<source>PJC</source>]</p>

<p><cs><col><b>Second hand</b></col>, <cd>the hand which marks the seconds on the dial of a watch or a clock.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Second</ent><br/
<hw>Sec"ond</hw>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Seconded</conjf>; <pos>p. pr. & vb. n.</pos> <conjf>Seconding</conjf>.]</vmorph> <ety>[Cf. F. <ets>seconder</ets>, L. <ets>secundare</ets>, from <ets>secundus</ets>. See <er>Second</er>, <pos>a.</pos>]</ety> <sn>1.</sn> <def>To follow in the next place; to succeed; to alternate.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>In the method of nature, a low valley is immediately <qex>seconded</qex> with an ambitious hill.</q> <rj><qau>Fuller.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Sin is <qex>seconded</qex> with sin.</q> <rj><qau>South.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To follow or attend for the purpose of assisting; to support; to back; to act as the second of; to assist; to forward; to encourage.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>We have supplies to <qex>second</qex> our attempt.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>In human works though labored on with pain,<br/
A thousand movements scarce one purpose gain;<br/
In God's, one single can its end produce,<br/
Yet serves to <qex>second</qex> too some other use.</q> <rj><qau>Pope.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <specif>Specifically,</specif> <fld>(Parliamentary Procedure)</fld> <def>to support, as a motion{6} or proposal, by adding one's voice to that of the mover or proposer.</def><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/  Under common parliamentary rules used by many organizations, especially legislative bodies, a motion must be <ex>seconded</ex> in order to come properly before the deliberative body for discussion.  Any motion{6} for which there is no <ex>second{8}</ex> dies for lack thereof.</note><br/
[<source>PJC</source>]</p>

<p><ent>Secondarily</ent><br/
<hw>Sec"ond*a*ri*ly</hw> <pr>(?)</pr>, <pos>adv.</pos> <sn>1.</sn> <def>In a secondary manner or degree.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Secondly; in the second place.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>God hath set some in the church, first apostles, <qex>secondarily</qex> prophets, thirdly teachers.</q> <rj><qau>1 Cor. xii. 28.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Secondariness</ent><br/
<hw>Sec"ond*a*ri*ness</hw>, <pos>n.</pos> <def>The state of being secondary.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Full of a girl's sweet sense of <qex>secondariness</qex> to the object of her love.</q> <rj><qau>Mrs. Oliphant.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Secondary</ent><br/
<hw>Sec"ond*a*ry</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[Cf. F. <ets>secondaire</ets>, L. <ets>secundaire</ets>. See <er>Second</er>, <pos>a.</pos>]</ety> <sn>1.</sn> <def>Succeeding next in order to the first; of second place, origin, rank, etc.; not primary; subordinate; not of the first order or rate.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Wheresoever there is moral right on the one hand, no <qex>secondary</qex> right can discharge it.</q> <rj><qau>L'Estrange.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Two are the radical differences; the <qex>secondary</qex> differences are as four.</q> <rj><qau>Bacon.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Acting by deputation or delegated authority; <as>as, the work of <ex>secondary</ex> hands</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <fld>(Chem.)</fld> <def>Possessing some quality, or having been subject to some operation (as substitution), in the second degree; <as>as, a <ex>secondary</ex> salt, a <ex>secondary</ex> amine, etc.</as> Cf. <er>primary</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><note>  A primary amine has the general formula <chform>R.NH2</chform>; a <ex>secondary</ex> amine has the general formula <chform>R.NH.R<prime/</chform>, where R and R<prime/ are alkyl or aryl groups.  A primary alcohol has the general formula <chform>R.CH2.OH</chform>; a <ex>secondary</ex> alcohol has the general formula <chform>R.CHOH.R<prime/</chform>.  Tertiary amines and alcohols have the general formulas <chform>R.CR<prime/N.R<Prime/</chform> and <chform>R.CR<prime/OH.R<Prime/</chform>, respectively.</note><br/
[<source>PJC</source>]</p>

<p><sn>4.</sn> <fld>(Min.)</fld> <def>Subsequent in origin; -- said of minerals produced by alteration or deposition subsequent to the formation of the original rock mass; also of characters of minerals (as <ex>secondary</ex> cleavage, etc.) developed by pressure or other causes.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <fld>(Zool.)</fld> <def>Pertaining to the second joint of the wing of a bird.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>6.</sn> <fld>(Med.)</fld> <sd>(a)</sd> <def>Dependent or consequent upon another disease; <as>as, Bright's disease is often <ex>secondary</ex> to scarlet fever</as>.</def> <sd>(b)</sd> <def>Occurring in the second stage of a disease; <as>as, the <ex>secondary</ex> symptoms of syphilis</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Secondary accent</b></col>. <cd>See the Note under <er>Accent</er>, <pos>n.</pos>, 1.</cd> -- <col><b>Secondary age</b></col>. <fld>(Geol.)</fld> <cd>The Mesozoic age, or age before the Tertiary. See <er>Mesozoic</er>, and Note under <er>Age</er>, <pos>n.</pos>, 8.</cd> -- <col><b>Secondary alcohol</b></col> <fld>(Chem.)</fld>, <cd>any one of a series of alcohols which contain the radical <chform>CH.OH</chform> united with two hydrocarbon radicals. On oxidation the secondary alcohols form ketones.</cd> -- <col><b>Secondary amputation</b></col> <fld>(Surg.)</fld>, <cd>an amputation for injury, performed after the constitutional effects of the injury have subsided.</cd> -- <col><b>Secondary axis</b></col> <fld>(Opt.)</fld>, <cd>any line which passes through the optical center of a lens but not through the centers of curvature, or, in the case of a mirror, which passes through the center of curvature but not through the center of the mirror.</cd> -- <col><b>Secondary battery</b></col>. <fld>(Elec.)</fld> <cd>See under <er>Battery</er>, <pos>n.</pos>, 4.</cd> -- <col><b>Secondary circle</b></col> <fld>(Geom. & Astron.)</fld>, <cd>a great circle that passes through the poles of another great circle and is therefore perpendicular to its plane.</cd> -- <mcol><col><b>Secondary circuit</b></col>, <col><b>Secondary coil</b></col></mcol> <fld>(Elec.)</fld>, <cd>a circuit or coil in which a current is produced by the induction of a current in a neighboring circuit or coil called the <xex>primary circuit</xex> or <xex>coil</xex>.</cd> -- <col><b>Secondary color</b></col>, <cd>a color formed by mixing any two primary colors in equal proportions.</cd> -- <col><b>Secondary coverts</b></col> <fld>(Zool.)</fld>, <cd>the longer coverts which overlie the basal part of the secondary quills of a bird. See <xex>Illust.</xex> under <er>Bird</er>.</cd> -- <col><b>Secondary crystal</b></col> <fld>(Min.)</fld>, <cd>a crystal derived from one of the primary forms.</cd> -- <col><b>Secondary current</b></col> <fld>(Elec.)</fld>, <cd>a momentary current induced in a closed circuit by a current of electricity passing through the same or a contiguous circuit at the beginning and also at the end of the passage of the primary current.</cd> -- <col><b>Secondary evidence</b></col>, <cd>that which is admitted upon failure to obtain the primary or best evidence.</cd> -- <col><b>Secondary fever</b></col> <fld>(Med.)</fld>, <cd>a fever coming on in a disease after the subsidence of the fever with which the disease began, as the fever which attends the outbreak of the eruption in smallpox.</cd> -- <col><b>Secondary hemorrhage</b></col> <fld>(Med.)</fld>, <cd>hemorrhage occuring from a wounded blood vessel at some considerable time after the original bleeding has ceased.</cd> -- <col><b>Secondary planet</b></col>. <fld>(Astron.)</fld> <cd>See the Note under <er>Planet</er>.</cd> -- <col><b>Secondary qualities</b></col>, <cd>those qualities of bodies which are not inseparable from them as such, but are dependent for their development and intensity on the organism of the percipient, such as color, taste, odor, etc.</cd> -- <mcol><col><b>Secondary quills</b></col> <it>or</it> <col><b>Secondary remiges</b></col></mcol> <fld>(Zool.)</fld>, <cd>the quill feathers arising from the forearm of a bird and forming a row continuous with the primaries; -- called also <altname>secondaries</altname>. See <xex>Illust.</xex> of <er>Bird</er>.</cd> -- <mcol><col><b>Secondary rocks</b></col> <it>or</it> <col><b>Secondary strata</b></col></mcol> <fld>(Geol.)</fld>, <cd>those lying between the Primary, or Paleozoic, and Tertiary (see <cref>Primary rocks</cref>, under <er>Primary</er>); -- later restricted to strata of the Mesozoic age, and at present but little used.</cd> -- <col><b>Secondary syphilis</b></col> <fld>(Med.)</fld>, <cd>the second stage of syphilis, including the period from the first development of constitutional symptoms to the time when the bones and the internal organs become involved.</cd> -- <col><b>Secondary tint</b></col>, <cd>any subdued tint, as gray.</cd> -- <col><b>Secondary union</b></col> <fld>(Surg.)</fld>, <cd>the union of wounds after suppuration; union by the second intention.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><syn><b>Syn.</b> -- Second; second-rate; subordinate; inferior.</syn><br/
[<source>1913 Webster</source>]</p>

<p><ent>Secondary</ent><br/
<hw>Sec"ond*a*ry</hw> <pr>(?)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Secondaries</plw> <pr>(<?/)</pr>.</plu> <sn>1.</sn> <def>One who occupies a subordinate, inferior, or auxiliary place; a delegate or deputy; one who is second or next to the chief officer; <as>as, the <ex>secondary</ex>, or undersheriff of the city of London</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Old Escalus . . . is thy <qex>secondary</qex>.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Astron.)</fld> <sd>(a)</sd> <def>A secondary circle.</def> <sd>(b)</sd> <def>A satellite.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <fld>(Zool.)</fld> <def>A secondary quill.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Second-class</ent><br/
<hw>Sec"ond-class`</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Of the rank or degree below the best or highest; inferior; second-rate; <as>as, a <ex>second-class</ex> house; a <ex>second-class</ex> passage; a <ex>second-class</ex> citizen</as>.</def><br/
[<source>1913 Webster</source> <source>+PJC</source>]</p>

<p><ent>Seconder</ent><br/
<hw>Sec"ond*er</hw> <pr>(?)</pr>, <pos>n.</pos> <def>One who seconds{3} or supports what another attempts, affirms, moves, or proposes; <as>as, the <ex>seconder</ex> of an enterprise or of a motion</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Secondhand</ent><br/
<hw>Sec"ond*hand`</hw> <pr>(?)</pr>, <pos>a.</pos> <sn>1.</sn> <def>Not original or primary; received from another; <as>as, <ex>secondhand</ex> information</as>.</def><br/
[<source>1913 Webster</source> <source>+PJC</source>]</p>

<p><q>They have but a <qex>secondhand</qex> or implicit knowledge.</q> <rj><qau>Locke.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Not new; already or previously possessed or used by another; <as>as, a <ex>secondhand</ex> book, garment</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>At second hand</b></col>. <cd>See <er>Hand</er>, <pos>n.</pos>, 10.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Secondly</ent><br/
<hw>Sec"ond*ly</hw>, <pos>adv.</pos> <def>In the second place.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Secondo</ent><br/
||<hw>Se*con"do</hw> <pr>(s<esl/*k<ocr/n"d<osl/; It. s<asl/*k<osl/n"d<osl/)</pr>, <pos>n.</pos> <ety>[It.]</ety> <fld>(Mus.)</fld> <def>The second part in a concerted piece.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Second-rate</ent><br/
<hw>Sec"ond-rate`</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Of the second size, rank, quality, or value; <as>as, a <ex>second-rate</ex> ship; <ex>second-rate</ex> cloth; a <ex>second-rate</ex> champion</as>.</def>  <rj><au>Dryden.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Second-sight</ent><br/
<hw>Sec"ond-sight`</hw> <pr>(?)</pr>, <pos>n.</pos> <def>The power of discerning what is not visible to the physical eye, or of foreseeing future events, esp. such as are of a disastrous kind; the capacity of a seer; prophetic vision.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>He was seized with a fit of <qex>second-sight</qex>.</q> <rj><qau>Addison.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Nor less availed his optic sleight,<br/
And Scottish gift of <qex>second-sight</qex>.</q> <rj><qau>Trumbull.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Second-sighted</ent><br/
<hw>Sec"ond-sight`ed</hw>, <pos>a.</pos> <def>Having the power of second-sight.</def> <mark>[R.]</mark> <rj><au>Addison.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Secre</ent><br/
<hw>Se"cre</hw> <pr>(? <or/ ?)</pr>, <pos>a.</pos> <def>Secret; secretive; faithful to a secret.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>To be holden stable and <qex>secre</qex>.</q> <rj><qau>Chaucer.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Secre</ent><br/
<hw>Se"cre</hw>, <pos>n.</pos> <def>A secret.</def> <mark>[Obs.]</mark>  <rj><au>Chaucer.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Secrecy</ent><br/
<hw>Se"cre*cy</hw> <pr>(?)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Secrecies</plw> <pr>(#)</pr>.</plu> <ety>[From <er>Secret</er>.]</ety> <sn>1.</sn> <def>The state or quality of being hidden; <as>as, his movements were detected in spite of their <ex>secrecy</ex></as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The Lady Anne,<br/
Whom the king hath in <qex>secrecy</qex> long married.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>That which is concealed; a secret.</def> <mark>[R.]</mark>  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>Seclusion; privacy; retirement.</def> <ldquo/The pensive <xex>secrecy</xex> of desert cell.<rdquo/  <rj><au>Milton.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>The quality of being secretive; fidelity to a secret; forbearance of disclosure or discovery.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>It is not with public as with private prayer; in this, rather <qex>secrecy</qex> is commanded than outward show.</q> <rj><qau>Hooker.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Secrely</ent><br/
<hw>Se"cre*ly</hw> <pr>(?)</pr>, <pos>adv.</pos> <def>Secretly.</def> <mark>[Obs.]</mark>  <rj><au>Chaucer.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Secreness</ent><br/
<hw>Se"cre*ness</hw>, <pos>n.</pos> <def>Secrecy; privacy.</def> <mark>[Obs.]</mark>  <rj><au>Chaucer.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Secret</ent><br/
<hw>Se"cret</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[F. <ets>secret</ets> (cf. Sp. & Pg. <ets>secreto</ets>, It. <ets>secreto</ets>, <ets>segreto</ets>), fr. L. <ets>secretus</ets>, p. p. of <ets>secernere</ets> to put apart, to separate. See <er>Certain</er>, and cf. <er>Secrete</er>, <er>Secern</er>.]</ety> <sn>1.</sn> <def>Hidden; concealed; <as>as, <ex>secret</ex> treasure; <ex>secret</ex> plans; a <ex>secret</ex> vow</as>.</def>  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>The <qex>secret</qex> things belong unto the Lord our God; but those things which are revealed belong unto us.</q> <rj><qau>Deut. xxix. 29.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Withdrawn from general intercourse or notice; in retirement or secrecy; secluded.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>There, <qex>secret</qex> in her sapphire cell,<br/
He with the Nais wont to dwell.</q> <rj><qau>Fenton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>Faithful to a secret; not inclined to divulge or betray confidence; secretive.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q><qex>Secret</qex> Romans, that have spoke the word,<br/
And will not palter.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>Separate; distinct.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>They suppose two other divine hypostases superior thereunto, which were perfectly <qex>secret</qex> from matter.</q> <rj><qau>Cudworth.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><syn><b>Syn.</b> -- Hidden; concealed; secluded; retired; unseen; unknown; private; obscure; recondite; latent; covert; clandestine; privy. See <er>Hidden</er>.</syn><br/
[<source>1913 Webster</source>]</p>

<p><ent>Secret</ent><br/
<hw>Se"cret</hw>, <pos>n.</pos> <ety>[F. <ets>secret</ets> (cf. Pr. <ets>secret</ets>, Sp. & Pg. <ets>secreto</ets>, It. <ets>secreto</ets>, <ets>segreto</ets>), from L. <ets>secretum</ets>. See <er>Secret</er>, <pos>a.</pos>]</ety> <sn>1.</sn> <def>Something studiously concealed; a thing kept from general knowledge; what is not revealed, or not to be revealed.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>To tell our own <qex>secrets</qex> is often folly; to communicate those of others is treachery.</q> <rj><qau>Rambler.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><-- p. 1301 pr=PCP --></p>

<p><sn>2.</sn> <def>A thing not discovered; what is unknown or unexplained; a mystery.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>All <qex>secrets</qex> of the deep, all nature's works.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <pluf>pl.</pluf> <def>The parts which modesty and propriety require to be concealed; the genital organs.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>In secret</b></col>, <cd>in a private place; in privacy or secrecy; in a state or place not seen; privately.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><q>Bread eaten <qex>in secret</qex> is pleasant.</q> <rj><qau>Prov. ix. 17.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Secret</ent><br/
<hw>Se"cret</hw> <pr>(?)</pr>, <pos>v. t.</pos> <def>To keep secret.</def> <mark>[Obs.]</mark>  <rj><au>Bacon.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Secretage</ent><br/
<hw>Se"cret*age</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F.]</ety> <def>A process in which mercury, or some of its salts, is employed to impart the property of felting to certain kinds of furs.</def>  <rj><au>Ure.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Secretarial</ent><br/
<hw>Sec`re*ta"ri*al</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Of or pertaining to a secretary; befitting a secretary.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q><qex>Secretarial</qex>, diplomatic, or other official training.</q> <rj><qau>Carlyle.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Secretariate</ent><br/
<ent>Secretariat</ent><br/
<mhw>{ <hw>Sec`re*ta"ri*at</hw> <pr>(?)</pr>, <hw>Sec`re*ta"ri*ate</hw> <pr>(?)</pr>, }</mhw> <pos>n.</pos> <ety>[F. <ets>secr<eacute/tariat</ets>.]</ety> <def>The office of a secretary; the place where a secretary transacts business, keeps records, etc.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Secretary</ent><br/
<hw>Sec"re*ta*ry</hw> <pr>(?)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Secretaries</plw> <pr>(#)</pr>.</plu> <ety>[F. <ets>secr<eacute/taire</ets> (cf. Pr. <ets>secretari</ets>, Sp. & Pg. <ets>secretario</ets>, It. <ets>secretario</ets>, <ets>segretario</ets>) LL. <ets>secretarius</ets>, originally, a confidant, one intrusted with secrets, from L. <ets>secretum</ets> a secret. See <er>Secret</er>, <pos>a.</pos> & <pos>n.</pos>]</ety> <sn>1.</sn> <def>One who keeps, or is intrusted with, secrets.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A person employed to write orders, letters, dispatches, public or private papers, records, and the like; an official scribe, amanuensis, or writer; one who attends to correspondence, and transacts other business, for an association, a public body, or an individual.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>That which is most of all profitable is acquaintance with the <qex>secretaries</qex>, and employed men of ambassadors.</q> <rj><qau>Bacon.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>An officer of state whose business is to superintend and manage the affairs of a particular department of government, and who is usually a member of the cabinet or advisory council of the chief executive; <as>as, the <ex>secretary</ex> of state, who conducts the correspondence and attends to the relations of a government with foreign courts; the <ex>secretary</ex> of the treasury, who manages the department of finance; the <ex>secretary</ex> of war, etc.</as></def><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>A piece of furniture, with conveniences for writing and for the arrangement of papers; an escritoire.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <fld>(Zool.)</fld> <def>The secretary bird.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Secretary bird</b></col>. <ety>[So called in allusion to the tufts of feathers at the back of its head, which were fancifully thought to resemble pens stuck behind the ear.]</ety> <fld>(Zool.)</fld> <cd>A large long-legged raptorial bird (<spn>Gypogeranus serpentarius</spn>), native of South Africa, but now naturalized in the West Indies and some other tropical countries. It has a powerful hooked beak, a crest of long feathers, and a long tail. It feeds upon reptiles of various kinds, and is much prized on account of its habit of killing and devouring snakes of all kinds. Called also <altname>serpent eater</altname>.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><syn><b>Syn.</b> -- See the Note under <er>Clerk</er>, <pos>n.</pos>, 4.</syn><br/
[<source>1913 Webster</source>]</p>

<p><ent>Secretaryship</ent><br/
<hw>Sec"re*ta*ry*ship</hw>, <pos>n.</pos> <def>The office, or the term of office, of a secretary.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Secrete</ent><br/
<hw>Se*crete"</hw> <pr>(?)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Secreted</conjf>; <pos>p. pr. & vb. n.</pos> <conjf>Secreting</conjf>.]</vmorph> <ety>[L. <ets>secretus</ets> separated, secret, hidden, p. p. of <ets>secernere</ets>. See <er>Secret</er>, and cf. <er>Discrete</er>, <er>Discreet</er>.]</ety> <sn>1.</sn> <def>To deposit in a place of hiding; to hide; to conceal; <as>as, to <ex>secrete</ex> stolen goods; to <ex>secrete</ex> one's self</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Physiol.)</fld> <def>To separate from the blood and elaborate by the process of secretion; to elaborate and emit as a secretion. See <er>Secretion</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Why one set of cells should <qex>secrete</qex> bile, another urea, and so on, we do not know.</q> <rj><qau>Carpenter.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><syn><b>Syn.</b> -- To conceal; hide. See <er>Conceal</er>.</syn><br/
[<source>1913 Webster</source>]</p>

<p><ent>Secretion</ent><br/
<hw>Se*cre"tion</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>secretio</ets>: cf. F. <ets>s<eacute/cr<eacute/tion</ets>.]</ety> <sn>1.</sn> <def>The act of secreting or concealing; <as>as, the <ex>secretion</ex> of dutiable goods</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Physiol.)</fld> <def>The act of secreting; the process by which material is separated from the blood through the agency of the cells of the various glands and elaborated by the cells into new substances so as to form the various secretions, as the saliva, bile, and other digestive fluids. The process varies in the different glands, and hence are formed the various secretions.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <fld>(Physiol.)</fld> <def>Any substance or fluid secreted, or elaborated and emitted, as the gastric juice.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Secretist</ent><br/
<hw>Se"cret*ist</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A dealer in secrets.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Secretitious</ent><br/
<hw>Se`cre*ti"tious</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Parted by animal secretion; <as>as, <ex>secretitious</ex> humors</as>.</def>  <rj><au>Floyer.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Secretive</ent><br/
<hw>Se*cret"ive</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Tending to secrete, or to keep secret or private; <as>as, a <ex>secretive</ex> disposition</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Secretiveness</ent><br/
<hw>Se*cret"ive*ness</hw>, <pos>n.</pos> <sn>1.</sn> <def>The quality of being secretive; disposition or tendency to conceal.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Phren.)</fld> <def>The faculty or propensity which impels to reserve, secrecy, or concealment.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Secretly</ent><br/
<hw>Se"cret*ly</hw> <pr>(?)</pr>, <pos>adv.</pos> <def>In a secret manner.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Secretness</ent><br/
<hw>Se"cret*ness</hw>, <pos>n.</pos> <sn>1.</sn> <def>The state or quality of being secret, hid, or concealed.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Secretiveness; concealment.</def>  <rj><au>Donne.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Secreto-motory</ent><br/
<hw>Se*cre`to-mo"to*ry</hw> <pr>(?)</pr>, <pos>a.</pos> <fld>(Physiol.)</fld> <def>Causing secretion; -- said of nerves which go to glands and influence secretion.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Secretory</ent><br/
<hw>Se*cre"to*ry</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[Cf. F. <ets>s<eacute/cr<eacute/toire</ets>. See <er>Secrete</er>.]</ety> <fld>(Physiol.)</fld> <def>Secreting; performing, or connected with, the office of secretion; secernent; <as>as, <ex>secretory</ex> vessels, nerves</as>.</def> -- <def2><pos>n.</pos> <def>A secretory vessel; a secernent.</def></def2><br/
[<source>1913 Webster</source>]</p>

<p><ent>Secret service</ent><br/
<hw>Se"cret serv"ice</hw>. <def>The detective service of a government. In the United States, in time of peace the bureau of secret service is under the treasury department, and in time of war it aids the war department in securing information concerning the movements of the enemy.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Sect</ent><br/
<hw>Sect</hw> <pr>(s<ecr/kt)</pr>, <pos>n.</pos> <ety>[L. <ets>secare</ets>, <ets>sectum</ets>, to cut.]</ety> <def>A cutting; a scion.</def> <mark>[Obs.]</mark>  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sect</ent><br/
<hw>Sect</hw> <pr>(s<ecr/kt)</pr>, <pos>n.</pos> <ety>[F. <ets>secte</ets>, L. <ets>secta</ets>, fr. <ets>sequi</ets> to follow; often confused with L. <ets>secare</ets>, <ets>sectum</ets>, to cut. See <er>Sue</er> to follow, and cf. <er>Sept</er>, <er>Suit</er>, <pos>n.</pos>]</ety> <def>Those following a particular leader or authority, or attached to a certain opinion; a company or set having a common belief or allegiance distinct from others; in religion, the believers in a particular creed, or upholders of a particular practice; especially, in modern times, a party dissenting from an established church; a denomination; in philosophy, the disciples of a particular master; a school; in society and the state, an order, rank, class, or party.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>He beareth the sign of poverty,<br/
And in that <qex>sect</qex> our Savior saved all mankind.</q> <rj><qau>Piers Plowman.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>As of the <qex>sect</qex> of which that he was born,<br/
He kept his lay, to which that he was sworn.</q> <rj><qau>Chaucer.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>The cursed <qex>sect</qex> of that detestable and false prophet Mohammed.</q> <rj><qau>Fabyan.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>As concerning this <qex>sect</qex> [Christians], we know that everywhere it is spoken against.</q> <rj><qau>Acts xxviii. 22.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sectant</ent><br/
<hw>Sec"tant</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>secare</ets>, <ets>sectum</ets>, to cut.]</ety> <def>One of the portions of space bounded by the three coordinate planes. Specif. <fld>(Crystallog.)</fld>, one of the parts of a crystal into which it is divided by the axial planes.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sectarian</ent><br/
<hw>Sec*ta"ri*an</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Pertaining to a sect, or to sects; peculiar to a sect; bigotedly attached to the tenets and interests of a denomination; <as>as, <ex>sectarian</ex> principles or prejudices</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sectarian</ent><br/
<hw>Sec*ta"ri*an</hw>, <pos>n.</pos> <def>One of a sect; a member or adherent of a special school, denomination, or religious or philosophical party; one of a party in religion which has separated itself from established church, or which holds tenets different from those of the prevailing denomination in a state.</def><br/
[<source>1913 Webster</source>]</p>

<p><syn><b>Syn.</b> -- See <er>Heretic</er>.</syn><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sectarianism</ent><br/
<hw>Sec*ta"ri*an*ism</hw> <pr>(?)</pr>, <pos>n.</pos> <def>The quality or character of a sectarian; devotion to the interests of a party; excess of partisan or denominational zeal; adherence to a separate church organization.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sectarianize</ent><br/
<hw>Sec*ta"ri*an*ize</hw> <pr>(?)</pr>, <pos>v. t.</pos> <def>To imbue with sectarian feelings; to subject to the control of a sect.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sectarism</ent><br/
<hw>Sec"ta*rism</hw>, <pos>n.</pos> <def>Sectarianism.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sectarist</ent><br/
<hw>Sec"ta*rist</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A sectary.</def> <mark>[R.]</mark>  <rj><au>T. Warton.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sectary</ent><br/
<hw>Sec"ta*ry</hw> <pr>(?)</pr>, <pos>n.</pos>;<plu><it>pl.</it> <plw>Sectaries</plw> <pr>(#)</pr>.</plu> <ety>[F. <ets>sectaire</ets>. See <er>Sect</er>.]</ety> <def>A sectarian; a member or adherent of a sect; a follower or disciple of some particular teacher in philosophy or religion; one who separates from an established church; a dissenter.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>I never knew that time in England when men of truest religion were not counted <qex>sectaries</qex>.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sectator</ent><br/
<hw>Sec*ta"tor</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L., fr. <ets>sectari</ets>, v. intens. fr. <ets>sequi</ets> to follow. See <er>Sue</er> to follow.]</ety> <def>A follower; a disciple; an adherent to a sect.</def> <mark>[Obs.]</mark>  <rj><au>Sir W. Raleigh.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sectile</ent><br/
<hw>Sec"tile</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>sectilis</ets>, fr. <ets>secare</ets>, <ets>sectum</ets>, to cut: cf. F. <ets>sectile</ets>. See <er>Section</er>.]</ety> <def>Capable of being cut; specifically <fld>(Min.)</fld>, capable of being severed by the knife with a smooth cut; -- said of minerals.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sectility</ent><br/
<hw>Sec*til"i*ty</hw> <pr>(?)</pr>, <pos>n.</pos> <def>The state or quality of being sectile.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Section</ent><br/
<hw>Sec"tion</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>sectio</ets>, fr. <ets>secare</ets>, <ets>sectum</ets>, to cut; akin to E. <ets>saw</ets> a cutting instrument: cf. F. <ets>section</ets>. See <er>Saw</er>, and cf. <er>Scion</er>, <er>Dissect</er>, <er>Insect</er>, <er>Secant</er>, <er>Segment</er>.]</ety> <sn>1.</sn> <def>The act of cutting, or separation by cutting; <as>as, the <ex>section</ex> of bodies</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A part separated from something; a division; a portion; a slice.</def> Specifically: --<br/
[<source>1913 Webster</source>]</p>

<p><sd>(a)</sd> <def>A distinct part or portion of a book or writing; a subdivision of a chapter; the division of a law or other writing; a paragraph; an article; hence, the character <sect/, often used to denote such a division.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>It is hardly possible to give a distinct view of his several arguments in distinct <qex>sections</qex>.</q> <rj><qau>Locke.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sd>(b)</sd> <def>A distinct part of a country or people, community, class, or the like; a part of a territory separated by geographical lines, or of a people considered as distinct.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The extreme <qex>section</qex> of one class consists of bigoted dotards, the extreme <qex>section</qex> of the other consists of shallow and reckless empirics.</q> <rj><qau>Macaulay.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sd>(c)</sd> <def>One of the portions, of one square mile each, into which the public lands of the United States are divided; one thirty-sixth part of a township. These sections are subdivided into quarter sections for sale under the homestead and preemption laws.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <fld>(Geom.)</fld> <def>The figure made up of all the points common to a superficies and a solid which meet, or to two superficies which meet, or to two lines which meet. In the first case the section is a superficies, in the second a line, and in the third a point.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <fld>(Nat. Hist.)</fld> <def>A division of a genus; a group of species separated by some distinction from others of the same genus; -- often indicated by the sign <sect/.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <fld>(Mus.)</fld> <def>A part of a musical period, composed of one or more phrases. See <er>Phrase</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>6.</sn> <def>The description or representation of anything as it would appear if cut through by any intersecting plane; depiction of what is beyond a plane passing through, or supposed to pass through, an object, as a building, a machine, a succession of strata; profile.</def><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ In mechanical drawing, as in these Illustrations of a cannon, a <xex>longitudinal section</xex> (<xex>a</xex>) usually represents the object as cut through its center lengthwise and vertically; a <xex>cross</xex> or <xex>transverse section</xex> (<xex>b</xex>), as cut crosswise and vertically; and a <xex>horizontal section</xex> (<xex>c</xex>), as cut through its center horizontally. <xex>Oblique sections</xex> are made at various angles. In architecture, a <xex>vertical section</xex> is a drawing showing the interior, the thickness of the walls, etc., as if made on a vertical plane passed through a building.</note><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Angular sections</b></col> <fld>(Math.)</fld>, <cd>a branch of analysis which treats of the relations of sines, tangents, etc., of arcs to the sines, tangents, etc., of their multiples or of their parts.</cd> <mark>[R.]</mark> -- <col><b>Conic sections</b></col>. <fld>(Geom.)</fld> <cd>See under <er>Conic</er>.</cd> -- <col><b>Section liner</b></col> <fld>(Drawing)</fld>, <cd>an instrument to aid in drawing a series of equidistant parallel lines, -- used in representing sections.</cd> -- <col><b>Thin section</b></col>, <cd>a section or slice, as of mineral, animal, or vegetable substance, thin enough to be transparent, and used for study under the microscope.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><syn><b>Syn.</b> -- Part; portion; division.</syn>  -- <usage><er>Section</er>, <er>Part</er>. The English more commonly apply the word <xex>section</xex> to a part or portion of a body of men; <as>as, a <ex>section</ex> of the clergy, a small <ex>section</ex> of the Whigs, etc.</as>  In the United States this use is less common, but another use, unknown or but little known in England, is very frequent, as in the phrases <ldquo/the eastern <xex>section</xex> of our country,<rdquo/ etc., the same sense being also given to the adjective <xex>sectional</xex>; as, <xex>sectional</xex> feelings, interests, etc.</usage><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sectional</ent><br/
<hw>Sec"tion*al</hw> <pr>(?)</pr>, <pos>a.</pos> <sn>1.</sn> <def>Of or pertaining to a section or distinct part of larger body or territory; local.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>All <qex>sectional</qex> interests, or party feelings, it is hoped, will hereafter yield to schemes of ambition.</q> <rj><qau>Story.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Consisting of sections, or capable of being divided into sections; <as>as, a <ex>sectional</ex> steam boiler</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sectionalism</ent><br/
<hw>Sec"tion*al*ism</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A disproportionate regard for the interests peculiar to a section of the country; local patriotism, as distinguished from national.</def> <mark>[U. S.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sectionality</ent><br/
<hw>Sec"tion*al"i*ty</hw> <pr>(?)</pr>, <pos>n.</pos> <def>The state or quality of being sectional; sectionalism.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sectionalize</ent><br/
<hw>Sec"tion*al*ize</hw> <pr>(?)</pr>, <pos>v. t.</pos> <def>To divide according to geographical sections or local interests.</def> <mark>[U. S.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>The principal results of the struggle were to <qex>sectionalize</qex> parties.</q> <rj><qau>Nicolay & Hay (Life of Lincoln).</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sectionally</ent><br/
<hw>Sec"tion*al*ly</hw>, <pos>adv.</pos> <def>In a sectional manner.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sectionize</ent><br/
<hw>Sec"tion*ize</hw> <pr>(?)</pr>, <pos>v. t.</pos> <def>To form into sections.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sectism</ent><br/
<hw>Sect"ism</hw> <pr>(?)</pr>, <pos>n.</pos> <def>Devotion to a sect.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sectist</ent><br/
<hw>Sect"ist</hw>, <pos>n.</pos> <def>One devoted to a sect; a sectary.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sectiuncle</ent><br/
<hw>Sect"i*un`cle</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A little or petty sect.</def> <mark>[R.]</mark> <ldquo/Some new sect or <xex>sectiuncle</xex>.<rdquo/  <rj><au>J. Martineau.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sector</ent><br/
<hw>Sec"tor</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L., properly, a cutter, fr. <ets>secare</ets>, <ets>sectum</ets>, to cut: cf. F. <ets>secteur</ets>. See <er>Section</er>.]</ety> <sn>1.</sn> <fld>(Geom.)</fld> <def>A part of a circle comprehended between two radii and the included arc.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A mathematical instrument, consisting of two rulers connected at one end by a joint, each arm marked with several scales, as of equal parts, chords, sines, tangents, etc., one scale of each kind on each arm, and all on lines radiating from the common center of motion. The sector is used for plotting, etc., to any scale.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>An astronomical instrument, the limb of which embraces a small portion only of a circle, used for measuring differences of declination too great for the compass of a micrometer. When it is used for measuring zenith distances of stars, it is called a <xex>zenith sector</xex>.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Dip sector</b></col>, <cd>an instrument used for measuring the dip of the horizon.</cd> -- <mcol><col><b>Sector of a sphere</b></col>, <it>or</it> <col><b>Spherical sector</b></col></mcol>, <cd>the solid generated by the revolution of the sector of a circle about one of its radii, or, more rarely, about any straight line drawn in the plane of the sector through its vertex.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sectoral</ent><br/
<hw>Sec"tor*al</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Of or pertaining to a sector; <as>as, a <ex>sectoral</ex> circle</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sectorial</ent><br/
<hw>Sec*to"ri*al</hw> <pr>(?)</pr>, <pos>a.</pos> <fld>(Anat.)</fld> <def>Adapted for cutting.</def> -- <def2><pos>n.</pos> <def>A sectorial, or carnassial, tooth.</def></def2><br/
[<source>1913 Webster</source>]</p>

<p><ent>Secular</ent><br/
<hw>Sec"u*lar</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[OE. <ets>secular</ets>, <ets>seculer</ets>. L. <ets>saecularis</ets>, fr. <ets>saeculum</ets> a race, generation, age, the times, the world; perhaps akin to E. <ets>soul</ets>: cf. F. <ets>s<eacute/culier</ets>.]</ety> <sn>1.</sn> <def>Coming or observed once in an age or a century.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The <qex>secular</qex> year was kept but once a century.</q> <rj><qau>Addison.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Pertaining to an age, or the progress of ages, or to a long period of time; accomplished in a long progress of time; <as>as, <ex>secular</ex> inequality; the <ex>secular</ex> refrigeration of the globe</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>Of or pertaining to this present world, or to things not spiritual or holy; relating to temporal as distinguished from eternal interests; not immediately or primarily respecting the soul, but the body; worldly.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>New foes arise,<br/
Threatening to bind our souls with <qex>secular</qex> chains.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <fld>(Eccl.)</fld> <def>Not regular; not bound by monastic vows or rules; not confined to a monastery, or subject to the rules of a religious community; <as>as, a <ex>secular</ex> priest</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>He tried to enforce a stricter discipline and greater regard for morals, both in the religious orders and the <qex>secular</qex> clergy.</q> <rj><qau>Prescott.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <def>Belonging to the laity; lay; not clerical.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>I speak of folk in <qex>secular</qex> estate.</q> <rj><qau>Chaucer.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Secular equation</b></col> <fld>(Astron.)</fld>, <cd>the algebraic or numerical expression of the magnitude of the inequalities in a planet's motion that remain after the inequalities of a short period have been allowed for.</cd> -- <col><b>Secular games</b></col> <fld>(Rom. Antiq.)</fld>, <cd>games celebrated, at long but irregular intervals, for three days and nights, with sacrifices, theatrical shows, combats, sports, and the like.</cd> -- <col><b>Secular music</b></col>, <cd>any music or songs not adapted to sacred uses.</cd> -- <mcol><col><b>Secular hymn</b></col> <it>or</it> <col><b>Secular poem</b></col></mcol>, <cd>a hymn or poem composed for the secular games, or sung or rehearsed at those games.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Secular</ent><br/
<hw>Sec"u*lar</hw>, <pos>n.</pos> <sn>1.</sn> <fld>(Eccl.)</fld> <def>A secular ecclesiastic, or one not bound by monastic rules.</def>  <rj><au>Burke.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Eccl.)</fld> <def>A church official whose functions are confined to the vocal department of the choir.</def>  <rj><au>Busby.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>A layman, as distinguished from a clergyman.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Secularism</ent><br/
<hw>Sec"u*lar*ism</hw> <pr>(?)</pr>, <pos>n.</pos> <sn>1.</sn> <def>The state or quality of being secular; a secular spirit; secularity.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>The tenets or principles of the secularists.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Secularist</ent><br/
<hw>Sec"u*lar*ist</hw>, <pos>n.</pos> <def>One who theoretically rejects every form of religious faith, and every kind of religious worship, and accepts only the facts and influences which are derived from the present life; also, one who believes that education and other matters of civil policy should be managed without the introduction of a religious element.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Secularity</ent><br/
<hw>Sec`u*lar"i*ty</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Cf.F. <ets>s<eacute/cularit<eacute/</ets>, LL. <ets>saecularitas</ets>.]</ety> <def>Supreme attention to the things of the present life; worldliness.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>A <qex>secularity</qex> of character which makes Christianity and its principal doctrines distasteful or unintelligible.</q> <rj><qau>I. Taylor.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Secularization</ent><br/
<hw>Sec`u*lar*i*za"tion</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Cf. F. <ets>s<eacute/cularisation</ets>.]</ety> <def>The act of rendering secular, or the state of being rendered secular; conversion from regular or monastic to secular; conversion from religious to lay or secular possession and uses; <as>as, the <ex>secularization</ex> of church property</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><-- p. 1302 pr=PCP --></p>

<p><ent>Secularize</ent><br/
<hw>Sec"u*lar*ize</hw> <pr>(?)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Secularized</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Secularizing</conjf> <pr>(?)</pr>.]</vmorph> <ety>[Cf. F. <ets>s<eacute/culariser</ets>.]</ety> <sn>1.</sn> <def>To convert from regular or monastic into secular; <as>as, to <ex>secularize</ex> a priest or a monk</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To convert from spiritual to secular or common use; <as>as, to <ex>secularize</ex> a church, or church property</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>At the Reformation the abbey was <qex>secularized</qex>.</q> <rj><qau>W. Coxe.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>To make worldly or unspiritual.</def>  <rj><au>Bp. Horsley.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Secularly</ent><br/
<hw>Sec"u*lar*ly</hw>, <pos>adv.</pos> <def>In a secular or worldly manner.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Secularness</ent><br/
<hw>Sec"u*lar*ness</hw>, <pos>n.</pos> <def>The quality or state of being secular; worldliness; worldly-mindedness.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Secund</ent><br/
<hw>Se"cund</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>secundus</ets> following the course or current of wind or water. See <er>Second</er>, <pos>a.</pos>]</ety> <fld>(Bot.)</fld> <def>Arranged on one side only, as flowers or leaves on a stalk.</def>  <rj><au>Gray.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Secundate</ent><br/
<hw>Se*cun"date</hw> <pr>(?)</pr>, <pos>v. t.</pos> <ety>[L. <ets>secundatus</ets>, p. p. of <ets>secundare</ets> to direct favorably.]</ety> <def>To make prosperous.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Secundation</ent><br/
<hw>Sec`un*da"tion</hw> <pr>(?)</pr>, <pos>n.</pos> <def>Prosperity.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Secundine</ent><br/
<hw>Sec"un*dine</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Cf. F. <ets>secondine</ets>.]</ety> <sn>1.</sn> <fld>(Bot.)</fld> <def>The second coat, or integument, of an ovule, lying within the primine.</def><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ In the ripened seed the primine and secundine are usually united to form the testa, or outer seed coat. When they remain distinct the secundine becomes the mesosperm, as in the castor bean.</note><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <ety>[Cf. F. <ets>secondines</ets>.]</ety> <def>The afterbirth, or placenta and membranes; -- generally used in the plural.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Secundo-geniture</ent><br/
<hw>Se*cun`do-gen"i*ture</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>secundus</ets> second + <ets>genitura</ets> a begetting, generation.]</ety> <def>A right of inheritance belonging to a second son; a property or possession so inherited.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The kingdom of Naples . . . was constituted a <qex>secundo-geniture</qex> of Spain.</q> <rj><qau>Bancroft.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Securable</ent><br/
<hw>Se*cur"a*ble</hw> <pr>(?)</pr>, <pos>a.</pos> <def>That may be secured.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Secure</ent><br/
<hw>Se*cure"</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>securus</ets>; pref. <ets>se-</ets> without + <ets>cura</ets> care. See <er>Cure</er> care, and cf. <er>Sure</er>, <pos>a.</pos>]</ety> <sn>1.</sn> <def>Free from fear, care, or anxiety; easy in mind; not feeling suspicion or distrust; confident.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>But thou, <qex>secure</qex> of soul, unbent with woes.</q> <rj><qau>Dryden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Overconfident; incautious; careless; -- in a bad sense.</def>  <rj><au>Macaulay.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>Confident in opinion; not entertaining, or not having reason to entertain, doubt; certain; sure; -- commonly with <xex>of</xex>; <as>as, <ex>secure</ex> of a welcome</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Confidence then bore thee on, <qex>secure</qex><br/
Either to meet no danger, or to find<br/
Matter of glorious trial.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>Not exposed to danger; safe; -- applied to persons and things, and followed by <xex>against</xex> or <xex>from</xex>.</def> <ldquo/<xex>Secure</xex> from fortune's blows.<rdquo/  <rj><au>Dryden.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><syn><b>Syn.</b> -- Safe; undisturbed; easy; sure; certain; assured; confident; careless; heedless; inattentive.</syn><br/
[<source>1913 Webster</source>]</p>

<p><ent>Secure</ent><br/
<hw>Se*cure"</hw>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Secured</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Securing</conjf>.]</vmorph> <sn>1.</sn> <def>To make safe; to relieve from apprehensions of, or exposure to, danger; to guard; to protect.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>I spread a cloud before the victor's sight,<br/
Sustained the vanquished, and <qex>secured</qex> his flight.</q> <rj><qau>Dryden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To put beyond hazard of losing or of not receiving; to make certain; to assure; to insure; -- frequently with <xex>against</xex> or <xex>from</xex>, rarely with <xex>of</xex>; <as>as, to <ex>secure</ex> a creditor against loss; to <ex>secure</ex> a debt by a mortgage</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>It <qex>secures</qex> its possessor of eternal happiness.</q> <rj><qau>T. Dick.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>To make fast; to close or confine effectually; to render incapable of getting loose or escaping; <as>as, to <ex>secure</ex> a prisoner; to <ex>secure</ex> a door, or the hatches of a ship</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>To get possession of; to make one's self secure of; to acquire certainly; <as>as, to <ex>secure</ex> an estate</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Secure arms</b></col> <fld>(Mil.)</fld>, <cd>a command and a position in the manual of arms, used in wet weather, the object being to guard the firearm from becoming wet. The piece is turned with the barrel to the front and grasped by the right hand at the lower band, the muzzle is dropped to the front, and the piece held with the guard under the right arm, the hand supported against the hip, and the thumb on the rammer.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Securely</ent><br/
<hw>Se*cure"ly</hw>, <pos>adv.</pos> <def>In a secure manner; without fear or apprehension; without danger; safely.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>His daring foe . . . <qex>securely</qex> him defied.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Securement</ent><br/
<hw>Se*cure"ment</hw> <pr>(?)</pr>, <pos>n.</pos> <def>The act of securing; protection.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>Society condemns the <qex>securement</qex> in all cases of perpetual protection by means of perpetual imprisonment.</q> <rj><qau>C. A. Ives.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Secureness</ent><br/
<hw>Se*cure"ness</hw>, <pos>n.</pos> <def>The condition or quality of being secure; exemption from fear; want of vigilance; security.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Securer</ent><br/
<hw>Se*cur"er</hw> <pr>(?)</pr>, <pos>n.</pos> <def>One who, or that which, secures.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Securifera</ent><br/
||<hw>Sec`u*rif"e*ra</hw> <pr>(?)</pr>, <pos>n. pl.</pos> <ety>[NL., from L. <ets>securis</ets> an ax + <ets>ferre</ets> to bear.]</ety> <fld>(Zool.)</fld> <def>The Serrifera.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Securiform</ent><br/
<hw>Se*cu"ri*form</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>securis</ets> an ax or hatchet + <ets>-form</ets>: cf. F. <ets>s<eacute/curiforme</ets>.]</ety> <fld>(Nat. Hist.)</fld> <def>Having the form of an ax or hatchet.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Securipalp</ent><br/
<hw>Se*cu"ri*palp</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>securis</ets> ax, hatchet + E. <ets>palp</ets>.]</ety> <fld>(Zool.)</fld> <def>One of a family of beetles having the maxillary palpi terminating in a hatchet-shaped joint.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Security</ent><br/
<hw>Se*cu"ri*ty</hw> <pr>(?)</pr>, <pos>n.</pos>; <plu><it>pl.</it>  <plw>Securities</plw> <pr>(#)</pr>.</plu> <ety>[L. <ets>securitas</ets>: cf. F. <ets>s<eacute/curit<eacute/</ets>. See <er>Secure</er>, and cf. <er>Surety</er>.]</ety> <sn>1.</sn> <def>The condition or quality of being secure; secureness.</def> <specif>Specifically:</specif> <sd>(a)</sd> <def>Freedom from apprehension, anxiety, or care; confidence of power or safety; hence, assurance; certainty.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>His trembling hand had lost the ease,<br/
Which marks <qex>security</qex> to please.</q> <rj><qau>Sir W. Scott.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sd>(b)</sd> <def>Hence, carelessness; negligence; heedlessness.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>He means, my lord, that we are too remiss,<br/
Whilst Bolingbroke, through our <qex>security</qex>,<br/
Grows strong and great in substance and in power.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sd>(c)</sd> <def>Freedom from risk; safety.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Give up yourself merely to chance and hazard,<br/
From firm <qex>security</qex>.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Some . . . alleged that we should have no <qex>security</qex> for our trade.</q> <rj><qau>Swift.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>That which secures or makes safe; protection; guard; defense.</def> Specifically: <sd>(a)</sd> <def>Something given, deposited, or pledged, to make certain the fulfillment of an obligation, the performance of a contract, the payment of a debt, or the like; surety; pledge.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Those who lent him money lent it on no <qex>security</qex> but his bare word.</q> <rj><qau>Macaulay.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sd>(b)</sd> <def>One who becomes surety for another, or engages himself for the performance of another's obligation.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>An evidence of debt or of property, as a bond, a certificate of stock, etc.; <as>as, government <ex>securities</ex></as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><syn><b>Syn.</b> -- Protection; defense; guard; shelter; safety; certainty; ease; assurance; carelessness; confidence; surety; pledge; bail.</syn><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sedan</ent><br/
<hw>Se*dan"</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Said to be named from <ets>Sedan</ets>, in France, where it was first made, and whence it was introduced into England in the time of King Charles I.]</ety> <def>A portable chair or covered vehicle for carrying a single person, -- usually borne on poles by two men. Called also <altname>sedan chair</altname>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sedate</ent><br/
<hw>Se*date"</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>sedatus</ets>, p. p. of <ets>sedare</ets>, <ets>sedatum</ets>, to allay, calm, causative of <ets>sedere</ets> to sit. See <er>Sit</er>.]</ety> <def>Undisturbed by passion or caprice; calm; tranquil; serene; not passionate or giddy; composed; staid; <as>as, a <ex>sedate</ex> soul, mind, or temper</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Disputation carries away the mind from that calm and <qex>sedate</qex> temper which is so necessary to contemplate truth.</q> <rj><qau>I. Watts.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Whatsoever we feel and know<br/
Too <qex>sedate</qex> for outward show.</q> <rj><qau>Wordsworth.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><syn><b>Syn.</b> -- Settled; composed; calm; quiet; tranquil; still; serene; unruffled; undisturbed; contemplative; sober; serious.</syn><br/
[<source>1913 Webster</source>]</p>

<p>-- <wordforms><wf>Se*date"ly</wf>, <pos>adv.</pos> -- <wf>Se*date"ness</wf>, <pos>n.</pos></wordforms><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sedation</ent><br/
<hw>Se*da"tion</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>sedatio</ets>.]</ety> <def>The act of calming, or the state of being calm.</def> <mark>[R.]</mark>  <rj><au>Coles.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sedative</ent><br/
<hw>Sed"a*tive</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[Cf. F.  <ets>s<eacute/datif</ets>.]</ety> <def>Tending to calm, moderate, or tranquilize</def>; specifically <fld>(Med.)</fld>, <def>allaying irritability and irritation; assuaging pain.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sedative</ent><br/
<hw>Sed"a*tive</hw>, <pos>n.</pos> <fld>(Med.)</fld> <def>A remedy which allays irritability and irritation, and irritative activity or pain.</def> <-- ##now usu. referring to tranquilizer or sleeping pill. --><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sedent</ent><br/
<hw>Se"dent</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>sedens</ets>, <ets>-entis</ets>, p. pr. of <ets>sedere</ets> to sit. See <er>Sit</er>.]</ety> <def>Sitting; inactive; quiet.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sedentarily</ent><br/
<hw>Sed"en*ta*ri*ly</hw> <pr>(?)</pr>, <pos>adv.</pos> <def>In a sedentary manner.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sedentariness</ent><br/
<hw>Sed"en*ta*ri*ness</hw>, <pos>n.</pos> <def>Quality of being sedentary.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sedentary</ent><br/
<hw>Sed"en*ta*ry</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>sedentarius</ets>, fr. <ets>sedere</ets> to sit: cf. F. <ets>se<eacute/dentaire</ets>. See <er>Sedent</er>.]</ety> <sn>1.</sn> <def>Accustomed to sit much or long; <as>as, a <ex>sedentary</ex> man</as>.</def> <ldquo/<xex>Sedentary</xex>, scholastic sophists.<rdquo/  <rj><au>Bp. Warburton.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Characterized by, or requiring, much sitting; <as>as, a <ex>sedentary</ex> employment; a <ex>sedentary</ex> life</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Any education that confined itself to <qex>sedentary</qex> pursuits was essentially imperfect.</q> <rj><qau>Beaconsfield.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>Inactive; motionless; sluggish; hence, calm; tranquil.</def> <mark>[R.]</mark> <ldquo/The <xex>sedentary</xex> earth.<rdquo/  <rj><au>Milton.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>The soul, considered abstractly from its passions, is of a remiss, <qex>sedentary</qex> nature.</q> <rj><qau>Spectator.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>Caused by long sitting.</def> <mark>[Obs.]</mark> <ldquo/<xex>Sedentary</xex> numbness.<rdquo/  <rj><au>Milton.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <fld>(Zool.)</fld> <def>Remaining in one place, especially when firmly attached to some object; <as>as, the oyster is a <ex>sedentary</ex> mollusk; the barnacles are <ex>sedentary</ex> crustaceans</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Sedentary spider</b></col> <fld>(Zool.)</fld>, <cd>one of a tribe of spiders which rest motionless until their prey is caught in their web.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sederunt</ent><br/
||<hw>Se*de"runt</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L., they sat, fr. <ets>sedere</ets> to sit.]</ety> <def>A sitting, as of a court or other body.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>'T is pity we have not Burns's own account of that long <qex>sederunt</qex>.</q> <rj><qau>Prof. Wilson.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Acts of sederunt</b></col> <fld>(Scots Law)</fld>, <cd>ordinances of the Court of Session for the ordering of processes and expediting of justice.</cd>  <rj><au>Bell.</au></rj></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sedge</ent><br/
<hw>Sedge</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[OE. <ets>segge</ets>, AS. <ets>secg</ets>; akin to LG. <ets>segge</ets>; -- probably named from its bladelike appearance, and akin to L. <ets>secare</ets> to cut, E. <ets>saw</ets> a cutting instrument; cf. Ir. <ets>seisg</ets>, W. <ets>hesg</ets>.  Cf. <er>Hassock</er>, <er>Saw</er> the instrument.]</ety> <sn>1.</sn> <fld>(Bot.)</fld> <def>Any plant of the genus <gen>Carex</gen>, perennial, endogenous, innutritious herbs, often growing in dense tufts in marshy places. They have triangular jointless stems, a spiked inflorescence, and long grasslike leaves which are usually rough on the margins and midrib. There are several hundred species.</def><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ The name is sometimes given to any other plant of the order <ord>Cyperaceae</ord>, which includes <gen>Carex</gen>, <gen>Cyperus</gen>, <gen>Scirpus</gen>, and many other genera of rushlike plants.</note><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Zool.)</fld> <def>A flock of herons.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Sedge hen</b></col> <fld>(Zool.)</fld>, <cd>the clapper rail. See under 5th <er>Rail</er>.</cd> -- <col><b>Sedge warbler</b></col> <fld>(Zool.)</fld>, <cd>a small European singing bird (<spn>Acrocephalus phragmitis</spn>). It often builds its nest among reeds; -- called also <altname>sedge bird</altname>, <altname>sedge wren</altname>, <altname>night warbler</altname>, and <altname>Scotch nightingale</altname>.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sedged</ent><br/
<hw>Sedged</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Made or composed of sedge.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>With your <qex>sedged</qex> crowns and ever-harmless looks.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sedgy</ent><br/
<hw>Sedg"y</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Overgrown with sedge.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>On the gentle Severn's <qex>sedgy</qex> bank.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sedilia</ent><br/
||<hw>Se*dil"i*a</hw> <pr>(?)</pr>, <pos>n. pl.</pos>; <sing>sing. <singw>Sedile</singw> <pr>(<?/)</pr></sing>. <ety>[L. <ets>sedile</ets> a seat.]</ety> <fld>(Arch.)</fld> <def>Seats in the chancel of a church near the altar for the officiating clergy during intervals of service.</def>  <rj><au>Hook.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sediment</ent><br/
<hw>Sed"i*ment</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F. <ets>s<eacute/diment</ets>, L. <ets>sedimentum</ets> a settling, fr. <ets>sedere</ets> to sit, to settle. See <er>Sit</er>.]</ety> <sn>1.</sn> <def>The matter which subsides to the bottom, from water or any other liquid; settlings; lees; dregs.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Geol.)</fld> <def>The material of which sedimentary rocks are formed.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sedimental</ent><br/
<hw>Sed`i*men"tal</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Sedimentary.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sedimentary</ent><br/
<hw>Sed`i*men"ta*ry</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[Cf. F. <ets>s<eacute/dimentaire</ets>.]</ety> <def>Of or pertaining to sediment; formed by sediment; containing matter that has subsided.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Sedimentary rocks</b></col>. <fld>(Geol.)</fld> <cd>See <cref>Aqueous rocks</cref>, under <er>Aqueous</er>.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sedimentation</ent><br/
<hw>Sed`i*men*ta"tion</hw> <pr>(?)</pr>, <pos>n.</pos> <def>The act of depositing a sediment; specifically <fld>(Geol.)</fld>, the deposition of the material of which sedimentary rocks are formed.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sedition</ent><br/
<hw>Se*di"tion</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[OE. <ets>sedicioun</ets>, OF.  <ets>sedition</ets>, F. <ets>s<eacute/dition</ets>, fr. L. <ets>seditio</ets>, originally, a going aside; hence, an insurrectionary separation; pref. <ets>se-</ets>, <ets>sed-</ets>, aside + <ets>itio</ets> a going, fr. <ets>ire</ets>, <ets>itum</ets>, to go.  Cf. <er>Issue</er>.]</ety> <sn>1.</sn> <def>The raising of commotion in a state, not amounting to insurrection; conduct tending to treason, but without an overt act; excitement of discontent against the government, or of resistance to lawful authority.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>In soothing them, we nourish 'gainst our senate<br/
The cockle of rebellion, insolence, <qex>sedition</qex>.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Noisy demagogues who had been accused of <qex>sedition</qex>.</q> <rj><qau>Macaulay.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Dissension; division; schism.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>Now the works of the flesh are manifest, . . . emulations, wrath, strife, <qex>seditions</qex>, heresies.</q> <rj><qau>Gal. v. 19, 20.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><syn><b>Syn.</b> -- Insurrection; tumult; uproar; riot; rebellion; revolt. See <er>Insurrection</er>.</syn><br/
[<source>1913 Webster</source>]</p>

<p><ent>Seditionary</ent><br/
<hw>Se*di"tion*a*ry</hw> <pr>(?)</pr>, <pos>n.</pos> <def>An inciter or promoter of sedition.</def>  <rj><au>Bp. Hall.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Seditious</ent><br/
<hw>Se*di"tious</hw> <pr>(?)</pr>, <pos>a.</pos><ety>[L. <ets>seditiosus</ets>: cf. F. <ets>s<eacute/ditieux</ets>.]</ety> <sn>1.</sn> <def>Of or pertaining to sedition; partaking of the nature of, or tending to excite, sedition; <as>as, <ex>seditious</ex> behavior; <ex>seditious</ex> strife; <ex>seditious</ex> words</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Disposed to arouse, or take part in, violent opposition to lawful authority; turbulent; factious; guilty of sedition; <as>as, <ex>seditious</ex> citizens</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p>-- <wordforms><wf>Se*di"tious*ly</wf>, <pos>adv.</pos> -- <wf>Se*di"tious*ness</wf>, <pos>n.</pos></wordforms><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sedlitz</ent><br/
<hw>Sed"litz</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Same as <er>Seidlitz</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Seduce</ent><br/
<hw>Se*duce"</hw> <pr>(?)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Seduced</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Seducing</conjf> <pr>(?)</pr>.]</vmorph> <ety>[L. <ets>seducere</ets>, <ets>seductum</ets>; pref. <ets>se-</ets> aside + <ets>ducere</ets> to lead. See <er>Duke</er>.]</ety> <sn>1.</sn> <def>To draw aside from the path of rectitude and duty in any manner; to entice to evil; to lead astray; to tempt and lead to iniquity; to corrupt.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>For me, the gold of France did not <qex>seduce</qex>.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Specifically, to induce to engage in sexual intercourse.</def><br/
[<source>PJC</source>]</p>

<p><syn><b>Syn.</b> -- To allure; entice; tempt; attract; mislead; decoy; inveigle. See <er>Allure</er>.</syn><br/
[<source>1913 Webster</source>]</p>

<p><ent>Seducement</ent><br/
<hw>Se*duce"ment</hw> <pr>(?)</pr>, <pos>n.</pos> <sn>1.</sn> <def>The act of seducing.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>The means employed to seduce, as flattery, promises, deception, etc.; arts of enticing or corrupting.</def>  <rj><au>Pope.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Seducer</ent><br/
<hw>Se*du"cer</hw> <pr>(?)</pr>, <pos>n.</pos> <def>One who, or that which, seduces.  For a female seducer, the term <er>seductress</er> is also used</def><br/
[<source>1913 Webster</source> <source>+PJC</source>]</p>

<p><q>He whose firm faith no reason could remove,<br/
Will melt before that soft <qex>seducer</qex>, love.</q> <rj><qau>Dryden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <specif>Specifically:</specif> <def>One who induces another to engage in sexual intercourse.</def><br/
[<source>PJC</source>]</p>

<p><ent>Seducible</ent><br/
<hw>Se*du"ci*ble</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Capable of being seduced; corruptible.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Seducing</ent><br/
<hw>Se*du"cing</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Seductive.</def>  <ldquo/Thy sweet <xex>seducing</xex> charms.<rdquo/ <au>Cowper.</au> -- <wordforms><wf>Se*du"cing*ly</wf>, <pos>adv.</pos></wordforms><br/
[<source>1913 Webster</source>]</p>

<p><ent>Seduction</ent><br/
<hw>Se*duc*tion</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>seductio</ets>: cf. F. <ets>s<eacute/duction</ets>. <ets>See</ets> <er>Seduce</er>.]</ety> <sn>1.</sn> <def>The act of seducing; enticement to wrong doing; enticement to fail in some duty.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <specif>Specifically:</specif> <sd>(a)</sd> <def>The offense of inducing a woman to consent to unlawful sexual intercourse, by enticements which overcome her scruples; the wrong or crime of persuading a woman to surrender her chastity.</def> <mark>[Archaic]</mark> <sd>(b)</sd> <def>Any successful enticement to engage in some sexual activity, especially intercourse.</def><br/
[<source>1913 Webster</source> <source>+PJC</source>]</p>

<p><sn>3.</sn> <def>That which seduces, or is adapted to seduce; means of leading astray; <as>as, the <ex>seductions</ex> of wealth</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Seductive</ent><br/
<hw>Se*duc"tive</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Tending to lead astray; apt to mislead by flattering appearances; tempting; alluring; <as>as, a <ex>seductive</ex> offer</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>This may enable us to understand how <qex>seductive</qex> is the influence of example.</q> <rj><qau>Sir W. Hamilton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Seductively</ent><br/
<hw>Se*duc"tive*ly</hw>, <pos>adv.</pos> <def>In a seductive manner.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Seductress</ent><br/
<hw>Se*duc"tress</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A woman who seduces; a female seducer.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sedulity</ent><br/
<hw>Se*du"li*ty</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>sedulitas</ets>. See <er>Sedulous</er>.]</ety> <def>The quality or state of being sedulous; diligent and assiduous application; constant attention; unremitting industry; sedulousness.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The industrious bee, by his <qex>sedulity</qex> in summer, lives in honey all the winter.</q> <rj><qau>Feltham.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sedulous</ent><br/
<hw>Sed"u*lous</hw> <pr>(?; 135)</pr>, <pos>a.</pos> <ety>[L. <ets>sedulus</ets>, perhaps from <ets>sedere</ets> to sit, and so akin to E. <ets>sit</ets>.]</ety> <def>Diligent in application or pursuit; constant, steady, and persevering in business, or in endeavors to effect an object; steadily industrious; assiduous; <as>as, the <ex>sedulous</ex> bee</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>What signifies the sound of words in prayer, without the affection of the heart, and a <qex>sedulous</qex> application of the proper means that may naturally lead us to such an end?</q> <rj><qau>L'Estrange.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><syn><b>Syn.</b> -- Assiduous; diligent; industrious; laborious; unremitting; untiring; unwearied; persevering.</syn><br/
[<source>1913 Webster</source>]</p>

<p>-- <wordforms><wf>Sed"u*lous*ly</wf>, <pos>adv.</pos> -- <wf>Sed"u*lous*ness</wf>, <pos>n.</pos></wordforms><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sedum</ent><br/
||<hw>Se"dum</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[NL., fr. L. <ets>sedere</ets> to sit; so called in allusion to the manner in which the plants attach themselves to rocks and walls.]</ety> <fld>(Bot.)</fld> <def>A genus of plants, mostly perennial, having succulent leaves and cymose flowers; orpine; stonecrop.</def>  <rj><au>Gray.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>See</ent><br/
<hw>See</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[OE. <ets>se</ets>, <ets>see</ets>, OF. <ets>se</ets>, <ets>sed</ets>, <ets>sied</ets>, fr. L. <ets>sedes</ets> a seat, or the kindred <ets>sedere</ets> to sit. See <er>Sit</er>, and cf. <er>Siege</er>.]</ety> <sn>1.</sn> <def>A seat; a site; a place where sovereign power is exercised.</def> <mark>[Obs.]</mark>  <rj><au>Chaucer.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Jove laughed on Venus from his sovereign <qex>see</qex>.</q> <rj><qau>Spenser.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <specif>Specifically:</specif> <sd>(a)</sd> <def>The seat of episcopal power; a diocese; the jurisdiction of a bishop; <as>as, the <ex>see</ex> of New York</as>.</def> <sd>(b)</sd> <def>The seat of an archbishop; a province or jurisdiction of an archbishop; <as>as, an archiepiscopal <ex>see</ex></as>.</def> <sd>(c)</sd> <def>The seat, place, or office of the pope, or Roman pontiff; <as>as, the papal <ex>see</ex></as>.</def> <sd>(d)</sd> <def>The pope or his court at Rome; <as>as, to appeal to the <ex>see</ex> of Rome</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Apostolic see</b></col>. <cd>See under <er>Apostolic</er>.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><-- p. 1303 pr=PCP --></p>

<p><ent>See</ent><br/
<hw>See</hw> <pr>(s<emac/)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp.</pos> <conjf>Saw</conjf> <pr>(s<add/)</pr>; <pos>p. p.</pos> <conjf>Seen</conjf> <pr>(s<emac/n)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Seeing</conjf>.]</vmorph> <ety>[OE. <ets>seen</ets>, <ets>sen</ets>, <ets>seon</ets>, AS. <ets>se<oacute/n</ets>; akin to OFries. <ets>s<imac/a</ets>, D. <ets>zien</ets>, OS. & OHG. <ets>sehan</ets>, G. <ets>sehen</ets>, Icel. <ets>sj<amac/</ets>, Sw. <ets>se</ets>, Dan. <ets>see</ets>, Goth. <ets>sa<iacute/hwan</ets>, and probably to L. <ets>sequi</ets> to follow (and so originally meaning, to follow with the eyes). Gr. <grk>"e`pesqai</grk>, Skr. <ets>sac</ets>.  Cf. <er>Sight</er>, <er>Sue</er> to follow.]</ety> <sn>1.</sn> <def>To perceive by the eye; to have knowledge of the existence and apparent qualities of by the organs of sight; to behold; to descry; to view.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>I will now turn aside, and <qex>see</qex> this great sight.</q> <rj><qau>Ex. iii. 3.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To perceive by mental vision; to form an idea or conception of; to note with the mind; to observe; to discern; to distinguish; to understand; to comprehend; to ascertain.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Go, I pray thee, <qex>see</qex> whether it be well with thy brethren.</q> <rj><qau>Gen. xxxvii. 14.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Jesus <qex>saw</qex> that he answered discreetly.</q> <rj><qau>Mark xii. 34.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Who's so gross<br/
That <qex>seeth</qex> not this palpable device?</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>To follow with the eyes, or as with the eyes; to watch; to regard attentively; to look after.</def>  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>I had a mind to <qex>see</qex> him out, and therefore did not care for contradicting him.</q> <rj><qau>Addison.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>To have an interview with; especially, to make a call upon; to visit; <as>as, to go to <ex>see</ex> a friend</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>And Samuel came no more to <qex>see</qex> Saul until the day of his death.</q> <rj><qau>1 Sam. xv. 35.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <def>To fall in with; to meet or associate with; to have intercourse or communication with; hence, to have knowledge or experience of; <as>as, to <ex>see</ex> military service</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Make us glad according to the days wherein thou hast afflicted us, and the years wherein we have <qex>seen</qex> evil.</q> <rj><qau>Ps. xc. 15.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Verily, verily, I say unto you, if a man keep my saying, he shall never <qex>see</qex> death.</q> <rj><qau>John viii. 51.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Improvement in wisdom and prudence by <qex>seeing</qex> men.</q> <rj><qau>Locke.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>6.</sn> <def>To accompany in person; to escort; to wait upon; <as>as, to <ex>see</ex> one home; to <ex>see</ex> one aboard the cars</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>7.</sn> <def>In poker and similar games at cards, to meet (a bet), or to equal the bet of (a player), by staking the same sum.</def> <ldquo/I'll <ex>see</ex> you and raise you ten.<rdquo/<br/
[<source>Webster 1913 Suppl.</source> <source>+PJC</source>]</p>

<p><cs><mcol><col><b>God you see</b></col> (<it>or</it> <col><b>God him see</b></col> <it>or</it> <col><b>God me see</b></col>, etc.)</mcol>, <cd>God keep you (him, me, etc.) in his sight; God protect you.</cd> <mark>[Obs.]</mark> <au>Chaucer.</au> -- <col><b>To see <plain>(anything)</plain> out</b></col>, <cd>to see (it) to the end; to be present at, work at, or attend, to the end.</cd> -- <col><b>To see stars</b></col>, <cd>to see flashes of light, like stars; -- sometimes the result of concussion of the head.</cd> <mark>[Colloq.]</mark> -- <col><b>To see <plain>(one)</plain> through</b></col>, <cd>to help, watch, or guard (one) to the end of a course or an undertaking.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>See</ent><br/
<hw>See</hw>, <pos>v. i.</pos> <sn>1.</sn> <def>To have the power of sight, or of perceiving by the proper organs; to possess or employ the sense of vision; <as>as, he <ex>sees</ex> distinctly</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Whereas I was blind, now I <qex>see</qex>.</q> <rj><qau>John ix. 25.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Figuratively: To have intellectual apprehension; to perceive; to know; to understand; to discern; -- often followed by a preposition, as <xex>through</xex>, or <xex>into</xex>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>For judgment I am come into this world, that they which <qex>see</qex> not might <qex>see</qex>; and that they which <qex>see</qex> might be made blind.</q> <rj><qau>John ix. 39.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Many sagacious persons will find us out, . . . and <qex>see</qex> through all our fine pretensions.</q> <rj><qau>Tillotson.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>To be attentive; to take care; to give heed; -- generally with <xex>to</xex>; <as>as, to <ex>see</ex> to the house</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q><qex>See</qex> that ye fall not out by the way.</q> <rj><qau>Gen. xlv. 24.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ <xex>Let me see</xex>, <xex>Let us see</xex>, are used to express consideration, or to introduce the particular consideration of a subject, or some scheme or calculation.</note><br/
[<source>1913 Webster</source>]</p>

<p><q>Cassio's a proper man, <qex>let me see</qex> now, -<br/
To get his place.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ <xex>See</xex> is sometimes used in the imperative for <xex>look</xex>, or <xex>behold</xex>. <ldquo/<xex>See</xex>. <xex>see</xex>! upon the banks of Boyne he stands.<rdquo/</note>  <rj><au>Halifax.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>To see about a thing</b></col>, <cd>to pay attention to it; to consider it.</cd> -- <col><b>To see on</b></col>, <cd>to look at.</cd> <mark>[Obs.]</mark> <ldquo/She was full more blissful <xex>on to see</xex>.<rdquo/ <au>Chaucer.</au> -- <col><b>To see to</b></col>. <sd>(a)</sd> <cd>To look at; to behold; to view</cd>. <mark>[Obs.]</mark> <ldquo/An altar by Jordan, a great altar <xex>to see to</xex><rdquo/ <au>Josh. xxii. 10.</au> <sd>(b)</sd> <cd>To take care about; to look after; <as>as, <ex>to see to</ex> a fire</as>.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Seecatch</ent><br/
<hw>See"catch</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Russ. <ets>siekach</ets>.]</ety> <fld>(Zool.)</fld> <def>A full-grown male fur seal.</def> <mark>[Alaska]</mark><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Seed</ent><br/
<hw>Seed</hw> <pr>(s<emac/d)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Seed</plw> or <plw>Seeds</plw> <pr>(#)</pr>.</plu> <ety>[OE. <ets>seed</ets>, <ets>sed</ets>, AS. <ets>s<aemac/d</ets>, fr. <ets>s<amac/wan</ets> to sow; akin to D. <ets>zaad</ets> seed, G. <ets>saat</ets>, Icel. <ets>s<amac/<edh/</ets>, <ets>sae<edh/i</ets>, Goth. mana<ets>s<emac/<thorn/s</ets> seed of men, world. See <er>Sow</er> to scatter seed, and cf. <er>Colza</er>.]</ety> <sn>1.</sn> <fld>(Bot.)</fld> <sd>(a)</sd> <def>A ripened ovule, consisting of an embryo with one or more integuments, or coverings; <as>as, an apple <ex>seed</ex>; a currant <ex>seed</ex>.</as> By germination it produces a new plant.</def> <sd>(b)</sd> <def>Any small seedlike fruit, though it may consist of a pericarp, or even a calyx, as well as the seed proper; <as>as, parsnip <ex>seed</ex>; thistle <ex>seed</ex>.</as></def><br/
[<source>1913 Webster</source>]</p>

<p><q>And God said, Let the earth bring forth grass, the herb yielding <qex>seed</qex>, and the fruit tree yielding fruit after his kind, whose <qex>seed</qex> is in itself.</q> <rj><qau>Gen. i. 11.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ The seed proper has an outer and an inner coat, and within these the kernel or nucleus. The kernel is either the embryo alone, or the embryo inclosed in the albumen, which is the material for the nourishment of the developing embryo. The scar on a seed, left where the stem parted from it, is called the <xex>hilum</xex>, and the closed orifice of the ovule, the <xex>micropyle</xex>.</note><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Physiol.)</fld> <def>The generative fluid of the male; semen; sperm; -- not used in the plural.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>That from which anything springs; first principle; original; source; <as>as, the <ex>seeds</ex> of virtue or vice</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>The principle of production.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Praise of great acts he scatters as a <qex>seed</qex>,<br/
<qex>Which may the like in coming ages breed</qex>.</q> <rj><qau>Waller.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <def>Progeny; offspring; children; descendants; <as>as, the <ex>seed</ex> of Abraham; the <ex>seed</ex> of David</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ In this sense the word is applied to one person, or to any number collectively, and admits of the plural form, though rarely used in the plural.</note><br/
[<source>1913 Webster</source>]</p>

<p><sn>6.</sn> <def>Race; generation; birth.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Of mortal <qex>seed</qex> they were not held.</q> <rj><qau>Waller.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Seed bag</b></col> <fld>(Artesian well)</fld>, <cd>a packing to prevent percolation of water down the bore hole. It consists of a bag encircling the tubing and filled with flax seed, which swells when wet and fills the space between the tubing and the sides of the hole.</cd> -- <col><b>Seed bud</b></col> <fld>(Bot.)</fld>, <cd>the germ or rudiment of the plant in the embryo state; the ovule.</cd> -- <col><b>Seed coat</b></col> <fld>(Bot.)</fld>, <cd>the covering of a seed.</cd> -- <mcol><col><b>Seed corn</b></col>, <it>or</it> <col><b>Seed grain</b></col></mcol> <fld>(Bot.)</fld>, <cd>corn or grain for seed.</cd>  -- <col><b>To eat the seed corn</b></col>,  <cd>To eat the corn which should be saved for seed, so as to forestall starvation; -- a desparate measure, since it only postpones disaster.</cd> <specif>Hence:</specif> <cd>any desparate action which creates a disastrous situation in the long-term, done in order to provide temporary relief.</cd> -- <col><b>Seed down</b></col> <fld>(Bot.)</fld>, <cd>the soft hairs on certain seeds, as cotton seed.</cd> -- <col><b>Seed drill</b></col>.  <cd>See 6th <er>Drill</er>, 2 <sd>(a)</sd>.</cd> -- <col><b>Seed eater</b></col> <fld>(Zool.)</fld>, <cd>any finch of the genera <gen>Sporophila</gen>, and <gen>Crithagra</gen>. They feed mainly on seeds.</cd> -- <col><b>Seed gall</b></col> <fld>(Zool.)</fld>, <cd>any gall which resembles a seed, formed on the leaves of various plants, usually by some species of Phylloxera.</cd> -- <col><b>Seed leaf</b></col> <fld>(Bot.)</fld>, <cd>a cotyledon.</cd> -- <col><b>Seed lobe</b></col> <fld>(Bot.)</fld>, <cd>a cotyledon; a seed leaf.</cd> -- <col><b>Seed oil</b></col>, <cd>oil expressed from the seeds of plants.</cd> -- <col><b>Seed oyster</b></col>, <cd>a young oyster, especially when of a size suitable for transplantation to a new locality.</cd> -- <col><b>Seed pearl</b></col>, <cd>a small pearl of little value.</cd> -- <mcol><col><b>Seed plat</b></col>, <it>or</it> <col><b>Seed plot</b></col></mcol>, <cd>the ground on which seeds are sown, to produce plants for transplanting; a nursery.</cd> -- <col><b>Seed stalk</b></col> <fld>(Bot.)</fld>, <cd>the stalk of an ovule or seed; a funicle.</cd> -- <col><b>Seed tick</b></col> <fld>(Zool.)</fld>, <cd>one of several species of ticks resembling seeds in form and color.</cd> -- <col><b>Seed vessel</b></col> <fld>(Bot.)</fld>, <cd>that part of a plant which contains the seeds; a pericarp.</cd> -- <col><b>Seed weevil</b></col> <fld>(Zool.)</fld>, <cd>any one of numerous small weevils, especially those of the genus <gen>Apion</gen>, which live in the seeds of various plants.</cd> -- <col><b>Seed wool</b></col>, <cd>cotton wool not yet cleansed of its seeds.</cd> <mark>[Southern U.S.]</mark></cs><br/
[<source>1913 Webster</source> <source>+PJC</source>]</p>

<p><ent>Seed</ent><br/
<hw>Seed</hw> <pr>(?)</pr>, <pos>v. i.</pos> <sn>1.</sn> <def>To sow seed.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To shed the seed.</def> <rj><au>Mortimer.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>To grow to maturity, and to produce seed.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Many interests have grown up, and <qex>seeded</qex>, and twisted their roots in the crevices of many wrongs.</q> <rj><qau>Landor.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Seed</ent><br/
<hw>Seed</hw>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Seeded</conjf>; <pos>p. pr. & vb. n.</pos> <conjf>Seeding</conjf>.]</vmorph> <sn>1.</sn> <def>To sprinkle with seed; to plant seeds in; to sow; <as>as, to <ex>seed</ex> a field</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To cover thinly with something scattered; to ornament with seedlike decorations.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>A sable mantle <qex>seeded</qex> with waking eyes.</q> <rj><qau>B. Jonson.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>To seed down</b></col>, <cd>to sow with grass seed.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Seedbox</ent><br/
<hw>Seed"box`</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Bot.)</fld> <sd>(a)</sd> <def>A capsule.</def> <sd>(b)</sd> <def>A plant (<spn>Ludwigia alternifolia</spn>) which has somewhat cubical or box-shaped capsules.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Seedcake</ent><br/
<hw>Seed"cake`</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A sweet cake or cooky containing aromatic seeds, as caraway.</def>  <rj><au>Tusser.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Seedcod</ent><br/
<hw>Seed"cod`</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A seedlip.</def> <mark>[Prov. Eng.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Seeder</ent><br/
<hw>Seed"er</hw> <pr>(?)</pr>, <pos>n.</pos> <def>One who, or that which, sows or plants seed.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Seediness</ent><br/
<hw>Seed`i*ness</hw> <pr>(?)</pr>, <pos>n.</pos> <def>The quality or state of being seedy, shabby, or worn out; a state of wretchedness or exhaustion.</def> <mark>[Colloq.]</mark>  <rj><au>G. Eliot.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>What is called <qex>seediness</qex>, after a debauch, is a plain proof that nature has been outraged.</q> <rj><qau>J. S. Blackie.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Seed-lac</ent><br/
<hw>Seed"-lac`</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A species of lac. See the Note under <er>Lac</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Seedless</ent><br/
<hw>Seed"less</hw>, <pos>a.</pos> <def>Without seed or seeds.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Seedling</ent><br/
<hw>Seed"ling</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Bot.)</fld> <def>A plant reared from the seed, as distinguished from one propagated by layers, buds, or the like.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Seedlop</ent><br/
<ent>Seedlip</ent><br/
<mhw>{ <hw>Seed"lip`</hw> <pr>(?)</pr>, <hw>Seed"lop`</hw> <pr>(?)</pr>, }</mhw> <pos>n.</pos> <ety>[AS. <ets>s<aemac/dle<aacute/p</ets>; <ets>s<aemac/d</ets> seed + <ets>le<aacute/p</ets> basket.]</ety> <def>A vessel in which a sower carries the seed to be scattered.</def> <mark>[Prov. Eng.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Seedman</ent><br/
<hw>Seed"man</hw> <pr>(?)</pr>, <def>See <er>Seedsman</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Seedness</ent><br/
<hw>Seed"ness</hw>, <pos>n.</pos> <def>Seedtime.</def> <mark>[Obs.]</mark>  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Seedsman</ent><br/
<hw>Seeds"man</hw> <pr>(?)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Seedsmen</plw> <pr>(-m<eit/n)</pr>.</plu> <sn>1.</sn> <def>A sower; one who sows or scatters seed.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The <qex>seedsman</qex><br/
Upon the slime and ooze scatters his grain.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A person who deals in seeds.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Seedtime</ent><br/
<hw>Seed"time`</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[AS. <ets>s<aemac/dt<imac/ma</ets>.]</ety> <def>The season proper for sowing.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>While the earth remaineth, <qex>seedtime</qex> and harvest, and cold and heat, and summer and winter, and day and night, shall not cease.</q>  <rj><qau>Gen. viii. 22.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Seedy</ent><br/
<hw>Seed"y</hw> <pr>(?)</pr>, <pos>a.</pos> <amorph>[<pos>Compar.</pos> <adjf>Seedier</adjf> <pr>(?)</pr>; <pos>superl.</pos> <adjf>Seediest</adjf>.]</amorph> <sn>1.</sn> <def>Abounding with seeds; bearing seeds; having run to seeds.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Having a peculiar flavor supposed to be derived from the weeds growing among the vines; -- said of certain kinds of French brandy.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>Old and worn out; exhausted; spiritless; also, poor and miserable looking; shabbily clothed; shabby looking; <as>as, he looked <ex>seedy</ex>; a <ex>seedy</ex> coat</as>.</def> <mark>[Colloq.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>Little Flanigan here . . . is a little <qex>seedy</qex>, as we say among us that practice the law.</q>  <rj><au>Goldsmith.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Seedy toe</b></col>, <cd>an affection of a horse's foot, in which a cavity filled with horn powder is formed between the laminae and the wall of the hoof.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Seeing</ent><br/
<hw>See"ing</hw> <pr>(?)</pr>, <pos>conj.</pos> (but originally a <pos>present participle</pos>). <def>In view of the fact (that); considering; taking into account (that); insmuch as; since; because; -- followed by a dependent clause; <as>as, he did well, <ex>seeing</ex> that he was so young</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Wherefore come ye to me, <qex>seeing</qex> ye hate me?</q> <rj><qau>Gen. xxvi. 27.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Seek</ent><br/
<hw>Seek</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Sick.</def> <mark>[Obs.]</mark>  <rj><au>Chaucer.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Seek</ent><br/
<hw>Seek</hw>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Sought</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Seeking</conjf>.]</vmorph> <ety>[OE. <ets>seken</ets>, AS. <ets>s<emac/can</ets>, <ets>s<emac/cean</ets>; akin to OS. <ets>s<omac/kian</ets>, LG. <ets>s<oum/ken</ets>, D. <ets>zoeken</ets>, OHG. <ets>suohhan</ets>, G. <ets>suchen</ets>, Icel. <ets>saekja</ets>, Sw. <ets>s<oum/ka</ets>, Dan. <ets>s<oum/ge</ets>, Goth. <ets>s<omac/kjan</ets>, and E. <ets>sake</ets>.  Cf. <er>Beseech</er>, <er>Ransack</er>, <er>Sagacious</er>, <er>Sake</er>, <er>Soc</er>.]</ety> <sn>1.</sn> <def>To go in search of; to look for; to search for; to try to find.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The man saked him, saying, What <qex>seekest</qex> thou? And he said, I <qex>seek</qex> my brethren.</q> <rj><qau>Gen. xxxvii. 15, 16.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To inquire for; to ask for; to solicit; to beseech.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Others, tempting him, <qex>sought</qex> of him a sign.</q> <rj><qau>Luke xi. 16.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>To try to acquire or gain; to strive after; to aim at; <as>as, to <ex>seek</ex> wealth or fame; to <ex>seek</ex> one's life</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>To try to reach or come to; to go to; to resort to.</def><br/
[<source>1913 Webster</source>]</p>

<p><q><qex>Seek</qex> not Bethel, nor enter into Gilgal.</q> <rj><qau>Amos v. 5.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Since great Ulysses <qex>sought</qex> the Phrygian plains.</q> <rj><qau>Pope.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Seek</ent><br/
<hw>Seek</hw> <pr>(?)</pr>, <pos>v. i.</pos> <def>To make search or inquiry; to endeavor to make discovery.</def><br/
[<source>1913 Webster</source>]</p>

<p><q><qex>Seek</qex> ye out of the book of the Lord, and read.</q> <rj><qau>Isa. xxxiv. 16.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>To seek</b></col>, <cd>needing to seek or search; hence, unprepared.</cd> <ldquo/Unpracticed, unprepared, and still <xex>to seek</xex>.<rdquo/ <au>Milton.</au> <mark>[Obs.]</mark> -- <col><b>To seek after</b></col>, <cd>to make pursuit of; to attempt to find or take.</cd> -- <col><b>To seek for</b></col>, <cd>to endeavor to find.</cd> -- <col><b>To seek to</b></col>, <cd>to apply to; to resort to; to court.</cd> <mark>[Obs.]</mark> <ldquo/All the earth <xex>sought to</xex> Solomon, to hear his wisdom.<rdquo/ <au>1 Kings x. 24.</au> -- <col><b>To seek upon</b></col>, <cd>to make strict inquiry after; to follow up; to persecute.</cd> <mark>[Obs.]</mark></cs><br/
[<source>1913 Webster</source>]</p>

<p><q><qex>To seek</qex><br/
<qex>Upon</qex> a man and do his soul unrest.</q> <rj><qau>Chaucer.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Seeker</ent><br/
<hw>Seek"er</hw> <pr>(?)</pr>, <pos>n.</pos> <sn>1.</sn> <def>One who seeks; that which is used in seeking or searching.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Eccl.)</fld> <def>One of a small heterogeneous sect of the 17th century, in Great Britain, who professed to be seeking the true church, ministry, and sacraments.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>A skeptic [is] ever seeking and never finds, like our new upstart sect of <qex>Seekers</qex>.</q> <rj><qau>Bullokar.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Seek-no-further</ent><br/
<hw>Seek"-no-fur`ther</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A kind of choice winter apple, having a subacid taste; -- formerly called <altname>go-no-further</altname>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Seek-sorrow</ent><br/
<hw>Seek"-sor`row</hw> <pr>(?)</pr>, <pos>n.</pos> <def>One who contrives to give himself vexation.</def> <mark>[Archaic.]</mark>  <rj><au>Sir P. Sidney.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Seel</ent><br/
<hw>Seel</hw> <pr>(s<emac/l)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Seeled</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Seeling</conjf>.]</vmorph> <ety>[F. <ets>siller</ets>, <ets>ciller</ets>, fr. <ets>cil</ets> an eyelash, L. <ets>cilium</ets>.]</ety> <sn>1.</sn> <fld>(Falconry)</fld> <def>To close the eyes of (a hawk or other bird) by drawing through the lids threads which were fastened over the head.</def>  <rj><au>Bacon.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Fools climb to fall: fond hopes, like <qex>seeled</qex> doves for want of better light, mount till they end their flight with falling.</q> <rj><qau>J. Reading.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Hence, to shut or close, as the eyes; to blind.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Come, <qex>seeling</qex> night,<br/
Scarf up the tender eye of pitiful day.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Cold death, with a violent fate, his sable eyes did <qex>seel</qex>.</q> <rj><qau>Chapman.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Seel</ent><br/
<hw>Seel</hw>, <pos>v. i.</pos> <ety>[Cf. LG. <ets>sielen</ets> to lead off water, F. <ets>siller</ets> to run ahead, to make headway, E. <ets>sile</ets>, v.t.]</ety> <def>To incline to one side; to lean; to roll, as a ship at sea.</def> <mark>[Obs.]</mark>  <rj><au>Sir W. Raleigh.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Seeling</ent><br/
<ent>Seel</ent><br/
<mhw>{ <hw>Seel</hw> <pr>(?)</pr>, <hw>Seel"ing</hw>, }</mhw> <pos>n.</pos> <def>The rolling or agitation of a ship in a storm.</def> <mark>[Obs.]</mark>  <rj><au>Sandys.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Seel</ent><br/
<hw>Seel</hw>, <pos>n.</pos> <ety>[AS. <ets>s<aemac/l</ets>, from <ets>s<aemac/l</ets> good, prosperous. See <er>Silly</er>.]</ety> <sn>1.</sn> <def>Good fortune; favorable opportunity; prosperity. <mark>[Obs.]</mark> <ldquo/So have I <xex>seel</xex><rdquo/.</def>  <rj><au>Chaucer.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Time; season; <as>as, hay <ex>seel</ex></as>.</def> <mark>[Prov. Eng.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Seelily</ent><br/
<hw>Seel"i*ly</hw> <pr>(?)</pr>, <pos>adv.</pos> <def>In a silly manner.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Seely</ent><br/
<hw>Seel"y</hw> <pr>(?)</pr>, <pos>a.</pos> <def>See <er>Silly</er>.</def> <mark>[Obs.]</mark>  <rj><au>Spenser.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Seem</ent><br/
<hw>Seem</hw> <pr>(s<emac/m)</pr>, <pos>v. i.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Seemed</conjf> <pr>(s<emac/md)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Seeming</conjf>.]</vmorph> <ety>[OE. <ets>semen</ets> to seem, to become, befit, AS. <ets>s<emac/man</ets> to satisfy, pacify; akin to Icel. <ets>saema</ets> to honor, to bear with, conform to, <ets>saemr</ets> becoming, fit, <ets>s<omac/ma</ets> to beseem, to befit, <ets>sama</ets> to beseem, <ets>semja</ets> to arrange, settle, put right, Goth. <ets>samjan</ets> to please, and to E. <ets>same</ets>. The sense is probably due to the adj. <ets>seemly</ets>. <root/191. See <er>Same</er>, <pos>a.</pos>, and cf. <er>Seemly</er>.]</ety> <def>To appear, or to appear to be; to have a show or semblance; to present an appearance; to look; to strike one's apprehension or fancy as being; to be taken as.</def> <ldquo/It now <xex>seemed</xex> probable.<rdquo/  <rj><au>Macaulay.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Thou picture of what thou <qex>seem'st</qex>.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>All <qex>seemed</qex> well pleased; all <qex>seemed</qex>, but were not all.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>There is a way which <qex>seemeth</qex> right unto a man; but the end thereof are the ways of death.</q> <rj><qau>Prov. xiv. 12.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>It seems</b></col>, <cd>it appears; it is understood as true; it is said.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><q>A prince of Italy, <qex>it seems</qex>, entertained his mistress on a great lake.</q> <rj><qau>Addison.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><syn><b>Syn.</b> -- To appear; look.</syn>  -- <usage><er>Seem</er>, <er>Appear</er>. To <xex>appear</xex> has reference to a thing's being presented to our view; as, the sun <xex>appears</xex>; to <xex>seem</xex> is connected with the idea of <xex>semblance</xex>, and usually implies an inference of our mind as to the probability of a thing's being so; as, a storm <xex>seems</xex> to be coming. <ldquo/The story <xex>appears</xex> to be true,<rdquo/ means that the facts, as presented, go to show its truth; <ldquo/the story <xex>seems</xex> to be true,<rdquo/ means that it has the semblance of being so, and we infer that it is true. <ldquo/His first and principal care being to <xex>appear</xex> unto his people such as he would have them be, and to be such as he <xex>appeared</xex>.<rdquo/ <au>Sir P. Sidney.</au></usage><br/
[<source>1913 Webster</source>]</p>

<p><q><it>Ham.</it> Ay, madam, it is common.<br/
<it>Queen</it>.  If it be,<br/
Why <qex>seems</qex> it so particular with thee?<br/
<it>Ham.</it>  <qex>Seems</qex>, madam! Nay, it is; I know not <ldquo/<qex>seems</qex>.<rdquo/</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Seem</ent><br/
<hw>Seem</hw>, <pos>v. t.</pos> <def>To befit; to beseem.</def> <mark>[Obs.]</mark>  <rj><au>Spenser.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Seemer</ent><br/
<hw>Seem"er</hw> <pr>(?)</pr>, <pos>n.</pos> <def>One who seems; one who carries or assumes an appearance or semblance.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Hence shall we see,<br/
If power change purpose, what our <qex>seemers</qex> be.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Seeming</ent><br/
<hw>Seem"ing</hw>, <pos>a.</pos> <def>Having a semblance, whether with or without reality; apparent; specious; befitting; <as>as, <ex>seeming</ex> friendship; <ex>seeming</ex> truth</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>My lord, you have lost a friend indeed;<br/
And I dare swear you borrow not that face<br/
Of <qex>seeming</qex> sorrow, it is sure your own.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Seeming</ent><br/
<hw>Seem"ing</hw>, <pos>n.</pos> <sn>1.</sn> <def>Appearance; show; semblance; fair appearance; speciousness.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>These keep<br/
<qex>Seeming</qex> and savor all the winter long.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Apprehension; judgment.</def> <mark>[Obs.]</mark>  <rj><au>Chaucer.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Nothing more clear unto their <qex>seeming</qex>.</q> <rj><qau>Hooker.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>His persuasive words, impregned<br/
With reason, to her <qex>seeming</qex>.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Seemingly</ent><br/
<hw>Seem"ing*ly</hw>, <pos>adv.</pos> <def>In appearance; in show; in semblance; apparently; ostensibly.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>This the father <qex>seemingly</qex> complied with.</q> <rj><qau>Addison.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Seemingness</ent><br/
<hw>Seem"ing*ness</hw>, <pos>n.</pos> <def>Semblance; fair appearance; plausibility.</def>  <rj><au>Sir K. Digby.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Seemless</ent><br/
<hw>Seem"less</hw>, <pos>a.</pos> <def>Unseemly.</def> <mark>[Obs.]</mark>  <rj><au>Spenser.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Seemlily</ent><br/
<hw>Seem"li*ly</hw> <pr>(?)</pr>, <pos>adv.</pos> <def>In a seemly manner.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Seemliness</ent><br/
<hw>Seem"li*ness</hw>, <pos>n.</pos> <def>The quality or state of being seemly: comeliness; propriety.</def><br/
[<source>1913 Webster</source>]</p>

<p><-- p. 1304 --></p>

<p><ent>Seemly</ent><br/
<hw>Seem"ly</hw> <pr>(s<emac/m"l<ycr/)</pr>, <pos>a.</pos> <amorph>[<pos>Compar.</pos><adjf>Seemlier</adjf> <pr>(s<emac/m"l<icr/*<etil/r)</pr>; <pos>superl.</pos> <adjf>Seeliest</adjf>.]</amorph> <ety>[Icel. <ets>s<ae/miligr</ets>, fr. <ets>s<ae/mr</ets> becoming, fit; akin to <ets>samr</ets> same, E. <ets>same</ets>; the sense being properly, the same or like, hence, fitting. See <er>Seem</er>, <pos>v. i.</pos>]</ety> <def>Suited to the object, occasion, purpose, or character; suitable; fit; becoming; comely; decorous.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>He had a <qex>seemly</qex> nose.</q> <rj><qau>Chaucer.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>I am a woman, lacking wit<br/
To make a <qex>seemly</qex> answer to such persons.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Suspense of judgment and exercise of charity were safer and <qex>seemlier</qex> for Christian men than the hot pursuit of these controversies.</q> <rj><qau>Hooker.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><syn><b>Syn.</b> -- Becoming; fit; suitable; proper; appropriate; congruous; meet; decent; decorous.</syn><br/
[<source>1913 Webster</source>]</p>

<p><ent>Seemly</ent><br/
<hw>Seem"ly</hw> <pr>(s<emac/m"l<ycr/)</pr>, <pos>adv.</pos> <amorph>[<pos>Compar.</pos> <adjf>Seemlier</adjf>; <pos>superl.</pos> <adjf>Seemliest</adjf>.]</amorph> <def>In a decent or suitable manner; becomingly.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Suddenly a men before him stood,<br/
Not rustic as before, but <qex>seemlier</qex> clad,<br/
As one in city or court or place bred.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Seemlyhed</ent><br/
<hw>Seem"ly*hed</hw> <pr>(s<emac/m"l<ycr/*h<ecr/d)</pr>, <pos>n.</pos> <ety>[See <er>-hood</er>.]</ety> <def>Comely or decent appearance.</def> <mark>[Obs.]</mark>  <rj><au>Rom. of R. Spenser.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Seen</ent><br/
<hw>Seen</hw> <pr>(s<emac/n)</pr>, <def><pos>p. p.</pos> of <er>See</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Seen</ent><br/
<hw>Seen</hw>, <pos>a.</pos> <def>Versed; skilled; accomplished.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>Well <qex>seen</qex> in every science that mote be.</q> <rj><qau>Spenser.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Noble Boyle, not less in nature <qex>seen</qex>,<br/
Than his great brother read in states and men.</q> <rj><qau>Dryden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sipe</ent><br/
<ent>Seep</ent><br/
<mhw>{ <hw>Seep</hw> <pr>(?)</pr>, <it>or</it> <hw>Sipe</hw> <pr>(?)</pr>, }</mhw> <pos>v. i.</pos> <ety>[AS. <ets>s<imac/pan</ets> to distill.]</ety> <def>To run or soak through fine pores and interstices; to ooze.</def> <mark>[Scot. & U. S.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>Water <qex>seeps</qex> up through the sidewalks.</q> <rj><qau>G. W. Cable.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sipage</ent><br/
<ent>Seepage</ent><br/
<mhw>{ <hw>Seep"age</hw> <pr>(?)</pr>, <it>or</it>  <hw>Sip"age</hw>  }</mhw>, <pos>n.</pos> <def>Water that seeped or oozed through a porous soil.</def> <mark>[Scot. & U. S.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Seepage</ent><br/
<hw>Seep"age</hw>, <pos>n.</pos> <sn>1.</sn> <def>The act or process of seeping; percolation.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><sn>2.</sn> <def>a fluid that seeps out of a container; <as>as, <ex>seepage</ex> from a  reservoir</as>.</def><br/
[<source>PJC</source>]</p>

<p><ent>Sipy</ent><br/
<ent>Seepy</ent><br/
<mhw>{ <hw>Seep"y</hw> <pr>(?)</pr>, <it>or</it>  <hw>Sip"y</hw>  }</mhw>, <pos>a.</pos> <def>Oozy; -- applied to land under cultivation that is not well drained.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Seer</ent><br/
<hw>Seer</hw> <pr>(s<emac/r)</pr>, <pos>a.</pos> <def>Sore; painful.</def> <mark>[Prov. Eng.]</mark>  <rj><au>Ray.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Seer</ent><br/
<hw>Se"er</hw> <pr>(s<emac/"<etil/r)</pr>, <pos>n.</pos> <def>One who sees.</def>  <rj><au>Addison.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Seer</ent><br/
<hw>Seer</hw> <pr>(s<emac/r)</pr>, <pos>n.</pos> <ety>[From <er>See</er>.]</ety> <def>A person who foresees events; a prophet.</def>  <rj><au>Milton.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Seeress</ent><br/
<hw>Seer"ess</hw>, <pos>n.</pos> <def>A female seer; a prophetess.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Seerfish</ent><br/
<hw>Seer"fish`</hw> <pr>(s<emac/r"f<icr/sh)</pr>, <pos>n.</pos> <fld>(Zool.)</fld> <def>A scombroid food fish of Madeira (<spn>Cybium Commersonii</spn>).</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Seerhand</ent><br/
<hw>Seer"hand</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Etymol. uncertain.]</ety> <def>A kind of muslin of a texture between nainsook and mull.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Seership</ent><br/
<hw>Seer"ship</hw>, <pos>n.</pos> <def>The office or quality of a seer.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Seersucker</ent><br/
<hw>Seer"suck`er</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A light fabric, originally made in the East Indies, of silk and linen, usually having alternating stripes, and a slightly craped or puckered surface; also, a cotton fabric of similar appearance.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Seerwood</ent><br/
<hw>Seer"wood`</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[See <er>Sear</er>.]</ety> <def>Dry wood.</def> <altsp>[Written also <asp>searwood</asp>.]</altsp> <mark>[Obs.]</mark>  <rj><au>Dryden.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Seesaw</ent><br/
<hw>See"saw`</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Probably a reduplication of <ets>saw</ets>, to express the alternate motion to and fro, as in the act of sawing.]</ety> <sn>1.</sn> <def>A play among children in which they are seated upon the opposite ends of a plank which is balanced in the middle, and move alternately up and down.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A plank or board adjusted for this play.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>A vibratory or reciprocating motion.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>He has been arguing in a circle; there is thus a <qex>seesaw</qex> between the hypothesis and fact.</q> <rj><qau>Sir W. Hamilton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <fld>(Whist.)</fld> <def>Same as <er>Crossruff</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Seesaw</ent><br/
<hw>See"saw`</hw>, <pos>v. i.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Seesawad</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Seesawing</conjf>.]</vmorph> <def>To move with a reciprocating motion; to move backward and forward, or upward and downward.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Seesaw</ent><br/
<hw>See"saw`</hw>, <pos>v. t.</pos> <def>To cause to move backward and forward in seesaw fashion.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>He <qex>seesaws</qex> himself to and fro.</q> <rj><qau>Ld. Lytton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Seesaw</ent><br/
<hw>See"saw`</hw>, <pos>a.</pos> <def>Moving up and down, or to and fro; having a reciprocating motion.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Seet</ent><br/
<hw>Seet</hw> <pr>(?)</pr>, <mark>obs.</mark> <pos>imp.</pos> <mord>of <er>Sit</er></mord>. <def>Sate; sat.</def>  <rj><au>Chaucer.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Seeth</ent><br/
<hw>Seeth</hw> <pr>(?)</pr>, <mark>obs.</mark> <def><pos>imp.</pos> of <er>Seethe</er>.</def>  <rj><au>Chaucer.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Seethe</ent><br/
<hw>Seethe</hw> <pr>(?)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp.</pos> <conjf>Seethed</conjf> <pr>(?)</pr> (<conjf>Sod</conjf> <pr>(<?/)</pr>, <mark>obs.</mark>); <pos>p. p.</pos> <conjf>Seethed</conjf>, <conjf>Sodden</conjf> <pr>(<?/)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Seething</conjf>.]</vmorph> <ety>[OE. <ets>sethen</ets>, AS. <ets>se<oacute/<?/an</ets>; akin to D. <ets>sieden</ets>, OHG. <ets>siodan</ets>, G.  <ets>sieden</ets>, Icel. <ets>sj<?/<?/a</ets>, Sw. <ets>sjuda</ets>, Dan. <ets>syde</ets>, Goth. <ets>saubs</ets> a burnt offering.  Cf. <er>Sod</er>, <pos>n.</pos>, <er>Sodden</er>, <er>Suds</er>.]</ety> <def>To decoct or prepare for food in hot liquid; to boil; <as>as, to <ex>seethe</ex> flesh</as>.</def> <altsp>[Written also <asp>seeth</asp>.]</altsp><br/
[<source>1913 Webster</source>]</p>

<p><q>Set on the great pot, and <qex>seethe</qex> pottage for the sons of the prophets.</q> <rj><qau>2 Kings iv. 38.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Seethe</ent><br/
<hw>Seethe</hw>, <pos>v. i.</pos> <def>To be a state of ebullition or violent commotion; to be hot; to boil.</def>  <rj><au>1 Sam. ii. 13.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>A long Pointe, round which the Mississippi used to whirl, and <qex>seethe</qex>, and foam.</q> <rj><qau>G. W. Cable.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Seether</ent><br/
<hw>Seeth"er</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A pot for boiling things; a boiler.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Like burnished gold the little <qex>seether</qex> shone.</q> <rj><qau>Dryden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Seg</ent><br/
<hw>Seg</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[See <er>Sedge</er>.]</ety> <fld>(Bot.)</fld> <sn>1.</sn> <def>Sedge.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>The gladen, and other species of Iris.</def>  <rj><au>Prior.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Seg</ent><br/
<hw>Seg</hw>, <pos>n.</pos> <ety>[Probably from the root of L. <ets>secare</ets> to cut.]</ety> <def>A castrated bull.</def> <mark>[Prov. Eng. & Scot.]</mark>  <rj><au>Halliwell.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Segar</ent><br/
<hw>Se*gar"</hw> <pr>(?)</pr>, <pos>n.</pos> <def>See <er>Cigar</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Seggar</ent><br/
<hw>Seg"gar</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Prov. E. <ets>saggard</ets> a seggar, <ets>seggard</ets> a sort of riding surtout, contr. fr. <ets>safeguard</ets>.]</ety> <def>A case or holder made of fire clay, in which fine pottery is inclosed while baking in the kin.</def> <altsp>[Written also <asp>saggar</asp>, <asp>sagger</asp>, and <asp>segger</asp>.]</altsp>  <rj><au>Ure.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Segge</ent><br/
<hw>Segge</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Zool.)</fld> <def>The hedge sparrow.</def> <mark>[Prov. Eng.]</mark>  <rj><au>Halliwell.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Segment</ent><br/
<hw>Seg"ment</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>segmentum</ets>, fr. <ets>secare</ets> to cut, cut off: cf. F. <ets>segment</ets>. See <er>Saw</er> a cutting instrument.]</ety> <sn>1.</sn> <def>One of the parts into which any body naturally separates or is divided; a part divided or cut off; a section; a portion; <as>as, a <ex>segment</ex> of an orange; a <ex>segment</ex> of a compound or divided leaf.</as></def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Geom.)</fld> <def>A part cut off from a figure by a line or plane; especially, that part of a circle contained between a chord and an arc of that circle, or so much of the circle as is cut off by the chord; <as>as, the <ex>segment acb</ex> in the Illustration</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <fld>(Mach.)</fld> <sd>(a)</sd> <def>A piece in the form of the sector of a circle, or part of a ring; <as>as, the <ex>segment</ex> of a sectional fly wheel or flywheel rim</as>.</def> <sd>(b)</sd> <def>A segment gear.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <fld>(Biol.)</fld> <sd>(a)</sd> <def>One of the cells or division formed by segmentation, as in egg cleavage or in fissiparous cell formation.</def> <sd>(b)</sd> <def>One of the divisions, rings, or joints into which many animal bodies are divided; a somite; a metamere; a somatome.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Segment gear</b></col>, <cd>a piece for receiving or communicating reciprocating motion from or to a cogwheel, consisting of a sector of a circular gear, or ring, having cogs on the periphery, or face.</cd> -- <col><b>Segment of a line</b></col>, <cd>the part of a line contained between two points on it.</cd> -- <col><b>Segment of a sphere</b></col>, <cd>the part of a sphere cut off by a plane, or included between two parallel planes.</cd> -- <col><b>Ventral segment</b></col>. <fld>(Acoustics)</fld> <cd>See <er>Loor</er>, <pos>n.</pos>, 5.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Segment</ent><br/
<hw>Seg"ment</hw> <pr>(?)</pr>, <pos>v. i.</pos> <fld>(Biol.)</fld> <def>To divide or separate into parts in growth; to undergo segmentation, or cleavage, as in the segmentation of the ovum.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Segmental</ent><br/
<hw>Seg*men"tal</hw> <pr>(?)</pr>, <pos>a.</pos> <sn>1.</sn> <def>Relating to, or being, a segment.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Anat. & Zool.)</fld> <sd>(a)</sd> <def>Of or pertaining to the segments of animals; <as>as, a <ex>segmental</ex> duct; <ex>segmental</ex> papillae.</as></def> <sd>(b)</sd> <def>Of or pertaining to the segmental organs.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Segmental duct</b></col> <fld>(Anat.)</fld>, <cd>the primitive duct of the embryonic excretory organs which gives rise to the Wolffian duct and ureter; the pronephric duct.</cd> -- <col><b>Segmental organs</b></col>. <sd>(a)</sd> <fld>(Anat.)</fld> <cd>The embryonic excretory organs of vertebrates, consisting primarily of the segmental tubes and segmental ducts</cd>. <sd>(b)</sd> <fld>(Zool.)</fld> <cd>The tubular excretory organs, a pair of which often occur in each of several segments in annelids. They serve as renal organs, and often, also, as oviducts and sperm ducts. See <xex>Illust.</xex> under <er>Sipunculacea</er>.</cd> -- <col><b>Segmental tubes</b></col> <fld>(Anat.)</fld>, <cd>the tubes which primarily open into the segmental duct, some of which become the urinary tubules of the adult.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Segmentation</ent><br/
<hw>Seg`men*ta"tion</hw> <pr>(?)</pr>, <pos>n.</pos> <def>The act or process of dividing into segments; specifically <fld>(Biol.)</fld>, a self-division into segments as a result of growth; cell cleavage; cell multiplication; endogenous cell formation.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Segmentation cavity</b></col> <fld>(Biol.)</fld>, <cd>the cavity formed by the arrangement of the cells in segmentation or cleavage of the ovum; the cavity of the blastosphere. In the gastrula stage, the segmentation cavity in which the mesoblast is formed lies between the entoblast and ectoblast. See Illust. of <er>Invagination</er>.</cd> -- <col><b>Segmentation nucleus</b></col> <fld>(Biol.)</fld>, <cd>the body formed by fusion of the male and female pronucleus in an impregnated ovum. See the Note under <er>Pronucleus</er>.</cd> -- <mcol><col><b>Segmentation of the ovum</b></col>, <it>or</it> <col><b>Egg cleavage</b></col></mcol> <fld>(Biol.)</fld>, <cd>the process by which the embryos of all the higher plants and animals are derived from the germ cell.  In the simplest case, that of small ova destitute of food yolk, the ovum or egg divides into two similar halves or segments (blastomeres), each of these again divides into two, and so on, thus giving rise to a mass of cells (mulberry mass, or <xex>morula</xex>), all equal and similar, from the growth and development of which the future animal is to be formed.  This constitutes <stype>regular segmentation</stype>.  Quite frequently, however, the equality and regularity of cleavage is interfered with by the presence of food yolk, from which results <xex>unequal segmentation</xex>. See <er>Holoblastic</er>, <er>Meroblastic</er>, <er>Alecithal</er>, <er>Centrolecithal</er>, <er>Ectolecithal</er>, and <er>Ovum</er>.</cd> -- <col><b>Segmentation sphere</b></col> <fld>(Biol.)</fld>, <cd>the blastosphere, or morula. See <er>Morula</er>.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Segmented</ent><br/
<hw>Seg"ment*ed</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Divided into segments or joints; articulated.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Segnity</ent><br/
<ent>Segnitude</ent><br/
<mhw>{ <hw>Seg"ni*tude</hw> <pr>(?)</pr>, <hw>Seg"ni*ty</hw> <pr>(?)</pr>, }</mhw> <pos>n.</pos> <ety>[L. <ets>segnitas</ets>, fr. <ets>segnis</ets> slow, sluggish.]</ety> <def>Sluggishness; dullness; inactivity.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Segno</ent><br/
||<hw>Se"gno</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[It. See <er>Sign</er>.]</ety> <fld>(Mus.)</fld> <def>A sign. See <er>Al segno</er>, and <er>Dal segno</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sego</ent><br/
<hw>Se"go</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Bot.)</fld> <def>A liliaceous plant (<spn>Calochortus Nuttallii</spn>) of Western North America, and its edible bulb; -- so called by the Ute Indians and the Mormons.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Segregate</ent><br/
<hw>Seg"re*gate</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>segregatus</ets>, p. p. of <ets>segregare</ets> to separate; pref. <ets>se-</ets> aside + <ets>grex</ets>, <ets>gregis</ets>, a flock or herd. See <er>Gregarious</er>.]</ety> <sn>1.</sn> <def>Separate; select.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Bot.)</fld> <def>Separated from others of the same kind.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Segregate</ent><br/
<hw>Seg"re*gate</hw> <pr>(?)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Segregated</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Segregating</conjf>.]</vmorph> <def>To separate from others; to set apart.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>They are still <qex>segregated</qex>, Christians from Christians, under odious designations.</q> <rj><qau>I. Taylor.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Segregate</ent><br/
<hw>Seg"re*gate</hw>, <pos>v. i.</pos> <fld>(Geol.)</fld> <def>To separate from a mass, and collect together about centers or along lines of fracture, as in the process of crystallization or solidification.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Segregation</ent><br/
<hw>Seg`re*ga"tion</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>segregatio</ets>: cf. F. <ets>s<eacute/gr<eacute/gation</ets>.]</ety> <sn>1.</sn> <def>The act of segregating, or the state of being segregated; separation from others; a parting.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Geol.)</fld> <def>Separation from a mass, and gathering about centers or into cavities at hand through cohesive attraction or the crystallizing process.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Seiches</ent><br/
||<hw>Seiches</hw> <pr>(?)</pr>, <pos>n. pl.</pos> <ety>[F.]</ety> <fld>(Geol.)</fld> <def>Local oscillations in level observed in the case of some lakes, as Lake Geneva.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Seid</ent><br/
<hw>Seid</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Ar <ets>seyid</ets> prince.]</ety> <def>A descendant of Mohammed through his daughter Fatima and nephew Ali.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Seidlitz</ent><br/
<hw>Seid"litz</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Of or pertaining to Seidlitz, a village in Bohemia.</def> <altsp>[Written also <asp>Sedlitz</asp>.]</altsp><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Seidlitz powders</b></col>, <cd>effervescing salts, consisting of two separate powders, one of which contains forty grains of sodium bicarbonate mixed with two drachms of Rochell<?/ salt (tartrate of potassium and sodium) and the other contains thirty-five grains of tartaric acid. The powders are mixed in water, and drunk while effervescing, as a mild cathartic; -- so called from the resemblance to the natural water of Seidlitz. Called also <altname>Rochelle powders</altname>.</cd> -- <col><b>Seidlitz water</b></col>, <cd>a natural water from Seidlitz, containing magnesium, sodium, calcium, and potassium sulphates, with calcium carbonate and a little magnesium chloride. It is used as an aperient.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Seigh</ent><br/
<hw>Seigh</hw> <pr>(s<amac/)</pr>, <mark>obs.</mark> <mord><pos>imp. sing.</pos> of <er>See</er>.</mord> <def><er>Saw</er>.</def>  <rj><au>Chaucer.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Seigneurial</ent><br/
<hw>Seign*eu"ri*al</hw> <pr>(s<emac/n*<umac/"r<icr/*<ait/l)</pr>, <pos>a.</pos> <ety>[F., fr. <ets>seigneur</ets>. See <er>Seignior</er>.]</ety> <sn>1.</sn> <def>Of or pertaining to the lord of a manor; manorial.</def>  <rj><au>Sir W. Temple.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Vested with large powers; independent.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Seignior</ent><br/
<hw>Seign"ior</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[OF. <ets>seignor</ets>, F. <ets>seigneur</ets>, cf. It. <ets>signore</ets>, Sp. <ets>se<ntil/or</ets> from an objective case of L. <ets>senier</ets> elder. See <er>Senior</er>.]</ety> <sn>1.</sn> <def>A lord; the lord of a manor.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A title of honor or of address in the South of Europe, corresponding to <xex>Sir</xex> or <xex>Mr</xex>. in English.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Grand Seignior</b></col>, <cd>the sultan of Turkey.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Seigniorage</ent><br/
<hw>Seign"ior*age</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F. <ets>seigneuriage</ets>, OF. <ets>seignorage</ets>.]</ety> <sn>1.</sn> <def>Something claimed or taken by virtue of sovereign prerogative; specifically, a charge or toll deducted from bullion brought to a mint to be coined; the difference between the cost of a mass of bullion and the value as money of the pieces coined from it.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>If government, however, throws the expense of coinage, as is reasonable, upon the holders, by making a charge to cover the expense (which is done by giving back rather less in coin than has been received in bullion, and is called <ldquo/levying a <qex>seigniorage</qex><rdquo/), the coin will rise to the extent of the <qex>seigniorage</qex> above the value of the bullion.</q> <rj><qau>J. S. Mill.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A share of the receipts of a business taken in payment for the use of a right, as a copyright or a patent.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Seignioral</ent><br/
<hw>Seign"ior*al</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Of or pertaining to a seignior; seigneurial.</def> <ldquo/Kingly or <xex>seignioral</xex> patronage.<rdquo/  <rj><au>Burke.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Seignioralty</ent><br/
<hw>Seign"ior*al*ty</hw> <pr>(?)</pr>, <pos>n.</pos> <def>The territory or authority of a seignior, or lord.</def>  <rj><au>Milman.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Seigniorial</ent><br/
<hw>Seign*io"ri*al</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Same as <er>Seigneurial</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Seigniorize</ent><br/
<hw>Seign"ior*ize</hw> <pr>(?)</pr>, <pos>v. t.</pos> <def>To lord it over.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>As proud as he that <qex>seigniorizeth</qex> hell.</q> <rj><qau>Fairfax.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Seigniory</ent><br/
<hw>Seign"ior*y</hw> <pr>(?)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>-ies</plw> <pr>(#)</pr>.</plu> <ety>[OE. <ets>seignorie</ets>, OF. <ets>seigneurie</ets>, F. <ets>seigneurie</ets>; cf. It. <ets>signoria</ets>.]</ety> <sn>1.</sn> <def>The power or authority of a lord; dominion.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>O'Neal never had any <qex>seigniory</qex> over that country but what by encroachment he got upon the English.</q> <rj><qau>Spenser.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>The territory over which a lord holds jurisdiction; a manor.</def> <altsp>[Written also <asp>seigneury</asp>, and <asp>seignory</asp>.]</altsp><br/
[<source>1913 Webster</source>]</p>

<p><ent>Seine</ent><br/
<hw>Seine</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F. <ets>seine</ets>, or AS. <ets>segene</ets>, b<?/th fr. L. <ets>sagena</ets>, Gr. <?/<?/<?/<?/.]</ety> <fld>(Fishing.)</fld> <def>A large net, one edge of which is provided with sinkers, and the other with floats. It hangs vertically in the water, and when its ends are brought together or drawn ashore incloses the fish.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Seine boat</b></col>, <cd>a boat specially constructed to carry and pay out a seine.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Seiner</ent><br/
<hw>Sein"er</hw> <pr>(?)</pr>, <pos>n.</pos> <def>One who fishes with a seine.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Seining</ent><br/
<hw>Sein"ing</hw>, <pos>n.</pos> <def>Fishing with a seine.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Seint</ent><br/
<hw>Seint</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[See <er>Cincture</er>.]</ety> <def>A girdle.</def> <mark>[Obs.]</mark> <ldquo/Girt with a <xex>seint</xex> of silk.<rdquo/  <rj><au>Chaucer.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Seint</ent><br/
<hw>Seint</hw>, <pos>n.</pos> <def>A saint.</def> <mark>[Obs.]</mark>  <rj><au>Chaucer.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Seintuary</ent><br/
<hw>Sein"tu*a*ry</hw> <pr>(?)</pr>, <pos>n.</pos> <def>Sanctuary.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Seirfish</ent><br/
<hw>Seir"fish`</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Zool.)</fld> <def>Same as <er>Seerfish</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Seirospore</ent><br/
<hw>Sei"ro*spore</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Gr. <?/<?/<?/ a cord + E. <ets>spore</ets>.]</ety> <fld>(Bot.)</fld> <def>One of several spores arranged in a chain as in certain algae of the genus <gen>Callithamnion</gen>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Seise</ent><br/
<hw>Seise</hw> <pr>(?)</pr>, <pos>v. t.</pos> <def>See <er>Seize</er>.</def>  <rj><au>Spenser.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ This is the common spelling in the law phrase <xex>to be seised of</xex> (an estate).</note><br/
[<source>1913 Webster</source>]</p>

<p><ent>Seisin</ent><br/
<hw>Sei"sin</hw> <pr>(?)</pr>, <pos>n.</pos> <def>See <er>Seizin</er>.</def>  <rj><au>Spenser.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Seismal</ent><br/
<ent>Seismic</ent><br/
<mhw>{ <hw>Seis"mic</hw> <pr>(?)</pr>, <hw>Seis"mal</hw> <pr>(?)</pr>, }</mhw> <pos>a.</pos> <ety>[Gr. <grk>seismo`s</grk> an earthquake, from <grk>sei`ein</grk> to shake.]</ety> <def>Of or pertaining to an earthquake; caused by an earthquake.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Seismic vertical</b></col>, <cd>the point upon the earth's surface vertically over the center of effort or focal point whence the earthquake's impulse proceeds, or the vertical line connecting these two points.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Seismogram</ent><br/
<hw>Seis"mo*gram</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Gr. <?/ earthquake + <ets>-gram</ets>.]</ety> <fld>(Physics)</fld> <def>The trace or record of an earth tremor, made by means of a seismograph.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Seismograph</ent><br/
<hw>Seis"mo*graph</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Gr. <?/<?/<?/ an earthquake + <ets>-graph</ets>.]</ety> <fld>(Physics)</fld> <def>An apparatus for registering the shocks and undulatory motions of earthquakes.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Seismographic</ent><br/
<hw>Seis`mo*graph"ic</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Of or pertaining to a seismograph; indicated by a seismograph.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Seismography</ent><br/
<hw>Seis*mog"ra*phy</hw> <pr>(?)</pr>, <pos>n.</pos> <sn>1.</sn> <def>A writing about, or a description of, earthquakes.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>The art of registering the shocks and undulatory movements of earthquakes.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Seismological</ent><br/
<hw>Seis`mo*log"ic*al</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Of or pertaining to seismology.</def> -- <wordforms><wf>Seis`mo*log"ic*al*ly</wf>, <pos>adv.</pos></wordforms><br/
[<source>1913 Webster</source>]</p>

<p><ent>Seismology</ent><br/
<hw>Seis*mol"o*gy</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Gr. <?/<?/<?/ an earthquake + <ets>-logy</ets>.]</ety> <def>The science of earthquakes.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Seismometer</ent><br/
<hw>Seis*mom"e*ter</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Gr. <?/<?/<?/ an earthquake + <ets>-meter</ets>.]</ety> <fld>(Physics)</fld> <def>An instrument for measuring the direction, duration, and force of earthquakes and like concussions.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Seismometric</ent><br/
<hw>Seis`mo*met"ric</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Of or pertaining to seismometry, or seismometer; <as>as, <ex>seismometric</ex> instruments; <ex>seismometric</ex> measurements</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Seismometry</ent><br/
<hw>Seis*mom"e*try</hw> <pr>(?)</pr>, <pos>n.</pos> <def>The mensuration of such phenomena of earthquakes as can be expressed in numbers, or by their relation to the coordinates of space.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Seismoscope</ent><br/
<hw>Seis"mo*scope</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Gr. <?/<?/<?/ an earthquake + <ets>-scope</ets>.]</ety> <fld>(Physics)</fld> <def>A seismometer.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Seity</ent><br/
<hw>Se"i*ty</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>se</ets> one's self.]</ety> <def>Something peculiar to one's self.</def> <mark>[R.]</mark>  <rj><au>Tatler.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Seizable</ent><br/
<hw>Seiz"a*ble</hw> <pr>(?)</pr>, <pos>a.</pos> <def>That may be seized.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Seize</ent><br/
<hw>Seize</hw> <pr>(?)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Seized</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Seizing</conjf>.]</vmorph> <ety>[OE. <ets>seisen</ets>, <ets>saisen</ets>, OF. <ets>seisir</ets>, <ets>saisir</ets>, F. <ets>saisir</ets>, of Teutonic origin, and akin to E. <ets>set</ets>. The meaning is properly, to set, put, place, hence, to put in possession of. See <er>Set</er>, <pos>v. t.</pos>]</ety> <sn>1.</sn> <def>To fall or rush upon suddenly and lay hold of; to gripe or grasp suddenly; to reach and grasp.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>For by no means the high bank he could <qex>seize</qex>.</q> <rj><qau>Spenser.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Seek you to <qex>seize</qex> and gripe into your hands<br/
The royalties and rights of banished Hereford?</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><-- p. 1305 --></p>

<p><sn>2.</sn> <def>To take possession of by force.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>At last they <qex>seize</qex><br/
The scepter, and regard not David's sons.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>To invade suddenly; to take sudden hold of; to come upon suddenly; <as>as, a fever <ex>seizes</ex> a patient</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Hope and deubt alternate <qex>seize</qex> her seul.</q> <rj><qau>Pope.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <fld>(law)</fld> <def>To take possession of by virtue of a warrant or other legal authority; <as>as, the sheriff <ex>seized</ex> the debtor's goods</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <def>To fasten; to fix.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>As when a bear hath <qex>seized</qex> her cruel claws<br/
Upon the carcass of some beast too weak.</q> <rj><qau>Spenser.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>6.</sn> <def>To grap with the mind; to comprehend fully and distinctly; <as>as, to <ex>seize</ex> an idea</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>7.</sn> <fld>(Naut.)</fld> <def>To bind or fasten together with a lashing of small stuff, as yarn or marline; <as>as, to <ex>seize</ex> ropes</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ This word, by writers on law, is commonly written <xex>seise</xex>, in the phrase <xex>to be seised of</xex> (an estate), as also, in composition, <xex>disseise</xex>, <xex>disseisin</xex>.</note><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>To be seized of</b></col>, <cd>to have possession, or right of possession; <as>as, A B was <ex>seized</ex> and possessed <ex>of</ex> the manor of Dale.</as></cd> <ldquo/Whom age might <xex>see seized of</xex> what youth made prize.<rdquo/ <au>Chapman.</au> -- <mcol><col><b>To seize on</b></col> <it>or</it> <col><b>To seize upon</b></col></mcol>, <cd>to fall on and grasp; to take hold on; to take possession of suddenly and forcibly.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><syn><b>Syn.</b> -- To catch; grasp; clutch; snatch; apprehend; arrest; take; capture.</syn><br/
[<source>1913 Webster</source>]</p>

<p><ent>Seizer</ent><br/
<hw>Seiz"er</hw> <pr>(?)</pr>, <pos>n.</pos> <def>One who, or that which, seizes.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Seizin</ent><br/
<hw>Sei"zin</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F. <ets>saisine</ets>. See <er>Seize</er>.]</ety> <sn>1.</sn> <fld>(Law)</fld> <def>Possession; possession of an estate of froehold. It may be either in <xex>deed</xex> or in <xex>law</xex>; the former when there is actual possession, the latter when there is a right to such possession by construction of law. In some of the United States <xex>seizin</xex> means merely <xex>ownership</xex>.</def>  <rj><au>Burrill.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>The act of taking possession.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>The thing possessed; property.</def>  <rj><au>Sir M. Halle.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ Commonly spelt by writers on law <xex>seisin</xex>.</note><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Livery of seizin</b></col>. <fld>(Eng. Law)</fld> <cd>See Note under <er>Livery</er>, 1.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Seizing</ent><br/
<hw>Seiz"ing</hw> <pr>(?)</pr>, <pos>n.</pos> <sn>1.</sn> <def>The act of taking or grasping suddenly.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Naut.)</fld> <sd>(a)</sd> <def>The operation of fastening together or lashing.</def> <sd>(b)</sd> <def>The cord or lashing used for such fastening.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Seizor</ent><br/
<hw>Sei"zor</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Law)</fld> <def>One who seizes, or takes possession.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Seizure</ent><br/
<hw>Sei"zure</hw> <pr>(?)</pr>, <pos>n.</pos> <sn>1.</sn> <def>The act of seizing, or the state of being seized; sudden and violent grasp or gripe; a taking into possession; <as>as, the <ex>seizure</ex> of a thief, a property, a throne, etc.</as></def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Retention within one's grasp or power; hold; possession; ownership.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Make o'er thy honor by a deed of trust,<br/
And give me <qex>seizure</qex> of the mighty wealth.</q> <rj><qau>Dryden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>That which is seized, or taken possession of; a thing laid hold of, or possessed.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sejeant</ent><br/
<ent>Sejant</ent><br/
<mhw>{ <hw>Se"jant</hw>, <hw>Se"jeant</hw>  }</mhw> <pr>(?)</pr>, <pos>a.</pos> <ety>[F. <ets>s<eacute/ant</ets>, p. pr. of <ets>seoir</ets> to sit, L. <ets>sedere</ets>.]</ety> <fld>(Her.)</fld> <def>Sitting, as a lion or other beast.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Sejant rampant</b></col>, <cd>sitting with the forefeet lifted up.</cd>  <rj><au>Wright.</au></rj></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sejein</ent><br/
<hw>Se*jein"</hw> <pr>(?)</pr>, <pos>v. t.</pos> <ety>[L. <ets>sejungere</ets>; pref. <ets>se-</ets> aside + <ets>jungere</ets> to join. See <er>Join</er>.]</ety> <def>To separate.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sejunction</ent><br/
<hw>Se*junc"tion</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>sejunctio</ets>. See <er>Sejoin</er>.]</ety> <def>The act of disjoining, or the state of being disjoined.</def> <mark>[Obs.]</mark>  <rj><au>Bp. Pearson.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sejungible</ent><br/
<hw>Se*jun"gi*ble</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[See <er>Sejoin</er>.]</ety> <def>Capable of being disjoined.</def> <mark>[Obs.]</mark>  <rj><au>Bp. Pearson.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Seke</ent><br/
<hw>Seke</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Sick.</def> <mark>[Obs.]</mark>  <rj><au>Chaucer.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Seke</ent><br/
<hw>Seke</hw> <pr>(?)</pr>, <pos>v. t. & i.</pos> <def>To seek.</def> <mark>[Obs.]</mark>  <rj><au>Chaucer.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sekes</ent><br/
||<hw>Se"kes</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[NL., fr. Gr. <?/<?/<?/ a pen, a sacred inclosure, a shrine.]</ety> <fld>(Arch.)</fld> <def>A place in a pagan temple in which the images of the deities were inclosed.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Selachian</ent><br/
<hw>Se*la"chi*an</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Zool.)</fld> <def>One of the Selachii. See <xex>Illustration</xex> in Appendix.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Selachii</ent><br/
||<hw>Se*la"chi*i</hw> <pr>(?)</pr>, <pos>n. pl.</pos> <ety>[NL., fr. Gr. <?/<?/<?/ a fish having cartilages instead of bones.]</ety> <fld>(Zool.)</fld> <def>An order of elasmobranchs including the sharks and rays; the Plagiostomi. Called also <altname>Selacha</altname>, <altname>Selache</altname>, and <altname>Selachoidei</altname>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Selachoidei</ent><br/
||<hw>Sel`a*choi"de*i</hw> <pr>(?)</pr>, <pos>n. pl.</pos> <ety>[NL. See <er>Selachii</er>, and <er>-oid</er>.]</ety> <fld>(Zool.)</fld> <def>Same as <er>Selachii</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Selachostomi</ent><br/
||<hw>Sel`a*chos"to*mi</hw> <pr>(?)</pr>, <pos>n. pl.</pos> <ety>[NL. See <er>Selachii</er>, and <er>Stoma</er>.]</ety> <fld>(Zool.)</fld> <def>A division of ganoid fishes which includes the paddlefish, in which the mouth is armed with small teeth.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Selaginella</ent><br/
||<hw>Sel`a*gi*nel"la</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[NL., fr. L. <ets>selago</ets>, <ets>-inis</ets>, a kind of plant.]</ety> <fld>(Bot.)</fld> <def>A genus of cryptogamous plants resembling Lycopodia, but producing two kinds of spores; also, any plant of this genus. Many species are cultivated in conservatories.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Selah</ent><br/
<hw>Se"lah</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Heb. <ets>sel<amac/h</ets>.]</ety> <fld>(Script.)</fld> <def>A word of doubtful meaning, occuring frequently in the Psalms; by some, supposed to signify silence or a pause in the musical performance of the song.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Beyond the fact that <qex>Selah</qex> is a musical term, we know absolutely nothing about it.</q> <rj><qau>Dr. W. Smith (Bib. Dict.)</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Selcouth</ent><br/
<hw>Sel"couth</hw> <pr>(s<ecr/l"k<oomac/th)</pr>, <pos>a.</pos> <ety>[AS. <ets>selc<umac/<edh/</ets>, <ets>seldc<umac/<edh/</ets>; <ets>seld</ets> rare + <ets>c<umac/<edh/</ets> known. See <er>Uncouth</er>.]</ety> <def>Rarely known; unusual; strange.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>[She] wondered much at his so <qex>selcouth</qex> case.</q> <rj><qau>Spenser.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Seld</ent><br/
<hw>Seld</hw> <pr>(s<ecr/ld)</pr>, <pos>a.</pos> <ety>[See <er>Seldom</er>.]</ety> <def>Rare; uncommon; unusual.</def> <mark>[Obs.]</mark>  <rj><au>Chaucer. Spenser.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Seld</ent><br/
<hw>Seld</hw>, <pos>adv.</pos> <def>Rarely; seldom.</def> <mark>[Obs.]</mark>  <rj><au>Chaucer.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Selden</ent><br/
<hw>Sel"den</hw> <pr>(s<ecr/l"d<eit/n)</pr>, <pos>adv.</pos> <def>Seldom.</def> <mark>[Obs.]</mark>  <rj><au>Chaucer.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Seldom</ent><br/
<hw>Sel"dom</hw> <pr>(s<ecr/l"d<ucr/m)</pr>, <pos>adv.</pos> <amorph>[Usually, <pos>Compar.</pos> <adjf>More seldom</adjf> <pr>(m<omac/r" s<ecr/l"d<ucr/m)</pr>; <pos>superl.</pos> <adjf>Most seldom</adjf> <pr>(m<omac/st" s<ecr/l"d<ucr/m)</pr>; but sometimes also, <adjf>Seldomer</adjf> <pr>(s<ecr/l"d<ucr/m*<etil/r)</pr>, <adjf>Seldomest</adjf>.]</amorph> <ety>[AS. <ets>seldan</ets>, <ets>seldon</ets>, <ets>seldum</ets>, fr. <ets>seld</ets> rare; akin to OFries. <ets>sielden</ets>, D. <ets>zelden</ets>, G. <ets>selten</ets>, OHG. <ets>seltan</ets>, Icel. <ets>sjaldan</ets>, Dan. <ets>sielden</ets>, Sw. <ets>s<aum/llan</ets>, Goth. <ets>sildaleiks</ets> marvelous.]</ety> <def>Rarely; not often; not frequently.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Wisdom and youth are <qex>seldom</qex> joined in one.</q> <rj><qau>Hooker.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Seldom</ent><br/
<hw>Sel"dom</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Rare; infrequent.</def> <mark>[Archaic.]</mark> <ldquo/A suppressed and <xex>seldom</xex> anger.<rdquo/  <rj><au>Jer. Taylor.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Seldomness</ent><br/
<hw>Sel"dom*ness</hw>, <pos>n.</pos> <def>Rareness.</def>  <rj><au>Hooker.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Seldseen</ent><br/
<hw>Seld"seen`</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[AS. <ets>seldsiene</ets>.]</ety> <def>Seldom seen.</def> <mark>[Obs.]</mark>  <rj><au>Drayton.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Seldshewn</ent><br/
<hw>Seld"shewn`</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[<ets>Seld</ets> + <ets>shown</ets>.]</ety> <def>Rarely shown or exhibited.</def> <mark>[Obs.]</mark>  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Select</ent><br/
<hw>Se*lect"</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>selectus</ets>, p. p. of <ets>seligere</ets> to select; pref. <ets>se-</ets> aside + <ets>levere</ets> to gather. See <er>Legend</er>.]</ety> <def>Taken from a number by preferance; picked out as more valuable or exellent than others; of special value or exellence; nicely chosen; selected; choice.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>A few <qex>select</qex> spirits had separated from the crowd, and formed a fit audience round a far greater teacher.</q> <rj><qau>Macaulay.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Select</ent><br/
<hw>Se*lect"</hw>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Selected</conjf>; <pos>p. pr. & vb. n.</pos> <conjf>Selecting</conjf>.]</vmorph> <def>To choose and take from a number; to take by preference from among others; to pick out; to cull; <as>as, to <ex>select</ex> the best authors for perusal</as>.</def> <ldquo/One peculiar nation to <xex>select</xex>.<rdquo/  <rj><au>Milton.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>The pious chief . . . <br/
A hundred youths from all his train <qex>selects</qex>.</q> <rj><qau>Dryden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Selectedly</ent><br/
<hw>Se*lect"ed*ly</hw>, <pos>adv.</pos> <def>With care and selection.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Selection</ent><br/
<hw>Se*lec"tion</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>selectio</ets>: cf. F. <ets>s<eacute/lection</ets>.]</ety> . <def>The act of selecting, or the state of being selected; choice, by preference.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>That which is selected; a collection of things chosen; <as>as, a choice <ex>selection</ex> of books</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Natural selection</b></col>. <fld>(Biol.)</fld> <cd>See under <er>Natural</er>.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Selective</ent><br/
<hw>Se*lect"ive</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Selecting; tending to select.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>This <qex>selective</qex> providence of the Almighty.</q> <rj><qau>Bp. Hall.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Selectman</ent><br/
<hw>Se*lect"man</hw> <pr>(?)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Selectmen</plw> <pr>(<?/)</pr>.</plu> <def>One of a board of town officers chosen annually in the New England States to transact the general public business of the town, and have a kind of executive authority. The number is usually from three to seven in each town.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The system of delegated town action was then, perhaps, the same which was defined in an <ldquo/order made in 1635 by the inhabitants of Charlestown at a full meeting for the government of the town, by <qex>selectmen</qex>;<rdquo/ the name presently extended throughout New England to municipal governors.</q> <rj><qau>Palfrey.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Selectness</ent><br/
<hw>Se*lect"ness</hw>, <pos>n.</pos> <def>The quality or state of being select.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Selector</ent><br/
<hw>Se*lect"or</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L.]</ety> <def>One who selects.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Selenate</ent><br/
<hw>Sel"e*nate</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Chem.)</fld> <def>A salt of selenic acid; -- formerly called also <altname>seleniate</altname>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Selenhydric</ent><br/
<hw>Sel`en*hy"dric</hw> <pr>(?)</pr>, <pos>a.</pos> <fld>(Chem.)</fld> <def>Of, pertaining to, or designating, hydrogen selenide, <chform>H2Se</chform>, regarded as an acid analogous to sulphydric acid.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Selenic</ent><br/
<hw>Se*len"ic</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[Cf. F. <ets>s<eacute/l<eacute/nique</ets>.]</ety> <fld>(Chem.)</fld> <def>Of or pertaining to selenium; derived from, or containing, selenium; specifically, designating those compounds in which the element has a higher valence as contrasted with <xex>selenious</xex> compounds.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Selenide</ent><br/
<hw>Sel"e*nide</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Chem.)</fld> <def>A binary compound of selenium, or a compound regarded as binary; <as>as, ethyl <ex>selenide</ex></as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Seleniferous</ent><br/
<hw>Sel`e*nif"er*ous</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[<ets>Selenium</ets> + <ets>-ferous</ets>. ]</ety> <def>Containing, or impregnated with, selenium; <as>as, <ex>seleniferous</ex> pyrites</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Selenio-</ent><br/
<hw>Se*le"ni*o-</hw> <pr>(<?/)</pr>. <fld>(Chem.)</fld> <def>A combining form (also used adjectively) denoting the <sig>presence of selenium</sig> or <xex>its compounds</xex>; <as>as, <ex>selenio</ex>-phosphate, a phosphate having selenium in place of all, or a part, of the oxygen</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Selenious</ent><br/
<hw>Se*le"ni*ous</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[Cf. F. <ets>s<eacute/l<eacute/nieux</ets>.]</ety> <fld>(Chem.)</fld> <def>Of, pertaining to, or containing, selenium; specifically, designating those compounds in which the element has a lower valence as contrasted with <contr>selenic</contr> compounds.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Selenite</ent><br/
<hw>Sel"e*nite</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Chem.)</fld> <def>A salt of selenious acid.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Selenite</ent><br/
<hw>Sel"e*nite</hw>, <pos>n.</pos> <ety>[L. <ets>selenites</ets>, Gr. <?/<?/<?/<?/ (sc. <?/<?/<?/), from <grk>selh`nh</grk> the moon. So called from a fancied resemblance in luster or appearance to the moon.]</ety> <fld>(Min.)</fld> <def>A variety of gypsum, occuring in transparent crystals or crystalline masses.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Selenitical</ent><br/
<ent>Selenitic</ent><br/
<mhw>{ <hw>Sel`e*nit"ic</hw> <pr>(?)</pr>, <hw>Sel`e*nit"ic*al</hw> <pr>(?)</pr>, }</mhw> <pos>a.</pos> <fld>(Min.)</fld> <def>Of or pertaining to selenite; resembling or containing selenite.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Selenium</ent><br/
<hw>Se*le"ni*um</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[NL., from Gr. <grk>selh`nh</grk> the moon. So called because of its chemical analogy to <ets>tellurium</ets> (from L. <ets>tellus</ets> the earth), being, as it were, a companion to it.]</ety> <fld>(Chem.)</fld> <def>A nonmetallic element of the sulphur group of atomic number 34, analogous to sulphur in its compounds. It is found in small quantities with sulphur and some sulphur ores, and obtained in the free state as a dark reddish powder or crystalline mass, or as a dark metallic-looking substance. It exhibits under the action of light a remarkable variation in electric conductivity, and is used in certain electric apparatus. Symbol Se. Atomic weight 78.96.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Seleniuret</ent><br/
<hw>Sel`e*ni"u*ret</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Chem.)</fld> <def>A selenide.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Seleniureted</ent><br/
<hw>Sel`e*ni"u*ret`ed</hw> <pr>(?)</pr>, <pos>a.</pos> <fld>(Chem.)</fld> <def>Combined with selenium as in a selenide; <as>as, <ex>seleniureted</ex> hydrogen</as>.</def> <altsp>[Written also <asp>seleniuretted</asp>.]</altsp> <mark>[Obsoles.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Selenocentric</ent><br/
<hw>Se*le`no*cen"tric</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[Gr. <grk>selh`nh</grk> the moon + E. <ets>centric</ets>.]</ety> <fld>(Astron.)</fld> <def>As seen or estimated from the center of the moon; with the moon central.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Selenograph</ent><br/
<hw>Se*le"no*graph</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A picture or delineation of the moon's surface, or of any part of it.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Selenographer</ent><br/
<hw>Sel`e*nog"ra*pher</hw> <pr>(?)</pr>, <pos>n.</pos> <def>One skilled in selenography.</def>  <rj><au>Wright.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Selenographical</ent><br/
<ent>Selenographic</ent><br/
<mhw>{ <hw>Sel`e*no*graph"ic</hw> <pr>(?)</pr>, <hw>Sel`e*no*graph"i*cal</hw> <pr>(?)</pr>, }</mhw> <pos>a.</pos> <ety>[Cf. F. <ets>s<eacute/l<eacute/nographique</ets>.]</ety> <def>Of or pertaining to selenography.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Selenographist</ent><br/
<hw>Sel`e*nog"ra*phist</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A selenographer.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Selenography</ent><br/
<hw>Sel`e*nog"ra*phy</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Gr. <grk>selh`nh</grk> the moon + <ets>-graphy</ets>.]</ety> <def>The science that treats of the physical features of the moon; -- corresponding to <xex>physical geography</xex> in respect to the earth.</def> <ldquo/Accurate <xex>selenography</xex>, or description of the moon.<rdquo/  <rj><au>Sir T. Browne.  </au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Selenology</ent><br/
<hw>Sel`e*nol"o*gy</hw>, <pos>n.</pos> <ety>[Gr. <grk>selh`nh</grk> the moonn + <ets>-logy</ets>.]</ety> <def>That branch of astronomy which treats of the moon.</def> -- <wordforms><wf>Sel`e*no*log"i*cal</wf> <pr>(#)</pr>, <pos>a.</pos></wordforms><br/
[<source>1913 Webster</source>]</p>

<p><ent>Selenonium</ent><br/
<hw>Sel`e*no"ni*um</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[<ets>Selen</ets>ium + sulph<ets>onium</ets>.]</ety> <fld>(Chem.)</fld> <def>A hypothetical radical of selenium, analogous to sulphonium.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Self</ent><br/
<hw>Self</hw> <pr>(s<ecr/lf)</pr>, <pos>a.</pos> <ety>[AS. <ets>self</ets>, <ets>seolf</ets>, <ets>sylf</ets>; akin to OS. <ets>self</ets>, OFries. <ets>self</ets>, D. <ets>zelf</ets>, G. <ets>selb</ets>, <ets>selber</ets>, <ets>selbst</ets>, Dan. <ets>selv</ets>. Sw. <ets>sjelf</ets>, Icel. <ets>sj<amac/lfr</ets>, Goth. <ets>silba</ets>.  Cf. <er>Selvage</er>.]</ety> <sn>1.</sn> <def>Same; particular; very; identical.</def> <mark>[Obs., except in the compound selfsame.]</mark> <ldquo/On these <xex>self</xex> hills.<rdquo/  <rj><au>Sir. W. Raleigh.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>To shoot another arrow that <qex>self</qex> way<br/
Which you did shoot the first.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>At that <qex>self</qex> moment enters Palamon.</q> <rj><qau>Dryden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Having its own or a single nature or character, as in color, composition, etc., without addition or change; unmixed; <as>as, a <ex>self</ex> bow, one made from a single piece of wood; <ex>self</ex> flower or plant, one which is wholly of one color; <ex>self</ex>-colored</as>.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Self</ent><br/
<hw>Self</hw>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Selves</plw> <pr>(<?/)</pr>.</plu> <sn>1.</sn> <def>The individual as the object of his own reflective consciousness; the man viewed by his own cognition as the subject of all his mental phenomena, the agent in his own activities, the subject of his own feelings, and the possessor of capacities and character; a person as a distinct individual; a being regarded as having personality.</def> <ldquo/Those who liked their real <xex>selves</xex>.<rdquo/  <rj><au>Addison.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>A man's <qex>self</qex> may be the worst fellow to converse with in the world.</q> <rj><qau>Pope.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>The <qex>self</qex>, the I, is recognized in every act of intelligence as the subject to which that act belongs. It is I that perceive, I that imagine, I that remember, I that attend, I that compare, I that feel, I that will, I that am conscious.</q> <rj><qau>Sir W. Hamilton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Hence, personal interest, or love of private interest; selfishness; <as>as, <ex>self</ex> is his whole aim</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>Personification; embodiment.</def> <mark>[Poetic.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>She was beauty's <qex>self</qex>.</q> <rj><qau>Thomson.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ <xex>Self</xex> is united to certain personal pronouns and pronominal adjectives to express emphasis or distinction. Thus, for emphasis; I <xex>myself</xex> will write; I will examine for <xex>myself</xex>; thou <xex>thyself</xex> shalt go; thou shalt see for <xex>thyself</xex>; you <xex>yourself</xex> shall write; you shall see for <xex>yourself</xex>; he <xex>himself</xex> shall write; he shall examine for <xex>himself</xex>; she <xex>herself</xex> shall write; she shall examine for <xex>herself</xex>; the child <xex>itself</xex> shall be carried; it shall be present <xex>itself</xex>. It is also used reflexively; as, I abhor <xex>myself</xex>; thou enrichest <xex>thyself</xex>; he loves <xex>himself</xex>; she admires <xex>herself</xex>; it pleases <xex>itself</xex>; we walue <xex>ourselves</xex>; ye hurry <xex>yourselves</xex>; they see <xex>themselves</xex>. <xex>Himself</xex>, <xex>herself</xex>, <xex>themselves</xex>, are used in the nominative case, as well as in the objective. <ldquo/Jesus <xex>himself</xex> baptized not, but his disciples.<rdquo/  <au>John iv. 2.</au></note><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ <ex>self</ex> is used in the formation of innumerable compounds, usually of obvious signification, in most of which it denotes either the agent or the object of the action expressed by the word with which it is joined, or the person in behalf of whom it is performed, or the person or thing to, for, or towards whom or which a quality, attribute, or feeling expressed by the following word belongs, is directed, or is exerted, or from which it proceeds; or it denotes the subject of, or object affected by, such action, quality, attribute, feeling, or the like; as, <ex>self</ex>-abandoning, <ex>self</ex>-abnegation, self-abhorring, <xex>self-absorbed</xex>, <ex>self</ex>-accusing, <ex>self</ex>-adjusting, <ex>self</ex>-balanced, <ex>self</ex>-boasting, <ex>self</ex>-canceled, <ex>self</ex>-combating, <ex>self</ex>-commendation, <ex>self</ex>-condemned, <ex>self</ex>-conflict, <ex>self</ex>-conquest, <ex>self</ex>-constituted, <ex>self</ex>-consumed, <ex>self</ex>-contempt, <ex>self</ex>-controlled, <ex>self</ex>-deceiving, <ex>self</ex>-denying, <ex>self</ex>-destroyed, <ex>self</ex>-disclosure, <ex>self</ex>-display, <ex>self</ex>-dominion, <ex>self</ex>-doomed, <ex>self</ex>-elected, <ex>self</ex>-evolved, <ex>self</ex>-exalting, <ex>self</ex>-excusing, <ex>self</ex>-exile, <ex>self</ex>-fed, <ex>self</ex>-fulfillment, <ex>self</ex>-governed, <ex>self</ex>-harming, <ex>self</ex>-helpless, <ex>self</ex>-humiliation, <ex>self</ex>-idolized, <ex>self</ex>-inflicted, <ex>self</ex>-improvement, <ex>self</ex>-instruction, <ex>self</ex>-invited, <ex>self</ex>-judging, <ex>self</ex>-justification, <ex>self</ex>-loathing, <ex>self</ex>-loving, <ex>self</ex>-maintenance, <ex>self</ex>-mastered, <ex>self</ex>-nourishment, <ex>self</ex>-perfect, <ex>self</ex>-perpetuation, <ex>self</ex>-pleasing, <ex>self</ex>-praising, <ex>self</ex>-preserving, <ex>self</ex>-questioned, <ex>self</ex>-relying, <ex>self</ex>-restraining, <ex>self</ex>-revelation, <ex>self</ex>-ruined, <ex>self</ex>-satisfaction, <ex>self</ex>-support, <ex>self</ex>-sustained, <ex>self</ex>-sustaining, <ex>self</ex>-tormenting, <ex>self</ex>-troubling, <ex>self</ex>-trust, <ex>self</ex>-tuition, <ex>self</ex>-upbraiding, <ex>self</ex>-valuing, <ex>self</ex>-worshiping, and many others.</note><br/
[<source>1913 Webster</source>]</p>

<p><ent>Self-abased</ent><br/
<hw>Self`-a*based"</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Humbled by consciousness of inferiority, unworthiness, guilt, or shame.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Self-abasement</ent><br/
<hw>Self`-a*base"ment</hw> <pr>(?)</pr>, <pos>n.</pos> <sn>1.</sn> <def>Degradation of one's self by one's own act.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Humiliation or abasement proceeding from consciousness of inferiority, guilt, or shame.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Self-abasing</ent><br/
<hw>Self`-a*bas"ing</hw>, <pos>a.</pos> <def>Lowering or humbling one's self.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Self-abhorrence</ent><br/
<hw>Self`-ab*hor"rence</hw> <pr>(?)</pr>, <pos>n.</pos> <def>Abhorrence of one's self.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Self-abnegation</ent><br/
<hw>Self`-ab`ne*ga"tion</hw> <pr>(?)</pr>, <pos>n.</pos> <def>Self-denial; self-renunciation; self-sacrifice.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Self-abuse</ent><br/
<hw>Self`-abuse"</hw> <pr>(?)</pr>, <pos>n.</pos> <sn>1.</sn> <def>The abuse of one's own self, powers, or faculties.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Self-deception; delusion.</def> <mark>[Obs.]</mark>  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>Masturbation; onanism; self-pollution.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Self-accused</ent><br/
<hw>Self"-ac*cused"</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Accused by one's self or by one's conscience.</def> <ldquo/Die <xex>self-accused</xex>.<rdquo/  <rj><au>Cowper.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Self-acting</ent><br/
<hw>Self`-act"ing</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Acting of or by one's self or by itself; -- said especially of a machine or mechanism which is made to perform of or for itself what is usually done by human agency; automatic; <as>as, a <ex>self-acting</ex> feed apparatus; a <ex>self-acting</ex> mule; a <ex>self-acting</ex> press</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Self-action</ent><br/
<hw>Self`-ac"tion</hw> <pr>(?)</pr>, <pos>n.</pos> <def>Action by, or originating in, one's self or itself.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Self-active</ent><br/
<hw>Self`-ac"tive</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Acting of one's self or of itself; acting without depending on other agents.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Self-activity</ent><br/
<hw>Self`-ac*tiv"i*ty</hw> <pr>(?)</pr>, <pos>n.</pos> <def>The quality or state of being self-active; self-action.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Self-adjusting</ent><br/
<hw>Self`-ad*just"ing</hw> <pr>(?)</pr>, <pos>a.</pos> <fld>(Mach.)</fld> <def>Capable of assuming a desired position or condition with relation to other parts, under varying circumstances, without requiring to be adjusted by hand; -- said of a piece in machinery.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Self-adjusting bearing</b></col> <fld>(Shafting)</fld>, <cd>a bearing which is supported in such a manner that it may tip to accomodate flexure or displacement of the shaft.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Self-admiration</ent><br/
<hw>Self`-ad`mi*ra"tion</hw> <pr>(?)</pr>, <pos>n.</pos> <def>Admiration of one's self.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Self-affairs</ent><br/
<hw>Self`-af*fairs"</hw> <pr>(?)</pr>, <pos>n. pl.</pos> <def>One's own affairs; one's private business.</def> <mark>[Obs.]</mark>  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><-- p. 1306 --></p>

<p><ent>Self-affrighted</ent><br/
<hw>Self`-af*fright"ed</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Frightened at or by one's self.</def>  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Self-aggrandizement</ent><br/
<hw>Self`-ag*gran"dize*ment</hw> <pr>(?)</pr>, <pos>n.</pos> <def>The aggrandizement of one's self.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Self-annihilated</ent><br/
<hw>Self`-an*ni"hi*la`ted</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Annihilated by one's self.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Self-annihilation</ent><br/
<hw>Self`-an*ni`hi*la"tion</hw> <pr>(?)</pr>, <pos>n.</pos> <def>Annihilation by one's own acts; annihilation of one's desires.</def>  <rj><au>Addison.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Self-applause</ent><br/
<hw>Self`-ap*plause"</hw> <pr>(?)</pr>, <pos>n.</pos> <def>Applause of one's self.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Self-applying</ent><br/
<hw>Self`-ap*ply"ing</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Applying to or by one's self.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Self-approving</ent><br/
<hw>Self`-ap*prov"ing</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Approving one's own action or character by one's own judgment.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>One <qex>self-approving</qex> hour whole years outweighs<br/
Of stupid starers and of loud huzzas.</q> <rj><qau>Pope.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Self-asserting</ent><br/
<hw>Self`-as*sert"ing</hw> <pr>(?)</pr>, <pos>a.</pos> <def>asserting one's self, or one's own rights or claims; hence, putting one's self forward in a confident or assuming manner.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Self-assertion</ent><br/
<hw>Self`-as*ser"tion</hw> <pr>(?)</pr>, <pos>n.</pos> <def>The act of asserting one's self, or one's own rights or claims; the quality of being self-asserting.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Self-assertive</ent><br/
<hw>Self`-as*sert"ive</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Disposed to self-assertion; self-asserting.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Self-assumed</ent><br/
<hw>Self`-as*sumed`</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Assumed by one's own act, or without authority.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Self-assured</ent><br/
<hw>Self`-as*sured`</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Assured by or of one's self; self-reliant; complacent.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Self-banished</ent><br/
<hw>Self`-ban"ished</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Exiled voluntarily.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Self-begotten</ent><br/
<hw>Self"-be*got"ten</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Begotten by one's self, or one's own powers.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Self-born</ent><br/
<hw>Self"-born`</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Born or produced by one's self.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Self-binder</ent><br/
<hw>Self`-bind"er</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A reaping machine containing mechanism for binding the grain into sheaves.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Self-centred</ent><br/
<ent>Self-centered</ent><br/
<mhw>{ <hw>Self`-cen"tered</hw>, <hw>Self`-cen"tred</hw>  }</mhw> <pr>(?)</pr>, <pos>a.</pos> <sn>1.</sn> <def>Centered in itself, or in one's self.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>There hangs the ball of earth and water mixt,<br/
<qex>Self-centered</qex> and unmoved.</q> <rj><qau>Dryden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>interested solely or primiarily in oneself, or one's own welfare; disinclined to consider the interests of others; selfish; egotistical.</def><br/
[<source>PJC</source>]</p>

<p><ent>Self-centring</ent><br/
<ent>Self-centering</ent><br/
<mhw>{ <hw>Self`-cen"ter*ing</hw> <pr>(?)</pr>, <hw>Self`-cen"tring</hw> <pr>(?)</pr> }</mhw> <pos>a.</pos> <def>Centering in one's self.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Self-centration</ent><br/
<hw>Self`-cen*tra"tion</hw> <pr>(?)</pr>, <pos>n.</pos> <def>The quality or state of being self-centered.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Self-charity</ent><br/
<hw>Self`-char"i*ty</hw> <pr>(?)</pr>, <pos>n.</pos> <def>Self-love.</def> <mark>[Obs.]</mark>  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Self-color</ent><br/
<hw>Self"-col`or</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A color not mixed or variegated.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Self-colored</ent><br/
<hw>Self`-col"ored</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Being of a single color; -- applied to flowers, animals, and textile fabrics.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Self-command</ent><br/
<hw>Self`-com*mand"</hw> <pr>(?)</pr>, <pos>n.</pos> <def>Control over one's own feelings, temper, etc.; self-control.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Self-commune</ent><br/
<hw>Self`-com*mune"</hw> <pr>(?)</pr>, <pos>n.</pos> <def>Self-communion.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Self-communicative</ent><br/
<hw>Self`-com*mu"ni*ca*tive</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Imparting or communicating by its own powers.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Self-communion</ent><br/
<hw>Self`-com*mun"ion</hw> <pr>(?)</pr>, <pos>n.</pos> <def>Communion with one's self; thoughts about one's self.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Self-complacency</ent><br/
<hw>Self`-com*pla"cen*cy</hw> <pr>(?)</pr>, <pos>n.</pos> <def>The quality of being self-complacent.</def>  <rj><au>J. Foster.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Self-complacent</ent><br/
<hw>Self`-com*pla"cent</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Satisfied with one's own character, capacity, and doings; self-satisfied.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Self-conceit</ent><br/
<hw>Self`-con*ceit"</hw> <pr>(?)</pr>, <pos>n.</pos> <def>Conceit of one's self; an overweening opinion of one's powers or endowments.</def><br/
[<source>1913 Webster</source>]</p>

<p><syn><b>Syn.</b> -- See <er>Egotism</er>.</syn><br/
[<source>1913 Webster</source>]</p>

<p><ent>Self-conceited</ent><br/
<hw>Self`-con*ceit"ed</hw>, <pos>a.</pos> <def>Having an overweening opinion of one's own powers, attainments; vain; conceited.</def> -- <wordforms><wf>Self`-con*ceit"ed*ness</wf>, <pos>n.</pos></wordforms><br/
[<source>1913 Webster</source>]</p>

<p><ent>Self-concern</ent><br/
<hw>Self`-con*cern"</hw> <pr>(?)</pr>, <pos>n.</pos> <def>Concern for one's self.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Self-condemnation</ent><br/
<hw>Self`-con`dem*na"tion</hw> <pr>(?)</pr>, <pos>n.</pos> <def>Condemnation of one's self by one's own judgment.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Self-confidence</ent><br/
<hw>Self`-con"fi*dence</hw> <pr>(?)</pr>, <pos>n.</pos> <def>The quality or state of being self-confident; self-reliance.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>A feeling of <qex>self-confidence</qex> which supported and sustained him.</q> <rj><qau>Beaconsfield.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Self-confident</ent><br/
<hw>Self`-con"fi*dent</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Confident of one's own strength or powers; relying on one's judgment or ability; self-reliant.</def> -- <wordforms><wf>Self`-con"fi*dent*ly</wf>, <pos>adv.</pos></wordforms><br/
[<source>1913 Webster</source>]</p>

<p><ent>Self-conjugate</ent><br/
<hw>Self`-con"ju*gate</hw> <pr>(?)</pr>, <pos>a.</pos> <fld>(Geom.)</fld> <def>Having the two things that are conjugate parts of the same figure; <as>as, <ex>self-conjugate</ex> triangles</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Self-conscious</ent><br/
<hw>Self`-con"scious</hw> <pr>(?)</pr>, <pos>a.</pos> <sn>1.</sn> <def>Conscious of one's acts or state as belonging to, or originating in, one's self.</def> <ldquo/My <xex>self-conscious</xex> worth.<rdquo/  <rj><au>Dryden.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Conscious of one's self as an object of the observation of others; <as>as, the speaker was too <ex>self-conscious</ex></as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Self-consciousness</ent><br/
<hw>Self`-con"scious*ness</hw>, <pos>n.</pos> <def>The quality or state of being self-conscious.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Self-considering</ent><br/
<hw>Self`-con*sid"er*ing</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Considering in one's own mind; deliberating.</def>  <rj><au>Pope.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Self-consistency</ent><br/
<hw>Self`-con*sist"en*cy</hw> <pr>(?)</pr>, <pos>n.</pos> <def>The quality or state of being self-consistent.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Self-consistent</ent><br/
<hw>Self`-con*sist"ent</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Consistent with one's self or with itself; not deviation from the ordinary standard by which the conduct is guided; logically consistent throughout; having each part consistent with the rest.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Self-consuming</ent><br/
<hw>Self`-con*sum"ing</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Consuming one's self or itself.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Self-contained</ent><br/
<hw>Self`-con*tained"</hw> <pr>(?)</pr>, <pos>a.</pos> <sn>1.</sn> <def>Having self-control; reserved; uncommunicative; wholly engrossed in one's self.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Mach.)</fld> <def>Having all the essential working parts connected by a bedplate or framework, or contained in a case, etc., so that mutual relations of the parts do not depend upon fastening outside of the machine itself.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Self-contained steam engine</b></col>. <sd>(a)</sd> <cd>A steam engine having both bearings for the crank shaft attached to the frame of the engine</cd>. <sd>(b)</sd> <cd>A steam engine and boiler combined and fastened together; a portable steam engine.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Self-contained Underwater Breathing Apparatus</ent><br/
<hw>Self`-con*tained" Underwater Breathing Apparatus</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A portable device to allow divers to breathe while under water, consisting of one or two tanks of compressed air which are strapped onto the back of the diver, and are connected by tubing to a mouthpiece through which the diver receives the air from the tanks at rate adjustable by a valve; -- called also <altname>SCUBA</altname>, <altname>SCUBA gear</altname>, or <altname>SCUBA apparatus</altname>.</def><br/
[<source>PJC</source>]</p>

<p><ent>Self-contradiction</ent><br/
<hw>Self`-con`tra*dic"tion</hw> <pr>(?)</pr>, <pos>n.</pos> <def>The act of contradicting one's self or itself; repugnancy in conceptions or in terms; a proposition consisting of two members, one of which contradicts the other; <as>as, to be and not to be at the same time is a <ex>self-contradiction</ex></as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Self-contradictory</ent><br/
<hw>Self`-con`tra*dict"o*ry</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Contradicting one's self or itself.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Self-control</ent><br/
<hw>Self`-con*trol"</hw> <pr>(?)</pr>, <pos>n.</pos> <def>Control of one's self; restraint exercised over one's self; self-command.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Self-convicted</ent><br/
<hw>Self`-con*vict"ed</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Convicted by one's own consciousness, knowledge, avowal, or acts.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Self-conviction</ent><br/
<hw>Self`-con*vic"tion</hw> <pr>(?)</pr>, <pos>n.</pos> <def>The act of convicting one's self, or the state of being self-convicted.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Self-created</ent><br/
<hw>Self`-cre*at"ed</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Created by one's self; not formed or constituted by another.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Self-culture</ent><br/
<hw>Self`-cul"ture</hw> <pr>(?)</pr>, <pos>n.</pos> <def>Culture, training, or education of one's self by one's own efforts.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Self-deceit</ent><br/
<hw>Self`-de*ceit"</hw> <pr>(?)</pr>, <pos>n.</pos> <def>The act of deceiving one's self, or the state of being self-deceived; self-deception.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Self-deceived</ent><br/
<hw>Self`-de*ceived"</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Deceived or misled respecting one's self by one's own mistake or error.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Self-deception</ent><br/
<hw>Self`-de*cep"tion</hw> <pr>(?)</pr>, <pos>n.</pos> <def>Self-deceit.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Self-defence</ent><br/
<hw>Self`-de*fence"</hw> <pr>(?)</pr>, <pos>n.</pos> <def>See <er>Self-defense</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Self-defense</ent><br/
<hw>Self`-de*fense"</hw> <pr>(?)</pr>, <pos>n.</pos> <def>The act of defending one's own person, property, or reputation.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>In self-defense</b></col> <fld>(Law)</fld>, <cd>in protection of self, -- it being permitted in law to a party on whom a grave wrong is attempted to resist the wrong, even at the peril of the life of the assailiant.</cd>  <rj><au>Wharton.</au></rj></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Self-defensive</ent><br/
<hw>Self`-de*fen"sive</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Defending, or tending to defend, one's own person, property, or reputation.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Self-degradation</ent><br/
<hw>Self`-deg`ra*da"tion</hw> <pr>(?)</pr>, <pos>n.</pos> <def>The act of degrading one's self, or the state of being so degraded.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Self-delation</ent><br/
<hw>Self`-de*la"tion</hw> <pr>(?)</pr>, <pos>n.</pos> <def>Accusation of one's self.</def> <mark>[R.]</mark>  <rj><au>Milman.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Self-delusion</ent><br/
<hw>Self`-de*lu"sion</hw> <pr>(?)</pr>, <pos>n.</pos> <def>The act of deluding one's self, or the state of being thus deluded.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Self-denial</ent><br/
<hw>Self`-de*ni"al</hw> <pr>(?)</pr>, <pos>n.</pos> <def>The denial of one's self; forbearing to gratify one's own desires; self-sacrifice.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Self-denying</ent><br/
<hw>Self`-de*ny"ing</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Refusing to gratify one's self; self-sacrificing.</def> -- <wordforms><wf>Self`-de*ny"ing*ly</wf>, <pos>adv.</pos></wordforms><br/
[<source>1913 Webster</source>]</p>

<p><ent>Self-dependent</ent><br/
<hw>Self`-de*pend"ent</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Dependent on one's self; self-depending; self-reliant.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Self-depending</ent><br/
<hw>Self`-de*pend"ing</hw>, <pos>a.</pos> <def>Depending on one's self.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Self-depraved</ent><br/
<hw>Self`-de*praved"</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Corrupted or depraved by one's self.</def>  <rj><au>Milton.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Self-destroyer</ent><br/
<hw>Self`-de*stroy"er</hw> <pr>(?)</pr>, <pos>n.</pos> <def>One who destroys himself; a suicide.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Self-destruction</ent><br/
<hw>Self`-de*struc"tion</hw> <pr>(?)</pr>, <pos>n.</pos> <def>The destruction of one's self; self-murder; suicide.</def>  <rj><au>Milton.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Self-destructive</ent><br/
<hw>Self`-de*struc"tive</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Destroying, or tending to destroy, one's self or itself; rucidal.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Self-determination</ent><br/
<hw>Self`-de*ter`mi*na"tion</hw> <pr>(?)</pr>, <pos>n.</pos> <def>Determination by one's self; or, determination of one's acts or states without the necessitating force of motives; -- applied to the voluntary or activity.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Self-determining</ent><br/
<hw>Self`-de*ter"min*ing</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Capable of self-determination; <as>as, the <ex>self-determining</ex> power of will</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Self-devised</ent><br/
<hw>Self`-de*vised"</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Devised by one's self.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Self-devoted</ent><br/
<hw>Self`-de*vot"ed</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Devoted in person, or by one's own will.</def>  <rj><au>Hawthorne.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Self-devotement</ent><br/
<hw>Self`-de*vote"ment</hw> <pr>(?)</pr>, <pos>n.</pos> <def>Self-devotion.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Self-devotion</ent><br/
<hw>Self`-de*vo"tion</hw> <pr>(?)</pr>, <pos>n.</pos> <def>The act of devoting one's self, or the state of being self-devoted; willingness to sacrifice one's own advantage or happiness for the sake of others; self-sacrifice.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Self-devouring</ent><br/
<hw>Self`-de*vour"ing</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Devouring one's self or itself.</def>  <rj><au>Danham.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Self-diffusive</ent><br/
<hw>Self`-dif*fu"sive</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Having power to diffuse itself; diffusing itself.</def>  <rj><au>Norris.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Self-discipline</ent><br/
<hw>Self`-dis"ci*pline</hw> <pr>(?)</pr>, <pos>n.</pos> <def>Correction or government of one's self for the sake of improvement.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Self-distrust</ent><br/
<hw>Self`-dis*trust"</hw> <pr>(?)</pr>, <pos>n.</pos> <def>Want of confidence in one' self; diffidence.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Self-educated</ent><br/
<hw>Self`-ed"u*ca`ted</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Educated by one's own efforts, without instruction, or without pecuniary assistance from others.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Self-elective</ent><br/
<hw>Self`-e*lect"ive</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Having the right of electing one's self, or, as a body, of electing its own members.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Self-enjoyment</ent><br/
<hw>Self`-en*joy"ment</hw>, <pr>(?)</pr> <pos>n.</pos> <def>Enjoyment of one's self; self-satisfaction.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Self-esteem</ent><br/
<hw>Self`-es*teem"</hw> <pr>(?)</pr>, <pos>n.</pos> <def>The holding a good opinion of one's self; self-complacency.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Self-estimation</ent><br/
<hw>Self`-es`ti*ma"tion</hw> <pr>(?)</pr>, <pos>n.</pos> <def>The act of estimating one's self; self-esteem.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Self-evidence</ent><br/
<hw>Self`-ev"i*dence</hw> <pr>(?)</pr>, <pos>n.</pos> <def>The quality or state of being self-evident.</def>  <rj><au>Locke.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Self-evident</ent><br/
<hw>Self`-ev"i*dent</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Evident without proof or reasoning; producing certainty or conviction upon a bare presentation to the mind; <as>as, a <ex>self-evident</ex> proposition or truth</as>.</def> -- <wordforms><wf>Self`-ev"i*dent*ly</wf>, <pos>adv.</pos></wordforms><br/
[<source>1913 Webster</source>]</p>

<p><ent>Self-evolution</ent><br/
<hw>Self`-ev`o*lu"tion</hw> <pr>(?)</pr>, <pos>n.</pos> <def>Evolution of one's self; development by inherent quality or power.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Self-exaltation</ent><br/
<hw>Self`-ex`al*ta"tion</hw> <pr>(?)</pr>, <pos>n.</pos> <def>The act of exalting one's self, or the state of being so exalted.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Self-examinant</ent><br/
<hw>Self`-ex*am"i*nant</hw> <pr>(?)</pr>, <pos>n.</pos> <def>One who examines himself; one given to self-examination.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The humiliated <qex>self-examinant</qex> feels that there is evil in our nature as well as good.</q> <rj><qau>Coleridge.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Self-examination</ent><br/
<hw>Self`-ex*am`i*na"tion</hw> <pr>(?)</pr>, <pos>n.</pos> <def>An examination into one's own state, conduct, and motives, particularly in regard to religious feelings and duties.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Self-excite</ent><br/
<hw>Self`-ex*cite"</hw>, <pos>v. t.</pos> <fld>(Elec.)</fld> <def>To energize or excite (the field magnets of a dynamo) by induction from the residual magnetism of its cores, leading all or a part of the current thus produced through the field-magnet coils.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Self-existence</ent><br/
<hw>Self`-ex*ist"ence</hw> <pr>(?)</pr>, <pos>n.</pos> <def>Inherent existence; existence possessed by virtue of a being's own nature, and independent of any other being or cause; -- an attribute peculiar to God.</def>  <rj><au>Blackmore.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Self-existent</ent><br/
<hw>Self`-ex*ist"ent</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Existing of or by himself, independent of any other being or cause; -- <as>as, God is the only <ex>self-existent</ex> being</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>self-explaining</ent><br/
<hw>self`-ex*plain"ing</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Explaining itself; capable of being understood without explanation.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Self-exposure</ent><br/
<hw>Self`-ex*po"sure</hw> <pr>(?)</pr>, <pos>n.</pos> <def>The act of exposing one's self; the state of being so exposed.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Self-fertilization</ent><br/
<hw>Self`-fer`ti*li*za"tion</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Bot.)</fld> <def>The fertilization of a flower by pollen from the same flower and without outer aid; autogamy.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Self-fertilized</ent><br/
<hw>Self`-fer"ti*lized</hw> <pr>(?)</pr>, <pos>a.</pos> <fld>(Bot.)</fld> <def>Fertilized by pollen from the same flower.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Self-glorious</ent><br/
<hw>Self`-glo"ri*ous</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Springing from vainglory or vanity; vain; boastful.</def>  <rj><au>Dryden.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Self-government</ent><br/
<hw>Self`-gov"ern*ment</hw> <pr>(?)</pr>, <pos>n.</pos> <sn>1.</sn> <def>The act of governing one's self, or the state of being governed by one's self; self-control; self-command.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Hence, government of a community, state, or nation by the joint action of the mass of people constituting such a civil body; also, the state of being so governed; democratic government; democracy.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>It is to <qex>self-government</qex>, the great principle of popular representation and administration, -- the system that lets in all to participate in the councels that are to assign the good or evil to all, -- that we may owe what we are and what we hope to be.</q> <rj><qau>D. Webster.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Self-gratulation</ent><br/
<hw>Self`-grat`u*la"tion</hw> <pr>(?)</pr>, <pos>n.</pos> <def>Gratulation of one's self.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Self-hardening</ent><br/
<hw>Self`-hard"en*ing</hw>, <pos>a.</pos> <fld>(Metal.)</fld> <def>Designating, or pert. to, any of various steels that harden when heated to above a red heat and cooled in air, usually in a blast of cold air with moderate rapidity, without quenching. Such steels are alloys of iron and carbon with manganese, tungsten and manganese, chromium, molybdenum and manganese, etc. They are chiefly used as high-speed steels.</def> -- <wordforms><wf>Self`-hard"en*ed</wf>, <pos>a.</pos></wordforms><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Self-heal</ent><br/
<hw>Self`-heal"</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Bot.)</fld> <def>A blue-flowered labiate plant (<spn>Brunella vulgaris</spn>); the healall.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Self-healing</ent><br/
<hw>Self`-heal"ing</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Having the power or property of healing itself.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Self-help</ent><br/
<hw>Self`-help"</hw> <pr>(?)</pr>, <pos>n.</pos> <sn>1.</sn> <def>The act of aiding one's self, without depending on the aid of others.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn>  <fld>(Law)</fld> <def>The right or fact of redressing or preventing wrongs by one's own action without recourse to legal proceedings, as in self-defense, distress, abatement of a nuisance, etc.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><sn>3.</sn>  <def>developing one's capabilities or solving one's own problems, especially psychological problems, without the assistance of a professional, by independent study or by association with other individuals having similar interests.</def><br/
[<source>PJC</source>]</p>

<p><ent>Self-help</ent><br/
<hw>Self`-help"</hw> <pr>(?)</pr>, <pos>a.</pos> <def>of, pertaining to, or useful for the process of developing one's capabilities or solving one's problems; <as>as, <ex>self-help</ex> books on writing for profit</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Self-homicide</ent><br/
<hw>Self`-hom"i*cide</hw> <pr>(?)</pr>, <pos>n.</pos> <def>The act of killing one's self; suicide.</def>  <rj><au>Hakewill.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Selfhood</ent><br/
<hw>Self"hood</hw> <pr>(?)</pr>, <pos>n.</pos> <def>Existence as a separate self, or independent person; conscious personality; individuality.</def>  <rj><au>Bib. Sacra.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Self-ignorance</ent><br/
<hw>Self`-ig"no*rance</hw> <pr>(?)</pr>, <pos>n.</pos> <def>Ignorance of one's own character, powers, and limitations.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Self-ignorant</ent><br/
<hw>Self`-ig"no*rant</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Ignorant of one's self.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Self-imparting</ent><br/
<hw>Self`-im*part"ing</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Imparting by one's own, or by its own, powers and will.</def>  <rj><au>Norris.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Self-importance</ent><br/
<hw>Self`-im*por"tance</hw> <pr>(?)</pr>, <pos>n.</pos> <def>An exaggerated estimate of one's own importance or merit, esp. as manifested by the conduct or manners; self-conceit.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Self-important</ent><br/
<hw>Self`-im*por"tant</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Having or manifesting an exaggerated idea of one's own importance or merit.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Self-imposed</ent><br/
<hw>Self`-im*posed"</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Voluntarily taken on one's self; <as>as, <ex>self-imposed</ex> tasks</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Self-imposture</ent><br/
<hw>Self`-im*pos"ture</hw> <pr>(?)</pr>, <pos>n.</pos> <def>Imposture practiced on one's self; self-deceit.</def>  <rj><au>South.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Self-indignation</ent><br/
<hw>Self`-in`dig*na"tion</hw> <pr>(?)</pr>, <pos>n.</pos> <def>Indignation at one's own character or actions.</def>  <rj><au>Baxter.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Self-induction</ent><br/
<hw>Self`-in*duc"tion</hw>, <pos>n.</pos> <fld>(Elec.)</fld> <def>Induction in a circuit due to the action of one portion of a current upon an adjacent portion during periods of varying current strength. The nature of the induction is such as to oppose the action which produces it.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Self-indulgence</ent><br/
<hw>Self`-in*dul"gence</hw> <pr>(?)</pr>, <pos>n.</pos> <def>Indulgence of one's appetites, desires, or inclinations; -- the opposite of <ant>self-restraint</ant>, and <ant>self-denial</ant>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Self-indulgent</ent><br/
<hw>Self`-in*dul"gent</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Indulging one's appetites, desires, etc., freely.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Self-interest</ent><br/
<hw>Self`-in"ter*est</hw> <pr>(?)</pr>, <pos>n.</pos> <def>Private interest; the interest or advantage of one's self.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Self-interested</ent><br/
<hw>Self`-in"ter*est*ed</hw>, <pos>a.</pos> <def>Particularly concerned for one's own interest or happiness.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Self-involution</ent><br/
<hw>Self`-in`vo*lu"tion</hw> <pr>(?)</pr>, <pos>n.</pos> <def>Involution in one's self; hence, abstraction of thought; reverie.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Selfish</ent><br/
<hw>Self"ish</hw> <pr>(?)</pr>, <pos>a.</pos> <sn>1.</sn> <def>Caring supremely or unduly for one's self; regarding one's own comfort, advantage, etc., in disregard, or at the expense, of those of others.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>They judge of things according to their own private appetites and <qex>selfish</qex> passions.</q> <rj><qau>Cudworth.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>In that throng of <qex>selfish</qex> hearts untrue.</q> <rj><qau>Keble.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Ethics)</fld> <def>Believing or teaching that the chief motives of human action are derived from love of self.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Hobbes and the <qex>selfish</qex> school of philosophers.</q> <rj><qau>Fleming.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Selfishly</ent><br/
<hw>Self"ish*ly</hw>, <pos>adv.</pos> <def>In a selfish manner; with regard to private interest only or chiefly.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Selfishness</ent><br/
<hw>Self"ish*ness</hw>, <pos>n.</pos> <def>The quality or state of being selfish; exclusive regard to one's own interest or happiness; that supreme self-love or self-preference which leads a person to direct his purposes to the advancement of his own interest, power, or happiness, without regarding those of others.</def><br/
[<source>1913 Webster</source>]</p>

<p><q><qex>Selfishness</qex>, -- a vice utterly at variance with the happiness of him who harbors it, and, as such, condemned by self-love.</q> <rj><qau>Sir J. Mackintosh.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><syn><b>Syn.</b> -- See <er>Self-love</er>.</syn><br/
[<source>1913 Webster</source>]</p>

<p><ent>Selfism</ent><br/
<hw>Self"ism</hw> <pr>(?)</pr>, <pos>n.</pos> <def>Concentration of one's interests on one's self; self-love; selfishness.</def>  <rj><au>Emerson.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Selfist</ent><br/
<hw>Self"ist</hw>, <pos>n.</pos> <def>A selfish person.</def> <mark>[R.]</mark>  <rj><au>I. Taylor.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Self-justifier</ent><br/
<hw>Self`-jus"ti*fi`er</hw> <pr>(?)</pr>, <pos>n.</pos> <def>One who excuses or justifies himself.</def>  <rj><au>J. M. Mason.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Self-kindled</ent><br/
<hw>Self`-kin"dled</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Kindled of itself, or without extraneous aid or power.</def>  <rj><au>Dryden.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Self-knowing</ent><br/
<hw>Self`-know"ing</hw> <pr>(?)</pr>, <pos>a.</pos> <sn>1.</sn> <def>Knowing one's self, or one's own character, powers, and limitations.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Knowing of itself, without help from another.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Self-knowledge</ent><br/
<hw>Self`-knowl"edge</hw> <pr>(?)</pr>, <pos>n.</pos> <def>Knowledge of one's self, or of one's own character, powers, limitations, etc.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Selfless</ent><br/
<hw>Self"less</hw>, <pos>a.</pos> <def>Having no regard to self; unselfish.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Lo now, what hearts have men! they never mount<br/
As high as woman in her <qex>selfless</qex> mood.</q> <rj><qau>Tennyson.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Selflessness</ent><br/
<hw>Self"less*ness</hw>, <pos>n.</pos> <def>Quality or state of being selfless.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Self-life</ent><br/
<hw>Self"-life`</hw> <pr>(?)</pr>, <pos>n.</pos> <def>Life for one's self; living solely or chiefly for one's own pleasure or good.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Self-love</ent><br/
<hw>Self`-love`</hw> <pr>(?)</pr>, <pos>n.</pos> <def>The love of one's self; desire for personal happiness; tendency to seek one's own benefit or advantage.</def>  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q><qex>Self-love</qex>, the spring of motion, acts the soul.</q> <rj><qau>Pope.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><syn><b>Syn.</b> -- Selfishness.</syn>  -- <usage><er>Self-love</er>, <er>Selfishness</er>. The term <xex>self-love</xex> is used in a twofold sense: 1. It denotes that longing for <xex>good</xex> or for <xex>well-being</xex> which actuates the breasts of all, entering into and characterizing every special desire. In this sense it has no moral quality, being, from the nature of the case, neither good nor evil. 2. It is applied to a voluntary regard for the gratification of special desires. In this sense it is morally good or bad according as these desires are conformed to duty or opposed to it. <xex>Selfishness</xex> is always voluntary and always wrong, being that regard to our own interests, gratification, etc., which is sought or indulged at the expense, and to the injury, of others. <ldquo/So long as <xex>self-love</xex> does not degenerate into <xex>selfishness</xex>, it is quite compatible with true benevolence.<rdquo/ <au>Fleming.</au> <ldquo/Not only is the phrase <xex>self-love</xex> used as synonymous with the desire of happiness, but it is often confounded with the word <xex>selfishness</xex>, which certainly, in strict propriety, denotes a very different disposition of mind.<rdquo/ <au>Slewart.</au></usage><br/
[<source>1913 Webster</source>]</p>

<p><ent>Self-luminous</ent><br/
<hw>Self`-lu"mi*nous</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Possessing in itself the property of emitting light.</def>  <rj><au>Sir D. Brewster.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><-- p. 1307 --></p>

<p><ent>Self-made</ent><br/
<hw>Self"-made`</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Made by one's self.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Self-made man</b></col>, <cd>a man who has risen from poverty or obscurity by means of his own talents or energies.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Self-mettle</ent><br/
<hw>Self"-met`tle</hw> <pr>(?)</pr>, <pos>n.</pos> <def>Inborn mettle or courage; one's own temper.</def> <mark>[Obs.]</mark>  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Self-motion</ent><br/
<hw>Self`-mo"tion</hw> <pr>(?)</pr>, <pos>n.</pos> <def>Motion given by inherent power, without external impulse; spontaneous or voluntary motion.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Matter is not induced with <qex>self-motion</qex>.</q> <rj><qau>Cheyne.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Self-moved</ent><br/
<hw>Self`-moved"</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Moved by inherent power., without the aid of external impulse.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Self-moving</ent><br/
<hw>Self`-mov"ing</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Moving by inherent power, without the aid of external impulse.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Self-murder</ent><br/
<hw>Self`-mur"der</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Suicide.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Self-murderer</ent><br/
<hw>Self`-mur"der*er</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A suicide.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Self-neglecting</ent><br/
<hw>Self`-neg*lect"ing</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A neglecting of one's self, or of one's own interests.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Self-love, my liege, is not so vile a sin<br/
As <qex>self-neglecting</qex>.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Selfness</ent><br/
<hw>Self"ness</hw>, <pos>n.</pos> <def>Selfishness.</def> <mark>[Obs.]</mark>  <rj><au>Sir. P. Sidney.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Self-one</ent><br/
<hw>Self`-one"</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Secret.</def> <mark>[Obs.]</mark>  <rj><au>Marston.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Self-opinion</ent><br/
<hw>Self`-o*pin"ion</hw> <pr>(?)</pr>, <pos>n.</pos> <def>Opinion, especially high opinion, of one's self; an overweening estimate of one's self or of one's own opinion.</def>  <rj><au>Collier.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Self-opinioned</ent><br/
<hw>Self`-o*pin"ioned</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Having a high opinion of one's self; opinionated; conceited.</def>  <rj><au>South.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Self-originating</ent><br/
<hw>Self`-o*rig"i*na`ting</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Beginning with, or springing from, one's self.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Self-partiality</ent><br/
<hw>Self`-par`ti*al"i*ty</hw> <pr>(?)</pr>, <pos>n.</pos> <def>That partiality to himself by which a man overrates his own worth when compared with others.</def>  <rj><au>Kames.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Self-perplexed</ent><br/
<hw>Self`-per*plexed"</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Perplexed by doubts originating in one's own mind.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Self-posited</ent><br/
<hw>Self`-pos"it*ed</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Disposed or arranged by an action originating in one's self or in itself.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>These molecular blocks of salt are <qex>self-posited</qex>.</q> <rj><qau>Tyndall.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Self-positing</ent><br/
<hw>Self`-pos"it*ing</hw>, <pos>a.</pos> <def>The act of disposing or arranging one's self or itself.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The <qex>self-positing</qex> of the molecules.</q> <rj><qau>R. Watts.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Self-possessed</ent><br/
<hw>Self"-pos*sessed"</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Composed or tranquil in mind, manner, etc.; undisturbed.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Self-possession</ent><br/
<hw>Self`-pos*ses"sion</hw> <pr>(?)</pr>, <pos>n.</pos> <def>The possession of one's powers; calmness; self-command; presence of mind; composure.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Self-praise</ent><br/
<hw>Self"-praise`</hw> <pr>(?)</pr>, <pos>n.</pos> <def>Praise of one's self.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Self-preservation</ent><br/
<hw>Self`-pres`er*va"tion</hw> <pr>(?)</pr>, <pos>n.</pos> <def>The preservation of one's self from destruction or injury.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Self-propagating</ent><br/
<hw>Self`-prop"a*ga`ting</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Propagating by one's self or by itself.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Self-registering</ent><br/
<hw>Self`-reg"is*ter*ing</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Registering itself; -- said of any instrument so contrived as to record its own indications of phenomena, whether continuously or at stated times, as at the maxima and minima of variations; <as>as, a <ex>self-registering</ex> anemometer or barometer</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Self-regulated</ent><br/
<hw>Self`-reg"u*la`ted</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Regulated by one's self or by itself.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Self-regulative</ent><br/
<hw>Self`-reg"u*la*tive</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Tending or serving to regulate one's self or itself.</def>  <rj><au>Whewell.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Self-reliance</ent><br/
<hw>Self`-re*li"ance</hw> <pr>(?)</pr>, <pos>n.</pos> <def>Reliance on one's own powers or judgment; self-trust.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Self-reliant</ent><br/
<hw>Self`-re*li"ant</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Reliant upon one's self; trusting to one's own powers or judgment.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Self-renunciation</ent><br/
<hw>Self`-re*nun`ci*a"tion</hw> <pr>(?)</pr>, <pos>n.</pos> <def>The act of renouncing, or setting aside, one's own wishes, claims, etc.; self-sacrifice.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Self-repellency</ent><br/
<hw>Self`-re*pel"len*cy</hw> <pr>(?)</pr>, <pos>n.</pos> <def>The quality or state of being self-repelling.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Self-repelling</ent><br/
<hw>Self`-re*pel"ling</hw>, <pr>(<?/)</pr> <pos>a.</pos> <def>Made up of parts, as molecules or atoms, which mutually repel each other; <as>as, gases are <ex>self-repelling</ex></as>.</def>
<-- gases aren't self-repelling!  Like charges are. --><br/
[<source>1913 Webster</source>]</p>

<p><ent>Self-repetition</ent><br/
<hw>Self`-rep`e*ti"tion</hw> <pr>(?)</pr>, <pos>n.</pos> <def>Repetition of one's self or of one's acts; the saying or doing what one has already said or done.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Self-reproach</ent><br/
<hw>Self`-re*proach"</hw> <pr>(?)</pr>, <pos>n.</pos> <def>The act of reproaching one's self; censure by one's own conscience.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Self-reproached</ent><br/
<hw>Self`-re*proached"</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Reproached by one's own conscience or judgment.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Self-reproaching</ent><br/
<hw>Self`-re*proach"ing</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Reproaching one's self.</def> -- <wordforms><wf>Self`-re*proach"ing*ly</wf>, <pos>adv.</pos></wordforms><br/
[<source>1913 Webster</source>]</p>

<p><ent>Self-reproof</ent><br/
<hw>Self`-re*proof"</hw> <pr>(?)</pr>, <pos>n.</pos> <def>The act of reproving one's self; censure of one's conduct by one's own judgment.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Self-reproved</ent><br/
<hw>Self`-re*proved"</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Reproved by one's own conscience or one's own sense of guilt.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Self-reproving</ent><br/
<hw>Self`-re*prov"ing</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Reproving one's self; reproving by consciousness of guilt.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Self-reprovingly</ent><br/
<hw>Self`-re*prov"ing*ly</hw>, <pos>adv.</pos> <def>In a self-reproving way.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Self-repugnant</ent><br/
<hw>Self`-re*pug"nant</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Self-contradictory; inconsistent.</def>  <rj><au>Brougham.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Self-repulsive</ent><br/
<hw>Self`-re*pul"sive</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Self-repelling.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Self-respect</ent><br/
<hw>Self`-re*spect"</hw> <pr>(?)</pr>, <pos>n.</pos> <def>Respect for one's self; regard for one's character; laudable self-esteem.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Self-restrained</ent><br/
<hw>Self`-re*strained"</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Restrained by one's self or itself; restrained by one's own power or will.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Self-restraint</ent><br/
<hw>Self`-re*straint"</hw> <pr>(?)</pr>, <pos>n.</pos> <def>Restraint over one's self; self-control; self-command.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Self-reverence</ent><br/
<hw>Self`-rev"er*ence</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A reverent respect for one's self.</def>  <rj><au>Tennyson.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Self-righteous</ent><br/
<hw>Self`-right"eous</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Righteous in one's own esteem; pharisaic.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Self-righteousness</ent><br/
<hw>Self`-right"eous*ness</hw>, <pos>n.</pos> <def>The quality or state of being self-righteous; pharisaism.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Self-sacrifice</ent><br/
<hw>Self`-sac"ri*fice</hw> <pr>(?)</pr>, <pos>n.</pos> <def>The act of sacrificing one's self, or one's interest, for others; self-devotion.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Self-sacrificing</ent><br/
<hw>Self`-sac"ri*fi`cing</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Yielding up one's own interest, feelings, etc; sacrificing one's self.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Selfsame</ent><br/
<hw>Self"same</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[<ets>Self</ets>, a. + <ets>same</ets>.]</ety> <def>Precisely the same; the very same; identical.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>His servant was healed in the <qex>selfsame</qex> hour.</q> <rj><qau>Matt. viii. 13.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Self-satisfaction</ent><br/
<hw>Self`-sat`is*fac"tion</hw> <pr>(?)</pr>, <pos>n.</pos> <def>The quality or state of being self-satisfied.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Self-satisfied</ent><br/
<hw>Self`-sat"is*fied</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Satisfied with one's self or one's actions; self-complacent.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Self-satisfying</ent><br/
<hw>Self`-sat"is*fy`ing</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Giving satisfaction to one's self.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Self-seeker</ent><br/
<hw>Self"-seek`er</hw>, <pos>n.</pos> <def>One who seeks only his own interest, advantage, or pleasure.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Self-seeking</ent><br/
<hw>Self"-seek`ing</hw>, <pos>a.</pos> <def>Seeking one's own interest or happiness; selfish.</def>  <rj><au>Arbuthnot.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Self-seeking</ent><br/
<hw>Self"-seek`ing</hw>, <pos>n.</pos> <def>The act or habit of seeking one's own interest or happiness; selfishness.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Self-slaughter</ent><br/
<hw>Self`-slaugh"ter</hw> <pr>(?)</pr>, <pos>n.</pos> <def>Suicide.</def>  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Self-starter</ent><br/
<hw>Self`-start"er</hw>, <pos>n.</pos> <def>A mechanism (usually one operated by electricity, compressed air, a spring, or an explosive gas), attached to an internal-combustion engine, as on an automobile, and used as a means of starting the engine without cranking it by hand; -- called also a <altname>starter</altname>.</def><br/
[<source>Webster 1913 Suppl.</source> <source>+PJC</source>]</p>

<p><ent>Self-sufficiency</ent><br/
<hw>Self`-suf*fi"cien*cy</hw> <pr>(?)</pr>, <pos>n.</pos> <def>The quality or state of being self-sufficient.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Self-sufficient</ent><br/
<hw>Self`-suf*fi"cient</hw> <pr>(?)</pr>, <pos>a.</pos> <sn>1.</sn> <def>Sufficient for one's self without external aid or cooperation.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Neglect of friends can never be proved rational till we prove the person using it omnipotent and <qex>self-sufficient</qex>, and such as can never need any mortal assistance.</q> <rj><qau>South.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Having an overweening confidence in one's own abilities or worth; hence, haughty; overbearing.</def> <ldquo/A rash and <xex>self-sufficient</xex> manner.<rdquo/  <rj><au>I. Watts.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Self-sufficing</ent><br/
<hw>Self`-suf*fi"cing</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Sufficing for one's self or for itself, without needing external aid; self-sufficient.</def> -- <wordforms><wf>Self`-suf*fi"cing*ness</wf>, <pos>n.</pos></wordforms>  <rj><au>J. C. Shairp.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Self-suspended</ent><br/
<hw>Self`-sus*pend"ed</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Suspended by one's self or by itself; balanced.</def>  <rj><au>Southey.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Self-suspicious</ent><br/
<hw>Self`-sus*pi"cious</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Suspicious or distrustful of one's self.</def>  <rj><au>Baxter.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Self-taught</ent><br/
<hw>Self"-taught`</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Taught by one's own efforts.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Self-tormentor</ent><br/
<hw>Self`-tor*ment"or</hw> <pr>(?)</pr>, <pos>n.</pos> <def>One who torments himself.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Self-torture</ent><br/
<hw>Self`-tor"ture</hw> <pr>(?)</pr>, <pos>n.</pos> <def>The act of inflicting pain on one's self; pain inflicted on one's self.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Self-trust</ent><br/
<hw>Self"-trust`</hw>, <pos>n.</pos> <def>Faith in one's self; self-reliance.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Self-uned</ent><br/
<hw>Self`-uned"</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[E. <ets>self</ets> + L. <ets>unus</ets> one.]</ety> <def>One with itself; separate from others.</def> <mark>[Obs.]</mark>  <rj><au>Sylvester.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Self-view</ent><br/
<hw>Self"-view`</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A view of one's self; specifically, carefulness or regard for one's own interests.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Self-will</ent><br/
<hw>Self`-will"</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[AS. <ets>selfwill</ets>.]</ety> <def>One's own will, esp. when opposed to that of others; obstinacy.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Self-willed</ent><br/
<hw>Self`-willed"</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Governed by one's own will; not yielding to the wishes of others; obstinate.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Self-willedness</ent><br/
<hw>Self`-willed"ness</hw>, <pos>n.</pos> <def>Obstinacy.</def>  <rj><au>Sir W. Scott.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Self-worship</ent><br/
<hw>Self"-wor`ship</hw> <pr>(?)</pr>, <pos>n.</pos> <def>The idolizing of one's self; immoderate self-conceit.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Self-wrong</ent><br/
<hw>Self"-wrong`</hw> <pr>(?)</pr>, <pos>n.</pos> <def>Wrong done by a person to himself.</def>  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Selion</ent><br/
<hw>Sel"ion</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[OF. <ets>seillon</ets> a measure of land, F. <ets>sillon</ets> a ridge, furrow, LL. <ets>selio</ets> a measure of land.]</ety> <def>A short piece of land in arable ridges and furrows, of uncertain quantity; also, a ridge of land lying between two furrows.</def> <mark>[Obs. or Prov. Eng.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Seljukian</ent><br/
<hw>Sel*juk"i*an</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Of or pertaining to <persfn><etsep>Seljuk</etsep></persfn>, a Tartar chief who embraced Mohammedanism, and began the subjection of Western Asia to that faith and rule; of or pertaining to the dynasty founded by him, or the empire maintained by his descendants from the 10th to the 13th century.</def>  <rj><au>J. H. Newman.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Seljuckian</ent><br/
<hw>Sel*juck"i*an</hw>, <pos>n.</pos> <def>A member of the family of <persfn><etsep>Seljuk</etsep></persfn>; an adherent of that family, or subject of its government;</def> (<pluf>pl.</pluf>) <def>the dynasty of Turkish sultans sprung from Seljuk.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sell</ent><br/
<hw>Sell</hw> <pr>(s<ecr/l)</pr>, <pos>n.</pos> <def>Self.</def> <mark>[Obs. or Scot.]</mark>  <rj><au>B. Jonson.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sell</ent><br/
<hw>Sell</hw> <pr>(s<ecr/l)</pr>, <pos>n.</pos> <def>A sill.</def> <mark>[Obs.]</mark>  <rj><au>Chaucer.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sell</ent><br/
<hw>Sell</hw> <pr>(s<ecr/l)</pr>, <pos>n.</pos> <def>A cell; a house.</def> <mark>[Obs.]</mark>  <rj><au>Chaucer.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sell</ent><br/
<hw>Sell</hw> <pr>(s<ecr/l)</pr>, <pos>n.</pos> <ety>[F. <ets>selle</ets>, L. <ets>sella</ets>, akin to <ets>sedere</ets> to sit. See <er>Sit</er>.]</ety> <sn>1.</sn> <def>A saddle for a horse.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>He left his lofty steed with golden <qex>self</qex>.</q> <rj><qau>Spenser.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A throne or lofty seat.</def> <mark>[Obs.]</mark>  <rj><au>Fairfax.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sell</ent><br/
<hw>Sell</hw> <pr>(s<ecr/l)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Sold</conjf> <pr>(s<omac/ld)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Selling</conjf>.]</vmorph> <ety>[OE. <ets>sellen</ets>, <ets>sillen</ets>, AS. <ets>sellan</ets>, <ets>syllan</ets>, to give, to deliver; akin to OS. <ets>sellian</ets>, OFries. <ets>sella</ets>, OHG. <ets>sellen</ets>, Icel. <ets>selja</ets> to hand over, to sell, Sw. <ets>s<aum/lja</ets> to sell, Dan. <ets>s<ae/lge</ets>, Goth. <ets>saljan</ets> to offer a sacrifice; all from a noun akin to E. <ets>sale</ets>.  Cf. <er>Sale</er>.]</ety> <sn>1.</sn> <def>To transfer to another for an equivalent; to give up for a valuable consideration; to dispose of in return for something, especially for money.  It is the correlative of <inv>buy</inv>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>If thou wilt be perfect, go and <qex>sell</qex> that thou hast, and give to the poor.</q> <rj><qau>Matt. xix. 21.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>I am changed; I'll go <qex>sell</qex> all my land.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ <xex>Sell</xex> is corellative to <xex>buy</xex>, as one party <xex>buys</xex> what the other <xex>sells</xex>. It is distinguished usually from <xex>exchange</xex> or <xex>barter</xex>, in which one commodity is given for another; whereas in <xex>selling</xex> the consideration is usually money, or its representative in current notes.</note><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To make a matter of bargain and sale of; to accept a price or reward for, as for a breach of duty, trust, or the like; to betray.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>You would have <qex>sold</qex> your king to slaughter.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>To impose upon; to trick; to deceive; to make a fool of; to cheat.</def> <mark>[Slang]</mark>  <rj><au>Dickens.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>To sell one's life dearly</b></col>, <cd>to cause much loss to those who take one's life, as by killing a number of one's assailants.</cd> -- <mcol><col><b>To sell</b></col> (anything) <col><b>out</b></col></mcol>, <cd>to dispose of it wholly or entirely; as, he had <xex>sold out</xex> his corn, or his interest in a business.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sell</ent><br/
<hw>Sell</hw> <pr>(s<ecr/l)</pr>, <pos>v. i.</pos> <sn>1.</sn> <def>To practice selling commodities.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>I will buy with you, <qex>sell</qex> with you; . . . but I will not eat with you.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To be sold; <as>as, corn <ex>sells</ex> at a good price</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>To sell out</b></col>, <cd>to sell one's whole stock in trade or one's entire interest in a property or a business.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sell</ent><br/
<hw>Sell</hw>, <pos>n.</pos> <def>An imposition; a cheat; a hoax.</def> <mark>[Colloq.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sellenders</ent><br/
<ent>Sellanders</ent><br/
<mhw>{ <hw>Sel"lan*ders</hw> <pr>(?)</pr>, <hw>Sel"len*ders</hw> <pr>(?)</pr>, }</mhw> <pos>n. pl.</pos> <fld>(Far.)</fld> <def>See <er>Sallenders</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Seller</ent><br/
<hw>Sell"er</hw> <pr>(?)</pr>, <pos>n.</pos> <def>One who sells.</def>  <rj><au>Chaucer.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Selters water</ent><br/
<hw>Sel"ters wa"ter</hw> <pr>(?)</pr>. <def>A mineral water from <xex>Sellers</xex>, in the district of Nassan, Germany, containing much free carbonic acid.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Seltzer water</ent><br/
<hw>Selt"zer wa"ter</hw> <pr>(?)</pr>. <def>See <er>Selters water</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Seltzo-gene</ent><br/
<hw>Selt"zo-gene</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[<ets>Seltzer</ets> water + the root of Gr. <?/<?/<?/ to be born.]</ety> <def>A gazogene.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Selvedge</ent><br/
<ent>Selvage</ent><br/
<mhw>{ <hw>Sel"vage</hw>, <hw>Sel"vedge</hw>  }</mhw> <pr>(?)</pr>, <pos>n.</pos> <ety>[<ets>Self</ets> + <ets>edge</ets>, i. e., its own proper edge; cf. OD. <ets>selfegge</ets>.]</ety> <sn>1.</sn> <def>The edge of cloth which is woven in such a manner as to prevent raveling.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>The edge plate of a lock, through which the bolt passes.</def>  <rj><au>Knight.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <fld>(Mining.)</fld> <def>A layer of clay or decomposed rock along the wall of a vein. See <er>Gouge</er>, <pos>n.</pos>, 4.</def>  <rj><au>Raymond.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Selvedged</ent><br/
<ent>Selvaged</ent><br/
<mhw>{ <hw>Sel"vaged</hw>,  <hw>Sel"vedged</hw>  }</mhw> <pr>(?)</pr>, <pos>a.</pos> <def>Having a selvage.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Selvagee</ent><br/
<hw>Sel`va*gee"</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Naut.)</fld> <def>A skein or hank of rope yarns wound round with yarns or marline, -- used for stoppers, straps, etc.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Selve</ent><br/
<hw>Selve</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Self; same.</def> <mark>[Obs.]</mark>  <rj><au>Chaucer.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Selves</ent><br/
<hw>Selves</hw> <pr>(?)</pr>, <pos>n.</pos>, <def><pos>pl.</pos> of <er>Self</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sely</ent><br/
<hw>Se"ly</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Silly.</def> <mark>[Obs.]</mark>  <rj><au>Chaucer. Wyclif.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Semaeostomata</ent><br/
||<hw>Se*mae`o*stom"a*ta</hw> <pr>(?)</pr>, <pos>n. pl.</pos> <ety>[NL., fr. Gr. <?/<?/<?/<?/ a military standard + <?/<?/<?/, <?/<?/<?/, mouth.]</ety> <fld>(Zool.)</fld> <def>A division of Discophora having large free mouth lobes. It includes <gen>Aurelia</gen>, and <gen>Pelagia</gen>. Called also <altname>Semeostoma</altname>. See <xex>Illustr.</xex> under <er>Discophora</er>, and <er>Medusa</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Semantical</ent><br/
<ent>Semantic</ent><br/
<mhw><hw>Sem*an"tic</hw>, <hw>Sem*an"tic*al</hw></mhw> <pr>(?)</pr>, <pos>n. sing. <or/ pl.</pos>  <ety>[Gr. <grk>shmantikos</grk> having meaning, from <grk>sh^ma</grk> a sign.]</ety> <sn>1.</sn> <def>pertaining to the meanings of words.</def><br/
[<source>PJC</source>]</p>

<p><sn>2.</sn> <def>of or pertaining to semantics.</def><br/
[<source>PJC</source>]</p>

<p><ent>Semantics</ent><br/
<hw>Sem*an"tics</hw> <pr>(?)</pr>, <pos>n. sing. <or/ pl.</pos>  <ety>[Gr. <grk>shmantikos</grk> having meaning, from <grk>sh^ma</grk> a sign.]</ety> <sn>1.</sn> <def>the study of the meanings of words and of the sense development of words; -- formerly called <altname>semasiology</altname>.</def><br/
[<source>PJC</source>]</p>

<p><sn>2.</sn> <def>a doctrine and philosophical approach to language and its relationship to thought and behavior, developed by Alfred Korzybski (1879-1950), which holds that the capacity to express ideas and thereby improve one's interaction with others and one's environment is enhanced by training in the more critical use of words and other symbols; -- also called <altname><er>general semantics</er></altname>.</def><br/
[<source>PJC</source>]</p>

<p><sn>3.</sn> <def>the meanings of words as they are used to achieve an effect; especially, the multiple meanings of words or the multiplicity of words having the same meaning; -- used in referring to the confusion that can be caused (intentionally or unintentionally) by multiple meanings; <as>as, there's no real difference, it's only a matter of <ex>semantics</ex></as>.</def><br/
[<source>PJC</source>]</p>

<p><ent>Semaphore</ent><br/
<hw>Sem"a*phore</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Gr. <grk>sh^ma</grk> a sign + <grk>fe`rein</grk> to bear: cf. F. <ets>s<eacute/maphore</ets>.]</ety> <def>A signal telegraph; an apparatus for giving signals by the disposition of lanterns, flags, oscillating arms, etc.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Semaphorical</ent><br/
<ent>Semaphoric</ent><br/
<mhw>{ <hw>Sem`a*phor"ic</hw> <pr>(?)</pr>, <hw>Sem`a*phor"ic*al</hw> <pr>(?)</pr> }</mhw> <pos>a.</pos> <ety>[Cf. F. <ets>s<eacute/maphorique</ets>.]</ety> <def>Of or pertaining to a semaphore, or semaphores; telegraphic.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Semaphorically</ent><br/
<hw>Sem`a*phor"ic*al*ly</hw>, <pos>adv.</pos> <def>By means of a semaphore.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Semaphorist</ent><br/
<hw>Se*maph"o*rist</hw> <pr>(?)</pr>, <pos>n.</pos> <def>One who manages or operates a semaphore.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Semasiology</ent><br/
<hw>Se*ma`si*ol"o*gy</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Gr. <grk>shmasi`a</grk> signification + <ets>-logy</ets>.]</ety> <fld>(Philol.)</fld> <def>The science of meanings or sense development (of words); the explanation of the development and changes of the meanings of words; -- more commonly referred to as <altname>semantics</altname>.</def> -- <wordforms><wf>Se*ma`si*o*log"ic*al</wf> <pr>(#)</pr>, <pos>a.</pos></wordforms><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Sematic</ent><br/
<hw>Se*mat"ic</hw> <pr>(s<esl/*m<acr/t"<icr/k)</pr>, <pos>a.</pos> <ety>[Gr. <grk>sh^ma</grk>, <grk>sh`matos</grk>, sign, token.]</ety> <def>Significant; ominous; serving as a warning of danger; -- applied esp. to the warning colors or forms of certain animals.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Sematology</ent><br/
<hw>Sem`a*tol"o*gy</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Gr. <grk>sh^ma</grk>, <grk>sh`matos</grk>, sign + <ets>-logy</ets>.]</ety> <def>The doctrine of signs as the expression of thought or reasoning; the science of indicating thought by signs.</def>  <rj><au>Smart.</au></rj><br/
<syn><b>Syn. --</b> semantics.</syn>
[<source>1913 Webster</source>]</p>

<p><ent>Sematrope</ent><br/
<hw>Sem"a*trope</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Gr. <grk>sh^ma</grk> sign + <grk>tre`pein</grk> to turn. ]</ety> <def>An instrument for signaling by reflecting the rays of the sun in different directions.</def>  <rj><au>Knight.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Semblable</ent><br/
<hw>Sem"bla*ble</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[F., from <ets>sembler</ets> to seem, resemble, L. <ets>similare</ets>, <ets>simulare</ets>. See <er>Simulate</er>.]</ety> <def>Like; similar; resembling.</def> <mark>[Obs.]</mark>  <rj><au>Chaucer.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Semblable</ent><br/
<hw>Sem"bla*ble</hw>, <pos>n.</pos> <def>Likeness; representation.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Semblably</ent><br/
<hw>Sem"bla*bly</hw>, <pos>adv.</pos> <def>In like manner.</def> <mark>[Obs.]</mark>  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Semblance</ent><br/
<hw>Sem"blance</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F. See <er>Semblable</er>, <pos>a.</pos>]</ety> <sn>1.</sn> <def>Seeming; appearance; show; figure; form.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Thier <qex>semblance</qex> kind, and mild their gestures were.</q> <rj><qau>Fairfax.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Likeness; resemblance, actual or apparent; similitude; <as>as, the <ex>semblance</ex> of worth; <ex>semblance</ex> of virtue</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Only <qex>semblances</qex> or imitations of shells.</q> <rj><qau>Woodward.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Semblant</ent><br/
<hw>Sem"blant</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[F. <ets>semblant</ets>, p. pr.]</ety> <sn>1.</sn> <def>Like; resembling.</def> <mark>[Obs.]</mark>  <rj><au>Prior.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Seeming, rather than real; apparent.</def> <mark>[R.]</mark>  <rj><au>Carlyle.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Semblant</ent><br/
<hw>Sem"blant</hw>, <pos>n.</pos> <ety>[F.]</ety> <sn>1.</sn> <def>Show; appearance; figure; semblance.</def> <mark>[Obs.]</mark>  <rj><au>Spenser.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>His flatterers made <qex>semblant</qex> of weeping.</q> <rj><qau>Chaucer.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>The face.</def> <mark>[Obs.]</mark>  <rj><au>Wyclif (Luke xxiv. 5).</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Semblative</ent><br/
<hw>Sem"bla*tive</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Resembling.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>And all is <qex>semblative</qex> a woman's part.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Semble</ent><br/
<hw>Sem"ble</hw> <pr>(?)</pr>, <pos>v. i.</pos> <ety>[F. <ets>sembler</ets>. See <er>Semblable</er>, <pos>a.</pos>]</ety> <sn>1.</sn> <def>To imitate; to make a representation or likeness.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>Where <qex>sembling</qex> art may carve the fair effect.</q> <rj><qau>Prior.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Law)</fld> <def>It seems; -- chiefly used impersonally in reports and judgments to express an opinion in reference to the law on some point not necessary to be decided, and not intended to be definitely settled in the cause.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Semble</ent><br/
<hw>Sem"ble</hw>, <pos>a.</pos> <def>Like; resembling.</def> <mark>[Obs.]</mark>  <rj><au>T. Hudson.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sembling</ent><br/
<hw>Sem"bling</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Cf. <er>Assemble</er>.]</ety> <fld>(Zool.)</fld> <def>The practice of attracting the males of Lepidoptera or other insects by exposing the female confined in a cage.</def><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ It is often adopted by collectors in order to procure specimens of rare species.</note><br/
[<source>1913 Webster</source>]</p>

<p><ent>seme</ent><br/
<hw>seme</hw> <pr>(s<emac/m)</pr>, <pos>n.</pos> <ety>[Gr. <grk>sh^ma</grk> a sign.]</ety> <sn>1.</sn> <fld>(Linguistics)</fld> <def>A linguistic sign.</def><br/
[<source>PJC</source>]</p>

<p><sn>2.</sn> <fld>(Linguistics)</fld> <def>A basic component of meaning of a morpheme, especially one which cannot be decomposed into more basic components; a primitive concept.</def><br/
[<source>PJC</source>]</p>

<p><ent>Seme</ent><br/
||<hw>Se*m<eacute/"</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[F. <ets>sem<eacute/</ets>, fr. <ets>semer</ets> to sow.]</ety> <fld>(Her.)</fld> <def>Sprinkled or sown; -- said of field, or a charge, when strewed or covered with small charges.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Semiography</ent><br/
<ent>Semeiography</ent><br/
<mhw>{ <hw>Se`mei*og"ra*phy</hw> <pr>(s<emac/`m<isl/*<ocr/g"r<adot/*f<ycr/)</pr>, <it>or</it> <hw>Se`mi*og"ra*phy</hw> <pr>(s<emac/`m<icr/*<ocr/g"r<adot/*f<ycr/)</pr>, }</mhw> <pos>n.</pos> <ety>[Gr. <grk>shmei^on</grk> sign + <ets>-graphy</ets>.]</ety> <fld>(Med.)</fld> <def>A description of the signs of disease.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Semiologioal</ent><br/
<ent>Semeiological</ent><br/
<mhw>{ <hw>Se`mei*o*log"ic*al</hw> <pr>(?)</pr>, <it>or</it> <hw>Se`mi*o*log"io*al</hw>  }</mhw>, <pos>a.</pos> <def>Of or pertaining to the science of signs, or the systematic use of signs; <as>as, a <ex>semeiological</ex> classification of the signs or symptoms of disease; a <ex>semeiological</ex> arrangement of signs used as signals.</as></def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Semiology</ent><br/
<ent>Semeiology</ent><br/
<mhw>{ <hw>Se`mei*ol"o*gy</hw> <pr>(s<emac/`m<esl/*<ocr/l"<osl/*j<ycr/ <it>or</it>  s<emac/`m<isl/*<ocr/l"<osl/*j<ycr/)</pr>, <it>or</it> <hw>Se`mi*ol"o*gy</hw>  <pr>(s<ecr/m`<esl/*<ocr/l"<osl/*j<ycr/)</pr> }</mhw>, <pos>n.</pos> <ety>[Gr. <grk>shmei^on</grk> a mark, a sign + <ets>-logy</ets>.]</ety> <def>The study of signs as an element of communication; the analysis of systems of communication; -- also called <altname>semiotics</altname>.</def> <specif>Specifically:</specif> <sd>(a)</sd> <fld>(Med.)</fld> <def>The science of the signs or symptoms of disease; symptomatology.</def> <sd>(b)</sd> <def>The art of using signs in signaling.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Semiotic</ent><br/
<ent>Semeiotic</ent><br/
<mhw>{ <hw>Se`mei*ot"ic</hw> <pr>(s<emac/`m<esl/*<ocr/t"<icr/k <it>or</it>  s<ecr/`m<esl/*<ocr/t"<icr/k)</pr>, <it>or</it> <hw>Se`mi*ot"ic</hw>  }</mhw>, <pos>a.</pos> <ety>[Gr. <grk>shmeiwtiko`s</grk>, fr. <grk>shmei^on</grk> a mark, a sign.]</ety> <sn>1.</sn> <def>Relating to signs or indications; pertaining to the language of signs, or to language generally as indicating thought.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Med.)</fld> <def>Of or pertaining to the signs or symptoms of diseases.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Semiotics</ent><br/
<ent>Semeiotics</ent><br/
<mhw>{ <hw>Se`mei*ot"ics</hw> <pr>(s<emac/`m<isl/*<ocr/t"<icr/ks)</pr>, <it>or</it> <hw>Se`mi*ot"ics</hw>  }</mhw>, <pos>n.</pos> <sn>1.</sn> <def>The study of signs as an element of communication; the analysis of systems of communication; -- also called <altname>semiology</altname>.</def><br/
[<source>PJC</source>]</p>

<p><sn>2.</sn> <def>a theory of signs and symbols, including as branches <er>semantics</er>, <er>pragmatics</er> and <er>syntactics</er>.</def> <au>[RHUD]</au><br/
[<source>PJC</source>]</p>

<p><ent>Semele</ent><br/
<hw>Sem"e*le</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L., fr. Gr. <?/<?/<?/.]</ety> <fld>(Gr. Myth.)</fld> <def>A daughter of Cadmus, and by Zeus mother of Bacchus.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Semen</ent><br/
||<hw>Se"men</hw> <pr>(s<emac/"m<ecr/n)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Semina</plw> <pr>(s<ecr/m"<icr/*n<adot/)</pr>.</plu> <ety>[L., from the root of <ets>serere</ets>, <ets>satum</ets>, to sow. See <er>Sow</er> to scatter seed.]</ety> <sn>1.</sn> <fld>(Bot.)</fld> <def>The seed of plants.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Physiol.)</fld> <def>The seed or fecundating fluid of male animals; sperm. It is a white or whitish viscid fluid secreted by the testes, characterized by the presence of spermatozoids to which it owes its generative power.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><mcol><col><b>Semen contra</b></col>, <it>or</it> <col><b>Semen cinae</b></col> <it>or</it> <col><b>Semen cynae</b></col></mcol>, <cd>a strong aromatic, bitter drug, imported from <city>Aleppo</city> and Barbary, said to consist of the leaves, peduncles, and unexpanded flowers of various species of <gen>Artemisia</gen>; wormseed.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Semeniferous</ent><br/
<hw>Sem`e*nif"er*ous</hw> <pr>(s<ecr/m`<esl/*n<icr/f"<etil/r*<ucr/s)</pr>, <pos>a.</pos> <fld>(Biol.)</fld> <def>Seminiferous.</def><br/
[<source>1913 Webster</source>]</p>

<p><-- p. 1308 --></p>

<p><ent>Semester</ent><br/
<hw>Se*mes"ter</hw> <pr>(s<esl/*m<ecr/s"t<etil/r)</pr>, <pos>n.</pos> <ety>[G., from L. <ets>semestris</ets> half-yearly; <ets>sex</ets> six + <ets>mensis</ets> a month.]</ety> <def>A period of six months; especially, a term in a college or university which divides the year into two terms.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Semi-</ent><br/
<hw>Sem"i-</hw> <pr>(?)</pr>. <ety>[L. <ets>semi</ets>; akin to Gr. <?/<?/<?/, Skr. <ets>s<amac/mi</ets>-, AS. <ets>s<amac/m</ets>-, and prob. to E.  <ets>same</ets>, from the division into two parts of the same size.  Cf. <er>Hemi-</er>, <er>Sandelend</er>.]</ety> <def>A prefix signifying <xex>half</xex>, and sometimes <xex>partly</xex> or <xex>imperfectly</xex>; <as>as, <ex>semi</ex>annual, half yearly; <ex>semi</ex>transparent, imperfectly transparent</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ The prefix <xex>semi</xex> is joined to another word either with the hyphen or without it. In this book the hyphen is omitted except before a capital letter; as, <xex>semi</xex>acid, <xex>semi</xex>aquatic, <xex>semi</xex>-Arian, <xex>semi</xex>axis, <xex>semi</xex>calcareous.</note><br/
[<source>1913 Webster</source>]</p>

<p><ent>Semiacid</ent><br/
<hw>Sem`i*ac"id</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Slightly acid; subacid.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Semiacidified</ent><br/
<hw>Sem`i*a*cid"i*fied</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Half acidified.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Semiadherent</ent><br/
<hw>Sem`i*ad*her"ent</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Adherent part way.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Semiamplexicaul</ent><br/
<hw>Sem`i*am*plex"i*caul</hw> <pr>(?)</pr>, <pos>a.</pos> <fld>(Bot.)</fld> <def>Partially amplexicaul; embracing the stem half round, as a leaf.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Semiangle</ent><br/
<hw>Sem"i*an`gle</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Geom.)</fld> <def>The half of a given, or measuring, angle.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Semiannual</ent><br/
<hw>Sem`i*an"nu*al</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Half-yearly.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Semiannually</ent><br/
<hw>Sem`i*an"nu*al*ly</hw>, <pos>adv.</pos> <def>Every half year.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Semiannular</ent><br/
<hw>Sem`i*an"nu*lar</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Having the figure of a half circle; forming a semicircle.</def>  <rj><au>Grew.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Semi-Arian</ent><br/
<hw>Sem`i-A"ri*an</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[See <er>Arian</er>.]</ety> <fld>(Eccl. Hist.)</fld> <def>A member of a branch of the Arians which did not acknowledge the Son to be consubstantial with the Father, that is, of the same substance, but admitted him to be of a like substance with the Father, not by nature, but by a peculiar privilege.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Semi-Arian</ent><br/
<hw>Sem"i-A"ri*an</hw>, <pos>a.</pos> <def>Of or pertaining to Semi-Arianism.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Semi-Arianism</ent><br/
<hw>Sem`i-A"ri*an*ism</hw> <pr>(?)</pr>, <pos>n.</pos> <def>The doctrines or tenets of the Semi-Arians.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Semiaxis</ent><br/
<hw>Sem`i*ax"is</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Geom.)</fld> <def>One half of the axis of an <?/llipse or other figure.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Semibarbarian</ent><br/
<hw>Sem`i*bar*ba"ri*an</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Half barbarous; partially civilized.</def> -- <def2><pos>n.</pos> <def>One partly civilized.</def></def2><br/
[<source>1913 Webster</source>]</p>

<p><ent>Semibarbaric</ent><br/
<hw>Sem`i*bar*bar"ic</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Half barbarous or uncivilized; <as>as, <ex>semibarbaric</ex> display</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Semibarbarism</ent><br/
<hw>Sem`i*bar"ba*rism</hw> <pr>(?)</pr>, <pos>n.</pos> <def>The quality or state of being half barbarous or uncivilized.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Semibarbarous</ent><br/
<hw>Sem`i*bar"ba*rous</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Half barbarous.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Semibreve</ent><br/
<hw>Sem"i*breve`</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Pref. <ets>semi-</ets> + <ets>breve</ets>: cf. F. <ets>semi-breve</ets>, It. <ets>semibreve</ets>.]</ety> <altsp>[Formerly written <asp>semibref</asp>.]</altsp> <fld>(Mus.)</fld> <def>A note of half the time or duration of the breve; -- now usually called a <xex>whole note</xex>. It is the longest note in general use.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Semibrief</ent><br/
<hw>Sem"i*brief`</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Mus.)</fld> <def>A semibreve.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Semibull</ent><br/
<hw>Sem"i*bull`</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(R.C.Ch.)</fld> <def>A bull issued by a pope in the period between his election and coronation.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Semicalcareous</ent><br/
<hw>Sem`i*cal*ca"re*ous</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Half or partially calcareous; <as>as, a <ex>semicalcareous</ex> plant</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Semicalcined</ent><br/
<hw>Sem`i*cal*cined"</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Half calcined; <as>as, <ex>semicalcined</ex> iron</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Semicastrate</ent><br/
<hw>Sem`i*cas"trate</hw> <pr>(?)</pr>, <pos>v. t.</pos> <def>To deprive of one testicle.</def> -- <wordforms><wf>Sem`i*cas*tra"tion</wf> <pr>(#)</pr>,<pos>n.</pos></wordforms><br/
[<source>1913 Webster</source>]</p>

<p><ent>Semicentennial</ent><br/
<hw>Sem`i*cen*ten"ni*al</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Of or pertaining to half of a century, or a period of fifty years; <as>as, a <ex>semicentennial</ex> commemoration</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Semicentennial</ent><br/
<hw>Sem`i*cen*ten"ni*al</hw>, <pos>n.</pos> <def>A fiftieth anniversary.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Semichaotic</ent><br/
<hw>Sem`i*cha*ot"ic</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Partially chaotic.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Semichorus</ent><br/
<hw>Sem"i*cho`rus</hw>, <pos>n.</pos> <fld>(Mus.)</fld> <def>A half chorus; a passage to be sung by a selected portion of the voices, as the female voices only, in contrast with the full choir.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Semi-Christianized</ent><br/
<hw>Sem`i-Chris"tian*ized</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Half Christianized.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Semicircle</ent><br/
<hw>Sem"i*cir`cle</hw> <pr>(?)</pr>, <pos>n.</pos> <sn>1.</sn> <sd>(a)</sd> <def>The half of a circle; the part of a circle bounded by its diameter and half of its circumference.</def> <sd>(b)</sd> <def>A semicircumference.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A body in the form of half of a circle, or half of a circumference.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>An instrument for measuring angles.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Semicircled</ent><br/
<hw>Sem"i*cir`cled</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Semicircular.</def>  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Semicircular</ent><br/
<hw>Sem`i*cir"cu*lar</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Having the form of half of a circle.</def>  <rj><au>Addison.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Semicircular canals</b></col> <fld>(Anat.)</fld>, <cd>certain canals of the inner ear. See under <er>Ear</er>.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Semi circumference</ent><br/
<hw>Sem`i cir*cum"fer*ence</hw> <pr>(?)</pr>, <pos>n.</pos> <def>Half of a circumference.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Semicirque</ent><br/
<hw>Sem"i*cirque</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A semicircular hollow or opening among trees or hills.</def>  <rj><au>Wordsworth.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Semicolon</ent><br/
<hw>Sem"i*co`lon</hw> <pr>(?)</pr>, <pos>n.</pos> <def>The punctuation mark [;] indicating a separation between parts or members of a sentence more distinct than that marked by a comma.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Semicolumn</ent><br/
<hw>Sem"i*col`umn</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A half column; a column bisected longitudinally, or along its axis.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Semicolumnar</ent><br/
<hw>Sem`i*co*lum"nar</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Like a semicolumn; flat on one side and round on the other; imperfectly columnar.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Semicompact</ent><br/
<hw>Sem`i*com*pact"</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Half compact; imperfectly indurated.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Semiconscious</ent><br/
<hw>Sem`i*con"scious</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Half conscious; imperfectly conscious.</def>  <rj><au>De Quincey.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Semicope</ent><br/
<hw>Sem"i*cope`</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A short cope, or an inferier kind of cope.</def> <mark>[Obs.]</mark>  <rj><au>Chaucer.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Semi crustaceous</ent><br/
<hw>Sem`i crus*ta"ceous</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Half crustaceous; partially crustaceous.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Semicrystalline</ent><br/
<hw>Sem`i*crys"tal*line</hw> <pr>(?)</pr>, <pos>a.</pos> <fld>(Min.)</fld> <def>Half crystalline; -- said of certain cruptive rocks composed partly of crystalline, partly of amorphous matter.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Semicubical</ent><br/
<hw>Sem`i*cu"bic*al</hw> <pr>(?)</pr>, <pos>a.</pos> <fld>(Math.)</fld> <def>Of or pertaining to the square root of the cube of a quantity.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Semicubical parabola</b></col>, <cd>a curve in which the ordinates are proportional to the square roots of the cubes of the abscissas.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Semicupium</ent><br/
<ent>Semicubium</ent><br/
<mhw>{ <hw>Sem`i*cu"bi*um</hw> <pr>(?)</pr>, <hw>Sem`i*cu"pi*um</hw> <pr>(?)</pr>, }</mhw> <pos>n.</pos> <ety>[LL., fr. L. <ets>semi</ets> half + <ets>cupa</ets> tub, cask.]</ety> <def>A half bath, or one that covers only the lewer extremities and the hips; a sitz-bath; a half bath, or hip bath.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Semicylyndrical</ent><br/
<ent>Semicylindric</ent><br/
<mhw>{ <hw>Sem`i*cy*lin"dric</hw> <pr>(?)</pr>, <hw>Sem`i*cy*lyn"dric*al</hw> <pr>(?)</pr> }</mhw> <pos>a.</pos> <def>Half cylindrical.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Semideistical</ent><br/
<hw>Sem`i*de*is"tic*al</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Half deisticsl; bordering on deism.</def>  <rj><au>S. Miller.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Semidemiquaver</ent><br/
<hw>Sem`i*dem"i*qua`ver</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Mus.)</fld> <def>A demisemiquaver; a thirty-second note.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Semidetached</ent><br/
<hw>Sem`i*de*tached"</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Half detached; partly distinct or separate.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Semidetached house</b></col>, <cd>one of two tenements under a single roof, but separated by a party wall.</cd> <mark>[Eng.]</mark></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Semidiameter</ent><br/
<hw>Sem`i*di*am"e*ter</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Math.)</fld> <def>Half of a diameter; a right line, or the length of a right line, drawn from the center of a circle, a sphere, or other curved figure, to its circumference or periphery; a radius.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Semidiapason</ent><br/
<hw>Sem`i*di`a*pa"son</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Mus.)</fld> <def>An imperfect octave.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Semidiapente</ent><br/
<hw>Sem`i*di`a*pen"te</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Mus.)</fld> <def>An imperfect or diminished fifth.</def>  <rj><au>Busby.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Semidiaphaneity</ent><br/
<hw>Sem`i*di`a*pha*ne"i*ty</hw> <pr>(?)</pr>, <pos>n.</pos> <def>Half or imperfect transparency; translucency.</def> <mark>[R.]</mark>  <rj><au>Boyle.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Semidiaphanous</ent><br/
<hw>Sem`i*di*aph"a*nous</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Half or imperfectly transparent; translucent.</def>  <rj><au>Woodward.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Semidiatessaron</ent><br/
<hw>Sem`i*di`a*tes"sa*ron</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Mus.)</fld> <def>An imperfect or diminished fourth.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Semi-Diesel</ent><br/
<hw>Sem`i-Die"sel</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Designating an internal-combustion engine of a type resembling the Diesel engine in using as fuel heavy oil which is injected in a spray just before the end of the compression stroke and is fired without electrical ignition. The fuel is sprayed into an iron box (called a <xex>hot bulb</xex> or <xex>hot pot</xex>) opening into the combustion chamber, and heated for ignition by a blast-lamp until the engine is running, when it is, ordinarily, kept red hot by the heat of combustion.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Semiditone</ent><br/
<hw>Sem`i*di"tone`</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Pref. <ets>semi-</ets> + <ets>ditone</ets>: cf. It. <ets>semiditono</ets>.  Cf. <er>Hemiditone</er>.]</ety> <fld>(Gr. Mus.)</fld> <def>A lesser third, having its terms as 6 to 5; a hemiditone.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Semidiurnal</ent><br/
<hw>Sem`i*di*ur"nal</hw> <pr>(?)</pr>, <pos>a.</pos> <sn>1.</sn> <def>Pertaining to, or accomplished in, half a day, or twelve hours; occurring twice every day.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Pertaining to, or traversed in, six hours, or in half the time between the rising and setting of a heavenly body; <as>as, a <ex>semidiurnal</ex> arc</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Semidome</ent><br/
<hw>Sem"i*dome`</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Arch.)</fld> <def>A roof or ceiling covering a semicircular room or recess, or one of nearly that shape, as the apse of a church, a niche, or the like. It is approximately the quarter of a hollow sphere.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Semidouble</ent><br/
<hw>Sem"i*dou`ble</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Eccl.)</fld> <def>An office or feast celebrated with less solemnity than the double ones. See <er>Double</er>, <pos>n.</pos>, 8.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Semidouble</ent><br/
<hw>Sem`i*dou"ble</hw>, <pos>a.</pos> <fld>(Bot.)</fld> <def>Having the outermost stamens converted into petals, while the inner ones remain perfect; -- said of a flower.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Semifable</ent><br/
<hw>Sem"i*fa`ble</hw> <pr>(?)</pr>, <pos>n.</pos> <def>That which is part fable and part truth; a mixture of truth and fable.</def>  <rj><au>De Quincey.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Semiflexed</ent><br/
<hw>Sem"i*flexed`</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Half bent.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Semifloret</ent><br/
<hw>Sem"i*flo`ret</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Bot.)</fld> <def>See <er>Semifloscule</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Semifloscular</ent><br/
<hw>Sem`i*flos"cu*lar</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Semiflosculous.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Semifloscule</ent><br/
<hw>Sem"i*flos`cule</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Bot.)</fld> <def>A floscule, or florest, with its corolla prolonged into a strap-shaped petal; -- called also <altname>semifloret</altname>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Semiflosculous</ent><br/
<hw>Sem`i*flos"cu*lous</hw> <pr>(?)</pr>, <pos>a.</pos> <fld>(Bot.)</fld> <def>Having all the florets ligulate, as in the dandelion.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Semifluid</ent><br/
<hw>Sem`i*flu"id</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Imperfectly fluid.</def> -- <def2><pos>n.</pos> <def>A semifluid substance.</def></def2><br/
[<source>1913 Webster</source>]</p>

<p><ent>Semiform</ent><br/
<hw>Sem"i*form`</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A half form; an imperfect form.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Semiformed</ent><br/
<hw>Sem"i*formed`</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Half formed; imperfectly formed; <as>as, <ex>semiformed</ex> crystals</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Semiglutin</ent><br/
<hw>Sem`i*glu"tin</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Physiol. Chem.)</fld> <def>A peptonelike body, insoluble in alcohol, formed by boiling collagen or gelatin for a long time in water. <xex>Hemicollin</xex>, a like body, is also formed at the same time, and differs from semiglutin by being partly soluble in alcohol.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Semihistorical</ent><br/
<hw>Sem`i*his*tor"i*cal</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Half or party historical.</def>  <rj><au>Sir G. C. Lewis.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Semihoral</ent><br/
<hw>Sem`i*ho"ral</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Half-hourly.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Semiindurated</ent><br/
<hw>Sem`i*in"du*ra`ted</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Imperfectly indurated or hardened.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Semilapidified</ent><br/
<hw>Sem`i*la*pid"i*fied</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Imperfectly changed into stone.</def>  <rj><au>Kirwan.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Semilens</ent><br/
<hw>Sem"i*lens`</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Opt.)</fld> <def>The half of a lens divided along a plane passing through its axis.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Semilenticular</ent><br/
<hw>Sem`i*len*tic"u*lar</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Half lenticular or convex; imperfectly resembling a lens.</def>  <rj><au>Kirwan.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Semiligneous</ent><br/
<hw>Sem`i*lig"ne*ous</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Half or partially ligneous, as a stem partly woody and partly herbaceous.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Semiliquid</ent><br/
<hw>Sem`i*liq"uid</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Half liquid; semifluid.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Semiliquidity</ent><br/
<hw>Sem`i*li*quid"i*ty</hw> <pr>(?)</pr>, <pos>n.</pos> <def>The quality or state of being semiliquid; partial liquidity.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Semilogical</ent><br/
<hw>Sem`i*log"ic*al</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Half logical; partly logical; said of fallacies.</def>  <rj><au>Whately.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Semilor</ent><br/
<hw>Sem"i*lor</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Cf. G. <ets>similor</ets>, <ets>semilor</ets>.]</ety> <def>A yellowish alloy of copper and zinc. See <er>Simplor</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Semilunar</ent><br/
<hw>Sem`i*lu"nar</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Shaped like a half moon.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Semilunar bone</b></col> <fld>(Anat.)</fld>, <cd>a bone of the carpus; the lunar. See <er>Lunar</er>, <pos>n.</pos></cd> -- <mcol><col><b>Semilunar valves</b></col>, <it>or</it> <col><b>Sigmoid valves</b></col></mcol> <fld>(Anat.)</fld>, <cd>the valves at the beginning of the aorta and of the pulmonary artery which prevent the blood from flowing back into the ventricle.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Semilunar</ent><br/
<hw>Sem`i*lu"nar</hw>, <pos>n.</pos> <fld>(Anat.)</fld> <def>The semilunar bone.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Semilunary</ent><br/
<hw>Sem`i*lu"na*ry</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Semilunar.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Semilunate</ent><br/
<hw>Sem`i*lu"nate</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Semilunar.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Semilune</ent><br/
<hw>Sem"i*lune`</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Geom.)</fld> <def>The half of a lune.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Semimetal</ent><br/
<hw>Sem"i*met`al</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Chem.)</fld> <def>An element possessing metallic properties in an inferior degree and not malleable, as arsenic, antimony, bismuth, molybdenum, uranium, etc.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Semimetallic</ent><br/
<hw>Sem`i*me*tal"lic</hw> <pr>(?)</pr>, <pos>a.</pos> <fld>(Chem.)</fld> <def>Of or pertaining to a semimetal; possessing metallic properties in an inferior degree; resembling metal.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Semimonthly</ent><br/
<hw>Sem`i*month"ly</hw> <pr>(?)</pr> <pos>a.</pos> <def>Coming or made twice in a month; <as>as, <ex>semimonthly</ex> magazine; a <ex>semimonthly</ex> payment.</as></def> -- <def2><pos>n.</pos> <def>Something done or made every half month; esp., a semimonthly periodical.</def></def2> -- <def2><pos>adv.</pos> <def>In a semimonthly manner; at intervals of half a month.</def></def2><br/
[<source>1913 Webster</source>]</p>

<p><ent>Semimute</ent><br/
<hw>Sem`i*mute"</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Having the faculty of speech but imperfectly developed or partially lost.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Semimute</ent><br/
<hw>Sem"i*mute`</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A semimute person.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Seminal</ent><br/
<hw>Sem"i*nal</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>seminalis</ets>, fr. <ets>semen</ets>, <ets>seminis</ets>, seed, akin to <ets>serere</ets> to sow: cf. F. <ets>seminal</ets>. See <er>Sow</er> to scatter seed.]</ety> <sn>1.</sn> <def>Pertaining to, containing, or consisting of, seed or semen; <as>as, the <ex>seminal</ex> fluid</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Contained in seed.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <specif>Hence:</specif> <def>Holding the first place in a series of developed results or consequents; serving as a source, or first principle; giving rise to related ideas or results; germinal; radical; primary; original; <as>as, <ex>seminal</ex> principles of generation; <ex>seminal</ex> virtue; a <ex>seminal</ex> discovery.</as></def><br/
[<source>1913 Webster</source> <source>+PJC</source>]</p>

<p><q>The idea of God is, beyond all question or comparison, the one great <qex>seminal</qex> principle.</q> <rj><qau>Hare.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Seminal leaf</b></col> <fld>(Bot.)</fld>, <cd>a seed leaf, or cotyleden.</cd> -- <col><b>Seminal receptacle</b></col>. <fld>(Zool.)</fld> <cd>Same as <er>Spermatheca</er>.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Seminal</ent><br/
<hw>Sem"i*nal</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A seed.</def> <mark>[Obs.]</mark>  <rj><au>Sir T. Browne.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Seminality</ent><br/
<hw>Sem`i*nal"i*ty</hw> <pr>(?)</pr>, <pos>n.</pos> <def>The quality or state of being seminal.</def>  <rj><au>Sir T. Browne.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Seminar</ent><br/
<hw>Sem`i*nar"</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[G. See <er>Seminary</er>, <pos>n.</pos>]</ety> <def>A group of students engaged, under the guidance of an instructor, in original research in a particular line of study, and in the exposition of the results by theses, lectures, etc.; -- formerly called also <altname>seminary</altname>, now seldom used in this sense.</def><br/
[<source>Webster 1913 Suppl.</source> <source>+PJC</source>]</p>

<p><ent>Seminarist</ent><br/
<ent>Seminarian</ent><br/
<mhw>{ <hw>Sem`i*na"ri*an</hw> <pr>(?)</pr>, <hw>Sem"i*na*rist</hw> <pr>(?)</pr>, }</mhw> <pos>n.</pos> <ety>[Cf. F. <ets>s<eacute/minariste</ets>.]</ety> <def>A member of, or one educated in, a seminary; specifically, an ecclesiastic educated for the priesthood in a seminary.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Seminary</ent><br/
<hw>Sem"i*na*ry</hw> <pr>(?)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Seminaries</plw> <pr>(#)</pr>.</plu> <ety>[L. <ets>seminarium</ets>, fr. <ets>seminarius</ets> belonging to seed, fr. <ets>semon</ets>, <ets>seminis</ets>, seed. See <er>Seminal</er>.]</ety> <sn>1.</sn> <def>A piece of ground where seed is sown for producing plants for transplantation; a nursery; a seed plat.</def> <mark>[Obs.]</mark>  <rj><au>Mortimer.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>But if you draw them [seedling] only for the thinning of your <qex>seminary</qex>, prick them into some empty beds.</q> <rj><qau>Evelyn.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Hence, the place or original stock whence anything is brought or produced.</def> <mark>[Obs.]</mark>  <rj><au>Woodward.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>A place of education, as a scool of a high grade, an academy, college, or university.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>Seminal state.</def> <mark>[Obs.]</mark>  <rj><au>Sir T. Browne.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <def>Fig.: A seed bed; a source.</def> <mark>[Obs.]</mark>  <rj><au>Harvey.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>6.</sn> <def>A Roman Catholic priest educated in a foreign seminary; a seminarist.</def> <mark>[Obs.]</mark>  <rj><au>Jer. Taylor.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Seminary</ent><br/
<hw>Sem"i*na*ry</hw>, <pos>a.</pos> <ety>[L. <ets>seminarius</ets>.]</ety> <def>Belonging to seed; seminal.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Seminate</ent><br/
<hw>Sem"i*nate</hw> <pr>(?)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Seminated</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Seminating</conjf>.]</vmorph> <ety>[L. <ets>seminatus</ets>, p. p. of <ets>seminare</ets> to sow, fr. <ets>semen</ets>, <ets>seminis</ets>, seed.]</ety> <def>To sow; to spread; to propagate.</def> <mark>[R.]</mark>  <rj><au>Waterhouse.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Semination</ent><br/
<hw>Sem`i*na"tion</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>seminatio</ets>: cf. F. <ets>s<eacute/mination</ets>.]</ety> <sn>1.</sn> <def>The act of sowing or spreading.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Bot.)</fld> <def>Natural dispersion of seeds.</def>  <rj><au>Martyn.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Semined</ent><br/
<hw>Sem"ined</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[See <er>Semen</er>.]</ety> <def>Thickly covered or sown, as with seeds.</def> <mark>[Obs.]</mark>  <rj><au>B. Jonson.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Seminiferous</ent><br/
<hw>Sem`i*nif"er*ous</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>semen</ets>, <ets>semenis</ets>, seed <ets>-ferous</ets>.]</ety> <fld>(Biol.)</fld> <def>Seed-bearing; producing seed; pertaining to, or connected with, the formation of semen; <as>as, <ex>seminiferous</ex> cells or vesicles</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Seminifical</ent><br/
<ent>Seminific</ent><br/
<mhw>{ <hw>Sem`i*nif"ic</hw> <pr>(?)</pr>, <hw>Sem`i*nif"ic*al</hw> <pr>(?)</pr>, }</mhw> <pos>a.</pos> <ety>[L. <ets>semen</ets>, <ets>seminis</ets>, seed + <ets>facere</ets> to make.]</ety> <fld>(Biol.)</fld> <def>Forming or producing seed, or the male generative product of animals or of plants.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Seminification</ent><br/
<hw>Sem`i*ni*fi*ca"tion</hw> <pr>(?)</pr>, <pos>n.</pos> <def>Propagation from seed.</def> <mark>[R.]</mark>  <rj><au>Sir M. Hale.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Seminist</ent><br/
<hw>Sem"i*nist</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Biol.)</fld> <def>A believer in the old theory that the newly created being is formed by the admixture of the seed of the male with the supposed seed of the female.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Seminoles</ent><br/
<hw>Sem"i*noles</hw> <pr>(?)</pr>, <pos>n. pl.</pos>; <sing>sing. <singw>Seminole</singw> <pr>(<?/)</pr></sing>. <fld>(Ethnol.)</fld> <def>A tribe of Indians who formerly occupied Florida, where some of them still remain. They belonged to the Creek Confideration.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Seminose</ent><br/
<hw>Sem"i*nose`</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>semen</ets> seed + <ets>glucose</ets>.]</ety> <fld>(Chem.)</fld> <def>A carbohydrate of the glucose group found in the thickened endosperm of certain seeds, and extracted as yellow sirup having a sweetish-bitter taste.</def>
<-- same as D-mannose; C6H12O6, a hexose opticaly isomeric with glucose. --><br/
[<source>1913 Webster</source>]</p>

<p><ent>Seminude</ent><br/
<hw>Sem`i*nude"</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Partially nude; half naked.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Seminymph</ent><br/
<hw>Sem"i*nymph`</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Zool.)</fld> <def>The pupa of insects which undergo only a slight change in passing to the imago state.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Semioccasionally</ent><br/
<hw>Sem`i*oc*ca"sion*al*ly</hw> <pr>(?)</pr>, <pos>adv.</pos> <def>Once in a while; on rare occasions.</def> <mark>[Colloq. U. S.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Semiofficial</ent><br/
<hw>Sem`i*of*fi"cial</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Half official; having some official authority or importance; <as>as, a <ex>semiofficial</ex> statement</as>.</def> -- <wordforms><wf>Sem`i*of*fi"cial*ly</wf>, <pos>adv.</pos></wordforms><br/
[<source>1913 Webster</source>]</p>

<p><ent>Semiological</ent><br/
<ent>Semiology</ent><br/
<ent>Semiography</ent><br/
<mhw>{ <hw>Se`mi*og"ra*phy</hw> <pr>(?)</pr>, <hw>Se`mi*ol"o*gy</hw> <pr>(?)</pr>, <hw>Se`mi*o*log"ic*al</hw> <pr>(?)</pr> }</mhw>. <def>Same as <er>Semeiography</er>, <er>Semeiology</er>, <er>Semeiological</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Semiopacous</ent><br/
<hw>Sem`i*o*pa"cous</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Semiopaque.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Semiopal</ent><br/
<hw>Sem"i*o`pal</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Min.)</fld> <def>A variety of opal not possessing opalescence.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Semiopaque</ent><br/
<hw>Sem`i*o*paque"</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Half opaque; only half transparent.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Semiorbicular</ent><br/
<hw>Sem`i*or*bic"u*lar</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Having the shape of a half orb or sphere.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Semiotic</ent><br/
<hw>Se`mi*ot"ic</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Same as <er>Semeiotic</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Semiotics</ent><br/
<hw>Se`mi*ot"ics</hw> <pr>(?)</pr>, <pos>n.</pos> <def>Same as <er>Semeiotics</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Semioval</ent><br/
<hw>Sem`i*o"val</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Half oval.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Semiovate</ent><br/
<hw>Sem`i*o"vate</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Half ovate.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Semioxygenated</ent><br/
<hw>Sem`i*ox"y*gen*a`ted</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Combined with oxygen only in part.</def>  <rj><au>Kirwan.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Semipagan</ent><br/
<hw>Sem`i*pa"gan</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Half pagan.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Semipalmated</ent><br/
<ent>Semipalmate</ent><br/
<mhw>{ <hw>Sem`i*pal"mate</hw> <pr>(?)</pr>, <hw>Sem`i*pal"ma*ted</hw> <pr>(?)</pr>, }</mhw> <pos>a.</pos> <fld>(Zool.)</fld> <def>Having the anterior toes joined only part way down with a web; half-webbed; <as>as, a <ex>semipalmate</ex> bird or foot</as>. See <xex>Illust.</xex> <it>k</it> under <er>Aves</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Semiparabola</ent><br/
<hw>Sem`i*pa*rab"o*la</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Geom.)</fld> <def>One branch of a parabola, being terminated at the principal vertex of the curve.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Semiped</ent><br/
<hw>Sem"i*ped</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>semipes</ets>, <ets>semipedis</ets>; pref.  <ets>semi-</ets> half + <ets>pes</ets>, <ets>pedis</ets>, a foot.]</ety> <fld>(Pros.)</fld> <def>A half foot in poetry.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Semipedal</ent><br/
<hw>Se*mip"e*dal</hw> <pr>(?)</pr>, <pos>a.</pos> <fld>(Pres.)</fld> <def>Containing a half foot.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Semi-Pelagian</ent><br/
<hw>Sem`i-Pe*la"gi*an</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Eccl. Hist.)</fld> <def>A follower of John Cassianus, a French monk (died about 448), who modified the doctrines of Pelagius, by denying human merit, and maintaining the necessity of the Spirit's influence, while, on the other hand, he rejected the Augustinian doctrines of election, the inability of man to do good, and the certain perseverance of the saints.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Semi-Pelagian</ent><br/
<hw>Sem`i-Pe*la"gi*an</hw>, <pos>a.</pos> <def>Of or pertaining to the Semi-Pelagians, or their tenets.</def><br/
[<source>1913 Webster</source>]</p>

<p><-- p. 1309 --></p>

<p><ent>Semi-Pelagianism</ent><br/
<hw>Sem`i-Pe*la"gi*an*ism</hw> <pr>(s<ecr/m`<icr/*p<esl/*l<amac/"j<icr/*<ait/n*<icr/z'm)</pr>, <pos>n.</pos> <def>The doctrines or tenets of the Semi-Pelagians.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Semipellucid</ent><br/
<hw>Sem`i*pel*lu"cid</hw> <pr>(s<ecr/m`<icr/*p<ecr/l*l<umac/"s<icr/d)</pr>, <pos>a.</pos> <def>Half clear, or imperfectly transparent; <as>as, a <ex>semipellucid</ex> gem</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Semipellucidity</ent><br/
<hw>Sem`i*pel`lu*cid"i*ty</hw> <pr>(s<ecr/m`<icr/*p<ecr/l`l<usl/*s<icr/d"<icr/*t<ycr/)</pr>, <pos>n.</pos> <def>The quality or state of being imperfectly transparent.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Semipenniform</ent><br/
<hw>Sem`i*pen"ni*form</hw> <pr>(s<ecr/m`<icr/*p<ecr/n"n<icr/*f<ocir/rm)</pr>, <pos>a.</pos> <fld>(Anat.)</fld> <def>Half or partially penniform; <as>as, a <ex>semipenniform</ex> muscle</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Semipermanent</ent><br/
<hw>Sem`i*per"ma*nent</hw>, <pos>n.</pos> <def>Half or partly permanent.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Semiperspicuous</ent><br/
<hw>Sem`i*per*spic"u*ous</hw> <pr>(s<ecr/m`<icr/*p<etil/r*sp<icr/k"<usl/*<ucr/s)</pr>, <pos>a.</pos> <def>Half transparent; imperfectly clear; semipellucid.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Semiphlogisticated</ent><br/
<hw>Sem`i*phlo*gis"ti*ca`ted</hw> <pr>(s<ecr/m`<icr/*fl<osl/*j<icr/s"t<icr/*k<amac/`t<ecr/d)</pr>, <pos>a.</pos> <fld>(Old Chem.)</fld> <def>Partially impregnated with phlogiston.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Semiplume</ent><br/
<hw>Sem"i*plume`</hw> <pr>(s<ecr/m`<icr/*pl<umac/m)</pr>, <pos>n.</pos> <fld>(Zool.)</fld> <def>A feather which has a plumelike web, with the shaft of an ordinary feather.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Semiprecious</ent><br/
<hw>Sem`i*pre"cious</hw> <pr>(s<ecr/m`<icr/*pr<ecr/sh"<ucr/s)</pr>, <pos>a.</pos> <def>Somewhat precious; <as>as, <ex>semiprecious</ex> stones or metals</as>.  Used mostly of gemstones used in jewelry, such as amethyst, garnet, or iolite, which are sufficiently rare to have commercial value, but are not considered as <contr>precious</contr>, as are the diamond, emerald, and ruby.</def><br/
[<source>1913 Webster</source> <source>+PJC</source>]</p>

<p><ent>Semiproof</ent><br/
<hw>Sem"i*proof`</hw> <pr>(s<ecr/m"<icr/*pr<oomac/f`)</pr>, <pos>n.</pos> <def>Half proof; evidence from the testimony of a single witness.</def> <mark>[Obs.]</mark>  <rj><au>Bailey.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Semi pupa</ent><br/
<hw>Sem`i pu"pa</hw> <pr>(s<ecr/m`<icr/*p<umac/"p<adot/)</pr>, <pos>n.</pos> <fld>(Zool.)</fld> <def>The young of an insect in a stage between the larva and pupa.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Semiquartile</ent><br/
<ent>Semiquadrate</ent><br/
<mhw>{ <hw>Sem"i*quad`rate</hw> <pr>(s<ecr/m"<icr/*kw<ocr/d`r<asl/t)</pr>, <hw>Sem"i*quar`tile</hw> <pr>(s<ecr/m"<icr/*kw<add/r`t<icr/l)</pr>, }</mhw> <pos>n.</pos> <fld>(Astrol.)</fld> <def>An aspect of the planets when distant from each other the half of a quadrant, or forty-five degrees, or one sign and a half.</def>  <rj><au>Hutton.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Semiquaver</ent><br/
<hw>Sem"i*qua`ver</hw> <pr>(s<ecr/m"<icr/*kw<amac/`v<etil/r)</pr>, <pos>n.</pos> <fld>(Mus.)</fld> <def>A note of half the duration of the quaver; -- now usually called a <altname>sixteenth note</altname>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Semiquintile</ent><br/
<hw>Sem"i*quin`tile</hw> <pr>(s<ecr/m"<icr/*kw<icr/n`t<icr/l)</pr>, <pos>n.</pos> <fld>(Astrol.)</fld> <def>An aspect of the planets when distant from each other half of the quintile, or thirty-six degrees.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Semiradial</ent><br/
<hw>Sem`i*ra"di*al</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Half radial.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Semiradial engine</ent><br/
<hw>Semiradial engine</hw>. <fld>(Mach.)</fld> <def>See <er>Radial engine</er>, above.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Semirecondite</ent><br/
<hw>Sem`i*rec"on*dite</hw> <pr>(s<ecr/m`<icr/*r<ecr/k"<ocr/n*d<imac/t <it>or</it> s<ecr/m`<icr/*r<esl/*k<ocr/n"d<icr/t)</pr>, <pos>a.</pos> <fld>(Zool.)</fld> <def>Half hidden or half covered; said of the head of an insect when half covered by the shield of the thorax.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Semiring</ent><br/
<hw>Sem"i*ring`</hw> <pr>(s<ecr/m"<icr/*r<icr/ng`)</pr>, <pos>n.</pos> <fld>(Anat.)</fld> <def>One of the incomplete rings of the upper part of the bronchial tubes of most birds.  The semirings form an essential part of the syrinx, or musical organ, of singing birds.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Semisavage</ent><br/
<hw>Sem`i*sav"age</hw> <pr>(s<ecr/m`<icr/*s<acr/v"<asl/j)</pr>, <pos>a.</pos> <def>Half savage.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Semisavage</ent><br/
<hw>Sem"i*sav`age</hw>, <pos>n.</pos> <def>One who is half savage.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Semi-Saxon</ent><br/
<hw>Sem`i-Sax"on</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Half Saxon; -- specifically applied to the language intermediate between Saxon and English, belonging to the period 1150-1250.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Semisextile</ent><br/
<hw>Sem"i*sex"tile</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Astrol.)</fld> <def>An aspect of the planets when they are distant from each other the twelfth part of a circle, or thirty degrees.</def>  <rj><au>Hutton.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Semisolid</ent><br/
<hw>Sem`i*sol"id</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Partially solid.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Semisoun</ent><br/
<hw>Sem"i*soun</hw> <pr>(-s<oomac/n)</pr>, <pos>n.</pos> <def>A half sound; a low tone.</def> <mark>[Obs.]</mark> <ldquo/Soft he cougheth with a <xex>semisoun</xex>.<rdquo/  <rj><au>Chaucer.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Semispherical</ent><br/
<ent>Semispheric</ent><br/
<mhw>{ <hw>Sem`i*spher"ic</hw> <pr>(?)</pr>, <hw>Sem`i*spher"ic*al</hw> <pr>(?)</pr>, }</mhw> <pos>a.</pos> <def>Having the figure of a half sphere.</def>  <rj><au>Kirwan.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Semispheroidal</ent><br/
<hw>Sem`i*sphe*roid"al</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Formed like a half spheroid.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Semisteel</ent><br/
<hw>Sem"i*steel`</hw> <pr>(<?/)</pr>, <pos>n.</pos> <def>Puddled steel.</def> <mark>[U. S. ]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Semita</ent><br/
||<hw>Sem"i*ta</hw> <pr>(?)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Semitae</plw>.</plu> <ety>[L., a path.]</ety> <fld>(Zool.)</fld> <def>A fasciole of a spatangoid sea urchin.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Semitangent</ent><br/
<hw>Sem"i*tan`gent</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Geom.)</fld> <def>The tangent of half an arc.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Semite</ent><br/
<hw>Sem"ite</hw> <pr>(?)</pr>, <pos>n.</pos> <def>One belonging to the Semitic race. Also used adjectively.</def> <altsp>[Written also <asp>Shemite</asp>.]</altsp><br/
[<source>1913 Webster</source>]</p>

<p><ent>Semiterete</ent><br/
<hw>Sem`i*te*rete"</hw> <pr>(?)</pr>, <pos>a.</pos> <fld>(Nat. Hist.)</fld> <def>Half terete.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Semitertian</ent><br/
<hw>Sem`i*ter"tian</hw> <pr>(?)</pr>, <pos>a.</pos> <fld>(Med.)</fld> <def>Having the characteristics of both a tertian and a quotidian intermittent.</def> -- <def2><pos>n.</pos> <def>An intermittent combining the characteristics of a tertian and a quotidian.</def></def2><br/
[<source>1913 Webster</source>]</p>

<p><ent>Semitic</ent><br/
<hw>Sem*it"ic</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Of or pertaining to Shem or his descendants; belonging to that division of the Caucasian race which includes the Arabs, Jews, and related races.</def> <altsp>[Written also <asp>Shemitic</asp>.]</altsp><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Semitic language</b></col>, <cd>a name used to designate a group of Asiatic and African languages, some living and some dead, namely: Hebrew and Ph<oe/nician, Aramaic, Assyrian, Arabic, Ethiopic (Geez and Ampharic).</cd>  <rj><au>Encyc. Brit.</au></rj></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Semitism</ent><br/
<hw>Sem"i*tism</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A Semitic idiom; a word of Semitic origin.</def> <altsp>[Written also <asp>Shemitism</asp>.]</altsp><br/
[<source>1913 Webster</source>]</p>

<p><ent>Semitone</ent><br/
<hw>Sem"i*tone</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Pref. <ets>semi-</ets> + <ets>tone</ets>. CF. <er>Hemitone</er>.]</ety> <fld>(Mus.)</fld> <def>Half a tone; -- the name commonly applied to the smaller intervals of the diatonic scale.</def><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ There is an impropriety in the use of this word, and <xex>half step</xex> is now preferred. See <er>Tone</er>.</note>  <rj><au>J. S. Dwight.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Semitonic</ent><br/
<hw>Sem`i*ton"ic</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Of or pertaining to a semitone; consisting of a semitone, or of semitones.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Semitontine</ent><br/
<hw>Sem`i*ton*tine"</hw> <pr>(?)</pr>, <pos>a.</pos> <fld>(LIfe Insurance)</fld> <def>Lit., half-tontine; -- used to designate a form of tontine life insurance. See <er>Tontine insurance</er>.</def> -- <wordforms><wf>Sem`i*ton*tine"</wf>, <pos>n.</pos></wordforms><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Semitransept</ent><br/
<hw>Sem"i*tran`sept</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Arch.)</fld> <def>The half of a transept; <as>as, the north <ex>semitransept</ex> of a church</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Semitranslucent</ent><br/
<hw>Sem`i*trans*lu"cent</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Slightly clear; transmitting light in a slight degree.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Semitransparency</ent><br/
<hw>Sem`i*trans*par"en*cy</hw> <pr>(?)</pr>, <pos>n.</pos> <def>Imperfect or partial transparency.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Semitransparent</ent><br/
<hw>Sem`i*trans*par"ent</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Half or imperfectly transparent.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Semiverticillate</ent><br/
<hw>Sem`i*ver*tic"il*late</hw>, <pr>(<?/)</pr> <pos>a.</pos> <def>Partially verticillate.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Semivif</ent><br/
<hw>Sem"i*vif</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>semivivus</ets>.]</ety> <def>Only half alive.</def> <mark>[Obs.]</mark>  <rj><au>Piers Plowman.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Semivitreous</ent><br/
<hw>Sem`i*vit"re*ous</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Partially vitreous.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Semivitrification</ent><br/
<hw>Sem`i*vit"ri*fi*ca"tion</hw> <pr>(?)</pr>, <pos>n.</pos> <sn>1.</sn> <def>The quality or state of being semivitrified.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A substance imperfectly vitrified.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Semivitrified</ent><br/
<hw>Sem`i*vit"ri*fied</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Half or imperfectly vitrified; partially converted into glass.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Semivocal</ent><br/
<hw>Sem`i*vo"cal</hw> <pr>(?)</pr>, <pos>a.</pos> <fld>(Phon.)</fld> <def>Of or pertaining to a semivowel; half cocal; imperfectly sounding.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Semivowel</ent><br/
<hw>Sem"i*vow`el</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Phon.)</fld> <sd>(a)</sd> <def>A sound intermediate between a vowel and a consonant, or partaking of the nature of both, as in the English <xex>w</xex> and <xex>y</xex>.</def> <sd>(b)</sd> <def>The sign or letter representing such a sound.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Semiweekly</ent><br/
<hw>Sem`i*week"ly</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Coming, or made, or done, once every half week; <as>as, a <ex>semiweekly</ex> newspaper; a <ex>semiweekly</ex> trip.</as></def> -- <def2><pos>n.</pos> <def>That which comes or happens once every half week, esp. a semiweekly periodical.</def></def2> -- <def2><pos>adv.</pos> <def>At intervals of half a week each.</def></def2><br/
[<source>1913 Webster</source>]</p>

<p><ent>Semolella</ent><br/
||<hw>Sem`o*lel"la</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[It.]</ety> <def>See <er>Semolina</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Semolina</ent><br/
<hw>Sem`o*li"na</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[It. <ets>semolino</ets>, from <ets>semola</ets> bran, L. <ets>simila</ets> the finest wheat flour.  Cf. <er>Semoule</er>, <er>Simnel</er>.]</ety> <def>The purified fine, hard parts of durum wheat, derived mostly from the endosperm, rounded by the attrition of the millstones, -- used in cookery, such as in the preparation of Italian pasta.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Semolino</ent><br/
||<hw>Sem`o*li"no</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[It.]</ety> <def>Same as <er>Semolina</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Semoule</ent><br/
||<hw>Se*moule"</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F.]</ety> <def>Same as <er>Semolina</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sempervirent</ent><br/
<hw>Sem`per*vi"rent</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>semper</ets> always + <ets>virens</ets>, p. pr. of <ets>virere</ets> to be green.]</ety> <def>Always fresh; evergreen.</def> <mark>[R.]</mark>  <rj><au>Smart.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sempervive</ent><br/
<hw>Sem"per*vive</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>semperviva</ets>, <ets>sempervivum</ets>, fr. <ets>sempervivus</ets> ever-living; <ets>semper</ets> always + <ets>vivus</ets> living.]</ety> <fld>(Bot.)</fld> <def>The houseleek.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sempervivum</ent><br/
||<hw>Sem`per*vi"vum</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Bot.)</fld> <def>A genus of fleshy-leaved plants, of which the houseleek (<spn>Sempervivum tectorum</spn>) is the commonest species.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sempiternal</ent><br/
<hw>Sem`pi*ter"nal</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>sempiternus</ets>, fr. <ets>semper</ets> always: cf. F. <ets>sempiternel</ets>.]</ety> <sn>1.</sn> <def>Of neverending duration; everlasting; endless; having beginning, but no end.</def>  <rj><au>Sir M. Hale.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Without beginning or end; eternal.</def>  <rj><au>Blackmore.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sempiterne</ent><br/
<hw>Sem"pi*terne</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Sempiternal.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sempiternity</ent><br/
<hw>Sem`pi*ter"ni*ty</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>sempiternitas</ets>.]</ety> <def>Future duration without end; the relation or state of being sempiternal.</def>  <rj><au>Sir M. Hale.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sempre</ent><br/
<hw>Sem"pre</hw> <pr>(?)</pr>, <pos>adv.</pos> <ety>[It., fr. L. <ets>semper</ets>.]</ety> <fld>(Mus.)</fld> <def>Always; throughout; <as>as, <ex>sempre</ex> piano, always soft</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sempster</ent><br/
<hw>Semp"ster</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A seamster.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sempstress</ent><br/
<hw>Semp"stress</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A seamstress.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Two hundred <qex>sepstress</qex> were employed to make me shirts.</q> <rj><qau>Swift.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sempstressy</ent><br/
<hw>Semp"stress*y</hw> <pr>(?)</pr>, <pos>n.</pos> <def>Seamstressy.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Semster</ent><br/
<hw>Sem"ster</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A seamster.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Semuncia</ent><br/
||<hw>Se*mun"ci*a</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L., fr. <ets>semi</ets> half + <ets>uncia</ets> ounce.]</ety> <fld>(Rom. Antiq.)</fld> <def>A Roman coin equivalent to one twenty-fourth part of a Roman pound.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sen</ent><br/
<hw>Sen</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A Japanese coin, worth about one half of a cent.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sen</ent><br/
<hw>Sen</hw>, <pos>adv., prep., & conj.</pos> <ety>[See <er>Since</er>.]</ety> <def>Since.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Senary</ent><br/
<hw>Sen"a*ry</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>senarius</ets>, fr. <ets>seni</ets> six each, fr. <ets>sex</ets> six. See <er>Six</er>.]</ety> <def>Of six; belonging to six; containing six.</def>  <rj><au>Dr. H. More.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Senate</ent><br/
<hw>Sen"ate</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[OE. <ets>senat</ets>, F. <ets>s<eacute/nat</ets>, fr. L. <ets>senatus</ets>, fr. <ets>senex</ets>, gen. <ets>senis</ets>, old, an old man. See <er>Senior</er>, <er>Sir</er>.]</ety> <sn>1.</sn> <def>An assembly or council having the highest deliberative and legislative functions.</def> Specifically: <sd>(a)</sd> <fld>(Anc. Rom.)</fld> <def>A body of elders appointed or elected from among the nobles of the nation, and having supreme legislative authority.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The <qex>senate</qex> was thus the medium through which all affairs of the whole government had to pass.</q> <rj><qau>Dr. W. Smith.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sd>(b)</sd> <def>The upper and less numerous branch of a legislature in various countries, as in France, in the United States, in most of the separate States of the United States, and in some Swiss cantons.</def> <sd>(c)</sd> <def>In general, a legislative body; a state council; the legislative department of government.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>The governing body of the Universities of Cambridge and London.</def> <mark>[Eng.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>In some American colleges, a council of elected students, presided over by the president of the college, to which are referred cases of discipline and matters of general concern affecting the students.</def> <mark>[U. S.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Senate chamber</b></col>, <cd>a room where a senate meets when it transacts business.</cd> -- <col><b>Senate house</b></col>, <cd>a house where a senate meets when it transacts business.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Senator</ent><br/
<hw>Sen"a*tor</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[OE. <ets>senatour</ets>, OF. <ets>senatour</ets>, F. <ets>s<eacute/nateur</ets>, fr. L. <ets>senator</ets>.]</ety> <sn>1.</sn> <def>A member of a senate.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The duke and <qex>senators</qex> of Venice greet you.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ In the United States, each State sends two senators for a term of six years to the national Congress.</note><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(O.Eng.Law)</fld> <def>A member of the king's council; a king's councilor.</def>  <rj><au>Burrill.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Senatorial</ent><br/
<hw>Sen`a*to"ri*al</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[F. <ets>s<eacute/natorial</ets>, or L. <ets>senatorius</ets>.]</ety> <sn>1.</sn> <def>Of or pertaining to a senator, or a senate; becoming to a senator, or a senate; <as>as, <ex>senatorial</ex> duties; <ex>senatorial</ex> dignity</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Entitled to elect a senator, or by senators; <as>as, the <ex>senatorial</ex> districts of a State</as>.</def> <mark>[U. S.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Senatorially</ent><br/
<hw>Sen`a*to"ri*al*ly</hw>, <pos>adv.</pos> <def>In a senatorial manner.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Senatorian</ent><br/
<hw>Sen`a*to"ri*an</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Senatorial.</def> <mark>[R.]</mark>  <rj><au>De Quincey.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Senatorious</ent><br/
<hw>Sen`a*to"ri*ous</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Senatorial.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Senatorship</ent><br/
<hw>Sen"a*tor*ship</hw> <pr>(?)</pr>, <pos>n.</pos> <def>The office or dignity of a senator.</def>  <rj><au>Carew.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Senatusconsult</ent><br/
<hw>Se*na`tus*con*sult"</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>senatus consultum</ets>.]</ety> <def>A decree of the Roman senate.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Send</ent><br/
<hw>Send</hw> <pr>(?)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Sent</conjf> <pr>(<?/)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Sending</conjf>.]</vmorph> <ety>[AS. <ets>sendan</ets>; akin to OS. <ets>sendian</ets>, D. <ets>zenden</ets>, G. <ets>senden</ets>, OHG. <ets>senten</ets>, Icel. <ets>senda</ets>, Sw. <ets>s<aum/nda</ets>, Dan. <ets>sende</ets>, Goth. <ets>sandjan</ets>, and to Goth. <ets>sinp</ets> a time (properly, a going), ga<ets>sinpa</ets> companion, OHG. <ets>sind</ets> journey, AS. <ets>s<imac/<?/</ets>, Icel. <ets>sinni</ets> a walk, journey, a time. W. <ets>hynt</ets> a way, journey, OIr. <ets>s<?/t</ets>.  Cf. <er>Sense</er>.]</ety> <sn>1.</sn> <def>To cause to go in any manner; to dispatch; to commission or direct to go; <as>as, to <ex>send</ex> a messenger</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>I have not <qex>sent</qex> these prophets, yet they ran.</q> <rj><qau>Jer. xxiii. 21.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>I proceeded forth and came from God; neither came I of myself, but he <qex>sent</qex> me.</q> <rj><qau>John viii. 42.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Servants, <qex>sent</qex> on messages, stay out somewhat longer than the message requires.</q> <rj><qau>Swift.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To give motion to; to cause to be borne or carried; to procure the going, transmission, or delivery of; <as>as, to <ex>send</ex> a message</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>He . . . <qex>sent</qex> letters by posts on horseback.</q> <rj><qau>Esther viii. 10.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>O <qex>send</qex> out thy light an thy truth; let them lead me.</q> <rj><qau>Ps. xliii. 3.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>To emit; to impel; to cast; to throw; to hurl; <as>as, to <ex>send</ex> a ball, an arrow, or the like</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>To cause to be or to happen; to bestow; to inflict; to grant; -- sometimes followed by a dependent proposition.</def> <ldquo/God <xex>send</xex> him well!<rdquo/  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>The Lord shall <qex>send</qex> upon thee cursing, vexation, and rebuke.</q> <rj><qau>Deut. xxviii. 20.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>And <qex>sendeth</qex> rain on the just and on the unjust.</q> <rj><qau>Matt. v. 45.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>God <qex>send</qex> your mission may bring back peace.</q> <rj><qau>Sir W. Scott.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Send</ent><br/
<hw>Send</hw> <pr>(?)</pr>, <pos>v. i.</pos> <sn>1.</sn> <def>To dispatch an agent or messenger to convey a message, or to do an errand.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>See ye how this son of a murderer hath <qex>sent</qex> to take away my head?</q> <rj><qau>2 Kings vi. 32.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Naut.)</fld> <def>To pitch; <as>as, the ship <ex>sends</ex> forward so violently as to endanger her masts</as>.</def>  <rj><au>Totten.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>To send for</b></col>, <cd>to request or require by message to come or be brought.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Send</ent><br/
<hw>Send</hw>, <pos>n.</pos> <fld>(Naut.)</fld> <def>The impulse of a wave by which a vessel is carried bodily.</def> <altsp>[Written also <asp>scend</asp>.]</altsp> <au>W. C. Russell.</au> <ldquo/The <xex>send</xex> of the sea<rdquo/.  <rj><au>Longfellow.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sendal</ent><br/
<hw>Sen"dal</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[OF. <ets>cendal</ets> (cf. Pr. & Sp. <ets>cendal</ets>, It. <ets>zendale</ets>), LL. <ets>cendallum</ets>, Gr. <?/<?/<?/ a fine Indian cloth.]</ety> <def>A light thin stuff of silk.</def> <altsp>[Written also <asp>cendal</asp>, and <asp>sendal</asp>.]</altsp>  <rj><au>Chaucer.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Wore she not a veil of twisted <qex>sendal</qex> embroidered with silver?</q>  <rj><au>Sir W. Scott.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sender</ent><br/
<hw>Send"er</hw> <pr>(?)</pr>, <pos>n.</pos> <def>One who sends.</def>  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Senecas</ent><br/
<hw>Sen"e*cas</hw> <pr>(?)</pr>, <pos>n. pl.</pos>; <sing>sing. <singw>Seneca</singw> <pr>(<?/)</pr></sing>. <fld>(Ethnol.)</fld> <def>A tribe of Indians who formerly inhabited a part of Western New York. This tribe was the most numerous and most warlike of the Five Nations.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Seneca grass</b></col><fld>(Bot.)</fld>, <cd>holy grass. See under <er>Holy</er>.</cd> -- <col><b>Seneca eil</b></col>, <cd>petroleum or naphtha.</cd> -- <mcol><col><b>Seneca root</b></col>, <it>or</it> <col><b>Seneca snakeroot</b></col></mcol> <fld>(Bot.)</fld>, <cd>the rootstock of an American species of milkworth (<spn>Polygala Senega</spn>) having an aromatic but bitter taste. It is often used medicinally as an expectorant and diuretic, and, in large doses, as an emetic and cathartic.</cd> <altsp>[Written also <asp>Senega root</asp>, and <asp>Seneka root</asp>.]</altsp></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Senecio</ent><br/
||<hw>Se*ne"ci*o</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L., groundsel, lit., an old man. So called in allusion to the hoary appearance of the pappus.]</ety> <fld>(Bot.)</fld> <def>A very large genus of composite plants including the groundsel and the golden ragwort.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Senectitude</ent><br/
<hw>Se*nec"ti*tude</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>senectus</ets> aged, old age, <ets>senex</ets> old.]</ety> <def>Old age.</def> <mark>[R.]</mark> <ldquo/<xex>Senectitude</xex>, weary of its toils.<rdquo/  <rj><au>H. Miller.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Senega</ent><br/
<hw>Sen"e*ga</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Med.)</fld> <def>Seneca root.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Senegal</ent><br/
<hw>Sen"e*gal</hw> <pr>(?)</pr>, <pos>n.</pos> <def>Gum senegal. See under <er>Gum</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Senegin</ent><br/
<hw>Sen"e*gin</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Med. Chem.)</fld> <def>A substance extracted from the rootstock of the <spn>Polygala Senega</spn> (Seneca root), and probably identical with polygalic acid.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Senescence</ent><br/
<hw>Se*nes"cence</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[See <er>Senescent</er>.]</ety> <def>The state of growing old; decay by time.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Senescent</ent><br/
<hw>Se*nes"cent</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>senescent</ets>, p. pr. of <ets>senescere</ets> to grow old, incho. fr. <ets>senere</ets> to be old.]</ety> <def>Growing old; decaying with the lapse of time.</def> <ldquo/The night was <xex>senescent</xex>.<rdquo/ <au>Poe.</au> <ldquo/With too <xex>senescent</xex> air.<rdquo/  <rj><au>Lowell.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Seneschal</ent><br/
<hw>Sen"es*chal</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[OF. <ets>seneschal</ets>, LL. <ets>seniscalcus</ets>, of Teutonic origin; cf. Goth. <ets>sineigs</ets> old, <ets>skalks</ets>, OHG. <ets>scalch</ets>, AS. <ets>scealc</ets>.  Cf. <er>Senior</er>, <er>Marshal</er>.]</ety> <def>An officer in the houses of princes and dignitaries, in the Middle Ages, who had the superintendence of feasts and domestic ceremonies; a steward. Sometimes the seneschal had the dispensing of justice, and was given high military commands.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Then marshaled feast<br/
Served up in hall with sewers and <qex>seneschale</qex>.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Philip Augustus, by a famous ordinance in 1190, first established royal courts of justice, held by the officers called baitiffs, or <qex>seneschals</qex>, who acted as the king's lieutenants in his demains.</q> <rj><qau>Hallam.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Seneschalship</ent><br/
<hw>Sen"es*chal*ship</hw>, <pos>n.</pos> <def>The office, dignity, or jurisdiction of a seneschal.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Senge</ent><br/
<hw>Senge</hw> <pr>(?)</pr>, <pos>v. t.</pos> <def>To singe.</def> <mark>[Obs.]</mark>  <rj><au>Chaucer.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sengreen</ent><br/
<hw>Sen"green</hw> <pr>(?)</pr>, <pos>n.</pos><ety>[AS. <ets>singr<?/ne</ets>, properly, evergreen, fr. <ets>sin</ets> (in composition) always + <ets>gr<eum/ne</ets> green; akin to OHG. <ets>sin-</ets> ever, L. <ets>semper</ets>.]</ety> <fld>(Bot.)</fld> <def>The houseleek.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Senhor</ent><br/
||<hw>Se*nhor"</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Pg. Cf. <er>Se<ntil/or</er>, <er>Senior</er>.]</ety> <def>A Portuguese title of courtesy corresponding to the Spanish <xex>se<ntil/or</xex> or the English <it>Mr.</it> or <it>sir</it>; also, a gentleman.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Senhora</ent><br/
||<hw>Se*nho"ra</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Pg. Cf. <er>Se<ntil/ora</er>.]</ety> <def>A Portuguese title of courtesy given to a lady; Mrs.; Madam; also, a lady.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Senile</ent><br/
<hw>Se"nile</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>senilis</ets>, from <ets>senex</ets>, gen. <ets>senis</ets>, old, an old man: cf. F. <ets>s<eacute/nile</ets>. See <er>Senior</er>.]</ety> <def>Of or pertaining to old age; proceeding from, or characteristic of, old age; affected with the infirmities of old age; <as>as, <ex>senile</ex> weakness</as>.</def> <ldquo/<xex>Senile</xex> maturity of judgment.<rdquo/  <rj><au>Boyle.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Senile gangrene</b></col> <fld>(Med.)</fld>, <cd>a form of gangrene occuring particularly in old people, and caused usually by insufficient blood supply due to degeneration of the walls of the smaller arteries.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Senility</ent><br/
<hw>Se*nil"i*ty</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Cf. F. <ets>s<eacute/nilit<eacute/</ets>.]</ety> <def>The quality or state of being senile; old age.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Senior</ent><br/
<hw>Sen"ior</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>senior</ets>, compar. of <ets>senex</ets>, gen. <ets>senis</ets>, old. See <er>Sir</er>.]</ety> <sn>1.</sn> <def>More advanced than another in age; prior in age; elder; hence, more advanced in dignity, rank, or office; superior; <as>as, <ex>senior</ex> member; <ex>senior</ex> counsel</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Belonging to the final year of the regular course in American colleges, or in professional schools.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Senior</ent><br/
<hw>Sen"ior</hw>, <pos>n.</pos> <sn>1.</sn> <def>A person who is older than another; one more advanced in life.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>One older in office, or whose entrance upon office was anterior to that of another; one prior in grade.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>An aged person; an older.</def>  <rj><au>Dryden.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Each village <qex>senior</qex> paused to scan,<br/
And speak the lovely caravan.</q> <rj><qau>Emerson.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>One in the fourth or final year of his collegiate course at an American college; -- originally called <altname>senior sophister</altname>; also, one in the last year of the course at a professional schools or at a seminary.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Seniority</ent><br/
<hw>Sen*ior"i*ty</hw> <pr>(?)</pr>, <pos>n.</pos> <def>The quality or state of being senior.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Seniorize</ent><br/
<hw>Sen"ior*ize</hw> <pr>(?)</pr>, <pos>v. i.</pos> <def>To exercise authority; to rule; to lord it.</def> <mark>[R.]</mark>  <rj><au>Fairfax.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Seniory</ent><br/
<hw>Sen"ior*y</hw> <pr>(?)</pr>, <pos>n.</pos> <def>Seniority.</def> <mark>[Obs.]</mark>  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Senna</ent><br/
<hw>Sen"na</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Cf. It. & Sp. <ets>sena</ets>, Pg. <ets>sene</ets>, F. <ets>s<eacute/n<eacute/</ets>; all fr. Ar. <ets>san<amac/</ets>.]</ety> <sn>1.</sn> <fld>(Med.)</fld> <def>The leaves of several leguminous plants of the genus <gen>Cassia</gen>. (<spn>Cassia acutifolia</spn>, <spn>Cassia angustifolia</spn>, etc.). They constitute a valuable but nauseous cathartic medicine.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Bot.)</fld> <def>The plants themselves, native to the East, but now cultivated largely in the south of Europe and in the West Indies.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Bladder senna</b></col>. <fld>(Bot.)</fld> <cd>See under <er>Bladder</er>.</cd> -- <col><b>Wild senna</b></col> <fld>(Bot.)</fld>, <cd>the <spn>Cassia Marilandica</spn>, growing in the United States, the leaves of which are used medicinally, like those of the officinal senna.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><-- p. 1310 --></p>

<p><ent>Sennachy</ent><br/
<hw>Sen"na*chy</hw> <pr>(?)</pr>, <pos>n.</pos> <def>See <er>Seannachie</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sennet</ent><br/
<hw>Sen"net</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Properly, a sign given for the entrance or exit of actors, from OF. <ets>sinet</ets>, <ets>signet</ets>, dim. of <ets>signe</ets>. See <er>Signet</er>.]</ety> <def>A signal call on a trumpet or cornet for entrance or exit on the stage.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sennet</ent><br/
<hw>Sen"net</hw>, <pos>n.</pos> <fld>(Zool.)</fld> <def>The barracuda.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sennight</ent><br/
<hw>Sen"night</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Contr. fr. <ets>sevennight</ets>.]</ety> <def>The space of seven nights and days; a week.</def> <altsp>[Written also <asp>se'nnight</asp>.]</altsp> <mark>[Archaic.]</mark>  <rj><au>Shak.</au>  <au>Tennyson.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sennit</ent><br/
<hw>Sen"nit</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[<ets>Seven</ets> + <ets>knit</ets>.]</ety> <sn>1.</sn> <fld>(Naut.)</fld> <def>A braided cord or fabric formed by plaiting together rope yarns or other small stuff.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Plaited straw or palm leaves for making hats.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Senocular</ent><br/
<hw>Se*noc"u*lar</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>seni</ets> six each (fr. <ets>sex</ets> six) + <ets>oculus</ets> eye.]</ety> <def>Having six eyes.</def> <mark>[R.]</mark>  <rj><au>Derham.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Senonian</ent><br/
<hw>Se*no"ni*an</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[F. <ets>s<eacute/nonien</ets>, from the district of <ets>S<eacute/nonais</ets>, in France.]</ety> <fld>(Geol.)</fld> <def>In european geology, a name given to the middle division of the Upper Cretaceous formation.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Senor</ent><br/
<ent>Senyor</ent><br/
||<hw>Se*<ntil/or"</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Sp.  Cf. <er>Senior</er>.]</ety> <def>A Spanish title of courtesy corresponding to the English <xex>Mr.</xex> or <xex>Sir</xex>; also, a gentleman.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Senora</ent><br/
<ent>Senyora</ent><br/
||<hw>Se*<ntil/o"ra</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Sp.]</ety> <def>A Spanish title of courtesy given to a lady; Mrs.; Madam; also, a lady.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Senorita</ent><br/
<ent>Senyorita</ent><br/
||<hw>Se`<ntil/o*ri"ta</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Sp.]</ety> <def>A Spanish title of courtesy given to a young lady; Miss; also, a young lady.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sens</ent><br/
<hw>Sens</hw> <pr>(?)</pr>, <pos>adv.</pos> <ety>[See <er>Since</er>.]</ety> <def>Since.</def> <mark>[Obs.]</mark>  <rj><au>Spenser.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sensate</ent><br/
<hw>Sen"sate</hw> <pr>(?)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Sensated</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Sensating</conjf>.]</vmorph> <ety>[See <er>Sensated</er>.]</ety> <def>To feel or apprehend more or less distinctly through a sense, or the senses; <as>as, to <ex>sensate</ex> light, or an odor</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>As those of the one are <qex>sensated</qex> by the ear, so those of the other are by the eye.</q> <rj><qau>R. Hooke.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sensated</ent><br/
<ent>Sensate</ent><br/
<mhw>{ <hw>Sen"sate</hw> <pr>(?)</pr>, <hw>Sen"sa*ted</hw> <pr>(?)</pr>, }</mhw> <pos>a.</pos> <ety>[L. <ets>sensatus</ets> gifted with sense, intelligent, fr. <ets>sensus</ets> sense. See <er>Sense</er>.]</ety> <def>Felt or apprehended through a sense, or the senses.</def> <mark>[R.]</mark>  <rj><au>Baxter.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sensation</ent><br/
<hw>Sen*sa"tion</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Cf. F. <ets>sensation</ets>. See <er>Sensate</er>.]</ety> <sn>1.</sn> <fld>(Physiol.)</fld> <def>An impression, or the consciousness of an impression, made upon the central nervous organ, through the medium of a sensory or afferent nerve or one of the organs of sense; a feeling, or state of consciousness, whether agreeable or disagreeable, produced either by an external object (stimulus), or by some change in the internal state of the body.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Perception is only a special kind of knowledge, and <qex>sensation</qex> a special kind of feeling.  . . . Knowledge and feeling, perception and <qex>sensation</qex>, though always coexistent, are always in the inverse ratio of each other.</q> <rj><qau>Sir W. Hamilton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A purely spiritual or psychical affection; agreeable or disagreeable feelings occasioned by objects that are not corporeal or material.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>A state of excited interest or feeling, or that which causes it.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The <qex>sensation</qex> caused by the appearance of that work is still remembered by many.</q> <rj><qau>Brougham.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><syn><b>Syn.</b> -- Perception.</syn>  -- <usage><er>Sensation</er>, <er>Perseption</er>. The distinction between these words, when used in mental philosophy, may be thus stated; if I simply smell a rose, I have a <xex>sensation</xex>; if I refer that smell to the external object which occasioned it, I have a <xex>perception</xex>. Thus, the former is mere feeling, without the idea of an object; the latter is the mind's apprehension of some external object as occasioning that feeling.  <ldquo/<xex>Sensation</xex> properly expresses <xex>that change in the state of the mind</xex> which is produced by an impression upon an organ of sense (of which change we can conceive the mind to be conscious, without any knowledge of external objects). <xex>Perception</xex>, on the other hand, expresses the <xex>knowledge</xex> or the intimations we obtain by means of our <xex>sensations</xex> concerning the qualities of matter, and consequently involves, in every instance, the notion of <xex>externality</xex>, or <xex>outness</xex>, which it is necessary to exclude in order to seize the precise import of the word <xex>sensation</xex>.<rdquo/ <au>Fleming.</au></usage><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sensational</ent><br/
<hw>Sen*sa"tion*al</hw> <pr>(?)</pr>, <pos>a.</pos> <sn>1.</sn> <def>Of or pertaining to sensation; <as>as, <ex>sensational</ex> nerves</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Of or pertaining to sensationalism, or the doctrine that sensation is the sole origin of knowledge.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>Suited or intended to excite temporarily great interest or emotion; melodramatic; emotional; <as>as, <ex>sensational</ex> plays or novels; <ex>sensational</ex> preaching; <ex>sensational</ex> journalism; a <ex>sensational</ex> report</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sensationalism</ent><br/
<hw>Sen*sa"tion*al*ism</hw> <pr>(?)</pr>, <pos>n.</pos> <sn>1.</sn> <fld>(Metaph.)</fld> <def>The doctrine held by Condillac, and by some ascribed to Locke, that our ideas originate solely in sensation, and consist of sensations transformed; sensualism; -- opposed to <contr>intuitionalism</contr>, and <contr>rationalism</contr>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>The practice or methods of sensational writing or speaking; <as>as, the <ex>sensationalism</ex> of a novel</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sensationalist</ent><br/
<hw>Sen*sa"tion*al*ist</hw>, <pos>n.</pos> <sn>1.</sn> <fld>(Metaph.)</fld> <def>An advocate of, or believer in, philosophical sensationalism.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>One who practices sensational writing or speaking.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sense</ent><br/
<hw>Sense</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>sensus</ets>, from <ets>sentire</ets>, <ets>sensum</ets>, to perceive, to feel, from the same root as E. <ets>send</ets>; cf. OHG. <ets>sin</ets> sense, mind, <ets>sinnan</ets> to go, to journey, G. <ets>sinnen</ets> to meditate, to think: cf. F. <ets>sens</ets>. For the change of meaning cf. <er>See</er>, <pos>v. t.</pos> See <er>Send</er>, and cf. <er>Assent</er>, <er>Consent</er>, <er>Scent</er>, <pos>v. t.</pos>, <er>Sentence</er>, <er>Sentient</er>.]</ety> <sn>1.</sn> <fld>(Physiol.)</fld> <def>A faculty, possessed by animals, of perceiving external objects by means of impressions made upon certain organs (sensory or sense organs) of the body, or of perceiving changes in the condition of the body; <as>as, the <ex>senses</ex> of sight, smell, hearing, taste, and touch</as>. See <cref>Muscular sense</cref>, under <er>Muscular</er>, and <cref>Temperature sense</cref>, under <er>Temperature</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Let fancy still my <qex>sense</qex> in Lethe steep.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>What surmounts the reach<br/
Of human <qex>sense</qex> I shall delineate.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>The traitor <qex>Sense</qex> recalls<br/
The soaring soul from rest.</q> <rj><qau>Keble.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Perception by the sensory organs of the body; sensation; sensibility; feeling.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>In a living creature, though never so great, the <qex>sense</qex> and the affects of any one part of the body instantly make a transcursion through the whole.</q> <rj><qau>Bacon.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>Perception through the intellect; apprehension; recognition; understanding; discernment; appreciation.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>This Basilius, having the quick <qex>sense</qex> of a lover.</q> <rj><qau>Sir P. Sidney.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>High disdain from <qex>sense</qex> of injured merit.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>Sound perception and reasoning; correct judgment; good mental capacity; understanding; also, that which is sound, true, or reasonable; rational meaning.</def> <ldquo/He speaks <xex>sense</xex>.<rdquo/  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>He raves; his words are loose<br/
As heaps of sand, and scattering wide from <qex>sense</qex>.</q> <rj><qau>Dryden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <def>That which is felt or is held as a sentiment, view, or opinion; judgment; notion; opinion.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>I speak my private but impartial <qex>sense</qex><br/
With freedom.</q> <rj><qau>Roscommon.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>The municipal council of the city had ceased to speak the <qex>sense</qex> of the citizens.</q> <rj><qau>Macaulay.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>6.</sn> <def>Meaning; import; signification; <as>as, the true <ex>sense</ex> of words or phrases; the <ex>sense</ex> of a remark</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>So they read in the book in the law of God distinctly, and gave the <qex>sense</qex>.</q> <rj><qau>Neh. viii. 8.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>I think 't was in another <qex>sense</qex>.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>7.</sn> <def>Moral perception or appreciation.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Some are so hardened in wickedness as to have no <qex>sense</qex> of the most friendly offices.</q> <rj><qau>L' Estrange.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>8.</sn> <fld>(Geom.)</fld> <def>One of two opposite directions in which a line, surface, or volume, may be supposed to be described by the motion of a point, line, or surface.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Common sense</b></col>, according to Sir W. Hamilton: <sd>(a)</sd> <cd><ldquo/The complement of those cognitions or convictions which we receive from nature, which all men possess in common, and by which they test the truth of knowledge and the morality of actions.<rdquo/</cd> <sd>(b)</sd> <cd><ldquo/The faculty of first principles.<rdquo/ These two are the philosophical significations.</cd> <sd>(c)</sd> <cd><ldquo/Such ordinary complement of intelligence, that,if a person be deficient therein, he is accounted mad or foolish.<rdquo/</cd> <sd>(d)</sd> <cd>When the substantive is emphasized: <ldquo/Native practical intelligence, natural prudence, mother wit, tact in behavior, acuteness in the observation of character, in contrast to habits of acquired learning or of speculation.<rdquo/</cd> -- <col><b>Moral sense</b></col>. <cd>See under <er>Moral</er>, <sd>(a)</sd>.</cd> -- <mcol><col><b>The inner sense</b></col>, <it>or</it> <col><b>The internal sense</b></col></mcol>, <cd>capacity of the mind to be aware of its own states; consciousness; reflection.</cd>  <ldquo/This source of ideas every man has wholly in himself, and though it be not sense, as having nothing to do with external objects, yet it is very like it, and might properly enough be called <xex>internal sense</xex>.<rdquo/ <au>Locke.</au> -- <col><b>Sense capsule</b></col> <fld>(Anat.)</fld>, <cd>one of the cartilaginous or bony cavities which inclose, more or less completely, the organs of smell, sight, and hearing.</cd> -- <col><b>Sense organ</b></col> <fld>(Physiol.)</fld>, <cd>a specially irritable mechanism by which some one natural force or form of energy is enabled to excite sensory nerves; as the eye, ear, an end bulb or tactile corpuscle, etc.</cd> -- <col><b>Sense organule</b></col> <fld>(Anat.)</fld>, <cd>one of the modified epithelial cells in or near which the fibers of the sensory nerves terminate.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><syn><b>Syn.</b> -- Understanding; reason.</syn>  -- <usage><er>Sense</er>, <er>Understanding</er>, <er>Reason</er>. Some philosophers have given a technical signification to these terms, which may here be stated. <xex>Sense</xex> is the mind's acting in the direct cognition either of material objects or of its own mental states. In the first case it is called the <xex>outer</xex>, in the second the <xex>inner</xex>, sense. <xex>Understanding</xex> is the logical faculty, <it>i. e.</it>, the power of apprehending under general conceptions, or the power of classifying, arranging, and making deductions. <xex>Reason</xex> is the power of apprehending those first or fundamental truths or principles which are the conditions of all real and scientific knowledge, and which control the mind in all its processes of investigation and deduction. These distinctions are given, not as established, but simply because they often occur in writers of the present day.</usage><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sense</ent><br/
<hw>Sense</hw> <pr>(?)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Sensed</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Sensing</conjf>.]</vmorph> <def>To perceive by the senses; to recognize.</def> <mark>[Obs. or Colloq.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>Is he sure that objects are not otherwise <qex>sensed</qex> by others than they are by him?</q> <rj><qau>Glanvill.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Senseful</ent><br/
<hw>Sense"ful</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Full of sense, meaning, or reason; reasonable; judicious.</def> <mark>[R.]</mark> <ldquo/<xex>Senseful</xex> speech.<rdquo/ <au>Spenser.</au> <ldquo/Men, otherwise <xex>senseful</xex> and ingenious.<rdquo/ <au>Norris.</au><br/
[<source>1913 Webster</source>]</p>

<p><ent>Senseless</ent><br/
<hw>Sense"less</hw>, <pos>a.</pos> <def>Destitute of, deficient in, or contrary to, sense; without sensibility or feeling; unconscious; stupid; foolish; unwise; unreasonable.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>You blocks, you stones, you worse than <qex>senseless</qex> things.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>The ears are <qex>senseless</qex> that should give us hearing.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>The <qex>senseless</qex> grave feels not your pious sorrows.</q> <rj><qau>Rowe.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>They were a <qex>senseless</qex>, stupid race.</q> <rj><qau>Swift.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>They would repent this their <qex>senseless</qex> perverseness when it would be too late.</q> <rj><qau>Clarendon.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p>-- <wordforms><wf>Sense"less*ly</wf>, <pos>adv.</pos> -- <wf>Sense"less*ness</wf>, <pos>n.</pos></wordforms><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sensibility</ent><br/
<hw>Sen`si*bil"i*ty</hw> <pr>(?)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Sensibilities</plw> <pr>(#)</pr>.</plu> <ety>[Cf. F. <ets>sensibilit<eacute/</ets>, LL. <ets>sensibilitas</ets>.]</ety> <sn>1.</sn> <fld>(Physiol.)</fld> <def>The quality or state of being sensible, or capable of sensation; capacity to feel or perceive.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>The capacity of emotion or feeling, as distinguished from the intellect and the will; peculiar susceptibility of impression, pleasurable or painful; delicacy of feeling; quick emotion or sympathy; <as>as, <ex>sensibility</ex> to pleasure or pain; <ex>sensibility</ex> to shame or praise; exquisite <ex>sensibility</ex></as>; -- often used in the plural.</def> <ldquo/<xex>Sensibilities</xex> so fine!<rdquo/  <rj><au>Cowper.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>The true lawgiver ought to have a heart full of <qex>sensibility</qex>.</q> <rj><qau>Burke.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>His <qex>sensibilities</qex> seem rather to have been those of patriotism than of wounded pride.</q> <rj><qau>Marshall.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>Experience of sensation; actual feeling.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>This adds greatly to my <qex>sensibility</qex>.</q> <rj><qau>Burke.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>That quality of an instrument which makes it indicate very slight changes of condition; delicacy; <as>as, the <ex>sensibility</ex> of a balance, or of a thermometer</as>.</def><-- usu. sensitivity --><br/
[<source>1913 Webster</source>]</p>

<p><syn><b>Syn.</b> -- Taste; susceptibility; feeling. See <er>Taste</er>.</syn><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sensible</ent><br/
<hw>Sen"si*ble</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[F., fr. L. <ets>sensibilis</ets>, fr. <ets>sensus</ets> sense.]</ety> <sn>1.</sn> <def>Capable of being perceived by the senses; apprehensible through the bodily organs; hence, also, perceptible to the mind; making an impression upon the sense, reason, or understanding; <?/<?/<?/<?/<?/<?/ heat; <xex>sensible</xex> resistance.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Air is <qex>sensible</qex> to the touch by its motion.</q> <rj><qau>Arbuthnot.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>The disgrace was more <qex>sensible</qex> than the pain.</q> <rj><qau>Sir W. Temple.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Any very <qex>sensible</qex> effect upon the prices of things.</q> <rj><qau>A. Smith.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Having the capacity of receiving impressions from external objects; capable of perceiving by the instrumentality of the proper organs; liable to be affected physsically or mentally; impressible.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Would your cambric were <qex>sensible</qex> as your finger.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>Hence: Liable to impression from without; easily affected; having nice perception or acute feeling; sensitive; also, readily moved or affected by natural agents; delicate; <as>as, a <ex>sensible</ex> thermometer</as>.</def> <ldquo/With affection wondrous <xex>sensible</xex>.<rdquo/  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>Perceiving or having perception, either by the senses or the mind; cognizant; perceiving so clearly as to be convinced; satisfied; persuaded.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>He [man] can not think at any time, waking or sleeping, without being <qex>sensible</qex> of it.</q> <rj><qau>Locke.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>They are now <qex>sensible</qex> it would have been better to comply than to refuse.</q> <rj><qau>Addison.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <def>Having moral perception; capable of being affected by moral good or evil.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>6.</sn> <def>Possessing or containing sense or reason; giftedwith, or characterized by, good or common sense; intelligent; wise.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Now a <qex>sensible</qex> man, by and by a fool.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><mcol><col><b>Sensible note</b></col> <it>or</it> <col><b>Sensible tone</b></col></mcol> <fld>(Mus.)</fld>, <cd>the major seventh note of any scale; -- so called because, being but a half step below the octave, or key tone, and naturally leading up to that, it makes the ear <xex>sensible</xex> of its approaching sound.  Called also the <altname>leading tone</altname>.</cd> -- <col><b>Sensible horizon</b></col>. <cd>See <er>Horizon</er>, <pos>n.</pos>, 2. <sd>(a)</sd>.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><syn><b>Syn.</b> -- Intelligent; wise.</syn>  -- <usage><er>Sensible</er>, <er>Intelligent</er>. We call a man <xex>sensible</xex> whose judgments and conduct are marked and governed by sound judgment or good common sense. We call one <xex>intelligent</xex> who is quick and clear in his understanding, <it>i. e.</it>, who discriminates readily and nicely in respect to difficult and important distinction. The sphere of the <xex>sensible</xex> man lies in matters of practical concern; of the <xex>intelligent</xex> man, in subjects of intellectual interest. <ldquo/I have been tired with accounts from <xex>sensible</xex> men, furnished with matters of fact which have happened within their own knowledge.<rdquo/ <au>Addison.</au> <ldquo/Trace out numerous footsteps . . . of a most wise and <xex>intelligent</xex> architect throughout all this stupendous fabric.<rdquo/  <au>Woodward.</au></usage><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sensible</ent><br/
<hw>Sen"si*ble</hw> <pr>(?)</pr>, <pos>n.</pos> <sn>1.</sn> <def>Sensation; sensibility.</def> <mark>[R.]</mark> <ldquo/Our temper changed . . . which must needs remove the <xex>sensible</xex> of pain.<rdquo/  <rj><au>Milton.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>That which impresses itself on the sense; anything perceptible.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Aristotle distinguished <qex>sensibles</qex> into common and proper.</q> <rj><qau>Krauth-Fleming.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>That which has sensibility; a sensitive being.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>This melancholy extends itself not to men only, but even to vegetals and <qex>sensibles</qex>.</q> <rj><qau>Burton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sensibleness</ent><br/
<hw>Sen"si*ble*ness</hw>, <pos>n.</pos> <sn>1.</sn> <def>The quality or state of being sensible; sensibility; appreciation; capacity of perception; susceptibility.</def> <ldquo/The <xex>sensibleness</xex> of the eye.<rdquo/ <au>Sharp.</au> <ldquo/<xex>Sensibleness</xex> and sorrow for sin.<rdquo/  <rj><au>Hammond.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>The <qex>sensibleness</qex> of the divine presence.</q> <rj><qau>Hallywell.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Intelligence; reasonableness; good sense.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sensibly</ent><br/
<hw>Sen"si*bly</hw>, <pos>adv.</pos> <sn>1.</sn> <def>In a sensible manner; so as to be perceptible to the senses or to the mind; appreciably; with perception; susceptibly; sensitively.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>What remains past cure,<br/
Bear not too <qex>sensibly</qex>.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>With intelligence or good sense; judiciously.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sensifacient</ent><br/
<hw>Sen`si*fa"cient</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>sensus</ets> sense + <ets>facere</ets> to make.]</ety> <def>Converting into sensation.</def>  <rj><au>Huxley.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sensiferous</ent><br/
<hw>Sen*sif"er*ous</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>sensifer</ets>; <ets>sensus</ets> sense + <ets>ferre</ets> to bear.]</ety> <def>Exciting sensation; conveying sensation.</def>  <rj><au>Huxley.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sensific</ent><br/
<hw>Sen*sif"ic</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>sensificus</ets>; <ets>sensus</ets> sense + <ets>facere</ets> to make.]</ety> <def>Exciting sensation.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sensificatory</ent><br/
<hw>Sen*sif"i*ca*to*ry</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Susceptible of, or converting into, sensation; <as>as, the <ex>sensificatory</ex> part of a nervous system</as>.</def>  <rj><au>Huxley.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sensigenous</ent><br/
<hw>Sen*sig"e*nous</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>sensus</ets> sense + <ets>-genous</ets>.]</ety> <def>Causing or exciting sensation.</def>  <rj><au>Huxley.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sensism</ent><br/
<hw>Sens"ism</hw> <pr>(?)</pr>, <pos>n.</pos> <def>Same as <er>Sensualism</er>, 2 & 3.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sensist</ent><br/
<hw>Sens"ist</hw>, <pos>n.</pos> <def>One who, in philosophy, holds to sensism.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sensitive</ent><br/
<hw>Sen"si*tive</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[F. <ets>sensitif</ets>. See <er>Sense</er>.]</ety> <sn>1.</sn> <def>Having sense of feeling; possessing or exhibiting the capacity of receiving impressions from external objects; <as>as, a <ex>sensitive</ex> soul</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Having quick and acute sensibility, either to the action of external objects, or to impressions upon the mind and feelings; highly susceptible; easily and acutely affected.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>She was too <qex>sensitive</qex> to abuse and calumny.</q> <rj><qau>Macaulay.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <sd>(a)</sd> <fld>(Mech.)</fld> <def>Having a capacity of being easily affected or moved; <as>as, a <ex>sensitive</ex> thermometer; <ex>sensitive</ex> scales</as>.</def> <sd>(b)</sd> <fld>(Chem. & Photog.)</fld> <def>Readily affected or changed by certain appropriate agents; <as>as, silver chloride or bromide, when in contact with certain organic substances, is extremely <ex>sensitive</ex> to actinic rays</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>Serving to affect the sense; sensible.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>A sensitive love of some <qex>sensitive</qex> objects.</q> <rj><qau>Hammond.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <def>Of or pertaining to sensation; depending on sensation; <as>as, <ex>sensitive</ex> motions; <ex>sensitive</ex> muscular motions excited by irritation</as>.</def>  <rj><au>E. Darwin.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Sensitive fern</b></col> <fld>(Bot.)</fld>, <cd>an American fern (<spn>Onoclea sensibilis</spn>), the leaves of which, when plucked, show a slight tendency to fold together.</cd> -- <col><b>Sensitive flame</b></col> <fld>(Physics)</fld>, <cd>a gas flame so arranged that under a suitable adjustment of pressure it is exceedingly sensitive to sounds, being caused to roar, flare, or become suddenly shortened or extinguished, by slight sounds of the proper pitch.</cd> -- <col><b>Sensitive joint vetch</b></col> <fld>(Bot.)</fld>, <cd>an annual leguminous herb (<spn>Aeschynomene hispida</spn>), with sensitive foliage.</cd> -- <col><b>Sensitive paper</b></col>, <cd>paper prepared for photographic purpose by being rendered sensitive to the effect of light.</cd> -- <col><b>Sensitive plant</b></col>. <fld>(Bot.)</fld> <sd>(a)</sd> <cd>A leguminous plant (<spn>Mimosa pudica</spn>, or <spn>Mimosa sensitiva</spn>, and other allied species), the leaves of which close at the slightest touch.</cd> <sd>(b)</sd> <cd>Any plant showing motions after irritation, as the sensitive brier (<gen>Schrankia</gen>) of the Southern States, two common American species of <gen>Cassia</gen> (<spn>Cassia nictitans</spn>, and <spn>Cassia Chamaecrista</spn>), a kind of sorrel (<spn>Oxalis sensitiva</spn>), etc.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><-- p. 1311 --></p>

<p>-- <wordforms><wf>Sen"si*tive*ly</wf> <pr>(#)</pr>, <pos>adv.</pos> -- <wf>Sen"si*tive*ness</wf>, <pos>n.</pos></wordforms><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sensitivity</ent><br/
<hw>Sen`si*tiv"i*ty</hw> <pr>(?)</pr>, <pos>n.</pos> <def>The quality or state of being sensitive; -- used chiefly in science and the arts; <as>as, the <ex>sensitivity</ex> of iodized silver</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q><qex>Sensitivity</qex> and emotivity have also been used as the scientific term for the capacity of feeling.</q> <rj><qau>Hickok.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sensitize</ent><br/
<hw>Sen"si*tize</hw> <pr>(?)</pr>, <pos>v. t.</pos> <fld>(Photog.)</fld> <def>To render sensitive, or susceptible of being easily acted on by the actinic rays of the sun; <as>as, <ex>sensitized</ex> paper or plate</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sensitizer</ent><br/
<hw>Sen"si*ti`zer</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Photog.)</fld> <def>An agent that sensitizes.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The <qex>sensitizer</qex> should be poured on the middle of the sheet.</q> <rj><qau>Wilis & Clements (The Platinotype).</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sensitometer</ent><br/
<hw>Sen`si*tom"e*ter</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[See <er>Sensitive</er>; <er>-meter</er>.]</ety> <fld>(Photog.)</fld> <def>An instrument or apparatus for comparing and grading the sensitiveness of plates, films, etc., as a screen divided into squares of different shades or colors, from which a picture is made on the plate to be tested.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Sensitory</ent><br/
<hw>Sen"si*to*ry</hw> <pr>(?)</pr>, <pos>n.</pos> <def>See <er>Sensory</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sensive</ent><br/
<hw>Sens"ive</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Having sense or sensibility; sensitive.</def> <mark>[Obs.]</mark>  <rj><au>Sir P. Sidney.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sensor</ent><br/
<hw>Sen"sor</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Sensory; <as>as, the <ex>sensor</ex> nerves</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sensorial</ent><br/
<hw>Sen*so"ri*al</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[Cf. F. <ets>sensorial</ets>. See <er>Sensorium</er>.]</ety> <def>Of or pertaining to the sensorium; <as>as, <ex>sensorial</ex> faculties, motions, powers</as>.</def>  <rj><au>A. Tucker.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sensorium</ent><br/
<hw>Sen*so"ri*um</hw> <pr>(?)</pr>, <pos>n.</pos>; <plu><it>pl.</it> E. <plw>Sensoriums</plw> <pr>(#)</pr>, L. <plw>Sensoria</plw> <pr>(#)</pr>.</plu> <ety>[L., fr. <ets>sentire</ets>, <ets>sensum</ets>, <ets>to discern or perceive by the senses</ets>.]</ety> <fld>(Physiol.)</fld> <def>The seat of sensation; the nervous center or centers to which impressions from the external world must be conveyed before they can be perceived; the place where external impressions are localized, and transformed into sensations, prior to being reflected to other parts of the organism; hence, the whole nervous system, when animated, so far as it is susceptible of common or special sensations.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sensori-volitional</ent><br/
<hw>Sen*so`ri-vo*li"tion*al</hw> <pr>(?)</pr>, <pos>a.</pos> <fld>(Physiol.)</fld> <def>Concerned both in sensation and volition; -- applied to those nerve fibers which pass to and from the cerebro-spinal axis, and are respectively concerned in sensation and volition.</def>  <rj><au>Dunglison.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sensory</ent><br/
<hw>Sen"so*ry</hw> <pr>(?)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Sensories</plw> <pr>(<?/)</pr>.</plu> <fld>(Physiol.)</fld> <def>Same as <er>Sensorium</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sensory</ent><br/
<hw>Sen"so*ry</hw>, <pos>a.</pos> <fld>(Physiol.)</fld> <def>Of or pertaining to the sensorium or sensation; <as>as, <ex>sensory</ex> impulses; -- especially applied to those nerves and nerve fibers which convey to a nerve center impulses resulting in sensation; also sometimes loosely employed in the sense of <ex>afferent</ex>, to indicate nerve fibers which convey impressions of any kind to a nerve center</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sensual</ent><br/
<hw>Sen"su*al</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>sensualis</ets>, from <ets>sensus</ets> sense: cf. F. <ets>sensuel</ets>.]</ety> <sn>1.</sn> <def>Pertaining to, consisting in, or affecting, the sense, or bodily organs of perception; relating to, or concerning, the body, in distinction from the spirit.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Pleasing and <qex>sensual</qex> rites and ceremonies.</q> <rj><qau>Bacon.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Far as creation's ample range extends,<br/
The scale of <qex>sensual</qex>, mental powers ascends.</q> <rj><qau>Pope.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Hence, not spiritual or intellectual; carnal; fleshly; pertaining to, or consisting in, the gratification of the senses, or the indulgence of appetites; wordly.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>These be they who separate themselves, <qex>sensual</qex>, having not the Spirit.</q> <rj><qau>Jude 19.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>The greatest part of men are such as prefer . . . that good which is <qex>sensual</qex> before whatsoever is most divine.</q> <rj><qau>Hooker.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>Devoted to the pleasures of sense and appetite; luxurious; voluptuous; lewd; libidinous.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>No small part of virtue consists in abstaining from that wherein <qex>sensual</qex> men place their felicity.</q> <rj><qau>Atterbury.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>Pertaining or peculiar to the philosophical doctrine of sensualism.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sensualism</ent><br/
<hw>Sen"su*al*ism</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Cf. F. <ets>sensualisme</ets>.]</ety> <sn>1.</sn> <def>The condition or character of one who is sensual; subjection to sensual feelings and appetite; sensuality.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Philos.)</fld> <def>The doctrine that all our ideas, or the operations of the understanding, not only originate in sensation, but are transformed sensations, copies or relics of sensations; sensationalism; sensism.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <fld>(Ethics)</fld> <def>The regarding of the gratification of the senses as the highest good.</def>  <rj><au>Krauth-Fleming.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sensualist</ent><br/
<hw>Sen"su*al*ist</hw>, <pos>n.</pos> <ety>[CF. F. <ets>sensualiste</ets>.]</ety> <sn>1.</sn> <def>One who is sensual; one given to the indulgence of the appetites or senses as the means of happiness.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>One who holds to the doctrine of sensualism.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sensualistic</ent><br/
<hw>Sen`su*al*is"tic</hw> <pr>(?)</pr>, <pos>a.</pos> <sn>1.</sn> <def>Sensual.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Adopting or teaching the doctrines of sensualism.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sensuality</ent><br/
<hw>Sen`su*al"i*ty</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[CF. F. <ets>sensualit<eacute/</ets>, L. <ets>sensualitas</ets> sensibility, capacity for sensation.]</ety> <def>The quality or state of being sensual; devotedness to the gratification of the bodily appetites; free indulgence in carnal or sensual pleasures; luxuriousness; voluptuousness; lewdness.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Those pampered animals<br/
That rage in savage <qex>sensuality</qex>.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>They avoid dress, lest they should have affections tainted by any <qex>sensuality</qex>.</q> <rj><qau>Addison.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sensualization</ent><br/
<hw>Sen`su*al*i*za"tion</hw> <pr>(?)</pr>, <pos>n.</pos> <def>The act of sensualizing, or the state of being sensualized.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sensualize</ent><br/
<hw>Sen"su*al*ize</hw> <pr>(?)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Sensualized</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Sensualizing</conjf> <pr>(?)</pr>.]</vmorph> <def>To make sensual; to subject to the love of sensual pleasure; to debase by carnal gratifications; to carnalize; <as>as, <ex>sensualized</ex> by pleasure</as>.</def>  <rj><au>Pope.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>By the neglect of prayer, the thoughts are <qex>sensualized</qex>.</q> <rj><qau>T. H. Skinner.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sensually</ent><br/
<hw>Sen"su*al*ly</hw>, <pos>adv.</pos> <def>In a sensual manner.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sensualness</ent><br/
<hw>Sen"su*al*ness</hw>, <pos>n.</pos> <def>Sensuality; fleshliness.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sensuism</ent><br/
<hw>Sen"su*ism</hw> <pr>(?)</pr>, <pos>n.</pos> <def>Sensualism.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sensuosity</ent><br/
<hw>Sen`su*os"i*ty</hw> <pr>(?)</pr>, <pos>n.</pos> <def>The quality or state of being sensuous; sensuousness.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sensuous</ent><br/
<hw>Sen"su*ous</hw> <pr>(?)</pr>, <pos>a.</pos> <sn>1.</sn> <def>Of or pertaining to the senses, or sensible objects; addressing the senses; suggesting pictures or images of sense.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>To this poetry would be made precedent, as being less subtle and fine, but more simple, <qex>sensuous</qex>, and passionate.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Highly susceptible to influence through the senses.</def><br/
[<source>1913 Webster</source>]</p>

<p>-- <wordforms><wf>Sen"su*ous*ly</wf> <pr>(#)</pr>, <pos>adv.</pos> -- <wf>Sen"su*ous*ness</wf>, <pos>n.</pos></wordforms><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sent</ent><br/
<hw>Sent</hw> <pr>(?)</pr>, <pos>v. & n.</pos> <def>See <er>Scent</er>, <pos>v. & n.</pos></def> <mark>[Obs.]</mark>  <rj><au>Spenser.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sent</ent><br/
<hw>Sent</hw>, <mark>obs.</mark> <def><pos>3d pers. sing. pres.</pos> of <er>Send</er>, for <xex>sendeth</xex>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sent</ent><br/
<hw>Sent</hw>, <def><pos>imp. & p. p.</pos> of <er>Send</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sentence</ent><br/
<hw>Sen"tence</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F., from L. <ets>sententia</ets>, for <ets>sentientia</ets>, from <ets>sentire</ets> to discern by the senses and the mind, to feel, to think. See <er>Sense</er>, <pos>n.</pos>, and cf. <er>Sentiensi</er>.]</ety> <sn>1.</sn> <def>Sense; meaning; significance.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>Tales of best <qex>sentence</qex> and most solace.</q> <rj><qau>Chaucer.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>The discourse itself, voluble enough, and full of <qex>sentence</qex>.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <sd>(a)</sd> <def>An opinion; a decision; a determination; a judgment, especially one of an unfavorable nature.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>My <qex>sentence</qex> is for open war.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>That by them [Luther's works] we may pass <qex>sentence</qex> upon his doctrines.</q> <rj><qau>Atterbury.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sd>(b)</sd> <def>A philosophical or theological opinion; a dogma; <as>as, Summary of the <ex>Sentences</ex>; Book of the <ex>Sentences</ex></as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <fld>(Law)</fld> <def>In civil and admiralty law, the judgment of a court pronounced in a cause; in criminal and ecclesiastical courts, a judgment passed on a criminal by a court or judge; condemnation pronounced by a judicial tribunal; doom.  In common law, the term is exclusively used to denote the judgment in criminal cases.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Received the <qex>sentence</qex> of the law.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>A short saying, usually containing moral instruction; a maxim; an axiom; a saw.</def>  <rj><au>Broome.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <fld>(Gram.)</fld> <def>A combination of words which is complete as expressing a thought, and in writing is marked at the close by a period, or full point. See <er>Proposition</er>, 4.</def><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ <xex>Sentences</xex> are simple or compound.  A simple sentence consists of one subject and one finite verb; as, <ldquo/The Lord reigns.<rdquo/  A compound sentence contains two or more subjects and finite verbs, as in this verse: -</note><br/
[<source>1913 Webster</source>]</p>

<p><q>He fills, he bounds, connects, and equals all.</q> <rj><qau>Pope.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Dark sentence</b></col>, <cd>a saying not easily explained.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><q>A king . . . understanding <qex>dark sentences</qex>.</q> <rj><qau>Dan. vii. 23.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sentence</ent><br/
<hw>Sen"tence</hw>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Sentenced</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Sentencing</conjf> <pr>(?)</pr>.]</vmorph> <sn>1.</sn> <def>To pass or pronounce judgment upon; to doom; to condemn to punishment; to prescribe the punishment of.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Nature herself is <qex>sentenced</qex> in your doom.</q> <rj><qau>Dryden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To decree or announce as a sentence.</def> <mark>[Obs.]</mark>  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>To utter sententiously.</def> <mark>[Obs.]</mark>  <rj><au>Feltham.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sentence method</ent><br/
<hw>Sen"tence meth`od</hw>. <fld>(Education)</fld> <def>A method of teaching reading by giving first attention to phrases and sentences and later analyzing these into their verbal and alphabetic components; -- contrasted with <contr>alphabet method</contr> and <contr>word method</contr>.  See also <contr>phonics</contr>.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Sentencer</ent><br/
<hw>Sen"ten*cer</hw> <pr>(?)</pr>, <pos>n.</pos> <def>One who pronounced a sentence or condemnation.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>sentential</ent><br/
<hw>sen*ten"tial</hw> <pr>(?)</pr>, <pos>a.</pos> <sn>1.</sn> <def>Comprising sentences; <as>as, a <ex>sentential</ex> translation</as>.</def>  <rj><au>Abp. Newcome.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Of or pertaining to a sentence, or full period; <as>as, a <ex>sentential</ex> pause</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sententially</ent><br/
<hw>Sen*ten"tial*ly</hw>, <pos>adv.</pos> <def>In a sentential manner.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sententiarist</ent><br/
<hw>Sen*ten"ti*a*rist</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A sententiary.</def>  <rj><au>Barnas Sears (Life of Luther).</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sententiary</ent><br/
<hw>Sen*ten"ti*ary</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[LL. <ets>sententiarius</ets>.]</ety> <def>One who read lectures, or commented, on the Sentences of Peter Lombard, Bishop of Paris (1159-1160), a school divine.</def>  <rj><au>R. Henry.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sententiosity</ent><br/
<hw>Sen*ten`ti*os"i*ty</hw> <pr>(?)</pr>, <pos>n.</pos> <def>The quality or state of being sententious.</def> <mark>[Obs.]</mark>  <rj><au>Sir T. Browne.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sententious</ent><br/
<hw>Sen*ten"tious</hw> <pr>(?)</pr>, <pos>a.</pos><ety>[L.  <ets>sentenciosus</ets>: cf. F. <ets>sentencieux</ets>.]</ety> <sn>1.</sn> <def>Abounding with sentences, axioms, and maxims; full of meaning; terse and energetic in expression; pithy; <as>as, a <ex>sententious</ex> style or discourse; <ex>sententious</ex> truth</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>How he apes his sire,<br/
Ambitiously <qex>sententious</qex>!</q> <rj><qau>Addison.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Comprising or representing sentences; sentential.</def> <mark>[Obs.]</mark> <ldquo/<xex>Sententious</xex> marks.<rdquo/  <rj><au>Grew.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p>-- <wordforms><wf>Sen*ten"tious*ly</wf>, <pos>adv.</pos> -- <wf>Sen*ten"tious*ness</wf>, <pos>n.</pos></wordforms><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sentery</ent><br/
<hw>Sen"ter*y</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A sentry.</def> <mark>[Obs.]</mark>  <rj><au>Milton.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Senteur</ent><br/
<hw>Sen"teur</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F.]</ety> <def>Scent.</def> <mark>[Obs.]</mark>  <rj><au>Holland.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sentiency</ent><br/
<ent>Sentience</ent><br/
<mhw>{ <hw>Sen"ti*ence</hw> <pr>(?)</pr>, <hw>Sen"ti*en*cy</hw> <pr>(?)</pr>, }</mhw> <pos>n.</pos> <ety>[See <er>Sentient</er>, <er>Sentence</er>.]</ety> <def>The quality or state of being sentient; esp., the quality or state of having sensation.</def>  <rj><au>G. H. Lewes.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>An example of harmonious action between the intelligence and the <qex>sentiency</qex> of the mind.</q> <rj><qau>Earle.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sentient</ent><br/
<hw>Sen"ti*ent</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>sentiens</ets>, <ets>-entis</ets>, p. pr. of <ets>sentire</ets> to discern or perceive by the senses. See <er>Sense</er>.]</ety> <def>Having a faculty, or faculties, of sensation and perception.</def> <specif>Specif.</specif> <fld>(Physiol.)</fld>, <def>especially sensitive; <as>as, the <ex>sentient</ex> extremities of nerves, which terminate in the various organs or tissues</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sentient</ent><br/
<hw>Sen"ti*ent</hw>, <pos>n.</pos> <def>One who has the faculty of perception; a sentient being.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sentiently</ent><br/
<hw>Sen"ti*ent*ly</hw>, <pos>adv.</pos> <def>In a sentient or perceptive way.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sentiment</ent><br/
<hw>Sen"ti*ment</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[OE. <ets>sentement</ets>, OF. <ets>sentement</ets>, F. <ets>sentiment</ets>, fr. L. <ets>sentire</ets> to perceive by the senses and mind, to feel, to think. See <er>Sentient</er>, <pos>a.</pos>]</ety> <sn>1.</sn> <def>A thought prompted by passion or feeling; a state of mind in view of some subject; feeling toward or respecting some person or thing; disposition prompting to action or expression.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The word <qex>sentiment</qex>, agreeably to the use made of it by our best English writers, expresses, in my own opinion very happily, those complex determinations of the mind which result from the cooperation of our rational powers and of our moral feelings.</q> <rj><qau>Stewart.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Alike to council or the assembly came,<br/
With equal souls and <qex>sentiments</qex> the same.</q> <rj><qau>Pope.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Hence, generally, a decision of the mind formed by deliberation or reasoning; thought; opinion; notion; judgment; <as>as, to express one's <ex>sentiments</ex> on a subject</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q><qex>Sentiments</qex> of philosophers about the perception of external objects.</q> <rj><qau>Reid.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Sentiment, as here and elsewhere employed by Reid in the meaning of opinion (sententia), is not to be imitated.</q> <rj><qau>Sir W. Hamilton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>A sentence, or passage, considered as the expression of a thought; a maxim; a saying; a toast.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>Sensibility; feeling; tender susceptibility.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Mr. Hume sometimes employs (after the manner of the French metaphysicians) <qex>sentiment</qex> as synonymous with feeling; a use of the word quite unprecedented in our tongue.</q> <rj><qau>Stewart.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Less of <qex>sentiment</qex> than sense.</q> <rj><qau>Tennyson.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><syn><b>Syn.</b> -- Thought; opinion; notion; sensibility; feeling.</syn>  -- <usage><er>Sentiment</er>, <er>Opinion</er>, <er>Feeling</er>. An <xex>opinion</xex> is an intellectual judgment in respect to any and every kind of truth. <xex>Feeling</xex> describes those affections of pleasure and pain which spring from the exercise of our sentient and emotional powers. <xex>Sentiment</xex> (particularly in the plural) lies between them, denoting <xex>settled opinions</xex> or principles in regard to subjects which interest the feelings strongly, and are presented more or less constantly in practical life. Hence, it is more appropriate to speak of our religious <xex>sentiments</xex> than <xex>opinions</xex>, unless we mean to exclude all reference to our feelings. The word <xex>sentiment</xex>, in the singular, leans ordinarily more to the side of feeling, and denotes a refined sensibility on subjects affecting the heart. <ldquo/On questions of feeling, taste, observation, or report, we define our <xex>sentiments</xex>. On questions of science, argument, or metaphysical abstraction, we define our <xex>opinions</xex>. The <xex>sentiments</xex> of the heart. The <xex>opinions</xex> of the mind . . . There is more of instinct in <xex>sentiment</xex>, and more of definition in <xex>opinion</xex>. The admiration of a work of art which results from first impressions is classed with our <xex>sentiments</xex>; and, when we have accounted to ourselves for the approbation, it is classed with our <xex>opinions</xex>.<rdquo/ <au>W. Taylor.</au></usage><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sentimental</ent><br/
<hw>Sen`ti*men"tal</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[Cf. F. <ets>sentimental</ets>.]</ety> <sn>1.</sn> <def>Having, expressing, or containing a sentiment or sentiments; abounding with moral reflections; containing a moral reflection; didactic.</def> <mark>[Obsoles.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>Nay, ev'n each moral <qex>sentimental</qex> stroke,<br/
Where not the character, but poet, spoke,<br/
He lopped, as foreign to his chaste design,<br/
Nor spared a useless, though a golden line.</q> <rj><qau>Whitehead.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Inclined to sentiment; having an excess of sentiment or sensibility; indulging the sensibilities for their own sake; artificially or affectedly tender; -- often in a reproachful sense.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>A <qex>sentimental</qex> mind is rather prone to overwrought feeling and exaggerated tenderness.</q> <rj><qau>Whately.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>Addressed or pleasing to the emotions only, usually to the weaker and the unregulated emotions.</def><br/
[<source>1913 Webster</source>]</p>

<p><syn><b>Syn.</b> -- Romantic.</syn>  -- <usage><er>Sentimental</er>, <er>Romantic</er>. <xex>Sentimental</xex> usually describes an error or excess of the sensibilities; <xex>romantic</xex>, a vice of the imagination. The votary of the former gives indulgence to his sensibilities for the mere luxury of their excitement; the votary of the latter allows his imagination to rove for the pleasure of creating scenes of ideal enjoiment. <ldquo/Perhaps there is no less danger in works called <xex>sentimental</xex>. They attack the heart more successfully, because more cautiously.<rdquo/ <au>V. Knox.</au> <ldquo/I can not but look on an indifferency of mind, as to the good or evil things of this life, as a mere <xex>romantic</xex> fancy of such who would be thought to be much wiser than they ever were, or could be.<rdquo/ <au>Bp. Stillingfleet.</au></usage><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sentimentalism</ent><br/
<hw>Sen`ti*men"tal*ism</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Cf. F. <ets>sentimentalisme</ets>.]</ety> <def>The quality of being sentimental; the character or behavior of a sentimentalist; sentimentality.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sentimentalist</ent><br/
<hw>Sen`ti*men"tal*ist</hw>, <pos>n.</pos> <ety>[Cf. F. <ets>sentimentaliste</ets>.]</ety> <def>One who has, or affects, sentiment or fine feeling.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sentimentality</ent><br/
<hw>Sen`ti*men*tal"i*ty</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[CF. F. <ets>sentimentalit<eacute/</ets>.]</ety> <def>The quality or state of being sentimental.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sentimentalize</ent><br/
<hw>Sen`ti*men"tal*ize</hw> <pr>(?)</pr>, <pos>v. t.</pos> <def>To regard in a sentimental manner; <as>as, to <ex>sentimentalize</ex> a subject</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sentimentalize</ent><br/
<hw>Sen`ti*men"tal*ize</hw>, <pos>v. i.</pos> <def>To think or act in a sentimental manner, or like a sentimentalist; to affect exquisite sensibility.</def>  <rj><au>C. Kingsley.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sentimentally</ent><br/
<hw>Sen`ti*men"tal*ly</hw>, <pos>adv.</pos> <def>In a sentimental manner.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sentine</ent><br/
<hw>Sen"tine</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>sentina</ets> bilge water, hold of a ship, dregs: cf. F. <ets>sentine</ets>.]</ety> <def>A place for dregs and dirt; a sink; a sewer.</def> <mark>[Obs.]</mark>  <rj><au>Latimer.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sentinel</ent><br/
<hw>Sen"ti*nel</hw>, <pos>n.</pos> <ety>[F. <ets>sentinelle</ets> (cf. It. <ets>sentinella</ets>); probably originally, a litle path, the sentinel's beat,, and a dim. of a word meaning, path; cf. F. <ets>sente</ets> path. L. <ets>semita</ets>; and OF. <ets>sentine</ets>, <ets>sentele</ets>, <ets>senteret</ets>, diminutive words.  Cf. <er>Sentry</er>.]</ety> <sn>1.</sn> <def>One who watches or guards; specifically <fld>(Mil.)</fld>, a soldier set to guard an army, camp, or other place, from surprise, to observe the approach of danger, and give notice of it; a sentry.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The <qex>sentinels</qex> who paced the ramparts.</q> <rj><qau>Macaulay.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Watch; guard.</def> <mark>[Obs.]</mark> <ldquo/That princes do keep due <xex>sentinel</xex>.<rdquo/  <rj><au>Bacon.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <fld>(Zool.)</fld> <def>A marine crab (<spn>Podophthalmus vigil</spn>) native of the Indian Ocean, remarkable for the great length of its eyestalks; -- called also <altname>sentinel crab</altname>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sentinel</ent><br/
<hw>Sen"ti*nel</hw>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Sentineled</conjf> <pr>(?)</pr> or <conjf>Sentinelled</conjf>; <pos>p. pr. & vb. n.</pos> <conjf>Sentineling</conjf> or <conjf>Sentinelling</conjf>.]</vmorph> <sn>1.</sn> <def>To watch over like a sentinel.</def> <ldquo/To <xex>sentinel</xex> enchanted land.<rdquo/ <mark>[R.]</mark>  <rj><au>Sir W. Scott.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To furnish with a sentinel; to place under the guard of a sentinel or sentinels.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sentisection</ent><br/
<hw>Sen`ti*sec"tion</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>sentire</ets> to feel + E. <ets>section</ets>.]</ety> <def>Painful vivisection; -- opposed to <contr>callisection</contr>.</def>  <rj><au>B. G. Wilder.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sentry</ent><br/
<hw>Sen"try</hw> <pr>(?)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Sentires</plw> <pr>(#)</pr>.</plu> <ety>[Probably from OF. <ets>senteret</ets> a little patch; cf. F. <ets>sentier</ets> path, and OF. <ets>sente</ets>. See <er>Sentinel</er>.]</ety> <sn>1.</sn> <fld>(Mil.)</fld> <def>A soldier placed on guard; a sentinel.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Guard; watch, as by a sentinel.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Here toils, and death, and death's half-brother, sleep,<br/
Forms terrible to view, their <qex>sentry</qex> keep.</q> <rj><qau>Dryden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Sentry box</b></col>, <cd>a small house or box to cover a sentinel at his post, and shelter him from the weather.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Senza</ent><br/
||<hw>Sen"za</hw> <pr>(?)</pr>, <pos>prep.</pos> <ety>[It.]</ety> <fld>(Mus.)</fld> <def>Without; <as>as, <ex>senza</ex> stromenti, without instruments</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sepal</ent><br/
<hw>Se"pal</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[NL. <ets>sepalum</ets>, formed in imitation of NL. <ets>petalum</ets>, petal, to denote one of the divisions of the calyx: cf. F. <ets>s<eacute/pale</ets>.]</ety> <fld>(Bot.)</fld> <def>A leaf or division of the calyx.</def><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ When the calyx consists of but one part, it is said to be <xex>monosepalous</xex>; when of two parts, it is said to be <xex>disepalous</xex>; when of a variable and indefinite number of parts, it is said to be <xex>polysepalous</xex>; when of several parts united, it is properly called <xex>gamosepalous</xex>.</note><br/
[<source>1913 Webster</source>]</p>

<p><-- p. 1312 --></p>

<p><ent>Sepaled</ent><br/
<hw>Se"paled</hw> <pr>(?)</pr>, <pos>a.</pos> <fld>(Bot.)</fld> <def>Having one or more sepals.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sepaline</ent><br/
<hw>Sep"al*ine</hw> <pr>(?)</pr>, <pos>a.</pos> <fld>(Bot.)</fld> <def>Relating to, or having the nature of, sepals.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sepalody</ent><br/
<hw>Se*pal"o*dy</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[<ets>Sepal</ets>  + Gr. <grk>e'i^dos</grk> form.]</ety> <fld>(Bot.)</fld> <def>The metamorphosis of other floral organs into sepals or sepaloid bodies.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sepaloid</ent><br/
<hw>Sep"al*oid</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[<ets>Sepal</ets>  + <ets>-oid</ets>.]</ety> <fld>(Bot.)</fld> <def>Like a sepal, or a division of a calyx.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sepalous</ent><br/
<hw>Sep"al*ous</hw> <pr>(?)</pr>, <pos>a.</pos> <fld>(Bot.)</fld> <def>Having, or relating to, sepals; -- used mostly in composition. See under <er>Sepal</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Separability</ent><br/
<hw>Sep`a*ra*bil"i*ty</hw> <pr>(?)</pr>, <pos>n.</pos> <def>Quality of being separable or divisible; divisibility; separableness.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Separable</ent><br/
<hw>Sep"a*ra*ble</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>separabilis</ets>: cf. F. <ets>s<eacute/parable</ets>.]</ety> <def>Capable of being separated, disjoined, disunited, or divided; <as>as, the <ex>separable</ex> parts of plants; qualities not <ex>separable</ex> from the substance in which they exist.</as></def> -- <wordforms><wf>Sep"a*ra*ble*ness</wf>, <pos>n.</pos> -- <wf>Sep"a*ra*bly</wf>, <pos>adv.</pos></wordforms><br/
[<source>1913 Webster</source>]</p>

<p><q>Trials permit me not to doubt of the <qex>separableness</qex> of a yellow tincture from gold.</q> <rj><qau>Boyle.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Separate</ent><br/
<hw>Sep"a*rate</hw> <pr>(?)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Separated</conjf> <pr>(<?/)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Separating</conjf>.]</vmorph> <ety>[L. <ets>separatus</ets>, p. p. of <ets>separare</ets> to separate; pfref. <ets>se-</ets> aside + <ets>parare</ets> to make ready, prepare. See <er>Parade</er>, and cf. <er>Sever</er>.]</ety> <sn>1.</sn> <def>To disunite; to divide; to disconnect; to sever; to part in any manner.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>From the fine gold I <qex>separate</qex> the alloy.</q> <rj><qau>Dryden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q><qex>Separate</qex> thyself, I pray thee, from me.</q> <rj><qau>Gen. xiii. 9.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Who shall <qex>separate</qex> us from the love of Christ?</q> <rj><qau>Rom. viii. 35.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To come between; to keep apart by occupying the space between; to lie between; <as>as, the Mediterranean Sea <ex>separates</ex> Europe and Africa</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>To set apart; to select from among others, as for a special use or service.</def><br/
[<source>1913 Webster</source>]</p>

<p><q><qex>Separate</qex> me Barnabas and Saul for the work whereunto I have called thaem.</q> <rj><qau>Acts xiii. 2.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Separated flowers</b></col> <fld>(Bot.)</fld>, <cd>flowers which have stamens and pistils in separate flowers; diclinous flowers.</cd>  <rj><au>Gray.</au></rj></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Separate</ent><br/
<hw>Sep"a*rate</hw>, <pos>v. i.</pos> <def>To part; to become disunited; to be disconnected; to withdraw from one another; <as>as, the family <ex>separated</ex></as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Separate</ent><br/
<hw>Sep"a*rate</hw> <pr>(?)</pr>, <pos>p. a.</pos> <ety>[L. <ets>separatus</ets>, p. p. ]</ety> <sn>1.</sn> <def>Divided from another or others; disjoined; disconnected; separated; -- said of things once connected.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Him that was <qex>separate</qex> from his brethren.</q> <rj><qau>Gen. xlix. 26.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Unconnected; not united or associated; distinct; -- said of things that have not been connected.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>For such an high priest became us, who is holy, harmless, undefiled, <qex>separate</qex> from sinnere.</q> <rj><qau>Heb. vii. 26.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>Disunited from the body; disembodied; <as>as, a <ex>separate</ex> spirit; the <ex>separate</ex> state of souls</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Separate estate</b></col> <fld>(Law)</fld>, <cd>an estate limited to a married woman independent of her husband.</cd> -- <col><b>Separate maintenance</b></col> <fld>(Law)</fld>, <cd>an allowance made to a wife by her husband under deed of separation.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p>-- <wordforms><wf>Sep"a*rate*ly</wf>, <pos>adv.</pos> -- <wf>Sep"a*rate*ness</wf>, <pos>n.</pos></wordforms><br/
[<source>1913 Webster</source>]</p>

<p><ent>Separatical</ent><br/
<hw>Sep`a*rat"ic*al</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Of or pertaining to separatism in religion; schismatical.</def> <mark>[R.]</mark>  <rj><au>Dr. T. Dwight.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Separating</ent><br/
<hw>Sep"a*ra`ting</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Designed or employed to separate.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Separating funnel</b></col> <fld>(Chem.)</fld>, <cd>a funnel, often globe-shaped, provided with a stopcock for the separate drawing off of immiscible liquids of different specific gravities.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Separation</ent><br/
<hw>Sep`a*ra"tion</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>separatio</ets>: cf. F. <ets>s<eacute/paration</ets>.]</ety> <def>The act of separating, or the state of being separated, or separate.</def> Specifically: <sd>(a)</sd> <def>Chemical analysis.</def> <sd>(b)</sd> <def>Divorce.</def> <sd>(c)</sd> <fld>(Steam Boilers)</fld> <def>The operation of removing water from steam.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Judicial separation</b></col> <fld>(Law)</fld>, <cd>a form of divorce; a separation of man and wife which has the effect of making each a single person for all legal purposes but without ability to contract a new marriage.</cd> <au>Mozley & W.</au></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Separatism</ent><br/
<hw>Sep"a*ra*tism</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[CF. F. <ets>s<eacute/paratisme</ets>.]</ety> <def>The character or act of a separatist; disposition to withdraw from a church; the practice of so withdrawing.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Separatist</ent><br/
<hw>Sep`a*ra*tist</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Cf. F.  <ets>s<eacute/paratiste</ets>.]</ety> <def>One who withdraws or separates himself; especially, one who withdraws from a church to which he has belonged; a seceder from an established church; a dissenter; a nonconformist; a schismatic; a sectary.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Heavy fines on divines who should preach in any meeting of <qex>separatist</qex> .</q> <rj><qau>Macaulay.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Separatistic</ent><br/
<hw>Sep`a*ra*tis"tic</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Of or pertaining to separatists; characterizing separatists; schismatical.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Separative</ent><br/
<hw>Sep"a*ra*tive</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>separativus</ets>.]</ety> <def>Causing, or being to cause, separation.</def> <ldquo/<xex>Separative</xex> virtue of extreme cold.<rdquo/  <rj><au>Boyle.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Separator</ent><br/
<hw>Sep"a*ra`tor</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L.]</ety> <def>One who, or that which, separates.</def> Specifically: <sd>(a)</sd> <fld>(Steam Boilers)</fld> <def>A device for depriving steam of particles of water mixed with it.</def> <sd>(b)</sd> <fld>(Mining)</fld> <def>An apparatus for sorting pulverized ores into grades, or separating them from gangue.</def> <sd>(c)</sd> <fld>(Weaving)</fld> <def>An instrument used for spreading apart the threads of the warp in the loom, etc.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Separatory</ent><br/
<hw>Sep"a*ra*to*ry</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Separative.</def>  <rj><au>Cheyne.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Separatory</ent><br/
<hw>Sep"a*ra*to*ry</hw>, <pos>n.</pos> <ety>[Cf. F. <ets>s<eacute/paratoire</ets>.]</ety> <sn>1.</sn> <fld>(Chem.)</fld> <def>An apparatus used in separating, as a separating funnel.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Surg.)</fld> <def>A surgical instrument for separating the pericranium from the cranium.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Separatrix</ent><br/
<hw>Sep`a*ra"trix</hw> <pr>(?)</pr>, <pos>n.</pos>; <plu><it>pl.</it> L. <plw>-trices</plw> <pr>(#)</pr>, E. <plw>-trixes</plw> <pr>(#)</pr>.</plu> <ety>[L., she that separates.]</ety> <fld>(Arith.)</fld> <def>The decimal point; the dot placed at the left of a decimal fraction, to separate it from the whole number which it follows. The term is sometimes also applied to other marks of separation.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sepawn</ent><br/
<hw>Se*pawn"</hw> <pr>(?)</pr>, <pos>n.</pos> <def>See <er>Supawn</er>.</def> <mark>[Local, U.S.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sepelible</ent><br/
<hw>Sep"e*li*ble</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>sepelibilis</ets>, fr.  <ets>sepelire</ets> to bury.]</ety> <def>Admitting of burial.</def> <mark>[Obs.]</mark>  <rj><au>Bailey.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sepelition</ent><br/
<hw>Sep`e*li"tion</hw> <pr>(?)</pr>, <pos>n.</pos> <def>Burial.</def> <mark>[Obs.]</mark>  <rj><au>Bp. Hall.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sephardic</ent><br/
<hw>Se*phar"dic</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[From <ets>Sephardim</ets>, a name applied to the Spanish Jews, fr. <etsep>Sephard</etsep>, name of a place where Jews were held in captivity (<au>Ob. 20</au>).]</ety> <def>Of, pertaining to, or designating, the Jews (the <xex>Sephardim</xex>, also called <xex>Spanish</xex> or <xex>Portuguese Jews</xex>) descended from Jewish families driven from Spain by the Inquisition.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Sephardim</ent><br/
<hw>Se*phar"dim</hw> <pr>(?)</pr>, <pos>n. pl.</pos> <ety>[NHeb.; orig. uncert.]</ety> <def>Jews who are descendants of the former Jews of Spain and Portugal. They are as a rule darker than the northern Jews, and have more delicate features.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Sephen</ent><br/
<hw>Se"phen</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Zool.)</fld> <def>A large sting ray of the genus <gen>Trygon</gen>, especially <spn>Trygon sephen</spn> of the Indian Ocean and the Red Sea. The skin is an article of commerce.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sepia</ent><br/
<hw>Se"pi*a</hw> <pr>(?)</pr>, <pos>n.</pos>; <plu><it>pl.</it> E. <plw>Sepias</plw> <pr>(#)</pr>, L. <plw>Sepiae</plw> <pr>(#)</pr>.</plu> <ety>[L., fr. Gr. <?/<?/<?/ the cuttlefish, or squid.]</ety> <sn>1.</sn> <fld>(Zool.)</fld> <sd>(a)</sd> <def>The common European cuttlefish.</def> <sd>(b)</sd> <def>A genus comprising the common cuttlefish and numerous similar species. See <xex>Illustr.</xex> under <er>Cuttlefish</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A pigment prepared from the ink, or black secretion, of the sepia, or cuttlefish. Treated with caustic potash, it has a rich brown color; and this mixed with a red forms <stype>Roman sepia</stype>.  Cf. <cref>India ink</cref>, under <er>India</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><mcol><col><b>Sepia drawing</b></col> <it>or</it> <col><b>Sepia picture</b></col></mcol>, <cd>a drawing in monochrome, made in sepia alone, or in sepia with other brown pigments.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sepia</ent><br/
<hw>Se"pi*a</hw>, <pos>a.</pos> <def>Of a dark brown color, with a little red in its composition; also, made of, or done in, sepia.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sepic</ent><br/
<hw>Se"pic</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Of or pertaining to sepia; done in sepia; <as>as, a <ex>sepic</ex> drawing</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sepidaceous</ent><br/
<hw>Sep`i*da"ceous</hw> <pr>(?)</pr>, <pos>a.</pos> <fld>(Zool.)</fld> <def>Like or pertaining to the cuttlefishes of the genus Sepia.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sepiment</ent><br/
<hw>Sep"i*ment</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>sepimentum</ets>, <ets>saepimentum</ets>, from <ets>sepire</ets>, <ets>saepire</ets>, to hedge in.]</ety> <def>Something that separates; a hedge; a fence.</def> <mark>[R.]</mark>  <rj><au>Bailey.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sepiolite</ent><br/
<hw>Se"pi*o*lite`</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[<ets>Septa</ets> + <ets>-lite</ets>.]</ety> <fld>(Min.)</fld> <def>Meerschaum. See <er>Meerschaum</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sepiostare</ent><br/
<hw>Se"pi*o*stare`</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[<ets>Sepia</ets> + Gr. <?/<?/<?/ a bone.]</ety> <fld>(Zool.)</fld> <def>The bone or shell of cuttlefish. See <xex>Illust.</xex> under <er>Cuttlefish</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sepon</ent><br/
<hw>Se*pon"</hw> <pr>(#)</pr>, <pos>n.</pos> <def>See <er>Supawn</er>.</def> <mark>[Local, U.S.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sepose</ent><br/
<hw>Se*pose"</hw> <pr>(?)</pr>, <pos>v. t.</pos> <ety>[L. pref <ets>se-</ets> aside + E. <ets>pose</ets>.]</ety> <def>To set apart.</def> <mark>[Obs.]</mark>  <rj><au>Donne.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Seposit</ent><br/
<hw>Se*pos"it</hw> <pr>(?)</pr>, <pos>v. t.</pos> <ety>[L. <ets>sepositus</ets>, p. p. of <ets>seponere</ets> to set aside.]</ety> <def>To set aside; to give up.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Seposition</ent><br/
<hw>Sep`o*si"tion</hw> <pr>(<?/)</pr>, <pos>n.</pos> <ety>[L. <ets>sepositio</ets>.]</ety> <def>The act of setting aside, or of giving up.</def> <mark>[Obs.]</mark>  <rj><au>Jer. Taylor.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sepoy</ent><br/
<hw>Se"poy</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Per. <ets>sip<amac/h<imac/</ets>, fr. <ets>sip<amac/h</ets> an army.  Cf. <er>Spahi</er>.]</ety> <def>A native of India employed as a soldier in the service of a European power, esp. of Great Britain; an Oriental soldier disciplined in the European manner.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Seppuku</ent><br/
||<hw>Sep*pu"ku</hw> <pr>(?)</pr>, <pos>n.</pos> <def>Same as <er>Hara-kiri</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q><qex>Seppuku</qex>, or hara-kiri, also came into vogue.</q> <rj><qau>W. E. Griffis.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sepsin</ent><br/
<hw>Sep"sin</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Gr. <?/<?/<?/ putrefaction.]</ety> <fld>(Physiol. Chem.)</fld> <def>A soluble poison (ptomaine) present in putrid blood. It is also formed in the putrefaction of proteid matter in general.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sepsis</ent><br/
||<hw>Sep"sis</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[NL., fr. Gr. <?/<?/<?/ putrefaction.]</ety> <fld>(Med.)</fld> <def>The poisoning of the system by the introduction of putrescent material into the blood.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sept</ent><br/
<hw>Sept</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[A corruption of <ets>sect</ets>, n.]</ety> <def>A clan, tribe, or family, proceeding from a common progenitor; -- used especially of the ancient clans in Ireland.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The chief, struck by the illustration, asked at once to be baptized, and all his <qex>sept</qex> followed his example.</q> <rj><qau>S. Lover.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Septaemia</ent><br/
||<hw>Sep*tae"mi*a</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[NL., fr. Gr. <?/<?/<?/ putrid + <?/<?/<?/ blood.]</ety> <fld>(Med.)</fld> <def>Septicaemia.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Septal</ent><br/
<hw>Sep"tal</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Of or pertaining to a septum or septa, as of a coral or a shell.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Septane</ent><br/
<hw>Sep"tane</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>septem</ets> seven.]</ety> <fld>(Chem.)</fld> <def>See <er>Heptane</er>.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Septangle</ent><br/
<hw>Sep"tan`gle</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[<ets>Septi-</ets> + <ets>angle</ets>.]</ety> <fld>(Geom.)</fld> <def>A figure which has seven angles; a heptagon.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Septangular</ent><br/
<hw>Sep*tan"gu*lar</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Heptagonal.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Septarium</ent><br/
||<hw>Sep*ta"ri*um</hw> <pr>(?)</pr>, <pos>n.</pos>;<plu><it>pl.</it> <plw>Septaria</plw> <pr>(#)</pr>.</plu> <ety>[NL., fr. L. <ets>septum</ets>, <ets>saeptum</ets>, an inclosure, a partition, fr. <ets>sepire</ets>, <ets>saepire</ets>, to inclose.]</ety> <fld>(Geol.)</fld> <def>A flattened concretionary nodule, usually of limestone, intersected within by cracks which are often filled with calcite, barite, or other minerals.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Septate</ent><br/
<hw>Sep"tate</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>septum</ets>, <ets>saeptum</ets>, partition.]</ety> <def>Divided by partition or partitions; having septa; <as>as, a <ex>septate</ex> pod or shell</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>September</ent><br/
<hw>Sep*tem"ber</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L., fr. <ets>septem</ets> seven, as being the seventh month of the Roman year, which began with March: cf. F. <ets>septembre</ets>. See <er>Seven</er>.]</ety> <def>The ninth month of the year, containing thurty days.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Septemberer</ent><br/
<hw>Sep*tem"ber*er</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A Setembrist.</def>  <rj><au>Carlyle.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Septembrist</ent><br/
<hw>Sep*tem"brist</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F. <ets>septembriste</ets>.]</ety> <def>An agent in the massacres in Paris, committed in patriotic frenzy, on the 22d of September, 1792.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Septemfluous</ent><br/
<hw>Sep*tem"flu*ous</hw> <pr>(?)</pr>, <pos>a.</pos><ety>[L. <ets>septemfluus</ets>; <ets>septem</ets> seven + <ets>fluere</ets> to flow.]</ety> <def>Flowing sevenfold; divided into seven streams or currents.</def> <mark>[R.]</mark>  <rj><au>Fuller.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Septempartite</ent><br/
<hw>Sep*tem"par*tite</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>septem</ets> seven + E. <ets>partite</ets>.]</ety> <def>Divided nearly to the base into seven parts; <as>as, a <ex>septempartite</ex> leaf</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Septemtrioun</ent><br/
<hw>Sep*tem"tri*oun</hw> <pr>(?)</pr>, <pos>n.</pos> <def>Septentrion.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Septemvir</ent><br/
||<hw>Sep*tem"vir</hw> <pr>(?)</pr>, <pos>n.</pos>; <plu><it>pl.</it> E. <plw>Septemvirs</plw> <pr>(#)</pr>, L. <plw>Septemviri</plw> <pr>(#)</pr>.</plu> <ety>[L. <ets>septemviri</ets>, pl.; <ets>septem</ets> seven + <ets>viri</ets>, pl. of <ets>vir</ets> man.]</ety> <fld>(Rom. Hist.)</fld> <def>One of a board of seven men associated in some office.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Septemvirate</ent><br/
<hw>Sep*tem"vi*rate</hw> <pr>(?)</pr>, <pos>n.</pos><ety>[L. <ets>septemviratus</ets>.]</ety> <def>The office of septemvir; a government by septimvirs.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Septenary</ent><br/
<hw>Sep"ten*a*ry</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>septenairus</ets>, from <ets>septeni</ets> seven each, <ets>septem</ets> seven: cf. F. <ets>sept<eacute/naire</ets>. See <er>Seven</er>. ]</ety> <sn>1.</sn> <def>Consisting of, or relating to, seven; <as>as, a <ex>septenary</ex> number</as>.</def>  <rj><au>I. Watts.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Lasting seven years; continuing seven years.</def> <ldquo/<xex>Septenary</xex> penance.<rdquo/  <rj><au>Fuller.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Septenary</ent><br/
<hw>Sep"ten*a*ry</hw>, <pos>n.</pos> <def>The number seven.</def> <mark>[R.]</mark>  <rj><au>Holinshed.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Septenate</ent><br/
<hw>Sep"ten*ate</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>septeni</ets> seven each.]</ety> <fld>(Bot.)</fld> <def>Having parts in sevens; heptamerous.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Septennate</ent><br/
<hw>Sep*ten"nate</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F. <ets>septennat</ets>.]</ety> <def>A period of seven years; <as>as, the <ex>septennate</ex> during which the President of the French Republic holds office</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Septennial</ent><br/
<hw>Sep*ten"ni*al</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>septennium</ets> a period of seven years; <ets>septem</ets> seven +  <ets>annus</ets> year. See <er>Seven</er>, and <er>Annual</er>.]</ety> <sn>1.</sn> <def>Lasting or continuing seven years; <as>as, <ex>septennial</ex> parliaments</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Happening or returning once in every seven years; <as>as, <ex>septennial</ex> elections in England</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Septennially</ent><br/
<hw>Sep*ten"ni*al*ly</hw>, <pos>adv.</pos> <def>Once in seven years.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Septentrial</ent><br/
<hw>Sep*ten"tri*al</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Septentrional.</def>  <rj><au>Drayton.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Septentrio</ent><br/
||<hw>Sep*ten"tri*o</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. See <er>Septentrion</er>.]</ety> <fld>(Astron.)</fld> <def>The constellation Ursa Major.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Septentrion</ent><br/
<hw>Sep*ten"tri*on</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>septentrio</ets> the northern regions, the north, fr. <ets>septentriones</ets> the seven stars near the north pole, called Charles's Wain, or the Great Bear, also those called the Little Bear; properly, the seven plow oxen; <ets>septem</ets> seven + <ets>trio</ets>, orig., a plow ox: cf. F. <ets>septentrion</ets>.]</ety> <def>The north or northern regions.</def>  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Both East West, South and <qex>Septentrioun</qex>.</q> <rj><qau>Chaucer.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Septentrional</ent><br/
<ent>Septentrion</ent><br/
<mhw>{ <hw>Sep*ten"tri*on</hw> <pr>(?)</pr>, <hw>Sep*ten"tri*on*al</hw> <pr>(?)</pr>, }</mhw> <pos>a.</pos> <ety>[L. <ets>septentrionalis</ets>: cf. F. <ets>septentrional</ets>.]</ety> <def>Of or pertaining to the north; northern.</def> <ldquo/From cold <xex>septentrion</xex> blasts.<rdquo/  <rj><au>Milton.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Septentrionality</ent><br/
<hw>Sep*ten`tri*on*al"i*ty</hw> <pr>(?)</pr>, <pos>n.</pos> <def>Northerliness.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Septentrionally</ent><br/
<hw>Sep*ten"tri*on*al*ly</hw> <pr>(?)</pr>, <pos>adv.</pos> <def>Northerly.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Septentrionate</ent><br/
<hw>Sep*ten"tri*on*ate</hw> <pr>(?)</pr>, <pos>v. i.</pos> <def>To tend or point toward the north; to north.</def>  <rj><au>Sir T. Browne.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Septette</ent><br/
<ent>Septet</ent><br/
<mhw>{ <hw>Sep*tet"</hw>, <hw>Sep*tette"</hw>  }</mhw> <pr>(?)</pr>, <pos>n.</pos> <ety>[From L. <ets>septem</ets> seven, like <ets>duet</ets>, from L. <ets>duo</ets>.]</ety> <sn>1.</sn> <def>A set of seven persons or objects; <as>as, a <ex>septet</ex> of singers</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Mus.)</fld> <def>A musical composition for seven instruments or seven voices; -- called also <altname>septuor</altname>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Septfoil</ent><br/
<hw>Sept"foil</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F. <ets>sept</ets> seven (L. <ets>septem</ets>) + E. <ets>foil</ets> leaf: cf. L. <ets>septifolium</ets>.]</ety> <sn>1.</sn> <fld>(Bot.)</fld> <def>A European herb, the tormentil. See <er>Tormentil</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Arch.)</fld> <def>An ornamental foliation having seven lobes.  Cf. <er>Cinquefoil</er>, <er>Quarterfoil</er>, and <er>Trefoil</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <fld>(Eccl.Art.)</fld> <def>A typical figure, consisting of seven equal segments of a circle, used to denote the gifts of the Holy Chost, the seven sacraments as recognized by the Roman Catholic Church, etc.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Septi-</ent><br/
<hw>Sep"ti-</hw> <pr>(?)</pr>, <ety>[L. <ets>septem</ets> seven.]</ety> <def>A combining form meaning <xex>seven</xex>; <as>as, <ex>septi</ex>folious, seven-leaved; <ex>septi</ex>-lateral, seven-sided</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Septic</ent><br/
<hw>Sep"tic</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[<ets>Septi-</ets> + <ets>-ic</ets>.]</ety> <fld>(Math.)</fld> <def>Of the seventh degree or order.</def> -- <def2><pos>n.</pos> <fld>(Alg.)</fld> <def>A quantic of the seventh degree.</def></def2><br/
[<source>1913 Webster</source>]</p>

<p><ent>Septical</ent><br/
<ent>Septic</ent><br/
<mhw>{ <hw>Sep"tic</hw> <pr>(?)</pr>, <hw>Sep"tic*al</hw> <pr>(?)</pr>, }</mhw> <pos>a.</pos> <ety>[L. <ets>septicus</ets>, Gr. <?/<?/<?/<?/, fr. <?/<?/<?/<?/ to make putrid: cf. F. <ets>septique</ets>.]</ety> <def>Having power to promote putrefaction.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Septic</ent><br/
<hw>Sep"tic</hw>, <pos>n.</pos> <def>A substance that promotes putrefaction.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Septicaemia</ent><br/
||<hw>Sep`ti*cae"mi*a</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[NL., from Gr. <?/<?/<?/ putrefactive + <?/<?/<?/ blood.]</ety> <fld>(Med.)</fld> <def>A poisoned condition of the blood produced by the absorption into it of septic or putrescent material; blood poisoning.  It is marked by chills, fever, prostration, and inflammation of the different serous membranes and of the lungs, kidneys, and other organs.</def>
<-- Now septicemia.  A condition caused by bacterial infection, the effects being mediated by the release of toxins by bacteria, within the circulation.  (local infection is sepsis)  Also septemia, septic fever, hematosepsis, sapremia --><br/
[<source>1913 Webster</source>]</p>

<p><ent>Septically</ent><br/
<hw>Sep"tic*al*ly</hw> <pr>(?)</pr>, <pos>adv.</pos> <def>In a septic manner; in a manner tending to promote putrefaction.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Septicidal</ent><br/
<hw>Sep"ti*ci`dal</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[<ets>Septum</ets> + L. <ets>caedere</ets> to cut: cf. F. <ets>septicide</ets>.]</ety> <fld>(Bot.)</fld> <def>Dividing the partitions; -- said of a method of dehiscence in which a pod splits through the partitions and is divided into its component carpels.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Septicity</ent><br/
<hw>Sep*tic"i*ty</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[See <er>Septic</er>.]</ety> <def>Tendency to putrefaction; septic quality.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Septifarious</ent><br/
<hw>Sep`ti*fa"ri*ous</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>septifariam</ets> sevenfold.  Cf. <er>Bifarious</er>.]</ety> <fld>(Bot.)</fld> <def>Turned in seven different ways.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Septiferous</ent><br/
<hw>Sep*tif"er*ous</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[<ets>Septum</ets> + <ets>-ferous</ets>: cf. F. <ets>septif<egrave/re</ets>.]</ety> <fld>(Bot.)</fld> <def>Bearing a partition; -- said of the valves of a capsule.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Septiferous</ent><br/
<hw>Sep*tif"er*ous</hw>, <pos>a.</pos> <ety>[Gr. <?/<?/<?/ putrefied + <ets>-ferous</ets>.]</ety> <def>Conveying putrid poison; <as>as, the virulence of <ex>septiferous</ex> matter</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Septifluous</ent><br/
<hw>Sep*tif"lu*ous</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[CF. <er>Septemfluous</er>.]</ety> <def>Flowing in seven streams; septemfluous.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Septifolious</ent><br/
<hw>Sep`ti*fo"li*ous</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[<ets>Septi-</ets> + L. <ets>folium</ets> leaf.]</ety> <fld>(Bot.)</fld> <def>Having seven leaves.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Septiform</ent><br/
<hw>Sep"ti*form</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[<ets>Septum</ets> + <ets>-form</ets>.]</ety> <def>Having the form of a septum.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Septifragal</ent><br/
<hw>Sep*tif"ra*gal</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[<ets>Septum</ets> + L. <ets>frangere</ets>, <ets>fractum</ets>, to break.]</ety> <fld>(Bot.)</fld> <def>Breaking from the partitions; -- said of a method of dehiscence in which the valves of a pod break away from the partitions, and these remain attached to the common axis.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Septilateral</ent><br/
<hw>Sep`ti*lat"er*al</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[<ets>Septi-</ets> + <ets>lateral</ets>.]</ety> <def>Having seven sides; <as>as, a <ex>septilateral</ex> figure</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Septillion</ent><br/
<hw>Sep*til"lion</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F. <ets>septilion</ets>, formed fr. L. <ets>septem</ets> seven, in imitation of <ets>million</ets>.]</ety> <def>According to the French method of numeration (which is followed also in the United States), the number expressed by a unit with twenty-four ciphers annexed. According to the English method, the number expressed by a unit with forty-two ciphers annexed. See <er>Numeration</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Septimole</ent><br/
<hw>Sep"ti*mole</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L.  <ets>septem</ets> seven.]</ety> <fld>(Mus.)</fld> <def>A group of seven notes to be played in the time of four or six.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Septinsular</ent><br/
<hw>Sep*tin"su*lar</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[<ets>Sept</ets>i- + <ets>insular</ets>.]</ety> <def>Consisting of seven islands; <as>as, the <ex>septinsular</ex> republic of the Ionian Isles</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Septisyllable</ent><br/
<hw>Sep"ti*syl`la*ble</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[<ets>Septi-</ets> + <ets>syllable</ets>.]</ety> <def>A word of seven syllables.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Septoic</ent><br/
<hw>Sep*to"ic</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>septem</ets> seven.]</ety> <fld>(Chem.)</fld> <def>See <er>Heptoic</er>.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Septomaxillary</ent><br/
<hw>Sep`to*max"il*la*ry</hw> <pr>(?)</pr>, <pos>a.</pos> <fld>(Anat.)</fld> <def>Of or pertaining to the nasal septum and the maxilla; situated in the region of these parts.</def> -- <def2><pos>n.</pos> <def>A small bone between the nasal septum and the maxilla in many reptiles and amphibians.</def></def2><br/
[<source>1913 Webster</source>]</p>

<p><ent>Septuagenarian</ent><br/
<hw>Sep`tu*a*ge*na"ri*an</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A person who is seventy years of age; a septuagenary.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Septuagenary</ent><br/
<hw>Sep`tu*ag"e*na*ry</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>septuagenarius</ets>, fr. <ets>septuageny</ets> seventy each; akin to <ets>septuaginta</ets> seventy, <ets>septem</ets> seven. See <er>Seven</er>.]</ety> <def>Consisting of seventy; also, seventy years old.</def> -- <def2><pos>n.</pos> <def>A septuagenarian.</def></def2><br/
[<source>1913 Webster</source>]</p>

<p><ent>Septuagesima</ent><br/
||<hw>Sep`tu*a*ges"i*ma</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[NL., fr. L. <ets>septuagesimus</ets> the seventieth, fr. <ets>septuaginta</ets> seventy.]</ety> <fld>(Eccl.)</fld> <def>The third Sunday before Lent; -- so called because it is about seventy days before Easter.</def><br/
[<source>1913 Webster</source>]</p>

<p><-- p. 1313 --></p>

<p><ent>Septuagesimal</ent><br/
<hw>Sep`tu*a*ges"i*mal</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Consisting of seventy days, years, etc.; reckoned by seventies.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Our abridged and <qex>septuagesimal</qex> age.</q> <rj><qau>Sir T. Browne.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Septuagint</ent><br/
<hw>Sep"tu*a*gint</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[From L. <ets>septuaginta</ets> seventy.]</ety> <def>A Greek version of the Old Testament; -- so called because it was believed to be the work of seventy (or rather of seventy-two) translators.</def><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ The causes which produced it [the Septuagint], the number and names of the translators, the times at which different portions were translated, are all uncertain. The only point in which all agree is that Alexandria was the birthplace of the version. On one other point there is a near agreement, namely, as to time, that the version was made, or at least commenced, in the time of the early Ptolemies, in the first half of the third century b.c.</note>  <rj><au>Dr. W. Smith (Bib. Dict.)</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Septuagint chronology</b></col>, <cd>the chronology founded upon the dates of the Septuagint, which makes 1500 years more from the creation to Abraham than the Hebrew Bible.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Septuary</ent><br/
<hw>Sep"tu*a*ry</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>septem</ets> seven.]</ety> <def>Something composed of seven; a week.</def> <mark>[R.]</mark>  <rj><au>Ash.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Septulate</ent><br/
<hw>Sep"tu*late</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[Dim. fr. <ets>septum</ets>.]</ety> <fld>(Bot.)</fld> <def>Having imperfect or spurious septa.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Septulum</ent><br/
||<hw>Sep"tu*lum</hw> <pr>(?)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Septula</plw> <pr>(#)</pr>.</plu> <ety>[NL., dim. of L. <ets>septum</ets> septum.]</ety> <fld>(Anat.)</fld> <def>A little septum; a division between small cavities or parts.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Septum</ent><br/
||<hw>Sep"tum</hw> <pr>(?)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Septa</plw> <pr>(#)</pr>.</plu> <ety>[L. <ets>septum</ets>, <ets>saeptum</ets>, an inclosure, hedge, fence, fr. <ets>sepire</ets>, <ets>saepire</ets>, to hedge in, inclose.]</ety> <sn>1.</sn> <def>A wall separating two cavities; a partition; <as>as, the nasal <ex>septum</ex></as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Bot.)</fld> <def>A partition that separates the cells of a fruit.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <fld>(Zool.)</fld> <sd>(a)</sd> <def>One of the radial calcareous plates of a coral.</def> <sd>(b)</sd> <def>One of the transverse partitions dividing the shell of a mollusk, or of a rhizopod, into several chambers. See <xex>Illust.</xex> under <er>Nautilus</er>.</def> <sd>(c)</sd> <def>One of the transverse partitions dividing the body cavity of an annelid.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Septuor</ent><br/
<hw>Sep"tu*or</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F.]</ety> <fld>(Mus.)</fld> <def>A septet.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Septuple</ent><br/
<hw>Sep"tu*ple</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[LL. <ets>septuplus</ets>; cf. Gr. <?/<?/<?/<?/<?/:cf. F. <ets>septuple</ets>.  Cf. <er>Double</er>, <er>Quadruple</er>.]</ety> <def>Seven times as much; multiplied by seven; sevenfold.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Septuple</ent><br/
<hw>Sep"tu*ple</hw>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Septupled</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Septupling</conjf> <pr>(?)</pr>.]</vmorph> <def>To multiply by seven; to make sevenfold.</def>  <rj><au>Sir J. Herschel.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sepulchre</ent><br/
<ent>Sepulcher</ent><br/
<mhw>{ <hw>Sep"ul*cher</hw>, <hw>Sep"ul*chre</hw>  }</mhw> <pr>(?)</pr>, <pos>n.</pos> <ety>[OE. <ets>sepulcre</ets>, OF. sepulcre, F. <ets>s<eacute/pulcre</ets>, fr. L. <ets>sepulcrum</ets>, <ets>sepulchrum</ets>, fr. <ets>sepelire</ets> to bury.]</ety> <def>The place in which the dead body of a human being is interred, or a place set apart for that purpose; a grave; a tomb.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The stony entrance of this <qex>sepulcher</qex>.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>The first day of the week cometh Mary Magdalene early, when it was yet dark, unto the <qex>sepulcher</qex>.</q> <rj><qau>John xx. 1.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>A whited sepulcher</b></col>. <cd>Fig.: Any person who is fair outwardly but unclean or vile within.  See <au>Matt. xxiii. 27.</au></cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sepulchre</ent><br/
<ent>Sepulcher</ent><br/
<mhw>{ <hw>Sep"ul*cher</hw>, <hw>Sep"ul*chre</hw>  }</mhw> <pr>(?)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Sepulchered</conjf> <pr>(?)</pr> or <conjf>Sepulchred</conjf> <pr>(<?/)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Sepulchering</conjf> <pr>(?)</pr> or <conjf>Sepulchring</conjf> <pr>(<?/)</pr>.]</vmorph> <def>To bury; to inter; to entomb; <as>as, obscurely <ex>sepulchered</ex></as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>And so <qex>sepulchered</qex> in such pomp dost lie<br/
That kings for such a tomb would wish to die.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sepulchral</ent><br/
<hw>Se*pul"chral</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>sepulcralis</ets>: cf. F. <ets>s<eacute/pulcral</ets>.]</ety> <sn>1.</sn> <def>Of or pertaining to burial, to the grave, or to monuments erected to the memory of the dead; <as>as, a <ex>sepulchral</ex> stone; a <ex>sepulchral</ex> inscription.</as></def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Unnaturally low and grave; hollow in tone; -- said of sound, especially of the voice.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>This exaggerated dulling of the voice . . . giving what is commonly called a <qex>sepulchral</qex> tone.</q> <rj><qau>H. Sweet.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sepulture</ent><br/
<hw>Sep"ul*ture</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F. <ets>s<eacute/pulture</ets>, L. <ets>sepultura</ets>, fr. <ets>sepelire</ets>, <ets>sepultum</ets>, to bury.]</ety> <sn>1.</sn> <def>The act of depositing the dead body of a human being in the grave; burial; interment.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Where we may royal <qex>sepulture</qex> prepare.</q> <rj><qau>Dryden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A sepulcher; a grave; a place of burial.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Drunkeness that is the horrible <qex>sepulture</qex> of man's reason.</q> <rj><qau>Chaucer.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sequacious</ent><br/
<hw>Se*qua"cious</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>sequax</ets>, <ets>-acis</ets>, fr. <ets>suquit</ets> to follow. See <er>Sue</er> to follow. ]</ety> <sn>1.</sn> <def>Inclined to follow a leader; following; attendant.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Trees uprooted left their place,<br/
<qex>Sequacious</qex> of the lyre.</q> <rj><qau>Dryden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Hence, ductile; malleable; pliant; manageable.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>In the greater bodies the forge was easy, the matter being ductile and <qex>sequacious</qex>.</q> <rj><qau>Ray.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>Having or observing logical sequence; logically consistent and rigorous; consecutive in development or transition of thought.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The scheme of pantheistic omniscience so prevalent among the <qex>sequacious</qex> thinkers of the day.</q> <rj><qau>Sir W. Hamilton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Milton was not an extensive or discursive thinker, as Shakespeare was; for the motions of his mind were slow, solemn, and <qex>sequacious</qex>, like those of the planets.</q> <rj><qau>De Quincey.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sequaciousness</ent><br/
<hw>Se*qua"cious*ness</hw>, <pos>n.</pos> <def>Quality of being sequacious.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sequacity</ent><br/
<hw>Se*quac"i*ty</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>sequacitas</ets>.]</ety> <def>Quality or state of being sequacious; sequaciousness.</def>  <rj><au>Bacon.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sequel</ent><br/
<hw>Se"quel</hw> <pr>(s<emac/"kw<ecr/l)</pr>, <pos>n.</pos> <ety>[L. <ets>sequela</ets>, fr. <ets>sequit</ets> to follow: cf. F. <ets>s<eacute/quelle</ets> a following. See <er>Sue</er> to follow.]</ety> <sn>1.</sn> <def>That which follows; a succeeding part; continuation; <as>as, the <ex>sequel</ex> of a man's advantures or history</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>O, let me say no more!<br/
Gather the <qex>sequel</qex> by that went before.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Consequence; event; effect; result; <as>as, let the sun cease, fail, or swerve, and the <ex>sequel</ex> would be ruin</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>Conclusion; inference.</def> <mark>[R.]</mark>  <rj><au>Whitgift.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sequela</ent><br/
||<hw>Se*que"la</hw> <pr>(?)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Sequelae</plw> <pr>(#)</pr>.</plu> <ety>[L., a follower, a result, from <ets>sequit</ets> to follow.]</ety> <def>One who, or that which, follows.</def> Specifically: <sd>(a)</sd> <def>An adherent, or a band or sect of adherents.</def> <ldquo/Coleridge and his <xex>sequela</xex>.<rdquo/ <au>G. P. Marsh.</au> <sd>(b)</sd> <def>That which follows as the logical result of reasoning; inference; conclusion; suggestion.</def><br/
[<source>1913 Webster</source>]</p>

<p><q><qex>Sequelae</qex>, or thoughts suggested by the preceding aphorisms.</q> <rj><qau>Coleridge.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sd>(c)</sd> <fld>(Med.)</fld> <def>A morbid phenomenon left as the result of a disease; a disease resulting from another.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sequence</ent><br/
<hw>Se"quence</hw> <pr>(s<emac/"kw<eit/ns)</pr>, <pos>n.</pos> <ety>[F. <ets>s<eacute/quence</ets>, L.  <ets>sequentia</ets>, fr.  <ets>sequens</ets>. See <er>Sequent</er>.]</ety> <sn>1.</sn> <def>The state of being sequent; succession; order of following; arrangement.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>How art thou a king<br/
But by fair <qex>sequence</qex> and succession?</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q><qex>Sequence</qex> and series of the seasons of the year.</q> <rj><qau>Bacon.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>That which follows or succeeds as an effect; sequel; consequence; result.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The inevitable <qex>sequences</qex> of sin and punishment.</q> <rj><qau>Bp. Hall.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <fld>(Philos.)</fld> <def>Simple succession, or the coming after in time, without asserting or implying causative energy; <as>as, the reactions of chemical agents may be conceived as merely invariable <ex>sequences</ex></as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <fld>(Mus.)</fld> <sd>(a)</sd> <def>Any succession of chords (or harmonic phrase) rising or falling by the regular diatonic degrees in the same scale; a succession of similar harmonic steps.</def> <sd>(b)</sd> <def>A melodic phrase or passage successively repeated one tone higher; a rosalia.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <fld>(R.C.Ch.)</fld> <def>A hymn introduced in the Mass on certain festival days, and recited or sung immediately before the gospel, and after the gradual or introit, whence the name.</def>  <rj><au>Bp. Fitzpatrick.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Originally the <qex>sequence</qex> was called a Prose, because its early form was rhythmical prose.</q> <rj><qau>Shipley.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>6.</sn> <fld>(Card Playing)</fld> <sd>(a)</sd> <fld>(Whist)</fld> <def>Three or more cards of the same suit in immediately consecutive order of value; as, ace, king, and queen; or knave, ten, nine, and eight.</def> <sd>(b)</sd> <fld>(Poker)</fld> <def>All five cards, of a hand, in consecutive order as to value, but not necessarily of the same suit; when of one suit, it is called a <stype>sequence flush</stype>.</def>
<-- sequence is usu. called a run, and five are now called straight and straight flush --><br/
[<source>1913 Webster</source>]</p>

<p><sn>7.</sn> <def>the specific order of any linear arrangement of items; <as>as, the <ex>sequence</ex> of amino acid residues in a protein; the <ex>sequence</ex> of instructions in a computer program; the <ex>sequence</ex> of acts in a variety show</as>.</def><br/
[<source>PJC</source>]</p>

<p><ent>Sequence</ent><br/
<hw>Se"quence</hw> <pr>(?)</pr>, <pos>v. t.</pos> <fld>(Biochem.)</fld> <def>to determine the sequence of; <as>as, to <ex>sequence</ex> a protein or a DNA fragment</as>.</def><br/
[<source>PJC</source>]</p>

<p><ent>Sequent</ent><br/
<hw>Se"quent</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>sequens</ets>, <ets>-entis</ets>, p. pr. of <ets>sequi</ets> to follow. See <er>Sue</er> to follow.]</ety> <sn>1.</sn> <def>Following; succeeding; in continuance.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>What to this was <qex>sequent</qex><br/
Thou knowest already.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Following as an effect; consequent.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sequent</ent><br/
<hw>Se"quent</hw>, <pos>n.</pos> <sn>1.</sn> <def>A follower.</def> <mark>[R.]</mark>  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>That which follows as a result; a sequence.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sequential</ent><br/
<hw>Se*quen"tial</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Succeeding or following in order.</def> -- <wordforms><wf>Se*quen"tial*ly</wf>, <pos>adv.</pos></wordforms><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sequester</ent><br/
<hw>Se*ques"ter</hw> <pr>(?)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Sequestered</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Sequestering</conjf>.]</vmorph> <ety>[F. <ets>s<eacute/questrer</ets>, L. <ets>sequestrare</ets> to give up for safe keeping, from <ets>sequester</ets> a depositary or trustee in whose hands the thing contested was placed until the dispute was settled.  Cf. <er>Sequestrate</er>.]</ety> <sn>1.</sn> <fld>(Law)</fld> <def>To separate from the owner for a time; to take from parties in controversy and put into the possession of an indifferent person; to seize or take possession of, as property belonging to another, and hold it till the profits have paid the demand for which it is taken, or till the owner has performed the decree of court, or clears himself of contempt; in international law, to confiscate.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Formerly the goods of a defendant in chancery were, in the last resort, <qex>sequestered</qex> and detained to enforce the decrees of the court. And now the profits of a benefice are <qex>sequestered</qex> to pay the debts of ecclesiastics.</q> <rj><qau>Blackstone.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To cause (one) to submit to the process of sequestration; to deprive (one) of one's estate, property, etc.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>It was his tailor and his cook, his fine fashions and his French ragouts, which <qex>sequestered</qex> him.</q> <rj><qau>South.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>To set apart; to put aside; to remove; to separate from other things.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>I had wholly <qex>sequestered</qex> my civil affairss.</q> <rj><qau>Bacon.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>To cause to retire or withdraw into obscurity; to seclude; to withdraw; -- often used reflexively.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>When men most <qex>sequester</qex> themselves from action.</q> <rj><qau>Hooker.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>A love and desire to <qex>sequester</qex> a man's self for a higher conversation.</q> <rj><qau>Bacon.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><-- 5. (Chem.) To bind, so as to make [a metal ion] unavailable in its normal form; -- said of chelating agents, such as EDTA, which, in a solution, bind tightly to multivalent metal cations, thereby lowering their effective concentration in solution.  Compounds employed particularly for this purpose are called sequestering agents, or chelating agents.  In biochemistry,
sequestration is one means of reversibly inhibiting enzymes which depend on divalent metal cations (such as Magnesium) for their activity.  Such agents are used, for example, to help preserve blood for storage and subsequent use in transfusion. --><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sequester</ent><br/
<hw>Se*ques"ter</hw>, <pos>v. i.</pos> <sn>1.</sn> <def>To withdraw; to retire.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>To <qex>sequester</qex> out of the world into Atlantic and Utopian politics.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Law)</fld> <def>To renounce (as a widow may) any concern with the estate of her husband.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sequester</ent><br/
<hw>Se*ques"ter</hw>, <pos>n.</pos> <sn>1.</sn> <def>Sequestration; separation.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Law)</fld> <def>A person with whom two or more contending parties deposit the subject matter of the controversy; one who mediates between two parties; a mediator; an umpire or referee.</def>  <rj><au>Bouvier.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <fld>(Med.)</fld> <def>Same as <er>Sequestrum</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sequestered</ent><br/
<hw>Se*ques"tered</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Retired; secluded.</def> <ldquo/<xex>Sequestered</xex> scenes.<rdquo/  <rj><au>Cowper.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Along the cool, <qex>sequestered</qex> vale of life.</q> <rj><qau>Gray.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sequestrable</ent><br/
<hw>Se*ques"tra*ble</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Capable of being sequestered; subject or liable to sequestration.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sequestral</ent><br/
<hw>Se*ques"tral</hw> <pr>(?)</pr>, <pos>a.</pos> <fld>(Med.)</fld> <def>Of or pertaining to a sequestrum.</def>  <rj><au>Quian.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sequestrate</ent><br/
<hw>Se*ques"trate</hw> <pr>(?)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Sequestrated</conjf>; <pos>p. pr. & vb. n.</pos> <conjf>Sequestrating</conjf>.]</vmorph> <def>To sequester.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sequestration</ent><br/
<hw>Seq`ues*tra"tion</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>sequestratio</ets>: cf. F. <ets>s<eacute/questration</ets>.]</ety> <sn>1.</sn> <sd>(a)</sd> <fld>(Civil & Com. Law)</fld> <def>The act of separating, or setting aside, a thing in controversy from the possession of both the parties that contend for it, to be delivered to the one adjudged entitled to it. It may be voluntary or involuntary.</def> <sd>(b)</sd> <fld>(Chancery)</fld> <def>A prerogative process empowering certain commissioners to take and hold a defendant's property and receive the rents and profits thereof, until he clears himself of a contempt or performs a decree of the court.</def> <sd>(c)</sd> <fld>(Eccl. Law)</fld> <def>A kind of execution for a rent, as in the case of a beneficed clerk, of the profits of a benefice, till he shall have satisfied some debt established by decree; the gathering up of the fruits of a benefice during a vacancy, for the use of the next incumbent; the disposing of the goods, by the ordinary, of one who is dead, whose estate no man will meddle with.</def> <au>Craig.</au> <au>Tomlins.</au> <au>Wharton.</au> <sd>(d)</sd> <fld>(Internat. Law)</fld> <def>The seizure of the property of an individual for the use of the state; particularly applied to the seizure, by a belligerent power, of debts due from its subjects to the enemy.</def>  <rj><au>Burrill.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>The state of being separated or set aside; separation; retirement; seclusion from society.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Since Henry Monmouth first began to reign, . . . <br/
This loathsome <qex>sequestration</qex> have I had.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>Disunion; disjunction.</def> <mark>[Obs.]</mark>  <rj><au>Boyle.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><-- 4. (Chem.) the act or process of sequestering (v. t. sense 5) --><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sequestrator</ent><br/
<hw>Seq"ues*tra`tor</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L., one that hinders or impedes.]</ety> <fld>(Law)</fld> <sd>(a)</sd> <def>One who sequesters property, or takes the possession of it for a time, to satisfy a demand out of its rents or profits.</def> <sd>(b)</sd> <def>One to whom the keeping of sequestered property is committed.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sequestrum</ent><br/
||<hw>Se*ques"trum</hw> <pr>(?)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Sequestra</plw> <pr>(#)</pr>.</plu> <ety>[NL. See <er>Sequester</er>.]</ety> <fld>(Med.)</fld> <def>A portion of dead bone which becomes separated from the sound portion, as in necrosis.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sequin</ent><br/
<hw>Se"quin</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F. <ets>sequin</ets>, It. <ets>zecchino</ets>, from <ets>zecca</ets> the mint, fr. Ar. <ets>sekkah</ets>, <ets>sikkah</ets>, a die, a stamp.  Cf. <er>Zechin</er>.]</ety> <def>An old gold coin of Italy and Turkey. It was first struck at Venice about the end of the 13th century, and afterward in the other Italian cities, and by the Levant trade was introduced into Turkey. It is worth about 9s. 3d. sterling, or about $2.25. The different kinds vary somewhat in value.</def> <altsp>[Written also <asp>chequin</asp>, and <asp>zequin</asp>.]</altsp><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sequoia</ent><br/
<hw>Se*quoi"a</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[NL. So called by Dr. Endlicher in honor of <ets>Sequoyah</ets>, who invented the Cherokee alphabet.]</ety> <fld>(Bot.)</fld> <def>A genus of coniferous trees, consisting of two species, <spn>Sequoia Washingtoniana</spn>, syn. <spn>Sequoia gigantea</spn>, the <ldquo/big tree<rdquo/ of California, and <spn>Sequoia sempervirens</spn>, the redwood, both of which attain an immense height.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sequoiene</ent><br/
<hw>Se*quoi"ene</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Chem.)</fld> <def>A hydrocarbon (<chform>C13H10</chform>) obtained in white fluorescent crystals, in the distillation products of the needles of the California <ldquo/big tree<rdquo/ (<spn>Sequoia gigantea</spn>).</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Serac</ent><br/
||<hw>S<eacute/`rac"</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F. (in the Alps), orig., a kind of solid cheese.]</ety> <def>A pinnacle of ice among the crevasses of a glacier; also, one of the blocks into which a glacier breaks on a steep grade.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Seraglio</ent><br/
<hw>Se*ragl"io</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[It. <ets>serraglio</ets>, originally, an inclosure of palisades, afterwards also, a palace, seraglio (by confusion with Per. <ets>ser<amac/<ium/</ets>a a palace, an entirely different word), fr. <ets>serrare</ets> to shut, fr. LL. <ets>serra</ets> a bar for fastening doors, L. <ets>sera</ets>. See <er>Serry</er>, <er>Series</er>.]</ety> <sn>1.</sn> <def>An inclosure; a place of separation.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>I went to the Ghetto, where the Jews dwell as in a suburb, by themselves. I passed by the piazza Judea, where their <qex>seraglio</qex> begins.</q> <rj><qau>Evelyn.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>The palace of the Grand Seignior, or Turkish sultan, at Constantinople, inhabited by the sultan himself, and all the officers and dependents of his court. In it are also kept the females of the harem.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>A harem; a place for keeping wives or concubines; sometimes, loosely, a place of licentious pleasure; a house of debauchery.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Serai</ent><br/
||<hw>Se*ra"i</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Per. <ets>ser<amac/<ium/</ets>, or <ets>sar<amac/<ium/</ets>, a palace, a king's court, a seraglio, an inn.  Cf. <er>Caravansary</er>.]</ety> <def>A palace; a seraglio; also, in the East, a place for the accommodation of travelers; a caravansary, or rest house.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Seralbumen</ent><br/
<hw>Ser`al*bu"men</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Physiol. CHem.)</fld> <def>Serum albumin.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Serang</ent><br/
||<hw>Se*rang"</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Per. <ets>sarhang</ets> a commander.]</ety> <def>The boatswain of a Lascar or East Ondian crew.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Serape</ent><br/
||<hw>Se*ra"pe</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Sp. Amer. <ets>sarape</ets>.]</ety> <def>A blanket or shawl worn as an outer garment by the Spanish Americans, as in Mexico.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Seraph</ent><br/
<hw>Ser"aph</hw> <pr>(?)</pr>, <pos>n.</pos>; <plu><it>pl.</it> E. <plw>Seraphs</plw> <pr>(#)</pr>, Heb. <plw>Seraphim</plw> <pr>(#)</pr>.</plu> <ety>[Heb. <ets>ser<amac/phim</ets>, pl.]</ety> <def>One of an order of celestial beings, each having three pairs of wings. In ecclesiastical art and in poetry, a seraph is represented as one of a class of angels.</def>  <rj><au>Isa. vi. 2.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>As full, as perfect, in vile man that mourns,<br/
As the rapt <qex>seraph</qex> that adores and burns.</q> <rj><qau>Pope.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Seraph moth</b></col> <fld>(Zool.)</fld>, <cd>any one of numerous species of geometrid moths of the genus <gen>Lobophora</gen>, having the hind wings deeply bilobed, so that they seem to have six wings.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Seraphical</ent><br/
<ent>Seraphic</ent><br/
<mhw>{ <hw>Se*raph"ic</hw> <pr>(?)</pr>, <hw>Se*raph"ic*al</hw> <pr>(?)</pr>, }</mhw> <pos>a.</pos> <ety>[Cf. F. <ets>s<eacute/raphique</ets>.]</ety> <def>Of or pertaining to a seraph; becoming, or suitable to, a seraph; angelic; sublime; pure; refined.</def> <ldquo/<xex>Seraphic</xex> arms and trophies.<rdquo/ <au>Milton.</au> <ldquo/<xex>Seraphical</xex> fervor.<rdquo/ <au>Jer. Taylor.</au> -- <wordforms><wf>Se*raph"ic*al*ly</wf>, <pos>adv.</pos> -- <wf>Se*raph"ic*al*ness</wf>, <pos>n.</pos></wordforms><br/
[<source>1913 Webster</source>]</p>

<p><ent>Seraphicism</ent><br/
<hw>Se*raph"i*cism</hw> <pr>(?)</pr>, <pos>n.</pos> <def>The character, quality, or state of a seraph; seraphicalness.</def> <mark>[R.]</mark>  <rj><au>Cudworth.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Seraphim</ent><br/
<hw>Ser"a*phim</hw> <pr>(?)</pr>, <pos>n.</pos> <def>The Hebrew plural of <er>Seraph</er>.  Cf. <er>Cherubim</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ The double plural form <xex>seraphims</xex> is sometimes used, as in the King James version of the Bible, <au>Isa. vi. 2 and 6.</au></note><br/
[<source>1913 Webster</source>]</p>

<p><-- p. 1314 --></p>

<p><ent>Seraphina</ent><br/
<hw>Ser`a*phi"na</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[NL.]</ety> <def>A seraphine.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Seraphine</ent><br/
<hw>Ser"a*phine</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[From <er>Seraph</er>.]</ety> <fld>(Mus.)</fld> <def>A wind instrument whose sounding parts are reeds, consisting of a thin tongue of brass playing freely through a slot in a plate. It has a case, like a piano, and is played by means of a similar keybord, the bellows being worked by the foot. The <xex>melodeon</xex> is a portable variety of this instrument.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Serapis</ent><br/
||<hw>Se*ra"pis</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L., fr. Gr. <?/<?/<?/, <?/<?/<?/.]</ety> <fld>(Myth.)</fld> <def>An Egyptian deity, at first a symbol of the Nile, and so of fertility; later, one of the divinities of the lower world. His worship was introduced into Greece and Rome.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Seraskier</ent><br/
<hw>Se*ras"kier</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Turk., fr. Per. <ets>ser</ets> head, chief + Ar. <ets>'asker</ets> an army.]</ety> <def>A general or commander of land forces in the Turkish empire; especially, the commander-in-chief of minister of war.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Seraskierate</ent><br/
<hw>Se*ras"kier*ate</hw> <pr>(?)</pr>, <pos>n.</pos> <def>The office or authority of a seraskier.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Serbonian</ent><br/
<hw>Ser*bo"ni*an</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Relating to the lake of Serbonis in Egypt, which by reason of the sand blowing into it had a deceptive appearance of being solid land, but was a bog.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>A gulf profound as that <qex>Serbonian</qex> bog . . . <br/
Where armies whole have sunk.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sere</ent><br/
<hw>Sere</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Dry; withered. Same as <er>Sear</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>But with its sound it shook the sails<br/
That were so thin and <qex>sere</qex>.</q> <rj><qau>Coleridge.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sere</ent><br/
<hw>Sere</hw>, <pos>n.</pos> <ety>[F. <ets>serre</ets>.]</ety> <def>Claw; talon.</def> <mark>[Obs.]</mark>  <rj><au>Chapman.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Serein</ent><br/
||<hw>Se*rein"</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F.  Cf. <er>Serenade</er>, <pos>n.</pos>]</ety> <fld>(Meteorol.)</fld> <def>A mist, or very fine rain, which sometimes falls from a clear sky a few moments after sunset.</def>  <rj><au>Tyndall.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Serenade</ent><br/
<hw>Ser`e*nade"</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F. <ets>s<eacute/r<eacute/nade</ets>, It. <ets>serenata</ets>, probably fr. L. <ets>serenus</ets> serene (cf. <er>Serene</er>), misunderstood as a derivative fr. L. <ets>serus</ets> late.  Cf. <er>Soir<eacute/e</er>.]</ety> <fld>(Mus.)</fld> <sd>(a)</sd> <def>Music sung or performed in the open air at nights; -- usually applied to musical entertainments given in the open air at night, especially by gentlemen, in a spirit of gallantry, under the windows of ladies.</def> <sd>(b)</sd> <def>A piece of music suitable to be performed at such times.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Serenade</ent><br/
<hw>Ser`e*nade"</hw>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Serenaded</conjf>; <pos>p. pr. & vb. n.</pos> <conjf>Serenading</conjf>.]</vmorph> <def>To entertain with a serenade.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Serenade</ent><br/
<hw>Ser`e*nade"</hw>, <pos>v. i.</pos> <def>To perform a serenade.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Serenader</ent><br/
<hw>Ser`e*nad"er</hw> <pr>(?)</pr>, <pos>n.</pos> <def>One who serenades.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Serenate</ent><br/
<ent>Serenata</ent><br/
<mhw>{ <hw>Ser`e*na"ta</hw> <pr>(?)</pr>, <hw>Ser"e*nate</hw> <pr>(?)</pr>, }</mhw> <pos>n.</pos> <ety>[It. <ets>serenata</ets>. See <er>Serenade</er>.]</ety> <fld>(Mus.)</fld> <def>A piece of vocal music, especially one on an amoreus subject; a serenade.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Or <qex>serenate</qex>, which the starved lover sings<br/
To his pround fair.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ The name <xex>serenata</xex> was given by Italian composers in the time of Handel, and by Handel himself, to a cantata of a pastoreal of dramatic character, to a secular ode, etc.; also by Mozart and others to an orchectral composition, in several movements, midway between the suite of an earlier period and the modern symphony.</note>  <rj><au>Grove.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Serene</ent><br/
<hw>Se*rene"</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>serenus</ets> to grow dry, Gr. <?/<?/<?/ hot, scorching.]</ety> <sn>1.</sn> <def>Bright; clear; unabscured; <as>as, a <ex>serene</ex> sky</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The moon <qex>serene</qex> in glory mounts the sky.</q> <rj><qau>Pope.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Full many a gem of purest ray <qex>serene</qex><br/
The dark unfathomed caves of ocean bear.</q> <rj><qau>Gray.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Calm; placid; undisturbed; unruffled; <as>as, a <ex>serene</ex> aspect; a <ex>serene</ex> soul</as>.</def>  <rj><au>Milton.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ In several countries of Europe, <xex>Serene</xex> is given as a tittle to princes and the members of their families; as, His <xex>Serene</xex> Highness.</note><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Drop serene</b></col>. <fld>(Med.)</fld> <cd>See <er>Amaurosis</er>.</cd>  <rj><au>Milton.</au></rj></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Serene</ent><br/
<hw>Se*rene"</hw>, <pos>n.</pos> <sn>1.</sn> <def>Serenity; clearness; calmness.</def> <mark>[Poetic.]</mark> <ldquo/The <xex>serene</xex> of heaven.<rdquo/  <rj><au>Southey.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>To their master is denied<br/
To share their sweet <qex>serene</qex>.</q> <rj><qau>Young.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <ety>[F. <ets>serein</ets> evening dew or damp. See <er>Serein</er>.]</ety> <def>Evening air; night chill.</def> <mark>[Obs.]</mark> <ldquo/Some <xex>serene</xex> blast me.<rdquo/  <rj><au>B. Jonson.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Serene</ent><br/
<hw>Se*rene"</hw>, <pos>v. t.</pos> <ety>[L. <ets>serenare</ets>.]</ety> <def>To make serene.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Heaven and earth, as if contending, vie<br/
To raise his being, and <qex>serene</qex> his soul.</q> <rj><qau>Thomson.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Serenely</ent><br/
<hw>Se*rene"ly</hw>, <pos>adv.</pos> <sn>1.</sn> <def>In a serene manner; clearly.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Now setting Ph<oe/bus shone <qex>serenely</qex> bright.</q> <rj><qau>Pope.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>With unruffled temper; coolly; calmly.</def>  <rj><au>Prior.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sereneness</ent><br/
<hw>Se*rene"ness</hw>, <pos>n.</pos> <def>Serenity.</def>  <rj><au>Feltham.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Serenitude</ent><br/
<hw>Se*ren"i*tude</hw> <pr>(?)</pr>, <pos>n.</pos> <def>Serenity.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Serenity</ent><br/
<hw>Se*ren"i*ty</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>serenuas</ets>: cf. F. <ets>s<eacute/r<eacute/nit<eacute/</ets>.]</ety> <sn>1.</sn> <def>The quality or state of being serene; clearness and calmness; quietness; stillness; peace.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>A general peace and <qex>serenity</qex> newly succeeded a general trouble.</q> <rj><qau>Sir W. Temple.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Calmness of mind; eveness of temper; undisturbed state; coolness; composure.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>I can not see how any men should ever transgress those moral rules with confidence and <qex>serenity</qex>.</q> <rj><qau>Locke.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ <xex>Serenity</xex> is given as a title to the members of certain princely families in Europe; as, Your <xex>Serenity</xex>.</note><br/
[<source>1913 Webster</source>]</p>

<p><ent>Serf</ent><br/
<hw>Serf</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F., fr. L. <ets>serus</ets> servant, slave; akin to <ets>servare</ets> to protect, preserve, observe, and perhaps originally, a client, a man under one's protection.  Cf. <er>Serve</er>, <pos>v. t.</pos>]</ety> <def>A servant or slave employed in husbandry, and in some countries attached to the soil and transferred with it, as formerly in Russia.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>In England, at least from the reign of Henry II, one only, and that the inferior species [of villeins], existed . . . But by the customs of France and Germany, persons in this abject state seem to have been called <qex>serfs</qex>, and distinguished from <qex>villeins</qex>, who were only bound to fixed payments and duties in respect of their lord, though, as it seems, without any legal redress if injured by him.</q> <rj><qau>Hallam.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><syn><b>Syn.</b> -- <er>Serf</er>, <er>Slave</er>.</syn> <usage> A <xex>slave</xex> is the absolute property of his master, and may be sold in any way. A <xex>serf</xex>, according to the strict sense of the term, is one bound to work on a certain estate, and thus attached to the soil, and sold with it into the service of whoever purchases the land.</usage><br/
[<source>1913 Webster</source>]</p>

<p><ent>Serfdom</ent><br/
<ent>Serfage</ent><br/
<mhw>{ <hw>Serf"age</hw> <pr>(?)</pr>, <hw>Serf"dom</hw> <pr>(?)</pr>, }</mhw> <pos>n.</pos> <def>The state or condition of a serf.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Serfism</ent><br/
<ent>Serfhood</ent><br/
<mhw><hw>Serf"hood</hw> <pr>(?)</pr>, <hw>Serf"ism</hw> <pr>(<?/)</pr></mhw>, <pos>n.</pos> <def>Serfage.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Serge</ent><br/
<hw>Serge</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F. <ets>serge</ets>, <ets>sarge</ets>, <ets>originally</ets>, <ets>a silken stuff</ets>, fr. L. <ets>serica</ets>, f. or neut. pl. of <ets>sericus</ets> silken. See <er>Sericeous</er>, <er>Silk</er>.]</ety> <def>A woolen twilled stuff, much used as material for clothing for both sexes.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Silk serge</b></col>, <cd>a twilled silk fabric used mostly by tailors for lining parts of gentlemen's coats.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Serge</ent><br/
<hw>Serge</hw>, <pos>n.</pos> <ety>[F. <ets>cierge</ets>.]</ety> <def>A large wax candle used in the ceremonies of various churches.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sergeancy</ent><br/
<hw>Ser"gean*cy</hw> <pr>(?)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Sergeancies</plw> <pr>(#)</pr>.</plu> <ety>[Cf. <er>Sergeanty</er>.]</ety> <def>The office of a sergeant; sergeantship.</def> <altsp>[Written also <asp>serjeancy</asp>.]</altsp><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sergeant</ent><br/
<hw>Ser"geant</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F. <ets>sergent</ets>, fr. L. <ets>serviens</ets>, <ets>-entis</ets>, p. pr. of <ets>servire</ets> to serve. See <er>Serve</er>, and cf. <er>Servant</er>.]</ety> <altsp>[Written also <asp>serjeant</asp>. Both spellings are authorized. In England <asp>serjeant</asp> is usually preferred, except for military officers. In the United States <asp>sergeant</asp> is common for civil officers also.]</altsp> <sn>1.</sn> <def>Formerly, in England, an officer nearly answering to the more modern bailiff of the hundred; also, an officer whose duty was to attend on the king, and on the lord high steward in court, to arrest traitors and other offenders. He is now called <xex>sergeant-at-arms</xex>, and two of these officers, by allowance of the sovereign, attend on the houses of Parliament (one for each house) to execute their commands, and another attends the Court Chancery.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The <qex>sergeant</qex> of the town of Rome them sought.</q> <rj><qau>Chaucer.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>The magistrates sent the <qex>serjeant</qex>, saying, Let those men go.</q> <rj><qau>Acts xvi. 35.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>This fell <qex>sergeant</qex>, Death,<br/
Is strict in his arrest.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Mil.)</fld> <def>In a company, battery, or troop, a noncommissioned officer next in rank above a corporal, whose duty is to instruct recruits in discipline, to form the ranks, etc.</def><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ In the United States service, besides the <xex>sergeants</xex> belonging to the companies there are, in each regiment, a <xex>sergeant major</xex>, who is the chief noncommissioned officer, and has important duties as the assistant to the adjutant; a <xex>quartermaster sergeant</xex>, who assists the quartermaster; a <xex>color sergeant</xex>, who carries the colors; and a <xex>commissary sergeant</xex>, who assists in the care and distribution of the stores. <xex>Ordnance sergeants</xex> have charge of the ammunition at military posts.</note><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <fld>(Law)</fld> <def>A lawyer of the highest rank, answering to the <xex>doctor</xex> of the civil law; -- called also <altname>serjeant at law</altname>.</def> <mark>[Eng.]</mark>  <rj><au>Blackstone.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>A title sometimes given to the servants of the sovereign; <as>as, <ex>sergeant</ex> surgeon, that is, a servant, or attendant, surgeon</as>.</def> <mark>[Eng.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <fld>(Zool.)</fld> <def>The cobia.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Drill sergeant</b></col>. <fld>(Mil.)</fld> <cd>See under <er>Drill</er>.</cd> -- <col><b>Sergeant-at-arms</b></col>, <cd>an officer of a legislative body, or of a deliberative or judicial assembly, who executes commands in preserving order and arresting offenders. See <er>Sergeant</er>, 1.</cd> -- <col><b>Sergeant major</b></col>. <sd>(a)</sd> <fld>(Mil.)</fld> <cd>See the Note under def. 2, above.</cd> <sd>(b)</sd> <fld>(Zool.)</fld> <cd>The cow pilot.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sergeantcy</ent><br/
<hw>Ser"geant*cy</hw> <pr>(?)</pr>, <pos>n.</pos> <def>Same as <er>Sergeancy</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sergeantry</ent><br/
<hw>Ser"geant*ry</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[CF. OF. <ets>sergenteric</ets>.]</ety> <def>See <er>Sergeanty</er>.</def> <mark>[R.]</mark> <altsp>[Written also <asp>serjeantry</asp>.]</altsp><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sergeantship</ent><br/
<hw>Ser"geant*ship</hw>, <pos>n.</pos> <def>The office of sergeant.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sergeanty</ent><br/
<hw>Ser"geant*y</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Cf. OF. <ets>sergentie</ets>, LL. <ets>sergentia</ets>. See <er>Sergeant</er>.]</ety> <fld>(Eng. Law)</fld> <def>Tenure of lands of the crown by an honorary kind of service not due to any lord, but to the king only.</def> <altsp>[Written also <asp>serjeanty</asp>.]</altsp><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Grand sergeanty</b></col>, <cd>a particular kind of tenure by which the tenant was bound to do some special honorary service to the king in person, as to carry his banner, his sword, or the like.</cd> <au>Tomlins.</au> <au>Cowell.</au> <au>Blackstone.</au> -- <col><b>Petit sergeanty</b></col>. <cd>See under <er>Petit</er>.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Serial</ent><br/
<hw>Se"ri*al</hw> <pr>(?)</pr>, <pos>a.</pos> <sn>1.</sn> <def>Of or pertaining to a series; consisting of a series; appearing in successive parts or numbers; <as>as, a <ex>serial</ex> work or publication</as>.</def> <ldquo/Classification . . . may be more or less <xex>serial</xex>.<rdquo/  <rj><au>H. Spencer.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Bot.)</fld> <def>Of or pertaining to rows.</def>  <rj><au>Gray.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Serial homology</b></col>. <fld>(Biol.)</fld> <cd>See under <er>Homology</er>.</cd> -- <col><b>Serial symmetry</b></col>. <fld>(Biol.)</fld> <cd>See under <er>Symmetry</er>.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Serial</ent><br/
<hw>Se"ri*al</hw>, <pos>n.</pos> <def>A publication appearing in a series or succession of part; a tale, or other writing, published in successive numbers of a periodical.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Seriality</ent><br/
<hw>Se`ri*al"i*ty</hw> <pr>(?)</pr>, <pos>n.</pos> <def>The quality or state of succession in a series; sequence.</def>  <rj><au>H. Spenser.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Serially</ent><br/
<hw>Se"ri*al*ly</hw>, <pos>adv.</pos> <def>In a series, or regular order; in a serial manner; <as>as, arranged <ex>serially</ex>; published <ex>serially</ex></as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Seriate</ent><br/
<hw>Se"ri*ate</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Arranged in a series or succession; pertaining to a series.</def> -- <wordforms><wf>Se"ri*ate*ly</wf>, <pos>adv.</pos></wordforms><br/
[<source>1913 Webster</source>]</p>

<p><ent>Seriatim</ent><br/
<hw>Se`ri*a"tim</hw> <pr>(?)</pr>, <pos>adv.</pos> <ety>[NL.]</ety> <def>In regular order; one after the other; severally.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Seriation</ent><br/
<hw>Se`ri*a"tion</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Chem.)</fld> <def>Arrangement or position in a series.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sericeous</ent><br/
<hw>Se*ri"ceous</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>sericus</ets> silken, <ets>sericum</ets> Seric stuff, silk, fr. <ets>Sericus</ets> belonging to the <ets>Seres</ets>, Gr. <?/<?/<?/, a people of Eastern Asia, the modern Chinese, celebrated for their silken fabrics.  Cf. <er>Silk</er>, <er>Serge</er> a woolen stuff.]</ety> <sn>1.</sn> <def>Of or pertaining to silk; consisting of silk; silky.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Bot.)</fld> <def>Covered with very soft hairs pressed close to the surface; <as>as, a <ex>sericeous</ex> leaf</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <fld>(Zool.)</fld> <def>Having a silklike luster, usually due to fine, close hairs.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sericin</ent><br/
<hw>Ser"i*cin</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>sericus</ets> silken.]</ety> <fld>(Chem.)</fld> <def>A gelatinous nitrogenous material extracted from crude silk and other similar fiber by boiling water; -- called also <altname>silk gelatin</altname>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sericite</ent><br/
<hw>Ser"i*cite</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>sericus</ets> silken.]</ety> <fld>(Min.)</fld> <def>A kind of muscovite occuring in silky scales having a fibrous structure. It is characteristic of sericite schist.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sericterium</ent><br/
||<hw>Ser`ic*te"ri*um</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[See <er>Sericeous</er>.]</ety> <fld>(Zool.)</fld> <def>A silk gland, as in the silkworms.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sericulture</ent><br/
<hw>Ser"i*cul`ture</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[See <er>Sericeous</er>, and <er>Culture</er>.]</ety> <def>The raising of silkworms.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Serie</ent><br/
<hw>Se"rie</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Cf. F. <ets>s<eacute/rie</ets>.]</ety> <def>Series.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Seriema</ent><br/
||<hw>Ser`i*e"ma</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Native name.]</ety> <fld>(Zool.)</fld> <def>either of two large South American birds related to the cranes, the <altname>cariama</altname> of Southern Brazil (<spn>Cariama cristata</spn>, formerly <spn>Dicholophus cristata</spn>) or the <spn>Chunga burmeisteri</spn> of Argentina. They have an erectile crest and a short, broad bill.  They are often domesticated.</def><br/
[<source>1913 Webster</source> <source>+PJC</source>]</p>

<p><ent>Series</ent><br/
<hw>Se"ries</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>series</ets>, fr. <ets>serere</ets>, <ets>sertum</ets>, to join or bind together; cf. Gr. <?/<?/<?/ to fasten, Skr. <ets>sarit</ets> thread.  Cf. <er>Assert</er>, <er>Desert</er> a solitude, <er>Exert</er>, <er>Insert</er>, <er>Seraglio</er>.]</ety> <sn>1.</sn> <def>A number of things or events standing or succeeding in order, and connected by a like relation; sequence; order; course; a succession of things; <as>as, a continuous <ex>series</ex> of calamitous events</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>During some years his life a <qex>series</qex> of triumphs.</q> <rj><qau>Macaulay.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Biol.)</fld> <def>Any comprehensive group of animals or plants including several subordinate related groups.</def><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ Sometimes a series includes several classes; sometimes only orders or families; in other cases only species.</note><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <fld>(Bot.)</fld> <def>In Engler's system of plant classification, a group of families showing certain structural or morphological relationships. It corresponds to the <altname>cohort</altname> of some writers, and to the <altname>order</altname> of many modern systematists.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><sn>4.</sn> <fld>(Math.)</fld> <def>An indefinite number of terms succeeding one another, each of which is derived from one or more of the preceding by a fixed law, called the <xex>law</xex> of the series; <as>as, an arithmetical <ex>series</ex>; a geometrical <ex>series</ex></as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn>  <fld>(Elec.)</fld> <def>A mode of arranging the separate parts of a circuit by connecting them successively end to end to form a single path for the current; -- opposed to <contr>parallel</contr>. The parts so arranged are said to be <col><b>in series</b></col>.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><sn>6.</sn>  <fld>(Com.)</fld> <def>A parcel of rough diamonds of assorted qualities.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Series dynamo</ent><br/
<hw>Series dynamo</hw>. <fld>(Elec.)</fld> <sd>(a)</sd> <def>A series-wound dynamo.</def> <sd>(b)</sd> <def>A dynamo running in series with another or others.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Series motor</ent><br/
<hw>Series motor</hw>. <fld>(Elec.)</fld> <sd>(a)</sd> <def>A series-wound motor.</def> <sd>(b)</sd> <def>A motor capable of being used in a series circuit.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Series turns</ent><br/
<hw>Series turns</hw>. <fld>(Elec.)</fld> <def>The turns in a series circuit.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Series winding</ent><br/
<hw>Series winding</hw>. <fld>(Elec.)</fld> <def>A winding in which the armature coil and the field-magnet coil are in series with the external circuits; -- opposed to <contr>shunt winding</contr>.</def> -- <wordforms><wf>Se"ries-wound`</wf>, <pos>a.</pos></wordforms><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Serigraph</ent><br/
<hw>Ser`i*graph</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>sericum</ets> silk + E. <ets>-graph</ets>.]</ety> <def>An autographic device to test the strength of raw silk.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Serin</ent><br/
<hw>Ser"in</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F. <ets>serin</ets>.]</ety> <fld>(Zool.)</fld> <def>A European finch (<spn>Serinus hortulanus</spn>) closely related to the canary.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Serine</ent><br/
<hw>Ser"ine</hw> <pr>(s<ecr/r"<emac/n; s<ecr/r"<icr/n; also, less correctly, s<emac/r"<emac/n)</pr>, <pos>n.</pos> <ety>[L. <ets>sericus</ets> silken.]</ety> <fld>(Chem.)</fld> <def>one of the natural <isa>L-amino acids</isa>, obtainable as a white crystalline nitrogenous substance by the action of dilute sulphuric acid on silk gelatin.  It is found in many proteins, and, having a free primary hydroxyl group on the side chain, is involved in the catalytic action at the active site of some enzymes, such as proteases.  The IUPAC abbreviation for serine in protein sequences is <abbr>Ser</abbr>.  Chemically it is <chname>2-amino-3-hydroxy-propanoic acid</chname> (<chform>C3H7NO3</chform>), <chform>HO.CH2.CH(NH2).COOH</chform>.</def><br/
[<source>1913 Webster</source> <source>+PJC</source>]</p>

<p><ent>Serio-comical</ent><br/
<ent>Serio-comic</ent><br/
<mhw>{ <hw>Se`ri*o-com"ic</hw> <pr>(?)</pr>, <hw>Se`ri*o-com"ic*al</hw> <pr>(?)</pr>, }</mhw> <pos>a.</pos> <def>Having a mixture of seriousness and sport; serious and comical.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Serious</ent><br/
<hw>Se"ri*ous</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>serius</ets>: cf. F. <ets>s<eacute/rieux</ets>, LL. <ets>seriosus</ets>.]</ety> <sn>1.</sn> <def>Grave in manner or disposition; earnest; thoughtful; solemn; not light, gay, or volatile.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>He is always <qex>serious</qex>, yet there is about his manner a graceful ease.</q> <rj><qau>Macaulay.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Really intending what is said; being in earnest; not jesting or deceiving.</def>  <rj><au>Beaconsfield.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>Important; weighty; not trifling; grave.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The holy Scriptures bring to our ears the most <qex>serious</qex> things in the world.</q> <rj><qau>Young.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>Hence, giving rise to apprehension; attended with danger; <as>as, a <ex>serious</ex> injury</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><syn><b>Syn.</b> -- Grave; solemn; earnest; sedate; important; weighty. See <er>Grave</er>.</syn><br/
[<source>1913 Webster</source>]</p>

<p>-- <wordforms><wf>Se"ri*ous*ly</wf>, <pos>adv.</pos> -- <wf>Se"ri*ous*ness</wf>, <pos>n.</pos></wordforms><br/
[<source>1913 Webster</source>]</p>

<p><ent>Seriph</ent><br/
<hw>Ser"iph</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Type Founding)</fld> <def>See <er>Ceriph</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Serjeantcy</ent><br/
<ent>Serjeant</ent><br/
<mhw>{ <hw>Ser"jeant</hw> <pr>(?)</pr>, <hw>Ser"jeant*cy</hw>, etc. }</mhw> <def>See <er>Sergeant</er>, <er>Sergeantcy</er>, etc.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Serjeant-at-arms</b></col>. <cd>See <cref>Sergeant-at-arms</cref>, under <er>Sergeant</er>.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sermocination</ent><br/
<hw>Ser*moc`i*na"tion</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>sermocinatio</ets>. See <er>Sermon</er>.]</ety> <def>The making of speeches or sermons; sermonizing.</def> <mark>[Obs.]</mark>  <rj><au>Peacham.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sermocinator</ent><br/
<hw>Ser*moc"i*na`tor</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L.]</ety> <def>One who makes sermons or speeches.</def> <mark>[Obs.]</mark>  <rj><au>Howell.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sermon</ent><br/
<hw>Ser"mon</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[OE. <ets>sermoun</ets>, <ets>sermun</ets>, F. <ets>sermon</ets>, fr. L. <ets>sermo</ets>, <ets>-onis</ets>, a speaking, discourse, probably fr. <ets>serer</ets>, <ets>sertum</ets>, to join, connect; hence, a connected speech. See <er>Series</er>.]</ety> <sn>1.</sn> <def>A discourse or address; a talk; a writing; <as>as, the <ex>sermons</ex> of Chaucer</as>.</def> <mark>[Obs.]</mark>  <rj><au>Chaucer.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Specifically, a discourse delivered in public, usually by a clergyman, for the purpose of religious instruction and grounded on some text or passage of Scripture.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>This our life exempt from public haunts<br/
Finds tongues in trees, books in the running brooks,<br/
<qex>Sermons</qex> in stones and good in everything.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>His preaching much, but more his practice, wrought,<br/
A living <qex>sermon</qex> of the truths he taught.</q> <rj><qau>Dryden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>Hence, a serious address; a lecture on one's conduct or duty; an exhortation or reproof; a homily; -- often in a depreciatory sense.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sermon</ent><br/
<hw>Ser"mon</hw>, <pos>v. i.</pos> <ety>[Cf. OF. <ets>sermoner</ets>, F. <ets>sermonner</ets> to lecture one.]</ety> <def>To speak; to discourse; to compose or deliver a sermon.</def> <mark>[Obs.]</mark>  <rj><au>Holinshed.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>What needeth it to <qex>sermon</qex> of it more?</q> <rj><qau>Chaucer.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sermon</ent><br/
<hw>Ser"mon</hw>, <pos>v. t.</pos> <sn>1.</sn> <def>To discourse to or of, as in a sermon.</def> <mark>[Obs.]</mark>  <rj><au>Spenser.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To tutor; to lecture.</def> <mark>[Poetic]</mark>  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sermoneer</ent><br/
<hw>Ser`mon*eer"</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A sermonizer.</def>  <rj><au>B. Jonson.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sermoner</ent><br/
<hw>Ser"mon*er</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A preacher; a sermonizer.</def> <mark>[Derogative or Jocose.]</mark>  <rj><au>Thackeray.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sermonet</ent><br/
<hw>Ser`mon*et"</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A short sermon.</def> <altsp>[Written also <asp>sermonette</asp>.]</altsp><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sermonical</ent><br/
<ent>Sermonic</ent><br/
<mhw>{ <hw>Ser*mon"ic</hw> <pr>(?)</pr>, <hw>Ser*mon"ic*al</hw> <pr>(?)</pr>, }</mhw> <pos>a.</pos> <def>Like, or appropriate to, a sermon; grave and didactic.</def> <mark>[R.]</mark> <ldquo/Conversation . . . satirical or <xex>sermonic</xex>.<rdquo/ <au>Prof. Wilson.</au> <ldquo/<xex>Sermonical</xex> style.<rdquo/ <au>V. Knox.</au><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sermoning</ent><br/
<hw>Ser"mon*ing</hw> <pr>(?)</pr>, <pos>n.</pos> <def>The act of discoursing; discourse; instruction; preaching.</def> <mark>[Obs.]</mark>  <rj><au>Chaucer.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sermonish</ent><br/
<hw>Ser"mon*ish</hw>, <pos>a.</pos> <def>Resembling a sermon.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sermonist</ent><br/
<hw>Ser"mon*ist</hw>, <pos>n.</pos> <def>See <er>Sermonizer</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sermonize</ent><br/
<hw>Ser"mon*ize</hw> <pr>(?)</pr>, <pos>v. i.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Sermonized</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Sermonizing</conjf> <pr>(?)</pr>.]</vmorph> <sn>1.</sn> <def>To compose or write a sermon or sermons; to preach.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To inculcate rigid rules.</def> <mark>[R.]</mark>  <rj><au>Chesterfield.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sermonize</ent><br/
<hw>Ser"mon*ize</hw>, <pos>v. t.</pos> <def>To preach or discourse to; to affect or influence by means of a sermon or of sermons.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>Which of us shall sing or <qex>sermonize</qex> the other fast asleep?</q> <rj><qau>Landor.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sermonizer</ent><br/
<hw>Ser"mon*i`zer</hw> <pr>(?)</pr>, <pos>n.</pos> <def>One who sermonizes.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Serolin</ent><br/
<hw>Ser"o*lin</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[<ets>Ser</ets>um + L. <ets>ole</ets>um oil.]</ety> <fld>(Physiol. Chem.)</fld> <sd>(a)</sd> <def>A peculiar fatty substance found in the blood, probably a mixture of fats, cholesterin, etc.</def> <sd>(b)</sd> <def>A body found in fecal matter and thought to be formed in the intestines from the cholesterin of the bile; -- called also <altname>stercorin</altname>, and <altname>stercolin</altname>.</def><br/
[<source>1913 Webster</source>]</p>

<p><-- p. 1315 --></p>

<p><ent>Seroon</ent><br/
<ent>Seron</ent><br/
<mhw>{ <hw>Se*ron"</hw> <pr>(?)</pr>, <hw>Se*roon"</hw> <pr>(?)</pr>, }</mhw> <pos>n.</pos> <ety>[Sp. <ets>seron</ets> a kind of hamper or pannier, aug. of <ets>sera</ets> a large pannier or basket.]</ety> <def>Same as <er>Ceroon</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ This word as expressing a quantity or weight has no definite signification.</note>  <rj><au>McElrath.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Serose</ent><br/
<hw>Se"rose`</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Serous.</def> <mark>[Obs.]</mark>  <rj><au>Dr. H. More.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Serosity</ent><br/
<hw>Se*ros"i*ty</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Cf. F.  <ets>serosit<eacute/</ets>. See <er>Serous</er>.]</ety> <sn>1.</sn> <def>The quality or state of being serous.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Physiol.)</fld> <def>A thin watery animal fluid, as synovial fluid and pericardial fluid.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Serotherapy</ent><br/
<hw>Se`ro*ther"a*py</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Med.)</fld> <sd>(a)</sd> <def>Serum-therapy.</def> <sd>(b)</sd> <def>The whey cure.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Serotine</ent><br/
<hw>Ser"o*tine</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F. <ets>s<eacute/rotine</ets>, fr. L. <ets>serotinus</ets> that comes or happens late.]</ety> <fld>(Zool.)</fld> <def>The European long-eared bat (<spn>Vesperugo serotinus</spn>).</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Serotinous</ent><br/
<hw>Se*rot"i*nous</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>serotinus</ets>, fr. <ets>serus</ets> late.]</ety> <fld>(Biol.)</fld> <def>Appearing or blossoming later in the season than is customary with allied species.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Serous</ent><br/
<hw>Se"rous</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[Cf. F. <ets>s<eacute/reux</ets>. See <er>Serum</er>.]</ety> <fld>(Physiol.)</fld> <sd>(a)</sd> <def>Thin; watery; like serum; <as>as, the <ex>serous</ex> fluids</as>.</def> <sd>(b)</sd> <def>Of or pertaining to serum; <as>as, the <ex>serous</ex> glands, membranes, layers</as>. See <er>Serum</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Serous membrane</b></col>. <fld>(Anat.)</fld> <cd>See under <er>Membrane</er>.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Surrow</ent><br/
<ent>Serow</ent><br/
<mhw>{ <hw>Ser"ow</hw> <pr>(?)</pr>, <hw>Sur"row</hw> <pr>(?)</pr>, }</mhw> <pos>n.</pos> <fld>(Zool.)</fld> <def>The thar.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Serpens</ent><br/
||<hw>Ser"pens</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. See <er>Serpent</er>.]</ety> <fld>(Astron.)</fld> <def>A constellation represented as a serpent held by Serpentarius.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Serpent</ent><br/
<hw>Ser"pent</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F., fr. L. <ets>serpens</ets>, <ets>-entis</ets> (sc. <ets>bestia</ets>), fr. <ets>serpens</ets>, p. pr. of <ets>serpere</ets> to creep; akin to Gr. <?/<?/<?/, Skr. <ets>sarp</ets>, and perhaps to L. <ets>repere</ets>, E. <ets>reptile</ets>.  Cf. <er>Herpes</er>.]</ety> <sn>1.</sn> <fld>(Zool.)</fld> <def>Any reptile of the order Ophidia; a snake, especially a large snake. See <xex>Illust.</xex> under <er>Ophidia</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ The serpents are mostly long and slender, and move partly by bending the body into undulations or folds and pressing them against objects, and partly by using the free edges of their ventral scales to cling to rough surfaces. Many species glide swiftly over the ground, some burrow in the earth, others live in trees. A few are entirely aquatic, and swim rapidly. See <er>Ophidia</er>, and <er>Fang</er>.</note><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Fig.: A subtle, treacherous, malicious person.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>A species of firework having a serpentine motion as it passess through the air or along the ground.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <fld>(Astron.)</fld> <def>The constellation Serpens.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <fld>(Mus.)</fld> <def>A bass wind instrument, of a loud and coarse tone, formerly much used in military bands, and sometimes introduced into the orchestra; -- so called from its form.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Pharaoh's serpent</b></col> <fld>(Chem.)</fld>, <cd>mercuric sulphocyanate, a combustible white substance which in burning gives off a poisonous vapor and leaves a peculiar brown voluminous residue which is expelled in a serpentine from. It is employed as a scientific toy.</cd> -- <col><b>Serpent cucumber</b></col> <fld>(Bot.)</fld>, <cd>the long, slender, serpentine fruit of the cucurbitaceous plant <spn>Trichosanthes colubrina</spn>; also, the plant itself.</cd> -- <col><b>Serpent eage</b></col> <fld>(Zool.)</fld>, <cd>any one of several species of raptorial birds of the genera <gen>Circaetus</gen> and <gen>Spilornis</gen>, which prey on serpents. They inhabit Africa, Southern Europe, and India.  The European serpent eagle is <spn>Circaetus Gallicus</spn>.</cd> -- <col><b>Serpent eater</b></col>. <fld>(Zool.)</fld> <sd>(a)</sd> <cd>The secretary bird</cd>. <sd>(b)</sd> <cd>An Asiatic antelope; the markhoor.</cd> -- <col><b>Serpent fish</b></col> <fld>(Zool.)</fld>, <cd>a fish (<spn>Cepola rubescens</spn>) with a long, thin, compressed body, and a band of red running lengthwise.</cd> -- <col><b>Serpent star</b></col> <fld>(Zool.)</fld>, <cd>an ophiuran; a brittle star.</cd> -- <col><b>Serpent's tongue</b></col> <fld>(Paleon.)</fld>, <cd>the fossil tooth of a shark; -- so called from its resemblance to a tongue with its root.</cd> -- <col><b>Serpent withe</b></col> <fld>(Bot.)</fld>, <cd>a West Indian climbing plant (<spn>Aristolochia odoratissima</spn>).</cd> -- <col><b>Tree serpent</b></col> <fld>(Zool.)</fld>, <cd>any species of African serpents belonging to the family <fam>Dendrophidae</fam>.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Serpent</ent><br/
<hw>Ser"pent</hw>, <pos>v. i.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Serpented</conjf>; <pos>p. pr. & vb. n.</pos> <conjf>Serpenting</conjf>.]</vmorph> <def>To wind like a serpent; to crook about; to meander.</def> <mark>[R.]</mark> <ldquo/The <xex>serpenting</xex> of the Thames.<rdquo/  <rj><au>Evelyn.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Serpent</ent><br/
<hw>Ser"pent</hw>, <pos>v. t.</pos> <def>To wind; to encircle.</def> <mark>[R.]</mark>  <rj><au>Evelyn.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Serpentaria</ent><br/
||<hw>Ser`pen*ta"ri*a</hw> <pr>(?)</pr>, <pos>a.</pos><ety>[L. (sc. <ets>herba</ets>), fr. <ets>serpens</ets> serpent.]</ety> <fld>(Med.)</fld> <def>The fibrous aromatic root of the Virginia snakeroot (<spn>Aristolochia Serpentaria</spn>).</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Serpentarius</ent><br/
||<hw>Ser`pen*ta"ri*us</hw> <pr>(?)</pr>, <pos>n.</pos><ety>[NL., fr. L. <ets>serpens</ets> serpent.]</ety> <fld>(Astron.)</fld> <def>A constellation on the equator, lying between Scorpio and Hercules; -- called also <altname>Ophiuchus</altname>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Serpentiform</ent><br/
<hw>Ser*pen"ti*form</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>serpens</ets> a serpent + <ets>-form</ets>.]</ety> <def>Having the form of a serpent.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Serpentigenous</ent><br/
<hw>Ser`pen*tig"e*nous</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>serpens</ets>, <ets>-entis</ets>, a serpent + <ets>-genous</ets>: cf. L. <ets>serpentigena</ets>.]</ety> <def>Bred of a serpent.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Serpentine</ent><br/
<hw>Ser"pen*tine</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>serpentinus</ets>: cf. F. <ets>serpentin</ets>.]</ety> <def>Resembling a serpent; having the shape or qualities of a serpent; subtle; winding or turning one way and the other, like a moving serpent; anfractuous; meandering; sinuous; zigzag; <as>as, <ex>serpentine</ex> braid</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Thy shape<br/
Like his, and color <qex>serpentine</qex>.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Serpentine</ent><br/
<hw>Ser"pen*tine</hw>, <pos>n.</pos> <ety>[Cf. (for sense 1) F. <ets>serpentine</ets>, (for sense 2) <ets>serpentin</ets>.]</ety> <sn>1.</sn> <fld>(Min.)</fld> <def>A mineral or rock consisting chiefly of the hydrous silicate of magnesia. It is usually of an obscure green color, often with a spotted or mottled appearance resembling a serpent's skin. <xex>Precious</xex>, or <xex>noble</xex>, <xex>serpentine</xex> is translucent and of a rich oil-green color.</def><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ Serpentine has been largely produced by the alteration of other minerals, especially of chrysolite.</note><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Ordnance)</fld> <def>A kind of ancient cannon.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Serpentine</ent><br/
<hw>Ser"pen*tine</hw>, <pos>v. i.</pos> <def>To serpentize.</def> <mark>[R.]</mark>  <rj><au>Lyttleton.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Serpentinely</ent><br/
<hw>Ser"pen*tine*ly</hw>, <pos>adv.</pos> <def>In a serpentine manner.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Serpentinian</ent><br/
<hw>Ser`pen*tin"i*an</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Eccl.)</fld> <def>See 2d <er>Ophite</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Serpentinize</ent><br/
<hw>Ser"pen*tin*ize</hw> <pr>(?)</pr>, <pos>v. t.</pos> <fld>(Min.)</fld> <def>To convert (a magnesian silicate) into serpentine.</def> -- <wordforms><wf>Ser`pen*tin`i*za"tion</wf> <pr>(#)</pr>, <pos>n.</pos></wordforms><br/
[<source>1913 Webster</source>]</p>

<p><ent>Serpentinous</ent><br/
<hw>Ser"pen*ti`nous</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Relating to, or like, serpentine; <as>as, a rock <ex>serpentinous</ex> in character</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Serpentize</ent><br/
<hw>Ser"pent*ize</hw> <pr>(?)</pr>, <pos>v. i.</pos> <def>To turn or bend like a serpent, first in one direction and then in the opposite; to meander; to wind; to serpentine.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>The river runs before the door, and <qex>serpentizes</qex> more than you can conceive.</q> <rj><qau>Walpole.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Serpentry</ent><br/
<hw>Ser"pent*ry</hw> <pr>(?)</pr>, <pos>n.</pos> <sn>1.</sn> <def>A winding like a serpent's.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A place inhabited or infested by serpents.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Serpent-tongued</ent><br/
<hw>Ser"pent-tongued`</hw> <pr>(?)</pr>, <pos>a.</pos> <fld>(Zool.)</fld> <def>Having a forked tongue, like a serpent.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Serpet</ent><br/
<hw>Ser"pet</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>sirpus</ets>, <ets>scirpus</ets>, a rush, bulrush.]</ety> <def>A basket.</def> <mark>[Obs.]</mark>  <rj><au>Ainsworth.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Serpette</ent><br/
||<hw>Ser*pette"</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F.]</ety> <def>A pruning knife with a curved blade.</def>  <rj><au>Knight.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Serpiginous</ent><br/
<hw>Ser*pig"i*nous</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[Cf. F. <ets>serpigineux</ets>.]</ety> <fld>(Med.)</fld> <def>Creeping; -- said of lesions which heal over one portion while continuing to advance at another.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Serpigo</ent><br/
||<hw>Ser*pi"go</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[LL., fr. L. <ets>serpere</ets> to creep.]</ety> <fld>(Med.)</fld> <def>A dry, scaly eruption on the skin; especially, a ringworm.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Serpolet</ent><br/
<hw>Ser"po*let</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F.]</ety> <fld>(Bot.)</fld> <def>Wild thyme.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Serpula</ent><br/
||<hw>Ser"pu*la</hw> <pr>(?)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Serpulae</plw> <pr>(#)</pr>, E. <plw>Serpulas</plw> <pr>(#)</pr>.</plu> <ety>[L., a little snake. See <er>Serpent</er>.]</ety> <fld>(Zool.)</fld> <def>Any one of numerous species of tubicolous annelids of the genus <gen>Serpula</gen> and allied genera of the family <fam>Serpulidae</fam>. They secrete a calcareous tube, which is usually irregularly contorted, but is sometimes spirally coiled. The worm has a wreath of plumelike and often bright-colored gills around its head, and usually an operculum to close the aperture of its tube when it retracts.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Serpulidan</ent><br/
<ent>Serpulian</ent><br/
<mhw>{ <hw>Ser*pu"li*an</hw> <pr>(?)</pr>, <hw>Ser*pu"li*dan</hw> <pr>(?)</pr>, }</mhw> <pos>n.</pos> <fld>(Zool.)</fld> <def>A serpula.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Serpulite</ent><br/
<hw>Ser"pu*lite</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A fossil serpula shell.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Serr</ent><br/
<hw>Serr</hw> <pr>(?)</pr>, <pos>v. t.</pos> <ety>[F. <ets>serrer</ets>. See <er>Serry</er>.]</ety> <def>To crowd, press, or drive together.</def> <mark>[Obs.]</mark>  <rj><au>Bacon.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Serranoid</ent><br/
<hw>Ser*ra"noid</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[NL. <ets>Serranus</ets>, a typical genus (fr. L. <ets>serra</ets> a saw) + <ets>-oid</ets>.]</ety> <fld>(Zool.)</fld> <def>Any fish of the family <fam>Serranidae</fam>, which includes the striped bass, the black sea bass, and many other food fishes.</def> -- <def2><pos>a.</pos> <fld>(Zool.)</fld> <def>Of or pertaining to the <fam>Serranidae</fam>.</def></def2><br/
[<source>1913 Webster</source>]</p>

<p><ent>Serrated</ent><br/
<ent>Serrate</ent><br/
<mhw>{ <hw>Ser"rate</hw> <pr>(?)</pr>, <hw>Ser"ra*ted</hw> <pr>(?)</pr>, }</mhw> <pos>a.</pos> <ety>[L. <ets>serratus</ets>, fr. <ets>serra</ets> a saw; perhaps akin to <ets>secare</ets> to cut, E. <ets>saw</ets> a cutting instrument.  Cf. <er>Sierra</er>.]</ety> <sn>1.</sn> <def>Notched on the edge, like a saw.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Bot.)</fld> <def>Beset with teeth pointing forwards or upwards; <as>as, <ex>serrate</ex> leaves</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Doubly serrate</b></col>, <cd>having small serratures upon the large ones, as the leaves of the elm.</cd> -- <col><b>Serrate-ciliate</b></col>, <cd>having fine hairs, like the eyelashes, on the serratures; -- said of a leaf.</cd> -- <col><b>Serrate-dentate</b></col>, <cd>having the serratures toothed.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Serration</ent><br/
<hw>Ser*ra"tion</hw> <pr>(?)</pr>, <pos>n.</pos> <sn>1.</sn> <def>Condition of being serrate; formation in the shape of a saw.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>One of the teeth in a serrate or serrulate margin.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Serratirostral</ent><br/
<hw>Ser*ra`ti*ros"tral</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[<ets>Serrate</ets> + <ets>rostral</ets>.]</ety> <fld>(Zool.)</fld> <def>Having a toothed bill, like that of a toucan.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Serrator</ent><br/
<hw>Ser*ra"tor</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[NL.]</ety> <fld>(Zool.)</fld> <def>The ivory gull (<spn>Larus eburneus</spn>).</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Serrature</ent><br/
<hw>Ser"ra*ture</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>serratura</ets> a sawing, fr. <ets>serrare</ets> to saw.]</ety> <sn>1.</sn> <def>A notching, like that between the teeth of a saw, in the edge of anything.</def>  <rj><au>Martyn.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>One of the teeth in a serrated edge; a serration.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Serricated</ent><br/
<hw>Ser"ri*ca`ted</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[See <er>Sericeous</er>.]</ety> <def>Covered with fine silky down.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Serricorn</ent><br/
<hw>Ser"ri*corn</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>serra</ets> saw + <ets>cornu</ets> horn.]</ety> <fld>(Zool.)</fld> <def>Having serrated antenn<?/.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Serricorn</ent><br/
<hw>Ser"ri*corn</hw>, <pos>n.</pos> <fld>(Zool.)</fld> <def>Any one of a numerous tribe of beetles (<fam>Serricornia</fam>). The joints of the antennae are prominent, thus producing a serrate appearance. See <xex>Illust.</xex> under <er>Antenna</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Serried</ent><br/
<hw>Ser"ried</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[See <er>Serry</er>.]</ety> <def>Crowded; compact; dense; pressed together.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Nor seemed it to relax their <qex>serried</qex> files.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Serrifera</ent><br/
||<hw>Ser*rif"e*ra</hw> <pr>(?)</pr>, <pos>n. pl.</pos> <ety>[NL., fr. L. <ets>serra</ets> saw + <ets>ferre</ets> to bear.]</ety> <fld>(Zool.)</fld> <def>A division of Hymenoptera comprising the sawflies.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Serrirostres</ent><br/
||<hw>Ser`ri*ros"tres</hw> <pr>(?)</pr>, <pos>n. pl.</pos> <ety>[NL. fr. L. <ets>serra</ets> saw + <ets>rostrum</ets> beak.]</ety> <fld>(Zool.)</fld> <def>Same as <er>Lamellirostres</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Serrous</ent><br/
<hw>Ser"rous</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>serra</ets> a saw.]</ety> <def>Like the teeth off a saw; jagged.</def> <mark>[Obs.]</mark>  <rj><au>Sir T. Browne.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Serrula</ent><br/
<hw>Ser"ru*la</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L., a little saw.]</ety> <fld>(Zool.)</fld> <def>The red-breasted merganser.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Serrulated</ent><br/
<ent>Serrulate</ent><br/
<mhw>{ <hw>Ser"rulate</hw> <pr>(?)</pr>, <hw>Ser"ru*la`ted</hw> <pr>(?)</pr>, }</mhw> <pos>a.</pos> <ety>[L. <ets>serrula</ets> a little saw, dim. of <ets>serra</ets> a saw.]</ety> <def>Finely serrate; having very minute teeth.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Serrulation</ent><br/
<hw>Ser`ru*la"tion</hw> <pr>(?)</pr>, <pos>n.</pos> <sn>1.</sn> <def>The state of being notched minutely, like a fine saw.</def>  <rj><au>Wright.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>One of the teeth in a serrulate margin.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Serry</ent><br/
<hw>Ser"ry</hw> <pr>(?)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Serried</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Serrying</conjf>.]</vmorph> <ety>[F. <ets>serrer</ets>, LL. <ets>serrare</ets>, <ets>serare</ets>, from L. <ets>sera</ets> a bar, bolt; akin to <ets>serere</ets> to join or bind together. See <er>Serries</er>.]</ety> <def>To crowd; to press together.</def> <note>[Now perhaps only in the form <xex>serried</xex>, p. p. or a.]</note><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sertularia</ent><br/
||<hw>Ser`tu*la"ri*a</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[NL., dim. fr. L. <ets>serta</ets> a garland.]</ety> <fld>(Zool.)</fld> <def>A genus of delicate branching hydroids having small sessile hydrothecae along the sides of the branches.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sertularian</ent><br/
<hw>Ser`tu*la"ri*an</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Zool.)</fld> <def>Any species of Sertularia, or of <fam>Sertularidae</fam>, a family of hydroids having branched chitinous stems and simple sessile hydrothecae. Also used adjectively.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Serum</ent><br/
<hw>Se"rum</hw> <pr>(s<emac/"r<ucr/m)</pr>, <pos>n.</pos> <ety>[L., akin to Gr. <?/<?/<?/, Skr. <ets>s<amac/ra</ets> curd.]</ety> <fld>(Physiol.)</fld> <sd>(a)</sd> <def>The watery portion of certain animal fluids, as blood, milk, etc.</def> <sd>(b)</sd> <def>A thin watery fluid, containing more or less albumin, secreted by the serous membranes of the body, such as the pericardium and peritoneum.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Blood serum</b></col>, <cd>the pale yellowish fluid which exudes from the clot formed in the coagulation of the blood; the liquid portion of the blood, after removal of the blood corpuscles and the fibrin.</cd> -- <col><b>Muscle serum</b></col>, <cd>the thin watery fluid which separates from the muscles after coagulation of the muscle plasma; the watery portion of the plasma. See <cref>Muscle plasma</cref>, under <er>Plasma</er>.</cd> -- <col><b>Serum albumin</b></col> <fld>(Physiol. Chem.)</fld>, <cd>an albuminous body, closely related to egg albumin, present in nearly all serous fluids; esp., the albumin of blood serum.</cd> -- <col><b>Serum globulin</b></col> <fld>(Physiol. Chem.)</fld>, <cd>paraglobulin.</cd> -- <col><b>Serum of milk</b></col> <fld>(Physiol. Chem.)</fld>, <cd>the whey, or fluid portion of milk, remaining after removal of the casein and fat.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Serum-therapy</ent><br/
<hw>Se"rum-ther`a*py</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Med.)</fld> <def>The treatment of disease by the injection of blood serum from immune animals.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Servable</ent><br/
<hw>Serv"a*ble</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[See <er>Serve</er>.]</ety> <sn>1.</sn> <def>Capable of being served.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <ety>[L. <ets>servabilis</ets>.]</ety> <def>Capable of being preserved.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Servage</ent><br/
<hw>Serv"age</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Cf. F. <ets>servage</ets>.]</ety> <def>Serfage; slavery; servitude.</def> <mark>[Obs.]</mark>  <rj><au>Chaucer.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Serval</ent><br/
<hw>Ser"val</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Cf. F. <ets>serval</ets>.]</ety> <fld>(Zool.)</fld> <def>An African wild cat (<spn>Felis serval</spn>) of moderate size; -- called also <altname>serval cat</altname>. It has rather long legs and a tail of moderate length. Its color is tawny, with black spots on the body and rings of black on the tail.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Servaline</ent><br/
<hw>Ser"val*ine</hw> <pr>(?)</pr>, <pos>a.</pos> <fld>(Zool.)</fld> <def>Related to, or resembling, the serval.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Servant</ent><br/
<hw>Serv"ant</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[OE. <ets>servant</ets>, <ets>servaunt</ets>, F. <ets>servant</ets>, a & p. pr. of <ets>servir</ets> to serve, L. <ets>servire</ets>. See <er>Serve</er>, and cf. <er>Sergeant</er>.]</ety> <sn>1.</sn> <def>One who serves, or does services, voluntarily or on compulsion; a person who is employed by another for menial offices, or for other labor, and is subject to his command; a person who labors or exerts himself for the benefit of another, his master or employer; a subordinate helper.</def> <ldquo/A yearly hired <xex>servant</xex>.<rdquo/  <rj><au>Lev. xxv. 53.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Men in office have begun to think themselves mere agents and <qex>servants</qex> of the appointing power, and not agents of the government or the country.</q> <rj><qau>D. Webster.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ In a legal sense, stewards, factors, bailiffs, and other agents, are <xex>servants</xex> for the time they are employed in such character, as they act in subordination to others. So any person may be legally the servant of another, in whose business, and under whose order, direction, and control, he is acting for the time being.</note>  <rj><au>Chitty.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>One in a state of subjection or bondage.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Thou wast a <qex>servant</qex> in the land of Egypt.</q> <rj><qau>Deut. v. 15.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>A professed lover or suitor; a gallant.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>In my time a <qex>servant</qex> was I one.</q> <rj><qau>Chaucer.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Servant of servants</b></col>, <cd>one debased to the lowest condition of servitude.</cd> -- <mcol><col><b>Your humble servant</b></col>, <it>or</it> <col><b>Your obedient servant</b></col></mcol>, <cd>phrases of civility formerly often used in closing a letter, now archaic; -- at one time such phrases were exaggerated to include <altname>Your most humble, most obedient servant</altname>.</cd></cs><br/
[<source>1913 Webster</source> <source>+PJC</source>]</p>

<p><q>Our betters tell us they are <qex>our humble servants</qex>, but understand us to be their slaves.</q> <rj><qau>Swift.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Servant</ent><br/
<hw>Serv"ant</hw>, <pos>v. t.</pos> <def>To subject.</def> <mark>[Obs.]</mark>  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Servantess</ent><br/
<hw>Serv"ant*ess</hw>, <pos>n.</pos> <def>A maidservant.</def> <mark>[Obs.]</mark>  <rj><au>Wyclif.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Servantry</ent><br/
<hw>Serv"ant*ry</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A body of servants; servants, collectively.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Serve</ent><br/
<hw>Serve</hw> <pr>(?)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Served</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Serving</conjf>.]</vmorph> <ety>[OE. <ets>serven</ets>, <ets>servien</ets>, OF. & F. <ets>servir</ets>, fr. L. <ets>servire</ets>; akin to <ets>servus</ets> a servant or slave, <ets>servare</ets> to protect, preserve, observe; cf. Zend <ets>har</ets> to protect, <ets>haurva</ets> protecting.  Cf. <er>Conserve</er>, <er>Desert</er> merit, <er>Dessert</er>, <er>Observe</er>, <er>Serf</er>, <er>Sergeant</er>.]</ety> <sn>1.</sn> <def>To work for; to labor in behalf of; to exert one's self continuously or statedly for the benefit of; to do service for; to be in the employment of, as an inferior, domestic, serf, slave, hired assistant, official helper, etc.; specifically, in a religious sense, to obey and worship.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>God is my witness, whom I <qex>serve</qex> with my spirit.</q> <rj><qau>Rom. i. 9.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Jacob loved Rachel; and said, I will <qex>serve</qex> thee seven years for Rachel thy younger daughter.</q> <rj><qau>Gen. xxix. 18.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>No man can <qex>serve</qex> two masters.</q> <rj><qau>Matt. vi. 24.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Had I but <qex>served</qex> my God with half the zeal<br/
I <qex>served</qex> my king, he would not in mine age<br/
Have left me naked to mine enemies.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To be subordinate to; to act a secondary part under; to appear as the inferior of; to minister to.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Bodies bright and greater should not <qex>serve</qex><br/
The less not bright.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>To be suitor to; to profess love to.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>To <qex>serve</qex> a lady in his beste wise.</q> <rj><qau>Chaucer.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>To wait upon; to supply the wants of; to attend; specifically, to wait upon at table; to attend at meals; to supply with food; <as>as, to <ex>serve</ex> customers in a shop</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Others, pampered in their shameless pride,<br/
Are <qex>served</qex> in plate and in their chariots ride.</q> <rj><qau>Dryden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <def>Hence, to bring forward, arrange, deal, or distribute, as a portion of anything, especially of food prepared for eating; -- often with <xex>up</xex>; formerly with <xex>in</xex>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Bid them cover the table, <qex>serve</qex> in the meat, and we will come in to dinner.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Some part he roasts, then <qex>serves</qex> it up so dressed.</q> <rj><qau>Dryde.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><-- p. 1316 --></p>

<p><sn>6.</sn> <def>To perform the duties belonging to, or required in or for; hence, to be of use to; <as>as, a curate may <ex>serve</ex> two churches; to <ex>serve</ex> one's country</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>7.</sn> <def>To contribute or conduce to; to promote; to be sufficient for; to satisfy; <as>as, to <ex>serve</ex> one's turn</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Turn it into some advantage, by observing where it can <qex>serve</qex> another end.</q> <rj><qau>Jer. Taylor.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>8.</sn> <def>To answer or be (in the place of something) to; <as>as, a sofa <ex>serves</ex> one for a seat and a couch</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>9.</sn> <def>To treat; to behave one's self to; to requite; to act toward; <as>as, he <ex>served</ex> me very ill</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>10.</sn> <def>To work; to operate; <as>as, to <ex>serve</ex> the guns</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>11.</sn> <fld>(Law)</fld> <sd>(a)</sd> <def>To bring to notice, deliver, or execute, either actually or constructively, in such manner as the law requires; <as>as, to <ex>serve</ex> a summons</as>.</def> <sd>(b)</sd> <def>To make legal service opon (a person named in a writ, summons, etc.); <as>as, to <ex>serve</ex> a witness with a subp<oe/na</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>12.</sn> <def>To pass or spend, as time, esp. time of punishment; <as>as, to <ex>serve</ex> a term in prison</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>13.</sn> <def>To copulate with; to cover; <as>as, a horse <ex>serves</ex> a mare</as>; -- said of the male.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>14.</sn> <fld>(Tennis)</fld> <def>To lead off in delivering (the ball).</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>15.</sn> <fld>(Naut.)</fld> <def>To wind spun yarn, or the like, tightly around (a rope or cable, etc.) so as to protect it from chafing or from the weather. See under <er>Serving</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><mcol><col><b>To serve an attachment</b></col> <it>or</it> <col><b>To serve a writ of attachment</b></col></mcol> <fld>(Law)</fld>, <cd>to levy it on the person or goods by seizure, or to seize.</cd> -- <col><b>To serve an execution</b></col> <fld>(Law)</fld>, <cd>to levy it on a lands, goods, or person, by seizure or taking possession.</cd> -- <col><b>To serve an office</b></col>, <cd>to discharge a public duty.</cd> -- <col><b>To serve a process</b></col> <fld>(Law)</fld>, <cd>in general, to read it, so as to give due notice to the party concerned, or to leave an attested copy with him or his attorney, or his usual place of abode.</cd> -- <col><b>To serve a warrant</b></col>, <cd>to read it, and seize the person against whom it is issued.</cd> -- <col><b>To serve a writ</b></col> <fld>(Law)</fld>, <cd>to read it to the defendant, or to leave an attested copy at his usual place of abode.</cd> -- <col><b>To serve one out</b></col>, <cd>to retaliate upon; to requite.</cd> <ldquo/I'll <xex>serve you out</xex> for this.<rdquo/ <au>C. Kingsley.</au> -- <col><b>To serve one right</b></col>, <cd>to treat, or cause to befall one, according to his deserts; -- used commonly of ill deserts; <as>as, it <ex>serves</ex> the scoundrel <ex>right</ex></as>.</cd> -- <col><b>To serve one's self of</b></col>, <cd>to avail one's self of; to make use of.</cd> <mark>[A Gallicism]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>I will <qex>serve myself of</qex> this concession.</q> <rj><qau>Chillingworth.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p> -- <col><b>To serve out</b></col>, <cd>to distribute; <as>as, <ex>to serve out</ex> rations</as>.</cd> -- <mcol><col><b>To serve the time</b></col> <it>or</it> <col><b>To serve the hour</b></col></mcol>, <cd>to regulate one's actions by the requirements of the time instead of by one's duty; to be a timeserver.</cd> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>They think herein we <qex>serve the time</qex>, because thereby we either hold or seek preferment.</q> <rj><qau>Hooker.</qau></rj>
</cs><br/
[<source>1913 Webster</source>]</p>

<p><syn><b>Syn.</b> -- To obey; minister to; subserve; promote; aid; help; assist; benefit; succor.</syn><br/
[<source>1913 Webster</source>]</p>

<p><ent>Serve</ent><br/
<hw>Serve</hw> <pr>(?)</pr>, <pos>v. i.</pos> <sn>1.</sn> <def>To be a servant or a slave; to be employed in labor or other business for another; to be in subjection or bondage; to render menial service.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The Lord shall give thee rest . . . from the hard bondage wherein thou wast made to <qex>serve</qex>.</q> <rj><qau>Isa. xiv. 3.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To perform domestic offices; to be occupied with household affairs; to prepare and dish up food, etc.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>But Martha . . . said, Lord, dost thou not care that my sister hath left me to <qex>serve</qex> alone?</q> <rj><qau>Luke x. 40.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>To be in service; to do duty; to discharge the requirements of an office or employment. Specifically, to act in the public service, as a soldier, seaman. etc.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Many . . . who had before been great commanders, but now <qex>served</qex> as private gentlemen without pay.</q> <rj><qau>Knolles.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>To be of use; to answer a purpose; to suffice; to suit; to be convenient or favorable.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>This little brand will <qex>serve</qex> to light your fire.</q> <rj><qau>Dryden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>As occasion <qex>serves</qex>, this noble queen<br/
And prince shall follow with a fresh supply.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <fld>(Tennis)</fld> <def>To lead off in delivering the ball.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Server</ent><br/
<hw>Serv"er</hw> <pr>(?)</pr>, <pos>n.</pos> <sn>1.</sn> <def>One who serves.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A tray for dishes; a salver.</def>  <rj><au>Randolph.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Servian</ent><br/
<hw>Ser"vi*an</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Of or pertaining to Servia, a kingdom of Southern Europe.</def> -- <def2><pos>n.</pos> <def>A native or inhabitant of Servia.</def></def2>
<-- modern spelling, Serbia --><br/
[<source>1913 Webster</source>]</p>

<p><ent>Service</ent><br/
<ent>Service</ent><br/
<mhw>{ <hw>Serv"ice</hw> <pr>(?)</pr>, <pos>n.</pos>, <it>or</it> <hw>Serv"ice</hw> <pr>(?)</pr> }</mhw>. <ety>[Properly, the tree which bears <ets>serve</ets>, OE. <ets>serves</ets>, pl., service berries, AS. <ets>syrfe</ets> service tree; akin to L. <ets>sorbus</ets>.]</ety> <fld>(Bot.)</fld> <def>A name given to several trees and shrubs of the genus <gen>Pyrus</gen>, as <spn>Pyrus domestica</spn> and <spn>Pyrus torminalis</spn> of Europe, the various species of mountain ash or rowan tree, and the American shad bush (see <cref>Shad bush</cref>, under <er>Shad</er>). They have clusters of small, edible, applelike berries.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Service berry</b></col> <fld>(Bot.)</fld>, <cd>the fruit of any kind of service tree. In British America the name is especially applied to that of the several species or varieties of the shad bush (<gen>Amelanchier</gen>.)</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Service</ent><br/
<hw>Serv"ice</hw>, <pos>n.</pos> <ety>[OE. <ets>servise</ets>, OF. <ets>servise</ets>, <ets>service</ets>, F. <ets>service</ets>, from L. <ets>servitium</ets>. See <er>Serve</er>.]</ety> <sn>1.</sn> <def>The act of serving; the occupation of a servant; the performance of labor for the benefit of another, or at another's command; attendance of an inferior, hired helper, slave, etc., on a superior, employer, master, or the like; also, spiritual obedience and love.</def> <ldquo/O God . . . whose <xex>service</xex> is perfect freedom.<rdquo/  <rj><au>Bk. of Com. Prayer.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Madam, I entreat true peace of you,<br/
 Which I will purchase with my duteous <qex>service</qex>.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>God requires no man's <qex>service</qex> upon hard and unreasonable terms.</q> <rj><qau>Tillotson.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>The deed of one who serves; labor performed for another; duty done or required; office.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>I have served him from the hour of my nativity, . . . and have nothing at his hands for my <qex>service</qex> but blows.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>This poem was the last piece of <qex>service</qex> I did for my master, King Charles.</q> <rj><qau>Dryden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>To go on the forlorn hope is a <qex>service</qex> of peril; who will understake it if it be not also a <qex>service</qex> of honor?</q> <rj><qau>Macaulay.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>Office of devotion; official religious duty performed; religious rites appropriate to any event or ceremonial; <as>as, a burial <ex>service</ex></as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The outward <qex>service</qex> of ancient religion, the rites, ceremonies, and ceremonial vestments of the old law.</q> <rj><qau>Coleridge.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>Hence, a musical composition for use in churches.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <def>Duty performed in, or appropriate to, any office or charge; official function; hence, specifically, military or naval duty; performance of the duties of a soldier.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>When he cometh to experience of <qex>service</qex> abroad . . . ne maketh a worthy soldier.</q> <rj><qau>Spenser.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>6.</sn> <def>Useful office; advantage conferred; that which promotes interest or happiness; benefit; avail.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The stork's plea, when taken in a net, was the <qex>service</qex> she did in picking up venomous creatures.</q> <rj><qau>L'Estrange.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>7.</sn> <def>Profession of respect; acknowledgment of duty owed.</def> <ldquo/Pray, do my <xex>service</xex> to his majesty.<rdquo/  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>8.</sn> <def>The act and manner of bringing food to the persons who eat it; order of dishes at table; also, a set or number of vessels ordinarily used at table; <as>as, the <ex>service</ex> was tardy and awkward; a <ex>service</ex> of plate or glass</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>There was no extraordinary <qex>service</qex> seen on the board.</q> <rj><qau>Hakewill.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>9.</sn> <fld>(Law)</fld> <def>The act of bringing to notice, either actually or constructively, in such manner as is prescribed by law; <as>as, the <ex>service</ex> of a subp<oe/na or an attachment</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>10.</sn> <fld>(Naut.)</fld> <def>The materials used for serving a rope, etc., as spun yarn, small lines, etc.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>11.</sn> <fld>(Tennis)</fld> <def>The act of serving the ball.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>12.</sn> <def>Act of serving or covering. See <er>Serve</er>, <pos>v. t.</pos>, 13.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Service book</b></col>, <cd>a prayer book or missal.</cd> -- <col><b>Service line</b></col> <fld>(Tennis)</fld>, <cd>a line parallel to the net, and at a distance of 21 feet from it.</cd> -- <mcol><col><b>Service of a writ</b></col>, <col><b>process</b></col>, etc.</mcol> <fld>(Law)</fld>, <cd>personal delivery or communication of the writ or process, etc., to the party to be affected by it, so as to subject him to its operation; the reading of it to the person to whom notice is intended to be given, or the leaving of an attested copy with the person or his attorney, or at his usual place of abode.</cd> -- <col><b>Service of an attachment</b></col> <fld>(Law)</fld>, <cd>the seizing of the person or goods according to the direction.</cd> -- <col><b>Service of an execution</b></col> <fld>(Law)</fld>, <cd>the levying of it upon the goods, estate, or person of the defendant.</cd> -- <col><b>Service pipe</b></col>, <cd>a pipe connecting mains with a dwelling, as in gas pipes, and the like.</cd> <au>Tomlinson.</au> -- <col><b>To accept service</b></col>. <fld>(Law)</fld> <cd>See under <er>Accept</er>.</cd> -- <col><b>To see service</b></col> <fld>(Mil.)</fld>, <cd>to do duty in the presence of the enemy, or in actual war.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Serviceable</ent><br/
<hw>Serv"ice*a*ble</hw> <pr>(?)</pr>, <pos>a.</pos> <sn>1.</sn> <def>Doing service; promoting happiness, interest, advantage, or any good; useful to any end; adapted to any good end use; beneficial; advantageous.</def> <ldquo/<xex>Serviceable</xex> to religion and learning<rdquo/. <au>Atterbury.</au> <ldquo/<xex>Serviceable</xex> tools.<rdquo/ <au>Macaulay.</au><br/
[<source>1913 Webster</source>]</p>

<p><q>I know thee well, a <qex>serviceable</qex> villain.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Prepared for rendering service; capable of, or fit for, the performance of duty; hence, active; diligent.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Courteous he was, lowly, and <qex>servysable</qex>.</q> <rj><qau>Chaucer.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Bright-hearnessed angels sit in order <qex>serviceable</qex>.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Seeing her so sweet and <qex>serviceable</qex>.</q> <rj><qau>Tennnyson.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p>-- <wordforms><wf>Serv"ice*a*ble*ness</wf>, <pos>n.</pos> -- <wf>Serv"ice*a*bly</wf>, <pos>adv.</pos></wordforms><br/
[<source>1913 Webster</source>]</p>

<p><ent>Serviceage</ent><br/
<hw>Serv"ice*age</hw> <pr>(?)</pr>, <pos>n.</pos> <def>Servitude.</def> <mark>[Obs.]</mark>  <rj><au>Fairfax.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Service hat</ent><br/
<ent>Service cap</ent><br/
<mhw>{ <hw>Serv"ice cap</hw> <it>or</it> <hw>Serv"ice hat</hw> }</mhw>. <fld>(Mil.)</fld> <def>A cap or hat worn by officers or enlisted men when full-dress uniform, or dress uniform, is not worn.</def> <note>In the United States army the <ex>service cap</ex> is round, about 3<frac12/ inches high, flat-topped, with  a visor. The <ex>service hat</ex> is of soft felt of khaki color, with  broad brim and high crown, creased down the middle.</note><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Service elevator</ent><br/
<hw>Serv"ice elevator</hw> <def>an elevator for carrying large or havy items, or for the use of service people.</def><br/
[<source>PJC</source>]</p>

<p><ent>Service entrance</ent><br/
<hw>Serv"ice entrance</hw> <def>an entrance for use of servants and those providing services, such as deliveries or repair work.</def><br/
[<source>PJC</source>]</p>

<p><ent>Service station</ent><br/
<hw>Serv"ice station</hw> <def>a business where automobiles may be serviced and repaired, usually also having a gasoline vending apparatus, in which case it is also called a <altname>gas station</altname> or <altname>gasoline station</altname>.  Facilities for repair of automobiles but without a gasoline dispensing function are commonly called <contr>repair shops</contr>.</def><br/
[<source>PJC</source>]</p>

<p><ent>Service uniform</ent><br/
<hw>Service uniform</hw>. <fld>(Mil. & Nav.)</fld> <def>The uniform prescribed in regulations for active or routine service, in distinction from dress, full dress, etc. In the United States army it is of olive-drab woolen or khaki-colored cotton, with all metal attachments of dull-finish bronze, with the exceptional of insignia of rank, which are of gold or silver finish.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Servient</ent><br/
<hw>Serv"i*ent</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>serviens</ets>, <ets>-entis</ets>, p. pr. See <er>Serve</er>.]</ety> <def>Subordinate.</def> <mark>[Obs. except in law.]</mark>  <rj><au>Dyer.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><mcol><col><b>Servient tenement</b></col> <it>or</it> <col><b>Servient estate</b></col></mcol> <fld>(Law)</fld>, <cd>that on which the burden of a servitude or an easement is imposed.  Cf. <xex>Dominant estate</xex>, under <er>Dominant</er>.</cd>  <rj><au>Gale & Whately.</au></rj></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Serviette</ent><br/
||<hw>Ser`viette"</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F.]</ety> <def>A table napkin.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Servile</ent><br/
<hw>Serv"ile</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>servile</ets>, fr. <ets>servus</ets> a servant or slave: cf. F. <ets>servile</ets>. See <er>Serve</er>.]</ety> <sn>1.</sn> <def>Of or pertaining to a servant or slave; befitting a servant or a slave; proceeding from dependence; hence, meanly submissive; slavish; mean; cringing; fawning; <as>as, <ex>servile</ex> flattery; <ex>servile</ex> fear; <ex>servile</ex> obedience.</as></def><br/
[<source>1913 Webster</source>]</p>

<p><q>She must bend the <qex>servile</qex> knee.</q> <rj><qau>Thomson.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Fearing dying pays death <qex>servile</qex> breath.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Held in subjection; dependent; enslaved.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Even fortune rules no more, O <qex>servile</qex> land!</q> <rj><qau>Pope.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <fld>(Gram.)</fld> <sd>(a)</sd> <def>Not belonging to the original root; <as>as, a <ex>servile</ex> letter</as>.</def> <sd>(b)</sd> <def>Not itself sounded, but serving to lengthen the preceeding vowel, as <xex>e</xex> in <xex>tune</xex>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Servile</ent><br/
<hw>Serv"ile</hw>, <pos>n.</pos> <fld>(Gram.)</fld> <def>An element which forms no part of the original root; -- opposed to <contr>radical</contr>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Servilely</ent><br/
<hw>Serv"ile*ly</hw>, <pos>adv.</pos> <def>In a servile manner; slavishly.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Servileness</ent><br/
<hw>Serv"ile*ness</hw>, <pos>n.</pos> <def>Quality of being servile; servility.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Servility</ent><br/
<hw>Ser*vil"i*ty</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Cf. F. <ets>servilit<eacute/</ets>.]</ety> <def>The quality or state of being servile; servileness.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>To be a queen in bondage is more vile<br/
Than is a slave in base <qex>servility</qex>.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Serving</ent><br/
<hw>Serv"ing</hw>, <def><pos>a. & n.</pos> from <er>Serve</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Serving board</b></col> <fld>(Naut.)</fld>, <cd>a flat piece of wood used in serving ropes.</cd> -- <col><b>Serving maid</b></col>, <cd>a female servant; a maidservant.</cd> -- <col><b>Serving mallet</b></col> <fld>(Naut.)</fld>, <cd>a wooden instrument shaped like a mallet, used in serving ropes.</cd> -- <col><b>Serving man</b></col>, <cd>a male servant, or attendant; a manservant.</cd> -- <col><b>Serving stuff</b></col> <fld>(Naut.)</fld>, <cd>small lines for serving ropes.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Servite</ent><br/
<hw>Serv"ite</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[It. <ets>servita</ets>.]</ety> <fld>(R.C.Ch.)</fld> <def>One of the order of the Religious Servants of the Holy Virgin, founded in Florence in 1223.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Servitor</ent><br/
<hw>Serv"i*tor</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L., fr. <ets>servire</ets> to serve: cf. F. <ets>serviteur</ets>.]</ety> <sn>1.</sn> <def>One who serves; a servant; an attendant; one who acts under another; a follower or adherent.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Your trusty and most valiant <qex>servitor</qex>.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Univ. of Oxford, Eng.)</fld> <def>An undergraduate, partly supported by the college funds, whose duty it formerly was to wait at table. A <xex>servitor</xex> corresponded to a <xex>sizar</xex> in Cambridge and Dublin universities.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Servitorship</ent><br/
<hw>Serv"i*tor*ship</hw>, <pos>n.</pos> <def>The office, rank, or condition of a servitor.</def>  <rj><au>Boswell.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Servitude</ent><br/
<hw>Serv"i*tude</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>servitudo</ets>: cf. F. <ets>servitude</ets>.]</ety> <sn>1.</sn> <def>The state of voluntary or compulsory subjection to a master; the condition of being bound to service; the condition of a slave; slavery; bondage; hence, a state of slavish dependence.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>You would have sold your king to slaughter,<br/
His princes and his peers to <qex>servitude</qex>.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>A splendid <qex>servitude</qex>; . . . for he that rises up early, and goes to bed late, only to receive addresses, is really as much abridged in his freedom as he that waits to present one.</q> <rj><qau>South.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Servants, collectively.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>After him a cumbrous train<br/
Of herds and flocks, and numerous <qex>servitude</qex>.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <fld>(Law)</fld> <def>A right whereby one thing is subject to another thing or person for use or convenience, contrary to the common right.</def><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ The object of a <xex>servitude</xex> is either to suffer something to be done by another, or to omit to do something, with respect to a thing. The <xex>easements</xex> of the English correspond in some respects with the <xex>servitudes</xex> of the Roman law. Both terms are used by common law writers, and often indiscriminately. The former, however, rather indicates the right enjoyed, and the latter the burden imposed.</note>  <rj><au>Ayliffe. Erskine. E. Washburn.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Penal servitude</b></col>. <cd>See under <er>Penal</er>.</cd> -- <col><b>Personal servitude</b></col> <fld>(Law)</fld>, <cd>that which arises when the use of a thing is granted as a real right to a particular individual other than the proprietor.</cd> -- <col><b>Predial servitude</b></col> <fld>(Law)</fld>, <cd>that which one estate owes to another estate. When it related to lands, vineyards, gardens, or the like, it is called <xex>rural</xex>; when it related to houses and buildings, it is called <xex>urban</xex>.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Serviture</ent><br/
<hw>Serv"i*ture</hw> <pr>(?)</pr>, <pos>n.</pos> <def>Servants, collectively.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Servitute</ent><br/
<hw>Serv"i*tute</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>servitus</ets>.]</ety> <def>Servitude.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Servocontrol</ent><br/
<hw>Ser`vo*con*trol"</hw> <pr>(?)</pr>, <pos>n.</pos> <sn>1.</sn> <def>control by a <er>servomechanism</er>.</def><br/
[<source>PJC</source>]</p>

<p><sn>2.</sn> <def>a <er>servomechanism</er>.</def><br/
[<source>PJC</source>]</p>

<p><ent>Servomechanism</ent><br/
<hw>Ser`vo*mech"an*ism</hw> <pr>(?)</pr>, <pos>n.</pos> <def>an electronically controlled mechanical or hydraulic device permitting a large action or strong forces to be controlled by a small electrical signal.</def><br/
[<source>PJC</source>]</p>

<p><ent>Servo-motor</ent><br/
<hw>Ser`vo-mo"tor</hw> <pr>(s<etil/r`v<osl/*m<omac/"t<etil/r)</pr>, <pos>n.</pos> <ety>[Sometimes erroneously spelt <ets>serro-motor</ets>.]</ety> <ety>[F. <ets>servo-moteur</ets>. See <er>Serf</er>; <er>Motor</er>.]</ety> <sn>1.</sn> <fld>(Mach.)</fld> <def>A relay apparatus;</def> <specif>specif.:</specif> <sd>(a)</sd> <def>An auxiliary motor, regulated by a hand lever, for quickly and easily moving the reversing gear of a large marine engine into any desired position indicated by that of the hand lever, which controls the valve of the motor.</def> <sd>(b)</sd> <def>In a Whitehead torpedo, a compressed-air motor, for moving the rudders so as to correct deviations from the course.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><sn>2.</sn> <def>a motor which forms part of a <er>servomechanism</er>.</def><br/
[<source>PJC</source>]</p>

<p><ent>Serye</ent><br/
<hw>Se"rye</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A series.</def> <mark>[Obs.]</mark>  <rj><au>Chaucer.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sesame</ent><br/
<hw>Ses"a*me</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>sesamum</ets>, <ets>sesama</ets>, Gr. <?/<?/<?/<?/, <?/<?/<?/: cf. F. <ets>s<eacute/same</ets>.]</ety> <fld>(Bot.)</fld> <def>Either of two annual herbaceous plants of the genus <gen>Sesamum</gen> (<spn>Sesamum Indicum</spn>, and <spn>Sesamum orientale</spn>), from the seeds of which an oil is expressed; also, the small obovate, flattish seeds of these plants, sometimes used as food. See <er>Benne</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Open Sesame</b></col>, <cd>the magical command which opened the door of the robber's den in the Arabian Nights' tale of <ldquo/<i>The Forty Thieves</i>;<rdquo/ hence, a magical password.</cd> -- <col><b>Sesame grass</b></col>. <fld>(Bot.)</fld> <cd>Same as <er>Gama grass</er>.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sesamoid</ent><br/
<hw>Ses"a*moid</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[Gr. <?/<?/<?/<?/ like sesame; <?/<?/<?/<?/<?/ sesame + <grk>e'i^dos</grk> form: cf. F. <ets>s<eacute/samo<ium/de</ets>.]</ety> <sn>1.</sn> <def>Resembling in shape the seeds of sesame.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Anat.)</fld> <def>Of or pertaining to the sesamoid bones or cartilages; sesamoidal.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><mcol><col><b>Sesamoid bones</b></col>, <col><b>Sesamoid cartilages</b></col></mcol> <fld>(Anat.)</fld>, <cd>small bones or cartilages formed in tendons, like the patella and pisiform in man.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sesamoid</ent><br/
<hw>Ses"a*moid</hw>, <pos>n.</pos> <fld>(Anat.)</fld> <def>A sesamoid bone or cartilage.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sesamoidal</ent><br/
<hw>Ses`a*moid"al</hw> <pr>(?)</pr>, <pos>a.</pos> <fld>(Anat.)</fld> <def>Sesamoid.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sesban</ent><br/
<hw>Ses"ban</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F., fr. Ar. <ets>saisab<amac/n</ets>, <ets>seiseb<amac/n</ets>, a kind of tree, fr. Per. <ets>s<imac/sab<amac/n</ets> seed of cinquefoil.]</ety> <fld>(Bot.)</fld> <def>A leguminous shrub (<spn>Sesbania aculeata</spn>) which furnishes a fiber used for making ropes.</def><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ The name is applied also to the similar plant, <spn>Sesbania Aegyptiaca</spn>, and other species of the same genus.</note><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sesqui-</ent><br/
||<hw>Ses`qui-</hw> <pr>(?)</pr>. <ety>[L., one half more, one and a half.]</ety> <fld>(Chem.)</fld> <def>A combining form (also used adjectively) denoting that <xex>three atoms or equivalents</xex> of the substance to the name of which it is prefixed are combined with <xex>two</xex> of some other element or radical; <as>as, <ex>sesqui</ex>bromide, <ex>sesqui</ex>carbonate, <ex>sesqui</ex>chloride, <ex>sesqui</ex>oxide</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ <xex>Sesquidupli-</xex> is sometimes, but rarely, used in the same manner to denote the proportions of <xex>two and a half</xex> to <xex>one</xex>, or rather of <xex>five</xex> to <xex>two</xex>.</note><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sesquialter</ent><br/
<hw>Ses`qui*al"ter</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Sesquialteral.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sesquialtera</ent><br/
<ent>Sesquialter</ent><br/
<mhw>{ <hw>Ses`qui*al"ter</hw> <pr>(?)</pr>, <hw>Ses`qui*al"ter*a</hw> <pr>(?)</pr>, }</mhw> <pos>n.</pos> <ety>[NL. <ets>sesquialtera</ets>.]</ety> <fld>(Mus.)</fld> <def>A stop on the organ, containing several ranks of pipes which reenforce some of the high harmonics of the ground tone, and make the sound more brilliant.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sesquialterate</ent><br/
<ent>Sesquialteral</ent><br/
<mhw>{ <hw>Ses`qui*al"ter*al</hw> <pr>(?)</pr>, <hw>Ses`qui*al"ter*ate</hw> <pr>(?)</pr>, }</mhw> <pos>a.</pos> <ety>[L. <ets>sesquialter</ets> once and a half; <ets>sesqui-</ets> + <ets>alter</ets> other: cf. F. <ets>sesquialt<eacute/re</ets>.]</ety> <def>Once and a half times as great as another; having the ratio of one and a half to one.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Sesquialteral ratio</b></col> <fld>(Math.)</fld>, <cd>the ratio of one and a half to one; <as>thus, 9 and 6 are in a <ex>sesquialteral ratio</ex></as>.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sesquialterous</ent><br/
<hw>Ses`qui*al"ter*ous</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Sesquialteral.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sesquibasic</ent><br/
<hw>Ses`qui*ba"sic</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[<ets>Sesqui-</ets> +  <ets>basic</ets>.]</ety> <fld>(Chem.)</fld> <def>Containing, or acting as, a base in the proportions of a sesqui compound.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sesquiduplicate</ent><br/
<hw>Ses`qui*du"pli*cate</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[<ets>Sesqui-</ets> + <ets>duplicate</ets>.]</ety> <def>Twice and a half as great (as another thing); having the ratio of two and a half to one.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Sesquiduplicate ratio</b></col> <fld>(Math.)</fld>, <cd>the ratio of two and a half to one, or one in which the greater term contains the lesser twice and a half, as that of 50 to 20.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sesquioxide</ent><br/
<hw>Ses`qui*ox"ide</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[<ets>Sesqui-</ets> + <ets>oxide</ets>.]</ety> <fld>(Chem.)</fld> <def>An oxide containing three atoms of oxygen with two atoms (or radicals) of some other substance; <as>thus, alumina, <chform>Al2O3</chform> is a <ex>sesquioxide</ex></as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sesquipedalian</ent><br/
<ent>Sesquipedal</ent><br/
<mhw>{ <hw>Ses*quip"e*dal</hw> <pr>(?)</pr>, <hw>Ses`qui*pe*da"li*an</hw> <pr>(?)</pr>, }</mhw> <pos>a.</pos> <ety>[<ets>Sesqui-</ets> + <ets>pedal</ets>: cf. F. <ets>sesquip<eacute/dal</ets>, L. <ets>sesquipedalis</ets>.]</ety> <def>Measuring or containing a foot and a half; <as>as, a <ex>sesquipedalian</ex> pygmy</as>; -- sometimes humorously applied to long words.</def><br/
[<source>1913 Webster</source>]</p>

<p><-- 2. having a tendency to use long words. --><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sesquipedalism</ent><br/
<ent>Sesquipedalianism</ent><br/
<mhw>{ <hw>Ses`qui*pe*da"li*an*ism</hw> <pr>(?)</pr>, <hw>Ses*quip"e*dal*ism</hw> <pr>(?)</pr>, }</mhw> <pos>n.</pos> <def>Sesquipedality.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sesquipedality</ent><br/
<hw>Ses`qui*pe*dal"i*ty</hw> <pr>(?)</pr>, <pos>n.</pos> <sn>1.</sn> <def>The quality or condition of being sesquipedal.</def>  <rj><au>Sterne.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>The use of sesquipedalian words; style characterized by the use of long words; sesquipedalism.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sesquiplicate</ent><br/
<p><hw>Ses*quip"li*cate</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[<ets>Sesqui-</ets> + <ets>plicate</ets>.]</ety> <fld>(Math.)</fld> <def>Subduplicate of the triplicate; -- a term applied to ratios; thus, <xex>a</xex> and <xex>a\'b7</xex> are in the <xex>sesquiplicate</xex> ratio of <xex>b</xex> and <xex>b\'b7</xex>, when <xex>a</xex> is to <xex>a\'b7</xex> as the square root of the cube of <xex>b</xex> is to the square root of the cube of <xex>b\'b7</xex>, or <mathex>a:a\'b7::<root/b<exp>3</exp>:<root/b\'b7<exp>3</exp></mathex>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The periodic times of the planets are in the <qex>sesquiplicate</qex> ratio of their mean distances.</q> <rj><qau>Sir I. Newton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sesquisalt</ent><br/
<hw>Ses"qui*salt</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[<ets>Sesqui-</ets> + <ets>salt</ets>.]</ety> <fld>(Chem.)</fld> <def>A salt derived from a sesquioxide base, or made up on the proportions of a sesqui compound.</def><br/
[<source>1913 Webster</source>]</p>

<p><-- p. 1317 --></p>

<p><ent>Sesquisulphide</ent><br/
<hw>Ses`qui*sul"phide</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[<ets>Sesqui-</ets> + <ets>sulphide</ets>.]</ety> <fld>(Chem.)</fld> <def>A sulphide, analogous to a sesquioxide, containing three atoms of sulphur to two of the other ingredient; -- formerly called also <altname>sesquisulphuret</altname>; <as>as, orpiment, <chform>As2S3</chform> is arsenic <ex>sesquisulphide</ex></as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sesquitertial</ent><br/
<hw>Ses`qui*ter"tial</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Sesquitertian.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sesquitertianal</ent><br/
<ent>Sesquitertian</ent><br/
<mhw>{ <hw>Ses`qui*ter"tian</hw> <pr>(?)</pr>, <hw>Ses`qui*ter"tian*al</hw> <pr>(?)</pr>, }</mhw> <pos>a.</pos> <ety>[<ets>Sesqui-</ets> + L. <ets>tertianus</ets> belonging to the third.  Cf. <er>Tertian</er>.]</ety> <fld>(Math.)</fld> <def>Having the ratio of one and one third to one (as 4 : 3).</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sesquitone</ent><br/
<hw>Ses"qui*tone</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[<ets>Sesqui-</ets> +  <ets>tone</ets>.]</ety> <fld>(Mus.)</fld> <def>A minor third, or interval of three semitones.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sess</ent><br/
<hw>Sess</hw> <pr>(?)</pr>, <pos>v. t.</pos> <ety>[Aphetic form of <ets>assess</ets>. See <er>Assess</er>, <er>Cess</er>.]</ety> <def>To lay a tax upon; to assess.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sess</ent><br/
<hw>Sess</hw>, <pos>n.</pos> <def>A tax; an assessment. See <er>Cess</er>.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sessa</ent><br/
<hw>Ses"sa</hw> <pr>(?)</pr>, <pos>interj.</pos> <def>Hurry; run.</def> <mark>[Obs.]</mark>  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sessile</ent><br/
<hw>Ses"sile</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>sessilis</ets> low, dwarf, from <ets>sedere</ets>, <ets>sessum</ets>, to sit: cf. F. <ets>sessile</ets>.]</ety> <sn>1.</sn> <def>Attached without any sensible projecting support.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Bot.)</fld> <def>Resting directly upon the main stem or branch, without a petiole or footstalk; <as>as, a <ex>sessile</ex> leaf or blossom</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <fld>(Zool.)</fld> <def>Permanently attached; -- said of the gonophores of certain hydroids which never became detached.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sessile-eyed</ent><br/
<hw>Ses"sile-eyed`</hw> <pr>(?)</pr>, <pos>a.</pos> <fld>(Zool.)</fld> <def>Having eyes which are not elevated on a stalk; -- opposed to <contr>stalk-eyed</contr>.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Sessile-eyed Crustacea</b></col>, <cd>the Arthrostraca.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Session</ent><br/
<hw>Ses"sion</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>sessio</ets>, fr.  <ets>sedere</ets>, <ets>sessum</ets>, to sit: cf. F. <ets>session</ets>. See <er>Sit</er>.]</ety> <sn>1.</sn> <def>The act of sitting, or the state of being seated.</def> <mark>[Archaic]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>So much his ascension into heaven and his <qex>session</qex> at the right hand of God do import.</q> <rj><qau>Hooker.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>But Viven, gathering somewhat of his mood, . . . <br/
Leaped from her <qex>session</qex> on his lap, and stood.</q> <rj><qau>Tennyson.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>The actual sitting of a court, council, legislature, etc., or the actual assembly of the members of such a body, for the transaction of business.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>It's fit this royal <qex>session</qex> do proceed.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>Hence, also, the time, period, or term during which a court, council, legislature, etc., meets daily for business; or, the space of time between the first meeting and the prorogation or adjournment; thus, a <xex>session</xex> of Parliaments is opened with a speech from the throne, and closed by prorogation. The <xex>session</xex> of a judicial court is called a <xex>term</xex>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>It was resolved that the convocation should meet at the beginning of the next <qex>session</qex> of Parliament.</q> <rj><qau>Macaulay.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ <xex>Sessions</xex>, in some of the States, is particularly used as a title for a court of justices, held for granting licenses to innkeepers, etc., and for laying out highways, and the like; it is also the title of several courts of criminal jurisdiction in England and the United States.</note><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Church session</b></col>, <cd>the lowest court in the Presbyterian Church, composed of the pastor and a body of elders elected by the members of a particular church, and having the care of matters pertaining to the religious interests of that church, as the admission and dismission of members, discipline, etc.</cd> -- <col><b>Court of Session</b></col>, <cd>the supreme civil court of Scotland.</cd> -- <col><b>Quarter sessions</b></col>. <fld>(Eng.Law)</fld> <cd>See under <er>Quarter</er>.</cd> -- <col><b>Sessions of the peace</b></col>, <cd>sittings held by justices of the peace.</cd> <mark>[Eng.]</mark></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sessional</ent><br/
<hw>Ses"sion*al</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Of or pertaining to a session or sessions.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sesspool</ent><br/
<hw>Sess"pool`</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Prov. E. <ets>suss</ets> hogwash, <ets>soss</ets> a dirty mess, a puddle + E. <ets>pool</ets> a puddle; cf. Gael. <ets>ses</ets> a coarse mess.]</ety> <def>Same as <er>Cesspool</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sesterce</ent><br/
<hw>Ses"terce</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>sestertius</ets> (sc. <ets>nummus</ets>), fr. <ets>sestertius</ets> two and a half; <ets>semis</ets> half + <ets>tertius</ets> third: cf. F. <ets>sesterce</ets>.]</ety> <fld>(Rom. Antiq.)</fld> <def>A Roman coin or denomination of money, in value the fourth part of a denarius, and originally containing two asses and a half, afterward four asses, -- equal to about two pence sterling, or four cents.</def><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ The sestertium was equivalent to one thousand sesterces, equal to <pound/8 17s 1d. sterling, or about $43 (in 1900), before the reign of Augustus.  After his reign its value was about <pound/7 16s. 3d. sterling.  The sesterce was originally coined only in silver, but later both in silver and brass.</note><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sestet</ent><br/
<hw>Ses*tet"</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[It. <ets>sestetto</ets>, fr. <ets>sesto</ets> sixth, L. <ets>sextus</ets>, fr. <ets>sex</ets> six.]</ety> <sn>1.</sn> <fld>(Mus.)</fld> <def>A piece of music composed for six voices or six instruments; a sextet; -- called also <altname>sestuor</altname>.</def> <altsp>[Written also <asp>sestett</asp>, <asp>sestette</asp>.]</altsp><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Poet.)</fld> <def>The last six lines of a sonnet.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sestetto</ent><br/
||<hw>Ses*tet"to</hw> <pr>(?)</pr>, <pos>n.</pos><ety>[It.]</ety> <fld>(Mus.)</fld> <def>A sestet.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sestine</ent><br/
<hw>Ses"tine</hw> <pr>(?)</pr>, <pos>n.</pos> <def>See <er>Sextain</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sestuor</ent><br/
<hw>Ses"tu*or</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F.]</ety> <def>A sestet.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Set</ent><br/
<hw>Set</hw> <pr>(s<ecr/t)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Set</conjf>; <pos>p. pr. & vb. n.</pos> <conjf>Setting</conjf>.]</vmorph> <ety>[OE. <ets>setten</ets>, AS. <ets>setton</ets>; akin to OS. <ets>settian</ets>, OFries. <ets>setta</ets>, D. <ets>zetten</ets>, OHG. <ets>sezzen</ets>, G. <ets>setzen</ets>, Icel. <ets>setja</ets>, Sw. <ets>s<aum/tta</ets>, Dan. <ets>s<?/tte</ets>, Goth. <ets>satjan</ets>; causative from the root of E. <ets>sit</ets>. <root/154. See <er>Sit</er>, and cf. <er>Seize</er>.]</ety> <sn>1.</sn> <def>To cause to sit; to make to assume a specified position or attitude; to give site or place to; to place; to put; to fix; <as>as, to <ex>set</ex> a house on a stone foundation; to <ex>set</ex> a book on a shelf; to <ex>set</ex> a dish on a table; to <ex>set</ex> a chest or trunk on its bottom or on end</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>I do <qex>set</qex> my bow in the cloud.</q> <rj><qau>Gen. ix. 13.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Hence, to attach or affix (something) to something else, or in or upon a certain place.</def><br/
[<source>1913 Webster</source>]</p>

<p><q><qex>Set</qex> your affection on things above.</q> <rj><qau>Col. iii. 2.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>The Lord <qex>set</qex> a mark upon Cain.</q> <rj><qau>Gen. iv. 15.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>To make to assume specified place, condition, or occupation; to put in a certain condition or state (described by the accompanying words); to cause to be.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The Lord thy God will <qex>set</qex> thee on high.</q> <rj><qau>Deut. xxviii. 1.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>I am come to <qex>set</qex> a man at variance against his father, and the daughter against her mother.</q> <rj><qau>Matt. x. 35.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Every incident <qex>sets</qex> him thinking.</q> <rj><qau>Coleridge.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>To fix firmly; to make fast, permanent, or stable; to render motionless; to give an unchanging place, form, or condition to.</def> <specif>Specifically:</specif> --<br/
[<source>1913 Webster</source>]</p>

<p><sd>(a)</sd> <def>To cause to stop or stick; to obstruct; to fasten to a spot; hence, to occasion difficulty to; to embarrass; <as>as, to <ex>set</ex> a coach in the mud</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>They show how hard they are <qex>set</qex> in this particular.</q> <rj><qau>Addison.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sd>(b)</sd> <def>To fix beforehand; to determine; hence, to make unyielding or obstinate; to render stiff, unpliant, or rigid; <as>as, to <ex>set</ex> one's countenance</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>His eyes were <qex>set</qex> by reason of his age.</q> <rj><qau>1 Kings xiv. 4.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>On these three objects his heart was <qex>set</qex>.</q> <rj><qau>Macaulay.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Make my heart as a millstone, <qex>set</qex> my face as a flint.</q> <rj><qau>Tennyson.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sd>(c)</sd> <def>To fix in the ground, as a post or a tree; to plant; <as>as, to <ex>set</ex> pear trees in an orchard</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sd>(d)</sd> <def>To fix, as a precious stone, in a border of metal; to place in a setting; hence, to place in or amid something which serves as a setting; <as>as, to <ex>set</ex> glass in a sash</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>And him too rich a jewel to be <qex>set</qex><br/
In vulgar metal for a vulgar use.</q> <rj><qau>Dryden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sd>(e)</sd> <def>To render stiff or solid; especially, to convert into curd; to curdle; <as>as, to <ex>set</ex> milk for cheese</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <def>To put into a desired position or condition; to adjust; to regulate; to adapt.</def> Specifically: --
<-- to set a table --><br/
[<source>1913 Webster</source>]</p>

<p><sd>(a)</sd> <def>To put in order in a particular manner; to prepare; <as>as, to <ex>set</ex> (that is, to hone) a razor; to <ex>set</ex> a saw</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Tables for to <qex>sette</qex>, and beddes make.</q> <rj><qau>Chaucer.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sd>(b)</sd> <def>To extend and bring into position; to spread; <as>as, to <ex>set</ex> the sails of a ship</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sd>(c)</sd> <def>To give a pitch to, as a tune; to start by fixing the keynote; <as>as, to <ex>set</ex> a psalm</as>.</def>  <rj><au>Fielding.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sd>(d)</sd> <def>To reduce from a dislocated or fractured state; to replace; <as>as, to <ex>set</ex> a broken bone</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sd>(e)</sd> <def>To make to agree with some standard; <as>as, to <ex>set</ex> a watch or a clock</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sd>(f)</sd> <fld>(Masonry)</fld> <def>To lower into place and fix solidly, as the blocks of cut stone in a structure.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>6.</sn> <def>To stake at play; to wager; to risk.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>I have <qex>set</qex> my life upon a cast,<br/
And I will stand the hazard of the die.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>7.</sn> <def>To fit with music; to adapt, as words to notes; to prepare for singing.</def><br/
[<source>1913 Webster</source>]</p>

<p><q><qex>Set</qex> thy own songs, and sing them to thy lute.</q> <rj><qau>Dryden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>8.</sn> <def>To determine; to appoint; to assign; to fix; <as>as, to <ex>set</ex> a time for a meeting; to <ex>set</ex> a price on a horse</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>9.</sn> <def>To adorn with something infixed or affixed; to stud; to variegate with objects placed here and there.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>High on their heads, with jewels richly <qex>set</qex>,<br/
Each lady wore a radiant coronet.</q> <rj><qau>Dryden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Pastoral dales thin <qex>set</qex> with modern farms.</q> <rj><qau>Wordsworth.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>10.</sn> <def>To value; to rate; -- with <xex>at</xex>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Be you contented, wearing now the garland,<br/
To have a son <qex>set</qex> your decrees at naught.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>I do not <qex>set</qex> my life at a pin's fee.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>11.</sn> <def>To point out the seat or position of, as birds, or other game; -- said of hunting dogs.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>12.</sn> <def>To establish as a rule; to furnish; to prescribe; to assign; <as>as, to <ex>set</ex> an example; to <ex>set</ex> lessons to be learned</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>13.</sn> <def>To suit; to become; <as>as, it <ex>sets</ex> him ill</as>.</def> <mark>[Scot.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><sn>14.</sn> <fld>(Print.)</fld> <def>To compose; to arrange in words, lines, etc.; <as>as, to <ex>set</ex> type; to <ex>set</ex> a page.</as></def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>To set abroach</b></col>. <cd>See <er>Abroach</er>.</cd> <mark>[Obs.]</mark> <au>Shak.</au> -- <col><b>To set against</b></col>, <cd>to oppose; to set in comparison with, or to oppose to, as an equivalent in exchange; <as>as, <ex>to set</ex> one thing <ex>against</ex> another</as>.</cd> -- <col><b>To set agoing</b></col>, <cd>to cause to move.</cd> -- <col><b>To set apart</b></col>, <cd>to separate to a particular use; to separate from the rest; to reserve.</cd> -- <col><b>To set a saw</b></col>, <cd>to bend each tooth a little, every alternate one being bent to one side, and the intermediate ones to the other side, so that the opening made by the saw may be a little wider than the thickness of the back, to prevent the saw from sticking.</cd> -- <col><b>To set aside</b></col>. <sd>(a)</sd> <cd>To leave out of account; to pass by; to omit; to neglect; to reject; to annul.</cd><br/
[<source>1913 Webster</source>]</p>

<p><q><qex>Setting aside</qex> all other considerations, I will endeavor to know the truth, and yield to that.</q> <rj><qau>Tillotson.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sd>(b)</sd> <cd>To set apart; to reserve; <as>as, <ex>to set aside</ex> part of one's income</as>.</cd> <sd>(c)</sd> <fld>(Law)</fld> <cd>See under <er>Aside</er>.</cd> -- <col><b>To set at defiance</b></col>, <cd>to defy.</cd> -- <col><b>To set at ease</b></col>, <cd>to quiet; to tranquilize; <as>as, <ex>to set</ex> the heart <ex>at ease</ex></as>.</cd> -- <col><b>To set at naught</b></col>, <cd>to undervalue; to contemn; to despise.</cd> <ldquo/Ye have <xex>set at naught</xex> all my counsel.<rdquo/ <au>Prov. i. 25.</au> -- <mcol><col><b>To set a trap</b></col> <col><b>To set a snare</b></col>, <it>or</it> <col><b>To set a gin</b></col></mcol>, <cd>to put it in a proper condition or position to catch prey; hence, to lay a plan to deceive and draw another into one's power.</cd> -- <mcol><col><b>To set at work</b></col>, <it>or</it> <col><b>To set to work</b></col></mcol>. <sd>(a)</sd> <cd>To cause to enter on work or action, or to direct how tu enter on work.</cd> <sd>(b)</sd> <cd>To apply one's self; -- used reflexively.</cd> -- <col><b>To set before</b></col>. <sd>(a)</sd> <cd>To bring out to view before; to exhibit.</cd> <sd>(b)</sd> <cd>To propose for choice to; to offer to.</cd> -- <col><b>To set by</b></col>. <sd>(a)</sd> <cd>To set apart or on one side; to reject.</cd> <sd>(b)</sd> <cd>To attach the value of (anything) to.</cd> <ldquo/I <xex>set</xex> not a straw <xex>by</xex> thy dreamings.<rdquo/ <au>Chaucer.</au> -- <col><b>To set by the compass</b></col>, <cd>to observe and note the bearing or situation of by the compass.</cd> -- <col><b>To set case</b></col>, <cd>to suppose; to assume.  Cf. <cref>Put case</cref>, under <er>Put</er>, <pos>v. t.</pos></cd> <mark>[Obs.]</mark> <au>Chaucer.</au> -- <col><b>To set down</b></col>. <sd>(a)</sd> <cd>To enter in writing; to register.</cd><br/
[<source>1913 Webster</source>]</p>

<p><q>Some rules were to be <qex>set down</qex> for the government of the army.</q> <rj><qau>Clarendon.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sd>(b)</sd> <cd>To fix; to establish; to ordain.</cd><br/
[<source>1913 Webster</source>]</p>

<p><q>This law we may name eternal, being that order which God . . . hath <qex>set down</qex> with himself, for himself to do all things by.</q> <rj><qau>Hooker.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sd>(c)</sd> <cd>To humiliate.</cd> -- <col><b>To set eyes on</b></col>, <cd>to see; to behold; to fasten the eyes on.</cd> -- <mcol><col><b>To set fire to</b></col>, <it>or</it> <col><b>To set on fire</b></col></mcol>, <cd>to communicate fire to; fig., to inflame; to enkindle the passions of; to irritate.</cd> -- <col><b>To set flying</b></col> <fld>(Naut.)</fld>, <cd>to hook to halyards, sheets, etc., instead of extending with rings or the like on a stay; -- said of a sail.</cd> -- <col><b>To set forth</b></col>.  <sd>(a)</sd> <cd>To manifest; to offer or present to view; to exhibt; to display.</cd> <sd>(b)</sd> <cd>To publish; to promulgate; to make appear</cd>. <au>Waller.</au> <sd>(c)</sd> <cd>To send out; to prepare and send.</cd> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>The Venetian admiral had a fleet of sixty galleys, <qex>set forth</qex> by the Venetians.</q> <rj><qau>Knolles.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p>-- <col><b>To set forward</b></col>. <sd>(a)</sd> <cd>To cause to advance.</cd> <sd>(b)</sd> <cd>To promote.</cd> -- <col><b>To set free</b></col>, <cd>to release from confinement, imprisonment, or bondage; to liberate; to emancipate.</cd> -- <col><b>To set in</b></col>, <cd>to put in the way; to begin; to give a start to.</cd> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>If you please to assist and <qex>set</qex> me <qex>in</qex>, I will recollect myself.</q> <rj><qau>Collier.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p>-- <col><b>To set in order</b></col>, <cd>to adjust or arrange; to reduce to method.</cd> <ldquo/The rest will I <xex>set in order</xex> when I come.<rdquo/ <au>1 Cor. xi. 34.</au> -- <col><b>To set milk</b></col>. <sd>(a)</sd> <cd>To expose it in open dishes in order that the cream may rise to the surface.</cd> <sd>(b)</sd> <cd>To cause it to become curdled as by the action of rennet.  See 4 <sd>(e)</sd>.</cd> -- <mcol><col><b>To set much by</b></col> <it>or</it> <col><b>To set little by</b></col></mcol>, <cd>to care much, or little, for.</cd> -- <col><b>To set of</b></col>, <cd>to value; to set by.</cd> <mark>[Obs.]</mark> <ldquo/I <xex>set</xex> not an haw <xex>of</xex> his proverbs.<rdquo/ <au>Chaucer.</au> -- <col><b>To set off</b></col>. <sd>(a)</sd> <cd>To separate from a whole; to assign to a particular purpose; to portion off; <as>as, <ex>to set off</ex> a portion of an estate</as>.</cd> <sd>(b)</sd> <cd>To adorn; to decorate; to embellish</cd>.<br/
[<source>1913 Webster</source>]</p>

<p><q>They . . . <qex>set off</qex> the worst faces with the best airs.</q> <rj><qau>Addison.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sd>(c)</sd> <cd>To give a flattering description of.</cd> -- <col><b>To set off against</b></col>, <cd>to place against as an equivalent; <as>as, <ex>to set off</ex> one man's services <ex>against</ex> another's</as>.</cd> -- <mcol><col><b>To set on</b></col> <it>or</it> <col><b>To set upon</b></col></mcol>. <sd>(a)</sd> <cd>To incite; to instigate.</cd> <ldquo/Thou, traitor, hast <xex>set on</xex> thy wife to this.<rdquo/ <au>Shak.</au> <sd>(b)</sd> <cd>To employ, as in a task.</cd> <ldquo/ <xex>Set on</xex> thy wife to observe.<rdquo/ <au>Shak.</au> <sd>(c)</sd> <cd>To fix upon; to attach strongly to; as, <xex>to set</xex> one's heart or affections <xex>on</xex> some object. See definition 2, above.</cd> -- <col><b>To set one's cap for</b></col>. <cd>See under <er>Cap</er>, <pos>n.</pos></cd> -- <col><b>To set one's self against</b></col>, <cd>to place one's self in a state of enmity or opposition to.</cd> -- <col><b>To set one's teeth</b></col>, <cd>to press them together tightly.</cd> -- <col><b>To set on foot</b></col>, <cd>to set going; to put in motion; to start.</cd> -- <col><b>To set out</b></col>. <sd>(a)</sd> <cd>To assign; to allot; to mark off; to limit; <as>as, <ex>to set out</ex> the share of each proprietor or heir of an estate; <ex>to set out</ex> the widow's thirds</as>.</cd> <sd>(b)</sd> <cd>To publish, as a proclamation</cd>. <mark>[Obs.]</mark> <sd>(c)</sd> <cd>To adorn; to embellish.</cd><br/
[<source>1913 Webster</source>]</p>

<p><q>An ugly woman, in rich habit <qex>set out</qex> with jewels, nothing can become.</q> <rj><qau>Dryden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sd>(d)</sd> <cd>To raise, equip, and send forth; to furnish.</cd> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>The Venetians pretend they could <qex>set out</qex>, in case of great necessity, thirty men-of-war.</q> <rj><qau>Addison.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sd>(e)</sd> <cd>To show; to display; to recommend; to set off.</cd><br/
[<source>1913 Webster</source>]</p>

<p><q>I could <qex>set out</qex> that best side of Luther.</q> <rj><qau>Atterbury.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sd>(f)</sd> <cd>To show; to prove.</cd> <mark>[R.]</mark> <ldquo/Those very reasons <xex>set out</xex> how heinous his sin was.<rdquo/ <au>Atterbury.</au> <sd>(g)</sd> <fld>(Law)</fld> <cd>To recite; to state at large.</cd><-- see under set, v.i.: to begin to act "he set out to prove" --> -- <col><b>To set over</b></col>. <sd>(a)</sd> <cd>To appoint or constitute as supervisor, inspector, ruler, or commander.</cd> <sd>(b)</sd> <cd>To assign; to transfer; to convey.</cd> -- <col><b>To set right</b></col>, <cd>to correct; to put in order.</cd> -- <col><b>To set sail</b></col>. <fld>(Naut.)</fld> <cd>See under <er>Sail</er>, <pos>n.</pos></cd> -- <col><b>To set store by</b></col>, <cd>to consider valuable.</cd> -- <col><b>To set the fashion</b></col>, <cd>to determine what shall be the fashion; to establish the mode.</cd> -- <col><b>To set the teeth on edge</b></col>, <cd>to affect the teeth with a disagreeable sensation, as when acids are brought in contact with them.</cd> -- <col><b>To set the watch</b></col> <fld>(Naut.)</fld>, <cd>to place the starboard or port watch on duty.</cd> -- <col><b>To set to</b></col>, <cd>to attach to; to affix to.</cd> <ldquo/He . . . hath <xex>set to</xex> his seal that God is true.<rdquo/ <au>John iii. 33.</au> -- <col><b>To set up</b></col>. <fld>(a)</fld> <cd>To erect; to raise; to elevate; as, <xex>to set up</xex> a building, or a machine; <xex>to set up</xex> a post, a wall, a pillar.</cd> <sd>(b)</sd> <cd>Hence, to exalt; to put in power</cd>. <ldquo/I will . . . <xex>set up</xex> the throne of David over Israel.<rdquo/ <au>2 Sam. iii. 10.</au> <sd>(c)</sd> <cd>To begin, as a new institution; to institute; to establish; to found; <as>as, <ex>to set up</ex> a manufactory; to <ex>set up</ex> a school</as>.</cd> <sd>(d)</sd> <cd>To enable to commence a new business; <as>as, <ex>to set up</ex> a son in trade</as></cd>. <sd>(e)</sd> <cd>To place in view; <as>as, <ex>to set up</ex> a mark</as></cd>. <sd>(f)</sd> <cd>To raise; to utter loudly; <as>as, <ex>to set up</ex> the voice</as></cd>.<br/
[<source>1913 Webster</source>]</p>

<p><q>I'll <qex>set up</qex> such a note as she shall hear.</q> <rj><qau>Dryden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sd>(g)</sd> <cd>To advance; to propose as truth or for reception; <as>as, <ex>to set up</ex> a new opinion or doctrine</as>.</cd> <au>T. Burnet.</au> <sd>(h)</sd> <cd>To raise from depression, or to a sufficient fortune; <as>as, this good fortune quite <ex>set</ex> him <ex>up</ex></as>.</cd> <sd>(i)</sd> <cd>To intoxicate</cd>. <mark>[Slang]</mark> <sd>(j)</sd> <fld>(Print.)</fld> <cd>To put in type; <as>as, <ex>to set up</ex> copy</as>; to arrange in words, lines, etc., ready for printing; <as>as, <ex>to set up</ex> type</as>.</cd> -- <col><b>To set up the rigging</b></col> <fld>(Naut.)</fld>, <cd>to make it taut by means of tackles.</cd> <au>R. H. Dana, Jr.</au></cs><br/
[<source>1913 Webster</source>]</p>

<p><syn><b>Syn.</b> -- See <er>Put</er>.</syn><br/
[<source>1913 Webster</source>]</p>

<p><ent>Set</ent><br/
<hw>Set</hw> <pr>(s<ecr/t)</pr>, <pos>v. i.</pos> <sn>1.</sn> <def>To pass below the horizon; to go down; to decline; to sink out of sight; to come to an end.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Ere the weary sun <qex>set</qex> in the west.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Thus this century <qex>sets</qex> with little mirth, and the next is likely to arise with more mourning.</q> <rj><qau>Fuller.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To fit music to words.</def> <mark>[Obs.]</mark>  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>To place plants or shoots in the ground; to plant.</def> <ldquo/To sow dry, and <xex>set</xex> wet.<rdquo/  <rj><au>Old Proverb.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>To be fixed for growth; to strike root; to begin to germinate or form; <as>as, cuttings <ex>set</ex> well; the fruit has <ex>set</ex> well (<it>i. e.</it>, not blasted in the blossom)</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <def>To become fixed or rigid; to be fastened.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>A gathering and serring of the spirits together to resist, maketh the teeth to <qex>set</qex> hard one against another.</q> <rj><qau>Bacon.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>6.</sn> <def>To congeal; to concrete; to solidify; -- of cements, glues, gels, concrete, substances polymerizing into plastics, etc.</def><br/
[<source>1913 Webster</source> <source>+PJC</source>]</p>

<p><q>That fluid substance in a few minutes begins to <qex>set</qex>.</q> <rj><qau>Boyle.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>7.</sn> <def>To have a certain direction in motion; to flow; to move on; to tend; <as>as, the current <ex>sets</ex> to the north; the tide <ex>sets</ex> to the windward</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>8.</sn> <def>To begin to move; to go out or forth; to start; -- now followed by <xex>out</xex>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The king is <qex>set</qex> from London.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>9.</sn> <def>To indicate the position of game; -- said of a dog; <as>as, the dog <ex>sets</ex> well</as>; also, to hunt game by the aid of a setter.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>10.</sn> <def>To apply one's self; to undertake earnestly; -- now followed by <xex>out</xex>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>If he <qex>sets</qex> industriously and sincerely to perform the commands of Christ, he can have no ground of doubting but it shall prove successful to him.</q> <rj><qau>Hammond.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>11.</sn> <def>To fit or suit one; to sit; <as>as, the coat <ex>sets</ex> well</as>.</def> <note>[Colloquially used, but improperly, for <ex>sit</ex>.]</note><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ The use of the verb <xex>set</xex> for <xex>sit</xex> in such expressions as, the hen is <xex>setting</xex> on thirteen eggs; a <xex>setting</xex> hen, etc., although colloquially common, and sometimes tolerated in serious writing, is not to be approved.</note><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>To set about</b></col>, <cd>to commence; to begin.</cd> -- <col><b>To set forward</b></col>, <cd>to move or march; to begin to march; to advance.</cd> -- <col><b>To set forth</b></col>, <cd>to begin a journey.</cd> -- <col><b>To set in</b></col>. <sd>(a)</sd> <cd>To begin; to enter upon a particular state; <as>as, winter <ex>set in</ex> early</as>.</cd> <sd>(b)</sd> <cd>To settle one's self; to become established</cd>. <ldquo/When the weather was <xex>set in</xex> to be very bad.<rdquo/ <au>Addison.</au> <sd>(c)</sd> <cd>To flow toward the shore; -- said of the tide.</cd> -- <col><b>To set off</b></col>.  <sd>(a)</sd> <cd>To enter upon a journey; to start.</cd> <sd>(b)</sd> <fld>(Typog.)</fld> <cd>To deface or soil the next sheet; -- said of the ink on a freshly printed sheet, when another sheet comes in contact with it before it has had time to dry.</cd> -- <mcol><col><b>To set on</b></col> <it>or</it> <col><b>To set upon</b></col></mcol>. <sd>(a)</sd> <cd>To begin, as a journey or enterprise; to set about.</cd><br/
[<source>1913 Webster</source>]</p>

<p><q>He that would seriously <qex>set upon</qex> the search of truth.</q> <rj><qau>Locke.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sd>(b)</sd> <cd>To assault; to make an attack.</cd>  <rj><au>Bacon.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Cassio hath here been <qex>set on</qex> in the dark.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p>-- <col><b>To set out</b></col>, <cd>to begin a journey or course; <as>as, <ex>to set out</ex> for London, or from London; <ex>to set out</ex> in business;<ex>to set out</ex> in life or the world</as>.</cd> -- <col><b>To set to</b></col>, <cd>to apply one's self to.</cd> -- <col><b>To set up</b></col>. <sd>(a)</sd> <cd>To begin business or a scheme of life; <as>as, <ex>to set up</ex> in trade; <ex>to set up</ex> for one's self</as>.</cd> <sd>(b)</sd> <cd>To profess openly; to make pretensions</cd>.<br/
[<source>1913 Webster</source>]</p>

<p><q>Those men who <qex>set up</qex> for mortality without regard to religion, are generally but virtuous in part.</q> <rj><qau>Swift.</qau></rj>
</cs><br/
[<source>1913 Webster</source>]</p>

<p><-- p. 1318 --></p>

<p><ent>Set</ent><br/
<hw>Set</hw> <pr>(s<ecr/t)</pr>, <pos>a.</pos> <sn>1.</sn> <def>Fixed in position; immovable; rigid; <as>as, a <ex>set</ex> line; a <ex>set</ex> countenance</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Firm; unchanging; obstinate; <as>as, <ex>set</ex> opinions or prejudices</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>Regular; uniform; formal; <as>as, a <ex>set</ex> discourse; a <ex>set</ex> battle</as>.</def> <ldquo/The <xex>set</xex> phrase of peace.<rdquo/  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>Established; prescribed; <as>as, <ex>set</ex> forms of prayer</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <def>Adjusted; arranged; formed; adapted.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Set hammer</b></col>. <sd>(a)</sd> <cd>A hammer the head of which is not tightly fastened upon the handle, but may be reversed</cd>. <au>Knight.</au> <sd>(b)</sd> <cd>A hammer with a concave face which forms a die for shaping anything, as the end of a bolt, rivet, etc.</cd> -- <col><b>Set line</b></col>, <cd>a line to which a number of baited hooks are attached, and which, supported by floats and properly secured, may be left unguarded during the absence of the fisherman.</cd> -- <col><b>Set nut</b></col>, <cd>a jam nut or lock nut. See under <er>Nut</er>.</cd> -- <col><b>Set screw</b></col> <fld>(Mach.)</fld>, <cd>a screw, sometimes cupped or printed at one end, and screwed through one part, as of a machine, tightly upon another part, to prevent the one from slipping upon the other.</cd> -- <col><b>Set speech</b></col>, <cd>a speech carefully prepared before it is delivered in public; a formal or methodical speech.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Set</ent><br/
<hw>Set</hw>, <pos>n.</pos> <sn>1.</sn> <def>The act of setting, as of the sun or other heavenly body; descent; hence, the close; termination.</def> <ldquo/Locking at the <xex>set</xex> of day.<rdquo/  <rj><au>Tennyson.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>The weary sun hath made a golden <qex>set</qex>.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>That which is set, placed, or fixed.</def> Specifically: -- <sd>(a)</sd> <def>A young plant for growth; <as>as, a <ex>set</ex> of white thorn</as>.</def> <sd>(b)</sd> <def>That which is staked; a wager; a venture; a stake; hence, a game at venture.</def> <mark>[Obs. or R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>We will in France, by God's grace, play a <qex>set</qex><br/
Shall strike his father's crown into the hazard.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>That was but civil war, an equal <qex>set</qex>.</q> <rj><qau>Dryden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sd>(c)</sd> <fld>(Mech.)</fld> <def>Permanent change of figure in consequence of excessive strain, as from compression, tension, bending, twisting, etc.; <as>as, the <ex>set</ex> of a spring</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sd>(d)</sd> <def>A kind of punch used for bending, indenting, or giving shape to, metal; <as>as, a saw <ex>set</ex></as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sd>(e)</sd> <fld>(Pile Driving)</fld> <def>A piece placed temporarily upon the head of a pile when the latter cannot be reached by the weight, or hammer, except by means of such an intervening piece.</def> <altsp>[Often incorrectly written <asp>sett</asp>.]</altsp><br/
[<source>1913 Webster</source>]</p>

<p><sd>(f)</sd> <fld>(Carp.)</fld> <def>A short steel spike used for driving the head of a nail below the surface.  Called also <altname>nail set</altname>.</def><br/
[<source>1913 Webster</source> <source>+PJC</source>]</p>

<p><sn>3.</sn> <ety>[Perhaps due to confusion with <ets>sect</ets>, <ets>sept</ets>.]</ety> <def>A number of things of the same kind, ordinarily used or classed together; a collection of articles which naturally complement each other, and usually go together; an assortment; a suit; <as>as, a <ex>set</ex> of chairs, of china, of surgical or mathematical instruments, of books, etc.</as></def> <altsp>[In this sense, sometimes incorrectly written <asp>sett</asp>.]</altsp><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>A number of persons associated by custom, office, common opinion, quality, or the like; a division; a group; a clique.</def> <ldquo/Others of our <xex>set</xex>.<rdquo/  <rj><au>Tennyson.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>This falls into different divisions, or <qex>sets</qex>, of nations connected under particular religions.</q> <rj><qau>R. P. Ward.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <def>Direction or course; <as>as, the <ex>set</ex> of the wind, or of a current</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>6.</sn> <def>In dancing, the number of persons necessary to execute a quadrille; also, the series of figures or movements executed.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>7.</sn> <def>The deflection of a tooth, or of the teeth, of a saw, which causes the the saw to cut a kerf, or make an opening, wider than the blade.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>8.</sn> <sd>(a)</sd> <def>A young oyster when first attached.</def> <sd>(b)</sd> <def>Collectively, the crop of young oysters in any locality.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>9.</sn> <fld>(Tennis)</fld> <def>A series of as many games as may be necessary to enable one side to win six. If at the end of the tenth game the score is a tie, the set is usually called a <xex>deuce set</xex>, and decided by an application of the rules for playing off deuce in a game. See <er>Deuce</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>10.</sn> <fld>(Type Founding)</fld> <def>That dimension of the body of a type called by printers the <altname>width</altname>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>11.</sn> <fld>(Textiles)</fld> <def>Any of various standards of measurement of the fineness of cloth; specif., the number of reeds in one inch and the number of threads in each reed.  The exact meaning varies according to the location where it is used.  Sometimes written <asp>sett</asp>.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><sn>12.</sn>  <def>A stone, commonly of granite, shaped like a short brick and usually somewhat larger than one, used for street paving. Commonly written <asp>sett</asp>.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><sn>13.</sn>  <def>Camber of a curved roofing tile.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><sn>14.</sn>  <def>The manner, state, or quality of setting or fitting; fit; <as>as, the <ex>set</ex> of a coat</as>.</def> <mark>[Colloq.]</mark><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><sn>15.</sn>  <def>Any collection or group of objects considered together.</def><br/
[<source>PJC</source>]</p>

<p><cs><col><b>Dead set</b></col>. <sd>(a)</sd> <cd>The act of a setter dog when it discovers the game, and remains intently fixed in pointing it out</cd>. <sd>(b)</sd> <cd>A fixed or stationary condition arising from obstacle or hindrance; a deadlock; as, to be at a <xex>dead set</xex>.</cd> <sd>(c)</sd> <cd>A concerted scheme to defraud by gaming; a determined onset.</cd> -- <col><b>To make a dead set</b></col>, <cd>to make a determined onset, literally or figuratively.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><syn><b>Syn.</b> -- Collection; series; group. See <er>Pair</er>.</syn><br/
[<source>1913 Webster</source>]</p>

<p><ent>Set</ent><br/
<hw>Set</hw> <pr>(?)</pr>, <pos>prop. n.</pos> <fld>(Egyptian Mythology)</fld> <def>An evil beast-headed god with high square ears and a long snout; his was the brother and murderer of <persfn>Osiris</persfn>.  Called also <altname>Seth</altname></def><br/
[<source>WordNet 1.6</source>]</p>

<p><ent>Seta</ent><br/
||<hw>Se"ta</hw> <pr>(s<emac/"t<adot/)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Setae</plw> <pr>(s<emac/"t<emac/)</pr>.</plu> <ety>[L. <ets>seta</ets>, <ets>saeta</ets>, a bristle.]</ety> <sn>1.</sn> <fld>(Biol.)</fld> <def>Any slender, more or less rigid, bristlelike organ or part; as the hairs of a caterpillar, the slender spines of a crustacean, the hairlike processes of a protozoan, the bristles or stiff hairs on the leaves of some plants, or the pedicel of the capsule of a moss.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Zool.)</fld> <sd>(a)</sd> <def>One of the movable chitinous spines or hooks of an annelid.  They usually arise in clusters from muscular capsules, and are used in locomotion and for defense.  They are very diverse in form.</def> <sd>(b)</sd> <def>One of the spinelike feathers at the base of the bill of certain birds.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Setaceous</ent><br/
<hw>Se*ta"ceous</hw> <pr>(s<esl/*t<amac/"sh<ucr/s)</pr>, <pos>a.</pos> <ety>[L. <ets>seta</ets> a bristle: cf. F. <ets>s<eacute/tac<eacute/</ets>.]</ety> <sn>1.</sn> <def>Set with, or consisting of, bristles; bristly; <as>as, a stiff, <ex>setaceous</ex> tail</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Bristelike in form or texture; <as>as, a <ex>setaceous</ex> feather; a <ex>setaceous</ex> leaf.</as></def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Setback</ent><br/
<hw>Set"back`</hw> <pr>(s<ecr/t"b<acr/k`)</pr>, <pos>n.</pos> <sn>1.</sn> <fld>(Arch.)</fld> <def>Offset, <pos>n.</pos>, 4.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A backset; a countercurrent; an eddy.</def> <mark>[U. S.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>A reversal of progress in an endeavor; a reverse; a backset; a check; a repulse; a relapse.</def> <mark>[Colloq. U.S.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Setbolt</ent><br/
<hw>Set"bolt`</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Shipbuilding)</fld> <sn>1.</sn> <def>An iron pin, or bolt, for fitting planks closely together.</def>  <rj><au>Craig.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A bolt used for forcing another bolt out of its hole.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Set chisel</ent><br/
<hw>Set chisel</hw>. <fld>(Mech.)</fld> <def>A kind of chisel or punch, variously shaped, with a broad flat end, used for stripping off rivet heads, etc.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Setdown</ent><br/
<hw>Set"down`</hw> <pr>(s<ecr/t"doun`)</pr>, <pos>n.</pos> <def>The humbling of a person by act or words, especially by a disparaging remark, a retort or a reproof; the retort or the reproof which has such effect.  Also called <altname>put-down</altname>.</def><br/
[<source>1913 Webster</source> <source>+PJC</source>]</p>

<p><ent>Setee</ent><br/
<hw>Set*ee"</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Naut.)</fld> <def>See 2d <er>Settee</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Seten</ent><br/
<hw>Set"en</hw> <pr>(?)</pr>, <pos>obs. imp. pl.</pos> <mord>of <er>Sit</er></mord>. <def>Sat.</def>  <rj><au>Chaucer.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Setewale</ent><br/
<hw>Set"e*wale</hw> <pr>(?)</pr>, <pos>n.</pos> <def>See <er>Cetewale</er>.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Set-fair</ent><br/
<hw>Set"-fair`</hw> <pr>(?)</pr>, <pos>n.</pos> <def>In plastering, a particularly good troweled surface.</def>  <rj><au>Knight.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Setfoil</ent><br/
<hw>Set"foil`</hw> <pr>(?)</pr>, <pos>n.</pos> <def>See <er>Septfoil</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Seth</ent><br/
<hw>Seth</hw> <pr>(?)</pr>, <pos>prop. n.</pos> <fld>(Egyptian Mythology)</fld> <def>An evil beast-headed god with high square ears and a long snout; his was the brother and murderer of Osiris.  Called also <altname>Set</altname></def><br/
[<source>WordNet 1.6</source>]</p>

<p><ent>Sethen</ent><br/
<hw>Seth"en</hw> <pr>(?)</pr>, <pos>adv. & conj.</pos> <def>See <er>Since</er>.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sethic</ent><br/
<hw>Seth"ic</hw> <pr>(?)</pr>, <pos>a.</pos> <def>See <er>Sothic</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Setiferous</ent><br/
<hw>Se*tif"er*ous</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>seta</ets> a bristle + <ets>-ferous</ets>.]</ety> <def>Producing, or having one or more, bristles.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Setiform</ent><br/
<hw>Se"ti*form</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[<ets>Seta</ets> + <ets>-form</ets>: cf. F. <ets>s<eacute/tiforme</ets>.]</ety> <def>Having the form or structure of setae.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Setiger</ent><br/
<hw>Se"ti*ger</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[NL. See <er>Setigerous</er>.]</ety> <fld>(Zool.)</fld> <def>An annelid having setae; a chaetopod.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Setigerous</ent><br/
<hw>Se*tig"er*ous</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[<ets>Seta</ets> + <ets>-gerous</ets>.]</ety> <def>Covered with bristles; having or bearing a seta or setae; setiferous; <as>as, <ex>setigerous</ex> glands; a <ex>setigerous</ex> segment of an annelid</as>; specifically <fld>(Bot.)</fld>, tipped with a bristle.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Setim</ent><br/
<hw>Se"tim</hw> <pr>(?)</pr>, <pos>n.</pos> <def>See <er>Shittim</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>set-in sleeve</ent><br/
<hw>set"-in sleeve`</hw> <pr>(s<ecr/t"<icr/n sl<emac/v`)</pr>, <pos>n.</pos> <def>A sleeve joined to the body of a garment by a seam starting at the edge of the shoulder and continuing around the armhole.  Contrasted to a <er>raglan sleeve</er>.</def><br/
[<source>PJC</source>]</p>

<p><ent>Setiparous</ent><br/
<hw>Se*tip"a*rous</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[<ets>Seta</ets> + L. <ets>papere</ets> to produce.]</ety> <fld>(Zool.)</fld> <def>Producing setae; -- said of the organs from which the setae of annelids arise.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Setireme</ent><br/
<hw>Se"ti*reme</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[<ets>Seta</ets> + L. <ets>remus</ets> an oar.]</ety> <fld>(Zool.)</fld> <def>A swimming leg (of an insect) having a fringe of hairs on the margin.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Setness</ent><br/
<hw>Set"ness</hw> <pr>(?)</pr>, <pos>n.</pos> <def>The quality or state of being set; formality; obstinacy.</def> <ldquo/The starched <xex>setness</xex> of a sententious writer.<rdquo/  <rj><au>R. Masters.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Set-off</ent><br/
<hw>Set"-off`</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[<ets>Set</ets> + <ets>off</ets>.]</ety> <sn>1.</sn> <def>That which is set off against another thing; an offset.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>I do not contemplate such a heroine as a <qex>set-off</qex> to the many sins imputed to me as committed against woman.</q> <rj><qau>D. Jerrold.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>That which is used to improve the appearance of anything; a decoration; an ornament.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <fld>(Law)</fld> <def>A counterclaim; a cross debt or demand; a distinct claim filed or set up by the defendant against the plaintiff's demand.</def><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ <xex>Set-off</xex> differs from <xex>recoupment</xex>, as the latter generally grows out of the same matter or contract with the plaintiff's claim, while the former grows out of distinct matter, and does not of itself deny the justice of the plaintiff's demand. <xex>Offset</xex> is sometimes improperly used for the legal term <xex>set-off</xex>. See <er>Recoupment</er>.</note><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <fld>(Arch.)</fld> <def>Same as <er>Offset</er>, <pos>n.</pos>, 4.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <fld>(Print.)</fld> <def>See <er>Offset</er>, 7.</def><br/
[<source>1913 Webster</source>]</p>

<p><syn><b>Syn.</b> -- <er>Set-off</er>, <er>Offset</er>.</syn>  -- <usage><xex>Offset</xex> originally denoted that which branches off or projects, as a shoot from a tree, but the term has long been used in America in the sense of <xex>set-off</xex>. This use is beginning to obtain in England; though Macaulay uses <xex>set-off</xex>, and so, perhaps, do a majority of English writers.</usage><br/
[<source>1913 Webster</source>]</p>

<p><ent>Seton</ent><br/
<hw>Se"ton</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F. <ets>s<eacute/ton</ets> (cf. It. <ets>setone</ets>), from L. <ets>seta</ets> a thick, stiff hair, a bristle.]</ety> <fld>(Med. & Far.)</fld> <def>A few silk threads or horsehairs, or a strip of linen or the like, introduced beneath the skin by a knife or needle, so as to form an issue; also, the issue so formed.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Setous</ent><br/
<ent>Setose</ent><br/
<mhw>{ <hw>Se*tose"</hw> <pr>(?)</pr>, <hw>Se"tous</hw> <pr>(?)</pr>, }</mhw> <pos>a.</pos> <ety>[L. <ets>setosus</ets>, <ets>saetosus</ets>, from <ets>seta</ets>, <ets>saeta</ets>, bristle: cf. F. <ets>s<eacute/teux</ets>.]</ety> <def>Thickly set with bristles or bristly hairs.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Setout</ent><br/
<hw>Set"out`</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A display, as of plate, equipage, etc.; that which is displayed.</def> <mark>[Coloq.]</mark>  <rj><au>Dickens.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Set-stitched</ent><br/
<hw>Set"-stitched`</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Stitched according to a formal pattern.</def> <ldquo/An old <xex>set-stiched</xex> chair, valanced, and fringed with party-colored worsted bobs.<rdquo/  <rj><au>Sterne.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sett</ent><br/
<hw>Sett</hw> <pr>(?)</pr>, <pos>n.</pos> <def>See <er>Set</er>, <pos>n.</pos>, 2 <sd>(e)</sd> and 3.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Settee</ent><br/
<hw>Set*tee"</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[From <er>Set</er>; cf. <er>Settle</er> a seat.]</ety> <def>A long seat with a back, -- made to accommodate several persons at once.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Settee</ent><br/
<hw>Set*tee"</hw>, <pos>n.</pos> <ety>[F. <ets>sc<eacute/tie</ets>, <ets>scitie</ets>.]</ety> <fld>(Naut.)</fld> <def>A vessel with a very long, sharp prow, carrying two or three masts with lateen sails, -- used in the Mediterranean.</def> <altsp>[Written also <asp>setee</asp>.]</altsp><br/
[<source>1913 Webster</source>]</p>

<p><ent>Setter</ent><br/
<hw>Set"ter</hw> <pr>(?)</pr>, <pos>n.</pos> <sn>1.</sn> <def>One who, or that which, sets; -- used mostly in composition with a noun, as type<xex>setter</xex>; or in combination with an adverb, as a <xex>setter</xex> on (or inciter), a <xex>setter</xex> up, a <xex>setter</xex> forth.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Zool.)</fld> <def>A hunting dog of a special breed originally derived from a cross between the spaniel and the pointer. Modern setters are usually trained to indicate the position of game birds by standing in a fixed position, but originally they indicated it by sitting or crouching.</def><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ There are several distinct varieties of setters; as, the <xex>Irish</xex>, or <xex>red</xex>, <xex>setter</xex>; the <xex>Gordon setter</xex>, which is usually red or tan varied with black; and the <xex>English setter</xex>, which is variously colored, but usually white and tawny red, with or without black.</note><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>One who hunts victims for sharpers.</def>  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>One who adapts words to music in composition.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <def>An adornment; a decoration; -- with <xex>off</xex>.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>They come as . . . <qex>setters</qex> off of thy graces.</q> <rj><qau>Whitlock.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>6.</sn> <fld>(Pottery)</fld> <def>A shallow seggar for porcelain.</def>  <rj><au>Ure.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Setter</ent><br/
<hw>Set"ter</hw>, <pos>v. t.</pos> <def>To cut the dewlap (of a cow or an ox), and to insert a seton, so as to cause an issue.</def> <mark>[Prov. Eng.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Setterwort</ent><br/
<hw>Set"ter*wort`</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Bot.)</fld> <def>The bear's-foot (<spn>Helleborus f<oe/tidus</spn>); -- so called because the root was used in <xex>settering</xex>, or inserting setons into the dewlaps of cattle. Called also <altname>pegroots</altname>.</def>  <rj><au>Dr. Prior.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Setting</ent><br/
<hw>Set"ting</hw> <pr>(?)</pr>, <pos>n.</pos> <sn>1.</sn> <def>The act of one who, or that which, sets; <as>as, the <ex>setting</ex> of type, or of gems; the <ex>setting</ex> of the sun; the <ex>setting</ex> (hardening) of moist plaster of Paris; the <ex>setting</ex> (set) of a current.</as></def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>The act of marking the position of game, as a setter does; also, hunting with a setter.</def>  <rj><au>Boyle.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>Something set in, or inserted.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Thou shalt set in it <qex>settings</qex> of stones.</q> <rj><qau>Ex. xxviii. 17.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>That in which something, as a gem, is set; <as>as, the gold <ex>setting</ex> of a jeweled pin</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <def>the time, place, and circumstances in which an event (real or fictional) occurs; <as>as, the <ex>setting</ex> of a novel</as>.</def><br/
[<source>PJC</source>]</p>

<p><cs><col><b>Setting coat</b></col> <fld>(Arch.)</fld>, <cd>the finishing or last coat of plastering on walls or ceilings.</cd> -- <col><b>Setting dog</b></col>, <cd>a setter. See <er>Setter</er>, <pos>n.</pos>, 2.</cd> -- <col><b>Setting pole</b></col>, <cd>a pole, often iron-pointed, used for pushing boats along in shallow water.</cd> -- <col><b>Setting rule</b></col>. <fld>(Print.)</fld> <cd>A composing rule.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Setting-up exercise</ent><br/
<hw>Set`ting-up" ex"er*cise</hw>. <def>Any one of a series of gymnastic exercises used, as in drilling recruits, for the purpose of giving an erect carriage, supple muscles, and an easy control of the limbs.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Settle</ent><br/
<hw>Set"tle</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[OE. <ets>setel</ets>, <ets>setil</ets>, a seat, AS. <ets>setl</ets>: akin to OHG. <ets>sezzal</ets>, G. <ets>sessel</ets>, Goth. <ets>sitls</ets>, and E. <ets>sit</ets>. <root/154. See <er>Sit</er>.]</ety> <sn>1.</sn> <def>A seat of any kind.</def> <mark>[Obs.]</mark> <ldquo/Upon the <xex>settle</xex> of his majesty<rdquo/  <rj><au>Hampole.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A bench; especially, a bench with a high back.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>A place made lower than the rest; a wide step or platform lower than some other part.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>And from the bottom upon the ground, even to the lower <qex>settle</qex>, shall be two cubits, and the breadth one cubit.</q> <rj><qau>Ezek. xliii. 14.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Settle bed</b></col>, <cd>a bed convertible into a seat.</cd> <mark>[Eng.]</mark></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Settle</ent><br/
<hw>Set"tle</hw>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Settled</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Settling</conjf> <pr>(?)</pr>.]</vmorph> <ety>[OE. <ets>setlen</ets>, AS. <ets>setlan</ets>. <root/154. See <er>Settle</er>, <pos>n.</pos> In senses 7, 8, and 9 perhaps confused with OE. <ets>sahtlen</ets> to reconcile, AS. <ets>sahtlian</ets>, fr. <ets>saht</ets> reconciliation, <ets>sacon</ets> to contend, dispute.  Cf. <er>Sake</er>.]</ety> <sn>1.</sn> <def>To place in a fixed or permanent condition; to make firm, steady, or stable; to establish; to fix; esp., to establish in life; to fix in business, in a home, or the like.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>And he <qex>settled</qex> his countenance steadfastly upon him, until he was ashamed.</q> <rj><qau>2 Kings viii. 11. (Rev. Ver.) </qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>The father thought the time drew on<br/
Of <qex>setting</qex> in the world his only son.</q> <rj><qau>Dryden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To establish in the pastoral office; to ordain or install as pastor or rector of a church, society, or parish; <as>as, to <ex>settle</ex> a minister</as>.</def> <mark>[U. S.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>To cause to be no longer in a disturbed condition; to render quiet; to still; to calm; to compose.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>God <qex>settled</qex> then the huge whale-bearing lake.</q> <rj><qau>Chapman.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Hoping that sleep might <qex>settle</qex> his brains.</q> <rj><qau>Bunyan.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>To clear of dregs and impurities by causing them to sink; to render pure or clear; -- said of a liquid; <as>as, to <ex>settle</ex> coffee, or the grounds of coffee</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <def>To restore or bring to a smooth, dry, or passable condition; -- said of the ground, of roads, and the like; <as>as, clear weather <ex>settles</ex> the roads</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>6.</sn> <def>To cause to sink; to lower; to depress; hence, also, to render close or compact; <as>as, to <ex>settle</ex> the contents of a barrel or bag by shaking it</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>7.</sn> <def>To determine, as something which is exposed to doubt or question; to free from unscertainty or wavering; to make sure, firm, or constant; to establish; to compose; to quiet; <as>as, to <ex>settle</ex> the mind when agitated; to <ex>settle</ex> questions of law; to <ex>settle</ex> the succession to a throne; to <ex>settle</ex> an allowance</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>It will <qex>settle</qex> the wavering, and confirm the doubtful.</q> <rj><qau>Swift.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>8.</sn> <def>To adjust, as something in discussion; to make up; to compose; to pacify; <as>as, to <ex>settle</ex> a quarrel</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>9.</sn> <def>To adjust, as accounts; to liquidate; to balance; <as>as, to <ex>settle</ex> an account</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>10.</sn> <def>Hence, to pay; <as>as, to <ex>settle</ex> a bill</as>.</def> <mark>[Colloq.]</mark>  <rj><au>Abbott.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>11.</sn> <def>To plant with inhabitants; to colonize; to people; <as>as, the French first <ex>settled</ex> Canada; the Puritans <ex>settled</ex> New England; Plymouth was <ex>settled</ex> in 1620.</as></def><br/
[<source>1913 Webster</source>]</p>

<p><cs><mcol><col><b>To settle on</b></col> <it>or</it> <col><b>To settle upon</b></col></mcol>, <sd>(a)</sd> <cd>to confer upon by permanent grant; to assure to.</cd> <ldquo/I . . . have <xex>settled upon</xex> him a good annuity.<rdquo/ <au>Addison.</au> <sd>(b)</sd> <cd>to choose; to decide on; -- sometimes with the implication that the choice is not ideal, but the best available.</cd> -- <col><b>To settle the land</b></col> <fld>(Naut.)</fld>, <cd>to cause it to sink, or appear lower, by receding from it.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><syn><b>Syn.</b> -- To fix; establish; regulate; arrange; compose; adjust; determine; decide.</syn><br/
[<source>1913 Webster</source>]</p>

<p><ent>Settle</ent><br/
<hw>Set"tle</hw>, <pos>v. i.</pos> <sn>1.</sn> <def>To become fixed or permanent; to become stationary; to establish one's self or itself; to assume a lasting form, condition, direction, or the like, in place of a temporary or changing state.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The wind came about and <qex>settled</qex> in the west.</q> <rj><qau>Bacon.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Chyle . . . runs through all the intermediate colors until it <qex>settles</qex> in an intense red.</q> <rj><qau>Arbuthnot.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To fix one's residence; to establish a dwelling place or home; <as>as, the Saxons who <ex>settled</ex> in Britain</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>To enter into the married state, or the state of a householder.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>As people marry now and <qex>settle</qex>.</q> <rj><qau>Prior.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>To be established in an employment or profession; <as>as, to <ex>settle</ex> in the practice of law</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <def>To become firm, dry, and hard, as the ground after the effects of rain or frost have disappeared; <as>as, the roads <ex>settled</ex> late in the spring</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>6.</sn> <def>To become clear after being turbid or obscure; to clarify by depositing matter held in suspension; <as>as, the weather <ex>settled</ex>; wine <ex>settles</ex> by standing</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>A government, on such occasions, is always thick before it <qex>settles</qex>.</q> <rj><qau>Addison.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>7.</sn> <def>To sink to the bottom; to fall to the bottom, as dregs of a liquid, or the sediment of a reserveir.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>8.</sn> <def>To sink gradually to a lower level; to subside, as the foundation of a house, etc.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>9.</sn> <def>To become calm; to cease from agitation.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Till the fury of his highness <qex>settle</qex>,<br/
Come not before him.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>10.</sn> <def>To adjust differences or accounts; to come to an agreement; <as>as, he has <ex>settled</ex> with his creditors</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>11.</sn> <def>To make a jointure for a wife.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>He sighs with most success that <qex>settles</qex> well.</q> <rj><qau>Garth.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Settledness</ent><br/
<hw>Set"tled*ness</hw> <pr>(?)</pr>, <pos>n.</pos> <def>The quality or state of being settled; confirmed state.</def> <mark>[R.]</mark>  <rj><au>Bp. Hall.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Settlement</ent><br/
<hw>Set"tle*ment</hw> <pr>(?)</pr>, <pos>n.</pos> <sn>1.</sn> <def>The act of setting, or the state of being settled.</def> Specifically: --<br/
[<source>1913 Webster</source>]</p>

<p><sd>(a)</sd> <def>Establishment in life, in business, condition, etc.; ordination or installation as pastor.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Every man living has a design in his head upon wealth power, or <qex>settlement</qex> in the world.</q> <rj><qau>L'Estrange.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sd>(b)</sd> <def>The act of peopling, or state of being peopled; act of planting, as a colony; colonization; occupation by settlers; as, the <xex>settlement</xex> of a new country.</def><br/
[<source>1913 Webster</source>]</p>

<p><-- p. 1319 --></p>

<p><sd>(c)</sd> <def>The act or process of adjusting or determining; composure of doubts or differences; pacification; liquidation of accounts; arrangement; adjustment; <as>as, <ex>settlement</ex> of a controversy, of accounts, etc.</as></def><br/
[<source>1913 Webster</source>]</p>

<p><sd>(d)</sd> <def>Bestowal, or giving possession, under legal sanction; the act of giving or conferring anything in a formal and permanent manner.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>My flocks, my fields, my woods, my pastures take,<br/
With <qex>settlement</qex> as good as law can make.</q> <rj><qau>Dryden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sd>(e)</sd> <fld>(Law)</fld> <def>A disposition of property for the benefit of some person or persons, usually through the medium of trustees, and for the benefit of a wife, children, or other relatives; jointure granted to a wife, or the act of granting it.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>That which settles, or is settled, established, or fixed.</def> Specifically: --<br/
[<source>1913 Webster</source>]</p>

<p><sd>(a)</sd> <def>Matter that subsides; settlings; sediment; lees; dregs.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>Fuller's earth left a thick <qex>settlement</qex>.</q> <rj><qau>Mortimer.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sd>(b)</sd> <def>A colony newly established; a place or region newly settled; <as>as, <ex>settlement</ex> in the West</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sd>(c)</sd> <def>That which is bestowed formally and permanently; the sum secured to a person; especially, a jointure made to a woman at her marriage; also, in the United States, a sum of money or other property formerly granted to a pastor in additional to his salary.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <fld>(Arch.)</fld> <sd>(a)</sd> <def>The gradual sinking of a building, whether by the yielding of the ground under the foundation, or by the compression of the joints or the material.</def> <sd>(b)</sd> <pluf>pl.</pluf> <def>Fractures or dislocations caused by settlement.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <fld>(Law)</fld> <def>A settled place of abode; residence; a right growing out of residence; legal residence or establishment of a person in a particular parish or town, which entitles him to maintenance if a pauper, and subjects the parish or town to his support.</def>  <rj><au>Blackstone. Bouvier.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Act of settlement</b></col> <fld>(Eng. Hist.)</fld>, <cd>the statute of 12 and 13 William III, by which the crown was limited to the present reigning house (the house of Hanover).</cd>  <rj><au>Blackstone.</au></rj></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Settler</ent><br/
<hw>Set"tler</hw> <pr>(?)</pr>, <pos>n.</pos> <sn>1.</sn> <def>One who settles, becomes fixed, established, etc.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Especially, one who establishes himself in a new region or a colony; a colonist; a planter; <as>as, the first <ex>settlers</ex> of New England</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>That which settles or finishes; hence, a blow, etc., which settles or decides a contest.</def> <mark>[Colloq.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>A vessel, as a tub, in which something, as pulverized ore suspended in a liquid, is allowed to settle.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Settling</ent><br/
<hw>Set"tling</hw> <pr>(?)</pr>, <pos>n.</pos> <sn>1.</sn> <def>The act of one who, or that which, settles; the act of establishing one's self, of colonizing, subsiding, adjusting, etc.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <pluf>pl.</pluf> <def>That which settles at the bottom of a liquid; lees; dregs; sediment.</def>  <rj><au>Milton.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Settling day</b></col>, <cd>a day for settling accounts, as in the stock market.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Set-to</ent><br/
<hw>Set"-to`</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A contest in boxing, in an argument, or the like.</def> <mark>[Colloq.]</mark>  <rj><au>Halliwell.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Setula</ent><br/
||<hw>Set"u*la</hw> <pr>(?)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Setulae</plw> <pr>(#)</pr>.</plu> <ety>[L. <ets>setula</ets>, <ets>saetula</ets>, dim. of <ets>seta</ets>, <ets>saeta</ets>, bristle.]</ety> <def>A small, short hair or bristle; a small seta.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Setule</ent><br/
<hw>Set"ule</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[See <er>Setula</er>.]</ety> <def>A setula.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Setulose</ent><br/
<hw>Set"u*lose`</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Having small bristles or setae.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Setwall</ent><br/
<hw>Set"wall`</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[CF. <er>Cetewale</er>.]</ety> <fld>(Bot.)</fld> <def>A plant formerly valued for its restorative qualities (<spn>Valeriana officinalis</spn>, or <spn>Valeriana Pyrenaica</spn>).</def> <mark>[Obs.]</mark> <altsp>[Written also <asp>setwal</asp>.]</altsp>  <rj><au>Chaucer.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Seven</ent><br/
<hw>Sev"en</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[OE. <ets>seven</ets>, <ets>seoven</ets>, <ets>seofen</ets>, AS. <ets>seofon</ets>, <ets>seofan</ets>, <ets>seofen</ets>; akin to D. <ets>zeven</ets>, OS., Goth., & OHG. <ets>sibun</ets>, G. <ets>sieben</ets>, Icel. <ets>sjau</ets>, <ets>sj<oum/</ets>, Sw. <ets>sju</ets>, Dan. <ets>syv</ets>, Lith. <ets>septyni</ets>, Russ. <ets>seme</ets>, W. <ets>saith</ets>, Gael. <ets>seachd</ets>, Ir. <ets>seacht</ets>, L. <ets>septem</ets>, Gr. <?/<?/<?/, Skr. <ets>saptan</ets>. <root/305.  Cf. <er>Hebdomad</er>, <er>Heptagon</er>, <er>September</er>.]</ety> <def>One more than six; six and one added; <as>as, <ex>seven</ex> days make one week</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Seven sciences</b></col>. <cd>See the Note under <er>Science</er>, <pos>n.</pos>, 4.</cd> -- <col><b>Seven stars</b></col> <fld>(Astron.)</fld>, <cd>the Pleiades.</cd> -- <col><b>Seven wonders of the world</b></col>. <cd>See under <er>Wonders</er>.</cd> -- <col><b>Seven-year apple</b></col> <fld>(Bot.)</fld>, <cd>a rubiaceous shrub (<spn>Genipa clusiifolia</spn>) growing in the West Indies; also, its edible fruit.</cd> -- <col><b>Seven-year vine</b></col> <fld>(Bot.)</fld>, <cd>a tropical climbing plant (<spn>Ipom<oe/a tuberosa</spn>) related to the morning-glory.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Seven</ent><br/
<hw>Sev"en</hw>, <pos>n.</pos> <sn>1.</sn> <def>The number greater by one than six; seven units or objects.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Of every beast, and bird, and insect small,<br/
Game <qex>sevens</qex> and pairs.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A symbol representing seven units, as 7, or vii.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sevenfold</ent><br/
<hw>Sev"en*fold`</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Repeated seven times; having seven thicknesses; increased to seven times the size or amount.</def> <ldquo/<xex>Sevenfold</xex> rage.<rdquo/  <rj><au>Milton.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sevenfold</ent><br/
<hw>Sev"en*fold`</hw>, <pos>adv.</pos> <def>Seven times as much or as often.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Whosoever slayeth Cain, vengeance shall be taken on him <qex>sevenfold</qex>.</q> <rj><qau>Gen. iv. 15.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sevennight</ent><br/
<hw>Seven"night</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A week; any period of seven consecutive days and nights. See <er>Sennight</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sevenscore</ent><br/
<hw>Sev"en*score`</hw> <pr>(?)</pr>, <pos>n. & a.</pos> <def>Seven times twenty, that is, a hundred and forty.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The old Countess of Desmond . . . lived <qex>sevenscore</qex> years.</q> <rj><qau>Bacon.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Seven-shooter</ent><br/
<hw>Sev"en-shoot`er</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A firearm, esp. a pistol, with seven barrels or chambers for cartridges, or one capable of firing seven shots without reloading.</def> <mark>[Colloq.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Seventeen</ent><br/
<hw>Sev"en*teen`</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[OE. <ets>seventene</ets>, AS.  <ets>seofont<ymac/ne</ets>, i. e., <ets>seven-ten</ets>.  Cf. <er>Seventy</er>.]</ety> <def>One more than sixteen; ten and seven added; <as>as, <ex>seventeen</ex> years</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Seventeen</ent><br/
<hw>Sev"en*teen`</hw>, <pos>n.</pos> <sn>1.</sn> <def>The number greater by one than sixteen; the sum of ten and seven; seventeen units or objects.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A symbol denoting seventeen units, as 17, or xvii.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Seventeenth</ent><br/
<hw>Sev"en*teenth`</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[From <er>Seventeen</er>: cf. AS. <ets>seofonte<oacute/<edh/a</ets>, <ets>seofonteoge<edh/a</ets>.]</ety> <sn>1.</sn> <def>Next in order after the sixteenth; coming after sixteen others.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>In . . . the <qex>seventeenth</qex> day of the month . . . were all the fountains of the great deep broken up.</q> <rj><qau>Gen. vii. 11.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Constituting or being one of seventeen equal parts into which anything is divided.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Seventeenth</ent><br/
<hw>Sev"en*teenth`</hw> <pr>(?)</pr>, <pos>n.</pos> <sn>1.</sn> <def>The next in order after the sixteenth; one coming after sixteen others.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>The quotient of a unit divided by seventeen; one of seventeen equal parts or divisions of one whole.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <fld>(Mus.)</fld> <def>An interval of two octaves and a third.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Seventh</ent><br/
<hw>Sev"enth</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[From <er>Seven</er>: cf. AS. <ets>seofo<edh/a</ets>.]</ety> <sn>1.</sn> <def>Next in order after the sixth;; coming after six others.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>On the <qex>seventh</qex> day, God ended his work which he had made; and he rested on the <qex>seventh</qex> day from all his work which he had made.</q> <rj><qau>Gen. ii. 2.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Constituting or being one of seven equal parts into which anything is divided; <as>as, the <ex>seventh</ex> part</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Seventh day</b></col>, <cd>the seventh day of the week; Saturday.</cd> -- <col><b>Seventh-day Baptists</b></col>. <cd>See under <er>Baptist</er>.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Seventh</ent><br/
<hw>Sev"enth</hw>, <pos>n.</pos> <sn>1.</sn> <def>One next in order after the sixth; one coming after six others.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>The quotient of a unit divided by seven; one of seven equal parts into which anything is divided.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <fld>(Mus.)</fld> <sd>(a)</sd> <def>An interval embracing seven diatonic degrees of the scale.</def> <sd>(b)</sd> <def>A chord which includes the interval of a seventh whether major, minor, or diminished.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Seven-thirties</ent><br/
<hw>Sev`en-thir"ties</hw> <pr>(?)</pr>, <pos>n. pl.</pos> <def>A name given to three several issues of United States Treasury notes, made during the Civil War, in denominations of $50 and over, bearing interest at the rate of seven and three tenths (thirty hundredths) per cent annually. Within a few years they were all redeemed or funded.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Seventhly</ent><br/
<hw>Sev"enth*ly</hw> <pr>(?)</pr>, <pos>adv.</pos> <def>In the seventh place.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Seventieth</ent><br/
<hw>Sev"en*ti*eth</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[AS. hund-<ets>seofontigo<edh/a</ets>.]</ety> <sn>1.</sn> <def>Next in order after the sixty-ninth; <as>as, a man in the <ex>seventieth</ex> year of his age</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Constituting or being one of seventy equal parts.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Seventieth</ent><br/
<hw>Sev"en*ti*eth</hw>, <pos>n.</pos> <sn>1.</sn> <def>One next in order after the sixty-ninth.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>The quotient of a unit divided by seventy; one of seventy equal parts or fractions.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Seventy</ent><br/
<hw>Sev"en*ty</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[AS. hund<ets>-seofontig</ets>. See <er>Seven</er>, and <er>Ten</er>, and cf. <er>Seventeen</er>, <er>Sixty</er>.]</ety> <def>Seven times ten; one more than sixty-nine.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Seventy</ent><br/
<hw>Sev"en*ty</hw>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Seventies</plw> <pr>(<?/)</pr>.</plu> <sn>1.</sn> <def>The sum of seven times ten; seventy units or objects.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A symbol representing seventy units, as 70, or lxx.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>The Seventy</b></col>, <cd>the translators of the Greek version of the Old Testament called the Septuagint. See <er>Septuagint</er>.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Seventy-four</ent><br/
<hw>Sev`en*ty-four"</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Naut.)</fld> <def>A naval vessel carrying seventy-four guns.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Seven-up</ent><br/
<hw>Sev"en-up`</hw>, <pos>n.</pos> <def>The game of cards called also <altname>all fours</altname>, and <altname>old sledge</altname>.</def> <mark>[U. S.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sever</ent><br/
<hw>Sev"er</hw> <pr>(?)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Severed</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Severing</conjf>.]</vmorph> <ety>[OF. <ets>sevrer</ets>, <ets>severer</ets>, to separate, F. <ets>sevrer</ets> to wean, fr. L. <ets>separare</ets>. See <er>Separate</er>, and cf. <er>Several</er>.]</ety> <sn>1.</sn> <def>To separate, as one from another; to cut off from something; to divide; to part in any way, especially by violence, as by cutting, rending, etc.; <as>as, to <ex>sever</ex> the head from the body</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The angels shall come forth, and <qex>sever</qex> the wicked from among the just.</q> <rj><qau>Matt. xiii. 49.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To cut or break open or apart; to divide into parts; to cut through; to disjoin; <as>as, to <ex>sever</ex> the arm or leg</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Our state can not be <qex>severed</qex>; we are one.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>To keep distinct or apart; to except; to exempt.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>I will <qex>sever</qex> in that day the land of Goshen, in which my people dwell, that no swarms of flies shall be there.</q> <rj><qau>Ex. viii. 22.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <fld>(Law)</fld> <def>To disunite; to disconnect; to terminate; <as>as, to <ex>sever</ex> an estate in joint tenancy</as>.</def>  <rj><au>Blackstone.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sever</ent><br/
<hw>Sev"er</hw>, <pos>v. i.</pos> <sn>1.</sn> <def>To suffer disjunction; to be parted, or rent asunder; to be separated; to part; to separate.</def>  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To make a separation or distinction; to distinguish.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The Lord shall <qex>sever</qex> between the cattle of Israel and the cattle of Egypt.</q> <rj><qau>Ex. ix. 4.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>They claimed the right of <qex>severing</qex> in their challenge.</q> <rj><qau>Macaulay.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Severable</ent><br/
<hw>Sev"er*a*ble</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Capable of being severed.</def>  <rj><au>Encyc. Dict.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Several</ent><br/
<hw>Sev"er*al</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[OF., fr. LL. <ets>separalis</ets>, fr. L. <ets>separ</ets> separate, different. See <er>Sever</er>, <er>Separate</er>.]</ety> <sn>1.</sn> <def>Separate; distinct; particular; single.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Each <qex>several</qex> ship a victory did gain.</q> <rj><qau>Dryden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Each might his <qex>several</qex> province well command,<br/
Would all but stoop to what they understand.</q> <rj><qau>Pope.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Diverse; different; various.</def>  <rj><au>Spenser.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Habits and faculties, <qex>several</qex>, and to be distinguished.</q> <rj><qau>Bacon.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Four <qex>several</qex> armies to the field are led.</q> <rj><qau>Dryden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>Consisting of a number more than two, but not very many; divers; sundry; <as>as, <ex>several</ex> persons were present when the event took place</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Several</ent><br/
<hw>Sev"er*al</hw>, <pos>adv.</pos> <def>By itself; severally.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>Every kind of thing is laid up <qex>several</qex> in barns or storehoudses.</q> <rj><qau>Robynson (More's Utopia).</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Several</ent><br/
<hw>Sev"er*al</hw>, <pos>n.</pos> <sn>1.</sn> <def>Each particular taken singly; an item; a detail; an individual.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>There was not time enough to hear . . . <br/
The <qex>severals</qex>.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Persons oe objects, more than two, but not very many.</def><br/
[<source>1913 Webster</source>]</p>

<p><q><qex>Several</qex> of them neither rose from any conspicuous family, nor left any behind them.</q> <rj><qau>Addison.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>An inclosed or separate place; inclosure.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>They had their <qex>several</qex> for heathen nations, their <qex>several</qex> for the people of their own nation.</q> <rj><qau>Hooker.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>In several</b></col>, <cd>in a state of separation.</cd> <mark>[R.]</mark> <ldquo/Where pastures <xex>in several</xex> be.<rdquo/  <rj><au>Tusser.</au></rj>
</cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Severality</ent><br/
<hw>Sev`er*al"i*ty</hw> <pr>(?)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Severalities</plw> <pr>(<?/)</pr>.</plu> <def>Each particular taken singly; distinction.</def> <mark>[Obs.]</mark>  <rj><au>Bp. Hall.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Severalize</ent><br/
<hw>Sev"er*al*ize</hw> <pr>(?)</pr>, <pos>v. t.</pos> <def>To distinguish.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Severally</ent><br/
<hw>Sev"er*al*ly</hw>, <pos>adv.</pos> <def>Separately; distinctly; apart from others; individually.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>There must be an auditor to check and revise each <qex>severally</qex> by itself.</q> <rj><qau>De Quincey.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Severalty</ent><br/
<hw>Sev"er*al*ty</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A state of separation from the rest, or from all others; a holding by individual right.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Forests which had never been owned in <qex>severalty</qex>.</q> <rj><qau>Bancroft.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Estate in severalty</b></col> <fld>(Law)</fld>, <cd>an estate which the tenant holds in his own right, without being joined in interest with any other person; -- distinguished from <xex>joint tenancy</xex>, <xex>coparcenary</xex>, and <xex>common</xex>.</cd>  <rj><au>Blackstone.</au></rj></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Severance</ent><br/
<hw>Sev"er*ance</hw> <pr>(?)</pr>, <pos>n.</pos> <sn>1.</sn> <def>The act of severing, or the state of being severed; partition; separation.</def>  <rj><au>Milman.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Law)</fld> <def>The act of dividing; the singling or severing of two or more that join, or are joined, in one writ; the putting in several or separate pleas or answers by two or more disjointly; the destruction of the unity of interest in a joint estate.</def>  <rj><au>Bouvier.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Severe</ent><br/
<hw>Se*vere"</hw> <pr>(?)</pr>, <pos>a.</pos> <amorph>[<pos>Compar.</pos> <adjf>Severer</adjf> <pr>(?)</pr>; <pos>superl.</pos> <adjf>Severest</adjf>.]</amorph> <ety>[L. <ets>severus</ets>; perhaps akin to Gr. <?/<?/<?/ awe, <?/<?/<?/ revered, holy, solemn, Goth. <ets>swikns</ets> innocent, chaste: cf. F. <ets>s<eacute/v<egrave/re</ets>.  Cf. <er>Asseverate</er>, <er>Persevere</er>.]</ety> <sn>1.</sn> <def>Serious in feeling or manner; sedate; grave; austere; not light, lively, or cheerful.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Your looks alter, as your subject does,<br/
From kind to fierce, from wanton to <qex>severe</qex>.</q> <rj><qau>Waller.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Very strict in judgment, discipline, or government; harsh; not mild or indulgent; rigorous; <as>as, <ex>severe</ex> criticism; <ex>severe</ex> punishment</as>.</def> <ldquo/Custody <xex>severe</xex>.<rdquo/  <rj><au>Milton.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Come! you are too <qex>severe</qex> a moraler.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Let your zeal, if it must be expressed in anger, be always more <qex>severe</qex> against thyself than against others.</q> <rj><qau>Jer. Taylor.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>Rigidly methodical, or adherent to rule or principle; exactly conformed to a standard; not allowing or employing unneccessary ornament, amplification, etc.; strict; -- said of style, argument, etc.</def> <ldquo/Restrained by reason and <xex>severe</xex> principles.<rdquo/  <rj><au>Jer. Taylor.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>The Latin, a most <qex>severe</qex> and compendious language.</q> <rj><qau>Dryden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>Sharp; afflictive; distressing; violent; extreme; <as>as, <ex>severe</ex> pain, anguish, fortune; <ex>severe</ex> cold</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <def>Difficult to be endured; exact; critical; rigorous; <as>as, a <ex>severe</ex> test</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><syn><b>Syn.</b> -- Strict; grave; austere; stern; morose; rigid; exact; rigorous; hard; rough; harsh; censorious; tart; acrimonious; sarcastic; satirical; cutting; biting; keen; bitter; cruel. See <er>Strict</er>.</syn><br/
[<source>1913 Webster</source>]</p>

<p>-- <wordforms><wf>Se*vere"ly</wf>, <pos>adv.</pos> -- <wf>Se*vere"ness</wf>, <pos>n.</pos></wordforms><br/
[<source>1913 Webster</source>]</p>

<p><ent>Severity</ent><br/
<hw>Se*ver"i*ty</hw> <pr>(?)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Severities</plw> <pr>(#)</pr>.</plu> <ety>[L. <ets>severitas</ets>: cf. F. <ets>s<eacute/v<eacute/rit<eacute/</ets>.]</ety> <def>The quality or state of being severe.</def> Specifically: --<br/
[<source>1913 Webster</source>]</p>

<p><sd>(a)</sd> <def>Gravity or austerity; extreme strictness; rigor; harshness; <as>as, the <ex>severity</ex> of a reprimand or a reproof; <ex>severity</ex> of discipline or government; <ex>severity</ex> of penalties</as>.</def> <ldquo/Strict age, and sour <xex>severity</xex>.<rdquo/  <rj><au>Milton.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sd>(b)</sd> <def>The quality or power of distressing or paining; extreme degree; extremity; intensity; inclemency; <as>as, the <ex>severity</ex> of pain or anguish; the <ex>severity</ex> of cold or heat; the <ex>severity</ex> of the winter</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sd>(c)</sd> <def>Harshness; cruel treatment; sharpness of punishment; <as>as, <ex>severity</ex> practiced on prisoners of war</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sd>(d)</sd> <def>Exactness; rigorousness; strictness; <as>as, the <ex>severity</ex> of a test</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Confining myself to the <qex>severity</qex> of truth.</q> <rj><qau>Dryden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Severy</ent><br/
<hw>Sev"er*y</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Prob. corrupted fr. <ets>ciborium</ets>. <ets>Oxf</ets>. <ets>Gloss</ets>.]</ety> <fld>(Arch.)</fld> <def>A bay or compartment of a vaulted ceiling.</def> <altsp>[Written also <asp>civery</asp>.]</altsp><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sevocation</ent><br/
<hw>Sev`o*ca"tion</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>sevocare</ets>, <ets>sevocatum</ets>, to call aside.]</ety> <def>A calling aside.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sevres blue</ent><br/
<hw>S<egrave/"vres blue`</hw> <pr>(?)</pr>. <def>A very light blue.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sevres ware</ent><br/
<hw>S<egrave/"vres ware`</hw> <pr>(?)</pr>. <def>Porcelain manufactured at <city><etsep>S<egrave/vres</etsep></city>, <country>France</country>, ecpecially in the national factory situated there.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sew</ent><br/
<hw>Sew</hw> <pr>(?)</pr>, <pos>n.</pos><ety>[OE. See <er>Sewer</er> household officer.]</ety> <def>Juice; gravy; a seasoned dish; a delicacy.</def> <mark>[Obs.]</mark>  <rj><au>Gower.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>I will not tell of their strange <qex>sewes</qex>.</q> <rj><qau>Chaucer.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sew</ent><br/
<hw>Sew</hw>, <pos>v. t.</pos> <ety>[See <er>Sue</er> to follow.]</ety> <def>To follow; to pursue; to sue.</def> <mark>[Obs.]</mark>  <rj><au>Chaucer. Spenser.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sew</ent><br/
<hw>Sew</hw> <pr>(?)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp.</pos> <conjf>Sewed</conjf> <pr>(?)</pr>; <pos>p. p.</pos> <conjf>Sewed</conjf>, rarely <conjf>Sewn</conjf> <pr>(<?/)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Sewing</conjf>.]</vmorph> <ety>[OE. <ets>sewen</ets>, <ets>sowen</ets>, AS. <ets>si<oacute/wian</ets>, <ets>s<imac/wian</ets>; akin to OHG. <ets>siuwan</ets>, Icel. <ets>s<?/ja</ets>, Sw. <ets>sy</ets>, Dan. <ets>sye</ets>, Goth. <ets>siujan</ets>, Lith. <ets>siuti</ets>, Russ, <ets>shite</ets>, L.  <ets>ssuere</ets>, Gr. <?/<?/<?/<?/, Skr. <ets>siv</ets>. <root/156.  Cf. <er>Seam</er> a suture, <er>Suture</er>.]</ety> <sn>1.</sn> <def>To unite or fasten together by stitches, as with a needle and thread.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>No man also <qex>seweth</qex> a piece of new cloth on an old garment.</q> <rj><qau>Mark ii. 21.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To close or stop by ssewing; -- often with <xex>up</xex>; <as>as, to <ex>sew</ex> up a rip</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>To inclose by sewing; -- sometimes with <xex>up</xex>; <as>as, to <ex>sew</ex> money in a bag</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sew</ent><br/
<hw>Sew</hw>, <pos>v. i.</pos> <def>To practice sewing; to work with needle and thread.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sew</ent><br/
<hw>Sew</hw> <pr>(?)</pr>, <pos>v. t.</pos> <ety>[<root/151 <it>b.</it> See <er>Sewer</er> a drain.]</ety> <def>To drain, as a pond, for taking the fish.</def> <mark>[Obs.]</mark>  <rj><au>Tusser.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sewage</ent><br/
<hw>Sew"age</hw> <pr>(?)</pr>, <pos>n.</pos> <sn>1.</sn> <def>The contents of a sewer or drain; refuse liquids or matter carried off by sewers</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Sewerage, 2.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sewe</ent><br/
<hw>Sewe</hw> <pr>(?)</pr>, <pos>v. i.</pos> <def>To perform the duties of a sewer. See 3d <er>Sewer</er>.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sewel</ent><br/
<hw>Sew"el</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Etymol. uncertain.]</ety> <def>A scarecrow, generally made of feathers tied to a string, hung up to prevent deer from breaking into a place.</def>  <rj><au>Halliwell.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sewellel</ent><br/
<hw>Se*wel"lel</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Of American Indian origin.]</ety> <fld>(Zool.)</fld> <def>A peculiar gregarious burrowing rodent (<spn>Haplodon rufus</spn>), native of the coast region of the Northwestern United States. It somewhat resembles a muskrat or marmot, but has only a rudimentary tail. Its head is broad, its eyes are small and its fur is brownish above, gray beneath. It constitutes the family <fam>Haplodontidae</fam>.  Called also <altname>boomer</altname>, <altname>showt'l</altname>, and <altname>mountain beaver</altname>.</def><br/
[<source>1913 Webster</source>]</p>

<p><-- p. 1320 --></p>

<p><ent>Sewen</ent><br/
<hw>Sew"en</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Zool.)</fld> <def>A British trout usually regarded as a variety (var. <varn>Cambricus</varn>) of the salmon trout.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sewer</ent><br/
<hw>Sew"er</hw> <pr>(?)</pr>, <pos>n.</pos> <sn>1.</sn> <def>One who sews, or stitches.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Zool.)</fld> <def>A small tortricid moth whose larva sews together the edges of a leaf by means of silk; <as>as, the apple-leaf <ex>sewer</ex> (<spn>Phoxopteris nubeculana</spn>)</as></def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sewer</ent><br/
<hw>Sew"er</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[OF. <ets>sewiere</ets>, <ets>seuwiere</ets>, ultimately fr. L. <ets>ex</ets> out + a derivative of <ets>aqua</ets> water; cf. OF. <ets>essevour</ets> a drain, <ets>essever</ets>, <ets>esseuwer</ets>, <ets>essiaver</ets>, to cause to flow, to drain, to flow, LL. <ets>exaquatorium</ets> a channel through which water runs off.  Cf. <er>Ewer</er>, <er>Aquarium</er>.]</ety> <def>A drain or passage to carry off water and filth under ground; a subterraneous channel, particularly in cities.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sewer</ent><br/
<hw>Sew"er</hw>, <pos>n.</pos> <ety>[Cf. OE. <ets>assewer</ets>, and <ets>asseour</ets>, OF. <ets>asseour</ets>, F. <ets>asseoir</ets> to seat, to set, L. <ets>assidere</ets> to sit by; <ets>ad + sedere</ets> to sit (cf. <er>Sit</er>); or cf. OE. <ets>sew</ets> pottage, sauce, boiled meat, AS. <ets>se<aacute/w</ets> juice, Skr. <ets>su</ets> to press out.]</ety> <def>Formerly, an upper servant, or household officer, who set on and removed the dishes at a feast, and who also brought water for the hands of the guests.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Then the <qex>sewer</qex><br/
Poured water from a great and golden ewer,<br/
That from their hands to a silver caldron ran.</q> <rj><qau>Chapman.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sewerage</ent><br/
<hw>Sew"er*age</hw> <pr>(?)</pr>, <pos>n.</pos> <sn>1.</sn> <def>The construction of a sewer or sewers.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>The system of sewers in a city, town, etc.; the general drainage of a city or town by means of sewers.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>The material collected in, and discharged by, sewers.</def> <altsp>[In this sense <asp>sewage</asp> is preferable and common.]</altsp><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sewin</ent><br/
<hw>Sew"in</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Zool.)</fld> <def>Same as <er>Sewen</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sewing</ent><br/
<hw>Sew"ing</hw> <pr>(?)</pr>, <pos>n.</pos> <sn>1.</sn> <def>The act or occupation of one who sews.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>That which is sewed with the needle.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Sewing horse</b></col> <fld>(Harness making)</fld>, <cd>a clamp, operated by the foot, for holding pieces of leather while being sewed.</cd> -- <col><b>Sewing machine</b></col>, <cd>a machine for sewing or stitching.</cd> -- <mcol><col><b>Sewing press</b></col>, <it>or</it> <col><b>Sewing table</b></col></mcol> <fld>(Bookbinding)</fld>, <cd>a fixture or table having a frame in which are held the cords to which the back edges of folded sheets are sewed to form a book.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sewster</ent><br/
<hw>Sew"ster</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A seamstress.</def> <mark>[Obs.]</mark>  <rj><au>B. Jonson.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sex-</ent><br/
<hw>Sex-</hw> <pr>(?)</pr>. <ety>[L. <ets>sex</ets> six. See <er>Six</er>.]</ety> <def>A combining form meaning <xex>six</xex>; <as>as, <ex>sex</ex>digitism; <ex>sex</ex>ennial</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sex</ent><br/
<hw>Sex</hw>, <pos>n.</pos> <ety>[L. <ets>sexus</ets>: cf. F. <ets>sexe</ets>.]</ety> <sn>1.</sn> <def>The distinguishing peculiarity of male or female in both animals and plants; the physical difference between male and female; the assemblage of properties or qualities by which male is distinguished from female.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>One of the two divisions of organic beings formed on the distinction of male and female.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <fld>(Bot.)</fld> <sd>(a)</sd> <def>The capability in plants of fertilizing or of being fertilized; <as>as, staminate and pistillate flowers are of opposite <ex>sexes</ex></as>.</def> <sd>(b)</sd> <def>One of the groups founded on this distinction.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>The sex</b></col>, <cd>the female sex; women, in general.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sexagenarian</ent><br/
<hw>Sex`a*ge*na"ri*an</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[See <er>Sexagenary</er>.]</ety> <def>A person who is sixty years old.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sexagenary</ent><br/
<hw>Sex*ag"e*na*ry</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>sexagenarius</ets>, fr. <ets>sexageni</ets> sixty each, akin to <ets>sexaginta</ets> sixty, <ets>sex</ets> six: cf. <ets>sexag<eacute/naire</ets>. See <er>Six</er>.]</ety> <def>Pertaining to, or designating, the number sixty; poceeding by sixties; sixty years old.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Sexagenary arithmetic</b></col>. <cd>See under <er>Sexagesimal</er>.</cd> -- <mcol><col><b>Sexagenary scale</b></col>, <it>or</it> <col><b>Sexagesimal scale</b></col></mcol> <fld>(Math.)</fld>, <cd>a scale of numbers in which the modulus is sixty.  It is used in treating the divisions of the circle.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sexagenary</ent><br/
<hw>Sex*ag"e*na*ry</hw>, <pos>n.</pos> <sn>1.</sn> <def>Something composed of sixty parts or divisions.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A sexagenarian.</def>  <rj><au>Sir W. Scott.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sexagesima</ent><br/
||<hw>Sex`a*ges"i*ma</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L., fem. of <ets>sexagesimus</ets> sixtieth, fr. <ets>sexaginta</ets> sixty.]</ety> <fld>(Eccl.)</fld> <def>The second Sunday before Lent; -- so called as being about the sixtieth day before Easter.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sexagesimal</ent><br/
<hw>Sex`a*ges"i*mal</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[Cf. F. <ets>sexag<eacute/simal</ets>.]</ety> <def>Pertaining to, or founded on, the number sixty.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><mcol><col><b>Sexagesimal fractions</b></col> <it>or</it> <col><b>Sexagesimal numbers</b></col></mcol> <fld>(Arith. & Alg.)</fld>, <cd>those fractions whose denominators are some power of sixty; as, <frac1x60/, <frac1x3600/, <frac1x216000/; -- called also <altname>astronomical fractions</altname>, because formerly there were no others used in astronomical calculations.</cd> -- <mcol><col><b>Sexagesimal arithmetic</b></col>, <it>or</it> <col><b>Sexagenary arithmetic</b></col></mcol>, <cd>the method of computing by the sexagenary scale, or by sixties.</cd> -- <col><b>Sexagesimal scale</b></col> <fld>(Math.)</fld>, <cd>the sexagenary scale.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sexagesimal</ent><br/
<hw>Sex`a*ges"i*mal</hw>, <pos>n.</pos> <def>A sexagesimal fraction.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sexangle</ent><br/
<hw>Sex"an`gle</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>sexangulus</ets> sexangular; <ets>sex</ets> six + <ets>angulus</ets> angle: cf. F. <ets>sexangle</ets>.  Cf. <er>Hexangular</er>.]</ety> <fld>(Geom.)</fld> <def>A hexagon.</def> <mark>[R.]</mark>  <rj><au>Hutton.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sexangular</ent><br/
<ent>Sexangled</ent><br/
<mhw>{ <hw>Sex"an`gled</hw> <pr>(?)</pr>, <hw>Sex*an"gu*lar</hw> <pr>(?)</pr> }</mhw> <pos>a.</pos> <ety>[Cf. F. <ets>sexangulaire</ets>.]</ety> <def>Having six angles; hexagonal.</def> <mark>[R.]</mark>  <rj><au>Dryden.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sexangularly</ent><br/
<hw>Sex*an"gu*lar*ly</hw>, <pos>adv.</pos> <def>Hexagonally.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sexavalent</ent><br/
<hw>Sex*av"a*lent</hw> <pr>(?)</pr>, <pos>a.</pos> <def>See <er>Sexivalent</er>.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sexdigitism</ent><br/
<hw>Sex*dig"it*ism</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[<ets>Sex-</ets> + <ets>digit</ets>.]</ety> <def>The state of having six fingers on a hand, or six toes on a foot.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sexdigitist</ent><br/
<hw>Sex*dig"it*ist</hw>, <pos>n.</pos> <def>One who has six fingers on a hand, or six toes on a foot.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sexed</ent><br/
<hw>Sexed</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Belonging to sex; having sex; distinctively male of female; <as>as, the <ex>sexed</ex> condition</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sexenary</ent><br/
<hw>Sex"e*na*ry</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Proceeding by sixes; sextuple; -- applied especially to a system of arithmetical computation in which the base is six.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sexennial</ent><br/
<hw>Sex*en"ni*al</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>sexennium</ets> a period of six years, <ets>sexennis</ets> of six years; <ets>sex</ets> six + <ets>annus</ets> a year. See <er>Six</er>, and <er>Annual</er>.]</ety> <def>Lasting six years, or happening once in six years.</def> -- <def2><pos>n.</pos> <def>A sexennial event.</def></def2><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sexennially</ent><br/
<hw>Sex*en"ni*al*ly</hw>, <pos>adv.</pos> <def>Once in six years.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sexifid</ent><br/
<ent>Sexfid</ent><br/
<mhw>{ <hw>Sex"fid</hw> <pr>(?)</pr>, <hw>Sex"i*fid</hw> <pr>(?)</pr>, }</mhw> <pos>a.</pos> <ety>[<ets>Sex-</ets> + root of L. <ets>findere</ets> to split: cf. F. <ets>sexfide</ets>.]</ety> <fld>(Bot.)</fld> <def>Six-cleft; <as>as, a <ex>sexfid</ex> calyx or nectary</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sexisyllabic</ent><br/
<hw>Sex`i*syl*lab"ic</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[<ets>Sex-</ets> + <ets>syllabic</ets>.]</ety> <def>Having six syllables.</def>  <rj><au>Emerson.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sexisyllable</ent><br/
<hw>Sex"i*syl`la*ble</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[<ets>Sex-</ets> + <ets>syllable</ets>.]</ety> <def>A word of six syllables.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sexivalent</ent><br/
<hw>Sex*iv"a*lent</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[<ets>Sex-</ets> + L. <ets>valens</ets>, p. pr. See <er>Valence</er>.]</ety> <fld>(hem.)</fld> <def>Hexavalent.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sexless</ent><br/
<hw>Sex"less</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Having no sex.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sexlocular</ent><br/
<hw>Sex`loc"u*lar</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[<ets>Sex-</ets> + <ets>locular</ets>: cf. F. <ets>sexloculaire</ets>.]</ety> <fld>(Bot.)</fld> <def>Having six cells for seeds; six-celled; <as>as, a <ex>sexlocular</ex> pericarp</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sexly</ent><br/
<hw>Sex"ly</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Pertaining to sex.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>Should I ascribe any of these things unto myself or my <qex>sexly</qex> weakness, I were not worthy to live.</q> <rj><qau>Queen Elizabeth.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sexradiate</ent><br/
<hw>Sex*ra"di*ate</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[<ets>Sex-</ets> + <ets>radiate</ets>.]</ety> <fld>(Zool.)</fld> <def>Having six rays; -- said of certain sponge spicules. See <xex>Illust.</xex> of <er>Spicule</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sext</ent><br/
<hw>Sext</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>sexta</ets>, fem. of <ets>sextus</ets> sixtt, fr. <ets>sex</ets> six: cf. F. <ets>sexte</ets>.]</ety> <fld>(R.C.Ch.)</fld> <sd>(a)</sd> <def>The office for the sixth canonical hour, being a part of the Breviary.</def> <sd>(b)</sd> <def>The sixth book of the decretals, added by Pope Boniface VIII.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sextain</ent><br/
<hw>Sex"tain</hw>, <pos>n.</pos> <ety>[L. <ets>sextus</ets> sixth, fr. <ets>sex</ets> six: cf. It. <ets>sestina</ets>.]</ety> <fld>(Pros.)</fld> <def>A stanza of six lines; a sestine.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sextans</ent><br/
||<hw>Sex"tans</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. See <er>Sextant</er>.]</ety> <sn>1.</sn> <fld>(Rom. Antiq.)</fld> <def>A Roman coin, the sixth part of an as.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Astron.)</fld> <def>A constellation on the equator south of Leo; the Sextant.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sextant</ent><br/
<hw>Sex"tant</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>sextans</ets>, <ets>-antis</ets>, the sixth part of an as, fr. <ets>sextus</ets> sixth, <ets>sex</ets> six. See <er>Six</er>.]</ety> <sn>1.</sn> <fld>(Math.)</fld> <def>The sixth part of a circle.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>An instrument for measuring angular distances between objects, -- used esp. at sea, for ascertaining the latitude and longitude. It is constructed on the same optical principle as Hadley's quadrant, but usually of metal, with a nicer graduation, telescopic sight, and its arc the sixth, and sometimes the third, part of a circle. See <er>Quadrant</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <fld>(Astron.)</fld> <def>The constellation Sextans.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Box sextant</b></col>, <cd>a small sextant inclosed in a cylindrical case to make it more portable.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sextary</ent><br/
<hw>Sex"ta*ry</hw> <pr>(?)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Sextaries</plw> <pr>(#)</pr>.</plu> <ety>[L. <ets>sextarius</ets> the sixth part of a measure, weight, etc., fr. <ets>sextus</ets> sixth, <ets>sex</ets> six.]</ety> <fld>(Rom. Antiq.)</fld> <def>An ancient Roman liquid and dry measure, about equal to an English pint.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sextary</ent><br/
<hw>Sex"ta*ry</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[For <ets>sextonry</ets>.]</ety> <def>A sacristy.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sextetto</ent><br/
<ent>Sextet</ent><br/
<mhw>{ <hw>Sex*tet"</hw> <pr>(?)</pr>, <hw>Sex*tet"to</hw> <pr>(?)</pr>, }</mhw> <pos>n.</pos> <fld>(Mus.)</fld> <def>See <er>Sestet</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sexteyn</ent><br/
<hw>Sex"teyn</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A sacristan.</def> <mark>[Obs.]</mark>  <rj><au>Chaucer.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sextic</ent><br/
<hw>Sex"tic</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>sextus</ets> sixth.]</ety> <fld>(Math.)</fld> <def>Of the sixth degree or order.</def> -- <def2><pos>n.</pos> <fld>(Alg.)</fld> <def>A quantic of the sixth degree.</def></def2><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sextile</ent><br/
<hw>Sex"tile</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[F. <ets>sextil</ets>, fr. L. <ets>sextus</ets> the sixth, from <ets>sex</ets> six. See <er>Six</er>.]</ety> <fld>(Astrol.)</fld> <def>Measured by sixty degrees; fixed or indicated by a distance of sixty degrees.</def>  <rj><au>Glanvill.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sextile</ent><br/
<hw>Sex"tile</hw>, <pos>n.</pos> <ety>[Cf. F. aspect <ets>sextil</ets>.]</ety> <fld>(Astrol.)</fld> <def>The aspect or position of two planets when distant from each other sixty degrees, or two signs. This position is marked thus: <8star/.</def>  <rj><au>Hutton.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sextillion</ent><br/
<hw>Sex*til"lion</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Formed (in imitation of <ets>million</ets>) fr. L. <ets>sextus</ets> sixth, <ets>sex</ets> six: cf. F. <ets>sextilion</ets>.]</ety> <def>According to the method of numeration (which is followed also in the United States), the number expressed by a unit with twenty-one ciphers annexed. According to the English method, a million raised to the sixth power, or the number expressed by a unit with thirty-six ciphers annexed. See <er>Numeration</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sexto</ent><br/
<hw>Sex"to</hw> <pr>(?)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Sextos</plw> <pr>(#)</pr>.</plu> <ety>[L. <ets>sextus</ets> sixth.]</ety> <def>A book consisting of sheets each of which is folded into six leaves.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sextodecimo</ent><br/
<hw>Sex`to*dec"i*mo</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>sextus-decimus</ets> the sixteenth; <ets>sextus</ets> the sixth (fr. <ets>sex</ets> six) + <ets>decimus</ets> the tenth, from <ets>decem</ets> ten. See <er>-mo</er>.]</ety> <def>Having sixteen leaves to a sheet; of, or equal to, the size of one fold of a sheet of printing paper when folded so as to make sixteen leaves, or thirty-two pages; <as>as, a <ex>sextodecimo</ex> volume</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sextodecimo</ent><br/
<hw>Sex`to*dec"imo</hw>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Sextodecimos</plw> <pr>(<?/)</pr>.</plu> <def>A book composed of sheets each of which is folded into sixteen leaves; hence, indicating, more or less definitely, a size of a book; -- usually written 16mo, or 16<deg/.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sextolet</ent><br/
<hw>Sex"to*let</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Mus.)</fld> <def>A double triplet; a group of six equal notes played in the time of four.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sexton</ent><br/
<hw>Sex"ton</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[OE. <ets>sextein</ets>, contr. fr. <ets>sacristan</ets>.]</ety> <def>An under officer of a church, whose business is to take care of the church building and the vessels, vestments, etc., belonging to the church, to attend on the officiating clergyman, and to perform other duties pertaining to the church, such as to dig graves, ring the bell, etc.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Sexton beetle</b></col> <fld>(Zool.)</fld>, <cd>a burying beetle.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sextoness</ent><br/
<hw>Sex"ton*ess</hw>, <pos>n.</pos> <def>A female sexton; a sexton's wife.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sextonry</ent><br/
<hw>Sex"ton*ry</hw> <pr>(?)</pr>, <pos>n.</pos> <def>Sextonship.</def> <mark>[Obs.]</mark>  <rj><au>Ld. Bernes.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sextonship</ent><br/
<hw>Sex"ton*ship</hw>, <pos>n.</pos> <def>The office of a sexton.</def>  <rj><au>Swift.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sextry</ent><br/
<hw>Sex"try</hw> <pr>(?)</pr>, <pos>n.</pos> <def>See <er>Sacristy</er>.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sextuple</ent><br/
<hw>Sex"tu*ple</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[Formed (in imitation of <ets>quadruple</ets>) fr. L. <ets>sextus</ets> sixth: cf. F. <ets>sextuple</ets>.]</ety> <sn>1.</sn> <def>Six times as much; sixfold.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Mus.)</fld> <def>Divisible by six; having six beats; <as>as, <ex>sixtuple</ex> measure</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sexual</ent><br/
<hw>Sex"u*al</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>sexualis</ets>, fr. <ets>sexus</ets> sex: cf. F. <ets>sexuel</ets>.]</ety> <def>Of or pertaining to sex, or the sexes; distinguishing sex; peculiar to the distinction and office of male or female; relating to the distinctive genital organs of the sexes; proceeding from, or based upon, sex; <as>as, <ex>sexual</ex> characteristics; <ex>sexual</ex> intercourse, connection, or commerce; <ex>sexual</ex> desire; <ex>sexual</ex> diseases; <ex>sexual</ex> generation</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Sexual dimorphism</b></col> <fld>(Biol.)</fld>, <cd>the condition of having one of the sexes existing in two forms, or varieties, differing in color, size, etc., as in many species of butterflies which have two kinds of females.</cd> -- <col><b>Sexual method</b></col> <fld>(Bot.)</fld>, <cd>a method of classification proposed by Linnaeus, founded mainly on difference in number and position of the stamens and pistils of plants.</cd> -- <col><b>Sexual selection</b></col> <fld>(Biol.)</fld>, <cd>the selective preference of one sex for certain characteristics in the other, such as bright colors, musical notes, etc.; also, the selection which results from certain individuals of one sex having more opportunities of pairing with the other sex, on account of greater activity, strength, courage, etc.; applied likewise to that kind of evolution which results from such sexual preferences.</cd> <au>Darwin.</au><br/
[<source>1913 Webster</source>]</p>

<p><q>In these cases, therefore, natural selection seems to have acted independently of <qex>sexual selection</qex>.</q> <rj><qau>A. R. Wallace.</qau></rj>
</cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sexualist</ent><br/
<hw>Sex"u*al*ist</hw>, <pos>n.</pos> <fld>(Bot.)</fld> <def>One who classifies plants by the sexual method of Linnaeus.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sexuality</ent><br/
<hw>Sex`u*al"i*ty</hw> <pr>(?)</pr>, <pos>n.</pos> <def>The quality or state of being distinguished by sex.</def>  <rj><au>Lindley.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sexualize</ent><br/
<hw>Sex"u*al*ize</hw> <pr>(?)</pr>, <pos>v. t.</pos> <def>To attribute sex to.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sexually</ent><br/
<hw>Sex"u*al*ly</hw>, <pos>adv.</pos> <def>In a sexual manner or relation.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Seyh</ent><br/
<ent>Sey</ent><br/
<mhw>{ <hw>Sey</hw> <pr>(?)</pr>, <hw>Seyh</hw> <pr>(?)</pr>, }</mhw> <mark>obs.</mark> <def><pos>imp. sing. & 2d pers. pl.</pos> of <er>See</er>.</def>  <rj><au>Chaucer.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Seyen</ent><br/
<ent>Seye</ent><br/
<mhw>{ <hw>Seye</hw> <pr>(?)</pr>, <hw>Seyen</hw> <pr>(?)</pr>, }</mhw> <mark>obs.</mark> <def><pos>imp. pl. & p. p.</pos> of <er>See</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Seynd</ent><br/
<hw>Seynd</hw> <pr>(?)</pr>, <mark>obs.</mark> <def><pos>p. p.</pos> of <er>Senge</er>, to singe.</def>  <rj><au> Chaucer.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Seynt</ent><br/
<hw>Seynt</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A gridle. See 1st <er>Seint</er>.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sforzato</ent><br/
<ent>Sforzando</ent><br/
<mhw>{ ||<hw>Sfor*zan"do</hw> <pr>(?)</pr>, ||<hw>Sfor*za"to</hw> <pr>(?)</pr>, }</mhw> <pos>a.</pos> <ety>[It. <ets>sforzando</ets>, p. pr., and <ets>sforzato</ets>, p. p. of <ets>sforzare</ets> to force.]</ety> <fld>(Mus.)</fld> <def>Forcing or forced; -- a direction placed over a note, to signify that it must be executed with peculiar emphasis and force; -- marked <xex>fz</xex> (an abbreviation of <xex>forzando</xex>), <xex>sf</xex>, <xex>sfz</xex>, or <?/.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sfumato</ent><br/
||<hw>Sfu*ma"to</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[It.]</ety> <fld>(Paint.)</fld> <def>Having vague outlines, and colors and shades so mingled as to give a misty appearance; -- said of a painting.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sgraffito</ent><br/
||<hw>Sgraf*fi"to</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[It.]</ety> <fld>(Paint.)</fld> <def>Scratched; -- said of decorative painting of a certain style, in which a white overland surface is cut or scratched through, so as to form the design from a dark ground underneath.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shab</ent><br/
<hw>Shab</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[OE. <ets>shabbe</ets>, AS. <ets>sc<?/b</ets>. See <er>Scab</er>.]</ety> <def>The itch in animals; also, a scab.</def> <mark>[Obs. or Prov. Eng.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shab</ent><br/
<hw>Shab</hw>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Shabbed</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Shabbing</conjf>.]</vmorph> <ety>[See <er>Scab</er>, 3.]</ety> <def>To play mean tricks; to act shabbily.</def> <mark>[Obs. or Prov. Eng.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shab</ent><br/
<hw>Shab</hw>, <pos>v. t.</pos> <def>To scratch; to rub.</def> <mark>[Obs.]</mark>  <rj><au>Farquhar.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shabbed</ent><br/
<hw>Shab"bed</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Shabby.</def> <mark>[Obs.]</mark>  <rj><au>Wood.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shabbily</ent><br/
<hw>Shab"bi*ly</hw> <pr>(?)</pr>, <pos>adv.</pos> <def>In a shabby manner.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shabbiness</ent><br/
<hw>Shab"bi*ness</hw>, <pos>n.</pos> <def>The quality or state of being sghabby.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shabble</ent><br/
<ent>Shabble</ent><br/
<mhw>{ <hw>Shab"ble</hw> <pr>(?)</pr>, <hw>Shab"ble</hw>  }</mhw>, <pos>n.</pos><ety>[Cf. D. <ets>sabel</ets>, and G. <ets>s<aum/bel</ets>.]</ety> <def>A kind of crooked sword or hanger.</def> <mark>[Scot.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shabby</ent><br/
<hw>Shab"by</hw> <pr>(?)</pr>, <pos>a.</pos> <amorph>[<pos>Compar.</pos> <adjf>Shabbier</adjf> <pr>(?)</pr>; <pos>superl.</pos> <adjf>Shabbiest</adjf>.]</amorph> <ety>[See <er>Shab</er>, <pos>n.</pos>, <er>Scabby</er>, and <er>Scab</er>.]</ety> <sn>1.</sn> <def>Torn or worn to rage; poor; mean; ragged.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Wearing <qex>shabby</qex> coats and dirty shirts.</q> <rj><qau>Macaulay.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Clothed with ragged, much worn, or soiled garments.</def> <ldquo/The dean was so <xex>shabby</xex>.<rdquo/  <rj><au>Swift.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>Mean; paltry; despicable; <as>as, <ex>shabby</ex> treatment</as>.</def> <ldquo/Very <xex>shabby</xex> fellows.<rdquo/  <rj><au>Clarendon.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shabrack</ent><br/
||<hw>Shab"rack</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Turk. <ets>tsh<amac/pr<amac/k</ets>, whence F. <ets>chabraque</ets>, G. <ets>shabracke</ets>.]</ety> <fld>(Mil.)</fld> <def>The saddlecloth or housing of a cavalry horse.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shack</ent><br/
<hw>Shack</hw> <pr>(?)</pr>, <pos>v. t.</pos> <ety>[Prov. E., to shake, to shed. See <er>Shake</er>.]</ety> <sn>1.</sn> <def>To shed or fall, as corn or grain at harvest.</def> <mark>[Prov. Eng.]</mark>  <rj><au>Grose.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To feed in stubble, or upon waste corn.</def> <mark>[Prov. Eng.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>To wander as a vagabond or a tramp.</def> <mark>[Prev.Eng.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shack</ent><br/
<hw>Shack</hw>, <pos>n.</pos> <ety>[Cf. <er>Shack</er>, <pos>v. i.</pos>]</ety> <def>a small simple dwelling, usually having only one room and of flimsy construction; a hut; a shanty; a cabin.</def> <mark>[Colloq.]</mark></p>

<p><ent>Shack</ent><br/
<hw>Shack</hw>, <pos>n.</pos> <ety>[Cf. Scot. <ets>shag</ets> refuse of barley or oats.]</ety> <sn>1.</sn> <def>The grain left after harvest or gleaning; also, nuts which have fallen to the ground.</def> <mark>[Prov. Eng.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Liberty of winter pasturage.</def> <mark>[Prov. Eng.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>A shiftless fellow; a low, itinerant beggar; a vagabond; a tramp.</def> <mark>[Prov. Eng. & Colloq. U.S.]</mark>  <rj><au>Forby.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>All the poor old <qex>shacks</qex> about the town found a friend in Deacon Marble.</q> <rj><qau>H. W. Beecher.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>These miserable <qex>shacks</qex> are so low that their occupants cannot stand erect.</q>  <rj><qau>D. C. Worcester.</qau></rj><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><cs><col><b>Common of shack</b></col> <fld>(Eng.Law)</fld>, <cd>the right of persons occupying lands lying together in the same common field to turn out their cattle to range in it after harvest.</cd>  <rj><au>Cowell.</au></rj></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shackatory</ent><br/
<hw>Shack"a*to*ry</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A hound.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shackle</ent><br/
<hw>Shac"kle</hw> <pr>(?)</pr>, <pos>n.</pos> <def>Stubble.</def> <mark>[Prov. Eng.]</mark>  <rj><au>Pegge.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shackle</ent><br/
<hw>Shac"kle</hw>, <pos>n.</pos> <ety>[Generally used in the plural.]</ety> <ety>[OE. <ets>schakkyll</ets>, <ets>schakle</ets>, AS. <ets>scacul</ets>, <ets>sceacul</ets>, a shackle, fr. <ets>scacan</ets> to shake; cf. D. <ets>schakel</ets> a link of a chain, a mesh, Icel. <ets>sk<oum/kull</ets> the pole of a cart. See <er>Shake</er>.]</ety> <sn>1.</sn> <def>Something which confines the legs or arms so as to prevent their free motion; specifically, a ring or band inclosing the ankle or wrist, and fastened to a similar shackle on the other leg or arm, or to something else, by a chain or a strap; a gyve; a fetter.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>His <qex>shackles</qex> empty left; himself escaped clean.</q> <rj><qau>Spenser.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Hence, that which checks or prevents free action.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>His very will seems to be in bonds and <qex>shackles</qex>.</q> <rj><qau>South.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>A fetterlike band worn as an ornament.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Most of the men and women . . . had all earrings made of gold, and gold <qex>shackles</qex> about their legs and arms.</q> <rj><qau>Dampier.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>A link or loop, as in a chain, fitted with a movable bolt, so that the parts can be separated, or the loop removed; a clevis.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <def>A link for connecting railroad cars; -- called also <altname>drawlink</altname>, <altname>draglink</altname>, etc.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>6.</sn> <def>The hinged and curved bar of a padlock, by which it is hung to the staple.</def>  <rj><au>Knight.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Shackle joint</b></col> <fld>(Anat.)</fld>, <cd>a joint formed by a bony ring passing through a hole in a bone, as at the bases of spines in some fishes.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><-- p. 1321 --></p>

<p><ent>Shackle</ent><br/
<hw>Shac"kle</hw> <pr>(?)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Shackled</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Shackling</conjf>.]</vmorph> <sn>1.</sn> <def>To tie or confine the limbs of, so as to prevent free motion; to bind with shackles; to fetter; to chain.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>To lead him <qex>shackled</qex>, and exposed to scorn<br/
Of gathering crowds, the Britons' boasted chief.</q> <rj><qau>J. Philips.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Figuratively: To bind or confine so as to prevent or embarrass action; to impede; to cumber.</def><br/
[<source>1913 Webster</source>]</p>

<p><q><qex>Shackled</qex> by her devotion to the king, she seldom could pursue that object.</q> <rj><qau>Walpole.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>To join by a link or chain, as railroad cars.</def> <mark>[U. S.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Shackle bar</b></col>, <cd>the coupling between a locomotive and its tender.</cd> <mark>[U.S.]</mark> -- <col><b>Shackle bolt</b></col>, <cd>a shackle.</cd>  <rj><au>Sir W. Scott.</au></rj></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shacklock</ent><br/
<hw>Shack"lock`</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A sort of shackle.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shackly</ent><br/
<hw>Shack"ly</hw>, <pos>a.</pos> <def>Shaky; rickety.</def> <mark>[Colloq. U. S.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shack up</ent><br/
<hw>Shack" up`</hw>, <pos>v. i.</pos> <sn>1.</sn> <def>to live together in a sexual relationship, without being legally married.</def> <mark>[Slang, U. S.]</mark><br/
[<source>PJC</source>]</p>

<p><sn>2.</sn> <def>to live in a cabin, shack, or other crude dwelling.</def> <mark>[Slang, U. S.]</mark><br/
[<source>PJC</source>]</p>

<p><ent>Shad</ent><br/
<hw>Shad</hw> <pr>(sh<acr/d)</pr>, <pos>n. sing. & pl.</pos> <ety>[AS. <ets>sceadda</ets> a kind of fish, akin to Prov. G. <ets>schade</ets>; cf. Ir. & Gael. <ets>sgadan</ets> a herring, W. <ets>ysgadan</ets> herrings; all perhaps akin to E. <ets>skate</ets> a fish.]</ety> <fld>(Zool.)</fld> <def>Any one of several species of food fishes of the Herring family. The American species (<spn>Alosa sapidissima</spn> formerly <spn>Clupea sapidissima</spn>), which is abundant on the Atlantic coast and ascends the larger rivers in spring to spawn, is an important market fish. The European allice shad, or alose (<spn>Alosa alosa</spn>  formerly <spn>Clupea alosa</spn>), and the twaite shad (<spn>Alosa finta</spn>  formerly <spn>Clupea finta</spn>), are less important species.</def> <altsp>[Written also <asp>chad</asp>.]</altsp><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ The name is loosely applied, also, to several other fishes, as the gizzard shad (see under <er>Gizzard</er>), called also <altname>mud shad</altname>, <altname>white-eyed shad</altname>, and <altname>winter shad</altname>.</note><br/
[<source>1913 Webster</source>]</p>

<p><cs><mcol><col><b>Hardboaded shad</b></col>, <it>or</it>  <col><b>Yellow-tailed shad</b></col></mcol>, <cd>the menhaden.</cd> -- <mcol><col><b>Hickory shad</b></col>, <it>or</it>  <col><b>Tailor shad</b></col></mcol>, <cd>the <er>mattowacca</er>.</cd> -- <col><b>Long-boned shad</b></col>, <cd>one of several species of important food fishes of the Bermudas and the West Indies, of the genus <gen>Gerres</gen>.</cd> -- <col><b>Shad bush</b></col> <fld>(Bot.)</fld>, <cd>a name given to the North American shrubs or small trees of the rosaceous genus <gen>Amelanchier</gen> (<spn>Amelanchier Canadensis</spn>, and <spn>Amelanchier alnifolia</spn>).  Their white racemose blossoms open in April or May, when the shad appear, and the edible berries (pomes) ripen in June or July, whence they are called <prod>Juneberries</prod>.  The plant is also called <altname>service tree</altname>, and <altname>Juneberry</altname>.</cd> -- <col><b>Shad frog</b></col>, <cd>an American spotted frog (<spn>Rana halecina</spn>); -- so called because it usually appears at the time when the shad begin to run in the rivers.</cd> -- <col><b>Trout shad</b></col>, <cd>the squeteague.</cd> -- <col><b>White shad</b></col>, <cd>the common shad.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shadbird</ent><br/
<hw>Shad"bird`</hw> <pr>(sh<acr/d"b<etil/rd)</pr>, <pos>n.</pos> <fld>(Zool.)</fld> <sd>(a)</sd> <def>The American, or Wilson's, snipe. See under <er>Snipe</er>. So called because it appears at the same time as the shad.</def> <sd>(b)</sd> <def>The common European sandpiper.</def> <mark>[Prov. Eng.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shadd</ent><br/
<hw>Shadd</hw> <pr>(sh<acr/d)</pr>, <pos>n.</pos> <fld>(Mining.)</fld> <def>Rounded stones containing tin ore, lying at the surface of the ground, and indicating a vein.</def>  <rj><au>Raymond.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shadde</ent><br/
<hw>Shad"de</hw> <pr>(?)</pr>, <def><pos>obs. imp.</pos> of <er>Shed</er>.</def>  <rj><au>Chaucer.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shaddock</ent><br/
<hw>Shad"dock</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Said to be so called from a Captain <ets>Shaddock</ets>, who first brought this fruit from the East Indies.]</ety> <fld>(Bot.)</fld> <def>A tree (<spn>Citrus decumana</spn>) and its fruit, which is a large species of orange; -- called also <altname>forbidden fruit</altname>, and <altname>pompelmous</altname>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shade</ent><br/
<hw>Shade</hw> <pr>(sh<amac/d)</pr>, <pos>n.</pos> <ety>[OE. <ets>shade</ets>, <ets>shadewe</ets>, <ets>schadewe</ets>, AS. <ets>sceadu</ets>, <ets>scead</ets>; akin to OS. <ets>skado</ets>, D. <ets>schaduw</ets>, OHG. <ets>scato</ets>, (gen. <ets>scatewes</ets>), G. <ets>schatten</ets>, Goth. <ets>skadus</ets>, Ir. & Gael. <ets>sgath</ets>, and probably to Gr. <grk>sko`tos</grk> darkness. <root/162.  Cf. <er>Shadow</er>, <er>Shed</er> a hat.]</ety> <sn>1.</sn> <def>Comparative obscurity owing to interception or interruption of the rays of light; partial darkness caused by the intervention of something between the space contemplated and the source of light.</def><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ <xex>Shade</xex> differs from <xex>shadow</xex> as it implies no particular form or definite limit; whereas a <xex>shadow</xex> represents in form the object which intercepts the light. When we speak of the <xex>shade</xex> of a tree, we have no reference to its form; but when we speak of measuring a pyramid or other object by its <xex>shadow</xex>, we have reference to its form and extent.</note><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Darkness; obscurity; -- often in the plural.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The <qex>shades</qex> of night were falling fast.</q> <rj><qau>Longfellow.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>An obscure place; a spot not exposed to light; hence, a secluded retreat.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Let us seek out some desolate <qex>shade</qex>, and there<br/
Weep our sad bosoms empty.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>That which intercepts, or shelters from, light or the direct rays of the sun; hence, also, that which protects from heat or currents of air; a screen; protection; shelter; cover; <as>as, a lamp <ex>shade</ex></as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The Lord is thy <qex>shade</qex> upon thy right hand.</q> <rj><qau>Ps. cxxi. 5.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Sleep under a fresh tree's <qex>shade</qex>.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Let the arched knife well sharpened now assail the spreading <qex>shades</qex> of vegetables.</q> <rj><qau>J. Philips.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <def>Shadow.</def> <mark>[Poetic.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>Envy will merit, as its <qex>shade</qex>, pursue.</q> <rj><qau>Pope.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>6.</sn> <def>The soul after its separation from the body; -- so called because the ancients it to be perceptible to the sight, though not to the touch; a spirit; a ghost; <as>as, the <ex>shades</ex> of departed heroes</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Swift as thought the flitting <qex>shade</qex><br/
Thro' air his momentary journey made.</q> <rj><qau>Dryden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>7.</sn> <fld>(Painting, Drawing, etc.)</fld> <def>The darker portion of a picture; a less illuminated part. See Def. 1, above.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>8.</sn> <def>Degree or variation of color, as darker or lighter, stronger or paler; <as>as, a delicate <ex>shade</ex> of pink</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>White, red, yellow, blue, with their several degrees, or <qex>shades</qex> and mixtures, as green only in by the eyes.</q> <rj><qau>Locke.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>9.</sn> <def>A minute difference or variation, as of thought, belief, expression, etc.; also, the quality or degree of anything which is distinguished from others similar by slight differences; <as>as, the <ex>shades</ex> of meaning in synonyms</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>New <qex>shades</qex> and combinations of thought.</q> <rj><qau>De Quincey.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Every <qex>shade</qex> of religious and political opinion has its own headquarters.</q> <rj><qau>Macaulay.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>The Shades</b></col>, <cd>the Nether World; the supposed abode of souls after leaving the body.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shade</ent><br/
<hw>Shade</hw> <pr>(?)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Shaded</conjf>; <pos>p. pr. & vb. n.</pos> <conjf>Shading</conjf>.]</vmorph> <sn>1.</sn> <def>To shelter or screen by intercepting the rays of light; to keep off illumination from.</def>  <rj><au>Milton.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>I went to crop the sylvan scenes,<br/
And <qex>shade</qex> our altars with their leafy greens.</q> <rj><qau>Dryden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To shelter; to cover from injury; to protect; to screen; to hide; <as>as, to <ex>shade</ex> one's eyes</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Ere in our own house I do <qex>shade</qex> my head.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>To obscure; to dim the brightness of.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Thou <qex>shad'st</qex><br/
The full blaze of thy beams.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>To pain in obscure colors; to darken.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <def>To mark with gradations of light or color.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>6.</sn> <def>To present a shadow or image of; to shadow forth; to represent.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>[The goddess] in her person cunningly did <qex>shade</qex><br/
That part of Justice which is Equity.</q> <rj><qau>Spenser.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shade</ent><br/
<hw>Shade</hw> <pr>(sh<amac/d)</pr>, <pos>v. i.</pos> <ety>[See <er>Shade</er>, <pos>n.</pos>]</ety> <def>To undergo or exhibit minute difference or variation, as of color, meaning, expression, etc.; to pass by slight changes; -- used chiefly with a preposition, as <xex>into</xex>, <xex>away</xex>, <xex>off</xex>.</def></p>

<p><q>This small group will be most conveniently treated with the emotional division, into which it <qex>shades</qex>.</q>  <rj><qau>Edmund Gurney.</qau></rj><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Shadeful</ent><br/
<hw>Shade"ful</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Full of shade; shady.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shadeless</ent><br/
<hw>Shade"less</hw>, <pos>a.</pos> <def>Being without shade; not shaded.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shader</ent><br/
<hw>Shad"er</hw> <pr>(?)</pr>, <pos>n.</pos> <def>One who, or that which, shades.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shadily</ent><br/
<hw>Shad"i*ly</hw> <pr>(?)</pr>, <pos>adv.</pos> <def>In a shady manner.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shadiness</ent><br/
<hw>Shad"i*ness</hw>, <pos>n.</pos> <def>Quality or state of being shady.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shading</ent><br/
<hw>Shad"ing</hw>, <pos>n.</pos> <sn>1.</sn> <def>Act or process of making a shade.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>That filling up which represents the effect of more or less darkness, expressing rotundity, projection, etc., in a picture or a drawing.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shadoof</ent><br/
||<hw>Sha*doof"</hw> <pr>(sh<adot/*d<oomac/f")</pr>, <pos>n.</pos> <ety>[Ar. <ets>sh<amac/d<umac/f</ets>.]</ety> <def>A machine, resembling a well sweep, used in Egypt for raising water from the Nile for irrigation.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shadow</ent><br/
<hw>Shad"ow</hw> <pr>(sh<acr/d"<osl/)</pr>, <pos>n.</pos> <ety>[Originally the same word as <ets>shade</ets>. <root/162. See <er>Shade</er>.]</ety> <sn>1.</sn> <def>Shade within defined limits; obscurity or deprivation of light, apparent on a surface, and representing the form of the body which intercepts the rays of light; <as>as, the <ex>shadow</ex> of a man, of a tree, or of a tower</as>. See the Note under <er>Shade</er>, <pos>n.</pos>, 1.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Darkness; shade; obscurity.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Night's sable <qex>shadows</qex> from the ocean rise.</q> <rj><qau>Denham.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>A shaded place; shelter; protection; security.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>In secret <qex>shadow</qex> from the sunny ray,<br/
On a sweet bed of lilies softly laid.</q> <rj><qau>Spenser.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>A reflected image, as in a mirror or in water.</def>  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <def>That which follows or attends a person or thing like a shadow; an inseparable companion; hence, an obsequious follower.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Sin and her <qex>shadow</qex> Death.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>6.</sn> <def>A spirit; a ghost; a shade; a phantom.</def> <ldquo/Hence, horrible <xex>shadow</xex>!<rdquo/  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>7.</sn> <def>An imperfect and faint representation; adumbration; indistinct image; dim bodying forth; hence, mystical representation; type.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The law having a <qex>shadow</qex> of good things to come.</q> <rj><qau>Heb. x. 1.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>[Types] and <qex>shadows</qex> of that destined seed.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>8.</sn> <def>A small degree; a shade.</def> <ldquo/No variableness, neither <xex>shadow</xex> of turning.<rdquo/  <rj><au>James i. 17.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>9.</sn> <def>An uninvited guest coming with one who is invited.</def> <mark>[A Latinism]</mark>  <rj><au>Nares.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>I must not have my board pastered with <qex>shadows</qex><br/
That under other men's protection break in<br/
Without invitement.</q> <rj><qau>Massinger.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Shadow of death</b></col>, <cd>darkness or gloom like that caused by the presence or the impending of death.</cd>  <rj><au>Ps. xxiii. 4.</au></rj></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shadow</ent><br/
<hw>Shad"ow</hw>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Shadowed</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Shadowing</conjf>.]</vmorph> <ety>[OE. <ets>shadowen</ets>, AS. <ets>sceadwian</ets>. See <er>adow</er>, <pos>n.</pos>]</ety> <sn>1.</sn> <def>To cut off light from; to put in shade; to shade; to throw a shadow upon; to overspead with obscurity.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The warlike elf much wondered at this tree,<br/
So fair and great, that <qex>shadowed</qex> all the ground.</q> <rj><qau>Spenser.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To conceal; to hide; to screen.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>Let every soldier hew him down a bough.<br/
And bear't before him; thereby shall we <qex>shadow</qex><br/
The numbers of our host.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>To protect; to shelter from danger; to shroud.</def><br/
[<source>1913 Webster</source>]</p>

<p><q><qex>Shadowing</qex> their right under your wings of war.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>To mark with gradations of light or color; to shade.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <def>To represent faintly or imperfectly; to adumbrate; hence, to represent typically.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Augustus is <qex>shadowed</qex> in the person of <AE/neas.</q> <rj><qau>Dryden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>6.</sn> <def>To cloud; to darken; to cast a gloom over.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The <qex>shadowed</qex> livery of the burnished sun.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Why sad?<br/
I must not see the face O love thus <qex>shadowed</qex>.</q> <rj><qau>Beau. & Fl.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>7.</sn> <def>To attend as closely as a shadow; to follow and watch closely, especially in a secret or unobserved manner; <as>as, a detective <ex>shadows</ex> a criminal</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shadowiness</ent><br/
<hw>Shad"ow*i*ness</hw> <pr>(?)</pr>, <pos>n.</pos> <def>The quality or state of being shadowy.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shadowing</ent><br/
<hw>Shad"ow*ing</hw>, <pos>n.</pos> <sn>1.</sn> <def>Shade, or gradation of light and color; shading.</def>  <rj><au>Feltham.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A faint representation; an adumbration.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>There are . . . in savage theology <qex>shadowings</qex>, quaint or majestic, of the conception of a Supreme Deity.</q> <rj><qau>Tylor.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shadowish</ent><br/
<hw>Shad"ow*ish</hw>, <pos>a.</pos> <def>Shadowy; vague.</def> <mark>[Obs.]</mark>  <rj><au>Hooker.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shadowless</ent><br/
<hw>Shad"ow*less</hw>, <pos>a.</pos> <def>Having no shadow.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shadowy</ent><br/
<hw>Shad"ow*y</hw> <pr>(?)</pr>, <pos>a.</pos> <sn>1.</sn> <def>Full of shade or shadows; causing shade or shadow.</def> <ldquo/<xex>Shadowy</xex> verdure.<rdquo/  <rj><au>Fenton.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>This <qex>shadowy</qex> desert, unfrequented woods.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Hence, dark; obscure; gloomy; dim.</def> <ldquo/The <xex>shadowy</xex> past.<rdquo/  <rj><au>Longfellow.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>Not brightly luminous; faintly light.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The moon . . . with more pleasing light,<br/
<qex>Shadowy</qex> sets off the face things.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>Faintly representative; hence, typical.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>From <qex>shadowy</qex> types to truth, from flesh to spirit.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <def>Unsubstantial; unreal; <as>as, <ex>shadowy</ex> honor</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Milton has brought into his poems two actors of a <qex>shadowy</qex><br/
and fictitious nature, in the persons of Sin and Death.</q> <rj><qau>Addison.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shadrach</ent><br/
<hw>Sha"drach</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Metal.)</fld> <def>A mass of iron on which the operation of smelting has failed of its intended effect; -- so called from <xex>Shadrach</xex>, one of the three Hebrews who came forth unharmed from the fiery furnace of Nebuchadnezzar. (See <au>Dan. iii. 26, 27.</au>)</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shad-spirit</ent><br/
<hw>Shad"-spir`it</hw> <pr>(?)</pr>, <pos>n.</pos> <def>See <er>Shadbird</er> <sd>(a)</sd></def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shad-waiter</ent><br/
<hw>Shad"-wait`er</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Zool.)</fld> <def>A lake whitefish; the roundfish. See <er>Roundfish</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shady</ent><br/
<hw>Shad"y</hw> <pr>(?)</pr>, <pos>a.</pos> <amorph>[<pos>Compar.</pos> <adjf>Shadier</adjf> <pr>(?)</pr>; <pos>superl.</pos> <adjf>Shadiest</adjf>.]</amorph> <sn>1.</sn> <def>Abounding in shade or shades; overspread with shade; causing shade.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The <qex>shady</qex> trees cover him with their shadow.</q> <rj><qau>Job. xl. 22.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>And Amaryllis fills the <qex>shady</qex> groves.</q> <rj><qau>Dryden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Sheltered from the glare of light or sultry heat.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Cast it also that you may have rooms <qex>shady</qex> for summer and warm for winter.</q> <rj><qau>Bacon.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>Of or pertaining to shade or darkness;</def> <specif>hence,</specif> <def>unfit to be seen or known; of questionable character; unsavory; equivocal; dubious, corrupt, or criminal; <as>as, a <ex>shady</ex> character</as>; -- of people or activities.</def> <mark>[Colloq.]</mark> <ldquo/A <xex>shady</xex> business.<rdquo/  <rj><au>London Sat. Rev.</au></rj></p>

<p><q><qex>Shady</qex> characters, disreputable, criminal.</q> <rj><qau>London Spectator.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>On the shady side of</b></col>, <cd>on the thither side of; as, <xex>on the shady side of</xex> fifty; that is, more than fifty.</cd> <mark>[Colloq.]</mark> -- <col><b>To keep shady</b></col>, <cd>to stay in concealment; also, to be reticent.</cd> <mark>[Slang]</mark></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shaffle</ent><br/
<hw>Shaf"fle</hw> <pr>(?)</pr>, <pos>v. i.</pos> <ety>[See <er>Shuffle</er>.]</ety> <def>To hobble or limp; to shuffle.</def> <mark>[Obs. or Prov. Eng.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shaffler</ent><br/
<hw>Shaf"fler</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A hobbler; one who limps; a shuffer.</def> <mark>[Obs. or Prov. Eng.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shafiite</ent><br/
<hw>Sha"fi*ite</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A member of one of the four sects of the Sunnites, or Orthodox Mohammedans; -- so called from its founder, Mohammed al-<xex>Shafe<ium/</xex>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shaft</ent><br/
<hw>Shaft</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[OE. <ets>shaft</ets>, <ets>schaft</ets>, AS. <ets>sceaft</ets>; akin to D. <ets>schacht</ets>, OHG. <ets>scaft</ets>, G. <ets>schaft</ets>, Dan. & Sw. <ets>skaft</ets> handle, haft, Icel. <ets>skapt</ets>, and probably to L. <ets>scapus</ets>, Gr. <?/<?/<?/<?/, <?/<?/<?/<?/, a staff. Probably originally, a shaven or smoothed rod.  Cf. <er>Scape</er>, <er>Scepter</er>, <er>Shave</er>.]</ety> <sn>1.</sn> <def>The slender, smooth stem of an arrow; hence, an arrow.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>His sleep, his meat, his drink, is him bereft,<br/
That lean he wax, and dry as is a <qex>shaft</qex>.</q> <rj><qau>Chaucer.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>A <qex>shaft</qex> hath three principal parts, the stele [stale], the feathers, and the head.</q> <rj><qau>Ascham.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>The long handle of a spear or similar weapon; hence, the weapon itself; (<mark>Fig.</mark>) anything regarded as a shaft to be thrown or darted; <as>as, <ex>shafts</ex> of light</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>And the thunder,<br/
Winged with red lightning and impetuous rage,<br/
Perhaps hath spent his <qex>shafts</qex>.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Some kinds of literary pursuits . . . have been attacked with all the <qex>shafts</qex> of ridicule.</q> <rj><qau>V. Knox.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>That which resembles in some degree the stem or handle of an arrow or a spear; a long, slender part, especially when cylindrical.</def> Specifically: (a) <fld>(Bot.)</fld> <def>The trunk, stem, or stalk of a plant.</def> <sd>(b)</sd> <fld>(Zool.)</fld> <def>The stem or midrib of a feather.</def> See <xex>Illust.</xex> of <er>Feather</er>. <sd>(c)</sd> <def>The pole, or tongue, of a vehicle; also, a thill.</def> <sd>(d)</sd> <def>The part of a candlestick which supports its branches.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Thou shalt make a candlestick of pure gold . . . his <qex>shaft</qex>, and his branches, his bowls, his knops, and his flowers, shall be of the same.</q> <rj><qau>Ex. xxv. 31.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sd>(e)</sd> <def>The handle or helve of certain tools, instruments, etc., as a hammer, a whip, etc.</def> <sd>(f)</sd> <def>A pole, especially a Maypole.</def> <mark>[Obs.]</mark> <au>Stow.</au> <sd>(g)</sd> <fld>(Arch.)</fld> <def>The body of a column; the cylindrical pillar between the capital and base (see <xex>Illust.</xex> of <er>Column</er>). Also, the part of a chimney above the roof. Also, the spire of a steeple.</def> <mark>[Obs. or R.]</mark> <au>Gwilt.</au> <sd>(h)</sd> <def>A column, an obelisk, or other spire-shaped or columnar monument.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Bid time and nature gently spare<br/
The <qex>shaft</qex> we raise to thee.</q> <rj><qau>Emerson.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sd>(i)</sd> <fld>(Weaving)</fld> <def>A rod at the end of a heddle.</def> <sd>(j)</sd> <fld>(Mach.)</fld> <def>A solid or hollow cylinder or bar, having one or more journals on which it rests and revolves, and intended to carry one or more wheels or other revolving parts and to transmit power or motion; <as>as, the <ex>shaft</ex> of a steam engine</as>.</def> See <xex>Illust.</xex> of <er>Countershaft</er>.<br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <fld>(Zool.)</fld> <def>A humming bird (<spn>Thaumastura cora</spn>) having two of the tail feathers next to the middle ones very long in the male; -- called also <altname>cora humming bird</altname>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <ety>[Cf. G. <ets>schacht</ets>.]</ety> <fld>(Mining)</fld> <def>A well-like excavation in the earth, perpendicular or nearly so, made for reaching and raising ore, for raising water, etc.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>6.</sn> <def>A long passage for the admission or outlet of air; an air shaft.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>7.</sn> <def>The chamber of a blast furnace.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Line shaft</b></col> <fld>(Mach.)</fld>, <cd>a main shaft of considerable length, in a shop or factory, usually bearing a number of pulleys by which machines are driven, commonly by means of countershafts; -- called also <altname>line</altname>, or <altname>main line</altname>.</cd> -- <col><b>Shaft alley</b></col> <fld>(Naut.)</fld>, <cd>a passage extending from the engine room to the stern, and containing the propeller shaft.</cd> -- <col><b>Shaft furnace</b></col> <fld>(Metal.)</fld>, <cd>a furnace, in the form of a chimney, which is charged at the top and tapped at the bottom.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shafted</ent><br/
<hw>Shaft"ed</hw>, <pos>a.</pos> <sn>1.</sn> <def>Furnished with a shaft, or with shafts; <as>as, a <ex>shafted</ex> arch</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Her.)</fld> <def>Having a shaft; -- applied to a spear when the head and the shaft are of different tinctures.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shafting</ent><br/
<hw>Shaft"ing</hw>, <pos>n.</pos> <fld>(Mach.)</fld> <def>Shafts, collectivelly; a system of connected shafts for communicating motion.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shaftment</ent><br/
<ent>Shaftman</ent><br/
<mhw>{ <hw>Shaft"man</hw> <pr>(?)</pr>, <hw>Shaft"ment</hw> <pr>(?)</pr>, }</mhw> <pos>n.</pos> <ety>[AS. <ets>sceaftmund</ets>.]</ety> <def>A measure of about six inches.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><-- p. 1322 --></p>

<p><ent>Shag</ent><br/
<hw>Shag</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[AS. <ets>sceacga</ets> a bush of hair; akin to Icel. <ets>skegg</ets> the beard, Sw. <ets>sk<aum/gg</ets>, Dan. <ets>skj<?/g</ets>.  Cf. <er>Schock</er> of hair.]</ety> <sn>1.</sn> <def>Coarse hair or nap; rough, woolly hair.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>True Witney broadcloth, with its <qex>shag</qex> unshorn.</q> <rj><qau>Gay.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A kind of cloth having a long, coarse nap.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <fld>(Com.)</fld> <def>A kind of prepared tobacco cut fine.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <fld>(Zool.)</fld> <def>Any species of cormorant.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shag</ent><br/
<hw>Shag</hw>, <pos>a.</pos> <def>Hairy; shaggy.</def>  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shag</ent><br/
<hw>Shag</hw>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Shagged</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Shagging</conjf>.]</vmorph> <def>To make hairy or shaggy; hence, to make rough.</def><br/
[<source>1913 Webster</source>]</p>

<p><q><qex>Shag</qex> the green zone that bounds the boreal skies.</q> <rj><qau>J. Barlow.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><-- (Sport)  To chase, as (a) to chase and return balls hit out of bounds. (b) (Baseball) to catch fly balls for practise. --><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shagbark</ent><br/
<hw>Shag"bark`</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Bot.)</fld> <def>A rough-barked species of hickory (<spn>Carya alba</spn>), its nut.  Called also <altname>shellbark</altname>.  See <er>Hickory</er>.</def> <sd>(b)</sd> <def>The West Indian <spn>Pithecolobium micradenium</spn>, a legiminous tree with a red coiled-up pod.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shagebush</ent><br/
<hw>Shage"bush`</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A sackbut.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shagged</ent><br/
<hw>Shag"ged</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Shaggy; rough.</def> <au>Milton.</au> -- <wordforms><wf>Shag"ged*ness</wf>, <pos>n.</pos> <au>Dr. H. More.</au></wordforms><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shagginess</ent><br/
<hw>Shag"gi*ness</hw> <pr>(?)</pr>, <pos>n.</pos> <def>The quality or state of being shaggy; roughness; shaggedness.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shaggy</ent><br/
<hw>Shag"gy</hw> <pr>(?)</pr>, <pos>a.</pos> <amorph>[<pos>Compar.</pos> <adjf>Shaggier</adjf> <pr>(?)</pr>; <pos>superl.</pos> <adjf>Shaggiest</adjf>.]</amorph> <ety>[From <er>Shag</er>, <pos>n.</pos>]</ety> <def>Rough with long hair or wool.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>About his shoulders hangs the <qex>shaggy</qex> skin.</q> <rj><qau>Dryden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Rough; rugged; jaggy.</def>  <rj><au>Milton.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>[A rill] that winds unseen beneath the <qex>shaggy</qex> fell.</q> <rj><qau>Keble.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shag-haired</ent><br/
<hw>Shag"-haired`</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Having shaggy hair.</def>  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shag-rag</ent><br/
<hw>Shag"-rag`</hw> <pr>(?)</pr>, <pos>n.</pos> <def>The unkempt and ragged part of the community.</def> <mark>[Colloq. or Slang.]</mark>  <rj><au>R. Browning.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shagreen</ent><br/
<hw>Sha*green"</hw> <pr>(?)</pr>, <pos>v. t.</pos> <def>To chagrin.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shagreen</ent><br/
<hw>Sha*green"</hw>, <pos>n.</pos> <ety>[F. <ets>chagrin</ets>, It. <ets>zigrino</ets>, fr. Turk. <ets>saghri</ets> the back of a horse or other beast of burden, shagreen.  Cf. <er>Chagrin</er>.]</ety> <sn>1.</sn> <def>A kind of untanned leather prepared in Russia and the East, from the skins of horses, asses, and camels, and grained so as to be covered with small round granulations. This characteristic surface is produced by pressing small seeds into the grain or hair side when moist, and afterward, when dry, scraping off the roughness left between them, and then, by soaking, causing the portions of the skin which had been compressed or indented by the seeds to swell up into relief. It is used for covering small cases and boxes.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>The skin of various small sharks and other fishes when having small, rough, bony scales. The dogfishes of the genus <gen>Scyllium</gen> furnish a large part of that used in the arts.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shagreened</ent><br/
<ent>Shagreen</ent><br/
<mhw>{ <hw>Sha*green"</hw> <pr>(?)</pr>, <hw>Sha*greened"</hw> <pr>(?)</pr> }</mhw> <pos>a.</pos> <sn>1.</sn> <def>Made or covered with the leather called shagreen.</def> <ldquo/A <xex>shagreen</xex> case of lancets.<rdquo/  <rj><au>T. Hook.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Zool.)</fld> <def>Covered with rough scales or points like those on shagreen.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shah</ent><br/
<hw>Shah</hw> <pr>(sh<aum/)</pr>, <pos>n.</pos> <ety>[Per. <ets>sh<amac/h</ets> a king, sovereign, prince.  Cf. <er>Checkmate</er>, <er>Chess</er>, <er>Pasha</er>.]</ety> <def>A former title of the supreme ruler in certain Eastern countries, especially Persia and Iran.</def> <altsp>[Written also <asp>schah</asp>.]</altsp><br/
[<source>1913 Webster</source> <source>+PJC</source>]</p>

<p><cs><col><b>Shah Nameh</b></col>. <ety>[Per., Book of Kings.]</ety> <cd>A celebrated historical poem written by Firdousi, being the most ancient in the modern Persian language.</cd>  <rj><au>Brande & C.</au></rj></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shahin</ent><br/
||<hw>Sha*hin"</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Ar. <ets>sh<amac/h<imac/n</ets>.]</ety> <fld>(Zool.)</fld> <def>A large and swift Asiatic falcon (<spn>Falco pregrinator</spn>) highly valued in falconry.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shaik</ent><br/
<hw>Shaik</hw> <pr>(?)</pr>, <pos>n.</pos> <def>See <er>Sheik</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shail</ent><br/
<hw>Shail</hw> <pr>(?)</pr>, <pos>v. i.</pos> <ety>[Cf. AS. <ets>sceolh</ets> squinting, Icel. <ets>skj<amac/gr</ets> wry, oblique, Dan. <ets>skele</ets> to squint.]</ety> <def>To walk sidewise.</def> <mark>[Obs.]</mark>  <rj><au>L'Estrange.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sheitan</ent><br/
<ent>Shaitan</ent><br/
<mhw>{ ||<hw>Shai"tan</hw>, ||<hw>Shei"tan</hw> }</mhw> <pr>(?)</pr>, <pos>n.</pos> <altsp>[Written also <asp>sheytan</asp>.]</altsp> <ety>[Hind. <ets>shait<amac/n</ets>, fr. Ar. <ets>shai<tsdot/<amac/n</ets>.]</ety> <sn>1.</sn> <mark>Among Mohammedans</mark>: <sd>(a)</sd> <def>An evil spirit; the evil one; the devil.</def> <sd>(b)</sd> <def>One of bad disposition; a fiend.</def> <mark>[Colloq.]</mark><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><sn>2.</sn>  <fld>(Meteor.)</fld> <def>A dust storm.</def> <mark>[India]</mark><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Shake</ent><br/
<hw>Shake</hw> <pr>(?)</pr>, <def><pos>obs. p. p.</pos> of <er>Shake</er>.</def>  <rj><au>Chaucer.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shake</ent><br/
<hw>Shake</hw>, <pos>v. t.</pos> <vmorph>[<pos>imp.</pos> <conjf>Shook</conjf> <pr>(?)</pr>; <pos>p. p.</pos> <conjf>Shaken</conjf> <pr>(?)</pr>, (<conjf>Shook</conjf>, <mark>obs.</mark>); <pos>p. pr. & vb. n.</pos> <conjf>Shaking</conjf>.]</vmorph> <ety>[OE. <ets>shaken</ets>, <ets>schaken</ets>, AS. <ets>scacan</ets>, <ets>sceacan</ets>; akin to Icel. & Sw. <ets>skaka</ets>, OS. <ets>skakan</ets>, to depart, to flee. <root/161.  Cf. <er>Shock</er>, <pos>v.</pos>]</ety> <sn>1.</sn> <def>To cause to move with quick or violent vibrations; to move rapidly one way and the other; to make to tremble or shiver; to agitate.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>As a fig tree casteth her untimely figs, when she is <qex>shaken</qex> of a mighty wind.</q> <rj><qau>Rev. vi. 13.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Ascend my chariot; guide the rapid wheels<br/
That <qex>shake</qex> heaven's basis.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Fig.: To move from firmness; to weaken the stability of; to cause to waver; to impair the resolution of.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>When his doctrines grew too strong to be <qex>shook</qex> by his enemies, they persecuted his reputation.</q> <rj><qau>Atterbury.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Thy equal fear that my firm faith and love<br/
Can by his fraud be <qex>shaken</qex> or seduced.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <fld>(Mus.)</fld> <def>To give a tremulous tone to; to trill; <as>as, to <ex>shake</ex> a note in music</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>To move or remove by agitating; to throw off by a jolting or vibrating motion; to rid one's self of; -- generally with an adverb, as <xex>off</xex>, <xex>out</xex>, etc.; <as>as, to <ex>shake</ex> fruit down from a tree</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q><qex>Shake</qex> off the golden slumber of repose.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>'Tis our fast intent<br/
To <qex>shake</qex> all cares and business from our age.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>I could scarcely <qex>shake</qex> him out of my company.</q> <rj><qau>Bunyan.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>To shake a cask</b></col> <fld>(Naut.)</fld>, <cd>to knock a cask to pieces and pack the staves.</cd> -- <col><b>To shake hands</b></col>, <cd>to perform the customary act of civility by clasping and moving hands, as an expression of greeting, farewell, good will, agreement, etc.</cd> -- <col><b>To shake out a reef</b></col> <fld>(Naut.)</fld>, <cd>to untile the reef points and spread more canvas.</cd> -- <col><b>To shake the bells</b></col>. <cd>See under <er>Bell</er>.</cd> -- <col><b>To shake the sails</b></col> <fld>(Naut.)</fld>, <cd>to luff up in the wind, causing the sails to shiver.</cd> <au>Ham. Nav. Encyc.</au></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shake</ent><br/
<hw>Shake</hw>, <pos>v. i.</pos> <def>To be agitated with a waving or vibratory motion; to tremble; to shiver; to quake; to totter.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Under his burning wheels<br/
The steadfast empyrean <qex>shook</qex> throughout,<br/
All but the throne itself of God.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>What danger? Who 's that that <qex>shakes</qex> behind there?</q> <rj><qau>Beau. & Fl.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Shaking piece</b></col>, <cd>a name given by butchers to the piece of beef cut from the under side of the neck. See <xex>Illust.</xex> of <er>Beef</er>.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shake</ent><br/
<hw>Shake</hw> <pr>(?)</pr>, <pos>n.</pos> <sn>1.</sn> <def>The act or result of shaking; a vacillating or wavering motion; a rapid motion one way and other; a trembling, quaking, or shivering; agitation.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The great soldier's honor was composed<br/
Of thicker stuff, which could endure a <qex>shake</qex>.</q> <rj><qau>Herbert.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Our salutations were very hearty on both sides, consisting of many kind <qex>shakes</qex> of the hand.</q> <rj><qau>Addison.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A fissure or crack in timber, caused by its being dried too suddenly.</def>  <rj><au>Gwilt.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>A fissure in rock or earth.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <fld>(Mus.)</fld> <def>A rapid alternation of a principal tone with another represented on the next degree of the staff above or below it; a trill.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <fld>(Naut.)</fld> <def>One of the staves of a hogshead or barrel taken apart.</def>  <rj><au>Totten.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>6.</sn> <def>A shook of staves and headings.</def>  <rj><au>Knight.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>7.</sn> <fld>(Zool.)</fld> <def>The redshank; -- so called from the nodding of its head while on the ground.</def> <mark>[Prov. Eng.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>No great shakes</b></col>, <cd>of no great importance.</cd> <mark>[Slang]</mark> <au>Byron.</au> -- <col><b>The shakes</b></col>, <cd>the fever and ague.</cd> <mark>[Colloq. U.S.]</mark></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shakedown</ent><br/
<hw>Shake"down`</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A temporary substitute for a bed, as one made on the floor or on chairs; -- perhaps originally from the shaking down of straw for this purpose.</def>  <rj><au>Sir W. Scott.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><-- shake down, v. t. subject to extortion. --><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shakefork</ent><br/
<hw>Shake"fork`</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A fork for shaking hay; a pitchfork.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shaken</ent><br/
<hw>Shak"en</hw> <pr>(?)</pr>, <pos>a.</pos> <sn>1.</sn> <def>Caused to shake; agitated; <as>as, a <ex>shaken</ex> bough</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Cracked or checked; split. See <er>Shake</er>, <pos>n.</pos>, 2.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Nor is the wood <qex>shaken</qex> or twisted.</q> <rj><qau>Barroe.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>Impaired, as by a shock.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shaker</ent><br/
<hw>Shak"er</hw> <pr>(?)</pr>, <pos>n.</pos> <sn>1.</sn> <def>A person or thing that shakes, or by means of which something is shaken.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>One of a religious sect who do not marry, popularly so called from the movements of the members in dancing, which forms a part of their worship.</def><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ The sect originated in England in 1747, and came to the United States in 1774, under the leadership of Mother Ann Lee. The Shakers are sometimes nicknamed <xex>Shaking Quakers</xex>, but they differ from the Quakers in doctrine and practice. They style themselves the <ldquo/United Society of Believers in Christ's Second Appearing.<rdquo/ The sect is now confined in the United States.</note><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <fld>(Zool.)</fld> <def>A variety of pigeon.</def>  <rj><au>P. J. Selby.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shakeress</ent><br/
<hw>Shak"er*ess</hw>, <pos>n.</pos> <def>A female Shaker.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shakerism</ent><br/
<hw>Shak"er*ism</hw> <pr>(?)</pr>, <pos>n.</pos> <def>Doctrines of the Shakers.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shakespearean</ent><br/
<hw>Shake*spear"e*an</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Of, pertaining to, or in the style of, Shakespeare or his works.</def> <altsp>[Written also <asp>Shakespearian</asp>, <asp>Shakspearean</asp>, <asp>Shakspearian</asp>, <asp>Shaksperean</asp>, <asp>Shaksperian</asp>.etc.]</altsp><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shakiness</ent><br/
<hw>Shak"i*ness</hw> <pr>(?)</pr>, <pos>n.</pos> <def>Quality of being shaky.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shakings</ent><br/
<hw>Shak"ings</hw> <pr>(?)</pr>, <pos>n. pl.</pos> <fld>(Naut.)</fld> <def>Deck sweepings, refuse of cordage, canvas, etc.</def>  <rj><au>Ham. Nav. Encyc.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shako</ent><br/
<hw>Shak"o</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Hung. <ets>cs<aacute/k<oacute/</ets>: cf. F. <ets>shako</ets>, <ets>schako</ets>.]</ety> <def>A kind of military cap or headdress.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shakudo</ent><br/
||<hw>Shak"u*do"</hw> <pr>(sh<acr/k"<oocr/*d<omac/")</pr>, <pos>n.</pos> <ety>[Jap.]</ety> <def>An alloy of copper, invented by the Japanese, having a very dark blue color approaching black.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Shaky</ent><br/
<hw>Shak"y</hw> <pr>(?)</pr>, <pos>a.</pos> <amorph>[<pos>Compar.</pos> <adjf>Shakier</adjf> <pr>(?)</pr>; <pos>superl.</pos> <adjf>Shakiest</adjf>.]</amorph> <sn>1.</sn> <def>Shaking or trembling; <as>as, a <ex>shaky</ex> spot in a marsh; a <ex>shaky</ex> hand</as>.</def>  <rj><au>Thackeray.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Full of shakes or cracks; cracked; <as>as, <ex>shaky</ex> timber</as>.</def>  <rj><au>Gwilt.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>Easily shaken; tottering; unsound; <as>as, a <ex>shaky</ex> constitution; <ex>shaky</ex> business credit</as>.</def> <mark>[Colloq.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shale</ent><br/
<hw>Shale</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[AS. <ets>scealy</ets>, <ets>scalu</ets>. See <er>Scalme</er>, and cf. <er>Shell</er>.]</ety> <sn>1.</sn> <def>A shell or husk; a cod or pod.</def> <ldquo/The green <xex>shales</xex> of a bean.<rdquo/  <rj><au>Chapman.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <ety>[G. <ets>shale</ets>.]</ety> <fld>(Geol.)</fld> <def>A fine-grained sedimentary rock of a thin, laminated, and often friable, structure.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Bituminous shale</b></col>. <cd>See under <er>Bituminous</er>.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shale</ent><br/
<hw>Shale</hw>, <pos>v. t.</pos> <def>To take off the shell or coat of; to shell.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Life, in its upper grades, was bursting its shell, or was <qex>shaling</qex> off its husk.</q> <rj><qau>I. Taylor.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shall</ent><br/
<hw>Shall</hw> <pr>(?)</pr>, <pos>v. i. & auxiliary.</pos> <vmorph>[<pos>imp.</pos> <conjf>Should</conjf> <pr>(?)</pr>.]</vmorph> <ety>[OE. <ets>shal</ets>, <ets>schal</ets>, imp. <ets>sholde</ets>, <ets>scholde</ets>, AS. <ets>scal</ets>, <ets>sceal</ets>, I am obliged, imp. <ets>scolde</ets>, <ets>sceolde</ets>, inf. <ets>sculan</ets>; akin to OS. <ets>skulan</ets>, pres. <ets>skal</ets>, imp. <ets>skolda</ets>, D. <ets>zullen</ets>, pres. <ets>zal</ets>, imp. <ets>zoude</ets>, <ets>zou</ets>, OHG. <ets>solan</ets>, <ets>scolan</ets>, pres. <ets>scal</ets>, <ets>sol</ets>. imp. <ets>scolta</ets>, <ets>solta</ets>, G. <ets>sollen</ets>, pres. <ets>soll</ets>, imp. <ets>sollte</ets>, Icel. <ets>skulu</ets>, pres. <ets>skal</ets>, imp. <ets>skyldi</ets>, SW. <ets>skola</ets>, pres. <ets>skall</ets>, imp. <ets>skulle</ets>, Dan. <ets>skulle</ets>, pres. <ets>skal</ets>, imp. <ets>skulde</ets>, Goth. <ets>skulan</ets>, pres. <ets>skal</ets>, imp. <ets>skulda</ets>, and to AS. <ets>scyld</ets> guilt, G. <ets>schuld</ets> guilt, fault, debt, and perhaps to L. <ets>scelus</ets> crime.]</ety> <note>[<xex>Shall</xex> is defective, having no infinitive, imperative, or participle.]</note> <sn>1.</sn> <def>To owe; to be under obligation for.</def> <mark>[Obs.]</mark> <ldquo/By the faith I <xex>shall</xex> to God<rdquo/  <rj><au>Court of Love.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To be obliged; must.</def> <mark>[Obs.]</mark> <ldquo/Me athinketh [I am sorry] that I <xex>shall</xex> rehearse it her.<rdquo/  <rj><au>Chaucer.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>As an auxiliary, <xex>shall</xex> indicates a duty or necessity whose obligation is derived from the person speaking; <as>as, you <ex>shall</ex> go; he <xex>shall</xex> go</as>; that is, I order or promise your going. It thus ordinarily expresses, in the second and third persons, a command, a threat, or a promise. If the auxillary be emphasized, the command is made more imperative, the promise or that more positive and sure. It is also employed in the language of prophecy; <as>as, <ldquo/the day <ex>shall</ex> come when . . . , <rdquo/</as> since a promise or threat and an authoritative prophecy nearly coincide in significance. In <xex>shall</xex> with the first person, the necessity of the action is sometimes implied as residing elsewhere than in the speaker; <as>as, I <ex>shall</ex> suffer; we <xex>shall</xex> see</as>; and there is always a less distinct and positive assertion of his volition than is indicated by <xex>will</xex>. <ldquo/I <xex>shall</xex> go<rdquo/ implies nearly a simple futurity; more exactly, a foretelling or an expectation of my going, in which, naturally enough, a certain degree of plan or intention may be included; emphasize the <xex>shall</xex>, and the event is described as certain to occur, and the expression approximates in meaning to our emphatic <ldquo/I <xex>will</xex> go.<rdquo/ In a question, the relation of speaker and source of obligation is of course transferred to the person addressed; as, <ldquo/<xex>Shall</xex> you go?<rdquo/ (answer, <ldquo/I <xex>shall</xex> go<rdquo/); <ldquo/<xex>Shall</xex> he go?<rdquo/ <it>i. e.</it>, <ldquo/Do you require or promise his going?<rdquo/ (answer, <ldquo/He <xex>shall</xex> go<rdquo/.) The same relation is transferred to either second or third person in such phrases as <ldquo/You say, or think, you <xex>shall</xex> go;<rdquo/ <ldquo/He says, or thinks, he <xex>shall</xex> go.<rdquo/  After a conditional conjunction (as <xex>if</xex>, whether) <xex>shall</xex> is used in all persons to express futurity simply; <as>as, if I, you, or he <ex>shall</ex> say they are right</as>.  <xex>Should</xex> is everywhere used in the same connection and the same senses as <xex>shall</xex>, as its imperfect. It also expresses duty or moral obligation; <as>as, he <ex>should</ex> do it whether he will or not</as>. In the early English, and hence in our English Bible, <ex>shall</ex> is the auxiliary mainly used, in all the persons, to express simple futurity. (Cf. <er>Will</er>, <pos>v. t.</pos>) <ex>Shall</ex> may be used elliptically; thus, with an adverb or other word expressive of motion <xex>go</xex> may be omitted.</def> <ldquo/He to England <xex>shall</xex> along with you.<rdquo/  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ <xex>Shall</xex> and <xex>will</xex> are often confounded by inaccurate speakers and writers. Say: I <xex>shall</xex> be glad to see you. <xex>Shall</xex> I do this? <xex>Shall</xex> I help you? (not <xex>Will</xex> I do this?) See <er>Will</er>.</note><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shalli</ent><br/
<hw>Shal"li</hw> <pr>(?)</pr>, <pos>n.</pos> <def>See <er>Challis</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shallon</ent><br/
<hw>Shal"lon</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Bot.)</fld> <def>An evergreen shrub (<spn>Gaultheria Shallon</spn>) of Northwest America; also, its fruit. See <er>Salal-berry</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shalloon</ent><br/
<hw>Shal*loon"</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F. <ets>chalon</ets>, from <ets>Ch<acir/lons</ets>, in France, where it was first made.]</ety> <def>A thin, loosely woven, twilled worsted stuff.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>In blue <qex>shalloon</qex> shall Hannibal be clad.</q> <rj><qau>Swift.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shallop</ent><br/
<hw>Shal"lop</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F. <ets>chaloupe</ets>, probably from D. <ets>sloep</ets>.  Cf. <er>Sloop</er>.]</ety> <fld>(Naut.)</fld> <def>A boat.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>[She] thrust the <qex>shallop</qex> from the floating strand.</q> <rj><qau>Spenser.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ The term <xex>shallop</xex> is applied to boats of all sizes, from a light canoe up to a large boat with masts and sails.</note><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shallot</ent><br/
<hw>Shal*lot"</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[OF. <ets>eschalote</ets> (for <ets>escalone</ets>), F. <ets><eacute/chalote</ets>. See <er>Scallion</er>, and cf. <er>Eschalot</er>.]</ety> <fld>(Bot.)</fld> <def>A small kind of onion (<spn>Allium Ascalonicum</spn>) growing in clusters, and ready for gathering in spring; a scallion, or eschalot.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shallow</ent><br/
<hw>Shal"low</hw> <pr>(?)</pr>, <pos>a.</pos> <amorph>[<pos>Compar.</pos> <adjf>Shallower</adjf> <pr>(?)</pr>; <pos>superl.</pos> <adjf>Shallowest</adjf>.]</amorph> <ety>[OE. <ets>schalowe</ets>, probably originally, sloping or shelving; cf. Icel. <ets>skj<amac/lgr</ets> wry, squinting, AS. <ets>sceolh</ets>, D. & G. <ets>scheel</ets>, OHG. <ets>schelah</ets>.  Cf. <er>Shelve</er> to slope, <er>Shoal</er> shallow.]</ety> <sn>1.</sn> <def>Not deep; having little depth; shoal.</def> <ldquo/<xex>Shallow</xex> brooks, and rivers wide.<rdquo/  <rj><au>Milton.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Not deep in tone.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>The sound perfecter and not so <qex>shallow</qex> and jarring.</q> <rj><qau>Bacon.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>Not intellectually deep; not profound; not penetrating deeply; simple; not wise or knowing; ignorant; superficial; <as>as, a <ex>shallow</ex> mind; <ex>shallow</ex> learning</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The king was neither so <qex>shallow</qex>, nor so ill advertised, as not to perceive the intention of the French king.</q> <rj><qau>Bacon.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Deep versed in books, and <qex>shallow</qex> in himself.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shallow</ent><br/
<hw>Shal"low</hw>, <pos>n.</pos> <sn>1.</sn> <def>A place in a body of water where the water is not deep; a shoal; a flat; a shelf.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>A swift stream is not heard in the channel, but upon <qex>shallows</qex> of gravel.</q> <rj><qau>Bacon.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Dashed on the <qex>shallows</qex> of the moving sand.</q> <rj><qau>Dryden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Zool.)</fld> <def>The rudd.</def> <mark>[Prov. Eng.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shallow</ent><br/
<hw>Shal"low</hw>, <pos>v. t.</pos> <def>To make shallow.</def>  <rj><au>Sir T. Browne.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shallow</ent><br/
<hw>Shal"low</hw>, <pos>v. i.</pos> <def>To become shallow, as water.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shallow-bodied</ent><br/
<hw>Shal"low-bod`ied</hw> <pr>(?)</pr>, <pos>a.</pos> <fld>(Naut.)</fld> <def>Having a moderate depth of hold; -- said of a vessel.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shallow-brained</ent><br/
<hw>Shal"low-brained`</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Weak in intellect; foolish; empty-headed.</def>  <rj><au>South.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shallow-hearted</ent><br/

<hw>Shal"low-heart`ed</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Incapable of deep feeling.</def>  <rj><au>Tennyson.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shallowly</ent><br/
<hw>Shal"low*ly</hw>, <pos>adv.</pos> <def>In a shallow manner.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shallowness</ent><br/
<hw>Shal"low*ness</hw>, <pos>n.</pos> <def>Quality or state of being shallow.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shallow-pated</ent><br/
<hw>Shal"low-pat`ed</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Shallow-brained.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shallow-waisted</ent><br/
<hw>Shal"low-waist`ed</hw> <pr>(?)</pr>, <pos>a.</pos> <fld>(Naut.)</fld> <def>Having a flush deck, or with only a moderate depression amidships; -- said of a vessel.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shalm</ent><br/
<hw>Shalm</hw> <pr>(sh<add/m)</pr>, <pos>n.</pos> <def>See <er>Shawm</er>.</def> <mark>[Obs.]</mark>  <rj><au>Knolles.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shalt</ent><br/
<hw>Shalt</hw> <pr>(sh<acr/lt)</pr>, <def><pos>2d per. sing.</pos> of <er>Shall</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shaly</ent><br/
<hw>Shal"y</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Resembling shale in structure.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sham</ent><br/
<hw>Sham</hw> <pr>(sh<acr/m)</pr>, <pos>n.</pos> <ety>[Originally the same word as <ets>shame</ets>, hence, a disgrace, a trick. See <er>Shame</er>, <pos>n.</pos>]</ety> <sn>1.</sn> <def>That which deceives expectation; any trick, fraud, or device that deludes and disappoints; a make-believe; delusion; imposture; humbug.</def> <ldquo/A mere <xex>sham</xex>.<rdquo/  <rj><au>Bp. Stillingfleet.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Believe who will the solemn <qex>sham</qex>, not I.</q> <rj><qau>Addison.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A false front, or removable ornamental covering.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Pillow sham</b></col>, <cd>a covering to be laid on a pillow.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sham</ent><br/
<hw>Sham</hw>, <pos>a.</pos> <def>False; counterfeit; pretended; feigned; unreal; <as>as, a <ex>sham</ex> fight</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>They scorned the <qex>sham</qex> independence proffered to them by the Athenians.</q> <rj><qau>Jowett (Thucyd)</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sham</ent><br/
<hw>Sham</hw>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Shammed</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Shamming</conjf>.]</vmorph> <sn>1.</sn> <def>To trick; to cheat; to deceive or delude with false pretenses.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Fooled and <qex>shammed</qex> into a conviction.</q> <rj><qau>L'Estrange.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To obtrude by fraud or imposition.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>We must have a care that we do not . . . <qex>sham</qex> fallacies upon the world for current reason.</q> <rj><qau>L'Estrange.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>To assume the manner and character of; to imitate; to ape; to feign.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><mcol><col><b>To sham Abram</b></col> <it>or</it> <col><b>To sham Abraham</b></col></mcol>, <cd>to feign sickness; to malinger. Hence a malingerer is called, in sailors' cant, <xex>Sham Abram</xex>, or <xex>Sham Abraham</xex>.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sham</ent><br/
<hw>Sham</hw>, <pos>v. i.</pos> <def>To make false pretenses; to deceive; to feign; to impose.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Wondering . . . whether those who lectured him were such fools as they professed to be, or were only <qex>shamming</qex>.</q> <rj><qau>Macaulay.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shama</ent><br/
||<hw>Sha"ma</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Hind. <ets>sh<amac/m<amac/</ets>.]</ety> <fld>(Zool.)</fld> <def>A saxicoline singing bird (<spn>Kittacincla macroura</spn>) of India, noted for the sweetness and power of its song. In confinement it imitates the notes of other birds and various animals with accuracy. Its head, neck, back, breast, and tail are glossy black, the rump white, the under parts chestnut.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shaman</ent><br/
<hw>Sha"man</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[From the native name.]</ety> <def>A priest of Shamanism; a wizard among the Shamanists.</def><br/
[<source>1913 Webster</source>]</p>

<p><-- p. 1323 --></p>

<p><ent>Shamanic</ent><br/
<hw>Sha*man"ic</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Of or pertaining to Shamanism.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shamanism</ent><br/
<hw>Sha"man*ism</hw> <pr>(?)</pr>, <pos>n.</pos> <def>The type of religion which once prevalied among all the Ural-Altaic peoples (Tungusic, Mongol, and Turkish), and which still survives in various parts of Northern Asia. The Shaman, or wizard priest, deals with good as well as with evil spirits, especially the good spirits of ancestors.</def>  <rj><au>Encyc. Brit.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shamanist</ent><br/
<hw>Sha"man*ist</hw>, <pos>n.</pos> <def>An adherent of Shamanism.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shamble</ent><br/
<hw>Sham"ble</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[OE. <ets>schamel</ets> a bench, stool, AS. <ets>scamel</ets>, <ets>sceamol</ets>, a bench, form, stool, fr. L. <ets>scamellum</ets>, dim. of <ets>scamnum</ets> a bench, stool.]</ety> <sn>1.</sn> <fld>(Mining)</fld> <def>One of a succession of niches or platforms, one above another, to hold ore which is thrown successively from platform to platform, and thus raised to a higher level.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <pluf>pl.</pluf> <def>A place where butcher's meat is sold.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>As summer flies are in the <qex>shambles</qex>.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <pluf>pl.</pluf> <def>A place for slaughtering animals for meat.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>To make a <qex>shambles</qex> of the parliament house.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shamble</ent><br/
<hw>Sham"ble</hw>, <pos>v. i.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Shambled</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Shambling</conjf> <pr>(?)</pr>.]</vmorph> <ety>[Cf. OD. <ets>schampelen</ets> to slip, <ets>schampen</ets> to slip away, escape.  Cf. <er>Scamble</er>, <er>Scamper</er>.]</ety> <def>To walk awkwardly and unsteadily, as if the knees were weak; to shuffle along.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shambling</ent><br/
<hw>Sham"bling</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Characterized by an awkward, irregular pace; <as>as, a <ex>shambling</ex> trot; <ex>shambling</ex> legs.</as></def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shambling</ent><br/
<hw>Sham"bling</hw>, <pos>n.</pos> <def>An awkward, irregular gait.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shame</ent><br/
<hw>Shame</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[OE. <ets>shame</ets>, <ets>schame</ets>, AS. <ets>scamu</ets>, <ets>sceamu</ets>; akin to OS. & OHG. <ets>scama</ets>, G. <ets>scham</ets>, Icel. <ets>sk<oum/mm</ets>, <ets>shkamm</ets>, Sw. & Dan. <ets>skam</ets>, D. & G. <ets>schande</ets>, Goth. <ets>skanda</ets> shame, <ets>skaman sik</ets> to be ashamed; perhaps from a root <ets>skam</ets> meaning to cover, and akin to the root (<ets>kam</ets>) of G. <ets>hemd</ets> shirt, E. <ets>chemise</ets>.  Cf. <er>Sham</er>.]</ety> <sn>1.</sn> <def>A painful sensation excited by a consciousness of guilt or impropriety, or of having done something which injures reputation, or of the exposure of that which nature or modesty prompts us to conceal.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>HIde, for <qex>shame</qex>,<br/
Romans, your grandsires' images,<br/
That blush at their degenerate progeny.</q> <rj><qau>Dryden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Have you no modesty, no maiden <qex>shame</qex>?</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Reproach incurred or suffered; dishonor; ignominy; derision; contempt.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Ye have borne the <qex>shame</qex> of the heathen.</q> <rj><qau>Ezek. xxxvi. 6.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Honor and <qex>shame</qex> from no condition rise.</q> <rj><qau>Pope.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>And every woe a tear can claim<br/
Except an erring sister's <qex>shame</qex>.</q> <rj><qau>Byron.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>The cause or reason of shame; that which brings reproach, and degrades a person in the estimation of others; disgrace.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>O C<?/sar, what a wounding <qex>shame</qex> is this!</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Guides who are the <qex>shame</qex> of religion.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>The parts which modesty requires to be covered; the private parts.</def>  <rj><au>Isa. xlvii. 3.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>For shame!</b></col> <cd>you should be ashamed; shame on you!</cd> -- <col><b>To put to shame</b></col>, <cd>to cause to feel shame; to humiliate; to disgrace.</cd> <ldquo/Let them be driven backward and <xex>put to shame</xex> that wish me evil.<rdquo/  <rj><au>Ps. xl. 14.</au></rj>
</cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shame</ent><br/
<hw>Shame</hw>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Shamed</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Shaming</conjf>.]</vmorph> <sn>1.</sn> <def>To make ashamed; to excite in (a person) a comsciousness of guilt or impropriety, or of conduct derogatory to reputation; to put to shame.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Were there but one righteous in the world, he would . . . <qex>shame</qex> the world, and not the world him.</q> <rj><qau>South.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To cover with reproach or ignominy; to dishonor; to disgrace.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>And with foul cowardice his carcass <qex>shame</qex>.</q> <rj><qau>Spenser.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>To mock at; to deride.</def> <mark>[Obs. or R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>Ye have <qex>shamed</qex> the counsel of the poor.</q> <rj><qau>Ps. xiv. 6.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shame</ent><br/
<hw>Shame</hw>, <pos>v. i.</pos> <ety>[AS. <ets>scamian</ets>, <ets>sceamian</ets>. See <er>Shame</er>, <pos>n.</pos>]</ety> <def>To be ashamed; to feel shame.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>I do <qex>shame</qex><br/
To think of what a noble strain you are.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shamefaced</ent><br/
<hw>Shame"faced`</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[For <ets>shamefast</ets>; AS. <ets>scamfaest</ets>. See <er>Shame</er>, <pos>n.</pos>, and <er>Fast</er> firm.]</ety> <def>Easily confused or put out of countenance; diffident; bashful; modest.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Your <qex>shamefaced</qex> virtue shunned the people's prise.</q> <rj><qau>Dryden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ <xex>Shamefaced</xex> was once <xex>shamefast</xex>, <xex>shamefacedness</xex> was <xex>shamefastness</xex>, like <xex>steadfast</xex> and <xex>steadfastness</xex>; but the ordinary manifestations of shame being by the face, have brought it to its present orthography.  <au>Trench.</au></note><br/
[<source>1913 Webster</source>]</p>

<p>-- <wordforms><wf>Shame"faced</wf>, <pos>adv.</pos> -- <wf>Shame"faced`ness</wf>, <pos>n.</pos></wordforms><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shamefast</ent><br/
<hw>Shame"fast</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[AS. <ets>scamf<ae/st</ets>.]</ety> <def>Modest; shamefaced.</def> -- <wordforms><wf>Shame"fast*ly</wf>, <pos>adv.</pos> -- <wf>Shame"fast*ness</wf>, <pos>n.</pos> <mark>[Archaic]</mark> See <er>Shamefaced</er>.</wordforms><br/
[<source>1913 Webster</source>]</p>

<p><q><qex>Shamefast</qex> she was in maiden <qex>shamefastness</qex>.</q> <rj><qau>Chaucer.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>[Conscience] is a blushing <qex>shamefast</qex> spirit.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Modest apparel with <qex>shamefastness</qex>.</q> <rj><qau>1 Tim. ii. 9 (Rev. Ver.).</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shameful</ent><br/
<hw>Shame"ful</hw> <pr>(?)</pr>, <pos>a.</pos> <sn>1.</sn> <def>Bringing shame or disgrace; injurious to reputation; disgraceful.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>His naval preparations were not more surprising than his quick and <qex>shameful</qex> retreat.</q> <rj><qau>Arbuthnot.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Exciting the feeling of shame in others; indecent; <as>as, a <ex>shameful</ex> picture; a <ex>shameful</ex> sight.</as></def>  <rj><au>Spenser.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><syn><b>Syn.</b> -- Disgraceful; reproachful; indecent; unbecoming; degrading; scandalous; ignominious; infamous.</syn><br/
[<source>1913 Webster</source>]</p>

<p>-- <wordforms><wf>Shame"ful*ly</wf>, <pos>adv.</pos> -- <wf>Shame"ful*ness</wf>, <pos>n.</pos></wordforms><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shameless</ent><br/
<hw>Shame"less</hw>, <pos>a.</pos> <ety>[AS. <ets>scamle<aacute/s</ets>.]</ety> <sn>1.</sn> <def>Destitute of shame; wanting modesty; brazen-faced; insensible to disgrace.</def> <ldquo/Such <xex>shameless</xex> bards we have.<rdquo/  <rj><au>Pope.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Shame enough to shame thee, wert thou not <qex>shameless</qex>.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Indicating want of modesty, or sensibility to disgrace; indecent; <as>as, a <ex>shameless</ex> picture or poem</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><syn><b>Syn.</b> -- Impudent; unblushing; audacious; immodest; indecent; indelicate.</syn><br/
[<source>1913 Webster</source>]</p>

<p>-- <wordforms><wf>Shame"less*ly</wf>, <pos>adv.</pos> -- <wf>Shame"less*ness</wf>, <pos>n.</pos></wordforms><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shame-proof</ent><br/
<hw>Shame"-proof`</hw> <pr>(?)</pr>, <pos>n.</pos> <def>Shameless.</def>  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shamer</ent><br/
<hw>Sham"er</hw> <pr>(?)</pr>, <pos>n.</pos> <def>One who, or that which, disgraces, or makes ashamed.</def>  <rj><au>Beau. & Fl.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shammer</ent><br/
<hw>Sham"mer</hw> <pr>(?)</pr>, <pos>n.</pos> <def>One who shams; an impostor.</def>  <rj><au>Johnson.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shammy</ent><br/
<hw>Sham"my</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F. <ets>chamious</ets> a chamois, shammy leather. See <er>Chamois</er>.]</ety> <sn>1.</sn> <fld>(Zool.)</fld> <def>The chamois.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A soft, pliant leather, prepared originally from the skin of the chamois, but now made also from the skin of the sheep, goat, kid, deer, and calf. See <er>Shamoying</er>.</def> <altsp>[Written also <asp>chamois</asp>, <asp>shamoy</asp>, and <asp>shamois</asp>.]</altsp><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shamoy</ent><br/
<ent>Shamois</ent><br/
<mhw>{ <hw>Sham"ois</hw>, <hw>Sham"oy</hw>  }</mhw> <pr>(?)</pr>, <pos>n.</pos> <def>See <er>Shammy</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shamoying</ent><br/
<hw>Sha*moy"ing</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[See <er>Shammy</er>.]</ety> <def>A process used in preparing certain kinds of leather, which consists in frizzing the skin, and working oil into it to supply the place of the astringent (tannin, alum, or the like) ordinarily used in tanning.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shampoo</ent><br/
<hw>Sham*poo"</hw> <pr>(?)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Shampooed</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Shampooing</conjf>.]</vmorph> <ety>[Hind. <ets>ch<amac/mpn<amac/</ets> to press, to squeeze.]</ety> <altsp>[Writing also <asp>champoo</asp>.]</altsp> <sn>1.</sn> <def>To press or knead the whole surface of the body of (a person), and at the same time to stretch the limbs and joints, in connection with the hot bath.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To wash throughly and rub the head of (a person), with the fingers, using either soap, or a soapy preparation, for the more thorough cleansing.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shampoo</ent><br/
<hw>Sham*poo"</hw>, <pos>n.</pos> <def>The act of shampooing.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shampooer</ent><br/
<hw>Sham*poo"er</hw> <pr>(?)</pr>, <pos>n.</pos> <def>One who shampoos.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shamrock</ent><br/
<hw>Sham"rock</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>seamrog</ets>, <ets>seamar</ets>, trefoil, white clover, white honeysuckle; akin to Gael. <ets>seamrag</ets>.]</ety> <fld>(Bot.)</fld> <def>A trifoliate plant used as a national emblem by the Irish. The legend is that St. Patrick once plucked a leaf of it for use in illustrating the doctrine of the trinity.</def><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ The original plant was probably a kind of wood sorrel (<spn>Oxalis Acetocella</spn>); but now the name is given to the white clover (<spn>Trifolium repens</spn>), and the black medic (<spn>Medicago lupulina</spn>).</note><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shandrydan</ent><br/
<hw>Shan"dry*dan</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A jocosely depreciative name for a vehicle.</def> <mark>[Ireland]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shandygaff</ent><br/
<hw>Shan"dy*gaff</hw> <pr>(<?/)</pr>, <pos>n.</pos> <def>A mixture of strong beer and ginger beer.</def> <mark>[Eng.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shanghai</ent><br/
<hw>Shang`hai"</hw> <pr>(?)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Shanghaied</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Shanghaiing</conjf>.]</vmorph> <def>To intoxicate and ship (a person) as a sailor while in this condition.</def> <altsp>[Written also <asp>shanghae</asp>.]</altsp> <mark>[Slang, U.S.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shanghai</ent><br/
<hw>Shang`hai"</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Zool.)</fld> <def>A large and tall breed of domestic fowl.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shank</ent><br/
<hw>Shank</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Zool.)</fld> <def>See <er>Chank</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shank</ent><br/
<hw>Shank</hw>, <pos>n.</pos> <ety>[OE. <ets>shanke</ets>, <ets>schanke</ets>, <ets>schonke</ets>, AS. <ets>scanca</ets>, <ets>sceanca</ets>, <ets>sconca</ets>, <ets>sceonca</ets>; akin to D. <ets>schonk</ets> a bone, G. <ets>schenkel</ets> thigh, shank, <ets>schinken</ets> ham, OHG. <ets>scincha</ets> shank, Dan. & Sw. <ets>skank</ets>. <root/161.  Cf. <er>Skink</er>, <pos>v.</pos>]</ety> <sn>1.</sn> <def>The part of the leg from the knee to the foot; the shin; the shin bone; also, the whole leg.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>His youthful hose, well saved, a world too wide<br/
For his shrunk <qex>shank</qex>.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Hence, that part of an instrument, tool, or other thing, which connects the acting part with a handle or other part, by which it is held or moved.</def> Specifically: <sd>(a)</sd> <def>That part of a key which is between the bow and the part which enters the wards of the lock.</def> <sd>(b)</sd> <def>The middle part of an anchor, or that part which is between the ring and the arms.</def> See <xex>Illustr.</xex> of <er>Anchor</er>. <sd>(c)</sd> <def>That part of a hoe, rake, knife, or the like, by which it is secured to a handle.</def> <sd>(d)</sd> <def>A loop forming an eye to a button.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <fld>(Arch.)</fld> <def>The space between two channels of the Doric triglyph.</def>  <rj><au>Gwilt.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <fld>(Founding)</fld> <def>A large ladle for molten metal, fitted with long bars for handling it.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <fld>(Print.)</fld> <def>The body of a type.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>6.</sn> <fld>(Shoemaking)</fld> <def>The part of the sole beneath the instep connecting the broader front part with the heel.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>7.</sn> <fld>(Zool.)</fld> <def>A wading bird with long legs; <as>as, the green-legged <ex>shank</ex>, or knot; the yellow <ex>shank</ex>, or tattler</as>; -- called also <altname>shanks</altname>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>8.</sn> <pluf>pl.</pluf> <def>Flat-nosed pliers, used by opticians for nipping off the edges of pieces of glass to make them round.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Shank painter</b></col> <fld>(Naut.)</fld>, <cd>a short rope or chain which holds the shank of an anchor against the side of a vessel when it is secured for a voyage.</cd> -- <col><b>To ride shank's mare</b></col>, <cd>to go on foot; to walk.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shank</ent><br/
<hw>Shank</hw>, <pos>v. i.</pos> <def>To fall off, as a leaf, flower, or capsule, on account of disease affecting the supporting footstalk; -- usually followed by <xex>off</xex>.</def>  <rj><au>Darwin.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shankbeer</ent><br/
<hw>Shank"beer`</hw> <pr>(?)</pr>, <pos>n.</pos> <def>See <er>Schenkbeer</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shanked</ent><br/
<hw>Shanked</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Having a shank.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shanker</ent><br/
<hw>Shank"er</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Med.)</fld> <def>See <er>Chancre</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shanny</ent><br/
<hw>Shan"ny</hw> <pr>(?)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Shannies</plw> <pr>(#)</pr>.</plu> <ety>[Etymol. uncertain.]</ety> <fld>(Zool.)</fld> <def>The European smooth blenny (<spn>Blennius pholis</spn>). It is olive-green with irregular black spots, and without appendages on the head.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shan't</ent><br/
<hw>Shan't</hw> <pr>(?)</pr>. <def>A contraction of <xex>shall not</xex>.</def> <mark>[Colloq.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shanty</ent><br/
<hw>Shan"ty</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Jaunty; showy.</def> <mark>[Prov. Eng.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shanty</ent><br/
<hw>Shan"ty</hw>, <pos>n.</pos>;<plu><it>pl.</it> <plw>Shanties</plw> <pr>(#)</pr>.</plu> <ety>[Said to be fr. Ir. <ets>sean</ets> old + <ets>tig</ets>. a house.]</ety> <def>A small, mean dwelling; a rough, slight building for temporary use; a hut.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shanty</ent><br/
<hw>Shan"ty</hw>, <pos>v. i.</pos> <def>To inhabit a shanty.</def>  <rj><au>S. H. Hammond.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shapable</ent><br/
<hw>Shap"a*ble</hw> <pr>(?)</pr>, <pos>a.</pos> <sn>1.</sn> <def>That may be shaped.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Shapely.</def> <mark>[R.]</mark> <ldquo/Round and <xex>shapable</xex>.<rdquo/  <rj><au>De Foe.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shape</ent><br/
<hw>Shape</hw> <pr>(sh<amac/p)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp.</pos> <conjf>Shaped</conjf> <pr>(sh<amac/pt)</pr>; <pos>p. p.</pos> <conjf>Shaped</conjf> or <conjf>Shapen</conjf> <pr>(sh<amac/p"'n)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Shaping</conjf>.]</vmorph> <ety>[OE. <ets>shapen</ets>, <ets>schapen</ets>, AS. <ets>sceapian</ets>. The p. p. <ets>shapen</ets> is from the strong verb, AS. <ets>scieppan</ets>, <ets>scyppan</ets>, <ets>sceppan</ets>, p. p. <ets>sceapen</ets>. See <er>Shape</er>, <pos>n.</pos>]</ety> <sn>1.</sn> <def>To form or create; especially, to mold or make into a particular form; to give proper form or figure to.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>I was <qex>shapen</qex> in iniquity.</q> <rj><qau>Ps. li. 5.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Grace <qex>shaped</qex> her limbs, and beauty decked her face.</q> <rj><qau>Prior.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To adapt to a purpose; to regulate; to adjust; to direct; <as>as, to <ex>shape</ex> the course of a vessel</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>To the stream, when neither friends, nor force,<br/
Nor speed nor art avail, he <qex>shapes</qex> his course.</q> <rj><qau>Denham.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Charmed by their eyes, their manners I acquire,<br/
And <qex>shape</qex> my foolishness to their desire.</q> <rj><qau>Prior.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>To imagine; to conceive; to call forth (ideas).</def> <mark>[archaic]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>Oft my jealousy<br/
<qex>Shapes</qex> faults that are not.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>To design; to prepare; to plan; to arrange.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>When <qex>shapen</qex> was all this conspiracy,<br/
From point to point.</q> <rj><qau>Chaucer.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Shaping machine</b></col>. <fld>(Mach.)</fld> <cd>Same as <er>Shaper</er>.</cd> -- <col><b>To shape one's self</b></col>, <cd>to prepare; to make ready.</cd> <mark>[Obs.]</mark></cs><br/
[<source>1913 Webster</source>]</p>

<p><q>I will early <qex>shape me</qex> therefor.</q> <rj><qau>Chaucer.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shape</ent><br/
<hw>Shape</hw> <pr>(?)</pr>, <pos>v. i.</pos> <def>To suit; to be adjusted or conformable.</def> <mark>[R.]</mark>  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shape</ent><br/
<hw>Shape</hw>, <pos>n.</pos> <ety>[OE. <ets>shap</ets>, <ets>schap</ets>, AS. <ets>sceap</ets> in <ets>gesceap</ets> creation, creature, fr. the root of <ets>scieppan</ets>, <ets>scyppan</ets>, <ets>sceppan</ets>, to shape, to do, to effect; akin to OS. gi<ets>skeppian</ets>, OFries. <ets>skeppa</ets>, D. <ets>scheppen</ets>, G. <ets>schaffen</ets>, OHG. <ets>scaffan</ets>, <ets>scepfen</ets>, <ets>skeffen</ets>, Icer. <ets>skapa</ets>, <ets>skepja</ets>, Dan. <ets>skabe</ets>, <ets>skaffe</ets>, Sw. <ets>skapa</ets>, <ets>skaffa</ets>, Goth. ga<ets>skapjan</ets>, and perhaps to E. <ets>shave</ets>, v.  Cf. <er>-ship</er>.]</ety> <sn>1.</sn> <def>Character or construction of a thing as determining its external appearance; outward aspect; make; figure; form; guise; <as>as, the <ex>shape</ex> of a tree; the <ex>shape</ex> of the head; an elegant <ex>shape</ex></as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>He beat me grievously, in the <qex>shape</qex> of a woman.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>That which has form or figure; a figure; an appearance; a being.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Before the gates three sat,<br/
On either side, a formidable <qex>shape</qex>.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>A model; a pattern; a mold.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>Form of embodiment, as in words; form, as of thought or conception; concrete embodiment or example, as of some quality.</def>  <rj><au>Milton.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <def>Dress for disguise; guise.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>Look better on this virgin, and consider<br/
This Persian <qex>shape</qex> laid by, and she appearing<br/
In a Greekish dress.</q> <rj><qau>Messinger.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>6.</sn> <fld>(Iron Manuf.)</fld> <sd>(a)</sd> <def>A rolled or hammered piece, as a bar, beam, angle iron, etc., having a cross section different from merchant bar.</def> <sd>(b)</sd> <def>A piece which has been roughly forged nearly to the form it will receive when completely forged or fitted.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>To take shape</b></col>, <cd>to assume a definite form.</cd></cs>
<-- in shape, having a good muscle tone; healthy.
   Get into shape, to exercise so as to acquire a good muscle tone. --><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shapeless</ent><br/
<hw>Shape"less</hw>, <pos>a.</pos> <def>Destitute of shape or regular form; wanting symmetry of dimensions; misshapen; -- opposed to <ant>shapely</ant>.</def> -- <wordforms><wf>Shape"less*ness</wf>, <pos>n.</pos></wordforms><br/
[<source>1913 Webster</source>]</p>

<p><q>The <qex>shapeless</qex> rock, or hanging precipice.</q> <rj><qau>Pope.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shapeliness</ent><br/
<hw>Shape"li*ness</hw> <pr>(?)</pr>, <pos>n.</pos> <def>The quality or state of being shapely.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shapely</ent><br/
<hw>Shape"ly</hw>, <pos>a.</pos> <amorph>[<pos>Compar.</pos> <adjf>Shapelier</adjf> <pr>(?)</pr>; <pos>superl.</pos> <adjf>Shapeliest</adjf>.]</amorph> <sn>1.</sn> <def>Well-formed; having a regular shape; comely; symmetrical.</def>  <rj><au>T. Warton.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Waste sandy valleys, once perplexed with thorn,<br/
The spiry fir and <qex>shapely</qex> box adorn.</q> <rj><qau>Pope.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Where the <qex>shapely</qex> column stood.</q> <rj><qau>Couper.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Fit; suitable.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q><qex>Shaply</qex> for to be an alderman.</q> <rj><qau>Chaucer.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shaper</ent><br/
<hw>Shap"er</hw> <pr>(?)</pr>, <pos>n.</pos> <sn>1.</sn> <def>One who shapes; <as>as, the <ex>shaper</ex> of one's fortunes</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The secret of those old <qex>shapers</qex> died with them.</q> <rj><qau>Lowell.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>That which shapes; a machine for giving a particular form or outline to an object.</def> Specifically; <sd>(a)</sd> <fld>(Metal Working)</fld> <def>A kind of planer in which the tool, instead of the work, receives a reciprocating motion, usually from a crank.</def> <sd>(b)</sd> <fld>(Wood Working)</fld> <def>A machine with a vertically revolving cutter projecting above a flat table top, for cutting irregular outlines, moldings, etc.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shapoo</ent><br/
<hw>Sha"poo</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Zool.)</fld> <def>The oorial.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shaps</ent><br/
<hw>Shaps</hw> <pr>(sh<acr/ps)</pr>, <pos>n. pl.</pos> <ety>[Shortened fr. <ets>chaparajos</ets>. Cf. <er>Chaps</er>.]</ety> <def>Chaparajos.</def> <mark>[Western U. S.]</mark></p>

<p><q>A pair of gorgeous buckskin <qex>shaps</qex>, embroidered up the sides and adorned with innumerable ermine skins.</q>  <rj><qau>The Century.</qau></rj><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Shard</ent><br/
<hw>Shard</hw> <pr>(sh<aum/rd)</pr>, <pos>n.</pos> <def>A plant; chard.</def> <mark>[Obs.]</mark>  <rj><au>Dryden.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shard</ent><br/
<hw>Shard</hw> <pr>(sh<aum/rd)</pr>, <pos>n.</pos> <ety>[AS. <ets>sceard</ets>, properly a p. p. from the root of <ets>scearn</ets> to shear, to cut; akin to D. <ets>schaard</ets> a fragment, G. <ets>scharte</ets> a notch, Icel. <ets>skar<edh/</ets>. See <er>Shear</er>, and cf. <er>Sherd</er>.]</ety> <altsp>[Written also <asp>sheard</asp>, and <asp>sherd</asp>.]</altsp> <sn>1.</sn> <def>A piece or fragment of an earthen vessel, or a like brittle substance, as the shell of an egg or snail.</def>  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>The precious dish<br/
Broke into <qex>shards</qex> of beauty on the board.</q> <rj><qau>E. Arnold.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Zool.)</fld> <def>The hard wing case of a beetle.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>They are his <qex>shards</qex>, and he their beetle.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>A gap in a fence.</def> <mark>[Obs.]</mark>  <rj><au>Stanyhurst.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>A boundary; a division.</def> <mark>[Obs. & R.]</mark>  <rj><au>Spenser.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shard-borne</ent><br/
<hw>Shard"-borne`</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Borne on shards or scaly wing cases.</def> <ldquo/The <xex>shard-borne</xex> beetle.<rdquo/  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sharded</ent><br/
<hw>Shard"ed</hw>, <pos>a.</pos> <fld>(Zool.)</fld> <def>Having elytra, as a beetle.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shardy</ent><br/
<hw>Shard"y</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Having, or consisting of, shards.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Share</ent><br/
<hw>Share</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[OE. <ets>schar</ets>, AS. <ets>scear</ets>; akin to OHG. <ets>scaro</ets>, G. <ets>schar</ets>, pflug<ets>shar</ets>, and E. <ets>shear</ets>, v. See <er>Shear</er>.]</ety> <sn>1.</sn> <def>The part (usually an iron or steel plate) of a plow which cuts the ground at the bottom of a furrow; a plowshare.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>The part which opens the ground for the reception of the seed, in a machine for sowing seed.</def>  <rj><au>Knight.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Share</ent><br/
<hw>Share</hw>, <pos>n.</pos> <ety>[OE. <ets>share</ets>, AS. <ets>scearu</ets>, <ets>scaru</ets>, fr. <ets>sceran</ets> to shear, cut. See <er>Shear</er>, <pos>v.</pos>]</ety> <sn>1.</sn> <def>A certain quantity; a portion; a part; a division; <as>as, a small <ex>share</ex> of prudence</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Especially, the part allotted or belonging to one, of any property or interest owned by a number; a portion among others; an apportioned lot; an allotment; a dividend.</def> <ldquo/My <xex>share</xex> of fame.<rdquo/  <rj><au>Dryden.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>Hence, one of a certain number of equal portions into which any property or invested capital is divided; <as>as, a ship owned in ten <ex>shares</ex></as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>The pubes; the sharebone.</def> <mark>[Obs.]</mark>  <rj><au>Holland.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>To go shares</b></col>, <cd>to partake; to be equally concerned.</cd> -- <col><b>Share and share alike</b></col>, <cd>in equal shares.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Share</ent><br/
<hw>Share</hw>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Shared</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Sharing</conjf>.]</vmorph> <sn>1.</sn> <def>To part among two or more; to distribute in portions; to divide.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Suppose I <qex>share</qex> my fortune equally between my children and a stranger.</q> <rj><qau>Swift.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><-- p. 1324 --></p>

<p><sn>2.</sn> <def>To partake of, use, or experience, with others; to have a portion of; to take and possess in common; <as>as, to <ex>share</ex> a shelter with another</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q><qex>While avarice and rapine share</qex> the land.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>To cut; to shear; to cleave; to divide.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>The <qex>shared</qex> visage hangs on equal sides.</q> <rj><qau>Dryden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Share</ent><br/
<hw>Share</hw> <pr>(?)</pr>, <pos>v. i.</pos> <def>To have part; to receive a portion; to partake, enjoy, or suffer with others.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>A right of inheritance gave every one a title to <qex>share</qex> in the goods of his father.</q> <rj><qau>Locke.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sharebeam</ent><br/
<hw>Share"beam`</hw> <pr>(?)</pr>, <pos>n.</pos> <def>The part of the plow to which the share is attached.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sharebone</ent><br/
<hw>Share"bone`</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Anat.)</fld> <def>The pubic bone.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sharebroker</ent><br/
<hw>Share"bro`ker</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A broker who deals in railway or other shares and securities.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sharecropper</ent><br/
<hw>Share"crop`per</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A farmer who rents a parcel of land from its owner and pays a portion of the crop as the rent.</def><br/
[<source>PJC</source>]</p>

<p><ent>Shareholder</ent><br/
<hw>Share"hold`er</hw> <pr>(?)</pr>, <pos>n.</pos> <def>One who holds or owns a share or shares in a joint fund or property.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sharer</ent><br/
<hw>Shar"er</hw> <pr>(sh<acir/r"<etil/r)</pr>, <pos>n.</pos> <def>One who shares; a participator; a partaker; also, a divider; a distributer.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sharewort</ent><br/
<hw>Share"wort`</hw> <pr>(sh<acir/r"w<ucir/rt`)</pr>, <pos>n.</pos> <fld>(Bot.)</fld> <def>A composite plant (<spn>Aster Tripolium</spn>) growing along the seacoast of Europe.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shark</ent><br/
<hw>Shark</hw> <pr>(sh<aum/rk)</pr>, <pos>n.</pos> <ety>[Of uncertain origin; perhaps through OF. fr. <ets>carcharus</ets> a kind of dogfish, Gr. <grk>karchari`as</grk>, so called from its sharp teeth, fr. <grk>ka`rcharos</grk> having sharp or jagged teeth; or perhaps named from its rapacity (cf. <er>Shark</er>, <pos>v. t. & i.</pos>); cf. Corn. <ets>scarceas</ets>.]</ety> <sn>1.</sn> <fld>(Zool.)</fld> <def>Any one of numerous species of elasmobranch fishes of the order <ord>Plagiostomi</ord>, found in all seas.</def><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ Some sharks, as the basking shark and the whale shark, grow to an enormous size, the former becoming forty feet or more, and the latter sixty feet or more, in length. Most of them are harmless to man, but some are exceedingly voracious.  The man-eating sharks mostly belong to the genera <gen>Carcharhinus</gen>, <gen>Carcharodon</gen>, and related genera. They have several rows of large sharp teeth with serrated edges, as the great white shark (<spn>Carcharodon carcharias</spn> <it>or</it> <spn>Carcharodon Rondeleti</spn>) of tropical seas, and the great blue shark (<spn>Carcharhinus glaucus</spn> syn. <spn>Prionace glauca</spn>) of all tropical and temperate seas.  The former sometimes becomes thirty-six feet long, and is the most voracious and dangerous species known.  The rare man-eating shark of the United States coast (<spn>Carcharodon Atwoodi</spn>) is thought by some to be a variety, or the young, of <spn>Carcharodon carcharias</spn>.  The dusky shark (<spn>Carcharhinus obscurus</spn>) is a common species on the coast of the United States of moderate size and not dangerous.  It feeds on shellfish and bottom fishes.</note><br/
[<source>1913 Webster</source>]</p>

<p><note>The original 1913 Webster also mentioned a <ldquo/smaller blue shark (<spn>C. caudatus</spn>)</rdquo/, but this species could not be found mentioned on the Web (August 2002).  The following is a list of Atlantic Ocean sharks:<br/
  *  *  *  *  *  *  *  *  *  *  *  *  *  *  * *<br/
Common and Scientific Names of Atlantic Sharks<br/
  *  *  *  *  *  *  *  *  *  *  *  *  *  *  * *<br/
from  <ldquo/Our Living Oceans 1995<rdquo/ (published by the National Printing Office):<br/
NMFS. 1999. Our Living Oceans. Report on the status of U.S. living marine resources, 1999. U.S. Dep. Commer., NOAA Tech. Memo. NMFS-F/SPO-41, on-line version, http://spo.nwr.noaa.gov/olo99.htm.<br/
<br/
(the following list is found at at http://spo.nwr.noaa.gov/app5.pdf)
<br/
(1) Pelagic Sharks<br/
<br/
Thresher shark  (<spn>Alopias vulpinus</spn>)<br/
Bigeye thresher  (<spn>Alopias superciliosus</spn>)<br/
Oceanic whitetip shark  (<spn>Carcharhinus longimanus</spn>)<br/
Sevengill shark  (<spn>Heptrachias perlo</spn>)<br/
Sixgill shark  (<spn>Hexanchus griseus</spn>)<br/
Bigeye sixgill shark  (<spn>Hexanchus vitulus</spn>)<br/
Shortfin mako  (<spn>Isurus oxyrinchus</spn>)<br/
Longfin mako  (<spn>Isurus paucus</spn>)<br/
Porbeagle  (<spn>Lamna nasus</spn>)<br/
Blue shark  (<spn>Prionace glauca</spn>)<br/
<br/
(2)Large Coastal Sharks<br/
<br/
Sandbar shark  (<spn>Carcharhinus plumbeus</spn>)<br/
Reef shark  (<spn>Carcharhinus perezi</spn>)<br/
Blacktip shark  (<spn>Carcharhinus limbatus</spn>)<br/
Dusky shark  (<spn>Carcharhinus obscurus</spn>)<br/
Spinner shark  (<spn>Carcharhinus brevipinna</spn>)<br/
Silky shark  (<spn>Carcharhinus falciformis</spn>)<br/
Bull shark  (<spn>Carcharhinus leucas</spn>)<br/
Bignose shark  (<spn>Carcharhinus altimus</spn>)<br/
Galapagos shark  (<spn>Carcharhinus galapagensis</spn>)<br/
Night shark  (<spn>Carcharhinus signatus</spn>)<br/
White shark  (<spn>Carcharodon carcharias</spn>)<br/
Basking shark  (<spn>Cetorhinus maximus</spn>)<br/
Tiger shark  (<spn>Galeocerdo cuvier</spn>)<br/
Nurse shark  (<spn>Ginglymostoma cirratum</spn>)<br/
Lemon shark  (<spn>Negaprion brevirostris</spn>)<br/
Ragged-tooth shark  (<spn>Odontaspis ferox</spn>)<br/
Whale shark  (<spn>Rhincodon typus</spn>)<br/
Scalloped hammerhead  (<spn>Sphyrna lewini</spn>)<br/
Great hammerhead  (<spn>Sphyrna mokarran</spn>)<br/
Smooth hammerhead  (<spn>Sphyrna zygaena</spn>)<br/
<br/
(3) Small Coastal Sharks<br/
<br/
Finetooth shark  (<spn>Carcharhinus isodon</spn>)<br/
Blacknose shark  (<spn>Carcharhinus acronotus</spn>)<br/
Atlantic sharpnose shark  (<spn>Rhizoprionodon erraenovae</spn>)<br/
Caribbean sharpnose shark  (<spn>Rhizoprionodon porosus</spn>)<br/
Bonnethead  (<spn>Sphyrna tiburo</spn>)<br/
Atlantic angel shark  (<spn>Squatina dumeril</spn>)<br/
</note><br/
[<source>PJC</source>]</p>

<p><sn>2.</sn> <def>A rapacious, artful person; a sharper.</def> <mark>[Colloq.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>Trickery; fraud; petty rapine; <as>as, to live upon the <ex>shark</ex></as>.</def> <mark>[Obs.]</mark>  <rj><au>South.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><mcol><col><b>Basking shark</b></col>, <col><b>Liver shark</b></col>, <col><b>Nurse shark</b></col>, <col><b>Oil shark</b></col>, <col><b>Sand shark</b></col>, <col><b>Tiger shark</b></col>, etc.</mcol> <cd>See under <er>Basking</er>, <er>Liver</er>, etc.  See also <er>Dogfish</er>, <er>Houndfish</er>, <er>Notidanian</er>, and <er>Tope</er>.</cd> -- <col><b>Gray shark</b></col>, <cd>the sand shark.</cd> -- <col><b>Hammer-headed shark</b></col>. <cd>See <er>Hammerhead</er>.</cd> -- <col><b>Port Jackson shark</b></col>.  <cd>See <er>Cestraciont</er>.</cd> -- <col><b>Shark barrow</b></col>, <cd>the eggcase of a shark; a sea purse.</cd> -- <col><b>Shark ray</b></col>. <cd>Same as <spn>Angel fish</spn> <sd>(a)</sd>, under <er>Angel</er>.</cd> -- <mcol><col><b>Thrasher shark</b></col> <it>or</it> <col><b>Thresher shark</b></col></mcol>, <cd>a large, voracious shark. See <er>Thrasher</er>.</cd> -- <col><b>Whale shark</b></col>, <cd>a huge harmless shark (<spn>Rhinodon typicus</spn>) of the Indian Ocean.  It becomes sixty feet or more in length, but has very small teeth.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shark</ent><br/
<hw>Shark</hw>, <pos>v. t.</pos> <ety>[Of uncertain origin; perhaps fr. <ets>shark</ets>, n., or perhaps related to E. <ets>shear</ets> (as <ets>hearken</ets> to <ets>hear</ets>), and originally meaning, to clip off.  Cf. <er>Shirk</er>.]</ety> <def>To pick or gather indiscriminately or covertly.</def> <mark>[Obs.]</mark>  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shark</ent><br/
<hw>Shark</hw>, <pos>v. i.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Sharked</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Sharking</conjf>.]</vmorph> <sn>1.</sn> <def>To play the petty thief; to practice fraud or trickery; to swindle.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Neither <qex>sharks</qex> for a cup or a reckoning.</q> <rj><qau>Bp. Earle.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To live by shifts and stratagems.</def>  <rj><au>Beau. & Fl.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sharker</ent><br/
<hw>Shark"er</hw> <pr>(?)</pr>, <pos>n.</pos> <def>One who lives by sharking.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sharking</ent><br/
<hw>Shark"ing</hw>, <pos>n.</pos> <def>Petty rapine; trick; also, seeking a livelihood by shifts and dishonest devices.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sharock</ent><br/
<hw>Shar"ock</hw> <pr>(?)</pr>, <pos>n.</pos> <def>An East Indian coin of the value of 12<frac12/ pence sterling, or about 25 cents.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sharp</ent><br/
<hw>Sharp</hw> <pr>(?)</pr>, <pos>a.</pos> <amorph>[<pos>Compar.</pos> <adjf>Sharper</adjf> <pr>(?)</pr>; <pos>superl.</pos> <adjf>Sharpest</adjf>.]</amorph> <ety>[OE. <ets>sharp</ets>, <ets>scharp</ets>, <ets>scarp</ets>, AS. <ets>scearp</ets>; akin to OS. <ets>skarp</ets>, LG. <ets>scharp</ets>, D. <ets>scherp</ets>, G. <ets>scharf</ets>, Dan. & Sw. <ets>skarp</ets>, Icel. <ets>skarpr</ets>.  Cf. <er>Escarp</er>, <er>Scrape</er>, <er>Scorpion</er>.]</ety> <sn>1.</sn> <def>Having a very thin edge or fine point; of a nature to cut or pierce easily; not blunt or dull; keen.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>He dies upon my scimeter's <qex>sharp</qex> point.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Terminating in a point or edge; not obtuse or rounded; somewhat pointed or edged; peaked or ridged; <as>as, a <ex>sharp</ex> hill; <ex>sharp</ex> features.</as></def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>Affecting the sense as if pointed or cutting, keen, penetrating, acute: to the taste or smell, pungent, acid, sour, as ammonia has a <xex>sharp</xex> taste and odor; to the hearing, piercing, shrill, as a <xex>sharp</xex> sound or voice; to the eye, instantaneously brilliant, dazzling, as a <xex>sharp</xex> flash.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <fld>(Mus.)</fld> <sd>(a)</sd> <def>High in pitch; acute; <as>as, a <ex>sharp</ex> note or tone</as>.</def> <sd>(b)</sd> <def>Raised a semitone in pitch; <as>as, C <ex>sharp</ex> (C<sharp/), which is a half step, or semitone, higher than C</as>.</def> <sd>(c)</sd> <def>So high as to be out of tune, or above true pitch; <as>as, the tone is <ex>sharp</ex>; that instrument is <ex>sharp</ex></as>. Opposed in all these senses to <ant>flat</ant>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <def>Very trying to the feelings; piercing; keen; severe; painful; distressing; <as>as, <ex>sharp</ex> pain, weather; a <ex>sharp</ex> and frosty air</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q><qex>Sharp</qex> misery had worn him to the bones.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>The morning <qex>sharp</qex> and clear.</q> <rj><qau>Cowper.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>In <qex>sharpest</qex> perils faithful proved.</q> <rj><qau>Keble.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>6.</sn> <def>Cutting in language or import; biting; sarcastic; cruel; harsh; rigorous; severe; <as>as, a <ex>sharp</ex> rebuke</as>.</def> <ldquo/That <xex>sharp</xex> look.<rdquo/  <rj><au>Tennyson.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>To that place the <qex>sharp</qex> Athenian law<br/
Can not pursue us.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Be thy words severe,<br/
<qex>Sharp</qex> as merits but the sword forbear.</q> <rj><qau>Dryden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>7.</sn> <def>Of keen perception; quick to discern or distinguish; having nice discrimination; acute; penetrating; sagacious; clever; <as>as, a <ex>sharp</ex> eye; <ex>sharp</ex> sight, hearing, or judgment</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Nothing makes men <qex>sharper</qex> . . . than want.</q> <rj><qau>Addison.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Many other things belong to the material world, wherein the <qex>sharpest</qex> philosophers have never ye<?/ arrived at clear and distinct ideas.</q> <rj><qau>L. Watts.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>8.</sn> <def>Eager in pursuit; keen in quest; impatient for gratification; keen; <as>as, a <ex>sharp</ex> appetite</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>9.</sn> <def>Fierce; ardent; fiery; violent; impetuous.</def> <ldquo/In <xex>sharp</xex> contest of battle.<rdquo/  <rj><au>Milton.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>A <qex>sharp</qex> assault already is begun.</q> <rj><qau>Dryden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>10.</sn> <def>Keenly or unduly attentive to one's own interest; close and exact in dealing; shrewd; <as>as, a <ex>sharp</ex> dealer; a <ex>sharp</ex> customer.</as></def><br/
[<source>1913 Webster</source>]</p>

<p><q>The necessity of being so <qex>sharp</qex> and exacting.</q> <rj><qau>Swift.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>11.</sn> <def>Composed of hard, angular grains; gritty; <as>as, <ex>sharp</ex> sand</as>.</def>  <rj><au>Moxon.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>12.</sn> <def>Steep; precipitous; abrupt; <as>as, a <ex>sharp</ex> ascent or descent; a <ex>sharp</ex> turn or curve</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>13.</sn> <fld>(Phonetics)</fld> <def>Uttered in a whisper, or with the breath alone, without voice, as certain consonants, such as <xex>p</xex>, <xex>k</xex>, <xex>t</xex>, <xex>f</xex>; surd; nonvocal; aspirated.</def><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ <xex>Sharp</xex> is often used in the formation of self-explaining compounds; as, <xex>sharp</xex>-cornered, <xex>sharp</xex>-edged, <xex>sharp</xex>-pointed, <xex>sharp</xex>-tasted, <xex>sharp</xex>-visaged, etc.</note><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Sharp practice</b></col>, <cd>the getting of an advantage, or the attempt to do so, by a tricky expedient.</cd> -- <mcol><col><b>To brace sharp</b></col>, <it>or</it> <col><b>To sharp up</b></col></mcol> <fld>(Naut.)</fld>, <cd>to turn the yards to the most oblique position possible, that the ship may lie well up to the wind.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><syn><b>Syn.</b> -- Keen; acute; piercing; penetrating; quick; sagacious; discerning; shrewd; witty; ingenious; sour; acid; tart; pungent; acrid; severe; poignant; biting; acrimonious; sarcastic; cutting; bitter; painful; afflictive; violent; harsh; fierce; ardent; fiery.</syn><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sharp</ent><br/
<hw>Sharp</hw> <pr>(?)</pr>, <pos>adv.</pos> <sn>1.</sn> <def>To a point or edge; piercingly; eagerly; sharply.</def>  <rj><au>M. Arnold.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>The head [of a spear] full <qex>sharp</qex> yground.</q> <rj><qau>Chaucer.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>You bite so <qex>sharp</qex> at reasons.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Precisely; exactly; <as>as, we shall start at ten o'clock <ex>sharp</ex></as>.</def> <mark>[Colloq.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Look sharp</b></col>, <cd>attend; be alert.</cd> <mark>[Colloq.]</mark></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sharp</ent><br/
<hw>Sharp</hw>, <pos>n.</pos> <sn>1.</sn> <def>A sharp tool or weapon.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>If butchers had but the manners to go to <qex>sharps</qex>, gentlemen would be contented with a rubber at cuffs.</q> <rj><qau>Collier.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Mus.)</fld> <sd>(a)</sd> <def>The character [<sharp/] used to indicate that the note before which it is placed is to be raised a half step, or semitone, in pitch.</def> <sd>(b)</sd> <def>A sharp tone or note.</def> <au>Shak.</au><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>A portion of a stream where the water runs very rapidly.</def> <mark>[Prov. Eng.]</mark>  <rj><au>C. Kingsley.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>A sewing needle having a very slender point; a needle of the most pointed of the three grades, <xex>blunts</xex>, <xex>betweens</xex>, and <xex>sharps</xex>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <pluf>pl.</pluf> <def>Same as <er>Middlings</er>, 1.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>6.</sn> <def>An expert.</def> <mark>[Slang]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sharp</ent><br/
<hw>Sharp</hw>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Sharped</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Sharping</conjf>.]</vmorph> <sn>1.</sn> <def>To sharpen.</def> <mark>[Obs.]</mark>  <rj><au>Spenser.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Mus.)</fld> <def>To raise above the proper pitch; to elevate the tone of; especially, to raise a half step, or semitone, above the natural tone.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sharp</ent><br/
<hw>Sharp</hw>, <pos>v. i.</pos> <sn>1.</sn> <def>To play tricks in bargaining; to act the sharper.</def>  <rj><au>L'Estrange.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Mus.)</fld> <def>To sing above the proper pitch.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sharp-cut</ent><br/
<hw>Sharp-cut`</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Cut sharply or definitely, or so as to make a clear, well-defined impression, as the lines of an engraved plate, and the like; clear-cut; hence, having great distinctness; well-defined; clear.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sharpen</ent><br/
<hw>Sharp"en</hw> <pr>(?)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Sarpened</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Sharpening</conjf>.]</vmorph> <ety>[See <er>Sharp</er>, <pos>a.</pos>]</ety> <def>To make sharp.</def> Specifically: <sd>(a)</sd> <def>To give a keen edge or fine point to; to make sharper; <as>as, to <ex>sharpen</ex> an ax, or the teeth of a saw</as>.</def> <sd>(b)</sd> <def>To render more quick or acute in perception; to make more ready or ingenious.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The air . . . <qex>sharpened</qex> his visual ray<br/
To objects distant far.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>He that wrestles with us strengthens our nerves and <qex>sharpens</qex> our skill.</q> <rj><qau>Burke.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sd>(c)</sd> <def>To make more eager; <as>as, to <ex>sharpen</ex> men's desires</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Epicurean cooks<br/
<qex>Sharpen</qex> with cloyless sauce his appetite.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sd>(d)</sd> <def>To make more pungent and intense; <as>as, to <ex>sharpen</ex> a pain or disease</as>.</def> <sd>(e)</sd> <def>To make biting, sarcastic, or severe.</def> <ldquo/<xex>Sharpen</xex> each word.<rdquo/ <au>E. Smith.</au> <sd>(f)</sd> <def>To render more shrill or piercing.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Inclosures not only preserve sound, but increase and <qex>sharpen</qex> it.</q> <rj><qau>Bacon.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sd>(g)</sd> <def>To make more tart or acid; to make sour; <as>as, the rays of the sun <ex>sharpen</ex> vinegar</as>.</def> <sd>(h)</sd> <fld>(Mus.)</fld> <def>To raise, as a sound, by means of a sharp; to apply a sharp to.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sharpen</ent><br/
<hw>Sharp"en</hw>, <pos>v. i.</pos> <def>To grow or become sharp.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sharper</ent><br/
<hw>Sharp"er</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A person who bargains closely, especially, one who cheats in bargains; a swinder; also, a cheating gamester.</def><br/
[<source>1913 Webster</source>]</p>

<p><q><qex>Sharpers</qex>, as pikes, prey upon their own kind.</q> <rj><qau>L'Estrange.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><syn><b>Syn.</b> -- Swindler; cheat; deceiver; trickster; rogue. See <er>Swindler</er>.</syn><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sharpie</ent><br/
<hw>Sharp"ie</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Naut.)</fld> <def>A long, sharp, flat-bottomed boat, with one or two masts carrying a triangular sail. They are often called <altname>Fair Haven sharpies</altname>, after the place on the coast of Connecticut where they originated.</def> <mark>[Local, U.S.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sharpling</ent><br/
<hw>Sharp"ling</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Zool.)</fld> <def>A stickleback.</def> <mark>[Prov. Eng.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sharply</ent><br/
<hw>Sharp"ly</hw>, <pos>adv.</pos> <def>In a sharp manner,; keenly; acutely.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>They are more <qex>sharply</qex> to be chastised and reformed than the rude Irish.</q> <rj><qau>Spenser.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>The soldiers were <qex>sharply</qex> assailed with wants.</q> <rj><qau>Hayward.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>You contract your eye when you would see <qex>sharply</qex>.</q> <rj><qau>Bacon.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sharpness</ent><br/
<hw>Sharp"ness</hw>, <pos>n.</pos> <ety>[AS. <ets>scearpness</ets>.]</ety> <def>The quality or condition of being sharp; keenness; acuteness.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sharpsaw</ent><br/
<hw>Sharp"saw`</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Zool.)</fld> <def>The great titmouse; -- so called from its harsh call notes.</def> <mark>[Prov. Eng.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sharp-set</ent><br/
<hw>Sharp"-set`</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Eager in appetite or desire of gratification; affected by keen hunger; ravenous; <as>as, an eagle or a lion <ex>sharp-set</ex></as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The town is <qex>sharp-set</qex> on new plays.</q> <rj><qau>Pope.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sharpshooter</ent><br/
<hw>Sharp"shoot`er</hw> <pr>(?)</pr>, <pos>n.</pos> <def>One skilled in shooting at an object with exactness; a good marksman.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sharpshooting</ent><br/
<hw>Sharp"shoot`ing</hw>, <pos>n.</pos> <def>A shooting with great precision and effect; hence, a keen contest of wit or argument.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sharp-sighted</ent><br/
<hw>Sharp"-sight`ed</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Having quick or acute sight; -- used literally and figuratively.</def> -- <wordforms><wf>Sharp`-sight`ed*ness</wf>, <pos>n.</pos></wordforms><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sharptail</ent><br/
<hw>Sharp"tail`</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Zool.)</fld> <sd>(a)</sd> <def>The pintail duck.</def> <sd>(b)</sd> <def>The pintail grouse, or prairie chicken.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sharp-witted</ent><br/
<hw>Sharp"-wit`ted</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Having an acute or nicely discerning mind.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shash</ent><br/
<hw>Shash</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[See <er>Sash</er>.]</ety> <sn>1.</sn> <def>The scarf of a turban.</def> <mark>[Obs.]</mark>  <rj><au>Fuller.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A sash.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shasta</ent><br/
<hw>Shas"ta</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A mountain peak, etc., in California.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Shasta daisy</ent><br/
<hw>Shasta daisy</hw>. <def>A large-flowered garden variety of the oxeye daisy.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Shasta fir</ent><br/
<hw>Shasta fir</hw>. <def>A Californian fir (<spn>Abies shastensis</spn>).</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Shasta Sam</ent><br/
<hw>Shasta Sam</hw>. <fld>(Card Playing)</fld> <def>A game like California Jack, except that the pack drawn from is turned face down.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Shastra</ent><br/
<ent>Shaster</ent><br/
<mhw>{ ||<hw>Shas"ter</hw> <pr>(?)</pr>, ||<hw>Shas"tra</hw> <pr>(?)</pr>, }</mhw> <pos>n.</pos> <ety>[Skr. <ets>c<amac/stra</ets> an order or command, a sacred book, fr. <ets>c<amac/s</ets> to order, instruct, govern.  Cf. <er>Sastra</er>.]</ety> <def>A treatise for authoritative instruction among the Hindoos; a book of institutes; especially, a treatise explaining the Vedas.</def> <altsp>[Written also <asp>sastra</asp>.]</altsp><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shathmont</ent><br/
<hw>Shath"mont</hw> <pr>(<?/)</pr>, <pos>n.</pos> <def>A shaftment.</def> <mark>[Scot.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shatter</ent><br/
<hw>Shat"ter</hw> <pr>(?)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Shattered</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Shattering</conjf>.]</vmorph> <ety>[OE. <ets>schateren</ets>, <ets>scateren</ets>, to scatter, to dash, AS. <ets>scateran</ets>; cf. D. <ets>schateren</ets> to crack, to make a great noise, OD. <ets>schetteren</ets> to scatter, to burst, to crack.  Cf. <er>Scatter</er>.]</ety> <sn>1.</sn> <def>To break at once into many pieces; to dash, burst, or part violently into fragments; to rend into splinters; <as>as, an explosion <ex>shatters</ex> a rock or a bomb; too much steam <ex>shatters</ex> a boiler; an oak is <ex>shattered</ex> by lightning.</as></def><br/
[<source>1913 Webster</source>]</p>

<p><q>A monarchy was <qex>shattered</qex> to pieces, and divided amongst revolted subjects.</q> <rj><qau>Locke.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To disorder; to derange; to render unsound; <as>as, to be <ex>shattered</ex> in intellect; his constitution was <ex>shattered</ex>; his hopes were <ex>shattered</ex></as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>A man of a loose, volatile, and <qex>shattered</qex> humor.</q> <rj><qau>Norris.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>To scatter about.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q><qex>Shatter</qex> your leaves before the mellowing year.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shatter</ent><br/
<hw>Shat"ter</hw>, <pos>v. i.</pos> <def>To be broken into fragments; to fall or crumble to pieces by any force applied.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Some fragile bodies break but where the force is; some <qex>shatter</qex> and fly in many places.</q> <rj><qau>Bacon.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shatter</ent><br/
<hw>Shat"ter</hw>, <pos>n.</pos> <def>A fragment of anything shattered; -- used chiefly or soley in the phrase <xex>into shatters</xex>; <as>as, to break a glass into <ex>shatters</ex></as>.</def>  <rj><au>Swift.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shatter-pated</ent><br/
<ent>Shatter-brained</ent><br/
<mhw>{ <hw>Shat"ter-brained`</hw> <pr>(?)</pr>, <hw>Shat"ter-pat`ed</hw> <pr>(?)</pr>, }</mhw> <pos>a.</pos> <def>Disordered or wandering in intellect; hence, heedless; wild; similar to <altname>scatter-brained</altname>.</def>  <rj><au>J. Goodman.</au></rj><br/
[<source>1913 Webster</source> <source>+PJC</source>]</p>

<p><ent>Shattery</ent><br/
<hw>Shat"ter*y</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Easily breaking into pieces; not compact; loose of texture; brittle; <as>as, <ex>shattery</ex> spar</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shave</ent><br/
<hw>Shave</hw> <pr>(sh<amac/v)</pr>, <def><pos>obs. p. p.</pos> of <er>Shave</er>.</def>  <rj><au>Chaucer.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>His beard was <qex>shave</qex> as nigh as ever he can.</q> <rj><qau>Chaucer.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shave</ent><br/
<hw>Shave</hw>, <pos>v. t.</pos> <vmorph>[<pos>imp.</pos> <conjf>Shaved</conjf> <pr>(sh<amac/vd)</pr>;<pos>p. p.</pos> <conjf>Shaved</conjf> or <conjf>Shaven</conjf> <pr>(sh<amac/v"'n)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Shaving</conjf>.]</vmorph> <ety>[OE. <ets>shaven</ets>, <ets>schaven</ets>, AS. <ets>scafan</ets>, <ets>sceafan</ets>; akin to D. <ets>schaven</ets>, G. <ets>schaben</ets>, Icel. <ets>skafa</ets>, Sw. <ets>skafva</ets>, Dan. <ets>skave</ets>, Goth. <ets>scaban</ets>, Russ. <ets>kopate</ets> to dig, Gr. <grk>ska`ptein</grk>, and probably to L. <ets>scabere</ets> to scratch, to scrape.  Cf. <er>Scab</er>, <er>Shaft</er>, <er>Shape</er>.]</ety> <sn>1.</sn> <def>To cut or pare off from the surface of a body with a razor or other edged instrument; to cut off closely, as with a razor; <as>as, to <ex>shave</ex> the beard</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To make bare or smooth by cutting off closely the surface, or surface covering, of; especially, to remove the hair from with a razor or other sharp instrument; to take off the beard or hair of; <as>as, to <ex>shave</ex> the face or the crown of the head; he <ex>shaved</ex> himself.</as></def><br/
[<source>1913 Webster</source>]</p>

<p><q>I'll <qex>shave</qex> your crown for this.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>The laborer with the bending scythe is seen<br/
<qex>Shaving</qex> the surface of the waving green.</q> <rj><qau>Gay.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>To cut off thin slices from; to cut in thin slices.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Plants bruised or <qex>shaven</qex> in leaf or root.</q> <rj><qau>Bacon.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>To skim along or near the surface of; to pass close to, or touch lightly, in passing.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Now <qex>shaves</qex> with level wing the deep.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <def>To strip; to plunder; to fleece.</def> <mark>[Colloq.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>To shave a note</b></col>, <cd>to buy it at a discount greater than the legal rate of interest, or to deduct in discounting it more than the legal rate allows.</cd> <mark>[Cant, U.S.]</mark></cs><br/
[<source>1913 Webster</source>]</p>

<p><-- p. 1325 --></p>

<p><ent>Shave</ent><br/
<hw>Shave</hw> <pr>(?)</pr>, <pos>v. i.</pos> <def>To use a razor for removing the beard; to cut closely; hence, to be hard and severe in a bargain; to practice extortion; to cheat.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shave</ent><br/
<hw>Shave</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[AS. <ets>scafa</ets>, sceafa, a sort of knife. See <er>Shave</er>, <pos>v. t.</pos>]</ety> <sn>1.</sn> <def>A thin slice; a shaving.</def>  <rj><au>Wright.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A cutting of the beard; the operation of shaving.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <sd>(a)</sd> <def>An exorbitant discount on a note.</def> <mark>[Cant, U.S.]</mark> <sd>(b)</sd> <def>A premium paid for an extension of the time of delivery or payment, or for the right to vary a stock contract in any particular.</def> <mark>[Cant, U.S.]</mark>  <rj><au>N. Biddle.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>A hand tool consisting of a sharp blade with a handle at each end; a drawing knife; a spokeshave.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <def>The act of passing very near to, so as almost to graze; <as>as, the bullet missed by a close <ex>shave</ex></as>.</def> <mark>[Colloq.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Shave grass</b></col> <fld>(Bot.)</fld>, <cd>the scouring rush. See the Note under <er>Equisetum</er>.</cd> -- <col><b>Shave hook</b></col>, <cd>a tool for scraping metals, consisting of a sharp-edged triangular steel plate attached to a shank and handle.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shaveling</ent><br/
<hw>Shave"ling</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A man shaved; hence, a monk, or other religious; -- used in contempt.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>I am no longer a <qex>shaveling</qex> than while my frock is on my back.</q> <rj><qau>Sir W. Scott.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shaver</ent><br/
<hw>Shav"er</hw> <pr>(?)</pr>, <pos>n.</pos> <sn>1.</sn> <def>One who shaves; one whose occupation is to shave.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>One who is close in bargains; a sharper.</def>  <rj><au>Swift.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>One who fleeces; a pillager; a plunderer.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>By these <qex>shavers</qex> the Turks were stripped.</q> <rj><qau>Knolles.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>A boy; a lad; a little fellow.</def> <mark>[Colloq.]</mark> <ldquo/These unlucky little <xex>shavers</xex>.<rdquo/
<-- often little shaver -->  <rj><au>Salmagundi.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>As I have mentioned at the door to this young <qex>shaver</qex>, I am on a chase in the name of the king.</q> <rj><qau>Dickens.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <fld>(Mech.)</fld> <def>A tool or machine for shaving.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>A note shaver</b></col>, <cd>a person who buys notes at a discount greater than the legal rate of interest.</cd> <mark>[Cant, U.S.]</mark></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shaving</ent><br/
<hw>Shav"ing</hw>, <pos>n.</pos> <sn>1.</sn> <def>The act of one who, or that which, shaves; specifically, the act of cutting off the beard with a razor.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>That which is shaved off; a thin slice or strip pared off with a shave, a knife, a plane, or other cutting instrument.</def> <ldquo/<xex>Shaving</xex> of silver.<rdquo/  <rj><au>Chaucer.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Shaving brush</b></col>, <cd>a brush used in lathering the face preparatory to shaving it.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shaw</ent><br/
<hw>Shaw</hw> <pr>(sh<add/)</pr>, <pos>n.</pos> <ety>[OE. <ets>schawe</ets>, <ets>scha<yogh/e</ets>, thicket, grove, AS. <ets>scaga</ets>; akin to Dan. <ets>skov</ets>, Sw. <ets>skog</ets>, Icel. <ets>sk<omac/gr</ets>.]</ety> <sn>1.</sn> <def>A thicket; a small wood or grove.</def> <mark>[Obs. or Prov. Eng. & Scot.]</mark>  <rj><au>Burns.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Gaillard he was as goldfinch in the <qex>shaw</qex>.</q> <rj><qau>Chaucer.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>The green <qex>shaws</qex>, the merry green woods.</q> <rj><qau>Howitt.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <pluf>pl.</pluf> <def>The leaves and tops of vegetables, as of potatoes, turnips, etc.</def> <mark>[Scot.]</mark>  <rj><au>Jamieson.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shawfowl</ent><br/
<hw>Shaw"fowl`</hw>, <pos>n.</pos> <ety>[Scot. <ets>schaw</ets>, <ets>shaw</ets>, show + <ets>fowl</ets>.]</ety> <def>The representation or image of a fowl made by fowlers to shoot at.</def>  <rj><au>Johnson.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shawl</ent><br/
<hw>Shawl</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Per. & Hind. <ets>sh<amac/l</ets>: cf. F. <ets>ch<acir/le</ets>.]</ety> <def>A square or oblong cloth of wool, cotton, silk, or other textile or netted fabric, used, especially by women, as a loose covering for the neck and shoulders.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>India shawl</b></col>, <cd>a kind of rich shawl made in India from the wool of the Cashmere goat. It is woven in pieces, which are sewed together.</cd> -- <col><b>Shawl goat</b></col> <fld>(Zool.)</fld>, <cd>the Cashmere goat.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shawl</ent><br/
<hw>Shawl</hw>, <pos>v. t.</pos> <def>To wrap in a shawl.</def>  <rj><au>Thackeray.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shawm</ent><br/
<hw>Shawm</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[OE. <ets>shalmie</ets>, OF. <ets>chalemie</ets>; cf. F. <ets>chalumeau</ets> shawm, <ets>chaume</ets> haulm, stalk; all fr. L. <ets>calamus</ets> a reed, reed pipe. See <er>Haulm</er>, and cf. <er>Calumet</er>.]</ety> <fld>(Mus.)</fld> <def>A wind instrument of music, formerly in use, supposed to have resembled either the clarinet or the hautboy in form.</def> <altsp>[Written also <asp>shalm</asp>, <asp>shaum</asp>.]</altsp>  <rj><au>Otway.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Even from the shrillest <qex>shaum</qex> unto the cornamute.</q> <rj><qau>Drayton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shawnees</ent><br/
<hw>Shaw`nees"</hw> <pr>(?)</pr>, <pos>n. pl.</pos>; <sing>sing. <singw>Shawnee</singw> <pr>(<?/)</pr></sing>. <fld>(Ethnol.)</fld> <def>A tribe of North American Indians who occupied Western New York and part of Ohio, but were driven away and widely dispersed by the Iroquois.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shay</ent><br/
<hw>Shay</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A chaise.</def> <mark>[Prov. Eng. & Local, U.S.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>She</ent><br/
<hw>She</hw> <pr>(?)</pr>, <pos>pron.</pos> <nmorph>[<pos>sing. nom.</pos> <decf>She</decf>; <pos>poss.</pos> <decf>Her</decf>. <pr>(<?/)</pr> or <decf>Hers</decf> <pr>(<?/)</pr>; <pos>obj.</pos> <decf>Her</decf>; <pos>pl. nom.</pos> <decf>They</decf> <pr>(?)</pr>; <pos>poss.</pos> <decf>Their</decf> <pr>(?)</pr> or <decf>Theirs</decf> <pr>(<?/)</pr>; <pos>obj.</pos> <decf>Them</decf> <pr>(?)</pr>.]</nmorph> <ety>[OE. <ets>she</ets>, <ets>sche</ets>, <ets>scheo</ets>, <ets>scho</ets>, AS. <ets>se<oacute/</ets>, fem. of the definite article, originally a demonstrative pronoun; cf. OS. <ets>siu</ets>, D. <ets>zij</ets>, G. <ets>sie</ets>, OHG. <ets>siu</ets>, <ets>s<imac/</ets>, <ets>si</ets>, Icel. <ets>s<umac/</ets>, <ets>sj<amac/</ets>, Goth. <ets>si</ets> she, <ets>s<omac/</ets>, fem. article, Russ. <ets>siia</ets>, fem., this, Gr. <?/, fem. article, Skr. <ets>s<amac/</ets>, <ets>sy<amac/</ets>. The possessive <ets>her</ets> or <ets>hers</ets>, and the objective <ets>her</ets>, are from a different root. See <er>Her</er>.]</ety> <sn>1.</sn> <def>This or that female; the woman understood or referred to; the animal of the female sex, or object personified as feminine, which was spoken of.</def><br/
[<source>1913 Webster</source>]</p>

<p><q><qex>She</qex> loved her children best in every wise.</q> <rj><qau>Chaucer.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Then Sarah denied, . . . for <qex>she</qex> was afraid.</q> <rj><qau>Gen. xviii. 15.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A woman; a female; -- used substantively.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>Lady, you are the cruelest <qex>she</qex> alive.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ <xex>She</xex> is used in composition with nouns of common gender, for <xex>female</xex>, to denote an animal of the female sex; as, a <xex>she</xex>-bear; a <xex>she</xex>-cat.</note><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sheading</ent><br/
<hw>Shead"ing</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[From AS. <ets>sc<amac/dan</ets>, <ets>sce<aacute/dan</ets>, to separate, divide. See <er>Shed</er>, <pos>v. t.</pos>]</ety> <def>A tithing, or division, in the Isle of Man, in which there is a coroner, or chief constable. The island is divided into six sheadings.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sheaf</ent><br/
<hw>Sheaf</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Mech.)</fld> <def>A sheave.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sheaf</ent><br/
<hw>Sheaf</hw>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Sheaves</plw> <pr>(#)</pr>.</plu> <ety>[OE. <ets>sheef</ets>, <ets>shef</ets>, <ets>schef</ets>, AS. <ets>sce<aacute/f</ets>; akin to D. <ets>schoof</ets>, OHG. <ets>scoub</ets>, G. <ets>schaub</ets>, Icel. <ets>skauf</ets> a fox's brush, and E. <ets>shove</ets>. See <er>Shove</er>.]</ety> <sn>1.</sn> <def>A quantity of the stalks and ears of wheat, rye, or other grain, bound together; a bundle of grain or straw.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The reaper fills his greedy hands,<br/
And binds the golden <qex>sheaves</qex> in brittle bands.</q> <rj><qau>Dryden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Any collection of things bound together; a bundle; specifically, a bundle of arrows sufficient to fill a quiver, or the allowance of each archer, -- usually twenty-four.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The <qex>sheaf</qex> of arrows shook and rattled in the case.</q> <rj><qau>Dryden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sheaf</ent><br/
<hw>Sheaf</hw>, <pos>v. t.</pos> <def>To gather and bind into a sheaf; to make into sheaves; <as>as, to <ex>sheaf</ex> wheat</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sheaf</ent><br/
<hw>Sheaf</hw> <pr>(?)</pr>, <pos>v. i.</pos> <def>To collect and bind cut grain, or the like; to make sheaves.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>They that reap must <qex>sheaf</qex> and bind.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sheafy</ent><br/
<hw>Sheaf"y</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Pertaining to, or consisting of, a sheaf or sheaves; resembling a sheaf.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sheal</ent><br/
<hw>Sheal</hw> <pr>(?)</pr>, <pos>n.</pos> <def>Same as <er>Sheeling</er>.</def> <mark>[Scot.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sheal</ent><br/
<hw>Sheal</hw>, <pos>v. t.</pos> <def>To put under a sheal or shelter.</def> <mark>[Scot.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sheal</ent><br/
<hw>Sheal</hw>, <pos>v. t.</pos> <ety>[See <er>Shell</er>.]</ety> <def>To take the husks or pods off from; to shell; to empty of its contents, as a husk or a pod.</def> <mark>[Obs. or Prov. Eng. & Scot.]</mark>  <rj><au>Jamieson.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>That's a <qex>shealed</qex> peascod.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sheal</ent><br/
<hw>Sheal</hw>, <pos>n.</pos> <def>A shell or pod.</def> <mark>[Obs. or Prov. Eng.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shealing</ent><br/
<hw>Sheal"ing</hw>, <pos>n.</pos> <def>The outer husk, pod, or shell, as of oats, pease, etc.; sheal; shell.</def> <mark>[Obs. or Prov. Eng. & Scot.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shealing</ent><br/
<hw>Sheal"ing</hw>, <pos>n.</pos> <def>Same as <er>Sheeling</er>.</def> <mark>[Scot.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shear</ent><br/
<hw>Shear</hw> <pr>(sh<emac/r)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp.</pos> <conjf>Sheared</conjf> <pr>(?)</pr> or <conjf>Shore</conjf> <pr>(<?/)</pr>;<pos>p. p.</pos> <conjf>Sheared</conjf> or <conjf>Shorn</conjf> <pr>(<?/)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Shearing</conjf>.]</vmorph> <ety>[OE. <ets>sheren</ets>, <ets>scheren</ets>, to shear, cut, shave, AS. <ets>sceran</ets>, <ets>scieran</ets>, <ets>scyran</ets>; akin to D. & G. <ets>scheren</ets>, Icel. <ets>skera</ets>, Dan. <ets>ski<?/re</ets>, Gr. <?/<?/<?/.  Cf. <er>Jeer</er>, <er>Score</er>, <er>Shard</er>, <er>Share</er>, <er>Sheer</er> to turn aside.]</ety> <sn>1.</sn> <def>To cut, clip, or sever anything from with shears or a like instrument; <as>as, to <ex>shear</ex> sheep; to <ex>shear</ex> cloth</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ It is especially applied to the cutting of wool from sheep or their skins, and the nap from cloth.</note><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To separate or sever with shears or a similar instrument; to cut off; to clip (something) from a surface; <as>as, to <ex>shear</ex> a fleece</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Before the golden tresses . . . were <qex>shorn</qex> away.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>To reap, as grain.</def> <mark>[Scot.]</mark>  <rj><au>Jamieson.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>Fig.: To deprive of property; to fleece.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <fld>(Mech.)</fld> <def>To produce a change of shape in by a shear. See <er>Shear</er>, <pos>n.</pos>, 4.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shear</ent><br/
<hw>Shear</hw>, <pos>n.</pos> <ety>[AS. <ets>sceara</ets>. See <er>Shear</er>, <pos>v. t.</pos>]</ety> <sn>1.</sn> <def>A pair of shears; -- now always used in the plural, but formerly also in the singular. See <er>Shears</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>On his head came razor none, nor <qex>shear</qex>.</q> <rj><qau>Chaucer.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Short of the wool, and naked from the <qex>shear</qex>.</q> <rj><qau>Dryden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A shearing; -- used in designating the age of sheep.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>After the second shearing, he is a two-<qex>shear</qex> ram; . . . at the expiration of another year, he is a three-<qex>shear</qex> ram; the name always taking its date from the time of shearing.</q> <rj><qau>Youatt.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <fld>(Engin.)</fld> <def>An action, resulting from applied forces, which tends to cause two contiguous parts of a body to slide relatively to each other in a direction parallel to their plane of contact; -- also called <altname>shearing stress</altname>, and <altname>tangential stress</altname>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <fld>(Mech.)</fld> <def>A strain, or change of shape, of an elastic body, consisting of an extension in one direction, an equal compression in a perpendicular direction, with an unchanged magnitude in the third direction.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Shear blade</b></col>, <cd>one of the blades of shears or a shearing machine.</cd> -- <col><b>Shear hulk</b></col>. <cd>See under <er>Hulk</er>.</cd> -- <col><b>Shear steel</b></col>, <cd>a steel suitable for shears, scythes, and other cutting instruments, prepared from fagots of blistered steel by repeated heating, rolling, and tilting, to increase its malleability and fineness of texture.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shear</ent><br/
<hw>Shear</hw>, <pos>v. i.</pos> <sn>1.</sn> <def>To deviate. See <er>Sheer</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Engin.)</fld> <def>To become more or less completely divided, as a body under the action of forces, by the sliding of two contiguous parts relatively to each other in a direction parallel to their plane of contact.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shearbill</ent><br/
<hw>Shear"bill`</hw>, <pos>n.</pos> <fld>(Zool.)</fld> <def>The black skimmer. See <er>Skimmer</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sheard</ent><br/
<hw>Sheard</hw> <pr>(?)</pr>, <pos>n.</pos> <def>See <er>Shard</er>.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shearer</ent><br/
<hw>Shear"er</hw> <pr>(?)</pr>, <pos>n.</pos> <sn>1.</sn> <def>One who shears.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Like a lamb dumb before his <qex>shearer</qex>.</q> <rj><qau>Acts viii. 32.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A reaper.</def> <mark>[Scot.]</mark>  <rj><au>Jamieson.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shearing</ent><br/
<hw>Shear"ing</hw>, <pos>n.</pos> <sn>1.</sn> <def>The act or operation of clipping with shears or a shearing machine, as the wool from sheep, or the nap from cloth.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>The product of the act or operation of clipping with shears or a shearing machine; <as>as, the whole <ex>shearing</ex> of a flock; the <ex>shearings</ex> from cloth</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>Same as <er>Shearling</er>.</def>  <rj><au>Youatt.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>The act or operation of reaping.</def> <mark>[Scot.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <def>The act or operation of dividing with shears; <as>as, the <ex>shearing</ex> of metal plates</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>6.</sn> <def>The process of preparing shear steel; tilting.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>7.</sn> <fld>(Mining)</fld> <def>The process of making a vertical side cutting in working into a face of coal.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Shearing machine</b></col>. <sd>(a)</sd> <cd>A machine with blades, or rotary disks, for dividing plates or bars of metal</cd>. <sd>(b)</sd> <cd>A machine for shearing cloth.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shearling</ent><br/
<hw>Shear"ling</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A sheep but once sheared.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shearman</ent><br/
<hw>Shear"man</hw> <pr>(?)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Shearmen</plw> <pr>(<?/)</pr>.</plu> <def>One whose occupation is to shear cloth.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shearn</ent><br/
<hw>Shearn</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[AS. <ets>scearn</ets>.  Cf. <er>Scarn</er>.]</ety> <def>Dung; excrement.</def> <mark>[Obs.]</mark> <altsp>[Written also <asp>shern</asp>.]</altsp>  <rj><au>Holland.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shears</ent><br/
<hw>Shears</hw> <pr>(?)</pr>, <pos>n. pl.</pos> <ety>[Formerly used also in the singular. See <er>Shear</er>, <pos>n.</pos>, 1.]</ety> <sn>1.</sn> <def>A cutting instrument.</def> Specifically: <sd>(a)</sd> <def>An instrument consisting of two blades, commonly with bevel edges, connected by a pivot, and working on both sides of the material to be cut, -- used for cutting cloth and other substances.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Fate urged the <qex>shears</qex>, and cut the sylph in twain.</q> <rj><qau>Pope.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sd>(b)</sd> <def>A similar instrument the blades of which are extensions of a curved spring, -- used for shearing sheep or skins.</def> <sd>(c)</sd> <def>A shearing machine; a blade, or a set of blades, working against a resisting edge.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Anything in the form of shears.</def> Specifically: <sd>(a)</sd> <def>A pair of wings.</def> <mark>[Obs.]</mark> <au>Spenser.</au> <sd>(b)</sd> <def>An apparatus for raising heavy weights, and especially for stepping and unstepping the lower masts of ships. It consists of two or more spars or pieces of timber, fastened together near the top, steadied by a guy or guys, and furnished with the necessary tackle.</def> <altsp>[Written also <asp>sheers</asp>.]</altsp><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <fld>(Mach.)</fld> <def>The bedpiece of a machine tool, upon which a table or slide rest is secured; <as>as, the <ex>shears</ex> of a lathe or planer</as>. See <xex>Illust.</xex> under <er>Lathe</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Rotary shears</b></col>. <cd>See under <er>Rotary</er>.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shear steel</ent><br/
<hw>Shear steel</hw>. <def>See under <er>Shear</er>.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Sheartail</ent><br/
<hw>Shear"tail`</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Zool.)</fld> <sd>(a)</sd> <def>The common tern.</def> <sd>(b)</sd> <def>Any one of several species of humming birds of the genus <gen>Thaumastura</gen> having a long forked tail.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shearwater</ent><br/
<hw>Shear"wa`ter</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[<ets>Shear</ets> + <ets>water</ets>; cf. G. <ets>wassersherer</ets>; -- so called from its running lightly along the surface of the water.]</ety> <fld>(Zool.)</fld> <def>Any one of numerous species of long-winged oceanic birds of the genus <gen>Puffinus</gen> and related genera. They are allied to the petrels, but are larger. The Manx shearwater (<spn>Puffinus Anglorum</spn>), the dusky shearwater (<spn>Puffinus obscurus</spn>), and the greater shearwater (<spn>Puffinus major</spn>), are well-known species of the North Atlantic. See <er>Hagdon</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sheatfish</ent><br/
<hw>Sheat"fish`</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Cf. dial. G. <ets>scheid</ets>, <ets>schaid</ets>, <ets>schaiden</ets>.]</ety> <fld>(Zool.)</fld> <def>A European siluroid fish (<spn>Silurus glanis</spn>) allied to the cat-fishes. It is the largest fresh-water fish of Europe, sometimes becoming six feet or more in length. See <er>Siluroid</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sheath</ent><br/
<hw>Sheath</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[OE. <ets>schethe</ets>, AS. <ets>sc<aemac/<edh/</ets>, <ets>sce<aacute/<edh/</ets>, <ets>sc<emac/<edh/</ets>; akin to OS. <ets>sk<emac/<edh/ia</ets>, D. <ets>scheede</ets>, G. <ets>scheide</ets>, OHG. <ets>sceida</ets>, Sw. <ets>skida</ets>, Dan. <ets>skede</ets>, Icel. <ets>skei<edh/ir</ets>, pl., and to E. <ets>shed</ets>, v.t., originally meaning, to separate, to part. See <er>Shed</er>.]</ety> <sn>1.</sn> <def>A case for the reception of a sword, hunting knife, or other long and slender instrument; a scabbard.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The dead knight's sword out of his <qex>sheath</qex> he drew.</q> <rj><qau>Spenser.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Any sheathlike covering, organ, or part.</def> Specifically: <sd>(a)</sd> <fld>(Bot.)</fld> <def>The base of a leaf when sheathing or investing a stem or branch, as in grasses.</def> <sd>(b)</sd> <fld>(Zool.)</fld> <def>One of the elytra of an insect.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Medullary sheath</b></col>. <fld>(Anat.)</fld> <cd>See under <er>Medullary</er>.</cd> -- <col><b>Primitive sheath</b></col>. <fld>(Anat.)</fld> <cd>See <er>Neurilemma</er>.</cd> -- <col><b>Sheath knife</b></col>, <cd>a knife with a fixed blade, carried in a sheath.</cd> -- <col><b>Sheath of Schwann</b></col>. <fld>(Anat.)</fld> <cd>See <er>Schwann's sheath</er>.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sheathbill</ent><br/
<hw>Sheath"bill`</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Zool.)</fld> <def>Either one of two species of birds composing the genus <gen>Chionis</gen>, and family <fam>Chionidae</fam>, native of the islands of the Antarctic seas.</def><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ They are related to the gulls and the plovers, but more nearly to the latter. The base of the bill is covered with a saddle-shaped horny sheath, and the toes are only slightly webbed. The plumage of both species is white.</note><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sheathe</ent><br/
<hw>Sheathe</hw> <pr>(?)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Sheathed</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Sheating</conjf>.]</vmorph> <ety>[Written also <ets>sheath</ets>.]</ety> <sn>1.</sn> <def>To put into a sheath, case, or scabbard; to inclose or cover with, or as with, a sheath or case.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The leopard . . . keeps the claws of his fore feet turned up from the ground, and <qex>sheathed</qex> in the skin of his toes.</q> <rj><qau>Grew.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>'T is in my breast she <qex>sheathes</qex> her dagger now.</q> <rj><qau>Dryden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To fit or furnish, as with a sheath.</def>  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>To case or cover with something which protects, as thin boards, sheets of metal, and the like; <as>as, to <ex>sheathe</ex> a ship with copper</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>To obtund or blunt, as acrimonious substances, or sharp particles.</def> <mark>[R.]</mark>  <rj><au>Arbuthnot.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>To sheathe the sword</b></col>, <cd>to make peace.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sheathed</ent><br/
<hw>Sheathed</hw> <pr>(?)</pr>, <pos>a.</pos> <sn>1.</sn> <def>Povided with, or inclosed in, sheath.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Bot.)</fld> <def>Invested by a sheath, or cylindrical membranaceous tube, which is the base of the leaf, as the stalk or culm in grasses; vaginate.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sheather</ent><br/
<hw>Sheath"er</hw> <pr>(?)</pr>, <pos>n.</pos> <def>One who sheathes.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sheathfish</ent><br/
<hw>Sheath"fish`</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Zool.)</fld> <def>Same as <er>Sheatfish</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sheathing</ent><br/
<hw>Sheath"ing</hw> <pr>(?)</pr>, <pos>p. pr. & a.</pos> <mord>from <er>Sheathe</er>.</mord> <def>Inclosing with a sheath; <as>as, the <ex>sheathing</ex> leaves of grasses; the <ex>sheathing</ex> stipules of many polygonaceous plants</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sheathing</ent><br/
<hw>Sheath"ing</hw>, <pos>n.</pos> <def>That which sheathes.</def> Specifically: <sd>(a)</sd> <def>The casing or covering of a ship's bottom and sides; the materials for such covering; <as>as, copper <ex>sheathing</ex></as>.</def> <sd>(b)</sd> <fld>(Arch.)</fld> <def>The first covering of boards on the outside wall of a frame house or on a timber roof; also, the material used for covering; ceiling boards in general.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sheathless</ent><br/
<hw>Sheath"less</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Without a sheath or case for covering; unsheathed.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sheath-winged</ent><br/
<hw>Sheath"-winged`</hw> <pr>(?)</pr>, <pos>a.</pos> <fld>(Zool.)</fld> <def>Having elytra, or wing cases, as a beetle.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sheathy</ent><br/
<hw>Sheath"y</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Forming or resembling a sheath or case.</def>  <rj><au>Sir T. Browne.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shea tree</ent><br/
<hw>She"a tree`</hw> <pr>(?)</pr>. <fld>(Bot.)</fld> <def>An African sapotaceous tree (<spn>Bassia Parkii</spn> syn. <spn>Butyrospermum Parkii</spn>), from the seeds of which a substance resembling butter is obtained; the African butter tree.</def><br/
[<source>1913 Webster</source>]</p>

<p><-- p. 1326 --></p>

<p><ent>Sheave</ent><br/
<hw>Sheave</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Akin to OD. <ets>schijve</ets> orb, disk, wheel, D. <ets>schiff</ets>, G. <ets>scheibe</ets>, Icel. <ets>sk<imac/fa</ets> a shaving, slice; cf. Gr. <?/<?/<?/ a staff.  Cf. <er>Shift</er>, <pos>v.</pos>, <er>Shive</er>.]</ety> <def>A wheel having a groove in the rim for a rope to work in, and set in a block, mast, or the like; the wheel of a pulley.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Sheave hole</b></col>, <cd>a channel cut in a mast, yard, rail, or other timber, in which to fix a sheave.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sheave</ent><br/
<hw>Sheave</hw>, <pos>v. t.</pos> <ety>[See <er>Sheaf</er> of straw.]</ety> <def>To gather and bind into a sheaf or sheaves; hence, to collect.</def>  <rj><au>Ashmole.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sheaved</ent><br/
<hw>Sheaved</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Made of straw.</def> <mark>[Obs.]</mark>  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shebander</ent><br/
<hw>Sheb"an*der</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Per. <ets>sh<amac/hbandar</ets>.]</ety> <def>A harbor master, or ruler of a port, in the East Indies.</def> <altsp>[Written also <asp>shebunder</asp>.]</altsp><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shebang</ent><br/
<hw>She*bang"</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Cf. <er>Shebeen</er>.]</ety> <sn>1.</sn> <def>A jocosely depreciative name for a dwelling or shop; a primitive dwelling; a shanty.</def> <mark>[Slang, U.S.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>The structure of an object, process, organization, or anything viewed as complicated; -- used primarily in the phrase <col><b>the whole <ex>shebang</ex></b></col>; <as>as, it comes with unnecessary frills, but you have to buy the whole <ex>shebang</ex></as>.</def> <mark>[informal]</mark> <br/
[<source>PJC</source>]</p>

<p><sn>3.</sn> <fld>(computers)</fld> <ety>[Possibly derived from <ets>shell bang</ets>; the character <it>!</it> is referred to in some computer contexts as <ets>bang</ets>.]</ety>  <def>The character sequence <it>#!</it>, which frequently begins shell scripts in  a Unix system.</def><br/
[<source>PJC</source>]</p>

<p><ent>Shebeen</ent><br/
<hw>She*been"</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Of Irish origin; cf. Ir. <ets>seapa</ets> a shop.]</ety> <def>A low public house; especially, a place where spirits and other excisable liquors are illegally and privately sold.</def> <mark>[Ireland]</mark><-- similar to a speakeasy? --><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shechinah</ent><br/
<hw>She*chi"nah</hw> <pr>(?)</pr>, <pos>n.</pos> <def>See <er>Shekinah</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shecklaton</ent><br/
<hw>Sheck"la*ton</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Cf. <er>Ciclatoun</er>.]</ety> <def>A kind of gilt leather. See <er>Checklaton</er>.</def> <mark>[Obs.]</mark>  <rj><au>Spenser.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shed</ent><br/
<hw>Shed</hw> <pr>(sh<ecr/d)</pr>, <pos>n.</pos> <ety>[The same word as <ets>shade</ets>. See <er>Shade</er>.]</ety> <sn>1.</sn> <def>A slight or temporary structure built to shade or shelter something; a structure often open in front; an outbuilding; a hut; <as>as, a wagon <ex>shed</ex>; a wood <ex>shed</ex></as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The first Aletes born in lowly <qex>shed</qex>.</q> <rj><qau>Fairfax.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q><qex>Sheds</qex> of reeds which summer's heat repel.</q> <rj><qau>Sandys.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Aeronautics)</fld> <def>A covered structure for housing aircraft; a hangar.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Shed</ent><br/
<hw>Shed</hw>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Shed</conjf>; <pos>p. pr. & vb. n.</pos> <conjf>Shedding</conjf>.]</vmorph> <ety>[OE. <ets>scheden</ets>, <ets>sch<?/den</ets>, to pour, to part, AS. <ets>sc<amac/dan</ets>, <ets>sce<aacute/dan</ets>, to pert, to separate; akin to OS. <ets>sk<?/<?/an</ets>, OFries. sk<?/tha, G. <ets>scheiden</ets>, OHG. <ets>sceidan</ets>, Goth. <ets>skaidan</ets>, and probably to Lith. <ets>sk<eum/du</ets> I part, separate, L. <ets>scindere</ets> to cleave, to split, Gr. <?/<?/<?/, Skr. <ets>chid</ets>, and perch. also to L. <ets>caedere</ets> to cut. <root/159.  Cf. <er>Chisel</er>, <er>Concise</er>, <er>Schism</er>, <er>Sheading</er>, <er>Sheath</er>, <er>Shide</er>.]</ety> <sn>1.</sn> <def>To separate; to divide.</def> <mark>[Obs. or Prov. Eng.]</mark>  <rj><au>Robert of Brunne.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To part with; to throw off or give forth from one's self; to emit; to diffuse; to cause to emanate or flow; to pour forth or out; to spill; <as>as, the sun <ex>sheds</ex> light; she <ex>shed</ex> tears; the clouds <ex>shed</ex> rain</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Did Romeo's hand <qex>shed</qex> Tybalt's blood?</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Twice seven consenting years have <qex>shed</qex><br/
Their utmost bounty on thy head.</q> <rj><qau>Wordsworth.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>To let fall; to throw off, as a natural covering of hair, feathers, shell; to cast; <as>as, fowls <ex>shed</ex> their feathers; serpents <ex>shed</ex> their skins; trees <ex>shed</ex> leaves</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>To cause to flow off without penetrating; <as>as, a tight roof, or covering of oiled cloth, <ex>sheeds</ex> water</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <def>To sprinkle; to intersperse; to cover.</def> <mark>[R.]</mark> <ldquo/Her hair . . . is <xex>shed</xex> with gray.<rdquo/  <rj><au>B. Jonson.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>6.</sn> <fld>(Weaving)</fld> <def>To divide, as the warp threads, so as to form a shed, or passageway, for the shuttle.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shed</ent><br/
<hw>Shed</hw>, <pos>v. i.</pos> <sn>1.</sn> <def>To fall in drops; to pour.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>Such a rain down from the welkin <qex>shadde</qex>.</q> <rj><qau>Chaucer.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To let fall the parts, as seeds or fruit; to throw off a covering or envelope.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>White oats are apt to <qex>shed</qex> most as they lie, and black as they stand.</q> <rj><qau>Mortimer.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shed</ent><br/
<hw>Shed</hw>, <pos>n.</pos> <sn>1.</sn> <def>A parting; a separation; a division.</def> <mark>[Obs. or Prov. Eng.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>They say also that the manner of making the <qex>shed</qex> of newwedded wives' hair with the iron head of a javelin came up then likewise.</q> <rj><qau>Sir T. North.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>The act of shedding or spilling; -- used only in composition, as in blood<xex>shed</xex>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>That which parts, divides, or sheds; -- used in composition, as in water<xex>shed</xex>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <fld>(Weaving)</fld> <def>The passageway between the threads of the warp through which the shuttle is thrown, having a sloping top and bottom made by raising and lowering the alternate threads.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shedder</ent><br/
<hw>Shed"der</hw> <pr>(?)</pr>, <pos>n.</pos> <sn>1.</sn> <def>One who, or that which, sheds; <as>as, a <ex>shedder</ex> of blood; a <ex>shedder</ex> of tears</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Zool.)</fld> <def>A crab in the act of casting its shell, or immediately afterwards while still soft; -- applied especially to the edible crabs, which are most prized while in this state.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shedding</ent><br/
<hw>Shed"ding</hw> <pr>(?)</pr>, <pos>n.</pos> <sn>1.</sn> <def>The act of shedding, separating, or casting off or out; <as>as, the <ex>shedding</ex> of blood</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>That which is shed, or cast off.</def> <mark>[R.]</mark>  <rj><au>Wordsworth.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shilfa</ent><br/
<ent>Sheelfa</ent><br/
<mhw>{ <hw>Sheel"fa</hw> <pr>(?)</pr>, <hw>Shil"fa</hw> <pr>(?)</pr>, }</mhw> <pos>n.</pos> <fld>(Zool.)</fld> <def>The chaffinch; -- so named from its call note.</def> <mark>[Prov. Eng.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sheeling</ent><br/
<hw>Sheel"ing</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Icel. <ets>skj<omac/l</ets> a shelter, a cover; akin to Dan. & Sw. <ets>skjul</ets>.]</ety> <def>A hut or small cottage in an exposed or a retired place (as on a mountain or at the seaside) such as is used by shepherds, fishermen, sportsmen, etc.; a summer cottage; also, a shed.</def> <altsp>[Written also <asp>sheel</asp>, <asp>shealing</asp>, <asp>sheiling</asp>, etc.]</altsp> <mark>[Scot.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sheely</ent><br/
<hw>Sheel"y</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Zool.)</fld> <def>Same as <er>Sheelfa</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sheen</ent><br/
<hw>Sheen</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[OE. <ets>sehene</ets>, AS. <ets>sci<eacute/ne</ets>, <ets>sc<?/ne</ets>, <ets>sc<?/ne</ets>, splendid, beautiful; akin to OFries. <ets>sk<?/ne</ets>, <ets>sk<?/ne</ets>, OS. <ets>sc<?/ni</ets>, D. <ets>schoon</ets>, G. <ets>sch<oum/n</ets>, OHG. <ets>sc<?/ni</ets>, Goth, <ets>skanus</ets>, and E. <ets>shew</ets>; the original meaning being probably, visible, worth seeing. It is not akin to E. <ets>shine</ets>. See <er>Shew</er>, <pos>v. t.</pos>]</ety> <def>Bright; glittering; radiant; fair; showy; sheeny.</def> <mark>[R., except in poetry.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>This holy maiden, that is so bright and <qex>sheen</qex>.</q> <rj><qau>Chaucer.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Up rose each warrier bold and brave,<br/
Glistening in filed steel and armor <qex>sheen</qex>.</q> <rj><qau>Fairfax.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sheen</ent><br/
<hw>Sheen</hw>, <pos>v. i.</pos> <def>To shine; to glisten.</def> <mark>[Poetic]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>This town,<br/
That, <qex>sheening</qex> far, celestial seems to be.</q> <rj><qau>Byron.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sheen</ent><br/
<hw>Sheen</hw>, <pos>n.</pos> <def>Brightness; splendor; glitter.</def> <ldquo/Throned in celestial <xex>sheen</xex>.<rdquo/  <rj><au>Milton.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sheenly</ent><br/
<hw>Sheen""ly</hw>, <pos>adv.</pos> <def>Brightly.</def>  <mark>[R.]</mark>  <rj><au>Mrs. Browning.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sheeny</ent><br/
<hw>Sheen"y</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Bright; shining; radiant; sheen.</def> <ldquo/A <xex>sheeny</xex> summer morn.<rdquo/  <rj><au>Tennyson.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sheep</ent><br/
<hw>Sheep</hw> <pr>(?)</pr>, <pos>n. sing. & pl.</pos> <ety>[OE. <ets>shep</ets>, <ets>scheep</ets>, AS. <ets>sc<?/p</ets>, <ets>sce<aacute/p</ets>; akin to OFries. <ets>sk<?/p</ets>, LG. & D. <ets>schaap</ets>, G. <ets>schaf</ets>, OHG. <ets>sc<amac/f</ets>, Skr. <ets>ch<amac/ga</ets>. <root/295.  Cf. <er>Sheepherd</er>.]</ety> <sn>1.</sn> <fld>(Zool.)</fld> <def>Any one of several species of ruminants of the genus <gen>Ovis</gen>, native of the higher mountains of both hemispheres, but most numerous in Asia.</def><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ The domestic sheep (<spn>Ovis aries</spn>) varies much in size, in the length and texture of its wool, the form and size of its horns, the length of its tail, etc. It was domesticated in prehistoric ages, and many distinct breeds have been produced; as the merinos, celebrated for their fine wool; the Cretan sheep, noted for their long horns; the fat-tailed, or Turkish, sheep, remarkable for the size and fatness of the tail, which often has to be supported on trucks; the Southdowns, in which the horns are lacking; and an Asiatic breed which always has four horns.</note><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A weak, bashful, silly fellow.</def>  <rj><au>Ainsworth.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <pluf>pl.</pluf> <def>Fig.: The people of God, as being under the government and protection of Christ, the great Shepherd.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Rocky mountain sheep</b></col>.<fld>(Zool.)</fld> <cd>See <er>Bighorn</er>.</cd> -- <col><b>Maned sheep</b></col>. <fld>(Zool.)</fld> <cd>See <er>Aoudad</er>.</cd> -- <col><b>Sheep bot</b></col> <fld>(Zool.)</fld>, <cd>the larva of the sheep botfly. See <er>Estrus</er>.</cd> -- <col><b>Sheep dog</b></col> <fld>(Zool.)</fld>, <cd>a shepherd dog, or collie.</cd> -- <col><b>Sheep laurel</b></col> <fld>(Bot.)</fld>, <cd>a small North American shrub (<spn>Kalmia angustifolia</spn>) with deep rose-colored flowers in corymbs.</cd> -- <col><b>Sheep pest</b></col> <fld>(Bot.)</fld>, <cd>an Australian plant (<spn>Acaena ovina</spn>) related to the burnet. The fruit is covered with barbed spines, by which it adheres to the wool of sheep.</cd> -- <col><b>Sheep run</b></col>, <cd>an extensive tract of country where sheep range and graze.</cd> -- <col><b>Sheep's beard</b></col> <fld>(Bot.)</fld>, <cd>a cichoraceous herb (<spn>Urospermum Dalechampii</spn>) of Southern Europe; -- so called from the conspicuous pappus of the achenes.</cd> -- <col><b>Sheep's bit</b></col><cd> <fld>(Bot.)</fld>, a European herb (<spn>Jasione montana</spn>) having much the appearance of scabious.</cd> -- <col><b>Sheep pox</b></col> <fld>(Med.)</fld>, <cd>a contagious disease of sheep, characterixed by the development of vesicles or pocks upon the skin.</cd> -- <col><b>Sheep scabious</b></col>. <fld>(Bot.)</fld> <cd>Same as <er>Sheep's bit</er>.</cd> -- <col><b>Sheep shears</b></col>, <cd>shears in which the blades form the two ends of a steel bow, by the elasticity of which they open as often as pressed together by the hand in cutting; -- so called because used to cut off the wool of sheep.</cd> -- <col><b>Sheep sorrel</b></col>. <fld>(Bot.)</fld>, <cd>a prerennial herb (<spn>Rumex Acetosella</spn>) growing naturally on poor, dry, gravelly soil. Its leaves have a pleasant acid taste like sorrel.</cd> -- <col><b>Sheep's-wool</b></col> <fld>(Zool.)</fld>, <cd>the highest grade of Florida commercial sponges (<spn>Spongia equina</spn>, variety <varn>gossypina</varn>).</cd> -- <col><b>Sheep tick</b></col> <fld>(Zool.)</fld>, <cd>a wingless parasitic insect (<spn>Melophagus ovinus</spn>) belonging to the Diptera. It fixes its proboscis in the skin of the sheep and sucks the blood, leaving a swelling. Called also <altname>sheep pest</altname>, and <altname>sheep louse</altname>.</cd> -- <col><b>Sheep walk</b></col>, <cd>a pasture for sheep; a sheep run.</cd> -- <col><b>Wild sheep</b></col>. <fld>(Zool.)</fld> <cd>See <er>Argali</er>, <er>Mouflon</er>, and <er>Oorial</er>.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sheepback</ent><br/
<hw>Sheep"back`</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Geol.)</fld> <def>A rounded knoll of rock resembling the back of a sheep. -- produced by glacial action. Called also <altname>roche moutonn<eacute/e</altname>; -- usually in the plural.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sheepberry</ent><br/
<hw>Sheep"ber`ry</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Bot.)</fld> <def>The edible fruit of a small North American tree of the genus <gen>Viburnum</gen> (<spn>Viburnum Lentago</spn>), having white flowers in flat cymes; also, the tree itself. Called also <altname>nannyberry</altname>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sheepbite</ent><br/
<hw>Sheep"bite`</hw> <pr>(?)</pr>, <pos>v. i.</pos> <def>To bite or nibble like a sheep; hence, to practice petty thefts.</def> <mark>[Obs.]</mark>  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sheepbiter</ent><br/
<hw>Sheep"bit`er</hw> <pr>(?)</pr>, <pos>n.</pos> <def>One who practices petty thefts.</def> <mark>[Obs.]</mark>  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>There are political <qex>sheepbiters</qex> as well as pastoral; betrayers of public trusts as well as of private.</q> <rj><qau>L'Estrange.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sheepcote</ent><br/
<ent>Sheepcot</ent><br/
<mhw>{ <hw>Sheep"cot`</hw> <pr>(?)</pr>, <hw>Sheep"cote`</hw> <pr>(?)</pr>, }</mhw> <pos>n.</pos> <def>A small inclosure for sheep; a pen; a fold.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sheep-faced</ent><br/
<hw>Sheep"-faced`</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Over-bashful; sheepish.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sheepfold</ent><br/
<hw>Sheep"fold`</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A fold or pen for sheep; a place where sheep are collected or confined.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sheep-headed</ent><br/
<hw>Sheep"-head`ed</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Silly; simple-minded; stupid.</def>  <rj><au>Taylor (1630)</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sheephook</ent><br/
<hw>Sheep"hook`</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A hook fastened to pole, by which shepherds lay hold on the legs or necks of their sheep; a shepherd's crook.</def>  <rj><au>Dryden.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sheepish</ent><br/
<hw>Sheep"ish</hw>, <pos>a.</pos> <sn>1.</sn> <def>Of or pertaining to sheep.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Like a sheep; bashful; over-modest; meanly or foolishly diffident; timorous to excess.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Wanting change of company, he will, when he comes abroad, be a <qex>sheepish</qex> or conceited creature.</q> <rj><qau>Locke.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p>-- <wordforms><wf>Sheep"ish*ly</wf>, <pos>adv.</pos> -- <wf>Sheep"ish*ness</wf>, <pos>n.</pos></wordforms><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sheepmaster</ent><br/
<hw>Sheep"mas`ter</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A keeper or feeder of sheep; also, an owner of sheep.</def>  <rj><au>2 Kings iii. 4.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sheeprack</ent><br/
<hw>Sheep"rack`</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Zool.)</fld> <def>The starling.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sheep's-eye</ent><br/
<hw>Sheep's"-eye`</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A modest, diffident look; a loving glance; -- commonly in the plural.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>I saw her just now give him the languishing eye, as they call it; . . . of old called the <qex>sheep's-eye</qex>.</q> <rj><qau>Wycherley.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sheep's-foot</ent><br/
<hw>Sheep's-foot`</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A printer's tool consisting of a metal bar formed into a hammer head at one end and a claw at the other, -- used as a lever and hammer.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sheepshank</ent><br/
<hw>Sheep"shank`</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Naut.)</fld> <def>A hitch by which a rope may be temporarily shortened.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sheepshead</ent><br/
<hw>Sheeps"head`</hw> <pr>(<?/)</pr>, <pos>n.</pos> <ety>[So called because of the fancied resemblance of its head and front teeth to those of a sheep.]</ety> <fld>(Zool.)</fld> <def>A large and valuable sparoid food fish (<spn>Archosargus probatocephalus</spn> syn. <spn>Diplodus probatocephalus</spn>) found on the Atlantic coast of the United States.  It often weighs from ten to twelve pounds.</def><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ The name is also locally, in a loose way, applied to various other fishes, as the butterfish, the fresh-water drumfish, the parrot fish, the porgy, and the moonfish.</note><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sheep-shearer</ent><br/
<hw>Sheep"-shear`er</hw> <pr>(?)</pr>, <pos>n.</pos> <def>One who shears, or cuts off the wool from, sheep.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sheep-shearing</ent><br/
<hw>Sheep"-shear`ing</hw> <pr>(?)</pr>, <pos>n.</pos> <sn>1.</sn> <def>Act of shearing sheep.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A feast at the time of sheep-shearing.</def>  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sheepskin</ent><br/
<hw>Sheep"skin`</hw> <pr>(?)</pr>, <pos>n.</pos> <sn>1.</sn> <def>The skin of a sheep; or, leather prepared from it.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A diploma; -- so called because usually written or printed on parchment prepared from the skin of the sheep.</def> <mark>[College Cant]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sheepsplit</ent><br/
<hw>Sheep"split`</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A split of a sheepskin; one of the thin sections made by splitting a sheepskin with a cutting knife or machine.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sheepy</ent><br/
<hw>Sheep"y</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Resembling sheep; sheepish.</def>  <rj><au>Testament of Love.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sheer</ent><br/
<hw>Sheer</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[OE. <ets>shere</ets>, <ets>skere</ets>, pure, bright, Icel. <ets>sk<?/rr</ets>; akin to <ets>sk<imac/rr</ets>, AS. <ets>sc<imac/r</ets>, OS. <ets>sk<imac/ri</ets>, MHG. <ets>sch<imac/r</ets>, G. <ets>schier</ets>, Dan. <ets>sk<?/r</ets>, Sw. <ets>sk<aum/r</ets>, Goth. <ets>skeirs</ets> clear, and E. <ets>shine</ets>. <root/157. See <er>Shine</er>, <pos>v. i.</pos>]</ety> <sn>1.</sn> <def>Bright; clear; pure; unmixed.</def> <ldquo/<xex>Sheer</xex> ale.<rdquo/  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Thou <qex>sheer</qex>, immaculate, and silver fountain.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Very thin or transparent; -- applied to fabrics; <as>as, <ex>sheer</ex> muslin</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>Being only what it seems to be; obvious; simple; mere; downright; <as>as, <ex>sheer</ex> folly; <ex>sheer</ex> nonsense</as>.</def> <ldquo/A <xex>sheer</xex> impossibility.<rdquo/  <rj><au>De Quincey.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>It is not a <qex>sheer</qex> advantage to have several strings to one's bow.</q> <rj><qau>M. Arnold.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>Stright up and down; vertical; prpendicular.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>A <qex>sheer</qex> precipice of a thousand feet.</q> <rj><qau>J. D. Hooker.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>It was at least<br/
Nine roods of <qex>sheer</qex> ascent.</q> <rj><qau>Wordsworth.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sheer</ent><br/
<hw>Sheer</hw>, <pos>adv.</pos> <def>Clean; quite; at once.</def> <mark>[Obs.]</mark>  <rj><au>Milton.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sheer</ent><br/
<hw>Sheer</hw>, <pos>v. t.</pos> <ety>[See <er>Shear</er>.]</ety> <def>To shear.</def> <mark>[Obs.]</mark>  <rj><au>Dryden.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sheer</ent><br/
<hw>Sheer</hw>, <pos>v. i.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Sheered</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Sheering</conjf>.]</vmorph> <ety>[D. <ets>sheren</ets> to shear, cut, withdraw, warp. See <er>Shear</er>.]</ety> <def>To decline or deviate from the line of the proper course; to turn aside; to swerve; <as>as, a ship <ex>sheers</ex> from her course; a horse <ex>sheers</ex> at a bicycle</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>To sheer off</b></col>, <cd>to turn or move aside to a distance; to move away.</cd> -- <col><b>To sheer up</b></col>, <cd>to approach obliquely.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sheer</ent><br/
<hw>Sheer</hw>, <pos>n.</pos> <sn>1.</sn> <fld>(Naut.)</fld> <sd>(a)</sd> <def>The longitudinal upward curvature of the deck, gunwale, and lines of a vessel, as when viewed from the side.</def> <sd>(b)</sd> <def>The position of a vessel riding at single anchor and swinging clear of it.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A turn or change in a course.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Give the canoe a <qex>sheer</qex> and get nearer to the shore.</q> <rj><qau>Cooper.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <pluf>pl.</pluf> <def>Shears See <er>Shear</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Sheer batten</b></col> <fld>(Shipbuilding)</fld>, <cd>a long strip of wood to guide the carpenters in following the sheer plan.</cd> -- <col><b>Sheer boom</b></col>, <cd>a boom slanting across a stream to direct floating logs to one side.</cd> -- <col><b>Sheer hulk</b></col>. <cd>See <cref>Shear hulk</cref>, under <er>Hulk</er>.</cd> -- <mcol><col><b>Sheer plan</b></col>, <it>or</it> <col><b>Sheer draught</b></col></mcol> <fld>(Shipbuilding)</fld>, <cd>a projection of the lines of a vessel on a vertical longitudinal plane passing through the middle line of the vessel.</cd> -- <col><b>Sheer pole</b></col> <fld>(Naut.)</fld>, <cd>an iron rod lashed to the shrouds just above the dead-eyes and parallel to the ratlines.</cd> -- <col><b>Sheer strake</b></col> <fld>(Shipbuilding)</fld>, <cd>the strake under the gunwale on the top side.</cd> <au>Totten.</au> -- <col><b>To break sheer</b></col> <fld>(Naut.)</fld>, <cd>to deviate from sheer, and risk fouling the anchor.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sheerly</ent><br/
<hw>Sheer"ly</hw> <pr>(?)</pr>, <pos>adv.</pos> <def>At once; absolutely.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sheerwater</ent><br/
<hw>Sheer"wa`ter</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Zool.)</fld> <def>The shearwater.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sheet</ent><br/
<hw>Sheet</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[OE. <ets>shete</ets>, <ets>schete</ets>, AS. <ets>sc<emac/te</ets>, <ets>sc<ymac/te</ets>, fr. <ets>sce<aacute/t</ets> a projecting corner, a fold in a garment (akin to D. <ets>schoot</ets> sheet, bosom, lap, G. <ets>schoss</ets> bosom, lap, flap of a coat, Icel. <ets>skaut</ets>, Goth. <ets>skauts</ets> the hem of a garment); originally, that which shoots out, from the root of AS. <ets>sce<oacute/tan</ets> to shoot. <root/159.  See <er>Shoot</er>, <pos>v. t.</pos>]</ety> <def>In general, a large, broad piece of anything thin, as paper, cloth, etc.; a broad, thin portion of any substance; an expanded superficies.</def> Specifically: <sd>(a)</sd> <def>A broad piece of cloth, usually linen or cotton, used for wrapping the body or for a covering; especially, one used as an article of bedding next to the body.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>He fell into a trance, and saw heaven opened, and a certain vessel descending unto him, as it had been a great <qex>sheet</qex> knit at the four corners.</q> <rj><qau>Acts x. 10, 11.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>If I do die before thee, prithee, shroud me<br/
In one of those same <qex>sheets</qex>.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sd>(b)</sd> <def>A broad piece of paper, whether folded or unfolded, whether blank or written or printed upon; hence, a letter; a newspaper, etc.</def> <sd>(c)</sd> <def>A single signature of a book or a pamphlet;</def> in <pluf>pl.</pluf>, <def>the book itself.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>To this the following <qex>sheets</qex> are intended for a full and distinct answer.</q> <rj><qau>Waterland.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sd>(d)</sd> <def>A broad, thinly expanded portion of metal or other substance; <as>as, a <ex>sheet</ex> of copper, of glass, or the like; a plate; a leaf</as>.</def> <sd>(e)</sd> <def>A broad expanse of water, or the like.</def> <ldquo/The two beautiful <xex>sheets</xex> of water.<rdquo/ <au>Macaulay.</au> <sd>(f)</sd> <def>A sail.</def> <au>Dryden.</au> <sd>(g)</sd> <fld>(Geol.)</fld> <def>An extensive bed of an eruptive rock intruded between, or overlying, other strata.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <ety>[AS. <ets>sce<aacute/ta</ets>. See the Etymology above.]</ety> <fld>(Naut.)</fld> <sd>(a)</sd> <def>A rope or chain which regulates the angle of adjustment of a sail in relation in relation to the wind; -- usually attached to the lower corner of a sail, or to a yard or a boom.</def> <sd>(b)</sd> <pluf>pl.</pluf> <def>The space in the forward or the after part of a boat where there are no rowers; <as>as, fore <ex>sheets</ex>; stern <ex>sheets</ex>.</as></def><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ <xex>Sheet</xex> is often used adjectively, or in combination, to denote that the substance to the name of which it is prefixed is in the form of sheets, or thin plates or leaves; as, <xex>sheet</xex> brass, or <xex>sheet</xex>-brass; <xex>sheet</xex> glass, or <xex>sheet</xex>-glass; <xex>sheet</xex> gold, or <xex>sheet</xex>-gold; sheet iron, or <xex>sheet</xex>-iron, etc.</note><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>A sheet in the wind</b></col>, <cd>half drunk.</cd> <mark>[Sailors' Slang]</mark> -- <col><b>Both sheets in the wind</b></col>, <cd>very drunk.</cd> <mark>[Sailors' Slang]</mark> -- <col><b>In sheets</b></col>, <cd>lying flat or expanded; not folded, or folded but not bound; -- said especially of printed sheets.</cd> -- <col><b>Sheet bend</b></col> <fld>(Naut.)</fld>, <cd>a bend or hitch used for temporarily fastening a rope to the bight of another rope or to an eye.</cd> -- <mcol><col><b>Sheet lightning</b></col>, <col><b>Sheet piling</b></col></mcol>, <cd>etc. See under <er>Lightning</er>, <er>Piling</er>, etc.</cd></cs>
<-- Three sheets to the wind, very drunk (now more common than "both sheets in the wind" --><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sheet</ent><br/
<hw>Sheet</hw>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Sheeted</conjf>; <pos>p. pr. & vb. n.</pos> <conjf>Sheeting</conjf>.]</vmorph> <sn>1.</sn> <def>To furnish with a sheet or sheets; to wrap in, or cover with, a sheet, or as with a sheet.</def> <ldquo/The <xex>sheeted</xex> dead.<rdquo/ <ldquo/When snow the pasture <xex>sheets</xex>.<rdquo/  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><-- p. 1327 --></p>

<p><sn>2.</sn> <def>To expand, as a sheet.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The star shot flew from the welkin blue,<br/
As it fell from the <qex>sheeted</qex> sky.</q> <rj><qau>J. R. Drake.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>To sheet home</b></col> <fld>(Naut.)</fld>, <cd>to haul upon a sheet until the sail is as flat, and the clew as near the wind, as possible.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sheet anchor</ent><br/
<hw>Sheet" an"chor</hw> <pr>(?)</pr>. <ety>[OE. <ets>scheten</ets> to shoot, AS. <ets>sce<oacute/tan</ets>; cf. OE. <ets>shoot anchor</ets>. See <er>Shoot</er>, <pos>v. t.</pos>]</ety> <sn>1.</sn> <fld>(Naut.)</fld> <def>A large anchor stowed on shores outside the waist of a vessel; -- called also <altname>waist anchor</altname>. See the Note under <er>Anchor</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Anything regarded as a sure support or dependence in danger; the best hope or refuge.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sheet cable</ent><br/
<hw>Sheet" ca"ble</hw> <pr>(?)</pr>. <fld>(Naut.)</fld> <def>The cable belonging to the sheet anchor.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sheet chain</ent><br/
<hw>Sheet" chain"</hw> <pr>(?)</pr>. <fld>(Naut.)</fld> <def>A chain sheet cable.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sheetful</ent><br/
<hw>Sheet"ful</hw> <pr>(?)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Sheetfuls</plw> <pr>(<?/)</pr>.</plu> <def>Enough to fill a sheet; as much as a sheet can hold.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sheeting</ent><br/
<hw>Sheet"ing</hw>, <pos>n.</pos> <sn>1.</sn> <def>Cotton or linen cloth suitable for bed sheets. It is sometimes made of double width.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Hydraul. Engin.)</fld> <def>A lining of planks or boards (rarely of metal) for protecting an embankment.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>The act or process of forming into sheets, or flat pieces; also, material made into sheets.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sheik</ent><br/
||<hw>Sheik</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Ar. <ets>sheikh</ets>, <ets>shaykh</ets>, a venerable old man, a chief, fr. <ets>sh<amac/kha</ets> to grow or be old.]</ety> <def>The head of an Arab family, or of a clan or a tribe; also, the chief magistrate of an Arab village. The name is also applied to Mohammedan ecclesiastics of a high grade.</def> <altsp>[Written also <asp>scheik</asp>, <asp>shaik</asp>, <asp>sheikh</asp>.]</altsp><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sheiling</ent><br/
<ent>Sheil</ent><br/
<mhw>{ <hw>Sheil</hw> <pr>(sh<emac/l)</pr>, <hw>Sheil"ing</hw>, }</mhw> <pos>n.</pos> <def>See <er>Sheeling</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shekel</ent><br/
<hw>Shek"el</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Heb. <ets>shegel</ets>, fr. <ets>sh<amac/gal</ets> to weigh.]</ety> <sn>1.</sn> <def>An ancient weight and coin used by the Jews and by other nations of the same stock.</def><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ A common estimate makes the shekel equal in weight to about 130 grains for gold, 224 grains for silver, and 450 grains for copper, and the approximate values of the coins are (gold) $5.00, (silver) 60 cents, and (copper half shekel), one and one half cents.</note><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <pluf>pl.</pluf> <def>A jocose term for <xex>money</xex>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shekinah</ent><br/
<hw>She*ki"nah</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Heb Talmud <ets>shek<imac/n<amac/h</ets>, fr. <ets>sh<amac/kan</ets> to inhabit.]</ety> <def>The visible majesty of the Divine Presence, especially when resting or dwelling between the cherubim on the mercy seat, in the Tabernacle, or in the Temple of Solomon; -- a term used in the Targums and by the later Jews, and adopted by Christians.</def> <altsp>[Written also <asp>Shechinah</asp>.]</altsp>  <rj><au>Dr. W. Smith (Bib. Dict.)</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sheld</ent><br/
<hw>Sheld</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[OE., fr. <ets>sheld</ets> a shield, probably in allusion to the ornamentation of shields. See <er>Shield</er>.]</ety> <def>Variegated; spotted; speckled; piebald.</def> <mark>[Prov. Eng.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sheldaple</ent><br/
<ent>Sheldafle</ent><br/
<mhw>{ <hw>Sheld"a*fle</hw> <pr>(?)</pr>, <hw>Sheld"a*ple</hw> <pr>(?)</pr>, }</mhw> <pos>n.</pos> <ety>[Perhaps for <ets>sheld dapple</ets>.  Cf. <er>Sheldrake</er>.]</ety> <fld>(Zool.)</fld> <def>A chaffinch.</def> <altsp>[Written also <asp>sheldapple</asp>, and <asp>shellapple</asp>.]</altsp><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sheldfowl</ent><br/
<hw>Sheld"fowl`</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Zool.)</fld> <def>The common sheldrake.</def> <mark>[Prov. Eng.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sheldrake</ent><br/
<hw>Shel"drake`</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[<ets>Sheld</ets> + <ets>drake</ets>.]</ety> <sn>1.</sn> <fld>(Zool.)</fld> <def>Any one of several species of large Old World ducks of the genus <gen>Tadorna</gen> and allied genera, especially the European and Asiatic species. (<spn>Tadorna cornuta</spn> syn. <spn>Tadorna tadorna</spn>), which somewhat resembles a goose in form and habit, but breeds in burrows.</def><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ It has the head and neck greenish black, the breast, sides, and forward part of the back brown, the shoulders and middle of belly black, the speculum green, and the bill and frontal bright red.  Called also <altname>shelduck</altname>, <altname>shellduck</altname>, <altname>sheldfowl</altname>, <altname>skeelduck</altname>, <altname>bergander</altname>, <altname>burrow duck</altname>, and <altname>links goose</altname>.</note><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ The Australian sheldrake (<spn>Tadorna radja</spn>) has the head, neck, breast, flanks, and wing coverts white, the upper part of the back and a band on the breast deep chestnut, and the back and tail black. The chestnut sheldrake of Australia (<spn>Casarca tadornoides</spn>) is varied with black and chestnut, and has a dark green head and neck. The ruddy sheldrake, or Braminy duck (<spn>Casarca rutila</spn>), and the white-winged sheldrake (<spn>Casarca leucoptera</spn>), are related Asiatic species.</note><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Any one of the American mergansers.</def><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ The name is also loosely applied to other ducks, as the canvasback, and the shoveler.</note><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shelduck</ent><br/
<hw>Shel"duck`</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[<ets>Sheld</ets> variegated + <ets>duck</ets>.]</ety> <fld>(Zool.)</fld> <def>The sheldrake.</def> <altsp>[Written also <asp>shellduck</asp>.]</altsp><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shelf</ent><br/
<hw>Shelf</hw> <pr>(?)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Shelves</plw> <pr>(#)</pr>.</plu> <ety>[OE. <ets>shelfe</ets>, <ets>schelfe</ets>, AS. <ets>scylfe</ets>; akin to G. <ets>schelfe</ets>, Icel. <ets>skj<amac/lf</ets>. In senses 2 & 3, perhaps a different word (cf. <er>Shelve</er>, <pos>v. i.</pos>).]</ety> <sn>1.</sn> <fld>(Arch.)</fld> <def>A flat tablet or ledge of any material set horizontally at a distance from the floor, to hold objects of use or ornament.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A sand bank in the sea, or a rock, or ledge of rocks, rendering the water shallow, and dangerous to ships.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>On the tawny sands and <qex>shelves</qex>.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>On the secret <qex>shelves</qex> with fury cast.</q> <rj><qau>Dryden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <fld>(Mining)</fld> <def>A stratum lying in a very even manner; a flat, projecting layer of rock.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <fld>(Naut.)</fld> <def>A piece of timber running the whole length of a vessel inside the timberheads.</def>  <rj><au>D. Kemp.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>To lay on the shelf</b></col>, <cd>to lay aside as unnecessary or useless; to dismiss; to discard.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shelfy</ent><br/
<hw>Shelf"y</hw> <pr>(?)</pr>, <pos>a.</pos> <sn>1.</sn> <def>Abounding in shelves; full of dangerous shallows.</def> <ldquo/A <xex>shelfy</xex> coast.<rdquo/  <rj><au>Dryden.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Full of strata of rock.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>The tillable fields are in some places . . . so <qex>shelfy</qex> that the corn hath much ado to fasten its root.</q> <rj><qau>Carew.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shell</ent><br/
<hw>Shell</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[OE. <ets>shelle</ets>, <ets>schelle</ets>, AS. <ets>scell</ets>, <ets>scyll</ets>; akin to D. <ets>shel</ets>, Icel. <ets>skel</ets>, Goth. <ets>skalja</ets> a tile, and E. <ets>skill</ets>.  Cf. <er>Scale</er> of fishes, <er>Shale</er>, <er>Skill</er>.]</ety> <sn>1.</sn> <def>A hard outside covering, as of a fruit or an animal.</def> Specifically: <sd>(a)</sd> <def>The covering, or outside part, of a nut; <as>as, a hazelnut <ex>shell</ex></as>.</def> <sd>(b)</sd> <def>A pod.</def> <sd>(c)</sd> <def>The hard covering of an egg.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Think him as a serpent's egg, . . .<br/
And kill him in the <qex>shell</qex>.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sd>(d)</sd> <fld>(Zool.)</fld> <def>The hard calcareous or chitinous external covering of mollusks, crustaceans, and some other invertebrates. In some mollusks, as the cuttlefishes, it is internal, or concealed by the mantle. Also, the hard covering of some vertebrates, as the armadillo, the tortoise, and the like.</def> <sd>(e)</sd> <fld>(Zool.)</fld> <def>Hence, by extension, any mollusks having such a covering.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Mil.)</fld> <def>A hollow projectile, of various shapes, adapted for a mortar or a cannon, and containing an explosive substance, ignited with a fuse or by percussion, by means of which the projectile is burst and its fragments scattered. See <er>Bomb</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>The case which holds the powder, or charge of powder and shot, used with breechloading small arms.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>Any slight hollow structure; a framework, or exterior structure, regarded as not complete or filled in; <as>as, the <ex>shell</ex> of a house</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <def>A coarse kind of coffin; also, a thin interior coffin inclosed in a more substantial one.</def>  <rj><au>Knight.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>6.</sn> <def>An instrument of music, as a lyre, -- the first lyre having been made, it is said, by drawing strings over a tortoise shell.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>When Jubal struck the chorded <qex>shell</qex>.</q> <rj><qau>Dryden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>7.</sn> <def>An engraved copper roller used in print works.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>8.</sn> <pluf>pl.</pluf> <def>The husks of cacao seeds, a decoction of which is often used as a substitute for chocolate, cocoa, etc.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>9.</sn> <fld>(Naut.)</fld> <def>The outer frame or case of a block within which the sheaves revolve.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>10.</sn> <def>A light boat the frame of which is covered with thin wood or with paper; <as>as, a racing <ex>shell</ex></as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>11.</sn> <def>Something similar in form or action to an ordnance shell;</def> <specif>specif.:</specif> <sd>(a)</sd> <fld>(Fireworks)</fld> <def>A case or cartridge containing a charge of explosive material, which bursts after having been thrown high into the air. It is often elevated through the agency of a larger firework in which it is contained.</def> <sd>(b)</sd> <fld>(Oil Wells)</fld> <def>A torpedo.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><sn>12.</sn>  <def>A concave rough cast-iron tool in which a convex lens is ground to shape.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><sn>13.</sn>  <def>A gouge bit or shell bit.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><cs><col><b>Message shell</b></col>, <cd>a bombshell inside of which papers may be put, in order to convey messages.</cd> -- <col><b>Shell bit</b></col>, <cd>a tool shaped like a gouge, used with a brace in boring wood. See <er>Bit</er>, <pos>n.</pos>, 3.</cd> -- <col><b>Shell button</b></col>. <sd>(a)</sd> <cd>A button made of shell</cd>. <sd>(b)</sd> <cd>A hollow button made of two pieces, as of metal, one for the front and the other for the back, -- often covered with cloth, silk, etc.</cd> -- <col><b>Shell cameo</b></col>, <cd>a cameo cut in shell instead of stone.</cd> -- <col><b>Shell flower</b></col>. <fld>(Bot.)</fld> <cd>Same as <er>Turtlehead</er>.</cd> -- <col><b>Shell gland</b></col>. <fld>(Zool.)</fld> <sd>(a)</sd> <cd>A glandular organ in which the rudimentary shell is formed in embryonic mollusks</cd>. <sd>(b)</sd> <cd>A glandular organ which secretes the eggshells of various worms, crustacea, mollusks, etc.</cd> -- <col><b>Shell gun</b></col>, <cd>a cannon suitable for throwing shells.</cd> -- <col><b>Shell ibis</b></col> <fld>(Zool.)</fld>, <cd>the openbill of India.</cd> -- <col><b>Shell jacket</b></col>, <cd>an undress military jacket.</cd> -- <col><b>Shell lime</b></col>, <cd>lime made by burning the shells of shellfish.</cd> -- <col><b>Shell marl</b></col> <fld>(Min.)</fld>, <cd>a kind of marl characterized by an abundance of shells, or fragments of shells.</cd> -- <col><b>Shell meat</b></col>, <cd>food consisting of shellfish, or testaceous mollusks.</cd> <au>Fuller.</au> -- <col><b>Shell mound</b></col>. <cd>See under <er>Mound</er>.</cd> -- <col><b>Shell of a boiler</b></col>, <cd>the exterior of a steam boiler, forming a case to contain the water and steam, often inclosing also flues and the furnace; the barrel of a cylindrical, or locomotive, boiler.</cd> -- <col><b>Shell road</b></col>, <cd>a road of which the surface or bed is made of shells, as oyster shells.</cd> -- <col><b>Shell sand</b></col>, <cd>minute fragments of shells constituting a considerable part of the seabeach in some places.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shell</ent><br/
<hw>Shell</hw>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Shelled</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Shelling</conjf>.]</vmorph> <sn>1.</sn> <def>To strip or break off the shell of; to take out of the shell, pod, etc.; <as>as, to <ex>shell</ex> nuts or pease; to <ex>shell</ex> oysters.</as></def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To separate the kernels of (an ear of Indian corn, wheat, oats, etc.) from the cob, ear, or husk.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>To throw shells or bombs upon or into; to bombard; <as>as, to <ex>shell</ex> a town</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>To shell out</b></col>, <cd>to distribute freely; to bring out or pay, as money.</cd> <mark>[Colloq.]</mark></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shell</ent><br/
<hw>Shell</hw>, <pos>v. i.</pos> <sn>1.</sn> <def>To fall off, as a shell, crust, etc.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To cast the shell, or exterior covering; to fall out of the pod or husk; <as>as, nuts <ex>shell</ex> in falling</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>To be disengaged from the ear or husk; <as>as, wheat or rye <ex>shells</ex> in reaping</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shell-lac</ent><br/
<ent>Shellac</ent><br/
<mhw>{ <hw>Shel"lac`</hw>, <hw>Shell"-lac`</hw> }</mhw> <pr>(?)</pr>, <pos>n.</pos> <ety>[<ets>Shell</ets> + <ets>lac</ets> a resinous substance; cf. D. <ets>shellak</ets>, G. <ets>schellack</ets>.]</ety> <sn>1.</sn> <def>Lac which has been reduced to a thin crust.  See the Note under 2d <er>Lac</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A solution of shellac{1} in alcohol or other volatile solvent, used as a varnish.</def><br/
[<source>PJC</source>]</p>

<p><sn>3.</sn> <def>A phonograph record, made of a material containing shellac; -- no longer manufactured.</def> <mark>[Obsolescent]</mark> <au>RHUD</au><br/
[<source>PJC</source>]</p>

<p><ent>Shellapple</ent><br/
<hw>Shell"ap`ple</hw>, <pos>n.</pos> <fld>(Zool.)</fld> <def>See <er>Sheldafle</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shellbark</ent><br/
<hw>Shell"bark`</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Bot.)</fld> <def>A species of hickory (<spn>Carya alba</spn>) whose outer bark is loose and peeling; a shagbark; also, its nut.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shelled</ent><br/
<hw>Shelled</hw> <pr>(?)</pr>, <pos>a.</pos> <fld>(Zool.)</fld> <def>Having a shell.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sheller</ent><br/
<hw>Shell"er</hw> <pr>(?)</pr>, <pos>n.</pos> <def>One who, or that which, shells; <as>as, an oyster <ex>sheller</ex>; a corn <ex>sheller</ex>.</as></def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shellfish</ent><br/
<hw>Shell"fish`</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Zool.)</fld> <def>Any aquatic animal whose external covering consists of a shell, either testaceous, as in oysters, clams, and other mollusks, or crustaceous, as in lobsters and crabs.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shelling</ent><br/
<hw>Shell"ing</hw>, <pos>n.</pos> <def>Groats; hulled oats.</def>  <rj><au>Simmonds.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shell-less</ent><br/
<hw>Shell"-less</hw>, <pos>a.</pos> <def>Having no shell.</def>  <rj><au>J. Burroughs.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shellproof</ent><br/
<hw>Shell"proof`</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Capable of resisting bombs or other shells; bombproof.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shellwork</ent><br/
<hw>Shell"work`</hw> <pr>(?)</pr>, <pos>n.</pos> <def>Work composed of shells, or adorned with them.</def>  <rj><au>Cotgrave.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shelly</ent><br/
<hw>Shell"y</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Abounding with shells; consisting of shells, or of a shell.</def> <ldquo/The <xex>shelly</xex> shore.<rdquo/  <rj><au>Prior.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Shrinks backward in his <qex>shelly</qex> cave.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shelter</ent><br/
<hw>Shel"ter</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Cf. OE. <ets>scheltrun</ets>, <ets>shiltroun</ets>, <ets>schelltrome</ets>, <ets>scheldtrome</ets>, a guard, squadron, AS. <ets>scildtruma</ets> a troop of men with shields; <ets>scild</ets> shield + <ets>truma</ets> a band of men. See <er>Shield</er>, <pos>n.</pos>]</ety> <sn>1.</sn> <def>That which covers or defends from injury or annoyance; a protection; a screen.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The sick and weak the healing plant shall aid,<br/
From storms a <qex>shelter</qex>, and from heat a shade.</q> <rj><qau>Pope.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>One who protects; a guardian; a defender.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Thou [God] hast been a <qex>shelter</qex> for me.</q> <rj><qau>Ps. lxi. 3.  </qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>The state of being covered and protected; protection; security.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Who into <qex>shelter</qex> takes their tender bloom.</q> <rj><qau>Young.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Shelter tent</b></col>,<cd>a small tent made of pieces of cotton duck arranged to button together. In field service the soldiers carry the pieces.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><syn><b>Syn.</b> -- Asylum; refuge; retreat; covert; sanctuary; protection; defense; security.</syn><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shelter</ent><br/
<hw>Shel"ter</hw> <pr>(?)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Sheltered</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Sheltering</conjf>.]</vmorph> <sn>1.</sn> <def>To be a shelter for; to provide with a shelter; to cover from injury or annoyance; to shield; to protect.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Those ruins <qex>sheltered</qex> once his sacred head.</q> <rj><qau>Dryden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>You have no convents . . . in which such persons may be received and <qex>sheltered</qex>.</q> <rj><qau>Southey.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To screen or cover from notice; to disguise.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>In vain I strove to cheek my growing flame,<br/
Or <qex>shelter</qex> passion under friendship's name.</q> <rj><qau>Prior.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>To betake to cover, or to a safe place; -- used reflexively.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>They <qex>sheltered</qex> themselves under a rock.</q> <rj><qau>Abp. Abbot.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shelter</ent><br/
<hw>Shel"ter</hw>, <pos>v. i.</pos> <def>To take shelter.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>There oft the Indian herdsman, shunning heat,<br/
<qex>Shelters</qex> in cool.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shelterless</ent><br/
<hw>Shel"ter*less</hw>, <pos>a.</pos> <def>Destitute of shelter or protection.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Now sad and <qex>shelterless</qex> perhaps she lies.</q> <rj><qau>Rowe.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sheltery</ent><br/
<hw>Shel"ter*y</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Affording shelter.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shelty</ent><br/
<ent>Sheltie</ent><br/
<mhw>{ <hw>Shel"tie</hw> <pr>(?)</pr>, <hw>Shel"ty</hw> <pr>(?)</pr>, }</mhw> <pos>n.</pos> <def>A Shetland pony.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shelve</ent><br/
<hw>Shelve</hw> <pr>(?)</pr>, <pos>v. t.</pos> <sn>1.</sn> <def>To furnish with shelves; <as>as, to <ex>shelve</ex> a closet or a library</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To place on a shelf. Hence: To lay on the shelf; to put aside; to dismiss from service; to put off indefinitely; <as>as, to <ex>shelve</ex> an officer; to <ex>shelve</ex> a claim</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shelve</ent><br/
<hw>Shelve</hw>, <pos>v. i.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Shelved</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Shelving</conjf>.]</vmorph> <ety>[Perhapss originally from the same source as <ets>shallow</ets>, but influenced by <ets>shelf</ets> a ledge, a platform.]</ety> <def>To incline gradually; to be slopping; <as>as, the bottom <ex>shelves</ex> from the shore</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shelving</ent><br/
<hw>Shelv"ing</hw>, <pos>a.</pos> <def>Sloping gradually; inclining; <as>as, a <ex>shelving</ex> shore</as>.</def> <au>Shak.</au> <ldquo/<xex>Shelving</xex> arches.<rdquo/  <rj><au>Addison.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shelving</ent><br/
<hw>Shelv"ing</hw>, <pos>n.</pos> <sn>1.</sn> <def>The act of fitting up shelves; <as>as, the job of <ex>shelving</ex> a closet</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>The act of laying on a shelf, or on the shelf; putting off or aside; <as>as, the <ex>shelving</ex> of a claim</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>Material for shelves; shelves, collectively.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shelvy</ent><br/
<hw>Shelv"y</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Sloping gradually; shelving.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The shore was <qex>shelving</qex> and shallow.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shemite</ent><br/
<hw>Shem"ite</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A descendant of Shem.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shemitish</ent><br/
<ent>Shemitic</ent><br/
<mhw>{ <hw>Shem*it"ic</hw> <pr>(?)</pr>, <hw>Shem"i*tish</hw> <pr>(?)</pr>, }</mhw> <pos>a.</pos> <def>Of or pertaining to <xex>Shem</xex>, the son of Noah, or his descendants. See <er>Semitic</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shemitism</ent><br/
<hw>Shem"i*tism</hw> <pr>(?)</pr>, <pos>n.</pos> <def>See <er>Semitism</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shend</ent><br/
<hw>Shend</hw> <pr>(?)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Shent</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Shending</conjf>.]</vmorph> <ety>[AS. <ets>scendan</ets> to disgrace, bring to shame, from <ets>sceand</ets>, <ets>sceond</ets>, disgrace, dishonor, shame; akin to G. <ets>schande</ets>, Goth. <ets>skanda</ets>. See <er>Shame</er>, <pos>n.</pos>]</ety> <sn>1.</sn> <def>To injure, mar, spoil, or harm.</def> <mark>[Obs.]</mark> <ldquo/Loss of time <xex>shendeth</xex> us.<rdquo/  <rj><au>Chaucer.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>I fear my body will be <qex>shent</qex>.</q> <rj><qau>Dryden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To blame, reproach, or revile; to degrade, disgrace, or put to shame.</def> <mark>[Archaic]</mark>  <rj><au>R. Browning.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>The famous name of knighthood foully <qex>shend</qex>.</q> <rj><qau>Spenser.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>She passed the rest as Cynthia doth <qex>shend</qex><br/
The lesser stars.</q> <rj><qau>Spenser.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shendful</ent><br/
<hw>Shend"ful</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Destructive; ruinous; disgraceful.</def> <mark>[Obs.]</mark> -- <wordforms><wf>Shend"ful*ly</wf>, <pos>adv.</pos> <mark>[Obs.]</mark></wordforms>  <rj><au>Fabyan.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shendship</ent><br/
<hw>Shend"ship</hw>, <pos>n.</pos> <def>Harm; ruin; also, reproach; disgrace.</def> <mark>[Obs.]</mark>  <rj><au>Chaucer.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shent</ent><br/
<hw>Shent</hw> <pr>(?)</pr>, <def><pos>obs. 3d pers. sing. pres.</pos> of <er>Shend</er>, for <xex>shendeth</xex>.</def>  <rj><au>Chaucer.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shent</ent><br/
<hw>Shent</hw>, <pos>v. t.</pos> <def>To shend.</def> <mark>[Obs.]</mark>  <rj><au>Chaucer.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sheol</ent><br/
<hw>She"ol</hw> <pr>(sh<emac/"<omac/l)</pr>, <pos>n.</pos> <ety>[Heb. <ets>sh<ecr/<omac/l</ets>.]</ety> <def>The place of departed spirits; Hades; also, the grave.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>For thou wilt not leave my soul to <qex>sheol</qex>.</q> <rj><qau>Ps. xvi. 10. (Rev. Ver.)</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shepen</ent><br/
<hw>Shep"en</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A stable; a shippen.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>The <qex>shepne</qex> brenning with the blacke smoke.</q> <rj><qau>Chaucer.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shepherd</ent><br/
<hw>Shep"herd</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[OE. <ets>schepherde</ets>, <ets>schephirde</ets>, AS. <ets>sce<aacute/phyrde</ets>; <ets>sce<aacute/p</ets> sheep + <ets>hyrde</ets>, <ets>hirde</ets>, <ets>heorde</ets>, a herd, a guardian.  See <er>Sheep</er>, and <er>Herd</er>.]</ety> <sn>1.</sn> <def>A man employed in tending, feeding, and guarding sheep, esp. a flock grazing at large.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>The pastor of a church; one with the religious guidance of others.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Shepherd bird</b></col> <fld>(Zool.)</fld>, <cd>the crested screamer. See <er>Screamer</er>.</cd> -- <col><b>Shepherd dog</b></col> <fld>(Zool.)</fld>, <cd>a breed of dogs used largely for the herding and care of sheep. There are several kinds, as the collie, or Scotch shepherd dog, and the English shepherd dog. Called also <altname>shepherd's dog</altname>.</cd> -- <col><b>Shepherd dog</b></col>, <cd>a name of Pan.</cd> <au>Keats.</au> -- <col><b>Shepherd kings</b></col>, <cd>the chiefs of a nomadic people who invaded Egypt from the East in the traditional period, and conquered it, at least in part. They were expelled after about five hundred years, and attempts have been made to connect their expulsion with narrative in the book of Exodus.</cd> -- <col><b>Shepherd's club</b></col> <fld>(Bot.)</fld>, <cd>the common mullein. See <er>Mullein</er>.</cd> -- <col><b>Shepherd's crook</b></col>, <cd>a long staff having the end curved so as to form a large hook, -- used by shepherds.</cd> -- <col><b>Shepherd's needle</b></col> <fld>(Bot.)</fld>, <cd>the lady's comb.</cd> -- <col><b>Shepherd's plaid</b></col>, <cd>a kind of woolen cloth of a checkered black and white pattern.</cd> -- <col><b>Shephered spider</b></col> <fld>(Zool.)</fld>, <cd>a daddy longlegs, or harvestman.</cd> -- <mcol><col><b>Shepherd's pouch</b></col>, <it>or</it> <col><b>Shepherd's purse</b></col></mcol> <fld>(Bot.)</fld>, <cd>an annual cruciferous plant (<spn>Capsella Bursapastoris</spn>) bearing small white flowers and pouchlike pods. See <xex>Illust.</xex> of <er>Silicle</er>.</cd> -- <mcol><col><b>Shepherd's rod</b></col>, <it>or</it> <col><b>Shepherd's staff</b></col></mcol> <fld>(Bot.)</fld>, <cd>the small teasel.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shepherd</ent><br/
<hw>Shep"herd</hw>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Shepherded</conjf>; <pos>p. pr. & vb. n.</pos> <conjf>Shepherding</conjf>.]</vmorph> <def>To tend as a shepherd; to guard, herd, lead, or drive, as a shepherd.</def> <mark>[Poetic]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>White, fleecy clouds . . . </q><br/
[<source>1913 Webster</source>]</p>

<p><q><qex>Shepherded</qex> by the slow, unwilling wind.</q>
 <rj><qau>Shelley.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shepherdess</ent><br/
<hw>Shep"herd*ess</hw>, <pos>n.</pos> <def>A woman who tends sheep; hence, a rural lass.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>She put herself into the garb of a <qex>shepherdess</qex>.</q> <rj><qau>Sir P. Sidney.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shepherdia</ent><br/
<hw>Shep*her"di*a</hw> <pr>(?)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Shepherdias</plw> <pr>(#)</pr>.</plu> <ety>[NL. So called from John <ets>Shepherd</ets>, an English botanist.]</ety> <fld>(Bot.)</fld> <def>A genus of shrubs having silvery scurfy leaves, and belonging to the same family as Elaeagnus; also, any plant of this genus. See <cref>Buffalo berry</cref>, under <er>Buffalo</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><-- p. 1328 --></p>

<p><ent>Shepherdish</ent><br/
<hw>Shep"herd*ish</hw> <pr>(?)</pr>, <pos>n.</pos> <def>Resembling a shepherd; suiting a shepherd; pastoral.</def>  <rj><au>Sir T. Sidney.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shepherdism</ent><br/
<hw>Shep"herd*ism</hw> <pr>(?)</pr>, <pos>n.</pos> <def>Pastoral life or occupation.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shepherdling</ent><br/
<hw>Shep"herd*ling</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A little shepherd.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shepherdly</ent><br/
<hw>Shep"herd*ly</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Resembling, or becoming to, a shepherd; pastoral; rustic.</def> <mark>[R.]</mark>  <rj><au>Jer. Taylor.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shepster</ent><br/
<hw>Shep"ster</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A seamstress.</def> <mark>[Obs.]</mark>  <rj><au>Caxton.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sherardize</ent><br/
<hw>Sher"ard*ize</hw> <pr>(?)</pr>, <pos>v. t.</pos> <ety>[From <etsep>Sherard</etsep> Cowper-Coles, the inventor.]</ety> <fld>(Metal.)</fld> <def>To subject to the process of vapor galvanizing (which see, below).</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Sherbet</ent><br/
<hw>Sher"bet</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Ar. <ets>sherbet</ets>, <ets>shorbet</ets>, <ets>sharbat</ets>, properly, one drink or sip, a draught, beverage, from <ets>shariba</ets> to drink.  Cf. <er>Sorbet</er>, <er>Sirup</er>, <er>Shrub</er> a drink.]</ety> <sn>1.</sn> <def>A refreshing drink, common in the East, made of the juice of some fruit, diluted, sweetened, and flavored in various ways; <as>as, orange <ex>sherbet</ex>; lemon <ex>sherbet</ex>; raspberry <ex>sherbet</ex>, etc.</as></def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A flavored water ice.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>A preparation of bicarbonate of soda, tartaric acid, sugar, etc., variously flavored, for making an effervescing drink; -- called also <altname>sherbet powder</altname>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sherd</ent><br/
<hw>Sherd</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A fragment; -- now used only in composition, as in pot<ex>sherd</ex>. See <er>Shard</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The thigh . . . which all in <qex>sherds</qex> it drove.</q> <rj><qau>Chapman.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sherif</ent><br/
<ent>Shereef</ent><br/
<mhw>{ ||<hw>Sher"eef</hw> <pr>(?)</pr>, ||<hw>Sher"if</hw> <pr>(?)</pr>, }</mhw> <pos>n.</pos> <ety>[Ar. <ets>sher<imac/f</ets> noble, holy, n., a prince.]</ety> <def>A member of an Arab princely family descended from Mohammed through his son-in-law Ali and daughter Fatima. The Grand Shereef is the governor of Mecca.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sheriat</ent><br/
||<hw>Sher"i*at</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Turk. <ets>sher<imac/ 'at</ets>]</ety> <def>The sacred law of the Turkish empire.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sheriff</ent><br/
<hw>Sher"iff</hw>, <pos>n.</pos> <ety>[OE. <ets>shereve</ets>, AS. <ets>sc<imac/r-ger<?/fa</ets>; <ets>sc<imac/r</ets> a shire + <ets>ger<?/fa</ets> a reeve. See Shire, and <er>Reeve</er>, and cf. <er>Shrievalty</er>.]</ety> <def>The chief officer of a shire or county, to whom is intrusted the execution of the laws, the serving of judicial writs and processes, and the preservation of the peace.</def><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ In England, sheriffs are appointed by the king. In the United States, sheriffs are elected by the legislature or by the citizens, or appointed and commissioned by the executive of the State. The office of sheriff in England is judicial and ministerial. In the United States, it is mainly ministerial. The sheriff, by himself or his deputies, executes civil and criminal process throughout the county, has charge of the jail and prisoners, attends courts, and keeps the peace. His judicial authority is generally confined to ascertaining damages on writs of inquiry and the like. <xex>Sheriff</xex>, in Scotland, called <xex>sheriff depute</xex>, is properly a judge, having also certain ministerial powers. <xex>Sheriff clerk</xex> is the clerk of the Sheriff's Court in Scotland. <xex>Sheriff's Court in London</xex> is a tribunal having cognizance of certain personal actions in that city.</note>  <rj><au>Wharton, Tomlins. Erskine.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sheriffwick</ent><br/
<ent>Sheriffship</ent><br/
<ent>Sheriffry</ent><br/
<ent>Sheriffdom</ent><br/
<ent>Sheriffalty</ent><br/
<mhw>{ <hw>Sher"iff*al*ty</hw> <pr>(?)</pr>, <hw>Sher"iff*dom</hw> <pr>(?)</pr>, <hw>Sher"iff*ry</hw> <pr>(?)</pr>, <hw>Sher"iff*ship</hw> <pr>(?)</pr>, <hw>Sher"iff*wick</hw> <pr>(?)</pr>, <pos>n.</pos> }</mhw> <def>The office or jurisdiction of sheriff. See <er>Shrievalty</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shern</ent><br/
<hw>Shern</hw> <pr>(?)</pr>, <pos>n.</pos> <def>See <er>Shearn</er>.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sherris</ent><br/
<hw>Sher"ris</hw> <pr>(?)</pr>, <pos>n.</pos> <def>Sherry.</def> <mark>[Obs.]</mark>  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sherry</ent><br/
<hw>Sher"ry</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[So called from <ets>Xeres</ets>, a Spanish town near Cadiz, <it>x</it> in Spanish having been formerly pronounced like <it>sh</it> in English.]</ety> <def>A Spanish light-colored dry wine, made in Andalusia. As prepared for commerce it is colored a straw color or a deep amber by mixing with it cheap wine boiled down.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Sherry cobbler</b></col>, <cd>a beverage prepared with sherry wine, water, lemon or orange, sugar, ice, etc., and usually imbided through a straw or a glass tube.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sherryvallies</ent><br/
<hw>Sher"ry*val`lies</hw> <pr>(?)</pr>, <pos>n. pl.</pos> <ety>[Cf. Sp. <ets>zaraquelles</ets> wide breeches or overalls.]</ety> <def>Trousers or overalls of thick cloth or leather, buttoned on the outside of each leg, and generally worn to protect other trousers when riding on horseback.</def> <mark>[Local, U.S.]</mark>  <rj><au>Bartlett.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shet</ent><br/
<hw>Shet</hw> <pr>(?)</pr>, <pos>v. t. & i.</pos> <vmorph>[<pos>imp.</pos> <conjf>Shet</conjf>. (<mark>Obs.</mark> <conjf>Shette</conjf> <pr>(<?/ <or/ <?/)</pr>); <pos>p. pr.</pos> <conjf>Shet</conjf>; <pos>p. pr. & vb. n.</pos> <conjf>Shetting</conjf>.]</vmorph> <def>To shut.</def> <mark>[Obs. or Prov. Eng.]</mark>  <rj><au>Chaucer.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shete</ent><br/
<hw>Shete</hw> <pr>(?)</pr>, <pos>v. t. & i.</pos> <def>To shoot.</def> <mark>[Obs.]</mark>  <rj><au>Chaucer.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sheth</ent><br/
<hw>Sheth</hw> <pr>(?)</pr>, <pos>n.</pos> <def>The part of a plow which projects downward beneath the beam, for holding the share and other working parts; -- also called <altname>standard</altname>, or <altname>post</altname>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shetland pony</ent><br/
<hw>Shet"land po"ny</hw> <pr>(?)</pr>. <def>One of a small, hardy breed of horses, with long mane and tail, which originated in the Shetland Islands; a sheltie.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shew</ent><br/
<hw>Shew</hw> <pr>(?)</pr>, <pos>v. t. & i.</pos> <def>See <er>Show</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shew</ent><br/
<hw>Shew</hw>, <pos>n.</pos> <def>Show.</def> <mark>[Obs. <plain>except in</plain> shewbread.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shewbread</ent><br/
<hw>Shew"bread`</hw> <pr>(?)</pr>. <def>See <er>Showbread</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shewel</ent><br/
<hw>Shew"el</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A scarecrow.</def> <mark>[Obs.]</mark>  <rj><au>Trench.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shewer</ent><br/
<hw>Shew"er</hw> <pr>(?)</pr>, <pos>n.</pos> <def>One who shews. See <er>Shower</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shewn</ent><br/
<hw>Shewn</hw> <pr>(?)</pr>, <def><pos>p. p.</pos> of <er>Shew</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shiah</ent><br/
<hw>Shi"ah</hw> <pr>(?)</pr>, <pos>prop. n.</pos> <def>Same as <er>Shiite</er>; when used with a plural verb, the Shiites, collectively.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shibboleth</ent><br/
<hw>Shib"bo*leth</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Heb. <ets>shibb<omac/leth</ets> an ear of corn, or a stream, a flood.]</ety> <sn>1.</sn> <def>A word which was made the criterion by which to distinguish the Ephraimites from the Gileadites. The Ephraimites, not being able to pronounce <xex>sh</xex>, called the word <xex>sibboleth</xex>. See <au>Judges xii.</au></def><br/
[<source>1913 Webster</source>]</p>

<p><q>Without reprieve, adjudged to death,<br/
For want of well pronouncing <qex>shibboleth</qex>.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><def>Also used in an extended sense.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The <qex>th</qex>, with its twofold value, is . . . the <qex>shibboleth</qex> of foreigners.</q> <rj><qau>Earle.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Hence, the criterion, test, or watchword of a party; a party cry or pet phrase.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shicer</ent><br/
<hw>Shi"cer</hw> <pr>(sh<imac/"s<etil/r)</pr>, <pos>n.</pos> <ety>[Prob. fr. G. <ets>scheisser</ets> one who defecates.]</ety> <fld>(Mining)</fld> <def>An unproductive mine; a duffer.</def> <mark>[Australia]</mark><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Shide</ent><br/
<hw>Shide</hw> <pr>(sh<imac/d)</pr>, <pos>n.</pos> <ety>[OE. <ets>shide</ets>, <ets>schide</ets>, AS. <ets>sc<imac/de</ets>; akin to OHG. <ets>sc<imac/t</ets>, G. <ets>scheit</ets>, Icel. <ets>sk<imac/<edh/</ets>, and E. <ets>shed</ets>, v.t.]</ety> <def>A thin board; a billet of wood; a splinter.</def> <mark>[Prov. Eng.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shie</ent><br/
<hw>Shie</hw> <pr>(?)</pr>, <pos>v. t.</pos> <def>See <er>Shy</er>, to throw.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shied</ent><br/
<hw>Shied</hw> <pr>(?)</pr>, <def><pos>imp. & p. p.</pos> of <er>Shy</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shiel</ent><br/
<hw>Shiel</hw>, <pos>n.</pos> <def>A sheeling.</def> <mark>[Scot.]</mark>  <rj><au>Burns.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shield</ent><br/
<hw>Shield</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[OE. <ets>sheld</ets>, <ets>scheld</ets>, AS. <ets>scield</ets>, <ets>scild</ets>, <ets>sceld</ets>, <ets>scyld</ets>; akin to OS. <ets>scild</ets>, OFries. <ets>skeld</ets>, D. & G. <ets>schild</ets>, OHG. <ets>scilt</ets>, Icel. <ets>skj<oum/ldr</ets>, Sw. <ets>sk<oum/ld</ets>, Dan. <ets>skiold</ets>, Goth. <ets>skildus</ets>; of uncertain origin.  Cf. <er>Sheldrake</er>.]</ety> <sn>1.</sn> <def>A broad piece of defensive armor, carried on the arm, -- formerly in general use in war, for the protection of the body. See <er>Buckler</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Now put your <qex>shields</qex> before your hearts and fight,<br/
With hearts more proof than <qex>shields</qex>.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Anything which protects or defends; defense; shelter; protection.</def> <ldquo/My council is my <xex>shield</xex>.<rdquo/  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>Figuratively, one who protects or defends.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Fear not, Abram; I am thy <qex>shield</qex>, and thy exceeding great reward.</q> <rj><qau>Gen. xv. 1.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <fld>(Bot.)</fld> <def>In lichens, a Hardened cup or disk surrounded by a rim and containing the fructification, or asci.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <fld>(Her.)</fld> <def>The escutcheon or field on which are placed the bearings in coats of arms.  Cf. <er>Lozenge</er>. See <xex>Illust.</xex> of <er>Escutcheon</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>6.</sn> <fld>(Mining & Tunneling)</fld> <def>A framework used to protect workmen in making an adit under ground, and capable of being pushed along as excavation progresses.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>7.</sn> <def>A spot resembling, or having the form of, a shield.</def> <ldquo/Bespotted as with <xex>shields</xex> of red and black.<rdquo/  <rj><au>Spenser.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>8.</sn> <def>A coin, the old French crown, or <eacute/cu, having on one side the figure of a shield.</def> <mark>[Obs.]</mark>  <rj><au>Chaucer.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Shield fern</b></col> <fld>(Bot.)</fld>, <cd>any fern of the genus <gen>Aspidium</gen>, in which the fructifications are covered with shield-shaped indusia; -- called also <altname>wood fern</altname>. See <xex>Illust.</xex> of <er>Indusium</er>.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shield</ent><br/
<hw>Shield</hw> <pr>(?)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Shielded</conjf>; <pos>p. pr. & vb. n.</pos> <conjf>Shielding</conjf>.]</vmorph> <ety>[AS. <ets>scidan</ets>, <ets>scyldan</ets>. <ets>See</ets> <er>Shield</er>, <pos>n.</pos>]</ety> <sn>1.</sn> <def>To cover with, or as with, a shield; to cover from danger; to defend; to protect from assault or injury.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Shouts of applause ran ringing through the field,<br/
To see the son the vanquished father <qex>shield</qex>.</q> <rj><qau>Dryden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>A woman's shape doth <qex>shield</qex> thee.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To ward off; to keep off or out.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>They brought with them their usual weeds, fit to <qex>shield</qex> the cold to which they had been inured.</q> <rj><qau>Spenser.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>To avert, as a misfortune; hence, as a supplicatory exclamation, forbid!</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>God <qex>shield</qex> that it should so befall.</q> <rj><qau>Chaucer.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>God <qex>shield</qex> I should disturb devotion!</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shield-bearer</ent><br/
<hw>Shield"-bear`er</hw> <pr>(?)</pr>, <pos>n.</pos> <sn>1.</sn> <def>One who, or that which, carries a shield.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Zool.)</fld> <def>Any small moth of the genus <gen>Aspidisca</gen>, whose larva makes a shieldlike covering for itself out of bits of leaves.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shielddrake</ent><br/
<hw>Shield"drake`</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Zool.)</fld> <def>A sheldrake.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shieldless</ent><br/
<hw>Shield"less</hw>, <pos>a.</pos> <def>Destitute of a shield, or of protection.</def> -- <wordforms><wf>Shield"less*ly</wf>, <pos>adv.</pos> -- <wf>Shield"less*ness</wf>, <pos>n.</pos></wordforms><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shieldtail</ent><br/
<hw>Shield"tail`</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Zool.)</fld> <def>Any species of small burrowing snakes of the family <fam>Uropeltidae</fam>, native of Ceylon and Southern Asia. They have a small mouth which can not be dilated.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shieling</ent><br/
<hw>Shiel"ing</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A hut or shelter for shepherds of fishers. See <er>Sheeling</er>.</def> <mark>[Scot.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shift</ent><br/
<hw>Shift</hw> <pr>(sh<icr/ft)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Shifted</conjf>; <pos>p. pr. & vb. n.</pos> <conjf>Shifting</conjf>.]</vmorph> <ety>[OE. <ets>shiften</ets>, <ets>schiften</ets>, to divide, change, remove. AS. <ets>sciftan</ets> to divide; akin to LG. & D. <ets>schiften</ets> to divide, distinguish, part Icel. <ets>skipta</ets> to divide, to part, to shift, to change, Dan <ets>skifte</ets>, Sw. <ets>skifta</ets>, and probably to Icel. <ets>sk<imac/fa</ets> to cut into slices, as n., a slice, and to E. <ets>shive</ets>, <ets>sheave</ets>, n., <ets>shiver</ets>, n.]</ety> <sn>1.</sn> <def>To divide; to distribute; to apportion.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>To which God of his bounty would <qex>shift</qex><br/
Crowns two of flowers well smelling.</q> <rj><qau>Chaucer.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To change the place of; to move or remove from one place to another; <as>as, to <ex>shift</ex> a burden from one shoulder to another; to <ex>shift</ex> the blame.</as></def><br/
[<source>1913 Webster</source>]</p>

<p><q>Hastily he <qex>schifte</qex> him[self].</q> <rj><qau>Piers Plowman.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Pare saffron between the two St. Mary's days,<br/
Or set or go <qex>shift</qex> it that knowest the ways.</q> <rj><qau>Tusser.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>To change the position of; to alter the bearings of; to turn; <as>as, to <ex>shift</ex> the helm or sails</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Carrying the oar loose, [they] <qex>shift</qex> it hither and thither at pleasure.</q> <rj><qau>Sir W. Raleigh.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>To exchange for another of the same class; to remove and to put some similar thing in its place; to change; <as>as, to <ex>shift</ex> the clothes; to <ex>shift</ex> the scenes.</as></def><br/
[<source>1913 Webster</source>]</p>

<p><q>I would advise you to <qex>shift</qex> a shirt.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <def>To change the clothing of; -- used reflexively.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>As it were to ride day and night; and . . . not to have patience to <qex>shift</qex> me.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>6.</sn> <def>To put off or out of the way by some expedient.</def> <ldquo/I <xex>shifted</xex> him away.<rdquo/  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>To shift off</b></col>, <cd>to delay; to defer; to put off; to lay aside.</cd> -- <col><b>To shift the scene</b></col>, <cd>to change the locality or the surroundings, as in a play or a story.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><q><qex>Shift the scene</qex> for half an hour;<br/
Time and place are in thy power.</q> <rj><qau>Swift.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shift</ent><br/
<hw>Shift</hw>, <pos>v. i.</pos> <sn>1.</sn> <def>To divide; to distribute.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>Some this, some that, as that him liketh <qex>shift</qex>.</q> <rj><qau>Chaucer.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To make a change or changes; to change position; to move; to veer; to substitute one thing for another; -- used in the various senses of the transitive verb.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The sixth age <qex>shifts</qex><br/
Into the lean and slippered pantaloon.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Here the Baillie <qex>shifted</qex> and fidgeted about in his seat.</q> <rj><qau>Sir W. Scott.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>To resort to expedients for accomplishing a purpose; to contrive; to manage.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Men in distress will look to themselves, and leave their companions to <qex>shift</qex> as well as they can.</q> <rj><qau>L'Estrange.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>To practice indirect or evasive methods.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>All those schoolmen, though they were exceeding witty, yet better teach all their followers to <qex>shift</qex>, than to resolve by their distinctions.</q> <rj><qau>Sir W. Raleigh.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <fld>(Naut.)</fld> <def>To slip to one side of a ship, so as to destroy the equilibrum; -- said of ballast or cargo; <as>as, the cargo <ex>shifted</ex></as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shift</ent><br/
<hw>Shift</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Cf. Icel. <ets>skipti</ets>. See <er>Shift</er>, <pos>v. t.</pos>]</ety> <sn>1.</sn> <def>The act of shifting.</def> Specifically: <sd>(a)</sd> <def>The act of putting one thing in the place of another, or of changing the place of a thing; change; substitution.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>My going to Oxford was not merely for <qex>shift</qex> of air.</q> <rj><qau>Sir H. Wotton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sd>(b)</sd> A turning from one thing to another; hence, an expedient tried in difficulty; often, an evasion; a trick; a fraud. <ldquo/Reduced to pitiable <xex>shifts</xex>.<rdquo/  <rj><au>Macaulay.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>I 'll find a thousand <qex>shifts</qex> to get away.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Little souls on little <qex>shifts</qex> rely.</q> <rj><qau>Dryden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Something frequently shifted; especially, a woman's under-garment; a chemise.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>The change of one set of workmen for another; hence, a spell, or turn, of work; also, a set of workmen who work in turn with other sets; <as>as, a night <ex>shift</ex></as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>In building, the extent, or arrangement, of the overlapping of plank, brick, stones, etc., that are placed in courses so as to break joints.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <fld>(Mining)</fld> <def>A breaking off and dislocation of a seam; a fault.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>6.</sn> <fld>(Mus.)</fld> <def>A change of the position of the hand on the finger board, in playing the violin.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>To make shift</b></col>, <cd>to contrive or manage in an exigency.</cd>  <ldquo/I shall <xex>make shift</xex> to go without him.<rdquo/  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>[They] <qex>made a shift</qex> to keep their own in Ireland.</q> <rj><qau>Milton.</qau></rj>
</cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shiftable</ent><br/
<hw>Shift"a*ble</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Admitting of being shifted.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shifter</ent><br/
<hw>Shift"er</hw> <pr>(?)</pr>, <pos>n.</pos> <sn>1.</sn> <def>One who, or that which, shifts; one who plays tricks or practices artifice; a cozener.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>'T was such a <qex>shifter</qex> that, if truth were known,<br/
Death was half glad when he had got him down.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Naut.)</fld> <def>An assistant to the ship's cook in washing, steeping, and shifting the salt provisions.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <fld>(Mach.)</fld> <sd>(a)</sd> <def>An arrangement for shifting a belt sidewise from one pulley to another.</def> <sd>(b)</sd> <fld>(Knitting Mach.)</fld> <def>A wire for changing a loop from one needle to another, as in narrowing, etc.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <fld>(construction, tunneling)</fld> <def>A foreman responsible for the work on one shift in one area, as in one heading{4}.</def><br/
[<source>RDH</source>]</p>

<p><ent>Shiftiness</ent><br/
<hw>Shift"i*ness</hw>, <pos>n.</pos> <def>The quality or state of being shifty.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Diplomatic <qex>shiftiness</qex> and political versatility.</q> <rj><qau>J. A. Syminds.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shifting</ent><br/
<hw>Shift"ing</hw>, <pos>a.</pos> <sn>1.</sn> <def>Changing in place, position, or direction; varying; variable; fickle; <as>as, <ex>shifting</ex> winds; <ex>shifting</ex> opinions or principles</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Adapted or used for shifting anything.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Shifting backstays</b></col> <fld>(Naut.)</fld>, <cd>temporary stays that have to be let go whenever the vessel tacks or jibes.</cd> -- <col><b>Shifting ballast</b></col>, <cd>ballast which may be moved from one side of a vessel to another as safety requires.</cd> -- <col><b>Shifting center</b></col>. <cd>See <er>Metacenter</er>.</cd> -- <col><b>Shifting locomotive</b></col>. <cd>See <cref>Switching engine</cref>, under <er>Switch</er>.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shiftingly</ent><br/
<hw>Shift"ing*ly</hw>, <pos>adv.</pos> <def>In a shifting manner.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shiftless</ent><br/
<hw>Shift"less</hw>, <pos>a.</pos> <def>Destitute of expedients, or not using successful expedients; characterized by failure, especially by failure to provide for one's own support, through negligence or incapacity; hence, lazy; improvident; thriftless; <as>as, a <ex>shiftless</ex> fellow; <ex>shiftless</ex> management.</as></def> -- <wordforms><wf>Shift"less*ly</wf>, <pos>adv.</pos> -- <wf>Shift"less*ness</wf>, <pos>n.</pos></wordforms><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shifty</ent><br/
<hw>Shift"y</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Full of, or ready with, shifts; fertile in expedients or contrivance.</def>  <rj><au>Wright.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q><qex>Shifty</qex> and thrifty as old Greek or modern Scot, there were few things he could not invent, and perhaps nothing he could not endure.</q> <rj><qau>C. Kingsley.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shiah</ent><br/
<ent>Shiite</ent><br/
<mhw>{ <hw>Shi"ite</hw> <pr>(?)</pr>, <hw>Shi"ah</hw> <pr>(?)</pr>, }</mhw> <pos>n.</pos> <ety>[Ar. <ets>sh<imac/'a<imac/</ets>a follower of the sect of Ali, fr. <ets>sh<imac/'at</ets>, <ets>sh<imac/'ah</ets>, a multitude following one another in pursuit of the same object, the sect of Ali, fr. <ets>sh<amac/'a</ets> to follow.]</ety> <def>A member of that branch of the Mohammedans to which the Persians belong. They reject the first three caliphs, and consider Ali as being the first and only rightful successor of Mohammed. They do not acknowledge the Sunna, or body of traditions respecting Mohammed, as any part of the law, and on these accounts are treated as heretics by the Sunnites, or orthodox Mohammedans.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shikari</ent><br/
<ent>Shikaree</ent><br/
<mhw>{ <hw>Shi*ka"ree</hw>, ||<hw>Shi*ka"ri</hw>  }</mhw> <pr>(?)</pr> <pos>n.</pos> <ety>[<ets>Hind</ets>.]</ety> <def>A sportsman; esp., a native hunter.</def> <mark>[India]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shilf</ent><br/
<hw>Shilf</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[CF. G. <ets>shilf</ets> sedge.]</ety> <def>Straw.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shill</ent><br/
<hw>Shill</hw> <pr>(?)</pr>, <pos>v. t.</pos> <def>To shell.</def> <mark>[Obs. or Prov. Eng.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shill</ent><br/
<hw>Shill</hw>, <pos>v. t.</pos> <ety>[Cf. <er>Sheal</er>.]</ety> <def>To put under cover; to sheal.</def> <mark>[Prov.ng.]</mark>  <rj><au>Brockett.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shillelah</ent><br/
<ent>Shillalah</ent><br/
<mhw>{ <hw>Shil*la"lah</hw>, <hw>Shil*le"lah</hw>  }</mhw> <pr>(?)</pr>, <pos>n.</pos> <def>An oaken sapling or cudgel; any cudgel; -- so called from <xex>Shillelagh</xex>, a place in Ireland of that name famous for its oaks.</def> <mark>[Irish]</mark> <altsp>[Written also <asp>shillaly</asp>, and <asp>shillely</asp>.]</altsp><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shilling</ent><br/
<hw>Shil"ling</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[OE. <ets>shilling</ets>, <ets>schilling</ets>, AS. <ets>scilling</ets>; akin to D. <ets>schelling</ets>, OS. & OHG. <ets>scilling</ets>, G. <ets>schilling</ets>, Sw. & Dan. <ets>skilling</ets>, Icel. <ets>skillingr</ets>, Goth. <ets>skilliggs</ets>, and perh. to OHG. <ets>scellan</ets> to sound, G. <ets>schallen</ets>.]</ety> <sn>1.</sn> <def>A silver coin, and money of account, of Great Britain and its dependencies, equal to twelve pence, or the twentieth part of a pound, equivalent to about twenty-four cents of the United States currency.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>In the United States, a denomination of money, differing in value in different States. It is not now legally recognized.</def><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ Many of the States while colonies had issued bills of credit which had depreciated in different degrees in the different colonies.  Thus, in New England currency (used also in Virginia, Kentucky, Tennessee, Indiana, Illinois, Missouri, Mississippi, Alabama, and Florida), after the adoption of the decimal system, the pound in paper money was worth only $3.333, and the shilling 16<frac23/ cts., or 6s. to $1; in New York currency (also in North Carolina, Ohio, and Michigan), the pound was worth $2.50, and the shilling 12<frac12/ cts., or 8s. to $1; in Pennsylvania currency (also in New Jersey, Delaware, and Maryland), the pound was worth $2.70, and the shilling 13<frac12/ cts., or 7s. 6d. to $1; and in Georgia currency (also in South Carolina), the pound was worth $4.20<frac67/, and the shilling 21<frac37/ cts., or 4s  8d. to $1.  In many parts of the country . . . the reckoning by shillings and pence is not yet entirely abandoned.</note>  <rj><au>Am. Cyc. (1890)</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><-- p. 1329 --></p>

<p><sn>3.</sn> <def>The Spanish real, of the value of one eight of a dollar, or 12<?/ cets; -- formerly so called in New York and some other States. See Note under 2.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>York shilling</b></col>. <cd>Same as <er>Shilling</er>,  3.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shilly-shally</ent><br/
<ent>Shill-I-shall-I</ent><br/
<mhw>{ <hw>Shill"-I-shall`-I</hw> <pr>(?)</pr>, <hw>Shil"ly-shal`ly</hw>, }</mhw> <pos>adv.</pos> <ety>[A reduplication of <ets>shall I</ets>.]</ety> <def>In an irresolute, undecided, or hesitating manner.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>I am somewhat dainty in making a resolution, because when I make it, I keep it; I don't stand <qex>shill-I-shall-I</qex> then; if I say 't, I'll do 't.</q> <rj><qau>Congreve.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shilly-shally</ent><br/
<hw>Shil"ly-shal`ly</hw>, <pos>v. i.</pos> <def>To hesitate; to act in an irresolute manner; hence, to occupy one's self with trifles.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shilly-shally</ent><br/
<hw>Shil"ly-shal`ly</hw>, <pos>n.</pos> <def>Irresolution; hesitation; also, occupation with trifles.</def><br/
[<source>1913 Webster</source>]</p>

<p><q><qex>She lost not one of her forty-five minutes in picking and choosing</qex>, -- <qex>no shilly-shally</qex> in Kate.</q> <rj><qau>De Quincey.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shiloh</ent><br/
<hw>Shi"loh</hw> <pr>(sh<imac/"l<omac/)</pr>, <pos>n.</pos> <ety>[Heb. <ets>sh<imac/l<omac/h</ets>, literally, quiet, rest, fr. <ets>sh<amac/l<amac/h</ets> to rest.]</ety> <fld>(Script.)</fld> <def>A word used by Jacob on his deathbed, and interpreted variously, as <ldquo/the Messiah,<rdquo/ or as the city <ldquo/Shiloh,<rdquo/ or as <ldquo/Rest.<rdquo/</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shily</ent><br/
<hw>Shi"ly</hw> <pr>(?)</pr>, <pos>adv.</pos> <def>See <er>Shyly</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shim</ent><br/
<hw>Shim</hw> <pr>(?)</pr>, <pos>n.</pos> <sn>1.</sn> <def>A kind of shallow plow used in tillage to break the ground, and clear it of weeds.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Mach.)</fld> <def>A thin piece of metal placed between two parts to make a fit.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shimmer</ent><br/
<hw>Shim"mer</hw> <pr>(?)</pr>, <pos>v. i.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Shimmered</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Shimmering</conjf>.]</vmorph> <ety>[OE. <ets>schimeren</ets>, AS. <ets>scimerian</ets>; akin to <ets>sc<imac/mian</ets>, <ets>sc<imac/man</ets>, to glitter, D. <ets>schemeren</ets>, G. <ets>schimmern</ets>, Dan. <ets>skimre</ets>, Sw. <ets>skimra</ets>, AS. <ets>sc<imac/ma</ets> a light, brightness, Icel. <ets>sk<imac/ma</ets>, Goth. <ets>skeima</ets> a torch, a lantern, and E. <ets>shine</ets>. <root/157. See <er>Shine</er>, <pos>v. i.</pos>]</ety> <def>To shine with a tremulous or intermittent light; to shine faintly; to gleam; to glisten; to glimmer.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The <qex>shimmering</qex> glimpses of a stream.</q> <rj><qau>Tennyson.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shimmer</ent><br/
<hw>Shim"mer</hw>, <pos>n.</pos> <def>A faint, tremulous light; a gleaming; a glimmer.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>TWo silver lamps, fed with perfumed oil, diffused . . . a trembling twilight-seeming <qex>shimmer</qex> through the quiet apartment.</q> <rj><qau>Sir W. Scott.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shimmering</ent><br/
<hw>Shim"mer*ing</hw>, <pos>n.</pos> <def>A gleam or glimmering.</def> <ldquo/A little <xex>shimmering</xex> of a light.<rdquo/  <rj><au>Chaucer.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shimmy</ent><br/
<hw>Shim"my</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A chemise.</def> <mark>[Colloq.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shin</ent><br/
<hw>Shin</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[OE. <ets>shine</ets>, <ets>schine</ets>, AS. scina; akin to D. <ets>scheen</ets>, OHG. <ets>scina</ets>, G. <ets>schiene</ets>, <ets>schien</ets>bein, Dan. <ets>skinne</ets>been, Sw. <ets>sken</ets>ben.  Cf. <er>Chine</er>.]</ety> <sn>1.</sn> <def>The front part of the leg below the knee; the front edge of the shin bone; the lower part of the leg; the shank.</def> <ldquo/On his <xex>shin</xex>.<rdquo/  <rj><au>Chaucer.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Railbroad)</fld> <def>A fish plate for rails.</def>  <rj><au>Knight.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Shin bone</b></col> <fld>(Anat.)</fld>, <cd>the tibia.</cd> -- <col><b>Shin leaf</b></col> <fld>(Bot.)</fld>, <cd>a perennial ericaceous herb (<spn>Pyrola elliptica</spn>) with a cluster of radical leaves and a raceme of greenish white flowers.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shin</ent><br/
<hw>Shin</hw>, <pos>v. i.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Shinned</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Shinning</conjf>.]</vmorph> <sn>1.</sn> <def>To climb a mast, tree, rope, or the like, by embracing it alternately with the arms and legs, without help of steps, spurs, or the like; -- used with <xex>up</xex>; <as>as, to <ex>shin</ex> up a mast</as>.</def> <mark>[Slang]</mark>
<-- now usu. shinny --><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To run about borrowing money hastily and temporarily, as for the payment of one's notes at the bank.</def> <mark>[Slang, U.S.]</mark>  <rj><au>Bartlett.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shin</ent><br/
<hw>Shin</hw>, <pos>v. t.</pos> <def>To climb (a pole, etc.) by shinning up.</def> <mark>[Slang]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shindle</ent><br/
<hw>Shin"dle</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[See 2d <er>Shingle</er>.]</ety> <def>A shingle; also, a slate for roofing.</def> <mark>[Obs.]</mark>  <rj><au>Holland.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shindle</ent><br/
<hw>Shin"dle</hw>, <pos>v. t.</pos> <def>To cover or roof with shindles.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shindy</ent><br/
<hw>Shin"dy</hw> <pr>(?)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Shindies</plw> <pr>(#)</pr>.</plu> <ety>[Etymol. uncertain; cf. <er>Shinney</er>, <er>Shinty</er>.]</ety> <sn>1.</sn> <def>An uproar or disturbance; a spree; a row; a riot.</def> <mark>[Slang]</mark>  <rj><au>Thackeray.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Hockey; shinney.</def>  <rj><au>Bartlett.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>A fancy or liking.</def> <mark>[Local, U. S.]</mark>  <rj><au>Bartlett.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shine</ent><br/
<hw>Shine</hw> <pr>(?)</pr>, <pos>v. i.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Shone</conjf> <pr>(<?/ <or/ <?/; 277)</pr> (<mark>archaic</mark> <conjf>Shined</conjf> <pr>(<?/)</pr>); <pos>p. pr. & vb. n.</pos> <conjf>Shining</conjf>.]</vmorph> <ety>[OE. <ets>shinen</ets>, <ets>schinen</ets>, AS. <ets>sc<imac/nan</ets>; akin to D. <ets>schijnen</ets>, OFries. <ets>sk<imac/na</ets>, OS. & OHG. <ets>sc<imac/nan</ets>, G. <ets>scheinen</ets>, Icel. <ets>sk<imac/na</ets>, Sw. <ets>skina</ets>, Dan. <ets>skinne</ets>, Goth. <ets>skeinan</ets>, and perh. to Gr. <?/<?/<?/ shadow. <root/157.  Cf. <er>Sheer</er> pure, and <er>Shimmer</er>.]</ety> <sn>1.</sn> <def>To emit rays of light; to give light; to beam with steady radiance; to exhibit brightness or splendor; <as>as, the sun <ex>shines</ex> by day; the moon <ex>shines</ex> by night.</as></def><br/
[<source>1913 Webster</source>]</p>

<p><q>Hyperion's quickening fire doth <qex>shine</qex>.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>God, who commanded the light to <qex>shine</qex> out of darkness, hath <qex>shined</qex> in our hearts, to give the light of the knowledge of the glory of God in the face of Jesus Cghrist.</q> <rj><qau>2 Cor. iv. 6.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Let thine eyes <qex>shine</qex> forth in their full luster.</q> <rj><qau>Denham.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To be bright by reflection of light; to gleam; to be glossy; <as>as, to <ex>shine</ex> like polished silver</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>To be effulgent in splendor or beauty.</def> <ldquo/So proud she <xex>shined</xex> in her princely state.<rdquo/  <rj><au>Spenser.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Once brightest <qex>shined</qex> this child of heat and air.</q> <rj><qau>Pope.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>To be eminent, conspicuous, or distinguished; to exhibit brilliant intellectual powers; <as>as, to <ex>shine</ex> in courts; to <ex>shine</ex> in conversation</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Few are qualified to <qex>shine</qex> in company; but it in most men's power to be agreeable.</q> <rj><qau>Swift.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><mcol><col><b>To make the face to shine upon</b></col>, <it>or</it> <col><b>To cause the face to shine upon</b></col></mcol>, <cd>to be propitious to; to be gracious to.</cd>  <rj><au>Num. vi. 25.</au></rj></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shine</ent><br/
<hw>Shine</hw>, <pos>v. t.</pos> <sn>1.</sn> <def>To cause to shine, as a light.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>He [God] doth not rain wealth, nor <qex>shine</qex> honor and virtues, upon men equally.</q> <rj><qau>Bacon.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To make bright; to cause to shine by reflected light; <as>as, in hunting, to <ex>shine</ex> the eyes of a deer at night by throwing a light on them</as>.</def> <mark>[U. S.]</mark>  <rj><au>Bartlett.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shine</ent><br/
<hw>Shine</hw>, <pos>n.</pos> <sn>1.</sn> <def>The quality or state of shining; brightness; luster, gloss; polish; sheen.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Now sits not girt with taper's holy <qex>shine</qex>.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Fair opening to some court's propitious <qex>shine</qex>.</q> <rj><qau>Pope.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>The distant <qex>shine</qex> of the celestial city.</q> <rj><qau>Hawthorne.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Sunshine; fair weather.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Be it fair or foul, or rain or <qex>shine</qex>.</q> <rj><qau>Dryden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>A liking for a person; a fancy.</def> <mark>[Slang, U.S.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>Caper; antic; row.</def> <mark>[Slang]</mark><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>To cut up shines</b></col>, <cd>to play pranks.</cd> <mark>[Slang, U.S.]</mark></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shine</ent><br/
<hw>Shine</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[AS. <ets>sc<imac/n</ets>. See <er>Shine</er>, <pos>v. i.</pos>]</ety> <def>Shining; sheen.</def> <mark>[Obs.]</mark>  <rj><au>Spenser.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shiner</ent><br/
<hw>Shin"er</hw> <pr>(?)</pr>, <pos>n.</pos> <def>That which shines.</def> Specifically: <sd>(a)</sd> <def>A luminary.</def> <sd>(b)</sd> <def>A bright piece of money.</def> <mark>[Slang]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>Has she the <qex>shiners</qex>, d' ye think?</q> <rj><qau>Foote.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><-- [Colloq.] A bruised eye; a <cref>black eye</cref>. --><br/
[<source>1913 Webster</source>]</p>

<p><sd>(c)</sd> <fld>(Zool.)</fld> <def>Any one of numerous species of small freshwater American cyprinoid fishes, belonging to <gen>Notropis</gen>, or <gen>Minnilus</gen>, and allied genera; as the redfin (<spn>Notropis megalops</spn>), and the golden <ex>shiner</ex> (<spn>Notemigonus chrysoleucus</spn>) of the Eastern United States; also loosely applied to various other silvery fishes, as the dollar fish, or horsefish, menhaden, moonfish, sailor's choice, and the sparada.</def> <sd>(d)</sd> <fld>(Zool.)</fld> <def>The common Lepisma, or furniture bug.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Blunt-nosed shiner</b></col> <fld>(Zool.)</fld>, <cd>the silver moonfish.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shiness</ent><br/
<hw>Shi"ness</hw> <pr>(?)</pr>, <pos>n.</pos> <def>See <er>Shyness</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shingle</ent><br/
<hw>Shin"gle</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Prob. from Norw. <ets>singl</ets>, <ets>singling</ets>, coarse gravel, small round stones.]</ety> <fld>(Geol.)</fld> <def>Round, water-worn, and loose gravel and pebbles, or a collection of roundish stones, such as are common on the seashore and elsewhere.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shingle</ent><br/
<hw>Shin"gle</hw>, <pos>n.</pos> <ety>[OE. <ets>shingle</ets>, <ets>shindle</ets>, fr. L. <ets>scindula</ets>, <ets>scandula</ets>; cf. <ets>scindere</ets> to cleave, to split, E. <ets>shed</ets>, v. t., Gr. <?/<?/<?/, <?/<?/<?/, shingle, <?/<?/<?/ to slit.]</ety> <sn>1.</sn> <def>A piece of wood sawed or rived thin and small, with one end thinner than the other, -- used in covering buildings, especially roofs, the thick ends of one row overlapping the thin ends of the row below.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>I reached St. Asaph, . . . where there is a very poor cathedral church covered with <qex>shingles</qex> or tiles.</q> <rj><qau>Ray.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A sign for an office or a shop; <as>as, to hang out one's <ex>shingle</ex></as>.</def> <mark>[Jocose, U. S.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Shingle oak</b></col> <fld>(Bot.)</fld>, <cd>a kind of oak (<spn>Quercus imbricaria</spn>) used in the Western States for making shingles.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shingle</ent><br/
<hw>Shin"gle</hw>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Shingled</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Shingling</conjf> <pr>(?)</pr>.]</vmorph> <sn>1.</sn> <def>To cover with shingles; <as>as, to <ex>shingle</ex> a roof</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>They <qex>shingle</qex> their houses with it.</q> <rj><qau>Evelyn.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To cut, as hair, so that the ends are evenly exposed all over the head, as shingles on a roof.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shingle</ent><br/
<hw>Shin"gle</hw>, <pos>v. t.</pos> <def>To subject to the process of shindling, as a mass of iron from the pudding furnace.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shingler</ent><br/
<hw>Shin"gler</hw> <pr>(?)</pr>, <pos>n.</pos> <sn>1.</sn> <def>One who shingles.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A machine for shingling puddled iron.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shingles</ent><br/
<hw>Shin"gles</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[OF. <ets>cengle</ets> a girth, F. <ets>sangle</ets>, fr. L. <ets>cingulum</ets> a girdle, fr. <ets>cingere</ets> to gird.  Cf. <er>Cincture</er>, <er>Cingle</er>, <er>Surcingle</er>.]</ety> <fld>(Med.)</fld> <def>A kind of herpes (<spn>Herpes zoster</spn>) which spreads half way around the body like a girdle, and is usually attended with violent neuralgic pain.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shingling</ent><br/
<hw>Shin"gling</hw> <pr>(?)</pr>, <pos>n.</pos> <sn>1.</sn> <def>The act of covering with shingles; shingles, collectively; a covering made of shingles.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Metal)</fld> <def>The process of expelling scoriae and other impurities by hammering and squeezing, in the production of wrought iron.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Shingling hammer</b></col>, <cd>a ponderous hammer moved by machinery, used in shingling puddled iron.</cd> -- <col><b>Shingling mill</b></col>, <cd>a mill or forge where puddled iron is shingled.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shingly</ent><br/
<hw>Shin"gly</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Abounding with shingle, or gravel.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shinhopple</ent><br/
<hw>Shin"hop`ple</hw> <pr>(?)</pr>, <pos>n.</pos> <def>The hobblebush.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shining</ent><br/
<hw>Shin"ing</hw> <pr>(?)</pr>, <pos>a.</pos> <sn>1.</sn> <def>Emitting light, esp. in a continuous manner; radiant; <as>as, <ex>shining</ex> lamps</as>; also, bright by the reflection of light; <as>as, <ex>shining</ex> armor</as>.</def> <ldquo/Fish . . . with their fins and <xex>shining</xex> scales.<rdquo/  <rj><au>Milton.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Splendid; illustrious; brilliant; distinguished; conspicious; <as>as, a <ex>shining</ex> example of charity</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>Having the surface smooth and polished; -- said of leaves, the surfaces of shells, etc.</def><br/
[<source>1913 Webster</source>]</p>

<p><syn><b>Syn.</b> -- Glistening; bright; radiant; resplendent; effulgent; lustrous; brilliant; glittering; splendid; illustrious.</syn>  -- <usage><er>Shining</er>, <er>Brilliant</er>, <er>Sparking</er>. <xex>Shining</xex> describes the steady emission of a strong light, or the steady reflection of light from a clear or polished surface. <xex>Brilliant</xex> denotes a shining of great brightness, but with gleams or flashes. <xex>Sparkling</xex> implies a fitful, intense shining from radiant points or sparks, by which the eye is dazzled. The same distinctions obtain when these epithets are figuratively applied. A man of <xex>shining</xex> talents is made conspicious by possessing them; if they flash upon the mind with a peculiarly striking effect, we call them <xex>brilliant</xex>; if his brilliancy is marked by great vivacity and occasional intensity, he is <xex>sparkling</xex>.</usage><br/
[<source>1913 Webster</source>]</p>

<p><q>True paradise . . . inclosed with <qex>shining</qex> rock.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Some in a <qex>brilliant</qex> buckle bind her waist,<br/
Some round her neck a circling light display.</q> <rj><qau>Gay.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>His <qex>sparkling</qex> blade about his head he blest.</q> <rj><qau>Spenser.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shining</ent><br/
<hw>Shin"ing</hw>, <pos>n.</pos> <def>Emission or reflection of light.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shiningness</ent><br/
<hw>Shin"ing*ness</hw>, <pos>n.</pos> <def>Brightness.</def>  <rj><au>J. Spence.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shinney</ent><br/
<hw>Shin"ney</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[CF. <er>Shindy</er>.]</ety> <def>The game of hockey; -- so called because of the liability of the players to receive blows on the shin.</def>  <rj><au>Halliwell.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shin Shu</ent><br/
||<hw>Shin Shu</hw> <pr>(?)</pr>. <ety>[Jap., lit., true sect.]</ety> <def>The leading and most progressive Buddhist sect of Japan, resting its faith rather upon Amida than Gautama Buddha. Rites and ceremonies are held useless without uprightness.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Shintyan</ent><br/
<ent>Shintiyan</ent><br/
<mhw>{ <hw>Shin"ti*yan</hw> <pr>(sh<icr/n"t<icr/*y<acr/n)</pr>, <hw>Shin"ty*an</hw> <pr>(sh<icr/n"t<icr/*<acr/n)</pr>, }</mhw> <pos>n.</pos>} <ety>[Ar. <ets>shint<imac/<amac/n</ets>.]</ety> <def>A kind of wide loose drawers or trousers worn by women in Mohammedan countries.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><-- shinny. Same as shin, to climb with hands and feet --><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shinplaster</ent><br/
<hw>Shin"plas`ter</hw> <pr>(?)</pr>, <pos>n.</pos> <def>Formerly, a jocose term for a bank note greatly depreciated in value; also, for paper money of a denomination less than a dollar.</def> <mark>[U. S.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shintiism</ent><br/
<ent>Shinto</ent><br/
<mhw>{ <hw>Shin"to</hw> <pr>(?)</pr>, <hw>Shin"ti*ism</hw> <pr>(?)</pr>, }</mhw> <pos>n.</pos> <ety>[Chin. <ets>shin</ets> god + <ets>tao</ets> way, doctrine.]</ety> <def>One of the two great systems of religious belief in Japan. Its essence is ancestor worship, and sacrifice to dead heroes.</def> <altsp>[Written also <asp>Sintu</asp>, and <asp>Sintuism</asp>.]</altsp><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shintoist</ent><br/
<hw>Shin"to*ist</hw> <pr>(?)</pr>, <pos>n.</pos> <def>An adherent of Shintoism.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shinty</ent><br/
<hw>Shin"ty</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Cf. Gael. <ets>sinteag</ets> a skip, a bound.]</ety> <def>A Scotch game resembling hockey; also, the club used in the game.</def>  <rj><au>Jamieson.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shiny</ent><br/
<hw>Shin"y</hw> <pr>(?)</pr>, <pos>a.</pos> <amorph>[<pos>Compar.</pos> <adjf>Shinier</adjf> <pr>(?)</pr>; <pos>superl.</pos> <adjf>Shiniest</adjf>.]</amorph> <def>Bright; luminous; clear; unclouded.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Like distant thunder on a <qex>shiny</qex> day.</q> <rj><qau>Dryden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>-ship</ent><br/
<hw>-ship</hw> <pr>(?)</pr>. <ety>[OE. <ets>-schipe</ets>, AS. <ets>-scipe</ets>; akin to OFries. <ets>-skipe</ets>, OLG. <ets>-skepi</ets>, D. <ets>-schap</ets>, OHG. <ets>-scaf</ets>, G. <ets>-schaft</ets>.  Cf. <er>Shape</er>, <pos>n.</pos>, and <er>Landscape</er>.]</ety> <def>A suffix denoting <xex>state</xex>, <xex>office</xex>, <xex>dignity</xex>, <xex>profession</xex>, or <xex>art</xex>; as in lord<xex>ship</xex>, friend<xex>ship</xex>, chancellor<xex>ship</xex>, steward<xex>ship</xex>, horseman<xex>ship</xex>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Ship</ent><br/
<hw>Ship</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[AS. <ets>scipe</ets>.]</ety> <def>Pay; reward.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>In withholding or abridging of the <qex>ship</qex> or the hire or the wages of servants.</q> <rj><qau>Chaucer.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Ship</ent><br/
<hw>Ship</hw>, <pos>n.</pos> <ety>[OE. <ets>ship</ets>, <ets>schip</ets>, AS. <ets>scip</ets>; akin to OFries. <ets>skip</ets>, OS. <ets>scip</ets>, D. <ets>schip</ets>, G. <ets>schiff</ets>, OHG. <ets>scif</ets>, Dan. <ets>skib</ets>, Sw. <ets>skeep</ets>, Icel. & Goth. <ets>skip</ets>; of unknown origin.  Cf. <er>Equip</er>, <er>Skiff</er>, <er>Skipper</er>.]</ety> <sn>1.</sn> <def>Any large seagoing vessel.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Like a stately <qex>ship</qex> . . . <br/
With all her bravery on, and tackle trim,<br/
Sails filled, and streamers waving.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Thou, too, sail on, O <qex>Ship</qex> of State!</q> <rj><qau>Longfellow.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Specifically, a vessel furnished with a bowsprit and three masts (a mainmast, a foremast, and a mizzenmast), each of which is composed of a lower mast, a topmast, and a topgallant mast, and square-rigged on all masts. See <xex>Illustation</xex> in Appendix.</def><br/
[<source>1913 Webster</source>]</p>

<p><-- ##?? illustration: Deck plan of a ship --><br/
[<source>1913 Webster</source>]</p>

<p><caption><xex>l</xex> Port or Larboard Side; <qex>s</qex> Starboard Side; 1 Roundhouse or Deck House; 2 Tiller; 3 Grating; 4 Wheel; 5 Wheel Chains; 6 Binnacle; 7 Mizzenmast; 8 Skylight; 9 Capstan; 10 Mainmast; 11 Pumps; 12 Galley or Caboose; 13 Main Hatchway; 14 Windlass; 15 Foremast; 16 Fore Hatchway; 17 Bitts; 18 Bowsprit; 19 Head Rail; 20 Boomkins; 21 Catheads on Port Bow and Starboard Bow; 22 Fore Chains; 23 Main Chains; 24 Mizzen Chains; 25 Stern.</caption><br/
[<source>1913 Webster</source>]</p>

<p><--illustration: Outline of a ship --><br/
[<source>1913 Webster</source>]</p>

<p><caption>1 Fore Royal Stay; 2 Flying Jib Stay; 3 Fore Topgallant Stay;4 Jib Stay; 5 Fore Topmast Stays; 6 Fore Tacks; 8 Flying Martingale; 9 Martingale Stay, shackled to Dolphin Striker; 10 Jib Guys; 11 Jumper Guys; 12 Back Ropes; 13 Robstays; 14 Flying Jib Boom; 15 Flying Jib Footropes; 16 Jib Boom; 17 Jib Foottropes; 18 Bowsprit; 19 Fore Truck; 20 Fore Royal Mast; 21 Fore Royal Lift; 22 Fore Royal Yard; 23 Fore Royal Backstays; 24 Fore Royal Braces; 25 Fore Topgallant Mast and Rigging; 26 Fore Topgallant Lift; 27 Fore Topgallant Yard; 28 Fore Topgallant Backstays; 29 Fore Topgallant Braces; 30 Fore Topmast and Rigging; 31 Fore Topsail Lift; 32 Fore Topsail Yard; 33 Fore Topsail Footropes; 34 Fore Topsail Braces; 35 Fore Yard; 36 Fore Brace; 37 Fore Lift; 38 Fore Gaff; 39 Fore Trysail Vangs; 40 Fore Topmast Studding-sail Boom; 41 Foremast and Rigging; 42 Fore Topmast Backstays; 43 Fore Sheets; 44 Main Truck and Pennant; 45 Main Royal Mast and Backstay; 46 Main Royal Stay; 47 Main Royal Lift; 48 Main Royal Yard; 49 Main Royal Braces; 50 Main Topgallant Mast and Rigging; 51 Main Topgallant Lift; 52 Main Topgallant Backstays; 53 Main Topgallant Yard; 54 Main Topgallant Stay; 55 Main Topgallant Braces; 56 Main Topmast and Rigging; 57 Topsail Lift; 58 Topsail Yard; 59 Topsail Footropes; 60 Topsail Braces; 61 Topmast Stays; 62 Main Topgallant Studding-sail Boom; 63 Main Topmast Backstay; 64 Main Yard; 65 Main Footropes; 66 Mainmast and Rigging; 67 Main Lift; 68 Main Braces; 69 Main Tacks; 70 Main Sheets; 71 Main Trysail Gaff; 72 Main Trysail Vangs; 73 Main Stays; 74 Mizzen Truck; 75 Mizzen Royal Mast and Rigging; 76 Mizzen Royal Stay; 77 Mizzen Royal Lift; 78 Mizzen Royal Yard; 79 Mizzen Royal Braces; 80 Mizzen Topgallant Mast and Rigging; 81 Mizzen Topgallant Lift; 82 Mizzen Topgallant Backstays; 83 Mizzen Topgallant Braces; 84 Mizzen Topgallant Yard; 85 Mizzen Topgallant Stay; 86 Mizzen Topmast and Rigging; 87 Mizzen Topmast Stay; 88 Mizzen Topsail Lift; 89 Mizzen Topmast Backstays; 90 Mizzen Topsail Braces; 91 Mizzen Topsail Yard; 92 Mizzen Topsail Footropes; 93 Crossjack Yard; 94 Crossjack Footropes; 95 Crossjack Lift; 96 Crossjack Braces; 97 Mizzenmast and Rigging; 98 Mizzen Stay; 99 Spanker Gaff; 100 Peak Halyards; 101 Spanker Vangs; 102 Spanker Boom; 103 Spanker Boom Topping Lift; 104 Jacob's Ladder, or Stern Ladder; 105 Spanker Sheet; 106 Cutwater; 107 Starboard Bow; 108 Starboard Beam; 109 Water Line; 110 Starboard Quarter; 111 Rudder.</caption><br/
[<source>1913 Webster</source>]</p>

<p><-- p. 1330 --></p>

<p><sn>3.</sn> <def>A dish or utensil (originally fashioned like the hull of a ship) used to hold incense.</def> <mark>[Obs.]</mark>  <rj><au>Tyndale.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Armed ship</b></col>, <cd>a private ship taken into the service of the government in time of war, and armed and equipped like a ship of war.</cd> <mark>[Eng.]</mark> <au>Brande & C.</au> -- <col><b>General ship</b></col>. <cd>See under <er>General</er>.</cd> -- <col><b>Ship biscuit</b></col>, <cd>hard biscuit prepared for use on shipboard; -- called also <altname>ship bread</altname>. See <er>Hardtack</er>.</cd> -- <col><b>Ship boy</b></col>, <cd>a boy who serves in a ship.</cd>  <ldquo/Seal up the <xex>ship boy's</xex> eyes.<rdquo/ <au>Shak.</au> -- <col><b>Ship breaker</b></col>, <cd>one who breaks up vessels when unfit for further use.</cd> -- <col><b>Ship broker</b></col>, <cd>a mercantile agent employed in buying and selling ships, procuring cargoes, etc., and generally in transacting the business of a ship or ships when in port.</cd> -- <col><b>Ship canal</b></col>, <cd>a canal suitable for the passage of seagoing vessels.</cd> -- <col><b>Ship carpenter</b></col>, <cd>a carpenter who works at shipbuilding; a shipwright.</cd> -- <col><b>Ship chandler</b></col>, <cd>one who deals in cordage, canvas, and other, furniture of vessels.</cd> -- <col><b>Ship chandlery</b></col>, <cd>the commodities in which a ship chandler deals; also, the business of a ship chandler.</cd> -- <col><b>Ship fever</b></col> <fld>(Med.)</fld>, <cd>a form of typhus fever; -- called also <altname>putrid fever</altname>, <altname>jail fever</altname>, <it>or</it> <altname>hospital fever</altname>.</cd> -- <col><b>Ship joiner</b></col>, <cd>a joiner who works upon ships.</cd> -- <col><b>Ship letter</b></col>, <cd>a letter conveyed by a ship not a mail packet.</cd> -- <col><b>Ship money</b></col> <fld>(Eng. Hist.)</fld>, <cd>an imposition formerly charged on the ports, towns, cities, boroughs, and counties, of England, for providing and furnishing certain ships for the king's service.  The attempt made by <person>Charles I.</person> to revive and enforce this tax was resisted by <person>John Hampden</person>, and was one of the causes which led to the death of <persfn>Charles</persfn>.  It was finally abolished.</cd> -- <col><b>Ship of the line</b></col>. <cd>See under <er>Line</er>.</cd> -- <col><b>Ship pendulum</b></col>, <cd>a pendulum hung amidships to show the extent of the rolling and pitching of a vessel.</cd> -- <col><b>Ship railway</b></col>. <sd>(a)</sd> <cd>An inclined railway with a cradelike car, by means of which a ship may be drawn out of water, as for repairs</cd>. <sd>(b)</sd> <cd>A railway arranged for the transportation of vessels overland between two water courses or harbors.</cd> -- <col><b>Ship's company</b></col>, <cd>the crew of a ship or other vessel.</cd> -- <col><b>Ship's days</b></col>, <cd>the days allowed a vessel for loading or unloading.</cd> -- <col><b>Ship's husband</b></col>. <cd>See under <er>Husband</er>.</cd> -- <col><b>Ship's papers</b></col> <fld>(Mar. Law)</fld>, <cd>papers with which a vessel is required by law to be provided, and the production of which may be required on certain occasions.  Among these papers are the register, passport or sea letter, charter party, bills of lading, invoice, log book, muster roll, bill of health, etc.</cd> <au>Bouvier.</au> <au>Kent.</au> -- <col><b>To make ship</b></col>, <cd>to embark in a ship or other vessel.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Ship</ent><br/
<hw>Ship</hw> <pr>(?)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Shipped</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Shipping</conjf>.]</vmorph> <sn>1.</sn> <def>To put on board of a ship, or vessel of any kind, for transportation; to send by water.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The timber was . . . <qex>shipped</qex> in the bay of Attalia, from whence it was by sea transported to Pelusium.</q> <rj><qau>Knolles.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>By extension, in commercial usage, to commit to any conveyance for transportation to a distance; <as>as, to <ex>ship</ex> freight by railroad</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>Hence, to send away; to get rid of.</def> <mark>[Colloq.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>To engage or secure for service on board of a ship; <as>as, to <ex>ship</ex> seamen</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <def>To receive on board ship; <as>as, to <ex>ship</ex> a sea</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>6.</sn> <def>To put in its place; <as>as, to <ex>ship</ex> the tiller or rudder</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Ship</ent><br/
<hw>Ship</hw>, <pos>v. i.</pos> <sn>1.</sn> <def>To engage to serve on board of a vessel; <as>as, to <ex>ship</ex> on a man-of-war</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To embark on a ship.</def>  <rj><au>Wyclif (Acts xxviii. 11)</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shipboard</ent><br/
<hw>Ship"board`</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[<ets>Ship</ets> + <ets>board.</ets> See <er>Board</er>, <pos>n.</pos>, 8]</ety> <def>A ship's side; hence, by extension, a ship; -- found chiefly in adverbial phrases; <as>as, on <ex>shipboard</ex>; a <ex>shipboard</ex></as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shipbuilder</ent><br/
<hw>Ship"build`er</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A person whose occupation is to construct ships and other vessels; a naval architect; a shipwright.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shipbuilding</ent><br/
<hw>Ship"build`ing</hw>, <pos>n.</pos> <def>Naval architecturel the art of constructing ships and other vessels.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shipful</ent><br/
<hw>Ship"ful</hw> <pr>(?)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Shipfuls</plw> <pr>(<?/)</pr>.</plu> <def>As much or as many as a ship will hold; enough to fill a ship.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shipholder</ent><br/
<hw>Ship"hold`er</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A shipowner.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shipless</ent><br/
<hw>Ship"less</hw>, <pos>a.</pos> <def>Destitute of ships.</def>  <rj><au>Gray.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shiplet</ent><br/
<hw>Ship"let</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A little ship.</def> <mark>[R.]</mark>  <rj><au>Holinshed.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shipload</ent><br/
<hw>Ship"load`</hw> <pr>(?)</pr>, <pos>n.</pos> <def>The load, or cargo, of a ship.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shipman</ent><br/
<hw>Ship"man</hw> <pr>(?)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Shipmen</plw> <pr>(<?/)</pr>.</plu> <def>A seaman, or sailor.</def> <mark>[Obs. or Poetic]</mark>  <rj><au>Chaucer. R. Browning.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>About midnight the <qex>shipmen</qex> deemed that they drew near to some country.</q> <rj><qau>Acts xxvii. 27.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Shipman's card</b></col>, <cd>the mariner's compass.</cd> <mark>[Obs.]</mark>  <rj><au>Shak.</au></rj></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shipmaster</ent><br/
<hw>Ship"mas`ter</hw> <pr>(?)</pr>, <pos>n.</pos> <def>The captain, master, or commander of a ship.</def>  <rj><au>Jonah i. 6.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shipmate</ent><br/
<hw>Ship"mate`</hw> <pr>(?)</pr>, <pos>n.</pos> <def>One who serves on board of the same ship with another; a fellow sailor.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shipment</ent><br/
<hw>Ship"ment</hw> <pr>(?)</pr>, <pos>n.</pos> <sn>1.</sn> <def>The act or process of shipping; <as>as, he was engaged in the <ex>shipment</ex> of coal for London; an active <ex>shipment</ex> of wheat from the West</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>That which is shipped.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The question is, whether the share of M. in the <qex>shipment</qex> is exempted from condemnation by reason of his neutral domicle.</q> <rj><qau>Story.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shipowner</ent><br/
<hw>Ship"own`er</hw> <pr>(?)</pr>, <pos>n.</pos> <def>Owner of a ship or ships.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shippen</ent><br/
<hw>Ship"pen</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[AS. <ets>scypen</ets>.  Cf. <er>Shop</er>, <er>Shepen</er>.]</ety> <def>A stable; a cowhouse.</def> <mark>[Obs. or Prov. Eng.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shipper</ent><br/
<hw>Ship"per</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[See <er>Ship</er>, <pos>n.</pos>, and cf. <er>Skipper</er>.]</ety> <def>One who sends goods from one place to another not in the same city or town, esp. one who sends goods by water.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shipping</ent><br/
<hw>Ship"ping</hw> <pr>(?)</pr>, <pos>a.</pos> <sn>1.</sn> <def>Relating to ships, their ownership, transfer, or employment; <as>as, <ex>shiping</ex> concerns</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Relating to, or concerned in, the forwarding of goods; <as>as, a <ex>shipping</ex> clerk</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shipping</ent><br/
<hw>Ship"ping</hw>, <pos>n.</pos> <sn>1.</sn> <def>The act of one who, or of that which, ships; <as>as, the <ex>shipping</ex> of flour to Liverpool</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>The collective body of ships in one place, or belonging to one port, country, etc.; vessels, generally; tonnage.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>Navigation.</def> <ldquo/God send 'em good <xex>shipping</xex>.<rdquo/  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Shipping articles</b></col>, <cd>articles of agreement between the captain of a vessel and the seamen on board, in respect to the amount of wages, length of time for which they are shipping, etc.</cd> <au>Bouvier.</au> -- <col><b>To take shipping</b></col>, <cd>to embark; to take ship.</cd> <mark>[Obs.]</mark> <au>John vi. 24.</au> <au>Shak.</au></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shipping note</ent><br/
<hw>Ship"ping note</hw>. <fld>(Com.)</fld> <def>A document used in shipping goods by sea. In the case of free goods the shipping notes are the <col><b>receiving note</b></col>, addressed by the shipper to the chief officer of the vessel, requesting him to receive on board specified goods, and a receipt for the mate to sign, on receiving whose signature it is called the <col><b>mate's receipt</b></col>, and is surrendered by the shipper for the bills of lading.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Shippo</ent><br/
||<hw>Ship*po"</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Jap. <ets>shipp<omac/</ets> seven precious things; Chin. <ets>ts'ih</ets> seven + <ets>pao</ets> gem.]</ety> <fld>(Japanese Art)</fld> <def>Cloisonn<eacute/ enamel on a background of metal or porcelain; -- also called <altname>shippo yaki</altname>.</def><br/
[<source>Webster 1913 Suppl.</source> <source>+PJC</source>]</p>

<p><ent>Shippon</ent><br/
<hw>Ship"pon</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A cowhouse; a shippen.</def> <mark>[Prov. Eng.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>Bessy would either do fieldwork, or attend to the cows, the <qex>shippon</qex>, or churn, or make cheese.</q> <rj><qau>Dickens.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Ship railway</ent><br/
<hw>Ship railway</hw>. <sd>(a)</sd> <def>An inclined railway running into the water with a cradelike car on which a vessel may be drawn out on land, as for repairs.</def> <sd>(b)</sd> <def>A railway on which to transport vessels overland between bodies of water.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Ship-rigged</ent><br/
<hw>Ship"-rigged`</hw> <pr>(?)</pr>, <pos>a.</pos> <fld>(Naut.)</fld> <def>Rigged like a ship, that is, having three masts, each with square sails.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shipshape</ent><br/
<hw>Ship"shape`</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Arranged in a manner befitting a ship; hence, trim; tidy; orderly.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Even then she expressed her scorn for the lubbery executioner's mode of tying a knot, and did it herself in a <qex>shipshape</qex> orthodox manner.</q> <rj><qau>De Quincey.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Keep everything <qex>shipshape</qex>, for I must go</q> <rj><qau>Tennyson.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shipshape</ent><br/
<hw>Ship"shape`</hw> <pr>(?)</pr>, <pos>adv.</pos> <def>In a shipshape or seamanlike manner.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shipworm</ent><br/
<hw>Ship"worm`</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Zool.)</fld> <def>Any long, slender, worm-shaped bivalve mollusk of <gen>Teredo</gen> and allied genera. The shipworms burrow in wood, and are destructive to wooden ships, piles of wharves, etc. See <er>Teredo</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shipwreck</ent><br/
<hw>Ship"wreck`</hw> <pr>(?)</pr>, <pos>n.</pos> <sn>1.</sn> <def>The breaking in pieces, or shattering, of a ship or other vessel by being cast ashore or driven against rocks, shoals, etc., by the violence of the winds and waves.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A ship wrecked or destroyed upon the water, or the parts of such a ship; wreckage.</def>  <rj><au>Dryden.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>Fig.: Destruction; ruin; irretrievable loss.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Holding faith and a good conscience, which some having put away concerning faith have made <qex>shipwreck</qex>.</q> <rj><qau>1 Tim. 1. 19.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>It was upon an Indian bill that the late ministry <qex>had made shipwreck</qex>.</q> <rj><qau>J. Morley.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shipwreck</ent><br/
<hw>Ship"wreck`</hw>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Shipwrecked</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Shipwrecking</conjf>.]</vmorph> <sn>1.</sn> <def>To destroy, as a ship at sea, by running ashore or on rocks or sandbanks, or by the force of wind and waves in a tempest.</def><br/
[<source>1913 Webster</source>]</p>

<p><q><qex>Shipwrecking</qex> storms and direful thunders break.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To cause to experience shipwreck, as sailors or passengers. Hence, to cause to suffer some disaster or loss; to destroy or ruin, as if by shipwreck; to wreck; <as>as, to <ex>shipwreck</ex> a business</as>.</def>  <rj><au>Addison.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shipwright</ent><br/
<hw>Ship"wright`</hw> <pr>(?)</pr>, <pos>n.</pos> <def>One whose occupation is to construct ships; a builder of ships or other vessels.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shipyard</ent><br/
<hw>Ship"yard`</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A yard, place, or inclosure where ships are built or repaired.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shiraz</ent><br/
<hw>Shi*raz"</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A kind of Persian wine; -- so called from the place whence it is brought.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shire</ent><br/
<hw>Shire</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[AS. <ets>sc<imac/re</ets>, <ets>sc<imac/r</ets>, a division, province, county.  Cf. <er>Sheriff</er>.]</ety> <sn>1.</sn> <def>A portion of Great Britain originally under the supervision of an earl; a territorial division, usually identical with a county, but sometimes limited to a smaller district; <as>as, Wilt<ex>shire</ex>, York<ex>shire</ex>, Richmond<ex>shire</ex>, Hallam<ex>shire</ex></as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>An indefinite number of these hundreds make up a county or <qex>shire</qex>.</q> <rj><qau>Blackstone.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A division of a State, embracing several contiguous townships; a county.</def> <mark>[U. S.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ <xex>Shire</xex> is commonly added to the specific designation of a county as a part of its name; as, York<xex>shire</xex> instead of York <xex>shire</xex>, or the <xex>shire</xex> of York; Berk<xex>shire</xex> instead of Berks <xex>shire</xex>. Such expressions as <xex>the county of Yorkshire</xex>, which in a strict sense are tautological, are used in England. In the United States the composite word is sometimes the only name of a county; as, <xex>Berkshire county</xex>, as it is called in Massachusetts, instead of <xex>Berks county</xex>, as in Pensylvania.</note><br/
[<source>1913 Webster</source>]</p>

<p><q>The Tyne, Tees, Humber, Wash, Yare, Stour, and Thames separate the counties of Northumberland, Durham, York<qex>shire</qex>, Lincoln<qex>shire</qex>, etc.</q> <rj><qau>Encyc. Brit.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Knight of the shire</b></col>. <cd>See under <er>Knight</er>.</cd> -- <col><b>Shire clerk</b></col>, <cd>an officer of a county court; also, an under sheriff.</cd> <mark>[Eng.]</mark> -- <col><b>Shire mote</b></col> <fld>(Old. Eng. Law)</fld>, <cd>the county court; sheriff's turn, or court.</cd> <mark>[Obs.]</mark> <au>Cowell.</au> <au>Blackstone.</au> -- <col><b>Shire reeve</b></col> <fld>(Old Eng. Law)</fld>, <cd>the reeve, or bailiff, of a shire; a sheriff.</cd> <au>Burrill.</au> -- <col><b>Shire town</b></col>, <cd>the capital town of a county; a county town.</cd> -- <col><b>Shire wick</b></col>, <cd>a county; a shire.</cd> <mark>[Obs.]</mark> <au>Holland.</au></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shire horse</ent><br/
<hw>Shire horse</hw>. <def>One of an English breed of heavy draft horses believed to be descended largely from the horses used in war in the days of heavy armor. They are the  largest of the British draft breeds, and have long hair on the back of the cannons and fetlocks. Brown or bay with white on the face and legs is now the commonest color.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Shirk</ent><br/
<hw>Shirk</hw> <pr>(?)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Shirked</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Shirking</conjf>.]</vmorph> <ety>[Probably the same word as <ets>shark</ets>. See <er>Shark</er>, <pos>v. t.</pos>]</ety> <sn>1.</sn> <def>To procure by petty fraud and trickery; to obtain by mean solicitation.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>You that never heard the call of any vocation, . . . that <qex>shirk</qex> living from others, but time from Yourselves.</q> <rj><qau>Bp. Rainbow.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To avoid; to escape; to neglect; -- implying unfaithfulness or fraud; <as>as, to <ex>shirk</ex> duty</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The usual makeshift by which they try to <qex>shirk</qex> difficulties.</q> <rj><qau>Hare.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shirk</ent><br/
<hw>Shirk</hw>, <pos>v. i.</pos> <sn>1.</sn> <def>To live by shifts and fraud; to shark.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To evade an obligation; to avoid the performance of duty, as by running away.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>One of the cities <qex>shirked</qex> from the league.</q> <rj><qau>Byron.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shirk</ent><br/
<hw>Shirk</hw>, <pos>n.</pos> <def>One who lives by shifts and tricks; one who avoids the performance of duty or labor.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shirker</ent><br/
<hw>Shirk"er</hw> <pr>(?)</pr>, <pos>n.</pos> <def>One who shirks.</def>  <rj><au>Macaulay.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shirky</ent><br/
<hw>Shirk"y</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Disposed to shirk.</def> <mark>[Colloq.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shirl</ent><br/
<hw>Shirl</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Shrill.</def> <mark>[Prov. Eng.]</mark>  <rj><au> Halliwell.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shirl</ent><br/
<hw>Shirl</hw>, <pos>n.</pos> <fld>(Min.)</fld> <def>See <er>Schorl</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shirley</ent><br/
<hw>Shir"ley</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Zool.)</fld> <def>The bullfinch.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shirr</ent><br/
<hw>Shirr</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Sewing)</fld> <def>A series of close parallel runnings which are drawn up so as to make the material between them set full by gatherings; -- called also <altname>shirring</altname>, and <altname>gauging</altname>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shirred</ent><br/
<hw>Shirred</hw> <pr>(?)</pr>, <pos>a.</pos> <sn>1.</sn> <fld>(Sewing)</fld> <def>Made or gathered into a shirr; <as>as, a <ex>shirred</ex> bonnet</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Cookery)</fld> <def>Broken into an earthen dish and baked over the fire; -- said of eggs.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shirt</ent><br/
<hw>Shirt</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[OE. <ets>schirte</ets>, <ets>sherte</ets>, <ets>schurte</ets>; akin to Icel. <ets>skyrta</ets>, Dan. <ets>skiorte</ets>, Sw. <ets>skjorta</ets>, Dan. <ets>ski<oum/rt</ets> a petticoat, D. <ets>schort</ets> a petticoat, an argon, G. <ets>schurz</ets>, <ets>sch<uum/rze</ets>, an argon; all probably from the root of E. <ets>short</ets>, as being originally a short garment. See <er>Short</er>, and cf. <er>Skirt</er>.]</ety> <def>A loose under-garment for the upper part of the body, made of cotton, linen, or other material; -- formerly used of the under-garment of either sex, now commonly restricted to that worn by men and boys.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Several persons in December had nothing over their shoulders but their <qex>shirts</qex>.</q> <rj><qau>Addison.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>She had her <qex>shirts</qex> and girdles of hair.</q> <rj><qau>Bp. Fisher.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shirt</ent><br/
<hw>Shirt</hw>, <pos>v. t. & i.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Shirted</conjf>; <pos>p. pr. & vb. n.</pos> <conjf>Shirting</conjf>.]</vmorph> <def>To cover or clothe with a shirt, or as with a shirt.</def>  <rj><au>Dryden.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shirting</ent><br/
<hw>Shirt"ing</hw>, <pos>n.</pos> <def>Cloth, specifically cotton cloth, suitable for making shirts.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shirtless</ent><br/
<hw>Shirt"less</hw>, <pos>a.</pos> <def>Not having or wearing a shirt.</def>  <rj><au>Pope.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p>-- <wordforms><wf>Shirt"less*ness</wf>, <pos>n.</pos></wordforms><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shirt waist</ent><br/
<ent>shirtwaist</ent><br/
<mhw><hw>shirtwaist</hw>, <hw>Shirt waist</hw></mhw>. <ety>[<ets>shirt</ets> + <ets>waist</ets>; see <er>waist{3}</er>, <pos>n.</pos>]</ety> <sn>1.</sn> <def>A woman's blouse resembling a men's shirt in cut and style; -- in England called a <altname>blouse</altname>.</def><br/
[<source>Webster 1913 Suppl.</source> <source>+PJC</source>]</p>

<p><sn>2.</sn> <def>a type of woman's dress with a bodice tailored like a shirt; -- also called <altname>shirt dress</altname>, <altname>shirtwaist dress</altname>, and <altname>shirtwaister</altname>.</def> <au>[RHUD]</au><br/
[<source>PJC</source>]</p>

<p><ent>Shirt-waist suit</ent><br/
<hw>Shirt-waist suit</hw>. <def>A costume consisting of a plain belted blouse and skirt of the same material.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Shistose</ent><br/
<ent>Shist</ent><br/
<mhw>{ <hw>Shist</hw> <pr>(?)</pr>, <hw>Shis*tose"</hw> <pr>(?)</pr> }</mhw>. <def>See <er>Shist</er>, <er>Schistose</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shittah tree</ent><br/
<ent>Shittah</ent><br/
<mhw>{ <hw>Shit"tah</hw> <pr>(?)</pr>, <hw>Shit"tah tree`</hw>  }</mhw>, <pos>n.</pos> <ety>[Heb. <ets>shitt<amac/h</ets>, pl. <ets>shitt<imac/m</ets>.]</ety> <def>A tree that furnished the precious wood of which the ark, tables, altars, boards, etc., of the Jewish tabernacle were made; -- now believed to have been the wood of the <spn>Acacia Seyal</spn>, which is hard, fine grained, and yellowish brown in color.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shittim wood</ent><br/
<ent>Shittim</ent><br/
<mhw>{ <hw>Shit"tim</hw> <pr>(?)</pr>, <hw>Shit"tim wood`</hw>  }</mhw>, <pos>n.</pos> <def>The wood of the shittah tree.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shittle</ent><br/
<hw>Shit"tle</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[See <er>Shuttle</er>.]</ety> <def>A shuttle.</def> <mark>[Obs.]</mark>  <rj><au>Chapman.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shittle</ent><br/
<hw>Shit"tle</hw>, <pos>a.</pos> <def>Wavering; unsettled; inconstant.</def> <mark>[Obs.]</mark>  <rj><au>Holland.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shittlecock</ent><br/
<hw>Shit"tle*cock`</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A shuttlecock.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shittleness</ent><br/
<hw>Shit"tle*ness</hw>, <pos>n.</pos> <def>Instability; inconstancy.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>The vain <qex>shittlenesse</qex> of an unconstant head.</q> <rj><qau>Baret.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shive</ent><br/
<hw>Shive</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[See <er>Sheave</er>, <pos>n.</pos>]</ety> <sn>1.</sn> <def>A slice; <as>as, a <ex>shive</ex> of bread</as>.</def> <mark>[Obs. or Prov. Eng.]</mark>  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A thin piece or fragment; specifically, one of the scales or pieces of the woody part of flax removed by the operation of breaking.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>A thin, flat cork used for stopping a wide-mouthed bottle; also, a thin wooden bung for casks.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shiver</ent><br/
<hw>Shiv"er</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[OE. <ets>schivere</ets>, fr. <ets>shive</ets>; cf. G. <ets>schifer</ets> a splinter, slate, OHG. <ets>scivere</ets> a splinter, Dan. & Sw. <ets>skifer</ets> a slate. See <er>Shive</er>, and cf. <er>Skever</er>.]</ety> <sn>1.</sn> <def>One of the small pieces, or splinters, into which a brittle thing is broken by sudden violence; -- generally used in the plural.</def> <ldquo/All to <xex>shivers</xex> dashed.<rdquo/  <rj><au>Milton.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A thin slice; a shive.</def> <mark>[Obs. or Prov. Eng.]</mark> <ldquo/A <xex>shiver</xex> of their own loaf.<rdquo/  <rj><au>Fuller.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Of your soft bread, not but a <qex>shiver</qex>.</q> <rj><qau>Chaucer.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <fld>(Geol.)</fld> <def>A variety of blue slate.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <fld>(Naut.)</fld> <def>A sheave or small wheel in a pulley.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <def>A small wedge, as for fastening the bolt of a window shutter.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>6.</sn> <def>A spindle.</def> <mark>[Obs.  or Prov. Eng.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shiver</ent><br/
<hw>Shiv"er</hw>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Shivered</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Shivering</conjf>.]</vmorph> <ety>[OE. <ets>schiveren</ets>, <ets>scheveren</ets>; cf. OD. <ets>scheveren</ets>. See <er>Shiver</er> a fragment.]</ety> <def>To break into many small pieces, or splinters; to shatter; to dash to pieces by a blow; <as>as, to <ex>shiver</ex> a glass goblet</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>All the ground<br/
With <qex>shivered</qex> armor strown.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shiver</ent><br/
<hw>Shiv"er</hw>, <pos>v. i.</pos> <def>To separate suddenly into many small pieces or parts; to be shattered.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>There <qex>shiver</qex> shafts upon shields thick.</q> <rj><qau>Chaucer</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>The natural world, should gravity once cease, . . . would instantly <qex>shiver</qex> into millions of atoms.</q> <rj><qau>Woodward.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shiver</ent><br/
<hw>Shiv"er</hw>, <pos>v. i.</pos> <ety>[OE. <ets>chiveren</ets>, <ets>cheveren</ets>; of uncertain origin. This word seems to have been confused with <ets>shiver</ets> to shatter.]</ety> <def>To tremble; to vibrate; to quiver; to shake, as from cold or fear.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Prometheus is laid<br/
On icy Caucasus to <qex>shiver</qex>.</q> <rj><qau>Swift.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>The man that <qex>shivered</qex> on the brink of sin,<br/
Thus steeled and hardened, ventures boldly in.</q> <rj><qau>Creech.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shiver</ent><br/
<hw>Shiv"er</hw>, <pos>v. t.</pos> <fld>(Naut.)</fld> <def>To cause to shake or tremble, as a sail, by steering close to the wind.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shiver</ent><br/
<hw>Shiv"er</hw>, <pos>n.</pos> <def>The act of shivering or trembling.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shiveringly</ent><br/
<hw>Shiv"er*ing*ly</hw>, <pos>adv.</pos> <def>In a shivering manner.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shiver-spar</ent><br/
<hw>Shiv"er-spar`</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Cf. G. <ets>schiefer-spath</ets>.]</ety> <fld>(Min.)</fld> <def>A variety of calcite, so called from its slaty structure; -- called also <altname>slate spar</altname>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shivery</ent><br/
<hw>Shiv"er*y</hw> <pr>(?)</pr>, <pos>a.</pos> <sn>1.</sn> <def>Tremulous; shivering.</def>  <rj><au>Mallet.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Easily broken; brittle; shattery.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shizoku</ent><br/
||<hw>Shi*zo"ku</hw> <pr>(?)</pr>, <pos>n. sing. & pl.</pos> <ety>[Jap. <ets>shi-zoku</ets>, fr. Chin. <ets>ch'</ets> (<ets>chi</ets>) branch, posterity + <ets>tsu</ets> kindered, class.]</ety> <def>The Japanese warrior gentry or middle class, formerly called <altname>samurai</altname>; also, any member of this class.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Shoad</ent><br/
<hw>Shoad</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Cf. G. <ets>schutt</ets> rubbish.]</ety> <fld>(Mining)</fld> <def>A train of vein material mixed with rubbish; fragments of ore which have become separated by the action of water or the weather, and serve to direct in the discovery of mines.</def> <altsp>[Written also <asp>shode</asp>.]</altsp><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shoading</ent><br/
<hw>Shoad"ing</hw>, <pos>n.</pos> <fld>(Mining)</fld> <def>The tracing of veins of metal by shoads.</def> <altsp>[Written also <asp>shoding</asp>.]</altsp>  <rj><au>Pryce.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shoal</ent><br/
<hw>Shoal</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[AS. <ets>scolu</ets>, <ets>sceolu</ets>, a company, multitude, crowd, akin to OS. <ets>skola</ets>; probably originally, a division, and akin to Icel. <ets>skilja</ets> to part, divide. See <er>Skill</er>, and cf. <er>School</er>. of fishes.]</ety> <def>A great multitude assembled; a crowd; a throng; -- said especially of fish; <as>as, a <ex>shoal</ex> of bass</as>.</def> <ldquo/Great <xex>shoals</xex> of people.<rdquo/  <rj><au>Bacon.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Beneath, a <qex>shoal</qex> of silver fishes glides.</q> <rj><qau>Waller.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shoal</ent><br/
<hw>Shoal</hw>, <pos>v. i.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Shoaled</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Shoaling</conjf>.]</vmorph> <def>To assemble in a multitude; to throng; <as>as, the fishes <ex>shoaled</ex> about the place</as>.</def>  <rj><au>Chapman.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shoal</ent><br/
<hw>Shoal</hw>, <pos>a.</pos> <ety>[Cf. <er>Shallow</er>; or cf. G. <ets>scholle</ets> a clod, glebe, OHG. <ets>scollo</ets>, <ets>scolla</ets>, prob. akin to E. <ets>shoal</ets> a multitude.]</ety> <def>Having little depth; shallow; <as>as, <ex>shoal</ex> water</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shoal</ent><br/
<hw>Shoal</hw>, <pos>n.</pos> <sn>1.</sn> <def>A place where the water of a sea, lake, river, pond, etc., is shallow; a shallow.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The depth of your pond should be six feet; and on the sides some <qex>shoals</qex> for the fish to lay their span.</q> <rj><qau>Mortimer.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Wolsey, that once trod the ways of glory,<br/
And sounded all the depths and <qex>shoals</qex> of honor.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A sandbank or bar which makes the water shoal.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The god himself with ready trident stands,<br/
And opes the deep, and spreads the moving sands,<br/
Then heaves them off the <qex>shoals</qex>.</q> <rj><qau>Dryden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shoal</ent><br/
<hw>Shoal</hw>, <pos>v. i.</pos> <def>To become shallow; <as>as, the color of the water shows where it <ex>shoals</ex></as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shoal</ent><br/
<hw>Shoal</hw>, <pos>v. t.</pos> <def>To cause to become more shallow; to come to a more shallow part of; <as>as, a ship <ex>shoals</ex> her water by advancing into that which is less deep</as>.</def>  <rj><au>Marryat.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shoaliness</ent><br/
<hw>Shoal"i*ness</hw> <pr>(?)</pr>, <pos>n.</pos> <def>The quality or state of being shoaly; little depth of water; shallowness.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shoaling</ent><br/
<hw>Shoal"ing</hw>, <pos>a.</pos> <def>Becoming shallow gradually.</def> <ldquo/A <xex>shoaling</xex> estuary.<rdquo/  <rj><au>Lyell.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shoaly</ent><br/
<hw>Shoal"y</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Full of shoals, or shallow places.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The tossing vessel sailed on <qex>shoaly</qex> ground.</q> <rj><qau>Dryden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shoar</ent><br/
<hw>Shoar</hw> <pr>(sh<omac/r)</pr>, <pos>n.</pos> <def>A prop. See 3d <er>Shore</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shoat</ent><br/
<hw>Shoat</hw> <pr>(sh<omac/t)</pr>, <pos>n.</pos> <def>A young hog. Same as <er>Shote</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><-- p. 1331 --></p>

<p><ent>Shock</ent><br/
<hw>Shock</hw> <pr>(sh<ocr/k)</pr>, <pos>n.</pos> <ety>[OE. <ets>schokke</ets>; cf. OD <ets>schocke</ets>, G. <ets>schock</ets> a heap, quantity, threescore, MHG. <ets>schoc</ets>, Sw. <ets>skok</ets>, and also G. <ets>hocke</ets> a heap of hay, Lith. <ets>kugis</ets>.]</ety> <sn>1.</sn> <def>A pile or assemblage of sheaves of grain, as wheat, rye, or the like, set up in a field, the sheaves varying in number from twelve to sixteen; a stook.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>And cause it on <qex>shocks</qex> to be by and by set.</q> <rj><qau>Tusser.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Behind the master walks, builds up the <qex>shocks</qex>.</q> <rj><qau>Thomson.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <ety>[G. <ets>schock</ets>.]</ety> <fld>(Com.)</fld> <def>A lot consisting of sixty pieces; -- a term applied in some Baltic ports to loose goods.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shock</ent><br/
<hw>Shock</hw>, <pos>v. t.</pos> <def>To collect, or make up, into a shock or shocks; to stook; <as>as, to <ex>shock</ex> rye</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shock</ent><br/
<hw>Shock</hw>, <pos>v. i.</pos> <def>To be occupied with making shocks.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Reap well, scatter not, gather clean that is shorn,<br/
Bind fast, <qex>shock</qex> apace.</q> <rj><qau>Tusser.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shock</ent><br/
<hw>Shock</hw>, <pos>n.</pos> <ety>[Cf. D. <ets>schok</ets> a bounce, jolt, or leap, OHG. <ets>scoc</ets> a swing, MHG. <ets>schoc</ets>, Icel. <ets>skykkjun</ets> tremuously, F. <ets>choc</ets> a shock, collision, a dashing or striking against, Sp. <ets>choque</ets>, It. <ets>ciocco</ets> a log. <root/161.  Cf. <er>Shock</er> to shake.]</ety> <sn>1.</sn> <def>A quivering or shaking which is the effect of a blow, collision, or violent impulse; a blow, impact, or collision; a concussion; a sudden violent impulse or onset.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>These strong, unshaken mounds resist the <qex>shocks</qex><br/
Of tides and seas tempestuous.</q> <rj><qau>Blackmore.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>He stood the <qex>shock</qex> of a whole host of foes.</q> <rj><qau>Addison.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A sudden agitation of the mind or feelings; a sensation of pleasure or pain caused by something unexpected or overpowering; also, a sudden agitating or overpowering event.</def> <ldquo/A <xex>shock</xex> of pleasure.<rdquo/  <rj><au>Talfourd.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <fld>(Med.)</fld> <def>A sudden depression of the vital forces of the entire body, or of a part of it, marking some profound impression produced upon the nervous system, as by severe injury, overpowering emotion, or the like.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <fld>(Elec.)</fld> <def>The sudden convulsion or contraction of the muscles, with the feeling of a concussion, caused by the discharge, through the animal system, of electricity from a charged body.</def><br/
[<source>1913 Webster</source>]</p>

<p><syn><b>Syn.</b> -- <er>Concussion</er>, <er>Shock</er>.</syn> <usage> Both words signify a sudden violent shaking caused by impact or colision; but <xex>concussion</xex> is restricted in use to matter, while <xex>shock</xex> is used also of mental states.</usage><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shock</ent><br/
<hw>Shock</hw>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Shocked</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Shocking</conjf>.]</vmorph> <ety>[OE. <ets>schokken</ets>; cf. D. <ets>schokken</ets>, F. <ets>choquer</ets>, Sp. <ets>chocar</ets>. <root/161.  Cf. <er>Chuck</er> to strike, <er>Jog</er>, <er>Shake</er>, <er>Shock</er> a striking, <er>Shog</er>, <pos>n.</pos> & <pos>v.</pos>]</ety> <sn>1.</sn> <def>To give a shock to; to cause to shake or waver; hence, to strike against suddenly; to encounter with violence.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Come the three corners of the world in arms,<br/
And we shall <qex>shock</qex> them.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>I shall never forget the force with which he <qex>shocked</qex> De Vipont.</q> <rj><qau>Sir W. Scott.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To strike with surprise, terror, horror, or disgust; to cause to recoil; <as>as, his violence <ex>shocked</ex> his associates</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Advise him not to <qex>shock</qex> a father's will.</q> <rj><qau>Dryden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <fld>(Physiol.)</fld> <def>To subject to the action of an electrical discharge so as to cause a more or less violent depression or commotion of the nervous system.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Shock</ent><br/
<hw>Shock</hw>, <pos>v. i.</pos> <def>To meet with a shock; to meet in violent encounter.</def> <ldquo/They saw the moment approach when the two parties would <xex>shock</xex> together.<rdquo/  <rj><au>De Quincey.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shock</ent><br/
<hw>Shock</hw>, <pos>n.</pos> <ety>[Cf. <er>Shag</er>.]</ety> <sn>1.</sn> <fld>(Zool.)</fld> <def>A dog with long hair or shag; -- called also <altname>shockdog</altname>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A thick mass of bushy hair; <as>as, a head covered with a <ex>shock</ex> of sandy hair</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shock</ent><br/
<hw>Shock</hw>, <pos>a.</pos> <def>Bushy; shaggy; <as>as, a <ex>shock</ex> hair</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>His red <qex>shock</qex> peruke . . . was laid aside.</q> <rj><qau>Sir W. Scott.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shockdog</ent><br/
<hw>Shock"dog`</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Zool.)</fld> <def>See 7th <er>Shock</er>, 1.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shock-head</ent><br/
<hw>Shock"-head`</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Shock-headed.</def>  <rj><au>Tennyson.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shock-headed</ent><br/
<hw>Shock"-head`ed</hw>, <pos>a.</pos> <def>Having a thick and bushy head of hair.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shocking</ent><br/
<hw>Shock"ing</hw>, <pos>a.</pos> <def>Causing to shake or tremble, as by a blow; especially, causing to recoil with horror or disgust; extremely offensive or disgusting.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The grossest and most <qex>shocking</qex> villainies.</q> <rj><qau>Secker.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p>-- <wordforms><wf>Shock"ing*ly</wf>, <pos>adv.</pos> -- <wf>Shock"ing*ness</wf>, <pos>n.</pos></wordforms><br/
[<source>1913 Webster</source>]</p>

<p><-- Shock troops, a highly trained or seasoned group within an army used to spearhead a strong offensive action. --><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shod</ent><br/
<hw>Shod</hw> <pr>(?)</pr>, <pos>imp. & p. p.</pos> <def>f <er>Shoe</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shoddy</ent><br/
<hw>Shod"dy</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Perhaps fr. <er>Shed</er>, <pos>v. t.</pos>; as meaning originally, waste stuff <ets>shed</ets> or thrown off; cf. dial. <ets>shod</ets> to shed, and E. <er>Shed</er> a parting, separation, <er>Shode</er> a parting.]</ety> <sn>1.</sn> <def>A fibrous material obtained by <ldquo/deviling,<rdquo/ or tearing into fibers, refuse woolen goods, old stockings, rags, druggets, etc. See <er>Mungo</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Fluffy, fibrous waste from wool carding, worsted spinning, or weaving of woolens.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><sn>3.</sn> <def>A fabric of inferior quality made of, or containing a large amount of, shoddy.</def><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ The great quantity of shoddy goods furnished as army supplies in the late Civil War in the United States gave wide currency to the word, and it came to be applied to persons who pretend to a higher position in society than that to which their breeding or worth entitles them; this term is now (1997) rarely used in that sense.</note><br/
[<source>1913 Webster</source> <source>+PJC</source>]</p>

<p><ent>Shoddy fever</ent><br/
<hw>Shoddy fever</hw>. <fld>(Med.)</fld> <def>A febrile disease characterized by dyspnoea and bronchitis caused by inhaling dust.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Shoddy</ent><br/
<hw>Shod"dy</hw>, <pos>a.</pos> <sn>1.</sn> <def>Made wholly or in part of shoddy; containing shoddy; <as>as, <ex>shoddy</ex> cloth; <ex>shoddy</ex> blankets</as>; hence, colloquially, not genuine; sham; pretentious; <as>as, <ex>shoddy</ex> aristocracy</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q><qex>Shoddy</qex> inventions designed to bolster up a factitious pride.</q> <rj><qau>Compton Reade.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>of poor quality or inferior workmanship.</def> <au>[RHUD]</au><br/
[<source>PJC</source>]</p>

<p><sn>3.</sn> <def>visibly worn or damaged from use; shabby.</def><br/
[<source>PJC</source>]</p>

<p><ent>Shoddyism</ent><br/
<hw>Shod"dy*ism</hw> <pr>(?)</pr>, <pos>n.</pos> <def>The quality or state of being shoddy.</def> <mark>[Colloq.]</mark> <see>See the Note under <er>Shoddy</er>, <pos>n.</pos></see><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shode</ent><br/
<hw>Shode</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[AS. <ets>sc<amac/de</ets>, fr. <ets>sce<aacute/dan</ets>. See <er>Shed</er>, <pos>v. t.</pos>]</ety> <sn>1.</sn> <def>The parting of the hair on the head.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>Full straight and even lay his jolly <qex>shode</qex>.</q> <rj><qau>Chaucer.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>The top of the head; the head.</def> <mark>[Obs.]</mark>  <rj><au>Chaucer.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shoding</ent><br/
<ent>Shode</ent><br/
<mhw>{ <hw>Shode</hw>, <hw>Shod"ing</hw>  }</mhw>. <def>See <er>Shoad</er>, <er>Shoading</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shoder</ent><br/
<hw>Sho"der</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A package of gold beater's skins in which gold is subjected to the second process of beating.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shoe</ent><br/
<hw>Shoe</hw> <pr>(sh<oomac/)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Shoes</plw> <pr>(sh<oomac/z)</pr>, <mark>formerly</mark> <plw>Shoon</plw> <pr>(sh<oomac/n)</pr>, <mark>now provincial</mark>.</plu> <ety>[OE. <ets>sho</ets>, <ets>scho</ets>, AS. <ets>sc<omac/h</ets>, <ets>sce<oacute/h</ets>; akin to OFries. <ets>sk<omac/</ets>, OS. <ets>sk<omac/h</ets>, D. <ets>schoe</ets>, <ets>schoen</ets>, G. <ets>schuh</ets>, OHG. <ets>scuoh</ets>, Icel. <ets>sk<omac/r</ets>, Dan. & Sw. <ets>sko</ets>, Goth. <ets>sk<omac/hs</ets>; of unknown origin.]</ety> <sn>1.</sn> <def>A covering for the human foot, usually made of leather, having a thick and somewhat stiff sole and a lighter top. It differs from a boot on not extending so far up the leg.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Your hose should be ungartered, . . . your<qex>shoe</qex> untied.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Spare none but such as go in clouted <qex>shoon</qex>.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Anything resembling a shoe in form, position, or use.</def> Specifically: <sd>(a)</sd> <def>A plate or rim of iron nailed to the hoof of an animal to defend it from injury.</def> <sd>(b)</sd> <def>A band of iron or steel, or a ship of wood, fastened to the bottom of the runner of a sleigh, or any vehicle which slides on the snow.</def> <sd>(c)</sd> <def>A drag, or sliding piece of wood or iron, placed under the wheel of a loaded vehicle, to retard its motion in going down a hill.</def> <sd>(d)</sd> <def>The part of an automobile or railroad car brake which presses upon the wheel to retard its motion.</def> <sd>(e)</sd> <fld>(Arch.)</fld> <def>A trough-shaped or spout-shaped member, put at the bottom of the water leader coming from the eaves gutter, so as to throw the water off from the building.</def> <sd>(f)</sd> <fld>(Milling.)</fld> <def>The trough or spout for conveying the grain from the hopper to the eye of the millstone.</def> <sd>(g)</sd> <def>An inclined trough in an ore-crushing mill.</def> <sd>(h)</sd> <def>An iron socket or plate to take the thrust of a strut or rafter.</def> <sd>(i)</sd> <def>An iron socket to protect the point of a wooden pile.</def> <sd>(j)</sd> <fld>(Mach.)</fld> <def>A plate, or notched piece, interposed between a moving part and the stationary part on which it bears, to take the wear and afford means of adjustment; -- called also <altname>slipper</altname>, and <altname>gib</altname>.</def><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ <xex>Shoe</xex> is often used adjectively, or in composition; as, <xex>shoe</xex> buckle, or <xex>shoe</xex>-buckle; <xex>shoe</xex> latchet, or <xex>shoe</xex>-latchet; <xex>shoe</xex> leathet, or <xex>shoe</xex>-leather; <xex>shoe</xex> string, <xex>shoe</xex>-string, or <xex>shoe</xex>string.</note><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>The outer cover or tread of a pneumatic tire, esp. for an automobile.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><cs><col><b>Shoe of an anchor</b></col>. <fld>(Naut.)</fld> <sd>(a)</sd> <cd>A small block of wood, convex on the back, with a hole to receive the point of the anchor fluke, -- used to prevent the anchor from tearing the planks of the vessel when raised or lowered.</cd> <sd>(b)</sd> <cd>A broad, triangular piece of plank placed upon the fluke to give it a better hold in soft ground.</cd> -- <col><b>Shoe block</b></col> <fld>(Naut.)</fld>, <cd>a block with two sheaves, one above the other, and at right angles to each other.</cd> -- <col><b>Shoe bolt</b></col>, <cd>a bolt with a flaring head, for fastening shoes on sleigh runners.</cd> -- <col><b>Shoe pac</b></col>, <cd>a kind of moccasin. See <er>Pac</er>.</cd> -- <col><b>Shoe stone</b></col>, <cd>a sharpening stone used by shoemakers and other workers in leather.</cd></cs>
<-- brake shoe. the movable portion of a drum brake on a vehicle which is pressed against the rotating drum to slow or stop the vehicle by the friction of the brakeshoe against the drum. --><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shoe</ent><br/
<hw>Shoe</hw> <pr>(?)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Shod</conjf>; <pos>p. pr. & vb. n.</pos> <conjf>Shoeing</conjf>.]</vmorph> <ety>[AS. <ets>sc<?/ian</ets>, <ets>sce<?/ian</ets>. See <er>Shoe</er>, <pos>n.</pos>]</ety> <sn>1.</sn> <def>To furnish with a shoe or shoes; to put a shoe or shoes on; <as>as, to <ex>shoe</ex> a horse, a sled, an anchor</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To protect or ornament with something which serves the purpose of a shoe; to tip.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The sharp and small end of the billiard stick, which is <qex>shod</qex> with brass or silver.</q> <rj><qau>Evelyn.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shoebill</ent><br/
<hw>Shoe"bill`</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Zool.)</fld> <def>A large African wading bird (<spn>Balaeniceps rex</spn>) allied to the storks and herons, and remarkable for its enormous broad swollen bill. It inhabits the valley of the White Nile. See <xex>Illust.</xex> (<it>l.</it>) of <er>Beak</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shoeblack</ent><br/
<hw>Shoe"black`</hw> <pr>(?)</pr>, <pos>n.</pos> <def>One who polishes shoes; same as <er>bootblack</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shoefly</ent><br/
<hw>Shoe"fly`</hw> <pr>(?)</pr>, <pos>n.</pos> <sn>1.</sn> <fld>(Railroading)</fld> <def>A contrivance for throwing the track temporarily to one side for  convenience in filling washouts or effecting other repairs.</def> <mark>[Cant, U. S.]</mark><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><sn>2.</sn>  <fld>(Print.)</fld> <def>In some cylinder presses, a device with long fingers for freeing the sheet from the cylinder.</def></p>

<p><ent>Shoeing-horn</ent><br/
<ent>Shoehorn</ent><br/
<mhw>{ <hw>Shoe"horn`</hw>, <hw>Shoe"ing-horn`</hw>  }</mhw> <pr>(?)</pr>, <pos>n.</pos> <sn>1.</sn> <def>A curved piece of polished horn, wood, or metal used to facilitate the entrance of the foot into a shoe.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> Figuratively: <sd>(a)</sd> <def>Anything by which a transaction is facilitated; a medium; -- by way of contempt.</def> <au>Spectator.</au> <sd>(b)</sd> <def>Anything which draws on or allures; an inducement.</def> <mark>[Low]</mark>  <rj><au>Beau. & Fl.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shoehorn</ent><br/
<hw>Shoe"horn</hw>, <pos>v. t.</pos> <def>to squeeze or force into a tight-fitting space, with or as though with a shoehorn; -- often used figuratively.</def><br/
[<source>PJC</source>]</p>

<p><ent>Shoeless</ent><br/
<hw>Shoe"less</hw>, <pos>a.</pos> <def>Destitute of shoes.</def>  <rj><au>Addison.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shoelace</ent><br/
<hw>Shoe"lace</hw>, <pos>n.</pos> <def>a length of cord for tying the upper parts of a shoe together.  Commercial <ex>shoelaces</ex> usually come in different lengths, and have each end confined in a narrow plastic tube for convenience in inserting the cords through the holes in a shoe upper.  Also called <altname>shoestring</altname>.</def><br/
[<source>PJC</source>]</p>

<p><ent>Shoemaker</ent><br/
<hw>Shoe"mak`er</hw> <pr>(?)</pr>, <pos>n.</pos> <sn>1.</sn> <def>One whose occupation it is to make shoes and boots.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Zool.)</fld> <sd>(a)</sd> <def>The threadfish.</def> <sd>(b)</sd> <def>The runner, 12.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shoemaking</ent><br/
<hw>Shoe"mak`ing</hw>, <pos>n.</pos> <def>The business of a shoemaker.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shoer</ent><br/
<hw>Sho"er</hw> <pr>(?)</pr>, <pos>n.</pos> <def>One who fits shoes to the feet; one who furnishes or puts on shoes; <as>as, a <ex>shoer</ex> of horses</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shoestring</ent><br/
<hw>Shoe"string`</hw>, <pos>n.</pos> <sn>1.</sn> <def>same as <er>shoelace</er>.</def><br/
[<source>PJC</source>]</p>

<p><sn>2.</sn> <def>a sum of money, inadequate or barely adequate for the purpose to which it is put to use; <as>as, to start up an enterprise on a <ex>shoestring</ex></as>.</def><br/
[<source>PJC</source>]</p>

<p><ent>Shog</ent><br/
<hw>Shog</hw> <pr>(sh<ocr/g)</pr>, <pos>n.</pos> <ety>[See <er>Shock</er> a striking.]</ety> <def>A shock; a jog; a violent concussion or impulse.</def> <mark>[R. or Scot.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shog</ent><br/
<hw>Shog</hw>, <pos>v. t.</pos> <def>To shake; to shock.</def> <mark>[R. or Scot.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shog</ent><br/
<hw>Shog</hw>, <pos>v. i.</pos> <ety>[Cf. W. <ets>ysgogi</ets> to wag, to stir.  Cf. <er>Jog</er>.]</ety> <def>To jog; to move on.</def> <mark>[R. or Scot.]</mark>  <rj><au>Beau. & Fl.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shoggle</ent><br/
<hw>Shog"gle</hw> <pr>(?)</pr>, <pos>v. t.</pos> <ety>[See <er>Shog</er>, <er>Joggle</er>.]</ety> <def>To joggle.</def> <mark>[Obs. or Prov. Eng. & Scot.]</mark>  <rj><au>Pegge.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shogun</ent><br/
<hw>Sho"gun</hw> <pr>(sh<omac/"g<oomac/n)</pr>, <pos>n.</pos> <ety>[Jap. military general, from Chin. <ets>tsiang ki<uum/n</ets> commander in chief.]</ety> <def>A title originally conferred by the Mikado on the military governor of the eastern provinces of Japan.  By gradual usurpation of power the <ex>Shoguns</ex> (known to foreigners as <altname>Tycoons</altname>) became finally the virtual rulers of Japan.  The title was abolished in 1867.</def> <altsp>[Written variously, <asp>Shiogun</asp>, <asp>Shiogoon</asp>, etc.]</altsp><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shogunate</ent><br/
<hw>Sho*gun"ate</hw> <pr>(sh<omac/*g<oomac/n"<asl/t)</pr>, <pos>n.</pos> <def>The office or dignity of a Shogun.</def> <altsp>[Written also <asp>Siogoonate</asp>.]</altsp><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shola</ent><br/
<hw>Sho"la</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Bot.)</fld> <def>See <er>Sola</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shole</ent><br/
<hw>Shole</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A plank fixed beneath an object, as beneath the rudder of a vessel, to protect it from injury; a plank on the ground under the end of a shore or the like.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shole</ent><br/
<hw>Shole</hw>, <pos>n.</pos> <def>See <er>Shoal</er>.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shonde</ent><br/
<hw>Shonde</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[AS. <ets>sceond</ets>.  Cf. <er>Shend</er>.]</ety> <def>Harm; disgrace; shame.</def> <mark>[Obs.]</mark>  <rj><au>Chaucer.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shone</ent><br/
<hw>Shone</hw> <pr>(?)</pr>, <def><pos>imp. & p. p.</pos> of <er>Shine</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shoo</ent><br/
<hw>Shoo</hw> <pr>(?)</pr>, <pos>interj.</pos> <ety>[Cf. G. <ets>scheuchen</ets> to scare, drive away.]</ety> <def>Begone; away; -- an expression used in frightening away animals, especially fowls.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>shoo-in</ent><br/
<-- <hw>shoo-in</hw>  <def>a candidate who is certain to win easily.</def> --><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Shooi</ent><br/
<hw>Sho"oi</hw>, <pos>n.</pos> <fld>(Zool.)</fld> <def>The Richardson's skua (<spn>Stercorarius parasiticus</spn>);- so called from its cry.</def> <mark>[Prov. Eng.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shook</ent><br/
<hw>Shook</hw> <pr>(sh<oocr/k)</pr>, <def><pos>imp.</pos> & <mark>obs.</mark>  or <mark>poet.</mark> <pos>p. p.</pos> of <er>Shake</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shook</ent><br/
<hw>Shook</hw>, <pos>n.</pos> <ety>[Cf. <er>Shock</er> a bundle of sheaves.]</ety> <fld>(Com.)</fld> <sd>(a)</sd> <def>A set of staves and headings sufficient in number for one hogshead, cask, barrel, or the like, trimmed, and bound together in compact form.</def> <sd>(b)</sd> <def>A set of boards for a sugar box.</def> <sd>(c)</sd> <def>The parts of a piece of house furniture, as a bedstead, packed together.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shook</ent><br/
<hw>Shook</hw>, <pos>v. t.</pos> <def>To pack, as staves, in a shook.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shoon</ent><br/
<hw>Shoon</hw> <pr>(?)</pr>, <pos>n.</pos>, <def><pos>pl.</pos> of <er>Shoe</er>.</def> <mark>[Archaic]</mark>  <rj><au>Chaucer.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>They shook the snow from hats and <qex>shoon</qex>.</q> <rj><qau>Emerson.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shoop</ent><br/
<hw>Shoop</hw> <pr>(?)</pr>, <mark>obs.</mark> <def><pos>imp.</pos> of <er>Shape</er>. Shaped.</def>  <rj><au>Chaucer.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shoot</ent><br/
<hw>Shoot</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F. <ets>chute</ets>. See <er>Chute</er>. Confused with <ets>shoot</ets> to let fly.]</ety> <def>An inclined plane, either artificial or natural, down which timber, coal, etc., are caused to slide; also, a narrow passage, either natural or artificial, in a stream, where the water rushes rapidly; esp., a channel, having a swift current, connecting the ends of a bend in the stream, so as to shorten the course.</def> <altsp>[Written also <asp>chute</asp>, and <asp>shute</asp>.]</altsp> <mark>[U. S.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>To take a shoot</b></col>, <cd>to pass through a shoot instead of the main channel; to take the most direct course.</cd> <mark>[U.S.]</mark></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shoot</ent><br/
<hw>Shoot</hw> <pr>(?)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Shot</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Shooting</conjf>. The old participle <conjf>Shotten</conjf> is obsolete. See <er>Shotten</er>.]</vmorph> <ety>[OE. <ets>shotien</ets>, <ets>schotien</ets>, AS. <ets>scotian</ets>, v. i., <ets>sce<oacute/tan</ets>; akin to D. <ets>schieten</ets>, G. <ets>schie<?/en</ets>, OHG. <ets>sciozan</ets>, Icel. <ets>skj<?/ta</ets>, Sw. <ets>skjuta</ets>, Dan. <ets>skyde</ets>; cf. Skr. <ets>skund</ets> to jump. <root/159.  Cf. <er>Scot</er> a contribution, <er>Scout</er> to reject, <er>Scud</er>, <er>Scuttle</er>, <pos>v. i.</pos>, <er>Shot</er>, <er>Sheet</er>, <er>Shut</er>, <er>Shuttle</er>, <er>Skittish</er>, <er>Skittles</er>.]</ety> <sn>1.</sn> <def>To let fly, or cause to be driven, with force, as an arrow or a bullet; -- followed by a word denoting the missile, as an object.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>If you please<br/
To shoot an arrow that self way.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To discharge, causing a missile to be driven forth; -- followed by a word denoting the weapon or instrument, as an object; -- often with <xex>off</xex>; <as>as, to <ex>shoot</ex> a gun</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The two ends od a bow, <qex>shot</qex> off, fly from one another.</q> <rj><qau>Boyle.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>To strike with anything shot; to hit with a missile; often, to kill or wound with a firearm; -- followed by a word denoting the person or thing hit, as an object.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>When Roger <qex>shot</qex> the hawk hovering over his master's dove house.</q> <rj><qau>A. Tucker.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>To send out or forth, especially with a rapid or sudden motion; to cast with the hand; to hurl; to discharge; to emit.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>An honest weaver as ever <qex>shot</qex> shuttle.</q> <rj><qau>Beau. & Fl.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>A pit into which the dead carts had nightly <qex>shot</qex> corpses by scores.</q> <rj><qau>Macaulay.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <def>To push or thrust forward; to project; to protrude; -- often with <xex>out</xex>; <as>as, a plant <ex>shoots</ex> out a bud</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>They <qex>shoot</qex> out the lip, they shake the head.</q> <rj><qau>Ps. xxii. 7.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Beware the secret snake that <qex>shoots</qex> a sting.</q> <rj><qau>Dryden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>6.</sn> <fld>(Carp.)</fld> <def>To plane straight; to fit by planing.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Two pieces of wood that are <qex>shot</qex>, that is, planed or else pared with a paring chisel.</q> <rj><qau>Moxon.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>7.</sn> <def>To pass rapidly through, over, or under; <as>as, to <ex>shoot</ex> a rapid or a bridge; to <ex>shoot</ex> a sand bar.</as></def><br/
[<source>1913 Webster</source>]</p>

<p><q>She . . . <qex>shoots</qex> the Stygian sound.</q> <rj><qau>Dryden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>8.</sn> <def>To variegate as if by sprinkling or intermingling; to color in spots or patches.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The tangled water courses slept,<br/
<qex>Shot</qex> over with purple, and green, and yellow.</q> <rj><qau>Tennyson.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>To be shot of</b></col>, <cd>to be discharged, cleared, or rid of.</cd> <mark>[Colloq.]</mark> <ldquo/Are you not glad <xex>to be shot of</xex> him?<rdquo/</cs>
 <rj><qau>Sir W. Scott.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shoot</ent><br/
<hw>Shoot</hw>, <pos>v. i.</pos> <sn>1.</sn> <def>To cause an engine or weapon to discharge a missile; -- said of a person or an agent; <as>as, they <ex>shot</ex> at a target; he <ex>shoots</ex> better than he rides</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The archers have . . . <qex>shot</qex> at him.</q> <rj><qau>Gen. xlix. 23.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To discharge a missile; -- said of an engine or instrument; <as>as, the gun <ex>shoots</ex> well</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>To be shot or propelled forcibly; -- said of a missile; to be emitted or driven; to move or extend swiftly, as if propelled; <as>as, a <ex>shooting</ex> star</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>There <qex>shot</qex> a streaming lamp along the sky.</q> <rj><qau>Dryden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>To penetrate, as a missile; to dart with a piercing sensation; <as>as, <ex>shooting</ex> pains</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Thy words <qex>shoot</qex> through my heart.</q> <rj><qau>Addison.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <def>To feel a quick, darting pain; to throb in pain.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>These preachers make<br/
His head to <qex>shoot</qex> and ache.</q> <rj><qau>Herbert.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>6.</sn> <def>To germinate; to bud; to sprout.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Onions, as they hang, will <qex>shoot</qex> forth.</q> <rj><qau>Bacon.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>But the wild olive <qex>shoots</qex>, and shades the ungrateful plain.</q> <rj><qau>Dryden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>7.</sn> <def>To grow; to advance; <as>as, to <ex>shoot</ex> up rapidly</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Well <qex>shot</qex> in years he seemed.</q> <rj><qau>Spenser.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Delightful task! to rear the tender thought,<br/
To teach the young idea how to <qex>shoot</qex>.</q> <rj><qau>Thomson.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>8.</sn> <def>To change form suddenly; especially, to solidify.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>If the menstruum be overcharged, metals will <qex>shoot</qex> into crystals.</q> <rj><qau>Bacon.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>9.</sn> <def>To protrude; to jut; to project; to extend; <as>as, the land <ex>shoots</ex> into a promontory</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>There <qex>shot</qex> up against the dark sky, tall, gaunt, straggling houses.</q> <rj><qau>Dickens.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>10.</sn> <fld>(Naut.)</fld> <def>To move ahead by force of momentum, as a sailing vessel when the helm is put hard alee.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>To shoot ahead</b></col>, <cd>to pass or move quickly forward; to outstrip others.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shoot</ent><br/
<hw>Shoot</hw>, <pos>n.</pos> <sn>1.</sn> <def>The act of shooting; the discharge of a missile; a shot; <as>as, the <ex>shoot</ex> of a shuttle</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The Turkish bow giveth a very forcible <qex>shoot</qex>.</q> <rj><qau>Bacon.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>One underneath his horse to get a <qex>shoot</qex> doth stalk.</q> <rj><qau>Drayton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A young branch or growth.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Superfluous branches and <qex>shoots</qex> of this second spring.</q> <rj><qau>Evelyn.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>A rush of water; a rapid.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <fld>(Min.)</fld> <def>A vein of ore running in the same general direction as the lode.</def>  <rj><au>Knight.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <fld>(Weaving)</fld> <def>A weft thread shot through the shed by the shuttle; a pick.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>6.</sn> <ety>[Perh. a different word.]</ety> <def>A shoat; a young hog.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shooter</ent><br/
<hw>Shoot"er</hw> <pr>(?)</pr>, <pos>n.</pos> <sn>1.</sn> <def>One who shoots, as an archer or a gunner.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>That which shoots.</def> Specifically: <sd>(a)</sd> <def>A firearm; <as>as, a five-<ex>shooter</ex></as>.</def> <mark>[Colloq. U.S.]</mark> <sd>(b)</sd> <def>A shooting star.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shooting</ent><br/
<hw>Shoot"ing</hw>, <pos>n.</pos> <sn>1.</sn> <def>The act of one who, or that which, shoots; <as>as, the <ex>shooting</ex> of an archery club; the <ex>shooting</ex> of rays of light.</as></def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A wounding or killing with a firearm; specifically <fld>(Sporting)</fld>, the killing of game; <as>as, a week of <ex>shooting</ex></as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>A sensation of darting pain; <as>as, a <ex>shooting</ex> in one's head</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shooting</ent><br/
<hw>Shoot"ing</hw>, <pos>a.</pos> <def>Of or pertaining to shooting; for shooting; darting.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Shooting board</b></col> <fld>(Joinery)</fld>, <cd>a fixture used in planing or shooting the edge of a board, by means of which the plane is guided and the board held true.</cd> -- <col><b>Shooting box</b></col>, <cd>a small house in the country for use in the shooting season.</cd> <au>Prof. Wilson.</au> -- <col><b>Shooting gallery</b></col>, <cd>a range, usually covered, with targets for practice with firearms.</cd><-- [Slang] a place, often a building or neighborhood, where addicts "shoot up" drugs.  -->  -- <col><b>Shooting iron</b></col>, <cd>a firearm.</cd> <mark>[Slang, U.S.]</mark> -- <col><b>Shooting star</b></col>. <sd>(a)</sd> <fld>(Astron.)</fld> <cd>A starlike, luminous meteor, that, appearing suddenly, darts quickly across some portion of the sky, and then as suddenly disappears, leaving sometimes, for a few seconds, a luminous train, -- called also <altname>falling star</altname>.</cd> <note><xex>Shooting stars</xex> are small cosmical bodies which encounter the earth in its annual revolution, and which become visible by coming with planetary velocity into the upper regions of the atmosphere. At certain periods, as on the 13th of November and 10th of August, they appear for a few hours in great numbers, apparently diverging from some point in the heavens, such displays being known as <xex>meteoric showers</xex>, or <xex>star showers</xex>. These bodies, before encountering the earth, were moving in orbits closely allied to the orbits of comets. See <er>Leonids</er>, <er>Perseids</er>.</note> <sd>(b)</sd> <fld>(Bot.)</fld> <cd>The American cowslip (<spn>Dodecatheon Meadia</spn>). See under <er>Cowslip</er>.</cd> -- <col><b>Shooting stick</b></col> <fld>(Print.)</fld>, <cd>a tapering piece of wood or iron, used by printers to drive up the quoins in the chase.</cd> <au>Hansard.</au></cs><br/
[<source>1913 Webster</source>]</p>

<p><-- p. 1332 --></p>

<p><ent>Shooty</ent><br/
<hw>Shoot"y</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Sprouting or coming up freely and regularly.</def> <mark>[Prev. Eng.]</mark>  <rj><au>Grose.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shop</ent><br/
<hw>Shop</hw> <pr>(?)</pr>, <mark>obs.</mark> <def><pos>imp.</pos> of <er>Shape</er>. Shaped.</def>  <rj><au>Chaucer.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shop</ent><br/
<hw>Shop</hw>, <pos>n.</pos> <ety>[OE. <ets>shoppe</ets>, <ets>schoppe</ets>, AS. <ets>sceoppa</ets> a treasury, a storehouse, stall, booth; akin to <ets>scypen</ets> a shed, LG. <ets>schup</ets> a shed, G. <ets>schoppen</ets>, <ets>schuppen</ets>, a shed, a coachhouse, OHG. <ets>scopf</ets>.]</ety> <sn>1.</sn> <def>A building or an apartment in which goods, wares, drugs, etc., are sold by retail.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>From <qex>shop</qex> to <qex>shop</qex><br/
Wandering, and littering with unfolded silks<br/
The polished counter.</q> <rj><qau>Cowper.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A building in which mechanics or artisans work; <as>as, a shoe <ex>shop</ex>; a car <ex>shop</ex>.</as></def><br/
[<source>1913 Webster</source>]</p>

<p><q>A tailor called me in his <qex>shop</qex>.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>A person's occupation, business, profession, or the like, as a subject of attention, interest, conversation, etc.; -- sometimes in deprecation or disapproval; <as>as, to talk <ex>shop</ex> at a party</as>.  Also used attributively, as in <ex>shop talk</ex>.</def><br/
[<source>Webster 1913 Suppl.</source> <source>+PJC</source>]</p>

<p><sn>4.</sn>  <def>A place where any industry is carried on; <as>as, a chemist's <ex>shop</ex></as>;</def> also, <fld>(Slang)</fld>, <def>any of the various places of business which are commonly called offices, as of a lawyer, doctor, broker, etc.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><sn>5.</sn>  <def>Any place of resort, as one's house, a restaurant, etc.</def> <mark>[Slang, Chiefly Eng.]</mark></p>

<p><sn>6.</sn>  <def>the group of workers and the activities controlled by an administrator; <as>as, to have five people in one's <ex>shop</ex></as>.</def> <mark>[Colloq.]</mark></p>

<p><note><hand/ <xex>Shop</xex> is often used adjectively or in composition; as, <xex>shop</xex> rent, or <xex>shop</xex>-rent; <xex>shop thief</xex>, <xex>or shop</xex>-thief; <xex>shop</xex> window, or <xex>shop</xex>-window, etc.</note><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>To smell of the shop</b></col>, <cd>to indicate too distinctively one's occupation or profession.</cd> -- <col><b>To talk shop</b></col>, <cd>to make one's business the topic of social conversation; also, to use the phrases peculiar to one's employment.</cd> <mark>[Colloq.]</mark></cs><br/
[<source>1913 Webster</source>]</p>

<p><syn><b>Syn.</b> -- Store; warehouse. See <er>Store</er>.</syn><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shop</ent><br/
<hw>Shop</hw>, <pos>v. i.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Shopped</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Shopping</conjf>.]</vmorph> <def>To visit shops for the purpose of purchasing goods.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>He was engaged with his mother and some ladies to go <qex>shopping</qex>.</q> <rj><qau>Byron.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shopboard</ent><br/
<hw>Shop"board`</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A bench or board on which work is performed; a workbench.</def>  <rj><au>South.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shopbook</ent><br/
<hw>Shop"book`</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A book in which a tradesman keeps his accounts.</def>  <rj><au>Locke.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shopboy</ent><br/
<hw>Shop"boy`</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A boy employed in a shop.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shopen</ent><br/
<hw>Sho"pen</hw> <pr>(?)</pr>, <mark>obs.</mark> <def><pos>p. p.</pos> of <er>Shape</er>.</def>  <rj><au>Chaucer.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shopgirl</ent><br/
<hw>Shop"girl`</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A girl employed in a shop.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shopkeeper</ent><br/
<hw>Shop"keep`er</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A trader who sells goods in a shop, or by retail; -- in distinction from one who sells by wholesale.</def>  <rj><au>Addison.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shoplifter</ent><br/
<hw>Shop"lift`er</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[<ets>Shop</ets> + <ets>lift.</ets> See <er>Lift</er> to steal.]</ety> <def>One who steals anything in a shop, or takes goods privately from a shop; one who, under pretense of buying goods, takes occasion to steal.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shoplifting</ent><br/
<hw>Shop"lift`ing</hw>, <pos>n.</pos> <def>Larceny committed in a shop; the stealing of anything from a shop.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shoplike</ent><br/
<hw>Shop"like`</hw>, <pos>a.</pos> <def>Suiting a shop; vulgar.</def>  <rj><au>B. Jonson.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shopmaid</ent><br/
<hw>Shop"maid`</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A shopgirl.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shopman</ent><br/
<hw>Shop"man</hw> <pr>(?)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Shopmen</plw> <pr>(<?/)</pr>.</plu> <sn>1.</sn> <def>A shopkeeper; a retailer.</def>  <rj><au>Dryden.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>One who serves in a shop; a salesman.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>One who works in a shop or a factory.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shopper</ent><br/
<hw>Shop"per</hw> <pr>(?)</pr>, <pos>n.</pos> <def>One who shops.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shoppish</ent><br/
<hw>Shop"pish</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Having the appearance or qualities of a shopkeeper, or shopman.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shoppy</ent><br/
<hw>Shop"py</hw> <pr>(?)</pr>, <pos>a.</pos> <sn>1.</sn> <def>Abounding with shops.</def> <mark>[Colloq.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Of or pertaining to shops, or one's own shop or business; <as>as, <ex>shoppy</ex> talk</as>.</def> <mark>[Colloq.]</mark>  <rj><au>Mrs. Gaskell.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shopshift</ent><br/
<hw>Shop"shift`</hw> <pr>(?)</pr>, <pos>n.</pos> <def>The trick of a shopkeeper; deception.</def> <mark>[Obs.]</mark>  <rj><au>B. Jonson.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>shop talk</ent><br/
<hw>shop talk</hw>, <def>conversation about a matters pertaining to person's occupation; -- often used of such discussions when conducted outside of working hours, as in social gatherings.</def><br/
[<source>PJC</source>]</p>

<p><ent>Shopwalker</ent><br/
<hw>Shop"walk`er</hw> <pr>(?)</pr>, <pos>n.</pos> <def>One who walks about in a shop as an overseer and director.  Cf. <er>Floorwalker</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shopwoman</ent><br/
<hw>Shop"wom`an</hw> <pr>(?)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Shopwomen</plw> <pr>(<?/)</pr>.</plu> <def>A woman employed in a shop.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shopworn</ent><br/
<hw>Shop"worn`</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Somewhat worn or damaged by having been kept for a time in a shop.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shorage</ent><br/
<hw>Shor"age</hw> <pr>(?)</pr>, <pos>n.</pos> <def>Duty paid for goods brought on shore.</def>  <rj><au>Grabb.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shore</ent><br/
<hw>Shore</hw> <pr>(?)</pr>, <def><pos>imp.</pos> of <er>Shear</er>.</def>  <rj><au>Chaucer.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shore</ent><br/
<hw>Shore</hw>, <pos>n.</pos> <def>A sewer.</def> <mark>[Obs. or Prov. Eng.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shore</ent><br/
<hw>Shore</hw>, <pos>n.</pos> <ety>[OE. <ets>schore</ets>; akin to LG. <ets>schore</ets>, D. <ets>schoor</ets>, OD. <ets>schoore</ets>, Icel. <ets>skor<?/a</ets>, and perhaps to E. <ets>shear</ets>, as being a piece cut off.]</ety> <def>A prop, as a timber, placed as a brace or support against the side of a building or other structure; a prop placed beneath anything, as a beam, to prevent it from sinking or sagging.</def> <altsp>[Written also <asp>shoar</asp>.]</altsp><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shore</ent><br/
<hw>Shore</hw>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Shored</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Shoring</conjf>.]</vmorph> <ety>[OE. <ets>schoren</ets>. See <er>Shore</er> a prop.]</ety> <def>To support by a shore or shores; to prop; -- usually with <xex>up</xex>; <as>as, to <ex>shore</ex> up a building</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shore</ent><br/
<hw>Shore</hw>, <pos>n.</pos> <ety>[OE. <ets>schore</ets>, AS. <ets>score</ets>, probably fr. <ets>scieran</ets>, and so meaning properly, that which is shorn off, edge; akin to OD. <ets>schoore</ets>, <ets>schoor</ets>. See <er>Shear</er>, <pos>v. t.</pos>]</ety> <def>The coast or land adjacent to a large body of water, as an ocean, lake, or large river.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Michael Cassio,<br/
Lieutenant to the warlike Moor Othello,<br/
Is come <qex>shore</qex>.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>The fruitful <qex>shore</qex> of muddy Nile.</q> <rj><qau>Spenser.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>In shore</b></col>, <cd>near the shore.</cd> <au>Marryat.</au> -- <col><b>On shore</b></col>. <cd>See under <er>On</er>.</cd> -- <col><b>Shore birds</b></col> <fld>(Zool.)</fld>, <cd>a collective name for the various limicoline birds found on the seashore.</cd> -- <col><b>Shore crab</b></col> <fld>(Zool.)</fld>, <cd>any crab found on the beaches, or between tides, especially any one of various species of grapsoid crabs, as <spn>Heterograpsus nudus</spn> of California.</cd> -- <col><b>Shore lark</b></col> <fld>(Zool.)</fld>, <cd>a small American lark (<spn>Otocoris alpestris</spn>) found in winter, both on the seacoast and on the Western plains. Its upper parts are varied with dark brown and light brown. It has a yellow throat, yellow local streaks, a black crescent on its breast, a black streak below each eye, and two small black erectile ear tufts. Called also <altname>horned lark</altname>.</cd> -- <col><b>Shore plover</b></col> <fld>(Zool.)</fld>, <cd>a large-billed Australian plover (<spn>Esacus magnirostris</spn>). It lives on the seashore, and feeds on crustaceans, etc.</cd> -- <col><b>Shore teetan</b></col> <fld>(Zool.)</fld>, <cd>the rock pipit (<spn>Anthus obscurus</spn>).</cd> <mark>[Prov. Eng.]</mark></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shore</ent><br/
<hw>Shore</hw> <pr>(?)</pr>, <pos>v. t.</pos> <def>To set on shore.</def> <mark>[Obs.]</mark>  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shoreless</ent><br/
<hw>Shore"less</hw>, <pos>a.</pos> <def>Having no shore or coast; of indefinite or unlimited extent; <as>as, a <ex>shoreless</ex> ocean</as>.</def>  <rj><au>Young.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shoreling</ent><br/
<hw>Shore"ling</hw> <pr>(?)</pr>, <pos>n.</pos> <def>See <er>Shorling</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shorer</ent><br/
<hw>Shor"er</hw> <pr>(?)</pr>, <pos>n.</pos> <def>One who, or that which, shores or props; a prop; a shore.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shoreward</ent><br/
<hw>Shore"ward</hw> <pr>(?)</pr>, <pos>adv.</pos> <def>Toward the shore.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shoring</ent><br/
<hw>Shor"ing</hw>, <pos>n.</pos> <sn>1.</sn> <def>The act of supporting or strengthening with a prop or shore.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A system of props; props, collectively.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shorlaceous</ent><br/
<ent>Shorl</ent><br/
<mhw><hw>Shorl</hw> <pr>(?)</pr>, <pos>n.</pos>, <hw>Shor*la"ceous</hw> <pr>(<?/)</pr>, <pos>a.</pos></mhw> <fld>(Min.)</fld> <def>See <er>Schorl</er>, <er>Schorlaceous</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shorling</ent><br/
<hw>Shor"ling</hw> <pr>(?)</pr>, <pos>n.</pos> <sn>1.</sn> <def>The skin of a sheen after the fleece is shorn off, as distinct from the <xex>morling</xex>, or skin taken from the dead sheep; also, a sheep of the first year's shearing.</def> <mark>[Prov. Eng.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A person who is shorn; a shaveling; hence, in contempt, a priest.</def> <mark>[Obs.]</mark>  <rj><au>Halliwell.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shorn</ent><br/
<hw>Shorn</hw> <pr>(<?/)</pr>, <def><pos>p. p.</pos> of <er>Shear</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Short</ent><br/
<hw>Short</hw> <pr>(?)</pr>, <pos>a.</pos> <amorph>[<pos>Compar.</pos> <adjf>Shorter</adjf> <pr>(?)</pr>; <pos>superl.</pos> <adjf>Shortest</adjf>.]</amorph> <ety>[OE. <ets>short</ets>, <ets>schort</ets>, AS. <ets>scort</ets>, <ets>sceort</ets>; akin to OHG. <ets>scurz</ets>, Icel. <ets>skorta</ets> to be short of, to lack, and perhaps to E. <ets>shear</ets>, v. t.  Cf. <er>Shirt</er>.]</ety> <sn>1.</sn> <def>Not long; having brief length or linear extension; <as>as, a <ex>short</ex> distance; a <ex>short</ex> piece of timber; a <ex>short</ex> flight.</as></def><br/
[<source>1913 Webster</source>]</p>

<p><q>The bed is <qex>shorter</qex> than that a man can stretch himself on it.</q> <rj><qau>Isa. xxviii. 20.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Not extended in time; having very limited duration; not protracted; <as>as, <ex>short</ex> breath</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The life so <qex>short</qex>, the craft so long to learn.</q> <rj><qau>Chaucer.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>To <qex>short</qex> absense I could yield.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>Limited in quantity; inadequate; insufficient; scanty; <as>as, a <ex>short</ex> supply of provisions, or of water</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>Insufficiently provided; inadequately supplied; scantily furnished; lacking; not coming up to a resonable, or the ordinary, standard; -- usually with <xex>of</xex>; <as>as, to be <ex>short</ex> of money</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>We shall be <qex>short</qex> in our provision.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <def>Deficient; defective; imperfect; not coming up, as to a measure or standard; <as>as, an account which is <ex>short</ex> of the trith</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>6.</sn> <def>Not distant in time; near at hand.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Marinell was sore offended<br/
That his departure thence should be so <qex>short</qex>.</q> <rj><qau>Spenser.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>He commanded those who were appointed to attend him to be ready by a <qex>short</qex> day.</q> <rj><qau>Clarendon.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>7.</sn> <def>Limited in intellectual power or grasp; not comprehensive; narrow; not tenacious, as memory.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Their own <qex>short</qex> understandings reach<br/
No farther than the present.</q> <rj><qau>Rowe.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>8.</sn> <def>Less important, efficaceous, or powerful; not equal or equivalent; less (than); -- with <xex>of</xex>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Hardly anything <qex>short</qex> of an invasion could rouse them again to war.</q> <rj><qau>Landor.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>9.</sn> <def>Abrupt; brief; pointed; petulant; <as>as, he gave a <ex>short</ex> answer to the question</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>10.</sn> <fld>(Cookery)</fld> <def>Breaking or crumbling readily in the mouth; crisp; <as>as, <ex>short</ex> pastry</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>11.</sn> <fld>(Metal)</fld> <def>Brittle.</def><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ Metals that are brittle when hot are called <xex><?/ot-short</xex>; as, cast iron may be <xex>hot-short</xex>, owing to the presence of sulphur. Those that are brittle when cold are called <xex>cold-short</xex>; as, cast iron may be <xex>cold-short</xex>, on account of the presence of phosphorus.</note><br/
[<source>1913 Webster</source>]</p>

<p><sn>12.</sn> <fld>(Stock Exchange)</fld> <def>Engaging or engaged to deliver what is not possessed; <as>as, <ex>short</ex> contracts; to be <ex>short</ex> of stock. See <ex>The shorts</ex>, under <er>Short</er>, <pos>n.</pos>, and <ex>To sell short</ex>, under <er>Short</er>, <pos>adv.</pos></as></def><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ In mercantile transactions, a note or bill is sometimes made payable <xex>at short sight</xex>, that is, in a little time after being presented to the payer.</note><br/
[<source>1913 Webster</source>]</p>

<p><sn>13.</sn> <fld>(Phon.)</fld> <def>Not prolonged, or relatively less prolonged, in utterance; -- opposed to <ant>long</ant>, and applied to vowels or to syllables. In English, the long and short of the same letter are not, in most cases, the long and short of the same sound; thus, the <xex>i</xex> in <xex>ill</xex> is the short sound, not of <xex>i</xex> in <xex>isle</xex>, but of <xex>ee</xex> in <xex>eel</xex>, and the <xex>e</xex> in <xex>pet</xex> is the short sound of <xex>a</xex> in <xex>pate</xex>, etc. See <er>Quantity</er>, and <xex>Guide to Pronunciation</xex>, <sect/<sect/22, 30.</def><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ <xex>Short</xex> is much used with participles to form numerous self-explaining compounds; as, <xex>short</xex>-armed, <xex>short</xex>-billed, <xex>short</xex>-fingered, <xex>short</xex>-haired, <xex>short</xex>-necked, <xex>short</xex>-sleeved, <xex>short</xex>-tailed, <xex>short</xex>-winged, <xex>short</xex>-wooled, etc.</note><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>At short notice</b></col>, <cd>in a brief time; promptly.</cd> -- <col><b>Short rib</b></col> <fld>(Anat.)</fld>, <cd>one of the false ribs.</cd> -- <col><b>Short suit</b></col> <fld>(Whist)</fld>, <cd>any suit having only three cards, or less than three.</cd> <au>R. A. Proctor.</au> -- <mcol><col><b>To come short</b></col>, <col><b>To cut short</b></col>, <col><b>To fall short</b></col></mcol>, <cd>etc. See under <er>Come</er>, <er>Cut</er>, etc.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Short</ent><br/
<hw>Short</hw>, <pos>n.</pos> <sn>1.</sn> <def>A summary account.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The <qex>short</qex> and the long is, our play is preferred.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <pluf>pl.</pluf> <def>The part of milled grain sifted out which is next finer than the bran.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The first remove above bran is <qex>shorts</qex>.</q> <rj><qau>Halliwell.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <pluf>pl.</pluf> <def>Short, inferior hemp.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <pluf>pl.</pluf> <def>Breeches; shortclothes.</def> <mark>[Slang]</mark>  <rj><au>Dickens.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <fld>(Phonetics)</fld> <def>A short sound, syllable, or vowel.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>If we compare the nearest conventional <qex>shorts</qex> and longs in English, as in <ldquo/bit<rdquo/ and <ldquo/beat,<rdquo/ <ldquo/not<rdquo/ and <ldquo/naught,<rdquo/ we find that the short vowels are generally wide, the long narrow, besides being generally diphthongic as well. Hence, originally short vowels can be lengthened and yet kept quite distinct from the original longs.</q> <rj><qau>H. Sweet.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>In short</b></col>, <cd>in few words; in brief; briefly.</cd> -- <col><b>The long and the short</b></col>, <cd>the whole; a brief summing up.</cd> -- <col><b>The shorts</b></col> <fld>(Stock Exchange)</fld>, <cd>those who are unsupplied with stocks which they contracted to deliver.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Short</ent><br/
<hw>Short</hw> <pr>(?)</pr>, <pos>adv.</pos> <def>In a short manner; briefly; limitedly; abruptly; quickly; <as>as, to stop <ex>short</ex> in one's course; to turn <ex>short</ex>.</as></def><br/
[<source>1913 Webster</source>]</p>

<p><q>He was taken up very <qex>short</qex>, and adjudged corrigible for such presumptuous language.</q> <rj><qau>Howell.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>To sell short</b></col> <fld>(Stock Exchange)</fld>, <cd>to sell, for future delivery, what the party selling does not own, but hopes to buy at a lower rate.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Short</ent><br/
<hw>Short</hw>, <pos>v. t.</pos> <ety>[AS. <ets>sceortian</ets>.]</ety> <def>To shorten.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Short</ent><br/
<hw>Short</hw>, <pos>v. i.</pos> <def>To fail; to decrease.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shortage</ent><br/
<hw>Short"age</hw> <pr>(?)</pr>, <pos>n.</pos> <def>Amount or extent of deficiency, as determined by some requirement or standard; <as>as, a <ex>shortage</ex> in money accounts</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Short-breathed</ent><br/
<hw>Short"-breathed`</hw> <pr>(?)</pr>, <pos>a.</pos> <sn>1.</sn> <def>Having short-breath, or quick respiration.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Having short life.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shortcake</ent><br/
<hw>Short"cake`</hw> <pr>(?)</pr>, <pos>n.</pos> <def>An unsweetened breakfast cake shortened with butter or lard, rolled thin, and baked.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Short circuit</ent><br/
<hw>Short" cir"cuit</hw> <pr>(?)</pr>. <fld>(Elec.)</fld> <def>A circuit formed or closed by a conductor of relatively low resistance because shorter or of relatively great conductivity.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Short-circuit</ent><br/
<hw>Short"-cir`cuit</hw>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Short-circuited</conjf>; <pos>p. pr. & vb. n.</pos> <conjf>Short-circuiting</conjf>.]</vmorph> <fld>(Elec.)</fld> <def>To join, as the electrodes of a battery or dynamo or any two points of a circuit, by a conductor of low resistance.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shortclothes</ent><br/
<hw>Short"clothes`</hw> <pr>(?)</pr>, <pos>n.</pos> <def>Coverings for the legs of men or boys, consisting of trousers which reach only to the knees, -- worn with long stockings.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shortcoming</ent><br/
<hw>Short"com`ing</hw> <pr>(?)</pr>, <pos>n.</pos> <def>The act of falling, or coming short</def>; as: <sd>(a)</sd> <def>The failure of a crop, or the like.</def> <sd>(b)</sd> <def>Neglect of, or failure in, performance of duty.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Short-dated</ent><br/
<hw>Short"-dat`ed</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Having little time to run from the date.</def> <ldquo/Thy <xex>short-dated</xex> life.<rdquo/  <rj><au>Sandys.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shorten</ent><br/
<hw>Short"en</hw> <pr>(?)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Shortened <?/</conjf>; <pos>p. pr. & vb. n.</pos> <conjf>Shortening</conjf>.]</vmorph> <ety>[See <er>Short</er>, <pos>a.</pos>]</ety> <sn>1.</sn> <def>To make short or shorter in measure, extent, or time; <as>as, to <ex>shorten</ex> distance; to <ex>shorten</ex> a road; to <ex>shorten</ex> days of calamity.</as></def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To reduce or diminish in amount, quantity, or extent; to lessen; to abridge; to curtail; to contract; <as>as, to <ex>shorten</ex> work, an allowance of food, etc.</as></def><br/
[<source>1913 Webster</source>]</p>

<p><q>Here, where the subject is so fruitful, I am <qex>shortened</qex> by my chain.</q> <rj><qau>Dryden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>To make deficient (as to); to deprive; -- with <xex>of</xex>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Spoiled of his nose, and <qex>shortened</qex> of his ears.</q> <rj><qau>Dryden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>To make short or friable, as pastry, with butter, lard, pot liquor, or the like.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>To shorten a rope</b></col> <fld>(Naut.)</fld>, <cd>to take in the slack of it.</cd> -- <col><b>To shorten sail</b></col> <fld>(Naut.)</fld>, <cd>to reduce sail by taking it in.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shorten</ent><br/
<hw>Short"en</hw>, <pos>v. i.</pos> <def>To become short or shorter; <as>as, the day <ex>shortens</ex> in northern latitudes from June to December; a metallic rod <ex>shortens</ex> by cold.</as></def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shortener</ent><br/
<hw>Short"en*er</hw> <pr>(?)</pr>, <pos>n.</pos> <def>One who, or that which, shortens.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shortening</ent><br/
<hw>Short"en*ing</hw>, <pos>n.</pos> <sn>1.</sn> <def>The act of making or becoming short or shorter.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Cookery)</fld> <def>That which renders pastry short or friable, as butter, lard, etc.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shorthand</ent><br/
<hw>Short"hand`</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A compendious and rapid method or writing by substituting characters, abbreviations, or symbols, for letters, words, etc.; short writing; stenography. See <xex>Illust.</xex> under <er>Phonography</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Short-handed</ent><br/
<hw>Short`-hand"ed</hw>, <pos>a.</pos> <def>Short of, or lacking the regular number of, servants or helpers.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shorthead</ent><br/
<hw>Short"head`</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A sucking whale less than one year old; -- so called by sailors.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shorthorn</ent><br/
<hw>Short"horn`</hw> <pr>(?)</pr>, <pos>a.</pos> <def>One of a breed of large, heavy domestic cattle having short horns. The breed was developed in England.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Short-jointed</ent><br/
<hw>Short"-joint`ed</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Having short intervals between the joints; -- said of a plant or an animal, especially of a horse whose pastern is too short.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Short-lived</ent><br/
<hw>Short"-lived`</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Not living or lasting long; being of short continuance; <as>as, a <ex>short-lived</ex> race of beings; <ex>short-lived</ex> pleasure; <ex>short-lived</ex> passion</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shortly</ent><br/
<hw>Short"ly</hw>, <pos>adv.</pos> <ety>[AS. <ets>sceortlice</ets>.]</ety> <sn>1.</sn> <def>In a short or brief time or manner; soon; quickly.</def>  <rj><au>Chaucer.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>I shall grow jealous of you <qex>shortly</qex>.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>The armies came <qex>shortly</qex> in view of each other.</q> <rj><qau>Clarendon.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>In few words; briefly; abruptly; curtly; <as>as, to express ideas more <ex>shortly</ex> in verse than in prose</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shortness</ent><br/
<hw>Short"ness</hw>, <pos>n.</pos> <def>The quality or state of being short; want of reach or extension; brevity; deficiency; <as>as, the <ex>shortness</ex> of a journey; the <ex>shortness</ex> of the days in winter; the <ex>shortness</ex> of an essay; the <ex>shortness</ex> of the memory; a <ex>shortness</ex> of provisions; <ex>shortness</ex> of breath</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shortsighted</ent><br/
<hw>Short"sight`ed</hw> <pr>(?)</pr>, <pos>a.</pos> <sn>1.</sn> <def>Not able to see far; nearsighted; myopic. See <er>Myopic</er>, and <er>Myopia</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Fig.: Not able to look far into futurity; unable to understand things deep; of limited intellect.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>Having little regard for the future; heedless.</def><br/
[<source>1913 Webster</source>]</p>

<p>-- <wordforms><wf>Short"sight`ed*ly</wf>, <pos>adv.</pos> -- <wf>Short"sight`ed*ness</wf>, <pos>n.</pos></wordforms><br/
[<source>1913 Webster</source>]</p>

<p><q>Cunning is a kind of <qex>shortsightedness</qex>.</q> <rj><qau>Addison.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Short-spoken</ent><br/
<hw>Short"-spo`ken</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Speaking in a quick or short manner; hence, gruff; curt.</def> <mark>[Colloq.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shortstop</ent><br/
<hw>Short"stop`</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Baseball)</fld> <def>The player stationed in the field bewtween the second and third bases.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Short-waisted</ent><br/
<hw>Short"-waist`ed</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Having a short waist.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Short-winded</ent><br/
<hw>Short"-wind`ed</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Affected with shortness of breath; having a quick, difficult respiration, as dyspnoic and asthmatic persons.</def>  <rj><au>May.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shortwing</ent><br/
<hw>Short"wing`</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Zool.)</fld> <def>Any one of several species of small wrenlike Asiatic birds having short wings and a short tail. They belong to <xex>Brachypterix</xex>, <xex>Callene</xex>, and allied genera.</def><br/
[<source>1913 Webster</source>]</p>

<p><-- p. 1333 --></p>

<p><ent>Short-witted</ent><br/
<hw>Short"-wit`ted</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Having little wit; not wise; having scanty intellect or judgment.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shory</ent><br/
<hw>Shor"y</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Lying near the shore.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shoshones</ent><br/
<hw>Sho*sho"nes</hw> <pr>(?)</pr>, <pos>n. pl.</pos>; <sing>sing. <singw>Shoshone</singw> <pr>(<?/)</pr></sing>. <fld>(Ethnol.)</fld> <def>A linguistic family or stock of North American Indians, comprising many tribes, which extends from Montana and Idaho into Mexico. In a restricted sense the name is applied especially to the Snakes, the most northern of the tribes.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shot</ent><br/
<hw>Shot</hw> <pr>(?)</pr>, <def><pos>imp. & p. p.</pos> of <er>Shoot</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shot</ent><br/
<hw>Shot</hw>, <pos>a.</pos> <def>Woven in such a way as to produce an effect of variegation, of changeable tints, or of being figured; <as>as, <ex>shot</ex> silks</as>. See <er>Shoot</er>, <pos>v. t.</pos>, 8.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shot</ent><br/
<hw>Shot</hw>, <pos>n.</pos> <ety>[AS. <ets>scot</ets>, <ets>sceot</ets>, fr. <ets>sce<oacute/tan</ets> to shoot; akin to D. <ets>sschot</ets>, Icel. <ets>skot</ets>. <root/159. See <er>Scot</er> a share, <er>Shoot</er>, <pos>v. t.</pos>, and cf. <er>Shot</er> a shooting.]</ety> <def>A share or proportion; a reckoning; a scot.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Here no <qex>shots</qex> are where all shares be.</q> <rj><qau>Chapman.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>A man is never . . . welcome to a place till some certain <qex>shot</qex> be paid and the hostess say <ldquo/Welcome.<rdquo/</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shot</ent><br/
<hw>Shot</hw>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Shot</plw>or <plw>Shots</plw> <pr>(#)</pr>.</plu> <ety>[OE. <ets>shot</ets>, <ets>schot</ets>, AS. <ets>gesceot</ets> a missile; akin to D. <ets>schot</ets> a shot, shoot, G. <ets>schuss</ets>, ge<ets>schoss</ets> a missile, Icel. <ets>skot</ets> a throwing, a javelin, and E. <ets>shoot</ets>, v.t. <root/159. See <er>Shoot</er>, and cf. <er>Shot</er> a share.]</ety> <sn>1.</sn> <def>The act of shooting; discharge of a firearm or other weapon which throws a missile.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>He caused twenty <qex>shot</qex> of his greatest cannon to be made at the king's army.</q> <rj><qau>Clarendon.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A missile weapon, particularly a ball or bullet; specifically, whatever is discharged as a projectile from firearms or cannon by the force of an explosive.</def><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ <xex>Shot</xex> used in war is of various kinds, classified according to the material of which it is composed, into <xex>lead</xex>, <xex>wrought-iron</xex>, and <xex>cast-iron</xex>; according to form, into <xex>spherical</xex> and <xex>oblong</xex>; according to structure and modes of operation, into <xex>solid</xex>, <xex>hollow</xex>, and <xex>case</xex>. See <cref>Bar shot</cref>, <cref>Chain shot</cref>, etc., under <er>Bar</er>, <er>Chain</er>, etc.</note><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>Small globular masses of lead, of various sizes, -- used chiefly as the projectiles in shotguns for killing game; <as>as, bird <ex>shot</ex>; buck<ex>shot</ex>.</as></def><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>The flight of a missile, or the distance which it is, or can be, thrown; <as>as, the vessel was distant more than a cannon <ex>shot</ex></as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <def>A marksman; one who practices shooting; <as>as, an exellent <ex>shot</ex></as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>6.</sn> <fld>(Fisheries)</fld> <sd>(a)</sd> <def>A cast of a net.</def> <sd>(b)</sd> <def>The entire throw of nets at one time.</def> <sd>(c)</sd> <def>A place or spot for setting nets.</def> <sd>(d)</sd> <def>A single draft or catch of fish made.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><sn>7.</sn>  <fld>(Athletics)</fld> <def>A spherical weight, to be put, or thrown, in competition for distance.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><sn>8.</sn>  <def>A stroke, throw, or other action to propel a ball or other game piece in certain games, as in billiards, hockey, basketball, curling, etc.; also, a move, as in chess.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><sn>9.</sn>  <def>A guess; conjecture; also, an attempt.</def> <mark>[Colloq.]</mark> <ldquo/I'll take a <ex>shot</ex> at it.<rdquo/<br/
[<source>Webster 1913 Suppl.</source> <source>+PJC</source>]</p>

<p><cs><col><b>Shot belt</b></col>, <cd>a belt having a pouch or compartment for carrying shot.</cd> -- <col><b>Shot cartridge</b></col>, <cd>a cartridge containing powder and small shot, forming a charge for a shotgun.</cd> -- <col><b>Shot garland</b></col> <fld>(Naut.)</fld>, <cd>a wooden frame to contain shot, secured to the coamings and ledges round the hatchways of a ship.</cd> -- <col><b>Shot gauge</b></col>, <cd>an instrument for measuring the diameter of round shot.</cd> <au>Totten.</au> -- <col><b>shot hole</b></col>, <cd>a hole made by a shot or bullet discharged.</cd> -- <col><b>Shot locker</b></col> <fld>(Naut.)</fld>, <cd>a strongly framed compartment in the hold of a vessel, for containing shot.</cd> -- <col><b>Shot of a cable</b></col> <fld>(Naut.)</fld>, <cd>the splicing of two or more cables together, or the whole length of the cables thus united.</cd> -- <col><b>Shot prop</b></col> <fld>(Naut.)</fld>, <cd>a wooden prop covered with tarred hemp, to stop a hole made by the shot of an enemy in a ship's side.</cd> -- <col><b>Shot tower</b></col>, <cd>a lofty tower for making shot, by dropping from its summit melted lead in slender streams. The lead forms spherical drops which cool in the descent, and are received in water or other liquid.</cd> -- <col><b>Shot window</b></col>, <cd>a window projecting from the wall. Ritson, quoted by Halliwell, explains it as a window that opens and shuts; and Wodrow describes it as a window of shutters made of timber and a few inches of glass above them.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shot</ent><br/
<hw>Shot</hw>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Shotted</conjf>; <pos>p. pr. & vb. n.</pos> <conjf>Shotting</conjf>.]</vmorph> <def>To load with shot, as a gun.</def>  <rj><au>Totten.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shot-clog</ent><br/
<hw>Shot"-clog`</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A person tolerated only because he pays the shot, or reckoning, for the rest of the company, otherwise a mere clog on them.</def> <mark>[Old Slang]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>Thou common <qex>shot-clog</qex>, gull of all companies.</q> <rj><qau>Chapman.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shote</ent><br/
<hw>Shote</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[AS. <ets>sce<oacute/ta</ets> a darting fish, a trout, fr. <ets>sce<oacute/tan</ets>. See <er>Shoot</er>, <pos>v. t.</pos>]</ety> <sn>1.</sn> <fld>(Zool.)</fld> <def>A fish resembling the trout.</def> <mark>[Obs. or Prov. Eng.]</mark>  <rj><au>Garew.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <ety>[Perh. a different word.]</ety> <def>A young hog; a shoat.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shot-free</ent><br/
<hw>Shot"-free`</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Not to be injured by shot; shot-proof.</def> <mark>[Obs.]</mark>  <rj><au>Feltham.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shot-free</ent><br/
<hw>Shot"-free`</hw>, <pos>a.</pos> <def>Free from charge or expense; hence, unpunished; scot-free.</def> <mark>[Obs.]</mark>  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shotgun</ent><br/
<hw>Shot"gun`</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A light, smooth-bored gun, often double-barreled, especially designed for firing small shot at short range, and killing small game.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shot-proof</ent><br/
<hw>Shot"-proof`</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Impenetrable by shot.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shots</ent><br/
<hw>Shots</hw> <pr>(?)</pr>, <pos>n. pl.</pos> <def>The refuse of cattle taken from a drove.</def> <mark>[Prov. Eng.]</mark>  <rj><au>Halliwell.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shot samples</ent><br/
<hw>Shot samples</hw>. <fld>(Metal.)</fld> <def>Samples taken for assay from  a molten metallic mass pouring a portion into water, to granulate it.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Shotted</ent><br/
<hw>Shot"ted</hw> <pr>(?)</pr>, <pos>a.</pos> <sn>1.</sn> <def>Loaded with shot.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Med.)</fld> <def>Having a shot attached; <as>as, a <ex>shotten</ex> suture</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shotten</ent><br/
<hw>Shot"ten</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Properly p. p. of <ets>shoot</ets>; AS. <ets>scoten</ets>, <ets>sceoten</ets>, p. p. of <ets>sce<oacute/tan</ets>.]</ety> <sn>1.</sn> <def>Having ejected the spawn; <as>as, a <ex>shotten</ex> herring</as>.</def>  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Shot out of its socket; dislocated, as a bone.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shough</ent><br/
<hw>Shough</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Zool.)</fld> <def>A shockdog.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shough</ent><br/
<hw>Shough</hw> <pr>(?)</pr>, <pos>interj.</pos> <def>See <er>Shoo</er>.</def>  <rj><au>Beau. & Fl.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Should</ent><br/
<hw>Should</hw> <pr>(sh<oocr/d)</pr>, <pos>imp.</pos> <mord>of <er>Shall</er></mord>. <ety>[OE. <ets>sholde</ets>, <ets>shulde</ets>, <ets>scholde</ets>, <ets>schulde</ets>, AS. <ets>scolde</ets>, <ets>sceolde</ets>. See <er>Shall</er>.]</ety> <def>Used as an auxiliary verb, to express a conditional or contingent act or state, or as a supposition of an actual fact; also, to express moral obligation (see <er>Shall</er>); <it>e. g.</it>: they <xex>should</xex> have come last week; if I <xex>should</xex> go; I <xex>should</xex> think you could go.</def> <ldquo/You have done that you <xex>should</xex> be sorry for.<rdquo/  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><syn><b>Syn.</b> -- See <er>Ought</er>.</syn><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shoulder</ent><br/
<hw>Shoul"der</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[OE. <ets>shulder</ets>, <ets>shuldre</ets>, <ets>schutder</ets>, AS. <ets>sculdor</ets>; akin to D. <ets>schoulder</ets>, G. <ets>schulter</ets>, OHG. <ets>scultarra</ets>, Dan. <ets>skulder</ets>, Sw. <ets>skuldra</ets>.]</ety> <sn>1.</sn> <fld>(Anat.)</fld> <def>The joint, or the region of the joint, by which the fore limb is connected with the body or with the shoulder girdle; the projection formed by the bones and muscles about that joint.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>The flesh and muscles connected with the shoulder joint; the upper part of the back; that part of the human frame on which it is most easy to carry a heavy burden; -- often used in the plural.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Then by main force pulled up, and on his <qex>shoulders</qex> bore<br/
The gates of Azza.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Adown her <qex>shoulders</qex> fell her length of hair.</q> <rj><qau>Dryden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>Fig.: That which supports or sustains; support.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>In thy <qex>shoulder</qex> do I build my seat.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>That which resembles a human shoulder, as any protuberance or projection from the body of a thing.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The north western <qex>shoulder</qex> of the mountain.</q> <rj><qau>Sir W. Scott.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <def>The upper joint of the fore leg and adjacent parts of an animal, dressed for market; <as>as, a <ex>shoulder</ex> of mutton</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>6.</sn> <fld>(Fort.)</fld> <def>The angle of a bastion included between the face and flank. See <xex>Illust.</xex> of <er>Bastion</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>7.</sn> <def>An abrupt projection which forms an abutment on an object, or limits motion, etc., as the projection around a tenon at the end of a piece of timber, the part of the top of a type which projects beyond the base of the raised character, etc.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Shoulder belt</b></col>, <cd>a belt that passes across the shoulder.</cd> -- <col><b>Shoulder blade</b></col> <fld>(Anat.)</fld>, <cd>the flat bone of the shoulder, to which the humerus is articulated; the scapula.</cd> -- <col><b>Shoulder block</b></col> <fld>(Naut.)</fld>, <cd>a block with a projection, or shoulder, near the upper end, so that it can rest against a spar without jamming the rope.</cd> -- <col><b>Shoulder clapper</b></col>, <cd>one who claps another on the shoulder, or who uses great familiarity.</cd> <mark>[Obs.]</mark> <au>Shak.</au> -- <col><b>Shoulder girdle</b></col>. <fld>(Anat.)</fld> <cd>See <cref>Pectoral girdle</cref>, under <er>Pectoral</er>.</cd> -- <col><b>Shoulder knot</b></col>, <cd>an ornamental knot of ribbon or lace worn on the shoulder; a kind of epaulet or braided ornament worn as part of a military uniform.</cd> -- <col><b>Shoulder-of-mutton sail</b></col> <fld>(Naut.)</fld>, <cd>a triangular sail carried on a boat's mast; -- so called from its shape.</cd> -- <col><b>Shoulder slip</b></col>, <cd>dislocation of the shoulder, or of the humerous.</cd> <au>Swift.</au> -- <col><b>Shoulder strap</b></col>, <cd>a strap worn on or over the shoulder. Specifically <fld>(Mil. & Naval)</fld>, a narrow strap worn on the shoulder of a commissioned officer, indicating, by a suitable device, the rank he holds in the service. See <xex>Illust.</xex> in App.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shoulder</ent><br/
<hw>Shoul"der</hw> <pr>(?)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Shouldered</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Shouldering</conjf>.]</vmorph> <sn>1.</sn> <def>To push or thrust with the shoulder; to push with violence; to jostle.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>As they the earth would <qex>shoulder</qex> from her seat.</q> <rj><qau>Spenser.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Around her numberless the rabble flowed,<br/
<qex>Shouldering</qex> each other, crowding for a view.</q> <rj><qau>Rowe.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To take upon the shoulder or shoulders; <as>as, to <ex>shoulder</ex> a basket</as>; hence, to assume the burden or responsibility of; <as>as, to <ex>shoulder</ex> blame; to <ex>shoulder</ex> a debt.</as></def><br/
[<source>1913 Webster</source>]</p>

<p><q>As if Hercules<br/
Or burly Atlas <qex>shouldered</qex> up their state.</q> <rj><qau>Marston.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Right shoulder arms</b></col> <fld>(Mil.)</fld>, <cd>a position in the Manual of Arms which the piece is placed on the right shoulder, with the lock plate up, and the muzzle elevated and inclined to the left, and held as in the illustration.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><-- illustr. of a soldier at right soulder arms. --></p>

<p><ent>Shoulder</ent><br/
<hw>Shoul"der</hw>, <pos>v. i.</pos> <def>To push with the shoulder; to make one's way, as through a crowd, by using the shoulders; to move swaying the shoulders from side to side.</def></p>

<p><q>A yoke of the great sulky white bullocks . . . came <qex>shouldering</qex> along together.</q>  <rj><qau>Kipling.</qau></rj><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Shouldered</ent><br/
<hw>Shoul"dered</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Having shoulders; -- used in composition; <as>as, a broad-<ex>shouldered</ex> man</as>.</def> <ldquo/He was short-<xex>shouldered</xex>.<rdquo/  <rj><au>Chaucer.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shoulder-shotten</ent><br/
<hw>Shoul"der-shot`ten</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Sprained in the shoulder, as a horse.</def>  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shout</ent><br/
<hw>Shout</hw> <pr>(shout)</pr>, <pos>v. i.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Shouted</conjf>; <pos>p. pr. & vb. n.</pos> <conjf>Shouting</conjf>.]</vmorph> <ety>[OE. <ets>shouten</ets>, of unknown origin; perhaps akin to <ets>shoot</ets>; cf. Icel. <ets>sk<umac/ta</ets>, <ets>sk<umac/ti</ets>, a taunt.]</ety> <sn>1.</sn> <def>To utter a sudden and loud outcry, as in joy, triumph, or exultation, or to attract attention, to animate soldiers, etc.</def><br/
[<source>1913 Webster</source>]</p>

<p><q><qex>Shouting</qex> of the men and women eke.</q> <rj><qau>Chaucer.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>They <qex>shouted</qex> thrice: what was the last cry for?</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To entertain with refreshments or the like gratuitously; to treat.</def> <mark>[Slang, Australia & U. S.]</mark><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><cs><col><b>To shout at</b></col>, <cd>to utter shouts at; to deride or revile with shouts.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shout</ent><br/
<hw>Shout</hw>, <pos>v. t.</pos> <sn>1.</sn> <def>To utter with a shout; to cry; -- sometimes with <xex>out</xex>; <as>as, to <ex>shout</ex>, or to <ex>shout</ex> out, a man's name</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To treat with shouts or clamor.</def>  <rj><au>Bp. Hall.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>To treat (one) to something; also, to give (something) by way of treating.</def> <mark>[Slang, Australia & U. S.]</mark><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Shout</ent><br/
<hw>Shout</hw>, <pos>n.</pos> <sn>1.</sn> <def>A loud burst of voice or voices; a vehement and sudden outcry, especially of a multitudes expressing joy, triumph, exultation, or animated courage.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The Rhodians, seeing the enemy turn their backs, gave a great <qex>shout</qex> in derision.</q> <rj><qau>Knolles.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A gratuitous entertainment, with refreshments or the like; a treat.</def> <mark>[Slang, Australia & U. S.]</mark><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Shouter</ent><br/
<hw>Shout"er</hw> <pr>(?)</pr>, <pos>n.</pos> <def>One who shouts.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shove</ent><br/
<hw>Shove</hw> <pr>(sh<ucr/v)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Shoved</conjf> <pr>(sh<ucr/vd)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Shoving</conjf>.]</vmorph> <ety>[OE. <ets>shoven</ets>, AS. <ets>scofian</ets>, fr. <ets>sc<umac/fan</ets>; akin to OFries. <ets>sk<umac/va</ets>, D. <ets>schuiven</ets>, G. <ets>schieben</ets>, OHG. <ets>scioban</ets>, Icel. <ets>sk<umac/fa</ets>, <ets>sk<ymac/fa</ets>, Sw. <ets>skuffa</ets>, Dan. <ets>skuffe</ets>, Goth. af<ets>skiuban</ets> to put away, cast away; cf. Skr. <ets>kshubh</ets> to become agitated, to quake, Lith. <ets>skubrus</ets> quick, <ets>skubinti</ets> to hasten. <root/160.  Cf. <er>Sheaf</er> a bundle of stalks, <er>Scoop</er>, <er>Scuffle</er>.]</ety> <sn>1.</sn> <def>To drive along by the direct and continuous application of strength; to push; especially, to push (a body) so as to make it move along the surface of another body; <as>as, to <ex>shove</ex> a boat on the water; to <ex>shove</ex> a table across the floor.</as></def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To push along, aside, or away, in a careless or rude manner; to jostle.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>And <qex>shove</qex> away the worthy bidden guest.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>He used to <qex>shove</qex> and elbow his fellow servants.</q> <rj><qau>Arbuthnot.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shove</ent><br/
<hw>Shove</hw>, <pos>v. i.</pos> <sn>1.</sn> <def>To push or drive forward; to move onward by pushing or jostling.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To move off or along by an act pushing, as with an oar a pole used by one in a boat; sometimes with <xex>off</xex>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>He grasped the oar,<br/
eceived his guests on board, and <qex>shoved</qex> from shore.</q> <rj><qau>Garth.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shove</ent><br/
<hw>Shove</hw> <pr>(?)</pr>, <pos>n.</pos> <def>The act of shoving; a forcible push.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>I rested . . . and then gave the boat another <qex>shove</qex>.</q> <rj><qau>Swift.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><syn><b>Syn.</b> -- See <er>Thrust</er>.</syn><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shove</ent><br/
<hw>Shove</hw>, <mark>obs.</mark> <def><pos>p. p.</pos> of <er>Shove</er>.</def>  <rj><au>Chaucer.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shovegroat</ent><br/
<ent>Shoveboard</ent><br/
<mhw>{ <hw>Shove"board`</hw> <pr>(?)</pr>, <hw>Shove"groat`</hw> <pr>(?)</pr>, }</mhw> <pos>n.</pos> <def>The same as <er>Shovelboard</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shovel</ent><br/
<hw>Shov"el</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[OE. <ets>shovele</ets>, <ets>schovele</ets>, AS. <ets>scoft</ets>, <ets>sceoft</ets>; akin to D. <ets>schoffel</ets>, G. <ets>schaufel</ets>, OHG. <ets>sc<?/vala</ets>, Dan. <ets>skovl</ets>, Sw. <ets>skofvel</ets>, <ets>skyffel</ets>, and to E. <ets>shove</ets>. <root/160. See <er>Shove</er>, <pos>v. t.</pos>]</ety> <def>An implement consisting of a broad scoop, or more or less hollow blade, with a handle, used for lifting and throwing earth, coal, grain, or other loose substances.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Shovel hat</b></col>, <cd>a broad-brimmed hat, turned up at the sides, and projecting in front like a shovel, -- worn by some clergy of the English Church.</cd> <mark>[Colloq.]</mark> -- <col><b>Shovelspur</b></col> <fld>(Zool.)</fld>, <cd>a flat, horny process on the tarsus of some toads, -- used in burrowing.</cd> -- <col><b>Steam shovel</b></col>, <cd>a machine with a scoop or scoops, operated by a steam engine, for excavating earth, as in making railway cuttings.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shovel</ent><br/
<hw>Shov"el</hw>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Shoveled</conjf> <pr>(?)</pr> or <conjf>Shovelled</conjf>; <pos>p. pr. & vb. n.</pos> <conjf>Shoveling</conjf> or <conjf>Shovelling</conjf>.]</vmorph> <sn>1.</sn> <def>To take up and throw with a shovel; <as>as, to <ex>shovel</ex> earth into a heap, or into a cart, or out of a pit</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To gather up as with a shovel.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shovelard</ent><br/
<hw>Shov"el*ard</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Zool.)</fld> <def>Shoveler.</def> <mark>[Prov. Eng.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shovelbill</ent><br/
<hw>Shov"el*bill`</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Zool.)</fld> <def>The shoveler.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shovelboard</ent><br/
<hw>Shov"el*board`</hw> <pr>(?)</pr>, <pos>n.</pos> <sn>1.</sn> <def>A board on which a game is played, by pushing or driving pieces of metal or money to reach certain marks; also, the game itself. Called also <altname>shuffleboard</altname>, <altname>shoveboard</altname>, <altname>shovegroat</altname>, <altname>shovelpenny</altname>.</def><-- now usu. shuffleboard. --><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A game played on board ship in which the aim is to shove or drive with a cue wooden disks into divisions chalked on the deck; -- called also <altname>shuffleboard</altname>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shoveler</ent><br/
<hw>Shov"el*er</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Also <ets>shoveller</ets>.]</ety> <sn>1.</sn> <def>One who, or that which, shovels.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Zool.)</fld> <def>A river duck (<spn>Spatula clypeata</spn>), native of Europe and America. It has a large bill, broadest towards the tip. The male is handsomely variegated with green, blue, brown, black, and white on the body; the head and neck are dark green. Called also <altname>broadbill</altname>, <altname>spoonbill</altname>, <altname>shovelbill</altname>, and <altname>maiden duck</altname>.  The Australian shoveler, or shovel-nosed duck (<spn>Spatula rhynchotis</spn>), is a similar species.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shovelful</ent><br/
<hw>Shov"el*ful</hw> <pr>(?)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Shovelfuls</plw> <pr>(<?/)</pr>.</plu> <def>As much as a shovel will hold; enough to fill a shovel.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shovelhead</ent><br/
<hw>Shov"el*head`</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Zool.)</fld> <def>A shark (<spn>Sphryna tiburio</spn>) allied to the hammerhead, and native of the warmer parts of the Atlantic and Pacific oceans; -- called also <altname>bonnet shark</altname>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shovelnose</ent><br/
<hw>Shov"el*nose`</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Zool.)</fld> <sd>(a)</sd> <def>The common sand shark. See under <er>Snad</er>.</def> <sd>(b)</sd> <def>A small California shark (<spn>Heptranchias maculatus</spn>), which is taken for its oil.</def> <sd>(c)</sd> <def>A Pacific Ocean shark (<spn>Hexanchus corinus</spn>).</def> <sd>(d)</sd> <def>A ganoid fish of the Sturgeon family (<spn>Scaphirhynchus platyrhynchus</spn>) of the Mississippi and Ohio rivers; -- called also <altname>white sturgeon</altname>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shovel-nosed</ent><br/
<hw>Shov"el-nosed`</hw> <pr>(?)</pr>, <pos>a.</pos> <fld>(Zool.)</fld> <def>Having a broad, flat nose; <as>as, the <ex>shovel-nosed</ex> duck, or shoveler</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shoven</ent><br/
<hw>Shov"en</hw> <pr>(?)</pr>, <mark>obs.</mark> <def><pos>p. p.</pos> of <er>Shove</er>.</def>  <rj><au>Chaucer.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Show</ent><br/
<hw>Show</hw> <pr>(?)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp.</pos> <conjf>Showed</conjf> <pr>(?)</pr>; <pos>p. p.</pos> <conjf>Shown</conjf> <pr>(?)</pr> or <conjf>Showed</conjf>; <pos>p. pr. & vb. n.</pos> <conjf>Showing</conjf>. <altsp>It is sometimes written <asp>shew</asp>, <asp>shewed</asp>, <asp>shewn</asp>, <asp>shewing</asp>.</altsp>]</vmorph> <ety>[OE. <ets>schowen</ets>, <ets>shewen</ets>, <ets>schewen</ets>, <ets>shawen</ets>, AS. <ets>sce<aacute/wian</ets>, to look, see, view; akin to OS. <ets>scaw<?/n</ets>, OFries. <ets>skawia</ets>, D. <ets>schouwen</ets>, OHG. <ets>scouw<?/n</ets>, G. <ets>schauen</ets>, Dan. <ets>skue</ets>, Sw. <ets>sk<?/da</ets>, Icel. <ets>sko<?/a</ets>, Goth. us<ets>skawjan</ets> to waken, <ets>skuggwa</ets> a mirror, Icel. <ets>skuggy</ets> shade, shadow, L. <ets>cavere</ets> to be on one's guard, Gr. <?/<?/<?/ to mark, perceive, hear, Skr. <ets>kavi</ets> wise.  Cf. <er>Caution</er>, <er>Scavenger</er>, <er>Sheen</er>.]</ety> <sn>1.</sn> <def>To exhibit or present to view; to place in sight; to display; -- the thing exhibited being the object, and often with an indirect object denoting the person or thing seeing or beholding; <as>as, to <ex>show</ex> a house; <ex>show</ex> your colors; shopkeepers <ex>show</ex> customers goods (show goods to customers).</as></def><br/
[<source>1913 Webster</source>]</p>

<p><q>Go thy way, <qex>shew</qex> thyself to the priest.</q> <rj><qau>Matt. viii. 4.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Nor want we skill or art from whence to raise<br/
Magnificence; and what can heaven <qex>show</qex> more?</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To exhibit to the mental view; to tell; to disclose; to reveal; to make known; <as>as, to <ex>show</ex> one's designs</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q><qex>Shew</qex> them the way wherein they must walk.</q> <rj><qau>Ex. xviii. 20.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>If it please my father to do thee evil, then I will <qex>shew</qex> it thee, and send thee away.</q> <rj><qau>1 Sam. xx. 13.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>Specifically, to make known the way to (a person); hence, to direct; to guide; to asher; to conduct; <as>as, to <ex>show</ex> a person into a parlor; to <ex>show</ex> one to the door</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>To make apparent or clear, as by evidence, testimony, or reasoning; to prove; to explain; also, to manifest; to evince; <as>as, to <ex>show</ex> the truth of a statement; to <ex>show</ex> the causes of an event</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>I 'll <qex>show</qex> my duty by my timely care.</q> <rj><qau>Dryden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <def>To bestow; to confer; to afford; <as>as, to <ex>show</ex> favor</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q><qex>Shewing</qex> mercy unto thousands of them that love me.</q> <rj><qau>Ex. xx. 6.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>To show forth</b></col>, <cd>to manifest; to publish; to proclaim.</cd> -- <col><b>To show his paces</b></col>, <cd>to exhibit the gait, speed, or the like; -- said especially of a horse.</cd> -- <col><b>To show off</b></col>, <cd>to exhibit ostentatiously.</cd> -- <col><b>To show up</b></col>, <cd>to expose.</cd> <mark>[Colloq.]</mark></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Show</ent><br/
<hw>Show</hw>, <pos>v. i.</pos> <ety>[Written also <ets>shew</ets>.]</ety> <sn>1.</sn> <def>To exhibit or manifest one's self or itself; to appear; to look; to be in appearance; to seem.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Just such she <qex>shows</qex> before a rising storm.</q> <rj><qau>Dryden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>All round a hedge upshoots, and <qex>shows</qex><br/
At distance like a little wood.</q> <rj><qau>Tennyson.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><-- p. 1334 --></p>

<p><sn>2.</sn> <def>To have a certain appearance, as well or ill, fit or unfit; to become or suit; to appear.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>My lord of York, it better <qex>showed</qex> with you.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>To show off</b></col>, <cd>to make a show; to display one's self.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Show</ent><br/
<hw>Show</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Formerly written also <ets>shew</ets>.]</ety> <sn>1.</sn> <def>The act of showing, or bringing to view; exposure to sight; exhibition.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>That which os shown, or brought to view; that which is arranged to be seen; a spectacle; an exhibition; <as>as, a traveling <ex>show</ex>; a cattle <ex>show</ex></as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>As for triumphs, masks, feasts, and such <qex>shows</qex>.</q> <rj><qau>Bacon.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>Proud or ostentatious display; parade; pomp.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>I envy none their pageantry and <qex>show</qex>.</q> <rj><qau>Young.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>Semblance; likeness; appearance.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>He through the midst unmarked,<br/
In <qex>show</qex> plebeian angel militant<br/
Of lowest order, passed.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <def>False semblance; deceitful appearance; pretense.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Beware of the scribes, . . . which devour widows' houses, and for a <qex>shew</qex> make long prayers.</q> <rj><qau>Luke xx. 46. 47.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>6.</sn> <fld>(Med.)</fld> <def>A discharge, from the vagina, of mucus streaked with blood, occuring a short time before labor.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>7.</sn> <fld>(Mining)</fld> <def>A pale blue flame, at the top of a candle flame, indicating the presence of fire damp.</def>  <rj><au>Raymond.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Show bill</b></col>, <cd>a broad sheet containing an advertisement in large letters.</cd> -- <col><b>Show box</b></col>, <cd>a box xontaining some object of curiosity carried round as a show.</cd> -- <col><b>Show card</b></col>, <cd>an advertising placard; also, a card for displaying samples.</cd> -- <col><b>Show case</b></col>, <cd>a gla<?/ed case, box, or cabinet for displaying and protecting shopkeepers' wares, articles on exhibition in museums, etc.</cd> -- <col><b>Show glass</b></col>, <cd>a glass which displays objects; a mirror.</cd> -- <col><b>Show of hands</b></col>, <cd>a raising of hands to indicate judgment; as, the vote was taken by a <xex>show of hands</xex>.</cd> -- <col><b>Show stone</b></col>, <cd>a piece of glass or crystal supposed to have the property of exhibiting images of persons or things not present, indicating in that way future events.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Showbread</ent><br/
<hw>Show"bread`</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Jewish Antiq.)</fld> <def>Bread of exhibition; loaves to set before God; -- the term used in translating the various phrases used in the Hebrew and Greek to designate the loaves of bread which the priest of the week placed before the Lord on the golden table in the sanctuary. They were made of fine flour unleavened, and were changed every Sabbath. The loaves, twelve in number, represented the twelve tribes of Israel. They were to be eaten by the priests only, and in the Holy Place.</def> <altsp>[Written also <asp>shewbread</asp>.]</altsp>  <rj><au>Mark ii. 26.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shower</ent><br/
<hw>Show"er</hw> <pr>(?)</pr>, <pos>n.</pos> <sn>1.</sn> <def>One who shows or exhibits.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>That which shows; a mirror.</def> <mark>[Obs.]</mark>  <rj><au>Wyclif.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shower</ent><br/
<hw>Show"er</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[OE. <ets>shour</ets>, <ets>schour</ets>, AS. <ets>se<?/r</ets>; akin to D. <ets>schoer</ets>, G. <ets>schauer</ets>, OHG. <ets>sc<?/r</ets>, Icel. <ets>sk<?/r</ets>, Sw. <ets>skur</ets>, Goth. <ets>sk<?/ra windis</ets> a storm of wind; of uncertain origin.]</ety> <sn>1.</sn> <def>A fall or rain or hail of short duration; sometimes, but rarely, a like fall of snow.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>In drought or else <qex>showers</qex>.</q> <rj><qau>Chaucer.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Or wet the thirsty earth with falling <qex>showers</qex>.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>That which resembles a shower in falling or passing through the air copiously and rapidly.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>With <qex>showers</qex> of stones he drives them far away.</q> <rj><qau>Pope.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>A copious supply bestowed.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>He and myself<br/
Have travail'd in the great <qex>shower</qex> of your gifts.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Shower bath</b></col>, <cd>a bath in which water is showered from above, and sometimes from the sides also.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shower</ent><br/
<hw>Show"er</hw>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Showered</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Showering</conjf>.]</vmorph> <sn>1.</sn> <def>To water with a shower; to <?/<?/t copiously with rain.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Lest it again dissolve and <qex>shower</qex> the earth.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To bestow liberally; to destribute or scatter in <?/undance; to rain.</def>  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>C<?/sar's favor,<br/
That <qex>showers</qex> down greatness on his friends.</q> <rj><qau>Addison.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shower</ent><br/
<hw>Show"er</hw>, <pos>v. i.</pos> <def>To rain in showers; to fall, as in a hower or showers.</def>  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Showerful</ent><br/
<hw>Show"er*ful</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Full of showers.</def>  <rj><au>Tennyson.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Showeriness</ent><br/
<hw>Show"er*i*ness</hw> <pr>(?)</pr>, <pos>n.</pos> <def>Quality of being showery.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Showerless</ent><br/
<hw>Show"er*less</hw>, <pos>a.</pos> <def>Rainless; freo from showers.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Showery</ent><br/
<hw>Show"er*y</hw> <pr>(?)</pr>, <pos>a.</pos> <sn>1.</sn> <def>Raining in showers; abounding with frequent showers of rain.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Of or pertaining to a shower or showers.</def> <ldquo/Colors of the <xex>showery</xex> arch.<rdquo/  <rj><au>Milton.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Showily</ent><br/
<hw>Show"i*ly</hw> <pr>(?)</pr>, <pos>adv.</pos> <def>In a showy manner; pompously; with parade.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Showiness</ent><br/
<hw>Show"i*ness</hw>, <pos>n.</pos> <def>The quality or state of being showy; pompousness; great parade; ostentation.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Showing</ent><br/
<hw>Show"ing</hw>, <pos>n.</pos> <sn>1.</sn> <def>Appearance; display; exhibition.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Presentation of facts; statement.</def>  <rj><au>J. S. Mill.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Showish</ent><br/
<hw>Show"ish</hw>, <pos>a.</pos> <def>Showy; ostentatious.</def>  <rj><au>Swift.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Showman</ent><br/
<hw>Show"man</hw> <pr>(?)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Showmen</plw> <pr>(#)</pr>.</plu> <def>One who exhibits a show; a proprietor of a show.</def><br/
[<source>1913 Webster</source>]</p>

<p><-- 1 (b) esp. a producer of an entertainment .
2.  One who has a knack for dramatic or entertaining presentation; an entertainer. --><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shown</ent><br/
<hw>Shown</hw> <pr>(?)</pr>, <def><pos>p. p.</pos> of <er>Show</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Showroom</ent><br/
<hw>Show"room`</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A room or apartment where a show is exhibited.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A room where merchandise is exposed for sale, or where samples are displayed.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Showy</ent><br/
<hw>Show"y</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[<ets>Compar</ets>. <er>Showier</er> <pr>(<?/)</pr>; <ets>superl</ets>. <er>Showiest</er>.]</ety> <def>Making a show; attracting attention; presenting a marked appearance; ostentatious; gay; gaudy.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>A present of everything that was rich and <qex>showy</qex>.</q> <rj><qau>Addison.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><syn><b>Syn.</b> -- Splendid; gay; gaudy; gorgeous; fine; magnificent; grand; stately; sumptuous; pompous.</syn><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shrag</ent><br/
<hw>Shrag</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[CF. <er>Scrag</er>.]</ety> <def>A twig of a tree cut off.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shrag</ent><br/
<hw>Shrag</hw>, <pos>v. t.</pos> <def>To trim, as trees; to lop.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shragger</ent><br/
<hw>Shrag"ger</hw> <pr>(?)</pr>, <pos>n.</pos> <def>One who lops; one who trims trees.</def> <mark>[Obs.]</mark>  <rj><au>Huloet.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shram</ent><br/
<hw>Shram</hw> <pr>(?)</pr>, <pos>v. t.</pos> <ety>[Cf. Shrink.]</ety> <def>To cause to shrink or shrivel with cold; to benumb.</def> <mark>[Prov. Eng.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shrank</ent><br/
<hw>Shrank</hw> <pr>(?)</pr>, <def><pos>imp.</pos> of <er>Shrink</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shrape</ent><br/
<ent>Shrap</ent><br/
<mhw>{ <hw>Shrap</hw> <pr>(?)</pr>, <hw>Shrape</hw> <pr>(?)</pr>, }</mhw> <pos>n.</pos> <ety>[Cf. <er>Scrap</er>, and <er>Scrape</er>.]</ety> <def>A place baited with chaff to entice birds.</def> <altsp>[Written also <asp>scrap</asp>.]</altsp> <mark>[Obs.]</mark>  <rj><au>Bp. Bedell.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shrapnel</ent><br/
<hw>Shrap"nel</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Applied as an appellation to a kind of shell invented by Gen. H. <xex>Shrapnel</xex> of the British army.</def> -- <def2><pos>n.</pos> <def>A shrapnel shell; shrapnel shells, collectively.</def></def2><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Shrapnel shell</b></col> <fld>(Gunnery)</fld>, <cd>a projectile for a cannon, consisting of a shell filled with bullets and a small bursting charge to scatter them at any given point while in flight. See the Note under <er>Case shot</er>.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shred</ent><br/
<hw>Shred</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[OE. <ets>shrede</ets>, <ets>schrede</ets>, AS. <ets>scre<aacute/de</ets>; akin to OD. <ets>schroode</ets>, G. <ets>schrot</ets> a piece cut off, Icel. <ets>skrjo<edh/r</ets> a shred, and to E. <ets>shroud</ets>.  Cf. <er>Screed</er>, <er>Scroll</er>, <er>Scrutiny</er>.]</ety> <sn>1.</sn> <def>A long, narrow piece cut or torn off; a strip.</def> <ldquo/<xex>Shreds</xex> of tanned leather.<rdquo/  <rj><au>Bacon.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>In general, a fragment; a piece; a particle.</def>  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shred</ent><br/
<hw>Shred</hw>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Shred</conjf> or <conjf>Shredded</conjf> <pr>(<?/)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Shredding</conjf>.]</vmorph> <ety>[OE. <ets>shreden</ets>, <ets>schreden</ets>, AS. <ets>scre<aacute/dian</ets>; akin to OD. <ets>schrooden</ets>, OHG. <ets>scr<?/tan</ets>, G. <ets>schroten</ets>. See <er>Shred</er>, <pos>n.</pos>]</ety> <sn>1.</sn> <def>To cut or tear into small pieces, particularly narrow and long pieces, as of cloth or leather.</def>  <rj><au>Chaucer.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To lop; to prune; to trim.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shredcook</ent><br/
<hw>Shred"cook`</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Zool.)</fld> <def>The fieldfare; -- so called from its harsh cry before rain.</def> <mark>[Prov. Eng.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shredding</ent><br/
<hw>Shred"ding</hw> <pr>(?)</pr>, <pos>n.</pos> <sn>1.</sn> <def>The act of cutting or tearing into shreds.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>That which is cut or torn off; a piece.</def>  <rj><au>Hooker.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shreddy</ent><br/
<hw>Shred"dy</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Consisting of shreds.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shredless</ent><br/
<hw>Shred"less</hw>, <pos>a.</pos> <def>Having no shreds; without a shred.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>And those which waved are <qex>shredless</qex> dust ere now.</q> <rj><qau>Byron.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shrew</ent><br/
<hw>Shrew</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[OE. <ets>shrewe</ets>, <ets>schrewe</ets>.  Cf. <er>Shrewd</er>.]</ety> <def>Wicked; malicious.</def> <mark>[Obs.]</mark>  <rj><au>Chaucer.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shrew</ent><br/
<hw>Shrew</hw>, <pos>n.</pos> <ety>[See <er>Shrew</er>, <pos>a.</pos>]</ety> <sn>1.</sn> <def>Originally, a brawling, turbulent, vexatious person of either sex, but now restricted in use to females; a brawler; a scold.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>A man . . . grudgeth that <qex>shrews</qex> [i. e., bad men] have prosperity, or else that good men have adversity.</q> <rj><qau>Chaucer.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>A man had got a <qex>shrew</qex> to his wife, and there could be no quiet in the house for her.</q> <rj><qau>L'Estrange.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <ety>[AS. <ets>scre<aacute/wa</ets>; -- so called because supposed to be venomous. ]</ety> <fld>(Zool.)</fld> <def>Any small insectivore of the genus <gen>Sorex</gen> and several allied genera of the family <fam>Sorecidae</fam>. In form and color they resemble mice, but they have a longer and more pointed nose. Some of them are the smallest of all mammals.</def><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ The common European species are the house shrew (<spn>Crocidura araneus</spn>), and the erd shrew (<spn>Sorex vulgaris</spn>) (see under <er>Erd</er>.). In the United States several species of <gen>Sorex</gen> and <gen>Blarina</gen> are common, as the broadnosed shrew (<spn>Sorex platyrhinus</spn>), Cooper's shrew (<spn>Sorex Cooperi</spn>), and the short-tailed, or mole, shrew (<spn>Blarina brevicauda</spn>).  Th American water, or marsh, shrew (<spn>Neosorex palustris</spn>), with fringed feet, is less common. The common European water shrews are <spn>Crossopus fodiens</spn>, and the oared shrew (see under <er>Oared</er>).</note><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Earth shrew</b></col>, <cd>any shrewlike burrowing animal of the family <fam>Centetidae</fam>, as the tendrac.</cd> -- <mcol><col><b>Elephant shrew</b></col>, <col><b>Jumping shrew</b></col>, <col><b>Mole shrew</b></col></mcol>. <cd>See under <er>Elephant</er>, <er>Jumping</er>, etc.</cd> -- <col><b>Musk shrew</b></col>. <cd>See <er>Desman</er>.</cd> -- <col><b>River shrew</b></col>, <cd>an aquatic West African insectivore (<spn>Potamogale velox</spn>) resembling a weasel in form and size, but having a large flattened and crested tail adapted for rapid swimming. It feeds on fishes.</cd> -- <col><b>Shrew mole</b></col>, <cd>a common large North American mole (<spn>Scalops aquaticus</spn>). Its fine, soft fur is gray with iridescent purple tints.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shrew</ent><br/
<hw>Shrew</hw>, <pos>v. t.</pos> <ety>[See <er>Shrew</er>, <pos>a.</pos>, and cf. <er>Beshrew</er>.]</ety> <def>To beshrew; to curse.</def> <mark>[Obs.]</mark> <ldquo/I <xex>shrew</xex> myself.<rdquo/  <rj><au>Chaucer.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shrewd</ent><br/
<hw>Shrewd</hw> <pr>(?)</pr>, <pos>a.</pos> <amorph>[<pos>Compar.</pos> <adjf>Shrewder</adjf> <pr>(?)</pr>; <pos>superl.</pos> <adjf>Shrewdest</adjf>.]</amorph> <ety>[Originally the p. p. of <ets>shrew</ets>, v.t.]</ety> <sn>1.</sn> <def>Inclining to shrew; disposing to curse or scold; hence, vicious; malicious; evil; wicked; mischievous; vexatious; rough; unfair; shrewish.</def> <mark>[Obs.]</mark>  <rj><au>Chaucer.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>[Egypt] hath many <qex>shrewd</qex> havens, because of the great rocks that ben strong and dangerous to pass by.</q> <rj><qau>Sir J. Mandeville.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Every of this happy number<br/
That have endured <qex>shrewd</qex> days and nights with us.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Artful; wily; cunning; arch.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>These women are <qex>shrewd</qex> tempters with their tongues.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>Able or clever in practical affairs; sharp in business; astute; sharp-witted; sagacious; keen; <as>as, a <ex>shrewd</ex> observer; a <ex>shrewd</ex> design; a <ex>shrewd</ex> reply.</as></def><br/
[<source>1913 Webster</source>]</p>

<p><q>Professing to despise the ill opinion of mankind creates a <qex>shrewd</qex> suspicion that we have deserved it.</q> <rj><qau>Secker.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><syn><b>Syn.</b> -- Keen; critical; subtle; artful; astute; sagacious; discerning; acute; penetrating.</syn>  -- <usage><er>Shrewd</er>, <er>Sagacious</er>. One who is shrewd is keen to detect errors, to penetrate disguises, to foresee and guard against the selfishness of others. <xex>Shrewd</xex> is a word of less dignity than <xex>sagacious</xex>, which implies a comprehensive as well as penetrating mind, whereas <xex>shrewd</xex> does not.</usage><br/
[<source>1913 Webster</source>]</p>

<p>-- <wordforms><wf>Shrewd"ly</wf>, <pos>adv.</pos> -- <wf>Shrewd"ness</wf>, <pos>n.</pos></wordforms><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shrewish</ent><br/
<hw>Shrew"ish</hw> <pr>(?)</pr>, <pos>a.</pos> <def>having the qualities of a shrew; having a scolding disposition; froward; peevish.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>My wife is <qex>shrewish</qex> when I keep not hours.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p>-- <wordforms><wf>Shrew"ish*ly</wf>, <pos>adv.</pos> -- <wf>Shrew"ish*ness</wf>, <pos>n.</pos></wordforms><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shrewmouse</ent><br/
<hw>Shrew"mouse`</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Zool.)</fld> <def>A shrew; especially, the erd shrew.</def>
<-- Sorex vulgaris --><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shriek</ent><br/
<hw>Shriek</hw> <pr>(?)</pr>, <pos>v. i.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Shrieked</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Shrieking</conjf>.]</vmorph> <ety>[OE. <ets>shriken</ets>, originallythe same word as E. <ets>screech</ets>. See <er>Screech</er>, and cf. <er>Screak</er>.]</ety> <def>To utter a loud, sharp, shrill sound or cry, as do some birds and beasts; to scream, as in a sudden fright, in horror or anguish.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>It was the owl that <qex>shrieked</qex>.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>At this she <qex>shrieked</qex> aloud; the mournful train<br/
Echoed her grief.</q> <rj><qau>Dryden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shriek</ent><br/
<hw>Shriek</hw> <pr>(?)</pr>, <pos>v. t.</pos> <def>To utter sharply and shrilly; to utter in or with a shriek or shrieks.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>On top whereof aye dwelt the ghostly owl,<br/
<qex>Shrieking</qex> his baleful note.</q> <rj><qau>Spenser.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>She <qex>shrieked</qex> his name<br/
To the dark woods.</q> <rj><qau>Moore.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shriek</ent><br/
<hw>Shriek</hw>, <pos>n.</pos> <def>A sharp, shrill outcry or scream; a shrill wild cry such as is caused by sudden or extreme terror, pain, or the like.</def><br/
[<source>1913 Webster</source>]</p>

<p><q><qex>Shrieks</qex>, clamors, murmurs, fill the frighted town.</q> <rj><qau>Dryden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Shriek owl</b></col>. <fld>(Zool.)</fld> <sd>(a)</sd> <cd>The screech owl</cd>. <sd>(b)</sd> <cd>The swift; -- so called from its cry.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shrieker</ent><br/
<hw>Shriek"er</hw> <pr>(?)</pr>, <pos>n.</pos> <def>One who utters a shriek.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shrieval</ent><br/
<hw>Shriev"al</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Of or pertaining to a sheriff.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shrievalty</ent><br/
<hw>Shriev"al*ty</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Contr. from <ets>sheriffalty</ets>. See <er>Shrieve</er>, <pos>n.</pos> <er>Sheriff</er>.]</ety> <def>The office, or sphere of jurisdiction, of a sheriff; sheriffalty.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>It was ordained by 28 Edward I that the people shall have election of sheriff in every shire where the <qex>shrievalty</qex> is not of inheritance.</q> <rj><qau>Blackstone.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shrieve</ent><br/
<hw>Shrieve</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Contr. from OE. <ets>shereve</ets>. See <er>Sheriff</er>.]</ety> <def>A sheriff.</def> <mark>[Obs.]</mark>  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shrieve</ent><br/
<hw>Shrieve</hw>, <pos>v. t.</pos> <def>To shrive; to question.</def> <mark>[Obs.]</mark> <ldquo/She gan him soft to <xex>shrieve</xex>.<rdquo/  <rj><au>Spenser.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shrift</ent><br/
<hw>Shrift</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[OE. <ets>shrift</ets>, <ets>schrift</ets>, AS. <ets>scrift</ets>, fr. <ets>scr<imac/fan</ets> to shrive. See <er>Shrive</er>.]</ety> <sn>1.</sn> <def>The act of shriving.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>In <qex>shrift</qex> and preaching is my diligence.</q> <rj><qau>Chaucer.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Confession made to a priest, and the absolution consequent upon it.</def>  <rj><au>Chaucer.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Have you got leave to go to <qex>shrift</qex> to-day?</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Therefore, my lord, address you to your <qex>shrift</qex>,<br/
And be yourself; for you must die this instant.</q> <rj><qau>Rowe.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Shrift father</b></col>, <cd>a priest to whom confession is made.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shright</ent><br/
<hw>Shright</hw> <pr>(?)</pr>, <mark>obs.</mark> <def><pos>imp. & p. p.</pos> of <er>Shriek</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>She cried alway and <qex>shright</qex>.</q> <rj><qau>Chaucer.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shright</ent><br/
<hw>Shright</hw>, <pos>n.</pos> <ety>[See <er>Shriek</er>.]</ety> <def>A shriek; shrieking.</def> <mark>[Obs]</mark> <au>Spenser.</au> <ldquo/All hoarse for <xex>shright</xex>.<rdquo/  <rj><au>Chaucer.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shrike</ent><br/
<hw>Shrike</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Akin to Icel. <ets>skr<imac/kja</ets> a shrieker, the shrike, and E. <ets>shriek</ets>; cf. AS. <ets>scr<imac/c</ets> a thrush. See <er>Shriek</er>, <pos>v. i.</pos>]</ety> <fld>(Zool.)</fld> <def>Any one of numerous species of oscinine birds of the family <fam>Laniidae</fam>, having a strong hooked bill, toothed at the tip. Most shrikes are insectivorous, but the common European gray shrike (<spn>Lanius excubitor</spn>), the great northern shrike (<spn>Lanius borealis</spn>), and several others, kill mice, small birds, etc., and often impale them on thorns, and are, on that account called also <altname>butcher birds</altname>. See under <er>Butcher</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ The ant shrikes, or bush shrikes, are clamatorial birds of the family <fam>Formicaridae</fam>. The cuckoo shrikes of the East Indies and Australia are Oscines of the family <fam>Campephagidae</fam>. The drongo shrikes of the same regions belong to the related family <fam>Dicruridae</fam>. See <er>Drongo</er>.</note><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Crow shrike</b></col>. <cd>See under <er>Crow</er>.</cd> -- <col><b>Shrike thrush</b></col>. <sd>(a)</sd> <cd>Any one of several species of Asiatic timaline birds of the genera <gen>Thamnocataphus</gen>, <gen>Gampsorhynchus</gen>, and allies</cd>. <sd>(b)</sd> <cd>Any one of several species of shrikelike Australian singing birds of the genus <gen>Colluricincla</gen>.</cd> -- <col><b>Shrike tit</b></col>. <sd>(a)</sd> <cd>Any one of several Australian birds of the genus <gen>Falcunculus</gen>, having a strong toothed bill and sharp claws. They creep over the bark of trees, like titmice, in search of insects.</cd> <sd>(b)</sd> <cd>Any one of several species of small Asiatic birds belonging to <gen>Allotrius</gen>, <gen>Pteruthius</gen>, <gen>Cutia</gen>, <gen>Leioptila</gen>, and allied genera, related to the true tits. Called also <altname>hill tit</altname>.</cd> -- <col><b>Swallow shrike</b></col>. <cd>See under <er>Swallow</er>.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shrill</ent><br/
<hw>Shrill</hw> <pr>(?)</pr>, <pos>a.</pos> <amorph>[<pos>Compar.</pos> <adjf>Shriller</adjf> <pr>(?)</pr>; <pos>superl.</pos> <adjf>Shrillest</adjf>.]</amorph> <ety>[OE. <ets>shril</ets>, <ets>schril</ets>; akin to LG. <ets>schrell</ets>, G. <ets>schrill</ets>. See <er>Shrill</er>,<pos>v. i.</pos>]</ety> <def>Acute; sharp; piercing; having or emitting a sharp, piercing tone or sound; -- said of a sound, or of that which produces a sound.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Hear the <qex>shrill</qex> whistle which doth order give<br/
To sounds confused.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Let winds be <qex>shrill</qex>, let waves roll high.</q> <rj><qau>Byron.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shrill</ent><br/
<hw>Shrill</hw>, <pos>n.</pos> <def>A shrill sound.</def> <mark>[Obs.]</mark>  <rj><au>Spenser.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shrill</ent><br/
<hw>Shrill</hw>, <pos>v. i.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Shrilled</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Shrilling</conjf>.]</vmorph> <ety>[OE. <ets>schrillen</ets>, akin to G. <ets>schrillen</ets>; cf. AS. <ets>scralletan</ets> to resound loudly, Icel. <ets>skr<oum/lta</ets> to jolt, Sw. <ets>skr<aum/lla</ets> to shrill, Norw. <ets>skryla</ets>, <ets>skr<?/la</ets>.  Cf. <er>Skirl</er>.]</ety> <def>To utter an acute, piercing sound; to sound with a sharp, shrill tone; to become shrill.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Break we our pipes, that <qex>shrilled</qex>loud as lark.</q> <rj><qau>Spenser.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>No sounds were heard but of the <qex>shrilling</qex> cock.</q> <rj><qau>Goldsmith.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>His voice <qex>shrilled</qex> with passion.</q> <rj><qau>L. Wallace.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shrill</ent><br/
<hw>Shrill</hw>, <pos>v. t.</pos> <def>To utter or express in a shrill tone; to cause to make a shrill sound.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>How poor Andromache <qex>shrills</qex> her dolors forth.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shrill-gorged</ent><br/
<hw>Shrill"-gorged`</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Having a throat which produces a shrill note.</def> <mark>[R.]</mark>  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shrillness</ent><br/
<hw>Shrill"ness</hw>, <pos>n.</pos> <def>The quality or state of being shrill.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shrill-tongued</ent><br/
<hw>Shrill"-tongued`</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Having a shrill voice.</def> <ldquo/When <xex>shrill-tongued</xex> Fulvia scolds.<rdquo/  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shrilly</ent><br/
<hw>Shril"ly</hw>, <pos>adv.</pos> <def>In a shrill manner; acutely; with a sharp sound or voice.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shrilly</ent><br/
<hw>Shril"ly</hw>, <pos>a.</pos> <def>Somewhat shrill.</def> <mark>[Poetic]</mark>  <rj><au>Sir W. Scott.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Some kept up a <qex>shrilly</qex> mellow sound.</q> <rj><qau>Keats.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shrimp</ent><br/
<hw>Shrimp</hw> <pr>(?)</pr>, <pos>v. t.</pos> <ety>[Cf. AS. <ets>scrimman</ets> to dry up, wither, MHG. <ets>schrimpfen</ets> to shrink, G. <ets>schrumpfen</ets>, Dan. <ets>skrumpe</ets>, <ets>skrumpes</ets>, Da. & Sw. <ets>skrumpen</ets> shriveled.  Cf. <er>Scrimp</er>, <er>Shrink</er>, <er>Shrivel</er>.]</ety> <def>To contract; to shrink.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shrimp</ent><br/
<hw>Shrimp</hw>, <pos>n.</pos> <ety>[OE. <ets>shrimp</ets>; -- probably so named from its shriveled appearance. See <er>Shrimp</er>, <pos>v.</pos>]</ety> <sn>1.</sn> <fld>(Zool.)</fld> <sd>(a)</sd> <def>Any one of numerous species of macruran Crustacea belonging to <gen>Crangon</gen> and various allied genera, having a slender body and long legs.  Many of them are used as food.  The larger kinds are called also <stype>prawns</stype>. See <xex>Illust.</xex> of <er>Decapoda</er>.</def> <sd>(b)</sd> <def>In a more general sense, any species of the macruran tribe <fam>Caridea</fam>, or any species of the order Schizopoda, having a similar form.</def> <sd>(c)</sd> <def>In a loose sense, any small crustacean, including some amphipods and even certain entomostracans; <as>as, the fairy <ex>shrimp</ex>, and brine <ex>shrimp</ex></as>. See under <er>Fairy</er>, and <er>Brine</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><-- p. 1335 --></p>

<p><sn>2.</sn> <def>Figuratively, a little wrinkled man; a dwarf; -- in contempt.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>This weak and writhled <qex>shrimp</qex>.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Opossum shrimp</b></col>. <fld>(Zool.)</fld> <cd>See under <er>Opossum</er>.</cd> -- <mcol><col><b>Spector shrimp</b></col>, <it>or</it> <col><b>Skeleton shrimp</b></col></mcol> <fld>(Zool.)</fld>, <cd>any slender amphipod crustacean of the genus <gen>Caprella</gen> and allied genera. See <xex>Illust.</xex> under <er>Laemodopoda</er>.</cd> -- <col><b>Shrimp catcher</b></col> <fld>(Zool.)</fld>, <cd>the little tern (<spn>Sterna minuta</spn>).</cd> -- <col><b>Shrimp net</b></col>, <cd>a dredge net fixed upon a pole, or a sweep net dragged over the fishing ground.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shrimper</ent><br/
<hw>Shrimp"er</hw> <pr>(?)</pr>, <pos>n.</pos> <def>One who fishes for shrimps.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shrine</ent><br/
<hw>Shrine</hw> <pr>(shr<imac/n)</pr>, <pos>n.</pos> <ety>[OE. <ets>schrin</ets>, AS. <ets>scr<imac/n</ets>, from L. <ets>scrinium</ets> a case, chest, box.]</ety> <sn>1.</sn> <def>A case, box, or receptacle, especially one in which are deposited sacred relics, as the bones of a saint.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Any sacred place, as an altar, tromb, or the like.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Too weak the sacred <qex>shrine</qex> guard.</q> <rj><qau>Byron.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>A place or object hallowed from its history or associations; <as>as, a <ex>shrine</ex> of art</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>Short for <col><b>Ancient Arabic Order of Nobles of the Mystic Shrine</b></col>, a secret fraternal organization professedly originated by one <person>Kalif Alu</person>, a son-in-law of Mohammed, at Mecca, in the year of the Hegira 25 (about 646 <sc>a. d.</sc>)  In the modern order, established in the United States in 1872, only Knights Templars or thirty-second degree Masons are eligible for admission, though the order itself is not Masonic.  A member of the order is popularly called a <member>Shriner</member>, and the order itself is sometimes called the <altname>Shriners</altname>.</def><br/
[<source>Webster 1913 Suppl.</source> <source>+PJC</source>]</p>

<p><ent>Shrine</ent><br/
<hw>Shrine</hw>, <pos>v. t.</pos> <def>To enshrine; to place reverently, as in a shrine.</def> <ldquo/<xex>Shrined</xex> in his sanctuary.<rdquo/  <rj><au>Milton.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shriner</ent><br/
<hw>Shrin"er</hw> <pr>(shr<imac/n"<etil/r)</pr>, <pos>n.</pos> <def>a member of the <membof>Ancient Arabic Order of Nobles of the Mystic Shrine</membof>.  See <er>shrine{4}</er>.</def><br/
[<source>PJC</source>]</p>

<p><ent>Shrink</ent><br/
<hw>Shrink</hw> <pr>(?)</pr>, <pos>v. i.</pos> <vmorph>[<pos>imp.</pos> <conjf>Shrank</conjf> <pr>(?)</pr> or <conjf>Shrunk</conjf> <pr>(?)</pr> <pos>p. p.</pos> <conjf>Shrunk</conjf> or <conjf>Shrunken</conjf> <pr>(<?/)</pr>, but the latter is now seldom used except as a participial adjective; <pos>p. pr. & vb. n.</pos> <conjf>Shrinking</conjf>.]</vmorph> <ety>[OE. <ets>shrinken</ets>, <ets>schrinken</ets>, AS. <ets>scrincan</ets>; akin to OD. <ets>schrincken</ets>, and probably to Sw. <ets>skrynka</ets> a wrinkle, <ets>skrynkla</ets> to wrinkle, to rumple, and E. <ets>shrimp</ets>, n. & v., <ets>scrimp</ets>. CF. <er>Shrimp</er>.]</ety> <sn>1.</sn> <def>To wrinkle, bend, or curl; to shrivel; hence, to contract into a less extent or compass; to gather together; to become compacted.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>And on a broken reed he still did stay<br/
His feeble steps, which <qex>shrunk</qex> when hard thereon he lay.</q> <rj><qau>Spenser.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>I have not found that water, by mixture of ashes, will <qex>shrink</qex> or draw into less room.</q> <rj><qau>Bacon.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Against this fire do I <qex>shrink</qex> up.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>And <qex>shrink</qex> like parchment in consuming fire.</q> <rj><qau>Dryden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>All the boards did <qex>shrink</qex>.</q> <rj><qau>Coleridge.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To withdraw or retire, as from danger; to decline action from fear; to recoil, as in fear, horror, or distress.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>What happier natures <qex>shrink</qex> at with affright,<br/
The hard inhabitant contends is right.</q> <rj><qau>Pope.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>They assisted us against the Thebans when you <qex>shrank</qex> from the task.</q> <rj><qau>Jowett (Thucyd.)</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>To express fear, horror, or pain by contracting the body, or part of it; to shudder; to quake.</def> <mark>[R.]</mark>  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shrink</ent><br/
<hw>Shrink</hw>, <pos>v. t.</pos> <sn>1.</sn> <def>To cause to contract or shrink; <as>as, to <ex>shrink</ex> finnel by imersing it in boiling water</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To draw back; to withdraw.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>The Libyc Hammon <qex>shrinks</qex> his horn.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>To shrink on</b></col> <fld>(Mach.)</fld>, <cd>to fix (one piece or part) firmly around (another) by natural contraction in cooling, as a tire on a wheel, or a hoop upon a cannon, which is made slightly smaller than the part it is to fit, and expanded by heat till it can be slipped into place.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shrink</ent><br/
<hw>Shrink</hw>, <pos>n.</pos> <sn>1.</sn> <def>The act shrinking; shrinkage; contraction; also, recoil; withdrawal.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Yet almost wish, with sudden <qex>shrink</qex>,<br/
That I had less to praise.</q> <rj><qau>Leigh Hunt.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <ety>[Contraction of head-<ets>shrink</ets>er, a colloquial term for psychiatrist.]</ety> <def>a psychiatrist.</def> <mark>[Coll.]</mark><br/
[<source>PJC</source>]</p>

<p><ent>Shrinkage</ent><br/
<hw>Shrink"age</hw> <pr>(?)</pr>, <pos>n.</pos> <sn>1.</sn> <def>The act of shrinking; a contraction into less bulk or measurement.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>The amount of such contraction; the bulk or dimension lost by shrinking, as of grain, castings, etc.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>Decrease in value; depreciation.</def> <mark>[Colloq.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shrinker</ent><br/
<hw>Shrink"er</hw> <pr>(?)</pr>, <pos>n.</pos> <def>One who shrinks; one who withdraws from danger.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shrinking</ent><br/
<hw>Shrink"ing</hw>, <def><pos>a. & n.</pos> from <er>Shrink</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Shrinking head</b></col> <fld>(Founding)</fld>, <cd>a body of molten metal connected with a mold for the purpose of supplying metal to compensate for the shrinkage of the casting; -- called also <altname>sinking head</altname>, and <altname>riser</altname>.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shrinkingly</ent><br/
<hw>Shrink"ing*ly</hw>, <pos>adv.</pos> <def>In a shrinking manner.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>shrinkpack</ent><br/
<ent>shrink-pack</ent><br/
<mhw><hw>shrink"-pack</hw>, <hw>shrink"pack</hw></mhw>, <pos>n.</pos> <def>same as <er>shrink-wrap</er>, <pos>n.</pos>.</def><br/
[<source>PJC</source>]</p>

<p><ent>shrinkwrap</ent><br/
<ent>shrink-wrap</ent><br/
<mhw><hw>shrink"-wrap</hw>, <hw>shrink"wrap</hw></mhw>  <pr>(shr<icr/<nsm/k"r<acr/p)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>shrink-wrapped</conjf> or <conjf>shrink"wrapped</conjf> <pr>(shr<icr/<nsm/k"r<acr/pt)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>shrink"wrapping</conjf> or <conjf>shrink"-wrapping</conjf>.]</vmorph> <def>to wrap and seal (an object for sale) in a transparent sheet of plastic material that is shrunken to a tight-fitting wrapping by the application of heat.</def><br/
[<source>PJC</source>]</p>

<p><ent>shrinkwrap</ent><br/
<ent>shrink-wrap</ent><br/
<mhw><hw>shrink"-wrap</hw>, <hw>shrink"wrap</hw></mhw>, <pos>n.</pos> <def>a type of plastic film, usually transparent, that shrinks upon application of heat, and may be used to form a wrapping around objects that fits tightly and closely follows the contours of the wrapped object; -- also called <altname>shrink-pack</altname>, <altname>shrinkpack</altname> or <altname>shrink-wrapping</altname>.</def><br/
[<source>PJC</source>]</p>

<p><ent>Shrivalty</ent><br/
<hw>Shriv"al*ty</hw> <pr>(?)</pr>, <pos>n.</pos> <def>Shrievalty.</def>  <rj><au>Johnson.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shrive</ent><br/
<hw>Shrive</hw> <pr>(?)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp.</pos> <conjf>Shrived</conjf> <pr>(?)</pr> or <conjf>Shrove</conjf> <pr>(<?/)</pr>; <pos>p. p.</pos> <conjf>Shriven</conjf> <pr>(?)</pr> or <conjf>Shrived</conjf>; <pos>p. pr. & vb. n.</pos> <conjf>Shriving</conjf>.]</vmorph> <ety>[OE. <ets>shriven</ets>, <ets>schriven</ets>, AS. <ets>scr<imac/van</ets> to shrive, to impose penance or punishment; akin to OFries. <ets>skr<imac/va</ets> to impose punishment; cf. OS. bi<ets>skr<imac/ban</ets> to be troubled.  Cf. <er>Shrift</er>, <er>Shrovetide</er>.]</ety> <sn>1.</sn> <def>To hear or receive the confession of; to administer confession and absolution to; -- said of a priest as the agent.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>That they should <qex>shrive</qex> their parishioners.</q> <rj><qau>Piers Plowman.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Doubtless he <qex>shrives</qex> this woman, . . . <br/
Else ne'er could he so long protract his speech.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Till my guilty soul be <qex>shriven</qex>.</q> <rj><qau>Longfellow.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To confess, and receive absolution; -- used reflexively.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Get you to the church and <qex>shrive</qex> yourself.</q> <rj><qau>Beau. & Fl.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shrive</ent><br/
<hw>Shrive</hw>, <pos>v. i.</pos> <def>To receive confessions, as a priest; to administer confession and absolution.</def>  <rj><au>Spenser.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shrivel</ent><br/
<hw>Shriv"el</hw> <pr>(?)</pr>, <pos>v. i.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Shriveled</conjf> <pr>(?)</pr> or <conjf>Shrivelled</conjf>; <pos>p. pr. & vb. n.</pos> <conjf>Shriveling</conjf> or <conjf>Shrivelling</conjf>.]</vmorph> <ety>[Probably akin to <ets>shrimp</ets>, <ets>shrink</ets>; cf. dial. AS. <ets>screpa</ets> to pine away, Norw. <ets>skrypa</ets> to waste, <ets>skryp</ets>, <ets>skryv</ets>, transitory, frail, Sw. <ets>skr<oum/pling</ets> feeble, Dan. <ets>skr<oum/belig</ets>, Icel. <ets>skrj<?/pr</ets> brittle, frail.]</ety> <def>To draw, or be drawn, into wrinkles; to shrink, and form corrugations; <as>as, a leaf <ex>shriveles</ex> in the hot sun; the skin <ex>shrivels</ex> with age</as>; -- often with <xex>up</xex>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shrivel</ent><br/
<hw>Shriv"el</hw> <pr>(?)</pr>, <pos>v. t.</pos> <def>To cause to shrivel or contract; to cause to shrink onto corruptions.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shriven</ent><br/
<hw>Shriv"en</hw> <pr>(?)</pr>, <def><pos>p. p.</pos> of <er>Shrive</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shriver</ent><br/
<hw>Shriv"er</hw> <pr>(?)</pr>, <pos>n.</pos> <def>One who shrives; a confessor.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shriving</ent><br/
<hw>Shriv"ing</hw>, <pos>n.</pos> <def>Shrift; confession.</def>  <rj><au>Spenser.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shroff</ent><br/
<hw>Shroff</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Ar. <ets>sarr<amac/f</ets>.]</ety> <def>A banker, or changer of money.</def> <mark>[East Indies]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shroffage</ent><br/
<hw>Shroff"age</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A money dealer's commission; also, more commonly, the examination of coins, and the separation of the good from the debased.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Shrood</ent><br/
<hw>Shrood</hw> <pr>(?)</pr>, <pos>v. t.</pos> <ety>[Cf. <er>Shroud</er>.]</ety> <altsp>[Written also <asp>shroud</asp>, and <asp>shrowd</asp>.]</altsp> <def>To trim; to lop.</def> <mark>[Prov. Eng.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shropshire</ent><br/
<hw>Shrop"shire</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[From <etsep>Shropshire</etsep>, country of England.]</ety> <def>An English breed of black-faced hornless sheep similar to the Southdown, but larger, now extensively raised in many parts of the world.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Shroud</ent><br/
<hw>Shroud</hw> <pr>(shroud)</pr>, <pos>n.</pos> <ety>[OE. <ets>shroud</ets>, <ets>shrud</ets>, <ets>schrud</ets>, AS. <ets>scr<umac/d</ets> a garment, clothing; akin to Icel.  <ets>skru<edh/</ets> the shrouds of a ship, furniture of a church, a kind of stuff, Sw. <ets>skrud</ets> dress, attire, and E. <ets>shred</ets>. See <er>Shred</er>, and cf. <er>Shrood</er>.]</ety> <sn>1.</sn> <def>That which clothes, covers, conceals, or protects; a garment.</def>  <rj><au>Piers Plowman.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Swaddled, as new born, in sable <qex>shrouds</qex>.</q> <rj><qau>Sandys.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Especially, the dress for the dead; a winding sheet.</def> <ldquo/A dead man in his <xex>shroud</xex>.<rdquo/  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>That which covers or shelters like a shroud.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Jura answers through her misty <qex>shroud</qex>.</q> <rj><qau>Byron.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>A covered place used as a retreat or shelter, as a cave or den; also, a vault or crypt.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>The <qex>shroud</qex> to which he won<br/
His fair-eyed oxen.</q> <rj><qau>Chapman.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>A vault, or <qex>shroud</qex>, as under a church.</q> <rj><qau>Withals.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <def>The branching top of a tree; foliage.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>The Assyrian wad a cedar in Lebanon, with fair branches and with a shadowing <qex>shroad</qex>.</q> <rj><qau>Ezek. xxxi. 3.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>6.</sn> <pluf>pl.</pluf> <fld>(Naut.)</fld> <def>A set of ropes serving as stays to support the masts. The lower shrouds are secured to the sides of vessels by heavy iron bolts and are passed around the head of the lower masts.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>7.</sn> <fld>(Mach.)</fld> <def>One of the two annular plates at the periphery of a water wheel, which form the sides of the buckets; a shroud plate.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Bowsprit shrouds</b></col> <fld>(Naut.)</fld>, <cd>ropes extending from the head of the bowsprit to the sides of the vessel.</cd> -- <col><b>Futtock shrouds</b></col> <fld>(Naut.)</fld>, <cd>iron rods connecting the topmast rigging with the lower rigging, passing over the edge of the top.</cd> -- <col><b>Shroud plate</b></col>. <sd>(a)</sd> <fld>(Naut.)</fld> <cd>An iron plate extending from the dead-eyes to the ship's side</cd>. <au>Ham. Nav. Encyc.</au> <sd>(b)</sd> <fld>(Mach.)</fld> <cd>A shroud. See def. 7, above.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shroud</ent><br/
<hw>Shroud</hw>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Shrouded</conjf>; <pos>p. pr. & vb. n.</pos> <conjf>Shrouding</conjf>.]</vmorph> <ety>[Cf. AS. <ets>scr<?/dan</ets>. See <er>Shroud</er>, <pos>n.</pos>]</ety> <sn>1.</sn> <def>To cover with a shroud; especially, to inclose in a winding sheet; to dress for the grave.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The ancient Egyptian mummies were <qex>shrouded</qex> in a number of folds of linen besmeared with gums.</q> <rj><qau>Bacon.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To cover, as with a shroud; to protect completely; to cover so as to conceal; to hide; to veil.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>One of these trees, with all his young ones, may <qex>shroud</qex> four hundred horsemen.</q> <rj><qau>Sir W. Raleigh.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Some tempest rise,<br/
And blow out all the stars that light the skies,<br/
To <qex>shroud</qex> my shame.</q> <rj><qau>Dryden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shroud</ent><br/
<hw>Shroud</hw>, <pos>v. i.</pos> <def>To take shelter or harbor.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>If your stray attendance be yet lodged,<br/
Or <qex>shroud</qex> within these limits.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shroud</ent><br/
<hw>Shroud</hw>, <pos>v. t.</pos> <def>To lop. See <er>Shrood</er>.</def> <mark>[Prov. Eng.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shrouded</ent><br/
<hw>Shroud"ed</hw>, <pos>a.</pos> <def>Provided with a shroud or shrouds.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Shrouded gear</b></col> <fld>(Mach.)</fld>, <cd>a cogwheel or pinion having flanges which form closed ends to the spaces between the teeth and thus strengthen the teeth by tying them together.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shrouding</ent><br/
<hw>Shroud"ing</hw>, <pos>n.</pos> <def>The shrouds. See <er>Shroud</er>, <pos>n.</pos>, 7.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shroud-laid</ent><br/
<hw>Shroud"-laid`</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Composed of four strands, and laid right-handed with a heart, or center; -- said of rope. See <xex>Illust.</xex> under <er>Cordage</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shroudless</ent><br/
<hw>Shroud"less</hw>, <pos>a.</pos> <def>Without a shroud.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shroudy</ent><br/
<hw>Shroud"y</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Affording shelter.</def> <mark>[R.]</mark>  <rj><au>Milton.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shrove</ent><br/
<hw>Shrove</hw> <pr>(?)</pr>, <def><pos>imp.</pos> of <er>Shrive</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Shrove Sunday</b></col>, <cd>Quinguagesima Sunday.</cd> -- <col><b>Shrove Tuesday</b></col>, <cd>the Tuesday following Quinguagesima Sunday, and preceding the first day of Lent, or Ash Wednesday.</cd> <note>It was formerly customary in England, on this day, for the people to confess their sins to their parish priests, after which they dined on pancakes, or fritters, and the occasion became one of merriment. The bell rung on this day is popularly called <xex>Pancake Bell</xex>, and the day itself <xex>Pancake Tuesday</xex>. <au>P. Cyc.</au></note></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shrove</ent><br/
<hw>Shrove</hw>, <pos>v. i.</pos> <def>To join in the festivities of Shrovetide; hence, to make merry.</def> <mark>[Obs.]</mark>  <rj><au>J. Fletcher.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shrovetide</ent><br/
<hw>Shrove"tide`</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[From <ets>shrive</ets> to take a confession (OE. imp. <ets>shrof</ets>, AS. <ets>scr<amac/f</ets>) + <ets>tide</ets>.]</ety> <def>The days immediately preceding Ash Widnesday, especially the period between the evening before Quinguagesima Sunday and the morning of Ash Wednesday.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shroving</ent><br/
<hw>Shrov"ing</hw>, <pos>n.</pos> <def>The festivity of Shrovetide.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shrow</ent><br/
<hw>Shrow</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A shrew.</def> <mark>[Obs.]</mark>  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shrowd</ent><br/
<hw>Shrowd</hw> <pr>(?)</pr>, <pos>v. t.</pos> <def>See <er>Shrood</er>.</def> <mark>[Prov. Eng.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shrub</ent><br/
<hw>Shrub</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Ar. <ets>shirb</ets>, <ets>shurb</ets>, a drink, beverage, fr. <ets>shariba</ets> to drink.  Cf. <er>Sirup</er>, <er>Sherbet</er>.]</ety> <def>A liquor composed of vegetable acid, especially lemon juice, and sugar, with spirit to preserve it.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shrub</ent><br/
<hw>Shrub</hw>, <pos>n.</pos> <ety>[OE. <ets>schrob</ets>, AS. <ets>scrob</ets>, <ets>scrobb</ets>; akin to Norw. <ets>skrubba</ets> the dwarf cornel tree.]</ety> <fld>(Bot.)</fld> <def>A woody plant of less size than a tree, and usually with several stems from the same root.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shrub</ent><br/
<hw>Shrub</hw>, <pos>v. t.</pos> <def>To lop; to prune.</def> <mark>[Obs.]</mark>  <rj><au>Anderson (1573).</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shrubbery</ent><br/
<hw>Shrub"ber*y</hw> <pr>(?)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Shrubberies</plw> <pr>(<?/)</pr>.</plu> <sn>1.</sn> <def>A collection of shrubs.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A place where shrubs are planted.</def>  <rj><au>Macaulay.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shrubbiness</ent><br/
<hw>Shrub"bi*ness</hw> <pr>(?)</pr>, <pos>n.</pos> <def>Quality of being shrubby.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shrubby</ent><br/
<hw>Shrub"by</hw> <pr>(?)</pr>, <pos>a.</pos> <amorph>[<pos>Compar.</pos> <adjf>Shrubbier</adjf> <pr>(?)</pr>; <pos>superl.</pos> <adjf>Shrubbiest</adjf>.]</amorph> <sn>1.</sn> <def>Full of shrubs.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Of the nature of a shrub; resembling a shrub.</def> <ldquo/<xex>Shrubby</xex> browse.<rdquo/  <rj><au>J. Philips.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shrubless</ent><br/
<hw>Shrub"less</hw>, <pos>a.</pos> <def>having no shrubs.</def>  <rj><au>Byron.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shruff</ent><br/
<hw>Shruff</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Cf. <er>Scruff</er>, <er>Scurf</er>.]</ety> <def>Rubbish.</def> <specif>Specifically:</specif> <sd>(a)</sd> <def>Dross or refuse of metals.</def> <mark>[Obs.]</mark> <sd>(b)</sd> <def>Light, dry wood, or stuff used for fuel.</def> <mark>[Prov. Eng.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shrug</ent><br/
<hw>Shrug</hw> <pr>(shr<ucr/g)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Shrugged</conjf> <pr>(shr<ucr/gd)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Shrugging</conjf> <pr>(shr<ucr/g"g<icr/ng)</pr>.]</vmorph> <ety>[Probably akin to <ets>shrink</ets>, p. p. <ets>shrunk</ets>; cf. Dan. <ets>skrugge</ets>, <ets>skrukke</ets>, to stoop, dial. Sw. <ets>skrukka</ets>, <ets>skruga</ets>, to crouch.]</ety> <def>To draw up or contract (the shoulders), especially by way of expressing doubt, indifference, dislike, dread, or the like.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>He <qex>shrugs</qex> his shoulders when you talk of securities.</q> <rj><qau>Addison.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shrug</ent><br/
<hw>Shrug</hw>, <pos>v. i.</pos> <def>To raise or draw up the shoulders, as in expressing doubt, indifference, dislike, dread, or the like.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>They grin, they <qex>shrug</qex>.<br/
They bow, they snarl, they snatch, they hug.</q> <rj><qau>Swift.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>shrug</ent><br/
<hw>shrug</hw>, <pos>n.</pos> <def>A gesture consisting of drawing up the shoulders, -- a motion usually expressing doubt, indifference, or dislike; -- it is sometimes accompanied by a slight turning of the hands outward or upward.  Such a gesture may be made, as in answering "who knows" to a question, suggesting utter ignorance of an answer and a disinclination to pursue the topic further.</def><br/
[<source>1913 Webster</source> <source>+PJC</source>]</p>

<p><q>On Sept. 23, in a major speech in New York, the chairman of the <org>Securities and Exchange Commision</org>, <person>Arthur Levitt</person> asked the <org>Big Board</org> to spike the rule [Rule 390] in the interest of free and unfettered markets. . . . 
   <person>Mr. Grasso</person> responded with a <qex>shrug</qex>, saying that he had no plans to kill the rule.</q> <rj><qau>Gretchen Morgenson (N. Y. Times Nov. 28, 1999 sect. 3 p. 1.</qau></rj><br/
[<source>PJC</source>]</p>

<p><q>The Spaniards talk in dialogues<br/
Of heads and shoulders, nods and <qex>shrugs</qex>.</q> <rj><qau>Hudibras.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>shrug off</ent><br/
<hw>shrug off</hw>, <pos>v. t.</pos> <def>To ignore; to disregard; to brush aside; to minimize{2} the effects of; <as>as, to <ex>shrug off</ex> predictions of disaster</as>.</def><br/
[<source>PJC</source>]</p>

<p><ent>Shrunken</ent><br/
<hw>Shrunk"en</hw> <pr>(?)</pr>, <def><pos>p. p. & a.</pos> from <er>Shrink</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>shtetl</ent><br/
<hw>shtetl</hw> <pr>(sht<ecr/t"'l)</pr>; <plu><it>pl.</it> Eng. <plw>shtetls</plw> <pr>(sht<ecr/t"'lz)</pr>, Yiddish <plw>shtetlach</plw>  <pr>(sht<ecr/t"l<aum/kh)</pr>.</plu>, <pos>n.</pos> <ety>[Yiddish, from MHG <ets>stetel</ets>, dim. of <ets>stat</ets>, place, town.]</ety> <def>A village or small town; -- usually referring to Jewish towns in Eastern Europe.</def><br/
[<source>PJC</source>]</p>

<p><ent>shtik</ent><br/
<ent>shtick</ent><br/
<mhw><hw>shtick</hw>, <hw>shtik</hw></mhw> <pr>(sht<icr/k)</pr>, <pos>n.</pos> <ety>[Yiddish, pranks; fr. MHG <ets>st<uum/cke</ets>, pieces.]</ety> <sn>1.</sn> <def>A person's special talent, line of business, or habitual activity.</def><br/
[<source>PJC</source>]</p>

<p><sn>2.</sn> <fld>(Show business)</fld> <def>A comic routine or a specific gag inserted in a show for laughs.</def><br/
[<source>PJC</source>]</p>

<p><ent>Shuck</ent><br/
<hw>Shuck</hw> <pr>(sh<ucr/k)</pr>, <pos>n.</pos> <def>A shock of grain.</def> <mark>[Prev. Eng.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shuck</ent><br/
<hw>Shuck</hw>, <pos>n.</pos> <ety>[Perhaps akin to G. <ets>shote</ets> a husk, pod, shell.]</ety> <sn>1.</sn> <def>A shell, husk, or pod; especially, the outer covering of such nuts as the hickory nut, butternut, peanut, and chestnut.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>The shell of an oyster or clam.</def> <mark>[U. S.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shuck</ent><br/
<hw>Shuck</hw>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Shucked</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Shucking</conjf>.]</vmorph> <sn>1.</sn> <def>To deprive of the shucks or husks; <as>as, to <ex>shuck</ex> walnuts, Indian corn, oysters, etc.</as></def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To remove or take off (shucks); hence, to discard; to lay aside; -- usually with <ptcl>off</ptcl>.</def> <mark>[Colloq.]</mark></p>

<p><q><ldquo/<qex>Shucking</qex><rdquo/ his coronet, after he had imbibed several draughts of fire water.</q>  <rj><qau>F. A. Ober.</qau></rj></p>

<p><q>He had only been in Africa long enough to <qex>shuck off</qex> the notions he had acquired about the engineering of a west coast colony.</q>  <rj><qau>Pall Mall Mag.</qau></rj><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Shucker</ent><br/
<hw>Shuck"er</hw> <pr>(?)</pr>, <pos>n.</pos> <def>One who shucks oysters or clams</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shudder</ent><br/
<hw>Shud"der</hw> <pr>(?)</pr>, <pos>v. i.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Shuddered</conjf> <pr>(?)</pr>;<pos>p. pr. & vb. n.</pos> <conjf>Shuddering</conjf>.]</vmorph> <ety>[OE. <ets>shoderen</ets>, <ets>schuderen</ets>; akin to LG. <ets>schuddern</ets>, D. <ets>schudden</ets> to shake, OS. <ets>skuddian</ets>, G. <ets>schaudern</ets> to shudder, <ets>sch<uum/tteln</ets> to shake, <ets>sch<uum/tten</ets> to pour, to shed, OHG. <ets>scutten</ets>, <ets>scuten</ets>, to shake.]</ety> <def>To tremble or shake with fear, horrer, or aversion; to shiver with cold; to quake.</def> <ldquo/With <xex>shuddering</xex> horror pale.<rdquo/  <rj><au>Milton.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>The <qex>shuddering</qex> tennant of the frigid zone.</q> <rj><qau>Goldsmith.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shudder</ent><br/
<hw>Shud"der</hw>, <pos>n.</pos> <def>The act of shuddering, as with fear.</def>  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shudderingly</ent><br/
<hw>Shud"der*ing*ly</hw>, <pos>adv.</pos> <def>In a shuddering manner.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shude</ent><br/
<hw>Shude</hw> <pr>(?)</pr>, <pos>n.</pos> <def>The husks and other refuse of rice mills, used to adulterate oil cake, or linseed cake.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shuffle</ent><br/
<hw>Shuf"fle</hw> <pr>(?)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Shuffled</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Shuffling</conjf> <pr>(?)</pr>.]</vmorph> <ety>[Originally the same word as <ets>scuffle</ets>, and properly a freq. of <ets>shove</ets>. See <er>Shove</er>, and <er>Scuffle</er>.]</ety> <sn>1.</sn> <def>To shove one way and the other; to push from one to another; <as>as, to <ex>shuffle</ex> money from hand to hand</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To mix by pushing or shoving; to confuse; to throw into disorder; especially, to change the relative positions of, as of the cards in a pack.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>A man may <qex>shuffle</qex> cards or rattle dice from noon to midnight without tracing a new idea in his mind.</q> <rj><qau>Rombler.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>To remove or introduce by artificial confusion.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>It was contrived by your enemies, and <qex>shuffled</qex> into the papers that were seizen.</q> <rj><qau>Dryden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>To shuffe off</b></col>, <cd>to push off; to rid one's self of.</cd> -- <col><b>To shuffe up</b></col>, <cd>to throw together in hastel to make up or form in confusion or with fraudulent disorder; <as>as, he <ex>shuffled up</ex> a peace</as>.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shuffle</ent><br/
<hw>Shuf"fle</hw>, <pos>v. i.</pos> <sn>1.</sn> <def>To change the relative position of cards in a pack; <as>as, to <ex>shuffle</ex> and cut</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To change one's position; to shift ground; to evade questions; to resort to equivocation; to prevaricate.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>I myself, . . . hiding mine honor in my necessity, am fain to <qex>shuffle</qex>.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>To use arts or expedients; to make shift.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Your life, good master,<br/
Must <qex>shuffle</qex> for itself.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>To move in a slovenly, dragging manner; to drag or scrape the feet in walking or dancing.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The aged creature came<br/
<qex>Shuffling</qex> along with ivory-headed wand.</q> <rj><qau>Keats.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><syn><b>Syn.</b> -- To equivicate; prevaricate; quibble; cavil; shift; sophisticate; juggle.</syn><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shuffle</ent><br/
<hw>Shuf"fle</hw>, <pos>n.</pos> <sn>1.</sn> <def>The act of shuffling; a mixing confusedly; a slovenly, dragging motion.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The unguided agitation and rude <qex>shuffles</qex> of matter.</q> <rj><qau>Bentley.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A trick; an artifice; an evasion.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The gifts of nature are beyond all shame and <qex>shuffles</qex>.</q> <rj><qau>L'Estrange.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shuffleboard</ent><br/
<hw>Shuf"fle*board`</hw> <pr>(?)</pr>, <pos>n.</pos> <def>See <er>Shovelboard</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shufflecap</ent><br/
<hw>Shuf"fle*cap`</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A play performed by shaking money in a hat or cap.</def> <mark>[R.]</mark>  <rj><au>Arbuthnot.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shuffler</ent><br/
<hw>Shuf"fler</hw> <pr>(?)</pr>, <pos>n.</pos> <sn>1.</sn> <def>One who shuffles.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Zool.)</fld> <def>Either one of the three common American scaup ducks. See <xex>Scaup duck</xex>, under <er>Scaup</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shufflewing</ent><br/
<hw>Shuf"fle*wing`</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Zool.)</fld> <def>The hedg sparrow.</def> <mark>[Prov. Eng.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shuffling</ent><br/
<hw>Shuf"fling</hw> <pr>(?)</pr>, <pos>a.</pos> <sn>1.</sn> <def>Moving with a dragging, scraping step.</def> <ldquo/A <xex>shuffling</xex> nag.<rdquo/  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Evasive; <as>as, a <ex>shuffling</ex> excuse</as>.</def>  <rj><au>T. Burnet.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shuffling</ent><br/
<hw>Shuf"fling</hw>, <pos>v.</pos> <def>In a shuffling manner.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shug</ent><br/
<hw>Shug</hw> <pr>(?)</pr>, <pos>v. i.</pos> <ety>[Cf. <er>Shrug</er>.]</ety> <sn>1.</sn> <def>To writhe the body so as to produce friction against one's clothes, as do those who have the itch.</def> <mark>[Prov. Eng.]</mark>  <rj><au>Halliwell.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Hence, to crawl; to sneak.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>There I 'll <qex>shug</qex> in and get a noble countenance.</q> <rj><qau>Ford.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shumac</ent><br/
<hw>Shu"mac</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Bot.)</fld> <def>Sumac.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shun</ent><br/
<hw>Shun</hw> <pr>(?)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Shunned</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Shunning</conjf>.]</vmorph> <ety>[OE. <ets>shunien</ets>, <ets>schunien</ets>, <ets>schonien</ets>, AS. <ets>scunian</ets>, <ets>sceonian</ets>; cf. D. <ets>schuinen</ets> to slepe, <ets>schuin</ets> oblique, sloping, Icel. <ets>skunda</ets>, <ets>skynda</ets>, to hasten.  Cf. <er>Schooner</er>, <er>Scoundrel</er>, <er>Shunt</er>.]</ety> <def>To avoid; to keep clear of; to get out of the way of; to escape from; to eschew; <as>as, to <ex>shun</ex> rocks, shoals, vice</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>I am pure from the blood of all men. For I have not shunned to declare unto you all the counsel of God.</q> <rj><qau>Acts xx. 26,27.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Scarcity and want shall <qex>shun</qex> you.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><syn><b>Syn.</b> -- See <er>Avoid</er>.</syn><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shunless</ent><br/
<hw>Shun"less</hw>, <pos>a.</pos> <def>Not to be shunned; inevitable; unavoidable.</def> <mark>[R.]</mark> <ldquo/<xex>Shunless</xex> destiny.<rdquo/  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><-- p. 1336 --></p>

<p><ent>Shunt</ent><br/
<hw>Shunt</hw> <pr>(?)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Shunted</conjf>; <pos>p. pr. & vb. n.</pos> <conjf>Shunting</conjf>.]</vmorph> <ety>[Prov. E., to move from, to put off, fr. OE. <ets>shunten</ets>, <ets>schunten</ets>, <ets>schounten</ets>; cf. D. <ets>schuinte</ets> a slant, slope, Icel. <ets>skunda</ets> to hasten.  Cf. <er>Shun</er>.]</ety> <sn>1.</sn> <def>To shun; to move from.</def> <mark>[Obs. or Prov. Eng.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To cause to move suddenly; to give a sudden start to; to shove.</def> <mark>[Obs. or Prov. Eng.]</mark>  <rj><au>Ash.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>To turn off to one side; especially, to turn off, as a grain or a car upon a side track; to switch off; to shift.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>For <qex>shunting</qex> your late partner on to me.</q> <rj><qau>T. Hughes.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <fld>(Elec.)</fld> <def>To provide with a shunt; <as>as, to <ex>shunt</ex> a galvanometer</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shunt</ent><br/
<hw>Shunt</hw> <pr>(?)</pr>, <pos>v. i.</pos> <def>To go aside; to turn off.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shunt</ent><br/
<hw>Shunt</hw>, <pos>n.</pos> <ety>[Cf. D. <ets>schuinte</ets> slant, slope, declivity. See <er>Shunt</er>, <pos>v. t.</pos>]</ety> <sn>1.</sn> <fld>(Railroad)</fld> <def>A turning off to a side or short track, that the principal track may be left free.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Elec.)</fld> <def>A conducting circuit joining two points in a conductor, or the terminals of a galvanometer or dynamo, so as to form a parallel or derived circuit through which a portion of the current may pass, for the purpose of regulating the amount passing in the main circuit.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <fld>(Gunnery)</fld> <def>The shifting of the studs on a projectile from the deep to the shallow sides of the grooves in its discharge from a shunt gun.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Shunt dynamo</b></col> <fld>(Elec.)</fld>, <cd>a dynamo in which the field circuit is connected with the main circuit so as to form a shunt to the letter, thus employing a portion of the current from the armature to maintain the field.</cd> -- <col><b>Shunt gun</b></col>, <cd>a firearm having shunt rifling. See under <er>Rifling</er>.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shunter</ent><br/
<hw>Shunt"er</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Railroad)</fld> <def>A person employed to shunt cars from one track to another.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shunting</ent><br/
<hw>Shunt"ing</hw>, <pos>p. pr. & vb. n.</pos> <mord>of <er>Shunt</er>.</mord> <specif>Specif.:</specif> <pos>vb. n.</pos> <sd>(a)</sd> <fld>(Railroads)</fld> <def>Switching; <as>as, <ex>shunting</ex> engine, yard, etc.</as></def> <mark>[British]</mark> <sd>(b)</sd> <fld>(Finance)</fld> <def>Arbitrage conducted between certain local markets without the necessity of the exchange involved in foreign arbitrage.</def> <mark>[Great Britain]</mark><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Shunt valve</ent><br/
<hw>Shunt valve</hw>. <fld>(Mach.)</fld> <def>A valve permitting a fluid under pressure an easier avenue of escape than normally; specif., a valve, actuated by the governor, used in one system of marine-engine governing to connect both ends of the low-pressure cylinder as a supplementary control.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Shunt winding</ent><br/
<hw>Shunt winding</hw>. <fld>(Elec.)</fld> <def>A winding so arranged as to divide the armature current and lead a portion of it around the field-magnet coils; -- opposed to <contr>series winding</contr>.</def> -- <wordforms><wf>Shunt"-wound`</wf> <pr>(#)</pr>, <pos>a.</pos></wordforms><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Shut</ent><br/
<hw>Shut</hw> <pr>(?)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Shut</conjf>; <pos>p. pr. & vb. n.</pos> <conjf>Shutting</conjf>.]</vmorph> <ety>[OE. <ets>shutten</ets>, <ets>schutten</ets>, <ets>shetten</ets>, <ets>schitten</ets>, AS. <ets>scyttan</ets> to shut or lock up (akin to D. <ets>schutten</ets>, G. <ets>sch<uum/tzen</ets> to protect), properly, to fasten with a bolt or bar <ets>shot</ets> across, fr. AS. <ets>sce<oacute/tan</ets> to shoot. <root/159. See <er>Shoot</er>.]</ety> <sn>1.</sn> <def>To close so as to hinder ingress or egress; <as>as, to <ex>shut</ex> a door or a gate; to <ex>shut</ex> one's eyes or mouth.</as></def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To forbid entrance into; to prohibit; to bar; <as>as, to <ex>shut</ex> the ports of a country by a blockade</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Shall that be <qex>shut</qex> to man which to the beast<br/
Is open?</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>To preclude; to exclude; to bar out.</def> <ldquo/<xex>Shut</xex> from every shore.<rdquo/  <rj><au>Dryden.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>To fold together; to close over, as the fingers; to close by bringing the parts together; <as>as, to <ex>shut</ex> the hand; to <ex>shut</ex> a book.</as></def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>To shut in</b></col>. <sd>(a)</sd> <cd>To inclose; to confine</cd>. <ldquo/The Lord <xex>shut</xex> him <xex>in</xex>.<rdquo/ <au>Cen. vii. 16.</au> <sd>(b)</sd> <cd>To cover or intercept the view of; as, one point <xex>shuts in</xex> another.</cd> -- <col><b>To shut off</b></col>. <sd>(a)</sd> <cd>To exclude</cd>. <sd>(b)</sd> <cd>To prevent the passage of, as steam through a pipe, or water through a flume, by closing a cock, valve, or gate.</cd> -- <col><b>To shut out</b></col>, <cd>to preclude from entering; to deny admission to; to exclude; <as>as, <ex>to shut out</ex> rain by a tight roof</as>.</cd> -- <col><b>To shut together</b></col>, <cd>to unite; to close, especially to close by welding.</cd> -- <col><b>To shut up</b></col>. <sd>(a)</sd> <cd>To close; to make fast the entrances into; <as>as, <ex>to shut up</ex> a house</as>.</cd> <sd>(b)</sd> <cd>To obstruct</cd>. <ldquo/Dangerous rocks <xex>shut up</xex> the passage.<rdquo/ <au>Sir W. Raleigh.</au> <sd>(c)</sd> <cd>To inclose; to confine; to imprison; to fasten in; <as>as, <ex>to shut up</ex> a prisoner</as>.</cd><br/
[<source>1913 Webster</source>]</p>

<p><q>Before faith came, we were kept under the law, <qex>shut up</qex> unto the faith which should afterwards be revealed.</q> <rj><qau>Gal. iii. 23.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sd>(d)</sd> <cd>To end; to terminate; to conclude</cd>.<br/
[<source>1913 Webster</source>]</p>

<p><q>When the scene of life is <qex>shut up</qex>, the slave will be above his master if he has acted better.</q> <rj><qau>Collier.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sd>(e)</sd> <cd>To unite, as two pieces of metal by welding.</cd> <sd>(f)</sd> <cd>To cause to become silent by authority, argument, or force</cd>.
</cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shut</ent><br/
<hw>Shut</hw>, <pos>v. i.</pos> <def>To close itself; to become closed; <as>as, the door <ex>shuts</ex>; it <ex>shuts</ex> hard.</as></def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>To shut up</b></col>, <cd>to cease speaking.</cd> <mark>[Colloq.]</mark>  <rj><au>T. Hughes.</au></rj></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shut</ent><br/
<hw>Shut</hw>, <pos>a.</pos> <sn>1.</sn> <def>Closed or fastened; <as>as, a <ex>shut</ex> door</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Rid; clear; free; <as>as, to get <ex>shut</ex> of a person</as>.</def> <mark>[Now dialectical or local, Eng. & U.S.]</mark>  <rj><au>L'Estrange.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <fld>(Phon.)</fld> <sd>(a)</sd> <def>Formed by complete closure of the mouth passage, and with the nose passage remaining closed; stopped, as are the mute consonants, <it>p</it>, <it>t</it>, <it>k</it>, <it>b</it>, <it>d</it>, and hard <it>g</it>.</def> <au>H. Sweet.</au> <sd>(b)</sd> <def>Cut off sharply and abruptly by a following consonant in the same syllable, as the English short vowels, <acr/, <ecr/, <icr/, <ocr/, <ucr/, always are.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shut</ent><br/
<hw>Shut</hw>, <pos>n.</pos> <def>The act or time of shutting; close; <as>as, the <ex>shut</ex> of a door</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Just then returned at <qex>shut</qex> of evening flowers.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A door or cover; a shutter.</def> <mark>[Obs.]</mark>  <rj><au>Sir I. Newton.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>The line or place where two pieces of metal are united by welding.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Cold shut</b></col>, <cd>the imperfection in a casting caused by the flowing of liquid metal upon partially chilled metal; also, the imperfect weld in a forging caused by the inadequate heat of one surface under working.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shute</ent><br/
<hw>Shute</hw> <pr>(?)</pr>, <pos>n.</pos> <def>Same as <er>Chute</er>, or <er>Shoot</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shutter</ent><br/
<hw>Shut"ter</hw> <pr>(?)</pr>, <pos>n.</pos> <sn>1.</sn> <def>One who shuts or closes.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A movable cover or screen for a window, designed to shut out the light, to obstruct the view, or to be of some strength as a defense; a blind.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>A removable cover, or a gate, for closing an aperture of any kind, as for closing the passageway for molten iron from a ladle.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <fld>(Photog.)</fld> <def>A mechanical device of various forms, attached to the aperture of a camera lens for opening and closing to expose the plate.  It is usually designed so that the time during which the aperture is opened may be varied by a manual dial or by some automatic mechanism, thereby allowing proper exposure of a photographic film under different intensities of light.</def><br/
[<source>Webster 1913 Suppl.</source> <source>+PJC</source>]</p>

<p><ent>Shuttered</ent><br/
<hw>Shut"tered</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Furnished with shutters.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shuttle</ent><br/
<hw>Shut"tle</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Also <ets>shittle</ets>, OE. <ets>schitel</ets>, <ets>scytyl</ets>, <ets>schetyl</ets>; cf. OE. <ets>schitel</ets> a bolt of a door, AS. <ets>scyttes</ets>; all from AS. <ets>sce<oacute/tan</ets> to shoot; akin to Dan. <ets>skyttel</ets>, <ets>skytte</ets>, shuttle, dial. Sw. <ets>skyttel</ets>, <ets>sk<oum/ttel</ets>. <root/159. See <er>Shoot</er>, and cf. <er>Shittle</er>, <er>Skittles</er>.]</ety> <sn>1.</sn> <def>An instrument used in weaving for passing or shooting the thread of the woof from one side of the cloth to the other between the threads of the warp.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Like <qex>shuttles</qex> through the loom, so swiftly glide<br/
My feathered hours.</q> <rj><qau>Sandys.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>The sliding thread holder in a sewing machine, which carries the lower thread through a loop of the upper thread, to make a lock stitch.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>A shutter, as for a channel for molten metal.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Shuttle box</b></col> <fld>(Weaving)</fld>, <cd>a case at the end of a shuttle race, to receive the shuttle after it has passed the thread of the warp; also, one of a set of compartments containing shuttles with different colored threads, which are passed back and forth in a certain order, according to the pattern of the cloth woven.</cd> -- <col><b>Shutten race</b></col>, <cd>a sort of shelf in a loom, beneath the warp, along which the shuttle passes; a channel or guide along which the shuttle passes in a sewing machine.</cd> -- <col><b>Shuttle shell</b></col> <fld>(Zool.)</fld>, <cd>any one of numerous species of marine gastropods of the genus <gen>Volva</gen>, or <gen>Radius</gen>, having a smooth, spindle-shaped shell prolonged into a channel at each end.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shuttle</ent><br/
<hw>Shut"tle</hw> <pr>(?)</pr>, <pos>v. i.</pos> <def>To move backwards and forwards, like a shuttle.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>I had to fly far and wide, <qex>shutting</qex> athwart the big Babel, wherever his calls and pauses had to be.</q> <rj><qau>Carlyle.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shuttlecock</ent><br/
<hw>Shut"tle*cock`</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A cork stuck with feathers, which is to be struck by a battledoor in play; also, the play itself.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shuttlecock</ent><br/
<hw>Shut"tle*cock</hw>, <pos>v. t.</pos> <def>To send or toss to and fro; to bandy; <as>as, to <ex>shuttlecock</ex> words</as>.</def>  <rj><au>Thackeray.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shuttlecork</ent><br/
<hw>Shut"tle*cork`</hw> <pr>(?)</pr>, <pos>n.</pos> <def>See <er>Shuttlecock</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shuttlewise</ent><br/
<hw>Shut"tle*wise`</hw> <pr>(?)</pr>, <pos>adv.</pos> <def>Back and forth, like the movement of a shuttle.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shwan-pan</ent><br/
<hw>Shwan"-pan</hw> <pr>(?)</pr>, <pos>n.</pos> <def>See <er>Schwan-pan</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shy</ent><br/
<hw>Shy</hw> <pr>(sh<imac/)</pr>, <pos>a.</pos> <amorph>[<pos>Compar.</pos> <adjf>Shier</adjf> <pr>(sh<imac/"<etil/r)</pr> or <adjf>Shyer</adjf>; <pos>superl.</pos> <adjf>Shiest</adjf> or <adjf>Shyest</adjf>.]</amorph> <ety>[OE. <ets>schey</ets>, <ets>skey</ets>, <ets>sceouh</ets>, AS. <ets>sce<oacute/h</ets>; akin to Dan. <ets>sky</ets>, Sw. <ets>skygg</ets>, D. <ets>schuw</ets>, MHG. <ets>schiech</ets>, G. <ets>scheu</ets>, OHG. <ets>sciuhen</ets> to be or make timid.  Cf. <er>Eschew</er>.]</ety> <sn>1.</sn> <def>Easily frightened; timid; <as>as, a <ex>shy</ex> bird</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The horses of the army . . . were no longer <qex>shy</qex>, but would come up to my very feet without starting.</q> <rj><qau>Swift.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Reserved; coy; disinclined to familiar approach.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>What makes you so <qex>shy</qex>, my good friend? There's nobody loves you better than I.</q> <rj><qau>Arbuthnot.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>The embarrassed look of <qex>shy</qex> distress<br/
And maidenly shamefacedness.</q> <rj><qau>Wordsworth.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>Cautious; wary; suspicious.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>I am very <qex>shy</qex> of using corrosive liquors in the preparation of medicines.</q> <rj><qau>Boyle.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Princes are, by wisdom of state, somewhat <qex>shy</qex> of thier successors.</q> <rj><qau>Sir H. Wotton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>Inadequately supplied; short; lacking; <as>as, the team is <ex>shy</ex> two players</as>.</def><mark>[Slang]</mark><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><sn>5.</sn> <fld>(Poker)</fld>, <def>owing money to the pot; -- in cases where an opponent's bet has exceeded a player's available stake or chips, but the player chooses to continue playing the hand before adding the required bet to the pot.</def> <mark>[Slang]</mark><br/
[<source>PJC</source>]</p>

<p><cs><col><b>To fight shy</b></col>. <cd>See under <er>Fight</er>, <pos>v. i.</pos></cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shy</ent><br/
<hw>Shy</hw>, <pos>v. i.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Shied</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Shying</conjf>.]</vmorph> <ety>[From <er>Shy</er>, <pos>a.</pos>]</ety> <def>To start suddenly aside through fright or suspicion; -- said especially of horses.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shy</ent><br/
<hw>Shy</hw>, <pos>v. t.</pos> <def>To throw sidewise with a jerk; to fling; <as>as, to <ex>shy</ex> a stone; to <ex>shy</ex> a slipper</as>.</def>  <rj><au>T. Hughes.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shy</ent><br/
<hw>Shy</hw>, <pos>n.</pos> <sn>1.</sn> <def>A sudden start aside, as by a horse.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A side throw; a throw; a fling.</def>  <rj><au>Thackeray.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>If Lord Brougham gets a stone in his hand, he must, it seems, have a <qex>shy</qex> at somebody.</q> <rj><qau>Punch.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shyly</ent><br/
<hw>Shy"ly</hw>, <pos>adv.</pos> <def>In a shy or timid manner; not familiarly; with reserve.</def> <altsp>[Written also <asp>shily</asp>.]</altsp><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shyness</ent><br/
<hw>Shy"ness</hw>, <pos>n.</pos> <def>The quality or state of being shy.</def> <altsp>[Written also <asp>shiness</asp>.]</altsp><br/
[<source>1913 Webster</source>]</p>

<p><q>Frequency in heavenly contemplation is particularly important to prevent a <qex>shyness</qex> bewtween God and thy soul.</q> <rj><qau>Baxter.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><syn><b>Syn.</b> -- Bashfulness; reserve; coyness; timidity; diffidence. See <er>Bashfulness</er>.</syn><br/
[<source>1913 Webster</source>]</p>

<p><ent>Shyster</ent><br/
<hw>Shy"ster</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Perh. from G. <ets>scheisse</ets> excrement.]</ety> <def>A trickish knave; one who carries on any business, especially legal business, in a mean and dishonest way.</def> <mark>[Slang, U.S.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Si</ent><br/
<hw>Si</hw> <pr>(?)</pr>. <ety>[It.]</ety> <fld>(Mus.)</fld> <def>A syllable applied, in solmization, to the note B; more recently, to the seventh tone of any major diatonic scale. It was added to Guido's scale by Le Maire about the end of the 17th century.</def><-- now called ti --><br/
[<source>1913 Webster</source>]</p>

<p><ent>Siaga</ent><br/
||<hw>Si*a"ga</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Zool.)</fld> <def>The ahu, or jairou.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sialogogue</ent><br/
<hw>Si*al"o*gogue</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Gr. <grk>si`alon</grk> saliva + <?/<?/<?/<?/ leading, from <?/<?/<?/ to lead: cf. F. <ets>sialagogue</ets>.]</ety> <fld>(Med.)</fld> <def>An agent which promotes the flow of saliva.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Siamang</ent><br/
||<hw>Si"a*mang`</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Malay <ets>si<amac/mang</ets>.]</ety> <fld>(Zool.)</fld> <def>A gibbon (<spn>Hylobates syndactylus</spn>), native of Sumatra. It has the second and third toes partially united by a web.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Siamese</ent><br/
<hw>Si`a*mese"</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Of or pertaining to Siam, its native people, or their language.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Siamese</ent><br/
<hw>Si`a*mese`</hw>, <pos>n. sing. & pl.</pos> <sn>1.</sn> <def>A native or inhabitant of Siam; <pluf>pl.</pluf>, the people of Siam.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <singf>sing.</singf> <def>The language of the Siamese.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sib</ent><br/
<hw>Sib</hw> <pr>(s<icr/b)</pr>, <pos>n.</pos> <ety>[AS. <ets>sibb</ets> alliance, <ets>gesib</ets> a relative. <root/289.  See <er>Gossip</er>.]</ety> <sn>1.</sn> <def>A blood relation.</def> <mark>[Obs.]</mark>  <rj><au>Nash.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>a <er>sibling</er>.</def><br/
[<source>PJC</source>]</p>

<p><ent>Sib</ent><br/
<hw>Sib</hw>, <pos>a.</pos> <def>Related by blood; akin.</def> <mark>[Obs. or Prov. Eng. & Scot.]</mark>  <rj><au>Sir W. Scott.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Your kindred is but . . . little <qex>sib</qex> to you.</q> <rj><qau>Chaucer.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>[He] is no fairy birn, ne <qex>sib</qex> at all<br/
To elfs, but sprung of seed terrestrial.</q> <rj><qau>Spenser.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sibbens</ent><br/
<hw>Sib"bens</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Etymol. uncertain.]</ety> <fld>(Med.)</fld> <def>A contagious disease, endemic in Scotland, resembling the yaws. It is marked by ulceration of the throat and nose and by pustules and soft fungous excrescences upon the surface of the body. In the Orkneys the name is applied to the itch.</def> <altsp>[Written also <asp>sivvens</asp>.]</altsp><br/
[<source>1913 Webster</source>]</p>

<p><ent>Siberian</ent><br/
<hw>Si*be"ri*an</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[From <ets>Siberia</ets>, Russ. <ets>Sibire</ets>.]</ety> <def>Of or pertaining to Siberia, a region comprising all northern Asia and belonging to Russia; <as>as, a <ex>Siberian</ex> winter</as>.</def> -- <def2><pos>n.</pos> <def>A native or inhabitant of Siberia.</def></def2><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Siberian crab</b></col> <fld>(Bot.)</fld>, <cd>the Siberian crab apple. See <cref>Crab apple</cref>, under <er>Crab</er>.</cd> -- <col><b>Siberian dog</b></col> <fld>(Zool.)</fld>, <cd>one of a large breed of dogs having erect ears and the hair of the body and tail very long. It is distinguished for endurance of fatigue when used for the purpose of draught.</cd> -- <col><b>Siberian pea tree</b></col> <fld>(Bot.)</fld>, <cd>a small leguminous tree (<spn>Cragana arborescens</spn>) with yellow flowers. It is a native of Siberia.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sibilancy</ent><br/
<ent>Sibilance</ent><br/
<mhw>{ <hw>Sib"i*lance</hw> <pr>(?)</pr>, <hw>Sib"i*lan*cy</hw> <pr>(?)</pr>, }</mhw> <pos>n.</pos> <def>The quality or state of being sibilant; sibilation.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Milton would not have avoided them for their <qex>sibilancy</qex>, he who wrote . . . verses that hiss like Medusa's head in wrath.</q> <rj><qau>Lowell.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sibilant</ent><br/
<hw>Sib"i*lant</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>sibilans</ets>, <ets>-antis</ets>, p. pr. of <ets>sibilare</ets> to hiss: cf. F. <ets>sibilant</ets>.]</ety> <def>Making a hissing sound; uttered with a hissing sound; hissing; <as>as, <it>s</it>, <it>z</it>, <it>sh</it>, and <it>zh</it>, are <ex>sibilant</ex> elementary sounds</as>.</def> -- <def2><pos>n.</pos> <def>A sibiliant letter.</def></def2><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sibilate</ent><br/
<hw>Sib"i*late</hw> <pr>(?)</pr>, <pos>v. t. & i.</pos> <def>To pronounce with a hissing sound, like that of the letter <it>s</it>; to mark with a character indicating such pronunciation.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sibilation</ent><br/
<hw>Sib`i*la"tion</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>sibilatio</ets>.]</ety> <def>Utterance with a hissing sound; also, the sound itself; a hiss.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>He, with a long, low <qex>sibilation</qex>, stared.</q> <rj><qau>Tennyson.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sibilatory</ent><br/
<hw>Sib"i*la*to*ry</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Hissing; sibilant.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sibilous</ent><br/
<hw>Sib"i*lous</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>sibilus</ets>.]</ety> <def>Having a hissing sound; hissing; sibilant.</def> <mark>[R.]</mark>  <rj><au>Pennant.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>sibling</ent><br/
<hw>sib"ling</hw> <ety>[<ets>sub</ets> + <ets>-ling</ets>.]</ety> <pr>(s<icr/b"l<icr/ng)</pr>, <pos>n.</pos> <def>a brother or a sister.</def> <note>Siblings have at least one parent in common.  Those related only by a common mother are <stype>uterine siblings</stype>; those related only by a common father are <stype>agnate siblings</stype> or <stype>consanguine siblings</stype> (a legal term).  A sibling having both parents in common is a <stype>sibling-german</stype> or a <stype>full brother</stype> or <stype>full sister</stype>.  These modifying terms are more commonly used for the more specific <stype>uterine brother</stype>, <stype>uterine sister</stype>, <stype>agnate brother</stype>, <stype>brother-german</stype>, etc.</note><br/
[<source>PJC</source>]</p>

<p><ent>sibling</ent><br/
<hw>sib"ling</hw> <pr>(s<icr/b"l<icr/ng)</pr>, <pos>a.</pos> <def>of or pertaining to a <er>sibling</er>, <pos>n.</pos>; <as>as, <ex>sibling</ex> rivalry: the common rivalry between siblings.</as></def><br/
[<source>PJC</source>]</p>

<p><ent>Sibyl</ent><br/
<hw>Sib"yl</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>sibylla</ets>, Gr. <?/<?/<?/<?/.]</ety> <sn>1.</sn> <fld>(Class. Antiq.)</fld> <def>A woman supposed to be endowed with a spirit of prophecy.</def><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ The number of the sibyls is variously stated by different authors; but the opinion of Varro, that there were ten, is generally adopted. They dwelt in various parts of Persia, Greece, and Italy.</note><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A female fortune teller; a pythoness; a prophetess.</def> <ldquo/An old highland <xex>sibyl</xex>.<rdquo/  <rj><au>Sir W. Scott.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sibylist</ent><br/
<hw>Sib"yl*ist</hw>, <pos>n.</pos> <def>One who believes in a sibyl or the sibylline prophecies.</def>  <rj><au>Cudworth.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sibylline</ent><br/
<hw>Sib"yl*line</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>sibyllinus</ets>.]</ety> <def>Pertaining to the sibyls; uttered, written, or composed by sibyls; like the productions of sibyls.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Sibylline books</b></col>. <sd>(a)</sd> <fld>(Rom. Antiq.)</fld> <cd>Books or documents of prophecies in verse concerning the fate of the Roman empire, said to have been purchased by Tarquin the Proud from a sibyl.</cd> <sd>(b)</sd> <cd>Certain Jewish and early Christian writings purporting to have been prophetic and of sibylline origin. They date from 100 <sc>b. c.</sc> to <sc>a. d.</sc> 500.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sic</ent><br/
<hw>Sic</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Such.</def> <mark>[Scot.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sic</ent><br/
||<hw>Sic</hw> <pr>(?)</pr>, <pos>adv.</pos> <ety>[L.]</ety> <def>Thus.</def><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ This word is sometimes inserted in a quotation [<it>sic</it>], to call attention to the fact that some remarkable or inaccurate expression, misspelling, or the like, is literally reproduced.</note><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sicamore</ent><br/
<hw>Sic"a*more</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Bot.)</fld> <def>See <er>Sycamore</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sicca</ent><br/
||<hw>Sic"ca</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Ar. <ets>sikka</ets>.]</ety> <def>A seal; a coining die; -- used adjectively to designate the silver currency of the Mogul emperors, or the Indian rupee of 192 grains.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Sicca rupee</b></col>, <cd>an East Indian coin, valued nominally at about two shillings sterling, or fifty cents.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Siccate</ent><br/
<hw>Sic"cate</hw> <pr>(?)</pr>, <pos>v. t.</pos> <ety>[L. <ets>siccatus</ets>, p. p. of <ets>siccare</ets> to dry, fr. <ets>siecus</ets> dry.]</ety> <def>To dry.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Siccation</ent><br/
<hw>Sic*ca"tion</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>siccatio</ets>.]</ety> <def>The act or process of drying.</def> <mark>[R.]</mark>  <rj><au>Bailey.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Siccative</ent><br/
<hw>Sic"ca*tive</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>siccativus</ets>.]</ety> <def>Drying; causing to dry.</def> -- <def2><pos>n.</pos> <def>That which promotes drying.</def></def2><br/
[<source>1913 Webster</source>]</p>

<p><ent>Siccific</ent><br/
<hw>Sic*cif"ic</hw> <pr>(?)</pr>, <pos>a.</pos><ety>[L. <ets>siccificus</ets>; <ets>siccus</ets> dry + <ets>facere</ets> to make. See <er>-fy</er>.]</ety> <def>Causing dryness.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Siccity</ent><br/
<hw>Sic"ci*ty</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>siccitas</ets>, fr. <ets>siccus</ets> dry.]</ety> <def>Dryness; aridity; destitution of moisture.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>The <qex>siccity</qex> and dryness of its flesh.</q> <rj><qau>Sir T. Browne.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sice</ent><br/
<hw>Sice</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F. <ets>six</ets>, fr. L. <ets>sex</ets> six.  See <er>Six</er>.]</ety> <def>The number six at dice.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sicer</ent><br/
<hw>Si"cer</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>sicera</ets>. See <er>Cider</er>.]</ety> <def>A strong drink; cider.</def> <mark>[Obs.]</mark>  <rj><au>Chaucer.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sich</ent><br/
<hw>Sich</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Such.</def> <mark>[Obs. or Colloq.]</mark>  <rj><au>Spenser.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sicilian</ent><br/
<hw>Si*cil"i*an</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Of or pertaining to Sicily or its inhabitants.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Sicilian vespers</b></col>, <cd>the great massacre of the French in Sicily, in the year 1282, on the evening of Easter Monday, at the hour of vespers.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sicilian</ent><br/
<hw>Si*cil"i*an</hw>, <pos>n.</pos> <def>A native or inhabitant of Sicily.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Siciliano</ent><br/
||<hw>Si*ci`li*a"no</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[It., Sicilian.]</ety> <def>A Sicilian dance, resembling the pastorale, set to a rather slow and graceful melody in 12-8 or 6-8 measure; also, the music to the dance.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sicilienne</ent><br/
||<hw>Si`ci`lienne"</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F., fem. of <ets>sicilien</ets> Sicilian.]</ety> <def>A kind of rich poplin.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sick</ent><br/
<hw>Sick</hw> <pr>(?)</pr>, <pos>a.</pos> <amorph>[<pos>Compar.</pos> <adjf>Sicker</adjf> <pr>(?)</pr>; <pos>superl.</pos> <adjf>Sickest</adjf>.]</amorph> <ety>[OE. <ets>sek</ets>, <ets>sik</ets>, ill, AS. <ets>se<oacute/c</ets>; akin to OS. <ets>siok</ets>, <ets>seoc</ets>, OFries. <ets>siak</ets>, D. <ets>ziek</ets>, G. <ets>siech</ets>, OHG. <ets>sioh</ets>, Icel. <ets>sj<?/kr</ets>, Sw. <ets>sjuk</ets>, Dan. <ets>syg</ets>, Goth. <ets>siuks</ets> ill, <ets>siukan</ets> to be ill.]</ety> <sn>1.</sn> <def>Affected with disease of any kind; ill; indisposed; not in health. See the Synonym under <er>Illness</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Simon's wife's mother lay <qex>sick</qex> of a fever.</q> <rj><qau>Mark i. 30.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Behold them that are <qex>sick</qex> with famine.</q> <rj><qau>Jer. xiv. 18.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Affected with, or attended by, nausea; inclined to vomit; <as>as, <ex>sick</ex> at the stomach; a <ex>sick</ex> headache.</as></def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>Having a strong dislike; disgusted; surfeited; -- with <xex>of</xex>; <as>as, to be <ex>sick</ex> of flattery</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>He was not so <qex>sick</qex> of his master as of his work.</q> <rj><qau>L'Estrange.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>Corrupted; imperfect; impaired; weakned.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>So great is his antipathy against episcopacy, that, if a seraphim himself should be a bishop, he would either find or make some <qex>sick</qex> feathers in his wings.</q> <rj><qau>Fuller.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Sick bay</b></col> <fld>(Naut.)</fld>, <cd>an apartment in a vessel, used as the ship's hospital.</cd> -- <col><b>Sick bed</b></col>, <cd>the bed upon which a person lies sick.</cd> -- <col><b>Sick berth</b></col>, <cd>an apartment for the sick in a ship of war.</cd> -- <col><b>Sick headache</b></col> <fld>(Med.)</fld>, <cd>a variety of headache attended with disorder of the stomach and nausea.</cd> -- <col><b>Sick list</b></col>, <cd>a list containing the names of the sick.</cd> -- <col><b>Sick room</b></col>, <cd>a room in which a person lies sick, or to which he is confined by sickness.</cd> <note>[These terms, <xex>sick bed</xex>, <xex>sick berth</xex>, etc., are also written both hyphened and solid.]</note></cs><br/
[<source>1913 Webster</source>]</p>

<p><syn><b>Syn.</b> -- Diseased; ill; disordered; distempered; indisposed; weak; ailing; feeble; morbid.</syn><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sick</ent><br/
<hw>Sick</hw>, <pos>n.</pos> <def>Sickness.</def> <mark>[Obs.]</mark>  <rj><au>Chaucer.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sick</ent><br/
<hw>Sick</hw>, <pos>v. i.</pos> <def>To fall sick; to sicken.</def> <mark>[Obs.]</mark>  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sick-brained</ent><br/
<hw>Sick"-brained`</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Disordered in the brain.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sicken</ent><br/
<hw>Sick"en</hw> <pr>(?)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Sickened</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Sickening</conjf>.]</vmorph> <sn>1.</sn> <def>To make sick; to disease.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Raise this strength, and <qex>sicken</qex> that to death.</q> <rj><qau>Prior.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To make qualmish; to nauseate; to disgust; <as>as, to <ex>sicken</ex> the stomach</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>To impair; to weaken.</def> <mark>[Obs.]</mark>  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sicken</ent><br/
<hw>Sick"en</hw>, <pos>v. i.</pos> <sn>1.</sn> <def>To become sick; to fall into disease.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The judges that sat upon the jail, and those that attended, <qex>sickened</qex> upon it and died.</q> <rj><qau>Bacon.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To be filled to disgust; to be disgusted or nauseated; to be filled with abhorrence or aversion; to be surfeited or satiated.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Mine eyes did <qex>sicken</qex> at the sight.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><-- p. 1337 --></p>

<p><sn>3.</sn> <def>To become disgusting or tedious.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The toiling pleasure <qex>sickens</qex> into pain.</q> <rj><qau>Goldsmith.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>To become weak; to decay; to languish.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>All pleasures <qex>sicken</qex>, and all glories sink.</q> <rj><qau>Pope.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sickening</ent><br/
<hw>Sick"en*ing</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Causing sickness; specif., causing surfeit or disgust; nauseating.</def> -- <wordforms><wf>Sick"en*ing*ly</wf>, <pos>adv.</pos></wordforms><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sicker</ent><br/
<hw>Sick"er</hw> <pr>(?)</pr>, <pos>v. i.</pos> <ety>[AS. <ets>sicerian</ets>.]</ety> <fld>(Mining)</fld> <def>To percolate, trickle, or ooze, as water through a crack.</def> <altsp>[Also written <asp>sigger</asp>, <asp>zigger</asp>, and <asp>zifhyr</asp>.]</altsp> <mark>[Prov. Eng.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Siker</ent><br/
<ent>Sicker</ent><br/
<mhw>{ <hw>Sick"er</hw>, <hw>Sik"er</hw>  }</mhw>, <pos>a.</pos> <ety>[OE. <ets>siker</ets>; cf. OS. <ets>sikur</ets>, LG. <ets>seker</ets>, D. <ets>zeker</ets>, Dan. <ets>sikker</ets>, OHG. <ets>sihhur</ets>, G. <ets>sicher</ets>; all fr. L. <ets>securus</ets>. See <er>Secure</er>, <er>Sure</er>.]</ety> <def>Sure; certain; trusty.</def> <mark>[Obs. or Prov. Eng. & Scot.]</mark>  <rj><au>Burns.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>When he is <qex>siker</qex> of his good name.</q> <rj><qau>Chaucer.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Siker</ent><br/
<ent>Sicker</ent><br/
<mhw>{ <hw>Sick"er</hw>, <hw>Sik"er</hw>  }</mhw>, <pos>adv.</pos> <def>Surely; certainly.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>Believe this as <qex>siker</qex> as your creed.</q> <rj><qau>Chaucer.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q><qex>Sicker</qex>, Willye, thou warnest well.</q> <rj><qau>Spenser.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sikerly</ent><br/
<ent>Sickerly</ent><br/
<mhw>{ <hw>Sick"er*ly</hw>, <hw>Sik"er*ly</hw>  }</mhw>, <pos>adv.</pos> <def>Surely; securely.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>But <qex>sikerly</qex>, withouten any fable.</q> <rj><qau>Chaucer.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sikerness</ent><br/
<ent>Sickerness</ent><br/
<mhw>{ <hw>Sick"er*ness</hw>, <hw>Sik"er*ness</hw>  }</mhw>, <pos>n.</pos> <def>The quality or state of being sicker, or certain.</def> <mark>[Obs.]</mark>  <rj><au>Chaucer. Spenser.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sickish</ent><br/
<hw>Sick"ish</hw>, <pos>a.</pos> <sn>1.</sn> <def>Somewhat sick or diseased.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Somewhat sickening; <as>as, a <ex>sickish</ex> taste</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p>-- <wordforms><wf>Sick"ish*ly</wf>, <pos>adv.</pos> -- <wf>Sick"ish*ness</wf>, <pos>n.</pos></wordforms><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sickle</ent><br/
<hw>Sic"kle</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[OE. <ets>sikel</ets>, AS. <ets>sicol</ets>; akin to D. <ets>sikkel</ets>, G. <ets>sichel</ets>, OHG. <ets>sihhila</ets>, Dan. <ets>segel</ets>, <ets>segl</ets>, L. <ets>secula</ets>, fr. <ets>secare</ets> to cut; or perhaps from L. <ets>secula</ets>. See <er>Saw</er> a cutting instrument.]</ety> <sn>1.</sn> <def>A reaping instrument consisting of a steel blade curved into the form of a hook, and having a handle fitted on a tang. The sickle has one side of the blade notched, so as always to sharpen with a serrated edge.  Cf. <cref>Reaping hook</cref>, under <er>Reap</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>When corn has once felt the <qex>sickle</qex>, it has no more benefit from the sunshine.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Astron.)</fld> <def>A group of stars in the constellation Leo. See <xex>Illust.</xex> of <er>Leo</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Sickle pod</b></col> <fld>(Bot.)</fld>, <cd>a kind of rock cress (<spn>Arabis Canadensis</spn>) having very long curved pods.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sicklebill</ent><br/
<hw>Sic"kle*bill`</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Zool.)</fld> <sd>(a)</sd> <def>Any one of three species of humming birds of the genus <gen>Eutoxeres</gen>, native of Central and South America. They have a long and strongly curved bill. Called also the <altname>sickle-billed hummer</altname>.</def> <sd>(b)</sd> <def>A curlew.</def> <sd>(c)</sd> <def>A bird of the genus <gen>Epimachus</gen> and allied genera.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sickled</ent><br/
<hw>Sic"kled</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Furnished with a sickle.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sickleman</ent><br/
<hw>Sic"kle*man</hw> <pr>(?)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Sicklemen</plw> <pr>(<?/)</pr>.</plu> <def>One who uses a sickle; a reaper.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>You sunburned <qex>sicklemen</qex>, of August weary.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sickler</ent><br/
<hw>Sic"kler</hw> <pr>(?)</pr>, <pos>n.</pos> <def>One who uses a sickle; a sickleman; a reaper.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sickless</ent><br/
<hw>Sick"less</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Free from sickness.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>Give me long breath, young beds, and <qex>sickless</qex> ease.</q> <rj><qau>Marston.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sicklewort</ent><br/
<hw>Sic"kle*wort`</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[AS. <ets>sicolwyrt</ets>.]</ety> <fld>(Bot.)</fld> <sd>(a)</sd> <def>A plant of the genus <gen>Coronilla</gen> (<spn>Coronilla scorpioides</spn>); -- so named from its curved pods.</def> <sd>(b)</sd> <def>The healall (<spn>Brunella vulgaris</spn>).</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sicklied</ent><br/
<hw>Sick"lied</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Made sickly. See <er>Sickly</er>, <pos>v.</pos></def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sickliness</ent><br/
<hw>Sick"li*ness</hw> <pr>(?)</pr>, <pos>n.</pos> <def>The quality or state of being sickly.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sickly</ent><br/
<hw>Sick"ly</hw> <pr>(?)</pr>, <pos>a.</pos> <amorph>[<pos>Compar.</pos> <adjf>Sicklier</adjf> <pr>(?)</pr>; <pos>superl.</pos> <adjf>Sickliest</adjf>.]</amorph> <sn>1.</sn> <def>Somewhat sick; disposed to illness; attended with disease; <as>as, a <ex>sickly</ex> body</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>This physic but prolongs thy <qex>sickly</qex> days.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Producing, or tending to, disease; <as>as, a <ex>sickly</ex> autumn; a <ex>sickly</ex> climate</as>.</def>  <rj><au>Cowper.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>Appearing as if sick; weak; languid; pale.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The moon grows <qex>sickly</qex> at the sight of day.</q> <rj><qau>Dryden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Nor torrid summer's <qex>sickly</qex> smile.</q> <rj><qau>Keble.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>Tending to produce nausea; sickening; <as>as, a <ex>sickly</ex> smell; <ex>sickly</ex> sentimentality.</as></def><br/
[<source>1913 Webster</source>]</p>

<p><syn><b>Syn.</b> -- Diseased; ailing; infirm; weakly; unhealthy; healthless; weak; feeble; languid; faint.</syn><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sickly</ent><br/
<hw>Sick"ly</hw>, <pos>adv.</pos> <def>In a sick manner or condition; ill.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>My people <qex>sickly</qex> [with ill will] beareth our marriage.</q> <rj><qau>Chaucer.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sickly</ent><br/
<hw>Sick"ly</hw>, <pos>v. t.</pos> <def>To make sick or sickly; -- with <xex>over</xex>, and probably only in the past participle.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q><qex>Sicklied</qex> o'er with the pale cast of thought.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Sentiments <qex>sicklied</qex> over . . . with that cloying heaviness into which unvaried sweetness is too apt to subside.</q> <rj><qau>Jeffrey.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sickness</ent><br/
<hw>Sick"ness</hw>, <pos>n.</pos> <ety>[AS. <ets>se<oacute/cness</ets>.]</ety> <sn>1.</sn> <def>The quality or state of being sick or diseased; illness; sisease or malady.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>I do lament the <qex>sickness</qex> of the king.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Trust not too much your now resistless charms;<br/
Those, age or <qex>sickness</qex> soon or late disarms.</q> <rj><qau>Pope.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Nausea; qualmishness; <as>as, <ex>sickness</ex> of stomach</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><syn><b>Syn.</b> -- Illness; disease; malady. See <er>Illness</er>.</syn><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sicle</ent><br/
<hw>Si"cle</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F., fr. L. <ets>silcus</ets>, Heb. <ets>shegel</ets>. See <er>Shekel</er>.]</ety> <def>A shekel.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>The holy mother brought five <qex>sicles</qex> and a pair of turtledoves to redeem the Lamb of God.</q> <rj><qau>Jer. Taylor.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sida</ent><br/
||<hw>Si"da</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[NL., fr. Gr. <?/<?/<?/ a kind of plant.]</ety> <fld>(Bot.)</fld> <def>A genus of malvaceous plants common in the tropics. All the species are mucilaginous, and some have tough ligneous fibers which are used as a substitute for hemp and flax.</def>  <rj><au>Balfour (Cyc. of India).</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Siddow</ent><br/
<hw>Sid"dow</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Soft; pulpy.</def> <mark>[Obs. or Prov. Eng.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Side</ent><br/
<hw>Side</hw> <pr>(s<imac/d)</pr>, <pos>n.</pos> <ety>[AS. <ets>s<imac/de</ets>; akin to D. <ets>zijde</ets>, G. <ets>seite</ets>, OHG. <ets>s<imac/ta</ets>, Icel. <ets>s<imac/<?/a</ets>, Dan. <ets>side</ets>, Sw. <ets>sida</ets>; cf. AS. <ets>s<imac/d</ets> large, spacious, Icel. <ets>s<imac/<?/r</ets> long, hanging.]</ety> <sn>1.</sn> <def>The margin, edge, verge, or border of a surface; especially (when the thing spoken of is somewhat oblong in shape), one of the longer edges as distinguished from the shorter edges, called <xex>ends</xex>; a bounding line of a geometrical figure; <as>as, the <ex>side</ex> of a field, of a square or triangle, of a river, of a road, etc.</as></def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>Any outer portion of a thing considered apart from, and yet in relation to, the rest; <as>as, the upper <ex>side</ex> of a sphere</as>; also, any part or position viewed as opposite to or contrasted with another; <as>as, this or that <ex>side</ex></as>.</def>
<-- any part of the surface which can be viewed from one vantage point. --><br/
[<source>1913 Webster</source>]</p>

<p><q>Looking round on every <qex>side</qex> beheld<br/
A pathless desert.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <sd>(a)</sd> <def>One of the halves of the body, of an animals or man, on either side of the mesial plane; or that which pertains to such a half; <as>as, a <ex>side</ex> of beef; a <ex>side</ex> of sole leather.</as></def> <sd>(b)</sd> <def>The right or left part of the wall or trunk of the body; <as>as, a pain in the <ex>side</ex></as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>One of the soldiers with a spear pierced his <qex>side</qex>.</q> <rj><qau>John xix. 34.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <def>A slope or declivity, as of a hill, considered as opposed to another slope over the ridge.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Along the <qex>side</qex> of yon small hill.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>6.</sn> <def>The position of a person or party regarded as opposed to another person or party, whether as a rival or a foe; a body of advocates or partisans; a party; hence, the interest or cause which one maintains against another; a doctrine or view opposed to another.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>God on our <qex>side</qex>, doubt not of victory.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>We have not always been of the . . . same <qex>side</qex> in politics.</q> <rj><qau>Landor.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Sets the passions on the <qex>side</qex> of truth.</q> <rj><qau>Pope.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>7.</sn> <def>A line of descent traced through one parent as distinguished from that traced through another.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>To sit upon thy father David's throne,<br/
By mother's <qex>side</qex> thy father.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>8.</sn> <def>Fig.: Aspect or part regarded as contrasted with some other; <as>as, the bright <ex>side</ex> of poverty</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>By the side of</b></col>, <cd>close at hand; near to.</cd> -- <col><b>Exterior side</b></col>. <fld>(Fort.)</fld> <cd>See <er>Exterior</er>, and <xex>Illust.</xex> of <er>Ravelin</er>.</cd> -- <col><b>Interior side</b></col> <fld>(Fort.)</fld>, <cd>the line drawn from the center of one bastion to that of the next, or the line curtain produced to the two oblique radii in front.</cd> <au>H. L. Scott.</au> -- <col><b>Side by side</b></col>, <cd>close together and abreast; in company or along with.</cd> -- <col><b>To choose sides</b></col>, <cd>to select those who shall compete, as in a game, on either side.</cd> -- <col><b>To take sides</b></col>, <cd>to attach one's self to, or give assistance to, one of two opposing sides or parties.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Side</ent><br/
<hw>Side</hw> <pr>(?)</pr>, <pos>a.</pos> <sn>1.</sn> <def>Of or pertaining to a side, or the sides; being on the side, or toward the side; lateral.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>One mighty squadron with a <qex>side</qex> wind sped.</q> <rj><qau>Dryden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Hence, indirect; oblique; collateral; incidental; <as>as, a <ex>side</ex> issue; a <ex>side</ex> view or remark.</as></def><br/
[<source>1913 Webster</source>]</p>

<p><q>The law hath no <qex>side</qex> respect to their persons.</q> <rj><qau>Hooker.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <ety>[AS. <ets>s<imac/d</ets>. Cf <er>Side</er>, <pos>n.</pos>]</ety> <def>Long; large; extensive.</def> <mark>[Obs. or Scot.]</mark>  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>His gown had <qex>side</qex> sleeves down to mid leg.</q> <rj><qau>Laneham.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Side action</b></col>, <cd>in breech-loading firearms, a mechanism for operating the breech block, which is moved by a lever that turns sidewise.</cd> -- <col><b>Side arms</b></col>, <cd>weapons worn at the side, as sword, bayonet, pistols, etc.</cd> -- <col><b>Side ax</b></col>, <cd>an ax of which the handle is bent to one side.</cd> -- <col><b>Side-bar rule</b></col> <fld>(Eng. Law.)</fld>, <cd>a rule authorized by the courts to be granted by their officers as a matter of course, without formal application being made to them in open court; -- so called because anciently moved for by the attorneys at <xex>side bar</xex>, that is, informally.</cd> <au>Burril.</au> -- <col><b>Side box</b></col>, <cd>a box or inclosed seat on the side of a theater.</cd><br/
[<source>1913 Webster</source>]</p>

<p><q>To insure a <qex>side-box</qex> station at half price.</q> <rj><qau>Cowper.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p>-- <col><b>Side chain</b></col>, <sd>(a)</sd> <cd>one of two safety chains connecting a tender with a locomotive, at the sides.</cd> <sd>(b)</sd> <fld>(Chem.)</fld> <cd>a chain of atoms attached to the main structure of a large molecule, especially of a polymer.</cd> -- <col><b>Side cut</b></col>, <cd>a canal or road branching out from the main one.</cd> <mark>[U.S.]</mark> -- <col><b>Side dish</b></col>, <cd>one of the dishes subordinate to the main course.</cd> -- <col><b>Side glance</b></col>, <cd>a glance or brief look to one side.</cd> -- <col><b>Side hook</b></col> <fld>(Carp.)</fld>, <cd>a notched piece of wood for clamping a board to something, as a bench.</cd> -- <col><b>Side lever</b></col>, <cd> a working beam of a side-lever engine.</cd> -- <col><b>Side-lever engine</b></col>, <cd>a marine steam engine having a working beam of each side of the cylinder, near the bottom of the engine, communicating motion to a crank that is above them.</cd> -- <col><b>Side pipe</b></col> <fld>(Steam Engine)</fld>, <cd>a steam or exhaust pipe connecting the upper and lower steam chests of the cylinder of a beam engine.</cd> -- <col><b>Side plane</b></col>, <cd>a plane in which the cutting edge of the iron is at the side of the stock.</cd> -- <col><b>Side posts</b></col> <fld>(Carp.)</fld>, <cd>posts in a truss, usually placed in pairs, each post set at the same distance from the middle of the truss, for supporting the principal rafters, hanging the tiebeam, etc.</cd> -- <col><b>Side rod</b></col>. <sd>(a)</sd> <cd>One of the rods which connect the piston-rod crosshead with the side levers, in a side-lever engine.</cd> <sd>(b)</sd> <cd>See <cref>Parallel rod</cref>, under <er>Parallel</er>.</cd> -- <col><b>Side screw</b></col> <fld>(Firearms)</fld>, <cd>one of the screws by which the lock is secured to the side of a firearm stock.</cd> -- <col><b>Side table</b></col>, <cd>a table placed either against the wall or aside from the principal table.</cd> -- <col><b>Side tool</b></col> <fld>(Mach.)</fld>, <cd>a cutting tool, used in a lathe or planer, having the cutting edge at the side instead of at the point.</cd> -- <col><b>Side wind</b></col>, <cd>a wind from one side; hence, an indirect attack, or indirect means.</cd> <au>Wright.</au></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Side</ent><br/
<hw>Side</hw>, <pos>v. i.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Sided</conjf>; <pos>p. pr. & vb. n.</pos> <conjf>Siding</conjf>.]</vmorph> <sn>1.</sn> <def>To lean on one side.</def> <mark>[Obs.]</mark>  <rj><au>Bacon.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To embrace the opinions of one party, or engage in its interest, in opposition to another party; to take sides; <as>as, to <ex>side</ex> with the ministerial party</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>All <qex>side</qex> in parties, and begin the attack.</q> <rj><qau>Pope.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Side</ent><br/
<hw>Side</hw>, <pos>v. t.</pos> <sn>1.</sn> <def>To be or stand at the side of; to be on the side toward.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>His blind eye that <qex>sided</qex> Paridell.</q> <rj><qau>Spenser.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To suit; to pair; to match.</def> <mark>[Obs.]</mark>  <rj><au>Clarendon.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <fld>(Shipbuilding)</fld> <def>To work (a timber or rib) to a certain thickness by trimming the sides.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>To furnish with a siding; <as>as, to <ex>side</ex> a house</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sideboard</ent><br/
<hw>Side"board`</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A piece of dining-room furniture having compartments and shelves for keeping or displaying articles of table service.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>At a stately <qex>sideboard</qex>, by the wine,<br/
That fragrant smell diffused.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sidebone</ent><br/
<hw>Side"bone`</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Far.)</fld> <def>A morbid growth or deposit of bony matter and at the sides of the coronet and coffin bone of a horse.</def>  <rj><au>J. H. Walsh.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Side-chain theory</ent><br/
<hw>Side"-chain` the`o*ry</hw>. <fld>(Physiol. Chem.)</fld> <def>A theory proposed by <person>P. Ehrlich</person> as a chemical explanation of immunity phenomena. In brief outline it is as follows: Animal cells and bacteria are complex aggregations of molecules, which are themselves complex. Complex molecules react with one another through certain of their side chains, but only when these side chains have a definite correspondence in structure (this accounts for the specific action of antitoxins).</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Sided</ent><br/
<hw>Sid"ed</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Having (such or so many) sides; -- used in composition; <as>as, one-<ex>sided</ex>; many-<ex>sided</ex>.</as></def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sideflash</ent><br/
<hw>Side"flash`</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Elec.)</fld> <def>A disruptive discharge between a conductor traversed by an oscillatory current of high frequency (as lightning) and neighboring masses of metal, or between different parts of the same conductor.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Sidehill</ent><br/
<hw>Side"hill`</hw> <pr>(?)</pr>, <pos>n.</pos> <def>The side or slope of a hill; sloping ground; a descent.</def> <mark>[U. S.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Side line</ent><br/
<ent>Sideline</ent><br/
<mhw><hw>Sideline</hw>, <hw>Side line</hw></mhw> <pos>n.</pos> <sn>1.</sn> <sd>(a)</sd> <def>A line pertaining or attached to the side of a thing.</def> <sd>(b)</sd> <def>Specif., a line for hobbling an animal by connecting the fore and the hind feet of the same side.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><sn>2.</sn>  <def>A line of goods sold in addition to one's principal articles of trade; a course of business pursued aside from one's regular occupation.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><sn>3.</sn> <def>A secondary road; esp., a byroad at right angles to a main road.</def> <mark>[Canada]</mark><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><sn>4.</sn> <fld>(Sports)</fld>  <def>Either of two marked lines which delimit the side boundaries of a playing field, as in football or soccer.</def><br/
[<source>PJC</source>]</p>

<p><sn>5.</sn> <fld>(Sports)</fld> <mark>[plural]</mark> <def>The areas in a sports arena just beyond the sidelines{4}, where coaches and alternative players sit or stand.</def><br/
[<source>PJC</source>]</p>

<p><cs><col><b>on the sidelines</b></col> <cd>not actively involved in an activity, such as a sporting event, but available for use when needed; in reserve; <as>as, the coach kept him <ex>on the sidelines</ex> for a week</as>.</cd></cs><br/
[<source>PJC</source>]</p>

<p><ent>Sideline</ent><br/
<hw>Sideline</hw>,  <pos>v. t.</pos> <def>To render unable to participate in an activity, such as a sporting event; <as>as, the pitcher was <ex>sidelined</ex> by a groin injury</as>.</def><br/
[<source>PJC</source>]</p>

<p><ent>Sideling</ent><br/
<hw>Side"ling</hw> <pr>(?)</pr>, <pos>adv.</pos> <ety>[OE. <ets>sideling</ets>, fr. <ets>side</ets> side. See <er>Side</er>, and cf. <er>Sidelong</er>, <er>Headlong</er>.]</ety> <def>Sidelong; on the side; laterally; also, obliquely; askew.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>A fellow nailed up maps . . . some <qex>sideling</qex>, and others upside down.</q> <rj><qau>Swift.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sideling</ent><br/
<hw>Side"ling</hw>, <pos>a.</pos> <def>Inclining to one side; directed toward one side; sloping; inclined; <as>as, <ex>sideling</ex> ground</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sidelong</ent><br/
<hw>Side"long`</hw> <pr>(?)</pr>, <pos>adv.</pos> <ety>[See <er>Sideling</er>, <pos>adv.</pos>]</ety> <sn>1.</sn> <def>Laterally; obliquely; in the direction of the side.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>On the side; <as>as, to lay a thing <ex>sidelong</ex></as>.</def> <note>[See <er>Sideling</er>, <pos>adv.</pos> ]</note>  <rj><au>Evelyn.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sidelong</ent><br/
<hw>Side"long`</hw>, <pos>a.</pos> <def>Lateral; oblique; not being directly in front; <as>as, a <ex>sidelong</ex> glance</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The bashful virgin's <qex>sidelong</qex> looks of love.</q> <rj><qau>Goldsmith.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sidepiece</ent><br/
<hw>Side"piece`</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Joinery)</fld> <def>The jamb, or cheek, of an opening in a wall, as of door or window.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sider</ent><br/
<hw>Sid"er</hw> <pr>(?)</pr>, <pos>n.</pos> <def>One who takes a side.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sider</ent><br/
<hw>Si"der</hw> <pr>(?)</pr>, <pos>n.</pos> <def>Cider.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sideral</ent><br/
<hw>Sid"er*al</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>sideralis</ets>. See <er>Sidereal</er>.]</ety> <sn>1.</sn> <def>Relating to the stars.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Astrol.)</fld> <def>Affecting unfavorably by the supposed influence of the stars; baleful.</def> <ldquo/<xex>Sideral</xex> blast.<rdquo/  <rj><au>Milton.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Siderated</ent><br/
<hw>Sid"er*a`ted</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>sideratus</ets>, p. p. of <ets>siderari</ets> to be blasted by a constellation, fr. <ets>sidus</ets>, <ets>sideris</ets>, a constellation.]</ety> <def>Planet-struck; blasted.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sideration</ent><br/
<hw>Sid`er*a"tion</hw>, <pos>n.</pos> <ety>[L. <ets>sideratio</ets>.]</ety> <def>The state of being siderated, or planet-struck; esp., blast in plants; also, a sudden and apparently causeless stroke of disease, as in apoplexy or paralysis.</def> <mark>[Obs.]</mark>  <rj><au>Ray.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sidereal</ent><br/
<hw>Si*de"re*al</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>sidereus</ets>, from <ets>sidus</ets>, <ets>sideris</ets>, a constellation, a star.  Cf. <er>Sideral</er>, <er>Consider</er>, <er>Desire</er>.]</ety> <sn>1.</sn> <def>Relating to the stars; starry; astral; <as>as, <ex>sidereal</ex> astronomy</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Astron.)</fld> <def>Measuring by the apparent motion of the stars; designated, marked out, or accompanied, by a return to the same position in respect to the stars; <as>as, the <ex>sidereal</ex> revolution of a planet; a <ex>sidereal</ex> day.</as></def><br/
[<source>1913 Webster</source>]</p>

<p><cs><mcol><col><b>Sidereal clock</b></col>, <col><b>day</b></col>, <col><b>month</b></col>, <col><b>year</b></col></mcol>. <cd>See under <er>Clock</er>, <er>Day</er>, etc.</cd> -- <col><b>Sideral time</b></col>, <cd>time as reckoned by sideral days, or, taking the sidereal day as the unit, the time elapsed since a transit of the vernal equinox, reckoned in parts of a sidereal day. This is, strictly, <xex>apparent sidereal time</xex>, <xex>mean sidereal time</xex> being reckoned from the transit, not of the <xex>true</xex>, but of the <xex>mean</xex>, equinoctial point.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Siderealize</ent><br/
<hw>Si*de"re*al*ize</hw> <pr>(?)</pr>, <pos>v. t.</pos> <def>To elevate to the stars, or to the region of the stars; to etherealize.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>German literature transformed, <qex>siderealized</qex>, as we see it in Goethe, reckons Winckelmann among its initiators.</q> <rj><qau>W. Pater.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sidereous</ent><br/
<hw>Si*de"re*ous</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>sidereus</ets>.]</ety> <def>Sidereal.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Siderite</ent><br/
<hw>Sid"er*ite</hw>, <pos>n.</pos> <ety>[L. <ets>sideritis</ets> loadstone, Gr. <?/<?/<?/<?/, <?/<?/<?/<?/, of iron, from <?/<?/<?/<?/ iron.]</ety> <sn>1.</sn> <fld>(Min.)</fld> <sd>(a)</sd> <def>Carbonate of iron, an important ore of iron occuring generally in cleavable masses, but also in rhombohedral crystals. It is of a light yellowish brown color. Called also <altname>sparry iron</altname>, <altname>spathic iron</altname>.</def> <sd>(b)</sd> <def>A meteorite consisting solely of metallic iron.</def> <sd>(c)</sd> <def>An indigo-blue variety of quartz.</def> <sd>(d)</sd> <def>Formerly, magnetic iron ore, or loadstone.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Bot.)</fld> <def>Any plant of the genus <gen>Sideritis</gen>; ironwort.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Siderographical</ent><br/
<ent>Siderographic</ent><br/
<mhw>{ <hw>Sid`er*o*graph"ic</hw> <pr>(?)</pr>, <hw>Sid`er*o*graph"ic*al</hw> <pr>(?)</pr>, }</mhw> <pos>a.</pos> <def>Of or pertaining to siderography; executed by engraved plates of steel; <as>as, <ex>siderographic</ex> art; <ex>siderographic</ex> impressions.</as></def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Siderographist</ent><br/
<hw>Sid`er*og"ra*phist</hw> <pr>(?)</pr>, <pos>n.</pos> <def>One skilled in siderography.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Siderography</ent><br/
<hw>Sid`er*og"ra*phy</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Gr. <?/<?/<?/ iron + <ets>-graphy</ets>.]</ety> <def>The art or practice of steel engraving; especially, the process, invented by Perkins, of multiplying facsimiles of an engraved steel plate by first rolling over it, when hardened, a soft steel cylinder, and then rolling the cylinder, when hardened, over a soft steel plate, which thus becomes a facsimile of the original. The process has been superseded by <xex>electrotypy</xex>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Siderolite</ent><br/
<hw>Sid"er*o*lite</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Gr. <?/<?/<?/ iron + <ets>-lite</ets>.]</ety> <def>A kind of meteorite. See under <er>Meteorite</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sideromancy</ent><br/
<hw>Sid"er*o*man`cy</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Gr. <?/<?/<?/ iron + <ets>-mancy</ets>.]</ety> <def>Divination by burning straws on red-hot iron, and noting the manner of their burning.</def>  <rj><au>Craig.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sideroscope</ent><br/
<hw>Sid"er*o*scope</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Gr. <?/<?/<?/ iron + <ets>-scope</ets>.]</ety> <def>An instrument for detecting small quantities of iron in any substance by means of a very delicate combination of magnetic needles.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Siderosis</ent><br/
||<hw>Sid`e*ro"sis</hw> <pr>(?)</pr>, <pos>n.</pos><ety>[NL., fr. Gr. <?/<?/<?/ iron.]</ety> <fld>(Med.)</fld> <def>A sort of pneumonia occuring in iron workers, produced by the inhalation of particles of iron.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Siderostat</ent><br/
<hw>Sid"er*o*stat</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>sidus</ets>, <ets>sideris</ets>, a star + Gr. <?/<?/<?/ standing, fixed, fr. <?/<?/<?/<?/ to place.]</ety> <fld>(Astron.)</fld> <def>An apparatus consisting essentially of a mirror moved by clockwork so as to throw the rays of the sun or a star in a fixed direction; -- a more general term for <stype>heliostat</stype>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sideroxylon</ent><br/
||<hw>Sid`e*rox"y*lon</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[NL., fr. Gr. <?/<?/<?/ iron + <?/<?/<?/ wood.]</ety> <fld>(Bot.)</fld> <def>A genus of tropical sapotaceous trees noted for their very hard wood; ironwood.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sidesaddle</ent><br/
<hw>Side"sad`dle</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A saddle for women, in which the rider sits with both feet on one side of the animal mounted.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Sidesaddle flower</b></col> <fld>(Bot.)</fld>, <cd>a plant with hollow leaves and curiously shaped flowers; -- called also <altname>huntsman's cup</altname>. See <er>Sarracenia</er>.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Side-slip</ent><br/
<hw>Side"-slip`</hw>, <pos>v. i.</pos> <def>See <er>Skid</er>, below.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Side slip</ent><br/
<hw>Side slip</hw>. <def>See <er>Skid</er>, below.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><-- p. 1338 --></p>

<p><ent>Sidesman</ent><br/
<hw>Sides"man</hw> <pr>(?)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Sidesmen</plw> <pr>(<?/)</pr>.</plu> <sn>1.</sn> <def>A party man; a partisan.</def>  <rj><au>Milton.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>An assistant to the churchwarden; a questman.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Side-taking</ent><br/
<hw>Side"-tak`ing</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A taking sides, as with a party, sect, or faction.</def>  <rj><au>Bp. Hall.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sidetrack</ent><br/
<hw>Side"track`</hw> <pr>(?)</pr>, <pos>v. t.</pos>  <vmorph>[<pos>imp. & p. p.</pos> <conjf>Sidetracked</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Sidetracking</conjf>.]</vmorph> <sn>1.</sn> <fld>(Railroads)</fld> <def>To transfer to a siding from a main line of track.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><sn>2.</sn>  <def>Hence, fig., to divert or reduce to a position or condition that is relatively secondary or subordinate in activity, importance, effectiveness, or the like; to switch off; to turn aside, as from a purpose.</def> <mark>[Colloq.]</mark></p>

<p><q>Such a project was, in fact, <qex>sidetracked</qex> in favor of the census of school children.</q>  <rj><qau>Pop. Sci. Monthly.</qau></rj><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Sidewalk</ent><br/
<hw>Side"walk`</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A walk for foot passengers at the side of a street or road; a foot pavement.</def> <mark>[U.S.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sideways</ent><br/
<hw>Side"ways`</hw> <pr>(?)</pr>, <pos>adv.</pos> <def>Toward the side; sidewise.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>A second refraction made <qex>sideways</qex>.</q> <rj><qau>Sir I. Newton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>His beard, a good palm's length, at least, . . . <br/
Shot <qex>sideways</qex>, like a swallow's wings.</q> <rj><qau>Longfellow.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Side-wheel</ent><br/
<hw>Side"-wheel`</hw>, <pos>a.</pos> <def>Having a paddle wheel on each side; -- said of steam vessels; <as>as, a <ex>side-wheel</ex> steamer</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sidewinder</ent><br/
<hw>Side"wind`er</hw> <pr>(?)</pr>, <pos>n.</pos> <sn>1.</sn> <fld>(Zool.)</fld> <def>See Horned rattler, under <er>Horned</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sidewise</ent><br/
<sn>2.</sn> <def>A heavy swinging blow from the side, which disables an adversary.</def> <mark>[Slang.]</mark>
<hw>Side"wise`</hw> <pr>(?)</pr>, <pos>adv.</pos> <def>On or toward one side; laterally; sideways.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>I saw them mask their awful glance<br/
<qex>Sidewise</qex> meek in gossamer lids.</q> <rj><qau>Emerson.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><-- the informal name of a specific type of heat-seeking air-to-air missile.  Also, sidewinder missile. --><br/
[<source>1913 Webster</source>]</p>

<p><ent>Siding</ent><br/
<hw>Sid"ing</hw> <pr>(?)</pr>, <pos>n.</pos> <sn>1.</sn> <def>Attaching one's self to a party.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A side track, as a railroad; a turnout.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <fld>(Carp.)</fld> <def>The covering of the outside wall of a frame house, whether made of weatherboards, vertical boarding with cleats, shingles, or the like.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <fld>(Shipbuilding)</fld> <def>The thickness of a rib or timber, measured, at right angles with its side, across the curved edge; <as>as, a timber having a <ex>siding</ex> of ten inches</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sidle</ent><br/
<hw>Si"dle</hw> <pr>(?)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Sidled</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Sidling</conjf> <pr>(?)</pr>.]</vmorph> <ety>[From <er>Side</er>.]</ety> <def>To go or move with one side foremost; to move sidewise; <as>as, to <ex>sidle</ex> through a crowd or narrow opening</as>.</def>  <rj><au>Swift.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>He . . . then <qex>sidled</qex> close to the astonished girl.</q> <rj><qau>Sir W. Scott.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Siege</ent><br/
<hw>Siege</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[OE. <ets>sege</ets>, OF. <ets>siege</ets>, F. <ets>si<egrave/ge</ets> a seat, a siege; cf. It. <ets>seggia</ets>, <ets>seggio</ets>, <ets>zedio</ets>, a seat, <ets>asseggio</ets>, <ets>assedio</ets>, a siege, F. <ets>assi<eacute/ger</ets> to besiege, It. & LL. <ets>assediare</ets>, L. <ets>obsidium</ets> a siege, besieging; all ultimately fr. L. <ets>sedere</ets> to sit. See <er>Sit</er>, and cf. <er>See</er>, <pos>n.</pos>]</ety> <sn>1.</sn> <def>A seat; especially, a royal seat; a throne.</def> <mark>[Obs.]</mark> <ldquo/Upon the very <xex>siege</xex> of justice.<rdquo/  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>A stately <qex>siege</qex> of sovereign majesty,<br/
And thereon sat a woman gorgeous gay.</q> <rj><qau>Spenser.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>In our great hall there stood a vacant chair . . . <br/
And Merlin called it <ldquo/The <qex>siege</qex> perilous.<rdquo/</q> <rj><qau>Tennyson.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Hence, place or situation; seat.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>Ah! traitorous eyes, come out of your shameless <qex>siege</qex> forever.</q> <rj><qau>Painter (Palace of Pleasure).</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>Rank; grade; station; estimation.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>I fetch my life and being<br/
From men of royal <qex>siege</qex>.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>Passage of excrements; stool; fecal matter.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>The <qex>siege</qex> of this mooncalf.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <def>The sitting of an army around or before a fortified place for the purpose of compelling the garrison to surrender; the surrounding or investing of a place by an army, and approaching it by passages and advanced works, which cover the besiegers from the enemy's fire. See the Note under <er>Blockade</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>6.</sn> <def>Hence, a continued attempt to gain possession.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Love stood the <qex>siege</qex>, and would not yield his breast.</q> <rj><qau>Dryden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>7.</sn> <def>The floor of a glass-furnace.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>8.</sn> <def>A workman's bench.</def>  <rj><au>Knught.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Siege gun</b></col>, <cd>a heavy gun for siege operations.</cd> -- <col><b>Siege train</b></col>, <cd>artillery adapted for attacking fortified places.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Siege</ent><br/
<hw>Siege</hw>, <pos>v. t.</pos> <def>To besiege; to beset.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>Through all the dangers that can <qex>siege</qex><br/
The life of man.</q> <rj><qau>Buron.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Siegework</ent><br/
<hw>Siege"work`</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A temporary fort or parallel where siege guns are mounted.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Siemens-Martin process</ent><br/
<hw>Sie"mens-Mar`tin proc"ess</hw> <pr>(?)</pr>. <def>See <cref>Open-hearth process</cref>, etc., under <er>Open</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Siemens-Martin steel</ent><br/
<hw>Sie"mens-Mar"tin steel</hw> <pr>(?)</pr>. <def>See <cref>Open-hearth steel</cref>, under <er>Open</er>.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Sienite</ent><br/
<hw>Si"e*nite</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Min.)</fld> <def>See <er>Syenite</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sienitic</ent><br/
<hw>Si`e*nit"ic</hw> <pr>(?)</pr>, <pos>a.</pos> <def>See <er>Syenitic</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sienna</ent><br/
<hw>Si*en"na</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[It. <ets>terra di Siena</ets>, fr. <ets>Siena</ets> in Italy.]</ety> <fld>(Chem.)</fld> <def>Clay that is colored red or brown by the oxides of iron or manganese, and used as a pigment. It is used either in the raw state or burnt.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Burnt sienna</b></col>, <cd>sienna made of a much redder color by the action of fire.</cd> -- <col><b>Raw sienna</b></col>, <cd>sienna in its natural state, of a transparent yellowish brown color.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Siennese</ent><br/
<hw>Si`en*nese"</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Of or pertaining to Sienna, a city of Italy.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sierra</ent><br/
||<hw>Si*er"ra</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Sp., properly, a saw, fr. L. <ets>serra</ets> a saw. See <er>Serrate</er>.]</ety> <def>A ridge of mountain and craggy rocks, with a serrated or irregular outline; <as>as, the <ex>Sierra</ex> Nevada</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The wild <qex>sierra</qex> overhead.</q> <rj><qau>Whitter.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Siesta</ent><br/
||<hw>Si*es"ta</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Sp., probably fr. L. <ets>sessitare</ets> to sit much or long, v. freq. of <ets>sedere</ets>, <ets>sessum</ets>, to sit. See <er>Sit</er>.]</ety> <def>A short sleep taken about the middle of the day, or after dinner; a midday nap.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sieur</ent><br/
||<hw>Sieur</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F., abbrev. from seigneur.  Cf. <er>Monsieur</er>, <er>Seignior</er>.]</ety> <def>Sir; -- a title of respect used by the French.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sieva</ent><br/
<hw>Sie"va</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Bot.)</fld> <def>A small variety of the Lima bean (<spn>Phaseolus lunatus</spn>).</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sieve</ent><br/
<hw>Sieve</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[OE. <ets>sive</ets>, AS. <ets>sife</ets>; akin to D. <ets>zeef</ets>, <ets>zift</ets>, OHG. <ets>sib</ets>, G. <ets>sieb</ets>. <root/151<it>a</it>.  Cf. <er>Sift</er>.]</ety> <sn>1.</sn> <def>A utensil for separating the finer and coarser parts of a pulverized or granulated substance from each other. It consist of a vessel, usually shallow, with the bottom perforated, or made of hair, wire, or the like, woven in meshes.</def> <ldquo/In a <xex>sieve</xex> thrown and sifted.<rdquo/  <rj><au>Chaucer.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A kind of coarse basket.</def>  <rj><au>Simmonds.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Sieve cells</b></col> <fld>(Bot.)</fld>, <cd>cribriform cells. See under <er>Cribriform</er>.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sifac</ent><br/
<hw>Si"fac</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Zool.)</fld> <def>The white indris of Madagascar. It is regarded by the natives as sacred.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sifflement</ent><br/
<hw>Sif"fle*ment</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F., a whistling or hissing.]</ety> <def>The act of whistling or hissing; a whistling sound; sibilation.</def> <mark>[Obs.]</mark>  <rj><au>A. Brewer.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sifilet</ent><br/
<hw>Sif"i*let</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Cf. F. <ets>siflet</ets>.]</ety> <fld>(Zool.)</fld> <def>The six-shafted bird of paradise. See <cref>Paradise bird</cref>, under <er>Paradise</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sift</ent><br/
<hw>Sift</hw> <pr>(?)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Sifted</conjf>; <pos>p. pr. & vb. n.</pos> <conjf>Sifting</conjf>.]</vmorph> <ety>[AS. <ets>siftan</ets>, from <ets>sife</ets> sieve. <root/151<it>a</it>.  See <er>Sieve</er>.]</ety> <sn>1.</sn> <def>To separate with a sieve, as the fine part of a substance from the coarse; <as>as, to <ex>sift</ex> meal or flour; to <ex>sift</ex> powder; to <ex>sift</ex> sand or lime.</as></def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To separate or part as if with a sieve.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>When yellow sands are <qex>sifted</qex> from below,<br/
The glittering billows give a golden show.</q> <rj><qau>Dryden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>To examine critically or minutely; to scrutinize.</def><br/
[<source>1913 Webster</source>]</p>

<p><q><qex>Sifting</qex> the very utmost sentence and syllable.</q> <rj><qau>Hooker.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Opportunity I here have had<br/
To try thee, <qex>sift</qex> thee.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Let him but narrowly <qex>sift</qex> his ideas.</q> <rj><qau>I. Taylor.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>To sift out</b></col>, <cd>to search out with care, as if by sifting.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sifter</ent><br/
<hw>Sift"er</hw> <pr>(?)</pr>, <pos>n.</pos> <sn>1.</sn> <def>One who, or that which, sifts.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Zool.)</fld> <def>Any lamellirostral bird, as a duck or goose; -- so called because it sifts or strains its food from the water and mud by means of the lamell<?/ of the beak.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sig</ent><br/
<hw>Sig</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Akin to AS. <ets>s<imac/gan</ets> to fall. <root/151<it>a.</it> <ets>See</ets> <er>Sink</er>, <pos>v. t.</pos>]</ety> <def>Urine.</def> <mark>[Prov. Eng.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sigaultian</ent><br/
<hw>Si*gaul"ti*an</hw> <pr>(?)</pr>, <pos>a.</pos> <fld>(Surg.)</fld> <def>Pertaining to <xex>Sigault</xex>, a French physician. See <er>Symphyseotomy</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sigger</ent><br/
<hw>Sig"ger</hw>, <pos>v. i.</pos> <def>Same as</def> <er>Sicker</er>. <mark>[Prov. Eng.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sigh</ent><br/
<hw>Sigh</hw> <pr>(?)</pr>, <pos>v. i.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Sighed</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Sighing</conjf>.]</vmorph> <ety>[OE. <ets>sighen</ets>, <ets>si<?/en</ets>; cf. also OE. <ets>siken</ets>, AS. <ets>s<imac/can</ets>, and OE. <ets>sighten</ets>, <ets>si<?/ten</ets>, <ets>sichten</ets>, AS. <ets>siccettan</ets>; all, perhaps, of imitative origin.]</ety> <sn>1.</sn> <def>To inhale a larger quantity of air than usual, and immediately expel it; to make a deep single audible respiration, especially as the result or involuntary expression of fatigue, exhaustion, grief, sorrow, or the like.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Hence, to lament; to grieve.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>He <qex>sighed</qex> deeply in his spirit.</q> <rj><qau>Mark viii. 12.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>To make a sound like sighing.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>And the coming wind did roar more loud,<br/
And the sails did <qex>sigh</qex> like sedge.</q> <rj><qau>Coleridge.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>The winter winds are wearily <qex>sighing</qex>.</q> <rj><qau>Tennyson.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ An extraordinary pronunciation of this word as <xex>s<imac/th</xex> is still heard in England and among the illiterate in the United States.</note><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sigh</ent><br/
<hw>Sigh</hw>, <pos>v. t.</pos> <sn>1.</sn> <def>To exhale (the breath) in sighs.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Never man <qex>sighed</qex> truer breath.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To utter sighs over; to lament or mourn over.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Ages to come, and men unborn,<br/
Shall bless her name, and <qex>sigh</qex> her fate.</q> <rj><qau>Pior.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>To express by sighs; to utter in or with sighs.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>They . . . <qex>sighed</qex> forth proverbs.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>The gentle swain . . . <qex>sighs</qex> back her grief.</q> <rj><qau>Hoole.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sigh</ent><br/
<hw>Sigh</hw>, <pos>n.</pos> <ety>[OE. <ets>sigh</ets>; cf. OE. <ets>sik</ets>. See <er>Sigh</er>, <pos>v. i.</pos>]</ety> <sn>1.</sn> <def>A deep and prolonged audible inspiration or respiration of air, as when fatigued or grieved; the act of sighing.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>I could drive the boat with my <qex>sighs</qex>.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Figuratively, a manifestation of grief; a lan<?/ent.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>With their <qex>sighs</qex> the air<br/
Frequenting, sent from hearts contrite.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sigh-born</ent><br/
<hw>Sigh"-born`</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Sorrowful; mournful.</def> <mark>[R.]</mark> <ldquo/<xex>Sigh-born</xex> thoughts.<rdquo/  <rj><au>De Quincey.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sigher</ent><br/
<hw>Sigh"er</hw> <pr>(?)</pr>, <pos>n.</pos> <def>One who sighs.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sighing</ent><br/
<hw>Sigh"ing</hw>, <pos>a.</pos> <def>Uttering sighs; grieving; lamenting.</def> <ldquo/<xex>Sighing</xex> millions.<rdquo/ <au>Cowper.</au> -- <wordforms><wf>Sigh"ing*ly</wf>, <pos>adv.</pos></wordforms><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sight</ent><br/
<hw>Sight</hw> <pr>(s<imac/t)</pr>, <pos>n.</pos> <ety>[OE. <ets>sight</ets>, <ets>si<thorn/t</ets>, <ets>siht</ets>, AS. <ets>siht</ets>, ge<ets>siht</ets>, ge<ets>sih<edh/</ets>, ge<ets>sieh<edh/</ets>, ge<ets>syh<edh/</ets>; akin to D. ge<ets>zicht</ets>, G. <ets>sicht</ets>, ge<ets>sicht</ets>, Dan. <ets>sigte</ets>, Sw. <ets>sigt</ets>, from the root of E. <ets>see</ets>. See <er>See</er>, <pos>v. t.</pos>]</ety> <sn>1.</sn> <def>The act of seeing; perception of objects by the eye; view; <as>as, to gain <ex>sight</ex> of land</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>A cloud received him out of their <qex>sight</qex>.</q> <rj><qau>Acts. i. 9.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>The power of seeing; the faculty of vision, or of perceiving objects by the instrumentality of the eyes.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Thy <qex>sight</qex> is young,<br/
And thou shalt read when mine begin to dazzle.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>O loss of <qex>sight</qex>, of thee I most complain!</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>The state of admitting unobstructed vision; visibility; open view; region which the eye at one time surveys; space through which the power of vision extends; <as>as, an object within <ex>sight</ex></as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>A spectacle; a view; a show; something worth seeing.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Moses said, I will now turn aside and see this great <qex>sight</qex>, why the bush is not burnt.</q> <rj><qau>Ex. iii. 3.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>They never saw a <qex>sight</qex> so fair.</q> <rj><qau>Spenser.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <def>The instrument of seeing; the eye.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Why cloud they not their <qex>sights</qex>?</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>6.</sn> <def>Inspection; examination; <as>as, a letter intended for the <ex>sight</ex> of only one person</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>7.</sn> <def>Mental view; opinion; judgment; <as>as, in their <ex>sight</ex> it was harmless</as>.</def>  <rj><au>Wake.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>That which is highly esteemed among men is abomination in the <qex>sight</qex> of God.</q> <rj><qau>Luke xvi. 15.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>8.</sn> <def>A small aperture or optical device through which objects are to be seen, and by which their direction is settled or ascertained; -- used on surveying instruments; <as>as, the <ex>sight</ex> of a quadrant</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Thier eyes of fire sparking through <qex>sights</qex> of steel.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>9.</sn> <def>An optical device or small piece of metal, fixed or movable, on the breech, muzzle, center, or trunnion of a gun, or on the breech and the muzzle of a rifle, pistol, etc., by means of which the eye is guided in aiming.  A telescope mounted on a weapon, such as a rifle, and used for accurate aiming at distant targets is called a <stype>telescopic sight</stype>.</def>  <rj><au>Farrow.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>10.</sn> <def>In a drawing, picture, etc., that part of the surface, as of paper or canvas, which is within the frame or the border or margin. In a frame or the like, the open space, the opening.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>11.</sn> <def>A great number, quantity, or sum; <as>as, a <ex>sight</ex> of money</as>.</def> <mark>[<plain>Now</plain> colloquial]</mark><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ <xex>Sight</xex> in this last sense was formerly employed in the best usage. <ldquo/A <xex>sight</xex> of lawyers.<rdquo/</note>  <rj><au>Latimer.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>A wonder <qex>sight</qex> of flowers.</q> <rj><qau>Gower.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>At sight</b></col>, <cd>as soon as seen, or presented to sight; <as>as, a draft payable <ex>at sight</ex>: to read Greek <ex>at sight</ex>; to shoot a person <ex>at sight</ex></as>.</cd> -- <col><b>Front sight</b></col> <fld>(Firearms)</fld>, <cd>the sight nearest the muzzle.</cd> -- <col><b>Open sight</b></col>. <fld>(Firearms)</fld> <sd>(a)</sd> <cd>A front sight through which the objects aimed at may be seen, in distinction from one that hides the object</cd>. <sd>(b)</sd> <cd>A rear sight having an open notch instead of an aperture.</cd> -- <mcol><col><b>Peep sight</b></col>, <col><b>Rear sight</b></col></mcol>. <cd>See under <er>Peep</er>, and <er>Rear</er>.</cd> -- <col><b>Sight draft</b></col>, <cd>an order, or bill of exchange, directing the payment of money at sight.</cd> -- <col><b>To take sight</b></col>, <cd>to take aim; to look for the purpose of directing a piece of artillery, or the like.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><syn><b>Syn.</b> -- Vision; view; show; spectacle; representation; exhibition.</syn><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sight</ent><br/
<hw>Sight</hw> <pr>(?)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Sighted</conjf>; <pos>p. pr. & vb. n.</pos> <conjf>Sighting</conjf>.]</vmorph> <sn>1.</sn> <def>To get sight of; to see; <as>as, to <ex>sight</ex> land; to <ex>sight</ex> a wreck.</as></def>  <rj><au>Kane.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To look at through a sight; to see accurately; <as>as, to <ex>sight</ex> an object, as a star</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>To apply sights to; to adjust the sights of; also, to give the proper elevation and direction to by means of a sight; <as>as, to <ex>sight</ex> a rifle or a cannon</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sight</ent><br/
<hw>Sight</hw>, <pos>v. i.</pos> <fld>(Mil.)</fld> <def>To take aim by a sight.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sighted</ent><br/
<hw>Sight"ed</hw>, <pos>a.</pos> <def>Having sight, or seeing, in a particular manner; -- used in composition; <as>as, long-<ex>sighted</ex>, short-<ex>sighted</ex>, quick-<ex>sighted</ex>, sharp-<ex>sighted</ex>, and the like</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sightful</ent><br/
<hw>Sight"ful</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Easily or clearly seen; distinctly visible; perspicuous.</def> <mark>[Obs.]</mark>  <rj><au>Testament of Love.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sightfulness</ent><br/
<hw>Sight"ful*ness</hw>, <pos>n.</pos> <def>The state of being sightful; perspicuity.</def> <mark>[Obs.]</mark>  <rj><au>Sir P. Sidney.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sight-hole</ent><br/
<hw>Sight"-hole`</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A hole for looking through; a peephole.</def> <ldquo/Stop all <xex>sight-holes</xex>.<rdquo/  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sighting</ent><br/
<hw>Sight"ing</hw>, <def><pos>a. & n.</pos> from <er>Sight</er>, <pos>v. t.</pos></def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Sighting shot</b></col>, <cd>a shot made to ascertain whether the sights of a firearm are properly adjusted; a trial shot.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sightless</ent><br/
<hw>Sight"less</hw>, <pos>a.</pos> <sn>1.</sn> <def>Lacking sight; without sight; blind.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Of all who blindly creep or <qex>sightless</qex> soar.</q> <rj><qau>Pope.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>That can not be seen; invisible.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>The <qex>sightless</qex> couriers of the air.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>Offensive or unpleasing to the eye; unsightly; <as>as, <ex>sightless</ex> stains</as>.</def> <mark>[R.]</mark>  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p>-- <wordforms><wf>Sight"less*ly</wf>, <pos>adv.</pos>- <wf>Sight"less*ness</wf>, <pos>n.</pos></wordforms><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sightliness</ent><br/
<hw>Sight"li*ness</hw> <pr>(?)</pr>, <pos>n.</pos> <def>The state of being sightly; comeliness; conspicuousness.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sightly</ent><br/
<hw>Sight"ly</hw> <pr>(?)</pr>, <pos>a.</pos> <sn>1.</sn> <def>Pleasing to the sight; comely.</def> <ldquo/Many brave, <xex>sightly</xex> horses.<rdquo/  <rj><au>L'Estrange.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Open to sight; conspicuous; <as>as, a house stands in a <ex>sightly</ex> place</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sightproof</ent><br/
<hw>Sight"proof`</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Undiscoverable to sight.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Hidden in their own <qex>sightproof</qex> bush.</q> <rj><qau>Lowell.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sight-seeing</ent><br/
<hw>Sight"-see`ing</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Engaged in, or given to, seeing sights; eager for novelties or curiosities.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sight-seeing</ent><br/
<hw>Sight"-see`ing</hw>, <pos>n.</pos> <def>The act of seeing sights; eagerness for novelties or curiosities.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sight-seer</ent><br/
<hw>Sight"-se`er</hw> <pr>(?)</pr>, <pos>n.</pos> <def>One given to seeing sights or noted things, or eager for novelties or curiosities.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sight-shot</ent><br/
<hw>Sight"-shot`</hw> <pr>(?)</pr>, <pos>n.</pos> <def>Distance to which the sight can reach or be thrown.</def> <mark>[R.]</mark>  <rj><au>Cowley.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sightsman</ent><br/
<hw>Sights"man</hw> <pr>(?)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Sightsmen</plw> <pr>(<?/)</pr>.</plu> <fld>(Mus.)</fld> <def>One who reads or performs music readily at first sight.</def> <mark>[R.]</mark>
<-- now, sight-reader -->  <rj><au>Busby.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sigil</ent><br/
<hw>Sig"il</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>sigillum</ets>. See <er>Seal</er> a stamp.]</ety> <def>A seal; a signature.</def>  <rj><au>Dryden.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Of talismans and <qex>sigils</qex> knew the power.</q> <rj><qau>Pope.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sigillaria</ent><br/
||<hw>Sig`il*la"ri*a</hw> <pr>(?)</pr>, <pos>n. pl.</pos> <ety>[L., from <ets>sigillum</ets> a seal. See <er>Sigil</er>.]</ety> <fld>(Rom. Antic.)</fld> <def>Little images or figures of earthenware exposed for sale, or given as presents, on the last two days of the Saturnalia; hence, the last two, or the sixth and seventh, days of the Saturnalia.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sigillaria</ent><br/
||<hw>Sig`il*la"ri*a</hw>, <pos>n.</pos> <ety>[NL., fem sing. fr. L. <ets>sigillum</ets> a seal.]</ety> <fld>(Paleon.)</fld> <def>A genus of fossil trees principally found in the coal formation; -- so named from the seallike leaf scars in vertical rows on the surface.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sigillarid</ent><br/
<hw>Sig`il*la"rid</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Paleon.)</fld> <def>One of an extinct family of cryptagamous trees, including the genus <gen>Sigillaria</gen> and its allies.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sigillated</ent><br/
<hw>Sig"il*la`ted</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>sigillatus</ets> adorned with little images.]</ety> <def>Decorated by means of stamps; -- said of pottery.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sigillative</ent><br/
<hw>Sig"il*la*tive</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>sigillum</ets> a seal: cf. OF. <ets>sigillatif</ets>.]</ety> <def>Fit to seal; belonging to a seal; composed of wax.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sigillum</ent><br/
||<hw>Si*gil"lum</hw> <pr>(?)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Sigilla</plw> <pr>(#)</pr>.</plu> <ety>[L.]</ety> <fld>(Rom. & Old Eng. Law)</fld> <def>A seal.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sigla</ent><br/
||<hw>Sig"la</hw> <pr>(?)</pr>, <pos>n. pl.</pos> <ety>[L.]</ety> <def>The signs, abbreviations, letters, or characters standing for words, shorthand, etc., in ancient manuscripts, or on coins, medals, etc.</def>  <rj><au>W. Savage.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sigma</ent><br/
||<hw>Sig"ma</hw> <pr>(?)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Sigmas</plw> <pr>(#)</pr>.</plu> <ety>[L., from Gr. <?/<?/<?/, <?/<?/<?/.]</ety> <def>The Greek letter <SIGMA/, <sigma/, or <sigmat/ (English <it>S</it>, or <it>s</it>). It originally had the form of the English C.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sigmodont</ent><br/
<hw>Sig"mo*dont</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Gr. <?/<?/<?/ sigma (<?/) + <?/<?/<?/, <?/<?/<?/, a tooth.]</ety> <fld>(Zool.)</fld> <def>Any one of a tribe (<fam>Sigmodontes</fam>) of rodents which includes all the indigenous rats and mice of America. So called from the form of the ridges of enamel on the crowns of the worn molars. Also used adjectively.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sigmoidal</ent><br/
<ent>Sigmoid</ent><br/
<mhw>{ <hw>Sig"moid</hw> <pr>(?)</pr>, <hw>Sig*moid"al</hw> <pr>(?)</pr>, }</mhw> <pos>a.</pos> <ety>[Gr. <?/<?/<?/; <?/<?/<?/ sigma + <grk>e'i^dos</grk> form, likeness: cf. F. <ets>sigmo<ium/de</ets>.]</ety> <def>Curved in two directions, like the letter S, or the Greek <sigmat/.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Sigmoid flexure</b></col> <fld>(Anat.)</fld>, <cd>the last curve of the colon before it terminates in the rectum. See <xex>Illust.</xex> under <er>Digestive</er>.</cd> -- <col><b>Sigmoid valves</b></col>. <fld>(Anat.)</fld> <cd>See <cref>Semilunar valves</cref>, under <er>Semilunar</er>.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sigmoidally</ent><br/
<hw>Sig*moid"al*ly</hw>, <pos>adv.</pos> <def>In a sigmoidal manner.</def><br/
[<source>1913 Webster</source>]</p>

<p><-- p. 1339 --></p>

<p><ent>Sign</ent><br/
<hw>Sign</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F. <ets>signe</ets>, L. <ets>signum</ets>; cf. AS. <ets>segen</ets>, <ets>segn</ets>, a sign, standard, banner, also fr. L. <ets>signum</ets>.  Cf. <er>Ensign</er>, <er>Resign</er>, <er>Seal</er> a stamp, <er>Signal</er>, <er>Signet</er>.]</ety> <def>That by which anything is made known or represented; that which furnishes evidence; a mark; a token; an indication; a proof.</def> Specifically: <sd>(a)</sd> <def>A remarkable event, considered by the ancients as indicating the will of some deity; a prodigy; an omen.</def> <sd>(b)</sd> <def>An event considered by the Jews as indicating the divine will, or as manifesting an interposition of the divine power for some special end; a miracle; a wonder.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Through mighty <qex>signs</qex> and wonders, by the power of the Spirit of God.</q> <rj><qau>Rom. xv. 19.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>It shall come to pass, if they will not believe thee, neither hearken to the voice of the first <qex>sign</qex>, that they will believe the voice of the latter <qex>sign</qex>.</q> <rj><qau>Ex. iv. 8.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sd>(c)</sd> <def>Something serving to indicate the existence, or preserve the memory, of a thing; a token; a memorial; a monument.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>What time the fire devoured two hundred and fifty men, and they became a <qex>sign</qex>.</q> <rj><qau>Num. xxvi. 10.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sd>(d)</sd> <def>Any symbol or emblem which prefigures, typifles, or represents, an idea; a type; hence, sometimes, a picture.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The holy symbols, or <qex>signs</qex>, are not barely significative; but what they represent is as certainly delivered to us as the symbols themselves.</q> <rj><qau>Brerewood.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Saint George of Merry England, the <qex>sign</qex> of victory.</q> <rj><qau>Spenser.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sd>(e)</sd> <def>A word or a character regarded as the outward manifestation of thought; as, words are the <xex>sign</xex> of ideas.</def> <sd>(f)</sd> <def>A motion, an action, or a gesture by which a thought is expressed, or a command or a wish made known.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>They made <qex>signs</qex> to his father, how he would have him called.</q> <rj><qau>Luke i. 62.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sd>(g)</sd> <def>Hence, one of the gestures of pantomime, or of a language of a signs such as those used by the North American Indians, or those used by the deaf and dumb.</def><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ Educaters of the deaf distinguish between <xex>natural signs</xex>, which serve for communicating ideas, and <xex>methodical</xex>, or <xex>systematic</xex>, <xex>signs</xex>, adapted for the dictation, or the rendering, of written language, word by word; and thus the <xex>signs</xex> are to be distinguished from the <xex>manual alphabet</xex>, by which words are spelled on the fingers.</note><br/
[<source>1913 Webster</source>]</p>

<p><sd>(h)</sd> <def>A military emblem carried on a banner or a standard.</def> <au>Milton.</au> <sd>(i)</sd> <def>A lettered board, or other conspicuous notice, placed upon or before a building, room, shop, or office to advertise the business there transacted, or the name of the person or firm carrying it on; a publicly displayed token or notice.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The shops were, therefore, distinguished by painted <qex>signs</qex>, which gave a gay and grotesque aspect to the streets.</q> <rj><qau>Macaulay.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sd>(j)</sd> <fld>(Astron.)</fld> <def>The twelfth part of the ecliptic or zodiac.</def><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ The <xex>signs</xex> are reckoned from the point of intersection of the ecliptic and equator at the vernal equinox, and are named, respectively, <stype>Aries</stype> (<Aries/), <stype>Taurus</stype> (<Taurus/), <stype>Gemini</stype> (II), <stype>Cancer</stype> (<Cancer/), <stype>Leo</stype> (<Leo/), <stype>Virgo</stype> (<Virgo/), <stype>Libra</stype> (<Libra/), <stype>Scorpio</stype> (<Scorpio/), <stype>Sagittarius</stype> (<Sagittarius/), <stype>Capricornus </stype> (<Capricorn/), <stype>Aquarius</stype> (<Aquarius/), <stype>Pisces</stype> (<Pisces/). These names were originally the names of the constellations occupying severally the divisions of the zodiac, by which they are still retained; but, in consequence of the procession of the equinoxes, the signs have, in process of time, become separated about 30 degrees from these constellations, and each of the latter now lies in the sign next in advance, or to the east of the one which bears its name, as the constellation Aries in the sign Taurus, etc.</note><br/
[<source>1913 Webster</source>]</p>

<p><sd>(k)</sd> <fld>(Alg.)</fld> <def>A character indicating the relation of quantities, or an operation performed upon them; <as>as, the <ex>sign</ex> + (plus); the <ex>sign</ex> -- (minus); the <ex>sign</ex> of division <divide/, and the like</as>.</def> <sd>(l)</sd> <fld>(Med.)</fld> <def>An objective evidence of disease; that is, one appreciable by some one other than the patient.</def><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ The terms <xex>symptom</xex> and and <xex>sign</xex> are often used synonymously; but they may be discriminated. A <xex>sign</xex> differs from a <xex>symptom</xex> in that the latter is perceived only by the patient himself. The term <xex>sign</xex> is often further restricted to the purely local evidences of disease afforded by direct examination of the organs involved, as distinguished from those evidence of general disturbance afforded by observation of the temperature, pulse, etc. In this sense it is often called <xex>physical sign</xex>.</note><br/
[<source>1913 Webster</source>]</p>

<p><sd>(m)</sd> <fld>(Mus.)</fld> <def>Any character, as a flat, sharp, dot, etc.</def> <sd>(n)</sd> <fld>(Theol.)</fld> <def>That which, being external, stands for, or signifies, something internal or spiritual; -- a term used in the Church of England in speaking of an ordinance considered with reference to that which it represents.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>An outward and visible <qex>sign</qex> of an inward and spiritual grace.</q> <rj><qau>Bk. of Common Prayer.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ See the Table of <er>Arbitrary Signs</er>, p. 1924.</note><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Sign manual</b></col>. <sd>(a)</sd> <fld>(Eng. Law)</fld> <cd>The royal signature superscribed at the top of bills of grants and letter patent, which are then sealed with the privy signet or great seal, as the case may be, to complete their validity</cd>. <sd>(b)</sd> <cd>The signature of one's name in one's own handwriting.</cd> <au>Craig. Tomlins. Wharton.</au></cs><br/
[<source>1913 Webster</source>]</p>

<p><syn><b>Syn.</b> -- Token; mark; note; symptom; indication; signal; symbol; type; omen; prognostic; presage; manifestation. See <er>Emblem</er>.</syn><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sign</ent><br/
<hw>Sign</hw> <pr>(?)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Signed</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Signing</conjf>.]</vmorph> <ety>[OE. <ets>seinen</ets> to bless, originally, to make the sign of the cross over; in this sense fr. ASS. <ets>segnian</ets> (from <ets>segn</ets>, n.), or OF. <ets>seignier</ets>, F. <ets>signer</ets>, to mark, to sign (in sense 3), fr. L. <ets>signare</ets> to mark, set a mark upon, from <ets>signum</ets>. See <er>Sign</er>, <pos>n.</pos>]</ety> <sn>1.</sn> <def>To represent by a sign; to make known in a typical or emblematic manner, in distinction from speech; to signify.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>I <qex>signed</qex> to Browne to make his retreat.</q> <rj><qau>Sir W. Scott.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To make a sign upon; to mark with a sign.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>We receive this child into the congregation of Christ's flock, and do <qex>sign</qex> him with the sign of the cross.</q> <rj><qau>Bk. of Com Prayer.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>To affix a signature to; to ratify by hand or seal; to subscribe in one's own handwriting.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Inquire the Jew's house out, give him this deed,<br/
And let him <qex>sign</qex> it.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>To assign or convey formally; -- used with <xex>away</xex>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <def>To mark; to make distinguishable.</def>  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sign</ent><br/
<hw>Sign</hw> <pr>(?)</pr>, <pos>v. i.</pos> <sn>1.</sn> <def>To be a sign or omen.</def> <mark>[Obs.]</mark>  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To make a sign or signal; to communicate directions or intelligence by signs.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <specif>Especially:</specif> <def>To communicate in <er>sign language</er>.</def><br/
[<source>PJC</source>]</p>

<p><sn>4.</sn> <def>To write one's name, esp. as a token of assent, responsibility, or obligation; <as>as, he <ex>signed</ex> in red ink</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Signable</ent><br/
<hw>Sign"a*ble</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Suitable to be signed; requiring signature; <as>as, a legal document <ex>signable</ex> by a particular person</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Signal</ent><br/
<hw>Sig"nal</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F., fr. LL. <ets>signale</ets>, fr. L. <ets>signum</ets>. See <er>Sign</er>, <pos>n.</pos>]</ety> <sn>1.</sn> <def>A sign made for the purpose of giving notice to a person of some occurence, command, or danger; also, a sign, event, or watchword, which has been agreed upon as the occasion of concerted action.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>All obeyed<br/
The wonted <qex>signal</qex> and superior voice<br/
Of this great potentate.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A token; an indication; a foreshadowing; a sign; anything taken as evidence of some process.</def><br/
[<source>1913 Webster</source> <source>+PJC</source>]</p>

<p><q>The weary sun . . . <br/
Gives <qex>signal</qex> of a goodly day to-morrow.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>There was not the least <qex>signal</qex> of the calamity to be seen.</q> <rj><qau>De Foc.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <specif>Hence:</specif> <fld>(Electronics)</fld> <def>A measureable electrical quantity, such as voltage or current, that conveys information by varying in magnitude over time; <as>as, the <ex>signals</ex> from the strongest commercial radio stations can be received over hundreds of miles</as>.</def><br/
[<source>PJC</source>]</p>

<p><ent>Signal to noise</ent><br/
<ent>Signal to noise ratio</ent><br/
<mhw><hw>Sig"nal to noise" ra`ti*o</hw>, <hw>Sig"nal to noise"</hw></mhw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Electronics, Information science, Experimental science)</fld> <def>The ratio of the intensity of a signal{2} to the background noise detected by a measuring instrument, especially in a communications channel; the higher the ratio, the more accurately the information contained can be interpreted.  The term is applied not only to human communications, but to the detection of information in any system that is being studied to gain knowledge; <as>as, the <ex>signal-to-noise ratio</ex> of light signals in older optic fibers drops to an unusable level after several miles.</as>.</def><br/
[<source>PJC</source>]</p>

<p><ent>Signal</ent><br/
<hw>Sig"nal</hw>, <pos>a.</pos> <ety>[From <ets>signal</ets>, n.: cf. F. <ets>signal<eacute/</ets>.]</ety> <sn>1.</sn> <def>Noticeable; distinguished from what is ordinary; eminent; remarkable; memorable; <as>as, a <ex>signal</ex> exploit; a <ex>signal</ex> service; a <ex>signal</ex> act of benevolence</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>As <qex>signal</qex> now in low, dejected state<br/
As erst in highest, behold him where he lies.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Of or pertaining to signals, or the use of signals in conveying information; <as>as, a <ex>signal</ex> flag or officer</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>The signal service</b></col>, <cd>a bureau of the government (in the United States connected with the War Department) organized to collect from the whole country simultaneous raports of local meteorological conditions, upon comparison of which at the central office, predictions concerning the weather are telegraphed to various sections, where they are made known by signals publicly displayed.</cd> -- <col><b>Signal station</b></col>, <cd>the place where a signal is displayed; specifically, an observation office of the signal service.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><syn><b>Syn.</b> -- Eminent; remarkable; memorable; extraordinary; notable; conspicuous.</syn><br/
[<source>1913 Webster</source>]</p>

<p><ent>Signal</ent><br/
<hw>Sig"nal</hw>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Signaled <pr>(<?/)</pr> or Signalled</conjf>; <pos>p. pr. & vb. n.</pos> <conjf>Signaling</conjf> or <conjf>Signalling</conjf>.]</vmorph> <sn>1.</sn> <def>To communicate by signals; <as>as, to <ex>signal</ex> orders</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To notify by a signals; to make a signal or signals to; <as>as, to <ex>signal</ex> a fleet to anchor</as>.</def>  <rj><au>M. Arnold.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Signalist</ent><br/
<hw>Sig"nal*ist</hw>, <pos>n.</pos> <def>One who makes signals; one who communicates intelligence by means of signals.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Signality</ent><br/
<hw>Sig*nal"i*ty</hw> <pr>(?)</pr>, <pos>n.</pos> <def>The quality or state of being signal or remarkable.</def> <mark>[Obs.]</mark>  <rj><au>Sir T. Browne.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Signalize</ent><br/
<hw>Sig"nal*ize</hw> <pr>(?)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Signalized</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Signalizing</conjf> <pr>(?)</pr>.]</vmorph> <ety>[From <er>Signal</er>, <pos>a.</pos>]</ety> <sn>1.</sn> <def>To make signal or eminent; to render distinguished from what is common; to distinguish.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>It is this passion which drives men to all the ways we see in use of <qex>signalizing</qex> themselves.</q> <rj><qau>Burke.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To communicate with by means of a signal; <as>as, a ship <ex>signalizes</ex> its consort</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>To indicate the existence, presence, or fact of, by a signal; <as>as, to <ex>signalize</ex> the arrival of a steamer</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Signally</ent><br/
<hw>Sig"nal*ly</hw>, <pos>adv.</pos> <def>In a signal manner; eminently.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Signalman</ent><br/
<hw>Sig"nal*man</hw> <pr>(?)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>-men</plw> <pr>(<?/)</pr>.</plu> <def>A man whose business is to manage or display signals; especially, one employed in setting the signals by which railroad trains are run or warned.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Signalment</ent><br/
<hw>Sig"nal*ment</hw> <pr>(?)</pr>, <pos>n.</pos> <def>The act of signaling, or of signalizing; hence, description by peculiar, appropriate, or characteristic marks.</def>  <rj><au>Mrs. Browning.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Signate</ent><br/
<hw>Sig"nate</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>signatus</ets>, p. p. See <er>Sign</er>, <pos>v. t.</pos>]</ety> <fld>(Zool.)</fld> <def>Having definite color markings.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Signation</ent><br/
<hw>Sig*na"tion</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>signatio</ets>. See <er>Sign</er>, <pos>v. t.</pos>]</ety> <def>Sign given; marking.</def> <mark>[Obs.]</mark>  <rj><au>Sir T. Browne.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Signatory</ent><br/
<hw>Sig"na*to*ry</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>signatorius</ets>.]</ety> <sn>1.</sn> <def>Relating to a seal; used in sealing.</def> <mark>[Obs.]</mark>  <rj><au>Bailey.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Signing; joining or sharing in a signature; <as>as, <ex>signatory</ex> powers</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Signatory</ent><br/
<hw>Sig"na*to*ry</hw>, <pos>n.</pos>; <plu><it>pl.</it> <plw>-ries</plw> <pr>(<?/)</pr>.</plu> <def>A signer; one who signs or subscribes; <as>as, a conference of <ex>signatories</ex></as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Signature</ent><br/
<hw>Sig"na*ture</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F. (cf. It. <ets>signatura</ets>, <ets>segnatura</ets>, Sp. & LL. <ets>signatura</ets>), from L. <ets>signare</ets>, <ets>signatum</ets>. See <er>Sign</er>, <pos>v. t.</pos>]</ety> <sn>1.</sn> <def>A sign, stamp, or mark impressed, as by a seal.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The brain, being well furnished with various traces, <qex>signatures</qex>, and images.</q> <rj><qau>I. Watts.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>The natural and indelible <qex>signature</qex> of God, which human souls . . . are supposed to be stamped with.</q> <rj><qau>Bentley.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Especially, the name of any person, written with his own hand, employed to signify that the writing which precedes accords with his wishes or intentions; a sign manual; an autograph.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <fld>(Physiol.)</fld> <def>An outward mark by which internal characteristics were supposed to be indicated.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Some plants bear a very evident <qex>signature</qex> of their nature and use.</q> <rj><qau>Dr. H. More.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <fld>(Old Med.)</fld> <def>A resemblance between the external characters of a disease and those of some physical agent, for instance, that existing between the red skin of scarlet fever and a red cloth; -- supposed to indicate this agent in the treatment of the disease.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <fld>(Mus.)</fld> <def>The designation of the key (when not C major, or its relative, A minor) by means of one or more sharps or flats at the beginning of the staff, immediately after the clef, affecting all notes of the same letter throughout the piece or movement. Each minor key has the same signature as its relative major.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>6.</sn> <fld>(Print.)</fld> <sd>(a)</sd> <def>A letter or figure placed at the bottom of the first page of each sheet of a book or pamphlet, as a direction to the binder in arranging and folding the sheets.</def> <sd>(b)</sd> <def>The printed sheet so marked, or the form from which it is printed; <as>as, to reprint one or more <ex>signatures</ex></as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ Star signatures (as A*, 1*) are the same characters, with the addition of asterisks, used on the first pages of offcuts, as in 12mo sheets.</note><br/
[<source>1913 Webster</source>]</p>

<p><sn>7.</sn> <fld>(Pharm.)</fld> <def>That part of a prescription which contains the directions to the patient. It is usually prefaced by <it>S</it> or <it>Sig.</it> (an abbreviation for the Latin <it>signa</it>, imperative of <it>signare</it> to sign or mark).</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Signature</ent><br/
<hw>Sig"na*ture</hw> <pr>(?)</pr>, <pos>v. t.</pos> <def>To mark with, or as with, a signature or signatures.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Signaturist</ent><br/
<hw>Sig"na*tur`ist</hw> <pr>(?)</pr>, <pos>n.</pos> <def>One who holds to the doctrine of signatures impressed upon objects, indicative of character or qualities.</def> <mark>[Obs.]</mark>  <rj><au>Sir T. Browne.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Signboard</ent><br/
<hw>Sign"board`</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A board, placed on or before a shop, office, etc., on which ssome notice is given, as the name of a firm, of a business, or the like.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Signer</ent><br/
<hw>Sign"er</hw> <pr>(?)</pr>, <pos>n.</pos> <def>One who signs or subscribes his name; <as>as, a memorial with a hundred <ex>signers</ex></as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Signet</ent><br/
<hw>Sig"net</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[OF. <ets>signet</ets> a signet, F., a bookmark, dim. of <ets>signe</ets>. See <er>Sign</er>, <pos>n.</pos>, and cf. <er>Sennet</er>.]</ety> <def>A seal; especially, in England, the seal used by the sovereign in sealing private letters and grants that pass by bill under the sign manual; -- called also <altname>privy signet</altname>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>I had my father's <qex>signet</qex> in my purse.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Signet ring</b></col>, <cd>a ring containing a signet or private seal.</cd> -- <col><b>Writer to the signet</b></col> <fld>(Scots Law)</fld>, <cd>a judicial officer who prepares warrants, writs, etc.; originally, a clerk in the office of the secretary of state.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Signeted</ent><br/
<hw>Sig"net*ed</hw>, <pos>a.</pos> <def>Stamped or marked with a signet.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Signifer</ent><br/
<hw>Sig"ni*fer</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L., from <ets>signum</ets> sign + <ets>ferre</ets> to bear.]</ety> <def>Bearing signs.</def> <mark>[Obs.]</mark> <ldquo/The <xex>signifer</xex> sphere, or zodiac.<rdquo/  <rj><au>Holland.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Significancy</ent><br/
<ent>Significance</ent><br/
<mhw>{ <hw>Sig*nif"i*cance</hw> <pr>(?)</pr>, <hw>Sig*nif"i*can*cy</hw> <pr>(?)</pr>, }</mhw> <pos>n.</pos> <ety>[L. <ets>significantia</ets>.]</ety> <sn>1.</sn> <def>The quality or state of being significant.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>That which is signified; meaning; import; <as>as, the <ex>significance</ex> of a nod, of a motion of the hand, or of a word or expression</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>Importance; moment; weight; consequence.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>With this brain I must work, in order to give <qex>significancy</qex> and value to the few facts which I possess.</q> <rj><qau>De Quincey.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Significant</ent><br/
<hw>Sig*nif"i*cant</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>significans</ets>, <ets>-antis</ets>, p. pr. of <ets>significare</ets>. See <er>Signify</er>.]</ety> <sn>1.</sn> <def>Fitted or designed to signify or make known something having a meaning; standing as a sign or token; expressive or suggestive; <as>as, a <ex>significant</ex> word or sound; a <ex>significant</ex> look</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>It was well said of Plotinus, that the stars were <qex>significant</qex>, but not efficient.</q> <rj><qau>Sir W. Raleigh.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Deserving to be considered; important; momentous; <as>as, a <ex>significant</ex> event</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Significant figures</b></col> <fld>(Arith.)</fld>, <cd>the figures which remain to any number, or decimal fraction, after the ciphers at the right or left are canceled. <as>Thus, the <ex>significant figures</ex> of 25,000, or of .0025, are 25</as>.</cd></cs><-- to (n) significant figures --><br/
[<source>1913 Webster</source>]</p>

<p><ent>Significant</ent><br/
<hw>Sig*nif"i*cant</hw>, <pos>n.</pos> <def>That which has significance; a sign; a token; a symbol.</def>  <rj><au>Wordsworth.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>In dumb <qex>significants</qex> proclaim your thoughts.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Significantly</ent><br/
<hw>Sig*nif"i*cant*ly</hw>, <pos>adv.</pos> <def>In a significant manner.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Significate</ent><br/
<hw>Sig*nif"i*cate</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>significatus</ets>, p. p. of <ets>significare</ets>. See <er>Signify</er>.]</ety> <fld>(Logic)</fld> <def>One of several things signified by a common term.</def>  <rj><au>Whately.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Signification</ent><br/
<hw>Sig`ni*fi*ca"tion</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F. <ets>signification</ets>, L. <ets>significatio</ets>.]</ety> <sn>1.</sn> <def>The act of signifying; a making known by signs or other means.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>A <qex>signification</qex> of being pleased.</q> <rj><qau>Landor.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>All speaking or <qex>signification</qex> of one's mind implies an act or addres of one man to another.</q> <rj><qau>South.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>That which is signified or made known; that meaning which a sign, character, or token is intended to convey; <as>as, the <ex>signification</ex> of words</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Significative</ent><br/
<hw>Sig*nif"i*ca*tive</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>significativus</ets>: cf. F. <ets>significatif</ets>.]</ety> <sn>1.</sn> <def>Betokening or representing by an external sign.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The holy symbols or signs are not barely <qex>significative</qex>.</q> <rj><qau>Brerewood.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Having signification or meaning; expressive of a meaning or purpose; significant.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Neither in the degrees of kindred they were destitute of <qex>significative</qex> words.</q> <rj><qau>Camden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p>-- <wordforms><wf>Sig*nif"i*ca*tive*ly</wf>, <pos>adv.</pos> -- <wf>Sig*nif"i*ca*tive*ness</wf>, <pos>n.</pos></wordforms><br/
[<source>1913 Webster</source>]</p>

<p><ent>Significator</ent><br/
<hw>Sig"ni*fi*ca`tor</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Cf. F. <ets>significateur</ets>.]</ety> <def>One who, or that which, signifies.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>In this diagram there was one <qex>significator</qex> which pressed remarkably upon our astrologer's attention.</q> <rj><qau>Sir W. Scott.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Significatory</ent><br/
<hw>Sig*nif"i*ca*to*ry</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>significatorius</ets>.]</ety> <def>Significant.</def> -- <def2><pos>n.</pos> <def>That which is significatory.</def></def2><br/
[<source>1913 Webster</source>]</p>

<p><ent>Significavit</ent><br/
||<hw>Sig`ni*fi*ca"vit</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L., (he) has signified, perf. ind. of <ets>significare</ets> to signify.]</ety> <fld>(Eng. Eccl. Law)</fld> <def>Formerly, a writ issuing out of chancery, upon certificate given by the ordinary, of a man's standing excommunicate by the space of forty days, for the laying him up in prison till he submit himself to the authority of the church.</def>  <rj><au>Crabb.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Signify</ent><br/
<hw>Sig"ni*fy</hw> <pr>(?)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Signified</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Signifying</conjf> <pr>(?)</pr>.]</vmorph> <ety>[F. <ets>signifier</ets>, L. <ets>significare</ets>; <ets>signum</ets> a sign + <ets>-ficare</ets> (in comp.) to make. See <er>Sign</er>, <pos>n.</pos>, and <er>-fy</er>.]</ety> <sn>1.</sn> <def>To show by a sign; to communicate by any conventional token, as words, gestures, signals, or the like; to announce; to make known; to declare; to express; <as>as, a <ex>signified</ex> his desire to be present</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>I 'll to the king; and <qex>signify</qex> to him<br/
That thus I have resign'd my charge to you.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>The government should <qex>signify</qex> to the Protestants of Ireland that want of silver is not to be remedied.</q> <rj><qau>Swift.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To mean; to import; to denote; to betoken.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>He bade her tell him what it <qex>signified</qex>.</q> <rj><qau>Chaucer.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>A tale<br/
Told by an idiot, full of sound and fury,<br/
<qex>Signifying</qex> nothing.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ <xex>Signify</xex> is often used impersonally; as, it <xex>signifies</xex> nothing, it does not <xex>signify</xex>, that is, it is of no importance.</note><br/
[<source>1913 Webster</source>]</p>

<p><syn><b>Syn.</b> -- To express; manifest; declare; utter; intimate; betoken; denote; imply; mean.</syn><br/
[<source>1913 Webster</source>]</p>

<p><ent>signing</ent><br/
<hw>sign"ing</hw> <pr>(s<imac/n"<icr/ng)</pr>, <pos>n.</pos> <ety>[Participle of <er>sign</er>, sense 3.]</ety> <def>The procedure or process of communicating by use of a <er>sign language</er>.</def><br/
[<source>PJC</source>]</p>

<p><ent>Signior</ent><br/
<hw>Sign"ior</hw> <pr>(?)</pr>, <pos>n.</pos> <def>Sir; Mr. The English form and pronunciation for the Italian <xex>Signor</xex> and the Spanish <xex>Se<ntil/or</xex>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Signiorize</ent><br/
<hw>Sign"ior*ize</hw> <pr>(?)</pr>, <pos>v. t.</pos> <ety>[See <er>Seigniorize</er>.]</ety> <def>To exercise dominion over; to lord it over.</def> <mark>[Obs.]</mark>  <rj><au>Shelton.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><-- p. 1340 --></p>

<p><ent>Signiorize</ent><br/
<hw>Sign"ior*ize</hw> <pr>(s<emac/n"y<etil/r*<imac/z)</pr>, <pos>v. i.</pos> <def>To exercise dominion; to seigniorize.</def> <mark>[Obs.]</mark>  <rj><au>Hewyt.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Signiorship</ent><br/
<hw>Sign"ior*ship</hw>, <pos>n.</pos> <def>State or position of a signior.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Signiory</ent><br/
<hw>Sign"ior*y</hw> <pr>(-<ycr/)</pr>, <pos>n.</pos> <def>Same as <er>Seigniory</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>sign language</ent><br/
<hw>sign" lang`uage</hw> <pr>(s<imac/n" l<acr/ng`w<asl/j)</pr>, <pos>n.</pos> <def>A form of language for communicating by use of gestures made by the hands, rather than by speech.  It includes alphabets made by hand gestures, as well as proper languages formed from signs.  Among the latter is the <stype>American Sign Language</stype> (ASL), used by the deaf.  See also <er>dactylology</er>.</def><br/
[<source>PJC</source>]</p>

<p><ent>sign off</ent><br/
<hw>sign` off"</hw> <pr>(s<imac/n` <add/f")</pr>, <pos>v. i.</pos> <def>To terminate a communication, especially one conducted by radio waves; to terminate a broadcast over radio or television; <as>as, this is your reporter in Cairo, <ex>signing off</ex></as>.</def><br/
[<source>PJC</source>]</p>

<p><ent>sign on</ent><br/
<hw>sign` on"</hw> <pr>(s<imac/n` <ocr/n")</pr>, <pos>v. i.</pos> <def>To begin a communication, especially one conducted by radio waves.</def><br/
[<source>PJC</source>]</p>

<p><ent>Signore</ent><br/
<ent>Signor</ent><br/
<mhw>{ ||<hw>Si*gnor"</hw> <pr>(?)</pr>, ||<hw>Si*gno"re</hw> <pr>(?)</pr>, }</mhw> <pos>n.</pos> <ety>[It. See <er>Seignior</er>.]</ety> <def>Sir; Mr.; -- a title of address or respect among the Italians. Before a noun the form is <xex>Signor</xex>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Signora</ent><br/
||<hw>Si*gno"ra</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[It.]</ety> <def>Madam; Mrs; -- a title of address or respect among the Italians.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Signorina</ent><br/
||<hw>Si`gno*ri"na</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[It.]</ety> <def>Miss; -- a title of address among the Italians.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Signpost</ent><br/
<hw>Sign"post`</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A post on which a sign hangs, or on which papers are placed to give public notice of anything.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sike</ent><br/
<ent>Sik</ent><br/
<mhw>{ <hw>Sik</hw> <pr>(?)</pr>, <hw>Sike</hw> <pr>(?)</pr>, }</mhw> <pos>a.</pos> <def>Such. See <er>Such</er>.</def> <mark>[Obs.]</mark> <ldquo/<xex>Sike</xex> fancies weren foolerie.<rdquo/  <rj><au>Spenser.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sike</ent><br/
<hw>Sike</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[AS. <ets>s<imac/c</ets>.  Cf. <er>Sig</er>.]</ety> <def>A gutter; a stream, such as is usually dry in summer.</def> <mark>[Prov. Eng. & Scot.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sike</ent><br/
<hw>Sike</hw>, <pos>n.</pos> <ety>[See <er>Sick</er>.]</ety> <def>A sick person.</def> <mark>[Prov. Eng.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sike</ent><br/
<hw>Sike</hw>, <pos>v. i.</pos> <def>To sigh.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>That for his wife weepeth and <qex>siketh</qex> sore.</q> <rj><qau>Chaucer.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sike</ent><br/
<hw>Sike</hw>, <pos>n.</pos> <def>A sigh.</def> <mark>[Obs.]</mark>  <rj><au>Chaucer.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sikerness</ent><br/
<ent>Sikerly</ent><br/
<ent>Siker</ent><br/
<mhw>{ <hw>Sik"er</hw> <pr>(?)</pr>, <pos>a. & adv.</pos>, <hw>Sik"er*ly</hw>, <pos>adv.</pos>, <hw>Sik"er*ness</hw>, <pos>n.</pos>, etc.}</mhw> <def>See 2d <er>Sicker</er>, <er>Sickerly</er>, etc.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sikhs</ent><br/
<hw>Sikhs</hw> <pr>(?)</pr>, <pos>n. pl.</pos>; <sing>sing. <singw>Sikh</singw> <pr>(<?/)</pr></sing>. <ety>[Hind. <ets>Sikh</ets>, properly, a disciple.]</ety> <def>A religious sect noted for warlike traits, founded in the Punjab at the end of the 15th century.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Silage</ent><br/
<hw>Si"lage</hw> <pr>(?)</pr>, <pos>n. & v.</pos> <def>Short for <er>Ensilage</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sile</ent><br/
<hw>Sile</hw> <pr>(?)</pr>, <pos>v. t.</pos> <ety>[Akin to Sw. <ets>sila</ets> to strain, <ets>sil</ets> sieve, G. <ets>sielen</ets> to draw away or lead off water. <root/151<it>a.</it>  See <er>Silt</er>.]</ety> <def>To strain, as fresh milk.</def> <mark>[Prov. Eng.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sile</ent><br/
<hw>Sile</hw>, <pos>v. i.</pos> <def>To drop; to flow; to fall.</def> <mark>[Prov. Eng.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sile</ent><br/
<hw>Sile</hw>, <pos>n.</pos> <sn>1.</sn> <def>A sieve with fine meshes.</def> <mark>[Prov. Eng.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Filth; sediment.</def> <mark>[Prov. Eng.]</mark>  <rj><au>Halliwell.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sile</ent><br/
<hw>Sile</hw>, <pos>n.</pos> <ety>[Icel. <ets>s<imac/ld</ets> herring; akin to Sw. <ets>sill</ets>, Dan. <ets>sild</ets>.  Cf. <er>Sill</er> the young of a herring.]</ety> <fld>(Zool.)</fld> <def>A young or small herring.</def> <mark>[Eng.]</mark>  <rj><au>Pennant.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Silence</ent><br/
<hw>Si"lence</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F., fr. L. <ets>silentium</ets>. See <er>Silent</er>.]</ety><br/
[<source>1913 Webster</source>]</p>

<p><sn>1.</sn> <def>The state of being silent; entire absence of sound or noise; absolute stillness.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>I saw and heared; for such a numerous host<br/
Fled not in <qex>silence</qex> through the frighted deep.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Forbearance from, or absence of, speech; taciturnity; muteness.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>Secrecy; <as>as, these things were transacted in <ex>silence</ex></as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The administration itself keeps a profound <qex>silence</qex>.</q> <rj><qau>D. Webster.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>The cessation of rage, agitation, or tumilt; calmness; quiest; <as>as, the elements were reduced to <ex>silence</ex></as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <def>Absence of mention; oblivion.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>And what most merits fame, in <qex>silence</qex> hid.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Silence</ent><br/
<hw>Si"lence</hw>, <pos>interj.</pos> <def>Be silent; -- used elliptically for <xex>let there be silence</xex>, or <xex>keep silence</xex>.</def>  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Silence</ent><br/
<hw>Si"lence</hw>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Silenced</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Silencing</conjf> <pr>(?)</pr>.]</vmorph> <sn>1.</sn> <def>To compel to silence; to cause to be still; to still; to hush.</def><br/
[<source>1913 Webster</source>]</p>

<p><q><qex>Silence</qex> that dreadful bell; it frights the isle.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To put to rest; to quiet.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>This would <qex>silence</qex> all further opposition.</q> <rj><qau>Clarendon.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>These would have <qex>silenced</qex> their scruples.</q> <rj><qau>Rogers.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>To restrain from the exercise of any function, privilege of instruction, or the like, especially from the act of preaching; <as>as, to <ex>silence</ex> a minister of the gospel</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The Rev. Thomas Hooker of Chelmsford, in Essex, was <qex>silenced</qex> for nonconformity.</q> <rj><qau>B. Trumbull.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>To cause to cease firing, as by a vigorous cannonade; <as>as, to <ex>silence</ex> the batteries of an enemy</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Silencer</ent><br/
<hw>Si"lenc*er</hw> <pr>(?)</pr>, <pos>n.</pos> <def>One that silences;</def> <specif>specif.</specif>: <sd>(a)</sd> <def>The muffler of an internal-combustion engine.</def> <sd>(b)</sd> <def>Any of various devices to silence the humming noise of telegraph wires.</def> <sd>(c)</sd> <def>A device for silencing the report of a firearm shooting its projectiles singly, as a tubular attachment for the muzzle having circular plates that permit the passage of the projectile but impart a rotary motion to, and thus retard, the exploding gases.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Silene</ent><br/
<hw>Si*le"ne</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[NL., fr. L. <ets>Silenus</ets>, the attendant of Bacchus.]</ety> <fld>(Bot.)</fld> <def>A genus of caryophyllaceous plants, usually covered with a viscid secretion by which insects are caught; catchfly.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Bon Sil<egrave/ne</b></col>. <cd>See <er>Sil<egrave/ne</er>, in the Vocabulary.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Silent</ent><br/
<hw>Si"lent</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>silens</ets>, <ets>-entis</ets>, p. pr. of <ets>silere</ets> to be silent; akin to Goth. ana-silan.]</ety> <sn>1.</sn> <def>Free from sound or noise; absolutely still; perfectly quiet.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>How <qex>silent</qex> is this town!</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Not speaking; indisposed to talk; speechless; mute; taciturn; not loquacious; not talkative.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Ulysses, adds he, was the most eloquent and most <qex>silent</qex> of men.</q> <rj><qau>Broome.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>This new-created world, whereof in hell<br/
Fame is not <qex>silent</qex>.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>Keeping at rest; inactive; calm; undisturbed; <as>as, the wind is <ex>silent</ex></as>.</def>  <rj><au>Parnell. Sir W. Raleigh.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <fld>(Pron.)</fld> <def>Not pronounced; having no sound; quiescent; <as>as, <it>e</it> is <ex>silent</ex> in <ldquo/fable.<rdquo/</as></def><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <def>Having no effect; not operating; inefficient.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>Cause . . . <qex>silent</qex>, virtueless, and dead.</q> <rj><qau>Sir W. Raleigh.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Silent partner</b></col>. <cd>See <cref>Dormant partner</cref>, under <er>Dormant</er>.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><syn><b>Syn.</b> -- Mute; taciturn; dumb; speechless; quiet; still. See <er>Mute</er>, and <er>Taciturn</er>.</syn><br/
[<source>1913 Webster</source>]</p>

<p><ent>Silent</ent><br/
<hw>Si"lent</hw>, <pos>n.</pos> <def>That which is silent; a time of silence.</def> <mark>[R.]</mark> <ldquo/The <xex>silent</xex> of the night.<rdquo/  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Silentiary</ent><br/
<hw>Si*len"ti*a*ry</hw> <pr>(s<isl/*l<ecr/n"sh<icr/*<asl/*r<ycr/)</pr>, <pos>n.</pos> <ety>[L. <ets>silentiarius</ets>: cf. F. <ets>silenciaire</ets>. See <er>Silence</er>.]</ety> <def>One appointed to keep silence and order in court; also, one sworn not to divulge secrets of state.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Silentious</ent><br/
<hw>Si*len"tious</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>silentiosus</ets>: cf. F. <ets>silencieux</ets>.]</ety> <def>Habitually silent; taciturn; reticent.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Silently</ent><br/
<hw>Si"lent*ly</hw> <pr>(?)</pr>, <pos>adv.</pos> <def>In a silent manner.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Silentness</ent><br/
<hw>Si"lent*ness</hw>, <pos>n.</pos> <def>State of being silent; silence.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Silenus</ent><br/
<hw>Si*le"nus</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>Silenus</ets> the tutor and attendant of Bacchus.]</ety> <fld>(Zool.)</fld> <def>See <er>Wanderoo</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Silesia</ent><br/
<hw>Si*le"si*a</hw> <pr>(?)</pr>, <pos>n.</pos> <sn>1.</sn> <def>A kind of linen cloth, originally made in <xex>Silesia</xex>, a province of Prussia.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A twilled cotton fabric, used for dress linings.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Silesian</ent><br/
<hw>Si*le"si*an</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Of or pertaining to Silesia.</def> -- <def2><pos>n.</pos> <def>A native or inhabitant of Silesia.</def></def2><br/
[<source>1913 Webster</source>]</p>

<p><ent>Silex</ent><br/
<hw>Si"lex</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L., a finit, a pebblestone.]</ety> <fld>(Min.)</fld> <def>Silica, <chform>SiO2</chform> as found in nature, constituting quarz, and most sands and sandstones. See <er>Silica</er>, and <er>Silicic</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Silhouette</ent><br/
<hw>Sil`hou*ette"</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F.; -- so called from Etienne de <ets>Silhoutte</ets>, a French minister of finance in 1759, whise diversion it was to make such portraits on the walls of his apartments.]</ety> <def>A representation of the outlines of an object filled in with a black color; a profile portrait in black, such as a shadow appears to be.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Silhouette</ent><br/
<hw>Sil`hou*ette"</hw>, <pos>v. t.</pos> <def>To represent by a silhouette; to project upon a background, so as to be like a silhouette.</def> <mark>[Recent]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>A flock of roasting vultures <qex>silhouetted</qex> on the sky.</q> <rj><qau>The Century.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Silica</ent><br/
<hw>Sil"i*ca</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[NL., from L. <ets>silex</ets>, <ets>silics</ets>, a flint.]</ety> <fld>(Chem.)</fld> <def>Silicon dioxide, SiO<?/. It constitutes ordinary quartz (also opal and tridymite), and is artifically prepared as a very fine, white, tasteless, inodorous powder.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Silicate</ent><br/
<hw>Sil"i*cate</hw> <pr>(?)</pr>, <pos>n.</pos><ety>[Cf. F. <ets>silicate</ets>.]</ety> <fld>(Chem.)</fld> <def>A salt of silicic acid.</def><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ In mineralogical chemistry the silicates include; the <xex>unisilicates</xex> or <xex>orthosilicates</xex>, salts of orthosilicic acid; the <xex>bisilicates</xex> or <xex>metasilicates</xex>, salts of metasilicic acid; the <xex>polysilicates</xex> or <xex>acid silicates</xex>, salts of the polysilicic acids; the <xex>basic silicates</xex> or <xex>subsilicates</xex>, in which the equivalent of base is greater than would be required to neutralize the acid; and the <xex>hydrous silicates</xex>, including the zeolites and many hydrated decomposition products.</note><br/
[<source>1913 Webster</source>]</p>

<p><ent>Silicated</ent><br/
<hw>Sil"i*ca`ted</hw> <pr>(?)</pr>, <pos>a.</pos> <fld>(Chem.)</fld> <def>Combined or impregnated with silicon or silica; <as>as, <ex>silicated</ex> hydrogen; <ex>silicated</ex> rocks</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Silicated soap</b></col>, <cd>a hard soap containing silicate of soda.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Silicatization</ent><br/
<hw>Sil`i*ca*ti*za"tion</hw> <pr>(?)</pr>, <pos>n.</pos> <def>Silicification.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Silicea</ent><br/
||<hw>Si*lic"e*a</hw> <pr>(?)</pr>, <pos>n. pl.</pos> <ety>[NL.]</ety> <fld>(Zool.)</fld> <def>Same as <er>Silicoidea</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Siliceous</ent><br/
<hw>Si*li"ceous</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>siliceus</ets>, fr. <ets>silex</ets>, <ets>silicis</ets>, a flint.]</ety> <def>Of or pertaining to silica; containing silica, or partaking of its nature.</def> <altsp>[Written also <asp>silisious</asp>.]</altsp><br/
[<source>1913 Webster</source>]</p>

<p><ent>Silicic</ent><br/
<hw>Si*lic"ic</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>silex</ets>, <ets>silicis</ets>, a flint: cf. F. <ets>silicique</ets>.]</ety> <fld>(Chem.)</fld> <def>Pertaining to, derived from, or resembling, silica; specifically, designating compounds of silicon; <as>as, <ex>silicic</ex> acid</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Silicic acid</b></col> <fld>(Chem.)</fld>, <cd>an amorphous gelatinous substance, <chform>Si(HO)4</chform>, very unstable and easily dried to silica, but forming many stable salts; -- called also <altname>orthosilicic acid</altname> <it>or</it> <altname>normal silicic acid</altname>.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Silicicalcareous</ent><br/
<hw>Si*lic`i*cal*ca"re*ous</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Consisting of silica and calcareous matter.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Silicide</ent><br/
<hw>Sil"i*cide</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Chem.)</fld> <def>A binary compound of silicon, or one regarded as binary.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Hydrogen silicide</b></col> <fld>(Chem.)</fld>, <cd>a colorless, spontaneously inflammable gas, <chform>SiH4</chform>, produced artifically from silicon, and analogous to methane; -- called also <altname>silico-methane</altname>, <altname>silicon hydride</altname>, and formerly <altname>siliciureted hydrogen</altname>.</cd></cs><-- now called silane, silicon hydride, or silicane.  The term silane is used as the theoretical parent compound of a large series of derivatives in which one or more of the hydrogens are substituted; the term is also used generically to refer to any one of a large series of silicon compounds, including a series containing silicon-silicon bonds, analogous to the compounds containing carbon. --><br/
[<source>1913 Webster</source>]</p>

<p><ent>Siliciferous</ent><br/
<hw>Sil`i*cif"er*ous</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>silex</ets>, <ets>silicis</ets>, a flint + <ets>-ferous</ets>.]</ety> <def>Producing silica; united with silica.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Silicification</ent><br/
<hw>Si*lic`i*fi*ca"tion</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[See <er>Silicify</er>.]</ety> <fld>(Chem.)</fld> <def>Thae act or process of combining or impregnating with silicon or silica; the state of being so combined or impregnated; <as>as, the <ex>silicification</ex> of wood</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Silicified</ent><br/
<hw>Si*lic"i*fied</hw> <pr>(?)</pr>, <pos>a.</pos> <fld>(Chem.)</fld> <def>Combined or impregnated with silicon or silica, especially the latter; <as>as, <ex>silicified</ex> wood</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Silicify</ent><br/
<hw>Si*lic"i*fy</hw> <pr>(?)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Silicified</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Silicifying</conjf> <pr>(?)</pr>.]</vmorph> <ety>[L. <ets>silex</ets>, <ets>silicis</ets>, a flint + <ets>-fy</ets>: cf. F. <ets>silicifier</ets>.]</ety> <fld>(Chem.)</fld> <def>To convert into, or to impregnate with, silica, or with the compounds of silicon.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The specimens found . . . are completely <qex>silicified</qex>.</q> <rj><qau>Say.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ The silica may take the form of agate, chalcedony, flint, hornstone, or crystalline quartz.</note><br/
[<source>1913 Webster</source>]</p>

<p><ent>Silicify</ent><br/
<hw>Si*lic"i*fy</hw>, <pos>v. i.</pos> <def>To become converted into silica, or to be impregnated with silica.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Silicioidea</ent><br/
||<hw>Sil`i*ci*oi"de*a</hw> <pr>(?)</pr>, <pos>n. pl.</pos> <fld>(Zool.)</fld> <def>Same as <er>Silicoidea</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Silicious</ent><br/
<hw>Si*li"cious</hw> <pr>(?)</pr>, <pos>a.</pos> <def>See <er>Siliceous</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Silicispongiae</ent><br/
||<hw>Sil`i*ci*spon"gi*ae</hw> <pr>(?)</pr>, <pos>n. pl.</pos> <ety>[NL. See <er>Silex</er>, and <er>Sponge</er>.]</ety> <fld>(Zool.)</fld> <def>Same as <er>Silicoidea</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Silicited</ent><br/
<hw>Si*lic"it*ed</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Silicified.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Silicium</ent><br/
<hw>Si*lic"i*um</hw> <pr>(?)</pr>, <pos>n.</pos> <def>See <er>Silicon</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Siliciureted</ent><br/
<hw>Si*lic"i*u*ret`ed</hw> <pr>(?)</pr>, <pos>a.</pos> <altsp>[Written also <asp>siliciuretted</asp>.]</altsp> <fld>(Old. Chem.)</fld> <def>Combined or impregnated with silicon.</def> <mark>[Obsoles.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Siliciureted hydrogen</b></col>. <fld>(Chem.)</fld> <cd>Hydrogen silicide.</cd> <mark>[Obs.]</mark></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Silicle</ent><br/
<hw>Sil"i*cle</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>silicula</ets>, dim. of <ets>siliqua</ets> a pod or husk: cf. F. <ets>silicule</ets>.]</ety> <fld>(Bot.)</fld> <def>A seed vessel resembling a silique, but about as broad as it is long. See <er>Silique</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Silico-</ent><br/
<hw>Sil"i*co-</hw> <pr>(?)</pr>. <fld>(Chem.)</fld> <def>A combining form (also used adjectively) denoting <xex>the presence of silicon</xex> or <xex>its compounds</xex>; <as>as, <ex>silico</ex>benzoic, <ex>silico</ex>fluoride, etc.</as></def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Silicofluoric</ent><br/
<hw>Sil`i*co*flu*or"ic</hw> <pr>(?)</pr>, <pos>a.</pos> <fld>(Chem.)</fld> <def>Containing, or composed of, silicon and fluorine; especially, denoting the compounds called <xex>silicofluorides</xex>.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Silicofluoric acid</b></col> <fld>(Chem.)</fld>, <cd>a compound of hydrofluoric acid and silicon fluoride, known only in watery solution. It is produced by the action of silicon fluoride on water, and is regarded as an acid, <chform>H2SiF6</chform>, and the type and origin of the silicofluorides.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Silicofluoride</ent><br/
<hw>Sil`i*co*flu"or*ide</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Chem.)</fld> <def>A fluosilicate; a salt of silicofluoric acid.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Silicoidea</ent><br/
||<hw>Sil`i*coi"de*a</hw> <pr>(?)</pr>, <pos>n. pl.</pos> <ety>[NL. See <er>Silex</er>, and <er>-oid</er>.]</ety> <fld>(Zool.)</fld> <def>An extensive order of Porifera, which includes those that have the skeleton composed mainly of siliceous fibers or spicules.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Silicon</ent><br/
<hw>Sil"i*con</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[See <er>Silica</er>.]</ety> <fld>(Chem.)</fld> <def>A nonmetalic element analogous to carbon. It always occurs combined in nature, and is artificially obtained in the free state, usually as a dark brown amorphous powder, or as a dark crystalline substance with a meetallic luster. Its oxide is silica, or common quartz, and in this form, or as silicates, it is, next to oxygen, the most abundant element of the earth's crust. Silicon is characteristically the element of the mineral kingdom, as carbon is of the organic world. Symbol Si. Atomic weight 28. Called also <altname>silicium</altname>.</def>
<-- it is used as the basis for the most common type of transistors, in the form of a highly purified silicon impregnated with small quantities of imourities such as phosphorus or antimony, giving it special semiconductor properties.  In this application, it forms the primary basis for the modern (post-1970) electronics industry, with integrated circuits containing millions of electronic componnets being imprinted by special processes on silicon chips less than one inch in diameter.  --><br/
[<source>1913 Webster</source>]</p>

<p><ent>Silicotungstic</ent><br/
<hw>Sil`i*co*tung"stic</hw> <pr>(?)</pr>, <pos>a.</pos> <fld>(Chem.)</fld> <def>Of, pertaining to, or designating, any one of a series of double acids of silicon and tungsten, known in the free state, and also in their salts (called <xex>silicotungstates</xex>).</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Silicula</ent><br/
||<hw>Si*lic"u*la</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L.]</ety> <fld>(Bot.)</fld> <def>A silicle.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Silicule</ent><br/
<hw>Sil"i*cule</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Bot.)</fld> <def>A silicle.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Siliculose</ent><br/
<hw>Si*lic"u*lose`</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[NL. <ets>siliculosus</ets>, fr. L. <ets>silicula</ets>: cf. F. <ets>siliculeux</ets>. See <er>Silicle</er>.]</ety> <sn>1.</sn> <fld>(Bot.)</fld> <def>Bearing silicles; pertaining to, or resembling, silicles.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Full of, or consisting of, husks; husky.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Siliginose</ent><br/
<hw>Si*lig"i*nose`</hw> <pr>(?)</pr>, <pos>a.</pos><ety>[L. <ets>siligineus</ets>, fr. <ets>siligo</ets>, <ets>-inis</ets>, fine and very white wheat.]</ety> <def>Made of fine wheat.</def> <mark>[Obs.]</mark>  <rj><au>Bailey.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Siling</ent><br/
<hw>Sil"ing</hw> <pr>(?)</pr>, <def><pos>a. & n.</pos> from <er>Sile</er> to strain.</def> <mark>[Obs. or Prov. Eng.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Siling dish</b></col>, <cd>a colander.</cd> <mark>[Obs. or Prov. Eng.]</mark></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Siliqua</ent><br/
<hw>Sil"i*qua</hw> <pr>(?)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Siliquae</plw> <pr>(#)</pr>.</plu> <ety>[L. See <er>Silique</er>.]</ety> <sn>1.</sn> <fld>(Bot.)</fld> <def>Same as <er>Silique</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A weight of four grains; a carat; -- a term used by jewelers, and refiners of gold.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Silique</ent><br/
<hw>Sil"ique</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>siliqua</ets> a pod or husk, a very small weight or measure: cf. F. <ets>silique</ets>.]</ety> <fld>(Bot.)</fld> <def>An oblong or elongated seed vessel, consisting of two valves with a dissepiment between, and opening by sutures at either margin. The seeds are attached to both edges of the dissepiment, alternately upon each side of it.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Siliquiform</ent><br/
<hw>Sil"i*qui*form</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[<ets>Silique</ets> + <ets>-form</ets>: cf. F. <ets>siliquiforme</ets>.]</ety> <fld>(Bot.)</fld> <def>Having the form of a silique.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Siliquosa</ent><br/
||<hw>Sil`i*quo"sa</hw> <pr>(?)</pr>, <pos>n. pl.</pos> <ety>[NL. See <er>Siliquose</er>.]</ety> <fld>(Bot.)</fld> <def>A Linnaean order of plants including those which bear siliques.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Siliquous</ent><br/
<ent>Siliquose</ent><br/
<mhw>{ <hw>Sil"i*quose`</hw> <pr>(?)</pr>, <hw>Sil"i*quous</hw> <pr>(?)</pr>, }</mhw> <pos>a.</pos> <ety>[NL. <ets>siliquosus</ets>: cf.  F. <ets>siliqueux</ets>.]</ety> <fld>(Bot.)</fld> <def>Bearing siliques; <as>as, <ex>siliquose</ex> plants</as>; pertaining to, or resembling, siliques; <as>as, <ex>siliquose</ex> capsules</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Silk</ent><br/
<hw>Silk</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[OE. <ets>silk</ets>, <ets>selk</ets>, AS. <ets>seolc</ets>, <ets>seoloc</ets>; akin to Icel. <ets>silki</ets>, SW. & Dan. <ets>silke</ets>; prob. through Slavic from an Oriental source; cf. Lith. <ets>szilkai</ets>, Russ. <ets>shelk'</ets>, and also L. <ets>sericum</ets> Seric stuff, silk.  Cf. <er>Sericeous</er>. <er>Serge</er> a woolen stuff.]</ety> <sn>1.</sn> <def>The fine, soft thread produced by various species of caterpillars in forming the cocoons within which the worm is inclosed during the pupa state, especially that produced by the larvae of <spn>Bombyx mori</spn>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Hence, thread spun, or cloth woven, from the above-named material.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>That which resembles silk, as the filiform styles of the female flower of maize.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Raw silk</b></col>, <cd>silk as it is wound off from the cocoons, and before it is manufactured.</cd> -- <col><b>Silk cotton</b></col>, <cd>a cottony substance enveloping the seeds of the silk-cotton tree.</cd> -- <col><b>Silk-cotton tree</b></col> <fld>(Bot.)</fld>, <cd>a name for several tropical trees of the genera <gen>Bombax</gen> and <gen>Eriodendron</gen>, and belonging to the order <ord>Bombaceae</ord>. The trees grow to an immense size, and have their seeds enveloped in a cottony substance, which is used for stuffing cushions, but can not be spun.</cd> -- <col><b>Silk flower</b></col>. <fld>(Bot.)</fld> <sd>(a)</sd> <cd>The silk tree</cd>. <sd>(b)</sd> <cd>A similar tree (<spn>Calliandra trinervia</spn>) of Peru.</cd> -- <col><b>Silk fowl</b></col> <fld>(Zool.)</fld>, <cd>a breed of domestic fowls having silky plumage.</cd> -- <col><b>Silk gland</b></col> <fld>(Zool.)</fld>, <cd>a gland which secretes the material of silk, as in spider or a silkworm; a sericterium.</cd> -- <col><b>Silk gown</b></col>, <cd>the distinctive robe of a barrister who has been appointed king's or queen's counsel; hence, the counsel himself. Such a one has precedence over mere barristers, who wear <xex>stuff gowns</xex>.</cd> <mark>[Eng.]</mark> -- <col><b>Silk grass</b></col> <fld>(Bot.)</fld>, <cd>a kind of grass (<spn>Stipa comata</spn>) of the Western United States, which has very long silky awns. The name is also sometimes given to various species of the genera <gen>Aqave</gen> and <gen>Yucca</gen>.</cd> -- <col><b>Silk moth</b></col> <fld>(Zool.)</fld>, <cd>the adult moth of any silkworm.  See <er>Silkworm</er>.</cd> -- <col><b>Silk shag</b></col>, <cd>a coarse, rough-woven silk, like plush, but with a stiffer nap.</cd> -- <col><b>Silk spider</b></col> <fld>(Zool.)</fld>, <cd>a large spider (<spn>Nephila plumipes</spn>), native of the Southern United States, remarkable for the large quantity of strong silk it produces and for the great disparity in the sizes of the sexes.</cd> -- <mcol><col><b>Silk thrower</b></col>, <col><b>Silk throwster</b></col></mcol>, <cd>one who twists or spins silk, and prepares it for weaving.</cd> <au>Brande & C.</au> -- <col><b>Silk tree</b></col> <fld>(Bot.)</fld>, <cd>an Asiatic leguminous tree (<spn>Albizzia Julibrissin</spn>) with finely bipinnate leaves, and large flat pods; -- so called because of the abundant long silky stamens of its blossoms. Also called <altname>silk flower</altname>.</cd> -- <col><b>Silk vessel</b></col>. <fld>(Zool.)</fld> <cd>Same as <cref>Silk gland</cref>, above.</cd> -- <col><b>Virginia silk</b></col> <fld>(Bot.)</fld>, <cd>a climbing plant (<spn>Periploca Gr<ae/ca</spn>) of the Milkweed family, having a silky tuft on the seeds. It is native in Southern Europe.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Silken</ent><br/
<hw>Silk"en</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[AS. <ets>seolcen</ets>, <ets>seolocen</ets>.]</ety> <sn>1.</sn> <def>Of or pertaining to silk; made of, or resembling, silk; <as>as, <ex>silken</ex> cloth; a <ex>silken</ex> veil</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Fig.: Soft; delicate; tender; smooth; <as>as, <ex>silken</ex> language</as>.</def> <ldquo/<xex>Silken</xex> terms precise.<rdquo/  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>Dressed in silk.</def> <ldquo/A . . . <xex>silken</xex> wanton.<rdquo/  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Silken</ent><br/
<hw>Silk"en</hw>, <pos>v. t.</pos> <def>To render silken or silklike.</def>  <rj><au>Dyer.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Silkiness</ent><br/
<hw>Silk"i*ness</hw> <pr>(?)</pr>, <pos>n.</pos> <sn>1.</sn> <def>The quality or state of being silky or silken; softness and smoothness.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Fig.: Effeminacy; weakness.</def> <mark>[R.]</mark>  <rj><au>B. Jonson.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Silkman</ent><br/
<hw>Silk"man</hw> <pr>(?)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Silkmen</plw> <pr>(<?/)</pr>.</plu> <def>A dealer in silks; a silk mercer.</def>  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><-- p. 1341 --></p>

<p><ent>Silkness</ent><br/
<hw>Silk"ness</hw> <pr>(?)</pr>, <pos>n.</pos> <def>Silkiness.</def> <mark>[Obs.]</mark>  <rj><au>B. Jonson.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Silk-stocking</ent><br/
<hw>Silk"-stock`ing</hw>, <pos>a.</pos> <def>Wearing silk stockings (which among men were formerly worn chiefly by the luxurious or aristocratic); hence, elegantly dressed; aristocratic; luxurious; -- chiefly applied to men, often by way of reproach.</def></p>

<p><q>[They] will find their levees crowded with <qex>silk-stocking</qex> gentry, but no yeomanry; an army of officers without soldiers.</q>  <rj><qau>Jefferson.</qau></rj><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Silkweed</ent><br/
<hw>Silk"weed`</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Bot.)</fld> <def>Any plant of the genera <gen>Asclepias</gen> and <gen>Acerates</gen> whose seed vessels contain a long, silky down; milkweed.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Silkworm</ent><br/
<hw>Silk"worm`</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[AS. <ets>seolcwyrm</ets>.]</ety> <fld>(Zool.)</fld> <def>The larva of any one of numerous species of bombycid moths, which spins a large amount of strong silk in constructing its cocoon before changing to a pupa.</def><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ The common species (<spn>Bombyx mori</spn>) feeds on the leaves of the white mulberry tree. It is native of China, but has long been introduced into other countries of Asia and Europe, and is reared on a large scale. In America it is reared only to small extent. The Ailanthus silkworm (<spn>Philosamia cynthia</spn>) is a much larger species, of considerable importance, which has been introduced into Europe and America from China. The most useful American species is the Polyphemus. See <er>Polyphemus</er>.</note><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Pernyi silkworm</b></col>, <cd>the larva of the Pernyi moth. See <er>Pernyi moth</er>.</cd> -- <col><b>Silkworm gut</b></col>, <cd>a substance prepared from the contents of the silk glands of silkworms and used in making lines for angling. See <er>Gut</er>.</cd> -- <col><b>Silkworm rot</b></col>, <cd>a disease of silkworms; muscardine.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Silky</ent><br/
<hw>Silk"y</hw> <pr>(?)</pr>, <pos>a.</pos> <amorph>[<pos>Compar.</pos> <adjf>Silkier</adjf> <pr>(?)</pr>; <pos>superl.</pos> <adjf>Silkiest</adjf>.]</amorph> <sn>1.</sn> <def>Of or pertaining to silk; made of, or resembling, silk; silken; silklike; <as>as, a <ex>silky</ex> luster</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Hence, soft and smooth; <as>as, <ex>silky</ex> wine</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>Covered with soft hairs pressed close to the surface, as a leaf; sericeous.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Silky oak</b></col> <fld>(Bot.)</fld>, <cd>a lofty Australian tree (<spn>Grevillea robusta</spn>) with silky tomentose lobed or incised leaves. It furnishes a valuable timber.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sill</ent><br/
<hw>Sill</hw> <pr>(s<icr/l)</pr>, <pos>n.</pos> <ety>[OE. <ets>sille</ets>, <ets>sylle</ets>, AS. <ets>syl</ets>, <ets>syll</ets>; akin to G. <ets>schwelle</ets>, OHG. <ets>swelli</ets>, Icel. <ets>syll</ets>, <ets>svill</ets>, Sw. <ets>syll</ets>, Dan. <ets>syld</ets>, Goth. <ets>gasuljan</ets> to lay a foundation, to found.]</ety> <def>The basis or foundation of a thing; especially, a horizontal piece, as a timber, which forms the lower member of a frame, or supports a structure; <as>as, the <ex>sills</ex> of a house, of a bridge, of a loom, and the like</as>.</def> Hence: <sd>(a)</sd> <def>The timber or stone at the foot of a door; the threshold.</def> <sd>(b)</sd> <def>The timber or stone on which a window frame stands; or, the lowest piece in a window frame.</def> <sd>(c)</sd> <def>The floor of a gallery or passage in a mine.</def> <sd>(d)</sd> <def>A piece of timber across the bottom of a canal lock for the gates to shut against.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Sill course</b></col> <fld>(Arch.)</fld>, <cd>a horizontal course of stone, terra cotta, or the like, built into a wall at the level of one or more window sills, these sills often forming part of it.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sill</ent><br/
<hw>Sill</hw>, <pos>n.</pos> <ety>[Cf. <er>Thill</er>.]</ety> <def>The shaft or thill of a carriage.</def> <mark>[Prov. Eng.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sill</ent><br/
<hw>Sill</hw>, <pos>n.</pos> <ety>[Cf. 4th <er>Sile</er>.]</ety> <def>A young herring.</def> <mark>[Eng.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sillabub</ent><br/
<hw>Sil"la*bub</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Cf. <ets>sile</ets> to strain, and <ets>bub</ets> liquor, also Prov. E. <ets>sillibauk</ets>.]</ety> <def>A dish made by mixing wine or cider with milk, and thus forming a soft curd; also, sweetened cream, flavored with wine and beaten to a stiff froth.</def> <altsp>[Written also <asp>syllabub</asp>.]</altsp><br/
[<source>1913 Webster</source>]</p>

<p><ent>Siller</ent><br/
<hw>Sil"ler</hw> <pr>(?)</pr>, <pos>n.</pos> <def>Silver.</def> <mark>[Scot.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sillily</ent><br/
<hw>Sil"li*ly</hw> <pr>(?)</pr>, <pos>adv.</pos> <ety>[From <er>Silly</er>.]</ety> <def>In a silly manner; foolishly.</def>  <rj><au>Dryden.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sillimanite</ent><br/
<hw>Sil"li*man*ite</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[After Benjamin <ets>Siliman</ets>, an American meneralogist.]</ety> <fld>(Min.)</fld> <def>Same as <er>Fibrolite</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Silliness</ent><br/
<hw>Sil"li*ness</hw>, <pos>n.</pos> <def>The quality or state of being silly.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sillock</ent><br/
<hw>Sil"lock</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Zool.)</fld> <def>The pollock, or coalfish.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sillon</ent><br/
<hw>Sil"lon</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F., a furrow.]</ety> <fld>(Fort.)</fld> <def>A work raised in the middle of a wide ditch, to defend it.</def>  <rj><au>Crabb.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Silly</ent><br/
<hw>Sil"ly</hw>, <pos>a.</pos> <amorph>[<pos>Compar.</pos> <adjf>Sillier</adjf> <pr>(?)</pr>; <pos>superl.</pos> <adjf>Silliest</adjf>.]</amorph> <ety>[OE. <ets>seely</ets>, <ets>sely</ets>, AS. <ets>s<?/lig</ets>, ge<ets>s<?/lig</ets>, happy, good, fr. <ets>s<?/l</ets>, <ets>s<?/l</ets>, good, happy, <ets>s<?/l</ets> good fortune, happines; akin to OS. <ets>s<amac/lig</ets>, a, good, happy, D. <ets>zalig</ets> blessed, G. <ets>selig</ets>, OHG. <ets>s<amac/l<imac/g</ets>, Icel. <ets>s<?/l</ets>, Sw. <ets>s<aum/ll</ets>, Dan. <ets>salig</ets>, Goth. <ets>s<?/ls</ets> good, kind, and perh. also to L. <ets>sollus</ets> whole, entire, Gr. <?/<?/<?/, Skr. <ets>sarva</ets>.  Cf. <er>Seel</er>, <pos>n.</pos>]</ety> <sn>1.</sn> <def>Happy; fortunate; blessed.</def> <mark>[Obs.]</mark>  <rj><au>Chaucer.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Harmless; innocent; inoffensive.</def> <mark>[Obs.]</mark> <ldquo/This <xex>silly</xex>, innocent Custance.<rdquo/  <rj><au>Chaucer.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>The <qex>silly</qex> virgin strove him to withstand.</q> <rj><qau>Spenser.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>A <qex>silly</qex>, innocent hare murdered of a dog.</q> <rj><qau>Robynson (More's Utopia).</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>Weak; helpless; frail.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>After long storms . . . <br/
With which my <qex>silly</qex> bark was tossed sore.</q> <rj><qau>Spenser.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>The <qex>silly</qex> buckets on the deck.</q> <rj><qau>Coleridge.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>Rustic; plain; simple; humble.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>A fourth man, in a <qex>silly</qex>habit.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>All that did their <qex>silly</qex> thoughts so busy keep.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <def>Weak in intellect; destitute of ordinary strength of mind; foolish; witless; simple; <as>as, a <ex>silly</ex> woman</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>6.</sn> <def>Proceeding from want of understanding or common judgment; characterized by weakness or folly; unwise; absurd; stupid; <as>as, <ex>silly</ex> conduct; a <ex>silly</ex> question</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><syn><b>Syn.</b> -- Simple; brainless; witless; shallow; foolish; unwise; indiscreet. See <er>Simple</er>.</syn><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sillyhow</ent><br/
<hw>Sil"ly*how</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Prov. E. <ets>silly-hew</ets>; cf. AS. <ets>s<aemac/lig</ets> happy, good, and <ets>h<umac/fe</ets> a cap, hood. See <er>Silly</er>, <pos>a.</pos>]</ety> <def>A caul. See <er>Caul</er>, <pos>n.</pos>, 3.</def> <mark>[Obs. or Prov. Eng. & Scot.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Silo</ent><br/
<hw>Si"lo</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F.]</ety> <def>A pit or vat for packing away green fodder for winter use so as to exclude air and outside moisture. See <er>Ensilage</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Silt</ent><br/
<hw>Silt</hw> <pr>(s<icr/lt)</pr>, <pos>n.</pos> <ety>[OE. <ets>silte</ets> gravel, fr. <ets>silen</ets> to drain, E. <ets>sile</ets>; probably of Scand. origin; cf. Sw. <ets>sila</ets>, prob. akin to AS. <ets>se<oacute/n</ets> to filter, <ets>s<imac/gan</ets> to fall, sink, cause to sink, G. <ets>seihen</ets> to strain, to filter, OHG. <ets>s<imac/han</ets>, Icel. <ets>s<imac/a</ets>, Skr. <ets>sic</ets> to pour; cf. Gr. <grk>'ikma`s</grk> moisture.  Cf. <er>Sig</er>, <er>Sile</er>.]</ety> <def>Mud or fine earth deposited from running or standing water.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Silt</ent><br/
<hw>Silt</hw>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Silted</conjf>; <pos>p. pr. & vb. n.</pos> <conjf>Silting</conjf>.]</vmorph> <def>To choke, fill, or obstruct with silt or mud.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Silt</ent><br/
<hw>Silt</hw>, <pos>v. i.</pos> <def>To flow through crevices; to percolate.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Silty</ent><br/
<hw>Silt"y</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Full of silt; resembling silt.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Silundum</ent><br/
<hw>Si*lun"dum</hw> <pr>(?)</pr>, <pos>n.</pos>  <mark>[Trade name]</mark> <def>A form of silicon carbide, produced in the electric furnace, possessing great hardness, and high electrical resistance, and not subject to oxidation below 2880<deg/ F., or 1600<deg/ C.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Silure</ent><br/
<hw>Si*lure"</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>silurus</ets> a sort of river fish, Gr. <?/<?/<?/: cf. F. <ets>silure</ets>.]</ety> <fld>(Zool.)</fld> <def>A fish of the genus <gen>Silurus</gen>, as the sheatfish; a siluroid.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Silurian</ent><br/
<hw>Si*lu"ri*an</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[From L. <ets>Silures</ets>, a people who anciently inhabited a part of England and Wales.]</ety> <fld>(Geol.)</fld> <def>Of or pertaining to the country of the ancient Silures; -- a term applied to the earliest of the Paleozoic eras, and also to the strata of the era, because most plainly developed in that country.</def><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ The Silurian formation, so named by Murchison, is divided into the <xex>Upper Silurian</xex> and <xex>Lower Silurian</xex>. The lower part of the Lower Silurian, with some underlying beds, is now separated under the name <xex>Cambrian</xex>, first given by Sedwick. Recently the term <xex>Ordovician</xex> has been proposed for the Lower Silurian, leawing the original word to apply only to the Upper Silurian.</note><br/
[<source>1913 Webster</source>]</p>

<p><ent>Silurian</ent><br/
<hw>Si*lu"ri*an</hw>, <pos>n.</pos> <def>The Silurian age.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Siluridan</ent><br/
<hw>Si*lu"ri*dan</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Zool.)</fld> <def>Any fish of the family <fam>Siluridae</fam> or of the order Siluroidei.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Siluroid</ent><br/
<hw>Si*lu"roid</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[<ets>Silurus</ets> + <ets>-oid</ets>.]</ety> <fld>(Zool.)</fld> <def>Belonging to the Siluroidei, or Nematognathi, an order of fishes including numerous species, among which are the American catfishes and numerous allied fresh-water species of the Old World, as the sheatfish (<spn>Silurus glanis</spn>) of Europe.</def> -- <def2><pos>n.</pos> <def>A siluroid fish.</def></def2><br/
[<source>1913 Webster</source>]</p>

<p><ent>Siluroidei</ent><br/
||<hw>Sil`u*roi"de*i</hw> <pr>(?)</pr>, <pos>n. pl.</pos> <ety>[NL.]</ety> <fld>(Zool.)</fld> <def>An order of fishes, the Nematognathi.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Silurus</ent><br/
||<hw>Si*lu"rus</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. See <er>Silure</er>.]</ety> <fld>(Zool.)</fld> <def>A genus of large malacopterygious fishes of the order Siluroidei. They inhabit the inland waters of Europe and Asia.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Silva</ent><br/
<hw>Sil"va</hw> <pr>(?)</pr>, <pos>n.</pos>; <plu><it>pl.</it> E. <plw>Silvas</plw> <pr>(#)</pr>, L. <plw>Silvae</plw> <pr>(<?/)</pr>.</plu> <ety>[L., properly, a wood, forest.]</ety> <altsp>[Written also <asp>sylva</asp>.]</altsp> <fld>(Bot.)</fld> <sd>(a)</sd> <def>The forest trees of a region or country, considered collectively.</def> <sd>(b)</sd> <def>A description or history of the forest trees of a country.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Silvan</ent><br/
<hw>Sil"van</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>silva</ets>, less correctly <ets>sylva</ets>, a wood or grove, perh. akin to Gr. <grk>"y`lh</grk>; cf. L. <ets>Silvanus</ets> Silvanus the god of woods: cf. F. <ets>sylvain</ets> silvan.  Cf. <er>Savage</er>.]</ety> <def>Of or pertaining to woods; composed of woods or groves; woody.</def> <altsp>[Written also <asp>sylvan</asp>.]</altsp><br/
[<source>1913 Webster</source>]</p>

<p><q>Betwixt two rows of rocks, a <qex>silvan</qex> scene<br/
Appears above, and groves forever green.</q> <rj><qau>Dryden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Silvan</ent><br/
<hw>Sil"van</hw>, <pos>n.</pos> <fld>(Old Chem.)</fld> <def>See <er>Sylvanium</er>.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Silvanite</ent><br/
<hw>Sil"van*ite</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Min.)</fld> <def>See <er>Sylvanite</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Selvas</ent><br/
<ent>Silvas</ent><br/
<mhw>{ <hw>Sil"vas</hw> <pr>(?)</pr> <or/ <hw>Sel"vas</hw> <pr>(?)</pr>, }</mhw> <pos>n. pl.</pos> <ety>[L. <ets>silva</ets> a forest, Sp. <ets>selva</ets>.]</ety> <def>Vast woodland plains of South America.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Silvate</ent><br/
<hw>Sil"vate</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Chem.)</fld> <def>Same as <er>Sylvate</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Silver</ent><br/
<hw>Sil"ver</hw> <pr>(s<icr/l"v<etil/r)</pr>, <pos>n.</pos> <ety>[OE. <ets>silver</ets>, <ets>selver</ets>, <ets>seolver</ets>, AS. <ets>seolfor</ets>, <ets>siolfur</ets>, <ets>siolufr</ets>, <ets>silofr</ets>, <ets>sylofr</ets>; akin to OS. <ets>silubar</ets>, OFries. <ets>selover</ets>, D. <ets>zilver</ets>, LG. <ets>sulver</ets>, OHG. <ets>silabar</ets>, <ets>silbar</ets>, G. <ets>silber</ets>, Icel. <ets>silfr</ets>, Sw. <ets>silfver</ets>, Dan. <ets>s<oum/lv</ets>, Goth. <ets>silubr</ets>, Russ. <ets>serebro</ets>, Lith. <ets>sidabras</ets>; of unknown origin.]</ety> <sn>1.</sn> <fld>(Chem.)</fld> <def>A soft white metallic element, sonorous, ductile, very malleable, and capable of a high degree of polish. It is found native, and also combined with sulphur, arsenic, antimony, chlorine, etc., in the minerals argentite, proustite, pyrargyrite, ceragyrite, etc.  Silver is one of the <ldquo/noble<rdquo/ metals, so-called, not being easily oxidized, and is used for coin, jewelry, plate, and a great variety of articles. Symbol Ag (<xex>Argentum</xex>). Atomic weight 107.7. Specific gravity 10.5.</def><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ Silver was known under the name of <xex>luna</xex> to the ancients and also to the alchemists. Some of its compounds, as the halogen salts, are remarkable for the effect of light upon them, and are used in photography.</note><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Coin made of silver; silver money.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>Anything having the luster or appearance of silver.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>The color of silver.</def><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ <xex>Silver</xex> is used in the formation of many compounds of obvious meaning; as, <xex>silver</xex>-armed, <xex>silver</xex>-bright, <xex>silver</xex>-buskined, <xex>silver</xex>-coated, <xex>silver</xex>-footed, <xex>silver</xex>-haired, <xex>silver</xex>-headed, <xex>silver</xex>-mantled, <xex>silver</xex>-plated, <xex>silver</xex>-slippered, <xex>silver</xex>-sounding, <xex>silver</xex>-studded, <xex>silver</xex>-tongued, <xex>silver</xex>-white. See <er>Silver</er>, <pos>a.</pos></note><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Black silver</b></col> <fld>(Min.)</fld>, <cd>stephanite; -- called also <altname>brittle silver ore</altname>, or <altname>brittle silver glance</altname>.</cd> -- <col><b>Fulminating silver</b></col>. <fld>(Chem.)</fld> <sd>(a)</sd> <cd>A black crystalline substance, <chform>Ag2O.(NH3)2</chform>, obtained by dissolving silver oxide in aqua ammonia. When dry it explodes violently on the slightest percussion.</cd> <sd>(b)</sd> <cd>Silver fulminate, a white crystalline substance, <chform>Ag2C2N2O2</chform>, obtained by adding alcohol to a solution of silver nitrate; -- also called <altname>fulminate of silver</altname>. When dry it is violently explosive.</cd> -- <col><b>German silver</b></col>. <fld>(Chem.)</fld> <cd>See under <er>German</er>.</cd> -- <col><b>Gray silver</b></col>. <fld>(Min.)</fld> <cd>See <er>Freieslebenite</er>.</cd> -- <col><b>Horn silver</b></col>. <fld>(Min.)</fld> <cd>See <er>Cerargyrite</er>.</cd> -- <col><b>King's silver</b></col>. <fld>(O. Eng. Law)</fld> <cd>See <er>Postfine</er>.</cd> -- <mcol><col><b>Red silver</b></col>, <it>or</it> <col><b>Ruby silver</b></col></mcol>. <fld>(Min.)</fld> <cd>See <er>Proustite</er>, and <er>Pyrargyrite</er>.</cd> -- <col><b>Silver beater</b></col>, <cd>one who beats silver into silver leaf or silver foil.</cd> -- <mcol><col><b>Silver glance</b></col>, <it>or</it> <col><b>Vitreous silver</b></col></mcol>. <fld>(Min.)</fld> <cd>See <er>Argentine</er>.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Silver</ent><br/
<hw>Sil"ver</hw>, <pos>a.</pos> <sn>1.</sn> <def>Of or pertaining to silver; made of silver; <as>as, <ex>silver</ex> leaf; a <ex>silver</ex> cup.</as></def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Resembling silver.</def> Specifically: <sd>(a)</sd> <def>Bright; resplendent; white.</def> <ldquo/<xex>Silver</xex> hair.<rdquo/  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Others, on <qex>silver</qex> lakes and rivers, bathed<br/
Their downy breast.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sd>(b)</sd> <def>Precious; costly.</def> <sd>(c)</sd> <def>Giving a clear, ringing sound soft and clear.</def> <ldquo/<xex>Silver</xex> voices.<rdquo/ <au>Spenser.</au> <sd>(d)</sd> <def>Sweet; gentle; peaceful.</def> <ldquo/<xex>Silver</xex> slumber.<rdquo/ <au>Spenser.</au><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>American silver fir</b></col> <fld>(Bot.)</fld>, <cd>the balsam fir. See under <er>Balsam</er>.</cd> -- <col><b>Silver age</b></col> <fld>(Roman Lit.)</fld>, <cd>the latter part (<sc>a. d.</sc> 14-180) of the classical period of Latinity, -- the time of writers of inferior purity of language, as compared with those of the previous <xex>golden age</xex>, so-called.</cd> -- <col><b>Silver-bell tree</b></col> <fld>(Bot.)</fld>, <cd>an American shrub or small tree (<spn>Halesia tetraptera</spn>) with white bell-shaped flowers in clusters or racemes; the snowdrop tree.</cd> -- <col><b>Silver bush</b></col> <fld>(Bot.)</fld>, <cd>a shrubby leguminous plant (<spn>Anthyllis Barba-Jovis</spn>) of Southern Europe, having silvery foliage.</cd> -- <col><b>Silver chub</b></col> <fld>(Zool.)</fld>, <cd>the fallfish.</cd> -- <col><b>Silver eel</b></col>. <fld>(Zool.)</fld> <sd>(a)</sd> <cd>The cutlass fish.</cd> <sd>(b)</sd> <cd>A pale variety of the common eel.</cd> -- <col><b>Silver fir</b></col> <fld>(Bot.)</fld>, <cd>a coniferous tree (<spn>Abies pectinata</spn>) found in mountainous districts in the middle and south of Europe, where it often grows to the height of 100 or 150 feet. It yields Burgundy pitch and Strasburg turpentine.</cd> -- <col><b>Silver foil</b></col>, <cd>foil made of silver.</cd> -- <col><b>Silver fox</b></col> <fld>(Zool.)</fld>, <cd>a variety of the common fox (<spn>Vulpes vulpes</spn>, variety <varn>argenteus</varn>) found in the northern parts of Asia, Europe, and America. Its fur is nearly black, with silvery tips, and is highly valued. Called also <altname>black fox</altname>, and <altname>silver-gray fox</altname>.</cd> -- <col><b>Silver gar</b></col>. <fld>(Zool.)</fld> <cd>See <er>Billfish</er> <sd>(a)</sd>.</cd> -- <col><b>Silver grain</b></col> <fld>(Bot.)</fld>, <cd>the lines or narrow plates of cellular tissue which pass from the pith to the bark of an exogenous stem; the medullary rays. In the wood of the oak they are much larger than in that of the beech, maple, pine, cherry, etc.</cd> -- <col><b>Silver grebe</b></col> <fld>(Zool.)</fld>, <cd>the red-throated diver. See <xex>Illust.</xex> under <er>Diver</er>.</cd> -- <col><b>Silver hake</b></col> <fld>(Zool.)</fld>, <cd>the American whiting.</cd> -- <col><b>Silver leaf</b></col>, <cd>leaves or sheets made of silver beaten very thin.</cd> -- <col><b>Silver lunge</b></col> <fld>(Zool.)</fld>, <cd>the namaycush.</cd> -- <col><b>Silver moonfish</b></col>.<fld>(Zool.)</fld> <cd>See <er>Moonfish</er> <sd>(b)</sd>.</cd> -- <col><b>Silver moth</b></col> <fld>(Zool.)</fld>, <cd>a lepisma.</cd> -- <col><b>Silver owl</b></col> <fld>(Zool.)</fld>, <cd>the barn owl.</cd> -- <col><b>Silver perch</b></col> <fld>(Zool.)</fld>, <cd>the mademoiselle, 2.</cd> -- <col><b>Silver pheasant</b></col> <fld>(Zool.)</fld>, <cd>any one of several species of beautiful crested and long-tailed Asiatic pheasants, of the genus <gen>Euplocamus</gen>.  They have the tail and more or less of the upper parts silvery white. The most common species (<spn>Euplocamus nychtemerus</spn>) is native of China.</cd> -- <col><b>Silver plate</b></col>, <sd>(a)</sd> <cd>domestic utensils made of a base metal coated with silver.</cd> <sd>(b)</sd> <cd>a plating of silver on a base metal.</cd>  -- <col><b>Silver plover</b></col> <fld>(Zool.)</fld>, <cd>the knot.</cd> -- <col><b>Silver salmon</b></col> <fld>(Zool.)</fld>, <cd>a salmon (<spn>Oncorhynchus kisutch</spn>) native of both coasts of the North Pacific. It ascends all the American rivers as far south as the Sacramento. Called also <altname>kisutch</altname>, <altname>whitefish</altname>, and <altname>white salmon</altname>.</cd> -- <col><b>Silver shell</b></col> <fld>(Zool.)</fld>, <cd>a marine bivalve of the genus Anomia. See <er>Anomia</er>.</cd><-- ##?why Anomia not italicised? --> -- <col><b>Silver steel</b></col>, <cd>an alloy of steel with a very small proportion of silver.</cd> -- <col><b>Silver stick</b></col>, <cd>a title given to the title field officer of the Life Guards when on duty at the palace.</cd> <mark>[Eng.]</mark> <au>Thackeray.</au> -- <col><b>Silver tree</b></col> <fld>(Bot.)</fld>, <cd>a South African tree (<spn>Leucadendron argenteum</spn>) with long, silvery, silky leaves.</cd> -- <col><b>Silver trout</b></col>, <fld>(Zool.)</fld> <cd>See <er>Trout</er>.</cd> -- <col><b>Silver wedding</b></col>. <cd>See under <er>Wedding</er>.</cd> -- <col><b>Silver whiting</b></col> <fld>(Zool.)</fld>, <cd>a marine sciaenoid food fish (<spn>Menticirrus littoralis</spn>) native of the Southern United States; -- called also <altname>surf whiting</altname>.</cd> -- <col><b>Silver witch</b></col> <fld>(Zool.)</fld>, <cd>A lepisma.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Silver</ent><br/
<hw>Sil"ver</hw> <pr>(?)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Silvered</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Silvering</conjf>.]</vmorph> <sn>1.</sn> <def>To cover with silver; to give a silvery appearance to by applying a metal of a silvery color; <as>as, to <ex>silver</ex> a pin; to <ex>silver</ex> a glass mirror plate with an amalgam of tin and mercury.</as></def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To polish like silver; to impart a brightness to, like that of silver.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>And smiling calmness <qex>silvered</qex> o'er the deep.</q> <rj><qau>Pope.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>To make hoary, or white, like silver.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>His head was <qex>silvered</qex> o'er with age.</q> <rj><qau>Gay.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Silver</ent><br/
<hw>Sil"ver</hw>, <pos>v. i.</pos> <def>To acquire a silvery color.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>The eastern sky began to <qex>silver</qex> and shine.</q> <rj><qau>L. Wallace.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Silverback</ent><br/
<hw>Sil"ver*back`</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Zool.)</fld> <def>The knot.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Silverberry</ent><br/
<hw>Sil"ver*ber`ry</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Bot.)</fld> <def>A tree or shrub (<spn>Elaeagnus argentea</spn>) with silvery foliage and fruit.</def>  <rj><au>Gray.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Silverbill</ent><br/
<hw>Sil"ver*bill`</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Zool.)</fld> <def>An Old World finch of the genus <gen>Minia</gen>, as the <spn>Minia Malabarica</spn> of India, and <spn>Minia cantans</spn> of Africa.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Silverboom</ent><br/
<hw>Sil"ver*boom`</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[D. <ets>zilver</ets> silver + <ets>boom</ets> tree.]</ety> <fld>(Bot.)</fld> <def>See <er>Leucadendron</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Silver certificate</ent><br/
<hw>Sil"ver cer*tif"i*cate</hw>. <def>A certificate issued by a government that there has been deposited with it silver to a specified amount, payable to the bearer on demand. In the United States and its possessions, it is issued against the deposit of silver coin, and is not legal tender, but is receivable for customs, taxes, and all public dues. In the United States the redeemability in silver of <ex>silver certificates</ex> was discontinued in the 1970's; they are still (1997) accepted as money at the face dollar value, but cannot be redeemed in silver.</def><br/
[<source>Webster 1913 Suppl.</source> <source>+PJC</source>]</p>

<p><ent>Silverfin</ent><br/
<hw>Sil"ver*fin`</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Zool.)</fld> <def>A small North American fresh-water cyprinoid fish (<spn>Notropis Whipplei</spn>).</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Silverfish</ent><br/
<hw>Sil"ver*fish`</hw> <pr>(?)</pr>, <pos>n.</pos> <sn>1.</sn> <fld>(Zool.)</fld> <sd>(a)</sd> <def>The tarpum.</def> <sd>(b)</sd> <def>A white variety of the goldfish.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>one of a variety of insects of the order <ord>Thysanura</ord>, especially <spn>Lepisma saccharina</spn>, which may infest houses, and eats starched clothing and sized papers.  See <er>Lepisma.</er></def><br/
[<source>PJC</source>]</p>

<p><ent>Silver-gray</ent><br/
<hw>Sil"ver-gray`</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Having a gray color with a silvery luster; <as>as, <ex>silver-gray</ex> hair</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Silveriness</ent><br/
<hw>Sil"ver*i*ness</hw> <pr>(?)</pr>, <pos>n.</pos> <def>The state of being silvery.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Silvering</ent><br/
<hw>Sil"ver*ing</hw>, <pos>n.</pos> <fld>(Metal.)</fld> <def>The art or process of covering metals, wood, paper, glass, etc., with a thin film of metallic silver, or a substance resembling silver; also, the firm do laid on; <as>as, the <ex>silvering</ex> of a glass speculum</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Silverite</ent><br/
<hw>Sil"ver*ite</hw> <pr>(?)</pr>, <pos>n.</pos> <def>One who favors the use or establishment of silver as a monetary standard; -- so called by those who favor the gold standard.</def> <mark>[Colloq. or Cant]</mark><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Silverize</ent><br/
<hw>Sil"ver*ize</hw> <pr>(?)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Silverized</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Silverizing</conjf>.]</vmorph> <def>To cover with silver.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Silverless</ent><br/
<hw>Sil"ver*less</hw>, <pos>a.</pos> <def>Having no silcver; hence, without money; impecunious.</def>  <rj><au>Piers Plowman.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Silverling</ent><br/
<hw>Sil"ver*ling</hw>, <pos>n.</pos> <def>A small silver coin.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>A thousand vines at a thousand <qex>silverings</qex>.</q> <rj><qau>Isa. vii. 23.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Silverly</ent><br/
<hw>Sil"ver*ly</hw>, <pos>adv.</pos> <def>Like silver in appearance or in sound.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Let me wipe off this honorable dew,<br/
That <qex>silverly</qex> doth progress on thy cheeks.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Silvern</ent><br/
<hw>Sil"vern</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[AS. <ets>seolfern</ets>, <ets>sylfren</ets>.]</ety> <def>Made of silver.</def> <mark>[Archaic.]</mark>  <rj><au>Wyclif (Acts xix. 24).</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Speech is <qex>silvern</qex>; silence is golden.</q> <rj><qau>Old Proverb.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Silversides</ent><br/
<hw>Sil"ver*sides`</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Zool.)</fld> <def>Any one of several species of small fishes of the family <fam>Atherinidae</fam>, having a silvery stripe along each side of the body. The common species of the American coast (<spn>Menidia notata</spn>) is very abundant. Called also <altname>silverside</altname>, <altname>sand smelt</altname>, <altname>friar</altname>, <altname>tailor</altname>, and <altname>tinker</altname>.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Brook silversides</b></col> <fld>(Zool.)</fld>, <cd>a small fresh-water North American fish (<spn>Labadesthes sicculus</spn>) related to the marine silversides.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><-- p. 1342 --></p>

<p><ent>Silversmith</ent><br/
<hw>Sil"ver*smith`</hw> <pr>(?)</pr>, <pos>n.</pos> <def>One whose occupation is to manufacture utensils, ornaments, etc., of silver; a worker in silver.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Silverspot</ent><br/
<hw>Sil"ver*spot`</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Zool.)</fld> <def>Any one of numerous species of butterflies of the genus <gen>Argynnis</gen> and allied genera, having silvery spots on the under side of the wings. See <xex>Illust.</xex> under <er>Aphrodite</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Silver State</ent><br/
<hw>Silver State</hw>. <def>Nevada; -- a nickname alluding to its silver mines.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Silverware</ent><br/
<hw>Sil"ver*ware`</hw> <pr>(?)</pr>, <pos>n.</pos> <def>Dishes, vases, ornaments, and utensils of various sorts, made of silver.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Silverweed</ent><br/
<hw>Sil"ver*weed`</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Bot.)</fld> <def>A perennial rosaceous herb (<spn>Potentilla Anserina</spn>) having the leaves silvery white beneath.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Silvery</ent><br/
<hw>Sil"ver*y</hw> <pr>(?)</pr>, <pos>a.</pos> <sn>1.</sn> <def>Resembling, or having the luster of, silver; grayish white and lustrous; of a mild luster; bright.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>All the enameled race, whose <qex>silvery</qex> wing<br/
Waves to the tepid zephyrs of the spring.</q> <rj><qau>Pope.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Besprinkled or covered with silver.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>Having the clear, musical tone of silver; soft and clear in sound; <as>as, <ex>silvery</ex> voices; a <ex>silvery</ex> laugh</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Silvery iron</b></col> <fld>(Metal.)</fld>, <cd>a peculiar light-gray fine-grained cast iron, usually obtained from clay iron ore.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Silviculture</ent><br/
<hw>Sil"vi*cul`ture</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Cf. F. <ets>silviculture</ets>.]</ety> <def>See <er>Sylviculture</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Silvics</ent><br/
<hw>Sil"vics</hw> <pr>(?)</pr>, <pos>n.</pos> <sn>1.</sn> <def>The science treating of the life of trees in the forest.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><sn>2.</sn>  <def>Habit or behavior of a forest tree.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Sima</ent><br/
<hw>Si"ma</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Arch.)</fld> <def>A cyma.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Simagre</ent><br/
<hw>Sim"a*gre</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F. <ets>simagr<eacute/e</ets>.]</ety> <def>A grimace.</def> <mark>[Obs.]</mark>  <rj><au>Dryden.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Simar</ent><br/
<hw>Si*mar"</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F. <ets>simarre</ets>. See <er>Chimere</er>.]</ety> <def>A woman's long dress or robe; also light covering; a scarf.</def> <altsp>[Written also <asp>cimar</asp>, <asp>cymar</asp>, <asp>samare</asp>, <asp>simare</asp>.]</altsp><br/
[<source>1913 Webster</source>]</p>

<p><ent>Simarre</ent><br/
||<hw>Si`marre"</hw> <pr>(?)</pr>. <ety>[F.]</ety> <def>See <er>Simar</er>.</def>  <rj><au>Sir W. Scott.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Simblot</ent><br/
<hw>Sim"blot</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F. <ets>simbleau</ets>.]</ety> <def>The harness of a drawloom.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Simia</ent><br/
||<hw>Sim"i*a</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L., an ape; cf. <ets>simus</ets> flatnosed, snub-nosed, Gr. <?/<?/<?/.]</ety> <fld>(Zool.)</fld> <def>A Linnaean genus of Quadrumana which included the types of numerous modern genera. By modern writers it is usually restricted to the genus which includes the orang-outang.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Simial</ent><br/
<hw>Sim"i*al</hw> <pr>(?)</pr>, <pos>a.</pos> <fld>(Zool.)</fld> <def>Simian; apelike.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Simian</ent><br/
<hw>Sim"i*an</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>simia</ets> an ape.]</ety> <fld>(Zool.)</fld> <def>Of or pertaining to the family <fam>Simiadae</fam>, which, in its widest sense, includes all the Old World apes and monkeys; also, apelike.</def> -- <def2><pos>n.</pos> <def>Any Old World monkey or ape.</def></def2><br/
[<source>1913 Webster</source>]</p>

<p><ent>Similar</ent><br/
<hw>Sim"i*lar</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[F. <ets>similaire</ets>, fr. L. <ets>similis</ets> like, similar. See <er>Same</er>, <pos>a.</pos>, and cf. <er>Simulate</er>.]</ety> <sn>1.</sn> <def>Exactly corresponding; resembling in all respects; precisely like.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Nearly corresponding; resembling in many respects; somewhat like; having a general likeness.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>Homogenous; uniform.</def> <mark>[R.]</mark>  <rj><au>Boyle.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Similar figures</b></col> <fld>(Geom.)</fld>, <cd>figures which differ from each other only in magnitude, being made up of the same number of like parts similarly situated.</cd> -- <col><b>Similar rectilineal figures</b></col>, <cd>such as have their several angles respectively equal, each to each, and their sides about the equal angles proportional.</cd> -- <col><b>Similar solids</b></col>, <cd>such as are contained by the same number of similar planes, similarly situated, and having like inclination to one another.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Similar</ent><br/
<hw>Sim"i*lar</hw>, <pos>n.</pos> <def>That which is similar to, or resembles, something else, as in quality, form, etc.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Similarity</ent><br/
<hw>Sim`i*lar"i*ty</hw> <pr>(?)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>-ties</plw> <pr>(#)</pr>.</plu> <ety>[Cf. F. <ets>similarit<eacute/</ets>.]</ety> <def>The quality or state of being similar; likeness; resemblance; <as>as, a <ex>similarity</ex> of features</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Hardly is there a <qex>similarity</qex> detected between two or three facts, than men hasten to extend it to all.</q> <rj><qau>Sir W. Hamilton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Similarly</ent><br/
<hw>Sim"i*lar*ly</hw> <pr>(?)</pr>, <pos>adv.</pos> <def>In a similar manner.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Similary</ent><br/
<hw>Sim"i*lar*y</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Similar.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>Rhyming cadences of <qex>similarly</qex> words.</q> <rj><qau>South.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Similative</ent><br/
<hw>Sim"i*la*tive</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Implying or indicating likeness or resemblance.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>In <qex>similative</qex> or instrumental relation to a pa. pple. [past participle], as almond-leaved, -scented, etc.</q> <rj><qau>New English Dict.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Simile</ent><br/
<hw>Sim"i*le</hw> <pr>(?)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Similes</plw> <pr>(#)</pr>.</plu> <ety>[L., from <ets>similis</ets>. See <er>Similar</er>.]</ety> <fld>(Rhet.)</fld> <def>A word or phrase by which anything is likened, in one or more of its aspects, to something else; a similitude; a poetical or imaginative comparison.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>A good swift <qex>simile</qex>, but something currish.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Similiter</ent><br/
||<hw>Si*mil"i*ter</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L., in like manner.]</ety> <fld>(Law)</fld> <def>The technical name of the form by which either party, in pleading, accepts the issue tendered by his opponent; -- called sometimes a <altname>joinder in issue</altname>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Similitude</ent><br/
<hw>Si*mil"i*tude</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F. <ets>similitude</ets>, L. <ets>similitudo</ets>, from <ets>similis</ets> similar. See <er>Similar</er>.]</ety> <sn>1.</sn> <def>The quality or state of being similar or like; resemblance; likeness; similarity; <as>as, <ex>similitude</ex> of substance</as>.</def>  <rj><au>Chaucer.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Let us make now man in our image, man<br/
In our <qex>similitude</qex>.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>If fate some future bard shall join<br/
In sad <qex>similitude</qex> of griefs to mine.</q> <rj><qau>Pope.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>The act of likening, or that which likens, one thing to another; fanciful or imaginative comparison; a simile.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Tasso, in his <qex>similitudes</qex>, never departed from the woods; that is, all his comparisons were taken from the country.</q> <rj><qau>Dryden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>That which is like or similar; a representation, semblance, or copy; a facsimile.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Man should wed his <qex>similitude</qex>.</q> <rj><qau>Chaucer.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Similitudinary</ent><br/
<hw>Si*mil`i*tu"di*na*ry</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Involving or expressing similitude.</def> <mark>[Obs.]</mark>  <rj><au>Coke.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Similize</ent><br/
<hw>Sim"i*lize</hw> <pr>(?)</pr>, <pos>v. t.</pos> <def>To liken; to compare; <as>as, to <ex>similize</ex> a person, thing, or act</as>.</def>  <rj><au>Lowell.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Similor</ent><br/
<hw>Sim"i*lor</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F., fr. L. <ets>similus</ets> similar + F. <ets>or</ets> gold, L. <ets>aurum</ets>.  Cf. <er>Semilor</er>.]</ety> <def>An alloy of copper and zinc, resembling brass, but of a golden color.</def>  <rj><au>Ure.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Simious</ent><br/
<hw>Sim"i*ous</hw> <pr>(?)</pr>, <pos>a.</pos><ety>[L. <ets>simia</ets> an ape.]</ety> <fld>(Zool.)</fld> <def>Of or pertaining to the Simi<ae/; monkeylike.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>That strange <qex>simious</qex>, schoolboy passion of giving pain to others.</q> <rj><qau>Sydney Smith.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Simitar</ent><br/
<hw>Sim"i*tar</hw> <pr>(?)</pr>, <pos>n.</pos> <def>See <er>Scimiter</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Simmer</ent><br/
<hw>Sim"mer</hw> <pr>(?)</pr>, <pos>v. i.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Simmered</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Simmering</conjf>.]</vmorph> <ety>[Prov. E. also <ets>simper</ets>; -- an onomatopoetic word.]</ety> <def>To boil gently, or with a gentle hissing; to begin to boil.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>I <qex>simmer</qex> as liquor doth on the fire before it beginneth to boil.</q> <rj><qau>Palsgrave.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Simmer</ent><br/
<hw>Sim"mer</hw>, <pos>v. t.</pos> <def>To cause to boil gently; to cook in liquid heated almost or just to the boiling point.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Simnel</ent><br/
<hw>Sim"nel</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[OF. <ets>simenel</ets> cake or bread of wheat flour, LL. <ets>simenellus</ets> wheat bread, fr. L. <ets>simila</ets> the finest wheat flour.  Cf. <er>Semolina</er>.]</ety> <sn>1.</sn> <def>A kind of cake made of fine flour; a cracknel.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>Not common bread, but vastel bread, or <qex>simnels</qex>.</q> <rj><qau>Fuller.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A kind of rich plum cake, eaten especially on Mid-Lent Sunday.</def> <mark>[Eng.]</mark>  <rj><au>Herrick.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Simoniac</ent><br/
<hw>Si*mo"ni*ac</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[LL. <ets>simoniacus</ets>. See <er>Simony</er>.]</ety> <def>One who practices simony, or who buys or sells preferment in the church.</def>  <rj><au>Ayliffe.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Simoniacal</ent><br/
<hw>Sim`o*ni"a*cal</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Of or pertaining to simony; guilty of simony; consisting of simony.</def> -- <wordforms><wf>Sim"o*ni`a*cal*ly</wf>, <pos>adv.</pos></wordforms><br/
[<source>1913 Webster</source>]</p>

<p><q>The flagitious profligacy of their lives, and the <qex>simoniacal</qex> arts by which they grasped at the popedom.</q> <rj><qau>J. S. Harford.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Simonial</ent><br/
<hw>Si*mo"ni*al</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Simoniacal.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Simonian</ent><br/
<hw>Si*mo"ni*an</hw> <pr>(?)</pr>, <pos>n.</pos><ety>[See <er>Simony</er>.]</ety> <def>One of the followers of Simon Magus; also, an adherent of certain heretical sects in the early Christian church.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Simonious</ent><br/
<hw>Si*mo"ni*ous</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Simoniacal.</def> <mark>[Obs.]</mark>  <rj><au>Milton.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Simonist</ent><br/
<hw>Sim"o*nist</hw> <pr>(?)</pr>, <pos>n.</pos> <def>One who practices simony.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Simon-pure</ent><br/
<hw>Si"mon-pure"</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Genuine; true; real; authentic; -- a term alluding to the comedy character Simon Pure, who is impersonated by another and is obliged to prove himself to be the <ldquo/real Simon Pure.<rdquo/</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Simony</ent><br/
<hw>Sim"o*ny</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F. <ets>simonie</ets>, LL. <ets>simonia</ets>, fr. <ets>Simon</ets> Magus, who wished to purchase the power of conferring the Holy Spirit. <ets>Acts</ets> viii.]</ety> <def>The crime of buying or selling ecclesiastical preferment; the corrupt presentation of any one to an ecclesiastical benefice for money or reward.</def>  <rj><au>Piers Plowman.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Simoon</ent><br/
<ent>Simoom</ent><br/
<mhw>{ <hw>Si*moom"</hw> <pr>(?)</pr>, <hw>Si*moon"</hw> <pr>(?)</pr>, }</mhw> <pos>n.</pos> <ety>[Ar. <ets>sam<?/m</ets>, fr. <ets>samma</ets> to poison.  Cf. <er>Samiel</er>.]</ety> <def>A hot, dry, suffocating, dust-laden wind, that blows occasionally in Arabia, Syria, and neighboring countries, generated by the extreme heat of the parched deserts or sandy plains.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Simous</ent><br/
<hw>Si"mous</hw> <pr>(?)</pr>, <pos>a.</pos><ety>[L. <ets>simus</ets>, Gr. <?/<?/<?/.]</ety> <def>Having a very flat or snub nose, with the end turned up.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Simpai</ent><br/
<hw>Sim"pai</hw> <pr>(?)</pr>, <pos>n.</pos><ety>[Malay <ets>simpei</ets>.]</ety> <fld>(Zool.)</fld> <def>A long-tailed monkey (<spn>Semnopitchecus melalophus</spn>) native of Sumatra. It has a crest of black hair. The forehead and cheeks are fawn color, the upper parts tawny and red, the under parts white. Called also <altname>black-crested monkey</altname>, and <altname>sinpae</altname>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Simper</ent><br/
<hw>Sim"per</hw> <pr>(?)</pr>, <pos>v. i.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Simpered</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Simpering</conjf>.]</vmorph> <ety>[Cf. Norw. <ets>semper</ets> fine, smart, dial. Dan. <ets>semper</ets>, <ets>simper</ets>, affected, coy, prudish, OSw. <ets>semper</ets> one who affectedly refrains from eating, Sw. <ets>sipp</ets> finical, prim, LG. <ets>sipp</ets>.]</ety> <sn>1.</sn> <def>To smile in a silly, affected, or conceited manner.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Behold yond <qex>simpering</qex> dame.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>With a made countenance about her mouth, between <qex>simpering</qex> and smiling.</q> <rj><qau>ir. P. Sidney.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To glimmer; to twinkle.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>Yet can I mark how stars above<br/
<qex>Simper</qex> and shine.</q> <rj><qau>Herbert.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Simper</ent><br/
<hw>Sim"per</hw>, <pos>n.</pos> <def>A constrained, self-conscious smile; an affected, silly smile; a smirk.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The conscious <qex>simper</qex>, and the jealous leer.</q> <rj><qau>Pope.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Simperer</ent><br/
<hw>Sim"per*er</hw> <pr>(?)</pr>, <pos>n.</pos> <def>One who simpers.</def>  <rj><au>Sir W. Scott.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>A <qex>simperer</qex> that a court affords.</q> <rj><qau>T. Nevile.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Simpering</ent><br/
<hw>Sim"per*ing</hw>, <def><pos>a. & n.</pos> from <er>Simper</er>, <pos>v.</pos></def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Simperingly</ent><br/
<hw>Sim"per*ing*ly</hw>, <pos>adv.</pos> <def>In a simpering manner.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Simple</ent><br/
<hw>Sim"ple</hw> <pr>(?)</pr>, <pos>a.</pos> <amorph>[<pos>Compar.</pos> <adjf>Simpler</adjf> <pr>(?)</pr>; <pos>superl.</pos> <adjf>Simplest</adjf>.]</amorph> <ety>[F., fr. L. <ets>simplus</ets>, or <ets>simplex</ets>, gen. <ets>simplicis</ets>. The first part of the Latin words is probably akin to E. <ets>same</ets>, and the sense, one, one and the same; cf. L. <ets>semel</ets> once, <ets>singuli</ets> one to each, single. Cg. <er>Single</er>, <pos>a.</pos>, <er>Same</er>, <pos>a.</pos>, and for the last part of the word cf. <er>Double</er>, <er>Complex</er>.]</ety> <sn>1.</sn> <def>Single; not complex; not infolded or entangled; uncombined; not compounded; not blended with something else; not complicated; <as>as, a <ex>simple</ex> substance; a <ex>simple</ex> idea; a <ex>simple</ex> sound; a <ex>simple</ex> machine; a <ex>simple</ex> problem; <ex>simple</ex> tasks.</as></def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Plain; unadorned; <as>as, <ex>simple</ex> dress</as>.</def> <ldquo/<xex>Simple</xex> truth.<rdquo/ <au>Spenser.</au> <ldquo/His <xex>simple</xex> story.<rdquo/ <au>Burns.</au><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>Mere; not other than; being only.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>A medicine . . . whose <qex>simple</qex> touch<br/
Is powerful to araise King Pepin.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>Not given to artifice, stratagem, or duplicity; undesigning; sincere; true.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Full many fine men go upon my score, as <qex>simple</qex> as I stand here, and I trust them.</q> <rj><qau>Marston.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Must thou trust Tradition's <qex>simple</qex> tongue?</q> <rj><qau>Byron.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>To be simple is to be great.</q> <rj><qau>Emerson.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <def>Artless in manner; unaffected; unconstrained; natural; inartificial;; straightforward.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>In <qex>simple</qex> manners all the secret lies.</q> <rj><qau>Young.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>6.</sn> <def>Direct; clear; intelligible; not abstruse or enigmatical; <as>as, a <ex>simple</ex> statement; <ex>simple</ex> language</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>7.</sn> <def>Weak in intellect; not wise or sagacious; of but moderate understanding or attainments; hence, foolish; silly.</def> <ldquo/You have <xex>simple</xex> wits.<rdquo/  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>The <qex>simple</qex> believeth every word; but the prudent man looketh well to his going.</q> <rj><qau>Prov. xiv. 15.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>8.</sn> <def>Not luxurious; without much variety; plain; <as>as, a <ex>simple</ex> diet; a <ex>simple</ex> way of living</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Thy <qex>simple</qex> fare and all thy plain delights.</q> <rj><qau>Cowper.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>9.</sn> <def>Humble; lowly; undistinguished.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>A <qex>simple</qex> husbandman in garments gray.</q> <rj><qau>Spenser.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Clergy and laity, male and female, gentle and <qex>simple</qex> made the fuel of the same fire.</q> <rj><qau>Fuller.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>10.</sn> <fld>(BOt.)</fld> <def>Without subdivisions; entire; <as>as, a <ex>simple</ex> stem; a <ex>simple</ex> leaf</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>11.</sn> <fld>(Chem.)</fld> <def>Not capable of being decomposed into anything more simple or ultimate by any means at present known; elementary; thus, atoms are regarded as <xex>simple</xex> bodies.  Cf. <er>Ultimate</er>, <pos>a.</pos></def><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ A <xex>simple</xex> body is one that has not as yet been decomposed. There are indications that many of our simple elements are still compound bodies, though their actual decomposition into anything simpler may never be accomplished.</note><-- see <er>fundamental particle</er> --><br/
[<source>1913 Webster</source>]</p>

<p><sn>12.</sn> <fld>(Min.)</fld> <def>Homogenous.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>13.</sn> <fld>(Zool.)</fld> <def>Consisting of a single individual or zooid; <as>as, a <ex>simple</ex> ascidian; -- opposed to <ex>compound</ex></as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Simple contract</b></col> <fld>(Law)</fld>, <cd>any contract, whether verbal or written, which is not of record or under seal.</cd> <au>J. W. Smith.</au> <au>Chitty.</au> -- <col><b>Simple equation</b></col> <fld>(Alg.)</fld>, <cd>an equation containing but one unknown quantity, and that quantity only in the first degree.</cd> -- <col><b>Simple eye</b></col> <fld>(Zool.)</fld>, <cd>an eye having a single lens; -- opposed to <ant>compound eye</ant>.</cd> -- <col><b>Simple interest</b></col>. <cd>See under <er>Interest</er>.</cd> -- <col><b>Simple larceny</b></col>. <fld>(Law)</fld> <cd>See under <er>Larceny</er>.</cd> -- <col><b>Simple obligation</b></col> <fld>(Rom. Law)</fld>, <cd>an obligation which does not depend for its execution upon any event provided for by the parties, or is not to become void on the happening of any such event.</cd> <au>Burrill.</au></cs><br/
[<source>1913 Webster</source>]</p>

<p><syn><b>Syn.</b> -- Single; uncompounded; unmingled; unmixed; mere; uncombined; elementary; plain; artless; sincere; harmless; undesigning; frank; open; unaffected; inartificial; unadorned; credulous; silly; foolish; shallow; unwise.</syn>  -- <usage><er>Simple</er>, <er>Silly</er>. One who is <xex>simple</xex> is sincere, unaffected, and inexperienced in duplicity, -- hence liable to be duped. A <xex>silly</xex> person is one who is ignorant or weak and also self-confident; hence, one who shows in speech and act a lack of good sense. <xex>Simplicity</xex> is incompatible with duplicity, artfulness, or vanity, while <xex>silliness</xex> is consistent with all three. <xex>Simplicity</xex> denotes lack of knowledge or of guile; <xex>silliness</xex> denotes want of judgment or right purpose, a defect of character as well as of education.</usage><br/
[<source>1913 Webster</source>]</p>

<p><q>I am a <qex>simple</qex> woman, much too weak<br/
To oppose your cunning.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>He is the companion of the <qex>silliest</qex> people in their most <qex>silly</qex> pleasure; he is ready for every impertinent entertainment and diversion.</q> <rj><qau>Law.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Simple</ent><br/
<hw>Sim"ple</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F. See <er>Simple</er>, <pos>a.</pos>]</ety> <sn>1.</sn> <def>Something not mixed or compounded.</def> <ldquo/Compounded of many <xex>simples</xex>.<rdquo/  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Med.)</fld> <def>A medicinal plant; -- so called because each vegetable was supposed to possess its particular virtue, and therefore to constitute a simple remedy.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>What virtue is in this remedy lies in the naked <qex>simple</qex> itself as it comes over from the Indies.</q> <rj><qau>Sir W. Temple.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <fld>(Weaving)</fld> <sd>(a)</sd> <def>A drawloom.</def> <sd>(b)</sd> <def>A part of the apparatus for raising the heddles of a drawloom.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <fld>(R. C. Ch.)</fld> <def>A feast which is not a double or a semidouble.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Simple</ent><br/
<hw>Sim"ple</hw>, <pos>v. i.</pos> <def>To gather simples, or medicinal plants.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>As <qex>simpling</qex> on the flowery hills she [Circe] strayed.</q> <rj><qau>Garth.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Simple-hearted</ent><br/
<hw>Sim"ple-heart`ed</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Sincere; inguenuous; guileless.</def>  <rj><au>Sir W. Scott.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Simple-minded</ent><br/
<hw>Sim"ple-mind`ed</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Artless; guileless; simple-hearted; undesigning; unsuspecting; devoid of duplicity.</def> <au>Blackstone.</au> -- <wordforms><wf>Sim"ple-mind`ed*ness</wf>, <pos>n.</pos></wordforms><br/
[<source>1913 Webster</source>]</p>

<p><ent>Simpleness</ent><br/
<hw>Sim"ple*ness</hw>, <pos>n.</pos> <def>The quality or state of being simple; simplicity.</def>  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Simpler</ent><br/
<hw>Sim"pler</hw> <pr>(?)</pr>, <pos>n.</pos> <def>One who collects simples, or medicinal plants; a herbalist; a simplist.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Simpler's joy</b></col>. <fld>(Bot.)</fld> <cd>Vervain.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Simpless</ent><br/
<hw>Sim"pless</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F. <ets>simplesse</ets>.]</ety> <def>Simplicity; silliness.</def> <mark>[Obs.]</mark>  <rj><au>Spenser.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Simplesse</ent><br/
<hw>Sim"plesse</hw> <pr>(?)</pr>, <pos>n.</pos> <def>a trade name for a fat substitute prepared as microparticles of protein from whey.  It has little fat, but has a consistency and taste resembling fat, and is thus used to prepare foods low in fat for dietetic purposes.</def> <mark>[Trade name]</mark><br/
[<source>PJC</source>]</p>

<p><ent>Simpleton</ent><br/
<hw>Sim"ple*ton</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Cf. F. <ets>simplet</ets>, It. <ets>semplicione</ets>.]</ety> <def>A person of weak intellect; a silly person.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Simplician</ent><br/
<hw>Sim*pli"cian</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Cf. OF. <ets>simplicien</ets>.]</ety> <def>One who is simple.</def> <mark>[Obs.]</mark>  <rj><au>Arnway.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Simplicity</ent><br/
<hw>Sim*plic"i*ty</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F. <ets>simplicit<eacute/</ets>, L. <ets>simplicitas</ets>. See <er>Simple</er>.]</ety> <sn>1.</sn> <def>The quality or state of being simple, unmixed, or uncompounded; <as>as, the <ex>simplicity</ex> of metals or of earths</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>The quality or state of being not complex, or of consisting of few parts; <as>as, the <ex>simplicity</ex> of a machine</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>Artlessness of mind; freedom from cunning or duplicity; lack of acuteness and sagacity.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Marquis Dorset, a man, for his harmless <qex>simplicity</qex> neither misliked nor much regarded.</q> <rj><qau>Hayward.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>In wit a man; <qex>simplicity</qex> a child.</q> <rj><qau>Pope.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>Freedom from artificial ornament, pretentious style, or luxury; plainness; <as>as, <ex>simplicity</ex> of dress, of style, or of language; <ex>simplicity</ex> of diet; <ex>simplicity</ex> of life</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <def>Freedom from subtlety or abstruseness; clearness; <as>as, the <ex>simplicity</ex> of a doctrine; the <ex>simplicity</ex> of an explanation or a demonstration</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>6.</sn> <def>Weakness of intellect; silliness; folly.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>How long, ye simple ones, will ye love <qex>simplicity</qex>? and the scorners delight in their scorning?</q> <rj><qau>Prov. i. 22.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Simplification</ent><br/
<hw>Sim`pli*fi*ca"tion</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Cf. F. <ets>simplification</ets>.]</ety> <def>The act of simplifying.</def>  <rj><au>A. Smith.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Simplify</ent><br/
<hw>Sim"pli*fy</hw> <pr>(?)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Simplified</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Simplifying</conjf> <pr>(?)</pr>.]</vmorph> <ety>[Cf. F. <ets>simplifier</ets>, LL. <ets>simplificare</ets>. See <er>Simple</er>, and <er>-fy</er>.]</ety> <def>To make simple; to make less complex; to make clear by giving the explanation for; to show an easier or shorter process for doing or making.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The collection of duties is drawn to a point, and so far <qex>simplified</qex>.</q> <rj><qau>A. Hamilton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>It is important, in scientific pursuits, to be caitious in <qex>simplifying</qex> our deductions.</q> <rj><qau>W. Nicholson.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Simplist</ent><br/
<hw>Sim"plist</hw> <pr>(?)</pr>, <pos>n.</pos> <def>One skilled in simples, or medicinal plants; a simpler.</def>  <rj><au>Sir T. Browne.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Simplistic</ent><br/
<hw>Sim*plis"tic</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Of or pertaining to simples, or a simplist.</def> <mark>[R.]</mark>  <rj><au>Wilkinson.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Simplity</ent><br/
<hw>Sim"pli*ty</hw> <pr>(?)</pr>, <pos>n.</pos> <def>Simplicity.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Simploce</ent><br/
<hw>Sim"plo*ce</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Gram.)</fld> <def>See <er>Symploce</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Simply</ent><br/
<hw>Sim"ply</hw> <pr>(?)</pr>, <pos>adv.</pos> <sn>1.</sn> <def>In a simple manner or state; considered in or by itself; without addition; along; merely; solely; barely.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>[They] make that now good or evil, . . . which otherwise of itself were not <qex>simply</qex> the one or the other.</q> <rj><qau>Hooker.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q><qex>Simply</qex> the thing I am<br/
Shall make me live.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><-- p. 1343 --></p>

<p><sn>2.</sn> <def>Plainly; without art or subtlety.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Subverting worldly strong and worldly wise<br/
By <qex>simply</qex> meek.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>Weakly; foolishly.</def>  <rj><au>Johnson.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Simulachre</ent><br/
<ent>Simulacher</ent><br/
<mhw>{ <hw>Sim"u*la`cher</hw>, <hw>Sim"u*la`chre</hw>  }</mhw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Cf. F. <ets>simulacre</ets>.]</ety> <def>See <er>Simulacrum</er>.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Simulacrum</ent><br/
||<hw>Sim`u*la"crum</hw> <pr>(?)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Simulacra</plw> <pr>(#)</pr>.</plu> <ety>[L. See <er>Simulate</er>.]</ety> <def>A likeness; a semblance; a mock appearance; a sham; -- now usually in a derogatory sense.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Beneath it nothing but a great <qex>simulacrum</qex>.</q> <rj><qau>Thackeray.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Simular</ent><br/
<hw>Sim"u*lar</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Cf. L. <ets>simulator</ets>, F. <ets>simulateur</ets>. See <er>Simulate</er>.]</ety> <def>One who pretends to be what he is not; one who, or that which, simulates or counterfeits something; a pretender.</def> <mark>[Obs.]</mark>  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Christ calleth the Pharisees hypocrites, that is to say, <qex>simulars</qex>, and painted sepulchers.</q> <rj><qau>Tyndale.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Simular</ent><br/
<hw>Sim"u*lar</hw>, <pos>a.</pos> <def>False; specious; counterfeit.</def> <mark>[R. & Obs.]</mark> <ldquo/Thou <xex>simular</xex> man of virtue.<rdquo/  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Simulate</ent><br/
<hw>Sim"u*late</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>simulatus</ets>, p. p. of <ets>simulare</ets> to simulate; akin to <ets>simul</ets> at the same time, together, <ets>similis</ets> like. See <er>Similar</er>, and cf. <er>Dissemble</er>, <er>Semblance</er>.]</ety> <def>Feigned; pretended.</def>  <rj><au>Bale.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Simulate</ent><br/
<hw>Sim"u*late</hw> <pr>(?)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Simulated</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Simulating</conjf>.]</vmorph> <def>To assume the mere appearance of, without the reality; to assume the signs or indications of, falsely; to counterfeit; to feign.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The Puritans, even in the depths of the dungeons to which she had sent them, prayed, and with no <qex>simulated</qex> fervor, that she might be kept from the dagger of the assassin.</q> <rj><qau>Macaulay.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Simulation</ent><br/
<hw>Sim`u*la"tion</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F. <ets>simulation</ets>, L. <ets>simulatio</ets>.]</ety> <def>The act of simulating, or assuming an appearance which is feigned, or not true; -- distinguished from <xex>dissimulation</xex>, which disguises or conceals what is true.</def><br/
[<source>1913 Webster</source>]</p>

<p><syn><b>Syn.</b> -- Counterfeiting; feint; pretense.</syn><br/
[<source>1913 Webster</source>]</p>

<p><ent>Simulator</ent><br/
<hw>Sim"u*la`tor</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L.]</ety> <def>One who simulates, or feigns.</def>  <rj><au> De Quincey.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Simulatory</ent><br/
<hw>Sim"u*la*to*ry</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Simulated, or capable of being simulated.</def>  <rj><au>Bp. Hall.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Simultaneity</ent><br/
<hw>Si`mul*ta*ne"i*ty</hw> <pr>(?)</pr>, <pos>n.</pos> <def>The quality or state of being simultaneous; simultaneousness.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Simultaneous</ent><br/
<hw>Si`mul*ta"ne*ous</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[LL. <ets>simultim</ets> at the same time, fr. L. <ets>simul</ets>. See <er>Simulate</er>.]</ety> <def>Existing, happening, or done, at the same time; <as>as, <ex>simultaneous</ex> events</as>.</def> -- <wordforms><wf>Si`mul*ta"ne*ous*ly</wf>, <pos>adv.</pos> -- <wf>Si`mul*ta"ne*ous*ness</wf>, <pos>n.</pos></wordforms><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Simultaneous equations</b></col> <fld>(Alg.)</fld>, <cd>two or more equations in which the values of the unknown quantities entering them are the same at the same time in both or in all.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Simulty</ent><br/
<hw>Sim"ul*ty</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>simultas</ets> a hostile encounter, drudge, originally, a (hostile) coming together, fr. <ets>simul</ets> together: cf. OF. <ets>simult<eacute/</ets>.]</ety> <def>Private grudge or quarrel; <as>as, domestic <ex>simulties</ex></as>.</def> <mark>[Obs.]</mark>  <rj><au>B. Jonson.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>simvastatin</ent><br/
<hw>sim`va*stat"in</hw> <pr>(?)</pr>, <pos>n.</pos> <def>a chemical substance (<chform>C25H38O5</chform>) having inhibitory activity on the enzyme HMG-CoA reductase, and used in medicine to reduce cholesterol levels in blood.</def><br/
<syn><b>Syn. --</b> synvinolin; MK-733; Zocor.</syn><br/
[<source>PJC</source>]</p>

<p><ent>Sin</ent><br/
<hw>Sin</hw> <pr>(?)</pr>, <pos>adv., prep., & conj.</pos> <def>Old form of <er>Since</er>.</def> <mark>[Obs. or Prov. Eng. & Scot.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q><qex>Sin</qex> that his lord was twenty year of age.</q> <rj><qau>Chaucer.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sin</ent><br/
<hw>Sin</hw>, <pos>n.</pos> <ety>[OE. <ets>sinne</ets>, AS. <ets>synn</ets>, <ets>syn</ets>; akin to D. <ets>zonde</ets>, OS. <ets>sundia</ets>, OHG. <ets>sunta</ets>, G. <ets>s<uum/nde</ets>, Icel., Dan. & Sw. <ets>synd</ets>, L. <ets>sons</ets>, <ets>sontis</ets>, guilty, perhaps originally from the p. pr. of the verb signifying, to be, and meaning, the one who it is.  Cf. <er>Authentic</er>, <er>Sooth</er>.]</ety> <sn>1.</sn> <def>Transgression of the law of God; disobedience of the divine command; any violation of God's will, either in purpose or conduct; moral deficiency in the character; iniquity; <as>as, <ex>sins</ex> of omission and <ex>sins</ex> of commission</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Whosoever committeth <qex>sin</qex> is the servant of <qex>sin</qex>.</q> <rj><qau>John viii. 34.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q><qex>Sin</qex> is the transgression of the law.</q> <rj><qau>1 John iii. 4.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>I think 't no <qex>sin</qex>.<br/
To cozen him that would unjustly win.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Enthralled<br/
By <qex>sin</qex> to foul, exorbitant desires.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>An offense, in general; a violation of propriety; a misdemeanor; <as>as, a <ex>sin</ex> against good manners</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>I grant that poetry's a crying <qex>sin</qex>.</q> <rj><qau>Pope.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>A sin offering; a sacrifice for sin.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>He hath made him to be <qex>sin</qex> for us, who knew no sin.</q> <rj><qau>2 Cor. v. 21.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>An embodiment of sin; a very wicked person.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>Thy ambition,<br/
Thou scarlet <qex>sin</qex>, robbed this bewailing land<br/
Of noble Buckingham.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ <xex>Sin</xex> is used in the formation of some compound words of obvious signification; as, <xex>sin</xex>-born; <xex>sin</xex>-bred, <xex>sin</xex>-oppressed, <xex>sin</xex>-polluted, and the like.</note><br/
[<source>1913 Webster</source>]</p>

<p><cs><mcol><col><b>Actual sin</b></col>, <col><b>Canonical sins</b></col>, <col><b>Original sin</b></col>, <col><b>Venial sin</b></col>.</mcol> <cd>See under <er>Actual</er>, <er>Canonical</er>, etc.</cd> -- <mcol><col><b>Deadly sins</b></col>, <it>or</it> <col><b>Mortal sins</b></col></mcol> <fld>(R. C. Ch.)</fld>, <cd>willful and deliberate transgressions, which take away divine grace; -- in distinction from <xex>vental sins</xex>. The <xex>seven deadly sins</xex> are pride, covetousness, lust, wrath, gluttony, envy, and sloth.</cd> -- <col><b>Sin eater</b></col>, <cd>a man who (according to a former practice in England) for a small gratuity ate a piece of bread laid on the chest of a dead person, whereby he was supposed to have taken the sins of the dead person upon himself.</cd> -- <col><b>Sin offering</b></col>, <cd>a sacrifice for sin; something offered as an expiation for sin.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><syn><b>Syn.</b> -- Iniquity; wickedness; wrong. See <er>Crime</er>.</syn><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sin</ent><br/
<hw>Sin</hw>, <pos>v. i.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Sinned</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Sinning</conjf>.]</vmorph> <ety>[OE. <ets>sinnen</ets>, <ets>singen</ets>, <ets>sinegen</ets>, AS. <ets>syngian</ets>. See <er>Sin</er>, <pos>n.</pos>]</ety> <sn>1.</sn> <def>To depart voluntarily from the path of duty prescribed by God to man; to violate the divine law in any particular, by actual transgression or by the neglect or nonobservance of its injunctions; to violate any known rule of duty; -- often followed by <xex>against</xex>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Against thee, thee only, have I <qex>sinned</qex>.</q> <rj><qau>Ps. li. 4.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>All have <qex>sinned</qex>, and come short of the glory of God.</q> <rj><qau>Rom. iii. 23.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To violate human rights, law, or propriety; to commit an offense; to trespass; to transgress.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>I am a man<br/
More <qex>sinned</qex> against than <qex>sinning</qex>.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Who but wishes to invert the laws<br/
Of order, <qex>sins</qex> against the eternal cause.</q> <rj><qau>Pope.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sinaitic</ent><br/
<ent>Sinaic</ent><br/
<mhw>{ <hw>Si*na"ic</hw> <pr>(?)</pr>, <hw>Si`na*it"ic</hw> <pr>(?)</pr>, }</mhw> <pos>a.</pos> <ety>[From Mount <ets>Sinai</ets>.]</ety> <def>Of or pertaining to Mount Sinai; given or made at Mount Sinai; <as>as, the <ex>Sinaitic</ex> law</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Sinaitic manuscript</b></col>, <cd>a fourth century Greek manuscript of the part Bible, discovered at Mount Sinai (the greater part of it in 1859) by Tisschendorf, a German Biblical critic; -- called also <altname>Codex Sinaiticus</altname>.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sinalbin</ent><br/
<hw>Sin*al"bin</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[From L. <ets>Sin</ets>apis + <ets>alba</ets>.]</ety> <fld>(Chem.)</fld> <def>A glucoside found in the seeds of white mustard (<spn>Brassica alba</spn>, formerly <spn>Sinapis alba</spn>), and extracted as a white crystalline substance.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sinamine</ent><br/
<hw>Sin*am"ine</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[<ets>Sin</ets>apis + mel<ets>amine</ets>.]</ety> <fld>(Chem.)</fld> <def>A bitter white crystalline nitrogenous substance, obtained indirectly from oil of mustard and ammonia; -- called also <altname>allyl melamine</altname>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sinapate</ent><br/
<hw>Sin"a*pate</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Chem.)</fld> <def>A salt of sinapic acid.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sinapic</ent><br/
<hw>Sin"a*pic</hw> <pr>(?)</pr>, <pos>a.</pos> <fld>(Chem.)</fld> <def>Of or pertaining to sinapine; specifically, designating an acid (<chform>C11H12O5</chform>) related to gallic acid, and obtained by the decomposition of sinapine, as a white crystalline substance.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sinapine</ent><br/
<hw>Sin"a*pine</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>sinapi</ets>, <ets>sinapis</ets>, mustard, Gr. <?/<?/<?/: cf. F. <ets>sinapine</ets>.]</ety> <fld>(Chem.)</fld> <def>An alkaloid occuring in the seeds of mustard. It is extracted, in combination with sulphocyanic acid, as a white crystalline substance, having a hot, bitter taste.  When sinapine is isolated it is unstable and undergoes decomposition.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sinapis</ent><br/
||<hw>Si*na"pis</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L.]</ety> <fld>(Bot.)</fld> <def>A disused generic name for mustard; -- now called <altname>Brassica</altname>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sinapisin</ent><br/
<hw>Sin"a*pis`in</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Chem.)</fld> <def>A substance extracted from mustard seed and probably identical with sinalbin.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sinapism</ent><br/
<hw>Sin"a*pism</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>sinapismus</ets>, Gr. <?/<?/<?/<?/, the use of a mustard blister, fr. <?/<?/<?/ to apply a mustard blister, fr. <?/<?/<?/<?/ mustard.]</ety> <fld>(Med.)</fld> <def>A plaster or poultice composed principally of powdered mustard seed, or containing the volatile oil of mustard seed. It is a powerful irritant.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sinapoleic</ent><br/
<hw>Sin`a*po*le"ic</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[<ets>Sina</ets>pis + <ets>oleic</ets>.]</ety> <fld>(Chem.)</fld> <def>Of or pertaining to mustard oil; specifically, designating an acid of the oleic acid series said to occur in mistard oil.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sinapoline</ent><br/
<hw>Si*nap"o*line</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[<ets>Sinapis</ets> + L. <ets>oleum</ets> oil.]</ety> <fld>(Chem.)</fld> <def>A nitrogenous base, <chform>CO.(NH.C3H5)2</chform>, related to urea, extracted from mustard oil, and also produced artifically, as a white crystalline substance; -- called also <altname>diallyl urea</altname>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sincaline</ent><br/
<hw>Sin"ca*line</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[So called because obtained by the action of al<ets>kal</ets>ies on <ets>sin</ets>apine.]</ety> <fld>(Chem.)</fld> <def>Choline.</def> <altsp>[Written also <asp>sinkaline</asp>.]</altsp><br/
[<source>1913 Webster</source>]</p>

<p><ent>Since</ent><br/
<hw>Since</hw> <pr>(s<icr/ns)</pr>, <pos>adv.</pos> <ety>[For <ets>sins</ets>, contr. fr. OE. <ets>sithens</ets>, <ets>sithenes</ets>, formed by an adverbial ending (cf. <er>Besides</er>) from OE. <ets>sithen</ets>, also shortened into <ets>sithe</ets>, <ets>sin</ets>, AS. <ets>si<edh/<edh/an</ets>, <ets>sy<edh/<edh/an</ets>, <ets>seo<edh/<edh/an</ets>, afterward, then, since, after; properly, after that; fr. <ets>s<imac/<edh/</ets> after, later, adv. and prep. (originally a comparative adv., akin to OS. <ets>s<imac/<edh/</ets> afterward, since, OHG. <ets>s<imac/d</ets>, G. <ets>seit</ets> since, Goth. <ets>sei<thorn/us</ets> late, ni <thorn/ana<ets>sei<thorn/s</ets> no longer) + <ets><edh/on</ets> instrumental of the demonstrative and article. See <er>That</er>.]</ety> <sn>1.</sn> <def>From a definite past time until now; <as>as, he went a month ago, and I have not seen him <ex>since</ex></as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>We <qex>since</qex> become the slaves to one man's lust.</q> <rj><qau>B. Jonson.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>In the time past, counting backward from the present; before this or now; ago.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>How many ages <qex>since</qex> has Virgil writ?</q> <rj><qau>Roscommon.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>About two years <qex>since</qex>, it so fell out, that he was brought to a great lady's house.</q> <rj><qau>Sir P. Sidney.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>When or that.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>Do you remember <qex>since</qex> we lay all night in the windmill in St. George's field?</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Since</ent><br/
<hw>Since</hw>, <pos>prep.</pos> <def>From the time of; in or during the time subsequent to; subsequently to; after; -- usually with a past event or time for the object.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The Lord hath blessed thee, <qex>since</qex> my coming.</q> <rj><qau>Gen. xxx. 30.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>I have a model by which he build a nobler poem than any extant <qex>since</qex> the ancients.</q> <rj><qau>Dryden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Since</ent><br/
<hw>Since</hw>, <pos>conj.</pos> <def>Seeing that; because; considering; -- formerly followed by <xex>that</xex>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q><qex>Since</qex> that my penitence comes after all,<br/
Imploring pardon.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q><qex>Since</qex> truth and constancy are vain,<br/
<qex>Since</qex> neither love, nor sense of pain,<br/
Nor force of reason, can persuade,<br/
Then let example be obeyed.</q> <rj><qau>Granville.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><syn><b>Syn.</b> -- Because; for; as; inasmuch as; considering. See <er>Because</er>.</syn><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sincere</ent><br/
<hw>Sin*cere"</hw> <pr>(?)</pr>, <pos>a.</pos> <amorph>[<pos>Compar.</pos> <adjf>Sincerer</adjf> <pr>(?)</pr>; <pos>superl.</pos> <adjf>Sincerest</adjf>.]</amorph> <ety>[L. <ets>sincerus</ets>, of uncertain origin; the first part perhaps akin to <ets>sin-</ets> in <ets>singuli</ets> (see <er>Single</er>), and the second to <ets>cernere</ets> to separate (cf. <er>Discern</er>): cf. F. <ets>sinc</ets><egrave/re.]</ety> <sn>1.</sn> <def>Pure; unmixed; unadulterated.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>There is no <qex>sincere</qex> acid in any animal juice.</q> <rj><qau>Arbuthnot.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>A joy which never was <qex>sincere</qex> till now.</q> <rj><qau>Dryden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Whole; perfect; unhurt; uninjured.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>The inviolable body stood <qex>sincere</qex>.</q> <rj><qau>Dryden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>Being in reality what it appears to be; having a character which corresponds with the appearance; not falsely assumed; genuine; true; real; <as>as, a <ex>sincere</ex> desire for knowledge; a <ex>sincere</ex> contempt for meanness</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>A <qex>sincere</qex> intention of pleasing God in all our actions.</q> <rj><qau>Law.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>Honest; free from hypocrisy or dissimulation; <as>as, a <ex>sincere</ex> friend; a <ex>sincere</ex> person</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The more <qex>sincere</qex> you are, the better it will fare with you at the great day of account.</q> <rj><qau>Waterland.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><syn><b>Syn.</b> -- Honest; unfeigned; unvarnished; real; true; unaffected; inartificial; frank; upright. See <er>Hearty</er>.</syn><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sincerely</ent><br/
<hw>Sin*cere"ly</hw>, <pos>adv.</pos> <def>In a sincere manner.</def> Specifically: <sd>(a)</sd> <def>Purely; without alloy.</def> <au>Milton.</au> <sd>(b)</sd> <def>Honestly; unfeignedly; without dissimulation; <as>as, to speak one's mind <ex>sincerely</ex>; to love virtue <ex>sincerely</ex></as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sincereness</ent><br/
<hw>Sin*cere"ness</hw>, <pos>n.</pos> <def>Same as <er>Sincerity</er>.</def>  <rj><au>Beau. & Fl.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sincerity</ent><br/
<hw>Sin*cer"i*ty</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>sinceritas</ets>: cf. F. <ets>sinc<eacute/rit<eacute/</ets>.]</ety> <def>The quality or state of being sincere; honesty of mind or intention; freedom from simulation, hypocrisy, disguise, or false pretense; sincereness.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>I protest, in the <qex>sincerity</qex> of love.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q><qex>Sincerity</qex> is a duty no less plain than important.</q> <rj><qau>Knox.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sinch</ent><br/
<hw>Sinch</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[See <er>Cinch</er>.]</ety> <def>A saddle girth made of leather, canvas, woven horsehair, or woven grass.</def> <mark>[Western U.S.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sinch</ent><br/
<hw>Sinch</hw>, <pos>v. t.</pos> <def>To gird with a sinch; to tighten the sinch or girth of (a saddle); <as>as, to <ex>sinch</ex> up a sadle</as>.</def> <mark>[Western U.S.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sincipital</ent><br/
<hw>Sin*cip"i*tal</hw> <pr>(?)</pr>, <pos>a.</pos> <fld>(Anat.)</fld> <def>Of or pertaining to the sinciput; being in the region of the sinciput.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sinciput</ent><br/
<hw>Sin"ci*put</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L., half a head; <ets>semi</ets>half + <ets>caput</ets> the head.]</ety> <sn>1.</sn> <fld>(Anat.)</fld> <def>The fore part of the head.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Zool.)</fld> <def>The part of the head of a bird between the base of the bill and the vertex.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sindi</ent><br/
||<hw>Sind"i</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Ar. <ets>Sind<imac/</ets>, fr. <ets>Sind</ets> Indian, Skr. <ets>sindhu</ets> river, sea, the river Indus, the country along the Indus. Cf. <er>Indian</er>, <er>Hindoo</er>.]</ety> <fld>(Ethnol.)</fld> <def>A native of Sind, India, esp. one of the native Hindoo stock.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Sindon</ent><br/
<hw>Sin"don</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L., a kind of fine Indian cotton stuff, Gr. <?/<?/<?/.]</ety> <sn>1.</sn> <def>A wrapper.</def> <mark>[Obs.]</mark> <ldquo/Wrapped in <xex>sindons</xex> of linen.<rdquo/  <rj><au>Bacon.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Surg.)</fld> <def>A small rag or pledget introduced into the hole in the cranium made by a trephine.</def>  <rj><au>Dunglison.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sine</ent><br/
<hw>Sine</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[LL. <ets>sinus</ets> a sine, L. <ets>sinus</ets> bosom, used in translating the Ar. <ets>jaib</ets>, properly, bosom, but probably read by mistake (the consonants being the same) for an original <ets>j<imac/ba</ets> sine, from Skr. <ets>j<imac/va</ets> bowstring, chord of an arc, sine.]</ety> <fld>(Trig.)</fld> <sd>(a)</sd> <def>The length of a perpendicular drawn from one extremity of an arc of a circle to the diameter drawn through the other extremity.</def> <sd>(b)</sd> <def>The perpendicular itself. See <cref>Sine of angle</cref>, below.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Artificial sines</b></col>, <cd>logarithms of the natural sines, or logarithmic sines.</cd> -- <col><b>Curve of sines</b></col>. <cd>See <er>Sinusoid</er>.</cd> -- <col><b>Natural sines</b></col>, <cd>the decimals expressing the values of the sines, the radius being unity.</cd> -- <col><b>Sine of an angle</b></col>, <cd>in a circle whose radius is unity, the sine of the arc that measures the angle; in a right-angled triangle, the side opposite the given angle divided by the hypotenuse. See <cref>Trigonometrical function</cref>, under <er>Function</er>.</cd> -- <col><b>Versed sine</b></col>, <cd>that part of the diameter between the sine and the arc.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sine</ent><br/
||<hw>Si"ne</hw> <pr>(?)</pr>, <pos>prep.</pos> <ety>[L.]</ety> <def>Without.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sinecural</ent><br/
<hw>Si"ne*cu`ral</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Of or pertaining to a sinecure; being in the nature of a sinecure.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sinecure</ent><br/
<hw>Si`ne*cure</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>sine</ets> without + <ets>cura</ets> care, LL., a cure. See <er>Cure</er>.]</ety> <sn>1.</sn> <def>An ecclesiastical benefice without the care of souls.</def>  <rj><au>Ayliffe.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Any office or position which requires or involves little or no responsibility, labor, or active service.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>A lucrative <qex>sinecure</qex> in the Excise.</q> <rj><qau>Macaulay.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sinecure</ent><br/
<hw>Si"ne*cure</hw>, <pos>v. t.</pos> <def>To put or place in a sinecure.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sinecurism</ent><br/
<hw>Si"ne*cu*rism</hw> <pr>(?)</pr>, <pos>n.</pos> <def>The state of having a sinecure.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sinecurist</ent><br/
<hw>Si"ne*cu*rist</hw> <pr>(?)</pr>, <pos>n.</pos> <def>One who has a sinecure.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sinew</ent><br/
<hw>Sin"ew</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[OE. <ets>sinewe</ets>, <ets>senewe</ets>, AS. <ets>sinu</ets>, <ets>seonu</ets>; akin to D. <ets>zenuw</ets>, OHG. <ets>senawa</ets>, G. <ets>sehne</ets>, Icel. <ets>sin</ets>, Sw. <ets>sena</ets>, Dan.  <ets>sene</ets>; cf. Skr. <ets>sn<amac/va</ets>. <root/290.]</ety><br/
[<source>1913 Webster</source>]</p>

<p><sn>1.</sn> <fld>(Anat.)</fld> <def>A tendon or tendonous tissue. See <er>Tendon</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Muscle; nerve.</def> <mark>[R.]</mark>  <rj><au>Sir J. Davies.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>Fig.: That which supplies strength or power.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The portion and <qex>sinew</qex> of her fortune, her marriage dowry.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>The bodies of men, munition, and money, may justly be called the <qex>sinews</qex> of war.</q> <rj><qau>Sir W. Raleigh.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ Money alone is often called <xex>the sinews of war</xex>.</note><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sinew</ent><br/
<hw>Sin"ew</hw>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Sinewed</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Sinewing</conjf>.]</vmorph> <def>To knit together, or make strong with, or as with, sinews.</def>  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Wretches, now stuck up for long tortures . . . might, if properly treated, serve to <qex>sinew</qex> the state in time of danger.</q> <rj><qau>Goldsmith.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sinewed</ent><br/
<hw>Sin"ewed</hw> <pr>(?)</pr>, <pos>a.</pos> <sn>1.</sn> <def>Furnished with sinews; <as>as, a strong-<ex>sinewed</ex> youth</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Fig.: Equipped; strengthened.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>When he sees<br/
Ourselves well <qex>sinewed</qex> to our defense.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sinewiness</ent><br/
<hw>Sin"ew*i*ness</hw> <pr>(?)</pr>, <pos>n.</pos> <def>Quality of being sinewy.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sinewish</ent><br/
<hw>Sin"ew*ish</hw>, <pos>a.</pos> <def>Sinewy.</def> <mark>[Obs.]</mark>  <rj><au>Holinshed.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sinewless</ent><br/
<hw>Sin"ew*less</hw>, <pos>a.</pos> <def>Having no sinews; hence, having no strength or vigor.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sinewous</ent><br/
<hw>Sin"ew*ous</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Sinewy.</def> <mark>[Obs.]</mark>  <rj><au>Holinshed.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sinew-shrunk</ent><br/
<hw>Sin"ew-shrunk`</hw> <pr>(?)</pr>, <pos>a.</pos> <fld>(Far.)</fld> <def>Having the sinews under the belly shrunk by excessive fatigue.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sinewy</ent><br/
<hw>Sin"ew*y</hw> <pr>(?)</pr>, <pos>a.</pos> <sn>1.</sn> <def>Pertaining to, consisting of, or resembling, a sinew or sinews.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The <qex>sinewy</qex> thread my brain lets fall.</q> <rj><qau>Donne.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Well braced with, or as if with, sinews; nervous; vigorous; strong; firm; tough; <as>as, the <ex>sinewy</ex> Ajax</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>A man whose words . . . were so close and <qex>sinewy</qex>.</q> <rj><qau>Hare.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sinful</ent><br/
<hw>Sin"ful</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[AAS. <ets>synfull</ets>.]</ety> <def>Tainted with, or full of, sin; wicked; iniquitous; criminal; unholy; <as>as, <ex>sinful</ex> men; <ex>sinful</ex> thoughts</as>.</def>  <rj><au>Piers Plowman.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Ah <qex>sinful</qex> nation, a people laden with iniquity.</q> <rj><qau>Isa. i. 4.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p>-- <wordforms><wf>Sin"ful*ly</wf>, <pos>adv.</pos> -- <wf>Sin"ful*ness</wf>, <pos>n.</pos></wordforms><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sing</ent><br/
<hw>Sing</hw> <pr>(s<icr/ng)</pr>, <pos>v. i.</pos> <vmorph>[<pos>imp.</pos> <conjf>Sung</conjf> <pr>(?)</pr> or <conjf>Sang</conjf> <pr>(<?/)</pr>; <pos>p. p.</pos> <conjf>Sung</conjf>; <pos>p. pr. & vb. n.</pos> <conjf>Singing</conjf>.]</vmorph> <ety>[AS. <ets>singan</ets>; akin to D. <ets>zingen</ets>, OS. & OHG. <ets>singan</ets>, G. <ets>singen</ets>, Icel. <ets>syngja</ets>, Sw. <ets>sjunga</ets>, Dan. <ets>synge</ets>, Goth. <ets>siggwan</ets>, and perhaps to E. <ets>say</ets>, v.t., or cf. Gr. <?/<?/<?/ voice.  Cf. <er>Singe</er>, <er>Song</er>.]</ety> <sn>1.</sn> <def>To utter sounds with musical inflections or melodious modulations of voice, as fancy may dictate, or according to the notes of a song or tune, or of a given part (as alto, tenor, etc.) in a chorus or concerted piece.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The noise of them that <qex>sing</qex> do I hear.</q> <rj><qau>Ex. xxxii. 18.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To utter sweet melodious sounds, as birds do.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>On every bough the briddes heard I <qex>sing</qex>.</q> <rj><qau>Chaucer.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q><qex>Singing</qex> birds, in silver cages hung.</q> <rj><qau>Dryden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>To make a small, shrill sound; <as>as, the air <ex>sings</ex> in passing through a crevice</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>O'er his head the flying spear<br/
<qex>Sang</qex> innocent, and spent its force in air.</q> <rj><qau>Pope.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>To tell or relate something in numbers or verse; to celebrate something in poetry.</def>  <rj><au>Milton.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Bid her . . . <qex>sing</qex><br/
Of human hope by cross event destroyed.</q> <rj><qau>Prior.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><-- p. 1344 --></p>

<p><sn>5.</sn> <def>To cry out; to complain.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>They should <qex>sing</qex> if thet they were bent.</q> <rj><qau>Chaucer.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sing</ent><br/
<hw>Sing</hw> <pr>(s<icr/ng)</pr>, <pos>v. t.</pos> <sn>1.</sn> <def>To utter with musical inflections or modulations of voice.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>And they <qex>sing</qex> the song of Moses, the servant of God, and the song of the Lamb.</q> <rj><qau>Rev. xv. 3.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>And in the darkness <qex>sing</qex> your carol of high praise.</q> <rj><qau>Keble.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To celebrate in song; to give praises to in verse; to relate or rehearse in numbers, verse, or poetry.</def>  <rj><au>Milton.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Arms and the man I <qex>sing</qex>.</q> <rj><qau>Dryden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>The last, the happiest British king,<br/
Whom thou shalt paint or I shall <qex>sing</qex>.</q> <rj><qau>Addison.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>To influence by singing; to lull by singing; <as>as, to <ex>sing</ex> a child to sleep</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>To accompany, or attend on, with singing.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>I heard them <qex>singing</qex> home the bride.</q> <rj><qau>Longfellow.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Singe</ent><br/
<hw>Singe</hw> <pr>(s<icr/nj)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Singed</conjf> <pr>(s<icr/njd)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Singeing</conjf> <pr>(s<icr/nj"<icr/ng)</pr>.]</vmorph> <ety>[OE. <ets>sengen</ets>, AS. <ets>sengan</ets> in be<ets>sengan</ets> (akin to D. <ets>zengen</ets>, G. <ets>sengen</ets>), originally, to cause to sing, fr. AS. <ets>singan</ets> to sing, in allusion to the <ets>singing</ets> or hissing sound often produced when a substance is singed, or slightly burned. See <er>Sing</er>.]</ety> <sn>1.</sn> <def>To burn slightly or superficially; to burn the surface of; to burn the ends or outside of; <as>as, to <ex>singe</ex> the hair or the skin</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>You sulphurous and thought-executing fires, . . . <br/
<qex>Singe</qex> my white head!</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>I <qex>singed</qex> the toes of an ape through a burning glass.</q> <rj><qau>L'Estrange.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <sd>(a)</sd> <def>To remove the nap of (cloth), by passing it rapidly over a red-hot bar, or over a flame, preliminary to dyeing it.</def> <sd>(b)</sd> <def>To remove the hair or down from (a plucked chicken or the like) by passing it over a flame.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Singe</ent><br/
<hw>Singe</hw>, <pos>n.</pos> <def>A burning of the surface; a slight burn.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Singer</ent><br/
<hw>Sin"ger</hw> <pr>(s<icr/n"j<etil/r)</pr>, <pos>n.</pos> <ety>[From <er>Singe</er>.]</ety> <def>One who, or that which, singes.</def> <specif>Specifically:</specif> <sd>(a)</sd> <def>One employed to singe cloth.</def> <sd>(b)</sd> <def>A machine for singeing cloth.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Singer</ent><br/
<hw>Sing"er</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[From <er>Sing</er>.]</ety> <def>One who sings; especially, one whose profession is to sing.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Singeress</ent><br/
<hw>Sing"er*ess</hw>, <pos>n.</pos> <def>A songstress.</def> <mark>[Obs.]</mark>  <rj><au>Wyclif.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Singhalese</ent><br/
<hw>Sin`gha*lese"</hw> <pr>(?)</pr>, <pos>n. & a.</pos> <ety>[Skr. <ets>Si<mtil/hala</ets> Ceylon.]</ety> <fld>(Ethnol.)</fld> <def>Same as <er>Cingalese</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Singing</ent><br/
<hw>Sing"ing</hw> <pr>(?)</pr>, <def><pos>a. & n.</pos> from <er>Sing</er>, <pos>v.</pos></def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Singing bird</b></col>. <fld>(Zool.)</fld> <sd>(a)</sd> <cd>Popularly, any bird that sings; a song bird</cd>. <sd>(b)</sd> <cd>Specifically, any one of the Oscines.</cd> -- <col><b>Singing book</b></col>, <cd>a book containing music for singing; a book of tunes.</cd> -- <mcol><col><b>Singing falcon</b></col> <it>or</it> <col><b>Singing hawk</b></col></mcol>. <fld>(Zool.)</fld> <cd>See <cref>Chanting falcon</cref>, under <er>Chanting</er>.</cd> -- <col><b>Singing fish</b></col> <fld>(Zool.)</fld>, <cd>a California toadfish (<spn>Porichthys porosissimus</spn>), called also <altname>midshipman</altname>; -- so called because it produces a buzzing sound with its air bladder.</cd> -- <col><b>Singing flame</b></col> <fld>(Acoustics)</fld>, <cd>a flame, as of hydrogen or coal gas, burning within a tube and so adjusted as to set the air within the tube in vibration, causing sound. The apparatus is called also <altname>chemical harmonicon</altname>.</cd> -- <col><b>Singing master</b></col>, <cd>a man who teaches vocal music.</cd> -- <col><b>Singing school</b></col>, <cd>a school in which persons are instructed in singing.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Singingly</ent><br/
<hw>Sing"ing*ly</hw>, <pos>adv.</pos> <def>With sounds like singing; with a kind of tune; in a singing tone.</def>  <rj><au>G. North (1575).</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Single</ent><br/
<hw>Sin"gle</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>singulus</ets>, a dim. from the root in <ets>simplex</ets> simple; cf. OE. & OF. <ets>sengle</ets>, fr. L. <ets>singulus</ets>. <ets>See</ets> <er>Simple</er>, and cf. <er>Singular</er>.]</ety> <sn>1.</sn> <def>One only, as distinguished from more than one; consisting of one alone; individual; separate; <as>as, a <ex>single</ex> star</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>No <qex>single</qex> man is born with a right of controlling the opinions of all the rest.</q> <rj><qau>Pope.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Alone; having no companion.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Who <qex>single</qex> hast maintained,<br/
Against revolted multitudes, the cause<br/
Of truth.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>Hence, unmarried; <as>as, a <ex>single</ex> man or woman</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Grows, lives, and dies in <qex>single</qex> blessedness.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q><qex>Single</qex> chose to live, and shunned to wed.</q> <rj><qau>Dryden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>Not doubled, twisted together, or combined with others; <as>as, a <ex>single</ex> thread; a <ex>single</ex> strand of a rope</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <def>Performed by one person, or one on each side; <as>as, a <ex>single</ex> combat</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>These shifts refuted, answer thy appellant, . . . <br/
Who now defles thee thrice ti <qex>single</qex> fight.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>6.</sn> <def>Uncompounded; pure; unmixed.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Simple ideas are opposed to complex, and <qex>single</qex> to compound.</q> <rj><qau>I. Watts.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>7.</sn> <def>Not deceitful or artful; honest; sincere.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>I speak it with a <qex>single</qex> heart.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>8.</sn> <def>Simple; not wise; weak; silly.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>He utters such <qex>single</qex> matter in so infantly a voice.</q> <rj><qau>Beau. & Fl.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><mcol><col><b>Single ale</b></col>, <col><b>Single beer</b></col>, <it>or</it> <col><b>Single drink</b></col></mcol>, <cd>small ale, etc., as contrasted with <contr>double ale</contr>, etc., which is stronger.</cd> <mark>[Obs.]</mark> <au>Nares.</au> -- <col><b>Single bill</b></col> <fld>(Law)</fld>, <cd>a written engagement, generally under seal, for the payment of money, without a penalty.</cd> <au>Burril.</au> -- <col><b>Single court</b></col> <fld>(Lawn Tennis)</fld>, <cd>a court laid out for only two players.</cd> -- <col><b>Single-cut file</b></col>. <cd>See the Note under 4th <er>File</er>.</cd> -- <col><b>Single entry</b></col>. <cd>See under <er>Bookkeeping</er>.</cd> -- <col><b>Single file</b></col>. <cd>See under 1st <er>File</er>.</cd> -- <col><b>Single flower</b></col> <fld>(Bot.)</fld>, <cd>a flower with but one set of petals, as a wild rose.</cd> -- <col><b>Single knot</b></col>. <cd>See <xex>Illust.</xex> under <er>Knot</er>.</cd> -- <col><b>Single whip</b></col> <fld>(Naut.)</fld>, <cd>a single rope running through a fixed block.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Single</ent><br/
<hw>Sin"gle</hw>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Singled</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Singling</conjf> <pr>(?)</pr>.]</vmorph> <sn>1.</sn> <def>To select, as an individual person or thing, from among a number; to choose out from others; to separate.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Dogs who hereby can <qex>single</qex> out their master in the dark.</q> <rj><qau>Bacon.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>His blood! she faintly screamed her mind<br/
Still <qex>singling</qex> one from all mankind.</q> <rj><qau>More.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To sequester; to withdraw; to retire.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>An agent <qex>singling</qex> itself from consorts.</q> <rj><qau>Hooker.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>To take alone, or one by one.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Men . . . commendable when they are <qex>singled</qex>.</q> <rj><qau>Hooker.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Single</ent><br/
<hw>Sin"gle</hw>, <pos>v. i.</pos> <def>To take the irrregular gait called single-foot; -- said of a horse. See <er>Single-foot</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Many very fleet horses, when overdriven, adopt a disagreeable gait, which seems to be a cross between a pace and a trot, in which the two legs of one side are raised almost but not quite, simultaneously. Such horses are said to <qex>single</qex>, or to be single-footed.</q> <rj><qau>W. S. Clark.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Single</ent><br/
<hw>Sin"gle</hw>, <pos>n.</pos> <sn>1.</sn> <def>A unit; one; <as>as, to score a <ex>single</ex></as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <pluf>pl.</pluf> <def>The reeled filaments of silk, twisted without doubling to give them firmness.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>A handful of gleaned grain.</def> <mark>[Prov. Eng. & Scot.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <fld>(Law Tennis)</fld> <def>A game with but one player on each side; -- usually in the plural.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <fld>(Baseball)</fld> <def>A hit by a batter which enables him to reach first base only.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Single-acting</ent><br/
<hw>Sin"gle-act`ing</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Having simplicity of action; especially <fld>(Mach.)</fld>, acting or exerting force during strokes in one direction only; -- said of a reciprocating engine, pump, etc.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Single-breasted</ent><br/
<hw>Sin"gle-breast`ed</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Lapping over the breast only far enough to permit of buttoning, and having buttons on one edge only; <as>as, a <ex>single-breasted</ex> coast</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Single-foot</ent><br/
<hw>Sin"gle-foot`</hw> <pr>(?)</pr>, <pos>n.</pos> <def>An irregular gait of a horse; -- called also <altname>single-footed pace</altname>. See <er>Single</er>, <pos>v. i.</pos></def><br/
[<source>1913 Webster</source>]</p>

<p><q><qex>Single-foot</qex> is an irregular pace, rather rare, distinguished by the posterior extremities moving in the order of a fast walk, and the anterior extremities in that of a slow trot.</q> <rj><qau>Stillman (The Horse in Motion.)</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Single-foot</ent><br/
<hw>Sin"gle-foot`</hw>, <pos>v. i.</pos> <def>To proceed by means of the single-foot, as a horse or other quadruped.</def> -- <wordforms><wf>Sin"gle-foot`er</wf>, <pos>n.</pos></wordforms><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>single-handed</ent><br/
<hw>sin"gle-hand"ed</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Having but one hand, or one workman; also, alone; unassisted.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>single-handedly</ent><br/
<hw>sin"gle-hand"ed*ly</hw> <pr>(?)</pr>, <pos>adv.</pos> <def>By oneself; alone; unassisted.</def><br/
[<source>PJC</source>]</p>

<p><ent>Single-hearted</ent><br/
<hw>Sin"gle-heart"ed</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Having an honest heart; free from duplicity.</def> -- <wordforms><wf>Sin"gle-heart"ed*ly</wf>, <pos>adv.</pos></wordforms><br/
[<source>1913 Webster</source>]</p>

<p><ent>single-minded</ent><br/
<hw>sin"gle-mind"ed</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Having a single purpose; concentrating on a single goal; hence, artless; guileless; single-hearted.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>single-mindedness</ent><br/
<hw>sin"gle-mind"ed*ness</hw> <pr>(?)</pr>, <pos>n.</pos> <def>The quality of being single-minded.</def><br/
[<source>PJC</source>]</p>

<p><ent>Singleness</ent><br/
<hw>Sin"gle*ness</hw>, <pos>n.</pos> <sn>1.</sn> <def>The quality or state of being single, or separate from all others; the opposite of doubleness, complication, or multiplicity.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Freedom from duplicity, or secondary and selfish ends; purity of mind or purpose; simplicity; sincerity; <as>as, <ex>singleness</ex> of purpose; <ex>singleness</ex> of heart</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>singles</ent><br/
<hw>sin"gles</hw> <pr>(?)</pr>, <pos>n. pl.</pos> <def>See <er>Single</er>, <pos>n.</pos>, 2.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>singles</ent><br/
<hw>sin"gles</hw> <pr>(?)</pr>, <pos>a.</pos> <def>For unmarried persons, or catering especially to unmarried persons; <as>as, a <ex>singles bar</ex>; a <ex>singles</ex> party</as>.</def><br/
[<source>PJC</source>]</p>

<p><ent>Singlestick</ent><br/
<hw>Sin"gle*stick`</hw> <pr>(?)</pr>, <pos>n.</pos> <sd>(a)</sd> <def>In England and Scotland, a cudgel used in fencing or fighting; a backsword.</def> <sd>(b)</sd> <def>The game played with singlesticks, in which he who first brings blood from his adversary's head is pronounced victor; backsword; cudgeling.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Single-surfaced</ent><br/
<hw>Sin"gle-sur"faced</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Having one surface; -- said specif. of aeroplanes or aerocurves that are covered with fabric, etc., on only one side.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Singlet</ent><br/
<hw>Sin"glet</hw> <pr>(?)</pr>, <pos>n.</pos> <def>An unlined or undyed waistcoat; a single garment; -- opposed to <contr>doublet</contr>.</def> <mark>[Prov. Eng.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Single tax</ent><br/
<hw>Sin"gle tax`</hw>. <fld>(Pol. Econ.)</fld> <def>A tax levied upon land alone, irrespective of improvements, -- advocated by certain economists as the sole source of public revenue.</def></p>

<p><q>Whatever may be thought of Henry George's <qex>single-tax</qex> theory as a whole, there can be little question that a relatively higher assessment of ground rent, with corresponding relief for those who have made improvements, is a much-needed reform.</q>  <rj><qau>A. T. Hadley.</qau></rj><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Singleton</ent><br/
<hw>Sin"gle*ton</hw> <pr>(?)</pr>, <pos>n.</pos> <def>In certain games at cards, as whist, a single card of any suit held at the deal by a player; <as>as, to lead a <ex>singleton</ex></as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Singletree</ent><br/
<hw>Sin"gle*tree`</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Cf. <er>Swingletree</er>.]</ety> <def>The pivoted or swinging bar to which the traces of a harnessed horse are fixed; a whiffletree.</def><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ When two horses draw abreast, a <xex>singletree</xex> is fixed at each end of another crosspiece, called the <xex>doubletree</xex>.</note><br/
[<source>1913 Webster</source>]</p>

<p><ent>Singly</ent><br/
<hw>Sin"gly</hw> <pr>(?)</pr>, <pos>adv.</pos> <sn>1.</sn> <def>Individually; particularly; severally; <as>as, to make men <ex>singly</ex> and personally good</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Only; by one's self; alone.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Look thee, 't is so! Thou <qex>singly</qex> honest man.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>Without partners, companions, or associates; single-handed; <as>as, to attack another <ex>singly</ex></as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>At omber <qex>singly</qex> to decide their doom.</q> <rj><qau>Pope.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>Honestly; sincerely; simply.</def> <mark>[R.]</mark>  <rj><au>Johnson.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <def>Singularly; peculiarly.</def> <mark>[Obs.]</mark>  <rj><au>Milton.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sing-sing</ent><br/
<hw>Sing"-sing`</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Zool.)</fld> <def>The kob.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Singsong</ent><br/
<hw>Sing"song`</hw> <pr>(?)</pr>, <pos>n.</pos> <sn>1.</sn> <def>Bad singing or poetry.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A drawling or monotonous tone, as of a badly executed song.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Singsong</ent><br/
<hw>Sing"song`</hw>, <pos>a.</pos> <def>Drawling; monotonous; having a monotonous cadence.</def><br/
[<source>1913 Webster</source> <source>+PJC</source>]</p>

<p><ent>Singsong</ent><br/
<hw>Sing"song`</hw>, <pos>v. i.</pos> <def>To write poor poetry.</def> <mark>[R.]</mark>  <rj><au>Tennyson.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Singspiel</ent><br/
||<hw>Sing"spiel`</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[G.; <ets>singen</ets> to sing + <ets>spiel</ets> to play.]</ety> <fld>(Music)</fld> <def>A dramatic work, partly in dialogue and partly in song, of a kind popular in Germany in the latter part of the 18th century. It was often comic, had modern characters, and patterned its music on folk song with strictly subordinated accompaniment.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Singster</ent><br/
<hw>Sing"ster</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A songstress.</def> <mark>[Obs.]</mark>  <rj><au>Wyclif.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Singular</ent><br/
<hw>Sin"gu*lar</hw> <pr>(s<icr/<nsm/"g<usl/*l<etil/r)</pr>, <pos>a.</pos> <ety>[OE. <ets>singuler</ets>, F. <ets>singulier</ets>, fr. L. <ets>singularius</ets>, <ets>singularis</ets>, fr. <ets>singulus</ets> single. See <er>Single</er>, <pos>a.</pos>]</ety> <sn>1.</sn> <def>Separate or apart from others; single; distinct.</def> <mark>[Obs.]</mark>  <rj><au>Bacon.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>And God forbid that all a company<br/
Should rue a <qex>singular</qex> man's folly.</q> <rj><qau>Chaucer.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Engaged in by only one on a side; single.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>To try the matter thus together in a <qex>singular</qex> combat.</q> <rj><qau>Holinshed.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <fld>(Logic)</fld> <def>Existing by itself; single; individual.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The idea which represents one . . . determinate thing, is called a <qex>singular</qex> idea, whether simple, complex, or compound.</q> <rj><qau>I. Watts.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <fld>(Law)</fld> <def>Each; individual; <as>as, to convey several parcels of land, all and <ex>singular</ex></as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <fld>(Gram.)</fld> <def>Denoting one person or thing; <as>as, the <ex>singular</ex> number</as>; -- opposed to <contr>dual</contr> and <ant>plural</ant>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>6.</sn> <def>Standing by itself; out of the ordinary course; unusual; uncommon; strange; <as>as, a <ex>singular</ex> phenomenon</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>So <qex>singular</qex> a sadness<br/
Must have a cause as strange as the effect.</q> <rj><qau>Denham.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>7.</sn> <def>Distinguished as existing in a very high degree; rarely equaled; eminent; extraordinary; exceptional; <as>as, a man of <ex>singular</ex> gravity or attainments</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>8.</sn> <def>Departing from general usage or expectations; odd; whimsical; -- often implying disapproval or censure.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>His zeal<br/
None seconded, as out of season judged,<br/
Or <qex>singular</qex> and rash.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>To be <qex>singular</qex> in anything that is wise and worthy, is not a disparagement, but a praise.</q> <rj><qau>Tillotson.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>9.</sn> <def>Being alone; belonging to, or being, that of which there is but one; unique.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>These busts of the emperors and empresses are all very scarce, and some of them almost <qex>singular</qex> in their kind.</q> <rj><qau>Addison.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Singular point in a curve</b></col> <fld>(Math.)</fld>, <cd>a point at which the curve possesses some peculiar properties not possessed by other points of the curve, as a cusp point, or a multiple point.</cd> -- <col><b>Singular proposition</b></col> <fld>(Logic)</fld>, <cd>a proposition having as its subject a singular term, or a common term limited to an individual by means of a singular sign.</cd> <au>Whately.</au> -- <col><b>Singular succession</b></col> <fld>(Civil Law)</fld>, <cd>division among individual successors, as distinguished from <xex>universal succession</xex>, by which an estate descended in intestacy to the heirs in mass.</cd> -- <col><b>Singular term</b></col> <fld>(Logic)</fld>, <cd>a term which represents or stands for a single individual.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><syn><b>Syn.</b> -- Unexampled; unprecedented; eminent; extraordinary; remarkable; uncommon; rare; unusual; peculiar; strange; odd; eccentric; fantastic.</syn><br/
[<source>1913 Webster</source>]</p>

<p><ent>Singular</ent><br/
<hw>Sin"gu*lar</hw>, <pos>n.</pos> <sn>1.</sn> <def>An individual instance; a particular.</def> <mark>[Obs.]</mark>  <rj><au>Dr. H. More.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Gram)</fld> <def>The singular number, or the number denoting one person or thing; a word in the singular number.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Singularist</ent><br/
<hw>Sin"gu*lar*ist</hw> <pr>(s<icr/<nsm/"g<usl/*l<etil/r*<icr/st)</pr>, <pos>n.</pos> <def>One who affects singularity.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>A clownish <qex>singularist</qex>, or nonconformist to ordinary usage.</q> <rj><qau>Borrow.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Singularity</ent><br/
<hw>Sin`gu*lar"i*ty</hw> <pr>(s<icr/<nsm/`g<usl/*l<acr/r"<icr/*t<ycr/)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Singularities</plw> <pr>(s<icr/<nsm/`g<usl/*l<acr/r"<icr/*t<icr/z)</pr>.</plu> <ety>[L. <ets>singularitas</ets>: cf. F. <ets>singularit<eacute/</ets>.]</ety> <sn>1.</sn> <def>The quality or state of being singular; some character or quality of a thing by which it is distinguished from all, or from most, others; peculiarity.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Pliny addeth this <qex>singularity</qex> to that soil, that the second year the very falling down of the seeds yieldeth corn.</q> <rj><qau>Sir. W. Raleigh.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>I took notice of this little figure for the <qex>singularity</qex> of the instrument.</q> <rj><qau>Addison.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Anything singular, rare, or curious.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Your gallery<br/
Have we passed through, not without much content<br/
In many <qex>singularities</qex>.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>Possession of a particular or exclusive privilege, prerogative, or distinction.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>No bishop of Rome ever took upon him this name of <qex>singularity</qex> [universal bishop].</q> <rj><qau>Hooker.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Catholicism . . . must be understood in opposition to the legal <qex>singularity</qex> of the Jewish nation.</q> <rj><qau>Bp. Pearson.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>Celibacy.</def> <mark>[Obs.]</mark>  <rj><au>Jer. Taylor.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Singularize</ent><br/
<hw>Sin"gu*lar*ize</hw> <pr>(s<icr/<nsm/"g<usl/*l<etil/r*<imac/z)</pr>, <pos>v. t.</pos> <def>To make singular or single; to distinguish.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Singularly</ent><br/
<hw>Sin"gu*lar*ly</hw>, <pos>adv.</pos> <sn>1.</sn> <def>In a singular manner; in a manner, or to a degree, not common to others; extraordinarily; <as>as, to be <ex>singularly</ex> exact in one's statements; <ex>singularly</ex> considerate of others.</as></def> <ldquo/<xex>Singularly</xex> handsome.<rdquo/  <rj><au>Milman.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Strangely; oddly; <as>as, to behave <ex>singularly</ex></as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>So as to express one, or the singular number.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Singult</ent><br/
<hw>Sin"gult</hw> <pr>(?)</pr>, <pos>n.</pos><ety>[L. <ets>singultus</ets>.]</ety> <def>A sigh or sobbing; also, a hiccough.</def> <mark>[Obs.]</mark>  <rj><au>Spenser.</au>  <au>W. Browne.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Singultous</ent><br/
<hw>Sin*gul"tous</hw> <pr>(?)</pr>, <pos>a.</pos> <fld>(Med.)</fld> <def>Relating to, or affected with, hiccough.</def>  <rj><au>Dunglison.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Singultus</ent><br/
||<hw>Sin*gul"tus</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L.]</ety> <fld>(Med.)</fld> <def>Hiccough.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sinic</ent><br/
<hw>Sin"ic</hw> <pr>(s<icr/n"<icr/k)</pr>, <pos>a.</pos> <ety>[See <er>Sinologue</er>.]</ety> <fld>(Ethnol.)</fld> <def>Of or pertaining to the Chinese and allied races; Chinese.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Sinical</ent><br/
<hw>Sin"i*cal</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[From <er>Sine</er>.]</ety> <fld>(Trig.)</fld> <def>Of or pertaining to a sine; employing, or founded upon, sines; <as>as, a <ex>sinical</ex> quadrant</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sinicism</ent><br/
<hw>Sin"i*cism</hw> <pr>(s<icr/n"<icr/*s<icr/z'm)</pr>, <pos>n.</pos> <fld>(Ethnol.)</fld> <def>Anything peculiar to the Chinese; esp., a Chinese peculiarity in manners or customs.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Sinigrin</ent><br/
<hw>Sin"i*grin</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[From NL. <ets>Sinapis nigra</ets>.]</ety> <fld>(Chem.)</fld> <def>A glucoside found in the seeds of black mustard (<spn>Brassica nigra</spn>, formerly <spn>Sinapis nigra</spn>) It resembles sinalbin, and consists of a potassium salt of myronic acid.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sinister</ent><br/
<hw>Sin"is*ter</hw> <pr>(s<icr/n"<icr/s*t<etil/r; 277)</pr>, <pos>a.</pos> <note>[Accented on the middle syllable by the older poets, as Shakespeare, Milton, Dryden.]</note> <ety>[L. <ets>sinister</ets>: cf. F. <ets>sinistre</ets>.]</ety> <sn>1.</sn> <def>On the left hand, or the side of the left hand; left; -- opposed to <ant>dexter</ant>, or <ant>right</ant>.</def> <ldquo/Here on his <xex>sinister</xex> cheek.<rdquo/  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>My mother's blood<br/
Runs on the dexter cheek, and this <qex>sinister</qex><br/
Bounds in my father's</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ In heraldy the <xex>sinister</xex> side of an escutcheon is the side which would be on the left of the bearer of the shield, and opposite the right hand of the beholder.</note><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Unlucky; inauspicious; disastrous; injurious; evil; -- the left being usually regarded as the unlucky side; <as>as, <ex>sinister</ex> influences</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>All the several ills that visit earth,<br/
Brought forth by night, with a <qex>sinister</qex> birth.</q> <rj><qau>B. Jonson.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>Wrong, as springing from indirection or obliquity; perverse; dishonest; corrupt; <as>as, <ex>sinister</ex> aims</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Nimble and <qex>sinister</qex> tricks and shifts.</q> <rj><qau>Bacon.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>He scorns to undermine another's interest by any <qex>sinister</qex> or inferior arts.</q> <rj><qau>South.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>He read in their looks . . . <qex>sinister</qex> intentions directed particularly toward himself.</q> <rj><qau>Sir W. Scott.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>Indicative of lurking evil or harm; boding covert danger; <as>as, a <ex>sinister</ex> countenance</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Bar sinister</b></col>. <fld>(Her.)</fld> <cd>See under <er>Bar</er>, <pos>n.</pos></cd> -- <col><b>Sinister aspect</b></col> <fld>(Astrol.)</fld>, <cd>an appearance of two planets happening according to the succession of the signs, as Saturn in Aries, and Mars in the same degree of Gemini.</cd> -- <mcol><col><b>Sinister base</b></col>, <col><b>Sinister chief</b></col></mcol>. <cd>See under <er>Escutcheon</er>.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sinister-handed</ent><br/
<hw>Sin"is*ter-hand"ed</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Left-handed; hence, unlucky.</def> <mark>[Obs.]</mark>  <rj><au>Lovelace.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sinisterly</ent><br/
<hw>Sin"is*ter*ly</hw>, <pos>adv.</pos> <def>In a sinister manner.</def>  <rj><au>Wood.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sinistrad</ent><br/
<hw>Sin"is*trad</hw> <pr>(?)</pr>, <pos>adv.</pos> <ety>[L. <ets>sinistra</ets> the left hand + <ets>ad</ets> to.]</ety> <fld>(Anat. & Zool.)</fld> <def>Toward the left side; sinistrally.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sinistral</ent><br/
<hw>Sin"is*tral</hw> <pr>(?)</pr>, <pos>a.</pos> <sn>1.</sn> <def>Of or pertaining to the left, inclining to the left; sinistrous; -- opposed to <ant>dextral</ant>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Zool.)</fld> <def>Having the whorls of the spire revolving or rising to the left; reversed; -- said of certain spiral shells.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sinistrality</ent><br/
<hw>Sin`is*tral"i*ty</hw> <pr>(?)</pr>, <pos>n.</pos> <def>The quality or state of being sinistral.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sinistrally</ent><br/
<hw>Sin"is*tral*ly</hw> <pr>(?)</pr>, <pos>adv.</pos> <def>Toward the left; in a sinistral manner.</def>  <rj><au>J. Le Conte.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sinistrin</ent><br/
<hw>Sin"is*trin</hw> <pr>(s<icr/n"<icr/s*tr<icr/n)</pr>, <pos>n.</pos> <ety>[L. <ets>sinister</ets> left.]</ety> <fld>(Chem.)</fld> <def>A mucilaginous carbohydrate, resembling achroodextrin, extracted from squill as a colorless amorphous substance; -- so called because it is levorotatory.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sinistrorsal</ent><br/
<hw>Sin`is*tror"sal</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>sinistrorsus</ets>, <ets>sinistroversus</ets>, turned toward the left side; <ets>sinister</ets> left + <ets>vertere</ets>, <ets>vortere</ets>, <ets>versum</ets>, <ets>vorsum</ets>, to turn.]</ety> <def>Rising spirally from right to left (of the spectator); sinistrorse.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sinistrorse</ent><br/
<hw>Sin"is*trorse`</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[See <er>Sinistrolsal</er>.]</ety> <def>Turning to the left (of the spectator) in the ascending line; -- the opposite of <xex>dextrorse</xex>. See <er>Dextrorse</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sinistrous</ent><br/
<hw>Sin"is*trous</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[See <er>Sinister</er>.]</ety> <sn>1.</sn> <def>Being on the left side; inclined to the left; sinistral.</def> <ldquo/<xex>Sinistrous</xex> gravity.<rdquo/  <rj><au>Sir T. Browne.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><-- p. 1345 --></p>

<p><sn>2.</sn> <def>Wrong; absurd; perverse.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>A knave or fool can do no harm, even by the most <qex>sinistrous</qex> and absurd choice.</q> <rj><qau>Bentley.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sinistrously</ent><br/
<hw>Sin"is*trous*ly</hw> <pr>(s<icr/n"<icr/s*tr<ucr/s*l<ycr/)</pr>, <pos>adv.</pos> <sn>1.</sn> <def>In a sinistrous manner; perversely; wrongly; unluckily.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>With a tendency to use the left hand.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Many, in their infancy, are <qex>sinistrously</qex> disposed, and divers continue all their life left-handed.</q> <rj><qau>Sir T. Browne.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sink</ent><br/
<hw>Sink</hw> <pr>(s<icr/<nsm/k)</pr>, <pos>v. i.</pos> <vmorph>[<pos>imp.</pos> <conjf>Sunk</conjf> <pr>(s<ucr/<nsm/k)</pr>, or (<conjf>Sank</conjf> <pr>(s<acr/<nsm/k)</pr>); <pos>p. p.</pos> <conjf>Sunk</conjf> (<mark>obs.</mark> <conjf>Sunken</conjf>, -- now used as <pos>adj.</pos>); <pos>p. pr. & vb. n.</pos> <conjf>Sinking</conjf>.]</vmorph> <ety>[OE. <ets>sinken</ets>, AS. <ets>sincan</ets>; akin to D. <ets>zinken</ets>, OS. <ets>sincan</ets>, G. <ets>sinken</ets>, Icel. <ets>s<oum/kkva</ets>, Dan. <ets>synke</ets>, Sw. <ets>sjunka</ets>, Goth. <ets>siggan</ets>, and probably to E. <ets>silt</ets>.  Cf. <er>Silt</er>.]</ety> <sn>1.</sn> <def>To fall by, or as by, the force of gravity; to descend lower and lower; to decline gradually; to subside; <as>as, a stone <ex>sinks</ex> in water; waves rise and <ex>sink</ex>; the sun <ex>sinks</ex> in the west</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>I <qex>sink</qex> in deep mire.</q> <rj><qau>Ps. lxix. 2.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To enter deeply; to fall or retire beneath or below the surface; to penetrate.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The stone <qex>sunk</qex> into his forehead.</q> <rj><qau>1 San. xvii. 49.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>Hence, to enter so as to make an abiding impression; to enter completely.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Let these sayings <qex>sink</qex> down into your ears.</q> <rj><qau>Luke ix. 44.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>To be overwhelmed or depressed; to fall slowly, as so the ground, from weakness or from an overburden; to fail in strength; to decline; to decay; to decrease.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>I think our country <qex>sinks</qex> beneath the yoke.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>He <qex>sunk</qex> down in his chariot.</q> <rj><qau>2 Kings ix. 24.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Let not the fire <qex>sink</qex> or slacken.</q> <rj><qau>Mortimer.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <def>To decrease in volume, as a river; to subside; to become diminished in volume or in apparent height.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The Alps and Pyreneans <qex>sink</qex> before him.</q> <rj><qau>Addison.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><syn><b>Syn.</b> -- To fall; subside; drop; droop; lower; decline; decay; decrease; lessen.</syn><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sink</ent><br/
<hw>Sink</hw> <pr>(s<icr/<nsm/k)</pr>, <pos>v. t.</pos> <sn>1.</sn> <def>To cause to sink; to put under water; to immerse or submerge in a fluid; <as>as, to <ex>sink</ex> a ship</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>[The Athenians] fell upon the wings and <qex>sank</qex> a single ship.</q> <rj><qau>Jowett (Thucyd.).</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Figuratively: To cause to decline; to depress; to degrade; hence, to ruin irretrievably; to destroy, as by drowping; <as>as, to <ex>sink</ex> one's reputation</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>I raise of <qex>sink</qex>, imprison or set free.</q> <rj><qau>Prior.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>If I have a conscience, let it <qex>sink</qex> me.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Thy cruel and unnatural lust of power<br/
Has <qex>sunk</qex> thy father more than all his years.</q> <rj><qau>Rowe.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>To make (a depression) by digging, delving, or cutting, etc.; <as>as, to <ex>sink</ex> a pit or a well; to <ex>sink</ex> a die.</as></def><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>To bring low; to reduce in quantity; to waste.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>You <qex>sunk</qex> the river repeated draughts.</q> <rj><qau>Addison.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <def>To conseal and appropriate.</def> <mark>[Slang]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>If sent with ready money to buy anything, and you happen to be out of pocket, <qex>sink</qex> the money, and take up the goods on account.</q> <rj><qau>Swift.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>6.</sn> <def>To keep out of sight; to suppress; to ignore.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>A courtly willingness to <qex>sink</qex> obnoxious truths.</q> <rj><qau>Robertson.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>7.</sn> <def>To reduce or extinguish by payment; <as>as, to <ex>sink</ex> the national debt</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sink</ent><br/
<hw>Sink</hw> <pr>(s<icr/<nsm/k)</pr>, <pos>n.</pos> <sn>1.</sn> <def>A drain to carry off filthy water; a jakes.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A shallow box or vessel of wood, stone, iron, or other material, connected with a drain, and used for receiving filthy water, etc., as in a kitchen.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>A hole or low place in land or rock, where waters sink and are lost; -- called also <altname>sink hole</altname>.</def> <mark>[U. S.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>The lowest part of a natural hollow or closed basin whence the water of one or more streams escapes by evaporation; <as>as, the <ex>sink</ex> of the Humboldt River</as>.</def> <mark>[Western U. S.]</mark><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><cs><col><b>Sink hole</b></col>. <sd>(a)</sd> <cd>The opening to a sink drain</cd>. <sd>(b)</sd> <cd>A cesspool</cd>. <sd>(c)</sd> <cd>Same as <er>Sink</er>, <pos>n.</pos>, 3.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sinker</ent><br/
<hw>Sink"er</hw> <pr>(?)</pr>, <pos>n.</pos> <def>One who, or that which, sinks.</def> Specifically: <sd>(a)</sd> <def>A weight on something, as on a fish line, to sink it.</def> <sd>(b)</sd> <def>In knitting machines, one of the thin plates, blades, or other devices, that depress the loops upon or between the needles.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Dividing sinker</b></col>, <cd>in knitting machines, a sinker between two jack sinkers and acting alternately with them.</cd> -- <col><b>Jack sinker</b></col>. <cd>See under <er>Jack</er>, <pos>n.</pos></cd> -- <col><b>Sinker bar</b></col>. <sd>(a)</sd> <cd>In knitting machines, a bar to which one set of the sinkers is attached</cd>. <sd>(b)</sd> <cd>In deep well boring, a heavy bar forming a connection between the lifting rope and the boring tools, above the jars.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sinking</ent><br/
<hw>Sink"ing</hw>, <def><pos>a. & n.</pos> from <er>Sink</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Sinking fund</b></col>. <cd>See under <er>Fund</er>.</cd> -- <col><b>Sinking head</b></col> <fld>(Founding)</fld>, <cd>a riser from which the mold is fed as the casting shrinks. See <er>Riser</er>, <pos>n.</pos>, 4.</cd> -- <col><b>Sinking pump</b></col>, <cd>a pump which can be lowered in a well or a mine shaft as the level of the water sinks.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sinless</ent><br/
<hw>Sin"less</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Free from sin.</def>  <rj><au>Piers Plowman.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p>-- <wordforms><wf>Sin"less*ly</wf>, <pos>adv.</pos> -- <wf>Sin"less*ness</wf>, <pos>n.</pos></wordforms><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sinner</ent><br/
<hw>Sin"ner</hw> <pr>(?)</pr>, <pos>n.</pos> <def>One who has sinned; especially, one who has sinned without repenting; hence, a persistent and incorrigible transgressor; one condemned by the law of God.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sinner</ent><br/
<hw>Sin"ner</hw>, <pos>v. i.</pos> <def>To act as a sinner.</def> <mark>[Humorous]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>Whether the charmer <qex>sinner</qex> it or saint it.</q> <rj><qau>Pope.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sinneress</ent><br/
<hw>Sin"ner*ess</hw>, <pos>n.</pos> <def>A woman who sins.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sinnet</ent><br/
<hw>Sin"net</hw> <pr>(?)</pr>, <pos>n.</pos> <def>See <er>Sennit</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sinological</ent><br/
<hw>Sin`o*log"ic*al</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[See <er>Sinologue</er>.]</ety> <def>Relating to the Chinese language, literature or culture.</def><br/
[<source>1913 Webster</source> <source>+PJC</source>]</p>

<p><ent>Sinologist</ent><br/
<hw>Si*nol"o*gist</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A student of China and the Chinese; one versed in the Chinese language, literature, history, politics and culture.  Same as <er>sinologue</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sinologue</ent><br/
<hw>Sin"o*logue</hw> <pr>(s<icr/n"<osl/*l<ocr/g)</pr>, <pos>n.</pos> <ety>[From L. <ets>Sinae</ets>, an Oriental people mentioned by Ptolemy, or Ar. <ets>Sin</ets> China or the Chinese + Gr. <grk>lo`gos</grk> discourse; formed like <ets>theologue</ets>: cf. F. <ets>sinologue</ets>.]</ety> <def>A student of China and the Chinese; one versed in the Chinese language, literature, history, politics and culture.</def><br/
[<source>1913 Webster</source> <source>+PJC</source>]</p>

<p><ent>Sinology</ent><br/
<hw>Si*nol"o*gy</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Cf. F. <ets>sinologie</ets>.]</ety> <def>That branch of systemized knowledge which treats of the Chinese, their language, literature, etc.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sinoper</ent><br/
<hw>Sin"o*per</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Min.)</fld> <def>Sinople.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sinopis</ent><br/
<ent>Sinopia</ent><br/
<mhw>{ <hw>Si*no"pi*a</hw> <pr>(?)</pr>, <hw>Si*no"pis</hw> <pr>(?)</pr>, }</mhw> <pos>n.</pos> <def>A red pigment made from sinopite.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sinopite</ent><br/
<hw>Sin"o*pite</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F., fr. L. <ets>sinopis</ets> (sc. <ets>terra</ets>), a red earth or ocher found in <ets>Sinope</ets>, a town in Paphlagoma, on the Black Sea, Gr. <?/<?/<?/<?/.]</ety> <fld>(Min.)</fld> <def>A brick-red ferruginous clay used by the ancients for red paint.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sinople</ent><br/
<hw>Sin"o*ple</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Min.)</fld> <def>Ferruginous quartz, of a blood-red or brownish red color, sometimes with a tinge of yellow.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sinople</ent><br/
<hw>Sin"o*ple</hw>, <pos>n.</pos> <ety>[F., fr. LL. <ets>sinopis</ets>. See <er>Sinople</er> a mineral.]</ety> <fld>(Her.)</fld> <def>The tincture vert; green.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sinque</ent><br/
<hw>Sinque</hw> <pr>(?)</pr>, <pos>n.</pos> <def>See <er>Cinque</er>.</def> <mark>[Obs.]</mark>  <rj><au>Beau. & Fl.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sinsring</ent><br/
<hw>Sins"ring</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Zool.)</fld> <def>Same as <er>Banxring</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sinter</ent><br/
<hw>Sin"ter</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[G.  Cf. <er>Cinder</er>.]</ety> <fld>(Min.)</fld> <def>Dross, as of iron; the scale which files from iron when hammered; -- applied as a name to various minerals.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Calcareous sinter</b></col>, <cd>a loose banded variety of calcite formed by deposition from lime-bearing waters; calcareous tufa; travertine.</cd> -- <col><b>Ceraunian sinter</b></col>, <cd>fulgurite.</cd> -- <col><b>Siliceous sinter</b></col>, <cd>a light cellular or fibrous opal; especially, geyserite (see <er>Geyserite</er>). It has often a pearly luster, and is then called <xex>pearl sinter</xex>.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sintoist</ent><br/
<ent>Sintoism</ent><br/
<ent>Sintu</ent><br/
<ent>Sinto</ent><br/
<mhw>{ <hw>Sin"to</hw> <pr>(?)</pr>, <it>or</it> <hw>Sin"tu</hw> <pr>(?)</pr>, <hw>Sin"to*ism</hw> <pr>(?)</pr>, <hw>Sin"to*ist</hw>  }</mhw>. <def>See <er>Shinto</er>, etc.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sintoc</ent><br/
||<hw>Sin"toc</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A kind of spice used in the East Indies, consisting of the bark of a species of <gen>Cinnamomum.</gen></def> <altsp>[Written also <asp>sindoc</asp>.]</altsp><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sinuate</ent><br/
<hw>Sin"u*ate</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>sinuatus</ets>, p. p. of <ets>sinuare</ets> to wind, bend, fr. <ets>sinus</ets> a bend.]</ety> <def>Having the margin alternately curved inward and outward; having rounded lobes separated by rounded sinuses; sinuous; wavy.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sinuate</ent><br/
<hw>Sin"u*ate</hw> <pr>(?)</pr>, <pos>v. i.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Sinuated</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Sinuating</conjf>.]</vmorph> <def>To bend or curve in and out; to wind; to turn; to be sinuous.</def>  <rj><au>Woodward.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sinuated</ent><br/
<hw>Sin"u*a`ted</hw> <pr>(<?/)</pr>, <pos>a.</pos> <def>Same as <er>Sinuate</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sinuation</ent><br/
<hw>Sin`u*a"tion</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>sinuatio</ets>.]</ety> <def>A winding or bending in and out.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sinuose</ent><br/
<hw>Sin"u*ose`</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Sinuous.</def>  <rj><au>Loudon.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sinuosity</ent><br/
<hw>Sin`u*os"i*ty</hw> <pr>(?)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Sinuosities</plw> <pr>(#)</pr>.</plu> <ety>[Cf. F. <ets>sinuosit<eacute/</ets>.]</ety> <sn>1.</sn> <def>Quality or state of being sinuous.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A bend, or a series of bends and turns; a winding, or a series of windings; a wave line; a curve.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>A line of coast certainly amounting, with its <qex>sinuosities</qex>, to more than 700 miles.</q> <rj><qau>Sydney Smith.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sinuous</ent><br/
<hw>Sin"u*ous</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>sinuosus</ets>, fr. <ets>sinus</ets> a bent surface, a curve: cf. F. <ets>sinueux</ets>. See <er>Sinus</er>.]</ety> <def>Bending in and out; of a serpentine or undulating form; winding; crooked.</def> -- <wordforms><wf>Sin"u*ous*ly</wf>, <pos>adv.</pos></wordforms><br/
[<source>1913 Webster</source>]</p>

<p><q>Streaking the ground with <qex>sinuous</qex> trace.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Gardens bright with <qex>sinuous</qex> rills.</q> <rj><qau>Coleridge.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sinupalliate</ent><br/
<hw>Si`nu*pal"li*ate</hw> <pr>(?)</pr>, <pos>a.</pos> <fld>(Zool.)</fld> <def>Having a pallial sinus. See under <er>Sinus</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sinus</ent><br/
<hw>Si"nus</hw> <pr>(?)</pr>, <pos>n.</pos>; <plu><it>pl.</it> L. <plw>Sinus</plw>, E. <plw>Sinuses</plw> <pr>(#)</pr>.</plu> <ety>[L., a bent surface, a curve, the folds or bosom of a garment, etc., a bay.  Cf. <er>Sine</er>, <pos>n.</pos>]</ety> <sn>1.</sn> <def>An opening; a hollow; a bending.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A bay of the sea; a recess in the shore.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <fld>(Anat. & Zool.)</fld> <def>A cavity; a depression.</def> Specifically: <sd>(a)</sd> <def>A cavity in a bone or other part, either closed or with a narrow opening.</def> <sd>(b)</sd> <def>A dilated vessel or canal.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <fld>(Med.)</fld> <def>A narrow, elongated cavity, in which pus is collected; an elongated abscess with only a small orifice.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <fld>(Bot.)</fld> <def>A depression between adjoining lobes.</def><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ A sinus may be rounded, as in the leaf of the white oak, or acute, as in that of the red maple.</note><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Pallial sinus</b></col>. <fld>(Zool.)</fld> <cd>See under <er>Pallial</er>.</cd> -- <col><b>Sinus venosus</b></col> <pr>(?)</pr>. <ety>[L., venous dilatation.]</ety> <fld>(Anat.)</fld> <sd>(a)</sd> <cd>The main part of the cavity of the right auricle of the heart in the higher vertebrates.</cd> <sd>(b)</sd> <cd>In the lower vertebrates, a distinct chamber of the heart formed by the union of the large systematic veins and opening into the auricle.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sinusoid</ent><br/
<hw>Si"nus*oid</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[<ets>Sinus</ets> + <ets>-oid</ets>.]</ety> <fld>(Geom.)</fld> <def>The curve whose ordinates are proportional to the sines of the abscissas, the equation of the curve being <mathex>y = a sin x</mathex>. It is also called the <altname>curve of sines</altname>.</def><-- pref. sine curve --><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sinusoidal</ent><br/
<hw>Si`nus*oid"al</hw> <pr>(?)</pr>, <pos>a.</pos> <fld>(Geom.)</fld> <def>Of or pertaining to a sinusoid; like a sinusoid.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Siogoon</ent><br/
<hw>Sio"goon</hw> <pr>(?)</pr>, <pos>n.</pos> <def>See <er>Shogun</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Siogoonate</ent><br/
<hw>Sio*goon"ate</hw> <pr>(?)</pr>, <pos>n.</pos> <def>See <er>Shogunate</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sioux</ent><br/
<hw>Sioux</hw> <pr>(?)</pr>, <pos>prop. n. sing. & pl.</pos> <fld>(Ethnol.)</fld> <def>A nation of American Indians; see <er>Dakotas</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sioux State</ent><br/
<hw>Sioux State</hw>. <def>North Dakota; -- a nickname.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Sip</ent><br/
<hw>Sip</hw> <pr>(s<icr/p)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Sipped</conjf> <pr>(s<icr/pt)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Sipping</conjf>.]</vmorph> <ety>[OE. <ets>sippen</ets>; akin to OD. <ets>sippen</ets>, and AS. <ets>s<?/pan</ets> to sip, suck up, drink. See <er>Sup</er>, <pos>v. t.</pos>]</ety> <sn>1.</sn> <def>To drink or imbibe in small quantities; especially, to take in with the lips in small quantities, as a liquid; <as>as, to <ex>sip</ex> tea</as>.</def> <ldquo/Every herb that <xex>sips</xex> the dew.<rdquo/  <rj><au>Milton.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To draw into the mouth; to suck up; <as>as, a bee <ex>sips</ex> nectar from the flowers</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>To taste the liquor of; to drink out of.</def> <mark>[Poetic]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>They skim the floods, and <qex>sip</qex> the purple flowers.</q> <rj><qau>Dryden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sip</ent><br/
<hw>Sip</hw>, <pos>v. i.</pos> <def>To drink a small quantity; to take a fluid with the lips; to take a sip or sips of something.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>[She] raised it to her mouth with sober grace;<br/
Then, <qex>sipping</qex>, offered to the next in place.</q> <rj><qau>Dryden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sip</ent><br/
<hw>Sip</hw>, <pos>n.</pos> <sn>1.</sn> <def>The act of sipping; the taking of a liquid with the lips.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A small draught taken with the lips; a slight taste.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>One <qex>sip</qex> of this<br/
Will bathe the drooping spirits in delight<br/
Beyond the bliss of dreams.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>A <qex>sip</qex> is all that the public ever care to take from reservoirs of abstract philosophy.</q> <rj><qau>De Quincey.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sipage</ent><br/
<hw>Sip"age</hw> <pr>(?)</pr>, <pos>n.</pos> <def>See <er>Seepage</er>.</def> <mark>[Scot. & U.S.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sipe</ent><br/
<hw>Sipe</hw> <pr>(s<imac/p)</pr>, <pos>v. i.</pos> <def>See <er>Seep</er>.</def> <mark>[Scot. & U.S.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Siphilis</ent><br/
<hw>Siph"i*lis</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Med.)</fld> <def>Syphilis.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Siphoid</ent><br/
<hw>Si"phoid</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>sipho</ets> a siphon + <ets>-oid</ets>: cf. F. <ets>vase sipho<ium/de</ets>.]</ety> <def>A siphon bottle. See under <er>Siphon</er>, <pos>n.</pos></def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Siphon</ent><br/
<hw>Si"phon</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F. <ets>siphon</ets>, L. <ets>sipho</ets>, <ets>-onis</ets>, fr. Gr. <?/<?/<?/ a siphon, tube, pipe.]</ety> <sn>1.</sn> <def>A device, consisting of a pipe or tube bent so as to form two branches or legs of unequal length, by which a liquid can be transferred to a lower level, as from one vessel to another, over an intermediate elevation, by the action of the pressure of the atmosphere in forcing the liquid up the shorter branch of the pipe immersed in it, while the continued excess of weight of the liquid in the longer branch (when once filled) causes a continuous flow. The flow takes place only when the discharging extremity of the pipe ia lower than the higher liquid surface, and when no part of the pipe is higher above the surface than the same liquid will rise by atmospheric pressure; that is, about 33 feet for water, and 30 inches for mercury, near the sea level.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Zool.)</fld> <sd>(a)</sd> <def>One of the tubes or folds of the mantle border of a bivalve or gastropod mollusk by which water is conducted into the gill cavity. See <xex>Illust.</xex> under <er>Mya</er>, and <er>Lamellibranchiata</er>.</def> <sd>(b)</sd> <def>The anterior prolongation of the margin of any gastropod shell for the protection of the soft siphon.</def> <sd>(c)</sd> <def>The tubular organ through which water is ejected from the gill cavity of a cephaloid. It serves as a locomotive organ, by guiding and confining the jet of water. Called also <altname>siphuncle</altname>. See <xex>Illust.</xex> under <er>Loligo</er>, and <er>Dibranchiata</er>.</def> <sd>(d)</sd> <def>The siphuncle of a cephalopod shell.</def> <sd>(e)</sd> <def>The sucking proboscis of certain parasitic insects and crustaceans.</def> <sd>(f)</sd> <def>A sproutlike prolongation in front of the mouth of many gephyreans.</def> <sd>(g)</sd> <def>A tubular organ connected both with the esophagus and the intestine of certain sea urchins and annelids.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>A siphon bottle.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Inverted siphon</b></col>, <cd>a tube bent like a siphon, but having the branches turned upward; specifically <fld>(Hydraulic Engineering)</fld>, a pipe for conducting water beneath a depressed place, as from one hill to another across an intervening valley, following the depression of the ground.</cd> -- <col><b>Siphon barometer</b></col>. <cd>See under <er>Barometer</er>.</cd> -- <col><b>Siphon bottle</b></col>, <cd>a bottle for holding aerated water, which is driven out through a bent tube in the neck by the gas within the bottle when a valve in the tube is opened; -- called also <altname>gazogene</altname>, and <altname>siphoid</altname>.</cd><-- seltzer bottle? --> -- <col><b>Siphon condenser</b></col>, <cd>a condenser for a steam engine, in which the vacuum is maintained by the downward flow of water through a vertical pipe of great height.</cd> -- <col><b>Siphon cup</b></col>, <cd>a cup with a siphon attached for carrying off any liquid in it; specifically <fld>(Mach.)</fld>, an oil cup in which oil is carried over the edge of a tube in a cotton wick, and so reaches the surface to be lubricated.</cd> -- <col><b>Siphon gauge</b></col>. <cd>See under <er>Gauge</er>.</cd> -- <col><b>Siphon pump</b></col>, <cd>a jet pump. See under <er>Jet</er>, <pos>n.</pos></cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Siphon</ent><br/
<hw>Si"phon</hw> <pr>(?)</pr>, <pos>v. t.</pos> <fld>(Chem.)</fld> <def>To convey, or draw off, by means of a siphon, as a liquid from one vessel to another at a lower level.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Siphonage</ent><br/
<hw>Si"phon*age</hw> <pr>(?)</pr>, <pos>n.</pos> <def>The action of a siphon.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Siphonal</ent><br/
<hw>Si"phon*al</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Of or pertaining to a siphon; resembling a siphon.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Siphonal stomach</b></col> <fld>(Zool.)</fld>, <cd>a stomach which is tubular and bent back upon itself, like a siphon, as in the salmon.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Siphonarid</ent><br/
<hw>Si`pho*na"rid</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Zool.)</fld> <def>Any one of numerous species of limpet-shaped pulmonate gastropods of the genus <gen>Siphonaria</gen>. They cling to rocks between high and low water marks and have both lunglike organs and gills.</def> -- <wordforms><wf>Si`pho*na"rid</wf>, <pos>a.</pos></wordforms><br/
[<source>1913 Webster</source>]</p>

<p><ent>Siphonata</ent><br/
||<hw>Si`pho*na"ta</hw> <pr>(?)</pr>, <pos>n. pl.</pos> <ety>[NL.]</ety> <fld>(Zool.)</fld> <def>A tribe of bivalve mollusks in which the posterior mantle border is prolonged into two tubes or siphons. Called also <altname>Siphoniata</altname>. See <er>Siphon</er>, 2 <sd>(a)</sd>, and <er>Quahaug</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Siphonate</ent><br/
<hw>Si"phon*ate</hw> <pr>(?)</pr>, <pos>a.</pos> <sn>1.</sn> <def>Having a siphon or siphons.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Zool.)</fld> <def>Belonging to the Siphonata.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Siphonet</ent><br/
<hw>Si"phon*et</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Zool.)</fld> <def>One of the two dorsal tubular organs on the hinder part of the abdomen of aphids. They give exit to the honeydew. See <xex>Illust.</xex> under <er>Aphis</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Siphonia</ent><br/
||<hw>Si*pho"ni*a</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[NL.]</ety> <fld>(Bot.)</fld> <def>A former name for a euphorbiaceous genus (<gen>Hevea</gen>) of South American trees, the principal source of caoutchouc.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Siphoniata</ent><br/
||<hw>Si*pho`ni*a"ta</hw> <pr>(?)</pr>, <pos>n. pl.</pos> <ety>[NL.]</ety> <fld>(Zool.)</fld> <def>Same as <er>Siphonata</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Siphonic</ent><br/
<hw>Si*phon"ic</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Of or pertaining to a siphon.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Siphonifer</ent><br/
<hw>Si*phon"i*fer</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[NL., fr. L. <ets>sipho</ets>, <ets>-onis</ets>, siphon + <ets>ferre</ets> to bear.]</ety> <fld>(Zool.)</fld> <def>Any cephalopod having a siphonate shell.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Siphoniferous</ent><br/
<hw>Si"phon*if"er*ous</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[<ets>Siphon</ets> + <ets>-ferous</ets>.]</ety> <fld>(Zool.)</fld> <def>Siphon-bearing, as the shell of the nautilus and other cephalopods.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Siphonium</ent><br/
||<hw>Si*pho"ni*um</hw> <pr>(?)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Siphonia</plw> <pr>(#)</pr>.</plu> <ety>[NL., from Gr. <?/<?/<?/<?/, dim. of <?/<?/<?/<?/. See <er>Siphon</er>.]</ety> <fld>(Anat.)</fld> <def>A bony tube which, in some birds, connects the tympanium with the air chambers of the articular piece of the mandible.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Siphonobranchiata</ent><br/
||<hw>Si`pho*no*bran`chi*a"ta</hw> <pr>(?)</pr>, <pos>n. pl.</pos> <ety>[NL. See <er>Siphon</er>, and <er>Branchia</er>.]</ety> <fld>(Zool.)</fld> <def>A tribe of gastropods having the mantle border, on one or both sides, prolonged in the form of a spout through which water enters the gill cavity. The shell itself is not always siphonostomatous in this group.</def><br/
[<source>1913 Webster</source>]</p>

<p><-- p. 1346 --></p>

<p><ent>Siphonobranchiate</ent><br/
<hw>Si`pho*no*bran"chi*ate</hw> <pr>(?)</pr>, <pos>a.</pos> <fld>(Zool.)</fld> <def>Having a siphon, or siphons, to convey water to the gills; belonging or pertaining to the Siphonobranchiata.</def> -- <def2><pos>n.</pos> <def>One of the Siphonobranchiata.</def></def2><br/
[<source>1913 Webster</source>]</p>

<p><ent>Siphonoglyphe</ent><br/
<hw>Si`pho*nog"ly*phe</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[<ets>Siphon</ets> +  Gr. <?/<?/<?/ to engrave.]</ety> <fld>(Zool.)</fld> <def>A gonidium.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Siphonophora</ent><br/
||<hw>Si`pho*noph"o*ra</hw> <pr>(?)</pr>, <pos>prop. n. pl.</pos> <ety>[NL., fr. Gr. <?/<?/<?/ a siphon + <grk>fe`rein</grk> to bear.]</ety> <fld>(Zool.)</fld> <def>An order of pelagic Hydrozoa including species which form complex free-swimming communities composed of numerous zooids of various kinds, some of which act as floats or as swimming organs, others as feeding or nutritive zooids, and others as reproductive zooids. See <xex>Illust.</xex> under <er>Physallia</er>, and <er>Porpita</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Siphonophoran</ent><br/
<hw>Si`pho*noph"o*ran</hw> <pr>(?)</pr>, <pos>a.</pos> <fld>(Zool.)</fld> <def>Belonging to the <ord>Siphonophora</ord>.</def> -- <def2><pos>n.</pos> <def>One of the <ord>Siphonophora</ord>.</def></def2><br/
[<source>1913 Webster</source>]</p>

<p><ent>Siphonophore</ent><br/
<hw>Si*phon"o*phore</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Zool.)</fld> <def>One of the <ord>Siphonophora</ord>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Siphonopoda</ent><br/
||<hw>Si`pho*nop"o*da</hw> <pr>(?)</pr>, <pos>n. pl.</pos> <ety>[NL. See <er>Siphon</er>, and <er>-poda</er>.]</ety> <fld>(Zool.)</fld> <def>A division of Scaphopoda including those in which the foot terminates in a circular disk.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Siphonostomata</ent><br/
||<hw>Si`pho*no*stom"a*ta</hw> <pr>(?)</pr>, <pos>n. pl.</pos> <ety>[NL. See <er>Siphon</er>, and <er>Stoma</er>.]</ety> <fld>(Zool.)</fld> <sd>(a)</sd> <def>A tribe of parasitic copepod Crustacea including a large number of species that are parasites of fishes, as the lerneans. They have a mouth adapted to suck blood.</def> <sd>(b)</sd> <def>An artificial division of gastropods including those that have siphonostomatous shells.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Siphonostomatous</ent><br/
<hw>Si`pho*no*stom"a*tous</hw> <pr>(?)</pr>, <pos>a.</pos> <fld>(Zool.)</fld> <sd>(a)</sd> <def>Having the front edge of the aperture of the shell prolonged in the shape of a channel for the protection of the siphon; -- said of certain gastropods.</def> <sd>(b)</sd> <def>Pertaining to the Siphonostomata.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Siphonostome</ent><br/
<hw>Si`pho*nos"tome</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Gr. <?/<?/<?/ a siphon + <?/<?/<?/ mouth.]</ety> <fld>(Zool.)</fld> <sd>(a)</sd> <def>Any parasitic entomostracan of the tribe Siphonostomata.</def> <sd>(b)</sd> <def>A siphonostomatous shell.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Siphorhinal</ent><br/
<hw>Si`pho*rhi"nal</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[<ets>Siphon</ets> + <ets>rhinal</ets>.]</ety> <fld>(Zool.)</fld> <def>Having tubular nostrils, as the petrels.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Siphorhinian</ent><br/
<hw>Si`pho*rhin"i*an</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Zool.)</fld> <def>A siphorhinal bird.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Siphuncle</ent><br/
<hw>Si"phun`cle</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>siphunculus</ets>, <ets>sipunculus</ets>, dim. of <ets>sipho</ets>. See <er>Siphon</er>.]</ety> <fld>(Zool.)</fld> <def>The tube which runs through the partitions of chambered cephalopod shells.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Siphuncled</ent><br/
<hw>Si"phun`cled</hw> <pr>(?)</pr>, <pos>a.</pos> <fld>(Zool.)</fld> <def>Having a siphuncle; siphunculated.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Siphuncular</ent><br/
<hw>Si*phun"cu*lar</hw> <pr>(?)</pr>, <pos>a.</pos> <fld>(Zool.)</fld> <def>Of or pertaining to the siphuncle.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Siphunculated</ent><br/
<hw>Si*phun"cu*la`ted</hw> <pr>(?)</pr>, <pos>a.</pos> <fld>(Zool.)</fld> <def>Having a siphuncle.</def>  <rj><au>Huxley.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sipid</ent><br/
<hw>Sip"id</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[See <er>Insipid</er>, <er>Sapid</er>.]</ety> <def>Having a taste or flavorl savory; sapid.</def> <mark>[Obs.]</mark>  <rj><au>Cockeram.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sipper</ent><br/
<hw>Sip"per</hw> <pr>(?)</pr>, <pos>n.</pos> <def>One whi sips.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sippet</ent><br/
<hw>Sip"pet</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[See <er>Sip</er>, <er>Sop</er>.]</ety> <def>A small sop; a small, thin piece of toasted bread soaked in milk, broth, or the like; a small piece of toasted or fried bread cut into some special shape and used for garnishing.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Your sweet <qex>sippets</qex> in widows' houses.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sipple</ent><br/
<hw>Sip"ple</hw> <pr>(?)</pr>, <pos>v. i.</pos> <ety>[Freq. of <ets>sip</ets>.]</ety> <def>To sip often.</def> <mark>[Obs. or Scot.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sippling</ent><br/
<hw>Sip"pling</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Sipping often.</def> <mark>[Obs.]</mark> <ldquo/Taken after a <xex>sippling</xex> sort.<rdquo/  <rj><au>Holland.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sipunculacea</ent><br/
||<hw>Si*pun`cu*la"ce*a</hw> <pr>(?)</pr>, <pos>n. pl.</pos> <ety>[NL., from <ets>Sipunculus</ets>, the typical genus. See <er>Siphuncle</er>.]</ety> <fld>(Zool.)</fld> <def>A suborder of Gephyrea, including those which have the body unarmed and the intestine opening anteriorly.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sipunculoid</ent><br/
<hw>Si*pun"cu*loid</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[NL. <ets>Sipunculus</ets>, the typical genus + <ets>-oid</ets>.]</ety> <fld>(Zool.)</fld> <def>Pertaining to the Sipunculoidea.</def> -- <def2><pos>n.</pos> <def>One of the Sipunculoidea.</def></def2><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sipunculoidea</ent><br/
||<hw>Si*pun`cu*loi"de*a</hw> <pr>(?)</pr>, <pos>n. pl.</pos> <ety>[NL.]</ety> <fld>(Zool.)</fld> <sd>(a)</sd> <def>Same as <er>Gephyrea</er>.</def> <sd>(b)</sd> <def>In a restricted sense, same as <er>Sipunculacea</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Si quis</ent><br/
||<hw>Si` quis"</hw> <pr>(?)</pr>. <ety>[L., if any one (the first words of the notice in Latin).]</ety> <fld>(Ch. of Eng.)</fld> <def>A notification by a candidate for orders of his intention to inquire whether any impediment may be alleged against him.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sir</ent><br/
<hw>Sir</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[OE. <ets>sire</ets>, F. <ets>sire</ets>, contr. from the nominative L. <ets>senior</ets> an elder, elderly person, compar. of <ets>senex</ets>,<ets>senis</ets>, an aged person; akin to Gr. <?/<?/<?/ old, Skr. <ets>sana</ets>, Goth. <ets>sineigs</ets> old, <ets>sinista</ets> eldest, Ir. & Gael. <ets>sean</ets> old, W. <ets>hen</ets>.  Cf. <er>Seignior</er>, <er>Senate</er>, <er>Seneschal</er>, <er>Senior</er>, <er>Senor</er>, <er>Signor</er>, <er>Sire</er>, <er>Sirrah</er>.]</ety> <sn>1.</sn> <def>A man of social authority and dignity; a lord; a master; a gentleman; -- in this sense usually spelled <xex>sire</xex>.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>He was crowned lord and <qex>sire</qex>.</q> <rj><qau>Gower.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>In the election of a <qex>sir</qex> so rare.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A title prefixed to the Christian name of a knight or a baronet.</def><br/
[<source>1913 Webster</source>]</p>

<p><q><qex>Sir</qex> Horace Vere, his brother, was the principal in the active part.</q> <rj><qau>Bacon.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>An English rendering of the LAtin <xex>Dominus</xex>, the academical title of a bachelor of arts; -- formerly colloquially, and sometimes contemptuously, applied to the clergy.</def>  <rj><au>Nares.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Instead of a faithful and painful teacher, they hire a <qex>Sir</qex> John, which hath better skill in playing at tables, or in keeping of a garden, than in God's word.</q> <rj><qau>Latimer.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>A respectful title, used in addressing a man, without being prefixed to his name; -- used especially in speaking to elders or superiors; sometimes, also, used in the way of emphatic formality.</def> <ldquo/What's that to you, <xex>sir</xex>?<rdquo/  <rj><au>Sheridan.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ Anciently, this title, was often used when a person was addressed as a man holding a certain office, or following a certain business. <ldquo/<xex>Sir</xex> man of law.<rdquo/ <ldquo/<xex>Sir</xex> parish priest.<rdquo/</note>  <rj><au>Chaucer.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Sir reverance</b></col>. <cd>See under <er>Reverence</er>, <pos>n.</pos></cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Siraskier</ent><br/
<hw>Si*ras"kier</hw> <pr>(?)</pr>, <pos>n.</pos> <def>See <er>Seraskier</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Siraskierate</ent><br/
<hw>Si*ras"kier*ate</hw> <pr>(?)</pr>, <pos>n.</pos> <def>See <er>Seraskierate</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sirbonian</ent><br/
<hw>Sir*bo"ni*an</hw> <pr>(?)</pr>, <pos>a.</pos> <def>See <er>Serbonian</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sircar</ent><br/
<hw>Sir*car"</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Hind. & Per. <ets>sark<amac/r</ets> a superintendant, overseer, chief; Per. <ets>sar</ets> the head + <ets>k<amac/r</ets> action, work.]</ety> <sn>1.</sn> <def>A Hindoo clerk or accountant.</def> <mark>[India]</mark><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A district or province; a circar.</def> <mark>[India]</mark><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>The government; the supreme authority of the state.</def> <mark>[India]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sirdar</ent><br/
<hw>Sir*dar"</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Hind. & Per. <ets>sard<amac/r</ets> a chief, general; <ets>sar</ets> the head, top + <ets>d<amac/r</ets> holding, possessing.]</ety> <def>A native chief in Hindostan; a headman.</def>  <rj><au>Malcom.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>In Turkey, Egypt, etc., a commander in chief, esp. the one commanding the Anglo-Egyptian army.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Sire</ent><br/
<hw>Sire</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F. <ets>sire</ets>, originally, an older person. See <er>Sir</er>.]</ety> <sn>1.</sn> <def>A lord, master, or other person in authority. See <er>Sir</er>.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>Pain and distress, sickness and ire,<br/
And melancholy that angry <qex>sire</qex>,<br/
Be of her palace senators.</q> <rj><qau>Rom. of R.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A tittle of respect formerly used in speaking to elders and superiors, but now only in addressing a sovereign.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>A father; the head of a family; the husband.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Jankin thet was our <qex>sire</qex> [i.e., husband].</q> <rj><qau>Chaucer.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>And raise his issue, like a loving <qex>sire</qex>.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>A creator; a maker; an author; an originator.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>[He] was the <qex>sire</qex> of an immortal strain.</q> <rj><qau>Shelley.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <def>The male parent of a beast; -- applied especially to horses; <as>as, the horse had a good <ex>sire</ex></as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ <xex>Sire</xex> is often used in composition; as in grand<xex>sire</xex>, grandfather; great-grand<xex>sire</xex>, great-grandfather.</note><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sire</ent><br/
<hw>Sire</hw>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Sired</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Siring</conjf>.]</vmorph> <def>To beget; to procreate; -- used of beasts, and especially of stallions.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Siredon</ent><br/
<hw>Si*re"don</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[NL., from Gr. <?/<?/<?/ a siren.]</ety> <fld>(Zool.)</fld> <def>The larval form of any salamander while it still has external gills; especially, one of those which, like the axolotl (<spn>Amblystoma Mexicanum</spn>), sometimes lay eggs while in this larval state, but which under more favorable conditions lose their gills and become normal salamanders. See also <er>Axolotl</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Siren</ent><br/
<hw>Si"ren</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L., fr. Gr. <?/<?/<?/: cf. F. <ets>sir<egrave/ne</ets>.]</ety> <sn>1.</sn> <fld>(Class. Myth.)</fld> <def>One of three sea nymphs, -- or, according to some writers, of two, -- said to frequent an island near the coast of Italy, and to sing with such sweetness that they lured mariners to destruction.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Next where the <qex>sirens</qex> dwell you plow the seas;<br/
Their song is death, and makes destruction please.</q> <rj><qau>Pope.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>An enticing, dangerous woman.</def>  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>Something which is insidious or deceptive.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Consumption is a <qex>siren</qex>.</q> <rj><qau>W. Irving.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>A mermaid.</def> <mark>[Obs.]</mark>  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <fld>(Zool.)</fld> <def>Any long, slender amphibian of the genus <gen>Siren</gen> or family <fam>Sirenidae</fam>, destitute of hind legs and pelvis, and having permanent external gills as well as lungs. They inhabit the swamps, lagoons, and ditches of the Southern United States. The more common species (<spn>Siren lacertina</spn>) is dull lead-gray in color, and becames two feet long.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>6.</sn> <ety>[F. <ets>sir<egrave/ne</ets>, properly, a siren in sense 1.]</ety> <fld>(Acoustics)</fld> <def>An instrument for producing musical tones and for ascertaining the number of sound waves or vibrations per second which produce a note of a given pitch. The sounds are produced by a perforated rotating disk or disks. A form with two disks operated by steam or highly compressed air is used sounding an alarm to vessels in fog.</def> <altsp>[Written also <asp>sirene</asp>, and <asp>syren</asp>.]</altsp><br/
[<source>1913 Webster</source>]</p>

<p><ent>Siren</ent><br/
<hw>Si"ren</hw>, <pos>a.</pos> <def>Of or pertaining to a siren; bewitching, like a siren; fascinating; alluring; <as>as, a <ex>siren</ex> song</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sirene</ent><br/
<hw>Si*rene"</hw> <pr>(?)</pr>, <pos>n.</pos> <def>See <er>Siren</er>, 6.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sirenia</ent><br/
<hw>Si*re"ni*a</hw> <pr>(?)</pr>, <pos>n. pl.</pos> <ety>[NL.]</ety> <fld>(Zool.)</fld> <def>An order of large aquatic herbivorous mammals, including the manatee, dugong, rytina, and several fossil genera.</def><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ The hind limbs are either rudimentary or wanting, and the front ones are changed to paddles. They have horny plates on the front part of the jaws, and usually flat-crowned molar teeth. The stomach is complex and the intestine long, as in other herbivorous mammals. See <er>Cetacea</er> <sd>(b)</sd>.</note><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sirenian</ent><br/
<hw>Si*re"ni*an</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Zool.)</fld> <def>Any species of Sirenia.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sirenical</ent><br/
<hw>Si*ren"ic*al</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Like, or appropriate to, a siren; fascinating; deceptive.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Here's couple of <qex>sirenical</qex> rascals shall enchant ye.</q> <rj><qau>Marton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sirenize</ent><br/
<hw>Si"ren*ize</hw> <pr>(?)</pr>, <pos>v. i.</pos> <def>To use the enticements of a siren; to act as a siren; to fascinate.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Siriasis</ent><br/
||<hw>Si*ri"a*sis</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L., fr. Gr. <?/<?/<?/, fr. <?/<?/<?/ the Dog Star, properly, scorching.]</ety> <fld>(Med.)</fld> <sd>(a)</sd> <def>A sunstroke.</def> <sd>(b)</sd> <def>The act of exposing to a sun bath. <mark>[Obs.]</mark> Cf. <er>Insolation</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sirius</ent><br/
<hw>Sir"i*us</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L., fr. Gr. <?/<?/<?/, properly, scorching.]</ety> <fld>(Astron.)</fld> <def>The Dog Star. See <er>Dog Star</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sirkeer</ent><br/
<hw>Sir"keer</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Zool.)</fld> <def>Any one of several species of Asiatic cuckoos of the genus <gen>Taccocua</gen>, as the Bengal <ex>sirkeer</ex> (<spn>Taccocua sirkee</spn>).</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sirloin</ent><br/
<hw>Sir"loin`</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[A corruption of <ets>surloin</ets>. Not so called because this cut of beef was once jocosely knighted (dubbed Sir Loin) by an English king, as according to a popular story.]</ety> <def>A loin of beef, or a part of a loin.</def> <altsp>[Written also <asp>surloin</asp>.]</altsp><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sirname</ent><br/
<hw>Sir"name`</hw> <pr>(?)</pr>, <pos>n.</pos> <def>See <er>Surname</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Siroc</ent><br/
<hw>Si"roc</hw> <pr>(?)</pr>, <pos>n.</pos> <def>See <er>Sirocco</er>.</def> <mark>[Poetic]</mark>  <rj><au>Emerson.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sirocco</ent><br/
<hw>Si*roc"co</hw> <pr>(s<icr/*r<ocr/k"k<osl/)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Siroccos</plw> <pr>(s<icr/*r<ocr/k"k<omac/z)</pr>.</plu> <ety>[It. <ets>sirocco</ets>, <ets>scirocco</ets>, Ar. <ets>shorug</ets>, fr. <ets>sharq</ets> the rising of the sun, the east, fr, <ets>sharaca</ets> to rise as the sun.  Cf. <er>Saracen</er>.]</ety> <sn>1.</sn> <def>An oppressive, relaxing wind from the Libyan deserts, chiefly experienced in Italy, Malta, and Sicily.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>In general, any hot dry wind of cyclonic origin, blowing from arid or heated regions, including the desert wind of Southern California, the <stype>harmattan</stype> of the west coasts of Africa, the hot winds of Kansas and Texas, the <stype>kamsin</stype> of Egypt, the <stype>leste</stype> of the Madeira Islands, and the <stype>leveche</stype> of Spain.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Sirrah</ent><br/
<hw>Sir"rah</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Probably from Icel. <ets>s<imac/ra</ets>, fr. F. <ets>sire</ets>. See <er>Sir</er>.]</ety> <def>A term of address implying inferiority and used in anger, contempt, reproach, or disrespectful familiarity, addressed to a man or boy, but sometimes to a woman. In sililoquies often preceded by <xex>ah</xex>. Not used in the plural.</def> <ldquo/Ah, <xex>sirrah</xex> mistress.<rdquo/
<-- archaic -->  <rj><au>Beau. & Fl.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Go, <qex>sirrah</qex>, to my cell.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sirt</ent><br/
<hw>Sirt</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[See <er>Syrt</er>.]</ety> <def>A quicksand.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Syrup</ent><br/
<ent>Sirup</ent><br/
<mhw>{ <hw>Sir"up</hw> <pr>(?)</pr> <hw>Syr"up</hw>  }</mhw>, <pos>n.</pos> <ety>[F. <ets>sirop</ets> (cf. It. <ets>siroppo</ets>, Sp. <ets>jarabe</ets>, <ets>jarope</ets>, LL. <ets>siruppus</ets>, <ets>syrupus</ets>), fr. Ar. <ets>shar<amac/b</ets> a drink, wine, coffee, sirup.  Cf. <er>Sherbet</er>.]</ety> <sn>1.</sn> <def>A thick and viscid liquid made from the juice of fruits, herbs, etc., boiled with sugar.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A thick and viscid saccharine solution of superior quality (as sugarhouse <xex>sirup</xex> or molasses, maple <xex>sirup</xex>); specifically, in pharmacy and often in cookery, a saturated solution of sugar and water (<xex>simple sirup</xex>), or such a solution flavored or medicated.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Lucent <qex>sirups</qex> tinct with cinnamon.</q> <rj><qau>Keats.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Mixing sirup</b></col>. <cd>See the Note under <er>Dextrose</er>.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Syruped</ent><br/
<ent>Siruped</ent><br/
<mhw>{ <hw>Sir"uped</hw> <pr>(?)</pr>, <hw>Syr"uped</hw>  }</mhw>, <pos>a.</pos> <def>Moistened, covered, or sweetened with sirup, or sweet juice.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Syrupy</ent><br/
<ent>Sirupy</ent><br/
<mhw>{ <hw>Sir"up*y</hw> <pr>(?)</pr>, <hw>Syr"up*y</hw>  }</mhw>, <pos>a.</pos> <def>Like sirup, or partaking of its qualities.</def>  <rj><au>Mortimer.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sirvente</ent><br/
||<hw>Sir`vente"</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F. <ets>sirvente</ets>, fr. Pr. <ets>sirventes</ets>, <ets>sirventesc</ets>, originally, the poem of, or concerning, a <ets>sirvent</ets>, fr. <ets>sirvent</ets>, properly, serving, n., one who serves (<it>e. g.</it>, as a soldier), fr. <ets>servir</ets> to serve, L. <ets>servire</ets>.]</ety> <def>A peculiar species of poetry, for the most part devoted to moral and religious topics, and commonly satirical, -- often used by the troubadours of the Middle Ages.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sis</ent><br/
<hw>Sis</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A colloquial abbreviation of <er>Sister</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sis</ent><br/
<hw>Sis</hw> <pr>(?)</pr>, <pos>n.</pos> <def>Six. See <er>Sise</er>.</def> <mark>[Obs.]</mark>  <rj><au>Chaucer.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sisal hemp</ent><br/
<ent>Sisal grass</ent><br/
<mhw>{ <hw>Si*sal" grass`</hw> <pr>(?)</pr>, <hw>Si*sal" hemp`</hw> <pr>(?)</pr>,}</mhw> <def>The prepared fiber of the <spn>Agave Americana</spn>, or American aloe, used for cordage; -- so called from <etsep>Sisal</etsep>, a port in Yucatan. See <cref>Sisal hemp</cref>, under <er>Hemp</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Siscowet</ent><br/
<hw>Sis"co*wet</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[OF American Indian origin.]</ety> <fld>(Zool.)</fld> <def>A large, fat variety of the namaycush found in Lake Superior; -- called also <altname>siskawet</altname>, <altname>siskiwit</altname>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sise</ent><br/
<hw>Sise</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[From <er>Assize</er>.]</ety> <def>An assize.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sise</ent><br/
<hw>Sise</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[See <er>Sice</er>.]</ety> <def>Six; the highest number on a die; the cast of six in throwing dice.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>In the new casting of a die, when ace is on the top, <qex>sise</qex> must needs be at the bottom.</q> <rj><qau>Fuller.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sisel</ent><br/
<hw>Sis"el</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Cf. G. <ets>ziesel</ets>.  Cf. <er>Zizel</er>.]</ety> <fld>(Zool.)</fld> <def>The suslik.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Siser</ent><br/
<hw>Si"ser</hw> <pr>(?)</pr>, <pos>n.</pos> <def>Cider. See <er>Sicer</er>.</def> <mark>[Obs.]</mark>  <rj><au>Chaucer.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Siserary</ent><br/
<ent>Siserara</ent><br/
<mhw>{ <hw>Sis"e*ra*ra</hw> <pr>(?)</pr>, <hw>Sis"e*ra*ry</hw> <pr>(?)</pr>, }</mhw> <pos>n.</pos> <def>A hard blow.</def> <mark>[Prov. Eng.]</mark>  <rj><au>Halliwell.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Siskin</ent><br/
<hw>Sis"kin</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Dan. <ets>sisgen</ets>; cf. Sw. <ets>siska</ets>, G. <ets>zeisig</ets>, D. <ets>sijsje</ets>; of Slav. origin; cf. Pol. <ets>czy<zdot/</ets>.]</ety> <fld>(Zool.)</fld> <sd>(a)</sd> <def>A small green and yellow European finch (<spn>Spinus spinus</spn>, or <spn>Carduelis spinus</spn>); -- called also <altname>aberdevine</altname>.</def> <sd>(b)</sd> <def>The American pinefinch (<spn>Spinus pinus</spn>); -- called also <altname>pine siskin</altname>. See <er>Pinefinch</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ The name is applied also to several other related species found in Asia and South America.</note><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Siskin green</b></col>, <cd>a delicate shade of yellowish green, as in the mineral torbernite.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Siskiwit</ent><br/
<hw>Sis"ki*wit</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Zool.)</fld> <def>The siscowet.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sismograph</ent><br/
<hw>Sis"mo*graph</hw> <pr>(?)</pr>, <pos>n.</pos> <def>See <er>Seismograph</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sismometer</ent><br/
<hw>Sis*mom"e*ter</hw> <pr>(?)</pr>, <pos>n.</pos> <def>See <er>Seismometer</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Siss</ent><br/
<hw>Siss</hw> <pr>(?)</pr>, <pos>v. i.</pos> <ety>[Of imitative origin; cf. D. <ets>sissen</ets>, G. <ets>zischen</ets>.]</ety> <def>To make a hissing sound; <as>as, a flatiron hot enough to <ex>siss</ex> when touched with a wet finger</as>.</def> <mark>[Colloq. U. S.; Local, Eng.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Siss</ent><br/
<hw>Siss</hw>, <pos>n.</pos> <def>A hissing noise.</def> <mark>[Colloq. U. S.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sissoo</ent><br/
<hw>Sis*soo"</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Hind. <ets>s<imac/s<?/</ets>.]</ety> <fld>(Bot.)</fld> <def>A leguminous tree (<spn>Dalbergia Sissoo</spn>) of the northern parts of India; also, the dark brown compact and durable timber obtained from it. It is used in shipbuilding and for gun carriages, railway ties, etc.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sist</ent><br/
<hw>Sist</hw> <pr>(?)</pr>, <pos>v. t.</pos> <ety>[L. <ets>sistere</ets> to bring to a stand, to stop.]</ety> <sn>1.</sn> <fld>(Scots Law)</fld> <def>To stay, as judicial proceedings; to delay or suspend; to stop.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To cause to take a place, as at the bar of a court; hence, to cite; to summon; to bring into court.</def> <mark>[Scot.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>Some, however, have preposterously <qex>sisted</qex> nature as the first or generative principle.</q> <rj><qau>Sir W. Hamilton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><-- p. 1347 --></p>

<p><ent>Sist</ent><br/
<hw>Sist</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Scots Law)</fld> <def>A stay or suspension of proceedings; an order for a stay of proceedings.</def>  <rj><au>Burril.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sister</ent><br/
<hw>Sis"ter</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[OE. <ets>sister</ets>, fr. Icel. systir; also <ets>suster</ets>, from AS. <ets>sweostor</ets>, <ets>sweoster</ets>, <ets>swuster</ets>, akin to OFries. <ets>sweester</ets>, <ets>suster</ets>, LG. <ets>s<uum/ster</ets>, <ets>suster</ets>, D. <ets>zuster</ets>, OS. & OHG. <ets>swestar</ets>, G. <ets>schwester</ets>, Icel. <ets>systir</ets>, Sw. <ets>syster</ets>, Dan. <ets>s<oum/ster</ets>, Goth. <ets>swistar</ets>, Lith. <ets>ses<?/</ets>, Russ. <ets>sestra</ets>, Pol. <ets>siostra</ets>, L. <ets>soror</ets>, Skr. <ets>svasr</ets>. <root/298.  Cf. <er>Cousin</er>.]</ety> <sn>1.</sn> <def>A female who has the same parents with another person, or who has one of them only. In the latter case, she is more definitely called a <xex>half sister</xex>. The correlative of <xex>brother</xex>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>I am the <qex>sister</qex> of one Claudio.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A woman who is closely allied to, or assocciated with, another person, as in the sdame faith, society, order, or community.</def>  <rj><au>James ii. 15.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>One of the same kind, or of the same condition; -- generally used adjectively; <as>as, <ex>sister</ex> fruits</as>.</def>  <rj><au>Pope.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Sister Block</b></col> <fld>(Naut.)</fld>, <cd>a tackle block having two sheaves, one above the other.</cd> -- <col><b>Sister hooks</b></col>, <cd>a pair of hooks fitted together, the shank of one forming a mousing for the other; -- called also <altname>match hook</altname>.</cd> -- <mcol><col><b>Sister of charity</b></col>, <col><b>Sister of mercy</b></col></mcol>. <fld>(R. C. Ch.)</fld> <cd>See under <er>Charity</er>, and <er>Mercy</er>.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sister</ent><br/
<hw>Sis"ter</hw>, <pos>v. t.</pos> <def>To be sister to; to resemble closely.</def> <mark>[Obs.]</mark>  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sisterhood</ent><br/
<hw>Sis"ter*hood</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[<ets>Sister</ets> + <ets>hood</ets>.]</ety> <sn>1.</sn> <def>The state or relation of being a sister; the office or duty of a sister.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>She . . . abhorr'd<br/
Her proper blood, and left to do the part<br/
Of <qex>sisterhood</qex>, to do that of a wife.</q> <rj><qau>Daniel.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A society of sisters; a society of women united in one faith or order; sisters, collectively.</def> <ldquo/A <xex>sisterhood</xex> of holy nuns.<rdquo/  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>The fair young flowers . . . a beauteous <qex>sisterhood</qex>.</q> <rj><qau>Bryant.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sistering</ent><br/
<hw>Sis"ter*ing</hw>, <pos>a.</pos> <def>Contiguous.</def> <mark>[Obs.]</mark>  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sister-in-law</ent><br/
<hw>Sis"ter-in-law`</hw> <pr>(?)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Sisters-in-law</plw> <pr>(<?/)</pr>.</plu> <def>The sister of one's husband or wife; also, the wife of one's brother; sometimes, the wife of one's husband's or wife's brother.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sisterly</ent><br/
<hw>Sis"ter*ly</hw>, <pos>a.</pos> <def>Like a sister; becoming a sister, affectionate; <as>as, <ex>sisterly</ex> kindness; <ex>sisterly</ex> remorse.</as></def>  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sistine</ent><br/
<hw>Sis"tine</hw> <pr>(?)</pr>, <pos>a.</pos><ety>[It. <ets>sistino</ets>.]</ety> <def>Of or pertaining to Pope Sixtus.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Sistine chapel</b></col>, <cd>a chapel in the Vatican at Rome, built by Pope Sixtus IV., and decorated with frescoes by Michael Angelo and others.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sistren</ent><br/
<hw>Sis"tren</hw> <pr>(?)</pr>, <pos>n. pl.</pos> <def>Sisters.</def> <mark>[Obs.]</mark>  <rj><au>Chaucer.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sistrum</ent><br/
||<hw>Sis"trum</hw> <pr>(?)</pr>, <ety>[L., fr. Gr. <?/<?/<?/, from <?/<?/<?/ to shake.]</ety> <fld>(Mus.)</fld> <def>An instrument consisting of a thin metal frame, through which passed a number of metal rods, and furnished with a handle by which it was shaken and made to rattle. It was peculiarly Egyptian, and used especially in the worship of Isis. It is still used in Nubia.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sisyphean</ent><br/
<hw>Sis`y*phe"an</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Relating to Sisyphus; incessantly recurring; <as>as, <ex>Sisyphean</ex> labors</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sisyphus</ent><br/
<hw>Sis"y*phus</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>Sisyphus</ets>, Sisyphus, fr. Gr. <?/<?/<?/<?/.]</ety> <fld>(Class. Myth.)</fld> <def>A king of Corinth, son of Aeolus, famed for his cunning. He was killed by Theseus, and in the lower world was condemned by Pluto to roll to the top of a hill a huge stone, which constantly rolled back again, making his task incessant.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sit</ent><br/
<hw>Sit</hw> <pr>(?)</pr>, <def><pos>obs. 3d pers. sing. pres.</pos> of <er>Sit</er>, for <xex>sitteth</xex>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sit</ent><br/
<hw>Sit</hw>, <pos>v. i.</pos> <vmorph>[<pos>imp.</pos> <conjf>Sat</conjf> <pr>(?)</pr> (<conjf>Sate</conjf> <pr>(?)</pr>, <mark>archaic</mark>); <pos>p. p.</pos> <conjf>Sat</conjf> (<conjf>Sitten</conjf> <pr>(?)</pr>, <mark>obs.</mark>); <pos>p. pr. & vb. n.</pos> <conjf>Sitting</conjf>.]</vmorph> <ety>[OE. <ets>sitten</ets>, AS.  <ets>sittan</ets>; akin to OS. <ets>sittian</ets>, OFries. <ets>sitta</ets>, D. <ets>zitten</ets>, G. <ets>sitzen</ets>, OHG. <ets>sizzen</ets>, Icel. <ets>sitja</ets>, SW. <ets>sitta</ets>, Dan. <ets>sidde</ets>, Goth. <ets>sitan</ets>, Russ. <ets>sidiete</ets>, L. <ets>sedere</ets>, Gr. <?/<?/<?/, Skr. <ets>sad</ets>. <root/154.  Cf. <er>Assess</er>,<er>Assize</er>, <er>Cathedral</er>, <er>Chair</er>, <er>Dissident</er>, <er>Excise</er>, <er>Insidious</er>, <er>Possess</er>, <er>Reside</er>, <er>Sanhedrim</er>, <er>Seance</er>, <er>Seat</er>, <pos>n.</pos>, <er>Sedate</er>, <er>4th Sell</er>, <er>Siege</er>, <er>Session</er>, <er>Set</er>, <pos>v. t.</pos>, <er>Sizar</er>, <er>Size</er>, <er>Subsidy</er>.]</ety> <sn>1.</sn> <def>To rest upon the haunches, or the lower extremity of the trunk of the body; -- said of human beings, and sometimes of other animals; <as>as, to <ex>sit</ex> on a sofa, on a chair, or on the ground</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>And he came and took the book put of the right hand of him that <qex>sate</qex> upon the seat.</q> <rj><qau>Bible (1551) (Rev. v. 7.)</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>I pray you, jest, sir, as you <qex>sit</qex> at dinner.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To perch; to rest with the feet drawn up, as birds do on a branch, pole, etc.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>To remain in a state of repose; to rest; to abide; to rest in any position or condition.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>And Moses said to . . . the children of Reuben, Shall your brothren go to war, and shall ye <qex>sit</qex> here?</q> <rj><qau>Num. xxxii. 6.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Like a demigod here <qex>sit</qex> I in the sky.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>To lie, rest, or bear; to press or weigh; -- with <xex>on</xex>; <as>as, a weight or burden <ex>sits</ex> lightly upon him</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The calamity <qex>sits</qex> heavy on us.</q> <rj><qau>Jer. Taylor.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <def>To be adjusted; to fit; <as>as, a coat <ex>sits</ex> well or ill</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>This new and gorgeous garment, majesty,<br/
<qex>Sits</qex> not so easy on me as you think.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>6.</sn> <def>To suit one well or ill, as an act; to become; to befit; -- used impersonally.</def> <mark>[Obs.]</mark>  <rj><au>Chaucer.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>7.</sn> <def>To cover and warm eggs for hatching, as a fowl; to brood; to incubate.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>As the partridge <qex>sitteth</qex> on eggs, and hatcheth them not.</q> <rj><qau>Jer. xvii. 11.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>8.</sn> <def>To have position, as at the point blown from; to hold a relative position; to have direction.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Like a good miller that knows how to grind, which way soever the wind <qex>sits</qex>.</q> <rj><qau>Selden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q><qex>Sits</qex> the wind in that quarter?</q> <rj><qau>Sir W. Scott.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>9.</sn> <def>To occupy a place or seat as a member of an official body; <as>as, to <ex>sit</ex> in Congress</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>10.</sn> <def>To hold a session; to be in session for official business; -- said of legislative assemblies, courts, etc.; <as>as, the court <ex>sits</ex> in January; the aldermen <ex>sit</ex> to-night</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>11.</sn> <def>To take a position for the purpose of having some artistic representation of one's self made, as a picture or a bust; <as>as, to <ex>sit</ex> to a painter</as>.</def>
<-- sit for seems more common now --><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>To sit at</b></col>, <cd>to rest under; to be subject to.</cd> <mark>[Obs.]</mark> <ldquo/A farmer can not husband his ground so well if he <xex>sit at</xex> a great rent<rdquo/. <au>Bacon.</au> -- <mcol><col><b>To sit at meat</b></col> <it>or</it> <col><b>To sit at table</b></col></mcol>, <cd>to be at table for eating.</cd> -- <col><b>To sit down</b></col>. <sd>(a)</sd> <cd>To place one's self on a chair or other seat; <as>as, <ex>to sit down</ex> when tired</as>.</cd>  <sd>(b)</sd> <cd>To begin a siege; <as>as, the enemy <ex>sat down</ex> before the town</as>.</cd>  <sd>(c)</sd> <cd>To settle; to fix a permanent abode</cd>. <au>Spenser.</au> <sd>(d)</sd> <cd>To rest; to cease as satisfied.</cd> <ldquo/Here we can not <xex>sit down</xex>, but still proceed in our search.<rdquo/ <au>Rogers.</au> -- <col><b>To sit for a fellowship</b></col>, <cd>to offer one's self for examination with a view to obtaining a fellowship.</cd> <mark>[Eng. Univ.]</mark> -- <col><b>To sit out</b></col>. <sd>(a)</sd> <cd>To be without engagement or employment</cd>. <mark>[Obs.]</mark> <au>Bp. Sanderson.</au> <sd>(b)</sd> <cd>To outstay.</cd><sd>(c)</sd><cd>To elect not to participate in, as a dance or a hand of cards.</cd><-- ##(c) to refrain from joining in [an activity, project] --> -- <col><b>To sit under</b></col>, <cd>to be under the instruction or ministrations of; as, <xex>to sit under</xex> a preacher; <xex>to sit under</xex> good preaching.</cd> -- <col><b>To sit up</b></col>, <cd>to rise from, or refrain from, a recumbent posture or from sleep; to sit with the body upright; as, <xex>to sit up</xex> late at night; also, to watch; <as>as, <ex>to sit up</ex> with a sick person</as>.</cd> <ldquo/He that was dead <xex>sat up</xex>, and began to speak.<rdquo/ <au>Luke vii. 15.</au></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sit</ent><br/
<hw>Sit</hw> <pr>(?)</pr>, <pos>v. t.</pos> <sn>1.</sn> <def>To sit upon; to keep one's seat upon; <as>as, he <ex>sits</ex> a horse well</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Hardly the muse can <qex>sit</qex> the headstrong horse.</q> <rj><qau>Prior.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To cause to be seated or in a sitting posture; to furnish a seat to; -- used reflexively.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>They <qex>sat</qex> them down to weep.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q><qex>Sit</qex> you down, father; rest you.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>To suit (well <it>or</it>  ill); to become.</def> <mark>[Obs. or R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Site</ent><br/
<hw>Site</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>situs</ets>, fr. <ets>sinere</ets>, <ets>situm</ets>, to let, p. p. <ets>situs</ets> placed, lying, situate: cf. F. <ets>site</ets>.  Cf. <er>Position</er>.]</ety> <sn>1.</sn> <def>The place where anything is fixed; situation; local position; <as>as, the <ex>site</ex> of a city or of a house</as>.</def>  <rj><au>Chaucer.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A place fitted or chosen for any certain permanent use or occupation; <as>as, a <ex>site</ex> for a church</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>The posture or position of a thing.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>The semblance of a lover fixed<br/
In melancholy <qex>site</qex>.</q> <rj><qau>Thomson.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sited</ent><br/
<hw>Sit"ed</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Having a site; situated.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>[The garden] <qex>sited</qex> was in fruitful soil.</q> <rj><qau>Chaucer.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sitfast</ent><br/
<hw>Sit"fast`</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[<ets>Sit</ets> + <ets>fast</ets>.]</ety> <def>Fixed; stationary; immovable.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>'T is good, when you have crossed the sea and back,<br/
To find the <qex>sitfast</qex> acres where you left them.</q> <rj><qau>Emerson.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sitfast</ent><br/
<hw>Sit"fast`</hw>, <pos>n.</pos> <fld>(Far.)</fld> <def>A callosity with inflamed edges, on the back of a horse, under the saddle.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sith</ent><br/
<hw>Sith</hw> <pr>(?)</pr>, <pos>prep., adv., & conj.</pos> <ety>[See <er>Since</er>.]</ety> <def>Since; afterwards; seeing that.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>We need not fear them, <qex>sith</qex> Christ is with us.</q> <rj><qau>Latimer.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q><qex>Sith</qex> thou art rightful judge.</q> <rj><qau>Chaucer.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sithe</ent><br/
<ent>Sith</ent><br/
<mhw>{ <hw>Sith</hw> <pr>(?)</pr>, <hw>Sithe</hw> <pr>(?)</pr>, }</mhw> <pos>n.</pos> <ety>[AS. <?/<?/<?/ a path, way, time, occasion.]</ety> <def>Time.</def> <mark>[Obs.]</mark>  <rj><au>Chaucer.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>And humbly thanked him a thousand <qex>sithes</qex>.</q> <rj><qau>Spenser.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sithe</ent><br/
<hw>Sithe</hw> <pr>(?)</pr>, <pos>v. i.</pos> <ety>[Cf. <er>Sigh</er>.]</ety> <def>To sigh.</def> <note>[A spelling of a corrupt and provincial pronunciation.]</note><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sithe</ent><br/
<hw>Sithe</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A scythe.</def> <mark>[Obs.]</mark>  <rj><au>Milton.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sithe</ent><br/
<hw>Sithe</hw>, <pos>v. t.</pos> <def>To cut with a scythe; to scythe.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sithed</ent><br/
<hw>Sithed</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Scythed.</def> <mark>[Obs.]</mark>  <rj><au>T. Warton.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sitheman</ent><br/
<hw>Sithe"man</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A mower.</def> <mark>[Obs.]</mark>  <rj><au>Marston.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sithen</ent><br/
<hw>Sith"en</hw> <pr>(?)</pr>, <pos>adv. & conj.</pos> <ety>[See <er>Since</er>.]</ety> <def>Since; afterwards. See 1st <er>Sith</er>.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>Fortune was first friend and <qex>sithen</qex> foe.</q> <rj><qau>Chaucer.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sithens</ent><br/
<ent>Sithence</ent><br/
<mhw>{ <hw>Sith"ence</hw>, <hw>Sith"ens</hw>  }</mhw> <pr>(?)</pr>, <pos>adv. & conj.</pos> <def>Since. See <er>Sith</er>, and <er>Sithen</er>.</def> <mark>[Obs.]</mark>  <rj><au>Piers Plowman.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Siththen</ent><br/
<hw>Sith"then</hw> <pr>(?)</pr>, <pos>adv. & conj.</pos> <def>See <er>Sithen</er>.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q><qex>Siththen</qex> that the world began.</q> <rj><qau>Chaucer.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sitology</ent><br/
<hw>Si*tol"o*gy</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Gr. <?/<?/<?/<?/ food + <ets>-logy</ets>.]</ety> <def>A treatise on the regulation of the diet; dietetics.</def> <altsp>[Written also <asp>sitiology</asp>.]</altsp><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sitophobia</ent><br/
<hw>Si`to*pho"bi*a</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[NL., fr. Gr. <?/<?/<?/<?/ food + <?/<?/<?/<?/ fear.]</ety> <fld>(Med.)</fld> <def>A version to food; refusal to take nourishment.</def> <altsp>[Written also <asp>sitiophobia</asp>.]</altsp><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sitten</ent><br/
<hw>Sit"ten</hw> <pr>(?)</pr>, <mark>obs.</mark> <def><pos>p. p.</pos> of <er>Sit</er>, for <xex>sat</xex>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sitter</ent><br/
<hw>Sit"ter</hw> <pr>(?)</pr>, <pos>n.</pos> <sn>1.</sn> <def>One who sits; esp., one who sits for a portrait or a bust.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A bird that sits or incubates.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sittine</ent><br/
<hw>Sit"tine</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[NL. <ets>sitta</ets> the nuthatch, from Gr. <?/<?/<?/.]</ety> <fld>(Zool.)</fld> <def>Of or pertaining to the family <fam>Sittidae</fam>, or nuthatches.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sitting</ent><br/
<hw>Sit"ting</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Being in the state, or the position, of one who, or that which, sits.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sitting</ent><br/
<hw>Sit"ting</hw>, <pos>n.</pos> <sn>1.</sn> <def>The state or act of one who sits; the posture of one who occupies a seat.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A seat, or the space occupied by or allotted for a person, in a church, theater, etc.; <as>as, the hall has 800 <ex>sittings</ex></as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>The act or time of sitting, as to a portrait painter, photographer, etc.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>The actual presence or meeting of any body of men in their seats, clothed with authority to transact business; a session; <as>as, a <ex>sitting</ex> of the judges of the King's Bench, or of a commission</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The <qex>sitting</qex> closed in great agitation.</q> <rj><qau>Macaulay.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <def>The time during which one sits while doing something, as reading a book, playing a game, etc.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>For the understanding of any one of St. Paul's Epistles I read it all through at one <qex>sitting</qex>.</q> <rj><qau>Locke.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>6.</sn> <def>A brooding over eggs for hatching, as by fowls.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The male bird . . . amuses her [the female] with his songs during the whole time of her <qex>sitting</qex>.</q> <rj><qau>Addison.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Sitting room</b></col>, <cd>an apartment where the members of a family usually sit, as distinguished from a drawing-room, parlor, chamber, or kitchen.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Situated</ent><br/
<ent>Situate</ent><br/
<mhw>{ <hw>Sit"u*ate</hw> <pr>(?; 135)</pr>, <hw>Sit"u*a`ted</hw> <pr>(?)</pr>, }</mhw> <pos>a.</pos> <ety>[LL. <ets>situatus</ets>, from <ets>situare</ets> to place, fr. L. <ets>situs</ets> situation, site. See <er>Site</er>.]</ety> <sn>1.</sn> <def>Having a site, situation, or location; being in a relative position; permanently fixed; placed; located; <as>as, a town <ex>situated</ex>, or <ex>situate</ex>, on a hill or on the seashore</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Placed; residing.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Pleasure <qex>situate</qex> in hill and dale.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ <xex>Situate</xex> is now less used than <xex>situated</xex>, but both are well authorized.</note><br/
[<source>1913 Webster</source>]</p>

<p><ent>Situate</ent><br/
<hw>Sit"u*ate</hw> <pr>(?)</pr>, <pos>v. t.</pos> <def>To place.</def> <mark>[R.]</mark>  <rj><au>Landor.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Situation</ent><br/
<hw>Sit`u*a"tion</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[LL. <ets>situatio</ets>: cf. F. <ets>situation</ets>.]</ety> <sn>1.</sn> <def>Manner in which an object is placed; location, esp. as related to something else; position; locality site; <as>as, a house in a pleasant <ex>situation</ex></as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Position, as regards the conditions and circumstances of the case.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>A <qex>situation</qex> of the greatest ease and tranquillity.</q> <rj><qau>Rogers.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>Relative position; circumstances; temporary state or relation at a moment of action which excites interest, as of persons in a dramatic scene.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>There's <qex>situation</qex> for you! there's an heroic group!</q> <rj><qau>Sheridan.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>Permanent position or employment; place; office; <as>as, a <ex>situation</ex> in a store; a <ex>situation</ex> under government</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><syn><b>Syn.</b> -- State; position; seat; site; station; post; place; office; condition; case; plight. See <er>State</er>.</syn><br/
[<source>1913 Webster</source>]</p>

<p><ent>Situs</ent><br/
||<hw>Si"tus</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L., situation.]</ety> <fld>(Bot.)</fld> <def>The method in which the parts of a plant are arranged; also, the position of the parts.</def>  <rj><au>Henslow.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sitz bath</ent><br/
<hw>Sitz" bath`</hw> <pr>(?)</pr>. <ety>[G. <ets>sitzbad</ets>.]</ety> <def>A tub in which one bathes in a sitting posture; also, a bath so taken; a hip bath.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Siva</ent><br/
||<hw>Si"va</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Skr. <ets>Civa</ets>, properly, kind, gracious.]</ety> <fld>(Hindoo Myth.)</fld> <def>One of the triad of Hindoo gods. He is the avenger or destroyer, and in modern worship symbolizes the reproductive power of nature.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sivan</ent><br/
||<hw>Si"van</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Heb. <ets>s<imac/v<amac/n</ets>.]</ety> <def>The third month of the Jewish ecclesiastical year; -- supposed to correspond nearly with our month of June.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sivatherium</ent><br/
||<hw>Siv`a*the"ri*um</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[NL., from E. <ets>Siva</ets> + Gr. <?/<?/<?/<?/ a beast, an animal.]</ety> <fld>(Paleon.)</fld> <def>A genus of very large extinct ruminants found in the Tertiary formation of India. The snout was prolonged in the form of a proboscis. The male had four horns, the posterior pair being large and branched. It was allied to the antelopes, but very much larger than any exsisting species.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Siver</ent><br/
<hw>Siv"er</hw> <pr>(?)</pr>, <pos>v. i.</pos> <def>To simmer.</def>  <mark>[Obs.]</mark>  <rj><au>Holland.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sivvens</ent><br/
<hw>Siv"vens</hw> <pr>(<?/)</pr>, <pos>n.</pos> <fld>(Med.)</fld> <def>See <er>Sibbens</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Siwin</ent><br/
<hw>Si"win</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Zool.)</fld> <def>Same as <er>Sewen</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Six</ent><br/
<hw>Six</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[AS. <ets>six</ets>, <ets>seox</ets>, <ets>siex</ets>; akin to OFries. <ets>sex</ets>, D. <ets>zes</ets>, OS. & OHG. <ets>sehs</ets>, G. <ets>sechs</ets>, Icel., Sw., & Dan. <ets>sex</ets>, Goth. <ets>sa<iacute/hs</ets>, Lith. <ets>szeszi</ets>, Russ. <ets>sheste</ets>, Gael. & Ir. <ets>se</ets>, W. <ets>chwech</ets>, L. <ets>sex</ets>, Gr. <?/<?/, Per. <ets>shesh</ets>, Skr. <ets>shash</ets>. <root/304.  Cf. <er>Hexagon</er>, <er>Hexameter</er>, <er>Samite</er>, <er>Senary</er>, <er>Sextant</er>, <er>Sice</er>.]</ety> <def>One more than five; twice three; <as>as, <ex>six</ex> yards</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Six Nations</b></col> <fld>(Ethnol.)</fld>, <cd>a confederation of North American Indians formed by the union of the Tuscaroras and the Five Nations.</cd> -- <col><b>Six points circle</b></col>. <fld>(Geom.)</fld> <cd>See <cref>Nine points circle</cref>, under <er>Nine</er>.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Six</ent><br/
<hw>Six</hw>, <pos>n.</pos> <sn>1.</sn> <def>The number greater by a unit than five; the sum of three and three; six units or objects.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A symbol representing six units, as 6, vi., or VI.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><mcol><col><b>To be at six and seven</b></col> <it>or</it> <col><b>To be at sixes and sevens</b></col></mcol>, <sd>(a)</sd> <cd>to be in disorder.</cd> <au>Bacon.</au>  <au>Shak.</au>  <au>Swift.</au>  <sd>(b)</sd> <cd>to be in a dispute or disagreement; -- often used with <ptcl>with</ptcl>.</cd></cs><br/
[<source>1913 Webster</source> <source>+PJC</source>]</p>

<p><ent>Sixfold</ent><br/
<hw>Six"fold`</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[AS. <ets>sixfealand</ets>.]</ety> <def>Six times repeated; six times as much or as many.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Six-footer</ent><br/
<hw>Six"-foot`er</hw> <pr>(?)</pr>, <pos>n.</pos> <def>One who is six feet tall.</def> <mark>[Colloq. U.S.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sixpence</ent><br/
<hw>Six"pence</hw> <pr>(?)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Sixpences</plw> <pr>(<?/)</pr>.</plu> <def>An English silver coin of the value of six pennies; half a shilling, or about twelve cents.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sixpenny</ent><br/
<hw>Six"pen`ny</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Of the value of, or costing, sixpence; <as>as, a <ex>sixpenny</ex> loaf</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sixscore</ent><br/
<hw>Six"score`</hw> <pr>(?)</pr>, <pos>a. & n.</pos> <ety>[<ets>Six</ets> + <ets>score</ets>, n.]</ety> <def>Six times twenty; one hundred and twenty.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Six-shooter</ent><br/
<hw>Six"-shoot`er</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A pistol or other firearm which can be fired six times without reloading especially, a six-chambered revolver.</def> <mark>[Colloq. U.S.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sixteen</ent><br/
<hw>Six"teen`</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[AS. <ets>sixt<?/ne</ets>, <ets>sixt<?/ne</ets>. See <er>Six</er>, and <er>Ten</er>, and cf. <er>Sixty</er>.]</ety> <def>Six and ten; consisting of six and ten; fifteen and one more.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sixteen</ent><br/
<hw>Six"teen`</hw>, <pos>n.</pos> <sn>1.</sn> <def>The number greater by a unit than fifteen; the sum of ten and six; sixteen units or objects.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A symbol representing sixteen units, as 16, or xvi.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sixteenmo</ent><br/
<hw>Six*teen"mo</hw> <pr>(?)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Sixteenmos</plw> <pr>(<?/)</pr>.</plu> <def>See <er>Sextodecimo</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sixteenth</ent><br/
<hw>Six"teenth`</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[From <er>Sixteen</er>: cf. AS. <ets>sixte<oacute/<?/a</ets>.]</ety> <sn>1.</sn> <def>Sixth after the tenth; next in order after the fifteenth.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Constituting or being one of sixteen equal parts into which anything is divided.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Sixteenth note</b></col> <fld>(Mus.)</fld>, <cd>the sixteenth part of a whole note; a semiquaver.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sixteenth</ent><br/
<hw>Six"teenth`</hw>, <pos>n.</pos> <sn>1.</sn> <def>The quotient of a unit divided by sixteen; one of sixteen equal parts of one whole.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>The next in order after the fifteenth; the sixth after the tenth.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <fld>(Mus.)</fld> <def>An interval comprising two octaves and a second.</def>  <rj><au>Moore (Encyc. of Music.)</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sixth</ent><br/
<hw>Sixth</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[From Six: cf. AS. <ets>sixta</ets>, <ets>siexta</ets>.]</ety> <sn>1.</sn> <def>First after the fifth; next in order after the fifth.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Constituting or being one of six equal parts into which anything is divided.</def><br/
[<source>1913 Webster</source>]</p>

<p><-- p. 1348 --></p>

<p><ent>Sixth</ent><br/
<hw>Sixth</hw> <pr>(?)</pr>, <pos>n.</pos> <sn>1.</sn> <def>The quotient of a unit divided by six; one of six equal parts which form a whole.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>The next in order after the fifth.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <fld>(Mus.)</fld> <def>The interval embracing six diatonic degrees of the scale.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sixthly</ent><br/
<hw>Sixth"ly</hw>, <pos>adv.</pos> <def>In the sixth place.</def>  <rj><au>Bacon.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sixtieth</ent><br/
<hw>Six"ti*eth</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[As. <ets>sixtiogo<edh/a</ets>, <ets>sixtigo<edh/a</ets>.]</ety> <sn>1.</sn> <def>Next in order after the fifty-ninth.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Constituting or being one one of sixty equal parts into which anything is divided.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sixtieth</ent><br/
<hw>Six"ti*eth</hw>, <pos>n.</pos> <sn>1.</sn> <def>The quotient of a unit divided by sixty; one of sixty equal parts forming a whole.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>The next in order after the fifty-ninth; the tenth after the fiftieth.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sixty</ent><br/
<hw>Six"ty</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[AS. <ets>siextig</ets>; akin to G. <ets>sechzig</ets>, Goth. <ets>sa<iacute/hs tigjus</ets>. See <er>Six</er>, <er>Ten</er>, and cf. <er>Sixteen</er>.]</ety> <def>Six times ten; fifty-nine and one more; threescore.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sixty</ent><br/
<hw>Six"ty</hw>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Sixties</plw> <pr>(<?/)</pr>.</plu> <sn>1.</sn> <def>The sum of six times ten; sixty units or objects.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A symbol representing sixty units, as 60, lx., or LX.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sixty-fourth</ent><br/
<hw>Six`ty-fourth"</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Constituting or being one of sixty-four equal parts into which a thing is divided.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Sixty-fourth note</b></col> <fld>(Mus.)</fld>, <cd>the sixty-fourth part of a whole note; a hemi-demi-semiquaver.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sizable</ent><br/
<hw>Siz"a*ble</hw> <pr>(?)</pr>, <pos>a.</pos> <sn>1.</sn> <def>Of considerable size or bulk.</def> <ldquo/A <xex>sizable</xex> volume.<rdquo/  <rj><au>Bp. Hurd.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Being of reasonable or suitable size; <as>as, <ex>sizable</ex> timber; <ex>sizable</ex> bulk</as>.</def>  <rj><au>Arbuthnot.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sizar</ent><br/
<hw>Si"zar</hw> <pr>(?)</pr>, <pos>n.</pos> <def>One of a body of students in the universities of Cambridge (Eng.) and Dublin, who, having passed a certain examination, are exempted from paying college fees and charges. A <xex>sizar</xex> corresponded to a <xex>servitor</xex> at Oxford.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The <qex>sizar</qex> paid nothing for food and tuition, and very little for lodging.</q> <rj><qau>Macaulay.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ They formerly waited on the table at meals; but this is done away with. They were probably so called from being thus employed in distributing the <xex>size</xex>, or provisions. See 4th <er>Size</er>, 2.</note><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sizarship</ent><br/
<hw>Si"zar*ship</hw>, <pos>n.</pos> <def>The position or standing of a sizar.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Size</ent><br/
<hw>Size</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[See <er>Sice</er>, and <er>Sise</er>.]</ety> <def>Six.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Size</ent><br/
<hw>Size</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[OIt. <ets>sisa</ets> glue used by painters, shortened fr. <ets>assisa</ets>, fr. <ets>assidere</ets>, p. p. <ets>assiso</ets>, to make to sit, to seat, to place, L. <ets>assidere</ets> to sit down; <ets>ad</ets> + <ets>sidere</ets> to sit down, akin to <ets>sedere</ets> to sit. See <er>Sit</er>, <pos>v. i.</pos>, and cf. <er>Assize</er>, <er>Size</er> bulk.]</ety> <sn>1.</sn> <def>A thin, weak glue used in various trades, as in painting, bookbinding, paper making, etc.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Any viscous substance, as gilder's varnish.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Size</ent><br/
<hw>Size</hw>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Sized</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Sizing</conjf>.]</vmorph> <def>To cover with size; to prepare with size.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Size</ent><br/
<hw>Size</hw>, <pos>n.</pos> <ety>[Abbrev. from <ets>assize</ets>. See <er>Assize</er>, and cf. <er>Size</er> glue.]</ety> <sn>1.</sn> <def>A settled quantity or allowance. See <er>Assize</er>.</def> <mark>[Obs.]</mark> <ldquo/To scant my <xex>sizes</xex>.<rdquo/  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Univ. of Cambridge, Eng.)</fld> <def>An allowance of food and drink from the buttery, aside from the regular dinner at commons; -- corresponding to <xex>battel</xex> at Oxford.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>Extent of superficies or volume; bulk; bigness; magnitude; <as>as, the <ex>size</ex> of a tree or of a mast; the <ex>size</ex> of a ship or of a rock</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>Figurative bulk; condition as to rank, ability, character, etc.; <as>as, the office demands a man of larger <ex>size</ex></as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Men of a less <qex>size</qex> and quality.</q> <rj><qau>L'Estrange.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>The middling or lower <qex>size</qex> of people.</q> <rj><qau>Swift.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <def>A conventional relative measure of dimension, as for shoes, gloves, and other articles made up for sale.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>6.</sn> <def>An instrument consisting of a number of perforated gauges fastened together at one end by a rivet, -- used for ascertaining the size of pearls.</def>  <rj><au>Knight.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Size roll</b></col>, <cd>a small piese of parchment added to a roll.</cd> -- <col><b>Size stick</b></col>, <cd>a measuring stick used by shoemakers for ascertaining the size of the foot.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><syn><b>Syn.</b> -- Dimension; bigness; largeness; greatness; magnitude.</syn><br/
[<source>1913 Webster</source>]</p>

<p><ent>Size</ent><br/
<hw>Size</hw>, <pos>v. t.</pos> <sn>1.</sn> <def>To fix the standard of.</def> <ldquo/To <xex>size</xex> weights and measures.<rdquo/ <mark>[R.]</mark>  <rj><au>Bacon.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To adjust or arrange according to size or bulk.</def> Specifically: <sd>(a)</sd> <fld>(Mil.)</fld> <def>To take the height of men, in order to place them in the ranks according to their stature.</def> <sd>(b)</sd> <fld>(Mining)</fld> <def>To sift, as pieces of ore or metal, in order to separate the finer from the coarser parts.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>To swell; to increase the bulk of.</def>  <rj><au>Beau. & Fl.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <fld>(Mech.)</fld> <def>To bring or adjust anything exactly to a required dimension, as by cutting.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>To size up</b></col>, <cd>to estimate or ascertain the character and ability of. See 4th <er>Size</er>, 4.</cd> <mark>[Slang, U.S.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>We had <qex>to size up</qex> our fellow legislators.</q> <rj><qau>The Century.</qau></rj>
</cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Size</ent><br/
<hw>Size</hw>, <pos>v. i.</pos> <sn>1.</sn> <def>To take greater size; to increase in size.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Our desires give them fashion, and so,<br/
As they wax lesser, fall, as they <qex>size</qex>, grow.</q> <rj><qau>Donne.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Univ. of Cambridge, Eng.)</fld> <def>To order food or drink from the buttery; hence, to enter a score, as upon the buttery book.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sized</ent><br/
<hw>Sized</hw> <pr>(?)</pr>, <pos>a.</pos> <sn>1.</sn> <def>Adjusted according to size.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Having a particular size or magnitude; -- chiefly used in compounds; <as>as, large-<ex>sized</ex>; common-<ex>sized</ex></as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sizel</ent><br/
<hw>Si"zel</hw> <pr>(?)</pr>, <pos>n.</pos> <def>Same as <er>Scissel</er>, 2.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sizer</ent><br/
<hw>Siz"er</hw> <pr>(?)</pr>, <pos>n.</pos> <sn>1.</sn> <def>See <er>Sizar</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Mech.)</fld> <sd>(a)</sd> <def>An instrument or contrivance to size articles, or to determine their size by a standard, or to separate and distribute them according to size.</def> <sd>(b)</sd> <def>An instrument or tool for bringing anything to an exact size.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Siziness</ent><br/
<hw>Siz"i*ness</hw> <pr>(?)</pr>, <pos>n.</pos> <def>The quality or state of being sizy; viscousness.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sizing</ent><br/
<hw>Siz"ing</hw>, <pos>n.</pos> <sn>1.</sn> <def>Act of covering or treating with size.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A weak glue used in various trades; size.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sizing</ent><br/
<hw>Siz"ing</hw>, <pos>n.</pos> <sn>1.</sn> <def>The act of sorting with respect to size.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>The act of bringing anything to a certain size.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <fld>(Univ. of Cambridge, Eng.)</fld> <def>Food and drink ordered from the buttery by a student.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sizy</ent><br/
<hw>Siz"y</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[From 2d <er>Size</er>.]</ety> <def>Sizelike; viscous; glutinous; <as>as, <ex>sizy</ex> blood</as>.</def>  <rj><au>Arbuthnot.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sizzle</ent><br/
<hw>Siz"zle</hw> <pr>(?)</pr>, <pos>v. i.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Sizzled</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Sizzling</conjf> <pr>(?)</pr>.]</vmorph> <ety>[See <er>Siss</er>.]</ety> <def>To make a hissing sound; to fry, or to dry and shrivel up, with a hissing sound.</def> <mark>[Prov. Eng. & Colloq. U. S.]</mark>  <rj><au>Forby.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sizzle</ent><br/
<hw>Siz"zle</hw>, <pos>n.</pos> <def>A hissing sound, as of something frying over a fire.</def> <mark>[Prov. Eng. & Colloq. U. S.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sizzling</ent><br/
<hw>Siz"zling</hw> <pr>(?)</pr>, <def><pos>a. & n.</pos> from <er>Sizzle</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Skaddle</ent><br/
<hw>Skad"dle</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Dim. of <ets>scath</ets>.]</ety> <def>Hurt; damage.</def> <mark>[Obs. or Prov. Eng.]</mark>  <rj><au>Ray.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Skaddle</ent><br/
<hw>Skad"dle</hw>, <pos>a.</pos> <def>Hurtful.</def> <mark>[Obs. or Prov. Eng.]</mark>  <rj><au>Ray.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Skaddon</ent><br/
<hw>Skad"don</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Zool.)</fld> <def>The larva of a bee.</def> <mark>[Prov. Eng.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Skag</ent><br/
<hw>Skag</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Naut.)</fld> <def>An additional piece fastened to the keel of a boat to prevent lateral motion. See <er>Skeg</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Skain</ent><br/
<hw>Skain</hw> <pr>(?)</pr>, <pos>n.</pos> <def>See <er>Skein</er>.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Skain</ent><br/
<hw>Skain</hw>, <pos>n.</pos> <def>See <er>Skean</er>.</def>  <rj><au>Drayton.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Skainsmate</ent><br/
<hw>Skains"mate`</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Perhaps originally, a companion in winding thread (see <er>Skein</er>), or a companion in arms, from <ets>skain</ets> a sword (see <er>Skean</er>).]</ety> <def>A messmate; a companion.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>Scurvy knave! I am none of his firt-gills; I am none of his <qex>skainsmates</qex>.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Skaith</ent><br/
<hw>Skaith</hw> <pr>(?)</pr>, <pos>n.</pos> <def>See <er>Scatch</er>.</def> <mark>[Scot.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Skald</ent><br/
<hw>Skald</hw> <pr>(?)</pr>, <pos>n.</pos> <def>See 5th <er>Scald</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Skaldic</ent><br/
<hw>Skald"ic</hw> <pr>(?)</pr>, <pos>a.</pos> <def>See <er>Scaldic</er>.</def>  <rj><au>Max M<uum/ller.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Skall</ent><br/
<hw>Skall</hw> <pr>(?)</pr>, <pos>v. t.</pos> <def>To scale; to mount.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Skare</ent><br/
<ent>Skar</ent><br/
<mhw>{ <hw>Skar</hw> <pr>(?)</pr>, <hw>Skare</hw> <pr>(?)</pr>, }</mhw> <pos>a.</pos> <ety>[From the root of <ets>scare</ets>.]</ety> <def>Wild; timid; shy.</def> <mark>[Prov. Eng. & Scot.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Skart</ent><br/
<hw>Skart</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Cf. <er>Scarf</er> a cormorant.]</ety> <fld>(Zool.)</fld> <def>The shag.</def> <mark>[Prov. Eng.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Skat</ent><br/
<hw>Skat</hw> <pr>(sk<aum/t)</pr>, <pos>n.</pos> <ety>[G., fr. It. <ets>scartare</ets> to discard.]</ety> <sn>1.</sn> <def>A three-handed card game played with 32 cards, of which  two constitute the skat (sense 2), or widow. The players bid for the privilege of attempting any of several games or tasks, in most of which the player undertaking the game must take tricks counting in aggregate at least 61 (the counting cards being ace 11, ten 10, king 4, queen 3, jack 2). The four jacks are the best trumps, ranking club, spade, heart, diamond, and ten outranks king or queen (but when the player undertakes to lose all the tricks, the cards rank as in whist). The value of hands depends upon the game played, trump suit, points taken, and number of matadores.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><sn>2.</sn>  <fld>(Skat)</fld> <def>A widow of two cards.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Skate</ent><br/
<hw>Skate</hw> <pr>(sk<amac/t)</pr>, <pos>n.</pos> <ety>[D. <ets>schaats</ets>.  Cf. <er>Scatches</er>.]</ety> <def>A metallic runner with a frame shaped to fit the sole of a shoe, -- made to be fastened under the foot, and used for moving rapidly on ice.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Batavia rushes forth; and as they sweep,<br/
On sounding <qex>skates</qex>, a thousand different ways,<br/
In circling poise, swift as the winds, along,<br/
The then gay land is maddened all to joy.</q> <rj><qau>Thomson.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Roller skate</b></col>. <cd>See under <er>Roller</er>.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Skate</ent><br/
<hw>Skate</hw>, <pos>v. i.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Skated</conjf>; <pos>p. pr. & vb. n.</pos> <conjf>Skating</conjf>.]</vmorph> <def>To move on skates.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Skate</ent><br/
<hw>Skate</hw>, <pos>n.</pos> <ety>[Icel. <ets>skata</ets>; cf. Prov. G. <ets>schatten</ets>, meer<ets>-schatten</ets>, L. <ets>squatus</ets>, <ets>squatina</ets>, and E. <ets>shad</ets>.]</ety> <fld>(Zool.)</fld> <def>Any one of numerous species of large, flat elasmobranch fishes of the genus <gen>Raia</gen>, having a long, slender tail, terminated by a small caudal fin. The pectoral fins, which are large and broad and united to the sides of the body and head, give a somewhat rhombic form to these fishes. The skin is more or less spinose.</def><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ Some of the species are used for food, as the European blue or gray skate (<spn>Raia batis</spn>), which sometimes weighs nearly 200 pounds. The American smooth, or barn-door, skate (<spn>Raia laevis</spn>) is also a large species, often becoming three or four feet across. The common spiny skate (<spn>Raia erinacea</spn>) is much smaller.</note><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Skate's egg</b></col>. <cd>See <er>Sea purse</er>.</cd> -- <col><b>Skate sucker</b></col>, <cd>any marine leech of the genus <gen>Pontobdella</gen>, parasitic on skates.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Skater</ent><br/
<hw>Skat"er</hw> <pr>(?)</pr>, <pos>n.</pos> <sn>1.</sn> <def>One who skates.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Zool.)</fld> <def>Any one of numerous species of hemipterous insects belonging to <gen>Gerris</gen>, <gen>Pyrrhocoris</gen>, <gen>Prostemma</gen>, and allied genera. They have long legs, and run rapidly over the surface of the water, as if skating.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Skatol</ent><br/
<hw>Ska"tol</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Gr. <?/<?/<?/, dung + <ets>-ol</ets>.]</ety> <fld>(Physiol. Chem.)</fld> <def>A constituent of human faeces formed in the small intestines as a product of the putrefaction of albuminous matter. It is also found in reduced indigo.  Chemically it is methyl indol, <chform>C9H9N</chform>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Skayles</ent><br/
<hw>Skayles</hw> <pr>(?)</pr>, <pos>n.</pos> <def>[<root/159.] Skittles.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Skean</ent><br/
<hw>Skean</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Ir <ets>sgian</ets>; akin to Gael. <ets>sgian</ets>, W. <ets>ysgien</ets> a large knife, a scimiter.]</ety> <def>A knife or short dagger, esp. that in use among the Highlanders of Scotland. <altsp>[Variously spelt.]</altsp></def> <ldquo/His <xex>skean</xex>, or pistol.<rdquo/  <rj><au>Spenser.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Skedaddle</ent><br/
<hw>Ske*dad"dle</hw> <pr>(?)</pr>, <pos>v. i.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Skedaddled</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Skedaddling</conjf> <pr>(?)</pr>.]</vmorph> <ety>[Of uncertain etymology.]</ety> <def>To betake one's self to flight, as if in a panic; to flee; to run away.</def> <mark>[Slang, U. S.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Skee</ent><br/
<hw>Skee</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Dan. <ets>ski</ets>; Icel. <ets>sk<imac/<?/</ets> a billet of wood. See <er>Skid</er>.]</ety> <def>A long strip of wood, curved upwards in front, used on the foot for sliding; now usually spelled <asp>ski</asp>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Skeed</ent><br/
<hw>Skeed</hw> <pr>(?)</pr>, <pos>n.</pos> <def>See <er>Skid</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Skeel</ent><br/
<hw>Skeel</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Icel. <ets>skj<?/la</ets> a pail, bucket.]</ety> <def>A shallow wooden vessel for holding milk or cream.</def> <mark>[Prov. Eng. & Scot.]</mark>  <rj><au>Grose.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Skeelgoose</ent><br/
<ent>Skeelduck</ent><br/
<mhw>{ <hw>Skeel"duck`</hw> <pr>(?)</pr>, <hw>Skeel"goose`</hw> <pr>(?)</pr>, }</mhw> <pos>n.</pos> <ety>[See <er>Sheldrake</er>.]</ety> <fld>(Zool.)</fld> <def>The common European sheldrake.</def> <mark>[Prov. Eng.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Skeet</ent><br/
<hw>Skeet</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Etymol. uncertain.]</ety> <fld>(Naut.)</fld> <def>A scoop with a long handle, used to wash the sides of a vessel, and formerly to wet the sails or deck.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Skeg</ent><br/
<hw>Skeg</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Prov. E., also a stump of a branch, a wooden peg; cf. Icel. <ets>sk<?/gr</ets> a wood, Sw. <ets>skog</ets>.  Cf. <er>Shaw</er>.]</ety> <sn>1.</sn> <def>A sort of wild plum.</def> <mark>[Obs.]</mark>  <rj><au>Holland.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <pluf>pl.</pluf> <def>A kind of oats.</def>  <rj><au>Farm. Encyc.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <fld>(Naut.)</fld> <def>The after part of the keel of a vessel, to which the rudder is attached.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Skegger</ent><br/
<hw>Skeg"ger</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Zool.)</fld> <def>The parr.</def>  <rj><au>Walton.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Skein</ent><br/
<hw>Skein</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[OE. <ets>skeyne</ets>, OF. <ets>escaigne</ets>, F. <ets><eacute/cagne</ets>, probably of Celtic origin; cf. Ir. <ets>sgainne</ets>, Gael. <ets>sgeinnidh</ets> thread, small twine; or perhaps the English word is immediately from Celtic.]</ety> <sn>1.</sn> <def>A quantity of yarn, thread, or the like, put up together, after it is taken from the reel, -- usually tied in a sort of knot.</def><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ A <xex>skein</xex> of cotton yarn is formed by eighty turns of the thread round a fifty-four inch reel.</note><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Wagon Making)</fld> <def>A metallic strengthening band or thimble on the wooden arm of an axle.</def>  <rj><au>Knight.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Skein</ent><br/
<hw>Skein</hw>, <pos>n.</pos> <fld>(Zool.)</fld> <def>A flight of wild fowl (wild geese or the like).</def> <mark>[Prov. Eng.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Skeine</ent><br/
<hw>Skeine</hw> <pr>(?)</pr>, <pos>n.</pos> <def>See <er>Skean</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Skelder</ent><br/
<hw>Skel"der</hw> <pr>(?)</pr>, <pos>v. t. & i.</pos> <ety>[Etymol. uncertain.]</ety> <def>To deceive; to cheat; to trick.</def> <mark>[Obs.]</mark>  <rj><au>B. Jonson.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Skelder</ent><br/
<hw>Skel"der</hw>, <pos>n.</pos> <def>A vagrant; a cheat.</def> <mark>[Obs.]</mark>  <rj><au>B. Jonson.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Skieldrake</ent><br/
<ent>Skeldrake</ent><br/
<mhw>{ <hw>Skel"drake`</hw> <pr>(?)</pr>, <it>or</it> <hw>Skiel"drake`</hw> <pr>(?)</pr>, }</mhw> <pos>n.</pos> <fld>(Zool.)</fld> <sd>(a)</sd> <def>The common European sheldrake.</def> <sd>(b)</sd> <def>The oyster catcher.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Skelet</ent><br/
<hw>Skel"et</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A skeleton. See <er>Scelet</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Skeletal</ent><br/
<hw>Skel"e*tal</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Pertaining to the skeleton.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Skeletogenous</ent><br/
<hw>Skel`e*tog"e*nous</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[<ets>Skeleton</ets> + <ets>-genous</ets>.]</ety> <def>Forming or producing parts of the skeleton.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Skeletology</ent><br/
<hw>Skel`e*tol"o*gy</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[<ets>Skeleton</ets> + <ets>-logy</ets>.]</ety> <def>That part of anatomy which treats of the skeleton; also, a treatise on the skeleton.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Skeleton</ent><br/
<hw>Skel"e*ton</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[NL., fr. Gr. <?/<?/<?/<?/ (sc. <?/<?/<?/) a dried body, a mummy, fr. <?/<?/<?/<?/ dried up, parched, <?/<?/<?/<?/ to dry, dry up, parch.]</ety> <sn>1.</sn> <fld>(Anat.)</fld> <sd>(a)</sd> <def>The bony and cartilaginous framework which supports the soft parts of a vertebrate animal.</def> <note>[See <xex>Illust.</xex> of the Human Skeleton, in Appendix.]</note> <sd>(b)</sd> <def>The more or less firm or hardened framework of an invertebrate animal.</def><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ In a wider sense, the skeleton includes the whole connective-tissue framework with the integument and its appendages. See <er>Endoskeleton</er>, and <er>Exoskeleton</er>.</note><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> Hence, figuratively: <sd>(a)</sd> <def>A very thin or lean person.</def> <sd>(b)</sd> <def>The framework of anything; the principal parts that support the rest, but without the appendages.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The great <qex>skeleton</qex> of the world.</q> <rj><qau>Sir M. Hale.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sd>(c)</sd> <def>The heads and outline of a literary production, especially of a sermon.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Skeleton</ent><br/
<hw>Skel"e*ton</hw>, <pos>a.</pos> <def>Consisting of, or resembling, a skeleton; consisting merely of the framework or outlines; having only certain leading features of anything; <as>as, a <ex>skeleton</ex> sermon; a <ex>skeleton</ex> crystal.</as></def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Skeleton bill</b></col>, <cd>a bill or draft made out in blank as to the amount or payee, but signed by the acceptor.</cd> <mark>[Eng.]</mark> -- <col><b>Skeleton key</b></col>, <cd>a key with nearly the whole substance of the web filed away, to adapt it to avoid the wards of a lock; a master key; -- used for opening locks to which it has not been especially fitted.</cd> -- <col><b>Skeleton leaf</b></col>, <cd>a leaf from which the pulpy part has been removed by chemical means, the fibrous part alone remaining.</cd> -- <col><b>Skeleton proof</b></col>, <cd>a proof of a print or engraving, with the inscription outlined in hair strokes only, such proofs being taken before the engraving is finished.</cd> -- <col><b>Skeleton regiment</b></col>, <cd>a regiment which has its complement of officers, but in which there are few enlisted men.</cd> -- <col><b>Skeleton shrimp</b></col> <fld>(Zool.)</fld>, <cd>a small crustacean of the genus <gen>Caprella</gen>. See <xex>Illust.</xex> under <er>Laemodipoda</er>.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Skeletonize</ent><br/
<hw>Skel"e*ton*ize</hw> <pr>(?)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Skeletonized</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Skeletonizing</conjf> <pr>(?)</pr>.]</vmorph> <def>To prepare a skeleton of; also, to reduce, as a leaf, to its skeleton.</def>  <rj><au>Pop. Sci. Monthly.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Skeletonizer</ent><br/
<hw>Skel"e*ton*i`zer</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Zool.)</fld> <def>Any small moth whose larva eats the parenchyma of leaves, leaving the skeleton; <as>as, the apple-leaf <ex>skeletonizer</ex></as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Skellum</ent><br/
<hw>Skel"lum</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Dan. <ets>schelm</ets>, fr. G. <ets>schelm</ets>.]</ety> <def>A scoundrel.</def> <mark>[Obs. or Scot.]</mark>  <rj><au>Pepys. Burns.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Skelly</ent><br/
<hw>Skel"ly</hw> <pr>(?)</pr>, <pos>v. i.</pos> <ety>[Cf. Dan. <ets>skele</ets>, Sw. <ets>skela</ets>.]</ety> <def>To squint.</def> <mark>[Prov. Eng. & Scot.]</mark>  <rj><au>Sir W. Scott.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Skelly</ent><br/
<hw>Skel"ly</hw>, <pos>n.</pos> <def>A squint.</def> <mark>[Prov. Eng. & Scot.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Skelp</ent><br/
<hw>Skelp</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Cf. Prov. E. <ets>skelp</ets> to kick severely, to move rapidly; Gael. <ets>sgealp</ets>, n., a slap with the palm of the hand, v., to strike with the palm of the hand.]</ety> <sn>1.</sn> <def>A blow; a smart stroke.</def> <mark>[Prov. Eng.]</mark>  <rj><au>Brockett.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A squall; also, a heavy fall of rain.</def> <mark>[Scot.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Skelp</ent><br/
<hw>Skelp</hw>, <pos>v. t.</pos> <sn>1.</sn> <def>To strike; to slap.</def> <mark>[Scot.]</mark>  <rj><au>C. Reade.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To form into skelp, as a plate or bar of iron by rolling; also, to bend round (a skelp) in tube making.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Skelp</ent><br/
<hw>Skelp</hw>, <pos>n.</pos> <def>A wrought-iron plate from which a gun barrel or pipe is made by bending and welding the edges together, and drawing the thick tube thus formed.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Skelter</ent><br/
<hw>Skel"ter</hw> <pr>(?)</pr>, <pos>v. i.</pos> <ety>[Cf. <er>Helter-skelter</er>.]</ety> <def>To run off helter-skelter; to hurry; to scurry; -- with <xex>away</xex> or <xex>off</xex>.</def> <mark>[Colloq.]</mark>  <rj><au>A. R. Wallace.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sken</ent><br/
<hw>Sken</hw> <pr>(?)</pr>, <pos>v. i.</pos> <def>To squint.</def> <mark>[Prov. Eng.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Skene</ent><br/
<hw>Skene</hw> <pr>(?)</pr>, <pos>n.</pos> <def>See <er>Skean</er>.</def>  <rj><au>C. Kingsley.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Skep</ent><br/
<hw>Skep</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Icel. <ets>skeppa</ets> a measure, bushel; cf. Gael. <ets>sgeap</ets> a basket, a beehive.]</ety> <sn>1.</sn> <def>A coarse round farm basket.</def> <mark>[Obs. or Prov. Eng. & Scot.]</mark>  <rj><au>Tusser.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A beehive.</def> <mark>[Prov. Eng. & Scot.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Skeptic</ent><br/
<hw>Skep"tic</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Gr. <grk>skeptiko`s</grk> thoughtful, reflective, fr. <grk>ske`ptesqai</grk> to look carefully or about, to view, consider: cf. L. <ets>scepticus</ets>, F. <ets>sceptique</ets>. See <er>Scope</er>.]</ety> <altsp>[Written also <asp>sceptic</asp>.]</altsp> <sn>1.</sn> <def>One who is yet undecided as to what is true; one who is looking or inquiring for what is true; an inquirer after facts or reasons.</def><br/
[<source>1913 Webster</source>]</p>

<p><-- p. 1349 --></p>

<p><sn>2.</sn> <fld>(Metaph.)</fld> <def>A doubter as to whether any fact or truth can be certainly known; a universal doubter; a Pyrrhonist; hence, in modern usage, occasionally, a person who questions whether any truth or fact can be established on philosophical grounds; sometimes, a critical inquirer, in opposition to a dogmatist.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>All this criticism [of Hume] proceeds upon the erroneous hypothesis that he was a dogmatist. He was a <qex>skeptic</qex>; that is, he accepted the principles asserted by the prevailing dogmatism: and only showed that such and such conclusions were, on these principles, inevitable.</q> <rj><qau>Sir W. Hamilton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <fld>(Theol.)</fld> <def>A person who doubts the existence and perfections of God, or the truth of revelation; one who disbelieves the divine origin of the Christian religion.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Suffer not your faith to be shaken by the sophistries of <qex>skeptics</qex>.</q> <rj><qau>S. Clarke.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ This word and its derivatives are often written with <xex>c</xex> instead of <xex>k</xex> in the first syllable, -- <xex>sceptic</xex>, <xex>sceptical</xex>, <xex>scepticism</xex>, etc. Dr. Johnson, struck with the extraordinary irregularity of giving <xex>c</xex> its hard sound before <xex>e</xex>, altered the spelling, and his example has been followed by most of the lexicographers who have succeeded him; yet the prevalent practice among English writers and printers is in favor of the other mode. In the United States this practice is reversed, a large and increasing majority of educated persons preferring the orthography which is most in accordance with etymology and analogy.</note><br/
[<source>1913 Webster</source>]</p>

<p><syn><b>Syn.</b> -- Infidel; unbeliever; doubter. -- See <er>Infidel</er>.</syn><br/
[<source>1913 Webster</source>]</p>

<p><ent>Skeptical</ent><br/
<ent>Skeptic</ent><br/
<mhw>{ <hw>Skep"tic</hw> <pr>(?)</pr>, <hw>Skep"tic*al</hw> <pr>(?)</pr>, }</mhw> <pos>a.</pos> <ety>[Written also <ets>sceptic</ets>, <ets>sceptical</ets>.]</ety> <sn>1.</sn> <def>Of or pertaining to a sceptic or skepticism; characterized by skepticism; hesitating to admit the certainly of doctrines or principles; doubting of everything.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Theol.)</fld> <def>Doubting or denying the truth of revelation, or the sacred Scriptures.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The <qex>skeptical</qex> system subverts the whole foundation of morals.</q> <rj><qau>R. Hall.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p>-- <wordforms><wf>Skep"tac*al*ly</wf>, <pos>adv.</pos> -- <wf>Skep"tic*al*ness</wf>, <pos>n.</pos></wordforms><br/
[<source>1913 Webster</source>]</p>

<p><ent>Skepticism</ent><br/
<hw>Skep"ti*cism</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Cf. F. <ets>scepticisme</ets>.]</ety> <altsp>[Written also <asp>scepticism</asp>.]</altsp> <sn>1.</sn> <def>An undecided, inquiring state of mind; doubt; uncertainty.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>That momentary amazement, and irresolution, and confusion, which is the result of <qex>skepticism</qex>.</q> <rj><qau>Hune.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Metaph.)</fld> <def>The doctrine that no fact or principle can be certainly known; the tenet that all knowledge is uncertain; Pyrrohonism; universal doubt; the position that no fact or truth, however worthy of confidence, can be established on philosophical grounds; critical investigation or inquiry, as opposed to the positive assumption or assertion of certain principles.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <fld>(Theol.)</fld> <def>A doubting of the truth of revelation, or a denial of the divine origin of the Christian religion, or of the being, perfections, or truth of God.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Let no . . . secret <qex>skepticism</qex> lead any one to doubt whether this blessed prospect will be realized.</q> <rj><qau>S. Miller.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Skepticize</ent><br/
<hw>Skep"ti*cize</hw> <pr>(?)</pr>, <pos>v. i.</pos> <def>To doubt; to pretend to doubt of everything.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>To <qex>skepticize</qex>, where no one else will . . . hesitate.</q> <rj><qau>Shaftesbury.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Skerry</ent><br/
<hw>Sker"ry</hw> <pr>(?)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Skerries</plw> <pr>(#)</pr>.</plu> <ety>[Of Scand. origin; cf. Icel. <ets>sker</ets>, Sw. <ets>sk<aum/r</ets>, Dan. <ets>ski<?/r</ets>.  Cf. <er>Scar</er> a bank.]</ety> <def>A rocky isle; an insulated rock.</def> <mark>[Scot.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sketch</ent><br/
<hw>Sketch</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[D. <ets>schets</ets>, fr. It. <ets>schizzo</ets> a sketch, a splash (whence also F. <ets>esquisse</ets>; cf. <er>Esquisse</er>.); cf. It. <ets>schizzare</ets> to splash, to sketch.]</ety> <def>An outline or general delineation of anything; a first rough or incomplete draught or plan of any design; especially, in the fine arts, such a representation of an object or scene as serves the artist's purpose by recording its chief features; also, a preliminary study for an original work.</def><br/
[<source>1913 Webster</source>]</p>

<p><syn><b>Syn.</b> -- Outline; delineation; draught; plan; design.</syn>  -- <usage><er>Sketch</er>, <er>Outline</er>, <er>Delineation</er>. An <xex>outline</xex> gives only the bounding lines of some scene or picture. A <xex>sketch</xex> fills up the <xex>outline</xex> in part, giving broad touches, by which an imperfect idea may be conveyed. A <xex>delineation</xex> goes further, carrying out the more striking features of the picture, and going so much into detail as to furnish a clear conception of the whole. Figuratively, we may speak of the <xex>outlines</xex> of a plan, of a work, of a project, etc., which serve as a basis on which the subordinate parts are formed, or of <xex>sketches</xex> of countries, characters, manners, etc., which give us a general idea of the things described.</usage>  <rj><au>Crabb.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sketch</ent><br/
<hw>Sketch</hw>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Sketched</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Sketching</conjf>.]</vmorph> <ety>[Cf D. <ets>schetsen</ets>, It. <ets>schizzare</ets>. See <er>Sketch</er>, <pos>n.</pos>]</ety> <sn>1.</sn> <def>To draw the outline or chief features of; to make a rought of.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To plan or describe by giving the principal points or ideas of.</def><br/
[<source>1913 Webster</source>]</p>

<p><syn><b>Syn.</b> -- To delineate; design; draught; depict.</syn><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sketch</ent><br/
<hw>Sketch</hw>, <pos>v. i.</pos> <def>To make sketches, as of landscapes.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sketchbook</ent><br/
<hw>Sketch"book`</hw>, <pos>n.</pos> <def>A book of sketches or for sketches.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sketcher</ent><br/
<hw>Sketch"er</hw> <pr>(?)</pr>, <pos>n.</pos> <def>One who sketches.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sketchily</ent><br/
<hw>Sketch"i*ly</hw> <pr>(?)</pr>, <pos>adv.</pos> <def>In a sketchy or incomplete manner.</def> <ldquo/<xex>Sketchily</xex> descriptive.<rdquo/  <rj><au>Bartlett.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sketchiness</ent><br/
<hw>Sketch"i*ness</hw>, <pos>n.</pos> <def>The quality or state of being sketchy; lack of finish; incompleteness.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sketchy</ent><br/
<hw>Sketch"y</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Containing only an outline or rough form; being in the manner of a sketch; incomplete.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The execution is <qex>sketchy</qex> throughout; the head, in particular, is left in the rough.</q> <rj><qau>J. S. Harford.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Skew</ent><br/
<hw>Skew</hw> <pr>(?)</pr>, <pos>adv.</pos> <ety>[Cf. D. <ets>scheef</ets>. Dan. <ets>ski<?/v</ets>, Sw. <ets>skef</ets>, Icel. <ets>skeifr</ets>, G. <ets>schief</ets>, also E. <ets>shy</ets>, a. & v. i.]</ety> <def>Awry; obliquely; askew.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Skew</ent><br/
<hw>Skew</hw>, <pos>a.</pos> <def>Turned or twisted to one side; situated obliquely; skewed; -- chiefly used in technical phrases.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Skew arch</b></col>, <cd>an oblique arch. See under <er>Oblique</er>.</cd> -- <col><b>Skew back</b></col>. <fld>(Civil Engin.)</fld> <sd>(a)</sd> <cd>The course of masonry, the stone, or the iron plate, having an inclined face, which forms the abutment for the voussoirs of a segmental arch</cd>. <sd>(b)</sd> <cd>A plate, cap, or shoe, having an inclined face to receive the nut of a diagonal brace, rod, or the end of an inclined strut, in a truss or frame.</cd> -- <col><b>Skew bridge</b></col>. <cd>See under <er>Bridge</er>, <pos>n.</pos></cd> -- <col><b>Skew curve</b></col> <fld>(Geom.)</fld>, <cd>a curve of double curvature, or a twisted curve. See <cref>Plane curve</cref>, under <er>Curve</er>.</cd> -- <mcol><col><b>Skew gearing</b></col>, <it>or</it> <col><b>Skew bevel gearing</b></col></mcol> <fld>(Mach.)</fld>, <cd>toothed gearing, generally resembling bevel gearing, for connecting two shafts that are neither parallel nor intersecting, and in which the teeth slant across the faces of the gears.</cd> -- <col><b>Skew surface</b></col> <fld>(Geom.)</fld>, <cd>a ruled surface such that in general two successive generating straight lines do not intersect; a warped surface; as, the helicoid is a <xex>skew surface</xex>.</cd> -- <col><b>Skew symmetrical determinant</b></col> <fld>(Alg.)</fld>, <cd>a determinant in which the elements in each column of the matrix are equal to the elements of the corresponding row of the matrix with the signs changed, as in (1), below.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><illust>
(1)  <matrix><row>0 2 -3</row><row>-2 0 5</row><row>3 -5 0</row></matrix>   (2)  <matrix><row>4 -1 7</row><row>1 8 -2</row><row>-7 2 1</row></matrix>
</illust><br/
[<source>1913 Webster</source>]</p>

<p>   <note>This requires that the numbers in the diagonal from the upper left to lower right corner be zeros. A like determinant in which the numbers in the diagonal are not zeros is a <xex>skew determinant</xex>, as in (2), above.</note><br/
[<source>1913 Webster</source>]</p>

<p><ent>Skew</ent><br/
<hw>Skew</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Arch.)</fld> <def>A stone at the foot of the slope of a gable, the offset of a buttress, or the like, cut with a sloping surface and with a check to receive the coping stones and retain them in place.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Skew</ent><br/
<hw>Skew</hw>, <pos>v. i.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Skewed</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Skewing</conjf>.]</vmorph> <sn>1.</sn> <def>To walk obliquely; to go sidling; to lie or move obliquely.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Child, you must walk straight, without <qex>skewing</qex>.</q> <rj><qau>L'Estrange.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To start aside; to shy, as a horse.</def> <mark>[Prov. Eng.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>To look obliquely; to squint; hence, to look slightingly or suspiciously.</def>  <rj><au>Beau. & Fl.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Skew</ent><br/
<hw>Skew</hw>, <pos>v. t.</pos> <ety>[See <er>Skew</er>, <pos>adv.</pos>]</ety> <sn>1.</sn> <def>To shape or form in an oblique way; to cause to take an oblique position.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To throw or hurl obliquely.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Skewbald</ent><br/
<hw>Skew"bald`</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Marked with spots and patches of white and some color other than black; -- usually distinguished from <xex>piebald</xex>, in which the colors are properly white and black. Said of horses.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Skewer</ent><br/
<hw>Skew"er</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Probably of Scand, origin; cf. Sw. & Dan. <ets>skifer</ets> a slate.  Cf. <er>Shuver</er> a fragment.]</ety> <def>A pin of wood or metal for fastening meat to a spit, or for keeping it in form while roasting.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Meat well stuck with <qex>skewers</qex> to make it look round.</q> <rj><qau>Swift.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Skewer</ent><br/
<hw>Skew"er</hw>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Skewered</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Skewering</conjf>.]</vmorph> <def>To fasten with skewers.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Ski</ent><br/
<hw>Ski</hw> <pr>(sk<emac/)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>skis</plw> <pr>(sk<emac/z)</pr></plu>. <ety>[Dan. <ets>ski</ets>; Icel. <ets>sk<imac/<edh/</ets> a billet of wood. See <er>Skid</er>.]</ety> <def>A long, flat, narrow runner made of wood, plastic or metal, curved upwards in front, having a fitting allowing it to be attached to the foot, and used for gliding or sliding over snow.  Commonly used in the plural, to designate the pair.</def> <altsp>[Also spelled <asp>skee</asp>.]</altsp><br/
[<source>Webster 1913 Suppl.</source> <source>+PJC</source>]</p>

<p><ent>Skiagraphy</ent><br/
<ent>Skiagraph</ent><br/
<mhw><hw>Ski"a*graph</hw> <pr>(?)</pr>, <pos>n.</pos> <hw>Ski*ag"ra*phy</hw> <pr>(<?/)</pr>, <pos>n.</pos>, etc.</mhw> <def>See <er>Sciagraph</er>, <er>Sciagraphy</er>, etc.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Sciascope</ent><br/
<ent>Skiascope</ent><br/
<mhw>{ <hw>Ski"a*scope</hw> <pr>(?)</pr>, <hw>Sci"a*scope</hw> <pr>(?)</pr>, }</mhw> <pos>n.</pos> <ety>[Gr. <?/ a shadow + <ets>-scope</ets>.]</ety> <fld>(Med.)</fld> <def>A device for determining the refractive state of the eye by observing the movements of the retinal lights and shadows.</def> -- <wordforms><wf>Ski*as"co*py</wf> <pr>(#)</pr>, <wf>Ski*as"co*py</wf> <pr>(#)</pr>, <pos>n.</pos></wordforms><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Skid</ent><br/
<hw>Skid</hw> <pr>(sk<icr/d)</pr>, <pos>n.</pos> <ety>[Icel. <ets>sk<imac/<edh/</ets> a billet of wood. See <er>Shide</er>.]</ety> <altsp>[Written also <asp>skeed</asp>.]</altsp> <sn>1.</sn> <def>A shoe or clog, as of iron, attached to a chain, and placed under the wheel of a wagon to prevent its turning when descending a steep hill; a drag; a skidpan; also, by extension, a hook attached to a chain, and used for the same purpose.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A piece of timber used as a support, or to receive pressure.</def> Specifically: <sd>(a)</sd> <pluf>pl.</pluf> <fld>(Naut.)</fld> <def>Large fenders hung over a vessel's side to protect it in handling a cargo.</def> <au>Totten.</au> <sd>(b)</sd> <def>One of a pair of timbers or bars, usually arranged so as to form an inclined plane, as form a wagon to a door, along which anything is moved by sliding or rolling.</def> <sd>(c)</sd> <def>One of a pair of horizontal rails or timbers for supporting anything, as a boat, a barrel, etc.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <fld>(Aeronautics)</fld> <def>A runner (one or two) under some flying machines, used for landing.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><sn>4.</sn> <def>A low movable platform for supporting heavy items to be transported, typically of two layers, and having a space between the layers into which the fork of a fork lift can be inserted; it is used to conveniently transport heavy objects by means of a fork lift; -- a <ex>skid</ex> without wheels is the same as a <er>pallet</er>.</def><br/
[<source>PJC</source>]</p>

<p><sn>5.</sn> <pluf>pl.</pluf> <def>Declining fortunes; a movement toward defeat or downfall; -- used mostly in the phrase <col><b>on the skids</b></col> and <col><b>hit the skids</b></col>.</def><br/
[<source>PJC</source>]</p>

<p><sn>6.</sn>  <ety>[From the <pos>v.</pos>]</ety> <def>Act of skidding; -- called also <altname>side slip</altname>.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Skid</ent><br/
<hw>Skid</hw>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Skidded</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Skidding</conjf>.]</vmorph> <sn>1.</sn> <def>To protect or support with a skid or skids; also, to cause to move on skids.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To check with a skid, as wagon wheels.</def>  <rj><au>Dickens.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <fld>(Forestry)</fld> <def>To haul (logs) to a skid and load on a skidway.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Skid</ent><br/
<hw>Skid</hw>, <pos>v. i.</pos> <sn>1.</sn> <def>To slide without rotating; -- said of a wheel held from turning while the vehicle moves onward.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><sn>2.</sn>  <def>To fail to grip the roadway; specif., to slip sideways on the road; to side-slip; -- said esp. of a cycle or automobile.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Skiddaw</ent><br/
<hw>Skid"daw`</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Zool.)</fld> <def>The black guillemot.</def> <mark>[Prov. Eng.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Skidder</ent><br/
<hw>Skid"der</hw> <pr>(?)</pr>, <pos>n.</pos> <def>One that skids; one that uses a skid;</def> <specif>specif.:</specif> <fld>(Logging)</fld> <sd>(a)</sd> <def>One that skids logs.</def> <sd>(b)</sd> <def>An engine for hauling the cable used in skidding logs.</def> <sd>(c)</sd> <def>The foreman of a construction gang making a skid road.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Skidpan</ent><br/
<hw>Skid"pan`</hw> <pr>(?)</pr>, <pos>n.</pos> <def>See <er>Skid</er>, <pos>n.</pos>, 1.</def> <mark>[Eng.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Skid road</ent><br/
<hw>Skid road</hw>. <fld>(Logging)</fld> <sd>(a)</sd> <def>A road along which logs are dragged to the skidway or landing; -- called also <altname>travois road</altname> <it>or</it> <altname>travoy road</altname>.</def> <sd>(b)</sd> <def>A road having partly sunken transverse logs (called <xex>skids</xex>) at intervals of about five feet.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Skied</ent><br/
<hw>Skied</hw> <pr>(sk<imac/d)</pr>, <def><pos>imp. & p. p.</pos> of <er>Sky</er>, <pos>v. t.</pos></def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Skied</ent><br/
<hw>Skied</hw> <pr>(sk<emac/d)</pr>, <def><pos>imp. & p. p.</pos> of <er>Ski</er>, <pos>v. i.</pos></def><br/
[<source>PJC</source>]</p>

<p><ent>Skiey</ent><br/
<hw>Ski"ey</hw> <pr>(?)</pr>, <pos>a.</pos> <def>See <er>Skyey</er>.</def>  <rj><au>Shelley.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Skiff</ent><br/
<hw>Skiff</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F. <ets>esquif</ets>, fr. OHG. <ets>skif</ets>, G. <ets>schiff</ets>. See <er>Ship</er>.]</ety> <def>A small, light boat.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The pilot of some small night-foundered <qex>skiff</qex>.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Skiff caterpillar</b></col> <fld>(Zool.)</fld>, <cd>the larva of a moth (<spn>Limacodes scapha</spn>); -- so called from its peculiar shape.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Skiff</ent><br/
<hw>Skiff</hw>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Skiffed</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Skiffing</conjf>.]</vmorph> <def>To navigate in a skiff.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Skiffling</ent><br/
<hw>Skif"fling</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Quarrying)</fld> <def>Rough dressing by knocking off knobs or projections; knobbing.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Skilder</ent><br/
<hw>Skil"der</hw> <pr>(?)</pr>, <pos>v. i.</pos> <def>To beg; to pilfer; to skelder.</def> <mark>[Prov. Eng. & Scot.]</mark>  <rj><au>Sir W. Scott.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Skilful</ent><br/
<hw>Skil"ful</hw> <pr>(?)</pr>, <pos>a.</pos> <def>See <er>Skilful</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Skill</ent><br/
<hw>Skill</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Icel. <ets>skil</ets> a distinction, discernment; akin to <ets>skilja</ets> to separate, divide, distinguish, Sw. <ets>skilja</ets>,. <ets>skille</ets> to separate, <ets>skiel</ets> reason, right, justice, Sw. <ets>sk<aum/l</ets> reason, Lith. <ets>skelli</ets> to cleave.  Cf. <er>Shell</er>, <er>Shoal</er>, a multitude.]</ety> <sn>1.</sn> <def>Discrimination; judgment; propriety; reason; cause.</def> <mark>[Obs.]</mark> <au>Shak.</au> <ldquo/As it was <xex>skill</xex> and right.<rdquo/ <au>Chaucer.</au><br/
[<source>1913 Webster</source>]</p>

<p><q>For great <qex>skill</qex> is, he prove that he wrought.</q> [For with good reason he should test what he created.]
<rj><qau>Chaucer.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Knowledge; understanding.</def> <mark>[Obsoles.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>That by his fellowship he color might<br/
Both his estate and love from <qex>skill</qex> of any wight.</q> <rj><qau>Spenser.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Nor want we <qex>skill</qex> or art.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>The familiar knowledge of any art or science, united with readiness and dexterity in execution or performance, or in the application of the art or science to practical purposes; power to discern and execute; ability to perceive and perform; expertness; aptitude; <as>as, the <ex>skill</ex> of a mathematician, physician, surgeon, mechanic, etc.</as></def><br/
[<source>1913 Webster</source>]</p>

<p><q>Phocion, . . . by his great wisdom and <qex>skill</qex> at negotiations, diverted Alexander from the conquest of Athens.</q> <rj><qau>Swift.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Where patience her sweet <qex>skill</qex> imparts.</q> <rj><qau>Keble.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>Display of art; exercise of ability; contrivance; address.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>Richard . . . by a thousand princely <qex>skills</qex>, gathering so much corn as if he meant not to return.</q> <rj><qau>Fuller.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <def>Any particular art.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>Learned in one <qex>skill</qex>, and in another kind of learning unskillful.</q> <rj><qau>Hooker.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><syn><b>Syn.</b> -- Dexterity; adroitness; expertness; art; aptitude; ability.</syn>  -- <usage><er>Skill</er>, <er>Dexterity</er>, <er>Adroitness</er>. <xex>Skill</xex> is more intelligent, denoting familiar knowledge united to readiness of performance. <xex>Dexterity</xex>, when applied to the body, is more mechanical, and refers to habitual ease of execution. <xex>Adroitness</xex> involves the same image with <xex>dexterity</xex>, and differs from it as implaying a general facility of movement (especially in avoidance of danger or in escaping from a difficalty). The same distinctions apply to the figurative sense of the words. A man is <xex>skillful</xex> in any employment when he understands both its theory and its practice. He is <xex>dexterous</xex> when he maneuvers with great lightness. He is <xex>adroit</xex> in the use od quick, sudden, and well-directed movements of the body or the mind, so as to effect the object he has in view.</usage><br/
[<source>1913 Webster</source>]</p>

<p><ent>Skill</ent><br/
<hw>Skill</hw> <pr>(?)</pr>, <pos>v. t.</pos> <def>To know; to understand.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>To <qex>skill</qex> the arts of expressing our mind.</q> <rj><qau>Barrow.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Skill</ent><br/
<hw>Skill</hw>, <pos>v. i.</pos> <sn>1.</sn> <def>To be knowing; to have understanding; to be dexterous in performance.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>I can not <qex>skill</qex> of these thy ways.</q> <rj><qau>Herbert.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To make a difference; to signify; to matter; -- used impersonally.</def>  <rj><au>Spenser.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>What <qex>skills</qex> it, if a bag of stones or gold<br/
About thy neck do drown thee?</q> <rj><qau>Herbert.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>It <qex>skills</qex> not talking of it.</q> <rj><qau>Sir W. Scott.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Skilled</ent><br/
<hw>Skilled</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Having familiar knowledge united with readiness and dexterity in its application; familiarly acquainted with; expert; skillful; -- often followed by <xex>in</xex>; <as>as, a person <ex>skilled</ex> in drawing or geometry</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Skillet</ent><br/
<hw>Skil"let</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[OF. <ets>escuelette</ets>, dim. of <ets>escuelle</ets> a porringer, F. <ets>ecuelle</ets>, fr. L. <ets>scutella</ets>, dim. of <ets>scutra</ets>, <ets>scuta</ets>, a dish.  Cf. <er>Scuttle</er> a basket.]</ety> <def>A small vessel of iron, copper, or other metal, with a handle, used for culinary purpose, as for stewing meat.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Skillful</ent><br/
<hw>Skill"ful</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[Written also <ets>skilful</ets>.]</ety> <sn>1.</sn> <def>Discerning; reasonable; judicious; cunning.</def> <mark>[Obs.]</mark> <ldquo/Of <xex>skillful</xex> judgment.<rdquo/  <rj><au>Chaucer.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Possessed of, or displaying, skill; knowing and ready; expert; well-versed; able in management; <as>as, a <ex>skillful</ex> mechanic</as>; -- often followed by <xex>at</xex>, <xex>in</xex>, or <xex>of</xex>; <as>as, <ex>skillful</ex> at the organ; <ex>skillful</ex> in drawing</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>And they shall call the husbandman to mourning, and such as are <qex>skillful</qex> of lamentations to wailing.</q> <rj><qau>Amos v. 16.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><syn><b>Syn.</b> -- Expert; skilled; dexterous; adept; masterly; adroit; clever; cunning.</syn><br/
[<source>1913 Webster</source>]</p>

<p>-- <wordforms><wf>Skill"ful*ly</wf>, <pos>adv.</pos> -- <wf>Skill"ful*ness</wf>, <pos>n.</pos></wordforms><br/
[<source>1913 Webster</source>]</p>

<p><ent>Skilligalee</ent><br/
<hw>Skil`li*ga*lee"</hw> <pr>(sk<icr/l`l<icr/*g<adot/*l<emac/")</pr>, <pos>n.</pos> <def>A kind of thin, weak broth or oatmeal porridge, served out to prisoners and paupers in England; also, a drink made of oatmeal, sugar, and water, sometimes used in the English navy or army.</def> <altsp>[Written also <asp>skilligolee</asp>, <asp>skillygalee</asp>, etc.]</altsp><br/
[<source>1913 Webster</source>]</p>

<p><ent>Skilling</ent><br/
<hw>Skil"ling</hw> <pr>(sk<icr/l"l<icr/ng)</pr>, <pos>n.</pos> <ety>[Cf. <er>Sheeling</er>.]</ety> <def>A bay of a barn; also, a slight addition to a cottage.</def> <mark>[Prov. Eng.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Skilling</ent><br/
<hw>Skil"ling</hw>, <pos>n.</pos> <ety>[Sw. & Dan. See <er>Shilling</er>.]</ety> <def>A money of account in Sweden, Norway, Denmark, and North Germany, and also a coin.  It had various values, from three fourths of a cent in Norway to more than two cents in L<uum/beck.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Skill-less</ent><br/
<hw>Skill"-less</hw>, <pos>a.</pos> <def>Lacking skill.</def>  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Skilts</ent><br/
<hw>Skilts</hw> <pr>(sk<icr/l)</pr>, <pos>n. pl.</pos> <def>A kind of large, coarse, short trousers formerly worn.</def> <mark>[Local, U. S.]</mark>  <rj><au>Bartlett.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Skilty</ent><br/
<hw>Skil"ty</hw> <pr>(?)</pr>, <pos>n.</pos> <def>The water rail.</def> <mark>[Prov. Eng.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Skim</ent><br/
<hw>Skim</hw> <pr>(sk<icr/m)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Skimmed</conjf> <pr>(sk<icr/md)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Skimming</conjf>.]</vmorph> <ety>[Cf. Sw. <ets>skymma</ets> to darken. <root/158. See <er>Scum</er>.]</ety> <sn>1.</sn> <def>To clear (a liquid) from scum or substance floating or lying thereon, by means of a utensil that passes just beneath the surface; <as>as, to <ex>skim</ex> milk; to <ex>skim</ex> broth.</as></def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To take off by skimming; <as>as, to <ex>skim</ex> cream</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>To pass near the surface of; to brush the surface of; to glide swiftly along the surface of.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Homer describes Mercury as flinging himself from the top of Olympus, and <qex>skimming</qex> the surface of the ocean.</q> <rj><qau>Hazlitt.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>Fig.: To read or examine superficially and rapidly, in order to cull the principal facts or thoughts; <as>as, to <ex>skim</ex> a book or a newspaper</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Skim</ent><br/
<hw>Skim</hw>, <pos>v. i.</pos> <sn>1.</sn> <def>To pass lightly; to glide along in an even, smooth course; to glide along near the surface.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Not so when swift Camilla scours the plain,<br/
Flies o'er the unbending corn, and <qex>skims</qex> along the main.</q> <rj><qau>Pope.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To hasten along with superficial attention.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>They <qex>skim</qex> over a science in a very superficial survey.</q> <rj><qau>I. Watts.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>To put on the finishing coat of plaster.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Skim</ent><br/
<hw>Skim</hw>, <pos>a.</pos> <def>Contraction of <er>Skimming</er> and <er>Skimmed</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Skim coat</b></col>, <cd>the final or finishing coat of plaster.</cd> -- <col><b>Skim colter</b></col>, <cd>a colter for paring off the surface of land.</cd> -- <col><b>Skim milk</b></col>, <cd>skimmed milk; milk from which the cream has been taken.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Srim</ent><br/
<hw>Srim</hw>, <pos>n.</pos> <def>Scum; refuse.</def>  <rj><au>Bryskett.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Skimback</ent><br/
<hw>Skim"back`</hw> <pr>(sk<icr/m"b<acr/k`)</pr>, <pos>n.</pos> <fld>(Zool.)</fld> <def>The quillback.</def> <mark>[Local, U.S.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Skimble-scamble</ent><br/
<hw>Skim"ble-scam`ble</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[A reduplication of <ets>scamble</ets>.]</ety> <def>Rambling; disorderly; unconnected.</def> <mark>[Colloq.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>Such a deal of <qex>skimble-scamble</qex> stuff.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Skimitry</ent><br/
<hw>Skim"i*try</hw> <pr>(?)</pr>, <pos>n.</pos> <def>See <er>Skimmington</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Skimmer</ent><br/
<hw>Skim"mer</hw> <pr>(?)</pr>, <pos>n.</pos> <sn>1.</sn> <def>One who, or that which, skims; esp., a utensil with which liquids are skimmed.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Zool.)</fld> <def>Any species of longwinged marine birds of the genus <gen>Rhynchops</gen>, allied to the terns, but having the lower mandible compressed and much longer than the upper one.  These birds fly rapidly along the surface of the water, with the lower mandible immersed, thus skimming out small fishes. The American species (<spn>Rhynchops nigra</spn>) is common on the southern coasts of the United States. Called also <altname>scissorbill</altname>, and <altname>shearbill</altname>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <fld>(Zool.)</fld> <def>Any one of several large bivalve shells, sometimes used for skimming milk, as the sea clams, and large scallops.</def><br/
[<source>1913 Webster</source>]</p>

<p><-- p. 1350 --></p>

<p><ent>Skimmerton</ent><br/
<hw>Skim"mer*ton</hw> <pr>(?)</pr>, <pos>n.</pos> <def>See <er>Skimmington</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Skimming</ent><br/
<hw>Skim"ming</hw> <pr>(?)</pr>, <pos>n.</pos> <sn>1.</sn> <def>The act of one who skims.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>That which is skimmed from the surface of a liquid; -- chiefly used in the plural; <as>as, the <ex>skimmings</ex> of broth</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Skimmingly</ent><br/
<hw>Skim"ming*ly</hw>, <pos>adv.</pos> <def>In a skimming manner.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Skimmington</ent><br/
<hw>Skim"ming*ton</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Etymol. uncertain. Perhaps the name of some notorius scold.]</ety> <def>A word employed in the phrase, <xex>To ride Skimmington</xex>; that is to ride on a horse with a woman, but behind her, facing backward, carrying a distaff, and accompanied by a procession of jeering neighbors making mock music; a cavalcade in ridicule of a henpecked man. The custom was in vogue in parts of England.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Skimp</ent><br/
<hw>Skimp</hw> <pr>(?)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Skimped</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Skimping</conjf>.]</vmorph> <ety>[Cf. <er>Skinch</er>, <er>Scamp</er>, <pos>v. t.</pos>]</ety> <sn>1.</sn> <def>To slight; to do carelessly; to scamp.</def> <mark>[Prov. Eng. & Colloq. U.S.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To make insufficient allowance for; to scant; to scrimp.</def> <mark>[Prov. Eng. & Colloq. U. S.]</mark>
<-- used with "on"; to skimp on clothing so as to have enough food. --><br/
[<source>1913 Webster</source>]</p>

<p><ent>Skimp</ent><br/
<hw>Skimp</hw>, <pos>v. i.</pos> <def>To save; to be parsimonious or niggardly.</def> <mark>[Prov. Eng. & Colloq. U.S.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Skimp</ent><br/
<hw>Skimp</hw>, <pos>a.</pos> <def>Scanty.</def> <mark>[Prov. Eng. & Colloq. U.S.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Skin</ent><br/
<hw>Skin</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Icel. <ets>skinn</ets>; akin to Sw. <ets>skinn</ets>, Dan. <ets>skind</ets>, AS. <ets>scinn</ets>, G. <ets>schined</ets> to skin.]</ety> <sn>1.</sn> <fld>(Anat.)</fld> <def>The external membranous integument of an animal.</def><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ In man, and the vertebrates generally, the skin consist of two layers, an outer nonsensitive and nonvascular <xex>epidermis</xex>, <xex>cuticle</xex>, or <xex>skarfskin</xex>, composed of cells which are constantly growing and multiplying in the deeper, and being thrown off in the superficial, layers; and an inner sensitive, and vascular <xex>dermis</xex>, <xex>cutis</xex>, <xex>corium</xex>, or <xex>true skin</xex>, composed mostly of connective tissue.</note><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>The hide of an animal, separated from the body, whether green, dry, or tanned; especially, that of a small animal, as a calf, sheep, or goat.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>A vessel made of skin, used for holding liquids. See <er>Bottle</er>, 1.</def> <ldquo/<xex>Skins</xex> of wine.<rdquo/  <rj><au>Tennyson.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>The bark or husk of a plant or fruit; the exterior coat of fruits and plants.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <fld>(Naut.)</fld> <sd>(a)</sd> <def>That part of a sail, when furled, which remains on the outside and covers the whole.</def> <au>Totten.</au> <sd>(b)</sd> <def>The covering, as of planking or iron plates, outside the framing, forming the sides and bottom of a vessel; the shell; also, a lining inside the framing.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><mcol><col><b>Skin friction</b></col>, <col><b>Skin resistance</b></col></mcol> <fld>(Naut.)</fld>, <cd>the friction, or resistance, caused by the tendency of water to adhere to the immersed surface (skin) of a vessel.</cd> -- <col><b>Skin graft</b></col> <fld>(Surg.)</fld>, <cd>a small portion of skin used in the process of grafting. See <er>Graft</er>, <pos>v. t.</pos>, 2.</cd> -- <col><b>Skin moth</b></col> <fld>(Zool.)</fld>, <cd>any insect which destroys the prepared skins of animals, especially the larva of Dermestes and Anthrenus.</cd> -- <col><b>Skin of the teeth</b></col>, <cd>nothing, or next to nothing; the least possible hold or advantage.</cd> <au>Job xix. 20.</au> -- <col><b>Skin wool</b></col>, <cd>wool taken from dead sheep.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Skin</ent><br/
<hw>Skin</hw>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Skinned</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Skinning</conjf>.]</vmorph> <sn>1.</sn> <def>To strip off the skin or hide of; to flay; to peel; <as>as, to <ex>skin</ex> an animal</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To cover with skin, or as with skin; hence, to cover superficially.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>It will but <qex>skin</qex> and film the ulcerous place.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>To strip of money or property; to cheat.</def> <mark>[Slang]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Skin</ent><br/
<hw>Skin</hw>, <pos>v. i.</pos> <sn>1.</sn> <def>To become covered with skin; <as>as, a wound <ex>skins</ex> over</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To produce, in recitation, examination, etc., the work of another for one's own, or to use in such exercise cribs, memeoranda, etc., which are prohibited.</def> <mark>[College Cant, U.S.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Skinbound</ent><br/
<hw>Skin"bound`</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Having the skin adhering closely and rigidly to the flesh; hidebound.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Skinbound disease</b></col>. <fld>(Med.)</fld> <cd>See <xex>Sclerema neonatorum</xex>, under <er>Sclerema</er>.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Skinch</ent><br/
<hw>Skinch</hw> <pr>(?)</pr>, <pos>v. t. & i.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Skinched</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Skinching</conjf>.]</vmorph> <ety>[Cf. <er>Scant</er>.]</ety> <def>To give scant measure; to squeeze or pinch in order to effect a saving.</def> <mark>[Prev. Eng. & Colloq. U.S.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Skin-deep</ent><br/
<hw>Skin"-deep`</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Not deeper than the skin; hence, superficial.</def>  <rj><au>Lowell.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Skinflint</ent><br/
<hw>Skin"flint`</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[<ets>Skin</ets> + <ets>flint</ets>.]</ety> <def>A penurious person; a miser; a niggard.</def>  <rj><au>Sir W. Scott.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Skinful</ent><br/
<hw>Skin"ful</hw> <pr>(?)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Skinfuls</plw> <pr>(<?/)</pr>.</plu> <def>As much as a skin can hold.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Skink</ent><br/
<hw>Skink</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>scincus</ets>, Gr. <?/<?/<?/<?/.]</ety> <altsp>[Written also <asp>scink</asp>.]</altsp> <fld>(Zool.)</fld> <def>Any one of numerous species of regularly scaled harmless lizards of the family <fam>Scincidae</fam>, common in the warmer parts of all the continents.</def><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ The officinal skink (<spn>Scincus officinalis</spn>) inhabits the sandy plains of South Africa. It was believed by the ancients to be a specific for various diseases. A common slender species (<spn>Seps tridactylus</spn>) of Southern Europe was formerly believed to produce fatal diseases in cattle by mere contact. The American skinks include numerous species of the genus <gen>Eumeces</gen>, as the blue-tailed skink (<spn>Eumeces fasciatus</spn>) of the Eastern United States. The ground skink, or ground lizard (<spn>Oligosoma laterale</spn>) inhabits the Southern United States.</note><br/
[<source>1913 Webster</source>]</p>

<p><ent>Skink</ent><br/
<hw>Skink</hw>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Skinked</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Skinking</conjf>.]</vmorph> <ety>[Icel. <ets>skenja</ets>; akin to Sw. <ets>sk<aum/ka</ets>, Dan. <ets>skienke</ets>, AS.  <ets>scencan</ets>, D. & G. <ets>schenken</ets>. As. <ets>scencan</ets> is usually derived from <ets>sceonc</ets>, <ets>sceanc</ets>, shank, a hollow bone being supposed to have been used to draw off liquor from a cask. <root/161. See <er>Shank</er>, and cf. <er>Nunchion</er>.]</ety> <def>To draw or serve, as drink.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>Bacchus the wine them <qex>skinketh</qex> all about.</q> <rj><qau>Chaucer.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Such wine as Ganymede doth <qex>skink</qex> to Jove.</q> <rj><qau>Shirley.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Skink</ent><br/
<hw>Skink</hw>, <pos>v. i.</pos> <def>To serve or draw liquor.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Skink</ent><br/
<hw>Skink</hw>, <pos>n.</pos> <def>Drink; also, pottage.</def> <mark>[Obs.]</mark>  <rj><au>Bacon.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Skinker</ent><br/
<hw>Skink"er</hw> <pr>(?)</pr>, <pos>n.</pos> <def>One who serves liquor; a tapster.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Skinless</ent><br/
<hw>Skin"less</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Having no skin, or a very thin skin; <as>as, <ex>skinless</ex> fruit</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Skinner</ent><br/
<hw>Skin"ner</hw> <pr>(?)</pr>, <pos>n.</pos> <sn>1.</sn> <def>One who skins.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>One who deals in skins, pelts, or hides.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Skinniness</ent><br/
<hw>Skin"ni*ness</hw> <pr>(?)</pr>, <pos>n.</pos> <def>Quality of being skinny.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Skinny</ent><br/
<hw>Skin"ny</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Consisting, or chiefly consisting, of skin; wanting flesh.</def> <ldquo/Her <xex>skinny</xex> lips.<rdquo/  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>He holds him with a <qex>skinny</qex> hand.</q> <rj><qau>Coleridge.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Skip</ent><br/
<hw>Skip</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[See <er>Skep</er>.]</ety> <sn>1.</sn> <def>A basket. See <er>Skep</er>.</def> <mark>[Obs. or Prov. Eng. & Scot.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A basket on wheels, used in cotton factories.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <fld>(Mining)</fld> <def>An iron bucket, which slides between guides, for hoisting mineral and rock.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <fld>(Sugar Manuf.)</fld> <def>A charge of sirup in the pans.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <def>A beehive; a skep.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Skip</ent><br/
<hw>Skip</hw>, <pos>v. i.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Skipped</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Skipping</conjf>.]</vmorph> <ety>[OE. <ets>skippen</ets>, of uncertain origin; cf. Icel. <ets>skopa</ets> run, <ets>skoppa</ets> to spin like a top, OSw. & dial. Sw. <ets>skimmpa</ets> to run, <ets>skimpa</ets>, <ets>skompa</ets>, to hop, skip; or Ir. <ets>sgiob</ets> to snatch, Gael. <ets>sgiab</ets> to start or move suddenly, to snatch, W. <ets>ysgipio</ets> to snatch.]</ety> <sn>1.</sn> <def>To leap lightly; to move in leaps and hounds; -- commonly implying a sportive spirit.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The lamb thy riot dooms to bleed to-day,<br/
Had he thy reason, would he <qex>skip</qex> and play?</q> <rj><qau>Pope.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>So she drew her mother away <qex>skipping</qex>, dancing, and frisking fantastically.</q> <rj><qau>Hawthorne.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Fig.: To leave matters unnoticed, as in reading, speaking, or writing; to pass by, or overlook, portions of a thing; -- often followed by <xex>over</xex>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Skip</ent><br/
<hw>Skip</hw>, <pos>v. t.</pos> <sn>1.</sn> <def>To leap lightly over; <as>as, to <ex>skip</ex> the rope</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To pass over or by without notice; to omit; to miss; <as>as, to <ex>skip</ex> a line in reading; to <ex>skip</ex> a lesson</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>They who have a mind to see the issue may <qex>skip</qex> these two chapters.</q> <rj><qau>Bp. Burnet.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>To cause to skip; <as>as, to <ex>skip</ex> a stone</as>.</def> <mark>[Colloq.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Skip</ent><br/
<hw>Skip</hw>, <pos>n.</pos> <sn>1.</sn> <def>A light leap or bound.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>The act of passing over an interval from one thing to another; an omission of a part.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <fld>(Mus.)</fld> <def>A passage from one sound to another by more than a degree at once.</def>  <rj><au>Busby.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Skip kennel</b></col>, <cd>a lackey; a footboy.</cd> <mark>[Slang.]</mark> <au>Swift.</au> -- <col><b>Skip mackerel</b></col>. <fld>(Zool.)</fld> <cd>See <er>Bluefish</er>, 1.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Skipjack</ent><br/
<hw>Skip"jack`</hw> <pr>(?)</pr>, <pos>n.</pos> <sn>1.</sn> <def>An upstart.</def> <mark>[Obs.]</mark>  <rj><au>Ford.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Zool.)</fld> <def>An elater; a snap bug, or snapping beetle.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <fld>(Zool.)</fld> <def>A name given to several kinds of a fish, as the common bluefish, the alewife, the bonito, the butterfish, the cutlass fish, the jurel, the leather jacket, the runner, the saurel, the saury, the threadfish, etc.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <fld>(Naut.)</fld> <def>A shallow sailboat with a rectilinear or V-shaped cross section.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Skipper</ent><br/
<hw>Skip"per</hw> <pr>(?)</pr>, <pos>n.</pos> <sn>1.</sn> <def>One who, or that which, skips.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A young, thoughtless person.</def>  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <fld>(Zool.)</fld> <def>The saury (<spn>Scomberesox saurus</spn>).</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>The cheese maggot. See <cref>Cheese fly</cref>, under <er>Cheese</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <fld>(Zool.)</fld> <def>Any one of numerous species of small butterflies of the family <fam>Hesperiadae</fam>; -- so called from their peculiar short, jerking flight.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Skipper</ent><br/
<hw>Skip"per</hw>, <pos>n.</pos> <ety>[D. <ets>schipper</ets>. See <er>Shipper</er>, and <er>Ship</er>.]</ety> <sn>1.</sn> <fld>(Naut.)</fld> <def>The master of a fishing or small trading vessel; hence, the master, or captain, of any vessel.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A ship boy.</def> <mark>[Obs.]</mark>  <rj><au>Congreve.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Skippet</ent><br/
<hw>Skip"pet</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Cf. Icel. <ets>skip</ets>, E. <ets>skipper</ets>. See <er>Ship</er>.]</ety> <sn>1.</sn> <def>A small boat; a skiff.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>A little <qex>skippet</qex> floating did appear.</q> <rj><qau>Spenser.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A small round box for keeping records.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Skippingly</ent><br/
<hw>Skip"ping*ly</hw> <pr>(?)</pr>, <pos>adv.</pos> <def>In a skipping manner; by skips, or light leaps.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Skirl</ent><br/
<hw>Skirl</hw> <pr>(?)</pr>, <pos>v. t. & i.</pos> <ety>[Of Scand. origin, and originally the same word as E. <ets>shrill</ets>.]</ety> <def>To utter in a shrill tone; to scream.</def> <mark>[Prov. Eng. & Scot.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Skirl</ent><br/
<hw>Skirl</hw>, <pos>n.</pos> <def>A shrill cry or sound.</def> <mark>[Prov. Eng. & Scot.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Skirlcock</ent><br/
<hw>Skirl"cock`</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Zool.)</fld> <def>The missel thrush; -- so called from its harsh alarm note.</def> <mark>[Prev. Eng.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Skirlcrake</ent><br/
<hw>Skirl"crake`</hw> <pr>(?)</pr>, <pos>n.</pos> <def>The turnstone.</def> <mark>[Prev. Eng.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Skirling</ent><br/
<hw>Skirl"ing</hw>, <pos>n.</pos> <def>A shrill cry or sound; a crying shrilly; a skirl.</def> <mark>[Prov. Eng. & Scot.]</mark>  <rj><au>Sir W. Scott.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>When the <qex>skirling</qex> of the pipes cleft the air his cold eyes softened.</q> <rj><qau>Mrs. J. H. Ewing.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Skirling</ent><br/
<hw>Skirl"ing</hw>, <pos>n.</pos> <fld>(Zool.)</fld> <def>A small trout or salmon; -- a name used loosely.</def> <mark>[Prov. Eng.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Skirmish</ent><br/
<hw>Skir"mish</hw> <pr>(?)</pr>, <pos>v. i.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Skirmished</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Skirmishing</conjf>.]</vmorph> <ety>[OE. <ets>skirmishen</ets>, <ets>scarmishen</ets>, OF. <ets>escremir</ets>, <ets>eskermir</ets>, to fence, fight, F. <ets>escrimer</ets>, of German origin; cf. OHG. <ets>scirmen</ets> to protect, defend, G. <ets>schirmen</ets>, OHG. <ets>scirm</ets>, <ets>scerm</ets>, protection, shield, G. <ets>schirm</ets>; perhaps akin to Gr. <?/<?/<?/<?/ a sunshade.  Cf. <er>Scaramouch</er>, <er>Scrimmage</er>.]</ety> <def>To fight slightly or in small parties; to engage in a skirmish or skirmishes; to act as skirmishers.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Skirmish</ent><br/
<hw>Skir"mish</hw>, <pos>n.</pos><ety>[OE. <ets>scarmishe</ets>, <ets>scrymishe</ets>. See <er>Skirmish</er>, <pos>v. i.</pos>]</ety> <sn>1.</sn> <def>A slight fight in war; a light or desultory combat between detachments from armies, or between detached and small bodies of troops.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A slight contest.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>They never meet but there's a <qex>skirmish</qex> of wit.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Skirmisher</ent><br/
<hw>Skir"mish*er</hw> <pr>(?)</pr>, <pos>n.</pos> <def>One who skirmishes.</def> Specifically: <pluf>pl.</pluf> <fld>(Mil.)</fld> <def>Soldiers deployed in loose order, to cover the front or flanks of an advancing army or a marching column.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Skirr</ent><br/
<hw>Skirr</hw> <pr>(?)</pr>, <pos>v. t.</pos> <ety>[Cf. <er>Scur</er>, <er>Scurry</er>.]</ety> <def>To ramble over in order to clear; to scour.</def> <mark>[Archaic]</mark>  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Skirr</ent><br/
<hw>Skirr</hw>, <pos>v. i.</pos> <def>To scour; to scud; to run.</def> <mark>[Archaic]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Skirr</ent><br/
<hw>Skirr</hw>, <pos>n.</pos> <fld>(Zool.)</fld> <def>A tern.</def> <mark>[Prov. Eng.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Skirret</ent><br/
<hw>Skir"ret</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[A corrupted form equivalent to <ets>sugarwort</ets>.]</ety> <fld>(Bot.)</fld> <def>An umbelliferous plant (<spn>Sium Sisarum</spn> syn. <spn>Pimpinella Sisarum</spn>).  It is a native of Asia, but has been long cultivated in Europe for its edible clustered tuberous roots, which are very sweet.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Skirrhus</ent><br/
<hw>Skir"rhus</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Med.)</fld> <def>See <er>Scirrhus</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Skirt</ent><br/
<hw>Skirt</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[OE. <ets>skyrt</ets>, of Scand. origin; cf. Icel. <ets>skyrta</ets> a shirt, Sw. <ets>sk<oum/rt</ets> a skirt, <ets>skjorta</ets> a shirt. See <er>Shirt</er>.]</ety> <sn>1.</sn> <def>The lower and loose part of a coat, dress, or other like garment; the part below the waist; <as>as, the <ex>skirt</ex> of a coat, a dress, or a mantle</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A loose edging to any part of a dress.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>A narrow lace, or a small <qex>skirt</qex> of ruffled linen, which runs along the upper part of the stays before, and crosses the breast, being a part of the tucker, is called the modesty piece.</q> <rj><qau>Addison.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>Border; edge; margin; extreme part of anything</def> <ldquo/Here in the <xex>skirts</xex> of the forest.<rdquo/  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>A petticoat.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <def>The diaphragm, or midriff, in animals.</def>  <rj><au>Dunglison.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Skirt</ent><br/
<hw>Skirt</hw>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Skirted</conjf>; <pos>p. pr. & vb. n.</pos> <conjf>Skirting</conjf>.]</vmorph> <sn>1.</sn> <def>To cover with a skirt; to surround.</def><br/
[<source>1913 Webster</source>]</p>

<p><q><qex>Skirted</qex> his loins and thighs with downy gold.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To border; to form the border or edge of; to run along the edge of; <as>as, the plain was <ex>skirted</ex> by rows of trees</as>.</def> <ldquo/When sundown <xex>skirts</xex> the moor.<rdquo/  <rj><au>Tennyson.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Skirt</ent><br/
<hw>Skirt</hw>, <pos>v. t.</pos> <def>To be on the border; to live near the border, or extremity.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Savages . . . who <qex>skirt</qex> along our western frontiers.</q> <rj><qau>S. S. Smith.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Skirting</ent><br/
<hw>Skirt"ing</hw>, <pos>n.</pos> <sn>1.</sn> <fld>(Arch.)</fld> <def>A skirting board.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Skirts, taken collectivelly; material for skirts.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Skirting board</b></col>, <cd>the board running around a room on the wall next the floor; baseboard.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Skit</ent><br/
<hw>Skit</hw> <pr>(?)</pr>, <pos>v. t.</pos> <ety>[Prov. E. <ets>skit</ets>to slide, as adj., hasty, precipitate, of Scand. origin, and akin to E. <ets>shoot</ets>, v.t.; cf. Icel. <ets>skyti</ets>, <ets>skytja</ets>, <ets>skytta</ets>, a marksman, shooter, <ets>skj<omac/ta</ets> to shoot, <ets>sk<umac/ta</ets> a taunt. <root/159. See <er>Shoot</er>.]</ety> <def>To cast reflections on; to asperse.</def> <mark>[Prov. Eng. & Scot.]</mark>  <rj><au>Crose.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Skit</ent><br/
<hw>Skit</hw>, <pos>n.</pos> <sn>1.</sn> <def>A reflection; a jeer or gibe; a sally; a brief satire; a squib.</def>  <rj><au>Tooke.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>A similar vein satire upon the emptiness of writers is given in his <ldquo/Tritical Essay upon the Faculties of the Human Mind;<rdquo/ but that is a mere <qex>skit</qex> compared with this strange performance.</q> <rj><qau>Leslie Stephen.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A wanton girl; a light wench.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>A short theatrical presentation, often comical or satirical.  Comical <ex>skits</ex> are sometimes presented by amateur or ad hoc groups at parties, dinners, or other social gatherings.</def><br/
[<source>PJC</source>]</p>

<p><ent>Skitter</ent><br/
<hw>Skit"ter</hw> <pr>(?)</pr>, <pos>v. t.</pos> <ety>[Cf. <er>Skit</er>, <pos>v. t.</pos>]</ety> <def>To move or pass (something) over a surface quickly so that it touches only at intervals; to skip.</def></p>

<p><q>The angler, standing in the bow, <qex>'skitters'</qex> or skips the spoon over the surface.</q>  <rj><qau>James A. Henshall.</qau></rj><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Skitter</ent><br/
<hw>Skit"ter</hw>, <pos>v. i.</pos> <def>To pass or glide lightly or with quick touches at intervals; to skip; to skim.</def></p>

<p><q>Some kinds of ducks in lighting strike the water with their tails first, and <qex>skitter</qex> along the surface for a feet before settling down.</q>  <rj><qau>T. Roosevelt.</qau></rj><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Skittish</ent><br/
<hw>Skit"tish</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[See <er>Skit</er>, <pos>v. t.</pos>]</ety> <sn>1.</sn> <def>Easily frightened; timorous; shy; untrustworthy; <as>as, a <ex>skittish</ex> colt</as>.</def> <ldquo/A restiff, <xex>skittish</xex> jade.<rdquo/  <rj><au>L'Estrange.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Wanton; restive; freakish; volatile; changeable; fickle.</def> <ldquo/<xex>Skittish</xex> Fortune's hall.<rdquo/  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p>-- <wordforms><wf>Skit"tish*ly</wf>, <pos>adv.</pos> -- <wf>Skit"tish*ness</wf>, <pos>n.</pos></wordforms><br/
[<source>1913 Webster</source>]</p>

<p><ent>Skittle</ent><br/
<hw>Skit"tle</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Pertaining to the game of skittles.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Skittle alley</b></col>, <cd>an alley or court in which the game of skittles is played.</cd> -- <col><b>Skittle ball</b></col>, <cd>a disk or flattish ball of wood for throwing at the pins in the game of skittles.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Skittle-dog</ent><br/
<hw>Skit"tle-dog`</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Zool.)</fld> <def>The piked dogfish.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Skittles</ent><br/
<hw>Skit"tles</hw> <pr>(?)</pr>, <pos>n. pl.</pos> <ety>[Of Scand. origin. <root/159. See <er>Shoot</er>, <pos>v. t.</pos>, and cf. <er>Shuttle</er>, <er>Skit</er>, <pos>v. t.</pos>]</ety> <def>An English game resembling ninepins, but played by throwing wooden disks, instead of rolling balls, at the pins.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Skitty</ent><br/
<hw>Skit"ty</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Cf. <er>Skittish</er>.]</ety> <fld>(Zool.)</fld> <def>A rail; as, the water rail (called also <altname>skitty cock</altname>, and <altname>skitty coot</altname>); the spotted crake (<spn>Porzana maruetta</spn>), and the moor hen.</def> <mark>[Prov. Eng.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Skive</ent><br/
<hw>Skive</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Cf. Icel. <ets>sk<imac/fa</ets> a shaving, slice, E. <ets>shive</ets>, <ets>sheave</ets>.]</ety> <def>The iron lap used by diamond polishers in finishing the facets of the gem.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Skive</ent><br/
<hw>Skive</hw> <pr>(?)</pr>, <pos>v. t.</pos> <def>To pare or shave off the rough or thick parts of (hides or leather).</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Skiver</ent><br/
<hw>Skiv"er</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Cf. <er>Skewer</er>, <er>Shiver</er> a fragment.]</ety> <sn>1.</sn> <def>An inferior quality of leather, made of split sheepskin, tanned by immersion in sumac, and dyed. It is used for hat linings, pocketbooks, bookbinding, etc.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>The cutting tool or machine used in splitting leather or skins, as sheepskins.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Skiving</ent><br/
<hw>Ski"ving</hw> <pr>(?)</pr>, <pos>n.</pos> <sn>1.</sn> <def>The act of paring or splitting leather or skins.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A piece made in paring or splitting leather; specifically, the part from the inner, or flesh, side.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sklayre</ent><br/
<hw>Sklayre</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Cf. G. <ets>schleier</ets>.]</ety> <def>A vell.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sklere</ent><br/
<hw>Sklere</hw> <pr>(?)</pr>, <pos>v. t.</pos> <def>To shelter; to cover.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Skolezite</ent><br/
<ent>Skolecite</ent><br/
<mhw>{ <hw>Skol"e*cite</hw> <pr>(?)</pr>, <hw>Skol"e*zite</hw> <pr>(?)</pr>, }</mhw> <pos>n.</pos> <fld>(Min.)</fld> <def>See <er>Scolecite</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Skonce</ent><br/
<hw>Skonce</hw> <pr>(?)</pr>, <pos>n.</pos> <def>See <er>Sconce</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Skopster</ent><br/
<hw>Skop"ster</hw> <pr>(sk<ocr/p"st<etil/r)</pr>, <pos>n.</pos> <def>The saury.</def> <mark>[Prov. Eng.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Skoptsy</ent><br/
||<hw>Skop*tsy"</hw> <pr>(?)</pr>, <pos>n. pl.</pos> <def>See <er>Raskolnik</er>.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Skorodite</ent><br/
<hw>Skor"o*dite</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Min.)</fld> <def>See <er>Scorodite</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Skout</ent><br/
<hw>Skout</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Zool.)</fld> <def>A guillemot.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Skowitz</ent><br/
<hw>Sko"witz</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Nisqually (American Indian) name.]</ety> <fld>(Zool.)</fld> <def>The silver salmon.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Skreen</ent><br/
<hw>Skreen</hw> <pr>(?)</pr>, <pos>n. & v.</pos> <def>See <er>Screen</er>.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Skrike</ent><br/
<hw>Skrike</hw> <pr>(?)</pr>, <pos>v. i. & t.</pos> <def>To shriek.</def> <mark>[Obs.]</mark>  <rj><au>Chaucer.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Skrike</ent><br/
<hw>Skrike</hw>, <pos>n.</pos> <fld>(Zool.)</fld> <def>The missel thrush.</def> <mark>[Prov. Eng.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Skrimmage</ent><br/
<hw>Skrim"mage</hw> <pr>(?)</pr>, <pos>n.</pos> <def>See <er>Scrimmage</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Skrimp</ent><br/
<hw>Skrimp</hw> <pr>(?)</pr>, <pos>v. t.</pos> <def>See <er>Scrimp</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Skringe</ent><br/
<hw>Skringe</hw> <pr>(?)</pr>, <pos>v. i.</pos> <def>See <er>Scringe</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Skrite</ent><br/
<hw>Skrite</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Zool.)</fld> <def>The skrike.</def> <mark>[Prov. Eng.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Skua</ent><br/
<hw>Sku"a</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Icel. <ets>sk<?/fr</ets>, <ets>sk<?/mr</ets>.]</ety> <fld>(Zool.)</fld> <def>Any jager gull; especially, the <spn>Megalestris skua</spn>; -- called also <altname>boatswain</altname>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Skue</ent><br/
<hw>Skue</hw> <pr>(?)</pr>, <pos>a. & n.</pos> <def>See <er>Skew</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Skulk</ent><br/
<hw>Skulk</hw> <pr>(?)</pr>, <pos>v. i.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Skulked</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Skulking</conjf>.]</vmorph> <ety>[Of Scand. origin; cf. Dan. <ets>skulke</ets> to spare or save one's self, to play the truant, Sw. <ets>skolka</ets> to be at leisure, to shirk, Icel. <ets>skolla</ets>.  Cf. <er>Scowl</er>.]</ety> <def>To hide, or get out of the way, in a sneaking manner; to lie close, or to move in a furtive way; to lurk.</def> <ldquo/Want <xex>skulks</xex> in holes and crevices.<rdquo/  <rj><au>W. C. Bryant.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Discovered and defeated of your prey,<br/
You <qex>skulked</qex> behind the fence, and sneaked away.</q> <rj><qau>Dryden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Skulk</ent><br/
<hw>Skulk</hw>, <pos>n.</pos> <ety>[Cf. Icel. <ets>skollr</ets>, <ets>skolli</ets>, a fox, and E. <ets>skulk</ets>, v.i.]</ety> <def>A number of foxes together.</def>  <rj><au>Wright.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><-- p. 1351 --></p>

<p><ent>Skulker</ent><br/
<ent>Skulk</ent><br/
<mhw>{ <hw>Skulk</hw> <pr>(?)</pr>, <hw>Skulk"er</hw> <pr>(?)</pr>, }</mhw> <pos>n.</pos> <def>One who, or that which, skulks.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Skulkingly</ent><br/
<hw>Skulk"ing*ly</hw>, <pos>adv.</pos> <def>In a skulking manner.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Skull</ent><br/
<hw>Skull</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[See <er>School</er> a multitude.]</ety> <def>A school, company, or shoal.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>A knavish <qex>skull</qex> of boys and girls did pelt at him.</q>  <rj><au>Warner.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>These fishes enter in great flotes and skulls.</q> <rj><qau>Holland.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Skull</ent><br/
<hw>Skull</hw>, <pos>n.</pos> <ety>[OE. <ets>skulle</ets>, <ets>sculle</ets>, <ets>scolle</ets>; akin to Scot. <ets>skull</ets>, <ets>skoll</ets>, a bowl, Sw. <ets>skalle</ets> skull, <ets>skal</ets> a shell, and E. <ets>scale</ets>; cf. G. hirn<ets>schale</ets>, Dan. hierne<ets>skal</ets>.  Cf. <er>Scale</er> of a balance.]</ety> <sn>1.</sn> <fld>(Anat.)</fld> <def>The skeleton of the head of a vertebrate animal, including the brain case, or cranium, and the bones and cartilages of the face and mouth. See <xex>Illusts</xex>. of <er>Carnivora</er>, of <cref>Facial angles</cref> under <er>Facial</er>, and of <cref>Skeleton</cref>, in Appendix.</def><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ In many fishes the skull is almost wholly cartilaginous but in the higher vertebrates it is more or less completely ossified, several bones are developed in the face, and the cranium is made up, wholly or partially, of bony plates arranged in three segments, the <xex>frontal</xex>, <xex>parietal</xex>, and <xex>occipital</xex>, and usually closely united in the adult.</note><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>The head or brain; the seat of intelligence; mind.</def><br/
[<source>1913 Webster</source>]</p>

<p><q><qex>Skulls</qex> that can not teach, and will not learn.</q> <rj><qau>Cowper.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>A covering for the head; a skullcap.</def> <mark>[Obs. & R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>Let me put on my <qex>skull</qex> first.</q> <rj><qau>Beau. & Fl.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>A sort of oar. See <er>Scull</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Skull and crossbones</b></col>, <cd>a symbol of death. See <er>Crossbones</er>.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Skullcap</ent><br/
<hw>Skull"cap`</hw> <pr>(?)</pr>, <pos>n.</pos> <sn>1.</sn> <def>A cap which fits the head closely; also, formerly, a headpiece of iron sewed inside of a cap for protection.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Bot.)</fld> <def>Any plant of the labiate genus <gen>Scutellaria</gen>, the calyx of whose flower appears, when inverted, like a helmet with the visor raised.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <fld>(Zool.)</fld> <def>The Lophiomys.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Mad-dog skullcap</b></col> <fld>(Bot.)</fld>, <cd>an American herb (<spn>Scetellaria lateriflora</spn>) formerly prescribed as a cure for hydrophobia.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Skullfish</ent><br/
<hw>Skull"fish`</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A whaler's name for a whale more than two years old.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Skulpin</ent><br/
<hw>Skul"pin</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Zool.)</fld> <def>See <er>Sculpin</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Skun</ent><br/
<hw>Skun</hw> <pr>(?)</pr>, <pos>n. & v.</pos> <def>See <er>Scum</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Skunk</ent><br/
<hw>Skunk</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Contr. from the Abenaki (American Indian) <ets>seganku</ets>.]</ety> <fld>(Zool.)</fld> <def>Any one of several species of American musteline carnivores of the genus <gen>Mephitis</gen> and allied genera. They have two glands near the anus, secreting an extremely fetid liquid, which the animal ejects at pleasure as a means of defense.</def><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ The common species of the Eastern United States (<spn>Mephitis mephitica</spn>) is black with more or less white on the body and tail. The spotted skunk (<spn>Spilogale putorius</spn>), native of the Southwestern United States and Mexico, is smaller than the common skunk, and is variously marked with black and white.</note><br/
[<source>1913 Webster</source>]</p>

<p><cs><mcol><col><b>Skunk bird</b></col>, <col><b>Skunk blackbird</b></col></mcol> <fld>(Zool.)</fld>, <cd>the bobolink; -- so called because the male, in the breeding season, is black and white, like a skunk.</cd> -- <col><b>Skunk cabbage</b></col> <fld>(Bot.)</fld>, <cd>an American aroid herb (<spn>Symplocarpus f<oe/tidus</spn>) having a reddish hornlike spathe in earliest spring, followed by a cluster of large cabbagelike leaves. It exhales a disagreeable odor. Also called <altname>swamp cabbage</altname>.</cd> -- <col><b>Skunk porpoise</b></col>. <fld>(Zool.)</fld> <cd>See under <er>Porpoise</er>.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Skunk</ent><br/
<hw>Skunk</hw>, <pos>v. t.</pos> <def>In games of chance and skill: To defeat (an opponent) (as in cards) so that he fails to gain a point, or (in checkers) to get a king.</def> <mark>[Colloq. U. S.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Skunkball</ent><br/
<hw>Skunk"ball`</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Zool.)</fld> <def>The surf duck.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Skunkhead</ent><br/
<hw>Skunk"head`</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Zool.)</fld> <sd>(a)</sd> <def>The surf duck.</def> <sd>(b)</sd> <def>A duck (<spn>Camptolaimus Labradorus</spn>) which formerly inhabited the Atlantic coast of New England. It is now supposed to be extinct. Called also <altname>Labrador duck</altname>, and <altname>pied duck</altname>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Skunkish</ent><br/
<hw>Skunk"ish</hw>, <pos>a.</pos> <def>Like the skunk, especially in odor.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Skunktop</ent><br/
<hw>Skunk"top`</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Zool.)</fld> <def>The surf duck.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Skunkweed</ent><br/
<hw>Skunk"weed`</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Bot.)</fld> <def>Skunk cabbage.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Skurry</ent><br/
<hw>Skur"ry</hw> <pr>(?)</pr>, <pos>n. & v.</pos> <def>See <er>Scurry</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Skute</ent><br/
<hw>Skute</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Icel. <ets>sk<?/ta</ets>; akin to Sw. <ets>skuta</ets>, Dan. <ets>skude</ets>, D. <ets>schuit</ets>, Lg. <ets>sch<uum/te</ets>, and E. <ets>schoot</ets>, v.t.]</ety> <def>A boat; a small vessel.</def> <mark>[Obs.]</mark>  <rj><au>Sir R. Williams.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Skutterudite</ent><br/
<hw>Skut"ter*ud*ite</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[From <ets>Skutterud</ets>, in Norway, whence it is obtained.]</ety> <fld>(Min.)</fld> <def>A mineral of a bright metallic luster and tin-white to pale lead-gray color. It consists of arsenic and cobalt.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sky</ent><br/
<hw>Sky</hw> <pr>(sk<imac/)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Skies</plw> <pr>(sk<imac/z)</pr>.</plu> <ety>[OE. <ets>skie</ets> a cloud, Icel. <ets>sk<ymac/</ets>; akin to Sw. & Dan. <ets>sky</ets>; cf. AS. <ets>sc<umac/a</ets>, <ets>sc<umac/wa</ets>, shadow, Icel. <ets>skuggi</ets>; probably from the same root as E. <ets>scum</ets>. <root/158. See <er>Scum</er>, and cf. <er>Hide</er> skin, <er>Obscure</er>.]</ety> <sn>1.</sn> <def>A cloud.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>[A wind] that blew so hideously and high,<br/
That it ne lefte not a <qex>sky</qex><br/
In all the welkin long and broad.</q> <rj><qau>Chaucer.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Hence, a shadow.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>She passeth as it were a <qex>sky</qex>.</q> <rj><qau>Gower.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>The apparent arch, or vault, of heaven, which in a clear day is of a blue color; the heavens; the firmament; -- sometimes in the plural.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The Norweyan banners flout the <qex>sky</qex>.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>The wheather; the climate.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Thou wert better in thy grave than to answer with thy uncovered body this extremity of the <qex>skies</qex>.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ <xex>Sky</xex> is often used adjectively or in the formation of self-explaining compounds; as, <xex>sky</xex> color, <xex>sky</xex>light, <xex>sky</xex>-aspiring, <xex>sky</xex>-born, <xex>sky</xex>-pointing, <xex>sky</xex>-roofed, etc.</note><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Sky blue</b></col>, <cd>an azure color.</cd> -- <col><b>Sky scraper</b></col> <fld>(Naut.)</fld>, <cd>a skysail of a triangular form.</cd> <au>Totten.</au> -- <col><b>Under open sky</b></col>, <cd>out of doors.</cd> <ldquo/<xex>Under open sky</xex> adored.<rdquo/ <au>Milton.</au></cs>
<-- sky scraper, a tall building, usu. skyscraper --><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sky</ent><br/
<hw>Sky</hw>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Skied</conjf> <pr>(?)</pr> or <conjf>Skyed</conjf>; <pos>p. pr. & vb. n.</pos> <conjf>Skying</conjf> <pr>(?)</pr>.]</vmorph> <sn>1.</sn> <def>To hang (a picture on exhibition) near the top of a wall, where it can not be well seen.</def> <mark>[Colloq.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>Brother Academicians who <qex>skied</qex> his pictures.</q> <rj><qau>The Century.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To throw towards the sky; <as>as, to <ex>sky</ex> a ball at cricket</as>.</def> <mark>[Colloq.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sky-blue</ent><br/
<hw>Sky"-blue</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Having the blue color of the sky; azure; <as>as, a <ex>sky-blue</ex> stone</as>.</def>  <rj><au>Wordsworth.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Skyed</ent><br/
<hw>Skyed</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Surrounded by sky.</def> <mark>[Poetic & R.]</mark> <ldquo/The <xex>skyed</xex> mountain.<rdquo/  <rj><au>Thomson.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Skye terrier</ent><br/
<hw>Skye" ter"ri*er</hw> <pr>(?)</pr>. <fld>(Zool.)</fld> <def>See <er>Terrier</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Skyey</ent><br/
<hw>Sky"ey</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Like the sky; ethereal; being in the sky.</def> <ldquo/<xex>Skyey</xex> regions.<rdquo/  <rj><au>Thackeray.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Sublime on the towers of my <qex>skyey</qex> bowers,<br/
Lightning, my pilot, sits.</q> <rj><qau>Shelley.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sky-high</ent><br/
<hw>Sky"-high`</hw> <pr>(?)</pr>, <pos>adv. & a.</pos> <def>Very high.</def> <mark>[Colloq.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Skyish</ent><br/
<hw>Sky"ish</hw>, <pos>a.</pos> <def>Like the sky, or approaching the sky; lofty; ethereal.</def> <mark>[R.]</mark>  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Skylark</ent><br/
<hw>Sky"lark`</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Zool.)</fld> <def>A lark that mounts and sings as it files, especially the common species (<spn>Alauda arvensis</spn>) found in Europe and in some parts of Asia, and celebrated for its melodious song; -- called also <altname>sky laverock</altname>. See under <er>Lark</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ The Australian skylark (<spn>Cincloramphus cantillans</spn>) is a pipit which has the habit of ascending perpendicularly like a skylark, but it lacks the song of a true lark.  The Missouri skylark is a pipit (<spn>Anthus Spraguei</spn>) of the Western United States, resembling the skylark in habit and song.</note><br/
[<source>1913 Webster</source>]</p>

<p><ent>Skylarking</ent><br/
<hw>Sky"lark"ing</hw>, <pos>n.</pos> <def>The act of running about the rigging of a vessel in sport; hence, frolicking; scuffing; sporting; carousing.</def> <mark>[Colloq.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Skylight</ent><br/
<hw>Sky"light`</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A window placed in the roof of a building, in the ceiling of a room, or in the deck of a ship, for the admission of light from above.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Skyman</ent><br/
<hw>Sky"man</hw> <pr>(?)</pr>, <pos>n.</pos>; <plu>pl. <plw>-men</plw>.</plu> <def>An aeronaut.</def> <mark>[Slang]</mark><br/
<syn><b>Syn. --</b> airman; pilot.</syn><br/
[<source>Webster 1913 Suppl.</source> + <source>PJC</source>]</p>

<p><ent>Sky pilot</ent><br/
<hw>Sky pilot</hw>. <fld>(Aeronautics)</fld> <def>A person licensed as an airplane pilot.</def> <mark>[Slang]</mark><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>skyrocket</ent><br/
<hw>sky"rock`et</hw> <pr>(sk<imac/"r<ocr/k`<ecr/t)</pr>, <pos>n.</pos> <def>A rocket that ascends high and burns as it flies; a species of fireworks.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>skyrocket</ent><br/
<hw>sky"rock`et</hw> <pr>(sk<imac/"r<ocr/k`<ecr/t)</pr>, <pos>v. i.</pos> <def>To rise rapidly; -- usually used figuratively, as of prices.</def><br/
[<source>PJC</source>]</p>

<p><ent>Skysail</ent><br/
<hw>Sky"sail</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Naut.)</fld> <def>The sail set next above the royal. See <xex>Illust.</xex> under <er>Sail</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Skyscraper</ent><br/
<hw>Sky"scrap`er</hw> <pr>(?)</pr>, <pos>n.</pos> <sd>(a)</sd> <fld>(Naut.)</fld> (1) <def>A skysail of a triangular form.</def> <mark>[Rare]</mark> (2) <def>A name for the one of the fancy sails alleged to have been sometimes set above the skysail.</def> <mark>[Obs.]</mark> <sd>(b)</sd> <def>A very tall building, especially one over 20 stories high.</def> <sd>(c)</sd> <def>Hence, anything usually large, high, or excessive.</def> <mark>[Slang or Colloq.]</mark><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Skyward</ent><br/
<hw>Sky"ward</hw> <pr>(?)</pr>, <pos>a. & adv.</pos> <def>Toward the sky.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Slab</ent><br/
<hw>Slab</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[OE. <ets>slabbe</ets>, of uncertain origin; perhaps originally meaning, a smooth piece, and akin to <ets>slape</ets>, Icel. <ets>sleipr</ets> slippery, and E. <ets>slip</ets>, v. i.]</ety> <sn>1.</sn> <def>A thin piece of anything, especially of marble or other stone, having plane surfaces.</def>  <rj><au>Gwilt.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>An outside piece taken from a log or timber in sawing it into boards, planks, etc.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <fld>(Zool.)</fld> <def>The wryneck.</def> <mark>[Prov. Eng.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <fld>(Naut.)</fld> <def>The slack part of a sail.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Slab line</b></col> <fld>(Naut.)</fld>, <cd>a line or small rope by which seamen haul up the foot of the mainsail or foresail.</cd>  <rj><au>Totten.</au></rj></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Slab</ent><br/
<hw>Slab</hw>, <pos>a.</pos> <ety>[Cf. Gael. & Ir. <ets>slaib</ets> mud, mire left on a river strand, and E. <ets>slop</ets> puddle.]</ety> <def>Thick; viscous.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>Make the gruel thick and <qex>slab</qex>.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Slab</ent><br/
<hw>Slab</hw>, <pos>n.</pos> <def>That which is slimy or viscous; moist earth; mud; also, a puddle.</def> <mark>[Obs.]</mark>  <rj><au>Evelyn.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Slabber</ent><br/
<hw>Slab"ber</hw> <pr>(?)</pr>, <pos>v. i.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Slabbered</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Slabbering</conjf>.]</vmorph> <ety>[OE. <ets>slaberen</ets>; akin to LG. & D. <ets>slabbern</ets>, G. <ets>schlabbern</ets>, LG. & D. <ets>slabben</ets>, G. <ets>schlabben</ets>, Icel. <ets>slafra</ets>.  Cf. <er>Slaver</er>, <er>Slobber</er>, <er>Slubber</er>.]</ety> <def>To let saliva or some liquid fall from the mouth carelessly, like a child or an idiot; to drivel; to drool.</def> <altsp>[Written also <asp>slaver</asp>, and <asp>slobber</asp>.]</altsp><br/
[<source>1913 Webster</source>]</p>

<p><ent>Slabber</ent><br/
<hw>Slab"ber</hw>, <pos>v. t.</pos> <sn>1.</sn> <def>To wet and foul spittle, or as if with spittle.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>He <qex>slabbered</qex> me over, from cheek to cheek, with his great tongue.</q> <rj><qau>Arbuthnot.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To spill liquid upon; to smear carelessly; to spill, as liquid foed or drink, in careless eating or drinking.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The milk pan and cream pot so <qex>slabbered</qex> and tost<br/
That butter is wanting and cheese is half lost.</q> <rj><qau>Tusser.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Slabber</ent><br/
<hw>Slab"ber</hw>, <pos>n.</pos> <def>Spittle; saliva; slaver.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Slabber</ent><br/
<hw>Slab"ber</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[See 1st <er>Slab</er>.]</ety> <fld>(Mach.)</fld> <sd>(a)</sd> <def>A saw for cutting slabs from logs.</def> <sd>(b)</sd> <def>A slabbing machine.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Slabberer</ent><br/
<hw>Slab"ber*er</hw> <pr>(?)</pr>, <pos>n.</pos> <def>One who slabbers, or drools; hence, an idiot.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Slabbery</ent><br/
<hw>Slab"ber*y</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Like, or covered with, slabber or slab; slippery; sloppy.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Slabbiness</ent><br/
<hw>Slab"bi*ness</hw> <pr>(?)</pr>, <pos>n.</pos> <def>Quality of being slabby.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Slabbing</ent><br/
<hw>Slab"bing</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[See 1st <er>Slab</er>.]</ety> <def>Adapted for forming slabs, or for dressing flat surfaces.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Slabbing machine</b></col>, <cd>a milling machine.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Slabby</ent><br/
<hw>Slab"by</hw> <pr>(?)</pr>, <pos>a.</pos> <amorph>[<pos>Compar.</pos> <adjf>Slabbier</adjf> <pr>(?)</pr>; <pos>superl.</pos> <adjf>Slabbiest</adjf>.]</amorph> <ety>[See <er>Slab</er>, <pos>a.</pos>]</ety> <sn>1.</sn> <def>Thick; viscous.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>They present you with a cup, and you must drink of a <qex>slabby</qex> stuff.</q> <rj><qau>Selden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Sloppy; slimy; miry. See <er>Sloppy</er>.</def>  <rj><au>Gay.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Slab-sided</ent><br/
<hw>Slab"-sid`ed</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Having flat sides; hence, tall, or long and lank.</def> <mark>[Colloq. U. S.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Slack</ent><br/
<hw>Slack</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Cf. <er>Slag</er>.]</ety> <def>Small coal; also, coal dust; culm.</def>  <rj><au>Raymond.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Slack</ent><br/
<hw>Slack</hw>, <pos>n.</pos> <ety>[Icel. <ets>slakki</ets> a slope on a mountain edge.]</ety> <def>A valley, or small, shallow dell.</def> <mark>[Prov. Eng.]</mark>  <rj><au>Grose.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Slack</ent><br/
<hw>Slack</hw>, <pos>a.</pos> <amorph>[<pos>Compar.</pos> <adjf>Slacker</adjf> <pr>(?)</pr>; <pos>superl.</pos> <adjf>Slackest</adjf>.]</amorph> <ety>[OE. <ets>slak</ets>, AS. <ets>sleac</ets>; akin to OS. <ets>slak</ets>, OHG. <ets>slah</ets>, Prov. G. <ets>schlack</ets>, Icel. <ets>slakr</ets>, Sw. <ets>slak</ets>; cf. Skr. <ets>s<rsdot/j</ets> to let loose, to throw.  Cf. <er>Slake</er>.]</ety> <def>Lax; not tense; not hard drawn; not firmly extended; <as>as, a <ex>slack</ex> rope</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Weak; not holding fast; <as>as, a <ex>slack</ex> hand</as>.</def>  <rj><au>Milton.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>Remiss; backward; not using due diligence or care; not earnest or eager; <as>as, <ex>slack</ex> in duty or service</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The Lord is not <qex>slack</qex> concerning his promise, as some men count slackness.</q> <rj><qau>2 Pet. iii. 9.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>Not violent, rapid, or pressing; slow; moderate; easy; <as>as, business is <ex>slack</ex></as>.</def> <ldquo/With <xex>slack</xex> pace.<rdquo/  <rj><au>Chaucer.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>C<?/sar . . . about sunset, hoisting sail with a <qex>slack</qex> southwest, at midnight was becalmed.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Slack in stays</b></col> <fld>(Naut.)</fld>, <cd>slow in going about, as a ship.</cd> -- <col><b>Slack water</b></col>, <cd>the time when the tide runs slowly, or the water is at rest; or the interval between the flux and reflux of the tide.</cd> -- <col><b>Slack-water navigation</b></col>, <cd>navigation in a stream the depth of which has been increased, and the current diminished, by a dam or dams.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><syn><b>Syn.</b> -- Loose; relaxed; weak; remiss; backward; abated; diminished; inactive; slow; tardy; dull.</syn><br/
[<source>1913 Webster</source>]</p>

<p><ent>Slack</ent><br/
<hw>Slack</hw> <pr>(?)</pr>, <pos>adv.</pos> <def>Slackly; <as>as, <ex>slack</ex> dried hops</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Slack</ent><br/
<hw>Slack</hw>, <pos>n.</pos> <def>The part of anything that hangs loose, having no strain upon it; <as>as, the <ex>slack</ex> of a rope or of a sail</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Slacken</ent><br/
<ent>Slack</ent><br/
<mhw>{ <hw>Slack</hw> <pr>(?)</pr>, <hw>Slack"en</hw> <pr>(?)</pr>, }</mhw> <pos>v. i.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Slacked</conjf> <pr>(?)</pr>, <conjf>Slackened</conjf> <pr>(<?/)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Slacking</conjf>, <conjf>Slackening</conjf>.]</vmorph> <ety>[See <er>Slack</er>, <pos>a.</pos>]</ety> <sn>1.</sn> <def>To become slack; to be made less tense, firm, or rigid; to decrease in tension; <as>as, a wet cord <ex>slackens</ex> in dry weather</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To be remiss or backward; to be negligent.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>To lose cohesion or solidity by a chemical combination with water; to slake; <as>as, lime <ex>slacks</ex></as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>To abate; to become less violent.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Whence these raging fires<br/
Will <qex>slacken</qex>, if his breath stir not their flames.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <def>To lose rapidity; to become more slow; <as>as, a current of water <ex>slackens</ex></as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>6.</sn> <def>To languish; to fail; to flag.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>7.</sn> <def>To end; to cease; to desist; to slake.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>That through your death your lineage should <qex>slack</qex>.</q> <rj><qau>Chaucer.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>They will not of that firste purpose <qex>slack</qex>.</q> <rj><qau>Chaucer.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Slacken</ent><br/
<ent>Slack</ent><br/
<mhw>{ <hw>Slack</hw>, <hw>Slack"en</hw>, }</mhw> <pos>v. t.</pos> <sn>1.</sn> <def>To render slack; to make less tense or firm; <as>as, to <ex>slack</ex> a rope; to <ex>slacken</ex> a bandage</as>.</def>  <rj><au>Wycklif (Acts xxvii. 40)</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To neglect; to be remiss in.</def> <mark>[Obs.]</mark>  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q><qex>Slack</qex> not the pressage.</q> <rj><qau>Dryden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>To deprive of cohesion by combining chemically with water; to slake; <as>as, to <ex>slack</ex> lime</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>To cause to become less eager; to repress; to make slow or less rapid; to retard; <as>as, to <ex>slacken</ex> pursuit; to <ex>slacken</ex> industry</as>.</def> <ldquo/Rancor for to <xex>slack</xex>.<rdquo/  <rj><au>Chaucer.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>I should be grieved, young prince, to think my presence<br/
Unbent your thoughts, and <qex>slackened</qex> 'em to arms.</q> <rj><qau>Addison.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>In this business of growing rich, poor men should <qex>slack</qex> their pace.</q> <rj><qau>South.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>With such delay<br/
Well plased, they <qex>slack</qex> their course.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <def>To cause to become less intense; to mitigate; to abate; to ease.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>To respite, or deceive, or <qex>slack</qex> thy pain<br/
Of this ill mansion.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Air-slacked lime</b></col>, <cd>lime slacked by exposure to the air, in consequence of the absorption of carton dioxide and water, by which it is converted into carbonate of lime and hydrate of lime.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Slacken</ent><br/
<hw>Slack"en</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Metal.)</fld> <def>A spongy, semivitrifled substance which miners or smelters mix with the ores of metals to prevent their fusion.</def> <altsp>[Written also <asp>slakin</asp>.]</altsp><br/
[<source>1913 Webster</source>]</p>

<p><ent>Slackly</ent><br/
<hw>Slack"ly</hw>, <pos>adv.</pos> <def>In a slack manner.</def>  <rj><au>Trench.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Slackness</ent><br/
<hw>Slack"ness</hw>, <pos>n.</pos> <def>The quality or state of being slack.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Slade</ent><br/
<hw>Slade</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[AS. <ets>sl<?/d</ets>.]</ety> <sn>1.</sn> <def>A little dell or valley; a flat piece of low, moist ground.</def> <mark>[Obs.]</mark>  <rj><au>Drayton.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>The sole of a plow.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Slag</ent><br/
<hw>Slag</hw> <pr>(sl<acr/g)</pr>, <pos>n.</pos> <ety>[Sw. <ets>slagg</ets>, or LG. <ets>slacke</ets>, whence G. <ets>schlacke</ets>; originally, perhaps, the splinters struck off from the metal by hammering. See <er>Slay</er>, <pos>v. t.</pos>]</ety> <sn>1.</sn> <def>The dross, or recrement, of a metal; also, vitrified cinders.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>The scoria of a volcano.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <fld>(Metal.)</fld> <def>A product of smelting, containing, mostly as silicates, the substances not sought to be produced as matte or metal, and having a lower specific gravity than the latter; -- called also, esp. in iron smelting, <altname>cinder</altname>. The slag of iron blast furnaces is essentially silicate of calcium, magnesium, and aluminium; that of lead and copper smelting furnaces contains iron.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><cs><mcol><col><b>Slag furnace</b></col>, <it>or</it>  <col><b>Slag hearth</b></col></mcol> <fld>(Metal.)</fld>, <cd>a furnace, or hearth, for extracting lead from slags or poor ore.</cd> -- <col><b>Slag wool</b></col>, <cd>mineral wool. See under <er>Mineral</er>.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Slag</ent><br/
<hw>Slag</hw>, <pos>v. i. & t.</pos>  <vmorph>[<pos>imp. & p. p.</pos> <conjf>Slagged</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Slagging</conjf>.]</vmorph> <fld>(Metal.)</fld> <def>To form, or form into, a slag; to agglomerate when heated below the fusion point.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Slaggy</ent><br/
<hw>Slag"gy</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Of or pertaining to slag; resembling slag; <as>as, <ex>slaggy</ex> cobalt</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Slaie</ent><br/
<hw>Slaie</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[See <er>Sley</er>.]</ety> <def>A weaver's reed; a sley.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Slake</ent><br/
<hw>Slake</hw> <pr>(?)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Slaked</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Slaking</conjf>.]</vmorph> <ety>[OE. <ets>slaken</ets> to render slack, to slake, AS. <ets>sleacian</ets>, fr. <ets>sleac</ets> slack. See <er>Slack</er>, <pos>v. & a.</pos>]</ety> <sn>1.</sn> <def>To allay; to quench; to extinguish; <as>as, to <ex>slake</ex> thirst</as>.</def> <ldquo/And <xex>slake</xex> the heavenly fire.<rdquo/  <rj><au>Spenser.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>It could not <qex>slake</qex> mine ire nor ease my heart.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To mix with water, so that a true chemical combination shall take place; to slack; <as>as, to <ex>slake</ex> lime</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Slake</ent><br/
<hw>Slake</hw>, <pos>v. i.</pos> <sn>1.</sn> <def>To go out; to become extinct.</def> <ldquo/His flame did <xex>slake</xex>.<rdquo/  <rj><au>Sir T. Browne.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To abate; to become less decided.</def> <mark>[R.]</mark>  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>To slacken; to become relaxed.</def> <ldquo/When the body's strongest sinews <xex>slake</xex>.<rdquo/ <mark>[R.]</mark>  <rj><au>Sir J. Davies.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>To become mixed with water, so that a true chemical combination takes place; <as>as, the lime <ex>slakes</ex></as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Slake trough</b></col>, <cd>a trough containing water in which a blacksmith cools a forging or tool.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Slakeless</ent><br/
<hw>Slake"less</hw>, <pos>a.</pos> <def>Not capable of being slaked.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Slakin</ent><br/
<hw>Slak"in</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Metal.)</fld> <def>Slacken.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Slam</ent><br/
<hw>Slam</hw> <pr>(?)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Slammed</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Slamming</conjf>.]</vmorph> <ety>[Of Scand. origin; cf. Icel. <ets>slamra</ets>, <ets>slambra</ets>, <ets>sl<?/ma</ets>, Norw. <ets>slemba</ets>, <ets>slemma</ets>, dial. Sw. <ets>sl<aum/mma</ets>.]</ety> <sn>1.</sn> <def>To shut with force and a loud noise; to bang; <as>as, he <ex>slammed</ex> the door</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To put in or on some place with force and loud noise; -- usually with <xex>down</xex>; <as>as, to <ex>slam</ex> a trunk down on the pavement</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>To strike with some implement with force; hence, to beat or cuff.</def> <mark>[Prov. Eng.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>To strike down; to slaughter.</def> <mark>[Prov. Eng.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <def>To defeat (opponents at cards) by winning all the tricks of a deal or a hand.</def>  <rj><au>Hoyle.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>To slam to</b></col>, <cd>to shut or close with a slam.</cd> <ldquo/He <xex>slammed to</xex> the door.<rdquo/ <au>W. D. Howells.</au></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Slam</ent><br/
<hw>Slam</hw>, <pos>v. i.</pos> <def>To come or swing against something, or to shut, with sudden force so as to produce a shock and noise; <as>as, a door or shutter <ex>slams</ex></as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Slam</ent><br/
<hw>Slam</hw>, <pos>n.</pos> <sn>1.</sn> <def>The act of one who, or that which, slams.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>The shock and noise produced in slamming.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The <qex>slam</qex> and the scowl were lost upon Sam.</q> <rj><qau>Dickens.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <fld>(Card Playing)</fld> <def>Winning all the tricks of a deal (called, in bridge, <col><b>grand slam</b></col>, the winning of all but one of the thirteen tricks being called a <col><b>little slam</b></col> or <col><b>small slam</b></col>).</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><sn>4.</sn> <def>The refuse of alum works.</def> <mark>[Prov. Eng.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Slam-bang</ent><br/
<hw>Slam"-bang`</hw> <pr>(?)</pr>, <pos>adv.</pos> <def>With great violence; with a slamming or banging noise.</def> <mark>[Colloq.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Slammerkin</ent><br/
<ent>Slamkin</ent><br/
<mhw>{ <hw>Slam"kin</hw> <pr>(?)</pr>, <hw>Slam"mer*kin</hw> <pr>(?)</pr>, }</mhw> <pos>n.</pos> <ety>[Cf. G. <ets>schlampe</ets>, <ets>schlamp</ets>, dim. <ets>schl<aum/mpchen</ets>; <ets>schlampen</ets> to dangle, to be slovenly in one's dress.]</ety> <def>A slut; a slatternly woman.</def> <mark>[Obs. or Prov. Eng.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Slander</ent><br/
<hw>Slan"der</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[OE. <ets>sclandere</ets>, OF. <ets>esclandre</ets>, <ets>esclandle</ets>, <ets>escandre</ets>, F. <ets>esclandre</ets>, fr. L. <ets>scandalum</ets>, Gr. <?/<?/<?/ a snare, stumbling block, offense, scandal; probably originally, the spring of a trap, and akin to Skr. <ets>skand</ets> to spring, leap. See <er>Scan</er>, and cf. <er>Scandal</er>.]</ety> <sn>1.</sn> <def>A false tale or report maliciously uttered, tending to injure the reputation of another; the malicious utterance of defamatory reports; the dissemination of malicious tales or suggestions to the injury of another.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Whether we speak evil of a man to his face or behind his back; the former way, indeed, seems to be the most generous, but yet is a great fault, and that which we call <ldquo/reviling;<rdquo/ the latter is more mean and base, and that which we properly call <ldquo/<qex>slander</qex><rdquo/, or <ldquo/Backbiting.<rdquo/</q> <rj><qau>Tillotson.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>[We] make the careful magistrate<br/
The mark of <qex>slander</qex>.</q> <rj><qau>B. Jonson.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><-- p. 1352 --></p>

<p><sn>2.</sn> <def>Disgrace; reproach; dishonor; opprobrium.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Thou <qex>slander</qex> of thy mother's heavy womb.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <fld>(Law)</fld> <def>Formerly, defamation generally, whether oral or written; in modern usage, defamation by words spoken; utterance of false, malicious, and defamatory words, tending to the damage and derogation of another; calumny. See the Note under <er>Defamation</er>.</def>  <rj><au>Burril.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Slander</ent><br/
<hw>Slan"der</hw> <pr>(?)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Slandered</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Slandering</conjf>.]</vmorph> <sn>1.</sn> <def>To defame; to injure by maliciously uttering a false report; to tarnish or impair the reputation of by false tales maliciously told or propagated; to calumniate.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>O, do not <qex>slander</qex> him, for he is kind.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To bring discredit or shame upon by one's acts.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Tax not so bad a voice<br/
To <qex>slander</qex> music any more than once.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><syn><b>Syn.</b> -- To asperse; defame; calumniate; vilify; malign; belie; scandalize; reproach. See <er>Asperse</er>.</syn><br/
[<source>1913 Webster</source>]</p>

<p><ent>Slanderer</ent><br/
<hw>Slan"der*er</hw> <pr>(?)</pr>, <pos>n.</pos> <def>One who slanders; a defamer; a calumniator.</def>  <rj><au>Jer. Taylor.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Slanderous</ent><br/
<hw>Slan"der*ous</hw> <pr>(?)</pr>, <pos>a.</pos> <sn>1.</sn> <def>Given or disposed to slander; uttering slander.</def> <ldquo/<xex>Slanderous</xex> tongue.<rdquo/  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Embodying or containing slander; calumnious; <as>as, <ex>slanderous</ex> words, speeches, or reports</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p>-- <wordforms><wf>Slan"der*ous*ly</wf>, <pos>adv.</pos> -- <wf>Slan"der*ous*ness</wf>, <pos>n.</pos></wordforms><br/
[<source>1913 Webster</source>]</p>

<p><ent>Slang</ent><br/
<hw>Slang</hw> <pr>(?)</pr>, <def><pos>imp.</pos> of <er>Sling</er>. Slung.</def> <mark>[Archaic]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Slang</ent><br/
<hw>Slang</hw>, <pos>n.</pos> <def>Any long, narrow piece of land; a promontory.</def> <mark>[Local, Eng.]</mark>  <rj><au>Holland.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Slang</ent><br/
<hw>Slang</hw>, <pos>n.</pos> <ety>[Cf. <er>Sling</er>.]</ety> <def>A fetter worn on the leg by a convict.</def> <mark>[Eng.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Slang</ent><br/
<hw>Slang</hw>, <pos>n.</pos> <ety>[Said to be of Gypsy origin; but probably from Scand., and akin to E. <ets>sling</ets>; cf. Norw. <ets>sleng</ets> a slinging, an invention, device, <ets>slengja</ets> to sling, to cast, <ets>slengja kjeften</ets> (literally, to sling the jaw) to use abusive language, to use slang, <ets>slenjeord</ets> (<ets>ord</ets> = word) an insulting word, a new word that has no just reason for being.]</ety> <def>Low, vulgar, unauthorized language; a popular but unauthorized word, phrase, or mode of expression; also, the jargon of some particular calling or class in society; low popular cant; <as>as, the <ex>slang</ex> of the theater, of college, of sailors, etc.</as></def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Slang</ent><br/
<hw>Slang</hw>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Slanged</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Slanging</conjf>.]</vmorph> <def>To address with slang or ribaldry; to insult with vulgar language.</def> <mark>[Colloq.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>Every gentleman abused by a cabman or <qex>slanged</qex> by a bargee was bound there and then to take off his coat and challenge him to fisticuffs.</q> <rj><qau>London Spectator.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Slanginess</ent><br/
<hw>Slang"i*ness</hw> <pr>(?)</pr>, <pos>n.</pos> <def>Quality of being slangy.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Slangous</ent><br/
<hw>Slan"gous</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Slangy.</def> <mark>[R.]</mark>  <rj><au>John Bee.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Slang-whanger</ent><br/
<hw>Slang"-whang`er</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[<ets>Slang</ets> + <ets>whang</ets> to beat.]</ety> <def>One who uses abusive slang; a ranting partisan.</def> <mark>[Colloq. or Humorous]</mark>  <rj><au>W. Irving.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Slangy</ent><br/
<hw>Slang"y</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Of or pertaining to slang; of the nature of slang; disposed to use slang.</def> <altsp>[Written also <asp>slangey</asp>.]</altsp><br/
[<source>1913 Webster</source>]</p>

<p><ent>Slank</ent><br/
<hw>Slank</hw> <pr>(?)</pr>, <def><pos>imp. & p. p.</pos> of <er>Slink</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Slant</ent><br/
<hw>Slant</hw> <pr>(?)</pr>, <pos>v. i.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Slanted</conjf>; <pos>p. pr. & vb. n.</pos> <conjf>Slanting</conjf>.]</vmorph> <ety>[OE. <ets>slenten</ets> to slope, slide; cf. Sw. <ets>slinta</ets> to slide.]</ety> <def>To be turned or inclined from a right line or level; to lie obliquely; to slope.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>On the side of younder <qex>slanting</qex> hill.</q> <rj><qau>Dodsley.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Slant</ent><br/
<hw>Slant</hw>, <pos>v. t.</pos> <def>To turn from a direct line; to give an oblique or sloping direction to; <as>as, to <ex>slant</ex> a line</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Slant</ent><br/
<hw>Slant</hw>, <pos>n.</pos> <sn>1.</sn> <def>A slanting direction or plane; a slope; <as>as, it lies on a <ex>slant</ex></as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>An oblique reflection or gibe; a sarcastic remark.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Slant or wind</b></col>, <cd>a local variation of the wind from its general direction.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Slant</ent><br/
<hw>Slant</hw>, <pos>a.</pos> <ety>[Cf. dial. Sw. <ets>slant</ets>. See <er>Slant</er>, <pos>v. i.</pos>]</ety> <def>Inclined from a direct line, whether horizontal or perpendicular; sloping; oblique.</def> <ldquo/The <xex>slant</xex> lightning.<rdquo/  <rj><au>Milton.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Slanting</ent><br/
<hw>Slant"ing</hw>, <pos>a.</pos> <def>Oblique; sloping.</def> -- <wordforms><wf>Slant"ing*ly</wf>, <pos>adv.</pos></wordforms><br/
[<source>1913 Webster</source>]</p>

<p><ent>Slantly</ent><br/
<ent>Slantwise</ent><br/
<mhw>{ <hw>Slant"wise`</hw> <pr>(?)</pr>, <hw>Slant"ly</hw>  }</mhw>, <pos>adv.</pos> <def>In an inclined direction; obliquely; slopingly.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Slap</ent><br/
<hw>Slap</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[OE. <ets>slappe</ets>; akin to LG. <ets>slappe</ets>, G. <ets>schlappe</ets>; probably of imitative origin.]</ety> <def>A blow, esp. one given with the open hand, or with something broad.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Slap</ent><br/
<hw>Slap</hw>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Slapped</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Slapping</conjf>.]</vmorph> <def>To strike with the open hand, or with something broad.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Slap</ent><br/
<hw>Slap</hw>, <pos>adv.</pos> <ety>[Cf. LG. <ets>slap</ets>, G. <ets>schlapp</ets>. See <er>Slap</er>, <pos>n.</pos>]</ety> <def>With a sudden and violent blow; hence, quickly; instantly; directly.</def> <mark>[Colloq.]</mark> <ldquo/The railroad cars drive <xex>slap</xex> into the city.<rdquo/  <rj><au>Thackeray.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Slapdash</ent><br/
<hw>Slap"dash`</hw> <pr>(?)</pr>, <pos>adv.</pos> <ety>[<ets>Slap</ets> + <ets>dash</ets>.]</ety> <sn>1.</sn> <def>In a bold, careless manner; at random.</def> <mark>[Colloq.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>With a slap; all at once; slap.</def> <mark>[Colloq.]</mark>  <rj><au>Prior.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Slapdash</ent><br/
<hw>Slap"dash`</hw>, <pos>v. t.</pos> <def>To apply, or apply something to, in a hasty, careless, or rough manner; to roughcast; <as>as, to <ex>slapdash</ex> mortar or paint on a wall, or to <ex>slapdash</ex> a wall</as>.</def> <mark>[Colloq.]</mark>  <rj><au>Halliwell.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Slape</ent><br/
<hw>Slape</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[Icel. <ets>sleipr</ets> slippery; akin to E. <ets>slip</ets>.]</ety> <def>Slippery; smooth; crafty; hypocritical.</def> <mark>[Prov. Eng.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Slape ale</b></col>, <cd>plain ale, as opposed to <contr>medicated</contr> or <contr>mixed</contr> ale.</cd> <mark>[Prov. Eng.]</mark></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Slapeface</ent><br/
<hw>Slape"face`</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A soft-spoken, crafty hypocrite.</def> <mark>[Prov. Eng.]</mark>  <rj><au>Halliwell.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Slapjack</ent><br/
<hw>Slap"jack`</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A flat batter cake cooked on a griddle; a flapjack; a griddlecake.</def> <mark>[Local, U.S.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Slapper</ent><br/
<hw>Slap"per</hw> <pr>(?)</pr>, <pos>n.</pos> <sn>1.</sn> <def>One who, or that which, slaps.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Anything monstrous; a whopper.</def> <mark>[Slang]</mark>  <rj><au>Grose.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Slapping</ent><br/
<ent>Slapper</ent><br/
<mhw>{ <hw>Slap"per</hw> <pr>(?)</pr>, <hw>Slap"ping</hw> <pr>(?)</pr>, }</mhw> <pos>a.</pos> <def>Very large; monstrous; big.</def> <mark>[Slang.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Slash</ent><br/
<hw>Slash</hw>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Slashed</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Slashing</conjf>.]</vmorph> <ety>[OE. <ets>slaschen</ets>, of uncertain origin; cf. OF. <ets>esclachier</ets> to break, <ets>esclechier</ets>, <ets>esclichier</ets>, to break, and E. <ets>slate</ets>, <ets>slice</ets>, <ets>slit</ets>, <pos>v. t.</pos>]</ety> <sn>1.</sn> <def>To cut by striking violently and at random; to cut in long slits.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To lash; to ply the whip to.</def> <mark>[R.]</mark>  <rj><au>King.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>To crack or snap, as a whip.</def> <mark>[R.]</mark>  <rj><au>Dr. H. More.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Slash</ent><br/
<hw>Slash</hw>, <pos>v. i.</pos> <def>To strike violently and at random, esp. with an edged instrument; to lay about one indiscriminately with blows; to cut hastily and carelessly.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Hewing and <qex>slashing</qex> at their idle shades.</q> <rj><qau>Spenser.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Slash</ent><br/
<hw>Slash</hw>, <pos>n.</pos> <sn>1.</sn> <def>A long cut; a cut made at random.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A large slit in the material of any garment, made to show the lining through the openings.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <ety>[Cf. <er>Slashy</er>.]</ety> <pluf>pl.</pluf> <def>Swampy or wet lands overgrown with bushes.</def> <mark>[Local, U.S.]</mark>  <rj><au>Bartlett.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>A opening or gap in a forest made by wind, fire, or other destructive agency.</def></p>

<p><q>We passed over the  shoulder of a ridge and around the edge of a fire <qex>slash</qex>, and then we had the mountain fairly before us.</q>  <rj><qau>Henry Van Dyke.</qau></rj><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Slashed</ent><br/
<hw>Slashed</hw> <pr>(?)</pr>, <pos>a.</pos> <sn>1.</sn> <def>Marked or cut with a slash or slashes; deeply gashed; especially, having long, narrow openings, as a sleeve or other part of a garment, to show rich lining or under vesture.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>A gray jerkin, with scarlet and <qex>slashed</qex> sleeves.</q> <rj><qau>Sir W. Scott.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Bot.)</fld> <def>Divided into many narrow parts or segments by sharp incisions; laciniate.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Slasher</ent><br/
<hw>Slash"er</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Textile Manuf.)</fld> <def>A machine for applying size to warp yarns.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Slash pine</ent><br/
<hw>Slash" pine"</hw> <pr>(?)</pr>. <fld>(Bot.)</fld> <def>A kind of pine tree (<spn>Pinus Cubensis</spn>) found in Southern Florida and the West Indies; -- so called because it grows in <ldquo/slashes.<rdquo/</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Slashy</ent><br/
<hw>Slash"y</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[Cf. Sw. <ets>slaska</ets> to dabble in water.  Cf. <er>Slush</er>.]</ety> <def>Wet and dirty; slushy.</def> <mark>[Prov. Eng.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Slat</ent><br/
<hw>Slat</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[CF. <er>Slot</er> a bar.]</ety> <def>A thin, narrow strip or bar of wood or metal; <as>as, the <ex>slats</ex> of a window blind</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Slat</ent><br/
<hw>Slat</hw>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Slatted</conjf>; <pos>p. pr. & vb. n.</pos> <conjf>Slatting</conjf>.]</vmorph> <ety>[OE. <ets>slatten</ets>; cf. Icel. <ets>sletta</ets> to slap, to dab.]</ety> <sn>1.</sn> <def>To slap; to strike; to beat; to throw down violently.</def> <mark>[Prov. Eng. & Colloq. U.S.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>How did you kill him?<br/
<qex>Slat<plain>[t]</plain>ed</qex> his brains out.</q> <rj><qau>Marston.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To split; to crack.</def> <mark>[Prov. Eng.]</mark>  <rj><au>Halliwell.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>To set on; to incite. See 3d <er>Slate</er>.</def> <mark>[Prov. Eng.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Slatch</ent><br/
<hw>Slatch</hw> <pr>(sl<acr/ch)</pr>, <pos>n.</pos> <ety>[See <er>Slack</er>.]</ety> <fld>(Naut.)</fld> <sd>(a)</sd> <def>The period of a transitory breeze.</def> <sd>(b)</sd> <def>An interval of fair weather.</def> <sd>(c)</sd> <def>The loose or slack part of a rope; slack.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Slate</ent><br/
<hw>Slate</hw> <pr>(sl<amac/t)</pr>, <pos>n.</pos> <ety>[OE. <ets>slat</ets>, <ets>sclat</ets>, OF. <ets>esclat</ets> a shiver, splinter, F. <ets><eacute/clat</ets>, fr. OF. <ets>esclater</ets> to shiver, to chip, F. <ets><eacute/clater</ets>, fr. OHG. <ets>sleizen</ets> to tear, slit, split, fr. <ets>sl<imac/zan</ets> to slit, G. <ets>schleissen</ets>. See <er>Slit</er>, <pos>v. t.</pos>, and cf. <er>Eclat</er>.]</ety> <sn>1.</sn> <fld>(Min.)</fld> <def>An argillaceous rock which readily splits into thin plates; argillite; argillaceous schist.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Any rock or stone having a slaty structure.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>A prepared piece of such stone.</def> Especially: <sd>(a)</sd> <def>A thin, flat piece, for roofing or covering houses, etc.</def> <sd>(b)</sd> <def>A tablet for writing upon.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>An artificial material, resembling slate, and used for the above purposes.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <def>A thin plate of any material; a flake.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><sn>6.</sn> <fld>(Politics)</fld> <def>A list of candidates, prepared for nomination or for election; a list of candidates, or a programme of action, devised beforehand.</def> <mark>[Cant, U.S.]</mark>  <rj><au>Bartlett.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Adhesive slate</b></col> <fld>(Min.)</fld>, <cd>a kind of slate of a greenish gray color, which absorbs water rapidly, and adheres to the tongue; whence the name.</cd> -- <mcol><col><b>Aluminous slate</b></col>, <it>or</it> <col><b>Alum slate</b></col></mcol> <fld>(Min.)</fld>, <cd>a kind of slate containing sulphate of alumina, -- used in the manufacture of alum.</cd> -- <col><b>Bituminous slate</b></col> <fld>(Min.)</fld>, <cd>a soft species of sectile clay slate, impregnated with bitumen.</cd> -- <col><b>Hornblende slate</b></col> <fld>(Min.)</fld>, <cd>a slaty rock, consisting essentially of hornblende and feldspar, useful for flagging on account of its toughness.</cd> -- <mcol><col><b>Slate ax</b></col> <it>or</it> <col><b>Slate axe</b></col></mcol>, <cd>a mattock with an ax end, used in shaping slates for roofs, and making holes in them for the nails.</cd> -- <col><b>Slate clay</b></col> <fld>(Geol.)</fld>, <cd>an indurated clay, forming one of the alternating beds of the coal measures, consisting of an infusible compound of alumina and silica, and often used for making fire bricks.</cd> <au>Tomlinson.</au> -- <col><b>Slate globe</b></col>, <cd>a globe the surface of which is made of an artificial slatelike material.</cd> -- <col><b>Slate pencil</b></col>, <cd>a pencil of slate, or of soapstone, used for writing on a slate.</cd> -- <col><b>Slate rocks</b></col> <fld>(Min.)</fld>, <cd>rocks which split into thin laminae, not necessarily parallel to the stratification; foliated rocks.</cd> -- <col><b>Slate spar</b></col> <fld>(Min.)</fld>, <cd>a variety of calcite of silvery white luster and of a slaty structure.</cd> -- <col><b>Transparent slate</b></col>, <cd>a plate of translucent material, as ground glass, upon which a copy of a picture, placed beneath it, can be made by tracing.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Slate</ent><br/
<hw>Slate</hw>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Slated</conjf>; <pos>p. pr. & vb. n.</pos> <conjf>Slating</conjf>.]</vmorph> <sn>1.</sn> <def>To cover with slate, or with a substance resembling slate; <as>as, to <ex>slate</ex> a roof; to <ex>slate</ex> a globe.</as></def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To register (as on a slate and subject to revision), for an appointment.</def> <mark>[Polit. Cant]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Slate</ent><br/
<hw>Slate</hw>, <pos>v. t.</pos> <ety>[Cf. AS. <ets>sl<aemac/ting</ets> a privilege of hunting.]</ety> <def>To set a dog upon; to bait; to slat. See 2d <er>Slat</er>, 3.</def> <mark>[Prov. Eng. & Scot.]</mark> <altsp>[Written also <asp>slete</asp>.]</altsp>  <rj><au>Ray.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><-- 2. To schedule.
    3. To reserve or designate for a specific purpose. --> <br/
[<source>1913 Webster</source>]</p>

<p><ent>Slate-color</ent><br/
<hw>Slate"-col`or</hw> <pr>(?)</pr>. <def>A dark bluish gray color.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Slate-gray</ent><br/
<hw>Slate"-gray`</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Of a dark gray, like slate.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Slater</ent><br/
<hw>Slat"er</hw> <pr>(?)</pr>, <pos>n.</pos> <def>One who lays slates, or whose occupation is to slate buildings.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Slater</ent><br/
<hw>Slat"er</hw>, <pos>n.</pos> <fld>(Zool.)</fld> <def>Any terrestrial isopod crustacean of the genus <gen>Porcellio</gen> and allied genera; a sow bug.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Slating</ent><br/
<hw>Slat"ing</hw>, <pos>n.</pos> <sn>1.</sn> <def>The act of covering with slate, slates, or a substance resembling slate; the work of a slater.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Slates, collectively; also, material for slating.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Slatt</ent><br/
<hw>Slatt</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[See <er>Slat</er> a strip of board.]</ety> <def>A slab of stone used as a veneer for coarse masonry.</def>  <rj><au>Knight.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Slatter</ent><br/
<hw>Slat"ter</hw> <pr>(?)</pr>, <pos>v. i.</pos> <ety>[E. <ets>slat</ets> to throw or dash about.]</ety> <def>To be careless, negligent, or aswkward, esp. with regard to dress and neatness; to be wasteful.</def>  <rj><au>Ray.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Slattern</ent><br/
<hw>Slat"tern</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A woman who is negligent of her dress or house; one who is not neat and nice.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Slattern</ent><br/
<hw>Slat"tern</hw>, <pos>a.</pos> <def>Resembling a slattern; sluttish; slatterny.</def> <ldquo/The <xex>slattern</xex> air.<rdquo/  <rj><au>Gay.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Slattern</ent><br/
<hw>Slat"tern</hw> <pr>(?)</pr>, <pos>v. t.</pos> <def>To consume carelessly or wastefully; to waste; -- with <xex>away</xex>.</def> <mark>[R.]</mark>  <rj><au>Chesterfield.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Slatternliness</ent><br/
<hw>Slat"tern*li*ness</hw> <pr>(?)</pr>, <pos>n.</pos> <def>The quality or state of being slatternly; slovenliness; untidiness.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Slatternly</ent><br/
<hw>Slat"tern*ly</hw>, <pos>a.</pos> <def>Resembling a slattern; sluttish; negligent; dirty.</def> -- <def2><pos>adv.</pos> <def>In a slatternly manner.</def></def2><br/
[<source>1913 Webster</source>]</p>

<p><ent>Slatterpouch</ent><br/
<hw>Slat"ter*pouch`</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A dance or game played by boys, requiring active exercise.</def> <mark>[Obs.]</mark>  <rj><au>Gayton.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Slatting</ent><br/
<hw>Slat"ting</hw> <pr>(?)</pr>, <def>Slats, collectively.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Slatting</ent><br/
<hw>Slat"ting</hw>, <pos>n.</pos> <def>The violent shaking or flapping of anything hanging loose in the wind, as of a sail, when being hauled down.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Slaty</ent><br/
<hw>Slat"y</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[From <er>Slate</er>.]</ety> <def>Resembling slate; having the nature, appearance, or properties, of slate; composed of thin parallel plates, capable of being separated by splitting; <as>as, a <ex>slaty</ex> color or texture</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Slaty cleavage</b></col> <fld>(Min.)</fld>, <cd>cleavage, as of rocks, into thin leaves or plates, like those of slate; -- applied especially to those cases in which the planes of cleavage are not parallel to the planes of stratification. It is now believed to be caused by the compression which the strata have undergone.</cd> -- <col><b>Slaty gneiss</b></col> <fld>(Min.)</fld>, <cd>a variety of gneiss in which the scales of mica or crystals of hornblende, which are usually minute, form thin laminae, rendering the rock easily cleavable.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Slaughter</ent><br/
<hw>Slaugh"ter</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[OE. <ets>slautir</ets>, <ets>slaughter</ets>, <ets>slaghter</ets>, Icel. <ets>sl<amac/tr</ets> slain flesh, modified by OE. <ets>slaught</ets>, <ets>slaht</ets>, slaughter, fr. AS. <ets>sleaht</ets> a stroke, blow; both from the root of E. slay. See <er>Slay</er>, <pos>v. t.</pos>, and cf. <er>Onslaught</er>.]</ety> <def>The act of killing.</def> Specifically: <sd>(a)</sd> <def>The extensive, violent, bloody, or wanton destruction of life; carnage.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>On war and mutual <qex>slaughter</qex> bent.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sd>(b)</sd> <def>The act of killing cattle or other beasts for market.</def><br/
[<source>1913 Webster</source>]</p>

<p><syn><b>Syn.</b> -- Carnage; massacre; butchery; murder; havoc.</syn><br/
[<source>1913 Webster</source>]</p>

<p><ent>Slaughter</ent><br/
<hw>Slaugh"ter</hw>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Slaughtered</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Slaughtering</conjf>.]</vmorph> <sn>1.</sn> <def>To visit with great destruction of life; to kill; to slay in battle.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Your castle is surprised; your wife and babes<br/
Savagely <qex>slaughtered</qex>.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To butcher; to kill for the market, as beasts.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Slaughterer</ent><br/
<hw>Slaugh"ter*er</hw> <pr>(?)</pr>, <pos>n.</pos> <def>One who slaughters.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Slaughterhouse</ent><br/
<hw>Slaugh"ter*house`</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A house where beasts are butchered for the market.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Slaughterman</ent><br/
<hw>Slaugh"ter*man</hw> <pr>(?)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Slaughtermen</plw> <pr>(<?/)</pr>.</plu> <def>One employed in slaughtering.</def>  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Slaughterous</ent><br/
<hw>Slaugh"ter*ous</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Destructive; murderous.</def> <au>Shak.</au> <au>M. Arnold.</au> -- <wordforms><wf>Slaugh"ter*ous*ly</wf>, <pos>adv.</pos></wordforms><br/
[<source>1913 Webster</source>]</p>

<p><ent>Slav</ent><br/
<hw>Slav</hw> <pr>(sl<aum/v <it>or</it>  sl<acr/v)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Slavs</plw> <pr>(#)</pr>.</plu> <ety>[A word originally meaning, intelligible, and used to contrast the people so called with foreigners who spoke languages unintelligible to the Slavs; akin to OSlav. <ets>slovo</ets> a word, <ets>slava</ets> fame, Skr. <ets><cced/ru</ets> to hear.  Cf. <er>Loud</er>.]</ety> <fld>(Ethnol.)</fld> <def>One of a race of people occupying a large part of Eastern and Northern Europe, including the Russians, Bulgarians, Roumanians, Servo-Croats, Slovenes, Poles, Czechs, Wends or Sorbs, Slovaks, etc.</def> <altsp>[Written also <asp>Slave</asp>, and <asp>Sclav</asp>.]</altsp><br/
[<source>1913 Webster</source>]</p>

<p><ent>Slave</ent><br/
<hw>Slave</hw> <pr>(sl<aum/v <it>or</it>  sl<acr/v; 277)</pr> <pos>n.</pos> <def>See <er>Slav</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Slave</ent><br/
<hw>Slave</hw> <pr>(sl<amac/v)</pr>, <pos>n.</pos> <ety>[Cf. F. <ets>esclave</ets>, D. <ets>slaaf</ets>, Dan. <ets>slave</ets>, <ets>sclave</ets>, Sw. <ets>slaf</ets>, all fr. G. <ets>sklave</ets>, MHG. also <ets>slave</ets>, from the national name of the Slavonians, or Sclavonians (in LL. <ets>Slavi</ets> or <ets>Sclavi</ets>), who were frequently made slaves by the Germans. See <er>Slav</er>.]</ety> <sn>1.</sn> <def>A person who is held in bondage to another; one who is wholly subject to the will of another; one who is held as a chattel; one who has no freedom of action, but whose person and services are wholly under the control of another.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Art thou our <qex>slave</qex>,<br/
Our captive, at the public mill our drudge?</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>One who has lost the power of resistance; one who surrenders himself to any power whatever; <as>as, a <ex>slave</ex> to passion, to lust, to strong drink, to ambition</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>A drudge; one who labors like a slave.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>An abject person; a wretch.</def>  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Slave ant</b></col> <fld>(Zool.)</fld>, <cd>any species of ants which is captured and enslaved by another species, especially <spn>Formica fusca</spn> of Europe and America, which is commonly enslaved by <spn>Formica sanguinea</spn>.</cd> -- <col><b>Slave catcher</b></col>, <cd>one who attempted to catch and bring back a fugitive slave to his master.</cd> -- <col><b>Slave coast</b></col>, <cd>part of the western coast of Africa to which slaves were brought to be sold to foreigners.</cd> -- <col><b>Slave driver</b></col>, <cd>one who superintends slaves at their work; hence, figuratively, a cruel taskmaster.</cd> -- <col><b>Slave hunt</b></col>. <sd>(a)</sd> <cd>A search after persons in order to reduce them to slavery</cd>. <au>Barth.</au> <sd>(b)</sd> <cd>A search after fugitive slaves, often conducted with bloodhounds.</cd> -- <col><b>Slave ship</b></col>, <cd>a vessel employed in the slave trade or used for transporting slaves; a slaver.</cd> -- <col><b>Slave trade</b></col>, <cd>the business of dealing in slaves, especially of buying them for transportation from their homes to be sold elsewhere.</cd> -- <col><b>Slave trader</b></col>, <cd>one who traffics in slaves.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><syn><b>Syn.</b> -- Bond servant; bondman; bondslave; captive; henchman; vassal; dependent; drudge. See <er>Serf</er>.</syn><br/
[<source>1913 Webster</source>]</p>

<p><ent>Slave</ent><br/
<hw>Slave</hw>, <pos>v. i.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Slaved</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Slaving</conjf>.]</vmorph> <def>To drudge; to toil; to labor as a slave.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Slave</ent><br/
<hw>Slave</hw>, <pos>v. t.</pos> <def>To enslave.</def>  <rj><au>Marston.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Slaveborn</ent><br/
<hw>Slave"born`</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Born in slavery.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Slaveholder</ent><br/
<hw>Slave"hold`er</hw> <pr>(?)</pr>, <pos>n.</pos> <def>One who holds slaves.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Slaveholding</ent><br/
<hw>Slave"hold`ing</hw>, <pos>a.</pos> <def>Holding persons in slavery.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Slaveocracy</ent><br/
<hw>Slave*oc"ra*cy</hw> <pr>(?)</pr>, <pos>n.</pos> <def>See <er>Slavocracy</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Slaver</ent><br/
<hw>Slav"er</hw> <pr>(?)</pr>, <pos>n.</pos> <sn>1.</sn> <def>A vessel engaged in the slave trade; a slave ship.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A person engaged in the purchase and sale of slaves; a slave merchant, or slave trader.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The <qex>slaver's</qex> hand was on the latch,<br/
He seemed in haste to go.</q> <rj><qau>Longfellow.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Slaver</ent><br/
<hw>Slav"er</hw> <pr>(?)</pr>, <pos>v. i.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Slavered</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Slavering</conjf>.]</vmorph> <ety>[Cf. Icel. <ets>slafra</ets>. See <er>Slabber</er>.]</ety> <sn>1.</sn> <def>To suffer spittle, etc., to run from the mouth.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To be besmeared with saliva.</def>  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Slaver</ent><br/
<hw>Slav"er</hw>, <pos>v. t.</pos> <def>To smear with saliva issuing from the mouth; to defile with drivel; to slabber.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Slaver</ent><br/
<hw>Slav"er</hw>, <pos>n.</pos> <def>Saliva driveling from the mouth.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Of all mad creatures, if the learned are right,<br/
It is the <qex>slaver</qex> kills, and not the bite.</q> <rj><qau>Pope.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Slaverer</ent><br/
<hw>Slav"er*er</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A driveler; an idiot.</def><br/
[<source>1913 Webster</source>]</p>

<p><-- p. 1353 --></p>

<p><ent>Slavering</ent><br/
<hw>Slav"er*ing</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Drooling; defiling with saliva.</def> -- <wordforms><wf>Slav"er*ing*ly</wf>, <pos>adv.</pos></wordforms><br/
[<source>1913 Webster</source>]</p>

<p><ent>Slavery</ent><br/
<hw>Slav"er*y</hw> <pr>(?)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Slaveries</plw> <pr>(#)</pr>.</plu> <ety>[See  2d <er>Slave</er>.]</ety> <sn>1.</sn> <def>The condition of a slave; the state of entire subjection of one person to the will of another.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Disguise thyself as thou wilt, still, <qex>slavery</qex>, said I, still thou art a bitter draught!</q> <rj><qau>Sterne.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>I wish, from my soul, that the legislature of this state [Virginia] could see the policy of a gradual abolition of <qex>slavery</qex>. It might prevent much future mischief.</q> <rj><qau>Washington.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A condition of subjection or submission characterized by lack of freedom of action or of will.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The vulgar <qex>slaveries</qex> rich men submit to.</q> <rj><qau>C. Lever.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>There is a <qex>slavery</qex> that no legislation can abolish, -- the <qex>slavery</qex> of caste.</q> <rj><qau>G. W. Cable.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>The holding of slaves.</def><br/
[<source>1913 Webster</source>]</p>

<p><syn><b>Syn.</b> -- Bondage; servitude; inthrallment; enslavement; captivity; bond service; vassalage.</syn><br/
[<source>1913 Webster</source>]</p>

<p><ent>Slavey</ent><br/
<hw>Slav"ey</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A maidservant.</def> <mark>[Colloq. & Jocose Eng.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Slavic</ent><br/
<hw>Slav"ic</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Slavonic.</def> -- <def2><pos>n.</pos> <def>The group of allied languages spoken by the Slavs.</def></def2><br/
[<source>1913 Webster</source>]</p>

<p><ent>Slavish</ent><br/
<hw>Slav"ish</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Of or pertaining to slaves; such as becomes or befits a slave; servile; excessively laborious; <as>as, a <ex>slavish</ex> life; a <ex>slavish</ex> dependance on the great.</as></def> -- <wordforms><wf>Slav"ish*ly</wf>, <pos>adv.</pos> -- <wf>Slav"ish*ness</wf>, <pos>n.</pos></wordforms><br/
[<source>1913 Webster</source>]</p>

<p><ent>Slavism</ent><br/
<hw>Slav"ism</hw> <pr>(?)</pr>, <pos>n.</pos> <def>The common feeling and interest of the Slavonic race.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Slavocracy</ent><br/
<hw>Slav*oc"ra*cy</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[<ets>Slave</ets> + <ets>-cracy</ets>, as in <ets>democracy</ets>.]</ety> <def>The persons or interest formerly representing slavery politically, or wielding political power for the preservation or advancement of slavery.</def> <mark>[U. S.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Slavonic</ent><br/
<ent>Slavonian</ent><br/
<mhw>{ <hw>Sla*vo"ni*an</hw> <pr>(?)</pr>, <hw>Sla*von"ic</hw> <pr>(?)</pr>, }</mhw> <pos>a.</pos> <sn>1.</sn> <def>Of or pertaining to Slavonia, or its inhabitants.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Of or pertaining to the Slavs, or their language.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Slavonian</ent><br/
<hw>Sla*vo"ni*an</hw>, <pos>n.</pos> <def>A native or inhabitant of Slavonia; ethnologically, a Slav.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Slavophile</ent><br/
<ent>Slavophil</ent><br/
<mhw>{ <hw>Slav"o*phil</hw> <pr>(?)</pr>, <hw>Slav"o*phile</hw> <pr>(?)</pr>, }</mhw> <pos>n.</pos> <ety>[<ets>Slavic</ets> + Gr. <?/<?/<?/ loving.]</ety> <def>One, not being a Slav, who is interested in the development and prosperity of that race.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Slaw</ent><br/
<hw>Slaw</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[D. <ets>sla</ets>, contr. fr. <ets>salade</ets>, OD. <ets>salaet</ets>, <ets>salad</ets>. See <er>Salad</er>.]</ety> <def>Sliced cabbage served as a salad, cooked or uncooked.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Slawen</ent><br/
<ent>Slaw</ent><br/
<mhw>{ <hw>Slaw</hw>, <hw>Slaw"en</hw>  }</mhw> <pr>(?)</pr>, <mark>obs.</mark> <def><pos>p. p.</pos> of <er>Slee</er>, to slay.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>With a sword drawn out he would have <qex>slaw</qex> himself.</q> <rj><qau>Wyclif (Acts xvi. 27.)</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Slay</ent><br/
<hw>Slay</hw> <pr>(?)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp.</pos> <conjf>Slew</conjf> <pr>(?)</pr>; <pos>p. p.</pos> <conjf>Slain</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Slaying</conjf>.]</vmorph> <ety>[OE. <ets>slan</ets>, <ets>sl<?/n</ets>, <ets>sleen</ets>, <ets>slee</ets>, AS. <ets>sle<aacute/n</ets> to strike, beat, slay; akin to OFries. <ets>sl<amac/</ets>, D. <ets>slaan</ets>, OS. & OHG. <ets>slahan</ets>, G. <ets>schlagen</ets>, Icel. <ets>sl<amac/</ets>, Dan. <ets>slaae</ets>, Sw. <ets>sl<?/</ets>, Goth. <ets>slahan</ets>; perhaps akin to L. <ets>lacerare</ets> to tear to pieces, Gr. <?/<?/<?/<?/, E. <ets>lacerate</ets>.  Cf. <er>Slaughter</er>, <er>Sledge</er> a hammer, <er>Sley</er>.]</ety> <def>To put to death with a weapon, or by violence; hence, to kill; to put an end to; to destroy.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>With this sword then will I <qex>slay</qex> you both.</q> <rj><qau>Chaucer.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>I will <qex>slay</qex> the last of them with the sword.</q> <rj><qau>Amos ix. 1.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>I'll <qex>slay</qex> more gazers than the basilisk.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><syn><b>Syn.</b> -- To kill; murder; slaughter; butcher.</syn><br/
[<source>1913 Webster</source>]</p>

<p><ent>Slayer</ent><br/
<hw>Slay"er</hw> <pr>(?)</pr>, <pos>n.</pos> <def>One who slays; a killer; a murderer; a destroyer of life.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Slazy</ent><br/
<hw>Sla"zy</hw> <pr>(sl<amac/"z<ycr/)</pr>, <pos>a.</pos> <def>See <er>Sleazy</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sle</ent><br/
<hw>Sle</hw> <pr>(sl<emac/)</pr>, <pos>v. t.</pos> <def>To slay.</def> <mark>[Obs.]</mark>  <rj><au>Chaucer.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sleave</ent><br/
<hw>Sleave</hw> <pr>(sl<emac/v)</pr>, <pos>n.</pos> <ety>[Cf. Dan. <ets>sl<oum/if</ets>, a knot loop, Sw. <ets>slejf</ets>, G. <ets>schleife</ets> a knot, sliding knot, and E. <ets>slip</ets>, v.i.]</ety> <sd>(a)</sd> <def>The knotted or entangled part of silk or thread.</def> <sd>(b)</sd> <def>Silk not yet twisted; floss; -- called also <altname>sleave silk</altname>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Sleep that knits up the ravell'd <qex>sleave</qex> of care.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sleave</ent><br/
<hw>Sleave</hw>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Sleaved</conjf> <pr>(sl<emac/vd)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Sleaving</conjf>.]</vmorph> <def>To separate, as threads; to divide, as a collection of threads; to sley; -- a weaver's term.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sleaved</ent><br/
<hw>Sleaved</hw> <pr>(sl<emac/vd)</pr>, <pos>a.</pos> <def>Raw; not spun or wrought; <as>as, <ex>sleaved</ex> thread or silk</as>.</def>  <rj><au>Holinshed.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sleaziness</ent><br/
<hw>Slea"zi*ness</hw> <pr>(?)</pr>, <pos>n.</pos> <def>Quality of being sleazy.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sleazy</ent><br/
<hw>Slea"zy</hw> <pr>(sl<emac/"z<ycr/)</pr>, <pos>a.</pos> <ety>[Cf. G. <ets>schleissig</ets> worn out, threadbare, from <ets>schleissen</ets> to slit, split, decay, or E. <ets>leasy</ets>.]</ety> <def>Lacking firmness of texture or substance; thin; flimsy; <as>as, <ex>sleazy</ex> silk or muslin</as>.</def> <altsp>[Spelt also <asp>slazy</asp>.]</altsp><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sled</ent><br/
<hw>Sled</hw> <pr>(sl<ecr/d)</pr>, <pos>n.</pos> <ety>[Akin to D. <ets>slede</ets>, G. <ets>schlitten</ets>, OHG. <ets>slito</ets>, Icel. <ets>sle<edh/i</ets>, Sw. <ets>sl<aum/de</ets>, Dan. <ets>sl<ae/de</ets>, and E. <ets>slide</ets>, v. See <er>Slide</er>, and cf. <er>Sledge</er> a vehicle, <er>Sleigh</er>.]</ety> <sn>1.</sn> <def>A vehicle on runners, used for conveying loads over the snow or ice; -- in England called <altname>sledge</altname>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A small, light vehicle with runners, used, mostly by young persons, for sliding on snow or ice.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sled</ent><br/
<hw>Sled</hw>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Sledded</conjf>; <pos>p. pr. & vb. n.</pos> <conjf>Sledding</conjf>.]</vmorph> <def>To convey or transport on a sled; <as>as, to <ex>sled</ex> wood or timber</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sledding</ent><br/
<hw>Sled"ding</hw> <pr>(?)</pr>, <pos>n.</pos> <sn>1.</sn> <def>The act of transporting or riding on a sled.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>The state of the snow which admits of the running of sleds; <as>as, the <ex>sledding</ex> is good</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sledge</ent><br/
<hw>Sledge</hw> <pr>(sl<ecr/j)</pr>, <pos>n.</pos> <ety>[Perhaps from <ets>sleds</ets>, pl. of <ets>sled</ets>, confused with <ets>sledge</ets> a hammer. See <er>Sled</er>, <pos>n.</pos>]</ety> <sn>1.</sn> <def>A strong vehicle with low runners or low wheels; or one without wheels or runners, made of plank slightly turned up at one end, used for transporting loads upon the snow, ice, or bare ground; a sled.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A hurdle on which, formerly, traitors were drawn to the place of execution.</def> <mark>[Eng.]</mark>  <rj><au>Sir W. Scott.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>A sleigh.</def> <mark>[Eng.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>A game at cards; -- called also <altname>old sledge</altname>, and <altname>all fours</altname>.</def>
<-- also called seven-up. See def above. --><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sledge</ent><br/
<hw>Sledge</hw> <pr>(sl<ecr/j)</pr>, <pos>v. i. & t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Sledged</conjf> <pr>(sl<ecr/jd)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Sledging</conjf>.]</vmorph> <def>To travel or convey in a sledge or sledges.</def>  <rj><au>Howitt.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sledge</ent><br/
<hw>Sledge</hw>, <pos>n.</pos> <ety>[AS. <ets>slecge</ets>,from <ets>sle<aacute/n</ets> to strike, beat. See <er>Slay</er>, <pos>v. t.</pos>]</ety> <def>A large, heavy hammer, usually wielded with both hands; -- called also <altname>sledge hammer</altname>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>With his heavy <qex>sledge</qex> he can it beat.</q> <rj><qau>Spenser.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Slee</ent><br/
<hw>Slee</hw> <pr>(sl<emac/)</pr>, <pos>v. t.</pos> <ety>[See <er>Slay</er>.]</ety> <def>To slay.</def> <mark>[Obs.]</mark>  <rj><au>Chaucer.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sleek</ent><br/
<hw>Sleek</hw> <pr>(sl<emac/k)</pr>, <pos>a.</pos> <amorph>[<pos>Compar.</pos> <adjf>Sleeker</adjf> <pr>(sl<emac/k"<etil/r)</pr>; <pos>superl.</pos> <adjf>Sleekest</adjf>.]</amorph> <ety>[OE. <ets>slik</ets>; akin to Icel. <ets>sl<imac/kr</ets>, and OE. <ets>sliken</ets> to glide, slide, G. <ets>schleichen</ets>, OHG. <ets>sl<imac/hhan</ets>, D. <ets>slik</ets>, <ets>slijk</ets>, mud, slime, and E. <ets>slink</ets>.  Cf. <er>Slick</er>, <er>Slink</er>.]</ety> <sn>1.</sn> <def>Having an even, smooth surface; smooth; hence, glossy; <as>as, <ex>sleek</ex> hair</as>.</def>  <rj><au>Chaucer.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>So <qex>sleek</qex> her skin, so faultless was her make.</q> <rj><qau>Dryden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Not rough or harsh.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Those rugged names to our like mouths grow <qex>sleek</qex>.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sleek</ent><br/
<hw>Sleek</hw>, <pos>adv.</pos> <def>With ease and dexterity.</def> <mark>[Low]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sleek</ent><br/
<hw>Sleek</hw>, <pos>n.</pos> <sn>1.</sn> <def>That which makes smooth; varnish.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A slick.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Sleek</ent><br/
<hw>Sleek</hw>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Sleeked</conjf> <pr>(?)</pr>;<pos>p. pr. & vb. n.</pos> <conjf>Sleeking</conjf>.]</vmorph> <def>To make even and smooth; to render smooth, soft, and glossy; to smooth over.</def><br/
[<source>1913 Webster</source>]</p>

<p><q><qex>Sleeking</qex> her soft alluring locks.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Gentle, my lord, <qex>sleek</qex> o'er your rugged looks.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sleekly</ent><br/
<hw>Sleek"ly</hw>, <pos>adv.</pos> <def>In a sleek manner; smoothly.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sleekness</ent><br/
<hw>Sleek"ness</hw>, <pos>n.</pos> <def>The quality or state of being sleek; smoothness and glossiness of surface.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sleeky</ent><br/
<hw>Sleek"y</hw> <pr>(?)</pr>, <pos>a.</pos> <sn>1.</sn> <def>Of a sleek, or smooth, and glossy appearance.</def>  <rj><au>Thomson.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Fawning and deceitful; sly.</def> <mark>[Scot.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sleep</ent><br/
<hw>Sleep</hw> <pr>(?)</pr>, <mark>obs.</mark> <def><pos>imp.</pos> of Sleep. Slept.</def>  <rj><au>Chaucer.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sleep</ent><br/
<hw>Sleep</hw>, <pos>v. i.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Slept</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Sleeping</conjf>.]</vmorph> <ety>[OE. <ets>slepen</ets>, AS. <ets>sl<?/pan</ets>; akin to OFries. <ets>sl<?/pa</ets>, OS. <ets>sl<amac/pan</ets>, D. <ets>slapen</ets>, OHG. <ets>sl<amac/fan</ets>, G. <ets>schlafen</ets>, Goth. <ets>sl<?/pan</ets>, and G. <ets>schlaff</ets> slack, loose, and L. <ets>labi</ets> to glide, slide, <ets>labare</ets> to totter.  Cf. <er>Lapse</er>.]</ety> <sn>1.</sn> <def>To take rest by a suspension of the voluntary exercise of the powers of the body and mind, and an apathy of the organs of sense; to slumber.</def>  <rj><au>Chaucer.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Watching at the head of these that <qex>sleep</qex>.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> Figuratively: <sd>(a)</sd> <def>To be careless, inattentive, or uncouncerned; not to be vigilant; to live thoughtlessly.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>We <qex>sleep</qex> over our happiness.</q> <rj><qau>Atterbury.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sd>(b)</sd> <def>To be dead; to lie in the grave.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Them also which <qex>sleep</qex> in Jesus will God bring with him.</q> <rj><qau>1 Thess. iv. 14.  </qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sd>(c)</sd> <def>To be, or appear to be, in repose; to be quiet; to be unemployed, unused, or unagitated; to rest; to lie dormant; <as>as, a question <ex>sleeps</ex> for the present; the law <ex>sleeps</ex></as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>How sweet the moonlight <qex>sleep</qex> upon this bank!</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sleep</ent><br/
<hw>Sleep</hw>, <pos>v. t.</pos> <sn>1.</sn> <def>To be slumbering in; -- followed by a cognate object; <as>as, to <ex>sleep</ex> a dreamless sleep</as>.</def>  <rj><au>Tennyson.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To give sleep to; to furnish with accomodations for sleeping; to lodge.</def> <mark>[R.]</mark>  <rj><au>Blackw. Mag.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>To sleep away</b></col>, <cd>to spend in sleep; <as>as, <ex>to sleep away</ex> precious time</as>.</cd> -- <col><b>To sleep off</b></col>, <cd>to become free from by sleep; <as>as, <ex>to sleep off</ex> drunkeness or fatigue</as>.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sleep</ent><br/
<hw>Sleep</hw>, <pos>n.</pos> <ety>[AS. <ets>sl<aemac/p</ets>; akin to OFries. <ets>sl<emac/p</ets>, OS. <ets>sl<amac/p</ets>, D. <ets>slaap</ets>, OHG. <ets>sl<amac/f</ets>, G. <ets>schlaf</ets>, Goth. <ets>sl<emac/ps</ets>. See <er>Sleep</er>, <pos>v. i.</pos>]</ety> <def>A natural and healthy, but temporary and periodical, suspension of the functions of the organs of sense, as well as of those of the voluntary and rational soul; that state of the animal in which there is a lessened acuteness of sensory perception, a confusion of ideas, and a loss of mental control, followed by a more or less unconscious state.</def> <ldquo/A man that waketh of his <xex>sleep</xex>.<rdquo/  <rj><au>Chaucer.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>O <qex>sleep</qex>, thou ape of death.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ Sleep is attended by a relaxation of the muscles, and the absence of voluntary activity for any rational objects or purpose. The pulse is slower, the respiratory movements fewer in number but more profound, and there is less blood in the cerebral vessels. It is susceptible of greater or less intensity or completeness in its control of the powers.</note>
<-- the dreaming portions of sleep occurs periodically, and is associated with "rapid eye movements" (REM), and in this state the sleeper is more easily wakened; the dreamiong alternates with a more profound sleep, from which it is more difficult to awake the sleeper. --><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Sleep of plants</b></col> <fld>(Bot.)</fld>, <cd>a state of plants, usually at night, when their leaflets approach each other, and the flowers close and droop, or are covered by the folded leaves.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><syn><b>Syn.</b> -- Slumber; repose; rest; nap; doze; drowse.</syn><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sleep-at-noon</ent><br/
<hw>Sleep"-at-noon"</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Bot.)</fld> <def>A plant (<spn>Tragopogon pratensis</spn>) which closes its flowers at midday; a kind of goat's beard.</def>  <rj><au>Dr. Prior.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sleep-charged</ent><br/
<hw>Sleep"-charged`</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Heavy with sleep.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sleeper</ent><br/
<hw>Sleep"er</hw> <pr>(?)</pr>, <pos>n.</pos> <sn>1.</sn> <def>One who sleeps; a slumberer; hence, a drone, or lazy person.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>That which lies dormant, as a law.</def> <mark>[Obs.]</mark>  <rj><au>Bacon.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>A sleeping car.</def> <mark>[Colloq. U.S.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <fld>(Zool.)</fld> <def>An animal that hibernates, as the bear.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <fld>(Zool.)</fld> <sd>(a)</sd> <def>A large fresh-water gobioid fish (<spn>Eleotris dormatrix</spn>).</def> <sd>(b)</sd> <def>A nurse shark. See under <er>Nurse</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sleeper</ent><br/
<hw>Sleep"er</hw>, <pos>n.</pos> <ety>[Cf. Norw. <ets>sleip</ets> a sleeper (a timber), as adj., slippery, smooth. See <er>Slape</er>.]</ety> <def>Something lying in a reclining posture or position.</def> Specifically: --<br/
[<source>1913 Webster</source>]</p>

<p><sd>(a)</sd> <def>One of the pieces of timber, stone, or iron, on or near the level of the ground, for the support of some superstructure, to steady framework, to keep in place the rails of a railway, etc.; a stringpiece.</def><br/
[<source>1913 Webster</source>]</p>

<p><sd>(b)</sd> <def>One of the joists, or roughly shaped timbers, laid directly upon the ground, to receive the flooring of the ground story.</def> <mark>[U.S.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><sd>(c)</sd> <fld>(Naut.)</fld> <def>One of the knees which connect the transoms to the after timbers on the ship's quarter.</def><br/
[<source>1913 Webster</source>]</p>

<p><sd>(d)</sd> <fld>(Naut.)</fld> <def>The lowest, or bottom, tier of casks.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sleepful</ent><br/
<hw>Sleep"ful</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Strongly inclined to sleep; very sleepy.</def> -- <wordforms><wf>Sleep"ful*ness</wf>, <pos>n.</pos></wordforms><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sleepily</ent><br/
<hw>Sleep"i*ly</hw> <pr>(?)</pr>, <pos>adv.</pos> <def>In a sleepy manner; drowsily.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sleepiness</ent><br/
<hw>Sleep"i*ness</hw>, <pos>n.</pos> <def>The quality or state of being sleepy.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sleeping</ent><br/
<hw>Sleep"ing</hw>, <def><pos>a. & n.</pos> from <er>Sleep</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Sleeping car</b></col>, <cd>a railway car or carrriage, arranged with apartments and berths for sleeping.</cd> -- <col><b>Sleeping partner</b></col> <fld>(Com.)</fld>, <cd>a dormant partner. See under <er>Dormant</er>.</cd> -- <col><b>Sleeping table</b></col> <fld>(Mining)</fld>, <cd>a stationary inclined platform on which pulverized ore is washed; a kind of buddle.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sleepish</ent><br/
<hw>Sleep"ish</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Disposed to sleep; sleepy; drowsy.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Your <qex>sleepish</qex>, and more than <qex>sleepish</qex>, security.</q> <rj><qau>Ford.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sleepless</ent><br/
<hw>Sleep"less</hw>, <pos>a.</pos> <sn>1.</sn> <def>Having no sleep; wakeful.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Having no rest; perpetually agitated.</def> <ldquo/Biscay's <xex>sleepless</xex> bay.<rdquo/  <rj><au>Byron.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p>-- <wordforms><wf>Sleep"less*ly</wf>, <pos>adv.</pos> -- <wf>Sleep"less*ness</wf>, <pos>n.</pos></wordforms><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sleepmarken</ent><br/
<hw>Sleep"mark`en</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Zool.)</fld> <def>See 1st <er>Hag</er>, 4.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sleepwaker</ent><br/
<hw>Sleep"wak`er</hw> <pr>(?)</pr>, <pos>n.</pos> <def>On in a state of magnetic or mesmeric sleep.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sleepwaking</ent><br/
<hw>Sleep"wak`ing</hw>, <pos>n.</pos> <def>The state of one mesmerized, or in a partial and morbid sleep.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sleepwalker</ent><br/
<hw>Sleep"walk`er</hw> <pr>(?)</pr>, <pos>n.</pos> <def>One who walks in his sleep; a somnambulist.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sleepwalking</ent><br/
<hw>Sleep"walk`ing</hw>, <pos>n.</pos> <def>Walking in one's sleep.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sleepy</ent><br/
<hw>Sleep"y</hw> <pr>(?)</pr>, <pos>a.</pos> <amorph>[<pos>Compar.</pos> <adjf>Sleepier</adjf> <pr>(?)</pr>; <pos>superl.</pos> <adjf>Sleepiest</adjf>.]</amorph> <ety>[AS. <ets>sl<?/pig</ets>. See <er>Sleep</er>, <pos>n.</pos>]</ety> <sn>1.</sn> <def>Drowsy; inclined to, or overcome by, sleep.</def>  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>She waked her <qex>sleepy</qex> crew.</q> <rj><qau>Dryden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Tending to induce sleep; soporiferous; somniferous; <as>as, a <ex>sleepy</ex> drink or potion</as>.</def>  <rj><au>Chaucer.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>Dull; lazy; heavy; sluggish.</def>  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>'Tis not <qex>sleepy</qex> business;<br/
But must be looked to speedily and strongly.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>Characterized by an absence of watchfulness; <as>as, <ex>sleepy</ex> security</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Sleepy duck</b></col> <fld>(Zool.)</fld>, <cd>the ruddy duck.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sleepyhead</ent><br/
<hw>Sleep"y*head`</hw> <pr>(?)</pr>, <pos>n.</pos> <sn>1.</sn> <def>A sleepy person.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>To bed, to bed, says <qex>Sleepyhead</qex>.</q> <rj><qau>Mother Goose.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Zool.)</fld> <def>The ruddy duck.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sleer</ent><br/
<hw>Sle"er</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A slayer.</def> <mark>[Obs.]</mark>  <rj><au>Chaucer.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sleet</ent><br/
<hw>Sleet</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Gun.)</fld> <def>The part of a mortar extending from the chamber to the trunnions.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sleet</ent><br/
<hw>Sleet</hw>, <pos>n.</pos> <ety>[OE. <ets>sleet</ets>; akin to MHG. <ets>sl<?/z</ets>, <ets>sl<?/ze</ets> hailstone, G. <ets>schlosse</ets>; of uncertain origin.]</ety> <def>Hail or snow, mingled with rain, usually falling, or driven by the wind, in fine particles.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sleet</ent><br/
<hw>Sleet</hw>, <pos>v. i.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Sleeted</conjf>; <pos>p. pr. & vb. n.</pos> <conjf>Sleeting</conjf>.]</vmorph> <def>To snow or hail with a mixture of rain.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sleetch</ent><br/
<hw>Sleetch</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Cf. <er>Slush</er>, <er>Slutch</er>.]</ety> <def>Mud or slime, such as that at the bottom of rivers.</def> <mark>[Scot.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sleetiness</ent><br/
<hw>Sleet"i*ness</hw> <pr>(?)</pr>, <pos>n.</pos> <def>The state of being sleety.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sleety</ent><br/
<hw>Sleet"y</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Of or pertaining to sleet; characterized by sleet; <as>as, a <ex>sleety</ex> storm; <ex>sleety</ex> weather</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sleeve</ent><br/
<hw>Sleeve</hw> <pr>(sl<emac/v)</pr>, <pos>n.</pos> <def>See <er>Sleave</er>, untwisted thread.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sleeve</ent><br/
<hw>Sleeve</hw>, <pos>n.</pos> <ety>[OE. <ets>sleeve</ets>, <ets>sleve</ets>, AS. <ets>sl<?/fe</ets>, <ets>sl<?/fe</ets>; akin to <ets>sl<?/fan</ets> to put on, to clothe; cf. OD. <ets>sloove</ets> the turning up of anything, <ets>sloven</ets> to turn up one's sleeves, <ets>sleve</ets> a sleeve, G. <ets>schlaube</ets> a husk, pod.]</ety> <sn>1.</sn> <def>The part of a garment which covers the arm; <as>as, the <ex>sleeve</ex> of a coat or a gown</as>.</def>  <rj><au>Chaucer.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A narrow channel of water.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>The Celtic Sea, called oftentimes the <qex>Sleeve</qex>.</q> <rj><qau>Drayton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <fld>(Mach.)</fld> <sd>(a)</sd> <def>A tubular part made to cover, sustain, or steady another part, or to form a connection between two parts.</def> <sd>(b)</sd> <def>A long bushing or thimble, as in the nave of a wheel.</def> <sd>(c)</sd> <def>A short piece of pipe used for covering a joint, or forming a joint between the ends of two other pipes.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <fld>(Elec.)</fld> <def>A double tube of copper, in section like the figure 8, into which the ends of bare wires are pushed so that when the tube is twisted an electrical connection is made. The joint thus made is called <col><b>a McIntire joint</b></col>.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><cs><col><b>Sleeve button</b></col>, <cd>a detachable button to fasten the wristband or cuff.</cd> -- <col><b>Sleeve links</b></col>, <cd>two bars or buttons linked together, and used to fasten a cuff or wristband.</cd> -- <mcol><col><b>To laugh in the sleeve</b></col> <it>or</it> <col><b>To laugh up one's sleeve</b></col></mcol> <cd>to laugh privately or unperceived, especially while apparently preserving a grave or serious demeanor toward the person or persons laughed at; that is, perhaps, originally, by hiding the face in the wide sleeves of former times.</cd> -- <mcol><col><b>To pinon the sleeve of</b></col>, <it>or</it> <col><b>To hang on the sleeve of</b></col></mcol>, <cd>to be, or make, dependent upon.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sleeve</ent><br/
<hw>Sleeve</hw>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Sleeved</conjf> <pr>(sl<emac/vd)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Sleeving</conjf>.]</vmorph> <def>To furnish with sleeves; to put sleeves into; <as>as, to <ex>sleeve</ex> a coat</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sleeved</ent><br/
<hw>Sleeved</hw> <pr>(sl<emac/vd)</pr>, <pos>a.</pos> <def>Having sleeves; furnished with sleeves; -- often in composition; <as>as, long-<ex>sleeved</ex></as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sleevefish</ent><br/
<hw>Sleeve"fish`</hw> <pr>(sl<emac/v"f<icr/sh`)</pr>, <pos>n.</pos> <fld>(Zool.)</fld> <def>A squid.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sleevehand</ent><br/
<hw>Sleeve"hand`</hw> <pr>(?)</pr>, <pos>n.</pos> <def>The part of a sleeve nearest the hand; a cuff or wristband.</def> <mark>[Obs.]</mark>  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sleeveless</ent><br/
<hw>Sleeve"less</hw>, <pos>a.</pos> <ety>[AS. <ets>sl<emac/fle<aacute/s</ets>.]</ety> <sn>1.</sn> <def>Having no sleeves.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Lacking a cover, pretext, or palliation; unreasonable; profitless; bootless; useless.</def> <mark>[Obs.]</mark>  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>The vexation of a <qex>sleeveless</qex> errand.</q> <rj><qau>Bp. Warburton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sleid</ent><br/
<hw>Sleid</hw> <pr>(?)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Sleided</conjf>; <pos>p. pr. & vb. n.</pos> <conjf>Sleiding</conjf>.]</vmorph> <ety>[See <er>Sley</er>.]</ety> <def>To sley, or prepare for use in the weaver's sley, or slaie.</def>  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sleigh</ent><br/
<hw>Sleigh</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Sly.</def> <mark>[Obs.]</mark>  <rj><au>Chaucer.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sleigh</ent><br/
<hw>Sleigh</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Cf. D. & LG. <ets>slede</ets>, <ets>slee</ets>, Icel. <ets>sle<?/i</ets>. See <er>Sled</er>.]</ety> <def>A vehicle moved on runners, and used for transporting persons or goods on snow or ice; -- in England commonly called a <xex>sledge</xex>.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Sleigh bell</b></col>, <cd>a small bell attached either to a horse when drawing a slegh, or to the sleigh itself; especially a globular bell with a loose ball which plays inside instead of a clapper.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sleighing</ent><br/
<hw>Sleigh"ing</hw>, <pos>n.</pos> <sn>1.</sn> <def>The act of riding in a sleigh.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>The state of the snow or ice which admits of running sleighs.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sleight</ent><br/
<hw>Sleight</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[OE. <ets>sleighte</ets>, <ets>sleihte</ets>, <ets>sleithe</ets>, Icel. <ets>sl<?/g<?/</ets> (for <ets>sl<?/g<?/</ets>) slyness, cunning, fr. <ets>sl<?/gr</ets> (for <ets>sl<?/gr</ets>) sly, cunning. See <er>Sly</er>.]</ety> <sn>1.</sn> <def>Cunning; craft; artful practice.</def> <mark>[Obs.]</mark> <ldquo/His <xex>sleight</xex> and his covin.<rdquo/  <rj><au>Chaucer.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>An artful trick; sly artifice; a feat so dexterous that the manner of performance escapes observation.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The world hath many subtle <qex>sleights</qex>.</q> <rj><qau>Latimer.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>Dexterous practice; dexterity; skill.</def> <au>Chaucer.</au> <ldquo/The juggler's <xex>sleight</xex>.<rdquo/  <rj><au>Hudibras.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Sleight of hand</b></col>, <cd>legerdemain; prestidigitation.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><-- p. 1354 --></p>

<p><ent>Sleightful</ent><br/
<hw>Sleight"ful</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Cunning; dexterous.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sleightly</ent><br/
<hw>Sleight"ly</hw> <pr>(?)</pr>, <pos>adv.</pos> <def>Cinningly.</def> <mark>[Obs.]</mark>  <rj><au>Huloet.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sleighty</ent><br/
<hw>Sleight"y</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Cinning; sly.</def> <mark>[Obs.]</mark>  <rj><au>Huloet.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Slender</ent><br/
<hw>Slen"der</hw> <pr>(?)</pr>, <pos>a.</pos> <amorph>[<pos>Compar.</pos> <adjf>Slenderer</adjf> <pr>(?)</pr>; <pos>superl.</pos> <adjf>Slenderest</adjf>.]</amorph> <ety>[OE. <ets>slendre</ets>, <ets>sclendre</ets>, fr. OD. <ets>slinder</ets> thin, slender, perhaps through a French form; cf. OD. <ets>slinderen</ets>, <ets>slidderen</ets>, to creep; perh. akin to E. <ets>slide</ets>.]</ety> <sn>1.</sn> <def>Small or narrow in proportion to the length or the height; not thick; slim; <as>as, a <ex>slender</ex> stem or stalk of a plant</as>.</def> <ldquo/A <xex>slender</xex>, choleric man.<rdquo/  <rj><au>Chaucer.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>She, as a veil down to the <qex>slender</qex> waist,<br/
Her unadorned golden tresses wore.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Weak; feeble; not strong; slight; <as>as, <ex>slender</ex> hope; a <ex>slender</ex> constitution</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Mighty hearts are held in <qex>slender</qex> chains.</q> <rj><qau>Pope.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>They have inferred much from <qex>slender</qex> premises.</q> <rj><qau>J. H. Newman.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>The <qex>slender</qex> utterance of the consonants.</q> <rj><qau>J. Byrne.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>Moderate; trivial; inconsiderable; slight; <as>as, a man of <ex>slender</ex> intelligence</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>A <qex>slender</qex> degree of patience will enable him to enjoy both the humor and the pathos.</q> <rj><qau>Sir W. Scott.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>Small; inadequate; meager; pitiful; <as>as, <ex>slender</ex> means of support; a <ex>slender</ex> pittance</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Frequent begging makes <qex>slender</qex> alms.</q> <rj><qau>Fuller.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <def>Spare; abstemious; frugal; <as>as, a <ex>slender</ex> diet</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The good Ostorius often deigned<br/
To grace my <qex>slender</qex> table with his presence.</q> <rj><qau>Philips.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>6.</sn> <fld>(Phon.)</fld> <def>Uttered with a thin tone; -- the opposite of <xex>broad</xex>; <as>as, the <ex>slender</ex> vowels long <ex>e</ex> and <ex>i</ex></as>.</def><br/
[<source>1913 Webster</source>]</p>

<p>-- <wordforms><wf>Slen"der*ly</wf>, <pos>adv.</pos> -- <wf>Slen"der*ness</wf>, <pos>n.</pos></wordforms><br/
[<source>1913 Webster</source>]</p>

<p><ent>Slent</ent><br/
<hw>Slent</hw> <pr>(?)</pr>, <pos>n. & v.</pos> <def>See <er>Slant</er>.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Slep</ent><br/
<hw>Slep</hw> <pr>(?)</pr>, <mark>obs.</mark> <def><pos>imp.</pos> of <er>Sleep</er>. Slept.</def>  <rj><au>Chaucer.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Slepez</ent><br/
<hw>Sle*pez"</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Russ. <ets>sliepets'</ets>.]</ety> <fld>(Zool.)</fld> <def>A burrowing rodent (<spn>Spalax typhlus</spn>), native of Russia and Asia Minor. It has the general appearance of a mole, and is destitute of eyes. Called also <altname>mole rat</altname>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Slept</ent><br/
<hw>Slept</hw> <pr>(?)</pr>, <def><pos>imp. & p. p.</pos> of <er>Sleep</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sleuth</ent><br/
<hw>Sleuth</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Icel. <ets>sl<omac/<edh/</ets>. See <er>Slot</er> a track.]</ety> <def>The track of man or beast as followed by the scent.</def> <mark>[Scot.]</mark>  <rj><au>Halliwell.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sleuthhound</ent><br/
<hw>Sleuth"hound`</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[See <er>Sleuth</er>, and cf. <er>Slothound</er>.]</ety> <fld>(Zool.)</fld> <def>A hound that tracks animals by the scent; specifically, a bloodhound.</def> <altsp>[Spelt variously <asp>slouthhound</asp>, <asp>sluthhound</asp>, etc.]</altsp><br/
[<source>1913 Webster</source>]</p>

<p><ent>Slew</ent><br/
<hw>Slew</hw> <pr>(?)</pr>, <def><pos>imp.</pos> of <er>Slay</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Slew</ent><br/
<hw>Slew</hw>, <pos>v. t.</pos> <def>See <er>Slue</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Slew</ent><br/
<hw>Slew</hw> <pr>(sl<oomac/)</pr>, <pos>n.</pos> <ety>[See <er>Slough</er> a wet place.]</ety> <def>A wet place; a river inlet.</def></p>

<p><q>The praire round about is wet, at times almost marshy, especially at the borders of the great reedy <qex>slews</qex>.</q>  <rj><qau>T. Roosevelt.</qau></rj><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Slewed</ent><br/
<hw>Slewed</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Somewhat drunk.</def> <mark>[Slang]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Slewth</ent><br/
<hw>Slewth</hw> <pr>(?)</pr>, <pos>n.</pos> <def>Sloth; idleness.</def> <mark>[Obs.]</mark>  <rj><au>Chaucer.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sley</ent><br/
<hw>Sley</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[AS. <ets>sl<?/</ets>, fr. <ets>sle<aacute/n</ets> to strike. See <er>Slay</er>, <pos>v. t.</pos>]</ety> <sn>1.</sn> <def>A weaver's reed.</def> <altsp>[Spelt also <asp>slaie</asp>.]</altsp><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A guideway in a knitting machine.</def>  <rj><au>Knight.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <fld>(Weaving)</fld> <def>The number of ends per inch in the cloth, provided each dent in the reed in which it was made contained an equal number of ends.</def>  <rj><au>E. Whitworth.</au></rj><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Sley</ent><br/
<hw>Sley</hw>, <pos>v. t.</pos> <def>To separate or part the threads of, and arrange them in a reed; -- a term used by weavers. See <er>Sleave</er>, and <er>Sleid</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Slibber</ent><br/
<hw>Slib"ber</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Slippery.</def> <mark>[Obs.]</mark>  <rj><au>Holland.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Slice</ent><br/
<hw>Slice</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[OE. <ets>slice</ets>, <ets>sclice</ets>, OF. <ets>esclice</ets>, from <ets>esclicier</ets>, <ets>esclichier</ets>, to break to pieces, of German origin; cf. OHG. <ets>sl<imac/zan</ets> to split, slit, tear, G. <ets>schleissen</ets> to slit. See <er>Slit</er>, <pos>v. t.</pos>]</ety> <sn>1.</sn> <def>A thin, broad piece cut off; <as>as, a <ex>slice</ex> of bacon; a <ex>slice</ex> of cheese; a <ex>slice</ex> of bread.</as></def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>That which is thin and broad, like a slice.</def> Specifically: <sd>(a)</sd> <def>A broad, thin piece of plaster.</def> <sd>(b)</sd> <def>A salver, platter, or tray.</def> <mark>[Obs.]</mark> <sd>(c)</sd> <def>A knife with a thin, broad blade for taking up or serving fish; also, a spatula for spreading anything, as paint or ink.</def> <sd>(d)</sd> <def>A plate of iron with a handle, forming a kind of chisel, or a spadelike implement, variously proportioned, and used for various purposes, as for stripping the planking from a vessel's side, for cutting blubber from a whale, or for stirring a fire of coals; a slice bar; a peel; a fire shovel.</def> <mark>[Cant]</mark> <sd>(e)</sd> <fld>(Shipbuilding)</fld> <def>One of the wedges by which the cradle and the ship are lifted clear of the building blocks to prepare for launching.</def> <sd>(f)</sd> <fld>(Printing)</fld> <def>A removable sliding bottom to galley.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Slice bar</b></col>, <cd>a kind of fire iron resembling a poker, with a broad, flat end, for stirring a fire of coals, and clearing it and the grate bars from clinkers, ashes, etc.; a slice.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Slice</ent><br/
<hw>Slice</hw>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Sliced</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Slicing</conjf> <pr>(?)</pr>.]</vmorph> <sn>1.</sn> <def>To cut into thin pieces, or to cut off a thin, broad piece from.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To cut into parts; to divide.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>To clear by means of a slice bar, as a fire or the grate bars of a furnace.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <fld>(Golf)</fld> <def>To hit (the ball) so that the face of the club draws across the face of the ball and deflects it.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Slicer</ent><br/
<hw>Sli"cer</hw> <pr>(?)</pr>, <pos>n.</pos> <def>One who, or that which, slices; specifically, the circular saw of the lapidary.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Slick</ent><br/
<ent>Slich</ent><br/
<mhw>{ <hw>Slich</hw> <pr>(?)</pr>, <hw>Slick</hw> <pr>(?)</pr>, }</mhw> <pos>n.</pos> <fld>(Metal.)</fld> <def>See <er>Schlich</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Slick</ent><br/
<hw>Slick</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[See <er>Sleek</er>.]</ety> <def>Sleek; smooth.</def> <ldquo/Both <xex>slick</xex> and dainty.<rdquo/  <rj><au>Chapman.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Slick</ent><br/
<hw>Slick</hw>, <pos>v. t.</pos> <def>To make sleek or smoth.</def> <ldquo/<xex>Slicked</xex> all with sweet oil.<rdquo/  <rj><au>Chapman.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Slick</ent><br/
<hw>Slick</hw>, <pos>n.</pos> <fld>(Joinery)</fld> <def>A wide paring chisel.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Slick</ent><br/
<hw>Slick</hw>, <pos>n.</pos> <def>A slick, or smooth and slippery, surface or place; a sleek.</def></p>

<p><q>The action of oil upon the water is upon the crest of the wave; the oil forming a <qex>slick</qex> upon the surface breaks the crest.</q>  <rj><qau>The Century.</qau></rj><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Slicken</ent><br/
<hw>Slick"en</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Sleek; smooth.</def> <mark>[Prov. Eng.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Slickens</ent><br/
<hw>Slick"ens</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Cf. <er>Slick</er>, <pos>n.</pos>]</ety> <fld>(Mining)</fld> <def>The pulverized matter from a quartz mill, or the lighter soil of hydraulic mines.</def> <mark>[Local, U. S.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Slickensides</ent><br/
<hw>Slick"en*sides`</hw> <pr>(?)</pr>, <pos>n.</pos> <sn>1.</sn> <def>The smooth, striated, or partially polished surfaces of a fissure or seam, supposed to have been produced by the sliding of one surface on another.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A variety of galena found in Derbyshire, England.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Slicker</ent><br/
<hw>Slick"er</hw> <pr>(?)</pr>, <pos>n.</pos> <def>That which makes smooth or sleek.</def> Specifically: <sd>(a)</sd> <def>A kind of burnisher for leather.</def> <sd>(b)</sd> <fld>(Founding)</fld> <def>A curved tool for smoothing the surfaces of a mold after the withdrawal of the pattern.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Slicker</ent><br/
<hw>Slick"er</hw>, <pos>n.</pos> <def>A waterproof coat.</def> <mark>[Western U.S.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Slicking</ent><br/
<hw>Slick"ing</hw>, <pos>n.</pos> <sn>1.</sn> <def>The act or process of smoothing.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <pluf>pl.</pluf> <fld>(Min.)</fld> <def>Narrow veins of ore.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Slickness</ent><br/
<hw>Slick"ness</hw>, <pos>n.</pos> <def>The state or quality of being slick; smoothness; sleekness.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Slid</ent><br/
<hw>Slid</hw> <pr>(?)</pr>, <def><pos>imp. & p. p.</pos> of <er>Slide</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Slidden</ent><br/
<hw>Slid"den</hw> <pr>(?)</pr>, <def><pos>p. p.</pos> of <er>Slide</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Slidder</ent><br/
<hw>Slid"der</hw> <pr>(?)</pr>, <pos>v. t.</pos> <ety>[AS. <ets>sliderian</ets>. See <er>Slide</er>, <pos>v. t.</pos>]</ety> <def>To slide with interruption.</def> <mark>[Obs.]</mark>  <rj><au>Dryden.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sliddery</ent><br/
<ent>Slidderly</ent><br/
<ent>Slidder</ent><br/
<mhw>{ <hw>Slid"der</hw>, <hw>Slid"der*ly</hw>, <hw>Slid"der*y</hw> <pr>(?)</pr>, }</mhw> <pos>a.</pos> <ety>[AS. <ets>slidor</ets>. See <er>Slide</er>, <pos>v. t.</pos>]</ety> <def>Slippery.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>To a drunk man the way is <qex>slidder</qex>.</q> <rj><qau>Chaucer.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Slide</ent><br/
<hw>Slide</hw> <pr>(?)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp.</pos> <conjf>Slid</conjf> <pr>(?)</pr>; <pos>p. p.</pos> <conjf>Slidden</conjf> <pr>(?)</pr>, <conjf>Slid</conjf>; <pos>p. pr. & vb. n.</pos> <conjf>Slidding</conjf> <pr>(?)</pr>.]</vmorph> <ety>[OE. <ets>sliden</ets>, AS. <ets>sl<imac/dan</ets>; akin to MHG. <ets>sl<imac/ten</ets>, also to AS. <ets>slidor</ets> slippery, E. <ets>sled</ets>, Lith. <ets>slidus</ets> slippery.  Cf. <er>Sled</er>.]</ety> <sn>1.</sn> <def>To move along the surface of any body by slipping, or without walking or rolling; to slip; to glide; <as>as, snow <ex>slides</ex> down the mountain's side</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Especially, to move over snow or ice with a smooth, uninterrupted motion, as on a sled moving by the force of gravity, or on the feet.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>They bathe in summer, and in winter <qex>slide</qex>.</q> <rj><qau>Waller.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>To pass inadvertently.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Beware thou <qex>slide</qex> not by it.</q> <rj><qau>Ecclus. xxviii. 26.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>To pass along smoothly or unobservedly; to move gently onward without friction or hindrance; <as>as, a ship or boat <ex>slides</ex> through the water</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Ages shall <qex>slide</qex> away without perceiving.</q> <rj><qau>Dryden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Parts answering parts shall <qex>slide</qex> into a whole.</q> <rj><qau>Pope.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <def>To slip when walking or standing; to fall.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Their foot shall <qex>slide</qex> in due time.</q> <rj><qau>Deut. xxxii. 35.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>6.</sn> <fld>(Mus.)</fld> <def>To pass from one note to another with no perceptible cassation of sound.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>7.</sn> <def>To pass out of one's thought as not being of any consequence.</def> <mark>[Obs. or Colloq.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>With good hope let he sorrow <qex>slide</qex>.</q> <rj><qau>Chaucer.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>With a calm carelessness letting everything <qex>slide</qex>.</q> <rj><qau>Sir P. Sidney.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Slide</ent><br/
<hw>Slide</hw>, <pos>v. t.</pos> <sn>1.</sn> <def>To cause to slide; to thrust along; <as>as, to <ex>slide</ex> one piece of timber along another</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To pass or put imperceptibly; to slip; <as>as, to <ex>slide</ex> in a word to vary the sense of a question</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Slide</ent><br/
<hw>Slide</hw>, <pos>n.</pos> <ety>[AS. <ets>sl<imac/de</ets>.]</ety> <sn>1.</sn> <def>The act of sliding; <as>as, a <ex>slide</ex> on the ice</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Smooth, even passage or progress.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>A better <qex>slide</qex> into their business.</q> <rj><qau>Bacon.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>That on which anything moves by sliding.</def> Specifically: <sd>(a)</sd> <def>An inclined plane on which heavy bodies slide by the force of gravity, esp. one constructed on a mountain side for conveying logs by sliding them down.</def> <sd>(b)</sd> <def>A surface of ice or snow on which children slide for amusement.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>That which operates by sliding.</def> Specifically: <sd>(a)</sd> <def>A cover which opens or closes an aperture by sliding over it.</def> <sd>(b)</sd> <fld>(Mach.)</fld> <def>A moving piece which is guided by a part or parts along which it slides.</def> <sd>(c)</sd> <def>A clasp or brooch for a belt, or the like.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <def>A plate or slip of glass on which is a picture or delineation to be exhibited by means of a magic lantern, stereopticon, or the like; a plate on which is an object to be examined with a microscope.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>6.</sn> <def>The descent of a mass of earth, rock, or snow down a hill or mountain side; <as>as, a land <ex>slide</ex>, or a snow <ex>slide</ex></as>; also, the track of bare rock left by a land slide.</def><-- also mudslide --><br/
[<source>1913 Webster</source>]</p>

<p><sn>7.</sn> <fld>(Geol.)</fld> <def>A small dislocation in beds of rock along a line of fissure.</def>  <rj><au>Dana.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>8.</sn> <fld>(Mus.)</fld> <sd>(a)</sd> <def>A grace consisting of two or more small notes moving by conjoint degrees, and leading to a principal note either above or below.</def> <sd>(b)</sd> <def>An apparatus in the trumpet and trombone by which the sounding tube is lengthened and shortened so as to produce the tones between the fundamental and its harmonics.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>9.</sn> <fld>(Phonetics)</fld> <def>A sound which, by a gradual change in the position of the vocal organs, passes imperceptibly into another sound.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>10.</sn> <fld>(Steam Engine)</fld> <sd>(a)</sd> <def>Same as <cref>Guide bar</cref>, under <er>Guide</er>.</def> <sd>(b)</sd> <def>A slide valve.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Slide box</b></col> <fld>(Steam Engine)</fld>, <cd>a steam chest. See under <er>Steam</er>.</cd> -- <col><b>Slide lathe</b></col>, <cd>an engine lathe. See under <er>Lathe</er>.</cd> -- <col><b>Slide rail</b></col>, <cd>a transfer table. See under <er>Transfer</er>.</cd> -- <col><b>Slide rest</b></col> <fld>(Turning lathes)</fld>, <cd>a contrivance for holding, moving, and guiding, the cutting tool, made to slide on ways or guides by screws or otherwise, and having compound motion.</cd> -- <col><b>Slide rule</b></col>, <cd>a mathematical instrument consisting of two parts, one of which slides upon the other, for the mechanical performance of addition and subtraction, and, by means of logarithmic scales, of multiplication and division.</cd> -- <col><b>Slide valve</b></col>. <sd>(a)</sd> <cd>Any valve which opens and closes a passageway by sliding over a port</cd>. <sd>(b)</sd> <cd>A particular kind of sliding valve, often used in steam engines for admitting steam to the piston and releasing it, alternately, having a cuplike cavity in its face, through which the exhaust steam passes. It is situated in the steam chest, and moved by the valve gear. It is sometimes called a <altname>D valve</altname>, -- a name which is also applied to a semicylindrical pipe used as a sliding valve.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><-- illustration of a slide valve -->
<illust>In the illustration, <xex>a</xex> is the cylinder of a steam engine, in which plays the piston <xex>p</xex>; <xex>b</xex> the steam chest, receiving its supply from the pipe <xex>i</xex>, and containing the slide valve <xex>s</xex>, which is shown as admitting steam to one end of the cylinder through the port <xex>e</xex>, and opening communication between the exhaust passage <xex>f</xex> and the port <xex>c</xex>, for the release of steam from the opposite end of the cylinder.</illust><br/
[<source>1913 Webster</source>]</p>

<p><ent>Slidegroat</ent><br/
<hw>Slide"groat</hw> <pr>(?)</pr>, <pos>n.</pos> <def>The game of shovelboard.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Slider</ent><br/
<hw>Slid"er</hw> <pr>(?)</pr>, <pos>a.</pos> <def>See <er>Slidder</er>.</def> <mark>[Obs.]</mark>  <rj><au>Chaucer.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Slider</ent><br/
<hw>Slid"er</hw>, <pos>n.</pos> <sn>1.</sn> <def>One who, or that which, slides; especially, a sliding part of an instrument or machine.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Zool.)</fld> <def>The red-bellied terrapin (<spn>Pseudemys rugosa</spn>).</def> <mark>[Local, U. S. ]</mark><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <fld>(Zool.)</fld> <def>any of several North American freshwater turtles of the genus <gen>Chrysemis</gen>; some, such as <spn>Chrysemis scripta</spn> are sold commercially as pets.</def> <au>[RHUD]</au><br/
[<source>PJC</source>]</p>

<p><sn>4.</sn> <fld>(Baseball)</fld> <def>a fast pitch that breaks slightly just in front of the batter, in the same direction as a curve ball (i. e. , away from the side from which it was thrown).</def><br/
[<source>PJC</source>]</p>

<p><cs><col><b>Slider pump</b></col>, <cd>a form of rotary pump.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Slide rule</ent><br/
<hw>Slide" rule`</hw>, <pos>n.</pos> <def>a thin, flat calculating device consisting of a fixed outer piece and a movable middle piece.  Both pieces are graduated in such a way (as, by a logarithmic scale) that multiplication, division, and other mathematical functions of an input variable may be rapidly determined by movement of the middle pieces to a location on one scale corresponding to the input value, and reading off the result on another scale.  A movable window with a hairline assists in alignment of the scales.  This device has been largely superseded by the electronic calculator, which has a greater precision than the slide rule.  Also called colloquially <altname>slipstick</altname>.</def><br/
[<source>PJC</source>]</p>

<p><ent>Slideway</ent><br/
<hw>Slide"way`</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A way along which something slides.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Sliding</ent><br/
<hw>Slid"ing</hw> <pr>(?)</pr>, <pos>a.</pos> <sn>1.</sn> <def>That slides or slips; gliding; moving smoothly.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Slippery; elusory.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>That <qex>sliding</qex> science hath me made so bare.</q> <rj><qau>Chaucer.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Sliding friction</b></col> <fld>(Mech.)</fld>, <cd>the resistance one body meets with in sliding along the surface of another, as distinguished from <xex>rolling friction</xex>.</cd> -- <col><b>Sliding gunter</b></col> <fld>(Naut.)</fld>, <cd>a topmast arranged with metallic fittings so as to be hoisted and lowered by means of halyards.</cd> -- <col><b>Sliding keel</b></col> <fld>(Naut)</fld>, <cd>a movable keel, similar to a centeboard.</cd> -- <col><b>Sliding pair</b></col>. <fld>(Mech.)</fld> <cd>See the Note under <er>Pair</er>, <pos>n.</pos>, 7.</cd> -- <col><b>Sliding rule</b></col>. <cd>Same as <cref>Slide rule</cref>, under <er>Slide</er>, <pos>n.</pos></cd> -- <col><b>Sliding scale</b></col>. <sd>(a)</sd> <cd>A scale for raising or lowering imposts in proportion to the fall or rise of prices</cd>. <sd>(b)</sd> <cd>A variable scale of wages or of prices</cd>. <sd>(c)</sd> <cd>A slide rule.</cd> -- <col><b>Sliding ways</b></col> <fld>(Naut.)</fld>, <cd>the timber guides used in launching a vessel.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Slidometer</ent><br/
<hw>Sli*dom"e*ter</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[<ets>Slide</ets> + <ets>-meter</ets>.]</ety> <def>An instrument for indicating and recording shocks to railway cars occasioned by sudden stopping.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Slight</ent><br/
<hw>Slight</hw> <pr>(?)</pr>, <pos>n.</pos> <def>Sleight.</def>  <rj><au>Spenser.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Slight</ent><br/
<hw>Slight</hw>, <pos>v. t.</pos> <ety>[Cf. D. <ets>slechten</ets> to level, to demolish.]</ety> <sn>1.</sn> <def>To overthrow; to demolish.</def> <mark>[Obs.]</mark>  <rj><au>Clarendon.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To make even or level.</def> <mark>[Obs.]</mark>  <rj><au>Hexham.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>To throw heedlessly.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>The rogue <qex>slighted</qex> me into the river.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Slight</ent><br/
<hw>Slight</hw> <pr>(?)</pr>, <pos>a.</pos> <amorph>[<pos>Compar.</pos> <adjf>Slighter</adjf> <pr>(?)</pr>; <pos>superl.</pos> <adjf>Slightest</adjf>.]</amorph> <ety>[OE. <ets>sli<?/t</ets>, <ets>sleght</ets>, probably from OD. <ets>slicht</ets>, <ets>slecht</ets>, simple, plain, D. <ets>slecht</ets>; akin to OFries. <ets>sliucht</ets>, G. <ets>schlecht</ets>, <ets>schlicht</ets>, OHG. <ets>sleht</ets> smooth, simple, Icel. <ets>sl<?/ttr</ets> smooth, Sw. <ets>sl<aum/t</ets>, Goth. <ets>sla<iacute/hts</ets>; or uncertain origin.]</ety> <sn>1.</sn> <def>Not decidedly marked; not forcible; inconsiderable; unimportant; insignificant; not severe; weak; gentle; -- applied in a great variety of circumstances; <as>as, a <ex>slight</ex> (<it>i. e.</it>, feeble) effort; a <ex>slight</ex> (<it>i. e.</it>, perishable) structure; a <ex>slight</ex> (<it>i. e.</it>, not deep) impression; a <ex>slight</ex> (<it>i. e.</it>, not convincing) argument; a <ex>slight</ex> (<it>i. e.</it>, not thorough) examination; <ex>slight</ex> (<it>i. e.</it>, not severe) pain, and the like.</as></def> <ldquo/At one <xex>slight</xex> bound.<rdquo/  <rj><au>Milton.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q><qex>Slight</qex> is the subject, but not so the praise.</q> <rj><qau>Pope.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Some firmly embrace doctrines upon <qex>slight</qex> grounds.</q> <rj><qau>Locke.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Not stout or heavy; slender.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>His own figure, which was formerly so <qex>slight</qex>.</q> <rj><qau>Sir W. Scott.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>Foolish; silly; weak in intellect.</def>  <rj><au>Hudibras.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Slight</ent><br/
<hw>Slight</hw>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Slighted</conjf>; <pos>p. pr. & vb. n.</pos> <conjf>Slighting</conjf>.]</vmorph> <def>To disregard, as of little value and unworthy of notice; to make light of; <as>as, to <ex>slight</ex> the divine commands</as>.</def>  <rj><au>Milton.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>The wretch who <qex>slights</qex> the bounty of the skies.</q> <rj><qau>Cowper.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><mcol><col><b>To slight off</b></col>, to treat slightingly; to drive off; to remove. <mark>[R.]</mark> -- <col><b>To slight over</b></col></mcol>, <cd>to run over in haste; to perform superficially; to treat carelessly; <as>as, <ex>to slight over</ex> a theme</as>.</cd>  <ldquo/They will but <xex>slight</xex> it over.<rdquo/  <rj><au>Bacon.</au></rj></cs><br/
[<source>1913 Webster</source>]</p>

<p><syn><b>Syn.</b> -- To neglect; disregard; disdain; scorn.</syn>  -- <usage><er>Slight</er>, <er>Neglect</er>. To <xex>slight</xex> is stronger than to <xex>neglect</xex>. We may neglect a duty or person from inconsiderateness, or from being over-occupied in other concerns. To <xex>slight</xex> is always a positive and intentional act, resulting from feelings of dislike or contempt. We ought to put a kind construction on what appears <xex>neglect</xex> on the part of a friend; but when he <xex>slights</xex> us, it is obvious that he is our friend no longer.</usage><br/
[<source>1913 Webster</source>]</p>

<p><q>Beware . . . lest the like befall . . . <br/
If they transgress and <qex>slight</qex> that sole command.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>This my long-sufferance, and my day of grace,<br/
Those who <qex>neglect</qex> and scorn shall never taste.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Slight</ent><br/
<hw>Slight</hw>, <pos>n.</pos> <def>The act of slighting; the manifestation of a moderate degree of contempt, as by neglect or oversight; neglect; indignity.</def><br/
[<source>1913 Webster</source>]</p>

<p><syn><b>Syn.</b> -- Neglect; disregard; inattention; contempt; disdain; scorn; disgrace; indignity; disparagement.</syn><br/
[<source>1913 Webster</source>]</p>

<p><ent>Slight</ent><br/
<hw>Slight</hw>, <pos>adv.</pos> <def>Slightly.</def> <mark>[Obs. or Poetic]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>Think not so <qex>slight</qex> of glory.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Slighten</ent><br/
<hw>Slight"en</hw> <pr>(?)</pr>, <pos>v. t.</pos> <def>To slight.</def> <mark>[Obs.]</mark>  <rj><au>B. Jonson.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Slighter</ent><br/
<hw>Slight"er</hw> <pr>(?)</pr>, <pos>n.</pos> <def>One who slights.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Slightful</ent><br/
<hw>Slight"ful</hw> <pr>(?)</pr>, <pos>a.</pos> <def>See <er>Sleightful</er>.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Slighting</ent><br/
<hw>Slight"ing</hw>, <pos>a.</pos> <def>Characterized by neglect or disregard.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Slightingly</ent><br/
<hw>Slight"ing*ly</hw>, <pos>adv.</pos> <def>In a slighting manner.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Slightly</ent><br/
<hw>Slight"ly</hw>, <pos>adv.</pos> <sn>1.</sn> <def>In a slight manner.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Slightingly; negligently.</def> <mark>[Obs.]</mark>  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Slightness</ent><br/
<hw>Slight"ness</hw>, <pos>n.</pos> <def>The quality or state of being slight; slenderness; feebleness; superficiality; also, formerly, negligence; indifference; disregard.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Slighty</ent><br/
<hw>Slight"y</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Slight.</def> <mark>[Obs.]</mark>  <rj><au>Echard.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Slik</ent><br/
<hw>Slik</hw> <pr>(sl<icr/k)</pr>, <pos>a.</pos> <ety>[See <er>Such</er>.]</ety> <def>Such.</def> <mark>[Obs. or Scot.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ Used by Chaucer as of the Northern dialect.</note><br/
[<source>1913 Webster</source>]</p>

<p><ent>Slikensides</ent><br/
<hw>Slik"en*sides`</hw>, <pos>n.</pos> <def>Same as <er>Slickensides</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Slily</ent><br/
<hw>Sli"ly</hw> <pr>(?)</pr>, <pos>adv.</pos> <def>See <er>Slyly</er>.</def>  <rj><au>South.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Slim</ent><br/
<hw>Slim</hw> <pr>(sl<icr/m)</pr>, <pos>a.</pos> <amorph>[<pos>Compar.</pos> <adjf>Slimmer</adjf> <pr>(?)</pr>; <pos>superl.</pos> <adjf>Slimmest</adjf>.]</amorph> <ety>[Formerly, bad, worthless, weak, slight, awry, fr. D. <ets>slim</ets>; akin to G. <ets>schlimm</ets>, MHG. <ets>slimp</ets> oblique, awry; of uncertain origin. The meaning of the English word seems to have been influenced by <ets>slender</ets>.]</ety> <sn>1.</sn> <def>Worthless; bad.</def> <mark>[Prov. Eng. & Scot.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Weak; slight; unsubstantial; poor; <as>as, a <ex>slim</ex> argument</as>.</def> <ldquo/That was a <xex>slim</xex> excuse.<rdquo/  <rj><au>Barrow.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>Of small diameter or thickness in proportion to the height or length; slender; <as>as, a <ex>slim</ex> person; a <ex>slim</ex> tree.</as></def>  <rj><au>Grose.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Slime</ent><br/
<hw>Slime</hw> <pr>(sl<imac/m)</pr>, <pos>n.</pos> <ety>[OE. <ets>slim</ets>, AS. <ets>sl<imac/m</ets>; akin to D. <ets>slijm</ets>, G. <ets>schleim</ets>, MHG. <ets>sl<imac/men</ets> to make smooth, Icel. <ets>sl<imac/m</ets> slime, Dan. <ets>sliim</ets>; cf. L. <ets>limare</ets> to file, polish, <ets>levis</ets> smooth, Gr. <?/<?/<?/; or cf. L. <ets>limus</ets> mud.]</ety> <sn>1.</sn> <def>Soft, moist earth or clay, having an adhesive quality; viscous mud.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>As it [Nilus] ebbs, the seedsman<br/
Upon the <qex>slime</qex> and ooze scatters his grain.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Any mucilaginous substance; any substance of a dirty nature, that is moist, soft, and adhesive.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <fld>(Script.)</fld> <def>Bitumen.</def> <mark>[Archaic]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q><qex>Slime</qex> had they for mortar.</q> <rj><qau>Gen. xi. 3.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><-- p. 1355 --></p>

<p><sn>4.</sn> <pluf>pl.</pluf> <fld>(Mining)</fld> <def>Mud containing metallic ore, obtained in the preparatory dressing.</def>  <rj><au>Pryce.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <fld>(Physiol.)</fld> <def>A mucuslike substance which exudes from the bodies of certain animals.</def>  <rj><au>Goldsmith.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Slime eel</b></col>. <fld>(Zool.)</fld> <cd>See 1st <er>Hag</er>, 4.</cd> -- <col><b>Slime pit</b></col>, <cd>a pit for the collection of slime or bitumen.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Slime</ent><br/
<hw>Slime</hw> <pr>(sl<imac/m)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Slimed</conjf> <pr>(sl<imac/md)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Sliming</conjf>.]</vmorph> <def>To smear with slime.</def>  <rj><au>Tennyson.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>slime mould</ent><br/
<ent>slime mold</ent><br/
<mhw><hw>slime mold</hw>, <hw>slime mould</hw></mhw> <pos>n.</pos> <sn>1.</sn> <def>An unusual fungus-like protist of the phylum <phylum>Myxomycota</phylum> or the class <class>Myxomycetes</class>, having a stage of growth in which it comprises a naked noncellular multinucleate mass of creeping protoplasm having characteristics of both plants and animals; it also has a propagative phase in which it develops fruiting bodies bearing spores; it is sometimes classified as a protist.  It is called also <altname><ecol><b>acellular slime mold</b></ecol></altname>.</def>
[<source>WordNet 1.5</source> <source>+PJC</source>]</p>

<p><sn>2.</sn> <def>Any of several remarkable amoebalike organisms of the phylum <phylum>Acrasiomycota</phylum>, mostly terrestrial, having a fruiting phase resembling that of the <cref>acellular slime molds</cref>, but being cellular and nucleate throughout their life cycle; called also <altname><ecol><b>cellular slime mold</b></ecol></altname>.  The most studied species is <styp><spn>Dictyostelium discoideum</spn></styp>.  In their feeding phase, they live like amoebae as individual cells, engulfing bacteria as a prime food source.  When the food source diminishes, they begin to aggregate, swarming together to form clumps which may move toward heat and light, so as to reach the surface of the ground; they then differentiate into a form with spores contained within a sporangium resting on a stalk.  When the spores are carried to another location with adequate food supplies, the spores may germinate to resume the life cycle.  The phase of aggregation appears to be initiated by release of cyclic AMP, serving as a signal between the individual cells.  The formation of the fruiting body has some similarities to differentiation in multicellular organisms, but the mechanisms are still under study.  Some biologists object to the classification of <gen>Dictyostelium</gen> as a <ex>slime mold</ex>, as it is neither a mold nor slimy.</def><br/
[<source>PJC</source>]</p>

<p><ent>Slimily</ent><br/
<hw>Slim"i*ly</hw> <pr>(?)</pr>, <pos>adv.</pos> <def>In a slimy manner.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sliminess</ent><br/
<hw>Slim"i*ness</hw>, <pos>n.</pos> <def>The quality or state of being slimy.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Slimly</ent><br/
<hw>Slim"ly</hw> <pr>(?)</pr>, <pos>adv.</pos> <def>In a state of slimness; in a slim manner; slenderly.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Slimness</ent><br/
<hw>Slim"ness</hw>, <pos>n.</pos> <def>The quality or state of being slim.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Slimsy</ent><br/
<hw>Slim"sy</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Flimsy; frail.</def> <mark>[Colloq. U.S.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Slimy</ent><br/
<hw>Slim"y</hw> <pr>(?)</pr>, <pos>a.</pos> <amorph>[<pos>Compar.</pos> <adjf>Slimier</adjf> <pr>(?)</pr>; <pos>superl.</pos> <adjf>Slimiest</adjf>.]</amorph> <def>Of or pertaining to slime; resembling slime; of the nature of slime; viscous; glutinous; also, covered or daubed with slime; yielding, or abounding in, slime.</def><br/
[<source>1913 Webster</source>]</p>

<p><q><qex>Slimy</qex> things did crawl with legs<br/
Upon the <qex>slimy</qex> sea.</q> <rj><qau>Coleridge.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sliness</ent><br/
<hw>Sli"ness</hw> <pr>(?)</pr>, <pos>n.</pos> <def>See <er>Slyness</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sling</ent><br/
<hw>Sling</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[OE. <ets>slinge</ets>; akin to OD. <ets>slinge</ets>, D. <ets>slinger</ets>, OHG. <ets>slinga</ets>; cf. OF. <ets>eslingue</ets>, of German origin. See <er>Sling</er>, <pos>v. t.</pos>]</ety> <sn>1.</sn> <def>An instrument for throwing stones or other missiles, consisting of a short strap with two strings fastened to its ends, or with a string fastened to one end and a light stick to the other. The missile being lodged in a hole in the strap, the ends of the string are taken in the hand, and the whole whirled rapidly round until, by loosing one end, the missile is let fly with centrifugal force.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>The act or motion of hurling as with a sling; a throw; figuratively, a stroke.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The <qex>slings</qex> and arrows of outrageous fortune.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>At one <qex>sling</qex><br/
Of thy victorius arm, well-pleasing Son.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>A contrivance for sustaining anything by suspension</def>; as: <sd>(a)</sd> <def>A kind of hanging bandage put around the neck, in which a wounded arm or hand is supported.</def> <sd>(b)</sd> <def>A loop of rope, or a rope or chain with hooks, for suspending a barrel, bale, or other heavy object, in hoisting or lowering.</def> <sd>(c)</sd> <def>A strap attached to a firearm, for suspending it from the shoulder.</def> <sd>(d)</sd> <fld>(Naut.)</fld> <def>A band of rope or iron for securing a yard to a mast; -- chiefly in the plural.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Sling cart</b></col>, <cd>a kind of cart used to transport cannon and their carriages, large stones, machines, etc., the objects transported being slung, or suspended by a chain attached to the axletree.</cd> -- <col><b>Sling dog</b></col>, <cd>one of a pair of iron hooks used as part of a sling. See def. 3 <sd>(b)</sd> above.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sling</ent><br/
<hw>Sling</hw>, <pos>v. t.</pos> <vmorph>[<pos>imp.</pos> <conjf>Slung</conjf> <pr>(?)</pr>, <mark>Archaic</mark> <conjf>Slang</conjf> <pr>(<?/)</pr>; <pos>p. p.</pos> <conjf>Slung</conjf>; <pos>p. pr. & vb. n.</pos> <conjf>Slinging</conjf>.]</vmorph> <ety>[AS. <ets>slingan</ets>; akin to D. <ets>slingeren</ets>, G. <ets>schlingen</ets>, to wind, to twist, to creep, OHG. <ets>slingan</ets> to wind, to twist, to move to and fro, Icel. <ets>slyngva</ets>, <ets>sl<oum/ngva</ets>, to sling, Sw. <ets>slunga</ets>, Dan. <ets>slynge</ets>, Lith. <ets>slinkti</ets> to creep.]</ety> <sn>1.</sn> <def>To throw with a sling.</def> <ldquo/Every one could <xex>sling</xex> stones at an hairbreadth, and not miss.<rdquo/  <rj><au>Judg. xx. 16.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To throw; to hurl; to cast.</def>  <rj><au>Addison.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>To hang so as to swing; <as>as, to <ex>sling</ex> a pack</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <fld>(Naut)</fld> <def>To pass a rope round, as a cask, gun, etc., preparatory to attaching a hoisting or lowering tackle.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sling</ent><br/
<hw>Sling</hw>, <pos>n.</pos> <ety>[Cf. G. <ets>schlingen</ets> to swallow.]</ety> <def>A drink composed of spirit (usually gin) and water sweetened.</def>
<-- as, a Singapore <ex>sling.</ex> --><br/
[<source>1913 Webster</source>]</p>

<p><ent>Slinger</ent><br/
<hw>Sling"er</hw> <pr>(?)</pr>, <pos>n.</pos> <def>One who slings, or uses a sling.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>slinger ring</ent><br/
<hw>slinger ring</hw> <pos>n.</pos> <def>tubular ring around the propeller hub of an airplane through which antifreeze solution is spread over the blades.</def>
[<source>WordNet 1.5</source>]</p>

<p><ent>Slink</ent><br/
<hw>Slink</hw> <pr>(?)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp.</pos> <conjf>Slunk</conjf> <pr>(?)</pr>, <mark>Archaic</mark> <conjf>Slank</conjf> <pr>(<?/)</pr>; <pos>p. p.</pos> <conjf>Slunk</conjf>; <pos>p. pr. & vb. n.</pos> <conjf>Slinking</conjf>.]</vmorph> <ety>[AS. <ets>slincan</ets>; probably akin to G. <ets>schleichen</ets>, E. <ets>sleek</ets>. See <er>Sleek</er>, <pos>a.</pos>]</ety> <sn>1.</sn> <def>To creep away meanly; to steal away; to sneak.</def> <ldquo/To <xex>slink</xex> away and hide.<rdquo/  <rj><au>Tale of Beryn.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Back to the thicket <qex>slunk</qex><br/
The guilty serpent.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>There were some few who <qex>slank</qex> obliquely from them as they passed.</q> <rj><qau>Landor.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To miscarry; -- said of female beasts.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Slink</ent><br/
<hw>Slink</hw>, <pos>v. t.</pos> <def>To cast prematurely; -- said of female beasts; <as>as, a cow that <ex>slinks</ex> her calf</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Slink</ent><br/
<hw>Slink</hw>, <pos>a.</pos> <sn>1.</sn> <def>Produced prematurely; <as>as, a <ex>slink</ex> calf</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Thin; lean.</def> <mark>[Scot.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Slink</ent><br/
<hw>Slink</hw>, <pos>n.</pos> <sn>1.</sn> <def>The young of a beast brought forth prematurely, esp. a calf brought forth before its time.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A thievish fellow; a sneak.</def> <mark>[Prov. Eng. & Scot.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Slinky</ent><br/
<hw>Slink"y</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Thin; lank.</def> <mark>[Prov. Eng. & U. S.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Slip</ent><br/
<hw>Slip</hw> <pr>(?)</pr>, <pos>v. i.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Slipped</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Slipping</conjf>.]</vmorph> <ety>[OE. <ets>slippen</ets>; akin to LG. & D. <ets>slippen</ets>, MHG. <ets>slipfen</ets> (cf. Dan. <ets>slippe</ets>, Sw. <ets>slippa</ets>, Icel. <ets>sleppa</ets>), and fr. OE. <ets>slipen</ets>, AS. <ets>sl<imac/pan</ets> (in comp.), akin to G. <ets>schleifen</ets> to slide, glide, drag, whet, OHG. <ets>sl<imac/fan</ets> to slide, glide, make smooth, Icel. <ets>sl<imac/pa</ets> to whet; cf. also AS. <ets>sl<?/pan</ets>, Goth. <ets>sliupan</ets>, OS. <ets>slopian</ets>, OHG. <ets>sliofan</ets>, G. <ets>schliefen</ets>, <ets>schl<?/pfen</ets>, which seem to come from a somewhat different root form.  Cf. <er>Slope</er>, <pos>n.</pos>]</ety> <sn>1.</sn> <def>To move along the surface of a thing without bounding, rolling, or stepping; to slide; to glide.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To slide; to lose one's footing or one's hold; not to tread firmly; <as>as, it is necessary to walk carefully lest the foot should <ex>slip</ex></as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>To move or fly (out of place); to shoot; -- often with <xex>out</xex>, <xex>off</xex>, etc.; <as>as, a bone may <ex>slip</ex> out of its place</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>To depart, withdraw, enter, appear, intrude, or escape as if by sliding; to go or come in a quiet, furtive manner; <as>as, some errors <ex>slipped</ex> into the work</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Thus one tradesman <qex>slips</qex> away,<br/
To give his partner fairer play.</q> <rj><qau>Prior.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Thrice the flitting shadow <qex>slipped</qex> away.</q> <rj><qau>Dryden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <def>To err; to fall into error or fault.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>There is one that <qex>slippeth</qex> in his speech, but not from his heart.</q> <rj><qau>Ecclus. xix. 16.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>To let slip</b></col>, <cd>to loose from the slip or noose, as a hound; to allow to escape.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><q>Cry, <ldquo/Havoc,<rdquo/ and <qex>let slip</qex> the dogs of war.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Slip</ent><br/
<hw>Slip</hw> <pr>(?)</pr>, <pos>v. t.</pos> <sn>1.</sn> <def>To cause to move smoothly and quickly; to slide; to convey gently or secretly.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>He tried to <qex>slip</qex> a powder into her drink.</q> <rj><qau>Arbuthnot.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To omit; to loose by negligence.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>And <qex>slip</qex> no advantage<br/
That my secure you.</q> <rj><qau>B. Jonson.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>To cut slips from; to cut; to take off; to make a slip or slips of; <as>as, to <ex>slip</ex> a piece of cloth or paper</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The branches also may be <qex>slipped</qex> and planted.</q> <rj><qau>Mortimer.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>To let loose in pursuit of game, as a greyhound.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Lucento <qex>slipped</qex> me like his greyhound.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <def>To cause to slip or slide off, or out of place; <as>as, a horse <ex>slips</ex> his bridle; a dog <ex>slips</ex> his collar</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>6.</sn> <def>To bring forth (young) prematurely; to slink.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>To slip a cable</b></col>. <fld>(Naut.)</fld> <cd>See under <er>Cable</er>.</cd> -- <col><b>To slip off</b></col>, <cd>to take off quickly; <as>as, <ex>to slip off</ex> a coat</as>.</cd> -- <col><b>To slip on</b></col>, <cd>to put on in haste or loosely; <as>as, <ex>to slip on</ex> a gown or coat</as>.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Slip</ent><br/
<hw>Slip</hw>, <pos>n.</pos> <ety>[AS. <ets>slipe</ets>, <ets>slip</ets>.]</ety> <sn>1.</sn> <def>The act of slipping; <as>as, a <ex>slip</ex> on the ice</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>An unintentional error or fault; a false step.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>This good man's <qex>slip</qex> mended his pace to martyrdom.</q> <rj><qau>Fuller.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>A twig separated from the main stock; a cutting; a scion; hence, a descendant; <as>as, a <ex>slip</ex> from a vine</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>A native <qex>slip</qex> to us from foreign seeds.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>The girlish <qex>slip</qex> of a Sicilian bride.</q> <rj><qau>R. Browning.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>A slender piece; a strip; <as>as, a <ex>slip</ex> of paper</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Moonlit <qex>slips</qex> of silver cloud.</q> <rj><qau>Tennyson.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>A thin <qex>slip</qex> of a girl, like a new moon<br/
Sure to be rounded into beauty soon.</q> <rj><qau>Longfellow.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <def>A leash or string by which a dog is held; -- so called from its being made in such a manner as to slip, or become loose, by relaxation of the hand.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>We stalked over the extensive plains with Killbuck and Lena in the <qex>slips</qex>, in search of deer.</q> <rj><qau>Sir S. Baker.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>6.</sn> <def>An escape; a secret or unexpected desertion; <as>as, to give one the <ex>slip</ex></as>.</def>  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>7.</sn> <fld>(Print.)</fld> <def>A portion of the columns of a newspaper or other work struck off by itself; a proof from a column of type when set up and in the galley.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>8.</sn> <def>Any covering easily slipped on.</def> Specifically: <sd>(a)</sd> <def>A loose garment worn by a woman.</def> <sd>(b)</sd> <def>A child's pinafore.</def> <sd>(c)</sd> <def>An outside covering or case; <as>as, a pillow <ex>slip</ex></as>.</def> <sd>(d)</sd> <def>The <ex>slip</ex> or sheath of a sword, and the like.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><sn>9.</sn> <def>A counterfeit piece of money, being brass covered with silver.</def> <mark>[Obs.]</mark>  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>10.</sn> <def>Matter found in troughs of grindstones after the grinding of edge tools.</def> <mark>[Prov. Eng.]</mark>  <rj><au>Sir W. Petty.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>11.</sn> <def>Potter's clay in a very liquid state, used for the decoration of ceramic ware, and also as a cement for handles and other applied parts.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>12.</sn> <def>A particular quantity of yarn.</def> <mark>[Prov. Eng.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><sn>13.</sn> <def>An inclined plane on which a vessel is built, or upon which it is hauled for repair.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>14.</sn> <def>An opening or space for vessels to lie in, between wharves or in a dock; <as>as, Peck <ex>slip</ex></as>.</def> <mark>[U. S.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><sn>15.</sn> <def>A narrow passage between buildings.</def> <mark>[Eng.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><sn>16.</sn> <def>A long seat or narrow pew in churches, often without a door.</def> <mark>[U. S.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><sn>17.</sn> <fld>(Mining.)</fld> <def>A dislocation of a lead, destroying continuity.</def>  <rj><au>Knight.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>18.</sn> <fld>(Engin.)</fld> <def>The motion of the center of resistance of the float of a paddle wheel, or the blade of an oar, through the water horozontally, or the difference between a vessel's actual speed and the speed which she would have if the propelling instrument acted upon a solid; also, the velocity, relatively to still water, of the backward current of water produced by the propeller.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>19.</sn> <fld>(Zool.)</fld> <def>A fish, the sole.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>20.</sn> <fld>(Cricket)</fld> <def>A fielder stationed on the off side and to the rear of the batsman. There are usually two of them, called respectively <stype>short slip</stype>, and <stype>long slip</stype>.</def><br/
[<source>1913 Webster</source>]</p>

<p><-- 21. A slip dock (see below) --><br/
[<source>1913 Webster</source>]</p>

<p><sn>22.</sn><fld>(Mach.)</fld> <sd>(a)</sd> <def>The retrograde movement on a pulley of a belt as it slips.</def> <sd>(b)</sd> <def>In a link motion, the  undesirable sliding movement of the link relatively to the link block, due to swinging of the link.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><sn>23.</sn>  <fld>(Elec.)</fld> <def>The difference between the actual and synchronous speed of an induction motor.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><sn>23.</sn>  <fld>(Marine Insurance)</fld> <def>A memorandum of the particulars of a risk for which a policy is to be executed. It usually bears the broker's name and is initiated by the underwrites.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><cs><col><b>To give one the slip</b></col>, <cd>to slip away from one; to elude one.</cd> -- <col><b>Slip dock</b></col>. <cd>See under <er>Dock</er>.</cd> -- <col><b>Slip link</b></col> <fld>(Mach.)</fld>, <cd>a connecting link so arranged as to allow some play of the parts, to avoid concussion.</cd> -- <col><b>Slip rope</b></col> <fld>(Naut.)</fld>, <cd>a rope by which a cable is secured preparatory to slipping.</cd> <au>Totten.</au> -- <col><b>Slip stopper</b></col> <fld>(Naut.)</fld>, <cd>an arrangement for letting go the anchor suddenly.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Slipboard</ent><br/
<hw>Slip"board`</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A board sliding in grooves.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>slip coach</ent><br/
<ent>slip carriage</ent><br/
<mhw><hw>slip carriage</hw>, <hw>slip coach</hw></mhw> <pos>n.</pos> <def>a railway car at the end of the train; it can be detached without stopping the train.</def> <mark>[British]</mark><br/
<syn><b>Syn. --</b> slip coach</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>slip clutch</ent><br/
<hw>slip clutch</hw> <pos>n.</pos> <def>a friction clutch that will slip when the torque is too great</def><br/
<syn><b>Syn. --</b> slip friction clutch.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Slipcoat cheese</ent><br/
<hw>Slip"coat` cheese"</hw> <pr>(?)</pr>. <def>A rich variety of new cheese, resembling butter, but white.</def>  <rj><au>Halliwell.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Slipes</ent><br/
<hw>Slipes</hw> <pr>(?)</pr>, <pos>n. pl.</pos> <ety>[Cf. <er>Slip</er>, <pos>v.</pos>]</ety> <def>Sledge runners on which a skip is dragged in a mine.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>slip-joint pliers</ent><br/
<hw>slip-joint pliers</hw> <pos>n.</pos>
<def>pliers with a joint adjustable to two positions in order to increase the opening of the jaws.</def><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Slipknot</ent><br/
<hw>Slip"knot`</hw> <pr>(?)</pr>, <pos>n.</pos> <def>knot which slips along the rope or line around which it is made.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>slip of the tongue</ent><br/
<hw>slip of the tongue</hw> <pos>n.</pos> <def>an unintentional utterance; a mistake in speaking.</def><br/
<syn><b>Syn. --</b> slip of the lip.</syn><br/
[<source>WordNet 1.5</source> <source>+PJC</source>]</p>

<p><ent>Slip-on</ent><br/
<hw>Slip"-on`</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A kind of overcoat worn upon the shoulders in the manner of a cloak.</def> <mark>[Scot.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Slippage</ent><br/
<hw>Slip"page</hw> <pr>(?)</pr>, <pos>n.</pos> <def>The act of slipping; also, the amount of slipping.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Slipper</ent><br/
<hw>Slip"per</hw> <pr>(?)</pr>, <pos>n.</pos> <sn>1.</sn> <def>One who, or that which, slips.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A kind of light shoe, which may be slipped on with ease, and worn in undress; a slipshoe.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>A kind of apron or pinafore for children.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>A kind of brake or shoe for a wagon wheel.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <fld>(Mach.)</fld> <def>A piece, usually a plate, applied to a sliding piece, to receive wear and afford a means of adjustment; -- also called <altname>shoe</altname>, and <altname>gib</altname>.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Slipper animalcule</b></col> <fld>(Zool.)</fld>, <cd>a ciliated infusorian of the genus <gen>Paramecium</gen>.</cd> -- <col><b>Slipper flower</b></col>.<fld>(Bot.)</fld> <cd>Slipperwort.</cd> -- <mcol><col><b>Slipper limpet</b></col>, <it>or</it> <col><b>Slipper shell</b></col></mcol> <fld>(Zool.)</fld>, <cd>a boat shell.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Slipper</ent><br/
<hw>Slip"per</hw>, <pos>a.</pos> <ety>[AS. <ets>slipur</ets>.]</ety> <def>Slippery.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>O! trustless state of earthly things, and <qex>slipper</qex> hope<br/
Of mortal men.</q> <rj><qau>Spenser.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Slippered</ent><br/
<hw>Slip"pered</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Wearing slippers.</def>  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Slipperily</ent><br/
<hw>Slip"per*i*ly</hw> <pr>(?)</pr>, <pos>adv.</pos> <def>In a slippery manner.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Slipperiness</ent><br/
<hw>Slip"per*i*ness</hw>, <pos>n.</pos> <def>The quality of being slippery.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Slipperness</ent><br/
<hw>Slip"per*ness</hw>, <pos>n.</pos> <def>Slipperiness.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>slipper orchid</ent><br/
<hw>slipper orchid</hw> <pos>n.</pos> <def>any of several chiefly American wildflowers having an inflated pouchlike lip; difficult or impossible to cultivate in the garden.</def><br/
<syn><b>Syn. --</b> lady's slipper, lady's-slipper, lady-slipper.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Slipperwort</ent><br/
<hw>Slip"per*wort`</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Bot.)</fld> <def>See <er>Calceolaria</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Slippery</ent><br/
<hw>Slip"per*y</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[See <er>Slipper</er>, <pos>a.</pos>]</ety> <sn>1.</sn> <def>Having the quality opposite to adhesiveness; allowing or causing anything to slip or move smoothly, rapidly, and easily upon the surface; smooth; glib; <as>as, oily substances render things <ex>slippery</ex></as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Not affording firm ground for confidence; <as>as, a <ex>slippery</ex> promise</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The <qex>slippery</qex> tops of human state.</q> <rj><qau>Cowley.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>Not easily held; liable or apt to slip away.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The <qex>slippery</qex> god will try to loose his hold.</q> <rj><qau>Dryden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>Liable to slip; not standing firm.</def>  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <def>Unstable; changeable; mutable; uncertain; inconstant; fickle.</def> <ldquo/The <xex>slippery</xex> state of kings.<rdquo/  <rj><au>Denham.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>6.</sn> <def>Uncertain in effect.</def>  <rj><au>L'Estrange.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>7.</sn> <def>Wanton; unchaste; loose in morals.</def>  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Slippery elm</b></col>. <fld>(Bot.)</fld> <sd>(a)</sd> <cd>An American tree (<spn>Ulmus fulva</spn>) with a mucilagenous and slightly aromatic inner bark which is sometimes used medicinally; also, the inner bark itself</cd>. <sd>(b)</sd> <cd>A malvaceous shrub (<spn>Fremontia Californica</spn>); -- so called on the Pacific coast.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Slippiness</ent><br/
<hw>Slip"pi*ness</hw> <pr>(?)</pr>, <pos>n.</pos> <def>Slipperiness.</def> <mark>[R.]</mark> <ldquo/The <xex>slippiness</xex> of the way.<rdquo/  <rj><au>Sir W. Scott.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Slippy</ent><br/
<hw>Slip"py</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[AS. <ets>slipeg</ets>.]</ety> <def>Slippery.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>slip ring</ent><br/
<hw>slip ring</hw> <pos>n.</pos> <def>a metal ring on a rotating part of a machine to provide a continuous electrical connection through brushes on stationary contacts</def><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>slip road</ent><br/
<hw>slip road</hw> <pos>n.</pos> <def>a short road giving access to an expressway</def>  <mark>[British]</mark> <ldquo/in England they call an access road a slip road.<rdquo/<br/
<syn><b>Syn. --</b> access road, slip road.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Slipshod</ent><br/
<hw>Slip"shod`</hw> <pr>(?)</pr>, <pos>a.</pos> <sn>1.</sn> <def>Wearing shoes or slippers down at the heel.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The shivering urchin bending as he goes,<br/
With <qex>slipshod</qex> heels.</q> <rj><qau>Cowper.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Figuratively: Careless in dress, manners, style, etc.; slovenly; shuffling; <as>as, <ex>slipshod</ex> manners; a <ex>slipshod</ex> or loose style of writing</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Thy wit shall ne'er go <qex>slipshod</qex>.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Slipshoe</ent><br/
<hw>Slip"shoe`</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A slipper.</def>  <rj><au>Halliwell.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Slipskin</ent><br/
<hw>Slip"skin`</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Evasive.</def> <mark>[Obs.]</mark>  <rj><au>Milton.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Slipslop</ent><br/
<hw>Slip"slop`</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[A reduplication of <ets>slop</ets>.]</ety> <def>Weak, poor, or flat liquor; weak, profitless discourse or writing.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>slip stitch</ent><br/
<hw>slip stitch</hw> <pos>n.</pos> <def>loose stitch catching only a thread or two of fabric, designed to be invisible from the right side.</def><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Slipstring</ent><br/
<hw>Slip"string`</hw> <pr>(?)</pr>, <pos>n.</pos> <def>One who has shaken off restraint; a prodigal.</def> <mark>[Obs.]</mark>  <rj><au>Cotgrave.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Slipthrift</ent><br/
<hw>Slip"thrift`</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A spendthrift.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Slish</ent><br/
<hw>Slish</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[A corruption of <ets>slash</ets>.]</ety> <def>A cut; <as>as, <ex>slish</ex> and slash</as>.</def> <mark>[Colloq.]</mark>  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Slit</ent><br/
<hw>Slit</hw> <pr>(?)</pr>, <mark>obs.</mark> <def><pos>3d. pers. sing. pres.</pos> of <er>Slide</er>.</def>  <rj><au>Chaucer.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Slit</ent><br/
<hw>Slit</hw> <pr>(?)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Slit</conjf> or <conjf>Slitted</conjf> <pr>(<?/)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Slitting</conjf>.]</vmorph> <ety>[OE. <ets>slitten</ets>, fr. <ets>sliten</ets>, AS. <ets>st<imac/tan</ets> to tear; akin to D. <ets>slijten</ets> to wear out, G. <ets>schleissen</ets> to slit, split, OHG. <ets>sl<imac/zan</ets> to split, tear, wear out, Icel. <ets>st<imac/ta</ets> to break, tear, wear out, Sw. <ets>slita</ets>, Dan. <ets>slide</ets>.  Cf. <er>Eclat</er>, <er>Slate</er>, <pos>n.</pos>, <er>Slice</er>.]</ety> <sn>1.</sn> <def>To cut lengthwise; to cut into long pieces or strips; <as>as, to <ex>slit</ex> iron bars into nail rods; to <ex>slit</ex> leather into straps.</as></def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To cut or make a long fissure in or upon; <as>as, to <ex>slit</ex> the ear or the nose</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>To cut; to sever; to divide.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>And <qex>slits</qex> the thin-spun life.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Slit</ent><br/
<hw>Slit</hw>, <pos>n.</pos> <ety>[AS. <ets>slite</ets>.]</ety> <def>A long cut; a narrow opening; <as>as, a <ex>slit</ex> in the ear</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Gill slit</b></col>. <fld>(Anat.)</fld> <cd>See <cref>Gill opening</cref>, under <er>Gill</er>.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>slither</ent><br/
<hw>slith"er</hw> <pr>(sl<icr/<th/"<etil/r)</pr>, <pos>v. i.</pos> <ety>[Cf. G. <ets>schlittern</ets>, LG. <ets>schliddern</ets>. See <er>slide</er>.]</ety> <def>To slide; to glide.</def> <mark>[Prov. Eng.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>slit-shell</ent><br/
<hw>slit"-shell`</hw> <pr>(sl<icr/t"sh<ecr/l`)</pr>, <pos>n.</pos> <fld>(Zool.)</fld> <def>Any species of <gen>Pleurotomaria</gen>, a genus of beautiful, pearly, spiral gastropod shells having a deep slit in the outer lip.  Many fossil species are known, and a few living ones are found in deep water in tropical seas.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>slitter</ent><br/
<hw>slit"ter</hw> <pr>(sl<icr/t"t<etil/r)</pr>, <pos>n.</pos> <def>One who, or that which, slits.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>slitting</ent><br/
<hw>slit"ting</hw> <pr>(sl<icr/t"t<icr/ng)</pr>, <def><pos>a. & n.</pos> from <er>slit</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>slitting file</b></col>. <cd>See <xex>Illust.</xex> <sd>(i)</sd> of <er>sile</er>.</cd> -- <col><b>slitting mill</b></col>. <sd>(a)</sd> <cd>A mill where iron bars or plates are slit into narrow strips, as nail rods, and the like</cd>. <sd>(b)</sd> <cd>A machine used by lapidaries for slicing stones, usually by means of a revolving disk, called a <part>slicer</part>, supplied with diamond powder.</cd> -- <col><b>slitting roller</b></col>, <cd>one of a pair of rollers furnished with ribs entering between similar ribs in the other roller, and cutting like shears, -- used in slitting metals.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Slive</ent><br/
<hw>Slive</hw> <pr>(sl<imac/v)</pr>, <pos>v. i.</pos> <ety>[Cf. <er>Slip</er>.]</ety> <def>To sneak.</def> <mark>[Prov. Eng.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Slive</ent><br/
<hw>Slive</hw>, <pos>v. t.</pos> <ety>[OE. <ets>sliven</ets> to split, cleave, AS. <ets>sl<imac/fan</ets>.]</ety> <def>To cut; to split; to separate.</def> <mark>[Obs.]</mark>  <rj><au>Holland.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>sliver</ent><br/
<hw>sliv"er</hw> <pr>(sl<icr/v"<etil/r <it>or</it> sl<imac/"v<etil/r; 277)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>slivered</conjf> <pr>(sl<icr/v"<etil/rd)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>slivering</conjf>.]</vmorph> <ety>[See <er>Slive</er>, <pos>v. t.</pos>]</ety> <def>To cut or divide into long, thin pieces, or into very small pieces; to cut or rend lengthwise; to slit; <as>as, to <ex>sliver</ex> wood</as>.</def>  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>They 'll <qex>sliver</qex> thee like a turnip.</q> <rj><qau>Sir W. Scott.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sliver</ent><br/
<hw>Sliv"er</hw>, <pos>n.</pos> <sn>1.</sn> <def>A long piece cut or rent off; a sharp, slender fragment, as of glass; a splinter.</def><br/
[<source>1913 Webster</source> <source>+PJC</source>]</p>

<p><sn>2.</sn> <def>A strand, or slender roll, of cotton or other fiber in a loose, untwisted state, produced by a carding machine and ready for the roving or slubbing which preceeds spinning.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <pluf>pl.</pluf> <def>Bait made of pieces of small fish.  Cf. <er>Kibblings</er>.</def> <mark>[Local, U.S.]</mark>  <rj><au>Bartlett.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>sloakan</ent><br/
<hw>sloak"an</hw> <pr>(sl<omac/k"<ait/n)</pr>, <pos>n.</pos> <fld>(Bot.)</fld> <def>A species of seaweed.  See 3d <er>Laver</er>.</def>  <altsp>[Spelled also <asp>slowcawn</asp>.]</altsp><br/
[<source>1913 Webster</source>]</p>

<p><ent>sloam</ent><br/
<hw>sloam</hw> <pr>(sl<omac/m)</pr>, <pos>n.</pos> <fld>(Mining)</fld> <def>A layer of earth between coal seams.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>sloat</ent><br/
<hw>sloat</hw> <pr>(sl<omac/t)</pr>, <pos>n.</pos> <ety>[See <er>slot</er> a bar.]</ety> <def>A narrow piece of timber which holds together large pieces; a slat; <as>as, the <ex>sloats</ex> of a cart</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Slobber</ent><br/
<hw>Slob"ber</hw> <pr>(?)</pr>, <pos>v. t. & i.</pos> <def>See <er>Slabber</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Slobber</ent><br/
<hw>Slob"ber</hw>, <pos>n.</pos> <sn>1.</sn> <def>See <er>Slabber</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Zool.)</fld> <def>A jellyfish.</def> <mark>[Prov. Eng.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <pluf>pl.</pluf> <fld>(Vet.)</fld> <def>Salivation.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Slobberer</ent><br/
<hw>Slob"ber*er</hw> <pr>(?)</pr>, <pos>n.</pos> <sn>1.</sn> <def>One who slobbers.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A slovenly farmer; a jobbing tailor.</def> <mark>[Prov. Eng.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Slobbery</ent><br/
<hw>Slob"ber*y</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Wet; sloppy, as land.</def>  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Slocken</ent><br/
<ent>Slock</ent><br/
<mhw>{ <hw>Slock</hw> <pr>(?)</pr>, <hw>Slock"en</hw> <pr>(?)</pr>, }</mhw> <pos>v. t.</pos> <def>To quench; to allay; to slake. See <er>Slake</er>.</def> <mark>[Obs. or Scot.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Slocking</ent><br/
<hw>Slock"ing</hw>, <def><pos>a. & n.</pos> from <er>Slock</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Slocking stone</b></col>, <cd>a rich piece of ore displayed in order to tempt persons to embark in a mining enterprise.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sloe</ent><br/
<hw>Sloe</hw> <pr>(sl<omac/)</pr>, <pos>n.</pos> <ety>[OE. <ets>slo</ets>, AS. <ets>sl<amac/</ets>; akin to D. <ets>slee</ets>, G. <ets>schlehe</ets>, OHG. <ets>sl<emac/ha</ets>, Dan. <ets>slaaen</ets>, Sw. <ets>sl<aring/n</ets>, perhaps originally, that which blunts the teeth, or sets them on edge (cf. <er>Slow</er>); cf. Lith. <ets>slywa</ets> a plum, Russ. <ets>sliva</ets>.]</ety> <fld>(Bot.)</fld> <def>A small, bitter, wild European plum, the fruit of the blackthorn (<spn>Prunus spinosa</spn>); also, the tree itself.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Slog</ent><br/
<hw>Slog</hw> <pr>(sl<ocr/g)</pr>, <pos>v. t. & i.</pos> <ety>[Cf. <er>Slug</er>, <pos>v. t.</pos>]</ety> <def>To hit hard, esp. with little attention to aim or the like, as in cricket or boxing; to slug.</def> <mark>[Cant or Slang]</mark><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Slog</ent><br/
<hw>Slog</hw> <pr>(sl<ocr/g)</pr>, <pos>v. i.</pos> <sn>1.</sn> <def>to walk heavily; to plod; to walk through resisting terrain, as in mud.</def><br/
[<source>PJC</source>]</p>

<p><sn>2.</sn> <def>To work steadily and ploddingly; to toil.</def><br/
[<source>PJC</source>]</p>

<p><ent>Slogger</ent><br/
<hw>Slog"ger</hw> <pr>(sl<ocr/g"g<etil/r)</pr>, <pos>n.</pos> <def>A hard hitter; a slugger.</def> <mark>[Cant or Slang]</mark>  <rj><au>T. Hughes.</au></rj><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Slogan</ent><br/
<hw>Slo"gan</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Gael. <ets>sluagh-ghairm</ets>, i.e., an army cry; <ets>sluagh</ets> army + <ets>gairm</ets> a call, calling.]</ety> <def>The war cry, or gathering word, of a Highland clan in Scotland.</def>  <rj><au>Sir W. Scott.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <specif>Hence:</specif> <def>A distinctive motto, phrase, or cry used by any person or party to express a purpose or ideal; a catchphrase; a rallying cry.</def><br/
[<source>PJC</source>]</p>

<p><-- p. 1356 --></p>

<p><ent>sloggy</ent><br/
<hw>slog"gy</hw> <pr>(sl<ocr/g"g<ycr/)</pr>, <pos>a.</pos> <def>Sluggish.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>Somnolence that is <qex>sloggy</qex> slumbering</q> <rj><qau>Chaucer.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>sloke</ent><br/
<hw>sloke</hw> <pr>(sl<omac/k)</pr>, <pos>n.</pos> <fld>(Bot.)</fld> <def>See <er>sloakan</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>slue</ent><br/
<ent>sloo</ent><br/
<mhw>{ <hw>sloo</hw> <pr>(sl<oomac/)</pr>, <it>or</it> <hw>slue</hw> <pr>(sl<umac/)</pr>, }</mhw> <pos>n.</pos> <def>A slough; a run or wet place. See 2d <er>Slough</er>, 2.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sloom</ent><br/
<hw>Sloom</hw> <pr>(?)</pr>, <pos>n.</pos> <def>Slumber.</def> <mark>[Prov. Eng.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sloomy</ent><br/
<hw>Sloom"y</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Sluggish; slow.</def> <mark>[Prov. Eng.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sloop</ent><br/
<hw>Sloop</hw> <pr>(?)</pr>, <pos>n.</pos><ety>[D. <ets>sloep</ets>, of uncertain origin.  Cf. <er>Shallop</er>.]</ety> <sn>1.</sn> <fld>(Naut.)</fld> <def>A vessel having one mast and fore-and-aft rig, consisting of a boom-and-gaff mainsail, jibs, staysail, and gaff topsail. The typical sloop has a fixed bowsprit, topmast, and standing rigging, while those of a cutter are capable of being readily shifted.  The sloop usually carries a centerboard, and depends for stability upon breadth of beam rather than depth of keel.  The two types have rapidly approximated since 1880.  One radical distinction is that a sloop may carry a centerboard. See <er>Cutter</er>, and <xex>Illustration</xex> in Appendix.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Naut.)</fld> <def>In modern usage, a sailing vessel having one mast, commonly with a Bermuda rig, with either a center-board or a keel.  In the United States, a sloop may have one or two headsails, while in Western Europe and Great Britain a sloop has only one headsail.</def><br/
[<source>RDH</source>]</p>

<p><cs><col><b>Sloop of war</b></col>, <cd>formerly, a vessel of war rigged either as a ship, brig, or schooner, and mounting from ten to thirty-two guns; now, any war vessel larger than a gunboat, and carrying guns on one deck only.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Slop</ent><br/
<hw>Slop</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[OE. <ets>sloppe</ets> a pool; akin to As. <ets>sloppe</ets>, <ets>slyppe</ets>, the sloppy droppings of a cow; cf. AS. <ets>sl<?/pan</ets> to slip, and E. <ets>slip</ets>, v.i.  Cf. <er>Cowslip</er>.]</ety> <sn>1.</sn> <def>Water or other liquid carelessly spilled or thrown aboyt, as upon a table or a floor; a puddle; a soiled spot.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Mean and weak drink or liquid food; -- usually in the plural.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <pluf>pl.</pluf> <def>Dirty water; water in which anything has been washed or rinsed; water from wash-bowls, etc.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><mcol><col><b>Slop basin</b></col>, <it>or</it> <col><b>Slop bowl</b></col></mcol>, <cd>a basin or bowl for holding slops, especially for receiving the rinsings of tea or coffee cups at the table.</cd> -- <col><b>Slop molding</b></col> <fld>(Brickmaking)</fld>, <cd>a process of manufacture in which the brick is carried to the drying ground in a wet mold instead of on a pallet.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Slop</ent><br/
<hw>Slop</hw>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Slopped</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Slopping</conjf>.]</vmorph> <sn>1.</sn> <def>To cause to overflow, as a liquid, by the motion of the vessel containing it; to spill.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To spill liquid upon; to soil with a liquid spilled.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Slop</ent><br/
<hw>Slop</hw>, <pos>v. i.</pos> <def>To overflow or be spilled as a liquid, by the motion of the vessel containing it; -- often with <xex>over</xex>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Slop</ent><br/
<hw>Slop</hw>, <pos>n.</pos> <ety>[AS. <ets>slop</ets> a frock or over-garment, fr. <ets>sl<?/pan</ets> to slip, to slide; akin to Icel. <ets>sloppr</ets> a thin garment; cf. OHG. <ets>slouf</ets> a garment.  Cf. <er>Slip</er>, <pos>v. i.</pos>]</ety> <sn>1.</sn> <def>Any kind of outer garment made of linen or cotton, as a night dress, or a smock frock.</def> <mark>[Obs.]</mark>  <rj><au>Halliwell.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A loose lower garment; loose breeches; chiefly used in the plural.</def> <ldquo/A pair of <xex>slops</xex>.<rdquo/  <rj><au>Sir P. Sidney.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>There's a French salutation to your French <qex>slop</qex>.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <pluf>pl.</pluf> <def>Ready-made clothes; also, among seamen, clothing, bedding, and other furnishings.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Slope</ent><br/
<hw>Slope</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Formed (like <ets>abode</ets> fr. <ets>abide</ets>) from OE. <ets>slipen</ets>. See <er>Slip</er>, <pos>v. i.</pos>]</ety> <sn>1.</sn> <def>An oblique direction; a line or direction including from a horizontal line or direction; also, sometimes, an inclination, as of one line or surface to another.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Any ground whose surface forms an angle with the plane of the horizon.</def><br/
[<source>1913 Webster</source>]</p>

<p><q> buildings the summit and <qex>slope</qex> of a hill.</q> <rj><qau>Macaulay.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Under the <qex>slopes</qex> of Pisgah.</q> <rj><qau>Deut. iv. 49. (Rev. Ver.).</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>The part of a continent descending toward, and draining to, a particular ocean; <as>as, the Pacific <ex>slope</ex></as>.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><note><hand/ A <xex>slope</xex>, considered as <xex>descending</xex>, is a <xex>declivity</xex>; considered as <xex>ascending</xex>, an <xex>acclivity</xex>.</note><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Slope of a plane</b></col> <fld>(Geom.)</fld>, <cd>the direction of the plane; <as>as, parallel planes have the same <ex>slope</ex></as>.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Slope</ent><br/
<hw>Slope</hw>, <pos>a.</pos> <def>Sloping.</def> <ldquo/Down the <xex>slope</xex> hills.<rdquo/  <rj><au>Milton.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>A bank not steep, but gently <qex>slope</qex>.</q> <rj><qau>Bacon.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Slope</ent><br/
<hw>Slope</hw>, <pos>adv.</pos> <def>In a sloping manner.</def> <mark>[Obs.]</mark>  <rj><au>Milton.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Slope</ent><br/
<hw>Slope</hw>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Sloped</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Sloping</conjf>.]</vmorph> <def>To form with a slope; to give an oblique or slanting direction to; to direct obliquely; to incline; to slant; <as>as, to <ex>slope</ex> the ground in a garden; to <ex>slope</ex> a piece of cloth in cutting a garment.</as></def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Slope</ent><br/
<hw>Slope</hw>, <pos>v. i.</pos> <sn>1.</sn> <def>To take an oblique direction; to be at an angle with the plane of the horizon; to incline; <as>as, the ground <ex>slopes</ex></as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To depart; to disappear suddenly.</def> <mark>[Slang]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Slopeness</ent><br/
<hw>Slope"ness</hw>, <pos>n.</pos> <def>State of being slope.</def>  <rj><au>Sir H. Wotton.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Slopewise</ent><br/
<hw>Slope"wise`</hw> <pr>(?)</pr>, <pos>adv.</pos> <def>Obliquely.</def> <mark>[Obs.]</mark>  <rj><au>Carew.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sloping</ent><br/
<hw>Slop"ing</hw>, <pos>a.</pos> <def>Inclining or inclined from the plane of the horizon, or from a horizontal or other right line; oblique; declivous; slanting.</def> -- <wordforms><wf>Slop"ing*ly</wf>, <pos>adv.</pos></wordforms><br/
[<source>1913 Webster</source>]</p>

<p><q>The <qex>sloping</qex> land recedes into the clouds.</q> <rj><qau>Cowper.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sloppiness</ent><br/
<hw>Slop"pi*ness</hw> <pr>(?)</pr>, <pos>n.</pos> <def>The quality or state of being sloppy; muddiness.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sloppy</ent><br/
<hw>Slop"py</hw> <pr>(?)</pr>, <pos>a.</pos> <amorph>[<pos>Compar.</pos> <adjf>Sloppier</adjf> <pr>(?)</pr>; <pos>superl.</pos> <adjf>Sloppiest</adjf>.]</amorph> <ety>[From <er>Slop</er>.]</ety> <def>Wet, so as to spatter easily; wet, as with something slopped over; muddy; plashy; <as>as, a <ex>sloppy</ex> place, walk, road</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Slopseller</ent><br/
<hw>Slop"sell`er</hw> <pr>(?)</pr>, <pos>n.</pos> <def>One who sells slops, or ready-made clothes. See 4th <er>Slop</er>, 3.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Slopshop</ent><br/
<hw>Slop"shop`</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A shop where slops. or ready-made clothes, are sold.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Slopwork</ent><br/
<hw>Slop"work`</hw> <pr>(?)</pr>, <pos>n.</pos> <def>The manufacture of slops, or cheap ready-made clothing; also, such clothing; hence, hasty, slovenly work of any kind.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>No <qex>slopwork</qex> ever dropped from his [Carlyle's] pen.</q> <rj><qau>Froude.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Slopy</ent><br/
<hw>Slop"y</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Sloping; inclined.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sloshy</ent><br/
<ent>Slosh</ent><br/
<mhw>{ <hw>Slosh</hw> <pr>(?)</pr>, <hw>Slosh"y</hw> <pr>(?)</pr> }</mhw>. <def>See <er>Slush</er>, <er>Slushy</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Slot</ent><br/
<hw>Slot</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[LG. & D. <ets>slot</ets> a lock, from a verb meaning to close., to shut, D. <ets>sluiten</ets>; akin to G. <ets>schliessen</ets>, OHG. <ets>sliozan</ets>, OFries. <ets>sl<?/ta</ets>, and probably to L. <ets>claudere</ets>.  Cf. <er>Close</er>, <er>Sluice</er>.]</ety> <sn>1.</sn> <def>A broad, flat, wooden bar; a slat or sloat.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A bolt or bar for fastening a door.</def> <mark>[Prov. Eng.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>A narrow depression, perforation, or aperture; esp., one for the reception of a piece fitting or sliding in it.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Slot</ent><br/
<hw>Slot</hw> <pr>(?)</pr>, <pos>v. t.</pos> <ety>[See <er>Slot</er> a bar.]</ety> <def>To shut with violence; to slam; <as>as, to <ex>slot</ex> a door</as>.</def> <mark>[Obs. or Prov. Eng.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Slot</ent><br/
<hw>Slot</hw>, <pos>n.</pos> <ety>[Cf. Icel. <ets>sl<?/<?/</ets>, and E. <ets>sleuth</ets>.]</ety> <def>The track of a deer; hence, a track of any kind.</def>  <rj><au>Milton.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>As a bloodhound follows the <qex>slot</qex> of a hurt deer.</q> <rj><qau>Sir W. Scott.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sloth</ent><br/
<hw>Sloth</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[OE. <ets>slouthe</ets>, <ets>sleuthe</ets>, AS. <ets>sl<?/w<?/</ets>, fr. <ets>sl<amac/w</ets> slow. See <er>Slow</er>.]</ety> <sn>1.</sn> <def>Slowness; tardiness.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>These cardinals trifle with me; I abhor<br/
This dilatory <qex>sloth</qex> and tricks of Rome.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Disinclination to action or labor; sluggishness; laziness; idleness.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>[They] change their course to pleasure, ease, and <qex>sloth</qex>.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q><qex>Sloth</qex>, like rust, consumes faster than labor wears.</q> <rj><qau>Franklin.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <fld>(Zool.)</fld> <def>Any one of several species of arboreal edentates constituting the family <fam>Bradypodidae</fam>, and the suborder Tardigrada. They have long exserted limbs and long prehensile claws.  Both jaws are furnished with teeth (see <xex>Illust.</xex> of <er>Edentata</er>), and the ears and tail are rudimentary. They inhabit South and Central America and Mexico.</def><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ The three-toed sloths belong to the genera <gen>Bradypus</gen> and <gen>Arctopithecus</gen>, of which several species have been described. They have three toes on each foot. The best-known species are collared sloth (<spn>Bradypus tridactylus</spn>), and the ai (<spn>Arctopitheus ai</spn>). The two-toed sloths, consisting the genus <gen>Cholopus</gen>, have two toes on each fore foot and three on each hind foot. The best-known is the unau (<spn>Cholopus didactylus</spn>) of South America. See <er>Unau</er>. Another species (<spn>Cholopus Hoffmanni</spn>) inhabits Central America.<br/
   Various large extinct terrestrial edentates, such as Megatherium and Mylodon, are often called <ex>sloths</ex>.</note><br/
[<source>1913 Webster</source>]</p>

<p><cs><mcol><col><b>Australian sloth</b></col>, <it>or</it>  <col><b>Native sloth</b></col></mcol> <fld>(Zool.)</fld>, <cd>the koala.</cd> -- <col><b>Sloth animalcule</b></col> <fld>(Zool.)</fld>, <cd>a tardigrade.</cd> -- <col><b>Sloth bear</b></col> <fld>(Zool.)</fld>, <cd>a black or brown long-haired bear (<spn>Melursus ursinus</spn>, <it>or</it> <spn>Melursus labiatus</spn>), native of India and Ceylon; -- called also <altname>aswail</altname>, <altname>labiated bear</altname>, and <altname>jungle bear</altname>.  It is easily tamed and can be taught many tricks.</cd> -- <col><b>Sloth monkey</b></col> <fld>(Zool.)</fld>, <cd>a loris.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sloth</ent><br/
<hw>Sloth</hw>, <pos>v. i.</pos> <def>To be idle.</def> <mark>[Obs.]</mark>  <rj><au>Gower.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Slothful</ent><br/
<hw>Sloth"ful</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Addicted to sloth; inactive; sluggish; lazy; indolent; idle.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>He also that is <qex>slothful</qex> in his work is brother to him that is a great waster.</q> <rj><qau>Prov. xviii. 9.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p>-- <wordforms><wf>Sloth"ful*ly</wf>, <pos>adv.</pos> -- <wf>Sloth"ful*ness</wf>, <pos>n.</pos></wordforms><br/
[<source>1913 Webster</source>]</p>

<p><ent>Slothhound</ent><br/
<hw>Sloth"hound`</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[See <er>Slot</er> a track, and cf. <er>Sleuthhound</er>.]</ety> <fld>(Zool.)</fld> <def>See <er>Sleuthhound</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Slot machine</ent><br/
<hw>Slot" ma*chine"</hw>. <def>A machine the operation of which is started by dropping a coin into a slot, for delivering small articles of merchandise, showing one's weight, exhibiting pictures, throwing dice, gambling, etc.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Slotted</ent><br/
<hw>Slot"ted</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Having a slot.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Slotting</ent><br/
<hw>Slot"ting</hw> <pr>(?)</pr>, <pos>n.</pos> <def>The act or process of making slots, or mortises.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Slouch</ent><br/
<hw>Slouch</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Cf. Icel. <ets>sl<?/kr</ets>a slouching felloew, and E. <ets>slack</ets>, <ets>slug</ets>, a lazy fellow.]</ety> <sn>1.</sn> <def>A hanging down of the head; a drooping attitude; a limp appearance; an ungainly, clownish gait; a sidewise depression or hanging down, as of a hat brim.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>An awkward, heavy, clownish fellow.</def> <mark>[Colloq.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Slouth hat</b></col>, <cd>a soft, limp hat of unstiffened cloth or felt.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Slouch</ent><br/
<hw>Slouch</hw>, <pos>v. i.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Slouched</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Slouching</conjf>.]</vmorph> <sn>1.</sn> <def>To droop, as the head.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To walk in a clumsy, lazy manner.</def> <mark>[Colloq.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Slouch</ent><br/
<hw>Slouch</hw>, <pos>v. t.</pos> <def>To cause to hang down; to depress at the side; <as>as, to <ex>slouth</ex> the hat</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Slouching</ent><br/
<hw>Slouch"ing</hw>, <pos>a.</pos> <def>Hanging down at the side; limp; drooping; without firmness or shapeliness; moving in an ungainly manner.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Slouchy</ent><br/
<hw>Slouch"y</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Slouching.</def> <mark>[Colloq.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Slough</ent><br/
<hw>Slough</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Slow.</def> <mark>[Obs.]</mark>  <rj><au>Chaucer.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Slough</ent><br/
<hw>Slough</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[OE. <ets>slogh</ets>, <ets>slough</ets>, AS. <ets>sl<omac/h</ets> a hollow place; cf. MHG. <ets>sl<umac/ch</ets> an abyss, gullet, G. <ets>schlucken</ets> to swallow; also Gael. & Ir. <ets>sloc</ets> a pit, pool. ditch, Ir. <ets>slug</ets> to swallow. Gr. <?/<?/<?/<?/<?/ to hiccough, to sob.]</ety> <sn>1.</sn> <def>A place of deep mud or mire; a hole full of mire.</def>  <rj><au>Chaucer.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>He's here stuck in a <qex>slough</qex>.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <pr>[<it>Pronounced</it> sl<oomac/.]</pr> <def>A wet place; a swale; a side channel or inlet from a river.</def> <note>[In this sense local or provincial; also spelt <asp>sloo</asp>, and <asp>slue</asp>.]</note><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Slough grass</b></col> <fld>(Bot.)</fld>, <cd>a name in the Mississippi valley for grasses of the genus <gen>Muhlenbergia</gen>; -- called also <altname>drop seed</altname>, and <altname>nimble Will</altname>.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Slough</ent><br/
<hw>Slough</hw>, <mark>obs.</mark> <def><pos>imp.</pos> of <er>Slee</er>, to slay. Slew.</def>  <rj><au>Chaucer.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Slough</ent><br/
<hw>Slough</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[OE. <ets>slugh</ets>, <ets>slouh</ets>; cf. MHG. <ets>sl<?/ch</ets> the skin of a serpent, G. <ets>schlauch</ets> a skin, a leather bag or bottle.]</ety> <sn>1.</sn> <def>The skin, commonly the cast-off skin, of a serpent or of some similar animal.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Med.)</fld> <def>The dead mass separating from a foul sore; the dead part which separates from the living tissue in mortification.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Slough</ent><br/
<hw>Slough</hw>, <pos>v. i.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Sloughed</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Sloughing</conjf>.]</vmorph> <fld>(Med.)</fld> <def>To form a slough; to separate in the form of dead matter from the living tissues; -- often used with <xex>off</xex>, or <xex>away</xex>; <as>as, a <ex>sloughing</ex> ulcer; the dead tissues <ex>slough</ex> off slowly.</as></def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Slough</ent><br/
<hw>Slough</hw>, <pos>v. t.</pos> <def>To cast off; to discard as refuse.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>New tint the plumage of the birds,<br/
And <qex>slough</qex> decay from grazing herds.</q> <rj><qau>Emerson.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sloughing</ent><br/
<hw>Slough"ing</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Zool.)</fld> <def>The act of casting off the skin or shell, as do insects and crustaceans; ecdysis.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sloughy</ent><br/
<hw>Slough"y</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Full of sloughs, miry.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sloughy</ent><br/
<hw>Slough"y</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Resembling, or of the nature of, a slough, or the dead matter which separates from living flesh.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sloven</ent><br/
<hw>Slov"en</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[D. <ets>slaf</ets> careless, negligent, a sloven; akin to LG. <ets>sluf</ets> slovenly.]</ety> <def>A man or boy habitually negligent of neathess and order; -- the correlative term to <xex>slattern</xex>, or <xex>slut</xex>.</def>  <rj><au>Pope.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>He became a confirmed <qex>sloven</qex>.</q> <rj><qau>Macaulay.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Slovenliness</ent><br/
<hw>Slov"en*li*ness</hw> <pr>(?)</pr>, <pos>n.</pos> <def>The quality or state of being slovenly.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Slovenly</ent><br/
<hw>Slov"en*ly</hw>, <pos>a.</pos> <sn>1.</sn> <def>Having the habits of a sloven; negligent of neatness and order, especially in dress.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>A <qex>slovenly</qex>, lazy fellow, lolling at his ease.</q> <rj><qau>L'Estrange.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Characteristic of a sloven; lacking neatness and order; evincing negligence; <as>as, <ex>slovenly</ex> dress</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Slovenly</ent><br/
<hw>Slov"en*ly</hw>, <pos>adv.</pos> <def>a slovenly manner.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Slovenness</ent><br/
<hw>Slov"en*ness</hw>, <pos>n.</pos> <def>Slovenliness.</def> <mark>[Obs.]</mark>  <rj><au>Fuller.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Slovenry</ent><br/
<hw>Slov"en*ry</hw> <pr>(?)</pr>, <pos>n.</pos> <def>Slovenliness.</def> <mark>[Obs.]</mark>  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Slow</ent><br/
<hw>Slow</hw> <pr>(sl<omac/)</pr>, <mark>obs.</mark> <pos>imp.</pos> <mord>of <er>Slee</er>, to slay.</mord> <def>Slew.</def>  <rj><au>Chaucer.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Slow</ent><br/
<hw>Slow</hw> <pr>(sl<omac/)</pr>, <pos>a.</pos> <amorph>[<pos>Compar.</pos> <adjf>Slower</adjf> <pr>(sl<omac/"<etil/r)</pr>; <pos>superl.</pos> <adjf>Slowest</adjf>.]</amorph> <ety>[OE. <ets>slow</ets>, <ets>slaw</ets>, AS. <ets>sl<amac/w</ets>; akin to OS. <ets>sl<emac/u</ets> blunt, dull, D. <ets>sleeuw</ets>, <ets>slee</ets>, sour, OHG. <ets>sl<emac/o</ets> blunt, dull, Icel. <ets>sl<omac/r</ets>, <ets>sl<ae/r</ets>, Dan. <ets>sl<oum/v</ets>, Sw. <ets>sl<oum/</ets>.  Cf. <er>Sloe</er>, and <er>Sloth</er>.]</ety> <sn>1.</sn> <def>Moving a short space in a relatively long time; not swift; not quick in motion; not rapid; moderate; deliberate; <as>as, a <ex>slow</ex> stream; a <ex>slow</ex> motion</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Not happening in a short time; gradual; late.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>These changes in the heavens, though <qex>slow</qex>, produced<br/
Like change on sea and land, sidereal blast.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>Not ready; not prompt or quick; dilatory; sluggish; <as>as, <ex>slow</ex> of speech, and <ex>slow</ex> of tongue</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Fixed on defense, the Trojans are not <qex>slow</qex><br/
To guard their shore from an expected foe.</q> <rj><qau>Dryden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>Not hasty; not precipitate; acting with deliberation; tardy; inactive.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>He that is <qex>slow</qex> to wrath is of great understanding.</q> <rj><qau>Prov. xiv. 29.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <def>Behind in time; indicating a time earlier than the true time; <as>as, the clock or watch is <ex>slow</ex></as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>6.</sn> <def>Not advancing or improving rapidly; <as>as, the <ex>slow</ex> growth of arts and sciences</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>7.</sn> <def>Heavy in wit; not alert, prompt, or spirited; wearisome; dull.</def> <mark>[Colloq.]</mark>  <rj><au>Dickens. Thackeray.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ <xex>Slow</xex> is often used in the formation of compounds for the most part self-explaining; as, <xex>slow</xex>-gaited, <xex>slow</xex>-paced, <xex>slow</xex>-sighted, <xex>slow</xex>-winged, and the like.</note><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Slow coach</b></col>, <cd>a slow person. See def.7, above.</cd> <mark>[Colloq.]</mark> -- <mcol><col><b>Slow lemur</b></col>, <it>or</it> <col><b>Slow loris</b></col></mcol> <fld>(Zool.)</fld>, <cd> an East Indian nocturnal lemurine animal (<spn>Nycticebus tardigradus</spn>) about the size of a small cat; -- so called from its slow and deliberate movements. It has very large round eyes and is without a tail. Called also <altname>bashful Billy</altname>.</cd> -- <col><b>Slow match</b></col>. <cd>See under <er>Match</er>.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><syn><b>Syn.</b> -- Dilatory; late; lingering; tardy; sluggish; dull; inactive.</syn>  -- <usage><er>Slow</er>, <er>Tardy</er>, <er>Dilatory</er>. <xex>Slow</xex> is the wider term, denoting either a want of rapid motion or inertness of intellect. <xex>Dilatory</xex> signifies a proneness to <xex>defer</xex>, a habit of delaying the performance of what we know must be done. <xex>Tardy</xex> denotes the habit of being behind hand; as, <xex>tardy</xex> in making up one's acounts.</usage><br/
[<source>1913 Webster</source>]</p>

<p><ent>Slow</ent><br/
<hw>Slow</hw>, <pos>adv.</pos> <def>Slowly.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Let him have time to mark how <qex>slow</qex> time goes<br/
In time of sorrow.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Slow</ent><br/
<hw>Slow</hw>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Slowed</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Slowing</conjf>.]</vmorph> <def>To render slow; to slacken the speed of; to retard; to delay; <as>as, to <ex>slow</ex> a steamer</as>.</def>  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Slow</ent><br/
<hw>Slow</hw>, <pos>v. i.</pos> <def>To go slower; -- often with <xex>up</xex>; <as>as, the train <ex>slowed</ex> up before crossing the bridge</as>.</def>
<-- also with down. --><br/
[<source>1913 Webster</source>]</p>

<p><ent>Slow</ent><br/
<hw>Slow</hw>, <pos>n.</pos> <def>A moth.</def> <mark>[Obs.]</mark>  <rj><au>Rom. of R.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Slowback</ent><br/
<hw>Slow"back`</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A lubber; an idle fellow; a loiterer.</def> <mark>[Old Slang]</mark>  <rj><au>Dr. Favour.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Slowh</ent><br/
<hw>Slowh</hw> <pr>(?)</pr>, <mark>obs.</mark> <def><pos>imp.</pos> of <er>Slee</er>,to slay.</def>  <rj><au>Chaucer.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Slowhound</ent><br/
<hw>Slow"hound`</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A sleuthhound.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Slowly</ent><br/
<hw>Slow"ly</hw>, <pos>adv.</pos> <def>In a slow manner; moderately; not rapidly; not early; not rashly; not readly; tardly.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Slowness</ent><br/
<hw>Slow"ness</hw>, <pos>n.</pos> <def>The quality or state of being slow.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Slows</ent><br/
<hw>Slows</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Med.)</fld> <def>Milk sickness.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Slow-witted</ent><br/
<hw>Slow"-wit`ted</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Dull of apprehension; not possessing quick intelligence.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Slowworm</ent><br/
<hw>Slow"worm`</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[AS. <ets>sl<amac/wyrm</ets>; the first part is probably akin to <ets>sle<aacute/n</ets> to strike, the reptile being supposed to be very poisonous. See <er>Slay</er>, <pos>v. t.</pos>, and <er>Worm</er>.]</ety> <fld>(Zool.)</fld> <def>A lecertilian reptile; the blindworm.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sloyd</ent><br/
<hw>Sloyd</hw> <pr>(?)</pr>, <pos>n.</pos> <altsp>[Written also <asp>slojd</asp>, and <asp>sloid</asp>.]</altsp> <ety>[Sw. <ets>sl<oum/jd</ets> skill, dexterity, esp. skilled labor, hence, manufacture, wood carving.]</ety> <def>Lit., skilled mechanical work, such as that required in wood carving; trade work; hence, a system (usually called the <xex>sloyd system</xex>) of manual training in the practical use of the tools and materials used in the trades, and of instruction in the making and use of the plans and specifications connected with trade work. The  sloyd system derives its name from the fact that it was adopted or largely developed from a similar Swedish system, in which wood carving was a chief feature. Its purpose is not only to afford practical skill in some trade, but also to develop the pupils mentally and physically.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Slub</ent><br/
<hw>Slub</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Etymol. uncertain.]</ety> <def>A roll of wool slightly twisted; a rove; -- called also <altname>slubbing</altname>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Slub</ent><br/
<hw>Slub</hw>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Slubbed</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Slubbing</conjf>.]</vmorph> <def>To draw out and twist slightly; -- said of slivers of wool.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Slubber</ent><br/
<hw>Slub"ber</hw> <pr>(?)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Slubbered</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Slubbering</conjf>.]</vmorph> <ety>[Cf. Dan. <ets>slubbre</ets>to swallow, to sup up, D. <ets>slobberen</ets> to lap, to slabber.  Cf. <er>Slabber</er>.]</ety> <sn>1.</sn> <def>To do lazily, imperfectly, or coarsely.</def><br/
[<source>1913 Webster</source>]</p>

<p><q><qex>Slubber</qex> not business for my sake.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To daub; to stain; to cover carelessly.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>There is no art that hath more . . . <qex>slubbered</qex> with aphorisming pedantry than the art of policy.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Slubber</ent><br/
<hw>Slub"ber</hw>, <pos>n.</pos> <def>A slubbing machine.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Slubberdegullion</ent><br/
<hw>Slub"ber*de*gul`lion</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[<ets>Slubber</ets> + Prov. E. <ets>gullion</ets> a wretch.]</ety> <def>A mean, dirty wretch.</def> <mark>[Low]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Slubberingly</ent><br/
<hw>Slub"ber*ing*ly</hw>, <pos>adv.</pos> <def>In a slovenly, or hurried and imperfect, manner.</def> <mark>[Low]</mark>  <rj><au>Drayton.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Slubbing</ent><br/
<hw>Slub"bing</hw> <pr>(?)</pr>, <def><pos>a. & n.</pos> from <er>Slub</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><mcol><col><b>Slubbing billy</b></col>, <it>or</it> <col><b>Slubbing machine</b></col></mcol>, <cd>the machine by which slubs are formed.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sludge</ent><br/
<hw>Sludge</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[CF. <er>Slush</er>.]</ety> <sn>1.</sn> <def>Mud; mire; soft mud; slush.</def>  <rj><au>Mortimer. Tennyson.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Small floating pieces of ice, or masses of saturated snow.</def>  <rj><au>Kane.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <fld>(Mining)</fld> <def>See <er>Slime</er>,  4.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>Anything resembling mud or slush; as: <sd>(a)</sd> A muddy or slimy deposit from sweage. <sd>(b)</sd> Mud from a drill hole in boring. <sd>(c)</sd> Muddy sediment in a steam boiler. <sd>(d)</sd> Settling of cottonseed oil, used in making soap, etc. <sd>(e)</sd> A residuum of crude paraffin-oil distillation.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><cs><col><b>Sludge hole</b></col>, <cd>the hand-hole, or manhole, in a steam boiler, by means of which sediment can be removed.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sludge acid</ent><br/
<hw>Sludge acid</hw>. <def>Impure dark-colored sulphuric acid that has been used in the refining of petroleum.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Sludger</ent><br/
<hw>Sludg"er</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A shovel for sludging out drains, etc.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><-- p. 1357 --></p>

<p><ent>Sludger</ent><br/
<hw>Slud"ger</hw> <pr>(sl<ucr/j"<etil/r)</pr>, <pos>n.</pos> <def>A bucket for removing mud from a bored hole; a sand pump.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sludy</ent><br/
<hw>Slud"y</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Miry; slushy.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Slue</ent><br/
<hw>Slue</hw> <pr>(?)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Slued</conjf> <pr>(<?/)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Sluing</conjf> <pr>(<?/)</pr>.]</vmorph> <ety>[Prov. E. <ets>slew</ets> to turn round, Scot. to lean or incline to a side; cf. Icel. <ets>sn<?/a</ets> to turn, bend.]</ety> <altsp>[Written also <asp>slew</asp>.]</altsp> <sn>1.</sn> <fld>(Naut.)</fld> <def>To turn about a fixed point, usually the center or axis, as a spar or piece of timber; to turn; -- used also of any heavy body.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>In general, to turn about; to twist; -- often used reflexively and followed by <xex>round</xex>.</def> <mark>[Colloq.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>They laughed, and <qex>slued</qex> themselves round.</q> <rj><qau>Dickens.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Slue</ent><br/
<hw>Slue</hw>, <pos>v. i.</pos> <def>To turn about; to turn from the course; to slip or slide and turn from an expected or desired course; -- often followed by <xex>round</xex>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Slue</ent><br/
<hw>Slue</hw>, <pos>n.</pos> <def>See <er>Sloough</er>, 2.</def> <mark>[Local]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Slug</ent><br/
<hw>Slug</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[OE. <ets>slugge</ets> slothful, <ets>sluggen</ets> to be slothful; cf. LG. <ets>slukk</ets> low-spirited, sad, E. <ets>slack</ets>, <ets>slouch</ets>, D. <ets>slak</ets>, <ets>slek</ets>, a snail.]</ety> <sn>1.</sn> <def>A drone; a slow, lazy fellow; a sluggard.</def>  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A hindrance; an obstruction.</def> <mark>[Obs.]</mark>  <rj><au>Bacon.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <fld>(Zool.)</fld> <def>Any one of numerous species of terrestrial pulmonate mollusks belonging to Limax and several related genera, in which the shell is either small and concealed in the mantle, or altogether wanting. They are closely allied to the land snails.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <fld>(Zool.)</fld> <def>Any smooth, soft larva of a sawfly or moth which creeps like a mollusk; <as>as, the pear <ex>slug</ex>; rose <ex>slug</ex></as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <def>A ship that sails slowly.</def> <mark>[Obs.]</mark>  <rj><au>Halliwell.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>His rendezvous for his fleet, and for all <qex>slugs</qex> to come to, should be between Calais and Dover.</q> <rj><qau>Pepys.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>6.</sn> <ety>[Perhaps a different word.]</ety> <def>An irregularly shaped piece of metal, used as a missile for a gun.</def><-- also, a colloq. term for bullet. --><br/
[<source>1913 Webster</source>]</p>

<p><sn>7.</sn> <fld>(Print.)</fld> <def>A thick strip of metal less than type high, and as long as the width of a column or a page, -- used in spacing out pages and to separate display lines, etc.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Sea slug</b></col>. <fld>(Zool.)</fld> <sd>(a)</sd> <cd>Any nudibranch mollusk</cd>. <sd>(b)</sd> <cd>A holothurian.</cd> -- <col><b>Slug caterpillar</b></col>. <cd>Same as <er>Slugworm</er>.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Slug</ent><br/
<hw>Slug</hw>, <pos>v. i.</pos> <def>To move slowly; to lie idle.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>To <qex>slug</qex> in sloth and sensual delight.</q> <rj><qau>Spenser.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Slug</ent><br/
<hw>Slug</hw>, <pos>v. t.</pos> <def>To make sluggish.</def> <mark>[Obs.]</mark>  <rj><au>Milton.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Slug</ent><br/
<hw>Slug</hw>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Slugged</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Slugging</conjf> <pr>(?)</pr>.]</vmorph> <sn>1.</sn> <def>To load with a slug or slugs; <as>as, to <ex>slug</ex> a gun</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To strike heavily.</def> <mark>[Cant or Slang]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Slug</ent><br/
<hw>Slug</hw>, <pos>v. i.</pos> <def>To become reduced in diameter, or changed in shape, by passing from a larger to a smaller part of the bore of the barrel; -- said of a bullet when fired from a gun, pistol, or other firearm.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Slugabed</ent><br/
<hw>Slug"a*bed`</hw> <pr>(?)</pr>, <pos>n.</pos> <def>One who indulges in lying abed; a sluggard.</def> <mark>[R.]</mark> <ldquo/Fie, you <xex>slugabed</xex>!<rdquo/  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sluggard</ent><br/
<hw>Slug"gard</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[<ets>Slug</ets> + <ets>-ard</ets>.]</ety> <def>A person habitually lazy, idle, and inactive; a drone.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Go to the ant, thou <qex>sluggard</qex>; consider her ways, and be wise.</q> <rj><qau>Prov. vi. 6.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sluggard</ent><br/
<hw>Slug"gard</hw>, <pos>a.</pos> <def>Sluggish; lazy.</def>  <rj><au>Dryden.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sluggardize</ent><br/
<hw>Slug"gard*ize</hw> <pr>(?)</pr>, <pos>v. t.</pos> <def>To make lazy.</def> <mark>[R.]</mark>  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sluggardy</ent><br/
<hw>Slug"gard*y</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[OE. <ets>sloggardye</ets>.]</ety> <def>The state of being a sluggard; sluggishness; sloth.</def>  <rj><au>Gower.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Idleness is rotten <qex>sluggardy</qex>.</q> <rj><qau>Chaucer.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Slugger</ent><br/
<hw>Slug"ger</hw> <pr>(?)</pr>, <pos>n.</pos> <def>One who strikes heavy blows; hence, a boxer; a prize fighter.</def> <mark>[Cant or Slang]</mark><br/
[<source>1913 Webster</source>]</p>

<p><-- (Baseball) A player with a high batting average, esp. one who hits many home runs. --><br/
[<source>1913 Webster</source>]</p>

<p><ent>Slugging match</ent><br/
<hw>Slug"ging match</hw>. <sd>(a)</sd> <def>A boxing match or prize fight marked rather by heavy hitting than skill.</def> <mark>[Cant or Slang]</mark> <sd>(b)</sd> <def>A ball game, esp. a baseball game, in which  there is much hard hitting of the ball.</def> <mark>[Slang, U. S.]</mark><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Slugging average</ent><br/
<hw>Slug"ging av`er*age</hw>. <fld>(Baseball)</fld> <def>a measure of the effectiveness of a batter at reaching base and advancing other runners, calculated as the sum of the number of bases reached on each hit, divided by the total number of times at bat.  A double counts two bases, a triple three, a home run four.  Thus a batter with four singles, two doubles and a triple in 20 official times at bat would have a <ex>slugging average</ex> of 0.550, and a <er>batting average</er> of 0.350.  Also called <altname>slugging percentage</altname>.  Compare <er>batting average</er>.</def><br/
[<source>PJC</source>]</p>

<p><ent>Sluggish</ent><br/
<hw>Slug"gish</hw> <pr>(?)</pr>, <pos>a.</pos> <sn>1.</sn> <def>Habitually idle and lazy; slothful; dull; inactive; <as>as, a <ex>sluggish</ex> man</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Slow; having little motion; <as>as, a <ex>sluggish</ex> stream</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>Having no power to move one's self or itself; inert.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Matter, being impotent, <qex>sluggish</qex>, and inactive, hath no power to stir or move itself.</q> <rj><qau>Woodward.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>And the <qex>sluggish</qex> land slumbers in utter neglect.</q> <rj><qau>Longfellow.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>Characteristic of a sluggard; dull; stupid; tame; simple.</def> <mark>[R.]</mark> <ldquo/So <xex>sluggish</xex> a conceit.<rdquo/  <rj><au>Milton.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><syn><b>Syn.</b> -- Inert; idle; lazy; slothful; indolent; dronish; slow; dull; drowsy; inactive. See <er>Inert</er>.</syn><br/
[<source>1913 Webster</source>]</p>

<p>-- <wordforms><wf>Slug"gish*ly</wf>, <pos>adv.</pos> -- <wf>Slug"gish*ness</wf>, <pos>n.</pos></wordforms><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sluggy</ent><br/
<hw>Slug"gy</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Sluggish.</def> <mark>[Obs.]</mark>  <rj><au>Chaucer.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Slug-horn</ent><br/
<hw>Slug"-horn`</hw> <pr>(?)</pr>, <pos>a.</pos> <def>An erroneous form of the Scotch word <asp>slughorne</asp>, or <asp>sloggorne</asp>, meaning <xex>slogan</xex>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Slugs</ent><br/
<hw>Slugs</hw> <pr>(?)</pr>, <pos>n. pl.</pos> <fld>(Mining)</fld> <def>Half-roasted ore.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Slugworm</ent><br/
<hw>Slug"worm`</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Zool.)</fld> <def>Any caterpillar which has the general appearance of a slug, as do those of certain moths belonging to <gen>Limacodes</gen> and allied genera, and those of certain sawflies.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sluice</ent><br/
<hw>Sluice</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[OF. <ets>escluse</ets>, F. <ets><eacute/cluse</ets>, LL. <ets>exclusa</ets>, <ets>sclusa</ets>, from L. <ets>excludere</ets>, <ets>exclusum</ets>, to shut out: cf. D. <ets>sluis</ets> sluice, from the Old French. See <er>Exclude</er>.]</ety> <sn>1.</sn> <def>An artifical passage for water, fitted with a valve or gate, as in a mill stream, for stopping or regulating the flow; also, a water gate or flood gate.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Hence, an opening or channel through which anything flows; a source of supply.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Each <qex>sluice</qex> of affluent fortune opened soon.</q> <rj><qau>Harte.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>This home familiarity . . . opens the <qex>sluices</qex> of sensibility.</q> <rj><qau>I. Taylor.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>The stream flowing through a flood gate.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <fld>(Mining)</fld> <def>A long box or trough through which water flows, -- used for washing auriferous earth.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Sluice gate</b></col>, <cd>the sliding gate of a sluice.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sluice</ent><br/
<hw>Sluice</hw>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Sluiced</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Sluicing</conjf> <pr>(?)</pr>.]</vmorph> <sn>1.</sn> <def>To emit by, or as by, flood gates.</def> <mark>[R.]</mark>  <rj><au>Milton.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To wet copiously, as by opening a sluice; <as>as, to <ex>sluice</ex> meadows</as>.</def>  <rj><au>Howitt.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>He dried his neck and face, which he had been <qex>sluicing</qex> with cold water.</q> <rj><qau>De Quincey.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>To wash with, or in, a stream of water running through a sluice; <as>as, to <ex>sluice</ex> eart or gold dust in mining</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sluiceway</ent><br/
<hw>Sluice"way`</hw> <pr>(?)</pr>, <pos>n.</pos> <def>An artificial channel into which water is let by a sluice; specifically, a trough constructed over the bed of a stream, so that logs, lumber, or rubbish can be floated down to some convenient place of delivery.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sluicy</ent><br/
<hw>Slui`cy</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Falling copiously or in streams, as from a sluice.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>And oft whole sheets descend of <qex>sluicy</qex> rain.</q> <rj><qau>Dryden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Slum</ent><br/
<hw>Slum</hw> <pr>(sl<ucr/m)</pr>, <pos>n.</pos> <ety>[CF. <er>Slump</er>, <pos>n.</pos>]</ety> <sn>1.</sn> <def>A foul back street of a city, especially one filled with a poor, dirty, degraded, and often vicious population; any low neighborhood or dark retreat; -- usually in the plural; <as>as, Westminster <ex>slums</ex> are haunts for theives</as>.</def>  <rj><au>Dickens.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <pluf>pl.</pluf> <fld>(Mining)</fld> <def>Same as <er>Slimes</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Slum</ent><br/
<hw>Slum</hw> <pr>(sl<ucr/m)</pr>, <pos>v. i.</pos> <def>To visit or frequent slums, esp. out of curiosity, or for purposes of study, etc.  Also called <altname>go slumming</altname>.</def> <mark>[Colloq.]</mark><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Slumber</ent><br/
<hw>Slum"ber</hw> <pr>(?)</pr>, <pos>v. i.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Slumbered</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Slumbering</conjf>.]</vmorph> <ety>[OE. <ets>slombren</ets>, <ets>slumberen</ets>, <ets>slumeren</ets>, AS. <ets>slumerian</ets>, fr. <ets>sluma</ets> slumber; akin to D. <ets>sluimeren</ets> to slumber, MHG. <ets>slummern</ets>, <ets>slumen</ets>, G. <ets>schlummern</ets>, Dan. <ets>slumre</ets>, Sw. <ets>slumra</ets>, Goth. <ets>slawan</ets> to be silent.]</ety> <sn>1.</sn> <def>To sleep; especially, to sleep lightly; to doze.</def>  <rj><au>Piers Plowman.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>He that keepeth Israel shall neither <qex>slumber</qex> nor sleep.</q> <rj><qau>Ps. cxxi. 4.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To be in a state of negligence, sloth, supineness, or inactivity.</def> <ldquo/Why <xex>slumbers</xex> Pope?<rdquo/  <rj><au>Young.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Slumber</ent><br/
<hw>Slum"ber</hw>, <pos>v. t.</pos> <sn>1.</sn> <def>To lay to sleep.</def> <mark>[R.]</mark>  <rj><au>Wotton.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To stun; to stupefy.</def> <mark>[Obs.]</mark>  <rj><au>Spenser.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Slumber</ent><br/
<hw>Slum"ber</hw>, <pos>n.</pos> <def>Sleep; especially, light sleep; sleep that is not deep or sound; repose.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>He at last fell into a <qex>slumber</qex>, and thence into a fast sleep, which detained him in that place until it was almost night.</q> <rj><qau>Bunyan.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Fast asleep? It is no matter;<br/
Enjoy the honey-heavy dew of <qex>slumber</qex>.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Rest to my soul, and <qex>slumber</qex> to my eyes.</q> <rj><qau>Dryden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Slumberer</ent><br/
<hw>Slum"ber*er</hw> <pr>(?)</pr>, <pos>n.</pos> <def>One who slumbers; a sleeper.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Slumberingly</ent><br/
<hw>Slum"ber*ing*ly</hw>, <pos>adv.</pos> <def>In a slumbering manner.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Slumberless</ent><br/
<hw>Slum"ber*less</hw>, <pos>a.</pos> <def>Without slumber; sleepless.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Slumberous</ent><br/
<hw>Slum"ber*ous</hw> <pr>(?)</pr>, <pos>a.</pos> <sn>1.</sn> <def>Inviting slumber; soporiferous.</def> <ldquo/Pensive in the <xex>slumberous</xex> shade.<rdquo/  <rj><au>Pope.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Being in the repose of slumber; sleepy; drowsy.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>His quiet and almost <qex>slumberous</qex> countenance.</q> <rj><qau>Hawthorne.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Slumbery</ent><br/
<hw>Slum"ber*y</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Sleepy.</def> <mark>[Obs.]</mark>  <rj><au>Chaucer.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Slumbrous</ent><br/
<hw>Slum"brous</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Slumberous.</def>  <rj><au>Keats.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Slumgum</ent><br/
<hw>Slum"gum`</hw> <pr>(?)</pr>, <pos>n.</pos> <def>The impure residue, consisting of cocoons, propolis, etc., remaining after the wax is extracted from honeycombs.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Slumming</ent><br/
<hw>Slum"ming</hw>, <pos>vb. n.</pos> <def>Visiting slums.  Sometimes used humorously, in reference to the appearance of a dignified person in a situation generally thought of as low-class.</def><br/
[<source>1913 Webster</source> <source>+PJC</source>]</p>

<p><ent>Slump</ent><br/
<hw>Slump</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Cf. D. <ets>slomp</ets> a mass, heap, Dan. <ets>slump</ets> a quantity, and E. <ets>slump</ets>, v.t.]</ety> <def>The gross amount; the mass; the lump.</def> <mark>[Scot.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Slump</ent><br/
<hw>Slump</hw>, <pos>v. t.</pos> <ety>[Cf. <er>Lump</er>; also Sw. <ets>slumpa</ets> to bargain for the lump.]</ety> <def>To lump; to throw into a mess.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>These different groups . . . are exclusively <qex>slumped</qex> together under that sense.</q> <rj><qau>Sir W. Hamilton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Slump</ent><br/
<hw>Slump</hw>, <pos>v. i.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Slumped</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Slumping</conjf>.]</vmorph> <ety>[Scot. <ets>slump</ets> a dull noise produced by something falling into a hole, a marsh, a swamp.]</ety> <sn>1.</sn> <def>To fall or sink suddenly through or in, when walking on a surface, as on thawing snow or ice, partly frozen ground, a bog, etc., not strong enough to bear the person.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The latter walk on a bottomless quag, into which unawares they may <qex>slump</qex>.</q> <rj><qau>Barrow.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To slide or slip on a declivity, so that the motion is perceptible; -- said of masses of earth or rock.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><sn>3.</sn>  <def>To undergo a slump, or sudden decline or falling off; <as>as, the stock <ex>slumped</ex> ten points</as>.</def> <mark>[Colloq.]</mark><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Slump</ent><br/
<hw>Slump</hw>, <pos>n.</pos> <sn>1.</sn> <def>A boggy place.</def> <mark>[Prov. Eng. & Scot.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>The noise made by anything falling into a hole, or into a soft, miry place.</def> <mark>[Scot.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>A falling or declining, esp. suddenly and markedly; a falling off; <as>as, a <ex>slump</ex> in trade, in stock market prices, in a batter's average, etc.</as></def> <mark>[Colloq.]</mark><br/
[<source>Webster 1913 Suppl.</source> <source>+PJC</source>]</p>

<p><ent>Slumpy</ent><br/
<hw>Slump"y</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Easily broken through; boggy; marshy; swampy.</def> <mark>[Prov. Eng. & Colloq. U.S.]</mark>  <rj><au>Bartlett.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Slung</ent><br/
<hw>Slung</hw> <pr>(?)</pr>, <def><pos>imp. & p. p.</pos> of <er>Sling</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Slung shot</b></col>, <cd>a metal ball of small size, with a string attached, used by ruffians for striking.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Slunk</ent><br/
<hw>Slunk</hw> <pr>(sl<ucr/<nsm/k)</pr>, <def><pos>imp. & p. p.</pos> of <er>Slink</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Slur</ent><br/
<hw>Slur</hw> <pr>(sl<ucir/r)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Slurred</conjf> <pr>(sl<ucir/rd)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Slurring</conjf> <pr>(sl<ucir/r"r<icr/ng)</pr>.]</vmorph> <ety>[Cf. OE. <ets>sloor</ets> mud, clay, Icel. <ets>sl<omac/ra</ets>, <ets>slo<edh/ra</ets>, to trail or drag one's self along, D. <ets>sleuren</ets>, <ets>sloren</ets>, to train, to drag, to do negligently and slovenly, D. <ets>sloor</ets>, <ets>sloerie</ets>, a sluttish girl.]</ety> <sn>1.</sn> <def>To soil; to sully; to contaminate; to disgrace.</def>  <rj><au>Cudworth.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To disparage; to traduce.</def>  <rj><au>Tennyson.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>To cover over; to disguise; to conceal; to pass over lightly or with little notice.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>With periods, points, and tropes, he <qex>slurs</qex> his crimes.</q> <rj><qau>Dryden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>To cheat, as by sliding a die; to trick.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>To <qex>slur</qex> men of what they fought for.</q> <rj><qau>Hudibras.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <def>To pronounce indistinctly; <as>as, to <ex>slur</ex> syllables; to <ex>slur</ex> one's words</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>6.</sn> <fld>(Mus.)</fld> <def>To sing or perform in a smooth, gliding style; to connect smoothly in performing, as several notes or tones.</def>  <rj><au>Busby.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>7.</sn> <fld>(Print.)</fld> <def>To blur or double, as an impression from type; to mackle.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Slur</ent><br/
<hw>Slur</hw>, <pos>n.</pos> <sn>1.</sn> <def>A mark or stain; hence, a slight reproach or disgrace; a stigma; a reproachful intimation; an innuendo.</def> <ldquo/Gaining to his name a lasting <xex>slur</xex>.<rdquo/  <rj><au>South.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A trick played upon a person; an imposition.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <fld>(Mus.)</fld> <def>A mark, thus [<upslur/ or <downslur/], connecting notes that are to be sung to the same syllable, or made in one continued breath of a wind instrument, or with one stroke of a bow; a tie; a sign of legato.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>In knitting machines, a contrivance for depressing the sinkers successively by passing over them.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Slurred</ent><br/
<hw>Slurred</hw> <pr>(sl<ucir/rd)</pr>, <pos>a.</pos> <fld>(Mus.)</fld> <def>Marked with a slur; performed in a smooth, gliding style, like notes marked with a slur.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Slush</ent><br/
<hw>Slush</hw> <pr>(sl<ucr/sh)</pr>, <pos>n.</pos> <ety>[Cf. Sw. <ets>slaska</ets> to paddle in water, <ets>slask</ets> wet, filth.]</ety> <altsp>[Written also <asp>slosh</asp>.]</altsp> <sn>1.</sn> <def>Soft mud.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A mixture of snow and water; half-melted snow.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>A soft mixture of grease and other materials, used for lubrication.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>The refuse grease and fat collected in cooking, especially on shipboard.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <fld>(Mach.)</fld> <def>A mixture of white lead and lime, with which the bright parts of machines, such as the connecting rods of steamboats, are painted to be preserved from oxidation.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Slush</ent><br/
<hw>Slush</hw> <pr>(sl<ucr/sh)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Slushed</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Slushing</conjf>.]</vmorph> <sn>1.</sn> <def>To smear with slush or grease; <as>as, to <ex>slush</ex> a mast</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To paint with a mixture of white lead and lime.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Slushy</ent><br/
<hw>Slush"y</hw> <pr>(sl<ucr/sh"<ycr/)</pr>, <pos>a.</pos> <def>Abounding in slush; characterized by soft mud or half-melted snow; <as>as, the streets are <ex>slushy</ex>; the snow is <ex>slushy</ex></as>.</def> <ldquo/A dark, drizzling, <xex>slushy</xex> day.<rdquo/  <rj><au>Blackw. Mag.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Slut</ent><br/
<hw>Slut</hw> <pr>(sl<ucr/t)</pr>, <pos>n.</pos> <ety>[OE. <ets>slutte</ets>; cf. OD. <ets>slodde</ets> a slut, Icel. <ets>sl<oum/ttr</ets> a heavy, loglike fellow, <ets>slota</ets> to droop.]</ety> <sn>1.</sn> <def>An untidy woman; a slattern.</def><br/
[<source>1913 Webster</source>]</p>

<p><q><qex>Sluts</qex> are good enough to make a sloven's porridge.</q> <rj><qau>Old Proverb.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A servant girl; a drudge.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>Our little girl Susan is a most admirable <qex>slut</qex>, and pleases us mightily, doing more service than both the others.</q> <rj><qau>Pepys.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>A female dog; a bitch.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Slutch</ent><br/
<hw>Slutch</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[CF. <er>Sludge</er>.]</ety> <def>Slush.</def> <mark>[Prov. Eng.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Slutchy</ent><br/
<hw>Slutch"y</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Slushy.</def> <mark>[Prov. Eng.]</mark>  <rj><au>Pennant.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sluthhound</ent><br/
<hw>Sluth"hound`</hw> <pr>(?)</pr>, <pos>n.</pos> <def>Sleuthhound.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sluttery</ent><br/
<hw>Slut"ter*y</hw> <pr>(?)</pr>, <pos>n.</pos> <def>The qualities and practices of a slut; sluttishness; slatternlines.</def>  <rj><au>Drayton.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sluttish</ent><br/
<hw>Slut"tish</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Like a slut; untidy; indecently negligent of cleanliness; disorderly; <as>as, a <ex>sluttish</ex> woman</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Why is thy lord so <qex>slutish</qex>, I thee pray.</q> <rj><qau>Chaucer.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>An air of liberal, though <qex>sluttish</qex>, plenty, indicated the wealthy farmer.</q> <rj><qau>Sir W. Scott.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p>-- <wordforms><wf>Slut"tish*ly</wf>, <pos>adv.</pos> -- <wf>Slut"tish*ness</wf>, <pos>n.</pos></wordforms><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sly</ent><br/
<hw>Sly</hw> <pr>(?)</pr>, <pos>a.</pos> <amorph>[<pos>Compar.</pos> <adjf>Slier</adjf> <pr>(?)</pr> or <adjf>Slyer</adjf>; <pos>superl.</pos> <adjf>Sliest</adjf> or <adjf>Slyest</adjf>.]</amorph> <ety>[OE. <ets>sli</ets>, <ets>slegh</ets>, <ets>sleih</ets>, Icel. <ets>sl<?/gr</ets>, for <ets>sl<?/gr</ets>; akin to Sw. <ets>slug</ets>, Dan. <ets>slu</ets>, LG. <ets>slou</ets>, G. <ets>schlau</ets>; probably to E. <ets>slay</ets>, v.t.; cf. G. ver<ets>schlagen</ets> sly. See <er>Slay</er>, <pos>v. t.</pos>, and cf. <er>Sleight</er>.]</ety> <sn>1.</sn> <def>Dexterous in performing an action, so as to escape notice; nimble; skillful; cautious; shrewd; knowing; -- in a good sense.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Be ye <qex>sly</qex> as serpents, and simple as doves.</q> <rj><qau>Wyclif (Matt. x. 16).</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Whom graver age<br/
And long experience hath made wise and <qex>sly</qex>.</q> <rj><qau>Fairfax.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Artfully cunning; secretly mischievous; wily.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>For my <qex>sly</qex> wiles and subtle craftiness,<br/
The litle of the kingdom I possess.</q> <rj><qau>Spenser.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>Done with, and marked by, artful and dexterous secrecy; subtle; <as>as, a <ex>sly</ex> trick</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Envy works in a <qex>sly</qex> and imperceptible manner.</q> <rj><qau>I. Watts.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>Light or delicate; slight; thin.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><cs><mcol><col><b>By the sly</b></col>, <it>or</it> <col><b>On the sly</b></col></mcol>, <cd>in a sly or secret manner.</cd> <mark>[Colloq.]</mark> <ldquo/Gazed on Hetty's charms <xex>by the sly</xex>.<rdquo/ <au>G. Eliot.</au> -- <col><b>Sly goose</b></col> <fld>(Zool.)</fld>, <cd>the common sheldrake; -- so named from its craftiness.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><syn><b>Syn.</b> -- Cunning; crafty; subtile; wily. See <er>Cunning</er>.</syn><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sly</ent><br/
<hw>Sly</hw>, <pos>adv.</pos> <def>Slyly.</def> <mark>[Obs. or Poetic]</mark>  <rj><au>Spenser.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Slyboots</ent><br/
<hw>Sly"boots`</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A humerous appellation for a sly, cunning, or waggish person.</def><br/
[<source>1913 Webster</source>]</p>

<p><q><qex>Slyboots</qex> was cursedly cunning to hide 'em.</q> <rj><qau>Goldsmith.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Slyly</ent><br/
<hw>Sly"ly</hw>, <pos>adv.</pos> <def>In a sly manner; shrewdly; craftily.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Honestly and <qex>slyly</qex> he it spent.</q> <rj><qau>Chaucer.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Slyness</ent><br/
<hw>Sly"ness</hw>, <pos>n.</pos> <def>The quality or state of being sly.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Slype</ent><br/
<hw>Slype</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Cf. D. <ets>sluipen</ets> to sneak.]</ety> <fld>(Arch.)</fld> <def>A narrow passage between two buildings, as between the transept and chapter house of a monastery.</def> <mark>[Eng.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Smack</ent><br/
<hw>Smack</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[D. <ets>smak</ets>; akin to LG. <ets>smack</ets>, <ets>smak</ets>, Dan. <ets>smakke</ets>, G. <ets>schmacke</ets>, F. <ets>semaque</ets>.]</ety> <fld>(Naut.)</fld> <def>A small sailing vessel, commonly rigged as a sloop, used chiefly in the coasting and fishing trade.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Smack</ent><br/
<hw>Smack</hw>, <pos>n.</pos> <ety>[OE. <ets>smak</ets>, AS. <ets>ssm<?/c</ets> taste, savor; akin to D. <ets>smaak</ets>, G. ge<ets>schmack</ets>, OHG. <ets>smac</ets>; cf. Lith. <ets>smagus</ets> pleasant.  Cf. <er>Smack</er>, <pos>v. i.</pos>]</ety> <sn>1.</sn> <def>Taste or flavor, esp. a slight taste or flavor; savor; tincture; <as>as, a <ex>smack</ex> of bitter in the medicine</as>. Also used figuratively.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>So quickly they have taken a <qex>smack</qex> in covetousness.</q> <rj><qau>Robynson (More's Utopia).</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>They felt the <qex>smack</qex> of this world.</q> <rj><qau>Latimer.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A small quantity; a taste.</def>  <rj><au>Dryden.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>A loud kiss; a buss.</def> <ldquo/A clamorous <xex>smack</xex>.<rdquo/  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>A quick, sharp noise, as of the lips when suddenly separated, or of a whip.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <def>A quick, smart blow; a slap.</def>  <rj><au>Johnson.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Smack</ent><br/
<hw>Smack</hw>, <pos>adv.</pos> <def>As if with a smack or slap.</def> <mark>[Colloq.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Smack</ent><br/
<hw>Smack</hw>, <pos>v. i.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Smacked</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Smacking</conjf>.]</vmorph> <ety>[OE. <ets>smaken</ets> to taste, have a taste, -- from the noun; cf. AS. <ets>smecan</ets> taste; akin to D. <ets>smaken</ets>, G. <ets>schmecken</ets>, OHG. <ets>smechen</ets> to taste, smach<?/n to have a taste (and, derived from the same source, G. <ets>schmatzen</ets> to smack the lips, to kiss with a sharp noise, MHG. <ets>smatzen</ets>, <ets>smackzeen</ets>), Icel. <ets>smakka</ets> to taste, Sw. <ets>smaka</ets>, Dan. <ets>smage</ets>. See 2d <er>Smack</er>, <pos>n.</pos>]</ety> <sn>1.</sn> <def>To have a smack; to be tinctured with any particular taste.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To have or exhibit and indication or suggestion of the presence of any character or quality; to have a taste, or flavor; -- used with <ptcl>of</ptcl>; <as>as, a remark <ex>smacking</ex> of contempt</as>.</def><br/
[<source>1913 Webster</source> + <source>PJC</source>]</p>

<p><q> All sects, all ages, <qex>smack</qex> of this vice.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>To kiss with a close compression of the lips, so as to make a sound when they separate; to kiss with a sharp noise; to buss.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>To make a noise by the separation of the lips after tasting anything.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Smack</ent><br/
<hw>Smack</hw>, <pos>v. t.</pos> <sn>1.</sn> <def>To kiss with a sharp noise; to buss.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To open, as the lips, with an inarticulate sound made by a quick compression and separation of the parts of the mouth; to make a noise with, as the lips, by separating them in the act of kissing or after tasting.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Drinking off the cup, and <qex>smacking</qex> his lips with an air of ineffable relish.</q> <rj><qau>Sir W. Scott.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>To make a sharp noise by striking; to crack; <as>as, to <ex>smack</ex> a whip</as>.</def> <ldquo/She <xex>smacks</xex> the silken thong.<rdquo/  <rj><au>Young.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Smack</ent><br/
<hw>Smack</hw>, <pos>n.</pos>  <def>Same as <er>heroin</er>; -- a slang term.</def> <mark>[slang]</mark><br/
[<source>PJC</source>]</p>

<p><ent>Smacking</ent><br/
<hw>Smack"ing</hw>, <pos>n.</pos> <def>A sharp, quick noise; a smack.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Like the faint <qex>smacking</qex> of an after kiss.</q> <rj><qau>Dryden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><-- p. 1358 --></p>

<p><ent>Smacking</ent><br/
<hw>Smack"ing</hw> <pr>(sm<acr/k"<icr/ng)</pr>, <pos>a.</pos> <def>Making a sharp, brisk sound; hence, brisk; <as>as, a <ex>smacking</ex> breeze</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Small</ent><br/
<hw>Small</hw> <pr>(sm<add/l)</pr>, <pos>a.</pos> <amorph>[<pos>Compar.</pos> <adjf>Smaller</adjf> <pr>(sm<add/l"<etil/r)</pr>; <pos>superl.</pos> <adjf>Smallest</adjf>.]</amorph> <ety>[OE. <ets>small</ets>, AS. <ets>smael</ets>; akin to D. <ets>smal</ets> narrow, OS. & OHG. <ets>smal</ets> small, G. <ets>schmal</ets> narrow, Dan. & Sw. <ets>smal</ets>, Goth. <ets>smals</ets> small, Icel. <ets>smali</ets> small cattle, sheep, or goats; cf. Gr. <grk>mh^lon</grk> a sheep or goat.]</ety> <sn>1.</sn> <def>Having little size, compared with other things of the same kind; little in quantity or degree; diminutive; not large or extended in dimension; not great; not much; inconsiderable; <as>as, a <ex>small</ex> man; a <ex>small</ex> river</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>To compare<br/
Great things with <qex>small</qex>.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Being of slight consequence; feeble in influence or importance; unimportant; trivial; insignificant; <as>as, a <ex>small</ex> fault; a <ex>small</ex> business</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>Envincing little worth or ability; not large-minded; -- sometimes, in reproach, paltry; mean.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>A true delineation of the <qex>smallest</qex> man is capable of interesting the greatest man.</q> <rj><qau>Carlyle.</qau></rj>
<-- the "g" of "greatest" was a blank in orig. --><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>Not prolonged in duration; not extended in time; short; <as>as, after a <ex>small</ex> space</as>.</def>  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <def>Weak; slender; fine; gentle; soft; not loud.</def> <ldquo/A still, <xex>small</xex> voice.<rdquo/  <rj><au>1 Kings xix. 12.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Great and small</b></col>,<cd>of all ranks or degrees; -- used especially of persons.</cd>  <ldquo/His quests, <xex>great and small</xex>.<rdquo/ <au>Chaucer.</au> -- <col><b>Small arms</b></col>, <cd>muskets, rifles, pistols, etc., in distinction from cannon.</cd> -- <col><b>Small beer</b></col>. <cd>See under <er>Beer</er>.</cd> -- <col><b>Small coal</b></col>. <sd>(a)</sd> <cd>Little coals of wood formerly used to light fires</cd>. <au>Gay.</au> <sd>(b)</sd> <cd>Coal about the size of a hazelnut, separated from the coarser parts by screening.</cd> -- <col><b>Small craft</b></col> <fld>(Naut.)</fld>, <cd>a vessel, or vessels in general, of a small size.</cd> -- <col><b>Small fruits</b></col>. <cd>See under <er>Fruit</er>.</cd> -- <col><b>Small hand</b></col>, <cd>a certain size of paper. See under <er>Paper</er>.</cd> -- <col><b>Small hours</b></col>. <cd>See under <er>Hour</er>.</cd> -- <col><b>Small letter</b></col>. <fld>(Print.)</fld>, <cd>a lower-case letter. See <er>Lower-case</er>, and <cref>Capital letter</cref>, under <er>Capital</er>, <pos>a.</pos></cd> -- <col><b>Small piece</b></col>, <cd>a Scotch coin worth about 2<frac14/d. sterling, or about 4<frac12/cents.</cd> -- <col><b>Small register</b></col>. <cd>See the Note under 1st <er>Register</er>, 7.</cd> -- <col><b>Small stuff</b></col> <fld>(Naut.)</fld>, <cd>spun yarn, marline, and the smallest kinds of rope.</cd> <au>R. H. Dana, Jr.</au> -- <col><b>Small talk</b></col>, <cd>light or trifling conversation; chitchat.</cd> -- <col><b>Small wares</b></col> <fld>(Com.)</fld>, <cd>various small textile articles, as tapes, braid, tringe, and the like.</cd> <au>M<lsquo/Culloch.</au></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Small</ent><br/
<hw>Small</hw>, <pos>adv.</pos> <sn>1.</sn> <def>In or to small extent, quantity, or degree; little; slightly.</def> <mark>[Obs.]</mark> <ldquo/I wept but <xex>small</xex>.<rdquo/ <au>Chaucer.</au> <ldquo/It <xex>small</xex> avails my mood.<rdquo/ <au>Shak.</au><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Not loudly; faintly; timidly.</def> <mark>[Obs. or Humorous]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>You may speak as <qex>small</qex> as you will.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Small</ent><br/
<hw>Small</hw>, <pos>n.</pos> <sn>1.</sn> <def>The small or slender part of a thing; <as>as, the <ex>small</ex> of the leg or of the back</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <pluf>pl.</pluf> <def>Smallclothes.</def> <mark>[Colloq.]</mark>  <rj><au>Hood. Dickens.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <pluf>pl.</pluf> <def>Same as <cref>Little go</cref>. See under <er>Little</er>, <pos>a.</pos></def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Small</ent><br/
<hw>Small</hw>, <pos>v. t.</pos> <def>To make little or less.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Smallage</ent><br/
<hw>Small"age</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[<ets>Small</ets> + F. <ets>ache</ets> smallage. See <er>Ach</er> parsley.]</ety> <fld>(Bot.)</fld> <def>A biennial umbelliferous plant (<spn>Apium graveolens</spn>) native of the seacoats of Europe and Asia. When deprived of its acrid and even poisonous properties by cultivation, it becomes <altname>celery</altname>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Smallclothes</ent><br/
<hw>Small"clothes`</hw> <pr>(?)</pr>, <pos>n. pl.</pos> <def>A man's garment for the hips and thighs; breeches. See <er>Breeches</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Smallish</ent><br/
<hw>Small"ish</hw>, <pos>a.</pos> <def>Somewhat small.</def>  <rj><au>G. W. Cable.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Smallness</ent><br/
<hw>Small"ness</hw>, <pos>n.</pos> <def>The quality or state of being small.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Smallpox</ent><br/
<hw>Small"pox`</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[<ets>Small</ets> + <ets>pox</ets>, <ets>pocks</ets>.]</ety> <fld>(Med.)</fld> <def>A contagious, constitutional, febrile disease characterized by a peculiar eruption; variola. The cutaneous eruption is at first a collection of papules which become vesicles (first flat, subsequently umbilicated) and then pustules, and finally thick crusts which slough after a certain time, often leaving a pit, or scar.</def><-- now no longer observed, after a long campaing of vaccination apparently succeeded in eliminating all human carriers by 1995. --><br/
[<source>1913 Webster</source>]</p>

<p><ent>Smalls</ent><br/
<hw>Smalls</hw> <pr>(?)</pr>, <pos>n. pl.</pos> <def>See <er>Small</er>, <pos>n.</pos>, 2, 3.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Smallsword</ent><br/
<hw>Small"sword`</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A light sword used for thrusting only; especially, the sword worn by civilians of rank in the eighteenth century.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Smally</ent><br/
<hw>Smal"ly</hw> <pr>(?)</pr>, <pos>adv.</pos> <def>In a small quantity or degree; with minuteness.</def> <mark>[R.]</mark>  <rj><au>Ascham.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Smalt</ent><br/
<hw>Smalt</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[It. <ets>smalto</ets>, LL. <ets>smaltum</ets>; of Teutonic origin; cf. OHG. <ets>smalz</ets> grease, butter, G. <ets>schmalz</ets> grease, OHG. <ets>smelzan</ets> to melt, G. <ets>schmelzen</ets>. See <er>Smelt</er>, <pos>v. t.</pos>, and cf. <er>Amel</er>, <er>Enamel</er>.]</ety> <def>A deep blue pigment or coloring material used in various arts. It is a vitreous substance made of cobalt, potash, and calcined quartz fused, and reduced to a powder.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Smalt-blue</ent><br/
<hw>Smalt"-blue`</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Deep blue, like smalt.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Smaltite</ent><br/
<ent>Smaltine</ent><br/
<mhw>{ <hw>Smalt"ine</hw> <pr>(?)</pr>, <hw>Smalt"ite</hw> <pr>(?)</pr>, }</mhw> <pos>n.</pos> <ety>[See <er>Smalt</er>.]</ety> <fld>(Min.)</fld> <def>A tin-white or gray mineral of metallic luster. It is an arsenide of cobalt, nickel, and iron. Called also <altname>speiskobalt</altname>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Smaragd</ent><br/
<hw>Smar"agd</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>smaragdus</ets>. See <er>Emerald</er>.]</ety> <def>The emerald.</def> <mark>[Obs.]</mark>  <rj><au>Bale.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Smaragdine</ent><br/
<hw>Sma*rag"dine</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>smaragdinus</ets>, Gr. <?/<?/<?/<?/.]</ety> <def>Of or pertaining to emerald; resembling emerald; of an emerald green.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Smaragdite</ent><br/
<hw>Sma*rag"dite</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Cf. F. <ets>smaragdite</ets>; -- so called from its emerald-green color. See <er>Smaragd</er>.]</ety> <fld>(Min.)</fld> <def>A green foliated kind of amphibole, observed in eclogite and some varietis of gabbro.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Smart</ent><br/
<hw>Smart</hw> <pr>(sm<aum/rt)</pr>, <pos>v. i.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Smarted</conjf>; <pos>p. pr. & vb. n.</pos> <conjf>Smarting</conjf>.]</vmorph> <ety>[OE. <ets>smarten</ets>, AS. <ets>smeortan</ets>; akin to D. <ets>smarten</ets>, <ets>smerten</ets>, G.  <ets>schmerzen</ets>, OHG. <ets>smerzan</ets>, Dan. <ets>smerte</ets>, Sw. <ets>sm<aum/rta</ets>, D. <ets>smart</ets>, <ets>smert</ets>, a pain, G. <ets>schmerz</ets>, OHG. <ets>smerzo</ets>, and probably to L. <ets>mordere</ets> to bite; cf. Gr. <grk>smerdno`s</grk>, <grk>smerdale`os</grk>, terrible, fearful, Skr. <ets>m<rsdot/d</ets> to rub, crush.  Cf. <er>Morsel</er>.]</ety> <sn>1.</sn> <def>To feel a lively, pungent local pain; -- said of some part of the body as the seat of irritation; <as>as, my finger <ex>smarts</ex>; these wounds <ex>smart</ex></as>.</def>  <rj><au>Chaucer.</au>  <au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To feel a pungent pain of mind; to feel sharp pain or grief; to suffer; to feel the sting of evil; <as>as, the team is still <ex>smarting</ex> from its loss of the championship</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>No creature <qex>smarts</qex> so little as a fool.</q> <rj><qau>Pope.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>He that is surety for a stranger shall <qex>smart</qex> for it.</q> <rj><qau>Prov. xi. 15.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Smart</ent><br/
<hw>Smart</hw>, <pos>v. t.</pos> <def>To cause a smart in.</def> <ldquo/A goad that . . . <xex>smarts</xex> the flesh.<rdquo/  <rj><au>T. Adams.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Smart</ent><br/
<hw>Smart</hw>, <pos>n.</pos> <ety>[OE. <ets>smerte</ets>. See <er>Smart</er>, <pos>v. i.</pos>]</ety> <sn>1.</sn> <def>Quick, pungent, lively pain; a pricking local pain, as the pain from puncture by nettles.</def> <ldquo/In pain's <xex>smart</xex>.<rdquo/  <rj><au>Chaucer.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Severe, pungent pain of mind; pungent grief; <as>as, the <ex>smart</ex> of affliction</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>To stand 'twixt us and our deserved <qex>smart</qex>.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Counsel mitigates the greatest <qex>smart</qex>.</q> <rj><qau>Spenser.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>A fellow who affects smartness, briskness, and vivacity; a dandy.</def> <mark>[Slang]</mark>  <rj><au>Fielding.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>Smart money (see below).</def> <mark>[Canf]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Smart</ent><br/
<hw>Smart</hw> <pr>(?)</pr>, <pos>a.</pos> <amorph>[<pos>Compar.</pos> <adjf>Smarter</adjf> <pr>(?)</pr>; <pos>superl.</pos> <adjf>Smartest</adjf>.]</amorph> <ety>[OE. <ets>smerte</ets>. See <er>Smart</er>, <pos>v. i.</pos>]</ety> <sn>1.</sn> <def>Causing a smart; pungent; pricking; <as>as, a <ex>smart</ex> stroke or taste</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>How <qex>smart</qex> lash that speech doth give my conscience.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Keen; severe; poignant; <as>as, <ex>smart</ex> pain</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>Vigorous; sharp; severe.</def> <ldquo/<xex>Smart</xex> skirmishes, in which many fell.<rdquo/  <rj><au>Clarendon.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>Accomplishing, or able to accomplish, results quickly; active; sharp; clever.</def> <mark>[Colloq.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <def>Efficient; vigorous; brilliant.</def> <ldquo/The stars shine <xex>smarter</xex>.<rdquo/  <rj><au>Dryden.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>6.</sn> <def>Marked by acuteness or shrewdness; quick in suggestion or reply; vivacious; witty; <as>as, a <ex>smart</ex> reply; a <ex>smart</ex> saying</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Who, for the poor renown of being <qex>smart</qex><br/
Would leave a sting within a brother's heart?</q> <rj><qau>Young.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>A sentence or two, . . . which I thought very <qex>smart</qex>.</q> <rj><qau>Addison.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>7.</sn> <def>Pretentious; showy; spruce; <as>as, a <ex>smart</ex> gown</as>.</def>
<-- in modifying dress or appearance, now used in the sense of "neat, trim", or "stylish, attractive, elegant." --><br/
[<source>1913 Webster</source>]</p>

<p><sn>8.</sn> <def>Brisk; fresh; <as>as, a <ex>smart</ex> breeze</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Smart money</b></col>. <sd>(a)</sd> <cd>Money paid by a person to buy himself off from some unpleasant engagement or some painful situation</cd>. <sd>(b)</sd> <fld>(Mil.)</fld> <cd>Money allowed to soldiers or sailors, in the English service, for wounds and injures received; also, a sum paid by a recruit, previous to being sworn in, to procure his release from service</cd>. <sd>(c)</sd> <fld>(Law)</fld> <cd>Vindictive or exemplary damages; damages beyond a full compensation for the actual injury done</cd>. <au>Burrill.</au> <au>Greenleaf.</au><-- = punitive damages?. (d) (Finance) Knowledgeable investors or bettors. "The smart money says that technology stocks are at a peak." --> -- <col><b>Smart ticket</b></col>, <cd>a certificate given to wounded seamen, entitling them to smart money.</cd> <mark>[Eng.]</mark> <au>Brande & C.</au></cs><br/
[<source>1913 Webster</source>]</p>

<p><syn><b>Syn.</b> -- Pungent; poignant; sharp; tart; acute; quick; lively; brisk; witty; clever; keen; dashy; showy.</syn>  -- <usage><er>Smart</er>, <er>Clever</er>. <xex>Smart</xex> has been much used in New England to describe a person who is intelligent, vigorous, and active; as, a <xex>smart</xex> young fellow; a <xex>smart</xex> workman, etc., conciding very nearly with the English sense of <xex>clever</xex>. The nearest approach to this in England is in such expressions as, he was <xex>smart</xex> (pungent or witty) in his reply, etc.; but <xex>smart</xex> and smartness, when applied to persons, more commonly refer to dress; as, a <xex>smart</xex> appearance; a <xex>smart</xex> gown, etc.</usage><br/
[<source>1913 Webster</source>]</p>

<p><ent>Smarten</ent><br/
<hw>Smart"en</hw> <pr>(?)</pr>, <pos>v. t.</pos> <def>To make smart or spruce; -- usually with <xex>up</xex>.</def> <mark>[Colloq.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>She had to go and <qex>smarten</qex> herself up somewhat.</q> <rj><qau>W. Black.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Smartle</ent><br/
<hw>Smar"tle</hw> <pr>(?)</pr>, <pos>v. i.</pos> <def>To waste away.</def> <mark>[Prov. Eng.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Smartly</ent><br/
<hw>Smart"ly</hw> <pr>(?)</pr>, <pos>adv.</pos> <def>In a smart manner.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Smartness</ent><br/
<hw>Smart"ness</hw>, <pos>n.</pos> <def>The quality or state of being smart.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Smartweed</ent><br/
<hw>Smart"weed`</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Bot.)</fld> <def>An acrid plant of the genus <gen>Polygonum</gen> (<spn>Polygonum Hydropiper</spn>), which produces smarting if applied where the skin is tender.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Smash</ent><br/
<hw>Smash</hw> <pr>(sm<acr/sh)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Smashed</conjf> <pr>(sm<acr/sht)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Smashing</conjf>.]</vmorph> <ety>[Cf. Sw. <ets>smisk</ets> a blow, stroke, <ets>smiska</ets> to strike, dial. Sw. <ets>smaske</ets> to kiss with a noise, and E. <ets>smack</ets> a loud kiss, a slap.]</ety> <sn>1.</sn> <def>To break in pieces by violence; to dash to pieces; to crush.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Here everything is broken and <qex>smashed</qex> to pieces.</q> <rj><qau>Burke.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Lawn Tennis)</fld> <def>To hit (the ball) from above the level of the net with a very hard overhand stroke.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Smash</ent><br/
<hw>Smash</hw>, <pos>v. i.</pos> <def>To break up, or to pieces suddenly, as the result of collision or pressure.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Smash</ent><br/
<hw>Smash</hw>, <pos>n.</pos> <sn>1.</sn> <def>A breaking or dashing to pieces; utter destruction; wreck.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Hence, bankruptcy.</def> <mark>[Colloq.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Smasher</ent><br/
<hw>Smash"er</hw> <pr>(sm<acr/sh"<etil/r)</pr>, <pos>n.</pos> <sn>1.</sn> <def>One who, or that which, smashes or breaks things to pieces.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Anything very large or extraordinary.</def> <mark>[Slang]</mark><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>One who passes counterfeit coin.</def> <mark>[Cant, Eng.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Smatch</ent><br/
<hw>Smatch</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[OE. <ets>smach</ets>, <ets>smak</ets>. See <er>Smack</er> taste.]</ety> <def>Taste; tincture; smack.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>Thy life hath had some <qex>smatch</qex> of honor in it.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Smatch</ent><br/
<hw>Smatch</hw>, <pos>v. i.</pos> <def>To smack.</def> <mark>[Obs.]</mark>  <rj><au>Banister (1578).</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Smatter</ent><br/
<hw>Smat"ter</hw> <pr>(?)</pr>, <pos>v. i.</pos> <ety>[OE. <ets>smateren</ets> to make a noise; cf. Sw. <ets>smattra</ets> to clatter, to crackle, G. <ets>schmettern</ets> to dash, crash, to warble, quaver.]</ety> <sn>1.</sn> <def>To talk superficially or ignorantly; to babble; to chatter.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Of state affairs you can not <qex>smatter</qex>.</q> <rj><qau>Swift.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To have a slight taste, or a slight, superficial knowledge, of anything; to smack.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Smatter</ent><br/
<hw>Smat"ter</hw>, <pos>v. t.</pos> <sn>1.</sn> <def>To talk superficially about.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To gain a slight taste of; to acquire a slight, superficial knowledge of; to smack.</def>  <rj><au>Chaucer.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Smatter</ent><br/
<hw>Smat"ter</hw>, <pos>n.</pos> <def>Superficial knowledge; a smattering.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Smatterer</ent><br/
<hw>Smat"ter*er</hw> <pr>(?)</pr>, <pos>n.</pos> <def>One who has only a slight, superficial knowledge; a sciolist.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Smattering</ent><br/
<hw>Smat"ter*ing</hw>, <pos>n.</pos> <def>A slight, superficial knowledge of something; sciolism.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>I had a great desire, not able to attain to a superficial skill in any, to have some <qex>smattering</qex> in all.</q> <rj><qau>Burton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Smear</ent><br/
<hw>Smear</hw> <pr>(sm<emac/r)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Smeared</conjf> <pr>(sm<emac/rd)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Smearing</conjf>.]</vmorph> <ety>[OE. <ets>smeren</ets>, <ets>smerien</ets>, AS. <ets>smierwan</ets>, <ets>smyrwan</ets>, fr. <ets>smeoru</ets> fat, grease; akin to D. <ets>smeren</ets>, OHG. <ets>smirwen</ets>, G. <ets>schmieren</ets>, Icel. <ets>smyrja</ets> to anoint. See <er>Smear</er>, <pos>n.</pos>]</ety> <sn>1.</sn> <def>To overspread with anything unctuous, viscous, or adhesive; to daub; <as>as, to <ex>smear</ex> anything with oil</as>.</def> <ldquo/<xex>Smear</xex> the sleepy grooms with blood.<rdquo/  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To soil in any way; to contaminate; to pollute; to stain morally; <as>as, to be <ex>smeared</ex> with infamy</as>.</def>  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>To smudge, blur, or render indistinct (writing, pictures, etc.).</def><br/
[<source>PJC</source>]</p>

<p><sn>4.</sn> <def>to vilify (a person); to damage (a person's reputation), especially falsely or by unfair innuendo, and with malicious intent.</def><br/
[<source>PJC</source>]</p>

<p><ent>Smear</ent><br/
<hw>Smear</hw>, <pos>n.</pos> <ety>[OE. <ets>smere</ets>,. <ets>smeoru</ets> fat, grease; akin to D. <ets>smeer</ets>, G. <ets>schmeer</ets>, OHG. <ets>smero</ets>, Icel. <ets>smj<oum/r</ets>, Sw. & Dan. <ets>sm<oum/r</ets> butter, Goth. <ets>sma<iacute/r<thorn/r</ets> fatness, <ets>smarna</ets> dung; cf. Lith. <ets>smarsas</ets> fat.  Cf. <er>Smirch</er>.]</ety> <sn>1.</sn> <def>A fat, oily substance; oinment.</def>  <rj><au>Johnson.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Hence, a spot made by, or as by, an unctuous or adhesive substance; a blot or blotch; a daub; a stain.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Slow broke the morn,<br/
All damp and rolling vapor, with no sun,<br/
But in its place a moving <qex>smear</qex> of light.</q> <rj><qau>Alexander Smith.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Smearcase</ent><br/
<hw>Smear"case`</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[G. <ets>schmierk<aum/se</ets>; <ets>schmier</ets> grease (or <ets>schmieren</ets> to smear) + <ets>k<aum/se</ets> cheese.]</ety> <def>Cottage cheese.</def> <mark>[Local, U. S.]</mark><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Smear dab</ent><br/
<hw>Smear" dab"</hw> <pr>(?)</pr>. <fld>(Zool.)</fld> <def>The sand fluke <sd>(b)</sd>.</def> <mark>[Prov. Eng.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Smeared</ent><br/
<hw>Smeared</hw> <pr>(?)</pr>, <pos>a.</pos> <fld>(Zool.)</fld> <def>Having the color mark ings ill defined, as if rubbed; <as>as, the <ex>smeared</ex> dagger moth (<spn>Apatela oblinita</spn>)</as>.</def>
<-- #sic  ?sp.  Under "dagger moth", it says genus = Apatalea --><br/
[<source>1913 Webster</source>]</p>

<p><ent>Smeary</ent><br/
<hw>Smear"y</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Tending to smear or soil; adhesive; viscous.</def>  <rj><au>Rowe.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Smeath</ent><br/
<hw>Smeath</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Zool.)</fld> <def>The smew.</def> <mark>[Prov. Eng.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Smectite</ent><br/
<hw>Smec"tite</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[G. <ets>smectit</ets>, fr. Gr. <?/<?/<?/<?/<?/ a kind of fuller's earth, fr. <?/<?/<?/<?/<?/ to wipe off.]</ety> <fld>(Min.)</fld> <def>A hydrous silicate of alumina, of a greenish color, which, in certain states of humidity, appears transparent and almost gelatinous.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Smee</ent><br/
<hw>Smee</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Cf. <er>Smew</er>.]</ety> <fld>(Zool.)</fld> <sd>(a)</sd> <def>The pintail duck.</def> <sd>(b)</sd> <def>The widgeon.</def> <sd>(c)</sd> <def>The poachard.</def> <sd>(d)</sd> <def>The smew.</def> <mark>[Prov. Eng.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Smeeth</ent><br/
<hw>Smeeth</hw> <pr>(?)</pr>, <pos>v. t.</pos> <ety>[Etymol. uncertain.]</ety> <def>To smoke; to blacken with smoke; to rub with soot.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Smeeth</ent><br/
<hw>Smeeth</hw> <pr>(?)</pr>, <pos>v. t.</pos> <ety>[OE. <ets>sme<?/en</ets>, AS. <ets>sm<?/<?/ian</ets>. See <er>Smooth</er>.]</ety> <def>To smooth.</def> <mark>[Prov. Eng.]</mark>  <rj><au>Halliwell.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Smegma</ent><br/
<hw>Smeg"ma</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[NL., fr. Gr. <?/<?/<?/<?/ soap, fr. <?/<?/<?/<?/<?/ to wash off.]</ety> <fld>(Physiol.)</fld> <def>The matter secreted by any of the sebaceous glands.</def> Specifically: <sd>(a)</sd> <def>The soapy substance covering the skin of newborn infants.</def> <sd>(b)</sd> <def>The cheesy, sebaceous matter which collects between the glans penis and the foreskin.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Smegmatic</ent><br/
<hw>Smeg*mat"ic</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Being of the nature of soap; soapy; cleansing; detersive.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Smeir</ent><br/
<hw>Smeir</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A salt glaze on pottery, made by adding common salt to an earthenware glaze.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Smell</ent><br/
<hw>Smell</hw> <pr>(sm<ecr/l)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Smelled</conjf> <pr>(?)</pr>, <conjf>Smelt</conjf> <pr>(<?/)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Smelling</conjf>.]</vmorph> <ety>[OE. <ets>smellen</ets>, <ets>smillen</ets>, <ets>smullen</ets>; cf. LG. <ets>smellen</ets>, <ets>smelen</ets>, <ets>sm<oum/len</ets>, <ets>schmelen</ets>, to smoke, to reek, D. <ets>smeulen</ets> to smolder, and E. <ets>smolder</ets>.  Cf. <er>Smell</er>, <pos>n.</pos>]</ety> <sn>1.</sn> <def>To perceive by the olfactory nerves, or organs of smell; to have a sensation of, excited through the nasal organs when affected by the appropriate materials or qualities; to obtain the scent of; <as>as, to <ex>smell</ex> a rose; to <ex>smell</ex> perfumes</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To detect or perceive, as if by the sense of smell; to scent out; -- often with <xex>out</xex>.</def> <ldquo/I <xex>smell</xex> a device.<rdquo/  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Can you <qex>smell</qex> him out by that?</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>To give heed to.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>From that time forward I began to <qex>smell</qex>the Word of God, and forsook the school doctors.</q> <rj><qau>Latimer.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>To smell a rat</b></col>, <cd>to have a sense of something wrong, not clearly evident; to have reason for suspicion.</cd> <mark>[Colloq.]</mark> -- <col><b>To smell out</b></col>, <cd>to find out by sagacity.</cd> <mark>[Colloq.]</mark></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Smell</ent><br/
<hw>Smell</hw>, <pos>v. i.</pos> <sn>1.</sn> <def>To affect the olfactory nerves; to have an odor or scent; -- often followed by <xex>of</xex>; <as>as, to <ex>smell</ex> of smoke, or of musk</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To have a particular tincture or smack of any quality; to savor; <as>as, a report <ex>smells</ex> of calumny</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Praises in an enemy are superfluous, or <qex>smell</qex> of craft.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>To exercise the sense of smell.</def>  <rj><au>Ex. xxx. 38.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>To exercise sagacity.</def>  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Smell</ent><br/
<hw>Smell</hw>, <pos>n.</pos> <ety>[OE. <ets>smel</ets>, <ets>smil</ets>, <ets>smul</ets>, <ets>smeol</ets>. See <er>Smell</er>, <pos>v. t.</pos>]</ety> <fld>(Physiol.)</fld> <sn>1.</sn> <def>The sense or faculty by which certain qualities of bodies are perceived through the instrumentally of the olfactory nerves. See <er>Sense</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>The quality of any thing or substance, or emanation therefrom, which affects the olfactory organs; odor; scent; fragrance; perfume; <as>as, the <ex>smell</ex> of mint</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Breathing the <qex>smell</qex> of field and grove.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>That which, above all others, yields the sweetest <qex>smell</qex> in the air, is the violent.</q> <rj><qau>Bacon.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><syn><b>Syn.</b> -- Scent; odor; perfume; fragrance.</syn><br/
[<source>1913 Webster</source>]</p>

<p><ent>Smeller</ent><br/
<hw>Smell"er</hw> <pr>(?)</pr>, <pos>n.</pos> <sn>1.</sn> <def>One who smells, or perceives by the sense of smell; one who gives out smell.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>The nose.</def> <mark>[Pugilists' Slang]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Smell-feast</ent><br/
<hw>Smell"-feast`</hw> <pr>(?)</pr>, <pos>n.</pos> <sn>1.</sn> <def>One who is apt to find and frequent good tables; a parasite; a sponger.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The epicure and the <qex>smell-feast</qex>.</q> <rj><qau>South.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A feast at which the guests are supposed to feed upon the odors only of the viands.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Smelling</ent><br/
<hw>Smell"ing</hw>, <pos>n.</pos> <sn>1.</sn> <def>The act of one who smells.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>The sense by which odors are perceived; the sense of smell.</def>  <rj><au>Locke.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Smelling bottle</b></col>, <cd>a small bottle filled with something suited to stimulate the sense of smell, or to remove faintness, as spirits of ammonia.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Smelling salts</ent><br/
<hw>Smell"ing salts</hw>. <def>An aromatic preparation of carbonate of ammonia and, often, some scent, to avoid or relieve faintness, headache, or the like.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Smell-less</ent><br/
<hw>Smell"-less</hw>, <pos>a.</pos> <def>Destitute of smell; having no odor.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Daisies <qex>smell-less</qex>, yet most quaint.</q> <rj><qau>Beau. & Fl.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Smelt</ent><br/
<hw>Smelt</hw> <pr>(?)</pr>, <def><pos>imp. & p. p.</pos> of <er>Smell</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Smelt</ent><br/
<hw>Smelt</hw>, <pos>n.</pos> <ety>[AS. <ets>smelt</ets>, <ets>smylt</ets>; akin to Dan. <ets>smelt</ets>.]</ety> <sn>1.</sn> <fld>(Zool.)</fld> <def>Any one of numerous species of small silvery salmonoid fishes of the genus <gen>Osmerus</gen> and allied genera, which ascend rivers to spawn, and sometimes become landlocked in lakes. They are esteemed as food, and have a peculiar odor and taste.</def><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ The most important species are the European smelt (<spn>Osmerus eperlans</spn>) (called also <altname>eperlan</altname>, <altname>sparling</altname>, and <altname>spirling</altname>), the Eastern American smelt (<spn>Osmerus mordax</spn>), the California smelt (<spn>Osmerus thalichthys</spn>), and the surf smelt (<spn>Hypomesus olidus</spn>).  The name is loosely applied to various other small fishes, as the lant, the California tomcod, the spawn eater, the silversides.</note><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <mark>Fig.:</mark> <def>A gull; a simpleton.</def> <mark>[Obs.]</mark>  <rj><au>Beau. & Fl.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Sand smelt</b></col> <fld>(Zool.)</fld>, <cd>the silverside.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Smelt</ent><br/
<hw>Smelt</hw>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Smelted</conjf>; <pos>p. pr. & vb. n.</pos> <conjf>Smelting</conjf>.]</vmorph> <ety>[Of foreign origin; cf. Sw. <ets>sm<aum/lta</ets>, D. <ets>smelten</ets>, Dan. <ets>smelte</ets>, Icel. <ets>smelta</ets>, G. <ets>schmelzen</ets> OHG. <ets>smelzan</ets>, <ets>smelzen</ets>; probably akin to Gr. <?/<?/<?/<?/<?/.  Cf. <er>Enamel</er>, <er>Melt</er>, <er>Mute</er>, <pos>v. i.</pos>, <er>Smalt</er>.]</ety> <fld>(Metal.)</fld> <def>To melt or fuse, as, ore, for the purpose of separating and refining the metal; hence, to reduce; to refine; to flux or scorify; <as>as, to <ex>smelt</ex> tin</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><-- p. 1359 --></p>

<p><ent>Smelter</ent><br/
<hw>Smelt"er</hw> <pr>(?)</pr>, <pos>n.</pos> <def>One who, or that which, smelts.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Smeltery</ent><br/
<hw>Smelt"er*y</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A house or place for smelting.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Smeltie</ent><br/
<hw>Smelt"ie</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A fish, the bib.</def> <mark>[Prov. Eng.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Smelting</ent><br/
<hw>Smelt"ing</hw>, <def><pos>a. & n.</pos> from <er>Smelt</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Smelting furnace</b></col> <fld>(Metal.)</fld>, <cd>a furnace in which ores are smelted or reduced.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Smerk</ent><br/
<hw>Smerk</hw> <pr>(?)</pr>, <pos>n. & v.</pos> <def>See <er>Smirk</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Smerky</ent><br/
<ent>Smerk</ent><br/
<mhw>{ <hw>Smerk</hw> <pr>(?)</pr>, <hw>Smerk"y</hw> <pr>(?)</pr>, }</mhw> <pos>a.</pos> <def>Smart; jaunty; spruce. See <er>Smirk</er>, <pos>a.</pos></def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>So <qex>smerk</qex>, so smooth, his pricked ears.</q> <rj><qau>Spenser.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Smerlin</ent><br/
<hw>Smer"lin</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Zool.)</fld> <def>A small loach.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Smew</ent><br/
<hw>Smew</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Perhaps for <ets>ice-mew</ets>.]</ety> <fld>(Zool.)</fld> <sd>(a)</sd> <def>small European merganser (<spn>Mergus albellus</spn>) which has a white crest; -- called also <altname>smee</altname>, <altname>smee duck</altname>, <altname>white merganser</altname>, and <altname>white nun</altname>.</def> <sd>(b)</sd> <def>The hooded merganser.</def> <mark>[Local, U.S.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Smicker</ent><br/
<hw>Smick"er</hw> <pr>(?)</pr>, <pos>v. i.</pos> <ety>[Akin to Sw. <ets>smickra</ets> to flatter, Dan. <ets>smigre</ets>, and perhaps to G. <ets>schmeicheln</ets>, and E. <ets>smile</ets>.  Cf. <er>Smicker</er>, <pos>a.</pos>]</ety> <def>To look amorously or wantonly; to smirk.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Smicker</ent><br/
<hw>Smick"er</hw>, <pos>a.</pos> <ety>[AS. <ets>smicere</ets> tasteful, trim. See <er>Smicker</er>, <pos>v.</pos>]</ety> <def>Amorous; wanton; gay; spruce.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Smickering</ent><br/
<hw>Smick"er*ing</hw>, <pos>n.</pos> <def>Amorous glance or inclination.</def> <mark>[Obs.]</mark> <ldquo/A <xex>smickering</xex> to our young lady.<rdquo/  <rj><au>Dryden.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Smicket</ent><br/
<hw>Smick"et</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Dim. of <ets>smock</ets>.]</ety> <def>A woman's under-garment; a smock.</def> <mark>[Obs. or Prov. Eng.]</mark>  <rj><au>Johnson.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Smickly</ent><br/
<hw>Smick"ly</hw>, <pos>adv.</pos> <def>Smugly; finically.</def> <mark>[Obs.]</mark>  <rj><au>Ford.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Smiddy</ent><br/
<hw>Smid"dy</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[See <er>Smithy</er>.]</ety> <def>A smithy.</def> <mark>[Prov. Eng. & Scot.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Smift</ent><br/
<hw>Smift</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A match for firing a charge of powder, as in blasting; a fuse.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Smight</ent><br/
<hw>Smight</hw> <pr>(?)</pr>, <pos>v. t.</pos> <def>To smite.</def> <mark>[Obs.]</mark>  <rj><au>Spenser.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Smilacin</ent><br/
<hw>Smil"a*cin</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Cf. F. <ets>similacine</ets>. See <er>Smilax</er>.]</ety> <fld>(Chem.)</fld> <def>See <er>Parrilin</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Smilax</ent><br/
<hw>Smi"lax</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L., bindweed, Gr. <?/<?/<?/.]</ety> <fld>(Bot.)</fld> <sd>(a)</sd> <def>A genus of perennial climbing plants, usually with a prickly woody stem; green brier, or cat brier. The rootstocks of certain species are the source of the medicine called sarsaparilla.</def> <sd>(b)</sd> <def>A delicate trailing plant (<spn>Myrsiphyllum asparagoides</spn>) much used for decoration. It is a native of the Cape of Good Hope.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Smile</ent><br/
<hw>Smile</hw> <pr>(sm<imac/l)</pr>, <pos>v. i.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Smiled</conjf> <pr>(sm<imac/ld)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Smiling</conjf>.]</vmorph> <ety>[OE. <ets>smilen</ets>; akin to Dan. <ets>smile</ets>, Sw. <ets>smila</ets>, MHG. <ets>smielen</ets>, <ets>smieren</ets>, L. <ets>mirari</ets> to wonder at, Skr. <ets>smi</ets> to smile; and probably to E. <ets>smicker</ets>. <root/173.  Cf. <er>Admire</er>, <er>Marvel</er>, <er>Smirk</er>.]</ety> <sn>1.</sn> <def>To express amusement, pleasure, moderate joy, or love and kindness, by the features of the face; to laugh silently.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>He doth nothing but frown. . . . He hears merry tales and <qex>smiles</qex> not.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>She <qex>smiled</qex> to see the doughty hero slain.</q> <rj><qau>Pope.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>When last I saw thy young blue eyes, they <qex>smiled</qex>.</q> <rj><qau>Byron.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To express slight contempt by a look implying sarcasm or pity; to sneer.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>'T was what I said to Craggs and Child,<br/
Who praised my modesty, and <qex>smiled</qex>.</q> <rj><qau>Pope.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>To look gay and joyous; to have an appearance suited to excite joy; <as>as, <ex>smiling</ex> spring; <ex>smiling</ex> plenty.</as></def><br/
[<source>1913 Webster</source>]</p>

<p><q>The desert <qex>smiled</qex>,<br/
And paradise was opened in the wild.</q> <rj><qau>Pope.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>To be propitious or favorable; to favor; to countenance; -- often with <xex>on</xex>; <as>as, to <ex>smile</ex> on one's labors</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Smile</ent><br/
<hw>Smile</hw>, <pos>v. t.</pos> <sn>1.</sn> <def>To express by a smile; <as>as, to <ex>smile</ex> consent; to <ex>smile</ex> a welcome to visitors.</as></def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To affect in a certain way with a smile.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>And sharply <qex>smile</qex> prevailing folly dead.</q> <rj><qau>Young.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Smile</ent><br/
<hw>Smile</hw>, <pos>n.</pos> <ety>[CF. Dan. <ets>smiil</ets>, Sw. <ets>smil</ets>. See <er>Smile</er>, <pos>v. i.</pos>]</ety> <sn>1.</sn> <def>The act of smiling; a peculiar change or brightening of the face, which expresses pleasure, moderate joy, mirth, approbation, or kindness; -- opposed to <contr>frown</contr>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Sweet intercourse<br/
Of looks and <qex>smiles</qex>: for <qex>smiles</qex> from reason flow.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A somewhat similar expression of countenance, indicative of satisfaction combined with malevolent feelings, as contempt, scorn, etc; <as>as, a scornful <ex>smile</ex></as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>Favor; countenance; propitiousness; <as>as, the <ex>smiles</ex> of Providence</as>.</def> <ldquo/The <xex>smile</xex> of heaven.<rdquo/  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>Gay or joyous appearance; <as>as, the <ex>smiles</ex> of spring</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The brightness of their [the flowers'] <qex>smile</qex> was gone.</q> <rj><qau>Bryant.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Smileless</ent><br/
<hw>Smile"less</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Not having a smile.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Smiler</ent><br/
<hw>Smil"er</hw> <pr>(?)</pr>, <pos>n.</pos> <def>One who smiles.</def>  <rj><au>Tennyson.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Smilet</ent><br/
<hw>Smil"et</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A little smile.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>Those happy <qex>smilets</qex><br/
That played on her ripe lip.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Smilingly</ent><br/
<hw>Smil"ing*ly</hw>, <pos>adv.</pos> <def>In a smiling manner.</def>  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Smilingness</ent><br/
<hw>Smil"ing*ness</hw>, <pos>n.</pos> <def>Quality or state of being smiling.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>And made despair a <qex>smilingness</qex> assume.</q> <rj><qau>Byron.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Smilodon</ent><br/
<hw>Smi"lo*don</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Gr. <?/<?/<?/ a carving knife + <?/<?/<?/, <?/<?/<?/, tooth.]</ety> <fld>(Paleon.)</fld> <def>An extinct genus of saber-toothed tigers. See <er>Mach<?/rodus</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Smilt</ent><br/
<hw>Smilt</hw> <pr>(?)</pr>, <pos>v. i.</pos> <def>To melt.</def> <mark>[Obs.]</mark>  <rj><au>Mortimer.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sminthurid</ent><br/
<hw>Smin*thu"rid</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Gr. <?/<?/<?/ a mouse + <?/<?/<?/ tail.]</ety> <fld>(Zool.)</fld> <def>Any one of numerous small species of springtails, of the family <fam>Sminthuridae</fam>, -- usually found on flowers. See <xex>Illust.</xex> under <er>Collembola</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Smirch</ent><br/
<hw>Smirch</hw> <pr>(?)</pr>, <pos>v. t.</pos> <ety>[From the root of <ets>smear</ets>.]</ety> <def>To smear with something which stains, or makes dirty; to smutch; to begrime; to soil; to sully.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>I'll . . . with a kind of umber <qex>smirch</qex> my face.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Smirch</ent><br/
<hw>Smirch</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A smutch; a dirty stain.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Smirk</ent><br/
<hw>Smirk</hw> <pr>(?)</pr>, <pos>v. i.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Smirked</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Smirking</conjf>.]</vmorph> <ety>[OE. <ets>smirken</ets>, ASS. <ets>smercian</ets>, <ets>smearcian</ets>; cf. MHG. <ets>smieren</ets>, <ets>smielen</ets>, to smile. See <er>Smile</er>, <pos>v. i.</pos>]</ety> <def>To smile in an affected or conceited manner; to smile with affected complaisance; to simper.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Smirk</ent><br/
<hw>Smirk</hw>, <pos>n.</pos> <def>A forced or affected smile; a simper.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The bride, all <qex>smirk</qex> and blush, had just entered.</q> <rj><qau>Sir W. Scott.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Smirk</ent><br/
<hw>Smirk</hw>, <pos>a.</pos> <def>Nice,; smart; spruce; affected; simpering.</def> <ldquo/So <xex>smirk</xex>, so smooth.<rdquo/  <rj><au>Spenser.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Smirkingly</ent><br/
<hw>Smirk"ing*ly</hw>, <pos>adv.</pos> <def>With smirking; with a smirk.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Smirky</ent><br/
<hw>Smirk"y</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Smirk; smirking.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Smit</ent><br/
<hw>Smit</hw> <pr>(?)</pr>, <mark>rare</mark> <def><pos>imp. & p. p.</pos> of <er>Smite</er>.</def>  <rj><au>Spenser.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q><qex>Smit</qex> with the beauty of so fair a scene.</q> <rj><qau>Cowper.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Smit</ent><br/
<hw>Smit</hw>, <mark>obs.</mark> <def><pos>3d. pers. sing. pres.</pos> of <er>Smite</er>.</def>  <rj><au>Chaucer.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Smite</ent><br/
<hw>Smite</hw> <pr>(sm<imac/t)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp.</pos> <conjf>Smote</conjf> <pr>(sm<omac/t)</pr>, rarely <conjf>Smit</conjf> <pr>(sm<icr/t)</pr>; <pos>p. p.</pos> <conjf>Smitten</conjf> <pr>(sm<icr/t"t'n)</pr>, rarely <conjf>Smit</conjf>, or <conjf>Smote</conjf>; <pos>p. pr. & vb. n.</pos> <conjf>Smiting</conjf> <pr>(sm<imac/t"<icr/ng)</pr>.]</vmorph> <ety>[AS. <ets>sm<imac/tan</ets> to smite, to soil, pollute; akin to OFries. <ets>sm<imac/ta</ets> to smite, LG. <ets>smiten</ets>, D. <ets>smijten</ets>, G. <ets>schmeissen</ets>, OHG. <ets>sm<imac/zan</ets> to smear, stroke, OSw. & dial. Sw. <ets>smita</ets> to smite, Dan. <ets>smide</ets> to throw, Goth. bi<ets>smeitan</ets>, to anoint, besmear; cf. Skr. <ets>m<emac/d</ets> to be fat. The original sense seems to have been, to daub on, to smear.  Cf. <er>Smut</er>.]</ety> <sn>1.</sn> <def>To strike; to inflict a blow upon with the hand, or with any instrument held in the hand, or with a missile thrown by the hand; <as>as, to <ex>smite</ex> with the fist, with a rod, sword, spear, or stone</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Whosoever shall <qex>smite</qex> thee on thy right cheek, turn to him the other also.</q> <rj><qau>Matt. v. 39.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>And David . . . took thence a stone, and slang it, and <qex>smote</qex> the Philistine in his forehead.</q> <rj><qau>1 Sam. xvii. 49.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To cause to strike; to use as an instrument in striking or hurling.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Prophesy, and <qex>smite</qex> thine hands together.</q> <rj><qau>Ezek. xxi. 14.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Saul . . . <qex>smote</qex> the javelin into the wall.</q> <rj><qau>1 Sam. xix. 10.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>To destroy the life of by beating, or by weapons of any kind; to slay by a blow; to kill; <as>as, to <ex>smite</ex> one with the sword, or with an arrow or other instrument</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>To put to rout in battle; to overthrow by war.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <def>To blast; to destroy the life or vigor of, as by a stroke or by some visitation.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The flax and the barly was <qex>smitten</qex>.</q> <rj><qau>Ex. ix. 31.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>6.</sn> <def>To afflict; to chasten; to punish.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Let us not mistake God's goodness, nor imagine, because he <qex>smites</qex> us, that we are forsaken by him.</q> <rj><qau>Wake.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>7.</sn> <def>To strike or affect with passion, as love or fear.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The charms that <qex>smite</qex> the simple heart.</q> <rj><qau>Pope.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q><qex>Smit</qex> with the love of sister arts we came.</q> <rj><qau>Pope.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>To smite off</b></col>, <cd>to cut off.</cd> -- <col><b>To smite out</b></col>, <cd>to knock out, as a tooth.</cd> <au>Exod. xxi. 27.</au> -- <col><b>To smite with the tongue</b></col>, <cd>to reproach or upbraid; to revile.</cd> <mark>[Obs.]</mark>  <rj><au>Jer. xviii. 18.</au></rj>
</cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Smite</ent><br/
<hw>Smite</hw>, <pos>v. i.</pos> <def>To strike; to collide; to beat.</def> <mark>[Archaic]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>The heart melteth, and the knees smite together.</q> <rj><qau>Nah. ii. 10.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Smite</ent><br/
<hw>Smite</hw>, <pos>n.</pos> <def>The act of smiting; a blow.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Smiter</ent><br/
<hw>Smit"er</hw> <pr>(sm<imac/t"<etil/r)</pr>, <pos>n.</pos> <def>One who smites.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>I give my back to the <qex>smiters</qex>.</q> <rj><qau>Isa. l. 6.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Smith</ent><br/
<hw>Smith</hw> <pr>(sm<icr/th)</pr>, <pos>n.</pos> <ety>[AS. <ets>smi<edh/</ets>; akin to D. <ets>smid</ets>, G. <ets>schmied</ets>, OHG. <ets>smid</ets>, Icel. <ets>smi<edh/r</ets>, Dan. & Sw. <ets>smed</ets>, Goth. <ets>smi<thorn/a</ets> (in comp.); cf. Gr. <grk>smi`lh</grk> a sort of knife, <grk>sminy`h</grk> a hoe, mattock.]</ety> <sn>1.</sn> <def>One who forges with the hammer; one who works in metals; <as>as, a black<ex>smith</ex>, gold<ex>smith</ex>, silver<ex>smith</ex>, and the like</as>.</def>  <rj><au>Piers Plowman.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Nor yet the <qex>smith</qex> hath learned to form a sword.</q> <rj><qau>Tate.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>One who makes or effects anything.</def> <mark>[R.]</mark>  <rj><au>Dryden.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Smith</ent><br/
<hw>Smith</hw>, <pos>v. t.</pos> <ety>[AS. <ets>smi<edh/ian</ets>. See <er>Smith</er>, <pos>n.</pos>]</ety> <def>To beat into shape; to forge.</def> <mark>[Obs.]</mark>  <rj><au>Chaucer.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>What smith that any [weapon] <qex>smitheth</qex>.</q> <rj><qau>Piers Plowman.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Smithcraft</ent><br/
<hw>Smith"craft`</hw> <pr>(-kr<adot/ft`)</pr>, <pos>n.</pos> <def>The art or occupation of a smith; smithing.</def> <mark>[R.]</mark>  <rj><au>Sir W. Raleigh.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Smither</ent><br/
<hw>Smith"er</hw> <pr>(sm<icr/<th/"<etil/r)</pr>, <pos>n.</pos> <sn>1.</sn> <def>Light, fine rain.</def> <mark>[Prov. Eng.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <pluf>pl.</pluf> <def>Fragments; atoms; finders.</def> <mark>[Prov. Eng.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>Smash the bottle to <qex>smithers</qex>.</q> <rj><qau>Tennyson.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Smithereens</ent><br/
<hw>Smith`er*eens"</hw> <pr>(sm<icr/<th/`<etil/r*<emac/nz")</pr>, <pos>n. pl.</pos> <def>Fragments; atoms; smithers.</def> <mark>[Colloq.]</mark>  <rj><au>W. Black.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Smithery</ent><br/
<hw>Smith"er*y</hw> <pr>(sm<icr/th"<etil/r*<ycr/)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>-ies</plw> <pr>(-<icr/z)</pr>.</plu> <sn>1.</sn> <def>The workshop of a smith; a smithy or stithy.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Work done by a smith; smithing.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The din of all his <qex>smithery</qex> may some time or other possibly wake this noble duke.</q> <rj><qau>Burke.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Smithing</ent><br/
<hw>Smith"ing</hw>, <pos>n.</pos> <def>The act or art of working or forging metals, as iron, into any desired shape.</def>  <rj><au>Moxon.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Smithsonian</ent><br/
<hw>Smith*so"ni*an</hw> <pr>(-s<omac/"n<icr/*<ait/n)</pr>, <pos>a.</pos> <def>Of or pertaining to the Englishman J. L. M. <etsep>Smithson</etsep>, or to the national institution of learning which he endowed at Washington, D. C.; <as>as, the <ex>Smithsonian</ex> Institution; <ex>Smithsonian</ex> Reports</as>.</def> -- <def2><pos>n.</pos> <def>The Smithsonian Institution.</def></def2><-- capitalized --><br/
[<source>1913 Webster</source>]</p>

<p><ent>Smithsonite</ent><br/
<hw>Smith"son*ite</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[See <er>Smithsonian</er>.]</ety> <fld>(Min.)</fld> <def>Native zinc carbonate. It generally occurs in stalactitic, reniform, or botryoidal shapes, of a white to gray, green, or brown color. See Note under <er>Calamine</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Smithy</ent><br/
<hw>Smith"y</hw> <pr>(-<ycr/)</pr>, <pos>n.</pos> <ety>[AS. <ets>smi<edh/<edh/e</ets>, fr. <ets>smi<edh/</ets>; akin to D. <ets>smidse</ets>, <ets>smids</ets>, OHG. <ets>smitta</ets>, G. <ets>schmiede</ets>, Icel. <ets>smi<edh/ja</ets>. See <er>Smith</er>, <pos>n.</pos>]</ety> <def>The workshop of a smith, esp. a blacksmith; a smithery; a stithy.</def> <altsp>[Written also <asp>smiddy</asp>.]</altsp><br/
[<source>1913 Webster</source>]</p>

<p><q>Under a spreading chestnut tree<br/
The village <qex>smithy</qex> stands.</q> <rj><qau>Longfellow.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Smitt</ent><br/
<hw>Smitt</hw> <pr>(sm<icr/t)</pr>, <pos>n.</pos> <ety>[CF. G. <ets>schmitz</ets> a stain, <ets>schmitzen</ets> besmear. See <er>Smite</er>, <pos>v. t.</pos>]</ety> <def>Fine clay or ocher made up into balls, used for marking sheep.</def> <mark>[Eng.]</mark>  <rj><au>Woodward.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Smitten</ent><br/
<hw>Smit"ten</hw> <pr>(sm<icr/t"t'n)</pr>, <def><pos>p. p.</pos> of <er>Smite</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Smittle</ent><br/
<hw>Smit"tle</hw> <pr>(-t'l)</pr>, <pos>v. t.</pos> <ety>[Freq. fr. OE. <ets>smitten</ets> to befoul. See <er>Smite</er>, <pos>v. t.</pos>]</ety> <def>To infect.</def> <mark>[Prov. Eng.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Smittle</ent><br/
<hw>Smit"tle</hw>, <pos>n.</pos> <def>Infection.</def> <mark>[Pov. Eng.]</mark>  <rj><au>Wright.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Smittlish</ent><br/
<ent>Smittle</ent><br/
<mhw>{ <hw>Smit"tle</hw> <pr>(sm<icr/t"t'l)</pr>, <hw>Smit"tlish</hw> <pr>(-tl<icr/sh)</pr>, }</mhw> <pos>a.</pos> <def>Infectious; catching.</def> <mark>[Scot. & Prov. Eng.]</mark>  <rj><au>H. Kingsley.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Smock</ent><br/
<hw>Smock</hw> <pr>(sm<ocr/k)</pr>, <pos>n.</pos> <ety>[AS. <ets>smocc</ets>; akin to OHG. <ets>smocho</ets>, Icel. <ets>smokkr</ets>, and from the root of AS. <ets>sm<umac/gan</ets> to creep, akin to G. <ets>schmiegen</ets> to cling to, press close, MHG. <ets>smiegen</ets>, Icel. <ets>smj<umac/ga</ets> to creep through, to put on a garment which has a hole to put the head through; cf. Lith. <ets>smukti</ets> to glide.  Cf. <er>Smug</er>, <er>Smuggle</er>.]</ety> <sn>1.</sn> <def>A woman's under-garment; a shift; a chemise.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>In her <qex>smock</qex>, with head and foot all bare.</q> <rj><qau>Chaucer.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A blouse; a smoock frock.</def>  <rj><au>Carlyle.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Smock</ent><br/
<hw>Smock</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Of or pertaining to a smock; resembling a smock; hence, of or pertaining to a woman.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Smock mill</b></col>, <cd>a windmill of which only the cap turns round to meet the wind, in distinction from a <xex>post mill</xex>, whose whole building turns on a post.</cd> -- <col><b>Smock race</b></col>, <cd>a race run by women for the prize of a smock.</cd> <mark>[Prov. Eng.]</mark></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Smock</ent><br/
<hw>Smock</hw>, <pos>v. t.</pos> <def>To provide with, or clothe in, a smock or a smock frock.</def>  <rj><au>Tennyson.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Smock-faced</ent><br/
<hw>Smock"-faced`</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Having a feminine countenance or complexion; smooth-faced; girlish.</def>  <rj><au>Fenton.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Smock frock</ent><br/
<hw>Smock" frock`</hw> <pr>(?)</pr>. <def>A coarse frock, or shirt, worn over the other dress, as by farm laborers.</def>  <rj><au>Macaulay.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Smockless</ent><br/
<hw>Smock"less</hw>, <pos>a.</pos> <def>Lacking a smock.</def>  <rj><au>Chaucer.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Smokable</ent><br/
<hw>Smok"a*ble</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Capable of being smoked; suitable or ready to be smoked; <as>as, <ex>smokable</ex> tobacco</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Smoke</ent><br/
<hw>Smoke</hw> <pr>(sm<omac/k)</pr>, <pos>n.</pos> <ety>[AS. <ets>smoca</ets>, fr. <ets>sme<oacute/can</ets> to smoke; akin to LG. & D. <ets>smook</ets> smoke, Dan. <ets>sm<oum/g</ets>, G. <ets>schmauch</ets>, and perh. to Gr. <?/<?/<?/ to burn in a smoldering fire; cf. Lith. <ets>smaugti</ets> to choke.]</ety> <sn>1.</sn> <def>The visible exhalation, vapor, or substance that escapes, or expelled, from a burning body, especially from burning vegetable matter, as wood, coal, peat, or the like.</def><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ The gases of hydrocarbons, raised to a red heat or thereabouts, without a mixture of air enough to produce combustion, disengage their carbon in a fine powder, forming <xex>smoke</xex>. The disengaged carbon when deposited on solid bodies is <xex>soot</xex>.</note><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>That which resembles smoke; a vapor; a mist.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>Anything unsubstantial, as idle talk.</def>  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>The act of smoking, esp. of smoking tobacco; <as>as, to have a <ex>smoke</ex></as>.</def> <mark>[Colloq.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ <xex>Smoke</xex> is sometimes joined with other word. forming self-explaining compounds; as, <xex>smoke</xex>-consuming, <xex>smoke</xex>-dried, <xex>smoke</xex>-stained, etc.</note><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Smoke arch</b></col>, <cd>the smoke box of a locomotive.</cd> -- <col><b>Smoke ball</b></col> <fld>(Mil.)</fld>, <cd>a ball or case containing a composition which, when it burns, sends forth thick smoke.</cd> -- <col><b>Smoke black</b></col>, <cd>lampblack.</cd> <mark>[Obs.]</mark> -- <col><b>Smoke board</b></col>, <cd>a board suspended before a fireplace to prevent the smoke from coming out into the room.</cd> -- <col><b>Smoke box</b></col>, <cd>a chamber in a boiler, where the smoke, etc., from the furnace is collected before going out at the chimney.</cd> -- <col><b>Smoke sail</b></col> <fld>(Naut.)</fld>, <cd>a small sail in the lee of the galley stovepipe, to prevent the smoke from annoying people on deck.</cd> -- <col><b>Smoke tree</b></col> <fld>(Bot.)</fld>, <cd>a shrub (<spn>Rhus Cotinus</spn>) in which the flowers are mostly abortive and the panicles transformed into tangles of plumose pedicels looking like wreaths of smoke.</cd> -- <col><b>To end in smoke</b></col>, <cd>to burned; hence, to be destroyed or ruined; figuratively, to come to nothing.</cd><-- same as go up in smoke. --></cs><br/
[<source>1913 Webster</source>]</p>

<p><syn><b>Syn.</b> -- Fume; reek; vapor.</syn><br/
[<source>1913 Webster</source>]</p>

<p><ent>Smoke</ent><br/
<hw>Smoke</hw>, <pos>v. i.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Smoked</conjf> <pr>(?)</pr>; <pos>p. pr. & vb n.</pos> <conjf>Smoking</conjf>.]</vmorph> <ety>[AS. <ets>smocian</ets>; akin to D. <ets>smoken</ets>, G. <ets>schmauchen</ets>, Dan. <ets>sm<oum/ge</ets>. See <er>Smoke</er>, <pos>n.</pos>]</ety> <sn>1.</sn> <def>To emit smoke; to throw off volatile matter in the form of vapor or exhalation; to reek.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Hard by a cottage chimney <qex>smokes</qex>.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Hence, to burn; to be kindled; to rage.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The anger of the Lord and his jealousy shall <qex>smoke</qex> agains. that man.</q> <rj><qau>Deut. xxix. 20.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>To raise a dust or smoke by rapid motion.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Proud of his steeds, he <qex>smokes</qex> along the field.</q> <rj><qau>Dryden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>To draw into the mouth the smoke of tobacco burning in a pipe or in the form of a cigar, cigarette, etc.; to habitually use tobacco in this manner.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <def>To suffer severely; to be punished.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Some of you shall <qex>smoke</qex> for it in Rome.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><-- To be smoking, (a) [Colloq] (Entertainment, sports) To perform in an exciting manner. (b) (Gambling) To be winning in a long streak --><br/
[<source>1913 Webster</source>]</p>

<p><ent>Smoke</ent><br/
<hw>Smoke</hw>, <pos>v. t.</pos> <sn>1.</sn> <def>To apply smoke to; to hang in smoke; to disinfect, to cure, etc., by smoke; <as>as, to <ex>smoke</ex> or fumigate infected clothing; to <ex>smoke</ex> beef or hams for preservation.</as></def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To fill or scent with smoke; hence, to fill with incense; to perfume.</def> <ldquo/<xex>Smoking</xex> the temple.<rdquo/  <rj><au>Chaucer.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>To smell out; to hunt out; to find out; to detect.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>I alone<br/
<qex>Smoked</qex> his true person, talked with him.</q> <rj><qau>Chapman.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>He was first <qex>smoked</qex> by the old Lord Lafeu.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Upon that . . . I began to <qex>smoke</qex> that they were a parcel of mummers.</q> <rj><qau>Addison.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>To ridicule to the face; to quiz.</def> <mark>[Old Slang]</mark><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <def>To inhale and puff out the smoke of, as tobacco; to burn or use in smoking; <as>as, to <ex>smoke</ex> a pipe or a cigar</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>6.</sn> <def>To subject to the operation of smoke, for the purpose of annoying or driving out; -- often with <xex>out</xex>; <as>as, to <ex>smoke</ex> a woodchuck out of his burrow</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><-- also used metaphorically, to expose, to cause to be made public; to drive out, as if by smoke.  --><br/
[<source>1913 Webster</source>]</p>

<p><ent>Smoke ball</ent><br/
<hw>Smoke ball</hw>. <def>Same as <er>Puffball</er>.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Smoke-dry</ent><br/
<hw>Smoke"-dry`</hw> <pr>(?)</pr>, <pos>v. t.</pos> <def>To dry by or in smoke.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Smokehouse</ent><br/
<hw>Smoke"house`</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A building where meat or fish is cured by subjecting it to a dense smoke.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Smokejack</ent><br/
<hw>Smoke"jack`</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A contrivance for turning a spit by means of a fly or wheel moved by the current of ascending air in a chimney.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Smokeless</ent><br/
<hw>Smoke"less</hw>, <pos>a.</pos> <def>Making or having no smoke.</def> <ldquo/<xex>Smokeless</xex> towers.<rdquo/  <rj><au>Pope.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Smokeless powder</ent><br/
<hw>Smoke"less pow"der</hw>. <def>A high-explosive gunpowder whose explosion produces little, if any, smoke.  It is usually based on guncotton.</def><br/
[<source>Webster 1913 Suppl.</source> <source>+PJC</source>]</p>

<p><ent>Smoker</ent><br/
<hw>Smok"er</hw> <pr>(?)</pr>, <pos>n.</pos> <sn>1.</sn> <def>One who dries or preserves by smoke.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>One who smokes tobacco or the like.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>A smoking car or compartment.</def> <mark>[U. S.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>A gathering for smoking and social intercourse.</def> <mark>[Colloq.]</mark></p>

<p><q>That evening A Company had a <ldquo/<qex>smoker</qex><rdquo/ in one of the disused huts of Shorncliffe Camp.</q>  <rj><qau>Strand Mag.</qau></rj><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><cs><col><b>black smoker</b></col>, <cd>a vent at the bottom of the ocean, usually at a mid-ocean ridge, through which large quantities of water carrying minerals flow, producing a jet of fluid with the appearance of black smoke.  The ocean water in crevices below the vent is heated to temperatures near 400<deg/ C, and dissolves quantities of metal salts, such as of copper, zinc, gold, and manganese.  When the saturated mineral solutions exit the vent, cooling by contact with the ocean causes the metals to precipitate, mainly as sulfide or sulfate salts.  Unusual forms of life such as <er>tube worms</er> have been found to live in the areas near black smokers. Additional information is available from <a HREF="http://www.nhm.ac.uk/science/mineral/project5/">The Natural History Museum of London</a> and <a HREF="http://www.amnhonline.org/expeditions/blacksmokers/">The American Museum of Natural History Expeditions</a>.</cd></cs></p>

<p><ent>Smokestack</ent><br/
<hw>Smoke"stack`</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A chimney; esp., a pipe serving as a chimney, as the pipe which carries off the smoke of a locomotive, the funnel of a steam vessel, etc.</def><br/
[<source>1913 Webster</source>]</p>

<p><-- p. 1360 --></p>

<p><ent>Smokily</ent><br/
<hw>Smok"i*ly</hw> <pr>(?)</pr>, <pos>adv.</pos> <def>In a smoky manner.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Smokiness</ent><br/
<hw>Smok"i*ness</hw>, <pos>n.</pos> <def>The quality or state of being smoky.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Smoking</ent><br/
<hw>Smok"ing</hw>, <def><pos>a. & n.</pos> from <er>Smoke</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Smoking bean</b></col> <fld>(Bot.)</fld>, <cd>the long pod of the catalpa, or Indian-bean tree, often smoked by boys as a substitute for cigars.</cd> -- <col><b>Smoking car</b></col>, <cd>a railway car carriage reserved for the use of passengers who smoke tobacco.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>smoking gun</ent><br/
<hw>smok"ing gun`</hw> <pr>(sm<omac/k"<icr/ng g<ucr/n`)</pr> , <pos>n.</pos> <mark>[metaphorical]</mark> <ety>[from imagined situations where a person holding a pistol, still smoking from having been fired, is found standing over a dead person]</ety> <def>Incontrovertible evidence; convincing evidence.</def><br/
[<source>PJC</source>]</p>

<p><ent>Smoky</ent><br/
<hw>Smok"y</hw> <pr>(?)</pr>, <pos>a.</pos> <amorph>[<pos>Compar.</pos> <adjf>Smokier</adjf> <pr>(?)</pr>; <pos>superl.</pos> <adjf>Smokiest</adjf>.]</amorph> <sn>1.</sn> <def>Emitting smoke, esp. in large quantities or in an offensive manner; fumid; <as>as, <ex>smoky</ex> fires</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Having the appearance or nature of smoke; <as>as, a <ex>smoky</ex> fog</as>.</def> <ldquo/Unlustrous as the <xex>smoky</xex> light.<rdquo/  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>Filled with smoke, or with a vapor resembling smoke; thick; <as>as, a <ex>smoky</ex> atmosphere</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>Subject to be filled with smoke from chimneys or fireplace; <as>as, a <ex>smoky</ex> house</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <def>Tarnished with smoke; noisome with smoke; <as>as, <ex>smoky</ex> rafters; <ex>smoky</ex> cells.</as></def><br/
[<source>1913 Webster</source>]</p>

<p><sn>6.</sn> <def>Suspicious; open to suspicion.</def> <mark>[Obs.]</mark>  <rj><au>Foote.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Smoky quartz</b></col> <fld>(Min.)</fld>, <cd>a variety of quartz crystal of a pale to dark smoky-brown color. See <er>Quartz</er>.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Smoulder</ent><br/
<ent>Smolder</ent><br/
<mhw>{ <hw>Smol"der</hw>, <hw>Smoul"der</hw>  }</mhw> <pr>(?)</pr>, <pos>v. i.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Smoldered</conjf> <pr>(?)</pr> or <conjf>Smouldered</conjf>; <pos>p. pr. & vb. n.</pos> <conjf>Smoldering</conjf> or <conjf>Smouldering</conjf>.]</vmorph> <ety>[OE. <ets>smolderen</ets>; cf. Prov. G. <ets>sm<oum/len</ets>, <ets>smelen</ets>, D. <ets>smeulen</ets>.  Cf. <er>Smell</er>.]</ety> <sn>1.</sn> <def>To burn and smoke without flame; to waste away by a slow and supressed combustion.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The <qex>smoldering</qex> dust did round about him smoke.</q> <rj><qau>Spenser.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To exist in a state of suppressed or smothered activity; to burn inwardly; <as>as, a <ex>smoldering</ex> feud</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Smoulder</ent><br/
<ent>Smolder</ent><br/
<mhw>{ <hw>Smol"der</hw>, <hw>Smoul"der</hw>, }</mhw> <pos>v. t.</pos> <def>To smother; to suffocate; to choke.</def> <mark>[Obs.]</mark>  <rj><au>Holinshed. Palsgrave.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Smoulder</ent><br/
<ent>Smolder</ent><br/
<mhw>{ <hw>Smol"der</hw>, <hw>Smoul"der</hw>, }</mhw> <pos>n.</pos> <def>Smoke; smother.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>The <qex>smolder</qex> stops our nose with stench.</q> <rj><qau>Gascoigne.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Smouldering</ent><br/
<ent>Smoldering</ent><br/
<mhw>{ <hw>Smol"der*ing</hw>, <hw>Smoul"der*ing</hw>, }</mhw> <pos>a.</pos> <def>Being in a state of suppressed activity; quiet but not dead.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Some evil chance<br/
Will make the <qex>smoldering</qex> scandal break and blaze.</q> <rj><qau>Tennyson.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Smoulderingness</ent><br/
<ent>Smolderingness</ent><br/
<mhw>{ <hw>Smol"der*ing*ness</hw>, <hw>Smoul"der*ing*ness</hw>  }</mhw> <pr>(?)</pr>, <pos>n.</pos> <def>The state of smoldering.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Smouldry</ent><br/
<ent>Smoldry</ent><br/
<mhw>{ <hw>Smol"dry</hw>, <hw>Smoul"dry</hw>  }</mhw> <pr>(?)</pr>, <pos>a.</pos> <def>Smoldering; suffocating; smothery.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>A flaming fire ymixt with <qex>smoldry</qex> smoke.</q> <rj><qau>Spenser.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Smolt</ent><br/
<hw>Smolt</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Zool.)</fld> <def>A young salmon two or three years old, when it has acquired its silvery color.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Smooch</ent><br/
<hw>Smooch</hw> <pr>(?)</pr>, <pos>v. t.</pos> <def>See <er>Smutch</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><-- 2. To kiss; to neck; to pet. --><br/
[<source>1913 Webster</source>]</p>

<p><-- Smooch, n. a kiss. --><br/
[<source>1913 Webster</source>]</p>

<p><ent>Smoor</ent><br/
<hw>Smoor</hw> <pr>(?)</pr>, <pos>v. t.</pos> <ety>[AS. <ets>smorian</ets>; akin to D. & LG. <ets>smoren</ets>, G. <ets>schmoren</ets> to stew.  Cf. <er>Smother</er>.]</ety> <def>To suffocate or smother.</def> <altsp>[Written also <asp>smore</asp>.]</altsp> <mark>[Obs. or Prov. Eng. & Scot.]</mark>  <rj><au>Sir T. More. Burns.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Smooth</ent><br/
<hw>Smooth</hw> <pr>(sm<oomac/<th/)</pr>, <pos>a.</pos> <amorph>[<pos>Compar.</pos> <adjf>Smoother</adjf> <pr>(sm<oomac/<th/"<etil/r)</pr>; <pos>superl.</pos> <adjf>Smoothest</adjf>.]</amorph> <ety>[OE. <ets>smothe</ets>, <ets>smethe</ets>, AS. <ets>sm<emac/<edh/e</ets>, <ets>sm<oe/<edh/e</ets>, where <emac/, <oe/, come from an older <omac/; cf. LG. <ets>sm<oum/de</ets>, <ets>sm<oum/e</ets>, <ets>sm<oum/dig</ets>; of uncertain origin.]</ety> <sn>1.</sn> <def>Having an even surface, or a surface so even that no roughness or points can be perceived by the touch; not rough; <as>as, <ex>smooth</ex> glass; <ex>smooth</ex> porcelain.</as></def>  <rj><au>Chaucer.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>The outlines must be <qex>smooth</qex>, imperceptible to the touch, and even, without eminence or cavities.</q> <rj><qau>Dryden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Evenly spread or arranged; sleek; <as>as, <ex>smooth</ex> hair</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>Gently flowing; moving equably; not ruffled or obstructed; <as>as, a <ex>smooth</ex> stream</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>Flowing or uttered without check, obstruction, or hesitation; not harsh; voluble; even; fluent.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The only <qex>smooth</qex> poet of those times.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Waller was <qex>smooth</qex>; but Dryden taught to join<br/
The varying verse, the full-resounding line.</q> <rj><qau>Pope.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>When sage Minerva rose,<br/
From her sweet lips <qex>smooth</qex> elocution flows.</q> <rj><qau>Gay.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <def>Bland; mild; smoothing; fattering.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>This <qex>smooth</qex> discourse and mild behavior oft<br/
Conceal a traitor.</q> <rj><qau>Addison.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>6.</sn> <fld>(Mech. & Physics)</fld> <def>Causing no resistance to a body sliding along its surface; frictionless.</def><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ <xex>Smooth</xex> is often used in the formation of self-explaining compounds; as, <xex>smooth</xex>-bodied, <xex>smooth</xex>-browed, <xex>smooth</xex>-combed, <xex>smooth</xex>-faced, <xex>smooth</xex>-finished, <xex>smooth</xex>-gliding, <xex>smooth</xex>-grained, <xex>smooth</xex>-leaved, <xex>smooth</xex>-sliding, <xex>smooth</xex>-speaking, <xex>smooth</xex>-woven, and the like.</note><br/
[<source>1913 Webster</source>]</p>

<p><syn><b>Syn.</b> -- Even; plain; level; flat; polished; glossy; sleek; soft; bland; mild; soothing; voluble; flattering; adulatory; deceptive.</syn><br/
[<source>1913 Webster</source>]</p>

<p><ent>Smooth</ent><br/
<hw>Smooth</hw>, <pos>adv.</pos> <def>Smoothly.</def>  <rj><au>Chaucer.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q><qex>Smooth</qex> runs the water where the brook is deep.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Smooth</ent><br/
<hw>Smooth</hw>, <pos>n.</pos> <sn>1.</sn> <def>The act of making smooth; a stroke which smooths.</def>  <rj><au>Thackeray.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>That which is smooth; the smooth part of anything.</def> <ldquo/The <xex>smooth</xex> of his neck.<rdquo/  <rj><au>Gen. xxvii. 16.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Smooth</ent><br/
<hw>Smooth</hw>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Smoothed</conjf> <pr>(sm<oomac/thd)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Smoothing</conjf>.]</vmorph> <ety>[OE. <ets>smothen</ets>, <ets>smethen</ets>, AS. <ets>sm<emac/<edh/ian</ets>; cf. LG. <ets>sm<oum/den</ets>. See <er>Smooth</er>, <pos>a.</pos>]</ety> <def>To make smooth; to make even on the surface by any means; <as>as, to <ex>smooth</ex> a board with a plane; to <ex>smooth</ex> cloth with an iron.</as></def> Specifically: --<br/
[<source>1913 Webster</source>]</p>

<p><sd>(a)</sd> <def>To free from obstruction; to make easy.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Thou, Abelard! the last sad office pay,<br/
And <qex>smooth</qex> my passage to the realms of day.</q> <rj><qau>Pope.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sd>(b)</sd> <def>To free from harshness; to make flowing.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>In their motions harmony divine<br/
So <qex>smooths</qex> her charming tones that God's own ear<br/
Listens delighted.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sd>(c)</sd> <def>To palliate; to gloze; <as>as, to <ex>smooth</ex> over a fault</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sd>(d)</sd> <def>To give a smooth or calm appearance to.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Each perturbation <qex>smoothed</qex> with outward calm.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sd>(e)</sd> <def>To ease; to regulate.</def>  <rj><au>Dryden.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Smooth</ent><br/
<hw>Smooth</hw>, <pos>v. i.</pos> <def>To flatter; to use blandishment.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Because I can not flatter and speak fair,<br/
Smile in men's faces, <qex>smooth</qex>, deceive and cog.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Smoothbore</ent><br/
<hw>Smooth"bore`</hw> <pr>(?)</pr>, <pos>a.</pos> <fld>(Gun.)</fld> <def>Having a bore of perfectly smooth surface; -- distinguished from <contr>rifled</contr>.</def> -- <def2><pos>n.</pos> <def>A smoothbore firearm.</def></def2><br/
[<source>1913 Webster</source>]</p>

<p><ent>Smooth-chinned</ent><br/
<hw>Smooth"-chinned`</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Having a smooth chin; beardless.</def>  <rj><au>Drayton.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Smoothen</ent><br/
<hw>Smooth"en</hw> <pr>(?)</pr>, <pos>v. t.</pos> <def>To make smooth.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Smoother</ent><br/
<hw>Smooth"er</hw> <pr>(?)</pr>, <pos>n.</pos> <def>One who, or that which, smooths.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Smoothing</ent><br/
<hw>Smooth"ing</hw>, <pos>a. & n.</pos> <def>fr. <er>Smooth</er>, <pos>v.</pos></def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Smoothing iron</b></col>, <cd>an iron instrument with a polished face, for smoothing clothes; a sadiron; a flatiron.</cd> -- <col><b>Smoothing plane</b></col>, <cd>a short, finely set plane, for smoothing and finishing work.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Smoothly</ent><br/
<hw>Smooth"ly</hw>, <pos>adv.</pos> <def>In a smooth manner.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Smoothness</ent><br/
<hw>Smooth"ness</hw>, <pos>n.</pos> <def>Quality or state of being smooth.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Smooth-spoken</ent><br/
<hw>Smooth"-spo`ken</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Speaking smoothly; plausible; flattering; smooth-tongued.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Smooth-tongued</ent><br/
<hw>Smooth"-tongued`</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Having a smooth tongue; plausible; flattering.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Smore</ent><br/
<hw>Smore</hw> <pr>(?)</pr>, <pos>v. t.</pos> <def>To smother. See <er>Smoor</er>.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>Some dying vomit blood, and some were <qex>smored</qex>.</q> <rj><qau>Du Bartas.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Smorsato</ent><br/
<ent>Smorzando</ent><br/
<mhw>{ ||<hw>Smor*zan"do</hw> <pr>(?)</pr>, ||<hw>Smor*sa"to</hw> <pr>(?)</pr>, }</mhw> <pos>a.</pos> <ety>[It.]</ety> <fld>(Mus.)</fld> <def>Growing gradually fainter and softer; dying away; morendo.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Smote</ent><br/
<hw>Smote</hw> <pr>(?)</pr>, <def><pos>imp. (<and/ rare p. p.)</pos> of <er>Smite</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Smoterlich</ent><br/
<hw>Smo"ter*lich</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[CF. <er>Smut</er>.]</ety> <def>Dirty; foul.</def> <mark>[Obs.]</mark>  <rj><au>Chaucer.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Smother</ent><br/
<hw>Smoth"er</hw> <pr>(?)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Smothered</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Smothering</conjf>.]</vmorph> <ety>[OE. <ets>smotheren</ets>; akin to E. <ets>smoor</ets>. See <er>Smoor</er>.]</ety> <sn>1.</sn> <def>To destroy the life of by suffocation; to deprive of the air necessary for life; to cover up closely so as to prevent breathing; to suffocate; <as>as, to <ex>smother</ex> a child</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To affect as by suffocation; to stife; to deprive of air by a thick covering, as of ashes, of smoke, or the like; <as>as, to <ex>smother</ex> a fire</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>Hence, to repress the action of; to cover from public view; to suppress; to conceal; <as>as, to <ex>smother</ex> one's displeasure</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Smother</ent><br/
<hw>Smoth"er</hw>, <pos>v. i.</pos> <sn>1.</sn> <def>To be suffocated or stifled.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To burn slowly, without sufficient air; to smolder.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Smother</ent><br/
<hw>Smoth"er</hw>, <pos>n.</pos> <ety>[OE. <ets>smorther</ets>. See <er>Smother</er>, <pos>v. t.</pos>]</ety> <sn>1.</sn> <def>Stifling smoke; thick dust.</def>  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A state of suppression.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>Not to keep their suspicions in <qex>smother</qex>.</q> <rj><qau>Bacon.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>That which smothers or causes a sensation of smothering, as smoke, fog, the foam of the sea, a confused multitude of things.</def></p>

<p><q>Then they vanished, swallowed up in the grayness of the evening and the smoke and <qex>smother</qex> of the storm.</q>  <rj><qau>The Century.</qau></rj><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><cs><col><b>Smother fly</b></col> <fld>(Zool.)</fld>, <cd>an aphid.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Smothered mate</ent><br/
<hw>Smoth"ered mate</hw>. <fld>(Chess)</fld> <def>Checkmate given when movement of the king is completely obstructed by his own men.  This can only occur when an opponent moves a knight into a position to give a check to the king.</def><br/
[<source>Webster 1913 Suppl.</source> <source>+PJC</source>]</p>

<p><ent>Smotheriness</ent><br/
<hw>Smoth"er*i*ness</hw> <pr>(?)</pr>, <pos>n.</pos> <def>The quality or state of being smothery.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Smotheringly</ent><br/
<hw>Smoth"er*ing*ly</hw>, <pos>adv.</pos> <def>In a smothering manner.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Smothery</ent><br/
<hw>Smoth"er*y</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Tending to smother; stifling.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Smouch</ent><br/
<hw>Smouch</hw> <pr>(?)</pr>, <pos>v. t.</pos> <ety>[Akin to <ets>smack</ets>.]</ety> <def>To kiss closely.</def> <mark>[Obs.]</mark>  <rj><au>P. Stubbes.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Smouch</ent><br/
<hw>Smouch</hw>, <pos>v. t.</pos> <ety>[See <er>Smutch</er>.]</ety> <def>To smutch; to soil; <as>as, to <ex>smouch</ex> the face</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Smouch</ent><br/
<hw>Smouch</hw>, <pos>n.</pos> <def>A dark soil or stain; a smutch.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Smoulder</ent><br/
<hw>Smoul"der</hw> <pr>(?)</pr>, <pos>v. i.</pos> <def>See <er>Smolder</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Smouldry</ent><br/
<hw>Smoul"dry</hw> <pr>(?)</pr>, <pos>a.</pos> <def>See <er>Smoldry</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Smudge</ent><br/
<hw>Smudge</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Cf. Dan. <ets>smuds</ets> smut, E. <ets>smutch</ets>, or <ets>smoke</ets>.]</ety> <sn>1.</sn> <def>A suffocating smoke.</def>  <rj><au>Grose.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A heap of damp combustibles partially ignited and burning slowly, placed on the windward side of a house, tent, or the like, in order, by the thick smoke, to keep off mosquitoes or other insects.</def> <mark>[U. S.]</mark>  <rj><au>Bartlett.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>That which is smeared upon anything; a stain; a blot; a smutch; a smear.</def><br/
[<source>1913 Webster</source>]</p>

<p><-- smudge pot, a device containing combustibles which burns smokily, used by farmers to inhibit damage to crops caused by frost. --><br/
[<source>1913 Webster</source>]</p>

<p><ent>Smudge</ent><br/
<hw>Smudge</hw>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Smudged</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Smudging</conjf>.]</vmorph> <sn>1.</sn> <def>To stifle or smother with smoke; to smoke by means of a smudge.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To smear; to smutch; to soil; to blacken with smoke.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Smudginess</ent><br/
<hw>Smudg"i*ness</hw> <pr>(?)</pr>, <pos>n.</pos> <def>The quality or state of being smudged, soiled, or blurred.</def>  <rj><au>C. A. Young.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Smug</ent><br/
<hw>Smug</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[Of. Scand. or Low German origin; cf. LG. <ets>smuck</ets>, G. <ets>schmuck</ets>, Dan. <ets>smuk</ets>, OSw. <ets>smuck</ets>, <ets>sm<oum/ck</ets>, and E. <ets>smock</ets>, <ets>smuggle</ets>; cf. G. <ets>schmuck</ets> ornament. See <er>Smock</er>.]</ety> <def>Studiously neat or nice, especially in dress; spruce; affectedly precise; smooth and prim.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>They be so <qex>smug</qex> and smooth.</q> <rj><qau>Robynson (More's Utopia).</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>The <qex>smug</qex> and scanty draperies of his style.</q> <rj><qau>De Quincey.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>A young, <qex>smug</qex>, handsome holiness has no fellow.</q> <rj><qau>Beau. & Fl.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Smug</ent><br/
<hw>Smug</hw>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Smugged</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Smugging</conjf>.]</vmorph> <def>To make smug, or spruce.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>Thus said, he <qex>smugged</qex> his beard, and stroked up fair.</q> <rj><qau>Dryton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Smuggle</ent><br/
<hw>Smug"gle</hw> <pr>(?)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Smuggled</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Smuggling</conjf> <pr>(?)</pr>.]</vmorph> <ety>[Of Low German or Scand. origin; cf. LG. <ets>smuggeln</ets>, D. <ets>smokkelen</ets>, G. <ets>schmuggeln</ets>, Dan. <ets>smugle</ets>, Sw. <ets>smyga</ets> to introduce or convey secretly, Dan. <ets>i smug</ets> secretly, D. <ets>smuigen</ets> to eat in secret, AS. <ets>sm<?/gan</ets> to creep. See <er>Smock</er>.]</ety> <sn>1.</sn> <def>To import or export secretly, contrary to the law; to import or export without paying the duties imposed by law; <as>as, to <ex>smuggle</ex> lace</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Fig.: To convey or introduce clandestinely.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Smuggle</ent><br/
<hw>Smug"gle</hw>, <pos>v. i.</pos> <def>To import or export in violation of the customs laws.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Smuggler</ent><br/
<hw>Smug"gler</hw> <pr>(?)</pr>, <pos>n.</pos> <sn>1.</sn> <def>One who smuggles.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A vessel employed in smuggling.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Smugly</ent><br/
<hw>Smug"ly</hw>, <pos>adv.</pos> <def>In a smug manner.</def> <mark>[R.]</mark>  <rj><au>Gay.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Smugness</ent><br/
<hw>Smug"ness</hw>, <pos>n.</pos> <def>The quality or state of being smug.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Smut</ent><br/
<hw>Smut</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Akin to Sw. <ets>smuts</ets>, Dan. <ets>smuds</ets>, MHG. <ets>smuz</ets>, G. <ets>schmutz</ets>, D. <ets>smet</ets> a spot or stain, <ets>smoddig</ets>, <ets>smodsig</ets>, <ets>smodderig</ets>, dirty, <ets>smodderen</ets> to smut; and probably to E. <ets>smite</ets>. See <er>Smite</er>, <pos>v. t.</pos>, and cf. <er>Smitt</er>, <er>Smutch</er>.]</ety> <sn>1.</sn> <def>Foul matter, like soot or coal dust; also, a spot or soil made by such matter.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Mining)</fld> <def>Bad, soft coal, containing much earthy matter, found in the immediate locality of faults.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <fld>(Bot.)</fld> <def>An infection of cereal grains producing a swelling which is at length resolved into a powdery sooty mass. It is caused by parasitic fungi of the genus <gen>Ustilago</gen>. <spn>Ustilago segetum</spn>, or <spn>Ustilago Carbo</spn>, is the commonest kind; that of Indian corn is <spn>Ustilago maydis</spn>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>Obscene language; ribaldry; obscenity.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>He does not stand upon decency . . . but will talk <qex>smut</qex>, <qex>though a priest and his mother be in the room</qex>.</q> <rj><qau>Addison.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Smut mill</b></col>, <cd>a machine for cleansing grain from smut.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Smut</ent><br/
<hw>Smut</hw> <pr>(?)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Smutted</conjf>; <pos>p. pr. & vb. n.</pos> <conjf>Smutting</conjf>.]</vmorph> <sn>1.</sn> <def>To stain or mark with smut; to blacken with coal, soot, or other dirty substance.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To taint with mildew, as grain.</def>  <rj><au>Bacon.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>To blacken; to sully or taint; to tarnish.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>To clear of smut; <as>as, to <ex>smut</ex> grain for the mill</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Smut</ent><br/
<hw>Smut</hw>, <pos>v. i.</pos> <sn>1.</sn> <def>To gather smut; to be converted into smut; to become smutted.</def>  <rj><au>Mortimer.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To give off smut; to crock.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Smutch</ent><br/
<hw>Smutch</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Prob. for <ets>smuts</ets>. See <er>Smut</er>, <pos>n.</pos>]</ety> <def>A stain; a dirty spot.</def>  <rj><au>B. Jonson.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Smutch</ent><br/
<hw>Smutch</hw>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Smutched</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Smutching</conjf>.]</vmorph> <def>To blacken with smoke, soot, or coal.</def> <altsp>[Written also <asp>smooch</asp>.]</altsp>  <rj><au>B. Jonson.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Smutchin</ent><br/
<hw>Smutch"in</hw> <pr>(?)</pr>, <pos>n.</pos> <def>Snuff.</def> <mark>[Obs.]</mark>  <rj><au>Howell.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Smutty</ent><br/
<hw>Smut"ty</hw> <pr>(?)</pr>, <pos>a.</pos> <amorph>[<pos>Compar.</pos> <adjf>Smuttier</adjf> <pr>(?)</pr>; <pos>superl.</pos> <adjf>Smuttiest</adjf>.]</amorph> <sn>1.</sn> <def>Soiled with smut; smutted.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Tainted with mildew; <as>as, <ex>smutty</ex> corn</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>Obscene; not modest or pure; <as>as, a <ex>smutty</ex> saying</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The <qex>smutty</qex> joke, ridiculously lewd.</q> <rj><qau>Smollett.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p>-- <wordforms><wf>Smut"ti*ly</wf> <pr>(#)</pr>, <pos>adv.</pos> -- <wf>Smut"ti*ness</wf>, <pos>n.</pos></wordforms><br/
[<source>1913 Webster</source>]</p>

<p><ent>Smyrniot</ent><br/
<hw>Smyr"ni*ot</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Of or pertaining to Smyrna.</def> -- <def2><pos>n.</pos> <def>A native or inhabitant of Smyrna.</def></def2><br/
[<source>1913 Webster</source>]</p>

<p><ent>Snack</ent><br/
<hw>Snack</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[See <er>Snatch</er>, <pos>v. t.</pos>]</ety> <sn>1.</sn> <def>A share; a part or portion; -- obsolete, except in the colloquial phrase, <cref>to go snacks</cref>, i. e., to share.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>At last he whispers, <ldquo/Do, and we <qex>go snacks</qex>.<rdquo/</q> <rj><qau>Pope.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A slight, hasty repast.</def> <mark>[Colloq.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Snacket</ent><br/
<hw>Snack"et</hw> <pr>(?)</pr>, <pos>n.</pos> <def>See <er>Snecket</er>.</def> <mark>[Prov. Eng.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Snacot</ent><br/
<hw>Snac"ot</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Said to be corrupted fr. NL. <ets>syngnathus</ets>, fr. Gr. <grk>sy`n</grk> together + <grk>gna`qos</grk> jaw, because the jaws can be only slightly separated.]</ety> <fld>(Zool.)</fld> <def>A pipefish of the genus <gen>Syngnathus</gen>. See <er>Pipefish</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Snaffle</ent><br/
<hw>Snaf"fle</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[D. <ets>snavel</ets> a beak, bill, snout; akin to G. <ets>schnabel</ets>, OHG. <ets>snabul</ets>,. <ets>sneb</ets>, <ets>snebbe</ets>, OFries. <ets>snavel</ets> mouth, Dan. & Sw. <ets>snabel</ets> beak, bill, Lith. <ets>snapas</ets>, and to E. <ets>snap</ets>, v. See <er>Snap</er>, and cf. <er>Neb</er>.]</ety> <def>A kind of bridle bit, having a joint in the part to be placed in the mouth, and rings and cheek pieces at the ends, but having no curb; -- called also <altname>snaffle bit</altname>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Snaffle</ent><br/
<hw>Snaf"fle</hw>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Snaffled</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Snaffling</conjf> <pr>(?)</pr>.]</vmorph> <def>To put a snaffle in the mouth of; to subject to the snaffle; to bridle.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Snag</ent><br/
<hw>Snag</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Prov. E., n., a lump on a tree where a branch has been cut off; v., to cut off the twigs and small branches from a tree, of Celtic origin; cf. Gael. <ets>snaigh</ets>, <ets>snaidh</ets>, to cut down, to prune, to sharpen, p. p. <ets>snaighte</ets>, <ets>snaidhte</ets>, cut off, lopped, Ir. <ets>snaigh</ets> a hewing, cutting.]</ety> <sn>1.</sn> <def>A stump or base of a branch that has been lopped off; a short branch, or a sharp or rough branch; a knot; a protuberance.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The coat of arms<br/
Now on a naked <qex>snag</qex> in triumph borne.</q> <rj><qau>Dryden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A tooth projecting beyond the rest; contemptuously, a broken or decayed tooth.</def>  <rj><au>Prior.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>A tree, or a branch of a tree, fixed in the bottom of a river or other navigable water, and rising nearly or quite to the surface, by which boats are sometimes pierced and sunk.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <fld>(Zool.)</fld> <def>One of the secondary branches of an antler.</def><br/
[<source>1913 Webster</source>]</p>

<p><-- 5. Any sharp protuberant part of an object, which may catch, scratch, or tear other objects brought into contact with it. --><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Snag boat</b></col>, <cd>a steamboat fitted with apparatus for removing snags and other obstructions in navigable streams.</cd> <mark>[U.S.]</mark> -- <col><b>Snag tooth</b></col>. <cd>Same as <er>Snag</er>, 2.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><q>How thy <qex>snag teeth</qex> stand orderly,<br/
Like stakes which strut by the water side.</q> <rj><qau>J. Cotgrave.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Snag</ent><br/
<hw>Snag</hw>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Snagged</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Snagging</conjf> <pr>(?)</pr>.]</vmorph> <sn>1.</sn> <def>To cut the snags or branches from, as the stem of a tree; to hew roughly.</def> <mark>[Prov. Eng.]</mark>  <rj><au>Halliwell.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To injure or destroy, as a steamboat or other vessel, by a snag, or projecting part of a sunken tree.</def> <mark>[U. S.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><-- 3. To catch on a snag{5}.
    4. (Fig.) To obtain by a quick action, as though by snagging{3} something passing by; -- often used of an opportunistic or fortunate action. --> <br/
[<source>1913 Webster</source>]</p>

<p><ent>Snagged</ent><br/
<hw>Snag"ged</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Full of snags; snaggy.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Snaggy</ent><br/
<hw>Snag"gy</hw> <pr>(?)</pr>, <pos>a.</pos> <sn>1.</sn> <def>Full of snags; full of short, rough branches or sharp points; abounding with knots.</def> <ldquo/Upon a <xex>snaggy</xex> oak.<rdquo/  <rj><au>Spenser.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Snappish; cross; ill-tempered.</def> <mark>[Prov. Eng.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Snail</ent><br/
<hw>Snail</hw> <pr>(sn<amac/l)</pr>, <pos>n.</pos> <ety>[OE. <ets>snaile</ets>, AS. <ets>sn<ae/gel</ets>, <ets>snegel</ets>, <ets>sn<ae/gl</ets>; akin to G. <ets>schnecke</ets>, OHG. <ets>snecko</ets>, Dan. <ets>snegl</ets>, Icel. <ets>snigill</ets>.]</ety> <sn>1.</sn> <fld>(Zool.)</fld> <sd>(a)</sd> <def>Any one of numerous species of terrestrial air-breathing gastropods belonging to the genus Helix and many allied genera of the family <fam>Helicidae</fam>. They are abundant in nearly all parts of the world except the arctic regions, and feed almost entirely on vegetation; a land snail.</def> <sd>(b)</sd> <def>Any gastropod having a general resemblance to the true snails, including fresh-water and marine species. See <cref>Pond snail</cref>, under <er>Pond</er>, and <er>Sea snail</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Hence, a drone; a slow-moving person or thing.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <fld>(Mech.)</fld> <def>A spiral cam, or a flat piece of metal of spirally curved outline, used for giving motion to, or changing the position of, another part, as the hammer tail of a striking clock.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>A tortoise; in ancient warfare, a movable roof or shed to protect besiegers; a testudo.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>They had also all manner of gynes [engines] . . . that needful is [in] taking or sieging of castle or of city, as <qex>snails</qex>, that was naught else but hollow pavises and targets, under the which men, when they fought, were heled [protected], . . . as the <qex>snail</qex> is in his house; therefore they cleped them <qex>snails</qex>.</q> <rj><qau>Vegetius (Trans.).</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <fld>(Bot.)</fld> <def>The pod of the sanil clover.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><mcol><col><b>Ear snail</b></col>, <col><b>Edible snail</b></col>, <col><b>Pond snail</b></col></mcol>, <cd>etc. See under <er>Ear</er>, <er>Edible</er>, etc.</cd> -- <col><b>Snail borer</b></col> <fld>(Zool.)</fld>, <cd>a boring univalve mollusk; a drill.</cd> -- <col><b>Snail clover</b></col> <fld>(Bot.)</fld>, <cd>a cloverlike plant (<spn>Medicago scuttellata</spn>, also, <spn>Medicago Helix</spn>); -- so named from its pods, which resemble the shells of snails; -- called also <altname>snail trefoil</altname>, <altname>snail medic</altname>, and <altname>beehive</altname>.</cd> -- <col><b>Snail flower</b></col> <fld>(Bot.)</fld>, <cd>a leguminous plant (<spn>Phaseolus Caracalla</spn>) having the keel of the carolla spirally coiled like a snail shell.</cd> -- <col><b>Snail shell</b></col> <fld>(Zool.)</fld>, <cd>the shell of snail.</cd> -- <col><b>Snail trefoil</b></col>. <fld>(Bot.)</fld> <cd>See <cref>Snail clover</cref>, above.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Snailfish</ent><br/
<hw>Snail"fish`</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Zool.)</fld> <def>See <er>Sea snail</er> <sd>(a)</sd>.</def><br/
[<source>1913 Webster</source>]</p>

<p><-- p. 1361 --></p>

<p><ent>Snail-like</ent><br/
<hw>Snail"-like`</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Like or suiting a snail; <as>as, <ex>snail-like</ex> progress</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Snail-like</ent><br/
<hw>Snail"-like`</hw>, <pos>adv.</pos> <def>In the manner of a snail; slowly.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Snail-paced</ent><br/
<hw>Snail"-paced`</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Slow-moving, like a snail.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Bid the <qex>snail-paced</qex> Ajax arm for shame.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>'Snails</ent><br/
<hw>'Snails</hw> <pr>(?)</pr>, <pos>interj.</pos> <def>God's nails, or His nails, that is, the nails with which the Savior was fastened to the cross; -- an ancient form of oath, corresponding to '<xex>Od's bodikins</xex> (dim. of <xex>body</xex>, i.e., God's dear body).</def>  <rj><au>Beau. & Fl.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Snake</ent><br/
<hw>Snake</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[AS. <ets>snaca</ets>; akin to LG. <ets>snake</ets>, <ets>schnake</ets>, Icel. <ets>sn<amac/kr</ets>, <ets>sn<?/kr</ets>, Dan. <ets>snog</ets>, Sw. <ets>snok</ets>; of uncertain origin.]</ety> <fld>(Zool.)</fld> <def>Any species of the order Ophidia; an ophidian; a serpent, whether harmless or venomous. See <er>Ophidia</er>, and <er>Serpent</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ Snakes are abundant in all warm countries, and much the larger number are harmless to man.</note><br/
[<source>1913 Webster</source>]</p>

<p><cs><mcol><col><b>Blind snake</b></col>, <col><b>Garter snake</b></col>, <col><b>Green snake</b></col>, <col><b>King snake</b></col>, <col><b>Milk snake</b></col>, <col><b>Rock snake</b></col>, <col><b>Water snake</b></col></mcol>, <cd>etc. See under <er>Blind</er>, <er>Garter</er>, etc.</cd> -- <col><b>Fetich snake</b></col> <fld>(Zool.)</fld>, <cd>a large African snake (<spn>Python Sebae</spn>) used by the natives as a fetich.</cd> -- <col><b>Ringed snake</b></col> <fld>(Zool.)</fld>, <cd>a common European columbrine snake (<spn>Tropidonotus natrix</spn>).</cd> -- <col><b>Snake eater</b></col>. <fld>(Zool.)</fld> <sd>(a)</sd> <cd>The markhoor</cd>. <sd>(b)</sd> <cd>The secretary bird.</cd> -- <col><b>Snake fence</b></col>, <cd>a worm fence (which see).</cd> <mark>[U.S.]</mark> -- <col><b>Snake fly</b></col> <fld>(Zool.)</fld>, <cd>any one of several species of neuropterous insects of the genus <gen>Rhaphidia</gen>; -- so called because of their large head and elongated neck and prothorax.</cd> -- <col><b>Snake gourd</b></col> <fld>(Bot.)</fld>, <cd>a cucurbitaceous plant (<spn>Trichosanthes anguina</spn>) having the fruit shorter and less snakelike than that of the serpent cucumber.</cd> -- <col><b>Snake killer</b></col>. <fld>(Zool.)</fld> <sd>(a)</sd> <cd>The secretary bird</cd>. <sd>(b)</sd> <cd>The chaparral cock.</cd> -- <col><b>Snake moss</b></col> <fld>(Bot.)</fld>, <cd>the common club moss (<spn>Lycopodium clavatum</spn>). See <er>Lycopodium</er>.</cd> -- <col><b>Snake nut</b></col> <fld>(Bot.)</fld>, <cd>the fruit of a sapindaceous tree (<spn>Ophiocaryon paradoxum</spn>) of Guiana, the embryo of which resembles a snake coiled up.</cd> -- <col><b>Tree snake</b></col> <fld>(Zool.)</fld>, <cd>any one of numerous species of colubrine snakes which habitually live in trees, especially those of the genus <gen>Dendrophis</gen> and allied genera.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Snake</ent><br/
<hw>Snake</hw>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Snaked</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Snaking</conjf>.]</vmorph> <sn>1.</sn> <def>To drag or draw, as a snake from a hole; -- often with <xex>out</xex>.</def> <mark>[Colloq. U.S.]</mark>  <rj><au>Bartlett.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Naut.)</fld> <def>To wind round spirally, as a large rope with a smaller, or with cord, the small rope lying in the spaces between the strands of the large one; to worm.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Snake</ent><br/
<hw>Snake</hw>, <pos>v. i.</pos> <def>To crawl like a snake.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Snakebird</ent><br/
<hw>Snake"bird`</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[So named from its snakelike neck.]</ety> <fld>(Zool.)</fld> <sn>1.</sn> <def>Any one of four species of aquatic birds of the genus <gen>Anhinga</gen> or <gen>Plotus</gen>.  They are allied to the gannets and cormorants, but have very long, slender, flexible necks, and sharp bills.</def><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ The American species (<spn>Anhinga anhinga</spn> syn. <spn>Plotus anhinga</spn>) inhabits the <geog>Southern United States</geog> and <geog>tropical America</geog>; -- called also <altname>darter</altname>, and <altname>water turkey</altname>.  The Asiatic species (<spn>Anhinga melanogaster</spn>) is native of <geog>Southern Asia</geog> and the <geog>East Indies</geog>.  Two other species inhabit <geog>Africa</geog> and <country>Australia</country> respectively.</note><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Zool.)</fld> <def>The wryneck.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Snakefish</ent><br/
<hw>Snake"fish`</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Zool.)</fld> <sd>(a)</sd> <def>The band fish.</def> <sd>(b)</sd> <def>The lizard fish.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Snakehead</ent><br/
<hw>Snake"head`</hw> <pr>(?)</pr>, <pos>n.</pos> <sn>1.</sn> <def>A loose, bent-up end of one of the strap rails, or flat rails, formerly used on American railroads. It was sometimes so bent by the passage of a train as to slip over a wheel and pierce the bottom of a car.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Bot.)</fld> <sd>(a)</sd> <def>The turtlehead.</def> <sd>(b)</sd> <def>The Guinea-hen flower. See <er>Snake's-head</er>, and under <er>Guinea</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Snakeneck</ent><br/
<hw>Snake"neck`</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Zool.)</fld> <def>The snakebird, 1.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Snakeroot</ent><br/
<hw>Snake"root`</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Bot.)</fld> <def>Any one of several plants of different genera and species, most of which are (or were formerly) reputed to be efficacious as remedies for the bites of serpents; also, the roots of any of these.</def><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ The Virginia snakeroot is <spn>Aristolochia Serpentaria</spn>; black snakeroot is <gen>Sanicula</gen>, esp. <spn>Sanicula Marilandica</spn>, also <spn>Cimicifuga racemosa</spn>; Seneca snakeroot is <spn>Polygala Senega</spn>; button snakeroot is <gen>Liatris</gen>, also <gen>Eryngium</gen>; white snakeroot is <spn>Eupatorium ageratoides</spn>. The name is also applied to some others besides these.</note><br/
[<source>1913 Webster</source>]</p>

<p><ent>Snake's-head</ent><br/
<hw>Snake's"-head`</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Bot.)</fld> <def>The Guinea-hen flower; -- so called in England because its spotted petals resemble the scales of a snake's head.</def>  <rj><au>Dr. Prior.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Snake's-head iris</b></col> <fld>(Bot.)</fld>, <cd>an iridaceous plant (<spn>Hermodactylus tuberosus</spn>) of the Mediterranean region. The flowers slightly resemble a serpent's open mouth.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Snakestone</ent><br/
<hw>Snake"stone`</hw> <pr>(?)</pr>, <pos>n.</pos> <sn>1.</sn> <def>A kind of hone slate or whetstone obtained in Scotland.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Paleon.)</fld> <def>An ammonite; -- so called from its form, which resembles that of a coiled snake.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Snake's-tongue</ent><br/
<hw>Snake's-tongue`</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Bot.)</fld> <def>Same as <er>Adder's-tongue</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Snakeweed</ent><br/
<hw>Snake"weed`</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Bot.)</fld> <sd>(a)</sd> <def>A kind of knotweed (<spn>Polygonum Bistorta</spn>).</def> <sd>(b)</sd> <def>The Virginia snakeroot. See <er>Snakeroot</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Snakewood</ent><br/
<hw>Snake"wood`</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Bot.)</fld> <sd>(a)</sd> <def>An East Indian climbing plant (<spn>Strychnos colubrina</spn>) having a bitter taste, and supposed to be a remedy for the bite of the hooded serpent.</def> <sd>(b)</sd> <def>An East Indian climbing shrub (<spn>Ophioxylon serpentinum</spn>) which has the roots and stems twisted so as to resemble serpents.</def> <sd>(c)</sd> <def>Same as <er>Trumpetwood</er>.</def> <sd>(d)</sd> <def>A tropical American shrub (<spn>Plumieria rubra</spn>) which has very fragrant red blossoms.</def> <sd>(e)</sd> <def>Same as <er>Letterwood</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Snakish</ent><br/
<hw>Snak"ish</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Having the qualities or characteristics of a snake; snaky.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Snaky</ent><br/
<hw>Snak"y</hw> <pr>(?)</pr>, <pos>a.</pos> <sn>1.</sn> <def>Of or pertaining to a snake or snakes; resembling a snake; serpentine; winding.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The red light playing upon its gilt and carving gave it an appearance of <qex>snaky</qex> life.</q> <rj><qau>L. Wallace.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Sly; cunning; insinuating; deceitful.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>So to the coast of Jordan he directs<br/
His easy steps, girded with <qex>snaky</qex> wiles.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>Covered with serpents; having serpents; <as>as, a <ex>snaky</ex> rod or wand</as>.</def>  <rj><au>Dryden.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>That <qex>snaky</qex>-headed, Gorgon shield.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Snap</ent><br/
<hw>Snap</hw> <pr>(?)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Snapped</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Snapping</conjf>.]</vmorph> <ety>[LG. or D. <ets>snappen</ets> to snap up, to snatch; akin to G. <ets>schnappen</ets>, MHG. <ets>snaben</ets>, Dan. <ets>snappe</ets>, and to D. <ets>snavel</ets> beak, bill.  Cf. <er>Neb</er>, <er>Snaffle</er>, <pos>n.</pos>]</ety> <sn>1.</sn> <def>To break at once; to break short, as substances that are brittle.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Breaks the doors open, <qex>snaps</qex> the locks.</q> <rj><qau>Prior.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To strike, to hit, or to shut, with a sharp sound.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>To bite or seize suddenly, especially with the teeth.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>He, by playing too often at the mouth of death, has been <qex>snapped</qex> by it at last.</q> <rj><qau>South.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>To break upon suddenly with sharp, angry words; to treat snappishly; -- usually with <xex>up</xex>.</def>  <rj><au>Granville.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <def>To crack; to cause to make a sharp, cracking noise; <as>as, to <ex>snap</ex> a whip</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>MacMorian <qex>snapped</qex> his fingers repeatedly.</q> <rj><qau>Sir W. Scott.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>6.</sn> <def>To project with a snap.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>7.</sn> <fld>(Cricket)</fld> <def>To catch out sharply (a batsman who has just snicked a bowled ball).</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><cs><col><b>To snap back</b></col> <fld>(Football)</fld>, <cd>to roll the ball back with the foot; -- done only by the center rush, who thus delivers the ball to the quarter back on his own side when both sides are ranged in line.</cd><-- now passed back with the hands, but still called "snap" --> -- <col><b>To snap off</b></col>. <sd>(a)</sd> <cd>To break suddenly</cd>. <sd>(b)</sd> <cd>To bite off suddenly.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Snap</ent><br/
<hw>Snap</hw>, <pos>v. i.</pos> <sn>1.</sn> <def>To break short, or at once; to part asunder suddenly; <as>as, a mast <ex>snaps</ex>; a needle <ex>snaps</ex></as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>But this weapon will <qex>snap</qex> short, unfaithful to the hand that employs it.</q> <rj><qau>Burke.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To give forth, or produce, a sharp, cracking noise; to crack; <as>as, blazing firewood <ex>snaps</ex></as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>To make an effort to bite; to aim to seize with the teeth; to catch eagerly (at anything); -- often with <xex>at</xex>; <as>as, a dog <ex>snaps</ex>at a passenger; a fish <ex>snaps</ex> at the bait.</as></def><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>To utter sharp, harsh, angry words; -- often with <xex>at</xex>; <as>as, to <ex>snap</ex> at a child</as>.</def><-- usu. impulsively and as a quick reaction to some perceived provocation --><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <def>To miss fire; <as>as, the gun <ex>snapped</ex></as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>6.</sn> <def>Of the eyes, to emit sudden, brief sparkles like those of a snapping fire, as sometimes in anger.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Snap</ent><br/
<hw>Snap</hw>, <pos>n.</pos> <ety>[Cf. D. <ets>snap</ets> a snatching. See <er>Snap</er>, <pos>v. t.</pos>]</ety> <sn>1.</sn> <def>A sudden breaking or rupture of any substance.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A sudden, eager bite; a sudden seizing, or effort to seize, as with the teeth.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>A sudden, sharp motion or blow, as with the finger sprung from the thumb, or the thumb from the finger.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>A sharp, abrupt sound, as that made by the crack of a whip; <as>as, the <ex>snap</ex> of the trigger of a gun</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <def>A greedy fellow.</def>  <rj><au>L'Estrange.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>6.</sn> <def>That which is, or may be, snapped up; something bitten off, seized, or obtained by a single quick movement; hence, a bite, morsel, or fragment; a scrap.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>He's a nimble fellow,<br/
And alike skilled in every liberal science,<br/
As having certain <qex>snaps</qex> of all.</q> <rj><qau>B. Jonson.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>7.</sn> <def>A sudden severe interval or spell; -- applied to the weather; <as>as, a cold <ex>snap</ex></as>.</def>
<-- but not a heat snap -->  <rj><au>Lowell.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>8.</sn> <def>A small catch or fastening held or closed by means of a spring, or one which closes with a snapping sound, as the catch of a bracelet, necklace, clasp of a book, etc.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>9.</sn> <fld>(Zool.)</fld> <def>A snap beetle.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>10.</sn> <def>A thin, crisp cake, usually small, and flavored with ginger; -- used chiefly in the plural.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>11.</sn> <def>Briskness; vigor; energy; decision.</def> <mark>[Colloq.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><sn>12.</sn> <def>Any circumstance out of which money may be made or an advantage gained.</def> <mark>[Slang]</mark><br/
[<source>1913 Webster</source>]</p>

<p><sn>13.</sn> <def>Any task, labor, set of circumstances, or the like, that yields satisfactory results or gives pleasure with little trouble or effort, as an easy course of study, a job where work is light, a bargain, etc.</def> <mark>[Slang, Chiefly U. S.]</mark><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><sn>14.</sn>  <def>A snap shot with a firearm.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><sn>15.</sn>  <fld>(Photog.)</fld> <def>A snapshot.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><sn>16.</sn>  <def>Something of no value; <as>as, not worth a <ex>snap</ex></as>.</def> <mark>[Colloq.]</mark><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><sn>17.</sn> <fld>(Football)</fld> <def>The action of snapping the ball back, from the center usu. to the quarterback, which commences the play (down), and, if the clock had stopped, restarts the timer clock; a <cref>snap back</cref>.</def><br/
[<source>PJC</source>]</p>

<p><cs><col><b>Snap back</b></col> <fld>(Football)</fld>, <cd>the act of snapping back the ball.</cd> -- <mcol><col><b>Snap beetle</b></col>, <it>or</it> <col><b>Snap bug</b></col></mcol> <fld>(Zool.)</fld>, <cd>any beetle of the family <fam>Elateridae</fam>, which, when laid on its back, is able to leap to a considerable height by means of a thoracic spring; -- called also <cref>snapping beetle</cref>.</cd> -- <col><b>Snap flask</b></col> <fld>(Molding)</fld>, <cd>a flask for small work, having its sides separable and held together by latches, so that the flask may be removed from around the sand mold.</cd> -- <col><b>Snap judgment</b></col>, <cd>a judgment formed on the instant without deliberation.</cd> -- <col><b>Snap lock</b></col>, <cd>a lock shutting with a catch or snap.</cd> -- <col><b>Snap riveting</b></col>, <cd>riveting in which the rivets have snapheads formed by a die or swaging tool.</cd> -- <col><b>Snap shot</b></col>, <cd>a quick offhand shot, without deliberately taking aim.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Snap</ent><br/
<hw>Snap</hw>, <pos>a.</pos> <def>Done, performed, made, executed, carried through, or the like, quickly and without deliberation; <as>as, a <ex>snap</ex> judgment or decision; a <ex>snap</ex> political convention</as>.</def> <mark>[Colloq.]</mark><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Snapdragon</ent><br/
<hw>Snap"drag`on</hw> <pr>(?)</pr>, <pos>n.</pos> <sn>1.</sn> <fld>(Bot.)</fld> <sd>(a)</sd> <def>Any plant of the scrrophulariaceous genus <gen>Antirrhinum</gen>, especially the cultivated <spn>Antirrhinum majus</spn>, whose showy flowers are fancifully likened to the face of a dragon.</def> <sd>(b)</sd> <def>A West Indian herb (<spn>Ruellia tuberosa</spn>) with curiously shaped blue flowers.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A play in which raisins are snatched from a vessel containing burning brandy, and eaten; also, that which is so eaten. See <er>Flapdragon</er>.</def>  <rj><au>Swift.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Snape</ent><br/
<hw>Snape</hw> <pr>(?)</pr>, <pos>v. t.</pos> <fld>(Shipbuilding)</fld> <def>To bevel the end of a timber to fit against an inclined surface.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Snaphance</ent><br/
<hw>Snap"hance`</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[D. <ets>snaphaan</ets> a gun, originally, the snapping cock of a gun. See <er>Snap</er>, and <er>Hen</er>.]</ety> <sn>1.</sn> <def>A spring lock for discharging a firearm; also, the firearm to which it is attached.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A trifling or second-rate thing or person.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Snaphead</ent><br/
<hw>Snap"head`</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A hemispherical or rounded head to a rivet or bolt; also, a swaging tool with a cavity in its face for forming such a rounded head.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Snapper</ent><br/
<hw>Snap"per</hw> <pr>(?)</pr>, <pos>n.</pos> <sn>1.</sn> <def>One who, or that which, snaps; <as>as, a <ex>snapper</ex> up of trifles; the <ex>snapper</ex> of a whip.</as></def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Zool.)</fld> <def>Any one of several species of large sparoid food fishes of the genus <gen>Lutjanus</gen>, abundant on the southern coasts of the United States and on both coasts of tropical America.</def><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ The red snapper (<spn>Lutjanus aya</spn> syn. <spn>Lutjanus Blackfordi</spn>) and the gray, or mangrove, snapper (<spn>Lutjanus griseus</spn>) are large and abundant species.  The name is loosely applied to various other fishes, as the bluefish, the rosefish, the red grouper, etc. See <er>Rosefish</er>.</note><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <fld>(Zool.)</fld> <def>A snapping turtle; <as>as, the alligator <ex>snapper</ex></as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <fld>(Zool.)</fld> <def>The green woodpecker, or yaffle.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <fld>(Zool.)</fld> <def>A snap beetle.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>6.</sn> <fld>(Teleg.)</fld> <def>A device with a flexible metal tongue for producing clicks like those of the sounder.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><sn>7.</sn>  <def>A string bean.</def> <mark>[Colloq., U. S.]</mark><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Snapping</ent><br/
<hw>Snap"ping</hw> <pr>(?)</pr>, <def><pos>a. & n.</pos> from <er>Snap</er>, <pos>v.</pos></def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Snapping beetle</b></col>. <fld>(Zool.)</fld> <cd>See <cref>Snap beetle</cref>, under <er>Snap</er>.</cd> -- <col><b>Snapping turtle</b></col>. <fld>(Zool.)</fld> <sd>(a)</sd> <cd>A large and voracious aquatic turtle (<spn>Chelydra serpentina</spn>) common in the fresh waters of the United States; -- so called from its habit of seizing its prey by a snap of its jaws. Called also <altname>mud turtle</altname>.</cd> <sd>(b)</sd> <cd>See <cref>Alligator snapper</cref>, under <er>Alligator</er>.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Snappish</ent><br/
<hw>Snap"pish</hw> <pr>(?)</pr>, <pos>a.</pos> <sn>1.</sn> <def>Apt to snap at persons or things; eager to bite; <as>as, a <ex>snapping</ex> cur</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Sharp in reply; apt to speak angrily or testily; easily provoked; tart; peevish.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The taunting address of a <qex>snappish</qex> misanthrope.</q> <rj><qau>Jeffrey.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p>-- <wordforms><wf>Snap"pish*ly</wf>, <pos>adv.</pos> -- <wf>Snap"pish*ness</wf>, <pos>n.</pos></wordforms><br/
[<source>1913 Webster</source>]</p>

<p><ent>Snappy</ent><br/
<hw>Snap"py</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Snappish.</def> <mark>[Colloq.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Snapsack</ent><br/
<hw>Snap"sack`</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Cf. Sw. <ets>snapps<aum/ck</ets>, G. <ets>schnappsack</ets>.]</ety> <def>A knapsack.</def> <mark>[Obs.]</mark>  <rj><au>South.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Snap shot</ent><br/
<ent>Snapshot</ent><br/
<hw>Snap"shot`</hw> <pr>(?)</pr>, <pos>n.</pos> <sn>1.</sn> <altsp>Commonly <hw>Snap shot</hw>.</altsp> <sd>(a)</sd> <def>A quick offhand shot, made without deliberately taking aim over the sights.</def> <sd>(b)</sd> <fld>(Photog.)</fld> <def>Act of taking a snapshot (in sense 2).</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><sn>2.</sn>  <def>An instantaneous photograph made, usually with a hand camera, without formal posing of, and often without the foreknowledge of, the subject.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><sn>3.</sn>  <def>a brief summary or appraisal, especially one that describes the state of a situation at one particular time; <as>as, a <ex>snapshot</ex> of the moon project in 1966</as>.</def><br/
[<source>PJC</source>]</p>

<p><ent>Snapweed</ent><br/
<hw>Snap"weed`</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Bot.)</fld> <def>See <er>Impatiens</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Snar</ent><br/
<hw>Snar</hw> <pr>(?)</pr>, <pos>v. i.</pos> <ety>[Akin to LG. & OD. <ets>snarren</ets>, G. <ets>schnarren</ets>, E. <ets>snore</ets>. See <er>Snore</er>, and cf. <er>Snarl</er> to growl.]</ety> <def>To snarl.</def> <mark>[Obs.]</mark>  <rj><au>Spenser.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Snare</ent><br/
<hw>Snare</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[AS. <ets>snear</ets>a cord, a string; akin to D. <ets>snoer</ets>, G. <ets>schnur</ets>, OHG. <ets>snour</ets> a cord, <ets>snarahha</ets> a noose, Dan. <ets>snare</ets>, Sw. & Icel. <ets>snara</ets>, Goth. <ets>sn<?/rj<?/</ets> a basket; and probably also to E. <ets>needle</ets>. See <er>Needle</er>, and cf. <er>Snarl</er> to entangle.]</ety> <sn>1.</sn> <def>A contrivance, often consisting of a noose of cord, or the like, by which a bird or other animal may be entangled and caught; a trap; a gin.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Hence, anything by which one is entangled and brought into trouble.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>If thou retire, the Dauphin, well appointed,<br/
Stands with the <qex>snares</qex> of war to tangle thee.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>The gut or string stretched across the lower head of a drum.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <fld>(Med.)</fld> <def>An instrument, consisting usually of a wireloop or noose, for removing tumors, etc., by avulsion.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Snare drum</b></col>, <cd>the smaller common military drum, as distinguished from the <xex>bass drum</xex>; -- so called because (in order to render it more resonant) it has stretched across its lower head a catgut string or strings.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Snare</ent><br/
<hw>Snare</hw>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Snared</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Snaring</conjf>.]</vmorph> <def>To catch with a snare; to insnare; to entangle; hence, to bring into unexpected evil, perplexity, or danger.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Lest that too heavenly form . . . <qex>snare</qex> them.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>The mournful crocodile<br/
With sorrow <qex>snares</qex> relenting passengers.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Snarer</ent><br/
<hw>Snar"er</hw> <pr>(?)</pr>, <pos>n.</pos> <def>One who lays snares, or entraps.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Snarl</ent><br/
<hw>Snarl</hw> <pr>(?)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Snarled</conjf> <pr>(?)</pr>; <pos>p. pr. & vvb. n.</pos> <conjf>Snarling</conjf>.]</vmorph> <ety>[Etymol. uncertain.]</ety> <def>To form raised work upon the outer surface of (thin metal ware) by the repercussion of a snarling iron upon the inner surface.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Snarl</ent><br/
<hw>Snarl</hw>, <pos>v. t.</pos> <ety>[From <er>Snare</er>, <pos>v. t.</pos>]</ety> <sn>1.</sn> <def>To entangle; to complicate; to involve in knots; <as>as, to <ex>snarl</ex> a skein of thread</as>.</def> <ldquo/Her <xex>snarled</xex> hair.<rdquo/  <rj><au>Spenser.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To embarrass; to insnare.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>[The] question that they would have <qex>snarled</qex> him with.</q> <rj><qau>Latimer.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Snarl</ent><br/
<hw>Snarl</hw>, <pos>n.</pos> <def>A knot or complication of hair, thread, or the like, difficult to disentangle; entanglement; hence, intricate complication; embarrassing difficulty.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Snarl</ent><br/
<hw>Snarl</hw>, <pos>v. i.</pos> <ety>[From <er>Snar</er>.]</ety> <sn>1.</sn> <def>To growl, as an angry or surly dog; to gnarl; to utter grumbling sounds.</def> <ldquo/An angry cur <xex>snarls</xex> while he feeds.<rdquo/  <rj><au>Dryden & Lee.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To speak crossly; to talk in rude, surly terms.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>It is malicious and unmanly to <qex>snarl</qex> at the little lapses of a pen, from which Virgil himself stands not exempted.</q> <rj><qau>Dryden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Snarl</ent><br/
<hw>Snarl</hw>, <pos>n.</pos> <def>The act of snarling; a growl; a surly or peevish expression; an angry contention.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Snarler</ent><br/
<hw>Snarl"er</hw> <pr>(?)</pr>, <pos>n.</pos> <def>One who snarls; a surly, growling animal; a grumbling, quarrelsome fellow.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Snarler</ent><br/
<hw>Snarl"er</hw>, <pos>n.</pos> <def>One who makes use of a snarling iron.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Snarling</ent><br/
<hw>Snarl"ing</hw>, <def><pos>a. & n.</pos> from <er>Snarl</er>, <pos>v.</pos></def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Snarling iron</b></col>, <cd>a tool with a long beak, used in the process of snarling. When one end is held in a vise, and the shank is struck with a hammer, the repercussion of the other end, or beak, within the article worked upon gives the requisite blow for producing raised work. See 1st <er>Snarl</er>.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Snary</ent><br/
<hw>Snar"y</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[From <er>Snare</er>.]</ety> <def>Resembling, or consisting of, snares; entangling; insidious.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Spiders in the vault their <qex>snary</qex> webs have spread.</q> <rj><qau>Dryden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Snast</ent><br/
<hw>Snast</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Cf. <er>Snite</er>, <pos>v. t.</pos>]</ety> <def>The snuff, or burnt wick, of a candle.</def> <mark>[Obs.]</mark>  <rj><au>Bacon.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Snatch</ent><br/
<hw>Snatch</hw> <pr>(?)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Snatched</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Snatching</conjf>.]</vmorph> <ety>[OE. <ets>snachen</ets>, <ets>snechen</ets>; akin to D. <ets>snakken</ets> to gasp, to long (for), to desire.  Cf. <er>Snack</er>, <pos>n.</pos>, <er>Sneck</er>.]</ety> <sn>1.</sn> <def>To take or seize hastily, abruptly, or without permission or ceremony; <as>as, to <ex>snatch</ex> a loaf or a kiss</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>When half our knowledge we must <qex>snatch</qex>, not take.</q> <rj><qau>Pope.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To seize and transport away; to rap.</def> <ldquo/<xex>Snatch</xex> me to heaven.<rdquo/  <rj><au>Thomson.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><syn><b>Syn.</b> -- To twitch; pluck; grab; catch; grasp; gripe.</syn><br/
[<source>1913 Webster</source>]</p>

<p><ent>Snatch</ent><br/
<hw>Snatch</hw>, <pos>v. i.</pos> <def>To attempt to seize something suddenly; to catch; -- often with <xex>at</xex>; <as>as, to <ex>snatch</ex> at a rope</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><-- p. 1362 --></p>

<p><ent>Snatch</ent><br/
<hw>Snatch</hw> <pr>(?)</pr>, <pos>n.</pos> <sn>1.</sn> <def>A hasty catching or seizing; a grab; a catching at, or attempt to seize, suddenly.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A short period of vigorous action; <as>as, a <ex>snatch</ex> at weeding after a shower</as>.</def>  <rj><au>Tusser.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>They move by fits and <qex>snatches</qex>.</q> <rj><qau>Bp. Wilkins.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>A small piece, fragment, or quantity; a broken part; a scrap.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>We have often little <qex>snatches</qex> of sunshine.</q> <rj><qau>Spectator.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Leave me your <qex>snatches</qex>, and yield me a direct answer.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Snatch block</ent><br/
<hw>Snatch block</hw> <fld>(Naut.)</fld>, <def>a kind of block with an opening in one side to receive the bight of a rope.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Snatcher</ent><br/
<hw>Snatch"er</hw> <pr>(?)</pr>, <pos>n.</pos> <def>One who snatches, or takes abruptly.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Snatchingly</ent><br/
<hw>Snatch"ing*ly</hw>, <pos>adv.</pos> <def>By snatching; abruptly.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Snath</ent><br/
<hw>Snath</hw> <pr>(sn<acr/th)</pr>, <pos>n.</pos> <ety>[Cf. AS. <ets>sn<imac/<edh/an</ets> to cut, to mow, <ets>sn<aemac/d</ets> a bite, bit, snip.]</ety> <def>The handle of a scythe; a snead.</def> <altsp>[Variously written in England <asp>snead</asp>, <asp>sneed</asp>, <asp>sneath</asp>, <asp>sneeth</asp>, <asp>snathe</asp>, etc.; in Scotland written <asp>sned</asp>.]</altsp><br/
[<source>1913 Webster</source>]</p>

<p><ent>Snathe</ent><br/
<hw>Snathe</hw> <pr>(sn<amac/<th/)</pr>, <pos>v. t.</pos> <ety>[Cf. Icel. <ets>snei<edh/a</ets> to cut into alices, <ets>sn<imac/<edh/a</ets> to cut; akin to AS. be<ets>sn<aemac/dan</ets>, <ets>sn<imac/<edh/an</ets>, G. <ets>schneiden</ets>, OHG. <ets>sn<imac/dan</ets>, Goth. <ets>snei<thorn/an</ets> to cut, to reap, and E. <ets>snath</ets>, <ets>snithe</ets>.]</ety> <def>To lop; to prune.</def> <mark>[Prov. Eng.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Snattock</ent><br/
<hw>Snat"tock</hw> <pr>(sn<acr/t"t<ucr/k)</pr>, <pos>n.</pos> <ety>[See <er>Snathe</er>.]</ety> <def>A chip; a slice.</def> <mark>[Prov. Eng.]</mark>  <rj><au>Gayton.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Snaw</ent><br/
<hw>Snaw</hw> <pr>(sn<add/)</pr>, <pos>n.</pos> <def>Snow.</def> <mark>[Obs. or Scot.]</mark>  <rj><au>Burns.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Snead</ent><br/
<hw>Snead</hw> <pr>(sn<emac/d)</pr>, <pos>n.</pos> <ety>[See <er>Snath</er>.]</ety> <sn>1.</sn> <def>A snath.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A line or cord; a string.</def> <mark>[Prov. Eng.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sneak</ent><br/
<hw>Sneak</hw> <pr>(sn<emac/k)</pr>, <pos>v. i.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Sneaked</conjf> <pr>(sn<emac/kt)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Sneaking</conjf>.]</vmorph> <ety>[OE. <ets>sniken</ets>, AS. <ets>sn<imac/can</ets> to creep; akin to Dan. <ets>snige sig</ets>; cf. Icel. <ets>sn<imac/kja</ets> to hanker after.]</ety> <sn>1.</sn> <def>To creep or steal (away or about) privately; to come or go meanly, as a person afraid or ashamed to be seen; <as>as, to <ex>sneak</ex> away from company</as>.</def>
<-- ? <pos>imp. & p. p.</pos> "snuck" is more common now, but not even mentioned here. In MW10, simply "sneaked or snuck" --><br/
[<source>1913 Webster</source>]</p>

<p><q>You skulked behind the fence, and <qex>sneaked</qex> away.</q> <rj><qau>Dryden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To act in a stealthy and cowardly manner; to behave with meanness and servility; to crouch.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sneak</ent><br/
<hw>Sneak</hw>, <pos>v. t.</pos> <def>To hide, esp. in a mean or cowardly manner.</def> <mark>[Obs.]</mark> <ldquo/[Slander]  <xex>sneaks</xex> its head.<rdquo/  <rj><au>Wake.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sneak</ent><br/
<hw>Sneak</hw>, <pos>n.</pos> <sn>1.</sn> <def>A mean, sneaking fellow.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>A set of simpletons and superstitious <qex>sneaks</qex>.</q> <rj><qau>Glanvill.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Cricket)</fld> <def>A ball bowled so as to roll along the ground; -- called also <altname>grub</altname>.</def> <mark>[Cant]</mark>  <rj><au>R. A. Proctor.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sneak-cup</ent><br/
<hw>Sneak"-cup`</hw> <pr>(?)</pr>, <pos>n.</pos> <def>One who sneaks from his cups; one who balks his glass.</def> <mark>[Obs.]</mark>  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sneak current</ent><br/
<hw>Sneak current</hw>. <fld>(Elec.)</fld> <def>A current which, though too feeble to blow the usual fuse or to injure at once telegraph or telephone instruments, will in time burn them out.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Sneaker</ent><br/
<hw>Sneak"er</hw> <pr>(sn<emac/k"<etil/r)</pr>, <pos>n.</pos> <sn>1.</sn> <def>One who sneaks.</def>  <rj><au>Lamb.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A vessel of drink.</def> <mark>[Prov. Eng.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>A <qex>sneaker</qex> of five gallons.</q> <rj><qau>Spectator.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>A type of soft shoe with a flat, pliable, typically rubber or other soft sole, and canvas-like upper, used in sports such as tennis, or for comfort.  Called <ex>sneaker</ex> because they give no warning of one's approach.  Usually used in the <pluf>pl.</pluf></def><br/
[<source>Webster 1913 Suppl.</source> <source>+PJC</source>]</p>

<p><sn>4.</sn>  <def>A punch bowl.</def> <mark>[Obs.]</mark>  <rj><au>Spectator.</au></rj><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Sneakiness</ent><br/
<hw>Sneak"i*ness</hw> <pr>(?)</pr>, <pos>n.</pos> <def>The quality of being sneaky.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sneaking</ent><br/
<hw>Sneak"ing</hw>, <pos>a.</pos> <def>Marked by cowardly concealment; deficient in openness and courage; underhand; mean; crouching.</def> -- <wordforms><wf>Sneak"ing*ly</wf>, <pos>adv.</pos> -- <wf>Sneak"ing*ness</wf>, <pos>n.</pos></wordforms><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sneaksby</ent><br/
<hw>Sneaks"by</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A paltry fellow; a sneak.</def> <mark>[Obs.]</mark> <ldquo/Such a bashful <xex>sneaksby</xex>.<rdquo/  <rj><au>Barrow.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sneaky</ent><br/
<hw>Sneak"y</hw> <pr>(?)</pr>, <pos>n.</pos> <def>Like a sneak; sneaking.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sneap</ent><br/
<hw>Sneap</hw> <pr>(?)</pr>, <pos>v. t.</pos> <ety>[Cf. Icel. <ets>sneypa</ets> to dishonor, disgrace, chide, but also E. <ets>snip</ets>, and <ets>snub</ets>.]</ety> <sn>1.</sn> <def>To check; to reprimand; to rebuke; to chide.</def> <mark>[Obs.]</mark>  <rj><au>Bp. Hall.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To nip; to blast; to blight.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>Biron is like an envious, <qex>sneaping</qex> frost.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sneap</ent><br/
<hw>Sneap</hw>, <pos>n.</pos> <def>A reprimand; a rebuke.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>My lord, I will not undergo this <qex>sneap</qex> without reply.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sneathe</ent><br/
<ent>Sneath</ent><br/
<mhw>{ <hw>Sneath</hw> <pr>(?)</pr>, <hw>Sneathe</hw> <pr>(?)</pr>, }</mhw> <pos>n.</pos> <def>See <er>Snath</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sneb</ent><br/
<hw>Sneb</hw> <pr>(?)</pr>, <pos>v. t.</pos> <ety>[See <er>Snib</er>.]</ety> <def>To reprimand; to sneap.</def> <mark>[Obs.]</mark> <ldquo/Scold and <xex>sneb</xex> the good oak.<rdquo/  <rj><au>Spenser.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sneck</ent><br/
<hw>Sneck</hw> <pr>(?)</pr>, <pos>v. t.</pos> <ety>[See <er>Snatch</er>.]</ety> <def>To fasten by a hatch; to latch, as a door.</def> <mark>[Scot. & Prov. Eng.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Sneck up</b></col>, <cd>be silent; shut up; hold your peace.</cd></cs>
<-- Obs or archaic? -->  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sneck</ent><br/
<hw>Sneck</hw>, <pos>n.</pos> <def>A door latch.</def> <mark>[Scot. & Prov. Eng.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Sneck band</b></col>, <cd>a latchstring.</cd> <au>Burns.</au> -- <col><b>Sneck drawer</b></col>, <cd>a latch lifter; a bolt drawer; hence, a sly person; a cozener; a cheat; -- called also <altname>sneckdraw</altname>.</cd> -- <col><b>Sneck drawing</b></col>, <cd>lifting the latch.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Snecket</ent><br/
<hw>Sneck"et</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A door latch, or sneck.</def> <mark>[Prov. Eng.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sned</ent><br/
<hw>Sned</hw> <pr>(?)</pr>, <pos>v. t.</pos> <def>To lop; to snathe.</def> <mark>[Prov. Eng.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sneed</ent><br/
<ent>Sned</ent><br/
<mhw>{ <hw>Sned</hw> <pr>(?)</pr>, <hw>Sneed</hw> <pr>(?)</pr>, }</mhw> <pos>n.</pos> <def>See <er>Snath</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sneer</ent><br/
<hw>Sneer</hw> <pr>(?)</pr>, <pos>v. i.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Sneered</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Sneering</conjf>.]</vmorph> <ety>[OE. <ets>sneren</ets>, Dan. <ets>sn<?/rre</ets> to snarl or grin (like a dog); cf. Prov. E. <ets>sneer</ets> to grin, <ets>sner</ets> to snort, <ets>snert</ets> to sneer at. See <er>Snore</er>, <pos>v. i.</pos>]</ety> <sn>1.</sn> <def>To show contempt by turning up the nose, or by a particular facial expression.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To inssinuate contempt by a covert expression; to speak derisively.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>I could be content to be a little <qex>sneared</qex> at.</q> <rj><qau>Pope.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>To show mirth awkwardly.</def> <mark>[R.]</mark>  <rj><au>Tatler.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><syn><b>Syn.</b> -- To scoff; gibe; jeer.</syn>  -- <usage><er>Sneer</er>, <er>Scoff</er>, <er>Jeer</er>. The verb to <xex>sneer</xex> implies to cast contempt indirectly or by covert expressions. To <xex>jeer</xex> is stronger, and denotes the use of several sarcastic reflections. To <xex>scoff</xex> is stronger still, implying the use of insolent mockery and derision.</usage><br/
[<source>1913 Webster</source>]</p>

<p><q>And <qex>sneers</qex> as learnedly as they,<br/
Like females o'er their morning tea.</q> <rj><qau>Swift.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Midas, exposed to all their <qex>jeers</qex>,<br/
Had lost his art, and kept his ears.</q> <rj><qau>Swift.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>The fop, with learning at defiance,<br/
<qex>Scoffs</qex> at the pedant and science.</q> <rj><qau>Gay.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sneer</ent><br/
<hw>Sneer</hw>, <pos>v. t.</pos> <sn>1.</sn> <def>To utter with a grimace or contemptuous expression; to utter with a sneer; to say sneeringly; <as>as, to <ex>sneer</ex> fulsome lies at a person</as>.</def>  <rj><au>Congreve.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q><ldquo/A ship of fools,<rdquo/ he <qex>sneered</qex>.</q> <rj><qau>Tennyson.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To treat with sneers; to affect or move by sneers.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Nor <qex>sneered</qex> nor bribed from virtue into shame.</q> <rj><qau>Savage.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sneer</ent><br/
<hw>Sneer</hw>, <pos>n.</pos> <sn>1.</sn> <def>The act of sneering.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A smile, grin, or contortion of the face, indicative of contempt; an indirect expression or insinuation of contempt.</def> <ldquo/Who can refute a <xex>sneer</xex>?<rdquo/  <rj><au>Raley.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sneerer</ent><br/
<hw>Sneer"er</hw> <pr>(?)</pr>, <pos>n.</pos> <def>One who sneers.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sneerful</ent><br/
<hw>Sneer"ful</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Given to sneering.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sneeringly</ent><br/
<hw>Sneer"ing*ly</hw>, <pos>adv.</pos> <def>In a sneering manner.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sneeze</ent><br/
<hw>Sneeze</hw> <pr>(?)</pr>, <pos>v. i.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Sneezed</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Sneezing</conjf>.]</vmorph> <ety>[OE. <ets>snesen</ets>; of uncertain origin; cf. D. <ets>snuse</ets> to sniff, E. <ets>neese</ets>, and AS. <ets>fne<oacute/san</ets>.]</ety> <def>To emit air, chiefly through the nose, audibly and violently, by a kind of involuntary convulsive force, occasioned by irritation of the inner membrane of the nose.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Not to be sneezed at</b></col>, <cd>not to be despised or contemned; not to be treated lightly.</cd> <mark>[Colloq.]</mark> <ldquo/He had to do with old women who were <xex>not to be sneezed at</xex>.<rdquo/  <rj><au>Prof. Wilson.</au></rj></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sneeze</ent><br/
<hw>Sneeze</hw>, <pos>n.</pos> <def>A sudden and violent ejection of air with an audible sound, chiefly through the nose.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sneezeweed</ent><br/
<hw>Sneeze"weed`</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Bot.)</fld> <def>A yellow-flowered composite plant (<spn>Helenium autumnale</spn>) the odor of which is said to cause sneezing.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sneezewood</ent><br/
<hw>Sneeze"wood`</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Bot.)</fld> <def>The wood of a South African tree. See <er>Neishout</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sneezewort</ent><br/
<hw>Sneeze"wort`</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Bot.)</fld> <def>A European herbaceous plant (<spn>Achillea Ptarmica</spn>) allied to the yarrow, having a strong, pungent smell.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sneezing</ent><br/
<hw>Sneez"ing</hw>, <pos>n.</pos> <fld>(Physiol.)</fld> <def>The act of violently forcing air out through the nasal passages while the cavity of the mouth is shut off from the pharynx by the approximation of the soft palate and the base of the tongue.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Snell</ent><br/
<hw>Snell</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[AS. <ets>snell</ets>; akin to D. <ets>snel</ets>, G. <ets>schnell</ets>, OHG. <ets>snel</ets>, Icel. <ets>snjallr</ets> valiant.]</ety> <def>Active; brisk; nimble; quick; sharp.</def> <mark>[Archaic or Prov. Eng. & Scot.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>That horny-handed, <qex>snell</qex>, peremptory little man.</q> <rj><qau>Dr. J. Brown.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Snell</ent><br/
<hw>Snell</hw>, <pos>n.</pos> <def>A short line of horsehair, gut, etc., by which a fishhook is attached to a longer line.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Snet</ent><br/
<hw>Snet</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Cf. G. <ets>schnitt</ets> that which is cut, fr. <ets>schneiden</ets> to cut, E. <ets>snath</ets>.]</ety> <def>The fat of a deer.</def> <mark>[Obs. or Prov. Eng.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Snet</ent><br/
<hw>Snet</hw>, <pos>v. t.</pos> <ety>[See <er>Snot</er>.]</ety> <def>The clear of mucus; to blow.</def> <mark>[Obs.]</mark> <ldquo/<xex>Snetting</xex> his nose.<rdquo/  <rj><au>Holland.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Snew</ent><br/
<hw>Snew</hw> <pr>(?)</pr>, <pos>v. i.</pos> <def>To snow; to abound.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>It <qex>snewed</qex> in his house of meat and drink.</q> <rj><qau>Chaucer.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Snib</ent><br/
<hw>Snib</hw> <pr>(?)</pr>, <pos>v. t.</pos> <ety>[OE. <ets>snibben</ets>; cf. Dan. <ets>snibbe</ets>, and E. <ets>snub</ets>, <pos>v. t.</pos>]</ety> <def>To check; to sneap; to sneb.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>Him would he <qex>snib</qex> sharply for the nones.</q> <rj><qau>Chaucer.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Snib</ent><br/
<hw>Snib</hw>, <pos>n.</pos> <def>A reprimand; a snub.</def> <mark>[Obs.]</mark>  <rj><au>Marston.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Snick</ent><br/
<hw>Snick</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Prov. E. <ets>snick</ets> a notch; cf. Icel. <ets>snikka</ets> nick, cut.]</ety> <sn>1.</sn> <def>A small cut or mark.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Cricket)</fld> <def>A slight hit or tip of the ball, often unintentional.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <fld>(Fiber)</fld> <def>A knot or irregularity in yarn.</def>  <rj><au>Knight.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <fld>(Furriery)</fld> <def>A snip or cut, as in the hair of a beast.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Snick and snee</b></col> <ety>[cf. D. <ets>snee</ets>, <ets>snede</ets>, a cut]</ety>, <cd>a combat with knives.</cd> <mark>[Obs.]</mark>  <rj><au>Wiseman.</au></rj></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Snick</ent><br/
<hw>Snick</hw>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Snicked</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Snicking</conjf>.]</vmorph> <sn>1.</sn> <def>To cut slightly; to strike, or strike off, as by cutting.</def>  <rj><au>H. Kingsley.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Cricket)</fld> <def>To hit (a ball) lightly.</def>  <rj><au>R. A. Proctor.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Snick</ent><br/
<hw>Snick</hw>, <pos>n. & v. t.</pos> <def>See <er>Sneck</er>.</def> <mark>[Prov. Eng. & Scot.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Snick up</b></col>, <cd>shut up; silenced. See <xex>Sneck up</xex>, under <er>Sneck</er>.</cd><br/
[<source>1913 Webster</source>]</p>

<p><q>Give him money, George, and let him go <qex>snick up</qex>.</q> <rj><qau>Beau. & Fl.</qau></rj>
</cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Snicker</ent><br/
<hw>Snick"er</hw> <pr>(?)</pr>, <pos>v. i.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Snickered</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Snickering</conjf>.]</vmorph> <ety>[Cf. D. <ets>snikken</ets> to sob, to sigh.]</ety> <altsp>[Written also <asp>snigger</asp>.]</altsp> <sn>1.</sn> <def>To laugh slyly; to laugh in one's sleeve.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To laugh with audible catches of voice, as when persons attempt to suppress loud laughter.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Snicker</ent><br/
<hw>Snick"er</hw>, <pos>n.</pos> <def>A half suppressed, broken laugh.</def> <altsp>[Written also <asp>snigger</asp>.]</altsp><br/
[<source>1913 Webster</source>]</p>

<p><ent>Snide</ent><br/
<hw>Snide</hw> <pr>(?)</pr>, <pos>a.</pos> <sn>1.</sn> <def>Tricky; deceptive; contemptible; <as>as, a <ex>snide</ex> lawyer; <ex>snide</ex> goods.</as></def> <mark>[Slang, archaic]</mark><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>derogatory in an insinuating manner; <as>as, a <ex>snide</ex> remark</as>.</def><br/
[<source>PJC</source>]</p>

<p><ent>Snider</ent><br/
<ent>Snider rifle</ent><br/
<mhw>{ <hw>Sni"der ri"fle</hw>, <it>or</it> <hw>Sni"der</hw>, }</mhw> <pos>n.</pos> <fld>(Mil.)</fld> <def>A breech-loading rifle formerly used in the British service; -- so called from the inventor.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Sniff</ent><br/
<hw>Sniff</hw> <pr>(?)</pr>, <pos>v. i.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Sniffed</conjf> <pr>(?)</pr> or <conjf>Snift</conjf>; <pos>p. pr. & vb. n.</pos> <conjf>Sniffing</conjf>.]</vmorph> <ety>[OE. <ets>sneven</ets>; akin to <ets>snivel</ets>, <ets>snuff</ets>; cf. Dan. <ets>snive</ets> to sniff. See <er>Snuff</er>, <pos>v. t.</pos>]</ety> <def>To draw air audibly up the nose; to snuff; -- sometimes done as a gesture of suspicion, offense, or contempt.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>So ye grow squeamish, gods, and <qex>sniff</qex> at heaven.</q> <rj><qau>M. Arnold.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sniff</ent><br/
<hw>Sniff</hw>, <pos>v. t.</pos> <sn>1.</sn> <def>To draw in with the breath through the nose; <as>as, to <ex>sniff</ex> the air of the country</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To perceive as by sniffing; to snuff, to scent; to smell; <as>as, to <ex>sniff</ex> danger</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sniff</ent><br/
<hw>Sniff</hw>, <pos>n.</pos> <def>The act of sniffing; perception by sniffing; that which is taken by sniffing; <as>as, a <ex>sniff</ex> of air</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sniffing</ent><br/
<hw>Sniff"ing</hw>, <pos>n.</pos> <fld>(Physiol.)</fld> <def>A rapid inspiratory act, in which the mouth is kept shut and the air drawn in through the nose.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sniffle</ent><br/
<hw>Snif"fle</hw> <pr>(?)</pr>, <pos>v. i.</pos> <ety>[Freq. of <ets>sniff</ets>. See <er>Snivel</er>.]</ety> <def>To snuffle, as one does with a catarrh.</def> <mark>[Prov. Eng.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Snift</ent><br/
<hw>Snift</hw> <pr>(?)</pr>, <pos>v. i.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Snifted</conjf>; <pos>p. pr. & vb. n.</pos> <conjf>Snifting</conjf>.]</vmorph> <ety>[From <er>Sniff</er>.]</ety> <sn>1.</sn> <def>To snort.</def> <mark>[Obs.]</mark> <ldquo/Resentment expressed by <xex>snifting</xex>.<rdquo/  <rj><au>Johnson.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To sniff; to snuff; to smell.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>It now appears that they were still <qex>snifing</qex> and hankering after their old quarters.</q> <rj><qau>Landor.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Snift</ent><br/
<hw>Snift</hw>, <pos>n.</pos> <sn>1.</sn> <def>A moment.</def> <mark>[Prov. Eng.]</mark>  <rj><au>Halliwell.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Slight snow; sleet.</def> <mark>[Prov. Eng.]</mark>  <rj><au>Halliwell.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Snifting</ent><br/
<hw>Snift"ing</hw>, <def><pos>a. & n.</pos> from <er>Snift</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Snifting valve</b></col>, <cd>a small valve opening into the atmosphere from the cylinder or condenser of a steam engine, to allow the escape of air when the piston makes a stroke; -- so called from the noise made by its action.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Snig</ent><br/
<hw>Snig</hw> <pr>(?)</pr>, <pos>v. t.</pos> <ety>[See <er>Snick</er> a small cut.]</ety> <def>To chop off; to cut.</def> <mark>[Prov. Eng.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Snig</ent><br/
<hw>Snig</hw>, <pos>v. i.</pos> <ety>[See <er>Sneak</er>.]</ety> <def>To sneak.</def> <mark>[Prov. Eng.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Snigg</ent><br/
<ent>Snig</ent><br/
<mhw>{ <hw>Snig</hw>, <hw>Snigg</hw>, }</mhw> <pos>n.</pos> <ety>[Cf. <er>Sneak</er>.]</ety> <fld>(Zool.)</fld> <def>A small eel.</def> <mark>[Prov. Eng.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Snigger</ent><br/
<hw>Snig"ger</hw> <pr>(?)</pr>, <pos>v. i.</pos> <def>See <er>Snicker</er>.</def>  <rj><au>Thackeray.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Snigger</ent><br/
<hw>Snig"ger</hw>, <pos>n.</pos> <def>See <er>Snicker</er>.</def>  <rj><au>Dickens.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sniggle</ent><br/
<hw>Snig"gle</hw> <pr>(?)</pr>, <pos>v. i.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Sniggled</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Sniggling</conjf><pr>(?)</pr>.]</vmorph> <ety>[See <er>Snig</er> a kind of eel.]</ety> <def>To fish for eels by thrusting the baited hook into their holes or hiding places.</def>  <rj><au>Walton.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sniggle</ent><br/
<hw>Snig"gle</hw>, <pos>v. t.</pos> <def>To catch, as an eel, by sniggling; hence, to hook; to insnare.</def>  <rj><au>Beau. & Fl.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Snip</ent><br/
<hw>Snip</hw> <pr>(?)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Snipped</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Snipping</conjf>.]</vmorph> <ety>[D. <ets>snippen</ets>; akin to G. <ets>schnippen</ets>.]</ety> <def>To cut off the nip or neb of, or to cut off at once with shears or scissors; to clip off suddenly; to nip; hence, to break off; to snatch away.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Curbed and <qex>snipped</qex> in my younger years by fear of my parents from those vicious excrescences to which that age was subject.</q> <rj><qau>Fuller.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>The captain seldom ordered anything out of the ship's stores . . . but I <qex>snipped</qex> some of it for my own share.</q> <rj><qau>De Foe.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Snip</ent><br/
<hw>Snip</hw> <pr>(?)</pr>, <pos>n.</pos> <sn>1.</sn> <def>A single cut, as with shears or scissors; a clip.</def>  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A small shred; a bit cut off.</def>  <rj><au>Wiseman.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>A share; a snack.</def> <mark>[Obs.]</mark>  <rj><au>L'Estrange</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>A tailor.</def> <mark>[Slang]</mark>  <rj><au>Nares. C. Kingsley.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <def>Small hand shears for cutting sheet metal.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Snipe</ent><br/
<hw>Snipe</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[OE. <ets>snipe</ets>; akin to D. <ets>snep</ets>, <ets>snip</ets>, LG. <ets>sneppe</ets>, <ets>snippe</ets>, G. <ets>schnepfe</ets>, Icel. <ets>sn<imac/pa</ets> (in comp.), Dan. <ets>sneppe</ets>, Sw. <ets>sn<aum/ppa</ets> a sanpiper, and possibly to E. <ets>snap</ets>. See <er>Snap</er>, <er>Snaffle</er>.]</ety> <sn>1.</sn> <fld>(Zool.)</fld> <def>Any one of numerous species of limicoline game birds of the family <fam>Scolopacidae</fam>, having a long, slender, nearly straight beak.</def><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ The common, or whole, snipe (<spn>Gallinago c<oe/lestis</spn>) and the great, or double, snipe (<spn>Gallinago major</spn>), are the most important European species. The Wilson's snipe (<spn>Gallinago delicata</spn>) (sometimes erroneously called <stype>English snipe</stype>) and the gray snipe, or dowitcher (<spn>Macrohamphus griseus</spn>), are well-known American species.</note><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A fool; a blockhead.</def> <mark>[R.]</mark>  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Half snipe</b></col>, <cd>the dunlin; the jacksnipe.</cd> -- <col><b>Jack snipe</b></col>. <cd>See <er>Jacksnipe</er>.</cd> -- <col><b>Quail snipe</b></col>. <cd>See under <er>Quail</er>.</cd> -- <col><b>Robin snipe</b></col>, <cd>the knot.</cd> -- <col><b>Sea snipe</b></col>. <cd>See in the Vocabulary.</cd> -- <col><b>Shore snipe</b></col>, <cd>any sandpiper.</cd> -- <col><b>Snipe hawk</b></col>, <cd>the marsh harrier.</cd> <mark>[Prov. Eng.]</mark> -- <col><b>Stone snipe</b></col>, <cd>the tattler.</cd> -- <col><b>Summer snipe</b></col>, <cd>the dunlin; the green and the common European sandpipers.</cd> -- <col><b>Winter snipe</b></col>. <cd>See <cref>Rock snipe</cref>, under <er>Rock</er>.</cd> -- <col><b>Woodcock snipe</b></col>, <cd>the great snipe.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Snipe</ent><br/
<hw>Snipe</hw> <pr>(?)</pr>, <pos>v. i.</pos>  <vmorph>[<pos>imp. & p. p.</pos> <conjf>Sniped</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Sniping</conjf> <pr>(?)</pr>.]</vmorph> <sn>1.</sn> <def>To shoot or hunt snipe.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><sn>2.</sn>  <def>To shoot at detached men of an enemy's forces at long range, esp. when not in action; -- often with <ptcl>at</ptcl>.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><cs><col><b>snipe at</b></col>, <cd>to aim petty or snide criticisms at (a person) in his absence.</cd></cs><br/
[<source>PJC</source>]</p>

<p><ent>Snipe</ent><br/
<hw>Snipe</hw> <pr>(?)</pr>, <pos>v. t.</pos> <sn>1.</sn> <def>To shoot at (detached men of an enemy's force) at long range, esp. when not in action.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><sn>2.</sn>  <def>To nose (a log) to make it drag or slip easily in skidding.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Snipebill</ent><br/
<hw>Snipe"bill`</hw> <pr>(?)</pr>, <pos>n.</pos> <sn>1.</sn> <def>A plane for cutting deep grooves in moldings.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A bolt by which the body of a cart is fastened to the axle.</def> <mark>[Local, U.S.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Snipefish</ent><br/
<hw>Snipe"fish`</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Zool.)</fld> <sd>(a)</sd> <def>The bellows fish.</def> <sd>(b)</sd> <def>A long, slender deep-sea fish (<spn>Nemichthys scolopaceus</spn>) with a slender beak.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Snippack</ent><br/
<hw>Snip"pack</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Cf. <er>Snipe</er>.]</ety> <fld>(Zool.)</fld> <def>The common snipe.</def> <mark>[Prov. Eng.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Snipper</ent><br/
<hw>Snip"per</hw> <pr>(?)</pr>, <pos>n.</pos> <def>One who snips.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Snipper-snaper</ent><br/
<hw>Snip"per-snap`er</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A small, insignificant fellow.</def> <mark>[Colloq.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Snippet</ent><br/
<hw>Snip"pet</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A small part or piece.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>To be cut into <qex>snippets</qex> and shreds.</q> <rj><qau>F. Harrison.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Snippety</ent><br/
<hw>Snip"pet*y</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Ridiculously small; petty.</def> <ldquo/<xex>Snippety</xex> facts.<rdquo/  <rj><au>London Spectator.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><-- 2. short-tempered; snappish. --><br/
[<source>1913 Webster</source>]</p>

<p><-- Snippy, adj.  snippety. --><br/
[<source>1913 Webster</source>]</p>

<p><ent>Snip-snap</ent><br/
<hw>Snip"-snap`</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Reduplication of <ets>snap</ets>.]</ety> <def>A tart dialogue with quick replies.</def> <mark>[R.]</mark>  <rj><au>Pope.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Snip-snap</ent><br/
<hw>Snip"-snap`</hw>, <pos>a.</pos> <def>Quick; short; sharp; smart.</def>  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Snipy</ent><br/
<hw>Snip"y</hw> <pr>(sn<imac/p"<ycr/)</pr>, <pos>a.</pos> <def>Like a snipe.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Snite</ent><br/
<hw>Snite</hw> <pr>(sn<imac/t)</pr>, <pos>n.</pos> <def>A snipe.</def> <mark>[Obs. or Scot.]</mark>  <rj><au>Carew.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Snite</ent><br/
<hw>Snite</hw>, <pos>v. t.</pos> <ety>[Icel. <ets>sn<imac/fa</ets>. See <er>Snout</er>.]</ety> <def>To blow, as the nose; to snuff, as a candle.</def> <mark>[Obs. or Scot.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Snithy</ent><br/
<ent>Snithe</ent><br/
<mhw>{ <hw>Snithe</hw> <pr>(?)</pr>, <hw>Snith"y</hw> <pr>(?)</pr>, }</mhw> <pos>a.</pos> <ety>[AS. <ets>sn<imac/<edh/an</ets> to cut. See <er>Snathe</er>.]</ety> <def>Sharp; piercing; cutting; -- applied to the wind.</def> <mark>[Prov. Eng.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Snivel</ent><br/
<hw>Sniv"el</hw> <pr>(?)</pr>, <pos>v. i.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Sniveled</conjf> <pr>(?)</pr> or <conjf>Snivelled</conjf>; <pos>p. pr. & vb. n.</pos> <conjf>Sniveling</conjf> or <conjf>Snivelling</conjf>.]</vmorph> <ety>[OE. <ets>snivelen</ets>, <ets>snevelen</ets>, <ets>snuvelen</ets>, freg. of <ets>sneven</ets>. See <er>Sniff</er>, and cf. <er>Snuffle</er>.]</ety> <sn>1.</sn> <def>To run at the nose; to make a snuffling noise.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To cry or whine with snuffling, as children; to cry weakly or whiningly.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Put stop to thy <qex>sniveling</qex> ditty.</q> <rj><qau>Sir W. Scott.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Snivel</ent><br/
<hw>Sniv"el</hw>, <pos>n.</pos> <ety>[AS. <ets>snofel</ets>.  Cf. <er>Snivel</er>, <pos>v. i.</pos>]</ety> <def>Mucus from the nose; snot.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sniveler</ent><br/
<hw>Sniv"el*er</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Written also <ets>sniveller</ets>.]</ety> <def>One who snivels, esp. one who snivels habitually.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Snively</ent><br/
<hw>Sniv"el*y</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Running at the nose; sniveling pitiful; whining.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Snob</ent><br/
<hw>Snob</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Icel. <ets>sn<amac/pr</ets> a dolt, impostor, charlatan.  Cf. <er>Snub</er>.]</ety> <sn>1.</sn> <def>A vulgar person who affects to be better, richer, or more fashionable, than he really is; a vulgar upstart; one who apes his superiors.</def>  <rj><au>Thackeray.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Essentially vulgar, a <qex>snob</qex>. -- a gilded <qex>snob</qex>, but none the less a <qex>snob</qex>.</q> <rj><qau>R. G. White.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Eng. Univ.)</fld> <def>A townsman.</def> <mark>[Canf]</mark><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>A journeyman shoemaker.</def> <mark>[Prov. Eng.]</mark>  <rj><au>Halliwell.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>A workman who accepts lower than the usual wages, or who refuses to strike when his fellows do; a rat; a knobstick.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Those who work for lower wages during a strike are called <qex>snobs</qex>, the men who stand out being <ldquo/nobs<rdquo/</q> <rj><qau>De Quincey.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><-- 5. One who adopts an offensive air of superiority to those he considers as inferiors, esp. by avoiding or ignoring them. --><br/
[<source>1913 Webster</source>]</p>

<p><ent>Snobbery</ent><br/
<hw>Snob"ber*y</hw> <pr>(?)</pr>, <pos>n.</pos> <def>The quality of being snobbish; snobbishness.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Snobbish</ent><br/
<hw>Snob"bish</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Of or pertaining to a snob; characteristic of, or befitting, a snob; vulgarly pretentious.</def> -- <wordforms><wf>Snob"bish*ly</wf>, <pos>adv.</pos></wordforms><br/
[<source>1913 Webster</source>]</p>

<p><ent>Snobbishness</ent><br/
<hw>Snob"bish*ness</hw>, <pos>n.</pos> <def>Vulgar affectation or ostentation; mean admiration of mean things; conduct or manners of a snob.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Snobbism</ent><br/
<hw>Snob"bism</hw> <pr>(?)</pr>, <pos>n.</pos> <def>Snobbery.</def><br/
[<source>1913 Webster</source>]</p>

<p><-- p. 1363 --></p>

<p><ent>Snobby</ent><br/
<hw>Snob"by</hw> <pr>(sn<ocr/b"b<ycr/)</pr>, <pos>a.</pos> <def>Snobbish.</def> <mark>[R.]</mark>  <rj><au>E. B. Ramsay.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Snobling</ent><br/
<hw>Snob"ling</hw>, <pos>n.</pos> <def>A little snob.</def> <mark>[Jocose]</mark>  <rj><au>Thackeray.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Snobocracy</ent><br/
<hw>Snob*oc"ra*cy</hw> <pr>(sn<ocr/b*<ocr/k"r<adot/*s<ycr/)</pr>, <pos>n.</pos> <ety>[<ets>Snob</ets> + <ets>-cracy</ets>, as in <ets>aristocracy</ets>, <ets>mobocracy</ets>.]</ety> <def>Snobs, collectively.</def> <mark>[Hybrid & Recent]</mark>  <rj><au>C. Kingsley.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Snod</ent><br/
<hw>Snod</hw> <pr>(sn<ocr/d)</pr>, <pos>n.</pos> <ety>[See <er>Snood</er>.]</ety> <def>A fillet; a headband; a snood.</def> <mark>[Obs. or Prov. Eng.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Snod</ent><br/
<hw>Snod</hw>, <pos>a.</pos> <ety>[Scot. <ets>snod</ets> to prune, put in order.]</ety> <def>Trimmed; smooth; neat; trim; sly; cunning; demure.</def> <mark>[Prov. Eng. & Scot.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Snoff</ent><br/
<hw>Snoff</hw> <pr>(sn<ocr/f; 115)</pr>, <pos>n.</pos> <ety>[Cf. <er>Snuff</er>.]</ety> <fld>(Mining)</fld> <def>A short candle end used for igniting a fuse.</def>  <rj><au>Raymond.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Snood</ent><br/
<hw>Snood</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[AS. <ets>sn<omac/d</ets>.  Cf. <ets>Snare</ets>.]</ety> <sn>1.</sn> <def>The fillet which binds the hair of a young unmarried woman, and is emblematic of her maiden character.</def> <mark>[Scot.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>And seldom was a <qex>snood</qex> amid<br/
Such wild, luxuriant ringlets hid.</q> <rj><qau>Sir W. Scott.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A short line (often of horsehair) connecting a fishing line with the hook; a snell; a leader.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Snood</ent><br/
<hw>Snood</hw>, <pos>v. t.</pos> <def>To bind or braid up, as the hair, with a snood.</def> <mark>[Scot.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Snooded</ent><br/
<hw>Snood"ed</hw>, <pos>a.</pos> <def>Wearing or having a snood.</def> <ldquo/The <xex>snooded</xex> daughter.<rdquo/  <rj><au>Whittier.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Snook</ent><br/
<hw>Snook</hw> <pr>(sn<oomac/k)</pr>, <pos>v. i.</pos> <ety>[Prov. E. <ets>snook</ets> to search out, to follow by the scent; cf. Sw. <ets>snoka</ets> to lurk, LG. <ets>sn<oum/ggen</ets>, <ets>snuckern</ets>, <ets>sn<oum/kern</ets>, to snuffle, to smell about, to search for.]</ety> <def>To lurk; to lie in ambush.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Snook</ent><br/
<hw>Snook</hw>, <pos>n.</pos> <ety>[D. <ets>snoek</ets>.]</ety> <fld>(Zool.)</fld> <sd>(a)</sd> <def>A large perchlike marine food fish (<spn>Centropomus undecimalis</spn>) found both on the Atlantic and Pacific coasts of tropical America; -- called also <altname>ravallia</altname>, and <altname>robalo</altname>.</def> <sd>(b)</sd> <def>The cobia.</def> <sd>(c)</sd> <def>The garfish.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Snooze</ent><br/
<hw>Snooze</hw> <pr>(sn<oomac/z)</pr>, <pos>n.</pos> <ety>[Scot. <ets>snooze</ets> to sleep; cf. Dan. & Sw. <ets>snus</ets> snuff.]</ety> <def>A short sleep; a nap.</def> <mark>[Colloq.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Snooze</ent><br/
<hw>Snooze</hw>, <pos>v. i.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Snoozed</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Snoozing</conjf>.]</vmorph> <def>To doze; to drowse; to take a short nap; to slumber.</def> <mark>[Colloq.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Snore</ent><br/
<hw>Snore</hw> <pr>(sn<omac/r)</pr>, <pos>v. i.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Snored</conjf> <pr>(sn<omac/rd)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Snoring</conjf>.]</vmorph> <ety>[OE. <ets>snoren</ets>, AS. <ets>snora</ets> a snoring; akin to LG. <ets>snoren</ets>, <ets>snorken</ets>, <ets>snurken</ets>, to snore, D. <ets>snorken</ets>, G. <ets>schnarchen</ets> to snore, <ets>schnarren</ets> to rattle, MHG. <ets>snarren</ets>, Sw. <ets>snarka</ets> to snore, Icel. <ets>snarka</ets> to sputter, fizzle.  Cf. <er>Snarl</er> to growl, <er>Sneer</er>, <er>Snort</er>. See <er>Snoring</er>.]</ety> <def>To breathe with a rough, hoarse, nasal voice in sleep.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Snore</ent><br/
<hw>Snore</hw>, <pos>n.</pos> <def>A harsh nasal noise made in sleep.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Snorer</ent><br/
<hw>Snor"er</hw> <pr>(?)</pr>, <pos>n.</pos> <def>One who snores.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Snoring</ent><br/
<hw>Snor"ing</hw>, <pos>n.</pos> <fld>(Physiol.)</fld> <def>The act of respiring through the open mouth so that the currents of inspired and expired air cause a vibration of the uvula and soft palate, thus giving rise to a sound more or less harsh. It is usually unvoluntary, but may be produced voluntarily.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Snort</ent><br/
<hw>Snort</hw> <pr>(?)</pr>, <pos>v. i.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Snorted</conjf>; <pos>p. pr. & vb. n.</pos> <conjf>Snorting</conjf>.]</vmorph> <ety>[OE. <ets>snorten</ets>; akin to <ets>snoren</ets>. <ets>See</ets> <er>Snore</er>.]</ety> <sn>1.</sn> <def>To force the air with violence through the nose, so as to make a noise, as do high-spirited horsed in prancing and play.</def>  <rj><au>Fairfax.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To snore.</def> <mark>[R.]</mark> <ldquo/The <xex>snorting</xex> citizens.<rdquo/  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>To laugh out loudly.</def> <mark>[Colloq.]</mark>  <rj><au>Halliwell.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Snort</ent><br/
<hw>Snort</hw>, <pos>n.</pos> <def>The act of snorting; the sound produced in snorting.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Snort</ent><br/
<hw>Snort</hw>, <pos>v. t.</pos> <def>To expel throught the nostrils with a snort; to utter with a snort.</def>  <rj><au>Keats.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Snorter</ent><br/
<hw>Snort"er</hw> <pr>(?)</pr>, <pos>n.</pos> <sn>1.</sn> <def>One who snorts.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Zool.)</fld> <def>The wheather; -- so called from its cry.</def> <mark>[Prov. Eng.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Snot</ent><br/
<hw>Snot</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[AS. <ets>snot</ets>; akin to D. <ets>snot</ets>, LG. <ets>snotte</ets>, Dan. <ets>snot</ets>, and to E. <ets>snout</ets>. <ets>See</ets> <er>Snout</er>.]</ety> <sn>1.</sn> <def>Mucus secreted in, or discharged from, the nose.</def> <mark>[Low]</mark><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A mean, insignificant fellow.</def> <mark>[Low]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Snot</ent><br/
<hw>Snot</hw>, <pos>v. t.</pos> <def>To blow, wipe, or clear, as the nose.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Snotter</ent><br/
<hw>Snot"ter</hw> <pr>(?)</pr>, <pos>v. i.</pos> <ety>[From <er>Snot</er>.]</ety> <def>To snivel; to cry or whine.</def> <mark>[Prov. Eng.]</mark>  <rj><au>Grose.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Snotter</ent><br/
<hw>Snot"ter</hw>, <pos>n.</pos> <ety>[Etymol. uncertain.]</ety> <fld>(Naut.)</fld> <def>A rope going over a yardarm, used to bend a tripping line to, in sending down topgallant and royal yards in vessels of war; also, the short line supporting the heel of the sprit in a small boat.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Snottery</ent><br/
<hw>Snot"ter*y</hw> <pr>(?)</pr>, <pos>n.</pos> <def>Filth; abomination.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>To purge the <qex>snottery</qex> of our slimy time.</q> <rj><qau>Marston.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Snotty</ent><br/
<hw>Snot"ty</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Foul with snot; hence, mean; dirty.</def><br/
[<source>1913 Webster</source>]</p>

<p>-- <wordforms><wf>Snot"ti*ly</wf> <pr>(#)</pr>, <pos>adv.</pos> -- <wf>Snot"ti*ness</wf>, <pos>n.</pos></wordforms><br/
[<source>1913 Webster</source>]</p>

<p><ent>Snout</ent><br/
<hw>Snout</hw> <pr>(snout)</pr>, <pos>n.</pos> <ety>[OE. <ets>snoute</ets>, probably of Scand, or Low German origin; cf. LG. <ets>snute</ets>, D. <ets>snuit</ets>, G. <ets>schnauze</ets>, Sw. <ets>snut</ets>, <ets>snyte</ets>, Dan. <ets>snude</ets>, Icel. <ets>sn<?/ta</ets> to blow the nose; probably akin to E. <ets>snuff</ets>, v.t.  Cf. <er>Snite</er>, <er>Snot</er>, <er>Snuff</er>.]</ety> <sn>1.</sn> <def>The long, projecting nose of a beast, as of swine.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>The nose of a man; -- in contempt.</def>  <rj><au>Hudibras.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>The nozzle of a pipe, hose, etc.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <fld>(Zool.)</fld> <sd>(a)</sd> <def>The anterior prolongation of the head of a gastropod; -- called also <altname>rostrum</altname>.</def> <sd>(b)</sd> <def>The anterior prolongation of the head of weevils and allied beetles.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Snout beetle</b></col> <fld>(Zool.)</fld>, <cd>any one of many species of beetles having an elongated snout and belonging to the tribe Rhynchophora; a weevil.</cd> -- <col><b>Snout moth</b></col> <fld>(Zool.)</fld>, <cd>any pyralid moth. See <er>Pyralid</er>.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Snout</ent><br/
<hw>Snout</hw>, <pos>v. t.</pos> <def>To furnish with a nozzle or point.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Snouty</ent><br/
<hw>Snout"y</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Resembling a beast's snout.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The nose was ugly, long, and big,<br/
Broad and <qex>snouty</qex> like a pig.</q> <rj><qau>Otway.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Snow</ent><br/
<hw>Snow</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[LG. <ets>snaue</ets>, or D. <ets>snaauw</ets>, from LG. <ets>snau</ets> a snout, a beak.]</ety> <fld>(Naut.)</fld> <def>A square-rigged vessel, differing from a brig only in that she has a trysail mast close abaft the mainmast, on which a large trysail is hoisted.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Snow</ent><br/
<hw>Snow</hw>, <pos>n.</pos> <ety>[OE. <ets>snow</ets>, <ets>snaw</ets>, AS. <ets>sn<amac/w</ets>; akin to D. <ets>sneeuw</ets>, OS. & OHG. <ets>sn<emac/o</ets>, G. <ets>schnee</ets>, Icel. <ets>sn<ae/r</ets>, <ets>snj<omac/r</ets>, <ets>snaj<amac/r</ets>, Sw. <ets>sn<oum/</ets>, Dan. <ets>snee</ets>, Goth. <ets>snaiws</ets>, Lith. <ets>sn<eum/gas</ets>, Russ. <ets>snieg'</ets>, Ir. & Gael. <ets>sneachd</ets>, W. <ets>nyf</ets>, L. <ets>nix</ets>, <ets>nivis</ets>, Gr. acc. <grk>ni`fa</grk>, also AS. <ets>sn<imac/wan</ets> to snow, G. <ets>schneien</ets>, OHG. <ets>sn<imac/wan</ets>, Lith. <ets>snigti</ets>, L. <ets>ningit</ets> it snows, Gr. <grk>ni`fei</grk>, Zend <ets>snizh</ets> to snow; cf. Skr. <ets>snih</ets> to be wet or sticky. <root/172.]</ety> <sn>1.</sn> <def>Watery particles congealed into white or transparent crystals or flakes in the air, and falling to the earth, exhibiting a great variety of very beautiful and perfect forms.</def><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ <xex>Snow</xex> is often used to form compounds, most of which are of obvious meaning; as, <xex>snow</xex>-capped, <xex>snow</xex>-clad, <xex>snow</xex>-cold, <xex>snow</xex>-crowned, <xex>snow</xex>-crust, <xex>snow</xex>-fed, <xex>snow</xex>-haired, <xex>snow</xex>like, <xex>snow</xex>-mantled, <xex>snow</xex>-nodding, <xex>snow</xex>-wrought, and the like.</note><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Fig.: Something white like snow, as the white color (argent) in heraldry; something which falls in, or as in, flakes.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The field of <qex>snow</qex> with eagle of black therein.</q> <rj><qau>Chaucer.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Red snow</b></col>. <cd>See under <er>Red</er>.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Snow bunting</b></col>. <fld>(Zool.)</fld> <cd>See <er>Snowbird</er>, 1.</cd> -- <col><b>Snow cock</b></col> <fld>(Zool.)</fld>, <cd>the snow pheasant.</cd> -- <col><b>Snow flea</b></col> <fld>(Zool.)</fld>, <cd>a small black leaping poduran (<spn>Achorutes nivicola</spn>) often found in winter on the snow in vast numbers.</cd> -- <col><b>Snow flood</b></col>, <cd>a flood from melted snow.</cd> -- <col><b>Snow flower</b></col> <fld>(Bot.)</fld>, <cd>the fringe tree.</cd> -- <mcol><col><b>Snow fly</b></col>, <it>or</it> <col><b>Snow insect</b></col></mcol> <fld>(Zool.)</fld>, <cd>any one of several species of neuropterous insects of the genus <gen>Boreus</gen>.  The male has rudimentary wings; the female is wingless.  These insects sometimes appear creeping and leaping on the snow in great numbers.</cd> -- <col><b>Snow gnat</b></col> <fld>(Zool.)</fld>, <cd>any wingless dipterous insect of the genus <gen>Chionea</gen> found running on snow in winter.</cd> -- <col><b>Snow goose</b></col> <fld>(Zool.)</fld>, <cd>any one of several species of arctic geese of the genus <gen>Chen</gen>.  The common snow goose (<spn>Chen hyperborea</spn>), common in the Western United States in winter, is white, with the tips of the wings black and legs and bill red.  Called also <altname>white brant</altname>, <altname>wavey</altname>, and <altname>Texas goose</altname>.  The blue, or blue-winged, snow goose (<spn>Chen coerulescens</spn>) is varied with grayish brown and bluish gray, with the wing quills black and the head and upper part of the neck white.  Called also <altname>white head</altname>, <altname>white-headed goose</altname>, and <altname>bald brant</altname>.</cd> -- <col><b>Snow leopard</b></col> <fld>(Zool.)</fld>, <cd>the ounce.</cd> -- <col><b>Snow line</b></col>, <cd>lowest limit of perpetual snow. In the Alps this is at an altitude of 9,000 feet, in the Andes, at the equator, 16,000 feet.</cd> -- <col><b>Snow mouse</b></col> <fld>(Zool.)</fld>, <cd>a European vole (<spn>Arvicola nivalis</spn>) which inhabits the Alps and other high mountains.</cd> -- <col><b>Snow pheasant</b></col> <fld>(Zool.)</fld>, <cd>any one of several species of large, handsome gallinaceous birds of the genus <gen>Tetraogallus</gen>, native of the lofty mountains of Asia. The Himalayn snow pheasant (<spn>Tetraogallus Himalayensis</spn>) in the best-known species. Called also <altname>snow cock</altname>, and <altname>snow chukor</altname>.</cd> -- <col><b>Snow partridge</b></col>. <fld>(Zool.)</fld> <cd>See under <er>Partridge</er>.</cd> -- <col><b>Snow pigeon</b></col> <fld>(Zool.)</fld>, <cd>a pigeon (<spn>Columba leuconota</spn>) native of the Himalaya mountains. Its back, neck, and rump are white, the top of the head and the ear coverts are black.</cd> -- <col><b>Snow plant</b></col> <fld>(Bot.)</fld>, <cd>a fleshy parasitic herb (<spn>Sarcodes sanguinea</spn>) growing in the coniferous forests of California. It is all of a bright red color, and is fabled to grow from the snow, through which it sometimes shoots up.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Snow</ent><br/
<hw>Snow</hw> <pr>(?)</pr>, <pos>v. i.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Snowed</conjf> <pr>(<?/)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Snowing</conjf>.]</vmorph> <def>To fall in or as snow; -- chiefly used impersonally; <as>as, it <ex>snows</ex>; it <ex>snowed</ex> yesterday.</as></def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Snow</ent><br/
<hw>Snow</hw>, <pos>v. t.</pos> <def>To scatter like snow; to cover with, or as with, snow.</def>  <rj><au>Donne. Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Snowball</ent><br/
<hw>Snow"ball`</hw> <pr>(?)</pr>, <pos>n.</pos> <sn>1.</sn> <def>A round mass of snow pressed or roller together, or anything resembling such a mass.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Bot.)</fld> <def>The Guelder-rose.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <fld>(Bot.)</fld> <def>a shrub of the genus <gen>Viburnum</gen>, having large clusters of white flowers.</def><br/
[<source>PJC</source>]</p>

<p><cs><mcol><col><b>Snowball bush</b></col> <it>or</it> <col><b>Snowball tree</b></col></mcol> <fld>(Bot.)</fld>, <cd>the <er>Guelder-rose</er>.</cd> -- <col><b>a snowball's chance in hell</b></col>, <mark>[Colloq.]</mark> <cd>no chance; an infinitesimal chance.</cd></cs><br/
[<source>1913 Webster</source> <source>+PJC</source>]</p>

<p><ent>Snowball</ent><br/
<hw>Snow"ball`</hw>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Snowballed</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Snowballing</conjf>.]</vmorph> <def>To pelt with snowballs; to throw snowballs at.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Snowball</ent><br/
<hw>Snow"ball`</hw>, <pos>v. i.</pos> <sn>1.</sn> <def>To throw snowballs.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To increase in magnitude at an accelerating rate, achieving large proportions; -- by analogy with a snowball rolling down a steep hill, causing a large snow slide.</def><br/
[<source>PJC</source>]</p>

<p><ent>Snow banner</ent><br/
<hw>Snow banner</hw>. <def>A bannerlike stream of snow blown into the air from a mountain peak, often having a pinkish color and extending horizontally for several miles across the sky.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Snowberry</ent><br/
<hw>Snow"ber`ry</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Bot.)</fld> <def>A name of several shrubs with white berries; as, the <spn>Symphoricarpus racemosus</spn> of the Northern United States, and the <spn>Chiococca racemosa</spn> of Florida and tropical America.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Creeping snowberry</b></col>. <fld>(Bot.)</fld> <cd>See under <er>Creeping</er>.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Snowbird</ent><br/
<hw>Snow"bird</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Zool.)</fld> <sd>(a)</sd> <def>An arctic finch (<spn>Plectrophenax nivalis</spn> syn. <spn>Plectrophanes nivalis</spn>) common, in winter, both in <geog>Europe</geog> and the <country>United States</country>, and often appearing in large flocks during snowstorms.  It is partially white, but variously marked with chestnut and brown.  Called also <altname>snow bunting</altname>, <altname>snowflake</altname>, <altname>snowfleck</altname>, and <altname>snowflight</altname>.</def> <sd>(b)</sd> <def>Any finch of the genus <gen>Junco</gen> which appears in flocks in winter time, especially <spn>Junco hyemalis</spn> in the Eastern United States; -- called also <altname>blue snowbird</altname>. See <er>Junco</er>.</def> <sd>(c)</sd> <def>The fieldfare.</def> <mark>[Prov. Eng.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Snow-blind</ent><br/
<hw>Snow"-blind`</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Affected with blindness by the brilliancy of snow.</def> -- <wordforms><wf>Snow"-blind`ness</wf>, <pos>n.</pos></wordforms><br/
[<source>1913 Webster</source>]</p>

<p><ent>Snow-bound</ent><br/
<hw>Snow"-bound`</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Enveloped in, or confined by, snow.</def>  <rj><au>Whittier.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Snow-broth</ent><br/
<hw>Snow"-broth`</hw> <pr>(?)</pr>, <pos>n.</pos> <def>Snow and water mixed, or snow just melted; very cold liquor.</def>  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Snowcap</ent><br/
<hw>Snow"cap`</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Zool.)</fld> <def>A very small humming bird (<spn>Microchaera albocoronata</spn>) native of New Grenada.</def><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ The feathers of the top of the head are white and snining, the body blue black with a purple and bronzy luster. The name is applied also to <spn>Microchaera parvirostris</spn> of Central America, which is similar in color.</note><br/
[<source>1913 Webster</source>]</p>

<p><ent>Snow-capped</ent><br/
<hw>Snow"-capped`</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Having the top capped or covered with snow; <as>as, <ex>snow-capped</ex> mountains</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Snowdrift</ent><br/
<hw>Snow"drift`</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A bank of drifted snow.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Snowdrop</ent><br/
<hw>Snow"drop`</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Bot.)</fld> <def>A bulbous plant (<spn>Galanthus nivalis</spn>) bearing white flowers, which often appear while the snow is on the ground. It is cultivated in gardens for its beauty.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Snowdrop tree</b></col>. <cd>See <cref>Silver-bell tree</cref>, under <er>Silver</er>, <pos>a.</pos></cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Snowflake</ent><br/
<hw>Snow"flake`</hw> <pr>(?)</pr>, <pos>n.</pos> <sn>1.</sn> <def>A flake, or small filmy mass, of snow.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Zool.)</fld> <def>See <er>Snowbird</er>, 1.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <fld>(Bot.)</fld> <def>A name given to several bulbous plants of the genus <gen>Leucoium</gen> (<spn>Leucoium vernum</spn>, <spn>Leucoium aestivum</spn>, etc.) resembling the snowdrop, but having all the perianth leaves of equal size.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Snowfleck</ent><br/
<hw>Snow"fleck`</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Zool.)</fld> <def>See <er>Snowbird</er>, 1.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Snowl</ent><br/
<hw>Snowl</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Zool.)</fld> <def>The hooded merganser.</def> <mark>[Local, U.S.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Snowless</ent><br/
<hw>Snow"less</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Destitute of snow.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Snowplough</ent><br/
<ent>Snowplow</ent><br/
<mhw>{ <hw>Snow"plow`</hw>, <hw>Snow"plough`</hw>  }</mhw> <pr>(?)</pr>, <pos>n.</pos> <def>An implement operating like a plow, but on a larger scale, for clearing away the snow from roads, railways, etc.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Snowshed</ent><br/
<hw>Snow"shed</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A shelter to protect from snow, esp. a long roof over an exposed part of a railroad.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Snowshoe</ent><br/
<hw>Snow"shoe`</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A slight frame of wood three or four feet long and about one third as wide, with thongs or cords stretched across it, and having a support and holder for the foot; -- used by persons for walking on soft snow.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Snowshoeing</ent><br/
<hw>Snow"shoe`ing</hw>, <pos>n.</pos> <def>Traveling on snowshoes.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Snowshoer</ent><br/
<hw>Snow"sho`er</hw> <pr>(?)</pr>, <pos>n.</pos> <def>One who travels on snowshoes; an expert in using snowshoes.</def>  <rj><au>W. G. Beers.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Snowslip</ent><br/
<hw>Snow"slip`</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A large mass or avalanche of snow which slips down the side of a mountain, etc.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Snowstorm</ent><br/
<hw>Snow"storm`</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A storm with falling snow.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Snow-white</ent><br/
<hw>Snow"-white`</hw> <pr>(?)</pr>, <pos>a.</pos> <def>White as snow; very white.</def> <ldquo/<xex>Snow-white</xex> and rose-red<rdquo/  <rj><au>Chaucer.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Snowy</ent><br/
<hw>Snow"y</hw> <pr>(?)</pr>, <pos>a.</pos> <sn>1.</sn> <def>White like snow.</def> <ldquo/So shows a <xex>snowy</xex> dove trooping with crows.<rdquo/  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Abounding with snow; covered with snow.</def> <ldquo/The <xex>snowy</xex> top of cold Olympus.<rdquo/  <rj><au>Milton.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>Fig.: Pure; unblemished; unstained; spotless.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>There did he lose his <qex>snowy</qex> innocence.</q> <rj><qau>J. Hall (1646).</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Snowy heron</b></col> <fld>(Zool.)</fld>, <cd>a white heron, or egret (<spn>Ardea candidissima</spn>), found in the Southern United States, and southward to Chile; -- called also <altname>plume bird</altname>.</cd> -- <col><b>Snowy lemming</b></col> <fld>(Zool.)</fld>, <cd>the collared lemming (<spn>Cuniculus torquatus</spn>), which turns white in winter.</cd> -- <col><b>Snowy owl</b></col> <fld>(Zool.)</fld>, <cd>a large arctic owl (<spn>Nyctea Scandiaca</spn>, or <spn>Nyctea nivea</spn>) common all over the northern parts of the United States and Europe in winter time.  Its plumage is sometimes nearly pure white, but it is usually more or less marked with blackish spots.  Called also <altname>white owl</altname>.</cd> -- <col><b>Snowy plover</b></col> <fld>(Zool.)</fld>, <cd>a small plover (<spn>Aegialitis nivosa</spn>) of the western parts of the United States and Mexico. It is light gray above, with the under parts and portions of the head white.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Snub</ent><br/
<hw>Snub</hw> <pr>(?)</pr>, <pos>v. i.</pos> <ety>[Cf. D. <ets>snuiven</ets> to snort, to pant, G. <ets>schnauben</ets>, MHG. <ets>sn<umac/ben</ets>, Prov. G. <ets>schnupfen</ets>, to sob, and E. <ets>snuff</ets>, v.t.]</ety> <def>To sob with convulsions.</def> <mark>[Obs.]</mark>  <rj><au>Bailey.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Snub</ent><br/
<hw>Snub</hw>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Snubbed</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Snubbing</conjf>.]</vmorph> <ety>[Cf. Icel. <ets>ssnubba</ets> to snub, chide, Sw. <ets>snubba</ets>, Icel. <ets>snubb<omac/ttr</ets> snubbed, nipped, and E. <ets>snib</ets>.]</ety> <sn>1.</sn> <def>To clip or break off the end of; to check or stunt the growth of; to nop.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To check, stop, or rebuke, with a tart, sarcastic reply or remark; to reprimand; to check.</def>  <rj><au>J. Foster.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>To treat with contempt or neglect, as a forward or pretentious person; to slight designedly.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><mcol><col><b>To snub a cable</b></col> <it>or</it> <col><b>To snub a rope</b></col></mcol> <fld>(Naut.)</fld>, <cd>to check it suddenly in running out.</cd>  <rj><au>Totten.</au></rj></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Snub</ent><br/
<hw>Snub</hw>, <pos>n.</pos> <sn>1.</sn> <def>A knot; a protuberance; a song.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>[A club] with ragged <qex>snubs</qex> and knotty grain.</q> <rj><qau>Spenser.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A check or rebuke; an intended slight.</def>  <rj><qau>J. Foster.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Snub nose</b></col>, <cd>a short or flat nose.</cd> -- <mcol><col><b>Snub post</b></col>, <it>or</it> <col><b>Snubbing post</b></col></mcol> <fld>(Naut.)</fld>, <cd>a post on a dock or shore, around which a rope is thrown to check the motion of a vessel.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Snub-nosed</ent><br/
<hw>Snub"-nosed`</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Having a short, flat nose, slightly turned up; <as>as, the <ex>snub-nosed</ex> eel</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Snub-nosed cachalot</b></col> <fld>(Zool.)</fld>, <cd>the pygmy sperm whale.</cd></cs>
<-- snub-nosed revolver, a revolver with a very short barrel. --  --><br/
[<source>1913 Webster</source>]</p>

<p><ent>Snudge</ent><br/
<hw>Snudge</hw> <pr>(?)</pr>, <pos>v. i.</pos> <ety>[Cf. <er>Snug</er>.]</ety> <def>To lie snug or quiet.</def> <mark>[Obs.]</mark>  <rj><au>Herbert.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Snudge</ent><br/
<hw>Snudge</hw>, <pos>n.</pos> <def>A miser; a sneaking fellow.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Snuff</ent><br/
<hw>Snuff</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Cf. G. <ets>schnuppe</ets> candle snuff, <ets>schnuppen</ets> to snuff a candle (see <er>Snuff</er>, <pos>v. t.</pos>, to snuff a candle), or cf. <er>Snub</er>, <pos>v. t.</pos>]</ety> <def>The part of a candle wick charred by the flame, whether burning or not.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>If the burning <qex>snuff</qex> happens to get out of the snuffers, you have a chance that it may fall into a dish of soup.</q> <rj><qau>Swift.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Snuff</ent><br/
<hw>Snuff</hw>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Snuffed</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Snuffing</conjf>.]</vmorph> <ety>[OE. <ets>snuffen</ets>. See <er>Snuff</er> of a candle <er>Snuff</er> to sniff.]</ety> <def>To crop the snuff of, as a candle; to take off the end of the snuff of.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>To snuff out</b></col>, <cd>to extinguish by snuffing.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><-- p. 1364 --></p>

<p><ent>Snuff</ent><br/
<hw>Snuff</hw> <pr>(?)</pr>, <pos>v. t.</pos><ety>[Akin to D. <ets>snuffen</ets>, G. <ets>schnupfen</ets>, <ets>schnuppen</ets>, to snuff, <ets>schnupfen</ets> a cold in the head, <ets>schnuppen</ets> to snuff (air), also, to snuff (a candle).  Cf. <er>Sniff</er>, <er>Snout</er>, <er>Snub</er>, <pos>v. i.</pos>]</ety> <sn>1.</sn> <def>To draw in, or to inhale, forcibly through the nose; to sniff.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>He <qex>snuffs</qex> the wind, his heels the sand excite.</q> <rj><qau>Dryden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To perceive by the nose; to scent; to smell.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Snuff</ent><br/
<hw>Snuff</hw>, <pos>v. i.</pos> <sn>1.</sn> <def>To inhale air through the nose with violence or with noise, as do dogs and horses.</def>  <rj><au>Dryden.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To turn up the nose and inhale air, as an expression of contempt; hence, to take offense.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Do the enemies of the church rage and <qex>snuff</qex>?</q> <rj><qau>Bp. Hall.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Snuff</ent><br/
<hw>Snuff</hw>, <pos>n.</pos> <sn>1.</sn> <def>The act of snuffing; perception by snuffing; a sniff.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Pulverized tobacco, etc., prepared to be taken into the nose; also, the amount taken at once.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>Resentment, displeasure, or contempt, expressed by a snuffing of the nose.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Snuff dipping</b></col>. <cd>See <er>Dipping</er>, <pos>n.</pos>, 5.</cd> -- <col><b>Snuff taker</b></col>, <cd>one who uses snuff by inhaling it through the nose.</cd> -- <col><b>To take it in snuff</b></col>, <cd>to be angry or offended.</cd> <au>Shak.</au> -- <col><b>Up to snuff</b></col>, <cd>not likely to be imposed upon; knowing; acute.</cd> <mark>[Slang]</mark><-- also, competent, able to do [the task] --></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Snuffbox</ent><br/
<hw>Snuff"box`</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A small box for carrying snuff about the person.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Snuffer</ent><br/
<hw>Snuff"er</hw> <pr>(?)</pr>, <pos>n.</pos> <sn>1.</sn> <def>One who snuffs.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Zool.)</fld> <def>The common porpoise.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Snuffers</ent><br/
<hw>Snuff"ers</hw> <pr>(?)</pr>, <pos>n. pl.</pos> <def>An instrument for cropping and holding the snuff of a candle.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Snuffingly</ent><br/
<hw>Snuff"ing*ly</hw>, <pos>adv.</pos> <def>In a snuffing manner.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Snuffle</ent><br/
<hw>Snuf"fle</hw> <pr>(?)</pr>, <pos>v. i.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Snuffled</conjf><pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Snuffling</conjf> <pr>(?)</pr>.]</vmorph> <ety>[Freq. of <ets>snuff</ets>, v.i.; akin to LG. <ets>snuffeln</ets>, G. <ets>schn<uum/ffeln</ets>, D. <ets>snuffeln</ets>, Dan. <ets>sn<oum/vle</ets>.  Cf. <er>Sniffle</er>.]</ety> <def>To speak through the nose; to breathe through the nose when it is obstructed, so as to make a broken sound.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>One clad in purple<br/
Eats, and recites some lamentable rhyme . . . <br/
<qex>Snuffling</qex> at nose, and croaking in his throat.</q> <rj><qau>Dryden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Snuffle</ent><br/
<hw>Snuf"fle</hw>, <pos>n.</pos> <sn>1.</sn> <def>The act of snuffing; a sound made by the air passing through the nose when obstructed.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>This dread sovereign, Breath, in its passage, gave a snort or <qex>snuffle</qex>.</q> <rj><qau>Coleridge.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>An affected nasal twang; hence, cant; hypocrisy.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <pluf>pl.</pluf> <def>Obstruction of the nose by mucus; nasal catarrh of infants or children.</def> <mark>[Colloq.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Snuffler</ent><br/
<hw>Snuf"fler</hw> <pr>(?)</pr>, <pos>n.</pos> <def>One who snuffles; one who uses cant.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Snuffy</ent><br/
<hw>Snuff"y</hw> <pr>(?)</pr>, <pos>a.</pos> <sn>1.</sn> <def>Soiled with snuff.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Sulky; angry; vexed.</def> <mark>[Obs. or Scot.]</mark>  <rj><au>Jamieson.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Snug</ent><br/
<hw>Snug</hw> <pr>(?)</pr>, <pos>a.</pos> <amorph>[<pos>Compar.</pos> <adjf>Snugger</adjf> <pr>(?)</pr>; <pos>superl.</pos> <adjf>Snuggest</adjf> <pr>(?)</pr>.]</amorph> <ety>[Prov. E. <ets>snug</ets> tight, handsome; cf. Icel. <ets>sn<oum/ggr</ets> smooth, ODan. <ets>sn<oum/g</ets> neat, Sw. <ets>snugg</ets>.]</ety> <sn>1.</sn> <def>Close and warm; <as>as, an infant lies <ex>snug</ex></as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Close; concealed; not exposed to notice.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Lie <qex>snug</qex>, and hear what critics say.</q> <rj><qau>Swift.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>Compact, convenient, and comfortable; <as>as, a <ex>snug</ex> farm, house, or property</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Snug</ent><br/
<hw>Snug</hw>, <pos>n.</pos> <fld>(Mach.)</fld> <def>Same as <er>Lug</er>, <pos>n.</pos>, 3.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Snug</ent><br/
<hw>Snug</hw>, <pos>v. i.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Snugged</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Snugging</conjf><pr>(?)</pr>.]</vmorph> <def>To lie close; to snuggle; to snudge; -- often with <xex>up</xex>, or <xex>together</xex>; <as>as, a child <ex>snugs</ex> up to its mother</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Snug</ent><br/
<hw>Snug</hw>, <pos>v. t.</pos> <sn>1.</sn> <def>To place snugly.</def> <mark>[R.]</mark>  <rj><au>Goldsmith.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To rub, as twine or rope, so as to make it smooth and improve the finish.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Snuggery</ent><br/
<hw>Snug"ger*y</hw> <pr>(?)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Snuggeries</plw> <pr>(<?/)</pr>.</plu> <def>A snug, cozy place.</def> <mark>[Colloq.]</mark>  <rj><au>Dickens.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Snuggle</ent><br/
<hw>Snug"gle</hw> <pr>(?)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Snuggled</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Snuggling</conjf> <pr>(?)</pr>.]</vmorph> <ety>[Freq. of <ets>snug</ets>.]</ety> <def>To move one way and the other so as to get a close place; to lie close for comfort; to cuddle; to nestle.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Snugly</ent><br/
<hw>Snug"ly</hw>, <pos>adv.</pos> <def>In a snug manner; closely; safely.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Snugness</ent><br/
<hw>Snug"ness</hw>, <pos>n.</pos> <def>The quality or state of being snug.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sny</ent><br/
<hw>Sny</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Cf. Icel. <ets>sn<umac/a</ets> to turn.]</ety> <def>An upward bend in a piece of timber; the sheer of a vessel.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Snying</ent><br/
<hw>Sny"ing</hw>, <pos>n.</pos> <fld>(Naut.)</fld> <def>A curved plank, placed edgewise, to work in the bows of a vessel.</def>  <rj><au>R. H. Dana, Jr.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>So</ent><br/
<hw>So</hw> <pr>(?)</pr>, <pos>adv.</pos> <ety>[OE. <ets>so</ets>, <ets>sa</ets>, <ets>swa</ets>, AS. <ets>sw<amac/</ets>; akin to OFries, <ets>s<amac/</ets>, <ets>s<?/</ets>, D. <ets>zoo</ets>, OS. & OHG. <ets>s<?/</ets>, G. <ets>so</ets>, Icel. <ets>sv<amac/</ets>, <ets>sv<?/</ets>, <ets>svo</ets>, <ets>so</ets>, Sw. <ets>s<?/</ets>, Dan. <ets>saa</ets>, Goth. <ets>swa</ets> so, <ets>sw<?/</ets> as; cf. L. <ets>suus</ets> one's own, Skr. <ets>sva</ets> one's own, one's self. <root/192.  Cf. As, <er>Custom</er>, <er>Ethic</er>, <er>Idiom</er>, <er>Such</er>.]</ety> <sn>1.</sn> <def>In that manner or degree; <as>as, indicated (in any way), or as implied, or as supposed to be known</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Why is his chariot <qex>so</qex> long in coming?</q> <rj><qau>Judges v. 28.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>In like manner or degree; in the same way; thus; for like reason; whith equal reason; -- used correlatively, following <xex>as</xex>, to denote comparison or resemblance; sometimes, also, following <xex>inasmuch as</xex>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>As a war should be undertaken upon a just motive, <qex>so</qex> a prince ought to consider the condition he is in.</q> <rj><qau>Swift.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>In such manner; to such degree; -- used correlatively with <xex>as</xex> or <xex>that</xex> following; <as>as, he was <ex>so</ex> fortunate as to escape</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>I viewed in may mind, <qex>so</qex> far as I was able, the beginning and progress of a rising world.</q> <rj><qau>T. Burnet.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>He is very much in Sir Roger's esteem, <qex>so</qex> that he lives in the family rather as a relation than dependent.</q> <rj><qau>Addison.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>Very; in a high degree; that is, in such a degree as can not well be expressed; <as>as, he is <ex>so</ex> good; he planned <ex>so</ex> wisely</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <def>In the same manner; as has been stated or suggested; in this or that condition or state; under these circumstances; in this way; -- with reflex reference to something just asserted or implied; used also with the verb <xex>to be</xex>, as a predicate.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Use him [your tutor] with great respect yourself, and cause all your family to do <qex>so</qex> too.</q> <rj><qau>Locke.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>It concerns every man, with the greatest seriousness, to inquire into those matters, whether they be <qex>so</qex> or not.</q> <rj><qau>Tillotson.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>He is Sir Robert's son, and <qex>so</qex> art thou.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>6.</sn> <def>The case being such; therefore; on this account; for this reason; on these terms; -- used both as an adverb and a conjuction.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>God makes him in his own image an intellectual creature, and <qex>so</qex> capable of dominion.</q> <rj><qau>Locke.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Here, then, exchange we mutually forgiveness;<br/
So may the guilt of all my broken vows,<br/
My perjuries to thee, be all forgotten.</q> <rj><qau>Rowe.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>7.</sn> <def>It is well; let it be as it is, or let it come to pass; -- used to express assent.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>And when 't is writ, for my sake read it over,<br/
And if it please you, <qex>so</qex>; if not, why, <qex>so</qex>.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>There is Percy; if your father will do me any honor, <qex>so</qex>; if not, let him kill the next Percy himself.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>8.</sn> <def>Well; the fact being as stated; -- used as an expletive; <as>as, <ex>so</ex> the work is done, is it?</as></def><br/
[<source>1913 Webster</source>]</p>

<p><sn>9.</sn> <def>Is it thus? do you mean what you say? -- with an upward tone; <as>as, do you say he refuses? <ex>So</ex>?</as></def> <mark>[Colloq.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><sn>10.</sn> <def>About the number, time, or quantity specified; thereabouts; more or less; <as>as, I will spend a week or <ex>so</ex> in the country; I have read only a page or <ex>so</ex>.</as></def>
<-- s.b. collocation "or so" --><br/
[<source>1913 Webster</source>]</p>

<p><q>A week or <qex>so</qex> will probably reconcile us.</q> <rj><qau>Gay.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ See the Note under <er>Ill</er>, <pos>adv.</pos></note><br/
[<source>1913 Webster</source>]</p>

<p><cs><mcol><col><b>So</b></col> . . . <col><b>as</b></col></mcol>. <cd><xex>So</xex> is now commonly used as a demonstrative correlative of <xex>as</xex> when it is the puprpose to emphasize the equality or comparison suggested, esp. in negative assertions, and questions implying a negative answer. By Shakespeare and others <xex>so</xex> . . . <xex>as</xex> was much used where <xex>as</xex> . . . <xex>as</xex> is now common. See the Note under <er>As</er>, 1.</cd><br/
[<source>1913 Webster</source>]</p>

<p><q><qex>So</qex> do, <qex>as</qex> thou hast said.</q> <rj><qau>Gen. xviii. 5.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q><qex>As</qex> a flower of the field, <qex>so</qex> he flourisheth.</q> <rj><qau>Ps. ciii. 15.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Had woman been <qex>so</qex> strong <qex>as</qex> men.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>No country suffered <qex>so</qex> much <qex>as</qex> England.</q> <rj><qau>Macaulay.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p>-- <col><b>So far</b></col>, <cd>to that point or extent; in that particular.</cd> <ldquo/The song was moral, and <xex>so far</xex> was right.<rdquo/ <au>Cowper.</au> -- <col><b>So far forth</b></col>, <cd>as far; to such a degree.</cd> <au>Shak.</au> <au>Bacon.</au> -- <col><b>So forth</b></col>, <cd>further in the same or similar manner; more of the same or a similar kind. See <cref>And so forth</cref>, under <er>And</er>.</cd>  -- <col><b>So, so</b></col>, <cd>well, well.</cd> <ldquo/<xex>So, so</xex>, it works; now, mistress, sit you fast.<rdquo/ <au>Dryden.</au> <cd>Also, moderately or tolerably well; passably; as, he succeeded but <xex>so so</xex>.</cd>  <ldquo/His leg is but <xex>so so</xex>.<rdquo/ <au>Shak.</au> -- <col><b>So that</b></col>, <cd>to the end that; in order that; with the effect or result that.</cd> -- <col><b>So then</b></col>, <cd>thus then it is; therefore; the consequence is.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>So</ent><br/
<hw>So</hw> <pr>(?)</pr>, <pos>conj.</pos> <def>Provided that; on condition that; in case that; if.</def><-- R. or archaic --><br/
[<source>1913 Webster</source>]</p>

<p><q>Though all the winds of doctrine were let loose play upon the earth, <qex>so</qex> truth be in the field, we do injuriously, by licensing and prohibiting, to misdoubt her strength.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>So</ent><br/
<hw>So</hw>, <pos>interj.</pos> <def>Be as you are; stand still; stop; that will do; right as you are; -- a word used esp. to cows; also used by sailors.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Soak</ent><br/
<hw>Soak</hw> <pr>(?)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Soaked</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Soaking</conjf>.]</vmorph> <ety>[OE. <ets>soken</ets>, AS. <ets>socian</ets> to sioak, steep, fr. <ets>s<?/can</ets>, <ets>s<?/gan</ets>, to suck. See <er>Suck</er>.]</ety> <sn>1.</sn> <def>To cause or suffer to lie in a fluid till the substance has imbibed what it can contain; to macerate in water or other liquid; to steep, as for the purpose of softening or freshening; <as>as, to <ex>soak</ex> cloth; to <ex>soak</ex> bread; to <ex>soak</ex> salt meat, salt fish, or the like.</as></def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To drench; to wet thoroughly.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Their land shall be <qex>soaked</qex> with blood.</q> <rj><qau>Isa. xxiv. 7.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>To draw in by the pores, or through small passages; <as>as, a sponge <ex>soaks</ex> up water; the skin <ex>soaks</ex> in moisture.</as></def><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>To make (its way) by entering pores or interstices; -- often with <xex>through</xex>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The rivulet beneath <qex>soaked</qex> its way obscurely through wreaths of snow.</q> <rj><qau>Sir W. Scott.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <def>Fig.: To absorb; to drain.</def> <mark>[Obs.]</mark>  <rj><au>Sir H. Wotton.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Soak</ent><br/
<hw>Soak</hw>, <pos>v. i.</pos> <sn>1.</sn> <def>To lie steeping in water or other liquid; to become sturated; <as>as, let the cloth lie and <ex>soak</ex></as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To enter (into something) by pores or interstices; <as>as, water <ex>soaks</ex> into the earth or other porous matter</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>To drink intemperately or gluttonously.</def> <mark>[Slang]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Soakage</ent><br/
<hw>Soak"age</hw> <pr>(?)</pr>, <pos>n.</pos> <def>The act of soaking, or the state of being soaked; also, the quantity that enters or issues by soaking.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Soaker</ent><br/
<hw>Soak"er</hw> <pr>(?)</pr>, <pos>n.</pos> <sn>1.</sn> <def>One who, or that which, soaks.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A hard drinker.</def> <mark>[Slang]</mark>  <rj><au>South.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Soaking</ent><br/
<hw>Soak"ing</hw>, <pos>a.</pos> <def>Wetting thoroughly; drenching; <as>as, a <ex>soaking</ex> rain</as>.</def> -- <wordforms><wf>Soak"ing*ly</wf>, <pos>adv.</pos></wordforms><br/
[<source>1913 Webster</source>]</p>

<p><ent>Soaky</ent><br/
<hw>Soak"y</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Full of moisture; wet; soppy.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Soal</ent><br/
<hw>Soal</hw> <pr>(?)</pr>, <pos>n.</pos> <sn>1.</sn> <def>The sole of a shoe.</def> <mark>[Obs. or R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Zool.)</fld> <def>See <er>Sole</er>, the fish.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Soal</ent><br/
<hw>Soal</hw>, <pos>n.</pos> <ety>[AS. <ets>sol</ets> mire.  Cf. <er>Sully</er>.]</ety> <def>A dirty pond.</def> <mark>[Prov. Eng.]</mark>  <rj><au>Halliwell.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Soam</ent><br/
<hw>Soam</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A chain by which a leading horse draws a plow.</def>  <rj><au>Knight.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Soap</ent><br/
<hw>Soap</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[OE. <ets>sope</ets>, AS. <ets>s<amac/pe</ets>; akin to D. <ets>zeep</ets>, G. <ets>seife</ets>, OHG. <ets>seifa</ets>, Icel. <ets>s<amac/pa</ets>, Sw. <ets>s<?/pa</ets>, Dan. <ets>s<?/be</ets>, and perhaps to AS. <ets>s<imac/pan</ets> to drip, MHG. <ets>s<imac/fen</ets>, and L. <ets>sebum</ets> tallow.  Cf. <er>Saponaceous</er>.]</ety> <def>A substance which dissolves in water, thus forming a lather, and is used as a cleansing agent. Soap is produced by combining fats or oils with alkalies or alkaline earths, usually by boiling, and consists of salts of sodium, potassium, etc., with the fatty acids (oleic, stearic, palmitic, etc.). See the Note below, and cf. <er>Saponification</er>. By extension, any compound of similar composition or properties, whether used as a cleaning agent or not.</def><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ In general, soaps are of two classes, <xex>hard</xex> and <xex>soft</xex>. Calcium, magnesium, lead, etc., form soaps, but they are insoluble and useless.</note><br/
[<source>1913 Webster</source>]</p>

<p><q>The purifying action of <qex>soap</qex> depends upon the fact that it is decomposed by a large quantity of water into free alkali and an insoluble acid salt. The first of these takes away the fatty dirt on washing, and the latter forms the <qex>soap</qex> lather which envelops the greasy matter and thus tends to remove it.</q> <rj><qau>Roscoe & Schorlemmer.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Castile soap</b></col>, <cd>a fine-grained hard soap, white or mottled, made of olive oil and soda; -- called also <altname>Marseilles soap</altname> <it>or</it> <altname>Venetian soap</altname>.</cd> -- <col><b>Hard soap</b></col>, <cd>any one of a great variety of soaps, of different ingredients and color, which are hard and compact.  All solid soaps are of this class.</cd> -- <col><b>Lead soap</b></col>, <cd>an insoluble, white, pliable soap made by saponifying an oil (olive oil) with lead oxide; -- used externally in medicine.  Called also <altname>lead plaster</altname>, <altname>diachylon</altname>, etc.</cd> -- <col><b>Marine soap</b></col>. <cd>See under <er>Marine</er>.</cd> -- <col><b>Pills of soap</b></col> <fld>(Med.)</fld>, <cd>pills containing soap and opium.</cd> -- <col><b>Potash soap</b></col>, <cd>any soap made with potash, esp. the soft soaps, and a hard soap made from potash and castor oil.</cd> -- <col><b>Pumice soap</b></col>, <cd>any hard soap charged with a gritty powder, as silica, alumina, powdered pumice, etc., which assists mechanically in the removal of dirt.</cd> -- <col><b>Resin soap</b></col>, <cd>a yellow soap containing resin, -- used in bleaching.</cd> -- <col><b>Silicated soap</b></col>, <cd>a cheap soap containing water glass (sodium silicate).</cd> -- <col><b>Soap bark</b></col>. <fld>(Bot.)</fld> <cd>See <er>Quillaia bark</er>.</cd> -- <col><b>Soap bubble</b></col>, <cd>a hollow iridescent globe, formed by blowing a film of soap suds from a pipe; figuratively, something attractive, but extremely unsubstantial.</cd><br/
[<source>1913 Webster</source>]</p>

<p><q>This <qex>soap bubble</qex> of the metaphysicians.</q> <rj><qau>J. C. Shairp.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p>-- <col><b>Soap cerate</b></col>, <cd>a cerate formed of soap, olive oil, white wax, and the subacetate of lead, sometimes used as an application to allay inflammation.</cd> -- <col><b>Soap fat</b></col>, <cd>the refuse fat of kitchens, slaughter houses, etc., used in making soap.</cd> -- <col><b>Soap liniment</b></col> <fld>(Med.)</fld>, <cd>a liniment containing soap, camphor, and alcohol.</cd> -- <col><b>Soap nut</b></col>, <cd>the hard kernel or seed of the fruit of the soapberry tree, -- used for making beads, buttons, etc.</cd> -- <col><b>Soap plant</b></col> <fld>(Bot.)</fld>, <cd>one of several plants used in the place of soap, as the <spn>Chlorogalum pomeridianum</spn>, a California plant, the bulb of which, when stripped of its husk and rubbed on wet clothes, makes a thick lather, and smells not unlike new brown soap. It is called also <altname>soap apple</altname>, <altname>soap bulb</altname>, and <altname>soap weed</altname>.</cd> -- <col><b>Soap tree</b></col>. <fld>(Bot.)</fld> <cd>Same as <er>Soapberry tree</er>.</cd> -- <col><b>Soda soap</b></col>, <cd>a soap containing a sodium salt. The soda soaps are all hard soaps.</cd> -- <col><b>Soft soap</b></col>, <cd>a soap of a gray or brownish yellow color, and of a slimy, jellylike consistence, made from potash or the lye from wood ashes. It is strongly alkaline and often contains glycerin, and is used in scouring wood, in cleansing linen, in dyehouses, etc. Figuratively, flattery; wheedling; blarney.</cd> <mark>[Colloq.]</mark> -- <col><b>Toilet soap</b></col>, <cd>hard soap for the toilet, usually colored and perfumed.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Soap</ent><br/
<hw>Soap</hw> <pr>(?)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Soaped</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Soaping</conjf>.]</vmorph> <sn>1.</sn> <def>To rub or wash over with soap.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To flatter; to wheedle.</def> <mark>[Slang]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Soapberry tree</ent><br/
<hw>Soap"ber`ry tree`</hw> <pr>(?)</pr>. <fld>(Bot.)</fld> <def>Any tree of the genus <gen>Sapindus</gen>, esp. <spn>Sapindus saponaria</spn>, the fleshy part of whose fruit is used instead of soap in washing linen; -- also called <altname>soap tree</altname>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Soapfish</ent><br/
<hw>Soap"fish`</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Zool.)</fld> <def>Any serranoid fish of the genus <gen>Rhypticus</gen>; -- so called from the soapy feeling of its skin.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Soapiness</ent><br/
<hw>Soap"i*ness</hw> <pr>(?)</pr>, <pos>n.</pos> <def>Quality or state of being soapy.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Soaproot</ent><br/
<hw>Soap"root`</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Bot.)</fld> <def>A perennial herb (<spn>Gypsophila Struthium</spn>) the root of which is used in Spain as a substitute for soap.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Soapstone</ent><br/
<hw>Soap"stone`</hw> <pr>(?)</pr>, <pos>n.</pos> <def>See <er>Steatite</er>, and <er>Talc</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Soapsuds</ent><br/
<hw>Soap"suds`</hw> <pr>(?)</pr>, <pos>n. pl.</pos> <def>Suds made with soap.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Soapwort</ent><br/
<hw>Soap"wort`</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Bot.)</fld> <def>A common plant (<spn>Saponaria officinalis</spn>) of the Pink family; -- so called because its bruised leaves, when agitated in water, produce a lather like that from soap. Called also <altname>Bouncing Bet</altname>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Soapy</ent><br/
<hw>Soap"y</hw> <pr>(?)</pr>, <pos>a.</pos> <amorph>[<pos>Compar.</pos> <adjf>Soapier</adjf> <pr>(?)</pr>; <pos>superl.</pos> <adjf>Soapiest</adjf>.]</amorph> <sn>1.</sn> <def>Resembling soap; having the qualities of, or feeling like, soap; soft and smooth.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Smeared with soap; covered with soap.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Soar</ent><br/
<hw>Soar</hw> <pr>(?)</pr>, <pos>v. i.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Soared</conjf> <pr>(#)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Soaring</conjf>.]</vmorph> <ety>[F. <ets>s'essorer</ets> to soar, <ets>essorer</ets> to dry (by exposing to the air), fr. L. <ets>ex</ets> out + <ets>aura</ets> the air, a breeze; akin to Gr. <?/<?/<?/<?/<?/.]</ety> <sn>1.</sn> <def>To fly aloft, as a bird; to mount upward on wings, or as on wings.</def>  <rj><au>Chaucer.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>When <qex>soars</qex> Gaul's vulture with his wings unfurled.</q> <rj><qau>Byron.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Fig.: To rise in thought, spirits, or imagination; to be exalted in mood.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Where the deep transported mind may <qex>soar</qex>.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Valor <qex>soars</qex> above<br/
What the world calls misfortune.</q> <rj><qau>Addison.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <fld>(Aeronautics)</fld> <def>To fly by wind power; to  glide indefinitely without loss of altitude.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Soar</ent><br/
<hw>Soar</hw>, <pos>n.</pos> <def>The act of soaring; upward flight.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>This apparent <qex>soar</qex> of the hooded falcon.</q> <rj><qau>Coleridge.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Soar</ent><br/
<hw>Soar</hw>, <pos>a.</pos> <def>See 3d <er>Sore</er>.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Soar</ent><br/
<hw>Soar</hw>, <pos>a.</pos> <def>See <er>Sore</er>, reddish brown.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Soar falcon</b></col>. <fld>(Zool.)</fld> <cd>See <cref>Sore falcon</cref>, under <er>Sore</er>.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Soaring</ent><br/
<hw>Soar"ing</hw>, <def><pos>a. & n.</pos> from <er>Soar</er>.</def> -- <wordforms><wf>Soar"ing*ly</wf>, <pos>adv.</pos></wordforms><br/
[<source>1913 Webster</source>]</p>

<p><ent>Soave</ent><br/
||<hw>So*a"ve</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[It.]</ety> <fld>(Mus.)</fld> <def>Sweet.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Soavemente</ent><br/
||<hw>So*a`ve*men"te</hw> <pr>(?)</pr>, <pos>adv.</pos> <ety>[It.]</ety> <fld>(Mus.)</fld> <def>Sweetly.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sob</ent><br/
<hw>Sob</hw> <pr>(s<ocr/b)</pr>, <pos>v. t.</pos> <ety>[See <er>Sop</er>.]</ety> <def>To soak.</def> <mark>[Obs.]</mark>  <rj><au>Mortimer.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sob</ent><br/
<hw>Sob</hw>, <pos>v. i.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Sobbed</conjf> <pr>(s<ocr/bd)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Sobbing</conjf>.]</vmorph> <ety>[OE. <ets>sobben</ets>; akin to AS. <ets>se<oacute/fian</ets>, <ets>si<oacute/fian</ets>, to complain, bewail, <ets>se<oacute/fung</ets>, <ets>si<oacute/fung</ets>, sobbing, lamentation; cf. OHG. <ets>s<umac/ft<omac/n</ets>, <ets>s<umac/ftj<omac/n</ets>, to sigh, MHG. <ets>siuften</ets>, <ets>siufzen</ets>, G. <ets>seufzen</ets>, MHG. <ets>s<umac/ft</ets> a sigh, properly, a drawing in of breath, from <ets>s<umac/fen</ets> to drink, OHG. <ets>s<umac/fan</ets>.  Cf. <er>Sup</er>.]</ety> <def>To sigh with a sudden heaving of the breast, or with a kind of convulsive motion; to sigh with tears, and with a convulsive drawing in of the breath.</def><br/
[<source>1913 Webster</source>]</p>

<p><q><qex>Sobbing</qex> is the same thing [as sighing], stronger.</q> <rj><qau>Bacon.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>She sighed, she <qex>sobbed</qex>, and, furious with despair.<br/
She rent her garments, and she tore her hair.</q> <rj><qau>Dryden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sob</ent><br/
<hw>Sob</hw>, <pos>n.</pos> <sn>1.</sn> <def>The act of sobbing; a convulsive sigh, or inspiration of the breath, as in sorrow.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Break, heart, or choke with <qex>sobs</qex> my hated breath.</q> <rj><qau>Dryden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Any sorrowful cry or sound.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The tremulous <qex>sob</qex> of the complaining owl.</q> <rj><qau>Wordsworth.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sobbing</ent><br/
<hw>Sob"bing</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A series of short, convulsive inspirations, the glottis being suddenly closed so that little or no air enters into the lungs.</def>
<-- a less intense form of crying --><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sober</ent><br/
<hw>Sober</hw> <pr>(?)</pr>, <pos>a.</pos> <amorph>[<pos>Compar.</pos> <adjf>Soberer</adjf> <pr>(?)</pr>; <pos>superl.</pos> <adjf>Soberest</adjf>.]</amorph> <ety>[OE. <ets>sobre</ets>, F. <ets>sobre</ets>, from L. <ets>sobrius</ets>, probably from a prefix <ets>so-</ets> expressing separation + <ets>ebrius</ets> drunken.  Cf. <er>Ebriety</er>.]</ety> <sn>1.</sn> <def>Temperate in the use of spirituous liquors; habitually temperate; <as>as, a <ex>sober</ex> man</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>That we may hereafter live a godly, righteous, and <qex>sober</qex> life, to the glory of Thy holy name.</q> <rj><qau>Bk. of Com. Prayer.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Not intoxicated or excited by spirituous liquors; <as>as, the sot may at times be <ex>sober</ex></as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>Not mad or insane; not wild, visionary, or heated with passion; exercising cool, dispassionate reason; self-controlled; self-possessed.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>There was not a <qex>sober</qex> person to be had; all was tempestuous and blustering.</q>  <rj><au>Druden.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>No <qex>sober</qex> man would put himself into danger for the applause of escaping without breaking his neck.</q> <rj><qau>Dryden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><-- p. 1365 --></p>

<p><sn>4.</sn> <def>Not proceeding from, or attended with, passion; calm; <as>as, <ex>sober</ex> judgment; a man in his <ex>sober</ex> senses.</as></def><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <def>Serious or subdued in demeanor, habit, appearance, or color; solemn; grave; sedate.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>What parts gay France from <qex>sober</qex> Spain?</q> <rj><qau>Prior.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>See her <qex>sober</qex> over a sampler, or gay over a jointed baby.</q> <rj><qau>Pope.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Twilight gray<br/
Had in her <qex>sober</qex> livery all things clad.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><syn><b>Syn.</b> -- Grave; temperate; abstinent; abstemious; moderate; regular; steady; calm; quiet; cool; collected; dispassionate; unimpassioned; sedate; staid; serious; solemn; somber. See <er>Grave</er>.</syn><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sober</ent><br/
<hw>So"ber</hw> <pr>(?)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Sobered</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Sobering</conjf>.]</vmorph> <def>To make sober.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>There shallow draughts intoxicate the brain,<br/
And drinking largely <qex>sobers</qex> us again.</q> <rj><qau>Pope.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sober</ent><br/
<hw>So"ber</hw>, <pos>v. i.</pos> <def>To become sober; -- often with <xex>down</xex>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Vance gradually <qex>sobered</qex> down.</q> <rj><qau>Ld. Lytton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Soberize</ent><br/
<hw>So"ber*ize</hw> <pr>(?)</pr>, <pos>v. t. & i.</pos> <def>To sober.</def> <mark>[R.]</mark>  <rj><au>Crabbe.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Soberly</ent><br/
<hw>So"ber*ly</hw>, <pos>adv.</pos> <def>In a sober manner; temperately; cooly; calmly; gravely; seriously.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Soberly</ent><br/
<hw>So"ber*ly</hw>, <pos>a.</pos> <def>Grave; serious; solemn; sad.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>[He] looked hollow and thereto <qex>soberly</qex>.</q> <rj><qau>Chaucer.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sober-minded</ent><br/
<hw>So"ber-mind`ed</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Having a disposition or temper habitually sober.</def> -- <wordforms><wf>So"ber-mind`ed*ness</wf>, <pos>n.</pos></wordforms><br/
[<source>1913 Webster</source>]</p>

<p><ent>Soberness</ent><br/
<hw>So"ber*ness</hw>, <pos>n.</pos> <def>The quality or state of being sober.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Soboles</ent><br/
<hw>Sob"o*les</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L., a short.]</ety> <fld>(Bot.)</fld> <sd>(a)</sd> <def>A shoot running along under ground, forming new plants at short distances.</def> <sd>(b)</sd> <def>A sucker, as of tree or shrub.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Soboliferous</ent><br/
<hw>Sob`o*lif"er*ous</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>soboles + -ferous</ets>.]</ety> <fld>(Bot.)</fld> <def>Producing soboles. See <xex>Illust.</xex> of <er>Houseleek</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sobranje</ent><br/
||<hw>So*bran"je</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Bulgarian, lit., assembly.]</ety> <def>The unicameral national assembly of Bulgaria, elected for a term of five years by universal suffrage of adult males.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Sobriety</ent><br/
<hw>So*bri"e*ty</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>sobrietas</ets>: cf. F. <ets>sobri<eacute/t<eacute/</ets>. See <er>Sober</er>.]</ety> <sn>1.</sn> <def>Habitual soberness or temperance as to the use of spirituous liquors; <as>as, a man of <ex>sobriety</ex></as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Public <qex>sobriety</qex> is a relative duty.</q> <rj><qau>Blackstone.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Habitual freedom from enthusiasm, inordinate passion, or overheated imagination; calmness; coolness; gravity; seriousness; <as>as, the <ex>sobriety</ex> of riper years</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Mirth makes them not mad,<br/
Nor <qex>sobriety</qex> sad.</q> <rj><qau>Denham.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><syn><b>Syn.</b> -- Soberness; temperance; abstinence; abstemiousness; moderation; regularity; steadness; calmness; coolness; sober-mindeness; sedateness; staidness; gravity; seriousness; solemnity.</syn><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sobriquet</ent><br/
||<hw>So`bri`quet"</hw> <pr>(s<osl/`br<esl/`k<asl/")</pr>, <pos>n.</pos><ety>[F. <ets>sobriquet</ets>, OF. <ets>soubzbriquet</ets>, <ets>soubriquet</ets>, a chuck under the chin, hence, an affront, a nickname; of uncertain origin; cf. It. <ets>sottobecco</ets> a chuck under the chin.]</ety> <def>An assumed name; a fanciful epithet or appellation; a nickname.</def> <altsp>[Sometimes less correctly written <asp>soubriquet</asp>.]</altsp><br/
[<source>1913 Webster</source>]</p>

<p><ent>Soc</ent><br/
<hw>Soc</hw> <pr>(s<ocr/k)</pr>, <pos>n.</pos> <ety>[AS. <ets>s<omac/c</ets> the power of holding court, sway, domain, properly, the right of investigating or seeking; akin to E. <ets>sake</ets>, <ets>seek</ets>. <er>Sake</er>, <er>Seek</er>, and cf. <er>Sac</er>, and <er>Soke</er>.]</ety> <altsp>[Written also <asp>sock</asp>, and <asp>soke</asp>.]</altsp> <sn>1.</sn> <fld>(O. Eng. Law)</fld> <sd>(a)</sd> <def>The lord's power or privilege of holding a court in a district, as in manor or lordship; jurisdiction of causes, and the limits of that jurisdiction.</def> <sd>(b)</sd> <def>Liberty or privilege of tenants excused from customary burdens.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>An exclusive privilege formerly claimed by millers of grinding all the corn used within the manor or township which the mill stands.</def> <mark>[Eng.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Soc and sac</b></col> <fld>(O. Eng. Law)</fld>, <cd>the full right of administering justice in a manor or lordship.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Socage</ent><br/
<hw>Soc"age</hw> <pr>(?)</pr>, <pos>n.</pos><ety>[From <er>Soc</er>; cf. LL. <ets>socagium</ets>.]</ety> <fld>(O.Eng. Law)</fld> <def>A tenure of lands and tenements by a certain or determinate service; a tenure distinct from chivalry or knight's service, in which the obligations were uncertain. The service must be certain, in order to be denominated <ex>socage</ex>, as to hold by fealty and twenty shillings rent.</def> <altsp>[Written also <asp>soccage</asp>.]</altsp><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ <ex>Socage</ex> is of two kinds; <stype>free socage</stype>, where the services are not only certain, but honorable; and <stype>villein socage</stype>, where the services, though certain, are of a baser nature.</note>  <rj><au>Blackstone.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Socager</ent><br/
<hw>Soc"a*ger</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(O. Eng. Law)</fld> <def>A tennant by socage; a socman.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>So-called</ent><br/
<hw>So"-called`</hw> <pr>(?)</pr>, <pos>a.</pos> <def>So named; called by such a name (but perhaps called thus with doubtful propriety).</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sociability</ent><br/
<hw>So`cia*bil"i*ty</hw> <pr>(?)</pr>, <pos>n.</pos><ety>[Cf. F. <ets>sociabilit<eacute/</ets>.]</ety> <def>The quality of being sociable; sociableness.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sociable</ent><br/
<hw>So"cia*ble</hw> <pr>(?)</pr>, <pos>a.</pos><ety>[F., fr. L. <ets>sociabilis</ets>, fr. <ets>sociare</ets> to associate, fr. <ets>socius</ets> a companion. See <er>Social</er>.]</ety> <sn>1.</sn> <def>Capable of being, or fit to be, united in one body or company; associable.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>They are <qex>sociable</qex> parts united into one body.</q> <rj><qau>Hooker.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Inclined to, or adapted for, society; ready to unite with others; fond of companions; social.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Society is no comfort to one not <qex>sociable</qex>.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>What can be more uneasy to this <qex>sociable</qex> creature than the dry, pensive retirements of solitude?</q> <rj><qau>South.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>Ready to converse; inclined to talk with others; not taciturn or reserved.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>Affording opportunites for conversation; characterized by much conversation; <as>as, a <ex>sociable</ex> party</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <def>No longer hostile; friendly.</def> <mark>[Obs.]</mark>  <rj><au>Beau. & Fl.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><mcol><col><b>Sociable bird</b></col>, <it>or</it>  <col><b>Sociable weaver</b></col></mcol> <fld>(Zool.)</fld>, <cd>a weaver bird which builds composite nests. See <er>Republican</er>, <pos>n.</pos>, 3. <sd>(b)</sd>.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><syn><b>Syn.</b> -- Social; companionable; conversible; friendly; familiar; communicative; accessible.</syn><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sociable</ent><br/
<hw>So"cia*ble</hw>, <pos>n.</pos> <sn>1.</sn> <def>A gathering of people for social purposes; an informal party or reception; <as>as, a church <ex>sociable</ex></as>.</def> <mark>[Colloq. U. S.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A carriage having two double seats facing each other, and a box for the driver.</def>  <rj><au>Miss Edgeworth.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sociableness</ent><br/
<hw>So"cia*ble*ness</hw>, <pos>n.</pos> <def>The quality of being sociable.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sociably</ent><br/
<hw>So"cia*bly</hw>, <pos>adv.</pos> <def>In a sociable manner.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Social</ent><br/
<hw>So"cial</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>socialis</ets>, from <ets>socius</ets> a companion; akin to <ets>sequi</ets> to follow: cf. F. <ets>social</ets>. See <er>Sue</er> to follow.]</ety> <sn>1.</sn> <def>Of or pertaining to society; relating to men living in society, or to the public as an aggregate body; <as>as, <ex>social</ex> interest or concerns; <ex>social</ex> pleasure; <ex>social</ex> benefits; <ex>social</ex> happiness; <ex>social</ex> duties.</as></def> <ldquo/<xex>Social</xex> phenomena.<rdquo/  <rj><au>J. S. Mill.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Ready or disposed to mix in friendly converse; companionable; sociable; <as>as, a <ex>social</ex> person</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>Consisting in union or mutual intercourse.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Best with thyself accompanied, seek'st not<br/
<qex>Social</qex> communication.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <fld>(Bot.)</fld> <def>Naturally growing in groups or masses; -- said of many individual plants of the same species.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <fld>(Zool.)</fld> <sd>(a)</sd> <def>Living in communities consisting of males, females, and neuters, as do ants and most bees.</def> <sd>(b)</sd> <def>Forming compound groups or colonies by budding from basal processes or stolons; <as>as, the <ex>social</ex> ascidians</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Social science</b></col>, <cd>the science of all that relates to the social condition, the relations and institutions which are involved in man's existence and his well-being as a member of an organized community; sociology. It concerns itself with questions of the public health, education, labor, punishment of crime, reformation of criminals, and the like.</cd> -- <col><b>Social whale</b></col> <fld>(Zool.)</fld>, <cd>the blackfish.</cd> -- <col><b>The social evil</b></col>, <cd>prostitution.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><syn><b>Syn.</b> -- Sociable; companionable; conversible; friendly; familiar; communicative; convival; festive.</syn><br/
[<source>1913 Webster</source>]</p>

<p><ent>Socialism</ent><br/
<hw>So"cial*ism</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Cf. F. <ets>socialisme</ets>.]</ety> <def>A theory or system of social reform which contemplates a complete reconstruction of society, with a more just and equitable distribution of property and labor. In popular usage, the term is often employed to indicate any lawless, revolutionary social scheme. See <er>Communism</er>, <er>Fourierism</er>, <er>Saint-Simonianism</er>, forms of socialism.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>[<qex>Socialism</qex>] was first applied in England to Owen's theory of social reconstruction, and in France to those also of St. Simon and Fourier . . . The word, however, is used with a great variety of meaning, . . . even by economists and learned critics. The general tendency is to regard as socialistic any interference undertaken by society on behalf of the poor, . . . radical social reform which disturbs the present system of private property . . . The tendency of the present <qex>socialism</qex> is more and more to ally itself with the most advanced democracy.</q> <rj><qau>Encyc. Brit.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>We certainly want a true history of <qex>socialism</qex>, meaning by that a history of every systematic attempt to provide a new social existence for the mass of the workers.</q> <rj><qau>F. Harrison.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs>-- <col><b>Socialism of the chair</b></col> <ety>[G. <ets>katheder socialismus</ets>]</ety>, <cd>a term applied about 1872, at first in ridicule, to a group of German political economists who advocated state aid for the betterment of the working classes.</cd></cs><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Socialist</ent><br/
<hw>So"cial*ist</hw>, <pos>n.</pos> <ety>[Cf. F. <ets>socialiste</ets>.]</ety> <def>One who advocates or practices the doctrines of socialism.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Socialistic</ent><br/
<ent>Socialist</ent><br/
<mhw>{ <hw>So"cial*ist</hw>, <hw>So`cial*is"tic</hw>, }</mhw> <pos>a.</pos> <def>Pertaining to, or of the nature of, socialism.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sociality</ent><br/
<hw>So`ci*al"i*ty</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Cf. F. <ets>socialist<eacute/</ets>, L. <ets>socialitas</ets>.]</ety> <def>The quality of being social; socialness.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Socialize</ent><br/
<hw>So"cial*ize</hw> <pr>(?)</pr>, <pos>v. t.</pos> <sn>1.</sn> <def>To render social.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To subject to, or regulate by, socialism.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Socially</ent><br/
<hw>So"cial*ly</hw>, <pos>adv.</pos> <def>In a social manner; sociably.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Socialness</ent><br/
<hw>So"cial*ness</hw>, <pos>n.</pos> <def>The quality or state of being social.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sociate</ent><br/
<hw>So"ci*ate</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>sociatus</ets>, p. p. of <ets>sociare</ets> to associate, fr. <ets>socius</ets> companion.]</ety> <def>Associated.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sociate</ent><br/
<hw>So"ci*ate</hw>, <pos>n.</pos> <def>An associate.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>As for you, Dr. Reynolds, and your <qex>sociates</qex>.</q> <rj><qau>Fuller.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sociate</ent><br/
<hw>So"ci*ate</hw> <pr>(?)</pr>, <pos>v. i.</pos> <def>To associate.</def> <mark>[Obs.]</mark>  <rj><au>Shelford.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Societarian</ent><br/
<hw>So*ci`e*ta"ri*an</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Of or pertaining to society; social.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The all-sweeping besom of <qex>societarian</qex> reformation.</q> <rj><qau>Lamb.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Societary</ent><br/
<hw>So*ci"e*ta*ry</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Societarian.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Society</ent><br/
<hw>So*ci"e*ty</hw> <pr>(?)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Societies</plw> <pr>(#)</pr>.</plu> <ety>[L. <ets>societas</ets>, fr. <ets>socius</ets> a companion: cf. F. <ets>soci<eacute/t<eacute/</ets>. See <er>Social</er>.]</ety> <sn>1.</sn> <def>The relationship of men to one another when associated in any way; companionship; fellowship; company.</def> <ldquo/Her loved <xex>society</xex>.<rdquo/  <rj><au>Milton.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>There is <qex>society</qex> where none intrudes<br/
By the deep sea, and music in its roar.</q> <rj><qau>Byron.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Connection; participation; partnership.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>The meanest of the people and such as have the least <qex>society</qex> with the acts and crimes of kings.</q> <rj><qau>Jer. Taylor.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>A number of persons associated for any temporary or permanent object; an association for mutual or joint usefulness, pleasure, or profit; a social union; a partnership; <as>as, a missionary <ex>society</ex></as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>The persons, collectively considered, who live in any region or at any period; any community of individuals who are united together by a common bond of nearness or intercourse; those who recognize each other as associates, friends, and acquaintances.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <def>Specifically, the more cultivated portion of any community in its social relations and influences; those who mutually give receive formal entertainments.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Society of Jesus</b></col>. <cd>See <er>Jesuit</er>.</cd> -- <col><b>Society verses</b></col> <ety>[a translation of F. <ets>vers de soci<eacute/t<eacute/</ets>]</ety>, <cd>the lightest kind of lyrical poetry; verses for the amusement of polite society.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Socinian</ent><br/
<hw>So*cin"i*an</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Of or pertaining to <xex>Socinus</xex>, or the Socinians.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Socinian</ent><br/
<hw>So*cin"i*an</hw>, <pos>n.</pos> <def>One of the followers of Socinus; a believer in Socinianism.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Socinianism</ent><br/
<hw>So*cin"i*an*ism</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Eccl. Hist.)</fld> <def>The tenets or doctrines of Faustus Socinus, an Italian theologian of the sixteenth century, who denied the Trinity, the deity of Christ, the personality of the Devil, the native and total depravity of man, the vicarious atonement, and the eternity of future punishment. His theory was, that Christ was a man divinely commissioned, who had no existence before he was conceived by the Virgin Mary; that human sin was the imitation of Adam's sin, and that human salvation was the imitation and adoption of Christ's virtue; that the Bible was to be interpreted by human reason; and that its language was metaphorical, and not to be taken literally.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Socinianize</ent><br/
<hw>So*cin"i*an*ize</hw> <pr>(?)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Socinianized</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Socinianizing</conjf> <pr>(?)</pr>.]</vmorph> <def>To cause to conform to Socinianism; to regulate by, or imbue with, the principles of Socinianism.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sociological</ent><br/
<ent>Sociologic</ent><br/
<mhw>{ <hw>So`ci*o*log"ic</hw> <pr>(?)</pr>, <hw>So`ci*o*log"ic*al</hw> <pr>(?)</pr> }</mhw> <pos>a.</pos> <def>Of or pertaining to sociology, or social science.</def> -- <wordforms><wf>So`ci*o*log"ic*al*ly</wf>, <pos>adv.</pos></wordforms><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sociologist</ent><br/
<hw>So`ci*ol"o*gist</hw> <pr>(?)</pr>, <pos>n.</pos> <def>One who treats of, or devotes himself to, the study of sociology.</def>  <rj><au>J. S. Mill.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sociology</ent><br/
<hw>So`ci*ol"o*gy</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>socius</ets> a companion + <ets>-logy</ets>.]</ety> <def>That branch of philosophy which treats of the constitution, phenomena, and development of human society; social science.</def>  <rj><au>H. Spencer.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sock</ent><br/
<hw>Sock</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F. <ets>soc</ets>, LL. <ets>soccus</ets>, perhaps of Celtic origin.]</ety> <def>A plowshare.</def>  <rj><au>Edin. Encyc.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sock</ent><br/
<hw>Sock</hw>, <pos>n.</pos> <ety>[OE. <ets>sock</ets>, AS. <ets>socc</ets>, fr. L. <ets>soccus</ets> a kind of low-heeled, light shoe.  Cf. <er>Sucket</er>.]</ety> <sn>1.</sn> <def>The shoe worn by actors of comedy in ancient Greece and Rome, -- used as a symbol of comedy, or of the comic drama, as distinguished from tragedy, which is symbolized by the <contr>buskin</contr>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Great Fletcher never treads in buskin here,<br/
Nor greater Jonson dares in <qex>socks</qex> appear.</q> <rj><qau>Dryden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A knit or woven covering for the foot and lower leg; a stocking with a short leg.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>A warm inner sole for a shoe.</def>  <rj><au>Simmonds.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sock</ent><br/
<hw>Sock</hw> <pr>(s<ocr/k)</pr>, <pos>v. t.</pos> <ety>[Perh. shortened fr. <ets>sockdolager</ets>.]</ety> <def>To hurl, drive, or strike violently; -- often with <xex>it</xex> as an object.</def> <mark>[Prov. or Vulgar]</mark>  <rj><au>Kipling.</au></rj><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Sock away</ent><br/
<hw>Sock away</hw> <pr>(?)</pr>, <pos>v. t.</pos> <def>to save (money); to put (something) into a reserve store.</def> <mark>[Colloq.]</mark><br/
[<source>PJC</source>]</p>

<p><ent>Sock in</ent><br/
<hw>Sock in</hw> <pr>(?)</pr>, <pos>v. t.</pos> <def>to cause (an airport) to cease functioning, by severe weather conditions, such as fog or a snowstorm.  Used often in the passive; <as>the airport was <ex>socked in</ex> for two days by a blizzard</as>.</def><br/
[<source>PJC</source>]</p>

<p><ent>Sockdolager</ent><br/
<hw>Sock*dol"a*ger</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[A corruption of <ets>doxology</ets>.]</ety> <altsp>[Written also <asp>sockdologer</asp>.]</altsp> <sn>1.</sn> <def>That which finishes or ends a matter; a settler; a poser, as a heavy blow, a conclusive answer, and the like.</def> <mark>[Slang, U.S.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Angling)</fld> <def>A combination of two hooks which close upon each other, by means of a spring, as soon as the fish bites.</def> <mark>[U. S.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>Something unusually large.</def> <mark>[Slang, U. S.]</mark><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Socket</ent><br/
<hw>Sock"et</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[OE. <ets>soket</ets>, a dim. through OF. fr. L. <ets>soccus</ets>. See <er>Sock</er> a covering for the foot.]</ety> <sn>1.</sn> <def>An opening into which anything is fitted; any hollow thing or place which receives and holds something else; <as>as, the <ex>sockets</ex> of the teeth</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>His eyeballs in their hollow <qex>sockets</qex> sink.</q> <rj><qau>Dryden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Especially, the hollow tube or place in which a candle is fixed in the candlestick.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <fld>(Electricity)</fld> <def>the receptacle of an electric lamp into which a light bulb is inserted, containing contacts to conduct electricity to the bulb.</def><br/
[<source>PJC</source>]</p>

<p><sn>4.</sn> <fld>(Electricity)</fld> <def>the receptacle fixed in a wall and connected by conductive wiring to an electrical supply, containing contacts to conduct electricity, and into which the plug of an electrical device is inserted; -- called also a <altname>wall socket</altname> or <altname>outlet</altname>.  The <ex>socket</ex> will typically have two or three contacts; if three, the third is connected to a ground for safety.</def><br/
[<source>PJC</source>]</p>

<p><q>And in the <qex>sockets</qex> oily bubbles dance.</q> <rj><qau>Dryden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Socket bolt</b></col> <fld>(Mach.)</fld>, <cd>a bolt that passes through a thimble that is placed between the parts connected by the bolt.</cd> -- <col><b>Socket chisel</b></col>. <cd>Same as <altname>Framing chisel</altname>. See under <er>Framing</er>.</cd> -- <col><b>Socket pipe</b></col>, <cd>a pipe with an expansion at one end to receive the end of a connecting pipe.</cd> -- <col><b>Socket pole</b></col>, <cd>a pole armed with iron fixed on by means of a socket, and used to propel boats, etc.</cd> <mark>[U.S.]</mark> -- <col><b>Socket wrench</b></col>, <cd>a wrench consisting of a socket at the end of a shank or rod, for turning a nut, bolthead, etc., in a narrow or deep recess.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Socketed</ent><br/
<hw>Sock"et*ed</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Having a socket.</def>  <rj><au>Dawkins.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sockless</ent><br/
<hw>Sock"less</hw>, <pos>a.</pos> <def>Destitute of socks or shoes.</def>  <rj><au>B. & Fl.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Socky</ent><br/
<hw>Sock"y</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Wet; soaky.</def> <mark>[Prov. Eng.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Socle</ent><br/
<hw>So"cle</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F., fr. L. <ets>socculus</ets>, dim. of <ets>soccus</ets>. See <er>Sock</er> a covering for the foot.  Cf. <er>Zocco</er>.]</ety> <fld>(Arch.)</fld> <sd>(a)</sd> <def>A plain block or plinth forming a low pedestal; any base; especially, the base of a statue, column, or the like. See <er>Plinth</er>.</def> <sd>(b)</sd> <def>A plain face or plinth at the lower part of a wall.</def>  <rj><au>Oxf. Gloss.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Socman</ent><br/
<hw>Soc"man</hw> <pr>(?)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Socmen</plw> <pr>(#)</pr>.</plu> <ety>[See <er>Socage</er>.]</ety> <fld>(O. Eng. Law)</fld> <def>One who holds lands or tenements by socage; a socager.</def>  <rj><au>Cowell.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Socmanry</ent><br/
<hw>Soc"man*ry</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(O.E. Law)</fld> <def>Tenure by socage.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Socome</ent><br/
<hw>Soc"ome</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[AS. <ets>s<omac/cen</ets>, <ets>s<omac/cn</ets>, searching, or the right of searching, the lord's court. See <er>Soc</er>.]</ety> <fld>(O.Eng. Law)</fld> <def>A custom of tenants to grind corn at the lord's mill.</def>  <rj><au>Cowell.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Socotrine</ent><br/
<hw>Soc"o*trine</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Of or pertaining to Socotra, an island in the Indian Ocean, on the east coast of Africa.</def> -- <def2><pos>n.</pos> <def>A native or inhabitant of Socotra.</def></def2><br/
[<source>1913 Webster</source>]</p>

<p><ent>Socratical</ent><br/
<ent>Socratic</ent><br/
<mhw>{ <hw>So*crat"ic</hw> <pr>(?)</pr>, <hw>So*crat"ic*al</hw> <pr>(?)</pr>, }</mhw> <pos>a.</pos> <ety>[L. <ets>Socraticus</ets>, Gr. <?/<?/<?/<?/.]</ety> <def>Of or pertaining to Socrates, the Grecian sage and teacher. (<sc>b. c.</sc> 469-399), or to his manner of teaching and philosophizing.</def><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ The <xex>Socratic</xex> method of reasoning and instruction was by a series of questions leading the one to whom they were addressed to perceive and admit what was true or false in doctrine, or right or wrong in conduct.</note><br/
[<source>1913 Webster</source>]</p>

<p><ent>Socratically</ent><br/
<hw>So*crat"ic*al*ly</hw>, <pos>adv.</pos> <def>In the Socratic method.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Socratism</ent><br/
<hw>Soc"ra*tism</hw> <pr>(?)</pr>, <pos>n.</pos> <def>The philosophy or the method of Socrates.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Socratist</ent><br/
<hw>Soc"ra*tist</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Gr. <?/<?/<?/<?/.]</ety> <def>A disciple or follower of Socrates.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sod</ent><br/
<hw>Sod</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Zool.)</fld> <def>The rock dove.</def> <mark>[Prov. Eng.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sod</ent><br/
<hw>Sod</hw>, <mark>obs.</mark> <def><pos>imp.</pos> of <er>Seethe</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sod</ent><br/
<hw>Sod</hw>, <pos>n.</pos> <ety>[Akin to LG. <ets>sode</ets>, D. <ets>zode</ets>, OD. <ets>sode</ets>, <ets>soode</ets>, OFries. satha, and E. <ets>seethe</ets>. So named from its <ets>sodden</ets> state in wet weather. See <er>Seethe</er>.]</ety> <def>That stratum of the surface of the soil which is filled with the roots of grass, or any portion of that surface; turf; sward.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>She there shall dress a sweeter <qex>sod</qex><br/
Than Fancy's feet have ever trod.</q> <rj><qau>Collins.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sod</ent><br/
<hw>Sod</hw>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Sodden</conjf>; <pos>p. pr. & vb. n.</pos> <conjf>Sodding</conjf>.]</vmorph> <def>To cover with sod; to turf.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Soda</ent><br/
<hw>So"da</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[It., soda, in OIt., ashes used in making glass, fr. L. <ets>solida</ets>, fem. of <ets>solidus</ets> solid; <ets>solida</ets> having probably been a name of glasswort. See <er>Solid</er>.]</ety> <sn>1.</sn> <fld>(Chem.)</fld> <sd>(a)</sd> <def>Sodium oxide or hydroxide.</def> <sd>(b)</sd> <def>Popularly, sodium carbonate or bicarbonate.  Sodium bicarbonate is also called <altname>baking soda</altname></def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>same as <er>sodium</er>, used in terms such as <cref>bicarbonate of soda</cref>.</def><br/
[<source>PJC</source>]</p>

<p><sn>3.</sn> <def>same as <er>soda water</er>.</def><br/
[<source>PJC</source>]</p>

<p><sn>4.</sn> <def>a non-alcoholic beverage, sweetened by various means, containing flavoring and supersaturated with carbon dioxide, so as to be effervescent when the container is opened; -- in different localities it is variously called also <altname>soda pop</altname>, <altname>pop</altname>, <altname>mineral water</altname>, and <altname>minerals</altname>. It has many variants. The sweetening agent may be natural, such as cane sugar or corn syrup, or artificial, such as saccharin or aspartame.  The flavoring varies widely, popular variants being fruit or cola flavoring.</def><br/
[<source>PJC</source>]</p>

<p><cs><col><b>Caustic soda</b></col>, <cd>sodium hydroxide.</cd> -- <col><b>Cooking soda</b></col>, <cd>sodium bicarbonate.</cd> <mark>[Colloq.]</mark> -- <col><b>Sal soda</b></col>. <cd>See <cref>Sodium carbonate</cref>, under <er>Sodium</er>.</cd> -- <col><b>Soda alum</b></col> <fld>(Min.)</fld>, <cd>a mineral consisting of the hydrous sulphate of alumina and soda.</cd> -- <col><b>Soda ash</b></col>, <cd>crude sodium carbonate; -- so called because formerly obtained from the ashes of sea plants and certain other plants, as saltwort (<gen>Salsola</gen>). See under <er>Sodium</er>.</cd> -- <col><b>Soda fountain</b></col>, <cd>an apparatus for drawing soda water, fitted with delivery tube, faucets, etc.</cd> -- <col><b>Soda lye</b></col>, <cd>a lye consisting essentially of a solution of sodium hydroxide, used in soap making.</cd> -- <col><b>Soda niter</b></col>. <cd>See <er>Nitratine</er>.</cd> -- <col><b>Soda salts</b></col>, <cd>salts having sodium for the base; specifically, sodium sulphate or Glauber's salts.</cd> -- <col><b>Soda waste</b></col>, <cd>the waste material, consisting chiefly of calcium hydroxide and sulphide, which accumulates as a useless residue or side product in the ordinary Leblanc process of soda manufacture; -- called also <altname>alkali waste</altname>.</cd> -- <col><b>Washing soda</b></col>, <cd>sodium carbonate.</cd> <mark>[Colloq.]</mark></cs><br/
[<source>1913 Webster</source>]</p>

<p><-- p. 1366 --></p>

<p><ent>Soda biscuit</ent><br/
<hw>So"da biscuit</hw> <pr>(?)</pr>, <pos>n.</pos> <def>a bicuit leavened with baking soda and with buttermilk or sour milk.</def><br/
[<source>PJC</source>]</p>

<p><ent>Soda bread</ent><br/
<hw>So"da bread</hw> <pr>(?)</pr>, <pos>n.</pos> <def>a bread made with buttermilk and leavened with baking soda.</def><br/
[<source>PJC</source>]</p>

<p><ent>Soda fountain</ent><br/
<hw>So"da foun`tain</hw> <pr>(?)</pr>, <pos>n.</pos> <def>a counter at which people may sit and be served soda pop, ice cream, or light meals.  Such counters may be located for example in restaurants, ice cream shops, drugstores, departments stores.</def><br/
[<source>PJC</source>]</p>

<p><ent>Sodaic</ent><br/
<hw>So*da"ic</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Pertaining to, or containing, soda.</def> <ldquo/<xex>Sodaic</xex> powder.<rdquo/  <rj><au>Ure.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Soda jerk</ent><br/
<hw>So"da jerk</hw> <pr>(?)</pr>, <pos>n.</pos> <def>a person who serves soda, ice cream, similar foods from behind a <er>soda fountain</er>.</def><br/
[<source>PJC</source>]</p>

<p><ent>Sodalite</ent><br/
<hw>So"da*lite</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[<ets>Soda</ets> + <ets>-lite</ets>: cf. F. <ets>sodalithe</ets>.]</ety> <fld>(Min.)</fld> <def>A mineral of a white to blue or gray color, occuring commonly in dodecahedrons, also massive.  It is a silicate of alumina and soda with some chlorine.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sodality</ent><br/
<hw>So*dal"i*ty</hw> <pr>(?)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Sodalities</plw> <pr>(#)</pr>.</plu> <ety>[L. <ets>sodalitas</ets>, fr. <ets>sodalis</ets> a comrade.]</ety> <sn>1.</sn> <def>A fellowship or fraternity; a brotherhood.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(R. C. Ch.)</fld> <def>Specifically, a lay association for devotion or for charitable purposes.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sodamide</ent><br/
<hw>Sod*am"ide</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Chem.)</fld> <def>A greenish or reddish crystalline substance, <chform>NaNH2</chform>, obtained by passing ammonia over heated sodium.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Soda water</ent><br/
<hw>So"da wat`er</hw> <pr>(?)</pr>, <pos>n.</pos> <sn>1.</sn> <def>originally, a beverage consisting of a weak solution of sodium bicarbonate to which an acid (such as phosphoric acid) has been added to generate carbon dioxide and thus cause effervescence; the term now is also used as a synonym for <altname>soda pop</altname>.  Soda water without flavoring is used as a beverage or as a component of a mixed beverage.  It is typically bottled or canned under pressure so that the carbon dioxide and effervescence will be preserved until use.</def><br/
[<source>PJC</source>]</p>

<p><sn>2.</sn> <def>a beverage containing supersaturated carbon dioxide  (carbonic acid) and thus effervescent.</def><br/
[<source>PJC</source>]</p>

<p><sn>3.</sn> <def>same as <er>soda pop</er>.</def><br/
[<source>PJC</source>]</p>

<p><ent>Soda pop</ent><br/
<hw>So"da pop</hw> <pr>(?)</pr>, <pos>n.</pos> <def>a popular non-alcoholic beverage, sweetened by various means, containing flavoring and supersaturated with carbon dioxide, so as to be effervescent when the container is opened; -- in different localities it is variously called also <altname>soda</altname>, <altname>pop</altname>, <altname>mineral water</altname>, and <altname>minerals</altname>. It has many variants. The sweetening agent may be natural, such as cane sugar or corn syrup, or artificial, such as saccharin or aspartame.  The flavoring varies widely, popular variants being fruit juices, fruit sirups, cream, or cola flavoring; the <ex>soda pop</ex> is usually served chilled.</def> <note>Several large corporations started primarily as bottlers of <ex>soda pop</ex>, such as <stype>Coca-Cola</stype>, <stype>Pepsi-Cola</stype>, and <stype>Dr. Pepper</stype>.</note><br/
[<source>PJC</source>]</p>

<p><ent>Sodden</ent><br/
<hw>Sod"den</hw> <pr>(?)</pr>, <pos>a.</pos>  [<pos>p. p.</pos> of <er>Seethe</er>.] <def>Boiled; seethed; also, soaked; heavy with moisture; saturated; <as>as, <ex>sodden</ex> beef; <ex>sodden</ex> bread; <ex>sodden</ex> fields</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sodden</ent><br/
<hw>Sod"den</hw>, <pos>v. i.</pos> <def>To be seethed; to become sodden.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sodden</ent><br/
<hw>Sod"den</hw>, <pos>v. t.</pos> <def>To soak; to make heavy with water.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sodden-witted</ent><br/
<hw>Sod"den-wit`ted</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Heavy; dull.</def>  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Soddy</ent><br/
<hw>Sod"dy</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[From <er>Sod</er>.]</ety> <def>Consisting of sod; covered with sod; turfy.</def>  <rj><au>Cotgrave.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Soder</ent><br/
<hw>Sod"er</hw> <pr>(?)</pr>, <pos>n. & v. t.</pos> <def>See <er>Solder</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sodger</ent><br/
<hw>So"dger</hw> <pr>(s<omac/"j<etil/r)</pr>, <pos>n. & v. i.</pos> <def>Variant of <er>Soldier</er>.</def> <mark>[Dial. or Slang]</mark><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Sodic</ent><br/
<hw>So"dic</hw> <pr>(?)</pr>, <pos>a.</pos> <fld>(Chem.)</fld> <def>Of or pertaining to sodium; containing sodium.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sodio-</ent><br/
<hw>So"di*o-</hw> <pr>(?)</pr>. <fld>(Chem.)</fld> <def>A combining form (also used adjectively) denoting <xex>the presence of sodium</xex> or <xex>one of its compounds</xex>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sodium</ent><br/
<hw>So"di*um</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[NL., fr.E. <ets>soda</ets>.]</ety> <fld>(Chem.)</fld> <def>A common metallic element of the alkali group, in nature always occuring combined, as in common salt, in albite, etc. It is isolated as a soft, waxy, white, unstable metal, so highly reactive that it combines violently with water, and to be preserved must be kept under petroleum or some similar liquid. Sodium is used combined in many salts, in the free state as a reducer, and as a means of obtaining other metals (as magnesium and aluminium) is an important commercial product. Symbol Na (<altname>Natrium</altname>). Atomic weight 22.990.  Specific gravity 0.97.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Sodium amalgam</b></col>, <cd>an alloy of sodium and mercury, usually produced as a gray metallic crystalline substance, which is used as a reducing agent, and otherwise.</cd> -- <col><b>Sodium carbonate</b></col>, <cd>a white crystalline substance, <chform>Na2CO3.10H2O</chform>, having a cooling alkaline taste, found in the ashes of many plants, and produced artifically in large quantities from common salt. It is used in making soap, glass, paper, etc., and as alkaline agent in many chemical industries. Called also <altname>sal soda</altname>, <altname>washing soda</altname>, or <altname>soda</altname>.  Cf. <er>Sodium bicarbonate</er>, and <er>Trona</er>.</cd> -- <col><b>Sodium chloride</b></col>, <cd>common, or table, salt, <chform>NaCl</chform>.</cd> -- <col><b>Sodium hydroxide</b></col>, <cd>a white opaque brittle solid, <chform>NaOH</chform>, having a fibrous structure, produced by the action of quicklime, or of calcium hydrate (milk of lime), on sodium carbonate. It is a strong alkali, and is used in the manufacture of soap, in making wood pulp for paper, etc. Called also <altname>sodium hydrate</altname>, and <altname>caustic soda</altname>. By extension, a solution of sodium hydroxide.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sodium bicarbonate</ent><br/
<hw>Sodium bicarbonate</hw>, <def>a white crystalline substance, <chform>HNaCO3</chform>, with a slight alkaline taste resembling that of sodium carbonate. It is found in many mineral springs and also produced artificially,. It is used in cookery, in baking powders, and as a source of carbonic acid gas (carbon dioxide) for soda water. Called also <altname>baking soda</altname>, <altname>cooking soda</altname>, <altname>bicarbonate of soda</altname>, <altname>bicarb</altname>, <altname>saleratus</altname>, and technically, <altname>acid sodium carbonate</altname>, <altname>sodium acid carbonate</altname>, <altname>primary sodium carbonate</altname>, <altname>sodium dicarbonate</altname>, etc.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sodium sulphate</ent><br/
<hw>So"di*um sul"phate</hw>. <def>A salt well known as a catharic under the name of <altname>Glauber's salt</altname>, which term is properly applied to the hydrate, <chform>Na2SO4.10H2O</chform>.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Sodomite</ent><br/
<hw>Sod"om*ite</hw> <pr>(?)</pr>, <pos>n.</pos> <sn>1.</sn> <def>An inhabitant of Sodom.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>One guilty of sodomy.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sodomitical</ent><br/
<hw>Sod`om*it"ic*al</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Pertaining to, or of the nature of, sodomy.</def> -- <wordforms><wf>Sod`om*it"ic*al*ly</wf>, <pos>adv.</pos></wordforms><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sodomy</ent><br/
<hw>Sod"om*y</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[From <ets>Sodom</ets>. a country mentioned in the Bible: cf. F. <ets>sodomite</ets>.]</ety> <def>Carnal copulation in a manner against nature; buggery.</def>  <rj><au>Gen. xix. 5.</au></rj>
<-- can we be more explicit? --><br/
[<source>1913 Webster</source>]</p>

<p><ent>Soe</ent><br/
<hw>Soe</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Scot. <ets>sae</ets>, <ets>say</ets>, <ets>saye</ets>; cf. Icel. <ets>s<amac/r</ets> a large cask, Sw. <ets>s<?/</ets> a tub.]</ety> <def>A large wooden vessel for holding water; a cowl.</def> <mark>[Obs. or Prov. Eng.]</mark>  <rj><au>Dr. H. More.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Soever</ent><br/
<hw>So*ev"er</hw> <pr>(?)</pr>. <def>A word compounded of <xex>so</xex> and <xex>ever</xex>, used in composition with <xex>who</xex>, <xex>what</xex>, <xex>where</xex>, <xex>when</xex>, <xex>how</xex>, etc., and indicating any out of all possible or supposable persons, things, places, times, ways, etc. It is sometimes used separate from the pronoun or adverb.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>For unto whom<qex>soever</qex> much is given, of him shall be much required.</q> <rj><qau>Luke xii. 48.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>What great thing <qex>soever</qex> a man proposed to do in his life, he should think of achieving it by fifty.</q> <rj><qau>Sir W. Temple.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sofa</ent><br/
<hw>So"fa</hw> <pr>(?)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Sofas</plw> <pr>(#)</pr>.</plu> <ety>[Ar. <ets>soffah</ets>, from <ets>saffa</ets> to dispose in order: cf. F. <ets>sofa</ets>, It. <ets>sof<agrave/</ets>.]</ety> <def>A long seat, usually with a cushioned bottom, back, and ends; -- much used as a comfortable piece of furniture.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Let fall the curtains, wheel the <qex>sofa</qex> round.</q> <rj><qau>Cowper.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Sofa bed</b></col>, <cd>a sofa so contrived that it may be extended to form a bed; -- called also <altname>sofa bedstead</altname>.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Soffit</ent><br/
<hw>Sof"fit</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[It. <ets>soffitta</ets>, <ets>soffitto</ets>, fr. <ets>soffiggere</ets> to hide, properly, to fix or fasten under, L. <ets>suffigere</ets> to fasten beneath or below; <ets>sub</ets> under, beneath + <ets>figere</ets> to fix, faste: cf. F. <ets>soffite</ets>.]</ety> <fld>(Arch.)</fld> <def>The under side of the subordinate parts and members of buildings, such as staircases, entablatures, archways, cornices, or the like. See <xex>Illust.</xex> of <er>Lintel</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sofi</ent><br/
<hw>So"fi</hw> <pr>(?)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Sofis</plw> <pr>(<?/)</pr>.</plu> <def>Same as <er>Sufi</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sofism</ent><br/
<hw>So"fism</hw> <pr>(?)</pr>, <pos>n.</pos> <def>Same as <er>Sufism</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Soft</ent><br/
<hw>Soft</hw> <pr>(s<ocr/ft; 115)</pr>, <pos>a.</pos> <amorph>[<pos>Compar.</pos> <adjf>Softer</adjf> <pr>(s<ocr/ft"<etil/r)</pr>; <pos>superl.</pos> <adjf>Softest</adjf>.]</amorph> <ety>[OE. <ets>softe</ets>, AS. <ets>s<omac/fte</ets>, properly adv. of <ets>s<emac/fte</ets>, adj.; akin to OS. <ets>s<amac/fto</ets>, adv., D. <ets>zacht</ets>, OHG. <ets>samfto</ets>, adv., <ets>semfti</ets>, adj., G. <ets>sanft</ets>, LG. <ets>sacht</ets>; of uncertain origin.]</ety> <sn>1.</sn> <def>Easily yielding to pressure; easily impressed, molded, or cut; not firm in resisting; impressible; yielding; also, malleable; -- opposed to <ant>hard</ant>; <as>as, a <ex>soft</ex> bed; a <ex>soft</ex> peach; <ex>soft</ex> earth; <ex>soft</ex> wood or metal.</as></def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Not rough, rugged, or harsh to the touch; smooth; delicate; fine; <as>as, <ex>soft</ex> silk; a <ex>soft</ex> skin.</as></def><br/
[<source>1913 Webster</source>]</p>

<p><q>They that wear <qex>soft</qex> clothing are in king's houses.</q> <rj><qau>Matt. xi. 8.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>Hence, agreeable to feel, taste, or inhale; not irritating to the tissues; <as>as, a <ex>soft</ex> liniment; <ex>soft</ex> wines</as>.</def> <ldquo/The <xex>soft</xex>, delicious air.<rdquo/  <rj><au>Milton.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>Not harsh or offensive to the sight; not glaring; pleasing to the eye; not exciting by intensity of color or violent contrast; <as>as, <ex>soft</ex> hues or tints</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The sun, shining upon the upper part of the clouds . . . made the <qex>softest</qex> lights imaginable.</q> <rj><qau>Sir T. Browne.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <def>Not harsh or rough in sound; gentle and pleasing to the ear; flowing; <as>as, <ex>soft</ex> whispers of music</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Her voice was ever <qex>soft</qex>,<br/
Gentle, and low, -- an excellent thing in woman.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q><qex>Soft</qex> were my numbers; who could take offense?</q> <rj><qau>Pope.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>6.</sn> <def>Easily yielding; susceptible to influence; flexible; gentle; kind.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>I would to God my heart were flint, like Edward's;<br/
Or Edward's <qex>soft</qex> and pitiful, like mine.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>The meek or <qex>soft</qex> shall inherit the earth.</q> <rj><qau>Tyndale.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>7.</sn> <def>Expressing gentleness, tenderness, or the like; mild; conciliatory; courteous; kind; <as>as, <ex>soft</ex> eyes</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>A <qex>soft</qex> answer turneth away wrath.</q> <rj><qau>Prov. xv. 1.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>A face with gladness overspread,<br/
<qex>Soft</qex> smiles, by human kindness bred.</q> <rj><qau>Wordsworth.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>8.</sn> <def>Effeminate; not courageous or manly, weak.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>A longing after sensual pleasures is a dissolution of the spirit of a man, and makes it loose, <qex>soft</qex>, and wandering.</q> <rj><qau>Jer. Taylor.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>9.</sn> <def>Gentle in action or motion; easy.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>On her <qex>soft</qex> axle, white she paces even,<br/
And bears thee soft with the smooth air along.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>10.</sn> <def>Weak in character; impressible.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The deceiver soon found this <qex>soft</qex> place of Adam's.</q> <rj><qau>Glanvill.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>11.</sn> <def>Somewhat weak in intellect.</def> <mark>[Colloq.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>He made <qex>soft</qex> fellows stark noddies, and such as were foolish quite mad.</q> <rj><qau>Burton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>12.</sn> <def>Quiet; undisturbed; paceful; <as>as, <ex>soft</ex> slumbers</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>13.</sn> <def>Having, or consisting of, a gentle curve or curves; not angular or abrupt; <as>as, <ex>soft</ex> outlines</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>14.</sn> <def>Not tinged with mineral salts; adapted to decompose soap; <as>as, <ex>soft</ex> water is the best for washing</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>15.</sn> <fld>(Phonetics)</fld> <sd>(a)</sd> <def>Applied to a palatal, a sibilant, or a dental consonant (as <xex>g</xex> in <xex>gem</xex>, <xex>c</xex> in <xex>cent</xex>, etc.) as distinguished from a guttural mute (as <xex>g</xex> in <xex>go</xex>, <xex>c</xex> in <xex>cone</xex>, etc.); -- opposed to <ant>hard</ant>.</def> <sd>(b)</sd> <def>Belonging to the class of sonant elements as distinguished from the surd, and considered as involving less force in utterance; <as>as, <ex>b</ex>, <ex>d</ex>, <ex>g</ex>, <ex>z</ex>, <ex>v</ex>, etc.</as>, in contrast with <xex>p</xex>, <xex>t</xex>, <xex>k</xex>, <xex>s</xex>, <xex>f</xex>, etc.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Soft clam</b></col> <fld>(Zool.)</fld>, <cd>the common or long clam (<spn>Mya arenaria</spn>). See <er>Mya</er>.</cd> -- <col><b>Soft coal</b></col>, <cd>bituminous coal, as distinguished from <xex>anthracite</xex>, or <xex>hard, coal</xex>.</cd> -- <col><b>Soft crab</b></col> <fld>(Zool.)</fld>, <cd>any crab which has recently shed its shell.</cd> -- <col><b>Soft dorsal</b></col> <fld>(Zool.)</fld>, <cd>the posterior part of the dorsal fin of fishes when supported by soft rays.</cd> -- <col><b>Soft grass</b></col>. <fld>(Bot.)</fld> <cd>See <er>Velvet grass</er>.</cd> -- <col><b>Soft money</b></col>, <cd>paper money, as distinguished from <xex>coin</xex>, or <xex>hard money</xex>.</cd> <mark>[Colloq. U.S.]</mark> -- <col><b>Soft mute</b></col>. <fld>(Phonetics)</fld> <cd>See <er>Media</er>.</cd> -- <col><b>Soft palate</b></col>. <cd>See the Note under <er>Palate</er>.</cd> -- <col><b>Soft ray</b></col> <fld>(Zool.)</fld>, <cd>a fin ray which is articulated and usually branched.</cd> -- <col><b>Soft soap</b></col>. <cd>See under <er>Soap</er>.</cd> -- <col><b>Soft-tack</b></col>, <cd>leavened bread, as distinguished from <contr>hard-tack</contr>, or <contr>ship bread</contr>.</cd> -- <col><b>Soft tortoise</b></col> <fld>(Zool.)</fld>, <cd>any river tortoise of the genus Trionyx. See <er>Trionyx</er>.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Soft</ent><br/
<hw>Soft</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A soft or foolish person; an idiot.</def> <mark>[Colloq.]</mark>  <rj><au>G. Eliot.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Soft</ent><br/
<hw>Soft</hw>, <pos>adv.</pos> <def>Softly; without roughness or harshness; gently; quietly.</def>  <rj><au>Chaucer.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>A knight <qex>soft</qex> riding toward them.</q> <rj><qau>Spenser.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Soft</ent><br/
<hw>Soft</hw>, <pos>interj.</pos> <def>Be quiet; hold; stop; not so fast.</def><-- archaic or obs. --><br/
[<source>1913 Webster</source>]</p>

<p><q><qex>Soft</qex>, you; a word or two before you go.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Softa</ent><br/
<hw>Sof"ta</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Corruption of Per. <ets>s<omac/khtah</ets> one who burns, is ardent or zealous.]</ety> <def>Any one attached to a Mohammedan mosque, esp. a student of the higher branches of theology in a mosque school.</def> <altsp>[Written also <asp>sophta</asp>.]</altsp><br/
[<source>1913 Webster</source>]</p>

<p><ent>Soften</ent><br/
<hw>Sof"ten</hw> <pr>(?)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Softened</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Softening</conjf>.]</vmorph> <def>To make soft or more soft.</def> Specifically: --<br/
[<source>1913 Webster</source>]</p>

<p><sd>(a)</sd> <def>To render less hard; -- said of matter.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Their arrow's point they <qex>soften</qex> in the flame.</q> <rj><qau>Gay.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sd>(b)</sd> <def>To mollify; to make less fierce or intractable.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Diffidence conciliates the proud, and <qex>softens</qex> the severe.</q> <rj><qau>Rambler.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sd>(c)</sd> <def>To palliate; to represent as less enormous; <as>as, to <ex>soften</ex> a fault</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sd>(d)</sd> <def>To compose; to mitigate; to assuage.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Music can <qex>soften</qex> pain to ease.</q> <rj><qau>Pope.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sd>(e)</sd> To make calm and placid.<br/
[<source>1913 Webster</source>]</p>

<p><q>All that cheers or <qex>softens</qex> life.</q> <rj><qau>Pope.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sd>(f)</sd> <def>To make less harsh, less rude, less offensive, or less violent, or to render of an opposite quality.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>He bore his great commision in his look,<br/
But tempered awe, and <qex>softened</qex> all he spoke.</q> <rj><qau>Dryden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sd>(g)</sd> <def>To make less glaring; to tone down; <as>as, to <ex>soften</ex> the coloring of a picture</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sd>(h)</sd> <def>To make tender; to make effeminate; to enervate; <as>as, troops <ex>softened</ex> by luxury</as>.</def>
<-- weaken --><br/
[<source>1913 Webster</source>]</p>

<p><sd>(i)</sd> <def>To make less harsh or grating, or of a quality the opposite; <as>as, to <ex>soften</ex> the voice</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Soften</ent><br/
<hw>Sof"ten</hw>, <pos>v. i.</pos> <def>To become soft or softened, or less rude, harsh, severe, or obdurate.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Softener</ent><br/
<hw>Sof"ten*er</hw> <pr>(?)</pr>, <pos>n.</pos> <def>One who, or that which, softens.</def> <mark>[Written also, less properly, softner.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><-- Water softener, a device which exchanges sodium ions for polyvalent metal cations in water, thus converting the "hard" mineral-containing water into "soft" water.  This treatment renders the water more usable for washing, as the polyvalent cations interfere with the detergent action of soaps. --><br/
[<source>1913 Webster</source>]</p>

<p><ent>Softening</ent><br/
<hw>Sof"ten*ing</hw>, <def><pos>a. & n.</pos> from <er>Soften</er>, <pos>v.</pos></def><br/
[<source>1913 Webster</source>]</p>

<p><cs><mcol><col><b>Softening of the brain</b></col>, <it>or</it> <col><b>Cerebral softening</b></col></mcol> <fld>(Med.)</fld>, <cd>a localized softening of the brain substance, due to hemorrhage or inflammation. Three varieties, distinguished by their color and representing different stages of the morbid process, are known respectively as <xex>red</xex>, <xex>yellow</xex>, and <xex>white</xex>, <xex>softening</xex>.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Soft-finned</ent><br/
<hw>Soft"-finned`</hw> <pr>(?)</pr>, <pos>a.</pos> <fld>(Zool.)</fld> <def>Having the fin rays cartilaginous or flexible; without spines; -- said of certain fishes.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Soft-headed</ent><br/
<hw>Soft"-head`ed</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Weak in intellect.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Soft-hearted</ent><br/
<hw>Soft"-heart`ed</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Having softness or tenderness of heart; susceptible of pity or other kindly affection; gentle; meek.</def> -- <wordforms><wf>Soft"-heart`ed*ness</wf>, <pos>n.</pos></wordforms><br/
[<source>1913 Webster</source>]</p>

<p><ent>Softish</ent><br/
<hw>Soft"ish</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Somewhat soft.</def>  <rj><au>De Witt Clinton.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Softling</ent><br/
<hw>Soft"ling</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A soft, effeminate person; a voluptuary.</def> <mark>[R.]</mark>  <rj><au>Bp. Woolton. .</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Softly</ent><br/
<hw>Soft"ly</hw>, <pos>adv.</pos> <def>In a soft manner.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Softner</ent><br/
<hw>Soft"ner</hw> <pr>(?)</pr>, <pos>n.</pos> <def>See <er>Softener</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Softness</ent><br/
<hw>Soft"ness</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[AS. <ets>s<?/ftness</ets>, <ets>s<?/ftnyss</ets>.]</ety> <def>The quality or state of being soft; -- opposed to <ant>hardness</ant>, and used in the various specific senses of the adjective.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Soft-shelled</ent><br/
<ent>Soft-shell</ent><br/
<mhw><hw>Soft"-shell`</hw> <pr>(?)</pr>, <hw>Soft"-shelled`</hw> <pr>(?)</pr>, }</mhw> <pos>a.</pos> <def>Having a soft or fragile shell.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Soft-shell clam</b></col> <fld>(Zool.)</fld>, <cd>the long clam. See <er>Mya</er>.</cd> -- <col><b>Soft-shelled crab</b></col>. <fld>(Zool.)</fld> <cd>See the Note under <er>Crab</er>, 1.</cd> -- <col><b>Soft-shelled turtle</b></col>. <fld>(Zool.)</fld> <cd>Same as <cref>Soft tortoise</cref>, under <er>Soft</er>.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Soft-spoken</ent><br/
<hw>Soft"-spo`ken</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Speaking softly; having a mild or gentle voice; hence, mild; affable.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Soft steel</ent><br/
<hw>Soft steel</hw>. <def>Steel low in carbon; mild steel; ingot iron.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Soger</ent><br/
<hw>So"ger</hw> <pr>(s<omac/"j<etil/r)</pr>, <pos>n. & v. i.</pos> <def>Var. of <er>Soldier</er>.</def> <mark>[Dial. or Slang]</mark>  <rj><au>R. H. Dana, Jr.</au></rj><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Sogginess</ent><br/
<hw>Sog"gi*ness</hw> <pr>(?)</pr>, <pos>n.</pos> <def>The quality or state of being soggy; soddenness; wetness.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Soggy</ent><br/
<hw>Sog"gy</hw> <pr>(?)</pr>, <pos>a.</pos> <amorph>[<pos>Compar.</pos> <adjf>Soggier</adjf> <pr>(?)</pr>; <pos>superl.</pos> <adjf>Soggiest</adjf>.]</amorph> <ety>[Cf. Icel. <ets>s<oum/ggr</ets> damp, wet, or E. <ets>soak</ets>.]</ety> <def>Filled with water; soft with moisture; sodden; soaked; wet; <as>as, <ex>soggy</ex> land or timber</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Soho</ent><br/
<hw>So*ho"</hw> <pr>(?)</pr>, <pos>interj.</pos> <def>Ho; -- a word used in calling from a distant place; a sportsman's halloo.</def>  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Soi-disant</ent><br/
||<hw>Soi`-di*sant"</hw> <pr>(sw<aum/"d<esl/`z<aum/N")</pr>, <pos>a.</pos> <ety>[F.; <ets>soi</ets> one's self + <ets>disant</ets>, p. pr. of <ets>dire</ets> to say.]</ety> <def>Self-named; self-styled; pretended; would-be.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Soil</ent><br/
<hw>Soil</hw> <pr>(soil)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Soiled</conjf> <pr>(soild)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Soiling</conjf>.]</vmorph> <ety>[OF. <ets>saoler</ets>, <ets>saouler</ets>, to satiate, F. <ets>so<ucir/ler</ets>, L. <ets>satullare</ets>, fr. <ets>satullus</ets>, dim. of <ets>satur</ets> sated. See <er>Satire</er>.]</ety> <def>To feed, as cattle or horses, in the barn or an inclosure, with fresh grass or green food cut for them, instead of sending them out to pasture; hence (such food having the effect of purging them), to purge by feeding on green food; <as>as, to <ex>soil</ex> a horse</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Soil</ent><br/
<hw>Soil</hw>, <pos>n.</pos> <ety>[OE. <ets>soile</ets>, F. <ets>sol</ets>, fr. L. <ets>solum</ets> bottom, soil; but the word has probably been influenced in form by <ets>soil</ets> a miry place.  Cf. <er>Saloon</er>, <er>Soil</er> a miry place, <er>Sole</er> of the foot.]</ety> <sn>1.</sn> <def>The upper stratum of the earth; the mold, or that compound substance which furnishes nutriment to plants, or which is particularly adapted to support and nourish them.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Land; country.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Must I thus leave thee, Paradise? thus leave<br/
Thee, native <qex>soil</qex>?</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>Dung; faeces; compost; manure; <as>as, night <ex>soil</ex></as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Improve land by dung and other sort of <qex>soils</qex>.</q> <rj><qau>Mortimer.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Soil pipe</b></col>, <cd>a pipe or drain for carrying off night soil.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Soil</ent><br/
<hw>Soil</hw>, <pos>v. t.</pos> <def>To enrich with soil or muck; to manure.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Men . . . <qex>soil</qex> their ground, not that they love the dirt, but that they expect a crop.</q> <rj><qau>South.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Soil</ent><br/
<hw>Soil</hw>, <pos>n.</pos> <ety>[OF. <ets>soil</ets>, <ets>souil</ets>, F. <ets>souille</ets>, from OF. <ets>soillier</ets>, F. <ets>souiller</ets>. See <er>Soil</er> to make dirty.]</ety> <def>A marshy or miry place to which a hunted boar resorts for refuge; hence, a wet place, stream, or tract of water, sought for by other game, as deer.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>As deer, being stuck, fly through many <qex>soils</qex>,<br/
Yet still the shaft sticks fast.</q> <rj><qau>Marston.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>To take soil</b></col>, <cd>to run into the mire or water; hence, to take refuge or shelter.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><q>O, sir, have you <qex>taken soil</qex> here? It is well a man may reach you after three hours' running.</q> <rj><qau>B. Jonson.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Soil</ent><br/
<hw>Soil</hw>, <pos>v. t.</pos><ety>[OE. <ets>soilen</ets>, OF. <ets>soillier</ets>, F. <ets>souiller</ets>, (assumed) LL. <ets>suculare</ets>, fr. L. <ets>sucula</ets> a little pig, dim. of <ets>sus</ets> a swine. See <er>Sow</er>, <pos>n.</pos>]</ety> <sn>1.</sn> <def>To make dirty or unclean on the surface; to foul; to dirty; to defile; <as>as, to <ex>soil</ex> a garment with dust</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Our wonted ornaments now <qex>soiled</qex> and stained.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To stain or mar, as with infamy or disgrace; to tarnish; to sully.</def>  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><syn><b>Syn.</b> -- To foul; dirt; dirty; begrime; bemire; bespatter; besmear; daub; bedaub; stain; tarnish; sully; defile; pollute.</syn><br/
[<source>1913 Webster</source>]</p>

<p><ent>Soil</ent><br/
<hw>Soil</hw>, <pos>v. i.</pos> <def>To become soiled; <as>as, light colors <ex>soil</ex> sooner than dark ones</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Soil</ent><br/
<hw>Soil</hw>, <pos>n.</pos> <ety>[See <er>Soil</er> to make dirty, <er>Soil</er> a miry place.]</ety> <def>That which soils or pollutes; a soiled place; spot; stain.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>A lady's honor . . . will not bear a <qex>soil</qex>.</q> <rj><qau>Dryden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Soiliness</ent><br/
<hw>Soil"i*ness</hw> <pr>(?)</pr>, <pos>n.</pos> <def>Stain; foulness.</def> <mark>[R.]</mark>  <rj><au>Bacon.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Soilless</ent><br/
<hw>Soil"less</hw>, <pos>a.</pos> <def>Destitute of soil or mold.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Soil pipe</ent><br/
<hw>Soil pipe</hw>. <def>A pipe or drain for carrying off night soil.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Soilure</ent><br/
<hw>Soil"ure</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[OF. <ets>soillure</ets>, F. <ets>souillure</ets>. See <er>Soil</er> to make dirty.]</ety> <def>Stain; pollution.</def>  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Then fearing rust or <qex>soilure</qex>, fashioned for it<br/
A case of silk.</q> <rj><qau>Tennyson.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Soily</ent><br/
<hw>Soil"y</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Dirty; soiled.</def> <mark>[Obs.]</mark>  <rj><au>Fuller.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Soiree</ent><br/
||<hw>Soi`ree"</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F., fr. <ets>soir</ets> evening, fr. L. <ets>serus</ets> late, <ets>serum</ets> late time.  Cf. <er>Serenade</er>.]</ety> <def>An evening party; -- distinguished from <xex>levee</xex>, and <xex>matin<eacute/e</xex>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Soja</ent><br/
<hw>So"ja</hw> <pr>(s<omac/"j<adot/ <or/ s<omac/"y<adot/)</pr>, <pos>n.</pos> <fld>(Bot.)</fld> <def>An Asiatic leguminous herb (<spn>Glycine max</spn>, formerly <spn>Glycine Soja</spn>) the seeds of which (called <prod>soy beans</prod>) are used in preparing the sauce called <prod>soy</prod>.  Called also <altname>soya</altname>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sojer</ent><br/
<hw>So"jer</hw> <pr>(s<omac/"j<etil/r)</pr>, <pos>n. & v. i.</pos> <def>Var. of <er>Soldier</er>.</def> <mark>[Dial. or Slang]</mark><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Sojourn</ent><br/
<hw>So"journ</hw> <pr>(?)</pr>, <pos>v. i.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Sojourned</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Sojourning</conjf>.]</vmorph> <ety>[OE. <ets>sojornen</ets>, <ets>sojournen</ets>, OF. <ets>sojorner</ets>, <ets>sejorner</ets>, F. <ets>s<eacute/journer</ets>, fr. L. <ets>sub</ets> under, about + <ets>diurnus</ets> belonging to the day. See <er>Journal</er>, <er>Diurnal</er>.]</ety> <def>To dwell for a time; to dwell or live in a place as a temporary resident or as a stranger, not considering the place as a permanent habitation; to delay; to tarry.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Abram went down into Egypt to <qex>sojourn</qex> there.</q> <rj><qau>Gen. xii. 30.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Home he goeth, he might not longer <qex>sojourn</qex>.</q> <rj><qau>Chaucer.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>The soldiers first assembled at Newcastle, and there <qex>sojourned</qex> three days.</q> <rj><qau>Hayward.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sojourn</ent><br/
<hw>So"journ</hw>, <pos>n.</pos> <ety>[Cf. OF. <ets>sujurn</ets>, <ets>sujur</ets>, <ets>sejor</ets>, F. <ets>s<eacute/jour</ets>. See <er>Sojourn</er>, <pos>v. i.</pos>]</ety> <def>A temporary residence, as that of a traveler in a foreign land.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Though long detained<br/
In that obscure <qex>sojourn</qex>.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sojourner</ent><br/
<hw>So"journ*er</hw> <pr>(?)</pr>, <pos>n.</pos> <def>One who sojourns.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>We are strangers before thee, and <qex>sojourners</qex>.</q> <rj><qau>1. Chron. xxix. 15.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sojourning</ent><br/
<hw>So"journ*ing</hw>, <pos>n.</pos> <def>The act or state of one who sojourns.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sojournment</ent><br/
<hw>So"journ*ment</hw> <pr>(?)</pr>, <pos>n.</pos> <def>Temporary residence, as that of a stranger or a traveler.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Soke</ent><br/
<hw>Soke</hw> <pr>(?)</pr>, <pos>n.</pos> <sn>1.</sn> <fld>(Eng. Law)</fld> <def>See <er>Soc</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><-- p. 1367 --></p>

<p><sn>2.</sn> <def>One of the small territorial divisions into which Lincolnshire, England, is divided.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sokeman</ent><br/
<hw>Soke"man</hw> <pr>(?)</pr>, <pos>n.</pos> <def>See <er>Socman</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sokemanry</ent><br/
<hw>Soke"man*ry</hw> <pr>(?)</pr>, <pos>n.</pos> <def>See <er>Socmanry</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Soken</ent><br/
<hw>Sok"en</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Cf. <er>Socome</er>.]</ety> <sn>1.</sn> <def>A toll. See <er>Soc</er>, <pos>n.</pos>, 2.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>Great <qex>sooken</qex> had this miller, out of doubt.</q> <rj><qau>Chaucer.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A district held by socage.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Soko</ent><br/
<hw>So"ko</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Zool.)</fld> <def>An African anthropoid ape, supposed to be a variety of the chimpanzee.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sol</ent><br/
||<hw>Sol</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L.]</ety> <sn>1.</sn> <def>The sun.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Alchem.)</fld> <def>Gold; -- so called from its brilliancy, color, and value.</def>  <rj><au>Chaucer.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sol</ent><br/
<hw>Sol</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[It.]</ety> <fld>(Mus.)</fld> <sd>(a)</sd> <def>A syllable applied in solmization to the note G, or to the fifth tone of any diatonic scale.</def> <sd>(b)</sd> <def>The tone itself.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sol</ent><br/
<hw>Sol</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[See <er>Sou</er>.]</ety> <sn>1.</sn> <def>A sou.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A silver and gold coin of Peru. The silver <xex>sol</xex> is the unit of value, and is worth about 68 cents.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sole</ent><br/
<ent>Sol</ent><br/
<mhw>{ <hw>Sol</hw> <hw>Sole</hw> }</mhw> <pr>(?)</pr>, <pos>n.</pos> <ety>[From <ets>hydrosol</ets> an aqueous colloidal solution, confused with G. <ets>sole</ets>, <ets>soole</ets>, salt water from which salt is obtained.]</ety> <fld>(Chem.)</fld> <def>A fluid mixture of a colloid and a liquid; a liquid colloidal solution or suspension.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Sola</ent><br/
||<hw>So"la</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L., fem. of <ets>solus</ets>.]</ety> <def>See <er>Solus</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sola</ent><br/
<hw>So"la</hw>, <pos>n.</pos> <ety>[Native name.]</ety> <fld>(Bot.)</fld> <def>A leguminous plant (<spn>Aeschynomene aspera</spn>) growing in moist places in Southern India and the East Indies. Its pithlike stem is used for making hats, swimming-jackets, etc.</def> <altsp>[Written also <asp>solah</asp>, <asp>shola</asp>.]</altsp><br/
[<source>1913 Webster</source>]</p>

<p><ent>Solace</ent><br/
<hw>Sol"ace</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[OF. <ets>solas</ets>, <ets>ssoulaz</ets>, L. <ets>solacium</ets>, <ets>solatium</ets>, fr. <ets>solari</ets> to comfort, console.  Cf. <er>Console</er>, <pos>v. t.</pos>]</ety> <sn>1.</sn> <def>Comfort in grief; alleviation of grief or anxiety; also, that which relieves in distress; that which cheers or consoles; relief.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>In business of mirth and of <qex>solace</qex>.</q> <rj><qau>Chaucer.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>The proper <qex>solaces</qex> of age are not music and compliments, but wisdom and devotion.</q> <rj><qau>Rambler.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Rest; relaxation; ease.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>To make his steed some <qex>solace</qex>.</q> <rj><qau>Chaucer.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><syn><b>Syn.</b> -- Comfort; consolation; alleviation; relief.</syn><br/
[<source>1913 Webster</source>]</p>

<p><ent>Solace</ent><br/
<hw>Sol"ace</hw>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Solaced</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Solacing</conjf> <pr>(?)</pr>.]</vmorph> <ety>[OF. <ets>solacier</ets>, <ets>soulacier</ets>, F. <ets>solacier</ets>, LL. <ets>solatiare</ets>. See <er>Solace</er>, <pos>n.</pos>]</ety> <sn>1.</sn> <def>To cheer in grief or under calamity; to comfort; to relieve in affliction, solitude, or discomfort; to console; -- applied to persons; <as>as, to <ex>solace</ex> one with the hope of future reward</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To allay; to assuage; to soothe; <as>as, to <ex>solace</ex> grief</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><syn><b>Syn.</b> -- To comfort; assuage; allay. See <er>Comfort</er>.</syn><br/
[<source>1913 Webster</source>]</p>

<p><ent>Solace</ent><br/
<hw>Sol"ace</hw>, <pos>v. i.</pos> <def>To take comfort; to be cheered.</def>  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Solacement</ent><br/
<hw>Sol"ace*ment</hw> <pr>(?)</pr>, <pos>n.</pos> <def>The act of solacing, or the state of being solaced; also, that which solaces.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Solacious</ent><br/
<hw>So*la"cious</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[Cf. OF. <ets>solacieux</ets>.]</ety> <def>Affording solace; <as>as, a <ex>solacious</ex> voice</as>.</def> <mark>[Obs.]</mark>  <rj><au>Bale.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Solanaceous</ent><br/
<hw>Sol`a*na"ceous</hw> <pr>(?)</pr>, <pos>a.</pos> <fld>(Bot.)</fld> <def>Of or pertaining to plants of the natural order <ord>Solanaceae</ord>, of which the nightshade (<gen>Solanum</gen>) is the type. The order includes also the tobacco, ground cherry, tomato, eggplant, red pepper, and many more.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Soland</ent><br/
<hw>So"land</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Zool.)</fld> <def>A solan goose.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Solander</ent><br/
<hw>So*lan"der</hw> <pr>(?)</pr>, <pos>n.</pos> <def>See <er>Sallenders</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Solan goose</ent><br/
<hw>So"lan goose`</hw> <pr>(?)</pr>. <ety>[Icel. <ets>s<?/la</ets>; akin to Norw. <ets>sula</ets>.]</ety> <fld>(Zool.)</fld> <def>The common gannet.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Solania</ent><br/
<hw>So*la"ni*a</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[NL.]</ety> <fld>(Chem.)</fld> <def>Solanine.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Solanicine</ent><br/
<hw>So*lan"i*cine</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[See <er>Solanine</er>.]</ety> <fld>(Chem.)</fld> <def>An alkaloid produced by the action of hydrochloric acid on solanidine, as a tasteless yellow crystalline substance.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Solanidine</ent><br/
<hw>So*lan"i*dine</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[See <er>Solanine</er>.]</ety> <fld>(Chem.)</fld> <def>An alkaloid produced by the decomposition of solanine, as a white crystalline substance having a harsh bitter taste.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Solanine</ent><br/
<hw>Sol"a*nine</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>solanum</ets> nightshade.]</ety> <fld>(Chem.)</fld> <def>A poisonous alkaloid glucoside extracted from the berries of common nightshade (<spn>Solanum nigrum</spn>), and of bittersweet, and from potato sprouts, as a white crystalline substance having an acrid, burning taste; -- called also <altname>solonia</altname>, and <altname>solanina</altname>.</def>
<-- C45H73NO15 --><br/
[<source>1913 Webster</source>]</p>

<p><ent>Solano</ent><br/
||<hw>So*la"no</hw> <pr>(?)</pr>, <ety>[Sp., fr. L. <ets>solanus</ets> (sc. <ets>ventus</ets>), from <ets>sol</ets> the sun.]</ety> <def>A hot, oppressive wind which sometimes blows in the Mediterranean, particularly on the eastern coast of Spain.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Solanoid</ent><br/
<hw>Sol"a*noid</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[<ets>Solanum</ets> + <ets>-oid</ets>.]</ety> <fld>(Med.)</fld> <def>Resembling a potato; -- said of a kind of cancer.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Solanum</ent><br/
<hw>So*la"num</hw> <pr>(?)</pr>, <pos>prop. n.</pos> <ety>[L., nightshade.]</ety> <fld>(Bot.)</fld> <def>A genus of plants comprehending the potato (<spn>Solanum tuberosum</spn>), the eggplant (<spn>Solanum melongena</spn>, and several hundred other species; nightshade.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Solar</ent><br/
<hw>So"lar</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[OE. <ets>soler</ets>, AS. <ets>solere</ets>, L. <ets>solarium</ets>, from <ets>sol</ets> the sun. See <er>Solar</er>, <pos>a.</pos>]</ety> <def>A loft or upper chamber; a garret room.</def> <mark>[Obs.]</mark> <altsp>[Written also <asp>soler</asp>, <asp>solere</asp>, <asp>sollar</asp>.]</altsp>  <rj><au>Oxf. Gloss.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Solar</ent><br/
<hw>So"lar</hw>, <pos>a.</pos> <ety>[L. <ets>solaris</ets>, fr. <ets>sol</ets> the sun; akin to As. <ets>s<omac/l</ets>, Icel. <ets>s<omac/l</ets>, Goth. <ets>sauil</ets>, Lith. <ets>saule</ets>, W. <ets>haul</ets>,. <ets>sul</ets>, Skr. <ets>svar</ets>, perhaps to E. <ets>sun</ets>:F. <ets>solaire</ets>.  Cf. <er>Parasol</er>. <er>Sun</er>.]</ety> <sn>1.</sn> <def>Of or pertaining to the sun; proceeding from the sun; <as>as, the <ex>solar</ex> system; <ex>solar</ex> light; <ex>solar</ex> rays; <ex>solar</ex> influence</as>. See <cref>Solar system</cref>, below.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Astrol.)</fld> <def>Born under the predominant influence of the sun.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>And proud beside, as <qex>solar</qex> people are.</q> <rj><qau>Dryden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>Measured by the progress or revolution of the sun in the ecliptic; <as>as, the <ex>solar</ex> year</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>Produced by the action of the sun, or peculiarly affected by its influence.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>They denominate some herbs <qex>solar</qex>, and some lunar.</q> <rj><qau>Bacon.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Solar cycle</b></col>. <cd>See under <er>Cycle</er>.</cd> -- <col><b>Solar day</b></col>. <cd>See <er>Day</er>, 2.</cd> -- <col><b>Solar engine</b></col>, <cd>an engine in which the energy of solar heat is used to produce motion, as in evaporating water for a steam engine, or expanding air for an air engine.</cd> -- <col><b>Solar flowers</b></col> <fld>(Bot.)</fld>, <cd>flowers which open and shut daily at certain hours.</cd> -- <col><b>Solar lamp</b></col>, <cd>an argand lamp.</cd> -- <col><b>Solar microscope</b></col>, <cd>a microscope consisting essentially, first, of a mirror for reflecting a beam of sunlight through the tube, which sometimes is fixed in a window shutter; secondly, of a condenser, or large lens, for converging the beam upon the object; and, thirdly, of a small lens, or magnifier, for throwing an enlarged image of the object at its focus upon a screen in a dark room or in a darkened box.</cd><br/
[<source>1913 Webster</source>]</p>

<p><-- Illustration of solar microscope --><br/
[<source>1913 Webster</source>]</p>

<p> -- <col><b>Solar month</b></col>. <cd>See under <er>Month</er>.</cd> -- <col><b>Solar oil</b></col>, <cd>a paraffin oil used an illuminant and lubricant.</cd> -- <col><b>Solar phosphori</b></col> <fld>(Physics)</fld>, <cd>certain substances, as the diamond, siulphide of barium (Bolognese or Bologna phosphorus), calcium sulphide, etc., which become phosphorescent, and shine in the dark, after exposure to sunlight or other intense light.</cd> -- <col><b>Solar plexus</b></col> <fld>(Anat.)</fld>, <cd>a nervous plexus situated in the dorsal and anterior part of the abdomen, consisting of several sympathetic ganglia with connecting and radiating nerve fibers; -- so called in allusion to the radiating nerve fibers.</cd> -- <col><b>Solar spots</b></col>. <cd>See <cref>Sun spots</cref>, under <er>Sun</er>.</cd> -- <col><b>Solar system</b></col> <fld>(Astron.)</fld>, <cd>the sun, with the group of celestial bodies which, held by its attraction, revolve round it. The system comprises the major planets, with their satellites; the minor planets, or asteroids, and the comets; also, the meteorids, the matter that furnishes the zodiacal light, and the rings of Saturn. The satellites that revolve about the major planets are twenty-two in number, of which the Earth has one (see <er>Moon</er>.), Mars two, Jupiter five, Saturn nine, Uranus four, and Neptune one. The asteroids, between Mars and Jupiter, thus far discovered (1900), number about five hundred, the first four of which were found near the beginning of the century, and are called Ceres, Pallas, Juno, and Vesta.</cd><br/
[<source>1913 Webster</source>]</p>

<p>  <note>The principal elements of the major planets, and of the comets seen at more than one perihelion passage, are exhibited in the following tables: --<br/
[<source>1913 Webster</source>]</p>

<p>
<table>
<title> I. -- Major Planets.</title>
<colheads><coltitle>Symbol.</coltitle><coltitle>Name.</coltitle><coltitle>Mean distance -- that of the Earth being unity.</coltitle><coltitle>Period in days.</coltitle><coltitle>Eccentricity.</coltitle><coltitle>Inclination of orbit.</coltitle><coltitle>Diameter in miles</coltitle></colheads>
<?/<?/<?/<?/<?/<?/<?/<?/<?/<?/<?/<?/<?/<?/<?/<?/<?/<?/<?/<?/<?/
</table><br/
[<source>1913 Webster</source>]</p>

<p><table>
<title> II. -- Periodic Comets.</title>
<colheads><coltitle>Name.</coltitle><coltitle>Greatest distance from sun.</coltitle><coltitle>Least distance from sun.</coltitle><coltitle>Inclination of orbit.</coltitle><coltitle>Perihelion passage.</coltitle></colheads>
<row><item></item><item></item><item></item><item></item><item><deg/ <min/ 54</item><item></item></row>
<row><item>Encke's</item><item>3.31</item><item>4.10</item><item>0.342</item><item>12 54</item><item>1885.2</item></row>
<?/<?/<?/<?/<?/<?/<?/<?/<?/<?/<?/<?/<?/<?/<?/<?/<?/<?/<?/<?/<?/
</table>
</note><br/
[<source>1913 Webster</source>]</p>

<p>-- <col><b>Solar telegraph</b></col>, <cd>telegraph for signaling by flashes of reflected sunlight.</cd> -- <col><b>Solar time</b></col>. <cd>See <cref>Apparent time</cref>, under <er>Time</er>.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Solarium</ent><br/
||<hw>So*la"ri*um</hw> <pr>(?)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Solaria</plw> <pr>(#)</pr>.</plu> <ety>[L. See <er>Solar</er>, <pos>n.</pos>]</ety> <sn>1.</sn> <def>An apartment freely exposed to the sun; anciently, an apartment or inclosure on the roof of a house; in modern times, an apartment in a hospital, used as a resort for convalescents.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Zool.)</fld> <def>Any one of several species of handsome marine spiral shells of the genus <gen>Solarium</gen> and allied genera.  The shell is conical, and usually has a large, deep umbilicus exposing the upper whorls. Called also <altname>perspective shell</altname>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Solarization</ent><br/
<hw>So`lar*i*za"tion</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Photog.)</fld> <def>Injury of a photographic picture caused by exposing it for too long a time to the sun's light in the camera; burning; excessive insolation.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Solarize</ent><br/
<hw>So"lar*ize</hw> <pr>(?)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Solarized</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Solarizing</conjf> <pr>(?)</pr>.]</vmorph> <fld>(Photog.)</fld> <def>To injure by too long exposure to the light of the sun in the camera; to burn.</def><-- now overexposure --><br/
[<source>1913 Webster</source>]</p>

<p><ent>Solarize</ent><br/
<hw>So"lar*ize</hw>, <pos>v. i.</pos> <fld>(Photog.)</fld> <def>To become injured by undue or too long exposure to the sun's rays in the camera; -- an older term now replaced by <altname>overexpose</altname>.</def><br/
[<source>1913 Webster</source> <source>+PJC</source>]</p>

<p><ent>Solar myth</ent><br/
<hw>So"lar myth</hw>. <def>A myth which essentially consists of allegory based upon ideas as to the sun's course, motion, influence, or the like.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Solar parallax</ent><br/
<hw>Solar parallax</hw>. <def>The parallax of the sun, that is, the angle subtended at the sun by the semidiameter of the earth.  It is 8.80<prime/<prime/, and is the fundamental datum.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Solary</ent><br/
<hw>So"la*ry</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Solar.</def> <mark>[Obs.]</mark>  <rj><au>Sir T. Browne.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Solas</ent><br/
<hw>Sol"as</hw> <pr>(?)</pr>, <pos>n.</pos> <def>Solace.</def> <mark>[Obs.]</mark>  <rj><au>Chaucer.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Solatium</ent><br/
||<hw>So*la"ti*um</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. See <er>Solace</er>, <pos>n.</pos>]</ety> <def>Anything which alleviates or compensates for suffering or loss; a compensation; esp., an additional allowance, as for injured feelings.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sold</ent><br/
<hw>Sold</hw> <pr>(?)</pr>, <def><pos>imp. & p. p.</pos> of <er>Sell</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sold</ent><br/
<hw>Sold</hw>, <pos>n.</pos> <ety>[F. <ets>solde</ets>. See <er>Soldier</er>, and cf. <er>Sou</er>.]</ety> <def>Solary; military pay.</def> <mark>[Obs.]</mark>  <rj><au>Spenser.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Soldan</ent><br/
<hw>Sol"dan</hw> <pr>(?)</pr>, <pos>n.</pos><ety>[OE. <ets>soudan</ets>, F. <ets>soudan</ets>, from the Arabic. See <er>Sultan</er>.]</ety> <def>A sultan.</def> <mark>[Obs.]</mark>  <rj><au>Milton.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Soldanel</ent><br/
<hw>Sol"da*nel</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Bot.)</fld> <def>A plant of the genus <gen>Soldanella</gen>, low Alpine herbs of the Primrose family.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Soldanrie</ent><br/
<hw>Sol"dan*rie</hw> <pr>(?)</pr>, <pos>n.</pos> <def>The country ruled by a soldan, or sultan; a sultanate{2}.</def> <mark>[Poet.]</mark>  <rj><au>Sir W. Scott.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Solder</ent><br/
<hw>Sol"der</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Formerly <ets>soder</ets>; F. <ets>soudure</ets>, OF. <ets>soudeure</ets>, fr. OF. & F. <ets>souder</ets> to solder, L. <ets>solidare</ets> to fasten, to make solid. See <er>Solid</er>, and cf. <er>Sawder</er>.]</ety> <def>A metal or metallic alloy used when melted for uniting adjacent metallic edges or surfaces; a metallic cement.</def> <specif>Hence,</specif> <def>anything which unites or cements.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Hard solder</b></col>, <cd>a solder which fuses only at a red heat, as one composed of zinc and copper, or silver and copper, etc.</cd> -- <col><b>Soft solder</b></col>, <cd>a solder fusible at comparatively low temperatures; <as>as, plumbers' solder, consisting of two parts lead and one part tin, is a <ex>soft solder</ex></as>.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Solder</ent><br/
<hw>Sol"der</hw>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Soldered</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Soldering</conjf>.]</vmorph> <ety>[Formerly<ets>soder</ets>. See <er>Solder</er>, <pos>n.</pos>]</ety> <sn>1.</sn> <def>To unite (metallic surfaces or edges) by the intervention of a more fusible metal or metallic alloy applied when melted; to join by means of metallic cement.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To mend; to patch up.</def> <ldquo/To <xex>solder</xex> up a broken cause.<rdquo/  <rj><au>Hooker.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Solderer</ent><br/
<hw>Sol"der*er</hw> <pr>(?)</pr>, <pos>n.</pos> <def>One who solders.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Soldering</ent><br/
<hw>Sol"der*ing</hw>, <def><pos>a. & n.</pos> from <er>Solder</er>, <pos>v. t.</pos></def><br/
[<source>1913 Webster</source>]</p>

<p><cs><mcol><col><b>Soldering iron</b></col>, <col><b>Soldering tool</b></col></mcol>, <cd>an instrument for soldering, consisting of a bit or bolt of copper having a pointed or wedge-shaped end, and furnished with a handle.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Soldier</ent><br/
<hw>Sol"dier</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[OE. <ets>souldier</ets>, <ets>soudiour</ets>, <ets>souder</ets>, OF. <ets>soldier</ets>, <ets>soldoier</ets>, <ets>soldeier</ets>, <ets>sodoier</ets>, <ets>soudoier</ets>, <ets>soudier</ets>, fr. L. <ets>solidus</ets> a piece of money (hence applied to the pay of a soldier), fr. <ets>solidus</ets> solid. See <er>Solid</er>, and cf. <er>Sold</er>, <pos>n.</pos>]</ety> <sn>1.</sn> <def>One who is engaged in military service as an officer or a private; one who serves in an army; one of an organized body of combatants.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>I am a <qex>soldier</qex> and unapt to weep.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Especially, a private in military service, as distinguished from an officer.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>It were meet that any one, before he came to be a captain, should have been a <qex>soldier</qex>.</q> <rj><qau>Spenser.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>A brave warrior; a man of military experience and skill, or a man of distinguished valor; -- used by way of emphasis or distinction.</def>  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <fld>(Zool.)</fld> <def>The red or cuckoo gurnard (<spn>Trigla pini</spn>.)</def> <mark>[Prov. Eng.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <fld>(Zool.)</fld> <def>One of the asexual polymorphic forms of white ants, or termites, in which the head and jaws are very large and strong. The soldiers serve to defend the nest. See <er>Termite</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Soldier beetle</b></col> <fld>(Zool.)</fld>, <cd>an American carabid beetle (<spn>Chauliognathus Americanus</spn>) whose larva feeds upon other insects, such as the plum curculio.</cd> -- <col><b>Soldier bug</b></col> <fld>(Zool.)</fld>, <cd>any hemipterous insect of the genus <gen>Podisus</gen> and allied genera, as the spined <ex>soldier bug</ex> (<spn>Podius spinosus</spn>). These bugs suck the blood of other insects.</cd> -- <col><b>Soldier crab</b></col> <fld>(Zool.)</fld> <sd>(a)</sd> <cd>The hermit crab.</cd> <sd>(b)</sd> <cd>The fiddler crab.</cd> -- <col><b>Soldier fish</b></col> <fld>(Zool.)</fld>, <cd>a bright-colored etheostomoid fish (<spn>Etheostoma coeruleum</spn>) found in the Mississippi River; -- called also <altname>blue darter</altname>, and <altname>rainbow darter</altname>.</cd> -- <col><b>Soldier fly</b></col> <fld>(Zool.)</fld>, <cd>any one of numerous species of small dipterous flies of the genus <gen>Stratyomys</gen> and allied genera. They are often bright green, with a metallic luster, and are ornamented on the sides of the back with markings of yellow, like epaulets or shoulder straps.</cd> -- <col><b>Soldier moth</b></col> <fld>(Zool.)</fld>, <cd>a large geometrid moth (<spn>Euschema militaris</spn>), having the wings bright yellow with bluish black lines and spots.</cd> -- <col><b>Soldier orchis</b></col> <fld>(Bot.)</fld>, <cd>a kind of orchis (<spn>Orchis militaris</spn>).</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Soldier</ent><br/
<hw>Sol"dier</hw>, <pos>v. i.</pos> <sn>1.</sn> <def>To serve as a soldier.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To make a pretense of doing something, or of performing any task.</def> <mark>[Colloq.U.S.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ In this sense the vulgar pronounciation <pr>(s<omac/"j<etil/r)</pr> is jocosely preserved.</note><br/
[<source>1913 Webster</source>]</p>

<p><q>It needs an opera glass to discover whether the leaders are pulling, or only <qex>soldiering</qex>.</q> <rj><qau>C. D. Warner.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Soldieress</ent><br/
<hw>Sol"dier*ess</hw>, <pos>n.</pos> <def>A female soldier.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Soldiering</ent><br/
<hw>Sol"dier*ing</hw>, <pos>n.</pos> <sn>1.</sn> <def>The act of serving as a soldier; the state of being a soldier; the occupation of a soldier.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>The act of feigning to work. See the Note under <er>Soldier</er>, <pos>v. i.</pos>, 2.</def> <mark>[Colloq. U.S.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Soldierlike</ent><br/
<hw>Sol"dier*like"</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Like a soldier; soldierly.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Soldierly</ent><br/
<hw>Sol"dier*ly</hw>, <pos>a.</pos> <def>Like or becoming a real soldier; brave; martial; heroic; honorable; soldierlike.</def> <ldquo/<xex>Soldierly</xex> discipline.<rdquo/  <rj><au>Sir P. Sidney.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Soldiership</ent><br/
<hw>Sol"dier*ship</hw>, <pos>n.</pos> <def>Military qualities or state; martial skill; behavior becoming a soldier.</def> <mark>[R.]</mark>  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Soldierwood</ent><br/
<hw>Sol"dier*wood`</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Bot.)</fld> <def>A showy leguminous plant (<spn>Calliandra purpurea</spn>) of the West Indies. The flowers have long tassels of purple stamens.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Soldiery</ent><br/
<hw>Sol"dier*y</hw> <pr>(?)</pr>, <pos>n.</pos> <sn>1.</sn> <def>A body of soldiers; soldiers, collectivelly; the military.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>A camp of faithful <qex>soldiery</qex>.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Military service.</def> <mark>[Obs.]</mark>  <rj><au>Sir P. Sidney.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Soldo</ent><br/
||<hw>Sol"do</hw> <pr>(?)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Soldi</plw> <pr>(#)</pr>.</plu> <ety>[It. See <er>Sou</er>.]</ety> <def>A small Italian coin worth a sou or a cent; the twentieth part of a lira.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sole</ent><br/
<hw>Sole</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F. <ets>sole</ets>, L. <ets>solea</ets>; -- so named from its flat shape. See <er>Sole</er> of the foot.]</ety> <fld>(Zool.)</fld> <sd>(a)</sd> <def>Any one of several species of flatfishes of the genus <gen>Solea</gen> and allied genera of the family <fam>Soleidae</fam>, especially the common European species (<spn>Solea vulgaris</spn>), which is a valuable food fish.</def> <sd>(b)</sd> <def>Any one of several American flounders somewhat resembling the true sole in form or quality, as the California sole (<spn>Lepidopsetta bilineata</spn>), the long-finned sole (<spn>Glyptocephalus zachirus</spn>), and other species.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><mcol><col><b>Lemon</b></col>, <or/ <col><b>French</b></col>, <col><b>sole</b></col></mcol> <fld>(Zool.)</fld>, <cd>a European species of sole (<spn>Solea pegusa</spn>).</cd> -- <col><b>Smooth sole</b></col> <fld>(Zool.)</fld>, <cd>the megrim.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><-- p. 1368 --></p>

<p><ent>Sole</ent><br/
<hw>Sole</hw> <pr>(s<omac/l)</pr>, <pos>n.</pos> <ety>[AS. <ets>sole</ets>, fr. L. <ets>soolea</ets> (or rather an assumed L. <ets>sola</ets>), akin to <ets>solum</ets>round, soil, sole of the foot.  Cf. <er>Exile</er>, <er>Saloon</er>, <er>Soil</er> earth, <er>Sole</er> the fish.]</ety> <sn>1.</sn> <def>The bottom of the foot; hence, also, rarely, the foot itself.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The dove found no rest for the <qex>sole</qex> of her foot.</q> <rj><qau>Gen. viii. 9.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Hast wandered through the world now long a day,<br/
Yet ceasest not thy weary <qex>soles</qex> to lead.</q> <rj><qau>Spenser.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>The bottom of a shoe or boot, or the piece of leather which constitutes the bottom.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The <ldquo/caliga<rdquo/ was a military shoe, with a very thick <qex>sole</qex>, tied above the instep.</q> <rj><qau>Arbuthnot.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>The bottom or lower part of anything, or that on which anything rests in standing.</def> Specifially: <sd>(a)</sd> <fld>(Agric.)</fld> <def>The bottom of the body of a plow; -- called also <altname>slade</altname>; also, the bottom of a furrow.</def> <sd>(b)</sd> <fld>(Far.)</fld> <def>The horny substance under a horse's foot, which protects the more tender parts.</def> <sd>(c)</sd> <fld>(Fort.)</fld> <def>The bottom of an embrasure.</def> <sd>(d)</sd> <fld>(Naut.)</fld> <def>A piece of timber attached to the lower part of the rudder, to make it even with the false keel.</def> <au>Totten.</au> <sd>(e)</sd> <fld>(Mining)</fld> <def>The seat or bottom of a mine; -- applied to horizontal veins or lodes.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Sole leather</b></col>, <cd>thick, strong, used for making the soles of boots and shoes, and for other purposes.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sole</ent><br/
<hw>Sole</hw>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Soled</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Soling</conjf>.]</vmorph> <def>To furnish with a sole; <as>as, to <ex>sole</ex> a shoe</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sole</ent><br/
<hw>Sole</hw>, <pos>a.</pos> <ety>[L. <ets>solus</ets>, or OF. <ets>sol</ets>, F. <ets>seul</ets> (fr. L. <ets>solus</ets>; cf. L. <ets>sollus</ets> whole, entire.  Cf. <er>Desolate</er>, <er>Solemn</er>, <er>Solo</er>, <er>Sullen</er>.]</ety> <sn>1.</sn> <def>Being or acting without another; single; individual; only.</def> <ldquo/The <xex>sole</xex> son of my queen.<rdquo/  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>He, be sure . . . first and last will reign<br/
<qex>Sole</qex> king.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Law)</fld> <def>Single; unmarried; <as>as, a feme <ex>sole</ex></as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Corporation sole</b></col>. <cd>See the Note under <er>Corporation</er>.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><syn><b>Syn.</b> -- Single; individual; only; alone; solitary.</syn><br/
[<source>1913 Webster</source>]</p>

<p><ent>Solecism</ent><br/
<hw>Sol"e*cism</hw> <pr>(?)</pr>, <pos>n.</pos><ety>[F. <ets>sol<eacute/cisme</ets>, L. <ets>soloecismus</ets>, Gr. <grk>soloikismo`s</grk>, fr. <grk>soloiki`zein</grk> to speak or write incorrectly, fr. <grk>so`loikos</grk> speaking incorrectly, from the corruption of the Attic dialect among the Athenian colonists of <grk>So`loi</grk> in Cilicia.]</ety> <sn>1.</sn> <def>An impropriety or incongruity of language in the combination of words or parts of a sentence; esp., deviation from the idiom of a language or from the rules of syntax.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>A barbarism may be in one word; a <qex>solecism</qex> must be of more.</q> <rj><qau>Johnson.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Any inconsistency, unfitness, absurdity, or impropriety, as in deeds or manners.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Caesar, by dismissing his guards and retaining his power, committed a dangerous <qex>solecism</qex> in politics.</q> <rj><qau>C. Middleton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>The idea of having committed the slightest <qex>solecism</qex> in politeness was agony to him.</q> <rj><qau>Sir W. Scott.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><syn><b>Syn.</b> -- Barbarism; impropriety; absurdity.</syn><br/
[<source>1913 Webster</source>]</p>

<p><ent>Solecist</ent><br/
<hw>Sol"e*cist</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Gr. <?/<?/<?/.]</ety> <def>One who commits a solecism.</def>  <rj><au>Blackwall.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Solecistic</ent><br/
<hw>Sol`e*cis"tic</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Solecistical.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Solecistical</ent><br/
<hw>Sol`e*cis"tic*al</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Pertaining to, or involving, a solecism; incorrect.</def> <ldquo/He thought it made the language <xex>solecistical</xex> and absurd.<rdquo/  <rj><au>Blackwall.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Solecistically</ent><br/
<hw>Sol`e*cis"tic*al*ly</hw>, <pos>adv.</pos> <def>In a solecistic manner.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Solecize</ent><br/
<hw>Sol"e*cize</hw> <pr>(?)</pr>, <pos>v. i.</pos> <ety>[Gr. <?/<?/<?/<?/<?/.]</ety> <def>To commit a solecism.</def> <mark>[R.]</mark>  <rj><au>Dr. H. More.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Solely</ent><br/
<hw>Sole"ly</hw> <pr>(?)</pr>, <pos>adv.</pos> <def>Singly; alone; only; without another; <as>as, to rest a cause <ex>solely</ex> one argument; to rely <ex>solely</ex>n one's own strength</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Solemn</ent><br/
<hw>Sol"emn</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[OE. <ets>solempne</ets>, OF. <ets>solempne</ets>, L. <ets>solemnis</ets>, <ets>solennis</ets>, <ets>sollemnis</ets>, <ets>sollennis</ets>; <ets>sollus</ets> all, entire + <ets>annus</ets> a year; properly, that takes place every year; -- used especially of religious solemnities.  Cf. <er>Silly</er>, <er>Annual</er>.]</ety> <sn>1.</sn> <def>Marked with religious rites and pomps; enjoined by, or connected with, religion; sacred.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>His holy rites and <qex>solemn</qex> feasts profaned.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>The worship of this image was advanced, and a <qex>solemn</qex> supplication observed everry year.</q> <rj><qau>Bp. Stillingfleet.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Pertaining to a festival; festive; festal.</def> <mark>[Obs.]</mark> <ldquo/On this <xex>solemn</xex> day.<rdquo/  <rj><au>Chaucer.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>Stately; ceremonious; grand.</def> <mark>[Archaic]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>His feast so <qex>solemn</qex> and so rich.</q> <rj><qau>Chaucer.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>To-night we hold a <qex>splemn</qex> supper.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>Fitted to awaken or express serious reflections; marked by seriousness; serious; grave; devout; <as>as, a <ex>solemn</ex> promise; <ex>solemn</ex> earnestness</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Nor wanting power to mitigate and swage<br/
With <qex>solemn</qex> touches troubled thoughts.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>There reigned a <qex>solemn</qex> silence over all.</q> <rj><qau>Spenser.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <def>Real; earnest; downright.</def> <mark>[Obs. & R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>Frederick, the emperor, . . . has spared no expense in strengthening this city; since which time we find no <qex>solemn</qex> taking it by the Turks.</q> <rj><qau>Fuller.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>6.</sn> <def>Affectedly grave or serious; <as>as, to put on a <ex>solemn</ex> face</as>.</def> <ldquo/A <xex>solemn</xex> coxcomb.<rdquo/  <rj><au>Swift.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>7.</sn> <fld>(Law)</fld> <def>Made in form; ceremonious; <as>as, <ex>solemn</ex> war</as>; conforming with all legal requirements; <as>as, probate in <ex>solemn</ex> form</as>.</def>  <rj><au>Burrill.</au>  <au>Jarman.</au>  <au>Greenleaf.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Solemn League and Covenant</b></col>. <cd>See <er>Covenant</er>, 2.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><syn><b>Syn.</b> -- Grave; formal; ritual; ceremonial; sober; serious; reverential; devotional; devout. See <er>Grave</er>.</syn><br/
[<source>1913 Webster</source>]</p>

<p><ent>Solemness</ent><br/
<hw>Sol"em*ness</hw> <pr>(?)</pr>, <pos>n.</pos> <def><er>Solemnness</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Some think he wanted <qex>solemnes</qex>.</q> <rj><qau>Sir H. Wotton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Solemnity</ent><br/
<hw>So*lem"ni*ty</hw> <pr>(?)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Solemnities</plw> <pr>(#)</pr>.</plu> <ety>[L. <ets>solemnitas</ets>, <ets>solennitas</ets>: cf. F. <ets>solennit<eacute/</ets>, <ets>solemnit<eacute/</ets>, OF. also <ets>sollempnit<eacute/</ets>.]</ety> <sn>1.</sn> <def>A rite or ceremony performed with religious reverence; religious or ritual ceremony; <as>as, the <ex>solemnity</ex> of a funeral, a sacrament</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Great was the cause; our old <qex>solemnities</qex><br/
From no blind zeal or fond tradition rise,<br/
But saved from death, our Argives yearly pay<br/
These grateful honors to the god of day.</q> <rj><qau>Pope.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>ceremony adapted to impress with awe.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The forms and <qex>solemnities</qex> of the last judgment.</q> <rj><qau>Atterburry.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>Ceremoniousness; impressiveness; seriousness; grave earnestness; formal dignity; gravity.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>With much glory and great <qex>solemnity</qex>.</q> <rj><qau>Chaucer.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>The statelines and gravity of the Spaniards shows itself in the <qex>solemnity</qex> of their language.</q> <rj><qau>Addison.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>These promises were often made with great <qex>solemnity</qex> and confirmed with an oath.</q> <rj><qau>J. Edwards.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>Hence, affected gravity or seriousness.</def><br/
[<source>1913 Webster</source>]</p>

<p><q><qex>Solemnity</qex> 's a cover for a sot.</q> <rj><qau>Young.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <def>Solemn state or feeling; awe or reverence; also, that which produces such a feeling; <as>as, the <ex>solemnity</ex> of an audience; the <ex>solemnity</ex> of Westminster Abbey</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>6.</sn> <fld>(Law)</fld> <def>A solemn or formal observance; proceeding according to due form; the formality which is necessary to render a thing done valid.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Solemnizate</ent><br/
<hw>So*lem"ni*zate</hw> <pr>(?)</pr>, <pos>v. t.</pos> <def>To solemnize; <as>as, to <ex>solemnizate</ex> matrimony</as>.</def> <mark>[R.]</mark>  <rj><au>Bp. Burnet.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Solemnization</ent><br/
<hw>Sol`em*ni*za"tion</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Cf. F. <ets>solemnisation</ets>, <ets>solennisation</ets>.]</ety> <def>The act of solemnizing; celebration; <as>as, the <ex>solemnization</ex> of a marriage</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Solemnize</ent><br/
<hw>Sol"em*nize</hw> <pr>(?)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Solemnized</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Solemnizing</conjf> <pr>(?)</pr>.]</vmorph> <ety>[Cf. F. <ets>solemniser</ets>, <ets>sollemniser</ets>.]</ety> <sn>1.</sn> <def>To perform with solemn or ritual ceremonies, or according to legal forms.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Baptism to be administered in one place, and marriage <qex>solemnized</qex> in another.</q> <rj><qau>Hooker.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To dignify or honor by ceremonies; to celebrate.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Their choice nobility and flowers . . . <br/
Met from all parts to <qex>solemnize</qex> this feast.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>To make grave, serious, and reverential.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Wordsworth was <qex>solemnizzed</qex> and elevated by this his first look on Yarrow.</q> <rj><qau>J. C. Shairp.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Every Israelite . . . arose, <qex>solemnized</qex> his face, looked towards Jerusalem . . . and prayed.</q> <rj><qau>L. Wallace.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Solemnize</ent><br/
<hw>Sol"em*nize</hw>, <pos>n.</pos> <def>Solemnization.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>Though spoused, yet wanting wedlock's <qex>solemnize</qex>.</q> <rj><qau>Spenser.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Solemnizer</ent><br/
<hw>Sol"em*ni`zer</hw> <pr>(?)</pr>, <pos>n.</pos> <def>One who solemnizes.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Solemnly</ent><br/
<hw>Sol"emn*ly</hw> <pr>(?)</pr>, <pos>adv.</pos> <def>In a solemn manner; with gravity; seriously; formally.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>There in deaf murmurs <qex>solemnly</qex> are wise.</q> <rj><qau>Dryden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>I do <qex>solemnly</qex> assure the reader.</q> <rj><qau>Swift.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Solemnness</ent><br/
<hw>Sol"emn*ness</hw>, <pos>n.</pos> <def>The state or quality of being solemn; solemnity; impressiveness; gravity; <as>as, the <ex>solemnness</ex> of public worship</as>.</def> <altsp>[Written also <asp>solemness</asp>.]</altsp><br/
[<source>1913 Webster</source>]</p>

<p><ent>Solempne</ent><br/
<hw>So*lemp"ne</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[See <er>Solemn</er>.]</ety> <def>Solemn; grand; stately; splendid; magnificent.</def> <mark>[Obs.]</mark>  <rj><au>Chaucer.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Solen</ent><br/
||<hw>So"len</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[NL., fr. Gr. <?/<?/<?/ channel, a shellfish.]</ety> <sn>1.</sn> <fld>(Med.)</fld> <def>A cradle, as for a broken limb. See <er>Cradle</er>, 6.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Zool.)</fld> <def>Any marine bivalve mollusk belonging to <gen>Solen</gen> or allied genera of the family <fam>Solenidae</fam>; a razor shell.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Solenacean</ent><br/
<hw>Sol`e*na"cean</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Zool.)</fld>. <def>Any species of marine bivalve shells belonging to the family <fam>Solenidae</fam>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Solenaceous</ent><br/
<hw>Sol`e*na"ceous</hw> <pr>(?)</pr>, <pos>a.</pos> <fld>(Zool.)</fld> <def>Of or pertaining to the solens or family <fam>Solenidae</fam>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Soleness</ent><br/
<hw>Sole"ness</hw> <pr>(?)</pr>, <pos>n.</pos> <def>The state of being sole, or alone; singleness.</def> <mark>[R.]</mark>  <rj><au>Chesterfield.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Solenette</ent><br/
<hw>Sole*nette"</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Zool.)</fld> <def>A small European sole (<spn>Solea minuta</spn>).</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Solenoconcha</ent><br/
||<hw>So*le`no*con"cha</hw> <pr>(?)</pr>, <pos>n. pl.</pos> <ety>[NL. See <er>Solen</er>, and <er>Conch</er>.]</ety> <fld>(Zool.)</fld> <def>Same as <er>Scaphopoda</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Solenodon</ent><br/
<hw>So*le"no*don</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Gr. <?/<?/<?/<?/ a channel + <?/<?/<?/<?/, <?/<?/<?/, a tooth.]</ety> <fld>(Zool.)</fld> <def>Either one of two species of singular West Indian insectivores, allied to the tenrec. One species (<spn>Solendon paradoxus</spn>), native of St. Domingo, is called also <stype>agouta</stype>; the other (<spn>Solendon Cubanus</spn>), found in Cuba, is called <stype>almique</stype>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Solenogastra</ent><br/
<hw>So*le`no*gas"tra</hw> <pr>(?)</pr>, <pos>n. pl.</pos> <ety>[NL., from Gr. <?/<?/<?/<?/ channel + <?/<?/<?/<?/, <?/<?/<?/<?/, stomach.]</ety> <fld>(Zool.)</fld> <def>An order of lowly organized Mollusca belonging to the Isopleura. A narrow groove takes the place of the foot of other gastropods.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Solenoglyph</ent><br/
<hw>So*le"no*glyph</hw> <pr>(?)</pr>, <pos>a.</pos> <fld>(Zool.)</fld> <def>Pertaining to the Selenoglypha. See <er>Ophidia</er>.</def> -- <def2><pos>n.</pos> <def>One of the Selenoglypha.</def></def2><br/
[<source>1913 Webster</source>]</p>

<p><ent>Solenoglypha</ent><br/
||<hw>So`le*nog"ly*pha</hw> <pr>(?)</pr>, <pos>n. pl.</pos> <ety>[NL., fr. Gr. <?/<?/<?/<?/ a channel + <?/<?/<?/<?/ to engrave.]</ety> <fld>(Zool.)</fld> <def>A suborder of serpents including those which have tubular erectile fangs, as the viper and rattlesnake. See <er>Fang</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Solenoid</ent><br/
<hw>So"len*oid</hw> <pr>(?)</pr>, <pos>n.</pos><ety>[Gr. <?/<?/<?/ channel + <ets>-oid</ets>.]</ety> <fld>(Elec.)</fld> <def>An electrodynamic spiral having the conjuctive wire turned back along its axis, so as to neutralize that component of the effect of the current which is due to the length of the spiral, and reduce the whole effect to that of a series of equal and parallel circular currents. When traversed by a current the solenoid exhibits polarity and attraction or repulsion, like a magnet.</def><br/
[<source>1913 Webster</source>]</p>

<p><-- 2. a switch or valve using such a solenoid circuit to drive a metal bar which opens or closes an electric circuit or a valve controlling fluid movement.
 --><br/
[<source>1913 Webster</source>]</p>

<p><ent>Solenostomi</ent><br/
||<hw>So`le*nos"to*mi</hw> <pr>(?)</pr>, <pos>n. pl.</pos> <ety>[NL., from Gr. <?/<?/<?/<?/<?/ a channel + <?/<?/<?/<?/ a mouth.]</ety> <fld>(Zool.)</fld> <def>A tribe of lophobranch fishes having a tubular snout. The female carries the eggs in a ventral pouch.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Soleplate</ent><br/
<hw>Sole"plate`</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Mach.)</fld> <sd>(a)</sd> <def>A bedplate; <as>as, the <ex>soleplate</ex> of a steam engine</as>.</def> <sd>(b)</sd> <def>The plate forming the back of a waterwheel bucket.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Solere</ent><br/
<ent>Soler</ent><br/
<mhw>{ <hw>So"ler</hw> <pr>(?)</pr>, <hw>So"lere</hw> <pr>(?)</pr>, }</mhw> <pos>n.</pos> <ety>[OE. See <er>Solar</er>, <pos>n.</pos>]</ety> <def>A loft or garret. See <er>Solar</er>, <pos>n.</pos></def>  <rj><au>Sir W. Scott.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Solert</ent><br/
<hw>So"lert</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>solers</ets>, <ets>sollers</ets>, <ets>-ertis</ets>,clever, skillful.]</ety> <def>Skillful; clever; crafty.</def> <mark>[Obs.]</mark>  <rj><au>Cudworth.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Solertiousness</ent><br/
<hw>So*ler"tious*ness</hw> <pr>(?)</pr>, <pos>n.</pos> <def>The quality or state of being solert.</def> <mark>[Obs.]</mark>  <rj><au>Bp. Hacket.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Soleship</ent><br/
<hw>Sole"ship</hw> <pr>(?)</pr>, <pos>n.</pos> <def>The state of being sole, or alone; soleness.</def> <mark>[R.]</mark>  <rj><au>Sir E. Dering.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sole trader</ent><br/
<hw>Sole trader</hw>. <def>A feme sole trader.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Sol-fa</ent><br/
<hw>Sol`-fa"</hw> <pr>(?)</pr>, <pos>v. i.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Sol-faed</conjf> <pr>(?)</pr>;<pos>p. pr. & vb. n.</pos> <conjf>Sol-faing</conjf>.]</vmorph> <ety>[It. <ets>solfa</ets> the gamut, from the syllables <ets>fa</ets>, <ets>sol</ets>.]</ety> <def>To sing the notes of the gamut, ascending or descending; <as>as, <ex>do</ex> or <ex>ut</ex>, <ex>re</ex>, <ex>mi</ex>, <ex>fa</ex>, <ex>sol</ex>, <ex>la</ex>, <ex>si</ex>, <ex>do</ex>, or the same in reverse order</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Yet can I neither <qex>solfe</qex> ne sing.</q> <rj><qau>Piers Plowman.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sol-fa</ent><br/
<hw>Sol`-fa"</hw> <pr>(?)</pr>, <pos>v. t.</pos> <def>To sing to solmization syllables.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Sol-fa</ent><br/
<hw>Sol"-fa"</hw>, <pos>n.</pos> <def>The gamut, or musical scale. See <cref>Tonic sol-fa</cref>, under <er>Tonic</er>, <pos>n.</pos></def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Solfanaria</ent><br/
<hw>Sol`fa*na"ri*a</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[It., from <ets>solfo</ets> sulphur.]</ety> <def>A sulphur mine.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Solfatara</ent><br/
||<hw>Sol`fa*ta"ra</hw> <pr>(?)</pr>, <pos>n.</pos><ety>[It., from <ets>solfo</ets> brimstone, sulphur, L. <ets>sulfur</ets>, E. <ets>sulphur</ets>.]</ety> <fld>(Geol.)</fld> <def>A volcanic area or vent which yields only sulphur vapors, steam, and the like. It represents the stages of the volcanic activity.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Solfeggiare</ent><br/
||<hw>Sol`feg*gia"re</hw> <pr>(?)</pr>, <pos>v. i.</pos><ety>[It.]</ety> <fld>(Mus.)</fld> <def>To sol-fa. See <er>Sol-fa</er>, <pos>v. i.</pos></def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Solfeggio</ent><br/
||<hw>Sol*feg"gio</hw> <pr>(?)</pr>, <pos>n.</pos><ety>[It., fr. <ets>solfa</ets> the gamut.]</ety> <fld>(Mus.)</fld> <def>The system of arranging the scale by the names <xex>do</xex>, <xex>re</xex>, <xex>mi</xex>, <xex>fa</xex>, <xex>sol</xex>, <xex>la</xex>, <xex>si</xex>, by which singing is taught; a singing exercise upon these syllables.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Solferino</ent><br/
<hw>Sol`fe*ri"no</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A brilliant deep pink color with a purplish tinge, one of the dyes derived from aniline; -- so called from Solferino in Italy, where a battle was fought about the time of its discovery.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Soli</ent><br/
||<hw>So"li</hw> <pr>(?)</pr>, <pos>n.</pos>, <def><pos>pl.</pos> of <er>Solo</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Solicit</ent><br/
<hw>So*lic"it</hw> <pr>(?)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Solicited</conjf>; <pos>p. pr. & vb. n.</pos> <conjf>Soliciting</conjf>.]</vmorph> <ety>[F. <ets>sollicier</ets>, L. <ets>sollicitare</ets>, <ets>solicitare</ets>, <ets>-atum</ets>, fr. <ets>sollicitus</ets> wholly (<it>i. e.</it>, violently) moved; <ets>sollus</ets> whole + <ets>citus</ets>, p. p. of <ets>ciere</ets> to move, excite. See <er>Solemn</er>, <er>Cite</er>.]</ety> <sn>1.</sn> <def>To ask from with earnestness; to make petition to; to apply to for obtaining something; <as>as, to <ex>solicit</ex> person for alms</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Did I <qex>solicit</qex> thee<br/
From darkness to promote me?</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To endeavor to obtain; to seek; to plead for; <as>as, to <ex>solicit</ex> an office; to <ex>solicit</ex> a favor</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>I view my crime, but kindle at the view,<br/
Repent old pleasures, and <qex>solicit</qex> new.</q> <rj><qau>Pope.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>To awake or excite to action; to rouse desire in; to summon; to appeal to; to invite.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>That fruit . . . <qex>solicited</qex> her longing eye.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Sounds and some tangible qualities <qex>solicit</qex> their proper senses, and force an entrance to the mind.</q> <rj><qau>Locke.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>To urge the claims of; to plead; to act as solicitor for or with reference to.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>Should<br/
My brother henceforth study to forget<br/
The vow that he hath made thee, I would ever<br/
<qex>Solicit</qex> thy deserts.</q> <rj><qau>Ford.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <def>To disturb; to disquiet; -- a Latinism rarely used.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Hath any ill <qex>solicited</qex> thine ears?</q> <rj><qau>Chapman.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>But anxious fears <qex>solicit</qex> my weak breast.</q> <rj><qau>Dryden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><syn><b>Syn.</b> To beseech; ask; request; crave; supplicate; entreat; beg; implore; importune. See <er>Beseech</er>.</syn><br/
[<source>1913 Webster</source>]</p>

<p><ent>Solicitant</ent><br/
<hw>So*lic"it*ant</hw> <pr>(?)</pr>, <pos>n.</pos><ety>[L. <ets>sollicitans</ets>, p. pr. ]</ety> <def>One who solicits.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Solicitate</ent><br/
<hw>So*lic"it*ate</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Solicitous.</def> <mark>[Obs.]</mark>  <rj><au>Eden.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Solicitation</ent><br/
<hw>So*lic`i*ta"tion</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F. <ets>sollicitation</ets>, or L. <ets>sollicitatio</ets>.]</ety> <sn>1.</sn> <def>The act of soliciting; earnest request; persistent asking; importunity.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Excitement; invitation; <as>as, the <ex>solicitation</ex> of the senses</as>.</def>  <rj><au>Locke.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Solicitor</ent><br/
<hw>So*lic"it*or</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F. <ets>solliciteur</ets>, L. <ets>sollicitator</ets>.]</ety> <sn>1.</sn> <def>One who solicits.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Law)</fld> <sd>(a)</sd> <def>An attorney or advocate; one who represents another in court; -- formerly, in English practice, the professional designation of a person admitted to practice in a court of chancery or equity. See the Note under <er>Attorney</er>.</def> <sd>(b)</sd> <def>The law officer of a city, town, department, or government; <as>as, the city <ex>solicitor</ex>; the <ex>solicitor</ex> of the treasury.</as></def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Solicitor-general</ent><br/
<hw>So*lic"it*or-gen"er*al</hw> <pr>(?)</pr>, <pos>n.</pos> <def>The second law officer in the government of Great Britain; also, a similar officer under the United States government, who is associated with the attorney-general; also, the chief law officer of some of the States.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Solicitous</ent><br/
<hw>So*lic"it*ous</hw> <pr>(?)</pr>, <pos>a.</pos><ety>[L. <ets>sollicitus</ets>, <ets>solicitus</ets>. See <er>Solicit</er>, <pos>v. t.</pos>]</ety> <def>Disposed to solicit; eager to obtain something desirable, or to avoid anything evil; concerned; anxious; careful.</def> <ldquo/<xex>Solicitous</xex> of my reputation.<rdquo/ <au>Dryden.</au> <ldquo/He was <xex>solicitous</xex> for his advice.<rdquo/ <au>Calerendon.</au><br/
[<source>1913 Webster</source>]</p>

<p><q>Enjoy the present, whatsoever it be, and be not <qex>solicitous</qex> about the future.</q> <rj><qau>Jer. Taylor.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>The colonel had been intent upon other things, and not enough <qex>solicitous</qex> to finish the fortifications.</q> <rj><qau>Clarendon.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p>-- <wordforms><wf>So*lic"it*ous*ly</wf>, <pos>adv.</pos> -- <wf>So*lic"it*ous*ness</wf>, <pos>n.</pos></wordforms><br/
[<source>1913 Webster</source>]</p>

<p><ent>Solicitress</ent><br/
<hw>So*lic"it*ress</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A woman who solicits.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Solicitude</ent><br/
<hw>So*lic"i*tude</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F. <ets>sollicitude</ets>,r L. <ets>sollicitudo</ets>.]</ety> <def>The state of being solicitous; uneasiness of mind occasioned by fear of evil or desire good; anxiety.</def>
<-- used now mainly of concern for the well-being of others --><br/
[<source>1913 Webster</source>]</p>

<p><q>The many cares and great labors of worldly men, their <qex>solicitude</qex> and outward shows.</q> <rj><qau>Sir W. Raleigh.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>The mother looked at her with fond <qex>solicitude</qex>.</q> <rj><qau>G. W. Cable.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><syn><b>Syn.</b> -- Carefulness; concern; anxiety. See <er>Care</er>.</syn><br/
[<source>1913 Webster</source>]</p>

<p><-- p. 1369 --></p>

<p><ent>Solid</ent><br/
<hw>Sol"id</hw> <pr>(s<ocr/l"<icr/d)</pr>, <pos>a.</pos> <ety>[L. <ets>solidus</ets>, probably akin to <ets>sollus</ets> whole, entire, Gr. <?/<?/<?/: cf. F. <ets>solide</ets>.  Cf. <er>Consolidate</er>,<er>Soda</er>, <er>Solder</er>, <er>Soldier</er>, <er>Solemn</er>.]</ety> <sn>1.</sn> <def>Having the constituent parts so compact, or so firmly adhering, as to resist the impression or penetration of other bodies; having a fixed form; hard; firm; compact; -- opposed to <contr>fluid</contr> and <contr>liquid</contr> or to <contr>plastic</contr>, like clay, or to <contr>incompact</contr>, like sand.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Not hollow; full of matter; <as>as, a <ex>solid</ex> globe or cone, as distinguished from a <ant>hollow</ant> one</as>; not spongy; dense; hence, sometimes, heavy.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <fld>(Arith.)</fld> <def>Having all the geometrical dimensions; cubic; <as>as, a <ex>solid</ex> foot contains 1,728 <ex>solid</ex> inches</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ In this sense, <xex>cubic</xex>s now generally used.</note><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>Firm; compact; strong; stable; unyielding; <as>as, a <ex>solid</ex> pier; a <ex>solid</ex> pile; a <ex>solid</ex> wall.</as></def><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <def>Applied to a compound word whose parts are closely united and form an unbroken word; -- opposed to <contr>hyphened</contr>.</def><-- unhyphenated, ligated? fused? --><br/
[<source>1913 Webster</source>]</p>

<p><sn>6.</sn> <def>Fig.: Worthy of credit, trust, or esteem; substantial, as opposed to <contr>frivolous</contr> or <contr>fallacious</contr>; weighty; firm; strong; valid; just; genuine.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The <qex>solid</qex> purpose of a sincere and virtuous answer.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>These, wanting wit, affect gravity, and go by the name of <qex>solid</qex> men.</q> <rj><qau>Dryden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>The genius of the Italians wrought by <qex>solid</qex> toil what the myth-making imagination of the Germans had projected in a poem.</q> <rj><qau>J. A. Symonds.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>7.</sn> <def>Sound; not weakly; <as>as, a <ex>solid</ex> constitution of body</as>.</def>  <rj><au>I. Watts.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>8.</sn> <fld>(Bot.)</fld> <def>Of a fleshy, uniform, undivided substance, as a bulb or root; not spongy or hollow within, as a stem.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>9.</sn> <fld>(Metaph.)</fld> <def>Impenetrable; resisting or excluding any other material particle or atom from any given portion of space; -- applied to the supposed ultimate particles of matter.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>10.</sn> <fld>(Print.)</fld> <def>Not having the lines separated by leads; not open.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>11.</sn> <def>United; without division; unanimous; <as>as, the delegation is <ex>solid</ex> for a candidate</as>.</def> <mark>[Polit. Cant. U.S.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Solid angle</b></col>. <fld>(Geom.)</fld> <cd>See under <er>Angle</er>.</cd> -- <col><b>Solid color</b></col>, <cd>an even color; one not shaded or variegated.</cd> -- <col><b>Solid green</b></col>. <cd>See <cref>Emerald green</cref> <sd>(a)</sd>, under <er>Green</er>.</cd> -- <col><b>Solid measure</b></col> <fld>(Arith.)</fld>, <cd>a measure for volumes, in which the units are each a cube of fixed linear magnitude, as a cubic foot, yard, or the like; thus, a foot, in <xex>solid measure</xex>, or a <xex>solid foot</xex>, contains 1,728 solid inches.</cd> -- <col><b>Solid newel</b></col> <fld>(Arch.)</fld>, <cd>a newel into which the ends of winding stairs are built, in distinction from a <xex>hollow newel</xex>. See under <er>Hollow</er>, <pos>a.</pos></cd> -- <col><b>Solid problem</b></col> <fld>(Geom.)</fld>, <cd>a problem which can be construed geometrically, only by the intersection of a circle and a conic section or of two conic sections.</cd> <au>Hutton.</au> -- <col><b>Solid square</b></col> <fld>(Mil.)</fld>, <cd>a square body or troops in which the ranks and files are equal.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><syn><b>Syn.</b> -- Hard; firm; compact; strong; substantial; stable; sound; real; valid; true; just; weighty; profound; grave; important.</syn>  -- <usage><er>Solid</er>, <er>Hard</er>. These words both relate to the internal constitution of bodies; but <xex>hard</xex>notes a more impenetrable nature or a firmer adherence of the component parts than <xex>solid</xex>. <xex>Hard</xex> is opposed to <xex>soft</xex>, and <xex>solid</xex> to <xex>fluid</xex>, <xex>liquid</xex>, <xex>open</xex>, or <xex>hollow</xex>. Wood is usually <xex>solid</xex>; but some kinds of wood are <xex>hard</xex>, and others are <xex>soft</xex>.</usage><br/
[<source>1913 Webster</source>]</p>

<p><q>Repose you there; while I [return] to this <qex>hard</qex> house,<br/
More <qex>harder</qex> than the stones whereof 't is raised.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>I hear his thundering voice resound,<br/
And trampling feet than shake the <qex>solid</qex> ground.</q> <rj><qau>Dryden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Solid</ent><br/
<hw>Sol"id</hw>, <pos>n.</pos> <sn>1.</sn> <def>A substance that is held in a fixed form by cohesion among its particles; a substance not fluid.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Geom.)</fld> <def>A magnitude which has length, breadth, and thickness; a part of space bounded on all sides.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Solid of revolution</b></col>. <fld>(Geom.)</fld> <cd>See <er>Revolution</er>, <pos>n.</pos>, 5.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Solidago</ent><br/
||<hw>Sol`i*da"go</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[NL., fr. L. <ets>solidare</ets> to strengthen, unite; -- so called in allusion to its reputed healing qualities.]</ety> <fld>(Bot.)</fld> <def>A genus of yellow-flowered composite perennial herbs; golden-rod.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Solidare</ent><br/
<hw>Sol"i*dare</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[LL. <ets>solidus</ets>.  Cf. <er>Sou</er>.]</ety> <def>A small piece of money.</def> <mark>[Obs.]</mark>  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Solidarity</ent><br/
<hw>Sol`i*dar"i*ty</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F. <ets>solidarit<eacute/</ets>, fr. <ets>solide</ets>. See <er>Solid</er>.]</ety> <def>An entire union or consolidation of interests and responsibilities; fellowship; community.</def><br/
[<source>1913 Webster</source>]</p>

<p><q><qex>Solidarity</qex> [a word which we owe to the French Communists], signifies a fellowship in gain and loss, in honor and dishonor, in victory and defeat, a being, so to speak, all in the same boat.</q> <rj><qau>Trench.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>The <qex>solidarity</qex> . . . of Breton and Welsh poetry.</q> <rj><qau>M. Arnold.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Solidary</ent><br/
<hw>Sol"i*da*ry</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Having community of interests and responsibilities.</def><br/
[<source>1913 Webster</source>]</p>

<p><q> Men are <qex>solidary</qex>, or copartners; and not isolated.</q> <rj><qau>M. Arnold.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Solidate</ent><br/
<hw>Sol"i*date</hw> <pr>(?)</pr>, <pos>v. t.</pos> <ety>[L. <ets>solidatus</ets>, p. p. of <ets>solidare</ets>. See <er>Solder</er>.]</ety> <def>To make solid or firm.</def> <mark>[Obs.]</mark>  <rj><au>Cowley.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Solid-drawn</ent><br/
<hw>Sol"id-drawn`</hw>, <pos>a.</pos> <def>Drawn out from a heated solid bar, as by a process of spiral rolling which first hollows the bar and then expands the cavity by forcing the bar over a pointed mandrel fixed in front of the rolls; -- said of a weldless tube.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Solidifiable</ent><br/
<hw>So*lid"i*fi`a*ble</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Capable of being solidified.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Solidification</ent><br/
<hw>So*lid`i*fi*ca"tion</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Cf. F. <ets>solidification</ets>.]</ety> <def>Act of solidifying, or state of being solidified.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Solidify</ent><br/
<hw>So*lid"i*fy</hw> <pr>(?)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Solidified</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Solidifying</conjf> <pr>(?)</pr>.]</vmorph> <ety>[<ets>Solid</ets> + <ets>-fy</ets>: cf. F. <ets>solidifier</ets>.]</ety> <def>To make solid or compact.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Every machine is a <qex>solidified</qex> mechanical theorem.</q> <rj><qau>H. Spencer.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Solidify</ent><br/
<hw>So*lid"i*fy</hw>, <pos>v. i.</pos> <def>To become solid; to harden.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Solidism</ent><br/
<hw>Sol"id*ism</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Med.)</fld> <def>The doctrine that refers all diseases to morbid changes of the solid parts of the body. It rests on the view that the solids alone are endowed with vital properties, and can receive the impression of agents tending to produce disease.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Solidist</ent><br/
<hw>Sol"id*ist</hw>, <pos>n.</pos> <fld>(Med.)</fld> <def>An advocate of, or believer in, solidism.</def>  <rj><au>Dunglison.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Solidity</ent><br/
<hw>So*lid"i*ty</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>soliditas</ets>: cf. F. <ets>solidit<eacute/</ets>.]</ety> <sn>1.</sn> <def>The state or quality of being solid; density; consistency, -- opposed to <contr>fluidity</contr>; compactness; fullness of matter, -- opposed to <contr>openness</contr> or <contr>hollowness</contr>; strength; soundness, -- opposed to <contr>weakness</contr> or <contr>instability</contr>; the primary quality or affection of matter by which its particles exclude or resist all others; hardness; massiveness.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>That which hinders the approach of two bodies when they are moving one toward another, I call <qex>solidity</qex>.</q> <rj><qau>Locke.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Moral firmness; soundness; strength; validity; truth; certainty; -- as opposed to <contr>weakness</contr> or <contr>fallaciousness</contr>; <as>as, the <ex>solidity</ex> of arguments or reasoning; the <ex>solidity</ex> of principles, triuths, or opinions</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <fld>(Geom.)</fld> <def>The solid contents of a body; volume; amount of inclosed space.</def><br/
[<source>1913 Webster</source>]</p>

<p><syn><b>Syn.</b> -- Firmness; solidness; hardness; density; compactness; strength; soundness; validity; certainty.</syn><br/
[<source>1913 Webster</source>]</p>

<p><ent>Solidly</ent><br/
<hw>Sol"id*ly</hw> <pr>(?)</pr>, <pos>adv.</pos> <def>In a solid manner; densely; compactly; firmly; truly.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Solidness</ent><br/
<hw>Sol"id*ness</hw>, <pos>n.</pos> <sn>1.</sn> <def>State or quality of being solid; firmness; compactness; solidity, as of material bodies.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Soundness; strength; truth; validity, as of arguments, reasons, principles, and the like.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Solidungula</ent><br/
||<hw>Sol`id*un"gu*la</hw> <pr>(?)</pr>, <pos>n. pl.</pos> <ety>[NL., from L. <ets>solidus</ets> solid + <ets>ungula</ets> a hoof.]</ety> <fld>(Zool.)</fld> <def>A tribe of ungulates which includes the horse, ass, and related species, constituting the family <fam>Equidae</fam>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Solidungular</ent><br/
<hw>Sol`id*un"gu*lar</hw> <pr>(?)</pr>, <pos>a.</pos> <fld>(Zool.)</fld> <def>Solipedous.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Solidungulate</ent><br/
<hw>Sol`id*un"gu*late</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[<ets>Solid</ets> + <ets>ungulate</ets>.]</ety> <fld>(Zool.)</fld> <def>Same as <er>Soliped</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Solidungulous</ent><br/
<hw>Sol`id*un"gu*lous</hw> <pr>(?)</pr>, <pos>a.</pos> <fld>(Zool.)</fld> <def>Solipedous.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Solifidian</ent><br/
<hw>Sol`i*fid"i*an</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>solus</ets> alone + <ets>fides</ets> faith.]</ety> <fld>(Eccl.)</fld> <def>One who maintains that faith alone, without works, is sufficient for justification; -- opposed to <contr>nullifidian</contr>.</def>  <rj><au>Hammond.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Solifidian</ent><br/
<hw>Sol`i*fid"i*an</hw>, <pos>a.</pos> <def>Holding the tenets of Solifidians; of or pertaining to the solifidians.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Solifidianism</ent><br/
<hw>Sol`i*fid"i*an*ism</hw>, <pos>n.</pos> <def>The state of Solifidians.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Soliform</ent><br/
<hw>Sol"i*form</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>sol</ets> sun + <ets>-form</ets>.]</ety> <def>Like the sun in form, appearance, or nature; resembling the sun.</def> <mark>[R.]</mark> <ldquo/<xex>Soliform</xex> things.<rdquo/  <rj><au>Cudworth.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Solifugae</ent><br/
||<hw>So*lif"u*gae</hw> <pr>(?)</pr>, <pos>n. pl.</pos> <ety>[NL., from L. <ets>solifuga</ets> (better <ets>solipuga</ets>), a kind of venomous ant, or spider.]</ety> <fld>(Zool.)</fld> <def>A division of arachnids having large, powerful fangs and a segmented abdomen; -- called also <altname>Solpugidea</altname>, and <altname>Solpugides</altname>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Soliloquize</ent><br/
<hw>So*lil"o*quize</hw> <pr>(?)</pr>, <pos>v. i.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Soliloquized</conjf> <pr>(#)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Soliloquizing</conjf> <pr>(#)</pr>.]</vmorph> <def>To utter a soliloquy; to talk to one's self.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Soliloquy</ent><br/
<hw>So*lil"o*quy</hw> <pr>(?)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Soliloquies</plw> <pr>(#)</pr>.</plu> <ety>[L. <ets>soliloquium</ets>; <ets>solus</ets> alone + <ets>loqui</ets> to speak. See <er>Sole</er> ly, and <er>Loquacious</er>.]</ety> <sn>1.</sn> <def>The act of talking to one's self; a discourse made by one in solitude to one's self; monologue.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Lovers are always allowed the comfort of <qex>soliloquy</qex>.</q> <rj><qau>Spectator.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A written composition, reciting what it is supposed a person says to himself.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The whole poem is a <qex>soliloquy</qex>.</q> <rj><qau>Prior.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Soliped</ent><br/
<hw>Sol"i*ped</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Cf. F. <ets>solip<egrave/de</ets>, It. <ets>solipede</ets>, Sp. <ets>solipedo</ets>; apparently fr. L. <ets>solus</ets> alone + <ets>pes</ets>, <ets>pedis</ets>, a foot; but probably fr. L. <ets>solidipes</ets> solid-footed, whole-hoofed. See <er>Solid</er>, and <er>Pedal</er>.]</ety> <fld>(Zool.)</fld> <def>A mammal having a single hoof on each foot, as the horses and asses; a solidungulate.</def> <altsp>[Written also <asp>solipede</asp>.]</altsp><br/
[<source>1913 Webster</source>]</p>

<p><q>The <qex>solipeds</qex>, or firm-hoofed animals, as horses, asses, and mules, etc., -- they are, also, in mighty number.</q> <rj><qau>Sir T. Browne.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Solipedous</ent><br/
<hw>So*lip"e*dous</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Having single hoofs.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Solipsism</ent><br/
<hw>So*lip"sism</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>solus</ets> alone + <ets>ipse</ets> self.]</ety> <sn>1.</sn> <fld>(Ethics)</fld> <def>Egotism.</def>  <rj><au>Krauth-Fleming.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Metaph.)</fld> <def>Egoism.</def>  <rj><au>Krauth-Fleming.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Solisequious</ent><br/
<hw>Sol`i*se"qui*ous</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>sol</ets> sun + <ets>sequi</ets> to follow.]</ety> <def>Following the course of the sun; <as>as, <ex>solisequious</ex> plants</as>.</def> <mark>[R.]</mark>  <rj><au>Sir T. Browne.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Solitaire</ent><br/
<hw>Sol`i*taire"</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F. See <er>Solitary</er>.]</ety> <sn>1.</sn> <def>A person who lives in solitude; a recluse; a hermit.</def>  <rj><au>Pope.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A single diamond in a setting; also, sometimes, a precious stone of any kind set alone.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Diamond <qex>solitaires</qex> blazing on his breast and wrists.</q> <rj><qau>Mrs. R. H. Davis.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>A game which one person can play alone; -- applied to many games of cards, etc.; also, to a game played on a board with pegs or balls, in which the object is, beginning with all the places filled except one, to remove all but one of the pieces by <ldquo/jumping,<rdquo/ as in draughts.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <fld>(Zool.)</fld> <sd>(a)</sd> <def>A large extinct bird (<spn>Pezophaps solitaria</spn>) which formerly inhabited the islands of Mauritius and Rodrigeuz. It was larger and taller than the wild turkey. Its wings were too small for flight. Called also <altname>solitary</altname>.</def> <sd>(b)</sd> <def>Any species of American thrushlike birds of the genus <gen>Myadestes</gen>. They are noted their sweet songs and retiring habits. Called also <altname>fly-catching thrush</altname>. A West Indian species (<spn>Myadestes sibilans</spn>) is called the <altname>invisible bird</altname>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Solitarian</ent><br/
<hw>Sol`i*ta"ri*an</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[See <er>Solitary</er>.]</ety> <def>A hermit; a solitary.</def> <mark>[Obs.]</mark>  <rj><au>Sir R. Twisden.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Solitariety</ent><br/
<hw>Sol`i*ta*ri"e*ty</hw> <pr>(?)</pr>, <pos>n.</pos> <def>The state of being solitary; solitariness.</def> <mark>[Obs.]</mark>  <rj><au>Cudworth.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Solitarily</ent><br/
<hw>Sol"i*ta*ri*ly</hw> <pr>(?)</pr>, <pos>adv.</pos> <def>In a solitary manner; in solitude; alone.</def>  <rj><au>Mic. vii. 14.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Solitariness</ent><br/
<hw>Sol"i*ta*ri*ness</hw>, <pos>n.</pos> <def>Condition of being solitary.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Solitary</ent><br/
<hw>Sol"i*ta*ry</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>solitarius</ets>, fr. <ets>solus</ets> alone: cf. F. <ets>solitaire</ets>. See <er>Sole</er>, <pos>a.</pos>, and cf. <er>Solitaire</er>.]</ety> <sn>1.</sn> <def>Living or being by one's self; having no companion present; being without associates; single; alone; lonely.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Those rare and <qex>solitary</qex>, these in flocks.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Hie home unto my chamber,<br/
Where thou shalt find me, sad and <qex>solitary</qex>.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Performed, passed, or endured alone; <as>as, a <ex>solitary</ex> journey; a <ex>solitary</ex> life</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Satan . . . explores his <qex>solitary</qex> flight.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>Not much visited or frequented; remote from society; retired; lonely; <as>as, a <ex>solitary</ex> residence or place</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>Not inhabited or occupied; without signs of inhabitants or occupation; desolate; deserted; silent; still; hence, gloomy; dismal; <as>as, the <ex>solitary</ex> desert</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>How doth the city sit <qex>solitary</qex>, that was full of people.</q> <rj><qau>Lam. i. 1.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Let that night be <qex>solitary</qex>; let no joyful voice come therein.</q> <rj><qau>Job iii. 7.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <def>Single; individual; sole; <as>as, a <ex>solitary</ex> instance of vengeance; a <ex>solitary</ex> example</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>6.</sn> <fld>(Bot.)</fld> <def>Not associated with others of the same kind.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Solitary ant</b></col> <fld>(Zool.)</fld>, <cd>any solitary hymenopterous insect of the family <fam>Mutillidae</fam>. The female of these insects is destitute of wings and has a powerful sting. The male is winged and resembles a wasp. Called also <altname>spider ant</altname>.</cd> -- <col><b>Solitary bee</b></col> <fld>(Zool.)</fld>, <cd>any species of bee which does not form communities.</cd> -- <col><b>Solitary sandpiper</b></col> <fld>(Zool.)</fld>, <cd>an American tattler (<spn>Totanus solitarius</spn>).</cd> -- <col><b>Solitary snipe</b></col> <fld>(Zool.)</fld>, <cd>the great snipe.</cd> <mark>[Prov. Eng.]</mark> -- <col><b>Solitary thrush</b></col> <fld>(Zool.)</fld> <cd>the starling.</cd> <mark>[Prov. Eng.]</mark></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Solitary</ent><br/
<hw>Sol"i*ta*ry</hw> <pr>(?)</pr>, <pos>n.</pos> <def>One who lives alone, or in solitude; an anchoret; a hermit; a recluse.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Solitude</ent><br/
<hw>Sol"i*tude</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F., from L. <ets>solitudo</ets>, <ets>solus</ets> alone. See <er>Sole</er>, <pos>a.</pos>]</ety> <sn>1.</sn> <def>state of being alone, or withdrawn from society; a lonely life; loneliness.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Whosoever is delighted with <qex>solitude</qex> is either a wild beast or a god.</q> <rj><qau>Bacon.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>O <qex>Solitude</qex>! where are the charms<br/
That sages have seen in thy face?</q> <rj><qau>Cowper.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Remoteness from society; destitution of company; seclusion; -- said of places; <as>as, the <ex>solitude</ex> of a wood</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The <qex>solitude</qex> of his little parish is become matter of great comfort to him.</q> <rj><qau>Law.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>solitary or lonely place; a desert or wilderness.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>In these deep <qex>solitudes</qex> and awful cells<br/
Where heavenly pensive contemplation dwells.</q> <rj><qau>Pope.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><syn><b>Syn.</b> Loneliness; soitariness; loneness; retiredness; recluseness. -- <er>Solitude</er>, <er>Retirement</er>, <er>Seclusion</er>, <er>Loneliness</er>.</syn> <usage><xex>Retirement</xex> is a withdrawal from general society, implying that a person has been engaged in its scenes. <xex>Solitude</xex> describes the fact that a person is alone; <xex>seclusion</xex>, that he is shut out from others, usually by his own choice; <xex>loneliness</xex>, that he feels the pain and oppression of being alone. Hence, <xex>retirement</xex> is opposed to a gay, active, or public life; <xex>solitude</xex>, to society; <xex>seclusion</xex>, to freedom of access on the part of others; and <xex>loneliness</xex>, enjoyment of that society which the heart demands.</usage><br/
[<source>1913 Webster</source>]</p>

<p><q>O blest <qex>retirement</qex>, friend to life's decline.</q> <rj><qau>Goldsmith.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Such only can enjoy the country who are capable of thinking when they are there; then they are prepared for <qex>solitude</qex>; and in that [the country] <qex>solitude</qex> is prepared for them.</q> <rj><qau>Dryden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>It is a place of <qex>seclusion</qex> from the external world.</q> <rj><qau>Bp. Horsley.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>These evils . . . seem likely to reduce it [a city] ere long to the <qex>loneliness</qex> and the insignificance of a village.</q> <rj><qau>Eustace.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Solivagant</ent><br/
<hw>So*liv"a*gant</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>solus</ets> alone + <ets>vagans</ets> wandering.]</ety> <def>Wandering alone.</def> <mark>[R.]</mark>  <rj><au>T. Grander.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Solivagous</ent><br/
<hw>So*liv"a*gous</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>solivagus</ets>.]</ety> <def>Solivagant.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sollar</ent><br/
<hw>Sol"lar</hw> <pr>(?)</pr>, <pos>n.</pos> <sn>1.</sn> <def>See <er>Solar</er>, <pos>n.</pos></def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Mining)</fld> <def>A platform in a shaft, especially one of those between the series of ladders in a shaft.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sollar</ent><br/
<hw>Sol"lar</hw>, <pos>v. t.</pos> <def>To cover, or provide with, a sollar.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sollein</ent><br/
<hw>Sol"lein</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Sullen; sad.</def> <mark>[Obs.]</mark>  <rj><au>Spenser.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Solleret</ent><br/
<hw>Sol*ler*et"</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F. <ets>soleret</ets>im. fr. OF. <ets>soler</ets> shoe.]</ety> <def>A flexible steel shoe (or one of the plates forming such a shoe), worn with mediaeval armor.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Solmization</ent><br/
<hw>Sol`mi*za"tion</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F. <ets>solmisation</ets>, fr. <ets>solmiser</ets> to sol-fa; -- called from the musical notes <ets>sol</ets>, <ets>mi</ets>. See <er>Sol-fa</er>.]</ety> <fld>(Mus.)</fld> <def>The act of sol-faing.</def> <altsp>[Written also <asp>solmisation</asp>.]</altsp><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ This art was practiced by the Greeks; but six of the seven syllables now in use are generally attributed to Guido d' Arezzo, an Italian monk of the eleventh century, who is said to have taken them from the first syllables of the first six lines of the following stanza of a monkish hymn to St. John the Baptist. --<br/
[<source>1913 Webster</source>]</p>

<p><q><qex>Ut</qex> queant laxis<br/
<qex>Re</qex>sonare fibris<br/
<qex>Mi</qex>ra gestorum<br/
<qex>Fa</qex>muli tuorum<br/
<qex>Sol</qex>ve polluti<br/
<qex>La</qex>bii reatum,<br/
Sancte Joannes.</q><br/
[<source>1913 Webster</source>]</p>

<p>Professor Skeat says the name of the seventh note, <xex>si</xex>, was also formed by him [Guido] from the initials of the two words of the last line; but this is disputed, Littr<eacute/ attributing the first use of it to Anselm of Flanders long afterwards. The syllable <xex>do</xex> is often substituted for <xex>ut</xex>.
</note><br/
[<source>1913 Webster</source>]</p>

<p><ent>Solo</ent><br/
<hw>So"lo</hw> <pr>(s<omac/"l<osl/)</pr>, <pos>n.</pos>; <plu><it>pl.</it> E. <plw>Solos</plw> <pr>(s<omac/"l<omac/z)</pr>, It. <plw>Soli</plw> <pr>(#)</pr>.</plu> <ety>[It., from L. <ets>solus</ets> alone. See <er>Sole</er>, <pos>a.</pos>]</ety> <fld>(Mus.)</fld> <def>A tune, air, strain, or a whole piece, played by a single person on an instrument, or sung by a single voice.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Solo</ent><br/
<hw>So"lo</hw>, <pos>a.</pos> <fld>(Music)</fld> <def>Performing, or performed, alone; uncombined, except with subordinate parts, voices, or instruments; not concerted.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Soloist</ent><br/
<hw>So"lo*ist</hw>, <pos>n.</pos> <fld>(Mus.)</fld> <def>One who sings or plays a solo.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Solomon</ent><br/
<hw>Sol"o*mon</hw> <pr>(?)</pr>, <pos>n.</pos> <def>One of the kings of Israel, noted for his superior wisdom and magnificent reign; hence, a very wise man.</def> -- <wordforms><wf>Sol`o*mon"ic</wf> <pr>(#)</pr>, <pos>a.</pos></wordforms><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Solomon's seal</b></col> <fld>(Bot.)</fld>, <cd>a perennial liliaceous plant of the genus <gen>Polygonatum</gen>, having simple erect or curving stems rising from thick and knotted rootstocks, and with white or greenish nodding flowers. The commonest European species is <spn>Polygonatum multiflorum</spn>.  <spn>Polygonatum biflorum</spn> and <spn>Polygonatum giganteum</spn> are common in the Eastern United States. See <xex>Illust.</xex> of <er>Rootstock</er>.</cd> -- <col><b>False Solomon's seal</b></col> <fld>(Bot.)</fld>, <cd>any plant of the liliaceous genus <gen>Smilacina</gen> having small whitish flowers in terminal racemes or panicles.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Solomon's seal</ent><br/
<hw>Sol"o*mon's seal</hw>. <def>A mystic symbol consisting of two interlaced triangles forming a star with six points, often with one triangle dark and one light, symbolic of the union of soul and body.  It is shaped identically to the <er>hexagram</er> and <er>Star of David</er>, distinguished only in its usage.</def><br/
[<source>Webster 1913 Suppl.</source> <source>+PJC</source>]</p>

<p><ent>Solon</ent><br/
<hw>So"lon</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A celebrated Athenian lawmaker, born about 638 <sc>b. c.</sc>; hence, a legislator; a publicist; -- often used ironically.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Solo whist</ent><br/
<hw>Solo whist</hw>. <def>A card game played with the full pack ranking as at whist, each player declaring for which of seven different points he proposes to play.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Solpugid</ent><br/
<hw>Sol*pu"gid</hw> <pr>(?)</pr>, <pos>a.</pos> <fld>(Zool.)</fld> <def>Of or pertaining to the Solifugae.</def> -- <def2><pos>n.</pos> <def>One of the Solifugae.</def></def2><br/
[<source>1913 Webster</source>]</p>

<p><ent>Solpugidea</ent><br/
||<hw>Sol`pu*gid"e*a</hw> <pr>(?)</pr>, <pos>n. pl.</pos> <ety>[NL. See <er>Solifugae</er>.]</ety> <fld>(Zool.)</fld> <def>Same as <er>Solifugae</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Solstice</ent><br/
<hw>Sol"stice</hw> <pr>(?)</pr>, <pos>n.</pos><ety>[L. <ets>solstitium</ets>; <ets>sol</ets> the sun + <ets>sistere</ets> to cause to stand, akin to <ets>stare</ets> to stand: cf. F. <ets>solstice</ets>. See <er>Solar</er>, <pos>a.</pos>, <er>Stand</er>, <pos>v. i.</pos>]</ety> <sn>1.</sn> <def>A stopping or standing still of the sun.</def> <mark>[Obs.]</mark>  <rj><au>Sir T. Browne.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Astron.)</fld> <sd>(a)</sd> <def>The point in the ecliptic at which the sun is farthest from the equator, north or south, namely, the first point of the sign Cancer and the first point of the sign Capricorn, the former being the <xex>summer solstice</xex>, latter the <xex>winter solstice</xex>, in northern latitudes; -- so called because the sun then apparently stands still in its northward or southward motion.</def> <sd>(b)</sd> <def>The time of the sun's passing the solstices, or solstitial points, namely, about June 21 and December 21. See <xex>Illust.</xex> in Appendix.</def><br/
[<source>1913 Webster</source>]</p>

<p><-- p. 1370 pr=RH --></p>

<p><ent>Solstitial</ent><br/
<hw>Sol*sti"tial</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>solstitialis</ets>: cf. F. <ets>solsticial</ets>.]</ety> <sn>1.</sn> <def>Of or pertaining to a solstice.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Happening at a solstice; esp. (with reference to the northern hemisphere), happening at the summer solstice, or midsummer.</def> <ldquo/<xex>Solstitial</xex> summer's heat.<rdquo/  <rj><au>Milton.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Solubility</ent><br/
<hw>Sol`u*bil"i*ty</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Cf. F. <ets>solubilit<eacute/</ets>.]</ety> <sn>1.</sn> <def>The quality, condition, or degree of being soluble or solvable; <as>as, the <ex>solubility</ex> of a salt; the <ex>solubility</ex> of a problem or intricate difficulty</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Bot.)</fld> <def>The tendency to separate readily into parts by spurious articulations, as the pods of tick trefoil.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Soluble</ent><br/
<hw>Sol"u*ble</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>solubilis</ets>, fr. <ets>solvere</ets>, <ets>solutum</ets>, to loosen, to dissolve: cf. F. <ets>soluble</ets>. See <er>Solve</er>, and cf. <er>Solvable</er>.]</ety> <sn>1.</sn> <def>Susceptible of being dissolved in a fluid; capable of solution; <as>as, some substances are <ex>soluble</ex> in alcohol which are not <ex>soluble</ex> in water</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Sugar is . . . <qex>soluble</qex> in water and fusible in fire.</q> <rj><qau>Arbuthnot.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Susceptible of being solved; <as>as, a <ex>soluble</ex> algebraic problem</as>; susceptible of being disentangled, unraveled, or explained; <as>as, the mystery is perhaps <ex>soluble</ex></as>.</def> <ldquo/More <xex>soluble</xex> is this knot.<rdquo/  <rj><au>Tennyson.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>Relaxed; open or readily opened.</def> <mark>[R.]</mark> <ldquo/The bowels must be kept <xex>soluble</xex>.<rdquo/  <rj><au>Dunglison.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Soluble glass</b></col>. <fld>(Chem.)</fld> <cd>See under <er>Glass</er>.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Solubleness</ent><br/
<hw>Sol"u*ble*ness</hw>, <pos>n.</pos> <def>Quality or state of being soluble.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sola</ent><br/
<ent>Solus</ent><br/
<mhw>{ ||<hw>So"lus</hw> <pr>(?)</pr>, <pos>masc. a.</pos>, <hw>So"la</hw> <pr>(?)</pr>, <pos>fem. a.</pos> }</mhw> <ety>[L.]</ety> <def>Alone; -- chiefly used in stage directions, and the like.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Solute</ent><br/
<hw>So*lute"</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>solutus</ets>, p. p. of <ets>solvere</ets> to loosen. See <er>Solve</er>.]</ety> <sn>1.</sn> <def>Loose; free; liberal; <as>as, a <ex>solute</ex> interpretation</as>.</def> <mark>[Obs.]</mark>  <rj><au>Bacon.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Relaxed; hence; merry; cheerful.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>A brow <qex>solute</qex>, and ever-laughing eye.</q> <rj><qau>Young.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>Soluble; <as>as, a <ex>solute</ex> salt</as>.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <fld>(Bot.)</fld> <def>Not adhering; loose; -- opposed to <contr>adnate</contr>; <as>as, a <ex>solute</ex> stipule</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Solute</ent><br/
<hw>So*lute"</hw>, <pos>v. t.</pos> <sn>1.</sn> <def>To dissolve; to resolve.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To absolve; <as>as, to <ex>solute</ex> sin</as>.</def> <mark>[Obs.]</mark>  <rj><au>Bale.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Solution</ent><br/
<hw>So*lu"tion</hw> <pr>(s<osl/*l<umac/"sh<ucr/n)</pr>, <pos>n.</pos> <ety>[OE. <ets>solucion</ets>, OF. <ets>solucion</ets>, F. <ets>solution</ets>, fr. L. <ets>solutio</ets>, fr. <ets>solvere</ets>, <ets>solutum</ets>, to loosen, dissolve. See <er>Solve</er>.]</ety> <sn>1.</sn> <def>The act of separating the parts of any body, or the condition of undergoing a separation of parts; disruption; breach.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>In all bodies there is an appetite of union and evitation of <qex>solution</qex> of continuity.</q> <rj><qau>Bacon.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>The act of solving, or the state of being solved; the disentanglement of any intricate problem or difficult question; explanation; clearing up; -- used especially in mathematics, either of the process of solving an equation or problem, or the result of the process.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>The state of being dissolved or disintegrated; resolution; disintegration.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>It is unquestionably an enterprise of more promise to assail the nations in their hour of faintness and <qex>solution</qex>, than at a time when magnificent and seductive systems of worship were at their height of energy and splendor.</q> <rj><qau>I. Taylor.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <fld>(Chem.Phys.)</fld> <def>The act or process by which a body (whether solid, liquid, or gaseous) is absorbed into a liquid, and, remaining or becoming fluid, is diffused throughout the solvent; also, the product resulting from such absorption.</def><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ When a solvent will not take in any more of a substance the solution is said to be <xex>saturated</xex>. Solution is of two kinds; viz.: (<it>a</it>) <stype>Mechanical solution</stype>, in which no marked chemical change takes place, and in which, in the case of solids, the dissolved body can be regained by evaporation, as in the solution of salt or sugar in water. (<it>b</it>) <stype>Chemical solution</stype>, in which there is involved a decided chemical change, as when limestone or zinc undergoes solution in hydrochloric acid. <stype>Mechanical solution</stype> is regarded as a form of molecular or atomic attraction, and is probably occasioned by the formation of certain very weak and unstable compounds which are easily dissociated and pass into new and similar compounds.</note><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ This word is not used in chemistry or mineralogy for <xex>fusion</xex>, or the melting of bodies by the heat of fire.</note><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <def>Release; deliverance; discharge.</def> <mark>[Obs.]</mark>  <rj><au>Barrow.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>6.</sn> <fld>(Med.)</fld> <sd>(a)</sd> <def>The termination of a disease; resolution.</def> <sd>(b)</sd> <def>A crisis.</def> <sd>(c)</sd> <def>A liquid medicine or preparation (usually aqueous) in which the solid ingredients are wholly soluble.</def>  <rj><au>U. S. Disp.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Fehling's solution</b></col> <fld>(Chem.)</fld>, <cd>a standardized solution of cupric hydrate in sodium potassium tartrate, used as a means of determining the reducing power of certain sugars and sirups by the amount of red cuprous oxide thrown down.</cd> -- <col><b>Heavy solution</b></col> <fld>(Min.)</fld>, <cd>a liquid of high density, as a solution of mercuric iodide in potassium iodide (called the <stype>Sonstadt solution</stype> or <stype>Thoulet solution</stype>) having a maximum specific gravity of 3.2, or of borotungstate of cadium (<stype>Klein solution</stype>, specific gravity 3.6), and the like. Such solutions are much used in determining the specific gravities of minerals, and in separating them when mechanically mixed as in a pulverized rock.</cd> -- <col><b>Nessler's solution</b></col>. <cd>See <er>Nesslerize</er>.</cd> -- <col><b>Solution of continuity</b></col>, <cd>the separation of connection, or of connected substances or parts; -- applied, in surgery, to a fracture, laceration, or the like.</cd> <ldquo/As in the natural body a wound, or <xex>solution of continuity</xex>, is worse than a corrupt humor, so in the spiritual.<rdquo/ <au>Bacon.</au> -- <col><b>Standardized solution</b></col> <fld>(Chem.)</fld>, <cd>a solution which is used as a reagent, and is of a known and standard strength; specifically, a normal solution, containing in each cubic centimeter as many milligrams of the element in question as the number representing its atomic weight; thus, a normal solution of silver nitrate would contain 107.7 mgr. of silver in each cubic centimeter.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Solutive</ent><br/
<hw>Sol"u*tive</hw> <pr>(s<ocr/l"<usl/*t<icr/v)</pr>, <pos>a.</pos> <ety>[Cf. F. <ets>solutif</ets>.]</ety> <def>Tending to dissolve; loosening; laxative.</def>  <rj><au>Bacon.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Solvability</ent><br/
<hw>Solv`a*bil"i*ty</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F. <ets>solvabilit<eacute/</ets>.]</ety> <sn>1.</sn> <def>The quality or state of being solvable; <as>as, the <ex>solvability</ex> of a difficulty; the <ex>solvability</ex> of a problem</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>The condition of being solvent; ability to pay all just debts; solvency; <as>as, the <ex>solvability</ex> of a merchant</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Solvable</ent><br/
<hw>Solv"a*ble</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[F. <ets>solvable</ets>. See <er>Solve</er>, and cf. <er>Soluble</er>, <er>Solvible</er>.]</ety> <sn>1.</sn> <def>Susceptible of being solved, resolved, or explained; admitting of solution.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Capable of being paid and discharged; <as>as, <ex>solvable</ex> obligations</as>.</def>  <rj><au>Tooke.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>Able to pay one's debts; solvent.</def> <mark>[Obs.]</mark>  <rj><au>Fuller.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Solvableness</ent><br/
<hw>Solv"a*ble*ness</hw> <pr>(?)</pr>, <pos>n.</pos> <def>Quality of being solvable.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Solve</ent><br/
<hw>Solve</hw> <pr>(s<ocr/lv)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Solved</conjf> <pr>(s<ocr/lvd)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Solving</conjf>.]</vmorph> <ety>[L. <ets>solvere</ets>, <ets>solutum</ets>; from a prefix <ets>so-</ets> expressing separation (cf. <er>Sober</er>) + <ets>luere</ets> to loosen; cf. OF. <ets>soldre</ets>, <ets>soudre</ets>. See <er>Loose</er>, and cf. <er>Absolve</er>.]</ety> <def>To explain; to resolve; to unfold; to clear up (what is obscure or difficult to be understood); to work out to a result or conclusion; <as>as, to <ex>solve</ex> a doubt; to <ex>solve</ex> difficulties; to <ex>solve</ex> a problem</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>True piety would effectually <qex>solve</qex> such scruples.</q> <rj><qau>South.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>God shall <qex>solve</qex> the dark decrees of fate.</q> <rj><qau>Tickell.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><syn><b>Syn.</b> -- To explain; resolve; unfold; clear up.</syn><br/
[<source>1913 Webster</source>]</p>

<p><ent>Solve</ent><br/
<hw>Solve</hw>, <pos>n.</pos> <def>A solution; an explanation.</def> <mark>[Obs.]</mark>  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Solvency</ent><br/
<hw>Sol"ven*cy</hw> <pr>(s<ocr/l"v<eit/n*s<ycr/)</pr>, <pos>n.</pos> <ety>[See <er>Solvent</er>.]</ety> <def>The quality or state of being solvent.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Solvend</ent><br/
<hw>Sol"vend</hw> <pr>(s<ocr/l"v<ecr/nd)</pr>, <pos>n.</pos> <ety>[L. <ets>solvendus</ets> to be loosened or dissolved, fr. <ets>solvere</ets>. See <er>Solution</er>.]</ety> <def>A substance to be dissolved.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Solvent</ent><br/
<hw>Sol"vent</hw> <pr>(s<ocr/l"v<eit/nt)</pr>, <pos>a.</pos> <ety>[L. <ets>solvens</ets>, p. pr. of <ets>solvere</ets>. See <er>Solvable</er>.]</ety> <sn>1.</sn> <def>Having the power of dissolving; dissolving; <as>as, a <ex>solvent</ex> fluid</as>.</def> <ldquo/The <xex>solvent</xex> body.<rdquo/  <rj><au>Boyle.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Able or sufficient to pay all just debts; <as>as, a <ex>solvent</ex> merchant; the estate is <ex>solvent</ex></as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Solvent</ent><br/
<hw>Sol"vent</hw>, <pos>n.</pos> <fld>(Chem.)</fld> <def>A substance (usually liquid) suitable for, or employed in, solution, or in dissolving something; <as>as, water is the appropriate <ex>solvent</ex> of most salts, alcohol of resins, ether of fats, and mercury or acids of metals, etc.</as></def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>That which resolves; <as>as, a <ex>solvent</ex> of mystery</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Solver</ent><br/
<hw>Sol"ver</hw> <pr>(s<ocr/lv"<etil/r)</pr>, <pos>n.</pos> <def>One who, or that which, solves.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Solvible</ent><br/
<hw>Solv"i*ble</hw> <pr>(-<icr/*b'l)</pr>, <pos>a.</pos> <def>See <er>Solvable</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Soly</ent><br/
<hw>Sol"y</hw> <pr>(s<omac/l"<ycr/)</pr>, <pos>adv.</pos> <def>Solely.</def> <mark>[Obs.]</mark>  <rj><au>Spenser.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Soma</ent><br/
||<hw>So"ma</hw> <pr>(s<omac/"m<adot/)</pr>, <pos>n.</pos> <ety>[NL., fr. Gr. <grk>sw^ma</grk>, <grk>sw`matos</grk>, the body.]</ety> <fld>(Anat.)</fld> <def>The whole axial portion of an animal, including the head, neck, trunk, and tail.</def>  <rj><au>B. G. Wilder.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Samaj</ent><br/
<ent>Somaj</ent><br/
<mhw>{ <hw>So"maj"</hw> <pr>(s<osl/*m<aum/j")</pr>, <hw>Sa*maj"</hw> <pr>(s<adot/*m<aum/j")</pr>, }</mhw> <pos>n.</pos> <def>A society; a congregation, a worshiping assembly, or church, esp. of the Brahmo-somaj.</def> <mark>[India]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Somal</ent><br/
<ent>Somali</ent><br/
<mhw>{ <hw>So*ma"li</hw> <pr>(s<osl/*m<aum/"l<esl/)</pr>, <hw>So*mal"</hw> <pr>(s<osl/*m<aum/l")</pr>, }</mhw> <pos>n.</pos> <sn>1.</sn> <fld>(Ethnol.)</fld> <def>A Hamitic people of East Central Africa.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>of or pertaining to Somalia or to the inhabitants of Somalia.</def><br/
[<source>PJC</source>]</p>

<p><ent>Somatic</ent><br/
<hw>So*mat"ic</hw> <pr>(s<osl/*m<acr/t"<icr/k)</pr>, <pos>a.</pos> <ety>[Gr. <grk>swmatiko`s</grk>, fr. <grk>sw^ma</grk> the body.]</ety> <sn>1.</sn> <def>Of or pertaining to the body as a whole; corporeal; <as>as, <ex>somatic</ex> death; <ex>somatic</ex> changes</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Of or pertaining to the wall of the body; somatopleuric; parietal; <as>as, the <ex>somatic</ex> stalk of the yolk sac of an embryo</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Somatic death</b></col>. <cd>See the Note under <er>Death</er>, <pos>n.</pos>, 1.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Somatical</ent><br/
<hw>So*mat"ic*al</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Somatic.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Somatics</ent><br/
<hw>So*mat"ics</hw> <pr>(?)</pr>, <pos>n.</pos> <def>The science which treats of the general properties of matter; somatology.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Somatist</ent><br/
<hw>So"ma*tist</hw> <pr>(?)</pr>, <pos>n.</pos> <def>One who admits the existence of material beings only; a materialist.</def>  <rj><au>Glanvill.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Somatocyst</ent><br/
<hw>So"ma*to*cyst</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Gr. <grk>sw^ma</grk>, <grk>sw`matos</grk>, body + <grk>ky`stis</grk> a bladder.]</ety> <fld>(Zool.)</fld> <def>A cavity in the primary nectocalyx of certain <ord>Siphonophora</ord>. See <xex>Illust.</xex> under <er>Nectocalyx</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Somatology</ent><br/
<hw>So`ma*tol"o*gy</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Gr. <grk>sw^ma</grk>, <grk>sw`matos</grk>, body + <ets>-logy</ets>.]</ety> <sn>1.</sn> <def>The doctrine or the science of the general properties of material substances; somatics.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A treatise on the human body; anatomy.</def>  <rj><au>Dunglison.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <fld>(Biol.)</fld> <def>The science which treats of anatomy and physiology, apart from psychology.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><sn>4.</sn>  <fld>(Anthropol.)</fld> <def>The consideration of the physical characters of races and classes of men and of mankind in general.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Somatome</ent><br/
<hw>So"ma*tome</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Gr. <grk>sw^ma</grk>, body + <grk>te`mnein</grk> to cut.]</ety> <fld>(Anat. & Zool.)</fld> <def>See <er>Somite</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Somatopleure</ent><br/
<hw>So"ma*to*pleure</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Gr. <grk>sw^ma</grk>, <grk>sw`matos</grk>, body + <grk>pleyra`</grk> side.]</ety> <fld>(Anat.)</fld> <def>The outer, or parietal, one of the two lamellae into which the vertebrate blastoderm divides on either side of the notochord, and from which the walls of the body and the amnion are developed. See <er>Splanchnopleure</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Somatopleuric</ent><br/
<hw>So`ma*to*pleu"ric</hw> <pr>(?)</pr>, <pos>a.</pos> <fld>(Anat.)</fld> <def>Of or pertaining to the somatopleure.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Somatotropism</ent><br/
<hw>So`ma*tot"ro*pism</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Gr. <grk>sw^ma</grk>, <grk>sw`matos</grk>, the body + <grk>tre`pein</grk> to turn.]</ety> <fld>(Physiol.)</fld> <def>A directive influence exercised by a mass of matter upon growing organs.</def>  <rj><au>Encyc. Brit.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sombre</ent><br/
<ent>Somber</ent><br/
<mhw>{ <hw>Som"ber</hw>, <hw>Som"bre</hw> }</mhw> <pr>(?; 277)</pr>, <pos>a.</pos> <ety>[F. <ets>sombre</ets>; cf. Sp. <ets>sombra</ets>, shade, prob. from LL. <ets>subumbrare</ets> to put in the shade; L. <ets>sub</ets> under + <ets>umbra</ets> shade. See <er>Umbrage</er>.]</ety> <sn>1.</sn> <def>Dull; dusky; somewhat dark; gloomy; <as>as, a <ex>somber</ex> forest; a <ex>somber</ex> house.</as></def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Melancholy; sad; grave; depressing; <as>as, a <ex>somber</ex> person; <ex>somber</ex> reflections.</as></def><br/
[<source>1913 Webster</source>]</p>

<p><q>The dinner was silent and <qex>somber</qex>; happily it was also short.</q> <rj><qau>Beaconsfield.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sombre</ent><br/
<ent>Somber</ent><br/
<mhw>{ <hw>Som"ber</hw>, <hw>Som"bre</hw>  }</mhw>, <pos>v. t.</pos> <def>To make somber, or dark; to make shady.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sombre</ent><br/
<ent>Somber</ent><br/
<mhw>{ <hw>Som"ber</hw>, <hw>Som"bre</hw>  }</mhw>, <pos>n.</pos> <def>Gloom; obscurity; duskiness; somberness.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sombrely</ent><br/
<ent>Somberly</ent><br/
<mhw>{ <hw>Som"ber*ly</hw>, <hw>Som"bre*ly</hw>  }</mhw>, <pos>adv.</pos> <def>In a somber manner; sombrously; gloomily; despondingly.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sombreness</ent><br/
<ent>Somberness</ent><br/
<mhw>{ <hw>Som"ber*ness</hw>, <hw>Som"bre*ness</hw>  }</mhw>, <pos>n.</pos> <def>The quality or state of being somber; gloominess.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sombrero</ent><br/
||<hw>Som*bre"ro</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Sp., from <ets>sombra</ets> shade. See <er>Sombre</er>.]</ety> <def>A kind of broad-brimmed hat, worn in Spain and in Spanish America.</def>  <rj><au>Marryat.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sombrous</ent><br/
<hw>Som"brous</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[Cf. Sp. <ets>sombroso</ets>.]</ety> <def>Gloomy; somber.</def> <ldquo/Tall and <xex>sombrous</xex> pines.<rdquo/  <rj><au>Longfellow.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p>-- <wordforms><wf>Som"brous*ly</wf>, <pos>adv.</pos> -- <wf>Som"brous*ness</wf>, <pos>n.</pos></wordforms><br/
[<source>1913 Webster</source>]</p>

<p><ent>-some</ent><br/
<hw>-some</hw> <pr>(-s<omac/m)</pr>. <def>A combining form or suffix from Gr. <grk>sw^ma</grk> (gen. <grk>sw`matos</grk>) the <xex>body</xex>; <as>as in mero<ex>some</ex>, a body segment; cephalo<ex>some</ex>, etc.</as></def><br/
[<source>1913 Webster</source>]</p>

<p><-- add chromosome, ribosome, --><br/
[<source>1913 Webster</source>]</p>

<p><ent>-some</ent><br/
<hw>-some</hw> <pr>(-s<ucr/m)</pr>. <ety>[AS. <ets>-sum</ets>; akin to G. & OHG. <ets>-sam</ets>, Icel. <ets>samr</ets>, Goth. lustu<ets>sams</ets> longed for. See <er>Same</er>, <pos>a.</pos>, and cf. <er>Some</er>, <pos>a.</pos>]</ety> <def>An adjective suffix having primarily the sense of <xex>like</xex> or <xex>same</xex>, and indicating a considerable degree of the thing or quality denoted in the first part of the compound; as in mettle<xex>some</xex>, full of mettle or spirit; glad<xex>some</xex>, full of gladness; win<xex>some</xex>, blithe<xex>some</xex>, etc.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Some</ent><br/
<hw>Some</hw> <pr>(s<ucr/m)</pr>, <pos>a.</pos> <ety>[OE. <ets>som</ets>, <ets>sum</ets>, AS. <ets>sum</ets>; akin to OS., OFries., & OHG. <ets>sum</ets>, OD. <ets>som</ets>, D. <ets>sommig</ets>, Icel. <ets>sumr</ets>, Dan. <ets>somme</ets> (pl.), Sw. <ets>somlige</ets> (pl.), Goth. <ets>sums</ets>, and E. <ets>same</ets>. <root/191. See <er>Same</er>, <pos>a.</pos>, and cf. <er>-some</er>.]</ety> <sn>1.</sn> <def>Consisting of a greater or less portion or sum; composed of a quantity or number which is not stated; -- used to express an indefinite quantity or number; <as>as, <ex>some</ex> wine; <ex>some</ex> water; <ex>some</ex> persons</as>.  Used also pronominally; <as>as, I have <ex>some</ex></as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q><qex>Some</qex> theoretical writers allege that there was a time when there was no such thing as society.</q> <rj><qau>Blackstone.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A certain; one; -- indicating a person, thing, event, etc., as not known individually, or designated more specifically; <as>as, <ex>some</ex> man, that is, some one man</as>.</def> <ldquo/<xex>Some</xex> brighter clime.<rdquo/  <rj><au>Mrs. Barbauld.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q><qex>Some</qex> man praiseth his neighbor by a wicked intent.</q> <rj><qau>Chaucer.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Most gentlemen of property, at <qex>some</qex> period or other of their lives, are ambitious of representing their county in Parliament.</q> <rj><qau>Blackstone.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>Not much; a little; moderate; <as>as, the censure was to <ex>some</ex> extent just</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>About; near; more or less; -- used commonly with numerals, but formerly also with a singular substantive of time or distance; <as>as, a village of <ex>some</ex> eighty houses; <ex>some</ex> two or three persons; <ex>some</ex> hour hence</as>.</def><-- approximately. -->  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>The number slain on the rebel's part were <qex>some</qex> two thousand.</q> <rj><qau>Bacon.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <def>Considerable in number or quantity.</def> <ldquo/Bore us <xex>some</xex> leagues to sea.<rdquo/  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>On its outer point, <qex>some</qex> miles away.<br/
The lighthouse lifts its massive masonry.</q> <rj><qau>Longfellow.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>6.</sn> <def>Certain; those of one part or portion; -- in distinction from <contr>other</contr> or <contr>others</contr>; <as>as, <ex>some</ex> men believe one thing, and <ex>others</ex> another</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q><qex>Some</qex> [seeds] fell among thorns; . . . but <qex>other</qex> fell into good ground.</q> <rj><qau>Matt. xiii. 7, 8.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>7.</sn> <def>A part; a portion; -- used pronominally, and followed sometimes by <xex>of</xex>; <as>as, <ex>some</ex> of our provisions</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Your edicts <qex>some</qex> reclaim from sins,<br/
But most your life and blest example wins.</q> <rj><qau>Dryden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>All and some</b></col>, <cd>one and all. See under <er>All</er>, <pos>adv.</pos></cd> <mark>[Obs.]</mark></cs><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ The illiterate in the United States and Scotland often use <xex>some</xex> as an adverb, instead of <xex>somewhat</xex>, or an equivalent expression; as, I am <xex>some</xex> tired; he is <xex>some</xex> better; it rains <xex>some</xex>, etc.</note><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Some . . . some</b></col>, <cd>one part . . . another part; these . . . those; -- used distributively.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><q><qex>Some</qex> to the shores do fly,<br/
<qex>Some</qex> to the woods, or whither fear advised.</q> <rj><qau>Daniel.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ Formerly used also of single persons or things: this one . . . that one; one . . . another.</note><br/
[<source>1913 Webster</source>]</p>

<p><q><qex>Some</qex> in his bed, <qex>some</qex> in the deep sea.</q> <rj><qau>Chaucer.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Somebody</ent><br/
<hw>Some"bod*y</hw> <pr>(s<ucr/m"b<ocr/d*<ycr/)</pr>, <pos>n.</pos> <sn>1.</sn> <def>A person unknown or uncertain; a person indeterminate; some person.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Jesus said, <qex>Somebody</qex> hath touched me.</q> <rj><qau>Luke viii. 46.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>We must draw in <qex>somebody</qex> that may stand<br/
'Twixt us and danger.</q> <rj><qau>Denham.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A person of consideration or importance.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Before these days rose up Theudas, boasting himself to be <qex>somebody</qex>.</q> <rj><qau>Acts v. 36.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Somedeal</ent><br/
<hw>Some"deal`</hw> <pr>(s<ucr/m"d<emac/l`)</pr>, <pos>adv.</pos> <def>In some degree; somewhat.</def> <altsp>[Written also <asp>sumdel</asp>, <asp>sumdeale</asp>, and <asp>sumdele</asp>.]</altsp> <mark>[Obs.]</mark> <ldquo/She was <xex>somedeal</xex> deaf.<rdquo/  <rj><au>Chaucer.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Thou lackest <qex>somedeal</qex> their delight.</q> <rj><qau>Spenser.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Somehow</ent><br/
<hw>Some"how`</hw> <pr>(s<ucr/m"hou`)</pr>, <pos>adv.</pos> <def>In one way or another; in some way not yet known or designated; by some means; <as>as, the thing must be done <ex>somehow</ex>; he lives <ex>somehow</ex>.</as></def><br/
[<source>1913 Webster</source>]</p>

<p><q>By their action upon one another they may be swelled <qex>somehow</qex>, so as to shorten the length.</q> <rj><qau>Cheyne.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ The indefiniteness of <xex>somehow</xex> is emphasized by the addition of <xex>or other</xex>.</note><br/
[<source>1913 Webster</source>]</p>

<p><q>Although youngest of the familly, he has <qex>somehow or other</qex> got the entire management of all the others.</q> <rj><qau>Sir W. Scott.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Somerset</ent><br/
<ent>Somersault</ent><br/
<mhw>{ <hw>Som"er*sault</hw> <pr>(?)</pr>, <hw>Som"er*set</hw> <pr>(?)</pr>, }</mhw> <pos>n.</pos> <ety>[F. <ets>soubresaut</ets> a jump, leap, OF. <ets>soubresault</ets>, It. <ets>soprassalto</ets> an overleap, fr. L. <ets>supra</ets> over + <ets>saltus</ets> a leap, fr. <ets>salire</ets> to leap; or the French may be from Sp. <ets>sobresalto</ets> a sudden asault, a surprise. See <er>Supra</er>, and <ets>Salient</ets>.]</ety> <def>A leap in which a person turns his heels over his head and lights upon his feet; a turning end over end.</def> <altsp>[Written also <asp>summersault</asp>, <asp>sommerset</asp>, <asp>summerset</asp>, etc.]</altsp> <ldquo/The vaulter's <xex>sombersalts</xex>.<rdquo/  <rj><au>Donne.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Now I'll only<br/
Make him break his neck in doing a <qex>sommerset</qex>.</q> <rj><qau>Beau. & Fl.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Something</ent><br/
<hw>Some"thing</hw> <pr>(?)</pr>, <pos>n.</pos> <sn>1.</sn> <def>Anything unknown, undetermined, or not specifically designated; a certain indefinite thing; an indeterminate or unknown event; an unspecified task, work, or thing.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>There is <qex>something</qex> in the wind.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>The whole world has <qex>something</qex> to do, <qex>something</qex> to talk of, <qex>something</qex> to wish for, and <qex>something</qex> to be employed about.</q> <rj><qau>Pope.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q><qex>Something</qex> attemped, <qex>something</qex> done,<br/
Has earned a night's repose.</q> <rj><qau>Longfellow.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A part; a portion, more or less; an indefinite quantity or degree; a little.</def><br/
[<source>1913 Webster</source>]</p>

<p><q><qex>Something</qex> yet of doubt remains.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q><qex>Something</qex> of it arises from our infant state.</q> <rj><qau>I. Watts.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>A person or thing importance.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>If a man thinketh himself to be <qex>something</qex>, when he is nothing, he deceiveth himself.</q> <rj><qau>Gal. vi. 3.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Something</ent><br/
<hw>Some"thing</hw>, <pos>adv.</pos> <def>In some degree; somewhat; to some extent; at some distance.</def>  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>I <qex>something</qex> fear my father's wrath.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>We have <qex>something</qex> fairer play than a reasoner could have expected formerly.</q> <rj><qau>Burke.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>My sense of touch is <qex>something</qex> coarse.</q> <rj><qau>Tennyson.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>It must be done to-night,<br/
And <qex>something</qex> from the palace.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sometime</ent><br/
<hw>Some"time`</hw> <pr>(?)</pr>, <pos>adv.</pos> <sn>1.</sn> <def>At a past time indefinitely referred to; once; formerly.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Did they not <qex>sometime</qex> cry <ldquo/All hail<rdquo/ to me?</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>At a time undefined; once in a while; now and then; sometimes.</def><br/
[<source>1913 Webster</source>]</p>

<p><q><qex>Sometime</qex> we see a cloud that's dragonish,<br/
A vapor <qex>sometime</qex> like a bear or lion.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>At one time or other hereafter; <as>as, I will do it <ex>sometime</ex></as>.</def> <ldquo/<xex>Sometime</xex> he reckon shall.<rdquo/  <rj><au>Chaucer.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><-- p. 1371 pr=RH --></p>

<p><ent>Sometime</ent><br/
<hw>Some"time`</hw> <pr>(s<ucr/m"t<imac/m`)</pr>, <pos>a.</pos> <def>Having been formerly; former; late; whilom.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Our <qex>sometime</qex> sister, now our queen.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Ion, our <qex>sometime</qex> darling, whom we prized.</q> <rj><qau>Talfourd.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sometimes</ent><br/
<hw>Some"times`</hw> <pr>(?)</pr>, <pos>adv.</pos> <ety>[<ets>Sometime</ets> + adverbial ending <ets>-s</ets>, as in <ets>-wards</ets>.]</ety> <sn>1.</sn> <def>Formerly; sometime.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>That fair and warlike form<br/
In which the majesty of buried Denmark<br/
Did <qex>sometimes</qex> march.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>At times; at intervals; not always; now and then; occasionally.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>It is good that we <qex>sometimes</qex> be contradicted.</q> <rj><qau>Jer. Taylor.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Sometimes . . . sometimes</b></col>, <cd>at certain times . . . at certain other times; <as>as, <ex>sometimes</ex> he is earnest, <ex>sometimes</ex> he is frivolous</as>.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sometimes</ent><br/
<hw>Some"times`</hw>, <pos>a.</pos> <def>Former; sometime.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>Thy <qex>sometimes</qex> brother's wife.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Somewhat</ent><br/
<hw>Some"what`</hw> <pr>(?)</pr>, <pos>n.</pos> <sn>1.</sn> <def>More or less; a certain quantity or degree; a part, more or less; something.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>These salts have <qex>somewhat</qex> of a nitrous taste.</q> <rj><qau>Grew.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q><qex>Somewhat</qex> of his good sense will suffer, in this transfusion, and much of the beauty of his thoughts will be lost.</q> <rj><qau>Dryden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A person or thing of importance; a somebody.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Here come those that worship me.<br/
They think that I am <qex>somewhat</qex>.</q> <rj><qau>Tennyson.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Somewhat</ent><br/
<hw>Some"what`</hw>, <pos>adv.</pos> <def>In some degree or measure; a little.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>His giantship is gone, <qex>somewhat</qex> crestfallen.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q><qex>Somewhat</qex> back from the village street.</q> <rj><qau>Longfellow.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Somewhen</ent><br/
<hw>Some"when`</hw>, <pos>adv.</pos> <def>At some indefinite time.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Somewhere</ent><br/
<hw>Some"where`</hw> <pr>(?)</pr>, <pos>adv.</pos> <def>In some place unknown or not specified; in one place or another.</def> <ldquo/<xex>Somewhere</xex> nigh at hand.<rdquo/  <rj><au>Milton.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Somewhile</ent><br/
<hw>Some"while`</hw> <pr>(?)</pr>, <pos>adv.</pos> <def>Once; for a time.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Though, under color of shepherds, <qex>somewhile</qex><br/
There crept in wolves, full of fraud and guile.</q> <rj><qau>Spenser.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Somewhither</ent><br/
<hw>Some"whith`er</hw> <pr>(?)</pr>, <pos>adv.</pos> <def>To some indeterminate place; to some place or other.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Driven by the winds of temptation <qex>somewhither</qex>.</q> <rj><qau>Barrow.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Somite</ent><br/
<hw>So"mite</hw> <pr>(s<omac/"m<imac/t)</pr>, <pos>n.</pos> <ety>[Gr. <grk>sw^ma</grk> body.]</ety> <fld>(Anat. & Zool.)</fld> <def>One of the actual or ideal serial segments of which an animal, esp. an articulate or vertebrate, is composed; somatome; metamere.</def> -- <wordforms><wf>So*mit`ic</wf> <pr>(#)</pr>, <pos>a.</pos></wordforms><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sommeil</ent><br/
||<hw>Som`meil"</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F.]</ety> <def>Slumber; sleep.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sommerset</ent><br/
<hw>Som"mer*set</hw> <pr>(?)</pr>, <pos>n.</pos> <def>See <er>Somersault</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Somnambular</ent><br/
<hw>Som*nam"bu*lar</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Of or pertaining to somnambulism; somnambulistic.</def>  <rj><au>Mrs. Browning.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Somnambulate</ent><br/
<hw>Som*nam"bu*late</hw> <pr>(?)</pr>, <pos>v. i. & t.</pos> <def>To walk when asleep.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Somnambulation</ent><br/
<hw>Som*nam`bu*la"tion</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>somnus</ets> sleep + <ets>ambulatio</ets> a walking about, from <ets>ambulare</ets> to walk. See <er>Somnolent</er>, <er>Amble</er>.]</ety> <def>The act of walking in sleep, called also <altname>sleepwalking</altname>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Somnambulator</ent><br/
<hw>Som*nam"bu*la`tor</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A somnambulist.</def><br/
<syn><b>Syn. --</b> sleepwalker.</syn><br/
[<source>1913 Webster</source>]</p>

<p><ent>Somnambule</ent><br/
<hw>Som*nam"bule</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F.]</ety> <def>A somnambulist.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Somnambulic</ent><br/
<hw>Som*nam"bu*lic</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Somnambulistic.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Somnambulism</ent><br/
<hw>Som*nam"bu*lism</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Cf. F. <ets>somnambulisme</ets>. See <er>Somnambulation</er>.]</ety> <def>A condition of the nervous system in which an individual during sleep performs actions appropriate to the waking state; a state of sleep in which some of the senses and voluntary powers are partially awake; noctambulism.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Somnambulist</ent><br/
<hw>Som*nam"bu*list</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A person who is subject to somnambulism; one who walks in his sleep; a sleepwalker; a noctambulist.</def><br/
<syn><b>Syn. --</b> somnambulator.</syn><br/
[<source>1913 Webster</source>]</p>

<p><ent>Somnambulistic</ent><br/
<hw>Som*nam`bu*lis"tic</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Of or pertaining to a somnambulist or somnambulism; affected by somnambulism; appropriate to the state of a somnambulist.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Whether this was an intentional and waking departure, or a <qex>somnambulistic</qex> leave-taking and walking in her sleep, may remain a subject of contention.</q> <rj><qau>Dickens.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Somne</ent><br/
<hw>Som"ne</hw> <pr>(?)</pr>, <pos>v. t.</pos> <def>To summon.</def> <mark>[Obs.]</mark>  <rj><au>Chaucer.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Somner</ent><br/
<hw>Som"ner</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A summoner; esp., one who summons to an ecclesiastical court.</def> <mark>[Obs.]</mark>  <rj><au>Piers Plowman.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Somnial</ent><br/
<hw>Som"ni*al</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>somnialis</ets> dream bringing, fr. <ets>somnium</ets> dream, fr. <ets>somnus</ets> sleep.]</ety> <def>Of or pertaining to sleep or dreams.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The <qex>somnial</qex> magic superinduced on, without suspending, the active powers of the mind.</q> <rj><qau>Coleridge.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Somniative</ent><br/
<hw>Som"ni*a*tive</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Somnial; somniatory.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Somniatory</ent><br/
<hw>Som"ni*a*to*ry</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Pertaining to sleep or dreams; somnial.</def> <mark>[Obs. or R.]</mark>  <rj><au>Urquhart.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Somniculous</ent><br/
<hw>Som*nic"u*lous</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>somniculosus</ets>.]</ety> <def>Inclined to sleep; drowsy; sleepy.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Somniferous</ent><br/
<hw>Som*nif"er*ous</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>somnifer</ets>; <ets>somnus</ets> sleep + <ets>ferre</ets> to bring.]</ety> <def>Causing or inducing sleep; soporific; dormitive; <as>as, a <ex>somniferous</ex> potion</as>.</def>  <rj><au>Walton.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Somnific</ent><br/
<hw>Som*nif"ic</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>somnificus</ets>; <ets>somnus</ets> sleep + <ets>facere</ets> to make.]</ety> <def>Causing sleep; somniferous.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Somnifugous</ent><br/
<hw>Som*nif"u*gous</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>somnus</ets> sleep + <ets>fugare</ets> to put to flight.]</ety> <def>Driving away sleep.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Somniloquence</ent><br/
<hw>Som*nil"o*quence</hw> <pr>(?)</pr>, <pos>n.</pos> <def>The act of talking in one's sleep; somniloquism.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Somniloquism</ent><br/
<hw>Som*nil"o*quism</hw> <pr>(?)</pr>, <pos>n.</pos> <def>The act or habit of talking in one's sleep; somniloquy.</def>  <rj><au>Coleridge.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Somniloquist</ent><br/
<hw>Som*nil"o*quist</hw>, <pos>n.</pos> <def>One who talks in his sleep.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Somniloquous</ent><br/
<hw>Som*nil"o*quous</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>somnus</ets> sleep + <ets>loqui</ets> to speak.]</ety> <def>Apt to talk in sleep.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Somniloquy</ent><br/
<hw>Som*nil"o*quy</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A talking in sleep; the talking of one in a state of somnipathy.</def> <mark>[R.]</mark>  <rj><au>Coleridge.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Somnipathist</ent><br/
<hw>Som*nip"a*thist</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A person in a state of somniapathy.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Somnipathy</ent><br/
<hw>Som*nip"a*thy</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>somnus</ets> sleep + Gr. <?/ a suffering of the body, fr. <?/, <?/, to suffer.]</ety> <def>Sleep from sympathy, or produced by mesmerism or the like.</def> <altsp>[Written also <asp>somnopathy</asp>.]</altsp><br/
[<source>1913 Webster</source>]</p>

<p><ent>Somnolency</ent><br/
<ent>Somnolence</ent><br/
<mhw>{ <hw>Som"no*lence</hw> <pr>(?)</pr>, <hw>Som"no*len*cy</hw> <pr>(?)</pr>, }</mhw> <pos>n.</pos> <ety>[L. <ets>somnolentia</ets>: cf. F. <ets>somnolence</ets>.]</ety> <def>Sleepiness; drowsiness; inclination to sleep.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Somnolent</ent><br/
<hw>Som"no*lent</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[F. <ets>somnolent</ets>, L. <ets>somnolentus</ets>, from <ets>somnus</ets> sleep, akin to Gr. <?/, Skr. <ets>svapna</ets> sleep, dream, <ets>svap</ets> to sleep, Icel. <ets>sofa</ets>, AS. <ets>swefn</ets> sleep.  Cf. <er>Hypnotic</er>, <er>Somnambulism</er>, <er>Soporific</er>.]</ety> <def>Sleepy; drowsy; inclined to sleep.</def> -- <wordforms><wf>Som"no*lent*ly</wf>, <pos>adv.</pos></wordforms><br/
[<source>1913 Webster</source>]</p>

<p><q>He had no eye for such phenomena, because he had a <qex>somnolent</qex> want of interest in them.</q> <rj><qau>De Quincey.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>somnolism</ent><br/
<hw>som"no*lism</hw> <pr>(?)</pr>, <pos>n.</pos> <sn>1.</sn> <def>The somnolent state induced by animal magnetism (hypnotism); the hypnotic state.</def> <mark>[archaic]</mark>  <rj><au>Thomas (Med. Dict.).</au></rj><br/
[<source>1913 Webster</source> <source>+PJC</source>]</p>

<p><sn>2.</sn> <def>the act or process of inducing the hypnotic state; hypnotism; hyonotizing.</def> <au>Stedman.</au><br/
[<source>PJC</source>]</p>

<p><ent>Somnopathy</ent><br/
<hw>Som*nop"a*thy</hw> <pr>(?)</pr>, <pos>n.</pos> <def>Somnipathy.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Somnour</ent><br/
<hw>Som"nour</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A summoner; an apparitor; a sompnour.</def> <mark>[Obs.]</mark>  <rj><au>Piers Plowman.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Somonce</ent><br/
<ent>Somonaunce</ent><br/
<mhw>{ <hw>Som"on*aunce</hw> <pr>(?)</pr>, <hw>Som"once</hw> <pr>(?)</pr>, }</mhw> <pos>n.</pos> <ety>[See <er>Summon</er>, <er>Summons</er>.]</ety> <def>A summons; a citation.</def> <mark>[Obs.]</mark>  <rj><au>Chaucer.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Somonour</ent><br/
<hw>Som"on*our</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A summoner.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sompne</ent><br/
<hw>Somp"ne</hw> <pr>(? <or/ ?)</pr>, <pos>v. t.</pos> <def>To summon; to cite.</def> <mark>[Obs.]</mark>  <rj><au>Chaucer.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sompnour</ent><br/
<hw>Somp"nour</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A summoner.</def> <mark>[Obs.]</mark>  <rj><au>Chaucer.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Son</ent><br/
<hw>Son</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[OE. <ets>sone</ets>, <ets>sune</ets>, AS. <ets>sunu</ets>; akin to D. <ets>zoon</ets>, OS., OFries., & OHG. <ets>sunu</ets>, G. <ets>sohn</ets>, Icel. <ets>sonr</ets>, Sw. <ets>son</ets>, Dan. <ets>s<oum/n</ets>, Goth. <ets>sunus</ets>, Lith. <ets>sunus</ets>, Russ. <ets>suin'</ets>, Skr. <ets>s<umac/nu</ets> (from <ets>s<umac/</ets> to beget, to bear), and Gr. <?/ son. <root/293.  Cf. <er>Sow</er>, <pos>n.</pos>]</ety> <sn>1.</sn> <def>A male child; the male issue, or offspring, of a parent, father or mother.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Sarah conceived, and bare Abraham a <qex>son</qex>.</q> <rj><qau>Gen. xxi. 2.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A male descendant, however distant; hence, in the plural, descendants in general.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>I am the <qex>son</qex> of the wise, the <qex>son</qex> of ancient kings.</q> <rj><qau>Isa. xix. 11.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>I am the Lord, I change not; therefore ye <qex>sons</qex> of Jacob are not consumed.</q> <rj><qau>Mal. iii. 6.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>Any young male person spoken of as a child; an adopted male child; a pupil, ward, or any other young male dependent.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The child grew, and she brought him unto Pharaoh's daughter, and he became her <qex>son</qex>.</q> <rj><qau>Ex. ii. 10.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Be plain, good <qex>son</qex>, and homely in thy drift.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>A native or inhabitant of some specified place; <as>as, <ex>sons</ex> of Albion; <ex>sons</ex> of New England</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <def>The produce of anything.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Earth's tall <qex>sons</qex>, the cedar, oak, and pine.</q> <rj><qau>Blackmore.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>6.</sn> (Commonly with the def. article) <def>Jesus Christ, the Savior; -- called the <xex>Son</xex> of God, and the <xex>Son</xex> of man.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>We . . . do testify that the Father sent the <qex>Son</qex> to be the Savior of the world.</q> <rj><qau>1 John iv. 14.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Who gave His <qex>Son</qex> sure all has given.</q> <rj><qau>Keble.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ The expressions <xex>son of pride</xex>, <xex>sons of light</xex>, <xex>son of Belial</xex>, are Hebraisms, which denote persons possessing the qualitites of pride, of light, or of Belial, as children inherit the qualities of their ancestors.</note><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Sons of the prophets</b></col>. <cd>See <xex>School of the prophets</xex>, under <er>Prophet</er>.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sonance</ent><br/
<hw>So"nance</hw> <pr>(?)</pr>, <pos>n.</pos> <sn>1.</sn> <def>A sound; a tune; <as>as, to sound the tucket <ex>sonance</ex></as>.</def> <mark>[Obs.]</mark>  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>The quality or state of being sonant.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sonant</ent><br/
<hw>So"nant</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>sonans</ets>, <ets>-antis</ets>, p. pr. of <ets>sonare</ets> to sound. See <er>Sound</er> a noise.]</ety> <sn>1.</sn> <def>Of or pertaining to sound; sounding.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Phonetics)</fld> <def>Uttered, as an element of speech, with tone or proper vocal sound, as distinguished from mere breath sound; intonated; voiced; vocal; tonic; the opposite of <contr>nonvocal</contr>, or <contr>surd</contr>; -- said of the vowels, semivowels, liquids, and nasals, and particularly of the consonants <xex>b</xex>, <xex>d</xex>, <xex>g</xex> hard, <xex>v</xex>, etc., as compared with their cognates <xex>p</xex>, <xex>t</xex>, <xex>k</xex>, <xex>f</xex>, etc., which are called <xex>nonvocal</xex>, <xex>surd</xex>, or <xex>aspirate</xex>.</def> -- <def2><pos>n.</pos> <def>A sonant letter.</def></def2><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sonata</ent><br/
<hw>So*na"ta</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[It., fr. It. & L. <ets>sonare</ets> to sound. See <er>Sound</er> a noise.]</ety> <fld>(Mus.)</fld> <def>An extended composition for one or two instruments, consisting usually of three or four movements; <as>as, Beethoven's <ex>sonatas</ex> for the piano, for the violin and piano, etc.</as></def><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ The same general structure prevails in symphonies, instrumental trios, quartets, etc., and even in classical concertos. The sonata form, distinctively, characterizes the quick opening movement, which may have a short, slow introduction; the second, or slow, movement is either in the song or variation form; third comes the playful minuet or the more modern scherzo; then the quick finale in the rondo form. But both form and order are sometimes exceptional.</note><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sonatina</ent><br/
||<hw>So`na*ti"na</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[It.]</ety> <fld>(Mus.)</fld> <def>A short and simple sonata.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sonsy</ent><br/
<ent>Soncy</ent><br/
<mhw>{ <hw>Son"cy</hw>, <hw>Son"sy</hw> <pr>(?)</pr>, }</mhw> <pos>a.</pos> <ety>[Scot. <ets>sonce</ets>, <ets>sons</ets>, prosperity, happiness, fr. Gael. & Ir. <ets>sonas</ets>.]</ety> <def>Lucky; fortunate; thriving; plump.</def> <mark>[Prov. Eng. & Scot.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sonde</ent><br/
<ent>Sond</ent><br/
<mhw>{ <hw>Sond</hw> <pr>(?)</pr>, <hw>Sonde</hw>  }</mhw>, <pos>n.</pos> <ety>[AS. <ets>sand</ets>. See <er>Send</er>, <pos>v. t.</pos>]</ety> <sn>1.</sn> <def>That which is sent; a message or messenger; hence, also, a visitation of providence; an affliction or trial.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>Ye have enough, parde, of Goddes <qex>sond</qex>.</q> <rj><qau>Chaucer.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Rocketry)</fld> <def>a rocket or balloon sent into the upper atmosphere to measure atmospheric conditions or other phenomena at altitudes unattainable by aircraft.</def><br/
[<source>PJC</source>]</p>

<p><ent>Sondeli</ent><br/
||<hw>Son"de*li</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Zool.)</fld> <def>The musk shrew. See under <er>Musk</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sonderclass</ent><br/
<hw>Son"der*class`</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[G. <ets>sonderklasse</ets> special class.]</ety> <fld>(Yachting)</fld> <def>A special class of small yachts developed in Germany under the patronage of  <person>Emperor William</person> and <person>Prince Henry</person> of Prussia, and so called because these yachts do not conform to the restrictions for the regular classes established by the rules of the International Yacht Racing Union. In yachts of the sonderclass, as prescribed for the season of 1911, the  aggregate of the length on water line, extreme beam, and  extreme draft must be not more than 32 feet; the weight, not less than 4,035 pounds (without crew); the sail area, not more than 550 square yards; and the cost of construction (for American boats) not more than $2400. The crew must be amateurs and citizens of the country in which the yacht was built.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Song</ent><br/
<hw>Song</hw> <pr>(s<ocr/ng; 115)</pr>, <pos>n.</pos> <ety>[AS. <ets>song</ets>, <ets>sang</ets>, fr. <ets>singan</ets> to sing; akin to D. <ets>zang</ets>, G. <ets>sang</ets>, Icel. <ets>s<oum/ngr</ets>, Goth. <ets>saggws</ets>. See <er>Sing</er>.]</ety> <sn>1.</sn> <def>That which is sung or uttered with musical modulations of the voice, whether of a human being or of a bird, insect, etc.</def> <ldquo/That most ethereal of all sounds, the <xex>song</xex> of crickets.<rdquo/  <rj><au>Hawthorne.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A lyrical poem adapted to vocal music; a ballad.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>More generally, any poetical strain; a poem.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The bard that first adorned our native tongue<br/
Tuned to his British lyre this ancient <qex>song</qex>.</q> <rj><qau>Dryden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>Poetical composition; poetry; verse.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>This subject for heroic <qex>song</qex>.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <def>An object of derision; a laughingstock.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>And now am I their <qex>song</qex>, yea, I am their byword.</q> <rj><qau>Job xxx. 9.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>6.</sn> <def>A trifle; an insignificant sum of money; <as>as, he bought it for a <ex>song</ex></as>.</def> <ldquo/The soldier's pay is a <xex>song</xex>.<rdquo/  <rj><au>Silliman.</au></rj><br/
[<source>1913 Webster</source> <source>+PJC</source>]</p>

<p><cs><col><b>Old song</b></col>, <cd>a trifle; nothing of value.</cd>  <ldquo/I do not intend to be thus put off with an <xex>old song</xex>.<rdquo/ <au>Dr. H. More.</au> -- <col><b>Song bird</b></col> <fld>(Zool.)</fld>, <cd>any singing bird; one of the <ord>Oscines</ord>.</cd> -- <col><b>Song sparrow</b></col> <fld>(Zool.)</fld>, <cd>a very common North American sparrow (<spn>Melospiza fasciata</spn>, or <spn>Melospiza melodia</spn>) noted for the sweetness of its song in early spring. Its breast is covered with dusky brown streaks which form a blotch in the center.</cd> -- <col><b>Song thrush</b></col> <fld>(Zool.)</fld>, <cd>a common European thrush (<spn>Turdus musicus</spn>), noted for its melodius song; -- called also <altname>mavis</altname>, <altname>throstle</altname>, and <altname>thrasher</altname>.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><syn><b>Syn.</b> -- Sonnet; ballad; canticle; carol; canzonet; ditty; hymn; descant; lay; strain; poesy; verse.</syn><br/
[<source>1913 Webster</source>]</p>

<p><ent>Songcraft</ent><br/
<hw>Song"craft`</hw> <pr>(s<ocr/ng"kr<adot/ft`)</pr>, <pos>n.</pos> <def>The art of making songs or verses; metrical composition; versification.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>A half-effaced inscription,<br/
Written with little skill of <qex>songcraft</qex>.</q> <rj><qau>Longfellow.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Songful</ent><br/
<hw>Song"ful</hw> <pr>(s<ocr/ng"f<usdot/l)</pr>, <pos>a.</pos> <def>Disposed to sing; full of song.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Songish</ent><br/
<hw>Song"ish</hw>, <pos>a.</pos> <def>Consisting of songs.</def> <mark>[R.]</mark>  <rj><au>Dryden.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Songless</ent><br/
<hw>Song"less</hw>, <pos>a.</pos> <def>Destitute of the power of song; without song; <as>as, <ex>songless</ex> birds; <ex>songless</ex> woods</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Songster</ent><br/
<hw>Song"ster</hw> <pr>(s<ocr/ng"st<etil/r)</pr>, <pos>n.</pos> <ety>[AS. <ets>sangestre</ets> a female singer.]</ety> <sn>1.</sn> <def>One who sings; one skilled in singing; -- not often applied to human beings.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Zool.)</fld> <def>A singing bird.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Songstress</ent><br/
<hw>Song"stress</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[See <er>Songster</er>, and <er>-ess</er>.]</ety> <def>A woman who sings; also, a female singing bird.</def>  <rj><au>Thomson.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sonifer</ent><br/
<hw>Son"i*fer</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[NL. See <er>Soniferous</er>.]</ety> <def>A kind of ear trumpet for the deaf, or the partially deaf.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Soniferous</ent><br/
<hw>So*nif"er*ous</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>sonus</ets> sound + <ets>-ferous</ets>.]</ety> <def>Sounding; producing sound; conveying sound.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sonification</ent><br/
<hw>Son`i*fi*ca"tion</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>sonus</ets> sound + <ets>-ficare</ets> (in comp.) to make. See <er>-fy</er>.]</ety> <def>The act of producing sound, as the stridulation of insects.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Son-in-law</ent><br/
<hw>Son"-in-law`</hw> <pr>(?)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Sons-in-law</plw> <pr>(<?/)</pr>.</plu> <def>The husband of one's daughter; a man in his relationship to his wife's parents.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>To take me as for thy <qex>son in lawe</qex>.</q> <rj><qau>Chaucer.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sonless</ent><br/
<hw>Son"less</hw>, <pos>a.</pos> <def>Being without a son.</def>  <rj><au>Marston.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>As no baron who was <qex>sonless</qex> could give a husband to his daughter, save with his lord's consent.</q> <rj><qau>J. R. Green.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sonnet</ent><br/
<hw>Son"net</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F., fr. It. <ets>sonetto</ets>, fr. <ets>suono</ets> a sound, a song, fr. L. <ets>sonus</ets> a sound. See <er>Sound</er> noise.]</ety> <sn>1.</sn> <def>A short poem, -- usually amatory.</def> <mark>[Obs.]</mark>  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>He had a wonderful desire to chant a <qex>sonnet</qex> or hymn unto <person>Apollo Pythius</person>.</q> <rj><qau>Holland.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A poem of fourteen lines, -- two stanzas, called the <part>octave</part>, being of four verses each, and two stanzas, called the <part>sestet</part>, of three verses each, the rhymes being adjusted by a particular rule.</def><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ In the proper sonnet each line has five accents, and the octave has but two rhymes, the second, third, sixth, and seventh lines being of one rhyme, and the first, fourth, fifth, and eighth being of another. In the sestet there are sometimes two and sometimes three rhymes; but in some way its two stazas rhyme together. Often the three lines of the first stanza rhyme severally with the three lines of the second. In Shakespeare's sonnets, the first twelve lines are rhymed alternately, and the last two rhyme together.</note><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sonnet</ent><br/
<hw>Son"net</hw>, <pos>v. i.</pos> <def>To compose sonnets.</def> <ldquo/Strains that come almost to <xex>sonneting</xex>.<rdquo/  <rj><au>Milton.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sonneteer</ent><br/
<hw>Son`net*eer"</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A composer of sonnets, or small poems; a small poet; -- usually in contempt.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>What woful stuff this madrigal would be<br/
In some starved hackney <qex>sonneteer</qex> or me!</q> <rj><qau>Pope.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sonneteer</ent><br/
<hw>Son`net*eer"</hw>, <pos>v. i.</pos> <def>To compose sonnets.</def>  <rj><au>Lowell.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sonneter</ent><br/
<hw>Son"net*er</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A composer of sonnets.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sonnetist</ent><br/
<hw>Son"net*ist</hw>, <pos>n.</pos> <def>A sonneter, or sonneteer.</def>  <rj><au>Bp. Hall.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sonnetize</ent><br/
<hw>Son"net*ize</hw> <pr>(?)</pr>, <pos>v. i.</pos> <def>To compose sonnets.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sonnish</ent><br/
<hw>Son"nish</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Like the sun; sunny; golden.</def> <mark>[Obs.]</mark> <ldquo/Her <xex>sonnish</xex> hairs.<rdquo/  <rj><au>Chaucer.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sonnite</ent><br/
<hw>Son"nite</hw> <pr>(?)</pr>, <pos>n.</pos> <def>See <er>Sunnite</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sonometer</ent><br/
<hw>So*nom"e*ter</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>sonus</ets> a sound + <ets>-meter</ets>.]</ety> <sn>1.</sn> <fld>(Physiol.)</fld> <def>An instrument for exhibiting the transverse vibrations of cords, and ascertaining the relations between musical notes. It consists of a cord stretched by weight along a box, and divided into different lengths at pleasure by a bridge, the place of which is determined by a scale on the face of the box.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>An instrument for testing the hearing capacity.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sonoran</ent><br/
<hw>So*no"ran</hw> <pr>(?)</pr>, <pos>a.</pos> <fld>(Biogeography)</fld> <def>Pertaining to or designating the arid division of the Austral zone, including the warmer parts of the western United States and central Mexico. It is divided into the <col><b>Upper Sonoran</b></col>, which lies next to the Transition zone, and the <col><b>Lower Sonoran</b></col>, next to the Tropical.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Sonorific</ent><br/
<hw>Son`o*rif"ic</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>sonor</ets>, <ets>-oris</ets>, a sound + <ets>facere</ets> to make. See <er>Sonorous</er>.]</ety> <def>Producing sound; <as>as, the <ex>sonorific</ex> quality of a body</as>.</def> <mark>[R.]</mark>  <rj><au>I. Watts.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sonority</ent><br/
<hw>So*nor"i*ty</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>sonoritas</ets>.]</ety> <def>The quality or state of being sonorous; sonorousness.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sonorous</ent><br/
<hw>So*no"rous</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>sonorus</ets>, fr. <ets>sonor</ets>, <ets>-oris</ets>, a sound, akin to <ets>sonus</ets> a sound. See <er>Sound</er>.]</ety> <sn>1.</sn> <def>Giving sound when struck; resonant; <as>as, <ex>sonorous</ex> metals</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Loud-sounding; giving a clear or loud sound; <as>as, a <ex>sonorous</ex> voice</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>Yielding sound; characterized by sound; vocal; sonant; <as>as, the vowels are <ex>sonorous</ex></as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>Impressive in sound; high-sounding.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The Italian opera, amidst all the meanness and familiarty of the thoughts, has something beautiful and <qex>sonorous</qex> in the expression.</q> <rj><qau>Addison.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>There is nothing of the artificial Johnsonian balance in his style. It is as often marked by a pregnant brevity as by a <qex>sonorous</qex> amplitude.</q> <rj><qau>E. Everett.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <fld>(Med.)</fld> <def>Sonant; vibrant; hence, of sounds produced in a cavity, deep-toned; <as>as, <ex>sonorous</ex> rhonchi</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Sonorous figures</b></col> <fld>(Physics)</fld>, <cd>figures formed by the vibrations of a substance capable of emitting a musical tone, as when the bow of a violin is drawn along the edge of a piece of glass or metal on which sand is strewed, and the sand arranges itself in figures according to the musical tone. Called also <altname>acoustic figures</altname>.</cd> -- <col><b>Sonorous tumor</b></col> <fld>(Med.)</fld>, <cd>a tumor which emits a clear, resonant sound on percussion.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p>-- <wordforms><wf>So*no"rous*ly</wf>, <pos>adv.</pos> -- <wf>So*no"rous*ness</wf>, <pos>n.</pos></wordforms><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sonship</ent><br/
<hw>Son"ship</hw> <pr>(?)</pr>, <pos>n.</pos> <def>The state of being a son, or of bearing the relation of a son; filiation.</def>  <rj><au>Dr. H. More.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sonsy</ent><br/
<hw>Son"sy</hw> <pr>(?)</pr>, <pos>a.</pos> <def>See <er>Soncy</er>.</def> <mark>[Scot.]</mark>  <rj><au>Burns.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sontag</ent><br/
<hw>Son"tag</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[So called from from Mme. Henriette <ets>Sontag</ets>, a famous singer.]</ety> <def>A knitted worsted jacket, worn over the waist of a woman's dress.</def><br/
[<source>1913 Webster</source>]</p>

<p><-- p. 1372 --></p>

<p><ent>Sonties</ent><br/
<hw>Son"ties</hw> <pr>(?)</pr>, <pos>n.</pos> <def>Probably from <ldquo/<xex>saintes</xex><rdquo/ saints, or from <xex>sanctities</xex>; -- used as an oath.</def> <mark>[Obs.]</mark>  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Soochong</ent><br/
<hw>Soo*chong"</hw> <pr>(?)</pr>, <pos>n.</pos> <def>Same as <er>Souchong</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Soodra</ent><br/
||<hw>Soo"dra</hw> <pr>(?)</pr>. <def>Same as <er>Sudra</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Soofeeism</ent><br/
<ent>Soofee</ent><br/
<mhw>{ <hw>Soo"fee</hw> <pr>(?)</pr>, <hw>Soo"fee*ism</hw> <pr>(?)</pr> }</mhw>. <def>Same as <er>Sufi</er>, <er>Sufism</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Soojee</ent><br/
<hw>Soo"jee</hw> <pr>(?)</pr>, <pos>n.</pos> <def>Same as <er>Suji</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Soon</ent><br/
<hw>Soon</hw> <pr>(s<oomac/n)</pr>, <pos>adv.</pos> <ety>[OE. <ets>sone</ets>, AS. <ets>s<omac/na</ets>; cf. OFries. <ets>s<omac/n</ets>, OS. <ets>s<amac/na</ets>, <ets>s<amac/no</ets>, OHG. <ets>s<amac/r</ets>, Goth. <ets>suns</ets>.]</ety> <sn>1.</sn> <def>In a short time; shortly after any time specified or supposed; <as>as, <ex>soon</ex> after sunrise</as>.</def> <ldquo/<xex>Sooner</xex> said than done.<rdquo/ <au>Old Proverb.</au> <ldquo/As <xex>soon</xex> as it might be.<rdquo/ <au>Chaucer.</au><br/
[<source>1913 Webster</source>]</p>

<p><q>She finished, and the subtle fiend his lore<br/
<qex>Soon</qex> learned.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Without the usual delay; before any time supposed; early.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>How is it that ye are come so <qex>soon</qex> to-day?</q> <rj><qau>Ex. ii. 18.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>Promptly; quickly; easily.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Small lights are <qex>soon</qex> blown out, huge fires abide.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>Readily; willingly; -- in this sense used with <xex>would</xex>, or some other word expressing <xex>will</xex>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>I would as <qex>soon</qex> see a river winding through woods or in meadows, as when it is tossed up in so many whimsical figures at Versailles.</q> <rj><qau>Addison.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><mcol><col><b>As soon as</b></col>, <it>or</it> <col><b>So soon as</b></col></mcol>, <cd>immediately at or after another event.</cd> <ldquo/<xex>As soon as</xex> he came nigh unto the camp . . . he saw the calf, and the dancing.<rdquo/ <au>Ex. xxxii. 19.</au> See <cref>So . . . as</cref>, under <er>So</er>. -- <col><b>Soon at</b></col>, <cd>as soon as; or, as soon as the time referred to arrives.</cd> <mark>[Obs.]</mark> <ldquo/I shall be sent for <xex>soon at</xex> night.<rdquo/ <au>Shak.</au> -- <col><b>Sooner or later</b></col>, <cd>at some uncertain time in the future; <as>as, he will discover his mistake <ex>sooner or later</ex></as>.</cd> -- <col><b>With the soonest</b></col>, <cd>as soon as any; among the earliest; too soon.</cd> <mark>[Obs.]</mark>  <au>Holland.</au></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Soon</ent><br/
<hw>Soon</hw>, <pos>a.</pos> <def>Speedy; quick.</def> <mark>[Obs.]</mark>  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Soonee</ent><br/
<hw>Soo"nee</hw> <pr>(?)</pr>, <pos>n.</pos> <def>See <er>Sunnite</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sooner</ent><br/
<hw>Soon"er</hw> <pr>(?)</pr>, <pos>n.</pos> <def>In the western United States, one who settles on government land before it is legally open to settlement in order to gain the prior claim that the law gives to the first settler when the land is opened to settlement; hence, any one who does a thing prematurely or anticipates another in acting in order to gain an unfair advantage.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Sooner State</ent><br/
<hw>Sooner State</hw>. <def>Oklahoma; -- a nickname.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Soonly</ent><br/
<hw>Soon"ly</hw> <pr>(?)</pr>, <pos>adv.</pos> <def>Soon.</def> <mark>[Obs.]</mark>  <rj><au>Dr. H. More.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Soord</ent><br/
<hw>Soord</hw> <pr>(?)</pr>, <pos>n.</pos> <def>Skin of bacon.</def> <mark>[Obs.]</mark>  <rj><au>Bp. Hall.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Soorma</ent><br/
<hw>Soor"ma</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Hind. & Per. <ets>surma</ets>.]</ety> <def>A preparation of antimony with which Mohammedan men anoint their eyelids.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sooshong</ent><br/
<hw>Soo*shong"</hw> <pr>(?)</pr>, <pos>n.</pos> <def>See <er>Souchong</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Soosoo</ent><br/
<hw>Soo"soo</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Zool.)</fld> <def>A kind of dolphin (<spn>Platanista Gangeticus</spn>) native of the river Ganges; the Gangetic dolphin. It has a long, slender, somewhat spatulate beak.</def> <altsp>[Written also <asp>susu</asp>.]</altsp><br/
[<source>1913 Webster</source>]</p>

<p><ent>Soot</ent><br/
<hw>Soot</hw> <pr>(? <or/ ?; 277)</pr>, <pos>n.</pos> <ety>[OE. <ets>sot</ets>, AS. <ets>s<?/t</ets>; akin to Icel. <ets>s<?/t</ets>, Sw. <ets>sot</ets>, Dan. <ets>sod</ets>, OD. <ets>soet</ets>, Lith. <ets>s<?/dis</ets>; cf. Gael. <ets>suith</ets>, Ir. <ets>suth</ets>.]</ety> <def>A black substance formed by combustion, or disengaged from fuel in the process of combustion, which rises in fine particles, and adheres to the sides of the chimney or pipe conveying the smoke; strictly, the fine powder, consisting chiefly of carbon, which colors smoke, and which is the result of imperfect combustion. See <er>Smoke</er>.</def><-- of "incomplete" combustion --><br/
[<source>1913 Webster</source>]</p>

<p><ent>Soot</ent><br/
<hw>Soot</hw>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Sooted</conjf>; <pos>p. pr. & vb. n.</pos> <conjf>Sooting</conjf>.]</vmorph> <def>To cover or dress with soot; to smut with, or as with, soot; <as>as, to <ex>soot</ex> land</as>.</def>  <rj><au>Mortimer.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Soote</ent><br/
<ent>Soot</ent><br/
<mhw>{ <hw>Soot</hw> <pr>(?)</pr>, <hw>Soot"e</hw> <pr>(?)</pr>, }</mhw> <pos>a.</pos> <ety>[See <er>Sweet</er>.]</ety> <def>Sweet.</def> <mark>[Obs.]</mark> <ldquo/The <xex>soote</xex> savour of the vine.<rdquo/  <rj><au>Chaucer.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sooterkin</ent><br/
<hw>Soot"er*kin</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Cf. Prov. G. <ets>suttern</ets> to boil gently.]</ety> <def>A kind of false birth, fabled to be produced by Dutch women from sitting over their stoves; also, an abortion, in a figurative sense; an abortive scheme.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Fruits of dull heat, and <qex>sooterkins</qex> of wit.</q> <rj><qau>Pope.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sooth</ent><br/
<hw>Sooth</hw> <pr>(s<oomac/th)</pr>, <pos>a.; <plain>also</plain> adv.</pos> <amorph>[<pos>Compar.</pos> <adjf>Soother</adjf> <pr>(s<oomac/th"<etil/r)</pr>; <pos>superl.</pos> <adjf>Soothest</adjf>.]</amorph> <ety>[OE. <ets>soth</ets>, AS. <ets>s<omac/<edh/</ets>, for <ets>san<edh/</ets>; akin to OS. <ets>s<omac/<edh/</ets>, OHG. <ets>sand</ets>, Icel. <ets>sannr</ets>, Sw. <ets>sann</ets>, Dan. <ets>sand</ets>, Skr. <ets>sat</ets>, <ets>sant</ets>, real, genuine, present, being; properly p. pr. from a root meaning, to be, Skr. <ets>as</ets>, L. <ets>esse</ets>; also akin to Goth. <ets>sunjis</ets> true, Gr. <grk>'eteo`s</grk>, Skr. <ets>satya</ets>. <root/9.  Cf. <er>Absent</er>, <er>Am</er>, <er>Essence</er>, <er>Is</er>, <er>Soothe</er>, <er>Sutee</er>.]</ety> <sn>1.</sn> <def>True; faithful; trustworthy.</def> <mark>[Obs. or Scot.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>The sentence [meaning] of it <qex>sooth</qex> is, out of doubt.</q> <rj><qau>Chaucer.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>That shall I <qex>sooth</qex> (said he) to you declare.</q> <rj><qau>Spensser.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Pleasing; delightful; sweet.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>The <qex>soothest</qex> shepherd that ever piped on plains.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>With jellies <qex>soother</qex> than the creamy curd.</q> <rj><qau>Keats.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sooth</ent><br/
<hw>Sooth</hw>, <pos>n.</pos> <ety>[AS. <ets>s<omac/<edh/</ets>. See <er>Sooth</er>, <pos>a.</pos>]</ety> <sn>1.</sn> <def>Truth; reality.</def> <mark>[Archaic]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>The <qex>sooth</qex> it this, the cut fell to the knight.</q> <rj><qau>Chaucer.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>In <qex>sooth</qex>, I know not why I am so sad.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>In good <qex>sooth</qex>,<br/
Its mystery is love, its meaninng youth.</q> <rj><qau>Longfellow.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Augury; prognostication.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>The <qex>soothe</qex> of birds by beating of their wings.</q> <rj><qau>Spenser.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>Blandishment; cajolery.</def> <mark>[Obs.]</mark>  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Soothe</ent><br/
<hw>Soothe</hw> <pr>(s<oomac/<th/)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Soothed</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Soothing</conjf>.]</vmorph> <ety>[Originally, to assent to as true; OE. <ets>so<edh/ien</ets> to verify, AS. <ets>ges<omac/<edh/ian</ets> to prove the truth of, to bear witness. See <er>Sooth</er>, <pos>a.</pos>]</ety> <sn>1.</sn> <def>To assent to as true.</def> <mark>[Obs.]</mark>  <rj><au>Testament of Love.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To assent to; to comply with; to gratify; to humor by compliance; to please with blandishments or soft words; to flatter.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Good, my lord, <qex>soothe</qex> him, let him take the fellow.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>I've tried the force of every reason on him,<br/
<qex>Soothed</qex> and caressed, been angry, <qex>soothed</qex> again.</q> <rj><qau>Addison.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>To assuage; to mollify; to calm; to comfort; <as>as, to <ex>soothe</ex> a crying child; to <ex>soothe</ex> one's sorrows</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Music hath charms to <qex>soothe</qex> the savage breast,<br/
To soften rocks, or bend a knotted oak.</q> <rj><qau>Congreve.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Though the sound of Fame<br/
May for a moment <qex>soothe</qex>, it can not slake<br/
The fever of vain longing.</q> <rj><qau>Byron.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><syn><b>Syn.</b> -- To soften; assuage; allay; compose; mollify; tranquilize; pacify; mitigate.</syn><br/
[<source>1913 Webster</source>]</p>

<p><ent>Soother</ent><br/
<hw>Sooth"er</hw> <pr>(?)</pr>, <pos>n.</pos> <def>One who, or that which, soothes.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Soothfast</ent><br/
<hw>Sooth"fast`</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[<ets>Sooth</ets> + <ets>fast</ets>, that is, fast or firm with respect to truth.]</ety> <def>Firmly fixed in, or founded upon, the thruth; true; genuine; real; also, truthful; faithful.</def> <mark>[Archaic]</mark> -- <wordforms><wf>Sooth"fast`ness</wf>, <pos>n.</pos> <mark>[Archaic]</mark></wordforms>  <ldquo/In very <xex>soothfastness.</xex><rdquo/  <rj><au>Chaucer.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Why do not you . . . bear leal and <qex>soothfast</qex> evidence in her behalf, as ye may with a clear conscience!</q> <rj><qau>Sir W. Scott.</qau></rj>
<-- leal = loyal, but not marked as archaic in this work. --><br/
[<source>1913 Webster</source>]</p>

<p><ent>Soothfast</ent><br/
<hw>Sooth"fast`</hw>, <pos>adv.</pos> <def>Soothly; really; in fact.</def> <mark>[Archaic]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>I care not if the pomps you show<br/
Be what they <qex>soothfast</qex> appear.</q> <rj><qau>Emerson.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Soothing</ent><br/
<hw>Sooth"ing</hw> <pr>(?)</pr>, <def><pos>a. & n.</pos> from <er>Soothe</er>, <pos>v.</pos></def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Soothingly</ent><br/
<hw>Sooth"ing*ly</hw>, <pos>adv.</pos> <def>In a soothing manner.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Soothly</ent><br/
<hw>Sooth"ly</hw> <pr>(?)</pr>, <pos>adv.</pos> <def>In truth; truly; really; verily.</def> <mark>[Obs.]</mark> <ldquo/<xex>Soothly</xex> for to say.<rdquo/  <rj><au>Chaucer.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Soothness</ent><br/
<hw>Sooth"ness</hw>, <pos>n.</pos> <def>Truth; reality.</def> <mark>[Obs.]</mark>  <rj><au>Chaucer.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Soothsay</ent><br/
<hw>Sooth"say`</hw> <pr>(?)</pr>, <pos>v. i.</pos> <ety>[<ets>Sooth</ets> + <ets>say</ets>; properly to say truth, tell the truth.]</ety> <def>To foretell; to predict.</def> <ldquo/You can not <xex>soothsay</xex>.<rdquo/ <au>Shak.</au> <ldquo/Old <xex>soothsaying</xex> Glaucus' spell.<rdquo/ <au>Milton.</au><br/
[<source>1913 Webster</source>]</p>

<p><ent>Soothsay</ent><br/
<hw>Sooth"say`</hw>, <pos>n.</pos> <sn>1.</sn> <def>A true saying; a proverb; a prophecy.</def> <mark>[Obs.]</mark>  <rj><au>Spenser.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Omen; portent. Having</def><br/
[<source>1913 Webster</source>]</p>

<p><q>God turn the same to good <qex>soothsay</qex>.</q> <rj><qau>Spenser.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Soothsayer</ent><br/
<hw>Sooth"say`er</hw> <pr>(?)</pr>, <pos>n.</pos> <sn>1.</sn> <def>One who foretells events by the art of soothsaying; a prognosticator.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Zool.)</fld> <def>A mantis.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Soothsaying</ent><br/
<hw>Sooth"say`ing</hw>, <pos>n.</pos> <sn>1.</sn> <def>A true saying; truth.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>The act of one who soothsays; the foretelling of events; the art or practice of making predictions.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>A damsel, possessed with a spirit of divination . . . which brought her masters much gain by <qex>soothsaying</qex>.</q> <rj><qau>Acts xvi. 16.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>A prediction; a prophecy; a prognostication.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Divinations and <qex>soothsayings</qex> and dreams are vain.</q> <rj><qau>Eclus. xxxiv. 5.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sootiness</ent><br/
<hw>Soot"i*ness</hw> <pr>(?)</pr>, <pos>n.</pos> <def>The quality or state of being sooty; fuliginousness.</def>  <rj><au>Johnson.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sootish</ent><br/
<hw>Soot"ish</hw>, <pos>a.</pos> <def>Sooty.</def>  <rj><au>Sir T. Browne.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sooty</ent><br/
<hw>Soot"y</hw> <pr>(?)</pr>, <pos>a.</pos> <amorph>[<pos>Compar.</pos> <adjf>Sootier</adjf> <pr>(?)</pr>; <pos>superl.</pos> <adjf>Sootiest</adjf>.]</amorph> <ety>[AS. <ets>s<?/tig</ets>. See <er>Soot</er>.]</ety> <sn>1.</sn> <def>Of or pertaining to soot; producing soot; soiled by soot.</def> <ldquo/Fire of <xex>sooty</xex> coal.<rdquo/  <rj><au>Milton.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Having a dark brown or black color like soot; fuliginous; dusky; dark.</def> <ldquo/The grisly legions that troop under the <xex>sooty</xex> flag of Acheron.<rdquo/  <rj><au>Milton.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Sooty albatross</b></col> <fld>(Zool.)</fld>, <cd>an albatross (<spn>Phoebetria fuliginosa</spn>) found chiefly in the Pacific Ocean; -- called also <altname>nellie</altname>.</cd> -- <col><b>Sooty tern</b></col> <fld>(Zool.)</fld>, <cd>a tern (<spn>Sterna fuliginosa</spn>) found chiefly in tropical seas.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sooty</ent><br/
<hw>Soot"y</hw>, <pos>v. t.</pos> <def>To black or foul with soot.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q><qex>Sootied</qex> with noisome smoke.</q> <rj><qau>Chapman.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sop</ent><br/
<hw>Sop</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[OE. <ets>sop</ets>, <ets>soppe</ets>; akin to AS. <ets>s<?/pan</ets> to sup, to sip, to drink, D. <ets>sop</ets> sop, G. <ets>suppe</ets> soup, Icel. <ets>soppa</ets> sop. See <er>Sup</er>, <pos>v. t.</pos>, and cf. <er>Soup</er>.]</ety> <sn>1.</sn> <def>Anything steeped, or dipped and softened, in any liquid; especially, something dipped in broth or liquid food, and intended to be eaten.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>He it is to whom I shall give a <qex>sop</qex>, when I have dipped it.</q> <rj><qau>John xiii. 26.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q><qex>Sops</qex> in wine, quantity, inebriate more than wine itself.</q> <rj><qau>Bacon.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>The bounded waters<br/
Should lift their bosoms higher than the shores,<br/
And make a <qex>sop</qex> of all this solid globe.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Anything given to pacify; -- so called from the sop given to Cerberus, as related in mythology.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>All nature is cured with a <qex>sop</qex>.</q> <rj><qau>L'Estrange.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>A thing of little or no value.</def> <mark>[Obs.]</mark>  <rj><au>P. Plowman.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Sops in wine</b></col> <fld>(Bot.)</fld>, <cd>an old name of the clove pink, alluding to its having been used to flavor wine.</cd><br/
[<source>1913 Webster</source>]</p>

<p><q>Garlands of roses and <qex>sops in wine</qex>.</q> <rj><qau>Spenser.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p>-- <col><b>Sops of wine</b></col> <fld>(Bot.)</fld>, <cd>an old European variety of apple, of a yellow and red color, shading to deep red; -- called also <altname>sopsavine</altname>, and <altname>red shropsavine</altname>.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sop</ent><br/
<hw>Sop</hw>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Sopped</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Sopping</conjf>.]</vmorph> <def>To steep or dip in any liquid.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sope</ent><br/
<hw>Sope</hw> <pr>(?)</pr>, <pos>n.</pos> <def>See <er>Soap</er>.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Soph</ent><br/
<hw>Soph</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Eng. Univ.)</fld> <def>A contraction of <er>Soph ister</er>.</def> <mark>[Colloq.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Soph</ent><br/
<hw>Soph</hw>, <pos>n.</pos> <fld>(Amer. Colleges)</fld> <def>A contraction of <er>Sophomore</er>.</def> <mark>[Colloq.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sophi</ent><br/
<hw>So"phi</hw> <pr>(?)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Sophis</plw> <pr>(<?/)</pr>.</plu> <def>See <er>Sufi</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sophical</ent><br/
<ent>Sophic</ent><br/
<mhw>{ <hw>Soph"ic</hw> <pr>(?)</pr>, <hw>Soph"ic*al</hw> <pr>(?)</pr>, }</mhw> <pos>a.</pos> <ety>[Gr. <?/, fr. <?/ wise, <?/ wisdom.]</ety> <def>Teaching wisdom.</def> <mark>[Obs.]</mark>  <rj><au>S. Harris.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sophime</ent><br/
<hw>So*phime"</hw> <pr>(? <or/ ?)</pr>, <pos>n.</pos> <ety>[OF. <ets>soffime</ets>, <ets>sophisme</ets>.]</ety> <def>Sophism.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>I trow ye study aboute some <qex>sophime</qex>.</q> <rj><qau>Chaucer.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sophism</ent><br/
<hw>Soph"ism</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F. <ets>sophisme</ets>, L. <ets>sophisma</ets>, fr. Gr. <?/, fr. <?/ to make wise, <?/ to be become wise, to play the sophist, fr. <?/ wise.]</ety> <def>The doctrine or mode of reasoning practiced by a sophist; hence, any fallacy designed to deceive.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>When a false argument puts on the appearance of a true one, then it is properly called a <qex>sophism</qex>, or <ldquo/fallacy<rdquo/.</q> <rj><qau>I. Watts.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Let us first rid ourselves of <qex>sophisms</qex>, those of depraved men, and those of heartless philosophers.</q> <rj><qau>I. Taylor.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sophist</ent><br/
<hw>Soph"ist</hw>, <pos>n.</pos> <ety>[F. <ets>sophiste</ets>, L. <ets>sophistes</ets>, fr. Gr. <?/. See <er>Sophism</er>.]</ety> <sn>1.</sn> <def>One of a class of men who taught eloquence, philosophy, and politics in ancient Greece; especially, one of those who, by their fallacious but plausible reasoning, puzzled inquirers after truth, weakened the faith of the people, and drew upon themselves general hatred and contempt.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Many of the <qex>Sophists</qex> doubdtless card not for truth or morality, and merely professed to teach how to make the worse appear the better reason; but there scems no reason to hold that they were a special class, teaching special opinions; even Socrates and Plato were sometimes styled <qex>Sophists</qex>.</q> <rj><qau>Liddell & Scott.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Hence, an impostor in argument; a captious or fallacious reasoner.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sophister</ent><br/
<hw>Soph"ist*er</hw> <pr>(?)</pr>, <pos>n.</pos> <sn>1.</sn> <def>A sophist. See <er>Sophist</er>.</def> <mark>[Obs.]</mark>  <rj><au>Hooker.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Eng. Univ.)</fld> <def>A student who is advanced beyond the first year of his residence.</def><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ The entire course at the university consists of three years and one term, during which the students have the titles of <xex>first-year men</xex>, or <xex>freshmen</xex>; <xex>second-year men</xex> or <xex>junior sophs</xex> or <xex>sophisters</xex>; <xex>third-year men</xex>, or <xex>senior sophs</xex> or <xex>sophisters</xex>; and, in the last term, <xex>questionists</xex>, with reference to the approaching examination. In the older American colleges, the junior and senior classes were originally called, and in some of them are still called, <xex>junior sophisters</xex> and <xex>senior sophisters</xex>.</note><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sophister</ent><br/
<hw>Soph"ist*er</hw> <pr>(?)</pr>, <pos>v. t.</pos> <def>To maintain by sophistry, or by a fallacious argument.</def> <mark>[Obs.]</mark>  <rj><au>obham.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sophistical</ent><br/
<ent>Sophistic</ent><br/
<mhw>{ <hw>So*phis"tic</hw> <pr>(?)</pr>, <hw>So*phis"tic*al</hw>  }</mhw>, <pos>a.</pos> <ety>[L. <ets>sophisticus</ets>, Gr. <?/: cf. F. <ets>sophistique</ets>.]</ety> <def>Of or pertaining to a sophist; embodying sophistry; fallaciously subtile; not sound.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>His argument . . . is altogether <qex>sophistical</qex>.</q> <rj><qau>Macaulay.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p>-- <wordforms><wf>So*phis"tic*al*ly</wf>, <pos>adv.</pos> -- <wf>So*phis"tic*al*ness</wf>, <pos>n.</pos></wordforms><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sophisticate</ent><br/
<hw>So*phis"ti*cate</hw> <pr>(?)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Sophisticated</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Sophisticating</conjf>.]</vmorph> <ety>[LL. <ets>sophisticatus</ets>, p. p. of <ets>sophisticare</ets> to sophisticate.]</ety> <def>To render worthless by admixture; to adulterate; to damage; to pervert; <as>as, to <ex>sophisticate</ex> wine</as>.</def>  <rj><au>Howell.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>To <qex>sophisticate</qex> the understanding.</q> <rj><qau>Southey.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Yet Butler professes to stick to plain facts, not to <qex>sophisticate</qex>, not to refine.</q> <rj><qau>M. Arnold.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>They purchase but <qex>sophisticated</qex> ware.</q> <rj><qau>Dryden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><syn><b>Syn.</b> -- To adulterate; debase; corrupt; vitiate.</syn><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sophisticated</ent><br/
<ent>Sophisticate</ent><br/
<mhw>{ <hw>So*phis"ti*cate</hw> <pr>(?)</pr>, <hw>So*phis"ti*ca`ted</hw> <pr>(?)</pr>, }</mhw> <pos>a.</pos> <def>Adulterated; not pure; not genuine.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>So truth, while only one supplied the state,<br/
Grew scare and dear, and yet <qex>sophisticate</qex>.</q> <rj><qau>Dryden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sophistication</ent><br/
<hw>So*phis`ti*ca"tion</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Cf. LL. <ets>sophisticatio</ets>, F. <ets>sophistication</ets>.]</ety> <def>The act of sophisticating; adulteration; <as>as, the <ex>sophistication</ex> of drugs</as>.</def>  <rj><au>Boyle.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sophisticator</ent><br/
<hw>So*phis"ti*ca`tor</hw> <pr>(?)</pr>, <pos>n.</pos> <def>One who sophisticates.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sophistry</ent><br/
<hw>Soph"ist*ry</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[OE. <ets>sophistrie</ets>, OF. <ets>sophisterie</ets>.]</ety> <sn>1.</sn> <def>The art or process of reasoning; logic.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>The practice of a sophist; fallacious reasoning; reasoning sound in appearance only.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The juggle of <qex>sophistry</qex> consists, for the most part, in usig a word in one sense in the premise, and in another sense in the conclusion.</q> <rj><qau>Coleridge.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><syn><b>Syn.</b> -- See <er>Fallacy</er>.</syn><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sophomore</ent><br/
<hw>Soph"o*more</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Probably fr. <ets>soph</ets> or <ets>sophister</ets> + Gr. <?/ foolish. The word was probably introduced into the United States at an early date, from the University of Cambridge, England. Among the cant terms at that university, as given in the Gradus ad Cantabrigiam, we find <ets>Soph-Mor</ets> as <ldquo/the next distinctive appellation to Freshman,<rdquo/ but the term has now almost ceased to be known at the English university from whence it came.]</ety> <def>One belonging to the second of the four classes in an American college, or one next above a freshman.</def> <altsp>[Formerly written also <asp>sophimore</asp>.]</altsp><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sophomorical</ent><br/
<ent>Sophomoric</ent><br/
<mhw>{ <hw>Soph`o*mor"ic</hw> <pr>(?)</pr>, <hw>Soph`o*mor"ic*al</hw> <pr>(?)</pr>, }</mhw> <pos>a.</pos> <def>Of or pertaining to a sophomore; resembling a sophomore; hence, pretentious; inflated in style or manner; <as>as, <ex>sophomoric</ex> affectation</as>.</def> <mark>[U. S.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><-- 2. Poorly informed yet overconfident of one's own knowledge; asserting incorrect facts based on scant knowledge. --><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sophora</ent><br/
<hw>So*pho"ra</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Ar. <ets><?/ufair</ets>.]</ety> <fld>(Bot.)</fld> <sd>(a)</sd> <def>A genus of leguminous plants.</def> <sd>(b)</sd> <def>A tree (<spn>Sophora Japonica</spn>) of Eastern Asia, resembling the common locust; occasionally planted in the United States.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sophta</ent><br/
<hw>Soph"ta</hw> <pr>(?)</pr>, <pos>n.</pos> <def>See <er>Softa</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sopite</ent><br/
<hw>So"pite</hw> <pr>(?)</pr>, <pos>v. t.</pos> <ety>[L. <ets>sopitus</ets>, p. p. of <ets>sopire</ets> to put to sleep; akin to <ets>sopor</ets> a sleeping draught, a heavy sleep.]</ety> <def>To lay asleep; to put to sleep; to quiet.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>The king's declaration for the <qex>sopiting</qex> of all Arminian heresies.</q> <rj><qau>Fuller.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sopition</ent><br/
<hw>So*pi"tion</hw> <pr>(?)</pr>, <pos>n.</pos> <def>The act of putting to sleep, or the state of being put to sleep; sleep.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>Dementation and <qex>sopition</qex> of reason.</q> <rj><qau>Sir T. Browne.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sopor</ent><br/
||<hw>So"por</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L.]</ety> <fld>(Med.)</fld> <def>Profound sleep from which a person can be roused only with difficulty.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Soporate</ent><br/
<hw>Sop"o*rate</hw> <pr>(?)</pr>, <pos>v. t.</pos> <ety>[L. <ets>soporatus</ets>, p. p. or <ets>soporare</ets> to put to sleep, fr. <ets>sopor</ets> a heavy sleep.]</ety> <def>To lay or put to sleep; to stupefy.</def> <mark>[Obs.]</mark>  <rj><au>Cudworth.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Soporiferous</ent><br/
<hw>Sop`o*rif"er*ous</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>soparifer</ets>; <ets>sopor</ets> a heavy sleep + <ets>ferere</ets> to bring.]</ety> <def>Causing sleep; somniferous; soporific.</def> <ldquo/<xex>Soporiferous</xex> medicine.<rdquo/  <rj><au>Swift.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p>-- <wordforms><wf>Sop`o*rif"er*ous*ly</wf>, <pos>adv.</pos> -- <wf>Sop`o*rif"er*ous*ness</wf>, <pos>n.</pos></wordforms><br/
[<source>1913 Webster</source>]</p>

<p><ent>Soporific</ent><br/
<hw>Sop`o*rif"ic</hw> <pr>(?; 277)</pr>, <pos>a.</pos> <ety>[L. <ets>sopor</ets> a heavy sleep (akin to <ets>somnus</ets> sleep) + <ets>facere</ets> to make. See <er>Somnolent</er>, <er>Fact</er>.]</ety> <def>Causing sleep; tending to cause sleep; soporiferous; <as>as, the <ex>soporific</ex> virtues of opium</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><syn><b>Syn.</b> -- Somniferous; narcotic; opiate; anodyne.</syn><br/
[<source>1913 Webster</source>]</p>

<p><ent>Soporific</ent><br/
<hw>Sop`o*rif"ic</hw>, <pos>n.</pos> <def>A medicine, drug, plant, or other agent that has the quality of inducing sleep; a narcotic.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Soporous</ent><br/
<ent>Soporose</ent><br/
<mhw>{ <hw>Sop"o*rose`</hw> <pr>(?)</pr>, <hw>Sop"o*rous</hw> <pr>(?)</pr>, }</mhw> <pos>a.</pos> <ety>[From <er>Sopor</er>; cf. L. <ets>soporus</ets>, fr. <ets>sopor</ets> a heavy sleep; F. <ets>soporeux</ets>.]</ety> <def>Causing sleep; sleepy.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sopper</ent><br/
<hw>Sop"per</hw> <pr>(?)</pr>, <pos>n.</pos> <def>One who sops.</def>  <rj><au>Johnson.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>sopping</ent><br/
<-- <hw>sopping</hw>, more recent version of <it>soppy.</it> Used esp. in phrase <it>sopping wet</it>. --><br/
[<source>1913 Webster</source>]</p>

<p><ent>Soppy</ent><br/
<hw>Sop"py</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Soaked or saturated with liquid or moisture; very wet or sloppy.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>It [Yarmouth] looked rather spongy and <qex>soppy</qex>.</q> <rj><qau>Dickens.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sopra</ent><br/
||<hw>So"pra</hw> <pr>(?)</pr>, <pos>adv.</pos> <ety>[It., from L. <ets>supra</ets> above.]</ety> <fld>(Mus.)</fld> <def>Above; before; over; upon.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sopranist</ent><br/
<hw>So*pra"nist</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Mus.)</fld> <def>A treble singer.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Soprano</ent><br/
<hw>So*pra"no</hw> <pr>(?)</pr>, <pos>n.</pos>; <plu><it>pl.</it> E. <plw>Sopranos</plw> <pr>(#)</pr>, It. <plw>Soprani</plw> <pr>(#)</pr>.</plu> <ety>[It., fr. <ets>soprano</ets> superior, highest, fr. <ets>sopra</ets> above, L. <ets>supra</ets>. See <er>Sovereign</er>.]</ety> <fld>(Mus.)</fld> <sd>(a)</sd> <def>The treble; the highest vocal register; the highest kind of female or boy's voice; the upper part in harmony for mixed voices.</def> <sd>(b)</sd> <def>A singer, commonly a woman, with a treble voice.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sopsavine</ent><br/
<hw>Sops"a*vine</hw> <pr>(?)</pr>, <pos>n.</pos> <def>See <cref>Sops of wine</cref>, under <er>Sop</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><-- p. 1373 --></p>

<p><ent>Sora</ent><br/
<hw>So"ra</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Zool.)</fld> <def>A North American rail (<spn>Porzana Carolina</spn>) common in the Eastern United States. Its back is golden brown, varied with black and white, the front of the head and throat black, the breast and sides of the head and neck slate-colored. Called also <altname>American rail</altname>, <altname>Carolina rail</altname>, <altname>Carolina crake</altname>, <altname>common rail</altname>, <altname>sora rail</altname>, <altname>soree</altname>, <altname>meadow chicken</altname>, and <altname>orto</altname>.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>King sora</b></col>, <cd>the Florida gallinule.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sorance</ent><br/
<hw>Sor"ance</hw> <pr>(? <or/ ?)</pr>, <pos>n.</pos> <def>Soreness.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sorb</ent><br/
<hw>Sorb</hw> <pr>(?)</pr>, <pos>n.</pos><ety>[L. <ets>sorbus the tree</ets>, <ets>sorbum</ets> the fruit; cf. F. <ets>sorbe</ets>. See <er>Service tree</er>.]</ety> <fld>(Bot.)</fld> <sd>(a)</sd> <def>The wild service tree (<spn>Pyrus torminalis</spn>) of Europe; also, the rowan tree.</def> <sd>(b)</sd> <def>The fruit of these trees.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Sorb apple</b></col>, <cd>the fruit of the sorb, or wild service tree.</cd> -- <col><b>Sorb tree</b></col>, <cd>the wild service tree.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sorbate</ent><br/
<hw>Sor"bate</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Cf. F. <ets>sorbate</ets>. See <er>Sorbic</er>.]</ety> <fld>(Chem.)</fld> <def>A salt of sorbic acid.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sorbefacient</ent><br/
<hw>Sor`be*fa"cient</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>sorbere</ets> to suck in, absorb + <ets>faciens</ets>, p. pr. of <ets>facere</ets> to make.]</ety> <fld>(Med.)</fld> <def>Producing absorption.</def> -- <def2><pos>n.</pos> <def>A medicine or substance which produces absorption.</def></def2><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sorbent</ent><br/
<hw>Sorb"ent</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>sorbens</ets>, p. pr. of <ets>sorbere</ets> to suck in, to absorb.]</ety> <def>An absorbent.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sorbet</ent><br/
<hw>Sor"bet</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F. <ets>sorbet</ets> or It. <ets>sorbetto</ets> or Sp. <ets>sorbete</ets>, from the same source as E. <ets>sherbet</ets>. See <er>Sherbet</er>.]</ety> <def>A kind of beverage; sherbet.</def>  <rj><au>Smolett.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sorbic</ent><br/
<hw>Sor"bic</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[Cf. F. <ets>sorbique</ets>. See <er>Sorb</er>.]</ety> <fld>(Chem.)</fld> <def>Pertaining to, or obtained from, the rowan tree, or sorb; specifically, designating an acid, C<?/H<?/CO<?/H, of the acetylene series, found in the unripe berries of this tree, and extracted as a white crystalline substance.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sorbile</ent><br/
<hw>Sor"bile</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>sorbilis</ets>, fr. <ets>sorbere</ets> to suck in, to drink down.]</ety> <def>Fit to be drunk or sipped.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sorbin</ent><br/
<hw>Sor"bin</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Chem.)</fld> <def>An unfermentable sugar, isomeric with glucose, found in the ripe berries of the rowan tree, or sorb, and extracted as a sweet white crystalline substance; -- called also <altname>mountain-ash sugar</altname>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sorbite</ent><br/
<hw>Sor"bite</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>sorbus</ets> service tree.]</ety> <fld>(Chem.)</fld> <def>A sugarlike substance, isomeric with mannite and dulcite, found with sorbin in the ripe berries of the sorb, and extracted as a sirup or a white crystalline substance.</def> -- <wordforms><wf>Sor*bit"ic</wf> <pr>(#)</pr>, <pos>a.</pos></wordforms><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sorbition</ent><br/
<hw>Sor*bi"tion</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>sorbitio</ets>.]</ety> <def>The act of drinking or sipping.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><-- sorbitol. A hexitol, C6H14O6, found in the mountain ash, now produced by hydrogenation or electrolytic reduction of glucose.  It has a variet of uses, notably as a sugar substitute for diabetics, and to improve the shelf-life of candy, as an osmotic agent, and as an excipient in pharmaceutical preparations.  --><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sorbonical</ent><br/
<hw>Sor*bon"ic*al</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Belonging to the Sorbonne or to a Sorbonist.</def>  <rj><au>Bale.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sorbonist</ent><br/
<hw>Sor"bon*ist</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F. <ets>sorboniste</ets>.]</ety> <def>A doctor of the <xex>Sorbonne</xex>, or theological college, in the University of Paris, founded by Robert de <xex>Sorbon</xex>, <sc>a. d.</sc> 1252.  It was suppressed in the Revolution of 1789.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sorcerer</ent><br/
<hw>Sor"cer*er</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Cf. F. <ets>sorcier</ets>. See <er>Sorcery</er>.]</ety> <def>A conjurer; an enchanter; a magician.</def>  <rj><au>Bacon.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p>
<q>Pharaoh also called the wise men and the <qex>sorcerers</qex>.</q> <rj><qau>Ex. vii. 11.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sorceress</ent><br/
<hw>Sor"cer*ess</hw>, <pos>n.</pos> <def>A female sorcerer.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sorcering</ent><br/
<hw>Sor"cer*ing</hw>, <pos>n.</pos> <def>Act or practice of using sorcery.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sorcerous</ent><br/
<hw>Sor"cer*ous</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Of or pertaining to sorcery.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sorcery</ent><br/
<hw>Sor"cer*y</hw> <pr>(?)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Sorceries</plw> <pr>(#)</pr>.</plu> <ety>[OE. <ets>sorcerie</ets>, OF. <ets>sorcerie</ets>, fr. OF. & F. <ets>sorcier</ets> a sorcerer, LL. <ets>sortiarius</ets>, fr. L. <ets>sors</ets>, <ets>sortis</ets>, a lot, decision by lot, fate, destiny. See <er>Sort</er>, <pos>n.</pos>]</ety> <def>Divination by the assistance, or supposed assistance, of evil spirits, or the power of commanding evil spirits; magic; necromancy; witchcraft; enchantment.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Adder's wisdom I have learned,<br/
To fence my ear against thy <qex>sorceries</qex>.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sord</ent><br/
<hw>Sord</hw> <pr>(? <or/ ?)</pr>, <pos>n.</pos> <def>See <er>Sward</er>.</def> <mark>[R.]</mark>  <rj><au>Milton.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sordes</ent><br/
||<hw>Sor"des</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L., fr. <ets>sordere</ets> to be dirty or foul.]</ety> <def>Foul matter; excretion; dregs; filthy, useless, or rejected matter of any kind; specifically <fld>(Med.)</fld>, the foul matter that collects on the teeth and tongue in low fevers and other conditions attended with great vital depression.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sordet</ent><br/
<hw>Sor"det</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[See <er>Sordine</er>.]</ety> <fld>(Mus.)</fld> <def>A sordine.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sordid</ent><br/
<hw>Sor"did</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>sordidus</ets>, fr. <ets>sordere</ets> to be filthy or dirty; probably akin to E. <ets>swart</ets>: cf. F. <ets>sordide</ets>. See <er>Swart</er>, <pos>a.</pos>]</ety> <sn>1.</sn> <def>Filthy; foul; dirty.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>A <qex>sordid</qex> god; down from his hoary chin<br/
A length of beard descends, uncombed, unclean.</q> <rj><qau>Dryden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Vile; base; gross; mean; <as>as, vulgar, <ex>sordid</ex> mortals</as>.</def> <ldquo/To scorn the <xex>sordid</xex> world.<rdquo/  <rj><au>Milton.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>Meanly avaricious; covetous; niggardly.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>He may be old,<br/
And yet <qex>sordid</qex>, who refuses gold.</q> <rj><qau>Sir J. Denham.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sordidly</ent><br/
<hw>Sor*did"ly</hw> <pr>(?)</pr>, <pos>n.</pos> <def>Sordidness.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sordidly</ent><br/
<hw>Sor"did*ly</hw> <pr>(?)</pr>, <pos>adv.</pos> <def>In a sordid manner.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sordidness</ent><br/
<hw>Sor"did*ness</hw>, <pos>n.</pos> <def>The quality or state of being sordid.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sordine</ent><br/
<hw>Sor"dine</hw> <pr>(? <or/ ?; 277)</pr>, <pos>n.</pos> <ety>[It. <ets>sordina</ets>, <ets>sordino</ets>, from <ets>sordo</ets> deaf, dull-sounding, L. <ets>surdus</ets>. See <er>Surd</er>.]</ety> <fld>(Mus.)</fld> <def>See <er>Damper</er>, and 5th <er>Mute</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sore</ent><br/
<hw>Sore</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[F. <ets>saure</ets>, <ets>sore</ets>, <ets>sor</ets>; faucon <ets>sor</ets> a sore falcon. See <er>Sorrel</er>, <pos>n.</pos>]</ety> <def>Reddish brown; sorrel.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Sore falcon</b></col>. <fld>(Zool.)</fld> <cd>See <er>Sore</er>, <pos>n.</pos>, 1.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sore</ent><br/
<hw>Sore</hw>, <pos>n.</pos> <fld>(Zool.)</fld> <def>A young hawk or falcon in the first year.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Zool.)</fld> <def>A young buck in the fourth year. See the Note under <er>Buck</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sore</ent><br/
<hw>Sore</hw>, <pos>a.</pos> <amorph>[<pos>Compar.</pos> <adjf>Sorer</adjf> <pr>(<?/)</pr>; <pos>superl.</pos> <adjf>Sorest</adjf>.]</amorph> <ety>[OE. <ets>sor</ets>, <ets>sar</ets>, AS. <ets>s<amac/r</ets>; akin to D. <ets>zeer</ets>, OS. & OHG. <ets>s<?/r</ets>, G. <ets>sehr</ets> very, Icel. <ets>s<amac/rr</ets>, Sw. <ets>s<aring/r</ets>, Goth. <ets>sair</ets> pain.  Cf. <er>Sorry</er>.]</ety> <sn>1.</sn> <def>Tender to the touch; susceptible of pain from pressure; inflamed; painful; -- said of the body or its parts; <as>as, a <ex>sore</ex> hand</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Fig.: Sensitive; tender; easily pained, grieved, or vexed; very susceptible of irritation.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Malice and hatred are very fretting and vexatious, and apt to make our minds <qex>sore</qex> and uneasy.</q> <rj><qau>Tillotson.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>Severe; afflictive; distressing; <as>as, a <ex>sore</ex> disease; <ex>sore</ex> evil or calamity</as>.</def>  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>Criminal; wrong; evil.</def> <mark>[Obs.]</mark>  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Sore throat</b></col> <fld>(Med.)</fld>, <cd>inflammation of the throat and tonsils; pharyngitis. See <er>Cynanche</er>.</cd> -- <mcol><col><b>Malignant sore throat</b></col>, <col><b>Ulcerated sore throat</b></col> <it>or</it> <col><b>Putrid sore throat</b></col></mcol>. <cd>See <er>Angina</er>, and under <er>Putrid</er>.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sore</ent><br/
<hw>Sore</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[OE. <ets>sor</ets>, <ets>sar</ets>, AS. <ets>s<amac/r</ets>. See <er>Sore</er>, <pos>a.</pos>]</ety> <sn>1.</sn> <def>A place in an animal body where the skin and flesh are ruptured or bruised, so as to be tender or painful; a painful or diseased place, such as an ulcer or a boil.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The dogs came and licked his <qex>sores</qex>.</q> <rj><qau>Luke xvi. 21.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Fig.: Grief; affliction; trouble; difficulty.</def>  <rj><au>Chaucer.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>I see plainly where his <qex>sore</qex> lies.</q> <rj><qau>Sir W. Scott.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Gold sore</b></col>. <fld>(Med.)</fld> <cd>See under <er>Gold</er>, <pos>n.</pos></cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sore</ent><br/
<hw>Sore</hw>, <pos>adv.</pos> <ety>[AS. <ets>s<amac/re</ets>. See <er>Sore</er>, <pos>a.</pos>]</ety> <sn>1.</sn> <def>In a sore manner; with pain; grievously.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Thy hand presseth me <qex>sore</qex>.</q> <rj><qau>Ps. xxxviii. 2.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Greatly; violently; deeply.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>[Hannah] prayed unto the Lord and wept <qex>sore</qex>.</q> <rj><qau>1 Sam. i. 10.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q><qex>Sore</qex> sighed the knight, who this long sermon heard.</q> <rj><qau>Dryden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Soredia</ent><br/
||<hw>So*re"di*a</hw> <pr>(?)</pr>, <pos>n.</pos>, <def><pos>pl.</pos> of <er>Soredium</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sorediate</ent><br/
<hw>So*re"di*ate</hw> <pr>(?)</pr>, <pos>a.</pos> <fld>(Bot.)</fld> <def>Sorediiferous.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sorediiferous</ent><br/
<ent>Sorediferous</ent><br/
<mhw>{<hw>Sor`e*dif"er*ous</hw> <pr>(?)</pr>, <it>or</it> <hw>So*re`di*if"er*ous</hw> <pr>(?)</pr>, }</mhw> <pos>a.</pos> <ety>[<ets>Soredium</ets> + <ets>-ferous</ets>.]</ety> <fld>(Bot.)</fld> <def>Bearing soredia; sorediate.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Soredium</ent><br/
||<hw>So*re"di*um</hw> <pr>(?)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Soredia</plw> <pr>(#)</pr>.</plu> <ety>[NL., fr. Gr. <?/ a heap.]</ety> <fld>(Bot.)</fld> <def>A patch of granular bodies on the surface of the thallus of lichens.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Soree</ent><br/
<hw>So"ree</hw> <pr>(<?/)</pr>, <pos>n.</pos> <fld>(Zool.)</fld> <def>Same as <er>Sora</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sorehead</ent><br/
<hw>Sore"head`</hw> <pr>(?)</pr>, <pos>n.</pos> <def>One who is disgruntled by a failure in politics, or the like.</def> <mark>[Slang, U.S.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sorehon</ent><br/
<hw>Sore"hon</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Corrupted from <ets>sojourn</ets>, Scot. <ets>soirne</ets>, <ets>sorn</ets>.]</ety> <def>Formerly, in Ireland, a kind of servile tenure which subjected the tenant to maintain his chieftain gratuitously whenever he wished to indulge in a revel.</def>  <rj><au>Spenser.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sorel</ent><br/
<hw>Sor"el</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[A diminutive. See <er>Sore</er> reddish brown.]</ety> <sn>1.</sn> <fld>(Zool.)</fld> <def>A young buck in the third year. See the Note under <er>Buck</er>.</def>  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A yellowish or reddish brown color; sorrel.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sorely</ent><br/
<hw>Sore"ly</hw> <pr>(?)</pr>, <pos>adv.</pos> <def>In a sore manner; grievously; painfully; <as>as, to be <ex>sorely</ex> afflicted</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sorema</ent><br/
||<hw>So*re"ma</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[NL., fr. Gr. <?/ a heap.]</ety> <fld>(Bot.)</fld> <def>A heap of carpels belonging to one flower.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Soreness</ent><br/
<hw>Sore"ness</hw> <pr>(?)</pr>, <pos>n.</pos> <def>The quality or state of being sore; tenderness; painfull; <as>as, the <ex>soreness</ex> of a wound; the <ex>soreness</ex> of an affliction</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sorex</ent><br/
||<hw>So"rex</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L., a shrew.]</ety> <fld>(Zool.)</fld> <def>A genus of small Insectivora, including the common shrews.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sorghe</ent><br/
<hw>Sor"ghe</hw> <pr>(s<ocir/r"g<esl/)</pr>, <pos>n.</pos> <fld>(Zool.)</fld> <def>The three-bearded rockling, or whistlefish.</def> <mark>[Prov. Eng.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sorghum</ent><br/
<hw>Sor"ghum</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[NL., probably of Chinese origin.]</ety> <fld>(Bot.)</fld> <sd>(a)</sd> <def>A genus of grasses, properly limited to two species, <spn>Sorghum Halepense</spn>, the Arabian millet, or Johnson grass (see <er>Johnson grass</er>), and <spn>Sorghum vulgare</spn>, the Indian millet (see <cref>Indian millet</cref>, under <er>Indian</er>).</def> <sd>(b)</sd> <def>A variety of <spn>Sorghum vulgare</spn>, grown for its saccharine juice; the Chinese sugar cane.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sorgo</ent><br/
<hw>Sor"go</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Cf. It. <ets>sorgo</ets>. See <er>Sorghum</er>.]</ety> <fld>(Bot.)</fld> <def>Indian millet and its varieties. See <er>Sorghum</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sori</ent><br/
||<hw>So"ri</hw> <pr>(?)</pr>, <pos>n.</pos>, <def><pos>pl.</pos> of <er>Sorus</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Soricine</ent><br/
<hw>So"ri*cine</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>sorricinus</ets>, fr. <ets>sorex</ets> a shrew.]</ety> <fld>(Zool.)</fld> <def>Of or pertaining to the Shrew family (<fam>Soricidae</fam>); like a shrew in form or habits; <as>as, the <ex>soricine</ex> bat (<spn>Glossophaga soricina</spn>)</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sorites</ent><br/
<hw>So*ri"tes</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L., from Gr. <grk>swrei`ths</grk> (sc. <grk>syllogismo`s</grk>), properly, heaped up (hence, a heap of syllogisms), fr. <grk>swro`s</grk> a heap.]</ety> <fld>(Logic)</fld> <def>An abridged form of stating of syllogisms in a series of propositions so arranged that the predicate of each one that precedes forms the subject of each one that follows, and the conclusion unites the subject of the first proposition with the predicate of the last proposition</def>, as in following example; --<br/
[<source>1913 Webster</source>]</p>

<p><q>The soul is a thinking agent;<br/
A thinking agent can not be severed into parts;<br/
That which can not be severed can not be destroyed;<br/
Therefore the soul can not be destroyed.</q><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ When the series is arranged in the reverse order, it is called the <xex>Goclenian sorites</xex>, from Goclenius, a philosopher of the sixteenth century.</note><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Destructive sorities</b></col>. <cd>See under <er>Destructive</er>.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><-- Sorites paradox, (philos.)  The paradox that arises from the assertion that if one item is removed from a heap (sorites) of objects, what remains is still a heap.  Continued application of that rule for any finite heap ultimately causes a contradiction, when the "heap" has no objects left.  Similar definitional problems prompted the invention of "fuzzy logic" --><br/
[<source>1913 Webster</source>]</p>

<p><ent>Soritical</ent><br/
<hw>So"rit"ic*al</hw>, <pos>a.</pos> <def>Of or pertaining to a sorites; resembling a sorites.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sorn</ent><br/
<hw>Sorn</hw> <pr>(?)</pr>, <pos>v. i.</pos> <ety>[See <er>Sorehon</er>.]</ety> <def>To obtrude one's self on another for bed and board.</def> <mark>[Scot.]</mark>  <rj><au>Sir W. Scott.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sorner</ent><br/
<hw>Sorn"er</hw> <pr>(?)</pr>, <pos>n.</pos> <def>One who obtrudes himself on another for bed and board.</def> <mark>[Scot.]</mark>  <rj><au>De Quncey.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sororal</ent><br/
<hw>So*ro"ral</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>soror</ets> sister: cf. F. <ets>sororal</ets>.]</ety> <def>Relating to a sister; sisterly.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sororicide</ent><br/
<hw>So*ror"i*cide</hw> <pr>(?; 277)</pr>, <pos>n.</pos> <ety>[L. <ets>sororocida</ets>, and <ets>sororicidium</ets>; <ets>soror</ets> a sister + <ets>caedere</ets> to kill.]</ety> <def>The murder of one's sister; also, one who murders or kills one's own sister.</def>  <rj><au>Johnson.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sororize</ent><br/
<hw>So*ro"rize</hw> <pr>(? <or/ ?)</pr>, <pos>v. i.</pos> <ety>[L. <ets>soror</ets>, <ets>sororis</ets>, a sister.]</ety> <def>To associate, or hold fellowship, as sisters; to have sisterly feelings; -- analogous to <xex>fraternize</xex>.</def> <mark>[Recent & R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sorosis</ent><br/
<hw>So*ro"sis</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[NL. See <er>Sororize</er>.]</ety> <def>A woman's club; an association of women.</def> <mark>[U. S.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sorosis</ent><br/
||<hw>So*ro"sis</hw>, <pos>n.</pos> <ety>[NL., fr. Gr. <?/ a heap.]</ety> <fld>(Bot.)</fld> <def>A fleshy fruit formed by the consolidation of many flowers with their receptacles, ovaries, etc., as the breadfruit, mulberry, and pineapple.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sorrage</ent><br/
<hw>Sor"rage</hw> <pr>(?; 48)</pr>, <pos>n.</pos> <ety>[Cf. <er>Sorrel</er>, <pos>n.</pos>]</ety> <def>The blades of green or barley.</def> <mark>[Obs.]</mark>  <rj><au>Bailey.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sorrance</ent><br/
<hw>Sor"rance</hw> <pr>(?)</pr>, <pos>n.</pos> <def>Same as <er>Sorance</er>.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sorrel</ent><br/
<hw>Sor"rel</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[F. <ets>saur</ets>, <ets>saure</ets>, OF. <ets>sor</ets>, <ets>sore</ets>, probably of Teutonic origin; cf. D. <ets>zoor</ets> dry, LG. <ets>soor</ets>; the meaning probably coming from the color of dry leaves. See <er>Sear</er>, <pos>a.</pos>, and cf. <er>Sorel</er>.]</ety> <def>Of a yellowish or redish brown color; <as>as, a <ex>sorrel</ex> horse</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sorrel</ent><br/
<hw>Sor"rel</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A yellowish or redish brown color.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sorrel</ent><br/
<hw>Sor"rel</hw>, <pos>n.</pos> <ety>[F. <ets>surelle</ets>, fr. <ets>sur</ets> sour, fr. OHG. <ets>s<?/r</ets> sour. See <er>Sour</er>.]</ety> <fld>(Bot.)</fld> <def>One of various plants having a sour juice; especially, a plant of the genus <gen>Rumex</gen>, as <spn>Rumex Acetosa</spn>, <spn>Rumex Acetosella</spn>, etc.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Mountain sorrel</b></col>. <fld>(Bot.)</fld> <cd>See under <er>Mountain</er>.</cd> -- <col><b>Red sorrel</b></col>. <fld>(Bot.)</fld> <sd>(a)</sd> <cd>A malvaceous plant (<spn>Hibiscus Sabdariffa</spn>) whose acid calyxes and capsules are used in the West Indies for making tarts and acid drinks</cd>. <sd>(b)</sd> <cd>A troublesome weed (<spn>Rumex Acetosella</spn>), also called <altname>sheep sorrel</altname>.</cd> -- <col><b>Salt of sorrel</b></col> <fld>(Chem.)</fld>, <cd>binoxalate of potassa; -- so called because obtained from the juice of <spn>Rumex Acetosella</spn>, or <spn>Rumex Axetosa</spn>.</cd> -- <col><b>Sorrel tree</b></col> <fld>(Bot.)</fld>, <cd>a small ericaceous tree (<spn>Oxydendrum arboreum</spn>) whose leaves resemble those of the peach and have a sour taste. It is common along the Alleghanies. Called also <altname>sourwood</altname>.</cd> -- <col><b>Wood sorrel</b></col> <fld>(Bot.)</fld>, <cd>any plant of the genus Oxalis.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sorrento work</ent><br/
<hw>Sor"ren"to work`</hw> <pr>(?)</pr>. <def>Ornamental work, mostly carved in olivewood, decorated with inlay, made at or near Sorrento, Italy. Hence, more rarely, jig-saw work and the like done anywhere.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sorrily</ent><br/
<hw>Sor"ri*ly</hw> <pr>(?)</pr>, <pos>adv.</pos> <def>In a sorry manner; poorly.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Thy pipe, O Pan, shall help, though I sing <qex>sorrily</qex>.</q> <rj><qau>Sir P. Sidney.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sorriness</ent><br/
<hw>Sor"ri*ness</hw>, <pos>n.</pos> <def>The quality or state of being sorry.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sorrow</ent><br/
<hw>Sor"row</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[OE. <ets>sorwe</ets>, <ets>sorewe</ets>, <ets>sor<?/e</ets>, AS. <ets>sorg</ets>, <ets>sorh</ets>; akin to D. <ets>zorg</ets> care, anxiety, OS. <ets>sorga</ets>, OHG. <ets>sorga</ets>, <ets>soraga</ets>, <ets>suorga</ets>, G. <ets>sorge</ets>, Icel., Sw., & Dan. <ets>sorg</ets>, Goth. <ets>sa<uacute/rga</ets>; of unknown origin.]</ety> <def>The uneasiness or pain of mind which is produced by the loss of any good, real or supposed, or by diseappointment in the expectation of good; grief at having suffered or occasioned evil; regret; unhappiness; sadness.</def>  <rj><au>Milton.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>How great a <qex>sorrow</qex> suffereth now Arcite!</q> <rj><qau>Chaucer.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>The safe and general antidote against <qex>sorrow</qex> is employment.</q> <rj><qau>Rambler.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><syn><b>Syn.</b> -- Grief; unhappiness; regret; sadness; heaviness; mourning; affliction. See <er>Affliction</er>, and <er>Grief</er>.</syn><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sorrow</ent><br/
<hw>Sor"row</hw>, <pos>v. i.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Sorrowed</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Sorrowing</conjf>.]</vmorph> <ety>[OE. <ets>sorowen</ets>, <ets>sorwen</ets>, <ets>sorhen</ets>, AS. <ets>sorgian</ets>; akin to Goth. <ets>sa<uacute/rgan</ets>. See <er>Sorrow</er>, <pos>n.</pos>]</ety> <def>To feel pain of mind in consequence of evil experienced, feared, or done; to grieve; to be sad; to be sorry.</def><br/
[<source>1913 Webster</source>]</p>

<p><q><qex>Sorrowing</qex> most of all . . . that they should see his face no more.</q> <rj><qau>Acts xx. 38.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>I desire no man to <qex>sorrow</qex> for me.</q> <rj><qau>Sir J. Hayward.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sorrowed</ent><br/
<hw>Sor"rowed</hw> <pr>(?)</pr> <pr>(<?/)</pr>, <pos>a.</pos> <def>Accompanied with sorrow; sorrowful.</def> <mark>[Obs.]</mark>  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sorrowful</ent><br/
<hw>Sor"row*ful</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[OE. <ets>sorweful</ets>, AS. <ets>sorgful</ets>.]</ety> <sn>1.</sn> <def>Full of sorrow; exhibiting sorrow; sad; dejected; distressed.</def> <ldquo/This <xex>sorrowful</xex> prisoner.<rdquo/  <rj><au>Chaucer.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>My soul is exceeding <qex>sorrowful</qex>, even unto death.</q> <rj><qau>Matt. xxvi. 38.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Producing sorrow; exciting grief; mournful; lamentable; grievous; <as>as, a <ex>sorrowful</ex> accident</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><syn><b>Syn.</b> -- Sad; mournful; dismal; disconsolate; drear; dreary; grievous; lamentable; doleful; distressing.</syn><br/
[<source>1913 Webster</source>]</p>

<p>-- <wordforms><wf>Sor"row*ful*ly</wf>, <pos>adv.</pos> -- <wf>Sor"row*ful*ness</wf>, <pos>n.</pos></wordforms><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sorrowless</ent><br/
<hw>Sor"row*less</hw>, <pos>a.</pos> <def>Free from sorrow.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sorry</ent><br/
<hw>Sor"ry</hw> <pr>(?)</pr>, <pos>a.</pos> <amorph>[<pos>Compar.</pos> <adjf>Sorrier</adjf> <pr>(?)</pr>; <pos>superl.</pos> <adjf>Sorriest</adjf>.]</amorph> <ety>[OE. <ets>sory</ets>, <ets>sary</ets>, AS. <ets>s<amac/rig</ets>, fr. <ets>s<amac/r</ets>, <pos>n.</pos>, <ets>sore</ets>. <ets>See</ets> <er>Sore</er>, <pos>n.</pos> & <pos>a.</pos> The original sense was, painful; hence, miserable, sad.]</ety> <sn>1.</sn> <def>Grieved for the loss of some good; pained for some evil; feeling regret; -- now generally used to express light grief or affliction, but formerly often used to express deeper feeling.</def> <ldquo/I am <xex>sorry</xex> for my sins.<rdquo/  <rj><au>Piers Plowman.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Ye were made <qex>sorry</qex> after a godly manner.</q> <rj><qau>2 Cor. vii. 9.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>I am <qex>sorry</qex> for thee, friend; 't is the duke's pleasure.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>She entered, were he lief or <qex>sorry</qex>.</q> <rj><qau>Spenser.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Melancholy; dismal; gloomy; mournful.</def>  <rj><au>Spenser.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>All full of chirking was this <qex>sorry</qex> place.</q> <rj><qau>Chaucer.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>Poor; mean; worthless; <as>as, a <ex>sorry</ex> excuse</as>.</def> <ldquo/With <xex>sorry</xex> grace.<rdquo/  <rj><au>Chaucer.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Cheeks of <qex>sorry</qex> grain will serve.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q> Good fruit will sometimes grow on a <qex>sorry</qex> tree.</q> <rj><qau>Sir W. Scott.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><syn><b>Syn.</b> -- Hurt; afflicted; mortified; vexed; chagrined; melancholy; dismal; poor; mean; pitiful.</syn><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sors</ent><br/
||<hw>Sors</hw> <pr>(?)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Sortes</plw> <pr>(#)</pr>.</plu> <ety>[L.]</ety> <def>A lot; also, a kind of divination by means of lots.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><mcol><col><b>Sortes Homericae</b></col> <it>or</it> <col><b>Sortes Virgilianae</b></col></mcol> <ety>[L., Homeric or Virgilian lots]</ety>, <cd>a form of divination anciently practiced, which consisted in taking the first passage on which the eye fell, upon opening a volume of <persfn>Homer</persfn> or <persfn>Virgil</persfn>, or a passage drawn from an urn which several were deposited, as indicating future events, or the proper course to be pursued.  In later times the <book>Bible</book> was used for the same purpose by Christians.</cd></cs>
<-- not very different from Chinese fortune cookies in concept. --><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sort</ent><br/
<hw>Sort</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F. <ets>sorl</ets>, L. <ets>sors</ets>, <ets>sortis</ets>. See <er>Sort</er> kind.]</ety> <def>Chance; lot; destiny.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>By aventure, or <qex>sort</qex>, or cas [chance].</q> <rj><qau>Chaucer.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Let blockish Ajax draw<br/
The <qex>sort</qex> to fight with Hector.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><-- p. 1374 --></p>

<p><ent>Sort</ent><br/
<hw>Sort</hw>, <pos>n.</pos> <ety>[F. <ets>sorie</ets> (cf. It. <ets>sorta</ets>, <ets>sorte</ets>), from L. <ets>sors</ets>, <ets>sorti</ets>, a lot, part, probably akin to <ets>serere</ets> to connect. See <er>Series</er>, and cf. <er>Assort</er>, <er>Consort</er>, <er>Resort</er>, <er>Sorcery</er>, <er>Sort</er> lot.]</ety> <sn>1.</sn> <def>A kind or species; any number or collection of individual persons or things characterized by the same or like qualities; a class or order; <as>as, a <ex>sort</ex> of men; a <ex>sort</ex> of horses; a <ex>sort</ex> of trees; a <ex>sort</ex> of poems.</as></def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Manner; form of being or acting.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Which for my part I covet to perform,<br/
In <qex>sort</qex> as through the world I did proclaim.</q> <rj><qau>Spenser.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Flowers, in such <qex>sort</qex> worn, can neither be smelt nor seen well by those that wear them.</q> <rj><qau>Hooker.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>I'll deceive you in another <qex>sort</qex>.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>To Adam in what <qex>sort</qex><br/
Shall I appear?</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>I shall not be wholly without praise, if in some <qex>sort</qex> I have copied his style.</q> <rj><qau>Dryden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>Condition above the vulgar; rank.</def> <mark>[Obs.]</mark>  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>A chance group; a company of persons who happen to be together; a troop; also, an assemblage of animals.</def> <mark>[Obs.]</mark> <ldquo/A <xex>sort</xex> of shepherds.<rdquo/ <au>Spenser.</au> <ldquo/A <xex>sort</xex> of steers.<rdquo/ <au>Spenser.</au> <ldquo/A <xex>sort</xex> of doves.<rdquo/ <au>Dryden.</au> <ldquo/A <xex>sort</xex> of rogues.<rdquo/ <au>Massinger.</au><br/
[<source>1913 Webster</source>]</p>

<p><q>A boy, a child, and we a <qex>sort</qex> of us,<br/
Vowed against his voyage.</q> <rj><qau>Chapman.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <def>A pair; a set; a suit.</def>  <rj><au>Johnson.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>6.</sn> <pluf>pl.</pluf> <fld>(Print.)</fld> <def>Letters, figures, points, marks, spaces, or quadrats, belonging to a case, separately considered.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Out of sorts</b></col> <fld>(Print.)</fld>, <cd>with some letters or sorts of type deficient or exhausted in the case or font; hence, colloquially, out of order; ill; vexed; disturbed.</cd> -- <col><b>To run upon sorts</b></col> <fld>(Print.)</fld>, <cd>to use or require a greater number of some particular letters, figures, or marks than the regular proportion, as, for example, in making an index.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><syn><b>Syn.</b> -- Kind; species; rank; condition.</syn>  -- <usage><er>Sort</er>, <er>Kind</er>. <xex>Kind</xex> originally denoted things of the same family, or bound together by some natural affinity; and hence, a class. <xex>Sort</xex> signifies that which constitutes a particular lot of parcel, not implying necessarily the idea of affinity, but of mere assemblage. the two words are now used to a great extent interchangeably, though <xex>sort</xex> (perhaps from its original meaning of <xex>lot</xex>) sometimes carries with it a slight tone of disparagement or contempt, as when we say, that <xex>sort</xex> of people, that <xex>sort</xex> of language.</usage>
<-- surprisingly, "type" is not included in this synonym-list! 
in MW10, the list under "type" includes kind and sort.
 "class" is mentioned in the def, but not on the list of synonyms. --><br/
[<source>1913 Webster</source>]</p>

<p><q>As when the total <qex>kind</qex><br/
Of birds, in orderly array on wing,<br/
Came summoned over Eden to receive<br/
Their names of there.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>None of noble <qex>sort</qex><br/
Would so offend a virgin.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sort</ent><br/
<hw>Sort</hw> <pr>(?)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Sorted</conjf>; <pos>p. pr. & vb. n.</pos> <conjf>Sorting</conjf>.]</vmorph> <sn>1.</sn> <def>To separate, and place in distinct classes or divisions, as things having different qualities; <as>as, to <ex>sort</ex> cloths according to their colors; to <ex>sort</ex> wool or thread according to its fineness.</as></def><br/
[<source>1913 Webster</source>]</p>

<p><q>Rays which differ in refrangibility may be parted and <qex>sorted</qex> from one another.</q> <rj><qau>Sir I. Newton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To reduce to order from a confused state.</def>  <rj><au>Hooker.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>To conjoin; to put together in distribution; to class.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Shellfish have been, by some of the ancients, compared and <qex>sorted</qex> with insects.</q> <rj><qau>Bacon.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>She <qex>sorts</qex> things present with things past.</q> <rj><qau>Sir J. Davies.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>To choose from a number; to select; to cull.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>That he may <qex>sort</qex> out a worthy spouse.</q> <rj><qau>Chapman.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>I'll <qex>sort</qex> some other time to visit you.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <def>To conform; to adapt; to accommodate.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>I pray thee, <qex>sort</qex> thy heart to patience.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sort</ent><br/
<hw>Sort</hw>, <pos>v. i.</pos> <sn>1.</sn> <def>To join or associate with others, esp. with others of the same kind or species; to agree.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Nor do metals only <qex>sort</qex> and herd with metals in the earth, and minerals with minerals.</q> <rj><qau>Woodward.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>The illiberality of parents towards children makes them base, and <qex>sort</qex> with any company.</q> <rj><qau>Bacon.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To suit; to fit; to be in accord; to harmonize.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>They are happy whose natures <qex>sort</qex> with their vocations.</q> <rj><qau>Bacon.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Things <qex>sort</qex> not to my will.</q> <rj><qau>herbert.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>I can not tell you precisely how they <qex>sorted</qex>.</q> <rj><qau>Sir W. Scott.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sortable</ent><br/
<hw>Sort"a*ble</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[Cf. F. <ets>sortable</ets> suitable.]</ety> <sn>1.</sn> <def>Capable of being sorted.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Suitable; befitting; proper.</def> <mark>[Obs.]</mark>  <rj><au>con.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sortably</ent><br/
<hw>Sort"a*bly</hw>, <pos>adv.</pos> <def>Suitable.</def> <mark>[Obs.]</mark>  <rj><au>otgrave.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sortal</ent><br/
<hw>Sort"al</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Pertaining to a sort.</def> <mark>[Obs.]</mark>  <rj><au>Locke.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sortance</ent><br/
<hw>Sort"ance</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[From <er>Sort</er>, <pos>v. i.</pos>]</ety> <def>Suitableness; agreement.</def> <mark>[Obs.]</mark>  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sorter</ent><br/
<hw>Sort"er</hw> <pr>(?)</pr>, <pos>n.</pos> <def>One who, or that which, sorts.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sortes</ent><br/
||<hw>Sor"tes</hw> <pr>(?)</pr>, <pos>n.</pos>, <def><pos>pl.</pos> of <er>Sors</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sortie</ent><br/
<hw>Sor"tie</hw> <pr>(?; 277)</pr>, <pos>n.</pos> <ety>[F., fr. <ets>sortir</ets> to go out, to issue, probably fr. L. <ets>sortus</ets>, for <ets>surrectus</ets>, p. p. of <ets>surgere</ets> to raise up, to rise up. See <er>Source</er>.]</ety> <fld>(Mil.)</fld> <def>The sudden issuing of a body of troops, usually small, from a besieged place to attack or harass the besiegers; a sally.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sortilege</ent><br/
<hw>Sor"ti*lege</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F. <ets>sortil<egrave/ge</ets>, fr. L. <ets>sors</ets>, <ets>sortis</ets>, a lot + <ets>legere</ets> to gather, to select.]</ety> <def>The act or practice of drawing lots; divination by drawing lots.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>A woman infamous for <qex>sortileges</qex> and witcheries.</q> <rj><qau>Sir W. Scott.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sortilegious</ent><br/
<hw>Sor`ti*le"gious</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Pertaining to sortilege.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sortilegy</ent><br/
<hw>Sor"til"e*gy</hw> <pr>(?)</pr>, <pos>n.</pos> <def>Sortilege.</def> <mark>[R.]</mark>  <rj><au>De Quincey.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sortita</ent><br/
||<hw>Sor*ti"ta</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[It., a coming out.]</ety> <sn>1.</sn> <def>The air sung by any of the principal characters in an opera on entering.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><sn>2.</sn>  <def>A closing voluntary; a postlude.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Sortition</ent><br/
<hw>Sor*ti"tion</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>sortitio</ets>, from <ets>sortiri</ets> to draw or cast lots, fr. <ets>sors</ets>, <ets>sortis</ets>, a lot.]</ety> <def>Selection or appointment by lot.</def> <mark>[Obs.]</mark>  <rj><au>Bp. Hall.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sortment</ent><br/
<hw>Sort"ment</hw> <pr>(?)</pr>, <pos>n.</pos> <def>Assortiment.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sorus</ent><br/
||<hw>So"rus</hw> <pr>(?)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Sori</plw> <pr>(#)</pr>.</plu> <ety>[NL., fr. Gr. <?/ a heap.]</ety> <sn>1.</sn> <fld>(Bot.)</fld> <def>One of the fruit dots, or small clusters of sporangia, on the back of the fronds of ferns.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <sd>(a)</sd> <def>In parasitic fungi, any mass of spores bursting through the epidermis of a host plant.</def> <sd>(b)</sd> <def>In lichens, a heap of soredia on the thallus.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Sorwe</ent><br/
<hw>Sor"we</hw> <pr>(?)</pr>, <pos>n. & v.</pos> <def>Sorrow.</def> <mark>[Obs.]</mark>  <rj><au>Chaucer.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sorweful</ent><br/
<hw>Sor"we*ful</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Sorrowful.</def> <mark>[Obs.]</mark>  <rj><au>Chaucer.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sory</ent><br/
<hw>So"ry</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>sory</ets>, Gr. <?/.]</ety> <fld>(Old Min. Chem.)</fld> <def>Green vitriol, or some earth imregnated with it.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>SOS</ent><br/
<hw>SOS</hw>. <def>The letters signified by the signal ( . . . --- . . . ) prescribed by the International Radiotelegraphic Convention of 1912 for use by ships in distress.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>So-so</ent><br/
<hw>So"-so`</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[<ets>So</ets> + <ets>so</ets>.]</ety> <def>Neither very good nor very bad; middling; passable; tolerable; indifferent.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>In some Irish houses, where things are <qex>so-so</qex>,<br/
One gammon of bacon hangs up for a show.</q> <rj><qau>Goldsmith.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>He [Burns] certainly wrote some <qex>so-so</qex> verses to the Tree of Liberty.</q> <rj><qau>Prof. Wilson.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>So-so</ent><br/
<hw>So"-so`</hw>, <pos>adv.</pos> <def>Tolerably; passably.</def>  <rj><au>H. James.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Soss</ent><br/
<hw>Soss</hw> <pr>(?; 115)</pr>, <pos>v. i.</pos> <ety>[Cf. <er>Souse</er>.]</ety> <def>To fall at once into a chair or seat; to sit lazily.</def> <mark>[Obs.]</mark>  <rj><au>Swift.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Soss</ent><br/
<hw>Soss</hw>, <pos>v. t.</pos> <def>To throw in a negligent or careless manner; to toss.</def> <mark>[Obs.]</mark>  <rj><au>Swift.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Soss</ent><br/
<hw>Soss</hw>, <pos>n.</pos> <sn>1.</sn> <def>A lazy fellow.</def> <mark>[Obs.]</mark>  <rj><au>Cotgrave.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A heavy fall.</def> <mark>[Prov. Eng.]</mark>  <rj><au>Hallowell.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Soss</ent><br/
<hw>Soss</hw>, <pos>n.</pos> <ety>[See <er>Sesspol</er>.]</ety> <def>Anything dirty or muddy; a dirty puddle.</def> <mark>[Prov. Eng.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sostenuto</ent><br/
||<hw>Sos`te*nu"to</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[It.]</ety> <fld>(Mus.)</fld> <def>Sustained; -- applied to a movement or passage the sounds of which are to sustained to the utmost of the nominal value of the time; also, to a passage the tones of which are to be somewhat prolonged or protacted.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sot</ent><br/
<hw>Sot</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F., fr. LL. <ets>sottus</ets>; of unknown origin, cf. Ir. <ets>sotal</ets> pride, <ets>soithir</ets> proud, or Chald. & NHeb. <ets>shoten</ets> foolish.]</ety> <sn>1.</sn> <def>A stupid person; a blockhead; a dull fellow; a dolt.</def> <mark>[Obs.]</mark>  <rj><au>outh.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>In Egypt oft has seen the <qex>sot</qex> bow down,<br/
And reverence some d<?/ified baboon.</q> <rj><qau>Oldham.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A person stupefied by excessive drinking; an habitual drunkard.</def> <ldquo/A brutal <xex>sot</xex>.<rdquo/  <rj><au>Granville.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Every sign<br/
That calls the staring <qex>sots</qex> to nasty wine.</q> <rj><qau>Roscommon.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sot</ent><br/
<hw>Sot</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Sottish; foolish; stupid; dull.</def> <mark>[Obs.]</mark> <ldquo/Rich, but <xex>sot</xex>.<rdquo/  <rj><au>Marston.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sot</ent><br/
<hw>Sot</hw>, <pos>v. t.</pos> <def>To stupefy; to infatuate; to besot.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>I hate to see a brave, bold fellow <qex>sotted</qex>.</q> <rj><qau>Dryden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sot</ent><br/
<hw>Sot</hw>, <pos>v. i.</pos> <def>To tipple to stupidity.</def> <mark>[R.]</mark>  <rj><au>Goldsmith.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sotadean</ent><br/
<hw>So`ta*de"an</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Sotadic.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sotadic</ent><br/
<hw>So*tad"ic</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Pertaining to, or resembling, the lascivious compositions of the Greek poet <xex>Sotades</xex>.</def> -- <def2><pos>n.</pos> <def>A Sotadic verse or poem.</def></def2><br/
[<source>1913 Webster</source>]</p>

<p><ent>sote</ent><br/
<hw>sote</hw> <pr>(s<omac/t)</pr>, <pos>a.</pos> <def>Sweet.</def> <mark>[Obs.]</mark>  <rj><au>Chaucer.</au>  <au>Fairfax.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sotil</ent><br/
<ent>sotel</ent><br/
<mhw>{ <hw>so"tel</hw> <pr>(s<omac/"t<ecr/l)</pr>, <hw>So"til</hw> <pr>(?)</pr>, }</mhw> <pos>a.</pos> <def>Subtile.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>soteriology</ent><br/
<hw>so*te`ri*ol"o*gy</hw> <pr>(s<osl/*t<emac/`r<icr/*<ocr/l"<osl/*j<ycr/)</pr>, <pos>n.</pos> <ety>[Gr. <grk>swthri`a</grk> safety (from <grk>swth`rios</grk> saving, <grk>swth`r</grk> a savior, <grk>sw`zein</grk> to save) + <ets>-logy</ets>.]</ety> <sn>1.</sn> <def>A discourse on health, or the science of promoting and preserving health.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Theol.)</fld> <def>The doctrine of salvation by <person>Jesus Christ</person>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sothe</ent><br/
<hw>Sothe</hw> <pr>(s<omac/th <it>or</it> s<omac/"th<eit/)</pr>, <pos>a.</pos> <def>Sooth.</def> <mark>[Obs.]</mark>  <rj><au>Chaucer.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sothic</ent><br/
<ent>Sothiac</ent><br/
<mhw>{ <hw>So"thi*ac</hw> <pr>(?)</pr>, <hw>Soth"ic</hw> <pr>(?)</pr>, }</mhw> <pos>a.</pos> <def>Of or pertaining to <persfn>Sothis</persfn>, the Egyptian name for the Dog Star; taking its name from the Dog Star; canicular.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><mcol><col><b>Sothiac year</b></col>, <it>or</it> <col><b>Sothic year</b></col></mcol> <fld>(Chronol.)</fld>, <cd>the Egyptian year of 365 days and 6 hours, as distinguished from the Egyptian <xex>vague year</xex>, which contained 365 days.  The <xex>Sothic period</xex> consists of 1,460 Sothic years, being equal to 1,461 vague years.  One of these periods ended in July, <sc>a. d.</sc> 139.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sotilte</ent><br/
<hw>So"til*te</hw> <pr>(?)</pr>, <pos>n.</pos> <def>Subtlety.</def> <mark>[Obs.]</mark>  <rj><au>Chaucer.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sotted</ent><br/
<hw>Sot"ted</hw> <pr>(?)</pr>, <def><pos>a. & p. p.</pos> of <er>Sot</er>. Befooled; deluded; besotted.</def> <mark>[Obs.]</mark> <ldquo/This <xex>sotted</xex> priest.<rdquo/  <rj><au>Chaucer.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sottery</ent><br/
<hw>Sot"ter*y</hw> <pr>(?)</pr>, <pos>n.</pos> <def>Folly.</def> <mark>[Obs.]</mark>  <rj><au>Gauden.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sottish</ent><br/
<hw>Sot"tish</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[From <er>Sot</er>.]</ety> <def>Like a sot; doltish; very foolish; drunken.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>How ignorant are <qex>sottish</qex> pretenders to astrology!</q> <rj><qau>Swift.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><syn><b>Syn.</b> -- Dull; stupid; senseless; doltish; infatuate.</syn><br/
[<source>1913 Webster</source>]</p>

<p> -- <wordforms><wf>Sot"tish*ly</wf>, <pos>adv.</pos> -- <wf>Sot"tish*ness</wf>, <pos>n.</pos></wordforms><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sotto voce</ent><br/
||<hw>Sot`to vo"ce</hw> <pr>(?)</pr>. <ety>[It.]</ety> <sn>1.</sn> <fld>(Mus.)</fld> <def>With a restrained voice or moderate force; in an undertone.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Spoken low or in an undertone.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sou</ent><br/
<hw>Sou</hw> <pr>(?)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Sous</plw> <pr>(#)</pr> <it>or</it> <pr>(#)</pr>.</plu> <ety>[F. <ets>sou</ets>, OF. <ets>sol</ets>, from L. <ets>solidus</ets> a gold coin, in LL., a coin of less value. See <er>Sold</er>, <pos>n.</pos>, <er>Solid</er>, and and cf. <er>Sol</er>, <er>Soldo</er>.]</ety> <def>An old French copper coin, equivalent in value to, and now displaced by, the five-centime piece (<frac1x20/ of a franc), which is popularly called a <ex>sou</ex>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Souari nut</ent><br/
<hw>Sou*a"ri nut`</hw> <pr>(?)</pr>. <fld>(Bot.)</fld> <def>The large edible nutlike seed of a tall tropical American tree (<spn>Caryocar nuciferum</spn>) of the same natural order with the tea plant; -- also called <altname>butternut</altname>.</def> <altsp>[Written also <asp>sawarra nut</asp>.]</altsp><br/
[<source>1913 Webster</source>]</p>

<p><ent>Soubah</ent><br/
||<hw>Sou"bah</hw> <pr>(?)</pr>, <pos>n.</pos> <def>See <er>Subah</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Soubahdar</ent><br/
||<hw>Sou"bah*dar</hw> <pr>(?)</pr>, <pos>n.</pos> <def>See <er>Subahdar</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Soubise</ent><br/
||<hw>Sou`bise"</hw> <pr>(s<oomac/`b<emac/z")</pr>, <pos>n.</pos> <sn>1.</sn> <ety>[F.]</ety> <def>A sauce made of white onions and melted butter mixed with velout<eacute/ sauce.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><sn>2.</sn>  <def>A kind of cravat worn by men in the late 18th century.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Soubrette</ent><br/
<hw>Sou`brette"</hw>, <pos>n.</pos> <ety>[F.]</ety> <def>A female servant or attendant; specifically, as a term of the theater, a lady's maid, in comedies, who acts the part of an intrigante; a meddlesome, mischievous female servant or young woman.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Soubriquet</ent><br/
<hw>Sou`bri`quet"</hw> <pr>(?)</pr>, <pos>n.</pos> <def>See <er>Sobriquet</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Souce</ent><br/
<hw>Souce</hw> <pr>(?)</pr>, <pos>n.</pos> <def>See 1st <er>Souse</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Souce</ent><br/
<hw>Souce</hw>, <pos>v. t. & i.</pos> <def>See <er>Souse</er>.</def> <mark>[Obs.]</mark>  <rj><au>penser.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Souchong</ent><br/
<hw>Sou*chong"</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Chin. <ets>seou chong</ets> little plant or sort.]</ety> <def>A kind of black tea of a fine quality.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Soudan</ent><br/
<hw>Sou*dan"</hw> <pr>(?)</pr>, <pos>n.</pos><ety>[F.]</ety> <def>A sultan.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Soudet</ent><br/
<ent>Souded</ent><br/
<mhw><hw>Soud"ed</hw> <pr>(<?/)</pr>, <hw>Soud"et</hw> <pr>(<?/)</pr></mhw>, <pos>a.</pos> <ety>[See <er>Solder</er>.]</ety> <def>United; consolidated; made firm; strengthened.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>O martyr <qex>souded</qex> for virginity!</q> <rj><qau>Chaucer.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Souffle</ent><br/
<hw>Souf"fle</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F.]</ety> <fld>(Med.)</fld> <def>A murmuring or blowing sound; <as>as, the uterine <ex>souffle</ex> heard over the pregnant uterus</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Souffle</ent><br/
<hw>Souf"fl<eacute/</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F., fr. <ets>souffl<eacute/</ets>, p. p. of <ets>souffler</ets> to puff.]</ety> <fld>(Cookery)</fld> <def>A side dish served hot from the oven at dinner, made of eggs, milk, and flour or other farinaceous substance, beaten till very light, and flavored with fruits, liquors, or essence.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Souffle</ent><br/
<ent>Soufflee</ent><br/
||<hw>Souf`fl<eacute/"</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[F., fr. <ets>souffl<eacute/</ets>, p. p. of <ets>souffler</ets> to puff.]</ety> <sn>1.</sn> <fld>(Ceramics)</fld> <def>Decorated with very small drops or sprinkles of color, as if blown from a bellows.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><sn>2.</sn>  <fld>(Cookery)</fld> <altsp>Often ||<hw>Souffl<eacute/e</hw>.</altsp> <def>Filled with air by beating, and baked; <as>as, an omelette <ex>souffl<eacute/</ex></as>.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Sough</ent><br/
<hw>Sough</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A sow.</def> <mark>[Obs.]</mark>  <rj><au>Chaucer.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sough</ent><br/
<hw>Sough</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Etymol. uncertain.]</ety> <def>A small drain; an adit.</def> <mark>[Prov. Eng.]</mark>  <rj><au>W. M. Buchanan.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sough</ent><br/
<hw>Sough</hw> <pr>(?; 277)</pr>, <pos>n.</pos> <ety>[Cf. Icel. <ets>s<?/gr</ets> (in comp.) a rushing sound, or OE. <ets>swough</ets>, <ets>swogh</ets>, a sound, AS. <ets>sw<?/gan</ets> to rustle.  Cf. <er>Surf</er>, <er>Swoon</er>, <pos>v. i.</pos>]</ety> <sn>1.</sn> <def>The sound produced by soughing; a hollow murmur or roaring.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The whispering leaves or solemn <qex>sough</qex> of the forest.</q> <rj><qau>W. Howitt.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Hence, a vague rumor or flying report.</def> <mark>[Scot.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>A cant or whining mode of speaking, especially in preaching or praying.</def> <mark>[Scot.]</mark>  <rj><au>Jamieson.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sough</ent><br/
<hw>Sough</hw>, <pos>v. i.</pos> <def>To whistle or sigh, as the wind.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sought</ent><br/
<hw>Sought</hw> <pr>(?)</pr>, <def><pos>imp. & p. p.</pos> of <er>Seek</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Souke</ent><br/
<hw>Souke</hw> <pr>(?)</pr>, <pos>v. t. & i.</pos> <def>To suck.</def> <mark>[Obs.]</mark>  <rj><au>Chaucer.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Soul</ent><br/
<hw>Soul</hw> <pr>(s<omac/l)</pr>, <pos>a.</pos> <def>Sole.</def> <mark>[Obs.]</mark>  <rj><au>Chaucer.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Soul</ent><br/
<hw>Soul</hw>, <pos>v. i.</pos> <ety>[F. <ets>so<ucir/ler</ets> to satiate. See <er>Soil</er> to feed.]</ety> <def>To afford suitable sustenance.</def> <mark>[Obs.]</mark>  <rj><au>Warner.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Soul</ent><br/
<hw>Soul</hw>, <pos>n.</pos> <ety>[OE. <ets>soule</ets>, <ets>saule</ets>, AS. <ets>s<amac/wel</ets>, <ets>s<amac/wl</ets>; akin to OFries. <ets>s<?/le</ets>, OS. <ets>s<?/ola</ets>, D. <ets>ziel</ets>, G. <ets>seele</ets>, OHG. <ets>s<?/la</ets>, <ets>s<?/ula</ets>, Icel. <ets>s<amac/la</ets>, Sw. <ets>sj<aum/l</ets>, Dan. <ets>siael</ets>, Goth. <ets>saiwala</ets>; of uncertain origin, perhaps akin to L. <ets>saeculum</ets> a lifetime, age (cf. <er>Secular</er>.)]</ety> <sn>1.</sn> <def>The spiritual, rational, and immortal part in man; that part of man which enables him to think, and which renders him a subject of moral government; -- sometimes, in distinction from the higher nature, or spirit, of man, the so-called animal soul, that is, the seat of life, the sensitive affections and phantasy, exclusive of the voluntary and rational powers; -- sometimes, in distinction from the mind, the moral and emotional part of man's nature, the seat of feeling, in distinction from intellect; -- sometimes, the intellect only; the understanding; the seat of knowledge, as distinguished from <xex>feeling</xex>.  In a more general sense, <ldquo/an animating, separable, surviving entity, the vehicle of individual personal existence.<rdquo/</def>  <rj><au>Tylor.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>The eyes of our <qex>souls</qex> only then begin to see, when our bodily eyes are closing.</q> <rj><qau>Law.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>The seat of real life or vitality; the source of action; the animating or essential part.</def> <ldquo/The hidden <xex>soul</xex> of harmony.<rdquo/  <rj><au>Milton.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Thou sun, of this great world both eye and <qex>soul</qex>.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>The leader; the inspirer; the moving spirit; the heart; <as>as, the <ex>soul</ex> of an enterprise; an able general is the <ex>soul</ex> of his army</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>He is the very <qex>soul</qex> of bounty!</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>Energy; courage; spirit; fervor; affection, or any other noble manifestation of the heart or moral nature; inherent power or goodness.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>That he wants algebra he must confess;<br/
But not a <qex>soul</qex> to give our arms success.</q> <rj><qau>Young.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <def>A human being; a person; -- a familiar appellation, usually with a qualifying epithet; <as>as, poor <ex>soul</ex></as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>As cold waters to a thirsty <qex>soul</qex>, so is good news from a far country.</q> <rj><qau>Prov. xxv. 25.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>God forbid so many simple <qex>souls</qex><br/
Should perish by the sword!</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Now mistress Gilpin (careful <qex>soul</qex>).</q> <rj><qau>Cowper.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>6.</sn> <def>A pure or disembodied spirit.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>That to his only Son . . . every <qex>soul</qex> in heaven<br/
Shall bend the knee.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>7.</sn> <def>A perceived shared community and awareness among African-Americans.</def><br/
[<source>PJC</source>]</p>

<p><sn>8.</sn> <def>Soul music.</def><br/
[<source>PJC</source>]</p>

<p><note><hand/ <xex>Soul</xex> is used in the formation of numerous compounds, most of which are of obvious signification; as, <xex>soul</xex>-betraying, <xex>soul</xex>-consuming, <xex>soul</xex>-destroying, <xex>soul</xex>-distracting, <xex>soul</xex>-enfeebling, <xex>soul</xex>-exalting, <xex>soul</xex>-felt, <xex>soul</xex>-harrowing, <xex>soul</xex>-piercing, <xex>soul</xex>-quickening, <xex>soul</xex>-reviving, <xex>soul</xex>-stirring, <xex>soul</xex>-subduing, <xex>soul</xex>-withering, etc.</note><br/
[<source>1913 Webster</source>]</p>

<p><syn><b>Syn.</b> -- Spirit; life; courage; fire; ardor.</syn><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Cure of souls</b></col>. <cd>See <er>Cure</er>, <pos>n.</pos>, 2.</cd> -- <col><b>Soul bell</b></col>, <cd>the passing bell.</cd> <au>Bp. Hall.</au> -- <col><b>Soul foot</b></col>. <cd>See <cref>Soul scot</cref>, below.</cd> <mark>[Obs.]</mark> -- <col><b>Soul scot</b></col> <it>or</it> <col><b>Soul shot</b></col>. <ety>[<ets>Soul</ets> + <ets>scot</ets>, or <ets>shot</ets>; cf. AS. <ets>s<amac/welsceat</ets>.]</ety> <fld>(O. Eccl. Law)</fld> <cd>A funeral duty paid in former times for a requiem for the soul.</cd> <au>Ayliffe.</au> </cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Soul</ent><br/
<hw>Soul</hw> <pr>(s<omac/l)</pr>, <pos>v. t.</pos> <def>To indue with a soul; to furnish with a soul or mind.</def> <mark>[Obs.]</mark>  <rj><au>Chaucer.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>soul</ent><br/
<hw>soul</hw> <pr>(s<omac/l)</pr>, <pos>a.</pos> <def>By or for African-Americans, or characteristic of their culture; <as>as, <ex>soul</ex> music; <ex>soul</ex> newspapers; <ex>soul</ex> food</as>.</def><br/
[<source>PJC</source>]</p>

<p><ent>Souled</ent><br/
<hw>Souled</hw> <pr>(s<omac/ld)</pr>, <pos>a.</pos> <def>Furnished with a soul; possessing soul and feeling; -- used chiefly in composition; <as>as, great-<ex>souled</ex> Hector</as>.</def> <ldquo/Grecian chiefs . . . largely <xex>souled</xex>.<rdquo/  <rj><au>Dryden.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Soulili</ent><br/
||<hw>Sou"li*li`</hw> <pr>(s<oomac/"l<esl/*l<emac/`)</pr>, <pos>n.</pos> <fld>(Zool.)</fld> <def>A long-tailed, crested Javan monkey (<spn>Semnopithecus mitratus</spn>).  The head, the crest, and the upper surface of the tail, are black.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>soul kiss</ent><br/
<hw>soul" kiss`</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A kiss in which both parties have their mouths open and pressed together, and the tongue of one or both is maneuvered within the mouth of the other.</def><br/
<syn><b>Syn. --</b> French kiss, deep kiss.</syn><br/
[<source>PJC</source>]</p>

<p><ent>Soulless</ent><br/
<hw>Soul"less</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Being without a soul, or without greatness or nobleness of mind; mean; spiritless.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Slave, <qex>souless</qex> villain, dog!</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Soullessly</ent><br/
<hw>Soul"less*ly</hw>, <pos>adv.</pos> <def>In a soulless manner.</def>  <rj><au>Tylor.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Soun</ent><br/
<hw>Soun</hw> <pr>(?)</pr>, <pos>n. & v.</pos> <def>Sound.</def> <mark>[Obs.]</mark>  <rj><au>Chaucer.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sound</ent><br/
<hw>Sound</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[AS. <ets>sund</ets> a swimming, akin to E. <ets>swim</ets>. See <er>Swim</er>.]</ety> <def>The air bladder of a fish; <as>as, cod <ex>sounds</ex> are an esteemed article of food</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sound</ent><br/
<hw>Sound</hw>, <pos>n.</pos> <fld>(Zool.)</fld> <def>A cuttlefish.</def> <mark>[Obs.]</mark>  <rj><au>Ainsworth.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sound</ent><br/
<hw>Sound</hw>, <pos>a.</pos> <amorph>[<pos>Compar.</pos> <adjf>Sounder</adjf> <pr>(?)</pr>; <pos>superl.</pos> <adjf>Soundest</adjf>.]</amorph> <ety>[OE. <ets>sound</ets>, AS. <ets>sund</ets>; akin to D. ge<ets>zond</ets>, G. ge<ets>sund</ets>, OHG. gi<ets>sunt</ets>, Dan. & Sw. <ets>sund</ets>, and perhaps to L. <ets>sanus</ets>.  Cf. <er>Sane</er>.]</ety> <sn>1.</sn> <def>Whole; unbroken; unharmed; free from flaw, defect, or decay; perfect of the kind; <as>as, <ex>sound</ex> timber; <ex>sound</ex> fruit; a <ex>sound</ex> tooth; a <ex>sound</ex> ship</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Healthy; not diseased; not being in a morbid state; -- said of body or mind; <as>as, a <ex>sound</ex> body; a <ex>sound</ex> constitution; a <ex>sound</ex> understanding</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>Firm; strong; safe.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The brasswork here, how rich it is in beams,<br/
And how, besides, it makes the whole house <qex>sound</qex>.</q> <rj><qau>Chapman.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>Free from error; correct; right; honest; true; faithful; orthodox; -- said of persons; <as>as, a <ex>sound</ex> lawyer; a <ex>sound</ex> thinker</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Do not I know you a favorer<br/
Of this new seat? Ye are nor <qex>sound</qex>.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <def>Founded in truth or right; supported by justice; not to be overthrown on refuted; not fallacious; <as>as, <ex>sound</ex> argument or reasoning; a <ex>sound</ex> objection; <ex>sound</ex> doctrine; <ex>sound</ex> principles</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Hold fast the form of <qex>sound</qex> words, which thou hast heard of me.</q> <rj><qau>2 Tim. i. 13.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>6.</sn> <def>heavy; laid on with force; <as>as, a <ex>sound</ex> beating</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>7.</sn> <def>Undisturbed; deep; profound; <as>as, <ex>sound</ex> sleep</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>8.</sn> <def>Founded in law; legal; valid; not defective; <as>as, a <ex>sound</ex> title to land</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ <xex>Sound</xex> is sometimes used in the formation of self-explaining compounds; as, <xex>sound</xex>-headed, <xex>sound</xex>-hearted, <xex>sound</xex>-timbered, etc.</note><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Sound currency</b></col> <fld>(Com.)</fld>, <cd>a currency whose actual value is the same as its nominal value; a currency which does not deteriorate or depreciate or fluctuate in comparision with the standard of values.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sound</ent><br/
<hw>Sound</hw>, <pos>adv.</pos> <def>Soundly.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>So <qex>sound</qex> he slept that naught might him awake.</q> <rj><qau>Spenser.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sound</ent><br/
<hw>Sound</hw>, <pos>n.</pos> <ety>[AS. <ets>sund</ets> a narrow sea or strait; akin to Icel., Sw., Dan. & G. <ets>sund</ets>, probably so named because it could be <ets>swum</ets> across. See <er>Swim</er>.]</ety> <fld>(Geog.)</fld> <def>A narrow passage of water, or a strait between the mainland and an island; also, a strait connecting two seas, or connecting a sea or lake with the ocean; <as>as, the <ex>Sound</ex> between the Baltic and the german Ocean; Long Island <ex>Sound</ex></as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The <qex>Sound</qex> of Denmark, where ships pay toll.</q> <rj><qau>Camden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Sound dues</b></col>, <cd>tolls formerly imposed by Denmark on vessels passing through the Baltic Sound.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sound</ent><br/
<hw>Sound</hw>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Sounded</conjf>; <pos>p. pr. & vb. n.</pos> <conjf>Sounding</conjf>.]</vmorph> <ety>[F. <ets>sonder</ets>; cf. AS. <ets>sund</ets>gyrd a sounding rod, <ets>sund</ets>line a sounding line (see <er>Sound</er> a narrow passage of water).]</ety> <sn>1.</sn> <def>To measure the depth of; to fathom; especially, to ascertain the depth of by means of a line and plummet.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Fig.: To ascertain, or try to ascertain, the thoughts, motives, and purposes of (a person); to examine; to try; to test; to probe.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>I was in jest,<br/
And by that offer meant to <qex>sound</qex> your breast.</q> <rj><qau>Dryden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>I've <qex>sounded</qex> my Numidians man by man.</q> <rj><qau>Addison.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <fld>(Med.)</fld> <def>To explore, as the bladder or urethra, with a sound; to examine with a sound; also, to examine by auscultation or percussion; <as>as, to <ex>sound</ex> a patient</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><-- p. 1375 --></p>

<p><ent>Sound</ent><br/
<hw>Sound</hw> <pr>(?)</pr>, <pos>v. i.</pos> <def>To ascertain the depth of water with a sounding line or other device.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>I <qex>sound</qex> as a shipman <qex>soundeth</qex> in the sea with his plummet to know the depth of sea.</q> <rj><qau>Palsgrave.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sound</ent><br/
<hw>Sound</hw>, <pos>n.</pos> <ety>[F. <ets>sonde</ets>. See <er>Sound</er> to fathom.]</ety> <fld>(Med.)</fld> <def>Any elongated instrument or probe, usually metallic, by which cavities of the body are sounded or explored, especially the bladder for stone, or the urethra for a stricture.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sound</ent><br/
<hw>Sound</hw>, <pos>n.</pos> <ety>[OE. <ets>soun</ets>, OF. <ets>son</ets>, <ets>sun</ets>, F. <ets>son</ets>, fr. L. <ets>sonus</ets> akin to Skr. <ets>svana</ets> sound, <ets>svan</ets> to sound, and perh. to E. <ets>swan</ets>.  Cf. <er>Assonant</er>, <er>Consonant</er>, <er>Person</er>, <er>Sonata</er>, <er>Sonnet</er>, <er>Sonorous</er>, <er>Swan</er>.]</ety> <sn>1.</sn> <def>The peceived object occasioned by the impulse or vibration of a material substance affecting the ear; a sensation or perception of the mind received through the ear, and produced by the impulse or vibration of the air or other medium with which the ear is in contact; the effect of an impression made on the organs of hearing by an impulse or vibration of the air caused by a collision of bodies, or by other means; noise; report; <as>as, the <ex>sound</ex> of a drum; the <ex>sound</ex> of the human voice; a horrid <ex>sound</ex>; a charming <ex>sound</ex>; a sharp, high, or shrill <ex>sound</ex></as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The warlike <qex>sound</qex><br/
Of trumpets loud and clarions.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>The occasion of sound; the impulse or vibration which would occasion sound to a percipient if present with unimpaired; hence, the theory of vibrations in elastic media such cause sound; <as>as, a treatise on <ex>sound</ex></as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ In this sense, sounds are spoken of as <xex>audible</xex> and <xex>inaudible</xex>.</note><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>Noise without signification; empty noise; noise and nothing else.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Sense and not <qex>sound</qex> . . . must be the principle.</q> <rj><qau>Locke.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Sound boarding</b></col>, <cd>boards for holding pugging, placed in partitions of under floors in order to deaden sounds.</cd> -- <col><b>Sound bow</b></col>, <cd>in a series of transverse sections of a bell, that segment against which the clapper strikes, being the part which is most efficacious in producing the sound. See <xex>Illust.</xex> of <er>Bell</er>.</cd> -- <col><b>Sound post</b></col>. <fld>(Mus.)</fld> <cd>See <cref>Sounding post</cref>, under <er>Sounding</er>.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sound</ent><br/
<hw>Sound</hw>, <pos>v. i.</pos> <ety>[OE. <ets>sounen</ets>, <ets>sownen</ets>, OF. <ets>soner</ets>, <ets>suner</ets>, F. <ets>sonner</ets>, from L. <ets>sonare</ets>. See <er>Sound</er> a noise.]</ety> <sn>1.</sn> <def>To make a noise; to utter a voice; to make an impulse of the air that shall strike the organs of hearing with a perceptible effect.</def> <ldquo/And first taught speaking trumpets how to <xex>sound</xex>.<rdquo/  <rj><au>Dryden.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>How silver-sweet <qex>sound</qex> lovers' tongues!</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To be conveyed in sound; to be spread or published; to convey intelligence by sound.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>From you <qex>sounded</qex> out the word of the Lord.</q> <rj><qau>1 Thess. i. 8.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>To make or convey a certain impression, or to have a certain import, when heard; hence, to seem; to appear; <as>as, this reproof <ex>sounds</ex> harsh; the story <ex>sounds</ex> like an invention</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Good sir, why do you start, and seem to fear<br/
Things that do <qex>sound</qex> so fair?</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><mcol><col><b>To sound in</b></col> <it>or</it> <col><b>To sound into</b></col></mcol>, <cd>to tend to; to partake of the nature of; to be consonant with.</cd> <mark>[Obs., except in the phrase To sound in damages, below.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q><qex>Soun[d]ing in</qex> moral virtue was his speech.</q> <rj><qau>Chaucer.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p>-- <col><b>To sound in damages</b></col> <fld>(Law)</fld>, <cd>to have the essential quality of damages. This is said of an action brought, not for the recovery of a specific thing, as replevin, etc., but for damages only, as trespass, and the like.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sound</ent><br/
<hw>Sound</hw>, <pos>v. t.</pos> <sn>1.</sn> <def>To cause to make a noise; to play on; <as>as, to <ex>sound</ex> a trumpet or a horn; to <ex>sound</ex> an alarm</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>A bagpipe well could he play and <qex>soun</qex>[d].</q> <rj><qau>Chaucer.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To cause to exit as a sound; <as>as, to <ex>sound</ex> a note with the voice, or on an instrument</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>To order, direct, indicate, or proclain by a sound, or sounds; to give a signal for by a certain sound; <as>as, to <ex>sound</ex> a retreat; to <ex>sound</ex> a parley</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The clock <qex>sounded</qex> the hour of noon.</q> <rj><qau>G. H. Lewes.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>To celebrate or honor by sounds; to cause to be reported; to publish or proclaim; <as>as, to <ex>sound</ex> the praises of fame of a great man or a great exploit</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <def>To examine the condition of (anything) by causing the same to emit sounds and noting their character; <as>as, to <ex>sound</ex> a piece of timber; to <ex>sound</ex> a vase; to <ex>sound</ex> the lungs of a patient</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>6.</sn> <def>To signify; to import; to denote.</def> <mark>[Obs.]</mark>  <rj><au>Milton.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q><qex>Soun[d]ing</qex> alway the increase of his winning.</q> <rj><qau>Chaucer.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Soundable</ent><br/
<hw>Sound"a*ble</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Capable of being sounded.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Soundage</ent><br/
<hw>Sound"age</hw> <pr>(?; 48)</pr>, <pos>n.</pos> <def>Dues for soundings.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sound-board</ent><br/
<hw>Sound"-board`</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A sounding-board.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>To many a row of pipes the <qex>sound-board</qex> breathes.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sounder</ent><br/
<hw>Sound"er</hw> <pr>(?)</pr>, <pos>n.</pos> <def>One who, or that which; sounds; specifically, an instrument used in telegraphy in place of a register, the communications being read by sound.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sounder</ent><br/
<hw>Sound"er</hw>, <pos>n.</pos> <fld>(Zool.)</fld> <def>A herd of wild hogs.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sounding</ent><br/
<hw>Sound"ing</hw>, <pos>a.</pos> <def>Making or emitting sound; hence, sonorous; <as>as, <ex>sounding</ex> words</as>.</def>  <rj><au>Dryden.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sounding</ent><br/
<hw>Sound"ing</hw>, <pos>n.</pos> <sn>1.</sn> <def>The act of one who, or that which, sounds (in any of the senses of the several verbs).</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Naut.)</fld> <ety>[From <er>Sound</er> to fathom.]</ety> <sd>(a)</sd> <def>measurement by sounding; also, the depth so ascertained.</def> <sd>(b)</sd> <def>Any place or part of the ocean, or other water, where a sounding line will reach the bottom; -- usually in the plural.</def> <sd>(c)</sd> <def>The sand, shells, or the like, that are brought up by the sounding lead when it has touched bottom.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Sounding lead</b></col>, <cd>the plummet at the end of a sounding line.</cd> -- <col><b>Sounding line</b></col>, <cd>a line having a plummet at the end, used in making soundings.</cd> -- <col><b>Sounding post</b></col> <fld>(Mus.)</fld>, <cd>a small post in a violin, violoncello, or similar instrument, set under the bridge as a support, for propagating the sounds to the body of the instrument; -- called also <altname>sound post</altname>.</cd> -- <col><b>Sounding rod</b></col> <fld>(Naut.)</fld>, <cd>a rod used to ascertain the depth of water in a ship's hold.</cd> -- <col><b>In soundings</b></col>, <cd>within the eighty-fathom line.</cd>  <rj><au>Ham. Nav. Encyc.</au></rj></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sounding balloon</ent><br/
<hw>Sound"ing bal*loon"</hw>. <def>An unmanned balloon sent aloft for meteorological or aeronautic purposes.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Sounding-board</ent><br/
<hw>Sound"ing-board`</hw> <pr>(?)</pr>, <pos>n.</pos> <sn>1.</sn> <fld>(Mus.)</fld> <def>A thin board which propagates the sound in a piano, in a violin, and in some other musical instruments.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A board or structure placed behind or over a pulpit or rostrum to give distinctness to a speaker's voice.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <pluf>pl.</pluf> <def>See <cref>Sound boarding</cref>, under <er>Sound</er>, a noise.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Soundless</ent><br/
<hw>Sound"less</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Not capable of being sounded or fathomed; unfathomable.</def>  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Soundless</ent><br/
<hw>Sound"less</hw>, <pos>a.</pos> <def>Having no sound; noiseless; silent.</def> -- <wordforms><wf>Sound"less*ly</wf>, <pos>adv.</pos> -- <wf>Sound"less*ness</wf>, <pos>n.</pos></wordforms><br/
[<source>1913 Webster</source>]</p>

<p><ent>Soundly</ent><br/
<hw>Sound"ly</hw>, <pos>adv.</pos> <def>In a sound manner.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Soundness</ent><br/
<hw>Sound"ness</hw>, <pos>n.</pos> <def>The quality or state of being sound; <as>as, the <ex>soundness</ex> of timber, of fruit, of the teeth, etc.; the <ex>soundness</ex> of reasoning or argument; <ex>soundness</ex> of faith.</as></def><br/
[<source>1913 Webster</source>]</p>

<p><syn><b>Syn.</b> -- Firmness; strength; solidity; healthiness; truth; rectitude.</syn><br/
[<source>1913 Webster</source>]</p>

<p><ent>Soune</ent><br/
<hw>Soune</hw> <pr>(?)</pr>, <pos>v. t. & i.</pos> <def>To sound.</def> <mark>[Obs.]</mark>  <rj><au>Chaucer.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sounst</ent><br/
<hw>Sounst</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Soused. See <er>Souse</er>.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Soup</ent><br/
<hw>Soup</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F. <ets>soupe</ets>, OF. <ets>sope</ets>, <ets>supe</ets>, <ets>soupe</ets>, perhaps originally, a piece of bread; probably of Teutonic origin; cf. D. <ets>sop</ets> sop, G. <ets>suppe</ets> soup. See <er>Sop</er> something dipped in a liquid, and cf. <er>Supper</er>.]</ety> <def>A liquid food of many kinds, usually made by boiling meat and vegetables, or either of them, in water, -- commonly seasoned or flavored; strong broth.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Soup kitchen</b></col>, <cd>an establishment for preparing and supplying soup to the poor.</cd> -- <col><b>Soup ticket</b></col>, <cd>a ticket conferring the privilege of receiving soup at a soup kitchen.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Soup</ent><br/
<hw>Soup</hw>, <pos>v. t.</pos> <def>To sup or swallow.</def> <mark>[Obs.]</mark>  <rj><au>Wyclif.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Soup</ent><br/
<hw>Soup</hw>, <pos>v. t.</pos> <def>To breathe out.</def> <mark>[Obs.]</mark>  <rj><au>amden.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Soup</ent><br/
<hw>Soup</hw>, <pos>v. t.</pos> <def>To sweep. See <er>Sweep</er>, and <er>Swoop</er>.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Soupcon</ent><br/
||<hw>Soup`<cced/on"</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F.]</ety> <def>A suspicion; a suggestion; hence, a very small portion; a taste; <as>as, coffee with a <ex>soup<cced/on</ex> of brandy; a <ex>soup<cced/on</ex> of coquetry</as>.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Soupe-maigre</ent><br/
||<hw>Soupe`-mai"gre</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F.]</ety> <fld>(Cookery)</fld> <def>Soup made chiefly from vegetables or fish with a little butter and a few condiments.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Souple</ent><br/
<hw>Sou"ple</hw> <pr>(?)</pr>, <pos>n.</pos> <def>That part of a flail which strikes the grain.</def>  <rj><au>Knight.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Soupy</ent><br/
<hw>Soup"y</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Resembling soup; souplike.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sour</ent><br/
<hw>Sour</hw> <pr>(?)</pr>, <pos>a.</pos> <amorph>[<pos>Compar.</pos> <adjf>Sourer</adjf> <pr>(?)</pr>; <pos>superl.</pos> <adjf>Sourest</adjf>.]</amorph> <ety>[OE. <ets>sour</ets>, <ets>sur</ets>, AS. <ets>s<?/r</ets>; akin to D. <ets>zuur</ets>, G. <ets>sauer</ets>, OHG. <ets>s<?/r</ets>, Icel. <ets>s<?/rr</ets>, Sw. <ets>sur</ets>, Dan. <ets>suur</ets>, Lith. <ets>suras</ets> salt, Russ. <ets>surovui</ets> harsh, rough.  Cf. <er>Sorrel</er>, the plant.]</ety> <sn>1.</sn> <def>Having an acid or sharp, biting taste, like vinegar, and the juices of most unripe fruits; acid; tart.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>All <qex>sour</qex> things, as vinegar, provoke appetite.</q> <rj><qau>Bacon.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Changed, as by keeping, so as to be acid, rancid, or musty, turned.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>Disagreeable; unpleasant; hence; cross; crabbed; peevish; morose; <as>as, a man of a <ex>sour</ex> temper; a <ex>sour</ex> reply.</as></def> <ldquo/A <xex>sour</xex> countenance.<rdquo/  <rj><au>Swift.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>He was a scholar . . . <br/
Lofty and <qex>sour</qex> to them that loved him not,<br/
But to those men that sought him sweet as summer.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>Afflictive; painful.</def> <ldquo/<xex>Sour</xex> adversity.<rdquo/  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <def>Cold and unproductive; <as>as, <ex>sour</ex> land; a <ex>sour</ex> marsh.</as></def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Sour dock</b></col> <fld>(Bot.)</fld>, <cd>sorrel.</cd> -- <col><b>Sour gourd</b></col> <fld>(Bot.)</fld>, <cd>the gourdlike fruit <spn>Adansonia Gregorii</spn>, and <spn>Adansonia digitata</spn>; also, either of the trees bearing this fruit. See <er>Adansonia</er>.</cd> -- <col><b>Sour grapes</b></col>. <cd>See under <er>Grape</er>.</cd> -- <col><b>Sour gum</b></col> <fld>(Bot.)</fld> <cd>See <er>Turelo</er>.</cd> -- <col><b>Sour plum</b></col> <fld>(Bot.)</fld>, <cd>the edible acid fruit of an Australian tree (<spn>Owenia venosa</spn>); also, the tree itself, which furnished a hard reddish wood used by wheelwrights.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><syn><b>Syn.</b> -- Acid; sharp; tart; acetous; acetose; harsh; acrimonious; crabbed; currish; peevish.</syn><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sour</ent><br/
<hw>Sour</hw>, <pos>n.</pos> <def>A sour or acid substance; whatever produces a painful effect.</def>  <rj><au>Spenser.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sour</ent><br/
<hw>Sour</hw>, <pos>v. t.</pos> <ety>[AS. <ets>s<?/rian</ets> to sour, to become sour.]</ety> <sn>1.</sn> <def>To cause to become sour; to cause to turn from sweet to sour; <as>as, exposure to the air <ex>sours</ex> many substances</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>So the sun's heat, with different powers,<br/
Ripens the grape, the liquor <qex>sours</qex>.</q> <rj><qau>Swift.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To make cold and unproductive, as soil.</def>  <rj><au>Mortimer.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>To make unhappy, uneasy, or less agreeable.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>To <qex>sour</qex> your happiness I must report,<br/
The queen is dead.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>To cause or permit to become harsh or unkindly.</def> <ldquo/<xex>Souring</xex> his cheeks.<rdquo/  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Pride had not <qex>sour'd</qex> nor wrath debased my heart.</q> <rj><qau>Harte.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <def>To macerate, and render fit for plaster or mortar; <as>as, to <ex>sour</ex> lime for business purposes</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sour</ent><br/
<hw>Sour</hw>, <pos>v. i.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Soured</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Souring</conjf>.]</vmorph> <def>To become sour; to turn from sweet to sour; <as>as, milk soon <ex>sours</ex> in hot weather; a kind temper sometimes <ex>sours</ex> in adversity.</as></def><br/
[<source>1913 Webster</source>]</p>

<p><q>They keep out melancholy from the virtuous, and hinder the hatred of vice from <qex>souring</qex> into severity.</q> <rj><qau>Addison.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Source</ent><br/
<hw>Source</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[OE. <ets>sours</ets>, OF. <ets>sourse</ets>, <ets>surse</ets>, <ets>sorse</ets>, F. <ets>source</ets>, fr. OF. <ets>sors</ets>, p. p. of OF. <ets>sordre</ets>, <ets>surdre</ets>, <ets>sourdre</ets>, to spring forth or up, F. <ets>sourdre</ets>, fr. L. <ets>surgere</ets> to lift or raise up, to spring up. See <er>Surge</er>, and cf. <er>Souse</er> to plunge or swoop as a bird upon its prey.]</ety> <sn>1.</sn> <def>The act of rising; a rise; an ascent.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>Therefore right as an hawk upon a <qex>sours</qex><br/
Up springeth into the air, right so prayers . . . <br/
Maken their <qex>sours</qex> to Goddes ears two.</q> <rj><qau>Chaucer.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>The rising from the ground, or beginning, of a stream of water or the like; a spring; a fountain.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Where as the Poo out of a welle small<br/
Taketh his firste springing and his <qex>sours</qex>.</q> <rj><qau>Chaucer.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Kings that rule<br/
Behind the hidden <qex>sources</qex> of the Nile.</q> <rj><qau>Addison.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>That from which anything comes forth, regarded as its cause or origin; the person from whom anything originates; first cause.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>This <qex>source</qex> of ideas every man has wholly in himself.</q> <rj><qau>Locke.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>The <qex>source</qex> of Newton's light, of Bacon's sense.</q> <rj><qau>Pope.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><syn><b>Syn.</b> -- See <er>Origin</er>.</syn><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sourcrout</ent><br/
<hw>Sour"crout`</hw> <pr>(?)</pr>, <pos>n.</pos> <def>See <er>Sauerkraut</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sourde</ent><br/
<hw>Sourde</hw> <pr>(?)</pr>, <pos>v. i.</pos> <ety>[F. <ets>sourdre</ets>. See <er>Source</er>.]</ety> <def>To have origin or source; to rise; to spring.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>Now might men ask whereof that pride <qex>sourdeth</qex>.</q> <rj><qau>Chaucer.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Souring</ent><br/
<hw>Sour"ing</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Bot.)</fld> <def>Any sour apple.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sourish</ent><br/
<hw>Sour"ish</hw>, <pos>a.</pos> <def>Somewhat sour; moderately acid; <as>as, <ex>sourish</ex> fruit; a <ex>sourish</ex> taste</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sourkrout</ent><br/
<hw>Sour"krout`</hw> <pr>(?)</pr>, <pos>n.</pos> <def>Same as <er>Sauerkraut</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sourly</ent><br/
<hw>Sour"ly</hw>, <pos>adv.</pos> <def>In a sour manner; with sourness.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sourness</ent><br/
<hw>Sour"ness</hw>, <pos>n.</pos> <def>The quality or state of being sour.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sours</ent><br/
<hw>Sours</hw> <pr>(?)</pr>, <pos>n.</pos> <def>Source. See <er>Source</er>.</def> <mark>[Obs.]</mark>  <rj><au>Chaucer.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Soursop</ent><br/
<hw>Sour"sop`</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Bot.)</fld> <def>The large succulent and slightly acid fruit of a small tree (<spn>Anona muricata</spn>) of the West Indies; also, the tree itself. It is closely allied to the custard apple.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sourwood</ent><br/
<hw>Sour"wood`</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Bot.)</fld> <def>The sorrel tree.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Souse</ent><br/
<ent>Sous</ent><br/
<mhw>{ <hw>Sous</hw>, <hw>Souse</hw>  }</mhw> <pr>(F. s<oomac/; <xex>colloq. Eng</xex>. sous)</pr>, <pos>n.</pos> <def>A corrupt form of Sou.</def> <mark>[Obs.]</mark>  <rj><au>Colman, the Elder.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Souse</ent><br/
<hw>Souse</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[OF. <ets>sausse</ets>. See <er>Sauce</er>.]</ety> <altsp>[Written also <asp>souce</asp>, <asp>sowce</asp>, and <asp>sowse</asp>.]</altsp> <sn>1.</sn> <def>Pickle made with salt.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Something kept or steeped in pickle; esp., the pickled ears, feet, etc., of swine.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>And he that can rear up a pig in his house,<br/
Hath cheaper his bacon, and sweeter his <qex>souse</qex>.</q> <rj><qau>Tusser.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>The ear; especially, a hog's ear.</def> <mark>[Prov. Eng.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>The act of sousing; a plunging into water.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Souse</ent><br/
<hw>Souse</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A drunkard.</def> <mark>[slang]</mark> <br/
[<source>PJC</source>]</p>

<p><ent>Souse</ent><br/
<hw>Souse</hw>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Soused</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Sousing</conjf>.]</vmorph> <ety>[Cf. F. <ets>saucer</ets> to wet with sauce. See <er>Souse</er> pickle.]</ety> <sn>1.</sn> <def>To steep in pickle; to pickle.</def> <ldquo/A <xex>soused</xex> gurnet.<rdquo/  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To plunge or immerse in water or any liquid.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>They <qex>soused</qex> me over head and ears in water.</q> <rj><qau>Addison.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>To drench, as by an immersion; to wet throughly.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Although I be well <qex>soused</qex> in this shower.</q> <rj><qau>Gascoigne.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Souse</ent><br/
<hw>Souse</hw>, <pos>v. i.</pos> <ety>[Probably fr. OF. <ets>sors</ets>, p. p. of <ets>sordre</ets> to rise, and first used of an upward swood, then of a swoop in general, but also confused with <er>Souse</er>, <pos>v. t.</pos> See <er>Source</er>.]</ety> <def>To swoop or plunge, as a bird upon its prey; to fall suddenly; to rush with speed; to make a sudden attack.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>For then I viewed his plunge and <qex>souse</qex><br/
Into the foamy main.</q> <rj><qau>Marston.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Jove's bird will <qex>souse</qex> upon the timorous hare.</q> <rj><qau>J. Dryden. Jr.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Souse</ent><br/
<hw>Souse</hw>, <pos>v. t.</pos> <def>To pounce upon.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>[The gallant monarch] like eagle o'er his serie towers,<br/
To <qex>souse</qex> annoyance that comes near his nest.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Souse</ent><br/
<hw>Souse</hw>, <pos>n.</pos> <def>The act of sousing, or swooping.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>As a falcon fair<br/
That once hath failed or her <qex>souse</qex> full near.</q> <rj><qau>Spenser.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Souse</ent><br/
<hw>Souse</hw>, <pos>adv.</pos> <def>With a sudden swoop; violently.</def>  <rj><au>Young.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Soused</ent><br/
<hw>Soused</hw> <pr>(?)</pr>, <pos>adj.</pos> <def>Thoroughly drunken; inebriated.</def> <mark>[slang]</mark><br/
<syn><b>Syn. --</b> bombed; pickled; drunk; intoxicated.</syn><br/
[<source>PJC</source>]</p>

<p><ent>Souslik</ent><br/
<hw>Sous"lik</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F.]</ety> <fld>(Zool.)</fld> <def>See <er>Suslik</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sout</ent><br/
<hw>Sout</hw> <pr>(?)</pr>, <pos>n.</pos> <def>Soot.</def> <mark>[Obs.]</mark>  <rj><au>Spenser.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Soutache</ent><br/
||<hw>Sou`tache"</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F.]</ety> <def>A kind of narrow braid, usually of silk; -- also known as <altname>Russian braid</altname>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Soutage</ent><br/
<hw>Sout"age</hw> <pr>(? <or/ ?; 48)</pr>, <pos>n.</pos> <ety>[Etymol. uncertain.]</ety> <def>That in which anything is packed; bagging, as for hops.</def> <mark>[Obs.]</mark>  <rj><au>Halliwell.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Soutane</ent><br/
||<hw>Sou`tane"</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F., fr. Sp. <ets>sotana</ets>, or It. <ets>sottana</ets>, LL. <ets>subtana</ets>, fr. L. <ets>subtus</ets> below, beneath, fr. <ets>sub</ets> under.]</ety> <fld>(Eccl. Costume)</fld> <def>A close garnment with straight sleeves, and skirts reaching to the ankles, and buttoned in front from top to bottom; especially, the black garment of this shape worn by the clergy in France and Italy as their daily dress; a cassock.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Souter</ent><br/
<hw>Sou"ter</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[AS. <ets>s<?/t<?/re</ets>, fr. It. <ets>sutor</ets>, fr. <ets>suere</ets> to sew.]</ety> <def>A shoemaker; a cobbler.</def> <mark>[Obs.]</mark>  <rj><au>Chaucer.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>There is no work better than another to please God: . . . to wash dishes, to be a <qex>souter</qex>, or an apostle, -- all is one.</q> <rj><qau>Tyndale.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Souterly</ent><br/
<hw>Sou"ter*ly</hw>, <pos>a.</pos> <def>Of or pertaining to a cobbler or cobblers; like a cobbler; hence, vulgar; low.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Souterrain</ent><br/
<hw>Sou"ter*rain</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F. See <er>Subterranean</er>.]</ety> <def>A grotto or cavern under ground.</def> <mark>[Obs.]</mark>  <rj><au>Arbuthnot.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>South</ent><br/
<hw>South</hw> <pr>(?; <xex>by sailors</xex> sou)</pr>, <pos>n.</pos> <ety>[OE. <ets>south</ets>, <ets>su<thorn/</ets>, AS. <ets>s<umac/<edh/</ets> for <ets>sun<edh/</ets>; akin to D. <ets>zuid</ets>, OHG. <ets>sund</ets>, G. <ets>s<uum/d</ets>, <ets>s<uum/den</ets>, Icel. <ets>su<edh/r</ets>, <ets>sunnr</ets>, Dan. <ets>syd</ets>, <ets>s<oum/nden</ets>, Sw. <ets>syd</ets>, <ets>s<oum/der</ets>, <ets>sunnan</ets>; all probably akin to E. <ets>sun</ets>, meaning, the side towards the sun. <root/297. See <er>Sun</er>.]</ety> <sn>1.</sn> <def>That one of the four cardinal points directly opposite to the north; the region or direction to the right or direction to the right of a person who faces the east.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A country, region, or place situated farther to the south than another; the southern section of a country.</def> <ldquo/The queen of the <xex>south</xex>.<rdquo/  <rj><au>Matt. xii. 42.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>Specifically: That part of the United States which is south of Mason and Dixon's line. See under <er>Line</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>The wind from the south.</def> <mark>[Obs.]</mark>  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>South</ent><br/
<hw>South</hw>, <pos>a.</pos> <def>Lying toward the south; situated at the south, or in a southern direction from the point of observation or reckoning; proceeding toward the south, or coming from the south; blowing from the south; southern; <as>as, the <ex>south</ex> pole</as>.</def> <ldquo/At the <xex>south</xex> entry.<rdquo/  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>South-Sea tea</b></col> <fld>(Bot.)</fld> <cd>See <er>Yaupon</er>.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>South</ent><br/
<hw>South</hw>, <pos>adv.</pos> <sn>1.</sn> <def>Toward the south; southward.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>From the south; <as>as, the wind blows <ex>south</ex></as>.</def>  <rj><au>Bacon.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>South</ent><br/
<hw>South</hw> <pr>(?)</pr>, <pos>v. i.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Southed</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Southing</conjf>.]</vmorph> <sn>1.</sn> <def>To turn or move toward the south; to veer toward the south.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Astron.)</fld> <def>To come to the meridian; to cross the north and south line; -- said chiefly of the moon; <as>as, the moon <ex>souths</ex> at nine</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Southcottian</ent><br/
<hw>South*cot"ti*an</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Eccl. Hist.)</fld> <def>A follower of Joanna <xex>Southcott</xex> (1750-1814), an Englishwoman who, professing to have received a miraculous calling, preached and prophesied, and committed many impious absurdities.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Southdown</ent><br/
<hw>South"down`</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Of or pertaining to the South Downs, a range of pasture hills south of the Thames, in England.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Southdown sheep</b></col> <fld>(Zool.)</fld>, <cd>a celebrated breed of shortwooled, hornless sheep, highly valued on account of the delicacy of their flesh. So called from the South Downs where the breed originated.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Southdown</ent><br/
<hw>South"down`</hw>, <pos>n.</pos> <def>A Southdown sheep.</def><br/
[<source>1913 Webster</source>]</p>

<p><-- p. 1376 --></p>

<p><ent>Southeast</ent><br/
<hw>South`east"</hw> <pr>(?; <xex>by sailors</xex> sou"-)</pr>, <pos>n.</pos> <def>The point of the compass equally distant from the south and the east; the southeast part or region.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Southeast</ent><br/
<hw>South`east</hw> <pr>(?; <xex>by sailors</xex> sou"-)</pr>, <pos>a.</pos> <def>Of or pertaining to the southeast; proceeding toward, or coming from, the southeast; <as>as, a <ex>southeast</ex> course; a <ex>southeast</ex> wind.</as></def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Southeaster</ent><br/
<hw>South`east"er</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A storm, strong wind, or gale coming from the southeast.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Southeaster</ent><br/
<hw>South`east"er</hw>, <pos>adv.</pos> <def>Toward the southeast.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Southeastern</ent><br/
<hw>South`east"ern</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Of or pertaining to the southeast; southeasterly.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Southeastwardly</ent><br/
<ent>Southeastward</ent><br/
<mhw>{ <hw>South`east"ward</hw> <pr>(?)</pr>, <hw>South`east"ward*ly</hw>  }</mhw>, <pos>adv.</pos> <def>Toward the southeast.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Souther</ent><br/
<hw>South"er</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A strong wind, gale, or storm from the south.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Southerliness</ent><br/
<hw>South"er*li*ness</hw> <pr>(?)</pr>, <pos>n.</pos> <def>The quality or state of being southerly; direction toward the south.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Southerly</ent><br/
<hw>South"er*ly</hw> <pr>(?; 277)</pr>, <pos>a.</pos> <def>Southern.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Southern</ent><br/
<hw>South"ern</hw> <pr>(?; 277)</pr>, <pos>a.</pos> <ety>[AS. <ets>s<umac/<edh/ern</ets>. See <er>South</er>.]</ety> <def>Of or pertaining to the south; situated in, or proceeding from, the south; situated or proceeding toward the south.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Southern Cross</b></col> <fld>(Astron.)</fld>, <cd>a constellation of the southern hemisphere containing several bright stars so related in position as to resemble a cross.</cd> -- <col><b>Southern Fish</b></col> <fld>(Astron.)</fld>, <cd>a constelation of the southern hemisphere (<spn>Piscis Australis</spn>) containing the bright star Fomalhaut.</cd> -- <col><b>Southern States</b></col> <fld>(U.S. Hist. & Geog.)</fld>, <cd>the States of the American Union lying south of Pennsylvania and the Ohio River, with Arkansas, Louisiana, and Texas. Before the Civil War, Missouri also, being a slave State, was classed as one of the Southern States.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Southern</ent><br/
<hw>South"ern</hw>, <pos>n.</pos> <def>A Southerner.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Southerner</ent><br/
<hw>South"ern*er</hw> <pr>(?)</pr>, <pos>n.</pos> <def>An inhabitant or native of the south, esp. of the Southern States of North America; opposed to <contr>Northerner</contr>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Southernliness</ent><br/
<hw>South"ern*li*ness</hw> <pr>(?)</pr>, <pos>n.</pos> <def>Southerliness.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Southernly</ent><br/
<hw>South"ern*ly</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Somewhat southern.</def> -- <def2><pos>adv.</pos> <def>In a southerly manner or course; southward.</def></def2><br/
[<source>1913 Webster</source>]</p>

<p><ent>Southernmost</ent><br/
<hw>South"ern*most`</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Farthest south.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Southernwood</ent><br/
<hw>South"ern*wood`</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Bot.)</fld> <def>A shrubby species of wormwood (<spn>Artemisia Abrotanum</spn>) having aromatic foliage. It is sometimes used in making beer.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Southing</ent><br/
<hw>South"ing</hw> <pr>(?; 277)</pr>, <pos>n.</pos> <sn>1.</sn> <def>Tendency or progress southward; <as>as, the <ex>southing</ex> of the sun</as>.</def>  <rj><au>Emerson.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>The time at which the moon, or other heavenly body, passes the meridian of a place.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <fld>(Astron.)</fld> <def>Distance of any heavenly body south of the equator; south declination; south latitude.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <fld>(Surv. & Navigation)</fld> <def>Distance southward from any point departure or of reckoning, measured on a meridian; -- opposed to <contr>northing</contr>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Southly</ent><br/
<hw>South"ly</hw> <pr>(?)</pr>, <pos>adv.</pos> <def>Southerly.</def> <mark>[Obs. & R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Southmost</ent><br/
<hw>South"most`</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Farthest toward the south; southernmost.</def> <mark>[R.]</mark>  <rj><au>Milton.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Southness</ent><br/
<hw>South"ness</hw>, <pos>n.</pos> <def>A tendency in the end of a magnetic needle to point toward the south pole.</def>  <rj><au>Faraday.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Southpaw</ent><br/
<hw>South"paw`</hw> <pr>(?)</pr>, <pos>a.</pos> <fld>(Baseball)</fld> <def>Using the left hand in pitching; said of a pitcher.</def> <mark>[Cant]</mark><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Southpaw</ent><br/
<hw>South"paw`</hw>, <pos>n.</pos> <def>A pitcher who pitches with the left hand.</def> <mark>[Cant]</mark><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Southren</ent><br/
<hw>South"ren</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Southern.</def> <mark>[Obs.]</mark> <ldquo/I am a <xex>Southren</xex> man.<rdquo/  <rj><au>Chaucer.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Southron</ent><br/
<hw>South"ron</hw> <pr>(?)</pr>, <pos>n.</pos> <def>An inhabitant of the more southern part of a country; formerly, a name given in Scotland to any Englishman.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Southsay</ent><br/
<hw>South"say`</hw> <pr>(?)</pr>, <pos>v. i.</pos> <def>See <er>Soothsay</er>.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Southsayer</ent><br/
<hw>South"say`er</hw> <pr>(?)</pr>, <pos>n.</pos> <def>See <er>Soothsayer</er>.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>South southerly</ent><br/
<hw>South` south"er*ly</hw> <pr>(?)</pr>. <fld>(Zool.)</fld> <def>the old squaw; -- so called in imitation of its cry. Called also <altname>southerly</altname>, and <altname>southerland</altname>. See under <er>Old</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Southwards</ent><br/
<ent>Southward</ent><br/
<mhw>{ <hw>South"ward</hw> <pr>(?; <it>colloq.</it> <?/)</pr>, <hw>South"wards</hw> <pr>(?; <it>colloq.</it> <?/)</pr>, }</mhw> <pos>adv.</pos> <def>Toward the south, or toward a point nearer the south than the east or west point; <as>as, to go <ex>southward</ex></as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Southward</ent><br/
<hw>South"ward</hw>, <pos>a.</pos> <def>Toward the south.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Southward</ent><br/
<hw>South"ward</hw>, <pos>n.</pos> <def>The southern regions or countries; the south.</def>  <rj><au>Sir W. Raleigh.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Southwardly</ent><br/
<hw>South"ward*ly</hw>, <pos>adv.</pos> <def>In a southern direction.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Southwest</ent><br/
<hw>South`west</hw> <pr>(?; <it>colloq.</it> sou"-.)</pr>, <pos>n.</pos> <def>The point of the compass equally from the south and the west; the southwest part or region.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Southwest</ent><br/
<hw>South`west"</hw>, <pos>a.</pos> <def>Pertaining to, or in the direction of, the southwest; proceeding toward the southwest; coming from the southwest; <as>as, a <ex>southwest</ex> wind</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Southwester</ent><br/
<hw>South`west"er</hw> <pr>(?; <xex>colloq</xex>. <?/)</pr>, <pos>n.</pos> <sn>1.</sn> <def>A storm, gale, or strong wind from the southwest.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A hat made of painted canvas, oiled cloth, or the like, with a flap at the back, -- worn in stormy weather.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Southwesterly</ent><br/
<hw>South`west"er*ly</hw>, <pos>a.</pos> <def>To ward or from the southwest; <as>as, a <ex>southwesterly</ex> course; a <ex>southwesterly</ex> wind.</as></def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Southwestern</ent><br/
<hw>South`west"ern</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Of or pertaining to the southwest; southwesterly; <as>as, to sail a <ex>southwestern</ex> course</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Southwestwardly</ent><br/
<ent>Southwestward</ent><br/
<mhw>{ <hw>South`west"ward</hw> <pr>(?)</pr>, <hw>South`west"ward*ly</hw>  }</mhw>, <pos>adv.</pos> <def>Toward the southwest.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sovenaunce</ent><br/
<ent>Souvenance</ent><br/
<mhw>{ <hw>Sou"ve*nance</hw> <pr>(?)</pr>, <hw>So"ve*naunce</hw> <pr>(?)</pr>, }</mhw> <pos>n.</pos> <ety>[F. <ets>souvenance</ets>.]</ety> <def>Remembrance.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>Of his way he had no <qex>sovenance</qex>.</q> <rj><qau>Spenser.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Souvenir</ent><br/
<hw>Sou`ve*nir</hw> <pr>(? <or/ ?)</pr>, <pos>n.</pos> <ety>[F., fr. <ets>souvenir</ets> to remember, fr. L. <ets>subvenire</ets> to come up, come to mind; <ets>sub</ets> under + <ets>venire</ets> to come, akin to E. <ets>come</ets>. <ets>See</ets> <er>Come</er>, and cf. <er>Subvention</er>.]</ety> <def>That which serves as a reminder; a remembrancer; a memento; a keepsake.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sovereign</ent><br/
<hw>Sov"er*eign</hw> <pr>(? <or/ ?; 277)</pr>, <pos>a.</pos> <ety>[OE. <ets>soverain</ets>, <ets>sovereyn</ets>, OF. <ets>soverain</ets>, <ets>suvrain</ets>, F. <ets>souverain</ets>, LL. <ets>superanus</ets>, fr. L. <ets>superus</ets> that is above, upper, higher, fr. <ets>super</ets> above. See <er>Over</er>, <er>Super</er>, and cf. <er>Soprano</er>. The modern spelling is due to a supposed connection with <ets>reign</ets>.]</ety> <sn>1.</sn> <def>Supreme or highest in power; superior to all others; chief; <as>as, our <ex>sovereign</ex> prince</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Independent of, and unlimited by, any other; possessing, or entitled to, original authority or jurisdiction; <as>as, a <ex>sovereign</ex> state; a <ex>sovereign</ex> discretion</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>Princely; royal.</def> <ldquo/Most <xex>sovereign</xex> name.<rdquo/  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>At Babylon was his <qex>sovereign</qex> see.</q> <rj><qau>Chaucer.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>Predominant; greatest; utmost; paramount.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>We acknowledge him [God] our <qex>sovereign</qex> good.</q> <rj><qau>Hooker.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <def>Efficacious in the highest degree; effectual; controlling; <as>as, a <ex>sovereign</ex> remedy</as>.</def>  <rj><au>Dryden.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Such a <qex>sovereign</qex> influence has this passion upon the regulation of the lives and actions of men.</q> <rj><qau>South.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Sovereign state</b></col>, <cd>a state which administers its own government, and is not dependent upon, or subject to, another power.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sovereign</ent><br/
<hw>Sov"er*eign</hw> <pr>(? <or/ ?; 277)</pr>, <pos>n.</pos> <sn>1.</sn> <def>The person, body, or state in which independent and supreme authority is vested; especially, in a monarchy, a king, queen, or emperor.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>No question is to be made but that the bed of the Mississippi belongs to the <qex>sovereign</qex>, that is, to the nation.</q> <rj><qau>Jefferson.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A gold coin of Great Britain, on which an effigy of the head of the reigning king or queen is stamped, valued at one pound sterling, or about $4.86.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <fld>(Zool.)</fld> <def>Any butterfly of the tribe <fam>Nymphalidi</fam>, or genus <gen>Basilarchia</gen>, as the ursula and the viceroy.</def><br/
[<source>1913 Webster</source>]</p>

<p><syn><b>Syn.</b> -- King; prince; monarch; potentate; emperor.</syn><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sovereignize</ent><br/
<hw>Sov"er*eign*ize</hw> <pr>(?)</pr>, <pos>v. i.</pos> <def>To exercise supreme authority.</def> <mark>[Obs.]</mark>  <rj><au>Sir T. Herbert.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sovereignly</ent><br/
<hw>Sov"er*eign*ly</hw>, <pos>adv.</pos> <def>In a sovereign manner; in the highest degree; supremely.</def>  <rj><au>Chaucer.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sovereignty</ent><br/
<hw>Sov"er*eign*ty</hw> <pr>(?)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Sovereignties</plw> <pr>(#)</pr>.</plu> <ety>[OE. <ets>soverainetee</ets>, OF. <ets>sovrainet<eacute/</ets>, F. <ets>souverainet<eacute/</ets>.]</ety> <def>The quality or state of being sovereign, or of being a sovereign; the exercise of, or right to exercise, supreme power; dominion; sway; supremacy; independence; also, that which is sovereign; a sovereign state; <as>as, Italy was formerly divided into many <ex>sovereignties</ex></as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Woman desiren to have <qex>sovereignty</qex><br/
As well over their husband as over their love.</q> <rj><qau>Chaucer.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sovran</ent><br/
<hw>Sov"ran</hw> <pr>(?)</pr>, <pos>a.</pos> <def>A variant of <er>Sovereign</er>.</def> <mark>[Poetic]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>On thy bald, awful head, O <qex>sovran</qex> Blanc.</q> <rj><qau>Coleridge.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sow</ent><br/
<hw>Sow</hw> <pr>(?)</pr>, <pos>v. i.</pos> <def>To sew. See <er>Sew</er>.</def> <mark>[Obs.]</mark>  <rj><au>Chaucer.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sow</ent><br/
<hw>Sow</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[OE. <ets>sowe</ets>, <ets>suwe</ets>, AS. <ets>sugu</ets>, akin to <ets>s<umac/</ets>, D. <ets>zog</ets>, <ets>zeug</ets>, OHG. <ets>s<umac/</ets>, G. <ets>sau</ets>, Icel. <ets>s<ymac/r</ets>, Dan. <ets>so</ets>, Sw. <ets>sugga</ets>, <ets>so</ets>, L. <ets>sus</ets>. Gr. <grk>"y^s</grk>, <grk>sy^s</grk>, Zend. hu boar; probably from the root seen in Skr. <ets>s<umac/</ets> to beget, to bear; the animal being named in allusion to its fecundity. <root/294.  Cf. <er>Hyena</er>, <er>Soil</er> to stain, <er>Son</er>, <er>Swine</er>.]</ety> <sn>1.</sn> <fld>(Zool.)</fld> <def>The female of swine, or of the hog kind.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Zool.)</fld> <def>A sow bug.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <fld>(Metal.)</fld> <sd>(a)</sd> <def>A channel or runner which receives the rows of molds in the pig bed.</def> <sd>(b)</sd> <def>The bar of metal which remains in such a runner.</def> <sd>(c)</sd> <def>A mass of solidified metal in a furnace hearth; a salamander.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <fld>(Mil.)</fld> <def>A kind of covered shed, formerly used by besiegers in filling up and passing the ditch of a besieged place, sapping and mining the wall, or the like.</def>  <rj><au>Craig.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Sow bread</b></col>. <fld>(Bot.)</fld> <cd>See <er>Cyclamen</er>.</cd> -- <mcol><col><b>Sow bug</b></col>, <it>or</it> <col><b>Sowbug</b></col></mcol> <fld>(Zool.)</fld>, <cd>any one of numerous species of terrestrial <ord>Isopoda</ord> belonging to <gen>Oniscus</gen>, <gen>Porcellio</gen>, and allied genera of the family <fam>Oniscidae</fam>.  They feed chiefly on decaying vegetable substances.</cd> -- <col><b>Sow thistle</b></col> <ety>[AS. <ets>sugepistel</ets>]</ety> <fld>(Bot.)</fld>, <cd>a composite plant (<spn>Sonchus oleraceus</spn>) said to be eaten by swine and some other animals.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sow</ent><br/
<hw>Sow</hw> <pr>(?)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp.</pos> <conjf>Sowed</conjf> <pr>(?)</pr>; <pos>p. p.</pos> <conjf>Sown</conjf> <pr>(?)</pr> or <conjf>Sowed</conjf>; <pos>p. pr. & vb. n.</pos> <conjf>Sowing</conjf>.]</vmorph> <ety>[OE. <ets>sowen</ets>, <ets>sawen</ets>, AS. <ets>s<amac/wan</ets>; akin to OFries. <ets>s<?/a</ets>, D. <ets>zaaijen</ets>, OS. & HG. <ets>s<amac/jan</ets>, G. <ets>s<aum/en</ets>, Icel. <ets>s<amac/</ets>, Sw. <ets>s<aring/</ets>, Dan. <ets>saae</ets>, Goth. <ets>saian</ets>, Lith. <ets>s<emac/ti</ets>, Russ. <ets>sieiate</ets>, L. <ets>serere</ets>, <ets>sevi</ets>.  Cf. <er>Saturday</er>, <er>Season</er>, <er>Seed</er>, <er>Seminary</er>.]</ety> <sn>1.</sn> <def>To scatter, as seed, upon the earth; to plant by strewing; <as>as, to <ex>sow</ex> wheat</as>. Also used figuratively: To spread abroad; to propagate.</def> <ldquo/He would <xex>sow some difficulty</xex>.<rdquo/  <rj><au>Chaucer.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>A sower went forth to <qex>sow</qex>; and when he <qex>sowed</qex>, some seeds fell by the wayside.</q> <rj><qau>Matt. xiii. 3, 4.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>And <qex>sow</qex> dissension in the hearts of brothers.</q> <rj><qau>Addison.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To scatter seed upon, in, or over; to supply or stock, as land, with seeds. Also used figuratively: To scatter over; to besprinkle.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The intellectual faculty is a goodly field, . . . and it is the worst husbandry in the world to <qex>sow</qex> it with trifles.</q> <rj><qau>Sir M. Hale.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>[He] <qex>sowed</qex> with stars the heaven.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Now morn . . . <qex>sowed</qex> the earth with orient pearl.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sow</ent><br/
<hw>Sow</hw>, <pos>v. i.</pos> <def>To scatter seed for growth and the production of a crop; -- literally or figuratively.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>They that <qex>sow</qex> in tears shall reap in joi.</q> <rj><qau>Ps. cxxvi. 5.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sowans</ent><br/
<hw>Sow"ans</hw> <pr>(? <or/ ?)</pr>, <pos>n. pl.</pos> <def>See <er>Sowens</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sowar</ent><br/
<hw>Sow"ar</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Per. <ets>saw<amac/r</ets> a horseman.]</ety> <def>In India, a mounted soldier.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sowbane</ent><br/
<hw>Sow"bane`</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Bot.)</fld> <def>The red goosefoot (<spn>Chenopodium rubrum</spn>), -- said to be fatal to swine.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sowce</ent><br/
<hw>Sowce</hw> <pr>(?)</pr>, <pos>n. & v.</pos> <def>See <er>Souse</er>.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sowdan</ent><br/
<hw>Sow"dan</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F. <ets>soudan</ets>. See <er>Soldan</er>.]</ety> <def>Sultan.</def> <mark>[Obs.]</mark>  <rj><au>Chaucer.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sowdanesse</ent><br/
<hw>Sow"dan*esse`</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A sultaness.</def> <mark>[Obs.]</mark>  <rj><au>Chaucer.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sowens</ent><br/
<hw>Sow"ens</hw> <pr>(? <or/ ?)</pr>, <pos>n. pl.</pos> <ety>[Scottish; cf. AS. <ets>se<aacute/w</ets> juice, glue, paste.]</ety> <def>A nutritious article of food, much used in Scotland, made from the husk of the oat by a process not unlike that by which common starch is made; -- called <altname>flummery</altname> in England.</def> <altsp>[Written also <asp>sowans</asp>, and <asp>sowins</asp>.]</altsp><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sower</ent><br/
<hw>Sow"er</hw> <pr>(?)</pr>, <pos>n.</pos> <def>One who, or that which, sows.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sowins</ent><br/
<hw>Sow"ins</hw> <pr>(? <or/ ?)</pr>, <pos>n. pl.</pos> <def>See <er>Sowens</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sowle</ent><br/
<ent>Sowl</ent><br/
<mhw>{ <hw>Sowl</hw>, <hw>Sowle</hw>  }</mhw> <pr>(?)</pr>, <pos>v. t.</pos> <ety>[Cf. prov. G. <ets>zaulen</ets>, <ets>zauseln</ets>, G. <ets>zausen</ets> to tug, drag.]</ety> <def>To pull by the ears; to drag about.</def> <mark>[Obs.]</mark>  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sowl</ent><br/
<hw>Sowl</hw>, <pos>v. i.</pos> <def>See <er>Soul</er>, <pos>v. i.</pos></def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sown</ent><br/
<hw>Sown</hw> <pr>(?)</pr>, <def><pos>p. p.</pos> of <er>Sow</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sowne</ent><br/
<hw>Sowne</hw> <pr>(?)</pr>, <pos>v. t. & i.</pos> <def>To sound.</def> <mark>[Obs.]</mark>  <rj><au>Chaucer.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sowse</ent><br/
<hw>Sowse</hw> <pr>(?)</pr>, <pos>n. & v.</pos> <def>See <er>Souse</er>.</def> <mark>[Obs.]</mark>  <rj><au>ryden.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sowter</ent><br/
<hw>Sow"ter</hw> <pr>(?)</pr>, <pos>n.</pos> <def>See <er>Souter</er>.</def> <mark>[Obs.]</mark>  <rj><au>B. Jonson.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Soya</ent><br/
<ent>Soy</ent><br/
<mhw><hw>Soy</hw> <pr>(soi)</pr>, <hw>Soy"a</hw></mhw> <pr>(soi"<adot/)</pr>, <pos>n.</pos> <ety>[Chinese <ets>sh<omac/y<umac/</ets>.]</ety> <sn>1.</sn> <def>A Chinese and Japanese liquid sauce for fish, etc., made by subjecting boiled beans (esp. soybeans), or beans and meal, to long fermentation and then long digestion in salt and water.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Bot.)</fld> <def>The <er>soybean</er>.</def><br/
[<source>1913 Webster</source> <source>+PJC</source>]</p>

<p><ent>Soybean</ent><br/
<hw>Soy"bean</hw> <pr>(soi"b<emac/n`)</pr>, <pos>n.</pos> <sn>1.</sn> <fld>(Bot.)</fld> <def>An Asiatic leguminous herb (<spn>Glycine max</spn>, formerly <spn>Glycine Soja</spn>) the seeds of which (also called <prod>soy beans</prod>) are used in preparing the sauce called <prod>soy</prod>.  Called also <altname>soya bean</altname> and <altname>soya</altname>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>the seeds of the <spn>Glycine max</spn>, which produce <er>soybean oil</er>; --  called also <altname>soya bean</altname>.</def><br/
[<source>PJC</source>]</p>

<p><ent>Soybean oil</ent><br/
<hw>Soy"bean oil</hw> <pr>(soi"b<emac/n` oil)</pr>, <pos>n.</pos> <def>an oil obtained from the soybean (<spn>Glycine max</spn>), rich in protein, fats, sterols, and phospholipids, used as a food and in paints and varnishes and in various industrial applications; -- called also <altname>soya oil</altname>.  It is also used in preparing the sauce called <prod>soy</prod> or <prod>soy sauce</prod>.</def><br/
[<source>PJC</source>]</p>

<p><ent>Soyle</ent><br/
<hw>Soyle</hw> <pr>(?)</pr>, <pos>v. t.</pos> <ety>[Aphetic form of <ets>assoil</ets>.]</ety> <def>To solve, to clear up; <as>as, to <ex>soyl</ex> all other texts</as>.</def> <mark>[Obs.]</mark>  <rj><au>Tyndate.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Soyle</ent><br/
<hw>Soyle</hw>, <pos>n.</pos> <ety>[Cf. <er>Soil</er> to feed.]</ety> <def>Prey.</def> <mark>[Obs.]</mark>  <rj><au>Spenser.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Soyned</ent><br/
<hw>Soyn"ed</hw> <pr>(? <or/ ?)</pr>, <pos>a.</pos> <ety>[F. <ets>soigner</ets> to care.]</ety> <def>Filled with care; anxious.</def> <mark>[Obs.]</mark>  <rj><au>Mir. for Mag.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sozzle</ent><br/
<hw>Soz"zle</hw> <pr>(?)</pr>, <pos>v. t.</pos> <ety>[Freq. from <ets>soss</ets>, v.]</ety> <sn>1.</sn> <def>To splash or wet carelessly; <as>as, to <ex>sozzle</ex> the feet in water</as>.</def> <mark>[Local, U.S.]</mark>  <rj><au>Bartlett.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To heap up in confusion.</def> <mark>[Prov. Eng.]</mark>  <rj><au>Forby.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sozzle</ent><br/
<hw>Soz"zle</hw>, <pos>n.</pos> <sn>1.</sn> <def>One who spills water or other liquids carelessly; specifically, a sluttish woman.</def> <mark>[Local, U.S.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A mass, or heap, confusedly mingled.</def> <mark>[Prov. Eng.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spa</ent><br/
<hw>Spa</hw> <pr>(?; 277)</pr>, <pos>n.</pos> <def>A spring or mineral water; -- so called from a place of this name in Belgium.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spaad</ent><br/
<hw>Spaad</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Cf. G. <ets>spath</ets> spar. See <er>Spar</er> the mineral.]</ety> <fld>(Min.)</fld> <def>A kind of spar; earth flax, or amianthus.</def> <mark>[Obs.]</mark>  <rj><au>oodward.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Space</ent><br/
<hw>Space</hw> <pr>(sp<amac/s)</pr>, <pos>n.</pos> <ety>[OE. <ets>space</ets>, F. <ets>espace</ets>, from L. <ets>spatium</ets> space; cf. Gr. <grk>spa^n</grk> to draw, to tear; perh. akin to E. <ets>span</ets>.  Cf. <er>Expatiate</er>.]</ety> <sn>1.</sn> <def>Extension, considered independently of anything which it may contain; that which makes extended objects conceivable and possible.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Pure <qex>space</qex> is capable neither of resistance nor motion.</q> <rj><qau>Locke.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Place, having more or less extension; room.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>They gave him chase, and hunted him as hare;<br/
Long had he no <qex>space</qex> to dwell [in].</q> <rj><qau>R. of Brunne.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>While I have time and <qex>space</qex>.</q> <rj><qau>Chaucer.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>A quantity or portion of extension; distance from one thing to another; an interval between any two or more objects; <as>as, the <ex>space</ex> between two stars or two hills; the sound was heard for the <ex>space</ex> of a mile</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Put a <qex>space</qex> betwixt drove and drove.</q> <rj><qau>Gen. xxxii. 16.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>Quantity of time; an interval between two points of time; duration; time.</def> <ldquo/Grace God gave him here, this land to keep long <xex>space</xex>.<rdquo/  <rj><au>R. of brunne.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Nine times the <qex>space</qex> that measures day and night.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>God may defer his judgments for a time, and give a people a longer <qex>space</qex> of repentance.</q> <rj><qau>Tillotson.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <def>A short time; a while.</def> <mark>[R.]</mark> <ldquo/To stay your deadly strife a <xex>space</xex>.<rdquo/  <rj><au>Spenser.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>6.</sn> <def>Walk; track; path; course.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>This ilke [same] monk let old things pace,<br/
And held after the new world the <qex>space</qex>.</q> <rj><qau>Chaucer.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>7.</sn> <fld>(Print.)</fld> <sd>(a)</sd> <def>A small piece of metal cast lower than a face type, so as not to receive the ink in printing, -- used to separate words or letters.</def> <sd>(b)</sd> <def>The distance or interval between words or letters in the lines, or between lines, as in books, on a computer screen, etc.</def><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ Spaces are of different thicknesses to enable the compositor to arrange the words at equal distances from each other in the same line.</note><br/
[<source>1913 Webster</source>]</p>

<p><sn>8.</sn> <fld>(Mus.)</fld> <def>One of the intervals, or open places, between the lines of the staff.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>9.</sn> <def>that portion of the universe outside the earth or its atmosphere; -- called also <altname>outer space</altname>.</def><br/
[<source>PJC</source>]</p>

<p><cs><mcol><col><b>Absolute space</b></col>, <col><b>Euclidian space</b></col></mcol>, <cd>etc. See under <er>Absolute</er>, <er>Euclidian</er>, etc.</cd>  -- <col><b>deep space</b></col>, <cd>the part of outer space which is beyond the limits of the solar system.</cd> -- <col><b>Space line</b></col> <fld>(Print.)</fld>, <cd>a thin piece of metal used by printers to open the lines of type to a regular distance from each other, and for other purposes; a lead.</cd> <au>Hansard.</au> -- <col><b>Space rule</b></col> <fld>(Print.)</fld>, <cd>a fine, thin, short metal rule of the same height as the type, used in printing short lines in tabular matter.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Space</ent><br/
<hw>Space</hw>, <pos>v. i.</pos> <ety>[Cf. OF. <ets>espacier</ets>, L. <ets>spatiari</ets>. See <er>Space</er>, <pos>n.</pos>]</ety> <def>To walk; to rove; to roam.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>And loved in forests wild to <qex>space</qex>.</q> <rj><qau>Spenser.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Space</ent><br/
<hw>Space</hw>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Spaced</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Spacong</conjf> <pr>(?)</pr>.]</vmorph> <ety>[Cf. F. <ets>espacer</ets>. See <er>Space</er>, <pos>n.</pos>]</ety> <fld>(Print.)</fld> <def>To arrange or adjust the spaces in or between; <as>as, to <ex>space</ex> words, lines, or letters</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Space age</ent><br/
<hw>Space age</hw>, <pos>n.</pos> <def>the time during which humans engaged in space exploration; -- generally considered as beginning on October 4, 1957 when the Soviet Union placed the first artificial satellite into orbit around the earth.</def><br/
[<source>PJC</source>]</p>

<p><ent>space key</ent><br/
<ent>space bar</ent><br/
<mhw>{ <hw>space bar</hw> <it>or</it> <hw>space key</hw> }</mhw>. <fld>(Mach., Computers)</fld> <def>A bar or key, in a typewriter, typesetting machine, or other keyboard (as a computer keyboard), used to insert a space between letters.</def><br/
[<source>Webster 1913 Suppl.</source> <source>+PJC</source>]</p>

<p><ent>Space cadet</ent><br/
<hw>Space cadet</hw>, <pos>n.</pos> <def>a person who appears disconnected from reality, or living in his own world; -- sometimes used of people who are under the influence of mind-altering drugs.</def> <mark>[derogatory]</mark><br/
[<source>PJC</source>]</p>

<p><ent>spacecraft</ent><br/
<hw>space"craft</hw>, <pos>n.</pos> <def>a vehicle capable of travelling in or into outer space; at present, all such vehicles are powered by rocket engine.</def><br/
[<source>PJC</source>]</p>

<p><ent>Space flight</ent><br/
<hw>Space flight</hw>, <pos>n.</pos> <def>the process or event in which vehicles travel into outer space.</def><br/
[<source>PJC</source>]</p>

<p><ent>Spaceful</ent><br/
<hw>Space"ful</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Wide; extensive.</def>  <rj><au>Sandys.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>space heater</ent><br/
<hw>space" heat`er</hw>, <pos>n.</pos> <def>a portable heating device used for warming the air of a single room; -- it may be electrical or use a combustible fuel.</def><br/
[<source>PJC</source>]</p>

<p><ent>Spaceless</ent><br/
<hw>Space"less</hw>, <pos>a.</pos> <def>Without space.</def>  <rj><au>Coleridge.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>spaceman</ent><br/
<hw>space"man`</hw>, <pos>n.</pos> <sn>1.</sn> <def>an <er>astronaut</er>.</def><br/
[<source>PJC</source>]</p>

<p><sn>2.</sn> <def>an intelligent creature from outer space; an extraterrestrial; -- a fictional or hypothetical being, never actually observed.</def><br/
[<source>PJC</source>]</p>

<p><ent>spaceport</ent><br/
<hw>space"port</hw>, <pos>n.</pos> <ety>[by analogy with <ets>airport</ets>.]</ety> <def>a facility where vehicles are launched into space.</def><br/
[<source>PJC</source>]</p>

<p><ent>space probe</ent><br/
<hw>space" probe</hw>, <pos>n.</pos> <def>an unmanned rocket carrying instruments to explore objects or conditions in outer space; -- used mostly for vehicles travelling beyond the earth's gravitational field, rather than in orbital or suborbital flight.</def><br/
[<source>PJC</source>]</p>

<p><ent>space shuttle</ent><br/
<hw>space" shut`tle</hw>, <pos>n.</pos> <def>a reusable vehicle that can carry people or objects into orbit around the earth and return to be used again for subsequent trips; -- distinguished from rockets that are used only once.  The present (1997) American space shuttle is a manned vehicle, having an external fuel tank that is expendable and not re-used.</def><br/
[<source>PJC</source>]</p>

<p><ent>space sickness</ent><br/
<hw>space" sick`ness</hw>, <pos>n.</pos> <def>a group of symptoms, prominently nausea, but sometimes including lethargy, headache, and sweating, occuring under the weightless conditions of space flight.</def> <au>[RHUD]</au><br/
[<source>PJC</source>]</p>

<p><ent>space station</ent><br/
<hw>space" sta`tion</hw>, <pos>n.</pos> <def>a manned artificial satellite orbiting the earth designed for extended occupation and use by multiple crews in succession.</def><br/
[<source>PJC</source>]</p>

<p><ent>Space suit</ent><br/
<hw>Space suit</hw>, <pos>n.</pos> <def>a protective suit covering the whole body, completely airtight and designed to withstand internal pressure, worn by astronauts to protect them from the vacuum and other hazards of outer space.  It typically contains an air supply and other life-support equipment to allow functioning independent of any other equipment for a period of time while in space.</def><br/
[<source>PJC</source>]</p>

<p><ent>space-time</ent><br/
<hw>space-time</hw>, <pos>n.</pos> <def>the four-dimensional coordinate system in which all physical objects of the known universe are located, and in which all physical events occur; it consists of three spatial dimensions and one time dimension; -- also called the <altname>space-time continuum</altname>.</def><br/
[<source>PJC</source>]</p>

<p><ent>Space travel</ent><br/
<hw>Space travel</hw>, <pos>n.</pos> <def>the process of travelling in or into outer space.</def><br/
[<source>PJC</source>]</p>

<p><ent>Space walk</ent><br/
<hw>Space walk</hw>, <pos>n.</pos> <def>any activity by a human outside of the protective environment of a spacecraft while it is in outer space, requiring a space suit to protect the person from the vacuum and other hazards of space.</def><br/
[<source>PJC</source>]</p>

<p><ent>Spacial</ent><br/
<hw>Spa"cial</hw> <pr>(?)</pr>, <pos>a.</pos> <def>See <er>Spatial</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spacially</ent><br/
<hw>Spa"cial*ly</hw>, <pos>adv.</pos> <def>See <er>Spatially</er>.</def>  <rj><au>Sir W. Hamilton.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spacious</ent><br/
<hw>Spa"cious</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>spatiousus</ets>: cf. F. <ets>spacieux</ets>. See <er>Space</er>, <pos>n.</pos>]</ety> <sn>1.</sn> <def>Extending far and wide; vast in extent.</def> <ldquo/A <xex>spacious</xex> plain outstretched in circuit wide.<rdquo/  <rj><au>Milton.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Inclosing an extended space; having large or ample room; not contracted or narrow; capacious; roomy; <as>as, <ex>spacious</ex> bounds; a <ex>spacious</ex> church; a <ex>spacious</ex> hall.</as></def> -- <wordforms><wf>Spa"cious*ly</wf>, <pos>adv.</pos> -- <wf>Spa"cious*ness</wf>, <pos>n.</pos></wordforms><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spad</ent><br/
<hw>Spad</hw> <pr>(sp<acr/d)</pr>, <pos>n.</pos> <fld>(Mining)</fld> <def>A nail one or two inches long, of iron, brass, tin, or tinner iron, with a hole through the flattened head, used to mark stations in underground surveying.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Spadassin</ent><br/
||<hw>Spa`das`sin"</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F., fr. It. <ets>spadaccino</ets> a swordsman, from <ets>spada</ets> a sword.]</ety> <def>A bravo; a bully; a duelist.</def>  <rj><au>Ld. Lytton.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spaddle</ent><br/
<hw>Spad"dle</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A little spade.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spade</ent><br/
<hw>Spade</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Cf. <er>Spay</er>, <pos>n.</pos>]</ety> <sn>1.</sn> <fld>(Zool.)</fld> <def>A hart or stag three years old.</def> <altsp>[Written also <asp>spaid</asp>, <asp>spayade</asp>.]</altsp><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <ety>[Cf. L. <ets>spado</ets>.]</ety> <def>A castrated man or beast.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spade</ent><br/
<hw>Spade</hw>, <pos>n.</pos> <ety>[AS. <ets>spaed</ets>; <ets>spada</ets>; akin to D. <ets>spade</ets>, G. <ets>spaten</ets>, Icel. <ets>spa<edh/i</ets>, Dan. & Sw. <ets>spade</ets>, L. <ets>spatha</ets> a spatula, a broad two-edged sword, a spathe, Gr. <grk>spa`qh</grk>.  Cf. <er>Epaulet</er>, <er>Spade</er> at cards, <er>Spathe</er>, <er>Spatula</er>.]</ety> <sn>1.</sn> <def>An implement for digging or cutting the ground, consisting usually of an oblong and nearly rectangular blade of iron, with a handle like that of a shovel.</def> <ldquo/With <xex>spade</xex> and pickax armed.<rdquo/  <rj><au>Milton.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <ety>[Sp. <ets>espada</ets>, literally, a sword; -- so caused because these cards among the Spanish bear the figure of a sword. Sp. <ets>espada</ets> is fr. L. <ets>spatha</ets>, Gr. <grk>spa`qh</grk>. See the Etymology above.]</ety> <def>One of that suit of cards each of which bears one or more figures resembling a spade.</def><br/
[<source>1913 Webster</source>]</p>

<p><q><ldquo/Let <qex>spades</qex> be trumps!<rdquo/ she said.</q> <rj><qau>Pope.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>A cutting instrument used in flensing a whale.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Spade bayonet</b></col>, <cd>a bayonet with a broad blade which may be used digging; -- called also <altname>trowel bayonet</altname>.</cd> -- <col><b>Spade handle</b></col> <fld>(Mach.)</fld>, <cd>the forked end of a connecting rod in which a pin is held at both ends. See <xex>Illust.</xex> of <cref>Knuckle joint</cref>, under <er>Knuckle</er>.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><-- p. 1377 --></p>

<p><ent>Spade</ent><br/
<hw>Spade</hw> <pr>(sp<amac/d)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Spaded</conjf>; <pos>p. pr. & vb. n.</pos> <conjf>Spading</conjf>.]</vmorph> <def>To dig with a spade; to pare off the sward of, as land, with a spade.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spadebone</ent><br/
<hw>Spade"bone`</hw> <pr>(sp<amac/d"b<omac/n`)</pr>, <pos>n.</pos> <def>Shoulder blade.</def> <mark>[Prov. Eng.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spadefish</ent><br/
<hw>Spade"fish`</hw> <pr>(sp<amac/d"f<icr/sh`)</pr>, <pos>n.</pos> <fld>(Zool.)</fld> <def>An American market fish (<spn>Chaetodipterus faber</spn>) common on the southern coasts; -- called also <altname>angel fish</altname>, <altname>moonfish</altname>, and <altname>porgy</altname>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spadefoot</ent><br/
<hw>Spade"foot`</hw> <pr>(sp<amac/d"f<oocr/t`)</pr>, <pos>n.</pos> <fld>(Zool.)</fld> <def>Any species of burrowing toads of the genus <gen>Scaphiopus</gen>, esp. <spn>Scaphiopus Holbrookii</spn>, of the Eastern United States; -- called also <altname>spade toad</altname>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spadeful</ent><br/
<hw>Spade"ful</hw> <pr>(sp<amac/d"f<usdot/l)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Spadefuls</plw> <pr>(sp<amac/d"f<usdot/lz)</pr>.</plu> <ety>[<ets>Spade</ets> + <ets>full</ets>.]</ety> <def>As much as a spade will hold or lift.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spader</ent><br/
<hw>Spad"er</hw> <pr>(?)</pr>, <pos>n.</pos> <def>One who, or that which, spades; specifically, a digging machine.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spadiceous</ent><br/
<hw>Spa*di"ceous</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>spadix</ets>, <ets>-icis</ets>, a date-brown or nut-brown color. See <er>Spadix</er>.]</ety> <sn>1.</sn> <def>Of a bright clear brown or chestnut color.</def>  <rj><au>Sir T. Browne.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Bot.)</fld> <def>Bearing flowers on a spadix; of the nature of a spadix.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spadicose</ent><br/
<hw>Spa"di*cose`</hw> <pr>(?)</pr>, <pos>a.</pos> <fld>(Bot.)</fld> <def>Spadiceous.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spadille</ent><br/
<hw>Spa*dille"</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F., fr. Sp. <ets>espadilla</ets>, dim. of <ets>espada</ets>. See <er>Spade</er> a card.]</ety> <fld>(Card Playing)</fld> <def>The ace of spades in omber and quadrille.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spadix</ent><br/
<hw>Spa"dix</hw> <pr>(?)</pr>, <pos>n.</pos>; <plu><it>pl.</it> L. <plw>Spadices</plw> <pr>(#)</pr>, E. <plw>Spadixes</plw> <pr>(#)</pr>.</plu> <ety>[L., a palm branch broken off, with its fruit, Gr. <?/.]</ety> <sn>1.</sn> <fld>(Bot.)</fld> <def>A fleshy spike of flowers, usually inclosed in a leaf called a <xex>spathe</xex>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Zool.)</fld> <def>A special organ of the nautilus, due to a modification of the posterior tentacles.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spado</ent><br/
||<hw>Spa"do</hw> <pr>(?)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Spadones</plw> <pr>(#)</pr>.</plu> <ety>[L., fr. Gr. <?/.]</ety> <sn>1.</sn> <def>Same as <er>Spade</er>, 2.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Law)</fld> <def>An impotent person.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spadroon</ent><br/
<hw>Spa*droon"</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Cf. F. & Sp. <ets>espadon</ets>, It. <ets>spadone</ets>. See <er>Espadon</er>, <er>Spade</er>.]</ety> <def>A sword, especially a broadsword, formerly used both to cut and thrust.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spae</ent><br/
<hw>Spae</hw> <pr>(?)</pr>, <pos>v. i.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Spaed</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Spaeing</conjf>.]</vmorph> <ety>[Scot. <ets>spae</ets>, <ets>spay</ets>, to foretell, to divine, Icel. <ets>sp<amac/</ets>.]</ety> <def>To foretell; to divine.</def> <mark>[Scot.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spaeman</ent><br/
<hw>Spae"man</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A prophet; a diviner.</def> <mark>[Scot.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spaewife</ent><br/
<hw>Spae"wife`</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A female fortune teller.</def> <mark>[Scot.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spaghetti</ent><br/
||<hw>Spa*ghet"ti</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[It.]</ety> <def>A variety or macaroni made in tubes of small diameter.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spagyrical</ent><br/
<ent>Spagyric</ent><br/
<mhw>{ <hw>Spa*gyr"ic</hw> <pr>(?)</pr>, <hw>Spa*gyr"ic*al</hw> <pr>(?)</pr>, }</mhw> <pos>a.</pos> <ety>[LL. <ets>sparygicus</ets>, fr. Gr. <?/ to draw, to separate + <?/ to assemble; cf. F. <ets>spagirique</ets>.]</ety> <def>Chemical; alchemical.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spagyric</ent><br/
<hw>Spa*gyr"ic</hw>, <pos>n.</pos> <def>A spagyrist.</def> <mark>[Obs.]</mark>  <rj><au>Bp. Hall.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spagyrist</ent><br/
<hw>Spag"y*rist</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Cf. F. <ets>spagiriste</ets>.]</ety> <sn>1.</sn> <def>A chemist, esp. one devoted to alchemistic pursuits.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>One of a sect which arose in the days of alchemy, who sought to discover remedies for disease by chemical means.  The spagyrists historically preceded the iatrochemists.</def>  <rj><au>Encyc. Brit.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spahee</ent><br/
<ent>Spahi</ent><br/
<mhw>{ ||<hw>Spa"hi</hw> <pr>(?)</pr>, ||<hw>Spa"hee</hw>  }</mhw>, <pos>n.</pos> <ety>[Per., Turk., & Hind. <ets>sip<amac/h<imac/</ets>: cf. F. <ets>spahi</ets>. See <er>Seroy</er>.]</ety> <sn>1.</sn> <def>Formerly, one of the Turkish cavalry.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>An Algerian cavalryman in the French army.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spaid</ent><br/
<hw>Spaid</hw> <pr>(?)</pr>, <pos>n.</pos> <def>See 1st <er>Spade</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spake</ent><br/
<hw>Spake</hw> <pr>(?)</pr>, <mark>archaic</mark> <def><pos>imp.</pos> of <er>Speak</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spakenet</ent><br/
<hw>Spake"net`</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Etymol. uncertain.]</ety> <def>A net for catching crabs.</def>  <rj><au>Halliwell.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spaky</ent><br/
<hw>Spak"y</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Specky.</def> <mark>[Obs.]</mark>  <rj><au>hapman.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spalding knife</ent><br/
<hw>Spald"ing knife`</hw> <pr>(?)</pr>. <def>A spalting knife.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spale</ent><br/
<hw>Spale</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Cf. <er>Spell</er> a splinter.]</ety> <sn>1.</sn> <def>A lath; a shaving or chip, as of wood or stone.</def> <mark>[Prov. Eng. & Scot.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Shipbuilding)</fld> <def>A strengthening cross timber.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spall</ent><br/
<hw>Spall</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[OF. <ets>espaule</ets>; cf. It. <ets>spalla</ets>. See <er>Epaule</er>.]</ety> <def>The shoulder.</def> <mark>[Obs.]</mark>  <rj><au>Spenser.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spall</ent><br/
<hw>Spall</hw>, <pos>n.</pos> <ety>[Prov. E. <ets>spall</ets>, <ets>spell</ets>. See <er>Spale</er>, <er>Spell</er> a splinter.]</ety> <def>A chip or fragment, especially a chip of stone as struck off the block by the hammer, having at least one feather-edge.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spall</ent><br/
<hw>Spall</hw>, <pos>v. t.</pos> <sn>1.</sn> <fld>(Mining)</fld> <def>To break into small pieces, as ore, for the purpose of separating from rock.</def>  <rj><au>Pryce.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Masonry)</fld> <def>To reduce, as irregular blocks of stone, to an approximately level surface by hammering.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spall</ent><br/
<hw>Spall</hw>, <pos>v. i.</pos> <def>To give off spalls, or wedge-shaped chips; -- said of stone, as when badly set, with the weight thrown too much on the outer surface.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spalpeen</ent><br/
<hw>Spal"peen</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Ir. <ets>spailpin</ets>, fr. <ets>spailp</ets> a beau, pride, self-conceit.]</ety> <def>A scamp; an Irish term for a good-for-nothing fellow; -- often used in good-humored contempt or ridicule.</def> <mark>[Colloq.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spalt</ent><br/
<hw>Spalt</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Cf. G. <ets>spalt</ets>stein, from <ets>spalten</ets> to split. See 1st <er>Spell</er>.]</ety> <fld>(Metal.)</fld> <def>Spelter.</def> <mark>[Colloq.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spalt</ent><br/
<hw>Spalt</hw>, <pos>a.</pos> <ety>[See 1st <er>Spell</er>.]</ety> <sn>1.</sn> <def>Liable to break or split; brittle; <as>as, <ex>spalt</ex> timber</as>.</def> <mark>[Prov. Eng.]</mark>  <rj><au>Halliwell.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Heedless; clumsy; pert; saucy.</def> <mark>[Prov. Eng.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spalt</ent><br/
<hw>Spalt</hw>, <pos>v. t. & i.</pos> <ety>[Cf. OE. <ets>spalden</ets>. See <er>Spalt</er>, <pos>a.</pos>]</ety> <def>To split off; to cleave off, as chips from a piece of timber, with an ax.</def> <mark>[Prov. Eng. & Local, U.S.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spalting knife</ent><br/
<hw>Spalt"ing knife`</hw> <pr>(?)</pr>. <def>A knife used in splitting codfish.</def> <altsp>[Written also <asp>spalding knife</asp>.]</altsp><br/
[<source>1913 Webster</source>]</p>

<p><ent>Span</ent><br/
<hw>Span</hw> <pr>(?)</pr>, <mark>archaic</mark> <def><pos>imp. & p. p.</pos> of <er>Spin</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Span</ent><br/
<hw>Span</hw>, <pos>n.</pos> <ety>[AS. <ets>spann</ets>; akin to D. <ets>span</ets>, OHG. <ets>spanna</ets>, G. <ets>spanne</ets>, Icel. <ets>sp<oum/nn</ets>. <root/170. See <er>Span</er>, <pos>v. t.</pos> ]</ety> <sn>1.</sn> <def>The space from the thumb to the end of the little finger when extended; nine inches; eighth of a fathom.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Hence, a small space or a brief portion of time.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Yet not to earth's contracted <qex>span</qex><br/
Thy goodness let me bound.</q> <rj><qau>Pope.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Life's but a <qex>span</qex>; I'll every inch enjoy.</q> <rj><qau>Farquhar.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>The spread or extent of an arch between its abutments, or of a beam, girder, truss, roof, bridge, or the like, between its supports.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <fld>(Naut.)</fld> <def>A rope having its ends made fast so that a purchase can be hooked to the bight; also, a rope made fast in the center so that both ends can be used.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <ety>[Cf. D. <ets>span</ets>, Sw. <ets>spann</ets>, Dan. <ets>spaend</ets>, G. <ets>gespann</ets>. See <er>Span</er>, <pos>v. t.</pos> ]</ety> <def>A pair of horses or other animals driven together; usually, such a pair of horses when similar in color, form, and action.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Span blocks</b></col> <fld>(Naut.)</fld>, <cd>blocks at the topmast and topgallant-mast heads, for the studding-sail halyards.</cd> -- <col><b>Span counter</b></col>, <cd>an old English child's game, in which one throws a counter on the ground, and another tries to hit it with his counter, or to get his counter so near it that he can span the space between them, and touch both the counters.</cd> <au>Halliwell.</au> <ldquo/Henry V., in whose time boys went to <xex>span counter</xex> for French crowns.<rdquo/ <au>Shak.</au> -- <col><b>Span iron</b></col> <fld>(Naut.)</fld>, <cd>a special kind of harpoon, usually secured just below the gunwale of a whaleboat.</cd> -- <col><b>Span roof</b></col>, <cd>a common roof, having two slopes and one ridge, with eaves on both sides.</cd> <au>Gwilt.</au> -- <col><b>Span shackle</b></col> <fld>(Naut.)</fld>, <cd>a large bolt driven through the forecastle deck, with a triangular shackle in the head to receive the heel of the old-fashioned fish davit.</cd> <au>Ham. Nav. Encyc.</au></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Span</ent><br/
<hw>Span</hw> <pr>(?)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Spanned</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Spanning</conjf>.]</vmorph> <ety>[AS. <ets>pannan</ets>; akin to D. & G. <ets>spannen</ets>, OHG. <ets>spannan</ets>, Sw. <ets>sp<aum/nna</ets>, Dan. <ets>spaende</ets>, Icel. <ets>spenna</ets>, and perh. to Gr. <?/ to draw, to drag, L. <ets>spatium</ets> space. <root/170.  Cf. <er>Spin</er>, <pos>v. t.</pos>, <er>Space</er>, <er>Spasm</er>.]</ety> <sn>1.</sn> <def>To measure by the span of the hand with the fingers extended, or with the fingers encompassing the object; <as>as, to <ex>span</ex> a space or distance; to <ex>span</ex> a cylinder.</as></def><br/
[<source>1913 Webster</source>]</p>

<p><q>My right hand hath <qex>spanned</qex> the heavens.</q> <rj><qau>Isa. xiviii. 13.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To reach from one side of to the order; to stretch over as an arch.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The rivers were <qex>spanned</qex> by arches of solid masonry.</q> <rj><qau>prescott.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>To fetter, as a horse; to hobble.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Span</ent><br/
<hw>Span</hw>, <pos>v. i.</pos> <def>To be matched, as horses.</def> <mark>[U. S.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spanaemia</ent><br/
||<hw>Spa*nae"mi*a</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[NL., fr. Gr. <?/ scarce + <?/ blood.]</ety> <fld>(Med.)</fld> <def>A condition of impoverishment of the blood; a morbid state in which the red corpuscles, or other important elements of the blood, are deficient.</def>
<-- [Obs.] this and next word not in Stedman's.  Presumably succeded by anaemia, which is in this dictionary.  Why no cross-ref? --><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spanaemic</ent><br/
<hw>Spa*nae"mic</hw> <pr>(? <or/ ?)</pr>, <pos>a.</pos> <fld>(Med.)</fld> <def>Of or pertaining to spanaemia; having impoverished blood.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spancel</ent><br/
<hw>Span"cel</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Perhaps <ets>span</ets> + AS. <ets>s<amac/l</ets> a rope.]</ety> <def>A rope used for tying or hobbling the legs of a horse or cow.</def> <mark>[Prov. Eng. & Local, U.S.]</mark>
<-- a different usage is in White's "Once and future king" -->  <rj><au>Grose.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spancel</ent><br/
<hw>Span"cel</hw>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Spanceled</conjf> <pr>(?)</pr> or <conjf>Spancelled</conjf>; <pos>p. pr. & vb. n.</pos> <conjf>Spanceling</conjf> <or/ <conjf>Spancelling</conjf>.]</vmorph> <def>To tie or hobble with a spancel.</def> <mark>[Prov. Eng. & Local, U.S.]</mark>  <rj><au>Malone.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spandex</ent><br/
<hw>Span"dex</hw> <pr>(?)</pr>, <pos>prop. n.</pos> <def>An elastic textile material, used for clothing</def> <mark>[Trade name]</mark> <br/
[<source>PJC</source>]</p>

<p><ent>Spandogs</ent><br/
<hw>Span"dogs`</hw> <pr>(?)</pr>, <pos>n. pl.</pos> <def>A pair of grappling dogs for hoisting logs and timber.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spandrel</ent><br/
<hw>Span"drel</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[From <er>Span</er>.]</ety> <sn>1.</sn> <fld>(Arch.)</fld> <def>The irregular triangular space between the curve of an arch and the inclosing right angle; or the space between the outer moldings of two contiguous arches and a horizontal line above them, or another arch above and inclosing them.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A narrow mat or passe partout for a picture.</def> <mark>[Cant]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spane</ent><br/
<hw>Spane</hw> <pr>(?)</pr>, <pos>v. t.</pos> <ety>[Akin to G. <ets>sp<aum/nen</ets>, LG. & D. <ets>spennen</ets>, AS. <ets>spanu</ets> a teat.]</ety> <def>To wean.</def> <mark>[Obs. or Prov. Eng.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spang</ent><br/
<hw>Spang</hw> <pr>(?)</pr>, <pos>v. t.</pos> <def>To spangle.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spang</ent><br/
<hw>Spang</hw>, <pos>v. i.</pos> <def>To spring; to bound; to leap.</def> <mark>[Scot.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>But when they <qex>spang</qex> o'er reason's fence,<br/
We smart for't at our own expense.</q> <rj><qau>Ramsay.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spang</ent><br/
<hw>Spang</hw>, <pos>n.</pos> <def>A bound or spring.</def> <mark>[Scot.]</mark>  <rj><au>Sir W. Scott.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spang</ent><br/
<hw>Spang</hw>, <pos>n.</pos> <ety>[AS. <ets>spange</ets> a clasp or fastening; akin to D. <ets>spang</ets>, G. <ets>spange</ets>, OHG. <ets>spanga</ets>, Icel. <ets>sp<oum/ng</ets> a spangle.]</ety> <def>A spangle or shining ornament.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>With glittering <qex>spangs</qex> that did like stars appear.</q> <rj><qau>Spenser.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spangle</ent><br/
<hw>Span"gle</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[OE. <ets>spangel</ets>, dim. of AS. <ets>spange</ets>. See <er>Spang</er> a spangle.]</ety> <sn>1.</sn> <def>A small plate or boss of shining metal; something brilliant used as an ornament, especially when stitched on the dress.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Figuratively, any little thing that sparkless.</def> <ldquo/The rich <xex>spangles</xex> that adorn the sky.<rdquo/  <rj><au>Waller.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Oak spangle</b></col>. <cd>See under <er>Oak</er>.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spangle</ent><br/
<hw>Span"gle</hw>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Spangled</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Spangling</conjf> <pr>(?)</pr>.]</vmorph> <def>To set or sprinkle with, or as with, spangles; to adorn with small, distinct, brilliant bodies; <as>as, a <ex>spangled</ex> breastplate</as>.</def>  <rj><au>Donne.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>What stars do <qex>spangle</qex> heaven with such beauty?</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Spangled coquette</b></col> <fld>(Zool.)</fld>, <cd>a tropical humming bird (<spn>Lophornis reginae</spn>). See <er>Coquette</er>, 2.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spangle</ent><br/
<hw>Span"gle</hw>, <pos>v. i.</pos> <def>To show brilliant spots or points; to glisten; to glitter.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Some men by feigning words as dark as mine<br/
Make truth to <qex>spangle</qex>, and its rays to shine.</q> <rj><qau>Bunyan.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spangler</ent><br/
<hw>Span"gler</hw> <pr>(?)</pr>, <pos>n.</pos> <def>One who, or that which, spangles.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spangly</ent><br/
<hw>Span"gly</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Resembling, or consisting of, spangles; glittering; <as>as, <ex>spangly</ex> light</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spaniard</ent><br/
<hw>Span"iard</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A native or inhabitant of Spain.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spaniel</ent><br/
<hw>Span"iel</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[OF. <ets>espagneul</ets>, F. <ets><eacute/pagneul</ets>, <ets>espagnol</ets> Spanish, Sp. <ets>espa<ntil/nol</ets>, fr. <ets>Espa<ntil/a</ets> Spain, from L. <ets>Hispania</ets>.]</ety> <sn>1.</sn> <fld>(Zool.)</fld> <def>One of a breed of small dogs having long and thick hair and large drooping ears. The legs are usually strongly feathered, and the tail bushy. See <xex>Illust.</xex> under <er>Clumber</er>, and <er>Cocker</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ There are several varieties of spaniels, some of which, known as <xex>field spaniels</xex>, are used in hunting; others are used for toy or pet dogs, as the Blenheim spaniel, and the King Charles spaniel (see under <er>Blenheim</er>). Of the field spaniels, the larger kinds are called <xex>springers</xex>, and to these belong the Sussex, Norfolk, and Clumber spaniels (see <er>Clumber</er>). The smaller field spaniels, used in hunting woodcock, are called <xex>cocker spaniels</xex> (see Cocker). Field spaniels are remarkable for their activity and intelligence.</note><br/
[<source>1913 Webster</source>]</p>

<p><q>As a <qex>spaniel</qex> she will on him leap.</q> <rj><qau>Chaucer.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A cringing, fawning person.</def>  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spaniel</ent><br/
<hw>Span"iel</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Cringing; fawning.</def>  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spaniel</ent><br/
<hw>Span"iel</hw>, <pos>v. i.</pos> <def>To fawn; to cringe; to be obsequious.</def> <mark>[R.]</mark>  <rj><au>Churchill.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spaniel</ent><br/
<hw>Span"iel</hw>, <pos>v. t.</pos> <def>To follow like a spaniel.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spanish</ent><br/
<hw>Span"ish</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Of or pertaining to Spain or the Spaniards.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Spanish bayonet</b></col> <fld>(Bot.)</fld>, <cd>a liliaceous plant (<spn>Yucca alorifolia</spn>) with rigid spine-tipped leaves. The name is also applied to other similar plants of the Southwestern United States and mexico. Called also <altname>Spanish daggers</altname>.</cd> -- <col><b>Spanish bean</b></col> <fld>(Bot.)</fld> <cd>See the Note under <er>Bean</er>.</cd> -- <col><b>Spanish black</b></col>, <cd>a black pigment obtained by charring cork.</cd> <au>Ure.</au> -- <col><b>Spanish broom</b></col> <fld>(Bot.)</fld>, <cd>a leguminous shrub (<spn>Spartium junceum</spn>) having many green flexible rushlike twigs.</cd> -- <col><b>Spanish brown</b></col>, <cd>a species of earth used in painting, having a dark reddish brown color, due to the presence of sesquioxide of iron.</cd> -- <col><b>Spanish buckeye</b></col> <fld>(Bot.)</fld>, <cd>a small tree (<spn>Ungnadia speciosa</spn>) of Texas, New Mexico, etc., related to the buckeye, but having pinnate leaves and a three-seeded fruit.</cd> -- <col><b>Spanish burton</b></col> <fld>(Naut.)</fld>, <cd>a purchase composed of two single blocks. A <col><b>double Spanish burton</b></col> has one double and two single blocks.</cd> <au>Luce (Textbook of Seamanship).</au> -- <col><b>Spanish chalk</b></col> <fld>(Min.)</fld>, <cd>a kind of steatite; -- so called because obtained from Aragon in Spain.</cd> -- <col><b>Spanish cress</b></col> <fld>(Bot.)</fld>, <cd>a cruciferous plant (<spn>Lepidium Cadamines</spn>), a species of peppergrass.</cd> -- <col><b>Spanish curlew</b></col> <fld>(Zool.)</fld>, <cd>the long-billed curlew.</cd> <mark>[U.S.]</mark> -- <col><b>Spanish daggers</b></col> <fld>(Bot.)</fld> <cd>See <cref>Spanish bayonet</cref>.</cd> -- <col><b>Spanish elm</b></col> <fld>(Bot.)</fld>, <cd>a large West Indian tree (<spn>Cordia Gerascanthus</spn>) furnishing hard and useful timber.</cd> -- <col><b>Spanish feretto</b></col>, <cd>a rich reddish brown pigment obtained by calcining copper and sulphur together in closed crucibles.</cd> -- <col><b>Spanish flag</b></col> <fld>(Zool.)</fld>, <cd>the California rockfish (<spn>Sebastichthys rubrivinctus</spn>). It is conspicuously colored with bands of red and white.</cd> -- <col><b>Spanish fly</b></col> <fld>(Zool.)</fld>, <cd>a brilliant green beetle, common in the south of Europe, used for raising blisters. See <cref>Blister beetle</cref> under <er>Blister</er>, and <er>Cantharis</er>.</cd> -- <col><b>Spanish fox</b></col> <fld>(Naut.)</fld>, <cd>a yarn twisted against its lay.</cd> -- <col><b>Spanish grass</b></col>. <fld>(Bot.)</fld> <cd>See <er>Esparto</er>.</cd> -- <col><b>Spanish juice</b></col> <fld>(Bot.)</fld>, <cd>licorice.</cd> -- <col><b>Spanish leather</b></col>. <cd>See <er>Cordwain</er>.</cd> -- <col><b>Spanish mackerel</b></col>. <fld>(Zool.)</fld> <sd>(a)</sd> <cd>A species of mackerel (<spn>Scomber colias</spn>) found both in Europe and America. In America called <altname>chub mackerel</altname>, <altname>big-eyed mackerel</altname>, and <altname>bull mackerel</altname>.</cd> <sd>(b)</sd> <cd>In the United States, a handsome mackerel having bright yellow round spots (<spn>Scomberomorus maculatus</spn>), highly esteemed as a food fish. The name is sometimes erroneously applied to other species. See <xex>Illust.</xex> under Mackerel.</cd> -- <col><b>Spanish main</b></col>, <cd>the name formerly given to the southern portion of the Caribbean Sea, together with the contiguous coast, embracing the route traversed by Spanish treasure ships from the New to the Old World.</cd> -- <col><b>Spanish moss</b></col>. <fld>(Bot.)</fld> <cd>See <er>Tillandsia</er> (and note at that entry).</cd> -- <col><b>Spanish needles</b></col> <fld>(Bot.)</fld>, <cd>a composite weed (<spn>Bidens bipinnata</spn>) having achenia armed with needlelike awns.</cd> -- <col><b>Spanish nut</b></col> <fld>(Bot.)</fld>, <cd>a bulbous plant (<spn>Iris Sisyrinchium</spn>) of the south of Europe.</cd> -- <col><b>Spanish potato</b></col> <fld>(Bot.)</fld>, <cd>the sweet potato. See under <er>Potato</er>.</cd> -- <col><b>Spanish red</b></col>, <cd>an ocherous red pigment resembling Venetian red, but slightly yellower and warmer.</cd> <au>Fairholt.</au> -- <col><b>Spanish reef</b></col> <fld>(Naut.)</fld>, <cd>a knot tied in the head of a jib-headed sail.</cd> -- <col><b>Spanish sheep</b></col> <fld>(Zool.)</fld>, <cd>a merino.</cd> -- <col><b>Spanish white</b></col>, <cd>an impalpable powder prepared from chalk by pulverizing and repeated washings, -- used as a white pigment.</cd> -- <col><b>Spanish windlass</b></col> <fld>(Naut.)</fld>, <cd>a wooden roller, with a rope wound about it, into which a marline spike is thrust to serve as a lever.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spanish</ent><br/
<hw>Span"ish</hw>, <pos>n.</pos> <def>The language of Spain.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spank</ent><br/
<hw>Spank</hw> <pr>(sp<acr/<nsm/k)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Spanked</conjf> <pr>(sp<acr/<nsm/kt)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Spanking</conjf>.]</vmorph> <ety>[Of unknown origin; cf. LG. <ets>spakken</ets>, <ets>spenkern</ets>, to run and spring about quickly.]</ety> <def>To strike, as the breech, with the open hand; to slap.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spank</ent><br/
<hw>Spank</hw>, <pos>n.</pos> <def>A blow with the open hand; a slap.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spank</ent><br/
<hw>Spank</hw>, <pos>v. i.</pos> <def>To move with a quick, lively step between a trot and gallop; to move quickly.</def>  <rj><au>Thackeray.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spanker</ent><br/
<hw>Spank"er</hw> <pr>(sp<acr/<nsm/k"<etil/r)</pr>, <pos>n.</pos> <sn>1.</sn> <def>One who spanks, or anything used as an instrument for spanking.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Naut.)</fld> <def>The after sail of a ship or bark, being a fore-and-aft sail attached to a boom and gaff; -- sometimes called <altname>driver</altname>. See <xex>Illust.</xex> under <er>Sail</er>.</def>  <rj><au>Totten.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>One who takes long, quick strides in walking; also, a fast horse.</def> <mark>[Colloq.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>Something very large, or larger than common; a whopper, as a stout or tall person.</def> <mark>[Colloq.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Spanker boom</b></col> <fld>(Naut.)</fld>, <cd>a boom to which a spanker sail is attached. See <xex>Illust.</xex> of <er>Ship</er>.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spanker</ent><br/
<hw>Spank"er</hw>, <pos>n.</pos> <def>A small coin.</def> <mark>[Obs. or Prov. Eng.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>spanking</ent><br/
<hw>spank"ing</hw>, <pos>a.</pos> <sn>1.</sn> <def>Moving with a quick, lively pace, or capable of so doing; dashing.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Four <qex>spanking</qex> grays ready harnessed.</q> <rj><qau>G. Colman, the Younger.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Large; considerable.</def> <mark>[Colloq.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>Very; extremely; -- used especially in the phrase <ecol><b>spanking new</b></ecol> or <ecol><b>brand spanking new</b></ecol>, meaning completely new and unused.</def> <mark>[informal]</mark><br/
[<source>PJC</source>]</p>

<p><ent>spanking breeze</ent><br/
<hw>spanking breeze</hw> <fld>(Naut.)</fld>, <def>a strong breeze.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>spanless</ent><br/
<hw>span"less</hw> <pr>(sp<acr/n"l<ecr/s)</pr>, <pos>a.</pos> <def>Incapable of being spanned.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>spanner</ent><br/
<hw>span"ner</hw> <pr>(sp<acr/n"n<etil/r)</pr>, <pos>n.</pos> <sn>1.</sn> <def>One who, or that which, spans.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>The lock of a fusee or carbine; also, the fusee or carbine itself.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>An iron instrument having a jaw to fit a nut or the head of a bolt, and used as a lever to turn it with; a wrench; specifically, a wrench for unscrewing or tightening the couplings of hose.</def> <mark>[Chiefly British usage]</mark><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <pluf>pl.</pluf> <def>A contrivance in some of the earlier steam engines for moving the valves for the alternate admission and shutting off of the steam.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>span-new</ent><br/
<hw>span"-new`</hw> <pr>(sp<acr/n"n<umac/`)</pr>, <pos>a.</pos> <ety>[Icel. <ets>sp<amac/nn<ymac/r</ets>, properly, new as a ship just split; <ets>sp<amac/nn</ets> chip + <ets>n<ymac/r</ets> new. See <er>Spoon</er>, and <er>New</er>.]</ety> <def>Quite new; brand-new; fire-new.</def> <ldquo/A <xex>span-new</xex> archbishop's chair.<rdquo/  <rj><au>Fuller.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>spannishing</ent><br/
<hw>span"nish*ing</hw> <pr>(sp<acr/n"n<icr/sh*<icr/ng)</pr>, <pos>n.</pos> <ety>[From OF. <ets>espanir</ets> to spread, F. <ets><eacute/panouir</ets>. See <er>Expand</er>.]</ety> <def>The full blooming of a flower.</def> <mark>[Obs.]</mark>  <rj><au>Rom. of R.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>spanpiece</ent><br/
<hw>span"piece</hw> <pr>(sp<acr/n"p<emac/s`)</pr>, <pos>n.</pos> <fld>(Arch.)</fld> <def>The collar of a roof; sparpiece.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>spanworm</ent><br/
<hw>span"worm`</hw> <pr>(sp<acr/n"w<ucir/rm`)</pr>, <pos>n.</pos> <fld>(Zool.)</fld> <def>The larva of any geometrid moth, as the cankerworm; a geometer; a measuring worm.</def><br/
[<source>1913 Webster</source>]</p>

<p><-- p. 1378 --></p>

<p><ent>spar</ent><br/
<hw>spar</hw> <pr>(sp<aum/r)</pr>, <pos>n.</pos> <ety>[AS. <ets>spaer</ets> in <ets>spaerst<amac/n</ets> chalkstone; akin to MHG. <ets>spar</ets>, G. <ets>spar</ets>kalk plaster.]</ety> <fld>(Min.)</fld> <def>An old name for a nonmetallic mineral, usually cleavable and somewhat lustrous; <as>as, calc <ex>spar</ex>, or calcite, fluor <ex>spar</ex>, etc.</as>  It was especially used in the case of the gangue minerals of a metalliferous vein.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><mcol><col><b>Blue spar</b></col>, <col><b>Cube spar</b></col></mcol>, <cd>etc. See under <er>Blue</er>, <er>Cube</er>, etc.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spar</ent><br/
<hw>Spar</hw>, <pos>n.</pos> <ety>[OE. <ets>sparre</ets>; akin to D. <ets>spar</ets>, G. <ets>sparren</ets>, OHG. <ets>sparro</ets>, Dan. & Sw. <ets>sparre</ets>, Icel. <ets>sparri</ets>; of uncertain origin.  <root/171.  Cf. <er>Spar</er>, <pos>v. t.</pos> ]</ety> <sn>1.</sn> <fld>(Naut.)</fld> <def>A general term any round piece of timber used as a mast, yard, boom, or gaff.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Arch.)</fld> <def>Formerly, a piece of timber, in a general sense; -- still applied locally to rafters.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>The bar of a gate or door.</def> <mark>[Obs.]</mark>  <rj><au>Chaucer.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Spar buoy</b></col> <fld>(Naut.)</fld>, <cd>a buoy anchored by one end so that the other end rises above the surface of the water.</cd> -- <col><b>Spar deck</b></col> <fld>(Naut.)</fld>, <cd>the upper deck of a vessel; especially, in a frigate, the deck which is continued in a straight line from the quarter-deck to the forecastle, and on which spare spars are usually placed. See under <er>Deck</er>.</cd> -- <col><b>Spar torpedo</b></col> <fld>(Naut.)</fld>, <cd>a torpedo carried on the end of a spar usually projecting from the bow of a vessel, and intended to explode upon contact with an enemy's ships.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spar</ent><br/
<hw>Spar</hw>, <pos>v. t.</pos> <ety>[OE. <ets>sparren</ets>, AS. <ets>sparrian</ets>; akin to G. <ets>sperren</ets>, Icel. <ets>sperra</ets>; from the noun. <root/171. See <er>Spara</er> beam, bar.]</ety> <sn>1.</sn> <def>To bolt; to bar.</def> <mark>[Obs.]</mark>  <rj><au>Chaucer.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To To supply or equip with spars, as a vessel.</def><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ A vessel equipped with spars that are too large or too small is said to be <xex>oversparred</xex> or <xex>undersparred</xex>.</note><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spar</ent><br/
<hw>Spar</hw>, <pos>v. i.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Sparred</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Sparring</conjf>.]</vmorph> <ety>[Of uncertain origin; cf. OF. <ets>esparer</ets> to kick, F. <ets><eacute/parer</ets>, or Icel. <ets>sperra</ets> to stretch out the legs, to struggle.]</ety> <sn>1.</sn> <def>To strike with the feet or spurs, as cocks do.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To use the fists and arms scientifically in attack or defense; to contend or combat with the fists, as for exercise or amusement; to box.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Made believe to <qex>spar</qex> at Paul with great science.</q> <rj><qau>Dickens.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>To contest in words; to wrangle.</def> <mark>[Colloq.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><-- sparring partner, (Boxing) one who spars with a boxer as an opponent for training purposes.  --><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spar</ent><br/
<hw>Spar</hw>, <pos>n.</pos> <sn>1.</sn> <def>A contest at sparring or boxing.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A movement of offense or defense in boxing.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sparable</ent><br/
<hw>Spar"a*ble</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Corrupted from <ets>sparrow bill</ets>.]</ety> <def>A kind of small nail used by shoemakers.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sparada</ent><br/
<hw>Spar"a*da</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Zool.)</fld> <def>A small California surf fish (<spn>Micrometrus aggregatus</spn>); -- called also <altname>shiner</altname>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sparadrap</ent><br/
<hw>Spar"a*drap</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F. <ets>sparadrap</ets>; cf. It. <ets>sparadrappo</ets>, NL. <ets>sparadrapa</ets>.]</ety> <sn>1.</sn> <def>A cerecloth.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Med.)</fld> <def>Any adhesive plaster.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sparagrass</ent><br/
<ent>Sparagus</ent><br/
<ent>Sparage</ent><br/
<mhw>{ <hw>Spar"age</hw> <pr>(?; 48)</pr>, <hw>Spar"a*gus</hw> <pr>(?)</pr>, <hw>Spar"a*grass`</hw> <pr>(?)</pr>, }</mhw> <pos>n.</pos> <def>Obs. or corrupt forms of <er>Asparagus</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sparble</ent><br/
<hw>Spar"ble</hw> <pr>(?)</pr>, <pos>v. t.</pos> <ety>[OF. <ets>esparpiller</ets> to scatter, F. <ets><eacute/parpiller</ets>.]</ety> <def>To scatter; to disperse; to rout.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>The king's host was <qex>sparbled</qex> and chased.</q> <rj><qau>Fabyan.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spare</ent><br/
<hw>Spare</hw> <pr>(?)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Spared</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Sparing</conjf>.]</vmorph> <ety>[AS. <ets>sparian</ets>, fr. <ets>spaer</ets> spare, sparing, saving; akin to D. & G. <ets>sparen</ets>, OHG. <ets>spar<?/n</ets>, Icel. & Sw. <ets>spara</ets>, Dan. <ets>spare</ets> See <er>Spare</er>, <pos>a.</pos>]</ety> <sn>1.</sn> <def>To use frugally or stintingly, as that which is scarce or valuable; to retain or keep unused; to save.</def> <ldquo/No cost would he <xex>spare</xex>.<rdquo/  <rj><au>Chaucer.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>[Thou] thy Father's dreadful thunder didst not <qex>spare</qex>.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>He that hath knowledge, <qex>spareth</qex> his words.</q> <rj><qau>Prov. xvii. 27.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To keep to one's self; to forbear to impart or give.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Be pleased your plitics to <qex>spare</qex>.</q> <rj><qau>Dryden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q><qex>Spare</qex> my sight the pain<br/
Of seeing what a world of tears it costs you.</q> <rj><qau>Dryden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>To preserve from danger or punishment; to forbear to punish, injure, or harm; to show mercy to.</def><br/
[<source>1913 Webster</source>]</p>

<p><q><qex>Spare</qex> us, good Lord.</q> <rj><qau>Book of Common Prayer.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Dim sadness did not <qex>spare</qex><br/
That time celestial visages.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Man alone can whom he conquers <qex>spare</qex>.</q> <rj><qau>Waller.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>To save or gain, as by frugality; to reserve, as from some occupation, use, or duty.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>All the time he could <qex>spare</qex> from the necessary cares of his weighty charge, he <?/estowed on . . . serving of God.</q> <rj><qau>Knolles.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <def>To deprive one's self of, as by being frugal; to do without; to dispense with; to give up; to part with.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Where angry Jove did never <qex>spare</qex><br/
One breath of kind and temperate air.</q> <rj><qau>Roscommon.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>I could have better <qex>spared</qex> a better man.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>To spare one's self</b></col>. <sd>(a)</sd> <cd>To act with reserve.</cd> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>Her thought that a lady should <qex>her spare</qex>.</q> <rj><qau>Chaucer.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sd>(b)</sd> <cd>To save one's self labor, punishment, or blame.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spare</ent><br/
<hw>Spare</hw> <pr>(?)</pr>, <pos>v. i.</pos> <sn>1.</sn> <def>To be frugal; not to be profuse; to live frugally; to be parsimonious.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>I, who at some times spend, at others <qex>spare</qex>,<br/
Divided between carelessness and care.</q> <rj><qau>Pope.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To refrain from inflicting harm; to use mercy or forbearance.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>He will not <qex>spare</qex> in the day of vengeance.</q> <rj><qau>Prov. vi. 34.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>To desist; to stop; to refrain.</def> <mark>[Obs.]</mark>  <rj><au>Chaucer.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spare</ent><br/
<hw>Spare</hw>, <pos>a.</pos> <amorph>[<pos>Compar.</pos> <adjf>Sparer</adjf> <pr>(?)</pr>; <pos>superl.</pos> <adjf>Sparest</adjf>; -- not used in all the senses of the word.]</amorph> <ety>[AS. <ets>spaer</ets> sparing.  Cf. <er>Spare</er>, <pos>v. t.</pos> ]</ety> <sn>1.</sn> <def>Scanty; not abundant or plentiful; <as>as, a <ex>spare</ex> diet</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Sparing; frugal; parsimonious; chary.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>He was <qex>spare</qex>, but discreet of speech.</q> <rj><qau>Carew.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>Being over and above what is necessary, or what must be used or reserved; not wanted, or not used; superfluous; <as>as, I have no <ex>spare</ex> time</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>If that no <qex>spare</qex> clothes he had to give.</q> <rj><qau>Spenser.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>Held in reserve, to be used in an emergency; <as>as, a <ex>spare</ex> anchor; a <ex>spare</ex> bed or room.</as></def><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <def>Lean; wanting flesh; meager; thin; gaunt.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>O, give me the <qex>spare</qex> men, and spare me the great ones.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>6.</sn> <def>Slow.</def> <mark>[Obs. or prov. Eng.]</mark>  <rj><au>Grose.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spare</ent><br/
<hw>Spare</hw> <pr>(?)</pr>, <pos>n.</pos> <sn>1.</sn> <def>The act of sparing; moderation; restraint.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>Killing for sacrifice, without any <qex>spare</qex>.</q> <rj><qau>Holland.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Parsimony; frugal use.</def> <mark>[Obs.]</mark>  <rj><au>Bacon.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Poured out their plenty without spite or <qex>spare</qex>.</q> <rj><qau>Spenser.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>An opening in a petticoat or gown; a placket.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>That which has not been used or expended.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <fld>(Tenpins)</fld> <def>The right of bowling again at a full set of pins, after having knocked all the pins down in less than three bowls. If all the pins are knocked down in one bowl it is a <xex>double spare</xex>; in two bowls, a <xex>single spare</xex>.  For the meaning in  modern bowling, see sense 6.</def><br/
[<source>1913 Webster</source> <source>+PJC</source>]</p>

<p><sn>6.</sn> <fld>(Bowling)</fld> <def>The act of knocking down all ten pins in two bowls, which entitles the bowler to add the number of pins knocked down in the next bowl to the score for the frame in which the spare occurred.</def><br/
[<source>PJC</source>]</p>

<p><ent>Spareful</ent><br/
<hw>Spare"ful</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Sparing; chary.</def> <mark>[Obs.]</mark>  <rj><au>Fairfax.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p>-- <wordforms><wf>Spare"ful*ness</wf>, <pos>n.</pos> <mark>[Obs.]</mark></wordforms>  <rj><au>Sir P. Sidney.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spareless</ent><br/
<hw>Spare"less</hw>, <pos>a.</pos> <def>Unsparing.</def>  <rj><au>Sylvester.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sparely</ent><br/
<hw>Spare"ly</hw>, <pos>adv.</pos> <def>In a spare manner; sparingly.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spareness</ent><br/
<hw>Spare"ness</hw>, <pos>n.</pos> <ety>[Cf. AS. <ets>spaernis</ets> frugality.]</ety> <def>The quality or state of being lean or thin; leanness.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sparer</ent><br/
<hw>Spar"er</hw> <pr>(?)</pr>, <pos>n.</pos> <def>One who spares.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sparerib</ent><br/
<hw>Spare"rib`</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[<ets>Spare</ets>, a. + <ets>rib</ets>.]</ety> <def>A piece of pork, consisting or ribs with little flesh on them.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sparge</ent><br/
<hw>Sparge</hw> <pr>(?)</pr>, <pos>v. t.</pos> <ety>[L. <ets>spargere</ets>; cf. F. <ets>asperger</ets>.]</ety> <def>To sprinkle; to moisten by sprinkling; <as>as, to <ex>sparge</ex> paper</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spargefaction</ent><br/
<hw>Spar`ge*fac"tion</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>spargere</ets> to strew + <ets>facere</ets>, <ets>factum</ets>, to make.]</ety> <def>The act of sprinkling.</def> <mark>[Obs.]</mark>  <rj><au>Swift.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sparger</ent><br/
<hw>Spar"ger</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Cf. F. <ets>asperger</ets> to sprinkle, L. <ets>aspergere</ets>, <ets>spargere</ets>.]</ety> <def>A vessel with a perforated cover, for sprinkling with a liquid; a sprinkler.</def><br/
[<source>1913 Webster</source>]</p>

<p><-- 2. A tube with a perforated or fritted end, to pass gases as small bubbles through a liquid. --><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sparhawk</ent><br/
<hw>Spar"hawk`</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[OE. <ets>sperhauke</ets>.]</ety> <fld>(Zool.)</fld> <def>The sparrow hawk.</def> <mark>[Prov. Eng.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spar-hung</ent><br/
<hw>Spar"-hung`</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Hung with spar, as a cave.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sparing</ent><br/
<hw>Spar"ing</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Spare; saving; frugal; merciful.</def>  <rj><au>Bacon.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p>-- <wordforms><wf>Spar"ing*ly</wf>, adv. -- <wf>Spar"ing*ness</wf>, <pos>n.</pos></wordforms><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spark</ent><br/
<hw>Spark</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[OE. <ets>sparke</ets>, AS. <ets>spearca</ets>; akin to D. <ets>spark</ets>, <ets>sperk</ets>; cf. Icel. <ets>spraka</ets> to crackle, Lith. <ets>sprag<eum/ti</ets>, Gr. <?/ a bursting with a noise, Skr. <ets>sph<?/rj</ets> to crackle, to thunder.  Cf. <er>Speak</er>.]</ety> <sn>1.</sn> <def>A small particle of fire or ignited substance which is emitted by a body in combustion.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Man is born unto trouble, as the <qex>sparks</qex> fly upward.</q> <rj><qau>Job v. 7.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A small, shining body, or transient light; a sparkle.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>That which, like a spark, may be kindled into a flame, or into action; a feeble germ; an elementary principle.</def> <ldquo/If any <xex>spark</xex> of life be yet remaining.<rdquo/ <au>Shak.</au> <ldquo/Small intellectual <xex>spark</xex>.<rdquo/ <au>Macaulay.</au> <ldquo/Vital <xex>spark</xex> of heavenly flame.<rdquo/ <au>Pope.</au><br/
[<source>1913 Webster</source>]</p>

<p><q>We have here and there a little clear light, some <qex>sparks of bright knowledge</qex>.</q> <rj><qau>Locke.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Bright gem instinct with music, vocal <qex>spark</qex>.</q> <rj><qau>Wordsworth.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Spark arrester</b></col>, <cd>a contrivance to prevent the escape of sparks while it allows the passage of gas, -- chiefly used in the smokestack of a wood-burning locomotive. Called also <altname>spark consumer</altname>.</cd> <mark>[U.S.]</mark></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spark</ent><br/
<hw>Spark</hw>, <pos>n.</pos> <ety>[Icel. <ets>sparkr</ets> lively, sprightly.]</ety> <sn>1.</sn> <def>A brisk, showy, gay man.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The finest <qex>sparks</qex> and cleanest beaux.</q> <rj><qau>Prior.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A lover; a gallant; a beau.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spark</ent><br/
<hw>Spark</hw>, <pos>v. i.</pos> <sn>1.</sn> <def>To sparkle.</def> <mark>[Obs.]</mark>  <rj><au>Spenser.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Elec.)</fld> <def>To produce, or give off, sparks, as a dynamo at the commutator when revolving under the collecting brushes.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Spark</ent><br/
<hw>Spark</hw>, <pos>v. i.</pos> <def>To play the spark, beau, or lover.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>A sure sign that his master was courting, or, as it is termed, <qex>sparking</qex>, within.</q> <rj><qau>W. Irwing.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spark coil</ent><br/
<hw>Spark coil</hw>. <fld>(Elec.)</fld> <sd>(a)</sd> <def>An induction coil used for producing sparks, esp. in an internal-combustion engine, wireless telegraph apparatus, etc.</def> <sd>(b)</sd> <def>A self-induction coil used to increase the spark in an electric gas-lighting apparatus.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Sparker</ent><br/
<hw>Spark"er</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A spark arrester.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sparkful</ent><br/
<hw>Spark"ful</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Lively; brisk; gay.</def> <mark>[Obs.]</mark> <ldquo/Our <xex>sparkful</xex> youth.<rdquo/  <rj><au>Camden.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spark gap</ent><br/
<hw>Spark gap</hw>. <fld>(Elec.)</fld> <def>The space filled with air or other dielectric between high potential terminals (as of an electrostatic machine, induction coil, or condenser), through which the discharge passes; the air gap of a jump spark.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Sparkish</ent><br/
<hw>Spark"ish</hw>, <pos>a.</pos> <sn>1.</sn> <def>Like a spark; airy; gay.</def>  <rj><au>W. Walsh.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Showy; well-dresed; fine.</def>  <rj><au>L'Estrange.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sparkle</ent><br/
<hw>Spar"kle</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Dim. of <ets>spark</ets>.]</ety> <sn>1.</sn> <def>A little spark; a scintillation.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>As fire is wont to quicken and go<br/
From a <qex>sparkle</qex> sprungen amiss,<br/
Till a city brent up is.</q> <rj><qau>Chaucer.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>The shock was sufficiently strong to strike out some <qex>sparkles</qex> of his fiery temper.</q> <rj><qau>Prescott.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Brilliancy; luster; <as>as, the <ex>sparkle</ex> of a diamond</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sparkle</ent><br/
<hw>Spar"kle</hw>, <pos>v. i.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Sparkled</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Sparkling</conjf> <pr>(?)</pr>.]</vmorph> <ety>[See <er>Sparkle</er>, <pos>n.</pos>, Spark of fire.]</ety> <sn>1.</sn> <def>To emit sparks; to throw off ignited or incandescent particles; to shine as if throwing off sparks; to emit flashes of light; to scintillate; to twinkle; <as>as, the blazing wood <ex>sparkles</ex>; the stars <ex>sparkle</ex></as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>A mantelet upon his shoulder hanging<br/
Bretful of rubies red, as fire <qex>sparkling</qex>.</q> <rj><qau>Chaucer.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To manifest itself by, or as if by, emitting sparks; to glisten; to flash.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>I see bright honor <qex>sparkle</qex> through your eyes.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>To emit little bubbles, as certain kinds of liquors; to effervesce; <as>as, <ex>sparkling</ex> wine</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><syn><b>Syn.</b> -- To shine; glisten; scintillate; radiate; coruscate; glitter; twinkle.</syn><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sparkle</ent><br/
<hw>Spar"kle</hw>, <pos>v. t.</pos> <def>To emit in the form or likeness of sparks.</def> <ldquo/Did <xex>sparkle</xex> forth great light.<rdquo/  <rj><au>Spenser.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sparkle</ent><br/
<hw>Spar"kle</hw>, <pos>v. t.</pos> <ety>[Cf. <er>Sparble</er>.]</ety> <sn>1.</sn> <def>To disperse.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>The Landgrave hath <qex>sparkled</qex> his army without any further enterprise.</q> <rj><qau>State Papers.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To scatter on or over.</def> <mark>[Obs.]</mark>  <rj><au>Purchas.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sparkler</ent><br/
<hw>Spar"kler</hw> <pr>(?)</pr>, <pos>n.</pos> <def>One who scatters; esp., one who scatters money; an improvident person.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sparkler</ent><br/
<hw>Spar"kler</hw>, <pos>n.</pos> <def>One who, or that which, sparkles.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sparkler</ent><br/
<hw>Spar"kler</hw>, <pos>n.</pos> <fld>(Zool.)</fld> <def>A tiger beetle.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sparklet</ent><br/
<hw>Spark"let</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A small spark.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sparkliness</ent><br/
<hw>Spark"li*ness</hw> <pr>(?)</pr>, <pos>n.</pos> <def>Vivacity.</def> <mark>[Obs.]</mark>  <rj><au>Aubrey.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sparkling</ent><br/
<hw>Spar"kling</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Emitting sparks; glittering; flashing; brilliant; lively; <as>as, <ex>sparkling</ex> wine; <ex>sparkling</ex> eyes.</as></def> -- <wordforms><wf>Spar"kling*ly</wf>, <pos>adv.</pos> -- <wf>Spar"kling*ness</wf>, <pos>n.</pos></wordforms><br/
[<source>1913 Webster</source>]</p>

<p><syn><b>Syn.</b> -- Brilliant; shining. See <er>Shining</er>.</syn><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spark plug</ent><br/
<hw>Spark plug</hw>. <def>In internal-combustion engines with electric ignition, a plug, screwed into the cylinder head, connected with the induction coil or magneto circuit on the outside, and used to produce a spark to ignite the fuel-air mixture in the engine.</def><br/
[<source>Webster 1913 Suppl.</source> <source>+PJC</source>]</p>

<p><ent>Sparling</ent><br/
<hw>Spar"ling</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Akin to G. <ets>spierling</ets>, <ets>spiering</ets>, D. <ets>spiering</ets>: cf. F. <ets><eacute/perlan</ets>.]</ety> <fld>(Zool.)</fld> <sd>(a)</sd> <def>The European smelt (<spn>Osmerus eperlanus</spn>).</def> <sd>(b)</sd> <def>A young salmon.</def> <sd>(c)</sd> <def>A tern.</def> <mark>[Scot. & Prov. Eng.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sparlyre</ent><br/
<hw>Spar"lyre`</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[AS. <ets>spear-lira</ets>.]</ety> <def>The calf of the leg.</def> <mark>[Obs.]</mark>  <rj><au>Wyclif (Deut. xxviii. 35).</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sparoid</ent><br/
<hw>Spa"roid</hw> <pr>(?; 277)</pr>, <pos>a.</pos> <ety>[L. <ets>sparus</ets> the gilthead + <ets>-oid</ets>: cf. F. <ets>sparo<ium/de</ets>.]</ety> <fld>(Zool.)</fld> <def>Of or pertaining to the <fam>Sparidae</fam>, a family of spinous-finned fishes which includes the scup, sheepshead, and sea bream.</def> -- <def2><pos>n.</pos> <def>One of the <fam>Sparidae</fam>.</def></def2><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sparpiece</ent><br/
<hw>Spar"piece`</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Arch.)</fld> <def>The collar beam of a roof; the spanpiece.</def>  <rj><au>Gwilt.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sparpoil</ent><br/
<hw>Spar"poil</hw> <pr>(?)</pr>, <pos>v. t.</pos> <ety>[See <er>Sparble</er>.]</ety> <def>To scatter; to spread; to disperse.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sparrow</ent><br/
<hw>Spar"row</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[OE. <ets>sparwe</ets>, AS. <ets>spearwa</ets>; akin to OHG. <ets>sparo</ets>, G. <ets>sper</ets>ling, Icel. <ets>sp<oum/rr</ets>, Dan. <ets>spurv</ets>, <ets>spurre</ets>, Sw. <ets>sparf</ets>, Goth. <ets>sparwa</ets>; -- originally, probably, the quiverer or flutterer, and akin to E. <ets>spurn</ets>. See <er>Spurn</er>, and cf. <er>Spavin</er>.]</ety> <sn>1.</sn> <fld>(Zool.)</fld> <def>One of many species of small singing birds of the family <fam>Fringilligae</fam>, having conical bills, and feeding chiefly on seeds. Many sparrows are called also <stype>finches</stype>, and <stype>buntings</stype>. The common sparrow, or house sparrow, of Europe (<spn>Passer domesticus</spn>) is noted for its familiarity, its voracity, its attachment to its young, and its fecundity. See <cref>House sparrow</cref>, under <er>House</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ The following American species are well known; the <stype>chipping sparrow</stype>, or <stype>chippy</stype>, the <stype>sage sparrow</stype>, the <stype>savanna sparrow</stype>, the <stype>song sparrow</stype>, the <stype>tree sparrow</stype>, and the <stype>white-throated sparrow</stype> (see <er>Peabody bird</er>). See these terms under <er>Sage</er>, <er>Savanna</er>, etc.</note><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Zool.)</fld> <def>Any one of several small singing birds somewhat resembling the true sparrows in form or habits, as the European hedge sparrow. See under <er>Hedge</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>He that doth the ravens feed,<br/
Yea, providently caters for the <qex>sparrow</qex>,<br/
Be comfort to my age!</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><mcol><col><b>Field sparrow</b></col>, <col><b>Fox sparrow</b></col>, etc.</mcol> <cd>See under <er>Field</er>, <er>Fox</er>, etc.</cd> -- <col><b>Sparrow bill</b></col>, <cd>a small nail; a castiron shoe nail; a sparable.</cd> -- <col><b>Sparrow hawk</b></col>. <fld>(Zool.)</fld> <sd>(a)</sd> <cd>A small European hawk (<spn>Accipiter nisus</spn>) or any of the allied species.</cd> <sd>(b)</sd> <cd>A small American falcon (<spn>Falco sparverius</spn>).</cd> <sd>(c)</sd> <cd>The Australian collared sparrow hawk (<spn>Accipiter torquatus</spn>).</cd> <note>The name is applied to other small hawks, as the European kestrel and the New Zealand quail hawk.</note> -- <col><b>Sparrow owl</b></col> <fld>(Zool.)</fld>, <cd>a small owl (<spn>Glaucidium passerinum</spn>) found both in the Old World and the New. The name is also applied to other species of small owls.</cd> -- <col><b>Sparrow spear</b></col> <fld>(Zool.)</fld>, <cd>the female of the reed bunting.</cd> <mark>[Prov. Eng.]</mark></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sparrowgrass</ent><br/
<hw>Spar"row*grass`</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Corrupted from <ets>asparagus</ets>.]</ety> <def>Asparagus.</def> <mark>[Colloq.]</mark> <see>See the Note under <er>Asparagus</er>.</see><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sparrowwort</ent><br/
<hw>Spar"row*wort`</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Bot.)</fld> <def>An evergreen shrub of the genus <gen>Erica</gen> (<spn>Erica passerina</spn>).</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sparry</ent><br/
<hw>Spar"ry</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[From <er>Spar</er>.]</ety> <def>Resembling spar, or consisting of spar; abounding with spar; having a confused crystalline structure; spathose.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Sparry iron</b></col> <fld>(Min.)</fld>, <cd>siderite. See <er>Siderite</er> <sd>(a)</sd>.</cd> -- <col><b>Sparry limestone</b></col> <fld>(Min.)</fld>, <cd>a coarsely crystalline marble.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sparse</ent><br/
<hw>Sparse</hw> <pr>(?)</pr>, <pos>a.</pos> <amorph>[<pos>Compar.</pos> <adjf>Sparser</adjf> <pr>(?)</pr>; <pos>superl.</pos> <adjf>Sparsest</adjf>.]</amorph> <ety>[L. <ets>sparsus</ets>, p. p. of <ets>spargere</ets> to strew, scatter.  Cf. <er>Asperse</er>, <er>Disperse</er>.]</ety> <sn>1.</sn> <def>Thinly scattered; set or planted here and there; not being dense or close together; <as>as, a <ex>sparse</ex> population</as>.</def>  <rj><au>Carlyle.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Bot.)</fld> <def>Placed irregularly and distantly; scattered; -- applied to branches, leaves, peduncles, and the like.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sparse</ent><br/
<hw>Sparse</hw>, <pos>v. t.</pos> <ety>[L. <ets>sparsus</ets>, p. p. of <ets>spargere</ets> to scatter.]</ety> <def>To scatter; to disperse.</def> <mark>[Obs.]</mark>  <rj><au>Spenser.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sparsedly</ent><br/
<hw>Spars"ed*ly</hw> <pr>(?)</pr>, <pos>adv.</pos> <def>Sparsely.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sparsely</ent><br/
<hw>Sparse"ly</hw>, <pos>adv.</pos> <def>In a scattered or sparse manner.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sparseness</ent><br/
<hw>Sparse"ness</hw>, <pos>n.</pos> <def>The quality or state of being sparse; <as>as, <ex>sparseness</ex> of population</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sparsim</ent><br/
||<hw>Spar"sim</hw> <pr>(?)</pr>, <pos>adv.</pos> <ety>[L., fr. <ets>spargere</ets> to scatter.]</ety> <def>Sparsely; scatteredly; here and there.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spartan</ent><br/
<hw>Spar"tan</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>Spartanus</ets>.]</ety> <def>Of or pertaining to Sparta, especially to ancient Sparta; hence, hardy; undaunted; <as>as, <ex>Spartan</ex> souls; <ex>Spartan</ex> bravey.</as></def> -- <def2><pos>n.</pos> <def>A native or inhabitant of Sparta; figuratively, a person of great courage and fortitude.</def></def2><br/
[<source>1913 Webster</source>]</p>

<p><-- marked by avoidance of luxury or comfort; spare, simple; marked by self-denial --><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sparteine</ent><br/
<hw>Spar"te*ine</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Chem.)</fld> <def>A narcotic alkaloid extracted from the tops of the common broom (<spn>Cytisus scoparius</spn>, formerly <spn>Spartium scoparium</spn>), as a colorless oily liquid of aniline-like odor and very bitter taste.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sparterie</ent><br/
<hw>Spar"ter*ie</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F., from Sp. <ets>esparto</ets> esparto, L. <ets>spartum</ets>, Gr. <?/.]</ety> <def>Articles made of the blades or fiber of the <spn>Lygeum Spartum</spn> and <spn>Stipa tenacissima</spn> (syn. <spn>Macrochloa tenacissima</spn>), kinds of grass used in <country>Spain</country> and other countries for making ropes, mats, baskets, nets, and mattresses.</def>  <rj><au>Loudon.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sparth</ent><br/
<hw>Sparth</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Cf. Icel. <ets>spar<?/a</ets>.]</ety> <def>An Anglo-Saxon battle-ax, or halberd.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>He hath a <qex>sparth</qex> of twenty pound of weight.</q> <rj><qau>Chaucer.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><-- p. 1379 --></p>

<p><ent>Sparve</ent><br/
<hw>Sparve</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Zool.)</fld> <def>The hedge sparrow.</def> <mark>[Prov. Eng.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spary</ent><br/
<hw>Spar"y</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Sparing; parsimonious.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spasm</ent><br/
<hw>Spasm</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F. <ets>spasme</ets>, L. <ets>spasmus</ets>, Gr. <?/, from <?/, <?/, to draw, to cause convulsion.  Cf. <er>Span</er>, <pos>v. t.</pos>]</ety> <sn>1.</sn> <fld>(Med.)</fld> <def>An involuntary and unnatural contraction of one or more muscles or muscular fibers.</def><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ Spasm are usually either <xex>clonic</xex> or <xex>tonic</xex>. In <xex>clonic spasm</xex>, the muscles or muscular fibers contract and relax alternately in very quick succession. In <xex>tonic spasm</xex>, the contraction is steady and uniform, and continues for a comparatively long time, as in tetanus.</note><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A sudden, violent, and temporary effort or emotion; <as>as, a <ex>spasm</ex> of repentance</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Cynic spasm</b></col> <fld>(Med.)</fld> <cd>See under <er>Cynic</er>.</cd> -- <col><b>Spasm of the chest</b></col>. <cd>See <cref>Angina pectoris</cref>, under <er>Angina</er>.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spasmatical</ent><br/
<hw>Spas*mat"ic*al</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Spasmodic.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spasmodic</ent><br/
<hw>Spas"mod"ic</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[Gr. <?/; <?/ a convulsion + <?/ likeness: cf. F. <ets>spasmotique</ets>.]</ety> <sn>1.</sn> <fld>(Med.)</fld> <def>Of or pertaining to spasm; consisting in spasm; occuring in, or characterized by, spasms; <as>as, a <ex>spasmodic</ex> asthma</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Soon relaxed or exhausted; convulsive; intermittent; <as>as, <ex>spasmodic</ex> zeal or industry</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Spasmodic croup</b></col> <fld>(Med.)</fld>, <cd>an affection of childhood characterized by a stoppage of brathing developed suddenly and without fever, and produced by spasmodic contraction of the vocal cords. It is sometimes fatal. Called also <altname>laryngismus stridulus</altname>, and <altname>childcrowing</altname>.</cd> -- <col><b>Spasmodic stricture</b></col>, <cd>a stricture caused by muscular spasm without structural change. See <cref>Organic stricture</cref>, under <er>Organic</er>.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spasmodic</ent><br/
<hw>Spas*mod"ic</hw>, <pos>n.</pos> <fld>(Med.)</fld> <def>A medicine for spasm.</def><-- = antispasmodic --><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spasmodical</ent><br/
<hw>Spas*mod"ic*al</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Same as <er>Spasmodic</er>, <pos>a.</pos></def> -- <wordforms><wf>Spas*mod"ic*al*ly</wf> <pr>(#)</pr>, <pos>adv.</pos></wordforms><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spastic</ent><br/
<hw>Spas"tic</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>spasticus</ets>, Gr. <?/, fr. <?/ to draw: cf. F. <ets>spastique</ets>. See <er>Spasm</er>.]</ety> <fld>(Med.)</fld> <def>Of or pertaining to spasm; spasmodic; especially, pertaining to tonic spasm; tetanic.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spastically</ent><br/
<hw>Spas"tic*al*ly</hw> <pr>(?)</pr>, <pos>adv.</pos> <def>Spasmodically.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spasticity</ent><br/
<hw>Spas*tic"i*ty</hw> <pr>(?)</pr>, <pos>n.</pos> <sn>1.</sn> <def>A state of spasm.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>The tendency to, or capability of suffering, spasm.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spat</ent><br/
<hw>Spat</hw> <pr>(?)</pr>, <def><pos>imp.</pos> of <er>Spit</er>.</def> <mark>[Obs. <plain>or</plain> R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spat</ent><br/
<hw>Spat</hw>, <pos>n.</pos> <ety>[From the root of <ets>spit</ets>; hence, literally, that which is ejected.]</ety> <def>A young oyster or other bivalve mollusk, both before and after it first becomes adherent, or such young, collectively.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spat</ent><br/
<hw>Spat</hw>, <pos>v. i. & t.</pos> <def>To emit spawn; to emit, as spawn.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spat</ent><br/
<hw>Spat</hw>, <pos>n.</pos> <ety>[Cf. <er>Pat</er>.]</ety> <sn>1.</sn> <def>A light blow with something flat.</def> <mark>[U.S. & Prov. Eng.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Hence, a petty combat, esp. a verbal one; a little quarrel, dispute, or dissension.</def> <mark>[U. S.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spat</ent><br/
<hw>Spat</hw>, <pos>v. i.</pos> <def>To dispute.</def> <mark>[R.]</mark>  <rj><au>Smart.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spat</ent><br/
<hw>Spat</hw>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Spatted</conjf>; <pos>p. pr. & vb. n.</pos> <conjf>Spatting</conjf>.]</vmorph> <def>To slap, as with the open hand; to clap together; as the hands.</def> <mark>[Local, U.S.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>Little Isabel leaped up and down, <qex>spatting</qex> her hands.</q> <rj><qau>Judd.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spat</ent><br/
<hw>Spat</hw>, <pos>n.</pos> <ety>[Short for <er>Spatterdash</er>.]</ety> <sn>1.</sn> <def>A legging; a gaiter.</def> <mark>[Scot. & Dial. Eng.]</mark><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><sn>2.</sn>  <def>A kind of short cloth or leather gaiter worn over the upper part of the shoe and fastened beneath the instep; -- chiefly in <pluf>pl.</pluf></def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Spatangoid</ent><br/
<hw>Spa*tan"goid</hw> <pr>(?)</pr>, <pos>a.</pos> <fld>(Zool.)</fld> <def>Of or pertaining to the <ord>Spatangoidea</ord>.</def> -- <def2><pos>n.</pos> <def>One of the <ord>Spatangoidea</ord>.</def></def2><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spatangoidea</ent><br/
||<hw>Spat`an*goi"de*a</hw> <pr>(?)</pr>, <pos>n. pl.</pos> <ety>[NL. See <er>Spatangus</er>, and <er>-oid</er>.]</ety> <fld>(Zool.)</fld> <def>An order of irregular sea urchins, usually having a more or less heart-shaped shell with four or five petal-like ambulacra above. The mouth is edentulous and situated anteriorly, on the under side.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spatangus</ent><br/
||<hw>Spa*tan"gus</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[NL., fr. L. <ets>spatangius</ets> a kind of sea urchin, Gr. <?/.]</ety> <fld>(Zool.)</fld> <def>A genus of heart-shaped sea urchins belonging to the <ord>Spatangoidea</ord>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spatchcock</ent><br/
<hw>Spatch"cock`</hw> <pr>(?)</pr>, <pos>n.</pos> <def>See <er>Spitchcock</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spate</ent><br/
<hw>Spate</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Of Celtic origin; cf. Ir. <ets>speid</ets>.]</ety> <def>A river flood; an overflow or inundation.</def>  <rj><au>Burns.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Gareth in a showerful spring<br/
Stared at the <qex>spate</qex>.</q> <rj><qau>Tennyson.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spatha</ent><br/
||<hw>Spa"tha</hw> <pr>(?)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Spathae</plw> <pr>(#)</pr>.</plu> <ety>[L.]</ety> <fld>(Bot.)</fld> <def>A spathe.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spathaceous</ent><br/
<hw>Spa*tha"ceous</hw> <pr>(?)</pr>, <pos>a.</pos> <fld>(Bot.)</fld> <def>Having a spathe; resembling a spathe; spathal.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spathal</ent><br/
<hw>Spa"thal</hw> <pr>(?)</pr>, <pos>a.</pos> <fld>(Bot.)</fld> <def>Furnished with a spathe; <as>as, <ex>spathal</ex> flowers</as>.</def>  <rj><au>Howitt.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spathe</ent><br/
<hw>Spathe</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>spatha</ets>, Gr. <?/: cf. F. <ets>spathe</ets>. See <er>Spade</er> for digging.]</ety> <fld>(Bot.)</fld> <def>A special involucre formed of one leaf and inclosing a spadix, as in aroid plants and palms. See the Note under <er>Bract</er>, and <xex>Illust.</xex> of <er>Spadix</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ The name is also given to the several-leaved involucre of the iris and other similar plants.</note><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spathed</ent><br/
<hw>Spathed</hw> <pr>(?)</pr>, <pos>a.</pos> <fld>(Bot.)</fld> <def>Having a spathe or calyx like a sheath.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spathic</ent><br/
<hw>Spath"ic</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[Cf. F. <ets>spathique</ets>, fr. F. & G. <ets>spath</ets> spar.]</ety> <def>Like spar; foliated or lamellar; spathose.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Spathic iron</b></col> <fld>(Min.)</fld>, <cd>siderite. See <er>Siderite</er> <sd>(a)</sd>.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spathiform</ent><br/
<hw>Spath"i*form</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[F. <ets>spathiforme</ets>.]</ety> <def>Resembling spar in form.</def> <ldquo/The ocherous, <xex>spathiform</xex>, and mineralized forms of uranite.<rdquo/  <rj><au>Lavoisier (Trans.).</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spathose</ent><br/
<hw>Spath"ose`</hw> <pr>(?)</pr>, <pos>a.</pos> <fld>(Min.)</fld> <def>See <er>Spathic</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spathose</ent><br/
<hw>Spath"ose`</hw>, <pos>a.</pos> <ety>[See <er>Spathe</er>.]</ety> <fld>(Bot.)</fld> <def>Having a spathe; resembling a spathe; spatheceous; spathal.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spathous</ent><br/
<hw>Spath"ous</hw> <pr>(?)</pr>, <pos>a.</pos> <fld>(Bot.)</fld> <def>Spathose.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spathulate</ent><br/
<hw>Spath"u*late</hw> <pr>(?)</pr>, <pos>a.</pos> <def>See <er>Spatulate</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spatial</ent><br/
<hw>Spa"tial</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Of or pertaining to space.</def> <ldquo/<xex>Spatial</xex> quantity and relations.<rdquo/  <rj><au>L. H. Atwater.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spatially</ent><br/
<hw>Spa"tial*ly</hw> <pr>(?)</pr>, <pos>adv.</pos> <def>As regards space.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spatiate</ent><br/
<hw>Spa"ti*ate</hw> <pr>(?)</pr>, <pos>v. t.</pos> <ety>[L. <ets>spatiatus</ets>, p. p. of <ets>spatiari</ets>, fr. <ets>spatiatum</ets>. See <er>Space</er>.]</ety> <def>To rove; to ramble.</def> <mark>[Obs.]</mark>  <rj><au>Bacon.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spatter</ent><br/
<hw>Spat"ter</hw> <pr>(?)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Spattered</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Spattering</conjf>.]</vmorph> <ety>[From the root of <ets>spit</ets> salvia.]</ety> <sn>1.</sn> <def>To sprinkle with a liquid or with any wet substance, as water, mud, or the like; to make wet of foul spots upon by sprinkling; <as>as, to <ex>spatter</ex> a coat; to <ex>spatter</ex> the floor; to <ex>spatter</ex> boots with mud</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Upon any occasion he is to be <qex>spattered</qex> over with the blood of his people.</q> <rj><qau>Burke.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To distribute by sprinkling; to sprinkle around; <as>as, to <ex>spatter</ex> blood</as>.</def>  <rj><au>Pope.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>Fig.: To injure by aspersion; to defame; to soil; also, to throw out in a defamatory manner.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spatter</ent><br/
<hw>Spat"ter</hw>, <pos>v. i.</pos> <def>To throw something out of the mouth in a scattering manner; to sputter.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>That mind must needs be irrecoverably depraved, which, . . . tasting but once of one just deed, <qex>spatters</qex> at it, and abhors the relish ever after.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spatterdashed</ent><br/
<hw>Spat"ter*dashed`</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Wearing spatterdashes.</def> <mark>[Colloq.]</mark>  <rj><au>Thackeray.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spatterdashes</ent><br/
<hw>Spat"ter*dash`es</hw> <pr>(?)</pr>, <pos>n. pl.</pos> <ety>[<ets>Spatter</ets> + <ets>dash</ets>.]</ety> <def>Coverings for the legs, to protect them from water and mud; long gaiters.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spatter-dock</ent><br/
<hw>Spat`ter-dock`</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Bot.)</fld> <def>The common yellow water lily (<spn>Nuphar advena</spn>).</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spattle</ent><br/
<hw>Spat"tle</hw> <pr>(?)</pr>, <pos>n.</pos> <def>Spawl; spittle.</def> <mark>[Obs.]</mark>  <rj><au>Bale.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spattle</ent><br/
<hw>Spat"tle</hw>, <pos>n.</pos> <sn>1.</sn> <def>A spatula.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Pottery)</fld> <def>A tool or implement for mottling a molded article with coloring matter</def>  <rj><au>Knoght.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spattling-poppy</ent><br/
<hw>Spat"tling-pop"py</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Prov. E. <ets>spattle</ets> to spit + E. <ets>poppy</ets>.]</ety> <fld>(Bot.)</fld> <def>A kind of catchfly (<spn>Silene inflata</spn>) which is sometimes frothy from the action of captured insects.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spatula</ent><br/
<hw>Spat"u*la</hw> <pr>(?; 135)</pr>, <pos>n.</pos> <ety>[L. <ets>spatula</ets>, <ets>spathula</ets>, dim. of <ets>spatha</ets> a spatula: F. <ets>spatule</ets>. See <er>Spade</er> for digging.]</ety> <def>An implement shaped like a knife, flat, thin, and somewhat flexible, used for spreading paints, fine plasters, drugs in compounding prescriptions, etc.  Cf. <cref>Palette knife</cref>, under <er>Palette</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spatulate</ent><br/
<hw>Spat"u*late</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[NL. <ets>spatulatus</ets>.]</ety> <fld>(Nat. Hist.)</fld> <def>Shaped like spatula, or like a battledoor, being roundish, with a long, narrow, linear base.</def> <altsp>[Also written <asp>spathulate</asp>.]</altsp><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spauld</ent><br/
<hw>Spauld</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[See <er>Spall</er> the shoulder.]</ety> <def>The shoulder.</def> <mark>[Scot.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spavin</ent><br/
<hw>Spav"in</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[OE. <ets>spaveyne</ets>, OF. <ets>esparvain</ets>, F. <ets><eacute/parvin</ets>; akin to OF. <ets>espervier</ets> a sparrow hawk, F. <ets><eacute/pervier</ets>, fr. OHG. <ets>sparw<amac/ri</ets> (G. <ets>sperber</ets>), fr. OHG. <ets>sparo</ets> sparrow, because this disease makes the horse raise the infirm leg in the manner of a sparrow hawk or sparrow. See <er>Sparrow</er>.]</ety> <fld>(Far.)</fld> <def>A disease of horses characterized by a bony swelling developed on the hock as the result of inflammation of the bones; also, the swelling itself. The resulting lameness is due to the inflammation, and not the bony tumor as popularly supposed.</def>  <rj><au>Harbaugh.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Bog spavin</b></col>, <cd>a soft swelling produced by distention of the capsular ligament of the hock; -- called also <altname>blood spavin</altname>.</cd> -- <col><b>Bone spavin</b></col>, <cd>spavin attended with exostosis; ordinary spavin.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spavined</ent><br/
<hw>Spav"ined</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Affected with spavin.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spaw</ent><br/
<hw>Spaw</hw> <pr>(?)</pr>, <pos>n.</pos> <def>See <er>Spa</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spawl</ent><br/
<hw>Spawl</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A splinter or fragment, as of wood or stone. See <er>Spall</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spawl</ent><br/
<hw>Spawl</hw>, <pos>n.</pos> <ety>[Cf. AS. <ets>sp<amac/tl</ets>, fr. <ets>sp<aemac/tan</ets> to spit; probably akin to <ets>sp<imac/wan</ets>, E. <ets>spew</ets>.  Cf. <er>Spew</er>.]</ety> <def>Scattered or ejected spittle.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spawl</ent><br/
<hw>Spawl</hw>, <pos>v. i. & t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Spawled</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Spawling</conjf>.]</vmorph> <ety>[Cf. AS. <ets>sp<amac/tlian</ets>.]</ety> <def>To scatter spittle from the mouth; to spit, as saliva.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Why must he sputter, <qex>spawl</qex>, and slaver it<br/
In vain, against the people's favorite.</q> <rj><qau>Swift.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spawling</ent><br/
<hw>Spawl"ing</hw>, <pos>n.</pos> <def>That which is spawled, or spit out.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spawn</ent><br/
<hw>Spawn</hw> <pr>(?)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Spawned</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Spawning</conjf>.]</vmorph> <ety>[OE. <ets>spanen</ets>, OF. <ets>espandre</ets>, properly, to shed, spread, L. <ets>expandere</ets> to spread out. See <er>Expand</er>.]</ety> <sn>1.</sn> <def>To produce or deposit (eggs), as fishes or frogs do.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To bring forth; to generate; -- used in contempt.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>One edition [of books] <qex>spawneth</qex> another.</q> <rj><qau>Fuller.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spawn</ent><br/
<hw>Spawn</hw>, <pos>v. i.</pos> <sn>1.</sn> <def>To deposit eggs, as fish or frogs do.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To issue, as offspring; -- used contemptuously.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spawn</ent><br/
<hw>Spawn</hw>, <pos>n.</pos> <ety>[<root/170. See <er>Spawn</er>, <pos>v. t.</pos>]</ety> <sn>1.</sn> <def>The ova, or eggs, of fishes, oysters, and other aquatic animals.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Any product or offspring; -- used contemptuously.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <fld>(Hort.)</fld> <def>The buds or branches produced from underground stems.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <fld>(Bot.)</fld> <def>The white fibrous matter forming the matrix from which fungi.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Spawn eater</b></col> <fld>(Zool.)</fld>, <cd>a small American cyprinoid fish (<spn>Notropis Hudsonius</spn>) allied to the dace.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spawner</ent><br/
<hw>Spawn"er</hw> <pr>(?)</pr>, <pos>n.</pos> <sn>1.</sn> <fld>(Zool.)</fld> <def>A mature female fish.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The barbel, for the preservation or their seed, both the <qex>spawner</qex> and the milter, cover their spawn with sand.</q> <rj><qau>Walton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Whatever produces spawn of any kind.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spay</ent><br/
<hw>Spay</hw> <pr>(?)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Spayed</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Spaying</conjf>.]</vmorph> <ety>[Cf. Armor. <ets>spac'hein</ets>, <ets>spaza</ets> to geld, W. <ets>dyspaddu</ets> to geld, L. <ets>spado</ets> a eunuch, Gr. <?/.]</ety> <def>To remove or extirpate the ovaries of, as a sow or a bitch; to castrate (a female animal).</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spay</ent><br/
<hw>Spay</hw>, <pos>n.</pos> <ety>[Cf. <er>Spade</er> a spay, <er>Spay</er>, <pos>v. t.</pos>]</ety> <fld>(Zool.)</fld> <def>The male of the red deer in his third year; a spade.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spayade</ent><br/
<ent>Spayad</ent><br/
<mhw>{ <hw>Spay"ad</hw> <pr>(?)</pr>, <hw>Spay"ade</hw> <pr>(?)</pr>, }</mhw> <pos>n.</pos> <fld>(Zool.)</fld> <def>A spay.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Speak</ent><br/
<hw>Speak</hw> <pr>(?)</pr>, <pos>v. i.</pos> <vmorph>[<pos>imp.</pos> <conjf>Spoke</conjf> <pr>(?)</pr> (<conjf>Spake</conjf> <pr>(<?/)</pr> <mark>Archaic</mark>); <pos>p. p.</pos> <conjf>Spoken</conjf> <pr>(?)</pr> (<conjf>Spoke</conjf>, <mark>Obs. <plain>or</plain> Colloq.</mark>); <pos>p. pr. & vb. n.</pos> <conjf>Speaking</conjf>.]</vmorph> <ety>[OE. <ets>speken</ets>, AS. <ets>specan</ets>, <ets>sprecan</ets>; akin to OF.ries. <ets>spreka</ets>, D. <ets>spreken</ets>, OS. <ets>spreken</ets>, G. <ets>sprechen</ets>, OHG. <ets>sprehhan</ets>, and perhaps to Skr. <ets>sph<umac/rj</ets> to crackle, to thunder.  Cf. <er>Spark</er> of fire, <er>Speech</er>.]</ety> <sn>1.</sn> <def>To utter words or articulate sounds, as human beings; to express thoughts by words; <as>as, the organs may be so obstructed that a man may not be able to <ex>speak</ex></as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Till at the last <qex>spake</qex> in this manner.</q> <rj><qau>Chaucer.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q><qex>Speak</qex>, Lord; for thy servant heareth.</q> <rj><qau>1 Sam. iii. 9.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To express opinions; to say; to talk; to converse.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>That fluid substance in a few minutes begins to set, as the tradesmen <qex>speak</qex>.</q> <rj><qau>Boyle.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>An honest man, is able to <qex>speak</qex> for himself, when a knave is not.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>During the century and a half which followed the Conquest, there is, to <qex>speak</qex> strictly, no English history.</q> <rj><qau>Macaulay.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>To utter a speech, discourse, or harangue; to adress a public assembly formally.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Many of the nobility made themselves popular by <qex>speaking</qex> in Parliament against those things which were most grateful to his majesty.</q> <rj><qau>Clarendon.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>To discourse; to make mention; to tell.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Lycan <qex>speaks</qex> of a part of Caesar's army that came to him from the Leman Lake.</q> <rj><qau>Addison.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <def>To give sound; to sound.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Make all our trumpets <qex>speak</qex>.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>6.</sn> <def>To convey sentiments, ideas, or intelligence as if by utterance; <as>as, features that <ex>speak</ex> of self-will</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Thine eye begins to <qex>speak</qex>.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>To speak of</b></col>, <cd>to take account of, to make mention of.</cd> <au>Robynson (More's Utopia).</au> -- <col><b>To speak out</b></col>, <cd>to speak loudly and distinctly; also, to speak unreservedly.</cd> -- <col><b>To speak well for</b></col>, <cd>to commend; to be favorable to.</cd> -- <col><b>To speak with</b></col>, <cd>to converse with.</cd> <ldquo/Would you <xex>speak with</xex> me?<rdquo/ <au>Shak.</au></cs><br/
[<source>1913 Webster</source>]</p>

<p><syn><b>Syn.</b> -- To say; tell; talk; converse; discourse; articulate; pronounce; utter.</syn><br/
[<source>1913 Webster</source>]</p>

<p><ent>Speak</ent><br/
<hw>Speak</hw> <pr>(?)</pr>, <pos>v. t.</pos> <sn>1.</sn> <def>To utter with the mouth; to pronounce; to utter articulately, as human beings.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>They sat down with him upn ground seven days and seven nights, and none <qex>spake</qex> a word unto him.</q> <rj><qau>Job. ii. 13.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To utter in a word or words; to say; to tell; to declare orally; <as>as, to <ex>speak</ex> the truth; to <ex>speak</ex> sense</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>To declare; to proclaim; to publish; to make known; to exhibit; to express in any way.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>It is my father;s muste<br/
To <qex>speak</qex> your deeds.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q><qex>Speaking</qex> a still good morrow with her eyes.</q> <rj><qau>Tennyson.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>And for the heaven's wide circuit, let it <qex>speak</qex><br/
The maker's high magnificence.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Report <qex>speaks</qex> you a bonny monk.</q> <rj><qau>Sir W. Scott.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>To talk or converse in; to utter or pronounce, as in conversation; <as>as, to <ex>speak</ex> Latin</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>And French she <qex>spake</qex> full fair and fetisely.</q> <rj><qau>Chaucer.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <def>To address; to accost; to speak to.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>[He will] thee in hope; he will <qex>speak</qex> thee fair.</q> <rj><qau>Ecclus. xiii. 6.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>each village senior paused to scan<br/
And <qex>speak</qex> the lovely caravan.</q> <rj><qau>Emerson.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>To speak a ship</b></col> <fld>(Naut.)</fld>, <cd>to hail and speak to her captain or commander.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Speakable</ent><br/
<hw>Speak"a*ble</hw> <pr>(?)</pr>, <pos>a.</pos> <sn>1.</sn> <def>Capable of being spoken; fit to be spoken.</def>  <rj><au>Ascham.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Able to speak.</def>  <rj><au>Milton.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>speakeasy</ent><br/
<hw>speak"eas`y</hw>, <pr>(sp<emac/k"<emac/`z<esl/)</pr> <pos>n.</pos>  <def>An establishment where alcoholic beverages were sold and drunk illegally, especially one operating during the prohibition era in the U.S. (1920-1932); a tavern or nightclub illegally selling alcoholic beverages.</def><br/
[<source>PJC</source>]</p>

<p><ent>Speaker</ent><br/
<hw>Speak"er</hw> <pr>(?)</pr>, <pos>n.</pos> <sn>1.</sn> <def>One who speaks.</def> Specifically: <sd>(a)</sd> <def>One who utters or pronounces a discourse; usually, one who utters a speech in public; <as>as, the man is a good <ex>speaker</ex>, or a bad <ex>speaker</ex></as>.</def> <sd>(b)</sd> <def>One who is the mouthpiece of others; especially, one who presides over, or speaks for, a delibrative assembly, preserving order and regulating the debates; <as>as, the <ex>Speaker</ex> of the House of Commons, originally, the mouthpiece of the House to address the king; the <ex>Speaker</ex> of a House of Representatives.</as></def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A book of selections for declamation.</def> <mark>[U. S.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Speakership</ent><br/
<hw>Speak"er*ship</hw>, <pos>n.</pos> <def>The office of speaker; <as>as, the <ex>speakership</ex> of the House of Representatives</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Speaking</ent><br/
<hw>Speak"ing</hw>, <pos>a.</pos> <sn>1.</sn> <def>Uttering speech; used for conveying speech; <as>as, man is a <ex>speaking</ex> animal; a <ex>speaking</ex> tube.</as></def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Seeming to be capable of speech; hence, lifelike; <as>as, a <ex>speaking</ex> likeness</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>A speaking acquaintance</b></col>, <cd>a slight acquaintance with a person, or one which merely permits the exchange of salutations and remarks on indifferent subjects.</cd> -- <col><b>Speaking trumpet</b></col>, <cd>an instrument somewhat resembling a trumpet, by which the sound of the human voice may be so intensified as to be conveyed to a great distance.</cd> -- <col><b>Speaking tube</b></col>, <cd>a tube for conveying speech, especially from one room to another at a distance.</cd> -- <col><b>To be on speaking terms</b></col>, <cd>to be slightly acquainted.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Speaking</ent><br/
<hw>Speak"ing</hw>, <pos>n.</pos> <sn>1.</sn> <def>The act of uttering words.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Public declamation; oratory.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spear</ent><br/
<hw>Spear</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[OE. <ets>spere</ets>, AS. <ets>spere</ets>; akin to D. & G. <ets>speer</ets>, OS. & OHS. <ets>sper</ets>, Icel. spj<oum/r, pl., Dan. <ets>spaer</ets>, L. <ets>sparus</ets>.]</ety> <sn>1.</sn> <def>A long, pointed weapon, used in war and hunting, by thrusting or throwing; a weapon with a long shaft and a sharp head or blade; a lance.</def> <note>[See <xex>Illust.</xex> of <er>Spearhead</er>.]</note> <ldquo/A sharp ground <xex>spear</xex>.<rdquo/  <rj><au>Chaucer.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>They shall beat their swords into plowshares, and their <qex>spears</qex> into pruning hooks.</q> <rj><qau>Micah iv. 3.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Fig.: A spearman.</def>  <rj><au>Sir W. Scott.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>A sharp-pointed instrument with barbs, used for stabbing fish and other animals.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>A shoot, as of grass; a spire.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <def>The feather of a horse. See <er>Feather</er>, <pos>n.</pos>, 4.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>6.</sn> <def>The rod to which the bucket, or plunger, of a pump is attached; a pump rod.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Spear foot</b></col>, <cd>the off hind foot of a horse.</cd> -- <col><b>Spear grass</b></col>. <fld>(Bot.)</fld> <sd>(a)</sd> <cd>The common reed. See <er>Reed</er>, <pos>n.</pos>, 1.</cd> <sd>(b)</sd> <cd>meadow grass. See under <er>Meadow</er>.</cd> -- <col><b>Spear hand</b></col>, <cd>the hand in which a horseman holds a spear; the right hand.</cd> <au>Crabb.</au> -- <col><b>Spear side</b></col>, <cd>the male line of a family.</cd> <au>Lowell.</au> -- <col><b>Spear thistle</b></col> <fld>(Bot.)</fld>, <cd>the common thistle (<spn>Cnicus lanceolatus</spn>).</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spear</ent><br/
<hw>Spear</hw>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Speared</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Spearing</conjf>.]</vmorph> <def>To pierce with a spear; to kill with a spear; <as>as, to <ex>spear</ex> a fish</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spear</ent><br/
<hw>Spear</hw>, <pos>v. i.</pos> <def>To shoot into a long stem, as some plants. See <er>Spire</er>.</def>  <rj><au>Mortimer.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spearer</ent><br/
<hw>Spear"er</hw> <pr>(?)</pr>, <pos>n.</pos> <def>One who uses a spear; <as>as, a <ex>spearer</ex> of fish</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spearfish</ent><br/
<hw>Spear"fish`</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Zool.)</fld> <sd>(a)</sd> <def>A large and powerful fish (<spn>Tetrapturus albidus</spn>) related to the swordfish, but having scales and ventral fins. It is found on the American coast and the Mediterranean.</def> <sd>(b)</sd> <def>The carp sucker.</def><br/
[<source>1913 Webster</source>]</p>

<p><-- p. 1380 --></p>

<p><ent>Spearhead</ent><br/
<hw>Spear"head`</hw> <pr>(?)</pr>, <pos>n.</pos> <def>The pointed head, or end, of a spear.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spearman</ent><br/
<hw>Spear"man</hw> <pr>(?)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Spearmen</plw> <pr>(<?/)</pr>.</plu> <def>One who is armed with a spear.</def>  <rj><au>Acts xxiii. 23.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spearmint</ent><br/
<hw>Spear"mint`</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[So named from its <ets>spiry</ets>, not capitate, inflorescence. <au>Dr. Prior.</au>]</ety> <fld>(Bot.)</fld> <def>A species of mint (<spn>Mentha viridis</spn>) growing in moist soil. It vields an aromatic oil. See <er>Mint</er>, and <er>Mentha</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spearwood</ent><br/
<hw>Spear"wood`</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Bot.)</fld> <def>An Australian tree (<spn>Acacia Doratoxylon</spn>), and its tough wood, used by the natives for spears.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spearwort</ent><br/
<hw>Spear"wort`</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[AS. <ets>sperewyrt</ets>.]</ety> <fld>(Bot.)</fld> <def>A name given to several species of crowfoot (<gen>Ranunculus</gen>) which have spear-shaped leaves.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Speary</ent><br/
<hw>Spear"y</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Having the form of a spear.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spece</ent><br/
<hw>Spece</hw> <pr>(?)</pr>, <pos>n.</pos> <def>Species; kind.</def> <mark>[Obs.]</mark>  <rj><au>Chaucer.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Specht</ent><br/
<hw>Specht</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[See <er>Speight</er>.]</ety> <fld>(Zool.)</fld> <def>A woodpecker.</def> <mark>[Obs. or prov. Eng.]</mark>  <rj><au>Sherwood.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Special</ent><br/
<hw>Spe"cial</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>specialis</ets>, fr. <ets>species</ets> a particular sort, kind, or quality: cf. F. <ets>sp<eacute/cial</ets>. See <er>Species</er>, and cf. <er>Especial</er>.]</ety> <sn>1.</sn> <def>Of or pertaining to a species; constituting a species or sort.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>A <qex>special</qex> is called by the schools a <ldquo/species<rdquo/.</q> <rj><qau>I. Watts.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Particular; peculiar; different from others; extraordinary; uncommon.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Our Savior is represented everywhere in Scripture as the <qex>special</qex> patron of the poor and the afficted.</q> <rj><qau>Atterbury.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>To this <qex>special</qex> evil an improvement of style would apply a <qex>special</qex> redress.</q> <rj><qau>De Quincey.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>Appropriate; designed for a particular purpose, occasion, or person; <as>as, a <ex>special</ex> act of Parliament or of Congress; a <ex>special</ex> sermon</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>Limited in range; confined to a definite field of action, investigation, or discussion; <as>as, a <ex>special</ex> dictionary of commercial terms; a <ex>special</ex> branch of study</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <def>Chief in excellence.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>The king hath drawn<br/
The <qex>special</qex> head of all the land together.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Special administration</b></col> <fld>(Law)</fld>, <cd>an administration limited to certain specified effects or acts, or one granted during a particular time or the existence of a special cause, as during a controversy respecting the probate of a will, or the right of administration, etc.</cd> -- <col><b>Special agency</b></col>, <cd>an agency confined to some particular matter.</cd> -- <mcol><col><b>Special bail</b></col>, <col><b>Bail above</b></col>, <it>or</it> <col><b>Bail to the action</b></col></mcol> <fld>(Law)</fld>, <cd>sureties who undertake that, if the defendant is convicted, he shall satisfy the plaintiff, or surrender himself into custody.</cd> <au>Tomlins.</au>  <au>Wharton (Law Dict.).</au> -- <col><b>Special constable</b></col>. <cd>See under <er>Constable</er>.</cd> <au>Bouvier.</au> -- <col><b>Special damage</b></col> <fld>(Law)</fld>, <cd>a damage resulting from the act complained of, as a natural, but not the necessary, consequence of it.</cd> -- <col><b>Special demurrer</b></col> <fld>(Law)</fld>, <cd>a demurrer for some defect of form in the opposite party pleading, in which the cause of demurrer is particularly stated.</cd> -- <col><b>Special deposit</b></col>, <cd>a deposit made of a specific thing to be kept distinct from others.</cd> -- <col><b>Special homology</b></col>. <fld>(Biol.)</fld> <cd>See under <er>Homology</er>.</cd> -- <col><b>Special injuction</b></col> <fld>(Law)</fld>, <cd>an injuction granted on special grounds, arising of the circumstances of the case.</cd> <au>Daniell.</au> -- <col><b>Special issue</b></col> <fld>(Law)</fld>, <cd>an issue produced upon a special plea.</cd> <au>Stephen.</au> -- <col><b>Special jury</b></col> <fld>(Law)</fld>, <cd>a jury consisting of persons of some particular calling, station, or qualification, which is called upon motion of either party when the cause is supposed to require it; a struck jury.</cd> -- <col><b>Special orders</b></col> <fld>(Mil.)</fld>, <cd>orders which do not concern, and are not published to, the whole command, such as those relating to the movement of a particular corps, a detail, a temporary camp, etc.</cd> -- <col><b>Special partner</b></col>, <cd>a limited partner; a partner with a limited or restricted responsibility; -- unknown at common law.</cd> -- <col><b>Special partnership</b></col>, <cd>a limited or particular partnership; -- a term sometimes applied to a partnership in a particular business, operation, or adventure.</cd> -- <col><b>Special plea in bar</b></col> <fld>(Law)</fld>, <cd>a plea setting forth particular and new matter, distinguished from the general issue.</cd> <au>Bouvier.</au> -- <col><b>Special pleader</b></col> <fld>(Law)</fld>, <cd>originally, a counsel who devoted himself to drawing special counts and pleas; in a wider sense, a lawyer who draws pleadings.</cd> -- <col><b>Special pleading</b></col> <fld>(Law)</fld>, <cd>the allegation of special or new matter, as distingiushed from a direct denial of matter previously alleged on the side.</cd> <au>Bouvier.</au> <cd>The popular denomination of the whole science of pleading.</cd> <au>Stephen.</au> <cd>The phrase is sometimes popularly applied to the specious, but unsound, argumentation of one whose aim is victory, and not truth.</cd> <au>Burrill.</au> -- <col><b>Special property</b></col> <fld>(Law)</fld>, <cd>a qualified or limited ownership possession, as in wild animals, things found or bailed.</cd> -- <col><b>Special session</b></col>, <cd>an extraordinary session; a session at an unusual time or for an unusual purpose; <as>as, a <ex>special session</ex> of Congress or of a legislature</as>.</cd> -- <mcol><col><b>Special statute</b></col>, <it>or</it> <col><b>Special law</b></col></mcol>, <cd>an act of the legislature which has reference to a particular person, place, or interest; a <cref>private law</cref>; -- in distinction from a <contr>general law</contr> or <contr>public law</contr>.</cd> -- <col><b>Special verdict</b></col> <fld>(Law)</fld>, <cd>a special finding of the facts of the case, leaving to the court the application of the law to them.</cd> <au>Wharton (Law Dict.).</au></cs><br/
[<source>1913 Webster</source>]</p>

<p><syn><b>Syn.</b> -- Peculiar; appropriate; specific; dictinctive; particular; exceptional; singular. See <er>Peculiar</er>.</syn><br/
[<source>1913 Webster</source>]</p>

<p><ent>Special</ent><br/
<hw>Spe"cial</hw>, <pos>n.</pos> <sn>1.</sn> <def>A particular.</def> <mark>[Obs.]</mark>  <rj><au>Hammond.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>One appointed for a special service or occasion.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>In special</b></col>, <cd>specially; in particular.</cd> <au>Chaucer.</au></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Specialism</ent><br/
<hw>Spe"cial*ism</hw> <pr>(?)</pr>, <pos>n.</pos> <def>Devotion to a particular and restricted part or branch of knowledge, art, or science; <as>as, medical <ex>specialism</ex></as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Specialist</ent><br/
<hw>Spe"cial*ist</hw> <pr>(?)</pr>, <pos>n.</pos> <def>One who devotes himself to some specialty; <as>as, a medical <ex>specialist</ex>, one who devotes himself to diseases of particular parts of the body, as the eye, the ear, the nerves, etc.</as></def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Speciality</ent><br/
<hw>Spe`ci*al"i*ty</hw> <pr>(?)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Specialities</plw> <pr>(#)</pr>.</plu> <ety>[See <er>Special</er>, and <er>Specialty</er>.]</ety> <sn>1.</sn> <def>A particular or peculiar case; a particularity.</def>  <rj><au>Sir M. Hale.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Law)</fld> <def>See <er>Specialty</er>, 3.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>The special or peculiar mark or characteristic of a person or thing; that for which a person is specially distinguished; an object of special attention; a special occupation or object of attention; a specialty.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>On these two general heads all other <qex>specialities</qex> are depedent.</q> <rj><qau>Hooker.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Strive, while improving your one talent, to enrich your whole capital as a man. It is in this way that you escape from the wretched narrow-mindedness which is the characteristic of every one who cultivates his <qex>speciality</qex>.</q> <rj><qau>Ld. Lytton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>We 'll say, instead, the inconsequent creature man, -<br/
For that'a his <qex>speciality</qex>.</q> <rj><qau>Mrs. Browning.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Think of this, sir, . . . remote from the impulses of passion, and apart from the <qex>specialities</qex> -- if I may use that strong remark -- of prejudice.</q> <rj><qau>Dickens.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>An attribute or quality peculiar to a species.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Specialization</ent><br/
<hw>Spe`cial*i*za"tion</hw> <pr>(?)</pr>, <pos>n.</pos> <sn>1.</sn> <def>The act of specializing, or the state of being spezialized.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Biol.)</fld> <def>The setting apart of a particular organ for the performance of a particular function.</def>  <rj><au>Darwin.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Specialize</ent><br/
<hw>Spe"cial*ize</hw> <pr>(?)</pr>, <pos>v. t.</pos> <sn>1.</sn> <def>To mention specially; to particularize.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To apply to some specialty or limited object; to assign to a specific use; <as>as, <ex>specialized</ex> knowledge</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <fld>(Biol.)</fld> <def>To supply with an organ or organs having a special function or functions.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Specially</ent><br/
<hw>Spe"cial*ly</hw>, <pos>adv.</pos> <sn>1.</sn> <def>In a special manner; particularly; especially.</def>  <rj><au>Chaucer.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>For a particular purpose; <as>as, a meeting of the legislature is <ex>specially</ex> summoned</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Specialty</ent><br/
<hw>Spe"cial*ty</hw> <pr>(?)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Specialties</plw> <pr>(#)</pr>.</plu> <ety>[F. <ets>sp<eacute/cialit</ets><eacute/.  Cf. <er>Speciality</er>.]</ety> <sn>1.</sn> <def>Particularity.</def><br/
[<source>1913 Webster</source>]</p>

<p><q><qex>Specialty</qex> of rule hath been neglected.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A particular or peculiar case.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <fld>(Law)</fld> <def>A contract or obligation under seal; a contract by deed; a writing, under seal, given as security for a debt particularly specified.</def>  <rj><au>Chitty.</au>  <au>Bouvier.</au>  <au>Wharton (Law Dict.).</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Let <qex>specialties</qex> be therefore drawn between us.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>That for which a person is distinguished, in which he is specially versed, or which he makes an object of special attention; a speciality.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Men of boundless knowledge, like Humbold, must have had once their <qex>specialty</qex>, their pet subject.</q> <rj><qau>C. Kingsley.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Specie</ent><br/
||<hw>Spe"ci*e</hw> <pr>(?)</pr>, <def><pos>abl.</pos> of L. <xex>species</xex> sort, kind.  Used in the phrase <xex>in specie</xex>, that is, in sort, in kind, in (its own) form.</def><br/
[<source>1913 Webster</source>]</p>

<p><q><ldquo/[The king] expects a return in <qex>specie</qex> from them<rdquo/ [<it>i. e.</it>, kindness for kindness].</q> <rj><qau>Dryden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>In specie</b></col> <fld>(Law)</fld>, <cd>in precise or definite form; specifically; according to the exact terms; of the very thing.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Specie</ent><br/
<hw>Spe"cie</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Formed as a singular from <ets>species</ets>, in sense 5.]</ety> <def>Coin; hard money.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Species</ent><br/
<hw>Spe"cies</hw> <pr>(?)</pr>, <pos>n. sing. & pl.</pos> <ety>[L., a sight, outward appearance, shape, form, a particular sort, kind, or quality, a species. See <er>Spice</er>, <pos>n.</pos>, and cf. <er>Specie</er>, <er>Special</er>.]</ety> <sn>1.</sn> <def>Visible or sensible presentation; appearance; a sensible percept received by the imagination; an image.</def> <mark>[R.]</mark> <ldquo/The <xex>species</xex> of the letters illuminated with indigo and violet.<rdquo/  <rj><au>Sir I. Newton.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Wit, . . . the faculty of imagination in the writer, which searches over all the memory for the <qex>species</qex> or ideas of those things which it designs to represent.</q> <rj><qau>Dryden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ In the scholastic philosophy, the species was <xex>sensible</xex> and <xex>intelligible</xex>.  The <xex>sensible</xex> species was that in any material, object which was in fact discerned by the mind through the organ of perception, or that in any object which rendered it possible that it should be perceived.  The sensible species, as apprehended by the understanding in any of the relations of thought, was called an <xex>intelligible</xex> species.  <ldquo/An apparent diversity between the <xex>species</xex> visible and audible is, that the visible doth not mingle in the medium, but the audible doth.<rdquo/</note>  <rj><au>Bacon.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Logic)</fld> <def>A group of individuals agreeing in common attributes, and designated by a common name; a conception subordinated to another conception, called a genus, or generic conception, from which it differs in containing or comprehending more attributes, and extending to fewer individuals.  Thus, <er>man</er> is a <xex>species</xex>, under <er>animal</er> as a <xex>genus</xex>; and <xex>man</xex>, in its turn, may be regarded as a <xex>genus</xex> with respect to <er>European</er>, <er>American</er>, or the like, as <xex>species</xex>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>In science, a more or less permanent group of existing things or beings, associated according to attributes, or properties determined by scientific observation.</def><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ In mineralogy and chemistry, objects which possess the same definite chemical structure, and are fundamentally the same in crystallization and physical characters, are classed as belonging to a <xex>species</xex>.  In Zoology and botany, a <xex>species</xex> is an ideal group of individuals which are believed to have descended from common ancestors, which agree in essential characteristics, and are capable of indefinitely continued fertile reproduction through the sexes.  A <xex>species</xex>, as thus defined, differs from a <xex>variety</xex> or <xex>subspecies</xex> only in the greater stability of its characters and in the absence of individuals intermediate between the related groups.</note><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>A sort; a kind; a variety; <as>as, a <ex>species</ex> of low cunning; a <ex>species</ex> of generosity; a <ex>species</ex> of cloth.</as></def><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <def>Coin, or coined silver, gold, or other metal, used as a circulating medium; specie.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>There was, in the splendor of the Roman empire, a less quantity of current <qex>species</qex> in Europe than there is now.</q> <rj><qau>Arbuthnot.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>6.</sn> <def>A public spectacle or exhibition.</def> <mark>[Obs.]</mark>  <rj><au>Bacon.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>7.</sn> <fld>(Pharmacy)</fld> <sd>(a)</sd> <def>A component part of a compound medicine; a simple.</def> <sd>(b)</sd> <fld>(Med.)</fld> <def>An officinal mixture or compound powder of any kind; esp., one used for making an aromatic tea or tisane; a tea mixture.</def>  <rj><au>Quincy.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>8.</sn> <fld>(Civil Law)</fld> <def>The form or shape given to materials; fashion or shape; form; figure.</def>  <rj><au>Burill.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Incipient species</b></col> <fld>(Zool.)</fld>, <cd>a subspecies, or variety, which is in process of becoming permanent, and thus changing to a true species, usually by isolation in localities from which other varieties are excluded.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Specifiable</ent><br/
<hw>Spec"i*fi`a*ble</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Admitting specification; capable of being specified.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>specific</ent><br/
<hw>spe*cif"ic</hw> <pr>(sp<esl/*s<icr/f"<icr/k)</pr>, <pos>a.</pos> <ety>[F. <ets>sp<eacute/cifique</ets>, or NL. <ets>spesificus</ets>; L. <ets>species</ets> a particular sort or kind + <ets>facere</ets> to make.  Cf. <er>specify</er>.]</ety> <sn>1.</sn> <def>Of or pertaining to a species; characterizing or constituting a species; possessing the peculiar property or properties of a thing which constitute its species, and distinguish it from other things; <as>as, the <ex>specific</ex> form of an animal or a plant; the <ex>specific</ex> qualities of a drug; the <ex>specific</ex> distinction between virtue and vice.</as></def><br/
[<source>1913 Webster</source>]</p>

<p><q><qex>Specific</qex> difference is that primary attribute which distinguishes each species from one another.</q> <rj><qau>I. Watts.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Specifying; definite, or making definite; limited; precise; discriminating; <as>as, a <ex>specific</ex> statement</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <fld>(Med.)</fld> <def>Exerting a peculiar influence over any part of the body; preventing or curing disease by a peculiar adaptation, and not on general principles; <as>as, quinine is a <ex>specific</ex> medicine in cases of malaria</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>In fact, all medicines will be found <qex>specific</qex> in the perfection of the science.</q> <rj><qau>Coleridge.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Specific character</b></col> <fld>(Nat. Hist.)</fld>, <cd>a characteristic or characteristics distinguishing one species from every other species of the same genus.</cd> -- <col><b>Specific disease</b></col> <fld>(Med.)</fld> <sd>(a)</sd> <cd>A disease which produces a determinate definite effect upon the blood and tissues or upon some special tissue</cd>. <sd>(b)</sd> <cd>A disease which is itself uniformly produced by a definite and peculiar poison or organism.</cd> -- <col><b>Specific duty</b></col>. <fld>(Com.)</fld> <cd>See under <er>Duty</er>.</cd> -- <col><b>Specific gravity</b></col>. <fld>(Physics)</fld> <cd>See under <er>Gravity</er>.</cd> -- <col><b>Specific heat</b></col> <fld>(Physics)</fld>, <cd>the quantity of heat required to raise the temperature of a body one degree, taking as the unit of measure the quantity required to raise the same weight of water from zero to one degree; <as>thus, the <ex>specific heat</ex> of mercury is 0.033, that of water being 1.000</as>.</cd> -- <col><b>Specific inductive capacity</b></col> <fld>(Physics)</fld>, <cd>the effect of a dielectric body in producing static electric induction as compared with that of some other body or bodies referred to as a standard.</cd> -- <col><b>Specific legacy</b></col> <fld>(Law)</fld>, <cd>a bequest of a particular thing, as of a particular animal or piece of furniture, specified and distinguished from all others.</cd> <au>Wharton.</au> <au>Burrill.</au> -- <col><b>Specific name</b></col> <fld>(Nat. Hist.)</fld>, <cd>the name which, appended to the name of the genus, constitutes the distinctive name of the species; -- originally applied by Linnaeus to the essential character of the species, or the <xex>essential difference</xex>.  The present <ex>specific name</ex> he at first called the <altname>trivial name</altname>.</cd> -- <col><b>Specific performance</b></col> <fld>(Law)</fld>, <cd>the peformance of a contract or agreement as decreed by a court of equity.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Specific</ent><br/
<hw>Spe*cif"ic</hw>, <pos>n.</pos> <sn>1.</sn> <fld>(Med.)</fld> <def>A specific remedy. See <er>Specific</er>, <pos>a.</pos>, 3.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>His parents were weak enough to believe that the royal touch was a <qex>specific</qex> for this malady.</q> <rj><qau>Macaulay.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Anything having peculiar adaption to the purpose to which it is applied.</def>  <rj><au>Dr. H. More.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Specifical</ent><br/
<hw>Spe*cif"ic*al</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Specific.</def>  <rj><au>Bacon.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Specifically</ent><br/
<hw>Spe*cif"ic*al*ly</hw> <pr>(?)</pr>, <pos>adv.</pos> <def>In a specific manner.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Specificalness</ent><br/
<hw>Spe*cif"ic*al*ness</hw>, <pos>n.</pos> <def>The quality of being specific.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Specificate</ent><br/
<hw>Spe*cif"i*cate</hw> <pr>(sp<esl/*s<icr/f"<icr/*k<amac/t)</pr>, <pos>v. t.</pos> <ety>[See <er>Specify</er>.]</ety> <def>To show, mark, or designate the species, or the distinguishing particulars of; to specify.</def> <mark>[Obs.]</mark>  <rj><au>ir M. Hale.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>specification</ent><br/
<hw>spec`i*fi*ca"tion</hw> <pr>(sp<ecr/s`<icr/*f<icr/*k<amac/"sh<ucr/n)</pr>, <pos>n.</pos> <ety>[Cf. F. <ets>sp<eacute/cification</ets>, LL. <ets>specificatio</ets>.]</ety> <sn>1.</sn> <def>The act of specifying or determining by a mark or limit; notation of limits.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>This <qex>specification</qex> or limitation of the question hinders the disputers from wandering away from the precise point of inquiry.</q> <rj><qau>I. Watts.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>The designation of particulars; particular mention; <as>as, the <ex>specification</ex> of a charge against an officer</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>A written statement containing a minute description or enumeration of particulars, as of charges against a public officer, the terms of a contract, the description of an invention, as in a patent; also, a single article, item, or particular, an allegation of a specific act, as in a charge of official misconduct.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>A detailed listing or description of the required properties of some object proposed to be built or bought; -- usually used in the <pluf>plural</pluf>; <as>as, the building <ex>specifications</ex> require that it withstand an earthquake of magnitude 8; the program <ex>specifications</ex> require an option to change the menus</as>.</def><br/
[<source>PJC</source>]</p>

<p><ent>Specificness</ent><br/
<hw>Spe*cif"ic*ness</hw> <pr>(?)</pr>, <pos>n.</pos> <def>The quality or state of being specific.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Specify</ent><br/
<hw>Spec"i*fy</hw> <pr>(?)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Specified</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Specifying</conjf> <pr>(?)</pr>.]</vmorph> <ety>[F. <ets>sp<eacute/cifier</ets>, or OF. <ets>especifier</ets>, fr. LL. <ets>specificare</ets>. See <er>Species</er>, <er>-fy</er>.]</ety> <def>To mention or name, as a particular thing; to designate in words so as to distinguish from other things; <as>as, to <ex>specify</ex> the uses of a plant; to <ex>specify</ex> articles purchased</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>He has there given us an exact geography of Greece, where the countries and the uses of their soils are <qex>specified</qex>.</q> <rj><qau>Pope.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Specillum</ent><br/
||<hw>Spe*cil"lum</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L.]</ety> <fld>(Med.)</fld> <def>See <er>Stylet</er>, 2.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Specimen</ent><br/
<hw>Spec"i*men</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L., fr. <ets>specere</ets> to look, to behold. See <er>Spy</er>.]</ety> <def>A part, or small portion, of anything, or one of a number of things, intended to exhibit the kind and quality of the whole, or of what is not exhibited; a sample; <as>as, a <ex>specimen</ex> of a man's handwriting; a <ex>specimen</ex> of a person's blood; a <ex>specimen</ex> of painting; a<ex>specimen</ex> of one's art</as>.</def><br/
[<source>1913 Webster</source> <source>+PJC</source>]</p>

<p><syn><b>Syn.</b> -- Sample; model; pattern.</syn>  -- <usage><er>Specimen</er>, <er>Sample</er>. A <xex>specimen</xex> is a representative of the class of things to which it belongs; as, a <xex>specimen</xex> of photography. A <xex>sample</xex> is a part of the thing itself, designed to show the quality of the whole; as, a <xex>sample</xex> of sugar or of broadcloth. A cabinet of minerals consists of <xex>specimens</xex>; if a part be broken off from any one of these, it is a <xex>sample</xex> of the mineral to which it belongs. <ldquo/Several persons have exhibited <xex>specimens</xex> of this art before multitudes of beholders.<rdquo/ <au>Addison.</au> <ldquo/I design this but for a <xex>sample</xex> of what I hope more fully to discuss.<rdquo/ <au>Woodward.</au></usage><br/
[<source>1913 Webster</source>]</p>

<p><ent>Speciosity</ent><br/
<hw>Spe`ci*os"i*ty</hw> <pr>(?)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Speciocities</plw> <pr>(#)</pr>.</plu> <ety>[Cf. LL. <er>speciositas</er>.]</ety> <sn>1.</sn> <def>The quality or state of being specious; speciousness.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Professions built so largely on <qex>speciosity</qex>, instead of performance.</q> <rj><qau>Carlyle.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>That which is specious.</def>  <rj><au>Dr. H. More.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><-- p. 1381 --></p>

<p><ent>Specious</ent><br/
<hw>Spe"cious</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>speciosus</ets>good-looking, beautiful, specious, fr. <ets>species</ets> look, show, appearance; cf. F. <ets>sp<eacute/coeux</ets>. See <er>Species</er>.]</ety> <sn>1.</sn> <def>Presenting a pleasing appearance; pleasing in form or look; showy.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Some [serpents] <qex>specious</qex> and beautiful to the eye.</q> <rj><qau>Bp. Richardson.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>The rest, far greater part,<br/
Will deem in outward rites and <qex>specious</qex> forms<br/
Religion satisfied.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Apparently right; superficially fair, just, or correct, but not so in reality; appearing well at first view; plausible; <as>as, <ex>specious</ex> reasoning; a <ex>specious</ex> argument.</as></def><br/
[<source>1913 Webster</source>]</p>

<p><q>Misled for a moment by the <qex>specious</qex> names of religion, liberty, and property.</q> <rj><qau>Macaulay.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>In consequence of their greater command of <qex>specious</qex> expression.</q> <rj><qau>J. Morley.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><syn><b>Syn.</b> -- Plausible; showy; ostensible; colorable; feasible. See <er>Plausible</er>.</syn><br/
[<source>1913 Webster</source>]</p>

<p>-- <wordforms><wf>Spe"xious*ly</wf> <pr>(#)</pr>, <pos>adv.</pos> -- <wf>Spe"cious*ness</wf>, <pos>n.</pos></wordforms><br/
[<source>1913 Webster</source>]</p>

<p><ent>Speck</ent><br/
<hw>Speck</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Cf. Icel. <ets>spik</ets> blubber, AS. <ets>spic</ets>, D. <ets>spek</ets>, G. <ets>speck</ets>.]</ety> <def>The blubber of whales or other marine mammals; also, the fat of the hippopotamus.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Speck falls</b></col> <fld>(Naut.)</fld>, <cd>falls or ropes rove through blocks for hoisting the blubber and bone of whales on board a whaling vessel.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Speck</ent><br/
<hw>Speck</hw>, <pos>n.</pos> <ety>[OE. <ets>spekke</ets>, AS. <ets>specca</ets>; cf. LG. <ets>spaak</ets>.]</ety> <sn>1.</sn> <def>A small discolored place in or on anything, or a small place of a color different from that of the main substance; a spot; a stain; a blemish; <as>as, a <ex>speck</ex> on paper or loth; <ex>specks</ex> of decay in fruit</as>.</def> <ldquo/Gray sand, with black <xex>specks</xex>.<rdquo/  <rj><au>Anson.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A very small thing; a particle; a mite; <as>as, <ex>specks</ex> of dust; he has not a <ex>speck</ex> of money.</as></def><br/
[<source>1913 Webster</source>]</p>

<p><q>Many bright <qex>specks</qex> bubble up along the blue Egean.</q> <rj><qau>Landor.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <fld>(Zool.)</fld> <def>A small etheostomoid fish (<spn>Ulocentra stigmaea</spn>) common in the Eastern United States.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Speck</ent><br/
<hw>Speck</hw>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Specked</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Specking</conjf>.]</vmorph> <def>To cause the presence of specks upon or in, especially specks regarded as defects or blemishes; to spot; to speckle; <as>as, paper <ex>specked</ex> by impurities in the water used in its manufacture</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Carnation, purple, azure, or <qex>specked</qex> with gold.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Speckle</ent><br/
<hw>Spec"kle</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Dim. of <ets>speck</ets>; cf. D. <ets>spikkel</ets>.]</ety> <def>A little or spot in or anything, of a different substance or color from that of the thing itself.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>An huge great serpent, all with <qex>speckles</qex> pied.</q> <rj><qau>Spebser.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Speckle</ent><br/
<hw>Spec"kle</hw>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Speckled</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Speckling</conjf> <pr>(?)</pr>.]</vmorph> <def>To mark with small spots of a different color from that of the rest of the surface; to variegate with spots of a different color from the ground or surface.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Speckled</ent><br/
<hw>Spec"kled</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Marked or variegated with small spots of a different color from that of the rest of the surface.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Speckled Indians</b></col> <fld>(Ethnol.)</fld>, <cd>the Pintos.</cd> -- <col><b>Speckled trout</b></col>. <fld>(Zool.)</fld> <sd>(a)</sd> <cd>The common American brook trout. See <er>Trout</er>.</cd> <sd>(b)</sd> <cd>The rainbow trout.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Speckled-belly</ent><br/
<hw>Spec"kled-bel`ly</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Zool.)</fld> <def>The gadwall.</def> <mark>[Local, U.S.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Speckled-bill</ent><br/
<hw>Spec"kled-bill"</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Zool.)</fld> <def>The American white-fronted goose (<spn>Anser albifrons</spn>).</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Speckledness</ent><br/
<hw>Spec"kled*ness</hw>, <pos>n.</pos> <def>The quality of being speckled.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Specksioneer</ent><br/
<hw>Speck`sion*eer"</hw> <pr>(?)</pr>, <pos>n.</pos> <def>The chief harpooner, who also directs in cutting up the speck, or blubber; -- so called among whalers.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Speckt</ent><br/
<hw>Speckt</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A woodpecker. See <er>Speight</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spectacle</ent><br/
<hw>Spec"ta*cle</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F., fr. L. <ets>spectaculum</ets>, fr. <ets>spectare</ets> to look at, to behold, v. intens. fr. <ets>specere</ets>. See <er>Spy</er>.]</ety> <sn>1.</sn> <def>Something exhibited to view; usually, something presented to view as extraordinary, or as unusual and worthy of special notice; a remarkable or noteworthy sight; a show; a pageant; a gazingstock.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>O, piteous <qex>spectacle</qex>? O, bloody times!</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A spy-glass; a looking-glass.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>Poverty a <qex>spectacle</qex> is, as thinketh me,<br/
Through which he may his very friends see.</q> <rj><qau>Chaucer.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <pluf>pl.</pluf> <def>An optical instrument consisting of two lenses set in a light frame, and worn to assist sight, to obviate some defect in the organs of vision, or to shield the eyes from bright light.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <pluf>pl.</pluf> <def>Fig.: An aid to the intellectual sight.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Shakespeare . . . needed not the <qex>spectacles</qex> of books to read nature.</q> <rj><qau>Dryden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><syn><b>Syn.</b> -- Show; sight; exhibition; representation; pageant.</syn><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spectacled</ent><br/
<hw>Spec"ta*cled</hw> <pr>(?)</pr>, <pos>a.</pos> <sn>1.</sn> <def>Furnished with spectacles; wearing spectacles.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>As <qex>spectacled</qex> she sits in chimney nook.</q> <rj><qau>Keats.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Zool.)</fld> <def>Having the eyes surrounded by color markings, or patches of naked skin, resembling spectacles.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Spectacled bear</b></col> <fld>(Zool.)</fld>, <cd>a South American bear (<spn>Tremarclos ornatus</spn>) which inhabits the high mountains of Chili and Peru. It has a light-colored ring around each eye.</cd> -- <mcol><col><b>Spectacled coot</b></col>, <it>or</it> <col><b>Spectacled duck</b></col></mcol> <fld>(Zool.)</fld>, <cd>the surf scoter, or surf duck.</cd> <mark>[Local, U.S.]</mark> -- <col><b>Spectacled eider</b></col> <fld>(Zool.)</fld> <cd>See <er>Eider</er>.</cd> -- <col><b>Spectacled goose</b></col> <fld>(Zool.)</fld>, <cd>the gannet.</cd> -- <col><b>Spectacled snake</b></col> <fld>(Zool.)</fld>, <cd>the cobra de capello.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spectacular</ent><br/
<hw>Spec*tac"u*lar</hw> <pr>(?)</pr>, <pos>a.</pos> <sn>1.</sn> <def>Of or pertaining to a shows; of the nature of a show.</def> <ldquo/<xex>Spectacular</xex> sports.<rdquo/  <rj><au>G. Hickes.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Adapted to excite wonder and admiration by a display of pomp or of scenic effects; <as>as, a <ex>spectacular</ex> celebration of some event; a <ex>spectacular</ex> play.</as></def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>Pertaining to spectacles, or glasses for the eyes.</def><br/
[<source>1913 Webster</source>]</p>

<p><-- 4. Unusual and striking. --><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spectant</ent><br/
<hw>Spec"tant</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>spectans</ets>, p. pr. of <ets>spectare</ets> to look at.]</ety> <def>Looking forward.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spectation</ent><br/
<hw>Spec*ta"tion</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>spectatio</ets>.]</ety> <def>Regard; aspect; appearance.</def>  <rj><au>Harvey.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spectator</ent><br/
<hw>Spec*ta"tor</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>spectator</ets>: cf. F. <ets>spectateur</ets>. See <er>Spectacle</er>.]</ety> <def>One who on; one who sees or beholds; a beholder; one who is personally present at, and sees, any exhibition; <as>as, the <ex>spectators</ex> at a show</as>.</def> <ldquo/Devised and played to take <xex>spectators</xex>.<rdquo/
<-- an eyewitness -->  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><syn><b>Syn.</b> -- Looker-on; beholder; observer; witness.</syn><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spectatorial</ent><br/
<hw>Spec`ta*to"ri*al</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Of or pertaining to a spectator.</def>  <rj><au>Addison.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spectatorship</ent><br/
<hw>Spec*ta"tor*ship</hw> <pr>(?)</pr>, <pos>n.</pos> <sn>1.</sn> <def>The office or quality of a spectator.</def> <mark>[R.]</mark>  <rj><au>Addison.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>The act of beholding.</def> <mark>[Obs.]</mark>  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spectatrix</ent><br/
<ent>Spectatress</ent><br/
<mhw>{ <hw>Spec*ta"tress</hw> <pr>(?)</pr>, <hw>Spec*ta"trix</hw> <pr>(?)</pr>, }</mhw> <pos>n.</pos> <ety>[L. <ets>spectatrix</ets>.]</ety> <def>A female beholder or looker-on.</def> <ldquo/A <xex>spectatress</xex> of the whole scene.<rdquo/  <rj><au>Jeffrey.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spectre</ent><br/
<ent>Specter</ent><br/
<mhw>{ <hw>Spec"ter</hw>, <hw>Spec"tre</hw>  }</mhw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F. <ets>spectre</ets>, fr. L. <ets>spectrum</ets> an appearance, image, specter, fr. <ets>specere</ets> to look. See <er>Spy</er>, and cf. <er>Spectrum</er>.]</ety> <sn>1.</sn> <def>Something preternaturally visible; an apparition; a ghost; a phantom.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The ghosts of traitors from the bridge descend,<br/
With bold fanatic <qex>specters</qex> to rejoice.</q> <rj><qau>Dryden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Zool.)</fld> <sd>(a)</sd> <def>The tarsius.</def> <sd>(b)</sd> <def>A stick insect.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Specter bat</b></col> <fld>(Zool.)</fld>, <cd>any phyllostome bat.</cd> -- <col><b>Specter candle</b></col> <fld>(Zool.)</fld>, <cd>a belemnite.</cd> -- <col><b>Specter shrimp</b></col> <fld>(Zool.)</fld>, <cd>a skeleton shrimp. See under <er>Skeleton</er>.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spectioneer</ent><br/
<hw>Spec`tion*eer"</hw> <pr>(?)</pr>, <pos>n.</pos> <def>Same as <er>Specsioneer</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spectral</ent><br/
<hw>Spec"tral</hw> <pr>(?)</pr>, <pos>a.</pos> <sn>1.</sn> <def>Of or pertaining to a specter; ghosty.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>He that feels timid at the <qex>spectral</qex> form of evil is not the man to spread light.</q> <rj><qau>F. W. Robertson.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Opt.)</fld> <def>Of or pertaining to the spectrum; made by the spectrum; <as>as, <ex>spectral</ex> colors; <ex>spectral</ex> analysis.</as></def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Spectral lemur</b></col>. <fld>(Zool.)</fld> <cd>See <er>Tarsius</er>.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spectrally</ent><br/
<hw>Spec"tral*ly</hw>, <pos>adv.</pos> <def>In the form or manner of a specter.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spectre</ent><br/
<hw>Spec"tre</hw> <pr>(?)</pr>, <pos>n.</pos> <def>See <er>Specter</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spectrobolometer</ent><br/
<hw>Spec`tro*bo*lom"e*ter</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Physics)</fld> <def>A combination of spectroscope and bolometer for determining the distribution of energy in a spectrum.</def> -- <wordforms><wf>Spec`tro*bo`lo*met"ric</wf> <pr>(#)</pr>, <pos>a.</pos></wordforms><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Spectroelectric</ent><br/
<hw>Spec`tro*e*lec"tric</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Pert. to or designating any form of spark tube the electric discharge within which is used in spectroscopic observations.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Spectrogram</ent><br/
<hw>Spec"tro*gram</hw> <pr>(?)</pr>, <pos>n.</pos>  <ety>[<ets>Spectrum</ets> + <ets>-gram</ets>.]</ety> <fld>(Physics)</fld> <def>A photograph, map, or diagram of a spectrum.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Spectrograph</ent><br/
<hw>Spec"tro*graph</hw> <pr>(?)</pr>, <pos>n.</pos>  <ety>[<ets>Spectrum</ets> + <ets>graph</ets>.]</ety> <fld>(Physics)</fld> <sd>(a)</sd> <def>An apparatus for photographing or mapping a spectrum.</def> <sd>(b)</sd> <def>A photograph or picture of a spectrum.</def> -- <wordforms><wf>Spec`tro*graph"ic</wf> <pr>(#)</pr>, <pos>a.</pos> -- <wf>Spec`tro*graph"ic*al*ly</wf> <pr>(#)</pr>, <pos>adv.</pos> -- <wf>Spec*trog"ra*phy</wf> <pr>(#)</pr>, <pos>n.</pos></wordforms><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Spectroheliogram</ent><br/
<hw>Spec`tro*he"li*o*gram`</hw> <pr>(?)</pr>, <pos>n.</pos>  <ety>[<ets>Spectrum</ets> + <ets>heloi-</ets> + <ets>-gram</ets>.]</ety> <fld>(Astrophysics)</fld> <def>A photograph of the sun made by monochromatic light, usually of the calcium line (k), and showing the sun's faculae and prominences.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Spectroheliograph</ent><br/
<hw>Spec`tro*he"li*o*graph</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Astrophysics)</fld> <def>An apparatus for making spectroheliograms, consisting of a spectroscopic camera used in combination with a telescope, and provided with clockwork for moving the  sun's image across the slit.</def> -- <wordforms><wf>Spec`tro*he`li*o*graph"ic</wf> <pr>(#)</pr>, <pos>a.</pos></wordforms><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Spectrological</ent><br/
<hw>Spec`tro*log"ic*al</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Of or pertaining to spectrology; <as>as, <ex>spectrological</ex> studies or experiments</as>.</def> -- <wordforms><wf>Spec`tro*log"ic*al*ly</wf>, <pos>adv.</pos></wordforms><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spectrology</ent><br/
<hw>Spec*trol"o*gy</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[<ets>Spectrum</ets> + <ets>-logy</ets>.]</ety> <fld>(Chem.Phys.)</fld> <def>The science of spectrum analysis in any or all of its relations and applications.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spectrometer</ent><br/
<hw>Spec*trom"e*ter</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[<ets>Spectrum</ets> + <ets>-meter</ets>.]</ety> <fld>(Physics)</fld> <def>A spectroscope fitted for measurements of the luminious spectra observed with it.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spectrometry</ent><br/
<hw>Spec*trom"e*try</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Physics)</fld> <def>Art or process of using a spectrometer, of any type; -- when used alone, it usually refers to the use of a spectrometer using light in the visible, infrared, or ultraviolet region.</def> -- <wordforms><wf>Spec`tro*met"ric</wf> <pr>(#)</pr>, <pos>a.</pos></wordforms><br/
[<source>Webster 1913 Suppl.</source> <source>+PJC</source>]</p>

<p><cs><col><b>mass spectrometry</b></col>, <cd>the art or process of using a mass spectrometer to analyse chemical samples.</cd></cs><br/
[<source>PJC</source>]</p>

<p><ent>Spectrophone</ent><br/
<hw>Spec"tro*phone</hw> <pr>(?)</pr>, <pos>n.</pos>  <ety>[<ets>Spectrum</ets> + Gr. <?/ sound.]</ety> <def>An instrument constructed on the principle of the photophone and used in spectrum analysis as an adjunct to the spectroscope.</def> -- <wordforms><wf>Spec`tro*phon"ic</wf> <pr>(#)</pr>, <pos>a.</pos></wordforms><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Spectrophotometer</ent><br/
<hw>Spec`tro*pho*tom"e*ter</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[<ets>Spectrum</ets> + <ets>photometer</ets>.]</ety> <sn>1.</sn> <fld>(Opt.)</fld> <def>An instrument for measuring or comparing the intensities of the colors of the spectrum.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Chem., Physics)</fld> <def>An analytical instrument for measuring the degree of absorption of light at different wavelengths by a chemical substance.  A specific pattern of absorption intensities at a series of wavelengths is a characteristic of each different chemical substance.</def><br/
[<source>PJC</source>]</p>

<p><ent>Spectrophotometry</ent><br/
<hw>Spec`tro*pho*tom"e*try</hw> <pr>(?)</pr>, <pos>n.</pos> <sn>1.</sn> <def>The art of comparing, photometrically, the brightness of two spectra, wave length by wave length; the use of the spectrophotometer.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><sn>2.</sn> <fld>(Chem., Physics)</fld> <def>the art or process of measuring the degree of absorption of light at different wavelengths by a chemical substance, by means of a spectrometer or spectrophotometer.  It is a technique for chemical analysis.</def><br/
[<source>PJC</source>]</p>

<p> -- <wordforms><wf>Spec`tro*pho`to*met"ric</wf> <pr>(#)</pr>, <pos>a.</pos></wordforms><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Spectroscope</ent><br/
<hw>Spec"tro*scope</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[<ets>Spectrum</ets> + <ets>-scope</ets>.]</ety> <fld>(Physics)</fld> <def>An optical instrument for forming and examining spectra (as that of solar light, or those produced by flames in which different substances are volatilized), so as to determine, from the position of the spectral lines, the composition of the substance.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spectroscopical</ent><br/
<ent>Spectroscopic</ent><br/
<mhw>{ <hw>Spec`tro*scop"ic</hw> <pr>(?)</pr>, <hw>Spec`tro*scop"ic*al</hw> <pr>(?)</pr>, }</mhw> <pos>a.</pos> <def>Of or pertaining to a spectroscope, or spectroscopy.</def> -- <wordforms><wf>Spec`tro*scop"ic*al*ly</wf>, <pos>adv.</pos></wordforms><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spectroscopist</ent><br/
<hw>Spec*tros"co*pist</hw> <pr>(? <or/ ?)</pr>, <pos>n.</pos> <def>One who investigates by means of a spectroscope; one skilled in the use of the spectroscope.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spectroscopy</ent><br/
<hw>Spec*tros"co*py</hw> <pr>(?)</pr>, <pos>n.</pos> <def>the art and science dealing with the use of a spectroscope, and the production and analysis of spectra; the action of using a spectroscope.</def><br/
[<source>Webster 1913 Suppl.</source> <source>+PJC</source>]</p>

<p><ent>Spectrum</ent><br/
<hw>Spec"trum</hw> <pr>(?)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Spectra</plw> <pr>(#)</pr>.</plu> <ety>[L. See <er>Specter</er>.]</ety> <sn>1.</sn> <def>An apparition; a specter.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Opt.)</fld> <sd>(a)</sd> <def>The several colored and other rays of which light is composed, separated by the refraction of a prism or other means, and observed or studied either as spread out on a screen, by direct vision, by photography, or otherwise. See <xex>Illust.</xex> of <er>Light</er>, and <er>Spectroscope</er>.</def> <sd>(b)</sd> <def>A luminous appearance, or an image seen after the eye has been exposed to an intense light or a strongly illuminated object. When the object is colored, the image appears of the complementary color, as a green image seen after viewing a red wafer lying on white paper. Called also <altname>ocular spectrum</altname>.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Absorption spectrum</b></col>, <cd>the spectrum of light which has passed through a medium capable of absorbing a portion of the rays. It is characterized by dark spaces, bands, or lines.</cd> -- <col><b>Chemical spectrum</b></col>, <cd>a spectrum of rays considered solely with reference to their chemical effects, as in photography. These, in the usual photogrophic methods, have their maximum influence at and beyond the violet rays, but are not limited to this region.</cd> -- <col><b>Chromatic spectrum</b></col>, <cd>the visible colored rays of the solar spectrum, exhibiting the seven principal colors in their order, and covering the central and larger portion of the space of the whole spectrum.</cd> -- <col><b>Continous spectrum</b></col>, <cd>a spectrum not broken by bands or lines, but having the colors shaded into each other continously, as that from an incandescent solid or liquid, or a gas under high pressure.</cd> -- <col><b>Diffraction spectrum</b></col>, <cd>a spectrum produced by diffraction, as by a grating.</cd> -- <col><b>Gaseous spectrum</b></col>, <cd>the spectrum of an incandesoent gas or vapor, under moderate, or especially under very low, pressure. It is characterized by bright bands or lines.</cd> -- <col><b>Normal spectrum</b></col>, <cd>a representation of a spectrum arranged upon conventional plan adopted as standard, especially a spectrum in which the colors are spaced proportionally to their wave lengths, as when formed by a diffraction grating.</cd> -- <col><b>Ocular spectrum</b></col>. <cd>See <er>Spectrum</er>, 2 <sd>(b)</sd>, above.</cd> -- <col><b>Prismatic spectrum</b></col>, <cd>a spectrum produced by means of a prism.</cd> -- <col><b>Solar spectrum</b></col>, <cd>the spectrum of solar light, especially as thrown upon a screen in a darkened room. It is characterized by numerous dark lines called <xex>Fraunhofer lines</xex>.</cd> -- <col><b>Spectrum analysis</b></col>, <cd>chemical analysis effected by comparison of the different relative positions and qualities of the fixed lines of spectra produced by flames in which different substances are burned or evaporated, each substance having its own characteristic system of lines.</cd> -- <col><b>Thermal spectrum</b></col>, <cd>a spectrum of rays considered solely with reference to their heating effect, especially of those rays which produce no luminous phenomena.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Specular</ent><br/
<hw>Spec"u*lar</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>specularis</ets> (cf., from the same root, <ets>specula</ets> a lookout, watchtower): cf. F. <ets>sp<eacute/culaire</ets>. See <er>Speculum</er>.]</ety> <sn>1.</sn> <def>Having the qualities of a speculum, or mirror; having a smooth, reflecting surface; <as>as, a <ex>specular</ex> metal; a <ex>specular</ex> surface.</as></def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Med.)</fld> <def>Of or pertaining to a speculum; conducted with the aid of a speculum; <as>as, a <ex>specular</ex> examination</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>Assisting sight, as a lens or the like.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>Thy <qex>specular</qex> orb<br/
Apply to well-dissected kernels; lo!<br/
In each observe the slender threads<br/
Of first-beginning trees.</q> <rj><qau>J. Philips.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>Affording view.</def> <mark>[R.]</mark> <ldquo/Look once more, ere we leave this <xex>specular</xex> mount.<rdquo/  <rj><au>Milton.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Specular iron</b></col>. <fld>(Min.)</fld> <cd>See <er>Hematite</er>.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Speculate</ent><br/
<hw>Spec"u*late</hw> <pr>(?)</pr>, <pos>v. i.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Speculated</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Speculating</conjf>.]</vmorph> <ety>[L. <ets>speculatus</ets>, p. p. of <ets>speculari</ets> to spy out, observe, fr. <ets>specula</ets> a lookout, fr. <ets>specere</ets> to look. See <er>Spy</er>.]</ety> <sn>1.</sn> <def>To consider by turning a subject in the mind, and viewing it in its different aspects and relations; to meditate; to contemplate; to theorize; <as>as, to <ex>speculate</ex> on questions in religion; to <ex>speculate</ex> on political events.</as></def><br/
[<source>1913 Webster</source>]</p>

<p><q>It is remarkable that persons who <qex>speculate</qex> the most boldly often conform with the most pefect quietude to the external regulations of society.</q> <rj><qau>Hawthorne.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Philos.)</fld> <def>To view subjects from certain premises given or assumed, and infer conclusions respecting them <xex>a priori</xex>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <fld>(Com.)</fld> <def>To purchase with the expectation of a contingent advance in value, and a consequent sale at a profit; -- often, in a somewhat depreciative sense, of unsound or hazardous transactions; <as>as, to <ex>speculate</ex> in coffee, in sugar, or in bank stock</as>.</def>
<-- (finance) --><br/
[<source>1913 Webster</source>]</p>

<p><ent>Speculate</ent><br/
<hw>Spec"u*late</hw>, <pos>v. t.</pos> <def>To consider attentively; <as>as, to <ex>speculate</ex> the nature of a thing</as>.</def> <mark>[R.]</mark>  <rj><au>Sir W. Hamilton.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Speculation</ent><br/
<hw>Spec`u*la"tion</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>speculatio</ets> a spying out, observation: cf. F. <ets>sp<eacute/culation</ets>.]</ety> <sn>1.</sn> <def>The act of speculating.</def> Specifically: --<br/
[<source>1913 Webster</source>]</p>

<p><sd>(a)</sd> <def>Examination by the eye; view.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><sd>(b)</sd> <def>Mental view of anything in its various aspects and relations; contemplation; intellectual examination.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Thenceforth to <qex>speculations</qex> high or deep<br/
I turned my thoughts.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sd>(c)</sd> <fld>(Philos.)</fld> <def>The act or process of reasoning <it>a priori</it> from premises given or assumed.</def><br/
[<source>1913 Webster</source>]</p>

<p><sd>(d)</sd> <fld>(Com.)</fld> <def>The act or practice of buying land, goods, shares, etc., in expectation of selling at a higher price, or of selling with the expectation of repurchasing at a lower price; a trading on anticipated fluctuations in price, as distinguished from trading in which the profit expected is the difference between the retail and wholesale prices, or the difference of price in different markets.</def>
<-- buying long is considered speculation only when the time of holding the object is short.  Longer-term trading (<gt/ 1 year) is considered investment. --><br/
[<source>1913 Webster</source>]</p>

<p><q>Sudden fortunes, indeed, are sometimes made in such places, by what is called the trade of <qex>speculation</qex>.</q> <rj><qau>A. Smith.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q><qex>Speculation</qex>, while confined within moderate limits, is the agent for equalizing supply and demand, and rendering the fluctuations of price less sudden and abrupt than they would otherwise be.</q> <rj><qau>F. A. Walker.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sd>(e)</sd> <def>Any business venture in involving unusual risks, with a chance for large profits.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A conclusion to which the mind comes by speculating; mere theory; view; notion; conjecture.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>From him Socrates derived the principles of morality, and most part of his natural <qex>speculations</qex>.</q> <rj><qau>Sir W. Temple.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>To his <qex>speculations</qex> on these subjects he gave the lofty name of the <ldquo/Oracles of Reason.<rdquo/</q> <rj><qau>Macaulay.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>Power of sight.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>Thou hast no <qex>speculation</qex> in those eyes.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>A game at cards in which the players buy from one another trumps or whole hands, upon a chance of getting the highest trump dealt, which entitles the holder to the pool of stakes.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Speculatist</ent><br/
<hw>Spec"u*la*tist</hw> <pr>(sp<ecr/k"<usl/*l<adot/*t<icr/st)</pr>, <pos>n.</pos> <def>One who speculates, or forms theories; a speculator; a theorist.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The very ingenious <qex>speculatist</qex>, Mr. Hume.</q> <rj><qau>V. Knox.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Speculative</ent><br/
<hw>Spec"u*la*tive</hw> <pr>(sp<ecr/k"<usl/*l<adot/*t<icr/v)</pr>, <pos>a.</pos> <ety>[Cf. F. <ets>sp<eacute/culatif</ets>, L. <ets>speculativus</ets>.]</ety> <sn>1.</sn> <def>Given to speculation; contemplative.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The mind of man being by nature <qex>speculative</qex>.</q> <rj><qau>Hooker.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Involving, or formed by, speculation; ideal; theoretical; not established by demonstration.</def>  <rj><au>Cudworth.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>Of or pertaining to vision; also, prying; inquisitive; curious.</def> <mark>[R.]</mark>  <rj><au>Bacon.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>Of or pertaining to speculation in land, goods, shares, etc.; <as>as, a <ex>speculative</ex> dealer or enterprise</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The <qex>speculative</qex> merchant exercises no one regular, established, or well-known branch of business.</q> <rj><qau>A. Smith.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <fld>(Finance)</fld> <def>More risky than typical investments; not investment grade.</def><br/
[<source>PJC</source>]</p>

<p>-- <wordforms><wf>Spec"u*la*tive*ly</wf>, <pos>adv.</pos> -- <wf>Spec"u*la*tive*ness</wf>, <pos>n.</pos></wordforms><br/
[<source>1913 Webster</source>]</p>

<p><ent>Speculator</ent><br/
<hw>Spec"u*la`tor</hw> <pr>(sp<ecr/k"<usl/*l<amac/`t<etil/r)</pr>, <pos>n.</pos> <ety>[L., a spy, explorer, investigator: cf. F. <ets>sp<eacute/culateur</ets>.]</ety> <def>One who speculates. Specifically: <sd>(a)</sd> An observer; a contemplator; hence, a spy; a watcher.</def> <mark>[Obs.]</mark>  <rj><au>Sir T. Browne.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sd>(b)</sd> <def>One who forms theories; a theorist.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>A <qex>speculator</qex> who had dared to affirm that the human soul is by nature mortal.</q> <rj><qau>Macaulay.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sd>(c)</sd> <fld>(Com.)</fld> <def>One who engages in speculation; one who buys and sells goods, land, etc., with the expectation of deriving profit from fluctuations in price.</def><br/
[<source>1913 Webster</source>]</p>

<p><-- p. 1382 --></p>

<p><ent>Speculatorial</ent><br/
<hw>Spec`u*la*to"ri*al</hw> <pr>(sp<ecr/k`<usl/*l<adot/*t<omac/"r<icr/*<ait/l)</pr>, <pos>a.</pos> <def>Speculatory; speculative.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Speculatory</ent><br/
<hw>Spec"u*la*to*ry</hw> <pr>(sp<ecr/k"<usl/*l<adot/*t<osl/*r<ycr/)</pr>, <pos>a.</pos> <ety>[L. <ets>speculatorius</ets> belonging to spies or scouts.]</ety> <sn>1.</sn> <def>Intended or adapted for viewing or espying; having oversight.</def>  <rj><au>T. Warton.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Exercising speculation; speculative.</def>  <rj><au>T. Carew.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Speculist</ent><br/
<hw>Spec"u*list</hw> <pr>(?)</pr>, <pos>n.</pos> <def>One who observes or considers; an observer.</def> <mark>[R.]</mark>  <rj><au>Goldsmith.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Speculum</ent><br/
<hw>Spec"u*lum</hw> <pr>(?)</pr>, <pos>n.</pos>; <plu><it>pl.</it> L. <plw>Specula</plw> <pr>(#)</pr>, E. <plw>Speculum</plw> <pr>(#)</pr>.</plu> <ety>[L., fr. <ets>specere</ets> to look, behold. See <er>Spy</er>.]</ety> <sn>1.</sn> <def>A mirror, or looking-glass; especially, a metal mirror, as in Greek and Roman archaeology.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A reflector of polished metal, especially one used in reflecting telescopes. See <cref>Speculum metal</cref>, below.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <fld>(Surg.)</fld> <def>An instrument for dilating certain passages of the body, and throwing light within them, thus facilitating examination or surgical operations.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <fld>(Zool.)</fld> <def>A bright and lustrous patch of color found on the wings of ducks and some other birds. It is usually situated on the distal portions of the secondary quills, and is much more brilliant in the adult male than in the female.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Speculum metal</b></col>, <cd>a hard, brittle alloy used for making the reflectors of telescopes and other instruments, usually consisting of copper and tin in various proportions, one of the best being that in which there are 126.4 parts of copper to 58.9 parts of tin, with sometimes a small proportion of arsenic, antimony, or zinc added to improve the whiteness.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sped</ent><br/
<hw>Sped</hw> <pr>(?)</pr>, <def><pos>imp. & p. p.</pos> of <er>Speed</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Speece</ent><br/
<hw>Speece</hw> <pr>(?)</pr>, <pos>n.</pos> <def>Species; sort.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Speech</ent><br/
<hw>Speech</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[OE. <ets>speche</ets>, AS. <ets>sp<?/c</ets>, <ets>spr<?/</ets>, fr. <ets>specan</ets>, <ets>sprecan</ets>, to speak; akin to D. <ets>spraak</ets> speech, OHG. <ets>spr<amac/hha</ets>, G. <ets>sprache</ets>, Sw. <ets>spr<?/k</ets>, Dan. <ets>sprog</ets>. See <er>Speak</er>.]</ety> <sn>1.</sn> <def>The faculty of uttering articulate sounds or words; the faculty of expressing thoughts by words or articulate sounds; the power of speaking.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>There is none comparable to the variety of instructive expressions by <qex>speech</qex>, wherewith man alone is endowed for the communication of his thoughts.</q> <rj><qau>Holder.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>he act of speaking; that which is spoken; words, as expressing ideas; language; conversation.</def><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ Speech is voice modulated by the throat, tongue, lips, etc., the modulation being accomplished by changing the form of the cavity of the mouth and nose through the action of muscles which move their walls.</note><br/
[<source>1913 Webster</source>]</p>

<p><q>O goode God! how gentle and how kind<br/
Ye seemed by your <qex>speech</qex> and your visage<br/
The day that maked was our marriage.</q> <rj><qau>Chaucer.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>The acts of God . . . to human ears<br/
Can nort without process of <qex>speech</qex> be told.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>A particular language, as distinct from others; a tongue; a dialect.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>People of a strange <qex>speech</qex> and of an hard language.</q> <rj><qau>Ezek. iii. 6.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>Talk; mention; common saying.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The duke . . . did of me demand<br/
What was the <qex>speech</qex> among the Londoners<br/
Concerning the French journey.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <def>formal discourse in public; oration; harangue.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The constant design of these orators, in all their <qex>speeches</qex>, was to drive some one particular point.</q> <rj><qau>Swift.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>6.</sn> <def>ny declaration of thoughts.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>I. with leave of <qex>speech</qex> implored, . . . replied.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><syn><b>Syn.</b> Harangue; language; address; oration. See <er>Harangue</er>, and <er>Language</er>.</syn><br/
[<source>1913 Webster</source>]</p>

<p><ent>Speech</ent><br/
<hw>Speech</hw>, <pos>v. i. & t.</pos> <def>To make a speech; to harangue.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Speechful</ent><br/
<hw>Speech"ful</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Full of speech or words; voluble; loquacious.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Speechification</ent><br/
<hw>Speech`i*fi*ca"tion</hw> <pr>(?)</pr>, <pos>n.</pos><ety>[See <er>Spechify</er>.]</ety> <def>The act of speechifying.</def> <mark>[Used humorously or in contempt.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Speechifier</ent><br/
<hw>Speech"i*fi`er</hw> <pr>(?)</pr>, <pos>n.</pos> <def>One who makes a speech or speeches; an orator; a declaimer.</def> <mark>[Used humorously or in contempt.]</mark>  <rj><au>G. Eliot.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Speechify</ent><br/
<hw>Speech"i*fy</hw> <pr>(?)</pr>, <pos>v. i.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Speechified</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Speechifying</conjf> <pr>(?)</pr>.]</vmorph> <ety>[<ets>Speech</ets> + <ets>-fy</ets>.]</ety> <def>To make a speech; to harangue.</def> <mark>[Used derisively or humorously.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Speechifying</ent><br/
<hw>Speech"i*fy`ing</hw>, <pos>n.</pos> <def>The act of making a speech or speeches.</def> <mark>[Used derisively or humorously.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>The dinner and <qex>speechifying</qex> . . . at the opening of the annual season for the buckhounds.</q> <rj><qau>M. Arnold.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Speeching</ent><br/
<hw>Speech"ing</hw>, <pos>n.</pos> <def>The act of making a speech.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Speechless</ent><br/
<hw>Speech"less</hw>, <pos>a.</pos> <sn>1.</sn> <def>Destitute or deprived of the faculty of speech.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Not speaking for a time; dumb; mute; silent.</def><br/
[<source>1913 Webster</source>]</p>

<p><q><qex>Speechless</qex> with wonder, and half dead with fear.</q> <rj><qau>Addison.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p>-- <wordforms><wf>Speech"less*ly</wf>, <pos>adv.</pos> -- <wf>Speech"less*ness</wf>, <pos>n.</pos></wordforms><br/
[<source>1913 Webster</source>]</p>

<p><ent>Speechmaker</ent><br/
<hw>Speech"mak`er</hw> <pr>(?)</pr>, <pos>n.</pos> <def>One who makes speeches; one accustomed to speak in a public assembly.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Speed</ent><br/
<hw>Speed</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[AS. <ets>sp<?/d</ets> success, swiftness, from <ets>sp<?/wan</ets> to succeed; akin to D. <ets>spoed</ets>d, OHG. <ets>spuot</ets> success, <ets>spuot</ets> to succees, Skr. <ets>sph<amac/</ets> to increase, grow fat. <root/170<it>b.</it>]</ety> <sn>1.</sn> <def>Prosperity in an undertaking; favorable issue; success.</def> <ldquo/For common <xex>speed</xex>.<rdquo/  <rj><au>Chaucer.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>O Lord God of my master Abraham, I pray thee, send me good <qex>speed</qex> this day.</q> <rj><qau>Gen. xxiv. 12.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>The act or state of moving swiftly; swiftness; velocity; rapidly; rate of motion; dispatch; <as>as, the <ex>speed</ex> a horse or a vessel</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q><qex>Speed</qex>, to describe whose swiftness number fails.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ In kinematics, <xex>speed</xex>is sometimes used to denote the amount of velocity without regard to direction of motion, while <xex>velocity</xex> is not regarded as known unless both the direction and the amount are known.</note><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>One who, or that which, causes or promotes speed or success.</def> <mark>[Obs.]</mark> <ldquo/Hercules be thy <xex>speed</xex>!<rdquo/  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>God speed</b></col>, <cd>Good speed; prosperity. See <er>Godspeed</er>.</cd> -- <mcol><col><b>Speed gauge</b></col>, <col><b>Speed indicator</b></col>, <and/ <col><b>Speed recorder</b></col></mcol> <fld>(Mach.)</fld>, <cd>devices for indicating or recording the rate of a body's motion, as the number of revolutions of a shaft in a given time.</cd> -- <col><b>Speed lathe</b></col> <fld>(Mach.)</fld>, <cd>a power lathe with a rapidly revolving spindle, for turning small objects, for polishing, etc.; a hand lathe.</cd> -- <col><b>Speed pulley</b></col>, <cd>a cone pulley with steps.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><syn><b>Syn.</b> -- Haste; swiftness; celerity; quickness; dispatch; expedition; hurry; acceleration. See <er>Haste</er>.</syn><br/
[<source>1913 Webster</source>]</p>

<p><ent>Speed</ent><br/
<hw>Speed</hw> <pr>(sp<emac/d)</pr>, <pos>v. i.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Sped</conjf> <pr>(sp<ecr/d)</pr>, <conjf>Speeded</conjf>; <pos>p. pr. & vb. n.</pos> <conjf>Speeding</conjf>.]</vmorph> <ety>[AS. <ets>sp<emac/dan</ets>, fr. <ets>sp<emac/d</ets>, n.; akin to D. <ets>spoeden</ets>, G. sich <ets>sputen</ets>.  See <er>Speed</er>, <pos>n.</pos>]</ety> <sn>1.</sn> <def>To go; to fare.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>To warn him now he is too farre <qex>sped</qex>.</q> <rj><qau>Remedy of Love.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To experience in going; to have any condition, good or ill; to fare.</def>  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Ships heretofore in seas like fishes <qex>sped</qex>;<br/
 The mightiest still upon the smallest fed.</q> <rj><qau>Waller.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>To fare well; to have success; to prosper.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Save London, and send true lawyers their meed!<br/
For whoso wants money with them shall not <qex>speed</qex>!</q> <rj><qau>Lydgate.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>I told ye then he should prevail, and <qex>speed</qex><br/
On his bad errand.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>To make haste; to move with celerity.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>I have <qex>speeded</qex> hither with the very extremest inch of possibility.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <def>To be expedient.</def> <mark>[Obs.]</mark>  <rj><au>Wyclif (2 Cor. xii. 1.)</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Speed</ent><br/
<hw>Speed</hw>, <pos>v. t.</pos> <sn>1.</sn> <def>To cause to be successful, or to prosper; hence, to aid; to favor.</def> <ldquo/Fortune <xex>speed</xex> us!<rdquo/  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>With rising gales that <qex>speed</qex> their happy flight.</q> <rj><qau>Dryden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To cause to make haste; to dispatch with celerity; to drive at full speed; hence, to hasten; to hurry.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>He <qex>sped</qex> him thence home to his habitation.</q> <rj><qau>Fairfax.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>To hasten to a conclusion; to expedite.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Judicial acts . . . are <qex>sped</qex> in open court at the instance of one or both of the parties.</q> <rj><qau>Ayliffe.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>To hurry to destruction; to put an end to; to ruin; to undo.</def> <ldquo/<xex>Sped</xex> with spavins.<rdquo/  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>A dire dilemma! either way I 'm <qex>sped</qex>.<br/
If foes, they write, if friends, they read, me dead.</q> <rj><qau>Pope.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <def>To wish success or god fortune to, in any undertaking, especially in setting out upon a journey.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Welcome the coming, <qex>speed</qex> the parting guest.</q> <rj><qau>Pope.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><mcol><col><b>God speed you</b></col>, <col><b>them</b></col></mcol>, <cd>etc., may God speed you; or, may you have good speed.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><syn><b>Syn.</b> -- To dispatch; hasten; expedite; accelerate; hurry.</syn><br/
[<source>1913 Webster</source>]</p>

<p><ent>Speed counter</ent><br/
<hw>Speed counter</hw>. <fld>(Mach.)</fld> <def>A device for automatically counting the revolutions or pulsations of an engine or other machine; -- called also simply <altname>counter</altname>.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Speeder</ent><br/
<hw>Speed"er</hw> <pr>(?)</pr>, <pos>n.</pos> <sn>1.</sn> <def>One who, or that which, speeds.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Spinning)</fld> <def>A machine for drawing and twisting slivers to form rovings.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Speedful</ent><br/
<hw>Speed"ful</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Full of speed (in any sense).</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Speedfully</ent><br/
<hw>Speed"ful*ly</hw>, <pos>adv.</pos> <def>In a speedful manner.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Speedily</ent><br/
<hw>Speed"i*ly</hw> <pr>(?)</pr>, <pos>adv.</pos> <def>In a speedy manner.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Speediness</ent><br/
<hw>Speed"i*ness</hw>, <pos>n.</pos> <def>The quality or state of being speedy.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Speedless</ent><br/
<hw>Speed"less</hw>, <pos>a.</pos> <def>Being without speed.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Speedwell</ent><br/
<hw>Speed"well</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Bot.)</fld> <def>Any plant of the genus <gen>Veronica</gen>, mostly low herbs with pale blue corollas, which quickly fall off.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Speedy</ent><br/
<hw>Speed"y</hw> <pr>(-<ycr/)</pr>, <pos>a.</pos> <amorph>[<pos>Compar.</pos> <adjf>Speedier</adjf> <pr>(?)</pr>; <pos>superl.</pos> <adjf>Speediest</adjf>.]</amorph> <ety>[AS. <ets>sp<emac/dyg</ets>.]</ety> <def>Not dilatory or slow; quick; swift; nimble; hasty; rapid in motion or performance; <as>as, a <ex>speedy</ex> flight; on <ex>speedy</ex> foot.</as></def><br/
[<source>1913 Webster</source>]</p>

<p><q>I will wish her <qex>speedy</qex> strength.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Darts, which not the good could shun,<br/
The <qex>speedy</qex> ould outfly.</q> <rj><qau>Dryden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Speer</ent><br/
<hw>Speer</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A sphere.</def> <mark>[Obs.]</mark>  <rj><au>Chaucer.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Speer</ent><br/
<hw>Speer</hw>, <pos>v. t.</pos> <def>To ask.</def> <mark>[Scot.]</mark> <see>See <er>Spere</er>.</see><br/
[<source>1913 Webster</source>]</p>

<p><ent>Speet</ent><br/
<hw>Speet</hw> <pr>(?)</pr>, <pos>v. t.</pos> <ety>[Cf. D. <ets>speten</ets>. See <er>Spit</er> an iron prong.]</ety> <def>To stab.</def> <mark>[Obs.]</mark>  <rj><au>Gammer Gurton's Needle.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Speight</ent><br/
<hw>Speight</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[G. <ets>specht</ets>, probably akin to L. <ets>picus</ets>: cf. D. <ets>specht</ets>. <root/169. See <er>Pie</er> a magpie.]</ety> <fld>(Zool.)</fld> <def>A woodpecker; -- called also <altname>specht</altname>, <altname>spekt</altname>, <altname>spight</altname>.</def> <mark>[Obs. or Prov. Eng.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Speir</ent><br/
<hw>Speir</hw> <pr>(?)</pr>, <pos>v. i.</pos> <def>To ask. See <er>Spere</er>.</def>  <rj><au>Sir W. Scott.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Speiskobalt</ent><br/
<hw>Speis`ko"balt</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[G.]</ety> <def>Smaltite.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Speiss</ent><br/
<hw>Speiss</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Cf. G. <ets>speise</ets> food, mixed metal for bells, etc.]</ety> <sn>1.</sn> <fld>(Metal.)</fld> <def>A regulus consisting essentially of nickel, obtained as a residue in fusing cobalt and nickel ores with silica and sodium carbonate to make smalt.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Metal.)</fld> <def>Impure metallic arsenides, principally of iron, produced in copper and lead smelting.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Spekboom</ent><br/
<hw>Spek"boom</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[D., lit. fat tree.]</ety> <fld>(Bot.)</fld> <def>The purslane tree of South Africa, -- said to be the favorite food of elephants.</def>  <rj><au>Balfour (Cyc. of India).</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Speke</ent><br/
<hw>Speke</hw> <pr>(?)</pr>, <pos>v. i. & t.</pos> <def>To speak.</def> <mark>[Obs.]</mark>  <rj><au>Chaucer.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spekehouse</ent><br/
<hw>Speke"house`</hw> <pr>(?)</pr>, <pos>n.</pos> <def>The parlor or reception room of a convent.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spelding</ent><br/
<hw>Spel"ding</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Scot. <ets>speld</ets> to spread out, <ets>spelder</ets> to split. spread open; cf. G. <ets>spalten</ets> split.]</ety> <def>A haddock or other small fish split open and dried in the sun; -- called also <altname>speldron</altname>.</def> <mark>[Scot.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spelicans</ent><br/
<hw>Spel"i*cans</hw> <pr>(?)</pr>, <pos>n. pl.</pos> <def>See <er>Spilikin</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spelk</ent><br/
<hw>Spelk</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[AS. <ets>spelc</ets>, <ets>spilc</ets>, a little rod by which a thing is kept straight, a splint for binding up broken bones, akin to Icel. <ets>spelkur</ets>, pl., a splint.  Cf. <er>Spell</er> a splinter.]</ety> <def>A small stick or rod used as a spike in thatching; a splinter.</def> <mark>[Prov. Eng.]</mark>  <rj><au>Grose.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spell</ent><br/
<hw>Spell</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[OE. <ets>speld</ets>, AS. <ets>speld</ets> a spill to light a candle with; akin to D. <ets>speld</ets> a pin, OD. <ets>spelle</ets>, G. <ets>spalten</ets> to split, OHG. <ets>spaltan</ets>, MHG. <ets>spelte</ets> a splinter, Icel. <ets>spjald</ets> a square tablet, Goth. <ets>spilda</ets> a writing tablet.  Cf. <er>Spill</er>splinter, roll of paper, <er>Spell</er> to tell the letters of.]</ety> <def>A spelk, or splinter.</def> <mark>[Obs.]</mark>  <rj><au>Holland.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spell</ent><br/
<hw>Spell</hw>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Spelled</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Spelling</conjf>.]</vmorph> <ety>[AS. <ets>spelian</ets> to supply another's place.]</ety> <def>To supply the place of for a time; to take the turn of, at work; to relieve; <as>as, to <ex>spell</ex> the helmsman</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spell</ent><br/
<hw>Spell</hw>, <pos>n.</pos> <sn>1.</sn> <def>The relief of one person by another in any piece of work or watching; also, a turn at work which is carried on by one person or gang relieving another; <as>as, a <ex>spell</ex> at the pumps; a <ex>spell</ex> at the masthead</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>A <qex>spell</qex> at the wheel is called a trick.</q> <rj><qau>Ham. Nav. Encyc.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>The time during which one person or gang works until relieved; hence, any relatively short period of time, whether a few hours, days, or weeks.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Nothing new has happened in this quarter, except the setting in of a severe <qex>spell</qex> of cold weather.</q> <rj><qau>Washington.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>One of two or more persons or gangs who work by spells.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>Their toil is so extreme that they can not endure it above four hours in a day, but are succeeded by <qex>spells</qex>.</q> <rj><qau>Garew.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>A gratuitous helping forward of another's work; <as>as, a logging <ex>spell</ex></as>.</def> <mark>[Local, U.S.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spell</ent><br/
<hw>Spell</hw> <pr>(?)</pr>, <pos>n.</pos><ety>[AS. <ets>spell</ets> a saying, tale, speech; akin to OS. & OHG. <ets>spel</ets>, Icel. <ets>spjall</ets>,Goth. <ets>spill</ets>.  Cf. <er>Gospel</er>, <er>Spell</er> to tell the letters of.]</ety> <sn>1.</sn> <def>A story; a tale.</def> <mark>[Obs.]</mark> <ldquo/Hearken to my <xex>spell</xex>.<rdquo/  <rj><au>Chaucer.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A stanza, verse, or phrase supposed to be endowed with magical power; an incantation; hence, any charm.</def><br/
[<source>1913 Webster</source>]</p>

<p><q> Start not; her actions shall be holy as<br/
You hear my <qex>spell</qex> is lawful.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spell</ent><br/
<hw>Spell</hw>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Spelled</conjf> <pr>(<?/)</pr> or <conjf>Spelt</conjf> <pr>(<?/)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Spelling</conjf>.]</vmorph> <ety>[OE. <ets>spellen</ets>, <ets>spellien</ets>, tell, relate, AS. <ets>spellian</ets>, fr. <ets>spell</ets> a saying, tale; akin to MHG. <ets>spellen</ets> to relate, Goth. <ets>spill<?/n</ets>.e <er>Spell</er> a tale. In sense 4 and those following, OE. <ets>spellen</ets>, perhaps originally a different word, and from or influenced by <ets>spell</ets> a splinter, from the use of a piece of wood to point to the letters in schools: cf. D. <ets>spellen</ets> to spell.  Cf. <er>Spell</er> splinter.]</ety> <sn>1.</sn> <def>To tell; to relate; to teach.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>Might I that legend find,<br/
By fairies <qex>spelt</qex> in mystic rhymes.</q> <rj><qau>T. Warton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To put under the influence of a spell; to affect by a spell; to bewitch; to fascinate; to charm.</def> <ldquo/<xex>Spelled</xex> with words of power.<rdquo/  <rj><au>Dryden.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>He was much <qex>spelled</qex> with Eleanor Talbot.</q> <rj><qau>Sir G. Buck.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>To constitute; to measure.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>The Saxon heptarchy, when seven kings put together did <qex>spell</qex> but one in effect.</q> <rj><qau>Fuller.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>To tell or name in their proper order letters of, as a word; to write or print in order the letters of, esp. the proper letters; to form, as words, by correct orthography.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The word <ldquo/satire<rdquo/ ought to be <qex>spelled</qex> with <it>i</it>, and not with <it>y</it>.</q> <rj><qau>Dryden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <def>To discover by characters or marks; to read with difficulty; -- usually with <xex>out</xex>; <as>as, to <ex>spell</ex> out the sense of an author; to <ex>spell</ex> out a verse in the Bible.</as></def><br/
[<source>1913 Webster</source>]</p>

<p><q>To <qex>spell</qex> out a God in the works of creation.</q> <rj><qau>South.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>To sit <qex>spelling</qex> and observing divine justice upon every accident.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spell</ent><br/
<hw>Spell</hw>, <pos>v. i.</pos> <sn>1.</sn> <def>To form words with letters, esp. with the proper letters, either orally or in writing.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>When what small knowledge was, in them did dwell,<br/
And he a god, who could but read or <qex>spell</qex>.</q> <rj><qau>Dryden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To study by noting characters; to gain knowledge or learn the meaning of anything, by study.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>Where I may sit and rightly <qex>spell</qex><br/
Of every star that heaven doth shew,<br/
And every herb that sips the dew.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spellable</ent><br/
<hw>Spell"a*ble</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Capable of being spelt.</def>  <rj><au>Carlyle.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spellbind</ent><br/
<hw>Spell"bind`</hw> <pr>(?)</pr>, <pos>v. t.</pos>  <vmorph>[<pos>imp. & p. p.</pos> <conjf>Spellbound</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Spellbinding</conjf>.]</vmorph> <def>To bind or hold by, or as if by, a spell or charm; to entrance or fascinate, esp. by eloquence of speech, as in a political campaign.</def> -- <wordforms><wf>Spell"bind`er</wf> <pr>(#)</pr>, <pos>n.</pos></wordforms><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Spellbound</ent><br/
<hw>Spell"bound`</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Bound by, or as by, a spell.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Speller</ent><br/
<hw>Spell"er</hw> <pr>(?)</pr>, <pos>n.</pos> <sn>1.</sn> <def>One who spells.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A spelling book.</def> <mark>[U. S.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spellful</ent><br/
<hw>Spell"ful</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Abounding in spells, or charms.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Here, while his eyes the learned leaves peruse,<br/
Each <qex>spellful</qex> mystery explained he views.</q> <rj><qau>Hoole.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spelling</ent><br/
<hw>Spell"ing</hw>, <pos>n.</pos> <def>The act of one who spells; formation of words by letters; orthography.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spelling</ent><br/
<hw>Spell"ing</hw>, <pos>a.</pos> <def>Of or pertaining to spelling.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Spelling bee</b></col>, <cd>a spelling match.</cd> <mark>[U.S.]</mark> -- <col><b>Spelling book</b></col>, <cd>a book with exercises for teaching children to spell; a speller.</cd> -- <col><b>Spelling match</b></col>, <cd>a contest of skill in spelling words, between two or more persons.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spellken</ent><br/
<hw>Spell"ken</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A theater.</def> <mark>[Slang]</mark>  <rj><au>Byron.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spellwork</ent><br/
<hw>Spell"work`</hw> <pr>(?)</pr>, <pos>n.</pos> <def>Power or effect of magic; that which is wrought by magic; enchantment.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Like those Peri isles of light<br/
That hang by <qex>spellwork</qex> in the air.</q> <rj><qau>Moore.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spelt</ent><br/
<hw>Spelt</hw> <pr>(?)</pr>, <def><pos>imp. & p. p.</pos> of <er>Spell</er>. Spelled.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spelt</ent><br/
<hw>Spelt</hw>, <pos>n.</pos> <ety>[AS. <ets>spelt</ets>, fr. L. <ets>spelta</ets>.]</ety> <fld>(Bot.)</fld> <def>A species of grain (<spn>Triticum Spelta</spn>) much cultivated for food in Germany and Switzerland; -- called also <altname>German wheat</altname>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spelt</ent><br/
<hw>Spelt</hw>, <pos>n.</pos> <ety>[See <er>Spalt</er>.]</ety> <fld>(Metal.)</fld> <def>Spelter.</def> <mark>[Colloq.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spelt</ent><br/
<hw>Spelt</hw>, <pos>v. t. & i.</pos> <ety>[See <er>Spell</er> a splinter.]</ety> <def>To split; to break; to spalt.</def> <mark>[Obs.]</mark>  <rj><au>Mortimer.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spelter</ent><br/
<hw>Spel"ter</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Cf. LG. <ets>spialter</ets>, G. & D. <ets>spiauter</ets>.  Cf. <er>Pewter</er>.]</ety> <fld>(Metal.)</fld> <def>Zinc; -- especially so called in commerce and arts.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spelunc</ent><br/
<hw>Spe*lunc"</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>spelunca</ets> cave.]</ety> <def>A cavern; a cave.</def> <mark>[Obs.]</mark>  <rj><au>Piers Plowman.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spence</ent><br/
<hw>Spence</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[OF. <ets>despense</ets>, F. <ets>d<eacute/pense</ets>, buffet, buttery, fr. OF. <ets>despendre</ets> to spend, distribute, L. <ets>dispendere</ets>, <ets>dispensum</ets>. See <er>Dispense</er>, <er>Spend</er>.]</ety> <sn>1.</sn> <def>A place where provisions are kept; a buttery; a larder; a pantry.</def>
<-- <mark>Chiefly Brit. dial. [MW10]</mark> --><br/
[<source>1913 Webster</source>]</p>

<p><q>In . . . his <qex>spence</qex>, or <ldquo/pantry<rdquo/ were hung the carcasses of a sheep or ewe, and two cows lately slaughtered.</q> <rj><qau>Sir W. Scott.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Bluff Harry broke into the <qex>spence</qex>,<br/
And turned the cowls adrift.</q> <rj><qau>Tennyson.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>The inner apartment of a country house; also, the place where the family sit and eat.</def> <mark>[Scot.]</mark>  <rj><au>Jamieson.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spencer</ent><br/
<hw>Spen"cer</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[OF. <ets>despensier</ets>. See <er>Spence</er>, and cf. <er>Dispenser</er>.]</ety> <def>One who has the care of the spence, or buttery.</def> <mark>[Obs.]</mark>  <rj><au>Promptorium Parvulorum.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spencer</ent><br/
<hw>Spen"cer</hw>, <pos>n.</pos> <ety>[From the third Earl <ets>Spencer</ets>, who first wore it, or brought it into fashion.]</ety> <def>A short jacket worn by men and by women.</def>  <rj><au>Ld. Lutton.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spencer</ent><br/
<hw>Spen"cer</hw>, <pos>n.</pos> <fld>(Naut.)</fld> <def>A fore-and-aft sail, abaft the foremast or the mainmast, hoisted upon a small supplementary mast and set with a gaff and no boom; a trysail carried at the foremast or mainmast; -- named after its inventor, Knight <xex>Spencer</xex>, of England [1802].</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Spencer mast</b></col>, <cd>a small mast just abaft the foremast or mainmast, for hoisting the spencer.</cd> <au>R. H. Dana, Jr.</au></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spend</ent><br/
<hw>Spend</hw> <pr>(?)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Spent</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Spending</conjf>.]</vmorph> <ety>[AS. <ets>spendan</ets> (in comp.), fr. L. <ets>expendere</ets> or <ets>dispendere</ets> to weigh out, to expend, dispense. See <er>Pendant</er>, and cf. <er>Dispend</er>, <er>Expend</er>, <er>Spence</er>, <er>Spencer</er>.]</ety> <sn>1.</sn> <def>To weigh or lay out; to dispose of; to part with; <as>as, to <ex>spend</ex> money for clothing</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q><qex>Spend</qex> thou that in the town.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Wherefore do ye <qex>spend</qex> money for that which is not bread?</q> <rj><qau>Isa. lv. 2.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><-- p. 1383 --></p>

<p><sn>2.</sn> <def>To bestow; to employ; -- often with <xex>on</xex> or <xex>upon</xex>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>I . . . am never loath<br/
To <qex>spend</qex> my judgment.</q> <rj><qau>Herbert.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>To consume; to waste; to squander; to exhaust; <as>as, to <ex>spend</ex> an estate in gaming or other vices</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>To pass, as time; to suffer to pass away; <as>as, to <ex>spend</ex> a day idly; to <ex>spend</ex> winter abroad</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>We <qex>spend</qex> our years as a tale that is told.</q> <rj><qau>Ps. xc. 9.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <def>To exhaust of force or strength; to waste; to wear away; <as>as, the violence of the waves was <ex>spent</ex></as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Their bodies <qex>spent</qex> with long labor and thirst.</q> <rj><qau>Knolles.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spend</ent><br/
<hw>Spend</hw> <pr>(?)</pr>, <pos>v. i.</pos> <sn>1.</sn> <def>To expend money or any other possession; to consume, use, waste, or part with, anything; <as>as, he who gets easily <ex>spends</ex> freely</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>He <qex>spends</qex> as a person who knows that he must come to a reckoning.</q> <rj><qau>South.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To waste or wear away; to be consumed; to lose force or strength; to vanish; <as>as, energy <ex>spends</ex> in the using of it</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The sound <qex>spendeth</qex> and is dissipated in the open air.</q> <rj><qau>Bacon.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>To be diffused; to spread.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The vines that they use for wine are so often cut, that their sap <qex>spendeth</qex> into the grapes.</q> <rj><qau>Bacon.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <fld>(Mining)</fld> <def>To break ground; to continue working.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spender</ent><br/
<hw>Spen"der</hw> <pr>(?)</pr>, <pos>n.</pos> <def>One who spends; esp., one who spends lavishly; a prodigal; a spendthrift.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spending</ent><br/
<hw>Spend"ing</hw>, <pos>n.</pos> <def>The act of expending; expenditure.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Spending money</b></col>, <cd>money set apart for extra (not necessary) personal expenses; pocket money.</cd> <mark>[Colloq.]</mark></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spendthrift</ent><br/
<hw>Spend"thrift`</hw> <pr>(?)</pr>, <pos>n.</pos> <def>One who spends money profusely or improvidently; a prodigal; one who lavishes or wastes his estate. Also used figuratively.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>A woman who was a generous <qex>spendthrift</qex> of life.</q> <rj><qau>Mrs. R. H. Davis.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spendthrift</ent><br/
<hw>Spend"thrift</hw>, <pos>a.</pos> <def>Prodigal; extravagant; wasteful.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spendthrifty</ent><br/
<hw>Spend"thrift`y</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Spendthrift; prodigal.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spenserian</ent><br/
<hw>Spen*se"ri*an</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Of or pertaining to the English poet <ets>Spenser</ets>; -- specifically applied to the stanza used in his poem <ldquo/The Fa<eum/rie Queene.<rdquo/</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spent</ent><br/
<hw>Spent</hw> <pr>(sp<ecr/nt)</pr>, <pos>a.</pos> <sn>1.</sn> <def>Exhausted; worn out; having lost energy or motive force.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Now thou seest me<br/
<qex>Spent</qex>, overpowered, despairing of success.</q> <rj><qau>Addison.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Heaps of <qex>spent</qex> arrows fall and strew the ground.</q> <rj><qau>Dryden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Zool.)</fld> <def>Exhausted of spawn or sperm; -- said especially of fishes.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Spent ball</b></col>, <cd>a ball shot from a firearm, which reaches an object without having sufficient force to penetrate it.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sperre</ent><br/
<ent>Sper</ent><br/
<mhw>{ <hw>Sper</hw> <pr>(?)</pr>, <hw>Sperre</hw>  }</mhw>, <pos>v. t.</pos> <ety>[See <er>Spar</er> bar.]</ety> <def>To shut in; to support; to inclose; to fasten.</def> <mark>[Obs.]</mark> <ldquo/To <xex>sperre</xex> the gate.<rdquo/  <rj><au>Spenser.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sperable</ent><br/
<hw>Spe"ra*ble</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>spearabilis</ets>, fr. <ets>sperare</ets> to hope.]</ety> <def>Within the range of hpe; proper to be hoped for.</def> <mark>[Obs.]</mark>  <rj><au>Bacon.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sperable</ent><br/
<hw>Sper"a*ble</hw> <pr>(?)</pr>, <pos>n.</pos> <def>See <er>Sperable</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sperage</ent><br/
<hw>Sper"age</hw> <pr>(?)</pr>, <pos>n.</pos> <def>Asperagus.</def> <mark>[Obs.]</mark>  <rj><au>Sylvester.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sperate</ent><br/
<hw>Spe"rate</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>speratus</ets>, p. p. of <ets>sperare</ets> to hope.]</ety> <def>Hoped for, or to be hoped for.</def> <mark>[R.]</mark>  <rj><au>Bouvier.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spere</ent><br/
<hw>Spere</hw> <pr>(?)</pr>, <pos>v. i.</pos> <ety>[AS. <ets>spyrian</ets> to inquire, properly, to follow the track; akin to D. <ets>speuren</ets>, G. <ets>sp<uum/ren</ets>, Icel. <ets>spyrja</ets>. <root/171. See Spoor.]</ety> <def>To search; to pry; to ask; to inquire.</def> <mark>[Prov. Eng. & Scot.]</mark> <altsp>[Written also <asp>speer</asp>, <asp>speir</asp>.]</altsp>  <rj><au>Jamieson.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spere</ent><br/
<hw>Spere</hw>, <pos>n.</pos> <ety>[See <er>Sphere</er>.]</ety> <def>A sphere.</def> <mark>[Obs.]</mark>  <rj><au>Chaucer.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sperge</ent><br/
<hw>Sperge</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Distilling)</fld> <def>A charge of wash for the still.</def>  <rj><au>Knight.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sperling</ent><br/
<hw>Sper"ling</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[See <er>Sparling</er>.]</ety> <fld>(Zool.)</fld> <sd>(a)</sd> <def>A smelt; a sparling.</def> <mark>[Prov. Eng.]</mark> <sd>(b)</sd> <def>A young herring.</def> <mark>[Local, U.S.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sperm</ent><br/
<hw>Sperm</hw> <pr>(?)</pr>, <pos>n.</pos><ety>[F. <ets>sperme</ets>, L. <ets>sperma</ets>, Gr. <?/<?/<?/, <?/<?/<?/, from <?/<?/<?/<?/ to sow.  Cf. <er>Spore</er>.]</ety> <fld>(Physiol.)</fld> <def>The male fecundating fluid; semen. See <er>Semen</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Sperm cell</b></col> <fld>(Physiol.)</fld>, <cd>one of the cells from which the spermatozoids are developed.</cd> -- <col><b>Sperm morula</b></col>. <fld>(Biol.)</fld> <cd>Same as <er>Spermosphere</er>.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sperm</ent><br/
<hw>Sperm</hw>, <pos>n.</pos><ety>[Contr. fr. <ets>spermaceti</ets>.]</ety> <def>Spermaceti.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Sperm oil</b></col>, <cd>a fatty oil found as a liquid, with spermaceti, in the head cavities of the sperm whale.</cd> -- <col><b>Sperm whale</b></col>. <fld>(Zool.)</fld> <cd>See in the Vocabulary.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spermaceti</ent><br/
<hw>Sper`ma*ce"ti</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>sperma</ets> sperm + <ets>cetus</ets>,gen. <ets>ceti</ets>, any large sea animal, a whale, Gr. <?/<?/<?/. See <er>Sperm</er>, <er>Cetaceous</er>.]</ety> <def>A white waxy substance obtained from cavities in the head of the sperm whale, and used making candles, oilments, cosmetics, etc. It consists essentially of ethereal salts of palmitic acid with ethal and other hydrocarbon bases.  The substance of spermaceti after the removal of certain impurities is sometimes called <xex>cetin</xex>.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Spermaceti whale</b></col> <fld>(Zool.)</fld>, <cd>the sperm whale.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spermalist</ent><br/
<hw>Sper"mal*ist</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Biol.)</fld> <def>See <er>Spermist</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spermaphore</ent><br/
<hw>Sper"ma*phore</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Gr. <?/<?/<?/<?/ sperm + <grk>fe`rein</grk> to bear.]</ety> <fld>(Bot.)</fld> <def>That part of the ovary from which the ovules arise; the placenta.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spermary</ent><br/
<hw>Sper"ma*ry</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Anat.)</fld> <def>An organ in which spermatozoa are developed; a sperm gland; a testicle.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spermatheca</ent><br/
||<hw>Sper`ma*the"ca</hw> <pr>(?)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Spermathecae</plw> <pr>(#)</pr>.</plu> <ety>[NL., from Gr. <?/<?/<?/<?/ seed + <?/<?/<?/<?/ case, or receptacle.]</ety> <fld>(Zool.)</fld> <def>A small sac connected with the female reproductive organs of insects and many other invertebrates, serving to receive and retain the spermatozoa.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spermatic</ent><br/
<hw>Sper*mat"ic</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>spermaticus</ets>, Gr. <?/<?/<?/<?/: cf. F. <ets>spermatique</ets>. See <er>Sperm</er>.]</ety> <fld>(Physiol.)</fld> <def>Of or pertaining to semen; <as>as, the <ex>spermatic</ex> fluid, the <ex>spermatic</ex> vessels, etc.</as></def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Spermatic cord</b></col> <fld>(Anat.)</fld>, <cd>the cord which suspends the testicle within the scrotum. It is made up of a connective tissue sheath inclosing the spermatic duct and accompanying vessels and nerves.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spermatical</ent><br/
<hw>Sper"mat"ic*al</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Spermatic.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spermatin</ent><br/
<hw>Sper"ma*tin</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Physiol. Chem.)</fld> <def>A substance allied to alkali albumin and to mucin, present in semen, to which it is said to impart the mucilaginous character.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spermatism</ent><br/
<hw>Sper"ma*tism</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Physiol.)</fld> <def>The emission of sperm, or semen.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spermatium</ent><br/
||<hw>Sper*ma"ti*um</hw> <pr>(?)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Spermatia</plw> <pr>(#)</pr>.</plu> <ety>[NL.]</ety> <fld>(Bot.)</fld> <def>One of the motionless spermatozoids in the conceptacles of certain fungi.</def>  <rj><au>J. H. Balfour.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spermatize</ent><br/
<hw>Sper"ma*tize</hw> <pr>(?)</pr>, <pos>v. i.</pos> <ety>[Gr. <?/<?/<?/. See <er>Sperm</er>.]</ety> <def>To yield seed; to emit seed, or sperm.</def> <mark>[Obs.]</mark>  <rj><au>Sir T. Browne.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spermo-</ent><br/
<ent>Spermato-</ent><br/
<mhw>{ <hw>Sper"ma*to-</hw> <pr>(?)</pr>, <hw>Sper"mo-</hw> <pr>(?)</pr> }</mhw>. <def>Combining forms from Gr. <grk>spe`rma</grk>, <grk>-atos</grk>, <xex>seed</xex>, <xex>sperm</xex>, <xex>semen</xex> (of plants or animals); <as>as, <ex>spermato</ex>blast, <ex>spermo</ex>blast</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spermatoblast</ent><br/
<hw>Sper"ma*to*blast</hw> <pr>(?)</pr>, <pos>n.</pos> <def>Same as <er>Spermoblast</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spermatocyte</ent><br/
<hw>Sper"ma*to*cyte</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[<ets>Spermato-</ets> + Gr. <?/<?/<?/<?/ a hollow vessel.]</ety> <fld>(Physiol.)</fld> <def>Same as <er>Spermoblast</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spermatogemma</ent><br/
<hw>Sper`ma*to*gem"ma</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[NL. See <er>Spermato-</er>, and <er>Gemma</er>.]</ety> <fld>(Physiol.)</fld> <def>Same as <er>Spermosphere</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spermatogenesis</ent><br/
<hw>Sper`ma*to*gen"e*sis</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[<ets>Spermato-</ets> + <ets>genesis</ets>.]</ety> <fld>(Biol.)</fld> <def>The development of the spermatozoids.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spermatogenetic</ent><br/
<hw>Sper`ma*to*ge*net"ic</hw> <pr>(?)</pr>, <pos>a.</pos> <fld>(Physiol.)</fld> <def>Relating to, or connected with, spermatogenesis; <as>as, <ex>spermatogenetic</ex> function</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spermatogenous</ent><br/
<hw>Sper`ma*tog"e*nous</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[<ets>Spermato-</ets> + <ets>-genous</ets>.]</ety> <fld>(Physiol.)</fld> <def>Sperm-producing.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spermatogonium</ent><br/
||<hw>Sper`ma*to*go"ni*um</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[NL., fr. Gr. <?/<?/<?/<?/, <?/<?/<?/, sperm + <?/<?/<?/ offspring.]</ety> <fld>(Physiol.)</fld> <def>A primitive seminal cell, occuring in masses in the seminal tubules. It divides into a mass (spermosphere) of small cells (spermoblast), which in turn give rise to spermatozoids.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spermatoid</ent><br/
<hw>Sper"ma*toid</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[<ets>Spermato-</ets> + <ets>-oid</ets>.]</ety> <fld>(Physiol.)</fld> <def>Spermlike; resembling sperm, or semen.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spermatoon</ent><br/
||<hw>Sper`ma*to"on</hw> <pr>(?)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Spermatoa</plw> <pr>(#)</pr>.</plu> <ety>[NL., fr. Gr. <grk>spe`rma</grk>, <grk>-atos</grk>, seed + <?/<?/<?/ an egg.]</ety> <fld>(Anat.)</fld> <def>A spermoblast.</def> -- <wordforms><wf>Sper`ma*to"al</wf> <pr>(#)</pr>, <pos>a.</pos> <au>Owen.</au></wordforms><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spermatophore</ent><br/
<hw>Sper"ma*to*phore</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[<ets>Spermato-</ets> + Gr. <grk>fe`rein</grk> to bear.]</ety> <sn>1.</sn> <fld>(Physiol.)</fld> <def>Same as <er>Spermospore</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Zool.)</fld> <def>A capsule or pocket inclosing a number of spermatozoa. They are present in many annelids, brachiopods, mollusks, and crustaceans. In cephalopods the structure of the capsule is very complex.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spermatophorous</ent><br/
<hw>Sper`ma*toph"o*rous</hw> <pr>(?)</pr>, <pos>a.</pos> <fld>(Physiol.)</fld> <def>Producing seed, or sperm; seminiferous; <as>as, the so-called <ex>spermatophorous</ex> cells</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spermatophyta</ent><br/
||<hw>Sper`ma*toph"y*ta</hw> <pr>(?)</pr>, <pos>n. pl.</pos> <ety>[NL.; <ets>spermato-</ets> + Gr. <?/ plant.]</ety> <fld>(Bot.)</fld> <def>A phylum embracing the highest plants, or those that produce seeds; the  seed plants, or flowering plants. They form the most numerous group, including over 120,000 species. In general, the group is characterized by the marked development of the sporophyte, with great differentiation of its parts (root, stem, leaves, flowers, etc.); by the extreme reduction of the gametophyte; and by the development of seeds. All the Spermatophyta are heterosporous; fertilization of the egg cell is either through a <col><b>pollen tube</b></col> emitted by the microspore or (in a few gymnosperms) by spermatozoids.</def> <note>The phrase <ldquo/flowering plants<rdquo/ is less distinctive than <ldquo/seed plants,<rdquo/ since the conifers, grasses, sedges, oaks, etc., do not produce flowers in the popular sense. For this reason the terms <altname>Anthrophyta</altname>, <altname>Phaenogamia</altname>, and <altname>Panerogamia</altname> have been superseded as names of the phylum by <ex>Spermatophyta</ex>.</note><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Spermatophyte</ent><br/
<hw>Sper"ma*to"phyte`</hw> <pr>(?)</pr>, <pos>n.</pos> <def>Any plant of the phylum Spermatophyta.</def> -- <wordforms><wf>Sper`ma*to*phyt"ic</wf> <pr>(#)</pr>, <pos>a.</pos></wordforms><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Spermatorrhoea</ent><br/
<ent>Spermatorrhea</ent><br/
<mhw>{<hw>Sper`ma*tor*rhe"a</hw>, <hw>Sper`ma*tor*rhoe"a</hw>, <pr>(<?/)</pr>, }</mhw> <pos>n.</pos> <ety>[NL., fr. Gr. <grk>spe`rma</grk>, <grk>-atos</grk>, seed + <?/<?/<?/ to flow.]</ety> <fld>(Med.)</fld> <def>Abnormally frequent involuntary emission of the semen without copulation.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spermatospore</ent><br/
<hw>Sper"ma*to*spore</hw> <pr>(?)</pr>, <pos>n.</pos> <def>Same as <er>Spermospore</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spermatozoid</ent><br/
<hw>Sper`ma*to*zo"id</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[<ets>Spermatozoon</ets> + Gr. <grk>e'i^dos</grk> form.]</ety> <fld>(Biol.)</fld> <def>The male germ cell in animals and plants, the essential element in fertilization; a microscopic animalcule-like particle, usually provided with one or more cilia by which it is capable of active motion. In animals, the familiar type is that of a small, more or less ovoid head, with a delicate threadlike cilium, or tail. Called also <altname>spermatozoon</altname>. In plants the more usual term is <stype>antherozoid</stype>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spermatozooid</ent><br/
<hw>Sper`ma*to*zo"oid</hw> <pr>(?)</pr> <pos>n.</pos> <fld>(Biol.)</fld> <def>A spermatozoid.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spermatozoon</ent><br/
||<hw>Sper`ma*to*zo"on</hw> <pr>(?)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Spermatozoa</plw> <pr>(#)</pr>.</plu> <ety>[NL., fr. Gr. <?/<?/<?/, sperm + <grk>zo^,on</grk> an animal.]</ety> <fld>(Biol.)</fld> <def>Same as <er>Spermatozoid</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spermic</ent><br/
<hw>Sper"mic</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Of or pertaining to sperm, or semen.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spermidium</ent><br/
||<hw>Sper*mid"i*um</hw> <pr>(?)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Spermidia</plw> <pr>(#)</pr>.</plu> <ety>[Nl., fr. Gr. <grk>spe`rma</grk> seed.]</ety> <fld>(Bot.)</fld> <def>An achenium.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spermism</ent><br/
<hw>Sperm"ism</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Gr. <?/ seed, sperm + <ets>-ism</ets>.]</ety> <fld>(Biol.)</fld> <def>The theory, formerly held by many, that the sperm or spermatozoon contains the germ of the future embryo; animalculism.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Spermist</ent><br/
<hw>Sperm"ist</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Biol.)</fld> <def>A believer in the doctrine, formerly current, of encasement in the male (see <er>Encasement</er>), in which the seminal thread, or spermatozoid, was considered as the real animal germ, the head being the true animal head and the tail the body.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spermoblast</ent><br/
<hw>Sper"mo*blast</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[<ets>Spermo-</ets> + <ets>-blast</ets>.]</ety> <fld>(Physiol.)</fld> <def>One of the cells formed by the division of the spermospore, each of which is destined to become a spermatozoid; a spermatocyte; a spermatoblast.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spermococcus</ent><br/
||<hw>Sper`mo*coc"cus</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[NL. See <er>Spermo-</er>, and <er>Coccus</er>.]</ety> <fld>(Physiol.)</fld> <def>The nucleus of the sperm cell.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spermoderm</ent><br/
<hw>Sper"mo*derm</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[<ets>Spermo-</ets> + <ets>derm</ets>: cf. F. <ets>spermoderme</ets>.]</ety> <fld>(Bot.)</fld> <def>The covering of a seed; -- sometimes limited to the outer coat or testa.</def>  <rj><au>Lindley.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spermogonium</ent><br/
||<hw>Sper`mo*go"ni*um</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[NL.; <ets>spermo-</ets> + Gr. <?/<?/<?/ offspring.]</ety> <fld>(Bot.)</fld> <def>A conceptacle of certain lichens, which contains spermatia.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spermologist</ent><br/
<hw>Sper*mol"o*gist</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Gr. <?/<?/<?/<?/<?/ picking up seeds; <grk>spe`rma</grk> sperm, seed + <?/<?/<?/<?/ to gather.]</ety> <def>One who treats of, or collects, seeds.</def>  <rj><au>Bailey.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spermophile</ent><br/
<hw>Sper"mo*phile</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Gr. <grk>spe`rma</grk> a seed + <grk>fi`los</grk> loving, fond.]</ety> <fld>(Zool.)</fld> <def>Any ground squirrel of the genus <gen>Spermophilus</gen>; a gopher. See <xex>Illust.</xex> under <er>Gopher</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spermophore</ent><br/
<hw>Sper"mo*phore</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Zool.)</fld> <def>A spermatophore.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spermophyta</ent><br/
||<hw>Sper*moph"y*ta</hw> <pr>(?)</pr>, <pos>n. pl.</pos> <ety>[Nl., from Gr. <grk>spe`rma</grk> a seed + <grk>fyto`n</grk> a plant.]</ety> <def>Plants which produce seed; phaenogamia. These plants constitute the highest grand division of the vegetable kingdom.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spermophyte</ent><br/
<hw>Sper"mo*phyte</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Bot.)</fld> <def>Any plant which produces true seeds; -- a term recently proposed to replace <xex>ph<ae/nogam</xex>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spermophytic</ent><br/
<hw>Sper`mo*phyt"ic</hw> <pr>(?)</pr>, <pos>a.</pos> <fld>(Bot.)</fld> <def>Capable of producing seeds; ph<ae/nogamic.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spermoplasma</ent><br/
||<hw>Sper`mo*plas"ma</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[NL. See <er>Spermo-</er>, and <er>Plasma</er>.]</ety> <fld>(Physiol.)</fld> <def>The protoplasm of the sperm cell.</def>  <rj><au>Haeckel.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spermosphere</ent><br/
<hw>Sper"mo*sphere</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[<ets>Spermo-</ets> + <ets>sphere</ets>.]</ety> <fld>(Physiol.)</fld> <def>A mass or ball of cells formed by the repeated division of a male germinal cell (spermospore), each constituent cell (spermoblast) of which is converted into a spermatozoid; a spermatogemma.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spermospore</ent><br/
<hw>Sper"mo*spore</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[<ets>Spermo-</ets> + <ets>spore</ets>.]</ety> <fld>(Physiol.)</fld> <def>The male germinal or seminal cell, from the breaking up of which the spermoblasts are formed and ultimately the spermatozoids; a spermatospore.</def>  <rj><au>Balfour.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spermule</ent><br/
<hw>Sper"mule</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Dim. fr. <ets>sperm</ets>.]</ety> <fld>(Physiol.)</fld> <def>A sperm cell.</def>  <rj><au>Haeckel.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sperm whale</ent><br/
<hw>Sperm" whale`</hw> <pr>(?)</pr>. <fld>(Zool.)</fld> <def>A very large toothed whale (<spn>Physeter macrocephalus</spn>), having a head of enormous size. The upper jaw is destitute of teeth. In the upper part of the head, above the skull, there is a large cavity, or case, filled with oil and spermaceti. This whale sometimes grows to the length of more than eighty feet. It is found in the warmer parts of all the oceans. Called also <altname>cachalot</altname>, and <altname>spermaceti whale</altname>.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Pygmy sperm whale</b></col> <fld>(Zool.)</fld>, <cd>a small whale (<spn>Kogia breviceps</spn>), seldom twenty feet long, native of tropical seas, but occasionally found on the American coast. Called also <altname>snub-nosed cachalot</altname>.</cd> -- <col><b>Sperm-whale porpoise</b></col> <fld>(Zool.)</fld>, <cd>a toothed cetacean (<spn>Hyperoodon bidens</spn>), found on both sides of the Atlantic and valued for its oil. The adult becomes about twenty-five feet long, and its head is very large and thick. Called also <altname>bottle-nosed whale</altname>.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sperrylite</ent><br/
<hw>Sper"ry*lite</hw> <pr>(?)</pr>, <pos>n.</pos><ety>[Named after F. L. <ets>Sperry</ets>, who discovered it.]</ety> <fld>(Min.)</fld> <def>An arsenide of platinum occuring in grains and minute isometric crystals of tin-white color. It is found near Sudbury, Ontario Canada, and is the only known compound of platinum occuring in nature.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sperse</ent><br/
<hw>Sperse</hw> <pr>(?)</pr>, <pos>v. t.</pos> <def>To disperse.</def> <mark>[Obs.]</mark>  <rj><au>Spenser.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spessartite</ent><br/
<hw>Spes"sart*ite</hw> <pr>(?)</pr>, <pos>n.</pos><ety>[From <ets>Spessart</ets>, in Germany.]</ety> <fld>(Min.)</fld> <def>A manganesian variety of garnet.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spet</ent><br/
<hw>Spet</hw> <pr>(?)</pr>, <pos>v. t.</pos> <ety>[AS. <ets>sp<?/tan</ets>. See <er>Spit</er>.]</ety> <def>To spit; to throw out.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spet</ent><br/
<hw>Spet</hw>, <pos>n.</pos> <def>Spittle.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spetches</ent><br/
<hw>Spetch"es</hw> <pr>(?)</pr>, <pos>n. pl.</pos> <def>Parings and refuse of hides, skins, etc., from which glue is made.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spew</ent><br/
<hw>Spew</hw> <pr>(?)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Spewed</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Spewing</conjf>.]</vmorph> <ety>[OE. <ets>spewen</ets>, <ets>speowen</ets>, AS. <ets>sp<imac/wan</ets>;n to D. <ets>spuwen</ets> to spit. OS & OHG. <ets>sp<imac/wan</ets>, G. <ets>speien</ets>, Icel. <ets>sp<?/ja</ets> to spew, Sw. <ets>spy</ets>, Dan. <ets>spye</ets>, Goth. <ets>spiewan</ets>, th. <ets>spjauti</ets>, L. <ets>spuere</ets> to split, Gr. <?/<?/<?/, Skr. <ets>shtiv</ets>, <ets>shth<imac/v</ets>.  Cf. <er>Pyke</er>, <er>Spit</er>.]</ety> <altsp>[Written also <asp>spue</asp>.]</altsp> <sn>1.</sn> <def>To eject from the stomach; to vomit.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To cast forth with abhorrence or disgust; to eject.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Because thou art lukewarm, and neither hot nor cold, I will <qex>spew</qex> thee out of my mouth.</q> <rj><qau>Rev. ii. 16.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spew</ent><br/
<hw>Spew</hw>, <pos>v. i.</pos> <sn>1.</sn> <def>To vomit.</def>  <rj><au>Chaucer.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To eject seed, as wet land swollen with frost.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spew</ent><br/
<hw>Spew</hw>, <pos>n.</pos> <def>That which is vomited; vomit.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spewer</ent><br/
<hw>Spew"er</hw> <pr>(?)</pr>, <pos>n.</pos> <def>One who spews.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spewiness</ent><br/
<hw>Spew"i*ness</hw> <pr>(?)</pr>, <pos>n.</pos> <def>The state of being spewy.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spewy</ent><br/
<hw>Spew"y</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Wet; soggy; inclined to spew.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sphacel</ent><br/
<hw>Sphac"el</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Gr. <?/<?/<?/: cf. F. <ets>sphac<egrave/le</ets>.]</ety> <fld>(Med.)</fld> <def>Gangrene.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sphacelate</ent><br/
<hw>Sphac"e*late</hw> <pr>(?)</pr>, <pos>v. i.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Sphacelated</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Sphacelating</conjf>.]</vmorph> <ety>[NL. <ets>sphacelare</ets>, <ets>sphacelatum</ets>, mortify: cf. F. <ets>sphac<eacute/ler</ets>. See <er>Sphacelus</er>.]</ety> <fld>(Med.)</fld> <def>To die, decay, or become gangrenous, as flesh or bone; to mortify.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sphacelate</ent><br/
<hw>Sphac"e*late</hw>, <pos>v. t.</pos> <fld>(Med.)</fld> <def>To affect with gangrene.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sphacelated</ent><br/
<ent>Sphacelate</ent><br/
<mhw>{ <hw>Sphac"e*late</hw> <pr>(?)</pr>, <hw>Sphac"e*la`ted</hw> <pr>(?)</pr>, }</mhw> <pos>a.</pos> <fld>(Med.)</fld> <def>Affected with gangrene; mortified.</def><br/
[<source>1913 Webster</source>]</p>

<p><-- p. 1384 --></p>

<p><ent>Sphacelation</ent><br/
<hw>Sphac`e*la"tion</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Med.)</fld> <def>The process of becoming or making gangrenous; mortification.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sphacelus</ent><br/
||<hw>Sphac"e*lus</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[NL., fr. Gr. <?/<?/<?/<?/.]</ety> <fld>(Med.)</fld> <def>Gangrenous part; gangrene; slough.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sphaerenchyma</ent><br/
||<hw>Sphae*ren"chy*ma</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[NL., fr. Gr. <?/<?/<?/ sphere + <ets>-enchyma</ets> as in <ets>parenchima</ets>.]</ety> <fld>(Bot.)</fld> <def>Vegetable tissue composed of thin-walled rounded cells, -- a modification of parenchyma.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sphaeridium</ent><br/
||<hw>Sphae*rid"i*um</hw> <pr>(?)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Sphaeridia</plw> <pr>(#)</pr>.</plu> <ety>[NL., fr. Gr. <?/<?/<?/ a sphere.]</ety> <fld>(Zool.)</fld> <def>A peculiar sense organ found upon the exterior of most kinds of sea urchins, and consisting of an oval or sherical head surmounting a short pedicel. It is generally supposed to be an olfactory organ.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sphaerospore</ent><br/
<hw>Sphae"ro*spore</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Gr. <?/<?/<?/<?/ sphere + E. <ets>spore</ets>.]</ety> <fld>(Bot.)</fld> <def>One of the nonsexual spores found in red algae; a tetraspore.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sphaerulite</ent><br/
<hw>Sphaer"u*lite</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Min.)</fld> <def>Same as <er>Spherulite</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sphagnicolous</ent><br/
<hw>Sphag*nic"o*lous</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[<ets>Sphagnum</ets> + L. <ets>colere</ets> to inhabit.]</ety> <fld>(Bot.)</fld> <def>Growing in moss of the genus <gen>Sphagnum</gen>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sphagnous</ent><br/
<hw>Sphag"nous</hw> <pr>(?)</pr>, <pos>a.</pos> <fld>(Bot.)</fld> <def>Pertaining to moss of the genus <gen>Sphagnum</gen>, or bog moss; abounding in peat or bog moss.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sphagnum</ent><br/
||<hw>Sphag"num</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[NL., fr/ Gr. <?/<?/<?/ a kind of moss.]</ety> <fld>(Bot.)</fld> <def>A genus of mosses having white leaves slightly tinged with red or green and found growing in marshy places; bog moss; peat moss.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sphalerite</ent><br/
<hw>Sphal"er*ite</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Gr. <?/<?/<?/ slippery, uncertain. See <er>Blende</er>.]</ety> <fld>(Min.)</fld> <def>Zinc sulphide; -- called also <altname>blende</altname>, <altname>black-jack</altname>, <altname>false galena</altname>, etc. See <er>Blende</er> <sd>(a)</sd>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sphene</ent><br/
<hw>Sphene</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F. <ets>sph<egrave/ne</ets>, fr. Gr. <grk>sfh`n</grk> a wedge.]</ety> <fld>(Min.)</fld> <def>A mineral found usually in thin, wedge-shaped crystals of a yellow or green to black color. It is a silicate of titanium and calcium; titanite.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sphenethmoid</ent><br/
<hw>Sphe*neth"moid</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[<ets>Sphen</ets>oid + <ets>ethmoid</ets>.]</ety> <fld>(Anat.)</fld> <def>Of or pertaining to both the sphenoidal and the ethmoidal regions of the skull, or the sphenethmoid bone; sphenethmoidal.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Sphenethmoid bone</b></col> <fld>(Anat.)</fld>, <cd>a bone of the skull which surrounds the anterior end of the brain in many amphibia; the girdle bone.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sphenethmoid</ent><br/
<hw>Sphe*neth"moid</hw>, <pos>n.</pos> <fld>(Anat.)</fld> <def>The sphenethmoid bone.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sphenethmoidal</ent><br/
<hw>Sphe`neth*moid"al</hw> <pr>(?)</pr>, <pos>a.</pos> <fld>(Anat.)</fld> <def>Relating to the sphenoethmoid bone; sphenoethmoid.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spheniscan</ent><br/
<hw>Sphe*nis"can</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Zool.)</fld> <def>Any species of penguin.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spheno-</ent><br/
<hw>Sphe"no-</hw> <pr>(?)</pr>. <def>A combining form used in anatomy to indicate <xex>connection with</xex>, or <xex>relation to</xex>, <xex>the sphenoid bone</xex>; as in <xex>spheno</xex>maxillary, <xex>spheno</xex>palatine.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sphenodon</ent><br/
<hw>Sphe"no*don</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Gr. <grk>sfh`n</grk> a wedge + <?/<?/<?/, <?/<?/<?/, a tooth.]</ety> <fld>(Zool.)</fld> <def>Same as <er>Hatteria</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sphenoethmoidal</ent><br/
<hw>Sphe"no*eth*moid`al</hw> <pr>(?)</pr>, <pos>a.</pos> <fld>(Anat.)</fld> <def>Sphenethmoid.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sphenogram</ent><br/
<hw>Sphe"no*gram</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Gr. <grk>sfh`n</grk> a wedge + <ets>-gram</ets>.]</ety> <def>A cuneiform, or arrow-headed, character.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sphenographer</ent><br/
<hw>Sphe*nog"ra*pher</hw>, <pos>n.</pos> <def>One skilled in sphenography; a sphenographist.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sphenographic</ent><br/
<hw>Sphe`no*graph"ic</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Of or pertaining to sphenography.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sphenographist</ent><br/
<hw>Sphe*nog"ra*phist</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A sphenographer.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sphenography</ent><br/
<hw>Sphe*nog"ra*phy</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Gr. <grk>sfh`n</grk> a wedge + <ets>-graphy</ets>.]</ety> <def>The art of writing in cuneiform characters, or of deciphering inscriptions made in such characters.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sphenoid</ent><br/
<hw>Sphe"noid</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[Gr. <?/<?/<?/; <grk>sfh`n</grk> a wedge + <grk>e'i^dos</grk> form: cf. F. <ets>sph<eacute/no<ium/de</ets>.]</ety> <sn>1.</sn> <def>Wedge-shaped; <as>as, a <ex>sphenoid</ex> crystal</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Anat.)</fld> <def>Of or pertaining to the sphenoid bone.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Sphenoid bone</b></col> <fld>(Anat.)</fld>, <cd>an irregularly shaped bone in front of the occipital in the base of the skull of the higher vertebrates. It is composed of several fetal bones which become united the adult. See <er>Alisphenoid</er>, <er>Basisphenoid</er>, <er>Orbitosphenoid</er>, <er>Presphenoid</er>.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sphenoid</ent><br/
<hw>Sphe"noid</hw> <pr>(?)</pr>, <pos>n.</pos> <sn>1.</sn> <fld>(Crystallog.)</fld> <def>A wedge-shaped crystal bounded by four equal isosceles triangles. It is the hemihedral form of a square pyramid.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Anat.)</fld> <def>The sphenoid bone.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sphenoidal</ent><br/
<hw>Sphe*noid"al</hw> <pr>(?)</pr>, <pos>a.</pos> <sn>1.</sn> <def>Sphenoid.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Crystalloq.)</fld> <def>Pertaining to, or resembling, a sphenoid.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sphenotic</ent><br/
<hw>Sphe*not"ic</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[<ets>Spheno-</ets> + <?/<?/<?/, <?/<?/<?/, the ear.]</ety> <fld>(Anat.)</fld> <def>Of, pertaining to, or designating, the sphenotic bone.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Sphenotic bone</b></col> <fld>(Anat.)</fld>, <cd>a bone on the anterior side of the auditory capsule of many fishes, and connected with, or adjoining, the sphenoid bone.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sphenotic</ent><br/
<hw>Sphe*not"ic</hw>, <pos>n.</pos> <fld>(Anat.)</fld> <def>The sphenotic bone.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spheral</ent><br/
<hw>Spher"al</hw> <pr>(?)</pr>, <pos>a.</pos> <sn>1.</sn> <def>Of or pertaining to a sphere or the spheres.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Rounded like a sphere; sphere-shaped; hence, symmetrical; complete; perfect.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sphere</ent><br/
<hw>Sphere</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[OE. <ets>spere</ets>, OF. <ets>espere</ets>, F. <ets>sph<egrave/re</ets>, L. <ets>sphaera</ets>,. Gr. <?/<?/<?/ a sphere, a ball.]</ety> <sn>1.</sn> <fld>(Geom.)</fld> <def>A body or space contained under a single surface, which in every part is equally distant from a point within called its <xex>center</xex>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Hence, any globe or globular body, especially a celestial one, as the sun, a planet, or the earth.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Of celestial bodies, first the sun,<br/
A mighty <qex>sphere</qex>, he framed.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <fld>(Astron.)</fld> <sd>(a)</sd> <def>The apparent surface of the heavens, which is assumed to be spherical and everywhere equally distant, in which the heavenly bodies appear to have their places, and on which the various astronomical circles, as of right ascension and declination, the equator, ecliptic, etc., are conceived to be drawn; an ideal geometrical sphere, with the astronomical and geographical circles in their proper positions on it.</def> <sd>(b)</sd> <def>In ancient astronomy, one of the concentric and eccentric revolving spherical transparent shells in which the stars, sun, planets, and moon were supposed to be set, and by which they were carried, in such a manner as to produce their apparent motions.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <fld>(Logic)</fld> <def>The extension of a general conception, or the totality of the individuals or species to which it may be applied.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <def>Circuit or range of action, knowledge, or influence; compass; province; employment; place of existence.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>To be called into a huge <qex>sphere</qex>, and not to be seen to move in 't.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Taking her out of the ordinary relations with humanity, and inclosing her in a <qex>sphere</qex> by herself.</q> <rj><qau>Hawthorne.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Each in his hidden <qex>sphere</qex> of joy or woe<br/
Our hermit spirits dwell.</q> <rj><qau>Keble.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>6.</sn> <def>Rank; order of society; social positions.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>7.</sn> <def>An orbit, as of a star; a socket.</def> <mark>[R.]</mark>  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><mcol><col><b>Armillary sphere</b></col>, <col><b>Crystalline sphere</b></col>, <col><b>Oblique sphere</b></col></mcol>,. <cd>See under <er>Armillary</er>, <er>Crystalline</er>,.</cd> -- <col><b>Doctrine of the sphere</b></col>, <cd>applications of the principles of spherical trigonometry to the properties and relations of the circles of the sphere, and the problems connected with them, in astronomy and geography, as to the latitudes and longitudes, distance and bearing, of places on the earth, and the right ascension and declination, altitude and azimuth, rising and setting, etc., of the heavenly bodies; spherical geometry.</cd> -- <col><b>Music of the spheres</b></col>. <cd>See under <er>Music</er>.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><syn><b>Syn.</b> -- Globe; orb; circle. See <er>Globe</er>.</syn><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sphere</ent><br/
<hw>Sphere</hw> <pr>(?)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Sphered</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Sphering</conjf>.]</vmorph> <sn>1.</sn> <def>To place in a sphere, or among the spheres; to insphere.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The glorious planet Sol<br/
In noble eminence enthroned and <qex>sphered</qex><br/
Amidst the other.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To form into roundness; to make spherical, or spheral; to perfect.</def>  <rj><au>Tennyson.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spheric</ent><br/
<ent>Spherical</ent><br/
<mhw>{ <hw>Spher"ic*al</hw> <pr>(?)</pr>, <hw>Spher"ic</hw> <pr>(?)</pr>, }</mhw> <pos>a.</pos> <ety>[L. <ets>sphaericus</ets>, Gr. <?/<?/<?/: cf. F. <ets>sph<eacute/rique</ets>.]</ety> <sn>1.</sn> <def>Having the form of a sphere; like a sphere; globular; orbicular; <as>as, a <ex>spherical</ex> body</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Of or pertaining to a sphere.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>Of or pertaining to the heavenly orbs, or to the sphere or spheres in which, according to ancient astronomy and astrology, they were set.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Knaves, thieves, and treachers by <qex>spherical</qex> predominance.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Though the stars were suns, and overburned<br/
Their <qex>spheric</qex> limitations.</q> <rj><qau>Mrs. Browning.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><mcol><col><b>Spherical angle</b></col>, <col><b>Spherical coordinate</b></col>, <col><b>Spherical excess</b></col></mcol>, <cd>etc. See under <er>Angle</er>, <er>Coordinate</er>, etc.</cd> -- <col><b>Spherical geometry</b></col>, <cd>that branch of geometry which treats of spherical magnitudes; the doctrine of the sphere, especially of the circles described on its surface.</cd> -- <col><b>Spherical harmonic analysis</b></col>. <cd>See under <er>Harmonic</er>, <pos>a.</pos></cd> -- <col><b>Spherical lune</b></col>,<cd>portion of the surface of a sphere included between two great semicircles having a common diameter.</cd> -- <col><b>Spherical opening</b></col>, <cd>the magnitude of a solid angle. It is measured by the portion within the solid angle of the surface of any sphere whose center is the angular point.</cd> -- <col><b>Spherical polygon</b></col>,<cd>portion of the surface of a sphere bounded by the arcs of three or more great circles.</cd> -- <col><b>Spherical projection</b></col>, <cd>the projection of the circles of the sphere upon a plane. See <er>Projection</er>.</cd> -- <col><b>Spherical sector</b></col>. <cd>See under <er>Sector</er>.</cd> -- <col><b>Spherical segment</b></col>, <cd>the segment of a sphere. See under <er>Segment</er>.</cd> -- <col><b>Spherical triangle</b></col>,<cd>re on the surface of a sphere, bounded by the arcs of three great circles which intersect each other.</cd> -- <col><b>Spherical trigonometry</b></col>. <cd>See <er>Trigonometry</er>.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p>-- <wordforms><wf>Spher"ic*al*ly</wf>, <pos>adv.</pos> -- <wf>Spher"ic*al*ness</wf>, <pos>n.</pos></wordforms><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sphericity</ent><br/
<hw>Sphe*ric"i*ty</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Cf. F. <ets>sph<eacute/ricit<eacute/</ets>.]</ety> <def>The quality or state of being spherial; roundness; <as>as, the <ex>sphericity</ex> of the planets, or of a drop of water</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sphericle</ent><br/
<hw>Spher"i*cle</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A small sphere.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spherics</ent><br/
<hw>Spher"ics</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Math.)</fld> <def>The doctrine of the sphere; the science of the properties and relations of the circles, figures, and other magnitudes of a sphere, produced by planes intersecting it; spherical geometry and trigonometry.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spherobacteria</ent><br/
||<hw>Sphe`ro*bac*te"ri*a</hw> <pr>(?)</pr>, <pos>n. pl.</pos>; <sing>sing. <singw>Spherobacterium</singw> <pr>(<?/)</pr></sing>. <ety>[NL. See <er>Sphere</er>, <er>and Bacterium</er>.]</ety> <fld>(Biol.)</fld> <def>See the Note under <er>Microbacteria</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spheroconic</ent><br/
<hw>Sphe`ro*con"ic</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Geom.)</fld> <def>A nonplane curve formed by the intersection of the surface of an oblique cone with the surface of a sphere whose center is at the vertex of the cone.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spherograph</ent><br/
<hw>Spher"o*graph</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[<ets>Sphere</ets> + <ets>-graph</ets>.]</ety> <def>An instrument for facilitating the practical use of spherics in navigation and astronomy, being constructed of two cardboards containing various circles, and turning upon each other in such a manner that any possible spherical triangle may be readily found, and the measures of the parts read off by inspection.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spheroid</ent><br/
<hw>Sphe"roid</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>spheroides</ets> ball-like, spherical, Gr. <?/<?/<?/; <?/<?/<?/<?/ sphere + <grk>e'i^dos</grk> form: cf. F. <ets>sph<eacute/ro<ium/de</ets>.]</ety> <def>A body or figure approaching to a sphere, but not perfectly spherical; esp., a solid generated by the revolution of an ellipse about one of its axes.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><mcol><col><b>Oblate spheroid</b></col>, <col><b>Prolate spheroid</b></col></mcol>.  <cd>See <er>Oblate</er>, <er>Prolate</er>, and <er>Ellipsoid</er>.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spheroidal</ent><br/
<hw>Sphe*roid"al</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[Cf. F. <ets>sph<eacute/ro<ium/dal</ets>.]</ety> <def>Having the form of a spheroid.</def> -- <wordforms><wf>Sphe*roid"al*ly</wf>, <pos>adv.</pos></wordforms><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Spheroidal state</b></col> <fld>(Physics.)</fld>, <cd>the state of a liquid, as water, when, on being thrown on a surface of highly heated metal, it rolls about in spheroidal drops or masses, at a temperature several degrees below ebullition, and without actual contact with the heated surface, -- a phenomenon due to the repulsive force of heat, the intervention of a cushion of nonconducting vapor, and the cooling effect of evaporation.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spheroidical</ent><br/
<ent>Spheroidic</ent><br/
<mhw>{ <hw>Sphe*roid"ic</hw> <pr>(?)</pr>, <hw>Sphe*roid"ic*al</hw> <pr>(?)</pr>, }</mhw> <pos>a.</pos> <def>See <er>Spheroidal</er>.</def>  <rj><au>Cheyne.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spheroidity</ent><br/
<ent>Spheroidicity</ent><br/
<mhw>{ <hw>Sphe`roi*dic"i*ty</hw> <pr>(?)</pr>, <hw>Sphe*roid"i*ty</hw> <pr>(?)</pr>, }</mhw> <pos>n.</pos> <def>The quality or state of being spheroidal.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spheromere</ent><br/
<hw>Sphe"ro*mere</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[<ets>Sphere</ets> + <ets>-mere</ets>.]</ety> <fld>(Zool.)</fld> <def>Any one of the several symmetrical segments arranged around the central axis and composing the body of a radiate anmal.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spherometer</ent><br/
<hw>Sphe*rom"e*ter</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[<ets>Sphere</ets> + <ets>-meter</ets>: cf. F. <ets>sph<eacute/rom<egrave/tre</ets>.]</ety> <fld>(Physics)</fld> <def>An instrument for measuring the curvature of spherical surface, as of lenses for telescope, etc.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spherosiderite</ent><br/
<hw>Spher`o*sid"er*ite</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[<ets>Sphere</ets> + <ets>siderite</ets>.]</ety> <fld>(Min.)</fld> <def>Siderite occuring in spheroidal masses.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spherosome</ent><br/
<hw>Sphe"ro*some</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[<ets>Sphere</ets> + <ets>-some</ets> body.]</ety> <fld>(Zool.)</fld> <def>The body wall of any radiate animal.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spherulate</ent><br/
<hw>Spher"u*late</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Covered or set with spherules; having one or more rows of spherules, or minute tubercles.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spherule</ent><br/
<hw>Spher"ule</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>spherula</ets>: cf. F. <ets>sph<eacute/rule</ets>.]</ety> <def>A little sphere or spherical body; <as>as, quicksilver, when poured upon a plane, divides itself into a great number of minute <ex>spherules</ex></as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spherulite</ent><br/
<hw>Spher"u*lite</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Cf.  F. <ets>sph<eacute/rulite</ets>.]</ety> <fld>(Min.)</fld> <def>A minute spherical crystalline body having a radiated structure, observed in some vitreous volcanic rocks, as obsidian and pearlstone.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spherulitic</ent><br/
<hw>Spher`u*lit"ic</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Of or pertaining to a spherulite; characterized by the presence of spherulites.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sphery</ent><br/
<hw>Spher"y</hw> <pr>(?)</pr>, <pos>a.</pos> <sn>1.</sn> <def>Round; spherical; starlike.</def> <mark>[R.]</mark> <ldquo/Hermia's <xex>sphery</xex> eyne.<rdquo/  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Of or pertaining to the spheres.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>She can teach ye how to climb<br/
Higher than the <qex>sphery</qex> chime.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sphex</ent><br/
<hw>Sphex</hw> <pr>(?)</pr>, <pos>n.</pos><ety>[NL., fr. Gr. <?/<?/<?/, <?/<?/<?/, a wasp.]</ety> <fld>(Zool.)</fld> <def>Any one of numerous species of sand wasps of the genus <gen>Sphex</gen> and allied genera. These wasps have the abdomen attached to the thorax by a slender pedicel. See <xex>Illust.</xex> of <cref>Sand wasp</cref>, under <er>Sand</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Sphex fly</b></col> <fld>(Zool.)</fld>, <cd>any one of numerous species of small dipterous flies of the genus <gen>Conops</gen> and allied genera. The form of the body is similar to that of a sphex.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sphigmometer</ent><br/
<hw>Sphig*mom"e*ter</hw> <pr>(?)</pr>, <pos>n.</pos> <def>See <er>Sphygmometer</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sphincter</ent><br/
<hw>Sphinc"ter</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[NL., fr. <?/<?/<?/ to bind tight.]</ety> <fld>(Anat.)</fld> <def>A muscle which surrounds, and by its contraction tends to close, a natural opening; <as>as, the <ex>sphincter</ex> of the bladder</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sphincter</ent><br/
<hw>Sphinc"ter</hw>, <pos>a.</pos> <fld>(Anat.)</fld> <def>Of, pertaining to, or designating, a sphincter; <as>as, a <ex>sphincter</ex> muscle</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sphingid</ent><br/
<hw>Sphin"gid</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Zool.)</fld> <def>A sphinx.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sphingid</ent><br/
<hw>Sphin"gid</hw>, <pos>a.</pos> <def>Of or pertaining to a sphinx, or the family <fam>Sphingidae</fam>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sphinx</ent><br/
<hw>Sphinx</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L., from Gr. <grk>sfi`gx</grk>, usually derived from <grk>sfi`ggein</grk> to bind tight or together, as if the Throttler.]</ety> <sn>1.</sn> <sd>(a)</sd> <def>In Egyptian art, an image of granite or porphyry, having a human head, or the head of a ram or of a hawk, upon the wingless body of a lion.</def>
<-- ETY sic.? something missing? --><br/
[<source>1913 Webster</source>]</p>

<p><q>The awful ruins of the days of old . . .<br/
Or jasper tomb, or mutilated <qex>sphinx</qex>.</q> <rj><qau>Shelley.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sd>(b)</sd> <def>On Greek art and mythology, a she-monster, usually represented as having the winged body of a lion, and the face and breast of a young woman.</def> <note>The most famous Grecian sphinx, that of Thebes in B<oe/otia, is said to have proposed a riddle to the Thebans, and killed those who were unable to guess it. The enigma was solved by <OE/dipus, whereupon the sphinx slew herself.</note> <ldquo/Subtle as <xex>sphinx</xex>.<rdquo/  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Hence: A person of enigmatical character and purposes, especially in politics and diplomacy.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <fld>(Zool.)</fld> <def>Any one of numerous species of large moths of the family <fam>Sphingidae</fam>; -- called also <altname>hawk moth</altname>.  See also <cref>tomato worm</cref>.</def><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ The larva is a stout naked caterpillar which, when at rest, often assumes a position suggesting the Egyptian sphinx, whence the name.</note><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <fld>(Zool.)</fld> <def>The Guinea, or sphinx, baboon (<spn>Cynocephalus sphinx</spn>).</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Sphinx baboon</b></col> <fld>(Zool.)</fld>, <cd>a large West African baboon (<spn>Cynocephalus sphinx</spn>), often kept in menageries.</cd> -- <col><b>Sphinx moth</b></col>. <fld>(Zool.)</fld> <cd>Same as <er>Sphinx</er>, 3.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sphragide</ent><br/
<hw>Sphrag"ide</hw> <pr>(?)</pr>, <pos>n.</pos><ety>[L. <ets>sphragis</ets>, <ets>-idis</ets>, Lemnian earth, fr. Gr. <?/<?/<?/, <?/<?/<?/, a seal; -- so called because sold in sealed packets.]</ety> <fld>(Min.)</fld> <def>Lemnian earth.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sphragistics</ent><br/
<hw>Sphra*gis"tics</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Gr. <?/<?/<?/<?/ of or for sealing, fr. <?/<?/<?/ a seal.]</ety> <def>The science of seals, their history, age, distinctions, etc., esp. as verifying the age and genuiness of documents.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sphrigosis</ent><br/
||<hw>Sphri*go"sis</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[NL., fr. Gr. <?/<?/<?/ to be full of strength.]</ety> <fld>(Bot.)</fld> <def>A condition of vegetation in which there is too abundant growth of the stem and leaves, accompanied by deficiency of flowers and fruit.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sphygmic</ent><br/
<hw>Sphyg"mic</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[Gr. <?/<?/<?/ the pulse.]</ety> <fld>(Physiol.)</fld> <def>Of or pertaining to the pulse.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sphygmogram</ent><br/
<hw>Sphyg"mo*gram</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Gr. <?/<?/<?/<?/ pulse + <ets>-gram</ets>.]</ety> <fld>(Physiol.)</fld> <def>A tracing, called a pulse tracing, consisting of a series of curves corresponding with the beats of the heart, obtained by the application of the sphygmograph.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sphygmograph</ent><br/
<hw>Sphyg"mo*graph</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Gr. <?/<?/<?/ the pulse + <ets>-graph</ets>.]</ety> <fld>(Physiol.)</fld> <def>An instrument which, when applied over an artery, indicates graphically the movements or character of the pulse. See <er>Sphygmogram</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sphygmographic</ent><br/
<hw>Sphyg`mo*graph"ic</hw> <pr>(?)</pr>, <pos>a.</pos> <fld>(Phusiol.)</fld> <def>Relating to, or produced by, a sphygmograph; <as>as, a <ex>sphygmographic</ex> tracing</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><-- p. 1385 --></p>

<p><ent>Sphygmometer</ent><br/
<hw>Sphyg*mom"e*ter</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Gr. <?/<?/<?/<?/ pulse + <ets>-meter</ets>.]</ety> <fld>(Physiol.)</fld> <def>An instrument for measuring the strength of the pulse beat; a sphygmograph.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sphygmophone</ent><br/
<hw>Sphyg"mo*phone</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Gr. <?/<?/<?/ the pulse + <?/<?/<?/ sound.]</ety> <fld>(Physiol.)</fld> <def>An electrical instrument for determining by the ear the rhythm of the pulse of a person at a distance.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sphygmoscope</ent><br/
<hw>Sphyg"mo*scope</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Gr. <?/<?/<?/ the pulse + <ets>-scope</ets>.]</ety> <fld>(Physiol.)</fld> <def>Same as <er>Sphygmograph</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sphyraenoid</ent><br/
<hw>Sphy*rae"noid</hw> <pr>(?)</pr>, <pos>a.</pos><ety>[L. <ets>sphyraena</ets> a kind of sea fish (Gr. <grk>sfy`raina</grk>) + <ets>-oid</ets>.]</ety> <fld>(Zool.)</fld> <def>Of or pertaining to the <fam>Sphyraenidae</fam>, a family of marine fishes including the barracudas.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spial</ent><br/
<hw>Spi"al</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A spy; a scout.</def> <mark>[Obs.]</mark>  <rj><au>Bacon.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spica</ent><br/
||<hw>Spi"ca</hw> <pr>(?)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Spicae</plw> <pr>(#)</pr>.</plu> <ety>[L., an ear, as of corn.]</ety> <sn>1.</sn> <fld>(Med.)</fld> <def>A kind of bandage passing, by successive turns and crosses, from an extremity to the trunk; -- so called from its resemblance to a spike of a barley.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Astron.)</fld> <def>A star of the first magnitude situated in the constellation Virgo.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spicated</ent><br/
<ent>Spicate</ent><br/
<mhw>{ <hw>Spi"cate</hw> <pr>(?)</pr>, <hw>Spi"ca*ted</hw> <pr>(?)</pr>, }</mhw> <pos>a.</pos> <ety>[L. <ets>spicatus</ets>, p. p. of <ets>spicare</ets> furnish with spikes, or ears, fr. <ets>spica</ets> a spike, or ear.]</ety> <fld>(Bot.)</fld> <def>Having the form of a spike, or ear; arranged in a spike or spikes.</def>  <rj><au>Lee.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spiccato</ent><br/
||<hw>Spic*ca"to</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[It., p. p. of <ets>spicare</ets> to detach, to separate.]</ety> <fld>(Mus.)</fld> <def>Detached; separated; -- a term indicating that every note is to be performed in a distinct and pointed manner.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spice</ent><br/
<hw>Spice</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[OE. <ets>spice</ets>, <ets>spece</ets>, spice, species, OF. <ets>espice</ets>, <ets>espece</ets>, F. <ets><eacute/pice</ets> spice, <ets>esp<egrave/ce</ets> species, fr. L. <ets>species</ets> particular sort or kind, a species, a sight, appearance, show, LL., spices, drugs, etc., of the same sort, fr. L. <ets>specere</ets> to look. See <er>Spy</er>, and cf. <er>Species</er>.]</ety> <sn>1.</sn> <def>Species; kind.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>The <qex>spices</qex> of penance ben three.</q> <rj><qau>Chaucer.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Abstain you from all evil <qex>spice</qex>.</q> <rj><qau>Wyclif (1. Thess,v. 22).</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Justice, although it be but one entire virtue, yet is described in two kinds of <qex>spices</qex>. The one is named justice distributive, the other is called commutative.</q> <rj><qau>Sir T. Elyot.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A vegetable production of many kinds, fragrant or aromatic and pungent to the taste, as pepper, cinnamon, nutmeg, mace, allspice, ginger, cloves, etc., which are used in cookery and to flavor sauces, pickles, etc.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Hast thou aught in thy purse [bag] any hot <qex>spices</qex>?</q> <rj><qau>Piers Plowman.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>Figuratively, that which enriches or alters the quality of a thing in a small degree, as spice alters the taste of food; that which gives zest or pungency; a slight flavoring; a relish; hence, a small quantity or admixture; a sprinkling; <as>as, a <ex>spice</ex> of mischief</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>So much of the will, with a <qex>spice</qex> of the willful.</q> <rj><qau>Coleridge.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spice</ent><br/
<hw>Spice</hw>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Spiced</conjf> <pr>(?)</pr>; <pos>p. p. & vb. n.</pos> <conjf>Spicing</conjf> <pr>(?)</pr>.]</vmorph> <sn>1.</sn> <def>To season with spice, or as with spice; to mix aromatic or pungent substances with; to flavor; to season; <as>as, to <ex>spice</ex> wine; to <ex>spice</ex> one's words with wit</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>She 'll receive thee, but will <qex>spice</qex> thy bread<br/
With flowery poisons.</q> <rj><qau>Chapman.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To fill or impregnate with the odor of spices.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>In the <qex>spiced</qex> Indian air, by night.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>To render nice or dainty; hence, to render scrupulous.</def> <mark>[Obs.]</mark> <ldquo/A <xex>spiced</xex> conscience.<rdquo/  <rj><au>Chaucer.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spicebush</ent><br/
<hw>Spice"bush`</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Bot.)</fld> <def>Spicewood.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spicenut</ent><br/
<hw>Spice"nut`</hw> <pr>(?)</pr>. <def>A small crisp cake, highly spiced.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spicer</ent><br/
<hw>Spi"cer</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Cf. OF. <ets>espicier</ets>, F. <ets><eacute/picier</ets>.]</ety> <sn>1.</sn> <def>One who seasons with spice.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>One who deals in spice.</def> <mark>[Obs.]</mark>  <rj><au>Piers Plowman.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spicery</ent><br/
<hw>Spi"cer*y</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[OF. <ets>espicerie</ets>, F. <ets><eacute/picerie</ets>.]</ety> <sn>1.</sn> <def>Spices, in general.</def>  <rj><au>Chaucer.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A repository of spices.</def>  <rj><au>Addison.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spicewood</ent><br/
<hw>Spice"wood`</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Bot.)</fld> <def>An American shrub (<spn>Lindera Benzoin</spn>), the bark of which has a spicy taste and odor; -- called also <altname>Benjamin</altname>, <altname>wild allspice</altname>, and <altname>fever bush</altname>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spiciferous</ent><br/
<hw>Spi*cif"er*ous</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>spicifer</ets> bearing spikes, or ears; <ets>spica</ets> ear + <ets>ferre</ets> to bear.]</ety> <def>Bearing ears, or spikes; spicate.</def> <mark>[Obs.]</mark>  <rj><au>Bailey.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spiciform</ent><br/
<hw>Spi"ci*form</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>spica</ets> a spike, ear + <ets>-form</ets>.]</ety> <fld>(Bot.)</fld> <def>Spike-shaped.</def>  <rj><au>Gray.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spicily</ent><br/
<hw>Spi"ci*ly</hw>, <pos>adv.</pos> <def>In a spicy manner.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spiciness</ent><br/
<hw>Spi"ci*ness</hw>, <pos>n.</pos> <def>The quality or state of being spicy.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spick</ent><br/
<hw>Spick</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Cf. Sw. <ets>spik</ets>. See <er>Spike</er> a nail.]</ety> <def>A spike or nail.</def> <mark>[Prov. Eng.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Spick and span</b></col>, <cd>quite new; that is, as new as a spike or nail just made and a chip just split; brand-new; as, a <xex>spick and span</xex> novelty. See <er>Span-new</er>.</cd></cs>
<-- now hypehnated, and more commonly meaning spotlessly clean -->  <rj><au>Howell.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spicknel</ent><br/
<hw>Spick"nel</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Contr. from <ets>spike nail</ets> a large, long nail; -- so called in allusion to the shape of its capillary leaves.]</ety> <fld>(Bot.)</fld> <def>An umbelliferous herb (<spn>Meum Athamanticum</spn>) having finely divided leaves, common in Europe; -- called also <altname>baldmoney</altname>, <altname>mew</altname>, and <altname>bearwort</altname>.</def> <altsp>[Written also <asp>spignel</asp>.]</altsp><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spicose</ent><br/
<hw>Spi*cose"</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>spica</ets> a spike, or ear.]</ety> <fld>(Bot.)</fld> <def>Having spikes, or ears, like corn spikes.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spicosity</ent><br/
<hw>Spi*cos"i*ty</hw> <pr>(?)</pr>, <pos>n.</pos> <def>The state of having, or being full of, ears like corn.</def> <mark>[R.]</mark>  <rj><au>Bailey.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spicous</ent><br/
<hw>Spi"cous</hw> <pr>(?)</pr>, <pos>a.</pos> <fld>(Bot.)</fld> <def>See <er>Spicose</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spicula</ent><br/
||<hw>Spic"u*la</hw> <pr>(?)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Spiculae</plw> <pr>(#)</pr>.</plu> <ety>[NL., dim. of L. <ets>spica</ets> a spike, ear.]</ety> <fld>(Bot.)</fld> <sd>(a)</sd> <def>A little spike; a spikelet.</def> <sd>(b)</sd> <def>A pointed fleshy appendage.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spicular</ent><br/
<hw>Spic"u*lar</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>spiculum</ets> a dart: cf. F. <ets>spiculaire</ets>.]</ety> <def>Resembling a dart; having sharp points.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spiculate</ent><br/
<hw>Spic"u*late</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>spiculatus</ets>, p. p. of <ets>spiculare</ets> to sharpen, to point, fr. <ets>spiculum</ets> a dart.]</ety> <sn>1.</sn> <def>Covered with, or having, spicules.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Bot.)</fld> <def>Covered with minute spiculae, or pointed fleshy appendages; divided into small spikelets.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spiculate</ent><br/
<hw>Spic"u*late</hw> <pr>(?)</pr>, <pos>v. t.</pos> <def>To sharpen to a point.</def> <mark>[R.]</mark> <ldquo/With <xex>spiculated</xex> paling.<rdquo/  <rj><au>Mason.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spicule</ent><br/
<hw>Spic"ule</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>spiculum</ets> a little point, a dart.]</ety> <sn>1.</sn> <def>A minute, slender granule, or point.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Bot.)</fld> <def>Same as <er>Spicula</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <fld>(Zool.)</fld> <def>Any small calcareous or siliceous body found in the tissues of various invertebrate animals, especially in sponges and in most Alcyonaria.</def><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ Spicules vary exceedingly in size and shape, and some of those found in siliceous sponges are very complex in structure and elegant in form. They are of great use in classification.</note><br/
[<source>1913 Webster</source>]</p>

<p><q><qex>Description of the Illustration</qex>:</q><br/
[<source>1913 Webster</source>]</p>

<p><q><qex>a</qex> Acerate; <qex>b</qex> Tricurvate, <qex>or</qex> Bowshaped; <qex>c d</qex> Hamate; <qex>e</qex> Broomshaped; <qex>f</qex> Scepterellate; <qex>g</qex> Spinispirulate; <qex>h</qex> Inequi-anchorate; <qex>i</qex> Sexradiate; <qex>j</qex> A Trichite Sheaf; <qex>k</qex> Six-rayed Capitate; <qex>l</qex> Rosette of <qex>Esperia</qex>; <qex>m</qex> Equi-anchorate.</q><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spiculiform</ent><br/
<hw>Spi*cu"li*form</hw> <pr>(?)</pr>, <pos>a.</pos> <fld>(Zool.)</fld> <def>Having the shape of a spicule.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spiculigenous</ent><br/
<hw>Spic`u*lig"e*nous</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>spiculum + -genous</ets>.]</ety> <fld>(Zool.)</fld> <def>Producing or containing spicules.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spiculispongiae</ent><br/
||<hw>Spic`u*li*spon"gi*ae</hw> <pr>(?)</pr>, <pos>n. pl.</pos> <ety>[NL.]</ety> <fld>(Zool.)</fld> <def>A division of sponges including those which have independent siliceous spicules.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spiculum</ent><br/
||<hw>Spic"u*lum</hw> <pr>(?)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Spicula</plw> <pr>(#)</pr>.</plu> <ety>[L., a little point.]</ety> <fld>(Zool.)</fld> <def>Same as <er>Spicule</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spicy</ent><br/
<hw>Spi"cy</hw> <pr>(?)</pr>, <pos>a.</pos> <amorph>[<pos>Compar.</pos> <adjf>Spicier</adjf> <pr>(?)</pr>; <pos>superl.</pos> <adjf>Spiciest</adjf>.]</amorph> <ety>[From <er>Spice</er>.]</ety> <sn>1.</sn> <def>Flavored with, or containing, spice or spices; fragrant; aromatic; <as>as, <ex>spicy</ex> breezes</as>.</def> <ldquo/The <xex>spicy</xex> nut-brown ale.<rdquo/  <rj><au>Milton.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Led by new stars, and borne by <qex>spicy</qex> gales.</q> <rj><qau>Pope.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Producing, or abounding with, spices.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>In hot Ceylon <qex>spicy</qex> forests grew.</q> <rj><qau>Dryden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>Fig.: Piquant; racy; <as>as, a <ex>spicy</ex> debate</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><syn><b>Syn.</b> -- Aromatic; fragrant; smart; pungent; pointed; keen. See <er>Racy</er>.</syn><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spider</ent><br/
<hw>Spi"der</hw> <pr>(?)</pr>, <pos>n.</pos><ety>[OE. <ets>spi<thorn/re</ets>, fr. AS. <ets>spinnan</ets> to spin; -- so named from spinning its web; cf. D. <ets>spin</ets> a spider, G. <ets>spinne</ets>, Sw. <ets>spindel</ets>. See <er>Spin</er>.]</ety> <sn>1.</sn> <fld>(Zool.)</fld> <def>Any one of numerous species of arachnids comprising the order Araneina. Spiders have the mandibles converted into poison fangs, or falcers. The abdomen is large and not segmented, with two or three pairs of spinnerets near the end, by means of which they spin threads of silk to form cocoons, or nests, to protect their eggs and young. Many species spin also complex webs to entrap the insects upon which they prey. The eyes are usually eight in number (rarely six), and are situated on the back of the cephalothorax. See <xex>Illust.</xex> under <er>Araneina</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ Spiders are divided into two principal groups: the Dipneumona, having two lungs: and the Tetrapneumona, having four lungs. See <er>Mygale</er>. The former group includes several tribes; as, the jumping spiders (see <er>Saltigradae</er>), the wolf spiders, or <fam>Citigradae</fam> (see under <er>Wolf</er>), the crab spiders, or <fam>Laterigradae</fam> (see under <er>Crab</er>), the garden, or geometric, spiders, or <fam>Orbitellae</fam> (see under <er>Geometrical</er>, and <er>Garden</er>), and others. See <cref>Bird spider</cref>, under <er>Bird</er>, <cref>Grass spider</cref>, under <er>Grass</er>, <cref>House spider</cref>, under <er>House</er>, <cref>Silk spider</cref>, under <er>Silk</er>.</note><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Zool.)</fld> <def>Any one of various other arachnids resembling the true spiders, especially certain mites, as the red spider (see under <er>Red</er>).</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>An iron pan with a long handle, used as a kitchen utensil in frying food. Originally, it had long legs, and was used over coals on the hearth.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>A trevet to support pans or pots over a fire.</def><-- = trivet --><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <fld>(Mach.)</fld> <def>A skeleton, or frame, having radiating arms or members, often connected by crosspieces; as, a casting forming the hub and spokes to which the rim of a fly wheel or large gear is bolted; the body of a piston head; a frame for strengthening a core or mold for a casting, etc.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Spider ant</b></col>. <fld>(Zool.)</fld> <cd>Same as <cref>Solitary ant</cref>, under <er>Solitary</er>.</cd> -- <col><b>Spider crab</b></col> <fld>(Zool.)</fld>, <cd>any one of numerous species of maioid crabs having a more or less triangular body and ten long legs. Some of the species grow to great size, as the great Japanese spider crab (<spn>Macrocheira Kempferi</spn>), measuring sometimes more than fifteen feet across the legs when they are extended.</cd> -- <col><b>Spider fly</b></col> <fld>(Zool.)</fld>, <cd>any one of numerous species of parasitic dipterous insects of the family <fam>Hippoboscidae</fam>. They are mostly destitute of wings, and live among the feathers of birds and the hair of bats. Called also <altname>bird tick</altname>, and <altname>bat tick</altname>.</cd> -- <col><b>Spider hunter</b></col> <fld>(Zool.)</fld>, <cd>any one of several species of East Indian sunbirds of the genus <gen>Arachnothera</gen>.</cd> -- <col><b>Spider lines</b></col>, <cd>filaments of a spider's web crossing the field of vision in optical instruments; -- used for determining the exact position of objects and making delicate measurements. Fine wires, silk fibers, or lines on glass similarly placed, are called <xex>spider lines</xex>.</cd> -- <col><b>Spider mite</b></col>. <fld>(Zool.)</fld> <sd>(a)</sd> <cd>Any one of several species of parasitic mites of the genus <gen>Argas</gen> and allied genera. See <er>Argas</er>.</cd> <sd>(b)</sd> <cd>Any one of numerous small mites injurious to plants.</cd> -- <col><b>Spider monkey</b></col> <fld>(Zool.)</fld>, <cd>any one of numerous species of South American monkeys of the genus <gen>Ateles</gen>, having very long legs and a long prehensile tail.</cd> -- <col><b>Spider orchis</b></col> <fld>(Bot.)</fld>, <cd>a European orchidaceous plant (<spn>Ophrys aranifera</spn>), having flowers which resemble spiders.</cd> -- <col><b>Spider shell</b></col> <fld>(Zool.)</fld>, <cd>any shell of the genus <gen>Pteroceras</gen>. See <er>Pteroceras</er>.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spidered</ent><br/
<hw>Spi"dered</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Infested by spiders; cobwebbed.</def>  <rj><au>Wolcott.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spiderlike</ent><br/
<hw>Spi"der*like`</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Like a spider.</def>  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spider stitch</ent><br/
<hw>Spi"der stitch</hw>. <def>A stitch in lace making used to fill in open spaces with threads resembling a cobweb.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Spider's web</ent><br/
<ent>Spider web</ent><br/
<mhw>{ <hw>Spi"der web"</hw> <pr>(?)</pr>, <it>or</it>  <hw>Spi"der's web"</hw>  }</mhw>. <fld>(Zool.)</fld> <def>The silken web which is formed by most kinds of spiders, particularly the web spun to entrap their prey; -- also called <altname>cobweb</altname>. See <cref>Geometric spider</cref>, <cref>Triangle spider</cref>, under <er>Geometric</er>, and <er>Triangle</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spiderwort</ent><br/
<hw>Spi"der*wort`</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Bot.)</fld> <def>An American endogenous plant (<spn>Tradescantia Virginica</spn>), with long linear leaves and ephemeral blue flowers. The name is sometimes extended to other species of the same genus.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spied</ent><br/
<hw>Spied</hw> <pr>(?)</pr>, <def><pos>imp. & p. p.</pos> of <er>Spy</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spiegeleisen</ent><br/
||<hw>Spie"gel*ei`sen</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[G. <ets>spiegel</ets> mirror + <ets>eisen</ets> iron.]</ety> <def>See <er>Spiegel iron</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spiegel iron</ent><br/
<hw>Spie"gel i`ron</hw> <pr>(?)</pr>. <ety>[G. <ets>spiegel</ets> mirror + E. <ets>iron</ets>.]</ety> <fld>(Metal.)</fld> <def>A fusible white cast iron containing a large amount of carbon (from three and a half to six per cent) and some manganese. When the manganese reaches twenty-five per cent and upwards it has a granular structure, and constitutes the alloy <xex>ferro manganese</xex>, largely used in the manufacture of Bessemer steel. Called also <altname>specular pig iron</altname>, <altname>spiegel</altname>, and <altname>spiegeleisen</altname>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spight</ent><br/
<hw>Spight</hw> <pr>(?)</pr>, <pos>n. & v.</pos> <def>Spite.</def> <mark>[Obs.]</mark>  <rj><au>Spenser.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spight</ent><br/
<hw>Spight</hw>, <pos>n.</pos> <def>A woodpecker. See <er>Speight</er>.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spignel</ent><br/
<hw>Spig"nel</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Bot.)</fld> <def>Same as <er>Spickenel</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spignet</ent><br/
<hw>Spig"net</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Corrupted fr. <ets>spikenard</ets>.]</ety> <fld>(Bot.)</fld> <def>An aromatic plant of America. See <er>Spikenard</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spigot</ent><br/
<hw>Spig"ot</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[From <ets>spick</ets>,or <ets>spike</ets>; cf. Ir. & Gael. <ets>spiocaid</ets> a spigot, Ir. <ets>spice</ets> a spike. See <er>Spike</er>.]</ety> <def>A pin or peg used to stop the vent in a cask; also, the plug of a faucet or cock.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Spigot and faucet joint</b></col>, <cd>a joint for uniting pipes, formed by the insertion of the end of one pipe, or pipe fitting, into a socket at the end of another.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spigurnel</ent><br/
<hw>Spi*gur"nel</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Eng. Law)</fld> <def>Formerly the title of the sealer of writs in chancery.</def>  <rj><au>Mozley & W.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spike</ent><br/
<hw>Spike</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Akin to LG. <ets>spiker</ets>, <ets>spieker</ets>, a large nail, D. <ets>spijker</ets>, Sw. <ets>spik</ets>, Dan. <ets>spiger</ets>, Icel. <ets>sp<imac/k</ets>; all perhaps from L. <ets>spica</ets> a point, an ear of grain; but in the sense of nail more likely akin to E. <ets>spoke</ets> of a wheel.  Cf. <er>Spine</er>.]</ety> <sn>1.</sn> <def>A sort of very large nail; also, a piece of pointed iron set with points upward or outward.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Anything resembling such a nail in shape.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>He wears on his head the <qex>corona radiata</qex> . . . ; the <qex>spikes</qex> that shoot out represent the rays of the sun.</q> <rj><qau>Addison.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>An ear of corn or grain.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <fld>(Bot.)</fld> <def>A kind of flower cluster in which sessile flowers are arranged on an unbranched elongated axis.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Spike grass</b></col> <fld>(Bot.)</fld>, <cd>either of two tall perennial American grasses (<spn>Uniola paniculata</spn>, and <spn>Uniola latifolia</spn>) having broad leaves and large flattened spikelets.</cd> -- <col><b>Spike rush</b></col>. <fld>(Bot.)</fld> <cd>See under <er>Rush</er>.</cd> -- <col><b>Spike shell</b></col> <fld>(Zool.)</fld>, <cd>any pteropod of the genus <gen>Styliola</gen> having a slender conical shell.</cd> -- <col><b>Spike team</b></col>, <cd>three horses, or a horse and a yoke of oxen, harnessed together, a horse leading the oxen or the span.</cd> <mark>[U.S.]</mark></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spike</ent><br/
<hw>Spike</hw>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Spiked</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Spiking</conjf>.]</vmorph> <sn>1.</sn> <def>To fasten with spikes, or long, large nails; <as>as, to <ex>spike</ex> down planks</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To set or furnish with spikes.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>To fix on a spike.</def> <mark>[R.]</mark>  <rj><au>Young.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>To stop the vent of (a gun or cannon) by driving a spike nail, or the like into it.</def><br/
[<source>1913 Webster</source>]</p>

<p><-- (Sport) To throw violently to the ground, so that it bounces.  Sometimes done by football players after scoring a goal, as a victory celebration. --><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spike</ent><br/
<hw>Spike</hw>, <pos>n.</pos> <ety>[Cf. G. <ets>spieke</ets>, L. <ets>spica</ets> an ear of grain. See <er>Spikenard</er>.]</ety> <fld>(Bot.)</fld> <def>Spike lavender. See <er>Lavender</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Oil of spike</b></col> <fld>(Chem.)</fld>, <cd>a colorless or yellowish aromatic oil extracted from the European broad-leaved lavender, or aspic (<spn>Lavendula Spica</spn>), used in artist's varnish and in veterinary medicine. It is often adulterated with oil of turpentine, which it much resembles.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spikebill</ent><br/
<hw>Spike"bill`</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Zool.)</fld> <sd>(a)</sd> <def>The hooded merganser.</def> <sd>(b)</sd> <def>The marbled godwit (<spn>Limosa fedoa</spn>).</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spiked</ent><br/
<hw>Spiked</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Furnished or set with spikes, as corn; fastened with spikes; stopped with spikes.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>A youth, leaping over the <qex>spiked</qex> pales, . . . was caught by those spikes.</q> <rj><qau>Wiseman.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spikefish</ent><br/
<hw>Spike"fish`</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Zool.)</fld> <def>See <er>Sailfish</er> <sd>(a)</sd></def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spikelet</ent><br/
<hw>Spike"let</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Bot.)</fld> <def>A small or secondary spike; especially, one of the ultimate parts of the in florescence of grasses. See <xex>Illust.</xex> of <er>Quaking grass</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spikenard</ent><br/
<hw>Spike"nard</hw> <pr>(?)</pr>, <pos>n.</pos><ety>[For <ets>spiked nard</ets>; cf. G. <ets>spieknarde</ets>, NL. <ets>spica nardi</ets>. See <er>Spike</er> an ear, and <er>Nard</er>.]</ety> <sn>1.</sn> <fld>(Bot.)</fld> <def>An aromatic plant. In the United States it is the <spn>Aralia racemosa</spn>, often called <altname>spignet</altname>, and used as a medicine. The spikenard of the ancients is the <spn>Nardostachys Jatamansi</spn>, a native of the Himalayan region.  From its blackish roots a perfume for the hair is still prepared in India.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A fragrant essential oil, as that from the <spn>Nardostachys Jatamansi</spn>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spiketail</ent><br/
<hw>Spike"tail`</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Zool.)</fld> <def>The pintail duck.</def> <mark>[Local, U.S.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spiky</ent><br/
<hw>Spik"y</hw> <pr>(sp<imac/k"<ycr/)</pr>, <pos>a.</pos> <sn>1.</sn> <def>Like a spike; spikelike.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>These <qex>spiky</qex>, vivid outbursts of metallic vapors.</q> <rj><qau>C. A. Young.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Having a sharp point, or sharp points; furnished or armed with spikes.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Or by the <qex>spiky</qex> harrow cleared away.</q> <rj><qau>Dyer.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>The <qex>spiky</qex> wheels through heaps of carnage tore.</q> <rj><qau>Pope.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spile</ent><br/
<hw>Spile</hw> <pr>(sp<imac/l)</pr>, <pos>n.</pos> <ety>[Cf. LG. <ets>spile</ets>, dial. G. <ets>speil</ets>, <ets>speiler</ets>, D. <ets>spijl</ets>. <root/170.]</ety> <sn>1.</sn> <def>A small plug or wooden pin, used to stop a vent, as in a cask.</def><br/
[<source>1913 Webster</source>]</p>

<p><-- p. 1386 --></p>

<p><sn>2.</sn> <def>A small tube or spout inserted in a tree for conducting sap, as from a sugar maple.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>A large stake driven into the ground as a support for some superstructure; a pile.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <fld>(Mining)</fld> <def>One of the thick laths or poles driven horizontally ahead on top of a set of the main timbering in advancing a level in loose ground.</def><br/
<syn><b>Syn. --</b> forepole; spill{2d}.</syn><br/
[<source>RDH</source>]</p>

<p><cs><col><b>Spile hole</b></col>, <cd>a small air hole in a cask; a vent.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spile</ent><br/
<hw>Spile</hw> <pr>(sp<imac/l)</pr>, <pos>v. t.</pos> <def>To supply with a spile or a spigot; to make a small vent in, as a cask.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spilikin</ent><br/
<hw>Spil"i*kin</hw> <pr>(sp<icr/l"<icr/*k<icr/n)</pr>, <pos>n.</pos> <ety>[OD. <ets>spelleken</ets> a small pin. See <er>Spill</er> a splinter.]</ety> <def>One of a number of small pieces or pegs of wood, ivory, bone, or other material, for playing a game, or for counting the score in a game, as in cribbage.  In the plural (<plw><it>spilikins</it></plw>), a game played with such pieces; pushpin.</def> <altsp>[Written also <asp>spillikin</asp>, <asp>spilliken</asp>.]</altsp><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spill</ent><br/
<hw>Spill</hw> <pr>(sp<icr/l)</pr>, <pos>n.</pos> <ety>[<root/170.  Cf. <er>Spell</er> a splinter.]</ety> <sn>1.</sn> <def>A bit of wood split off; a splinter.</def> <mark>[Obs. or Prov. Eng.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A slender piece of anything.</def> Specifically: --<br/
[<source>1913 Webster</source>]</p>

<p><sd>(a)</sd> <def>A peg or pin for plugging a hole, as in a cask; a spile.</def><br/
[<source>1913 Webster</source>]</p>

<p><sd>(b)</sd> <def>A metallic rod or pin.</def><br/
[<source>1913 Webster</source>]</p>

<p><sd>(c)</sd> <def>A small roll of paper, or slip of wood, used as a lamplighter, etc.</def><br/
[<source>1913 Webster</source>]</p>

<p><sd>(d)</sd> <fld>(Mining)</fld> <def>One of the thick laths or poles driven horizontally ahead on top of a set of the main timbering in advancing a level in loose ground.</def><br/
<syn><b>Syn. --</b> forepole; spile{4}.</syn><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>A little sum of money.</def> <mark>[Obs.]</mark>  <rj><au>Ayliffe.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spill</ent><br/
<hw>Spill</hw>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Spilt</conjf> <pr>(sp<icr/lt)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Spilling</conjf>.]</vmorph> <def>To cover or decorate with slender pieces of wood, metal, ivory, etc.; to inlay.</def> <mark>[Obs.]</mark>  <rj><au>Spenser.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spill</ent><br/
<hw>Spill</hw> <pr>(sp<icr/l)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Spilled</conjf> <pr>(sp<icr/ld)</pr>, or <conjf>Spilt</conjf> <pr>(sp<icr/lt)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Spilling</conjf>.]</vmorph> <ety>[OE. <ets>spillen</ets>, usually, to destroy, AS. <ets>spillan</ets>, <ets>spildan</ets>, to destroy; akin to Icel. <ets>spilla</ets> to destroy, Sw. <ets>spilla</ets> to spill, Dan. <ets>spilde</ets>, LG. & D. <ets>spillen</ets> to squander, OHG. <ets>spildan</ets>.]</ety> <sn>1.</sn> <def>To destroy; to kill; to put an end to.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>And gave him to the queen, all at her will<br/
To choose whether she would him save or <qex>spill</qex>.</q> <rj><qau>Chaucer.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Greater glory think [it] to save than <qex>spill</qex>.</q> <rj><qau>Spenser.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To mar; to injure; to deface; hence, to destroy by misuse; to waste.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>They [the colors] disfigure the stuff and <qex>spill</qex> the whole workmanship.</q> <rj><qau>Puttenham.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q><qex>Spill</qex> not the morning, the quintessence of day, in recreations.</q> <rj><qau>Fuller.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>To suffer to fall or run out of a vessel; to lose, or suffer to be scattered; -- applied to fluids and to substances whose particles are small and loose; <as>as, to <ex>spill</ex> water from a pail; to <ex>spill</ex> quicksilver from a vessel; to <ex>spill</ex> powder from a paper; to <ex>spill</ex> sand or flour</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ <xex>Spill</xex> differs from <xex>pour</xex> in expressing accidental loss, -- a loss or waste contrary to purpose.</note><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>To cause to flow out and be lost or wasted; to shed, or suffer to be shed, as in battle or in manslaughter; <as>as, a man <ex>spills</ex> another's blood, or his own blood</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>And to revenge his blood so justly <qex>spilt</qex>.</q> <rj><qau>Dryden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <fld>(Naut.)</fld> <def>To relieve a sail from the pressure of the wind, so that it can be more easily reefed or furled, or to lessen the strain.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Spilling line</b></col> <fld>(Naut.)</fld>, <cd>a rope used for spilling, or dislodging, the wind from the belly of a sail.</cd>  <rj><au>Totten.</au></rj></cs><br/
[<source>1913 Webster</source>]</p>

<p><-- Spill, n. An instance of spilling.
  Oil spill, an accidental release of oil, usually into the ocean, due to damage to an oil tanker or uncontrolled release from an underwater well. --><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spill</ent><br/
<hw>Spill</hw>, <pos>v. i.</pos> <sn>1.</sn> <def>To be destroyed, ruined, or wasted; to come to ruin; to perish; to waste.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>That thou wilt suffer innocents to <qex>spill</qex>.</q> <rj><qau>Chaucer.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To be shed; to run over; to fall out, and be lost or wasted.</def> <ldquo/He was so topful of himself, that he let it <xex>spill</xex> on all the company.<rdquo/  <rj><au>I. Watts.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spiller</ent><br/
<hw>Spill"er</hw> <pr>(?)</pr>, <pos>n.</pos> <sn>1.</sn> <def>One who, or that which, spills.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A kind of fishing line with many hooks; a boulter.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spilliard fishing</ent><br/
<ent>Spillet fishing</ent><br/
<mhw>{ <hw>Spil"let fish`ing</hw> <pr>(?)</pr>, <hw>Spil"liard fish`ing</hw> <pr>(?)</pr>, }</mhw> <def>A system or method of fishing by means of a number of hooks set on snoods all on one line; -- in North America, called <altname>trawl fishing</altname>, <altname>bultow</altname>, or <altname>bultow fishing</altname>, and <altname>long-line fishing</altname>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spillikin</ent><br/
<hw>Spil"li*kin</hw> <pr>(?)</pr>, <pos>n.</pos> <def>See <er>Spilikin</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spillway</ent><br/
<hw>Spill"way`</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A sluiceway or passage for superfluous water in a reservoir, to prevent too great pressure on the dam.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spilt</ent><br/
<hw>Spilt</hw> <pr>(?)</pr>, <def><pos>imp. & p. p.</pos> of <er>Spill</er>. Spilled.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spilter</ent><br/
<hw>Spil"ter</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[From <er>Spill</er>, <pos>n.</pos>]</ety> <def>Any one of the small branches on a stag's head.</def> <mark>[Obs.]</mark>  <rj><au>Howell.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spilth</ent><br/
<hw>Spilth</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[From <er>Spill</er>.]</ety> <def>Anything spilt, or freely poured out; slop; effusion.</def> <mark>[Archaic]</mark> <ldquo/With drunken <xex>spilth</xex> of wine.<rdquo/  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Choicest cates, and the flagon's best <qex>spilth</qex>.</q> <rj><qau>R. Browning.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spin</ent><br/
<hw>Spin</hw> <pr>(sp<icr/n)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Spun</conjf> <pr>(?)</pr> (<mark>Archaic</mark> <pos>imp.</pos> <conjf>Span</conjf> <pr>(<?/)</pr>); <pos>p. pr. & vb. n.</pos> <conjf>Spinning</conjf>.]</vmorph> <ety>[AS. <ets>spinnan</ets>; akin to D. & G. <ets>spinnen</ets>, Icel. & Sw. <ets>spinna</ets>, Dan. <ets>spinde</ets>, Goth. <ets>spinnan</ets>, and probably to E. <ets>span</ets>. <root/170.  Cf. <er>Span</er>, <pos>v. t.</pos>, <er>Spider</er>.]</ety> <sn>1.</sn> <def>To draw out, and twist into threads, either by the hand or machinery; <as>as, to <ex>spin</ex> wool, cotton, or flax; to <ex>spin</ex> goat's hair</as>; to produce by drawing out and twisting a fibrous material.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>All the yarn she [Penelope] <qex>spun</qex> in Ulysses' absence did but fill Ithaca full of moths.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To draw out tediously; to form by a slow process, or by degrees; to extend to a great length; -- with <xex>out</xex>; <as>as, to <ex>spin</ex> out large volumes on a subject</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Do you mean that story is tediously <qex>spun</qex> out?</q> <rj><qau>Sheridan.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>To protract; to spend by delays; <as>as, to <ex>spin</ex> out the day in idleness</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>By one delay after another they <qex>spin</qex> out their whole lives.</q> <rj><qau>L'Estrange.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>To cause to turn round rapidly; to whirl; to twirl; <as>as, to <ex>spin</ex> a top</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <def>To form (a web, a cocoon, silk, or the like) from threads produced by the extrusion of a viscid, transparent liquid, which hardens on coming into contact with the air; -- said of the spider, the silkworm, etc.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>6.</sn> <fld>(Mech.)</fld> <def>To shape, as malleable sheet metal, into a hollow form, by bending or buckling it by pressing against it with a smooth hand tool or roller while the metal revolves, as in a lathe.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>To spin a yarn</b></col> <fld>(Naut.)</fld>, <cd>to tell a story, esp. a long or fabulous tale.</cd> -- <col><b>To spin hay</b></col> <fld>(Mil.)</fld>, <cd>to twist it into ropes for convenient carriage on an expedition.</cd> -- <col><b>To spin street yarn</b></col>, <cd>to gad about gossiping.</cd> <mark>[Collog.]</mark></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spin</ent><br/
<hw>Spin</hw> <pr>(?)</pr>, <pos>v. i.</pos> <sn>1.</sn> <def>To practice spinning; to work at drawing and twisting threads; to make yarn or thread from fiber; <as>as, the woman knows how to <ex>spin</ex>; a machine or jenny <ex>spins</ex> with great exactness.</as></def><br/
[<source>1913 Webster</source>]</p>

<p><q>They neither know to <qex>spin</qex>, nor care to toll.</q> <rj><qau>Prior.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To move round rapidly; to whirl; to revolve, as a top or a spindle, about its axis.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Round about him <qex>spun</qex> the landscape,<br/
Sky and forest reeled together.</q> <rj><qau>Longfellow.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>With a whirligig of jubilant mosquitoes <qex>spinning</qex> about each head.</q> <rj><qau>G. W. Cable.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>To stream or issue in a thread or a small current or jet; <as>as, blood <ex>spins</ex>from a vein</as>.</def>  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>To move swifty; <as>as, to <ex>spin</ex> along the road in a carriage, on a bicycle, etc.</as></def> <mark>[Colloq.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spin</ent><br/
<hw>Spin</hw>, <pos>n.</pos> <sn>1.</sn> <def>The act of spinning; <as>as, the <ex>spin</ex> of a top; a <ex>spin</ex> a bicycle</as>.</def> <mark>[Colloq.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Kinematics)</fld> <def>Velocity of rotation about some specified axis.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <fld>(Politics)</fld> <def>an interpretation of an event which is favorable to the interpreter or to the person s/he supports.  A person whose task is to provide such interpretations for public relations purposes is called a <er>spin doctor</er>.</def><br/
[<source>PJC</source>]</p>

<p><-- <col><b>go for a spin</b></col>, <col><b>take a spin</b></col>, <cd>take a trip in a wheeled vehicle, usu. an automobile.</cd>  --><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spina bifida</ent><br/
||<hw>Spi"na bif"i*da</hw> <pr>(?)</pr>. <fld>(Med.)</fld> <ety>[L., cleft spine.]</ety> <def>A congenital malformation in which the spinal column is cleft at its lower portion, and the membranes of the spinal cord project as an elastic swelling from the gap thus formed.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spinaceous</ent><br/
<hw>Spi*na"ceous</hw> <pr>(?)</pr>, <pos>a.</pos> <fld>(Bot.)</fld> <def>Of, pertaining to, or resembling, the plant spinach, or the family of plants to which it belongs.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spinage</ent><br/
<ent>Spinach</ent><br/
<mhw>{ <hw>Spin"ach</hw>, <hw>Spin"age</hw>  }</mhw> <pr>(?)</pr>, <pos>n.</pos> <ety>[OF. <ets>espinache</ets>, <ets>espinoche</ets>, F. <ets><eacute/pinard</ets>; cf. F. <ets>spinace</ets>, Sp. <ets>espinaca</ets>; all fr. Ar. <ets>isf<amac/n<amac/j</ets>, <ets>isfin<amac/j</ets>, <ets>aspan<amac/kh</ets>, probably of Persian origin.]</ety> <fld>(Bot.)</fld> <def>A common pot herb (<spn>Spinacia oleracea</spn>) belonging to the Goosefoot family.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Mountain spinach</b></col>. <cd>See <cref>Garden orache</cref>, under <er>Orache</er>.</cd> -- <col><b>New Zealand spinach</b></col> <fld>(Bot.)</fld>, <cd>a coarse herb (<spn>Tetragonia expansa</spn>), a poor substitute for spinach.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ Various other pot herbs are locally called <xex>spinach</xex>.</note><br/
[<source>1913 Webster</source>]</p>

<p><ent>spinach beet</ent><br/
<hw>spinach beet</hw> <pos>n.</pos> <sn>1.</sn>  <def>a beet lacking a swollen root; it is grown as a vegetable for its edible leaves and stalks.</def><br/
<syn><b>Syn. --</b> chard, Swiss chard, spinach beet, leaf beet, chard plant, Beta vulgaris cicla.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><b>2.</b> <def>a plant with long succulent whitish stalks with large green leaves.</def><br/
<syn><b>Syn. --</b> chard, Swiss chard, spinach beet, leaf beet.</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>spinach blight</ent><br/
<hw>spinach blight</hw> <pos>n.</pos> <def>a disease of spinach plants.</def><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Spinal</ent><br/
<hw>Spi"nal</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>spinalis</ets>, fr. <ets>spina</ets> the spine: cf. F. <ets>spinal</ets>. See <er>Spine</er>.]</ety> <sn>1.</sn> <fld>(Anat.)</fld> <def>Of, pertaining to, or in the region of, the backbone, or vertebral column; rachidian; vertebral.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Of or pertaining to a spine or spines.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Spinal accessory nerves</b></col>, <cd>the eleventh pair of cranial nerves in the higher vertebrates. They originate from the spinal cord and pass forward into the skull, from which they emerge in company with the pneumogastrics.</cd> -- <col><b>Spinal column</b></col>, <cd>the backbone, or connected series or vertebrae which forms the axis of the vertebrate skeleton; the spine; rachis; vertebral column.</cd> -- <col><b>Spinal cord</b></col>, <cd>the great nervous cord extending backward from the brain along the dorsal side of the spinal column of a vertebrate animal, and usually terminating in a threadlike appendage called the <xex>filum terminale</xex>; the spinal, or vertebral, marrow; the myelon. The nervous tissue consists of nerve fibers and nerve cells, the latter being confined to the so-called <xex>gray matter</xex> of the central portions of the cord, while the peripheral <xex>white matter</xex> is composed of nerve fibers only. The center of the cord is traversed by a slender canal connecting with the ventricles of the brain.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spinate</ent><br/
<hw>Spi"nate</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Bearing a spine; spiniform.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spindle</ent><br/
<hw>Spin"dle</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[AS. <ets>spinal</ets>, fr. <ets>spinnan</ets> to spin; akin to D. <ets>spil</ets>, G. <ets>spille</ets>, <ets>spindel</ets>, OHG. <ets>spinnala</ets>. <root/170. See <er>Spin</er>.]</ety> <sn>1.</sn> <def>The long, round, slender rod or pin in spinning wheels by which the thread is twisted, and on which, when twisted, it is wound; also, the pin on which the bobbin is held in a spinning machine, or in the shuttle of a loom.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A slender rod or pin on which anything turns; an axis; <as>as, the <ex>spindle</ex> of a vane</as>.</def> Specifically: --<br/
[<source>1913 Webster</source>]</p>

<p><sd>(a)</sd> <fld>(Mach.)</fld> <def>The shaft, mandrel, or arbor, in a machine tool, as a lathe or drilling machine, etc., which causes the work to revolve, or carries a tool or center, etc.</def><br/
[<source>1913 Webster</source>]</p>

<p><sd>(b)</sd> <fld>(Mach.)</fld> <def>The vertical rod on which the runner of a grinding mill turns.</def><br/
[<source>1913 Webster</source>]</p>

<p><sd>(c)</sd> <fld>(Founding)</fld> <def>A shaft or pipe on which a core of sand is formed.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>The fusee of a watch.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>A long and slender stalk resembling a spindle.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <def>A yarn measure containing, in cotton yarn, 15,120 yards; in linen yarn, 14,400 yards.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>6.</sn> <fld>(Geom.)</fld> <def>A solid generated by the revolution of a curved line about its base or double ordinate or chord.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>7.</sn> <fld>(Zool.)</fld> <sd>(a)</sd> <def>Any marine univalve shell of the genus <gen>Rostellaria</gen>; -- called also <altname>spindle stromb</altname>.</def> <sd>(b)</sd> <def>Any marine gastropod of the genus <gen>Fusus</gen>.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Dead spindle</b></col> <fld>(Mach.)</fld>, <cd>a spindle in a machine tool that does not revolve; the spindle of the tailstock of a lathe.</cd> -- <col><b>Live spindle</b></col> <fld>(Mach.)</fld>, <cd>the revolving spindle of a machine tool; the spindle of the headstock of a turning lathe.</cd> -- <col><b>Spindle shell</b></col>. <fld>(Zool.)</fld> <cd>See <er>Spindle</er>, 7. above.</cd> -- <col><b>Spindle side</b></col>, <cd>the female side in descent; in the female line; opposed to <ant>spear side</ant>.</cd> <au>Ld. Lytton.</au> <mark>[R.]</mark> <ldquo/King Lycaon, grandson, by the <xex>spindle side</xex>, of Oceanus.<rdquo/ <au>Lowell.</au> -- <col><b>Spindle tree</b></col> <fld>(Bot.)</fld>, <cd>any shrub or tree of the genus <gen>Eunymus</gen>. The wood of <spn>Eunymus Europaeus</spn> was used for spindles and skewers. See <er>Prickwood</er>.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spindle</ent><br/
<hw>Spin"dle</hw>, <pos>v. i.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Spindled</conjf><pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Spindling</conjf> <pr>(?)</pr>.]</vmorph> <def>To shoot or grow into a long, slender stalk or body; to become disproportionately tall and slender.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>It has begun to <qex>spindle</qex> into overintellectuality.</q> <rj><qau>Lowell.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spindle-legged</ent><br/
<hw>Spin"dle-legged`</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Having long, slender legs.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spindlelegs</ent><br/
<hw>Spin"dle*legs`</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A spindleshanks.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spindle-shanked</ent><br/
<hw>Spin"dle-shanked`</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Having long, slender legs.</def>  <rj><au>Addison.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spindleshanks</ent><br/
<hw>Spin"dle*shanks`</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A person with slender shanks, or legs; -- used humorously or in contempt.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spindle-shaped</ent><br/
<hw>Spin"dle-shaped`</hw> <pr>(?)</pr>, <pos>a.</pos> <sn>1.</sn> <def>Having the shape of a spindle.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Bot.)</fld> <def>Thickest in the middle, and tapering to both ends; fusiform; -- applied chiefly to roots.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spindletail</ent><br/
<hw>Spin"dle*tail`</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Zool.)</fld> <def>The pintail duck.</def> <mark>[Local, U.S.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spindleworm</ent><br/
<hw>Spin"dle*worm`</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Zool.)</fld> <def>The larva of a noctuid moth (<spn>Achatodes zeae</spn>) which feeds inside the stalks of corn (maize), sometimes causing much damage. It is smooth, with a black head and tail and a row of black dots across each segment.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spindling</ent><br/
<hw>Spin"dling</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Long and slender, or disproportionately tall and slender; <as>as, a <ex>spindling</ex> tree; a <ex>spindling</ex> boy.</as></def><br/
<syn><b>Syn. --</b> spindly.</syn>
[<source>1913 Webster</source>]</p>

<p><ent>Spindly</ent><br/
<hw>Spin"dly</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Long and slender, or disproportionately tall and slender; <as>as, <ex>spindly</ex> legs.</as>.  Used in some cases to suggest weakness.</def><br/
<syn><b>Syn. --</b> spindling.</syn>
[<source>PJC</source>]</p>

<p><ent>spin doctor</ent><br/
<hw>spin doctor</hw> <pos>n.</pos> <def>a spokesperson for a political party or candidate who tries to forestall negative publicity.</def><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>spin dryer</ent><br/
<ent>spin drier</ent><br/
<mhw><hw>spin drier</hw>, <hw>spin dryer</hw></mhw> <pos>n.</pos> <def>a machine that uses centrifugal motion to dry the clothes that are put into it.</def><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Spindrift</ent><br/
<hw>Spin"drift</hw> <pr>(?)</pr>, <pos>n.</pos> <def>Same as <er>Spoondrift</er>.</def></p>

<p><q>The ocean waves are broken up by wind, ultimately producing the storm wrack and <qex>spindrift</qex> of the tempest-tossed sea.</q>  <rj><qau>J. E. Marr.</qau></rj><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Spine</ent><br/
<hw>Spine</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>spina</ets> thorn, the spine; akin to <ets>spica</ets> a point: cf. OF. <ets>espine</ets>, F. <ets><eacute/pine</ets>.  Cf. <er>Spike</er>, <er>Spinet</er> a musical instrument, <er>Spinny</er>.]</ety> <sn>1.</sn> <fld>(Bot.)</fld> <def>A sharp appendage to any of a plant; a thorn.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Zool.)</fld> <sd>(a)</sd> <def>A rigid and sharp projection upon any part of an animal.</def> <sd>(b)</sd> <def>One of the rigid and undivided fin rays of a fish.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <fld>(Anat.)</fld> <def>The backbone, or spinal column, of an animal; -- so called from the projecting processes upon the vertebrae.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>Anything resembling the spine or backbone; a ridge.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spineback</ent><br/
<hw>Spine"back`</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Zool.)</fld> <def>A fish having spines in, or in front of, the dorsal fins.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spinebill</ent><br/
<hw>Spine"bill`</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Zool.)</fld> <def>Any species of Australian birds of the genus <gen>Acanthorhynchus</gen>. They are related to the honey eaters.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spined</ent><br/
<hw>Spined</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Furnished with spines; spiny.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spine-finned</ent><br/
<hw>Spine"-finned`</hw> <pr>(?)</pr>, <pos>a.</pos> <fld>(Zool.)</fld> <def>Having fine supported by spinous fin rays; -- said of certain fishes.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spinelle</ent><br/
<ent>Spinel</ent><br/
<mhw>{ <hw>Spi*nel"</hw> <pr>(?)</pr>, <hw>Spi*nelle"</hw> <pr>(?)</pr>, }</mhw> <pos>n.</pos> <ety>[F. <ets>spinelle</ets>, or LL. <ets>spinellus</ets>, perhaps from L. <ets>spina</ets> a thorn, a prickle, in allusion to its pointed crystals.]</ety> <fld>(Min.)</fld> <def>A mineral occuring in octahedrons of great hardness and various colors, as red, green, blue, brown, and black, the red variety being the gem spinel ruby. It consist essentially of alumina and magnesia, but commonly contains iron and sometimes also chromium.</def><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ The <xex>spinel group</xex> includes spinel proper, also magnetite, chromite, franklinite, gahnite, etc., all of which may be regarded as composed of a sesquioxide and a protoxide in equal proportions.</note><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spinel</ent><br/
<hw>Spin"el</hw> <pr>(?)</pr>, <pos>n.</pos> <def>Bleached yarn in making the linen tape called <xex>inkle</xex>; unwrought inkle.</def>  <rj><au>Knight.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spineless</ent><br/
<hw>Spine"less</hw> <pr>(?)</pr>, <pos>a.</pos> <sn>1.</sn> <def>Having no spine, or vertebral column.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Having no thorns or barbs.</def><br/
[<source>PJC</source>]</p>

<p><sn>3.</sn> <def>lacking in strength of character; cowardly, in a physical or moral sense.</def><br/
[<source>PJC</source>]</p>

<p><ent>Spinescence</ent><br/
<hw>Spi*nes"cence</hw> <pr>(?)</pr>, <pos>n.</pos> <def>The state or quality of being spinescent or spiny; also, a spiny growth or covering, as of certain animals.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Spinescent</ent><br/
<hw>Spi*nes"cent</hw> <pr>(?)</pr>, <pos>a.</pos><ety>[L. <ets>spinescens</ets>, <ets>-entis</ets>, p. pr. of <ets>spinescere</ets> to know to grow thorny, fr. <ets>spina</ets> a thorn: cf. F. <ets>spinescent</ets>.]</ety> <fld>(Bot.)</fld> <def>Becoming hard and thorny; tapering gradually to a rigid, leafless point; armed with spines.</def>  <rj><au>Gray.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spinet</ent><br/
<hw>Spin"et</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[OF. <ets>espinete</ets>, F. <ets><eacute/pinette</ets> (cf. It. <ets>spinetta</ets>), fr. L. <ets>spina</ets> a thorn; -- so called because its quills resemble thorns. See <er>Spine</er>.]</ety> <fld>(Mus.)</fld> <def>A keyed instrument of music resembling a harpsichord, but smaller, with one string of brass or steel wire to each note, sounded by means of leather or quill plectrums or jacks. It was formerly much used.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Dumb spinet</b></col>. <fld>(Mus.)</fld> <cd>See <er>Manichordon</er>.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spinet</ent><br/
<hw>Spi"net</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>spinetum</ets>. See <er>Spinny</er>.]</ety> <def>A spinny.</def> <mark>[Obs.]</mark>  <rj><au>B. Jonson.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spinetail</ent><br/
<hw>Spine"tail`</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Zool.)</fld> <sd>(a)</sd> <def>Any one or several species of swifts of the genus <gen>Acanthylis</gen>, or <gen>Chaetura</gen>, and allied genera, in which the shafts of the tail feathers terminate in rigid spines.</def> <sd>(b)</sd> <def>Any one of several species of South American and Central American clamatorial birds belonging to <gen>Synallaxis</gen> and allied genera of the family <fam>Dendrocolaptidae</fam>. They are allied to the ovenbirds.</def> <sd>(c)</sd> <def>The ruddy duck.</def> <mark>[Local, U.S.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spine-tailed</ent><br/
<hw>Spine"-tailed</hw> <pr>(?)</pr>, <pos>a.</pos> <fld>(Zool.)</fld> <def>Having the tail quills ending in sharp, naked tips.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Spine-tailed swift</b></col>. <fld>(Zool.)</fld> <cd>See <er>Spinetail</er> <sd>(a)</sd>.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spineted</ent><br/
<hw>Spin"et*ed</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Slit; cleft.</def> <mark>[Obs. & R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spiniferous</ent><br/
<hw>Spi*nif"er*ous</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>spinifer</ets>; <ets>spina</ets> thorn + <ets>ferre</ets> to produce.]</ety> <def>Producing spines; bearing thorns or spines; thorny; spiny.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spinifex</ent><br/
<hw>Spin"i*fex</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[NL.; L. <ets>spina</ets> spine + <ets>facere</ets> to make.]</ety> <sn>1.</sn> <fld>(Bot.)</fld> <def>A genus of chiefly Australian grasses, the seeds of which bear an elastic spine. <spn>Spinifex hirsutus</spn> (black grass) and <spn>Spinifex longifolius</spn> are useful as sand binders. <spn>Spinifex paradoxus</spn>is a valuable perennial fodder plant. Also, a plant of this genus.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><sn>2.</sn>  <def>Any of several Australian grasses of the genus <gen>Tricuspis</gen>, which often form dense, almost impassable growth, their leaves being stiff and sharp-pointed.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Spiniform</ent><br/
<hw>Spin"i*form</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Shaped like a spine.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spinigerous</ent><br/
<hw>Spi*nig"er*ous</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>spiniger</ets>; <ets>spina</ets> spine + <ets>gerere</ets> to bear.]</ety> <def>Bearing a spine or spines; thorn-bearing.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spininess</ent><br/
<hw>Spin"i*ness</hw> <pr>(?)</pr>, <pos>n.</pos> <def>Quality of being spiny.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spini-spirulate</ent><br/
<hw>Spin`i-spir"u*late</hw> <pr>(?)</pr>, <pos>a.</pos> <fld>(Zool.)</fld> <def>Having spines arranged spirally. See <er>Spicule</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spink</ent><br/
<hw>Spink</hw> <pr>(sp<icr/<nsm/k)</pr>, <pos>n.</pos> <ety>[Cf. dial. Sw. <ets>spink</ets> a kind of small bird, Gr. <grk>spi`ggos</grk>, and E. <ets>finch</ets>.]</ety> <fld>(Zool.)</fld> <def>The chaffinch.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spinnaker</ent><br/
<hw>Spin"na*ker</hw> <pr>(sp<icr/n"n<adot/*k<etil/r)</pr>, <pos>n.</pos> <fld>(Naut.)</fld> <def>A large triangular sail set upon a boom, -- used when running before the wind.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spinner</ent><br/
<hw>Spin"ner</hw> <pr>(sp<icr/n"n<etil/r)</pr>, <pos>n.</pos> <sn>1.</sn> <def>One who, or that which, spins one skilled in spinning; a spinning machine.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A spider.</def> <ldquo/Long-legged <xex>spinners</xex>.<rdquo/  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <fld>(Zool.)</fld> <def>A goatsucker; -- so called from the peculiar noise it makes when darting through the air.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <fld>(Zool.)</fld> <def>A spinneret.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Ring spinner</b></col>, <cd>a machine for spinning, in which the twist, given to the yarn by a revolving bobbin, is regulated by the drag of a small metal loop which slides around a ring encircling the bobbin, instead of by a throstle.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spinneret</ent><br/
<hw>Spin"ner*et</hw> <pr>(sp<icr/n"n<etil/r*<ecr/t)</pr>, <pos>n.</pos> <fld>(Zool.)</fld> <def>One of the special jointed organs situated on the under side, and near the end, of the abdomen of spiders, by means of which they spin their webs. Most spiders have three pairs of spinnerets, but some have only two pairs. The ordinary silk line of the spider is composed of numerous smaller lines jointed after issuing from the spinnerets.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spinnerule</ent><br/
<hw>Spin"ner*ule</hw> <pr>(sp<icr/n"n<etil/r*<usl/l)</pr>, <pos>n.</pos> <fld>(Zool.)</fld> <def>One of the numerous small spinning tubes on the spinnerets of spiders.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spinney</ent><br/
<hw>Spin"ney</hw> <pr>(sp<icr/n"n<ycr/)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Spinneys</plw> <pr>(sp<icr/n"n<icr/z)</pr>.</plu> <def>Same as <er>Spinny</er>.</def>  <rj><au>T. Hughes.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><-- p. 1387 --></p>

<p><ent>Spinning</ent><br/
<hw>Spin"ning</hw> <pr>(?)</pr>, <def><pos>a. & n.</pos> from <er>Spin</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Spinning gland</b></col> <fld>(Zool.)</fld>, <cd>one of the glands which form the material for spinning the silk of silkworms and other larvae.</cd> -- <col><b>Spinning house</b></col>, <cd>formerly a common name for a house of correction in England, the women confined therein being employed in spinning.</cd> -- <col><b>Spinning jenny</b></col> <fld>(Mach.)</fld>, <cd>an engine or machine for spinning wool or cotton, by means of a large number of spindles revolving simultaneously.</cd> -- <col><b>Spinning mite</b></col> <fld>(Zool.)</fld>, <cd>the red spider.</cd> -- <col><b>Spinning wheel</b></col>, <cd>a machine for spinning yarn or thread, in which a wheel drives a single spindle, and is itself driven by the hand, or by the foot acting on a treadle.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spinny</ent><br/
<hw>Spin"ny</hw> <pr>(?)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Spinnies</plw> <pr>(#)</pr>.</plu> <ety>[OF. <ets>espinaye</ets>,<ets>espinoye</ets>, <ets>espinei</ets>, <ets>espanoi</ets>, F. <ets><eacute/pinaie</ets>, from L. <ets>spinetum</ets> a thicket of thorns, fr. <ets>spina</ets> a thorn. See <er>Spine</er>.]</ety> <def>A small thicket or grove with undergrowth; a clump of trees.</def> <altsp>[Written also <asp>spinney</asp>, and <asp>spinny</asp>.]</altsp><br/
[<source>1913 Webster</source>]</p>

<p><q>The downs rise steep, crowned with black fir <qex>spinnies</qex>.</q> <rj><qau>C. Kingsley.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spinny</ent><br/
<hw>Spin"ny</hw>, <pos>a.</pos> <ety>[Cf. <er>Spiny</er>, <pos>a.</pos>]</ety> <def>Thin and long; slim; slender.</def> <mark>[Obs. or Prov. Eng.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spinose</ent><br/
<hw>Spi*nose"</hw> <pr>(?)</pr>, <pos>a.</pos><ety>[L. <ets>spinosus</ets>, from <ets>spina</ets> a thorn.]</ety> <def>Full of spines; armed with thorns; thorny.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spinosity</ent><br/
<hw>Spi*nos"i*ty</hw> <pr>(?)</pr>, <pos>n.</pos> <def>The quality or state of being spiny or thorny; spininess.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spinous</ent><br/
<hw>Spi"nous</hw> <pr>(?)</pr>, <pos>a.</pos> <sn>1.</sn> <def>Spinose; thorny.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Having the form of a spine or thorn; spinelike.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Spinous process of a vertebra</b></col> <fld>(Anat.)</fld>, <cd>the dorsal process of the neural arch of a vertebra; a neurapophysis.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spinozism</ent><br/
<hw>Spi*no"zism</hw> <pr>(?)</pr>, <pos>n.</pos> <def>The form of Pantheism taught by Benedict <xex>Spinoza</xex>, that there is but one substance, or infinite essence, in the universe, of which the so-called material and spiritual beings and phenomena are only modes, and that one this one substance is God.</def> <altsp>[Written also <asp>Spinosism</asp>.]</altsp><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spinozist</ent><br/
<hw>Spi*no"zist</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A believer in Spinozism.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spinster</ent><br/
<hw>Spin"ster</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[<ets>Spin</ets> + <ets>-ster</ets>.]</ety> <sn>1.</sn> <def>A woman who spins, or whose occupation is to spin.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>She spake to <qex>spinster</qex> to spin it out.</q> <rj><qau>Piers Plowman.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>The <qex>spinsters</qex> and the knitters in the sun.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A man who spins.</def> <mark>[Obs.]</mark>  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <fld>(Law)</fld> <def>An unmarried or single woman; -- used in legal proceedings as a title, or addition to the surname.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>If a gentlewoman be termed a <qex>spinster</qex>, she may abate the writ.</q> <rj><qau>Coke.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>A woman of evil life and character; -- so called from being forced to spin in a house of correction.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spinstress</ent><br/
<hw>Spin"stress</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A woman who spins.</def>  <rj><au>T. Brown.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spinstry</ent><br/
<hw>Spin"stry</hw> <pr>(?)</pr>, <pos>n.</pos> <def>The business of one who spins; spinning.</def> <mark>[Obs.]</mark>  <rj><au>Milton.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spinthariscope</ent><br/
<hw>Spin*thar"i*scope</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Gr. <grk>spinqari`s</grk> spark + <ets>-scope</ets>.]</ety> <def>A small instrument containing a minute particle of a radium compound mounted in front of a fluorescent screen and viewed with magnifying lenses. The tiny flashes produced by the continual bombardment of the screen by the <alpha/ rays are thus rendered visible.</def> -- <wordforms><wf>Spin*thar`i*scop"ic</wf> <pr>(#)</pr>, <pos>a.</pos></wordforms><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>spin the bottle</ent><br/
<hw>spin the bottle</hw> <pos>n.</pos> <def>a party game in which a player spins a bottle and kisses the person that it points to when it stops spinning; -- usually played by children.</def><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>spin the platter</ent><br/
<ent>spin the plate</ent><br/
<mhw><hw>spin the plate</hw>, <hw>spin the platter</hw></mhw> <pos>n.</pos> <def>a game in which something round (as a plate) is spun on edge and the name of a player is called; the named player must catch the spinning object before it falls or pay a forfeit.</def><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Spinule</ent><br/
<hw>Spin"ule</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>spinula</ets>, dim. of <ets>spina</ets> a spine: cf. F. <ets>spinule</ets>.]</ety> <def>A minute spine.</def>  <rj><au>Dana.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spinulescent</ent><br/
<hw>Spin`u*les"cent</hw> <pr>(?)</pr>, <pos>a.</pos> <fld>(Bot.)</fld> <def>Having small spines; somewhat thorny.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spinulous</ent><br/
<ent>Spinulose</ent><br/
<mhw>{ <hw>Spin"u*lose`</hw> <pr>(?)</pr>, <hw>Spin"u*lous</hw> <pr>(?)</pr>, }</mhw> <pos>a.</pos> <ety>[Cf. F. <ets>spinuleux</ets>.]</ety> <def>Covered with small spines.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spiny</ent><br/
<hw>Spin"y</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[From <er>Spine</er>.]</ety> <sn>1.</sn> <def>Full of spines; thorny; <as>as, a <ex>spiny</ex> tree</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Like a spine in shape; slender.</def> <ldquo/<xex>Spiny</xex> grasshoppers sit chirping.<rdquo/  <rj><au>Chapman.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>Fig.: Abounding with difficulties or annoyances.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The <qex>spiny</qex> deserts of scholastic philosophy.</q> <rj><qau>Bp. Warburton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Spiny lobster</b></col>. <fld>(Zool.)</fld> <cd>Same as <cref>Rock lobster</cref>, under <er>Rock</er>. See also <er>Lobster</er>.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spiny</ent><br/
<hw>Spin"y</hw>, <pos>n.</pos> <def>See <er>Spinny</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spiodea</ent><br/
||<hw>Spi*o"de*a</hw> <pr>(?)</pr>, <pos>n. pl.</pos> <ety>[NL., fr. <ets>Spio</ets> the typical genus.]</ety> <fld>(Zool.)</fld> <def>An extensive division of marine Annelida, including those that are without oral tentacles or cirri, and have the gills, when present, mostly arranged along the sides of the body. They generally live in burrows or tubes.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spirable</ent><br/
<hw>Spi"ra*ble</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>spirabilis</ets>.]</ety> <def>Capable of being breathed; respirable.</def> <mark>[Obs.]</mark>  <rj><au>Nash.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spiracle</ent><br/
<hw>Spir"a*cle</hw> <pr>(?)</pr>, <pos>n.</pos><ety>[L. <ets>spiraculum</ets>, fr. <ets>spirare</ets> to breathe: cf. F. <ets>spiracule</ets>. See <er>Spirit</er>.]</ety> <sn>1.</sn> <fld>(Anat.)</fld> <def>The nostril, or one of the nostrils, of whales, porpoises, and allied animals.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Zool.)</fld> <sd>(a)</sd> <def>One of the external openings communicating with the air tubes or tracheae of insects, myriapods, and arachnids. They are variable in number, and are usually situated on the sides of the thorax and abdomen, a pair to a segment. These openings are usually elliptical, and capable of being closed. See <xex>Illust.</xex> under <er>Coleoptera</er>.</def> <sd>(a)</sd> <def>A tubular orifice communicating with the gill cavity of certain ganoid and all elasmobranch fishes. It is the modified first gill cleft.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>Any small aperture or vent for air or other fluid.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spiracular</ent><br/
<hw>Spi*rac"u*lar</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Of or pertaining to a spiracle.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spiraea</ent><br/
<hw>Spi*rae"a</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L., fr. Gr. <?/<?/<?/, fr. <?/<?/<?/ a coil.]</ety> <fld>(Bot.)</fld> <def>A genus of shrubs or perennial herbs including the meadowsweet and the hardhack.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spiraeic</ent><br/
<hw>Spi*rae"ic</hw> <pr>(?)</pr>, <pos>a.</pos> <fld>(Chem.)</fld> <def>Of, pertaining to, or derived from, the meadowsweet (<gen>Spiraea</gen>); formerly, designating an acid which is now called <xex>salicylic acid</xex>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spiral</ent><br/
<hw>Spi"ral</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[Cf. F. <ets>spiral</ets>. See <er>Spire</er> a winding line.]</ety> <sn>1.</sn> <def>Winding or circling round a center or pole and gradually receding from it; <as>as, the <ex>spiral</ex> curve of a watch spring</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Winding round a cylinder or imaginary axis, and at the same time rising or advancing forward; winding like the thread of a screw; helical.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <fld>(Geom.)</fld> <def>Of or pertaining to a spiral; like a spiral.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><mcol><col><b>Spiral gear</b></col>, <it>or</it> <col><b>Spiral wheel</b></col></mcol> <fld>(Mach.)</fld>, <cd>a gear resembling in general a spur gear, but having its teeth cut at an angle with its axis, or so that they form small portions of screws or spirals.</cd> -- <col><b>Spiral gearing</b></col>, <cd>a kind of gearing sometimes used in light machinery, in which spiral gears, instead of bevel gears, are used to transmit motion between shafts that are not parallel.</cd> -- <col><b>Spiral operculum</b></col>, <cd>an operculum whih has spiral lines of growth.</cd> -- <col><b>Spiral shell</b></col>, <cd>any shell in which the whorls form a spiral or helix.</cd> -- <col><b>Spiral spring</b></col>. <cd>See the <xex>Note</xex> under <er>Spring</er>, <pos>n.</pos>, 4.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spiral</ent><br/
<hw>Spi"ral</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Cf. F. <ets>spirale</ets>. See <er>Spiral</er>, <pos>a.</pos>]</ety> <sn>1.</sn> <fld>(Geom.)</fld> <def>A plane curve, not reentrant, described by a point, called the <xex>generatrix</xex>, moving along a straight line according to a mathematical law, while the line is revolving about a fixed point called the <xex>pole</xex>.  Cf. <er>Helix</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Anything which has a spiral form, as a spiral shell.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Equiangular spiral</b></col>,<cd>a plane curve which cuts all its generatrices at the same angle. Same as <altname>Logarithmic spiral</altname>, under <er>Logarithmic</er>.</cd> -- <col><b>Spiral of Archimedes</b></col>, <cd>a spiral the law of which is that the generatrix moves uniformly along the revolving line, which also moves uniformly.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spirality</ent><br/
<hw>Spi*ral"i*ty</hw> <pr>(?)</pr>, <pos>n.</pos> <def>The quality or states of being spiral.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spirally</ent><br/
<hw>Spi"ral*ly</hw> <pr>(?)</pr>, <pos>adv.</pos> <def>In a spiral form, manner, or direction.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spiralozooid</ent><br/
<hw>Spi*ra`lo*zo"oid</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[<ets>Spiral</ets> + <ets>zooid</ets>. So called because they often have a spiral form when contracted.]</ety> <fld>(Zool.)</fld> <def>One of the special defensive zooids of certain hydroids. They have the form of long, slender tentacles, and bear lasso cells.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spirant</ent><br/
<hw>Spi"rant</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>spirans</ets>, <ets>-antis</ets>, p. pr. of <ets>spirare</ets> to breathe. See <er>Spirit</er>.]</ety> <fld>(Phon.)</fld> <def>A term used differently by different authorities; -- by some as equivalent to <xex>fricative</xex>, -- that is, as including all the continuous consonants, except the nasals <xex>m</xex>, <xex>n</xex>, <xex>ng</xex>; with the further exception, by others, of the liquids <xex>r</xex>, <xex>l</xex>, and the semivowels <xex>w</xex>, <xex>y</xex>; by others limited to <xex>f</xex>, <xex>v</xex>, <xex>th</xex> surd and sonant, and the sound of German <xex>ch</xex>, -- thus excluding the sibilants, as well as the nasals, liquids, and semivowels. See <xex>Guide to Pronunciation</xex>, <sect/<sect/ 197-208.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spiranthy</ent><br/
<hw>Spi*ran"thy</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Gr. <?/<?/<?/ a coil + <?/<?/<?/ flower.]</ety> <fld>(Bot.)</fld> <def>The occasional twisted growth of the parts of a flower.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spiration</ent><br/
<hw>Spi*ra"tion</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>spiratio</ets>, fr. <ets>spirare</ets> to breathe.]</ety> <def>The act of breathing.</def> <mark>[Obs.]</mark>  <rj><au>Barrow.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spire</ent><br/
<hw>Spire</hw> <pr>(?)</pr>, <pos>v. i.</pos> <ety>[L. <ets>spirare</ets> to breathe. See <er>Spirit</er>.]</ety> <def>To breathe.</def> <mark>[Obs.]</mark>  <rj><au>Shenstone.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spire</ent><br/
<hw>Spire</hw>, <pos>n.</pos> <ety>[OE. <ets>spire</ets>, <ets>spir</ets>, a blade of grass, a young shoot, AS. <ets>sp<imac/r</ets>; akin to G. <ets>spier</ets> a blade of grass, Dan. <ets>spire</ets> a sprout, sprig, Sw. <ets>spira</ets> a spar, Icel. <ets>sp<imac/ra</ets>.]</ety> <sn>1.</sn> <def>A slender stalk or blade in vegetation; <as>as, a <ex>spire</ex> grass or of wheat</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>An oak cometh up a little <qex>spire</qex>.</q> <rj><qau>Chaucer.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A tapering body that shoots up or out to a point in a conical or pyramidal form. Specifically <fld>(Arch.)</fld>, the roof of a tower when of a pyramidal form and high in proportion to its width; also, the pyramidal or aspiring termination of a tower which can not be said to have a roof, such as that of Strasburg cathedral; the tapering part of a steeple, or the steeple itself.</def> <ldquo/With glistering <xex>spires</xex> and pinnacles adorned.<rdquo/  <rj><au>Milton.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>A <qex>spire</qex> of land that stand apart,<br/
Cleft from the main.</q> <rj><qau>Tennyson.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Tall <qex>spire</qex> from which the sound of cheerful bells<br/
Just undulates upon the listening ear.</q> <rj><qau>Cowper.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <fld>(Mining)</fld> <def>A tube or fuse for communicating fire to the chargen in blasting.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>The top, or uppermost point, of anything; the summit.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The <qex>spire</qex> and top of praises.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spire</ent><br/
<hw>Spire</hw>, <pos>v. i.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Spired</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Spiring</conjf>.]</vmorph> <def>To shoot forth, or up in, or as if in, a spire.</def>  <rj><au>Emerson.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>It is not so apt to <qex>spire</qex> up as the other sorts, being more inclined to branch into arms.</q> <rj><qau>Mortimer.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spire</ent><br/
<hw>Spire</hw>, <pos>n.</pos> <ety>[L. <ets>spira</ets> coil, twist; akin to Gr. <?/<?/<?/: cf. F. <ets>spire</ets>.]</ety> <sn>1.</sn> <def>A spiral; a curl; a whorl; a twist.</def>  <rj><au>Dryden.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Geom.)</fld> <def>The part of a spiral generated in one revolution of the straight line about the pole. See <er>Spiral</er>, <pos>n.</pos></def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Spire bearer</b></col>. <fld>(Paleon.)</fld> <cd>Same as <er>Spirifer</er>.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spired</ent><br/
<hw>Spired</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Having a spire; being in the form of a spire; <as>as, a <ex>spired</ex> steeple</as>.</def>  <rj><au>Mason.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spiricle</ent><br/
<hw>Spi"ri*cle</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Dim., fr. L. <ets>spira</ets> a coil.]</ety> <fld>(Bot.)</fld> <def>One of certain minute coiled threads in the coating of some seeds. When moistened these threads protrude in great numbers.</def>  <rj><au>Gray.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spirifer</ent><br/
<hw>Spi"ri*fer</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[NL., fr. L. <ets>spira</ets> a coil + <ets>ferre</ets>to bear.]</ety> <fld>(Paleon.)</fld> <def>Any one of numerous species of fossil brachipods of the genus <gen>Spirifer</gen>, or <gen>Delthyris</gen>, and allied genera, in which the long calcareous supports of the arms form a large spiral, or helix, on each side.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spirillum</ent><br/
||<hw>Spi*ril"lum</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[NL., dim. of L. <ets>spira</ets> a coil.]</ety> <sn>1.</sn> <fld>(Biol.)</fld> <def>A genus of common gram-negative motile microorganisms (family <fam>Spirillaceae</fam>) having the form of spiral-shaped filaments.  They are obligately microaerophilic, and do not ferment carbohydrates.</def> <au>Stedman.</au><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A species of bacteria (<spn>Borrelia recurrentis</spn>, also known as <altname>Obermeier's spirillum</altname>), once believed to belong to the genus <gen>Spirillum</gen>, which causes relapsing fever.</def><br/
[<source>PJC</source>]</p>

<p><ent>Spiring</ent><br/
<hw>Spir"ing</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Shooting up in a spire or spires.</def> <ldquo/The <xex>spiring</xex> grass.<rdquo/  <rj><au>Dryton.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spirit</ent><br/
<hw>Spir"it</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[OF. <ets>espirit</ets>, <ets>esperit</ets>, F. <ets>esprit</ets>, L. <ets>spiritus</ets>, from <ets>spirare</ets> to breathe, to blow.  Cf. <er>Conspire</er>, <er>Expire</er>, <er>Esprit</er>, <er>Sprite</er>.]</ety> <sn>1.</sn> <def>Air set in motion by breathing; breath; hence, sometimes, life itself.</def> <mark>[Obs.]</mark> <ldquo/All of <xex>spirit</xex> would deprive.<rdquo/  <rj><au>Spenser.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>The mild air, with season moderate,<br/
Gently attempered, and disposed eo well,<br/
That still it breathed foorth sweet <qex>spirit</qex>.</q> <rj><qau>Spenser.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A rough breathing; an aspirate, as the letter <xex>h</xex>; also, a mark to denote aspiration; a breathing.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>Be it a letter or <qex>spirit</qex>, we have great use for it.</q> <rj><qau>B. Jonson.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>Life, or living substance, considered independently of corporeal existence; an intelligence conceived of apart from any physical organization or embodiment; vital essence, force, or energy, as distinct from matter.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>The intelligent, immaterial and immortal part of man; the soul, in distinction from the body in which it resides; the agent or subject of vital and spiritual functions, whether spiritual or material.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>There is a <qex>spirit</qex> in man; and the inspiration of the Almighty giveth them understanding.</q> <rj><qau>Job xxxii. 8.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>As the body without the <qex>spirit</qex> is dead, so faith without works is dead also.</q> <rj><qau>James ii. 26.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q><qex>Spirit</qex> is a substance wherein thinking, knowing, doubting, and a power of moving, do subsist.</q> <rj><qau>Locke.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <def>Specifically, a disembodied soul; the human soul after it has left the body.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Then shall the dust return to the earth as it was, and the <qex>spirit</qex> shall return unto God who gave it.</q> <rj><qau>Eccl. xii. 7.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Ye gentle <qex>spirits</qex> far away,<br/
With whom we shared the cup of grace.</q> <rj><qau>Keble.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>6.</sn> <def>Any supernatural being, good or bad; an apparition; a specter; a ghost; also, sometimes, a sprite,; a fairy; an elf.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Whilst young, preserve his tender mind from all impressions of <qex>spirits</qex> and goblins in the dark.</q> <rj><qau>Locke.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>7.</sn> <def>Energy, vivacity, ardor, enthusiasm, courage, etc.</def><br/
[<source>1913 Webster</source>]</p>

<p><q><ldquo/Write it then, quickly,<rdquo/ replied Bede; and summoning all his <qex>spirits</qex> together, like the last blaze of a candle going out, he indited it, and expired.</q> <rj><qau>Fuller.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>8.</sn> <def>One who is vivacious or lively; one who evinces great activity or peculiar characteristics of mind or temper; <as>as, a ruling <ex>spirit</ex>; a schismatic <ex>spirit</ex></as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Such <qex>spirits</qex> as he desired to please, such would I choose for my judges.</q> <rj><qau>Dryden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>9.</sn> <def>Temper or disposition of mind; mental condition or disposition; intellectual or moral state; -- often in the plural; <as>as, to be cheerful, or in good <ex>spirits</ex>; to be downhearted, or in bad <ex>spirits</ex></as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>God has . . . made a <qex>spirit</qex> of building succeed a <qex>spirit</qex> of pulling down.</q> <rj><qau>South.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>A perfect judge will read each work of wit<br/
With the same <qex>spirit</qex> that its author writ.</q> <rj><qau>Pope.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>10.</sn> <def>Intent; real meaning; -- opposed to the <xex>letter</xex>, or to <xex>formal statement</xex>; also, characteristic quality, especially such as is derived from the individual genius or the personal character; <as>as, the <ex>spirit</ex> of an enterprise, of a document, or the like</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>11.</sn> <def>Tenuous, volatile, airy, or vapory substance, possessed of active qualities.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>All bodies have <qex>spirits</qex> . . . within them.</q> <rj><qau>Bacon.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>12.</sn> <def>Any liquid produced by distillation; especially, alcohol, the <xex>spirits</xex>, or <xex>spirit</xex>, of wine (it having been first distilled from wine): -- often in the plural.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>13.</sn> <pluf>pl.</pluf> <def>Rum, whisky, brandy, gin, and other distilled liquors having much alcohol, in distinction from wine and malt liquors.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>14.</sn> <fld>(Med.)</fld> <def>A solution in alcohol of a volatile principle.  Cf. <er>Tincture</er>.</def>  <rj><au>U. S. Disp.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>15.</sn> <fld>(Alchemy)</fld> <def>Any one of the four substances, sulphur, sal ammoniac, quicksilver, or arsenic (or, according to some, orpiment).</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The four <qex>spirits</qex> and the bodies seven.</q> <rj><qau>Chaucer.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>16.</sn> <fld>(Dyeing)</fld> <def>Stannic chloride. See under <er>Stannic</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ <xex>Spirit</xex> is sometimes joined with other words, forming compounds, generally of obvious signification; as, <xex>spirit</xex>-moving, <xex>spirit</xex>-searching, <xex>spirit</xex>-stirring, etc.</note><br/
[<source>1913 Webster</source>]</p>

<p><cs><mcol><col><b>Astral spirits</b></col>, <col><b>Familiar spirits</b></col></mcol>, <cd>etc. See under <er>Astral</er>, <er>Familiar</er>, etc.</cd> -- <col><b>Animal spirits</b></col>. <sd>(a)</sd> <fld>(Physiol.)</fld> <cd>The fluid which at one time was supposed to circulate through the nerves and was regarded as the agent of sensation and motion; -- called also the <altname>nervous fluid</altname>, or <altname>nervous principle</altname>.</cd> <sd>(b)</sd> <cd>Physical health and energy; frolicsomeness; sportiveness.</cd> -- <col><b>Ardent spirits</b></col>, <cd>strong alcoholic liquors, as brandy, rum, whisky, etc., obtained by distillation.</cd> -- <mcol><col><b>Holy Spirit</b></col>, <it>or</it> <col><b>The Spirit</b></col></mcol> <fld>(Theol.)</fld>, <cd>the Spirit of God, or the third person of the Trinity; the Holy Ghost.  <xex>The spirit</xex> also signifies the human spirit as influenced or animated by the Divine Spirit.</cd> -- <col><b>Proof spirit</b></col>. <fld>(Chem.)</fld> <cd>See under <er>Proof</er>.</cd> -- <col><b>Rectified spirit</b></col> <fld>(Chem.)</fld>, <cd>spirit rendered purer or more concentrated by redistillation, so as to increase the percentage of absolute alcohol.</cd> -- <col><b>Spirit butterfly</b></col> <fld>(Zool.)</fld>, <cd>any one of numerous species of delicate butterflies of tropical America belonging to the genus <gen>Ithomia</gen>. The wings are gauzy and nearly destitute of scales.</cd> -- <col><b>Spirit duck</b></col>. <fld>(Zool.)</fld> <sd>(a)</sd> <cd>The buffle-headed duck</cd>. <sd>(b)</sd> <cd>The golden-eye.</cd> -- <col><b>Spirit lamp</b></col> <fld>(Art)</fld>, <cd>a lamp in which alcohol or methylated spirit is burned.</cd> -- <col><b>Spirit level</b></col>. <cd>See under <er>Level</er>.</cd> -- <col><b>Spirit of hartshorn</b></col>. <fld>(Old Chem.)</fld> <cd>See under <er>Hartshorn</er>.</cd> -- <col><b>Spirit of Mindererus</b></col> <fld>(Med.)</fld>, <cd>an aqueous solution of acetate of ammonium; -- named after R. <etsep>Minderer</etsep>, physician of Augsburg.</cd> -- <col><b>Spirit of nitrous ether</b></col> <fld>(Med. Chem.)</fld>, <cd>a pale yellow liquid, of a sweetish taste and a pleasant ethereal odor. It is obtained by the distillation of alcohol with nitric and sulphuric acids, and consists essentially of ethyl nitrite with a little acetic aldehyde. It is used as a diaphoretic, diuretic, antispasmodic, etc.  Called also <altname>sweet spirit of niter</altname>.</cd> -- <col><b>Spirit of salt</b></col> <fld>(Chem.)</fld>, <cd>hydrochloric acid; -- so called because obtained from salt and sulphuric acid.</cd> <mark>[Obs.]</mark> -- <col><b>Spirit of sense</b></col>, <cd>the utmost refinement of sensation.</cd> <mark>[Obs.]</mark> <au>Shak.</au> -- <mcol><col><b>Spirits of turpentine</b></col>, <it>or</it> <col><b>Spirit of turpentine</b></col></mcol> <fld>(Chem.)</fld>, <cd>rectified oil of turpentine, a transparent, colorless, volatile, and very inflammable liquid, distilled from the turpentine of the various species of pine; camphine.  It is commonly used to remove paint from surfaces, or to dissole oil-based paint.  See <er>Camphine</er>.</cd> -- <col><b>Spirit of vitriol</b></col> <fld>(Chem.)</fld>, <cd>sulphuric acid; -- so called because formerly obtained by the distillation of green vitriol.</cd> <mark>[Obs.]</mark> -- <col><b>Spirit of vitriolic ether</b></col> <fld>(Chem.)</fld> <cd>ethyl ether; -- often but incorrectly called <altname>sulphuric ether</altname>.  See <er>Ether</er>.</cd> <mark>[Obs.]</mark> -- <mcol><col><b>Spirits of wine</b></col>, <it>or</it> <col><b>Spirit of wine</b></col></mcol> <fld>(Chem.)</fld>, <cd>alcohol; -- so called because formerly obtained by the distillation of wine.</cd> -- <col><b>Spirit rapper</b></col>, <cd>one who practices spirit rapping; a <ldquo/medium<rdquo/ so called.</cd> -- <col><b>Spirit rapping</b></col>, <cd>an alleged form of communication with the spirits of the dead by raps. See <er>Spiritualism</er>, 3.</cd> -- <col><b>Sweet spirit of niter</b></col>. <cd>See <cref>Spirit of nitrous ether</cref>, above.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><-- p. 1388 --></p>

<p><syn><b>Syn.</b> -- Life; ardor; energy; fire; courage; animatioon; cheerfulness; vivacity; enterprise.</syn><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spirit</ent><br/
<hw>Spir"it</hw> <pr>(?)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Spirited</conjf>; <pos>p. pr. & vb. n.</pos> <conjf>Spiriting</conjf>.]</vmorph> <sn>1.</sn> <def>To animate with vigor; to excite; to encourage; to inspirit; <as>as, civil dissensions often <ex>spirit</ex> the ambition of private men</as>; -- sometimes followed by <xex>up</xex>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Many officers and private men <qex>spirit</qex> up and assist those obstinate people to continue in their rebellion.</q> <rj><qau>Swift.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To convey rapidly and secretly, or mysteriously, as if by the agency of a spirit; to kidnap; -- often with <xex>away</xex>, or <xex>off</xex>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The ministry had him <qex>spirited</qex> away, and carried abroad as a dangerous person.</q> <rj><qau>Arbuthnot & Pope.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>I felt as if I had been <qex>spirited</qex> into some castle of antiquity.</q> <rj><qau>Willis.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Spiriting away</b></col> <fld>(Law)</fld>, <cd>causing to leave; the offense of inducing a witness to leave a jurisdiction so as to evade process requiring attendance at trial.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spiritally</ent><br/
<hw>Spir"it*al*ly</hw> <pr>(?)</pr>, <pos>adv.</pos><ety>[L. <ets>spiritalis</ets> belonging to breathing.]</ety> <def>By means of the breath.</def> <mark>[Obs.]</mark>  <rj><au>Holder.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spirited</ent><br/
<hw>Spir"it*ed</hw>, <pos>a.</pos> <sn>1.</sn> <def>Animated or possessed by a spirit.</def> <mark>[Obs.]</mark> <ldquo/So talked the <xex>spirited</xex>, sly snake.<rdquo/  <rj><au>Milton.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Animated; full of life or vigor; lively; full of spirit or fire; <as>as, a <ex>spirited</ex> oration; a <ex>spirited</ex> answer</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ <xex>Spirited</xex> is much used in composition; as in high-<xex>spirited</xex>, low-<xex>spirited</xex>, mean-<xex>spirited</xex>, etc.</note><br/
[<source>1913 Webster</source>]</p>

<p><syn><b>Syn.</b> -- Lively; vivacious; animated; ardent; active; bold; courageous.</syn><br/
[<source>1913 Webster</source>]</p>

<p>-- <wordforms><wf>Spir"it*ed*ly</wf>, <pos>adv.</pos> -- <wf>Spir"it*ed*ness</wf>, <pos>n.</pos></wordforms><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spiritful</ent><br/
<hw>Spir"it*ful</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Full of spirit; spirited.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>The <qex>spiritful</qex> and orderly life of our own grown men.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p>-- <wordforms><wf>Spir"it*ful*ly</wf>, <pos>adv.</pos> -- <wf>Spir"it*ful*ness</wf>, <pos>n.</pos></wordforms><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spiritism</ent><br/
<hw>Spir"it*ism</hw> <pr>(?)</pr>, <pos>n.</pos> <def>Spiritualsm.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spiritist</ent><br/
<hw>Spir"it*ist</hw>, <pos>n.</pos> <def>A spiritualist.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spiritless</ent><br/
<hw>Spir"it*less</hw>, <pos>a.</pos> <sn>1.</sn> <def>Destitute of spirit; wanting animation; wanting cheerfulness; dejected; depressed.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Destitute of vigor; wanting life, courage, or fire.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>A men so faint, so <qex>spiritless</qex>,<br/
So dull, so dead in lock, so woebegone.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>Having no breath; extinct; dead.</def> <ldquo/The <xex>spiritless</xex> body.<rdquo/  <rj><au>Greenhill.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p>-- <wordforms><wf>Spir"it*less*ly</wf>, <pos>adv.</pos> -- <wf>Spir"it*less*ness</wf>, <pos>n.</pos></wordforms><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spiritoso</ent><br/
||<hw>Spi`ri*to"so</hw> <pr>(?)</pr>, <pos>a. & adv.</pos> <ety>[It.]</ety> <fld>(Mus.)</fld> <def>Spirited; spiritedly; -- a direction to perform a passage in an animated, lively manner.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spiritous</ent><br/
<hw>Spir"it*ous</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[Cf. <er>Spirituous</er>.]</ety> <sn>1.</sn> <def>Like spirit; refined; defecated; pure.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>More refined, more <qex>spirituous</qex> and pure.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Ardent; active.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spiritousness</ent><br/
<hw>Spir"it*ous*ness</hw>, <pos>n.</pos> <def>Quality of being spiritous.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spiritual</ent><br/
<hw>Spir"it*u*al</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>spiritualis</ets>: cf. F. <ets>spirituel</ets>. See <er>Spirit</er>.]</ety> <sn>1.</sn> <def>Consisting of spirit; not material; incorporeal; <as>as, a <ex>spiritual</ex> substance or being</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>It is sown a natural body, it is raised a <qex>spiritual</qex> body.</q> <rj><qau>1 Cor. xv. 44.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Of or pertaining to the intellectual and higher endowments of the mind; mental; intellectual.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>Of or pertaining to the moral feelings or states of the soul, as distinguished from the external actions; reaching and affecting the spirits.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>God's law is <qex>spiritual</qex>; it is a transcript of the divine nature, and extends its authority to the acts of the soul of man.</q> <rj><qau>Sir T. Browne.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>Of or pertaining to the soul or its affections as influenced by the Spirit; controlled and inspired by the divine Spirit; proceeding from the Holy Spirit; pure; holy; divine; heavenly-minded; -- opposed to <ant>carnal</ant>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>That I may impart unto you some <qex>spiritual</qex> gift.</q> <rj><qau>Rom. i. ll.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Blessed be the God and Father of our Lord Jesus Christ, who hath blessed us with all <qex>spiritual</qex> blessings.</q> <rj><qau>Eph. i. 3.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>If a man be overtaken in a fault, ye which are <qex>spiritual</qex>, restore such an one.</q> <rj><qau>Gal. vi. 1.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <def>Not lay or temporal; relating to sacred things; ecclesiastical; <as>as, the <ex>spiritual</ex> functions of the clergy; lords <ex>spiritual</ex> and temporal; a <ex>spiritual</ex> corporation</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Spiritual coadjuctor</b></col>. <fld>(Eccl.)</fld> <cd>See the Note under <er>Jesuit</er>.</cd> -- <col><b>Spiritual court</b></col> <fld>(Eccl. Law)</fld>, <cd>an ecclesiastical court, or a court having jurisdiction in ecclesiastical affairs; a court held by a bishop or other ecclesiastic.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spiritual</ent><br/
<hw>Spir"it*u*al</hw>, <pos>n.</pos> <def>A spiritual function, office, or affair. See <er>Spirituality</er>, 2.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>He assigns supremacy to the pope in <qex>spirituals</qex>, and to the emperor in temporals.</q> <rj><qau>Lowell.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spiritualism</ent><br/
<hw>Spir"it*u*al*ism</hw> <pr>(?)</pr>, <pos>n.</pos> <sn>1.</sn> <def>The quality or state of being spiritual.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Physiol.)</fld> <def>The doctrine, in opposition to the materialists, that all which exists is spirit, or soul -- that what is called the external world is either a succession of notions impressed on the mind by the Deity, as maintained by Berkeley, or else the mere educt of the mind itself, as taught by Fichte.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>A belief that departed spirits hold intercourse with mortals by means of physical phenomena, as by rapping, or during abnormal mental states, as in trances, or the like, commonly manifested through a person of special susceptibility, called a <xex>medium</xex>; spiritism; the doctrines and practices of spiritualists.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>What is called <qex>spiritualism</qex> should, I think, be called a mental species of materialism.</q> <rj><qau>R. H. Hutton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spiritualist</ent><br/
<hw>Spir"it*u*al*ist</hw> <pr>(?)</pr>, <pos>n.</pos> <sn>1.</sn> <def>One who professes a regard for spiritual things only; one whose employment is of a spiritual character; an ecclesiastic.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>One who maintains the doctrine of spiritualism.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>One who believes in direct intercourse with departed spirits, through the agency of persons commonly called <xex>mediums</xex>, by means of physical phenomena; one who attempts to maintain such intercourse; a spiritist.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spiritualist</ent><br/
<hw>Spir"it*u*al*ist</hw>, <pos>a.</pos> <def>Spiritualistic.</def>  <rj><au>Taylor.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spiritualistic</ent><br/
<hw>Spir`it*u*al*is"tic</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Relating to, or connected with, spiritualism.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spirituality</ent><br/
<hw>Spir`it*u*al"i*ty</hw> <pr>(?)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Spiritualities</plw> <pr>(#)</pr>.</plu> <ety>[L. <ets>spiritualitas</ets>: cf. F. <ets>spiritualit<eacute/</ets>.]</ety> <sn>1.</sn> <def>The quality or state of being spiritual; incorporeality; heavenly-mindedness.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>A pleasure made for the soul, suitable to its <qex>spirituality</qex>.</q> <rj><qau>South.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>If this light be not spiritual, yet it approacheth nearest to <qex>spirituality</qex>.</q> <rj><qau>Sir W. Raleigh.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Much of our <qex>spirituality</qex> and comfort in public worship depends on the state of mind in which we come.</q> <rj><qau>Bickersteth.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Eccl.)</fld> <def>That which belongs to the church, or to a person as an ecclesiastic, or to religion, as distinct from temporalities.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>During the vacancy of a see, the archbishop is guardian of the <qex>spiritualities</qex> thereof.</q> <rj><qau>Blackstone.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>An ecclesiastical body; the whole body of the clergy, as distinct from, or opposed to, the temporality.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>Five entire subsidies were granted to the king by the <qex>spirituality</qex>.</q> <rj><qau>Fuller.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spiritualization</ent><br/
<hw>Spir`it*u*al*i*za"tion</hw> <pr>(?)</pr>, <pos>n.</pos> <def>The act of spiritualizing, or the state of being spiritualized.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spiritualize</ent><br/
<hw>Spir"it*u*al*ize</hw> <pr>(?)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Spiritualized</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Spiritualizing</conjf> <pr>(?)</pr>.]</vmorph> <ety>[Cf. F. <ets>spiritualiser</ets>.]</ety> <sn>1.</sn> <def>To refine intellectiually or morally; to purify from the corrupting influence of the world; to give a spiritual character or tendency to; <as>as, to <ex>spiritualize</ex> soul</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>This seen in the clear air, and the whole <qex>spiritualized</qex> by endless recollections, fills the eye and the heart more forcibly than I can find words to say.</q> <rj><qau>Carlyle.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To give a spiritual meaning to; to take in a spiritual sense; -- opposed to <contr>literalize</contr>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <fld>(Old Chem.)</fld> <def>To extract spirit from; also, to convert into, or impregnate with, spirit.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spiritualizer</ent><br/
<hw>Spir"it*u*al*i`zer</hw> <pr>(?)</pr>, <pos>n.</pos> <def>One who spiritualizes.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spiritually</ent><br/
<hw>Spir"it*u*al*ly</hw>, <pos>adv.</pos> <def>In a spiritual manner; with purity of spirit; like a spirit.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spiritual-minded</ent><br/
<hw>Spir"it*u*al-mind`ed</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Having the mind set on spiritual things, or filled with holy desires and affections.</def> -- <wordforms><wf>Spir"it*u*al-mind`ed*ness</wf>, <pos>n.</pos></wordforms><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spiritualness</ent><br/
<hw>Spir"it*u*al*ness</hw>, <pos>n.</pos> <def>The quality or state of being spiritual or spiritual-minded; spirituality.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spiritualty</ent><br/
<hw>Spir"it*u*al*ty</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[See <er>Spirituality</er>.]</ety> <fld>(Eccl.)</fld> <def>An ecclesiastical body; a spirituality.</def>  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spirituelle</ent><br/
||<hw>Spi`ri`tu`elle"</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[F.]</ety> <def>Of the nature, or having the appearance, of a spirit; pure; refined; ethereal.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spirituosity</ent><br/
<hw>Spir`it*u*os"i*ty</hw> <pr>(?)</pr>, <pos>n.</pos> <def>The quality or state of being spirituous; spirituousness.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spirituous</ent><br/
<hw>Spir"it*u*ous</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[Cf. (for sense 2) F. <ets>spiritueux</ets>.  Cf. <er>Spiritous</er>.]</ety> <sn>1.</sn> <def>Having the quality of spirit; tenuous in substance, and having active powers or properties; ethereal; immaterial; spiritual; pure.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Containing, or of the nature of, alcoholic (esp. distilled) spirit; consisting of refined spirit; alcoholic; ardent; <as>as, <ex>spirituous</ex> liquors</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>Lively; gay; vivid; airy.</def> <mark>[Obs.]</mark>  <rj><au>Sir H. Wotton.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>The mind of man is of that <qex>spirituous</qex>, stirring nature, that it is perpetually at work.</q> <rj><qau>South.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spirituousness</ent><br/
<hw>Spir"it*u*ous*ness</hw>, <pos>n.</pos> <def>The quality or state of being spirituous.</def> <mark>[R.]</mark>  <rj><au>Boyle.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spirketing</ent><br/
<hw>Spirk"et*ing</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Naut.)</fld> <def>The planking from the waterways up to the port sills.</def>  <rj><au>Totten.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spirling</ent><br/
<hw>Spirl"ing</hw> <pr>(?)</pr>, <pos>n.</pos> <def>Sparling.</def> <mark>[Prov. Eng.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spirobacteria</ent><br/
||<hw>Spi`ro*bac*te"ri*a</hw> <pr>(?)</pr>, <pos>n. pl.</pos>; <sing>sing. <singw>Spirobacterium</singw> <pr>(<?/)</pr></sing>. <ety>[NL. See 4th <er>Spire</er>, and <er>Bacterium</er>.]</ety> <fld>(Biol.)</fld> <def>See the Note under <er>Microbacteria</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spirochaete</ent><br/
<ent>Spirochaeta</ent><br/
<mhw>{ ||<hw>Spi`ro*chae"ta</hw> <pr>(?)</pr>, ||<hw>Spi`ro*chae"te</hw> <pr>(?)</pr>, }</mhw> <pos>n.</pos> <ety>[L. <ets>spira</ets> a coil + Gr. <?/<?/<?/ hair.]</ety> <fld>(Biol.)</fld> <def>A genus of Spirobacteria similar to Spirillum, but distinguished by its motility. One species, the <spn>Spirochaete Obermeyeri</spn>, is supposed to be the cause of relapsing fever.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spirograph</ent><br/
<hw>Spi"ro*graph</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>spirare</ets> to breathe + <ets>-graph</ets>.]</ety> <fld>(Physiol.)</fld> <def>An instrument for recording the respiratory movements, as the sphygmograph does those of the pulse.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spirometer</ent><br/
<hw>Spi*rom"e*ter</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>spirare</ets> to breathe + <ets>-meter</ets>.]</ety> <def>An instrument for measuring the vital capacity of the lungs, or the volume of air which can be expelled from the chest after the deepest possible inspiration.  Cf. <er>Pneumatometer</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spirometry</ent><br/
<hw>Spi*rom"e*try</hw> <pr>(?)</pr>, <pos>n.</pos> <def>The act or process of measuring the chest capacity by means of a spirometer.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spiroscope</ent><br/
<hw>Spi"ro*scope</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>spirare</ets> to breathe + <ets>-scope</ets>.]</ety> <fld>(Physiol.)</fld> <def>A wet meter used to determine the breathing capacity of the lungs.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spiroylous</ent><br/
<ent>Spiroylic</ent><br/
<mhw>{ <hw>Spi*royl"ic</hw> <pr>(?)</pr>, <hw>Spi*royl"ous</hw> <pr>(?)</pr>, }</mhw> <pos>a.</pos> <ety>[NL. <ets>Spir<?/</ets> meadowsweet (a source of salicylal) + <ets>-yl</ets> + <ets>-ic</ets>, <ets>-ous</ets>.]</ety> <fld>(Chem.)</fld> <def>Pertaining to, or designating, a substance now called <xex>salicylal</xex>.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spirt</ent><br/
<hw>Spirt</hw> <pr>(?)</pr>, <pos>v. & n.</pos> <def>Same as <er>Spurt</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spirtle</ent><br/
<hw>Spir"tle</hw> <pr>(?)</pr>, <pos>v. t.</pos> <def>To spirt in a scattering manner.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spirula</ent><br/
||<hw>Spir"u*la</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[NL., dim. of L. <ets>spira</ets> a coil.]</ety> <fld>(Zool.)</fld> <def>A genus of cephalopods having a multilocular, internal, siphunculated shell in the form of a flat spiral, the coils of which are not in contact.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spirulate</ent><br/
<hw>Spir"u*late</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Zool.)</fld> <def>Having the color spots, or structural parts, arranged spirally.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spiry</ent><br/
<hw>Spir"y</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[From <er>Spire</er> a winding line.]</ety> <def>Of a spiral form; wreathed; curled; serpentine.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Hid in the <qex>spiry</qex> volumes of the snake.</q> <rj><qau>Dryden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spiry</ent><br/
<hw>Spir"y</hw>, <pos>a.</pos> <ety>[FR. <er>Spire</er> a steeple.]</ety> <def>Of or pertaining to a spire; like a spire, tall, slender, and tapering; abounding in spires; <as>as, <ex>spiry</ex> turrets</as>.</def> <ldquo/<xex>Spiry</xex> towns.<rdquo/  <rj><au>Thomson.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spiss</ent><br/
<hw>Spiss</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>spissus</ets>.]</ety> <def>Thick; crowded; compact; dense.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>This <qex>spiss</qex> and . . . copious, yet concise, treatise.</q> <rj><qau>Brerewood.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spissated</ent><br/
<hw>Spis"sa*ted</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Rendered dense or compact, as by evaporation; inspissated; thickened.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>The <qex>spissated</qex> juice of the poppy.</q> <rj><qau>Bp. Warburton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spissitude</ent><br/
<hw>Spis"si*tude</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>spissitudo</ets>.]</ety> <def>The quality or state of being spissated; <as>as, the <ex>spissitude</ex> of coagulated blood, or of any coagulum</as>.</def>  <rj><au>Arbuthnot.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spit</ent><br/
<hw>Spit</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[OE. <ets>spite</ets>, AS. <ets>spitu</ets>; akin to D. <ets>spit</ets>, G. <ets>spiess</ets>, OHG. <ets>spiz</ets>, Dan. <ets>spid</ets>. Sw. <ets>spett</ets>, and to G. <ets>spitz</ets> pointed. <root/170.]</ety> <sn>1.</sn> <def>A long, slender, pointed rod, usually of iron, for holding meat while roasting.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A small point of land running into the sea, or a long, narrow shoal extending from the shore into the sea; <as>as, a <ex>spit</ex> of sand</as>.</def>  <rj><au>Cook.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>The depth to which a spade goes in digging; a spade; a spadeful.</def> <mark>[Prov. Eng.]</mark>  <rj><au>Halliwell.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spit</ent><br/
<hw>Spit</hw>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Spitted</conjf>; <pos>p. pr. & vb. n.</pos> <conjf>Spitting</conjf>.]</vmorph> <ety>[From <er>Spit</er>, <pos>n.</pos>; cf. <er>Speet</er>.]</ety> <sn>1.</sn> <def>To thrust a spit through; to fix upon a spit; hence, to thrust through or impale; <as>as, to <ex>spit</ex> a loin of veal</as>.</def> <ldquo/Infants <xex>spitted</xex> upon pikes.<rdquo/  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To spade; to dig.</def> <mark>[Prov. Eng.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spit</ent><br/
<hw>Spit</hw>, <pos>v. i.</pos> <def>To attend to a spit; to use a spit.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>She's <qex>spitting</qex> in the kitchen.</q> <rj><qau>Old Play.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spit</ent><br/
<hw>Spit</hw>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Spit</conjf> (<conjf>Spat</conjf>, <mark>archaic</mark>); <pos>p. pr. & vb. n.</pos> <conjf>Spitting</conjf>.]</vmorph> <ety>[AS. <ets>spittan</ets>; akin to G. <ets>sp<uum/tzen</ets>, Dan. <ets>spytte</ets>, Sw. <ets>spotta</ets>,Icel. <ets>sp<?/ta</ets>, and prob. E. <ets>spew</ets>. The past tense <ets>spat</ets> is due to AS. <ets>sp<?/tte</ets>, from <ets>sp<?/tan</ets> to spit.  Cf. <er>Spat</er>, <pos>n.</pos>, <er>Spew</er>, <er>Spawl</er>, <er>Spot</er>, <pos>n.</pos>]</ety> <sn>1.</sn> <def>To eject from the mouth; to throw out, as saliva or other matter, from the mouth.</def> <ldquo/Thus <xex>spit</xex> I out my venom.<rdquo/
<-- spat is a common pp and not archaic -->  <rj><au>Chaucer.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To eject; to throw out; to belch.</def><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ <xex>Spitted</xex> was sometimes used as the preterit and the past participle. <ldquo/He . . . shall be mocked, and spitefully entreated, and <xex>spitted</xex> on.<rdquo/</note>  <rj><au>Luke xviii. 32.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spit</ent><br/
<hw>Spit</hw>, <pos>n.</pos> <def>The secretion formed by the glands of the mouth; spitle; saliva; sputum.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spit</ent><br/
<hw>Spit</hw>, <pos>v. i.</pos> <sn>1.</sn> <def>To throw out saliva from the mouth.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To rain or snow slightly, or with sprinkles.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>It had been <qex>spitting</qex> with rain.</q> <rj><qau>Dickens.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><mcol><col><b>To spit on</b></col> <it>or</it> <col><b>To spit upon</b></col></mcol>, <cd>to insult grossly; to treat with contempt.</cd> <ldquo/<xex>Spitting upon</xex> all antiquity.<rdquo/  <rj><au>South.</au></rj></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spital</ent><br/
<hw>Spit"al</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Abbreviated from <ets>hospital</ets>.]</ety> <altsp>[Written also <asp>spittle</asp>.]</altsp> <def>A hospital.</def> <mark>[Obs.]</mark>  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spitalhouse</ent><br/
<hw>Spit"al*house`</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A hospital.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>spit ball</ent><br/
<ent>spitball</ent><br/
<mhw><hw>spit"ball`</hw>, <hw>spit" ball`</hw></mhw> <pr>(?)</pr>, <pos>n.</pos> <sn>1.</sn> <def>Paper chewed, and rolled into a ball, to be thrown as a missile; -- a childish prank.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Baseball)</fld> <def>A pitched ball in throwing which the pitcher grips the ball between two, or three, fingers on one side (which is made slippery, as by saliva) and the thumb on the other side, and delivers it so that it slips off the fingers with the least possible friction; -- also called <altname>spitter</altname>. When pitched directly overhand a spit ball darts downward, when pitched with the arm extended sidewise it darts down and out.  The use of such a pitch is against the rules of professional baseball.</def> <mark>[Cant]</mark> -- <wordforms><wf>Spit baller</wf>.</wordforms><br/
[<source>Webster 1913 Suppl.</source> <source>+PJC</source>]</p>

<p><ent>Spitbox</ent><br/
<hw>Spit"box`</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A vessel to receive spittle.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spitchcock</ent><br/
<hw>Spitch"cock`</hw> <pr>(?)</pr>, <pos>v. t.</pos> <ety>[1st <ets>spit</ets> + <ets>cock</ets>.]</ety> <fld>(Cookery)</fld> <def>To split (as an eel) lengthwise, and broil it, or fry it in hot fat.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spitchcock</ent><br/
<hw>Spitch"cock`</hw>, <pos>n.</pos> <fld>(Cookery)</fld> <def>An eel split and broiled.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spitchcocked</ent><br/
<hw>Spitch"cocked`</hw> <pr>(?)</pr>, <pos>a.</pos> <fld>(Cookery)</fld> <def>Broiled or fried after being split lengthwise; -- said of eels.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spit curl</ent><br/
<hw>Spit" curl`</hw> <pr>(?)</pr>. <def>A little lock of hair, plastered in a spiral form on the temple or forehead with spittle, or other adhesive substance.</def> <mark>[Colloq.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spite</ent><br/
<hw>Spite</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Abbreviated fr. <ets>despite</ets>.]</ety> <sn>1.</sn> <def>Ill-will or hatred toward another, accompanied with the disposition to irritate, annoy, or thwart; petty malice; grudge; rancor; despite.</def>  <rj><au>Pope.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>This is the deadly <qex>spite</qex> that angers.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Vexation; chargrin; mortification.</def> <mark>[R.]</mark>  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><mcol><col><b>In spite of</b></col>, <it>or</it> <col><b>Spite of</b></col></mcol>, <cd>in opposition to all efforts of; in defiance or contempt of; notwithstanding.</cd>  <ldquo/Continuing, <xex>spite of</xex> pain, to use a knee after it had been slightly injured.<rdquo/ <au>H. Spenser.</au> <ldquo/And saved me <xex>in spite of</xex> the world, the devil, and myself.<rdquo/ <au>South.</au> <ldquo/<xex>In spite of</xex> all applications, the patient grew worse every day.<rdquo/ <au>Arbuthnot.</au>  <see>See Syn. under <er>Notwithstanding</er>.</see> -- <col><b>To owe one a spite</b></col>, <cd>to entertain a mean hatred for him.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><syn><b>Syn.</b> -- Pique, rancor; malevolence; grudge.</syn>  -- <usage><er>Spite</er>, <er>Malice</er>. <xex>Malice</xex> has more reference to the disposition, and <xex>spite</xex> to the manifestation of it in words and actions. It is, therefore, meaner than <xex>malice</xex>, thought not always more criminal. <ldquo/ <xex>Malice</xex> . . . is more frequently employed to express the dispositions of inferior minds to execute every purpose of mischief within the more limited circle of their abilities.<rdquo/ <au>Cogan.</au> <ldquo/Consider eke, that <xex>spite</xex> availeth naught.<rdquo/ <au>Wyatt.</au> See <er>Pique</er>.</usage><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spite</ent><br/
<hw>Spite</hw>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Spited</conjf>; <pos>p. pr. & vb. n.</pos> <conjf>Spiting</conjf>.]</vmorph> <sn>1.</sn> <def>To be angry at; to hate.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>The Danes, then . . . pagans, <qex>spited</qex> places of religion.</q> <rj><qau>Fuller.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To treat maliciously; to try to injure or thwart.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>To fill with spite; to offend; to vex.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>Darius, <qex>spited</qex> at the Magi, endeavored to abolish not only their learning, but their language.</q> <rj><qau>Sir. W. Temple.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><-- p. 1389 --></p>

<p><ent>Spiteful</ent><br/
<hw>Spite"ful</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Filled with, or showing, spite; having a desire to vex, annoy, or injure; malignant; malicious; <as>as, a <ex>spiteful</ex> person or act</as>.</def> <au>Shak.</au> -- <wordforms><wf>Spite"ful*ly</wf>, <pos>adv.</pos> <wf>Spite"ful*ness</wf>, <pos>n.</pos></wordforms><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spitfire</ent><br/
<hw>Spit"fire`</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A violent, irascible, or passionate person.</def> <mark>[Colloq.]</mark>  <rj><au>Grose.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spitful</ent><br/
<hw>Spit"ful</hw> <pr>(?)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Spitfuls</plw> <pr>(<?/)</pr>.</plu> <def>A spadeful.</def> <mark>[Prov. Eng.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spitous</ent><br/
<hw>Spit"ous</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Having spite; spiteful.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spitously</ent><br/
<hw>Spit"ous*ly</hw>, <pos>adv.</pos> <def>Spitefully.</def> <mark>[Obs.]</mark>  <rj><au>Chaucer.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spitscocked</ent><br/
<hw>Spits"cocked`</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Spitchcocked.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spitted</ent><br/
<hw>Spit"ted</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[From <er>Spit</er>.]</ety> <sn>1.</sn> <def>Put upon a spit; pierced as if by a spit.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Shot out long; -- said of antlers.</def>  <rj><au>Bacon.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spitted</ent><br/
<hw>Spit"ted</hw>, <def><pos>p. p.</pos> of <er>Spit</er>, <pos>v. i.</pos>, to eject, to spit.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spitter</ent><br/
<hw>Spit"ter</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[See <er>Spit</er> to eject from the mouth.]</ety> <def>One who ejects saliva from the mouth.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spitter</ent><br/
<hw>Spit"ter</hw>, <pos>n.</pos> <ety>[See <er>Spit</er> an iron prong.]</ety> <sn>1.</sn> <def>One who puts meat on a spit.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Zool.)</fld> <def>A young deer whose antlers begin to shoot or become sharp; a brocket, or pricket.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spittle</ent><br/
<hw>Spit"tle</hw> <pr>(?)</pr>, <pos>n.</pos> <def>See <er>Spital</er>.</def> <mark>[Obs.]</mark>  <rj><au>B. Jonson.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spittle</ent><br/
<hw>Spit"tle</hw>, <pos>v. t.</pos> <ety>[See <er>Spit</er> to spade.]</ety> <def>To dig or stir with a small spade.</def> <mark>[Prov. Eng.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spittle</ent><br/
<hw>Spit"tle</hw>, <pos>n.</pos> <def>A small sort of spade.</def> <mark>[Prov. Eng.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spittle</ent><br/
<hw>Spit"tle</hw>, <pos>n.</pos> <ety>[From <er>Spit</er> to eject from the mouth: cf. <er>Spattle</er>, and AS. <ets>sp<amac/tl</ets>.]</ety> <def>The thick, moist matter which is secreted by the salivary glands; saliva; spit.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Spittle insect</b></col>. <fld>(Zool.)</fld> <cd>See <cref>Cuckoo spit</cref> <sd>(b)</sd>, under <er>Cuckoo</er>.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spittly</ent><br/
<hw>Spit"tly</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Like spittle; slimy.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spittoon</ent><br/
<hw>Spit*toon"</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A spitbox; a cuspidor.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spit-venom</ent><br/
<hw>Spit"-ven"om</hw> <pr>(?)</pr>, <pos>n.</pos> <def>Poison spittle; poison ejected from the mouth.</def> <mark>[R.]</mark>  <rj><au>Hooker.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spitz dog</ent><br/
<hw>Spitz" dog"</hw> <pr>(?)</pr>. <ety>[G. <ets>spitz</ets>, <ets>spitzhund</ets>.]</ety> <fld>(Zool.)</fld> <def>A breed of dogs having erect ears and long silky hair, usually white; -- called also <altname>Pomeranian dog</altname>, and <altname>louploup</altname>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spitzenburgh</ent><br/
<hw>Spitz"en*burgh</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A kind of red and yellow apple, of medium size and spicy flavor. It originated at Newtown, on Long Island.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Splanchnapophysis</ent><br/
||<hw>Splanch`napoph"y*sis</hw> <pr>(?)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Splanchnapophyses</plw> <pr>(#)</pr>.</plu> <ety>[NL. See <er>Splanchnic</er>, and <er>Apophysis</er>.]</ety> <fld>(Anat.)</fld> <def>Any element of the skeleton in relation with the alimentary canal, as the jaws and hyoidean apparatus.</def> -- <wordforms><wf>Splanch`nap`o*phys"i*al</wf> <pr>(#)</pr>, <pos>a.</pos></wordforms>  <rj><au> Mivart.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Splanchnic</ent><br/
<hw>Splanch"nic</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[Gr. <?/<?/<?/ an entrail.]</ety> <fld>(Anat.)</fld> <def>Of or pertaining to the viscera; visceral.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Splanchnography</ent><br/
<hw>Splanch*nog"ra*phy</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Gr. <?/<?/<?/<?/<?/ an entrail + <ets>-graphy</ets>.]</ety> <def>Splanchnology.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Splanchnology</ent><br/
<hw>Splanch*nol"o*gy</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Gr. <?/<?/<?/<?/ an entrail + <ets>-logy</ets>.]</ety> <def>That part of anatomy which treats of the viscera; also, a treatise on the viscera.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Splanchnopleure</ent><br/
<hw>Splanch"no*pleure</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Gr. <?/<?/<?/<?/ an entrail + <?/<?/<?/<?/ side.]</ety> <fld>(Anat.)</fld> <def>The inner, or visceral, one of the two lamellae into which the vertebrate blastoderm divides on either side of the notochord, and from which the walls of the enteric canal and the umbilical vesicle are developed. See <er>Somatopleure</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p>-- <wordforms><wf>Splanch`no*pleu"ric</wf> <pr>(#)</pr>, <pos>a.</pos></wordforms><br/
[<source>1913 Webster</source>]</p>

<p><ent>Splanchno-skeleton</ent><br/
<hw>Splanch`no-skel"e*ton</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Gr. <?/<?/<?/<?/ an entrail + E. <ets>skeleton</ets>.]</ety> <fld>(Anat.)</fld> <def>That part of the skeleton connected with the sense organs and the viscera.</def>  <rj><au>Owen.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Splanchnotomy</ent><br/
<hw>Splanch*not"o*my</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Gr. <?/<?/<?/<?/<?/ an entrail + <?/<?/<?/<?/ to cut.]</ety> <def>The dissection, or anatomy, of the viscera.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Splandrel</ent><br/
<hw>Splan"drel</hw> <pr>(?)</pr>, <pos>n.</pos> <def>See <er>Spandrel</er>.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Splash</ent><br/
<hw>Splash</hw> <pr>(?)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Splashed</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Splashing</conjf>.]</vmorph> <ety>[Akin to <ets>plash</ets>.]</ety> <sn>1.</sn> <def>To strike and dash about, as water, mud, etc.; to plash.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To spatter water, mud, etc., upon; to wet.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Splash</ent><br/
<hw>Splash</hw>, <pos>v. i.</pos> <def>To strike and dash about water, mud, etc.; to dash in such a way as to spatter.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Splash</ent><br/
<hw>Splash</hw>, <pos>n.</pos> <sn>1.</sn> <def>Water, or water and dirt, thrown upon anything, or thrown from a puddle or the like; also, a spot or daub, as of matter which wets or disfigures.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A noise made by striking upon or in a liquid.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Splashboard</ent><br/
<hw>Splash"board`</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A guard in the front part of vehicle, to prevent splashing by a mud or water from the horse's heels; -- in the United States commonly called <xex>dashboard</xex>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Splasher</ent><br/
<hw>Splash"er</hw> <pr>(?)</pr>, <pos>n.</pos> <sn>1.</sn> <def>One who, or that which, splashes.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>One of the guarde over the wheels, as of a carriage, locomotive, etc.</def>  <rj><au>Weale.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>A guard to keep off splashes from anything.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Splashy</ent><br/
<hw>Splash"y</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Full of dirty water; wet and muddy, so as be easily splashed about; slushy.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Splatter</ent><br/
<hw>Splat"ter</hw> <pr>(?)</pr>, <pos>v. i. & t.</pos> <def>To spatter; to splash.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Splatterdash</ent><br/
<hw>Splat"ter*dash`</hw> <pr>(?)</pr>, <pos>n.</pos> <def>Uproar.</def>  <rj><au>Jamieson.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Splay</ent><br/
<hw>Splay</hw> <pr>(?)</pr>, <pos>v. t.</pos> <ety>[Abbrev. of <ets>display</ets>.]</ety> <sn>1.</sn> <def>To display; to spread.</def> <mark>[Obs.]</mark> <ldquo/Our ensigns <xex>splayed</xex>.<rdquo/  <rj><au>Gascoigne.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To dislocate, as a shoulder bone.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>To spay; to castrate.</def> <mark>[Obs. or Prov. Eng.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>To turn on one side; to render oblique; to slope or slant, as the side of a door, window, etc.</def>  <rj><au>Oxf. Gloss.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Splay</ent><br/
<hw>Splay</hw>, <pos>a.</pos> <def>Displayed; spread out; turned outward; hence, flat; ungainly; <as>as, <ex>splay</ex> shoulders</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Sonwthing <qex>splay</qex>, something blunt-edged, unhandy, and infelicitous.</q> <rj><qau>M. Arnold.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Splay</ent><br/
<hw>Splay</hw>, <pos>a.</pos> <fld>(Arch.)</fld> <def>A slope or bevel, especially of the sides of a door or window, by which the opening is made larged at one face of the wall than at the other, or larger at each of the faces than it is between them.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Splayfoot</ent><br/
<hw>Splay"foot`</hw> <pr>(?)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Splayfeet</plw> <pr>(<?/)</pr>.</plu> <def>A foot that is abnormally flattened and spread out; flat foot.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Splayfooted</ent><br/
<ent>Splayfoot</ent><br/
<mhw>{ <hw>Splay"foot`</hw>, <hw>Splay"foot`ed</hw>  }</mhw> <pos>a.</pos> <def>Having a splayfoot or splayfeet.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Splaymouth</ent><br/
<hw>Splay"mouth`</hw> <pr>(?)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Splaymouths</plw> <pr>(<?/)</pr>.</plu> <def>A wide mouth; a mouth stretched in derision.</def>  <rj><au>Dryden.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Splaymouthed</ent><br/
<hw>Splay`mouthed"</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Having a splaymouth.</def>  <rj><au>T. Brown.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spleen</ent><br/
<hw>Spleen</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>splen</ets>, Gr. <?/<?/<?/<?/ the milt or spleen, affection of the spleen; cf. L. <ets>lien</ets>, <ets>plihan</ets>, <ets>pl<imac/han</ets>.]</ety> <sn>1.</sn> <fld>(Anat.)</fld> <def>A peculiar glandlike but ductless organ found near the stomach or intestine of most vertebrates and connected with the vascular system; the milt. Its exact function in not known.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Anger; latent spite; ill humor; malice; <as>as, to vent one's <ex>spleen</ex></as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>In noble minds some dregs remain,<br/
Not yet purged off, of <qex>spleen</qex> and sour disdain.</q> <rj><qau>Pope.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>A fit of anger; choler.</def>  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>A sudden motion or action; a fit; a freak; a whim.</def> <mark>[Obs. or R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>A thousand <qex>spleens</qex> bear her a thousand ways.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <def>Melancholy; hypochondriacal affections.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Bodies changed to various forms by <qex>spleen</qex>.</q> <rj><qau>Pope.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>There is a luxury in self-dispraise:<br/
And inward self-disparagement affords<br/
To meditative <qex>spleen</qex> a grateful feast.</q> <rj><qau>Wordsworth.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>6.</sn> <def>A fit of immoderate laughter or merriment.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>Thy silly thought enforces my <qex>spleen</qex>.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spleen</ent><br/
<hw>Spleen</hw>, <pos>v. t.</pos> <def>To dislke.</def> <mark>[Obs.]</mark>  <rj><au>Bp. Hacket.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spleened</ent><br/
<hw>Spleened</hw> <pr>(?)</pr>, <pos>a.</pos> <sn>1.</sn> <def>Deprived of the spleen.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Angered; annoyed.</def> <mark>[Obs.]</mark>  <rj><au>R. North.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spleenful</ent><br/
<hw>Spleen"ful</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Displaying, or affected with, spleen; angry; fretful; melancholy.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Myself have calmed their <qex>spleenful</qex> mutiny.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Then rode Geraint, a little <qex>spleenful</qex> yet,<br/
Across the bridge that spann'd the dry ravine.</q> <rj><qau>Tennyson.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spleenish</ent><br/
<hw>Spleen"ish</hw>, <pos>a.</pos> <def>Spleeny; affected with spleen; fretful.</def> -- <wordforms><wf>Spleen"ish*ly</wf>, <pos>adv.</pos> -- <wf>Spleen"ish*ness</wf>, <pos>n.</pos></wordforms><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spleenless</ent><br/
<hw>Spleen"less</hw>, <pos>a.</pos> <def>Having no spleen; hence, kind; gentle; mild.</def> <mark>[Obs.]</mark>  <rj><au>Chapman.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spleenwort</ent><br/
<hw>Spleen"wort`</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[<ets>Spleen</ets> + <ets>wort</ets>; cf. L. <ets>splenium</ets>, <ets>asplenium</ets>, Gr. <?/<?/<?/<?/, <?/<?/<?/<?/, <?/<?/<?/. ]</ety> <fld>(Bot.)</fld> <def>Any fern of the genus <gen>Asplenium</gen>, some species of which were anciently used as remedies for disorders of the spleen.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spleeny</ent><br/
<hw>Spleen"y</hw> <pr>(?)</pr>, <pos>a.</pos> <sn>1.</sn> <def>Irritable; peevish; fretful.</def><br/
[<source>1913 Webster</source>]</p>

<p><q><qex>Spleeny</qex> Lutheran, and not wholesome to<br/
Our cause.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Affected with nervous complaints; melancholy.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spleget</ent><br/
<hw>Spleg"et</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Cf. <er>Pledget</er>.]</ety> <fld>(Med.)</fld> <def>A cloth dipped in a liquid for washing a sore.</def>  <rj><au>Crabb.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Splenalgia</ent><br/
||<hw>Sple*nal"gi*a</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[NL., fr. Gr. <?/<?/<?/ spleen + <?/<?/<?/ pain.]</ety> <fld>(Med.)</fld> <def>Pain over the region of the spleen.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Splenculus</ent><br/
||<hw>Splen"cu*lus</hw> <pr>(?)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Splenculi</plw> <pr>(#)</pr>.</plu> <ety>[NL., dim. of L. <ets>splen</ets>.]</ety> <fld>(Anat.)</fld> <def>A lienculus.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Splendent</ent><br/
<hw>Splen"dent</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>splendens</ets>, <ets>-entis</ets>, p. pr. of <ets>splendere</ets> to shine.]</ety> <sn>1.</sn> <def>Shining; glossy; beaming with light; lustrous; <as>as, <ex>splendent</ex> planets; <ex>splendent</ex> metals</as>. See the Note under 3d <er>Luster</er>, 4.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Very conspicuous; illustrious.</def> <ldquo/Great and <xex>splendent</xex> fortunes.<rdquo/<-- MW10 "brillian; splendent genius" -->  <rj><au>Sir H. Wotton.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Splendid</ent><br/
<hw>Splen"did</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>splendidus</ets>, fr. <ets>splendere</ets> shine; cf. Lith. <ets>splend<eum/ti</ets>: cf. F. <ets>splendide</ets>.]</ety> <sn>1.</sn> <def>Possessing or displaying splendor; shining; very bright; <as>as, a <ex>splendid</ex> sun</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Showy; magnificent; sumptuous; pompous; <as>as, a <ex>splendid</ex> palace; a <ex>splendid</ex> procession or pageant.</as></def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>Illustrious; heroic; brilliant; celebrated; famous; <as>as, a <ex>splendid</ex> victory or reputation</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Splendidious</ent><br/
<hw>Splen*did"i*ous</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Splendid.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Splendidly</ent><br/
<hw>Splen"did*ly</hw> <pr>(?)</pr>, <pos>adv.</pos> <def>In a splendid manner; magnificently.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Splendidness</ent><br/
<hw>Splen"did*ness</hw>, <pos>n.</pos> <def>The quality of being splendid.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Splendidous</ent><br/
<hw>Splen"did*ous</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Splendid.</def> <mark>[Obs.]</mark>  <rj><au>B. Jonson.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Splendiferous</ent><br/
<hw>Splen*dif"er*ous</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Splendor-bearing; splendid.</def> <au>Bale (1538).</au> <ldquo/A <xex>splendiferous</xex> woman.<rdquo/ <au>Haliburton.</au> <mark>[Now used humorously.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Splendor</ent><br/
<hw>Splen"dor</hw> <pr>(?)</pr>, <pos>n.</pos><ety>[L. fr. <ets>splendere</ets> to shine: cf. F. <ets>splendeur</ets>.]</ety> <sn>1.</sn> <def>Great brightness; brilliant luster; brilliancy; <as>as, the <ex>splendor</ex> ot the sun</as>.</def>  <rj><au>B. Jonson.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Magnifience; pomp; parade; <as>as, the <ex>splendor</ex> of equipage, ceremonies, processions, and the like</as>.</def> <ldquo/Rejoice in <xex>splendor</xex> of mine own.<rdquo/  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>Brilliancy; glory; <as>as, the <ex>splendor</ex> of a victory</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><syn><b>Syn.</b> -- Luster; brilliancy; magnifience; gorgeousness; display; showiness; pomp; parade; grandeur.</syn><br/
[<source>1913 Webster</source>]</p>

<p><ent>Splendorous</ent><br/
<ent>Splendrous</ent><br/
<mhw>{ <hw>Splen"drous</hw> <pr>(?)</pr>, <hw>Splen"dor*ous</hw> <pr>(?)</pr>, }</mhw> <pos>a.</pos> <def>Splendid.</def>  <rj><au>Drayton.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Splenetic</ent><br/
<hw>Splen"e*tic</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>spleneticus</ets>: cf. F. <ets>spl<eacute/n<eacute/tique</ets>. See <er>Spleen</er>.]</ety> <def>Affected with spleen; malicious; spiteful; peevish; fretful.</def> <ldquo/<xex>Splenetic</xex> guffaw.<rdquo/  <rj><au>G. Eliot.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>You humor me when I am sick;<br/
Why not when I am <qex>splenetic</qex>?</q> <rj><qau>Pope.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><syn><b>Syn.</b> -- Morese; gloomy; sullen; peevish; fretful.</syn><br/
[<source>1913 Webster</source>]</p>

<p><ent>Splenetic</ent><br/
<hw>Splen"e*tic</hw>, <pos>n.</pos> <def>A person affected with spleen.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Splenetical</ent><br/
<hw>Sple*net"ic*al</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Splenetic.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Splenetically</ent><br/
<hw>Sple*net"ic*al*ly</hw>, <pos>adv.</pos> <def>In a splenetical manner.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Splenial</ent><br/
<hw>Sple"ni*al</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>splenium</ets> a plaster, a patch, Gr. <?/<?/<?/<?/ a bandage.]</ety> <fld>(Anat.)</fld> <sd>(a)</sd> <def>Designating the splenial bone.</def> <sd>(b)</sd> <def>Of or pertaining to the splenial bone or splenius muscle.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Splenial bone</b></col> <fld>(Anat.)</fld>, <cd>a thin splintlike bone on the inner side of the proximal portion of the mandible of many vertebrates.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Splenial</ent><br/
<hw>Sple"ni*al</hw>, <pos>n.</pos> <fld>(Anat.)</fld> <def>The splenial bone.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Splenic</ent><br/
<hw>Splen"ic</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>splenicus</ets>, Gr. <grk>splhniko`s</grk>; cf. F. <ets>spl<eacute/nique</ets>.]</ety> <fld>(Anat.)</fld> <def>Of or pertaining to the spleen; lienal; <as>as, the <ex>splenic</ex> vein</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><mcol><col><b>Splenic apoplexy</b></col> <it>or</it>  <col><b>Splenic fever</b></col></mcol>. <fld>(Med.)</fld> <cd>See <er>Anthrax</er>, <pos>n.</pos>, 3.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Splenical</ent><br/
<hw>Splen"ic*al</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Splenic.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Splenish</ent><br/
<hw>Splen"ish</hw>, <pos>a.</pos> <def>Spleenish.</def> <mark>[Obs.]</mark>  <rj><au>Drayton.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Splenitis</ent><br/
||<hw>Sple*ni"tis</hw> <pr>(?)</pr>, <pos>n.</pos><ety>[NL., fr. Gr. <grk>splhni^tis</grk> of the spleen.]</ety> <fld>(Med.)</fld> <def>Inflammation of the spleen.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Splenitive</ent><br/
<hw>Splen"i*tive</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Splenetic.</def>  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Even and smooth as seemed the temperament of the nonchalant, languid Virginian -- not <qex>splenitive</qex> or rash.</q> <rj><qau>T. N. Page.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Splenium</ent><br/
||<hw>Sple"ni*um</hw> <pr>(?)</pr>, <pos>n.</pos><ety>[L., a plaster, a patch, from Gr. <?/<?/<?/<?/ a bandage, compress.]</ety> <fld>(Anat.)</fld> <def>The thickened posterior border of the corpus callosum; -- so called in allusion to its shape.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Splenius</ent><br/
||<hw>Sple"ni*us</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[NL.]</ety> <fld>(Anat.)</fld> <def>A flat muscle of the back of the neck.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Splenization</ent><br/
<hw>Splen`i*za"tion</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Med.)</fld> <def>A morbid state of the lung produced by inflammation, in which its tissue resembles that of the spleen.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Splenocele</ent><br/
<hw>Splen"o*cele</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Gr. <grk>splh`n</grk> spleen + <?/<?/<?/ a tumor.]</ety> <fld>(Med.)</fld> <def>Hernia formed by the spleen.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Splenography</ent><br/
<hw>Sple*nog"ra*phy</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Gr. <grk>splh`n</grk> spleen + <ets>-graphy</ets>.]</ety> <def>A description of the spleen.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Splenoid</ent><br/
<hw>Sple"noid</hw> <pr>(?)</pr>, <pos>a.</pos><ety>[Gr. <grk>splh`n</grk> spleen + <ets>-oid</ets>.]</ety> <fld>(Anat.)</fld> <def>Resembling the spleen; spleenlike.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Splenology</ent><br/
<hw>Sple*nol"o*gy</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Gr. <grk>splh`n</grk> spleen + <ets>-logy</ets>.]</ety> <def>The branch of science which treats of the spleen.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Splenotomy</ent><br/
<hw>Sple*not"o*my</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Gr. <grk>splh`n</grk> spleen + <grk>te`mnein</grk> to cut.]</ety> <sd>(a)</sd> <fld>(Anat.)</fld> <def>Dissection or anatomy of the spleen.</def> <sd>(b)</sd> <fld>(Med.)</fld> <def>An incision into the spleen; removal of the spleen by incision.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Splent</ent><br/
<hw>Splent</hw> <pr>(spl<ecr/nt)</pr>, <pos>n.</pos> <sn>1.</sn> <def>See <er>Splent</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>See <cref>Splent coal</cref>, below.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Splent coal</b></col>, <cd>an inferior kind of cannel coal from Scotch collieries; -- called also <altname>splent</altname>, <altname>splint</altname>, and <altname>splint coal</altname>.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spleuchan</ent><br/
<hw>Spleu"chan</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Gael. <ets>spliuchan</ets>.]</ety> <def>A pouch, as for tobacco.</def> <mark>[Scot.]</mark>  <rj><au>Sir W. Scott.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Splice</ent><br/
<hw>Splice</hw> <pr>(spl<imac/s)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Spliced</conjf> <pr>(spl<imac/st)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Splicing</conjf> <pr>(?)</pr>.]</vmorph> <ety>[D. <ets>splitsen</ets>, <ets>splitten</ets>; akin to G. <ets>splissen</ets>, Sw. <ets>splissa</ets>, Dan. <ets>splisse</ets>, and E. <ets>split</ets>; -- from the dividing or splitting the ends into separate strands. See <er>Split</er>, <pos>v. t.</pos>]</ety> <sn>1.</sn> <def>To unite, as two ropes, or parts of a rope, by a particular manner of interweaving the strands, -- the union being between two ends, or between an end and the body of a rope.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To unite, as spars, timbers, rails, etc., by lapping the two ends together, or by applying a piece which laps upon the two ends, and then binding, or in any way making fast.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>To unite in marriage.</def> <mark>[Slang]</mark><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Splice grafting</b></col>.<cd>ee under <er>Grafting</er>.</cd> -- <col><b>To splice the main brace</b></col> <fld>(Naut.)</fld>, <cd>to give out, or drink, an extra allowance of spirits on occasion of special exposure to wet or cold, or to severe fatigue; hence, to take a dram.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Splice</ent><br/
<hw>Splice</hw>, <pos>n.</pos> <def>A junction or joining made by splicing.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spline</ent><br/
<hw>Spline</hw> <pr>(?)</pr>, <pos>n.</pos> <sn>1.</sn> <def>A rectangular piece fitting grooves like key seats in a hub and a shaft, so that while the one may slide endwise on the other, both must revolve together; a feather; also, sometimes, a groove to receive such a rectangular piece.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A long, flexble piece of wood sometimes used as a ruler.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Splining</ent><br/
<hw>Splin"ing</hw>, <pos>a.</pos> <def>Of or pertaining to a spline.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Splining machine</b></col>, <cd>a machine tool for cutting grooves, key seats, or slots; a slotting machine.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Splint</ent><br/
<hw>Splint</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Akin to D. <ets>splinter</ets>,G. <ets>splinter</ets>, <ets>splitter</ets>, Dan. <ets>splint</ets>, Sw. <ets>splint</ets> a kind of spike, a forelock (in nautical use), Sw. <ets>splinta</ets>to splint, splinter, Dan. <ets>splinte</ets>, and E. <ets>split</ets>. See <er>Split</er>, <pos>v. t.</pos>, and cf. <er>Splent</er>.]</ety> <sn>1.</sn> <def>A piece split off; a splinter.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Surg.)</fld> <def>A thin piece of wood, or other substance, used to keep in place, or protect, an injured part, especially a broken bone when set.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <fld>(Anat.)</fld> <def>A splint bone.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <fld>(Far.)</fld> <def>A disease affecting the splint bones, as a callosity or hard excrescence.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <fld>(Anc. Armor.)</fld> <def>One of the small plates of metal used in making splint armor. See <cref>Splint armor</cref>, below.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The knees and feet were defended by <qex>splints</qex>, or thin plates of steel.</q> <rj><qau>Sir. W. Scott.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>6.</sn> <def>Splint, or splent, coal. See <cref>Splent coal</cref>, under <er>Splent</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Splint armor</b></col>,<cd>a kind of ancient armor formed of thin plates of metal, usually overlapping each other and allowing the limbs to move freely.</cd> -- <col><b>Splint bone</b></col> <fld>(Anat.)</fld>, <cd>one of the rudimentary, splintlike metacarpal or metatarsal bones on either side of the cannon bone in the limbs of the horse and allied animals.</cd> -- <col><b>Splint coal</b></col>. <cd>See <cref>Splent coal</cref>, under <er>Splent</er>.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Splint</ent><br/
<hw>Splint</hw>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Splinted</conjf>; <pos>p. pr. & vb. n.</pos> <conjf>Splinting</conjf>.]</vmorph> <def>To split into splints, or thin, slender pieces; to splinter; to shiver.</def> <mark>[Obs. or R.]</mark>  <rj><au>Florio.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To fasten or confine with splints, as a broken limb. See <er>Splint</er>, <pos>n.</pos>, 2.</def> <mark>[R.]</mark>  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Splinter</ent><br/
<hw>Splin"ter</hw> <pr>(?)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Splintered</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Splintering</conjf>.]</vmorph> <ety>[Cf. LG. <ets>splittern</ets>, <ets>splinteren</ets>. See <er>Splint</er>, <pos>n.</pos>, <er>Split</er>.]</ety> <sn>1.</sn> <def>To split or rend into long, thin pieces; to shiver; <as>as, the lightning <ex>splinters</ex> a tree</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>After <qex>splintering</qex> their lances, they wheeled about, and . . . abandoned the field to the enemy.</q> <rj><qau>Prescott.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To fasten or confine with splinters, or splints, as a broken limb.</def>  <rj><au>Bp. Wren.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Splinter</ent><br/
<hw>Splin"ter</hw>, <pos>v. i.</pos> <def>To become split into long pieces.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Splinter</ent><br/
<hw>Splin"ter</hw>, <pos>n.</pos> <ety>[See <er>Splinter</er>, <pos>v.</pos>, or <er>Splint</er>, <pos>n.</pos>]</ety> <def>A thin piece split or rent off lengthwise, as from wood, bone, or other solid substance; a thin piece; a sliver; <as>as, <ex>splinters</ex> of a ship's mast rent off by a shot</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><-- p. 1390 pr=VMG --></p>

<p><cs><col><b>Splinter bar</b></col>. <sd>(a)</sd> <cd>A crossbar in a coach, which supports the springs</cd>. <sd>(b)</sd> <cd>The bar to which the traces are attached; a roller bolt; a whiffletree.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Splinterproof</ent><br/
<hw>Splin"ter*proof`</hw> <pr>(spl<icr/n"t<etil/r*pr<oomac/f`)</pr>, <pos>a.</pos> <fld>(Mil.)</fld> <def>Proof against the splinters, or fragments, of bursting shells.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Splintery</ent><br/
<hw>Splin"ter*y</hw> <pr>(spl<icr/n"t<etil/r*<ycr/)</pr>, <pos>a.</pos> <def>Consisting of splinters; resembling splinters; <as>as, the <ex>splintery</ex> fracture of a mineral</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Split</ent><br/
<hw>Split</hw> <pr>(spl<icr/t)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Split</conjf> (<conjf>Splitted</conjf>, <mark>R.</mark>); <pos>p. pr. & vb. n.</pos> <conjf>Splitting</conjf>.]</vmorph> <ety>[Probably of Scand. or Low German origin; cf. Dan. <ets>splitte</ets>, LG. <ets>splitten</ets>, OD. <ets>splitten</ets>, <ets>spletten</ets>, D. <ets>splijten</ets>, G. <ets>spleissen</ets>, MHG. <ets>spl<imac/zen</ets>.  Cf. <er>Splice</er>, <er>Splint</er>, <er>Splinter</er>.]</ety> <sn>1.</sn> <def>To divide lengthwise; to separate from end to end, esp. by force; to divide in the direction of the grain or layers; to rive; to cleave; <as>as, to <ex>split</ex> a piece of timber or a board; to <ex>split</ex> a gem; to <ex>split</ex> a sheepskin</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Cold winter <qex>split</qex> the rocks in twain.</q> <rj><qau>Dryden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To burst; to rupture; to rend; to tear asunder.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>A huge vessel of exceeding hard marble <qex>split</qex> asunder by congealed water.</q> <rj><qau>Boyle.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>To divide or break up into parts or divisions, as by discord; to separate into parts or parties, as a political party; to disunite.</def> <mark>[Colloq.]</mark>  <rj><au>South.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <fld>(Chem.)</fld> <def>To divide or separate into components; -- often used with <xex>up</xex>; <as>as, to <ex>split</ex> up sugar into alcohol and carbonic acid</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>To split hairs</b></col>, <cd>to make distinctions of useless nicety.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Split</ent><br/
<hw>Split</hw>, <pos>v. i.</pos> <sn>1.</sn> <def>To part asunder; to be rent; to burst; <as>as, vessels <ex>split</ex> by the freezing of water in them</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To be broken; to be dashed to pieces.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The ship <qex>splits</qex> on the rock.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>To separate into parties or factions.</def> <mark>[Colloq.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>To burst with laughter.</def> <mark>[Colloq.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>Each had a gravity would make you <qex>split</qex>.</q> <rj><qau>Pope.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <def>To divulge a secret; to betray confidence; to peach.</def> <mark>[Slang]</mark>  <rj><au>Thackeray.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>6.</sn> <fld>(Blackjack)</fld> <def>To divide one hand of blackjack into two hands; -- a strategy allowed to a player when the first two cards dealt to the player have the same value.</def><br/
[<source>PJC</source>]</p>

<p><sn>7.</sn> <def>To leave; to depart (from a place or gathering); <as>as, let's <ex>split</ex></as>.</def> <mark>[Slang]</mark><br/
[<source>PJC</source>]</p>

<p><cs><col><b>To split on a rock</b></col>, <cd>to fail; to to err fatally; to have the hopes and designs frustrated.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Split</ent><br/
<hw>Split</hw>, <pos>n.</pos> <sn>1.</sn> <def>A crack, rent, or longitudinal fissure.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A breach or separation, as in a political party; a division.</def> <mark>[Colloq.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>A piece that is split off, or made thin, by splitting; a splinter; a fragment.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <specif>Specif:</specif> <fld>(Leather Manuf.)</fld>, <def>One of the sections of a skin made by dividing it into two or more thicknesses.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <fld>(Faro)</fld> <def>A division of a stake happening when two cards of the kind on which the stake is laid are dealt in the same turn.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>6.</sn> <sd>(a)</sd> <fld>(Basketwork)</fld> <def>Any of the three or four strips into which osiers are commonly cleft for certain kinds of work; -- usually in <pluf>pl.</pluf></def> <sd>(b)</sd> <fld>(Weaving)</fld> <def>Any of the dents of a reed.</def> <sd>(c)</sd> <def>Any of the air currents in a mine formed by dividing a larger current.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><sn>7.</sn>  <def>Short for <er>Split shot</er> or <er>split stroke</er>.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><sn>8.</sn>  <fld>(Gymnastics)</fld> <def>The feat of going down to the floor so that the legs extend in a straight line, either with one on each side or with one in front and the other behind.</def> <mark>[Cant or Slang]</mark><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><sn>9.</sn>  <def>A small bottle (containing about half a pint) of some drink; -- so called as containing half the quantity of the customary smaller commercial size of bottle; also, a drink of half the usual quantity; a half glass.</def> <mark>[Cant or Slang]</mark><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><sn>10.</sn> <fld>(Finance)</fld> <def>The substitution of more than one share of a corporation's stock for one share.  The market price of the stock usually drops in proportion to the increase in outstanding shares of stock.  The <ex>split</ex> may be in any ratio, <as>as, a two-for-one <ex>split</ex>; a three-for-two <ex>split</ex></as>.</def><br/
[<source>PJC</source>]</p>

<p><sn>11.</sn> <fld>(Blackjack)</fld> <def>The division by a player of one hand of blackjack into two hands, allowed when the first two cards dealt to a player have the same value; the player who chooses to split is obliged to increase the amount wagered by placing a sum equal to the original bet on the new hand thus created.  See <er>split</er>{6}, <pos>v.i.</pos></def><br/
[<source>PJC</source>]</p>

<p><ent>Split</ent><br/
<hw>Split</hw>, <pos>a.</pos> <sn>1.</sn> <def>Divided; cleft.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Bot.)</fld> <def>Divided deeply; cleft.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <fld>(Exchanges)</fld> <sd>(a)</sd> <def>Divided so as to be done or executed part at one time or price and part at another time or price; -- said of an order, sale, etc.</def> <sd>(b)</sd> <def>Of quotations, given in sixteenth, quotations in eighths being regular; <as>as, 10<frac3x16/ is a <ex>split</ex> quotation</as>.</def> <sd>(c)</sd> <fld>(London Stock Exchange)</fld> <def>Designating ordinary stock that has been divided into preferred ordinary and deferred ordinary.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><cs><col><b>Split pease</b></col>, <cd>hulled pease split for making soup, etc.</cd> -- <col><b>Split pin</b></col> <fld>(Mach.)</fld>, <cd>a pin with one end split so that it may be spread open to secure it in its place.</cd> -- <col><b>Split pulley</b></col>, <cd>a parting pulley. See under <er>Pulley</er>.</cd> -- <col><b>Split ring</b></col>, <cd>a ring with overlapped or interlocked ends which may be sprung apart so that objects, as keys, may be strung upon the ring or removed from it.</cd> -- <col><b>Split ticket</b></col>, <cd>a ballot in which a voter votes for a portion of the candidates nominated by one party, candidates of other parties being substituted for those omitted.</cd> <mark>[U.S.]</mark></cs><br/
[<source>1913 Webster</source> <source>+PJC</source>]</p>

<p><ent>Split dynamometer</ent><br/
<hw>Split dynamometer</hw>. <fld>(Elec.)</fld> <def>An electric dynamometer having two coils so arranged that one carries the primary current, and the other the secondary current, of a transformer.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Splitfeet</ent><br/
<hw>Split"feet`</hw> <pr>(spl<icr/t"f<emac/t`)</pr>, <pos>n. pl.</pos> <fld>(Zool.)</fld> <def>The Fissipedia.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Split infinitive</ent><br/
<hw>Split infinitive</hw>. <fld>(Gram.)</fld> <def>A simple infinitive with <it>to</it>, having a modifier between the verb and the <it>to</it>; as in, <xex>to largely decrease</xex>. Called also <altname>cleft infinitive</altname>.</def>  <note>The use of the <ex>split infinitive</ex> is commonly considered to be ungrammatical, but by most grammarians it is considered acceptible.</note><br/
[<source>Webster 1913 Suppl.</source> <source>+PJC</source>]</p>

<p><ent>Split key</ent><br/
<hw>Split key</hw>. <fld>(Mach.)</fld> <def>A key split at one end like a split pin, for the same purpose.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>split stroke</ent><br/
<ent>split shot</ent><br/
<mhw>{ <hw>split shot</hw> <it>or</it>  <hw>split stroke</hw> }</mhw>. <def>In croquet, etc., a shot or stroke in which one drives in different directions one's own and the opponent's ball placed in contact.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Split stitch</ent><br/
<hw>Split stitch</hw>. <def>A stitch used in stem work to produce a fine line, much used in old church embroidery to work the hands and faces of figures.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Split stuff</ent><br/
<hw>Split stuff</hw>. <def>Timber sawn into lengths and then split.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Split switch</ent><br/
<hw>Split switch</hw>. <fld>(Railroading)</fld> <def>same as <er>Point switch</er>.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Split-tail</ent><br/
<hw>Split"-tail`</hw> <pr>(-t<amac/l`)</pr>, <pos>n.</pos> <fld>(Zool.)</fld> <sd>(a)</sd> <def>A California market fish (<spn>Pogonichthys macrolepidotus</spn>) belonging to the Carp family.</def> <sd>(b)</sd> <def>The pintail duck.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Splitter</ent><br/
<hw>Split"ter</hw> <pr>(-t<etil/r)</pr>, <pos>n.</pos> <def>One who, or that which, splits.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Split-tongued</ent><br/
<hw>Split"-tongued`</hw> <pr>(spl<icr/t"t<ucr/ngd`)</pr>, <pos>a.</pos> <fld>(Zool.)</fld> <def>Having a forked tongue, as that of snakes and some lizards.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Split wheel</ent><br/
<hw>Split wheel</hw>. <def>same as <er>Split pulley</er>.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Splotch</ent><br/
<hw>Splotch</hw> <pr>(spl<ocr/ch)</pr>, <pos>n.</pos> <ety>[Cf. <er>Splash</er>.]</ety> <def>A spot; a stain; a daub.</def>  <rj><au>R. Browning.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Splotchy</ent><br/
<hw>Splotch"y</hw> <pr>(-<ycr/)</pr>, <pos>a.</pos> <def>Covered or marked with splotches.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Splurge</ent><br/
<hw>Splurge</hw> <pr>(spl<ucir/rj)</pr>, <pos>n.</pos> <def>A blustering demonstration, or great effort; a great display.</def> <mark>[Slang, U.S.]</mark>  <rj><au>Bartlett.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Splurge</ent><br/
<hw>Splurge</hw>, <pos>v. i.</pos> <def>To make a great display in any way, especially in oratory.</def> <mark>[Slang, U.S.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><-- 2. To spend money freely or extravagantly, in one episode; usu. with "on" -- as, to <ex>splurge</ex> on a new hi-fi system. --><br/
[<source>1913 Webster</source>]</p>

<p><-- v.t. to spend extravagantly. --><br/
[<source>1913 Webster</source>]</p>

<p><ent>Splutter</ent><br/
<hw>Splut"ter</hw> <pr>(spl<ucr/t"t<etil/r)</pr>, <pos>v. i.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Spluttered</conjf> <pr>(-t<etil/rd)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Spluttering</conjf>.]</vmorph> <ety>[Prov. E. <ets>splutter</ets>, eqivalent to <ets>sputter</ets>.  Cf. <er>Sputter</er>.]</ety> <def>To speak hastily and confusedly; to sputter.</def> <mark>[Colloq.]</mark>  <rj><au>Carleton.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Splutter</ent><br/
<hw>Splut"ter</hw>, <pos>n.</pos> <def>A confused noise, as of hasty speaking.</def> <mark>[Colloq.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Splutterer</ent><br/
<hw>Splut"ter*er</hw> <pr>(spl<ucr/t"t<etil/r<etil/r)</pr>, <pos>n.</pos> <def>One who splutters.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spodomancy</ent><br/
<hw>Spod"o*man`cy</hw> <pr>(sp<ocr/d"<osl/*m<acr/n`s<ycr/)</pr>, <pos>n.</pos> <ety>[Gr. <grk>spodo`s</grk> ashes + <ets>-mancy</ets>.]</ety> <def>Divination by means of ashes.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spodomantic</ent><br/
<hw>Spod`o*man"tic</hw> <pr>(sp<ocr/d"<osl/*m<acr/n"t<icr/k)</pr>, <pos>a.</pos> <def>Relating to spodomancy, or divination by means of ashes.</def>  <rj><au>C. Kingsley.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spodumene</ent><br/
<hw>Spod"u*mene</hw> <pr>(sp<ocr/d"<usl/*m<emac/n; 135)</pr>, <pos>n.</pos> <ety>[Gr. <?/, p. pr. pass. from <?/ to burn to ashes, from <grk>spodo`s</grk> ashes; cf. F. <ets>spodum<egrave/ne</ets>.]</ety> <fld>(Min.)</fld> <def>A mineral of a white to yellowish, purplish, or emerald-green color, occurring in prismatic crystals, often of great size.  It is a silicate of alumina and lithia. See <er>Hiddenite</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spoffish</ent><br/
<hw>Spof"fish</hw> <pr>(sp<ocr/f"f<icr/sh)</pr>, <pos>a.</pos> <ety>[Probably from Prov. E. <ets>spoffle</ets> to be spoffish.]</ety> <def>Earnest and active in matters of no moment; bustling.</def> <mark>[Colloq. Eng.]</mark>  <rj><au>Dickens.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spoil</ent><br/
<hw>Spoil</hw> <pr>(spoil)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Spoiled</conjf> <pr>(spoild)</pr> or <conjf>Spoilt</conjf> <pr>(spoilt)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Spoiling</conjf>.]</vmorph> <ety>[F. <ets>spolier</ets>, OF. <ets>espoillier</ets>, fr. L. <ets>spoliare</ets>, fr. <ets>spolium</ets> spoil.  Cf. <er>Despoil</er>, <er>Spoliation</er>.]</ety> <sn>1.</sn> <def>To plunder; to strip by violence; to pillage; to rob; -- with <xex>of</xex> before the name of the thing taken; <as>as, to <ex>spoil</ex> one of his goods or possessions</as>.</def> <ldquo/Ye shall <xex>spoil</xex> the Egyptians.<rdquo/  <rj><au>Ex. iii. 22.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>My sons their old, unhappy sire despise,<br/
<qex>Spoiled</qex> of his kingdom, and deprived of eyes.</q> <rj><qau>Pope.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To seize by violence; to take by force; to plunder.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>No man can enter into a strong man's house, and <qex>spoil</qex> his goods, except he will first bind the strong man.</q> <rj><qau>Mark iii. 27.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>To cause to decay and perish; to corrupt; to vitiate; to mar.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Spiritual pride <qex>spoils</qex> many graces.</q> <rj><qau>Jer. Taylor.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>To render useless by injury; to injure fatally; to ruin; to destroy; <as>as, to <ex>spoil</ex> paper; to have the crops <ex>spoiled</ex> by insects; to <ex>spoil</ex> the eyes by reading.</as></def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spoil</ent><br/
<hw>Spoil</hw> <pr>(spoil)</pr>, <pos>v. i.</pos> <sn>1.</sn> <def>To practice plunder or robbery.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Outlaws, which, lurking in woods, used to break forth to rob and <qex>spoil</qex>.</q> <rj><qau>Spenser.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To lose the valuable qualities; to be corrupted; to decay; <as>as, fruit will soon <ex>spoil</ex> in warm weather</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spoil</ent><br/
<hw>Spoil</hw>, <pos>n.</pos> <ety>[Cf. OF. <ets>espoille</ets>, L. <ets>spolium</ets>.]</ety> <sn>1.</sn> <def>That which is taken from another by violence; especially, the plunder taken from an enemy; pillage; booty.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Gentle gales,<br/
Fanning their odoriferous wings, dispense<br/
Native perfumes, and whisper whence they stole<br/
Those balmy <qex>spoils</qex>.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Public offices and their emoluments regarded as the peculiar property of a successful party or faction, to be bestowed for its own advantage; -- commonly in the plural; <as>as, to the victor belong the <ex>spoils</ex></as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>From a principle of gratitude I adhered to the coalition; my vote was counted in the day of battle, but I was overlooked in the division of the <qex>spoil</qex>.</q> <rj><qau>Gibbon.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>That which is gained by strength or effort.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Each science and each art his <qex>spoil</qex>.</q> <rj><qau>Bentley.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>The act or practice of plundering; robbery; waste.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The man that hath no music in himself,<br/
Nor is not moved with concord of sweet sounds,<br/
Is fit for treason, stratagems, and <qex>spoils</qex>.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <def>Corruption; cause of corruption.</def> <mark>[Archaic]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>Villainous company hath been the <qex>spoil</qex> of me.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>6.</sn> <def>The slough, or cast skin, of a serpent or other animal.</def> <mark>[Obs.]</mark>  <rj><au>Bacon.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Spoil bank</b></col>, <cd>a bank formed by the earth taken from an excavation, as of a canal.</cd> -- <col><b>The spoils system</b></col>, <cd>the theory or practice of regarding public offices and their emoluments as so much plunder to be distributed among their active partisans by those who are chosen to responsible offices of administration.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spoilable</ent><br/
<hw>Spoil"a*ble</hw> <pr>(spoil"<adot/*b'l)</pr>, <pos>a.</pos> <def>Capable of being spoiled.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spoiler</ent><br/
<hw>Spoil"er</hw> <pr>(spoil"<etil/r)</pr>, <pos>n.</pos> <sn>1.</sn> <def>One who spoils; a plunderer; a pillager; a robber; a despoiler.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>One who corrupts, mars, or renders useless.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <fld>(Aeronautics)</fld> <def>A device attached to the wing of an airplane or other airfoil, which breaks the smooth flow of air during flight and decreases the lift of the airfoil; -- used to control the attitude of the airplane during banking or descent.</def><br/
[<source>PJC</source>]</p>

<p><sn>4.</sn> <def>A device attached to a car to decrease lift and increase traction, usually shaped as a flat surface and attached above the rear of the car, and working on the same principal as the aircraft spoiler{3}.</def><br/
[<source>PJC</source>]</p>

<p><ent>Spoilfive</ent><br/
<hw>Spoil"five`</hw> <pr>(spoil"f<imac/v`)</pr>, <pos>n.</pos> <def>A certain game at cards in which, if no player wins three of the five tricks possible on any deal, the game is said to be <xex>spoiled</xex>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spoilful</ent><br/
<hw>Spoil"ful</hw> <pr>(spoil"f<usdot/l)</pr>, <pos>a.</pos> <def>Wasteful; rapacious.</def> <mark>[Poetic]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spoilsman</ent><br/
<hw>Spoils"man</hw> <pr>(spoilz"m<ait/n)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Spoilsmen</plw> <pr>(spoilz"m<eit/n)</pr>.</plu> <def>One who serves a cause or a party for a share of the spoils; in United States politics, one who makes or recognizes a demand for public office on the ground of partisan service; also, one who sanctions such a policy in appointments to the public service.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spoilsmonger</ent><br/
<hw>Spoils"mon`ger</hw> <pr>(spoilz"m<ucr/<nsm/`g<etil/r)</pr>, <pos>n.</pos> <def>One who promises or distributes public offices and their emoluments as the price of services to a party or its leaders.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spoke</ent><br/
<hw>Spoke</hw> <pr>(sp<omac/k)</pr>, <def><pos>imp.</pos> of <er>Speak</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spoke</ent><br/
<hw>Spoke</hw>, <pos>n.</pos> <ety>[OE. <ets>spoke</ets>, <ets>spake</ets>, AS. <ets>sp<amac/ca</ets>; akin to D. <ets>speek</ets>, LG. <ets>speke</ets>, OHG. <ets>speihha</ets>, G. <ets>speiche</ets>. <root/170.  Cf. <er>Spike</er> a nail.]</ety> <sn>1.</sn> <def>The radius or ray of a wheel; one of the small bars which are inserted in the hub, or nave, and which serve to support the rim or felly.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Naut.)</fld> <def>A projecting handle of a steering wheel.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>A rung, or round, of a ladder.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>A contrivance for fastening the wheel of a vehicle, to prevent it from turning in going down a hill.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>To put a spoke in one's wheel</b></col>, <cd>to thwart or obstruct one in the execution of some design.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spoke</ent><br/
<hw>Spoke</hw>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Spoked</conjf> <pr>(sp<omac/kt)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Spoking</conjf>.]</vmorph> <def>To furnish with spokes, as a wheel.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spoken</ent><br/
<hw>Spo"ken</hw> <pr>(sp<omac/"k'n)</pr>, <pos>a.</pos> <ety>[p. p. of <er>Speak</er>.]</ety> <sn>1.</sn> <def>Uttered in speech; delivered by word of mouth; oral; <as>as, a <ex>spoken</ex> narrative; the <ex>spoken</ex> word</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Characterized by a certain manner or style in speaking; -- often in composition; <as>as, a pleasant-<ex>spoken</ex> man</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Methinks you 're better <qex>spoken</qex>.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spokeshave</ent><br/
<hw>Spoke"shave`</hw> <pr>(sp<omac/k"sh<amac/v`)</pr>, <pos>n.</pos> <def>A kind of drawing knife or planing tool for dressing the spokes of wheels, the shells of blocks, and other curved work.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spokesman</ent><br/
<hw>Spokes"man</hw> <pr>(sp<omac/ks"m<ait/n)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Spokesmen</plw> <pr>(-m<eit/n)</pr>.</plu> <ety>[<ets>Speak</ets>, <ets>spoke</ets> + <ets>man</ets>.]</ety> <def>One who speaks for another.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>He shall be thy <qex>spokesman</qex> unto the people.</q> <rj><qau>Ex. iv. 16.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spoliate</ent><br/
<hw>Spo"li*ate</hw> <pr>(sp<omac/"l<icr/*<amac/t)</pr>, <pos>v. t. & i.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Spoliated</conjf> <pr>(-<amac/`t<ecr/d)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Spoliating</conjf> <pr>(-<amac/`t<icr/ng)</pr>.]</vmorph> <ety>[L. <ets>spoliatus</ets>, p. p. of <ets>spoliare</ets> to spoil. See <er>Spoil</er>, <pos>v. t.</pos>]</ety> <def>To plunder; to pillage; to despoil; to rob.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spoliation</ent><br/
<hw>Spo`li*a"tion</hw> <pr>(sp<omac/"l<icr/*<amac/"sh<ucr/n)</pr>, <pos>n.</pos> <ety>[L. <ets>spoliatio</ets>: cf. F. <ets>spoliation</ets>. See <er>Spoil</er>, <pos>v. t.</pos>]</ety> <sn>1.</sn> <def>The act of plundering; robbery; deprivation; despoliation.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Legal <qex>spoliation</qex>, which will impoverish one part of the community in order to corrupt the remainder.</q> <rj><qau>Sir G. C. Lewis.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Robbery or plunder in war; especially, the authorized act or practice of plundering neutrals at sea.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <fld>(Eccl. Law)</fld> <sd>(a)</sd> <def>The act of an incumbent in taking the fruits of his benefice without right, but under a pretended title.</def>  <rj><au>Blackstone.</au></rj> <sd>(b)</sd> <def>A process for possession of a church in a spiritual court.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <fld>(Law)</fld> <def>Injury done to a document.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spoliative</ent><br/
<hw>Spo"li*a*tive</hw> <pr>(sp<omac/"l<icr/*<asl/*t<icr/v)</pr>, <pos>a.</pos> <ety>[Cf. F. <ets>spoliatif</ets>.]</ety> <def>Serving to take away, diminish, or rob; esp. <fld>(Med.)</fld>, serving to diminish sensibly the amount of blood in the body; <as>as, <ex>spoliative</ex> bloodletting</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spoliator</ent><br/
<hw>Spo"li*a`tor</hw> <pr>(sp<omac/"l<icr/*<amac/`t<etil/r)</pr>, <pos>n.</pos> <def>One who spoliates; a spoiler.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spoliatory</ent><br/
<hw>Spo"li*a*to*ry</hw> <pr>(sp<omac/"l<icr/*<adot/*t<osl/*r<ycr/)</pr>, <pos>a.</pos> <def>Tending to spoil; destructive; spoliative.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spondaical</ent><br/
<ent>Spondaic</ent><br/
<mhw>{ <hw>Spon*da"ic</hw> <pr>(sp<ocr/n*d<amac/"<icr/k)</pr>, <hw>Spon*da"ic*al</hw> <pr>(-<icr/*k<ait/l)</pr>, }</mhw> <pos>a.</pos> <ety>[L. <ets>spondaicus</ets>, <ets>spondiacus</ets>, Gr. <?/: cf. F. <ets>sponda<ium/que</ets>.]</ety> <sn>1.</sn> <def>Of or pertaining to a spondee; consisting of spondees.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Containing spondees in excess; marked by spondees; <as>as, a <ex>spondaic</ex> hexameter, <it>i. e.</it>, one which has a spondee instead of a dactyl in the fifth foot</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spondee</ent><br/
<hw>Spon"dee</hw> <pr>(sp<ocr/n"d<emac/)</pr>, <pos>n.</pos> <ety>[L. <ets>spondeus</ets>, Gr. <?/ (sc. <?/), fr. <?/ a drink offering, libation, fr. <?/ to pour out, make a libation: cf. F. <ets>spond<eacute/e</ets>. So called because at libations slow, solemn melodies were used, chiefly in this meter.]</ety> <fld>(Pros.)</fld> <def>A poetic foot of two long syllables, as in the Latin word <xex>l<emac/g<emac/s</xex>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spondulics</ent><br/
<hw>Spon*du"lics</hw> <pr>(sp<ocr/n*d<umac/"l<icr/ks)</pr>, <pos>n.</pos> <def>Money.</def> <mark>[Slang, U.S.]</mark>  <rj><au>Bartlett.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spondyle</ent><br/
<ent>Spondyl</ent><br/
<mhw>{ <hw>Spon"dyl</hw>, <hw>Spon"dyle</hw>  }</mhw> <pr>(sp<ocr/n"d<icr/l)</pr>, <pos>n.</pos> <ety>[L. <ets>spondylus</ets>, Gr. <?/, <?/: cf. F. <ets>spondyle</ets>.]</ety> <fld>(Anat.)</fld> <def>A joint of the backbone; a vertebra.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spong</ent><br/
<hw>Spong</hw> <pr>(sp<ocr/ng)</pr>, <pos>n.</pos> <ety>[Etymol. uncertain.]</ety> <def>An irregular, narrow, projecting part of a field.</def> <mark>[Prov. Eng.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sponge</ent><br/
<hw>Sponge</hw> <pr>(sp<ucr/nj)</pr>, <pos>n.</pos> <ety>[OF. <ets>esponge</ets>, F. <ets><eacute/ponge</ets>, L. <ets>spongia</ets>, Gr. <grk>spoggia`</grk>, <grk>spo`ggos</grk>.  Cf. <er>Fungus</er>, <er>Spunk</er>.]</ety> <altsp>[Formerly written also <asp>spunge</asp>.]</altsp> <sn>1.</sn> <fld>(Zool.)</fld> <def>Any one of numerous species of Spongiae, or Porifera. See <xex>Illust.</xex> and Note under <er>Spongiae</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>The elastic fibrous skeleton of many species of horny Spongiae (Keratosa), used for many purposes, especially the varieties of the genus <gen>Spongia</gen>.  The most valuable sponges are found in the Mediterranean and the Red Sea, and on the coasts of Florida and the West Indies.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <mark>Fig.:</mark> <def>One who lives upon others; a pertinacious and indolent dependent; a parasite; a sponger.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>Any spongelike substance.</def> <specif>Specifically:</specif> <sd>(a)</sd> <def>Dough before it is kneaded and formed into loaves, and after it is converted into a light, spongy mass by the agency of the yeast or leaven.</def> <sd>(b)</sd> <def>Iron from the puddling furnace, in a pasty condition.</def> <sd>(c)</sd> <def>Iron ore, in masses, reduced but not melted or worked.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <fld>(Gun.)</fld> <def>A mop for cleaning the bore of a cannon after a discharge. It consists of a cylinder of wood, covered with sheepskin with the wool on, or cloth with a heavy looped nap, and having a handle, or staff.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>6.</sn> <fld>(Far.)</fld> <def>The extremity, or point, of a horseshoe, answering to the heel.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Bath sponge</b></col>, <cd>any one of several varieties of coarse commercial sponges, especially <spn>Spongia equina</spn>.</cd> -- <col><b>Cup sponge</b></col>, <cd>a toilet sponge growing in a cup-shaped form.</cd> -- <col><b>Glass sponge</b></col>. <cd>See <er>Glass-sponge</er>, in the Vocabulary.</cd> -- <col><b>Glove sponge</b></col>, <cd>a variety of commercial sponge (<spn>Spongia officinalis</spn>, variety <varn>tubulifera</varn>), having very fine fibers, native of Florida, and the West Indies.</cd> -- <col><b>Grass sponge</b></col>, <cd>any one of several varieties of coarse commercial sponges having the surface irregularly tufted, as <spn>Spongia graminea</spn>, and <spn>Spongia equina</spn>, variety <varn>cerebriformis</varn>, of Florida and the West Indies.</cd> -- <col><b>Horse sponge</b></col>, <cd>a coarse commercial sponge, especially <spn>Spongia equina</spn>.</cd> -- <col><b>Platinum sponge</b></col>. <fld>(Chem.)</fld> <cd>See under <er>Platinum</er>.</cd> -- <col><b>Pyrotechnical sponge</b></col>, <cd>a substance made of mushrooms or fungi, which are boiled in water, dried, and beaten, then put in a strong lye prepared with saltpeter, and again dried in an oven. This makes the black match, or tinder, brought from Germany.</cd> -- <col><b>Sheep's-wool sponge</b></col>, <cd>a fine and durable commercial sponge (<spn>Spongia equina</spn>, variety <varn>gossypina</varn>) found in Florida and the West Indies. The surface is covered with larger and smaller tufts, having the oscula between them.</cd> -- <col><b>Sponge cake</b></col>, <cd>a kind of sweet cake which is light and spongy.</cd> -- <mcol><col><b>Sponge lead</b></col>, <it>or</it> <col><b>Spongy lead</b></col></mcol> <fld>(Chem.)</fld>, <cd>metallic lead brought to a spongy form by reduction of lead salts, or by compressing finely divided lead; -- used in secondary batteries and otherwise.</cd> -- <col><b>Sponge tree</b></col> <fld>(Bot.)</fld>, <cd>a tropical leguminous tree (<spn>Acacia Farnesiana</spn>), with deliciously fragrant flowers, which are used in perfumery.</cd> -- <col><b>Toilet sponge</b></col>, <cd>a very fine and superior variety of Mediterranean sponge (<spn>Spongia officinalis</spn>, variety <varn>Mediterranea</varn>); -- called also <altname>Turkish sponge</altname>.</cd> -- <col><b>To set a sponge</b></col> <fld>(Cookery)</fld>, <cd>to leaven a small mass of flour, to be used in leavening a larger quantity.</cd> -- <col><b>To throw up the sponge</b></col>, <cd>to give up a contest; to acknowledge defeat; -- from a custom of the prize ring, the person employed to sponge a pugilist between rounds throwing his sponge in the air in token of defeat; -- now, <altname>throw in the towel</altname> is more common, and has the same origin and meaning.</cd> <mark>[Cant or Slang]</mark> <ldquo/He was too brave a man <xex>to throw up the sponge</xex> to fate.<rdquo/ <au>Lowell.</au> -- <col><b>Vegetable sponge</b></col>. <fld>(Bot.)</fld> <cd>See <er>Loof</er>.</cd> -- <col><b>Velvet sponge</b></col>, <cd>a fine, soft commercial sponge (<spn>Spongia equina</spn>, variety <varn>meandriniformis</varn>) found in Florida and the West Indies.</cd> -- <col><b>Vitreous sponge</b></col>. <cd>See <er>Glass-sponge</er>.</cd> -- <col><b>Yellow sponge</b></col>, <cd>a common and valuable commercial sponge (<spn>Spongia agaricina</spn>, variety <varn>corlosia</varn>) found in Florida and the West Indies.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sponge</ent><br/
<hw>Sponge</hw>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Sponged</conjf> <pr>(sp<ucr/njd)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Sponging</conjf> <pr>(sp<ucr/n"j<icr/ng)</pr>.]</vmorph> <sn>1.</sn> <def>To cleanse or wipe with a sponge; <as>as, to <ex>sponge</ex> a slate or a cannon</as>; to wet with a sponge; <as>as, to <ex>sponge</ex> cloth</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To wipe out with a sponge, as letters or writing; to efface; to destroy all trace of.</def>  <rj><au>Hooker.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>Fig.: To deprive of something by imposition.</def> <ldquo/How came such multitudes of our nation . . . to be <xex>sponged</xex> of their plate and their money?<rdquo/  <rj><au>South.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>Fig.: To get by imposition or mean arts without cost; <as>as, to <ex>sponge</ex> a breakfast</as>.</def>  <rj><au>Swift.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sponge</ent><br/
<hw>Sponge</hw>, <pos>v. i.</pos> <sn>1.</sn> <def>To suck in, or imbibe, as a sponge.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Fig.: To gain by mean arts, by intrusion, or hanging on; <as>as, an idler <ex>sponges</ex> on his neighbor</as>.</def>  <rj><au>E. Eggleston.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>The fly is an intruder, and a common smell-feast, that <qex>sponges</qex> upon other people's trenchers.</q> <rj><qau>L'Estrange.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>To be converted, as dough, into a light, spongy mass by the agency of yeast, or leaven.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spongelet</ent><br/
<hw>Sponge"let</hw> <pr>(-l<ecr/t)</pr>, <pos>n.</pos> <def>See <er>Spongiole</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spongeous</ent><br/
<hw>Spon"geous</hw> <pr>(sp<ucr/n"j<ucr/s)</pr>, <pos>a.</pos> <ety>[See <er>Spongious</er>.]</ety> <def>Resembling sponge; having the nature or qualities of sponge.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sponger</ent><br/
<hw>Spon"ger</hw> <pr>(-j<etil/r)</pr>, <pos>n.</pos> <sn>1.</sn> <def>One who sponges, or uses a sponge.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>One employed in gathering sponges.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>Fig.: A parasitical dependent; a hanger-on.</def><br/
[<source>1913 Webster</source>]</p>

<p><-- p. 1391 pr=VMG --></p>

<p><ent>Spongiae</ent><br/
||<hw>Spon"gi*ae</hw> <pr>(sp<ocr/n"j<icr/*<emac/)</pr>, <pos>n. pl.</pos> <ety>[See <er>Sponge</er>.]</ety> <fld>(Zool.)</fld> <def>The grand division of the animal kingdom which includes the sponges; -- called also <altname>Spongida</altname>, <altname>Spongiaria</altname>, <altname>Spongiozoa</altname>, and <altname>Porifera</altname>.</def><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ In the Spongiae, the soft sarcode of the body is usually supported by a skeleton consisting of horny fibers, or of siliceous or calcareous spicules.  The common sponges contain larger and smaller cavities and canals, and numerous small ampullae which are lined with ciliated cells capable of taking in solid food.  The outer surface usually has minute pores through which water enters, and large openings for its exit.  Sponges produce eggs and spermatozoa, and the egg when fertilized undergoes segmentation to form a ciliated embryo.</note><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spongida</ent><br/
||<hw>Spon"gi*da</hw> <pr>(sp<ocr/n"j<icr/*d<adot/)</pr>, <pos>n. pl.</pos> <ety>[NL.]</ety> <def>Spongiae.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spongiform</ent><br/
<hw>Spon"gi*form</hw> <pr>(sp<ucr/n"j<icr/*f<ocir/rm)</pr>, <pos>a.</pos> <def>Resembling a sponge; soft and porous; porous.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spongilla</ent><br/
||<hw>Spon*gil"la</hw> <pr>(sp<ocr/n*j<icr/l"l<adot/)</pr>, <pos>n.</pos> <ety>[NL., dim. of <ets>spongia</ets> a sponge.]</ety> <fld>(Zool.)</fld> <def>A genus of siliceous sponges found in fresh water.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spongin</ent><br/
<hw>Spon"gin</hw> <pr>(sp<ucr/n"j<icr/n)</pr>, <pos>n.</pos> <fld>(Physiol. Chem.)</fld> <def>The chemical basis of sponge tissue, a nitrogenous, hornlike substance which on decomposition with sulphuric acid yields leucin and glycocoll.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sponginess</ent><br/
<hw>Spon"gi*ness</hw> <pr>(sp<ucr/n"j<icr/*n<ecr/s)</pr>, <pos>n.</pos> <def>The quality or state of being spongy.</def>  <rj><au>Dr. H. More.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sponging</ent><br/
<hw>Spon"ging</hw> <pr>(-j<icr/ng)</pr>, <def><pos>a. & n.</pos> from <er>Sponge</er>, <pos>v.</pos></def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Sponging house</b></col> <fld>(Eng. Law)</fld>, <cd>a bailiff's or other house in which debtors are put before being taken to jail, or until they compromise with their creditors. At these houses extortionate charges are commonly made for food, lodging, etc.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spongiole</ent><br/
<hw>Spon"gi*ole</hw> <pr>(sp<ucr/n"j<icr/*<omac/l; 277)</pr>, <pos>n.</pos> <ety>[L. <ets>spongiola</ets> a rose gall, small roots, dim. of <ets>spongia</ets>: cf. F. <ets>spongiole</ets>.]</ety> <fld>(Bot.)</fld> <def>A supposed spongelike expansion of the tip of a rootlet for absorbing water; -- called also <altname>spongelet</altname>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spongiolite</ent><br/
<hw>Spon"gi*o*lite</hw> <pr>(-<osl/*l<imac/t)</pr>, <pos>n.</pos> <ety>[Gr. <grk>spoggia`</grk> sponge + <ets>-lite</ets>.]</ety> <fld>(Paleon.)</fld> <def>One of the microscopic siliceous spicules which occur abundantly in the texture of sponges, and are sometimes found fossil, as in flints.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spongiopilin</ent><br/
<hw>Spon`gi*o*pi"lin</hw> <pr>(-<osl/*p<imac/"l<icr/n)</pr>, <pos>n.</pos> <ety>[Gr. <grk>spoggi`on</grk>, dim. of <grk>spo`ggos</grk> a sponge + <grk>pi^los</grk> felt.]</ety> <fld>(Med.)</fld> <def>A kind of cloth interwoven with small pieces of sponge and rendered waterproof on one side by a covering of rubber.  When moistened with hot water it is used as a poultice.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spongious</ent><br/
<ent>Spongiose</ent><br/
<mhw>{ <hw>Spon"gi*ose`</hw> <pr>(sp<ucr/n"j<icr/*<omac/s`)</pr>, <hw>Spon"gi*ous</hw> <pr>(sp<ucr/n"j<icr/*<ucr/s)</pr>, }</mhw> <pos>a.</pos> <ety>[L. <ets>spongiosus</ets>, <ets>spongeosus</ets>: cf. F. <ets>spongieux</ets>. See <er>Sponge</er>.]</ety> <def>Somewhat spongy; spongelike; full of small cavities like sponge; <as>as, <ex>spongious</ex> bones</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spongiozoa</ent><br/
||<hw>Spon`gi*o*zo"a</hw> <pr>(sp<ocr/n`j<icr/*<osl/*z<omac/"<adot/)</pr>, <pos>n. pl.</pos> <ety>[NL., Gr. <grk>spoggia`</grk> sponge + <grk>zo^,on</grk> an animal.]</ety> <fld>(Zool.)</fld> <def>See <er>Spongiae</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spongoblast</ent><br/
<hw>Spon"go*blast</hw> <pr>(sp<ocr/<nsm/"g<osl/*bl<acr/st)</pr>, <pos>n.</pos> <ety>[Gr. <grk>spo`ggos</grk> sponge + <ets>-blast</ets>.]</ety> <fld>(Zool.)</fld> <def>One of the cells which, in sponges, secrete the spongin, or the material of the horny fibers.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spongoid</ent><br/
<hw>Spon"goid</hw> <pr>(sp<ucr/n"goid <it>or</it> sp<ocr/<nsm/"-; 277)</pr>, <pos>a.</pos> <ety>[Gr. <grk>spo`ggos</grk> sponge + <ets>-oid</ets>.]</ety> <def>Resembling sponge; like sponge.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spongy</ent><br/
<hw>Spon"gy</hw> <pr>(sp<ucr/n"j<ycr/)</pr>, <pos>a.</pos> <sn>1.</sn> <def>Soft, and full of cavities; of an open, loose, pliable texture; <as>as, a <ex>spongy</ex> excrescence; <ex>spongy</ex> earth; <ex>spongy</ex> cake; <ex>spongy</ex> bones.</as></def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Wet; drenched; soaked and soft, like sponge; rainy.</def> <ldquo/<xex>Spongy</xex> April.<rdquo/  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>Having the quality of imbibing fluids, like a sponge.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Spongy lead</b></col> <fld>(Chem.)</fld>, <cd>sponge lead. See under <er>Sponge</er>.</cd> -- <col><b>Spongy platinum</b></col>. <cd>See under <er>Platinum</er>.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sponk</ent><br/
<hw>Sponk</hw> <pr>(sp<ucr/<nsm/k)</pr>, <pos>n.</pos> <def>See <er>Spunk</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sponsal</ent><br/
<hw>Spon"sal</hw> <pr>(sp<ocr/n"s<ait/l)</pr>, <pos>a.</pos> <ety>[L. <ets>sponsalis</ets>, fr. <ets>sponsus</ets> a betrothal, fr. <ets>spondere</ets>, <ets>sponsum</ets>, to betroth. See <er>Spouse</er>, and cf. <er>Espousal</er>, <er>Spousal</er>.]</ety> <def>Relating to marriage, or to a spouse; spousal.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sponsible</ent><br/
<hw>Spon"si*ble</hw> <pr>(-s<icr/*b'l)</pr>, <pos>a.</pos> <ety>[Abbrev. from <ets>responsible</ets>.]</ety> <def>Responsible; worthy of credit.</def> <mark>[Prov. Eng. & Scot.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sponsion</ent><br/
<hw>Spon"sion</hw> <pr>(-sh<ucr/n)</pr>, <pos>n.</pos> <ety>[L. <ets>sponsio</ets>, fr. <ets>spondere</ets>, <ets>sponsum</ets>, to promise solemnly.]</ety> <sn>1.</sn> <def>The act of becoming surety for another.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Internat. Law)</fld> <def>An act or engagement on behalf of a state, by an agent not specially authorized for the purpose, or by one who exceeds the limits of authority.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sponsional</ent><br/
<hw>Spon"sion*al</hw> <pr>(-<ait/l)</pr>, <pos>a.</pos> <def>Of or pertaining to a pledge or agreement; responsible.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>He is righteous even in that representative and <qex>sponsional</qex> person he put on.</q> <rj><qau>Abp. Leighton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sponson</ent><br/
<hw>Spon"son</hw> <pr>(-s<ucr/n)</pr>, <pos>n.</pos> <fld>(Shipbuilding)</fld> <sd>(a)</sd> <def>One of the triangular platforms in front of, and abaft, the paddle boxes of a steamboat.</def> <sd>(b)</sd> <def>One of the slanting supports under the guards of a steamboat.</def> <sd>(c)</sd> <def>One of the armored projections fitted with gun ports, used on modern war vessels.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sponsor</ent><br/
<hw>Spon"sor</hw> <pr>(-s<etil/r)</pr>, <pos>n.</pos> <ety>[L., from <ets>spondere</ets>, <ets>sponsum</ets>, to engage one's self. See <er>Spouse</er>.]</ety> <sn>1.</sn> <def>One who binds himself to answer for another, and is responsible for his default; a surety.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>One who at the baptism of an infant professes the Christian faith in its name, and guarantees its religious education; a godfather or godmother.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>A person who vouches for another as fit for some post or task; <as>as, one needs two <ex>sponsors</ex> to be considered for membership</as>.</def><br/
[<source>PJC</source>]</p>

<p><sn>4.</sn> <def>A person or group that assumes financial responsibility for some activity, and may or may not participate in its organization and execution.</def><br/
[<source>PJC</source>]</p>

<p><sn>5.</sn> <def>A person or organization, usually a commercial organization, which pays the cost of an activity, such as a radio or television broadcast, and in return is given the right to advertise itself or its products as part of the activity; <as>as, now a word from our <ex>sponsor</ex></as>.</def><br/
[<source>PJC</source>]</p>

<p><ent>Sponsorial</ent><br/
<hw>Spon*so"ri*al</hw> <pr>(-s<omac/"r<icr/*<ait/l)</pr>, <pos>a.</pos> <def>Pertaining to a sponsor.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sponsorship</ent><br/
<hw>Spon"sor*ship</hw> <pr>(sp<ocr/n"s<etil/r*sh<icr/p)</pr>, <pos>n.</pos> <def>State of being a sponsor.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spontaneity</ent><br/
<hw>Spon`ta*ne"i*ty</hw> <pr>(sp<ocr/n`t<adot/*n<emac/"<icr/*t<ycr/)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Spontaneities</plw> <pr>(-t<icr/z)</pr>.</plu> <ety>[Cf. F. <ets>spontan<eacute/it<eacute/</ets>.]</ety> <sn>1.</sn> <def>The quality or state of being spontaneous, or acting from native feeling, proneness, or temperament, without constraint or external force.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Romney Leigh, who lives by diagrams,<br/
And crosses not the <qex>spontaneities</qex><br/
Of all his individual, personal life<br/
With formal universals.</q> <rj><qau>Mrs. Browning.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Biol.)</fld> <sd>(a)</sd> <def>The tendency to undergo change, characteristic of both animal and vegetable organisms, and not restrained or checked by the environment.</def> <sd>(b)</sd> <def>The tendency to activity of muscular tissue, including the voluntary muscles, when in a state of healthful vigor and refreshment.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spontaneous</ent><br/
<hw>Spon*ta"ne*ous</hw> <pr>(sp<ocr/n*t<amac/"n<esl/*<ucr/s)</pr>, <pos>a.</pos> <ety>[L. <ets>spontaneus</ets>, fr. <ets>sponte</ets> of free will, voluntarily.]</ety> <sn>1.</sn> <def>Proceeding from natural feeling, temperament, or disposition, or from a native internal proneness, readiness, or tendency, without constraint; <as>as, a <ex>spontaneous</ex> gift or proposition</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Proceeding from, or acting by, internal impulse, energy, or natural law, without external force; <as>as, <ex>spontaneous</ex> motion; <ex>spontaneous</ex> growth.</as></def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>Produced without being planted, or without human labor; <as>as, a <ex>spontaneous</ex> growth of wood</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Spontaneous combustion</b></col>, <cd>combustion produced in a substance by the evolution of heat through the chemical action of its own elements; as, the <xex>spontaneous combustion</xex> of waste matter saturated with oil.</cd> -- <col><b>Spontaneous generation</b></col>. <fld>(Biol.)</fld> <cd>See under <er>Generation</er>.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><syn><b>Syn.</b> -- Voluntary; uncompelled; willing.</syn>  -- <usage><er>Spontaneous</er>, <er>Voluntary</er>. What is <xex>voluntary</xex> is the result of a <xex>volition</xex>, or act of choice; it therefore implies some degree of consideration, and may be the result of mere reason without excited feeling. What is <xex>spontaneous</xex> springs wholly from feeling, or a sudden impulse which admits of no reflection; as, a <xex>spontaneous</xex> burst of applause. Hence, the term is also applied to things inanimate when they are produced without the determinate purpose or care of man. <ldquo/Abstinence which is but <xex>voluntary</xex> fasting, and . . . exercise which is but <xex>voluntary</xex> labor.<rdquo/ <au>J. Seed.</au><br/
[<source>1913 Webster</source>]</p>

<p><q><qex>Spontaneous</qex> joys, where nature has its play,<br/
The soul adopts, and owns their firstborn away.</q> <rj><qau>Goldsmith.</qau></rj>
</usage><br/
[<source>1913 Webster</source>]</p>

<p>-- <wordforms><wf>Spon*ta"ne*ous*ly</wf>, <pos>adv.</pos> -- <wf>Spon*ta"ne*ous*ness</wf>, <pos>n.</pos></wordforms><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spontoon</ent><br/
<hw>Spon*toon"</hw> <pr>(sp<ocr/n*t<oomac/n")</pr>, <pos>n.</pos> <ety>[F. <ets>sponton</ets>, <ets>esponton</ets>, It. <ets>spontone</ets>, <ets>spuntone</ets>.]</ety> <fld>(Mil.)</fld> <def>A kind of half-pike, or halberd, formerly borne by inferior officers of the British infantry, and used in giving signals to the soldiers.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spook</ent><br/
<hw>Spook</hw> <pr>(sp<oomac/k)</pr>, <pos>n.</pos> <ety>[D. <ets>spook</ets>; akin to G. <ets>spuk</ets>, Sw. <ets>sp<oum/ke</ets>, Dan. <ets>sp<oum/gelse</ets> a specter, <ets>sp<oum/ge</ets> to play, sport, joke, <ets>sp<oum/g</ets> a play, joke.]</ety> <sn>1.</sn> <def>A spirit; a ghost; an apparition; a hobgoblin.</def> <altsp>[Written also <asp>spuke</asp>.]</altsp>  <rj><au>Ld. Lytton.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Zool.)</fld> <def>The chimaera.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spool</ent><br/
<hw>Spool</hw> <pr>(sp<oomac/l)</pr>, <pos>n.</pos> <ety>[OE. <ets>spole</ets>, OD. <ets>spoele</ets>, D. <ets>spoel</ets>; akin to G. <ets>spule</ets>, OHG. <ets>spuola</ets>, Dan. & Sw. <ets>spole</ets>.]</ety> <def>A piece of cane or reed with a knot at each end, or a hollow cylinder of wood with a ridge at each end, used to wind thread or yarn upon.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Spool stand</b></col>, <cd>an article holding spools of thread, turning on pins, -- used by women at their work.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spool</ent><br/
<hw>Spool</hw>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Spooled</conjf> <pr>(sp<oomac/ld)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Spooling</conjf>.]</vmorph> <def>To wind on a spool or spools.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spooler</ent><br/
<hw>Spool"er</hw> <pr>(-<etil/r)</pr>, <pos>n.</pos> <def>One who, or that which, spools.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spoom</ent><br/
<hw>Spoom</hw> <pr>(sp<oomac/m)</pr>, <pos>v. i.</pos> <ety>[Probably fr. <ets>spume</ets> foam. See <er>Spume</er>.]</ety> <fld>(Naut.)</fld> <def>To be driven steadily and swiftly, as before a strong wind; to be driven before the wind without any sail, or with only a part of the sails spread; to scud under bare poles.</def> <altsp>[Written also <asp>spoon</asp>.]</altsp><br/
[<source>1913 Webster</source>]</p>

<p><q>When virtue <qex>spooms</qex> before a prosperous gale,<br/
My heaving wishes help to fill the sail.</q> <rj><qau>Dryden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spoon</ent><br/
<hw>Spoon</hw> <pr>(sp<oomac/n)</pr>, <pos>v. i.</pos> <fld>(Naut.)</fld> <def>See <er>Spoom</er>.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>We might have <qex>spooned</qex> before the wind as well as they.</q> <rj><qau>Pepys.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spoon</ent><br/
<hw>Spoon</hw>, <pos>n.</pos> <ety>[OE. <ets>spon</ets>, AS. <ets>sp<omac/n</ets>, a chip; akin to D. <ets>spaan</ets>, G. <ets>span</ets>, Dan. <ets>spaan</ets>, Sw. <ets>sp<aring/n</ets>, Icel. <ets>sp<aacute/nn</ets>, <ets>sp<oacute/nn</ets>, a chip, a spoon. <root/170.  Cf. <er>Span-new</er>.]</ety> <sn>1.</sn> <def>An implement consisting of a small bowl (usually a shallow oval) with a handle, used especially in preparing or eating food.</def><br/
[<source>1913 Webster</source>]</p>

<p><q><ldquo/Therefore behoveth him a full long <qex>spoon</qex><br/
That shall eat with a fiend,<rdquo/ thus heard I say.</q> <rj><qau>Chaucer.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>He must have a long <qex>spoon</qex> that must eat with the devil.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Anything which resembles a spoon in shape; esp. <fld>(Fishing)</fld>, a spoon bait.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>Fig.: A simpleton; a spooney.</def> <mark>[Slang]</mark>  <rj><au>Hood.</au></rj><br/
[<source>1913 Webster</source>]</p>


<p><sn>4.</sn> <fld>(Golf)</fld> <def>A wooden club with a lofted face.</def>  <rj><au>Encyc. of Sport.</au></rj><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><cs><col><b>Spoon bait</b></col> <fld>(Fishing)</fld>, <cd>a lure used in trolling, consisting of a glistening metallic plate shaped like the bowl of a spoon with a fishhook attached.</cd> -- <col><b>Spoon bit</b></col>, <cd>a bit for boring, hollowed or furrowed along one side.</cd> -- <col><b>Spoon net</b></col>, <cd>a net for landing fish.</cd> -- <col><b>Spoon oar</b></col>. <cd>See under <er>Oar</er>.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spoon</ent><br/
<hw>Spoon</hw>, <pos>v. t.</pos> <sn>1.</sn> <def>To take up in, or as in, a spoon.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Fishing)</fld> <def>To catch by fishing with a spoon bait.</def></p>

<p><q>He had with him all the tackle necessary for <qex>spooning</qex> pike.</q>  <rj><qau>Mrs. Humphry Ward.</qau></rj><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><sn>3.</sn>  <def>In croquet, golf, etc., to push or shove (a ball) with a lifting motion, instead of striking with an audible knock.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Spoon</ent><br/
<hw>Spoon</hw>, <pos>v. i.</pos> <def>To act with demonstrative or foolish fondness, as one in love.</def> <mark>[Colloq.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spoon</ent><br/
<hw>Spoon</hw>, <pos>v. i.</pos> <sn>1.</sn> <def>To fish with a spoon bait.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><sn>2.</sn>  <def>In croquet, golf, etc., to spoon a ball.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Spoonbill</ent><br/
<hw>Spoon"bill`</hw> <pr>(-b<icr/l`)</pr>, <pos>n.</pos> <fld>(Zool.)</fld> <sd>(a)</sd> <def>Any one of several species of wading birds of the genera <gen>Ajaja</gen> and <gen>Platalea</gen>, and allied genera, in which the long bill is broadly expanded and flattened at the tip.</def><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ The roseate spoonbill of America (<spn>Ajaja ajaja</spn>), and the European spoonbill (<spn>Platalea leucorodia</spn>) are the best known. The royal spoonbill (<spn>Platalea regia</spn>) of Australia is white, with the skin in front of the eyes naked and black. The male in the breeding season has a fine crest.</note><br/
[<source>1913 Webster</source>]</p>

<p><sd>(b)</sd> <def>The shoveler. See <er>Shoveler</er>, 2.</def> <sd>(c)</sd> <def>The ruddy duck. See under <er>Ruddy</er>.</def> <sd>(d)</sd> <def>The paddlefish.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spoon-billed</ent><br/
<hw>Spoon"-billed`</hw> <pr>(-b<icr/ld`)</pr>, <pos>a.</pos> <fld>(Zool.)</fld> <def>Having the bill expanded and spatulate at the end.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spoondrift</ent><br/
<hw>Spoon"drift</hw> <pr>(-dr<icr/ft)</pr>, <pos>n.</pos> <ety>[<ets>Spoom</ets> + <ets>drift</ets>.]</ety> <def>Spray blown from the tops of waves during a gale at sea; also, snow driven in the wind at sea; -- <altsp>written also <asp>spindrift</asp></altsp>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spooney</ent><br/
<hw>Spoon"ey</hw> <pr>(-<ycr/)</pr>, <pos>a.</pos> <def>Weak-minded; demonstratively fond; <as>as, <ex>spooney</ex> lovers</as>.</def> <altsp>[Spelt also <asp>spoony.</asp>]</altsp> <mark>[Colloq.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spooney</ent><br/
<hw>Spoon"ey</hw>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Spooneys</plw> <pr>(-<icr/z)</pr>.</plu> <def>A weak-minded or silly person; one who is foolishly fond.</def> <mark>[Colloq.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>There is no doubt, whatever, that I was a lackadaisical young <qex>spooney</qex>.</q> <rj><qau>Dickens.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spoonflower</ent><br/
<hw>Spoon"flow`er</hw> <pr>(-flou`<etil/r)</pr>, <pos>n.</pos> <def>The yautia.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Spoonful</ent><br/
<hw>Spoon"ful</hw> <pr>(-f<usdot/l)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Spoonfuls</plw> <pr>(-f<usdot/lz)</pr>.</plu> <sn>1.</sn> <def>The quantity which a spoon contains, or is able to contain; <as>as, a tea<ex>spoonful</ex>; a table<ex>spoonful</ex>.</as></def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Hence, a small quantity.</def>  <rj><au>Arbuthnot.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spoonily</ent><br/
<hw>Spoon"i*ly</hw> <pr>(-<icr/*l<ycr/)</pr>, <pos>adv.</pos> <def>In a spoony manner.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spoon-meat</ent><br/
<hw>Spoon"-meat`</hw> <pr>(-m<emac/t`)</pr>, <pos>n.</pos> <def>Food that is, or must be, taken with a spoon; liquid food.</def> <ldquo/Diet most upon <xex>spoon-meats</xex>.<rdquo/  <rj><au>Harvey.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spoonwood</ent><br/
<hw>Spoon"wood`</hw> <pr>(-w<oocr/d`)</pr>, <pos>n.</pos> <fld>(Bot.)</fld> <def>The mountain laurel (<spn>Kalmia latifolia</spn>).</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spoonworm</ent><br/
<hw>Spoon"worm`</hw> <pr>(sp<oomac/n"w<ucir/rm`)</pr>, <pos>n.</pos> <fld>(Zool.)</fld> <def>A gephyrean worm of the genus <gen>Thalassema</gen>, having a spoonlike proboscis.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spoonwort</ent><br/
<hw>Spoon"wort`</hw> <pr>(-w<ucir/rt`)</pr>, <pos>n.</pos> <fld>(Bot.)</fld> <def>Scurvy grass.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spoony</ent><br/
<hw>Spoon"y</hw> <pr>(-<ycr/)</pr>, <pos>a. & n.</pos> <def>Same as <er>Spooney</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spoor</ent><br/
<hw>Spoor</hw> <pr>(sp<oomac/r)</pr>, <pos>n.</pos> <ety>[D. <ets>spoor</ets>; akin to AS. <ets>spor</ets>, G. <ets>spur</ets>, and from the root of E. <ets>spur</ets>. <root/171. See <er>Spur</er>.]</ety> <def>The track or trail of any wild animal; <as>as, the <ex>spoor</ex> of an elephant; -- used originally by travelers in South Africa</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spoor</ent><br/
<hw>Spoor</hw>, <pos>v. i.</pos> <def>To follow a spoor or trail.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sporades</ent><br/
||<hw>Spor"a*des</hw> <pr>(sp<ocr/r"<adot/*d<emac/z)</pr>, <pos>n. pl.</pos> <ety>[L., fr. Gr. <grk>spora`des</grk>.  Cf. <er>Sporadic</er>.]</ety> <fld>(Astron.)</fld> <def>Stars not included in any constellation; -- called also <xex>informed</xex>, or <xex>unformed</xex>, stars.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sporadial</ent><br/
<hw>Spo*ra"di*al</hw> <pr>(sp<osl/*r<amac/"d<icr/*<ait/l)</pr>, <pos>a.</pos> <def>Sporadic.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sporadic</ent><br/
<hw>Spo*rad"ic</hw> <pr>(-r<acr/d"<icr/k)</pr>, <pos>a.</pos> <ety>[Gr. <?/ scattered, fr. <?/, <?/, scattered, fr. <?/ to sow seed, to scatter like seed: cf. F. <ets>sporadique</ets>. See <er>Spore</er>.]</ety> <def>Occurring singly, or apart from other things of the same kind, or in scattered instances; separate; single; <as>as, a <ex>sporadic</ex> fireball; a <ex>sporadic</ex> case of disease; a <ex>sporadic</ex> example of a flower.</as></def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Sporadic disease</b></col> <fld>(Med.)</fld>, <cd>a disease which occurs in single and scattered cases. See the Note under <er>Endemic</er>, <pos>a.</pos></cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sporadical</ent><br/
<hw>Spo*rad"ic*al</hw> <pr>(-<icr/*k<ait/l)</pr>, <pos>a.</pos> <def>Sporadic.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sporadically</ent><br/
<hw>Spo*rad"ic*al*ly</hw>, <pos>adv.</pos> <def>In a sporadic manner.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sporangiophore</ent><br/
<hw>Spo*ran"gi*o*phore</hw> <pr>(sp<osl/*r<acr/n"j<icr/*<osl/*f<omac/r)</pr>, <pos>n.</pos> <ety>[<ets>Sporangium</ets> + Gr. <grk>fe`rein</grk> to bear.]</ety> <fld>(Bot.)</fld> <def>The axis or receptacle in certain ferns (as <gen>Trichomanes</gen>), which bears the sporangia.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sporangium</ent><br/
||<hw>Spo*ran"gi*um</hw> <pr>(sp<osl/*r<acr/n"j<icr/*<ucr/m)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Sporangia</plw> <pr>(sp<osl/*r<acr/n"j<icr/*<adot/)</pr>.</plu> <ety>[NL., fr. Gr. <?/ a sowing, seed + <?/ a receptacle.]</ety> <fld>(Bot.)</fld> <def>A spore case in the cryptogamous plants, as in ferns, etc.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spore</ent><br/
<hw>Spore</hw> <pr>(sp<omac/r)</pr>, <pos>n.</pos> <ety>[Gr. <?/ a sowing, seed, from <?/ to sow.  Cf. <er>Sperm</er>.]</ety> <sn>1.</sn> <fld>(Bot.)</fld> <sd>(a)</sd> <def>One of the minute grains in flowerless plants, which are analogous to seeds, as serving to reproduce the species.</def><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ Spores are produced differently in the different classes of cryptogamous plants, and as regards their nature are often so unlike that they have only their minuteness in common.  The peculiar spores of diatoms (called <stype>auxospores</stype>) increase in size, and at length acquire a siliceous coating, thus becoming new diatoms of full size.  Compare <er>Macrospore</er>, <er>Microspore</er>, <er>Oospore</er>, <er>Resting spore</er>, <er>Sphaerospore</er>, <er>Swarmspore</er>, <er>Tetraspore</er>, <er>Zoospore</er>, and <er>Zygospore</er>.</note><br/
[<source>1913 Webster</source>]</p>

<p><sd>(b)</sd> <def>An embryo sac or embryonal vesicle in the ovules of flowering plants.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Biol.)</fld> <sd>(a)</sd> <def>A minute grain or germ; a small, round or ovoid body, formed in certain organisms, and by germination giving rise to a new organism; <as>as, the reproductive <ex>spores</ex> of bacteria, etc.</as></def> <sd>(b)</sd> <def>One of the parts formed by fission in certain Protozoa. See <cref>Spore formation</cref>, belw.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Spore formation</b></col>. <sd>(a)</sd> <fld>(Biol)</fld> <cd>A mode of reproduction resembling multiple fission, common among Protozoa, in which the organism breaks up into a number of pieces, or spores, each of which eventually develops into an organism like the parent form.</cd> <au>Balfour.</au> <sd>(b)</sd> <cd>The formation of reproductive cells or spores, as in the growth of bacilli.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sporid</ent><br/
<hw>Spo"rid</hw> <pr>(sp<omac/"r<icr/d)</pr>, <pos>n.</pos> <fld>(Bot.)</fld> <def>A sporidium.</def>  <rj><au>Lindley.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sporidiferous</ent><br/
<hw>Spo`ri*dif"er*ous</hw> <pr>(sp<omac/`r<icr/*d<icr/f"<etil/r*<ucr/s)</pr>, <pos>a.</pos> <ety>[<ets>Sporidium</ets> + <ets>-ferous</ets>.]</ety> <fld>(Bot.)</fld> <def>Bearing sporidia.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sporidium</ent><br/
||<hw>Spo*rid"i*um</hw> <pr>(sp<osl/*r<icr/d"<icr/*<ucr/m)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Sporidia</plw> <pr>(-<adot/)</pr>.</plu> <ety>[NL. See <er>Spore</er>.]</ety> <fld>(Bot.)</fld> <sd>(a)</sd> <def>A secondary spore, or a filament produced from a spore, in certain kinds of minute fungi.</def> <sd>(b)</sd> <def>A spore.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sporiferous</ent><br/
<hw>Spo*rif"er*ous</hw> <pr>(-r<icr/f"<etil/r*<ucr/s)</pr>, <pos>a.</pos> <ety>[<ets>Spore</ets> + <ets>-ferous</ets>.]</ety> <fld>(Biol.)</fld> <def>Bearing or producing spores.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sporification</ent><br/
<hw>Spo`ri*fi*ca"tion</hw> <pr>(sp<omac/`r<icr/*f<icr/*k<amac/"sh<ucr/n)</pr>, <pos>n.</pos> <ety>[<ets>Spore</ets> + L. <ets>-ficare</ets> (in comp.) to make. See <er>-fy</er>.]</ety> <fld>(Biol.)</fld> <def>Spore formation. See <cref>Spore formation</cref> <sd>(b)</sd>, under <er>Spore</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sporocarp</ent><br/
<hw>Spo"ro*carp</hw> <pr>(sp<omac/"r<osl/*k<aum/rp)</pr>, <pos>n.</pos> <ety>[<ets>Spore</ets> + Gr. <?/ fruit.]</ety> <fld>(Bot.)</fld> <sd>(a)</sd> <def>A closed body or conceptacle containing one or more masses of spores or sporangia.</def> <sd>(b)</sd> <def>A sporangium.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sporocyst</ent><br/
<hw>Spo"ro*cyst</hw> <pr>(-s<icr/st)</pr>, <pos>n.</pos> <ety>[Gr. <?/ seed + <?/ bladder.]</ety> <sn>1.</sn> <fld>(Zool.)</fld> <def>An asexual zooid, usually forming one of a series of larval forms in the agamic reproduction of various trematodes and other parasitic worms. The sporocyst generally develops from an egg, but in its turn produces other larvae by internal budding, or by the subdivision of a part or all of its contents into a number of minute germs. See <er>Redia</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Zool.)</fld> <def>Any protozoan when it becomes encysted and produces germs by sporulation.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sporogenesis</ent><br/
<hw>Spo`ro*gen"e*sis</hw> <pr>(-j<ecr/n"<esl/*s<icr/s)</pr>, <pos>n.</pos> <ety>[<ets>Spore</ets> + <ets>genesis</ets>.]</ety> <fld>(Biol.)</fld> <def>Reproduction by spores.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sporogony</ent><br/
<hw>Spo*rog"o*ny</hw> <pr>(sp<osl/*r<ocr/g"<osl/*n<ycr/)</pr>, <pos>n.</pos> <ety>[<ets>Spore</ets> + root of Gr. <?/ to be born.]</ety> <fld>(Zool.)</fld> <def>The growth or development of an animal or a zooid from a nonsexual germ.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sporophore</ent><br/
<hw>Spo"ro*phore</hw> <pr>(sp<omac/"r<osl/*f<omac/r)</pr>, <pos>n.</pos> <ety>[<ets>Spore</ets> + Gr. <grk>fe`rein</grk> to bear.]</ety> <fld>(Bot.)</fld> <sd>(a)</sd> <def>A placenta.</def> <sd>(b)</sd> <def>That alternately produced form of certain cryptogamous plants, as ferns, mosses, and the like, which is nonsexual, but produces spores in countless numbers. In ferns it is the leafy plant, in mosses the capsule.  Cf. <er>Oophore</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sporophoric</ent><br/
<hw>Spo`ro*phor"ic</hw> <pr>(-f<ocr/r"<icr/k)</pr>, <pos>a.</pos> <fld>(Bot.)</fld> <def>Having the nature of a sporophore.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sporophyte</ent><br/
<hw>Spo"ro*phyte</hw> <pr>(sp<omac/"r<osl/*f<imac/t)</pr>, <pos>n.</pos>  <ety>[<ets>Spore</ets> + Gr. <grk>fyto`n</grk> plant.]</ety> <fld>(Bot.)</fld> <def>In plants exhibiting alternation of generations, the generation which bears asexual spores; -- opposed to <contr>gametophyte</contr>. It is not clearly differentiated in the life cycle of the lower plants.</def> -- <wordforms><wf>Spo`ro*phyt"ic</wf> <pr>(#)</pr>, <pos>a.</pos></wordforms><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><-- p. 1392 pr=VMG --></p>

<p><ent>Sporosac</ent><br/
<hw>Spo"ro*sac</hw> <pr>(sp<omac/"r<osl/*s<acr/k)</pr>, <pos>n.</pos> <ety>[<ets>Spore</ets> + <ets>sac</ets>.]</ety> <fld>(Zool.)</fld> <sd>(a)</sd> <def>A hydrozoan reproductive zooid or gonophore which does not become medusoid in form or structure. See <xex>Illust.</xex> under <er>Athecata</er>.</def> <sd>(b)</sd> <def>An early or simple larval stage of trematode worms and some other invertebrates, which is capable of reproducing other germs by asexual generation; a nurse; a redia.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sporozoa</ent><br/
||<hw>Spo`ro*zo"a</hw> <pr>(-z<omac/"<adot/)</pr>, <pos>n. pl.</pos> <ety>[NL., from Gr. <grk>spo`ros</grk> a spore + <grk>zo^,on</grk> an animal.]</ety> <fld>(Zool.)</fld> <def>An extensive division of parasitic Protozoa, which increase by sporulation. It includes the Gregarinida.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sporozoid</ent><br/
<hw>Spo`ro*zo"id</hw> <pr>(-<icr/d)</pr>, <pos>n.</pos> <ety>[<ets>Spore</ets> + Gr. <grk>zo^,on</grk> an animal.]</ety> <fld>(Bot.)</fld> <def>Same as <er>Zoospore</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sporozoite</ent><br/
<hw>Spo`ro*zo"ite</hw> <pr>(sp<omac/`r<osl/*z<omac/"<imac/t)</pr>, <pos>n.</pos> <fld>(Zool.)</fld> <def>In certain Sporozoa, a small active, usually elongate, sickle-shaped or somewhat amoeboid spore, esp. one of those produced by division of the passive spores into which the zygote divides. The sporozoites reproduce asexually.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Sporran</ent><br/
<hw>Spor"ran</hw> <pr>(sp<ocr/r"r<ait/n)</pr>, <pos>n.</pos> <ety>[Gael. <ets>sporan</ets>.]</ety> <def>A large purse or pouch made of skin with the hair or fur on, worn in front of the kilt by Highlanders when in full dress.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sport</ent><br/
<hw>Sport</hw> <pr>(sp<omac/rt)</pr>, <pos>n.</pos> <ety>[Abbreviated from <ets>disport</ets>.]</ety> <sn>1.</sn> <def>That which diverts, and makes mirth; pastime; amusement.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>It is as <qex>sport</qex> to a fool to do mischief.</q> <rj><qau>Prov. x. 23.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Her <qex>sports</qex> were such as carried riches of knowledge upon the stream of delight.</q> <rj><qau>Sir P. Sidney.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Think it but a minute spent in <qex>sport</qex>.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Mock; mockery; contemptuous mirth; derision.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Then make <qex>sport</qex> at me; then let me be your jest.</q>  <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>That with which one plays, or which is driven about in play; a toy; a plaything; an object of mockery.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Flitting leaves, the <qex>sport</qex> of every wind.</q> <rj><qau>Dryden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Never does man appear to greater disadvantage than when he is the <qex>sport</qex> of his own ungoverned passions.</q> <rj><qau>John Clarke.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>Play; idle jingle.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>An author who should introduce such a <qex>sport</qex> of words upon our stage would meet with small applause.</q> <rj><qau>Broome.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <def>Diversion of the field, as fowling, hunting, fishing, racing, games, and the like, esp. when money is staked.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>6.</sn> <fld>(Bot. & Zool.)</fld> <def>A plant or an animal, or part of a plant or animal, which has some peculiarity not usually seen in the species; an abnormal variety or growth. See <cref>Sporting plant</cref>, under <er>Sporting</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>7.</sn> <def>A sportsman; a gambler.</def> <mark>[Slang]</mark><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>In sport</b></col>, <cd>in jest; for play or diversion.</cd> <ldquo/So is the man that deceiveth his neighbor, and saith, Am not I <xex>in sport</xex>?<rdquo/  <rj><au>Prov. xxvi. 19.</au></rj>
</cs><br/
[<source>1913 Webster</source>]</p>

<p><syn><b>Syn.</b> -- Play; game; diversion; frolic; mirth; mock; mockery; jeer.</syn><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sport</ent><br/
<hw>Sport</hw>, <pos>v. i.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Sported</conjf>; <pos>p. pr. & vb. n.</pos> <conjf>Sporting</conjf>.]</vmorph> <sn>1.</sn> <def>To play; to frolic; to wanton.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>[Fish], <qex>sporting</qex> with quick glance,<br/
Show to the sun their waved coats dropt with gold.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To practice the diversions of the field or the turf; to be given to betting, as upon races.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>To trifle.</def> <ldquo/He <xex>sports</xex> with his own life.<rdquo/  <rj><au>Tillotson.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <fld>(Bot. & Zool.)</fld> <def>To assume suddenly a new and different character from the rest of the plant or from the type of the species; -- said of a bud, shoot, plant, or animal. See <er>Sport</er>, <pos>n.</pos>, 6.</def>  <rj><au>Darwin.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><syn><b>Syn.</b> -- To play; frolic; game; wanton.</syn><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sport</ent><br/
<hw>Sport</hw>, <pos>v. t.</pos> <sn>1.</sn> <def>To divert; to amuse; to make merry; -- used with the reciprocal pronoun.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Against whom do ye <qex>sport</qex> yourselves?</q> <rj><qau>Isa. lvii. 4.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To represent by any kind of play.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Now <qex>sporting</qex> on thy lyre the loves of youth.</q> <rj><qau>Dryden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>To exhibit, or bring out, in public; to use or wear; <as>as, to <ex>sport</ex> a new equipage</as>.</def> <mark>[Colloq.]</mark>  <rj><au>Grose.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>To give utterance to in a sportive manner; to throw out in an easy and copious manner; -- with <xex>off</xex>; <as>as, to <ex>sport</ex> off epigrams</as>.</def> <mark>[R.]</mark>  <rj><au>Addison.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>To sport one's oak</b></col>. <cd>See under <er>Oak</er>, <pos>n.</pos></cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sportability</ent><br/
<hw>Sport`a*bil"i*ty</hw> <pr>(-<adot/*b<icr/l"<icr/*t<ycr/)</pr>, <pos>n.</pos> <def>Sportiveness.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sportal</ent><br/
<hw>Sport"al</hw> <pr>(sp<omac/rt"<ait/l)</pr>, <pos>a.</pos> <def>Of or pertaining to sports; used in sports.</def> <mark>[R.]</mark> <ldquo/<xex>Sportal</xex> arms.<rdquo/  <rj><au>Dryden.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sporter</ent><br/
<hw>Sport"er</hw> <pr>(-<etil/r)</pr>, <pos>n.</pos> <def>One who sports; a sportsman.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>As this gentleman and I have been old fellow <qex>sporters</qex>, I have a friendship for him.</q> <rj><qau>Goldsmith.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sportful</ent><br/
<hw>Sport"ful</hw> <pr>(-f<usdot/l)</pr>, <pos>a.</pos> <sn>1.</sn> <def>Full of sport; merry; frolicsome; full of jesting; indulging in mirth or play; playful; wanton; <as>as, a <ex>sportful</ex> companion</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Down he alights among the <qex>sportful</qex> herd.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Done in jest, or for mere play; sportive.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>They are no <qex>sportful</qex> productions of the soil.</q> <rj><qau>Bentley.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p>-- <wordforms><wf>Sport"ful*ly</wf>, <pos>adv.</pos> -- <wf>Sport"ful*ness</wf>, <pos>n.</pos></wordforms><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sporting</ent><br/
<hw>Sport"ing</hw>, <pos>a.</pos> <def>Of, pertaining to, or engaging in, sport or sports; exhibiting the character or conduct of one who, or that which, sports.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Sporting book</b></col>, <cd>a book containing a record of bets, gambling operations, and the like.</cd> <au>C. Kingsley.</au> -- <col><b>Sporting house</b></col>, <cd>a house frequented by sportsmen, gamblers, and the like.</cd> -- <col><b>Sporting man</b></col>, <cd>one who practices field sports; also, a horse racer, a pugilist, a gambler, or the like.</cd> -- <col><b>Sporting plant</b></col> <fld>(Bot.)</fld>, <cd>a plant in which a single bud or offset suddenly assumes a new, and sometimes very different, character from that of the rest of the plant.</cd> <au>Darwin.</au></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sportingly</ent><br/
<hw>Sport"ing*ly</hw>, <pos>adv.</pos> <def>In sport; sportively.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The question you there put, you do it, I suppose, but <qex>sportingly</qex>.</q> <rj><qau>Hammond.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sportive</ent><br/
<hw>Sport"ive</hw> <pr>(-<icr/v)</pr>, <pos>a.</pos> <def>Tending to, engaged in, or provocative of, sport; gay; frolicsome; playful; merry.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Is it I<br/
That drive thee from the <qex>sportive</qex> court?</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p>-- <wordforms><wf>Sport"ive*ly</wf>, <pos>adv.</pos> -- <wf>Sport"ive*ness</wf>, <pos>n.</pos></wordforms><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sportless</ent><br/
<hw>Sport"less</hw>, <pos>a.</pos> <def>Without sport or mirth; joyless.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sportling</ent><br/
<hw>Sport"ling</hw> <pr>(-l<icr/ng)</pr>, <pos>n.</pos> <def>A little person or creature engaged in sports or in play.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>When again the lambkins play --<br/
Pretty <qex>sportlings</qex>, full of May.</q> <rj><qau>Philips.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sportsman</ent><br/
<hw>Sports"man</hw> <pr>(sp<omac/rts"m<ait/n)</pr>, <pos>n.</pos>;<plu><it>pl.</it> <plw>Sportsmen</plw> <pr>(-m<eit/n)</pr>.</plu> <def>One who pursues the sports of the field; one who hunts, fishes, etc.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sportsmanship</ent><br/
<hw>Sports"man*ship</hw>, <pos>n.</pos> <def>The practice of sportsmen; skill in field sports.</def><br/
[<source>1913 Webster</source>]</p>

<p><-- conduct becoming to one participating in sport or competition, as fair play, or graciousness in winning or losing. --><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sportula</ent><br/
||<hw>Spor"tu*la</hw> <pr>(sp<ocir/r"t<usl/*l<adot/)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Sportulae</plw> <pr>(-l<emac/)</pr>.</plu> <ety>[L.]</ety> <def>A gift; a present; a prize; hence, an alms; a largess.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>To feed luxuriously, to frequent sports and theaters, to run for the <qex>sportula</qex>.</q> <rj><qau>South.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sportulary</ent><br/
<hw>Spor"tu*la*ry</hw> <pr>(sp<ocir/r"t<usl/*l<asl/*r<ycr/)</pr>, <pos>a.</pos> <def>Subsisting on alms or charitable contributions.</def> <mark>[Obs.]</mark>  <rj><au>Bp. Hall.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sportule</ent><br/
<hw>Spor"tule</hw> <pr>(-t<umac/l)</pr>, <pos>n.</pos> <ety>[L. <ets>sportula</ets> a little basket, a gift, dim. of <ets>sporta</ets> a basket: cf. F. <ets>sportule</ets>.]</ety> <def>A charitable gift or contribution; a gift; an alms; a dole; a largess; a sportula.</def> <mark>[Obs.]</mark>  <rj><au>Ayliffe.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sporulation</ent><br/
<hw>Spor`u*la"tion</hw> <pr>(sp<ocr/r`<usl/*l<amac/"sh<ucr/n)</pr>, <pos>n.</pos> <fld>(Biol.)</fld> <def>The act or process of forming spores; spore formation. See <xex>Illust.</xex> of <er>Bacillus</er>, <sd>(b)</sd>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sporule</ent><br/
<hw>Spor"ule</hw> <pr>(sp<ocr/r"<usl/l)</pr>, <pos>n.</pos> <ety>[Dim. of <ets>spore</ets>.]</ety> <fld>(Biol.)</fld> <def>A small spore; a spore.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sporuliferous</ent><br/
<hw>Spor`u*lif"er*ous</hw> <pr>(-<usl/*l<icr/f"<etil/r*<ucr/s)</pr>, <pos>a.</pos> <ety>[<ets>Sporule</ets> + <ets>-ferous</ets>.]</ety> <fld>(Biol.)</fld> <def>Producing sporules.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spot</ent><br/
<hw>Spot</hw> <pr>(sp<ocr/t)</pr>, <pos>n.</pos> <ety>[Cf. Scot. & D. <ets>spat</ets>, Dan. <ets>spette</ets>, Sw. <ets>spott</ets> spittle, slaver; from the root of E. <ets>spit</ets>. See <er>Spit</er> to eject from the mouth, and cf. <er>Spatter</er>.]</ety> <sn>1.</sn> <def>A mark on a substance or body made by foreign matter; a blot; a place discolored.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Out, damned <qex>spot</qex>! Out, I say!</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A stain on character or reputation; something that soils purity; disgrace; reproach; fault; blemish.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Yet Chloe, sure, was formed without a <qex>spot</qex>.</q> <rj><qau>Pope.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>A small part of a different color from the main part, or from the ground upon which it is; <as>as, the <ex>spots</ex> of a leopard; the <ex>spots</ex> on a playing card</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>A small extent of space; a place; any particular place.</def> <ldquo/Fixed to one <xex>spot</xex>.<rdquo/  <rj><au>Otway.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>That <qex>spot</qex> to which I point is Paradise.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q><ldquo/A jolly place,<rdquo/ said he, <ldquo/in times of old!<br/
But something ails it now: the <qex>spot</qex> is cursed.<rdquo/</q> <rj><qau>Wordsworth.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <fld>(Zool.)</fld> <def>A variety of the common domestic pigeon, so called from a spot on its head just above its beak.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>6.</sn> <fld>(Zool.)</fld> <sd>(a)</sd> <def>A sciaenoid food fish (<spn>Liostomus xanthurus</spn>) of the Atlantic coast of the United States. It has a black spot behind the shoulders and fifteen oblique dark bars on the sides. Called also <altname>goody</altname>, <altname>Lafayette</altname>, <altname>masooka</altname>, and <altname>old wife</altname>.</def> <sd>(b)</sd> <def>The southern redfish, or red horse, which has a spot on each side at the base of the tail. See <er>Redfish</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>7.</sn> <pluf>pl.</pluf> <def>Commodities, as merchandise and cotton, sold for immediate delivery.</def> <mark>[Broker's Cant]</mark><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Crescent spot</b></col> <fld>(Zool.)</fld>, <cd>any butterfly of the family <fam>Melitaeidae</fam> having crescent-shaped white spots along the margins of the red or brown wings.</cd> -- <col><b>Spot lens</b></col> <fld>(Microscopy)</fld>, <cd>a condensing lens in which the light is confined to an annular pencil by means of a small, round diaphragm (the <xex>spot</xex>), and used in dark-field illumination; -- called also <altname>spotted lens</altname>.</cd> -- <col><b>Spot rump</b></col> <fld>(Zool.)</fld>, <cd>the Hudsonian godwit (<spn>Limosa haemastica</spn>).</cd> -- <col><b>Spots on the sun</b></col>. <fld>(Astron.)</fld> <cd>See <cref>Sun spot</cref>, ander <er>Sun</er>.</cd> -- <mcol><col><b>On the spot</b></col>, <it>or</it>  <col><b>Upon the spot</b></col></mcol>, <cd>immediately; before moving; without changing place; <as>as, he made his decision <ex>on the spot</ex></as>.</cd><br/
<br/
<q>It was determined <qex>upon the spot</qex>.</q> <rj><qau>Swift.</qau></rj></cs><br/
[<source>1913 Webster</source>]</p>

<p><syn><b>Syn.</b> -- Stain; flaw; speck; blot; disgrace; reproach; fault; blemish; place; site; locality.</syn><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spot</ent><br/
<hw>Spot</hw>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Spotted</conjf>; <pos>p. pr. & vb. n.</pos> <conjf>Spotting</conjf>.]</vmorph> <sn>1.</sn> <def>To make visible marks upon with some foreign matter; to discolor in or with spots; to stain; to cover with spots or figures; <as>as, to <ex>spot</ex> a garment; to <ex>spot</ex> paper.</as></def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To mark or note so as to insure recognition; to recognize; to detect; <as>as, to <ex>spot</ex> a criminal</as>.</def> <mark>[Cant]</mark><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>To stain; to blemish; to taint; to disgrace; to tarnish, as reputation; to asperse.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>My virgin life no <qex>spotted</qex> thoughts shall stain.</q> <rj><qau>Sir P. Sidney.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>If ever I shall close these eyes but once,<br/
May I live <qex>spotted</qex> for my perjury.</q> <rj><qau>Beau. & Fl.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>To spot timber</b></col>, <cd>to cut or chip it, in preparation for hewing.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spot</ent><br/
<hw>Spot</hw>, <pos>v. i.</pos> <def>To become stained with spots.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spot</ent><br/
<hw>Spot</hw> <pr>(sp<ocr/t)</pr>, <pos>a.</pos> <def>Lit., being on the spot, or place;</def> <specif>hence</specif> <fld>(Com.)</fld>, <def>on hand for immediate delivery after sale; -- said of commodities; <as>as, <ex>spot</ex> wheat</as>.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Spot cash</ent><br/
<hw>Spot cash</hw>. <fld>(Com.)</fld> <def>Cash paid or ready for payment at once upon delivery of property purchased.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Spotless</ent><br/
<hw>Spot"less</hw>, <pos>a.</pos> <def>Without a spot; especially, free from reproach or impurity; pure; untainted; innocent; <as>as, a <ex>spotless</ex> mind; <ex>spotless</ex> behavior</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>A <qex>spotless</qex> virgin, and a faultless wife.</q> <rj><qau>Waller.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><syn><b>Syn.</b> -- Blameless; unspotted; unblemished; pure; immaculate; irreproachable. See <er>Blameless</er>.</syn><br/
[<source>1913 Webster</source>]</p>

<p>-- <wordforms><wf>Spot"less*ly</wf>, <pos>adv.</pos> -- <wf>Spot"less*ness</wf>, <pos>n.</pos></wordforms><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spotlight</ent><br/
<hw>Spot"light`</hw> <pr>(sp<ocr/t"l<imac/t`)</pr>, <pos>n.</pos> <sn>1.</sn> <def>The projected spot or circle of light used to illuminate brilliantly a single person or object or group on the stage, leaving the rest of the stage more or less unilluminated.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><sn>2.</sn> <def>hence, conspicuous public notice.</def> <mark>[Cant or Colloq.]</mark><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><sn>3.</sn> <def>A lighting apparatus designed to throw an intense beam of light onto a small area.</def><br/
[<source>PJC</source>]</p>

<p><ent>Spot stroke</ent><br/
<hw>Spot stroke</hw>. <fld>(Eng. Billiards)</fld> <def>The pocketing of the red ball in a top corner pocket from off its own spot so as to leave the cue ball in position for an easy winning hazard in either top corner pocket.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Spotted</ent><br/
<hw>Spot"ted</hw>, <pos>a.</pos> <def>Marked with spots; <as>as, a <ex>spotted</ex> garment or character</as>.</def> <ldquo/The <xex>spotted</xex> panther.<rdquo/  <rj><au>Spenser.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Spotted fever</b></col> <fld>(Med.)</fld>, <cd>a name applied to various eruptive fevers, esp. to typhus fever and cerebro-spinal meningitis.</cd> -- <col><b>Spotted tree</b></col> <fld>(Bot.)</fld>, <cd>an Australian tree (<spn>Flindersia maculosa</spn>); -- so called because its bark falls off in spots.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spottedness</ent><br/
<hw>Spot"ted*ness</hw>, <pos>n.</pos> <def>State or quality of being spotted.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>spotter</ent><br/
<hw>spot"ter</hw> <pr>(sp<ocr/t"t<etil/r)</pr>, <pos>n.</pos> <def>One who spots.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>spottiness</ent><br/
<hw>spot"ti*ness</hw> <pr>(sp<ocr/t"t<icr/*n<ecr/s)</pr>, <pos>n.</pos> <def>The state or quality of being spotty.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spotty</ent><br/
<hw>Spot"ty</hw> <pr>(sp<ocr/t"t<ycr/)</pr>, <pos>a.</pos> <def>Full of spots; marked with spots.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spousage</ent><br/
<hw>Spous"age</hw> <pr>(spouz"<asl/j; 48)</pr>, <pos>n.</pos> <ety>[OF. <ets>espousaige</ets>, from <ets>espouser</ets>. See <er>Spouse</er>, <pos>v. t.</pos>]</ety> <def>Espousal.</def> <mark>[Obs.]</mark>  <rj><au>Bale.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spousal</ent><br/
<hw>Spous"al</hw> <pr>(-<ait/l)</pr>, <pos>a.</pos> <ety>[See <er>Espousal</er>, <er>Sponsal</er>, and <er>Spouse</er>.]</ety> <def>Of or pertaining to a spouse or marriage; nuptial; matrimonial; conjugal; connubial; bridal; <as>as, <ex>spousal</ex> rites; <ex>spousal</ex> ornaments</as>.</def>  <rj><au>Wordsworth.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spousal</ent><br/
<hw>Spous"al</hw>, <pos>n.</pos> <ety>[See <er>Espousal</er>, <er>Spouse</er>.]</ety> <def>Marriage; nuptials; espousal; -- generally used in the plural; <as>as, the <ex>spousals</ex> of Hippolita</as>.</def>  <rj><au>Dryden.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Boweth your head under that blissful yoke . . .<br/
Which that men clepeth <qex>spousal</qex> or wedlock.</q> <rj><qau>Chaucer.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>The <qex>spousals</qex> of the newborn year.</q> <rj><qau>Emerson.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spouse</ent><br/
<hw>Spouse</hw> <pr>(spouz)</pr>, <pos>n.</pos> <ety>[OF. <ets>espous</ets>, <ets>espos</ets>, fem. <ets>espouse</ets>, F. <ets><eacute/poux</ets>, <ets><eacute/pouse</ets>, fr. L. <ets>sponsus</ets>, <ets>sponsa</ets>, prop. p. p. of <ets>spondere</ets>, <ets>sponsum</ets>, to promise solemnly, to engage one's self.  Cf. <er>Despond</er>, <er>Espouse</er>, <er>Respond</er>, <er>Sponsor</er>.]</ety> <sn>1.</sn> <def>A man or woman engaged or joined in wedlock; a married person, husband or wife.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>At last such grace I found, and means I wrought,<br/
That I that lady to my <qex>spouse</qex> had won.</q> <rj><qau>Spenser.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A married man, in distinction from a <contr>spousess</contr> or <contr>married woman</contr>; a bridegroom or husband.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>At which marriage was [were] no persons present but the <qex>spouse</qex>, the spousess, the Duchess of Bedford her mother, the priest, two gentlewomen, and a young man.</q> <rj><qau>Fabyan.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spouse</ent><br/
<hw>Spouse</hw> <pr>(spouz)</pr>, <pos>v. t.</pos> <ety>[See <er>Espouse</er>, and <er>Spouse</er>, <pos>n.</pos>]</ety> <def>To wed; to espouse.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>This markis hath her <qex>spoused</qex> with a ring.</q> <rj><qau>Chaucer.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Though <qex>spoused</qex>, yet wanting wedlock's solemnize.</q> <rj><qau>Spenser.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>She was found again, and <qex>spoused</qex> to Marinell.</q> <rj><qau>Spenser.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spouse-breach</ent><br/
<hw>Spouse"-breach`</hw> <pr>(-br<emac/ch`)</pr>, <pos>n.</pos> <def>Adultery.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spouseless</ent><br/
<hw>Spouse"less</hw>, <pos>a.</pos> <def>Destitute of a spouse; unmarried.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spousess</ent><br/
<hw>Spous"ess</hw>, <pos>n.</pos> <def>A wife or bride.</def> <mark>[Obs.]</mark>  <rj><au>Fabyan.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spout</ent><br/
<hw>Spout</hw> <pr>(spout)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Spouted</conjf>; <pos>p. pr. & vb. n.</pos> <conjf>Spouting</conjf>.]</vmorph> <ety>[Cf. Sw. <ets>sputa</ets>, <ets>spruta</ets>, to spout, D. <ets>spuit</ets> a spout, <ets>spuiten</ets> to spout, and E. <ets>spurt</ets>, <ets>sprit</ets>, v., <ets>sprout</ets>, <ets>sputter</ets>; or perhaps akin to E. <ets>spit</ets> to eject from the mouth.]</ety> <sn>1.</sn> <def>To throw out forcibly and abundantly, as liquids through an orifice or a pipe; to eject in a jet; <as>as, an elephant <ex>spouts</ex> water from his trunk</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Who kept Jonas in the fish's maw<br/
Till he was <qex>spouted</qex> up at Ninivee?</q> <rj><qau>Chaucer.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Next on his belly floats the mighty whale . . . <br/
He <qex>spouts</qex> the tide.</q> <rj><qau>Creech.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To utter magniloquently; to recite in an oratorical or pompous manner.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Pray, <qex>spout</qex> some French, son.</q> <rj><qau>Beau. & Fl.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>To pawn; to pledge; <as>as, to <ex>spout</ex> a watch</as>.</def> <mark>[Cant]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spout</ent><br/
<hw>Spout</hw>, <pos>v. i.</pos> <sn>1.</sn> <def>To issue with violence, or in a jet, as a liquid through a narrow orifice, or from a spout; <as>as, water <ex>spouts</ex> from a hole; blood <ex>spouts</ex> from an artery</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>All the glittering hill<br/
Is bright with <qex>spouting</qex> rills.</q> <rj><qau>Thomson.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To eject water or liquid in a jet.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>To utter a speech, especially in a pompous manner.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spout</ent><br/
<hw>Spout</hw>, <pos>n.</pos> <ety>[Cf. Sw. <ets>spruta</ets> a squirt, a syringe. See <er>Spout</er>, <pos>v. t.</pos>]</ety> <sn>1.</sn> <def>That through which anything spouts; a discharging lip, pipe, or orifice; a tube, pipe, or conductor of any kind through which a liquid is poured, or by which it is conveyed in a stream from one place to another; <as>as, the <ex>spout</ex> of a teapot; a <ex>spout</ex> for conducting water from the roof of a building.</as></def> <au>Addison.</au> <ldquo/A conduit with three issuing <xex>spouts</xex>.<rdquo/ <au>Shak.</au><br/
[<source>1913 Webster</source>]</p>

<p><q>In whales . . . an ejection thereof [water] is contrived by a fistula, or <qex>spout</qex>, at the head.</q> <rj><qau>Sir T. Browne.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>From silver <qex>spouts</qex> the grateful liquors glide.</q> <rj><qau>Pope.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A trough for conducting grain, flour, etc., into a receptacle.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>A discharge or jet of water or other liquid, esp. when rising in a column; also, a waterspout.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><mcol><col><b>To put up the spout</b></col>, <col><b>To shove up the spout</b></col>, <it>or</it> <col><b>To pop up the spout</b></col></mcol>, <cd>to pawn or pledge at a pawnbroker's; -- in allusion to the <xex>spout</xex> up which the pawnbroker sent the ticketed articles.</cd> <mark>[Cant]</mark></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spouter</ent><br/
<hw>Spout"er</hw> <pr>(-<etil/r)</pr>, <pos>n.</pos> <def>One who, or that which, spouts.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spoutfish</ent><br/
<hw>Spout"fish`</hw> <pr>(-f<icr/sh`)</pr>, <pos>n.</pos> <fld>(Zool.)</fld> <def>A marine animal that spouts water; -- applied especially to certain bivalve mollusks, like the long clams (<gen>Mya</gen>), which spout, or squirt out, water when retiring into their holes.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spoutless</ent><br/
<hw>Spout"less</hw>, <pos>a.</pos> <def>Having no spout.</def>  <rj><au>Cowper.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spoutshell</ent><br/
<hw>Spout"shell`</hw> <pr>(-sh<ecr/l`)</pr>, <pos>n.</pos> <fld>(Zool.)</fld> <def>Any marine gastropod shell of the genus <gen>Apporhais</gen> having an elongated siphon. See <xex>Illust.</xex> under <er>Rostrifera</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sprack</ent><br/
<hw>Sprack</hw> <pr>(spr<acr/k)</pr>, <pos>a.</pos> <ety>[Cf. Icel. <ets>spraekr</ets> sprightly, dial. Sw. <ets>spr<aum/k</ets>, <ets>spr<aum/g</ets>, spirited, mettlesome; or Gael. <ets>spraic</ets> vigor.]</ety> <def>Quick; lively; alert.</def> <mark>[Prov. Eng. & Scot.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sprad</ent><br/
<hw>Sprad</hw> <pr>(spr<acr/d)</pr>, <mark>obs.</mark> <def><pos>p. p.</pos> of <er>Spread</er>.</def>  <rj><au>Chaucer.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spradde</ent><br/
<hw>Sprad"de</hw> <pr>(-d<eit/)</pr>, <mark>obs.</mark> <def><pos>imp.</pos> of <er>Spread</er>.</def>  <rj><au>Chaucer.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sprag</ent><br/
<hw>Sprag</hw> <pr>(spr<acr/g)</pr>, <pos>n.</pos> <ety>[Cf. Icel. <ets>spraka</ets> a small flounder.]</ety> <fld>(Zool.)</fld> <def>A young salmon.</def> <mark>[Prov. Eng.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sprag</ent><br/
<hw>Sprag</hw>, <pos>n.</pos> <ety>[See <er>Spray</er> a branch.]</ety> <def>A billet of wood; a piece of timber used as a prop.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sprag</ent><br/
<hw>Sprag</hw>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Spragged</conjf> <pr>(spr<acr/gd)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Spragging</conjf> <pr>(-g<icr/ng)</pr>.]</vmorph> <sn>1.</sn> <def>To check the motion of, as a carriage on a steep grade, by putting a sprag between the spokes of the wheel.</def>  <rj><au>R. S. Poole.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To prop or sustain with a sprag.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sprag</ent><br/
<hw>Sprag</hw>, <pos>a.</pos> <def>See <er>Sprack</er>, <pos>a.</pos></def>  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sprain</ent><br/
<hw>Sprain</hw> <pr>(spr<amac/n)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Sprained</conjf> <pr>(spr<amac/nd)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Spraining</conjf>.]</vmorph> <ety>[OF. <ets>espreindre</ets>to press, to force out, F. <ets><eacute/preindre</ets>, fr. L. <ets>exprimere</ets>. See <er>Express</er>, <pos>v. t.</pos>, and cf. <er>Spraints</er>.]</ety> <def>To weaken, as a joint, ligament, or muscle, by sudden and excessive exertion, as by wrenching; to overstrain, or stretch injuriously, but without luxation; <as>as, to <ex>sprain</ex> one's ankle</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sprain</ent><br/
<hw>Sprain</hw>, <pos>n.</pos> <def>The act or result of spraining; lameness caused by spraining; <as>as, a bad <ex>sprain</ex> of the wrist</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Sprain fracture</b></col> <fld>(Med.)</fld>, <cd>the separation of a tendon from its point of insertion, with the detachment of a shell of bone to which the tendon is attached.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spraints</ent><br/
<hw>Spraints</hw> <pr>(spr<amac/nts)</pr>, <pos>n. pl.</pos> <ety>[OF. <ets>espraintes</ets>, <ets>espreintes</ets>, F. <ets><eacute/preintes</ets>, from <ets>espreinte</ets> a desire to go to stool, from <ets>espreindre</ets>. See <er>Sprain</er>, <pos>v. t.</pos>]</ety> <def>The dung of an otter.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sprang</ent><br/
<hw>Sprang</hw> <pr>(spr<acr/ng)</pr>, <def><pos>imp.</pos> of <er>Spring</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sprat</ent><br/
<hw>Sprat</hw> <pr>(spr<acr/t)</pr>, <pos>n.</pos> <ety>[OE. <ets>sprot</ets>, <ets>sprotte</ets>, D. <ets>sprot</ets>; akin to G. <ets>sprotte</ets>.]</ety> <fld>(Zool.)</fld> <sd>(a)</sd> <def>A small European herring (<spn>Clupea sprattus</spn>) closely allied to the common herring and the pilchard; -- called also <altname>garvie</altname>. The name is also applied to small herring of different kinds.</def> <sd>(b)</sd> <def>A California surf-fish (<spn>Rhacochilus toxotes</spn>); -- called also <altname>alfione</altname>, and <altname>perch</altname>.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Sprat borer</b></col> <fld>(Zool.)</fld>, <cd>the red-throated diver; -- so called from its fondness for sprats. See <er>Diver</er>.</cd> -- <col><b>Sprat loon</b></col>. <fld>(Zool.)</fld> <sd>(a)</sd> <cd>The young of the great northern diver</cd>. <mark>[Prov. Eng.]</mark> <sd>(b)</sd> <cd>The red-throated diver. See <er>Diver</er>.</cd> -- <col><b>Sprat mew</b></col> <fld>(Zool.)</fld>, <cd>the kittiwake gull.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><-- p. 1393 pr=VMG --></p>

<p><ent>Sprawl</ent><br/
<hw>Sprawl</hw> <pr>(spr<add/l)</pr>, <pos>v. i.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Sprawled</conjf> <pr>(spr<add/ld)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Sprawling</conjf>.]</vmorph> <ety>[OE. <ets>spraulen</ets>; cf. Sw. <ets>sprattla</ets> to sprawl, dial. Sw. <ets>spralla</ets>, Dan. <ets>spraelle</ets>, <ets>spraelde</ets>, D. <ets>spartelen</ets>, <ets>spertelen</ets>, to flounder, to struggle.]</ety> <sn>1.</sn> <def>To spread and stretch the body or limbs carelessly in a horizontal position; to lie with the limbs stretched out ungracefully.</def>
<-- also sprawl out, as to sprawl out all over the couch. --><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To spread irregularly, as vines, plants, or trees; to spread ungracefully, as chirography.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>To move, when lying down, with awkward extension and motions of the limbs; to scramble in creeping.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The birds were not fledged; but upon <qex>sprawling</qex> and struggling to get clear of the flame, down they tumbled.</q> <rj><qau>L'Estrange.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>sprawl</ent><br/
<hw>sprawl</hw> <pr>(spr<add/l)</pr>, <pos>n.</pos> <def>The position or state resulting from sprawling; <as>as, he sat on the couch in a <ex>sprawl</ex>; uncontrolled urban <ex>sprawl</ex></as>.</def> <br/
[<source>PJC</source>]</p>

<p><ent>Sprawls</ent><br/
<hw>Sprawls</hw> <pr>(spr<add/lz)</pr>, <pos>n. pl.</pos> <def>Small branches of a tree; twigs; sprays.</def> <mark>[Prov. Eng.]</mark>  <rj><au>Halliwell.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spray</ent><br/
<hw>Spray</hw> <pr>(spr<amac/)</pr>, <pos>n.</pos> <ety>[Cf. Dan. <ets>sprag</ets>. See <er>Sprig</er>.]</ety> <sn>1.</sn> <def>A small shoot or branch; a twig.</def>  <rj><au>Chaucer.</au></rj><br/
<syn><b>Syn. --</b> sprig.</syn><br/
[<source>1913 Webster</source>]</p>

<p><q>The painted birds, companions of the spring,<br/
Hopping from <qex>spray</qex> to <qex>spray</qex>, were heard to sing.</q> <rj><qau>Dryden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A collective body of small branches, or cut flowers with long stems; <as>as, the tree has a beautiful <ex>spray</ex>; many <ex>sprays</ex> were sent in condolence to teh funeral home</as>.</def><br/
[<source>1913 Webster</source> <source>+PJC</source>]</p>

<p><q>And from the trees did lop the needless <qex>spray</qex>.</q> <rj><qau>Spenser.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <fld>(Founding)</fld> <sd>(a)</sd> <def>A side channel or branch of the runner of a flask, made to distribute the metal in all parts of the mold.</def> <sd>(b)</sd> <def>A group of castings made in the same mold and connected by sprues formed in the runner and its branches.</def>  <rj><au>Knight.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Spray drain</b></col> <fld>(Agric.)</fld>, <cd>a drain made by laying under earth the sprays or small branches of trees, which keep passages open.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spray</ent><br/
<hw>Spray</hw>, <pos>n.</pos> <ety>[Probably from a Dutch or Low German form akin to E. <ets>spread</ets>. See <er>Spread</er>, <pos>v. t.</pos>]</ety> <sn>1.</sn> <def>Water flying in small drops or particles, as by the force of wind, or the dashing of waves, or from a waterfall, and the like.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Med.)</fld> <sd>(a)</sd> <def>A jet of fine medicated vapor, used either as an application to a diseased part or to charge the air of a room with a disinfectant or a deodorizer.</def> <sd>(b)</sd> <def>An instrument for applying such a spray; an atomizer.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Spray condenser</b></col> <fld>(Steam Engine)</fld>, <cd>an injection condenser in which the steam is condensed by a spray of water which mingles with it.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spray</ent><br/
<hw>Spray</hw>, <pos>v. t.</pos> <sn>1.</sn> <def>To let fall in the form of spray.</def> <mark>[Poetic]</mark>  <rj><au>M. Arnold.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To throw spray upon; to treat with a liquid in the form of spray; <as>as, to <ex>spray</ex> a wound, or a surgical instrument, with carbolic acid</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sprayboard</ent><br/
<hw>Spray"board`</hw> <pr>(spr<amac/"b<omac/rd`)</pr>, <pos>n.</pos> <sn>1.</sn> <fld>(Naut.)</fld> <def>See <er>Dashboard</er>, <pos>n.</pos>, 2 <sd>(b)</sd>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Naut.)</fld> <def>A screen raised above any part of the gunwale of a boat to keep out spray.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Spray can</ent><br/
<hw>Spray" can`</hw> <pr>(spr<amac/"k<acr/n`)</pr>, <pos>n.</pos> <def>a small can containing a liquid under pressure of a gas that forces the liquid out in an aerosol form when the valve is opened.</def><br/
[<source>PJC</source>]</p>

<p><ent>Sprayer</ent><br/
<hw>Spray"er</hw> <pr>(spr<amac/"<etil/r)</pr>, <pos>n.</pos> <def>One that sprays; any instrument for vaporizing and spraying liquids.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Spray gun</ent><br/
<hw>Spray" gun`</hw> <pr>(spr<amac/"g<ucr/n`)</pr>, <pos>n.</pos> <def>a container for liquid, fitted with a conduit that can be supplied with compressed air, desiged to allowing the contents to be sprayed as an aerosol.  Such devices are used to spray paints and various other liquids.</def><br/
[<source>PJC</source>]</p>

<p><ent>Spray paint</ent><br/
<hw>Spray" paint`</hw> <pr>(spr<amac/"p<amac/nt`)</pr>, <pos>n.</pos> <def>paint packaged in a <er>spray can</er>.</def><br/
[<source>PJC</source>]</p>

<p><ent>Spray-paint</ent><br/
<hw>Spray"-paint</hw> <pr>(spr<amac/"p<amac/nt)</pr>, <pos>v. t.</pos> <def>To paint (a surface) by means of a spray, using a <er>spray can</er> or <er>spray gun</er>.</def><br/
[<source>PJC</source>]</p>

<p><ent>Spread</ent><br/
<hw>Spread</hw> <pr>(spr<ecr/d)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Spread</conjf>; <pos>p. pr. & vb. n.</pos> <conjf>Spreading</conjf>.]</vmorph> <ety>[OE. <ets>spreden</ets>, AS. <ets>spraedan</ets>; akin to D. <ets>spreiden</ets>, <ets>spreijen</ets>, LG. <ets>spreden</ets>, <ets>spreen</ets>, <ets>spreien</ets>, G. <ets>spreiten</ets>, Dan. <ets>sprede</ets>, Sw. <ets>sprida</ets>.  Cf. <er>Spray</er> water flying in drops.]</ety> <sn>1.</sn> <def>To extend in length and breadth, or in breadth only; to stretch or expand to a broad or broader surface or extent; to open; to unfurl; <as>as, to <ex>spread</ex> a carpet; to <ex>spread</ex> a tent or a sail</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>He bought a parcel of a field where he had <qex>spread</qex> his tent.</q> <rj><qau>Gen. xxxiii. 19.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Here the Rhone<br/
Hath <qex>spread</qex> himself a couch.</q> <rj><qau>Byron.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To extend so as to cover something; to extend to a great or greater extent in every direction; to cause to fill or cover a wide or wider space.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Rose, as in a dance, the stately trees, and <qex>spread</qex><br/
Their branches hung with copious fruit.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>To divulge; to publish, as news or fame; to cause to be more extensively known; to disseminate; to make known fully; <as>as, to <ex>spread</ex> a report; -- often accompanied by <ex>abroad</ex></as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>They, when they were departed, <qex>spread</qex> abroad his fame in all that country.</q> <rj><qau>Matt. ix. 31.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>To propagate; to cause to affect great numbers; <as>as, to <ex>spread</ex> a disease</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <def>To diffuse, as emanations or effluvia; to emit; <as>as, odoriferous plants <ex>spread</ex> their fragrance</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>6.</sn> <def>To strew; to scatter over a surface; <as>as, to <ex>spread</ex> manure; to <ex>spread</ex> lime on the ground</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>7.</sn> <def>To prepare; to set and furnish with provisions; <as>as, to <ex>spread</ex> a table</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Boiled the flesh, and <qex>spread</qex> the board.</q> <rj><qau>Tennyson.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>To spread cloth</b></col>, <cd>to unfurl sail.</cd> <mark>[Obs.]</mark>  <rj><au>Evelyn.</au></rj></cs><br/
[<source>1913 Webster</source>]</p>

<p><syn><b>Syn.</b> -- To diffuse; propagate; disperse; publish; distribute; scatter; circulate; disseminate; dispense.</syn><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spread</ent><br/
<hw>Spread</hw>, <pos>v. i.</pos> <sn>1.</sn> <def>To extend in length and breadth in all directions, or in breadth only; to be extended or stretched; to expand.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Plants, if they <qex>spread</qex> much, are seldom tall.</q> <rj><qau>Bacon.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Governor Winthrop, and his associates at Charlestown, had for a church a large, <qex>spreading</qex> tree.</q> <rj><qau>B. Trumbull.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To be extended by drawing or beating; <as>as, some metals <ex>spread</ex> with difficulty</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>To be made known more extensively, as news.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>To be propagated from one to another; <as>as, the disease <ex>spread</ex> into all parts of the city</as>.</def>  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spread</ent><br/
<hw>Spread</hw>, <pos>n.</pos> <sn>1.</sn> <def>Extent; compass.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>I have got a fine <qex>spread</qex> of improvable land.</q> <rj><qau>Addison.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Expansion of parts.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>No flower hath <qex>spread</qex> like that of the woodbine.</q> <rj><qau>Bacon.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>A cloth used as a cover for a table or a bed.</def><-- bedspread --><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>A table, as spread or furnished with a meal; hence, an entertainment of food; a feast.</def> <mark>[Colloq.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <def>A privilege which one person buys of another, of demanding certain shares of stock at a certain price, or of delivering the same shares of stock at another price, within a time agreed upon.</def> <mark>[Brokers' Cant]</mark><br/
[<source>1913 Webster</source>]</p>

<p><sn>6.</sn> <fld>(Geom.)</fld> <def>An unlimited expanse of discontinuous points.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>7.</sn> <fld>(Finance)</fld> <def>An arbitrage transaction operated by buying and selling simultaneously in two separate markets, as Chicago and New York, when there is an abnormal difference in price between the two markets.  It is called a <col><b>back spread</b></col>when the difference in price is less than the normal one.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><sn>8.</sn>  <fld>(Gems)</fld> <def>Surface in proportion to the depth of a cut stone.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Spread</ent><br/
<hw>Spread</hw>, <def><pos>imp. & p. p.</pos> of <er>Spread</er>, <pos>v.</pos></def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Spread eagle</b></col>. <sd>(a)</sd> <cd>An eagle with outspread wings, the national emblem of the United States.</cd> <sd>(b)</sd> <cd>The figure of an eagle, with its wings elevated and its legs extended; often met as a device upon military ornaments, and the like.</cd> <sd>(c)</sd> <fld>(Her.)</fld> <cd>An eagle displayed; an eagle with the wings and legs extended on each side of the body, as in the double-headed eagle of Austria and Russia. See <er>Displayed</er>, 2.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>spread-eagle</ent><br/
<hw>spread"-ea`gle</hw> <pr>(spr<ecr/d"<emac/`g'l)</pr>, <pos>a.</pos> <def>Characterized by a pretentious, boastful, exaggerated style; defiantly or extravagantly bombastic, especially regarding of the greatness of the U. S.; <as>as, a <ex>spread-eagle</ex> orator; a <ex>spread-eagle</ex> speech</as>.</def> <mark>[Colloq. & Humorous]</mark> <au>[MW10]</au><br/
[<source>1913 Webster</source> <source>+PJC</source>]</p>

<p><ent>spread-eagle</ent><br/
<hw>spread"-ea`gle</hw> <pos>v. i.</pos> <def>To assume a spread-eagled position; -- it may be done reclining, for relaxation, or momentarily, as an exhibitionary maneuver in a sport.</def><br/
[<source>PJC</source>]</p>

<p><ent>spread-eagled</ent><br/
<hw>spread"-ea`gled</hw> <pos>a.</pos> <def>Being in a position with the arms and legs extended fully.</def><br/
[<source>PJC</source>]</p>

<p><ent>Spreader</ent><br/
<hw>Spread"er</hw> <pr>(-<etil/r)</pr>, <pos>n.</pos> <sn>1.</sn> <def>One who, or that which, spreads, expands, or propagates.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A machine for combining and drawing fibers of flax to form a sliver preparatory to spinning.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spreadingly</ent><br/
<hw>Spread"ing*ly</hw>, <pos>adv.</pos> <def>Increasingly.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The best times were <qex>spreadingly</qex> infected.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sprechery</ent><br/
<hw>Sprech"er*y</hw> <pr>(spr<ecr/ch"<etil/r*<ycr/)</pr>, <pos>n.</pos> <ety>[Cf. Gael. <ets>spreidh</ets> cattle.]</ety> <def>Movables of an inferior description; especially, such as have been collected by depredation.</def> <mark>[Scot.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spree</ent><br/
<hw>Spree</hw> <pr>(spr<emac/)</pr>, <pos>n.</pos> <ety>[Cf. Ir. <ets>spre</ets> a spark, animation, spirit, Gael. <ets>spraic</ets>.  Cf. <er>Sprack</er>.]</ety> <def>A merry frolic; especially, a drinking frolic; a carousal.</def> <mark>[Colloq.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>spending spree</b></col> <cd>an incident in which one spends money freely; -- usually designating indiscreet or reckless spending on unneeded items.</cd></cs><br/
[<source>PJC</source>]</p>

<p><ent>Sprenge</ent><br/
<hw>Sprenge</hw> <pr>(spr<ecr/nj)</pr>, <pos>v. t.</pos> <ety>[OE. <ets>sprengen</ets>, p. p. <ets>sprent</ets>, <ets>spreint</ets>, from AS. <ets>sprengan</ets> to sprinkle. See <er>Sprinkle</er>.]</ety> <def>To sprinkle; to scatter.</def> <mark>[Obs.]</mark>  <rj><au>Wyclif (1 Pet. i. 2).</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sprengel pump</ent><br/
<hw>Spreng"el pump`</hw> <pr>(spr<ecr/ng"<eit/l p<ucr/mp`)</pr>. <fld>(Physics)</fld> <def>A form of air pump in which exhaustion is produced by a stream of mercury running down a narrow tube, in the manner of an aspirator; -- named from the inventor.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sprent</ent><br/
<hw>Sprent</hw> <pr>(spr<ecr/nt)</pr>, <mark>obs.</mark> <def><pos>p. p.</pos> of <er>Sprenge</er>. Sprinkled.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>All the ground with purple blood was <qex>sprent</qex>.</q> <rj><qau>Spenser.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sprew</ent><br/
<hw>Sprew</hw> <pr>(spr<udd/)</pr>, <pos>n.</pos> <ety>[Cf. D. <ets>sprouw</ets>, <ets>spruw</ets>.]</ety> <fld>(Med.)</fld> <def>Thrush.</def> <mark>[Local, U.S.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spreynd</ent><br/
<hw>Spreynd</hw> <pr>(spr<amac/nd)</pr>, <mark>obs.</mark> <def><pos>p. p.</pos> of <er>Sprenge</er>. Sprinkled.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>When <qex>spreynd</qex> was holy water.</q> <rj><qau>Chaucer.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sprig</ent><br/
<hw>Sprig</hw> <pr>(spr<icr/g)</pr>, <pos>n.</pos> <ety>[AS. <ets>sprec</ets>; akin to Icel. <ets>sprek</ets> a stick.  Cf. <er>Spray</er> a branch.]</ety> <sn>1.</sn> <def>A small shoot or twig of a tree or other plant; a spray; <as>as, a <ex>sprig</ex> of laurel or of parsley</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A youth; a lad; -- used humorously or in slight disparagement.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>A <qex>sprig</qex> whom I remember, with a whey-face and a satchel, not so many years ago.</q> <rj><qau>Sir W. Scott.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>A brad, or nail without a head.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <fld>(Naut.)</fld> <def>A small eyebolt ragged or barbed at the point.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sprig</ent><br/
<hw>Sprig</hw>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Sprigged</conjf> <pr>(spr<icr/gd)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Sprigging</conjf> <pr>(-g<icr/ng)</pr>.]</vmorph> <def>To mark or adorn with the representation of small branches; to work with sprigs; <as>as, to <ex>sprig</ex> muslin</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sprigged</ent><br/
<hw>Sprigged</hw> <pr>(spr<icr/gd <it>or</it> spr<icr/g"g<ecr/d)</pr>, <pos>a.</pos> <def>Having sprigs.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spriggy</ent><br/
<hw>Sprig"gy</hw> <pr>(spr<icr/g"g<ycr/)</pr>, <pos>a.</pos> <def>Full of sprigs or small branches.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spright</ent><br/
<hw>Spright</hw> <pr>(spr<imac/t)</pr>, <pos>n.</pos> <ety>[See <er>Sprite</er>.]</ety> <sn>1.</sn> <def>Spirit; mind; soul; state of mind; mood.</def> <mark>[Obs.]</mark> <ldquo/The high heroic <xex>spright</xex>.<rdquo/  <rj><au>Spenser.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Wondrous great grief groweth in my <qex>spright</qex>.</q> <rj><qau>Spenser.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A supernatural being; a spirit; a shade; an apparition; a ghost.</def><br/
<syn><b>Syn. --</b> sprite.</syn><br/
[<source>1913 Webster</source>]</p>

<p><q>Forth he called, out of deep darkness dread,<br/
Legions of <qex>sprights</qex>.</q> <rj><qau>Spenser.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>To thee, O Father, Son, and Sacred <qex>Spright</qex>.</q> <rj><qau>Fairfax.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>A kind of short arrow.</def> <mark>[Obs.]</mark>  <rj><au>Bacon.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spright</ent><br/
<hw>Spright</hw>, <pos>v. t.</pos> <def>To haunt, as a spright.</def> <mark>[Obs.]</mark>  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sprightful</ent><br/
<hw>Spright"ful</hw> <pr>(-f<usdot/l)</pr>, <pos>a.</pos> <ety>[<ets>Spright</ets> sprite + <ets>full</ets>.]</ety> <def>Full of spirit or of life; spirited; earnest; vivacious; lively; brisk; nimble; gay.</def> <mark>[Obs.]</mark> -- <wordforms><wf>Spright"ful*ly</wf>, <pos>adv.</pos> <mark>[Obs.]</mark></wordforms>  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p>-- <wordforms><wf>Spright"ful*ness</wf>, <pos>n.</pos> <mark>[Obs.]</mark></wordforms><br/
[<source>1913 Webster</source>]</p>

<p><q>Spoke like a <qex>sprightful</qex> noble gentleman.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Steeds <qex>sprightful</qex> as the light.</q> <rj><qau>Cowley.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sprightless</ent><br/
<hw>Spright"less</hw>, <pos>a.</pos> <def>Destitute of life; dull; sluggish.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sprightliness</ent><br/
<hw>Spright"li*ness</hw> <pr>(-l<icr/*n<ecr/s)</pr>, <pos>n.</pos> <def>The quality or state of being sprightly; liveliness; life; briskness; vigor; activity; gayety; vivacity.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>In dreams, observe with what a <qex>sprightliness</qex> and alacrity does she [the soul] exert herself!</q> <rj><qau>Addison.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sprightly</ent><br/
<hw>Spright"ly</hw> <pr>(-l<ycr/)</pr>, <pos>a.</pos> <amorph>[<pos>Compar.</pos> <adjf>Sprightlier</adjf> <pr>(-l<icr/*<etil/r)</pr>; <pos>superl.</pos> <adjf>Sprightliest</adjf>.]</amorph> <ety>[See <er>Sprite</er>.]</ety> <def>Sprightlike, or spiritlike; lively; brisk; animated; vigorous; airy; gay; <as>as, a <ex>sprightly</ex> youth; a <ex>sprightly</ex> air; a <ex>sprightly</ex> dance.</as></def> <ldquo/<xex>Sprightly</xex> wit and love inspires.<rdquo/  <rj><au>Dryden.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>The <qex>sprightly</qex> Sylvia trips along the green.</q> <rj><qau>Pope.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sprigtail</ent><br/
<hw>Sprig"tail`</hw> <pr>(spr<icr/g"t<amac/l`)</pr>, <pos>n.</pos> <fld>(Zool.)</fld> <sd>(a)</sd> <def>The pintail duck; -- called also <altname>sprig</altname>, and <altname>spreet-tail</altname>.</def> <mark>[Local, U.S.]</mark> <sd>(b)</sd> <def>The sharp-tailed grouse.</def> <mark>[Local, U.S.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spring</ent><br/
<hw>Spring</hw> <pr>(spr<icr/ng)</pr>, <pos>v. i.</pos> <vmorph>[<pos>imp.</pos> <conjf>Sprang</conjf> <pr>(spr<acr/ng)</pr> or <conjf>Sprung</conjf> <pr>(spr<ucr/ng)</pr>; <pos>p. p.</pos> <conjf>Sprung</conjf>; <pos>p. pr. & vb. n.</pos> <conjf>Springing</conjf>.]</vmorph> <ety>[AS. <ets>springan</ets>; akin to D. & G. <ets>springen</ets>, OS. & OHG. <ets>springan</ets>, Icel. & Sw. <ets>springa</ets>, Dan. <ets>springe</ets>; cf. Gr. <grk>spe`rchesqai</grk> to hasten.  Cf. <er>Springe</er>, <er>Sprinkle</er>.]</ety><br/
[<source>1913 Webster</source>]</p>

<p><sn>1.</sn> <def>To leap; to bound; to jump.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The mountain stag that <qex>springs</qex><br/
From height to height, and bounds along the plains.</q> <rj><qau>Philips.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To issue with speed and violence; to move with activity; to dart; to shoot.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>And sudden light<br/
<qex>Sprung</qex> through the vaulted roof.</q> <rj><qau>Dryden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>To start or rise suddenly, as from a covert.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Watchful as fowlers when their game will <qex>spring</qex>.</q> <rj><qau>Otway.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>To fly back; <as>as, a bow, when bent, <ex>springs</ex> back by its elastic power</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <def>To bend from a straight direction or plane surface; to become warped; <as>as, a piece of timber, or a plank, sometimes <ex>springs</ex> in seasoning</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>6.</sn> <def>To shoot up, out, or forth; to come to the light; to begin to appear; to emerge; as a plant from its seed, as streams from their source, and the like; -- often followed by <ptcl>up</ptcl>, <ptcl>forth</ptcl>, or <ptcl>out</ptcl>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Till well nigh the day began to <qex>spring</qex>.</q> <rj><qau>Chaucer.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>To satisfy the desolate and waste ground, and to cause the bud of the tender herb to <qex>spring</qex> forth.</q> <rj><qau>Job xxxviii. 27.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Do not blast my <qex>springing</qex> hopes.</q> <rj><qau>Rowe.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>O, <qex>spring</qex> to light; auspicious Babe, be born.</q> <rj><qau>Pope.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>7.</sn> <def>To issue or proceed, as from a parent or ancestor; to result, as from a cause, motive, reason, or principle.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>[They found] new hope to <qex>spring</qex><br/
Out of despair, joy, but with fear yet linked.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>8.</sn> <def>To grow; to thrive; to prosper.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>What makes all this, but Jupiter the king,<br/
At whose command we perish, and we <qex>spring</qex>?</q> <rj><qau>Dryden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>To spring at</b></col>, <cd>to leap toward; to attempt to reach by a leap.</cd> -- <col><b>To spring forth</b></col>, <cd>to leap out; to rush out.</cd> -- <col><b>To spring in</b></col>, <cd>to rush in; to enter with a leap or in haste.</cd> -- <mcol><col><b>To spring on</b></col> <it>or</it> <col><b>To spring upon</b></col></mcol>, <cd>to leap on; to rush on with haste or violence; to assault.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spring</ent><br/
<hw>Spring</hw> <pr>(spr<icr/ng)</pr>, <pos>v. t.</pos> <sn>1.</sn> <def>To cause to spring up; to start or rouse, as game; to cause to rise from the earth, or from a covert; <as>as, to <ex>spring</ex> a pheasant</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To produce or disclose suddenly or unexpectedly; <as>as, to <ex>spring</ex> a surprise on someone; to <ex>spring</ex> a joke</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>She starts, and leaves her bed, and <qex>springs</qex> a light.</q> <rj><qau>Dryden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>The friends to the cause <qex>sprang</qex> a new project.</q> <rj><qau>Swift.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>To cause to explode; <as>as, to <ex>spring</ex> a mine</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>To crack or split; to bend or strain so as to weaken; <as>as, to <ex>spring</ex> a mast or a yard</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <def>To cause to close suddenly, as the parts of a trap operated by a spring; <as>as, to <ex>spring</ex> a trap</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>6.</sn> <def>To bend by force, as something stiff or strong; to force or put by bending, as a beam into its sockets, and allowing it to straighten when in place; -- often with <xex>in</xex>, <xex>out</xex>, etc.; <as>as, to <ex>spring</ex> in a slat or a bar</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>7.</sn> <def>To pass over by leaping; <as>as, to <ex>spring</ex> a fence</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>8.</sn> <def>To release (a person) from confinement, especially from a prison.</def> <mark>[colloquial]</mark> <br/
[<source>PJC</source>]</p>

<p><cs><col><b>To spring a butt</b></col> <fld>(Naut.)</fld>, <cd>to loosen the end of a plank in a ship's bottom.</cd> -- <col><b>To spring a leak</b></col> <fld>(Naut.)</fld>, <cd>to begin to leak.</cd> -- <col><b>To spring an arch</b></col> <fld>(Arch.)</fld>, <cd>to build an arch; -- a common term among masons; <as>as, <ex>to spring an arch</ex> over a lintel</as>.</cd> -- <col><b>To spring a rattle</b></col>, <cd>to cause a rattle to sound. See <cref>Watchman's rattle</cref>, under <er>Watchman</er>.</cd> -- <col><b>To spring the luff</b></col> <fld>(Naut.)</fld>, <cd>to ease the helm, and sail nearer to the wind than before; -- said of a vessel.</cd> <au>Mar. Dict.</au> -- <mcol><col><b>To spring a mast</b></col> <it>or</it> <col><b>To spring a spar</b></col></mcol> <fld>(Naut.)</fld>, <cd>to strain it so that it is unserviceable.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spring</ent><br/
<hw>Spring</hw>, <pos>n.</pos> <ety>[AS. <ets>spring</ets> a fountain, a leap. See <er>Spring</er>, <pos>v. i.</pos>]</ety> <sn>1.</sn> <def>A leap; a bound; a jump.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The prisoner, with a <qex>spring</qex>, from prison broke.</q> <rj><qau>Dryden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A flying back; the resilience of a body recovering its former state by its elasticity; <as>as, the <ex>spring</ex> of a bow</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>Elastic power or force.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Heavens! what a <qex>spring</qex> was in his arm!</q> <rj><qau>Dryden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>An elastic body of any kind, as steel, India rubber, tough wood, or compressed air, used for various mechanical purposes, as receiving and imparting power, diminishing concussion, regulating motion, measuring weight or other force.</def><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ The principal varieties of springs used in mechanisms are the <stype>spiral spring</stype> (Fig. <it>a</it>), the <stype>coil spring</stype> (Fig. <it>b</it>), the <stype>elliptic spring</stype> (Fig. <it>c</it>), the <stype>half-elliptic spring</stype> (Fig. <it>d</it>), the <stype>volute spring</stype>, the <stype>India-rubber spring</stype>, the <stype>atmospheric spring</stype>, etc.</note><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <def>Any source of supply; especially, the source from which a stream proceeds; an issue of water from the earth; a natural fountain.</def> <ldquo/All my <xex>springs</xex> are in thee.<rdquo/ <au>Ps. lxxxvii. 7.</au> <ldquo/A secret <xex>spring</xex> of spiritual joy.<rdquo/ <au>Bentley.</au> <ldquo/The sacred <xex>spring</xex> whence right and honor streams.<rdquo/  <rj><au>Sir J. Davies.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>6.</sn> <def>Any active power; that by which action, or motion, is produced or propagated; cause; origin; motive.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Our author shuns by vulgar <qex>springs</qex> to move<br/
The hero's glory, or the virgin's love.</q> <rj><qau>Pope.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>7.</sn> <def>That which springs, or is originated, from a source;</def> <specif>as:</specif> <sd>(a)</sd> <def>A race; lineage.</def> <mark>[Obs.]</mark> <au>Chapman.</au> <sd>(b)</sd> <def>A youth; a springal.</def> <mark>[Obs.]</mark> <au>Spenser.</au> <sd>(c)</sd> <def>A shoot; a plant; a young tree; also, a grove of trees; woodland.</def> <mark>[Obs.]</mark> <au>Spenser. Milton.</au><br/
[<source>1913 Webster</source>]</p>

<p><sn>8.</sn> <def>That which causes one to spring; specifically, a lively tune.</def> <mark>[Obs.]</mark>  <rj><au>Beau. & Fl.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>9.</sn> <def>The season of the year when plants begin to vegetate and grow; the vernal season, usually comprehending the months of March, April, and May, in the middle latitudes north of the equator.</def> <ldquo/The green lap of the new-come <xex>spring</xex>.<rdquo/  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ <xex>Spring</xex> of the astronomical year begins with the vernal equinox, about March 21st, and ends with the summer solstice, about June 21st.</note><br/
[<source>1913 Webster</source>]</p>

<p><sn>10.</sn> <def>The time of growth and progress; early portion; first stage; <as>as, the <xex>spring</xex> of life</as>.</def> <ldquo/The <xex>spring</xex> of the day.<rdquo/  <rj><au>1 Sam. ix. 26.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>O how this <qex>spring</qex> of love resembleth<br/
The uncertain glory of an April day.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>11.</sn> <fld>(Naut.)</fld> <sd>(a)</sd> <def>A crack or fissure in a mast or yard, running obliquely or transversely.</def> <sd>(b)</sd> <def>A line led from a vessel's quarter to her cable so that by tightening or slacking it she can be made to lie in any desired position; a line led diagonally from the bow or stern of a vessel to some point upon the wharf to which she is moored.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><mcol><col><b>Air spring</b></col>, <col><b>Boiling spring</b></col></mcol>, <cd>etc. See under <er>Air</er>, <er>Boiling</er>, etc.</cd> -- <col><b>Spring back</b></col> <fld>(Bookbinding)</fld>, <cd>a back with a curved piece of thin sheet iron or of stiff pasteboard fastened to the inside, the effect of which is to make the leaves of a book thus bound (as a ledger or other account or blank book) spring up and lie flat.</cd> -- <col><b>Spring balance</b></col>, <cd>a contrivance for measuring weight or force by the elasticity of a spiral spring of steel.</cd> -- <col><b>Spring beam</b></col>, <cd>a beam that supports the side of a paddle box. See <cref>Paddle beam</cref>, under <er>Paddle</er>, <pos>n.</pos></cd> -- <col><b>Spring beauty</b></col>. <sd>(a)</sd> <fld>(Bot.)</fld> <cd>Any plant of the genus <gen>Claytonia</gen>, delicate herbs with somewhat fleshy leaves and pretty blossoms, appearing in springtime</cd>. <sd>(b)</sd> <fld>(Zool.)</fld> <cd>A small, elegant American butterfly (<spn>Erora laeta</spn>) which appears in spring. The hind wings of the male are brown, bordered with deep blue; those of the female are mostly blue.</cd> -- <col><b>Spring bed</b></col>, <cd>a mattress, under bed, or bed bottom, in which springs, as of metal, are employed to give the required elasticity.</cd> -- <col><b>Spring beetle</b></col> <fld>(Zool.)</fld>, <cd>a snapping beetle; an elater.</cd> -- <col><b>Spring box</b></col>, <cd>the box or barrel in a watch, or other piece of mechanism, in which the spring is contained.</cd> -- <col><b>Spring fly</b></col> <fld>(Zool.)</fld>, <cd>a caddice fly; -- so called because it appears in the spring.</cd> -- <col><b>Spring grass</b></col> <fld>(Bot.)</fld>, <cd>vernal grass. See under <er>Vernal</er>.</cd> -- <col><b>Spring gun</b></col>, <cd>a firearm discharged by a spring, when this is trodden upon or is otherwise moved.</cd> -- <col><b>Spring hook</b></col> <fld>(Locomotive Engines)</fld>, <cd>one of the hooks which fix the driving-wheel spring to the frame.</cd> -- <col><b>Spring latch</b></col>, <cd>a latch that fastens with a spring.</cd> -- <col><b>Spring lock</b></col>, <cd>a lock that fastens with a spring.</cd> -- <col><b>Spring mattress</b></col>, <cd>a spring bed.</cd> -- <col><b>Spring of an arch</b></col> <fld>(Arch.)</fld> <cd>See <cref>Springing line of an arch</cref>, under <er>Springing</er>.</cd> -- <col><b>Spring of pork</b></col>, <cd>the lower part of a fore quarter, which is divided from the neck, and has the leg and foot without the shoulder.</cd> <mark>[Obs.]</mark> <au>Nares.</au><br/
<br/
<q>Sir, pray hand the <qex>spring of pork</qex> to me.</q> <rj><qau>Gayton.</qau></rj><br/
<br/
  -- <col><b>Spring pin</b></col> <fld>(Locomotive Engines)</fld>, <cd>an iron rod fitted between the springs and the axle boxes, to sustain and regulate the pressure on the axles.</cd> -- <col><b>Spring rye</b></col>, <cd>a kind of rye sown in the spring; -- in distinction from <xex>winter rye</xex>, sown in autumn.</cd> -- <col><b>Spring stay</b></col> <fld>(Naut.)</fld>, <cd>a preventer stay, to assist the regular one.</cd> <au>R. H. Dana, Jr.</au> -- <col><b>Spring tide</b></col>, <cd>the tide which happens at, or soon after, the new and the full moon, and which rises higher than common tides.</cd> See <er>Tide</er>. -- <col><b>Spring wagon</b></col>, <cd>a wagon in which springs are interposed between the body and the axles to form elastic supports.</cd> -- <col><b>Spring wheat</b></col>, <cd>any kind of wheat sown in the spring; -- in distinction from <xex>winter wheat</xex>, which is sown in autumn.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><-- p. 1394 pr=VMG --></p>

<p><ent>Springall</ent><br/
<ent>Springald</ent><br/
<ent>Springal</ent><br/
<mhw>{ <hw>Spring"al</hw> <pr>(spr<icr/ng"<ait/l)</pr>, <hw>Spring"ald</hw> <pr>(spr<icr/ng"<ait/ld)</pr>, <hw>Spring"all</hw> <pr>(spr<icr/ng"<ait/l)</pr>, }</mhw> <pos>n.</pos> <ety>[Scot. <ets>springald</ets>, <ets>springel</ets>, fr. Scot. & E. <ets>spring</ets>.]</ety> <def>An active, springy young man.</def> <mark>[Obs.]</mark> <ldquo/There came two <xex>springals</xex> of full tender years.<rdquo/  <rj><au>Spenser.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Joseph, when he was sold to Potiphar, that great man, was a fair young <qex>springall</qex>.</q> <rj><qau>Latimer.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Springal</ent><br/
<hw>Spring"al</hw>, <pos>n.</pos> <ety>[OF. <ets>espringale</ets>; of Teutonic origin, akin to E. <ets>spring</ets>.]</ety> <def>An ancient military engine for casting stones and arrows by means of a spring.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Springboard</ent><br/
<hw>Spring"board`</hw> <pr>(-b<omac/rd`)</pr>, <pos>n.</pos> <def>An elastic board, secured at the ends, or at one end, often by elastic supports, used in performing feats of agility or in exercising.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Springbuck</ent><br/
<ent>Springbok</ent><br/
<mhw>{ ||<hw>Spring"bok`</hw> <pr>(-b<ocr/k`)</pr>, <hw>Spring"buck`</hw> <pr>(-b<ucr/k`)</pr>, }</mhw> <pos>n.</pos> <ety>[D. <ets>springbok</ets>; <ets>springen</ets> to spring, leap + <ets>bok</ets> a he-goat, buck.]</ety> <fld>(Zool.)</fld> <def>A South African gazelle (<spn>Gazella euchore</spn>) noted for its graceful form and swiftness, and for its peculiar habit of springing lightly and suddenly into the air.  It has a white dorsal stripe, expanding into a broad patch of white on the rump and tail.  Called also <altname>springer</altname>.</def> <altsp>[Written also <asp>springboc</asp>, and <asp>springbock</asp>.]</altsp><br/
[<source>1913 Webster</source>]</p>

<p><ent>Springe</ent><br/
<hw>Springe</hw> <pr>(spr<icr/nj)</pr>, <pos>n.</pos> <ety>[From <er>Spring</er>, <pos>v. i.</pos>: cf. G. <ets>sprenkel</ets>, Prov. E. <ets>springle</ets>.]</ety> <def>A noose fastened to an elastic body, and drawn close with a sudden spring, whereby it catches a bird or other animal; a gin; a snare.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>As a woodcock to mine own <qex>springe</qex>.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Springe</ent><br/
<hw>Springe</hw>, <pos>v. t.</pos> <def>To catch in a springe; to insnare.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Springe</ent><br/
<hw>Spring"e</hw> <pr>(spr<icr/ng"<eit/ <or/ spr<icr/ng)</pr>, <pos>v. t.</pos> <ety>[OE. <ets>sprengen</ets>. See <er>Sprinkle</er>.]</ety> <def>To sprinkle; to scatter.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>He would sowen some difficulty,<br/
Or <qex>springen</qex> cockle in our cleane corn.</q> <rj><qau>Chaucer.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Springer</ent><br/
<hw>Spring"er</hw> <pr>(spr<icr/ng"<etil/r)</pr>, <pos>n.</pos> <sn>1.</sn> <def>One who, or that which, springs; specifically, one who rouses game.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A young plant.</def> <mark>[Obs.]</mark>  <rj><au>Evelyn.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <fld>(Arch.)</fld> <sd>(a)</sd> <def>The impost, or point at which an arch rests upon its support, and from which it seems to spring.</def> Hence: <sd>(b)</sd> <def>The bottom stone of an arch, which lies on the impost. The skew back is one form of springer.</def> <sd>(c)</sd> <def>The rib of a groined vault, as being the solid abutment for each section of vaulting.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <fld>(Zool.)</fld> <def>The grampus.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <fld>(Zool.)</fld> <def>A variety of the field spaniel. See <er>Spaniel</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>6.</sn> <fld>(Zool.)</fld> <def>A species of antelope; the springbok.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Springhalt</ent><br/
<hw>Spring"halt`</hw> <pr>(-h<add/lt`)</pr>, <pos>n.</pos> <fld>(Far.)</fld> <def>A kind of lameness in a horse. See <er>Stringhalt</er>.</def>  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Springhead</ent><br/
<hw>Spring"head`</hw> <pr>(-h<ecr/d`)</pr>, <pos>n.</pos> <def>A fountain or source.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Springiness</ent><br/
<hw>Spring"i*ness</hw> <pr>(-<icr/*n<ecr/s)</pr>, <pos>n.</pos> <def>The state or quality of being springy.</def>  <rj><au>Boyle.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Springing</ent><br/
<hw>Spring"ing</hw>, <pos>n.</pos> <sn>1.</sn> <def>The act or process of one who, or that which, springs.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Growth; increase; also, that which springs up; a shoot; a plant.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Thou blessest the <qex>springing</qex> thereof.</q> <rj><qau>Ps. lxv. 10.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Springing line of an arch</b></col> <fld>(Arch.)</fld>, <cd>the horizontal line drawn through the junction of the vertical face of the impost with the curve of the intrados; -- called also <altname>spring of an arch</altname>.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Springle</ent><br/
<hw>Sprin"gle</hw> <pr>(spr<icr/<nsm/"g'l)</pr>, <pos>n.</pos> <def>A springe.</def> <mark>[Prov. Eng.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Springlet</ent><br/
<hw>Spring"let</hw> <pr>(spr<icr/ng"l<ecr/t)</pr>, <pos>n.</pos> <def>A little spring.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>But yet from out the little hill<br/
Oozes the slender <qex>springlet</qex> still.</q> <rj><qau>Sir W. Scott.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spring steel</ent><br/
<hw>Spring steel</hw>. <def>A variety of steel, elastic, strong, and tough, rolled for springs, etc.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Springtail</ent><br/
<hw>Spring"tail`</hw> <pr>(-t<amac/l`)</pr>, <pos>n.</pos> <fld>(Zool.)</fld> <def>Any one of numerous species of small apterous insects belonging to the order <ord>Thysanura</ord>. They have two elastic caudal stylets which can be bent under the abdomen and then suddenly extended like a spring, thus enabling them to leap to a considerable distance. See <er>Collembola</er>, and <er>Podura</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Springtide</ent><br/
<hw>Spring"tide`</hw> <pr>(-t<imac/d`)</pr>, <pos>n.</pos> <def>The time of spring; springtime.</def>  <rj><au>Thomson.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Springtime</ent><br/
<hw>Spring"time`</hw> <pr>(-t<imac/m`)</pr>, <pos>n.</pos> <def>The season of spring; springtide.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Springy</ent><br/
<hw>Spring"y</hw> <pr>(-<ycr/)</pr>, <pos>a.</pos> <amorph>[<pos>Compar.</pos> <adjf>Springier</adjf> <pr>(-<icr/*<etil/r)</pr>; <pos>superl.</pos> <adjf>Springiest</adjf>.]</amorph> <ety>[From <er>Spring</er>.]</ety> <sn>1.</sn> <def>Resembling, having the qualities of, or pertaining to, a spring; elastic; <as>as, <ex>springy</ex> steel; a <ex>springy</ex> step.</as></def><br/
[<source>1913 Webster</source>]</p>

<p><q>Though her little frame was slight, it was firm and <qex>springy</qex>.</q> <rj><qau>Sir W. Scott.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Abounding with springs or fountains; wet; spongy; <as>as, <ex>springy</ex> land</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sprinkle</ent><br/
<hw>Sprin"kle</hw> <pr>(spr<icr/<nsm/"k'l)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Sprinkled</conjf> <pr>(-k'ld)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Sprinkling</conjf> <pr>(-kl<icr/ng)</pr>.]</vmorph> <ety>[OE. <ets>sprenkelen</ets>, freq. of <ets>sprengen</ets> to sprinkle, to scatter, AS. <ets>sprengan</ets>, properly, to make to spring, causative of <ets>springan</ets> to spring; akin to D. <ets>sprenkelen</ets> to sprinkle, G. <ets>sprengen</ets>. See <er>Spring</er>, <pos>v. i.</pos>, and cf. <er>Sprent</er>.]</ety> <sn>1.</sn> <def>To scatter in small drops or particles, as water, seed, etc.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To scatter on; to disperse something over in small drops or particles; to besprinkle; <as>as, to <ex>sprinkle</ex> the earth with water; to <ex>sprinkle</ex> a floor with sand</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>To baptize by the application of a few drops, or a small quantity, of water; hence, to cleanse; to purify.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Having our hearts <qex>sprinkled</qex> from an evil conscience.</q> <rj><qau>Heb. x. 22.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sprinkle</ent><br/
<hw>Sprin"kle</hw>, <pos>v. i.</pos> <sn>1.</sn> <def>To scatter a liquid, or any fine substance, so that it may fall in particles.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>And the priest shall . . . <qex>sprinkle</qex> of the oil with his finger seven times before the Lord.</q> <rj><qau>Lev. xiv. 16.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To rain moderately, or with scattered drops falling now and then; <as>as, it <ex>sprinkles</ex></as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>To fly or be scattered in small drops or particles.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sprinkle</ent><br/
<hw>Sprin"kle</hw>, <pos>n.</pos> <sn>1.</sn> <def>A small quantity scattered, or sparsely distributed; a sprinkling.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A utensil for sprinkling; a sprinkler.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sprinkler</ent><br/
<hw>Sprin"kler</hw> <pr>(-kl<etil/r)</pr>, <pos>n.</pos> <sn>1.</sn> <def>One who sprinkles.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>An instrument or vessel used in sprinkling; specifically, a watering pot.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sprinkling</ent><br/
<hw>Sprin"kling</hw> <pr>(-kl<icr/ng)</pr>, <pos>n.</pos> <sn>1.</sn> <def>The act of one who, or that which, sprinkles.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Baptism may well enough be performed by <qex>sprinkling</qex> or effusion of water.</q> <rj><qau>Ayliffe.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A small quantity falling in distinct drops or particles; <as>as, a <ex>sprinkling</ex> of rain or snow</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>Hence, a moderate number or quantity distributed like separate drops, or as if scattered like drops.</def>  <rj><au>Craik.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sprint</ent><br/
<hw>Sprint</hw> <pr>(spr<icr/nt)</pr>, <pos>v. i.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Sprinted</conjf>; <pos>p. pr. & vb. n.</pos> <conjf>Sprinting</conjf>.]</vmorph> <ety>[Cf. <er>Sprunt</er>.]</ety> <def>To run very rapidly; to run at full speed.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>A runner [in a quarter-mile race] should be able to <qex>sprint</qex> the whole way.</q> <rj><qau>Encyc. Brit.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sprint</ent><br/
<hw>Sprint</hw>, <pos>n.</pos> <def>The act of sprinting; a run of a short distance at full speed.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Sprint race</b></col>, <cd>a foot race at the highest running speed; -- usually limited to distances under a quarter of a mile.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sprinter</ent><br/
<hw>Sprint"er</hw> <pr>(-<etil/r)</pr>, <pos>n.</pos> <def>One who sprints; one who runs in sprint races; <as>as, a champion <ex>sprinter</ex></as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sprit</ent><br/
<hw>Sprit</hw> <pr>(spr<icr/t)</pr>, <pos>v. t.</pos> <ety>[Akin to G. <ets>spritzen</ets>, <ets>spr<uum/tzen</ets>. See <er>Sprit</er>, <pos>v. i.</pos>]</ety> <def>To throw out with force from a narrow orifice; to eject; to spurt out.</def> <mark>[Obs.]</mark>  <rj><au>Sir T. Browne.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sprit</ent><br/
<hw>Sprit</hw>, <pos>v. i.</pos> <ety>[AS. <ets>spryttan</ets> to sprout, bud. See <er>Sprout</er>, <pos>v. i.</pos>, and cf. <er>Spurt</er>, <pos>v. t.</pos>, <er>Sprit</er> a spar.]</ety> <def>To sprout; to bud; to germinate, as barley steeped for malt.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sprit</ent><br/
<hw>Sprit</hw>, <pos>n.</pos> <def>A shoot; a sprout.</def> <mark>[Obs.]</mark>  <rj><au>Mortimer.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sprit</ent><br/
<hw>Sprit</hw>, <pos>n.</pos> <ety>[OE. <ets>spret</ets>, AS. <ets>spre<oacute/t</ets> a sprit; spear; akin to D. <ets>spriet</ets>, and E. <ets>sprout</ets>, <ets>sprit</ets>, v.t. & i. See <er>Sprout</er>, <pos>v. i.</pos>]</ety> <fld>(Naut.)</fld> <def>A small boom, pole, or spar, which crosses the sail of a boat diagonally from the mast to the upper aftmost corner, which it is used to extend and elevate.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sprite</ent><br/
<hw>Sprite</hw> <pr>(spr<imac/t)</pr>, <pos>n.</pos> <ety>[OE. <ets>sprit</ets>, F. <ets>esprit</ets>, fr. L. <ets>spiritus</ets>. See <er>Spirit</er>, and cf. <er>Sprightly</er>.]</ety> <sn>1.</sn> <def>A spirit; a soul; a shade; also, an apparition.  See <er>Spright</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Gaping graves received the wandering, guilty <qex>sprite</qex>.</q> <rj><qau>Dryden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>An elf; a fairy; a goblin.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <fld>(Zool.)</fld> <def>The green woodpecker, or yaffle.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spritely</ent><br/
<ent>Spriteliness</ent><br/
<ent>Spritefully</ent><br/
<ent>Spriteful</ent><br/
<mhw><hw>Sprite"ful</hw> <pr>(-f<usdot/l)</pr>, <pos>a.</pos>, <hw>Sprite"ful*ly</hw>, <pos>adv.</pos>, <hw>Sprite"li*ness</hw> <pr>(-l<icr/*n<ecr/s)</pr>, <pos>n.</pos>, <hw>Sprite"ly</hw>, <pos>a.</pos>, etc.</mhw> <def>See <er>Sprightful</er>, <er>Sprightfully</er>, <er>Sprightliness</er>, <er>Sprightly</er>, etc.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spritsail</ent><br/
<hw>Sprit"sail</hw> <pr>(spr<icr/t"s<amac/l; <it>among seamen</it> spr<icr/t"s'l)</pr>, <pos>n.</pos> <fld>(Naut.)</fld> <sd>(a)</sd> <def>A sail extended by a sprit.</def> <sd>(b)</sd> <def>A sail formerly hung under the bowsprit, from the spritsail yard.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sprocket</ent><br/
<hw>Sprock"et</hw> <pr>(spr<ocr/k"<ecr/t)</pr>, <pos>n.</pos> <ety>[Etymology uncertain.]</ety> <fld>(Mach.)</fld> <sd>(a)</sd> <def>A tooth or projection, as on the periphery of a wheel, shaped so as to engage with a chain.</def> <sd>(b)</sd> <def>A sprocket wheel.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Sprocket wheel</ent><br/
<hw>Sprock"et wheel`</hw> <pr>(spr<ocr/k"<ecr/t hw<emac/l`)</pr>. <ety>[Etymology of <ets>sprocket</ets> is uncertain.]</ety> <fld>(Mach.)</fld> <def>Same as <er>Chain wheel</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sprod</ent><br/
<hw>Sprod</hw> <pr>(spr<ocr/d)</pr>, <pos>n.</pos> <ety>[Cf. Gael. & Ir. <ets>bradan</ets> a salmon.]</ety> <fld>(Zool.)</fld> <def>A salmon in its second year.</def> <mark>[Prov. Eng.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sprong</ent><br/
<hw>Sprong</hw> <pr>(spr<ocr/ng)</pr>, <mark>obs.</mark> <pos>imp.</pos> <mord>of <er>Spring</er>.</mord> <def>Sprung.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sprout</ent><br/
<hw>Sprout</hw> <pr>(sprout)</pr>, <pos>v. i.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Sprouted</conjf>; <pos>p. pr. & vb. n.</pos> <conjf>Sprouting</conjf>.]</vmorph> <ety>[OE. <ets>sprouten</ets>, <ets>spruten</ets>; akin to OFries. <ets>spr<umac/ta</ets>, AS. <ets>spre<oacute/tan</ets>, D. <ets>spruiten</ets>, G. <ets>spriessen</ets>, Sw. <ets>spruta</ets> to squirt, to spout.  Cf. <er>Sprit</er>, <pos>v. t. & i.</pos>, <er>Sprit</er> a spar, <er>Spout</er>, <pos>v. t.</pos>, <er>Spurt</er>.]</ety> <sn>1.</sn> <def>To shoot, as the seed of a plant; to germinate; to push out new shoots; hence, to grow like shoots of plants.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To shoot into ramifications.</def> <mark>[Obs.]</mark>  <rj><au>Bacon.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sprout</ent><br/
<hw>Sprout</hw>, <pos>v. t.</pos> <sn>1.</sn> <def>To cause to sprout; <as>as, the rain will <ex>sprout</ex> the seed</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To deprive of sprouts; <as>as, to <ex>sprout</ex> potatoes</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sprout</ent><br/
<hw>Sprout</hw>, <pos>n.</pos> <ety>[Cf. AS. <ets>sprote</ets> a sprout, sprig; akin to Icel. <ets>sproti</ets>, G. <ets>sprosse</ets>. See <er>Sprout</er>, <pos>v. i.</pos>]</ety> <sn>1.</sn> <def>The shoot of a plant; a shoot from the seed, from the stump, or from the root or tuber, of a plant or tree; more rarely, a shoot from the stem of a plant, or the end of a branch.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <pluf>pl.</pluf> <def>Young coleworts; Brussels sprouts.</def>  <rj><au>Johnson.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Brussels sprouts</b></col> <fld>(Bot.)</fld> <cd>See under <er>Brussels</er>.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spruce</ent><br/
<hw>Spruce</hw> <pr>(spr<udd/s)</pr>, <pos>n.</pos> <ety>[OE. <ets>Spruce</ets> or <ets>Pruse</ets>, Prussia, Prussian. So named because it was first known as a native of Prussia, or because its sprouts were used for making, <ets>spruce beer</ets>.  Cf. <ets>Spruce beer</ets>, below, <er>Spruce</er>, <pos>a.</pos>]</ety> <sn>1.</sn> <fld>(Bot.)</fld> <def>Any coniferous tree of the genus <gen>Picea</gen>, as the Norway spruce (<spn>Picea excelsa</spn>), and the white and black spruces of America (<spn>Picea alba</spn> and <spn>Picea nigra</spn>), besides several others in the far Northwest. See <er>Picea</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>The wood or timber of the spruce tree.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>Prussia leather; pruce.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q><qex>Spruce</qex>, a sort of leather corruptly so called for Prussia leather.</q> <rj><qau>E. Phillips.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Douglas spruce</b></col> <fld>(Bot.)</fld>, <cd>a valuable timber tree (<spn>Pseudotsuga Douglasii</spn>) of Northwestern America.</cd> -- <col><b>Essence of spruce</b></col>, <cd>a thick, dark-colored, bitterish, and acidulous liquid made by evaporating a decoction of the young branches of spruce.</cd> -- <col><b>Hemlock spruce</b></col> <fld>(Bot.)</fld>, <cd>a graceful coniferous tree (<spn>Tsuga Canadensis</spn>) of North America. Its timber is valuable, and the bark is largely used in tanning leather.</cd> -- <col><b>Spruce beer</b></col>. <ety>[G. <ets>sprossenbier</ets>; <ets>sprosse</ets> sprout, shoot (akin to E. <ets>sprout</ets>, n.) + <ets>bier</ets> beer.  The word was changed into <ex>spruce</ex> beer because the beer came from Prussia (OE. <ets>Spruce</ets>), or because it was made from the sprouts of the spruce.  See <er>Sprout</er>, <pos>n.</pos>, <er>Beer</er>, and cf. <er>Spruce</er>, <pos>n.</pos>]</ety> <cd>A kind of beer which is tinctured or flavored with spruce, either by means of the extract or by decoction.</cd> -- <col><b>Spruce grouse</b></col>. <fld>(Zool.)</fld> <cd>Same as <cref>Spruce partridge</cref>, below.</cd> -- <col><b>Spruce leather</b></col>. <cd>See <er>Spruce</er>, <pos>n.</pos>, 3.</cd> -- <col><b>Spruce partridge</b></col> <fld>(Zool.)</fld>, <cd>a handsome American grouse (<spn>Dendragapus Canadensis</spn>) found in Canada and the Northern United States; -- called also <altname>Canada grouse</altname>.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spruce</ent><br/
<hw>Spruce</hw> <pr>(spr<udd/s)</pr>, <pos>a.</pos> <amorph>[<pos>Compar.</pos> <adjf>Sprucer</adjf> <pr>(spr<udd/"s<etil/r)</pr>; <pos>superl.</pos> <adjf>Sprucest</adjf>  <pr>(spr<udd/"s<ecr/st)</pr>.]</amorph> <ety>[Perhaps fr. <ets>spruce</ets> a sort of leather from Prussia, which was an article of finery. See <er>Spruce</er>, <pos>n.</pos>]</ety> <sn>1.</sn> <def>Neat, without elegance or dignity; smart; trim; -- formerly applied to things with a serious meaning; now chiefly applied to persons.</def> <ldquo/Neat and <xex>spruce</xex> array.<rdquo/  <rj><au>Remedy of Love.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Sprightly; dashing.</def> <mark>[Obs.]</mark> <ldquo/Now, my <xex>spruce</xex> companions.<rdquo/  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>He is so <qex>spruce</qex> that he can never be genteel.</q> <rj><qau>Tatler.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><syn><b>Syn.</b> -- Finical; neat; trim. See <er>Finical</er>.</syn><br/
[<source>1913 Webster</source>]</p>

<p>-- <wordforms><wf>Spruce"ly</wf>, <pos>adv.</pos> -- <wf>Spruce"ness</wf>, <pos>n.</pos></wordforms><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spruce</ent><br/
<hw>Spruce</hw>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Spruced</conjf> <pr>(spr<udd/st)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Sprucing</conjf> <pr>(spr<udd/"s<icr/ng)</pr>.]</vmorph> <def>To dress with affected neatness; to trim; to make spruce; -- often used with <ptcl>up</ptcl>; <as>as, to <ex>spruce</ex> up the house for Company</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spruce</ent><br/
<hw>Spruce</hw>, <pos>v. i.</pos> <def>To dress one's self with affected neatness; <as>as, to <ex>spruce</ex> up</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sprue</ent><br/
<hw>Sprue</hw> <pr>(spr<udd/)</pr>, <pos>n.</pos> <ety>[Etymol. uncertain.]</ety> <sn>1.</sn> <fld>(Founding)</fld> <sd>(a)</sd> <def>Strictly, the hole through which melted metal is poured into the gate, and thence into the mold.</def> <sd>(b)</sd> <def>The waste piece of metal cast in this hole; hence, dross.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Med.)</fld> <def>Same as <er>Sprew</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sprug</ent><br/
<hw>Sprug</hw> <pr>(spr<ucr/g)</pr>, <pos>v. t.</pos> <ety>[Cf. Prov. E. <ets>sprug up</ets> to dress neatly, <ets>sprag</ets> to prop, a., lively.]</ety> <def>To make smart.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sprung</ent><br/
<hw>Sprung</hw> <pr>(spr<ucr/ng)</pr>, <def><pos>imp. & p. p.</pos> of <er>Spring</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sprung</ent><br/
<hw>Sprung</hw>, <pos>a.</pos> <fld>(Naut.)</fld> <def>Said of a spar that has been cracked or strained.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sprunt</ent><br/
<hw>Sprunt</hw> <pr>(spr<ucr/nt)</pr>, <pos>v. i.</pos> <ety>[Cf. <er>Sprout</er>, <pos>v. i.</pos>]</ety> <def>To spring up; to germinate; to spring forward or outward.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>To sprunt up</b></col>, <cd>to draw one's self up suddenly, as in anger or defiance; to bristle up.</cd> <mark>[Local, U.S.]</mark></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sprunt</ent><br/
<hw>Sprunt</hw>, <pos>n.</pos> <sn>1.</sn> <def>Anything short and stiff.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A leap; a spring.</def> <mark>[Obs. <or/ Prov. Eng.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>A steep ascent in a road.</def> <mark>[Prov. Eng.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sprunt</ent><br/
<hw>Sprunt</hw>, <pos>a.</pos> <def>Active; lively; vigorous.</def> <mark>[Obs.]</mark>  <rj><au>Kersey.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spruntly</ent><br/
<hw>Sprunt"ly</hw>, <pos>adv.</pos> <def>In a sprunt manner; smartly; vigorously; youthfully.</def> <mark>[Obs.]</mark>  <rj><au>B. Jonson.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spry</ent><br/
<hw>Spry</hw> <pr>(spr<imac/)</pr>, <pos>a.</pos> <amorph>[<pos>Compar.</pos> <adjf>Sprier</adjf> or <adjf>Spryer</adjf> <pr>(-<etil/r)</pr>; <pos>superl.</pos> <adjf>Spriest</adjf> or <adjf>Spryest</adjf>.]</amorph> <ety>[Cf. dial. Sw. <ets>sprygg</ets> lively, skittish, and E. <ets>sprag</ets>.]</ety> <def>Having great power of leaping or running; nimble; active.</def> <mark>[U.S. & Local Eng.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>She is as <qex>spry</qex> as a cricket.</q> <rj><qau>S. Judd (Margaret).</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>If I'm not so large as you,<br/
You are not so small as I,<br/
And not half so <qex>spry</qex>.</q> <rj><qau>Emerson.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spud</ent><br/
<hw>Spud</hw> <pr>(sp<ucr/d)</pr>, <pos>n.</pos> <ety>[Cf. Dan. <ets>spyd</ets> a spear.]</ety> <sn>1.</sn> <def>A sharp, narrow spade, usually with a long handle, used by farmers for digging up large-rooted weeds; a similarly shaped implement used for various purposes.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>My <qex>spud</qex> these nettles from the stone can part.</q> <rj><qau>Swift.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A dagger.</def> <mark>[Obs.]</mark>  <rj><au>Holland.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>Anything short and thick; specifically, a piece of dough boiled in fat.</def> <mark>[Local, U.S.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>A potato.</def> <mark>[Colloq.]</mark><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Spue</ent><br/
<hw>Spue</hw> <pr>(sp<umac/)</pr>, <pos>v. t. & i.</pos> <def>See <er>Spew</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spuilzie</ent><br/
<hw>Spuil"zie</hw> <pr>(sp<ucr/l"z<icr/ <or/ -y<icr/)</pr>, <pos>n.</pos> <def>See <er>Spulzie</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spuke</ent><br/
<hw>Spuke</hw> <pr>(sp<umac/k)</pr>, <pos>n.</pos> <def>See <er>Spook</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spuller</ent><br/
<hw>Spul"ler</hw> <pr>(sp<ucr/l"l<etil/r)</pr>, <pos>n.</pos> <altsp>[For <asp>spooler</asp>.]</altsp> <ety>[See <er>Spool</er>.]</ety> <def>One employed to inspect yarn, to see that it is well spun, and fit for the loom.</def> <mark>[Prov. Eng.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>spulzie</ent><br/
<hw>spul"zie</hw> <pr>(sp<ucr/l"z<icr/ <or/ sp<ucr/l"y<icr/)</pr>, <pos>n.</pos> <ety>[Cf. <er>Spoil</er>.]</ety> <def>Plunder, or booty.</def> <altsp>[Written also <asp>spuilzie</asp>, and <asp>spulye</asp>.]</altsp>  <rj><au>Sir W. Scott.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spume</ent><br/
<hw>Spume</hw> <pr>(sp<umac/m)</pr>, <pos>n.</pos> <ety>[L. <ets>spuma</ets>.  Cf. <er>Pumice</er>, <er>Spoom</er>.]</ety> <def>Frothy matter raised on liquids by boiling, effervescence, or agitation; froth; foam; scum.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Materials dark and crude,<br/
Of spiritous and fiery <qex>spume</qex>.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spume</ent><br/
<hw>Spume</hw>, <pos>v. i.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Spumed</conjf> <pr>(sp<umac/md)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Spuming</conjf>.]</vmorph> <ety>[L. <ets>spumare</ets>.]</ety> <def>To froth; to foam.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spumeous</ent><br/
<hw>Spume"ous</hw> <pr>(-<ucr/s)</pr>, <pos>a.</pos> <def>Spumous.</def> <mark>[Obs.]</mark>  <rj><au>Dr. H. More.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spumescence</ent><br/
<hw>Spu*mes"cence</hw> <pr>(sp<usl/*m<ecr/s"s<eit/ns)</pr>, <pos>n.</pos> <ety>[See <er>Spumescent</er>.]</ety> <def>The state of being foamy; frothiness.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spumescent</ent><br/
<hw>Spu*mes"cent</hw> <pr>(-s<eit/nt)</pr>, <pos>a.</pos> <ety>[L. <ets>spumescens</ets>, p. pr. of <ets>spumescere</ets> to grow foamy, from <ets>spuma</ets> foam.]</ety> <def>Resembling froth or foam; foaming.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spumid</ent><br/
<hw>Spum"id</hw> <pr>(sp<umac/m"<icr/d)</pr>, <pos>a.</pos> <ety>[L. <ets>spumidus</ets>.]</ety> <def>Spumous; frothy.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spumiferous</ent><br/
<hw>Spu*mif"er*ous</hw> <pr>(sp<usl/*m<icr/f"<etil/r*<ucr/s)</pr>, <pos>a.</pos> <ety>[L. <ets>spumifier</ets>; <ets>spuma</ets> foam + <ets>ferre</ets> to bear.]</ety> <def>Producing foam.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spuminess</ent><br/
<hw>Spum"i*ness</hw> <pr>(sp<umac/m"<icr/*n<ecr/s)</pr>, <pos>n.</pos> <def>The quality or condition of being spumy; spumescence.</def><br/
[<source>1913 Webster</source>]</p>

<p><-- p. 1395 pr=VMG --></p>

<p><ent>Spumy</ent><br/
<ent>Spumous</ent><br/
<mhw>{ <hw>Spum"ous</hw> <pr>(sp<umac/m"<ucr/s)</pr>, <hw>Spum"y</hw> <pr>(sp<umac/m"<ycr/)</pr>, }</mhw> <pos>a.</pos> <ety>[L. <ets>spumosus</ets>, fr. <ets>spuma</ets> foam: cf. F. <ets>spumeux</ets>.]</ety> <def>Consisting of, containing, or covered with, froth, scum, or foam; frothy; foamy.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The <qex>spumous</qex> and florid state of the blood.</q> <rj><qau>Arbuthnot.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>The <qex>spumy</qex> waves proclaim the watery war.</q> <rj><qau>Dryden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spun</ent><br/
<hw>Spun</hw> <pr>(sp<ucr/n)</pr>, <def><pos>imp. & p. p.</pos> of <er>Spin</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Spun hay</b></col>, <cd>hay twisted into ropes for convenient carriage, as on a military expedition.</cd> -- <col><b>Spun silk</b></col>, <cd>a cheap article produced from floss, or short-fibered, broken, and waste silk, carded and spun, in distinction from the long filaments wound from the cocoon. It is often mixed with cotton.</cd> -- <col><b>Spun yarn</b></col> <fld>(Naut.)</fld>, <cd>a line formed of two or more rope-yarns loosely twisted.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spunge</ent><br/
<hw>Spunge</hw> <pr>(sp<ucr/nj)</pr>, <pos>n.</pos> <def>A sponge.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spunk</ent><br/
<hw>Spunk</hw> <pr>(sp<ucr/<nsm/k)</pr>, <pos>n.</pos> <ety>[Gael. <ets>spong</ets>, or Ir. <ets>sponc</ets>, tinder, sponge; cf. AS. <ets>sponge</ets> a sponge (L. <ets>spongia</ets>), <ets>sp<omac/n</ets> a chip.  Cf. <er>Sponge</er>, <er>Punk</er>.]</ety> <altsp>[Written also <asp>sponk</asp>.]</altsp> <sn>1.</sn> <def>Wood that readily takes fire; touchwood; also, a kind of tinder made from a species of fungus; punk; amadou.</def>  <rj><au>Sir T. Browne.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>An inflammable temper; spirit; mettle; pluck; <as>as, a man of <ex>spunk</ex></as>.</def> <mark>[Colloq.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>A lawless and dangerous set, men of <qex>spunk</qex>, and spirit, and power, both of mind and body.</q> <rj><qau>Prof. Wilson.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spunky</ent><br/
<hw>Spunk"y</hw> <pr>(-<ycr/)</pr>, <pos>a.</pos> <amorph>[<pos>Compar.</pos> <adjf>Spunkier</adjf> <pr>(-<icr/*<etil/r)</pr>; <pos>superl.</pos> <adjf>Spunkiest</adjf>.]</amorph> <def>Full of spunk; quick; spirited.</def> <mark>[Colloq.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spur</ent><br/
<hw>Spur</hw> <pr>(sp<ucir/r)</pr>, <pos>n.</pos> <ety>[See <er>Sparrow</er>.]</ety> <fld>(Zool.)</fld> <sd>(a)</sd> <def>A sparrow.</def> <mark>[Scot.]</mark> <sd>(b)</sd> <def>A tern.</def> <mark>[Prov. Eng.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spur</ent><br/
<hw>Spur</hw>, <pos>n.</pos> <ety>[OE. <ets>spure</ets>, <ets>spore</ets>, AS. <ets>spura</ets>, <ets>spora</ets>; akin to D. <ets>spoor</ets>, G. <ets>sporn</ets>, OHG. <ets>sporo</ets>, Icel. <ets>spori</ets>, Dan. <ets>spore</ets>, Sw. <ets>sporre</ets>, and to AS. <ets>spor</ets> a trace, footstep, <ets>spyrian</ets> to trace, track, examine, and E. <ets>spurn</ets>. <root/171.  Cf. <er>Sparrow</er>, <er>Spere</er>, <er>Spoor</er>, <er>Spurn</er>.]</ety> <sn>1.</sn> <def>An implement secured to the heel, or above the heel, of a horseman, to urge the horse by its pressure. Modern spurs have a small wheel, or rowel, with short points. Spurs were the badge of knighthood.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>And on her feet a pair of <qex>spurs</qex> large.</q> <rj><qau>Chaucer.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><-- To earn one's spurs, win one's spurs, to earn recognition for achievement or for expertise. --><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>That which goads to action; an incitement.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Fame is the <qex>spur</qex> that the clear spirit doth raise<br/
(That last infirmity of noble mind)<br/
To scorn delights and live laborious days.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>Something that projects; a snag.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>One of the large or principal roots of a tree.</def>  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <fld>(Zool.)</fld> <def>Any stiff, sharp spine, as on the wings and legs of certain birds, on the legs of insects, etc.; especially, the spine on a cock's leg.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>6.</sn> <def>A mountain that shoots from any other mountain, or range of mountains, and extends to some distance in a lateral direction, or at right angles.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>7.</sn> <def>A spiked iron worn by seamen upon the bottom of the boot, to enable them to stand upon the carcass of a whale, to strip off the blubber.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>8.</sn> <fld>(Carp.)</fld> <def>A brace strengthening a post and some connected part, as a rafter or crossbeam; a strut.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>9.</sn> <fld>(Arch.)</fld> <sd>(a)</sd> <def>The short wooden buttress of a post.</def> <sd>(b)</sd> <def>A projection from the round base of a column, occupying the angle of a square plinth upon which the base rests, or bringing the bottom bed of the base to a nearly square form. It is generally carved in leafage.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>10.</sn> <fld>(Bot.)</fld> <sd>(a)</sd> <def>Any projecting appendage of a flower looking like a spur.</def> <au>Gray.</au> <sd>(b)</sd> <def>Ergotized rye or other grain.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><sn>11.</sn> <fld>(Fort.)</fld> <def>A wall that crosses a part of a rampart and joins to an inner wall.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>12.</sn> <fld>(Shipbuilding)</fld> <sd>(a)</sd> <def>A piece of timber fixed on the bilge ways before launching, having the upper ends bolted to the vessel's side.</def> <sd>(b)</sd> <def>A curved piece of timber serving as a half beam to support the deck where a whole beam can not be placed.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>13.</sn> <fld>(Mining)</fld> <def>A branch of a vein.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><sn>14.</sn>  <def>The track of an animal, as an otter; a spoor.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><cs><col><b>Spur fowl</b></col> <fld>(Zool.)</fld>, <cd>any one of several species of Asiatic gallinaceous birds of the genus <gen>Galloperdix</gen>, allied to the jungle fowl. The males have two or more spurs on each leg.</cd> -- <col><b>Spur gear</b></col> <fld>(Mach.)</fld>, <cd>a cogwheel having teeth which project radially and stand parallel to the axis; a spur wheel.</cd> -- <col><b>Spur gearing</b></col>, <cd>gearing in which spur gears are used. See under <er>Gearing</er>.</cd> -- <col><b>Spur pepper</b></col>. <fld>(Bot.)</fld> <cd>See the Note under <er>Capsicum</er>.</cd> -- <col><b>Spur wheel</b></col>. <cd>Same as <cref>Spur gear</cref>, above.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spur</ent><br/
<hw>Spur</hw>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Spurred</conjf> <pr>(sp<ucir/rd)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Spurring</conjf>.]</vmorph> <sn>1.</sn> <def>To prick with spurs; to incite to a more hasty pace; to urge or goad; <as>as, to <ex>spur</ex> a horse</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To urge or encourage to action, or to a more vigorous pursuit of an object; to incite; to stimulate; to instigate; to impel; to drive.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Love will not be <qex>spurred</qex> to what it loathes.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>To put spurs on; <as>as, a <ex>spurred</ex> boot</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spur</ent><br/
<hw>Spur</hw>, <pos>v. i.</pos> <def>To spur on one's horse; to travel with great expedition; to hasten; hence, to press forward in any pursuit.</def> <ldquo/Now <xex>spurs</xex> the lated traveler.<rdquo/  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>The Parthians shall be there,<br/
And, <qex>spurring</qex> from the fight, confess their fear.</q> <rj><qau>Dryden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>The roads leading to the capital were covered with multitudes of yeomen, <qex>spurring</qex> hard to Westminster.</q> <rj><qau>Macaulay.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Some bold men, . . . by <qex>spurring</qex> on, refine themselves.</q> <rj><qau>Grew.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spurgall</ent><br/
<hw>Spur"gall`</hw> <pr>(-g<add/l`)</pr>, <pos>n.</pos> <def>A place galled or excoriated by much using of the spur.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spurgall</ent><br/
<hw>Spur"gall`</hw>, <pos>v. t.</pos> <def>To gall or wound with a spur.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spurge</ent><br/
<hw>Spurge</hw> <pr>(sp<ucir/rj)</pr>, <pos>v. t.</pos> <ety>[Etymol. uncertain.]</ety> <def>To emit foam; to froth; -- said of the emission of yeast from beer in course of fermentation.</def> <mark>[Obs.]</mark>  <rj><au>W. Cartright.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spurge</ent><br/
<hw>Spurge</hw>, <pos>n.</pos> <ety>[OF. <ets>espurge</ets>, F. <ets><eacute/purge</ets>, from OF. <ets>espurgier</ets> to purge, L. <ets>expurgare</ets>. See <er>Expurgate</er>, <er>Purge</er>.]</ety> <fld>(Bot.)</fld> <def>Any plant of the genus Euphorbia. See <er>Euphorbia</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Spurge flax</b></col>, <cd>an evergreen shrub (<spn>Daphne Gnidium</spn>) with crowded narrow leaves. It is a native of Southern Europe.</cd> -- <col><b>Spurge laurel</b></col>, <cd>a European shrub (<spn>Daphne Laureola</spn>) with oblong evergreen leaves.</cd> -- <col><b>Spurge nettle</b></col>. <cd>See under <er>Nettle</er>.</cd> -- <col><b>Spurge olive</b></col>, <cd>an evergreen shrub (<spn>Daphne oleoides</spn>) found in the Mediterranean region.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spurgewort</ent><br/
<hw>Spurge"wort`</hw> <pr>(sp<ucir/rj"w<ucir/rt`)</pr>, <pos>n.</pos> <fld>(Bot.)</fld> <def>Any euphorbiaceous plant.</def>  <rj><au>Lindley.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spurging</ent><br/
<hw>Spur"ging</hw> <pr>(sp<ucir/r"j<icr/ng)</pr>, <pos>n.</pos> <ety>[See 2d <er>Spurge</er>.]</ety> <def>A purging.</def> <mark>[Obs.]</mark>  <rj><au>B. Jonson.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spurious</ent><br/
<hw>Spu"ri*ous</hw> <pr>(sp<umac/"r<icr/*<ucr/s)</pr>, <pos>a.</pos> <ety>[L. <ets>spurius</ets>.]</ety> <sn>1.</sn> <def>Not proceeding from the true source, or from the source pretended; not genuine; counterfeit; false; adulterate.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Not legitimate; bastard; <as>as, <ex>spurious</ex> issue</as>.</def> <ldquo/Her <xex>spurious</xex> firstborn.<rdquo/  <rj><au>Milton.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><mcol><col><b>Spurious primary</b></col>, <it>or</it> <col><b>Spurious quill</b></col></mcol> <fld>(Zool.)</fld>, <cd>the first, or outer, primary quill when rudimentary or much reduced in size, as in certain singing birds.</cd> -- <col><b>Spurious wing</b></col> <fld>(Zool.)</fld>, <cd>the bastard wing, or alula.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><syn><b>Syn.</b> -- Counterfeit; false; adulterate; supposititious; fictitious; bastard.</syn><br/
[<source>1913 Webster</source>]</p>

<p>-- <wordforms><wf>Spu"ri*ous*ly</wf>, <pos>adv.</pos> -- <wf>Spu"ri*ous*ness</wf>, <pos>n.</pos></wordforms><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spurless</ent><br/
<hw>Spur"less</hw> <pr>(sp<ucir/r"l<ecr/s)</pr>, <pos>a.</pos> <def>Having no spurs.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spurling</ent><br/
<hw>Spur"ling</hw> <pr>(-l<icr/ng)</pr>, <pos>n.</pos> <ety>[See <er>Sparling</er>.]</ety> <fld>(Zool.)</fld> <def>A tern.</def> <mark>[Obs. or Prov. Eng.]</mark>  <rj><au>Tusser.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spurling-line</ent><br/
<hw>Spur"ling-line`</hw> <pr>(-l<imac/n`)</pr>, <pos>n.</pos> <ety>[Cf. Prov. E. <ets>spurling</ets> the rut of a wheel, a cart rut, AS. <ets>spor</ets> a track, trace, E. <ets>spoor</ets>, Scot. <ets>spurl</ets> to sprawl.]</ety> <fld>(Naut.)</fld> <def>The line which forms the communication between the steering wheel and the telltale.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spurn</ent><br/
<hw>Spurn</hw> <pr>(sp<ucir/rn)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Spurned</conjf> <pr>(sp<ucir/rnd)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Spurning</conjf>.]</vmorph> <ety>[OE. <ets>spurnen</ets> to kick against, to stumble over, AS. <ets>spurnan</ets> to kick, offend; akin to <ets>spura</ets> spur, OS. & OHG. <ets>spurnan</ets> to kick, Icel. <ets>spyrna</ets>, L. <ets>spernere</ets> to despise, Skr. <ets>sphur</ets> to jerk, to push. <root/171. See <er>Spur</er>.]</ety> <sn>1.</sn> <def>To drive back or away, as with the foot; to kick.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>[The bird] with his foot will <qex>spurn</qex> adown his cup.</q> <rj><qau>Chaucer.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>I <qex>spurn</qex> thee like a cur out of my way.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To reject with disdain; to scorn to receive or accept; to treat with contempt.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>What safe and nicely I might well delay<br/
By rule of knighthood, I disdain and <qex>spurn</qex>.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Domestics will pay a more cheerful service when they find themselves not <qex>spurned</qex> because fortune has laid them at their master's feet.</q> <rj><qau>Locke.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spurn</ent><br/
<hw>Spurn</hw>, <pos>v. i.</pos> <sn>1.</sn> <def>To kick or toss up the heels.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The miller <qex>spurned</qex> at a stone.</q> <rj><qau>Chaucer.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>The drunken chairman in the kennel <qex>spurns</qex>.</q> <rj><qau>Gay.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To manifest disdain in rejecting anything; to make contemptuous opposition or resistance.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Nay, more, to <qex>spurn</qex> at your most royal image.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spurn</ent><br/
<hw>Spurn</hw>, <pos>n.</pos> <sn>1.</sn> <def>A kick; a blow with the foot.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>What defense can properly be used in such a despicable encounter as this but either the slap or the <qex>spurn</qex>?</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Disdainful rejection; contemptuous treatment.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The insolence of office, and the <qex>spurns</qex><br/
That patient merit of the unworthy takes.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <fld>(Mining)</fld> <def>A body of coal left to sustain an overhanging mass.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spurner</ent><br/
<hw>Spurn"er</hw> <pr>(-<etil/r)</pr>, <pos>n.</pos> <def>One who spurns.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spurn-water</ent><br/
<hw>Spurn"-wa`ter</hw> <pr>(-w<add/`t<etil/r)</pr>, <pos>n.</pos> <fld>(Naut.)</fld> <def>A channel at the end of a deck to restrain the water.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spurred</ent><br/
<hw>Spurred</hw> <pr>(sp<ucir/rd)</pr>, <pos>a.</pos> <sn>1.</sn> <def>Wearing spurs; furnished with a spur or spurs; having shoots like spurs.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Affected with spur, or ergot; <as>as, <ex>spurred</ex> rye</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Spurred corolla</b></col> <fld>(Bot.)</fld>, <cd>a corolla in which there are one or more petals with a spur.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spurrer</ent><br/
<hw>Spur"rer</hw> <pr>(sp<ucir/r"r<etil/r)</pr>, <pos>n.</pos> <def>One who spurs.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spurrey</ent><br/
<hw>Spur"rey</hw> <pr>(sp<ucr/r"r<ycr/)</pr>, <pos>n.</pos> <fld>(Bot.)</fld> <def>See <er>Spurry</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spurrier</ent><br/
<hw>Spur"ri*er</hw> <pr>(sp<ucir/r"r<icr/*<etil/r)</pr>, <pos>n.</pos> <def>One whose occupation is to make spurs.</def> <au>B. Jonson.</au> <ldquo/The saddlers and <xex>spurriers</xex> would be ruined by thousands.<rdquo/ <au>Macaulay.</au><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spur-royal</ent><br/
<hw>Spur"-roy`al</hw> <pr>(sp<ucir/r"roi`<ait/l)</pr>, <pos>n.</pos> <def>A gold coin, first made in the reign of Edward IV., having a star on the reverse resembling the rowel of a spur. In the reigns of Elizabeth and of James I., its value was fifteen shillings.</def> <altsp>[Written also <asp>spur-rial</asp>, and <asp>spur-ryal</asp>.]</altsp><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spurry</ent><br/
<hw>Spur"ry</hw> <pr>(sp<ucr/r"r<ycr/)</pr>, <pos>n.</pos> <ety>[D. or OF. <ets>spurrie</ets>; cf. G. <ets>spergel</ets>, NL. <ets>spergula</ets>.]</ety> <fld>(Bot.)</fld> <def>An annual herb (<spn>Spergula arvensis</spn>) with whorled filiform leaves, sometimes grown in Europe for fodder.</def> <altsp>[Written also <asp>spurrey</asp>.]</altsp><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Sand spurry</b></col> <fld>(Bot.)</fld>, <cd>any low herb of the genus <gen>Lepigonum</gen>, mostly found in sandy places.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spur-shell</ent><br/
<hw>Spur"-shell`</hw> <pr>(sp<ucir/r"sh<ecr/l`)</pr>, <pos>n.</pos> <fld>(Zool.)</fld> <def>Any one of several species of handsome marine gastropod shells of the genus <gen>Trochus</gen>, or <gen>Imperator</gen>.  The shell is conical, with the margin toothed somewhat like the rowel of a spur.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spurt</ent><br/
<hw>Spurt</hw> <pr>(sp<ucir/rt)</pr>, <pos>v. i.</pos> <ety>[Written also <ets>spirt</ets>, and originally the same word as <ets>sprit</ets>; OE. <ets>sprutten</ets> to sprout, AS. <ets>spryttan</ets>. See <er>Sprit</er>, <pos>v. i.</pos>, <er>Sprout</er>, <pos>v. i.</pos>]</ety> <def>To gush or issue suddenly or violently out in a stream, as liquor from a cask; to rush from a confined place in a small stream or jet; to spirt.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Thus the small jet, which hasty hands unlock,<br/
<qex>Spurts</qex> in the gardener's eyes who turns the cock.</q> <rj><qau>Pope.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spurt</ent><br/
<hw>Spurt</hw>, <pos>v. t.</pos> <def>To throw out, as a liquid, in a stream or jet; to drive or force out with violence, as a liquid from a pipe or small orifice; <as>as, to <ex>spurt</ex> water from the mouth</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spurt</ent><br/
<hw>Spurt</hw>, <pos>n.</pos> <sn>1.</sn> <def>A sudden or violent ejection or gushing of a liquid, as of water from a tube, orifice, or other confined place, or of blood from a wound; a jet; a spirt.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A shoot; a bud.</def> <mark>[Obs.]</mark>  <rj><au>Holland.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <mark>Fig.:</mark>  <def>A sudden outbreak; <as>as, a <ex>spurt</ex> of jealousy</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Spurt grass</b></col> <fld>(Bot.)</fld>, <cd>a rush fit for basket work.</cd>  <rj><au>Dr. Prior.</au></rj></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spurt</ent><br/
<hw>Spurt</hw> <pr>(sp<ucir/rt)</pr>, <pos>n.</pos> <ety>[Cf. Icel. <ets>sprettr</ets> a spurt, spring, run, <ets>spretta</ets> to spirt, spring.]</ety> <def>A sudden and energetic effort, as in an emergency; an increased exertion for a brief space.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The long, steady sweep of the so-called <ldquo/paddle<rdquo/ tried him almost as much as the breathless strain of the <qex>spurt</qex>.</q> <rj><qau>T. Hughes.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spurt</ent><br/
<hw>Spurt</hw>, <pos>v. i.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Spurted</conjf>; <pos>p. pr. & vb. n.</pos> <conjf>Spurting</conjf>.]</vmorph> <def>To make a sudden and violent exertion, as in an emergency.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spurtle</ent><br/
<hw>Spur"tle</hw> <pr>(sp<ucir/r"t'l)</pr>, <pos>v. t.</pos> <ety>[Freq. of <ets>spurt</ets>.]</ety> <def>To spurt or shoot in a scattering manner.</def> <mark>[Obs.]</mark>  <rj><au>Drayton.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spurway</ent><br/
<hw>Spur"way`</hw> <pr>(sp<ucir/r"w<amac/`)</pr>, <pos>n.</pos> <ety>[Prov. E. <ets>spoor</ets> a track, trace (AS. <ets>spor</ets>) + <ets>way</ets>.]</ety> <def>A bridle path.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spur-winged</ent><br/
<hw>Spur"-winged`</hw> <pr>(-w<icr/ngd`)</pr>, <pos>a.</pos> <fld>(Zool.)</fld> <def>Having one or more spurs on the bend of the wings.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Spur-winged goose</b></col> <fld>(Zool.)</fld>, <cd>any one of several species of long-legged African geese of the genus <gen>Plectropterus</gen> and allied genera, having a strong spur on the bend of the wing, as the Gambo goose (<spn>Plectropterus Gambensis</spn>) and the Egyptian, or Nile, goose (<spn>Alopochen Aegyptiaca</spn>).</cd> -- <col><b>Spur-winged plover</b></col> <fld>(Zool.)</fld>, <cd>an Old World plover (<spn>Hoplopterus spinosus</spn>) having a sharp spur on the bend of the wing.  It inhabits Northern Africa and the adjacent parts of Asia and Europe.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sput</ent><br/
<hw>Sput</hw> <pr>(sp<ucr/t)</pr>, <pos>n.</pos> <fld>(Steam Boiler)</fld> <def>An annular reenforce, to strengthen a place where a hole is made.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sputation</ent><br/
<hw>Spu*ta"tion</hw> <pr>(sp<usl/*t<amac/"sh<ucr/n)</pr>, <pos>n.</pos> <ety>[L. <ets>sputare</ets> to spit, v. intens. fr. <ets>spuere</ets> to spit: cf. F. <ets>sputation</ets>.]</ety> <def>The act of spitting; expectoration.</def>  <rj><au>Harvey.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sputative</ent><br/
<hw>Spu"ta*tive</hw> <pr>(sp<umac/"t<adot/*t<icr/v)</pr>, <pos>a.</pos> <def>Inclined to spit; spitting much.</def>  <rj><au>Sir H. Wotton.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spute</ent><br/
<hw>Spute</hw> <pr>(sp<umac/t)</pr>, <pos>v. t.</pos> <ety>[Abbrev. from <ets>dispute</ets>.]</ety> <def>To dispute; to discuss.</def> <mark>[Obs.]</mark>  <rj><au>Wyclif.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sputter</ent><br/
<hw>Sput"ter</hw> <pr>(sp<ucr/t"t<etil/r)</pr>, <pos>v. i.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Sputtered</conjf> <pr>(-t<etil/rd)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Sputtering</conjf>.]</vmorph> <ety>[From the root of <ets>spout</ets> or <ets>spit</ets> to eject from the mouth.  Cf. <er>Splutter</er>.]</ety> <sn>1.</sn> <def>To spit, or to emit saliva from the mouth in small, scattered portions, as in rapid speaking.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To utter words hastily and indistinctly; to speak so rapidly as to emit saliva.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>They could neither of them speak their rage, and so fell a <qex>sputtering</qex> at one another, like two roasting apples.</q> <rj><qau>Congreve.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>To throw out anything, as little jets of steam, with a noise like that made by one sputtering.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Like the green wood . . . <qex>sputtering</qex> in the flame.</q> <rj><qau>Dryden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sputter</ent><br/
<hw>Sput"ter</hw>, <pos>v. t.</pos> <def>To spit out hastily by quick, successive efforts, with a spluttering sound; to utter hastily and confusedly, without control over the organs of speech.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>In the midst of caresses, and without the least pretended incitement, to <qex>sputter</qex> out the basest accusations.</q> <rj><qau>Swift.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sputter</ent><br/
<hw>Sput"ter</hw>, <pos>n.</pos> <def>Moist matter thrown out in small detached particles; also, confused and hasty speech.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sputterer</ent><br/
<hw>Sput"ter*er</hw> <pr>(-<etil/r)</pr>, <pos>n.</pos> <def>One who sputters.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sputum</ent><br/
||<hw>Spu"tum</hw> <pr>(sp<umac/"t<ucr/m)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Sputa</plw> <pr>(-t<adot/)</pr>.</plu> <ety>[L., from <ets>spuere</ets>, <ets>sputum</ets>, to spit.]</ety> <def>That which is expectorated; a salival discharge; spittle; saliva.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spy</ent><br/
<hw>Spy</hw> <pr>(sp<imac/)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Spied</conjf> <pr>(sp<imac/d)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Spying</conjf>.]</vmorph> <ety>[OE. <ets>spien</ets>, <ets>espien</ets>, OF. <ets>espier</ets>, F. <ets><eacute/pier</ets>, OHG. <ets>speh<omac/n</ets>, G. <ets>sp<aum/hen</ets>; akin to L. <ets>specere</ets> to see, Skr. <ets>spa<cced/</ets>. <root/ 169.  Cf. <er>Espy</er>, <pos>v. t.</pos>, <er>Aspect</er>, <er>Auspice</er>, <er>Circumspect</er>, <er>Conspicuous</er>, <er>Despise</er>, <er>Frontispiece</er>, <er>Inspect</er>, <er>Prospect</er>, <er>Respite</er>, <er>Scope</er>, <er>Specimen</er>, <er>Spectacle</er>, <er>Specter</er>, <er>Speculate</er>, <er>Spice</er>, <er>Spite</er>, <er>Suspicion</er>.]</ety> <sn>1.</sn> <def>To gain sight of; to discover at a distance, or in a state of concealment; to espy; to see.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>One, in reading, skipped over all sentences where he <qex>spied</qex> a note of admiration.</q> <rj><qau>Swift.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To discover by close search or examination.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Look about with your eyes; <qex>spy</qex> what things are to be reformed in the church of England.</q> <rj><qau>Latimer.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>To explore; to view, inspect, and examine secretly, as a country; -- usually with <ptcl>out</ptcl>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Moses sent to <qex>spy</qex> out Jaazer, and they took the villages thereof.</q> <rj><qau>Num. xxi. 32.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spy</ent><br/
<hw>Spy</hw>, <pos>v. i.</pos> <def>To search narrowly; to scrutinize.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>It is my nature's plague<br/
To <qex>spy</qex> into abuses.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spy</ent><br/
<hw>Spy</hw>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Spies</plw> <pr>(sp<imac/z)</pr>.</plu> <ety>[See <er>Spy</er>, <pos>v.</pos>, and cf. <er>Espy</er>, <pos>n.</pos>]</ety> <sn>1.</sn> <def>One who keeps a constant watch of the conduct of others.</def> <ldquo/These wretched <xex>spies</xex> of wit.<rdquo/  <rj><au>Dryden.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Mil.)</fld> <def>A person sent secretly into an enemy's camp, territory, or fortifications, to inspect his works, ascertain his strength, movements, or designs, and to communicate such intelligence to the proper officer.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Spy money</b></col>, <cd>money paid to a spy; the reward for private or secret intelligence regarding the enemy.</cd> -- <col><b>Spy Wednesday</b></col> <fld>(Eccl.)</fld>, <cd>the Wednesday immediately preceding the festival of Easter; -- so called in allusion to the betrayal of Christ by Judas Iscariot.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><syn><b>Syn.</b> -- See <er>Emissary</er>, and <er>Scout</er>.</syn><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spyboat</ent><br/
<hw>Spy"boat`</hw> <pr>(-b<omac/t`)</pr>, <pos>n.</pos> <def>A boat sent to make discoveries and bring intelligence.</def>  <rj><au>Arbuthnot.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spyglass</ent><br/
<hw>Spy"glass`</hw> <pr>(-gl<adot/s`)</pr>, <pos>n.</pos> <def>A small telescope for viewing distant terrestrial objects.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spyism</ent><br/
<hw>Spy"ism</hw> <pr>(-<icr/z'm)</pr>, <pos>n.</pos> <def>Act or business of spying.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Spyne</ent><br/
<ent>Spynace</ent><br/
<mhw>{ <hw>Spy"nace</hw> <pr>(sp<imac/"n<asl/s; 48)</pr>, <hw>Spyne</hw> <pr>(sp<imac/n)</pr>, }</mhw> <pos>n.</pos> <fld>(Naut.)</fld> <def>See <er>Pinnace</er>, <pos>n.</pos>, 1 <sd>(a)</sd>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Squab</ent><br/
<hw>Squab</hw> <pr>(skw<ocr/b)</pr>, <pos>a.</pos> <ety>[Cf. dial. Sw. <ets>sqvabb</ets> a soft and fat body, <ets>sqvabba</ets> a fat woman, Icel. <ets>kvap</ets> jelly, jellylike things, and E. <ets>quab</ets>.]</ety> <sn>1.</sn> <def>Fat; thick; plump; bulky.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Nor the <qex>squab</qex> daughter nor the wife were nice.</q> <rj><qau>Betterton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Unfledged; unfeathered; <as>as, a <ex>squab</ex> pigeon</as>.</def>  <rj><au>King.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Squab</ent><br/
<hw>Squab</hw>, <pos>n.</pos> <sn>1.</sn> <fld>(Zool.)</fld> <def>A nestling of a pigeon or other similar bird, esp. when very fat and not fully fledged.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A person of a short, fat figure.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Gorgonious sits abdominous and wan,<br/
Like a fat <qex>squab</qex> upon a Chinese fan.</q> <rj><qau>Cowper.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>A thickly stuffed cushion; especially, one used for the seat of a sofa, couch, or chair; also, a sofa.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Punching the <qex>squab</qex> of chairs and sofas.</q> <rj><qau>Dickens.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>On her large <qex>squab</qex> you find her spread.</q> <rj><qau>Pope.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Squab</ent><br/
<hw>Squab</hw>, <pos>adv.</pos> <ety>[Cf. dial. Sw. <ets>sqvapp</ets>, a word imitative of a splash, and E. <ets>squab</ets> fat, unfledged.]</ety> <def>With a heavy fall; plump.</def> <mark>[Vulgar]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>The eagle took the tortoise up into the air, and dropped him down, <qex>squab</qex>, upon a rock.</q> <rj><qau>L'Estrange.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Squab</ent><br/
<hw>Squab</hw>, <pos>v. i.</pos> <def>To fall plump; to strike at one dash, or with a heavy stroke.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Squabash</ent><br/
<hw>Squa*bash"</hw> <pr>(skw<adot/*b<acr/sh")</pr>, <pos>v. t.</pos> <def>To crush; to quash; to squash.</def> <mark>[Colloq. or Slang, Scot.]</mark>  <rj><au>Sir W. Scott.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Squabbish</ent><br/
<hw>Squab"bish</hw> <pr>(skw<ocr/b"b<icr/sh)</pr>, <pos>a.</pos> <def>Thick; fat; heavy.</def><br/
[<source>1913 Webster</source>]</p>

<p><-- p. 1396 pr=VMG --></p>

<p><ent>Squabble</ent><br/
<hw>Squab"ble</hw> <pr>(skw<ocr/b"b'l)</pr>, <pos>v. i.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Squabbled</conjf> <pr>(-b'ld)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Squabbling</conjf> <pr>(-bl<icr/ng)</pr>.]</vmorph> <ety>[Cf. dial. Sw. <ets>skvabbel</ets> a dispute, <ets>skvappa</ets> to chide.]</ety> <sn>1.</sn> <def>To contend for superiority in an unseemly manner; to scuffle; to struggle; to wrangle; to quarrel.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To debate peevishly; to dispute.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The sense of these propositions is very plain, though logicians might <qex>squabble</qex> a whole day whether they should rank them under negative or affirmative.</q> <rj><qau>I. Watts.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><syn><b>Syn.</b> -- To dispute; contend; scuffle; wrangle; quarrel; struggle.</syn><br/
[<source>1913 Webster</source>]</p>

<p><ent>Squabble</ent><br/
<hw>Squab"ble</hw>, <pos>v. t.</pos> <fld>(Print.)</fld> <def>To disarrange, so that the letters or lines stand awry or are mixed and need careful readjustment; -- said of type that has been set up.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Squabble</ent><br/
<hw>Squab"ble</hw>, <pos>n.</pos> <def>A scuffle; a wrangle; a brawl.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Squabbler</ent><br/
<hw>Squab"bler</hw> <pr>(-bl<etil/r)</pr>, <pos>n.</pos> <def>One who squabbles; a contentious person; a brawler.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Squabby</ent><br/
<hw>Squab"by</hw> <pr>(-b<ycr/)</pr>, <pos>a.</pos> <def>Short and thick; sqabbish.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Squab-chick</ent><br/
<hw>Squab"-chick`</hw> <pr>(-ch<icr/k`)</pr>, <pos>n.</pos> <fld>(Zool.)</fld> <def>A young chicken before it is fully fledged.</def> <mark>[Prov. Eng.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Squacco</ent><br/
<hw>Squac"co</hw> <pr>(skw<acr/k"k<osl/)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Squaccos</plw> <pr>(-k<omac/z)</pr>.</plu> <fld>(Zool.)</fld> <def>A heron (<spn>Ardea comata</spn>) found in Asia, Northern Africa, and Southern Europe.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Squad</ent><br/
<hw>Squad</hw> <pr>(skw<ocr/d)</pr>, <pos>n.</pos> <ety>[F. <ets>escouade</ets>, fr. Sp. <ets>escuadra</ets>, or It. <ets>squadra</ets>, (assumed) LL. <ets>exquadrare</ets> to square; L. <ets>ex + quadra</ets> a square. See <er>Square</er>.]</ety> <sn>1.</sn> <fld>(Mil.)</fld> <def>A small party of men assembled for drill, inspection, or other purposes.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Hence, any small party.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Squad</ent><br/
<hw>Squad</hw>, <pos>n.</pos> <def>Sloppy mud.</def> <mark>[Prov. Eng.]</mark>  <rj><au>Tennyson.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Squadron</ent><br/
<hw>Squad"ron</hw> <pr>(skw<ocr/d"r<ucr/n)</pr>, <pos>n.</pos> <ety>[F. <ets>escadron</ets>, formerly also <ets>esquadron</ets>, Sp. <ets>escuadron</ets>, or It. <ets>squadrone</ets>. See <er>Squad</er>.]</ety> <sn>1.</sn> <def>Primarily, a square; hence, a square body of troops; a body of troops drawn up in a square.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>Those half-rounding quards<br/
Just met, and, closing, stood in <qex>squadron</qex> joined.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Mil.)</fld> <def>A body of cavalry comprising two companies or troops, and averaging from one hundred and twenty to two hundred men.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <fld>(Naut.)</fld> <def>A detachment of vessels employed on any particular service or station, under the command of the senior officer; <as>as, the North Atlantic <ex>Squadron</ex></as>.</def>  <rj><au>Totten.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Flying squadron</b></col>, <cd>a squadron of observation or practice, that cruises rapidly about from place to place.</cd></cs>
 <rj><qau>Ham. Nav. Encyc.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Squadroned</ent><br/
<hw>Squad"roned</hw> <pr>(skw<ocr/d"r<ucr/nd)</pr>, <pos>a.</pos> <def>Formed into squadrons, or squares.</def> <mark>[R.]</mark>  <rj><au>Milton.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Squail</ent><br/
<hw>Squail</hw> <pr>(skw<amac/l)</pr>, <pos>v. i.</pos> <def>To throw sticks at cocks; to throw anything about awkwardly or irregularly.</def> <mark>[Prov. Eng.]</mark>  <rj><au>Southey.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Squaimous</ent><br/
<hw>Squai"mous</hw> <pr>(skw<amac/"m<ucr/s)</pr>, <pos>a.</pos> <def>Squeamish.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Squali</ent><br/
||<hw>Squa"li</hw> <pr>(skw<amac/"l<imac/)</pr>, <pos>n. pl.</pos> <ety>[NL., fr. L. <ets>squalus</ets> a kind of sea fish.]</ety> <fld>(Zool.)</fld> <def>The suborder of elasmobranch fishes which comprises the sharks.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Squalid</ent><br/
<hw>Squal"id</hw> <pr>(skw<ocr/l"<icr/d)</pr>, <pos>a.</pos> <ety>[L. <ets>squalidus</ets>, fr. <ets>squalere</ets> to be foul or filthy.]</ety> <def>Dirty through neglect; foul; filthy; extremely dirty.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Uncombed his locks, and <qex>squalid</qex> his attire.</q>  <rj><qau>Dryden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Those <qex>squalid</qex> dens, which are the reproach of large capitals.</q> <rj><qau>Macaulay.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Squalidity</ent><br/
<hw>Squa*lid"i*ty</hw> <pr>(skw<adot/*l<icr/d"<icr/*t<ycr/)</pr>, <pos>n.</pos> <ety>[L. <ets>squaliditas</ets>.]</ety> <def>The quality or state of being squalid; foulness; filthiness.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Squalidly</ent><br/
<hw>Squal"id*ly</hw> <pr>(skw<ocr/l"<icr/d*l<ycr/)</pr>, <pos>adv.</pos> <def>In a squalid manner.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Squalidness</ent><br/
<hw>Squal"id*ness</hw>, <pos>n.</pos> <def>Quality or state of being squalid.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Squall</ent><br/
<hw>Squall</hw> <pr>(skw<add/l)</pr>, <pos>n.</pos> <ety>[Cf. Sw. <ets>sqval</ets> an impetuous running of water, <ets>sqval</ets>regn a violent shower of rain, <ets>sqvala</ets> to stream, to gush.]</ety> <def>A sudden and violent gust of wind often attended with rain or snow.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The gray skirts of a lifting <qex>squall</qex>.</q> <rj><qau>Tennyson.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Black squall</b></col>, <cd>a squall attended with dark, heavy clouds.</cd> -- <col><b>Thick squall</b></col>, <cd>a black squall accompanied by rain, hail, sleet, or snow.</cd> <au>Totten.</au> -- <col><b>White squall</b></col>, <cd>a squall which comes unexpectedly, without being marked in its approach by the clouds.</cd>  <rj><au>Totten.</au></rj></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Squall</ent><br/
<hw>Squall</hw>, <pos>v. i.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Squalled</conjf> <pr>(skw<add/ld)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Squalling</conjf>.]</vmorph> <ety>[Icel. <ets>skvala</ets>.  Cf. <er>Squeal</er>.]</ety> <def>To cry out; to scream or cry violently, as a woman frightened, or a child in anger or distress; <as>as, the infant <ex>squalled</ex></as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Squall</ent><br/
<hw>Squall</hw>, <pos>n.</pos> <def>A loud scream; a harsh cry.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>There oft are heard the notes of infant woe, --<br/
The short, thick sob, loud scream, and shriller <qex>squall</qex>.</q> <rj><qau>Pope.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Squaller</ent><br/
<hw>Squall"er</hw> <pr>(-<etil/r)</pr>, <pos>n.</pos> <def>One who squalls; a screamer.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Squally</ent><br/
<hw>Squall"y</hw> <pr>(-<ycr/)</pr>, <pos>a.</pos> <sn>1.</sn> <def>Abounding with squalls; disturbed often with sudden and violent gusts of wind; gusty; <as>as, <ex>squally</ex> weather</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Agric.)</fld> <def>Interrupted by unproductive spots; -- said of a field of turnips or grain.</def> <mark>[Prov. Eng.]</mark>  <rj><au>Halliwell.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <fld>(Weaving)</fld> <def>Not equally good throughout; not uniform; uneven; faulty; -- said of cloth.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Squalodon</ent><br/
<hw>Squa"lo*don</hw> <pr>(skw<amac/"l<osl/*d<ocr/n)</pr>, <pos>n.</pos> <ety>[NL. <ets>Squalus</ets> a genus of sharks + Gr. <grk>'odoy`s</grk>, <grk>'odo`ntos</grk>, a tooth.]</ety> <fld>(Paleon.)</fld> <def>A genus of fossil whales belonging to the Phocodontia; -- so called because their teeth are serrated, like a shark's.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Squalodont</ent><br/
<hw>Squa"lo*dont</hw> <pr>(-d<ocr/nt)</pr>, <pos>a.</pos> <fld>(Zool.)</fld> <def>Pertaining to Squalodon.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Squaloid</ent><br/
<hw>Squa"loid</hw> <pr>(skw<amac/"loid)</pr>, <pos>a.</pos> <ety>[NL. <ets>Squalus</ets> a genus of sharks (fr. L. <ets>squalus</ets> a kind of sea fish) + <ets>-oid</ets>.]</ety> <fld>(Zool.)</fld> <def>Like or pertaining to a shark or sharks.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Squalor</ent><br/
<hw>Squa"lor</hw> <pr>(skw<amac/"l<ocir/r)</pr>, <pos>n.</pos> <ety>[L., fr. <ets>squalere</ets> to be foul or filthy.]</ety> <def>Squalidness; foulness; filthiness; squalidity.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The heterogeneous indigent multitude, everywhere wearing nearly the same aspect of <qex>squalor</qex>.</q> <rj><qau>I. Taylor.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>To bring this sort of <qex>squalor</qex> among the upper classes.</q> <rj><qau>Dickens.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Squam</ent><br/
<hw>Squam</hw> <pr>(skw<ocr/m)</pr>, <pos>n.</pos> <ety>[From <ets>Squam</ets>, shortened fr. <etsep>Annisquam</etsep>, name of a village on the coast of Massachusetts.]</ety> <def>An oilskin hat or southwester; -- a fisherman's name.</def> <mark>[U. S.]</mark><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Squama</ent><br/
||<hw>Squa"ma</hw> <pr>(skw<amac/"m<adot/)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Squamae</plw> <pr>(-m<emac/)</pr>.</plu> <ety>[L. a scale.]</ety> <fld>(Med.)</fld> <def>A scale cast off from the skin; a thin dry shred consisting of epithelium.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Squamaceous</ent><br/
<hw>Squa*ma"ceous</hw> <pr>(skw<adot/*m<amac/"sh<ucr/s)</pr>, <pos>a.</pos> <def>Squamose.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Squamata</ent><br/
||<hw>Squa*ma"ta</hw> <pr>(skw<adot/*m<amac/"t<adot/)</pr>, <pos>n. pl.</pos> <ety>[NL., fr. L. <ets>squamatus</ets> scaly.]</ety> <fld>(Zool.)</fld> <def>A division of edentates having the body covered with large, imbricated horny scales. It includes the pangolins.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Squamated</ent><br/
<ent>Squamate</ent><br/
<mhw>{ <hw>Squa"mate</hw> <pr>(skw<amac/"m<asl/t)</pr>, <hw>Squa"ma*ted</hw> <pr>(-m<asl/*t<ecr/d)</pr>, }</mhw> <pos>a.</pos> <ety>[L. <ets>squamatus</ets>.]</ety> <def>Same as <er>Squamose</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Squam duck</ent><br/
<hw>Squam" duck`</hw> <pr>(skw<ocr/m" d<ucr/k`)</pr>. <fld>(Zool.)</fld> <def>The American eider duck.</def> <mark>[Local, U.S.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Squame</ent><br/
<hw>Squame</hw> <pr>(skw<amac/m)</pr>, <pos>n.</pos> <ety>[L. <ets>squama</ets> scale.]</ety> <sn>1.</sn> <def>A scale.</def> <mark>[Obs.]</mark>  <ldquo/Iron <xex>squames</xex>.<rdquo/  <rj><au>Chaucer.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Zool.)</fld> <def>The scale, or exopodite, of an antenna of a crustacean.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Squamella</ent><br/
||<hw>Squa*mel"la</hw> <pr>(skw<adot/*m<ecr/l"l<adot/)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Squamellae</plw> <pr>(-l<emac/)</pr>.</plu> <ety>[NL., dim. fr. L. <ets>squama</ets> a scale.]</ety> <fld>(Bot.)</fld> <def>A diminutive scale or bractlet, such as those found on the receptacle in many composite plants; a palea.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Squamellate</ent><br/
<hw>Squa*mel"late</hw> <pr>(-l<asl/t)</pr>, <pos>a.</pos> <def>Furnished or covered with little scales; squamulose.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Squamiform</ent><br/
<hw>Squa"mi*form</hw> <pr>(skw<amac/"m<icr/*f<ocir/rm)</pr>, <pos>a.</pos> <ety>[L. <ets>squama</ets> a scale + <ets>-form</ets>.]</ety> <def>Having the shape of a scale.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Squamigerous</ent><br/
<hw>Squa*mig"er*ous</hw> <pr>(skw<adot/*m<icr/j"<etil/r*<ucr/s)</pr>, <pos>a.</pos> <ety>[L. <ets>squamiger</ets>; <ets>squama</ets> a scale + <ets>gerere</ets> to bear.]</ety> <fld>(Zool.)</fld> <def>Bearing scales.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Squamipen</ent><br/
<hw>Squam"i*pen</hw> <pr>(skw<acr/m"<icr/*p<ecr/n <or/ skw<amac/"m<icr/-)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Squamipennes</plw> <pr>(-p<ecr/n"n<emac/z)</pr>.</plu> <ety>[L. <ets>squama</ets> a scale + <ets>penna</ets> a fin: cf. F. <ets>squamipenne</ets>.]</ety> <fld>(Zool.)</fld> <def>Any one of a group of fishes having the dorsal and anal fins partially covered with scales.</def><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ They are compressed and mostly, bright-colored tropical fishes, belonging to <gen>Chaetodon</gen> and allied genera.  Many of them are called <stype>coral fishes</stype>, and <stype>angel fishes</stype>.</note><br/
[<source>1913 Webster</source>]</p>

<p><ent>Squamoid</ent><br/
<hw>Squa"moid</hw> <pr>(skw<amac/"moid)</pr>, <pos>a.</pos> <ety>[L. <ets>squama</ets> scale + <ets>-oid</ets>.]</ety> <def>Resembling a scale; also, covered with scales; scaly.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Squamosal</ent><br/
<hw>Squa*mo"sal</hw> <pr>(skw<adot/*m<omac/"s<ait/l)</pr>, <pos>a.</pos> <fld>(Anat.)</fld> <sd>(a)</sd> <def>Scalelike; squamous; <as>as, the <ex>squamosal</ex> bone</as>.</def> <sd>(b)</sd> <def>Of or pertaining to the squamosal bone.</def> -- <def2><pos>n.</pos> <def>The squamous part of the temporal bone, or a bone corresponding to it, in many of the lower vertebrates. See <ex>Temporal bone</ex>, under <er>Temporal</er>.</def></def2><br/
[<source>1913 Webster</source>]</p>

<p><ent>Squamous</ent><br/
<ent>Squamose</ent><br/
<mhw>{ <hw>Squa*mose"</hw> <pr>(skw<adot/*m<omac/s" <or/ skw<amac/"m<omac/s`)</pr>, <hw>Squa"mous</hw> <pr>(skw<amac/"m<ucr/s)</pr>, }</mhw> <pos>a.</pos> <ety>[L. <ets>squamosus</ets>, fr. <ets>squama</ets> a scale: cf. F. <ets>squameux</ets>.]</ety> <sn>1.</sn> <def>Covered with, or consisting of, scales; resembling a scale; scaly; <as>as, the <ex>squamose</ex> cones of the pine; <ex>squamous</ex> epithelial cells; the <ex>squamous</ex> portion of the temporal bone, which is so called from a fancied resemblance to a scale.</as></def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Anat.)</fld> <def>Of or pertaining to the squamosal bone; squamosal.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Squamozygomatic</ent><br/
<hw>Squa`mo*zyg`o*mat"ic</hw> <pr>(skw<amac/`m<osl/*z<icr/g`<osl/*m<acr/t"<icr/k)</pr>, <pos>a.</pos> <fld>(Anat.)</fld> <def>Of or pertaining to both the squamosal and zygomatic bones; -- applied to a bone, or a center of ossification, in some fetal skulls.</def> -- <def2><pos>n.</pos> <def>A squamozygomatic bone.</def></def2><br/
[<source>1913 Webster</source>]</p>

<p><ent>Squamula</ent><br/
||<hw>Squam"u*la</hw> <pr>(skw<acr/m"<usl/*l<adot/ <or/ skw<amac/"m<usl/*l<adot/)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Squamulae</plw> <pr>(-l<emac/)</pr>.</plu> <ety>[L., dim. of <ets>squama</ets> a scale.]</ety> <fld>(Bot.)</fld> <def>One of the little hypogynous scales found in the flowers of grasses; a lodicule.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Squamulate</ent><br/
<hw>Squam"u*late</hw> <pr>(-l<asl/t)</pr>, <pos>a.</pos> <def>Same as <er>Squamulose</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Squamule</ent><br/
<hw>Squam"ule</hw> <pr>(-<usl/l)</pr>, <pos>n.</pos> <fld>(Bot.)</fld> <def>Same as <er>Squamula</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Squamulose</ent><br/
<hw>Squam"u*lose`</hw> <pr>(-l<omac/s`; 277)</pr>, <pos>a.</pos> <def>Having little scales; squamellate; squamulate.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Squander</ent><br/
<hw>Squan"der</hw> <pr>(skw<ocr/n"d<etil/r)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Squandered</conjf> <pr>(-d<etil/rd)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Squandering</conjf>.]</vmorph> <ety>[Cf. Scot. <ets>squatter</ets> to splash water about, to scatter, to squander, Prov. E. <ets>swatter</ets>, Dan. <ets>sqvatte</ets>, Sw. <ets>sqv<aum/tta</ets> to squirt, <ets>sqv<aum/ttra</ets> to squander, Icel. <ets>skvetta</ets> to squirt out, to throw out water.]</ety> <sn>1.</sn> <def>To scatter; to disperse.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>Our <qex>squandered</qex> troops he rallies.</q> <rj><qau>Dryden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To spend lavishly or profusely; to spend prodigally or wastefully; to use without economy or judgment; to dissipate; <as>as, to <ex>squander</ex> an estate</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The crime of <qex>squandering</qex> health is equal to the folly.</q> <rj><qau>Rambler.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><syn><b>Syn.</b> -- To spend; expend; waste; scatter; dissipate.</syn><br/
[<source>1913 Webster</source>]</p>

<p><ent>Squander</ent><br/
<hw>Squan"der</hw>, <pos>v. i.</pos> <sn>1.</sn> <def>To spend lavishly; to be wasteful.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>They often <qex>squandered</qex>, but they never gave.</q> <rj><qau>Savage.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To wander at random; to scatter.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>The wise man's folly is anatomized<br/
Even by <qex>squandering</qex> glances of the fool.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Squander</ent><br/
<hw>Squan"der</hw>, <pos>n.</pos> <def>The act of squandering; waste.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Squanderer</ent><br/
<hw>Squan"der*er</hw> <pr>(-<etil/r)</pr>, <pos>n.</pos> <def>One who squanders.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Squanderingly</ent><br/
<hw>Squan"der*ing*ly</hw>, <pos>adv.</pos> <def>In a squandering manner.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Square</ent><br/
<hw>Square</hw> <pr>(skw<acir/r)</pr>, <pos>n.</pos> <ety>[OF. <ets>esquarre</ets>, <ets>esquierre</ets>, F. <ets><eacute/querre</ets> a carpenter's square (cf. It. <ets>squadra</ets>), fr. (assumed) LL. <ets>exquadrare</ets> to make square; L. <ets>ex + quadrus</ets> a square, fr. <ets>quattuor</ets> four. See <er>Four</er>, and cf. <er>Quadrant</er>, <er>Squad</er>, <er>Squire</er> a square.]</ety> <sn>1.</sn> <fld>(Geom.)</fld> <sd>(a)</sd> <def>The corner, or angle, of a figure.</def> <mark>[Obs.]</mark> <sd>(b)</sd> <def>A parallelogram having four equal sides and four right angles.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Hence, anything which is square, or nearly so</def>; as: <sd>(a)</sd> <def>A square piece or fragment.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>He bolted his food down his capacious throat in <qex>squares</qex> of three inches.</q> <rj><qau>Sir W. Scott.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sd>(b)</sd> <def>A pane of glass.</def> <sd>(c)</sd> <fld>(Print.)</fld> <def>A certain number of lines, forming a portion of a column, nearly square; -- used chiefly in reckoning the prices of advertisements in newspapers.</def> <sd>(d)</sd> <fld>(Carp.)</fld> <def>One hundred superficial feet.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>An area of four sides, generally with houses on each side; sometimes, a solid block of houses; also, an open place or area for public use, as at the meeting or intersection of two or more streets.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The statue of Alexander VII. stands in the large <qex>square</qex> of the town.</q> <rj><qau>Addison.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <fld>(Mech. & Joinery)</fld> <def>An instrument having at least one right angle and two or more straight edges, used to lay out or test square work. It is of several forms, as the T square, the carpenter's square, the try-square., etc.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <def>Hence, a pattern or rule.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><sn>6.</sn> <fld>(Arith. & Alg.)</fld> <def>The product of a number or quantity multiplied by itself; <as>thus, 64 is the <ex>square</ex> of 8, for <mathex>8 <times/ 8 = 64</mathex>; the <ex>square</ex> of <mathex>a + b</mathex> is <mathex>a<exp>2</exp> + 2ab + b<exp>2</exp></mathex></as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>7.</sn> <def>Exact proportion; justness of workmanship and conduct; regularity; rule.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>They of Galatia [were] much more out of <qex>square</qex>.</q> <rj><qau>Hooker.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>I have not kept my <qex>square</qex>.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>8.</sn> <fld>(Mil.)</fld> <def>A body of troops formed in a square, esp. one formed to resist a charge of cavalry; a squadron.</def> <ldquo/The brave <xex>squares</xex> of war.<rdquo/  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>9.</sn> <def>Fig.: The relation of harmony, or exact agreement; equality; level.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>We live not on the <qex>square</qex> with such as these.</q> <rj><qau>Dryden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>10.</sn> <fld>(Astrol.)</fld> <def>The position of planets distant ninety degrees from each other; a quadrate.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><sn>11.</sn> <def>The act of squaring, or quarreling; a quarrel.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><sn>12.</sn> <def>The front of a woman's dress over the bosom, usually worked or embroidered.</def> <mark>[Obs.]</mark>  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>fair and square</b></col> <cd>in a fair, straightforward,  and honest manner; justly; <as>as, he beat me <ex>fair and square</ex></as>.</cd> -- <col><b>Geometrical square</b></col>. <cd>See <er>Quadrat</er>, <pos>n.</pos>, 2.</cd> -- <col><b>Hollow square</b></col> <fld>(Mil.)</fld>, <cd>a formation of troops in the shape of a square, each side consisting of four or five ranks, and the colors, officers, horses, etc., occupying the middle.</cd> -- <mcol><col><b>Least square</b></col>, <col><b>Magic square</b></col></mcol>, <cd>etc. See under <er>Least</er>, <er>Magic</er>, etc.</cd> -- <mcol><col><b>On the square</b></col>, <it>or</it> <col><b>Upon the square</b></col></mcol>, <sd>(a)</sd> <cd>in an open, fair manner; honestly, or upon honor; justly.</cd>  <mark>[Obs or Colloq.]</mark>  <sd>(b)</sd> <cd>at right angles.</cd> -- <mcol><col><b>On the square with</b></col>, <it>or</it> <col><b>Upon the square with</b></col></mcol>, <cd>upon equality with; even with.</cd> <au>Nares.</au> -- <col><b>To be all squares</b></col>, <cd>to be all settled.</cd> <mark>[Colloq.]</mark> <au>Dickens.</au> -- <col><b>To be at square</b></col>, <cd>to be in a state of quarreling.</cd> <mark>[Obs.]</mark> <au>Nares.</au> -- <col><b>To break no squares</b></col>, <cd>to give no offense; to make no difference.</cd> <mark>[Obs.]</mark> -- <col><b>To break squares</b></col>, <cd>to depart from an accustomed order.</cd> <mark>[Obs.]</mark> -- <col><b>To see how the squares go</b></col>, <cd>to see how the game proceeds; -- a phrase taken from the game of chess, the chessboard being formed with squares.</cd> <mark>[Obs.]</mark> <au>L'Estrange.</au></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Square</ent><br/
<hw>Square</hw> <pr>(skw<acir/r)</pr>, <pos>a.</pos> <sn>1.</sn> <fld>(Geom.)</fld> <def>Having four equal sides and four right angles; <as>as, a <ex>square</ex> figure</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Forming a right angle; <as>as, a <ex>square</ex> corner</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>Having a shape broad for the height, with rectilineal and angular rather than curving outlines; <as>as, a man of a <ex>square</ex> frame</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>Exactly suitable or correspondent; true; just.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>She's a most triumphant lady, if report be <qex>square</qex> to her.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <def>Rendering equal justice; exact; fair; honest; <as>as, <ex>square</ex> dealing</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>6.</sn> <def>Even; leaving no balance; <as>as, to make or leave the accounts <ex>square</ex></as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>7.</sn> <def>Leaving nothing; hearty; vigorous.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>By Heaven, <qex>square</qex> eaters.<br/
More meat, I say.</q> <rj><qau>Beau. & Fl.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>8.</sn> <fld>(Naut.)</fld> <def>At right angles with the mast or the keel, and parallel to the horizon; -- said of the yards of a square-rigged vessel when they are so braced.</def><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ <ex>Square</ex> is often used in self-explaining compounds or combinations, as in <ex>square</ex>-built, <ex>square</ex>-cornered, <ex>square</ex>-cut, <ex>square</ex>-nosed, etc.</note><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Square foot</b></col>, <cd>an area equal to that of a square the sides of which are twelve inches; 144 square inches.</cd> -- <col><b>Square knot</b></col>, <cd>a knot in which the terminal and standing parts are parallel to each other; a reef knot. See <xex>Illust.</xex> under <er>Knot</er>.</cd> -- <col><b>Square measure</b></col>, <cd>the measure of a superficies or surface which depends on the length and breadth taken conjointly. The units of square measure are squares whose sides are the linear measures; as, <xex>square</xex> inches, <xex>square</xex> feet, <xex>square</xex> meters, etc.</cd> -- <col><b>Square number</b></col>. <cd>See <er>Square</er>, <pos>n.</pos>, 6.</cd> -- <mcol><col><b>Square root of a number</b></col> <it>or</it> <col><b>Square root of a quantity</b></col></mcol> <fld>(Math.)</fld>, <cd>that number or quantity which, multiplied by itself, produces the given number or quantity.</cd> -- <col><b>Square sail</b></col> <fld>(Naut.)</fld>, <cd>a four-sided sail extended upon a yard suspended by the middle; sometimes, the foresail of a schooner set upon a yard; also, a cutter's or sloop's sail boomed out. See <xex>Illust.</xex> of <er>Sail</er>.</cd> -- <col><b>Square stern</b></col> <fld>(Naut.)</fld>, <cd>a stern having a transom and joining the counter timbers at an angle, as distinguished from a <xex>round stern</xex>, which has no transom.</cd> -- <mcol><col><b>Three-square</b></col>, <col><b>Five-square</b></col>, etc.</mcol>, <cd>having three, five, etc., equal sides; as, a <xex>three-square</xex> file.</cd> -- <col><b>To get square with</b></col>, <cd>to get even with; to pay off.</cd> <mark>[Colloq.]</mark></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Square</ent><br/
<hw>Square</hw>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Squared</conjf> <pr>(skw<acir/rd)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Squaring</conjf>.]</vmorph> <ety>[Cf. OF. <ets>escarrer</ets>, <ets>esquarrer</ets>. See <er>Square</er>, <pos>n.</pos>]</ety> <sn>1.</sn> <def>To form with four equal sides and four right angles.</def>  <rj><au>Spenser.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To form with right angles and straight lines, or flat surfaces; <as>as, to <ex>square</ex> masons' work</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>To compare with, or reduce to, any given measure or standard.</def>  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>To adjust; to regulate; to mold; to shape; to fit; <as>as, to <ex>square</ex> our actions by the opinions of others</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q><qex>Square</qex> my trial<br/
To my proportioned strength.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <def>To make even, so as to leave no remainder or difference; to balance; <as>as, to <ex>square</ex> accounts</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>6.</sn> <fld>(Math.)</fld> <def>To multiply by itself; <as>as, to <ex>square</ex> a number or a quantity</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>7.</sn> <fld>(Astrol.)</fld> <def>To hold a quartile position respecting.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The icy Goat and Crab that <qex>square</qex> the Scales.</q> <rj><qau>Creech.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>8.</sn> <fld>(Naut.)</fld> <def>To place at right angles with the keel; <as>as, to <ex>square</ex> the yards</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>To square one's shoulders</b></col>, <cd>to raise the shoulders so as to give them a square appearance, -- a movement expressing contempt or dislike.</cd> <au>Sir W. Scott.</au> -- <col><b>To square the circle</b></col> <fld>(Math.)</fld>, <cd>to determine the exact contents of a circle in square measure. The solution of this famous problem is now generally admitted to be impossible.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Square</ent><br/
<hw>Square</hw>, <pos>v. i.</pos> <sn>1.</sn> <def>To accord or agree exactly; to be consistent with; to conform or agree; to suit; to fit.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>No works shall find acceptance . . .<br/
That <qex>square</qex> not truly with the Scripture plan.</q> <rj><qau>Cowper.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To go to opposite sides; to take an attitude of offense or defense, or of defiance; to quarrel.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>Are you such fools<br/
To <qex>square</qex> for this?</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>To take a boxing attitude; -- often with <xex>up</xex>, sometimes with <xex>off</xex>.</def> <mark>[Colloq.]</mark>  <rj><au>Dickens.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Squarely</ent><br/
<hw>Square"ly</hw>, <pos>adv.</pos> <def>In a square form or manner.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Squareness</ent><br/
<hw>Square"ness</hw>, <pos>n.</pos> <def>The quality of being square; <as>as, an instrument to try the <ex>squareness</ex> of work</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Squarer</ent><br/
<hw>Squar"er</hw> <pr>(skw<acir/r"<etil/r)</pr>, <pos>n.</pos> <sn>1.</sn> <def>One who, or that which, squares.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>One who squares, or quarrels; a hot-headed, contentious fellow.</def> <mark>[Obs.]</mark>  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><-- p. 1397 pr=VMG --></p>

<p><ent>Square-rigged</ent><br/
<hw>Square"-rigged`</hw> <pr>(skw<acir/r"r<icr/gd`)</pr>, <pos>a.</pos> <fld>(Naut.)</fld> <def>Having the sails extended upon yards suspended horizontally by the middle, as distinguished from fore-and-aft sails; thus, a ship and a brig are <xex>square-rigged</xex> vessels.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Square-toed</ent><br/
<hw>Square"-toed`</hw> <pr>(-t<omac/d`)</pr>, <pos>a.</pos> <def>Having the toe square.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Obsolete as fardingales, ruffs, and <qex>square-toed</qex> shoes.</q> <rj><qau>V. Knox.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Square-toes</ent><br/
<hw>Square"-toes`</hw> <pr>(-t<omac/z`)</pr>, <pos>n.</pos> <def>A precise person; -- used contemptuously or jocularly.</def>  <rj><au>Thackeray.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Squarish</ent><br/
<hw>Squar"ish</hw>, <pos>a.</pos> <def>Nearly square.</def>  <rj><au>Pennant.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Squarrose</ent><br/
<hw>Squar*rose"</hw> <pr>(skw<ocr/r*r<ocr/s" <or/ skw<ocr/r"r<omac/s`; 277)</pr>, <pos>a.</pos> <ety>[L. <ets>squarrosus</ets> (perhaps) scurfy, scabby.]</ety> <def>Ragged or full of loose scales or projecting parts; rough; jagged</def>; as: <sd>(a)</sd> <fld>(Bot. & Zool.)</fld> <def>Consisting of scales widely divaricating; having scales, small leaves, or other bodies, spreading widely from the axis on which they are crowded; -- said of a calyx or stem.</def> <sd>(b)</sd> <fld>(Bot.)</fld> <def>Divided into shreds or jags, raised above the plane of the leaf, and not parallel to it; -- said of a leaf.</def> <sd>(c)</sd> <fld>(Zool.)</fld> <def>Having scales spreading every way, or standing upright, or at right angles to the surface; -- said of a shell.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Squarrose-slashed</b></col> <fld>(Bot.)</fld>, <cd>doubly slashed, with the smaller divisions at right angles to the others, as a leaf.</cd>  <rj><au>Lindley.</au></rj></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Squarroso-dentate</ent><br/
<hw>Squar*ro`so-den"tate</hw> <pr>(skw<ocr/r*r<omac/`s<osl/-d<ecr/n"t<asl/t)</pr>, <pos>a.</pos> <fld>(Bot.)</fld> <def>Having the teeth bent out of the plane of the lamina; -- said of a leaf.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Squarrous</ent><br/
<hw>Squar"rous</hw> <pr>(skw<acr/r"r<ucr/s <or/ skw<ocr/r"-)</pr>, <pos>a.</pos> <def>Squarrose.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Squarrulose</ent><br/
<hw>Squar"ru*lose`</hw> <pr>(-r<usdot/*l<omac/s`)</pr>, <pos>a.</pos> <ety>[Dim. of <ets>squarrose</ets>.]</ety> <fld>(Bot.)</fld> <def>Somewhat squarrose; slightly squarrose.</def>  <rj><au>Gray.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Squash</ent><br/
<hw>Squash</hw> <pr>(skw<ocr/sh)</pr>, <pos>n.</pos> <ety>[Cf. <er>Musquash</er>.]</ety> <fld>(Zool.)</fld> <def>An American animal allied to the weasel.</def> <mark>[Obs.]</mark>  <rj><au>Goldsmith.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Squash</ent><br/
<hw>Squash</hw>, <pos>n.</pos> <ety>[Massachusetts Indian <ets>asq</ets>, pl. <ets>asquash</ets>, raw, green, immature, applied to fruit and vegetables which were used when green, or without cooking; <ets>askutasquash</ets> vine apple.]</ety> <fld>(Bot.)</fld> <def>A plant and its fruit of the genus <gen>Cucurbita</gen>, or gourd kind.</def><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ The species are much confused. The long-neck squash is called <spn>Cucurbita verrucosa</spn>, the Barbary or China squash, <spn>Cucurbita moschata</spn>, and the great winter squash, <spn>Cucurbita maxima</spn>, but the distinctions are not clear.</note><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Squash beetle</b></col> <fld>(Zool.)</fld>, <cd>a small American beetle (<spn>Diabrotica vittata</spn>, syn. <spn>Galeruca vittata</spn>) which is often abundant and very injurious to the leaves of squash, cucumber, etc.  It is striped with yellow and black.  The name is applied also to other allied species.</cd> -- <col><b>Squash bug</b></col> <fld>(Zool.)</fld>, <cd>a large black American hemipterous insect (<spn>Coreus tristis</spn> syn. <spn>Anasa tristis</spn>) injurious to squash vines.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Squash</ent><br/
<hw>Squash</hw>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Squashed</conjf> <pr>(skw<ocr/sht)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Squashing</conjf>.]</vmorph> <ety>[OE. <ets>squachen</ets>, OF. <ets>escachier</ets>, <ets>esquachier</ets>, to squash, to crush, F. <ets><eacute/cacher</ets>, perhaps from (assumed) LL. <ets>excoacticare</ets>, fr. L. <ets>ex</ets> + <ets>coactare</ets> to constrain, from <ets>cogere</ets>, <ets>coactum</ets>, to compel.  Cf. <er>Cogent</er>, <er>Squat</er>, <pos>v. i.</pos>]</ety> <def>To beat or press into pulp or a flat mass; to crush.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Squash</ent><br/
<hw>Squash</hw>, <pos>n.</pos> <sn>1.</sn> <def>Something soft and easily crushed; especially, an unripe pod of pease.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Not yet old enough for a man, nor young enough for a boy; as a <qex>squash</qex> is before 't is a peascod.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Hence, something unripe or soft; -- used in contempt.</def> <ldquo/This <xex>squash</xex>, this gentleman.<rdquo/  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>A sudden fall of a heavy, soft body; also, a shock of soft bodies.</def>  <rj><au>Arbuthnot.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>My fall was stopped by a terrible <qex>squash</qex>.</q> <rj><qau>Swift.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>A game much like rackets, played in a walled court with soft rubber balls and bats like tennis rackets; -- called also <altname>squash rackets</altname>.</def><br/
[<source>PJC</source>]</p>

<p><ent>Squasher</ent><br/
<hw>Squash"er</hw> <pr>(-<etil/r)</pr>, <pos>n.</pos> <def>One who, or that which, squashes.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Squashiness</ent><br/
<hw>Squash"i*ness</hw> <pr>(-<icr/*n<ecr/s)</pr>, <pos>n.</pos> <def>The quality or state of being squashy, or soft.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Squashy</ent><br/
<hw>Squash"y</hw> <pr>(-<ycr/)</pr>, <pos>a.</pos> <def>Easily squashed; soft.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Squat</ent><br/
<hw>Squat</hw> <pr>(skw<ocr/t)</pr>, <pos>n.</pos> <fld>(Zool.)</fld> <def>The angel fish (<spn>Squatina angelus</spn>).</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Squat</ent><br/
<hw>Squat</hw>, <pos>v. i.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Squatted</conjf>; <pos>p. pr. & vb. n.</pos> <conjf>Squatting</conjf>.]</vmorph> <ety>[OE. <ets>squatten</ets> to crush, OF. <ets>esquater</ets>, <ets>esquatir</ets> (cf. It. <ets>quatto</ets> squat, cowering), perhaps fr. L. <ets>ex</ets> + <ets>coactus</ets>, p. p. of <ets>cogere</ets> to drive or urge together. See <er>Cogent</er>, <er>Squash</er>, <pos>v. t.</pos>]</ety> <sn>1.</sn> <def>To sit down upon the hams or heels; <as>as, the savages <ex>squatted</ex> near the fire</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To sit close to the ground; to cower; to stoop, or lie close, to escape observation, as a partridge or rabbit.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>To settle on another's land without title; also, to settle on common or public lands.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Squat</ent><br/
<hw>Squat</hw>, <pos>v. t.</pos> <def>To bruise or make flat by a fall.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Squat</ent><br/
<hw>Squat</hw>, <pos>a.</pos> <sn>1.</sn> <def>Sitting on the hams or heels; sitting close to the ground; cowering; crouching.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Him there they found,<br/
<qex>Squat</qex> like a toad, close at the ear of Eve.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Short and thick, like the figure of an animal squatting.</def> <ldquo/The round, <xex>squat</xex> turret.<rdquo/  <rj><au>R. Browning.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>The head [of the squill insect] is broad and <qex>squat</qex>.</q> <rj><qau>Grew.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Squat</ent><br/
<hw>Squat</hw>, <pos>n.</pos> <sn>1.</sn> <def>The posture of one that sits on his heels or hams, or close to the ground.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A sudden or crushing fall.</def> <mark>[Obs.]</mark>  <rj><au>Herbert.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <fld>(Mining)</fld> <sd>(a)</sd> <def>A small vein of ore.</def> <sd>(b)</sd> <def>A mineral consisting of tin ore and spar.</def>  <rj><au>Halliwell.</au>  <au>Woodward.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Squat snipe</b></col> <fld>(Zool.)</fld>, <cd>the jacksnipe; -- called also <altname>squatter</altname>.</cd> <mark>[Local, U.S.]</mark></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Squaterole</ent><br/
<hw>Squat"er*ole</hw> <pr>(-<etil/r*<omac/l)</pr>, <pos>n.</pos> <fld>(Zool.)</fld> <def>The black-bellied plover.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Squatter</ent><br/
<hw>Squat"ter</hw> <pr>(-t<etil/r)</pr>, <pos>n.</pos> <sn>1.</sn> <def>One who squats;</def> <specif>specifically,</specif> <def>one who settles unlawfully upon land without a title.  In the United States and Australia the term is sometimes applied also to a person who settles lawfully upon government land under legal permission and restrictions, before acquiring title.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>In such a tract, <qex>squatters</qex> and trespassers were tolerated to an extent now unknown.</q> <rj><qau>Macaulay.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Zool.)</fld> <def>See <cref>Squat snipe</cref>, under <er>Squat</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Squatter sovereignty</b></col>, <cd>the right claimed by the squatters, or actual residents, of a Territory of the United States to make their own laws.</cd> <mark>[Local, U.S.]</mark> <au>Bartlett.</au></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Squatty</ent><br/
<hw>Squat"ty</hw> <pr>(skw<ocr/t"t<ycr/)</pr>, <pos>a.</pos> <def>Squat; dumpy.</def>  <rj><au>J. Burroughs.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Squaw</ent><br/
<hw>Squaw</hw> <pr>(skw<add/)</pr>, <pos>n.</pos> <ety>[Massachusetts Indian <ets>squa</ets>, <ets>eshqua</ets>; Narragansett <ets>squ<acir/ws</ets>; Delaware <ets>ochqueu</ets>, and <ets>khqueu</ets>; used also in compound words (as the names of animals) in the sense of <ets>female</ets>.]</ety> <def>A female; a woman, especially a married woman; a wife; -- in the language of Indian tribes of the Algonquin family, correlative of <inv>sannup</inv>.</def> <mark>[Considered offensive by some American indians.]</mark><br/
[<source>1913 Webster</source> <source>+PJC</source>]</p>

<p><cs><col><b>Old squaw</b></col>. <fld>(Zool.)</fld> <cd>See under <er>Old</er>.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Squawberry</ent><br/
<hw>Squaw"ber`ry</hw> <pr>(-b<ecr/r`r<ycr/)</pr>, <pos>n.</pos> <fld>(Bot.)</fld> <def>A local name for the partridge berry; also, for the deerberry.</def> <mark>[U. S.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Squawk</ent><br/
<hw>Squawk</hw> <pr>(skw<add/k)</pr>, <pos>v. i.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Squawked</conjf> <pr>(skw<add/kt)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Squawking</conjf>.]</vmorph> <ety>[See <er>Squeak</er>.]</ety> <def>To utter a shrill, abrupt scream; to squeak harshly.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Squawking thrush</b></col> <fld>(Zool.)</fld>, <cd>the missel turush; -- so called from its note when alarmed.</cd> <mark>[Prov. Eng.]</mark></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Squawk</ent><br/
<hw>Squawk</hw>, <pos>n.</pos> <sn>1.</sn> <def>Act of squawking; a harsh squeak.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Zool.)</fld> <def>The American night heron. See under <er>Night</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Squawk duck</b></col> <fld>(Zool.)</fld>, <cd>the bimaculate duck (<spn>Anas glocitans</spn>). It has patches of reddish brown behind, and in front of, each eye.</cd> <mark>[Prov. Eng.]</mark></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Squawl</ent><br/
<hw>Squawl</hw> <pr>(skw<add/l)</pr>, <pos>v. i.</pos> <def>See <er>Squall</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Squaw man</ent><br/
<hw>Squaw man</hw>. <def>A white man who has married an Indian squaw; sometimes, one who has gained tribal rights by such a marriage; -- often a term of contempt.</def> <mark>[Western U. S.]</mark><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Squawroot</ent><br/
<hw>Squaw"root`</hw> <pr>(skw<add/"r<oomac/t`)</pr>, <pos>n.</pos> <fld>(Bot.)</fld> <def>A scaly parasitic plant (<spn>Conopholis Americana</spn>) found in oak woods in the United States; -- called also <altname>cancer root</altname>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Squaw vine</ent><br/
<hw>Squaw vine</hw>. <fld>(Bot.)</fld> <def>The partridge berry (<spn>Mitchella repens</spn>).</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Squawweed</ent><br/
<hw>Squaw"weed`</hw> <pr>(skw<add/"w<emac/d`)</pr>, <pos>n.</pos> <fld>(Bot.)</fld> <def>The golden ragwort. See under <er>Ragwort</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Squeak</ent><br/
<hw>Squeak</hw> <pr>(skw<emac/k)</pr>, <pos>v. i.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Squeaked</conjf> <pr>(skw<emac/kt)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Squeaking</conjf>.]</vmorph> <ety>[Probably of imitative origin; cf. Sw. <ets>sqv<aum/ka</ets> to croak, Icel. <ets>skvakka</ets> to give a sound as of water shaken in a bottle.]</ety> <sn>1.</sn> <def>To utter a sharp, shrill cry, usually of short duration; to cry with an acute tone, as an animal; or, to make a sharp, disagreeable noise, as a pipe or quill, a wagon wheel, a door; to creak.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Who can endure to hear one of the rough old Romans <qex>squeaking</qex> through the mouth of an eunuch?</q> <rj><qau>Addison.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Zoilus calls the companions of Ulysses the <ldquo/<qex>squeaking</qex> pigs<rdquo/ of Homer.</q> <rj><qau>Pope.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To break silence or secrecy for fear of pain or punishment; to speak; to confess.</def> <mark>[Colloq.]</mark><br/
<syn><b>Syn. --</b> squeal.</syn><br/
[<source>1913 Webster</source>]</p>

<p><q>If he be obstinate, put a civil question to him upon the rack, and he <qex>squeaks</qex>, I warrant him.</q> <rj><qau>Dryden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Squeak</ent><br/
<hw>Squeak</hw>, <pos>n.</pos> <def>A sharp, shrill, disagreeable sound suddenly uttered, either of the human voice or of any animal or instrument, such as is made by carriage wheels when dry, by the soles of leather shoes, or by a pipe or reed.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Squeaker</ent><br/
<hw>Squeak"er</hw> <pr>(-<etil/r)</pr>, <pos>n.</pos> <sn>1.</sn> <def>One who, or that which, squeaks.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Zool.)</fld> <def>The Australian gray crow shrike (<spn>Strepera anaphonesis</spn>); -- so called from its note.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <sd>(a)</sd> <def>A contest won by a small margin.</def> <sd>(b)</sd> <def>A narrow escape from harm.</def> <mark>[Colloq.]</mark><br/
[<source>PJC</source>]</p>

<p><ent>Squeakingly</ent><br/
<hw>Squeak"ing*ly</hw>, <pos>adv.</pos> <def>In a squeaking manner.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Squeal</ent><br/
<hw>Squeal</hw> <pr>(skw<emac/l)</pr>, <pos>v. i.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Squealed</conjf> <pr>(skw<emac/ld)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Squealing</conjf>.]</vmorph> <ety>[Of Scand. origin; cf. Sw. <ets>sqv<aum/la</ets>, Norw. <ets>skvella</ets>.  Cf. <er>Squeak</er>, <er>Squall</er>.]</ety> <sn>1.</sn> <def>To cry with a sharp, shrill, prolonged sound, as certain animals do, indicating want, displeasure, or pain.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To turn informer; to betray a secret.</def> <mark>[Slang]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Squeal</ent><br/
<hw>Squeal</hw>, <pos>n.</pos> <def>A shrill, sharp, somewhat prolonged cry.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Squealer</ent><br/
<hw>Squeal"er</hw> <pr>(-<etil/r)</pr>, <pos>n.</pos> <sn>1.</sn> <def>One who, or that which, squeals.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Zool.)</fld> <sd>(a)</sd> <def>The European swift.</def> <sd>(b)</sd> <def>The harlequin duck.</def> <sd>(c)</sd> <def>The American golden plover.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Squeamish</ent><br/
<hw>Squeam"ish</hw> <pr>(skw<emac/m"<icr/sh)</pr>, <pos>a.</pos> <ety>[OE. <ets>squaimous</ets>, <ets>sweymous</ets>, probably from OE. <ets>sweem</ets>, <ets>swem</ets>, dizziness, a swimming in the head; cf. Icel. <ets>sveimr</ets> a bustle, a stir, Norw. <ets>sveim</ets> a hovering about, a sickness that comes upon one, Icel. <ets>svimi</ets> a giddiness, AS. <ets>sw<imac/ma</ets>.  The word has been perhaps confused with <ets>qualmish</ets>.  Cf. <er>Swim</er> to be dizzy.]</ety> <def>Having a stomach that is easily turned or nauseated; hence, nice to excess in taste; fastidious; easily disgusted; apt to be offended at trifling improprieties.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Quoth he, that honor's very <qex>squeamish</qex><br/
That takes a basting for a blemish.</q> <rj><qau>Hudibras.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>His muse is rustic, and perhaps too plain<br/
The men of <qex>squeamish</qex> taste to entertain.</q> <rj><qau>Southern.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>So ye grow <qex>squeamish</qex>, Gods, and sniff at heaven.</q> <rj><qau>M. Arnold.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><syn><b>Syn.</b> -- Fastidious; dainty; overnice; scrupulous. See <er>Fastidious</er>.</syn><br/
[<source>1913 Webster</source>]</p>

<p>-- <wordforms><wf>Squeam"ish*ly</wf>, <pos>adv.</pos> -- <wf>Squeam"ish*ness</wf>, <pos>n.</pos></wordforms><br/
[<source>1913 Webster</source>]</p>

<p><ent>Squeamous</ent><br/
<hw>Squeam"ous</hw> <pr>(-<ucr/s)</pr>, <pos>a.</pos> <def>Squeamish.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Squeasiness</ent><br/
<hw>Squea"si*ness</hw> <pr>(skw<emac/"z<icr/*n<ecr/s)</pr>, <pos>n.</pos> <def>Queasiness.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Squeasy</ent><br/
<hw>Squea"sy</hw> <pr>(-z<ycr/)</pr>, <pos>a.</pos> <def>Queasy; nice; squeamish; fastidious; scrupulous.</def> <mark>[Obs.]</mark>  <rj><au>Bp. Earle.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Squeegee</ent><br/
<hw>Squee"gee</hw> <pr>(skw<emac/"j<esl/)</pr>, <pos>n.</pos> <def>Formerly, a small swab for drying a vessel's deck; now, a kind of scraper having a blade or edge of rubber or of leather, -- used for removing superfluous, water or other liquids, as from a vessel's deck after washing, from window panes, photographer's plates, etc.</def> <altsp>[Written also <asp>squilgee</asp>, <asp>squillagee</asp>.]</altsp><br/
[<source>1913 Webster</source>]</p>

<p><ent>Squeegee</ent><br/
<hw>Squee"gee</hw>, <pos>v. t.</pos>  <vmorph>[<pos>imp. & p. p.</pos> <conjf>Squeegeed</conjf> <pr>(skw<emac/"j<emac/d; skw<emac/*j<emac/d")</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Squeegeeing</conjf> <pr>(skw<emac/"j<emac/*<icr/ng; skw<emac/*j<emac/"<icr/ng)</pr>.]</vmorph> <def>To smooth, clean, press, or treat with a squeegee; to squilgee.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Squeegee roller</ent><br/
<hw>Squeegee roller</hw>. <def>A small India-rubber roller with a handle, used esp. in printing and photography as a squeegee.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Squeeze</ent><br/
<hw>Squeeze</hw> <pr>(skw<emac/z)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Squeezed</conjf> <pr>(skw<emac/zd)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Squeezing</conjf>.]</vmorph> <ety>[OE. <ets>queisen</ets>, AS. <ets>cw<emac/san</ets>, <ets>cw<ymac/san</ets>, <ets>cw<imac/san</ets>, of uncertain origin. The <ets>s-</ets> was probably prefixed through the influence of <ets>squash</ets>, v.t.]</ety> <sn>1.</sn> <def>To press between two bodies; to press together closely; to compress; often, to compress so as to expel juice, moisture, etc.; <as>as, to <ex>squeeze</ex> an orange with the fingers; to <ex>squeeze</ex> the hand in friendship.</as></def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Fig.: To oppress with hardships, burdens, or taxes; to harass; to crush.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>In a civil war, people must expect to be crushed and <qex>squeezed</qex> toward the burden.</q> <rj><qau>L'Estrange.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>To force, or cause to pass, by compression; often with <xex>out</xex>, <xex>through</xex>, etc.; <as>as, to <ex>squeeze</ex> water through felt</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><syn><b>Syn.</b> -- To compress; hug; pinch; gripe; crowd.</syn><br/
[<source>1913 Webster</source>]</p>

<p><ent>Squeeze</ent><br/
<hw>Squeeze</hw>, <pos>v. i.</pos> <def>To press; to urge one's way, or to pass, by pressing; to crowd; -- often with <xex>through</xex>, <xex>into</xex>, etc.; <as>as, to <ex>squeeze</ex> hard to get through a crowd</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Squeeze</ent><br/
<hw>Squeeze</hw>, <pos>n.</pos> <sn>1.</sn> <def>The act of one who squeezes; compression between bodies; pressure.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A facsimile impression taken in some soft substance, as pulp, from an inscription on stone.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <fld>(Mining)</fld> <def>The gradual closing of workings by the weight of the overlying strata.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><sn>4.</sn>  <def>Pressure or constraint used to force the making of a gift, concession, or the like; exaction; extortion; <as>as, to put the <ex>squeeze</ex> on someone</as>.</def> <mark>[Colloq.]</mark></p>

<p><q>One of the many <ldquo/<qex>squeezes</qex><rdquo/ imposed by the mandarins.</q>  <rj><qau>A. R. Colquhoun.</qau></rj><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Squeezer</ent><br/
<hw>Squeez"er</hw> <pr>(skw<emac/z"<etil/r)</pr>, <pos>n.</pos> <sn>1.</sn> <def>One who, or that which, squeezes; <as>as, a lemon <ex>squeezer</ex></as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Forging)</fld> <sd>(a)</sd> <def>A machine like a large pair of pliers, for shingling, or squeezing, the balls of metal when puddled; -- used only in the plural.</def> <sd>(b)</sd> <def>A machine of several forms for the same purpose; -- used in the singular.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Squeezing</ent><br/
<hw>Squeez"ing</hw>, <pos>n.</pos> <sn>1.</sn> <def>The act of pressing; compression; oppression.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <pluf>pl.</pluf> <def>That which is forced out by pressure; dregs.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>Same as <er>Squeeze</er>, <pos>n.</pos>, 2.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Squelch</ent><br/
<hw>Squelch</hw> <pr>(skw<ecr/lch)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Squelched</conjf> <pr>(skw<ecr/lcht)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Squelching</conjf>.]</vmorph> <ety>[Cf. Prov. E. <ets>quelch</ets> a blow, and <ets>quell</ets> to crush, to kill.]</ety> <def>To quell; to crush; to silence or put down.</def> <mark>[Colloq.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>Oh 't was your luck and mine to be <qex>squelched</qex>.</q> <rj><qau>Beau. & Fl.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>If you deceive us you will be <qex>squelched</qex>.</q> <rj><qau>Carlyle.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Squelch</ent><br/
<hw>Squelch</hw>, <pos>n.</pos> <sn>1.</sn> <def>A heavy fall, as of something flat.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <specif>Hence:</specif> <def>A crushing reply; <as>as, the perfect <ex>squelch</ex> for a conceited remark</as>.</def> <mark>[Colloq.]</mark>  <rj><au>Hudibras.</au></rj><br/
[<source>1913 Webster</source> <source>+PJC</source>]</p>

<p><ent>Squelch</ent><br/
<hw>Squelch</hw> <pr>(skw<ecr/lch)</pr>, <pos>v. i.</pos> <ety>[Perh. imitative. Cf. <er>Squelch</er>.]</ety> <def>To make a sound like that made by the feet of one walking in mud or slush; to make a kind of swashing sound; to squish; also, to move with such a sound.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><q>He turned and strode to the fire, his boots <qex>squelching</qex> as he walked.</q>  <rj><qau>P. L. Ford.</qau></rj><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><q>A crazy old collier <qex>squelching</qex> along under squared yards.</q>  <rj><qau>W. C. Russell.</qau></rj><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>squeteague</ent><br/
<hw>sque*teague"</hw> <pr>(skw<esl/*t<emac/g")</pr>, <pos>n.</pos> <ety>[From the North American Indian name.]</ety> <fld>(Zool.)</fld> <def>An American sciaenoid fish (<spn>Cynoscion regalis</spn>), abundant on the Atlantic coast of the United States, and much valued as a food fish.  It is of a bright silvery color, with iridescent reflections.  Called also <altname>weakfish</altname>, <altname>squitee</altname>, <altname>chickwit</altname>, and <altname>sea trout</altname>.  The spotted squeteague (<spn>Cynoscion nebulosus</spn>) of the Southern United States is a similar fish, but the back and upper fins are spotted with black.  It is called also <altname>spotted weakfish</altname> and <altname>squit</altname>, and, locally, <altname>sea trout</altname>, and <altname>sea salmon</altname>.  See also under <er>squitee</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Squib</ent><br/
<hw>Squib</hw> <pr>(skw<icr/b)</pr>, <pos>n.</pos> <ety>[OE. <ets>squippen</ets>, <ets>swippen</ets>, to move swiftly, Icel. <ets>svipa</ets> to swoop, flash, dart, whip; akin to AS. <ets>swipian</ets> to whip, and E. <ets>swift</ets>, a. See <er>Swift</er>, <pos>a.</pos>]</ety> <sn>1.</sn> <def>A little pipe, or hollow cylinder of paper, filled with powder or combustible matter, to be thrown into the air while burning, so as to burst there with a crack.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Lampoons, like <qex>squibs</qex>, may make a present blaze.</q> <rj><qau>Waller.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>The making and selling of fireworks, and <qex>squibs</qex> . . . is punishable.</q> <rj><qau>Blackstone.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Mining)</fld> <def>A kind of slow match or safety fuse.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>A sarcastic speech or publication; a petty lampoon; a brief, witty essay.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Who copied his <qex>squibs</qex>, and reechoed his jokes.</q> <rj><qau>Goldsmith.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>A writer of lampoons.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>The <qex>squibs</qex> are those who in the common phrase of the world are called libelers, lampooners, and pamphleteers.</q> <rj><qau>Tatler.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <def>A paltry fellow.</def> <mark>[Obs.]</mark>  <rj><au>Spenser.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Squib</ent><br/
<hw>Squib</hw>, <pos>v. i.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Squibbed</conjf> <pr>(skw<icr/bd)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Squibbing</conjf>.]</vmorph> <def>To throw squibs; to utter sarcastic or severe reflections; to contend in petty dispute; <as>as, to <ex>squib</ex> a little in debate</as>.</def> <mark>[Colloq.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Squid</ent><br/
<hw>Squid</hw> <pr>(skw<icr/d)</pr>, <pos>n.</pos> <ety>[Cf. <er>Squirt</er>.]</ety> <sn>1.</sn> <fld>(Zool.)</fld> <def>Any one of numerous species of ten-armed cephalopods having a long, tapered body, and a caudal fin on each side; especially, any species of <gen>Loligo</gen>, <gen>Ommastrephes</gen>, and related genera. See <er>Calamary</er>, <er>Decacerata</er>, <er>Dibranchiata</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ Some of these squids are very abundant on the Atlantic coast of North America, and are used in large quantities for bait, especially in the cod fishery. The most abundant of the American squids are the northern squid (<spn>Ommastrephes illecebrosus</spn>), ranging from Southern New England to Newfoundland, and the southern squid (<spn>Loligo Pealii</spn>), ranging from Virginia to Massachusetts.</note><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A fishhook with a piece of bright lead, bone, or other substance, fastened on its shank to imitate a squid.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><mcol><col><b>Flying squid</b></col>, <col><b>Giant squid</b></col></mcol>. <fld>(Zool.)</fld> <cd>See under <er>Flying</er>, and <er>Giant</er>.</cd> -- <col><b>Squid hound</b></col> <fld>(Zool.)</fld>, <cd>the striped bass.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Squier</ent><br/
<hw>Squier</hw> <pr>(skw<imac/r)</pr>, <pos>n.</pos> <def>A square. See 1st <er>Squire</er>.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>Not the worst of the three but jumps twelve foot and a half by the <qex>squier</qex>.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Squiery</ent><br/
<ent>Squierie</ent><br/
<mhw>{ <hw>Squi"er*ie</hw>, <hw>Squi"er*y</hw> }</mhw> <pr>(skw<imac/"<etil/r*<icr/)</pr>, <pos>n.</pos> <ety>[OF. <ets>escuierie</ets>. See <er>Esquire</er>.]</ety> <def>A company of squires; the whole body of squires.</def><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ This word is found in Tyrwhitt's Chaucer, but is not in the modern editions.</note><br/
[<source>1913 Webster</source>]</p>

<p><ent>Squiffy</ent><br/
<hw>Squif"fy</hw> <pr>(skw<icr/f"f<ycr/)</pr>, <pos>a.</pos> <def>Somewhat intoxicated; tipsy.</def> <mark>[Slang]</mark>  <rj><au>Kipling.</au></rj><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Squiggle</ent><br/
<hw>Squig"gle</hw> <pr>(skw<icr/g"g'l)</pr>, <pos>v. i.</pos> <ety>[Cf. Prov. E. <ets>swiggle</ets> to drink greedily, to shake liquor in a close vessel, and E. <ets>swig</ets>.]</ety> <def>To shake and wash a fluid about in the mouth with the lips closed.</def> <mark>[Prov. Eng.]</mark>  <rj><au>Forby.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Squiggle</ent><br/
<hw>Squig"gle</hw>, <pos>v. i.</pos> <ety>[Cf. <er>Squirm</er>, <er>Wiggle</er>.]</ety> <def>To move about like an eel; to squirm.</def> <mark>[Low, U.S.]</mark>  <rj><au>Bartlett.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><-- p. 1398 pr=VMG --></p>

<p><ent>Squilgee</ent><br/
<hw>Squil"gee</hw> <pr>(skw<icr/l"j<esl/)</pr>, <pos>n.</pos> <def>Formerly, a small swab for drying a vessel's deck; now, a kind of scraper having a blade or edge of rubber or of leather, -- used for removing superfluous water or other liquids, as from a vessel's deck after washing, from window panes, photographer's plates, etc.</def> <altsp>[Written also <asp>squillgee</asp>, <asp>squillagee</asp>, <asp>squeegee.</asp>]</altsp><br/
[<source>1913 Webster</source>]</p>

<p><ent>Squilgee</ent><br/
<hw>Squil"gee</hw> <pr>(skw<icr/l"j<esl/)</pr>, <pos>v. t.</pos>  <vmorph>[<pos>imp. & p. p.</pos> <conjf>Squilgeed</conjf> <pr>(skw<icr/l"j<emac/d)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Squilgeeing</conjf>.]</vmorph> <def>To swab, press, or treat with a squilgee; <as>as, to <ex>squilgee</ex> a vessel's deck</as>.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Squill</ent><br/
<hw>Squill</hw> <pr>(skw<icr/l)</pr>, <pos>n.</pos> <ety>[F. <ets>squille</ets> (also <ets>scille</ets> a squill, in sense 1), L. <ets>squilla</ets>, <ets>scilla</ets>, Gr. <?/.]</ety> <sn>1.</sn> <fld>(Bot.)</fld> <sd>(a)</sd> <def>A European bulbous liliaceous plant (<spn>Urginea maritima</spn>, formerly <spn>Scilla maritima</spn>), of acrid, expectorant, diuretic, and emetic properties, used in medicine.  Called also <altname>sea onion</altname>.</def> <sd>(b)</sd> <def>Any bulbous plant of the genus <gen>Scilla</gen>; <as>as, the bluebell <ex>squill</ex> (<spn>Scilla mutans</spn>)</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Zool.)</fld> <sd>(a)</sd> <def>A squilla.</def> <sd>(b)</sd> <def>A mantis.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Squilla</ent><br/
||<hw>Squil"la</hw> <pr>(skw<icr/l"l<adot/)</pr>, <pos>n.</pos>; <plu><it>pl.</it> E. <plw>Squillas</plw> <pr>(-l<adot/z)</pr>, L. <plw>Squillae</plw> <pr>(-l<emac/)</pr>.</plu> <ety>[L., a sea onion, also, a prawn or shrimp. See <er>Squill</er>.]</ety> <fld>(Zool.)</fld> <def>Any one of numerous stomapod crustaceans of the genus <gen>Squilla</gen> and allied genera.  They make burrows in mud or beneath stones on the seashore.  Called also <altname>mantis shrimp</altname>. See <xex>Illust.</xex> under <er>Stomapoda</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Squillitic</ent><br/
<hw>Squill*it"ic</hw> <pr>(skw<icr/l*<icr/t"<icr/k)</pr>, <pos>a.</pos> <def>Of or pertaining to squills.</def> <mark>[R.]</mark> <ldquo/<xex>Squillitic</xex> vinegar.<rdquo/  <rj><au>Holland.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Squinancy</ent><br/
<ent>Squinance</ent><br/
<mhw>{ <hw>Squin"ance</hw> <pr>(skw<icr/n"<ait/ns)</pr>, <hw>Squin"an*cy</hw> <pr>(-<ait/n*s<ycr/)</pr>, }</mhw> <pos>n.</pos> <ety>[F. <ets>esquinancie</ets>, OF. <ets>squinance</ets>, <ets>esquinance</ets>. See <er>Quinsy</er>.]</ety> <sn>1.</sn> <fld>(Med.)</fld> <def>The quinsy. See <er>Quinsy</er>.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Bot.)</fld> <def>A European perennial herb (<spn>Asperula cynanchica</spn>) with narrowly linear whorled leaves; -- formerly thought to cure the quinsy. Also called <altname>quincewort</altname>.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Squinancy berries</b></col>, <cd>black currants; -- so called because used to cure the quinsy.</cd> <au>Dr. Prior.</au></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Squinch</ent><br/
<hw>Squinch</hw> <pr>(skw<icr/nch)</pr>, <pos>n.</pos> <ety>[Corrupted fr. <ets>sconce</ets>.]</ety> <fld>(Arch.)</fld> <def>A small arch thrown across the corner of a square room to support a superimposed mass, as where an octagonal spire or drum rests upon a square tower; -- called also <altname>sconce</altname>, and <altname>sconcheon</altname>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Squinsy</ent><br/
<hw>Squin"sy</hw> <pr>(skw<icr/n"z<ycr/)</pr>, <pos>n.</pos> <fld>(Med.)</fld> <def>See <er>Quinsy</er>.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Squint</ent><br/
<hw>Squint</hw> <pr>(skw<icr/nt)</pr>, <pos>a.</pos> <ety>[Cf. D. <ets>schuinte</ets> a slope, <ets>schuin</ets>, <ets>schuinsch</ets>, sloping, oblique, <ets>schuins</ets> slopingly.  Cf. <er>Askant</er>, <er>Askance</er>, <er>Asquint</er>.]</ety> <sn>1.</sn> <def>Looking obliquely.</def> <specif>Specifically:</specif> <fld>(Med.)</fld>, <def>not having the optic axes coincident; -- said of the eyes. See <er>Squint</er>, <pos>n.</pos>, 2.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <mark>Fig.:</mark>  <def>Looking askance.</def> <ldquo/<xex>Squint</xex> suspicion.<rdquo/  <rj><au>Milton.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Squint</ent><br/
<hw>Squint</hw>, <pos>v. i.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Squinted</conjf>; <pos>p. pr. & vb. n.</pos> <conjf>Squinting</conjf>.]</vmorph> <sn>1.</sn> <def>To see or look obliquely, asquint, or awry, or with a furtive glance.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Some can <qex>squint</qex> when they will.</q> <rj><qau>Bacon.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Med.)</fld> <def>To have the axes of the eyes not coincident; to be cross-eyed.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>To deviate from a true line; to run obliquely.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>To have an indirect bearing, reference, or implication; to have an allusion to, or inclination towards, something.</def></p>

<p><q>Yet if the following sentence means anything, it is a <qex>squinting</qex> toward hypnotism.</q>  <rj><qau>The Forum.</qau></rj><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><sn>5.</sn> <def>To look with the eyes partly closed.</def><br/
[<source>PJC</source>]</p>

<p><ent>Squint</ent><br/
<hw>Squint</hw>, <pos>v. t.</pos> <sn>1.</sn> <def>To turn to an oblique position; to direct obliquely; <as>as, to <ex>squint</ex> an eye</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To cause to look with noncoincident optic axes.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>He . . . <qex>squints</qex> the eye, and makes the harelid.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Squint</ent><br/
<hw>Squint</hw>, <pos>n.</pos> <sn>1.</sn> <def>The act or habit of squinting.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Med.)</fld> <def>A want of coincidence of the axes of the eyes; strabismus.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <fld>(Arch.)</fld> <def>Same as <er>Hagioscope</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Squinter</ent><br/
<hw>Squint"er</hw> <pr>(-<etil/r)</pr>, <pos>n.</pos> <def>One who squints.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Squint-eye</ent><br/
<hw>Squint"-eye`</hw> <pr>(-<imac/`)</pr>, <pos>n.</pos> <def>An eye that squints.</def>  <rj><au>Spenser.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Squint-eyed</ent><br/
<hw>Squint"-eyed`</hw> <pr>(-<imac/d`)</pr>, <pos>a.</pos> <sn>1.</sn> <def>Having eyes that squint; having eyes with axes not coincident; cross-eyed; also called <altname>squinty-eyed</altname>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Looking obliquely, or asquint; malignant; <as>as, <ex>squint-eyed</ex> praise; <ex>squint-eyed</ex> jealousy</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Squintifego</ent><br/
<hw>Squint`i*fe"go</hw> <pr>(-<icr/*f<emac/"g<osl/)</pr>, <pos>a.</pos> <def>Squinting.</def> <mark>[Obs. & R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Squinting</ent><br/
<hw>Squint"ing</hw> <pr>(skw<icr/nt"<icr/ng)</pr>, <def><pos>a. & n.</pos> from <er>Squint</er>, <pos>v.</pos></def> -- <wordforms><wf>Squint"ing*ly</wf>, <pos>adv.</pos></wordforms><br/
[<source>1913 Webster</source>]</p>

<p><ent>Squiny</ent><br/
<hw>Squin"y</hw> <pr>(skw<icr/n"<ycr/)</pr>, <pos>v. i.</pos> <def>To squint.</def> <mark>[Obs.]</mark>  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Squinzey</ent><br/
<hw>Squin"zey</hw> <pr>(-z<ycr/)</pr>, <pos>n.</pos> <fld>(Med.)</fld> <def>See <er>Quinsy</er>.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Squir</ent><br/
<hw>Squir</hw> <pr>(skw<etil/r)</pr>, <pos>v. t.</pos> <def>To throw with a jerk; to throw edge foremost.</def> <mark>[Obs.]</mark> <altsp>[Written also <asp>squirr</asp>.]</altsp>  <rj><au>Addison.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Squiralty</ent><br/
<hw>Squir"al*ty</hw> <pr>(skw<imac/r"<acr/l*t<ycr/)</pr>, <pos>n.</pos> <def>Same as <er>Squirarchy</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>That such weight and influence be put thereby into the hands of the <qex>squiralty</qex> of my kingdom.</q> <rj><qau>Sterne.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Squirarch</ent><br/
<hw>Squir"arch</hw> <pr>(skw<imac/r"<aum/rk)</pr>, <pos>n.</pos> <ety>[<ets>Squire</ets> + <ets>-arch</ets>.]</ety> <def>One who belongs to the squirarchy.</def> -- <wordforms><wf>Squir"arch*al</wf> <pr>(-<ait/l)</pr>, <pos>a.</pos></wordforms><br/
[<source>1913 Webster</source>]</p>

<p><ent>Squirarchy</ent><br/
<hw>Squir"arch*y</hw> <pr>(-<ycr/)</pr>, <pos>n.</pos> <ety>[<ets>Squire</ets> + <ets>-archy</ets>.]</ety> <def>The gentlemen, or gentry, of a country, collectively.</def> <altsp>[Written also <asp>squirearchy</asp>.]</altsp><br/
[<source>1913 Webster</source>]</p>

<p><ent>Squire</ent><br/
<hw>Squire</hw> <pr>(skw<imac/r)</pr>, <pos>n.</pos> <ety>[OF. <ets>esquierre</ets>, F. <ets><eacute/querre</ets>. See <er>Square</er>, <pos>n.</pos>]</ety> <def>A square; a measure; a rule.</def> <mark>[Obs.]</mark> <ldquo/With golden <xex>squire</xex>.<rdquo/  <rj><au>Spenser.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Squire</ent><br/
<hw>Squire</hw>, <pos>n.</pos> <ety>[Aphetic form of <ets>esquire</ets>.]</ety> <sn>1.</sn> <def>A shield-bearer or armor-bearer who attended a knight.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A title of dignity next in degree below <xex>knight</xex>, and above <xex>gentleman</xex>. See <er>Esquire</er>.</def> <mark>[Eng.]</mark> <ldquo/His privy knights and <xex>squires</xex>.<rdquo/  <rj><au>Chaucer.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>A male attendant on a great personage; also (<mark>Colloq.</mark>), a devoted attendant or follower of a lady; a beau.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>A title of office and courtesy. See under <er>Esquire</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Squire</ent><br/
<hw>Squire</hw>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>squired</conjf> <pr>(skw<imac/rd)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>squiring</conjf>.]</vmorph> <sn>1.</sn> <def>To attend as a squire.</def>  <rj><au>Chaucer.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To attend as a beau, or gallant, for aid and protection; <as>as, to <ex>squire</ex> a lady</as>.</def> <mark>[Colloq.]</mark>  <rj><au>Goldsmith.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Squireen</ent><br/
<hw>Squir*een"</hw> <pr>(skw<imac/r*<emac/n")</pr>, <pos>n.</pos> <def>One who is half squire and half farmer; -- used humorously.</def> <mark>[Eng.]</mark>  <rj><au>C. Kingsley.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Squirehood</ent><br/
<hw>Squire"hood</hw> <pr>(skw<imac/r"h<oocr/d)</pr>, <pos>n.</pos> <def>The rank or state of a squire; squireship.</def>  <rj><au>Swift.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>squireling</ent><br/
<hw>squire"ling</hw> <pr>(-l<icr/ng)</pr>, <pos>n.</pos> <def>A petty squire.</def>  <rj><au>Tennyson.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Squirely</ent><br/
<hw>Squire"ly</hw>, <pos>a. & adv.</pos> <def>Becoming a squire; like a squire.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>squireship</ent><br/
<hw>squire"ship</hw>, <pos>n.</pos> <def>Squirehood.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Squirm</ent><br/
<hw>Squirm</hw> <pr>(skw<etil/rm)</pr>, <pos>v. i.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Squirmed</conjf> <pr>(skw<etil/rmd)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Squirming</conjf>.]</vmorph> <ety>[Cf. <er>Swarm</er> to climb a tree.]</ety> <def>To twist about briskly with contortions like an eel or a worm; to wriggle; to writhe.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Squirr</ent><br/
<hw>Squirr</hw> <pr>(skw<etil/r)</pr>, <pos>v. t.</pos> <def>See <er>Squir</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Squirrel</ent><br/
<hw>Squir"rel</hw> <pr>(skw<etil/r"r<ecr/l <or/ skw<icr/r"-; 277)</pr>, <pos>n.</pos> <ety>[OE. <ets>squirel</ets>, OF. <ets>esquirel</ets>, <ets>escurel</ets>, F. <ets><eacute/cureuil</ets>, LL. <ets>squirelus</ets>, <ets>squirolus</ets>, <ets>scuriolus</ets>, dim. of L. <ets>sciurus</ets>, Gr. <grk>si`oyros</grk>; <grk>skia`</grk> shade + <grk>o'yra`</grk> tail.  Cf. <er>Shine</er>, <pos>v. i.</pos>]</ety> <sn>1.</sn> <fld>(Zool.)</fld> <def>Any one of numerous species of small rodents belonging to the genus <gen>Sciurus</gen> and several allied genera of the family <fam>Sciuridae</fam>. Squirrels generally have a bushy tail, large erect ears, and strong hind legs. They are commonly arboreal in their habits, but many species live in burrows.</def><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ Among the common North American squirrels are the gray squirrel (<spn>Sciurus Carolinensis</spn>) and its black variety; the fox, or cat, squirrel (<spn>Sciurus cinereus</spn>, or <spn>Sciurus niger</spn>) which is a large species, and variable in color, the southern variety being frequently black, while the northern and western varieties are usually gray or rusty brown; the red squirrel (see <er>Chickaree</er>); the striped, or chipping, squirrel (see <er>Chipmunk</er>); and the California gray squirrel (<spn>Sciurus fossor</spn>).  Several other species inhabit Mexico and Central America.  The common European species (<spn>Sciurus vulgaris</spn>) has a long tuft of hair on each ear.  The so-called Australian <xex>squirrels</xex> are marsupials.  See <er>Petaurist</er>, and <er>Phalanger</er>.</note><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>One of the small rollers of a carding machine which work with the large cylinder.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Barking squirrel</b></col> <fld>(Zool.)</fld>, <cd>the prairie dog.</cd> -- <col><b>Federation squirrel</b></col> <fld>(Zool.)</fld>, <cd>the striped gopher. See <er>Gopher</er>, 2.</cd> -- <col><b>Flying squirrel</b></col> <fld>(Zool.)</fld>. <cd>See <er>Flying squirrel</er>, in the Vocabulary.</cd> -- <col><b>Java squirrel</b></col>. <fld>(Zool.)</fld>. <cd>See <er>Jelerang</er>.</cd> -- <col><b>Squirrel corn</b></col> <fld>(Bot.)</fld>, <cd>a North American herb (<spn>Dicentra Canadensis</spn>) bearing little yellow tubers.</cd> -- <col><b>Squirrel cup</b></col> <fld>(Bot.)</fld>, <cd>the blossom of the <spn>Hepatica triloba</spn>, a low perennial herb with cup-shaped flowers varying from purplish blue to pink or even white. It is one of the earliest flowers of spring.</cd> -- <col><b>Squirrel fish</b></col>. <fld>(Zool.)</fld> <sd>(a)</sd> <cd>A sea bass (<spn>Serranus fascicularis</spn>) of the Southern United States.</cd> <sd>(b)</sd> <cd>The sailor's choice (<spn>Diplodus rhomboides</spn>).</cd> <sd>(c)</sd> <cd>The redmouth, or grunt.</cd> <sd>(d)</sd> <cd>A market fish of Bermuda (<spn>Holocentrum Ascensione</spn>).</cd> -- <col><b>Squirrel grass</b></col> <fld>(Bot.)</fld>, <cd>a pestiferous grass (<spn>Hordeum murinum</spn>) related to barley. In California the stiffly awned spikelets work into the wool of sheep, and into the throat, flesh, and eyes of animals, sometimes even producing death.</cd> -- <col><b>Squirrel hake</b></col> <fld>(Zool.)</fld>, <cd>a common American hake (<spn>Phycis tenuis</spn>); -- called also <altname>white hake</altname>.</cd> -- <col><b>Squirrel hawk</b></col> <fld>(Zool.)</fld>, <cd>any rough-legged hawk; especially, the California species <spn>Archibuteo ferrugineus</spn>.</cd> -- <col><b>Squirrel monkey</b></col>. <fld>(Zool.)</fld> <sd>(a)</sd> <cd>Any one of several species of small, soft-haired South American monkeys of the genus <gen>Callithrix</gen>. They are noted for their graceful form and agility. See <er>Teetee</er>.</cd> <sd>(b)</sd> <cd>A marmoset.</cd> -- <col><b>Squirrel petaurus</b></col> <fld>(Zool.)</fld>, <cd>a flying phalanger of Australia. See <er>Phalanger</er>, <er>Petaurist</er>, and <cref>Flying phalanger</cref> under <er>Flying</er>.</cd> -- <col><b>Squirrel shrew</b></col> <fld>(Zool.)</fld>, <cd>any one of several species of East Indian and Asiatic insectivores of the genus <gen>Tupaia</gen>. They are allied to the shrews, but have a bushy tail, like that of a squirrel.</cd> -- <col><b>Squirrel-tail grass</b></col> <fld>(Bot.)</fld>, <cd>a grass (<spn>Hordeum jubatum</spn>) found in salt marshes and along the Great Lakes, having a dense spike beset with long awns.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Squirt</ent><br/
<hw>Squirt</hw> <pr>(skw<etil/rt)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Squirted</conjf>; <pos>p. pr. & vb. n.</pos> <conjf>Squirting</conjf>.]</vmorph> <ety>[Cf. LG. <ets>swirtjen</ets> to squirt, OSw. <ets>sqv<aum/tta</ets>, E. <ets>squander</ets>.]</ety> <def>To drive or eject in a stream out of a narrow pipe or orifice; <as>as, to <ex>squirt</ex> water</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The hard-featured miscreant coolly rolled his tobacco in his cheek, and <qex>squirted</qex> the juice into the fire grate.</q> <rj><qau>Sir W. Scott.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Squirting cucumber</b></col>. <fld>(Bot.)</fld> <cd>See <er>Ecballium</er>.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Squirt</ent><br/
<hw>Squirt</hw>, <pos>v. i.</pos> <sn>1.</sn> <def>To be thrown out, or ejected, in a rapid stream, from a narrow orifice; -- said of liquids.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Hence, to throw out or utter words rapidly; to prate.</def> <mark>[Low]</mark>  <rj><au>L'Estrange.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Squirt</ent><br/
<hw>Squirt</hw>, <pos>n.</pos> <sn>1.</sn> <def>An instrument out of which a liquid is ejected in a small stream with force.</def>  <rj><au>Young.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A small, quick stream; a jet.</def>  <rj><au>Bacon.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <fld>(Hydrodynamics)</fld> <def>The whole system of flow in the vicinity of a source.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><sn>4.</sn> <def>a youngster.</def><br/
[<source>PJC</source>]</p>

<p><sn>5.</sn> <def>a short, overly assertive, or impudent person, especially when young; -- used in contempt.</def><br/
[<source>PJC</source>]</p>

<p><ent>Squirter</ent><br/
<hw>Squirt"er</hw> <pr>(-<etil/r)</pr>, <pos>n.</pos> <def>One who, or that which, squirts.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Squiry</ent><br/
<hw>Squir"y</hw> <pr>(skw<imac/r"<ycr/)</pr>, <pos>n.</pos> <ety>[See <er>Squiery</er>.]</ety> <def>The body of squires, collectively considered; squirarchy.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>The flower of chivalry and <qex>squiry</qex>.</q> <rj><qau>Ld. Berners.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>squish</ent><br/
<hw>squish</hw> <pr>(skw<icr/sh)</pr>, <pos>v. i.</pos> <ety>[Perh. imitative.  Cf. <er>Squelch</er>.]</ety> <def>To make the sound of squirting water, like that made by the feet of one walking in mud or slush; to make a kind of gushing, swashing, or splashing sound; to move with such a sound.</def><br/
[<source>PJC</source>]</p>

<p><ent>squish</ent><br/
<hw>squish</hw> <pr>(skw<icr/sh)</pr>, <pos>n.</pos> <def>A sound like that made by the feet of one walking in mud or slush; a gushing, swashing, or splashing sound; a squishing sound.</def><br/
[<source>PJC</source>]</p>

<p><ent>Squitch grass</ent><br/
<hw>Squitch" grass`</hw> <pr>(skw<icr/ch" gr<adot/s`)</pr>. <fld>(Bot.)</fld> <def>Quitch grass.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Squitee</ent><br/
<hw>Squi*tee"</hw> <pr>(skw<icr/*t<emac/")</pr>, <pos>n.</pos> <ety>[From the N. American Indian name.]</ety> <fld>(Zool.)</fld> <def>The squeteague; -- called also <altname>squit</altname>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stab</ent><br/
<hw>Stab</hw> <pr>(st<acr/b)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Stabbed</conjf> <pr>(st<acr/bd)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Stabbing</conjf>.]</vmorph> <ety>[Cf. OD. <ets>staven</ets> to fix, fasten, fr. <ets>stave</ets>, <ets>staff</ets>, a staff, rod; akin to G. <ets>stab</ets> a staff, stick, E. <ets>staff</ets>; also Gael. <ets>stob</ets> to stab, as n., a stake, a stub.  Cf. <er>Staff</er>.]</ety> <sn>1.</sn> <def>To pierce with a pointed weapon; to wound or kill by the thrust of a pointed instrument; <as>as, to <ex>stab</ex> a man with a dagger</as>; also, to thrust; <as>as, to <ex>stab</ex> a dagger into a person</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Fig.: To injure secretly or by malicious falsehood or slander; <as>as, to <ex>stab</ex> a person's reputation</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stab</ent><br/
<hw>Stab</hw>, <pos>v. i.</pos> <sn>1.</sn> <def>To give a wound with a pointed weapon; to pierce; to thrust with a pointed weapon.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>None shall dare<br/
With shortened sword to <qex>stab</qex> in closer war.</q> <rj><qau>Dryden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To wound or pain, as if with a pointed weapon.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>She speaks poniards, and every word <qex>stabs</qex>.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>To stab at</b></col>, <cd>to offer or threaten to stab; to thrust a pointed weapon at.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stab</ent><br/
<hw>Stab</hw>, <pos>n.</pos> <sn>1.</sn> <def>The thrust of a pointed weapon.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A wound with a sharp-pointed weapon; <as>as, to fall by the <ex>stab</ex> of an assassin</as>.</def>  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>Fig.: An injury inflicted covertly or suddenly; <as>as, a <ex>stab</ex> given to character</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stabat Mater</ent><br/
||<hw>Sta"bat Ma"ter</hw> <pr>(st<amac/"b<acr/t m<amac/"t<etil/r)</pr>. <ety>[L., the mother was standing.]</ety> <def>A celebrated Latin hymn, beginning with these words, commemorating the sorrows of the mother of our Lord at the foot of the cross. It is read in the Mass of the Sorrows of the Virgin Mary, and is sung by Catholics when making <ldquo/the way of the cross<rdquo/ (<xex>Via Crucis</xex>). See <er>Station</er>, 7 <sd>(c)</sd>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stabber</ent><br/
<hw>Stab"ber</hw> <pr>(st<acr/b"b<etil/r)</pr>, <pos>n.</pos> <sn>1.</sn> <def>One who, or that which, stabs; a privy murderer.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Naut.)</fld> <def>A small marline spike; a pricker.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stabbingly</ent><br/
<hw>Stab"bing*ly</hw> <pr>(-b<icr/ng*l<ycr/)</pr>, <pos>adv.</pos> <def>By stabbing; with intent to injure covertly.</def>  <rj><au>Bp. Parker.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stab culture</ent><br/
<hw>Stab culture</hw>. <fld>(Bacteriol.)</fld> <def>A culture made by inoculating a solid medium, as gelatin, with the puncture of a needle or wire; -- called also <altname>stab</altname>. The growths are usually of characteristic form.</def><br/
[<source>Webster 1913 Suppl.</source> <source>+PJC</source>]</p>

<p><ent>Stabiliment</ent><br/
<hw>Sta*bil"i*ment</hw> <pr>(st<adot/*b<icr/l"<icr/*m<eit/nt)</pr>, <pos>n.</pos> <ety>[L. <ets>stabilimentum</ets>, fr. <ets>stabilire</ets> to make firm or stable, fr. <ets>stabilis</ets>. See <er>Stable</er>, <pos>a.</pos>]</ety> <def>The act of making firm; firm support; establishment.</def> <mark>[R.]</mark>  <rj><au>Jer. Taylor.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>They serve for <qex>stabiliment</qex>, propagation, and shade.</q> <rj><qau>Derham.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stabilitate</ent><br/
<hw>Sta*bil"i*tate</hw> <pr>(-t<amac/t)</pr>, <pos>v. t.</pos> <ety>[LL. <ets>stabilitatus</ets>, p. p. of <ets>stabilitare</ets> to make stable.]</ety> <def>To make stable; to establish.</def> <mark>[Obs.]</mark>  <rj><au>Dr. H. More.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stability</ent><br/
<hw>Sta*bil"i*ty</hw> <pr>(st<adot/*b<icr/l"<icr/*t<ycr/)</pr>, <pos>n.</pos> <ety>[L. <ets>stabilitas</ets>; cf. F. <ets>stabilit<eacute/</ets>. See <er>Stable</er>, <pos>a.</pos>]</ety> <sn>1.</sn> <def>The state or quality of being stable, or firm; steadiness; stableness; firmness; strength to stand without being moved or overthrown; <as>as, the <ex>stability</ex> of a structure; the <ex>stability</ex> of a throne or a constitution.</as></def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Steadiness or firmness of character; firmness of resolution or purpose; the quality opposite to <ant>fickleness</ant>, <ant>irresolution</ant>, or <ant>inconstancy</ant>; constancy; steadfastness; <as>as, a man of little <ex>stability</ex>, or of unusual <ex>stability</ex></as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>Fixedness; -- as opposed to <ant>fluidity</ant>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Since fluidness and <qex>stability</qex> are contrary qualities.</q> <rj><qau>Boyle.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><syn><b>Syn.</b> -- Steadiness; stableness; constancy; immovability; firmness.</syn><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stable</ent><br/
<hw>Sta"ble</hw> <pr>(st<amac/"b'l)</pr>, <pos>a.</pos> <ety>[OF. <ets>estable</ets>, F. <ets>stable</ets>, fr. L. <ets>stabilis</ets>, fr. <ets>stare</ets> to stand. See <er>Stand</er>, <pos>v. i.</pos> and cf. <er>Establish</er>.]</ety> <sn>1.</sn> <def>Firmly established; not easily moved, shaken, or overthrown; fixed; <as>as, a <ex>stable</ex> government</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>In this region of chance, . . . where nothing is <qex>stable</qex>.</q> <rj><qau>Rogers.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Steady in purpose; constant; firm in resolution; not easily diverted from a purpose; not fickle or wavering; <as>as, a man of <ex>stable</ex> character</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>And to her husband ever meek and <qex>stable</qex>.</q> <rj><qau>Chaucer.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>Durable; not subject to overthrow or change; firm; <as>as, a <ex>stable</ex> foundation; a <ex>stable</ex> position.</as></def><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <fld>(Physics)</fld> <def>So placed as to resist forces tending to cause motion; of such structure as to resist distortion or molecular or chemical disturbance; -- said of any body or substance.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><cs><col><b>Stable equilibrium</b></col> <fld>(Mech.)</fld>, <cd>the kind of equilibrium of a body so placed that if disturbed it returns to its former position, as in the case when the center of gravity is below the point or axis of support; -- opposed to <contr>unstable equilibrium</contr>, in which the body if disturbed does not tend to return to its former position, but to move farther away from it, as in the case of a body supported at a point below the center of gravity.  Cf. <cref>Neutral equilibrium</cref>, under <er>Neutral</er>.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><syn><b>Syn.</b> -- Fixed; steady; constant; abiding; strong; durable; firm.</syn><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stable</ent><br/
<hw>Sta"ble</hw>, <pos>v. t.</pos> <def>To fix; to establish.</def> <mark>[Obs.]</mark>  <rj><au>Chaucer.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stable</ent><br/
<hw>Sta"ble</hw>, <pos>n.</pos> <ety>[OF. <ets>estable</ets>, F. <ets><eacute/table</ets>, from L. <ets>stabulum</ets>, fr. <ets>stare</ets> to stand. See <er>Stand</er>, <pos>v. i.</pos>]</ety> <def>A house, shed, or building, for beasts to lodge and feed in; esp., a building or apartment with stalls, for horses; <as>as, a horse <ex>stable</ex>; a cow <ex>stable</ex>.</as></def>  <rj><au>Milton.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Stable fly</b></col> <fld>(Zool.)</fld>, <cd>a common dipterous fly (<spn>Stomoxys calcitrans</spn>) which is abundant about stables and often enters dwellings, especially in autumn; called also <altname>biting house fly</altname>.  These flies, unlike the common house flies, which they resemble, bite severely, and are troublesome to horses and cattle.  They differ from the larger <er>horse fly</er>.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stable</ent><br/
<hw>Sta"ble</hw>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Stabled</conjf> <pr>(-b'ld)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Stabling</conjf> <pr>(-bl<icr/ng)</pr>.]</vmorph> <def>To put or keep in a stable.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stable</ent><br/
<hw>Sta"ble</hw>, <pos>v. i.</pos> <def>To dwell or lodge in a stable; to dwell in an inclosed place; to kennel.</def>  <rj><au>Milton.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stableman</ent><br/
<ent>Stableboy</ent><br/
<mhw>{ <hw>Sta"ble*boy`</hw> <pr>(-boi`)</pr>, <hw>Sta"ble*man</hw> <pr>(-m<acr/n)</pr>, }</mhw> <pos>n.</pos> <def>A boy or man who attends in a stable; a groom; a hostler.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stableness</ent><br/
<hw>Sta"ble*ness</hw>, <pos>n.</pos> <def>The quality or state of being stable, or firmly established; stability.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stabler</ent><br/
<hw>Sta"bler</hw> <pr>(-bl<etil/r)</pr>, <pos>n.</pos> <def>A stable keeper.</def>  <rj><au>De Foe.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stable stand</ent><br/
<hw>Sta"ble stand`</hw> <pr>(st<acr/nd`)</pr>. <fld>(O.Eng. Law)</fld> <def>The position of a man who is found at his standing in the forest, with a crossbow or a longbow bent, ready to shoot at a deer, or close by a tree with greyhounds in a leash ready to slip; -- one of the four presumptions that a man intends stealing the king's deer.</def>  <rj><au>Wharton.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stabling</ent><br/
<hw>Sta"bling</hw> <pr>(-bl<icr/ng)</pr>, <pos>n.</pos> <sn>1.</sn> <def>The act or practice of keeping horses and cattle in a stable.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A building, shed, or room for horses and cattle.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stablish</ent><br/
<hw>Stab"lish</hw> <pr>(st<acr/b"l<icr/sh)</pr>, <pos>v. t.</pos> <ety>[Aphetic form of <ets>establish</ets>.]</ety> <def>To settle permanently in a state; to make firm; to establish; to fix.</def> <mark>[Obs.]</mark>  <rj><au>2 Sam. vii. 13.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stablishment</ent><br/
<hw>Stab"lish*ment</hw> <pr>(-m<eit/nt)</pr>, <pos>n.</pos> <def>Establishment.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stably</ent><br/
<hw>Sta"bly</hw> <pr>(st<amac/"bl<ycr/)</pr>, <pos>adv.</pos> <def>In a stable manner; firmly; fixedly; steadily; <as>as, a government <ex>stably</ex> settled</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stabulation</ent><br/
<hw>Stab`u*la"tion</hw> <pr>(st<acr/b`<usl/*l<amac/"sh<ucr/n)</pr>, <pos>n.</pos> <ety>[L. <ets>stabulatio</ets>, fr. <ets>stabulari</ets> to stable cattle, fr. <ets>stabulum</ets>. See <er>Stable</er>, <pos>n.</pos>]</ety> <sn>1.</sn> <def>The act of stabling or housing beasts.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A place for lodging beasts; a stable.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Staccato</ent><br/
||<hw>Stac*ca"to</hw> <pr>(st<adot/k*k<aum/"t<osl/)</pr>, <pos>a.</pos> <ety>[It., p. p. of <ets>staccare</ets>, equivalent to <ets>distaccare</ets>. See <er>Detach</er>.]</ety> <sn>1.</sn> <fld>(Mus.)</fld> <def>Disconnected; separated; distinct; -- a direction to perform the notes of a passage in a short, distinct, and pointed manner.  It is opposed to <contr>legato</contr>, and often indicated by heavy accents written over or under the notes, or by dots when the performance is to be less distinct and emphatic.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Expressed in a brief, pointed manner.</def><br/
[<source>1913 Webster</source>]</p>

<p><q><qex>Staccato</qex> and peremptory [literary criticism].</q> <rj><qau>G. Eliot.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stack</ent><br/
<hw>Stack</hw> <pr>(st<acr/k)</pr>, <pos>n.</pos> <ety>[Icel. <ets>stakkr</ets>; akin to Sw. <ets>stack</ets>, Dan. <ets>stak</ets>.  Cf. <er>Stake</er>.]</ety> <sn>1.</sn> <def>A large and to some degree orderly pile of hay, grain, straw, or the like, usually of a nearly conical form, but sometimes rectangular or oblong, contracted at the top to a point or ridge, and sometimes covered with thatch.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>But corn was housed, and beans were in the <qex>stack</qex>.</q> <rj><qau>Cowper.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <specif>Hence:</specif> <def>An orderly pile of any type of object, indefinite in quantity; -- used especially of piles of wood.  A <ex>stack</ex> is usually more orderly than a <contr>pile</contr></def><br/
[<source>1913 Webster</source> <source>+PJC</source>]</p>

<p><q>Against every pillar was a <qex>stack</qex> of billets above a man's height.</q> <rj><qau>Bacon.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <specif>Specifically:</specif> <def>A pile of wood containing 108 cubic feet.</def> <mark>[Eng.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <specif>Hence:</specif> <def>A large quantity; <as>as, a <ex>stack</ex> of cash</as>.</def> <mark>[Informal]</mark><br/
[<source>PJC</source>]</p>

<p><sn>5.</sn> <fld>(Arch.)</fld> <sd>(a)</sd> <def>A number of flues embodied in one structure, rising above the roof.</def> <specif>Hence:</specif> <sd>(b)</sd> <def>Any single insulated and prominent structure, or upright pipe, which affords a conduit for smoke; <as>as, the brick smoke<ex>stack</ex> of a factory; the smoke<ex>stack</ex> of a steam vessel</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>6.</sn> <fld>(Computer programming)</fld> <sd>(a)</sd> <def>A section of memory in a computer used for temporary storage of data, in which the last datum stored is the first retrieved.</def> <sd>(b)</sd> <def>A data structure within random-access memory used to simulate a hardware stack; <as>as, a push-down <ex>stack</ex></as>.</def><br/
[<source>PJC</source>]</p>

<p><sn>7.</sn> <pluf>pl.</pluf> <def>The section of a library containing shelves which hold books less frequently requested.</def><br/
[<source>PJC</source>]</p>

<p><cs><col><b>Stack of arms</b></col> <fld>(Mil.)</fld>, <cd>a number of muskets or rifles set up together, with the bayonets crossing one another, forming a sort of conical self-supporting pile.</cd> -- <col><b>to blow one's stacks</b></col> <cd>to become very angry and lose one's self-control, and especially to display one's fury by shouting.</cd></cs><br/
[<source>1913 Webster</source> <source>+PJC</source>]</p>

<p><ent>Stack</ent><br/
<hw>Stack</hw>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Stacked</conjf> <pr>(st<acr/kt)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Stacking</conjf>.]</vmorph> <ety>[Cf. Sw. <ets>stacka</ets>, Dan. <ets>stakke</ets>. See <er>Stack</er>, <pos>n.</pos>]</ety> <sn>1.</sn> <def>To lay in a conical or other pile; to make into a large pile; <as>as, to <ex>stack</ex> hay, cornstalks, or grain; to <ex>stack</ex> or place wood.</as></def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <specif>Specifically:</specif> <def>To place in a vertical arrangement so that each item in a pile is resting on top of another item in the pile, except for the bottom item; <as>as, to <ex>stack</ex> the papers neatly on the desk; to <ex>stack</ex> the bricks</as>.</def><br/
[<source>PJC</source>]</p>

<p><sn>3.</sn> <def>To select or arrange dishonestly so as to achieve an unfair advantage; <as>as, to <ex>stack</ex> a deck of cards; to <ex>stack</ex> a jury with persons prejudiced against the defendant</as>.</def><br/
[<source>PJC</source>]</p>

<p><cs><col><b>To stack arms</b></col> <fld>(Mil.)</fld>, <cd>to set up a number of muskets or rifles together, with the bayonets crossing one another, and forming a sort of conical pile.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><-- p. 1399 pr=VMG --></p>

<p><ent>Stackage</ent><br/
<hw>Stack"age</hw> <pr>(st<acr/k"<asl/j)</pr>, <pos>n.</pos> <sn>1.</sn> <def>Hay, grain, or the like, in stacks; things stacked.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A tax on things stacked.</def> <mark>[R.]</mark>  <rj><au>Holinshed.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stacket</ent><br/
<hw>Stack"et</hw> <pr>(-<ecr/t)</pr>, <pos>n.</pos> <ety>[Cf. F. <ets>estacade</ets> and E. <ets>stockade</ets>.]</ety> <fld>(Mil.)</fld> <def>A stockade.</def> <mark>[Scot.]</mark>  <rj><au>Sir W. Scott.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stack-guard</ent><br/
<hw>Stack"-guard`</hw> <pr>(-g<aum/rd`)</pr>, <pos>n.</pos> <def>A covering or protection, as of canvas, for a stack.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stacking</ent><br/
<hw>Stack"ing</hw>, <def><pos>a. & n.</pos> from <er>Stack</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><mcol><col><b>Stacking band</b></col>, <col><b>Stacking belt</b></col></mcol>, <cd>a band or rope used in binding thatch or straw upon a stack.</cd> -- <col><b>Stacking stage</b></col>, <cd>a stage used in building stacks.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stackstand</ent><br/
<hw>Stack"stand`</hw> <pr>(-st<acr/nd`)</pr>, <pos>n.</pos> <def>A staging for supporting a stack of hay or grain; a staddle{2}; a rickstand.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stackyard</ent><br/
<hw>Stack"yard`</hw> <pr>(-y<aum/rd`)</pr>, <pos>n.</pos> <def>A yard or inclosure for stacks of hay or grain.</def>  <rj><au>A. Smith.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stacte</ent><br/
<hw>Stac"te</hw> <pr>(st<acr/k"t<esl/)</pr>, <pos>n.</pos> <ety>[L., fr. Gr. <grk>stakth`</grk>, strictly fem. of <grk>stakto`s</grk> oozing out in drops, fr. <grk>sta`zein</grk> to drop.]</ety> <def>One of the sweet spices used by the ancient Jews in the preparation of incense.  It was perhaps an oil or other form of myrrh or cinnamon, or a kind of storax.</def>  <rj><au>Ex. xxx. 34.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Staddle</ent><br/
<hw>Stad"dle</hw> <pr>(st<acr/d"d'l)</pr>, <pos>n.</pos> <ety>[AS. <ets>sta<edh/ol</ets>, <ets>sta<edh/ul</ets>, a foundation, firm seat; akin to E. <ets>stand</ets>.  <root/163.  See <er>Stand</er>, <pos>v. i.</pos>]</ety> <altsp>[Formerly written <asp>stadle</asp>.]</altsp> <sn>1.</sn> <def>Anything which serves for support; a staff; a prop; a crutch; a cane.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>His weak steps governing<br/
And aged limbs on cypress <qex>stadle</qex> stout.</q> <rj><qau>Spenser.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>The frame of a stack of hay or grain.</def> <mark>[Eng.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>A row of dried or drying hay, etc.</def> <mark>[Eng.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>A small tree of any kind, especially a forest tree.</def><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ In America, trees are called <xex>staddles</xex> from the time that they are three or four years old till they are six or eight inches in diameter, or more. This is also the sense in which the word is used by Bacon and Tusser.</note><br/
[<source>1913 Webster</source>]</p>

<p><ent>Staddle</ent><br/
<hw>Stad"dle</hw>, <pos>v. t.</pos> <sn>1.</sn> <def>To leave the staddles, or saplings, of, as a wood when it is cut.</def> <mark>[R.]</mark>  <rj><au>Tusser.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To form into staddles, as hay.</def> <mark>[Eng.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stade</ent><br/
<hw>Stade</hw> <pr>(st<amac/d)</pr>, <pos>n.</pos> <ety>[Cf. F. <ets>stade</ets>.]</ety> <def>A stadium.</def>  <rj><au>Donne.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stade</ent><br/
<hw>Stade</hw>, <pos>n.</pos> <ety>[Cf. G. <ets>gestade</ets> shore.]</ety> <def>A landing place or wharf.</def>  <rj><au>Knight.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stadimeter</ent><br/
<hw>Sta*dim"e*ter</hw> <pr>(st<adot/*d<icr/m"<esl/*t<etil/r)</pr>, <pos>n.</pos> <ety>[<ets>Stadium</ets> + <ets>-meter</ets>.]</ety> <def>A horizontal graduated bar mounted on a staff, used as a stadium, or telemeter, for measuring distances.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stadia wires</ent><br/
<ent>Stadia hairs</ent><br/
<mhw>{ <hw>Sta"di*a hairs</hw> <or/ <hw>Sta"di*a wires</hw>  }</mhw> <pr>(st<amac/"d<icr/*<adot/ h<acir/rz)</pr>. <fld>(Surv.)</fld> <def>In a theodolite, etc., horizontal cross wires or hairs equidistant from the central horizontal cross wire.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Stadium</ent><br/
||<hw>Sta"di*um</hw> <pr>(st<amac/"d<icr/*<ucr/m)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Stadia</plw> <pr>(st<amac/"d<icr/*<adot/)</pr>.</plu> <ety>[L., a stadium (in sense 1), from Gr. <grk>sta`dion</grk>.]</ety> <sn>1.</sn> <def>A Greek measure of length, being the chief one used for itinerary distances, also adopted by the Romans for nautical and astronomical measurements. It was equal to 600 Greek or 625 Roman feet, or 125 Roman paces, or to 606 feet 9 inches English. This was also called the <altname>Olympic stadium</altname>, as being the exact length of the foot-race course at Olympia.</def>  <rj><au>Dr. W. Smith.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <specif>Hence:</specif> <def>A race course; especially, the Olympic course for foot races.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <specif>Hence:</specif> <def>A modern structure, with its inclosure, resembling the ancient stadium{2}, used for athletic games which are typically played out-of-doors; such stadiums are usually large structures without roofs, though some modern stadiums may have a protective dome overhead.  It may be contrasted with the <contr>arena</contr>, the term commonly used for smaller structures at which indoor games are played.</def><br/
[<source>Webster 1913 Suppl.</source> <source>+PJC</source>]</p>

<p><sn>4.</sn> <def>A kind of telemeter for measuring the distance of an object of known dimensions, by observing the angle it subtends; especially <fld>(Surveying)</fld>, a graduated rod used to measure the distance of the place where it stands from an instrument having a telescope, by observing the number of the graduations of the rod that are seen between certain parallel wires (<xex>stadia wires</xex>) in the field of view of the telescope; -- also called <altname>stadia</altname>, and <altname>stadia rod</altname>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stadtholder</ent><br/
<hw>Stadt"hold`er</hw> <pr>(st<acr/t"h<omac/ld`<etil/r)</pr>, <pos>n.</pos> <ety>[D. <ets>stadhouder</ets>; <ets>stad</ets> a city, a town + <ets>houder</ets> a holder.]</ety> <def>Formerly, the chief magistrate of the United Provinces of Holland; also, the governor or lieutenant governor of a province.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stadtholdership</ent><br/
<ent>Stadtholderate</ent><br/
<mhw>{ <hw>Stadt"hold`er*ate</hw> <pr>(-<asl/t)</pr>, <hw>Stadt"hold`er*ship</hw> <pr>(-sh<icr/p)</pr>, }</mhw> <pos>n.</pos> <def>The office or position of a stadtholder.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stafette</ent><br/
<hw>Sta*fette"</hw> <pr>(st<adot/*f<ecr/t")</pr>, <pos>n.</pos> <ety>[Cf. G. <ets>stafette</ets>. See <er>Estafet</er>.]</ety> <def>An estafet.</def> <mark>[R.]</mark>  <rj><au>Carlyle.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Staff</ent><br/
<hw>Staff</hw> <pr>(st<adot/f)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Staves</plw> <pr>(st<amac/vz <or/ st<aum/vz; 277)</pr> or <plw>Staffs</plw> <pr>(st<adot/fs)</pr> in senses 1-9, <plw>Staffs</plw> in senses 10, 11.</plu> <ety>[AS. <ets>staef</ets> a staff; akin to LG. & D. <ets>staf</ets>, OFries. <ets>stef</ets>, G. <ets>stab</ets>, Icel. <ets>stafr</ets>, Sw. <ets>staf</ets>, Dan. <ets>stav</ets>, Goth. <ets>stabs</ets> element, rudiment, Skr. <ets>sth<amac/pay</ets> to cause to stand, to place. See <er>Stand</er>, and cf. <er>Stab</er>, <er>Stave</er>, <pos>n.</pos>]</ety> <sn>1.</sn> <def>A long piece of wood; a stick; the long handle of an instrument or weapon; a pole or stick, used for many purposes; <as>as, a surveyor's <ex>staff</ex>; the <ex>staff</ex> of a spear or pike.</as></def><br/
[<source>1913 Webster</source>]</p>

<p><q>And he put the <qex>staves</qex> into the rings on the sides of the altar to bear it withal.</q> <rj><qau>Ex. xxxviii. 7.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>With forks and <qex>staves</qex> the felon to pursue.</q> <rj><qau>Dryden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A stick carried in the hand for support or defense by a person walking; hence, a support; that which props or upholds.</def> <ldquo/Hooked <xex>staves</xex>.<rdquo/  <rj><au>Piers Plowman.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>The boy was the very <qex>staff</qex> of my age.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>He spoke of it [beer] in <ldquo/The Earnest Cry,<rdquo/ and likewise in the <ldquo/Scotch Drink,<rdquo/ as one of the <qex>staffs</qex> of life which had been struck from the poor man's hand.</q> <rj><qau>Prof. Wilson.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>A pole, stick, or wand borne as an ensign of authority; a badge of office; <as>as, a constable's <ex>staff</ex></as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Methought this <qex>staff</qex>, mine office badge in court,<br/
Was broke in twain.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>All his officers brake their <qex>staves</qex>; but at their return new <qex>staves</qex> were delivered unto them.</q> <rj><qau>Hayward.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>A pole upon which a flag is supported and displayed.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <def>The round of a ladder.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>I ascended at one [ladder] of six hundred and thirty-nine <qex>staves</qex>.</q> <rj><qau>Dr. J. Campbell (E. Brown's Travels).</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>6.</sn> <def>A series of verses so disposed that, when it is concluded, the same order begins again; a stanza; a stave.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Cowley found out that no kind of <qex>staff</qex> is proper for an heroic poem, as being all too lyrical.</q> <rj><qau>Dryden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>7.</sn> <fld>(Mus.)</fld> <def>The five lines and the spaces on which music is written; -- formerly called <altname>stave</altname>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>8.</sn> <fld>(Mech.)</fld> <def>An arbor, as of a wheel or a pinion of a watch.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>9.</sn> <fld>(Surg.)</fld> <def>The grooved director for the gorget, or knife, used in cutting for stone in the bladder.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>10.</sn> <ety>[From <er>Staff</er>, 3, a badge of office.]</ety> <fld>(Mil.)</fld> <def>An establishment of officers in various departments attached to an army, to a section of an army, or to the commander of an army. The general's staff consists of those officers about his person who are employed in carrying his commands into execution. See <er><Eacute/tat Major</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>11.</sn> <def>Hence: A body of assistants serving to carry into effect the plans of a superintendent or manager; sometimes used for the entire group of employees of an enterprise, excluding the top management; <as>as, the <ex>staff</ex> of a newspaper</as>.</def><br/
[<source>1913 Webster</source> <source>+PJC</source>]</p>

<p><cs><col><b>Jacob's staff</b></col> <fld>(Surv.)</fld>, <cd>a single straight rod or staff, pointed and iron-shod at the bottom, for penetrating the ground, and having a socket joint at the top, used, instead of a tripod, for supporting a compass.</cd> -- <col><b>Staff angle</b></col> <fld>(Arch.)</fld>, <cd>a square rod of wood standing flush with the wall on each of its sides, at the external angles of plastering, to prevent their being damaged.</cd> -- <col><b>The staff of life</b></col>, <cd>bread.</cd> <ldquo/Bread is <xex>the staff of life</xex>.<rdquo/ <au>Swift.</au> -- <col><b>Staff tree</b></col> <fld>(Bot.)</fld>, <cd>any plant of the genus <gen>Celastrus</gen>, mostly climbing shrubs of the northern hemisphere.  The American species (<spn>Celastrus scandens</spn>) is commonly called <stype>bittersweet</stype>. See 2d <er>Bittersweet</er>, 3 <sd>(b)</sd>.</cd> -- <mcol><col><b>To set up one's staff</b></col>, <col><b>To put up one's staff</b></col>, <col><b>To set down one's staff</b></col>  <it>or</it> <col><b>To put down one's staff</b></col></mcol>, <cd>to take up one's residence; to lodge.</cd> <mark>[Obs.]</mark></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Staff</ent><br/
<hw>Staff</hw> <pr>(st<adot/f)</pr>, <pos>n.</pos> <ety>[G. <ets>staffiren</ets> to fill or fit out, adorn, fr. D. <ets>stoffeeren</ets>, OF. <ets>estoffer</ets>, F. <ets><eacute/toffer</ets>, fr. OF. <ets>estoffe</ets> stuff, F. <ets><eacute/toffe</ets>. See <er>Stuff</er>, <pos>n.</pos>]</ety> <fld>(Arch.)</fld> <def>Plaster combined with fibrous and other materials so as to be suitable for sculpture in relief or in the round, or for forming flat plates or boards of considerable size which can be nailed to framework to make the exterior of a larger structure, forming joints which may afterward be repaired and concealed with fresh plaster.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Staffier</ent><br/
<hw>Staf"fi*er</hw> <pr>(st<adot/f"f<icr/*<etil/r)</pr>, <pos>n.</pos> <def>An attendant bearing a staff.</def> <mark>[Obs.]</mark> <ldquo/<xex>Staffiers</xex> on foot.<rdquo/  <rj><au>Hudibras.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Staffish</ent><br/
<hw>Staff"ish</hw> <pr>(st<adot/f"<icr/sh)</pr>, <pos>a.</pos> <def>Stiff; harsh.</def> <mark>[Obs.]</mark>  <rj><au>Ascham.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Staffman</ent><br/
<hw>Staff"man</hw> <pr>(st<adot/f"m<ait/n)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Staffmen</plw> <pr>(-m<eit/n)</pr>.</plu> <def>A workman employed in silk throwing.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stag</ent><br/
<hw>Stag</hw> <pr>(st<acr/g)</pr>, <pos>n.</pos> <ety>[Icel. <ets>steggr</ets> the male of several animals; or a doubtful AS. <ets>stagga</ets>.  Cf. <er>Steg</er>.]</ety> <sn>1.</sn> <fld>(Zool.)</fld> <sd>(a)</sd> <def>The adult male of the red deer (<spn>Cervus elaphus</spn>), a large European species closely related to the American elk, or wapiti.</def> <sd>(b)</sd> <def>The male of certain other species of large deer.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A colt, or filly; also, a romping girl.</def> <mark>[Prov. Eng.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>A castrated bull; -- called also <altname>bull stag</altname>, and <altname>bull seg</altname>. See the Note under <er>Ox</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <fld>(Stock Exchange)</fld> <sd>(a)</sd> <def>An outside irregular dealer in stocks, who is not a member of the exchange.</def> <mark>[Cant]</mark> <sd>(b)</sd> <def>One who applies for the allotment of shares in new projects, with a view to sell immediately at a premium, and not to hold the stock.</def> <mark>[Cant]</mark><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <fld>(Zool.)</fld> <def>The European wren.</def> <mark>[Prov. Eng.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Stag beetle</b></col> <fld>(Zool.)</fld>, <cd>any one of numerous species of lamellicorn beetles belonging to <gen>Lucanus</gen> and allied genera, especially <spn>Lucanus cervus</spn> of Europe and <spn>Lucanus dama</spn> of the United States.  The mandibles are large and branched, or forked, whence the name.  The larva feeds on the rotten wood of dead trees.  Called also <altname>horned bug</altname>, and <altname>horse beetle</altname>.</cd> -- <col><b>Stag dance</b></col>, <cd>a dance by men only.</cd> <mark>[Slang, U.S.]</mark> -- <col><b>Stag hog</b></col> <fld>(Zool.)</fld>, <cd>the babiroussa.</cd> -- <col><b>Stag-horn coral</b></col> <fld>(Zool.)</fld>, <cd>any one of several species of large branching corals of the genus <gen>Madrepora</gen>, which somewhat resemble the antlers of the stag, especially <spn>Madrepora cervicornis</spn>, and <spn>Madrepora palmata</spn>, of Florida and the West Indies.</cd> -- <col><b>Stag-horn fern</b></col> <fld>(Bot.)</fld>, <cd>an Australian and West African fern (<spn>Platycerium alcicorne</spn>) having the large fronds branched like a stag's horns; also, any species of the same genus.</cd> -- <col><b>Stag-horn sumac</b></col> <fld>(Bot.)</fld>, <cd>a common American shrub (<spn>Rhus typhina</spn>) having densely velvety branchlets. See <er>Sumac</er>.</cd> -- <col><b>Stag party</b></col>, <cd>a party consisting of men only.</cd> <mark>[Slang, U. S.]</mark> -- <col><b>Stag tick</b></col> <fld>(Zool.)</fld>, <cd>a parasitic dipterous insect of the family <fam>Hippoboscidae</fam>, which lives upon the stag and is usually wingless.  The same species lives also upon the European grouse, but in that case has wings.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stag</ent><br/
<hw>Stag</hw>, <pos>v. i.</pos> <fld>(Com.)</fld> <def>To act as a <ldquo/stag,<rdquo/ or irregular dealer in stocks.</def> <mark>[Cant]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stag</ent><br/
<hw>Stag</hw>, <pos>v. t.</pos> <def>To watch; to dog, or keep track of.</def> <mark>[Prov. Eng. or Slang]</mark>  <rj><au>H. Kingsley.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stage</ent><br/
<hw>Stage</hw> <pr>(st<amac/j)</pr>, <pos>n.</pos> <ety>[OF. <ets>estage</ets>, F. <ets><eacute/tage</ets>, (assumed) LL. <ets>staticum</ets>, from L. <ets>stare</ets> to stand.  See <er>Stand</er>, and cf. <er>Static</er>.]</ety> <sn>1.</sn> <def>A floor or story of a house.</def> <mark>[Obs.]</mark>  <rj><au>Wyclif.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>An elevated platform on which an orator may speak, a play be performed, an exhibition be presented, or the like.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>A floor elevated for the convenience of mechanical work, or the like; a scaffold; a staging.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>A platform, often floating, serving as a kind of wharf.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <def>The floor for scenic performances; hence, the theater; the playhouse; hence, also, the profession of representing dramatic compositions; the drama, as acted or exhibited.</def>
<-- stars of stage and screen --><br/
[<source>1913 Webster</source>]</p>

<p><q>Knights, squires, and steeds, must enter on the <qex>stage</qex>.</q> <rj><qau>Pope.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Lo! where the <qex>stage</qex>, the poor, degraded <qex>stage</qex>,<br/
Holds its warped mirror to a gaping age.</q> <rj><qau>C. Sprague.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>6.</sn> <def>A place where anything is publicly exhibited; the scene of any noted action or career; the spot where any remarkable affair occurs; <as>as, politicians must live their lives on the public <ex>stage</ex></as>.</def><br/
[<source>1913 Webster</source> <source>+PJC</source>]</p>

<p><q>When we are born, we cry that we are come<br/
To this great <qex>stage</qex> of fools.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Music and ethereal mirth<br/
Wherewith the <qex>stage</qex> of air and earth did ring.</q> <rj><qau>Miton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>7.</sn> <def>The platform of a microscope, upon which an object is placed to be viewed. See <xex>Illust.</xex> of <er>Microscope</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>8.</sn> <def>A place of rest on a regularly traveled road; a stage house; a station; a place appointed for a relay of horses.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>9.</sn> <def>A degree of advancement in a journey; one of several portions into which a road or course is marked off; the distance between two places of rest on a road; <as>as, a <ex>stage</ex> of ten miles</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>A <qex>stage</qex> . . . signifies a certain distance on a road.</q> <rj><qau>Jeffrey.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>He traveled by gig, with his wife, his favorite horse performing the journey by easy <qex>stages</qex>.</q> <rj><qau>Smiles.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>10.</sn> <def>A degree of advancement in any pursuit, or of progress toward an end or result.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Such a polity is suited only to a particular <qex>stage</qex> in the progress of society.</q> <rj><qau>Macaulay.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>11.</sn> <def>A large vehicle running from station to station for the accommodation of the public; a stagecoach; an omnibus.</def> <ldquo/A parcel sent you by the <xex>stage</xex>.<rdquo/  <rj><au>Cowper.</au></rj> <mark>[Obsolescent]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>I went in the sixpenny <qex>stage</qex>.</q> <rj><qau>Swift.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>12.</sn> <fld>(Biol.)</fld> <def>One of several marked phases or periods in the development and growth of many animals and plants; <as>as, the larval <ex>stage</ex>; pupa <ex>stage</ex>; zoea <ex>stage</ex>.</as></def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Stage box</b></col>, <cd>a box close to the stage in a theater.</cd> -- <col><b>Stage carriage</b></col>, <cd>a stagecoach.</cd> -- <col><b>Stage door</b></col>, <cd>the actors' and workmen's entrance to a theater.</cd> -- <col><b>Stage lights</b></col>, <cd>the lights by which the stage in a theater is illuminated.</cd> -- <col><b>Stage micrometer</b></col>, <cd>a graduated device applied to the stage of a microscope for measuring the size of an object.</cd> -- <col><b>Stage wagon</b></col>, <cd>a wagon which runs between two places for conveying passengers or goods.</cd> -- <col><b>Stage whisper</b></col>, <cd>a loud whisper, as by an actor in a theater, supposed, for dramatic effect, to be unheard by one or more of his fellow actors, yet audible to the audience; an aside.</cd></cs>
<-- <col><b>stage of the game</b></col>, [Colloq.] stage n. 10. --><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stage</ent><br/
<hw>Stage</hw> <pr>(st<amac/j)</pr>, <pos>v. t.</pos> <def>To exhibit upon a stage, or as upon a stage; to display publicly.</def>  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stagecoach</ent><br/
<hw>Stage"coach`</hw> <pr>(st<amac/j"k<omac/ch`)</pr>, <pos>n.</pos> <def>A coach that runs regularly from one stage, station, or place to another, for the conveyance of passengers.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stagecoachman</ent><br/
<hw>Stage"coach`man</hw> <pr>(st<amac/j"k<omac/ch`m<ait/n)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Stagecoachmen</plw> <pr>(st<amac/j"k<omac/ch`m<eit/n)</pr>.</plu> <def>One who drives a stagecoach.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stage director</ent><br/
<hw>Stage director</hw>. <fld>(Theat.)</fld> <def>One who prepares a play for production. He arranges the details of the stage settings, the business to be used, all stage effects, and instructs the actors, excepting usually the star, in the general interpretation of their parts.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Stage fright</ent><br/
<hw>Stage fright</hw>. <def>Nervousness felt before an audience.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Stagehouse</ent><br/
<hw>Stage"house`</hw> <pr>(-hous`)</pr>, <pos>n.</pos> <def>A house where a stage regularly stops for passengers or a relay of horses.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stagely</ent><br/
<hw>Stage"ly</hw>, <pos>a.</pos> <def>Pertaining to a stage; becoming the theater; theatrical.</def> <mark>[Obs.]</mark>  <rj><au>Jer. Taylor.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stage manager</ent><br/
<hw>Stage manager</hw>. <fld>(Theat.)</fld> <def>One in control of the stage during the production of a play. He directs the stage hands, property man, etc., has charge of all details behind the curtain, except the acting, and has a general oversight of the actors. Sometimes he is also the <er>stage director</er>.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Stageplay</ent><br/
<hw>Stage"play`</hw> <pr>(-pl<amac/`)</pr>, <pos>n.</pos> <def>A dramatic or theatrical entertainment.</def>  <rj><au>Dryden.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stageplayer</ent><br/
<hw>Stage"play`er</hw> <pr>(-<etil/r)</pr>, <pos>n.</pos> <def>An actor on the stage; one whose occupation is to represent characters on the stage; <as>as, <person>Garrick was</person> a celebrated <ex>stageplayer</ex></as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stager</ent><br/
<hw>Sta"ger</hw> <pr>(st<amac/"j<etil/r)</pr>, <pos>n.</pos> <sn>1.</sn> <def>A player.</def> <mark>[R.]</mark>  <rj><au>B. Jonson.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>One who has long acted on the stage of life; a practitioner; a person of experience, or of skill derived from long experience.</def> <ldquo/You will find most of the old <xex>stagers</xex> still stationary there.<rdquo/  <rj><au>Sir W. Scott.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>A horse used in drawing a stage.</def> <mark>[Colloq.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stagery</ent><br/
<hw>Sta"ger*y</hw> <pr>(-<ycr/)</pr>, <pos>n.</pos> <def>Exhibition on the stage.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stage-struck</ent><br/
<hw>Stage"-struck`</hw> <pr>(st<amac/j"str<ucr/k`)</pr>, <pos>a.</pos> <def>Fascinated by the stage; seized by a passionate desire to become an actor.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stag-evil</ent><br/
<hw>Stag"-e`vil</hw> <pr>(st<acr/g"<emac/`v'l)</pr>, <pos>n.</pos> <fld>(Far.)</fld> <def>A kind of palsy affecting the jaw of a horse.</def>  <rj><au>Crabb.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Staggard</ent><br/
<hw>Stag"gard</hw> <pr>(-g<etil/rd)</pr>, <pos>n.</pos> <ety>[From <er>Stag</er>.]</ety> <fld>(Zool.)</fld> <def>The male red deer when four years old.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stagger</ent><br/
<hw>Stag"ger</hw> <pr>(-g<etil/r)</pr>, <pos>v. i.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Staggered</conjf> <pr>(-g<etil/rd)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Staggering</conjf>.]</vmorph> <ety>[OE. <ets>stakeren</ets>, Icel. <ets>stakra</ets> to push, to stagger, fr. <ets>staka</ets> to punt, push, stagger; cf. OD. <ets>staggeren</ets> to stagger.  Cf. <er>Stake</er>, <pos>n.</pos>]</ety> <sn>1.</sn> <def>To move to one side and the other, as if about to fall, in standing or walking; not to stand or walk with steadiness; to sway; to reel or totter.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Deep was the wound; he <qex>staggered</qex> with the blow.</q> <rj><qau>Dryden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To cease to stand firm; to begin to give way; to fail.</def> <ldquo/The enemy <xex>staggers</xex>.<rdquo/  <rj><au>Addison.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>To begin to doubt and waver in purpose; to become less confident or determined; to hesitate.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>He [Abraham] <qex>staggered</qex> not at the promise of God through unbelief.</q> <rj><qau>Rom. iv. 20.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stagger</ent><br/
<hw>Stag"ger</hw>, <pos>v. t.</pos> <sn>1.</sn> <def>To cause to reel or totter.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>That hand shall burn in never-quenching fire<br/
That <qex>staggers</qex> thus my person.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To cause to doubt and waver; to make to hesitate; to make less steady or confident; to shock.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Whosoever will read the story of this war will find himself much <qex>staggered</qex>.</q> <rj><qau>Howell.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Grants to the house of Russell were so enormous, as not only to outrage economy, but even to <qex>stagger</qex> credibility.</q> <rj><qau>Burke.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>To arrange (a series of parts) on each side of a median line alternately, as the spokes of a wheel or the rivets of a boiler seam.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stagger</ent><br/
<hw>Stag"ger</hw>, <pos>n.</pos> <sn>1.</sn> <def>An unsteady movement of the body in walking or standing, as if one were about to fall; a reeling motion; vertigo; -- often in the plural; <as>as, the <ex>stagger</ex> of a drunken man</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <pluf>pl.</pluf> <fld>(Far.)</fld> <def>A disease of horses and other animals, attended by reeling, unsteady gait or sudden falling; <as>as, parasitic <ex>staggers</ex>; apopletic or sleepy <ex>staggers</ex>.</as></def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <pluf>pl.</pluf> <def>Bewilderment; perplexity.</def> <mark>[R.]</mark>  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Stomach staggers</b></col> <fld>(Far.)</fld>, <cd>distention of the stomach with food or gas, resulting in indigestion, frequently in death.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Staggerbush</ent><br/
<hw>Stag"ger*bush`</hw> <pr>(-b<usdot/sh`)</pr>, <pos>n.</pos> <fld>(Bot.)</fld> <def>An American shrub (<spn>Andromeda Mariana</spn>) having clusters of nodding white flowers. It grows in low, sandy places, and is said to poison lambs and calves.</def>  <rj><au>Gray.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Staggeringly</ent><br/
<hw>Stag"ger*ing*ly</hw>, <pos>adv.</pos> <def>In a staggering manner.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Staggerwort</ent><br/
<hw>Stag"ger*wort`</hw> <pr>(-w<ucir/rt`)</pr>, <pos>n.</pos> <fld>(Bot.)</fld> <def>A kind of ragwort (<spn>Senecio Jacobaea</spn>).</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stag-horn fern</ent><br/
<ent>Stag-horn coral</ent><br/
<mhw>{ <hw>Stag"-horn` cor"al</hw> <pr>(st<acr/g"h<ocir/rn` k<ocr/r"<ait/l)</pr>, <hw>Stag"-horn` fern`</hw> <pr>(f<etil/rn`)</pr>, etc. }</mhw> <def>See under <er>Stag</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stag-horned</ent><br/
<hw>Stag"-horned`</hw> <pr>(-h<ocir/rnd`)</pr>, <pos>a.</pos> <fld>(Zool.)</fld> <def>Having the mandibles large and palmate, or branched somewhat like the antlers of a stag; -- said of certain beetles.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Staghound</ent><br/
<hw>Stag"hound`</hw> <pr>(-hound`)</pr>, <pos>n.</pos> <fld>(Zool.)</fld> <def>A large and powerful hound formerly used in hunting the stag, the wolf, and other large animals. The breed is nearly extinct.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Staging</ent><br/
<hw>Sta"ging</hw> <pr>(st<amac/"j<icr/ng)</pr>, <pos>n.</pos> <def>A structure of posts and boards for supporting workmen, etc., as in building.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>The business of running stagecoaches; also, the act of journeying in stagecoaches.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stagirite</ent><br/
<hw>Stag"i*rite</hw> <pr>(st<acr/j"<icr/*r<imac/t)</pr>, <pos>n.</pos> <def>A native of, or resident in, <xex>Stagira</xex>, in ancient Macedonia; especially, Aristotle.</def> <altsp>[Written also <asp>Stagyrite</asp>.]</altsp><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stagnancy</ent><br/
<hw>Stag"nan*cy</hw> <pr>(st<acr/g"n<ait/n*s<ycr/)</pr>, <pos>n.</pos> <def>State of being stagnant.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stagnant</ent><br/
<hw>Stag"nant</hw> <pr>(-n<ait/nt)</pr>, <pos>a.</pos> <ety>[L. <ets>stagnans</ets>, <ets>-antis</ets>, p. pr. of <ets>stagnare</ets>. See <er>Stagnate</er>.]</ety> <sn>1.</sn> <def>That stagnates; not flowing; not running in a current or steam; motionless; hence, impure or foul from want of motion; <as>as, a <ex>stagnant</ex> lake or pond; <ex>stagnant</ex> blood in the veins.</as></def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Not active or brisk; dull; <as>as, business is <ex>stagnant</ex></as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>That gloomy slumber of the <qex>stagnant</qex> soul.</q> <rj><qau>Johnson.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>For him a <qex>stagnant</qex> life was not worth living.</q> <rj><qau>Palfrey.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stagnantly</ent><br/
<hw>Stag"nant*ly</hw>, <pos>adv.</pos> <def>In a stagnant manner.</def><br/
[<source>1913 Webster</source>]</p>

<p><-- p. 1400 pr=VMG --></p>

<p><ent>Stagnate</ent><br/
<hw>Stag"nate</hw> <pr>(st<acr/g"n<amac/t)</pr>, <pos>v. i.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Stagnated</conjf> <pr>(-n<asl/*t<ecr/d)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Stagnating</conjf>.]</vmorph> <ety>[L. <ets>stagnatus</ets>, p. p. of <ets>stagnare</ets> to stagnate, make stagnant, from <ets>stagnum</ets> a piece of standing water. See <er>Stank</er> a pool, and cf. <er>Stanch</er>, <pos>v. t.</pos>]</ety> <sn>1.</sn> <def>To cease to flow; to be motionless; <as>as, blood <ex>stagnates</ex> in the veins of an animal</as>; hence, to become impure or foul by want of motion; <as>as, air <ex>stagnates</ex> in a close room</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To cease to be brisk or active; to become dull or inactive; <as>as, commerce <ex>stagnates</ex>; business <ex>stagnates</ex></as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Ready-witted tenderness . . . never <qex>stagnates</qex> in vain lamentations while there is any room for hope.</q> <rj><qau>Sir W. Scott.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stagnate</ent><br/
<hw>Stag"nate</hw> <pr>(-n<asl/t)</pr>, <pos>a.</pos> <def>Stagnant.</def> <mark>[Obs.]</mark> <ldquo/A <xex>stagnate</xex> mass of vapors.<rdquo/  <rj><au>Young.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stagnation</ent><br/
<hw>Stag*na"tion</hw> <pr>(-n<amac/"sh<ucr/n)</pr>, <pos>n.</pos> <ety>[Cf. F. <ets>stagnation</ets>.]</ety> <sn>1.</sn> <def>The condition of being stagnant; cessation of flowing or circulation, as of a fluid; the state of being motionless; <as>as, the <ex>stagnation</ex> of the blood; the <ex>stagnation</ex> of water or air; the <ex>stagnation</ex> of vapors</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>The cessation of action, or of brisk action; the state of being dull; <as>as, the <ex>stagnation</ex> of business</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stagworm</ent><br/
<hw>Stag"worm`</hw> <pr>(st<acr/g"w<ucir/rm`)</pr>, <pos>n.</pos> <fld>(Zool.)</fld> <def>The larva of any species of botfly which is parasitic upon the stag, such as <spn>Oestrus actaeon</spn>, or <spn>Hypoderma actaeon</spn>, which burrows beneath the skin, and <spn>Cephalomyia auribarbis</spn>, which lives in the nostrils.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stagy</ent><br/
<hw>Sta"gy</hw> <pr>(st<amac/"j<ycr/)</pr>, <pos>a.</pos> <altsp>[Written also <asp>stagey</asp>.]</altsp> <def>Having an air or manner characteristic of the stage; theatrical; artificial; <as>as, a <ex>stagy</ex> tone or bearing</as>; -- chiefly used depreciatively.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Stahlian</ent><br/
<hw>Stahl"ian</hw> <pr>(st<aum/l"y<ait/n)</pr>, <pos>a.</pos> <def>Pertaining to, or taught by, <persfn><etsep>Stahl</etsep></persfn>, a German physician and chemist of the 17th century; <as>as, the <ex>Stahlian</ex> theory of phlogiston</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stahlian</ent><br/
<hw>Stahl"ian</hw>, <pos>n.</pos> <def>A believer in, or advocate of, Stahlism.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stahlianism</ent><br/
<ent>Stahlism</ent><br/
<mhw>{ <hw>Stahl"ism</hw> <pr>(st<aum/l"<icr/z'm)</pr>, <hw>Stahl"ian*ism</hw> <pr>(-y<ait/n*<icr/z'm)</pr>, }</mhw> <pos>n.</pos> <def>The Stahlian theory, that every vital action is a function or operation of the soul.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Staid</ent><br/
<hw>Staid</hw> <pr>(st<amac/d)</pr>, <def><pos>imp. & p. p.</pos> of <er>Stay</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Staid</ent><br/
<hw>Staid</hw>, <pos>a.</pos> <ety>[From <er>Stay</er> to stop.]</ety> <def>Sober; grave; steady; sedate; composed; regular; not wild, volatile, flighty, or fanciful.</def> <ldquo/Sober and <xex>staid</xex> persons.<rdquo/  <rj><au>Addison.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>O'erlaid with black, <qex>staid</qex> Wisdom's hue.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><syn><b>Syn.</b> -- Sober; grave; steady; steadfast; composed; regular; sedate.</syn><br/
[<source>1913 Webster</source>]</p>

<p><ent>Staidly</ent><br/
<hw>Staid"ly</hw>, <pos>adv.</pos> <def>In a staid manner, sedately.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Staidness</ent><br/
<hw>Staid"ness</hw>, <pos>n.</pos> <def>The quality or state of being staid; seriousness; steadiness; sedateness; regularity; -- the opposite of <contr>wildness</contr>, or <ant>levity</ant>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>If sometimes he appears too gay, yet a secret gracefulness of youth accompanies his writings, though the <qex>staidness</qex> and sobriety of age be wanting.</q> <rj><qau>Dryden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><syn><b>Syn.</b> -- Sobriety; gravity; steadiness; regularity; constancy; firmness; stability; sedateness.</syn><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stail</ent><br/
<hw>Stail</hw> <pr>(st<amac/l)</pr>, <pos>n.</pos> <def>A handle, as of a mop; a stale.</def> <mark>[Eng.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stain</ent><br/
<hw>Stain</hw> <pr>(st<amac/n)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Stained</conjf> <pr>(st<amac/nd)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Staining</conjf>.]</vmorph> <ety>[Abbrev. fr. <ets>distain</ets>.]</ety> <sn>1.</sn> <def>To discolor by the application of foreign matter; to make foul; to spot; <as>as, to <ex>stain</ex> the hand with dye; armor <ex>stained</ex> with blood</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To color, as wood, glass, paper, cloth, or the like, by processes affecting, chemically or otherwise, the material itself; to tinge with a color or colors combining with, or penetrating, the substance; to dye; <as>as, to <ex>stain</ex> wood with acids, colored washes, paint rubbed in, etc.; to <ex>stain</ex> glass</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>To spot with guilt or infamy; to bring reproach on; to blot; to soil; to tarnish.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Of honor void,<br/
Of innocence, of faith, of purity,<br/
Our wonted ornaments now soiled and <qex>stained</qex>.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>To cause to seem inferior or soiled by comparison.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>She <qex>stains</qex> the ripest virgins of her age.</q> <rj><qau>Beau. & Fl.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>That did all other beasts in beauty <qex>stain</qex>.</q> <rj><qau>Spenser.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Stained glass</b></col>, <cd>glass colored or stained by certain metallic pigments fused into its substance, -- often used for making ornamental windows.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><syn><b>Syn.</b> -- To paint; dye; blot; soil; sully; discolor; disgrace; taint.</syn>  -- <usage><er>Paint</er>, <er>Stain</er>, <er>Dye</er>. These denote three different processes; the first mechanical, the other two, chiefly chemical. To <xex>paint</xex> a thing is to spread a coat of coloring matter over it; to <xex>stain</xex> or <xex>dye</xex> a thing is to impart color to its substance. To <xex>stain</xex> is said chiefly of solids, as wood, glass, paper; to <xex>dye</xex>, of fibrous substances, textile fabrics, etc.; the one, commonly, a simple process, as applying a wash; the other more complex, as fixing colors by mordants.</usage><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stain</ent><br/
<hw>Stain</hw>, <pos>v. i.</pos> <def>To give or receive a stain; to grow dim.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stain</ent><br/
<hw>Stain</hw>, <pos>n.</pos> <sn>1.</sn> <def>A discoloration by foreign matter; a spot; <as>as, a <ex>stain</ex> on a garment or cloth</as>.</def>  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A natural spot of a color different from the gound.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Swift trouts, diversified with crimson <qex>stains</qex>.</q> <rj><qau>Pope.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>Taint of guilt; tarnish; disgrace; reproach.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Nor death itself can wholly wash their <qex>stains</qex>.</q> <rj><qau>Dryden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Our opinion . . . is, I trust, without any blemish or <qex>stain</qex> of heresy.</q> <rj><qau>Hooker.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>Cause of reproach; shame.</def>  <rj><au>Sir P. Sidney.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <def>A tincture; a tinge.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>You have some <qex>stain</qex> of soldier in you.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><syn><b>Syn.</b> -- Blot; spot; taint; pollution; blemish; tarnish; color; disgrace; infamy; shame.</syn><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stainer</ent><br/
<hw>Stain"er</hw> <pr>(-<etil/r)</pr>, <pos>n.</pos> <sn>1.</sn> <def>One who stains or tarnishes.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A workman who stains; <as>as, a <ex>stainer</ex> of wood</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stainless</ent><br/
<hw>Stain"less</hw>, <pos>a.</pos> <def>Free from stain; immaculate.</def>  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>The very care he took to keep his name<br/
<qex>Stainless</qex>, with some was evidence of shame.</q> <rj><qau>Crabbe.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><syn><b>Syn.</b> -- Blameless; spotless; faultless. See <er>Blameless</er>.</syn><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stainlessly</ent><br/
<hw>Stain"less*ly</hw>, <pos>adv.</pos> <def>In a stainless manner.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stair</ent><br/
<hw>Stair</hw> <pr>(st<acir/r)</pr>, <pos>n.</pos> <ety>[OE. <ets>steir</ets>, <ets>steyer</ets>, AS. <ets>st<aemac/ger</ets>, from <ets>st<imac/gan</ets> to ascend, rise.  <root/164. See <er>Sty</er> to ascend.]</ety> <sn>1.</sn> <def>One step of a series for ascending or descending to a different level; -- commonly applied to those within a building.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A series of steps, as for passing from one story of a house to another; -- commonly used in the plural; but originally used in the singular only.</def> <ldquo/I a winding <xex>stair</xex> found.<rdquo/  <rj><au>Chaucer's Dream.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Below stairs</b></col>, <cd>in the basement or lower part of a house, where the servants are.</cd> -- <col><b>Flight of stairs</b></col>, <cd>the stairs which make the whole ascent of a story.</cd> -- <col><b>Pair of stairs</b></col>, <cd>a set or flight of stairs. -- <xex>pair</xex>, in this phrase, having its old meaning of a <xex>set</xex>. See <er>Pair</er>, <pos>n.</pos>, 1.</cd> -- <col><b>Run of stairs</b></col> <fld>(Arch.)</fld>, <cd>a single set of stairs, or section of a stairway, from one platform to the next.</cd> -- <col><b>Stair rod</b></col>, <cd>a rod, usually of metal, for holding a stair carpet to its place.</cd> -- <col><b>Up stairs</b></col>. <cd>See <er>Upstairs</er> in the Vocabulary.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Staircase</ent><br/
<hw>Stair"case`</hw> <pr>(st<acir/r"k<amac/s`)</pr>, <pos>n.</pos> <def>A flight of stairs with their supporting framework, casing, balusters, etc.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>To make a complete <qex>staircase</qex> is a curious piece of architecture.</q> <rj><qau>Sir H. Wotton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Staircase shell</b></col>. <fld>(Zool.)</fld> <sd>(a)</sd> <cd>Any scalaria, or wentletrap.</cd> <sd>(b)</sd> <cd>Any species of <gen>Solarium</gen>, or perspective shell.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stairhead</ent><br/
<hw>Stair"head`</hw> <pr>(-h<ecr/d`)</pr>, <pos>n.</pos> <def>The head or top of a staircase.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stairway</ent><br/
<hw>Stair"way`</hw> <pr>(-w<amac/`)</pr>, <pos>n.</pos> <def>A flight of stairs or steps; a staircase.</def> <ldquo/A rude and narrow <xex>stairway</xex>.<rdquo/  <rj><au>Moore.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Staith</ent><br/
<hw>Staith</hw> <pr>(st<amac/th)</pr>, <pos>n.</pos> <ety>[AS. <ets>stae<edh/</ets> a bank, shore, from the root of E. <ets>stead</ets>.]</ety> <def>A landing place; an elevated staging upon a wharf for discharging coal, etc., as from railway cars, into vessels.</def> <mark>[Eng.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Staithman</ent><br/
<hw>Staith"man</hw> <pr>(-m<ait/n)</pr>, <pos>n.</pos> <def>A man employed in weighing and shipping at a staith.</def> <mark>[Eng.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stake</ent><br/
<hw>Stake</hw> <pr>(st<amac/k)</pr>, <pos>n.</pos> <ety>[AS. <ets>staca</ets>, from the root of E. <ets>stick</ets>; akin to OFries. & LG. <ets>stake</ets>, D. <ets>staak</ets>, Sw. <ets>stake</ets>, Dan. <ets>stage</ets>. See <er>Stick</er>, <pos>v. t.</pos>, and cf. <er>Estacade</er>, <er>Stockade</er>.]</ety> <sn>1.</sn> <def>A piece of wood, usually long and slender, pointed at one end so as to be easily driven into the ground as a support or stay; <as>as, a <ex>stake</ex> to support vines, fences, hedges, etc.</as></def><br/
[<source>1913 Webster</source>]</p>

<p><q>A sharpened <qex>stake</qex> strong Dryas found.</q> <rj><qau>Dryden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A stick inserted upright in a loop, eye, or mortise, at the side or end of a cart, a flat car, or the like, to prevent goods from falling off.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>The piece of timber to which a martyr was affixed to be burned; hence, martyrdom by fire.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>A small anvil usually furnished with a tang to enter a hole in a bench top, -- used by tinsmiths, blacksmiths, etc., for light work, punching upon, etc.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <def>That which is laid down as a wager; that which is staked or hazarded; a pledge.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>6.</sn> <fld>(Mormon Ch.)</fld> <def>A territorial division; -- called also <altname>stake of Zion</altname>.</def></p>

<p><q>Every city, or <ldquo/<qex>stake</qex>,<rdquo/ including a chief town and surrounding towns, has its president, with two counselors; and this president has a high council of chosen men.</q>  <rj><qau>Schaff-Herzog Encyc.</qau></rj><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><cs><col><b>At stake</b></col>, <cd>in danger; hazarded; pledged.</cd> <ldquo/I see my reputation is <xex>at stake</xex>.<rdquo/ <au>Shak.</au></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stake</ent><br/
<hw>Stake</hw>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Staked</conjf> <pr>(st<amac/kd)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Staking</conjf>.]</vmorph> <sn>1.</sn> <def>To fasten, support, or defend with stakes; <as>as, to <ex>stake</ex> vines or plants</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To mark the limits of by stakes; -- with <xex>out</xex>; <as>as, to <ex>stake</ex> out land; to <ex>stake</ex> out a new road</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>To put at hazard upon the issue of competition, or upon a future contingency; to wager; to pledge.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>I'll <qex>stake</qex> yon lamb, that near the fountain plays.</q> <rj><qau>Pope.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>To pierce or wound with a stake.</def>  <rj><au>Spectator.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stake-driver</ent><br/
<hw>Stake"-driv`er</hw> <pr>(-dr<imac/v`<etil/r)</pr>, <pos>n.</pos> <fld>(Zool.)</fld> <def>The common American bittern (<spn>Botaurus lentiginosus</spn>); -- so called because one of its notes resembles the sound made in driving a stake into the mud.  Called also <altname>meadow hen</altname>, and <altname>Indian hen</altname>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stakehead</ent><br/
<hw>Stake"head`</hw> <pr>(st<amac/k"h<ecr/d`)</pr>, <pos>n.</pos> <fld>(Rope Making)</fld> <def>A horizontal bar on a stake, used for supporting the yarns which are kept apart by pins in the bar.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stakeholder</ent><br/
<hw>Stake"hold`er</hw> <pr>(-h<omac/ld`<etil/r)</pr>, <pos>n.</pos> <def>The holder of a stake; one with whom the bets are deposited when a wager is laid.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Staktometer</ent><br/
<hw>Stak*tom"e*ter</hw> <pr>(st<acr/k*t<ocr/m"<esl/*t<etil/r)</pr>, <pos>n.</pos> <ety>[Gr. <grk>stakto`s</grk> falling by drops + <ets>-meter</ets>.]</ety> <def>A drop measurer; a glass tube tapering to a small orifice at the point, and having a bulb in the middle, used for finding the number of drops in equal quantities of different liquids. See <er>Pipette</er>.</def>  <rj><au>Sir D. Brewster.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stal</ent><br/
<hw>Stal</hw> <pr>(st<aum/l)</pr>, <mark>obs.</mark> <pos>imp.</pos> <mord>of <er>Steal</er></mord>. <def>Stole.</def> <rj><au>Chaucer.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stalactical</ent><br/
<ent>Stalactic</ent><br/
<mhw>{ <hw>Sta*lac"tic</hw> <pr>(st<adot/*l<acr/k"t<icr/k)</pr>, <hw>Sta*lac"tic*al</hw> <pr>(-t<icr/"k<ait/l)</pr>, }</mhw> <pos>a.</pos> <fld>(Geol.)</fld> <def>Stalactitic.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stalactiform</ent><br/
<hw>Sta*lac"ti*form</hw> <pr>(-t<icr/*f<ocir/rm)</pr>, <pos>a.</pos> <def>Like a stalactite; resembling a stalactite.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stalactite</ent><br/
<hw>Sta*lac"tite</hw> <pr>(-t<imac/t)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Stalactites</plw> <pr>(-t<imac/ts)</pr>.</plu> <ety>[Gr. <grk>stalakto`s</grk> oozing out in drops, dropping, fr. <grk>stala`zein</grk> to drop: cf. F. <ets>stalactite</ets>.]</ety> <fld>(Geol.)</fld> <sd>(a)</sd> <def>A pendent cone or cylinder of calcium carbonate resembling an icicle in form and mode of attachment. Stalactites are found depending from the roof or sides of caverns, and are produced by deposition from waters which have percolated through, and partially dissolved, the overlying limestone rocks.</def> <sd>(b)</sd> <def>In an extended sense, any mineral or rock of similar form and origin; <as>as, a <ex>stalactite</ex> of lava</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stalactites</ent><br/
||<hw>Stal`ac*ti"tes</hw> <pr>(st<acr/l`<acr/k*t<imac/"t<emac/z)</pr>, <pos>n.</pos> <ety>[NL.]</ety> <def>A stalactite.</def> <mark>[Obs.]</mark>  <rj><au>Woodward.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stalactitical</ent><br/
<ent>Stalactitic</ent><br/
<mhw>{ <hw>Stal`ac*tit"ic</hw> <pr>(-t<icr/t"<icr/k)</pr>, <hw>Stal`ac*tit"ic*al</hw> <pr>(-<icr/*k<ait/l)</pr>, }</mhw> <pos>a.</pos> <ety>[Cf. F. <ets>stalactitique</ets>.]</ety> <fld>(Geol.)</fld> <def>Of or pertaining to a stalactite; having the form or characters of a stalactite; stalactic.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stalactitiform</ent><br/
<hw>Stal`ac*tit"i*form</hw> <pr>(st<acr/l`<acr/k*t<icr/t"<icr/*f<ocir/rm)</pr>, <pos>a.</pos> <def>Having the form of a stalactite; stalactiform.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stalagmite</ent><br/
<hw>Sta*lag"mite</hw> <pr>(st<adot/*l<acr/g"m<imac/t)</pr>, <pos>n.</pos> <ety>[Gr. <grk>sta`lagma</grk> that which drops, a drop, fr. <grk>stala`zein</grk> to drop: cf. F. <ets>stalagmite</ets>.]</ety> <fld>(Geol.)</fld> <def>A deposit more or less resembling an inverted stalactite, formed by calcareous water dropping on the floors of caverns; hence, a similar deposit of other material.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stalagmitical</ent><br/
<ent>Stalagmitic</ent><br/
<mhw>{ <hw>Stal`ag*mit"ic</hw> <pr>(st<acr/l`<acr/g*m<icr/t"<icr/k)</pr>, <hw>Stal`ag*mit"ic*al</hw> <pr>(-m<icr/t"<icr/*k<ait/l)</pr>, }</mhw> <pos>a.</pos> <def>Having the form or structure of stalagmites.</def> -- <wordforms><wf>Stal`ag*mit"ic*al*ly</wf>, <pos>adv.</pos></wordforms><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stalder</ent><br/
<hw>Stal"der</hw> <pr>(st<add/l"d<etil/r)</pr>, <pos>n.</pos> <ety>[From the root of <ets>stall</ets>.]</ety> <def>A wooden frame to set casks on.</def> <mark>[Prov. Eng.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stale</ent><br/
<hw>Stale</hw> <pr>(st<amac/l)</pr>, <pos>n.</pos> <ety>[OE. <ets>stale</ets>, <ets>stele</ets>, AS. <ets>stael</ets>, <ets>stel</ets>; akin to LG. & D. <ets>steel</ets>, G. <ets>stiel</ets>; cf. L. <ets>stilus</ets> stake, stalk, stem, Gr. <grk>steleo`n</grk> a handle, and E. <ets>stall</ets>, <ets>stalk</ets>, n.]</ety> <def>The stock or handle of anything; <as>as, the <ex>stale</ex> of a rake</as>.</def> <altsp>[Written also <asp>steal</asp>, <asp>stele</asp>, etc.]</altsp><br/
[<source>1913 Webster</source>]</p>

<p><q>But seeing the arrow's <qex>stale</qex> without, and that the head did go<br/
No further than it might be seen.</q> <rj><qau>Chapman.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stale</ent><br/
<hw>Stale</hw>, <pos>a.</pos> <ety>[Akin to <ets>stale</ets> urine, and to <ets>stall</ets>, n.; probably from Low German or Scandinavian.  Cf. <er>Stale</er>, <pos>v. i.</pos>]</ety> <sn>1.</sn> <def>Vapid or tasteless from age; having lost its life, spirit, and flavor, from being long kept; <as>as, <ex>stale</ex> beer</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Not new; not freshly made; <as>as, <ex>stale</ex> bread</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>Having lost the life or graces of youth; worn out; decayed.</def> <ldquo/A <xex>stale</xex> virgin.<rdquo/  <rj><au>Spectator.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>Worn out by use or familiarity; having lost its novelty and power of pleasing; trite; common.</def>  <rj><au>Swift.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Wit itself, if <qex>stale</qex> is less pleasing.</q> <rj><qau>Grew.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>How weary, <qex>stale</qex> flat, and unprofitable<br/
Seem to me all the uses of this world!</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Stale affidavit</b></col> <fld>(Law)</fld>, <cd>an affidavit held above a year.</cd> <au>Craig.</au> -- <col><b>Stale demand</b></col> <fld>(Law)</fld>, <cd>a claim or demand which has not been pressed or demanded for a long time.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stale</ent><br/
<hw>Stale</hw>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Staled</conjf> <pr>(st<amac/ld)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Staling</conjf>.]</vmorph> <def>To make vapid or tasteless; to destroy the life, beauty, or use of; to wear out.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Age can not wither her, nor custom <qex>stale</qex><br/
Her infinite variety.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stale</ent><br/
<hw>Stale</hw>, <pos>v. i.</pos> <ety>[Akin to D. & G. <ets>stallen</ets>, Dan. <ets>stalle</ets>, Sw. <ets>stalla</ets>, and E. <ets>stall</ets> a stable.  <root/ 163. See <er>Stall</er>, <pos>n.</pos>, and cf. <er>Stale</er>, <pos>a.</pos>]</ety> <def>To make water; to discharge urine; -- said especially of horses and cattle.</def>  <rj><au>Hudibras.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stale</ent><br/
<hw>Stale</hw>, <pos>n.</pos> <ety>[See <er>Stale</er>, <pos>a.</pos> & <pos>v. i.</pos>]</ety> <sn>1.</sn> <def>That which is stale or worn out by long keeping, or by use.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A prostitute.</def> <mark>[Obs.]</mark>  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>Urine, esp. that of beasts.</def> <ldquo/<xex>Stale</xex> of horses.<rdquo/  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stale</ent><br/
<hw>Stale</hw>, <pos>n.</pos> <ety>[Cf. OF. <ets>estal</ets> place, position, abode, market, F. <ets><eacute/tal</ets> a butcher's stall, OHG. <ets>stal</ets> station, place, stable, G. <ets>stall</ets> (see <er>Stall</er>, <pos>n.</pos>); or from OE. <ets>stale</ets> theft, AS. <ets>stalu</ets> (see <er>Steal</er>, <pos>v. t.</pos>).]</ety> <sn>1.</sn> <def>Something set, or offered to view, as an allurement to draw others to any place or purpose; a decoy; a stool pigeon.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>Still, as he went, he crafty <qex>stales</qex> did lay.</q> <rj><qau>Spenser.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A stalking-horse.</def> <mark>[Obs.]</mark>  <rj><au>B. Jonson.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <fld>(Chess)</fld> <def>A stalemate.</def> <mark>[Obs.]</mark>  <rj><au>Bacon.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>A laughingstock; a dupe.</def> <mark>[Obs.]</mark>  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stalely</ent><br/
<hw>Stale"ly</hw>, <pos>adv.</pos> <sn>1.</sn> <def>In a stale manner.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Of old; long since.</def> <mark>[Obs.]</mark>  <rj><au>B. Jonson.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stalemate</ent><br/
<hw>Stale"mate`</hw> <pr>(-m<amac/t`)</pr>, <pos>n.</pos> <fld>(Chess)</fld> <def>The position of the king when he can not move without being placed in check and there is no other piece which can be moved.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stalemate</ent><br/
<hw>Stale"mate`</hw>, <pos>v. t.</pos> <fld>(Chess)</fld> <def>To subject to a stalemate; hence, to bring to a stand.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Staleness</ent><br/
<hw>Stale"ness</hw>, <pos>n.</pos> <def>The quality or state of being stale.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stalk</ent><br/
<hw>Stalk</hw> <pr>(st<add/k)</pr>, <pos>n.</pos> <ety>[OE. <ets>stalke</ets>, fr. AS. <ets>stael</ets>, <ets>stel</ets>, a stalk. See <er>Stale</er> a handle, <er>Stall</er>.]</ety> <sn>1.</sn> <fld>(Bot.)</fld> <sd>(a)</sd> <def>The stem or main axis of a plant; <as>as, a <ex>stalk</ex> of wheat, rye, or oats; the <ex>stalks</ex> of maize or hemp.</as></def> <sd>(b)</sd> <def>The petiole, pedicel, or peduncle, of a plant.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>That which resembles the stalk of a plant, as the stem of a quill.</def>  <rj><au>Grew.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <fld>(Arch.)</fld> <def>An ornament in the Corinthian capital resembling the stalk of a plant, from which the volutes and helices spring.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>One of the two upright pieces of a ladder.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>To climb by the rungs and the <qex>stalks</qex>.</q> <rj><qau>Chaucer.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <fld>(Zool.)</fld> <sd>(a)</sd> <def>A stem or peduncle, as of certain barnacles and crinoids.</def> <sd>(b)</sd> <def>The narrow basal portion of the abdomen of a hymenopterous insect.</def> <sd>(c)</sd> <def>The peduncle of the eyes of decapod crustaceans.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>6.</sn> <fld>(Founding)</fld> <def>An iron bar with projections inserted in a core to strengthen it; a core arbor.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Stalk borer</b></col> <fld>(Zool.)</fld>, <cd>the larva of a noctuid moth (<spn>Gortyna nitela</spn>), which bores in the stalks of the raspberry, strawberry, tomato, asters, and many other garden plants, often doing much injury.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><-- p. 1401 pr=VMG --></p>

<p><ent>Stalk</ent><br/
<hw>Stalk</hw>, <pos>v. i.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Stalked</conjf> <pr>(st<add/kt)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Stalking</conjf>.]</vmorph> <ety>[AS. <ets>staelcan</ets>, <ets>stealcian</ets> to go slowly; cf. <ets>stealc</ets> high, elevated, Dan. <ets>stalke</ets> to stalk; probably akin to 1st <ets>stalk</ets>.]</ety> <sn>1.</sn> <def>To walk slowly and cautiously; to walk in a stealthy, noiseless manner; -- sometimes used with a reflexive pronoun.</def>  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Into the chamber he <qex>stalked</qex> him full still.</q> <rj><qau>Chaucer.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>[Bertran] <qex>stalks</qex> close behind her, like a witch's fiend,<br/
Pressing to be employed.</q> <rj><qau>Dryden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To walk behind something as a screen, for the purpose of approaching game; to proceed under cover.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The king . . . crept under the shoulder of his led horse; . . . <ldquo/I must <qex>stalk</qex>,<rdquo/ said he.</q> <rj><qau>Bacon.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>One underneath his horse, to get a shoot doth <qex>stalk</qex>.</q> <rj><qau>Drayton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>To walk with high and proud steps; -- usually implying the affectation of dignity, and indicating dislike.  The word is used, however, especially by the poets, to express dignity of step.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>With manly mien he <qex>stalked</qex> along the ground.</q> <rj><qau>Dryden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Then <qex>stalking</qex> through the deep,<br/
He fords the ocean.</q> <rj><qau>Addison.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>I forbear myself from entering the lists in which he has long <qex>stalked</qex> alone and unchallenged.</q> <rj><qau>Merivale.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stalk</ent><br/
<hw>Stalk</hw> <pr>(st<add/k)</pr>, <pos>v. t.</pos> <sn>1.</sn> <def>To approach under cover of a screen, or by stealth, for the purpose of killing, as game.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>As for shooting a man from behind a wall, it is cruelly like to <qex>stalking</qex> a deer.</q> <rj><qau>Sir W. Scott.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To follow (a person) persistently, with or without attempts to evade detection; <as>as, the paparazzi <ex>stalk</ex> celebrities to get candid photographs; obsessed fans may <ex>stalk</ex> their favorite movie stars</as>.</def><br/
[<source>PJC</source>]</p>

<p><ent>Stalk</ent><br/
<hw>Stalk</hw>, <pos>n.</pos> <sn>1.</sn> <def>A high, proud, stately step or walk.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Thus twice before, . . . <br/
With martial <qex>stalk</qex> hath he gone by our watch.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>The which with monstrous <qex>stalk</qex> behind him stepped.</q> <rj><qau>Spenser.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>The act or process of stalking.</def></p>

<p><q>When the <qex>stalk</qex> was over (the antelope took alarm and ran off before I was within rifle shot) I came back.</q>  <rj><qau>T. Roosevelt.</qau></rj><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Stalked</ent><br/
<hw>Stalked</hw> <pr>(st<add/kt)</pr>, <pos>a.</pos> <def>Having a stalk or stem; borne upon a stem.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Stalked barnacle</b></col> <fld>(Zool.)</fld>, <cd>a goose barnacle, or anatifer; -- called also <altname>stalk barnacle</altname>.</cd> -- <col><b>Stalked crinoid</b></col> <fld>(Zool.)</fld>, <cd>any crinoid having a jointed stem.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stalker</ent><br/
<hw>Stalk"er</hw> <pr>(st<add/k"<etil/r)</pr>, <pos>n.</pos> <sn>1.</sn> <def>One who stalks.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A kind of fishing net.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stalk-eyed</ent><br/
<hw>Stalk"-eyed`</hw> <pr>(st<add/k"<imac/d`)</pr>, <pos>a.</pos> <fld>(Zool.)</fld> <def>Having the eyes raised on a stalk, or peduncle; -- opposed to <contr>sessile-eyed</contr>. Said especially of podophthalmous crustaceans.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Stalk-eyed crustaceans</b></col>. <fld>(Zool.)</fld> <cd>See <er>Podophthalmia</er>.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stalking-horse</ent><br/
<hw>Stalk"ing-horse`</hw> <pr>(st<add/k"<icr/ng-h<ocir/rs`)</pr>, <pos>n.</pos> <sn>1.</sn> <def>A horse, or a figure resembling a horse, behind which a hunter conceals himself from the game he is aiming to kill.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <mark>Fig.:</mark>  <def>Something used to cover up a secret project; a mask; a pretense.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Hypocrisy is the devil's <qex>stalking-horse</qex> under an affectation of simplicity and religion.</q> <rj><qau>L'Estrange.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>How much more abominable is it to make of him [Christ] and religion a <qex>stalking-horse</qex>, to get and enjoy the world!</q> <rj><qau>Bunyan.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stalkless</ent><br/
<hw>Stalk"less</hw>, <pos>a.</pos> <def>Having no stalk.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stalky</ent><br/
<hw>Stalk"y</hw> <pr>(-<ycr/)</pr>, <pos>a.</pos> <def>Hard as a stalk; resembling a stalk.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>At the top [it] bears a great <qex>stalky</qex> head.</q> <rj><qau>Mortimer.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stall</ent><br/
<hw>Stall</hw> <pr>(st<add/l)</pr>, <pos>n.</pos> <ety>[OE. <ets>stal</ets>, AS. <ets>steall</ets>, <ets>stall</ets>, a place, seat, or station, a stable; akin to D. & OHG. <ets>stal</ets>, G. & Sw. <ets>stall</ets>, Icel. <ets>stallr</ets>, Dan. <ets>stald</ets>, originally, a standing place; akin also to G. <ets>stelle</ets> a place, <ets>stellen</ets> to place, Gr. <grk>ste`llein</grk> to set, place, send, and E. <ets>stand</ets>.  <root/163. See <er>Stand</er>, and cf. <er>Apostle</er>, <er>Epistle</er>, <er>Forestall</er>, <er>Install</er>, <er>Stale</er>, <pos>a.</pos> & <pos>v. i.</pos>, 1st <er>Stalk</er>, <er>Stallion</er>, <er>Still</er>.]</ety> <sn>1.</sn> <def>A stand; a station; a fixed spot; hence, the stand or place where a horse or an ox is kept and fed; the division of a stable, or the compartment, for one horse, ox, or other animal.</def> <ldquo/In an oxes <xex>stall</xex>.<rdquo/  <rj><au>Chaucer.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A stable; a place for cattle.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>At last he found a <qex>stall</qex> where oxen stood.</q> <rj><qau>Dryden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>A small apartment or shed in which merchandise is exposed for sale; <as>as, a butcher's <ex>stall</ex>; a book<ex>stall</ex></as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>A bench or table on which small articles of merchandise are exposed for sale.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>How peddlers' <qex>stalls</qex> with glittering toys are laid.</q> <rj><qau>Gay.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <def>A seat in the choir of a church, for one of the officiating clergy.  It is inclosed, either wholly or partially, at the back and sides.  The stalls are frequently very rich, with canopies and elaborate carving.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The dignified clergy, out of humility, have called their thrones by the names of <qex>stalls</qex>.</q> <rj><qau>Bp. Warburton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Loud the monks sang in their <qex>stalls</qex>.</q> <rj><qau>Longfellow.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>6.</sn> <def>In the theater, a seat with arms or otherwise partly inclosed, as distinguished from the benches, sofas, etc.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>7.</sn> <fld>(Mining)</fld> <def>The space left by excavation between pillars. See <cref>Post and stall</cref>, under <er>Post</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>8.</sn> <def>A covering or sheath, as of leather, horn, of iron, for a finger or thumb; a cot; <as>as, a thumb <ex>stall</ex>; a finger <ex>stall</ex></as>.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><cs><col><b>Stall reader</b></col>, <cd>one who reads books at a stall where they are exposed for sale.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><q>Cries the <qex>stall reader</qex>, <ldquo/Bless us! what a word on<br/
A titlepage is this!<rdquo/</q>  <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stall</ent><br/
<hw>Stall</hw>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Stalled</conjf> <pr>(st<add/ld)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Stalling</conjf>.]</vmorph> <ety>[Cf. Sw. <ets>stalla</ets>, Dan. <ets>stalde</ets>.]</ety> <sn>1.</sn> <def>To put into a stall or stable; to keep in a stall or stalls; <as>as, to <ex>stall</ex> an ox</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Where King Latinus then his oxen <qex>stalled</qex>.</q>  <rj><qau>Dryden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To fatten; <as>as, to <ex>stall</ex> cattle</as>.</def> <mark>[Prov. Eng.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>To place in an office with the customary formalities; to install.</def> <mark>[Obs.]</mark> <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>To plunge into mire or snow so as not to be able to get on; to set; to fix; <as>as, to <ex>stall</ex> a cart</as>.</def>  <rj><au>Burton.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>His horses had been <qex>stalled</qex> in the snow.</q> <rj><qau>E. E. Hale.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <def>To forestall; to anticipate.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>This is not to be <qex>stall'd</qex> by my report.</q> <rj><qau>Massinger.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>6.</sn> <def>To keep close; to keep secret.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q><qex>Stall</qex> this in your bosom.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stall</ent><br/
<hw>Stall</hw>, <pos>v. i.</pos> <ety>[AS. <ets>steallian</ets> to have room. See <er>Stall</er>, <pos>n.</pos>]</ety> <sn>1.</sn> <def>To live in, or as in, a stall; to dwell.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>We could not <qex>stall</qex> together<br/
In the whole world.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To kennel, as dogs.</def>  <rj><au>Johnson.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>To be set, as in mire or snow; to stick fast.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>To be tired of eating, as cattle.</def> <mark>[Prov. Eng.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stallage</ent><br/
<hw>Stall"age</hw> <pr>(st<add/l"<asl/j ; 48)</pr>, <pos>n.</pos> <ety>[Cf. OF. <ets>estallage</ets>, of German origin. See <er>Stall</er>, <pos>n.</pos>]</ety> <sn>1.</sn> <fld>(Eng. Law)</fld> <def>The right of erecting a stall or stalls in fairs; rent paid for a stall.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Dung of cattle or horses, mixed with straw.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stallation</ent><br/
<hw>Stal*la"tion</hw> <pr>(st<ocr/l*l<amac/"sh<ucr/n)</pr>, <pos>n.</pos> <def>Installation.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stalled</ent><br/
<hw>Stalled</hw> <pr>(st<add/ld <or/ st<add/l"<ecr/d)</pr>, <pos>a.</pos> <def>Put or kept in a stall; hence, fatted.</def> <ldquo/A <xex>stalled</xex> ox.<rdquo/  <rj><au>Prov. xv. 17.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Staller</ent><br/
<hw>Stall"er</hw> <pr>(st<add/l"<etil/r)</pr>, <pos>n.</pos> <def>A standard bearer.</def> <mark>[R.]</mark>  <rj><au>Fuller.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stall-feed</ent><br/
<hw>Stall"-feed`</hw> <pr>(-f<emac/d`)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Stall-fed</conjf> <pr>(-f<ecr/d`)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Stall-feeding</conjf>.]</vmorph> <def>To feed and fatten in a stall or on dry fodder; <as>as, to <ex>stall-feed</ex> an ox</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stalling</ent><br/
<hw>Stall"ing</hw> <pr>(st<add/l"<icr/ng)</pr>, <pos>n.</pos> <def>Stabling.</def>  <rj><au>Tennyson.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stallion</ent><br/
<hw>Stal"lion</hw> <pr>(st<acr/l"y<ucr/n)</pr>, <pos>n.</pos> <ety>[OE. <ets>stalon</ets>, OF. <ets>estalon</ets>, F. <ets><eacute/talon</ets>, fr. OHG. <ets>stal</ets> a stable. See <er>Stall</er>, <pos>n.</pos>]</ety> <def>A male horse not castrated; a male horse kept for breeding.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stallman</ent><br/
<hw>Stall"man</hw> <pr>(st<add/l"m<ait/n)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Stallmen</plw> <pr>(-m<eit/n)</pr>.</plu> <def>One who keeps a stall for the sale of merchandise, especially books.</def>  <rj><au>Sterne.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stallon</ent><br/
<hw>Stal"lon</hw> <pr>(st<acr/l"l<ucr/n)</pr>, <pos>n.</pos> <def>A slip from a plant; a scion; a cutting.</def> <mark>[R.]</mark>  <rj><au>Holinshed.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stalworth</ent><br/
<ent>Stalwart</ent><br/
<mhw>{ <hw>Stal"wart</hw> <pr>(st<ocr/l"w<etil/rt <or/ st<add/l"-; 277)</pr>, <hw>Stal"worth</hw> <pr>(-w<etil/rth)</pr>, }</mhw> <pos>a.</pos> <ety>[OE. <ets>stalworth</ets>, AS. <ets>staelwyr<edh/</ets> serviceable, probably originally, good at stealing, or worth stealing or taking, and afterwards extended to other causes of estimation. See <er>Steal</er>, <pos>v. t.</pos>, <er>Worth</er>, <pos>a.</pos>]</ety> <def>Brave; bold; strong; redoubted; daring; vehement; violent.</def> <ldquo/A <xex>stalwart</xex> tiller of the soil.<rdquo/  <rj><au>Prof. Wilson.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Fair man he was and wise, <qex>stalworth</qex> and bold.</q>
 <rj><qau>R. of Brunne.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ <xex>Stalworth</xex> is now disused, or but little used, <xex>stalwart</xex> having taken its place.</note><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stalwartly</ent><br/
<hw>Stal"wart*ly</hw> <pr>(-w<etil/rt*l<ycr/)</pr>, <pos>adv.</pos> <def>In a stalwart manner.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stalwartness</ent><br/
<hw>Stal"wart*ness</hw>, <pos>n.</pos> <def>The quality of being stalwart.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stalworthness</ent><br/
<ent>Stalworthhood</ent><br/
<mhw>{ <hw>Stal"worth*hood</hw> <pr>(-w<etil/rth*h<oocr/d)</pr>, <hw>Stal"worth*ness</hw> <pr>(-w<etil/rth*n<ecr/s)</pr>}</mhw>, <pos>n.</pos> <def>The quality or state of being stalworth; stalwartness; boldness; daring.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stamen</ent><br/
<hw>Sta"men</hw> <pr>(st<amac/"m<ecr/n)</pr>, <pos>n.</pos>; <plu><it>pl.</it> E. <plw>Stamens</plw> <pr>(-m<ecr/nz)</pr> (used only in the second sense); L. <plw>Stamina</plw> <pr>(st<acr/m"<icr/*n<adot/)</pr> (in the first sense).</plu> <ety>[L. <ets>stamen</ets> the warp, a thread, fiber, akin to Gr. <grk>sth`mwn</grk> the warp, fr. <grk>'ista`nai</grk> to stand, akin to E. <ets>stand</ets>.  See <er>Stand</er>, and cf. <er>Stamin</er>, <er>Stamina</er>.]</ety> <sn>1.</sn> <def>A thread; especially, a warp thread.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <plu>(<it>pl.</it> <er>Stamens</er>, rarely <er>Stamina</er>.)</plu> <fld>(Bot.)</fld> <def>The male organ of flowers for secreting and furnishing the pollen or fecundating dust.  It consists of the <part>anther</part> and <part>filament</part>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stamened</ent><br/
<hw>Sta"mened</hw> <pr>(-m<ecr/nd)</pr>, <pos>a.</pos> <def>Furnished with stamens.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stamin</ent><br/
<hw>Sta"min</hw> <pr>(st<amac/"m<icr/n)</pr>, <pos>n.</pos> <ety>[OF. <ets>estamine</ets>, F. <ets><eacute/tamine</ets>, LL. <ets>staminea</ets>, <ets>stamineum</ets>, fr. L. <ets>stamineus</ets> consisting of threads, fr. <ets>stamen</ets> a thread. See <er>Stamen</er>, and cf. <er>Stamineous</er>, 2d <er>Stammel</er>, <er>Tamine</er>.]</ety> <def>A kind of woolen cloth.</def> <altsp>[Written also <asp>stamine</asp>.]</altsp> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stamina</ent><br/
<hw>Stam"i*na</hw> <pr>(st<acr/m"<icr/*n<adot/)</pr>, <pos>n. pl.</pos> <def>See <er>Stamen</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stamina</ent><br/
<hw>Stam"i*na</hw>, <pos>n. pl.</pos> <sn>1.</sn> <def>The fixed, firm part of a body, which supports it or gives it strength and solidity; <as>as, the bones are the <ex>stamina</ex> of animal bodies; the ligneous parts of trees are the <ex>stamina</ex> which constitute their strength</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Whatever constitutes the principal strength or support of anything; backbone; vigor; <as>as, the <ex>stamina</ex> of a constitution or of life; the <ex>stamina</ex> of a State</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>He succeeded to great captains who had sapped the whole <qex>stamina</qex> and resistance of the contest.</q> <rj><qau>De Quincey.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <specif>Hence:</specif> <def>The power of endurance; the ability to withstand fatigue, disease, deprivation, etc., and continue working.</def><br/
[<source>1913 Webster</source> <source>+PJC</source>]</p>

<p><ent>Staminal</ent><br/
<hw>Stam"i*nal</hw> <pr>(-n<ait/l)</pr>, <pos>a.</pos> <ety>[Cf. F. <ets>staminal</ets>.]</ety> <def>Of or pertaining to stamens or stamina; consisting in stamens.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Staminate</ent><br/
<hw>Stam"i*nate</hw> <pr>(-n<asl/t)</pr>, <pos>a.</pos> <ety>[L. <ets>staminatus</ets> consisting of threads, fr. <ets>stamen</ets> thread: cf. F. <ets>stamin<eacute/</ets>.]</ety> <fld>(Bot.)</fld> <sd>(a)</sd> <def>Furnished with stamens; producing stamens.</def> <sd>(b)</sd> <def>Having stamens, but lacking pistils.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Staminate</ent><br/
<hw>Stam"i*nate</hw> <pr>(-n<amac/t)</pr>, <pos>v. t.</pos> <def>To indue with stamina.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stamineous</ent><br/
<ent>Stamineal</ent><br/
<mhw>{ <hw>Sta*min"e*al</hw> <pr>(st<adot/*m<icr/n"<esl/*<ait/l)</pr>, <hw>Sta*min"e*ous</hw> <pr>(-m<icr/n"<esl/*<ucr/s)</pr>, }</mhw> <pos>a.</pos> <ety>[L. <ets>stamineus</ets>, from <ets>stamen</ets> thread.]</ety> <sn>1.</sn> <def>Consisting of stamens or threads.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Bot.)</fld> <def>Of or pertaining to the stamens; possessing stamens; also, attached to the stamens; <as>as, a <ex>stamineous</ex> nectary</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Staminiferous</ent><br/
<hw>Stam`i*nif"er*ous</hw> <pr>(st<acr/m`<icr/*n<icr/f"<etil/r*<ucr/s)</pr>, <pos>a.</pos> <ety>[<ets>Stamen</ets> + <ets>-ferous</ets>.]</ety> <def>Bearing or having stamens.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Staminode</ent><br/
<hw>Stam"i*node</hw> <pr>(st<acr/m"<icr/*n<omac/d)</pr>, <pos>n.</pos> <fld>(Bot.)</fld> <def>A staminodium.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Staminodium</ent><br/
||<hw>Stam`i*no"di*um</hw> <pr>(-n<omac/"d<icr/*<ucr/m)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Staminodia</plw> <pr>(-<adot/)</pr>.</plu> <ety>[NL. See <er>Stamen</er>, and <er>-oid</er>.]</ety> <fld>(Bot.)</fld> <def>An abortive stamen, or any organ modified from an abortive stamen.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stammel</ent><br/
<hw>Stam"mel</hw> <pr>(st<acr/m"m<ecr/l)</pr>, <pos>n.</pos> <def>A large, clumsy horse.</def> <mark>[Prov. Eng.]</mark>  <rj><au>Wright.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stammel</ent><br/
<hw>Stam"mel</hw>, <pos>n.</pos> <ety>[OF. <ets>estamel</ets>; cf. OF. <ets>estamet</ets> a coarse woolen cloth, LL. <ets>stameta</ets> a kind of cloth, the same as <ets>staminea</ets>, and OF. <ets>estame</ets> a woolen stuff. See <er>Stamin</er>.]</ety> <sn>1.</sn> <def>A kind of woolen cloth formerly in use. It seems to have been often of a red color.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A red dye, used in England in the 15th and 16th centuries.</def>  <rj><au>B. Jonson.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stammel</ent><br/
<hw>Stam"mel</hw>, <pos>a.</pos> <def>Of the color of stammel; having a red color, thought inferior to scarlet.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stammer</ent><br/
<hw>Stam"mer</hw> <pr>(st<acr/m"m<etil/r)</pr>, <pos>v. i.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Stammered</conjf> <pr>(-m<etil/rd)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Stammering</conjf>.]</vmorph> <ety>[OE. <ets>stameren</ets>, fr. AS. <ets>stamur</ets>, <ets>stamer</ets>, stammering; akin to D. & LG. <ets>stameren</ets> to stammer, G. <ets>stammeln</ets>, OHG. <ets>stammal<omac/n</ets>, <ets>stamm<emac/n</ets>, Dan. <ets>stamme</ets>, Sw. <ets>stamma</ets>, Icel. <ets>stama</ets>, <ets>stamma</ets>, OHG. & Dan. <ets>stam</ets> stammering, Icel. <ets>stamr</ets>, Goth. <ets>stamms</ets>, and to G. <ets>stemmen</ets> to bear against, <ets>stumm</ets> dumb, D. <ets>stom</ets>.  Cf. <er>Stem</er> to resist, <er>Stumble</er>.]</ety> <def>To make involuntary stops in uttering syllables or words; to hesitate or falter in speaking; to speak with stops and difficulty; to stutter.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>I would thou couldst <qex>stammer</qex>, that thou mightest pour this concealed man out of thy mouth, as wine comes out of a narrow-mouthed bottle, either too much at once, or none at all.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stammer</ent><br/
<hw>Stam"mer</hw> <pr>(st<acr/m"m<etil/r)</pr>, <pos>v. t.</pos> <def>To utter or pronounce with hesitation or imperfectly; -- sometimes with <xex>out</xex>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stammer</ent><br/
<hw>Stam"mer</hw>, <pos>n.</pos> <def>Defective utterance, or involuntary interruption of utterance; a stutter.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stammerer</ent><br/
<hw>Stam"mer*er</hw> <pr>(-<etil/r)</pr>, <pos>n.</pos> <def>One who stammers.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stammering</ent><br/
<hw>Stam"mer*ing</hw>, <pos>a.</pos> <def>Apt to stammer; hesitating in speech; stuttering.</def> -- <wordforms><wf>Stam"mer*ing*ly</wf>, <pos>adv.</pos></wordforms><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stammering</ent><br/
<hw>Stam"mer*ing</hw>, <pos>n.</pos> <fld>(Physiol.)</fld> <def>A disturbance in the formation of sounds.  It is due essentially to long-continued spasmodic contraction of the diaphragm, by which expiration is prevented, and hence it may be considered as a spasmodic inspiration.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stamp</ent><br/
<hw>Stamp</hw> <pr>(st<acr/mp)</pr> <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Stamped</conjf> <pr>(st<acr/mt ; 215)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Stamping</conjf>.]</vmorph> <ety>[OE. <ets>stampen</ets>; akin to LG. & D. <ets>stampen</ets>, G. <ets>stampfen</ets>, OHG. <ets>stampf<omac/n</ets>, Dan. <ets>stampe</ets>, Sw. <ets>stampa</ets>, Icel. <ets>stappa</ets>, G. <ets>stampf</ets> a pestle and E. <ets>step</ets>. See <er>Step</er>, <pos>v. i.</pos>, and cf. <er>Stampede</er>.]</ety> <sn>1.</sn> <def>To strike beat, or press forcibly with the bottom of the foot, or by thrusting the foot downward.</def>  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>He frets, he fumes, he stares, he <qex>stamps</qex> the ground.</q> <rj><qau>Dryden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To bring down (the foot) forcibly on the ground or floor; <as>as, he <ex>stamped</ex> his foot with rage</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>To crush; to pulverize; specifically <fld>(Metal.)</fld>, to crush by the blow of a heavy stamp, as ore in a mill.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>I took your sin, the calf which ye had made, and burnt it with fire, and <qex>stamped</qex> it, and ground it very small.</q> <rj><qau>Deut. ix. 21.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>To impress with some mark or figure; <as>as, to <ex>stamp</ex> a plate with arms or initials</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <def>Fig.: To impress; to imprint; to fix deeply; <as>as, to <ex>stamp</ex> virtuous principles on the heart</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>God . . . has <qex>stamped</qex> no original characters on our minds wherein we may read his being.</q> <rj><qau>Locke.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>6.</sn> <def>To cut out, bend, or indent, as paper, sheet metal, etc., into various forms, by a blow or suddenly applied pressure with a stamp or die, etc.; to mint; to coin.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>7.</sn> <def>To put a stamp on, as for postage; <as>as, to <ex>stamp</ex> a letter; to <ex>stamp</ex> a legal document</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>To stamp out</b></col>, <cd>to put an end to by sudden and energetic action; to extinguish; <as>as, <ex>to stamp out</ex> a rebellion</as>.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stamp</ent><br/
<hw>Stamp</hw>, <pos>v. i.</pos> <sn>1.</sn> <def>To strike; to beat; to crush.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>These cooks how they <qex>stamp</qex> and strain and grind.</q> <rj><qau>Chaucer.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To strike the foot forcibly downward.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>But starts, exclaims, and <qex>stamps</qex>, and raves, and dies.</q> <rj><qau>Dennis.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stamp</ent><br/
<hw>Stamp</hw>, <pos>n.</pos> <sn>1.</sn> <def>The act of stamping, as with the foot.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>The which stamps; any instrument for making impressions on other bodies, as a die.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>'T is gold so pure<br/
It can not bear the <qex>stamp</qex> without alloy.</q> <rj><qau>Dryden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>The mark made by stamping; a mark imprinted; an impression.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>That sacred name gives ornament and grace,<br/
And, like his <qex>stamp</qex>, makes basest metals pass.</q> <rj><qau>Dryden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>That which is marked; a thing stamped.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Hanging a golden <qex>stamp</qex> about their necks.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <ety>[F. <ets>estampe</ets>, of German origin. See <er>Stamp</er>, <pos>v. t.</pos>]</ety> <def>A picture cut in wood or metal, or made by impression; a cut; a plate.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>At Venice they put out very curious <qex>stamps</qex> of the several edifices which are most famous for their beauty and magnificence.</q> <rj><qau>Addison.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>6.</sn> <def>An official mark set upon things chargeable with a duty or tax to government, as evidence that the duty or tax is paid; <as>as, the <ex>stamp</ex> on a bill of exchange</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>7.</sn> <specif>Hence:</specif> <def>A stamped or printed device, usually paper, issued by the government at a fixed price, and required by law to be affixed to, or stamped on, certain papers, as evidence that the government dues are paid; <as>as, a postage <ex>stamp</ex>; a tax  <ex>stamp</ex>; a receipt <ex>stamp</ex>, etc.</as></def><br/
[<source>1913 Webster</source>]</p>

<p><sn>8.</sn> <def>An instrument for cutting out, or shaping, materials, as paper, leather, etc., by a downward pressure.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>9.</sn> <def>A character or reputation, good or bad, fixed on anything as if by an imprinted mark; current value; authority; <as>as, these persons have the <ex>stamp</ex> of dishonesty; the Scriptures bear the <ex>stamp</ex> of a divine origin</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Of the same <qex>stamp</qex> is that which is obtruded on us, that an adamant suspends the attraction of the loadstone.</q> <rj><qau>Sir T. Browne.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>10.</sn> <def>Make; cast; form; character; <as>as, a man of the same <ex>stamp</ex>, or of a different <ex>stamp</ex></as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>A soldier of this season's <qex>stamp</qex>.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>11.</sn> <def>A kind of heavy hammer, or pestle, raised by water or steam power, for beating ores to powder; anything like a pestle, used for pounding or beating.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>12.</sn> <def>A half-penny.</def> <mark>[Obs.]</mark>  <rj><au>Beau. & Fl.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>13.</sn> <pluf>pl.</pluf> <def>Money, esp. paper money.</def> <mark>[Slang, U.S.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Stamp act</b></col>, <cd>an act of the British Parliament [1765] imposing a duty on all paper, vellum, and parchment used in the American colonies, and declaring all writings on unstamped materials to be null and void.</cd> -- <col><b>Stamp collector</b></col>, <sd>(a)</sd> <cd>an officer who receives or collects stamp duties.</cd> <sd>(b)</sd> <cd>one who collects postage or other stamps, as an avocation or for investment; a philatelist.</cd> -- <col><b>Stamp duty</b></col>, <cd>a duty, or tax, imposed on paper and parchment used for certain writings, as deeds, conveyances, etc., the evidence of the payment of the duty or tax being a stamp.</cd> <mark>[Eng.]</mark> -- <col><b>Stamp hammer</b></col>, <cd>a hammer, worked by power, which rises and falls vertically, like a stamp in a stamp mill.</cd> -- <col><b>Stamp head</b></col>, <cd>a heavy mass of metal, forming the head or lower end of a bar, which is lifted and let fall, in a stamp mill.</cd> -- <col><b>Stamp mill</b></col> <fld>(Mining)</fld>, <cd>a mill in which ore is crushed with stamps; also, a machine for stamping ore.</cd> -- <col><b>Stamp note</b></col>, <cd>a stamped certificate from a customhouse officer, which allows goods to be received by the captain of a ship as freight.</cd> <mark>[Eng.]</mark> -- <col><b>Stamp office</b></col>, <cd>an office for the issue of stamps and the reception of stamp duties.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stampede</ent><br/
<hw>Stam*pede"</hw> <pr>(st<acr/m*p<emac/d")</pr>, <pos>n.</pos> <ety>[Sp. <ets>estampida</ets> (in America) a stampede, <ets>estampido</ets> a crackling, akin to <ets>estampar</ets> to stamp, of German origin. See <er>Stamp</er>, <pos>v. t.</pos>]</ety> <sn>1.</sn> <def>A wild, headlong scamper, or running away, of a number of animals; usually caused by fright; hence, any sudden flight or dispersion, as of a crowd or an army in consequence of a panic.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Any sudden unconcerted moving or acting together of a number of persons, as from  some common impulse; <as>as, a <ex>stampede</ex> to the gold regions; a <ex>stampede</ex> in a convention</as>; a <ex>stampede</ex> toward U. S. bonds in the credit markets.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><q>She and her husband would join in the general <qex>stampede</qex>.</q> <rj><qau>W. Black.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><-- p. 1402 pr=VMG --></p>

<p><ent>Stampede</ent><br/
<hw>Stam*pede"</hw> <pr>(st<acr/m*p<emac/d")</pr>, <pos>v. i.</pos> <def>To run away in a panic; -- said of droves of cattle, horses, etc., also of armies.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stampede</ent><br/
<hw>Stam*pede"</hw>, <pos>v. t.</pos> <def>To disperse by causing sudden fright, as a herd or drove of animals.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stamper</ent><br/
<hw>Stamp"er</hw> <pr>(st<acr/mp"<etil/r)</pr>, <pos>n.</pos> <sn>1.</sn> <def>One who stamps.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>An instrument for pounding or stamping.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stamping</ent><br/
<hw>Stamp"ing</hw>, <def><pos>a. & n.</pos> from <er>Stamp</er>, <pos>v.</pos></def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Stamping ground</b></col>, <cd>a place frequented, and much trodden, by animals, wild or domesticated</cd>; hence (<mark>Colloq.</mark>), <cd>the scene of one's labors or exploits; also, one's favorite resort; in this sense, often called <altname>stomping ground</altname>.</cd> <mark>[U.S.]</mark> -- <col><b>Stamping machine</b></col>, <cd>a machine for forming metallic articles or impressions by stamping.</cd> -- <col><b>Stamping mill</b></col> <fld>(Mining)</fld>, <cd>a stamp mill.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stance</ent><br/
<hw>Stance</hw> <pr>(st<acr/ns)</pr>, <pos>n.</pos> <ety>[OF. <ets>estance</ets>. See <er>Stanza</er>.]</ety> <sn>1.</sn> <def>A stanza.</def> <mark>[Obs.]</mark>  <rj><au>Chapman.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A station; a position; a site.</def> <mark>[Scot.]</mark>  <rj><au>Sir W. Scott.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <fld>(Golf, Baseball)</fld> <def>The position of a player's feet, relative to each other and to the ball, when he is making a stroke or at bat.</def><br/
[<source>Webster 1913 Suppl.</source> <source>+PJC</source>]</p>

<p><sn>4.</sn> <def>the mental attitude, position, or approach a person adopts in respect to something.</def><br/
[<source>PJC</source>]</p>

<p><ent>Stanch</ent><br/
<hw>Stanch</hw> <pr>(st<adot/nch)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Stanched</conjf> <pr>(st<adot/ncht)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Stanching</conjf>.]</vmorph> <ety>[OF. <ets>estanchier</ets>, F. <ets><eacute/tancher</ets> to stop a liquid from flowing; akin to Pr., Sp., & Pg. <ets>estancar</ets>, It. <ets>stancare</ets> to weary, LL. <ets>stancare</ets>, <ets>stagnare</ets>, to stanch, fr. L. <ets>stagnare</ets> to be or make stagnant. See <er>Stagnate</er>.]</ety> <sn>1.</sn> <def>To stop the flowing of, as blood; to check; also, to stop the flowing of blood from; <as>as, to <ex>stanch</ex> a wound</as>.</def> <altsp>[Written also <asp>staunch</asp>.]</altsp><br/
[<source>1913 Webster</source>]</p>

<p><q>Iron or a stone laid to the neck doth <qex>stanch</qex> the bleeding of the nose.</q> <rj><qau>Bacon.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To extinguish; to quench, as fire or thirst.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stanch</ent><br/
<hw>Stanch</hw>, <pos>v. i.</pos> <def>To cease, as the flowing of blood.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Immediately her issue of blood <qex>stanched</qex>.</q> <rj><qau>Luke viii. 44.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stanch</ent><br/
<hw>Stanch</hw>, <pos>n.</pos> <sn>1.</sn> <def>That which stanches or checks.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A flood gate by which water is accumulated, for floating a boat over a shallow part of a stream by its release.</def>  <rj><au>Knight.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stanch</ent><br/
<hw>Stanch</hw>, <pos>a.</pos> <amorph>[<pos>Compar.</pos> <adjf>Stancher</adjf> <pr>(-<etil/r)</pr>; <pos>superl.</pos> <adjf>Stanchest</adjf>.]</amorph> <ety>[From <er>Stanch</er>, <pos>v. t.</pos>, and hence literally signifying, stopped or stayed; cf. Sp. <ets>estanco</ets> stopped, tight, not leaky, as a ship. See <er>Stanch</er>, <pos>v. t.</pos>]</ety> <altsp>[Written also <asp>staunch</asp>.]</altsp> <sn>1.</sn> <def>Strong and tight; sound; firm; <as>as, a <ex>stanch</ex> ship</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>One of the closets is parqueted with plain deal, set in diamond, exceeding <qex>stanch</qex> and pretty.</q> <rj><qau>Evelyn.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Firm in principle; constant and zealous; loyal; hearty; steady; steadfast; <as>as, a <ex>stanch</ex> churchman; a <ex>stanch</ex> friend or adherent</as>.</def>  <rj><au>V. Knox.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>In politics I hear you 're <qex>stanch</qex>.</q> <rj><qau>Prior.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>Close; secret; private.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>This is to be kept <qex>stanch</qex>.</q> <rj><qau>Locke.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stanch</ent><br/
<hw>Stanch</hw>, <pos>v. t.</pos> <def>To prop; to make stanch, or strong.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>His gathered sticks to <qex>stanch</qex> the wall<br/
Of the snow tower when snow should fall.</q> <rj><qau>Emerson.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stanchel</ent><br/
<hw>Stan"chel</hw> <pr>(st<acr/n"sh<ecr/l)</pr>, <pos>n.</pos> <def>A stanchion.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stancher</ent><br/
<hw>Stanch"er</hw> <pr>(st<adot/nch"<etil/r)</pr>, <pos>n.</pos> <def>One who, or that which, stanches, or stops, the flowing, as of blood.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stanchion</ent><br/
<hw>Stan"chion</hw> <pr>(st<acr/n"sh<ucr/n ; 277)</pr>, <pos>n.</pos> <ety>[OF. <ets>estanson</ets>, <ets>estan<cced/on</ets>, F. <ets><eacute/tan<cced/on</ets>, from OF. <ets>estance</ets> a stay, a prop, from L. <ets>stans</ets>, <ets>stantis</ets>, standing, p. pr. of <ets>stare</ets> to stand. See <er>Stand</er>, and cf. <er>Stanza</er>.]</ety> <altsp>[Written also <asp>stanchel</asp>.]</altsp> <sn>1.</sn> <fld>(Arch.)</fld> <def>A prop or support; a piece of timber in the form of a stake or post, used for a support or stay.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Naut.)</fld> <def>Any upright post or beam used as a support, as for the deck, the quarter rails, awnings, etc.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>A vertical bar for confining cattle in a stall.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stanchless</ent><br/
<hw>Stanch"less</hw> <pr>(st<adot/nch"l<ecr/s)</pr>, <pos>a.</pos> <sn>1.</sn> <def>Incapable of being stanched, or stopped.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Unquenchable; insatiable.</def> <mark>[Obs.]</mark>  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stanchly</ent><br/
<hw>Stanch"ly</hw>, <pos>adv.</pos> <def>In a stanch manner.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stanchness</ent><br/
<hw>Stanch"ness</hw>, <pos>n.</pos> <def>The quality or state of being stanch.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stand</ent><br/
<hw>Stand</hw> <pr>(st<acr/nd)</pr>, <pos>v. i.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Stood</conjf> <pr>(st<oocr/d)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Standing</conjf>.]</vmorph> <ety>[OE. <ets>standen</ets>; AS. <ets>standan</ets>; akin to OFries. <ets>stonda</ets>, <ets>st<amac/n</ets>, D. <ets>staan</ets>, OS. <ets>standan</ets>, <ets>st<amac/n</ets>, OHG. <ets>stantan</ets>, <ets>st<amac/n</ets>, G. <ets>stehen</ets>, Icel. <ets>standa</ets>, Dan. <ets>staae</ets>, Sw. <ets>st<aring/</ets>, Goth. <ets>standan</ets>, Russ. <ets>stoiate</ets>, L. <ets>stare</ets>, Gr. <grk>'ista`nai</grk> to cause to stand, <grk>sth^nai</grk> to stand, Skr. <ets>sth<amac/</ets>.  <root/163.  Cf. <er>Assist</er>, <er>Constant</er>, <er>Contrast</er>, <er>Desist</er>, <er>Destine</er>, <er>Ecstasy</er>, <er>Exist</er>, <er>Interstice</er>, <er>Obstacle</er>, <er>Obstinate</er>, <er>Prest</er>, <pos>n.</pos>, <er>Rest</er> remainder, <er>Solstice</er>, <er>Stable</er>, <pos>a. & n.</pos>, <er>Staff</er>, <er>Stage</er>, <er>Stall</er>, <pos>n.</pos>, <er>Stamen</er>, <er>Stanchion</er>, <er>Stanza</er>, <er>State</er>, <pos>n.</pos>, <er>Statute</er>, <er>Stead</er>, <er>Steed</er>, <er>Stool</er>, <er>Stud</er> of horses, <er>Substance</er>, <er>System</er>.]</ety> <sn>1.</sn> <def>To be at rest in an erect position; to be fixed in an upright or firm position</def>; as: <sd>(a)</sd> <def>To be supported on the feet, in an erect or nearly erect position; -- opposed to <contr>lie</contr>, <contr>sit</contr>, <contr>kneel</contr>, etc.</def>  <ldquo/I pray you all, <xex>stand</xex> up!<rdquo/ <au>Shak.</au> <sd>(b)</sd> <def>To continue upright in a certain locality, as a tree fixed by the roots, or a building resting on its foundation.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>It <qex>stands</qex> as it were to the ground yglued.</q> <rj><qau>Chaucer.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>The ruined wall<br/
<qex>Stands</qex> when its wind-worn battlements are gone.</q> <rj><qau>Byron.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To occupy or hold a place; to have a situation; to be situated or located; <as>as, Paris <ex>stands</ex> on the Seine</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Wite ye not where there <qex>stands</qex> a little town?</q> <rj><qau>Chaucer.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>To cease from progress; not to proceed; to stop; to pause; to halt; to remain stationary.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>I charge thee, <qex>stand</qex>,<br/
And tell thy name.</q> <rj><qau>Dryden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>The star, which they saw in the east, went before them, till it came and <qex>stood</qex> over where the young child was.</q> <rj><qau>Matt. ii. 9.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>To remain without ruin or injury; to hold good against tendencies to impair or injure; to be permanent; to endure; to last; hence, to find endurance, strength, or resources.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>My mind on its own center <qex>stands</qex> unmoved.</q> <rj><qau>Dryden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <def>To maintain one's ground; to be acquitted; not to fail or yield; to be safe.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Readers by whose judgment I would <qex>stand</qex> or fall.</q> <rj><qau>Spectator.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>6.</sn> <def>To maintain an invincible or permanent attitude; to be fixed, steady, or firm; to take a position in resistance or opposition.</def> <ldquo/The <xex>standing</xex> pattern of their imitation.<rdquo/  <rj><au>South.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>The king granted the Jews . . . to gather themselves together, and to <qex>stand</qex> for their life.</q> <rj><qau>Esther viii. 11.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>7.</sn> <def>To adhere to fixed principles; to maintain moral rectitude; to keep from falling into error or vice.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>We must labor so as to <qex>stand</qex> with godliness, according to his appointment.</q> <rj><qau>Latimer.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>8.</sn> <def>To have or maintain a position, order, or rank; to be in a particular relation; <as>as, Christian charity, or love, <ex>stands</ex> first in the rank of gifts</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>9.</sn> <def>To be in some particular state; to have essence or being; to be; to consist.</def> <ldquo/Sacrifices . . . which <xex>stood</xex> only in meats and drinks.<rdquo/  <rj><au>Heb. ix. 10.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Accomplish what your signs foreshow;<br/
I <qex>stand</qex> resigned, and am prepared to go.</q> <rj><qau>Dryden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Thou seest how it <qex>stands</qex> with me, and that I may not tarry.</q> <rj><qau>Sir W. Scott.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>10.</sn> <def>To be consistent; to agree; to accord.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Doubt me not; by heaven, I will do nothing<br/
But what may <qex>stand</qex> with honor.</q> <rj><qau>Massinger.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>11.</sn> <fld>(Naut.)</fld> <def>To hold a course at sea; <as>as, to <ex>stand</ex> from the shore; to <ex>stand</ex> for the harbor</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>From the same parts of heaven his navy <qex>stands</qex>.</q> <rj><qau>Dryden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>12.</sn> <def>To offer one's self, or to be offered, as a candidate.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>He <qex>stood</qex> to be elected one of the proctors of the university.</q> <rj><qau>Walton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>13.</sn> <def>To stagnate; not to flow; to be motionless.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Or the black water of Pomptina <qex>stands</qex>.</q> <rj><qau>Dryden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>14.</sn> <def>To measure when erect on the feet.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Six feet two, as I think, he <qex>stands</qex>.</q> <rj><qau>Tennyson.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>15.</sn> <fld>(Law)</fld> <sd>(a)</sd> <def>To be or remain as it is; to continue in force; to have efficacy or validity; to abide.</def> <au>Bouvier.</au> <sd>(b)</sd> <def>To appear in court.</def> <au>Burrill.</au><br/
[<source>1913 Webster</source>]</p>

<p><sn>16.</sn> <fld>(Card Playing)</fld> <def>To be, or signify that one is, willing to play with one's hand as dealt.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><cs><col><b>Stand by</b></col> <fld>(Naut.)</fld>, <cd>a preparatory order, equivalent to <altname>Be ready</altname>.</cd> -- <col><b>To stand against</b></col>, <cd>to oppose; to resist.</cd> -- <col><b>To stand by</b></col>. <sd>(a)</sd> <cd>To be near; to be a spectator; to be present</cd>. <sd>(b)</sd> <cd>To be aside; to be set aside with disregard</cd>. <ldquo/In the interim [we] let the commands <xex>stand by</xex> neglected.<rdquo/ <au>Dr. H. More.</au> <sd>(c)</sd> <cd>To maintain; to defend; to support; not to desert; <as>as, <ex>to stand by</ex> one's principles or party</as>.</cd> <sd>(d)</sd> <cd>To rest on for support; to be supported by</cd>. <au>Whitgift.</au> <sd>(e)</sd> <cd>To remain as a spectator, and take no part in an action; <as>as, we can't just <ex>stand</ex> idly <ex>by</ex> while people are being killed</as>.</cd> -- <col><b>To stand corrected</b></col>, <cd>to be set right, as after an error in a statement of fact; to admit having been in error.</cd> <au>Wycherley.</au> -- <col><b>To stand fast</b></col>, <cd>to be fixed; to be unshaken or immovable.</cd> -- <col><b>To stand firmly on</b></col>, <cd>to be satisfied or convinced of.</cd> <ldquo/Though Page be a secure fool, and <xex>stands</xex> so <xex>firmly on</xex> his wife's frailty.<rdquo/ <au>Shak.</au> -- <col><b>To stand for</b></col>. <sd>(a)</sd> <cd>To side with; to espouse the cause of; to support; to maintain, or to profess or attempt to maintain; to defend.</cd>  <ldquo/I <xex>stand</xex> wholly <xex>for</xex> you.<rdquo/ <au>Shak.</au> <sd>(b)</sd> <cd>To be in the place of; to be the substitute or representative of; to represent; <as>as, a cipher at the left hand of a figure <ex>stands for</ex> nothing</as>.</cd> <ldquo/I will not trouble myself, whether these names <xex>stand for</xex> the same thing, or really include one another.<rdquo/ <au>Locke.</au> <sd>(c)</sd> <cd>To tolerate; <as>as, I won't <ex>stand for</ex> any delay</as>.</cd> -- <col><b>To stand in</b></col>, <cd>to cost.</cd> <ldquo/The same <xex>standeth</xex> them <xex>in</xex> much less cost.<rdquo/ <au>Robynson (More's Utopia).</au><br/
<br/
<q>The Punic wars could not have <qex>stood</qex> the human race <qex>in</qex> less than three millions of the species.</q> <rj><qau>Burke.</qau></rj><br/
<br/
  -- <col><b>To stand in hand</b></col>, <cd>to conduce to one's interest; to be serviceable or advantageous.</cd> -- <col><b>To stand off</b></col>. <sd>(a)</sd> <cd>To keep at a distance</cd>. <sd>(b)</sd> <cd>Not to comply</cd>. <sd>(c)</sd> <cd>To keep at a distance in friendship, social intercourse, or acquaintance</cd>. <sd>(d)</sd> <cd>To appear prominent; to have relief</cd>. <ldquo/Picture is best when it <xex>standeth off</xex>, as if it were carved.<rdquo/ <au>Sir H. Wotton.</au> -- <col><b>To stand off and on</b></col> <fld>(Naut.)</fld>, <cd>to remain near a coast by sailing toward land and then from it.</cd> -- <col><b>To stand on</b></col> <fld>(Naut.)</fld>, <cd>to continue on the same tack or course.</cd> -- <col><b>To stand out</b></col>. <sd>(a)</sd> <cd>To project; to be prominent</cd>. <ldquo/Their eyes <xex>stand out</xex> with fatness.<rdquo/ <au>Psalm lxxiii. 7.</au> <sd>(b)</sd> <cd>To persist in opposition or resistance; not to yield or comply; not to give way or recede.</cd><br/
<br/
<q>His spirit is come in,<br/
That so <qex>stood out</qex> against the holy church.</q> <rj><qau>Shak.</qau></rj><br/
<br/
   -- <col><b>To stand to</b></col>. <sd>(a)</sd> <cd>To ply; to urge; to persevere in using</cd>. <ldquo/<xex>Stand to</xex> your tackles, mates, and stretch your oars.<rdquo/ <au>Dryden.</au> <sd>(b)</sd> <cd>To remain fixed in a purpose or opinion.</cd> <ldquo/I will <xex>stand to</xex> it, that this is his sense.<rdquo/ <au>Bp. Stillingfleet.</au> <sd>(c)</sd> <cd>To abide by; to adhere to; as to a contract, assertion, promise, etc.; <as>as, <ex>to stand to</ex> an award; <ex>to stand to</ex> one's word</as>.</cd> <sd>(d)</sd> <cd>Not to yield; not to fly; to maintain, as one's ground</cd>. <ldquo/Their lives and fortunes were put in safety, whether they <xex>stood to</xex> it or ran away.<rdquo/ <au>Bacon.</au> <sd>(e)</sd> <cd>To be consistent with; to agree with; <as>as, it <ex>stands to</ex> reason that he could not have done so</as>; same as <cref>stand with</cref>, below .</cd> <sd>(f)</sd> <cd>To support; to uphold</cd>. <ldquo/<xex>Stand to</xex> me in this cause.<rdquo/ <au>Shak.</au> -- <col><b>To stand together</b></col>, <cd>to be consistent; to agree.</cd> -- <col><b>To stand to reason</b></col> <cd>to be reasonable; to be expected.</cd> -- <col><b>To stand to sea</b></col> <fld>(Naut.)</fld>, <cd>to direct the course from land.</cd> -- <col><b>To stand under</b></col>, <cd>to undergo; to withstand.</cd> <au>Shak.</au> -- <col><b>To stand up</b></col>. <sd>(a)</sd> <cd>To rise from sitting; to be on the feet</cd>. <sd>(b)</sd> <cd>To arise in order to speak or act</cd>. <ldquo/Against whom, when the accusers <xex>stood up</xex>, they brought none accusation of such things as I supposed.<rdquo/ <au>Acts xxv. 18.</au> <sd>(c)</sd> <cd>To rise and stand on end, as the hair.</cd> <sd>(d)</sd> <cd>To put one's self in opposition; to contend</cd>. <ldquo/Once we <xex>stood up</xex> about the corn.<rdquo/ <au>Shak.</au> -- <col><b>To stand up for</b></col>, <cd>to defend; to justify; to support, or attempt to support; <as>as, <ex>to stand up for</ex> the administration</as>.</cd> -- <col><b>To stand upon</b></col>. <sd>(a)</sd> <cd>To concern; to interest</cd>. <sd>(b)</sd> <cd>To value; to esteem</cd>. <ldquo/We highly esteem and <xex>stand</xex> much <xex>upon</xex> our birth.<rdquo/ <au>Ray.</au> <sd>(c)</sd> <cd>To insist on; to attach much importance to; <as>as, <ex>to stand upon</ex> security; <ex>to stand upon</ex> ceremony</as>.</cd> <sd>(d)</sd> <cd>To attack; to assault.</cd> <mark>[A Hebraism]</mark> <ldquo/So I <xex>stood upon</xex> him, and slew him.<rdquo/ <au>2 Sam. i. 10.</au> -- <col><b>To stand with</b></col>, <cd>to be consistent with.</cd> <ldquo/It <xex>stands with</xex> reason that they should be rewarded liberally.<rdquo/ <au>Sir J. Davies.</au></cs><-- usu. . --><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stand</ent><br/
<hw>Stand</hw> <pr>(st<acr/nd)</pr>, <pos>v. t.</pos> <sn>1.</sn> <def>To endure; to sustain; to bear; <as>as, I can not <ex>stand</ex> the cold or the heat</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To resist, without yielding or receding; to withstand.</def> <ldquo/Love <xex>stood</xex> the siege.<rdquo/  <rj><au>Dryden.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>He <qex>stood</qex> the furious foe.</q> <rj><qau>Pope.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>To abide by; to submit to; to suffer.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Bid him disband his legions, . . . <br/
And <qex>stand</qex> the judgment of a Roman senate.</q> <rj><qau>Addison.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>To set upright; to cause to stand; <as>as, to <ex>stand</ex> a book on the shelf; to <ex>stand</ex> a man on his feet</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <def>To be at the expense of; to pay for; <as>as, to <ex>stand</ex> a treat</as>.</def> <mark>[Colloq.]</mark>  <rj><au>Thackeray.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>To stand fire</b></col>, <cd>to receive the fire of arms from an enemy without giving way.</cd> -- <col><b>To stand one's ground</b></col>, <cd>to keep the ground or station one has taken; to maintain one's position.</cd> <ldquo/Peasants and burghers, however brave, are unable <xex>to stand their ground</xex> against veteran soldiers.<rdquo/ <au>Macaulay.</au> -- <col><b>To stand trial</b></col>, <cd>to sustain the trial or examination of a cause; not to give up without trial.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stand</ent><br/
<hw>Stand</hw> <pr>(st<acr/nd)</pr>, <pos>n.</pos> <ety>[AS. <ets>stand</ets>. See <er>Stand</er>, <pos>v. i.</pos>]</ety> <sn>1.</sn> <def>The act of standing.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>I took my <qex>stand</qex> upon an eminence . . . to look into their several ladings.</q> <rj><qau>Spectator.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A halt or stop for the purpose of defense, resistance, or opposition; <as>as, to come to, or to make, a <ex>stand</ex></as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Vice is at <qex>stand</qex>, and at the highest flow.</q> <rj><qau>Dryden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>A place or post where one stands; a place where one may stand while observing or waiting for something.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>I have found you out a <qex>stand</qex> most fit,<br/
Where you may have such vantage on the duke,<br/
He shall not pass you.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>A station in a city or town where carriages or wagons stand for hire; <as>as, a cab <ex>stand</ex></as>.</def>  <rj><au>Dickens.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <def>A raised platform or station where a race or other outdoor spectacle may be viewed; <as>as, the judge's or the grand <ex>stand</ex> at a race course</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>6.</sn> <def>A small table; also, something on or in which anything may be laid, hung, or placed upright; <as>as, a hat<ex>stand</ex>; an umbrella <ex>stand</ex>; a music <ex>stand</ex></as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>7.</sn> <def>The place where a witness stands to testify in court.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>8.</sn> <def>The situation of a shop, store, hotel, etc.; <as>as, a good, bad, or convenient <ex>stand</ex> for business</as>.</def> <mark>[U. S.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><sn>9.</sn> <def>Rank; post; station; standing.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Father, since your fortune did attain<br/
So high a <qex>stand</qex>, I mean not to descend.</q> <rj><qau>Daniel.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>10.</sn> <def>A state of perplexity or embarrassment; <as>as, to be at a <ex>stand</ex> what to do</as>.</def>  <rj><au>L'Estrange.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>11.</sn> <def>A young tree, usually reserved when other trees are cut; also, a tree growing or standing upon its own root, in distinction from one produced from a scion set in a stock, either of the same or another kind of tree.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>12.</sn> <fld>(Com.)</fld> <def>A weight of from two hundred and fifty to three hundred pounds, -- used in weighing pitch.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Microscope stand</b></col>, <cd>the instrument, excepting the eyepiece, objective, and other removable optical parts.</cd> -- <col><b>Stand of ammunition</b></col>, <cd>the projectile, cartridge, and sabot connected together.</cd> -- <col><b>Stand of arms</b></col>. <fld>(Mil.)</fld> <cd>See under <er>Arms</er>.</cd> -- <col><b>Stand of colors</b></col> <fld>(Mil.)</fld>, <cd>a single color, or flag.</cd> <au>Wilhelm (Mil. Dict.)</au> -- <col><b>To be at a stand</b></col>, <cd>to be stationary or motionless; to be at a standstill; hence, to be perplexed; to be embarrassed.</cd> -- <col><b>To make a stand</b></col>, <cd>to halt for the purpose of offering resistance to a pursuing enemy.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><syn><b>Syn.</b> -- Stop; halt; rest; interruption; obstruction; perplexity; difficulty; embarrassment; hesitation.</syn><br/
[<source>1913 Webster</source>]</p>

<p><ent>Standage</ent><br/
<hw>Stand"age</hw> <pr>(-<asl/j ; 48)</pr>, <pos>n.</pos> <fld>(Mining)</fld> <def>A reservoir in which water accumulates at the bottom of a mine.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Standard</ent><br/
<hw>Stand"ard</hw> <pr>(-<etil/rd)</pr>, <pos>n.</pos> <ety>[OF. <ets>estendart</ets>, F. <ets><eacute/tendard</ets>, probably fr. L. <ets>extendere</ets> to spread out, extend, but influenced by E. <ets>stand</ets>. See <er>Extend</er>.]</ety> <sn>1.</sn> <def>A flag; colors; a banner; especially, a national or other ensign.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>His armies, in the following day,<br/
On those fair plains their <qex>standards</qex> proud display.</q> <rj><qau>Fairfax.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>That which is established by authority as a rule for the measure of quantity, extent, value, or quality; esp., the original specimen weight or measure sanctioned by government, as the standard pound, gallon, or yard.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>That which is established as a rule or model by authority, custom, or general consent; criterion; test.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The court, which used to be the <qex>standard</qex> of propriety and correctness of speech.</q> <rj><qau>Swift.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>A disposition to preserve, and an ability to improve, taken together, would be my <qex>standard</qex> of a statesman.</q> <rj><qau>Burke.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <fld>(Coinage)</fld> <def>The proportion of weights of fine metal and alloy established by authority.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>By the present <qex>standard</qex> of the coinage, sixty-two shillings is coined out of one pound weight of silver.</q> <rj><qau>Arbuthnot.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <fld>(Hort.)</fld> <def>A tree of natural size supported by its own stem, and not dwarfed by grafting on the stock of a smaller species nor trained upon a wall or trellis.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>In France part of their gardens is laid out for flowers, others for fruits; some <qex>standards</qex>, some against walls.</q> <rj><qau>Sir W. Temple.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>6.</sn> <fld>(Bot.)</fld> <def>The upper petal or banner of a papilionaceous corolla.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>7.</sn> <fld>(Mech. & Carp.)</fld> <def>An upright support, as one of the poles of a scaffold; any upright in framing.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>8.</sn> <fld>(Shipbuilding)</fld> <def>An inverted knee timber placed upon the deck instead of beneath it, with its vertical branch turned upward from that which lies horizontally.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>9.</sn> <def>The sheth of a plow.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>10.</sn> <def>A large drinking cup.</def>  <rj><au>Greene.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Standard bearer</b></col>, <cd>an officer of an army, company, or troop, who bears a standard; -- commonly called <xex>color sergeant</xex>or <xex>color bearer</xex>; hence, the leader of any organization; <as>as, the <ex>standard bearer</ex> of a political party</as>.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Standard</ent><br/
<hw>Stand"ard</hw>, <pos>a.</pos> <sn>1.</sn> <def>Being, affording, or according with, a standard for comparison and judgment; <as>as, <ex>standard</ex> time; <ex>standard</ex> weights and measures; a <ex>standard</ex> authority as to nautical terms; <ex>standard</ex> gold or silver</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Hence: Having a recognized and permanent value; <as>as, <ex>standard</ex> works in history; <ex>standard</ex> authors</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <fld>(Hort.)</fld> <sd>(a)</sd> <def>Not supported by, or fastened to, a wall; <as>as, <ex>standard</ex> fruit trees</as>.</def> <sd>(b)</sd> <def>Not of the dwarf kind; <as>as, a <ex>standard</ex> pear tree</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><mcol><col><b>Standard candle</b></col>, <col><b>Standard gauge</b></col></mcol>. <cd>See under <er>Candle</er>, and <er>Gauge</er>.</cd> -- <col><b>Standard solution</b></col>. <fld>(Chem.)</fld> <cd>See <cref>Standardized solution</cref>, under <er>Solution</er>.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Standard-bred</ent><br/
<hw>Stand"ard-bred`</hw> <pr>(-br<ecr/d`)</pr>, <pos>a.</pos> <def>Bred in conformity to a standard. Specif., applied to a registered trotting horse which comes up to the standard adopted by the <org>National Association of Trotting-horse Breeders</org>.</def> <mark>[U. S.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Standardize</ent><br/
<hw>Stand"ard*ize</hw> <pr>(-<imac/z)</pr>, <pos>v. t.</pos> <fld>(Chem.)</fld> <def>To reduce to a normal standard; to calculate or adjust the strength of, by means of, and for uses in, analysis.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Standard-wing</ent><br/
<hw>Stand"ard-wing`</hw> <pr>(st<acr/nd"<etil/rd-w<icr/ng`)</pr>, <pos>n.</pos> <fld>(Zool.)</fld> <def>A curious paradise bird (<spn>Semioptera Wallacii</spn>) which has two long special feathers standing erect on each wing.</def><br/
[<source>1913 Webster</source>]</p>

<p><-- p. 1403 pr=VMG --></p>

<p><ent>Stand-by</ent><br/
<hw>Stand"-by`</hw> <pr>(st<acr/nd"b<imac/`)</pr>, <pos>n.</pos> <def>One who, or that which, stands by one in need; something upon which one relies for constant use or in an emergency.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Standel</ent><br/
<hw>Stand"el</hw> <pr>(-<ecr/l)</pr>, <pos>n.</pos> <def>A young tree, especially one reserved when others are cut.</def> <mark>[Obs.]</mark>  <rj><au>Fuller.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stander</ent><br/
<hw>Stand"er</hw> <pr>(-<etil/r)</pr>, <pos>n.</pos> <sn>1.</sn> <def>One who stands.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Same as <er>Standel</er>.</def> <mark>[Obs.]</mark>  <rj><au>Ascham.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Staenderat</ent><br/
<ent>Staenderat</ent><br/
<ent>Standerath</ent><br/
<mhw>{ ||<hw>St<aum/n"de*rath`</hw>, <or/ <hw>St<aum/n"de*rat`</hw> <pr>(sht<ecr/n"d<etil/*r<aum/t`)</pr>, }</mhw> <pos>n.</pos> <ety>[G.]</ety> <fld>(Switzerland)</fld> <def>The legislature of <country>Switzerland</country>.  See <er>Legislature</er>, above.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Stander-by</ent><br/
<hw>Stand"er-by`</hw> <pr>(-b<imac/`)</pr>, <pos>n.</pos> <def>One who stands near; one who is present; a bystander.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Standergrass</ent><br/
<hw>Stand"er*grass`</hw> <pr>(-gr<adot/s`)</pr>, <pos>n.</pos> <fld>(Bot.)</fld> <def>A plant (<spn>Orchis mascula</spn>); -- called also <altname>standerwort</altname>, and <altname>long purple</altname>. See <cref>Long purple</cref>, under <er>Long</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Standgale</ent><br/
<hw>Stand"gale`</hw> <pr>(-g<amac/l`)</pr>, <pos>n.</pos> <def>See <er>Stannel</er>.</def> <mark>[Prov. Eng.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Standing</ent><br/
<hw>Stand"ing</hw>, <pos>a.</pos> <sn>1.</sn> <def>Remaining erect; not cut down; <as>as, <ex>standing</ex> corn</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Not flowing; stagnant; <as>as, <ex>standing</ex> water</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>Not transitory; not liable to fade or vanish; lasting; <as>as, a <ex>standing</ex> color</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>Established by law, custom, or the like; settled; continually existing; permanent; not temporary; <as>as, a <ex>standing</ex> army; legislative bodies have <ex>standing</ex> rules of proceeding and <ex>standing</ex> committees</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <def>Not movable; fixed; <as>as, a <ex>standing</ex> bed (distinguished from a <ex>trundle</ex>-bed)</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Standing army</b></col>. <cd>See <cref>Standing army</cref>, under <er>Army</er>.</cd> -- <col><b>Standing bolt</b></col>. <cd>See <cref>Stud bolt</cref>, under <er>Stud</er>, a stem.</cd> -- <col><b>Standing committee</b></col>, <cd>in legislative bodies, etc., a committee appointed for the consideration of all subjects of a particular class which shall arise during the session or a stated period.</cd> -- <col><b>Standing cup</b></col>, <cd>a tall goblet, with a foot and a cover.</cd> -- <col><b>Standing finish</b></col> <fld>(Arch.)</fld>, <cd>that part of the interior fittings, esp. of a dwelling house, which is permanent and fixed in its place, as distinguished from doors, sashes, etc.</cd> -- <col><b>Standing order</b></col> <sd>(a)</sd> <fld>(Eccl.)</fld>, <cd>the denomination (Congregational) established by law; -- a term formerly used in Connecticut. See also under <er>Order</er>.</cd> <sd>(a)</sd> <fld>(Com.)</fld>  <cd>an order for goods which are to be delivered periodically, without the need for renewal of the order before each delivery.</cd> -- <col><b>Standing part</b></col>. <fld>(Naut.)</fld> <sd>(a)</sd> <cd>That part of a tackle which is made fast to a block, point, or other object</cd>. <sd>(b)</sd> <cd>That part of a rope around which turns are taken with the running part in making a knot or the like.</cd> -- <col><b>Standing rigging</b></col> <fld>(Naut.)</fld>, <cd>the cordage or ropes which sustain the masts and remain fixed in their position, as the shrouds and stays, -- distinguished from <contr>running rigging</contr>.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Standing</ent><br/
<hw>Stand"ing</hw>, <pos>n.</pos> <sn>1.</sn> <def>The act of stopping, or coming to a stand; the state of being erect upon the feet; stand.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Maintenance of position; duration; duration or existence in the same place or condition; continuance; <as>as, a custom of long <ex>standing</ex>; an officer of long <ex>standing</ex></as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>An ancient thing of long <qex>standing</qex>.</q> <rj><qau>Bunyan.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>Place to stand in; station; stand.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>I will provide you a good <qex>standing</qex> to see his entry.</q> <rj><qau>Bacon.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>I sink in deep mire, where there is no <qex>standing</qex>.</q> <rj><qau>Ps. lxix. 2.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>Condition in society; relative position; reputation; rank; <as>as, a man of good <ex>standing</ex>, or of high <ex>standing</ex></as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Standing off</b></col> <fld>(Naut.)</fld>, <cd>sailing from the land.</cd> -- <col><b>Standing on</b></col> <fld>(Naut.)</fld>, <cd>sailing toward land.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Standish</ent><br/
<hw>Stand"ish</hw>, <pos>n.</pos> <ety>[<ets>Stand</ets> + <ets>dish</ets>.]</ety> <def>A stand, or case, for pen and ink.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>I bequeath to Dean Swift, Esq., my large silver <qex>standish</qex>.</q> <rj><qau>Swift.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>standoff</ent><br/
<hw>stand"off</hw> <pos>n.</pos> <sn>1.</sn> <def>A situation in a contest in which the winner is undecided; a draw  or stalemate.</def><br/
<syn><b>Syn. --</b> draw; tie; stalemate; deadlock</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><sn>2.</sn> <def>A successful stand.</def><br/
<syn><b>Syn. --</b> repulsion, repelling</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><sn>3.</sn> <def>Something that serves to keep one object at a distance from another, such as a prop attached to the top of a ladder to keep it from leaning directly against the vertical surface which suports it.</def><br/
[<source>PJC</source>]</p>

<p><ent>standoff</ent><br/
<hw>stand"off</hw> <pos>a.</pos> <def>Serving to keep two objects or parts of a machine separated; <as>as, a <ex>standoff</ex> insulator</as>.</def><br/
[<source>PJC</source>]</p>

<p><ent>stand-offish</ent><br/
<ent>standoffish</ent><br/
<mhw><hw>stand*off"ish</hw>, <hw>stand-off"ish</hw></mhw> <pos>a.</pos> <def>Shy or distant in manner; cold; aloof; reserved; not friendly.</def><br/
<syn><b>Syn. --</b> offish.</syn><br/
[<source>PJC</source>]</p>

<p>-- <wordforms><wf>stand*off"ish*ness</wf>, <pos>n.</pos> -- <wf>stand*off"ish*ly</wf>, <pos>adv.</pos></wordforms><br/
[<source>PJC</source>]</p>

<p><ent>Standpipe</ent><br/
<hw>Stand"pipe`</hw> <pr>(-p<imac/p`)</pr>, <pos>n.</pos> <sn>1.</sn> <fld>(Engin.)</fld> <def>A vertical pipe, open at the top, between a hydrant and a reservoir, to equalize the flow of water; also, a large vertical pipe, near a pumping engine, into which water is forced up, so as to give it sufficient head to rise to the required level at a distance.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Steam Boiler)</fld> <def>A supply pipe of sufficient elevation to enable the water to flow into the boiler, notwithstanding the pressure of the steam.</def>  <rj><au>Knight.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Standpoint</ent><br/
<hw>Stand"point`</hw> <pr>(-point`)</pr>, <pos>n.</pos> <ety>[Cf. G. <ets>standpunkt</ets>.]</ety> <def>A fixed point or station; a basis or fundamental principle; a position from which objects or principles are viewed, and according to which they are compared and judged.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>stands</ent><br/
<hw>stands</hw> <pos>n.</pos> <def>a structure (often made of wood and sometimes temporary) with seats or benches where people can sit to watch an event (such as a game or parade).</def><br/
<syn><b>Syn. --</b> stand</syn><br/
[<source>WordNet 1.5</source> <source>+PJC</source>]</p>

<p><ent>Standstill</ent><br/
<hw>Stand"still`</hw> <pr>(-st<icr/l`)</pr>, <pos>n.</pos> <def>A standing without moving forward or backward; a stop; a state of rest.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stane</ent><br/
<hw>Stane</hw> <pr>(st<amac/n)</pr>, <pos>n.</pos> <def>A stone.</def> <mark>[Scot. & Prov. Eng.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stang</ent><br/
<hw>Stang</hw> <pr>(st<acr/ng)</pr>, <def><pos>imp.</pos> of <er>Sting</er>.</def> <mark>[Archaic]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stang</ent><br/
<hw>Stang</hw>, <pos>n.</pos> <ety>[OE. <ets>stange</ets>, of Scand. or Dutch origin; cf. Icel. <ets>st<oum/ng</ets>, akin to Dan. <ets>stang</ets>, Sw. <ets>st<aring/ng</ets>, D. <ets>stang</ets>, G. <ets>stange</ets>, OHG. <ets>stanga</ets>, AS. <ets>steng</ets>; from the root of E. <ets>sting</ets>.]</ety> <sn>1.</sn> <def>A long bar; a pole; a shaft; a stake.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>In land measure, a pole, rod, or perch.</def> <mark>[Obs. or Prov. Eng.]</mark>  <rj><au>Swift.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Stang ball</b></col>, <cd>a projectile consisting of two half balls united by a bar; a bar shot. See <xex>Illust.</xex> of <cref>Bar shot</cref>, under <er>Bar</er>.</cd> -- <col><b>To ride the stang</b></col>, <cd>to be carried on a pole on men's shoulders. This method of punishing wife beaters, etc., was once in vogue in some parts of England.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stang</ent><br/
<hw>Stang</hw>, <pos>v. i.</pos> <ety>[Akin to <ets>sting</ets>; cf. Icel. <ets>stanga</ets> to prick, to goad.]</ety> <def>To shoot with pain.</def> <mark>[Prov. Eng.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stanhope</ent><br/
<hw>Stan"hope</hw> <pr>(st<acr/n"h<omac/p ; <mark>colloq.</mark> st<acr/n"<ucr/p)</pr>, <pos>n.</pos> <def>A light two-wheeled, or sometimes four-wheeled, carriage, without a top; -- so called from Lord <ets>Stanhope</ets>, for whom it was contrived.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Staniel</ent><br/
<hw>Stan"iel</hw> <pr>(st<acr/n"y<ecr/l)</pr>, <pos>n.</pos> <fld>(Zool.)</fld> <def>See <er>Stannel</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stanielry</ent><br/
<hw>Stan"iel*ry</hw> <pr>(-r<ycr/)</pr>, <pos>n.</pos> <def>Hawking with staniels, or stannels, -- a base kind of falconry.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stank</ent><br/
<hw>Stank</hw> <pr>(st<acr/<nsm/k)</pr>, <pos>a.</pos> <ety>[OF. <ets>estanc</ets>, or It. <ets>stanco</ets>. See <er>Stanch</er>, <pos>a.</pos>]</ety> <def>Weak; worn out.</def> <mark>[Obs.]</mark>  <rj><au>Spenser.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stank</ent><br/
<hw>Stank</hw>, <pos>v. i.</pos> <ety>[Cf. Sw. <ets>st<aring/nka</ets> to pant. <root/165.]</ety> <def>To sigh.</def> <mark>[Obs. or Prov. Eng.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stank</ent><br/
<hw>Stank</hw>, <pos>imp.</pos> <mord>of <er>Stink</er></mord>. <def>Stunk.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stank</ent><br/
<hw>Stank</hw>, <pos>n.</pos> <ety>[OF. <ets>estang</ets>, F. <ets><eacute/tang</ets>, from L. <ets>stagnum</ets> a pool.  Cf. <er>Stagnate</er>, <er>Tank</er> a cistern.]</ety> <sn>1.</sn> <def>Water retained by an embankment; a pool of water.</def> <mark>[Prov. Eng. & Scot.]</mark>  <rj><au>Robert of Brunne.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A dam or mound to stop water.</def> <mark>[Prov. Eng.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Stank hen</b></col> <fld>(Zool.)</fld>, <cd>the moor hen; -- called also <altname>stankie</altname>.</cd> <mark>[Prov. Eng.]</mark></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stannary</ent><br/
<hw>Stan"na*ry</hw> <pr>(st<acr/n"n<adot/*r<ycr/)</pr>, <pos>a.</pos> <ety>[L. <ets>stannum</ets> tin, an alloy of silver and lead.]</ety> <def>Of or pertaining to tin mines, or tin works.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The <qex>stannary</qex> courts of Devonshire and Cornwall, for the administration of justice among the tinners therein, are also courts of record.</q> <rj><qau>Blackstone.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stannary</ent><br/
<hw>Stan"na*ry</hw>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Stannaries</plw> <pr>(-r<icr/z)</pr>.</plu> <ety>[LL. <ets>stannaria</ets>.]</ety> <def>A tin mine; tin works.</def>  <rj><au>Bp. Hall.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stannate</ent><br/
<hw>Stan"nate</hw> <pr>(-n<asl/t)</pr>, <pos>n.</pos> <ety>[Cf. F. <ets>stannate</ets>.]</ety> <fld>(Chem.)</fld> <def>A salt of stannic acid.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stannel</ent><br/
<hw>Stan"nel</hw> <pr>(-n<ecr/l)</pr>, <pos>n.</pos> <ety>[AS. <ets>st<amac/ngella</ets>, <ets>stangilla</ets>; properly, stone yeller, <it>i. e.</it>, a bird that yells from the rocks. See <er>Stone</er>, and <er>Yell</er>, and cf. <er>Stonegall</er>.]</ety> <fld>(Zool.)</fld> <def>The kestrel; -- called also <altname>standgale</altname>, <altname>standgall</altname>, <altname>stanchel</altname>, <altname>stand hawk</altname>, <altname>stannel hawk</altname>, <altname>steingale</altname>, <altname>stonegall</altname>.</def> <altsp>[Written also <asp>staniel</asp>, <asp>stannyel</asp>, and <asp>stanyel</asp>.]</altsp><br/
[<source>1913 Webster</source>]</p>

<p><q>With what wing the <qex>staniel</qex> checks at it.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stannic</ent><br/
<hw>Stan"nic</hw> <pr>(-n<icr/k)</pr>, <pos>a.</pos> <ety>[L. <ets>stannum</ets> tin: cf. F. <ets>stannique</ets>.]</ety> <fld>(Chem.)</fld> <def>Of or pertaining to tin; derived from or containing tin; specifically, designating those compounds in which the element has a higher valence as contrasted with <contr>stannous</contr> compounds.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Stannic acid</b></col>. <sd>(a)</sd> <cd>A hypothetical substance, <chform>Sn(OH)4</chform>, analogous to silicic acid, and called also <altname>normal stannic acid</altname>.</cd> <sd>(b)</sd> <cd>Metastannic acid.</cd> -- <col><b>Stannic chloride</b></col>, <cd>a thin, colorless, fuming liquid, <chform>SnCl4</chform>, used as a mordant in calico printing and dyeing; -- formerly called <altname>spirit of tin</altname>, or <altname>fuming liquor of Libavius</altname>.</cd> -- <col><b>Stannic oxide</b></col>, <cd>tin oxide, <chform>SnO2</chform>, produced artificially as a white amorphous powder, and occurring naturally in the mineral cassiterite. It is used in the manufacture of white enamels, and, under the name of <altname>putty powder</altname>, for polishing glass, etc.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stanniferous</ent><br/
<hw>Stan*nif"er*ous</hw> <pr>(st<acr/n*n<icr/f"<etil/r*<ucr/s)</pr>, <pos>a.</pos> <ety>[L. <ets>stannum</ets> tin + <ets>-ferous</ets>.]</ety> <def>Containing or affording tin.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stannite</ent><br/
<ent>Stannine</ent><br/
<mhw>{ <hw>Stan"nine</hw> <pr>(-n<icr/n)</pr>, <hw>Stan"nite</hw> <pr>(-n<imac/t)</pr>, }</mhw> <pos>n.</pos> <fld>(Min.)</fld> <def>A mineral of a steel-gray or iron-black color; tin pyrites. It is a sulphide of tin, copper, and iron.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stanno-</ent><br/
<hw>Stan"no-</hw> <pr>(-n<osl/-)</pr>. <ety>[L. <ets>stannum</ets> tin.]</ety> <fld>(Chem.)</fld> <def>A combining form (also used adjectively) denoting <sig>relating to tin</sig>, or <sig>connected with tin</sig>, or indicating <sig>containing tin as an ingredient</sig>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stannofluoride</ent><br/
<hw>Stan`no*flu"or*ide</hw> <pr>(-fl<umac/"<ocr/r*<icr/d <it>or</it> -<imac/d)</pr>, <pos>n.</pos> <fld>(Chem.)</fld> <def>Any one of a series of double fluorides of tin (<it>stannum</it>) and some other element.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stannoso-</ent><br/
<hw>Stan*no"so-</hw> <pr>(st<acr/n*n<omac/"s<osl/-)</pr>, <pos>a.</pos> <fld>(Chem.)</fld> <def>A combining form (also used adjectively) denoting <sig>relation to stannous compounds</sig>, or <sig>connection with stannous compounds</sig>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stannotype</ent><br/
<hw>Stan"no*type</hw> <pr>(st<acr/n"n<osl/*t<imac/p)</pr>, <pos>n.</pos> <ety>[<ets>Stanno-</ets> + <ets>-type</ets>.]</ety> <fld>(Photog.)</fld> <def>A photograph taken upon a tin plate; a tintype.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stannous</ent><br/
<hw>Stan"nous</hw> <pr>(-n<ucr/s)</pr>, <pos>a.</pos> <fld>(Chem.)</fld> <def>Pertaining to, or containing, tin; specifically, designating those compounds in which the element has a lower valence as contrasted with <contr>stannic</contr> compounds.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Stannous chloride</b></col> <fld>(Chem.)</fld>, <cd>a white crystalline substance, <chform>SnCl2.(H2O)2</chform>, obtained by dissolving tin in hydrochloric acid. It is used as a mordant in dyeing.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stannum</ent><br/
||<hw>Stan"num</hw> <pr>(-n<ucr/m)</pr>, <pos>n.</pos> <ety>[L., alloy of silver and lead; later, tin.]</ety> <fld>(Chem.)</fld> <def>The technical name of tin. See <er>Tin</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stanyel</ent><br/
<ent>Stannyel</ent><br/
<mhw>{ <hw>Stann"yel</hw>, <hw>Stan"yel</hw>  }</mhw> <pr>(st<acr/n"y<ecr/l)</pr>, <pos>n.</pos> <fld>(Zool.)</fld> <def>See <er>Stannel</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stont</ent><br/
<ent>Stant</ent><br/
<mhw>{ <hw>Stant</hw> <pr>(st<aum/nt)</pr>, <hw>Stont</hw> <pr>(st<ocr/nt)</pr>, }</mhw> <mark>obs.</mark> <pos>3d pers. sing. pres.</pos> <mord>of <er>Stand</er>, for <xex>standeth</xex></mord>. <def>Stands.</def>  <rj><au>Chaucer.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stanza</ent><br/
<hw>Stan"za</hw> <pr>(st<acr/n"z<adot/)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Stanzas</plw> <pr>(-z<adot/z)</pr>.</plu> <ety>[It. <ets>stanza</ets> a room, habitation, a stanza, <it>i. e.</it>, a stop, fr. L. <ets>stans</ets>, p. pr. of <ets>stare</ets> to stand. See <er>Stand</er>, and cf. <er>Estancia</er>, <er>Stance</er>, <er>Stanchion</er>.]</ety> <sn>1.</sn> <def>A number of lines or verses forming a division of a song or poem, and agreeing in meter, rhyme, number of lines, etc., with other divisions; a part of a poem, ordinarily containing every variation of measure in that poem; a combination or arrangement of lines usually recurring, whether like or unlike, in measure.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Horace confines himself strictly to one sort of verse, or <qex>stanza</qex>, in every ode.</q> <rj><qau>Dryden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Arch.)</fld> <def>An apartment or division in a building; a room or chamber.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stanzaic</ent><br/
<hw>Stan*za"ic</hw> <pr>(st<acr/n*z<amac/"<icr/k)</pr>, <pos>a.</pos> <def>Pertaining to, or consisting of, stanzas; <as>as, a couplet in <ex>stanzaic</ex> form</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stapedial</ent><br/
<hw>Sta*pe"di*al</hw> <pr>(st<adot/*p<emac/"d<icr/*<ait/l)</pr>, <pos>a.</pos> <ety>[LL. <ets>stapes</ets> stirrup.]</ety> <fld>(Anat.)</fld> <def>Of or pertaining to the stapes.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stapelia</ent><br/
||<hw>Sta*pe"li*a</hw> <pr>(-l<icr/*<adot/)</pr>, <pos>n.</pos> <ety>[NL. So named after <person>John Bodaeus a <etsep>Stapel</etsep></person>, a physician of Amsterdam.]</ety> <fld>(Bot.)</fld> <def>An extensive and curious genus of African plants of the natural order <ord>Asclepiadaceae</ord> (Milkweed family). They are succulent plants without leaves, frequently covered with dark tubercles giving them a very grotesque appearance. The odor of the blossoms is like that of carrion.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stapes</ent><br/
||<hw>Sta"pes</hw> <pr>(st<amac/"p<emac/z)</pr>, <pos>n.</pos> <ety>[LL., a stirrup.]</ety> <fld>(Anat.)</fld> <def>The innermost of the ossicles of the ear; the stirrup, or stirrup bone; -- so called from its form. See <xex>Illust.</xex> of <er>Ear</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Staphyline</ent><br/
<hw>Staph"y*line</hw> <pr>(st<acr/f"<icr/*l<imac/n)</pr>, <pos>a.</pos> <ety>[Gr. <grk>stafy`linos</grk> botryoidal, from <grk>stafylh`</grk> a bunch of grapes.]</ety> <fld>(Anat.)</fld> <def>Of or pertaining to the uvula or the palate.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Staphylinid</ent><br/
<hw>Staph`y*li"nid</hw> <pr>(st<acr/f`<icr/*l<imac/"n<icr/d)</pr>, <pos>n.</pos> <ety>[Gr. <grk>stafyli^nos</grk> a kind of insect.]</ety> <fld>(Zool.)</fld> <def>Any rove beetle.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Staphyloma</ent><br/
||<hw>Staph`y*lo"ma</hw> <pr>(-l<omac/"m<adot/)</pr>, <pos>n.</pos> <ety>[L., fr. Gr. <grk>stafy`lwma</grk>, from <grk>stafylh`</grk> a bunch of grapes.]</ety> <fld>(Med.)</fld> <def>A protrusion of any part of the globe of the eye; <as>as, a <ex>staphyloma</ex> of the cornea</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Staphylomatous</ent><br/
<hw>Staph`y*lo"ma*tous</hw> <pr>(st<acr/f`<icr/*l<omac/"m<adot/*t<ucr/s)</pr>, <pos>a.</pos> <fld>(Med.)</fld> <def>Of or pertaining to staphyloma; affected with staphyloma.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Staphyloplasty</ent><br/
<hw>Staph"y*lo*plas`ty</hw> <pr>(st<acr/f"<icr/*l<osl/*pl<acr/s`t<ycr/)</pr>, <pos>n.</pos> <ety>[Gr. <?/ a bunch of grapes, also, the uvula when swollen at the lower end + <ets>-plasty</ets>.]</ety> <fld>(Surg.)</fld> <def>The operation for restoring or replacing the soft palate when it has been lost.</def> <au>Dunglison.</au> -- <wordforms><wf>Staph`y*lo*plas"tic</wf> <pr>(-l<osl/*pl<acr/s"t<icr/k)</pr>, <pos>a.</pos></wordforms><br/
[<source>1913 Webster</source>]</p>

<p><ent>Staphylorrhaphy</ent><br/
<ent>Staphyloraphy</ent><br/
<mhw>{ <hw>Staph`y*lor"a*phy</hw>, <hw>Staph`y*lor"rha*phy</hw>  }</mhw> <pr>(-l<ocr/r"<adot/*f<ycr/)</pr>, <pos>n.</pos> <ety>[Gr. <grk>stafylh`</grk> the uvula when swollen + <grk>"ra`ptein</grk> to sew: cf. F. <ets>staphylorraphie</ets>.]</ety> <fld>(Surg.)</fld> <def>The operation of uniting a cleft palate, consisting in paring and bringing together the edges of the cleft.</def> -- <wordforms><wf>Staph`y*lo*raph"ic</wf> <pr>(st<acr/f`<icr/*l<osl/*r<acr/f"<icr/k)</pr>, <wf>Staph`y*lor*rhaph"ic</wf> <pr>(-l<ocr/r*r<acr/f"<icr/k)</pr>, <pos>a.</pos></wordforms><br/
[<source>1913 Webster</source>]</p>

<p><ent>Staphylotomy</ent><br/
<hw>Staph`y*lot"o*my</hw> <pr>(-l<ocr/t"<osl/*m<ycr/)</pr>, <pos>n.</pos> <ety>[Gr. <?/ the uvula when swollen + <?/ to cut.]</ety> <fld>(Surg.)</fld> <def>The operation of removing a staphyloma by cutting.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Staple</ent><br/
<hw>Sta"ple</hw> <pr>(st<amac/"p'l)</pr>, <pos>n.</pos> <ety>[AS. <ets>stapul</ets>, <ets>stapol</ets>, <ets>stapel</ets>, a step, a prop, post, table, fr. <ets>stapan</ets> to step, go, raise; akin to D. <ets>stapel</ets> a pile, stocks, emporium, G. <ets>stapel</ets>a heap, mart, stake, <ets>staffel</ets> step of a ladder, Sw. <ets>stapel</ets>, Dan. <ets>stabel</ets>, and E. <ets>step</ets>; cf. OF. <ets>estaple</ets> a mart, F. <ets><eacute/tape</ets>. See <er>Step</er>.]</ety> <sn>1.</sn> <def>A settled mart; an emporium; a city or town to which merchants brought commodities for sale or exportation in bulk; a place for wholesale traffic.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The customs of Alexandria were very great, it having been the <qex>staple</qex> of the Indian trade.</q> <rj><qau>Arbuthnot.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>For the increase of trade and the encouragement of the worthy burgesses of Woodstock, her majesty was minded to erect the town into a <qex>staple</qex> for wool.</q> <rj><qau>Sir W. Scott.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ In England, formerly, the king's <xex>staple</xex> was established in certain ports or towns, and certain goods could not be exported without being first brought to these places to be rated and charged with the duty payable to the king or the public. The principal commodities on which customs were levied were wool, skins, and leather; and these were originally the <ex>staple</ex> commodities.</note><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Hence: Place of supply; source; fountain head.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Whitehall naturally became the chief <qex>staple</qex> of news. Whenever there was a rumor that any thing important had happened or was about to happen, people hastened thither to obtain intelligence from the fountain head.</q> <rj><qau>Macaulay.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>The principal commodity of traffic in a market; a principal commodity or production of a country or district; <as>as, wheat, maize, and cotton are great <ex>staples</ex> of the United States</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>We should now say, Cotton is the great <qex>staple</qex>, that is, the established merchandise, of Manchester.</q> <rj><qau>Trench.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>The principal constituent in anything; chief item.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <def>Unmanufactured material; raw material.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>6.</sn> <def>The fiber of wool, cotton, flax, or the like; <as>as, a coarse <ex>staple</ex>; a fine <ex>staple</ex>; a long or short <ex>staple</ex></as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>7.</sn> <def>A loop of metal such as iron, or a bar or wire, bent and formed with two points to be driven into wood, to hold a hook, pin, or the like.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>8.</sn> <specif>Specifically:</specif> <def>A small loop of metal such as steel, bent into a U-shape with the points sharpened, used to fasten sheets of paper together by driving the staple{8} through the stacked sheets and into a formed receptacle which curls the ends in and backward, thus holding the papers firmly together; also, a similar, slightly larger such fastener which may be driven into wood to fasten objects to a wooden backing.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>9.</sn> <fld>(Mining)</fld> <sd>(a)</sd> <def>A shaft, smaller and shorter than the principal one, joining different levels.</def> <sd>(b)</sd> <def>A small pit.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>10.</sn> <def>A district granted to an abbey.</def> <mark>[Obs.]</mark>  <rj><au>Camden.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Staple</ent><br/
<hw>Sta"ple</hw>, <pos>a.</pos> <sn>1.</sn> <def>Pertaining to, or being a market or staple for, commodities; <as>as, a <ex>staple</ex> town</as>.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Established in commerce; occupying the markets; settled; <as>as, a <ex>staple</ex> trade</as>.</def>  <rj><au>Dryden.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>Fit to be sold; marketable.</def> <mark>[R.]</mark>  <rj><au>Swift.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>Regularly produced or manufactured in large quantities; belonging to wholesale traffic; principal; chief.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Wool, the great <qex>staple</qex> commodity of England.</q> <rj><qau>Hallam.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>staple</ent><br/
<hw>sta"ple</hw>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>stapled</conjf> <pr>(-p'ld)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>stapling</conjf>.]</vmorph> <sn>1.</sn> <def>To sort according to its staple; <as>as, to <ex>staple</ex> cotton</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To fasten together with a staple{9} or staples; <as>as, to <ex>staple</ex> a check to a letter</as>.</def><br/
[<source>PJC</source>]</p>

<p><ent>staple gun</ent><br/
<hw>sta"ple gun`</hw>, <pos>n.</pos> <def>A device used to drive a heavy staple{8} through multiple objects, so as to fasten them together; it has a spring mechanism which stores force as a lever is pulled by the operator's hand, and the force is released all at once to drive the staple fully into the object to be fastened.</def><br/
[<source>PJC</source>]</p>

<p><ent>Stapler</ent><br/
<hw>Sta"pler</hw> <pr>(-pl<etil/r)</pr>, <pos>n.</pos> <sn>1.</sn> <def>A dealer in staple goods.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>One employed to assort wool according to its staple.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>A device used to drive a staple{8} into objects so as to fasten them together.</def><br/
[<source>PJC</source>]</p>

<p><ent>Star</ent><br/
<hw>Star</hw> <pr>(st<aum/r)</pr>, <pos>n.</pos> <ety>[OE. <ets>sterre</ets>, AS. <ets>steorra</ets>; akin to OFries. <ets>stera</ets>, OS. <ets>sterro</ets>, D. <ets>ster</ets>, OHG. <ets>sterno</ets>, <ets>sterro</ets>, G. <ets>stern</ets>, Icel. <ets>stjarna</ets>, Sw. <ets>stjerna</ets>, Dan. <ets>stierne</ets>, Goth. <ets>sta<iacute/rn<omac/</ets>, Armor. & Corn. <ets>steren</ets>, L. <ets>stella</ets>, Gr. <grk>'asth`r</grk>, <grk>'a`stron</grk>, Skr. <ets>star</ets>; perhaps from a root meaning, to scatter, Skr. <ets>st<rsdot/</ets>, L. <ets>sternere</ets> (cf. <er>Stratum</er>), and originally applied to the stars as being strewn over the sky, or as being scatterers or spreaders of light. <root/296.  Cf. <er>Aster</er>, <er>Asteroid</er>, <er>Constellation</er>, <er>Disaster</er>, <er>Stellar</er>.]</ety> <sn>1.</sn> <def>One of the innumerable luminous bodies seen in the heavens; any heavenly body other than the sun, moon, comets, and nebulae.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>His eyen twinkled in his head aright,<br/
As do the <qex>stars</qex> in the frosty night.</q> <rj><qau>Chaucer.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ The stars are distinguished as <stype>planets</stype>, and <stype>fixed stars</stype>. See <er>Planet</er>, <cref>Fixed stars</cref> under <er>Fixed</er>, and <cref>Magnitude of a star</cref> under <er>Magnitude</er>.</note><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>The polestar; the north star.</def>  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <fld>(Astrol.)</fld> <def>A planet supposed to influence one's destiny; (usually <pluf>pl.</pluf>) a configuration of the planets, supposed to influence fortune.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>O malignant and ill-brooding <qex>stars</qex>.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Blesses his <qex>stars</qex>, and thinks it luxury.</q> <rj><qau>Addison.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>That which resembles the figure of a star, as an ornament worn on the breast to indicate rank or honor.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>On whom . . . <br/
Lavish Honor showered all her <qex>stars</qex>.</q> <rj><qau>Tennyson.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <def>Specifically, a radiated mark in writing or printing; an asterisk [thus, *]; -- used as a reference to a note, or to fill a blank where something is omitted, etc.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>6.</sn> <fld>(Pyrotechny)</fld> <def>A composition of combustible matter used in the heading of rockets, in mines, etc., which, exploding in the air, presents a starlike appearance.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>7.</sn> <def>A person of brilliant and attractive qualities, especially on public occasions, as a distinguished orator, a leading theatrical performer, etc.</def><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ <xex>Star</xex> is used in the formation of compound words generally of obvious signification; as, <xex>star</xex>-aspiring, <xex>star</xex>-bespangled, <xex>star</xex>-bestudded, <xex>star</xex>-blasting, <xex>star</xex>-bright, <xex>star</xex>-crowned, <xex>star</xex>-directed, <xex>star</xex>-eyed, <xex>star</xex>-headed, <xex>star</xex>-paved, <xex>star</xex>-roofed, <xex>star</xex>-sprinkled, <xex>star</xex>-wreathed.</note><br/
[<source>1913 Webster</source>]</p>

<p><-- p. 1404 pr=VMG --></p>

<p><cs><mcol><col><b>Blazing star</b></col>, <col><b>Double star</b></col>, <col><b>Multiple star</b></col>, <col><b>Shooting star</b></col>, etc.</mcol> <cd>See under <er>Blazing</er>, <er>Double</er>, etc.</cd> -- <col><b>Nebulous star</b></col> <fld>(Astron.)</fld>, <cd>a small well-defined circular nebula, having a bright nucleus at its center like a star.</cd> -- <col><b>Star anise</b></col> <fld>(Bot.)</fld>, <cd>any plant of the genus Illicium; -- so called from its star-shaped capsules.</cd> -- <col><b>Star apple</b></col> <fld>(Bot.)</fld>, <cd>a tropical American tree (<spn>Chrysophyllum Cainito</spn>), having a milky juice and oblong leaves with a silky-golden pubescence beneath. It bears an applelike fruit, the carpels of which present a starlike figure when cut across. The name is extended to the whole genus of about sixty species, and the natural order (<ord>Sapotaceae</ord>) to which it belongs is called the <xex>Star-apple family</xex>.</cd> -- <col><b>Star conner</b></col>, <cd>one who cons, or studies, the stars; an astronomer or an astrologer.</cd> <au>Gascoigne.</au> -- <col><b>Star coral</b></col> <fld>(Zool.)</fld>, <cd>any one of numerous species of stony corals belonging to <gen>Astraea</gen>, <gen>Orbicella</gen>, and allied genera, in which the calicles are round or polygonal and contain conspicuous radiating septa.</cd> -- <col><b>Star cucumber</b></col>. <fld>(Bot.)</fld> <cd>See under <er>Cucumber</er>.</cd> -- <col><b>Star flower</b></col>. <fld>(Bot.)</fld> <sd>(a)</sd> <cd>A plant of the genus <gen>Ornithogalum</gen>; star-of-Bethlehem</cd>. <sd>(b)</sd> <cd>See <er>Starwort</er> <sd>(b)</sd></cd>. <sd>(c)</sd> <cd>An American plant of the genus <gen>Trientalis</gen> (<spn>Trientalis Americana</spn>)</cd>. <au>Gray.</au> -- <col><b>Star fort</b></col> <fld>(Fort.)</fld>, <cd>a fort surrounded on the exterior with projecting angles; -- whence the name.</cd> -- <col><b>Star gauge</b></col> <fld>(Ordnance)</fld>, <cd>a long rod, with adjustable points projecting radially at its end, for measuring the size of different parts of the bore of a gun.</cd> -- <col><b>Star grass</b></col>. <fld>(Bot.)</fld> <sd>(a)</sd> <cd>A small grasslike plant (<spn>Hypoxis erecta</spn>) having star-shaped yellow flowers</cd>. <sd>(b)</sd> <cd>The colicroot. See <er>Colicroot</er>.</cd> -- <col><b>Star hyacinth</b></col> <fld>(Bot.)</fld>, <cd>a bulbous plant of the genus <gen>Scilla</gen> (<spn>Scilla autumnalis</spn>); -- called also <altname>star-headed hyacinth</altname>.</cd> -- <col><b>Star jelly</b></col> <fld>(Bot.)</fld>, <cd>any one of several gelatinous plants (<spn>Nostoc commune</spn>, <spn>Nostoc edule</spn>, etc.). See <er>Nostoc</er>.</cd> -- <col><b>Star lizard</b></col>. <fld>(Zool.)</fld> <cd>Same as <er>Stellion</er>.</cd> -- <col><b>Star-of-Bethlehem</b></col> <fld>(Bot.)</fld>, <cd>a bulbous liliaceous plant (<spn>Ornithogalum umbellatum</spn>) having a small white starlike flower.</cd> -- <col><b>Star-of-the-earth</b></col> <fld>(Bot.)</fld>, <cd>a plant of the genus <gen>Plantago</gen> (<spn>Plantago coronopus</spn>), growing upon the seashore.</cd> -- <col><b>Star polygon</b></col> <fld>(Geom.)</fld>, <cd>a polygon whose sides cut each other so as to form a star-shaped figure.</cd> -- <col><b>Stars and Stripes</b></col>, <cd>a popular name for the flag of the United States, which consists of thirteen horizontal stripes, alternately red and white, and a union having, in a blue field, white stars to represent the several States, one for each.</cd><br/
<br/
<q>With the old flag, the true American flag, the Eagle, and the <qex>Stars and Stripes</qex>, waving over the chamber in which we sit.</q> <rj><qau>D. Webster.</qau></rj><br/
<br/
  -- <col><b>Star showers</b></col>. <cd>See <cref>Shooting star</cref>, under <er>Shooting</er>.</cd> -- <col><b>Star thistle</b></col> <fld>(Bot.)</fld>, <cd>an annual composite plant (<spn>Centaurea solstitialis</spn>) having the involucre armed with stout radiating spines.</cd> -- <col><b>Star wheel</b></col> <fld>(Mach.)</fld>, <cd>a star-shaped disk, used as a kind of ratchet wheel, in repeating watches and the feed motions of some machines.</cd> -- <col><b>Star worm</b></col> <fld>(Zool.)</fld>, <cd>a gephyrean.</cd> -- <col><b>Temporary star</b></col> <fld>(Astron.)</fld>, <cd>a star which appears suddenly, shines for a period, and then nearly or quite disappears.  These stars were supposed by some astronomers to be variable stars of long and undetermined periods.  More recently, variations star in start intensity are classified more specifically, and this term is now obsolescent.  See also <er>nova</er>.</cd> <mark>[Obsolescent]</mark> -- <col><b>Variable star</b></col> <fld>(Astron.)</fld>, <cd>a star whose brilliancy varies periodically, generally with regularity, but sometimes irregularly; -- called <stype>periodical star</stype> when its changes occur at fixed periods.</cd> -- <col><b>Water star grass</b></col> <fld>(Bot.)</fld>, <cd>an aquatic plant (<spn>Schollera graminea</spn>) with small yellow starlike blossoms.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Star</ent><br/
<hw>Star</hw> <pr>(st<aum/r)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Starred</conjf> <pr>(st<aum/rd)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Starring</conjf>.]</vmorph> <def>To set or adorn with stars, or bright, radiating bodies; to bespangle; <as>as, a robe <ex>starred</ex> with gems</as>.</def> <ldquo/A sable curtain <xex>starred</xex> with gold.<rdquo/  <rj><au>Young.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Star</ent><br/
<hw>Star</hw>, <pos>v. i.</pos> <def>To be bright, or attract attention, as a star; to shine like a star; to be brilliant or prominent; to play a part as a theatrical star.</def>
<-- i.e., to be the most prominent or one of the two most prominent actors in the cast of a drama or film. -->  <rj><au>W. Irving.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Star-blind</ent><br/
<hw>Star"-blind`</hw> <pr>(-bl<imac/nd`)</pr>, <pos>a.</pos> <def>Half blind.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Starboard</ent><br/
<hw>Star"board`</hw> <pr>(-b<omac/rd` <or/ -b<etil/rd)</pr>, <pos>n.</pos> <ety>[OE. <ets>sterbord</ets>, AS. <ets>ste<oacute/rbord</ets>, i.e., steer board. See <er>Steer</er>, <pos>v. t.</pos>, <er>Board</er> of a vessel, and cf. <er>Larboard</er>.]</ety> <fld>(Naut.)</fld> <def>That side of a vessel which is on the right hand of a person who stands on board facing the bow; -- opposed to <ant>larboard</ant>, or <ant>port</ant>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Starboard</ent><br/
<hw>Star"board`</hw>, <pos>a.</pos> <fld>(Naut.)</fld> <def>Pertaining to the right-hand side of a ship; being or lying on the right side; <as>as, the <ex>starboard</ex> shrouds; <ex>starboard</ex> quarter; <ex>starboard</ex> tack.</as></def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Starboard</ent><br/
<hw>Star"board`</hw>, <pos>v. t.</pos> <fld>(Naut.)</fld> <def>To put to the right, or starboard, side of a vessel; <as>as, to <ex>starboard</ex> the helm</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Star-bowlines</ent><br/
<hw>Star"-bow`lines</hw> <pr>(-b<omac/`l<icr/nz)</pr>, <pos>n. pl.</pos> <fld>(Naut.)</fld> <def>The men in the starboard watch.</def> <mark>[Obs.]</mark>  <rj><au>R. H. Dana, Jr.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Starch</ent><br/
<hw>Starch</hw> <pr>(st<aum/rch)</pr>, <pos>a.</pos> <ety>[AS. <ets>stearc</ets> stark, strong, rough. See <er>Stark</er>.]</ety> <def>Stiff; precise; rigid.</def> <mark>[R.]</mark>  <rj><au>Killingbeck.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Starch</ent><br/
<hw>Starch</hw>, <pos>n.</pos> <ety>[From <ets>starch</ets> stiff, cf. G. <ets>st<aum/rke</ets>, fr. <ets>stark</ets> strong.]</ety> <sn>1.</sn> <fld>(Chem.)</fld> <def>A widely diffused vegetable substance found especially in seeds, bulbs, and tubers, and extracted (as from potatoes, corn, rice, etc.) as a white, glistening, granular or powdery substance, without taste or smell, and giving a very peculiar creaking sound when rubbed between the fingers. It is used as a food, in the production of commercial grape sugar, for stiffening linen in laundries, in making paste, etc.</def><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ Starch is a carbohydrate, being the typical amylose, <chform>C6H10O5</chform>, and is detected by the fine blue color given to it by free iodine. It is not fermentable as such, but is changed by diastase into dextrin and maltose, and by heating with dilute acids into dextrose.  Cf. <er>Sugar</er>, <er>Inulin</er>, and <er>Lichenin</er>.</note><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Fig.: A stiff, formal manner; formality.</def>  <rj><au>Addison.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Starch hyacinth</b></col> <fld>(Bot.)</fld>, <cd>the grape hyacinth; -- so called because the flowers have the smell of boiled starch. See under <er>Grape</er>.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Starch</ent><br/
<hw>Starch</hw>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Starched</conjf> <pr>(st<aum/rcht)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Starching</conjf>.]</vmorph> <def>To stiffen with starch.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Star-chamber</ent><br/
<ent>Star chamber</ent><br/
<mhw><hw>Star" cham`ber</hw>, <hw>Star"-cham`ber</hw></mhw> <pr>(st<aum/r"ch<amac/m`b<etil/r)</pr>, <pos>n.</pos> <ety>[So called (as conjectured by Blackstone) from being held in a room at the Exchequer where the chests containing certain Jewish contracts and obligations called <ets>starrs</ets> (from the Heb. <ets>shetar</ets>, pron. <ets>shtar</ets>) were kept; or from the <ets>stars</ets> with which the ceiling is supposed to have been decorated.]</ety> <sn>1.</sn> <fld>(Eng. Hist.)</fld> <def>An ancient high court exercising jurisdiction in certain cases, mainly criminal, which sat without the intervention of a jury.  It consisted of the king's council, or of the privy council only with the addition of certain judges.  It could proceed on mere rumor or examine witnesses; it could apply torture.  It was abolished by the Long Parliament in 1641.</def>  <rj><au>Encyc. Brit.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <specif>Hence:</specif> <mark>(Metaphorical)</mark> <def>Any court, committee, or other tribunal which exercises arbitrary and unaccountable power, or uses unfair or illegal methods, in investigation or judgment of persons.</def><br/
[<source>PJC</source>]</p>

<p><ent>Starched</ent><br/
<hw>Starched</hw> <pr>(st<aum/rcht)</pr>, <pos>a.</pos> <sn>1.</sn> <def>Stiffened with starch.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Stiff; precise; formal.</def>  <rj><au>Swift.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Starchedness</ent><br/
<hw>Starch"ed*ness</hw> <pr>(st<aum/rch"<ecr/d*n<ecr/s)</pr>, <pos>n.</pos> <def>The quality or state of being starched or starchy{5}; stiffness in manners; formality.</def><br/
<syn><b>Syn. --</b> starchiness, starchness.</syn><br/
[<source>1913 Webster</source>]</p>

<p><ent>Starcher</ent><br/
<hw>Starch"er</hw> <pr>(-<etil/r)</pr>, <pos>n.</pos> <def>One who starches.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>starchiness</ent><br/
<hw>starch"i*ness</hw> <pr>(st<aum/rch"<esl/*n<ecr/s)</pr>, <pos>n.</pos> <def>The quality or state of being starchy; stiffness in manners; formality.</def><br/
<syn><b>Syn. --</b> starchedness, starchness.</syn><br/
[<source>1913 Webster</source>]</p>

<p><ent>Starchly</ent><br/
<hw>Starch"ly</hw>, <pos>adv.</pos> <def>In a starched or starch manner.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Starchness</ent><br/
<hw>Starch"ness</hw>, <pos>n.</pos> <def>The quality or state of being starched or starchy{5}; stiffness of manner; preciseness.</def><br/
<syn><b>Syn. --</b> starchedness.</syn><br/
[<source>1913 Webster</source>]</p>

<p><ent>Starchwort</ent><br/
<hw>Starch"wort`</hw> <pr>(-w<ucir/rt`)</pr>, <pos>n.</pos> <fld>(Bot.)</fld> <def>The cuckoopint, the tubers of which yield a fine quality of starch.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>starchy</ent><br/
<hw>starch"y</hw> <pr>(-<ycr/)</pr>, <pos>a.</pos> <sn>1.</sn> <def>Of or pertaining to starch.</def><br/
[<source>PJC</source>]</p>

<p><sn>2.</sn> <def>Containing or consisting of starch; -- used especially of foods.</def><br/
[<source>PJC</source>]</p>

<p><sn>3.</sn> <def>Resembling starch; stiff.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>Stiffened by use of starch; starched{1}; -- of clothing.</def><br/
[<source>PJC</source>]</p>

<p><sn>5.</sn> <def>Formal in manner; precise.</def><br/
<syn><b>Syn. --</b> starch.</syn><br/
[<source>1913 Webster</source>]</p>

<p><ent>Starcraft</ent><br/
<hw>Star"craft</hw> <pr>(st<aum/r"kr<adot/ft)</pr>, <pos>n.</pos> <def>Astrology.</def> <mark>[R.]</mark>  <rj><au>Tennyson.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Star-crossed</ent><br/
<hw>Star"-crossed`</hw> <pr>(st<aum/r"kr<ocr/st` ; 115)</pr>, <pos>a.</pos> <def>Not favored by the stars; ill-fated.</def> <mark>[Poetic]</mark>  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Such is my <qex>star-crossed</qex> destiny.</q> <rj><qau>Massinger.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Star drift</ent><br/
<hw>Star drift</hw>. <fld>(Astron.)</fld> <def>Similar and probably related motion of the stars of an asterism, as distinguished from apparent change of place due to solar motion.-- ##?? = star streaming? --</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Stare</ent><br/
<hw>Stare</hw> <pr>(st<acir/r)</pr>, <pos>n.</pos> <ety>[AS. <ets>staer</ets>. See <er>Starling</er>.]</ety> <fld>(Zool.)</fld> <def>The starling.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>stare</ent><br/
<hw>stare</hw>, <pos>v. i.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>stared</conjf> <pr>(st<acir/rd)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>staring</conjf>.]</vmorph> <ety>[AS. <ets>starian</ets>; akin to LG. & D. <ets>staren</ets>, OHG. <ets>star<emac/n</ets>, G. <ets>starren</ets>, Icel. <ets>stara</ets>; cf. Icel. <ets>stira</ets>, Dan. <ets>stirre</ets>, Sw. <ets>stirra</ets>, and G. <ets>starr</ets> stiff, rigid, fixed, Gr. <grk>stereo`s</grk> solid (E. <ets>stereo-</ets>), Skr. <ets>sthira</ets> firm, strong.  <root/166.  Cf. <er>Sterile</er>.]</ety> <sn>1.</sn> <def>To look with fixed eyes wide open, as through fear, wonder, surprise, impudence, etc.; to fasten an earnest and prolonged gaze on some object.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>For ever upon the ground I see thee <qex>stare</qex>.</q> <rj><qau>Chaucer.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Look not big, nor stamp, nor <qex>stare</qex>, nor fret.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To be very conspicuous on account of size, prominence, color, or brilliancy; <as>as, <ex>staring</ex> windows or colors</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>To stand out; to project; to bristle.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>Makest my blood cold, and my hair to <qex>stare</qex>.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Take off all the <qex>staring</qex> straws and jags in the hive.</q> <rj><qau>Mortimer.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><syn><b>Syn.</b> -- To gaze; to look earnestly. See <er>Gaze</er>.</syn><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stare</ent><br/
<hw>Stare</hw> <pr>(st<acir/r)</pr>, <pos>v. t.</pos> <def>To look earnestly at; to gaze at.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>I will <qex>stare</qex> him out of his wits.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>To stare in the face</b></col>, <cd>to be before the eyes, or to be undeniably evident.</cd> <ldquo/The law . . . <xex>stares</xex> them <xex>in the face</xex> whilst they are breaking it.<rdquo/ <au>Locke.</au></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stare</ent><br/
<hw>Stare</hw>, <pos>n.</pos> <def>The act of staring; a fixed look with eyes wide open.</def> <ldquo/A dull and stupid <xex>stare</xex>.<rdquo/  <rj><au>Churchill.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Starer</ent><br/
<hw>Star"er</hw> <pr>(st<acir/r"<etil/r)</pr>, <pos>n.</pos> <def>One who stares, or gazes.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Starf</ent><br/
<hw>Starf</hw> <pr>(st<aum/rf)</pr>, <mark>obs.</mark> <pos>imp.</pos> <mord>of <er>Starve</er></mord>. <def>Starved.</def>  <rj><au>Chaucer.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Starfinch</ent><br/
<hw>Star"finch`</hw> <pr>(st<aum/r"f<icr/nch`)</pr>, <pos>n.</pos> <fld>(Zool.)</fld> <def>The European redstart.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Starfish</ent><br/
<hw>Star"fish`</hw> <pr>(-f<icr/sh`)</pr>, <pos>n.</pos> <sn>1.</sn> <fld>(Zool.)</fld> <def>Any one of numerous species of echinoderms belonging to the class <class>Asterioidea</class>, in which the body is star-shaped and usually has five rays, though the number of rays varies from five to forty or more.  The rays are often long, but are sometimes so short as to appear only as angles to the disklike body. Called also <altname>sea star</altname>, <altname>five-finger</altname>, and <altname>stellerid</altname>.</def><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ The ophiuroids are also sometimes called starfishes. See <er>Brittle star</er>, and <er>Ophiuroidea</er>.</note><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Zool.)</fld> <def>The dollar fish, or butterfish.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stargazer</ent><br/
<hw>Star"gaz`er</hw> <pr>(-g<amac/z`<etil/r)</pr>, <pos>n.</pos> <sn>1.</sn> <def>One who gazes at the stars; an astrologer; sometimes, in derision or contempt, an astronomer.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Zool.)</fld> <def>Any one of several species of spiny-rayed marine fishes belonging to <gen>Uranoscopus</gen>, <gen>Astroscopus</gen>, and allied genera, of the family <fam>Uranoscopidae</fam>.  The common species of the Eastern United States are <spn>Astroscopus anoplus</spn>, and <spn>Astroscopus guttatus</spn>.  So called from the position of the eyes, which look directly upward.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stargazing</ent><br/
<hw>Star"gaz`ing</hw>, <pos>n.</pos> <sn>1.</sn> <def>The act or practice of observing the stars with attention; contemplation of the stars as connected with astrology or astronomy.</def>  <rj><au>Swift.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Hence, absent-mindedness; abstraction.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Staringly</ent><br/
<hw>Star"ing*ly</hw> <pr>(st<acir/r"<icr/ng*l<ycr/)</pr>, <pos>adv.</pos> <def>With a staring look.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stark</ent><br/
<hw>Stark</hw> <pr>(st<aum/rk)</pr>, <pos>a.</pos> <amorph>[<pos>Compar.</pos> <adjf>Starker</adjf> <pr>(-<etil/r)</pr>; <pos>superl.</pos> <adjf>Starkest</adjf>.]</amorph> <ety>[OE. <ets>stark</ets> stiff, strong, AS. <ets>stearc</ets>; akin to OS. <ets>starc</ets> strong, D. <ets>sterk</ets>, OHG. <ets>starc</ets>, <ets>starah</ets>, G. & Sw. <ets>stark</ets>, Dan. <ets>staerk</ets>, Icel. <ets>sterkr</ets>, Goth. ga<ets>sta<uacute/rknan</ets> to become dried up, Lith. <ets>str<eum/gti</ets> to stiffen, to freeze.  Cf. <er>Starch</er>, <pos>a.</pos> & <pos>n.</pos>]</ety> <sn>1.</sn> <def>Stiff; rigid.</def>  <rj><au>Chaucer.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Whose senses all were straight benumbed and <qex>stark</qex>.</q> <rj><qau>Spenser.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>His heart gan wax as <qex>stark</qex> as marble stone.</q> <rj><qau>Spenser.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Many a nobleman lies <qex>stark</qex> and stiff<br/
Under the hoofs of vaunting enemies.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>The north is not so <qex>stark</qex> and cold.</q> <rj><qau>B. Jonson.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Complete; absolute; full; perfect; entire.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>Consider the <qex>stark</qex> security<br/
The common wealth is in now.</q> <rj><qau>B. Jonson.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>Strong; vigorous; powerful.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>A <qex>stark</qex>, moss-trooping Scot.</q> <rj><qau>Sir W. Scott.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q><qex>Stark</qex> beer, boy, stout and strong beer.</q> <rj><qau>Beau. & Fl.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>Severe; violent; fierce.</def> <mark>[Obs.]</mark> <ldquo/In <xex>starke</xex> stours<rdquo/  [<it>i. e.</it>, in fierce combats].  <rj><au>Chaucer.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <def>Mere; sheer; gross; entire; downright.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>He pronounces the citation <qex>stark</qex> nonsense.</q> <rj><qau>Collier.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Rhetoric is very good or <qex>stark</qex> naught; there's no medium in rhetoric.</q> <rj><qau>Selden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stark</ent><br/
<hw>Stark</hw> <pr>(st<aum/rk)</pr>, <pos>adv.</pos> <def>Wholly; entirely; absolutely; quite; <as>as, <ex>stark</ex> mad</as>.</def>  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Held him strangled in his arms till he was <qex>stark</qex> dead.</q> <rj><qau>Fuller.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Stark naked</b></col>, <cd>wholly naked; quite bare.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><q>Strip your sword <qex>stark naked</qex>.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ According to Professor Skeat, <ldquo/<xex>stark-naked</xex><rdquo/ is derived from <xex>steort-naked</xex>, or <xex>start-naked</xex>, literally <xex>tail-naked</xex>, and hence <xex>wholly naked</xex>. If this etymology be true the preferable form is <xex>stark-naked</xex>.</note><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stark</ent><br/
<hw>Stark</hw>, <pos>v. t.</pos> <def>To stiffen.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>If horror have not <qex>starked</qex> your limbs.</q> <rj><qau>H. Taylor.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Starkly</ent><br/
<hw>Stark"ly</hw>, <pos>adv.</pos> <def>In a stark manner; stiffly; strongly.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Its onward force too <qex>starkly</qex> pent<br/
In figure, bone, and lineament.</q> <rj><qau>Emerson.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Starkness</ent><br/
<hw>Stark"ness</hw>, <pos>n.</pos> <def>The quality or state of being stark.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Starless</ent><br/
<hw>Star"less</hw> <pr>(st<aum/r"l<ecr/s)</pr>, <pos>a.</pos> <def>Being without stars; having no stars visible; <as>as, a <ex>starless</ex> night</as>.</def>  <rj><au>Milton.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Starlight</ent><br/
<hw>Star"light`</hw> <pr>(-l<imac/t`)</pr>, <pos>n.</pos> <def>The light given by the stars.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Nor walk by moon,<br/
Or glittering <qex>starlight</qex>, without thee is sweet.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Starlight</ent><br/
<hw>Star"light`</hw>, <pos>a.</pos> <def>Lighted by the stars, or by the stars only; <as>as, a <ex>starlight</ex> night</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>A <qex>starlight</qex> evening and a morning fair.</q> <rj><qau>Dryden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Starlike</ent><br/
<hw>Star"like`</hw> <pr>(-l<imac/k`)</pr>, <pos>a.</pos> <sn>1.</sn> <def>Resembling a star; stellated; radiated like a star; <as>as, <ex>starlike</ex> flowers</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Shining; bright; illustrious.</def>  <rj><au>Dryden.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>The having turned many to righteousness shall confer a <qex>starlike</qex> and immortal brightness.</q> <rj><qau>Boyle.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Starling</ent><br/
<hw>Star"ling</hw> <pr>(-l<icr/ng)</pr>, <pos>n.</pos> <ety>[OE. <ets>sterlyng</ets>, a dim. of OE. <ets>stare</ets>, AS. <ets>staer</ets>; akin to AS. <ets>stearn</ets>, G. <ets>star</ets>, <ets>staar</ets>, OHG. <ets>stara</ets>, Icel. <ets>starri</ets>, <ets>stari</ets>, Sw. <ets>stare</ets>, Dan. <ets>staer</ets>, L. <ets>sturnus</ets>.  Cf. <er>Stare</er> a starling.]</ety> <sn>1.</sn> <fld>(Zool.)</fld> <def>Any passerine bird belonging to <gen>Sturnus</gen> and allied genera. The European starling (<spn>Sturnus vulgaris</spn>) is dark brown or greenish black, with a metallic gloss, and spotted with yellowish white. It is a sociable bird, and builds about houses, old towers, etc. Called also <altname>stare</altname>, and <altname>starred</altname>. The pied starling of India is <spn>Sternopastor contra</spn>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Zool.)</fld> <def>A California fish; the rock trout.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>A structure of piles driven round the piers of a bridge for protection and support; -- called also <altname>sterling</altname>.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Rose-colored starling</b></col>. <fld>(Zool.)</fld> <cd>See <er>Pastor</er>.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Starlit</ent><br/
<hw>Star"lit`</hw> <pr>(-l<icr/t`)</pr>, <pos>a.</pos> <def>Lighted by the stars; starlight.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Starmonger</ent><br/
<hw>Star"mon`ger</hw> <pr>(-m<ucr/<nsm/`g<etil/r)</pr>, <pos>n.</pos> <def>A fortune teller; an astrologer; -- used in contempt.</def>  <rj><au>B. Jonson.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Starn</ent><br/
<hw>Starn</hw> <pr>(st<aum/rn)</pr>, <pos>n.</pos> <fld>(Zool.)</fld> <def>The European starling.</def> <mark>[Prov. Eng.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Starnose</ent><br/
<hw>Star"nose`</hw> <pr>(st<aum/r"n<omac/z`)</pr>, <pos>n.</pos> <fld>(Zool.)</fld> <def>A curious American mole (<spn>Condylura cristata</spn>) having the nose expanded at the end into a stellate disk; -- called also <altname>star-nosed mole</altname>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Star of David</ent><br/
<hw>Star" of Dav"id</hw>. <def>A symbol consisting of two overlapped equilateral triangles forming a star with six points, used as a symbol of Judaism.  It is also called <altname>Magen David</altname>, <altname>Mogen David</altname>, and <altname>Shield of David</altname>, and is shaped identically to the <er>hexagram</er> and <er>Solomon's seal</er>.  It is used on the flag of the modern state of Israel.</def><br/
[<source>PJC</source>]</p>

<p><ent>Starost</ent><br/
<hw>Star"ost</hw> <pr>(st<acr/r"<ocr/st)</pr>, <pos>n.</pos> <ety>[Pol. <ets>starosta</ets>, from <ets>stary</ets> old.]</ety> <def>A nobleman who possessed a starosty.</def> <mark>[Poland]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Starosty</ent><br/
<hw>Star"os*ty</hw> <pr>(-<ocr/s*t<ycr/)</pr>, <pos>n.</pos> <def>A castle and domain conferred on a nobleman for life.</def> <mark>[Poland]</mark>  <rj><au>Brande & C.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Starproof</ent><br/
<hw>Star"proof`</hw> <pr>(st<aum/r"pr<oomac/f`)</pr>, <pos>a.</pos> <def>Impervious to the light of the stars; <as>as, a <ex>starproof</ex> elm</as>.</def> <mark>[Poetic]</mark>  <rj><au>Milton.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Star-read</ent><br/
<hw>Star"-read`</hw> <pr>(-r<emac/d`)</pr>, <pos>n.</pos> <def>Doctrine or knowledge of the stars; star lore; astrology; astronomy.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>Which in <qex>star-read</qex> were wont have best insight.</q> <rj><qau>Spenser.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Starred</ent><br/
<hw>Starred</hw> <pr>(st<aum/rd)</pr>, <pos>a.</pos> <ety>[From <er>Star</er>.]</ety> <sn>1.</sn> <def>Adorned or studded with stars; bespangled.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Influenced in fortune by the stars.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>My third comfort,<br/
<qex>Starred</qex> most unluckily.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Starriness</ent><br/
<hw>Star"ri*ness</hw> <pr>(st<aum/r"r<icr/*n<ecr/s)</pr>, <pos>n.</pos> <def>The quality or state of being starry; <as>as, the <ex>starriness</ex> of the heavens</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Starry</ent><br/
<hw>Star"ry</hw> <pr>(st<aum/r"r<ycr/)</pr>, <pos>a.</pos> <sn>1.</sn> <def>Abounding with stars; adorned with stars.</def> <ldquo/Above the <xex>starry</xex> sky.<rdquo/  <rj><au>Pope.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Consisting of, or proceeding from, the stars; stellar; stellary; <as>as, <ex>starry</ex> light; <ex>starry</ex> flame</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Do not Christians and Heathens, Jews and Gentiles, poets and philosophers, unite in allowing the <qex>starry</qex> influence?</q> <rj><qau>Sir W. Scott.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>Shining like stars; sparkling; <as>as, <ex>starry</ex> eyes</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>Arranged in rays like those of a star; stellate.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Starry ray</b></col> <fld>(Zool.)</fld>, <cd>a European skate (<spn>Raia radiata</spn>); -- so called from the stellate bases of the dorsal spines.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Starshine</ent><br/
<hw>Star"shine`</hw> <pr>(-sh<imac/n`)</pr>, <pos>n.</pos> <def>The light of the stars.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>The <qex>starshine</qex> lights upon our heads.</q> <rj><qau>R. L. Stevenson.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Starshoot</ent><br/
<hw>Star"shoot`</hw> <pr>(-sh<oomac/t`)</pr>, <pos>n.</pos> <def>See <er>Nostoc</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Star-spangled</ent><br/
<hw>Star"-span`gled</hw> <pr>(st<aum/r"-sp<acr/<nsm/`g'ld)</pr>, <pos>a.</pos> <def>Spangled or studded with stars.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Star-spangled banner</b></col>, <cd>the popular name for the national ensign of the United States; also the name of a poem, the words of which were composed by Francis Scott Key, which was adopted as the national anthem of the United States.</cd> <au>F. S. Key.</au></cs><br/
[<source>1913 Webster</source> <source>+PJC</source>]</p>

<p><ent>Star stereogram</ent><br/
<hw>Star stereogram</hw>. <def>A view of the universe of brighter stars as it would appear to an observer transported into space outside or beyond our universe of stars.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Starstone</ent><br/
<hw>Star"stone`</hw> <pr>(st<aum/r"st<omac/n`)</pr>, <pos>n.</pos> <fld>(Min.)</fld> <def>Asteriated sapphire.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>start</ent><br/
<hw>start</hw> <pr>(st<aum/rt)</pr>, <pos>v. i.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>started</conjf>; <pos>p. pr. & vb. n.</pos> <conjf>starting</conjf>.]</vmorph> <ety>[OE. <ets>sterten</ets>; akin to D. <ets>storten</ets> to hurl, rush, fall, G. <ets>st<uum/rzen</ets>, OHG. <ets>sturzen</ets> to turn over, to fall, Sw. <ets>st<oum/rta</ets> to cast down, to fall, Dan. <ets>styrte</ets>, and probably also to E. <ets>start</ets> a tail; the original sense being, perhaps, to show the tail, to tumble over suddenly.  <root/166.  Cf. <er>Start</er> a tail.]</ety> <sn>1.</sn> <def>To leap; to jump.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To move suddenly, as with a spring or leap, from surprise, pain, or other sudden feeling or emotion, or by a voluntary act.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>And maketh him out of his sleep to <qex>start</qex>.</q> <rj><qau>Chaucer.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>I <qex>start</qex> as from some dreadful dream.</q> <rj><qau>Dryden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Keep your soul to the work when ready to <qex>start</qex> aside.</q> <rj><qau>I. Watts.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>But if he <qex>start</qex>,<br/
It is the flesh of a corrupted heart.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>To set out; to commence a course, as a race or journey; to begin; <as>as, to <ex>start</ex> in business</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>At once they <qex>start</qex>, advancing in a line.</q> <rj><qau>Dryden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>At intervals some bird from out the brakes<br/
<qex>Starts</qex> into voice a moment, then is still.</q> <rj><qau>Byron.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>To become somewhat displaced or loosened; <as>as, a rivet or a seam may <ex>start</ex> under strain or pressure</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>To start after</b></col>, <cd>to set out after; to follow; to pursue.</cd> -- <col><b>To start against</b></col>, <cd>to act as a rival candidate against.</cd> -- <col><b>To start for</b></col>, <cd>to be a candidate for, as an office.</cd> -- <col><b>To start up</b></col>, <cd>to rise suddenly, as from a seat or couch; to come suddenly into notice or importance.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><-- p. 1405 pr=VMG --></p>

<p><ent>Start</ent><br/
<hw>Start</hw> <pr>(st<aum/rt)</pr>, <pos>v. t.</pos> <sn>1.</sn> <def>To cause to move suddenly; to disturb suddenly; to startle; to alarm; to rouse; to cause to flee or fly; <as>as, the hounds <ex>started</ex> a fox</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Upon malicious bravery dost thou come<br/
To <qex>start</qex> my quiet?</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Brutus will <qex>start</qex> a spirit as soon as Caesar.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To bring into being or into view; to originate; to invent.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Sensual men agree in the pursuit of every pleasure they can <qex>start</qex>.</q> <rj><qau>Sir W. Temple.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>To cause to move or act; to set going, running, or flowing; <as>as, to <ex>start</ex> a railway train; to <ex>start</ex> a mill; to <ex>start</ex> a stream of water; to <ex>start</ex> a rumor; to <ex>start</ex> a business</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>I was engaged in conversation upon a subject which the people love to <qex>start</qex> in discourse.</q> <rj><qau>Addison.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>To move suddenly from its place or position; to displace or loosen; to dislocate; <as>as, to <ex>start</ex> a bone; the storm <ex>started</ex> the bolts in the vessel</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>One, by a fall in wrestling, <qex>started</qex> the end of the clavicle from the sternum.</q> <rj><qau>Wiseman.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <ety>[Perh. from D. <ets>storten</ets>, which has this meaning also.]</ety> <fld>(Naut.)</fld> <def>To pour out; to empty; to tap and begin drawing from; <as>as, to <ex>start</ex> a water cask</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Start</ent><br/
<hw>Start</hw>, <pos>n.</pos> <sn>1.</sn> <def>The act of starting; a sudden spring, leap, or motion, caused by surprise, fear, pain, or the like; any sudden motion, or beginning of motion.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The fright awakened Arcite with a <qex>start</qex>.</q> <rj><qau>Dryden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A convulsive motion, twitch, or spasm; a spasmodic effort.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>For she did speak in <qex>starts</qex> distractedly.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Nature does nothing by <qex>starts</qex> and leaps, or in a hurry.</q> <rj><qau>L'Estrange.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>A sudden, unexpected movement; a sudden and capricious impulse; a sally; <as>as, <ex>starts</ex> of fancy</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>To check the <qex>starts</qex> and sallies of the soul.</q> <rj><qau>Addison.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>The beginning, as of a journey or a course of action; first motion from a place; act of setting out; the outset; -- opposed to <ant>finish</ant>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The <qex>start</qex> of first performance is all.</q> <rj><qau>Bacon.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>I see you stand like greyhounds in the slips,<br/
Straining upon the <qex>start</qex>.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>At a start</b></col>, <cd>at once; in an instant.</cd> <mark>[Obs.]</mark></cs><br/
[<source>1913 Webster</source>]</p>

<p><q><qex>At a start</qex> he was betwixt them two.</q> <rj><qau>Chaucer.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><mcol><col><b>To get the start</b></col>, <it>or</it> <col><b>To have the start</b></col></mcol>, <cd>to begin before another; to gain or have the advantage in a similar undertaking; -- usually with <xex>of</xex>.</cd>  <ldquo/<xex>Get the start</xex> of the majestic world.<rdquo/ <au>Shak.</au> <ldquo/She might have forsaken him if he had not <xex>got the start</xex> of her.<rdquo/ <au>Dryden.</au></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Start</ent><br/
<hw>Start</hw>, <pos>n.</pos> <ety>[OE. <ets>stert</ets> a tail, AS. <ets>steort</ets>; akin to LG. <ets>stert</ets>, <ets>steert</ets>, D. <ets>staart</ets>, G. <ets>sterz</ets>, Icel. <ets>stertr</ets>, Dan. <ets>stiert</ets>, Sw. <ets>stjert</ets>. <root/166.  Cf. <ets>Stark naked</ets>, under <er>Stark</er>, <er>Start</er>, <pos>v. i.</pos>]</ety> <sn>1.</sn> <def>A tail, or anything projecting like a tail.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>The handle, or tail, of a plow; also, any long handle.</def> <mark>[Prov. Eng.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>The curved or inclined front and bottom of a water-wheel bucket.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <fld>(Mining)</fld> <def>The arm, or lever, of a gin, drawn around by a horse.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>START</ent><br/
<hw>START</hw> <pr>(st<aum/rt)</pr>, <pos>n.</pos> <ety>[From <ets>S</ets>trategic <ets>A</ets>rms <ets>R</ets>eduction <ets>T</ets>reaty.]</ety> <def>A Treaty between the United States and the Soviet Union which provided for stepwise reductions in the number of nuclear weapons possessed by each country.</def><br/
[<source>PJC</source>]</p>

<p><ent>Starter</ent><br/
<hw>Start"er</hw> <pr>(st<aum/rt"<etil/r)</pr>, <pos>n.</pos> <sn>1.</sn> <def>One who, or that which, starts; <as>as, a <ex>starter</ex> on a journey; the <ex>starter</ex> of a race</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A dog that rouses game.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Startful</ent><br/
<hw>Start"ful</hw> <pr>(st<aum/rt"f<usdot/l)</pr>, <pos>a.</pos> <def>Apt to start; skittish.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Startfulness</ent><br/
<hw>Start"ful*ness</hw>, <pos>n.</pos> <def>Aptness to start.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Starthroat</ent><br/
<hw>Star"throat`</hw> <pr>(st<aum/r"thr<omac/t`)</pr>, <pos>n.</pos> <fld>(Zool.)</fld> <def>Any humming bird of the genus <gen>Heliomaster</gen>.  The feathers of the throat have a brilliant metallic luster.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Starting</ent><br/
<hw>Start"ing</hw> <pr>(st<aum/rt"<icr/ng)</pr>, <def><pos>a. & n.</pos> from <er>Start</er>, <pos>v.</pos></def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Starting bar</b></col> <fld>(Steam Eng.)</fld>, <cd>a hand lever for working the valves in starting an engine.</cd> -- <col><b>Starting hole</b></col>, <cd>a loophole; evasion.</cd> <mark>[Obs.]</mark> -- <col><b>Starting point</b></col>, <cd>the point from which motion begins, or from which anything starts.</cd> -- <col><b>Starting post</b></col>, <cd>a post, stake, barrier, or place from which competitors in a race start, or begin the race.</cd></cs><-- starting pistol, a pistol used to fire blank cartridges to signal the start of a race; a starter's pistol. --><br/
[<source>1913 Webster</source>]</p>

<p><ent>Startingly</ent><br/
<hw>Start"ing*ly</hw>, <pos>adv.</pos> <def>By sudden fits or starts; spasmodically.</def>  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Startish</ent><br/
<hw>Start"ish</hw>, <pos>a.</pos> <def>Apt to start; skittish; shy; -- said especially of a horse.</def> <mark>[Colloq.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Startle</ent><br/
<hw>Star"tle</hw> <pr>(st<aum/r"t'l)</pr>, <pos>v. i.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Startled</conjf> <pr>(st<aum/r"t'ld)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Startling</conjf> <pr>(st<aum/r"tl<icr/ng)</pr>.]</vmorph> <ety>[Freq. of <ets>start</ets>.]</ety> <def>To move suddenly, or be excited, on feeling alarm; to start.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Why shrinks the soul<br/
Back on herself, and <qex>startles</qex> at destruction?</q> <rj><qau>Addison.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Startle</ent><br/
<hw>Star"tle</hw> <pr>(st<aum/r"t'l)</pr>, <pos>v. t.</pos> <sn>1.</sn> <def>To excite by sudden alarm, surprise, or apprehension; to frighten suddenly and not seriously; to alarm; to surprise.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The supposition, at least, that angels do sometimes assume bodies need not <qex>startle</qex> us.</q> <rj><qau>Locke.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To deter; to cause to deviate.</def> <mark>[R.]</mark>  <rj><au>Clarendon.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><syn><b>Syn.</b> -- To start; shock; fright; frighten; alarm.</syn><br/
[<source>1913 Webster</source>]</p>

<p><ent>Startle</ent><br/
<hw>Star"tle</hw>, <pos>n.</pos> <def>A sudden motion or shock caused by an unexpected alarm, surprise, or apprehension of danger.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>After having recovered from my first <qex>startle</qex>, I was very well pleased with the accident.</q> <rj><qau>Spectator.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Startlingly</ent><br/
<hw>Star"tling*ly</hw> <pr>(-tl<icr/ng*l<ycr/)</pr>, <pos>adv.</pos> <def>In a startling manner.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Startlish</ent><br/
<hw>Star"tlish</hw> <pr>(-tl<icr/sh)</pr>, <pos>a.</pos> <def>Easily startled; apt to start; startish; skittish; -- said especially of a horse.</def> <mark>[Colloq.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Start-up</ent><br/
<hw>Start"-up`</hw> <pr>(st<aum/rt"<ucr/p`)</pr>, <pos>n.</pos> <sn>1.</sn> <def>One who comes suddenly into notice; an upstart.</def> <mark>[Obs.]</mark>  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A kind of high rustic shoe.</def> <mark>[Obs.]</mark>  <rj><au>Drayton.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>A <qex>startuppe</qex>, or clownish shoe.</q> <rj><qau>Spenser.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Start-up</ent><br/
<hw>Start"-up`</hw>, <pos>a.</pos> <def>Upstart.</def> <mark>[R.]</mark>  <rj><au>Walpole.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Starvation</ent><br/
<hw>Star*va"tion</hw> <pr>(st<aum/r*v<amac/"sh<ucr/n)</pr>, <pos>n.</pos> <def>The act of starving, or the state of being starved.</def><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ This word was first used, according to Horace Walpole, by Henry Dundas, the first Lord Melville, in a speech on American affairs in 1775, which obtained for him the nickname of <xex>Starvation</xex> Dundas.<br/
    <ldquo/<xex>Starvation</xex>, we are also told, belongs to the class of 'vile compounds' from being a mongrel; as if English were not full of mongrels, and as if it would not be in distressing straits without them.<rdquo/ <au>Fitzed. Hall.</au></note><br/
[<source>1913 Webster</source>]</p>

<p><ent>Starve</ent><br/
<hw>Starve</hw> <pr>(st<aum/rv)</pr>, <pos>v. i.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Starved</conjf> <pr>(st<aum/rvd)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Starving</conjf>.]</vmorph> <ety>[OE. <ets>sterven</ets> to die, AS. <ets>steorfan</ets>; akin to D. <ets>sterven</ets>, G. <ets>sterben</ets>, OHG. <ets>sterban</ets>, Icel. <ets>starf</ets> labor, toil.]</ety> <sn>1.</sn> <def>To die; to perish.</def> <mark>[Obs., except in the sense of perishing with cold or hunger.]</mark>  <rj><au>Lydgate.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>In hot coals he hath himself raked . . . <br/
Thus <qex>starved</qex> this worthy mighty Hercules.</q> <rj><qau>Chaucer.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To perish with hunger; to suffer extreme hunger or want; to be very indigent.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Sometimes virtue <qex>starves</qex>, while vice is fed.</q> <rj><qau>Pope.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>To perish or die with cold.</def>  <rj><au>Spenser.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Have I seen the naked <qex>starve</qex> for cold?</q> <rj><qau>Sandys.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q><qex>Starving</qex> with cold as well as hunger.</q> <rj><qau>W. Irving.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ In this sense, still common in England, but rarely used in the United States.</note><br/
[<source>1913 Webster</source>]</p>

<p><ent>Starve</ent><br/
<hw>Starve</hw>, <pos>v. t.</pos> <sn>1.</sn> <def>To destroy with cold.</def> <mark>[Eng.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>From beds of raging fire, to <qex>starve</qex> in ice<br/
Their soft ethereal warmth.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To kill with hunger; <as>as, maliciously to <ex>starve</ex> a man is, in law, murder</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>To distress or subdue by famine; <as>as, to <ex>starve</ex> a garrison into a surrender</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Attalus endeavored to <qex>starve</qex> Italy by stopping their convoy of provisions from Africa.</q> <rj><qau>Arbuthnot.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>To destroy by want of any kind; <as>as, to <ex>starve</ex> plants by depriving them of proper light and air</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <def>To deprive of force or vigor; to disable.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The pens of historians, writing thereof, seemed <qex>starved</qex> for matter in an age so fruitful of memorable actions.</q> <rj><qau>Fuller.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>The powers of their minds are <qex>starved</qex> by disuse.</q> <rj><qau>Locke.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Starvedly</ent><br/
<hw>Starv"ed*ly</hw> <pr>(st<aum/rv"<ecr/d*l<ycr/)</pr>, <pos>adv.</pos> <def>In the condition of one starved or starving; parsimoniously.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Some boasting housekeeper which keepth open doors for one day, . . . and lives <qex>starvedly</qex> all the year after.</q> <rj><qau>Bp. Hall.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Starveling</ent><br/
<hw>Starve"ling</hw> <pr>(st<aum/rv"l<icr/ng)</pr>, <pos>n.</pos> <ety>[<ets>Starve</ets> + <ets>-ling</ets>.]</ety> <def>One who, or that which, pines from lack of food, or nutriment.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Old Sir John hangs with me, and thou knowest he is no <qex>starveling</qex>.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Starveling</ent><br/
<hw>Starve"ling</hw>, <pos>a.</pos> <def>Hungry; lean; pining with want.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Starwort</ent><br/
<hw>Star"wort`</hw> <pr>(st<aum/r"w<ucir/rt`)</pr>, <pos>n.</pos> <fld>(Bot.)</fld> <sd>(a)</sd> <def>Any plant of the genus <gen>Aster</gen>. See <er>Aster</er>.</def> <sd>(b)</sd> <def>A small plant of the genus <gen>Stellaria</gen>, having star-shaped flowers; star flower; chickweed.</def>  <rj><au>Gray.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Water starwort</b></col>, <cd>an aquatic plant (<spn>Callitriche verna</spn>) having some resemblance to chickweed.</cd> -- <col><b>Yellow starwort</b></col>, <cd>a plant of the genus <gen>Inula</gen>; elecampane.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stasimon</ent><br/
||<hw>Stas"i*mon</hw> <pr>(st<acr/s"<icr/*m<ocr/n)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Stasima</plw> <pr>(st<acr/s"<icr/*m<adot/)</pr>.</plu> <ety>[NL., from Gr. <grk>sta`simon</grk>, neut. of <grk>sta`simos</grk> stationary, steadfast.]</ety> <def>In the Greek tragedy, a song of the chorus, continued without the interruption of dialogue or anapaestics.</def>  <rj><au>Liddell & Scott.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stasis</ent><br/
||<hw>Sta"sis</hw> <pr>(st<amac/"s<icr/s <or/ st<acr/s"<icr/s)</pr>, <pos>n.</pos> <ety>[NL., fr. Gr. <grk>sta`sis</grk> a standing still.]</ety> <fld>(Physiol.)</fld> <def>A slackening or arrest of the blood current in the vessels, due not to a lessening of the heart's beat, but presumably to some abnormal resistance of the capillary walls.  It is one of the phenomena observed in the capillaries in inflammation.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Statable</ent><br/
<hw>Stat"a*ble</hw> <pr>(st<amac/t"<adot/*b'l)</pr>, <pos>a.</pos> <def>That can be stated; <as>as, a <ex>statable</ex> grievance; the question at issue is <ex>statable</ex>.</as></def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Statal</ent><br/
<hw>Sta"tal</hw> <pr>(st<amac/"t<ait/l)</pr>, <pos>a.</pos> <def>Of, pertaining to, or existing with reference to, a State of the American Union, as distinguished from the general government.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>I have no knowledge of any other kind of political citizenship, higher or lower, <qex>statal</qex> or national.</q> <rj><qau>Edward Bates.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Statant</ent><br/
<hw>Sta"tant</hw> <pr>(-t<ait/nt)</pr>, <pos>a.</pos> <ety>[L. <ets>stare</ets> to stand.]</ety> <fld>(Her.)</fld> <def>In a standing position; <as>as, a lion <ex>statant</ex></as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Statarian</ent><br/
<hw>Sta*ta"ri*an</hw> <pr>(st<adot/*t<amac/"r<icr/*<ait/n)</pr>, <pos>a.</pos> <def>Fixed; settled; steady; statary.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Statarianly</ent><br/
<hw>Sta*ta"ri*an*ly</hw>, <pos>adv.</pos> <def>Fixedly; steadily.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Statary</ent><br/
<hw>Sta"ta*ry</hw> <pr>(st<amac/"t<adot/*r<ycr/)</pr>, <pos>a.</pos> <ety>[L. <ets>statarius</ets> standing fast, fr. <ets>stare</ets> to stand.]</ety> <def>Fixed; settled.</def> <mark>[Obs.]</mark> <ldquo/The set and <xex>statary</xex> times of paring of nails and cutting of hair.<rdquo/  <rj><au>Sir T. Browne.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>State</ent><br/
<hw>State</hw> <pr>(st<amac/t)</pr>, <pos>n.</pos> <ety>[OE. <ets>stat</ets>, OF. <ets>estat</ets>, F. <ets><eacute/tat</ets>, fr. L. <ets>status</ets> a standing, position, fr. <ets>stare</ets>, <ets>statum</ets>, to stand. See <er>Stand</er>, and cf. <er>Estate</er>, <er>Status</er>.]</ety> <sn>1.</sn> <def>The circumstances or condition of a being or thing at any given time.</def><br/
[<source>1913 Webster</source>]</p>

<p><q><qex>State</qex> is a term nearly synonymous with <ldquo/mode,<rdquo/ but of a meaning more extensive, and is not exclusively limited to the mutable and contingent.</q> <rj><qau>Sir W. Hamilton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Declare the past and present <qex>state</qex> of things.</q> <rj><qau>Dryden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Keep the <qex>state</qex> of the question in your eye.</q> <rj><qau>Boyle.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Rank; condition; quality; <as>as, the <ex>state</ex> of honor</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Thy honor, <qex>state</qex>, and seat is due to me.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>Condition of prosperity or grandeur; wealthy or prosperous circumstances; social importance.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>She instructed him how he should keep <qex>state</qex>, and yet with a modest sense of his misfortunes.</q> <rj><qau>Bacon.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Can this imperious lord forget to reign,<br/
Quit all his <qex>state</qex>, descend, and serve again?</q> <rj><qau>Pope.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>Appearance of grandeur or dignity; pomp.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Where least of <qex>state</qex> there most of love is shown.</q> <rj><qau>Dryden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <def>A chair with a canopy above it, often standing on a dais; a seat of dignity; also, the canopy itself.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>His high throne, . . . under <qex>state</qex><br/
Of richest texture spread.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>When he went to court, he used to kick away the <qex>state</qex>, and sit down by his prince cheek by jowl.</q> <rj><qau>Swift.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>6.</sn> <def>Estate; possession.</def> <mark>[Obs.]</mark>  <rj><au>Daniel.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Your <qex>state</qex>, my lord, again is yours.</q> <rj><qau>Massinger.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>7.</sn> <def>A person of high rank.</def> <mark>[Obs.]</mark>  <rj><au>Latimer.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>8.</sn> <def>Any body of men united by profession, or constituting a community of a particular character; <as>as, the civil and ecclesiastical <ex>states</ex>, or the lords spiritual and temporal and the commons, in Great Britain</as>.  Cf. <er>Estate</er>, <pos>n.</pos>, 6.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>9.</sn> <def>The principal persons in a government.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The bold design<br/
Pleased highly those infernal <qex>states</qex>.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>10.</sn> <def>The bodies that constitute the legislature of a country; <as>as, the <ex>States</ex>-general of Holland</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>11.</sn> <def>A form of government which is not monarchial, as a republic.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>Well monarchies may own religion's name,<br/
But <qex>states</qex> are atheists in their very fame.</q> <rj><qau>Dryden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>12.</sn> <def>A political body, or body politic; the whole body of people who are united under one government, whatever may be the form of the government; a nation.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Municipal law is a rule of conduct prescribed by the supreme power in a <qex>state</qex>.</q> <rj><qau>Blackstone.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>The Puritans in the reign of Mary, driven from their homes, sought an asylum in Geneva, where they found a <qex>state</qex> without a king, and a church without a bishop.</q> <rj><qau>R. Choate.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>13.</sn> <def>In the United States, one of the commonwealths, or bodies politic, the people of which make up the body of the nation, and which, under the national constitution, stand in certain specified relations with the national government, and are invested, as commonwealths, with full power in their several spheres over all matters not expressly inhibited.</def><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ The term <xex>State</xex>, in its technical sense, is used in distinction from the federal system, <it>i. e.</it>, the government of the United States.</note><br/
[<source>1913 Webster</source>]</p>

<p><sn>14.</sn> <def>Highest and stationary condition, as that of maturity between growth and decline, or as that of crisis between the increase and the abating of a disease; height; acme.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ When <xex>state</xex> is joined with another word, or used adjectively, it denotes public, or what belongs to the community or body politic, or to the government; also, what belongs to the States severally in the American Union; as, <xex>state</xex> affairs; <xex>state</xex> policy; <xex>State</xex> laws of Iowa.</note><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Nascent state</b></col>. <fld>(Chem.)</fld> <cd>See under <er>Nascent</er>.</cd> -- <col><b>Secretary of state</b></col>. <cd>See <er>Secretary</er>, <pos>n.</pos>, 3.</cd> -- <col><b>State barge</b></col><cd>a royal barge, or a barge belonging to a government.</cd> -- <col><b>State bed</b></col>, <cd>an elaborately carved or decorated bed.</cd> -- <col><b>State carriage</b></col>, <cd>a highly decorated carriage for officials going in state, or taking part in public processions.</cd> -- <col><b>State paper</b></col>, <cd>an official paper relating to the interests or government of a state.</cd> <au>Jay.</au> -- <col><b>State prison</b></col>, <cd>a public prison or penitentiary; -- called also <altname>State's prison</altname>.</cd> -- <col><b>State prisoner</b></col>, <cd>one in confinement, or under arrest, for a political offense.</cd> -- <mcol><col><b>State rights</b></col>, <it>or</it> <col><b>States' rights</b></col></mcol>, <cd>the rights of the several independent States, as distinguished from the rights of the Federal government.  It has been a question as to what rights have been vested in the general government.</cd> <mark>[U.S.]</mark> -- <col><b>State's evidence</b></col>. <cd>See <er>Probator</er>, 2, and under <er>Evidence</er>.</cd> -- <col><b>State sword</b></col>, <cd>a sword used on state occasions, being borne before a sovereign by an attendant of high rank.</cd> -- <col><b>State trial</b></col>, <cd>a trial of a person for a political offense.</cd> -- <col><b>States of the Church</b></col>. <cd>See under <er>Ecclesiastical</er>.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><syn><b>Syn.</b> -- <er>State</er>, <er>Situation</er>, <er>Condition</er>.</syn> <usage><xex>State</xex> is the generic term, and denotes in general the mode in which a thing stands or exists. The <xex>situation</xex> of a thing is its state in reference to external objects and influences; its <xex>condition</xex> is its internal state, or what it is in itself considered.  Our <xex>situation</xex> is good or bad as outward things bear favorably or unfavorably upon us; our <xex>condition</xex> is good or bad according to the state we are actually in as respects our persons, families, property, and other things which comprise our sources of enjoyment.</usage><br/
[<source>1913 Webster</source>]</p>

<p><q>I do not, brother,<br/
Infer as if I thought my sister's <qex>state</qex><br/
Secure without all doubt or controversy.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>We hoped to enjoy with ease what, in our <qex>situation</qex>, might be called the luxuries of life.</q> <rj><qau>Cook.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>And, O, what man's <qex>condition</qex> can be worse<br/
Than his whom plenty starves and blessings curse?</q> <rj><qau>Cowley.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>State</ent><br/
<hw>State</hw> <pr>(st<amac/t)</pr>, <pos>a.</pos> <sn>1.</sn> <def>Stately.</def> <mark>[Obs.]</mark>  <rj><au>Spenser.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Belonging to the state, or body politic; public.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>State</ent><br/
<hw>State</hw>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Stated</conjf>; <pos>p. pr. & vb. n.</pos> <conjf>Stating</conjf>.]</vmorph> <sn>1.</sn> <def>To set; to settle; to establish.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>I myself, though meanest <qex>stated</qex>,<br/
And in court now almost hated.</q> <rj><qau>Wither.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Who calls the council, <qex>states</qex> the certain day.</q> <rj><qau>Pope.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To express the particulars of; to set down in detail or in gross; to represent fully in words; to narrate; to recite; <as>as, to <ex>state</ex> the facts of a case, one's opinion, etc.</as></def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>To state it</b></col>. <cd>To assume state or dignity.</cd> <mark>[Obs.]</mark> <ldquo/Rarely dressed up, and taught <xex>to state it</xex>.<rdquo/ <au>Beau. & Fl.</au></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>State</ent><br/
<hw>State</hw>, <pos>n.</pos> <def>A statement; also, a document containing a statement.</def> <mark>[R.]</mark>  <rj><au>Sir W. Scott.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Statecraft</ent><br/
<hw>State"craft`</hw> <pr>(st<amac/t"kr<adot/ft`)</pr>, <pos>n.</pos> <def>The art of conducting state affairs; state management; statesmanship.</def><br/
[<source>1913 Webster</source>]</p>

<p><-- p. 1406 pr=VMG --></p>

<p><ent>Stated</ent><br/
<hw>Stat"ed</hw> <pr>(st<amac/t"<ecr/d)</pr>, <pos>a.</pos> <sn>1.</sn> <def>Settled; established; fixed.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>He is capable of corruption who receives more than what is the <qex>stated</qex> and unquestionable fee of his office.</q> <rj><qau>Addison.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Recurring at regular times; not occasional; <as>as, <ex>stated</ex> preaching; <ex>stated</ex> business hours</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Statedly</ent><br/
<hw>Stat"ed*ly</hw>, <pos>adv.</pos> <def>At stated times; regularly.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stateful</ent><br/
<hw>State"ful</hw> <pr>(st<amac/t"f<usdot/l)</pr>, <pos>a.</pos> <def>Full of state; stately.</def> <mark>[Obs.]</mark> <ldquo/A <xex>stateful</xex> silence.<rdquo/  <rj><au>Marston.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Statehood</ent><br/
<hw>State"hood</hw> <pr>(st<amac/t"h<oocr/d)</pr>, <pos>n.</pos> <def>The condition of being a State; <as>as, a territory seeking <ex>Statehood</ex></as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Statehouse</ent><br/
<hw>State"house`</hw> <pr>(st<amac/t"hous`)</pr>, <pos>n.</pos> <def>The building in which a State legislature holds its sessions; a State capitol.</def> <mark>[U. S.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stateless</ent><br/
<hw>State"less</hw>, <pos>a.</pos> <def>Without state or pomp.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Statelily</ent><br/
<hw>State"li*ly</hw> <pr>(st<amac/t"l<icr/*l<ycr/)</pr>, <pos>adv.</pos> <def>In a stately manner.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stateliness</ent><br/
<hw>State"li*ness</hw>, <pos>n.</pos> <def>The quality or state of being stately.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>For <qex>stateliness</qex> and majesty, what is comparable to a horse?</q> <rj><qau>Dr. H. More.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stately</ent><br/
<hw>State"ly</hw>, <pos>a.</pos> <amorph>[<pos>Compar.</pos> <adjf>Statelier</adjf> <pr>(st<amac/t"l<icr/*<etil/r)</pr>; <pos>superl.</pos> <adjf>Stateliest</adjf>.]</amorph> <def>Evincing state or dignity; lofty; majestic; grand; <as>as, <ex>stately</ex> manners; a <ex>stately</ex> gait</as>.</def> <ldquo/The <xex>stately</xex> homes of England!<rdquo/ <au>Mrs. Hemans.</au> <ldquo/Filled with <xex>stately</xex> temples.<rdquo/ <au>Prescott.</au><br/
[<source>1913 Webster</source>]</p>

<p><q>Here is a silly <qex>stately</qex> style indeed!</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><syn><b>Syn.</b> -- Lofty; dignified; majestic; grand; august; magnificent.</syn><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stately</ent><br/
<hw>State"ly</hw>, <pos>adv.</pos> <def>Majestically; loftily.</def>  <rj><au>Milton.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Statement</ent><br/
<hw>State"ment</hw> <pr>(st<amac/t"m<eit/nt)</pr>, <pos>n.</pos> <sn>1.</sn> <def>The act of stating, reciting, or presenting, orally or on paper; <as>as, to interrupt a speaker in the <ex>statement</ex> of his case</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>That which is stated; a formal embodiment in language of facts or opinions; a narrative; a recital.</def> <ldquo/Admirable perspicuity of <xex>statement</xex>.<rdquo/  <rj><au>Brougham.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Statemonger</ent><br/
<hw>State"mon`ger</hw> <pr>(st<amac/t"m<ucr/<nsm/`g<etil/r)</pr>, <pos>n.</pos> <def>One versed in politics, or one who dabbles in state affairs.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>State prison</ent><br/
<hw>State` pris"on</hw> <pr>(st<amac/t` pr<icr/z"'n)</pr>. <def>See under <er>State</er>, <pos>n.</pos></def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stater</ent><br/
<hw>Stat"er</hw> <pr>(st<amac/t"<etil/r)</pr>, <pos>n.</pos> <def>One who states.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stater</ent><br/
||<hw>Sta"ter</hw> <pr>(st<amac/"t<etil/r)</pr>, <pos>n.</pos> <ety>[L. <ets>stater</ets>, Gr. <grk>stath`r</grk>.]</ety> <fld>(Gr. Antiq.)</fld> <def>The principal gold coin of ancient Greece.  It varied much in value, the stater best known at Athens being worth about <pound/1 2s., or about $5.35 (in 1890 value).  The Attic silver <altname>tetradrachm</altname> was in later times called <ex>stater</ex>.</def><br/
[<source>1913 Webster</source> <source>+PJC</source>]</p>

<p><ent>Stateroom</ent><br/
<hw>State"room`</hw> <pr>(st<amac/t"r<oomac/m`)</pr>, <pos>n.</pos> <sn>1.</sn> <def>A magnificent room in a palace or great house.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A small apartment for lodging or sleeping in the cabin, or on the deck, of a vessel; also, a somewhat similar apartment in a railway sleeping car.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>States-general</ent><br/
<hw>States"-gen"er*al</hw> <pr>(st<amac/ts"j<ecr/n"<etil/r*<ait/l)</pr>, <pos>n.</pos> <sn>1.</sn> <def>In France, before the Revolution, the assembly of the three orders of the kingdom, namely, the clergy, the nobility, and the third estate, or commonalty.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>In the Netherlands, the legislative body, composed of two chambers.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Statesman</ent><br/
<hw>States"man</hw> <pr>(-m<ait/n)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Statesmen</plw> <pr>(-m<eit/n)</pr>.</plu> <sn>1.</sn> <def>A man versed in public affairs and in the principles and art of government; especially, one eminent for political abilities.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The minds of some of our <qex>statesmen</qex>, like the pupil of the human eye, contract themselves the more, the stronger light there is shed upon them.</q> <rj><qau>Moore.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>One occupied with the affairs of government, and influential in shaping its policy.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>A small landholder.</def> <mark>[Prov. Eng.]</mark>  <rj><au>Halliwell.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Statesmanlike</ent><br/
<hw>States"man*like`</hw> <pr>(-l<imac/k`)</pr>, <pos>a.</pos> <def>Having the manner or wisdom of statesmen; becoming a statesman.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Statesmanly</ent><br/
<hw>States"man*ly</hw>, <pos>a.</pos> <def>Becoming a statesman.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Statesmanship</ent><br/
<hw>States"man*ship</hw>, <pos>n.</pos> <def>The qualifications, duties, or employments of a statesman.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>State socialism</ent><br/
<hw>State socialism</hw>. <def>A form of socialism, esp. advocated in Germany, which, while retaining the right of private property and the institution of the family and other features of the present form of the state, would intervene by various measures intended to give or maintain equality of opportunity, as compulsory state insurance, old-age pensions, etc., answering closely to socialism of the chair.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Stateswoman</ent><br/
<hw>States"wom`an</hw> <pr>(-w<oocr/m`<ait/n)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Stateswomen</plw> <pr>(-w<icr/m`<ecr/n)</pr>.</plu> <def>A woman concerned in public affairs.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>A rare <qex>stateswoman</qex>; I admire her bearing.</q> <rj><qau>B. Jonson.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stathmograph</ent><br/
<hw>Stath"mo*graph</hw> <pr>(st<acr/th"m<osl/*gr<adot/f)</pr>, <pos>n.</pos> <ety>[Gr. <grk>sta`qmh</grk> a measuring line + <ets>-graph</ets>.]</ety> <def>A contrivance for recording the speed of a railway train.</def>  <rj><au>Knight.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Statical</ent><br/
<ent>Static</ent><br/
<mhw>{ <hw>Stat"ic</hw> <pr>(st<acr/t"<icr/k)</pr>, <hw>Stat"ic*al</hw> <pr>(-<icr/*k<ait/l)</pr>, }</mhw> <pos>a.</pos> <ety>[Gr. <grk>statiko`s</grk> causing to stand, skilled in weighing, fr. <grk>'ista`nai</grk> to cause to stand: cf. F. <ets>statique</ets>. See <er>Stand</er>, and cf. <er>Stage</er>.]</ety> <sn>1.</sn> <def>Resting; acting by mere weight without motion; <as>as, <ex>statical</ex> pressure; <ex>static</ex> objects.</as></def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Pertaining to bodies at rest or in equilibrium.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><mcol><col><b>Static electricity</b></col>, <col><b>Statical electricity</b></col></mcol>. <cd>See the Note under <er>Electricity</er>, 1.</cd> -- <col><b>Statical moment</b></col>. <cd>See under <er>Moment</er>.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Statically</ent><br/
<hw>Stat"ic*al*ly</hw>, <pos>adv.</pos> <def>In a statical manner.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Statics</ent><br/
<hw>Stat"ics</hw> <pr>(-<icr/ks)</pr>, <pos>n.</pos> <ety>[Cf. F. <ets>statique</ets>, Gr. <grk>statikh`</grk> the art of weighing, fr. <grk>statiko`s</grk>. See <er>Static</er>.]</ety> <def>That branch of mechanics which treats of the equilibrium of forces, or relates to bodies as held at rest by the forces acting on them; -- distinguished from <contr>dynamics</contr>.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Social statics</b></col>, <cd>the study of the conditions which concern the existence and permanence of the social state.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stating</ent><br/
<hw>Stat"ing</hw> <pr>(st<amac/t"<icr/ng)</pr>, <pos>n.</pos> <def>The act of one who states anything; statement; <as>as, the <ex>stating</ex>of one's opinions</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Station</ent><br/
<hw>Sta"tion</hw> <pr>(st<amac/"sh<ucr/n)</pr>, <pos>n.</pos> <ety>[F., fr. L. <ets>statio</ets>, from <ets>stare</ets>, <ets>statum</ets>, to stand. See <er>Stand</er>.]</ety> <sn>1.</sn> <def>The act of standing; also, attitude or pose in standing; posture.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>A <qex>station</qex> like the herald, Mercury.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Their manner was to stand at prayer, whereupon their meetings unto that purpose . . . had the names of <qex>stations</qex> given them.</q> <rj><qau>Hooker.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A state of standing or rest; equilibrium.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>All progression is performed by drawing on or impelling forward some part which was before in <qex>station</qex>, or at quiet.</q> <rj><qau>Sir T. Browne.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>The spot or place where anything stands, especially where a person or thing habitually stands, or is appointed to remain for a time; <as>as, the <ex>station</ex> of a sentinel</as>.</def> Specifically: <sd>(a)</sd> <def>A regular stopping place in a stage road or route; a place where railroad trains regularly come to a stand, for the convenience of passengers, taking in fuel, moving freight, etc.</def> <sd>(b)</sd> <def>The headquarters of the police force of any precinct.</def> <sd>(c)</sd> <def>The place at which an instrument is planted, or observations are made, as in surveying.</def> <sd>(d)</sd> <fld>(Biol.)</fld> <def>The particular place, or kind of situation, in which a species naturally occurs; a habitat.</def> <sd>(e)</sd> <fld>(Naut.)</fld> <def>A place to which ships may resort, and where they may anchor safely.</def> <sd>(f)</sd> <def>A place or region to which a government ship or fleet is assigned for duty.</def> <sd>(g)</sd> <fld>(Mil.)</fld> <def>A place calculated for the rendezvous of troops, or for the distribution of them; also, a spot well adapted for offensive or defensive measures.</def> <au>Wilhelm (Mil. Dict.)</au>. <sd>(h)</sd> <fld>(Mining)</fld> <def>An enlargement in a shaft or galley, used as a landing, or passing place, or for the accommodation of a pump, tank, etc.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>Post assigned; office; the part or department of public duty which a person is appointed to perform; sphere of duty or occupation; employment.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>By spending this day [Sunday] in religious exercises, we acquire new strength and resolution to perform God's will in our several <qex>stations</qex> the week following.</q> <rj><qau>R. Nelson.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <def>Situation; position; location.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The fig and date -- why love they to remain<br/
In middle <qex>station</qex>, and an even plain?</q> <rj><qau>Prior.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>6.</sn> <def>State; rank; condition of life; social status.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The greater part have kept, I see,<br/
Their <qex>station</qex>.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>They in France of the best rank and <qex>station</qex>.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>7.</sn> <fld>(Eccl.)</fld> <sd>(a)</sd> <def>The fast of the fourth and sixth days of the week, Wednesday and Friday, in memory of the council which condemned Christ, and of his passion.</def> <sd>(b)</sd> <fld>(R. C. Ch.)</fld> <def>A church in which the procession of the clergy halts on stated days to say stated prayers.</def> <au>Addis & Arnold.</au> <sd>(c)</sd> <def>One of the places at which ecclesiastical processions pause for the performance of an act of devotion; formerly, the tomb of a martyr, or some similarly consecrated spot; now, especially, one of those representations of the successive stages of our Lord's passion which are often placed round the naves of large churches and by the side of the way leading to sacred edifices or shrines, and which are visited in rotation, stated services being performed at each; -- called also <altname>Station of the cross</altname>.</def> <au>Fairholt.</au><br/
[<source>1913 Webster</source>]</p>

<p><sn>8.</sn> <def>In Australia, a sheep run or cattle run, together with the buildings belonging to it; also, the  homestead and buildings belonging to such a run.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><cs><col><b>Station bill</b></col>. <fld>(Naut.)</fld> <cd>Same as <cref>Quarter bill</cref>, under <er>Quarter</er>.</cd> -- <col><b>Station house</b></col>. <sd>(a)</sd> <cd>The house serving for the headquarters of the police assigned to a certain district, and as a place of temporary confinement</cd>. <sd>(b)</sd> <cd>The house used as a shelter at a railway station.</cd> -- <col><b>Station master</b></col>, <cd>one who has charge of a station, esp. of a railway station.</cd> -- <col><b>Station pointer</b></col> <fld>(Surv.)</fld>, <cd>an instrument for locating on a chart the position of a place from which the angles subtended by three distant objects, whose positions are known, have been observed.</cd> -- <col><b>Station staff</b></col> <fld>(Surv.)</fld>, <cd>an instrument for taking angles in surveying.</cd> <au>Craig.</au></cs><br/
[<source>1913 Webster</source>]</p>

<p><syn><b>Syn.</b> -- <er>Station</er>, <er>Depot</er>.</syn> <usage> In the United States, a stopping place on a railway for passengers and freight is commonly called a <xex>depot</xex>: but to a considerable extent in official use, and in common speech, the more appropriate name, <xex>station</xex>, has been adopted.</usage><br/
[<source>1913 Webster</source>]</p>

<p><ent>Station</ent><br/
<hw>Sta"tion</hw> <pr>(st<amac/"sh<ucr/n)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Stationed</conjf> <pr>(-sh<ucr/nd)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Stationing</conjf>.]</vmorph> <def>To place; to set; to appoint or assign to the occupation of a post, place, or office; <as>as, to <ex>station</ex> troops on the right of an army; to <ex>station</ex> a sentinel on a rampart; to <ex>station</ex> ships on the coast of Africa</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>He gained the brow of the hill, where the English phalanx was <qex>stationed</qex>.</q> <rj><qau>Lyttelton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stational</ent><br/
<hw>Sta"tion*al</hw> <pr>(-<ait/l)</pr>, <pos>a.</pos> <ety>[L. <ets>stationalis</ets>: cf. F. <ets>stationnale</ets> (<ets><eacute/glise</ets>).]</ety> <def>Of or pertaining to a station.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stationariness</ent><br/
<hw>Sta"tion*a*ri*ness</hw> <pr>(-<asl/*r<icr/*n<ecr/s)</pr>, <pos>n.</pos> <def>The quality or state of being stationary; fixity.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stationary</ent><br/
<hw>Sta"tion*a*ry</hw> <pr>(-<asl/*r<ycr/)</pr>, <pos>a.</pos> <ety>[L. <ets>stationarius</ets>: cf. F. <ets>stationnaire</ets>.  Cf. <er>Stationer</er>.]</ety> <sn>1.</sn> <def>Not moving; not appearing to move; stable; fixed.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Charles Wesley, who is a more <qex>stationary</qex> man, does not believe the story.</q> <rj><qau>Southey.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Not improving or getting worse; not growing wiser, greater, better, more excellent, or the contrary.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>Appearing to be at rest, because moving in the line of vision; not progressive or retrograde, as a planet.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Stationary air</b></col> <fld>(Physiol.)</fld>, <cd>the air which under ordinary circumstances does not leave the lungs in respiration.</cd> -- <col><b>Stationary engine</b></col>. <sd>(a)</sd> <cd>A steam engine that is permanently placed, in distinction from a portable engine, locomotive, marine engine, etc.</cd> <specif>Specifically:</specif> <sd>(b)</sd> <cd>A factory engine, in distinction from a blowing, pumping, or other kind of engine which is also permanently placed.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stationary</ent><br/
<hw>Sta"tion*a*ry</hw>, <pos>n.</pos>; <plu><it>pl.</it> <plw>-ries</plw> <pr>(-r<icr/z)</pr>.</plu> <def>One who, or that which, is stationary, as a planet when apparently it has neither progressive nor retrograde motion.</def>  <rj><au>Holland.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stationer</ent><br/
<hw>Sta"tion*er</hw> <pr>(-<etil/r)</pr>, <pos>n.</pos> <ety>[Cf. <er>Stationary</er>, <pos>a.</pos>]</ety> <sn>1.</sn> <def>A bookseller or publisher; -- formerly so called from his occupying a stand, or <xex>station</xex>, in the market place or elsewhere.</def> <mark>[Obs.]</mark>  <rj><au>Dryden.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>One who sells paper, pens, quills, inkstands, pencils, blank books, and other articles used in writing.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stationery</ent><br/
<hw>Sta"tion*er*y</hw> <pr>(-<ecr/r*<ycr/)</pr>, <pos>n.</pos> <def>The articles usually sold by stationers, as paper, pens, ink, quills, blank books, etc.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stationery</ent><br/
<hw>Sta"tion*er*y</hw>, <pos>a.</pos> <def>Belonging to, or sold by, a stationer.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Statism</ent><br/
<hw>Sta"tism</hw> <pr>(st<amac/"t<icr/z'm)</pr>, <pos>n.</pos> <ety>[From <er>State</er>.]</ety> <def>The art of governing a state; statecraft; policy.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>The enemies of God . . . call our religion <qex>statism</qex>.</q> <rj><qau>South.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Statist</ent><br/
<hw>Sta"tist</hw> <pr>(-t<icr/st)</pr>, <pos>n.</pos> <ety>[From <er>State</er>.]</ety> <sn>1.</sn> <def>A statesman; a politician; one skilled in government.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q><qex>Statists</qex> indeed,<br/
And lovers of their country.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A statistician.</def>  <rj><au>Fawcett.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Statistical</ent><br/
<ent>Statistic</ent><br/
<mhw>{ <hw>Sta*tis"tic</hw> <pr>(st<adot/*t<icr/s"t<icr/k)</pr>, <hw>Sta*tis"tic*al</hw> <pr>(-t<icr/*k<ait/l)</pr>, }</mhw> <pos>a.</pos> <ety>[Cf. F. <ets>statistique</ets>.]</ety> <def>Of or pertaining to statistics; <as>as, <ex>statistical</ex> knowledge; <ex>statistical</ex> tabulation</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Statistically</ent><br/
<hw>Sta*tis"tic*al*ly</hw>, <pos>adv.</pos> <def>In the way of statistics.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Statistician</ent><br/
<hw>Stat`is*ti"cian</hw> <pr>(st<acr/t`<icr/s*t<icr/sh"<ait/n)</pr>, <pos>n.</pos> <ety>[Cf. F. <ets>statisticien</ets>.]</ety> <def>One versed in statistics; one who collects and classifies facts for statistics.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Statistics</ent><br/
<hw>Sta*tis"tics</hw> <pr>(st<adot/*t<icr/s"t<icr/ks)</pr>, <pos>n.</pos> <ety>[Cf. F. <ets>statistique</ets>, G. <ets>statistik</ets>. See <er>State</er>, <pos>n.</pos>]</ety> <sn>1.</sn> <def>The science which has to do with the collection, classification, and analysis of facts of a numerical nature regarding any topic.</def> <specif>Specifically:</specif> <def>The science dealing with collection, tabulation, and analysis of facts respecting the condition of the people in a state.</def> <note>[In this sense grammatically singular.]</note><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <pluf>pl.</pluf> <def>Classified facts of a numerical nature regarding any topic.</def> <specif>Specifically:</specif> <sd>(a)</sd> <def>Numerical facts respecting the condition of the people in a state, their health, their longevity, domestic economy, arts, property, and political strength, their resources, the state of the country, etc., or respecting any particular class or interest; especially, those facts which can be stated in numbers, or in tables of numbers, or in any tabular and classified arrangement.</def> <sd>(b)</sd> <fld>(Sport)</fld> <def>Numerical facts regarding the performance of athletes or athletic teams, such as winning percentages, numbers of games won or lost in a season, batting averages (for baseball players), total yards gained (for football players).  The creation and classification of such numbers is limited only by the imagination of those wishing to describe athletic performance numerically.</def><br/
<syn><b>Syn. --</b> stats.</syn><br/
[<source>1913 Webster</source> <source>+PJC</source>]</p>

<p><sn>3.</sn> <def>The branch of mathematics which studies methods for the calculation of probabilities.</def><br/
[<source>PJC</source>]</p>

<p><ent>Statistology</ent><br/
<hw>Stat`is*tol"o*gy</hw> <pr>(st<acr/t`<icr/s*t<ocr/l"<osl/*j<ycr/)</pr>, <pos>n.</pos> <ety>[<ets>Statistics</ets> + <ets>-logy</ets>.]</ety> <def>See <er>Statistics</er>, 2.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stative</ent><br/
<hw>Sta"tive</hw> <pr>(st<amac/"t<icr/v)</pr>, <pos>a.</pos> <ety>[L. <ets>stativus</ets>, fr. <ets>stare</ets>, <ets>statum</ets>, to stand.]</ety> <fld>(Mil.)</fld> <def>Of or pertaining to a fixed camp, or military posts or quarters.</def> <mark>[Obs. or R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Statoblast</ent><br/
<hw>Stat"o*blast</hw> <pr>(st<acr/t"<osl/*bl<acr/st)</pr>, <pos>n.</pos> <ety>[Gr. <grk>stato`s</grk> standing (<it>i. e.</it>, remaining) + <ets>-blast</ets>.]</ety> <fld>(Zool.)</fld> <def>One of a peculiar kind of internal buds, or germs, produced in the interior of certain Bryozoa and sponges, especially in the fresh-water species; -- also called <altname>winter buds</altname>.</def><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ They are protected by a firm covering, and are usually destined to perpetuate the species during the winter season. They burst open and develop in the spring. In some fresh-water sponges they serve to preserve the species during the dry season. See <xex>Illust.</xex> under <er>Phylactolaemata</er>.</note><br/
[<source>1913 Webster</source>]</p>

<p><ent>Statocracy</ent><br/
<hw>Sta*toc"ra*cy</hw> <pr>(st<adot/*t<ocr/k"r<adot/*s<ycr/)</pr>, <pos>n.</pos> <ety>[<ets>State</ets> + <ets>-cracy</ets>, as in <ets>democracy</ets>.]</ety> <def>Government by the state, or by political power, in distinction from government by ecclesiastical power.</def> <mark>[R.]</mark>  <rj><au>O. A. Brownson.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>stator</ent><br/
<hw>sta"tor</hw> <pr>(st<amac/"t<etil/r; st<acr/t"<etil/r)</pr>, <pos>n.</pos> <fld>(Mach.)</fld> <def>A stationary part in or about which another part (the rotor) revolves, esp. when both are large;</def> <specif>as,</specif> <sd>(a)</sd> <fld>(Elec.)</fld> <def>The stationary member of an electrical machine, as of an induction motor.</def> <sd>(b)</sd> <fld>(Steam Turbine)</fld> <def>The case inclosing a turbine wheel; the body of stationary blades or nozzles.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>stats</ent><br/
<hw>stats</hw> <pr>(st<acr/ts)</pr>, <pos>n.</pos> <def>Shortened form of <er>statistics</er>{2} <sd>(b)</sd>.</def> <mark>[Slang]</mark><br/
[<source>PJC</source>]</p>

<p><ent>Statua</ent><br/
<hw>Stat"u*a</hw> <pr>(st<acr/t"<usl/*<adot/)</pr>, <pos>n.</pos> <ety>[L.]</ety> <def>A statue.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>They spake not a word;<br/
But, like dumb <qex>statuas</qex> or breathing stones,<br/
Gazed each on other.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Statuary</ent><br/
<hw>Stat"u*a*ry</hw> <pr>(-<asl/*r<ycr/)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Statuaries</plw> <pr>(-r<icr/z)</pr>.</plu> <ety>[L. <ets>statuarius</ets>, n., fr. <ets>statuarius</ets>, a., of or belonging to statues, fr. <ets>statua</ets> statue: cf. F. <ets>statuaire</ets>. See <er>Statue</er>.]</ety> <sn>1.</sn> <def>One who practices the art of making statues.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>On other occasions the <qex>statuaries</qex> took their subjects from the poets.</q> <rj><qau>Addison.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <ety>[L. <ets>statuaria</ets> (sc. <ets>ars</ets>): cf. F. <ets>statuaire</ets>.]</ety> <def>The art of carving statues or images as representatives of real persons or things; a branch of sculpture.</def>  <rj><au>Sir W. Temple.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>A collection of statues; statues, collectively.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Statue</ent><br/
<hw>Stat"ue</hw> <pr>(st<acr/ch"<oomac/; also, st<acr/t"<usl/; 135)</pr>, <pos>n.</pos> <ety>[F., fr. L. <ets>statua</ets> (akin to <ets>stativus</ets> standing still), fr. <ets>stare</ets>, <ets>statum</ets>, to stand. See <er>Stand</er>.]</ety> <sn>1.</sn> <def>The likeness of a living being sculptured or modeled in some solid substance, as marble, bronze, or wax; an image; <as>as, a <ex>statue</ex> of Hercules, or of a lion</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>I will raise her <qex>statue</qex> in pure gold.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A portrait.</def> <mark>[Obs.]</mark>  <rj><au>Massinger.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Statue</ent><br/
<hw>Stat"ue</hw>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Statued</conjf> <pr>(-<usl/d)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Statuing</conjf>.]</vmorph> <def>To place, as a statue; to form a statue of; to make into a statue.</def> <ldquo/The whole man becomes as if <xex>statued</xex> into stone and earth.<rdquo/  <rj><au>Feltham.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Statued</ent><br/
<hw>Stat"ued</hw> <pr>(st<acr/t"<usl/d)</pr>, <pos>a.</pos> <def>Adorned with statues.</def> <ldquo/The <xex>statued</xex> hall.<rdquo/ <au>Longfellow.</au> <ldquo/<xex>Statued</xex> niches.<rdquo/ <au>G. Eliot.</au><br/
[<source>1913 Webster</source>]</p>

<p><ent>Statueless</ent><br/
<hw>Stat"ue*less</hw> <pr>(st<acr/t"<usl/*l<ecr/s)</pr>, <pos>a.</pos> <def>Without a statue.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Statuelike</ent><br/
<hw>Stat"ue*like`</hw> <pr>(st<acr/ch"<oomac/*l<imac/k` <it>or</it> st<acr/t"<usl/*l<imac/k`)</pr>, <pos>a.</pos> <def>Like a statue; motionless.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Statuesque</ent><br/
<hw>Stat`u*esque"</hw> <pr>(st<acr/ch`<oomac/*<ecr/sk" <it>or</it> st<acr/t`<usl/*<ecr/sk")</pr>, <pos>a.</pos> <def>Partaking of, or exemplifying, the characteristics of a statue; having the symmetry, or other excellence, of a statue artistically made; <as>as, <ex>statuesque</ex>limbs; a <ex>statuesque</ex> attitude</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Their characters are mostly <qex>statuesque</qex> even in this respect, that they have no background.</q> <rj><qau>Hare.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Statuesquely</ent><br/
<hw>Stat`u*esque"ly</hw>, <pos>adv.</pos> <def>In a statuesque manner; in a way suggestive of a statue; like a statue.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>A character <qex>statuesquely</qex> simple in its details.</q> <rj><qau>Lowell.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Statuette</ent><br/
<hw>Stat`u*ette"</hw> <pr>(st<acr/ch`<oomac/*<ecr/t" <it>or</it> st<acr/t`<usl/*<ecr/t")</pr>, <pos>n.</pos> <ety>[F., cf. It. <ets>statuetta</ets>.]</ety> <def>A small statue; -- usually applied to a figure much less than life size, especially when of marble or bronze, or of plaster or clay as a preparation for the marble or bronze, as distinguished from a figure in terra cotta or the like.  Cf. <er>Figurine</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Statuminate</ent><br/
<hw>Sta*tu"mi*nate</hw> <pr>(st<adot/*t<umac/"m<icr/*n<amac/t)</pr>, <pos>v. t.</pos> <ety>[L. <ets>statuminatus</ets>, p. p. of <ets>statuminare</ets> to prop, fr. <ets>statumen</ets> a prop, fr. <ets>statuere</ets> to place.]</ety> <def>To prop or support.</def> <mark>[Obs.]</mark>  <rj><au>B. Jonson.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stature</ent><br/
<hw>Stat"ure</hw> <pr>(st<acr/ch"<etil/r; <it>also</it>, st<acr/t"<usl/r; 135)</pr>, <pos>n.</pos> <ety>[F. <ets>stature</ets>, OF. <ets>estature</ets>, from L. <ets>statura</ets>, originally, an upright posture; hence, height or size of the body, from <ets>stare</ets>, <ets>statum</ets>, to stand. See <er>Stand</er>.]</ety> <def>The natural height of an animal body; -- generally used of the human body.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Foreign men of mighty <qex>stature</qex> came.</q> <rj><qau>Dryden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Statured</ent><br/
<hw>Stat"ured</hw> <pr>(-<usl/rd)</pr>, <pos>a.</pos> <def>Arrived at full stature.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Status</ent><br/
||<hw>Sta"tus</hw> <pr>(st<amac/"t<ucr/s)</pr>, <pos>n.</pos> <ety>[L.]</ety> <def>State; condition; position of affairs.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Status quo</ent><br/
<ent>Status in quo</ent><br/
<mhw>{ ||<hw>Sta"tus in` quo"</hw> <pr>(st<amac/"t<ucr/s <icr/n` kw<omac/")</pr>, ||<hw>Sta"tus quo"</hw> <pr>(st<amac/"t<ucr/s kw<omac/")</pr>.}</mhw> <ety>[L., state in which.]</ety> <def>The state in which anything is already.  The phrase is also used retrospectively, as when, on a treaty of peace, matters return to the <it>status quo ante bellum</it>, or are left <it>in statu quo ante bellum</it>, i.e., the state (or, in the state) before the war.  The form <ex>status quo</ex> is in most common use.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Statutable</ent><br/
<hw>Stat"u*ta*ble</hw> <pr>(st<acr/t"<usl/*t<adot/*b'l ; 135)</pr>, <pos>a.</pos> <sn>1.</sn> <def>Made or introduced by statute; proceeding from an act of the legistature; <as>as, a <ex>statutable</ex> provision or remedy</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Made or being in conformity to statute; standard; <as>as, <ex>statutable</ex> measures</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Statutably</ent><br/
<hw>Stat"u*ta*bly</hw>, <pos>adv.</pos> <def>Conformably to statute.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Statute</ent><br/
<hw>Stat"ute</hw> <pr>(-<usl/t)</pr>, <pos>n.</pos> <ety>[F. <ets>statut</ets>, LL. <ets>statutum</ets>, from L. <ets>statutus</ets>, p. p. of <ets>statuere</ets> to set, station, ordain, fr. <ets>status</ets> position, station, fr. <ets>stare</ets>, <ets>statum</ets>, to stand. See <er>Stand</er>, and cf. <er>Constitute</er>, <er>Destitute</er>.]</ety> <sn>1.</sn> <def>An act of the legislature of a state or country, declaring, commanding, or prohibiting something; a positive law; the written will of the legislature expressed with all the requisite forms of legislation; -- used in distinction from <cref>common law</cref>. See <cref>Common law</cref>, under <er>Common</er>, <pos>a.</pos></def>  <rj><au>Bouvier.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ <xex>Statute</xex> is commonly applied to the acts of a legislative body consisting of representatives.  In monarchies, the laws of the sovereign are called <xex>edicts</xex>, <xex>decrees</xex>, <xex>ordinances</xex>, <xex>rescripts</xex>, etc.  In works on international law and in the Roman law, the term is used as embracing all laws imposed by competent authority.  Statutes in this sense are divided into statutes real, statutes personal, and statutes mixed; <xex>statutes real</xex> applying to immovables; <xex>statutes personal</xex> to movables; and <xex>statutes mixed</xex> to both classes of property.</note><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>An act of a corporation or of its founder, intended as a permanent rule or law; <as>as, the <ex>statutes</ex> of a university</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>An assemblage of farming servants (held possibly by statute) for the purpose of being hired; -- called also <altname>statute fair</altname>.</def> <mark>[Eng.]</mark> Cf. 3d <er>Mop</er>, 2.  <rj><au>Halliwell.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Statute book</b></col>, <cd>a record of laws or legislative acts.</cd> <au>Blackstone.</au> -- <col><b>Statute cap</b></col>, <cd>a kind of woolen cap; -- so called because enjoined to be worn by a statute, dated in 1571, in behalf of the trade of cappers.</cd> <mark>[Obs.]</mark> <au>Halliwell.</au> -- <col><b>Statute fair</b></col>. <cd>See <er>Statute</er>, <pos>n.</pos>, 3, above.</cd> -- <col><b>Statute labor</b></col>, <cd>a definite amount of labor required for the public service in making roads, bridges, etc., as in certain English colonies.</cd> -- <col><b>Statute merchant</b></col> <fld>(Eng. Law)</fld>, <cd>a bond of record pursuant to the stat. 13 Edw. I., acknowledged in form prescribed, on which, if not paid at the day, an execution might be awarded against the body, lands, and goods of the debtor, and the obligee might hold the lands until out of the rents and profits of them the debt was satisfied; -- called also a <altname>pocket judgment</altname>. It is now fallen into disuse.</cd> <au>Tomlins.</au> <au>Bouvier.</au> -- <col><b>Statute mile</b></col>. <cd>See under <er>Mile</er>.</cd> -- <col><b>Statute of limitations</b></col> <fld>(Law)</fld>, <cd>a statute assigning a certain time, after which rights can not be enforced by action.</cd> -- <col><b>Statute staple</b></col>, <cd>a bond of record acknowledged before the mayor of the staple, by virtue of which the creditor may, on nonpayment, forthwith have execution against the body, lands, and goods of the debtor, as in the <xex>statute merchant</xex>. It is now disused.</cd> <au>Blackstone.</au></cs><br/
[<source>1913 Webster</source>]</p>

<p><syn><b>Syn.</b> -- Act; regulation; edict; decree. See <er>Law</er>.</syn><br/
[<source>1913 Webster</source>]</p>

<p><-- p. 1407 pr=VMG --></p>

<p><ent>Statutory</ent><br/
<hw>Stat"u*to*ry</hw> <pr>(st<acr/t"<usl/*t<osl/*r<ycr/; 135)</pr>, <pos>a.</pos> <def>Enacted by statute; depending on statute for its authority; <as>as, a <ex>statutory</ex> provision</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Staunchness</ent><br/
<ent>Staunchly</ent><br/
<ent>Staunch</ent><br/
<mhw>{ <hw>Staunch</hw> <pr>(st<aum/nch)</pr>, <hw>Staunch"ly</hw>, <hw>Staunch"ness</hw>, etc. }</mhw> <def>See <er>Stanch</er>, <er>Stanchly</er>, etc.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Staurolite</ent><br/
<hw>Stau"ro*lite</hw> <pr>(st<add/"r<osl/*l<imac/t)</pr>, <pos>n.</pos> <ety>[Gr. <grk>stayro`s</grk> a cross + <ets>-lite</ets>.]</ety> <fld>(Min.)</fld> <def>A mineral of a brown to black color occurring in prismatic crystals, often twinned so as to form groups resembling a cross.  It is a silicate of alumina and iron, and is generally found imbedded in mica schist.  Called also <altname>granatite</altname>, and <altname>grenatite</altname>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Staurolitic</ent><br/
<hw>Stau`ro*lit"ic</hw> <pr>(-l<icr/t"<icr/k)</pr>, <pos>a.</pos> <fld>(Min.)</fld> <def>Of or pertaining to staurolite; resembling or containing staurolite.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stauroscope</ent><br/
<hw>Stau"ro*scope</hw> <pr>(st<add/"r<osl/*sk<omac/p)</pr>, <pos>n.</pos> <ety>[Gr. <grk>stayro`s</grk> a cross + <ets>-scope</ets>.]</ety> <fld>(Crystallog.)</fld> <def>An optical instrument used in determining the position of the planes of light-vibration in sections of crystals.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Staurotide</ent><br/
<hw>Stau"ro*tide</hw> <pr>(st<add/"r<osl/*t<imac/d)</pr>, <pos>n.</pos> <ety>[F. <ets>staurotide</ets>, from Gr. <grk>stayrwto`s</grk> cruciform (from Gr. <grk>stayro`s</grk> a cross) + <grk>e'i^dos</grk> form.]</ety> <fld>(Min.)</fld> <def>Staurolite.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stave</ent><br/
<hw>Stave</hw> <pr>(st<amac/v)</pr>, <pos>n.</pos> <ety>[From <er>Staff</er>, and corresponding to the pl. <ets>staves</ets>. See <er>Staff</er>.]</ety> <sn>1.</sn> <def>One of a number of narrow strips of wood, or narrow iron plates, placed edge to edge to form the sides, covering, or lining of a vessel or structure; esp., one of the strips which form the sides of a cask, a pail, etc.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>One of the cylindrical bars of a lantern wheel; one of the bars or rounds of a rack, a ladder, etc.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>A metrical portion; a stanza; a staff.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Let us chant a passing <qex>stave</qex><br/
In honor of that hero brave.</q> <rj><qau>Wordsworth.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <fld>(Mus.)</fld> <def>The five horizontal and parallel lines on and between which musical notes are written or printed; the staff{7}.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Stave jointer</b></col>, <cd>a machine for dressing the edges of staves.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stave</ent><br/
<hw>Stave</hw>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Staved</conjf> <pr>(st<amac/vd)</pr> or <conjf>Stove</conjf> <pr>(st<omac/v)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Staving</conjf>.]</vmorph> <ety>[From <er>Stave</er>, <pos>n.</pos>, or <er>Staff</er>, <pos>n.</pos>]</ety> <sn>1.</sn> <def>To break in a stave or the staves of; to break a hole in; to burst; -- often with <xex>in</xex>; <as>as, to <ex>stave</ex> a cask; to <ex>stave</ex> in a boat.</as></def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To push, as with a staff; -- with <xex>off</xex>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The condition of a servant <qex>staves</qex> him off to a distance.</q> <rj><qau>South.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>To delay by force or craft; to drive away; -- usually with <xex>off</xex>; <as>as, to <ex>stave</ex> off the execution of a project</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>And answered with such craft as women use,<br/
Guilty or guiltless, to <qex>stave</qex> off a chance<br/
That breaks upon them perilously.</q> <rj><qau>Tennyson.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>To suffer, or cause, to be lost by breaking the cask.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>All the wine in the city has been <qex>staved</qex>.</q> <rj><qau>Sandys.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <def>To furnish with staves or rundles.</def>  <rj><au>Knolles.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>6.</sn> <def>To render impervious or solid by driving with a calking iron; <as>as, to <ex>stave</ex> lead, or the joints of pipes into which lead has been run</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>To stave and tail</b></col>, <cd>in bear baiting, (<xex>to stave</xex>) to interpose with the staff, doubtless to stop the bear; (<xex>to tail</xex>) to hold back the dog by the tail.</cd> <au>Nares.</au></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stave</ent><br/
<hw>Stave</hw>, <pos>v. i.</pos> <def>To burst in pieces by striking against something; to dash into fragments.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Like a vessel of glass she <qex>stove</qex> and sank.</q> <rj><qau>Longfellow.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Staves</ent><br/
<hw>Staves</hw> <pr>(st<amac/vz <or/ st<aum/vz ; 277)</pr>, <pos>n.</pos>; <def><pos>pl.</pos> of <er>Staff</er>.</def> <ldquo/Banners, scarves and <xex>staves</xex>.<rdquo/ <au>R. Browning.</au> <def2>Also <pr>(st<amac/vz)</pr>, <def><pos>pl.</pos> of <er>Stave</er>.</def></def2><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stavesacre</ent><br/
<hw>Staves"a`cre</hw> <pr>(st<amac/vz"<amac/`k<etil/r)</pr>, <pos>n.</pos> <ety>[Corrupted from NL. <ets>staphis agria</ets>, Gr. <grk>stafi`s</grk> dried grape + <grk>'a`grios</grk> wild.]</ety> <fld>(Bot.)</fld> <def>A kind of larkspur (<spn>Delphinium Staphysagria</spn>), and its seeds, which are violently purgative and emetic.  They are used as a parasiticide, and in the East for poisoning fish.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stavewood</ent><br/
<hw>Stave"wood`</hw> <pr>(st<amac/v"w<oocr/d`)</pr>, <pos>n.</pos> <fld>(Bot.)</fld> <def>A tall tree (<spn>Simaruba amara</spn>) growing in tropical America. It is one of the trees which yields quassia.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Staving</ent><br/
<hw>Stav"ing</hw> <pr>(st<amac/v"<icr/ng)</pr>, <pos>n.</pos> <def>A casing or lining of staves; especially, one encircling a water wheel.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Staw</ent><br/
<hw>Staw</hw> <pr>(st<add/)</pr>, <pos>v. i.</pos> <ety>[Cf. Dan. <ets>staae</ets> to stand, Sw. <ets>st<aring/</ets>. <root/163.]</ety> <def>To be fixed or set; to stay.</def> <mark>[Prov. Eng.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stay</ent><br/
<hw>Stay</hw> <pr>(st<amac/)</pr>, <pos>n.</pos> <ety>[AS. <ets>staeg</ets>, akin to D., G., Icel., Sw., & Dan. <ets>stag</ets>; cf. OF. <ets>estai</ets>, F. <ets><eacute/tai</ets>, of Teutonic origin.]</ety> <fld>(Naut.)</fld> <def>A large, strong rope, employed to support a mast, by being extended from the head of one mast down to some other, or to some part of the vessel. Those which lead forward are called <xex>fore-and-aft stays</xex>; those which lead to the vessel's side are called <xex>backstays</xex>. See <xex>Illust.</xex> of <er>Ship</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><mcol><col><b>In stays</b></col>, <or/ <col><b>Hove in stays</b></col></mcol> <fld>(Naut.)</fld>, <cd>in the act or situation of staying, or going about from one tack to another.</cd> <au>R. H. Dana, Jr.</au> -- <col><b>Stay holes</b></col> <fld>(Naut.)</fld>, <cd>openings in the edge of a staysail through which the hanks pass which join it to the stay.</cd> -- <col><b>Stay tackle</b></col> <fld>(Naut.)</fld>, <cd>a tackle attached to a stay and used for hoisting or lowering heavy articles over the side.</cd> -- <col><b>To miss stays</b></col> <fld>(Naut.)</fld>, <cd>to fail in the attempt to go about.</cd> <au>Totten.</au> -- <col><b>Triatic stay</b></col> <fld>(Naut.)</fld>, <cd>a rope secured at the ends to the heads of the foremast and mainmast with thimbles spliced to its bight into which the stay tackles hook.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stay</ent><br/
<hw>Stay</hw> <pr>(st<amac/)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Stayed</conjf> <pr>(st<amac/d)</pr> or <conjf>Staid</conjf> <pr>(st<amac/d)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Staying</conjf>.]</vmorph> <ety>[OF. <ets>estayer</ets>, F. <ets><eacute/tayer</ets> to prop, fr. OF. <ets>estai</ets>, F. <ets><eacute/tai</ets>, a prop, probably fr. OD. <ets>stade</ets>, <ets>staeye</ets>, a prop, akin to E. <ets>stead</ets>; or cf. <ets>stay</ets> a rope to support a mast.  Cf. <er>Staid</er>, <pos>a.</pos>, <er>Stay</er>, <pos>v. i.</pos>]</ety> <sn>1.</sn> <def>To stop from motion or falling; to prop; to fix firmly; to hold up; to support.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Aaron and Hur <qex>stayed</qex> up his hands, the one on the one side, and the other on the other side.</q> <rj><qau>Ex. xvii. 12.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Sallows and reeds . . . for vineyards useful found<br/
To <qex>stay</qex> thy vines.</q> <rj><qau>Dryden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To support from sinking; to sustain with strength; to satisfy in part or for the time.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>He has devoured a whole loaf of bread and butter, and it has not <qex>staid</qex> his stomach for a minute.</q> <rj><qau>Sir W. Scott.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>To bear up under; to endure; to support; to resist successfully.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>She will not <qex>stay</qex> the siege of loving terms,<br/
Nor bide the encounter of assailing eyes.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>To hold from proceeding; to withhold; to restrain; to stop; to hold.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Him backward overthrew and down him <qex>stayed</qex><br/
With their rude hands and grisly grapplement.</q> <rj><qau>Spenser.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>All that may <qex>stay</qex> their minds from thinking that true which they heartily wish were false.</q> <rj><qau>Hooker.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <def>To hinder; to delay; to detain; to keep back.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Your ships are <qex>stayed</qex> at Venice.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>This business <qex>staid</qex> me in London almost a week.</q> <rj><qau>Evelyn.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>I was willing to <qex>stay</qex> my reader on an argument that appeared to me new.</q> <rj><qau>Locke.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>6.</sn> <def>To remain for the purpose of; to wait for.</def> <ldquo/I <xex>stay</xex> dinner there.<rdquo/  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>7.</sn> <def>To cause to cease; to put an end to.</def><br/
[<source>1913 Webster</source>]</p>

<p><q><qex>Stay</qex> your strife.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>For flattering planets seemed to say<br/
This child should ills of ages <qex>stay</qex>.</q> <rj><qau>Emerson.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>8.</sn> <fld>(Engin.)</fld> <def>To fasten or secure with stays; <as>as, to <ex>stay</ex> a flat sheet in a steam boiler</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>9.</sn> <fld>(Naut.)</fld> <def>To tack, as a vessel, so that the other side of the vessel shall be presented to the wind.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>To stay a mast</b></col> <fld>(Naut.)</fld>, <cd>to incline it forward or aft, or to one side, by the stays and backstays.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stay</ent><br/
<hw>Stay</hw> <pr>(st<amac/)</pr>, <pos>v. i.</pos> <ety>[<root/163. See <er>Stay</er> to hold up, prop.]</ety> <sn>1.</sn> <def>To remain; to continue in a place; to abide fixed for a space of time; to stop; to stand still.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>She would command the hasty sun to <qex>stay</qex>.</q> <rj><qau>Spenser.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q><qex>Stay</qex>, I command you; <qex>stay</qex> and hear me first.</q> <rj><qau>Dryden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>I <qex>stay</qex> a little longer, as one <qex>stays</qex><br/
To cover up the embers that still burn.</q> <rj><qau>Longfellow.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To continue in a state.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The flames augment, and <qex>stay</qex><br/
At their full height, then languish to decay.</q> <rj><qau>Dryden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>To wait; to attend; to forbear to act.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>I 'll tell thee all my whole device<br/
When I am in my coach, which <qex>stays</qex> for us.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>The father can not <qex>stay</qex> any longer for the fortune.</q> <rj><qau>Locke.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>To dwell; to tarry; to linger.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>I must <qex>stay</qex> a little on one action.</q> <rj><qau>Dryden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <def>To rest; to depend; to rely; to stand; to insist.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>I <qex>stay</qex> here on my bond.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Ye despise this word, and trust in oppression and perverseness, and <qex>stay</qex> thereon.</q> <rj><qau>Isa. xxx. 12.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>6.</sn> <def>To come to an end; to cease; <as>as, that day the storm <ex>stayed</ex></as>.</def> <mark>[Archaic]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>Here my commission <qex>stays</qex>.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>7.</sn> <def>To hold out in a race or other contest; <as>as, a horse <ex>stays</ex> well</as>.</def> <mark>[Colloq.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><sn>8.</sn> <fld>(Naut.)</fld> <def>To change tack, as a ship.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stay</ent><br/
<hw>Stay</hw>, <pos>n.</pos> <ety>[Cf. OF. <ets>estai</ets>, F. <ets><eacute/tai</ets> support, and E. <ets>stay</ets> a rope to support a mast.]</ety> <sn>1.</sn> <def>That which serves as a prop; a support.</def> <ldquo/My only strength and <xex>stay</xex>.<rdquo/  <rj><au>Milton.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Trees serve as so many <qex>stays</qex> for their vines.</q> <rj><qau>Addison.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Lord Liverpool is the single <qex>stay</qex> of this ministry.</q> <rj><qau>Coleridge.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <pluf>pl.</pluf> <def>A corset stiffened with whalebone or other material, worn by women, and rarely by men.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>How the strait <qex>stays</qex> the slender waist constrain.</q> <rj><qau>Gay.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>Continuance in a place; abode for a space of time; sojourn; <as>as, you make a short <ex>stay</ex> in this city</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Make haste, and leave thy business and thy care;<br/
No mortal interest can be worth thy <qex>stay</qex>.</q> <rj><qau>Dryden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Embrace the hero and his <qex>stay</qex> implore.</q> <rj><qau>Waller.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>Cessation of motion or progression; stand; stop.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Made of sphere metal, never to decay<br/
Until his revolution was at <qex>stay</qex>.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Affairs of state seemed rather to stand at a <qex>stay</qex>.</q> <rj><qau>Hayward.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <def>Hindrance; let; check.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>They were able to read good authors without any <qex>stay</qex>, if the book were not false.</q> <rj><qau>Robynson (More's Utopia).</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>6.</sn> <def>Restraint of passion; moderation; caution; steadiness; sobriety.</def> <mark>[Obs.]</mark> <ldquo/Not grudging that thy lust hath bounds and <xex>stays</xex>.<rdquo/  <rj><au>Herbert.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>The wisdom, <qex>stay</qex>, and moderation of the king.</q> <rj><qau>Bacon.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>With prudent <qex>stay</qex> he long deferred<br/
The rough contention.</q> <rj><qau>Philips.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>7.</sn> <fld>(Engin.)</fld> <def>Strictly, a part in tension to hold the parts together, or stiffen them.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Stay bolt</b></col> <fld>(Mech.)</fld>, <cd>a bolt or short rod, connecting opposite plates, so as to prevent them from being bulged out when acted upon by a pressure which tends to force them apart, as in the leg of a steam boiler.</cd> -- <col><b>Stay busk</b></col>, <cd>a stiff piece of wood, steel, or whalebone, for the front support of a woman's stays.  Cf. <er>Busk</er>.</cd> -- <col><b>Stay rod</b></col>, <cd>a rod which acts as a stay, particularly in a steam boiler.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stayed</ent><br/
<hw>Stayed</hw> <pr>(st<amac/d)</pr>, <pos>a.</pos> <def>Staid; fixed; settled; sober; -- now written <xex>staid</xex>. See <er>Staid</er>.</def>  <rj><au>Bacon.</au>  <au>Pope.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stayedly</ent><br/
<hw>Stayed"ly</hw>, <pos>adv.</pos> <def>Staidly. See <er>Staidly</er>.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stayedness</ent><br/
<hw>Stayed"ness</hw>, <pos>n.</pos> <sn>1.</sn> <def>Staidness.</def> <mark>[Archaic]</mark>  <rj><au>W. Whately.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Solidity; weight.</def> <mark>[R.]</mark>  <rj><au>Camden.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stayer</ent><br/
<hw>Stay"er</hw> <pr>(st<amac/"<etil/r)</pr>, <pos>n.</pos> <def>One who upholds or supports that which props; one who, or that which, stays, stops, or restrains; also, colloquially, a horse, man, etc., that has endurance, as in a race.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Staylace</ent><br/
<hw>Stay"lace`</hw> <pr>(-l<amac/s`)</pr>, <pos>n.</pos> <def>A lace for fastening stays.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stayless</ent><br/
<hw>Stay"less</hw>, <pos>a.</pos> <def>Without stop or delay.</def>  <rj><au>Mir. for Mag.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Staymaker</ent><br/
<hw>Stay"mak`er</hw> <pr>(-m<amac/k`<etil/r)</pr>, <pos>n.</pos> <def>One whose occupation is to make stays.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Staynil</ent><br/
<hw>Stay"nil</hw> <pr>(-n<icr/l)</pr>, <pos>n.</pos> <fld>(Zool.)</fld> <def>The European starling.</def> <mark>[Prov. Eng.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Staysail</ent><br/
<hw>Stay"sail`</hw> <pr>(st<amac/"s<amac/l`;<it>among sailors</it>, st<amac/"s'l)</pr>, <pos>n.</pos> <fld>(Naut.)</fld> <def>Any sail extended on a stay.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stayship</ent><br/
<hw>Stay"ship`</hw> <pr>(-sh<icr/p`)</pr>, <pos>n.</pos> <fld>(Zool.)</fld> <def>A remora, -- fabled to stop ships by attaching itself to them.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stead</ent><br/
<hw>Stead</hw> <pr>(st<ecr/d)</pr>, <pos>n.</pos> <ety>[OE. <ets>stede</ets> place, AS. <ets>stede</ets>; akin to LG. & D. <ets>stede</ets>, OS. <ets>stad</ets>, <ets>stedi</ets>, OHG. <ets>stat</ets>, G. <ets>statt</ets>, <ets>st<aum/tte</ets>, Icel. <ets>sta<edh/r</ets>, Dan. <ets>sted</ets>, Sw. <ets>stad</ets>, Goth. <ets>sta<thorn/s</ets>, and E. <ets>stand</ets>.  <root/163. See <er>Stand</er>, and cf. <er>Staith</er>, <er>Stithy</er>.]</ety> <sn>1.</sn> <def>Place, or spot, in general.</def> <mark>[Obs., except in composition.]</mark>  <rj><au>Chaucer.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Fly, therefore, fly this fearful <qex>stead</qex> anon.</q> <rj><qau>Spenser.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Place or room which another had, has, or might have.</def> <ldquo/Stewards of your <xex>steads</xex>.<rdquo/  <rj><au>Piers Plowman.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>In <qex>stead</qex> of bounds, he a pillar set.</q> <rj><qau>Chaucer.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>A frame on which a bed is laid; a bedstead.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>The genial bed,<br/
Sallow the feet, the borders, and the <qex>stead</qex>.</q> <rj><qau>Dryden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>A farmhouse and offices.</def> <mark>[Prov. Eng. & Scot.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ The word is now commonly used as the last part of a compound; as, farm<xex>stead</xex>, home<xex>stead</xex>, road<xex>stead</xex>, etc.</note><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>In stead of</b></col>, <cd>in place of. See <er>Instead</er>.</cd> -- <mcol><col><b>To stand in stead</b></col>, <it>or</it> <col><b>To do stead</b></col></mcol>, <cd>to be of use or great advantage.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><q>The smallest act . . . shall <qex>stand</qex> us <qex>in</qex> great <qex>stead</qex>.</q> <rj><qau>Atterbury.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Here thy sword can <qex>do</qex> thee little <qex>stead</qex>.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stead</ent><br/
<hw>Stead</hw>, <pos>v. t.</pos> <sn>1.</sn> <def>To help; to support; to benefit; to assist.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Perhaps my succour or advisement meet,<br/
Mote <qex>stead</qex> you much your purpose to subdue.</q> <rj><qau>Spenser.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>It nothing <qex>steads</qex> us<br/
To chide him from our eaves.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To fill the place of.</def> <mark>[Obs.]</mark>  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Steadfast</ent><br/
<hw>Stead"fast</hw> <pr>(-f<adot/st)</pr>, <pos>a.</pos> <ety>[<ets>Stead</ets> + <ets>fast</ets>, that is, fast in place.]</ety> <ety>[Written also <ets>stedfast</ets>.]</ety> <sn>1.</sn> <def>Firmly fixed or established; fast fixed; firm.</def> <ldquo/This <xex>steadfast</xex> globe of earth.<rdquo/  <rj><au>Spenser.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Not fickle or wavering; constant; firm; resolute; unswerving; steady.</def> <ldquo/<xex>Steadfast</xex> eye.<rdquo/  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Abide <qex>steadfast</qex> unto him [thy neighbor] in the time of his trouble.</q> <rj><qau>Ecclus. xxii. 23.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Whom resist <qex>steadfast</qex> in the faith.</q> <rj><qau>1 Pet. v. 9.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Steadfastly</ent><br/
<hw>Stead"fast*ly</hw>, <pos>adv.</pos> <def>In a steadfast manner; firmly.</def><br/
[<source>1913 Webster</source>]</p>

<p><q><qex>Steadfastly</qex> believe that whatever God has revealed is infallibly true.</q> <rj><qau>Wake.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Steadfastness</ent><br/
<hw>Stead"fast*ness</hw>, <pos>n.</pos> <def>The quality or state of being steadfast; firmness; fixedness; constancy.</def> <ldquo/The <xex>steadfastness</xex> of your faith.<rdquo/  <rj><au>Col. ii. 5.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>To prove her wifehood and her <qex>steadfastness</qex>.</q> <rj><qau>Chaucer.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Steadily</ent><br/
<hw>Stead"i*ly</hw> <pr>(-<icr/*l<ycr/)</pr>, <pos>adv.</pos> <def>In a steady manner.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Steadiness</ent><br/
<hw>Stead"i*ness</hw>, <pos>n.</pos> <def>The quality or state of being steady.</def><br/
[<source>1913 Webster</source>]</p>

<p><q><qex>Steadiness</qex> is a point of prudence as well as of courage.</q> <rj><qau>L'Estrange.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><syn><b>Syn.</b> -- Constancy; resolution; unchangeableness.</syn><br/
[<source>1913 Webster</source>]</p>

<p><ent>Steading</ent><br/
<hw>Stead"ing</hw> <pr>(st<ecr/d"<icr/ng)</pr>, <pos>n.</pos> <def>The barns, stables, cattle-yards, etc., of a farm; -- called also <altname>onstead</altname>, <altname>farmstead</altname>, <altname>farm offices</altname>, or <altname>farmery</altname>.</def> <mark>[Prov. Eng. & Scot.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Steady</ent><br/
<hw>Stead"y</hw> <pr>(-<ycr/)</pr>, <pos>a.</pos> <amorph>[<pos>Compar.</pos> <adjf>Steadier</adjf> <pr>(-<icr/*<etil/r)</pr>; <pos>superl.</pos> <adjf>Steadiest</adjf>.]</amorph> <ety>[Cf. AS. <ets>stedig</ets> sterile, barren, <ets>stae<edh/<edh/ig</ets>, steady (in <ets>gestae<edh/<edh/ig</ets>), D. <ets>stedig</ets>, <ets>stadig</ets>, <ets>steeg</ets>, G. <ets>st<aum/tig</ets>, <ets>stetig</ets>. See <er>Stead</er>, <pos>n.</pos>]</ety> <sn>1.</sn> <def>Firm in standing or position; not tottering or shaking; fixed; firm.</def> <ldquo/The softest, <xex>steadiest</xex> plume.<rdquo/  <rj><au>Keble.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Their feet <qex>steady</qex>, their hands diligent, their eyes watchful, and their hearts resolute.</q> <rj><qau>Sir P. Sidney.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Constant in feeling, purpose, or pursuit; not fickle, changeable, or wavering; not easily moved or persuaded to alter a purpose; resolute; <as>as, a man <ex>steady</ex> in his principles, in his purpose, or in the pursuit of an object</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>Regular; constant; undeviating; uniform; <as>as, the <ex>steady</ex> course of the sun; a <ex>steady</ex> breeze of wind.</as></def><br/
[<source>1913 Webster</source>]</p>

<p><syn><b>Syn.</b> -- Fixed; regular; uniform; undeviating; invariable; unremitted; stable.</syn><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Steady rest</b></col> <fld>(Mach)</fld>, <cd>a rest in a turning lathe, to keep a long piece of work from trembling.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Steady</ent><br/
<hw>Stead"y</hw>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Steadied</conjf> <pr>(-<icr/d)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Steadying</conjf>.]</vmorph> <def>To make steady; to hold or keep from shaking, reeling, or falling; to make or keep firm; to support; to make constant, regular, or resolute.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Steady</ent><br/
<hw>Stead"y</hw>, <pos>v. i.</pos> <def>To become steady; to regain a steady position or state; to move steadily.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Without a breeze, without a tide,<br/
She <qex>steadies</qex> with upright keel.</q> <rj><qau>Coleridge.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Steak</ent><br/
<hw>Steak</hw> <pr>(st<amac/k)</pr>, <pos>n.</pos> <ety>[OE. <ets>steike</ets>, Icel. <ets>steik</ets>, akin to Icel. <ets>steikja</ets> to roast, <ets>stikna</ets> to be roasted or scorched, and E. <ets>stick</ets>, the steak being broiled on a spit. See <er>Stick</er>, <pos>v. t.</pos>]</ety> <def>A slice of beef, broiled, or cut for broiling; -- also extended to the meat of other large animals; <as>as, venison <ex>steak</ex>; bear <ex>steak</ex>; pork <ex>steak</ex>; turtle <ex>steak</ex></as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Steal</ent><br/
<hw>Steal</hw> <pr>(st<emac/l)</pr>, <pos>n.</pos> <ety>[See <er>Stale</er> a handle.]</ety> <def>A handle; a stale, or stele.</def> <mark>[Archaic or Prov. Eng.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>And in his hand a huge poleax did bear.<br/
Whose <qex>steale</qex> was iron-studded but not long.</q> <rj><qau>Spenser.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Steal</ent><br/
<hw>Steal</hw> <pr>(st<emac/l)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp.</pos> <conjf>Stole</conjf> <pr>(st<omac/l)</pr>; <pos>p. p.</pos> <conjf>Stolen</conjf> <pr>(st<omac/"l'n)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Stealing</conjf>.]</vmorph> <ety>[OE. <ets>stelen</ets>, AS. <ets>stelan</ets>; akin to OFries. <ets>stela</ets>, D. <ets>stelen</ets>, OHG. <ets>stelan</ets>, G. <ets>stehlen</ets>, Icel. <ets>stela</ets>, SW. <ets>stj<aum/la</ets>, Dan. <ets>stiaele</ets>, Goth. <ets>stilan</ets>.]</ety> <sn>1.</sn> <def>To take, and carry away, feloniously; to take without right or leave, and with intent to keep wrongfully; <as>as, to <ex>steal</ex> the personal goods of another</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Maugre thy heed, thou must for indigence<br/
Or <qex>steal</qex>, or beg, or borrow, thy dispense.</q> <rj><qau>Chaucer.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>The man who <qex>stole</qex> a goose and gave away the giblets in alms.</q> <rj><qau>G. Eliot.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><-- p. 1408 pr=VMG --></p>

<p><sn>2.</sn> <def>To withdraw or convey clandestinely (reflexive); hence, to creep furtively, or to insinuate.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>They could insinuate and <qex>steal</qex> themselves under the same by their humble carriage and submission.</q> <rj><qau>Spenser.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>He will <qex>steal</qex> himself into a man's favor.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>To gain by insinuating arts or covert means.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>So Absalom <qex>stole</qex> the hearts of the men of Israel.</q> <rj><qau>2 Sam. xv. 6.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>To get into one's power gradually and by imperceptible degrees; to take possession of by a gradual and imperceptible appropriation; -- with <xex>away</xex>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Variety of objects has a tendency to <qex>steal</qex> away the mind from its steady pursuit of any subject.</q> <rj><qau>I. Watts.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <def>To accomplish in a concealed or unobserved manner; to try to carry out secretly; <as>as, to <ex>steal</ex> a look</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Always, when thou changest thine opinion or course, profess it plainly, . . . and do not think to <qex>steal</qex> it.</q> <rj><qau>Bacon.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>To steal a march</b></col>, <cd>to march in a covert way; to gain an advantage unobserved; -- formerly followed by <xex>of</xex>, but now by <xex>on</xex> or <xex>upon</xex>, and sometimes by <xex>over</xex>; as, <xex>to steal a march</xex> upon one's political rivals.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><q>She yesterday wanted <qex>to steal a march</qex> of poor Liddy.</q> <rj><qau>Smollett.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Fifty thousand men can not easily <qex>steal a march</qex> over the sea.</q> <rj><qau>Walpole.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><syn><b>Syn.</b> -- To filch; pilfer; purloin; thieve.</syn><br/
[<source>1913 Webster</source>]</p>

<p><ent>Steal</ent><br/
<hw>Steal</hw> <pr>(st<emac/l)</pr>, <pos>v. i.</pos> <sn>1.</sn> <def>To practice, or be guilty of, theft; to commit larceny or theft.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Thou shalt not <qex>steal</qex>.</q> <rj><qau>Ex. xx. 15.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To withdraw, or pass privily; to slip in, along, or away, unperceived; to go or come furtively.</def>  <rj><au>Chaucer.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Fixed of mind to avoid further entreaty, and to fly all company, one night she <qex>stole</qex> away.</q> <rj><qau>Sir P. Sidney.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>From whom you now must <qex>steal</qex>, and take no leave.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>A soft and solemn breathing sound<br/
Rose like a steam of rich, distilled perfumes,<br/
And <qex>stole</qex> upon the air.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stealer</ent><br/
<hw>Steal"er</hw> <pr>(-<etil/r)</pr>, <pos>n.</pos> <sn>1.</sn> <def>One who steals; a thief.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Shipbuilding)</fld> <def>The endmost plank of a strake which stops short of the stem or stern.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stealing</ent><br/
<hw>Steal"ing</hw>, <pos>n.</pos> <sn>1.</sn> <def>The act of taking feloniously the personal property of another without his consent and knowledge; theft; larceny.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>That which is stolen; stolen property; -- chiefly used in the plural.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stealingly</ent><br/
<hw>Steal"ing*ly</hw>, <pos>adv.</pos> <def>By stealing, or as by stealing, furtively, or by an invisible motion.</def>  <rj><au>Sir P. Sidney.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stealth</ent><br/
<hw>Stealth</hw> <pr>(st<ecr/lth)</pr>, <pos>n.</pos> <ety>[OE. <ets>stal<thorn/e</ets>. See <er>Steal</er>, <pos>v. t.</pos>]</ety> <sn>1.</sn> <def>The act of stealing; theft.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>The owner proveth the <qex>stealth</qex> to have been committed upon him by such an outlaw.</q> <rj><qau>Spenser.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>The thing stolen; stolen property.</def> <mark>[Obs.]</mark>  <ldquo/Sluttish dens . . . serving to cover <xex>stealths</xex>.<rdquo/  <rj><au>Sir W. Raleigh.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>The bringing to pass anything in a secret or concealed manner; a secret procedure; a clandestine practice or action; -- in either a good or a bad sense.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Do good by <qex>stealth</qex>, and blush to find it fame.</q> <rj><qau>Pope.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>The monarch, blinded with desire of wealth,<br/
With steel invades the brother's life by <qex>stealth</qex>.</q> <rj><qau>Dryden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>I told him of your <qex>stealth</qex> unto this wood.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stealthful</ent><br/
<hw>Stealth"ful</hw> <pr>(-f<usdot/l)</pr>, <pos>a.</pos> <def>Given to stealth; stealthy.</def> <mark>[Obs.]</mark> -- <wordforms><wf>Stealth"ful*ly</wf>, <pos>adv.</pos> <mark>[Obs.]</mark> -- <wf>Stealth"ful*ness</wf>, <pos>n.</pos> <mark>[Obs.]</mark></wordforms><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stealthily</ent><br/
<hw>Stealth"i*ly</hw> <pr>(-<icr/*l<ycr/)</pr>, <pos>adv.</pos> <def>In a stealthy manner.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stealthiness</ent><br/
<hw>Stealth"i*ness</hw>, <pos>n.</pos> <def>The state, quality, or character of being stealthy; stealth.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stealthlike</ent><br/
<hw>Stealth"like`</hw> <pr>(-l<imac/k`)</pr>, <pos>a.</pos> <def>Stealthy; sly.</def>  <rj><au>Wordsworth.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stealthy</ent><br/
<hw>Stealth"y</hw> <pr>(-<ycr/)</pr>, <pos>a.</pos> <amorph>[<pos>Compar.</pos> <adjf>Stealthier</adjf> <pr>(-<icr/*<etil/r)</pr>; <pos>superl.</pos> <adjf>Stealthiest</adjf>.]</amorph> <def>Done by stealth; accomplished clandestinely; unperceived; secret; furtive; sly.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>[Withered murder] with his <qex>stealthy</qex> pace, . . . <br/
Moves like a ghost.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Steam</ent><br/
<hw>Steam</hw> <pr>(st<emac/m)</pr>, <pos>n.</pos> <ety>[OE. <ets>stem</ets>, <ets>steem</ets>, vapor, flame, AS. <ets>ste<aacute/m</ets> vapor, smoke, odor; akin to D. <ets>stoom</ets> steam, perhaps originally, a pillar, or something rising like a pillar; cf. Gr. <grk>sty`ein</grk> to erect, <grk>sty^los</grk> a pillar, and E. <ets>stand</ets>.]</ety> <sn>1.</sn> <def>The elastic, aeriform fluid into which water is converted when heated to the boiling point; water in the state of vapor; gaseous water.</def><br/
[<source>1913 Webster</source> + <source>PJC</source>]</p>

<p><sn>2.</sn> <def>The mist formed by condensed vapor; visible vapor; -- so called in popular usage.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>Any exhalation.</def> <ldquo/A <xex>steam</xex> of rich, distilled perfumes.<rdquo/  <rj><au>Milton.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Dry steam</b></col>, <cd>steam which does not contain water held in suspension mechanically; -- sometimes applied to superheated steam.</cd> -- <col><b>Exhaust steam</b></col>. <cd>See under <er>Exhaust</er>.</cd> -- <mcol><col><b>High steam</b></col>, <it>or</it> <col><b>High-pressure steam</b></col></mcol>, <cd>steam of which the pressure greatly exceeds that of the atmosphere.</cd> -- <mcol><col><b>Low steam</b></col>, <it>or</it> <col><b>Low-pressure steam</b></col></mcol>, <cd>steam of which the pressure is less than, equal to, or not greatly above, that of the atmosphere.</cd> -- <col><b>Saturated steam</b></col>, <cd>steam at the temperature of the boiling point which corresponds to its pressure; -- sometimes also applied to <altname>wet steam</altname>.</cd> -- <col><b>Superheated steam</b></col>, <cd>steam heated to a temperature higher than the boiling point corresponding to its pressure. It can not exist in contact with water, nor contain water, and resembles a perfect gas; -- called also <altname>surcharged steam</altname>, <altname>anhydrous steam</altname>, and <altname>steam gas</altname>.</cd> -- <col><b>Wet steam</b></col>, <cd>steam which contains water held in suspension mechanically; -- called also <altname>misty steam</altname>.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ <xex>Steam</xex> is often used adjectively, and in combination, to denote, <xex>produced by heat</xex>, or <xex>operated by power</xex>, <xex>derived from steam</xex>, in distinction from other sources of power; as in <xex>steam</xex> boiler or <xex>steam</xex>-boiler, <xex>steam</xex> dredger or <xex>steam</xex>-dredger, <xex>steam</xex> engine or <xex>steam</xex>-engine, <xex>steam</xex> heat, <xex>steam</xex> plow or <xex>steam</xex>-plow, etc.</note><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Steam blower</b></col>. <sd>(a)</sd> <cd>A blower for producing a draught consisting of a jet or jets of steam in a chimney or under a fire</cd>. <sd>(b)</sd> <cd>A fan blower driven directly by a steam engine.</cd> -- <col><b>Steam boiler</b></col>, <cd>a boiler for producing steam. See <er>Boiler</er>, 3, and Note. In the illustration, the shell <xex>a</xex> of the boiler is partly in section, showing the tubes, or flues, which the hot gases, from the fire beneath the boiler, enter, after traversing the outside of the shell, and through which the gases are led to the smoke pipe <xex>d</xex>, which delivers them to the chimney; <xex>b</xex> is the manhole; <xex>c</xex> the dome; <xex>e</xex> the steam pipe; <xex>f</xex> the feed and blow-off pipe; <xex>g</xex> the safety valve; <xex>h</xex>the water gauge.</cd> -- <col><b>Steam car</b></col>, <cd>a car driven by steam power, or drawn by a locomotive.</cd> -- <col><b>Steam carriage</b></col>, <cd>a carriage upon wheels moved on common roads by steam.</cd> -- <col><b>Steam casing</b></col>. <cd>See <cref>Steam jacket</cref>, under <er>Jacket</er>.</cd> -- <col><b>Steam chest</b></col>, <cd>the box or chamber from which steam is distributed to the cylinder of a steam engine, steam pump, etc., and which usually contains one or more valves; -- called also <altname>valve chest</altname>, and <altname>valve box</altname>.  See <xex>Illust.</xex> of <cref>Slide valve</cref>, under <er>Slide</er>.</cd> -- <col><b>Steam chimney</b></col>, <cd>an annular chamber around the chimney of a boiler furnace, for drying steam.</cd> -- <col><b>Steam coil</b></col>, <cd>a coil of pipe, or a collection of connected pipes, for containing steam; -- used for heating, drying, etc.</cd> -- <col><b>Steam colors</b></col> <fld>(Calico Printing)</fld>, <cd>colors in which the chemical reaction fixing the coloring matter in the fiber is produced by steam.</cd> -- <col><b>Steam cylinder</b></col>, <cd>the cylinder of a steam engine, which contains the piston. See <xex>Illust.</xex> of <cref>Slide valve</cref>, under <er>Slide</er>.</cd> -- <col><b>Steam dome</b></col> <fld>(Steam Boilers)</fld>, <cd>a chamber upon the top of the boiler, from which steam is conducted to the engine. See <xex>Illust.</xex> of <xex>Steam boiler</xex>, above.</cd> -- <col><b>Steam fire engine</b></col>, <cd>a fire engine consisting of a steam boiler and engine, and pump which is driven by the engine, combined and mounted on wheels. It is usually drawn by horses, but is sometimes made self-propelling.</cd> -- <col><b>Steam fitter</b></col>, <cd>a fitter of steam pipes.</cd> -- <col><b>Steam fitting</b></col>, <cd>the act or the occupation of a steam fitter; also, a pipe fitting for steam pipes.</cd> -- <col><b>Steam gas</b></col>. <cd>See <cref>Superheated steam</cref>, above.</cd> -- <col><b>Steam gauge</b></col>, <cd>an instrument for indicating the pressure of the steam in a boiler. The <stype>mercurial steam gauge</stype> is a bent tube partially filled with mercury, one end of which is connected with the boiler while the other is open to the air, so that the steam by its pressure raises the mercury in the long limb of the tube to a height proportioned to that pressure.  A more common form, especially for high pressures, consists of a spring pressed upon by the steam, and connected with the pointer of a dial. The spring may be a flattened, bent tube, closed at one end, which the entering steam tends to straighten, or it may be a diaphragm of elastic metal, or a mass of confined air, etc.</cd> -- <col><b>Steam gun</b></col>, <cd>a machine or contrivance from which projectiles may be thrown by the elastic force of steam.</cd> -- <col><b>Steam hammer</b></col>, <cd>a hammer for forging, which is worked directly by steam; especially, a hammer which is guided vertically and operated by a vertical steam cylinder located directly over an anvil. In the variety known as <xex>Nasmyth's</xex>, the cylinder is fixed, and the hammer is attached to the piston rod. In that known as <xex>Condie's</xex>, the piston is fixed, and the hammer attached to the lower end of the cylinder.</cd> -- <col><b>Steam heater</b></col>. <sd>(a)</sd> <cd>A radiator heated by steam</cd>. <sd>(b)</sd> <cd>An apparatus consisting of a steam boiler, radiator, piping, and fixures for warming a house by steam.</cd> -- <col><b>Steam jacket</b></col>. <cd>See under <er>Jacket</er>.</cd> -- <col><b>Steam packet</b></col>, <cd>a packet or vessel propelled by steam, and running periodically between certain ports.</cd> -- <col><b>Steam pipe</b></col>, <cd>any pipe for conveying steam; specifically, a pipe through which steam is supplied to an engine.</cd> -- <mcol><col><b>Steam plow</b></col> <it>or</it> <col><b>Steam plough</b></col></mcol>, <cd>a plow, or gang of plows, moved by a steam engine.</cd> -- <col><b>Steam port</b></col>, <cd>an opening for steam to pass through, as from the steam chest into the cylinder.</cd> -- <col><b>Steam power</b></col>, <cd>the force or energy of steam applied to produce results; power derived from a steam engine.</cd> -- <col><b>Steam propeller</b></col>. <cd>See <er>Propeller</er>.</cd> -- <col><b>Steam pump</b></col>, <cd>a small pumping engine operated by steam. It is usually direct-acting.</cd> -- <col><b>Steam room</b></col> <fld>(Steam Boilers)</fld>, <cd>the space in the boiler above the water level, and in the dome, which contains steam.</cd> -- <col><b>Steam table</b></col>, <cd>a table on which are dishes heated by steam for keeping food warm in the carving room of a hotel, restaurant, etc.</cd> -- <col><b>Steam trap</b></col>, <cd>a self-acting device by means of which water that accumulates in a pipe or vessel containing steam will be discharged without permitting steam to escape.</cd> -- <col><b>Steam tug</b></col>, <cd>a steam vessel used in towing or propelling ships.</cd> -- <col><b>Steam vessel</b></col>, <cd>a vessel propelled by steam; a steamboat or steamship; a steamer.</cd> -- <col><b>Steam whistle</b></col>, <cd>an apparatus attached to a steam boiler, as of a locomotive, through which steam is rapidly discharged, producing a loud whistle which serves as a warning or a signal.  The steam issues from a narrow annular orifice around the upper edge of the lower cup or hemisphere, striking the thin edge of the bell above it, and producing sound in the manner of an organ pipe or a common whistle.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Steam</ent><br/
<hw>Steam</hw> <pr>(st<emac/m)</pr>, <pos>v. i.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Steamed</conjf> <pr>(st<emac/md)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Steaming</conjf>.]</vmorph> <sn>1.</sn> <def>To emit steam or vapor.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>My brother's ghost hangs hovering there,<br/
O'er his warm blood, that <qex>steams</qex> into the air.</q> <rj><qau>Dryden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Let the crude humors dance<br/
In heated brass, <qex>steaming</qex> with fire intense.</q> <rj><qau>J. Philips.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To rise in vapor; to issue, or pass off, as vapor.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The dissolved amber . . . <qex>steamed</qex> away into the air.</q> <rj><qau>Boyle.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>To move or travel by the agency of steam.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The vessel <qex>steamed</qex> out of port.</q> <rj><qau>N. P. Willis.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>To generate steam; <as>as, the boiler <ex>steams</ex> well</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Steam</ent><br/
<hw>Steam</hw> <pr>(st<emac/m)</pr>, <pos>v. t.</pos> <sn>1.</sn> <def>To exhale.</def> <mark>[Obs.]</mark>  <rj><au>Spenser.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To expose to the action of steam; to apply steam to for softening, dressing, or preparing; <as>as, to <ex>steam</ex> wood; to <ex>steam</ex>cloth; to <ex>steam</ex> food, etc.</as></def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Steamboat</ent><br/
<hw>Steam"boat`</hw> <pr>(-b<omac/t`)</pr>, <pos>n.</pos> <def>A boat or vessel propelled by steam power; -- generally used of river or coasting craft, as distinguished from ocean steamers.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Steamboating</ent><br/
<hw>Steam"boat`ing</hw>, <pos>n.</pos> <sn>1.</sn> <def>The occupation or business of running a steamboat, or of transporting merchandise, passengers, etc., by steamboats.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Bookbinding)</fld> <def>The shearing of a pile of books which are as yet uncovered, or out of boards.</def>  <rj><au>Knight.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Steam engine</ent><br/
<hw>Steam" en"gine</hw> <pr>(<ecr/n"j<icr/n)</pr>. <def>An engine moved by steam.</def><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ In its most common forms its essential parts are a <xex>piston</xex>, a <xex>cylinder</xex>, and a <xex>valve gear</xex>. The piston works in the cylinder, to which steam is admitted by the action of the valve gear, and communicates motion to the machinery to be actuated. Steam engines are thus classified: 1. According to the way the steam is used or applied, as <xex>condensing</xex>, <xex>noncondensing</xex>, <xex>compound</xex>, <xex>double-acting</xex>, <xex>single-acting</xex>, <xex>triple-expansion</xex>, etc. 2. According to the motion of the piston, as <xex>reciprocating</xex>, <xex>rotary</xex>, etc. 3. According to the motion imparted by the engine, as <xex>rotative</xex> and <xex>nonrotative</xex>. 4. According to the arrangement of the engine, as <xex>stationary</xex>, <xex>portable</xex>, and <xex>semiportable</xex> engines, <xex>horizontal</xex> and <xex>vertical</xex> engines, <xex>beam</xex> engine, <xex>oscillating</xex> engine, <xex>direct-acting</xex> and <xex>back-acting</xex> engines, etc. 5. According to their uses, as <xex>portable</xex>, <xex>marine</xex>, <xex>locomotive</xex>, <xex>pumping</xex>, <xex>blowing</xex>, <xex>winding</xex>, and <xex>stationary</xex> engines, the latter term referring to factory engines, etc., and not technically to pumping or blowing engines. <xex>Locomotive</xex> and <xex>portable</xex> engines are usually high-pressure, noncondensing, rotative, and direct-acting.  <xex>Marine</xex> engines are high or low pressure, rotative, and generally condensing, double-acting, and compound. <xex>Paddle</xex> engines are generally beam, side-lever, oscillating, or direct-acting.  <xex>Screw</xex> engines are generally direct-acting, back-acting, or oscillating.  <xex>Stationary</xex> engines belong to various classes, but are generally rotative. A horizontal or inclined stationary steam engine is called a <xex>left-hand</xex> or a <xex>right-hand</xex> engine when the crank shaft and driving pulley are on the left-hand side, or the right-hand side, respectively, of the engine, to a person looking at them from the cylinder, and is said to run <xex>forward</xex> or <xex>backward</xex> when the crank traverses the upward half, or lower half, respectively, of its path, while the piston rod makes its stroke outward from the cylinder. A marine engine, or the engine of a locomotive, is said to run forward when its motion is such as would propel the vessel or the locomotive forward. Steam engines are further classified as <xex>double-cylinder</xex>, <xex>disk</xex>, <xex>semicylinder</xex>, <xex>trunk</xex> engines, etc. Machines, such as cranes, hammers, etc., of which the steam engine forms a part, are called <xex>steam cranes</xex>, <xex>steam hammers</xex>, etc. See <xex>Illustration</xex> in Appendix.</note><br/
[<source>1913 Webster</source>]</p>

<p><cs><mcol><col><b>Back-acting steam engine</b></col>, <it>or</it> <col><b>Back-action steam engine</b></col></mcol>, <cd>a steam engine in which the motion is transmitted backward from the crosshead to a crank which is between the crosshead and the cylinder, or beyond the cylinder.</cd> -- <col><b>Portable steam engine</b></col>, <cd>a steam engine combined with, and attached to, a boiler which is mounted on wheels so as to admit of easy transportation; -- used for driving machinery in the field, as thrashing machines, draining pumps, etc.</cd> -- <col><b>Semiportable steam engine</b></col>, <cd>a steam engine combined with, and attached to, a steam boiler, but not mounted on wheels.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Steamer</ent><br/
<hw>Steam"er</hw> <pr>(-<etil/r)</pr>, <pos>n.</pos> <sn>1.</sn> <def>A vessel propelled by steam; a steamship or steamboat.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A steam fire engine. See under <er>Steam</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>A road locomotive for use on common roads, as in agricultural operations.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>A vessel in which articles are subjected to the action of steam, as in washing, in cookery, and in various processes of manufacture.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <fld>(Zool.)</fld> <def>The steamer duck.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Steamer duck</b></col> <fld>(Zool.)</fld>, <cd>a sea duck (<spn>Tachyeres cinereus</spn>), native of Patagonia and Terra del Fuego, which swims and dives with great agility, but which, when full grown, is incapable of flight, owing to its very small wings.  Called also <altname>loggerhead</altname>, <altname>race horse</altname>, and <altname>side-wheel duck</altname>.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Steaminess</ent><br/
<hw>Steam"i*ness</hw> <pr>(-<icr/*n<ecr/s)</pr>, <pos>n.</pos> <def>The quality or condition of being steamy; vaporousness; mistiness.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Steamship</ent><br/
<hw>Steam"ship`</hw> <pr>(-sh<icr/p`)</pr>, <pos>n.</pos> <def>A ship or seagoing vessel propelled by the power of steam; a steamer.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>steam up</ent><br/
<hw>Steam` up"</hw> <pr>(st<emac/m` <ucr/p")</pr>, <pos>v. t.</pos> <def>To cause to be covered by a translucent layer of condensed water in fine droplets, such as by breathing on a cold window; to fog; <as>as, to <ex>steam up</ex> one's eyeglasses</as>.</def><br/
[<source>PJC</source>]</p>

<p><ent>Steamy</ent><br/
<hw>Steam"y</hw> <pr>(-<ycr/)</pr>, <pos>a.</pos> <sn>1.</sn> <def>Consisting of, or resembling, steam; vaporous; misty.</def>  <rj><au>Cowper.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Emitting steam; full of steam.</def><br/
[<source>PJC</source>]</p>

<p><sn>3.</sn> <def>Erotic; consisting of or depicting passionate sexual activity; <as>as, a <ex>steamy</ex> love affair; the <ex>steamy</ex> scenes were cut from the film to avoid an X rating</as>.</def><br/
[<source>PJC</source>]</p>

<p><ent>Stean</ent><br/
<hw>Stean</hw> <pr>(st<emac/n)</pr>, <pos>n. & v.</pos> <def>See <er>Steen</er>.</def>  <rj><au>Spenser.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Steaning</ent><br/
<hw>Stean"ing</hw>, <pos>n.</pos> <def>See <er>Steening</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Steapsin</ent><br/
<hw>Ste*ap"sin</hw> <pr>(st<esl/*<acr/p"s<icr/n)</pr>, <pos>n.</pos> <fld>(Physiol. Chem.)</fld> <def>An unorganized ferment or enzyme present in pancreatic juice.  It decomposes neutral fats into glycerin and fatty acids.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stearate</ent><br/
<hw>Ste"a*rate</hw> <pr>(st<emac/"<adot/*r<asl/t)</pr>, <pos>n.</pos> <fld>(Chem.)</fld> <def>A salt of stearic acid; <as>as, ordinary soap consists largely of sodium or potassium <ex>stearates</ex></as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><-- p. 1409 pr=VMG --></p>

<p><ent>Stearic</ent><br/
<hw>Ste*ar"ic</hw> <pr>(st<esl/*<acr/r"<icr/k)</pr>, <pos>a.</pos> <ety>[Cf. F. <ets>st<eacute/arique</ets>.]</ety> <fld>(Physiol. Chem.)</fld> <def>Pertaining to, or obtained from, stearin or tallow; resembling tallow.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Stearic acid</b></col> <fld>(Chem.)</fld>, <cd>a monobasic fatty acid, obtained in the form of white crystalline scales, soluble in alcohol and ether. It melts to an oily liquid at 69<deg/C.</cd></cs><-- b.p. 383<deg/. <chform>C18H36O2</chform>, <chform>CH3.(CH2)16.COOH</chform>; sodium stearate, with sodium palmitate, is the main component of ordinary bar soaps (Such as Ivory soap). --><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stearin</ent><br/
<hw>Ste"a*rin</hw> <pr>(st<emac/"<adot/*r<icr/n)</pr>, <pos>n.</pos> <ety>[Gr. <grk>ste`ar</grk> tallow, suet: cf. F. <ets>st<eacute/arine</ets>.]</ety> <fld>(Physiol. Chem.)</fld> <def>One of the constituents of animal fats and also of some vegetable fats, as the butter of cacao. It is especially characterized by its solidity, so that when present in considerable quantity it materially increases the hardness, or raises the melting point, of the fat, as in mutton tallow. Chemically, it is a compound of glyceryl with three molecules of stearic acid, and hence is technically called <xex>tristearin</xex>, or <xex>glyceryl tristearate</xex>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stearolic</ent><br/
<hw>Ste`a*rol"ic</hw> <pr>(-r<ocr/l"<icr/k)</pr>, <pos>a.</pos> <ety>[<ets>Stear</ets>ic + <ets>ol</ets>eic + <ets>-ic</ets>.]</ety> <fld>(Chem.)</fld> <def>Of, pertaining to, or designating, an acid of the acetylene series, isologous with stearic acid, and obtained, as a white crystalline substance, from oleic acid.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stearone</ent><br/
<hw>Ste"a*rone</hw> <pr>(st<emac/"<adot/*r<omac/n)</pr>, <pos>n.</pos> <fld>(Chem.)</fld> <def>The ketone of stearic acid, obtained as a white crystalline substance, (<chform>C17H35)2.CO</chform>, by the distillation of calcium stearate.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stearoptene</ent><br/
<hw>Ste`a*rop"tene</hw> <pr>(-r<ocr/p"t<emac/n)</pr>, <pos>n.</pos> <ety>[<ets>Stear</ets>ic + <ets>-optene</ets> as in elae<ets>optene</ets>.]</ety> <fld>(Chem.)</fld> <def>The more solid ingredient of certain volatile oils; -- contrasted with <xex>elaeoptene</xex>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stearrhea</ent><br/
||<hw>Ste`ar*rhe"a</hw> <pr>(st<emac/`<ait/r*r<emac/"<adot/)</pr>, <pos>n.</pos> <ety>[NL., from Gr. <grk>ste`ar</grk> tallow + <grk>"rei^n</grk> to flow.]</ety> <fld>(Med.)</fld> <def>Seborrhea.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stearyl</ent><br/
<hw>Ste"a*ryl</hw> <pr>(st<emac/"<adot/*r<icr/l)</pr>, <pos>n.</pos> <ety>[<ets>Stear</ets>ic + <ets>-yl</ets>.]</ety> <fld>(Chem.)</fld> <def>The hypothetical radical characteristic of stearic acid.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Steatite</ent><br/
<hw>Ste"a*tite</hw> <pr>(-t<imac/t)</pr>, <pos>n.</pos> <ety>[Gr. <grk>ste`ar</grk>, <grk>ste`atos</grk>, fat, tallow: cf. F. <ets>st<eacute/atite</ets>.]</ety> <fld>(Min.)</fld> <def>A massive variety of talc, of a grayish green or brown color. It forms extensive beds, and is quarried for fireplaces and for coarse utensils. Called also <altname>potstone</altname>, <altname>lard stone</altname>, and <altname>soapstone</altname>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Steatitic</ent><br/
<hw>Ste`a*tit"ic</hw> <pr>(-t<icr/t"<icr/k)</pr>, <pos>a.</pos> <fld>(Min.)</fld> <def>Pertaining to, or of the nature of, steatite; containing or resembling steatite.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Steatoma</ent><br/
||<hw>Ste`a*to"ma</hw> <pr>(-t<omac/"m<adot/)</pr>, <pos>n.</pos> <ety>[L., fr. Gr. <grk>stea`twma</grk>, fr. <grk>steatoy^n</grk> to turn into tallow or suet, fr. <grk>ste`ar</grk>, <grk>-atos</grk>, fat, suet.]</ety> <fld>(Med.)</fld> <def>A cyst containing matter like suet.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Steatomatous</ent><br/
<hw>Ste`a*tom"a*tous</hw> <pr>(-t<ocr/m"<adot/*t<ucr/s <or/ -t<omac/"m<adot/- ; 277)</pr>, <pos>a.</pos> <ety>[Cf. F. <ets>st<eacute/atomateux</ets>.]</ety> <fld>(Med.)</fld> <def>Of the nature of a steatoma.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Steatopyga</ent><br/
||<hw>Ste`a*top"y*ga</hw> <pr>(-t<ocr/p"<icr/*g<adot/)</pr>, <pos>n.</pos> <ety>[NL., fr. Gr. <grk>ste`ar</grk>, <grk>ste`atos</grk>, fat + <grk>pygh`</grk> the buttocks.]</ety> <def>A remarkable accretion of fat upon the buttocks of Africans of certain tribes, especially of Hottentot (Khoikhoi) women.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Steatopygous</ent><br/
<hw>Ste`a*top"y*gous</hw> <pr>(-g<ucr/s)</pr>, <pos>a.</pos> <def>Having fat buttocks.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Specimens of the <qex>steatopygous</qex> Abyssinian breed.</q> <rj><qau>Burton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stedfastly</ent><br/
<ent>Stedfast</ent><br/
<ent>Sted</ent><br/
<mhw><hw>Sted</hw> <pr>(st<ecr/d)</pr>, <pos>n.</pos>, <hw>Sted"fast</hw> <pr>(-f<adot/st)</pr>, <pos>a.</pos>, <hw>Sted"fast*ly</hw>, <pos>adv.</pos>, etc.</mhw> <def>See <er>Stead</er>, <er>Steadfast</er>, etc.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stee</ent><br/
<hw>Stee</hw> <pr>(st<emac/)</pr>, <pos>n.</pos> <ety>[Cf. G. <ets>stiege</ets>. <root/164. See <er>Stair</er>.]</ety> <def>A ladder.</def> <mark>[Obs. or Prov. Eng.]</mark> <altsp>[Written also <asp>stey</asp>.]</altsp><br/
[<source>1913 Webster</source>]</p>

<p><ent>Steed</ent><br/
<hw>Steed</hw> <pr>(st<emac/d)</pr>, <pos>n.</pos> <ety>[OE. <ets>stede</ets>, AS. <ets>st<emac/da</ets> a stud-horse, war horse, fr. <ets>st<omac/d</ets> a stud of breeding steeds; akin to G. <ets>stute</ets> a mare, Icel. <ets>stedda</ets>, <ets>st<omac/<edh/</ets>, a stud.  <root/163. See <er>Stud</er> of horses.]</ety> <def>A horse, especially a spirited horse for state or war; -- used chiefly in poetry or stately prose.</def> <ldquo/A knight upon a <xex>steed</xex>.<rdquo/  <rj><au>Chaucer.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Mounted upon a hot and fiery <qex>steed</qex>.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Steedless</ent><br/
<hw>Steed"less</hw>, <pos>a.</pos> <def>Having no steed; without a horse.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Steik</ent><br/
<ent>Steek</ent><br/
<mhw>{ <hw>Steek</hw>, <hw>Steik</hw> }</mhw> <pr>(st<emac/k)</pr>, <pos>v. t.</pos> <ety>[Cf. <er>Stick</er>, <pos>v. t.</pos>]</ety> <def>To pierce with a sharp instrument; hence, to stitch; to sew; also, to fix; to fasten.</def> <mark>[Scot.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Steel</ent><br/
<hw>Steel</hw> <pr>(st<emac/l)</pr>, <pos>n.</pos> <ety>[AS. <ets>st<emac/l</ets>, <ets>st<ymac/l</ets>, <ets>st<ymac/le</ets>; akin to D. <ets>staal</ets>, G. <ets>stahl</ets>, OHG. <ets>stahal</ets>, Icel. <ets>st<amac/l</ets>, Dan. <ets>staal</ets>, Sw. <ets>st<aring/l</ets>, Old Prussian <ets>stakla</ets>.]</ety> <sn>1.</sn> <fld>(Metal)</fld> <def>A variety of iron intermediate in composition and properties between wrought iron and cast iron (containing between one half of one per cent and one and a half per cent of carbon), and consisting of an alloy of iron with an iron carbide. Steel, unlike wrought iron, can be tempered, and retains magnetism. Its malleability decreases, and fusibility increases, with an increase in carbon.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>An instrument or implement made of steel</def>; as: --<br/
[<source>1913 Webster</source>]</p>

<p><sd>(a)</sd> <def>A weapon, as a sword, dagger, etc.</def> <ldquo/Brave Macbeth . . . with his brandished <xex>steel</xex>.<rdquo/  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>While doubting thus he stood,<br/
Received the <qex>steel</qex> bathed in his brother's blood.</q> <rj><qau>Dryden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sd>(b)</sd> <def>An instrument of steel (usually a round rod) for sharpening knives.</def><br/
[<source>1913 Webster</source>]</p>

<p><sd>(c)</sd> <def>A piece of steel for striking sparks from flint.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>Fig.: Anything of extreme hardness; that which is characterized by sternness or rigor.</def> <ldquo/Heads of <xex>steel</xex>.<rdquo/ <au>Johnson.</au> <ldquo/Manhood's heart of <xex>steel</xex>.<rdquo/ <au>Byron.</au><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <fld>(Med.)</fld> <def>A chalybeate medicine.</def>  <rj><au>Dunglison.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ <xex>Steel</xex> is often used in the formation of compounds, generally of obvious meaning; as, <xex>steel</xex>-clad, <xex>steel</xex>-girt, <xex>steel</xex>-hearted, <xex>steel</xex>-plated, <xex>steel</xex>-pointed, etc.</note><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Bessemer steel</b></col> <fld>(Metal.)</fld> <cd>See in the Vocabulary.</cd> -- <col><b>Blister steel</b></col>. <fld>(Metal.)</fld> <cd>See under <er>Blister</er>.</cd> -- <col><b>Cast steel</b></col> <fld>(Metal.)</fld>, <cd>a fine variety of steel, originally made by smelting blister or cementation steel; hence, ordinarily, steel of any process of production when remelted and cast.</cd> -- <mcol><col><b>Chrome steel</b></col>, <col><b>Chromium steel</b></col></mcol> <fld>(Metal.)</fld>, <cd>a hard, tenacious variety containing a little chromium, and somewhat resembling <cref>tungsten steel</cref>.</cd> -- <col><b>Mild steel</b></col> <fld>(Metal.)</fld>, <cd>a kind of steel having a lower proportion of carbon than ordinary steel, rendering it softer and more malleable.</cd> -- <col><b>Puddled steel</b></col> <fld>(Metal.)</fld>, <cd>a variety of steel produced from cast iron by the puddling process.</cd> -- <col><b>Steel duck</b></col> <fld>(Zool.)</fld>, <cd>the goosander, or merganser.</cd> <mark>[Prov. Eng.]</mark> -- <col><b>Steel mill</b></col>. <sd>(a)</sd> <fld>(Firearms)</fld> <cd>See <cref>Wheel lock</cref>, under <er>Wheel</er>.</cd> <sd>(b)</sd> <cd>A mill which has steel grinding surfaces</cd>. <sd>(c)</sd> <cd>A mill where steel is manufactured.</cd> -- <col><b>Steel trap</b></col>, <cd>a trap for catching wild animals. It consists of two iron jaws, which close by means of a powerful steel spring when the animal disturbs the catch, or tongue, by which they are kept open.</cd> -- <col><b>Steel wine</b></col>, <cd>wine, usually sherry, in which steel filings have been placed for a considerable time, -- used as a medicine.</cd> -- <col><b>Tincture of steel</b></col> <fld>(Med.)</fld>, <cd>an alcoholic solution of the chloride of iron.</cd> -- <col><b>Tungsten steel</b></col> <fld>(Metal.)</fld>, <cd>a variety of steel containing a small amount of tungsten, and noted for its tenacity and hardness, as well as for its malleability and tempering qualities. It is also noted for its magnetic properties.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Steel</ent><br/
<hw>Steel</hw> <pr>(st<emac/l)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Steeled</conjf> <pr>(st<emac/ld)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Steeling</conjf>.]</vmorph> <ety>[AS. <ets>st<ymac/lan</ets>: cf. Icel. <ets>staela</ets>. See <er>Steel</er>, <pos>n.</pos>]</ety> <sn>1.</sn> <def>To overlay, point, or edge with steel; <as>as, to <ex>steel</ex> a razor; to <ex>steel</ex> an ax.</as></def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <mark>Fig.:</mark> <def>To make hard or strong; hence, to make insensible or obdurate.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Lies well <qex>steeled</qex> with weighty arguments.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>O God of battles! <qex>steel</qex> my soldiers' hearts.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Why will you fight against so sweet a passion,<br/
And <qex>steel</qex> your heart to such a world of charms?</q> <rj><qau>Addison.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <mark>Fig.:</mark> <def>To cause to resemble steel, as in smoothness, polish, or other qualities.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>These waters, <qex>steeled</qex><br/
By breezeless air to smoothest polish.</q> <rj><qau>Wordsworth.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <fld>(Elec.)</fld> <def>To cover, as an electrotype plate, with a thin layer of iron by electrolysis. The iron thus deposited is very hard, like steel.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Steelbow goods</ent><br/
<hw>Steel"bow` goods"</hw> <pr>(st<emac/l"bou` g<oocr/dz")</pr>. <fld>(Scots Law)</fld> <def>Those goods on a farm, such as corn, cattle, implements of husbandry, etc., which may not be carried off by a removing tenant, as being the property of the landlord.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Steeler</ent><br/
<hw>Steel"er</hw> <pr>(-<etil/r)</pr>, <pos>n.</pos> <def>One who points, edges, or covers with steel.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Steeler</ent><br/
<hw>Steel"er</hw>, <pos>n.</pos> <fld>(Shipbuilding)</fld> <def>Same as <er>Stealer</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Steelhead</ent><br/
<hw>Steel"head`</hw> <pr>(-h<ecr/d`)</pr>, <pos>n.</pos> <sn>1.</sn> <fld>(Zool.)</fld> <def>A North Pacific salmon (<spn>Salmo Gairdneri</spn>) found from Northern California to Siberia; -- called also <altname>hardhead</altname>, and <altname>preestl</altname>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Zool.)</fld> <def>The ruddy duck.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Steeliness</ent><br/
<hw>Steel"i*ness</hw> <pr>(-<icr/*n<ecr/s)</pr>, <pos>n.</pos> <def>The quality of being steely.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Steeling</ent><br/
<hw>Steel"ing</hw>, <pos>n.</pos> <def>The process of pointing, edging, or overlaying with steel; specifically, acierage. See <er>Steel</er>, <pos>v.</pos></def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Steely</ent><br/
<hw>Steel"y</hw> <pr>(-<ycr/)</pr>, <pos>a.</pos> <sn>1.</sn> <def>Made of steel; consisting of steel.</def> <ldquo/The <xex>steely</xex> point of Clifford's lance.<rdquo/  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Around his shop the <qex>steely</qex> sparkles flew.</q> <rj><qau>Gay.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Resembling steel; hard; firm; having the color of steel.</def> <ldquo/His hair was <xex>steely</xex> gray.<rdquo/  <rj><au>The Century.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>She would unarm her noble heart of that <qex>steely</qex> resistance against the sweet blows of love.</q> <rj><qau>Sir P. Sidney.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Steely iron</b></col>, <cd>a compound of iron containing less than one half of one per cent of carbon.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Steelyard</ent><br/
<hw>Steel"yard</hw> <pr>(st<emac/l"y<aum/rd ; <mark>colloq.</mark> st<icr/l"y<etil/rd ; 277)</pr>, <pos>n.</pos> <ety>[So named from a place in London called the <ets>Steelyard</ets>, which was a yard in which steel was sold.]</ety> <def>A form of balance in which the body to be weighed is suspended from the shorter arm of a lever, which turns on a fulcrum, and a counterpoise is caused to slide upon the longer arm to produce equilibrium, its place upon this arm (which is notched or graduated) indicating the weight; a Roman balance; -- very commonly used also in the plural form, <xex>steelyards</xex>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Steem</ent><br/
<hw>Steem</hw> <pr>(st<emac/m)</pr>, <pos>v. & n.</pos> <def>See <er>Esteem</er>.</def> <mark>[Obs.]</mark>  <rj><au>Spenser.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Steem</ent><br/
<hw>Steem</hw>, <pos>v. & n.</pos> <def>See 1st and 2nd <er>Stem</er>.</def> <mark>[Obs.]</mark>  <rj><au>Chaucer.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Steen</ent><br/
<hw>Steen</hw> <pr>(st<emac/n)</pr>, <pos>n.</pos> <ety>[AS. <ets>st<aemac/na</ets>. See <er>Stone</er>.]</ety> <altsp>[Written also <asp>stean</asp>.]</altsp> <sn>1.</sn> <def>A vessel of clay or stone.</def> <ldquo/An huge great earth-pot <xex>steane</xex>.<rdquo/  <rj><au>Spenser.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A wall of brick, stone, or cement, used as a lining, as of a well, cistern, etc.; a steening.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Steen</ent><br/
<hw>Steen</hw>, <pos>v. t.</pos> <ety>[AS. <ets>st<aemac/nan</ets> to adorn with stones or gems. See <er>Stone</er>.]</ety> <def>To line, as a well, with brick, stone, or other hard material.</def> <altsp>[Written also <asp>stean</asp>, and <asp>stein</asp>.]</altsp><br/
[<source>1913 Webster</source>]</p>

<p><ent>Steenbok</ent><br/
||<hw>Steen"bok`</hw> <pr>(-b<ocr/k`)</pr>, <pos>n.</pos> <ety>[D. <ets>steen</ets> stone + <ets>bok</ets> buck.]</ety> <fld>(Zool.)</fld> <def>Same as <er>Steinbock</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Steening</ent><br/
<hw>Steen"ing</hw>, <pos>n.</pos> <def>A lining made of brick, stone, or other hard material, as for a well.</def> <altsp>[Written also <asp>steaning</asp>.]</altsp><br/
[<source>1913 Webster</source>]</p>

<p><ent>Steinkirk</ent><br/
<ent>Steenkirk</ent><br/
<mhw>{ <hw>Steen"kirk`</hw> <pr>(st<emac/n"k<etil/rk`)</pr>, <hw>Stein"kirk`</hw> <pr>(st<imac/n"-)</pr>, }</mhw> <pos>n.</pos> <ety>[So called from the battle of <ets>Steinkirk</ets>, in 1692, on which occasion the French nobles had no time to arrange their lace neckcloths.]</ety> <def>A kind of neckcloth worn in a loose and disorderly fashion.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Steep</ent><br/
<hw>Steep</hw> <pr>(st<emac/p)</pr>, <pos>a.</pos> <def>Bright; glittering; fiery.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>His eyen <qex>steep</qex>, and rolling in his head.</q> <rj><qau>Chaucer.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Steep</ent><br/
<hw>Steep</hw>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Steeped</conjf> <pr>(st<emac/pt)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Steeping</conjf>.]</vmorph> <ety>[OE. <ets>stepen</ets>, probably fr. Icel. <ets>steypa</ets> to cause to stoop, cast down, pour out, to cast metals, causative of <ets>st<umac/pa</ets> to stoop; cf. Sw. <ets>st<oum/pa</ets> to cast, to steep, Dan. <ets>st<oum/be</ets>, D. & G. <ets>stippen</ets> to steep, to dip.  Cf. <er>Stoop</er>, <pos>v. i.</pos>]</ety> <def>To soak in a liquid; to macerate; to extract the essence of by soaking; <as>as, to soften seed by <ex>steeping</ex> it in water</as>. Often used figuratively.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Let fancy still my sense in Lethe <qex>steep</qex>.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>In refreshing dew to <qex>steep</qex><br/
The little, trembling flowers.</q> <rj><qau>Wordsworth.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>The learned of the nation were <qex>steeped</qex> in Latin.</q> <rj><qau>Earle.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Steep</ent><br/
<hw>Steep</hw>, <pos>v. i.</pos> <def>To undergo the process of soaking in a liquid; <as>as, the tea is <ex>steeping</ex></as>.</def> <mark>[Colloq.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Steep</ent><br/
<hw>Steep</hw>, <pos>n.</pos> <sn>1.</sn> <def>Something steeped, or used in steeping; a fertilizing liquid to hasten the germination of seeds.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A rennet bag.</def> <mark>[Prov. Eng.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Steep</ent><br/
<hw>Steep</hw>, <pos>a.</pos> <amorph>[<pos>Compar.</pos> <adjf>Steeper</adjf> <pr>(-<etil/r)</pr>; <pos>superl.</pos> <adjf>Steepest</adjf>.]</amorph> <ety>[OE. <ets>steep</ets>, <ets>step</ets>, AS. <ets>ste<aacute/p</ets>; akin to Icel. <ets>steyp<edh/r</ets> steep, and <ets>st<umac/pa</ets> to stoop, Sw. <ets>stupa</ets> to fall, to tilt; cf. OFries. <ets>stap</ets> high.  Cf. <er>Stoop</er>, <pos>v. i.</pos>, <er>Steep</er>, <pos>v. t.</pos>, <er>Steeple</er>.]</ety> <sn>1.</sn> <def>Making a large angle with the plane of the horizon; ascending or descending rapidly with respect to a horizontal line or a level; precipitous; <as>as, a <ex>steep</ex> hill or mountain; a <ex>steep</ex> roof; a <ex>steep</ex> ascent; a <ex>steep</ex> declivity; a <ex>steep</ex> barometric gradient</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Difficult of access; not easily reached; lofty; elevated; high.</def> <mark>[Obs.]</mark>  <rj><au>Chapman.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>Excessive; <as>as, a <ex>steep</ex> price</as>.</def> <mark>[Slang]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Steep</ent><br/
<hw>Steep</hw>, <pos>n.</pos> <def>A precipitous place, hill, mountain, rock, or ascent; any elevated object sloping with a large angle to the plane of the horizon; a precipice.</def>  <rj><au>Dryden.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>We had on each side naked rocks and mountains broken into a thousand irregular <qex>steeps</qex> and precipices.</q> <rj><qau>Addison.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Bare <qex>steeps</qex>, where desolation stalks.</q> <rj><qau>Wordsworth.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Steep-down</ent><br/
<hw>Steep"-down`</hw> <pr>(st<emac/p"doun`)</pr>, <pos>a.</pos> <def>Deep and precipitous; having steep descent.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>Wash me in <qex>steep-down</qex> gulfs of liquid fire.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Steepen</ent><br/
<hw>Steep"en</hw> <pr>(st<emac/p"'n)</pr>, <pos>v. i.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Steepened</conjf> <pr>(st<emac/p"'nd)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Steepening</conjf>.]</vmorph> <def>To become steep or steeper.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>As the way <qex>steepened</qex> . . . I could detect in the hollow of the hill some traces of the old path.</q> <rj><qau>H. Miller.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Steeper</ent><br/
<hw>Steep"er</hw> <pr>(-<etil/r)</pr>, <pos>n.</pos> <def>A vessel, vat, or cistern, in which things are steeped.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Steepiness</ent><br/
<hw>Steep"i*ness</hw> <pr>(-<icr/*n<ecr/s)</pr>, <pos>n.</pos> <def>Steepness.</def>  <rj><au>Howell.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Steepish</ent><br/
<hw>Steep"ish</hw>, <pos>a.</pos> <def>Somewhat steep.</def>  <rj><au>Carlyle.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Steeple</ent><br/
<hw>Stee"ple</hw> <pr>(st<emac/"p'l)</pr>, <pos>n.</pos> <ety>[OE. <ets>stepel</ets>, AS. <ets>st<emac/pel</ets>, <ets>st<ymac/pel</ets>; akin to E. <ets>steep</ets>, a.]</ety> <fld>(Arch.)</fld> <def>A spire; also, the tower and spire taken together; the whole of a structure if the roof is of spire form. See <er>Spire</er>.</def> <ldquo/A weathercock on a <xex>steeple</xex>.<rdquo/  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Rood steeple</b></col>. <cd>See <cref>Rood tower</cref>, under <er>Rood</er>.</cd> -- <col><b>Steeple bush</b></col> <fld>(Bot.)</fld>, <cd>a low shrub (<spn>Spiraea tomentosa</spn>) having dense panicles of minute rose-colored flowers; hardhack.</cd> -- <col><b>Steeple chase</b></col>, <cd>a race across country between a number of horsemen, to see which can first reach some distant object, as a church steeple; hence, a race over a prescribed course obstructed by such obstacles as one meets in riding across country, as hedges, walls, etc.</cd> -- <col><b>Steeple chaser</b></col>, <cd>one who rides in a steeple chase; also, a horse trained to run in a steeple chase.</cd> -- <col><b>Steeple engine</b></col>, <cd>a vertical back-acting steam engine having the cylinder beneath the crosshead.</cd> -- <col><b>Steeple house</b></col>, <cd>a church.</cd> <mark>[Obs.]</mark> <au>Jer. Taylor.</au></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Steeplechasing</ent><br/
<hw>Stee"ple*chas`ing</hw> <pr>(-ch<amac/s`<icr/ng)</pr>, <pos>n.</pos> <def>The act of riding steeple chases.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Steeple-crowned</ent><br/
<hw>Stee"ple-crowned`</hw> <pr>(-kround`)</pr>, <pos>a.</pos> <sn>1.</sn> <def>Bearing a steeple; <as>as, a <ex>steeple-crowned</ex> building</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Having a crown shaped like a steeple; <as>as, a <ex>steeple-crowned</ex> hat</as>; also, wearing a hat with such a crown.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>This grave, bearded, sable-cloaked, and <qex>steeple-crowned</qex> progenitor.</q> <rj><qau>Hawthorne.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Steepled</ent><br/
<hw>Stee"pled</hw> <pr>(st<emac/"p'ld)</pr>, <pos>a.</pos> <def>Furnished with, or having the form of, a steeple; adorned with steeples.</def>  <rj><au>Fairfax.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Steeply</ent><br/
<hw>Steep"ly</hw> <pr>(st<emac/p"l<ycr/)</pr>, <pos>adv.</pos> <def>In a steep manner; with steepness; with precipitous declivity.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Steepness</ent><br/
<hw>Steep"ness</hw>, <pos>n.</pos> <sn>1.</sn> <def>Quality or state of being steep; precipitous declivity; <as>as, the <ex>steepness</ex>of a hill or a roof</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Height; loftiness.</def> <mark>[Obs.]</mark>  <rj><au>Chapman.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Steep-up</ent><br/
<hw>Steep"-up`</hw> <pr>(-<ucr/p`)</pr>, <pos>a.</pos> <def>Lofty and precipitous.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>Her stand she takes upon a <qex>steep-up</qex> hill.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Steepy</ent><br/
<hw>Steep"y</hw> <pr>(-<ycr/)</pr>, <pos>a.</pos> <def>Steep; precipitous.</def> <mark>[Poetic]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>No more, my goats, shall I behold you climb<br/
The <qex>steepy</qex> cliffs, or crop the flow'ry thyme.</q> <rj><qau>Dryden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Steer</ent><br/
<hw>Steer</hw> <pr>(st<emac/r)</pr>, <pos>n.</pos> <ety>[OE. <ets>steer</ets>, AS. <ets>ste<oacute/r</ets>; akin to D. & G. <ets>stier</ets> a bull, OHG. <ets>stior</ets>, Icel. <ets>stj<omac/rr</ets>, <ets><thorn/j<omac/rr</ets>, Sw. <ets>tjur</ets>, Dan. <ets>tyr</ets>, Goth. <ets>stiur</ets>, Russ. <ets>tur'</ets>, Pol. <ets>tur</ets>, Ir. & Gael. <ets>tarbh</ets>, W. <ets>tarw</ets>, L. <ets>taurus</ets>, Gr. <grk>tay^ros</grk>, Skr. <ets>sth<umac/ra</ets> strong, stout, AS. <ets>stor</ets> large, Icel. <ets>st<omac/rr</ets>, OHG. <ets>st<umac/ri</ets>, <ets>stiuri</ets>.  <root/168.  Cf. <er>Stirk</er>, <er>Taurine</er>, <pos>a.</pos>]</ety> <def>A young male of the ox kind; especially, a common ox; a castrated taurine male from two to four years old. See the Note under <er>Ox</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Steer</ent><br/
<hw>Steer</hw>, <pos>v. t.</pos> <def>To castrate; -- said of male calves.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Steer</ent><br/
<hw>Steer</hw>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Steered</conjf> <pr>(st<emac/rd)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Steering</conjf>.]</vmorph> <ety>[OE. <ets>steeren</ets>, <ets>steren</ets>, AS. <ets>sti<eacute/ran</ets>, <ets>st<ymac/ran</ets>, <ets>ste<oacute/ran</ets>; akin to OFries. <ets>stiora</ets>, <ets>stiura</ets>, D. <ets>sturen</ets>, OD. <ets>stieren</ets>, G. <ets>steuern</ets>, OHG. <ets>stiuren</ets> to direct, support, G. <ets>steuer</ets> contribution, tax, Icel. <ets>st<ymac/ra</ets> to steer, govern, Sw. <ets>styra</ets>, Dan. <ets>styre</ets>, Goth. <ets>stiurjan</ets> to establish, AS. <ets>ste<oacute/r</ets> a rudder, a helm, and probably to Icel. <ets>staurr</ets> a pale, stake, Gr. <grk>stayro`s</grk>, and perhaps ultimately to E. <ets>stand</ets>.  <root/168.  Cf. <er>Starboard</er>, <er>Stern</er>, <pos>n.</pos>]</ety> <def>To direct the course of; to guide; to govern; -- applied especially to a vessel in the water.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>That with a staff his feeble steps did <qex>steer</qex>.</q> <rj><qau>Spenser.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Steer</ent><br/
<hw>Steer</hw>, <pos>v. i.</pos> <sn>1.</sn> <def>To direct a vessel in its course; to direct one's course.</def> <ldquo/No helmsman <xex>steers</xex>.<rdquo/  <rj><au>Tennyson.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To be directed and governed; to take a direction, or course; to obey the helm; <as>as, the boat <ex>steers</ex> easily</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Where the wind<br/
Veers oft, as oft [a ship] so <qex>steers</qex>, and shifts her sail.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>To conduct one's self; to take or pursue a course of action.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Steer</ent><br/
<hw>Steer</hw>, <pos>n.</pos> <ety>[AS. <ets>ste<oacute/r</ets>, <ets>sti<oacute/r</ets>; akin to D. <ets>stuur</ets>, G. <ets>steuer</ets>, Icel. <ets>st<ymac/ri</ets>.  <root/168.  See <er>Steer</er>, <pos>v. t.</pos>]</ety> <altsp>[Written also <asp>stere</asp>.]</altsp> <def>A rudder or helm.</def> <mark>[Obs.]</mark>  <rj><au>Chaucer.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Steer</ent><br/
<hw>Steer</hw>, <pos>n.</pos> <ety>[AS. <ets>ste<oacute/ra</ets>. See <er>Steer</er> a rudder.]</ety> <def>A helmsman; a pilot.</def> <mark>[Obs.]</mark>  <rj><au>Chaucer.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Steerable</ent><br/
<hw>Steer"a*ble</hw> <pr>(-<adot/*b'l)</pr>, <pos>a.</pos> <def>Capable of being steered; dirigible.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Steerage</ent><br/
<hw>Steer"age</hw> <pr>(st<emac/r"<asl/j ; 48)</pr>, <pos>n.</pos> <sn>1.</sn> <def>The act or practice of steering, or directing; <as>as, the <ex>steerage</ex> of a ship</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>He left the city, and, in a most tempestuous season, forsook the helm and <qex>steerage</qex> of the commonwealth.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Naut.)</fld> <sd>(a)</sd> <def>The effect of the helm on a ship; the manner in which an individual ship is affected by the helm.</def> <sd>(b)</sd> <def>The hinder part of a vessel; the stern.</def> <mark>[R.]</mark> <au>Swift.</au> <sd>(c)</sd> <def>Properly, the space in the after part of a vessel, under the cabin, but used generally to indicate any part of a vessel having the poorest accommodations and occupied by passengers paying the lowest rate of fare.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>Direction; regulation; management; guidance.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>He that hath the <qex>steerage</qex> of my course.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>That by which a course is directed.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>Here he hung on high,<br/
The <qex>steerage</qex> of his wings.</q> <rj><qau>Dryden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Steerage passenger</b></col>, <cd>a passenger who takes passage in the steerage of a vessel.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Steerageway</ent><br/
<hw>Steer"age*way`</hw> <pr>(-w<amac/`)</pr>, <pos>n.</pos> <fld>(Naut.)</fld> <def>A rate of motion through the water sufficient to render a vessel governable by the helm.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Steerer</ent><br/
<hw>Steer"er</hw> <pr>(-<etil/r)</pr>, <pos>n.</pos> <def>One who steers; <as>as, a boat <ex>steerer</ex></as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Steering</ent><br/
<hw>Steer"ing</hw>, <def><pos>a. & n.</pos> from <er>Steer</er>, <pos>v.</pos></def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Steering wheel</b></col> <fld>(Naut.)</fld>, <cd>the wheel by means of which the rudder of a vessel is turned and the vessel is steered.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Steerless</ent><br/
<hw>Steer"less</hw>, <pos>a.</pos> <def>Having no rudder.</def> <mark>[Obs.]</mark>  <rj><au>Chaucer.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Steerling</ent><br/
<hw>Steer"ling</hw> <pr>(-l<icr/ng)</pr>, <pos>n.</pos> <def>A young or small steer.</def><br/
[<source>1913 Webster</source>]</p>

<p><-- p. 1410 --></p>

<p><ent>Steersman</ent><br/
<hw>Steers"man</hw> <pr>(st<emac/rz"m<ait/n)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Steersmen</plw> <pr>(st<emac/rz"m<eit/n)</pr>.</plu> <ety>[<ets>Steer</ets> a rudder + <ets>man</ets>: cf. AS. <ets>ste<oacute/rmann</ets>.]</ety> <def>One who steers; the helmsman of a vessel.</def>  <rj><au>Milton.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Steersmate</ent><br/
<hw>Steers"mate</hw> <pr>(st<emac/rz"m<amac/t`)</pr>, <pos>n.</pos> <ety>[<ets>Steer</ets> a rudder + <ets>mate</ets> a companion.]</ety> <def>One who steers; steersman.</def> <mark>[Obs.]</mark>  <rj><au>Milton.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Steeve</ent><br/
<hw>Steeve</hw> <pr>(st<emac/v)</pr>, <pos>v. i.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Steeved</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Steeving</conjf>.]</vmorph> <ety>[Cf. OD. <ets>steve</ets> staff, E. <ets>stem</ets>, n.]</ety> <fld>(Shipbuilding)</fld> <def>To project upward, or make an angle with the horizon or with the line of a vessel's keel; -- said of the bowsprit, etc.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Steeve</ent><br/
<hw>Steeve</hw>, <pos>v. t.</pos> <sn>1.</sn> <fld>(Shipbuilding)</fld> <def>To elevate or fix at an angle with the horizon; -- said of the bowsprit, etc.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To stow, as bales in a vessel's hold, by means of a steeve. See <er>Steeve</er>, <pos>n.</pos> <sd>(b)</sd>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Steeve</ent><br/
<hw>Steeve</hw>, <pos>n.</pos> <fld>(Naut.)</fld> <sd>(a)</sd> <def>The angle which a bowsprit makes with the horizon, or with the line of the vessel's keel; -- called also <altname>steeving</altname>.</def> <sd>(b)</sd> <def>A spar, with a block at one end, used in stowing cotton bales, and similar kinds of cargo which need to be packed tightly.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Steeving</ent><br/
<hw>Steev"ing</hw>, <pos>n.</pos> <sn>1.</sn> <def>The act or practice of one who steeves.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Naut.)</fld> <def>See <er>Steeve</er>, <pos>n.</pos> <sd>(a)</sd>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Steg</ent><br/
<hw>Steg</hw> <pr>(st<ecr/g)</pr>, <pos>n.</pos> <ety>[Icel. <ets>steggr</ets> the male of several animals.  Cf. <er>Stag</er>.]</ety> <fld>(Zool.)</fld> <def>A gander.</def> <altsp>[Written also <asp>stag</asp>.]</altsp> <mark>[Prov. Eng.]</mark>  <rj><au>Halliwell.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Steganographist</ent><br/
<hw>Steg`a*nog"ra*phist</hw> <pr>(st<ecr/g`<adot/*n<ocr/g"r<adot/*f<icr/st)</pr>, <pos>n.</pos> <def>One skilled in steganography; a cryptographer.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Steganography</ent><br/
<hw>Steg`a*nog"ra*phy</hw> <pr>(st<ecr/g`<adot/*n<ocr/g"r<adot/*f<ycr/)</pr>, <pos>n.</pos> <ety>[Gr. <grk>stegano`s</grk> covered (fr. <grk>ste`gein</grk> to cover closely) + <ets>-graphy</ets>.]</ety> <sn>1.</sn> <def>The art of writing in cipher, or in characters which are not intelligible except to persons who have the key; cryptography.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <specif>Specifically:</specif> <def>The branch of cryptography in which messages are hidden inside other messages; -- used commonly for the process of hiding messages inside a computerized image file, as for example hiding the name and copyright notice of the owner of an image as protection against violation of the copyright.</def><br/
[<source>PJC</source>]</p>

<p><ent>Steganophthalmata</ent><br/
||<hw>Steg`a*noph*thal"ma*ta</hw> <pr>(?)</pr>, <pos>n. pl.</pos> <ety>[NL., from Gr. <grk>stegano`s</grk> covered + <grk>'ofqalmo`s</grk> the eye.]</ety> <fld>(Zool.)</fld> <def>The Discophora, or Phanerocarpae. Called also <altname>Steganophthalmia</altname>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Steganopod</ent><br/
<hw>Ste*gan"o*pod</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Zool.)</fld> <def>One of the Steganopodes.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Steganopodes</ent><br/
||<hw>Steg`a*nop"o*des</hw> <pr>(st<ecr/g`<adot/*n<ocr/p"<osl/*d<emac/z)</pr>, <pos>n. pl.</pos> <ety>[NL., fr. Gr. <grk>stegano`poys</grk>, <grk>stegano`podos</grk>, web-footed; <grk>stegano`s</grk> covered + <grk>poy`s</grk> foot.]</ety> <fld>(Zool.)</fld> <def>A division of swimming birds in which all four toes are united by a broad web. It includes the pelicans, cormorants, gannets, and others.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Steganopodous</ent><br/
<hw>Steg`a*nop"o*dous</hw> <pr>(?)</pr>, <pos>a.</pos> <fld>(Zool.)</fld> <def>Having all four toes webbed together.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stegnosis</ent><br/
||<hw>Steg*no"sis</hw> <pr>(st<ecr/g*n<omac/"s<icr/s)</pr>, <pos>n.</pos> <ety>[NL., fr. Gr. <grk>ste`gnwsis</grk>. See <er>Stegnotic</er>.]</ety> <fld>(Med.)</fld> <def>Constipation; also, constriction of the vessels or ducts.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stegnotic</ent><br/
<hw>Steg*no"tic</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[Gr. <grk>stegnwtiko`s</grk>, fr. <grk>stegnoy^n</grk> to cover, to make costive, fr. <grk>stegno`s</grk>, <grk>stegano`s</grk>, covered, closed.]</ety> <fld>(Med.)</fld> <def>Tending to render costive, or to diminish excretions or discharges generally.</def> -- <def2><pos>n.</pos> <def>A stegnotic medicine; an astringent.</def></def2><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stegocephala</ent><br/
||<hw>Steg`o*ceph"a*la</hw> <pr>(st<ecr/g`<osl/*s<ecr/f"<adot/*l<adot/)</pr>, <pos>n. pl.</pos> <ety>[NL., fr. Gr. <grk>ste`gh</grk> roof + <grk>kefalh`</grk> head.]</ety> <fld>(Paleon.)</fld> <def>An extinct order of amphibians found fossil in the Mesozoic rocks; called also <altname>Stegocephali</altname>, and <altname>Labyrinthodonta</altname>.</def><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ Their teeth, in transverse sections, usually show a labyrinthiform arrangement of the cement and dentine. The under side of the body was covered with bony plates. Some of the Stegocephala were of very large size, and the form of the body varied from short, stout forms to others that were as slender as serpents.</note><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stegosauria</ent><br/
||<hw>Steg`o*sau"ri*a</hw> <pr>(st<ecr/g`<osl/*s<add/"r<icr/*<adot/)</pr>, <pos>n. pl.</pos> <ety>[NL. See <er>Stegosaurus</er>.]</ety> <fld>(Paleon.)</fld> <def>An extinct order of herbivorous dinosaurs, including the genera <styp><gen>Stegosaurus</gen></styp>, <styp><gen>Omosaurus</gen></styp>, and their allies.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stegosaurus</ent><br/
||<hw>Steg`o*sau"rus</hw> <pr>(st<ecr/g`<osl/*s<add/"r<ucr/s)</pr>, <pos>n.</pos> <ety>[NL., fr. Gr. <grk>ste`gh</grk> roof + <grk>say^ros</grk> a lizard.]</ety> <fld>(Paleon.)</fld> <def>A genus of large Jurassic dinosaurs remarkable for a powerful dermal armature of plates and spines.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Steik</ent><br/
<hw>Steik</hw> <pr>(?)</pr>, <pos>v. t.</pos> <def>See <er>Steek</er>.</def> <mark>[Scot.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stein</ent><br/
<hw>Stein</hw> <pr>(?)</pr>, <pos>n. & v.</pos> <def>See <er>Steen</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Steinbock</ent><br/
<hw>Stein"bock`</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[G. <ets>stein</ets> stone + <ets>bock</ets> buck, D. <ets>bok</ets>.  Cf. <er>Steenbok</er>.]</ety> <fld>(Zool.)</fld> <sd>(a)</sd> <def>The European ibex.</def> <sd>(b)</sd> <def>A small South African antelope (<spn>Nanotragus tragulus</spn>) which frequents dry, rocky districts; -- called also <altname>steenbok</altname>.</def> <altsp>[Written also <asp>steinboc</asp>, and <asp>steinbok</asp>; also called <asp>stonebock</asp>, and <asp>stonebuck</asp>.]</altsp><br/
[<source>1913 Webster</source>]</p>

<p><ent>Steingale</ent><br/
<hw>Stein"gale</hw> <pr>(?)</pr>, <pos>n.</pos> <def>The stannel.</def> <mark>[Prov. Eng.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Steining</ent><br/
<hw>Stein"ing</hw> <pr>(?)</pr>, <pos>n.</pos> <def>See <er>Steening</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Steinkirk</ent><br/
<hw>Stein"kirk`</hw> <pr>(?)</pr>, <pos>n.</pos> <def>Same as <er>Steenkirk</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Steinkle</ent><br/
<hw>Stein"kle</hw> <pr>(?)</pr>, <pos>n.</pos> <def>The wheater.</def> <mark>[Prov. Eng.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stela</ent><br/
||<hw>Ste"la</hw> <pr>(?)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Stelae</plw> <pr>(#)</pr>.</plu> <ety>[L., from Gr. <?/ a post, an upright stone.]</ety> <fld>(Gr. Antiq.)</fld> <def>A small column or pillar, used as a monument, milestone, etc.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stele</ent><br/
||<hw>Ste"le</hw> <pr>(st<emac/"l<emac/)</pr>, <pos>n.</pos> <ety>[NL.]</ety> <def>Same as <er>Stela</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>One of these <qex>steles</qex>, containing the Greek version of the ordinance, has recently been discovered.</q> <rj><qau>I. Taylor (The Alphabet).</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stele</ent><br/
<hw>Stele</hw> <pr>(st<emac/l)</pr>, <pos>n.</pos> <ety>[See <er>Stale</er> a handle.]</ety> <def>A stale, or handle; a stalk.</def> <mark>[Obs.]</mark>  <rj><au>Chaucer.</au>  <au>Holland.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stelene</ent><br/
<hw>Ste"lene</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[See <er>Stela</er>.]</ety> <def>Resembling, or used as, a stela; columnar.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stell</ent><br/
<hw>Stell</hw> <pr>(?)</pr>, <pos>v. t.</pos> <ety>[AS. <ets>stellan</ets>. <root/163.]</ety> <def>To place or fix firmly or permanently.</def> <mark>[Obs.]</mark>  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stell</ent><br/
<hw>Stell</hw>, <pos>n.</pos> <ety>[See <er>Stell</er>, <pos>v. t.</pos>]</ety> <sn>1.</sn> <def>A prop; a support, as for the feet in standing or cilmbing.</def> <mark>[Scot.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A partial inclosure made by a wall or trees, to serve as a shelter for sheep or cattle.</def> <mark>[Prov. Eng. & Scot.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stellary</ent><br/
<ent>Stellar</ent><br/
<mhw>{ <hw>Stel"lar</hw> <pr>(?)</pr>, <hw>Stel"la*ry</hw> <pr>(?)</pr>, }</mhw> <pos>a.</pos> <ety>[L. <ets>stellaris</ets>, fr. <ets>stella</ets> a star. See <er>Star</er>.]</ety> <sn>1.</sn> <def>Of or pertaining to stars; astral; <as>as, a <ex>stellar</ex> figure; <ex>stellary</ex> orbs</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>[These soft fires] in part shed down<br/
Their <qex>stellar</qex> virtue.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Full of stars; starry; <as>as, <ex>stellar</ex> regions</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stellated</ent><br/
<ent>Stellate</ent><br/
<mhw>{ <hw>Stel"late</hw> <pr>(?)</pr>, <hw>Stel"la*ted</hw> <pr>(?)</pr>, }</mhw> <pos>a.</pos> <ety>[L. <ets>stellatus</ets>, p. p. of <ets>stellare</ets> to set or cover with stars, from <ets>stella</ets> a star. See <er>Stellar</er>.]</ety> <sn>1.</sn> <def>Resembling a star; pointed or radiated, like the emblem of a star.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Bot.)</fld> <def>Starlike; having similar parts radiating from a common center; <as>as, <ex>stellate</ex> flowers</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stellation</ent><br/
<hw>Stel*la"tion</hw> <pr>(?)</pr>, <pos>n.</pos> <def>Radiation of light.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stelled</ent><br/
<hw>Stelled</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[See <er>Stell</er> to place.]</ety> <def>Firmly placed or fixed.</def> <mark>[Obs.]</mark> <ldquo/The <xex>stelled</xex> fires<rdquo/ [the stars]. <au>Shak.</au> <note>[In this passage by some defined as <ldquo/starry,<rdquo/ as if from <xex>stellatus</xex>.]</note><br/
[<source>1913 Webster</source>]</p>

<p><ent>steller</ent><br/
<hw>stel"ler</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[After <person>Geo. W. <etsep>Steller</etsep></person>, a German naturalist.]</ety> <fld>(Zool.)</fld> <def>The rytina; -- called also <altname>stellerine</altname> and <altname>Steller's sea cow</altname>.  See <er>rytina</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stellerid</ent><br/
<hw>Stel"ler*id</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>stella</ets> a star.]</ety> <fld>(Zool.)</fld> <def>A starfish.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stellerida</ent><br/
||<hw>Stel*ler"i*da</hw> <pr>(?)</pr>, <pos>n. pl.</pos> <ety>[NL.]</ety> <fld>(Zool.)</fld> <def>An extensive group of echinoderms, comprising the starfishes and ophiurans.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stelleridean</ent><br/
<ent>Stelleridan</ent><br/
<mhw>{ <hw>Stel*ler"i*dan</hw> <pr>(?)</pr>, <hw>Stel`ler*id"e*an</hw> <pr>(?)</pr>, }</mhw> <pos>n.</pos> <fld>(Zool.)</fld> <def>A starfish, or brittle star.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Steller's sea cow</ent><br/
<hw>Stel"ler's sea" cow`</hw> <pr>(?)</pr>, <pos>prop. n.</pos> <ety>[After <person>Geo. W. <etsep>Steller</etsep></person>, a German naturalist.]</ety> <fld>(Zool.)</fld> <def>The rytina; -- called also <altname>stellerine</altname> and <altname>steller</altname>.  See <er>rytina</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stelliferous</ent><br/
<hw>Stel*lif"er*ous</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>stellifer</ets>; <ets>stella</ets> star + <ets>ferre</ets> to bear.]</ety> <def>Having, or abounding with, stars.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stelliform</ent><br/
<hw>Stel"li*form</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>stella</ets> a star + <ets>-form</ets>.]</ety> <def>Like a star; star-shaped; radiated.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stellify</ent><br/
<hw>Stel"li*fy</hw> <pr>(?)</pr>, <pos>v. t.</pos> <ety>[L. <ets>stella</ets> a star + <ets>-fy</ets>.]</ety> <def>To turn into a star; to cause to appear like a star; to place among the stars, or in heaven.</def> <mark>[Obs. or R.]</mark>  <rj><au>B. Jonson.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stellion</ent><br/
<hw>Stel"lion</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>stellio</ets> a newt having starlike spots on its back, fr. <ets>stella</ets> a star.]</ety> <fld>(Zool.)</fld> <def>A lizard (<spn>Stellio vulgaris</spn>), common about the Eastern Mediterranean among ruins. In color it is olive-green, shaded with black, with small stellate spots. Called also <altname>hardim</altname>, and <altname>star lizard</altname>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stellionate</ent><br/
<hw>Stel"lion*ate</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>stellionatus</ets> cozenage, trickery, fr. <ets>stellio</ets> a newt, a crafty, knavish person.]</ety> <fld>(Scots & Roman Law)</fld> <def>Any fraud not distinguished by a more special name; -- chiefly applied to sales of the same property to two different persons, or selling that for one's own which belongs to another, etc.</def>  <rj><au>Erskine.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stellular</ent><br/
<hw>Stel"lu*lar</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>stellula</ets>, dim. of <ets>stella</ets> a star.]</ety> <sn>1.</sn> <def>Having the shape or appearance of little stars; radiated.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Marked with starlike spots of color.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stellulate</ent><br/
<hw>Stel"lu*late</hw> <pr>(?)</pr>, <pos>a.</pos> <fld>(Bot.)</fld> <def>Minutely stellate.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stelmatopoda</ent><br/
||<hw>Stel`ma*top"o*da</hw> <pr>(?)</pr>, <pos>n. pl.</pos> <ety>[NL., fr. Gr. <?/ a block, post + <?/, <?/, eye + <?/, <?/, foot.]</ety> <fld>(Zool.)</fld> <def>Same as <er>Gymnolaemata</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stelography</ent><br/
<hw>Ste*log"ra*phy</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Gr. <?/ a post, slab, pillar + <ets>-graphy</ets>: cf. Gr. <?/ an inscription on a tablet.]</ety> <def>The art of writing or inscribing characters on pillars.</def> <mark>[R.]</mark>  <rj><au>Stackhouse.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Steem</ent><br/
<ent>Stem</ent><br/
<mhw>{ <hw>Stem</hw> <pr>(?)</pr>, <hw>Steem</hw> <pr>(?)</pr>, }</mhw> <pos>v. i.</pos> <def>To gleam.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>His head bald, that shone as any glass, . . . <br/
[And] <qex>stemed</qex> as a furnace of a leed [caldron].</q> <rj><qau>Chaucer.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Steem</ent><br/
<ent>Stem</ent><br/
<mhw>{ <hw>Stem</hw>, <hw>Steem</hw>  }</mhw>, <pos>n.</pos> <def>A gleam of light; flame.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stem</ent><br/
<hw>Stem</hw> <pr>(st<ecr/m)</pr>, <pos>n.</pos> <ety>[AS. <ets>stemn</ets>, <ets>stefn</ets>, <ets>staefn</ets>; akin to OS. <ets>stamn</ets> the stem of a ship, D. <ets>stam</ets> stem, <ets>steven</ets> stem of a ship, G. <ets>stamm</ets> stem, <ets>steven</ets> stem of a ship, Icel. <ets>stafn</ets>, <ets>stamn</ets>, stem of a ship, <ets>stofn</ets>, <ets>stomn</ets>, stem, Sw. <ets>stam</ets> a tree trunk, Dan. <ets>stamme</ets>.  Cf. <er>Staff</er>, <er>Stand</er>.]</ety> <sn>1.</sn> <def>The principal body of a tree, shrub, or plant, of any kind; the main stock; the part which supports the branches or the head or top.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>After they are shot up thirty feet in length, they spread a very large top, having no bough nor twig in the trunk or the <qex>stem</qex>.</q> <rj><qau>Sir W. Raleigh.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>The lowering spring, with lavish rain,<br/
Beats down the slender <qex>stem</qex> and breaded grain.</q> <rj><qau>Dryden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A little branch which connects a fruit, flower, or leaf with a main branch; a peduncle, pedicel, or petiole; <as>as, the <ex>stem</ex> of an apple or a cherry</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>The stock of a family; a race or generation of progenitors.</def> <ldquo/All that are of noble <xex>stem</xex>.<rdquo/  <rj><au>Milton.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>While I do pray, learn here thy <qex>stem</qex><br/
And true descent.</q> <rj><qau>Herbert.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>A branch of a family.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>This is a <qex>stem</qex><br/
Of that victorious stock.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <fld>(Naut.)</fld> <def>A curved piece of timber to which the two sides of a ship are united at the fore end. The lower end of it is scarfed to the keel, and the bowsprit rests upon its upper end. Hence, the forward part of a vessel; the bow.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>6.</sn> <def>Fig.: An advanced or leading position; the lookout.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Wolsey sat at the <qex>stem</qex> more than twenty years.</q> <rj><qau>Fuller.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>7.</sn> <def>Anything resembling a stem or stalk; <as>as, the <ex>stem</ex> of a tobacco pipe; the <ex>stem</ex> of a watch case, or that part to which the ring, by which it is suspended, is attached.</as></def><br/
[<source>1913 Webster</source>]</p>

<p><sn>8.</sn> <fld>(Bot.)</fld> <def>That part of a plant which bears leaves, or rudiments of leaves, whether rising above ground or wholly subterranean.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>9.</sn> <fld>(Zool.)</fld> <sd>(a)</sd> <def>The entire central axis of a feather.</def> <sd>(b)</sd> <def>The basal portion of the body of one of the Pennatulacea, or of a gorgonian.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>10.</sn> <fld>(Mus.)</fld> <def>The short perpendicular line added to the body of a note; the tail of a crotchet, quaver, semiquaver, etc.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>11.</sn> <fld>(Gram.)</fld> <def>The part of an inflected word which remains unchanged (except by euphonic variations) throughout a given inflection; theme; base.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>From stem to stern</b></col> <fld>(Naut.)</fld>, <cd>from one end of the ship to the other, or through the whole length.</cd> -- <col><b>Stem leaf</b></col> <fld>(Bot.)</fld>, <cd>a leaf growing from the stem of a plant, as contrasted with a basal or radical leaf.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stem</ent><br/
<hw>Stem</hw>, <pos>v. t.</pos> <sn>1.</sn> <def>To remove the stem or stems from; <as>as, to <ex>stem</ex> cherries</as>; to remove the stem and its appendages (ribs and veins) from; <as>as, to <ex>stem</ex> tobacco leaves</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To ram, as clay, into a blasting hole.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stem</ent><br/
<hw>Stem</hw>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Stemmed</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Stemming</conjf>.]</vmorph> <ety>[Either from <ets>stem</ets>, n., or akin to <ets>stammer</ets>; cf. G. <ets>stemmen</ets> to press against.]</ety> <def>To oppose or cut with, or as with, the stem of a vessel; to resist, or make progress against; to stop or check the flow of, as a current.</def> <ldquo/An argosy to <xex>stem</xex> the waves.<rdquo/  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>[They] <qex>stem</qex> the flood with their erected breasts.</q> <rj><qau>Denham.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q><qex>Stemmed</qex> the wild torrent of a barbarous age.</q> <rj><qau>Pope.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stem</ent><br/
<hw>Stem</hw>, <pos>v. i.</pos> <def>To move forward against an obstacle, as a vessel against a current.</def><br/
[<source>1913 Webster</source>]</p>

<p><q><qex>Stemming</qex> nightly toward the pole.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stem-clasping</ent><br/
<hw>Stem"-clasp`ing</hw> <pr>(?)</pr>, <pos>a.</pos> <fld>(Bot.)</fld> <def>Embracing the stem with its base; amplexicaul, as a leaf or petiole.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stemless</ent><br/
<hw>Stem"less</hw>, <pos>a.</pos> <def>Having no stem;</def> <fld>(Bot.)</fld> <def>acaulescent.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stemlet</ent><br/
<hw>Stem"let</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A small or young stem.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stemma</ent><br/
||<hw>Stem"ma</hw> <pr>(?)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Stemmata</plw> <pr>(#)</pr>.</plu> <ety>[NL., fr. Gr. <?/, pl. <?/, a garland or chaplet.]</ety> <fld>(Zool.)</fld> <sd>(a)</sd> <def>One of the ocelli of an insect. See <er>Ocellus</er>.</def> <sd>(b)</sd> <def>One of the facets of a compound eye of any arthropod.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stemmer</ent><br/
<hw>Stem"mer</hw> <pr>(?)</pr>, <pos>n.</pos> <def>One who, or that which, stems (in any of the senses of the verbs).</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stemmery</ent><br/
<hw>Stem"mer*y</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A large building in which tobacco is stemmed.</def> <mark>[U. S.]</mark>  <rj><au>Bartlett.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stemmy</ent><br/
<hw>Stem"my</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Abounding in stems, or mixed with stems; -- said of tea, dried currants, etc.</def> <mark>[Colloq.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stemple</ent><br/
<hw>Stem"ple</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[G. <ets>stempel</ets> a stamp, a prop, akin to E. <ets>stamp</ets>.]</ety> <fld>(Mining)</fld> <def>A crossbar of wood in a shaft, serving as a step.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stemson</ent><br/
<hw>Stem"son</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[See <er>Stem</er>, <pos>n.</pos>, and <er>Keelson</er>, and cf. <er>Sternson</er>.]</ety> <fld>(Shipbuilding)</fld> <def>A piece of curved timber bolted to the stem, keelson, and apron in a ship's frame near the bow.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stem-winder</ent><br/
<hw>Stem"-wind`er</hw> <pr>(?)</pr>, <pos>n.</pos> <sn>1.</sn> <def>A stem-winding watch.</def> <mark>[Colloq.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <specif>Hence:</specif> <mark>(Fig.)</mark> <def>Anything of superior quality, as was attributed to the stem-winding watch; -- esp. used to describe a stirring speech, as in the phrase <ldquo/a <ex>stem-winder</ex> of a speech<rdquo/ or <ldquo/delivered a <ex>stem-winder</ex><rdquo/.</def><br/
[<source>PJC</source>]</p>

<p><ent>Stem-winding</ent><br/
<hw>Stem"-wind`ing</hw>, <pos>a.</pos> <def>Wound by mechanism connected with the stem; <as>as, a <ex>stem-winding</ex> watch</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stench</ent><br/
<hw>Stench</hw> <pr>(?)</pr>, <pos>v. t.</pos> <def>To stanch.</def> <mark>[Obs.]</mark>  <rj><au>Harvey.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stench</ent><br/
<hw>Stench</hw>, <pos>n.</pos> <ety>[AS. <ets>stenc</ets> a strong smell, fr. <ets>stincan</ets>. See <er>Stink</er>, <pos>v. i.</pos>]</ety> <sn>1.</sn> <def>A smell; an odor.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>Clouds of savory <qex>stench</qex> involve the sky.</q> <rj><qau>Dryden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>An ill smell; an offensive odor; a stink.</def>  <rj><au>Cowper.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Stench trap</b></col>, <cd>a contrivance to prevent stench or foul air from rising from the openings of sewers, drains, etc.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stench</ent><br/
<hw>Stench</hw>, <pos>v. t.</pos> <ety>[AS. <ets>stencan</ets> to emit a smell, fr. <ets>stincan</ets> to smell. See <er>Stench</er>, <pos>n.</pos>]</ety> <def>To cause to emit a disagreeable odor; to cause to stink.</def> <mark>[Obs.]</mark>  <rj><au>Young.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stenchy</ent><br/
<hw>Stench"y</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Having a stench.</def> <mark>[Obs.]</mark>  <rj><au>Dyer.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stencil</ent><br/
<hw>Sten"cil</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Probably from OF. <ets>estincelle</ets> spangle, spark, F. <ets><eacute/tincelle</ets> spark, L. <ets>scintilla</ets>. See <er>Scintillate</er>, and cf. <er>Tinsel</er>.]</ety> <def>A thin plate of metal, leather, or other material, used in painting, marking, etc. The pattern is cut out of the plate, which is then laid flat on the surface to be marked, and the color brushed over it. Called also <altname>stencil plate</altname>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stencil</ent><br/
<hw>Sten"cil</hw>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Stenciled</conjf> <pr>(?)</pr> or <conjf>Stencilled</conjf>; <pos>p. pr. & vb. n.</pos> <conjf>Stenciling</conjf> or <conjf>Stencilling</conjf>.]</vmorph> <def>To mark, paint, or color in figures with stencils; to form or print by means of a stencil.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stenciler</ent><br/
<hw>Sten"cil*er</hw> <pr>(?)</pr>, <pos>n.</pos> <def>One who paints or colors in figures by means of stencil.</def> <altsp>[Written also <asp>stenciller</asp>.]</altsp><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stenoderm</ent><br/
<hw>Sten"o*derm</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Gr. <grk>steno`s</grk> narrow, little + <ets>-derm</ets>.]</ety> <fld>(Zool.)</fld> <def>Any species of bat belonging to the genus <gen>Stenoderma</gen>, native of the West Indies and South America. These bats have a short or rudimentary tail and a peculiarly shaped nose membrane.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stenodermine</ent><br/
<hw>Sten`o*der"mine</hw> <pr>(?)</pr>, <pos>a.</pos> <fld>(Zool.)</fld> <def>Of or pertaining to the genus <gen>Stenoderma</gen>, which includes several West Indian and South American nose-leaf bats.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stenograph</ent><br/
<hw>Sten"o*graph</hw> <pr>(?)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Stenographed</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Stenographing</conjf> <pr>(?)</pr>.]</vmorph> <def>To write or report in stenographic characters.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stenograph</ent><br/
<hw>Sten"o*graph</hw>, <pos>n.</pos> <def>A production of stenography; anything written in shorthand.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>I saw the reporters' room, in which they redact their hasty <qex>stenographs</qex>.</q> <rj><qau>Emerson.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stenographer</ent><br/
<hw>Ste*nog"ra*pher</hw> <pr>(?)</pr>, <pos>n.</pos> <def>One who is skilled in stenography; a writer of shorthand.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stenographical</ent><br/
<ent>Stenographic</ent><br/
<mhw>{ <hw>Sten`o*graph"ic</hw> <pr>(?)</pr>, <hw>Sten`o*graph"ic*al</hw> <pr>(?)</pr>, }</mhw> <pos>a.</pos> <ety>[Cf. F. <ets>st<eacute/nographique</ets>.]</ety> <def>Of or pertaining to stenography.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stenographist</ent><br/
<hw>Ste*nog"ra*phist</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A stenographer.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stenography</ent><br/
<hw>Ste*nog"ra*phy</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Gr. <grk>steno`s</grk> narrow, close + <ets>graphy</ets>: cf. F. <ets>st<eacute/nographie</ets>, G. <ets>stenographie</ets>.]</ety> <def>The art of writing in shorthand, by using abbreviations or characters for whole words; shorthand.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stenophyllous</ent><br/
<hw>Ste*noph"yl*lous</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[Gr. <grk>steno`s</grk> narrow + <grk>fy`llon</grk> leaf.]</ety> <fld>(Bot.)</fld> <def>Having narrow leaves.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stenosis</ent><br/
||<hw>Ste*no"sis</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[NL., fr. Gr. <grk>steno`s</grk> narrow.]</ety> <fld>(Med.)</fld> <def>A narrowing of the opening or hollow of any passage, tube, or orifice; <as>as, <ex>stenosis</ex> of the pylorus</as>. It differs from <xex>stricture</xex> in being applied especially to diffused rather than localized contractions, and in always indicating an origin organic and not spasmodic.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stenostome</ent><br/
<hw>Sten"o*stome</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[Gr. <grk>steno`s</grk> narrow, little + <grk>sto`ma</grk> mouth.]</ety> <fld>(Zool.)</fld> <def>Having a small or narrow mouth; -- said of certain small ground snakes (<fam>Opoterodonta</fam>), which are unable to dilate their jaws.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stent</ent><br/
<hw>Stent</hw> <pr>(?)</pr>, <pos>v. t.</pos> <vmorph>[<pos>Obs. imp.</pos> <conjf>Stente</conjf> <pr>(?)</pr>; <pos>obs. p. p.</pos> <conjf>Stent</conjf>.]</vmorph> <ety>[See <er>Stint</er>.]</ety> <def>To keep within limits; to restrain; to cause to stop, or cease; to stint.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Then would he weep, he might not be <qex>stent</qex>.</q> <rj><qau>Chaucer.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Yet n'ould she <qex>stent</qex><br/
Her bitter railing and foul revilement.</q> <rj><qau>Spenser.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stent</ent><br/
<hw>Stent</hw>, <pos>v. i.</pos> <def>To stint; to stop; to cease.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>And of this cry they would never <qex>stenten</qex>.</q> <rj><qau>Chaucer.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stent</ent><br/
<hw>Stent</hw>, <pos>n.</pos> <def>An allotted portion; a stint.</def> <ldquo/Attain'd his journey's <xex>stent</xex>.<rdquo/  <rj><au>Mir. for Mag.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stenting</ent><br/
<hw>Stent"ing</hw>, <pos>n.</pos> <def>An opening in a wall in a coal mine.</def> <altsp>[Written also <asp>stenton</asp>.]</altsp> <mark>[Prov. Eng.]</mark>  <rj><au>Halliwell.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stentor</ent><br/
<hw>Sten"tor</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>Stentor</ets>, Gr. <?/.]</ety> <sn>1.</sn> <def>A herald, in the Iliad, who had a very loud voice; hence, any person having a powerful voice.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Zool.)</fld> <def>Any species of ciliated Infusoria belonging to the genus <gen>Stentor</gen> and allied genera, common in fresh water. The stentors have a bell-shaped, or cornucopia-like, body with a circle of cilia around the spiral terminal disk. See <xex>Illust.</xex> under <er>Heterotricha</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <fld>(Zool.)</fld> <def>A howling monkey, or howler.</def><br/
[<source>1913 Webster</source>]</p>

<p><-- p. 1411 --></p>

<p><ent>Stentorian</ent><br/
<hw>Sten*to"ri*an</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>stentoreus</ets>; cf. Gr. <?/.]</ety> <def>Of or pertaining to a stentor; extremely loud; powerful; <as>as, a <ex>stentorian</ex> voice; <ex>stentorian</ex> lungs</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stentorin</ent><br/
<hw>Sten"to*rin</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Chem.)</fld> <def>A blue coloring matter found in some stentors. See <er>Stentor</er>, 2.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stentorious</ent><br/
<hw>Sten*to"ri*ous</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Stentorian.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stentoronic</ent><br/
<hw>Sten`to*ron"ic</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Stentorian.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stentorophonic</ent><br/
<hw>Sten`to*ro*phon"ic</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[Gr. <?/ Stentor + <?/ a sound, voice. See <er>Stentor</er>.]</ety> <def>Speaking or sounding very loud; stentorian.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>Of this <qex>stentorophonic</qex> horn of Alexander there is a preserved in the Vatican.</q> <rj><qau>Derham.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Step</ent><br/
<hw>Step</hw> <pr>(?)</pr>, <pos>v. i.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Stepped</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Stepping</conjf>.]</vmorph> <ety>[AS. <ets>staeppan</ets>; akin to OFries. <ets>steppa</ets>, D. <ets>stappen</ets> to step, <ets>stap</ets> a step, OHG. <ets>stepfen</ets> to step, G. <ets>stapfe</ets> a footstep, OHG. <ets>stapfo</ets>, G. <ets>stufe</ets> a step to step on; cf. Gr. <?/ to shake about, handle roughly, stamp (?).  Cf. <er>Stamp</er>, <pos>n.</pos> & <pos>a.</pos>]</ety> <sn>1.</sn> <def>To move the foot in walking; to advance or recede by raising and moving one of the feet to another resting place, or by moving both feet in succession.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To walk; to go on foot; esp., to walk a little distance; <as>as, to <ex>step</ex> to one of the neighbors</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>To walk slowly, gravely, or resolutely.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Home the swain retreats,<br/
His flock before him <qex>stepping</qex> to the fold.</q> <rj><qau>Thomson.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>Fig.: To move mentally; to go in imagination.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>They are <qex>stepping</qex> almost three thousand years back into the remotest antiquity.</q> <rj><qau>Pope.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>To step aside</b></col>, <cd>to walk a little distance from the rest; to retire from company.</cd> -- <col><b>To step forth</b></col>, <cd>to move or come forth.</cd> -- <mcol><col><b>To step in</b></col> <it>or</it> <col><b>To step into</b></col></mcol>. <sd>(a)</sd> <cd>To walk or advance into a place or state, or to advance suddenly in.</cd><br/
[<source>1913 Webster</source>]</p>

<p><q>Whosoever then first, after the troubling of the water, <qex>stepped in</qex>, was made whole of whatsoever disease he had.</q> <rj><qau>John v. 4.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sd>(b)</sd> <cd>To enter for a short time; <as>as, I just <ex>stepped into</ex> the house</as>.</cd> <sd>(c)</sd> <cd>To obtain possession without trouble; to enter upon easily or suddenly; <as>as, <ex>to step into</ex> an estate</as></cd>. -- <col><b>To step out</b></col>. <sd>(a)</sd> <fld>(Mil.)</fld> <cd>To increase the length, but not the rapidity, of the step, extending it to thirty-tree inches.</cd> <sd>(b)</sd> <cd>To go out for a short distance or a short time</cd>. -- <col><b>To step short</b></col> <fld>(Mil.)</fld>, <cd>to diminish the length or rapidity of the step according to the established rules.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Step</ent><br/
<hw>Step</hw>, <pos>v. t.</pos> <sn>1.</sn> <def>To set, as the foot.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Naut.)</fld> <def>To fix the foot of (a mast) in its step; to erect.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>To step off</b></col>, <cd>to measure by steps, or paces; hence, to divide, as a space, or to form a series of marks, by successive measurements, as with dividers.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Step</ent><br/
<hw>Step</hw>, <pos>n.</pos> <ety>[AS. <ets>staepe</ets>. See <er>Step</er>, <pos>v. i.</pos>]</ety> <sn>1.</sn> <def>An advance or movement made by one removal of the foot; a pace.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A rest, or one of a set of rests, for the foot in ascending or descending, as a stair, or a round of a ladder.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The breadth of every single <qex>step</qex> or stair should be never less than one foot.</q> <rj><qau>Sir H. Wotton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>The space passed over by one movement of the foot in walking or running; <as>as, one <ex>step</ex> is generally about three feet, but may be more or less</as>. Used also figuratively of any kind of progress; <as>as, he improved <ex>step</ex> by <ex>step</ex>, or by <ex>steps</ex></as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>To derive two or three general principles of motion from phenomena, and afterwards to tell us how the properties and actions of all corporeal things follow from those manifest principles, would be a very great <qex>step</qex> in philosophy.</q> <rj><qau>Sir I. Newton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>A small space or distance; <as>as, it is but a <ex>step</ex></as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <def>A print of the foot; a footstep; a footprint; track.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>6.</sn> <def>Gait; manner of walking; <as>as, the approach of a man is often known by his <ex>step</ex></as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>7.</sn> <def>Proceeding; measure; action; an act.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The reputation of a man depends on the first <qex>steps</qex> he makes in the world.</q> <rj><qau>Pope.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Beware of desperate <qex>steps</qex>. The darkest day,<br/
Live till to-morrow, will have passed away.</q> <rj><qau>Cowper.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>I have lately taken <qex>steps</qex> . . . to relieve the old gentleman's distresses.</q> <rj><qau>G. W. Cable.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>8.</sn> <pluf>pl.</pluf> <def>Walk; passage.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Conduct my <qex>steps</qex> to find the fatal tree.</q> <rj><qau>Dryden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>9.</sn> <pluf>pl.</pluf> <def>A portable framework of stairs, much used indoors in reaching to a high position.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>10.</sn> <fld>(Naut.)</fld> <def>In general, a framing in wood or iron which is intended to receive an upright shaft; specif., a block of wood, or a solid platform upon the keelson, supporting the heel of the mast.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>11.</sn> <fld>(Mach.)</fld> <sd>(a)</sd> <def>One of a series of offsets, or parts, resembling the steps of stairs, as one of the series of parts of a cone pulley on which the belt runs.</def> <sd>(b)</sd> <def>A bearing in which the lower extremity of a spindle or a vertical shaft revolves.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>12.</sn> <fld>(Mus.)</fld> <def>The intervak between two contiguous degrees of the csale.</def><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ The word <xex>tone</xex> is often used as the name of this interval; but there is evident incongruity in using <xex>tone</xex> for indicating the interval between tones. As the word <xex>scale</xex> is derived from the Italian <xex>scala</xex>, a ladder, the intervals may well be called <xex>steps</xex>.</note><br/
[<source>1913 Webster</source>]</p>

<p><sn>13.</sn> <fld>(Kinematics)</fld> <def>A change of position effected by a motion of translation.</def>  <rj><au>W. K. Clifford.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>14.</sn> <fld>(Fives)</fld> <def>At Eton College, England, a shallow step dividing the court into an inner and an outer portion.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><cs><mcol><col><b>Back step</b></col>, <col><b>Half step</b></col></mcol>, <cd>etc. See under <er>Back</er>, <er>Half</er>, etc.</cd> -- <col><b>Step grate</b></col>, <cd>a form of grate for holding fuel, in which the bars rise above one another in the manner of steps.</cd> -- <col><b>To take steps</b></col>, <cd>to take action; to move in a matter.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Step-</ent><br/
<hw>Step-</hw>. <ety>[AS. <ets>ste<oacute/p-</ets>; akin to OFries. <ets>stiap-</ets>, <ets>stiep-</ets>, D. & G. <ets>stief-</ets>, OHG. <ets>stiuf-</ets>, Icel. <ets>stj<?/p-</ets>, Sw. <ets>styf-</ets>, and to AS. <ets><amac/st<emac/pan</ets>, <ets><amac/ste<oacute/pan</ets>, to deprive, bereave, as children of their parents, OHG. <ets>stiufen</ets>.]</ety> <def>A prefix used before <xex>father</xex>, <xex>mother</xex>, <xex>brother</xex>, <xex>sister</xex>, <xex>son</xex>, <xex>daughter</xex>, <xex>child</xex>, etc., to indicate that the person thus spoken of is not a blood relative, but is a relative by the marriage of a parent; <as>as, a <ex>step</ex>mother to X is the wife of the father of X, married by him after the death of the mother of X</as>. See <er>Stepchild</er>, <er>Stepdaughter</er>, <er>Stepson</er>, etc.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stepbrother</ent><br/
<hw>Step"broth`er</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A brother by the marriage of one's father with the mother of another, or of one's mother with the father of another.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stepchild</ent><br/
<hw>Step"child`</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[AS. <ets>ste<oacute/pcild</ets>.]</ety> <sn>1.</sn> <def>A bereaved child; one who has lost father or mother.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A son or daughter of one's wife or husband by a former marriage.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stepdame</ent><br/
<hw>Step"dame`</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A stepmother.</def>  <rj><au>Spenser.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stepdaughter</ent><br/
<hw>Step"daugh`ter</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[AS. <ets>ste<oacute/pdohtor</ets>.]</ety> <def>A daughter of one's wife or husband by a former marriage.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Step-down</ent><br/
<hw>Step"-down`</hw>, <pos>a.</pos> <fld>(Elec.)</fld> <def>Transforming or converting a current of high potential or voltage into one of low voltage; <as>as, a <ex>step-down</ex> transformer</as>.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Stepfather</ent><br/
<hw>Step"fa`ther</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[AS. <ets>ste<oacute/pfaeder</ets>.]</ety> <def>The husband of one's mother by a subsequent marriage.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stephanion</ent><br/
<hw>Ste*pha"ni*on</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[NL., from Gr. <?/ a crown.]</ety> <fld>(Anat.)</fld> <def>The point on the side of the skull where the temporal line, or upper edge of the temporal fossa, crosses the coronal suture.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stephanite</ent><br/
<hw>Steph"an*ite</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[So named after the Archduke <ets>Stephan</ets>, mining director of Austria.]</ety> <fld>(Min.)</fld> <def>A sulphide of antimony and silver of an iron-black color and metallic luster; called also <altname>black silver</altname>, and <altname>brittle silver ore</altname>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stephanotis</ent><br/
||<hw>Steph`a*no"tis</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[NL., fr. Gr. <?/ fit for a crown, fr. <?/ crown.]</ety> <sn>1.</sn> <fld>(Bot.)</fld> <def>A genus of climbing asclepiadaceous shrubs, of Madagascar, Malaya, etc. They have fleshy or coriaceous opposite leaves, and large white waxy flowers in cymes.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A perfume said to be prepared from the flowers of <spn>Stephanotis floribunda</spn>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stepladder</ent><br/
<hw>Step"lad`der</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A portable set of steps.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stepmother</ent><br/
<hw>Step"moth`er</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[AS. <ets>ste<oacute/pm<omac/der</ets>.]</ety> <def>The wife of one's father by a subsequent marriage.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stepparent</ent><br/
<hw>Step"par`ent</hw> <pr>(?)</pr>, <pos>n.</pos> <def>Stepfather or stepmother.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Steppe</ent><br/
<hw>Steppe</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[From Russ. <ets>stepe</ets>, through G. or F. <ets>steppe</ets>.]</ety> <def>One of the vast plains in Southeastern Europe and in Asia, generally elevated, and free from wood, analogous to many of the prairies in Western North America. See <er>Savanna</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Steppe murrain</b></col>. <fld>(Far.)</fld> <cd>See <er>Rinderpest</er>.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stepped</ent><br/
<hw>Stepped</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Provided with a step or steps; having a series of offsets or parts resembling the steps of stairs; <as>as, a <ex>stepped</ex> key</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Stepped gear</b></col>, <cd>a cogwheel of which the teeth cross the face in a series of steps.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stepper</ent><br/
<hw>Step"per</hw> <pr>(?)</pr>, <pos>n.</pos> <def>One who, or that which, steps; <as>as, a quick <ex>stepper</ex></as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><-- a stepping motor --><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stepping-stone</ent><br/
<hw>Step"ping-stone`</hw> <pr>(?)</pr>, <pos>n.</pos> <sn>1.</sn> <def>A stone to raise the feet above the surface of water or mud in walking.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Fig.: A means of progress or advancement.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>These obstacles his genius had turned into <qex>stepping-stones</qex>.</q> <rj><qau>Macaulay.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>That men may rise on <qex>stepping-stones</qex><br/
Of their dead selves to higher things.</q> <rj><qau>Tennyson.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stepsister</ent><br/
<hw>Step"sis`ter</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A daughter of one's stepfather or stepmother by a former marriage.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stepson</ent><br/
<hw>Step"son`</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[AS. <ets>ste<oacute/psunu</ets>.]</ety> <def>A son of one's husband or wife by a former marriage.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stepstone</ent><br/
<hw>Step"stone`</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A stone laid before a door as a stair to rise on in entering the house.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Step-up</ent><br/
<hw>Step"-up`</hw>, <pos>a.</pos> <fld>(Elec.)</fld>  <def>Transforming or converting a low-voltage current into one of high voltage; <as>as, a <ex>step-up</ex> transformer</as>.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>-ster</ent><br/
<hw>-ster</hw> <pr>(?)</pr>. <ety>[OE. & AS. <ets>-estre</ets>, <ets>-istre</ets>.]</ety> <def>A suffix denoting <xex>the agent</xex> (originally a woman), especially a person who does something <xex>with skill</xex> or <xex>as an occupation</xex>; as in spin<xex>ster</xex> (originally, a woman who spins), song<xex>ster</xex>, ba<xex>xter</xex> (= bake<xex>ster</xex>), young<xex>ster</xex>.</def><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ Brewing, baking, and weaving were formerly feminine labors, and consequently <xex>brewster</xex>, <xex>baxter</xex>, and <xex>webster</xex> meant, originally, the woman (not the man) who brews, bakes, or weaves. When men began to perform these duties the feminine appellations were retained.</note><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stercobilin</ent><br/
<hw>Ster`co*bi"lin</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>stercus</ets> dung + E. <ets>bilin</ets>.]</ety> <fld>(Physiol. Chem.)</fld> <def>A coloring matter found in the faeces, a product of the alteration of the bile pigments in the intestinal canal, -- identical with <altname>hydrobilirubin</altname>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stercolin</ent><br/
<hw>Ster"co*lin</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>stercus</ets> dung + <ets>ol</ets>eum oil.]</ety> <fld>(Physiol. Chem.)</fld> <def>Same as <er>Serolin</er> <sd>(b)</sd>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stercoraceous</ent><br/
<hw>Ster`co*ra"ceous</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>stercus</ets>, <ets>-oris</ets>, dung.]</ety> <def>Of or pertaining to dung; partaking of the nature of, or containing, dung.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stercoranism</ent><br/
<hw>Ster"co*ra*nism</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Eccl. Hist.)</fld> <def>The doctrine or belief of the Stercoranists.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stercoranist</ent><br/
<hw>Ster"co*ra*nist</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[LL. <ets>stercoranista</ets>, fr. L. <ets>stercus</ets>, <ets>-oris</ets>, dung.]</ety> <fld>(Eccl. Hist.)</fld> <def>A nickname formerly given to those who held, or were alleged to hold, that the consecrated elements in the eucharist undergo the process of digestion in the body of the recipient.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stercorarian</ent><br/
<hw>Ster`co*ra"ri*an</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A Stercoranist.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stercorary</ent><br/
<hw>Ster"co*ra*ry</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[LL. <ets>stercorarium</ets>, from L. <ets>stercorarius</ets> belonging to dung.]</ety> <def>A place, properly secured from the weather, for containing dung.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stercorate</ent><br/
<hw>Ster"co*rate</hw> <pr>(?)</pr>, <pos>n.</pos> <def>Excrement; dung.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stercoration</ent><br/
<hw>Ster`co*ra"tion</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>stercoratio</ets>, from <ets>stercorare</ets> to dung.]</ety> <def>Manuring with dung.</def> <mark>[Obs.]</mark>  <rj><au>Bacon.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stercorianism</ent><br/
<hw>Ster*co"ri*an*ism</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Eccl.)</fld> <def>The doctrine or belief of the Stercoranists.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stercorin</ent><br/
<hw>Ster"co*rin</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>stercus</ets>, <ets>-oris</ets>, dung.]</ety> <fld>(Physiol. Chem.)</fld> <def>Same as <er>Serolin</er> <sd>(b)</sd>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stercory</ent><br/
<hw>Ster"co*ry</hw> <pr>(?)</pr>, <pos>n.</pos> <def>Excrement; dung.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sterculiaceous</ent><br/
<hw>Ster*cu`li*a"ceous</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[NL. <ets>Sterculia</ets>, the typical genus, fr. L. <ets>Sterculius</ets> the deity that presided over manuring, from <ets>stercus</ets> dung. So called because one of the original species is fetid.]</ety> <fld>(Bot.)</fld> <def>Of or pertaining to a natural order (<ord>Sterculiaceae</ord>) of polypetalous exogenous plants, mostly tropical. The cacao (<spn>Theobroma Cacao</spn>) is the most useful plant of the order.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stere</ent><br/
<hw>Stere</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F. <ets>st<egrave/re</ets>, fr. Gr. <?/ solid.]</ety> <def>A unit of cubic measure in the metric system, being a cubic meter, or kiloliter, and equal to 35.3 cubic feet, or nearly 1<frac13/ cubic yards.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stere</ent><br/
<hw>Stere</hw> <pr>(?)</pr>, <pos>v. t. & i.</pos> <def>To stir.</def> <mark>[Obs.]</mark>  <rj><au>Chaucer.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stere</ent><br/
<hw>Stere</hw>, <pos>n.</pos> <def>A rudder. See 5th <er>Steer</er>.</def> <mark>[Obs.]</mark>  <rj><au>Chaucer.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stere</ent><br/
<hw>Stere</hw>, <pos>n.</pos> <def>Helmsman. See 6th <er>Steer</er>.</def> <mark>[Obs.]</mark>  <rj><au>Chaucer.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sterelmintha</ent><br/
||<hw>Ster`el*min"tha</hw> <pr>(?)</pr>, <pos>n. pl.</pos> <ety>[NL., fr. Gr. <grk>stereo`s</grk> solid + <?/ a worm.]</ety> <fld>(Zool.)</fld> <def>Same as <er>Platyelminthes</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stereo-</ent><br/
<hw>Ste"re*o-</hw> <pr>(?)</pr>. <ety>[Gr. <grk>stereo`s</grk> solid. See <er>Stare</er> to gaze.]</ety> <def>A combining form meaning <xex>solid</xex>, <xex>hard</xex>, <xex>firm</xex>, as in <xex>stereo</xex>-chemistry, <xex>stereo</xex>graphy.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stereobate</ent><br/
<hw>Ste"re*o*bate</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Gr. <grk>stereo`s</grk> solid + <?/ that treads or covers, akin to <?/ base; cf. F. <ets>st<eacute/r<eacute/obate</ets>.]</ety> <fld>(Arch.)</fld> <def>The lower part or basement of a building or pedestal; -- used loosely for several different forms of basement.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stereo-chemical</ent><br/
<ent>Stereo-chemic</ent><br/
<mhw>{ <hw>Ste`re*o-chem"ic</hw> <pr>(?)</pr>, <hw>Ste`re*o-chem"ic*al</hw> <pr>(?)</pr>, }</mhw> <pos>a.</pos> <fld>(Chem.)</fld> <def>Pertaining to, or illustrating, the hypothetical space relations of atoms in the molecule; <as>as, a <ex>stereo-chemic</ex> formula</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stereo-chemistry</ent><br/
<hw>Ste`re*o-chem"is*try</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[<ets>Stereo-</ets> + <ets>chemistry</ets>.]</ety> <fld>(Chem.)</fld> <def>Chemistry considered with reference to the space relations of atoms.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stereochrome</ent><br/
<hw>Ste"re*o*chrome</hw> <pr>(?)</pr>, <pos>n.</pos> <def>Stereochromic picture.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stereochromic</ent><br/
<hw>Ste`re*o*chro"mic</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Pertaining to the art of stereochromy; produced by stereochromy.</def> -- <wordforms><wf>Ste`re*o*chro"mic*al*ly</wf> <pr>(#)</pr>, <pos>adv.</pos></wordforms><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stereochromy</ent><br/
<hw>Ste`re*och"ro*my</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[<ets>Stereo-</ets> + Gr. <grk>chrw^ma</grk> color.]</ety> <def>A style of painting on plastered walls or stone, in which the colors are rendered permanent by sprinklings of water, in which is mixed a proportion of soluble glass (a silicate of soda).</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stereoelectric</ent><br/
<hw>Ste`re*o*e*lec"tric</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[<ets>Stereo-</ets> + <ets>electric</ets>.]</ety> <fld>(Physics)</fld> <def>Of or pertaining to the generation of electricity by means of solid bodies alone; <as>as, a <ex>stereoelectric</ex> current is one obtained by means of solids, without any liquid</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stereogram</ent><br/
<hw>Ste"re*o*gram</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[<ets>Stereo-</ets> + <ets>-gram</ets>.]</ety> <def>A diagram or picture which represents objects in such a way as to give the impression of relief or solidity; also, a stereograph.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stereograph</ent><br/
<hw>Ste"re*o*graph</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[<ets>Stereo-</ets> + <ets>-graph</ets>.]</ety> <def>Any picture, or pair of pictures, prepared for exhibition in the stereoscope. Stereographs are now commonly made by means of photography.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stereographical</ent><br/
<ent>Stereographic</ent><br/
<mhw>{ <hw>Ste`re*o*graph"ic</hw> <pr>(?)</pr>, <hw>Ste`re*o*graph"ic*al</hw> <pr>(?)</pr>, }</mhw> <pos>a.</pos> <ety>[Cf. F. <ets>st<eacute/r<eacute/ographique</ets>.]</ety> <def>Made or done according to the rules of stereography; delineated on a plane; <as>as, a <ex>stereographic</ex> chart of the earth</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Stereographic projection</b></col> <fld>(Geom.)</fld>, <cd>a method of representing the sphere in which the center of projection is taken in the surface of the sphere, and the plane upon which the projection is made is at right andles to the diameter passing through the center of projection.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stereographically</ent><br/
<hw>Ste`re*o*graph"ic*al*ly</hw>, <pos>adv.</pos> <def>In a stereographical manner; by delineation on a plane.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stereography</ent><br/
<hw>Ste`re*og"ra*phy</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[<ets>Stereo-</ets> + <ets>graphy</ets>: cf. F. <ets>st<eacute/r<eacute/ographie</ets>.]</ety> <def>The art of delineating the forms of solid bodies on a plane; a branch of solid geometry which shows the construction of all solids which are regularly defined.</def><br/
[<source>1913 Webster</source>]</p>

<p><-- Illustration: "Stereography".
   5 figures, illustrating arrangements of triangles, squares, or pentagons which can be drawn on a surface and folded into the five regular polyhedra. --><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ By cutting pieces of cardboard, or other suitable material, in the forms represented in the cut, folding them along the lines indicated, and joining their edges, the five regular solids may be formed.</note><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stereometer</ent><br/
<hw>Ste`re*om"e*ter</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[<ets>Stereo-</ets> + <ets>meter</ets>.]</ety> <fld>(Physics)</fld> <sn>1.</sn> <def>An instrument for measuring the solid contents of a body, or the capacity of a vessel; a volumenometer.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>An instrument for determining the specific gravity of liquid bodies, porous bodies, and powders, as well as solids.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stereometrical</ent><br/
<ent>Stereometric</ent><br/
<mhw>{ <hw>Ste`re*o*met"ric</hw> <pr>(?)</pr>, <hw>Ste`re*o*met"ric*al</hw> <pr>(?)</pr>, }</mhw> <pos>a.</pos> <ety>[Cf. F. <ets>st<eacute/r<eacute/om<eacute/trique</ets>.]</ety> <def>Of or pertaining to stereometry; performed or obtained by stereometry.</def> -- <wordforms><wf>Ste`re*o*met"ric*al*ly</wf>, <pos>adv.</pos></wordforms><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stereometry</ent><br/
<hw>Ste`re*om"e*try</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[<ets>Stereo-</ets> + <ets>-metry</ets>: cf. F. <ets>st<eacute/r<eacute/om<eacute/trie</ets>.]</ety> <def>The art of measuring and computing the cubical contents of bodies and figures; -- distinguished from <xex>planimetry</xex>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stereomonoscope</ent><br/
<hw>Ste`re*o*mon"o*scope</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[<ets>Stereo-</ets> + <ets>mono-</ets> + <ets>-scope</ets>.]</ety> <def>An instrument with two lenses, by which an image of a single picture projected upon a screen of ground glass is made to present an appearance of relief, and may be viewed by several persons at once.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stereoplasm</ent><br/
<hw>Ste"re*o*plasm</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[<ets>Stereo-</ets> + Gr. <?/ anything formed or molded.]</ety> <fld>(Biol.)</fld> <def>The solid or insoluble portion of the cell protoplasm. See <er>Hygroplasm</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stereopticon</ent><br/
<hw>Ste`re*op"ti*con</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[NL. See <er>Stereo-</er>, and <er>Optic</er>.]</ety> <def>An instrument, consisting essentially of a magic lantern in which photographic pictures are used, by which the image of a landscape, or any object, may be thrown upon a screen in such a manner as to seem to stand out in relief, so as to form a striking and accurate representation of the object itself; also, a pair of magic lanterns for producing the effect of dissolving views.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stereoscope</ent><br/
<hw>Ste"re*o*scope</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[<ets>Stereo-</ets> + <ets>-scope</ets>.]</ety> <def>An optical instrument for giving to pictures the appearance of solid forms, as seen in nature. It combines in one, through a bending of the rays of light, two pictures, taken for the purpose from points of view a little way apart. It is furnished with two eyeglasses, and by refraction or reflection the pictures are superimposed, so as to appear as one to the observer.</def><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ In the <xex>reflecting stereoscope</xex>, the rays from the two pictures are turned into the proper direction for stereoscopic vision by two plane mirrors set at an angle with each other, and between the pictures. In the <xex>lenticular stereoscope</xex>, the form in general use, the eyeglasses are semilenses, or marginal portions of the same convex lenses, set with their edges toward each other, so that they deflect the rays coming from the picture so as to strike the eyes as if coming direct from an intermediate point, where the two pictures are seen apparently as one.</note><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stereoscopical</ent><br/
<ent>Stereoscopic</ent><br/
<mhw>{ <hw>Ste`re*o*scop"ic</hw> <pr>(?)</pr>, <hw>Ste`re*o*scop"ic*al</hw> <pr>(?)</pr>, }</mhw> <pos>a.</pos> <def>Of or pertaining to the stereoscope; characteristic of, or adapted to, the stereoscope; <as>as, a <ex>stereoscopic</ex> effect; the <ex>stereoscopic</ex> function of the eyeglasses; <ex>stereoscopic</ex> views</as>.</def> -- <wordforms><wf>Ste`re*o*scop"ic*al*ly</wf>, <pos>adv.</pos></wordforms><br/
[<source>1913 Webster</source>]</p>

<p><-- 2. Permitting the perception of depth by exhibiting different views to each eye; as, a sereoscopic microscope, i.e. one in which each eye views the subject from a different angle, through separate optical paths. --><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stereoscopist</ent><br/
<hw>Ste`re*os"co*pist</hw> <pr>(?)</pr>, <pos>n.</pos> <def>One skilled in the use or construction of stereoscopes.</def><br/
[<source>1913 Webster</source>]</p>

<p><-- p. 1412 --></p>

<p><ent>Stereoscopy</ent><br/
<hw>Ste`re*os"co*py</hw> <pr>(?)</pr>, <pos>n.</pos> <def>The art or science of using the stereoscope, or of constructing the instrument or the views used with it.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stereostatic</ent><br/
<hw>Ste`re*o*stat"ic</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[<ets>Stereo-</ets> + <ets>static</ets>.]</ety> <fld>(Civil. Engin.)</fld> <def>Geostatic.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stereotomical</ent><br/
<ent>Stereotomic</ent><br/
<mhw>{ <hw>Ste`re*o*tom"ic</hw> <pr>(?)</pr>, <hw>Ste`re*o*tom"ic*al</hw> <pr>(?)</pr>, }</mhw> <pos>a.</pos> <def>Of or pertaining to stereotomy; performed by stereotomy.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stereotomy</ent><br/
<hw>Ste`re*ot"o*my</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[<ets>Stereo-</ets> + Gr. <?/ to cut: cf. F. <ets>st<eacute/r<eacute/otomie</ets>.]</ety> <def>The science or art of cutting solids into certain figures or sections, as arches, and the like; especially, the art of stonecutting.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stereotype</ent><br/
<hw>Ste"re*o*type</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[<ets>Stereo-</ets> + <ets>-type</ets>: cf. F. <ets>st<eacute/r<eacute/otype</ets>.]</ety> <sn>1.</sn> <def>A plate forming an exact faximile of a page of type or of an engraving, used in printing books, etc.; specifically, a plate with type-metal face, used for printing.</def><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ A <xex>stereotype</xex>, or <xex>stereotypr plate</xex>, is made by setting movable type as for ordinary printing; from these a cast is taken in plaster of Paris, paper pulp, or the like, and upon this cast melted type metal is poured, which, when hardened, makes a solid page or column, from which the impression is taken as from type.</note><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>The art or process of making such plates, or of executing work by means of them.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Stereotype block</b></col>, <cd>a block, usually of wood, to which a stereotype plate is attached while being used in printing.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stereotype</ent><br/
<hw>Ste"re*o*type</hw>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Stereotyped</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Stereotyping</conjf> <pr>(?)</pr>.]</vmorph> <ety>[Cf. F. <ets>st<eacute/r<eacute/otyper</ets>.]</ety> <sn>1.</sn> <def>To prepare for printing in stereotype; to make the stereotype plates of; <as>as, to <ex>stereotype</ex> the Bible</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Fig.: To make firm or permanent; to fix.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Powerful causes tending to <qex>stereotype</qex> and aggravate the poverty of old conditions.</q> <rj><qau>Duke of Argyll (1887).</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stereotyped</ent><br/
<hw>Ste"re*o*typed</hw> <pr>(?)</pr>, <pos>a.</pos> <sn>1.</sn> <def>Formed into, or printed from, stereotype plates.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Fig.: Formed in a fixed, unchangeable manner; <as>as, <ex>stereotyped</ex> opinions</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Our civilization, with its <qex>stereotyped</qex> ways and smooth conventionalities.</q> <rj><qau>J. C. Shairp.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stereotyper</ent><br/
<hw>Ste"re*o*ty`per</hw> <pr>(?)</pr>, <pos>n.</pos> <def>One who stereotypes; one who makes stereotype plates, or works in a stereotype foundry.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stereotypery</ent><br/
<hw>Ste"re*o*ty`per*y</hw> <pr>(?)</pr>, <pos>n.</pos> <sn>1.</sn> <def>The art, process, or employment of making stereotype plates.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A place where stereotype plates are made; a stereotype foundry.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stereotypic</ent><br/
<hw>Ste`re*o*typ"ic</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Of or pertaining to stereotype, or stereotype plates.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stereotypist</ent><br/
<hw>Ste"re*o*ty`pist</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A stereotyper.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stereotypographer</ent><br/
<hw>Ste`re*o*ty*pog"ra*pher</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A stereotype printer.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stereotypography</ent><br/
<hw>Ste`re*o*ty*pog"ra*phy</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[<ets>Stereo-</ets> + <ets>typography</ets>.]</ety> <def>The act or art of printing from stereotype plates.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stereotypy</ent><br/
<hw>Ste"re*o*ty`py</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Cf. F. <ets>st<eacute/r<eacute/otypie</ets>.]</ety> <def>The art or process of making stereotype plates.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sterhydraulic</ent><br/
<hw>Ster`hy*drau"lic</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[<ets>Stereo-</ets> + <ets>hydraulic</ets>.]</ety> <def>Pertaining to, or designating, a kind of hydraulic press; resembling such a press in action or principle.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Sterhydraulic press</b></col>, <cd>an hydraulic press producing pressure or motion by the introduction of a solid substance (as a long rod, or a cord wound on a roller) into a cylinder previously filled with a liquid.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sterile</ent><br/
<hw>Ster"ile</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[F. <ets>st<eacute/rile</ets>, L. <ets>sterilis</ets>, akin to Gr. <grk>stereo`s</grk> stiff, solid, <grk>stei^ros</grk> barren, <grk>stei^ra</grk> a cow that has not calved, Goth. <ets>stair<omac/</ets>, fem., barren. See <er>Stare</er> to gaze.]</ety> <sn>1.</sn> <def>Producing little or no crop; barren; unfruitful; unproductive; not fertile; <as>as, <ex>sterile</ex> land; a <ex>sterile</ex> desert; a <ex>sterile</ex> year</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Biol.)</fld> <sd>(a)</sd> <def>Incapable of reproduction; unfitted for reproduction of offspring; not able to germinate or bear fruit; unfruitful; <as>as, a <ex>sterile</ex> flower, which bears only stamens</as>.</def> <sd>(b)</sd> <def>Free from reproductive spores or germs; <as>as, a <ex>sterile</ex> fluid</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>Fig.: Barren of ideas; destitute of sentiment; <as>as, a <ex>sterile</ex> production or author</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sterility</ent><br/
<hw>Ste*ril"i*ty</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>sterilitas</ets>: cf. F. <ets>st<eacute/rilit<eacute/</ets>.]</ety> <sn>1.</sn> <def>The quality or condition of being sterile.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Biol.)</fld> <def>Quality of being sterile; infecundity; also, the state of being free from germs or spores.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sterilization</ent><br/
<hw>Ster`il*i*za"tion</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Biol.)</fld> <def>The act or process of sterilizing, or rendering sterile; also, the state of being sterile.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sterilize</ent><br/
<hw>Ster"il*ize</hw> <pr>(?)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Sterilized</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Sterilizing</conjf> <pr>(?)</pr>.]</vmorph> <ety>[Cf. F. <ets>st<eacute/riliser</ets>.]</ety> <sn>1.</sn> <def>To make sterile or unproductive; to impoverish, as land; to exhaust of fertility.</def> <mark>[R.]</mark> <ldquo/<xex>Sterilizing</xex> the earth.<rdquo/  <rj><au>Woodward.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Biol.)</fld> <def>To deprive of the power of reproducing; to render incapable of germination or fecundation; to make sterile.</def></p>

<p><sn>3.</sn> <fld>(Microbiology, Medicine)</fld> <def>To destroy all spores or germs in (an organic fluid or mixture) or on (a medical instrument), as by heat, so as to prevent contamination by bacteria or other organisms.  A common method of sterilization in laboratories and medical facilities is to heat a liquid sample or an instrument in an autoclave.</def><br/
[<source>1913 Webster</source> <source>+PJC</source>]</p>

<p><sn>4.</sn> <def>To destroy all spores or germs on (a surface) by wetting with an antiseptic liquid, such as an alcoholic solution.</def><br/
[<source>PJC</source>]</p>

<p><ent>Sterilizer</ent><br/
<hw>Ster"il*i`zer</hw> <pr>(?)</pr>, <pos>n.</pos> <def>One that sterilizes anything; specif., an apparatus for sterilizing equipment or an organic fluid.</def><br/
[<source>Webster 1913 Suppl.</source> <source>+PJC</source>]</p>

<p><ent>Sterlet</ent><br/
<hw>Ster"let</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Russ. <ets>sterliade</ets>.]</ety> <fld>(Zool.)</fld> <def>A small sturgeon (<spn>Acipenser ruthenus</spn>) found in the Caspian Sea and its rivers, and highly esteemed for its flavor. The finest caviar is made from its roe.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sterling</ent><br/
<hw>Ster"ling</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Engin.)</fld> <def>Same as <er>Starling</er>, 3.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sterling</ent><br/
<hw>Ster"ling</hw>, <pos>n.</pos> <ety>[OE. <ets>sterlynge</ets>, <ets>starling</ets>, for <ets>easterling</ets>, LL. <ets>esterlingus</ets>, probably from <ets>Easterling</ets>, once the popular name of German trades in England, whose money was of the purest quality: cf. MHG. <ets>sterlink</ets> a certain coin.  Cf. <er>East</er>. <ldquo/Certain merchants of Norwaie, Denmarke, and of others those parties, called Ostomanni, or (as in our vulgar language we tearme them), <ets>easterlings</ets>, because they lie <ets>east</ets> in respect of us.<rdquo/ <au>Holinshed.</au> <ldquo/In the time of . . . King Richard the First, monie coined in the east parts of Germanie began to be of especiall request in England for the puritie thereof, and was called <ets>Easterling</ets> monie, as all inhabitants of those parts were called <ets>Easterlings</ets>, and shortly after some of that countrie, skillful in mint matters and allaies, were sent for into this realme to bring the coine to perfection; which since that time was called of them <ets>sterling</ets>, for <ets>Easterling</ets>.<rdquo/ <au>Camden.</au> <ldquo/Four thousand pound of <ets>sterlings</ets>.<rdquo/ <au>R. of Gloucester.</au>]</ety> <sn>1.</sn> <def>Any English coin of standard value; coined money.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>So that ye offer nobles or <qex>sterlings</qex>.</q> <rj><qau>Chaucer.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>And Roman wealth in English <qex>sterling</qex> view.</q> <rj><qau>Arbuthnot.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A certain standard of quality or value for money.</def><br/
[<source>1913 Webster</source>]</p>

<p><q><qex>Sterling</qex> was the known and approved standard in England, in all probability, from the beginning of King Henry the Second's reign.</q> <rj><qau>S. M. Leake.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sterling</ent><br/
<hw>Ster"ling</hw> <pr>(?)</pr>, <pos>a.</pos> <sn>1.</sn> <def>Belonging to, or relating to, the standard British money of account, or the British coinage; <as>as, a pound <ex>sterling</ex>; a shilling <ex>sterling</ex>; a penny <ex>sterling</ex>; -- now chiefly applied to the lawful money of England; but <ex>sterling</ex> cost, <ex>sterling</ex> value, are used.</as></def> <ldquo/With <xex>sterling</xex> money.<rdquo/  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Genuine; pure; of excellent quality; conforming to the highest standard; of full value; <as>as, a work of <ex>sterling</ex> merit; a man of <ex>sterling</ex> good sense</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stern</ent><br/
<hw>Stern</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[AS. <ets>stearn</ets> a kind of bird. See <er>Starling</er>.]</ety> <fld>(Zool.)</fld> <def>The black tern.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stern</ent><br/
<hw>Stern</hw>, <pos>a.</pos> <amorph>[<pos>Compar.</pos> <adjf>Sterner</adjf> <pr>(?)</pr>; <pos>superl.</pos> <adjf>Sternest</adjf>.]</amorph> <ety>[OE. <ets>sterne</ets>, <ets>sturne</ets>, AS. <ets>styrne</ets>; cf. D. <ets>stuurish</ets> stern, Sw. <ets>stursk</ets> refractory. <root/166.]</ety> <def>Having a certain hardness or severity of nature, manner, or aspect; hard; severe; rigid; rigorous; austere; fixed; unchanging; unrelenting; hence, serious; resolute; harsh; <as>as, a <ex>stern</ex>resolve; a <ex>stern</ex> necessity; a <ex>stern</ex> heart; a <ex>stern</ex> gaze; a <ex>stern</ex> decree.</as></def><br/
[<source>1913 Webster</source>]</p>

<p><q>The <qex>sterne</qex> wind so loud gan to rout.</q> <rj><qau>Chaucer.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>I would outstare the <qex>sternest</qex> eyes that look.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>When that the poor have cried, Caesar hath wept;<br/
Ambition should be made of <qex>sterner</qex> stuff.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q><qex>Stern</qex> as tutors, and as uncles hard.</q> <rj><qau>Dryden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>These barren rocks, your <qex>stern</qex> inheritance.</q> <rj><qau>Wordsworth.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><syn><b>Syn.</b> -- Gloomy; sullen; forbidding; strict; unkind; hard-hearted; unfeeling; cruel; pitiless.</syn><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stern</ent><br/
<hw>Stern</hw>, <pos>n.</pos> <ety>[Icel. <ets>stj<omac/rn</ets> a steering, or a doubtful AS. <ets>ste<oacute/rn</ets>. <root/166. See <er>Steer</er>, <pos>v. t.</pos>]</ety> <sn>1.</sn> <def>The helm or tiller of a vessel or boat; also, the rudder.</def> <mark>[Obs.]</mark>  <rj><au>Chaucer.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Naut.)</fld> <def>The after or rear end of a ship or other vessel, or of a boat; the part opposite to the stem, or prow.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>Fig.: The post of management or direction.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>And sit chiefest <qex>stern</qex> of public weal.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>The hinder part of anything.</def>  <rj><au>Spenser.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <def>The tail of an animal; -- now used only of the tail of a dog.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>By the stern</b></col>. <fld>(Naut.)</fld> <cd>See <cref>By the head</cref>, under <er>By</er>.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stern</ent><br/
<hw>Stern</hw>, <pos>a.</pos> <def>Being in the stern, or being astern; <as>as, the <ex>stern</ex> davits</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Stern board</b></col> <fld>(Naut.)</fld>, <cd>a going or falling astern; a loss of way in making a tack; <as>as, to make a <ex>stern board</ex></as>. See <er>Board</er>, <pos>n.</pos>, 8 <sd>(b)</sd>.</cd> -- <col><b>Stern chase</b></col>. <fld>(Naut.)</fld> <sd>(a)</sd> <cd>See under <er>Chase</er>, <pos>n.</pos></cd> <sd>(b)</sd> <cd>A stern chaser.</cd> -- <col><b>Stern chaser</b></col> <fld>(Naut.)</fld>, <cd>a cannon placed in a ship's stern, pointing backward, and intended to annoy a ship that is in pursuit.</cd> -- <col><b>Stern fast</b></col> <fld>(Naut.)</fld>, <cd>a rope used to confine the stern of a ship or other vessel, as to a wharf or buoy.</cd> -- <col><b>Stern frame</b></col> <fld>(Naut.)</fld>, <cd>the framework of timber forms the stern of a ship.</cd> -- <col><b>Stern knee</b></col>. <cd>See <er>Sternson</er>.</cd> -- <col><b>Stern port</b></col> <fld>(Naut.)</fld>, <cd>a port, or opening, in the stern of a ship.</cd> -- <col><b>Stern sheets</b></col> <fld>(Naut.)</fld>, <cd>that part of an open boat which is between the stern and the aftmost seat of the rowers, -- usually furnished with seats for passengers.</cd> -- <col><b>Stern wheel</b></col>, <cd>a paddle wheel attached to the stern of the steamboat which it propels.</cd></cs><-- thus, <col><b>stern wheeler</b></col>. --><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sternage</ent><br/
<hw>Stern"age</hw> <pr>(?)</pr>, <pos>n.</pos> <def>Stern.</def> <mark>[R.]</mark>  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sternal</ent><br/
<hw>Ster"nal</hw> <pr>(?)</pr>, <pos>a.</pos> <fld>(Anat.)</fld> <def>Of or pertaining to the sternum; in the region of the sternum.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Sternal ribs</b></col>. <cd>See the Note under <er>Rib</er>, <pos>n.</pos>, 1.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sternbergite</ent><br/
<hw>Stern"berg*ite</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[So named after Count Kaspar <ets>Sternberg</ets> of Prague.]</ety> <fld>(Min.)</fld> <def>A sulphide of silver and iron, occurring in soft flexible laminae varying in color from brown to black.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sternebra</ent><br/
||<hw>Ster"ne*bra</hw> <pr>(?)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Sternebrae</plw> <pr>(#)</pr>.</plu> <ety>[NL., fr. <ets>sternum</ets> + <ets>-bra</ets> of <ets>vertebra</ets>.]</ety> <fld>(Anat.)</fld> <def>One of the segments of the sternum.</def> -- <wordforms><wf>Ster"ne*bral</wf> <pr>(#)</pr>, <pos>a.</pos></wordforms><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sterned</ent><br/
<hw>Sterned</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Having a stern of a particular shape; -- used in composition; <as>as, square-<ex>sterned</ex></as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sterner</ent><br/
<hw>Stern"er</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[See 3d <er>Stern</er>.]</ety> <def>A director.</def> <mark>[Obs. & R.]</mark>  <rj><au>Dr. R. Clerke.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sternforemost</ent><br/
<hw>Stern`fore"most`</hw> <pr>(?)</pr>, <pos>adv.</pos> <def>With the stern, instead of the bow, in advance; hence, figuratively, in an awkward, blundering manner.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>A fatal genius for going <qex>sternforemost</qex>.</q> <rj><qau>Lowell.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sternite</ent><br/
<hw>Ster"nite</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[From <er>Sternum</er>.]</ety> <fld>(Zool.)</fld> <def>The sternum of an arthropod somite.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sternly</ent><br/
<hw>Stern"ly</hw> <pr>(?)</pr>, <pos>adv.</pos> <def>In a stern manner.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sternmost</ent><br/
<hw>Stern"most`</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Farthest in the rear; farthest astern; <as>as, the <ex>sternmost</ex> ship in a convoy</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sternness</ent><br/
<hw>Stern"ness</hw>, <pos>n.</pos> <def>The quality or state of being stern.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sterno-</ent><br/
<hw>Ster"no-</hw> <pr>(?)</pr>. <def>A combining form used in anatomy to indicate <xex>connection with</xex>, or <xex>relation to</xex>, <xex>the sternum</xex>; <as>as, <ex>sterno</ex>costal, <ex>sterno</ex>scapular</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sternocoracoid</ent><br/
<hw>Ster`no*cor"a*coid</hw> <pr>(?)</pr>, <pos>a.</pos> <fld>(Anat.)</fld> <def>Of or pertaining to the sternum and the coracoid.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sternocostal</ent><br/
<hw>Ster`no*cos"tal</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[<ets>Sterno-</ets> + <ets>costal</ets>.]</ety> <fld>(Anat.)</fld> <def>Of or pertaining to the sternum and the ribs; <as>as, the <ex>sternocostal</ex> cartilages</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sternohyoid</ent><br/
<hw>Ster`no*hy"oid</hw> <pr>(?)</pr>, <pos>a.</pos> <fld>(Anat.)</fld> <def>Of or pertaining to the sternum and the hyoid bone or cartilage.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sternomastoid</ent><br/
<hw>Ster`no*mas"toid</hw> <pr>(?)</pr>, <pos>a.</pos> <fld>(Anat.)</fld> <def>Of or pertaining to the sternum and the mastoid process.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sternothyroid</ent><br/
<hw>Ster`no*thy"roid</hw> <pr>(?)</pr>, <pos>a.</pos> <fld>(Anat.)</fld> <def>Of or pertaining to the sternum and the thyroid cartilage.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sternpost</ent><br/
<hw>Stern"post`</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Naut.)</fld> <def>A straight piece of timber, or an iron bar or beam, erected on the extremity of the keel to support the rudder, and receive the ends of the planks or plates of the vessel.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sternsman</ent><br/
<hw>Sterns"man</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A steersman.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sternson</ent><br/
<hw>Stern"son</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[See <er>Stern</er>, <pos>n.</pos>, and cf. <er>Stemson</er>.]</ety> <fld>(Naut.)</fld> <def>The end of a ship's keelson, to which the sternpost is bolted; -- called also <altname>stern knee</altname>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sternum</ent><br/
<hw>Ster"num</hw> <pr>(?)</pr>, <pos>n.</pos>; <plu><it>pl.</it> L. <plw>Sterna</plw> <pr>(#)</pr>, E. <plw>Sternums</plw> <pr>(#)</pr>.</plu> <ety>[NL., from Gr. <?/, the breast, chest.]</ety> <sn>1.</sn> <fld>(Anat.)</fld> <def>A plate of cartilage, or a series of bony or cartilaginous plates or segments, in the median line of the pectoral skeleton of most vertebrates above fishes; the breastbone.</def><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ The sternum is connected with the ribs or the pectorial girdle, or with both. In man it is a flat bone, broad anteriorly, narrowed behind, and connected with the clavicles and the cartilages of the seven anterior pairs of ribs. In most birds it has a high median keel for the attachment of the muscles of the wings.</note><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Zool.)</fld> <def>The ventral part of any one of the somites of an arthropod.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sternutation</ent><br/
<hw>Ster`nu*ta"tion</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>sternutatio</ets>, fr. <ets>sternutare</ets> to sneeze, intens. from <ets>sternuere</ets>.]</ety> <def>The act of sneezing.</def>  <rj><au>Quincy.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sternutative</ent><br/
<hw>Ster*nu"ta*tive</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Having the quality of provoking to sneeze.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sternutatory</ent><br/
<hw>Ster*nu"ta*to*ry</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Sternutative.</def> -- <def2><pos>n.</pos> <def>A sternutatory substance or medicine.</def></def2><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sternway</ent><br/
<hw>Stern"way`</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Naut.)</fld> <def>The movement of a ship backward, or with her stern foremost.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stern-wheel</ent><br/
<hw>Stern"-wheel`</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Having a paddle wheel at the stern; <as>as, a <ex>stern-wheel</ex> steamer</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stern-wheeler</ent><br/
<hw>Stern"-wheel`er</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A steamboat having a stern wheel instead of side wheels.</def> <mark>[Colloq. U.S.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sterquilinous</ent><br/
<hw>Ster*quil"i*nous</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>sterquilinium</ets> a dung pit, fr. <ets>stercus</ets> dung.]</ety> <def>Pertaining to a dunghill; hence, mean; dirty; paltry.</def> <mark>[Obs.]</mark>  <rj><au>Howell.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sterre</ent><br/
<hw>Ster"re</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A star.</def> <mark>[Obs.]</mark>  <rj><au>Chaucer.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sterrink</ent><br/
<hw>Ster"rink</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Zool.)</fld> <def>The crab-eating seal (<spn>Lobodon carcinophaga</spn>) of the Antarctic Ocean.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sterrometal</ent><br/
<hw>Ster"ro*met`al</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Gr. <?/ firm, solid + E. <ets>metal</ets>.]</ety> <def>Any alloy of copper, zinc, tin, and iron, of which cannon are sometimes made.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stert</ent><br/
<hw>Stert</hw> <pr>(?)</pr>, <mark>obs.</mark> <pos>p. p.</pos> <mord>of <er>Start</er></mord>. <def>Started.</def>  <rj><au>Chaucer.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sterte</ent><br/
<hw>Ster"te</hw> <pr>(?)</pr>, <mark>obs.</mark> <def><pos>p. p.</pos> of <er>Start</er>.</def>  <rj><au>Chaucer.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stertorious</ent><br/
<hw>Ster*to"ri*ous</hw> <pr>(st<etil/r*t<omac/"r<icr/*<ucr/s)</pr>, <pos>a.</pos> <def>Stertorous.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stertorous</ent><br/
<hw>Ster"to*rous</hw> <pr>(st<etil/r"t<osl/*r<ucr/s)</pr>, <pos>a.</pos> <ety>[L. <ets>stertere</ets> to snore: cf. F. <ets>stertoreux</ets>.]</ety> <def>Characterized by a deep snoring, which accompanies inspiration in some diseases, especially apoplexy; hence, hoarsely breathing; snoring.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Burning, <qex>stertorous</qex> breath that hurt her cheek.</q> <rj><qau>Mrs. Browning.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>The day has ebbed away, and it is night in his room, before his <qex>stertorous</qex> breathing lulls.</q> <rj><qau>Dickens.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sterve</ent><br/
<hw>Sterve</hw> <pr>(st<etil/rv)</pr>, <pos>v. t. & i.</pos> <def>To die, or cause to die; to perish. See <er>Starve</er>.</def> <mark>[Obs.]</mark>  <rj><au>Chaucer.</au>  <au>Spenser.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stet</ent><br/
<hw>Stet</hw> <pr>(st<ecr/t)</pr>, <ety>L., <pos>subj. 3d pers. sing.</pos> of <ets>stare</ets> to stand, remain.</ety> <ety>[See <er>Stand</er>.]</ety> <fld>(Print.)</fld> <def>Let it stand; -- a word used by proof readers to signify that something once erased, or marked for omission, is to remain.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stet</ent><br/
<hw>Stet</hw>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Stetted</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Stetting</conjf>.]</vmorph> <fld>(Print.)</fld> <def>To cause or direct to remain after having been marked for omission; to mark with the word <xex>stet</xex>, or with a series of dots below or beside the matter; <as>as, the proof reader <ex>stetted</ex> a deled footnote</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stethal</ent><br/
<hw>Steth"al</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[<ets>Ste</ets>aric + <ets>ethal</ets>.]</ety> <fld>(Chem.)</fld> <def>One of the higher alcohols of the methane series, homologous with ethal, and found in small quantities as an ethereal salt of stearic acid in spermaceti.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stethograph</ent><br/
<hw>Steth"o*graph</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Gr. <?/ the breast + <ets>-graph</ets>.]</ety> <fld>(Physiol.)</fld> <def>See <er>Pneumatograph</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stethometer</ent><br/
<hw>Ste*thom"e*ter</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Gr. <?/ chest + <ets>-meter</ets>.]</ety> <fld>(Physiol.)</fld> <def>An apparatus for measuring the external movements of a given point of the chest wall, during respiration; -- also called <altname>thoracometer</altname>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stethoscope</ent><br/
<hw>Steth"o*scope</hw> <pr>(st<ecr/th"<osl/*sk<omac/p)</pr>, <pos>n.</pos> <ety>[Gr. <grk>sth^qos</grk> the breast + <ets>-scope</ets>: cf. F. <ets>st<eacute/thoscope</ets>.]</ety> <fld>(Med.)</fld> <def>An instrument used in auscultation for examining the organs of the chest, as the heart and lungs, by conveying to the ear of the examiner the sounds produced in the thorax.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stethoscope</ent><br/
<hw>Steth"o*scope</hw>, <pos>v. t.</pos> <def>To auscultate, or examine, with a stethoscope.</def>  <rj><au>M. W. Savage.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stethoscopical</ent><br/
<ent>Stethoscopic</ent><br/
<mhw>{ <hw>Steth`o*scop"ic</hw> <pr>(?)</pr>, <hw>Steth`o*scop"ic*al</hw> <pr>(?)</pr>, }</mhw> <pos>a.</pos> <ety>[Cf. F. <ets>st<eacute/thoscopique</ets>.]</ety> <def>Of or pertaining to a stethoscope; obtained or made by means of a stethoscope.</def> -- <wordforms><wf>Steth`o*scop"ic*al*ly</wf>, <pos>adv.</pos></wordforms><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stethoscopist</ent><br/
<hw>Ste*thos"co*pist</hw> <pr>(?)</pr>, <pos>n.</pos> <def>One skilled in the use of the stethoscope.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stethoscopy</ent><br/
<hw>Ste*thos"co*py</hw> <pr>(?)</pr>, <pos>n.</pos> <def>The art or process of examination by the stethoscope.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Steve</ent><br/
<hw>Steve</hw> <pr>(?)</pr>, <pos>v. t.</pos> <ety>[See <er>Stevedore</er>.]</ety> <def>To pack or stow, as cargo in a ship's hold. See <er>Steeve</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stevedore</ent><br/
<hw>Ste"ve*dore`</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Sp. <ets>estivador</ets> a packer, a stower, fr. <ets>estivar</ets> to pack, to stow, L. <ets>stipare</ets> to press, compress, probably akin to E. <ets>stiff</ets>. See <er>Stiff</er>, <er>Stive</er> to stuff.]</ety> <def>One whose occupation is to load and unload vessels in port; one who stows a cargo in a hold.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Steven</ent><br/
<hw>Ste"ven</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[AS. <ets>stefn</ets>, <ets>stemn</ets>, voice; akin to D. <ets>stem</ets>, G. <ets>stimme</ets>, Goth. <ets>stibna</ets>.]</ety> <sn>1.</sn> <def>Voice; speech; language.</def> <mark>[Obs. or Scot.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>Ye have as merry a <qex>steven</qex><br/
As any angel hath that is in heaven.</q> <rj><qau>Chaucer.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>An outcry; a loud call; a clamor.</def> <mark>[Obs.]</mark>  <rj><au>Spenser.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>To set steven</b></col>, <cd>to make an appointment.</cd> <mark>[Obs.]</mark></cs><br/
[<source>1913 Webster</source>]</p>

<p><q>They <qex>setten steven</qex> for to meet<br/
To playen at the dice.</q> <rj><qau>Chaucer.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stew</ent><br/
<hw>Stew</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Cf. <er>Stow</er>.]</ety> <sn>1.</sn> <def>A small pond or pool where fish are kept for the table; a vivarium.</def> <mark>[Obs. or Prov. Eng.]</mark>  <rj><au>Chaucer. Evelyn.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>An artificial bed of oysters.</def> <mark>[Local, U.S.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stew</ent><br/
<hw>Stew</hw>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Stewed</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Stewing</conjf>.]</vmorph> <ety>[OE. <ets>stuven</ets>, OF. <ets>estuver</ets>, F. <ets><eacute/tuver</ets>, fr. OF. <ets>estuve</ets>, F. <ets><eacute/tuve</ets>, a sweating house, a room heated for a bath; probably of Teutonic origin, and akin to E. <ets>stove</ets>. See <er>Stove</er>, and cf. <er>Stive</er> to stew.]</ety> <def>To boil slowly, or with the simmering or moderate heat; to seethe; to cook in a little liquid, over a gentle fire, without boiling; <as>as, to <ex>stew</ex> meat; to <ex>stew</ex> oysters; to <ex>stew</ex> apples</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><-- p. 1413 --></p>

<p><ent>Stew</ent><br/
<hw>Stew</hw> <pr>(?)</pr>, <pos>v. i.</pos> <def>To be seethed or cooked in a slow, gentle manner, or in heat and moisture.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stew</ent><br/
<hw>Stew</hw>, <pos>n.</pos> <ety>[OE. <ets>stue</ets>, <ets>stuwe</ets>, OF. <ets>estuve</ets>. See <er>Stew</er>, <pos>v. t.</pos>]</ety> <sn>1.</sn> <def>A place of stewing or seething; a place where hot bathes are furnished; a hothouse.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>As burning Aetna from his boiling <qex>stew</qex><br/
Doth belch out flames.</q> <rj><qau>Spenser.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>The Lydians were inhibited by Cyrus to use any armor, and give themselves to baths and <qex>stews</qex>.</q> <rj><qau>Abp. Abbot.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A brothel; -- usually in the plural.</def>  <rj><au>Bacon. South.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>There be that hate harlots, and never were at the <qex>stews</qex>.</q> <rj><qau>Aschman.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>A prostitute.</def> <mark>[Obs.]</mark>  <rj><au>Sir A. Weldon.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>A dish prepared by stewing; <as>as, a <ex>stew</ex>of pigeons</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <def>A state of agitating excitement; a state of worry; confusion; <as>as, to be in a <ex>stew</ex></as>.</def> <mark>[Colloq.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Steward</ent><br/
<hw>Stew"ard</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[OE. <ets>stiward</ets>, AS. <ets>st<imac/weard</ets>, <ets>stigweard</ets>, literally, a sty ward; <ets>stigu</ets> sty + <ets>weard</ets> warden, guardian, -- his first duty having been probably to attend to the domestic animals. <root/164. See <er>Sty</er> pen for swine, <er>Ward</er>.]</ety> <sn>1.</sn> <def>A man employed in a large family, or on a large estate, to manage the domestic concerns, supervise other servants, collect the rents or income, keep accounts, and the like.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Worthy to be <qex>stewards</qex> of rent and land.</q> <rj><qau>Chaucer.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>They came near to the <qex>steward</qex> of Joseph's house.</q> <rj><qau>Gen. xliii. 19.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>As good <qex>stewards</qex> of the manifold grace of God.</q> <rj><qau>1 Pet. iv. 10.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A person employed in a hotel, or a club, or on board a ship, to provide for the table, superintend the culinary affairs, etc. In naval vessels, the captain's <xex>steward</xex>, wardroom <xex>steward</xex>, steerage <xex>steward</xex>, warrant officers <xex>steward</xex>, etc., are petty officers who provide for the messes under their charge.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>A fiscal agent of certain bodies; <as>as, a <ex>steward</ex> in a Methodist church</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>In some colleges, an officer who provides food for the students and superintends the kitchen; also, an officer who attends to the accounts of the students.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <def>In Scotland, a magistrate appointed by the crown to exercise jurisdiction over royal lands.</def>  <rj><au>Erskine.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Lord high steward</b></col>, <cd>formerly, the first officer of the crown; afterward, an officer occasionally appointed, as for a coronation, or upon the trial of a peer.</cd> <mark>[Eng.]</mark></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Steward</ent><br/
<hw>Stew"ard</hw>, <pos>v. t.</pos> <def>To manage as a steward.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stewardess</ent><br/
<hw>Stew"ard*ess</hw>, <pos>n.</pos> <def>A female steward; specifically, a woman employed in passenger vessels to attend to the wants of female passengers.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stewardly</ent><br/
<hw>Stew"ard*ly</hw>, <pos>adv.</pos> <def>In a manner, or with the care, of a steward.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>To be <qex>stewardly</qex> dispensed, not wastefully spent.</q> <rj><qau>Tooker.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stewardship</ent><br/
<hw>Stew"ard*ship</hw>, <pos>n.</pos> <def>The office of a steward.</def>  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stewartry</ent><br/
<hw>Stew"art*ry</hw> <pr>(?)</pr>, <pos>n.</pos> <sn>1.</sn> <def>An overseer or superintendent.</def> <mark>[R.]</mark> <ldquo/The <xex>stewartry</xex> of provisions.<rdquo/  <rj><au>Tooke.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>The office of a steward; stewardship.</def> <mark>[R.]</mark>  <rj><au>Byron.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>In Scotland, the jurisdiction of a steward; also, the lands under such jurisdiction.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stewish</ent><br/
<hw>Stew"ish</hw>, <pos>a.</pos> <def>Suiting a stew, or brothel.</def>  <rj><au>Bp. Hall.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stewpan</ent><br/
<hw>Stew"pan`</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A pan used for stewing.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stewpot</ent><br/
<hw>Stew"pot`</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A pot used for stewing.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stey</ent><br/
<hw>Stey</hw> <pr>(?)</pr>, <pos>n.</pos> <def>See <er>Stee</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sthenic</ent><br/
<hw>Sthen"ic</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[Gr. <?/ strength: cf. F. <ets>sth<eacute/nique</ets>.]</ety> <fld>(Med.)</fld> <def>Strong; active; -- said especially of morbid states attended with excessive action of the heart and blood vessels, and characterized by strength and activity of the muscular and nervous system; <as>as, a <ex>sthenic</ex> fever</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Sthenic theory</b></col>. <cd>See <er>Stimulism</er> <sd>(a)</sd>.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stiacciato</ent><br/
||<hw>Sti*ac*cia"to</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[It., crushed, flattened.]</ety> <fld>(Sculp.)</fld> <def>The lowest relief, -- often used in Italian sculpture of the 15th and 16th centuries.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stian</ent><br/
<hw>Sti"an</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A sty on the eye. See <er>Styan</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stibborn</ent><br/
<hw>Stib"born</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Stubborn.</def> <mark>[Obs.]</mark>  <rj><au>Chaucer.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stibial</ent><br/
<hw>Stib"i*al</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[See <er>Stibium</er>.]</ety> <def>Like, or having the qualities of, antimony; antimonial.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stibialism</ent><br/
<hw>Stib"i*al*ism</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Med.)</fld> <def>Antimonial intoxication or poisoning.</def>  <rj><au>Dunglison.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stibiated</ent><br/
<hw>Stib"i*a`ted</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[NL. <ets>stibiatus</ets>, from L. <ets>stibium</ets> antimony.]</ety> <fld>(Med. Chem.)</fld> <def>Combined or impregnated with antimony (<xex>stibium</xex>).</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Stibiated tartar</b></col>. <cd>See <cref>Tartar emetic</cref>, under <er>Tartar</er>.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stibic</ent><br/
<hw>Stib"ic</hw> <pr>(?)</pr>, <pos>a.</pos> <fld>(Chem.)</fld> <def>Antimonic; -- used with reference to certain compounds of antimony.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stibiconite</ent><br/
<hw>Stib"i*co*nite</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Min.)</fld> <def>A native oxide of antimony occurring in masses of a yellow color.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stibine</ent><br/
<hw>Stib"ine</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Chem.)</fld> <def>Antimony hydride, or hydrogen antimonide, a colorless gas produced by the action of nascent hydrogen on antimony. It has a characteristic odor and burns with a characteristic greenish flame. Formerly called also <altname>antimoniureted hydrogen</altname>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stibious</ent><br/
<hw>Stib"i*ous</hw> <pr>(?)</pr>, <pos>a.</pos> <fld>(Chem.)</fld> <def>Antimonious.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stibium</ent><br/
||<hw>Stib"i*um</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>stibium</ets>, <ets>stibi</ets>, Gr. <?/, <?/.]</ety> <sn>1.</sn> <fld>(Chem.)</fld> <def>The technical name of antimony.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Min.)</fld> <def>Stibnite.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stibnite</ent><br/
<hw>Stib"nite</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Min.)</fld> <def>A mineral of a lead-gray color and brilliant metallic luster, occurring in prismatic crystals; sulphide of antimony; -- called also <altname>antimony glance</altname>, and <altname>gray antimony</altname>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stibonium</ent><br/
<hw>Sti*bo"ni*um</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Chem.)</fld> <def>The hypothetical radical <chform>SbH4</chform>, analogous to ammonium; -- called also <altname>antimonium</altname>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sticcado</ent><br/
<hw>Stic*ca"do</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Cf. It. <ets>steccato</ets> a palisade.]</ety> <fld>(Mus.)</fld> <def>An instrument consisting of small bars of wood, flat at the bottom and rounded at the top, and resting on the edges of a kind of open box. They are unequal in size, gradually increasing from the smallest to the largest, and are tuned to the diatonic scale. The tones are produced by striking the pieces of wood with hard balls attached to flexible sticks.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stich</ent><br/
<hw>Stich</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Gr. <grk>sti`chos</grk> a row, line, akin to to go, march, E. <ets>sty</ets>, v. i.]</ety> <sn>1.</sn> <def>A verse, of whatever measure or number of feet.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A line in the Scriptures; specifically <fld>(Hebrew Scriptures)</fld>, one of the rhythmic lines in the poetical books and passages of the Old Treatment, as written in the oldest Hebrew manuscripts and in the Revised Version of the English Bible.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>A row, line, or rank of trees.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stichic</ent><br/
<hw>Stich"ic</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[Gr. <grk>stichiko`s</grk>.]</ety> <def>Of or pertaining to stichs, or lines; consisting of stichs, or lines.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stichidium</ent><br/
||<hw>Sti*chid"i*um</hw> <pr>(?)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Stichida</plw> <pr>(#)</pr>.</plu> <ety>[NL., fr. Gr. <?/, dim. of <?/ a row.]</ety> <fld>(Bot.)</fld> <def>A special podlike or fusiform branch containing tetraspores. It is found in certain red algae.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stichomancy</ent><br/
<hw>Stich"o*man`cy</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Gr. <?/ a line + <ets>-mancy</ets>.]</ety> <def>Divination by lines, or passages of books, taken at hazard.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stichometrical</ent><br/
<hw>Stich`o*met"ric*al</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Of or pertaining to stichometry; characterized by stichs, or lines.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stichometry</ent><br/
<hw>Stich*om"e*try</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Gr. <?/ a line + <ets>-metry</ets>.]</ety> <sn>1.</sn> <def>Measurement of books by the number of lines which they contain.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Division of the text of a book into lines; especially, the division of the text of books into lines accommodated to the sense, -- a method of writing manuscripts used before punctuation was adopted.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stichwort</ent><br/
<hw>Stich"wort`</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Bot.)</fld> <def>A kind of chickweed (<spn>Stellaria Holostea</spn>).</def> <altsp>[Written also <asp>stitchwort</asp>.]</altsp><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stick</ent><br/
<hw>Stick</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[OE. <ets>sticke</ets>, AS. <ets>sticca</ets>; akin to <ets>stician</ets> to stab, prick, pierce, G. <ets>stecken</ets> a stick, staff, OHG. <ets>steccho</ets>, Icel. <ets>stik</ets> a stick. See <er>Stick</er>, <pos>v. t.</pos>.]</ety> <sn>1.</sn> <def>A small shoot, or branch, separated, as by a cutting, from a tree or shrub; also, any stem or branch of a tree, of any size, cut for fuel or timber.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Withered <qex>sticks</qex> to gather, which might serve<br/
Against a winter's day.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Any long and comparatively slender piece of wood, whether in natural form or shaped with tools; a rod; a wand; a staff; <as>as, the <ex>stick</ex> of a rocket; a walking <ex>stick</ex>.</as></def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>Anything shaped like a stick; <as>as, a <ex>stick</ex> of wax</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>A derogatory expression for a person; one who is inert or stupid; <as>as, an odd <ex>stick</ex>; a poor <ex>stick</ex></as>.</def> <mark>[Colloq.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <fld>(Print.)</fld> <def>A composing stick. See under <er>Composing</er>. It is usually a frame of metal, but for posters, handbills, etc., one made of wood is used.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>6.</sn> <def>A thrust with a pointed instrument; a stab.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>A stick of eels</b></col>, <cd>twenty-five eels.</cd> <mark>[Prov. Eng.]</mark> -- <col><b>Stick chimney</b></col>, <cd>a chimney made of sticks laid crosswise, and cemented with clay or mud, as in some log houses.</cd> <mark>[U.S.]</mark> -- <col><b>Stick insect</b></col>, <fld>(Zool.)</fld>, <cd>any one of various species of wingless orthopterous insects of the family <fam>Phasmidae</fam>, which have a long round body, resembling a stick in form and color, and long legs, which are often held rigidly in such positions as to make them resemble small twigs. They thus imitate the branches and twigs of the trees on which they live. The common American species is <spn>Diapheromera femorata</spn>. Some of the Asiatic species are more than a foot long.</cd> -- <mcol><col><b>To cut one's stick</b></col>, <it>or</it> <col><b>To cut stick</b></col></mcol>, <cd>to run away.</cd> <mark>[Slang]</mark> <au>De Quincey.</au></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stick</ent><br/
<hw>Stick</hw>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Stuck</conjf> <pr>(?)</pr> (<mark>Obs.</mark> <conjf>Sticked</conjf> <pr>(<?/)</pr>); <pos>p. pr. & vb. n.</pos> <conjf>Sticking</conjf>.]</vmorph> <ety>[OE. <ets>stikien</ets>, v.t. & i., combined with <ets>steken</ets>, whence E. <ets>stuck</ets>), AS. <ets>stician</ets>, v.t. & i., and (assumed) <ets>stecan</ets>, v.t.; akin to OFries. <ets>steka</ets>, OS. <ets>stekan</ets>, OHG. <ets>stehhan</ets>, G. <ets>stechen</ets>, and to Gr. <?/ to prick, Skr. <ets>tij</ets> to be sharp.  Cf. <er>Distinguish</er>, <er>Etiquette</er>, <er>Extinct</er>, <er>Instigate</er>, <er>Instinct</er>, <er>Prestige</er>, <er>Stake</er>, <er>Steak</er>, <er>Stick</er>, <pos>n.</pos>, <er>Stigma</er>, <er>Stimulate</er>, <er>Sting</er>, <er>Stitch</er> in sewing, <er>Style</er> for or in writing.]</ety> <sn>1.</sn> <def>To penetrate with a pointed instrument; to pierce; to stab; hence, to kill by piercing; <as>as, to <ex>stick</ex> a beast</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>And <qex>sticked</qex> him with bodkins anon.</q> <rj><qau>Chaucer.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>It was a shame . . . to <qex>stick</qex> him under the other gentleman's arm while he was redding the fray.</q> <rj><qau>Sir W. Scott.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To cause to penetrate; to push, thrust, or drive, so as to pierce; <as>as, to <ex>stick</ex> a needle into one's finger</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Thou <qex>stickest</qex> a dagger in me.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>To fasten, attach, or cause to remain, by thrusting in; hence, also, to adorn or deck with things fastened on as by piercing; <as>as, to <ex>stick</ex> a pin on the sleeve</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>My shroud of white, <qex>stuck</qex> all with yew.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>The points of spears are <qex>stuck</qex> within the shield.</q> <rj><qau>Dryden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>To set; to fix in; <as>as, to <ex>stick</ex> card teeth</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <def>To set with something pointed; <as>as, to <ex>stick</ex> cards</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>6.</sn> <def>To fix on a pointed instrument; to impale; <as>as, to <ex>stick</ex> an apple on a fork</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>7.</sn> <def>To attach by causing to adhere to the surface; <as>as, to <ex>stick</ex> on a plaster; to <ex>stick</ex> a stamp on an envelope; also, to attach in any manner</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>8.</sn> <fld>(Print.)</fld> <def>To compose; to set, or arrange, in a composing stick; <as>as, to <ex>stick</ex> type</as>.</def> <mark>[Cant]</mark><br/
[<source>1913 Webster</source>]</p>

<p><sn>9.</sn> <fld>(Joinery)</fld> <def>To run or plane (moldings) in a machine, in contradistinction to working them by hand. Such moldings are said to be <xex>stuck</xex>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>10.</sn> <def>To cause to stick; to bring to a stand; to pose; to puzzle; <as>as, to <ex>stick</ex> one with a hard problem</as>.</def> <mark>[Colloq.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><sn>11.</sn> <def>To impose upon; to compel to pay; sometimes, to cheat.</def> <mark>[Slang]</mark><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>To stick out</b></col>, <cd>to cause to project or protrude; to render prominent.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stick</ent><br/
<hw>Stick</hw> <pr>(?)</pr>, <pos>v. i.</pos> <sn>1.</sn> <def>To adhere; <as>as, glue <ex>sticks</ex> to the fingers; paste <ex>sticks</ex> to the wall</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The green caterpillar breedeth in the inward parts of roses not blown, where the dew <qex>sticketh</qex>.</q> <rj><qau>Bacon.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To remain where placed; to be fixed; to hold fast to any position so as to be moved with difficulty; to cling; to abide; to cleave; to be united closely.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>A friend that <qex>sticketh</qex> closer than a brother.</q> <rj><qau>Prov. xviii. 24.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>I am a kind of bur; I shall <qex>stick</qex>.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>If on your fame our sex a bolt has thrown,<br/
'T will ever <qex>stick</qex> through malice of your own.</q> <rj><qau>Young.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>To be prevented from going farther; to stop by reason of some obstacle; to be stayed.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>I had most need of blessing, and <ldquo/Amen<rdquo/<br/
<qex>Stuck</qex> in my throat.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>The trembling weapon passed<br/
Through nine bull hides, . . . and <qex>stuck</qex> within the last.</q> <rj><qau>Dryden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>To be embarrassed or puzzled; to hesitate; to be deterred, as by scruples; to scruple; -- often with <xex>at</xex>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>They will <qex>stick</qex> long at part of a demonstration for want of perceiving the connection of two ideas.</q> <rj><qau>Locke.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Some <qex>stick</qex> not to say, that the parson and attorney forged a will.</q> <rj><qau>Arbuthnot.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <def>To cause difficulties, scruples, or hesitation.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>This is the difficulty that <qex>sticks</qex> with the most reasonable.</q> <rj><qau>Swift.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>To stick by</b></col>. <sd>(a)</sd> <cd>To adhere closely to; to be firm in supporting</cd>. <ldquo/We are your only friends; <xex>stick by</xex> us, and we will <xex>stick by</xex> you.<rdquo/ <au>Davenant.</au> <sd>(b)</sd> <cd>To be troublesome by adhering.</cd> <ldquo/I am satisfied to trifle away my time, rather than let it <xex>stick by</xex> me.<rdquo/ <au>Pope.</au> -- <col><b>To stick out</b></col>. <sd>(a)</sd> <cd>To project; to be prominent.</cd> <ldquo/His bones that were not seen <xex>stick out</xex>.<rdquo/ <au>Job xxxiii. 21.</au> <sd>(b)</sd> <cd>To persevere in a purpose; to hold out; as, the garrison <xex>stuck out</xex> until relieved.</cd> <mark>[Colloq.]</mark><-- also <pos>v. i.</pos> to stick it out. --> -- <col><b>To stick to</b></col>, <cd>to be persevering in holding to; <as>as, <ex>to stick to</ex> a party or cause</as>.</cd> <ldquo/The advantage will be on our side if we <xex>stick to</xex> its essentials.<rdquo/ <au>Addison.</au> -- <col><b>To stick up</b></col>, <cd>to stand erect; <as>as, his hair <ex>sticks up</ex></as>.</cd> -- <col><b>To stick up for</b></col>, <cd>to assert and defend; <as>as, <ex>to stick up for</ex> one's rights or for a friend</as>.</cd> <mark>[Colloq.]</mark> -- <col><b>To stick upon</b></col>, <cd>to dwell upon; not to forsake.</cd> <ldquo/If the matter be knotty, the mind must stop and buckle to it, and <xex>stick upon</xex> it with labor and thought.<rdquo/ <au>Locke.</au></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sticked</ent><br/
<hw>Stick"ed</hw> <pr>(?)</pr>, <mark>obs.</mark> <pos>imp.</pos> <mord>of <er>Stick</er></mord>. <def>Stuck.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>And in the sand her ship <qex>sticked</qex> so fast.</q> <rj><qau>Chaucer.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>They <qex>sticked</qex> not to give their bodies to be burnt.</q> <rj><qau>Sir T. Browne.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sticker</ent><br/
<hw>Stick"er</hw> <pr>(?)</pr>, <pos>n.</pos> <sn>1.</sn> <def>One who, or that which, sticks; <as>as, a bill <ex>sticker</ex></as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>That which causes one to stick; that which puzzles or poses.</def> <mark>[Colloq.]</mark>  <rj><au>Tackeray.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <fld>(Mus.)</fld> <def>In the organ, a small wooden rod which connects (in part) a key and a pallet, so as to communicate motion by pushing.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>Same as <er>Paster</er>, 2.</def> <mark>[Political Cant, U.S.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stickful</ent><br/
<hw>Stick"ful</hw> <pr>(?)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Stickfuls</plw> <pr>(<?/)</pr>.</plu> <fld>(Print.)</fld> <def>As much set type as fills a composing stick.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stickiness</ent><br/
<hw>Stick"i*ness</hw> <pr>(?)</pr>, <pos>n.</pos> <def>The quality of being sticky; <as>as, the <ex>stickiness</ex> of glue or paste</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sticking</ent><br/
<hw>Stick"ing</hw>, <def><pos>a. & n.</pos> from <er>Stick</er>, <pos>v.</pos></def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Sticking piece</b></col>, <cd>a piece of beef cut from the neck.</cd> <mark>[Eng.]</mark> -- <col><b>Sticking place</b></col>, <cd>the place where a thing sticks, or remains fast; sticking point.</cd><br/
[<source>1913 Webster</source>]</p>

<p><q>But screw your courage to the <qex>sticking place</qex>,<br/
And we'll not fail.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p>-- <col><b>Sticking plaster</b></col>, <cd>an adhesive plaster for closing wounds, and for similar uses.</cd> -- <col><b>Sticking point</b></col>. <cd>Same as <cref>Sticking place</cref>, above.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stickit</ent><br/
<hw>Stick"it</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Stuck; spoiled in making.</def> <mark>[Scot.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Stickit minister</b></col>, <cd>a candidate for the clerical office who fails, disqualified by incompetency or immorality.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stick-lac</ent><br/
<hw>Stick"-lac`</hw> <pr>(?)</pr>, <pos>n.</pos> <def>See the Note under <er>Lac</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stickle</ent><br/
<hw>Stic"kle</hw> <pr>(?)</pr>, <pos>v. i.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Stickled</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Stickling</conjf>.]</vmorph> <ety>[Probably fr. OE. <ets>stightlen</ets>, <ets>sti<?/tlen</ets>, to dispose, arrange, govern, freq. of <ets>stihten</ets>, AS. <ets>stihtan</ets>: cf. G. <ets>stiften</ets> to found, to establish.]</ety> <sn>1.</sn> <def>To separate combatants by intervening.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>When he [the angel] sees half of the Christians killed, and the rest in a fair way of being routed, he <qex>stickles</qex> betwixt the remainder of God's host and the race of fiends.</q> <rj><qau>Dryden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To contend, contest, or altercate, esp. in a pertinacious manner on insufficient grounds.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Fortune, as she 's wont, turned fickle,<br/
And for the foe began to <qex>stickle</qex>.</q> <rj><qau>Hudibras.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>While for paltry punk they roar and <qex>stickle</qex>.</q> <rj><qau>Dryden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>The obstinacy with which he <qex>stickles</qex> for the wrong.</q> <rj><qau>Hazlitt.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>To play fast and loose; to pass from one side to the other; to trim.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stickle</ent><br/
<hw>Stic"kle</hw>, <pos>v. t.</pos> <sn>1.</sn> <def>To separate, as combatants; hence, to quiet, to appease, as disputants.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>Which [question] violently they pursue,<br/
Nor <qex>stickled</qex> would they be.</q> <rj><qau>Drayton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To intervene in; to stop, or put an end to, by intervening; hence, to arbitrate.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>They ran to him, and, pulling him back by force, <qex>stickled</qex> that unnatural fray.</q> <rj><qau>Sir P. Sidney.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stickle</ent><br/
<hw>Stic"kle</hw>, <pos>n.</pos> <ety>[Cf. <er>stick</er>, <pos>v. t. & i.</pos>]</ety> <def>A shallow rapid in a river; also, the current below a waterfall.</def> <mark>[Obs. or Prov. Eng.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>Patient anglers, standing all the day<br/
Near to some shallow <qex>stickle</qex> or deep bay.</q> <rj><qau>W. Browne.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stickleback</ent><br/
<hw>Stic"kle*back`</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[OE. & Prov E. <ets>stickle</ets> a prickle, spine, sting (AS. <ets>sticel</ets>) + <ets>back</ets>. See <er>Stick</er>, <pos>v. t.</pos>, and cf. <er>Banstickle</er>.]</ety> <fld>(Zool.)</fld> <def>Any one of numerous species of small fishes of the genus <gen>Gasterosteus</gen> and allied genera. The back is armed with two or more sharp spines. They inhabit both salt and brackish water, and construct curious nests. Called also <altname>sticklebag</altname>, <altname>sharpling</altname>, and <altname>prickleback</altname>.</def><br/
[<source>1913 Webster</source>]</p>

<p><-- p. 1414 --></p>

<p><ent>Stickler</ent><br/
<hw>Stic"kler</hw> <pr>(st<icr/k"kl<etil/r)</pr>, <pos>n.</pos> <ety>[See <er>Stickle</er>, <pos>v. t.</pos>]</ety> <def>One who stickles.</def> Specifically: --<br/
[<source>1913 Webster</source>]</p>

<p><sd>(a)</sd> <def>One who arbitrates a duel; a sidesman to a fencer; a second; an umpire.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>Basilius, the judge, appointed <qex>sticklers</qex> and trumpets whom the others should obey.</q> <rj><qau>Sir P. Sidney.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Our former chiefs, like <qex>sticklers</qex> of the war,<br/
First sought to inflame the parties, then to poise.</q> <rj><qau>Dryden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sd>(b)</sd> <def>One who pertinaciously contends for some trifling things, as a point of etiquette; an unreasonable, obstinate contender; <as>as, a <ex>stickler</ex> for ceremony</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The Tory or High-church were the greatest <qex>sticklers</qex> against the exorbitant proceedings of King James II.</q> <rj><qau>Swift.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stick-seed</ent><br/
<hw>Stick"-seed`</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Bot.)</fld> <def>A plant (<spn>Echinospermum Lappula</spn>) of the Borage family, with small blue flowers and prickly nutlets.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sticktail</ent><br/
<hw>Stick"tail`</hw> <pr>(?)</pr>, <pos>n.</pos> <def>The ruddy duck.</def> <mark>[Local, U.S.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>stick-tight</ent><br/
<hw>stick"-tight`</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Bot.)</fld> <sn>1.</sn> <def>any of several plants having seeds tipped with barbs that may cling to fur or clothing, especially those of the genus <gen>Bidens</gen>, also called <altname>bur marigold</altname>, <altname>beggar-ticks</altname>, and <altname>Beggar's ticks</altname>, which have prickly flattened achenes.</def> <au>RHUD</au><br/
[<source>1913 Webster</source> <source>+PJC</source>]</p>

<p><sn>2.</sn> <def>a seed from a sticktight{1}.</def><br/
[<source>PJC</source>]</p>

<p><ent>Sticky</ent><br/
<hw>Stick"y</hw> <pr>(?)</pr>, <pos>a.</pos> <amorph>[<pos>Compar.</pos> <adjf>Stickier</adjf> <pr>(?)</pr>; <pos>superl.</pos> <adjf>Stickiest</adjf>.]</amorph> <def>Having the quality of sticking to a surface; adhesive; gluey; viscous; viscid; glutinous; tenacious.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Herbs which last longest are those of strong smell, and with a <qex>sticky</qex> stalk.</q> <rj><qau>Bacon.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stiddy</ent><br/
<hw>Stid"dy</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[See <er>Stithy</er>.]</ety> <def>An anvil; also, a smith shop. See <er>Stithy</er>.</def> <mark>[Prov. Eng.]</mark>  <rj><au>Halliwell.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stiff</ent><br/
<hw>Stiff</hw> <pr>(?)</pr>, <pos>a.</pos> <amorph>[<pos>Compar.</pos> <adjf>Stiffer</adjf> <pr>(?)</pr>; <pos>superl.</pos> <adjf>Stiffest</adjf>.]</amorph> <ety>[OE. <ets>stif</ets>, AS. <ets>st<imac/f</ets>; akin to D. <ets>stijf</ets>, G. <ets>steif</ets>, Dan. <ets>stiv</ets>, Sw. <ets>styf</ets>, Icel. <ets>st<imac/fr</ets>, Lith. <ets>stipti</ets> to be stiff; cf. L. <ets>stipes</ets> a post, trunk of a tree, <ets>stipare</ets> to press, compress.  Cf. <er>Costive</er>, <er>Stifle</er>, <er>Stipulate</er>, <er>Stive</er> to stuff.]</ety> <sn>1.</sn> <def>Not easily bent; not flexible or pliant; not limber or flaccid; rigid; firm; <as>as, <ex>stiff</ex> wood, paper, joints</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>[They] rising on <qex>stiff</qex> pennons, tower<br/
The mid aerial sky.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Not liquid or fluid; thick and tenacious; inspissated; neither soft nor hard; <as>as, the paste is <ex>stiff</ex></as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>Firm; strong; violent; difficult to oppose; <as>as, a <ex>stiff</ex> gale or breeze</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>Not easily subdued; unyielding; stubborn; obstinate; pertinacious; <as>as, a <ex>stiff</ex> adversary</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>It is a shame to stand <qex>stiff</qex> in a foolish argument.</q> <rj><qau>Jer. Taylor.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>A war ensues: the Cretans own their cause,<br/
<qex>Stiff</qex> to defend their hospitable laws.</q> <rj><qau>Dryden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <def>Not natural and easy; formal; constrained; affected; starched; <as>as, <ex>stiff</ex> behavior; a <ex>stiff</ex> style</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The French are open, familiar, and talkative; the Italians <qex>stiff</qex>, ceremonious, and reserved.</q> <rj><qau>Addison.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>6.</sn> <def>Harsh; disagreeable; severe; hard to bear.</def> <mark>[Obs. or Colloq.]</mark> <ldquo/This is <xex>stiff</xex> news.<rdquo/  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>7.</sn> <fld>(Naut.)</fld> <def>Bearing a press of canvas without careening much; <as>as, a <ex>stiff</ex> vessel</as>; -- opposed to <contr>crank</contr>.</def>  <rj><au>Totten.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>8.</sn> <def>Very large, strong, or costly; powerful; <as>as, a <ex>stiff</ex> charge; a <ex>stiff</ex> price</as>.</def> <mark>[Slang]</mark><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Stiff neck</b></col>, <cd>a condition of the neck such that the head can not be moved without difficulty and pain.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><syn><b>Syn.</b> -- Rigid; inflexible; strong; hardly; stubborn; obstinate; pertinacious; harsh; formal; constrained; affected; starched; rigorous.</syn><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stiff-backed</ent><br/
<hw>Stiff"-backed`</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Obstinate.</def>  <rj><au>J. H. Newman.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stiffen</ent><br/
<hw>Stiff"en</hw> <pr>(?)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Stiffened</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Stiffening</conjf>.]</vmorph> <ety>[See <er>Stiff</er>.]</ety> <sn>1.</sn> <def>To make stiff; to make less pliant or flexible; <as>as, to <ex>stiffen</ex> cloth with starch</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q><qex>Stiffen</qex> the sinews, summon up the blood.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To inspissate; to make more thick or viscous; <as>as, to <ex>stiffen</ex> paste</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>To make torpid; to benumb.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stiffen</ent><br/
<hw>Stiff"en</hw>, <pos>v. i.</pos> <def>To become stiff or stiffer, in any sense of the adjective.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Like bristles rose my <qex>stiffening</qex> hair.</q> <rj><qau>Dryden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>The tender soil then <qex>stiffening</qex> by degrees.</q> <rj><qau>Dryden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Some souls we see,<br/
Grow hard and <qex>stiffen</qex> with adversity.</q> <rj><qau>Dryden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stiffener</ent><br/
<hw>Stiff"en*er</hw> <pr>(?)</pr>, <pos>n.</pos> <def>One who, or that which, stiffens anything, as a piece of stiff cloth in a cravat.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stiffening</ent><br/
<hw>Stiff"en*ing</hw>, <pos>n.</pos> <sn>1.</sn> <def>Act or process of making stiff.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Something used to make anything stiff.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Stiffening order</b></col> <fld>(Com.)</fld>, <cd>a permission granted by the customs department to take cargo or ballast on board before the old cargo is out, in order to steady the ship.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stiff-hearted</ent><br/
<hw>Stiff"-heart`ed</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[<ets>Stiff</ets> + <ets>heart</ets>.]</ety> <def>Obstinate; stubborn; contumacious.</def>  <rj><au>Ezek. ii. 4.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stiffish</ent><br/
<hw>Stiff"ish</hw>, <pos>a.</pos> <def>Somewhat stiff.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stiffly</ent><br/
<hw>Stiff"ly</hw> <pr>(?)</pr>, <pos>adv.</pos> <def>In a stiff manner.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stiff-necked</ent><br/
<hw>Stiff"-necked`</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Stubborn; inflexibly obstinate; contumacious; <as>as, <ex>stiff-necked</ex> pride; a <ex>stiff-necked</ex> people.</as></def>  <rj><au>Ex. xxxii. 9.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stiff-neckedness</ent><br/
<hw>Stiff"-neck`ed*ness</hw> <pr>(?)</pr>, <pos>n.</pos> <def>The quality or state of being stiff-necked; stubbornness.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stiffness</ent><br/
<hw>Stiff"ness</hw>, <pos>n.</pos> <def>The quality or state of being stiff; <as>as, the <ex>stiffness</ex> of cloth or of paste; <ex>stiffness</ex> of manner; <ex>stiffness</ex> of character.</as></def><br/
[<source>1913 Webster</source>]</p>

<p><q>The vices of old age have the <qex>stiffness</qex> of it too.</q> <rj><qau>South.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stifftail</ent><br/
<hw>Stiff"tail`</hw> <pr>(?)</pr>, <pos>n.</pos> <def>The ruddy duck.</def> <mark>[Local, U.S.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stiff-tailed</ent><br/
<hw>Stiff"-tailed`</hw> <pr>(?)</pr>, <pos>a.</pos> <fld>(Zool.)</fld> <def>Having the quill feathers of the tail somewhat rigid.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stifle</ent><br/
<hw>Sti"fle</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[From <er>Stiff</er>.]</ety> <fld>(Far.)</fld> <def>The joint next above the hock, and near the flank, in the hind leg of the horse and allied animals; the joint corresponding to the knee in man; -- called also <altname>stifle joint</altname>. See <xex>Illust.</xex> under <er>Horse</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Stifle bone</b></col>, <cd>a small bone at the stifle joint; the patella, or kneepan.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stifle</ent><br/
<hw>Sti"fle</hw>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Stifled</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Stifling</conjf> <pr>(?)</pr>.]</vmorph> <ety>[Freq. of OE. <ets>stif</ets> stiff; cf. Icel. <ets>st<imac/fla</ets> to dam up.]</ety> <sn>1.</sn> <def>To stop the breath of by crowding something into the windpipe, or introducing an irrespirable substance into the lungs; to choke; to suffocate; to cause the death of by such means; <as>as, to <ex>stifle</ex> one with smoke or dust</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q><qex>Stifled</qex> with kisses, a sweet death he dies.</q> <rj><qau>Dryden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>I took my leave, being half <qex>stifled</qex> with the closeness of the room.</q> <rj><qau>Swift.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To stop; to extinguish; to deaden; to quench; <as>as, to <ex>stifle</ex> the breath; to <ex>stifle</ex> a fire or flame</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Bodies . . . <qex>stifle</qex> in themselves the rays which they do not reflect or transmit.</q> <rj><qau>Sir I. Newton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>To suppress the manifestation or report of; to smother; to conceal from public knowledge; <as>as, to <ex>stifle</ex> a story; to <ex>stifle</ex> passion</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>I desire only to have things fairly represented as they really are; no evidence smothered or <qex>stifled</qex>.</q> <rj><qau>Waterland.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stifle</ent><br/
<hw>Sti"fle</hw> <pr>(?)</pr>, <pos>v. i.</pos> <def>To die by reason of obstruction of the breath, or because some noxious substance prevents respiration.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>You shall <qex>stifle</qex> in your own report.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stifled</ent><br/
<hw>Sti"fled</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Stifling.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The close and <qex>stifled</qex> study.</q> <rj><qau>Hawthorne.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stifler</ent><br/
<hw>Sti"fler</hw> <pr>(?)</pr>, <pos>n.</pos> <sn>1.</sn> <def>One who, or that which, stifles.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Mil.)</fld> <def>See <er>Camouflet</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stigma</ent><br/
<hw>Stig"ma</hw> <pr>(?)</pr>, <pos>n.</pos>; <plu><it>pl.</it> E. <plw>Stigmas</plw> <pr>(#)</pr>, L. <plw>Stigmata</plw> <pr>(#)</pr>.</plu> <ety>[L., a mark, a brand, from Gr. <?/, <?/, the prick or mark of a pointed instrument, a spot, mark, from <?/ to prick, to brand. See <er>Stick</er>, <pos>v. t.</pos>]</ety> <sn>1.</sn> <def>A mark made with a burning iron; a brand.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Any mark of infamy or disgrace; sign of moral blemish; stain or reproach caused by dishonorable conduct; reproachful characterization.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The blackest <qex>stigma</qex> that can be fastened upon him.</q> <rj><qau>Bp. Hall.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>All such slaughters were from thence called Bartelmies, simply in a perpetual <qex>stigma</qex> of that butchery.</q> <rj><qau>Sir G. Buck.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <fld>(Bot.)</fld> <def>That part of a pistil which has no epidermis, and is fitted to receive the pollen. It is usually the terminal portion, and is commonly somewhat glutinous or viscid. See <xex>Illust.</xex> of <er>Stamen</er> and of <er>Flower</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <fld>(Anat.)</fld> <def>A small spot, mark, scar, or a minute hole; -- applied especially to a spot on the outer surface of a Graafian follicle, and to spots of intercellular substance in scaly epithelium, or to minute holes in such spots.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <fld>(Pathol.)</fld> <def>A red speck upon the skin, produced either by the extravasation of blood, as in the bloody sweat characteristic of certain varieties of religious ecstasy, or by capillary congestion, as in the case of drunkards.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>6.</sn> <fld>(Zool.)</fld> <sd>(a)</sd> <def>One of the external openings of the tracheae of insects, myriapods, and other arthropods; a spiracle.</def> <sd>(b)</sd> <def>One of the apertures of the pulmonary sacs of arachnids. See <xex>Illust.</xex> of <er>Scorpion</er>.</def> <sd>(c)</sd> <def>One of the apertures of the gill of an ascidian, and of Amphioxus.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>7.</sn> <fld>(Geom.)</fld> <def>A point so connected by any law whatever with another point, called an <xex>index</xex>, that as the index moves in any manner in a plane the first point or stigma moves in a determinate way in the same plane.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>8.</sn> <pluf>pl.</pluf> <fld>(R. C. Ch.)</fld> <def>Marks believed to have been supernaturally impressed upon the bodies of certain persons in imitation of the wounds on the crucified body of Christ. See def. 5, above.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stigmaria</ent><br/
||<hw>Stig*ma"ri*a</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[NL. See <er>Stigma</er>.]</ety> <fld>(Paleon.)</fld> <def>The fossil root stem of a coal plant of the genus <gen>Sigillaria</gen>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stigmata</ent><br/
||<hw>Stig"ma*ta</hw> <pr>(?)</pr>, <pos>n.</pos>; <def><pos>pl.</pos> of <er>Stigma</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stigmatic</ent><br/
<hw>Stig*mat"ic</hw> <pr>(?)</pr>, <pos>n.</pos> <sn>1.</sn> <def>A notorious profligate or criminal who has been branded; one who bears the marks of infamy or punishment.</def> <mark>[R.]</mark>  <rj><au>Bullokar.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A person who is marked or deformed by nature.</def>  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><-- <sn>3.</sn> <def>A person bearing the wounds on the hands and feet resembling those of Jesus Christ caused by His crucifixion; -- for true <ex>stigmantics</ex> the wounds are supposed to have been caused miraculously, as a sign of great holiness.</def> --><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stigmatical</ent><br/
<ent>Stigmatic</ent><br/
<mhw>{ <hw>Stig*mat"ic</hw> <pr>(?)</pr>, <hw>Stig*mat"ic*al</hw> <pr>(?)</pr>, }</mhw> <pos>a.</pos> <ety>[See <er>Stigma</er>.]</ety> <sn>1.</sn> <def>Marked with a stigma, or with something reproachful to character.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Impressing with infamy or reproach.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <fld>(Bot., Anat., etc)</fld> <def>Of or pertaining to a stigma or stigmata.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><mcol><col><b>Stigmatic geometry</b></col>, <it>or</it> <col><b>Stigmatics</b></col></mcol>, <cd>that science in which the correspondence of index and stigma (see <er>Stigma</er>, 7) is made use of to establish geometrical proportions.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stigmatically</ent><br/
<hw>Stig*mat"ic*al*ly</hw>, <pos>adv.</pos> <def>With a stigma, or mark of infamy or deformity.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stigmatist</ent><br/
<hw>Stig"ma*tist</hw> <pr>(?)</pr>, <pos>n.</pos> <def>One believed to be supernaturally impressed with the marks of Christ's wounds. See <er>Stigma</er>, 8.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stigmatization</ent><br/
<hw>Stig`ma*ti*za"tion</hw> <pr>(?)</pr>, <pos>n.</pos> <sn>1.</sn> <def>The act of stigmatizing.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(R. C. Ch.)</fld> <def>The production of stigmata upon the body. See <er>Stigma</er>, 8.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stigmatize</ent><br/
<hw>Stig"ma*tize</hw> <pr>(?)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Stigmatized</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Stigmatizing</conjf> <pr>(?)</pr>.]</vmorph> <ety>[F. <ets>stigmatiser</ets>, Gr. <?/.]</ety> <sn>1.</sn> <def>To mark with a stigma, or brand; <as>as, the ancients <ex>stigmatized</ex> their slaves and soldiers</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>That . . . hold out both their ears with such delight and ravishment, to be <qex>stigmatized</qex> and bored through in witness of their own voluntary and beloved baseness.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To set a mark of disgrace on; to brand with some mark of reproach or infamy.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>To find virtue extolled and vice <qex>stigmatized</qex>.</q> <rj><qau>Addison.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stigmatose</ent><br/
<hw>Stig"ma*tose`</hw> <pr>(?)</pr>, <pos>a.</pos> <fld>(Bot.)</fld> <def>Same as <er>Stigmatic</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stigonomancy</ent><br/
<hw>Stig"o*no*man`cy</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Gr. <?/, <?/, one who is marked, or one who marks (<?/ to mark with a pointed instrument, to prick) + <ets>-mancy</ets>.]</ety> <def>Divination by writing on the bark of a tree.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stike</ent><br/
<hw>Stike</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[See <er>Stich</er>.]</ety> <def>Stanza.</def> <mark>[Obs.]</mark>  <rj><au>Sackville.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stilar</ent><br/
<hw>Sti"lar</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[From <er>Stile</er> a style.]</ety> <def>Of or pertaining to the style of a dial.</def> <altsp>[Written also <asp>stylar</asp>.]</altsp><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stilbene</ent><br/
<hw>Stil"bene</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[See <er>Stilbite</er>.]</ety> <fld>(Chem.)</fld> <def>A hydrocarbon, <chform>C14H12</chform>, produced artificially in large, fine crystals; -- called also <altname>diphenyl ethylene</altname>, <altname>toluylene</altname>, etc.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stilbite</ent><br/
<hw>Stil"bite</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Gr. <?/ to glitter, shine: cf. F. <ets>stilbite</ets>.]</ety> <fld>(Min.)</fld> <def>A common mineral of the zeolite family, a hydrous silicate of alumina and lime, usually occurring in sheaflike aggregations of crystals, also in radiated masses. It is of a white or yellowish color, with pearly luster on the cleavage surface. Called also <altname>desmine</altname>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stile</ent><br/
<hw>Stile</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[See <er>Style</er>.]</ety> <sn>1.</sn> <def>A pin set on the face of a dial, to cast a shadow; a style. See <er>Style</er>.</def>  <rj><au>Moxon.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Mode of composition. See <er>Style</er>.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>May I not write in such a <qex>stile</qex> as this?</q> <rj><qau>Bunyan.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stile</ent><br/
<hw>Stile</hw>, <pos>n.</pos> <ety>[OE. <ets>stile</ets>, AS. <ets>stigel</ets> a step, a ladder, from <ets>st<imac/gan</ets> to ascend; akin to OHG. <ets>stigila</ets> a stile. <root/164. See <er>Sty</er>, <pos>v. i.</pos>, and cf. <er>Stair</er>.]</ety> <sn>1.</sn> <def>A step, or set of steps, for ascending and descending, in passing a fence or wall.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>There comes my master . . . over the <qex>stile</qex>, this way.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Over this <qex>stile</qex> in the way to Doubting Castle.</q> <rj><qau>Bunyan.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Arch.)</fld> <def>One of the upright pieces in a frame; one of the primary members of a frame, into which the secondary members are mortised.</def><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ In an ordinary door the principal upright pieces are called <xex>stiles</xex>, the subordinate upright pieces <xex>mullions</xex>, and the crosspieces <xex>rails</xex>. In wainscoting the principal pieces are sometimes called <xex>stiles</xex>, even when horizontal.</note><br/
[<source>1913 Webster</source>]</p>

<p><cs><mcol><col><b>Hanging stile</b></col>, <col><b>Pulley stile</b></col></mcol>. <cd>See under <er>Hanging</er>, and <er>Pulley</er>.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stilet</ent><br/
<hw>Sti"let</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Written also <ets>stilette</ets>, and <ets>stylet</ets>.]</ety> <sn>1.</sn> <def>A stiletto.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Surg.)</fld> <def>See <er>Stylet</er>, 2.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stiletto</ent><br/
<hw>Sti*let"to</hw> <pr>(?)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Stilettos</plw> <pr>(#)</pr>.</plu> <ety>[It., dim. of <ets>stilo</ets> a dagger, fr. L. <ets>stilus</ets> a pointed instrument. See <er>Style</er> for writing, and cf. <er>Stylet</er>.]</ety> <sn>1.</sn> <def>A kind of dagger with a slender, rounded, and pointed blade.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A pointed instrument for making eyelet holes in embroidery.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>A beard trimmed into a pointed form.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>The very quack of fashions, the very he that<br/
Wears a <qex>stiletto</qex> on his chin.</q> <rj><qau>Ford.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stiletto</ent><br/
<hw>Sti*let"to</hw>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Stilettoed</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Stilettoing</conjf> <pr>(?)</pr>.]</vmorph> <def>To stab or kill with a stiletto.</def>  <rj><au>Bacon.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Still</ent><br/
<hw>Still</hw> <pr>(?)</pr>, <pos>a.</pos> <amorph>[<pos>Compar.</pos> <adjf>Stiller</adjf> <pr>(?)</pr>; <pos>superl.</pos> <adjf>Stillest</adjf>.]</amorph> <ety>[OE. <ets>stille</ets>, AS. <ets>stille</ets>; akin to D. <ets>stil</ets>, OS. & OHG. <ets>stilli</ets>, G. <ets>still</ets>, Dan. <ets>stille</ets>, Sw. <ets>stilla</ets>, and to E. <ets>stall</ets>; from the idea of coming to a stand, or halt.  Cf. <er>Still</er>, <pos>adv.</pos>]</ety> <sn>1.</sn> <def>Motionless; at rest; quiet; <as>as, to stand <ex>still</ex>; to lie or sit <ex>still</ex></as>.</def> <ldquo/<xex>Still</xex> as any stone.<rdquo/  <rj><au>Chaucer.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Uttering no sound; silent; <as>as, the audience is <ex>still</ex>; the animals are <ex>still</ex></as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The sea that roared at thy command,<br/
At thy command was <qex>still</qex>.</q> <rj><qau>Addison.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>Not disturbed by noise or agitation; quiet; calm; <as>as, a <ex>still</ex> evening; a <ex>still</ex> atmosphere</as>.</def> <ldquo/When all the woods are <xex>still</xex>.<rdquo/  <rj><au>Milton.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>Comparatively quiet or silent; soft; gentle; low.</def> <ldquo/A <xex>still</xex> small voice.<rdquo/  <rj><au>1 Kings xix. 12.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <def>Constant; continual.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>By <qex>still</qex> practice learn to know thy meaning.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>6.</sn> <def>Not effervescing; not sparkling; <as>as, <ex>still</ex> wines</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Still life</b></col>. <fld>(Fine Arts)</fld> <sd>(a)</sd> <cd>Inanimate objects</cd>. <sd>(b)</sd> <fld>(Painting)</fld> <cd>The class or style of painting which represents inanimate objects, as fruit, flowers, dead game, etc.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><syn><b>Syn.</b> -- Quiet; calm; noiseless; serene; motionless; inert; stagnant.</syn><br/
[<source>1913 Webster</source>]</p>

<p><ent>Still</ent><br/
<hw>Still</hw>, <pos>n.</pos> <ety>[Cf. G. <ets>stille</ets>.]</ety> <sn>1.</sn> <def>Freedom from noise; calm; silence; <as>as, the <ex>still</ex> of midnight</as>.</def> <mark>[Poetic]</mark><-- the still of the night --><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A steep hill or ascent.</def> <mark>[Obs.]</mark>  <rj><au>W. Browne.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Still</ent><br/
<hw>Still</hw>, <pos>adv.</pos> <ety>[AS. <ets>stille</ets> quietly. See <er>Still</er>, <pos>a.</pos> The modern senses come from the idea of stopping and <ets>staying</ets> still, or motionless.]</ety> <sn>1.</sn> <def>To this time; until and during the time now present; now no less than before; yet.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>It hath been anciently reported, and is <qex>still</qex> received.</q> <rj><qau>Bacon.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>In the future as now and before.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Hourly joys be <qex>still</qex> upon you!</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>In continuation by successive or repeated acts; always; ever; constantly; uniformly.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The desire of fame betrays an ambitious man into indecencies that lessen his reputation; he is <qex>still</qex> afraid lest any of his actions should be thrown away in private.</q> <rj><qau>Addison.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Chemists would be rich if they could <qex>still</qex> do in great quantities what they have sometimes done in little.</q> <rj><qau>Boyle.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>In an increasing or additional degree; even more; -- much used with comparatives.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The guilt being great, the fear doth <qex>still</qex> exceed.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <def>Notwithstanding what has been said or done; in spite of what has occured; nevertheless; -- sometimes used as a conjunction. See Synonym of <er>But</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>As sunshine, broken in the rill,<br/
Though turned astray, is sunshine <qex>still</qex>.</q> <rj><qau>Moore.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>6.</sn> <def>After that; after what is stated.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>In the primitive church, such as by fear being compelled to sacrifice to strange gods, after repented, and kept <qex>still</qex> the office of preaching the gospel.</q> <rj><qau>Whitgift.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Still and anon</b></col>, <cd>at intervals and repeatedly; continually; ever and anon; now and then.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><q>And like the watchful minutes to the hour,<br/
<qex>Still and anon</qex> cheered up the heavy time.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Still</ent><br/
<hw>Still</hw>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Stilled</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Stilling</conjf>.]</vmorph> <ety>[AS. <ets>stillan</ets>, from <ets>stille</ets> still, quiet, firm. See <er>Still</er>, <pos>a.</pos>]</ety> <sn>1.</sn> <def>To stop, as motion or agitation; to cause to become quiet, or comparatively quiet; to check the agitation of; <as>as, to <ex>still</ex> the raging sea</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>He having a full sway over the water, had power to <qex>still</qex> and compose it, as well as to move and disturb it.</q> <rj><qau>Woodward.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To stop, as noise; to silence.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>With his name the mothers <qex>still</qex> their babies.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>To appease; to calm; to quiet, as tumult, agitation, or excitement; <as>as, to <ex>still</ex> the passions</as>.</def>  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Toil that would, at least, have <qex>stilled</qex> an unquiet impulse in me.</q> <rj><qau>Hawthorne.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><syn><b>Syn.</b> -- To quiet; calm; allay; lull; pacify; appease; subdue; suppress; silence; stop; check; restrain.</syn><br/
[<source>1913 Webster</source>]</p>

<p><-- p. 1415 --></p>

<p><ent>Still</ent><br/
<hw>Still</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Cf. OE. <ets>stillatorie</ets>. See <er>Still</er>, <pos>v.</pos>, to distill.]</ety> <sn>1.</sn> <def>A vessel, boiler, or copper used in the distillation of liquids; specifically, one used for the distillation of alcoholic liquors; a retort. The name is sometimes applied to the whole apparatus used in in vaporization and condensation.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A house where liquors are distilled; a distillery.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Still watcher</b></col>, <cd>a device for indicating the progress of distillation by the density of the liquid given over.</cd> <au>Knight.</au></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Still</ent><br/
<hw>Still</hw>, <pos>v. t.</pos> <ety>[Abbreviated fr. <ets>distill</ets>.]</ety> <sn>1.</sn> <def>To cause to fall by drops.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To expel spirit from by heat, or to evaporate and condense in a refrigeratory; to distill.</def>  <rj><au>Tusser.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Still</ent><br/
<hw>Still</hw>, <pos>v. i.</pos> <ety>[L. <ets>stillare</ets>.  Cf. <er>Distill</er>.]</ety> <def>To drop, or flow in drops; to distill.</def> <mark>[Obs.]</mark>  <rj><au>Spenser.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stillage</ent><br/
<hw>Stil"lage</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Bleaching)</fld> <def>A low stool to keep the goods from touching the floor.</def>  <rj><au>Knight.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stillatitious</ent><br/
<hw>Stil`la*ti"tious</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>stillaticius</ets>, fr. <ets>stillare</ets> to drop, <ets>stilla</ets> a drop.]</ety> <def>Falling in drops; drawn by a still.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stillatory</ent><br/
<hw>Stil"la*to*ry</hw> <pr>(?)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>-ries</plw> <pr>(#)</pr>.</plu> <ety>[From <er>Still</er>, for <ets>distill</ets>.  Cf. <er>Still</er>, <pos>n.</pos>, and <er>Distillatory</er>, <pos>a.</pos>]</ety> <sn>1.</sn> <def>An alembic; a vessel for distillation.</def> <mark>[R.]</mark>  <rj><au>Bacon.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A laboratory; a place or room in which distillation is performed.</def> <mark>[R.]</mark>  <rj><au>Dr. H. More.</au> <au>Sir H. Wotton.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stillbirth</ent><br/
<hw>Still"birth`</hw> <pr>(?)</pr>, <pos>n.</pos> <sn>1.</sn> <def>The birth of a dead fetus.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>a fetus which was born dead.</def><br/
[<source>PJC</source>]</p>

<p><ent>Stillborn</ent><br/
<hw>Still"born`</hw> <pr>(?)</pr>, <pos>a.</pos> <sn>1.</sn> <def>Dead at the birth; <as>as, a <ex>stillborn</ex> child</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Fig.: Abortive; <as>as, a <ex>stillborn</ex> poem</as>.</def>  <rj><au>Swift.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Still-burn</ent><br/
<hw>Still"-burn`</hw> <pr>(?)</pr>, <pos>v. t.</pos> <ety>[<pos>imp. & p. p.</pos> <er>still-burnt</er> <pr>(?)</pr> or <er>Still-burned</er> <pr>(<?/)</pr>; <pos>p. pr. & vb. n.</pos> <er>Still-burning</er>.]</ety> <def>To burn in the process of distillation; <as>as, to <ex>still-burn</ex> brandy</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Still-closing</ent><br/
<hw>Still"-clos"ing</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Ever closing.</def> <mark>[Obs.]</mark> <ldquo/<xex>Still-clothing</xex> waters.<rdquo/  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stiller</ent><br/
<hw>Still"er</hw> <pr>(?)</pr>, <pos>n.</pos> <def>One who stills, or quiets.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stillhouse</ent><br/
<hw>Still"house`</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A house in which distillation is carried on; a distillery.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Still-hunt</ent><br/
<hw>Still"-hunt`</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A hunting for game in a quiet and cautious manner, or under cover; stalking; hence, colloquially, the pursuit of any object quietly and cautiously.</def> -- <wordforms><wf>Still"-hunt`er</wf> <pr>(#)</pr>, <pos>n.</pos> -- <wf>Still"-hunt`ing</wf>, <pos>n.</pos> <mark>[U.S.]</mark></wordforms><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stillicide</ent><br/
<hw>Stil"li*cide</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>stillicidium</ets>; <ets>stilla</ets> a drop + <ets>cadere</ets> to fall.]</ety> <def>A continual falling or succession of drops; rain water falling from the eaves.</def>  <rj><au>Bacon.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stillicidious</ent><br/
<hw>Stil`li*cid"i*ous</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Falling in drops.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stilliform</ent><br/
<hw>Stil"li*form</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>stilla</ets> a drop + <ets>-form</ets>.]</ety> <def>Having the form of a drop.</def>  <rj><au>Owen.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stilling</ent><br/
<hw>Still"ing</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Cf. LG. <ets>stelling</ets>, G. <ets>stellen</ets> to set, to place.]</ety> <def>A stillion.</def> <mark>[Obs. or Prov. Eng.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stillion</ent><br/
<hw>Stil"lion</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[See <er>Stilling</er>.]</ety> <def>A stand, as for casks or vats in a brewery, or for pottery while drying.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Still life</ent><br/
<hw>Still" life</hw> <pr>(?)</pr>, <pos>n.</pos> <sn>1.</sn> <def>a type of painting in which inanimate objects such as flowers or fruit are represented.</def><br/
[<source>PJC</source>]</p>

<p><sn>2.</sn> <def>the inanimate subject matter of a <er>still life</er> painting or photograph; <as>as, to painted only <ex>still life</ex></as>.</def><br/
[<source>PJC</source>]</p>

<p><ent>Stillness</ent><br/
<hw>Still"ness</hw> <pr>(?)</pr>, <pos>n.</pos> <sn>1.</sn> <def>The quality or state of being still; quietness; silence; calmness; inactivity.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Painting, then, was the art demanded by the modern intellect upon its emergence from the <qex>stillness</qex> of the Middle Ages.</q> <rj><qau>J. A. Symonds.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Habitual silence or quiet; taciturnity.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The gravity and <qex>stillness</qex> of your youth<br/
The world hath noted.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stillroom</ent><br/
<hw>Still"room`</hw> <pr>(?)</pr>, <pos>n.</pos> <sn>1.</sn> <def>A room for distilling.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>An apartment in a house where liquors, preserves, and the like, are kept.</def> <mark>[Eng.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>Floors are rubbed bright, . . . <qex>stillroom</qex> and kitchen cleared for action.</q> <rj><qau>Dickens.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stillson wrench</ent><br/
<hw>Still"son wrench</hw> <pr>(?)</pr>. <def>A pipe wrench having an adjustable <universbold>L</universbold>-shaped jaw piece sliding in a sleeve that is pivoted to, and loosely embraces, the handle. Pressure on the handle increases the grip.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Stillstand</ent><br/
<hw>Still"stand`</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A standstill.</def> <mark>[R.]</mark>  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Still water</ent><br/
<hw>Still" wat*er</hw> <pr>(?)</pr>, <pos>n.</pos> <def>a section of a stream that is flat and moves slowly.</def><br/
[<source>PJC</source>]</p>

<p><ent>Still wine</ent><br/
<hw>Still" wine`</hw> <pr>(?)</pr>, <pos>n.</pos> <def>a table wine which is not effervescent; a non-sparkling wine.</def><br/
[<source>PJC</source>]</p>

<p><ent>Stilly</ent><br/
<hw>Still"y</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Still; quiet; calm.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The <qex>stilly</qex> hour when storms are gone.</q> <rj><qau>Moore.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stilly</ent><br/
<hw>Stil"ly</hw> <pr>(?)</pr>, <pos>adv.</pos> <def>In a still manner; quietly; silently; softly.</def>  <rj><au>Dr. H. More.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>The hum of either army <qex>stilly</qex> sounds.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stilpnomelane</ent><br/
<hw>Stilp*nom"e*lane</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Gr. <grk>stilpno`s</grk> shining + <grk>me`las</grk>, <grk>-anos</grk>, black.]</ety> <fld>(Min.)</fld> <def>A black or greenish black mineral occurring in foliated flates, also in velvety bronze-colored incrustations. It is a hydrous silicate of iron and alumina.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stilt</ent><br/
<hw>Stilt</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[OE. <ets>stilte</ets>; akin to Dan. <ets>stylte</ets>, Sw. <ets>stylta</ets>, LG. & D. <ets>stelt</ets>, OHG. <ets>stelza</ets>, G. <ets>stelze</ets>, and perh. to E. <ets>stout</ets>.]</ety> <sn>1.</sn> <def>A pole, or piece of wood, constructed with a step or loop to raise the foot above the ground in walking. It is sometimes lashed to the leg, and sometimes prolonged upward so as to be steadied by the hand or arm.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Ambition is but avarice on <qex>stilts</qex>, and masked.</q> <rj><qau>Landor.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A crutch; also, the handle of a plow.</def> <mark>[Prov. Eng.]</mark>  <rj><au>Halliwell.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <fld>(Zool.)</fld> <def>Any species of limicoline birds belonging to <gen>Himantopus</gen> and allied genera, in which the legs are remarkably long and slender. Called also <altname>longshanks</altname>, <altname>stiltbird</altname>, <altname>stilt plover</altname>, and <altname>lawyer</altname>.</def><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ The American species (<spn>Himantopus Mexicanus</spn>) is well known. The European and Asiatic stilt (<spn>Himantopus candidus</spn>) is usually white, except the wings and interscapulars, which are greenish black. The white-headed stilt (<spn>Himantopus leucocephalus</spn>) and the banded stilt (<spn>Cladorhynchus pectoralis</spn>) are found in Australia.</note><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Stilt plover</b></col> <fld>(Zool.)</fld>, <cd>the stilt.</cd> -- <col><b>Stilt sandpiper</b></col> <fld>(Zool.)</fld>, <cd>an American sandpiper (<spn>Micropalama himantopus</spn>) having long legs. The bill is somewhat expanded at the tip.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stilt</ent><br/
<hw>Stilt</hw> <pr>(?)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Stilted</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Stilting</conjf>.]</vmorph> <def>To raise on stilts, or as if on stilts.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stiltbird</ent><br/
<hw>Stilt"bird`</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Zool.)</fld> <def>See <er>Stilt</er>, <pos>n.</pos>, 3.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stilted</ent><br/
<hw>Stilt"ed</hw>, <pos>a.</pos> <def>Elevated as if on stilts; hence, pompous; bombastic; <as>as, a <ex>stilted</ex> style; <ex>stilted</ex> declamation.</as></def>
<-- also, formal or stiff; -- said of linguistic style --><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Stilted arch</b></col> <fld>(Arch.)</fld>, <cd>an arch in which the springing line is some distance above the impost, the space between being occupied by a vertical member, molded or ornamented, as a continuation of the archivolt, intrados, etc.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stiltify</ent><br/
<hw>Stilt"i*fy</hw> <pr>(?)</pr>, <pos>v. t.</pos> <ety>[<ets>Stilt</ets> + <ets>-fy</ets>.]</ety> <def>To raise upon stilts, or as upon stilts; to stilt.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stilton</ent><br/
<ent>Stilton cheese</ent><br/
<mhw>{ <hw>Stil"ton cheese"</hw> <pr>(?)</pr>, <it>or</it> <hw>Stil"ton</hw>  }</mhw>, <pos>n.</pos> <def>A peculiarly flavored unpressed cheese made from milk with cream added; -- so called from the village or parish of <etsep>Stilton</etsep>, England, where it was originally made. It is very rich in fat.</def></p>

<p><q>Thus, in the outset he was gastronomic; discussed the dinner from the soup to the <qex>stilton</qex>.</q>  <rj><qau>C. Lever.</qau></rj><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Stilty</ent><br/
<hw>Stilt"y</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Unreasonably elevated; pompous; stilted; <as>as, a <ex>stilty</ex> style</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stime</ent><br/
<hw>Stime</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Etymology uncertain.]</ety> <def>A slight gleam or glimmer; a glimpse.</def> <mark>[Prov. Eng.]</mark>  <rj><au>Halliwell.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stimie</ent><br/
<ent>Stimey</ent><br/
<mhw>{ <hw>Sti"mey</hw>, <hw>Sti"mie</hw>  }</mhw> <pr>(?)</pr>, <pos>n. & v. t.</pos> <def>See <er>Stymie</er>.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Stimulant</ent><br/
<hw>Stim"u*lant</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>stimulans</ets>, p. pr.; cf. F. <ets>stimulant</ets>. See <er>Stimulate</er>.]</ety> <sn>1.</sn> <def>Serving to stimulate.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Physiol.)</fld> <def>Produced increased vital action in the organism, or in any of its parts.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stimulant</ent><br/
<hw>Stim"u*lant</hw>, <pos>n.</pos> <ety>[Cf. F. <ets>stimulant</ets>.]</ety> <sn>1.</sn> <def>That which stimulates, provokes, or excites.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>His feelings had been exasperated by the constant application of <qex>stimulants</qex>.</q> <rj><qau>Macaulay.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Physiol. & Med.)</fld> <def>An agent which produces a temporary increase of vital activity in the organism, or in any of its parts; -- sometimes used without qualification to signify an alcoholic beverage used as a stimulant.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stimulate</ent><br/
<hw>Stim"u*late</hw> <pr>(?)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Stimulated</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Stimulating</conjf>.]</vmorph> <ety>[L. <ets>stimulatus</ets>, p. p. of <ets>stimulare</ets> to prick or goad on, to incite, fr. <ets>stimulus</ets> a goad. See <er>Stimulus</er>.]</ety> <sn>1.</sn> <def>To excite as if with a goad; to excite, rouse, or animate, to action or more vigorous exertion by some pungent motive or by persuasion; <as>as, to <ex>stimulate</ex> one by the hope of reward, or by the prospect of glory</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>To excite and <qex>stimulate</qex> us thereunto.</q> <rj><qau>Dr. J. Scott.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Physiol.)</fld> <def>To excite; to irritate; especially, to excite the activity of (a nerve or an irritable muscle), as by electricity.</def><br/
[<source>1913 Webster</source>]</p>

<p><syn><b>Syn.</b> -- To animate; incite; encourage; impel; urge; instigate; irritate; exasperate; incense.</syn><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stimulation</ent><br/
<hw>Stim`u*la"tion</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>stimulatio</ets>: cf. F. <ets>stimulation</ets>.]</ety> <sn>1.</sn> <def>The act of stimulating, or the state of being stimulated.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Physiol.)</fld> <def>The irritating action of various agents (stimuli) on muscles, nerves, or a sensory end organ, by which activity is evoked; especially, the nervous impulse produced by various agents on nerves, or a sensory end organ, by which the part connected with the nerve is thrown into a state of activity; irritation.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stimulative</ent><br/
<hw>Stim"u*la*tive</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Having the quality of stimulating.</def> -- <def2><pos>n.</pos> <def>That which stimulates.</def></def2><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stimulator</ent><br/
<hw>Stim"u*la`tor</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L.: cf. F. <ets>stimulateur</ets>.]</ety> <def>One who stimulates.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stimulatress</ent><br/
<hw>Stim"u*la`tress</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A woman who stimulates.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stimulism</ent><br/
<hw>Stim"u*lism</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Med.)</fld> <sd>(a)</sd> <def>The theory of medical practice which regarded life as dependent upon stimulation, or excitation, and disease as caused by excess or deficiency in the amount of stimulation.</def> <sd>(b)</sd> <def>The practice of treating disease by alcoholic stimulants.</def>  <rj><au>Dr. H. Hartshorne.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stimulus</ent><br/
<hw>Stim"u*lus</hw> <pr>(?)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Stimuli</plw> <pr>(#)</pr>.</plu> <ety>[L., for <ets>stigmulus</ets>, akin to L. <ets>instigare</ets> to stimulate. See <er>Instigare</er>, <er>Stick</er>, <pos>v. t.</pos>]</ety> <sn>1.</sn> <def>A goad; hence, something that rouses the mind or spirits; an incentive; <as>as, the hope of gain is a powerful <ex>stimulus</ex> to labor and action</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>That which excites or produces a temporary increase of vital action, either in the whole organism or in any of its parts; especially <fld>(Physiol.)</fld>, any substance or agent capable of evoking the activity of a nerve or irritable muscle, or capable of producing an impression upon a sensory organ or more particularly upon its specific end organ.</def><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ Of the stimuli applied to the sensory apparatus, physiologists distinguish two kinds: (<xex>a</xex>) <stype>Homologous stimuli</stype>, which act only upon the end organ, and for whose action the sense organs are especially adapted, as the rods and cones of the retina for the vibrations of the either. (<xex>b</xex>) <stype>Heterologous stimuli</stype>, which are mechanical, chemical, electrical, etc., and act upon the nervous elements of the sensory apparatus along their entire course, producing, for example, the flash of light beheld when the eye is struck.</note>  <rj><au>Landois & Stirling.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sting</ent><br/
<hw>Sting</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[AS. <ets>sting</ets> a sting. See <er>Sting</er>, <pos>v. t.</pos>]</ety> <sn>1.</sn> <fld>(Zool.)</fld> <def>Any sharp organ of offense and defense, especially when connected with a poison gland, and adapted to inflict a wound by piercing; as the caudal <xex>sting</xex> of a scorpion. The sting of a bee or wasp is a modified ovipositor. The caudal sting, or spine, of a sting ray is a modified dorsal fin ray. The term is sometimes applied to the fang of a serpent. See <xex>Illust.</xex> of <er>Scorpion</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Bot.)</fld> <def>A sharp-pointed hollow hair seated on a gland which secrets an acrid fluid, as in nettles. The points of these hairs usually break off in the wound, and the acrid fluid is pressed into it.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>Anything that gives acute pain, bodily or mental; <as>as, the <ex>stings</ex> of remorse; the <ex>stings</ex> of reproach.</as></def><br/
[<source>1913 Webster</source>]</p>

<p><q>The <qex>sting</qex> of death is sin.</q> <rj><qau>1 Cor. xv. 56.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>The thrust of a sting into the flesh; the act of stinging; a wound inflicted by stinging.</def> <ldquo/The lurking serpent's mortal <xex>sting</xex>.<rdquo/  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <def>A goad; incitement.</def>  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>6.</sn> <def>The point of an epigram or other sarcastic saying.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Sting moth</b></col> <fld>(Zool.)</fld>, <cd>an Australian moth (<spn>Doratifera vulnerans</spn>) whose larva is armed, at each end of the body, with four tubercles bearing powerful stinging organs.</cd> -- <col><b>Sting ray</b></col>. <fld>(Zool.)</fld> <cd>See under 6th <er>Ray</er>.</cd> -- <col><b>Sting winkle</b></col> <fld>(Zool.)</fld>, <cd>a spinose marine univalve shell of the genus Murex, as the European species (<spn>Murex erinaceus</spn>). See <xex>Illust.</xex> of <er>Murex</er>.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sting</ent><br/
<hw>Sting</hw>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Stung</conjf> <pr>(?)</pr> (<mark>Archaic</mark> <conjf>Stang</conjf> <pr>(<?/)</pr>); <pos>p. pr. & vb. n.</pos> <conjf>Stinging</conjf>.]</vmorph> <ety>[AS. <ets>stingan</ets>; akin to Icel. & Sw. <ets>stinga</ets>, Dan. <ets>stinge</ets>, and probably to E. <ets>stick</ets>, v.t.; cf. Goth. us<ets>stiggan</ets> to put out, pluck out.  Cf. <er>Stick</er>, <pos>v. t.</pos>]</ety> <sn>1.</sn> <def>To pierce or wound with a sting; <as>as, bees will <ex>sting</ex> an animal that irritates them; the nettles <ex>stung</ex> his hands.</as></def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To pain acutely; <as>as, the conscience is <ex>stung</ex> with remorse</as>; to bite.</def> <ldquo/Slander <xex>stings</xex> the brave.<rdquo/  <rj><au>Pope.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>To goad; to incite, as by taunts or reproaches.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stingaree</ent><br/
<hw>Sting`a*ree"</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Zool.)</fld> <def>Any sting ray. See under 6th <er>Ray</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stingbull</ent><br/
<hw>Sting"bull`</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Zool.)</fld> <def>The European greater weever fish (<spn>Trachinus draco</spn>), which is capable of inflicting severe wounds with the spinous rays of its dorsal fin. See <er>Weever</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stinger</ent><br/
<hw>Sting"er</hw> <pr>(?)</pr>, <pos>n.</pos> <def>One who, or that which, stings.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Professor E. Forbes states that only a small minority of the medusae of our seas are <qex>stingers</qex>.</q> <rj><qau>Owen.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stingfish</ent><br/
<hw>Sting"fish`</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Zool.)</fld> <def>The weever.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stingily</ent><br/
<hw>Stin"gi*ly</hw> <pr>(?)</pr>, <pos>adv.</pos> <def>In a stingy manner.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stinginess</ent><br/
<hw>Stin"gi*ness</hw>, <pos>n.</pos> <def>The quality or state of being stingy.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stinging</ent><br/
<hw>Sting"ing</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Piercing, or capable of piercing, with a sting; inflicting acute pain as if with a sting, goad, or pointed weapon; pungent; biting; <as>as, <ex>stinging</ex> cold; a <ex>stinging</ex> rebuke.</as></def> -- <wordforms><wf>Sting"ing*ly</wf>, <pos>adv.</pos></wordforms><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Stinging cell</b></col>. <fld>(Zool.)</fld> <cd>Same as <cref>Lasso cell</cref>, under <er>Lasso</er>.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stingless</ent><br/
<hw>Sting"less</hw>, <pos>a.</pos> <def>Having no sting.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stingo</ent><br/
<hw>Stin"go</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[From <er>Sting</er>.]</ety> <def>Old beer; sharp or strong liquor.</def> <mark>[Old Slang]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>Shall I set a cup of old <qex>stingo</qex> at your elbow?</q> <rj><qau>Addison.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stingray</ent><br/
<ent>Sting ray</ent><br/
<mhw>{ <hw>Sting ray</hw> <it>or</it> <hw>Sting"ray`</hw>  }</mhw>, <pos>n.</pos> <def>Any one of numerous rays of the family <fam>Dasyatidae</fam>, syn. <fam>Trygonidae</fam>, having one or more large sharp barbed dorsal spines, on the whiplike tail, capable of inflicting severe wounds. Some species reach a large size, and some, esp., on the American Pacific coast, are very destructive to oysters.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Stingtail</ent><br/
<hw>Sting"tail`</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Zool.)</fld> <def>A sting ray.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stingy</ent><br/
<hw>Sting"y</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Stinging; able to sting.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stingy</ent><br/
<hw>Stin"gy</hw> <pr>(?)</pr>, <pos>a.</pos> <amorph>[<pos>Compar.</pos> <adjf>Stingier</adjf> <pr>(?)</pr>; <pos>superl.</pos> <adjf>Stingiest</adjf>.]</amorph> <ety>[Probably from <ets>sting</ets>, and meaning originally, stinging; hence, biting, nipping (of the wind), churlish, avaricious; or cf. E. <ets>skinch</ets>.]</ety> <def>Extremely close and covetous; meanly avaricious; niggardly; miserly; penurious; <as>as, a <ex>stingy</ex> churl</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>A <qex>stingy</qex>, narrow-hearted fellow that had a deal of choice fruit, had not the heart to touch it till it began to be rotten.</q> <rj><qau>L'estrange.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stink</ent><br/
<hw>Stink</hw> <pr>(?)</pr>, <pos>v. i.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Stunk</conjf> <pr>(?)</pr>, <conjf>Stank</conjf> <pr>(<?/)</pr>, <pos>p. pr. & vb. n.</pos> <conjf>Stinking</conjf>.]</vmorph> <ety>[AS. <ets>stinkan</ets> to have a smell (whether good or bad); akin to OHG. <ets>stinchan</ets>, G. & D. <ets>stinken</ets> to stink; of uncertain origin; cf. Icel. <ets>st<oum/kkva</ets> to leap, to spring, Goth. <ets>stigqan</ets> to push, strike, or Gr. <?/ rancid.  Cf. <er>Stench</er>.]</ety> <def>To emit a strong, offensive smell; to send out a disgusting odor.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stink</ent><br/
<hw>Stink</hw>, <pos>v. t.</pos> <def>To cause to stink; to affect by a stink.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stink</ent><br/
<hw>Stink</hw>, <pos>n.</pos> <ety>[AS. <ets>stinc</ets>.]</ety> <def>A strong, offensive smell; a disgusting odor; a stench.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Fire stink</b></col>. <cd>See under <er>Fire</er>.</cd> -- <col><b>Stink-fire lance</b></col>. <cd>See under <er>Lance</er>.</cd> -- <col><b>Stink rat</b></col> <fld>(Zool.)</fld>, <cd>the musk turtle.</cd> <mark>[Local, U.S.]</mark> -- <col><b>Stink shad</b></col> <fld>(Zool.)</fld>, <cd>the gizzard shad.</cd> <mark>[Local, U.S.]</mark> -- <col><b>Stink trap</b></col>, <cd>a stench trap. See under <er>Stench</er>.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stinkard</ent><br/
<hw>Stink"ard</hw> <pr>(?)</pr>, <pos>n.</pos> <sn>1.</sn> <def>A mean, stinking, paltry fellow.</def>  <rj><au>B. Jonson.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Zool.)</fld> <def>The teledu of the East Indies. It emits a disagreeable odor.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stinkball</ent><br/
<hw>Stink"ball`</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A composition of substances which in combustion emit a suffocating odor; -- used formerly in naval warfare.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stinker</ent><br/
<hw>Stink"er</hw> <pr>(?)</pr>, <pos>n.</pos> <sn>1.</sn> <def>One who, or that which, stinks.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Zool.)</fld> <def>Any one of the several species of large antarctic petrels which feed on blubber and carrion and have an offensive odor, as the giant fulmar.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stinkhorn</ent><br/
<hw>Stink"horn`</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Bot.)</fld> <def>A kind of fungus of the genus <gen>Phallus</gen>, which emits a fetid odor.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stinking</ent><br/
<hw>Stink"ing</hw>, <def><pos>a. & n.</pos> from <er>Stink</er>, <pos>v.</pos></def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Stinking badger</b></col> <fld>(Zool.)</fld>, <cd>the teledu.</cd> -- <col><b>Stinking cedar</b></col> <fld>(Bot.)</fld>, <cd>the California nutmeg tree; also, a related tree of Florida (<spn>Torreya taxifolia</spn>).</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stinkingly</ent><br/
<hw>Stink"ing*ly</hw>, <pos>adv.</pos> <def>In a stinking manner; with an offensive smell.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stinkpot</ent><br/
<hw>Stink"pot`</hw> <pr>(?)</pr>, <pos>n.</pos> <sn>1.</sn> <def>An earthen jar charged with powder, grenades, and other materials of an offensive and suffocating smell, -- sometimes used in boarding an enemy's vessel.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A vessel in which disinfectants are burned.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <fld>(Zool.)</fld> <def>The musk turtle, or musk tortoise. See under <er>Musk</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stinkstone</ent><br/
<hw>Stink"stone`</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Min.)</fld> <def>One of the varieties of calcite, barite, and feldspar, which emit a fetid odor on being struck; -- called also <altname>swinestone</altname>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stinkweed</ent><br/
<hw>Stink"weed`</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Bot.)</fld> <def>Stramonium. See <er>Jamestown weed</er>, and <er>Datura</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><-- p. 1416 --></p>

<p><ent>Stinkwood</ent><br/
<hw>Stink"wood`</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Bot.)</fld> <def>A name given to several kinds of wood with an unpleasant smell, as that of the <spn>Foetidia Mauritiana</spn> of the Mauritius, and that of the South African <spn>Ocotea bullata</spn>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stint</ent><br/
<hw>Stint</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Zool.)</fld> <sd>(a)</sd> <def>Any one of several species of small sandpipers, as the sanderling of Europe and America, the dunlin, the little stint of India (<spn>Tringa minuta</spn>), etc. Called also <altname>pume</altname>.</def> <sd>(b)</sd> <def>A phalarope.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stint</ent><br/
<hw>Stint</hw>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Stinted</conjf>; <pos>p. pr. & vb. n.</pos> <conjf>Stinting</conjf>.]</vmorph> <ety>[OE. <ets>stinten</ets>, <ets>stenten</ets>, <ets>stunten</ets>, to cause to cease, AS. <ets>styntan</ets> (in comp.) to blunt, dull, fr. <ets>stunt</ets> dull, stupid; akin to Icel. <ets>stytta</ets> to shorten, <ets>stuttr</ets> short, dial, Sw. <ets>stynta</ets> to shorten, <ets>stunt</ets> short.  Cf. <er>Stent</er>, <er>Stunt</er>.]</ety> <sn>1.</sn> <def>To restrain within certain limits; to bound; to confine; to restrain; to restrict to a scant allowance.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>I shall not go about to extenuate the latitude of the curse upon the earth, or <qex>stint</qex> it only to the production of weeds.</q> <rj><qau>Woodward.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>She <qex>stints</qex> them in their meals.</q> <rj><qau>Law.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To put an end to; to stop.</def> <mark>[Obs.]</mark>  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>To assign a certain (<it>i. e.</it>, limited) task to (a person), upon the performance of which one is excused from further labor for the day or for a certain time; to stent.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>To serve successfully; to get with foal; -- said of mares.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The majority of maiden mares will become <qex>stinted</qex> while at work.</q> <rj><qau>J. H. Walsh.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stint</ent><br/
<hw>Stint</hw>, <pos>v. i.</pos> <def>To stop; to cease.</def> <mark>[Archaic]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>They can not <qex>stint</qex> till no thing be left.</q> <rj><qau>Chaucer.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>And <qex>stint</qex> thou too, I pray thee.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>The damsel <qex>stinted</qex> in her song.</q> <rj><qau>Sir W. Scott.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><-- 2. to be parsimonious in expending some resource; -- used with "on" to indicate the item conserved.  "Don't stint on the potatos!" --><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stint</ent><br/
<hw>Stint</hw>, <pos>n.</pos> <ety>[Also written <ets>stent</ets>. See <er>Stint</er>, <pos>v. t.</pos>]</ety> <sn>1.</sn> <def>Limit; bound; restraint; extent.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>God has wrote upon no created thing the utmost <qex>stint</qex> of his power.</q> <rj><qau>South.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Quantity or task assigned; proportion allotted.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>His old <qex>stint</qex> -- three thousand pounds a year.</q> <rj><qau>Cowper.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><-- 3. A period of work at a specific task; as, to do one's stint in the army, an actor who did a stint as a cab driver. --><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stintance</ent><br/
<hw>Stint"ance</hw> <pr>(?)</pr>, <pos>n.</pos> <def>Restraint; stoppage.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stintedness</ent><br/
<hw>Stint"ed*ness</hw>, <pos>n.</pos> <def>The state of being stinted.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stinter</ent><br/
<hw>Stint"er</hw> <pr>(?)</pr>, <pos>n.</pos> <def>One who, or that which, stints.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stintless</ent><br/
<hw>Stint"less</hw>, <pos>a.</pos> <def>Without stint or restraint.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The <qex>stintless</qex>tears of old Heraclitus.</q> <rj><qau>Marston.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stipe</ent><br/
<hw>Stipe</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>stipes</ets> a stock, post, branch: cf. F. <ets>stipe</ets>.]</ety> <fld>(Bot.)</fld> <sd>(a)</sd> <def>The stalk or petiole of a frond, as of a fern.</def> <sd>(b)</sd> <def>The stalk of a pistil.</def> <sd>(c)</sd> <def>The trunk of a tree.</def> <sd>(d)</sd> <def>The stem of a fungus or mushroom.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stipel</ent><br/
<hw>Sti"pel</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[See <er>Stipule</er>.]</ety> <fld>(Bot.)</fld> <def>The stipule of a leaflet.</def>  <rj><au>Gray.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stipellate</ent><br/
<hw>Sti*pel"late</hw> <pr>(?)</pr>, <pos>a.</pos> <fld>(Bot.)</fld> <def>Having stipels.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stipend</ent><br/
<hw>Sti"pend</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>stipendium</ets>; <ets>stips</ets>, gen. <ets>stipis</ets>, a gift, donation, given in small coin + <ets>pendere</ets> to weigh or pay out.]</ety> <def>Settled pay or compensation for services, whether paid daily, monthly, or annually.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stipend</ent><br/
<hw>Sti"pend</hw>, <pos>v. t.</pos> <def>To pay by settled wages.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stipendiarian</ent><br/
<hw>Sti*pen`di*a"ri*an</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Acting from mercenary considerations; stipendiary.</def>  <rj><au>A. Seward.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stipendiary</ent><br/
<hw>Sti*pen"di*a*ry</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>stipendiarius</ets>: cf. F. <ets>stipendiaire</ets>.]</ety> <def>Receiving wages, or salary; performing services for a stated price or compensation.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>His great <qex>stipendiary</qex> prelates came with troops of evil-appointed horseman not half full.</q> <rj><qau>Knolles.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stipendiary</ent><br/
<hw>Sti*pen"di*a*ry</hw>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Stipendiaries</plw> <pr>(<?/)</pr>.</plu> <def>One who receives a stipend.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>If thou art become<br/
A tyrant's vile <qex>stipendiary</qex>.</q> <rj><qau>Glover.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stipendiate</ent><br/
<hw>Sti*pen"di*ate</hw> <pr>(?)</pr>, <pos>v. t.</pos> <ety>[L. <ets>stipendiatus</ets>, p. p. of <ets>stipendiari</ets> to receive pay.]</ety> <def>To provide with a stipend, or salary; to support; to pay.</def>  <rj><au>Evelyn.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>It is good to endow colleges, and to found chairs, and to <qex>stipendiate</qex> professors.</q> <rj><qau>I. Taylor.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stipendless</ent><br/
<hw>Sti"pend*less</hw> <pr>(st<imac/"p<ecr/nd*l<ecr/s)</pr>, <pos>a.</pos> <def>Having no stipend.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stipes</ent><br/
||<hw>Sti"pes</hw> <pr>(-p<emac/z)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Stipites</plw> <pr>(#)</pr>.</plu> <ety>[L., a stock.]</ety> <fld>(Zool.)</fld> <sd>(a)</sd> <def>The second joint of a maxilla of an insect or a crustacean.</def> <sd>(b)</sd> <def>An eyestalk.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stipitate</ent><br/
<hw>Stip"i*tate</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[NL. <ets>stipitatus</ets>, from L. <ets>stipes</ets>, gen. <ets>stipitis</ets>, a stock. See <er>Stipe</er>.]</ety> <fld>(Bot.)</fld> <def>Supported by a stipe; elevated on a stipe, as the fronds of most ferns, or the pod of certain cruciferous plants.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stipitiform</ent><br/
<hw>Stip"i*ti*form</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[<ets>Stipes</ets> + <ets>-form</ets>.]</ety> <fld>(Bot.)</fld> <def>Having the shape of a stalk; stalklike.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stipple</ent><br/
<hw>Stip"ple</hw> <pr>(?)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Stippled</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Stippling</conjf> <pr>(?)</pr>.]</vmorph> <ety>[D. <ets>stippelen</ets> to make points, to spot, dot, from <ets>stippel</ets>, dim. of <ets>stip</ets> a dot, spot.]</ety> <sn>1.</sn> <def>To engrave by means of dots, in distinction from engraving in lines.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The interlaying of small pieces can not altogether avoid a broken, <qex>stippled</qex>, spotty effect.</q> <rj><qau>Milman.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To paint, as in water colors, by small, short touches which together produce an even or softly graded surface.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stippling</ent><br/
<ent>Stipple</ent><br/
<mhw>{ <hw>Stip"ple</hw> <pr>(?)</pr>, <hw>Stip"pling</hw> <pr>(?)</pr>, }</mhw> <pos>n.</pos> <fld>(Engraving)</fld> <def>A mode of execution which produces the effect by dots or small points instead of lines.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Paint.)</fld> <def>A mode of execution in which a flat or even tint is produced by many small touches.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stiptic</ent><br/
<hw>Stip"tic</hw> <pr>(?)</pr>, <pos>a. & n.</pos> <fld>(Med.)</fld> <def>See <er>Styptic</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stipula</ent><br/
||<hw>Stip"u*la</hw> <pr>(?)</pr>, <pos>n.</pos>; <plu><it>pl.</it> E. <plw>Stipulas</plw> <pr>(#)</pr>, L. <plw>Stipulae</plw> <pr>(#)</pr>.</plu> <ety>[L., a stalk, stem.]</ety> <sn>1.</sn> <fld>(Bot.)</fld> <def>A stipule.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Zool.)</fld> <def>A newly sprouted feather.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stipular</ent><br/
<ent>Stipulaceous</ent><br/
<mhw>{ <hw>Stip`u*la"ceous</hw> <pr>(?)</pr>, <hw>Stip"u*lar</hw> <pr>(?)</pr>, }</mhw> <pos>a.</pos> <ety>[Cf. F. <ets>stipulac<eacute/</ets>, <ets>stipulaire</ets>. See <er>Stipula</er>.]</ety> <fld>(Bot.)</fld> <def>Of or pertaining to stipules; resembling stipules; furnished with stipules; growing on stipules, or close to them; occupying the position of stipules; <as>as, <ex>stipular</ex> glands and <ex>stipular</ex> tendrils</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stipulary</ent><br/
<hw>Stip"u*la*ry</hw> <pr>(?)</pr>, <pos>a.</pos> <fld>(Bot.)</fld> <def>Of or pertaining to stipules; stipular.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stipulate</ent><br/
<hw>Stip"u*late</hw> <pr>(?)</pr>, <pos>a.</pos> <fld>(Bot.)</fld> <def>Furnished with stipules; <as>as, a <ex>stipulate</ex> leaf</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stipulate</ent><br/
<hw>Stip"u*late</hw> <pr>(?)</pr>, <pos>v. i.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Stipulated</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Stipulating</conjf>.]</vmorph> <ety>[L. <ets>stipulatus</ets>, p. p. of <ets>stipulari</ets> to stipulate, fr. OL. <ets>stipulus</ets> firm, fast; probably akin to L. <ets>stipes</ets> a post.  Cf. <er>Stiff</er>.]</ety> <def>To make an agreement or covenant with any person or company to do or forbear anything; to bargain; to contract; to settle terms; <as>as, certain princes <ex>stipulated</ex> to assist each other in resisting the armies of France</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stipulation</ent><br/
<hw>Stip`u*la"tion</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>stipulatio</ets>: cf. F. <ets>stipulation</ets>.]</ety> <sn>1.</sn> <def>The act of stipulating; a contracting or bargaining; an agreement.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>That which is stipulated, or agreed upon; that which is definitely arranged or contracted; an agreement; a covenant; a contract or bargain; also, any particular article, item, or condition, in a mutual agreement; <as>as, the <ex>stipulations</ex> of the allied powers to furnish each his contingent of troops</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <fld>(Law)</fld> <def>A material article of an agreement; an undertaking in the nature of bail taken in the admiralty courts; a bargain.</def>  <rj><au>Bouvier. Wharton.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><syn><b>Syn.</b> -- Agreement; contract; engagement. See <er>Covenant</er>.</syn><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stipulation</ent><br/
<hw>Stip`u*la"tion</hw>, <pos>n.</pos> <ety>[See <er>Stipule</er>.]</ety> <fld>(Bot.)</fld> <def>The situation, arrangement, and structure of the stipules.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stipulator</ent><br/
<hw>Stip"u*la`tor</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L.]</ety> <def>One who stipulates, contracts, or covenants.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stipule</ent><br/
<hw>Stip"ule</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>stipula</ets> a stalk, stem, straw: cf. F. <ets>stipule</ets>.  Cf. <er>Stubble</er>.]</ety> <fld>(Bot.)</fld> <def>An appendage at the base of petioles or leaves, usually somewhat resembling a small leaf in texture and appearance.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stipuled</ent><br/
<hw>Stip"uled</hw> <pr>(?)</pr>, <pos>a.</pos> <fld>(Bot.)</fld> <def>Furnished with stipules, or leafy appendages.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stir</ent><br/
<hw>Stir</hw> <pr>(?)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Stirred</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Stirring</conjf>.]</vmorph> <ety>[OE. <ets>stiren</ets>, <ets>steren</ets>, <ets>sturen</ets>, AS. <ets>styrian</ets>; probably akin to D. <ets>storen</ets> to disturb, G. <ets>st<oum/ren</ets>, OHG. <ets>st<omac/ren</ets> to scatter, destroy. <root/166.]</ety> <sn>1.</sn> <def>To change the place of in any manner; to move.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>My foot I had never yet in five days been able to <qex>stir</qex>.</q> <rj><qau>Sir W. Temple.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To disturb the relative position of the particles of, as of a liquid, by passing something through it; to agitate; <as>as, to <ex>stir</ex> a pudding with a spoon</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>My mind is troubled, like a fountain <qex>stirred</qex>.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>To bring into debate; to agitate; to moot.</def><br/
[<source>1913 Webster</source>]</p>

<p><q><qex>Stir</qex> not questions of jurisdiction.</q> <rj><qau>Bacon.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>To incite to action; to arouse; to instigate; to prompt; to excite.</def> <ldquo/To <xex>stir</xex> men to devotion.<rdquo/  <rj><au>Chaucer.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>An Ate, <qex>stirring</qex> him to blood and strife.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>And for her sake some mutiny will <qex>stir</qex>.</q> <rj><qau>Dryden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ In all senses except the first, <xex>stir</xex> is often followed by <xex>up</xex> with an intensive effect; as, to <xex>stir</xex> up fire; to <xex>stir</xex> up sedition.</note><br/
[<source>1913 Webster</source>]</p>

<p><syn><b>Syn.</b> -- To move; incite; awaken; rouse; animate; stimulate; excite; provoke.</syn><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stir</ent><br/
<hw>Stir</hw>, <pos>v. i.</pos> <sn>1.</sn> <def>To move; to change one's position.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>I had not power to <qex>stir</qex> or strive,<br/
But felt that I was still alive.</q> <rj><qau>Byron.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To be in motion; to be active or bustling; to exert or busy one's self.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>All are not fit with them to <qex>stir</qex> and toil.</q> <rj><qau>Byron.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>The friends of the unfortunate exile, far from resenting his unjust suspicions, were <qex>stirring</qex> anxiously in his behalf.</q> <rj><qau>Merivale.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>To become the object of notice; to be on foot.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>They fancy they have a right to talk freely upon everything that <qex>stirs</qex> or appears.</q> <rj><qau>I. Watts.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>To rise, or be up, in the morning.</def> <mark>[Colloq.]</mark>  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stir</ent><br/
<hw>Stir</hw>, <pos>n.</pos> <sn>1.</sn> <def>The act or result of stirring; agitation; tumult; bustle; noise or various movements.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Why all these words, this clamor, and this <qex>stir</qex>?</q> <rj><qau>Denham.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Consider, after so much <qex>stir</qex> about genus and species, how few words we have yet settled definitions of.</q> <rj><qau>Locke.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Public disturbance or commotion; tumultuous disorder; seditious uproar.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Being advertised of some <qex>stirs</qex> raised by his unnatural sons in England.</q> <rj><qau>Sir J. Davies.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>Agitation of thoughts; conflicting passions.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stirabout</ent><br/
<hw>Stir"a*bout`</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A dish formed of oatmeal boiled in water to a certain consistency and frequently stirred, or of oatmeal and dripping mixed together and stirred about in a pan; a hasty pudding.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stiriated</ent><br/
<hw>Stir"i*a`ted</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>stiria</ets> an icicle.]</ety> <def>Adorned with pendants like icicles.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stirious</ent><br/
<hw>Stir"i*ous</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>stiria an icicle</ets>.]</ety> <def>Resembling icicles.</def> <mark>[Obs.]</mark>  <rj><au>Sir T. Browne.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stirk</ent><br/
<hw>Stirk</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[AS. <ets>stric</ets>, from <ets>ste<oacute/r</ets> a steer. See <er>Steer</er> a young ox.]</ety> <def>A young bullock or heifer.</def> <mark>[Prov. Eng. & Scot.]</mark>  <rj><au>Sir W. Scott.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stirless</ent><br/
<hw>Stir"less</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Without stirring; very quiet; motionless.</def> <ldquo/Lying helpless and <xex>stirless</xex>.<rdquo/  <rj><au>Hare.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stirp</ent><br/
<hw>Stirp</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>stirps</ets>, <ets>stirpis</ets>.]</ety> <def>Stock; race; family.</def> <mark>[Obs.]</mark>  <rj><au>Bacon.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stirpiculture</ent><br/
<hw>Stir"pi*cul`ture</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>stirps</ets>, <ets>stirpis</ets>, stem, stock, race + <ets>cultura</ets> culture.]</ety> <def>The breeding of special stocks or races.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stirps</ent><br/
||<hw>Stirps</hw> <pr>(?)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Stirpes</plw> <pr>(#)</pr>.</plu> <ety>[L., stem, stock.]</ety> <sn>1.</sn> <fld>(Law)</fld> <def>Stock; race; family.</def>  <rj><au>Blackstone.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Bot.)</fld> <def>A race, or a fixed and permanent variety.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stirrage</ent><br/
<hw>Stir"rage</hw> <pr>(?)</pr>, <pos>n.</pos> <def>The act of stirring; stir; commotion.</def> <mark>[Obs.]</mark>  <rj><au>T. Granger.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stirrer</ent><br/
<hw>Stir"rer</hw> <pr>(?)</pr>, <pos>n.</pos> <def>One who, or that which, stirs something; also, one who moves about, especially after sleep; <as>as, an early <ex>stirrer</ex></as>.</def>  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Stirrer up</b></col>, <cd>an instigator or inciter.</cd> <au>Atterbury.</au></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stirring</ent><br/
<hw>Stir"ring</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Putting in motion, or being in motion; active; active in business; habitually employed in some kind of business; accustomed to a busy life.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>A more <qex>stirring</qex> and intellectual age than any which had gone before it.</q> <rj><qau>Southey.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><syn><b>Syn.</b> -- Animating; arousing; awakening; stimulating; quickening; exciting.</syn><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stirrup</ent><br/
<hw>Stir"rup</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[OE. <ets>stirop</ets>, AS. <ets>stigr<amac/p</ets>; <ets>st<imac/gan</ets> to mount, ascend + <ets>r<amac/p</ets> a rope; akin to G. <ets>stegreif</ets> a stirrup. <root/164. See <er>Sty</er>, <pos>v. i.</pos>, and <er>Rope</er>.]</ety> <sn>1.</sn> <def>A kind of ring, or bent piece of metal, wood, leather, or the like, horizontal in one part for receiving the foot of a rider, and attached by a strap to the saddle, -- used to assist a person in mounting a horse, and to enable him to sit steadily in riding, as well as to relieve him by supporting a part of the weight of the body.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Our host upon his <qex>stirpoes</qex> stood anon.</q> <rj><qau>Chaucer.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Carp. & Mach.)</fld> <def>Any piece resembling in shape the stirrup of a saddle, and used as a support, clamp, etc. See <er>Bridle iron</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <fld>(Naut.)</fld> <def>A rope secured to a yard, with a thimble in its lower end for supporting a footrope.</def>  <rj><au>Totten.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Stirrup bone</b></col> <fld>(Anat.)</fld>, <cd>the stapes.</cd> -- <col><b>Stirrup cup</b></col>, <cd>a parting cup taken after mounting.</cd> -- <col><b>Stirrup iron</b></col>, <cd>an iron stirrup.</cd> -- <mcol><col><b>Stirrup leather</b></col>, <it>or</it> <col><b>Stirrup strap</b></col></mcol>, <cd>the strap which attaches a stirrup to the saddle. See <er>Stirrup</er>, 1.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stirt</ent><br/
<hw>Stirt</hw> <pr>(?)</pr>, <mark>obs.</mark> <pos>p. p.</pos> <mord>of <er>Start</er>, <pos>v. i.</pos></mord> <def>Started; leaped.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>They privily be <qex>stirt</qex> into a well.</q> <rj><qau>Chaucer.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stirte</ent><br/
<hw>Stir"te</hw> <pr>(?)</pr>, <mark>obs.</mark> <def><pos>imp.</pos> of <er>Start</er>, <pos>v. i. & t.</pos></def>  <rj><au>Chaucer.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stitch</ent><br/
<hw>Stitch</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[OE. <ets>stiche</ets>, AS. <ets>stice</ets> a pricking, akin to <ets>stician</ets> to prick. See <er>Stick</er>, <pos>v. i.</pos>]</ety> <sn>1.</sn> <def>A single pass of a needle in sewing; the loop or turn of the thread thus made.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A single turn of the thread round a needle in knitting; a link, or loop, of yarn; <as>as, to let down, or drop, a <ex>stitch</ex>; to take up a <ex>stitch</ex>.</as></def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <ety>[Cf. OE. <ets>sticche</ets>, <ets>stecche</ets>, <ets>stucche</ets>, a piece, AS. <ets>stycce</ets>.  Cf. <er>Stock</er>.]</ety> <def>A space of work taken up, or gone over, in a single pass of the needle; hence, by extension, any space passed over; distance.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>You have gone a good <qex>stitch</qex>.</q> <rj><qau>Bunyan.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>In Syria the husbandmen go lightly over with their plow, and take no deep <qex>stitch</qex> in making their furrows.</q> <rj><qau>Holland.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>A local sharp pain; an acute pain, like the piercing of a needle; <as>as, a <ex>stitch</ex> in the side</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>He was taken with a cold and with <qex>stitches</qex>, which was, indeed, a pleurisy.</q> <rj><qau>Bp. Burnet.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <def>A contortion, or twist.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>If you talk,<br/
Or pull your face into a <qex>stitch</qex> again,<br/
I shall be angry.</q> <rj><qau>Marston.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>6.</sn> <def>Any least part of a fabric or dress; <as>as, to wet every <ex>stitch</ex> of clothes</as>.</def> <mark>[Colloq.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><sn>7.</sn> <def>A furrow.</def>  <rj><au>Chapman.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>8.</sn> <def>An arrangement of stitches, or method of stitching in some particular way or style; <as>as, cross-<ex>stitch</ex>; herringbone <ex>stitch</ex>, etc.</as></def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><cs><mcol><col><b>Chain stitch</b></col>, <col><b>Lock stitch</b></col></mcol>. <cd>See in the Vocabulary.</cd> -- <mcol><col><b>Pearl stitch</b></col>, <it>or</it>  <col><b>Purl stitch</b></col></mcol>. <cd>See 2nd <er>Purl</er>, 2.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stitch</ent><br/
<hw>Stitch</hw>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Stitched</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Stitching</conjf>.]</vmorph> <sn>1.</sn> <def>To form stitches in; especially, to sew in such a manner as to show on the surface a continuous line of stitches; <as>as, to <ex>stitch</ex> a shirt bosom</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To sew, or unite together by stitches; <as>as, to <ex>stitch</ex> printed sheets in making a book or a pamphlet</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <fld>(Agric.)</fld> <def>To form land into ridges.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>To stitch up</b></col>, <cd>to mend or unite with a needle and thread; <as>as, <ex>to stitch up</ex> a rent; <ex>to stitch up</ex> an artery</as>.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stitch</ent><br/
<hw>Stitch</hw>, <pos>v. i.</pos> <def>To practice stitching, or needlework.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stitchel</ent><br/
<hw>Stitch"el</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A kind of hairy wool.</def> <mark>[Prov.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stitcher</ent><br/
<hw>Stitch"er</hw> <pr>(?)</pr>, <pos>n.</pos> <def>One who stitches; a seamstress.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stitchery</ent><br/
<hw>Stitch"er*y</hw> <pr>(?)</pr>, <pos>n.</pos> <def>Needlework; -- in contempt.</def>  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stitching</ent><br/
<hw>Stitch"ing</hw>, <pos>n.</pos> <sn>1.</sn> <def>The act of one who stitches.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Work done by sewing, esp. when a continuous line of stitches is shown on the surface; stitches, collectively.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stitchwort</ent><br/
<hw>Stitch"wort`</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Bot.)</fld> <def>See <er>Stichwort</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stith</ent><br/
<hw>Stith</hw> <pr>(st<icr/th)</pr>, <pos>a.</pos> <ety>[AS. <ets>st<imac/<edh/</ets>.]</ety> <def>Strong; stiff; rigid.</def> <mark>[Obs. or Prov. Eng.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stith</ent><br/
<hw>Stith</hw>, <pos>n.</pos> <ety>[Icel. <ets>ste<edh/i</ets> an anvil, akin to <ets>sta<edh/r</ets> place. See <er>Stead</er>.]</ety> <def>An anvil; a stithy.</def> <mark>[Obs. or Prov. Eng.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>He invented also pincers, hammers, iron crows, and the anvil, or <qex>stith</qex>.</q> <rj><qau>Holland.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stithy</ent><br/
<hw>Stith"y</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[See <er>Stith</er>, and cf. <er>Stiddy</er>.]</ety> <sn>1.</sn> <def>An anvil.</def>  <rj><au>Sir W. Scott.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A smith's shop; a smithy; a smithery; a forge.</def> <ldquo/As foul as Vulcan's <xex>stithy</xex>.<rdquo/  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stithy</ent><br/
<hw>Stith"y</hw>, <pos>v. t.</pos> <def>To forge on an anvil.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The forge that <qex>stithied</qex> Mars his helm.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stive</ent><br/
<hw>Stive</hw> <pr>(?)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Stived</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Stiving</conjf>.]</vmorph> <ety>[Probably fr. F. <ets>estiver</ets> to compress, stow, L. <ets>stipare</ets>: cf. It. <ets>stivare</ets>, Sp. <ets>estivar</ets>.  Cf. <er>Stevedore</er>, <er>Stiff</er>.]</ety> <def>To stuff; to crowd; to fill full; hence, to make hot and close; to render stifling.</def>  <rj><au>Sandys.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>His chamber was commonly <qex>stived</qex> with friends or suitors of one kind or other.</q> <rj><qau>Sir H. Wotton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stive</ent><br/
<hw>Stive</hw>, <pos>v. i.</pos> <def>To be stifled or suffocated.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stive</ent><br/
<hw>Stive</hw>, <pos>n.</pos> <def>The floating dust in flour mills caused by the operation or grinding.</def>  <rj><au>De Colange.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stiver</ent><br/
<hw>Sti"ver</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[D. <ets>stuiver</ets>; akin to G. <ets>st<uum/ber</ets>, Dan. <ets>styver</ets>, Sw. <ets>styfver</ets>.]</ety> <def>A Dutch coin, and money of account, of the value of two cents, or about one penny sterling; hence, figuratively, anything of little worth.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stives</ent><br/
<hw>Stives</hw> <pr>(?)</pr>, <pos>n. pl.</pos> <ety>[OE. See <er>Stew</er>.]</ety> <def>Stews; a brothel.</def> <mark>[Obs.]</mark>  <rj><au>Chaucer.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stoak</ent><br/
<hw>Stoak</hw> <pr>(?)</pr>, <pos>v. t.</pos> <ety>[Cf. G. <ets>stocken</ets>.]</ety> <fld>(Naut.)</fld> <def>To stop; to choke.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stoat</ent><br/
<hw>Stoat</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[OE. <ets>stot</ets> a stoat, horse, bullock; perhaps originally only of male animals, and akin to D. <ets>stooten</ets> to push, E. <ets>stutter</ets>; cf. Icel. <ets>st<?/tr</ets> a bull, Sw. <ets>stut</ets> a bullock.  Cf. <er>Stot</er>.]</ety> <fld>(Zool.)</fld> <def>The ermine in its summer pelage, when it is reddish brown, but with a black tip to the tail. The name is sometimes applied also to other brown weasels.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stocah</ent><br/
<hw>Sto"cah</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Ir. & Gael. <ets>stocach</ets> an idle fellow who lives on the industry of others, a lounger.]</ety> <def>A menial attendant.</def> <mark>[Obs.]</mark>  <rj><au>Spenser.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stoccade</ent><br/
<hw>Stoc*cade"</hw> <pr>(?)</pr>, <pos>n. & v.</pos> <def>See <er>Stockade</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stoccado</ent><br/
<hw>Stoc*ca"do</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F. <ets>estocade</ets>, fr. Sp. <ets>estocada</ets>, or It. <ets>stoccata</ets>, from Sp. <ets>estoque</ets>, or It. <ets>stocco</ets>, a rapier, fr. G. <ets>stock</ets> a stick. See <er>Stock</er>.]</ety> <def>A stab; a thrust with a rapier.</def>  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><-- p. 1417 --></p>

<p><ent>Stochastic</ent><br/
<hw>Sto*chas"tic</hw> <pr>(st<osl/*k<acr/s"t<icr/k)</pr>, <pos>a.</pos> <ety>[Gr. <grk>stochastiko`s</grk>, from <grk>stocha`zesqai</grk> to aim, to guess, fr. <grk>sto`chos</grk> mark or aim.]</ety> <sn>1.</sn> <def>Conjectural; able to conjecture.</def> <mark>[Obs.]</mark>  <rj><au>Whitefoot.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>random; chance; involving probability; opposite of <ant>deterministic</ant>.</def><br/
[<source>GG</source>]</p>

<p><sn>3.</sn> <fld>(Statistics)</fld> <def>of or pertaining to a process in which a series of calculations, selections, or observations are made, each one being randomly determined as a sample from a probability distribution.</def> <note>Where physical phenomena are modelled as a stochastic process, each subsequent calculation of a series may depend on the result of the previous calculation, as in the modelling of the process of diffusion of molecules.  Many series may be calculated, and the results averaged, to estimate the most likely result.  See also <er>Markov chain</er>.</note><br/
[<source>PJC</source>]</p>

<p>-- <wordforms><wf>sto*chas"tic*al*ly</wf>, <pos>adv.</pos></wordforms><br/
[<source>PJC</source>]</p>

<p><ent>Stock</ent><br/
<hw>Stock</hw> <pr>(st<ocr/k)</pr>, <pos>n.</pos> <ety>[AS. <ets>stocc</ets> a stock, trunk, stick; akin to D. <ets>stok</ets>, G. <ets>stock</ets>, OHG. <ets>stoc</ets>, Icel. <ets>stokkr</ets>, Sw. <ets>stock</ets>, Dan. <ets>stok</ets>, and AS. <ets>stycce</ets> a piece; cf. Skr. <ets>tuj</ets> to urge, thrust.  Cf. <er>Stokker</er>, <er>Stucco</er>, and <er>Tuck</er> a rapier.]</ety> <sn>1.</sn> <def>The stem, or main body, of a tree or plant; the fixed, strong, firm part; the trunk.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Though the root thereof wax old in the earth, and the <qex>stock</qex> thereof die in the ground, yet through the scent of water it will bud, and bring forth boughs like a plant.</q> <rj><qau>Job xiv. 8,9.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>The stem or branch in which a graft is inserted.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The scion overruleth the <qex>stock</qex> quite.</q> <rj><qau>Bacon.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>A block of wood; something fixed and solid; a pillar; a firm support; a post.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>All our fathers worshiped <qex>stocks</qex> and stones.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Item, for a <qex>stock</qex> of brass for the holy water, seven shillings; which, by the canon, must be of marble or metal, and in no case of brick.</q> <rj><qau>Fuller.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>Hence, a person who is as dull and lifeless as a stock or post; one who has little sense.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Let's be no stoics, nor no <qex>stocks</qex>.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <def>The principal supporting part; the part in which others are inserted, or to which they are attached.</def> Specifically: --<br/
[<source>1913 Webster</source>]</p>

<p><sd>(a)</sd> <def>The wood to which the barrel, lock, etc., of a rifle or like firearm are secured; also, a long, rectangular piece of wood, which is an important part of several forms of gun carriage.</def><br/
[<source>1913 Webster</source>]</p>

<p><sd>(b)</sd> <def>The handle or contrivance by which bits are held in boring; a bitstock; a brace.</def><br/
[<source>1913 Webster</source>]</p>

<p><sd>(c)</sd> <fld>(Joinery)</fld> <def>The block of wood or metal frame which constitutes the body of a plane, and in which the plane iron is fitted; a plane stock.</def><br/
[<source>1913 Webster</source>]</p>

<p><sd>(d)</sd> <fld>(Naut.)</fld> <def>The wooden or iron crosspiece to which the shank of an anchor is attached. See <xex>Illust.</xex> of <er>Anchor</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sd>(e)</sd> <def>The support of the block in which an anvil is fixed, or of the anvil itself.</def><br/
[<source>1913 Webster</source>]</p>

<p><sd>(f)</sd> <def>A handle or wrench forming a holder for the dies for cutting screws; a diestock.</def><br/
[<source>1913 Webster</source>]</p>

<p><sd>(g)</sd> <def>The part of a tally formerly struck in the exchequer, which was delivered to the person who had lent the king money on account, as the evidence of indebtedness. See <er>Counterfoil</er>.</def> <mark>[Eng.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><sn>6.</sn> <def>The original progenitor; also, the race or line of a family; the progenitor of a family and his direct descendants; lineage; family.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>And stand betwixt them made, when, severally,<br/
All told their <qex>stock</qex>.</q> <rj><qau>Chapman.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Thy mother was no goddess, nor thy <qex>stock</qex><br/
From Dardanus.</q> <rj><qau>Denham.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>7.</sn> <fld>(Finance)</fld> <def>Money or capital which an individual or a firm employs in business; fund; in the United States, the capital of a bank or other company, in the form of transferable shares, each of a certain amount; money funded in government securities, called also <altname>the public funds</altname>; in the plural, property consisting of shares in joint-stock companies, or in the obligations of a government for its funded debt; -- so in the United States, but in England the latter only are called <altname>stocks</altname>, and the former <altname>shares</altname>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>8.</sn> <fld>(Bookkeeping)</fld> <def>Same as <cref>Stock account</cref>, below.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>9.</sn> <def>Supply provided; store; accumulation; especially, a merchant's or manufacturer's store of goods; <as>as, to lay in a <ex>stock</ex> of provisions</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Add to that <qex>stock</qex> which justly we bestow.</q> <rj><qau>Dryden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>10.</sn> <fld>(Agric.)</fld> <def>Domestic animals or beasts collectively, used or raised on a farm; <as>as, a <ex>stock</ex> of cattle or of sheep, etc.</as>; -- called also <altname>live stock</altname>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>11.</sn> <fld>(Card Playing)</fld> <def>That portion of a pack of cards not distributed to the players at the beginning of certain games, as gleek, etc., but which might be drawn from afterward as occasion required; a bank.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>I must buy the <qex>stock</qex>; send me good cardings.</q> <rj><qau>Beau. & Fl.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>12.</sn> <def>A thrust with a rapier; a stoccado.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><sn>13.</sn> <ety>[Cf. <er>Stocking</er>.]</ety> <def>A covering for the leg, or leg and foot; <as>as, upper <ex>stocks</ex> (breeches); nether <ex>stocks</ex> (stockings).</as></def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>With a linen <qex>stock</qex> on one leg.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>14.</sn> <def>A kind of stiff, wide band or cravat for the neck; <as>as, a silk <ex>stock</ex></as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>15.</sn> <pluf>pl.</pluf> <def>A frame of timber, with holes in which the feet, or the feet and hands, of criminals were formerly confined by way of punishment.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>He shall rest in my <qex>stocks</qex>.</q> <rj><qau>Piers Plowman.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>16.</sn> <pluf>pl.</pluf> <fld>(Shipbuilding)</fld> <def>The frame or timbers on which a ship rests while building.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>17.</sn> <pluf>pl.</pluf> <def>Red and gray bricks, used for the exterior of walls and the front of buildings.</def> <mark>[Eng.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><sn>18.</sn> <fld>(Bot.)</fld> <def>Any cruciferous plant of the genus <gen>Matthiola</gen>; <as>as, common <ex>stock</ex> (<spn>Matthiola incana</spn>) (see <er>Gilly-flower</er>); ten-weeks <ex>stock</ex> (<spn>Matthiola annua</spn>)</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>19.</sn> <fld>(Geol.)</fld> <def>An irregular metalliferous mass filling a large cavity in a rock formation, as a <xex>stock</xex> of lead ore deposited in limestone.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>20.</sn> <def>A race or variety in a species.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>21.</sn> <fld>(Biol.)</fld> <def>In tectology, an aggregate or colony of persons (see <er>Person</er>), as trees, chains of salpae, etc.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>22.</sn> <def>The beater of a fulling mill.</def>  <rj><au>Knight.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>23.</sn> <fld>(Cookery)</fld> <def>A liquid or jelly containing the juices and soluble parts of meat, and certain vegetables, etc., extracted by cooking; -- used in making soup, gravy, etc.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>24.</sn> <def>Raw material; that out of which something is manufactured; <as>as, paper <ex>stock</ex></as>.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><sn>25.</sn>  <fld>(Soap Making)</fld> <def>A plain soap which is made into  toilet soap by adding perfumery, coloring matter, etc.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><cs><col><b>Bit stock</b></col>. <cd>See <er>Bitstock</er>.</cd> -- <col><b>Dead stock</b></col> <fld>(Agric.)</fld>, <cd>the implements of husbandry, and produce stored up for use; -- in distinction from <xex>live stock</xex>, or the domestic animals on the farm. See def. 10, above.</cd> -- <col><b>Head stock</b></col>. <cd>See <er>Headstock</er>.</cd> -- <col><b>Paper stock</b></col>, <cd>rags and other material of which paper is made.</cd> -- <col><b>Stock account</b></col> <fld>(Bookkeeping)</fld>, <cd>an account on a merchant's ledger, one side of which shows the original capital, or stock, and the additions thereto by accumulation or contribution, the other side showing the amounts withdrawn.</cd> -- <col><b>Stock car</b></col>, <cd>a railway car for carrying cattle.</cd> -- <col><b>Stock company</b></col> <fld>(Com.)</fld>, <cd>an incorporated company the capital of which is represented by marketable shares having a certain equal par value.</cd><-- also, joint-stock company --> -- <col><b>Stock duck</b></col> <fld>(Zool.)</fld>, <cd>the mallard.</cd> -- <col><b>Stock exchange</b></col>. <sd>(a)</sd> <cd>The building or place where stocks are bought and sold; stock market; hence, transactions of all kinds in stocks</cd>. <sd>(b)</sd> <cd>An association or body of stockbrokers who meet and transact business by certain recognized forms, regulations, and usages</cd>. <au>Wharton. Brande & C.</au> -- <col><b>Stock farmer</b></col>, <cd>a farmer who makes it his business to rear live stock.</cd> -- <col><b>Stock gillyflower</b></col> <fld>(Bot.)</fld>, <cd>the common stock. See <er>Stock</er>, <pos>n.</pos>, 18.</cd> -- <col><b>Stock gold</b></col>, <cd>gold laid up so as to form a stock, or hoard.</cd> -- <col><b>Stock in trade</b></col>, <cd>the goods kept for sale by a shopkeeper; the fittings and appliances of a workman.</cd> <au>Simmonds.</au> -- <col><b>Stock list</b></col>, <cd>a list of stocks, or shares, dealt in, of transactions, and of prices.</cd> -- <col><b>Stock lock</b></col>, <cd>a lock inclosed in a wooden case and attached to the face of a door.</cd> -- <col><b>Stock market</b></col>. <sd>(a)</sd> <cd>A place where stocks are bought and sold; the stock exchange</cd>. <sd>(b)</sd> <cd>A market for live stock.</cd> -- <col><b>Stock pigeon</b></col>. <fld>(Zool.)</fld> <cd>Same as <er>Stockdove</er>.</cd> -- <col><b>Stock purse</b></col>. <sd>(a)</sd> <cd>A common purse, as distinguished from a private purse.</cd> <sd>(b)</sd> <fld>(Mil.)</fld> <cd>Moneys saved out of the expenses of a company or regiment, and applied to objects of common interest</cd>. <mark>[Eng.]</mark> -- <col><b>Stock shave</b></col>, <cd>a tool used by blockmakers.</cd> -- <col><b>Stock station</b></col>, <cd>a place or district for rearing stock.</cd> <mark>[Australia]</mark> <au>W. Howitt.</au> -- <col><b>Stock tackle</b></col> <fld>(Naut.)</fld>, <cd>a tackle used when the anchor is hoisted and secured, to keep its stock clear of the ship's sides.</cd> <au>Totten.</au> -- <col><b>Stock taking</b></col>, <cd>an examination and inventory made of goods or stock in a shop or warehouse; -- usually made periodically.</cd> -- <col><b>Tail stock</b></col>. <cd>See <er>Tailstock</er>.</cd> -- <col><b>To have something on the stock</b></col>, <cd>to be at work at something.</cd> -- <col><b>To take stock</b></col>, <cd>to take account of stock; to make an inventory of stock or goods on hand.</cd> <au>Dickens.</au> -- <col><b>To take stock in</b></col>. <sd>(a)</sd> <cd>To subscribe for, or purchase, shares in a stock company.</cd> <sd>(b)</sd> <cd>To put faith in; to accept as trustworthy; <as>as, <ex>to take stock in</ex> a person's fidelity</as></cd>. <mark>[Slang]</mark> -- <col><b>To take stock of</b></col>, <cd>to take account of the stock of; to take an inventory of; hence, to ascertain the facts in regard to (something).</cd> <mark>[Eng.]</mark></cs><br/
[<source>1913 Webster</source>]</p>

<p><q>At the outset of any inquiry it is proper <qex>to take stock of</qex> the results obtained by previous explorers of the same field.</q> <rj><qau>Leslie Stephen.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><syn><b>Syn.</b> -- Fund; capital; store; supply; accumulation; hoard; provision.</syn><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stock</ent><br/
<hw>Stock</hw> <pr>(st<ocr/k)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Stocked</conjf> <pr>(st<ocr/kt)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Stocking</conjf>.]</vmorph> <sn>1.</sn> <def>To lay up; to put aside for future use; to store, as merchandise, and the like.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To provide with material requisites; to store; to fill; to supply; <as>as, to <ex>stock</ex> a warehouse, that is, to fill it with goods; to <ex>stock</ex> a farm, that is, to supply it with cattle and tools; to <ex>stock</ex> land, that is, to occupy it with a permanent growth, especially of grass.</as></def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>To suffer to retain milk for twenty-four hours or more previous to sale, as cows.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>To put in the stocks.</def> <mark>[R.]</mark>  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>To stock an anchor</b></col> <fld>(Naut.)</fld>, <cd>to fit it with a stock, or to fasten the stock firmly in place.</cd> -- <col><b>To stock cards</b></col> <fld>(Card Playing)</fld>, <cd>to arrange cards in a certain manner for cheating purposes; -- also called <altname>to stack the deck</altname>.</cd> <mark>[Cant]</mark> -- <col><b>To stock down</b></col> <fld>(Agric.)</fld>, <cd>to sow, as plowed land, with grass seed, in order that it may become swarded, and produce grass.</cd> -- <col><b>To stock up</b></col>, <cd>to extirpate; to dig up.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stock</ent><br/
<hw>Stock</hw>, <pos>a.</pos> <def>Used or employed for constant service or application, as if constituting a portion of a stock or supply; standard; permanent; standing; <as>as, a <ex>stock</ex> actor; a <ex>stock</ex> play; a <ex>stock</ex> phrase; a <ex>stock</ex> response; a <ex>stock</ex> sermon.</as></def> <ldquo/A <xex>stock</xex> charge against Raleigh.<rdquo/  <rj><au>C. Kingsley.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Stock company</b></col> <fld>(Theater)</fld>, <cd>a company of actors regularly employed at one theater, or permanently acting together in various plays under one management.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stockade</ent><br/
<hw>Stock*ade"</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F. <ets>estacade</ets> stockade, boom (confused in French with <ets>estocade</ets>; see 1st <er>Stoccado</er>); fr. It. <ets>steccata</ets> a palisade (influenced by OF. <ets>estach</ets>, <ets>estaque</ets>, a stake, post), or from Sp. <ets>estacada</ets> a palisade; both of German origin, and akin to E. <ets>stake</ets>, <ets>stick</ets>; cf. G. <ets>stecken</ets> stick, OHG. <ets>steccho</ets>. See <er>Stake</er>, <pos>n.</pos>, <er>Stick</er>, <pos>n.</pos> & <pos>v. t.</pos>, and cf. <er>Estacade</er>, <er>Stacket</er>.]</ety> <sn>1.</sn> <fld>(Mil.)</fld> <def>A line of stout posts or timbers set firmly in the earth in contact with each other (and usually with loopholes) to form a barrier, or defensive fortification.</def> <altsp>[Written also <asp>stoccade</asp>.]</altsp><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>An inclosure, or pen, made with posts and stakes.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stockade</ent><br/
<hw>Stock*ade"</hw>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Stockaded</conjf>; <pos>p. pr. & vb. n.</pos> <conjf>Stockading</conjf>.]</vmorph> <def>To surround, fortify, or protect with a stockade.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stock-blind</ent><br/
<hw>Stock"-blind`</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Blind as a stock; wholly blind.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stockbroker</ent><br/
<hw>Stock"bro`ker</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A broker who deals in stocks.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stockdove</ent><br/
<hw>Stock"dove`</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Zool.)</fld> <def>A common European wild pigeon (<spn>Columba aenas</spn>), so called because at one time believed to be the stock of the domestic pigeon, or, according to some, from its breeding in the stocks, or trunks, of trees.</def><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ The name is applied, also, to other related species, as the Indian <xex>stockdove</xex> (<spn>Palumbaena Eversmanni</spn>).</note><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stocker</ent><br/
<hw>Stock"er</hw> <pr>(?)</pr>, <pos>n.</pos> <def>One who makes or fits stocks, as of guns or gun carriages, etc.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stockfish</ent><br/
<hw>Stock"fish`</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Cf. D. <ets>stokvisch</ets>.]</ety> <sn>1.</sn> <def>Salted and dried fish, especially codfish, hake, ling, and torsk; also, codfish dried without being salted.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Zool.)</fld> <def>Young fresh cod.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stockholder</ent><br/
<hw>Stock"hold`er</hw> <pr>(?)</pr>, <pos>n.</pos> <def>One who is a holder or proprietor of stock in the public funds, or in the funds of a bank or other stock company.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stockinet</ent><br/
<hw>Stock`i*net"</hw> <pr>(?)</pr>, <pos>n.</pos> <def>An elastic textile fabric imitating knitting, of which stockings, under-garments, etc., are made.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stocking</ent><br/
<hw>Stock"ing</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[From <er>Stock</er>, which was formerly used of a covering for the legs and feet, combining breeches, or upper stocks, and <ets>stockings</ets>, or nether stocks.]</ety> <sn>1.</sn> <def>A close-fitting covering for the foot and leg, usually knit or woven.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Any of various things resembling, or likened to, a stocking{1};</def> <specif>as:</specif> <sd>(a)</sd> <def>A broad ring of color, differing from the general color, on the lower part of the leg of a quadruped; esp., a white ring between the coronet and the hock or knee of a dark-colored horse.</def> <sd>(b)</sd> <def>A knitted hood of cotton thread which is eventually converted by a special process into an incandescent mantle for gas lighting.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><cs><col><b>Blue stocking</b></col>. <cd>See <er>Bluestocking</er>.</cd> -- <col><b>Stocking frame</b></col>, <cd>a machine for knitting stockings or other hosiery goods.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stocking</ent><br/
<hw>Stock"ing</hw>, <pos>v. t.</pos> <def>To dress in GBs.</def>  <rj><au>Dryden.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stockinger</ent><br/
<hw>Stock"ing*er</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A stocking weaver.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stockish</ent><br/
<hw>Stock"ish</hw>, <pos>a.</pos> <def>Like a stock; stupid; blockish.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Since naught so <qex>stockish</qex>, hard, and full of rage,<br/
But music for the time doth change his nature.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stockjobber</ent><br/
<hw>Stock"job`ber</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[<ets>Stock</ets> + <ets>job</ets>.]</ety> <def>One who speculates in stocks for gain; one whose occupation is to buy and sell stocks. In England a jobber acts as an intermediary between brokers.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stockjobbing</ent><br/
<hw>Stock"job`bing</hw> <pr>(?)</pr>, <pos>n.</pos> <def>The act or art of dealing in stocks; the business of a stockjobber.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stockman</ent><br/
<hw>Stock"man</hw> <pr>(?)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Stockmen</plw> <pr>(<?/)</pr>.</plu> <def>A herdsman; a ranchman; one owning, or having charge of, herds of live stock.</def> <mark>[Australia & U.S.]</mark>  <rj><au>W. Howitt.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stock-still</ent><br/
<hw>Stock"-still`</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[CF. G. <ets>stock-still</ets>.]</ety> <def>Still as a stock, or fixed post; perfectly still.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>His whole work stands <qex>stock-still</qex>.</q> <rj><qau>Sterne.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stockwork</ent><br/
<hw>Stock"work`</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[G. <ets>stockwerk</ets>.]</ety> <sn>1.</sn> <fld>(Mining)</fld> <def>A system of working in ore, etc., when it lies not in strata or veins, but in solid masses, so as to be worked in chambers or stories.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Geol.)</fld> <def>A metalliferous deposit characterized by the impregnation of the mass of rock with many small veins or nests irregularly grouped. This kind of deposit is especially common with tin ore. Such deposits are worked in floors or stories.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stocky</ent><br/
<hw>Stock"y</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[From <er>Stock</er>.]</ety> <sn>1.</sn> <def>Short and thick; thick rather than tall or corpulent.</def>  <rj><au>Addison.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q><qex>Stocky</qex>, twisted, hunchback stems.</q> <rj><qau>Mrs. H. H. Jackson.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Headstrong.</def> <mark>[Prov. Eng.]</mark>  <rj><au>G. Eliot.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stodgy</ent><br/
<hw>Stodg"y</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Wet.</def> <mark>[Prov. Eng.]</mark>  <rj><au>G. Eliot.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stoechiometry</ent><br/
<ent>Stoechiology</ent><br/
<mhw><hw>Stoech`i*ol"o*gy</hw> <pr>(?)</pr>, <pos>n.</pos>, <hw>Stoech`i*om"e*try</hw> <pr>(<?/)</pr>, <pos>n.</pos>, etc.</mhw> <def>See <er>Stoichiology</er>, <er>Stoichiometry</er>, etc.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stogie</ent><br/
<hw>Sto"gie</hw>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Stogies</plw> <pr>(<?/)</pr></plu>. <def>an inexpensive cigar; same as <er>stogy</er>.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Stogy</ent><br/
<hw>Sto"gy</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[Etym. uncertain. Cf. <er>Stocky</er>.]</ety> <def>heavy; coarse; clumsy.</def> <mark>[Colloq.]</mark><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Stogy</ent><br/
<hw>Sto"gy</hw>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Stogies</plw> <pr>(<?/)</pr></plu>. <altsp>[Written also <asp>stogie</asp>.]</altsp> <ety>[<ets>Colloq</ets>.]</ety> <sn>1.</sn> <def>A stout, coarse boot or shoe; a brogan.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><sn>2.</sn>  <def>A kind of cheap, but not necessary inferior, cigar made in the form of a cylindrical roll.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Stoic</ent><br/
<hw>Sto"ic</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>stoicus</ets>, Gr. <?/, fr. <?/, adj., literally, of or pertaining to a colonnade, from <?/ a roofed colonnade, a porch, especially, a porch in Athens where Zeno and his successors taught.]</ety> <sn>1.</sn> <def>A disciple of the philosopher Zeno; one of a Greek sect which held that men should be free from passion, unmoved by joy or grief, and should submit without complaint to unavoidable necessity, by which all things are governed.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Hence, a person not easily excited; an apathetic person; one who is apparently or professedly indifferent to pleasure or pain.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>A <qex>Stoic</qex> of the woods, a man without a tear.</q> <rj><qau>Campbell.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>School of Stoics</b></col>. <cd>See <cref>The Porch</cref>, under <er>Porch</er>.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stoical</ent><br/
<ent>Stoic</ent><br/
<mhw>{ <hw>Sto"ic</hw> <pr>(?)</pr>, <hw>Sto"ic*al</hw> <pr>(?)</pr>, }</mhw> <pos>a.</pos> <ety>[L. <ets>stoicus</ets>, Gr. <?/: cf. F. <ets>sto<ium/que</ets>. See <er>Stoic</er>, <pos>n.</pos>]</ety> <sn>1.</sn> <def>Of or pertaining to the Stoics; resembling the Stoics or their doctrines.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Not affected by passion; manifesting indifference to pleasure or pain; especially, bearing pain, suffering, or bad fortune without complaint.</def> -- <wordforms><wf>Sto"ic*al*ly</wf>, <pos>adv.</pos> -- <wf>Sto"ic*al*ness</wf>, <pos>n.</pos></wordforms><br/
[<source>1913 Webster</source> <source>+PJC</source>]</p>

<p><ent>Stoichiological</ent><br/
<hw>Stoi`chi*o*log"ic*al</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Of or pertaining to stoichiology.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stoichiology</ent><br/
<hw>Stoi`chi*ol"o*gy</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Gr. <?/ a first element + <ets>-logy</ets>.]</ety> <altsp>[Written also <asp>stoechiology</asp>.]</altsp> <sn>1.</sn> <def>That part of the science of physiology which treats of the elements, or principles, composing animal tissues.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Logic)</fld> <def>The doctrine of the elementary requisites of mere thought.</def>  <rj><au>Sir W. Hamilton.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>The statement or discussion of the first principles of any science or art.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stoichiometrical</ent><br/
<ent>Stoichiometric</ent><br/
<mhw>{ <hw>Stoi`chi*o*met"ric</hw> <pr>(?)</pr>, <hw>Stoi`chi*o*met"ric*al</hw> <pr>(?)</pr>, }</mhw> <pos>a.</pos> <def>Of or pertaining to stoichiometry; employed in, or obtained by, stoichiometry.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Chem.)</fld> <def>of, pertaining to, using, or consuming, reagents in the exact proportions required for a given reaction; <as>as, a <ex>stoichiometric</ex> reaction, <it>i. e.</it>, a reaction which goes to completion, rather than stopping partway at an equilibrium point</as>.</def><br/
[<source>PJC</source>]</p>

<p><ent>Stoichiometry</ent><br/
<hw>Stoi`chi*om"e*try</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Gr. <?/ a first principle, or element + <ets>-metry</ets>.]</ety> <def>The art or process of calculating the atomic proportions, combining weights, and other numerical relations of chemical elements and their compounds.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stoicism</ent><br/
<hw>Sto"i*cism</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Cf. F. <ets>sto<ium/cisme</ets>.]</ety> <sn>1.</sn> <def>The opinions and maxims of the Stoics.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A real or pretended indifference to pleasure or pain; insensibility; impassiveness.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stoicity</ent><br/
<hw>Sto*ic"i*ty</hw> <pr>(?)</pr>, <pos>n.</pos> <def>Stoicism.</def> <mark>[Obs.]</mark>  <rj><au>B. Jonson.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stoke</ent><br/
<hw>Stoke</hw> <pr>(?)</pr>, <pos>v. t.</pos> <ety>[OE. <ets>stoken</ets>, fr. D. <ets>stoken</ets>, fr. <ets>stok</ets> a stick (cf. OF. <ets>estoquier</ets> to thrust, stab; of Teutonic origin, and akin to D. <ets>stok</ets>). See <er>Stock</er>.]</ety> <sn>1.</sn> <def>To stick; to thrust; to stab.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>Nor short sword for to <qex>stoke</qex>, with point biting.</q> <rj><qau>Chaucer.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To poke or stir up, as a fire; hence, to tend, as the fire of a furnace, boiler, etc.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stoke</ent><br/
<hw>Stoke</hw>, <pos>v. i.</pos> <def>To poke or stir up a fire; hence, to tend the fires of furnaces, steamers, etc.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stokehold</ent><br/
<hw>Stoke"hold`</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Naut.)</fld> <def>The space, or any of the spaces, in front of the boilers of a ship, from which the furnaces are fed; the stokehole of a ship; also, a room containing a ship's boilers; <as>as, forced draft with  closed <ex>stokehold</ex></as>; -- called also, in American ships, <altname>fireroom</altname>.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Stokehole</ent><br/
<hw>Stoke"hole`</hw> <pr>(?)</pr>, <pos>n.</pos> <def>The mouth to the grate of a furnace; also, the space in front of the furnace, where the stokers stand.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stoker</ent><br/
<hw>Stok"er</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[D. See <er>Stoke</er>, <pos>v. t.</pos>]</ety> <sn>1.</sn> <def>One who is employed to tend a furnace and supply it with fuel, especially the furnace of a locomotive or of a marine steam boiler; also, a machine for feeding fuel to a fire.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A fire poker.</def> <mark>[R.]</mark>  <rj><au>C. Richardson (Dict.).</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stokey</ent><br/
<hw>Sto"key</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Close; sultry.</def> <mark>[Prov. Eng.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><-- p. 1418 --></p>

<p><ent>Stola</ent><br/
||<hw>Sto"la</hw> <pr>(?)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Stolae</plw> <pr>(#)</pr>.</plu> <ety>[L. See <er>Stole</er> a garment.]</ety> <fld>(Rom. Antiq.)</fld> <def>A long garment, descending to the ankles, worn by Roman women.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The <qex>stola</qex> was not allowed to be worn by courtesans, or by women who had been divorced from their husbands.</q> <rj><qau>Fairholt.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stole</ent><br/
<hw>Stole</hw> <pr>(?)</pr>, <def><pos>imp.</pos> of <er>Steal</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stole</ent><br/
<hw>Stole</hw>, <pos>n.</pos> <ety>[L. <ets>stolo</ets>, <ets>-onis</ets>.]</ety> <fld>(Bot.)</fld> <def>A stolon.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stole</ent><br/
<hw>Stole</hw>, <pos>n.</pos> <ety>[AS. <ets>stole</ets>, L. <ets>stola</ets>, Gr. <?/ a stole, garment, equipment, fr. <?/ to set, place, equip, send, akin to E. <ets>stall</ets>. See <er>Stall</er>.]</ety> <sn>1.</sn> <def>A long, loose garment reaching to the feet.</def>  <rj><au>Spenser.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>But when mild morn, in saffron <qex>stole</qex>,<br/
First issues from her eastern goal.</q> <rj><qau>T. Warton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Eccl.)</fld> <def>A narrow band of silk or stuff, sometimes enriched with embroidery and jewels, worn on the left shoulder of deacons, and across both shoulders of bishops and priests, pendent on each side nearly to the ground. At Mass, it is worn crossed on the breast by priests. It is used in various sacred functions.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Groom of the stole</b></col>, <cd>the first lord of the bedchamber in the royal household.</cd> <mark>[Eng.]</mark> <au>Brande & C.</au></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stoled</ent><br/
<hw>Stoled</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Having or wearing a stole.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>After them flew the prophets, brightly <qex>stoled</qex><br/
In shining lawn.</q> <rj><qau>G. Fletcher.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stolen</ent><br/
<hw>Stol"en</hw> <pr>(?)</pr>, <def><pos>p. p.</pos> of <er>Steal</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stolid</ent><br/
<hw>Stol"id</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>stolidus</ets>.]</ety> <def>Hopelessly insensible or stupid; not easily aroused or excited; dull; impassive; foolish.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stolidity</ent><br/
<hw>Sto*lid"i*ty</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>stoliditas</ets>.]</ety> <def>The state or quality of being stolid; dullness of intellect; obtuseness; stupidity.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Indocile, intractable fools, whose <qex>stolidity</qex> can baffle all arguments, and be proof against demonstration itself.</q> <rj><qau>Bentley.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stolidness</ent><br/
<hw>Stol"id*ness</hw> <pr>(?)</pr>, <pos>n.</pos> <def>Same as <er>Stolidity</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stolon</ent><br/
<hw>Sto"lon</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>stolo</ets>, <ets>-onis</ets>: cf. F. <ets>stolon</ets>.  Cf. <er>Stole</er> a stolon, 1st <er>Stool</er>.]</ety> <sn>1.</sn> <fld>(Bot.)</fld> <def>A trailing branch which is disposed to take root at the end or at the joints; a stole.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Zool.)</fld> <def>An extension of the integument of the body, or of the body wall, from which buds are developed, giving rise to new zooids, and thus forming a compound animal in which the zooids usually remain united by the stolons. Such stolons are often present in Anthozoa, Hydroidea, Bryozoa, and social ascidians. See <xex>Illust.</xex> under <er>Scyphistoma</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stoloniferous</ent><br/
<hw>Stol`o*nif"er*ous</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[<ets>Stolon</ets> + <ets>-ferous</ets>: cf. F. <ets>stolonif<egrave/re</ets>.]</ety> <def>Producing stolons; putting forth suckers.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stoma</ent><br/
||<hw>Sto"ma</hw> <pr>(?)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Stomata</plw> <pr>(#)</pr>.</plu> <ety>[NL., fr. Gr. <?/, <?/, a mouth.]</ety> <sn>1.</sn> <fld>(Anat.)</fld> <def>One of the minute apertures between the cells in many serous membranes.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Bot.)</fld> <sd>(a)</sd> <def>The minute breathing pores of leaves or other organs opening into the intercellular spaces, and usually bordered by two contractile cells.</def> <sd>(b)</sd> <def>The line of dehiscence of the sporangium of a fern. It is usually marked by two transversely elongated cells. See <xex>Illust.</xex> of <er>Sporangium</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <fld>(Zool.)</fld> <def>A stigma. See <er>Stigma</er>, <pos>n.</pos>, 6 <sd>(a)</sd> & <sd>(b)</sd>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stomach</ent><br/
<hw>Stom"ach</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[OE. <ets>stomak</ets>, F. <ets>estomac</ets>, L. <ets>stomachus</ets>, fr. Gr. <grk>sto`machos</grk> stomach, throat, gullet, fr. <grk>sto`ma</grk> a mouth, any outlet or entrance.]</ety> <sn>1.</sn> <fld>(Anat.)</fld> <def>An enlargement, or series of enlargements, in the anterior part of the alimentary canal, in which food is digested; any cavity in which digestion takes place in an animal; a digestive cavity. See <er>Digestion</er>, and <cref>Gastric juice</cref>, under <er>Gastric</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>The desire for food caused by hunger; appetite; <as>as, a good <ex>stomach</ex> for roast beef</as>.</def>  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>Hence appetite in general; inclination; desire.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>He which hath no <qex>stomach</qex> to this fight,<br/
Let him depart.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>Violence of temper; anger; sullenness; resentment; willful obstinacy; stubbornness.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>Stern was his look, and full of <qex>stomach</qex> vain.</q> <rj><qau>Spenser.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>This sort of crying proceeding from pride, obstinacy, and <qex>stomach</qex>, the will, where the fault lies, must be bent.</q> <rj><qau>Locke.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <def>Pride; haughtiness; arrogance.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>He was a man<br/
Of an unbounded <qex>stomach</qex>.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Stomach pump</b></col> <fld>(Med.)</fld>, <cd>a small pump or syringe with a flexible tube, for drawing liquids from the stomach, or for injecting them into it.</cd> -- <col><b>Stomach tube</b></col> <fld>(Med.)</fld>, <cd>a long flexible tube for introduction into the stomach.</cd> -- <col><b>Stomach worm</b></col> <fld>(Zool.)</fld>, <cd>the common roundworm (<spn>Ascaris lumbricoides</spn>) found in the human intestine, and rarely in the stomach.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stomach</ent><br/
<hw>Stom"ach</hw>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Stomached</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Stomaching</conjf>.]</vmorph> <ety>[Cf. L. <ets>stomachari</ets>, v.t. & i., to be angry or vexed at a thing.]</ety> <sn>1.</sn> <def>To resent; to remember with anger; to dislike.</def>  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>The lion began to show his teeth, and to <qex>stomach</qex> the affront.</q> <rj><qau>L'Estrange.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>The Parliament sit in that body . . . to be his counselors and dictators, though he <qex>stomach</qex> it.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To bear without repugnance; to brook.</def> <mark>[Colloq.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stomach</ent><br/
<hw>Stom"ach</hw>, <pos>v. i.</pos> <def>To be angry.</def> <mark>[Obs.]</mark>  <rj><au>Hooker.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stomachal</ent><br/
<hw>Stom"ach*al</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[Cf. F. <ets>stomacal</ets>.]</ety> <sn>1.</sn> <def>Of or pertaining to the stomach; gastric.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Helping the stomach; stomachic; cordial.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stomachal</ent><br/
<hw>Stom"ach*al</hw>, <pos>n.</pos> <def>A stomachic.</def>  <rj><au>Dunglison.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stomacher</ent><br/
<hw>Stom"ach*er</hw> <pr>(?)</pr>, <pos>n.</pos> <sn>1.</sn> <def>One who stomachs.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <pr>(<?/ <or/ <?/)</pr> <def>An ornamental covering for the breast, worn originally both by men and women. Those worn by women were often richly decorated.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>A stately lady in a diamond <qex>stomacher</qex>.</q> <rj><qau>Johnson.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stomachful</ent><br/
<hw>Stom"ach*ful</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Willfully obstinate; stubborn; perverse.</def> <mark>[Obs.]</mark> -- <wordforms><wf>Stom"ach*ful*ly</wf>, <pos>adv.</pos> <mark>[Obs.]</mark> -- <wf>Stom"ach*ful*ness</wf>, <pos>n.</pos> <mark>[Obs.]</mark></wordforms><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stomachical</ent><br/
<ent>Stomachic</ent><br/
<mhw>{ <hw>Sto*mach"ic</hw> <pr>(?)</pr>, <hw>Sto*mach"ic*al</hw> <pr>(?)</pr>, }</mhw> <pos>a.</pos> <ety>[L. <ets>stomachicus</ets>, Gr. <?/: cf. F. <ets>stomachique</ets>.]</ety> <sn>1.</sn> <def>Of or pertaining to the stomach; <as>as, <ex>stomachic</ex> vessels</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Strengthening to the stomach; exciting the action of the stomach; stomachal; cordial.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stomachic</ent><br/
<hw>Sto*mach"ic</hw>, <pos>n.</pos> <fld>(Med.)</fld> <def>A medicine that strengthens the stomach and excites its action.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stomaching</ent><br/
<hw>Stom"ach*ing</hw> <pr>(?)</pr>, <pos>n.</pos> <def>Resentment.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stomachless</ent><br/
<hw>Stom"ach*less</hw>, <pos>a.</pos> <sn>1.</sn> <def>Being without a stomach.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Having no appetite.</def> <mark>[R.]</mark>  <rj><au>Bp. Hall.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stomachous</ent><br/
<hw>Stom"ach*ous</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>stomachosus</ets> angry, peexish. See <er>Stomach</er>.]</ety> <def>Stout; sullen; obstinate.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>With stern looks and <qex>stomachous</qex> disdain.</q> <rj><qau>Spenser.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stomachy</ent><br/
<hw>Stom"ach*y</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Obstinate; sullen; haughty.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>A little, bold, solemn, <qex>stomachy</qex> man, a great professor of piety.</q> <rj><qau>R. L. Stevenson.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stomapod</ent><br/
<hw>Sto"ma*pod</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Zool.)</fld> <def>One of the Stomapoda.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stomapoda</ent><br/
||<hw>Sto*map"o*da</hw> <pr>(?)</pr>, <pos>n. pl.</pos> <ety>[NL. See <er>Stoma</er>, and <er>-poda</er>.]</ety> <fld>(Zool.)</fld> <def>An order of Crustacea including the squillas. The maxillipeds are leglike in form, and the large claws are comblike. They have a large and elongated abdomen, which contains a part of the stomach and heart; the abdominal appendages are large, and bear the gills. Called also <altname>Gastrula</altname>, <altname>Stomatopoda</altname>, and <altname>Squilloidea</altname>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stomate</ent><br/
<hw>Sto"mate</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Bot.)</fld> <def>A stoma.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stomatic</ent><br/
<hw>Sto*mat"ic</hw> <pr>(?)</pr>, <pos>a.</pos> <fld>(Bot.)</fld> <def>Of or pertaining to a stoma; of the nature of a stoma.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stomatic</ent><br/
<hw>Sto*mat"ic</hw>, <pos>n.</pos> <ety>[Gr. <grk>sto`ma</grk>, <grk>-atos</grk>, mouth.]</ety> <fld>(Med.)</fld> <def>A medicine for diseases of the mouth.</def>  <rj><au>Dunglison.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stomatiferous</ent><br/
<hw>Stom`a*tif"er*ous</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[Gr. <grk>sto`ma</grk>, <grk>-atos</grk> mouth + <ets>-ferous</ets>.]</ety> <def>Having or producing stomata.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stomatitis</ent><br/
||<hw>Stom`a*ti"tis</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[NL., fr. Gr. <grk>sto`ma</grk>, <grk>-atos</grk>, mouth + <ets>-itis</ets>.]</ety> <fld>(Med.)</fld> <def>Inflammation of the mouth.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stomatoda</ent><br/
||<hw>Stom`a*to"da</hw> <pr>(?)</pr>, <pos>n. pl.</pos> <ety>[NL., fr. Gr. <?/, <?/, mouth.]</ety> <fld>(Zool.)</fld> <def>A division of Protozoa in which a mouthlike opening exists.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stomatodaeum</ent><br/
||<hw>Stom`a*to*dae"um</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Anat.)</fld> <def>Same as <er>Stomodaeum</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stomatode</ent><br/
<hw>Stom"a*tode</hw> <pr>(?)</pr>, <pos>a.</pos> <fld>(Zool.)</fld> <def>Having a mouth; -- applied to certain Protozoa.</def> -- <def2><pos>n.</pos> <def>One of the Stomatoda.</def></def2><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stomatogastric</ent><br/
<hw>Stom`a*to*gas"tric</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[Gr. <?/, <?/, mouth + E. <ets>gastric</ets>.]</ety> <def>Of or pertaining to the mouth and the stomach; <as>as, the <ex>stomatogastric</ex> ganglion of certain Mollusca</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stomatology</ent><br/
<hw>Sto`ma*tol"o*gy</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Gr. <?/, <?/, mouth + <ets>-logy</ets>.]</ety> <fld>(Med.)</fld> <def>Scientific study or knowledge of the mouth.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Stomatoplastic</ent><br/
<hw>Stom`a*to*plas"tic</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[Gr. <?/, <?/, mouth + <ets>-plastic</ets>.]</ety> <fld>(Med.)</fld> <def>Of or pertaining to the operation of forming a mouth where the aperture has been contracted, or in any way deformed.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stomatoplasty</ent><br/
<hw>Stom"a*to*plas`ty</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Gr. <?/, <?/, mouth + <ets>-plasty</ets>.]</ety> <def>Plastic surgery of the mouth.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Stomatopod</ent><br/
<hw>Stom"a*to*pod</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Zool.)</fld> <def>One of the Stomatopoda.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stomatopoda</ent><br/
||<hw>Stom`a*top"o*da</hw> <pr>(?)</pr>, <pos>n. pl.</pos> <ety>[NL. See <er>Stoma</er>, and <er>-pod</er>.]</ety> <fld>(Zool.)</fld> <def>Same as <er>Stomapoda</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stomatopodous</ent><br/
<hw>Stom`a*top"o*dous</hw> <pr>(?)</pr>, <pos>a.</pos> <fld>(Zool.)</fld> <def>Of or pertaining to the Stomatopoda.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stomatoscope</ent><br/
<hw>Stom"a*to*scope</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Gr. <?/, <?/, mouth + <ets>-scope</ets>.]</ety> <fld>(Med.)</fld> <def>An apparatus for examining the interior of the mouth.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stomatous</ent><br/
<hw>Stom"a*tous</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Having a stoma.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stomodaeum</ent><br/
||<hw>Stom`o*dae"um</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[NL., from Gr. <?/, <?/, mouth + <?/ to divide.]</ety> <sn>1.</sn> <fld>(Anat.)</fld> <def>A part of the alimentary canal. See under <er>Mesenteron</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Zool.)</fld> <def>The primitive mouth and esophagus of the embryo of annelids and arthropods.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stomp</ent><br/
<hw>Stomp</hw> <pr>(?)</pr>, <pos>v. i.</pos> <ety>[See <er>Stamp</er>.]</ety> <def>To stamp with the foot.</def> <mark>[Colloq.]</mark> <ldquo/In gallant procession, the priests mean to <xex>stomp</xex>.<rdquo/  <rj><au>R. Browning.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>stomping ground</ent><br/
<hw>stomping ground</hw> <pr>(?)</pr>, <pos>n.</pos> <def>Same as <cref>stamping ground</cref>.  See under <er>stamp</er>.</def><br/
[<source>PJC</source>]</p>

<p><ent>Stond</ent><br/
<hw>Stond</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[For <ets>stand</ets>.]</ety> <sn>1.</sn> <def>Stop; halt; hindrance.</def> <mark>[Obs.]</mark>  <rj><au>Bacon.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A stand; a post; a station.</def> <mark>[Obs.]</mark>  <rj><au>Spenser.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stond</ent><br/
<hw>Stond</hw>, <pos>v. i.</pos> <def>To stand.</def> <mark>[Obs.]</mark>  <rj><au>Chaucer.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stone</ent><br/
<hw>Stone</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[OE. <ets>ston</ets>, <ets>stan</ets>, AS. <ets>st<amac/n</ets>; akin to OS. & OFries. <ets>st<emac/n</ets>, D. <ets>steen</ets>, G. <ets>stein</ets>, Icel. <ets>steinn</ets>, Sw. <ets>sten</ets>, Dan. <ets>steen</ets>, Goth. <ets>stains</ets>, Russ. <ets>stiena</ets> a wall, Gr. <?/, <?/, a pebble. <root/167.  Cf. <er>Steen</er>.]</ety> <sn>1.</sn> <def>Concreted earthy or mineral matter; also, any particular mass of such matter; <as>as, a house built of <ex>stone</ex>; the boy threw a <ex>stone</ex>; pebbles are rounded <ex>stones</ex>.</as></def> <ldquo/Dumb as a <xex>stone</xex>.<rdquo/  <rj><au>Chaucer.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>They had brick for <qex>stone</qex>, and slime . . . for mortar.</q> <rj><qau>Gen. xi. 3.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ In popular language, very large masses of stone are called <xex>rocks</xex>; small masses are called <xex>stones</xex>; and the finer kinds, <xex>gravel</xex>, or <xex>sand</xex>, or <xex>grains of sand</xex>. Stone is much and widely used in the construction of buildings of all kinds, for walls, fences, piers, abutments, arches, monuments, sculpture, and the like.</note><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A precious stone; a gem.</def> <ldquo/Many a rich <xex>stone</xex>.<rdquo/ <au>Chaucer.</au> <ldquo/Inestimable <xex>stones</xex>, unvalued jewels.<rdquo/ <au>Shak.</au><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>Something made of stone. Specifically: -</def><br/
[<source>1913 Webster</source>]</p>

<p><sd>(a)</sd> <def>The glass of a mirror; a mirror.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>Lend me a looking-glass;<br/
If that her breath will mist or stain the <qex>stone</qex>,<br/
Why, then she lives.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sd>(b)</sd> <def>A monument to the dead; a gravestone.</def>  <rj><au>Gray.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Should some relenting eye<br/
Glance on the where our cold relics lie.</q> <rj><qau>Pope.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <fld>(Med.)</fld> <def>A calculous concretion, especially one in the kidneys or bladder; the disease arising from a calculus.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <def>One of the testes; a testicle.</def>  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>6.</sn> <fld>(Bot.)</fld> <def>The hard endocarp of drupes; <as>as, the <ex>stone</ex> of a cherry or peach</as>. See <xex>Illust.</xex> of <er>Endocarp</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>7.</sn> <def>A weight which legally is fourteen pounds, but in practice varies with the article weighed.</def> <mark>[Eng.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ The <xex>stone</xex> of butchers' meat or fish is reckoned at 8 lbs.; of cheese, 16 lbs.; of hemp, 32 lbs.; of glass, 5 lbs.</note><br/
[<source>1913 Webster</source>]</p>

<p><sn>8.</sn> <def>Fig.: Symbol of hardness and insensibility; torpidness; insensibility; <as>as, a heart of <ex>stone</ex></as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>I have not yet forgot myself to <qex>stone</qex>.</q> <rj><qau>Pope.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>9.</sn> <fld>(Print.)</fld> <def>A stand or table with a smooth, flat top of stone, commonly marble, on which to arrange the pages of a book, newspaper, etc., before printing; -- called also <altname>imposing stone</altname>.</def><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ <xex>Stone</xex> is used adjectively or in composition with other words to denote <xex>made of stone</xex>, <xex>containing a stone</xex> or <xex>stones</xex>, <xex>employed on stone</xex>, or, more generally, <xex>of</xex> or <xex>pertaining to stone</xex> or <xex>stones</xex>; as, <xex>stone</xex> fruit, or <xex>stone</xex>-fruit; <xex>stone</xex>-hammer, or <xex>stone</xex> hammer; <xex>stone</xex> falcon, or <xex>stone</xex>-falcon. Compounded with some adjectives it denotes a degree of the quality expressed by the adjective equal to that possessed by a stone; as, <xex>stone</xex>-dead, <xex>stone</xex>-blind, <xex>stone</xex>-cold, <xex>stone</xex>-still, etc.</note><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Atlantic stone</b></col>, <cd>ivory.</cd> <mark>[Obs.]</mark> <ldquo/Citron tables, or <xex>Atlantic stone</xex>.<rdquo/ <au>Milton.</au> -- <col><b>Bowing stone</b></col>. <cd>Same as <er>Cromlech</er>.</cd> <au>Encyc. Brit.</au> -- <col><b>Meteoric stones</b></col>, <cd>stones which fall from the atmosphere, as after the explosion of a meteor.</cd> -- <col><b>Philosopher's stone</b></col>. <cd>See under <er>Philosopher</er>.</cd> -- <col><b>Rocking stone</b></col>. <cd>See <er>Rocking-stone</er>.</cd> -- <col><b>Stone age</b></col>, <cd>a supposed prehistoric age of the world when stone and bone were habitually used as the materials for weapons and tools; -- called also <altname>flint age</altname>. The <cref>bronze age</cref> succeeded to this.</cd> -- <col><b>Stone bass</b></col> <fld>(Zool.)</fld>, <cd>any one of several species of marine food fishes of the genus <gen>Serranus</gen> and allied genera, as <spn>Serranus Couchii</spn>, and <spn>Polyprion cernium</spn> of Europe; -- called also <altname>sea perch</altname>.</cd> -- <col><b>Stone biter</b></col> <fld>(Zool.)</fld>, <cd>the wolf fish.</cd> -- <col><b>Stone boiling</b></col>, <cd>a method of boiling water or milk by dropping hot stones into it, -- in use among savages.</cd> <au>Tylor.</au> -- <col><b>Stone borer</b></col> <fld>(Zool.)</fld>, <cd>any animal that bores stones; especially, one of certain bivalve mollusks which burrow in limestone. See <er>Lithodomus</er>, and <er>Saxicava</er>.</cd> -- <col><b>Stone bramble</b></col> <fld>(Bot.)</fld>, <cd>a European trailing species of bramble (<spn>Rubus saxatilis</spn>).</cd> -- <col><b>Stone-break</b></col>. <ety>[Cf. G. <ets>steinbrech</ets>.]</ety> <fld>(Bot.)</fld> <cd>Any plant of the genus <gen>Saxifraga</gen>; saxifrage.</cd> -- <col><b>Stone bruise</b></col>, <cd>a sore spot on the bottom of the foot, from a bruise by a stone.</cd> -- <col><b>Stone canal</b></col>. <fld>(Zool.)</fld> <cd>Same as <cref>Sand canal</cref>, under <er>Sand</er>.</cd> -- <col><b>Stone cat</b></col> <fld>(Zool.)</fld>, <cd>any one of several species of small fresh-water North American catfishes of the genus <gen>Noturus</gen>. They have sharp pectoral spines with which they inflict painful wounds.</cd> -- <col><b>Stone coal</b></col>, <cd>hard coal; mineral coal; anthracite coal.</cd> -- <col><b>Stone coral</b></col> <fld>(Zool.)</fld>, <cd>any hard calcareous coral.</cd> -- <col><b>Stone crab</b></col>. <fld>(Zool.)</fld> <sd>(a)</sd> <cd>A large crab (<spn>Menippe mercenaria</spn>) found on the southern coast of the United States and much used as food.</cd> <sd>(b)</sd> <cd>A European spider crab (<spn>Lithodes maia</spn>)</cd>. <col><b>Stone crawfish</b></col> <fld>(Zool.)</fld>, <cd>a European crawfish (<spn>Astacus torrentium</spn>), by many writers considered only a variety of the common species (<spn>Astacus fluviatilis</spn>).</cd> -- <col><b>Stone curlew</b></col>. <fld>(Zool.)</fld> <sd>(a)</sd> <cd>A large plover found in Europe (<spn>Edicnemus crepitans</spn>). It frequents stony places. Called also <altname>thick-kneed plover</altname> or <altname>bustard</altname>, and <altname>thick-knee</altname>.</cd> <sd>(b)</sd> <cd>The whimbrel</cd>. <mark>[Prov. Eng.]</mark> <sd>(c)</sd> <cd>The willet.</cd> <mark>[Local, U.S.]</mark> -- <col><b>Stone crush</b></col>. <cd>Same as <cref>Stone bruise</cref>, above.</cd> -- <col><b>Stone eater</b></col>. <fld>(Zool.)</fld> <cd>Same as <cref>Stone borer</cref>, above.</cd> -- <col><b>Stone falcon</b></col> <fld>(Zool.)</fld>, <cd>the merlin.</cd> -- <col><b>Stone fern</b></col> <fld>(Bot.)</fld>, <cd>a European fern (<spn>Asplenium Ceterach</spn>) which grows on rocks and walls.</cd> -- <col><b>Stone fly</b></col> <fld>(Zool.)</fld>, <cd>any one of many species of pseudoneuropterous insects of the genus <gen>Perla</gen> and allied genera; a perlid. They are often used by anglers for bait. The larvae are aquatic.</cd> -- <col><b>Stone fruit</b></col> <fld>(Bot.)</fld>, <cd>any fruit with a stony endocarp; a drupe, as a peach, plum, or cherry.</cd> -- <col><b>Stone grig</b></col> <fld>(Zool.)</fld>, <cd>the mud lamprey, or pride.</cd> -- <col><b>Stone hammer</b></col>, <cd>a hammer formed with a face at one end, and a thick, blunt edge, parallel with the handle, at the other, -- used for breaking stone.</cd> -- <col><b>Stone hawk</b></col> <fld>(Zool.)</fld>, <cd>the merlin; -- so called from its habit of sitting on bare stones.</cd> -- <col><b>Stone jar</b></col>, <cd>a jar made of stoneware.</cd> -- <col><b>Stone lily</b></col> <fld>(Paleon.)</fld>, <cd>a fossil crinoid.</cd> -- <col><b>Stone lugger</b></col>. <fld>(Zool.)</fld> <cd>See <cref>Stone roller</cref>, below.</cd> -- <col><b>Stone marten</b></col> <fld>(Zool.)</fld>, <cd>a European marten (<spn>Mustela foina</spn>) allied to the pine marten, but having a white throat; -- called also <altname>beech marten</altname>.</cd> -- <col><b>Stone mason</b></col>, <cd>a mason who works or builds in stone.</cd> -- <col><b>Stone-mortar</b></col> <fld>(Mil.)</fld>, <cd>a kind of large mortar formerly used in sieges for throwing a mass of small stones short distances.</cd> -- <col><b>Stone oil</b></col>, <cd>rock oil, petroleum.</cd> -- <col><b>Stone parsley</b></col> <fld>(Bot.)</fld>, <cd>an umbelliferous plant (<spn>Seseli Labanotis</spn>). See under <er>Parsley</er>.</cd> -- <col><b>Stone pine</b></col>. <fld>(Bot.)</fld> <cd>A nut pine. See the Note under <er>Pine</er>, and <er>Pi<ntil/on</er>.</cd> -- <col><b>Stone pit</b></col>, <cd>a quarry where stones are dug.</cd> -- <col><b>Stone pitch</b></col>, <cd>hard, inspissated pitch.</cd> -- <col><b>Stone plover</b></col>. <fld>(Zool.)</fld> <sd>(a)</sd> <cd>The European stone curlew</cd>. <sd>(b)</sd> <cd>Any one of several species of Asiatic plovers of the genus <gen>Esacus</gen>; <as>as, the large <ex>stone plover</ex> (<spn>Esacus recurvirostris</spn>)</as>.</cd> <sd>(c)</sd> <cd>The gray or black-bellied plover</cd>. <mark>[Prov. Eng.]</mark> <sd>(d)</sd> <cd>The ringed plover.</cd> <sd>(e)</sd> <cd>The bar-tailed godwit. <mark>[Prov. Eng.]</mark> Also applied to other species of limicoline birds.</cd> -- <col><b>Stone roller</b></col>. <fld>(Zool.)</fld> <sd>(a)</sd> <cd>An American fresh-water fish (<spn>Catostomus nigricans</spn>) of the Sucker family. Its color is yellowish olive, often with dark blotches. Called also <altname>stone lugger</altname>, <altname>stone toter</altname>, <altname>hog sucker</altname>, <altname>hog mullet</altname>.</cd> <sd>(b)</sd> <cd>A common American cyprinoid fish (<spn>Campostoma anomalum</spn>); -- called also <altname>stone lugger</altname>.</cd> -- <mcol><col><b>Stone's cast</b></col>, <it>or</it> <col><b>Stone's throw</b></col></mcol>, <cd>the distance to which a stone may be thrown by the hand; <as>as, they live a <ex>stone's throw</ex> from each other</as>.</cd> -- <col><b>Stone snipe</b></col> <fld>(Zool.)</fld>, <cd>the greater yellowlegs, or tattler.</cd> <mark>[Local, U.S.]</mark> -- <col><b>Stone toter</b></col>. <fld>(Zool.)</fld> <sd>(a)</sd> <cd>See <cref>Stone roller</cref> <sd>(a)</sd>, above</cd>. <sd>(b)</sd> <cd>A cyprinoid fish (<spn>Exoglossum maxillingua</spn>) found in the rivers from <state>Virginia</state> to <stype>New York</stype>.  It has a three-lobed lower lip; -- called also <altname>cutlips</altname>.</cd> -- <col><b>To leave no stone unturned</b></col>, <cd>to do everything that can be done; to use all practicable means to effect an object.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stone</ent><br/
<hw>Stone</hw> <pr>(?)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Stoned</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Stoning</conjf>.]</vmorph> <ety>[From <er>Stone</er>, <pos>n.</pos>: cf. AS. <ets>st<?/nan</ets>, Goth. <ets>stainjan</ets>.]</ety> <sn>1.</sn> <def>To pelt, beat, or kill with stones.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>And they <qex>stoned</qex> Stephen, calling upon God, and saying, Lord Jesus, receive my spirit.</q> <rj><qau>Acts vii. 59.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To make like stone; to harden.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>O perjured woman! thou dost <qex>stone</qex> my heart.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>To free from stones; also, to remove the seeds of; <as>as, to <ex>stone</ex> a field; to <ex>stone</ex> cherries; to <ex>stone</ex> raisins</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>To wall or face with stones; to line or fortify with stones; <as>as, to <ex>stone</ex> a well; to <ex>stone</ex> a cellar</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <def>To rub, scour, or sharpen with a stone.</def><br/
[<source>1913 Webster</source>]</p>

<p><-- get stoned, be stoned? --><br/
[<source>1913 Webster</source>]</p>

<p><-- p. 1419 --></p>

<p><ent>Stonebird</ent><br/
<hw>Stone"bird`</hw> <pr>(?)</pr>, <pos>n.</pos> <def>The yellowlegs; -- called also <altname>stone snipe</altname>. See <er>Tattler</er>, 2.</def> <mark>[Local, U.S.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stone-blind</ent><br/
<hw>Stone"-blind`</hw> <pr>(?)</pr>, <pos>a.</pos> <def>As blind as a stone; completely blind.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stonebow</ent><br/
<hw>Stone"bow`</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A kind of crossbow formerly used for shooting stones.</def>  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stonebrash</ent><br/
<hw>Stone"brash`</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A subsoil made up of small stones or finely-broken rock; brash.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stonebrearer</ent><br/
<hw>Stone"brear`er</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A machine for crushing or hammering stone.</def>  <rj><au>Knight.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stonebuck</ent><br/
<hw>Stone"buck`</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Zool.)</fld> <def>See <er>Steinbock</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stonechat</ent><br/
<hw>Stone"chat`</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[<ets>Stone</ets> + <ets>chat</ets>.]</ety> <ety>[So called from the similarity of its alarm note to the clicking together of two pebbles.]</ety> <fld>(Zool.)</fld> <sd>(a)</sd> <def>A small, active, and very common European singing bird (<spn>Pratincola rubicola</spn>); -- called also <altname>chickstone</altname>, <altname>stonechacker</altname>, <altname>stonechatter</altname>, <altname>stoneclink</altname>, <altname>stonesmith</altname>.</def> <sd>(b)</sd> <def>The wheatear.</def> <sd>(c)</sd> <def>The blue titmouse.</def><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ The name is sometimes applied to various species of <gen>Saxicola</gen>, <gen>Pratincola</gen>, and allied genera; as, the pied <xex>stonechat</xex> of India (<spn>Saxicola picata</spn>).</note><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stone-cold</ent><br/
<hw>Stone"-cold`</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Cold as a stone.</def><br/
[<source>1913 Webster</source>]</p>

<p><q><qex>Stone-cold</qex> without, within burnt with love's flame.</q> <rj><qau>Fairfax.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stonecray</ent><br/
<hw>Stone"cray`</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[<ets>Stone</ets> + F. <ets>craie</ets> chalk, L. <ets>creta</ets>.]</ety> <def>A distemper in hawks.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stonecrop</ent><br/
<hw>Stone"crop`</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[AS. <ets>st<amac/ncropp</ets>.]</ety> <sn>1.</sn> <def>A sort of tree.</def> <mark>[Obs.]</mark>  <rj><au>Mortimer.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Bot.)</fld> <def>Any low succulent plant of the genus <gen>Sedum</gen>, esp. <spn>Sedum acre</spn>, which is common on bare rocks in Europe, and is spreading in parts of America. See <er>Orpine</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><mcol><col><b>Virginian stonecrop</b></col>, <it>or</it> <col><b>Ditch stonecrop</b></col></mcol>, <cd>an American plant (<spn>Penthorum sedoides</spn>).</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stonecutter</ent><br/
<hw>Stone"cut`ter</hw> <pr>(?)</pr>, <pos>n.</pos> <def>One whose occupation is to cut stone; also, a machine for dressing stone.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stonecutting</ent><br/
<hw>Stone"cut`ting</hw> <pr>(?)</pr>, <pos>n.</pos> <def>Hewing or dressing stone.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stone-dead</ent><br/
<hw>Stone"-dead`</hw> <pr>(?)</pr>, <pos>a.</pos> <def>As dead as a stone.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stone-deaf</ent><br/
<hw>Stone"-deaf`</hw> <pr>(?)</pr>, <pos>a.</pos> <def>As deaf as a stone; completely deaf.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stonegall</ent><br/
<hw>Stone"gall`</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Cf. D. <ets>steengal</ets>, G. <ets>steingall</ets>. See <er>Stannel</er>.]</ety> <fld>(Zool.)</fld> <def>See <er>Stannel</er>.</def> <mark>[Prov. Eng.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stonehatch</ent><br/
<hw>Stone"hatch`</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Zool.)</fld> <def>The ring plover, or dotterel.</def> <mark>[Prov. Eng.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stone-hearted</ent><br/
<hw>Stone"-heart`ed</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Hard-hearted; cruel; pitiless; unfeeling.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stonehenge</ent><br/
<hw>Stone"henge</hw> <pr>(?)</pr>, <pos>n.</pos> <def>An assemblage of upright stones with others placed horizontally on their tops, on Salisbury Plain, England, -- generally supposed to be the remains of an ancient Druidical temple.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stone-horse</ent><br/
<hw>Stone"-horse`</hw> <pr>(?)</pr>, <pos>n.</pos> <def>Stallion.</def> <mark>[Obs.]</mark>  <rj><au>Mortimer.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stoner</ent><br/
<hw>Ston"er</hw> <pr>(?)</pr>, <pos>n.</pos> <sn>1.</sn> <def>One who stones; one who makes an assault with stones.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>One who walls with stones.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stoneroot</ent><br/
<hw>Stone"root`</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Bot.)</fld> <def>A North American plant (<spn>Collinsonia Canadensis</spn>) having a very hard root; horse balm. See <cref>Horse balm</cref>, under <er>Horse</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stonerunner</ent><br/
<hw>Stone"run`ner</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Zool.)</fld> <sd>(a)</sd> <def>The ring plover, or the ringed dotterel.</def> <mark>[Prov. Eng.]</mark> <sd>(b)</sd> <def>The dotterel.</def> <mark>[Prov. Eng.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stonesmickle</ent><br/
<hw>Stone"smic`kle</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Zool.)</fld> <def>The stonechat; -- called also <altname>stonesmitch</altname>.</def> <mark>[Prov. Eng.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stone-still</ent><br/
<hw>Stone"-still`</hw> <pr>(?)</pr>, <pos>a.</pos> <def>As still as a stone.</def>  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stoneware</ent><br/
<hw>Stone"ware`</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A species of coarse potter's ware, glazed and baked.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stoneweed</ent><br/
<hw>Stone"weed`</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Bot.)</fld> <def>Any plant of the genus <gen>Lithospermum</gen>, herbs having a fruit composed of four stony nutlets.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stonework</ent><br/
<hw>Stone"work`</hw> <pr>(?)</pr>, <pos>n.</pos> <def>Work or wall consisting of stone; mason's work of stone.</def>  <rj><au>Mortimer.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stonewort</ent><br/
<hw>Stone"wort`</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Bot.)</fld> <def>Any plant of the genus Chara; -- so called because they are often incrusted with carbonate of lime. See <er>Chara</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stonily</ent><br/
<hw>Ston"i*ly</hw> <pr>(?)</pr>, <pos>adv.</pos> <def>In a stony manner.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stoniness</ent><br/
<hw>Ston"i*ness</hw>, <pos>n.</pos> <def>The quality or state of being stony.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stonish</ent><br/
<hw>Ston"ish</hw>, <pos>a.</pos> <def>Stony.</def> <mark>[R.]</mark> <ldquo/Possessed with <xex>stonish</xex> insensibility.<rdquo/  <rj><au>Robynson (More's Utopia).</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stont</ent><br/
<hw>Stont</hw> <pr>(?)</pr>, <mark>obs.</mark> <def><pos>3d pers. sing. present</pos> of <er>Stand</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stony</ent><br/
<hw>Ston"y</hw> <pr>(?)</pr>, <pos>a.</pos> <amorph>[<pos>Compar.</pos> <adjf>Stonier</adjf> <pr>(?)</pr>; <pos>superl.</pos> <adjf>Stoniest</adjf>.]</amorph> <ety>[AS. <ets>st<amac/nig</ets>. See <er>Stone</er>.]</ety> <sn>1.</sn> <def>Of or pertaining to stone, consisting of, or abounding in, stone or stones; resembling stone; hard; <as>as, a <ex>stony</ex> tower; a <ex>stony</ex> cave; <ex>stony</ex> ground; a <ex>stony</ex> crust</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Converting into stone; petrifying; petrific.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The <qex>stony</qex> dart of senseless cold.</q> <rj><qau>Spenser.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>Inflexible; cruel; unrelenting; pitiless; obdurate; perverse; cold; morally hard; appearing as if petrified; <as>as, a <ex>stony</ex> heart; a <ex>stony</ex> gaze</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Stony coral</b></col>. <fld>(Zool.)</fld> <cd>Same as <cref>Stone coral</cref>, under <er>Stone</er>.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stood</ent><br/
<hw>Stood</hw> <pr>(?)</pr>, <def><pos>imp. & p. p.</pos> of <er>Stand</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stook</ent><br/
<hw>Stook</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Scot. <ets>stook</ets>, <ets>stouk</ets>; cf. LG. <ets>stuke</ets> a heap, bundle, G. <ets>stauche</ets> a truss, bundle of flax.]</ety> <fld>(Agric.)</fld> <def>A small collection of sheaves set up in the field; a shock; in England, twelve sheaves.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stook</ent><br/
<hw>Stook</hw>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Stooked</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Stooking</conjf>.]</vmorph> <fld>(Agric.)</fld> <def>To set up, as sheaves of grain, in stooks.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stool</ent><br/
<hw>Stool</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>stolo</ets>. See <er>Stolon</er>.]</ety> <fld>(Hort.)</fld> <def>A plant from which layers are propagated by bending its branches into the soil.</def>  <rj><au>P. Henderson.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stool</ent><br/
<hw>Stool</hw>, <pos>v. i.</pos> <fld>(Agric.)</fld> <def>To ramfy; to tiller, as grain; to shoot out suckers.</def>  <rj><au>R. D. Blackmore.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stool</ent><br/
<hw>Stool</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[AS. <ets>st<omac/l</ets> a seat; akin to OFries. & OS. <ets>st<omac/l</ets>, D. <ets>stoel</ets>, G. <ets>stuhl</ets>, OHG. <ets>stuol</ets>, Icel. <ets>st<omac/ll</ets>, Sw. & Dan. <ets>stol</ets>, Goth. <ets>st<omac/ls</ets>, Lith. <ets>stalas</ets> a table, Russ. <ets>stol'</ets>; from the root of E. <ets>stand</ets>. <root/163. See <er>Stand</er>, and cf. <er>Fauteuil</er>.]</ety> <sn>1.</sn> <def>A single seat with three or four legs and without a back, made in various forms for various uses.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A seat used in evacuating the bowels; hence, an evacuation; a discharge from the bowels.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>A stool pigeon, or decoy bird.</def> <mark>[U. S.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <fld>(Naut.)</fld> <def>A small channel on the side of a vessel, for the dead-eyes of the backstays.</def>  <rj><au>Totten.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <def>A bishop's seat or see; a bishop-stool.</def>  <rj><au>J. P. Peters.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>6.</sn> <def>A bench or form for resting the feet or the knees; a footstool; <as>as, a kneeling <ex>stool</ex></as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>7.</sn> <def>Material, such as oyster shells, spread on the sea bottom for oyster spat to adhere to.</def> <mark>[Local, U.S.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><cs><mcol><col><b>Stool of a window</b></col>, <it>or</it> <col><b>Window stool</b></col></mcol> <fld>(Arch.)</fld>, <cd>the flat piece upon which the window shuts down, and which corresponds to the sill of a door; in the United States, the narrow shelf fitted on the inside against the actual sill upon which the sash descends. This is called a <xex>window seat</xex> when broad and low enough to be used as a seat.</cd> -- <col><b>Stool of repentance</b></col>, <cd>the cuttystool.</cd> <mark>[Scot.]</mark> -- <col><b>Stool pigeon</b></col>, <cd>a pigeon used as a decoy to draw others within a net; hence, a person used as a decoy for others.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stoolball</ent><br/
<hw>Stool"ball`</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A kind of game with balls, formerly common in England, esp. with young women.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Nausicaa<br/
With other virgins did at <qex>stoolball</qex> play.</q> <rj><qau>Chapman.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stoom</ent><br/
<hw>Stoom</hw> <pr>(?)</pr>, <pos>v. t.</pos> <ety>[D. <ets>stommen</ets> to adulterate, to drug (wine). <root/163.  Cf. <er>Stum</er>.]</ety> <def>To stum.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stoop</ent><br/
<hw>Stoop</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[D. <ets>stoep</ets>.]</ety> <fld>(Arch.)</fld> <def>Originally, a covered porch with seats, at a house door; the Dutch <xex>stoep</xex> as introduced by the Dutch into New York. Afterward, an out-of-door flight of stairs of from seven to fourteen steps, with platform and parapets, leading to an entrance door some distance above the street; the French <xex>perron</xex>. Hence, any porch, platform, entrance stairway, or small veranda, at a house door.</def> <mark>[U. S.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stoop</ent><br/
<hw>Stoop</hw>, <pos>n.</pos> <ety>[OE. <ets>stope</ets>, Icel. <ets>staup</ets>; akin to AS. <ets>ste<aacute/p</ets>, D. <ets>stoop</ets>, G. <ets>stauf</ets>, OHG. <ets>stouph</ets>.]</ety> <def>A vessel of liquor; a flagon.</def> <altsp>[Written also <asp>stoup</asp>.]</altsp><br/
[<source>1913 Webster</source>]</p>

<p><q>Fetch me a <qex>stoop</qex> of liquor.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stoop</ent><br/
<hw>Stoop</hw>, <pos>n.</pos> <ety>[Cf. Icel. <ets>staup</ets> a knobby lump.]</ety> <def>A post fixed in the earth.</def> <mark>[Prov. Eng.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stoop</ent><br/
<hw>Stoop</hw>, <pos>v. i.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Stooped</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Stooping</conjf>.]</vmorph> <ety>[OE. <ets>stoupen</ets>; akin to AS. <ets>st<?/pian</ets>, OD. <ets>stuypen</ets>, Icel. <ets>st<umac/pa</ets>, Sw. <ets>stupa</ets> to fall, to tilt. Cf 5th <er>Steep</er>.]</ety> <sn>1.</sn> <def>To bend the upper part of the body downward and forward; to bend or lean forward; to incline forward in standing or walking; to assume habitually a bent position.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To yield; to submit; to bend, as by compulsion; to assume a position of humility or subjection.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Mighty in her ships stood Carthage long, . . . <br/
Yet <qex>stooped</qex> to Rome, less wealthy, but more strong.</q> <rj><qau>Dryden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>These are arts, my prince,<br/
In which your Zama does not <qex>stoop</qex> to Rome.</q> <rj><qau>Addison.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>To descend from rank or dignity; to condescend.</def> <ldquo/She <xex>stoops</xex> to conquer.<rdquo/  <rj><au>Goldsmith.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Where men of great wealth <qex>stoop</qex> to husbandry, it multiplieth riches exceedingly.</q> <rj><qau>Bacon.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>To come down as a hawk does on its prey; to pounce; to souse; to swoop.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The bird of Jove, <qex>stooped</qex> from his aery tour,<br/
Two birds of gayest plume before him drove.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <def>To sink when on the wing; to alight.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>And <qex>stoop</qex> with closing pinions from above.</q> <rj><qau>Dryden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Cowering low<br/
With blandishment, each bird <qex>stooped</qex> on his wing.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><syn><b>Syn.</b> -- To lean; yield; submit; condescend; descend; cower; shrink.</syn><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stoop</ent><br/
<hw>Stoop</hw>, <pos>v. t.</pos> <sn>1.</sn> <def>To bend forward and downward; to bow down; <as>as, to <ex>stoop</ex> the body</as>.</def> <ldquo/Have <xex>stooped</xex> my neck.<rdquo/  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To cause to incline downward; to slant; <as>as, to <ex>stoop</ex> a cask of liquor</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>To cause to submit; to prostrate.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>Many of those whose states so tempt thine ears<br/
Are <qex>stooped</qex> by death; and many left alive.</q> <rj><qau>Chapman.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>To degrade.</def> <mark>[Obs.]</mark>  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stoop</ent><br/
<hw>Stoop</hw>, <pos>n.</pos> <sn>1.</sn> <def>The act of stooping, or bending the body forward; inclination forward; also, an habitual bend of the back and shoulders.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Descent, as from dignity or superiority; condescension; an act or position of humiliation.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Can any loyal subject see<br/
With patience such a <qex>stoop</qex> from sovereignty?</q> <rj><qau>Dryden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>The fall of a bird on its prey; a swoop.</def>  <rj><au>L'Estrange.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stooper</ent><br/
<hw>Stoop"er</hw> <pr>(?)</pr>, <pos>n.</pos> <def>One who stoops.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stooping</ent><br/
<hw>Stoop"ing</hw>, <def><pos>a. & n.</pos> from <er>Stoop</er>.</def> -- <wordforms><wf>Stoop"ing*ly</wf>, <pos>adv.</pos></wordforms><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stoor</ent><br/
<hw>Stoor</hw> <pr>(?)</pr>, <pos>v. i.</pos> <ety>[Cf. D. <ets>storen</ets> to disturb.  Cf. <er>Stir</er>.]</ety> <def>To rise in clouds, as dust.</def> <mark>[Prov. Eng.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stor</ent><br/
<ent>Stoor</ent><br/
<mhw>{ <hw>Stoor</hw> <pr>(?)</pr>, <hw>Stor</hw> <pr>(?)</pr>, }</mhw> <pos>a.</pos> <ety>[AS. <ets>st<omac/r</ets>; akin to LG. <ets>stur</ets>, Icel. <ets>st<omac/rr</ets>.]</ety> <def>Strong; powerful; hardy; bold; audacious.</def> <mark>[Obs. or Scot.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>O stronge lady <qex>stoor</qex>, what doest thou?</q> <rj><qau>Chaucer.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stop</ent><br/
<hw>Stop</hw> <pr>(?)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Stopped</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Stopping</conjf>.]</vmorph> <ety>[OE. <ets>stoppen</ets>, AS. <ets>stoppian</ets> (in comp.); akin to LG. & D. <ets>stoppen</ets>, G. <ets>stopfen</ets>, Icel. <ets>stoppa</ets>, Sw. <ets>stoppa</ets>, Dan. <ets>stoppe</ets>; all probably fr. LL. <ets>stopare</ets>, <ets>stupare</ets>, fr. L. <ets>stuppa</ets> the coarse part of flax, tow, oakum.  Cf. <er>Estop</er>, <er>Stuff</er>, <er>Stupe</er> a fomentation.]</ety> <sn>1.</sn> <def>To close, as an aperture, by filling or by obstructing; <as>as, to <ex>stop</ex> the ears</as>; hence, to stanch, as a wound.</def>  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To obstruct; to render impassable; <as>as, to <ex>stop</ex> a way, road, or passage</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>To arrest the progress of; to hinder; to impede; to shut in; <as>as, to <ex>stop</ex> a traveler; to <ex>stop</ex> the course of a stream, or a flow of blood</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>To hinder from acting or moving; to prevent the effect or efficiency of; to cause to cease; to repress; to restrain; to suppress; to interrupt; to suspend; <as>as, to <ex>stop</ex> the execution of a decree, the progress of vice, the approaches of old age or infirmity</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Whose disposition all the world well knows<br/
Will not be rubbed nor <qex>stopped</qex>.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <fld>(Mus.)</fld> <def>To regulate the sounds of, as musical strings, by pressing them against the finger board with the finger, or by shortening in any way the vibrating part.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>6.</sn> <def>To point, as a composition; to punctuate.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>If his sentences were properly <qex>stopped</qex>.</q> <rj><qau>Landor.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>7.</sn> <fld>(Naut.)</fld> <def>To make fast; to stopper.</def><br/
[<source>1913 Webster</source>]</p>

<p><syn><b>Syn.</b> -- To obstruct; hinder; impede; repress; suppress; restrain; discontinue; delay; interrupt.</syn><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>To stop off</b></col> <fld>(Founding)</fld>, <cd>to fill (a part of a mold) with sand, where a part of the cavity left by the pattern is not wanted for the casting.</cd> -- <col><b>To stop the mouth</b></col>. <cd>See under <er>Mouth</er>.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stop</ent><br/
<hw>Stop</hw> <pr>(?)</pr>, <pos>v. i.</pos> <sn>1.</sn> <def>To cease to go on; to halt, or stand still; to come to a stop.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>He bites his lip, and starts;<br/
<qex>Stops</qex> on a sudden, looks upon the ground;<br/
Then lays his finger on his temple: strait<br/
Springs out into fast gait; then <qex>stops</qex> again.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To cease from any motion, or course of action.</def><br/
[<source>1913 Webster</source>]</p>

<p><q><qex>Stop</qex>, while ye may, suspend your mad career!</q> <rj><qau>Cowper.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>To spend a short time; to reside temporarily; to stay; to tarry; <as>as, to <ex>stop</ex> with a friend</as>.</def> <mark>[Colloq.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>By <qex>stopping</qex> at home till the money was gone.</q> <rj><qau>R. D. Blackmore.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>To stop over</b></col>, <cd>to stop at a station or airport beyond the time of the departure of the train or airplane on which one came, with the purpose of continuing one's journey on a subsequent train or airplane; to break one's journey.  See <er>stopover</er>, <pos>n.</pos></cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stop</ent><br/
<hw>Stop</hw>, <pos>n.</pos> <sn>1.</sn> <def>The act of stopping, or the state of being stopped; hindrance of progress or of action; cessation; repression; interruption; check; obstruction.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>It is doubtful . . . whether it contributed anything to the <qex>stop</qex> of the infection.</q> <rj><qau>De Foe.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Occult qualities put a <qex>stop</qex> to the improvement of natural philosophy.</q> <rj><qau>Sir I. Newton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>It is a great step toward the mastery of our desires to give this <qex>stop</qex> to them.</q> <rj><qau>Locke.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>That which stops, impedes, or obstructs; as obstacle; an impediment; an obstruction.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>A fatal <qex>stop</qex> traversed their headlong course.</q> <rj><qau>Daniel.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>So melancholy a prospect should inspire us with zeal to oppose some <qex>stop</qex> to the rising torrent.</q> <rj><qau>Rogers.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <fld>(Mach.)</fld> <def>A device, or piece, as a pin, block, pawl, etc., for arresting or limiting motion, or for determining the position to which another part shall be brought.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <fld>(Mus.)</fld> <sd>(a)</sd> <def>The closing of an aperture in the air passage, or pressure of the finger upon the string, of an instrument of music, so as to modify the tone; hence, any contrivance by which the sounds of a musical instrument are regulated.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The organ sound a time survives the <qex>stop</qex>.</q> <rj><qau>Daniel.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sd>(b)</sd> <def>In the organ, one of the knobs or handles at each side of the organist, by which he can draw on or shut off any register or row of pipes; the register itself; as, the vox humana <xex>stop</xex>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <fld>(Arch.)</fld> <def>A member, plain or molded, formed of a separate piece and fixed to a jamb, against which a door or window shuts. This takes the place, or answers the purpose, of a rebate. Also, a pin or block to prevent a drawer from sliding too far.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>6.</sn> <def>A point or mark in writing or printing intended to distinguish the sentences, parts of a sentence, or clauses; a mark of punctuation. See <er>Punctuation</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>7.</sn> <fld>(Opt.)</fld> <def>The diaphragm used in optical instruments to cut off the marginal portions of a beam of light passing through lenses.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>8.</sn> <fld>(Zool.)</fld> <def>The depression in the face of a dog between the skull and the nasal bones. It is conspicuous in the bulldog, pug, and some other breeds.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>9.</sn> <fld>(Phonetics)</fld> <def>Some part of the articulating organs, as the lips, or the tongue and palate, closed <sd>(a)</sd> so as to cut off the passage of breath or voice through the mouth and the nose (distinguished as a <xex>lip-stop</xex>, or a <xex>front-stop</xex>, etc., as in <xex>p</xex>, <xex>t</xex>, <xex>d</xex>, etc.), or <sd>(b)</sd> so as to obstruct, but not entirely cut off, the passage, as in <xex>l</xex>, <xex>n</xex>, etc.; also, any of the consonants so formed.</def>  <rj><au>H. Sweet.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Stop bead</b></col> <fld>(Arch.)</fld>, <cd>the molding screwed to the inner side of a window frame, on the face of the pulley stile, completing the groove in which the inner sash is to slide.</cd> -- <col><b>Stop motion</b></col> <fld>(Mach.)</fld>, <cd>an automatic device for arresting the motion of a machine, as when a certain operation is completed, or when an imperfection occurs in its performance or product, or in the material which is supplied to it, etc.</cd> -- <col><b>Stop plank</b></col>, <cd>one of a set of planks employed to form a sort of dam in some hydraulic works.</cd> -- <col><b>Stop valve</b></col>, <cd>a valve that can be closed or opened at will, as by hand, for preventing or regulating flow, as of a liquid in a pipe; -- in distinction from a valve which is operated by the action of the fluid it restrains.</cd> -- <col><b>Stop watch</b></col>, <cd>a watch the hands of which can be stopped in order to tell exactly the time that has passed, as in timing a race. See <cref>Independent seconds watch</cref>, under <er>Independent</er>, <pos>a.</pos></cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><syn><b>Syn.</b> -- Cessation; check; obstruction; obstacle; hindrance; impediment; interruption.</syn><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stopcock</ent><br/
<hw>Stop"cock`</hw> <pr>(?)</pr>, <pos>n.</pos> <sn>1.</sn> <def>A bib, faucet, or short pipe, fitted with a turning stopper or plug for permitting or restraining the flow of a liquid or gas; a cock or valve for checking or regulating the flow of water, gas, etc., through or from a pipe, etc.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>The turning plug, stopper, or spigot of a faucet.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stope</ent><br/
<hw>Stope</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Cf. <er>Step</er>, <pos>n.</pos> & <pos>v. i.</pos>]</ety> <fld>(Mining)</fld> <def>A horizontal working forming one of a series, the working faces of which present the appearance of a flight of steps.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stope</ent><br/
<hw>Stope</hw>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Stoped</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Stoping</conjf>.]</vmorph> <fld>(Mining)</fld> <sd>(a)</sd> <def>To excavate in the form of stopes.</def> <sd>(b)</sd> <def>To fill in with rubbish, as a space from which the ore has been worked out.</def><br/
[<source>1913 Webster</source>]</p>

<p><-- p. 1420 --></p>

<p><ent>Stopen</ent><br/
<ent>Stope</ent><br/
<mhw>{ <hw>Stope</hw> <pr>(?)</pr>, <hw>Sto"pen</hw> <pr>(?)</pr>, }</mhw> <pos>p. p.</pos> <mord>of <er>Step</er></mord>. <def>Stepped; gone; advanced.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>A poor widow, somedeal <qex>stope</qex> in age.</q> <rj><qau>Chaucer.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stop-gap</ent><br/
<hw>Stop"-gap`</hw> <pr>(?)</pr>, <pos>n.</pos> <def>That which closes or fills up an opening or gap; hence, a temporary expedient.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Moral prejudices are the <qex>stop-gaps</qex> of virtue.</q> <rj><qau>Hare.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stoping</ent><br/
<hw>Stop"ing</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Mining)</fld> <def>The act of excavating in the form of stopes.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stopless</ent><br/
<hw>Stop"less</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Not to be stopped.</def>  <rj><au>Davenant.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stop order</ent><br/
<hw>Stop order</hw>. <fld>(Finance)</fld> <def>An order in a financial market that aims to limit losses by fixing a figure at which purchases shall be sold or sales bought in, as where stock is bought at 100 and the broker is directed to sell if the market price drops to 98.</def>  <note>For a simple <ex>stop order</ex>, when the specified price has been reached by a traded security, the order becomes a market order to buy or sell at the market price.  Thus in the given example, the stock bought at 100 might in fact be sold at 97 if the market is dropping rapidly.  A <contr>stop-limit order</contr> is allowed by some exchanges and traders, in which case, when the specified price has been reached by a traded security, the order becomes a <contr>limit order</contr> to buy or sell, with the possibility that the ordered transaction may not take place.</note><br/
[<source>Webster 1913 Suppl.</source> <source>+PJC</source>]</p>

<p><ent>Stop-over</ent><br/
<ent>Stopover</ent><br/
<mhw><hw>Stop"o`ver</hw>, <hw>Stop"-o`ver</hw></mhw> <pr>(?)</pr>, <pos>a.</pos> <def>Permitting one to stop over; <as>as, a <ex>stop-over</ex> check or ticket</as>. See <cref>To stop over</cref>, under <er>Stop</er>, <pos>v. i.</pos></def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stop-over</ent><br/
<ent>Stopover</ent><br/
<mhw><hw>Stop"o`ver</hw>, <hw>Stop"-o`ver</hw></mhw>, <pos>n.</pos> <sn>1.</sn> <def>the act or privilege of stopping over; stopping at a station or airport beyond the time of the departure of the train or airplane on which one came, with the purpose of continuing one's journey on a subsequent train or airplane; the temporary interruption of one's journey.</def><br/
[<source>Webster 1913 Suppl.</source> <source>+PJC</source>]</p>

<p><sn>2.</sn> <def>a brief visit.</def><br/
[<source>PJC</source>]</p>

<p><ent>Stoppage</ent><br/
<hw>Stop"page</hw> <pr>(?)</pr>, <pos>n.</pos> <def>The act of stopping, or arresting progress, motion, or action; also, the state of being stopped; <as>as, the <ex>stoppage</ex> of the circulation of the blood; the <ex>stoppage</ex> of commerce.</as></def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stopped</ent><br/
<hw>Stopped</hw> <pr>(?)</pr>, <pos>a.</pos> <fld>(Phonetics)</fld> <def>Made by complete closure of the mouth organs; shut; -- said of certain consonants (<xex>p</xex>, <xex>b</xex>, <xex>t</xex>, <xex>d</xex>, etc.).</def>  <rj><au>H. Sweet.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><-- glottal stop? --><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stopper</ent><br/
<hw>Stop"per</hw> <pr>(?)</pr>, <pos>n.</pos> <sn>1.</sn> <def>One who stops, closes, shuts, or hinders; that which stops or obstructs; that which closes or fills a vent or hole in a vessel.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Naut.)</fld> <def>A short piece of rope having a knot at one or both ends, with a lanyard under the knot, -- used to secure something.</def>  <rj><au>Totten.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <fld>(Bot.)</fld> <def>A name to several trees of the genus Eugenia, found in Florida and the West Indies; <as>as, the red <ex>stopper</ex></as>. See <er>Eugenia</er>.</def>  <rj><au>C. S. Sargent.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Ring stopper</b></col> <fld>(Naut.)</fld>, <cd>a short rope or chain passing through the anchor ring, to secure the anchor to the cathead.</cd> -- <col><b>Stopper bolt</b></col> <fld>(Naut.)</fld>, <cd>a large ringbolt in a ship's deck, to which the deck stoppers are hooked.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stopper</ent><br/
<hw>Stop"per</hw>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Stoppered</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Stoppering</conjf>.]</vmorph> <def>To close or secure with a stopper.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stopping</ent><br/
<hw>Stop"ping</hw> <pr>(?)</pr>, <pos>n.</pos> <sn>1.</sn> <def>Material for filling a cavity.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Mining)</fld> <def>A partition or door to direct or prevent a current of air.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <fld>(Far.)</fld> <def>A pad or poultice of dung or other material applied to a horse's hoof to keep it moist.</def>  <rj><au>Youatt.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stopping-out</ent><br/
<hw>Stop"ping-out`</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A method adopted in etching, to keep the acid from those parts which are already sufficiently corroded, by applying varnish or other covering matter with a brush, but allowing the acid to act on the other parts.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stopple</ent><br/
<hw>Stop"ple</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Cf. G. <ets>st<oum/pfel</ets>, <ets>st<oum/psel</ets>. See <er>Stop</er>, <pos>n.</pos> & <pos>v. t.</pos>]</ety> <def>That which stops or closes the mouth of a vessel; a stopper; <as>as, a glass <ex>stopple</ex>; a cork <ex>stopple</ex>.</as></def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stopple</ent><br/
<hw>Stop"ple</hw>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Stoppled</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Stoppling</conjf>.]</vmorph> <def>To close the mouth of anything with a stopple, or as with a stopple.</def>  <rj><au>Cowper.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stopship</ent><br/
<hw>Stop"ship`</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Zool.)</fld> <def>A remora. It was fabled to stop ships by attaching itself to them.</def>  <rj><au>Sylvester.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stor</ent><br/
<hw>Stor</hw> <pr>(?)</pr>, <pos>a.</pos> <def>See <er>Stoor</er>.</def> <mark>[Obs.]</mark>  <rj><au>Chaucer.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Storage</ent><br/
<hw>Stor"age</hw> <pr>(?)</pr>, <pos>n.</pos> <sn>1.</sn> <def>The act of depositing in a store or warehouse for safe keeping; also, the safe keeping of goods in a warehouse.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Space for the safe keeping of goods.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>The price changed for keeping goods in a store.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Storage battery</b></col>. <fld>(Physics)</fld> <cd>See the Note under <er>Battery</er>.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Storax</ent><br/
<hw>Sto"rax</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>storax</ets>, <ets>styrax</ets>, Gr. <?/.  Cf. <er>Styrax</er>.]</ety> <def>Any one of a number of similar complex resins obtained from the bark of several trees and shrubs of the Styrax family. The most common of these is <stype>liquid storax</stype>, a brown or gray semifluid substance of an agreeable aromatic odor and balsamic taste, sometimes used in perfumery, and in medicine as an expectorant.</def><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ A yellow aromatic honeylike substance, resembling, and often confounded with, storax, is obtained from the American sweet gum tree (<spn>Liquidambar styraciflua</spn>), and is much used as a chewing gum, called <xex>sweet gum</xex>, and <xex>liquid storax</xex>.  Cf. <er>Liquidambar</er>.</note><br/
[<source>1913 Webster</source>]</p>

<p><ent>Store</ent><br/
<hw>Store</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[OE. <ets>stor</ets>, <ets>stoor</ets>, OF. <ets>estor</ets>, provisions, supplies, fr. <ets>estorer</ets> to store. See <er>Store</er>, <pos>v. t.</pos>]</ety> <sn>1.</sn> <def>That which is accumulated, or massed together; a source from which supplies may be drawn; hence, an abundance; a great quantity, or a great number.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The ships are fraught with <qex>store</qex> of victuals.</q> <rj><qau>Bacon.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>With <qex>store</qex> of ladies, whose bright eyes<br/
Rain influence, and give the prize.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A place of deposit for goods, esp. for large quantities; a storehouse; a warehouse; a magazine.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>Any place where goods are sold, whether by wholesale or retail; a shop.</def> <mark>[U.S. & British Colonies]</mark><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <pluf>pl.</pluf> <def>Articles, especially of food, accumulated for some specific object; supplies, as of provisions, arms, ammunition, and the like; <as>as, the <ex>stores</ex> of an army, of a ship, of a family</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>His swine, his horse, his <qex>stoor</qex>, and his poultry.</q> <rj><qau>Chaucer.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>In store</b></col>, <cd>in a state of accumulation; in keeping; hence, in a state of readiness.</cd> <ldquo/I have better news in <xex>store</xex> for thee.<rdquo/ <au>Shak.</au> -- <col><b>Store clothes</b></col>, clothing purchased at a shop or store; -- in distinction from that which is <xex>home-made</xex>. <mark>[Colloq. U.S.]</mark> -- <col><b>Store pay</b></col>, payment for goods or work in articles from a shop or store, instead of money. <mark>[U.S.]</mark> -- <col><b>To set store by</b></col>, <cd>to value greatly; to have a high appreciation of.</cd> -- <col><b>To tell no store of</b></col>, <cd>to make no account of; to consider of no importance.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><syn><b>Syn.</b> -- Fund; supply; abundance; plenty; accumulation; provision.</syn>  -- <usage><er>Store</er>, <er>Shop</er>. The English call the place where goods are sold (however large or splendid it may be) a <xex>shop</xex>, and confine the word <xex>store</xex> to its original meaning; viz., a warehouse, or place where goods are <xex>stored</xex>. In America the word <xex>store</xex> is applied to all places, except the smallest, where goods are sold. In some British colonies the word <xex>store</xex> is used as in the United States.</usage>
<-- also syn. = stock --><br/
[<source>1913 Webster</source>]</p>

<p><q>In his needy <qex>shop</qex> a tortoise hung,<br/
An alligator stuffed, and other skins<br/
Of ill-shaped fishes; and about his shelves<br/
A beggarly account of empty boxes.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Sulphurous and nitrous foam, . . . <br/
Concocted and adjusted, they reduced<br/
To blackest grain, and into <qex>store</qex> conveyed.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Store</ent><br/
<hw>Store</hw>, <pos>a.</pos> <def>Accumulated; hoarded.</def>  <rj><au>Bacon.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Store</ent><br/
<hw>Store</hw> <pr>(?)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Stored</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Storing</conjf>.]</vmorph> <ety>[OE. <ets>storen</ets>, OF. <ets>estorer</ets> to construct, restore, store, LL. <ets>staurare</ets>, for L. <ets>instaurare</ets> to renew, restore; <ets>in + staurare</ets> (in comp.) Cf. <er>Instore</er>, <er>Instaurate</er>, <er>Restore</er>, <er>Story</er> a floor.]</ety> <sn>1.</sn> <def>To collect as a reserved supply; to accumulate; to lay away.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Dora <qex>stored</qex> what little she could save.</q> <rj><qau>Tennyson.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To furnish; to supply; to replenish; esp., to stock or furnish against a future time.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Her mind with thousand virtues <qex>stored</qex>.</q> <rj><qau>Prior.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Wise Plato said the world with men was <qex>stored</qex>.</q> <rj><qau>Denham.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Having <qex>stored</qex> a pond of four acres with carps, tench, and other fish.</q> <rj><qau>Sir M. Hale.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>To deposit in a store, warehouse, or other building, for preservation; to warehouse; <as>as, to <ex>store</ex> goods</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stored</ent><br/
<hw>Stored</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Collected or accumulated as a reserve supply; <as>as, <ex>stored</ex> electricity</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>It is charged with <qex>stored</qex> virtue.</q> <rj><qau>Bagehot.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Storehouse</ent><br/
<hw>Store"house`</hw> <pr>(?)</pr>, <pos>n.</pos> <sn>1.</sn> <def>A building for keeping goods of any kind, especially provisions; a magazine; a repository; a warehouse.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Joseph opened all the <qex>storehouses</qex>, and sold unto Egyptians.</q> <rj><qau>Gen. xli. 56.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>The Scripture of God is a <qex>storehouse</qex> abounding with estimable treasures of wisdom and knowledge.</q> <rj><qau>Hooker.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A mass or quality laid up.</def> <mark>[Obs.]</mark>  <rj><au>Spenser.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Storekeeper</ent><br/
<hw>Store"keep`er</hw> <pr>(?)</pr>, <pos>n.</pos> <sn>1.</sn> <def>A man in charge of stores or goods of any kind; <as>as, a naval <ex>storekeeper</ex></as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>One who keeps a <ldquo/store;<rdquo/ a shopkeeper. See 1st <er>Store</er>, 3.</def> <mark>[U. S.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Storer</ent><br/
<hw>Stor"er</hw> <pr>(?)</pr>, <pos>n.</pos> <def>One who lays up or forms a store.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Storeroom</ent><br/
<hw>Store"room`</hw> <pr>(?)</pr>, <pos>n.</pos> <def>Room in a storehouse or repository; a room in which articles are stored.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Storeship</ent><br/
<hw>Store"ship`</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A vessel used to carry naval stores for a fleet, garrison, or the like.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Storey</ent><br/
<hw>Sto"rey</hw> <pr>(?)</pr>, <pos>n.</pos> <def>See <er>Story</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Storge</ent><br/
||<hw>Stor"ge</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[NL., from Gr. <?/, <?/, to love.]</ety> <def>Parental affection; the instinctive affection which animals have for their young.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Storial</ent><br/
<hw>Sto"ri*al</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Historical.</def> <mark>[Obs.]</mark>  <rj><au>Chaucer.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Storied</ent><br/
<hw>Sto"ried</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[From <er>Story</er>.]</ety> <sn>1.</sn> <def>Told in a story.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Having a history; interesting from the stories which pertain to it; venerable from the associations of the past.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Some greedy minion, or imperious wife,<br/
The trophied arches, <qex>storied</qex> halls, invade.</q> <rj><qau>Pope.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Can <qex>storied</qex> urn, or animated bust,<br/
Back to its mansion call the fleeting breath?</q> <rj><qau>Gray.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>Having (such or so many) stories; -- chiefly in composition; <as>as, a two-<ex>storied</ex> house</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Storier</ent><br/
<hw>Sto"ri*er</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A relater of stories; an historian.</def> <mark>[Obs.]</mark>  <rj><au>Bp. Peacock.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Storify</ent><br/
<hw>Sto"ri*fy</hw> <pr>(?)</pr>, <pos>v. t.</pos> <ety>[<ets>Story</ets> + <ets>-fy</ets>.]</ety> <def>To form or tell stories of; to narrate or describe in a story.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stork</ent><br/
<hw>Stork</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[AS. <ets>storc</ets>; akin to G. <ets>storch</ets>, OHG. <ets>storah</ets>, Icel. <ets>storkr</ets>, Dan. & Sw. <ets>stork</ets>, and perhaps to Gr. <?/ a vulture.]</ety> <fld>(Zool.)</fld> <def>Any one of several species of large wading birds of the family <fam>Ciconidae</fam>, having long legs and a long, pointed bill.  They are found both in the Old World and in America, and belong to <gen>Ciconia</gen> and several allied genera.  The European white stork (<spn>Ciconia alba</spn>) is the best known.  It commonly makes its nests on the top of a building, a chimney, a church spire, or a pillar.  The black stork (<spn>Ciconia nigra</spn>) is native of Asia, Africa, and Europe.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Black-necked stork</b></col>, <cd>the East Indian jabiru.</cd> -- <col><b>Hair-crested stork</b></col>, <cd>the smaller adjutant of India (<spn>Leptoptilos Javanica</spn>).</cd> -- <col><b>Giant stork</b></col>, <cd>the adjutant.</cd> -- <col><b>Marabou stork</b></col>. <cd>See <er>Marabou</er>. -- Saddle-billed stork, the African jabiru. See <er>Jabiru</er>.</cd> -- <col><b>Stork's bill</b></col> <fld>(Bot.)</fld>, <cd>any plant of the genus <gen>Pelargonium</gen>; -- so called in allusion to the beaklike prolongation of the axis of the receptacle of its flower. See <er>Pelargonium</er>.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stork-billed</ent><br/
<hw>Stork"-billed`</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Having a bill like that of the stork.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Storm</ent><br/
<hw>Storm</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[AS. <ets>storm</ets>; akin to D. <ets>storm</ets>, G. <ets>sturm</ets>, Icel. <ets>stormr</ets>; and perhaps to Gr. <?/ assault, onset, Skr. <ets>s<?/</ets> to flow, to hasten, or perhaps to L. <ets>sternere</ets> to strew, prostrate (cf. <er>Stratum</er>). <root/166.]</ety> <sn>1.</sn> <def>A violent disturbance of the atmosphere, attended by wind, rain, snow, hail, or thunder and lightning; hence, often, a heavy fall of rain, snow, or hail, whether accompanied with wind or not.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>We hear this fearful tempest sing,<br/
Yet seek no shelter to avoid the <qex>storm</qex>.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A violent agitation of human society; a civil, political, or domestic commotion; sedition, insurrection, or war; violent outbreak; clamor; tumult.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>I will stir up in England some black <qex>storm</qex>.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Her sister<br/
Began to scold and raise up such a <qex>storm</qex>.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>A heavy shower or fall, any adverse outburst of tumultuous force; violence.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>A brave man struggling in the <qex>storms</qex> of fate.</q> <rj><qau>Pope.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <fld>(Mil.)</fld> <def>A violent assault on a fortified place; a furious attempt of troops to enter and take a fortified place by scaling the walls, forcing the gates, or the like.</def><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ <xex>Storm</xex> is often used in the formation of self-explained compounds; as, <xex>storm</xex>-presaging, <xex>storm</xex>proof, <xex>storm</xex>-tossed, and the like.</note><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Anticyclonic storm</b></col> <fld>(Meteor.)</fld>, <cd>a storm characterized by a central area of high atmospheric pressure, and having a system of winds blowing spirally outward in a direction contrary to that cyclonic storms. It is attended by low temperature, dry air, infrequent precipitation, and often by clear sky. Called also <altname>high-area storm</altname>, <altname>anticyclone</altname>. When attended by high winds, snow, and freezing temperatures such storms have various local names, as <stype>blizzard</stype>, <stype>wet norther</stype>, <stype>purga</stype>, <stype>buran</stype>, etc.</cd> -- <col><b>Cyclonic storm</b></col>. <fld>(Meteor.)</fld> <cd>A cyclone, or low-area storm. See <er>Cyclone</er>, above.</cd> -- <col><b>Magnetic storm</b></col>. <cd>See under <er>Magnetic</er>.</cd> -- <col><b>Storm-and-stress period</b></col> <ety>[a translation of G. <ets>sturm und drang periode</ets>]</ety>, <cd>a designation given to the literary agitation and revolutionary development in Germany under the lead of Goethe and Schiller in the latter part of the 18th century.</cd> -- <col><b>Storm center</b></col> <fld>(Meteorol.)</fld>, <cd>the center of the area covered by a storm, especially by a storm of large extent.</cd> -- <col><b>Storm door</b></col> <fld>(Arch.)</fld>, <cd>an extra outside door to prevent the entrance of wind, cold, rain, etc.; -- usually removed in summer.</cd><-- or replaced with a screen door; storm and screen door. --> -- <col><b>Storm path</b></col> <fld>(Meteorol.)</fld>, <cd>the course over which a storm, or storm center, travels.</cd> -- <col><b>Storm petrel</b></col>. <fld>(Zool.)</fld> <cd>See <cref>Stormy petrel</cref>, under <er>Petrel</er>.</cd> -- <col><b>Storm sail</b></col> <fld>(Naut.)</fld>, <cd>any one of a number of strong, heavy sails that are bent and set in stormy weather.</cd> -- <col><b>Storm scud</b></col>. <cd>See the Note under <er>Cloud</er>.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><syn><b>Syn.</b> -- Tempest; violence; agitation; calamity.</syn>  -- <usage><er>Storm</er>, <er>Tempest</er>. <xex>Storm</xex> is violent agitation, a commotion of the elements by wind, etc., but not necessarily implying the fall of anything from the clouds. Hence, to call a mere fall or rain without wind a <xex>storm</xex> is a departure from the true sense of the word. A <xex>tempest</xex> is a sudden and violent storm, such as those common on the coast of Italy, where the term originated, and is usually attended by a heavy rain, with lightning and thunder.</usage><br/
[<source>1913 Webster</source>]</p>

<p><q><qex>Storms</qex> beat, and rolls the main;<br/
O! beat those <qex>storms</qex>, and roll the seas, in vain.</q> <rj><qau>Pope.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>What at first was called a gust, the same<br/
Hath now a <qex>storm's</qex>, anon a <qex>tempest's</qex> name.</q> <rj><qau>Donne.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Storm</ent><br/
<hw>Storm</hw> <pr>(?)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Stormed</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Storming</conjf>.]</vmorph> <fld>(Mil.)</fld> <def>To assault; to attack, and attempt to take, by scaling walls, forcing gates, breaches, or the like; <as>as, to <ex>storm</ex> a fortified town</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Storm</ent><br/
<hw>Storm</hw>, <pos>v. i.</pos> <ety>[Cf. AS. <ets>styrman</ets>.]</ety> <sn>1.</sn> <def>To raise a tempest.</def>  <rj><au>Spenser.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To blow with violence; also, to rain, hail, snow, or the like, usually in a violent manner, or with high wind; -- used impersonally; <as>as, it <ex>storms</ex></as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>To rage; to be in a violent passion; to fume.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The master <qex>storms</qex>, the lady scolds.</q> <rj><qau>Swift.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Storm-beat</ent><br/
<hw>Storm"-beat`</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Beaten, injured, or impaired by storms.</def>  <rj><au>Spenser.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stormcock</ent><br/
<hw>Storm"cock`</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Zool.)</fld> <sd>(a)</sd> <def>The missel thrush.</def> <sd>(b)</sd> <def>The fieldfare.</def> <sd>(c)</sd> <def>The green woodpecker.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stormfinch</ent><br/
<hw>Storm"finch`</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Zool.)</fld> <def>The storm petrel.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stormful</ent><br/
<hw>Storm"ful</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Abounding with storms.</def> <ldquo/The <xex>stormful</xex> east.<rdquo/ <au>Carlyle.</au> -- <wordforms><wf>Storm"ful*ness</wf>, <pos>n.</pos></wordforms><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stormglass</ent><br/
<hw>Storm"glass`</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A glass vessel, usually cylindrical, filled with a solution which is sensitive to atmospheric changes, indicating by a clouded appearance, rain, snow, etc., and by clearness, fair weather.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stormily</ent><br/
<hw>Storm"i*ly</hw> <pr>(?)</pr>, <pos>adv.</pos> <def>In a stormy manner.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Storminess</ent><br/
<hw>Storm"i*ness</hw>, <pos>n.</pos> <def>The state of being stormy; tempestuousness; biosteruousness; impetuousness.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Storming</ent><br/
<hw>Storm"ing</hw>, <def><pos>a. & n.</pos> from <er>Storm</er>, <pos>v.</pos></def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Storming party</b></col> <fld>(Mil.)</fld>, <cd>a party assigned to the duty of making the first assault in storming a fortress.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stormless</ent><br/
<hw>Storm"less</hw>, <pos>a.</pos> <def>Without storms.</def>  <rj><au>Tennyson.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stormwind</ent><br/
<hw>Storm"wind`</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A heavy wind; a wind that brings a storm; the blast of a storm.</def>  <rj><au>Longfellow.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stormy</ent><br/
<hw>Storm"y</hw> <pr>(?)</pr>, <pos>a.</pos> <amorph>[<pos>Compar.</pos> <adjf>Stormier</adjf> <pr>(?)</pr>; <pos>superl.</pos> <adjf>Stormiest</adjf>.]</amorph> <sn>1.</sn> <def>Characterized by, or proceeding from, a storm; subject to storms; agitated with furious winds; biosterous; tempestous; <as>as, a <ex>stormy</ex> season; a <ex>stormy</ex> day or week.</as></def> <ldquo/Beyond the <xex>stormy</xex> Hebrides.<rdquo/  <rj><au>Milton.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Proceeding from violent agitation or fury; <as>as, a <ex>stormy</ex> sound; <ex>stormy</ex> shocks.</as></def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>Violent; passionate; rough; <as>as, <ex>stormy</ex> passions</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q><qex>Stormy</qex> chiefs of a desert but extensive domain.</q> <rj><qau>Sir W. Scott.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Storthing</ent><br/
||<hw>Stor"thing</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Norw. <ets>storting</ets>; <ets>stor</ets> great + <ets>ting</ets> court, court of justice; cf. Dan. <ets>ting</ets>, <ets>thing</ets>.]</ety> <def>The Parliament of Norway, chosen by indirect election once in three years, but holding annual sessions.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Storven</ent><br/
<hw>Stor"ven</hw> <pr>(?)</pr>, <mark>obs.</mark> <def><pos>p. p.</pos> of <er>Starve</er>.</def>  <rj><au>Chaucer.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Story</ent><br/
<hw>Sto"ry</hw> <pr>(?)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Stories</plw> <pr>(#)</pr>.</plu> <ety>[OF. <ets>estor<eacute/</ets>, <ets>estor<eacute/e</ets>, built, erected, p. p. of <ets>estorer</ets> to build, restore, to store. See <er>Store</er>, <pos>v. t.</pos>]</ety> <def>A set of rooms on the same floor or level; a floor, or the space between two floors. Also, a horizontal division of a building's exterior considered architecturally, which need not correspond exactly with the stories within.</def> <altsp>[Written also <asp>storey</asp>.]</altsp><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ A <xex>story</xex> comprehends the distance from one floor to another; as, a <xex>story</xex> of nine or ten feet elevation. The spaces between floors are numbered in order, from below upward; as, the lower, second, or third <xex>story</xex>; a house of one <xex>story</xex>, of two <xex>stories</xex>, of five <xex>stories</xex>.</note><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Story post</b></col> <fld>(Arch.)</fld>, <cd>a vertical post used to support a floor or superincumbent wall.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Story</ent><br/
<hw>Sto"ry</hw>, <pos>n.</pos> <ety>[OE. <ets>storie</ets>, OF. <ets>estoire</ets>, F. <ets>histoire</ets>, fr. L. <ets>historia</ets>. See <er>History</er>.]</ety> <sn>1.</sn> <def>A narration or recital of that which has occurred; a description of past events; a history; a statement; a record.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>One malcontent who did indeed get a name in <qex>story</qex>.</q> <rj><qau>Barrow.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Venice, with its unique city and its Impressive <qex>story</qex>.</q> <rj><qau>Ed. Rev.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>The four great monarchies make the subject of ancient <qex>story</qex>.</q> <rj><qau>Sir W. Temple.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>The relation of an incident or minor event; a short narrative; a tale; especially, a fictitious narrative less elaborate than a novel; a short romance.</def>  <rj><au>Addison.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>A euphemism or child's word for <ldquo/a lie;<rdquo/ a fib; <as>as, to tell a <ex>story</ex></as>.</def> <mark>[Colloq.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Story</ent><br/
<hw>Sto"ry</hw>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Storied</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Storying</conjf>.]</vmorph> <def>To tell in historical relation; to make the subject of a story; to narrate or describe in story.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>How worthy he is I will leave to appear hereafter, rather than <qex>story</qex> him in his own hearing.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>It is <qex>storied</qex> of the brazen colossus in Rhodes, that it was seventy cubits high.</q> <rj><qau>Bp. Wilkins.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Storybook</ent><br/
<hw>Sto"ry*book`</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A book containing stories, or short narratives, either true or false.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Story-teller</ent><br/
<hw>Sto"ry-tell`er</hw> <pr>(?)</pr>, <pos>n.</pos> <sn>1.</sn> <def>One who tells stories; a narrator of anecdotes,incidents, or fictitious tales; <as>as, an amusing <ex>story-teller</ex></as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>An historian; -- in contempt.</def>  <rj><au>Swift.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>A euphemism or child's word for <ldquo/a liar.<rdquo/</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Story-telling</ent><br/
<hw>Sto"ry-tell`ing</hw>, <pos>a.</pos> <def>Being accustomed to tell stories.</def> -- <def2><pos>n.</pos> <def>The act or practice of telling stories.</def></def2><br/
[<source>1913 Webster</source>]</p>

<p><ent>Story-writer</ent><br/
<hw>Sto"ry-writ`er</hw> <pr>(?)</pr>, <pos>n.</pos> <sn>1.</sn> <def>One who writes short stories, as for magazines.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>An historian; a chronicler.</def> <mark>[Obs.]</mark> <ldquo/Rathums, the <xex>story-writer</xex>.<rdquo/  <rj><au>1 Esdr. ii. 17.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stot</ent><br/
<hw>Stot</hw> <pr>(st<ocr/t)</pr>, <pos>n.</pos> <ety>[AS. <ets>stotte</ets> a hack, jade, or worthless horse; cf. Sw. <ets>stut</ets> a bull, Dan. <ets>stud</ets> an ox.  Cf. <er>Stoat</er>.]</ety> <sn>1.</sn> <def>A horse.</def> <mark>[Obs.]</mark>  <rj><au>Chaucer. Thorold Rogers.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A young bull or ox, especially one three years old.</def> <mark>[Prov. Eng. & Scot.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><-- p. 1421 --></p>

<p><ent>Stote</ent><br/
<hw>Stote</hw> <pr>(st<omac/t)</pr>, <pos>n.</pos> <fld>(Zool.)</fld> <def>See <er>Stoat</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stound</ent><br/
<hw>Stound</hw> <pr>(stound)</pr>, <pos>v. i.</pos> <ety>[Cf. <er>Astound</er>, <er>Stun</er>.]</ety> <def>To be in pain or sorrow.</def> <mark>[Obs. or Prov. Eng. & Scot.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stound</ent><br/
<hw>Stound</hw> <pr>(stound)</pr>, <pos>a.</pos> <ety>[See <er>Stound</er>, <pos>v. i.</pos>]</ety> <def>Stunned.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stound</ent><br/
<hw>Stound</hw>, <pos>n.</pos> <sn>1.</sn> <def>A sudden, severe pain or grief; peril; alarm.</def> <mark>[Obs.]</mark>  <rj><au>Spenser.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Astonishment; amazement.</def> <mark>[Obs.]</mark>  <rj><au>Spenser. Gay.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stound</ent><br/
<hw>Stound</hw>, <pos>n.</pos> <ety>[AS. <ets>stund</ets>; akin to D. <ets>stond</ets>, G. <ets>stunde</ets>, Icel. <ets>stund</ets>.]</ety> <sn>1.</sn> <def>Hour; time; season.</def> <mark>[Obs.]</mark>  <rj><au>Chaucer.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A brief space of time; a moment.</def> <mark>[Obs.]</mark>  <rj><au>Chaucer.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>In a stound</b></col>, <cd>suddenly.</cd> <mark>[Obs.]</mark> <au>Chaucer.</au></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stound</ent><br/
<hw>Stound</hw>, <pos>n.</pos> <ety>[Cf. <er>Stand</er>.]</ety> <def>A vessel for holding small beer.</def> <mark>[Prov. Eng.]</mark>  <rj><au>Halliwell.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stoup</ent><br/
<hw>Stoup</hw> <pr>(st<oomac/p)</pr>, <pos>n.</pos> <ety>[See <er>Stoop</er> a vessel.]</ety> <sn>1.</sn> <def>A flagon; a vessel or measure for liquids.</def> <mark>[Scot.]</mark> <rj><au>Jamieson.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Eccl.)</fld> <def>A basin at the entrance of Roman Catholic churches for containing the holy water with which those who enter, dipping their fingers in it, cross themselves; -- called also <altname>holy-water stoup</altname> or <altname>holy-water font</altname>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stour</ent><br/
<hw>Stour</hw> <pr>(stour <it>or</it> st<oomac/r)</pr>, <pos>n.</pos> <ety>[OF. <ets>estour</ets>, <ets>estor</ets>, tumult, combat, of Teutonic origin. See <er>Storm</er>.]</ety> <def>A battle or tumult; encounter; combat; disturbance; passion.</def> <mark>[Obs.]</mark> <au>Fairfax.</au> <ldquo/That woeful <xex>stowre</xex>.<rdquo/  <rj><au>Spenser.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>She that helmed was in starke <qex>stours</qex> [fierce conflicts].</q> <rj><qau>Chaucer.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stour</ent><br/
<hw>Stour</hw>, <pos>a.</pos> <ety>[See <er>Stoor</er>, <pos>a.</pos>]</ety> <def>Tall; strong; stern.</def> <mark>[Obs. or Prov. Eng. & Scot.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stout</ent><br/
<hw>Stout</hw> <pr>(stout)</pr>, <pos>a.</pos> <amorph>[<pos>Compar.</pos> <adjf>Stouter</adjf> <pr>(stout"<etil/r)</pr>; <pos>superl.</pos> <adjf>Stoutest</adjf>.]</amorph> <ety>[D. <ets>stout</ets> bold (or OF. <ets>estout</ets> bold, proud, of Teutonic origin); akin to AS. <ets>stolt</ets>, G. <ets>stolz</ets>, and perh. to E. <ets>stilt</ets>.]</ety> <sn>1.</sn> <def>Strong; lusty; vigorous; robust; sinewy; muscular; hence, firm; resolute; dauntless.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>With hearts stern and <qex>stout</qex>.</q> <rj><qau>Chaucer.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>A <qex>stouter</qex> champion never handled sword.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>He lost the character of a bold, <qex>stout</qex>, magnanimous man.</q> <rj><qau>Clarendon.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>The lords all stand<br/
To clear their cause, most resolutely <qex>stout</qex>.</q> <rj><qau>Daniel.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Proud; haughty; arrogant; hard.</def> <mark>[Archaic]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>Your words have been <qex>stout</qex> against me.</q> <rj><qau>Mal. iii. 13.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Commonly . . . they that be rich are lofty and <qex>stout</qex>.</q> <rj><qau>Latimer.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>Firm; tough; materially strong; enduring; <as>as, a <ex>stout</ex> vessel, stick, string, or cloth</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>Large; bulky; corpulent.</def><br/
[<source>1913 Webster</source>]</p>

<p><syn><b>Syn.</b> -- <er>Stout</er>, <er>Corpulent</er>, <er>Portly</er>.</syn> <usage> <xex>Corpulent</xex> has reference simply to a superabundance or excess of flesh. <xex>Portly</xex> implies a kind of stoutness or corpulence which gives a dignified or imposing appearance. <xex>Stout</xex>, in our early writers (as in the English Bible), was used chiefly or wholly in the sense of <xex>strong</xex> or <xex>bold</xex>; <as>as, a <ex>stout</ex> champion; a <ex>stout</ex> heart; a <ex>stout</ex> resistance, etc.</as> At a later period it was used for <xex>thickset</xex> or <xex>bulky</xex>, and more recently, especially in England, the idea has been carried still further, so that Taylor says in his Synonyms: <ldquo/The <xex>stout</xex> man has the proportions of an ox; he is corpulent, fat, and fleshy in relation to his size.<rdquo/ In America, <xex>stout</xex> is still commonly used in the original sense of <xex>strong</xex> as, a <xex>stout</xex> boy; a <xex>stout</xex> pole.</usage><br/
[<source>1913 Webster</source>]</p>

<p><ent>stout</ent><br/
<hw>stout</hw>, <pos>n.</pos> <def>A strong, dark malt brew having a higher percentage of hops than porter; strong porter; a popular variety sold in the U. S. is <stype>Guinness' stout</stype>.</def>  <rj><au>Swift.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stout-hearted</ent><br/
<hw>Stout"-heart`ed</hw> <pr>(stout"h<aum/rt`<ecr/d)</pr>, <pos>a.</pos> <def>Having a brave heart; courageous.</def> -- <wordforms><wf>Stout"-heart`ed*ness</wf>, <pos>n.</pos></wordforms><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stoutish</ent><br/
<hw>Stout"ish</hw>, <pos>a.</pos> <def>Somewhat stout; somewhat corpulent.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stoutly</ent><br/
<hw>Stout"ly</hw>, <pos>adv.</pos> <def>In a stout manner; lustily; boldly; obstinately; <as>as, he <ex>stoutly</ex> defended himself</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stoutness</ent><br/
<hw>Stout"ness</hw>, <pos>n.</pos> <def>The state or quality of being stout.</def><br/
[<source>1913 Webster</source>]</p>

<p><syn><b>Syn.</b> -- Strength; bulk; courage; force; valor; lustiness; brawniness; boldness; fortitude; stubbornness.</syn><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stovine</ent><br/
<ent>Stovain</ent><br/
<mhw>{ <hw>Sto"va*in</hw> <pr>(?)</pr>, <pos>n.</pos> Also <hw>Sto"vine</hw>  }</mhw>. <ety>[<ets>Stove</ets> (a translation of the name of the discoverer, <ets>Fourneau</ets> + <ets>-in</ets>, <ets>-ine</ets>.]</ety> <fld>(Pharm.)</fld> <def>A substance, <chform>C14H22O2NCl</chform>, the hydrochloride of an amino compound containing benzol, used, in solution with strychnine, as a local anaesthetic, esp. by injection into the sheath of the spinal cord, producing anaesthesia below the point of introduction.  Called also <altname>amylocaine hydrochloride</altname>.  Chemically it is the hydrochloride of the benzoyl ester of <chname>1-(dimethylaminomethyl)-1-methyl propanol</chname>.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Stove</ent><br/
<hw>Stove</hw> <pr>(st<omac/v)</pr>, <def><pos>imp.</pos> of <er>Stave</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stove</ent><br/
<hw>Stove</hw>, <pos>n.</pos> <ety>[D. <ets>stoof</ets> a foot stove, originally, a heated room, a room for a bath; akin to G. <ets>stube</ets> room, OHG. <ets>stuba</ets> a heated room, AS. <ets>stofe</ets>, Icel. <ets>stofa</ets> a room, bathing room, Sw. <ets>stufva</ets>, <ets>stuga</ets>, a room, Dan. <ets>stue</ets>; of unknown origin.  Cf. <er>Estufa</er>, <er>Stew</er>, <er>Stufa</er>.]</ety> <sn>1.</sn> <def>A house or room artificially warmed or heated; a forcing house, or hothouse; a drying room; -- formerly, designating an artificially warmed dwelling or room, a parlor, or a bathroom, but now restricted, in this sense, to heated houses or rooms used for horticultural purposes or in the processes of the arts.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>When most of the waiters were commanded away to their supper, the parlor or <qex>stove</qex> being nearly emptied, in came a company of musketeers.</q> <rj><qau>Earl of Strafford.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>How tedious is it to them that live in <qex>stoves</qex> and caves half a year together, as in Iceland, Muscovy, or under the pole!</q> <rj><qau>Burton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>An apparatus, consisting essentially of a receptacle for fuel, made of iron, brick, stone, or tiles, and variously constructed, in which fire is made or kept for warming a room or a house, or for culinary or other purposes.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <specif>Hence, in modern dwellings:</specif> <def>An appliance having a top surface with fittings suitable for heating pots and pans for cooking, frying, or boiling food, most commonly heated by gas or electricity, and often combined with an oven in a single unit; a <cref>cooking stove</cref>.  Such units commonly have two to six heating surfaces, called <part>burners</part>, even if they are heated by electricity rather than a gas flame.</def><br/
[<source>PJC</source>]</p>

<p><cs><col><b>Cooking stove</b></col>, <cd>a stove with an oven, opening for pots, kettles, and the like, -- used for cooking.</cd> -- <col><b>Dry stove</b></col>. <cd>See under <er>Dry</er>.</cd> -- <col><b>Foot stove</b></col>. <cd>See under <er>Foot</er>.</cd> -- <col><b>Franklin stove</b></col>. <cd>See in the Vocabulary.</cd> -- <col><b>Stove plant</b></col> <fld>(Bot.)</fld>, <cd>a plant which requires artificial heat to make it grow in cold or cold temperate climates.</cd> -- <col><b>Stove plate</b></col>, <cd>thin iron castings for the parts of stoves.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stove</ent><br/
<hw>Stove</hw>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Stoved</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Stoving</conjf>.]</vmorph> <sn>1.</sn> <def>To keep warm, in a house or room, by artificial heat; <as>as, to <ex>stove</ex> orange trees</as>.</def>  <rj><au>Bacon.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To heat or dry, as in a stove; <as>as, to <ex>stove</ex> feathers</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stovehouse</ent><br/
<hw>Stove"house`</hw> <pr>(st<omac/v"hous`)</pr>, <pos>n.</pos> <def>A hothouse.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stovepipe</ent><br/
<hw>Stove"pipe`</hw> <pr>(st<omac/v"p<imac/p`)</pr>, <pos>n.</pos> <def>Pipe made of sheet iron in length and angular or curved pieces fitting together, -- used to connect a portable stove with a chimney flue.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Stovepipe hat</b></col>, <cd>A tall silk hat with a brim, worn commonly as an item of formal dress by gentlemen in the late 1800's.</cd> <mark>[Slang, U.S.]</mark></cs><br/
[<source>1913 Webster</source> <source>+PJC</source>]</p>

<p><ent>Stover</ent><br/
<hw>Sto"ver</hw> <pr>(st<omac/"v<etil/r)</pr>, <pos>n.</pos> <ety>[OE. <ets>estoveir</ets>, <ets>estovoir</ets>, necessity, provisions, properly an inf., <ldquo/to be necessary.<rdquo/  Cf. <er>Estovers</er>.]</ety> <def>Fodder for cattle, especially straw or coarse hay.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Where live nibbling sheep,<br/
And flat meads thatched with <qex>stover</qex> them to keep.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Thresh barley as yet but as need shall require,<br/
Fresh threshed for <qex>stover</qex> thy cattle desire.</q> <rj><qau>Tusser.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stow</ent><br/
<hw>Stow</hw> <pr>(st<omac/)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Stowed</conjf> <pr>(st<omac/d)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Stowing</conjf>.]</vmorph> <ety>[OE. <ets>stowen</ets>, fr. <ets>stowe</ets> a place, AS. <ets>stow</ets>; cf. Icel. eld<ets>st<omac/</ets>a fireplace, hearth, OFries. <ets>st<omac/</ets>, and E. <ets>stand</ets>. <root/163.]</ety> <sn>1.</sn> <def>To place or arrange in a compact mass; to put in its proper place, or in a suitable place; to pack; <as>as, to <ex>stow</ex>bags, bales, or casks in a ship's hold; to <ex>stow</ex> hay in a mow; to <ex>stow</ex> sheaves</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Some <qex>stow</qex> their oars, or stop the leaky sides.</q> <rj><qau>Dryden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To put away in some place; to hide; to lodge.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Foul thief! where hast thou <qex>stowed</qex> my daughter?</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>To arrange anything compactly in; to fill, by packing closely; <as>as, to <ex>stow</ex> a box, car, or the hold of a ship</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stowage</ent><br/
<hw>Stow"age</hw> <pr>(?)</pr>, <pos>n.</pos> <sn>1.</sn> <def>The act or method of stowing; <as>as, the <ex>stowage</ex> of provisions in a vessel</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Room in which things may be stowed.</def>  <rj><au>Cook.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>In every vessel is <qex>stowage</qex> for immense treasures.</q> <rj><qau>Addison.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>The state of being stowed, or put away.</def> <ldquo/To have them in safe <xex>stowage</xex>.<rdquo/  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>Things stowed or packed.</def>  <rj><au>Beau. & Fl.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <def>Money paid for stowing goods.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stowaway</ent><br/
<hw>Stow"a*way`</hw> <pr>(?)</pr>, <pos>n.</pos> <def>One who conceals himself board of a vessel about to leave port, or on a railway train, in order to obtain a free passage.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stowboard</ent><br/
<hw>Stow"board</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A place into which rubbish is put.</def> <altsp>[Written also <asp>stowbord</asp>.]</altsp><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stowce</ent><br/
<hw>Stowce</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Mining)</fld> <sd>(a)</sd> <def>A windlass.</def> <sd>(b)</sd> <def>A wooden landmark, to indicate possession of mining land.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stowing</ent><br/
<hw>Stow"ing</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Mining)</fld> <def>A method of working in which the waste is packed into the space formed by excavating the vein.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stowre</ent><br/
<hw>Stowre</hw> <pr>(?)</pr>, <pos>a.</pos> <def>See <er>Stour</er>, <pos>a.</pos></def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stowre</ent><br/
<hw>Stowre</hw>, <pos>n.</pos> <def>See <er>Stour</er>, <pos>n.</pos></def> <mark>[Obs.]</mark>  <rj><au>Spenser.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Strabism</ent><br/
<hw>Stra"bism</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Med.)</fld> <def>Strabismus.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Strabismometer</ent><br/
<hw>Stra`bis*mom"e*ter</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[<ets>Strabismus</ets> + <ets>-meter</ets>.]</ety> <fld>(Med.)</fld> <def>An instrument for measuring the amount of strabismus.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Strabismus</ent><br/
<hw>Stra*bis"mus</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[NL., fr. Gr. <?/, fr. <?/ to squint, fr. <?/ distorted, squinting.]</ety> <fld>(Med.)</fld> <def>An affection of one or both eyes, in which the optic axes can not be directed to the same object, -- a defect due either to undue contraction or to undue relaxation of one or more of the muscles which move the eyeball; squinting; cross-eye.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Strabotomy</ent><br/
<hw>Stra*bot"o*my</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Gr. <?/ squinting + <?/ to cut.]</ety> <fld>(Surg.)</fld> <def>The operation for the removal of squinting by the division of such muscles as distort the eyeball.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Straddle</ent><br/
<hw>Strad"dle</hw> <pr>(?)</pr>, <pos>v. i.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Straddled</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Straddling</conjf> <pr>(?)</pr>.]</vmorph> <ety>[Freq. from the root of <ets>stride</ets>.]</ety> <sn>1.</sn> <def>To part the legs wide; to stand or to walk with the legs far apart.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To stand with the ends staggered; -- said of the spokes of a wagon wheel where they join the hub.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Straddle</ent><br/
<hw>Strad"dle</hw>, <pos>v. t.</pos> <def>To place one leg on one side and the other on the other side of; to stand or sit astride of; <as>as, to <ex>straddle</ex> a fence or a horse</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Straddle</ent><br/
<hw>Strad"dle</hw>, <pos>n.</pos> <sn>1.</sn> <def>The act of standing, sitting, or walking, with the feet far apart.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>The position, or the distance between the feet, of one who straddles; as, a wide <xex>straddle</xex>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>A stock option giving the holder the double privilege of a <ldquo/put<rdquo/ and a <ldquo/call,<rdquo/ <it>i. e.</it>, securing to the buyer of the option the right either to demand of the seller at a certain price, within a certain time, certain securities, or to require him to take at the same price, and within the same time, the same securities.</def> <mark>[Broker's Cant]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Straddling</ent><br/
<hw>Strad"dling</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Applied to spokes when they are arranged alternately in two circles in the hub. See <er>Straddle</er>, <pos>v. i.</pos>, and <er>Straddle</er>, <pos>v. t.</pos>, 3.</def>  <rj><au>Knight.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stradometrical</ent><br/
<hw>Strad`o*met"ric*al</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[It. <ets>strada</ets> street or road + E. <ets>metrical</ets>.]</ety> <def>Of, or relating to, the measuring of streets or roads.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Straggle</ent><br/
<hw>Strag"gle</hw> <pr>(?)</pr>, <pos>v. i.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Straggled</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Straggling</conjf> <pr>(?)</pr>.]</vmorph> <ety>[Freq. of OE. <ets>straken</ets> to roam, to stroke. See <er>Stroke</er>, <pos>v. t.</pos>]</ety> <sn>1.</sn> <def>To wander from the direct course or way; to rove; to stray; to wander from the line of march or desert the line of battle; <as>as, when troops are on the march, the men should not <ex>straggle</ex></as>.</def>  <rj><au>Dryden.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To wander at large; to roam idly about; to ramble.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The wolf spied out a <qex>straggling</qex> kid.</q> <rj><qau>L'Estrange.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>To escape or stretch beyond proper limits, as the branches of a plant; to spread widely apart; to shoot too far or widely in growth.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Trim off the small, superfluous branches on each side of the hedge that <qex>straggle too far out</qex>.</q> <rj><qau>Mortimer.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>To be dispersed or separated; to occur at intervals.</def> <ldquo/<xex>Straggling</xex> pistol shots.<rdquo/  <rj><au>Sir W. Scott.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>They came between Scylla and Charybdis and the <qex>straggling</qex> rocks.</q> <rj><qau>Sir W. Raleigh.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Straggle</ent><br/
<hw>Strag"gle</hw>, <pos>n.</pos> <def>The act of straggling.</def> <mark>[R.]</mark>  <rj><au>Carlyle.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Straggler</ent><br/
<hw>Strag"gler</hw> <pr>(?)</pr>, <pos>n.</pos> <sn>1.</sn> <def>One who straggles, or departs from the direct or proper course, or from the company to which he belongs; one who falls behind the rest; one who rambles without any settled direction.</def><-- they shot all the stragglers --><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A roving vagabond.</def>  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>Something that shoots, or spreads out, beyond the rest, or too far; an exuberant growth.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Let thy hand supply the pruning knife,<br/
And crop luxuriant <qex>stragglers</qex>.</q> <rj><qau>Dryden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>Something that stands alone or by itself.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Straggling</ent><br/
<hw>Strag"gling</hw> <pr>(?)</pr>, <def><pos>a. & n.</pos> from <er>Straggle</er>, <pos>v.</pos></def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stragglingly</ent><br/
<hw>Strag"gling*ly</hw>, <pos>adv.</pos> <def>In a straggling manner.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stragulum</ent><br/
||<hw>Strag"u*lum</hw> <pr>(?)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Stragula</plw> <pr>(#)</pr>.</plu> <ety>[L., a spread or covering, from <ets>sternere</ets> to spread out.]</ety> <fld>(Zool.)</fld> <def>The mantle, or pallium, of a bird.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Straight</ent><br/
<hw>Straight</hw> <pr>(?)</pr>, <pos>a.</pos> <def>A variant of <er>Strait</er>, <pos>a.</pos></def> <mark>[Obs. or R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>Egypt is a long country, but it is <qex>straight</qex>, that is to say, narrow.</q> <rj><qau>Sir J. Mandeville.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Straight</ent><br/
<hw>Straight</hw>, <pos>a.</pos> <amorph>[<pos>Compar.</pos> <adjf>Straighter</adjf> <pr>(?)</pr>; <pos>superl.</pos> <adjf>Straightest</adjf>.]</amorph> <ety>[OE. <ets>strei<?/t</ets>, properly p. p. of <ets>strecchen</ets> to stretch, AS. <ets>streht</ets>, p. p. of <ets>streccan</ets> to stretch, to extend. See <er>Stretch</er>.]</ety> <sn>1.</sn> <def>Right, in a mathematical sense; passing from one point to another by the nearest course; direct; not deviating or crooked; <as>as, a <ex>straight</ex> line or course; a <ex>straight</ex> piece of timber</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>And the crooked shall be made <qex>straight</qex>.</q> <rj><qau>Isa. xl. 4.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>There are many several sorts of crooked lines, but there is only one which is <qex>straight</qex>.</q> <rj><qau>Dryden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Bot.)</fld> <def>Approximately straight; not much curved; <as>as, <ex>straight</ex> ribs are such as pass from the base of a leaf to the apex, with a small curve</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <fld>(Card Playing)</fld> <def>Composed of cards which constitute a regular sequence, as the ace, king, queen, jack, and ten-spot; <as>as, a <ex>straight</ex> hand; a <ex>straight</ex> flush.</as></def>
<-- previously called also sequence, which see. --><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>Conforming to justice and rectitude; not deviating from truth or fairness; upright; <as>as, <ex>straight</ex> dealing</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <def>Unmixed; undiluted; <as>as, to take liquor <ex>straight</ex></as>.</def> <mark>[Slang]</mark><br/
[<source>1913 Webster</source>]</p>

<p><sn>6.</sn> <def>Making no exceptions or deviations in one's support of the organization and candidates of a political party; <as>as, a <ex>straight</ex> Republican; a <ex>straight</ex> Democrat</as>; also, containing the names of all the regularly nominated candidates of a party and no others; <as>as, a <ex>straight</ex> ballot</as>.</def> <mark>[Political Cant, U.S.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Straight arch</b></col> <fld>(Arch.)</fld>, <cd>a form of arch in which the intrados is straight, but with its joints drawn radially, as in a common arch.</cd> -- <col><b>A straight face</b></col>, <cd>one giving no evidence of merriment or other emotion.</cd> -- <col><b>A straight line</b></col>. <cd><ldquo/That which lies evenly between its extreme points.<rdquo/ <au>Euclid.</au> <ldquo/The shortest line between two points.<rdquo/ <au>Chauvenet.</au> <ldquo/A line which has the same direction through its whole length.<rdquo/ <au>Newcomb.</au></cd> -- <col><b>Straight-way valve</b></col>, <cd>a valve which, when opened widely, affords a straight passageway, as for water.</cd></cs>
<-- the straight and narrow, proper ethical conduct; -- used esp. in the phrase <it>walk the straight and narrow</it>. --><br/
[<source>1913 Webster</source>]</p>

<p><ent>Straight</ent><br/
<hw>Straight</hw> <pr>(?)</pr>, <pos>adv.</pos> <def>In a straight manner; directly; rightly; forthwith; immediately; <as>as, the arrow went <ex>straight</ex> to the mark</as>.</def> <ldquo/Floating <xex>straight</xex>.<rdquo/  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>I know thy generous temper well;<br/
Fling but the appearance of dishonor on it,<br/
It <qex>straight</qex> takes fire, and mounts into a blaze.</q> <rj><qau>Addison.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Everything was going on <qex>straight</qex>.</q> <rj><qau>W. Black.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Straight</ent><br/
<hw>Straight</hw>, <pos>n.</pos> <fld>(Poker)</fld> <def>A hand of five cards in consecutive order as to value; a sequence. When they are of one suit, it is calles <xex>straight flush</xex>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Straight</ent><br/
<hw>Straight</hw>, <pos>v. t.</pos> <def>To straighten.</def> <mark>[R.]</mark>  <rj><au>A Smith.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Straightedge</ent><br/
<hw>Straight"edge`</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A board, or piece of wood or metal, having one edge perfectly straight, -- used to ascertain whether a line is straight or a surface even, and for drawing straight lines.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Straighten</ent><br/
<hw>Straight"en</hw> <pr>(?)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Straighted</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Straighting</conjf>.]</vmorph> <sn>1.</sn> <def>To make straight; to reduce from a crooked to a straight form.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To make right or correct; to reduce to order; <as>as, to <ex>straighten</ex> one's affairs; to <ex>straighten</ex> an account</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>To straighten one's face</b></col>, <cd>to cease laughing or smiling, etc., and compose one's features.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Straighten</ent><br/
<hw>Straight"en</hw>, <pos>v. t.</pos> <def>A variant of <er>Straiten</er>.</def> <mark>[Obs. or R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Straightener</ent><br/
<hw>Straight"en*er</hw> <pr>(?)</pr>, <pos>n.</pos> <def>One who, or that which, straightens.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Straightforth</ent><br/
<hw>Straight"forth`</hw> <pr>(?)</pr>, <pos>adv.</pos> <def>Straightway.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Straightforward</ent><br/
<hw>Straight`for"ward</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Proceeding in a straight course or manner; not deviating; honest; frank.</def> -- <def2><pos>adv.</pos> <def>In a straightforward manner.</def></def2> -- <wordforms><wf>Straight`for"ward*ly</wf>, <pos>adv.</pos> -- <wf>Straight`for"ward*ness</wf>, <pos>n.</pos></wordforms><br/
[<source>1913 Webster</source>]</p>

<p><ent>Straighthorn</ent><br/
<hw>Straight"horn`</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Paleon.)</fld> <def>An orthoceras.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Straight-joint</ent><br/
<hw>Straight"-joint`</hw> <pr>(?)</pr>, <pos>a.</pos> <fld>(Arch.)</fld> <def>Having straight joints.</def> Specifically: <sd>(a)</sd> <def>Applied to a floor the boards of which are so laid that the joints form a continued line transverse to the length of the boards themselves.</def> <au>Brandle & C.</au> <sd>(b)</sd> <def>In the United States, applied to planking or flooring put together without the tongue and groove, the pieces being laid edge to edge.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Straight-lined</ent><br/
<hw>Straight"-lined`</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Having straight lines.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Straightly</ent><br/
<hw>Straight"ly</hw>, <pos>adv.</pos> <def>In a right line; not crookedly.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Straightly</ent><br/
<hw>Straight"ly</hw>, <pos>adv.</pos> <def>A variant of <er>Straitly</er>. See 1st <er>Straight</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Straightness</ent><br/
<hw>Straight"ness</hw>, <pos>n.</pos> <def>The quality, condition, or state, of being straight; <as>as, the <ex>straightness</ex> of a path</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Straightness</ent><br/
<hw>Straight"ness</hw>, <pos>n.</pos> <def>A variant of <er>Straitness</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Straight-out</ent><br/
<hw>Straight"-out`</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Acting without concealment, obliquity, or compromise; hence, unqualified; thoroughgoing.</def> <mark>[Colloq. U.S.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q><qex>Straight-out</qex> and generous indignation.</q> <rj><qau>Mrs. Stowe.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Straight-pight</ent><br/
<hw>Straight"-pight`</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Straight in form or upright in position; erect.</def> <mark>[Obs.]</mark>  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Straight-spoken</ent><br/
<hw>Straight"-spo`ken</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Speaking with directness; plain-spoken.</def> <mark>[Colloq. U.S.]</mark>  <rj><au>Lowell.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Straightway</ent><br/
<hw>Straight"way`</hw> <pr>(?)</pr>, <pos>adv.</pos> <def>Immediately; without loss of time; without delay.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>He took the damsel by the hand, and said unto her, Talitha cumi. . . . And <qex>straightway</qex> the damsel arose.</q> <rj><qau>Mark v. 41,42.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Straightways</ent><br/
<hw>Straight"ways`</hw> <pr>(?)</pr>, <pos>adv.</pos> <def>Straightway.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Straik</ent><br/
<hw>Straik</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A strake.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Strain</ent><br/
<hw>Strain</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[See <er>Strene</er>.]</ety> <sn>1.</sn> <def>Race; stock; generation; descent; family.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>He is of a noble <qex>strain</qex>.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>With animals and plants a cross between different varieties, or between individuals of the same variety but of another <qex>strain</qex>, gives vigor and fertility to the offspring.</q> <rj><qau>Darwin.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Hereditary character, quality, or disposition.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Intemperance and lust breed diseases, which, propogated, spoil the <qex>strain</qex> of nation.</q> <rj><qau>Tillotson.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>Rank; a sort.</def> <ldquo/The common <xex>strain</xex>.<rdquo/  <rj><au>Dryden.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <fld>(Hort.)</fld> <def>A cultural subvariety that is only slightly differentiated.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Strain</ent><br/
<hw>Strain</hw>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Strained</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Straining</conjf>.]</vmorph> <ety>[OF. <ets>estraindre</ets>, <ets>estreindre</ets>, F. <ets><eacute/treindre</ets>, L. <ets>stringere</ets> to draw or bind tight; probably akin to Gr. <?/ a halter, <?/ that which is squeezwd out, a drop, or perhaps to E. <ets>strike</ets>.  Cf. <er>Strangle</er>, <er>Strike</er>, <er>Constrain</er>, <er>District</er>, <er>Strait</er>, <pos>a.</pos> <er>Stress</er>, <er>Strict</er>, <er>Stringent</er>.]</ety> <sn>1.</sn> <def>To draw with force; to extend with great effort; to stretch; <as>as, to <ex>strain</ex> a rope; to <ex>strain</ex> the shrouds of a ship; to <ex>strain</ex> the cords of a musical instrument</as>.</def> <ldquo/To <xex>strain</xex> his fetters with a stricter care.<rdquo/  <rj><au>Dryden.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Mech.)</fld> <def>To act upon, in any way, so as to cause change of form or volume, as forces on a beam to bend it.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>To exert to the utmost; to ply vigorously.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>He sweats,<br/
<qex>Strains</qex> his young nerves.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>They <qex>strain</qex> their warbling throats<br/
To welcome in the spring.</q> <rj><qau>Dryden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>To stretch beyond its proper limit; to do violence to, in the matter of intent or meaning; <as>as, to <ex>strain</ex> the law in order to convict an accused person</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>There can be no other meaning in this expression, however some may pretend to <qex>strain</qex> it.</q> <rj><qau>Swift.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <def>To injure by drawing, stretching, or the exertion of force; <as>as, the gale <ex>strained</ex> the timbers of the ship</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>6.</sn> <def>To injure in the muscles or joints by causing to make too strong an effort; to harm by overexertion; to sprain; <as>as, to <ex>strain</ex> a horse by overloading; to <ex>strain</ex> the wrist; to <ex>strain</ex> a muscle</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Prudes decayed about may track,<br/
<qex>Strain</qex> their necks with looking back.</q> <rj><qau>Swift.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>7.</sn> <def>To squeeze; to press closely.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Evander with a close embrace<br/
<qex>Strained</qex> his departing friend.</q> <rj><qau>Dryden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>8.</sn> <def>To make uneasy or unnatural; to produce with apparent effort; to force; to constrain.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>He talks and plays with Fatima, but his mirth<br/
Is forced and <qex>strained</qex>.</q> <rj><qau>Denham.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>The quality of mercy is not <qex>strained</qex>.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>9.</sn> <def>To urge with importunity; to press; <as>as, to <ex>strain</ex> a petition or invitation</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Note, if your lady <qex>strain</qex> his entertainment.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>10.</sn> <def>To press, or cause to pass, through a strainer, as through a screen, a cloth, or some porous substance; to purify, or separate from extraneous or solid matter, by filtration; to filter; <as>as, to <ex>strain</ex> milk through cloth</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>To strain a point</b></col>, <cd>to make a special effort; especially, to do a degree of violence to some principle or to one's own feelings.</cd> -- <col><b>To strain courtesy</b></col>, <cd>to go beyond what courtesy requires; to insist somewhat too much upon the precedence of others; -- often used ironically.</cd> <au>Shak.</au></cs><br/
[<source>1913 Webster</source>]</p>

<p><-- p. 1422 --></p>

<p><ent>Strain</ent><br/
<hw>Strain</hw> <pr>(str<amac/n)</pr>, <pos>v. i.</pos> <sn>1.</sn> <def>To make violent efforts.</def> <ldquo/<xex>Straining</xex> with too weak a wing.<rdquo/  <rj><au>Pope.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>To build his fortune I will <qex>strain</qex> a little.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To percolate; to be filtered; <as>as, water <ex>straining</ex> through a sandy soil</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Strain</ent><br/
<hw>Strain</hw>, <pos>n.</pos> <sn>1.</sn> <def>The act of straining, or the state of being strained.</def> Specifically: --<br/
[<source>1913 Webster</source>]</p>

<p><sd>(a)</sd> <def>A violent effort; an excessive and hurtful exertion or tension, as of the muscles; <as>as, he lifted the weight with a <ex>strain</ex>; the <ex>strain</ex> upon a ship's rigging in a gale;</as> also, the hurt or injury resulting; a sprain.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Whether any poet of our country since Shakespeare has exerted a greater variety of powers with less <qex>strain</qex> and less ostentation.</q> <rj><qau>Landor.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Credit is gained by custom, and seldom recovers a <qex>strain</qex>.</q> <rj><qau>Sir W. Temple.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sd>(b)</sd> <fld>(Mech. Physics)</fld> <def>A change of form or dimensions of a solid or liquid mass, produced by a stress.</def>  <rj><au>Rankine.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Mus.)</fld> <def>A portion of music divided off by a double bar; a complete musical period or sentence; a movement, or any rounded subdivision of a movement.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Their heavenly harps a lower <qex>strain</qex> began.</q> <rj><qau>Dryden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>Any sustained note or movement; a song; a distinct portion of an ode or other poem; also, the pervading note, or burden, of a song, poem, oration, book, etc.; theme; motive; manner; style; also, a course of action or conduct; <as>as, he spoke in a noble <ex>strain</ex>; there was a <ex>strain</ex> of woe in his story; a <ex>strain</ex> of trickery appears in his career</as>.</def> <ldquo/A <xex>strain</xex> of gallantry.<rdquo/  <rj><au>Sir W. Scott.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Such take too high a <qex>strain</qex> at first.</q> <rj><qau>Bacon.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>The genius and <qex>strain</qex> of the book of Proverbs.</q> <rj><qau>Tillotson.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>It [Pilgrim's Progress] seems a novelty, and yet contains<br/
Nothing but sound and honest gospel <qex>strains</qex>.</q> <rj><qau>Bunyan.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>Turn; tendency; inborn disposition.  Cf. 1st <er>Strain</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Because heretics have a <qex>strain</qex> of madness, he applied her with some corporal chastisements.</q> <rj><qau>Hayward.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Strainable</ent><br/
<hw>Strain"a*ble</hw> <pr>(-<adot/*b'l)</pr>, <pos>a.</pos> <sn>1.</sn> <def>Capable of being strained.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Violent in action.</def>  <rj><au>Holinshed.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Strainably</ent><br/
<hw>Strain"a*bly</hw>, <pos>adv.</pos> <def>Violently.</def>  <rj><au>Holinshed.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Strained</ent><br/
<hw>Strained</hw> <pr>(?)</pr>, <pos>a.</pos> <sn>1.</sn> <def>Subjected to great or excessive tension; wrenched; weakened; <as>as, <ex>strained</ex> relations between old friends</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Done or produced with straining or excessive effort; <as>as, his wit was <ex>strained</ex></as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Strainer</ent><br/
<hw>Strain"er</hw> <pr>(?)</pr>, <pos>n.</pos> <sn>1.</sn> <def>One who strains.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>That through which any liquid is passed for purification or to separate it from solid matter; anything, as a screen or a cloth, used to strain a liquid; a device of the character of a sieve or of a filter; specifically, an openwork or perforated screen, as for the end of the suction pipe of a pump, to prevent large solid bodies from entering with a liquid.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Straining</ent><br/
<hw>Strain"ing</hw>, <def><pos>a. & n.</pos> from <er>Strain</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Straining piece</b></col> <fld>(Arch.)</fld>, <cd>a short piece of timber in a truss, used to maintain the ends of struts or rafters, and keep them from slipping. See <xex>Illust.</xex> of <er>Queen-post</er>.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Straint</ent><br/
<hw>Straint</hw> <pr>(str<amac/nt)</pr>, <pos>n.</pos> <ety>[OF. <ets>estrainte</ets>, <ets>estreinte</ets>, F. <ets><eacute/trainte</ets>. See 2nd <er>Strain</er>.]</ety> <def>Overexertion; excessive tension; strain.</def> <mark>[Obs.]</mark>  <rj><au>Spenser.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Strait</ent><br/
<hw>Strait</hw> <pr>(?)</pr>, <pos>a.</pos> <def>A variant of <er>Straight</er>.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Strait</ent><br/
<hw>Strait</hw> <pr>(?)</pr>, <pos>a.</pos> <amorph>[<pos>Compar.</pos> <adjf>Straiter</adjf> <pr>(?)</pr>; <pos>superl.</pos> <adjf>Straitest</adjf>.]</amorph> <ety>[OE. <ets>straight</ets>, <ets>streyt</ets>, <ets>streit</ets>, OF. <ets>estreit</ets>, <ets>estroit</ets>, F. <ets><eacute/troit</ets>, from L. <ets>strictus</ets> drawn together, close, tight, p. p. of <ets>stringere</ets> to draw tight. See 2nd <er>Strait</er>, and cf. <er>Strict</er>.]</ety> <sn>1.</sn> <def>Narrow; not broad.</def><br/
[<source>1913 Webster</source>]</p>

<p><q><qex>Strait</qex> is the gate, and narrow is the way, which leadeth unto life, and few there be that find it.</q> <rj><qau>Matt. vii. 14.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Too <qex>strait</qex> and low our cottage doors.</q> <rj><qau>Emerson.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Tight; close; closely fitting.</def>  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>Close; intimate; near; familiar.</def> <mark>[Obs.]</mark> <ldquo/A <xex>strait</xex> degree of favor.<rdquo/  <rj><au>Sir P. Sidney.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>Strict; scrupulous; rigorous.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Some certain edicts and some <qex>strait</qex> decrees.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>The <qex>straitest</qex> sect of our religion.</q> <rj><qau>Acts xxvi. 5 (Rev. Ver.).</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <def>Difficult; distressful; straited.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>To make your <qex>strait</qex> circumstances yet <qex>straiter</qex>.</q> <rj><qau>Secker.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>6.</sn> <def>Parsimonious; niggargly; mean.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>I beg cold comfort, and you are so <qex>strait</qex>,<br/
And so ingrateful, you deny me that.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Strait</ent><br/
<hw>Strait</hw> <pr>(?)</pr>, <pos>adv.</pos> <def>Strictly; rigorously.</def> <mark>[Obs.]</mark>  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Strait</ent><br/
<hw>Strait</hw>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Straits</plw> <pr>(#)</pr>.</plu> <ety>[OE. <ets>straight</ets>, <ets>streit</ets>, OF. <ets>estreit</ets>, <ets>estroit</ets>. See <er>Strait</er>, <pos>a.</pos>]</ety> <sn>1.</sn> <def>A narrow pass or passage.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>He brought him through a darksome narrow <qex>strait</qex><br/
To a broad gate all built of beaten gold.</q> <rj><qau>Spenser.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Honor travels in a <qex>strait</qex> so narrow<br/
Where one but goes abreast.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> Specifically: <fld>(Geog.)</fld> <def>A (comparatively) narrow passageway connecting two large bodies of water; -- often in the plural; <as>as, the <ex>strait</ex>, or <ex>straits</ex>, of Gibraltar; the <ex>straits</ex> of Magellan; the <ex>strait</ex>, or <ex>straits</ex>, of Mackinaw.</as></def><br/
[<source>1913 Webster</source>]</p>

<p><q>We steered directly through a large outlet which they call a <qex>strait</qex>, though it be fifteen miles broad.</q> <rj><qau>De Foe.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>A neck of land; an isthmus.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>A dark <qex>strait</qex> of barren land.</q> <rj><qau>Tennyson.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>Fig.: A condition of narrowness or restriction; doubt; distress; difficulty; poverty; perplexity; -- sometimes in the plural; <as>as, reduced to great <ex>straits</ex></as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>For I am in a <qex>strait</qex> betwixt two.</q> <rj><qau>Phil. i. 23.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Let no man, who owns a Providence, grow desperate under any calamity or <qex>strait</qex> whatsoever.</q> <rj><qau>South.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Ulysses made use of the pretense of natural infirmity to conceal the <qex>straits</qex> he was in at that time in his thoughts.</q> <rj><qau>Broome.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Strait</ent><br/
<hw>Strait</hw>, <pos>v. t.</pos> <def>To put to difficulties.</def> <mark>[Obs.]</mark>  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Straiten</ent><br/
<hw>Strait"en</hw> <pr>(?)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Straitened</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Straitening</conjf>.]</vmorph> <sn>1.</sn> <def>To make strait; to make narrow; hence, to contract; to confine.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Waters, when <qex>straitened</qex>, as at the falls of bridges, give a roaring noise.</q> <rj><qau>Bacon.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>In narrow circuit, <qex>straitened</qex> by a foe.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To make tense, or tight; to tighten.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>They <qex>straiten</qex> at each end the cord.</q> <rj><qau>Pope.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>To restrict; to distress or embarrass in respect of means or conditions of life; -- used chiefly in the past participle; -- as, a man <xex>straitened</xex> in his circumstances.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Strait-handed</ent><br/
<hw>Strait"-hand`ed</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Parsimonious; sparing; niggardly.</def> <mark>[R.]</mark> -- <wordforms><wf>Strait"-hand`ed*ness</wf>, <pos>n.</pos> <mark>[R.]</mark></wordforms><br/
[<source>1913 Webster</source>]</p>

<p><ent>Strait-jacket</ent><br/
<hw>Strait"-jack`et</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A dress of strong materials for restraining maniacs or those who are violently delirious. It has long sleeves, which are closed at the ends, confining the hands, and may be tied behind the back.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Strait-laced</ent><br/
<hw>Strait"-laced`</hw> <pr>(?)</pr>, <pos>a.</pos> <sn>1.</sn> <def>Bound with stays.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Let nature have scope to fashion the body as she thinks best; we have few well-shaped that are <qex>strait-laced</qex>.</q> <rj><qau>Locke.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Restricted; stiff; constrained.</def> <mark>[R.]</mark>  <rj><au>Fuller.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>Rigid in opinion; strict in manners or morals.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Straitly</ent><br/
<hw>Strait"ly</hw>, <pos>adv.</pos> <sn>1.</sn> <def>In a strait manner; narrowly; strictly; rigorously.</def>  <rj><au>Mark i. 43.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Closely; intimately.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Straitness</ent><br/
<hw>Strait"ness</hw>, <pos>n.</pos> <def>The quality or condition of being strait; especially, a pinched condition or situation caused by poverty; <as>as, the <ex>straitness</ex>of their circumstances</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Strait-waistcoat</ent><br/
<hw>Strait"-waist`coat</hw> <pr>(?)</pr>, <pos>n.</pos> <def>Same as <er>Strait-jacket</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Strake</ent><br/
<hw>Strake</hw> <pr>(?)</pr>, <mark>obs.</mark> <def><pos>imp.</pos> of <er>Strike</er>.</def>  <rj><au>Spenser.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Strake</ent><br/
<hw>Strake</hw>, <pos>n.</pos> <ety>[See <er>Streak</er>.]</ety> <sn>1.</sn> <def>A streak.</def> <mark>[Obs.]</mark> <au>Spenser.</au><ldquo/White <xex>strake</xex>.<rdquo/ <au>Gen. xxx. 37.</au><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>An iron band by which the fellies of a wheel are secured to each other, being not continuous, as the tire is, but made up of separate pieces.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <fld>(Shipbuilding)</fld> <def>One breadth of planks or plates forming a continuous range on the bottom or sides of a vessel, reaching from the stem to the stern; a streak.</def><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ The planks or plates next the keel are called the <xex>garboard strakes</xex>; the next, or the heavy strakes at the bilge, are the <xex>bilge strakes</xex>; the next, from the water line to the lower port sill, the <xex>wales</xex>; and the upper parts of the sides, the <xex>sheer strakes</xex>.</note><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <fld>(Mining)</fld> <def>A trough for washing broken ore, gravel, or sand; a launder.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Strale</ent><br/
<hw>Strale</hw> <pr>(?)</pr>, <pos>n.</pos> <def>Pupil of the eye.</def> <mark>[Prov. Eng.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stram</ent><br/
<hw>Stram</hw> <pr>(?)</pr>, <pos>v. t.</pos> <ety>[Cf. LG. <ets>strammen</ets> to strain, straiten, stretch, D. <ets>stram</ets> strained, tight, G. <ets>stramm</ets>.]</ety> <def>To spring or recoil with violence.</def> <mark>[Prov. Eng.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stram</ent><br/
<hw>Stram</hw>, <pos>v. t.</pos> <def>To dash down; to beat.</def> <mark>[Prov. Eng.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stramash</ent><br/
<hw>Stram"ash</hw> <pr>(?)</pr>, <pos>v. t.</pos> <ety>[Cf. <er>Stramazoun</er>.]</ety> <def>To strike, beat, or bang; to break; to destroy.</def> <mark>[Scot. & Prov. Eng.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stramash</ent><br/
<hw>Stram"ash</hw>, <pos>n.</pos> <def>A turmoil; a broil; a fray; a fight.</def> <mark>[Scot. & Prov. Eng.]</mark>  <rj><au>Barham.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stramazoun</ent><br/
<hw>Stram"a*zoun</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F. <ets>estrama<cced/on</ets>, It. <ets>stramazzone</ets>.]</ety> <def>A direct descending blow with the edge of a sword.</def> <mark>[Obs.]</mark>  <rj><au>B. Jonson.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stramineous</ent><br/
<hw>Stra*min"e*ous</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>stramineus</ets>, fr. <ets>stramen</ets> straw, fr. <ets>sternere</ets>, <ets>stratum</ets>, to spread out, to strew.]</ety> <sn>1.</sn> <def>Strawy; consisting of straw.</def>  <rj><au>Robinson.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Chaffy; like straw; straw-colored.</def>  <rj><au>Burton.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stramonium</ent><br/
<hw>Stra*mo"ni*um</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[NL.; Cf. F. <ets>stramoine</ets>.]</ety> <fld>(Bot.)</fld> <def>A poisonous plant (<spn>Datura Stramonium</spn>); stinkweed. See <er>Datura</er>, and <er>Jamestown weed</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stramony</ent><br/
<hw>Stram"o*ny</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Bot.)</fld> <def>Stramonium.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Strand</ent><br/
<hw>Strand</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Probably fr. D. <ets>streen</ets> a skein; akin to G. <ets>str<aum/hne</ets> a skein, lock of hair, strand of a rope.]</ety> <def>One of the twists, or strings, as of fibers, wires, etc., of which a rope is composed.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Strand</ent><br/
<hw>Strand</hw>, <pos>v. t.</pos> <def>To break a strand of (a rope).</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Strand</ent><br/
<hw>Strand</hw>, <pos>n.</pos> <ety>[AS. <ets>strand</ets>; akin to D., G., Sw., & Dan. <ets>strand</ets>, Icel. <ets>str<oum/nd</ets>.]</ety> <def>The shore, especially the beach of a sea, ocean, or large lake; rarely, the margin of a navigable river.</def>  <rj><au>Chaucer.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Strand birds</b></col>. <fld>(Zool.)</fld> <cd>See <cref>Shore birds</cref>, under <er>Shore</er>.</cd> -- <col><b>Strand plover</b></col> <fld>(Zool.)</fld>, <cd>a black-bellied plover. See <xex>Illust.</xex> of <er>Plover</er>.</cd> -- <col><b>Strand wolf</b></col> <fld>(Zool.)</fld>, <cd>the brown hyena.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Strand</ent><br/
<hw>Strand</hw>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Stranded</conjf>; <pos>p. pr. & vb. n.</pos> <conjf>Stranding</conjf>.]</vmorph> <def>To drive on a strand; hence, to run aground; <as>as, to <ex>strand</ex> a ship</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Strand</ent><br/
<hw>Strand</hw> <pr>(?)</pr>, <pos>v. i.</pos> <def>To drift, or be driven, on shore to run aground; <as>as, the ship <ex>stranded</ex> at high water</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Strang</ent><br/
<hw>Strang</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[See <er>Strong</er>.]</ety> <def>Strong.</def> <mark>[Obs. or Prov. Eng. & Scot.]</mark>  <rj><au>Halliwell.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Strange</ent><br/
<hw>Strange</hw> <pr>(?)</pr>, <pos>a.</pos> <amorph>[<pos>Compar.</pos> <adjf>Stranger</adjf> <pr>(?)</pr>; <pos>superl.</pos> <adjf>Strangest</adjf> <pr>(?)</pr>.]</amorph> <ety>[OE. <ets>estrange</ets>, F. <ets><eacute/trange</ets>, fr. L. <ets>extraneus</ets> that is without, external, foreign, fr. <ets>extra</ets> on the outside. See <er>Extra</er>, and cf. <er>Estrange</er>, <er>Extraneous</er>.]</ety> <sn>1.</sn> <def>Belonging to another country; foreign.</def> <ldquo/To seek <xex>strange</xex> strands.<rdquo/  <rj><au>Chaucer.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>One of the <qex>strange</qex> queen's lords.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>I do not contemn the knowledge of <qex>strange</qex> and divers tongues.</q> <rj><qau>Ascham.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Of or pertaining to others; not one's own; not pertaining to one's self; not domestic.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>So she, impatient her own faults to see,<br/
Turns from herself, and in <qex>strange</qex> things delights.</q> <rj><qau>Sir J. Davies.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>Not before known, heard, or seen; new.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Here is the hand and seal of the duke; you know the character, I doubt not; and the signet is not <qex>strange</qex> to you.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>Not according to the common way; novel; odd; unusual; irregular; extraordinary; unnatural; queer.</def> <ldquo/He is sick of a <xex>strange</xex> fever.<rdquo/  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Sated at length, erelong I might perceive<br/
<qex>Strange</qex> alteration in me.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <def>Reserved; distant in deportment.</def>  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>She may be <qex>strange</qex> and shy at first, but will soon learn to love thee.</q> <rj><qau>Hawthorne.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>6.</sn> <def>Backward; slow.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>Who, loving the effect, would not be <qex>strange</qex><br/
In favoring the cause.</q> <rj><qau>Beau. & Fl.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>7.</sn> <def>Not familiar; unaccustomed; inexperienced.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>In thy fortunes am unlearned and <qex>strange</qex>.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ <xex>Strange</xex> is often used as an exclamation.</note><br/
[<source>1913 Webster</source>]</p>

<p><q><qex>Strange</qex>! what extremes should thus preserve the snow<br/
High on the Alps, or in deep caves below.</q> <rj><qau>Waller.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Strange sail</b></col> <fld>(Naut.)</fld>, <cd>an unknown vessel.</cd> -- <col><b>Strange woman</b></col> <fld>(Script.)</fld>, <cd>a harlot.</cd> <au>Prov. v. 3.</au> -- <col><b>To make it strange</b></col>. <sd>(a)</sd> <cd>To assume ignorance, suspicion, or alarm, concerning it</cd>. <au>Shak.</au> <sd>(b)</sd> <cd>To make it a matter of difficulty.</cd> <mark>[Obs.]</mark> <au>Chaucer.</au> -- <mcol><col><b>To make strange</b></col>, <col><b>To make one's self strange</b></col></mcol>. <sd>(a)</sd> <cd>To profess ignorance or astonishment</cd>. <sd>(b)</sd> <cd>To assume the character of a stranger</cd>. <au>Gen. xlii. 7.</au></cs><br/
[<source>1913 Webster</source>]</p>

<p><syn><b>Syn.</b> -- Foreign; new; outlandish; wonderful; astonishing; marvelous; unusual; odd; uncommon; irregular; queer; eccentric.</syn><br/
[<source>1913 Webster</source>]</p>

<p><ent>Strange</ent><br/
<hw>Strange</hw>, <pos>adv.</pos> <def>Strangely.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>Most <qex>strange</qex>, but yet most truly, will I speak.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Strange</ent><br/
<hw>Strange</hw>, <pos>v. t.</pos> <def>To alienate; to estrange.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Strange</ent><br/
<hw>Strange</hw>, <pos>v. i.</pos> <sn>1.</sn> <def>To be estranged or alienated.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To wonder; to be astonished.</def> <mark>[Obs.]</mark>  <rj><au>Glanvill.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Strangely</ent><br/
<hw>Strange"ly</hw>, <pos>adv.</pos> <sn>1.</sn> <def>As something foreign, or not one's own; in a manner adapted to something foreign and strange.</def> <mark>[Obs.]</mark>  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>In the manner of one who does not know another; distantly; reservedly; coldly.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>You all look <qex>strangely</qex> on me.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>I do in justice charge thee . . . <br/
That thou commend it <qex>strangely</qex> to some place<br/
Where chance may nurse or end it.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>In a strange manner; in a manner or degree to excite surprise or wonder; wonderfully.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>How <qex>strangely</qex> active are the arts of peace!</q> <rj><qau>Dryden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>It would <qex>strangely</qex> delight you to see with what spirit he converses.</q> <rj><qau>Law.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Strangeness</ent><br/
<hw>Strange"ness</hw>, <pos>n.</pos> <def>The state or quality of being strange (in any sense of the adjective).</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stranger</ent><br/
<hw>Stran"ger</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[OF. <ets>estrangier</ets>, F. <ets><eacute/tranger</ets>. See <er>Strange</er>.]</ety> <sn>1.</sn> <def>One who is strange, foreign, or unknown.</def> Specifically: --<br/
[<source>1913 Webster</source>]</p>

<p><sd>(a)</sd> <def>One who comes from a foreign land; a foreigner.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>I am a most poor woman and a <qex>stranger</qex>,<br/
Born out of your dominions.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sd>(b)</sd> <def>One whose home is at a distance from the place where he is, but in the same country.</def><br/
[<source>1913 Webster</source>]</p>

<p><sd>(c)</sd> <def>One who is unknown or unacquainted; <as>as, the gentleman is a <ex>stranger</ex> to me</as>; hence, one not admitted to communication, fellowship, or acquaintance.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Melons on beds of ice are taught to bear,<br/
And <qex>strangers</qex> to the sun yet ripen here.</q> <rj><qau>Granville.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>My child is yet a <qex>stranger</qex> in the world.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>I was no <qex>stranger</qex> to the original.</q> <rj><qau>Dryden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>One not belonging to the family or household; a guest; a visitor.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>To honor and receive<br/
Our heavenly <qex>stranger</qex>.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <fld>(Law)</fld> <def>One not privy or party an act, contract, or title; a mere intruder or intermeddler; one who interferes without right; <as>as, actual possession of land gives a good title against a <ex>stranger</ex> having no title; as to <ex>strangers</ex>, a mortgage is considered merely as a pledge; a mere <ex>stranger</ex> to the levy</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stranger</ent><br/
<hw>Stran"ger</hw>, <pos>v. t.</pos> <def>To estrange; to alienate.</def> <mark>[Obs.]</mark>  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Strangle</ent><br/
<hw>Stran"gle</hw> <pr>(?)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Strangled</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Strangling</conjf> <pr>(?)</pr>.]</vmorph> <ety>[OF. <ets>estrangler</ets>, F. <ets><eacute/trangler</ets>, L. <ets>strangulare</ets>, Gr. <?/, <?/, fr. <?/ a halter; and perhaps akin to E. <ets>string</ets>, n.  Cf. <er>Strain</er>, <er>String</er>.]</ety> <sn>1.</sn> <def>To compress the windpipe of (a person or animal) until death results from stoppage of respiration; to choke to death by compressing the throat, as with the hand or a rope.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Our Saxon ancestors compelled the adulteress to <qex>strangle</qex> herself.</q> <rj><qau>Ayliffe.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To stifle, choke, or suffocate in any manner.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Shall I not then be stifled in the vault, . . . <br/
And there die <qex>strangled</qex> ere my Romeo comes?</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>To hinder from appearance; to stifle; to suppress.</def> <ldquo/<xex>Strangle</xex> such thoughts.<rdquo/  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Strangle</ent><br/
<hw>Stran"gle</hw>, <pos>v. i.</pos> <def>To be strangled, or suffocated.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Strangleable</ent><br/
<hw>Stran"gle*a*ble</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Capable of being strangled.</def> <mark>[R.]</mark>  <rj><au>Chesterfield.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Strangle hold</ent><br/
<hw>Stran"gle hold</hw>. <sn>1.</sn> <fld>(Wrestling)</fld> <def>a hold by which one's opponent is choked. It is usually not allowed.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><sn>2.</sn> <def>a forceful influence that strongly inhibits freedom of action; <as>as, the oil price jump put a <ex>stranglehold</ex> on the economy</as>.</def><br/
[<source>PJC</source>]</p>

<p><q>Zona believes that Microsoft's entry into this fray breaks what has been, by all appearances, Netscape's <qex>stranglehold</qex> on the Internet.  Microsoft is not licensing any technology from Netscape, and as part of this announcement, several alliances which historically would have been with Netscape were included. The impact of this announcement on Netscape was huge.</q><br/
[<source>PJC</source>]</p>

<p><q>SAN FRANCISCO, November 6, 1996 --  If Microsoft Corp. CEO Bill Gates were a plumber, he would control all the water in the world and force anyone who wanted some of the precious liquid to cut a deal with him.<br/
   With that statement, Oracle Corp. CEO Larry Ellison closed a question-and-answer session with reporters and analysts held here Tuesday at Oracle OpenWorld. During his hour-long appearance, Ellison agitated in favor of low-cost, generic network computers and against Windows-based PCs.<br/
   "The notion of one company controlling the whole [computer] industry is shocking and unacceptable," Ellison said, referring to what he describes as Microsoft's <qex>stranglehold</qex> on PCs through its ubiquitous Windows operating systems.</q><br/
[<source>PJC</source>]</p>

<p><ent>Strangler</ent><br/
<hw>Stran"gler</hw> <pr>(?)</pr>, <pos>n.</pos> <def>One who, or that which, strangles.</def> <ldquo/The very <xex>strangler</xex> of their amity.<rdquo/  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><-- p. 1423 --></p>

<p><ent>Strangles</ent><br/
<hw>Stran"gles</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A disease in horses and swine, in which the upper part of the throat, or groups of lymphatic glands elsewhere, swells.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Strangulate</ent><br/
<hw>Stran"gu*late</hw> <pr>(?)</pr>, <pos>a.</pos> <fld>(Bot.)</fld> <def>Strangulated.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Strangulated</ent><br/
<hw>Stran"gu*la`ted</hw> <pr>(?)</pr>, <pos>a.</pos> <sn>1.</sn> <fld>(Med.)</fld> <def>Having the circulation stopped by compression; attended with arrest or obstruction of circulation, caused by constriction or compression; <as>as, a <ex>strangulated</ex> hernia</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Bot.)</fld> <def>Contracted at irregular intervals, if tied with a ligature; constricted.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Strangulated hernia</b></col>. <fld>(Med.)</fld> <cd>See under <er>Hernia</er>.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Strangulation</ent><br/
<hw>Stran"gu*la`tion</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>strangulatio</ets>: cf. F. <ets>strangulation</ets>. See <er>Strangle</er>.]</ety> <sn>1.</sn> <def>The act of strangling, or the state of being strangled.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Med.)</fld> <def>Inordinate compression or constriction of a tube or part, as of the throat; especially, such as causes a suspension of breathing, of the passage of contents, or of the circulation, as in cases of hernia.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Strangurious</ent><br/
<hw>Stran*gu"ri*ous</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>stranguriosus</ets>.]</ety> <fld>(Med.)</fld> <def>Of or pertaining to strangury.</def>  <rj><au>Cheyne.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Strangury</ent><br/
<hw>Stran"gu*ry</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>stranguria</ets>, Gr. <?/; <?/, <?/, a drop + <?/ to make water, <?/ urine: cf. F. <ets>strangurie</ets>. See <er>Strangle</er>, and <er>Urine</er>.]</ety> <sn>1.</sn> <fld>(Med.)</fld> <def>A painful discharge of urine, drop by drop, produced by spasmodic muscular contraction.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Bot.)</fld> <def>A swelling or other disease in a plant, occasioned by a ligature fastened tightly about it.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Strany</ent><br/
<hw>Stra"ny</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Zool.)</fld> <def>The guillemot.</def> <mark>[Prov. Eng.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Strap</ent><br/
<hw>Strap</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[OE. <ets>strope</ets>, AS. <ets>stropp</ets>, L. <ets>stroppus</ets>, <ets>struppus</ets>, perhaps fr. Gr. <?/ a band or cord, fr. <?/ to twist, to turn (cf. <er>Strophe</er>).  Cf. <er>Strop</er> a strap, a piece of rope.]</ety> <sn>1.</sn> <def>A long, narrow, pliable strip of leather, cloth, or the like; specifically, a strip of thick leather used in flogging.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>A lively cobbler that . . . had scarce passed a day without giving her [his wife] the discipline of the <qex>strap</qex>.</q> <rj><qau>Addison.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Something made of such a strip, or of a part of one, or a combination of two or more for a particular use; <as>as, a boot <ex>strap</ex>, shawl <ex>strap</ex>, stirrup <ex>strap</ex></as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>A piece of leather, or strip of wood covered with a suitable material, for sharpening a razor; a strop.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>A narrow strip of anything, as of iron or brass.</def> Specifically: --<br/
[<source>1913 Webster</source>]</p>

<p><sd>(a)</sd> <fld>(Carp. & Mach.)</fld> <def>A band, plate, or loop of metal for clasping and holding timbers or parts of a machine.</def><br/
[<source>1913 Webster</source>]</p>

<p><sd>(b)</sd> <fld>(Naut.)</fld> <def>A piece of rope or metal passing around a block and used for fastening it to anything.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <fld>(Bot.)</fld> <sd>(a)</sd> <def>The flat part of the corolla in ligulate florets, as those of the white circle in the daisy.</def> <sd>(b)</sd> <def>The leaf, exclusive of its sheath, in some grasses.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>6.</sn> <def>A shoulder strap. See under <er>Shoulder</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Strap bolt</b></col>, <cd>a bolt of which one end is a flat bar of considerable length.</cd> -- <col><b>Strap head</b></col> <fld>(Mach.)</fld>, <cd>a journal box, or pair of brasses, secured to the end of a connecting rod by a strap. See <xex>Illust.</xex> of <cref>Gib and key</cref>, under <er>Gib</er>.</cd> -- <col><b>Strap hinge</b></col>, <cd>a hinge with long flaps by which it is fastened, as to a door or wall.</cd> -- <col><b>Strap rail</b></col> <fld>(Railroads)</fld>, <cd>a flat rail formerly used.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Strap</ent><br/
<hw>Strap</hw>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Strapped</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Strapping</conjf>.]</vmorph> <sn>1.</sn> <def>To beat or chastise with a strap.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To fasten or bind with a strap.</def>  <rj><au>Cowper.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>To sharpen by rubbing on a strap, or strop; <as>as, to <ex>strap</ex> a razor</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Strappado</ent><br/
<hw>Strap*pa"do</hw> <pr>(?)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Strappadoes</plw> <pr>(#)</pr>.</plu> <ety>[It. <ets>strappata</ets> a pull, the strappado, from <ets>strappare</ets> to pull, from Prov. G. <ets>strapfen</ets>: cf. G. <ets>straff</ets> tense, stretched.]</ety> <def>A military punishment formerly practiced, which consisted in drawing an offender to the top of a beam and letting him fall to the length of the rope, by which means a limb was often dislocated.</def>  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Strappado</ent><br/
<hw>Strap*pa"do</hw>, <pos>v. t.</pos> <def>To punish or torture by the strappado.</def>  <rj><au>Milton.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Strapper</ent><br/
<hw>Strap"per</hw> <pr>(?)</pr>, <pos>n.</pos> <sn>1.</sn> <def>One who uses strap.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A person or thing of uncommon size.</def> <mark>[Colloq.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Strapping</ent><br/
<hw>Strap"ping</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Tall; strong; lusty; large; <as>as, a <ex>strapping</ex> fellow</as>.</def> <mark>[Colloq.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>There are five and thirty <qex>strapping</qex> officers gone.</q> <rj><qau>Farquhar.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Strapple</ent><br/
<hw>Strap"ple</hw> <pr>(?)</pr>, <pos>v. t.</pos> <def>To hold or bind with, or as with, a strap; to entangle.</def> <mark>[Obs.]</mark>  <rj><au>Chapman.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Strap-shaped</ent><br/
<hw>Strap"-shaped`</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Shaped like a strap; ligulate; <as>as, a <ex>strap-shaped</ex> corolla</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Strapwork</ent><br/
<hw>Strap"work`</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Arch.)</fld> <def>A kind of ornament consisting of a narrow fillet or band folded, crossed, and interlaced.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Strass</ent><br/
<hw>Strass</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[So called from its inventor, a German jeweler: cf. F. <ets>stras</ets>.]</ety> <fld>(Chem.)</fld> <def>A brilliant glass, used in the manufacture of artificial paste gems, which consists essentially of a complex borosilicate of lead and potassium.  Cf. <er>Glass</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Strata</ent><br/
<hw>Stra"ta</hw> <pr>(?)</pr>, <pos>n.</pos>, <def><pos>pl.</pos> of <er>Stratum</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stratagem</ent><br/
<hw>Strat"a*gem</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F. <ets>stratag<egrave/me</ets> (cf. Sp. <ets>estratagema</ets>, It. <ets>stratagemma</ets>), L. <ets>strategema</ets>, Gr. <?/, fr. <?/ to be leader of an army, fr. <?/ a general; <?/ an army (probably as being spread out; cf. <er>Stratum</er>) + <?/ to lead. See <er>Agent</er>.]</ety> <def>An artifice or trick in war for deceiving the enemy; hence, in general, artifice; deceptive device; secret plot; evil machination.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Fit for treasons, <qex>stratagems</qex>, and spoils.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Those oft are <qex>stratagems</qex> which error seem,<br/
Nor is it Homer nods, but we that dream.</q> <rj><qau>Pope.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stratagemical</ent><br/
<hw>Strat`a*gem"ic*al</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Containing stratagem; <as>as, a <ex>stratagemical</ex> epistle</as>.</def> <mark>[R.]</mark>  <rj><au>Swift.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stratarithmetry</ent><br/
<hw>Strat`a*rith"me*try</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Gr. <?/ army + <?/ number + <ets>-metry</ets>.]</ety> <fld>(Mil.)</fld> <def>The art of drawing up an army, or any given number of men, in any geometrical figure, or of estimating or expressing the number of men in such a figure.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Strategetical</ent><br/
<ent>Strategetic</ent><br/
<mhw>{ <hw>Strat`e*get"ic</hw> <pr>(?)</pr>, <hw>Strat`e*get"ic*al</hw> <pr>(?)</pr>, }</mhw> <pos>a.</pos> <def>Strategic.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Strategetics</ent><br/
<hw>Strat`e*get"ics</hw> <pr>(?)</pr>, <pos>n.</pos> <def>Strategy.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Strategical</ent><br/
<ent>Strategic</ent><br/
<mhw>{ <hw>Stra*te"gic</hw> <pr>(?)</pr>, <hw>Stra*te"gic*al</hw>, }</mhw> <pos>a.</pos> <ety>[Gr. <?/ of or for a general: cf. F. <ets>strat<eacute/gique</ets>.]</ety> <def>Of or pertaining to strategy; effected by artifice.</def> -- <wordforms><wf>Stra*te"gic*al*ly</wf>, <pos>adv.</pos></wordforms><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Strategic line</b></col> <fld>(Mil.)</fld>, <cd>a line joining strategic points.</cd> -- <col><b>Strategic point</b></col> <fld>(Mil.)</fld>, <cd>any point or region in the theater or warlike operations which affords to its possessor an advantage over his opponent, as a mountain pass, a junction of rivers or roads, a fortress, etc.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Strategics</ent><br/
<hw>Stra*te"gics</hw> <pr>(?)</pr>, <pos>n.</pos> <def>Strategy.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Strategist</ent><br/
<hw>Strat"e*gist</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Cf. F. <ets>strat<eacute/giste</ets>.]</ety> <def>One skilled in strategy, or the science of directing great military movements.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Strategus</ent><br/
||<hw>Stra*te"gus</hw> <pr>(?)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Strategi</plw> <pr>(#)</pr>.</plu> <ety>[L., fr. Gr. <?/. See <er>Stratagem</er>.]</ety> <fld>(Gr. Antiq.)</fld> <def>The leader or commander of an army; a general.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Strategy</ent><br/
<hw>Strat"e*gy</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Gr. <?/: cf. F. <ets>strat<eacute/gie</ets>. See <er>Stratagem</er>.]</ety> <sn>1.</sn> <def>The science of military command, or the science of projecting campaigns and directing great military movements; generalship.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>The use of stratagem or artifice.</def><br/
[<source>1913 Webster</source>]</p>

<p><-- 3. a plan of action encompassing the methods to be adopted from beginning to end of a task or endeavor, focussing on the general methods; -- contrasted with tactics, which is a plan for accomplishing subgoals of lesser extent than the primary goal.  Thus, a strategy is a plan for winning a war, and a tactic is a plan for winning a battle.
   4. Biol.  A behavior evolved and exhibited by a living organism to accomplish some important goal, as a foraging strategy. --><br/
[<source>1913 Webster</source>]</p>

<p><ent>Strath</ent><br/
<hw>Strath</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Gael. <ets>srath</ets>.]</ety> <def>A valley of considerable size, through which a river runs; a valley bottom; -- often used in composition with the name of the river; <as>as, <ex>Strath</ex> Spey, <ex>Strath</ex>don, <ex>Strath</ex>more</as>.</def> <mark>[Scot.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>The long green <qex>strath</qex> of Napa valley.</q> <rj><qau>R. L. Stevenson.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Strathspey</ent><br/
<hw>Strath"spey`</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[So called from the district of <ets>Strath Spey</ets> in Scotland.]</ety> <def>A lively Scottish dance, resembling the reel, but slower; also, the tune.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Straticulate</ent><br/
<hw>Stra*tic"u*late</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[Dim. Fr. <ets>stratum</ets>.]</ety> <fld>(Min.)</fld> <def>Characterized by the presence of thin parallel strata, or layers, as in an agate.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stratification</ent><br/
<hw>Strat`i*fi*ca"tion</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Cf. F. <ets>stratification</ets>.]</ety> <sn>1.</sn> <def>The act or process of laying in strata, or the state of being laid in the form of strata, or layers.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Physiol.)</fld> <def>The deposition of material in successive layers in the growth of a cell wall, thus giving rise to a stratified appearance.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stratified</ent><br/
<hw>Strat"i*fied</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Having its substance arranged in strata, or layers; <as>as, <ex>stratified</ex> rock</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stratiform</ent><br/
<hw>Strat"i*form</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Having the form of strata.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stratify</ent><br/
<hw>Strat"i*fy</hw> <pr>(?)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Stratified</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Stratifying</conjf> <pr>(?)</pr>.]</vmorph> <ety>[<ets>Stratum</ets> + <ets>-fy</ets>: cf. F. <ets>stratifier</ets>.]</ety> <def>To form or deposit in strata, or layers, as substances in the earth; to arrange in strata.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stratigraphical</ent><br/
<ent>Stratigraphic</ent><br/
<mhw>{ <hw>Strat`i*graph"ic</hw> <pr>(?)</pr>, <hw>Strat`i*graph"ic*al</hw> <pr>(?)</pr>, }</mhw> <pos>a.</pos> <fld>(Geol.)</fld> <def>Pertaining to, or depended upon, the order or arrangement of strata; <as>as, <ex>stratigraphical</ex> evidence</as>.</def> -- <wordforms><wf>Strat`i*graph"ic*al*ly</wf>, <pos>adv.</pos></wordforms><br/
[<source>1913 Webster</source>]</p>

<p><ent>-ical</ent><br/
<ent>Stratigraphic</ent><br/
<mhw>{ <hw>Strat`i*graph"ic</hw>, <hw>-ic*al</hw>  }</mhw>, <pos>a.</pos> <fld>(Mil.)</fld> <def>See <er>Stratographic</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stratigraphy</ent><br/
<hw>Stra*tig"ra*phy</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[<ets>Stratum</ets> + <ets>-graphy</ets>.]</ety> <def>That branch of geology which treats of the arrangement and succession of strata.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Strato-cirrus</ent><br/
<hw>Stra`to-cir"rus</hw> <pr>(?)</pr>, <pos>n.</pos>  <ety>[<ets>Stratus</ets> + <ets>cirrus</ets>.]</ety> <fld>(Meteor.)</fld> <def>An alto-stratus cloud.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Stratocracy</ent><br/
<hw>Stra*toc"ra*cy</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Gr. <?/ an army + <ets>-cracy</ets>, as in demo<ets>cracy</ets>: cf. F. <ets>stratocratie</ets>.]</ety> <def>A military government; government by military chiefs and an army.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Strato-cumulus</ent><br/
<hw>Stra`to-cu"mu*lus</hw> <pr>(?)</pr>, <pos>n.</pos>  <ety>[<ets>Stratus</ets> + <ets>cumulus</ets>.]</ety> <fld>(Meteor.)</fld> <def>Large balls or rolls of dark cloud which frequently cover the whole sky, esp. in winter, and give it at times an undulated appearance.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Stratographical</ent><br/
<ent>Stratographic</ent><br/
<mhw>{ <hw>Strat`o*graph"ic</hw> <pr>(?)</pr>, <hw>Strat`o*graph"ic*al</hw> <pr>(?)</pr>, }</mhw> <pos>a.</pos> <def>Of or pertaining to stratography.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stratography</ent><br/
<hw>Stra*tog"ra*phy</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Gr. <?/ an army + <ets>-graphy</ets>.]</ety> <def>A description of an army, or of what belongs to an army.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stratonic</ent><br/
<hw>Stra*ton"ic</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[Gr. <?/ an army.]</ety> <def>Of or pertaining to an army.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stratotic</ent><br/
<hw>Stra*tot"ic</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Warlike; military.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stratum</ent><br/
<hw>Stra"tum</hw> <pr>(?)</pr>, <pos>n.</pos>; <plu><it>pl.</it> E. <plw>Stratums</plw> <pr>(#)</pr>, L. <plw>Strata</plw> <pr>(#)</pr>. The latter is more common.</plu> <ety>[L., from <ets>sternere</ets>, <ets>stratum</ets>, to spread; akin to Gr. <?/ to spread, strew. See <er>Strew</er>, and cf. <er>Consternation</er>, <er>Estrade</er>, <er>Prostrate</er>, <er>Stratus</er>, <er>Street</er>.]</ety> <sn>1.</sn> <fld>(Geol.)</fld> <def>A bed of earth or rock of one kind, formed by natural causes, and consisting usually of a series of layers, which form a rock as it lies between beds of other kinds. Also used figuratively.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A bed or layer artificially made; a course.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stratus</ent><br/
<hw>Stra"tus</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>stratus</ets> a spreading out, scattering, from <ets>sternere</ets>, <ets>stratum</ets>, to spread.]</ety> <fld>(Meteor.)</fld> <def>A form of clouds in which they are arranged in a horizontal band or layer. See <er>Cloud</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Straught</ent><br/
<hw>Straught</hw> <pr>(?)</pr>, <mark>obs.</mark> <def><pos>imp. & p. p.</pos> of <er>Stretch</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Straught</ent><br/
<hw>Straught</hw>, <pos>v. t.</pos> <def>To stretch; to make straight.</def> <altsp>[Written also <asp>straucht</asp>.]</altsp> <mark>[Scot.]</mark>  <rj><au>Sir W. Scott.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Straw</ent><br/
<hw>Straw</hw> <pr>(?)</pr>, <pos>v. t.</pos> <def>To spread or scatter. See <er>Strew</er>, and <er>Strow</er>.</def>  <rj><au>Chaucer.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Straw</ent><br/
<hw>Straw</hw>, <pos>n.</pos> <ety>[OE. <ets>straw</ets>, <ets>stre</ets>, <ets>stree</ets>, AS. <ets>stre<aacute/w</ets>, from the root of E. <ets>strew</ets>; akin to OFries. <ets>str<emac/</ets>, D. <ets>stroo</ets>, G. <ets>stroh</ets>, OHG. <ets>str<omac/</ets>, Icel. <ets>str<amac/</ets>, Dan. <ets>straa</ets>, Sw. <ets>str<aring/</ets>. <root/166. See <er>Strew</er>.]</ety> <sn>1.</sn> <def>A stalk or stem of certain species of grain, pulse, etc., especially of wheat, rye, oats, barley, more rarely of buckwheat, beans, and pease.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>The gathered and thrashed stalks of certain species of grain, etc.; <as>as, a bundle, or a load, of rye <ex>straw</ex></as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>Anything proverbially worthless; the least possible thing; a mere trifle.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>I set not a <qex>straw</qex> by thy dreamings.</q> <rj><qau>Chaucer.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ <xex>Straw</xex> is often used in the formation of self-explaining compounds; as, <xex>straw</xex>-built, <xex>straw</xex>-crowned, <xex>straw</xex>-roofed, <xex>straw</xex>-stuffed, and the like.</note><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Man of straw</b></col>, <cd>an effigy formed by stuffing the garments of a man with straw; hence, a fictitious person; an irresponsible person; a puppet.</cd><-- now usu. straw man, as in <it>set up a straw man</it>; -- used in disputation.  Typically, one party accuses an opponent of <it>setting up a straw man</it>, meaning that the opponent is distorting his true opinion in order to make it look absurd. --> -- <col><b>Straw bail</b></col>, <cd>worthless bail, as being given by irresponsible persons.</cd> <mark>[Colloq. U.S.]</mark> -- <col><b>Straw bid</b></col>, <cd>a worthless bid; a bid for a contract which the bidder is unable or unwilling to fulfill.</cd> <mark>[Colloq. U.S.]</mark> -- <col><b>Straw cat</b></col> <fld>(Zool.)</fld>, <cd>the pampas cat.</cd> -- <col><b>Straw color</b></col>, <cd>the color of dry straw, being a delicate yellow.</cd> -- <col><b>Straw drain</b></col>, <cd>a drain filled with straw.</cd> -- <mcol><col><b>Straw plait</b></col>, <it>or</it> <col><b>Straw plat</b></col></mcol>, <cd>a strip formed by plaiting straws, used for making hats, bonnets, etc.</cd> -- <col><b>To be in the straw</b></col>, <cd>to be brought to bed, as a pregnant woman.</cd> <mark>[Slang]</mark><-- archaic? Similar is "a roll in the hay" --></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Strawberry</ent><br/
<hw>Straw"ber*ry</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[AS. <ets>stre<aacute/wberige</ets>; <ets>stre<aacute/w</ets> straw + <ets>berie</ets> berry; perhaps from the resemblance of the runners of the plant to straws.]</ety> <fld>(Bot.)</fld> <def>A fragrant edible berry, of a delicious taste and commonly of a red color, the fruit of a plant of the genus <gen>Fragaria</gen>, of which there are many varieties. Also, the plant bearing the fruit. The common American strawberry is <spn>Fragaria virginiana</spn>; the European, <spn>Fragaria vesca</spn>.  There are also other less common species.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Strawberry bass</b></col>. <fld>(Zool.)</fld> <cd>See <cref>Calico bass</cref>, under <er>Calico</er>.</cd> -- <col><b>Strawberry blite</b></col>. <fld>(Bot.)</fld> <cd>See under <er>Blite</er>.</cd> -- <col><b>Strawberry borer</b></col> <fld>(Zool.)</fld>, <cd>any one of several species of insects whose larvae burrow in the crown or roots of the strawberry vine.</cd> Especially: <sd>(a)</sd> <cd>The root borer (<spn>Anarsia lineatella</spn>), a very small dark gray moth whose larvae burrow both in the larger roots and crown, often doing great damage.</cd> <sd>(b)</sd> <cd>The crown borer (<spn>Tyloderma fragariae</spn>), a small brown weevil whose larva burrows in the crown and kills the plant.</cd> -- <col><b>Strawberry bush</b></col> <fld>(Bot.)</fld>, <cd>an American shrub (<spn>Euonymus Americanus</spn>), a kind of spindle tree having crimson pods and the seeds covered with a scarlet aril.</cd> -- <col><b>Strawberry crab</b></col> <fld>(Zool.)</fld>, <cd>a small European spider crab (<spn>Eurynome aspera</spn>); -- so called because the back is covered with pink tubercles.</cd> -- <col><b>Strawberry fish</b></col> <fld>(Zool.)</fld>, <cd>the amadavat.</cd> -- <col><b>Strawberry geranium</b></col> <fld>(Bot.)</fld>, <cd>a kind of saxifrage (<spn>Saxifraga sarmentosa</spn>) having reniform leaves, and producing long runners like those of the strawberry.</cd> -- <col><b>Strawberry leaf</b></col>. <sd>(a)</sd> <cd>The leaf of the strawberry</cd>. <sd>(b)</sd> <cd>The symbol of the rank or estate of a duke, because the ducal coronet is twined with strawberry leaves</cd>. <ldquo/The <xex>strawberry leaves</xex> on her chariot panels are engraved on her ladyship's heart.<rdquo/ <au>Thackeray.</au> -- <col><b>Strawberry-leaf roller</b></col> <fld>(Zool.)</fld>, <cd>any one of several species of moths whose larvae roll up, and feed upon, the leaves of the strawberry vine; especially, <spn>Phoxopteris fragariae</spn>, and <spn>Eccopsis permundana</spn>.</cd> -- <col><b>Strawberry moth</b></col> <fld>(Zool.)</fld>, <cd>any one of several species of moth whose larvae feed on the strawberry vines</cd>; as: <sd>(a)</sd> <cd>The smeared dagger (<spn>Apatela oblinita</spn>), whose large hairy larva is velvety black with two rows of bright yellow spots on each side.</cd> <sd>(b)</sd> <cd>A geometrid (<spn>Angerona crocataria</spn>) which is yellow with dusky spots on the wings. Called also <altname>currant moth</altname>.</cd> -- <col><b>Strawberry pear</b></col> <fld>(Bot.)</fld>, <cd>the red ovoid fruit of a West Indian plant of the genus <gen>Cereus</gen> (<spn>Cereus triangularia</spn>).  It has a sweetish flavor, and is slightly acid, pleasant, and cooling.  Also, the plant bearing the fruit.</cd> -- <col><b>Strawberry sawfly</b></col> <fld>(Zool.)</fld>, <cd>a small black sawfly (<spn>Emphytus maculatus</spn>) whose larva eats the leaves of the strawberry vine.</cd> -- <col><b>Strawberry tomato</b></col>. <fld>(Bot.)</fld> <cd>See <er>Alkekengi</er>.</cd> -- <col><b>Strawberry tree</b></col>. <fld>(Bot.)</fld> <cd>See <er>Arbutus</er>.</cd> -- <col><b>Strawberry vine</b></col> <fld>(Bot.)</fld>, <cd>the plant which yields the strawberry.</cd> -- <col><b>Strawberry worm</b></col> <fld>(Zool.)</fld>, <cd>the larva of any moth which feeds on the strawberry vine.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Strawboard</ent><br/
<hw>Straw"board`</hw> <pr>(?)</pr>, <pos>n.</pos> <def>Pasteboard made of pulp of straw.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Straw-colored</ent><br/
<hw>Straw"-col`ored</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Being of a straw color. See <cref>Straw color</cref>, under <er>Straw</er>, <pos>n.</pos></def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Straw-cutter</ent><br/
<hw>Straw"-cut`ter</hw> <pr>(?)</pr>, <pos>n.</pos> <def>An instrument to cut straw for fodder.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Strawed</ent><br/
<hw>Strawed</hw> <pr>(?)</pr>, <def><pos>imp. & p. p.</pos> of <er>Straw</er>.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Strawworm</ent><br/
<hw>Straw"worm`</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A caddice worm.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Strawy</ent><br/
<hw>Straw"y</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Of or pertaining to straw; made of, or resembling, straw.</def>  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stray</ent><br/
<hw>Stray</hw> <pr>(?)</pr>, <pos>v. i.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Strayed</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Straying</conjf>.]</vmorph> <ety>[OF. <ets>estraier</ets>, <ets>estraer</ets>, to stray, or as adj., stray, fr. (assumed) L. <ets>stratarius</ets> roving the streets, fr. L. <ets>strata</ets> (sc. <ets>via</ets>) a paved road. See <er>Street</er>, and <er>Stray</er>, <pos>a.</pos>]</ety> <sn>1.</sn> <def>To wander, as from a direct course; to deviate, or go out of the way.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Thames among the wanton valleys <qex>strays</qex>.</q> <rj><qau>Denham.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To wander from company, or from the proper limits; to rove at large; to roam; to go astray.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Now, until the break of day,<br/
Through this house each fairy <qex>stray</qex>.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>A sheep doth very often <qex>stray</qex>.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>Figuratively, to wander from the path of duty or rectitude; to err.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>We have erred and <qex>strayed</qex> from thy ways.</q> <rj><qau><?/<?/<?/ of Com. Prayer.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>While meaner things, whom instinct leads,<br/
Are rarely known to <qex>stray</qex>.</q> <rj><qau>Cowper.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><syn><b>Syn.</b> -- To deviate; err; swerve; rove; roam; wander.</syn><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stray</ent><br/
<hw>Stray</hw>, <pos>v. t.</pos> <def>To cause to stray.</def> <mark>[Obs.]</mark>  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stray</ent><br/
<hw>Stray</hw>, <pos>a.</pos> <ety>[Cf. OF. <ets>estrai<eacute/</ets>, p. p. of <ets>estraier</ets>. See <er>Stray</er>, <pos>v. i.</pos>, and cf. <er>Astray</er>, <er>Estray</er>.]</ety> <def>Having gone astray; strayed; wandering; <as>as, a <ex>stray</ex>horse or sheep</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Stray line</b></col> <fld>(Naut.)</fld>, <cd>that portion of the log line which is veered from the reel to allow the chip to get clear of the stern eddies before the glass is turned.</cd> -- <col><b>Stray mark</b></col> <fld>(Naut.)</fld>, <cd>the mark indicating the end of the stray line.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stray</ent><br/
<hw>Stray</hw>, <pos>n.</pos> <sn>1.</sn> <def>Any domestic animal that has an inclosure, or its proper place and company, and wanders at large, or is lost; an estray.  Used also figuratively.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Seeing him wander about, I took him up for a <qex>stray</qex>.</q> <rj><qau>Dryden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>The act of wandering or going astray.</def> <mark>[R.]</mark>  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><-- p. 1424 --></p>

<p><ent>Strayer</ent><br/
<hw>Stray"er</hw> <pr>(?)</pr>, <pos>n.</pos> <def>One who strays; a wanderer.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stre</ent><br/
<hw>Stre</hw> <pr>(?)</pr>, <pos>n.</pos> <def>Straw.</def> <mark>[Obs.]</mark>  <rj><au>Chaucer.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Streak</ent><br/
<hw>Streak</hw> <pr>(?)</pr>, <pos>v. t.</pos> <ety>[Cf. <er>Stretch</er>, <er>Streek</er>.]</ety> <def>To stretch; to extend; hence, to lay out, as a dead body.</def> <mark>[Obs. or Prov. Eng. & Scot.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Streak</ent><br/
<hw>Streak</hw>, <pos>n.</pos> <ety>[OE. <ets>streke</ets>; akin to D. <ets>streek</ets> a line, stroke, G. <ets>strich</ets>, AS. <ets>strica</ets>, Sw. <ets>strek</ets>, Dan. <ets>streg</ets>, Goth. <ets>stricks</ets>, and E. <ets>strike</ets>, <ets>stroke</ets>. See <er>Strike</er>, <er>Stroke</er>, <pos>n.</pos>, and cf. <er>Strake</er>.]</ety> <sn>1.</sn> <def>A line or long mark of a different color from the ground; a stripe; a vein.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>What mean those colored <qex>streaks</qex> in heaven?</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Shipbuilding)</fld> <def>A strake.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <fld>(Min.)</fld> <def>The fine powder or mark yielded by a mineral when scratched or rubbed against a harder surface, the color of which is sometimes a distinguishing character.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>The rung or round of a ladder.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Streak</ent><br/
<hw>Streak</hw>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Streaked</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Streaking</conjf>.]</vmorph> <sn>1.</sn> <def>To form streaks or stripes in or on; to stripe; to variegate with lines of a different color, or of different colors.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>A mule . . . <qex>streaked</qex> and dappled with white and black.</q> <rj><qau>Sandys.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Now <qex>streaked</qex> and glowing with the morning red.</q> <rj><qau>Prior.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>With <xex>it</xex> as an object: To run swiftly.</def> <mark>[Colloq.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Streaked</ent><br/
<hw>Streaked</hw> <pr>(?)</pr>, <pos>a.</pos> <sn>1.</sn> <def>Marked or variegated with stripes.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Uncomfortable; out of sorts.</def> <mark>[Local, U.S.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Streaky</ent><br/
<hw>Streak"y</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Same as <er>Streaked</er>, 1.</def> <ldquo/The <xex>streaky</xex> west.<rdquo/  <rj><au>Cowper.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stream</ent><br/
<hw>Stream</hw> <pr>(str<emac/m)</pr>, <pos>n.</pos> <ety>[AS. <ets>stre<aacute/m</ets>; akin to OFries. <ets>str<amac/m</ets>, OS. <ets>str<omac/m</ets>, D. <ets>stroom</ets>, G. <ets>strom</ets>, OHG. <ets>stroum</ets>, <ets>str<umac/m</ets>, Dan. & Sw. <ets>str<oum/m</ets>, Icel. <ets>straumr</ets>, Ir. <ets>sroth</ets>, Lith. <ets>srove</ets>, Russ. <ets>struia</ets>, Gr. <grk>"ry`sis</grk> a flowing, <grk>"rei^n</grk> to flow, Skr. <ets>sru</ets>. <root/174.  Cf. <er>Catarrh</er>, <er>Diarrhea</er>, <er>Rheum</er>, <er>Rhythm</er>.]</ety> <sn>1.</sn> <def>A current of water or other fluid; a liquid flowing continuously in a line or course, either on the earth, as a river, brook, etc., or from a vessel, reservoir, or fountain; specifically, any course of running water; <as>as, many <ex>streams</ex> are blended in the Mississippi; gas and steam came from the earth in <ex>streams</ex>; a <ex>stream</ex> of molten lead from a furnace; a <ex>stream</ex> of lava from a volcano.</as></def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A beam or ray of light.</def> <ldquo/Sun <xex>streams</xex>.<rdquo/  <rj><au>Chaucer.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>Anything issuing or moving with continued succession of parts; <as>as, a <ex>stream</ex> of words; a <ex>stream</ex> of sand</as>.</def> <ldquo/The <xex>stream</xex> of beneficence.<rdquo/ <au>Atterbury.</au> <ldquo/The <xex>stream</xex> of emigration.<rdquo/ <au>Macaulay.</au><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>A continued current or course; <as>as, a <ex>stream</ex> of weather</as>.</def> <ldquo/The very <xex>stream</xex> of his life.<rdquo/  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <def>Current; drift; tendency; series of tending or moving causes; <as>as, the <ex>stream</ex> of opinions or manners</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Gulf stream</b></col>. <cd>See under <er>Gulf</er>.</cd> -- <mcol><col><b>Stream anchor</b></col>, <col><b>Stream cable</b></col></mcol>. <fld>(Naut.)</fld> <cd>See under <er>Anchor</er>, and <er>Cable</er>.</cd> -- <col><b>Stream ice</b></col>, <cd>blocks of ice floating in a mass together in some definite direction.</cd> -- <col><b>Stream tin</b></col>, <cd>particles or masses of tin ore found in alluvial ground; -- so called because a <xex>stream</xex> of water is the principal agent used in separating the ore from the sand and gravel.</cd> -- <col><b>Stream works</b></col> <fld>(Cornish Mining)</fld>, <cd>a place where an alluvial deposit of tin ore is worked.</cd> <au>Ure.</au> -- <col><b>To float with the stream</b></col>, <cd>figuratively, to drift with the current of opinion, custom, etc., so as not to oppose or check it.</cd>
<-- Colloq. = go with the flow, blow with the wind. --></cs><br/
[<source>1913 Webster</source>]</p>

<p><syn><b>Syn.</b> -- Current; flow; rush; tide; course.</syn>  -- <usage><er>Stream</er>, <er>Current</er>. These words are often properly interchangeable; but <xex>stream</xex> is the broader word, denoting a prevailing onward course. The <xex>stream</xex> of the Mississippi rolls steadily on to the Gulf of Mexico, but there are reflex <xex>currents</xex> in it which run for a while in a contrary direction.</usage><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stream</ent><br/
<hw>Stream</hw>, <pos>v. i.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Streamed</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Streaming</conjf>.]</vmorph> <sn>1.</sn> <def>To issue or flow in a stream; to flow freely or in a current, as a fluid or whatever is likened to fluids; <as>as, tears <ex>streamed</ex> from her eyes</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Beneath those banks where rivers <qex>stream</qex>.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To pour out, or emit, a stream or streams.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>A thousand suns will <qex>stream</qex> on thee.</q> <rj><qau>Tennyson.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>To issue in a stream of light; to radiate.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>To extend; to stretch out with a wavy motion; to float in the wind; <as>as, a flag <ex>streams</ex> in the wind</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stream</ent><br/
<hw>Stream</hw>, <pos>v. t.</pos> <def>To send forth in a current or stream; to cause to flow; to pour; <as>as, his eyes <ex>streamed</ex> tears</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>It may so please that she at length will <qex>stream</qex><br/
Some dew of grace into my withered heart.</q> <rj><qau>Spenser.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To mark with colors or embroidery in long tracts.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The herald's mantle is <qex>streamed</qex> with gold.</q> <rj><qau>Bacon.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>To unfurl.</def>  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>To stream the buoy</b></col>. <fld>(Naut.)</fld> <cd>See under <er>Buoy</er>.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stream clock</ent><br/
<hw>Stream clock</hw>. <fld>(Physiol.)</fld> <def>An instrument for ascertaining the velocity of the blood in a vessel.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Streamer</ent><br/
<hw>Stream"er</hw> <pr>(?)</pr>, <pos>n.</pos> <sn>1.</sn> <def>An ensign, flag, or pennant, which floats in the wind; specifically, a long, narrow, ribbonlike flag.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Brave Rupert from afar appears,<br/
Whose waving <qex>streamers</qex> the glad general knows.</q> <rj><qau>Dryden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>A stream or column of light shooting upward from the horizon, constituting one of the forms of the aurora borealis.</def>  <rj><au>Macaulay.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>While overhead the North's dumb <qex>streamers</qex> shoot.</q> <rj><qau>Lowell.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <fld>(Mining)</fld> <def>A searcher for stream tin.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <fld>(Journalism)</fld> <def>a banner.</def><br/
[<source>PJC</source>]</p>

<p><ent>Streamful</ent><br/
<hw>Stream"ful</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Abounding in streams, or in water.</def> <ldquo/The <xex>streamful</xex> tide.<rdquo/  <rj><au>Drayton.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stream gold</ent><br/
<hw>Stream gold</hw>. <fld>(Mining)</fld> <def>Gold in alluvial deposits; placer gold.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Streaminess</ent><br/
<hw>Stream"i*ness</hw> <pr>(?)</pr>, <pos>n.</pos> <def>The state of being streamy; a trailing.</def>  <rj><au>R. A. Proctor.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Streaming</ent><br/
<hw>Stream"ing</hw>, <pos>a.</pos> <def>Sending forth streams.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Streaming</ent><br/
<hw>Stream"ing</hw>, <pos>n.</pos> <sn>1.</sn> <def>The act or operation of that which streams; the act of that which sends forth, or which runs in, streams.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Mining)</fld> <def>The reduction of stream tin; also, the search for stream tin.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <fld>(Biol.)</fld> <def>the rapid flow of cytoplasm within a cell; -- called also <altname>protoplasmic streaming</altname>.</def><br/
[<source>PJC</source>]</p>

<p><ent>Streamless</ent><br/
<hw>Stream"less</hw>, <pos>a.</pos> <def>Destitute of streams, or of a stream, as a region of country, or a dry channel.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Streamlet</ent><br/
<hw>Stream"let</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A small stream; a rivulet; a rill.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stream line</ent><br/
<hw>Stream line</hw>. <def>The path of a constituent particle of a flowing fluid undisturbed by eddies or the like.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Streamline</ent><br/
<hw>Stream"line`</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Of or pert. to a stream line; designating a motion or flow that is free from turbulence, like that of a particle in a streamline; hence, designating a surface, body, etc., that is designed so as to afford an unbroken flow of a fluid about it, esp. when the resistance to flow is the least possible; <as>as, a <ex>streamline</ex> body for an automobile or airship</as>; -- the current usuage prefers the term <altname>streamlined</altname>.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>streamline</ent><br/
<hw>stream"line</hw>, <pos>v. t.</pos> <sn>1.</sn> <def>to design or modify so as to present the least possible resistance to fluid flow; -- used mostly of vehicles, such as automobiles, airplanes, or ships.</def><br/
[<source>PJC</source>]</p>

<p><sn>2.</sn> <mark>[Fig.]</mark> <def>to change so as to make more efficient; -- used especially of organizations, procedures, or methods.</def> <ldquo/The <ex>streamlined</ex> company became a formidable competitor.<rdquo/<br/
[<source>PJC</source>]</p>

<p><ent>Stream wheel</ent><br/
<hw>Stream wheel</hw>. <def>A wheel used for measuring, by its motion when submerged, the velocity of flowing water; a current wheel.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Streamy</ent><br/
<hw>Stream"y</hw> <pr>(?)</pr>, <pos>a.</pos> <sn>1.</sn> <def>Abounding with streams, or with running water; streamful.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Arcadia<br/
However <qex>streamy</qex> now, adust and dry,<br/
Denied the goddess water.</q> <rj><qau>Prior.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Resembling a stream; issuing in a stream.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>His nodding helm emits a <qex>streamy</qex> ray.</q> <rj><qau>Pope.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stree</ent><br/
<hw>Stree</hw> <pr>(?)</pr>, <pos>n.</pos> <def>Straw.</def> <mark>[Obs.]</mark>  <rj><au>Chaucer.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Streek</ent><br/
<hw>Streek</hw> <pr>(?)</pr>, <pos>v. t.</pos> <def>To stretch; also, to lay out, as a dead body. See <er>Streak</er>.</def> <mark>[Obs. or Prov. Eng. & Scot.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Streel</ent><br/
<hw>Streel</hw> <pr>(?)</pr>, <pos>v. i.</pos> <ety>[Cf. <er>Stroll</er>.]</ety> <def>To trail along; to saunter or be drawn along, carelessly, swaying in a kind of zigzag motion.</def> <mark>[Colloq.]</mark>  <rj><au>Thackeray.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Streen</ent><br/
<hw>Streen</hw> <pr>(str<emac/n)</pr>, <pos>n.</pos> <def>See <er>Strene</er>.</def> <mark>[Obs.]</mark>  <rj><au>Chaucer.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Street</ent><br/
<hw>Street</hw> <pr>(str<emac/t)</pr>, <pos>n.</pos> <ety>[OE. <ets>strete</ets>, AS. <ets>str<aemac/t</ets>, fr. L. <ets>strata</ets> (sc. <ets>via</ets>) a paved way, properly fem. p. p. of <ets>sternere</ets>, <ets>stratum</ets>, to spread; akin to E. <ets>strew</ets>. See <er>Strew</er>, and cf. <er>Stratum</er>, <er>Stray</er>, <pos>v.</pos> & <pos>a.</pos>]</ety> <sn>1.</sn> <def>Originally, a paved way or road; a public highway; now commonly, a thoroughfare in a city or village, bordered by dwellings or business houses.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>He removed [the body of] Amasa from the <qex>street</qex> unto the field.</q> <rj><qau>Coverdale.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>At home or through the high <qex>street</qex> passing.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ In an extended sense, <xex>street</xex> designates besides the roadway, the walks, houses, shops, etc., which border the thoroughfare.</note><br/
[<source>1913 Webster</source>]</p>

<p><q>His deserted mansion in Duke <qex>Street</qex>.</q> <rj><qau>Macaulay.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>the roadway of a street{1}, as distinguished from the sidewalk; <as>as, children playing in the <ex>street</ex></as>.</def><br/
[<source>PJC</source>]</p>

<p><sn>3.</sn> <def>the inhabitants of a particular street; <as>as, the whole <ex>street</ex> knew about their impending divorce</as>.</def><br/
[<source>PJC</source>]</p>

<p><cs><col><b>The street</b></col> <fld>(Broker's Cant)</fld>, <cd>that thoroughfare of a city where the leading bankers and brokers do business; also, figuratively, those who do business there; <as>as, <ex>the street</ex> would not take the bonds</as>.</cd> -- <col><b>on the street</b></col>, <sd>(a)</sd> <cd>homeless.</cd> <sd>(b)</sd> <cd>unemployed.</cd> <sd>(a)</sd> <cd>not in prison, or released from prison; <as>the murderer is still <ex>on the street</ex></as>.</cd> --<mcol><col><b>Street Arab</b></col>, <col><b>Street broker</b></col></mcol>, <cd>etc. See under <er>Arab</er>, <er>Broker</er>, etc.</cd> -- <col><b>Street door</b></col>, <cd>a door which opens upon a street, or is nearest the street.</cd> -- <col><b>street person</b></col>, <cd>a homeless person; a vagrant.</cd></cs><br/
[<source>1913 Webster</source> <source>+PJC</source>]</p>

<p><syn><b>Syn.</b> -- See <er>Way</er>.</syn><br/
[<source>1913 Webster</source>]</p>

<p><ent>Streetwalker</ent><br/
<hw>Street"walk`er</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A common prostitute who walks the streets to find customers.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Streetward</ent><br/
<hw>Street"ward`</hw> <pr>(?)</pr>, <pos>n.</pos> <def>An officer, or ward, having the care of the streets.</def> <mark>[Obs.]</mark>  <rj><au>Cowell.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Streetward</ent><br/
<hw>Street"ward</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Facing toward the street.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Their little <qex>streetward</qex> sitting room.</q> <rj><qau>Tennyson.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Streight</ent><br/
<hw>Streight</hw> <pr>(?)</pr>, <pos>a., n., & adv.</pos> <def>See 2nd <er>Strait</er>.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Streighten</ent><br/
<hw>Streight"en</hw> <pr>(?)</pr>, <pos>v. t.</pos> <def>See <er>Straiten</er>.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Strein</ent><br/
<hw>Strein</hw> <pr>(?)</pr>, <pos>v. t.</pos> <def>To strain.</def> <mark>[Obs.]</mark>  <rj><au>Chaucer.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Streit</ent><br/
<hw>Streit</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[See <er>Stretch</er>.]</ety> <def>Drawn.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>Pyrrhus with his <qex>streite</qex> sword.</q> <rj><qau>Chaucer.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Streit</ent><br/
<hw>Streit</hw>, <pos>a.</pos> <def>Close; narrow; strict.</def> <mark>[Obs.]</mark> <see>See <er>Strait</er>.</see><br/
[<source>1913 Webster</source>]</p>

<p><ent>Streite</ent><br/
<hw>Streite</hw>, <pos>adv.</pos> <def>Narrowly; strictly; straitly.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Strelitz</ent><br/
<hw>Strel"itz</hw> <pr>(?)</pr>, <pos>n. sing. & pl.</pos> <ety>[Russ. <ets>strieli<eacute/ts'</ets> a shooter, archer.]</ety> <def>A soldier of the ancient Muscovite guard or Russian standing army; also, the guard itself.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Strelitzia</ent><br/
||<hw>Stre*litz"i*a</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[NL., named after Charlotte, Princess of Mecklenburg-<etsep>Strelitz</etsep>, and queen of George III of Great Britain.]</ety> <fld>(Bot.)</fld> <def>A genus of plants related to the banana, found at the Cape of Good Hope. They have rigid glaucous distichous leaves, and peculiar richly colored flowers.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Strene</ent><br/
<hw>Strene</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[OE. <ets>stren</ets>, <ets>streen</ets>, <ets>streon</ets>, AS. ge<ets>stri<eacute/nan</ets>, ge<ets>str<?/nan</ets>, ge<ets>stre<oacute/nan</ets>, to beget, to obtain, ge<ets>stre<oacute/n</ets> gain, wealth; akin to OHG. <ets>striunan</ets> to gain.  Cf. <er>Strian</er> race, family.]</ety> <def>Race; offspring; stock; breed; strain.</def> <mark>[Obs.]</mark>  <rj><au>Chaucer.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Strengest</ent><br/
<ent>Strenger</ent><br/
<mhw>{ <hw>Stren"ger</hw> <pr>(?)</pr>, <hw>Stren"gest</hw> <pr>(?)</pr>, }</mhw> <def>the original <pos>compar.</pos> & <pos>superl.</pos> of <er>Strong</er>.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>Two of us shall <qex>strenger</qex> be than one.</q> <rj><qau>Chaucer.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Strength</ent><br/
<hw>Strength</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[OE. <ets>strengthe</ets>, AS. <ets>streng<edh/u</ets>, fr. <ets>strang</ets> strong. See <er>Strong</er>.]</ety> <sn>1.</sn> <def>The quality or state of being strong; ability to do or to bear; capacity for exertion or endurance, whether physical, intellectual, or moral; force; vigor; power; <as>as, <ex>strength</ex> of body or of the arm; <ex>strength</ex> of mind, of memory, or of judgment</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>All his [Samson's] <qex>strength</qex> in his hairs were.</q> <rj><qau>Chaucer.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Thou must outlive<br/
Thy youth, thy <qex>strength</qex>, thy beauty.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Power to resist force; solidity or toughness; the quality of bodies by which they endure the application of force without breaking or yielding; -- in this sense opposed to <contr>frangibility</contr>; <as>as, the <ex>strength</ex> of a bone, of a beam, of a wall, a rope, and the like</as>.</def> <ldquo/The brittle <xex>strength</xex> of bones.<rdquo/  <rj><au>Milton.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>Power of resisting attacks; impregnability.</def> <ldquo/Our castle's <xex>strength</xex> will laugh a siege to scorn.<rdquo/  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>That quality which tends to secure results; effective power in an institution or enactment; security; validity; legal or moral force; logical conclusiveness; <as>as, the <ex>strength</ex> of social or legal obligations; the <ex>strength</ex> of law; the <ex>strength</ex> of public opinion; <ex>strength</ex> of evidence; <ex>strength</ex> of argument</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <def>One who, or that which, is regarded as embodying or affording force, strength, or firmness; that on which confidence or reliance is based; support; security.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>God is our refuge and <qex>strength</qex>.</q> <rj><qau>Ps. xlvi. 1.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>What they boded would be a mischief to us, you are providing shall be one of our principal <qex>strengths</qex>.</q> <rj><qau>Sprat.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Certainly there is not a greater <qex>strength</qex> against temptation.</q> <rj><qau>Jer. Taylor.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>6.</sn> <def>Force as measured; amount, numbers, or power of any body, as of an army, a navy, and the like; as, what is the <xex>strength</xex> of the enemy by land, or by sea?</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>7.</sn> <def>Vigor or style; force of expression; nervous diction; -- said of literary work.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>And praise the easy vigor of a life<br/
Where Denham's <qex>strength</qex> and Waller's sweetness join.</q> <rj><qau>Pope.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>8.</sn> <def>Intensity; -- said of light or color.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Bright Phoebus in his <qex>strength</qex>.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>9.</sn> <def>Intensity or degree of the distinguishing and essential element; spirit; virtue; excellence; -- said of liquors, solutions, etc.; <as>as, the <ex>strength</ex> of wine or of acids</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>10.</sn> <def>A strong place; a stronghold.</def> <mark>[Obs.]</mark>  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><mcol><col><b>On the strength of</b></col>, <it>or</it> <col><b>Upon the strength of</b></col></mcol>, <cd>in reliance upon.</cd> <ldquo/The allies, after a successful summer, are too apt, <xex>upon the strength of</xex> it, to neglect their preparations for the ensuing campaign.<rdquo/ <au>Addison.</au></cs><br/
[<source>1913 Webster</source>]</p>

<p><syn><b>Syn.</b> -- Force; robustness; toughness; hardness; stoutness; brawniness; lustiness; firmness; puissance; support; spirit; validity; authority. See <er>Force</er>.</syn><br/
[<source>1913 Webster</source>]</p>

<p><ent>Strength</ent><br/
<hw>Strength</hw>, <pos>v. t.</pos> <def>To strengthen.</def> <mark>[Obs.]</mark>  <rj><au>Chaucer.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Strengthen</ent><br/
<hw>Strength"en</hw> <pr>(?)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Strengthened</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Strengthening</conjf>.]</vmorph> <sn>1.</sn> <def>To make strong or stronger; to add strength to; <as>as, to <ex>strengthen</ex> a limb, a bridge, an army; to <ex>strengthen</ex> an obligation; to <ex>strengthen</ex> authority</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Let noble Warwick, Cobham, and the rest, . . . <br/
With powerful policy <qex>strengthen</qex> themselves.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To animate; to encourage; to fix in resolution.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Charge Joshua, and encourage him, and <qex>strengthen</qex> him.</q> <rj><qau>Deut. iii. 28.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><syn><b>Syn.</b> -- To invigorate; confirm; establish; fortify; animate; encourage.</syn><br/
[<source>1913 Webster</source>]</p>

<p><ent>Strengthen</ent><br/
<hw>Strength"en</hw> <pr>(?)</pr>, <pos>v. i.</pos> <def>To grow strong or stronger.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The young disease, that must subdue at length,<br/
Grows with his growth, and <qex>strengthens</qex> with his strength.</q> <rj><qau>Pope.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Strengthener</ent><br/
<hw>Strength"en*er</hw> <pr>(?)</pr>, <pos>n.</pos> <def>One who, or that which, gives or adds strength.</def>  <rj><au>Sir W. Temple.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Strengthening</ent><br/
<hw>Strength"en*ing</hw>, <pos>a.</pos> <def>That strengthens; giving or increasing strength.</def> -- <wordforms><wf>Strength"en*ing*ly</wf>, <pos>adv.</pos></wordforms><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Strengthening plaster</b></col> <fld>(Med.)</fld>, <cd>a plaster containing iron, and supposed to have tonic effects.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Strengthful</ent><br/
<hw>Strength"ful</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Abounding in strength; full of strength; strong.</def> -- <wordforms><wf>Strength"ful*ness</wf>, <pos>n.</pos></wordforms><br/
[<source>1913 Webster</source>]</p>

<p><q>Florence my friend, in court my faction<br/
Not meanly <qex>strengthful</qex>.</q> <rj><qau>Marston.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Strengthing</ent><br/
<hw>Strength"ing</hw>, <pos>n.</pos> <def>A stronghold.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Strengthless</ent><br/
<hw>Strength"less</hw>, <pos>a.</pos> <def>Destitute of strength.</def>  <rj><au>Boyle.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Strengthner</ent><br/
<hw>Strength"ner</hw> <pr>(?)</pr>, <pos>n.</pos> <def>See <er>Strengthener</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Strengthy</ent><br/
<hw>Strength"y</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Having strength; strong.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Strenuity</ent><br/
<hw>Stre*nu"i*ty</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>strenuatis</ets>.]</ety> <def>Strenuousness; activity.</def> <mark>[Obs.]</mark>  <rj><au>Chapman.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Strenuous</ent><br/
<hw>Stren"u*ous</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>strenuus</ets>; cf. Gr. <?/ strong, hard, rough, harsh.]</ety> <def>Eagerly pressing or urgent; zealous; ardent; earnest; bold; valiant; intrepid; <as>as, a <ex>strenuous</ex> advocate for national rights; a <ex>strenuous</ex> reformer; a <ex>strenuous</ex> defender of his country</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>And spirit-stirring wine, that <qex>strenuous</qex> makes.</q> <rj><qau>Chapman.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q><qex>Strenuous</qex>, continuous labor is pain.</q> <rj><qau>I. Taylor.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p>-- <wordforms><wf>Stren"u*ous*ly</wf>, <pos>adv.</pos> -- <wf>Stren"u*ous*ness</wf>, <pos>n.</pos></wordforms><br/
[<source>1913 Webster</source>]</p>

<p><ent>Strepent</ent><br/
<hw>Strep"ent</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>strepens</ets>, p. pr. of <ets>strepere</ets> to make a noise.]</ety> <def>Noisy; loud.</def> <mark>[R.]</mark>  <rj><au>Shenstone.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Streperous</ent><br/
<hw>Strep"er*ous</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[LL. <ets>streperus</ets>, fr. L. <ets>strepere</ets>. See <er>Strepent</er>, and cf. <er>Obstreperous</er>.]</ety> <def>Loud; boisterous.</def> <mark>[R.]</mark>  <rj><au>Sir T. Browne.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Strepitores</ent><br/
||<hw>Strep`i*to"res</hw> <pr>(?)</pr>, <pos>n. pl.</pos> <ety>[NL., fr. L. <ets>strepitus</ets> clamor.]</ety> <fld>(Zool.)</fld> <def>A division of birds, including the clamatorial and picarian birds, which do not have well developed singing organs.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Strepsipteran</ent><br/
<ent>Strepsipter</ent><br/
<mhw>{ <hw>Strep*sip"ter</hw> <pr>(?)</pr>, <hw>Strep*sip"ter*an</hw> <pr>(?)</pr>, }</mhw> <pos>n.</pos> <fld>(Zool.)</fld> <def>One of the Strepsiptera.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Strepsiptera</ent><br/
||<hw>Strep*sip"te*ra</hw> <pr>(?)</pr>, <pos>n. pl.</pos> <ety>[NL., fr. Gr. <?/ a turning (fr. <?/ to twist) + <?/ a wing.]</ety> <fld>(Zool.)</fld> <def>A group of small insects having the anterior wings rudimentary, and in the form of short and slender twisted appendages, while the posterior ones are large and membranous. They are parasitic in the larval state on bees, wasps, and the like; -- called also <altname>Rhipiptera</altname>. See <xex>Illust.</xex> under <er>Rhipipter</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Strepsipterous</ent><br/
<hw>Strep*sip"ter*ous</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[See <er>Strepsiptera</er>.]</ety> <fld>(Zool.)</fld> <def>Of or pertaining to Strepsiptera.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Strepsorhina</ent><br/
||<hw>Strep`so*rhi"na</hw> <pr>(?)</pr>, <pos>n. pl.</pos> <ety>[NL., fr. Gr. <?/ a turning + <?/, <?/, the nose.]</ety> <fld>(Zool.)</fld> <def>Same as <er>Lemuroidea</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Strepsorhine</ent><br/
<hw>Strep"so*rhine</hw> <pr>(?)</pr>, <pos>a.</pos> <fld>(Zool.)</fld> <def>Having twisted nostrils; -- said of the lemurs.</def> -- <def2><pos>n.</pos> <fld>(Zool.)</fld> <def>One of the Strepsorhina; a lemur. See <xex>Illust.</xex> under <er>Monkey</er>.</def></def2><br/
[<source>1913 Webster</source>]</p>

<p><ent>Streptobacteria</ent><br/
||<hw>Strep`to*bac*te"ri*a</hw> <pr>(?)</pr>, <pos>n. pl.</pos>; <sing>sing. <singw>Streptobracterium</singw> <pr>(<?/)</pr></sing>. <ety>[NL., fr. Gr. <?/ pliant, bent + E. & NL. <ets>bacteria</ets>.]</ety> <fld>(Biol.)</fld> <def>A so-called variety of bacterium, consisting in reality of several bacteria linked together in the form of a chain.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Streptococcus</ent><br/
||<hw>Strep`to*coc"cus</hw> <pr>(?)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Streptococci</plw> <pr>(#)</pr>.</plu> <ety>[NL., fr. Gr. <?/ pliant, curved + <?/ a grain, seed.]</ety> <fld>(Biol.)</fld> <def>A long or short chain of micrococci, more or less curved.</def><br/
[<source>1913 Webster</source>]</p>

<p><-- Streptomyces. A genus of gram-positive bacteria growing in long filamentous, often branched chains.  They are plentiful in soil and produce the characteristic odor of soils.  Some of the species have proved to be useful as sources of important pharmaceutical agents.  Of these, the best known are streptomycin, chloramphenicaol, and tetracycline.  --><br/
[<source>1913 Webster</source>]</p>

<p><-- Streptomycete.  Any member of the family of bacteria called Streptomycetaceae, including the genus Streptomyces.  They are typically aerobic saprophytes producing begetative areial mycelium.  --><br/
[<source>1913 Webster</source>]</p>

<p><ent>Streptoneura</ent><br/
||<hw>Strep`to*neu"ra</hw> <pr>(?)</pr>, <pos>n. pl.</pos> <ety>[NL., fr. Gr. <?/ curved + <?/ a sinew.]</ety> <fld>(Zool.)</fld> <def>An extensive division of gastropod Mollusca in which the loop or visceral nerves is twisted, and the sexes separate. It is nearly to equivalent to Prosobranchiata.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Streptothrix</ent><br/
||<hw>Strep"to*thrix</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[NL., fr. Gr. <?/ pliant, bent + <?/ a hair.]</ety> <fld>(Biol.)</fld> <def>A genus of bacilli occurring of the form of long, smooth and apparently branched threads, either straight or twisted.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stress</ent><br/
<hw>Stress</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Abbrev. fr. <ets>distress</ets>; or cf. OF. <ets>estrecier</ets> to press, pinch, (assumed) LL. <ets>strictiare</ets>, fr. L. <ets>strictus</ets>. See <er>Distress</er>.]</ety> <sn>1.</sn> <def>Distress.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>Sad hersal of his heavy <qex>stress</qex>.</q> <rj><qau>Spenser.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Pressure, strain; -- used chiefly of immaterial things; except in mechanics; hence, urgency; importance; weight; significance.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The faculties of the mind are improved by exercise, yet they must not be put to a <qex>stress</qex> beyond their strength.</q> <rj><qau>Locke.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>A body may as well lay too little as too much <qex>stress</qex> upon a dream.</q> <rj><qau>L'Estrange.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <fld>(Mech. & Physics)</fld> <def>The force, or combination of forces, which produces a strain; force exerted in any direction or manner between contiguous bodies, or parts of bodies, and taking specific names according to its direction, or mode of action, as <xex>thrust</xex> or <xex>pressure</xex>, <xex>pull</xex> or <xex>tension</xex>, <xex>shear</xex> or <xex>tangential stress</xex>.</def>  <rj><au>Rankine.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q><qex>Stress</qex> is the mutual action between portions of matter.</q> <rj><qau>Clerk Maxwell.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <fld>(Pron.)</fld> <def>Force of utterance expended upon words or syllables. Stress is in English the chief element in accent and is one of the most important in emphasis. See <cref>Guide to pronunciation</cref>, <sect/<sect/ 31-35.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <fld>(Scots Law)</fld> <def>Distress; the act of distraining; also, the thing distrained.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Stress of voice</b></col>, <cd>unusual exertion of the voice.</cd> -- <col><b>Stress of weather</b></col>, <cd>constraint imposed by continued bad weather; <as>as, to be driven back to port by <ex>stress of weather</ex></as>.</cd> -- <col><b>To lay stress upon</b></col>, <cd>to attach great importance to; to emphasize.</cd> <ldquo/Consider how great a <xex>stress</xex> is <xex>laid upon</xex> this duty.<rdquo/ <au>Atterbury.</au> -- <mcol><col><b>To put stress upon</b></col>, <it>or</it> <col><b>To put to a stress</b></col></mcol>, <cd>to strain.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><-- p. 1425 --></p>

<p><ent>Stress</ent><br/
<hw>Stress</hw> <pr>(str<ecr/s)</pr>, <pos>v. t.</pos> <sn>1.</sn> <def>To press; to urge; to distress; to put to difficulties.</def> <mark>[R.]</mark>  <rj><au>Spenser.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To subject to stress, pressure, or strain.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>To subject to phonetic stress; to accent.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><sn>4.</sn>  <def>To place emphasis on; to make emphatic; emphasize.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Stressful</ent><br/
<hw>Stress"ful</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Having much stress.</def>  <rj><au>Rush.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stretch</ent><br/
<hw>Stretch</hw> <pr>(?)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Stretched</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Stretching</conjf>.]</vmorph> <ety>[OE. <ets>strecchen</ets>, AS. <ets>streccan</ets>; akin to D. <ets>strekken</ets>, G. <ets>strecken</ets>, OHG. <ets>strecchen</ets>, Sw. <ets>str<aum/cka</ets>, Dan. <ets>straekke</ets>; cf. AS. <ets>straeck</ets>, <ets>strec</ets>, strong, violent, G. <ets>strack</ets> straight; of uncertain origin, perhaps akin to E. <ets>strong</ets>.  Cf. <er>Straight</er>.]</ety> <sn>1.</sn> <def>To reach out; to extend; to put forth.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>And <qex>stretch</qex> forth his neck long and small.</q> <rj><qau>Chaucer.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>I in conquest <qex>stretched</qex> mine arm.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To draw out to the full length; to cause to extend in a straight line; <as>as, to <ex>stretch</ex> a cord or rope</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>To cause to extend in breadth; to spread; to expand; <as>as, to <ex>stretch</ex> cloth; to <ex>stretch</ex> the wings</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>To make tense; to tighten; to distend forcibly.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The ox hath therefore <qex>stretched</qex> his yoke in vain.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <def>To draw or pull out to greater length; to strain; <as>as, to <ex>stretch</ex> a tendon or muscle</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Awake, my soul, <qex>stretch</qex> every nerve.</q> <rj><qau>Doddridge.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>6.</sn> <def>To exaggerate; to extend too far; <as>as, to <ex>stretch</ex> the truth; to <ex>stretch</ex> one's credit</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>They take up, one day, the most violent and <qex>stretched</qex> prerogative.</q> <rj><qau>Burke.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stretch</ent><br/
<hw>Stretch</hw>, <pos>v. i.</pos> <sn>1.</sn> <def>To be extended; to be drawn out in length or in breadth, or both; to spread; to reach; <as>as, the iron road <ex>stretches</ex> across the continent; the lake <ex>stretches</ex> over fifty square miles</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>As far as <qex>stretcheth</qex> any ground.</q> <rj><qau>Gower.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To extend or spread one's self, or one's limbs; <as>as, the lazy man yawns and <ex>stretches</ex></as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>To be extended, or to bear extension, without breaking, as elastic or ductile substances.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The inner membrane . . . because it would <qex>stretch</qex> and yield, remained umbroken.</q> <rj><qau>Boyle.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>To strain the truth; to exaggerate; <as>as, a man apt to <ex>stretch</ex> in his report of facts</as>.</def> <mark>[Obs. or Colloq.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <fld>(Naut.)</fld> <def>To sail by the wind under press of canvas; <as>as, the ship <ex>stretched</ex> to the eastward</as>.</def>  <rj><au>Ham. Nav. Encyc.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Stretch out</b></col>, <cd>an order to rowers to extend themselves forward in dipping the oar.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stretch</ent><br/
<hw>Stretch</hw>, <pos>n.</pos> <sn>1.</sn> <def>Act of stretching, or state of being stretched; reach; effort; struggle; strain; <as>as, a <ex>stretch</ex> of the limbs; a <ex>stretch</ex> of the imagination</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>By <qex>stretch</qex> of arms the distant shore to gain.</q> <rj><qau>Dryden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Those put a lawful authority upon the <qex>stretch</qex>, to the abuse of yower, under the color of prerogative.</q> <rj><qau>L'Estrange.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A continuous line or surface; a continuous space of time; <as>as, grassy <ex>stretches</ex> of land</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>A great <qex>stretch</qex> of cultivated country.</q> <rj><qau>W. Black.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>But all of them left me a week at a <qex>stretch</qex>.</q> <rj><qau>E. Eggleston.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>The extent to which anything may be stretched.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Quotations, in their utmost <qex>stretch</qex>, can signify no more than that Luther lay under severe agonies of mind.</q> <rj><qau>Atterbury.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>This is the utmost <qex>stretch</qex> that nature can.</q> <rj><qau>Granville.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <fld>(Naut.)</fld> <def>The reach or extent of a vessel's progress on one tack; a tack or board.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <def>Course; direction; <as>as, the <ex>stretch</ex> of seams of coal</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>To be on the stretch</b></col>, <cd>to be obliged to use one's utmost powers.</cd> -- <col><b>Home stretch</b></col>. <cd>See under <er>Home</er>, <pos>a.</pos></cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stretcher</ent><br/
<hw>Stretch"er</hw> <pr>(?)</pr>, <pos>n.</pos> <sn>1.</sn> <def>One who, or that which, stretches.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Masonry)</fld> <def>A brick or stone laid with its longer dimension in the line of direction of the wall.</def>  <rj><au>Gwilt.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <fld>(Arch.)</fld> <def>A piece of timber used in building.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <fld>(Naut.)</fld> <sd>(a)</sd> <def>A narrow crosspiece of the bottom of a boat against which a rower braces his feet.</def> <sd>(b)</sd> <def>A crosspiece placed between the sides of a boat to keep them apart when hoisted up and griped.</def>  <rj><au>Dana.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <def>A litter, or frame, for carrying disabled, wounded, or dead persons.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>6.</sn> <def>An overstretching of the truth; a lie.</def> <mark>[Slang]</mark><br/
[<source>1913 Webster</source>]</p>

<p><sn>7.</sn> <def>One of the rods in an umbrella, attached at one end to one of the ribs, and at the other to the tube sliding upon the handle.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>8.</sn> <def>An instrument for stretching boots or gloves.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>9.</sn> <def>The frame upon which canvas is stretched for a painting.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stretching</ent><br/
<hw>Stretch"ing</hw> <pr>(?)</pr>, <def><pos>a. & n.</pos> from <er>Stretch</er>, <pos>v.</pos></def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Stretching course</b></col> <fld>(Masonry)</fld>, <cd>a course or series of stretchers. See <er>Stretcher</er>, 2.</cd> <au>Britton.</au></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stretto</ent><br/
||<hw>Stret"to</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[It., close or contacted, pressed.]</ety> <fld>(Mus.)</fld> <sd>(a)</sd> <def>The crowding of answer upon subject near the end of a fugue.</def> <sd>(b)</sd> <def>In an opera or oratorio, a coda, or winding up, in an accelerated time.</def> <altsp>[Written also <asp>stretta</asp>.]</altsp><br/
[<source>1913 Webster</source>]</p>

<p><ent>Strew</ent><br/
<hw>Strew</hw> <pr>(?)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Strewed</conjf> <pr>(?)</pr>; <pos>p. p.</pos> <conjf>strewn</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Strewing</conjf>.]</vmorph> <ety>[OE. <ets>strewen</ets>, <ets>strawen</ets>, AS. <ets>strewian</ets>, <ets>stre<oacute/wian</ets>; akin to Ofries. <ets>strewa</ets>, OS. <ets>strewian</ets>, D. <ets>strooijen</ets>, G. <ets>streuen</ets>, OHG. <ets>strewen</ets>, Icel. <ets>str<amac/</ets>, Sw. <ets>str<oum/</ets>, Dan. <ets>str<oum/e</ets>, Goth. <ets>straujan</ets>, L. <ets>sternere</ets>, <ets>stratum</ets>, Gr. <?/, <?/, Skr. <ets>st<?/</ets>. <root/166.  Cf. <er>Stratum</er>, <er>Straw</er>, <er>Street</er>.]</ety> <sn>1.</sn> <def>To scatter; to spread by scattering; to cast or to throw loosely apart; -- used of solids, separated or separable into parts or particles; <as>as, to <ex>strew</ex> seed in beds; to <ex>strew</ex> sand on or over a floor; to <ex>strew</ex> flowers over a grave.</as></def><br/
[<source>1913 Webster</source>]</p>

<p><q>And <qex>strewed</qex> his mangled limbs about the field.</q> <rj><qau>Dryden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>On a principal table a desk was open and many papers [were] <qex>strewn</qex> about.</q> <rj><qau>Beaconsfield.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To cover more or less thickly by scattering something over or upon; to cover, or lie upon, by having been scattered; <as>as, they <ex>strewed</ex> the ground with leaves; leaves <ex>strewed</ex> the ground.</as></def><br/
[<source>1913 Webster</source>]</p>

<p><q>The snow which does the top of Pindus <qex>strew</qex>.</q> <rj><qau>Spenser.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Is thine alone the seed that <qex>strews</qex> the plain?</q> <rj><qau>Pope.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>To spread abroad; to disseminate.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>She may <qex>strew</qex> dangerous conjectures.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Strewing</ent><br/
<hw>Strew"ing</hw> <pr>(?)</pr>, <pos>n.</pos> <sn>1.</sn> <def>The act of scattering or spreading.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Anything that is, or may be, strewed; -- used chiefly in the plural.</def>  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Strewment</ent><br/
<hw>Strew"ment</hw> <pr>(?)</pr>, <pos>n.</pos> <def>Anything scattered, as flowers for decoration.</def> <mark>[Obs.]</mark>  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Strewn</ent><br/
<hw>Strewn</hw> <pr>(?)</pr>, <def><pos>p. p.</pos> of <er>Strew</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stria</ent><br/
<hw>Stri"a</hw> <pr>(?)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Striae</plw> <pr>(#)</pr>.</plu> <ety>[L., a furrow, channel, hollow.]</ety> <sn>1.</sn> <def>A minute groove, or channel; a threadlike line, as of color; a narrow structural band or line; a striation; <as>as, the <ex>striae</ex>, or groovings, produced on a rock by a glacier passing over it; the <ex>striae</ex> on the surface of a shell; a <ex>stria</ex> of nervous matter in the brain</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Arch.)</fld> <def>A fillet between the flutes of columns, pilasters, or the like.</def>  <rj><au>Oxf. Gloss.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Striate</ent><br/
<hw>Stri"ate</hw> <pr>(?)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Striated</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Striating</conjf>.]</vmorph> <ety>[See <er>Striate</er>, <pos>a.</pos>]</ety> <def>To mark with striaae.</def> <ldquo/<xex>Striated</xex> longitudinally.<rdquo/  <rj><au>Owen.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Striated</ent><br/
<ent>Striate</ent><br/
<mhw>{ <hw>Stri"ate</hw> <pr>(?)</pr>, <hw>Stri"a*ted</hw> <pr>(?)</pr>, }</mhw> <pos>a.</pos> <ety>[L. <ets>striatus</ets>, p. p. of <ets>striare</ets> to furnish with channels, from <ets>stria</ets> a channel.]</ety> <def>Marked with striaae, or fine grooves, or lines of color; showing narrow structural bands or lines; <as>as, a <ex>striated</ex> crystal; <ex>striated</ex> muscular fiber</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Striation</ent><br/
<hw>Stri*a"tion</hw> <pr>(?)</pr>, <pos>n.</pos> <sn>1.</sn> <def>The quality or condition of being striated.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A stria; <as>as, the <ex>striations</ex> on a shell</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Striatum</ent><br/
||<hw>Stri*a"tum</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[NL.]</ety> <fld>(Anat.)</fld> <def>The corpus striatum.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Striature</ent><br/
<hw>Stri"a*ture</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>striatura</ets>.]</ety> <def>A stria.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Strich</ent><br/
<hw>Strich</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Cf. L. <ets>strix</ets>, <ets>strigs</ets>, a streech owl.]</ety> <fld>(Zool.)</fld> <def>An owl.</def> <mark>[Obs.]</mark>  <rj><au>Spenser.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Strick</ent><br/
<hw>Strick</hw>, <pos>n.</pos> <def>A bunch of hackled flax prepared for drawing into slivers.</def>  <rj><au>Knight.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stricken</ent><br/
<hw>Strick"en</hw> <pr>(?)</pr>, <pos>p. p. & a.</pos> from <er>Strike</er>. <sn>1.</sn> <def>Struck; smitten; wounded; <as>as, the <ex>stricken</ex> deer</as>.</def> <note>[See <er>Strike</er>, <pos>n.</pos>]</note><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Worn out; far gone; advanced. See <er>Strike</er>, <pos>v. t.</pos>, 21.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Abraham was old and well <qex>stricken</qex> in age.</q> <rj><qau>Gen. xxiv. 1.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>Whole; entire; -- said of the hour as marked by the striking of a clock.</def> <mark>[Scot.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>He persevered for a <qex>stricken</qex> hour in such a torrent of unnecessary tattle.</q> <rj><qau>Sir W. Scott.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Speeches are spoken by the <qex>stricken</qex> hour, day after day, week, perhaps, after week.</q> <rj><qau>Bayne.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Strickle</ent><br/
<hw>Stric"kle</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[See <er>Strike</er>.]</ety> <sn>1.</sn> <def>An instrument to strike grain to a level with the measure; a strike.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>An instrument for whetting scythes; a rifle.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <fld>(Founding)</fld> <def>An instrument used for smoothing the surface of a core.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <fld>(Carp. & Mason.)</fld> <def>A templet; a pattern.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <def>An instrument used in dressing flax.</def> <mark>[Prov. Eng.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Strickler</ent><br/
<hw>Stric"kler</hw> <pr>(?)</pr>, <pos>n.</pos> <def>See <er>Strickle</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Strickless</ent><br/
<hw>Strick"less</hw>, <pos>n.</pos> <def>See <er>Strickle</er>.</def> <mark>[Prov. Eng.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Strict</ent><br/
<hw>Strict</hw> <pr>(?)</pr>, <pos>a.</pos> <amorph>[<pos>Compar.</pos> <adjf>Stricter</adjf> <pr>(?)</pr>; <pos>superl.</pos> <adjf>Strictest</adjf>.]</amorph> <ety>[L. <ets>strictus</ets>, p. p. of <ets>stringere</ets> to draw or bind tight, to strain. See <er>Strain</er>, and cf. <er>Strait</er>, <pos>a.</pos>]</ety> <sn>1.</sn> <def>Strained; drawn close; tight; <as>as, a <ex>strict</ex> embrace; a <ex>strict</ex> ligature</as>.</def>  <rj><au>Dryden.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Tense; not relaxed; <as>as, a <ex>strict</ex> fiber</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>Exact; accurate; precise; rigorously nice; <as>as, to keep <ex>strict</ex> watch; to pay <ex>strict</ex> attention</as>.</def>  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>It shall be still in <qex>strictest</qex> measure.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>Governed or governing by exact rules; observing exact rules; severe; rigorous; <as>as, very <ex>strict</ex> in observing the Sabbath</as>.</def> <ldquo/Through the <xex>strict</xex> senteries.<rdquo/  <rj><au>Milton.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <def>Rigidly; interpreted; exactly limited; confined; restricted; <as>as, to understand words in a <ex>strict</ex> sense</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>6.</sn> <fld>(Bot.)</fld> <def>Upright, or straight and narrow; -- said of the shape of the plants or their flower clusters.</def><br/
[<source>1913 Webster</source>]</p>

<p><syn><b>Syn.</b> -- Exact; accurate; nice; close; rigorous; severe.</syn>  -- <usage><er>Strict</er>, <er>Severe</er>. <xex>Strict</xex>, applied to a person, denotes that he conforms in his motives and acts to a principle or code by which he is bound; <xex>severe</xex> is strict with an implication often, but not always, of harshness. <xex>Strict</xex> is opposed to <xex>lax</xex>; <xex>severe</xex> is opposed to <xex>gentle</xex>.</usage><br/
[<source>1913 Webster</source>]</p>

<p><q>And rules as <qex>strict</qex> his labored work confine,<br/
As if the Stagirite o'erlooked each line.</q> <rj><qau>Pope.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Soon moved with touch of blame, thus Eve: -<br/
<ldquo/What words have passed thy lips, Adam <qex>severe</qex>!<rdquo/</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><mcol><col><b>The Strict Observance</b></col>, <it>or</it> <col><b>Friars of the Strict Observance</b></col></mcol>. <fld>(R. C. Ch.)</fld> <cd>See <er>Observance</er>.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Striction</ent><br/
<hw>Stric"tion</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>strictio</ets>. See <er>Stringent</er>.]</ety> <def>The act of constricting, or the state of being constricted.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Line of striction</b></col> <fld>(Geom.)</fld>, <cd>the line on a skew surface that cuts each generator in that point of it that is nearest to the succeeding generator.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Strictly</ent><br/
<hw>Strict"ly</hw>, <pos>adv.</pos> <def>In a strict manner; closely; precisely.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Strictness</ent><br/
<hw>Strict"ness</hw>, <pos>n.</pos> <def>Quality or state of being strict.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stricture</ent><br/
<hw>Stric"ture</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>strictura</ets> a contraction, from <ets>stringere</ets>, <ets>strictum</ets>, to draw tight: cf. F. <ets>stricture</ets>. See <er>Strict</er>.]</ety> <sn>1.</sn> <def>Strictness.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>A man of <qex>stricture</qex> and firm abstinence.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A stroke; a glance; a touch.</def> <mark>[Obs.]</mark>  <rj><au>Sir M. Hale.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>A touch of adverse criticism; censure.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>[I have] given myself the liberty of these <qex>strictures</qex> by way of reflection on all and every passage.</q> <rj><qau>Hammond.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <fld>(Med.)</fld> <def>A localized morbid contraction of any passage of the body.  Cf. <cref>Organic stricture</cref>, and <cref>Spasmodic stricture</cref>, under <er>Organic</er>, and <er>Spasmodic</er>.</def>  <rj><au>Arbuthnot.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Strictured</ent><br/
<hw>Stric"tured</hw> <pr>(?)</pr>, <pos>a.</pos> <fld>(Med.)</fld> <def>Affected with a stricture; <as>as, a <ex>strictured</ex> duct</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Strid</ent><br/
<hw>Strid</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[See <er>Stride</er>.]</ety> <def>A narrow passage between precipitous rocks or banks, which looks as if it might be crossed at a stride.</def> <mark>[Prov. Eng.]</mark>  <rj><au>Howitt.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>This striding place is called the <qex>Strid</qex>.</q> <rj><qau>Wordsworth.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stride</ent><br/
<hw>Stride</hw> <pr>(?)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp.</pos> <conjf>Strode</conjf> <pr>(?)</pr> (<mark>Obs</mark>. <conjf>Strid</conjf> <pr>(<?/)</pr>); <pos>p. p.</pos> <conjf>Stridden</conjf> <pr>(?)</pr> (<mark>Obs.</mark> <conjf>Strid</conjf>); <pos>p. pr. & vb. n.</pos> <conjf>Striding</conjf>.]</vmorph> <ety>[AS. <ets>str<imac/dan</ets> to stride, to strive; akin to LG. <ets>striden</ets>, OFries. <ets>str<imac/da</ets> to strive, D. <ets>strijden</ets> to strive, to contend, G. <ets>streiten</ets>, OHG. <ets>str<imac/tan</ets>; of uncertain origin.  Cf. <er>Straddle</er>.]</ety> <sn>1.</sn> <def>To walk with long steps, especially in a measured or pompous manner.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Mars in the middle of the shining shield<br/
Is graved, and <qex>strides</qex> along the liquid field.</q> <rj><qau>Dryden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To stand with the legs wide apart; to straddle.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stride</ent><br/
<hw>Stride</hw>, <pos>v. t.</pos> <sn>1.</sn> <def>To pass over at a step; to step over.</def> <ldquo/A debtor that not dares to <xex>stride</xex> a limit.<rdquo/  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To straddle; to bestride.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>I mean to <qex>stride</qex> your steed.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stride</ent><br/
<hw>Stride</hw>, <pos>n.</pos> <def>The act of stridding; a long step; the space measured by a long step; <as>as, a masculine <ex>stride</ex></as>.</def>  <rj><au>Pope.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>God never meant that man should scale the heavens<br/
By <qex>strides</qex> of human wisdom.</q> <rj><qau>Cowper.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Strident</ent><br/
<hw>Stri"dent</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>stridens</ets>, <ets>-entis</ets>, p. pr. of <ets>stridere</ets> to make a grating or creaking noise.]</ety> <def>Characterized by harshness; grating; shrill.</def> <ldquo/A <xex>strident</xex> voice.<rdquo/  <rj><au>Thackeray.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stridor</ent><br/
||<hw>Stri"dor</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L., from <ets>stridere</ets> to make any harsh, grating, or creaking sound.]</ety> <def>A harsh, shrill, or creaking noise.</def>  <rj><au>Dryden.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stridulate</ent><br/
<hw>Strid"u*late</hw> <pr>(?)</pr>, <pos>v. t.</pos> <ety>[See <er>Stridulous</er>.]</ety> <def>To make a shrill, creaking noise</def>; specifically <fld>(Zool.)</fld>, <def>to make a shrill or musical sound, such as is made by the males of many insects.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stridulation</ent><br/
<hw>Strid`u*la"tion</hw> <pr>(?)</pr>, <pos>n.</pos> <def>The act of stridulating.</def> Specifically: <fld>(Zool.)</fld> <sd>(a)</sd> <def>The act of making shrill sounds or musical notes by rubbing together certain hard parts, as is done by the males of many insects, especially by Orthoptera, such as crickets, grasshoppers, and locusts.</def> <sd>(b)</sd> <def>The noise itself.</def><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ The crickets stridulate by rubbing together the strong nervures of the fore wings. Many grasshoppers stridulate by rubbing the hind legs across strong nervures on the fore wings. The green grasshoppers and katydids stridulate by means of special organs at the base of the fore wings.</note><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stridulator</ent><br/
<hw>Strid"u*la`tor</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[NL.]</ety> <def>That which stridulates.</def>  <rj><au>Darwin.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stridulatory</ent><br/
<hw>Strid"u*la*to*ry</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Stridulous; able to stridulate; used in stridulating; adapted for stridulation.</def>  <rj><au>Darwin.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stridulous</ent><br/
<hw>Strid"u*lous</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>stridulus</ets>. See <er>Strident</er>.]</ety> <def>Making a shrill, creaking sound.</def>  <rj><au>Sir T. Browne.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>The Sarmatian boor driving his <qex>stridulous</qex> cart.</q> <rj><qau>Longfellow.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Stridulous laryngitis</b></col> <fld>(Med.)</fld>, <cd>a form of croup, or laryngitis, in children, associated with dyspnoea, occurring usually at night, and marked by crowing or stridulous breathing.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Strife</ent><br/
<hw>Strife</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[OF. <ets>estrif</ets>. See <er>Strive</er>.]</ety> <sn>1.</sn> <def>The act of striving; earnest endeavor.</def> <mark>[Archaic]</mark>  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Exertion or contention for superiority; contest of emulation, either by intellectual or physical efforts.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Doting about questions and <qex>strifes</qex> of words.</q> <rj><qau>1 Tim. vi. 4.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Thus gods contended -- noble <qex>strife</qex> -<br/
Who most should ease the wants of life.</q> <rj><qau>Congreve.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>Altercation; violent contention; fight; battle.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Twenty of them fought in this black <qex>strife</qex>.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>These vows, thus granted, raised a <qex>strife</qex> above<br/
Betwixt the god of war and queen of love.</q> <rj><qau>Dryden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>That which is contended against; occasion of contest.</def> <mark>[Obs.]</mark> <ldquo/Lamenting her unlucky <xex>strife</xex>.<rdquo/  <rj><au>Spenser.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><syn><b>Syn.</b> -- Contest; struggle; quarrel. See <er>Contention</er>.</syn><br/
[<source>1913 Webster</source>]</p>

<p><ent>Strifeful</ent><br/
<hw>Strife"ful</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Contentious; discordant.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The ape was <qex>strifeful</qex> and ambitious.</q> <rj><qau>Spenser.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Strigate</ent><br/
<hw>Stri"gate</hw> <pr>(?)</pr>, <pos>a.</pos> <fld>(Zool.)</fld> <def>Having transverse bands of color.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Striges</ent><br/
||<hw>Stri"ges</hw> <pr>(?)</pr>, <pos>n. pl.</pos> <ety>[L., pl. of <ets>strix</ets> a streech owl; cf. Gr. <?/ a screaming night bird.]</ety> <fld>(Zool.)</fld> <def>The tribe of birds which comprises the owls.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Strigil</ent><br/
<hw>Strig"il</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>strigilis</ets>, from <ets>stringere</ets> to graze, scrape.]</ety> <fld>(Gr. & Rom. Antiq.)</fld> <def>An instrument of metal, ivory, etc., used for scraping the skin at the bath.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Strigillose</ent><br/
<hw>Strig"il*lose`</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[Dim. fr. <ets>strigose</ets>.]</ety> <fld>(Bot.)</fld> <def>Set with stiff, slender bristles.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Strigine</ent><br/
<hw>Stri"gine</hw> <pr>(?)</pr>, <pos>a.</pos> <fld>(Zool.)</fld> <def>Of or pertaining to owls; owl-like.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Strigment</ent><br/
<hw>Strig"ment</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>strigmentum</ets>.]</ety> <def>Scraping; that which is scraped off.</def> <mark>[Obs.]</mark>  <rj><au>Sir T. Browne.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Strigose</ent><br/
<hw>Stri*gose"</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[Cf. F. <ets>strigueux</ets>. See <er>Strigil</er>.]</ety> <fld>(Bot.)</fld> <def>Set with stiff, straight bristles; hispid; <as>as, a <ex>strigose</ex> leaf</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Strigous</ent><br/
<hw>Stri"gous</hw> <pr>(?)</pr>, <pos>a.</pos> <fld>(Bot.)</fld> <def>Strigose.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><-- p. 1426 --></p>

<p><ent>Strike</ent><br/
<hw>Strike</hw> <pr>(?)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp.</pos> <conjf>Struck</conjf> <pr>(?)</pr>; <pos>p. p.</pos> <conjf>Struck</conjf>, <conjf>Stricken</conjf> <pr>(<?/)</pr> (<conjf>Stroock</conjf> <pr>(<?/)</pr>, <conjf>Strucken</conjf> <pr>(<?/)</pr>, <mark>Obs.</mark>); <pos>p. pr. & vb. n.</pos> <conjf>Striking</conjf>. <xex>Struck</xex> is more commonly used in the p. p. than <xex>stricken</xex>.]</vmorph> <ety>[OE. <ets>striken</ets> to strike, proceed, flow, AS. <ets>str<imac/can</ets> to go, proceed, akin to D. <ets>strijken</ets> to rub, stroke, strike, to move, go, G. <ets>streichen</ets>, OHG. <ets>str<imac/hhan</ets>, L. <ets>stringere</ets> to touch lightly, to graze, to strip off (but perhaps not to L. <ets>stringere</ets> in sense to draw tight), <ets>striga</ets> a row, a furrow.  Cf. <er>Streak</er>, <er>Stroke</er>.]</ety> <sn>1.</sn> <def>To touch or hit with some force, either with the hand or with an instrument; to smite; to give a blow to, either with the hand or with any instrument or missile.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>He at Philippi kept<br/
His sword e'en like a dancer; while I <qex>struck</qex><br/
The lean and wrinkled Cassius.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To come in collision with; to strike against; <as>as, a bullet <ex>struck</ex> him; the wave <ex>struck</ex> the boat amidships; the ship <ex>struck</ex> a reef</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>To give, as a blow; to impel, as with a blow; to give a force to; to dash; to cast.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>They shall take of the blood, and <qex>strike</qex> it on the two sideposts.</q> <rj><qau>Ex. xii. 7.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Who would be free, themselves must <qex>strike</qex> the blow.</q> <rj><qau>Byron.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>To stamp or impress with a stroke; to coin; <as>as, to <ex>strike</ex> coin from metal: to <ex>strike</ex> dollars at the mint</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <def>To thrust in; to cause to enter or penetrate; to set in the earth; <as>as, a tree <ex>strikes</ex> its roots deep</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>6.</sn> <def>To punish; to afflict; to smite.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>To punish the just is not good, nor <qex>strike</qex> princes for equity.</q> <rj><qau>Prov. xvii. 26.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>7.</sn> <def>To cause to sound by one or more beats; to indicate or notify by audible strokes; <as>as, the clock <ex>strikes</ex> twelve; the drums <ex>strike</ex> up a march</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>8.</sn> <def>To lower; to let or take down; to remove; <as>as, to <ex>strike</ex> sail; to <ex>strike</ex> a flag or an ensign, as in token of surrender; to <ex>strike</ex> a yard or a topmast in a gale; to <ex>strike</ex> a tent; to <ex>strike</ex> the centering of an arch</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>9.</sn> <def>To make a sudden impression upon, as by a blow; to affect sensibly with some strong emotion; <as>as, to <ex>strike</ex> the mind, with surprise; to <ex>strike</ex> one with wonder, alarm, dread, or horror</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Nice works of art <qex>strike</qex> and surprise us most on the first view.</q> <rj><qau>Atterbury.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>They please as beauties, here as wonders <qex>strike</qex>.</q> <rj><qau>Pope.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>10.</sn> <def>To affect in some particular manner by a sudden impression or impulse; <as>as, the plan proposed <ex>strikes</ex> me favorably; to <ex>strike</ex> one dead or blind</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>How often has <qex>stricken</qex> you dumb with his irony!</q> <rj><qau>Landor.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>11.</sn> <def>To cause or produce by a stroke, or suddenly, as by a stroke; <as>as, to <ex>strike</ex> a light</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Waving wide her myrtle wand,<br/
She <qex>strikes</qex> a universal peace through sea and land.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>12.</sn> <def>To cause to ignite; <as>as, to <ex>strike</ex> a match</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>13.</sn> <def>To make and ratify; <as>as, to <ex>strike</ex> a bargain</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ Probably borrowed from the L. <xex>foedus ferrire</xex>, to <xex>strike</xex> a compact, so called because an animal was <xex>struck</xex> and killed as a sacrifice on such occasions.</note><br/
[<source>1913 Webster</source>]</p>

<p><sn>14.</sn> <def>To take forcibly or fraudulently; <as>as, to <ex>strike</ex> money</as>.</def> <mark>[Old Slang]</mark><br/
[<source>1913 Webster</source>]</p>

<p><sn>15.</sn> <def>To level, as a measure of grain, salt, or the like, by scraping off with a straight instrument what is above the level of the top.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>16.</sn> <fld>(Masonry)</fld> <def>To cut off, as a mortar joint, even with the face of the wall, or inward at a slight angle.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>17.</sn> <def>To hit upon, or light upon, suddenly; <as>as, my eye <ex>struck</ex> a strange word; they soon <ex>struck</ex> the trail</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>18.</sn> <def>To borrow money of; to make a demand upon; <as>as, he <ex>struck</ex> a friend for five dollars</as>.</def> <mark>[Slang]</mark><br/
[<source>1913 Webster</source>]</p>

<p><sn>19.</sn> <def>To lade into a cooler, as a liquor.</def>  <rj><au>B. Edwards.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>20.</sn> <def>To stroke or pass lightly; to wave.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Behold, I thought, He will . . . <qex>strike</qex> his hand over the place, and recover the leper.</q> <rj><qau>2 Kings v. 11.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>21.</sn> <def>To advance; to cause to go forward; -- used only in past participle.</def> <ldquo/Well <xex>struck</xex> in years.<rdquo/  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><mcol><col><b>To strike an attitude</b></col>, <col><b>To strike a balance</b></col></mcol>. <cd>See under <er>Attitude</er>, and <er>Balance</er>.</cd> -- <col><b>To strike a jury</b></col> <fld>(Law)</fld>, <cd>to constitute a special jury ordered by a court, by each party striking out a certain number of names from a prepared list of jurors, so as to reduce it to the number of persons required by law.</cd> <au>Burrill.</au> -- <col><b>To strike a lead</b></col>. <sd>(a)</sd> <fld>(Mining)</fld> <cd>To find a vein of ore.</cd> <sd>(b)</sd> <cd>Fig.: To find a way to fortune.</cd> <mark>[Colloq.]</mark> -- <mcol><col><b>To strike a ledger</b></col> <it>or</it> <col><b>To strike an account</b></col></mcol>, <cd>to balance it.</cd> -- <col><b>To strike hands with</b></col>. <sd>(a)</sd> <cd>To shake hands with</cd>. <au>Halliwell.</au> <sd>(b)</sd> <cd>To make a compact or agreement with; to agree with.</cd> -- <col><b>To strike off</b></col>. <sd>(a)</sd> <cd>To erase from an account; to deduct; <as>as, <ex>to strike off</ex> the interest of a debt</as></cd>. <sd>(b)</sd> <fld>(Print.)</fld> <cd>To impress; to print; <as>as, <ex>to strike off</ex> a thousand copies of a book</as></cd>.<-- = to run off? [copies] --> <sd>(c)</sd> <cd>To separate by a blow or any sudden action; <as>as, <ex>to strike off</ex> what is superfluous or corrupt</as>.</cd> -- <col><b>To strike oil</b></col>, <cd>to find petroleum when boring for it; figuratively, to make a lucky hit financially.</cd> <mark>[Slang, U.S.]</mark> -- <col><b>To strike one luck</b></col>, <cd>to shake hands with one and wish good luck.</cd> <mark>[Obs.]</mark> <au>Beau. & Fl.</au> -- <col><b>To strike out</b></col>. <sd>(a)</sd> <cd>To produce by collision; to force out, <as>as, <ex>to strike out</ex> sparks with steel</as>.</cd> <sd>(b)</sd> <cd>To blot out; to efface; to erase</cd>. <ldquo/To methodize is as necessary as <xex>to strike out</xex>.<rdquo/ <au>Pope.</au> <sd>(c)</sd> <cd>To form by a quick effort; to devise; to invent; to contrive, <as>as, <ex>to strike out</ex> a new plan of finance</as>.</cd> <sd>(d)</sd> <fld>(Baseball)</fld> <cd>To cause a player to strike out; -- said of the pitcher. See <cref>To strike out</cref>, under <er>Strike</er>, <pos>v. i.</pos></cd> -- <col><b>To strike sail</b></col>. <cd>See under <er>Sail</er>.</cd> -- <col><b>To strike up</b></col>. <sd>(a)</sd> <cd>To cause to sound; to begin to beat</cd>. <ldquo/<xex>Strike up</xex> the drums.<rdquo/ <au>Shak.</au> <sd>(b)</sd> <cd>To begin to sing or play; as, <xex>to strike up</xex> a tune.</cd> <sd>(c)</sd> <cd>To raise (as sheet metal), in making diahes, pans, etc., by blows or pressure in a die.</cd> -- <col><b>To strike work</b></col>, <cd>to quit work; to go on a strike.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Strike</ent><br/
<hw>Strike</hw> <pr>(?)</pr>, <pos>v. i.</pos> <def>To move; to advance; to proceed; to take a course; <as>as, to <ex>strike</ex> into the fields</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>A mouse . . . <qex>struck</qex> forth sternly [bodily].</q> <rj><qau>Piers Plowman.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To deliver a quick blow or thrust; to give blows.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>And fiercely took his trenchant blade in hand,<br/
With which he <qex>stroke</qex> so furious and so fell.</q> <rj><qau>Spenser.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q><qex>Strike</qex> now, or else the iron cools.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>To hit; to collide; to dush; to clash; <as>as, a hammer <ex>strikes</ex> against the bell of a clock</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>To sound by percussion, with blows, or as with blows; to be struck; <as>as, the clock <ex>strikes</ex></as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>A deep sound <qex>strikes</qex> like a rising knell.</q> <rj><qau>Byron.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <def>To make an attack; to aim a blow.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>A puny subject <qex>strikes</qex><br/
At thy great glory.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q><qex>Struck</qex> for throne, and <qex>striking</qex> found his doom.</q> <rj><qau>Tennyson.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>6.</sn> <def>To touch; to act by appulse.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Hinder light but from <qex>striking</qex> on it [porphyry], and its colors vanish.</q> <rj><qau>Locke.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>7.</sn> <def>To run upon a rock or bank; to be stranded; <as>as, the ship <ex>struck</ex> in the night</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>8.</sn> <def>To pass with a quick or strong effect; to dart; to penetrate.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Till a dart <qex>strike</qex> through his liver.</q> <rj><qau>Prov. vii. 23.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Now and then a glittering beam of wit or passion <qex>strikes</qex> through the obscurity of the poem.</q> <rj><qau>Dryden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>9.</sn> <def>To break forth; to commence suddenly; -- with <xex>into</xex>; <as>as, to <ex>strike</ex> into reputation; to <ex>strike</ex> into a run</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>10.</sn> <def>To lower a flag, or colors, in token of respect, or to signify a surrender of a ship to an enemy.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>That the English ships of war should not <qex>strike</qex> in the Danish seas.</q> <rj><qau>Bp. Burnet.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>11.</sn> <def>To quit work in order to compel an increase, or prevent a reduction, of wages.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>12.</sn> <def>To become attached to something; -- said of the spat of oysters.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>13.</sn> <def>To steal money.</def> <mark>[Old Slang, Eng.]</mark>  <rj><au>Nares.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>To strike at</b></col>, <cd>to aim a blow at.</cd> -- <col><b>To strike for</b></col>, <cd>to start suddenly on a course for.</cd> -- <col><b>To strike home</b></col>, <cd>to give a blow which reaches its object, to strike with effect.</cd> -- <col><b>To strike in</b></col>. <sd>(a)</sd> <cd>To enter suddenly</cd>. <sd>(b)</sd> <cd>To disappear from the surface, with internal effects, as an eruptive disease</cd>. <sd>(c)</sd> <cd>To come in suddenly; to interpose; to interrupt</cd>. <ldquo/I proposed the embassy of Constantinople for Mr. Henshaw, but my Lord Winchelsea <xex>struck in</xex>.<rdquo/ <au>Evelyn.</au> <sd>(d)</sd> <cd>To join in after another has begun,as in singing.</cd> -- <col><b>To strike in with</b></col>, <cd>to conform to; to suit itself to; to side with, to join with at once.</cd> <ldquo/To assert this is <xex>to strike in with</xex> the known enemies of God's grace.<rdquo/ <au>South.</au> -- <col><b>To strike out</b></col>. <sd>(a)</sd> <cd>To start; to wander; to make a sudden excursion; <as>as, <ex>to strike out</ex> into an irregular course of life</as>.</cd> <sd>(b)</sd> <cd>To strike with full force</cd>. <sd>(c)</sd> <fld>(Baseball)</fld> <cd>To be put out for not hitting the ball during one's turn at the bat.</cd> -- <col><b>To strike up</b></col>, <cd>to commence to play as a musician; to begin to sound, as an instrument.</cd>  <ldquo/Whilst any trump did sound, or drum <xex>struck up</xex>.<rdquo/ <au>Shak.</au></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Strike</ent><br/
<hw>Strike</hw> <pr>(?)</pr>, <pos>n.</pos> <sn>1.</sn> <def>The act of striking.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>An instrument with a straight edge for leveling a measure of grain, salt, and the like, scraping off what is above the level of the top; a strickle.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>A bushel; four pecks.</def> <mark>[Prov. Eng.]</mark>  <rj><au>Tusser.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>An old measure of four bushels.</def> <mark>[Prov. Eng.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <def>Fullness of measure; hence, excellence of quality.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Three hogsheads of ale of the first <qex>strike</qex>.</q> <rj><qau>Sir W. Scott.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>6.</sn> <def>An iron pale or standard in a gate or fence.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><sn>7.</sn> <def>The act of quitting work; specifically, such an act by a body of workmen, usually organized by a labor union, done as a means of enforcing compliance with demands made on their employer.</def><br/
[<source>1913 Webster</source> <source>+PJC</source>]</p>

<p><q><qex>Strikes</qex> are the insurrections of labor.</q> <rj><qau>F. A. Walker.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>8.</sn> <fld>(Iron Working)</fld> <def>A puddler's stirrer.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>9.</sn> <fld>(Geol.)</fld> <def>The horizontal direction of the outcropping edges of tilted rocks; or, the direction of a horizontal line supposed to be drawn on the surface of a tilted stratum. It is at right angles to the dip.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>10.</sn> <def>The extortion of money, or the attempt to extort money, by threat of injury; blackmailing.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>11.</sn> <def>A sudden finding of rich ore in mining; hence, any sudden success or good fortune, esp. financial.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><sn>12.</sn>  <fld>(Bowling, U. S.)</fld> <def>The act of leveling all the pins with the first bowl; also, the score thus made. Sometimes called <altname>double spare</altname>.  Throwing a strike entitles the player to add to the score for that frame the total number of pins knocked down in the next two bowls.</def><br/
[<source>Webster 1913 Suppl.</source> <source>+PJC</source>]</p>

<p><sn>13.</sn>  <fld>(Baseball)</fld> <def>Any actual or constructive striking at the pitched ball, three of which, if the ball is not hit fairly, cause the batter to be put out; hence, any of various acts or events which are ruled as equivalent to such a striking, as failing to strike at a ball so pitched that the batter should have struck at it.</def> <ldquo/It's one, two, three <ex>strikes</ex> you're out in the old ball game.<rdquo/ <au>[Take me out to the ball game]</au><br/
[<source>Webster 1913 Suppl.</source> <source>+PJC</source>]</p>

<p><sn>14.</sn>  <fld>(Tenpins)</fld> <def>Same as <er>Ten-strike</er>.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><cs><col><b>Strike block</b></col> <fld>(Carp.)</fld>, <cd>a plane shorter than a jointer, used for fitting a short joint.</cd> <au>Moxon.</au> -- <col><b>Strike of flax</b></col>, <cd>a handful that may be hackled at once.</cd> <mark>[Obs. or Prov. Eng.]</mark> <au>Chaucer.</au> -- <col><b>Strike of sugar</b></col>. <fld>(Sugar Making)</fld> <sd>(a)</sd> <cd>The act of emptying the teache, or last boiler, in which the cane juice is exposed to heat, into the coolers</cd>. <sd>(b)</sd> <cd>The quantity of the sirup thus emptied at once.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Strikeout</ent><br/
<hw>Strike"out</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Baseball)</fld> <def>an out made by a batter against whom three strikes were called in a single at-bat; also, the same out as credited to the pitcher, in the scoring of the game.</def><br/
[<source>PJC</source>]</p>

<p><ent>Striker</ent><br/
<hw>Strik"er</hw> <pr>(?)</pr>, <pos>n.</pos> <sn>1.</sn> <def>One who, or that which, strikes; specifically, a blacksmith's helper who wields the sledge.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A harpoon; also, a harpooner.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Wherever we come to an anchor, we always send out our <qex>strikers</qex>, and put out hooks and lines overboard, to try fish.</q> <rj><qau>Dampier.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>A wencher; a lewd man.</def> <mark>[Obs.]</mark>  <rj><au>Massinger.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>A workman who is on a strike.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <def>A blackmailer in politics; also, one whose political influence can be bought.</def> <mark>[Political Cant]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Strike zone</ent><br/
<hw>Strike" zone</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Baseball)</fld> <def>the area above home plate where, if a pitched ball passes through and is not hit by the batter, the pitch counts as a strike.  Specifically, it is directly above home plate and at a height from the knees to the shoulders of the batter.</def><br/
[<source>PJC</source>]</p>

<p><ent>Striking</ent><br/
<hw>Strik"ing</hw>, <def><pos>a. & n.</pos> from <er>Strike</er>, <pos>v.</pos></def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Striking distance</b></col>, <cd>the distance through which an object can be reached by striking; the distance at which a force is effective when directed to a particular object.</cd><-- the distance which can be covered in one easy leg of a journey. --> -- <col><b>Striking plate</b></col>. <sd>(a)</sd> <cd>The plate against which the latch of a door lock strikes as the door is closed</cd>. <sd>(b)</sd> <cd>A part of the centering of an arch, which is driven back to loosen the centering in striking it.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Striking</ent><br/
<hw>Strik"ing</hw>, <pos>a.</pos> <def>Affecting with strong emotions; surprising; forcible; impressive; very noticeable; <as>as, a <ex>striking</ex> representation or image; a <ex>striking</ex> resemblance.</as></def> <ldquo/A <xex>striking</xex> fact.<rdquo/ <au>De Quincey.</au> -- <wordforms><wf>Strik"ing*ly</wf>, <pos>adv.</pos> -- <wf>Strik"ing*ness</wf>, <pos>n.</pos></wordforms><br/
[<source>1913 Webster</source>]</p>

<p><ent>Strikle</ent><br/
<hw>Strik"le</hw> <pr>(?)</pr>, <pos>n.</pos> <def>See <er>Strickle</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>String</ent><br/
<hw>String</hw> <pr>(str<icr/ng)</pr>, <pos>n.</pos> <ety>[OE. <ets>string</ets>, <ets>streng</ets>, AS. <ets>streng</ets>; akin to D. <ets>streng</ets>, G. <ets>strang</ets>, Icel. <ets>strengr</ets>, Sw. <ets>str<aum/ng</ets>, Dan. <ets>straeng</ets>; probably from the adj., E. <ets>strong</ets> (see <er>Strong</er>); or perhaps originally meaning, twisted, and akin to E. <ets>strangle</ets>.]</ety> <sn>1.</sn> <def>A small cord, a line, a twine, or a slender strip of leather, or other substance, used for binding together, fastening, or tying things; a cord, larger than a thread and smaller than a rope; <as>as, a shoe <ex>string</ex>; a bonnet <ex>string</ex>; a silken <ex>string</ex>.</as></def>  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Round Ormond's knee thou tiest the mystic <qex>string</qex>.</q> <rj><qau>Prior.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A thread or cord on which a number of objects or parts are strung or arranged in close and orderly succession; hence, a line or series of things arranged on a thread, or as if so arranged; a succession; a concatenation; a chain; <as>as, a <ex>string</ex> of shells or beads; a <ex>string</ex> of dried apples; a <ex>string</ex> of houses; a <ex>string</ex> of arguments.</as></def> <ldquo/A <xex>string</xex> of islands.<rdquo/  <rj><au>Gibbon.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>A strip, as of leather, by which the covers of a book are held together.</def>  <rj><au>Milton.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>The cord of a musical instrument, as of a piano, harp, or violin; specifically (<pluf>pl.</pluf>), the stringed instruments of an orchestra, in distinction from the wind instruments; <as>as, the <ex>strings</ex> took up the theme</as>.</def> <ldquo/An instrument of ten <xex>strings</xex>.<rdquo/  <rj><au>Ps. xxx. iii. 2.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Me softer airs befit, and softer <qex>strings</qex><br/
Of lute, or viol still.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <def>The line or cord of a bow.</def>  <rj><au>Ps. xi. 2.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>He twangs the grieving <qex>string</qex>.</q> <rj><qau>Pope.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>6.</sn> <def>A fiber, as of a plant; a little, fibrous root.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Duckweed putteth forth a little <qex>string</qex> into the water, from the bottom.</q> <rj><qau>Bacon.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>7.</sn> <def>A nerve or tendon of an animal body.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The <qex>string</qex> of his tongue was loosed.</q> <rj><qau>Mark vii. 35.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>8.</sn> <fld>(Shipbuilding)</fld> <def>An inside range of ceiling planks, corresponding to the sheer strake on the outside and bolted to it.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>9.</sn> <fld>(Bot.)</fld> <def>The tough fibrous substance that unites the valves of the pericap of leguminous plants, and which is readily pulled off; <as>as, the <ex>strings</ex> of beans</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>10.</sn> <fld>(Mining)</fld> <def>A small, filamentous ramification of a metallic vein.</def>  <rj><au>Ure.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>11.</sn> <fld>(Arch.)</fld> <def>Same as <er>Stringcourse</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>12.</sn> <fld>(Billiards)</fld> <def>The points made in a game.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>13.</sn> <sd>(a)</sd> <def>In various indoor games, a score or tally, sometimes, as in American billiard games, marked by buttons threaded on a string or wire.</def> <sd>(b)</sd> <def>In various games, competitions, etc., a certain number of turns at play, of rounds, etc.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><sn>14.</sn>  <fld>(Billiards & Pool)</fld> <sd>(a)</sd> <def>The line from behind and over which the cue ball must be played after being out of play as by being pocketed or knocked off the table; -- called also <altname>string line</altname>.</def> <sd>(b)</sd> <def>Act of stringing for break.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><sn>15.</sn>  <def>A hoax; a trumped-up or <ldquo/fake<rdquo/ story.</def> <mark>[Slang]</mark><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><sn>16.</sn>  <def>a sequence of similar objects or events sufficiently close in time or space to be perceived as a group; <as>a <ex>string</ex> of accidents; a <ex>string</ex> of restaurants on a highway</as>.</def><br/
[<source>PJC</source>]</p>

<p><sn>17.</sn> <fld>(Physics)</fld> <def>A one-dimensional string-like mathematical object used as a means of representing the properties of fundamental particles in <er>string theory</er>, one theory of particle physics; such hypothetical objects are one-dimensional and very small (10<exp>-33</exp> cm) but exist in more than four spatial dimensions, and have various modes of vibration.  Considering particles as <ex>strings</ex> avoids some of the problems of treating particles as points, and allows a unified treatment of gravity along with the other three forces (electromagnetism, the weak force, and the strong force) in a manner consistent with quantum mechanics.  See also <er>string theory</er>.</def><br/
[<source>PJC</source>]</p>

<p><cs><col><b>String band</b></col> <fld>(Mus.)</fld>, <cd>a band of musicians using only, or chiefly, stringed instruments.</cd> -- <col><b>String beans</b></col>. <sd>(a)</sd> <cd>A dish prepared from the unripe pods of several kinds of beans; -- so called because the strings are stripped off</cd>. <sd>(b)</sd> <cd>Any kind of beans in which the pods are used for cooking before the seeds are ripe; usually, the low bush bean.</cd> -- <col><b>To have two strings to one's bow</b></col>, <cd>to have a means or expedient in reserve in case the one employed fails.</cd></cs>
<-- = to have other strings in one's bow --><br/
[<source>1913 Webster</source>]</p>

<p><ent>String</ent><br/
<hw>String</hw> <pr>(str<icr/ng)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp.</pos> <conjf>Strung</conjf> <pr>(str<ucr/ng)</pr>; <pos>p. p.</pos> <conjf>Strung</conjf> (<mark>R.</mark> <conjf>Stringed</conjf> <pr>(str<icr/ngd)</pr>); <pos>p. pr. & vb. n.</pos> <conjf>Stringing</conjf>.]</vmorph> <sn>1.</sn> <def>To furnish with strings; <as>as, to <ex>string</ex> a violin</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Has not wise nature <qex>strung</qex> the legs and feet<br/
With firmest nerves, designed to walk the street?</q> <rj><qau>Gay.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To put in tune the strings of, as a stringed instrument, in order to play upon it.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>For here the Muse so oft her harp has <qex>strung</qex>,<br/
That not a mountain rears its head unsung.</q> <rj><qau>Addison.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>To put on a string; to file; <as>as, to <ex>string</ex> beads</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>To make tense; to strengthen.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Toil <qex>strung</qex> the nerves, and purified the blood.</q> <rj><qau>Dryden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <def>To deprive of strings; to strip the strings from; <as>as, to <ex>string</ex> beans</as>. See <er>String</er>, <pos>n.</pos>, 9.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>6.</sn> <def>To hoax; josh; jolly; often used with <ptcl>along</ptcl>; <as>as, we <ex>strung</ex> him <ex>along</ex> all day until he realized we were kidding</as>.</def> <mark>[Slang]</mark><br/
[<source>Webster 1913 Suppl.</source> <source>+PJC</source>]</p>

<p><ent>String</ent><br/
<hw>String</hw>, <pos>v. i.</pos> <def>To form into a string or strings, as a substance which is stretched, or people who are moving along, etc.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Stringboard</ent><br/
<hw>String"board`</hw> <pr>(-b<omac/rd`)</pr>, <pos>n.</pos> <def>Same as <er>Stringpiece</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stringcourse</ent><br/
<hw>String"course`</hw> <pr>(-k<omac/rs`)</pr>, <pos>n.</pos> <fld>(Arch.)</fld> <def>A horizontal band in a building, forming a part of the design, whether molded, projecting, or carved, or in any way distinguished from the rest of the work.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stringed</ent><br/
<hw>Stringed</hw> <pr>(str<icr/ngd)</pr>, <pos>a.</pos> <sn>1.</sn> <def>Having strings; <as>as, a <ex>stringed</ex> instrument</as>.</def>  <rj><au>Ps. cl. 4.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Produced by strings.</def> <ldquo/Answering the <xex>stringed</xex> noise.<rdquo/  <rj><au>Milton.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stringency</ent><br/
<hw>Strin"gen*cy</hw> <pr>(str<icr/n"j<eit/n*s<ycr/)</pr>, <pos>n.</pos> <def>The quality or state of being stringent.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stringendo</ent><br/
||<hw>Strin"gen*do</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[It.]</ety> <fld>(Mus.)</fld> <def>Urging or hastening the time, as to a climax.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stringent</ent><br/
<hw>Strin"gent</hw> <pr>(str<icr/n"j<eit/nt)</pr>, <pos>a.</pos> <ety>[L. <ets>stringens</ets>, <ets>-entis</ets>, p. pr. of <ets>stringere</ets> to draw or bind tight. See <er>Strain</er>.]</ety> <def>Binding strongly; making strict requirements; restrictive; rigid; severe; <as>as, <ex>stringent</ex> rules</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>They must be subject to a sharper penal code, and to a more <qex>stringent</qex> code of procedure.</q> <rj><qau>Macaulay.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p>-- <wordforms><wf>Strin"gent*ly</wf>, <pos>adv.</pos> -- <wf>Strin"gent*ness</wf>, <pos>n.</pos></wordforms><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stringer</ent><br/
<hw>String"er</hw> <pr>(str<icr/ng"<etil/r)</pr>, <pos>n.</pos> <sn>1.</sn> <def>One who strings; one who makes or provides strings, especially for bows.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Be content to put your trust in honest <qex>stringers</qex>.</q> <rj><qau>Ascham.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A libertine; a wencher.</def> <mark>[Obs.]</mark>  <rj><au>Beau. & Fl.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <fld>(Railroad)</fld> <def>A longitudinal sleeper.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <fld>(Shipbuilding)</fld> <def>A streak of planking carried round the inside of a vessel on the under side of the beams.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <fld>(Carp.)</fld> <def>A long horizontal timber to connect uprights in a frame, or to support a floor or the like.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>6.</sn> <fld>(Newspapers)</fld> <def>A reporter or correspondent who works for a news agency on a part-time basis, especially one covering local news for a newspaper published in a different area; -- called also <altname>string correspondent</altname>.</def><br/
[<source>PJC</source>]</p>

<p><sn>7.</sn> <fld>(Aviation)</fld> <def>a longitudinal supporting structure to reinforce the skin of an airplane fuselage.</def><br/
[<source>PJC</source>]</p>

<p><ent>Stringhalt</ent><br/
<hw>String"halt`</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Far.)</fld> <def>An habitual sudden twitching of the hinder leg of a horse, or an involuntary or convulsive contraction of the muscles that raise the hock.</def> <altsp>[Written also <asp>springhalt</asp>.]</altsp><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stringiness</ent><br/
<hw>String"i*ness</hw> <pr>(?)</pr>, <pos>n.</pos> <def>Quality of being stringy.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stringless</ent><br/
<hw>String"less</hw>, <pos>a.</pos> <def>Having no strings.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>His tongue is now a <qex>stringless</qex> instrument.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stringpiece</ent><br/
<hw>String"piece`</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Arch.)</fld> <sd>(a)</sd> <def>A long piece of timber, forming a margin or edge of any piece of construction; esp.:</def> <sd>(b)</sd> <def>One of the longitudinal pieces, supporting the treads and rises of a flight or run of stairs.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>string theory</ent><br/
<hw>string" the`o*ry</hw> <pr>(str<icr/ng" th<emac/`<osl/*r<ycr/)</pr>, <pos>n.</pos> <fld>(Physics)</fld> <def>A mathematical theory for describing the properties of fundamental particles, which represents the particles as one-dimensional string-like objects, which exist in the normal four dimensions of space-time plus additional dimensions, the total dimensions being ten, eleven, or twenty-six depending on the version of the theory.  The properties of fundamental particles in string theory and their manner of interaction with each other depend upon the modes of vibration of the strings{17}.  The attractiveness of this theory rests in part on its ability to provide a unified treatment of gravity as well as the three other basic forces of nature, in a manner consistent with quantum mechanics.  The great difficulty of doing the calculations required by the theory, however, has thus far (1999) made it impossible to calculate the observable properties, such as the mass, of known particles, such as the electron, proton, mesons, quarks, and neutron; thus there is as yet no experimental verification for the theory.  The most popular version of the theory depends on a mathematical property called supersymmetry, and the theory derived form this principle is properly called <altname>superstring theory</altname>, a term which is often used interchangeably with <ex>string theory</ex>.  See also string{17}.</def><br/
[<source>PJC</source>]</p>

<p><ent>Stringy</ent><br/
<hw>String"y</hw> <pr>(?)</pr>, <pos>a.</pos> <sn>1.</sn> <def>Consisting of strings, or small threads; fibrous; filamentous; <as>as, a <ex>stringy</ex> root</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Capable of being drawn into a string, as a glutinous substance; ropy; viscid; gluely.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Stringy bark</b></col> <fld>(Bot.)</fld>, <cd>a name given in Australia to several trees of the genus <gen>Eucalyptus</gen> (as <spn>Eucalyptus amygdalina</spn>, <spn>Eucalyptus obliqua</spn>, <spn>Eucalyptus capitellata</spn>, <spn>Eucalyptus macrorhyncha</spn>, <spn>Eucalyptus piperita</spn>, <spn>Eucalyptus pilularis</spn>, <and/ <spn>Eucalyptus tetradonta</spn>), which have a fibrous bark used by the aborigines for making cordage and cloth.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Strip</ent><br/
<hw>Strip</hw> <pr>(?)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Stripped</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Stripping</conjf>.]</vmorph> <ety>[OE. <ets>stripen</ets>, <ets>strepen</ets>, AS. <ets>str<?/pan</ets> in be<ets>str<?/pan</ets> to plunder; akin to D. <ets>stroopen</ets>, MHG. <ets>stroufen</ets>, G. <ets>streifen</ets>.]</ety> <sn>1.</sn> <def>To deprive; to bereave; to make destitute; to plunder; especially, to deprive of a covering; to skin; to peel; <as>as, to <ex>strip</ex> a man of his possession, his rights, his privileges, his reputation; to <ex>strip</ex> one of his clothes; to <ex>strip</ex> a beast of his skin; to <ex>strip</ex> a tree of its bark.</as></def><br/
[<source>1913 Webster</source>]</p>

<p><q>And <qex>strippen</qex> her out of her rude array.</q> <rj><qau>Chaucer.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>They <qex>stripped</qex> Joseph out of his coat.</q> <rj><qau>Gen. xxxvii. 23.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Opinions which . . . no clergyman could have avowed without imminent risk of being <qex>stripped</qex> of his gown.</q> <rj><qau>Macaulay.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To divest of clothing; to uncover.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Before the folk herself <qex>strippeth</qex> she.</q> <rj><qau>Chaucer.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q><qex>Strip</qex> your sword stark naked.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <fld>(Naut.)</fld> <def>To dismantle; <as>as, to <ex>strip</ex> a ship of rigging, spars, etc.</as></def><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <fld>(Agric.)</fld> <def>To pare off the surface of, as land, in strips.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <def>To deprive of all milk; to milk dry; to draw the last milk from; hence, to milk with a peculiar movement of the hand on the teats at the last of a milking; <as>as, to <ex>strip</ex> a cow</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>6.</sn> <def>To pass; to get clear of; to outstrip.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>When first they <qex>stripped</qex> the Malean promontory.</q> <rj><qau>Chapman.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Before he reached it he was out of breath,<br/
And then the other <qex>stripped</qex> him.</q> <rj><qau>Beau. & Fl.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>7.</sn> <def>To pull or tear off, as a covering; to remove; to wrest away; <as>as, to <ex>strip</ex> the skin from a beast; to <ex>strip</ex> the bark from a tree; to <ex>strip</ex> the clothes from a man's back; to <ex>strip</ex> away all disguisses.</as></def><br/
[<source>1913 Webster</source>]</p>

<p><q>To <qex>strip</qex> bad habits from a corrupted heart, is <qex>stripping</qex> off the skin.</q> <rj><qau>Gilpin.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>8.</sn> <fld>(Mach.)</fld> <sd>(a)</sd> <def>To tear off (the thread) from a bolt or nut; <as>as, the thread is <ex>stripped</ex></as>.</def> <sd>(b)</sd> <def>To tear off the thread from (a bolt or nut); <as>as, the bolt is <ex>stripped</ex></as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>9.</sn> <def>To remove the metal coating from (a plated article), as by acids or electrolytic action.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>10.</sn> <fld>(Carding)</fld> <def>To remove fiber, flock, or lint from; -- said of the teeth of a card when it becomes partly clogged.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>11.</sn> <def>To pick the cured leaves from the stalks of (tobacco) and tie them into <ldquo/hands<rdquo/; to remove the midrib from (tobacco leaves).</def><br/
[<source>1913 Webster</source>]</p>

<p><-- strip mine. A mine in which the unwanted layers (called the overburdewn) above the desirable ore is stripped, i.e. removed by excavation, leaving a pit in which the ore is exposed; in contrast with mines in which the ore is accessed and removed through a shaft or tunnel, without removing the layers of earth above it.  --><br/
[<source>1913 Webster</source>]</p>

<p><-- striptease, an act in which a performer (usu. female) removes her clothing piece by piece; -- often performed to musical accompaniment.  It was popular in burlesque theaters. --><br/
[<source>1913 Webster</source>]</p>

<p><-- p. 1427 --></p>

<p><ent>Strip</ent><br/
<hw>Strip</hw> <pr>(?)</pr>, <pos>v. i.</pos> <sn>1.</sn> <def>To take off, or become divested of, clothes or covering; to undress.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Mach.)</fld> <def>To fail in the thread; to lose the thread, as a bolt, screw, or nut. See <er>Strip</er>, <pos>v. t.</pos>, 8.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Strip</ent><br/
<hw>Strip</hw>, <pos>n.</pos> <sn>1.</sn> <def>A narrow piece, or one comparatively long; <as>as, a <ex>strip</ex> of cloth; a <ex>strip</ex> of land.</as></def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Mining)</fld> <def>A trough for washing ore.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <fld>(Gunnery)</fld> <def>The issuing of a projectile from a rifled gun without acquiring the spiral motion.</def>  <rj><au>Farrow.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stripe</ent><br/
<hw>Stripe</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[OD. <ets>strijpe</ets> a stripe, streak; akin to LG. <ets>stripe</ets>, D. <ets>streep</ets>, Dan. <ets>stribe</ets>, G. <ets>strief</ets>, <ets>striefen</ets>, MHG. <ets>striefen</ets> to glide, march.]</ety> <sn>1.</sn> <def>A line, or long, narrow division of anything of a different color or structure from the ground; hence, any linear variation of color or structure; <as>as, a <ex>stripe</ex>, or streak, of red on a green ground; a raised <ex>stripe</ex>.</as></def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Weaving)</fld> <def>A pattern produced by arranging the warp threads in sets of alternating colors, or in sets presenting some other contrast of appearance.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>A strip, or long, narrow piece attached to something of a different color; <as>as, a red or blue <ex>stripe</ex> sewed upon a garment</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>A stroke or blow made with a whip, rod, scourge, or the like, such as usually leaves a mark.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Forty <qex>stripes</qex> he may give him, and not exceed.</q> <rj><qau>Deut. xxv. 3.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <def>A long, narrow discoloration of the skin made by the blow of a lash, rod, or the like.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Cruelty marked him with inglorious <qex>stripes</qex>.</q> <rj><qau>Thomson.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>6.</sn> <def>Color indicating a party or faction; hence, distinguishing characteristic; sign; likeness; sort; <as>as, persons of the same political <ex>stripe</ex></as>.</def> <mark>[Colloq. U.S.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><sn>7.</sn> <pluf>pl.</pluf> <fld>(Mil.)</fld> <def>The chevron on the coat of a noncommissioned officer.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Stars and Stripes</b></col>. <cd>See under <er>Star</er>, <pos>n.</pos></cd></cs>
<-- To earn one's stripes, to acquire recognized credentials by competent performance at a germane task. --><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stripe</ent><br/
<hw>Stripe</hw>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Striped</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Striping</conjf>.]</vmorph> <sn>1.</sn> <def>To make stripes upon; to form with lines of different colors or textures; to variegate with stripes.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To strike; to lash.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Striped</ent><br/
<hw>Striped</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Having stripes of different colors; streaked.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Striped bass</b></col>. <fld>(Zool.)</fld> <cd>See under <er>Bass</er>.</cd> -- <col><b>Striped maple</b></col> <fld>(Bot.)</fld>, <cd>a slender American tree (<spn>Acer Pennsylvanicum</spn>) with finely striped bark. Called also <altname>striped dogwood</altname>, and <altname>moosewood</altname>.</cd> -- <col><b>Striped mullet</b></col>. <fld>(Zool.)</fld> <cd>See under <er>Mullet</er>, 2.</cd> -- <col><b>Striped snake</b></col> <fld>(Zool.)</fld>, <cd>the garter snake.</cd> -- <col><b>Striped squirrel</b></col> <fld>(Zool.)</fld>, <cd>the chipmunk.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Strip-leaf</ent><br/
<hw>Strip"-leaf`</hw> <pr>(?)</pr>, <pos>n.</pos> <def>Tobacco which has been stripped of its stalks before packing.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stripling</ent><br/
<hw>Strip"ling</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Dim. of <ets>strip</ets>; as if a small strip from the main stock or steam.]</ety> <def>A youth in the state of adolescence, or just passing from boyhood to manhood; a lad.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Inquire thou whose son the <qex>stripling</qex> is.</q> <rj><qau>1 Sam. xvii. 56.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stripper</ent><br/
<hw>Strip"per</hw> <pr>(?)</pr>, <pos>n.</pos> <sn>1.</sn> <def>One who, or that which, strips;</def> <specif>specifically,</specif> <def>a machine for stripping cards.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Agric.)</fld> <def>A cow that has nearly stopped giving milk, so that it can be obtained from her only by stripping.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><sn>3.</sn> <fld>(Agric.)</fld> <def>A harvesting machine that strips the seed from the stalks of grain plants.</def><br/
[<source>PJC</source>]</p>

<p><sn>4.</sn> <def>a solvent or chemical solution used to remove coatings of paint, varnish, lacquer, etc. prior to refinishing a surface.</def><br/
[<source>PJC</source>]</p>

<p><sn>5.</sn> <def>a person (usually a woman) who strips off her clothes as an entertainment performance; one who does a strip-tease; -- also called an <altname>ecdysiast</altname>, <altname>exotic dancer</altname>, or <altname>strip-teaser</altname>.</def><br/
[<source>PJC</source>]</p>

<p><ent>Strippet</ent><br/
<hw>Strip"pet</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Dim. of <ets>strip</ets>.]</ety> <def>A small stream.</def> <mark>[Obs.]</mark> <ldquo/A little brook or <xex>strippet</xex>.<rdquo/  <rj><au>Holinshed.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stripping</ent><br/
<hw>Strip"ping</hw> <pr>(?)</pr>, <pos>n.</pos> <sn>1.</sn> <def>The act of one who strips.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The mutual bows and courtesies . . . are remants of the original prostrations and <qex>strippings</qex> of the captive.</q> <rj><qau>H. Spencer.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Never were cows that required such <qex>stripping</qex>.</q> <rj><qau>Mrs. Gaskell.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <pluf>pl.</pluf> <def>The last milk drawn from a cow at a milking.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Strisores</ent><br/
||<hw>Stri*so"res</hw> <pr>(?)</pr>, <pos>n. pl.</pos> <ety>[NL.; cf. L. <ets>stridere</ets> to creak, whiz, buzz.]</ety> <fld>(Zool.)</fld> <def>A division of passerine birds including the humming birds, swifts, and goatsuckers. It is now generally considered an artificial group.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Strive</ent><br/
<hw>Strive</hw> <pr>(?)</pr>, <pos>v. i.</pos> <vmorph>[<pos>imp.</pos> <conjf>Strove</conjf> <pr>(?)</pr>; <pos>p. p.</pos> <conjf>Striven</conjf> <pr>(?)</pr> (<mark>Rarely</mark>, <conjf>Strove</conjf>); <pos>p. pr. & vb. n.</pos> <conjf>Striving</conjf>.]</vmorph> <ety>[OF. <ets>estriver</ets>; of Teutonic origin, and akin to G. <ets>streben</ets>, D. <ets>streven</ets>, Dan. <ets>straebe</ets>, Sw. <ets>str<aum/fva</ets>.  Cf. <er>Strife</er>.]</ety> <sn>1.</sn> <def>To make efforts; to use exertions; to endeavor with earnestness; to labor hard.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Was for this his ambition <qex>strove</qex><br/
To equal Caesar first, and after, Jove?</q> <rj><qau>Cowley.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To struggle in opposition; to be in contention or dispute; to contend; to contest; -- followed by <xex>against</xex> or <xex>with</xex> before the person or thing opposed; <as>as, <ex>strive</ex> against temptation; <ex>strive</ex> for the truth</as>.</def>  <rj><au>Chaucer.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>My Spirit shall not always <qex>strive</qex> with man.</q> <rj><qau>Gen. vi. 3.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Why dost thou <qex>strive</qex> against him?</q> <rj><qau>Job xxxiii. 13.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Now private pity <qex>strove</qex> with public hate,<br/
Reason with rage, and eloquence with fate.</q> <rj><qau>Denham.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>To vie; to compete; to be a rival.</def>  <rj><au>Chaucer.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>[Not] that sweet grove<br/
Of Daphne, by Orontes and the inspired<br/
Castalian spring, might with this paradise<br/
Of Eden <qex>strive</qex>.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><syn><b>Syn.</b> -- To contend; vie; struggle; endeavor; aim.</syn><br/
[<source>1913 Webster</source>]</p>

<p><ent>Strive</ent><br/
<hw>Strive</hw>, <pos>n.</pos> <sn>1.</sn> <def>An effort; a striving.</def> <mark>[R.]</mark>  <rj><au>Chapman.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Strife; contention.</def> <mark>[Obs.]</mark>  <rj><au>Wyclif (luke xxi. 9).</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Strived</ent><br/
<hw>Strived</hw> <pr>(?)</pr>, <mark>obs.</mark> <pos>p. p.</pos> <mord>of <er>Strive</er></mord>. <def>Striven.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Yea, so have I <qex>strived</qex> to preach the gospel.</q> <rj><qau>Rom. xv. 20.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Striven</ent><br/
<hw>Striv"en</hw> <pr>(?)</pr>, <def><pos>p. p.</pos> of <er>Strive</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Striver</ent><br/
<hw>Striv"er</hw> <pr>(?)</pr>, <pos>n.</pos> <def>One who strives.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Striving</ent><br/
<hw>Striv"ing</hw> <pr>(?)</pr>, <def><pos>a. & n.</pos> from <er>Strive</er>.</def> -- <wordforms><wf>Striv"ing*ly</wf>, <pos>adv.</pos></wordforms><br/
[<source>1913 Webster</source>]</p>

<p><ent>Strix</ent><br/
<hw>Strix</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>strix</ets>, <ets>strigis</ets>.]</ety> <fld>(Arch.)</fld> <def>One of the flutings of a column.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stroam</ent><br/
<hw>Stroam</hw> <pr>(?)</pr>, <pos>v. i.</pos> <ety>[Prov. E. <ets>strome</ets> to walk with long strides.]</ety> <sn>1.</sn> <def>To wander about idly and vacantly.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To take long strides in walking.</def> <mark>[Prov. Eng.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Strobila</ent><br/
||<hw>Stro*bi"la</hw> <pr>(?)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Strobilae</plw> <pr>(#)</pr>.</plu> <ety>[NL., fr. Gr. <?/ anything twisted, a pine cone.]</ety> <fld>(Zool.)</fld> <sd>(a)</sd> <def>A form of the larva of certain Discophora in a state of development succeeding the scyphistoma. The body of the strobila becomes elongated, and subdivides transversely into a series of lobate segments which eventually become ephyrae, or young medusae.</def> <sd>(b)</sd> <def>A mature tapeworm.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Strobilaceous</ent><br/
<hw>Strob`i*la"ceous</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[See <er>Strobila</er>.]</ety> <fld>(Bot.)</fld> <sd>(a)</sd> <def>Of or pertaining to a strobile or cone.</def> <sd>(b)</sd> <def>Producing strobiles.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Strobilation</ent><br/
<hw>Strob`i*la"tion</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Zool.)</fld> <def>The act or phenomenon of spontaneously dividing transversely, as do certain species of annelids and helminths; transverse fission. See <xex>Illust.</xex> under <er>Syllidian</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Strobile</ent><br/
<hw>Strob"ile</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>strobilus</ets> a pine cone, Gr. <?/: cf. F. <ets>strobole</ets>.]</ety> <altsp>[Written also <asp>strobil</asp>.]</altsp> <sn>1.</sn> <fld>(Bot.)</fld> <def>A scaly multiple fruit resulting from the ripening of an ament in certain plants, as the hop or pine; a cone. See <er>Cone</er>, <pos>n.</pos>, 3.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Biol.)</fld> <def>An individual asexually producing sexual individuals differing from itself also in other respects, as the tapeworm, -- one of the forms that occur in metagenesis.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <fld>(Zool.)</fld> <def>Same as <er>Strobila</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Strobiliform</ent><br/
<hw>Stro*bil"i*form</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Shaped like a strobile.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Strobiline</ent><br/
<hw>Strob"i*line</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Of or pertaining to a strobile; strobilaceous; strobiliform; <as>as, <ex>strobiline</ex> fruits</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stroboscope</ent><br/
<hw>Strob"o*scope</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Gr. <?/ a whirling + <ets>-scope</ets>.]</ety> <sn>1.</sn> <def>An instrument for studying or observing the successive phases of a periodic or varying motion by means of light which is periodically interrupted.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>An optical toy similar to the phenakistoscope. See <er>Phenakistoscope</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Strockle</ent><br/
<hw>Stroc"kle</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Glass Manuf.)</fld> <def>A shovel with a turned-up edge, for frit, sand, etc.</def> <altsp>[Written also <asp>strocal</asp>, <asp>strocle</asp>, <asp>strokal</asp>.]</altsp><br/
[<source>1913 Webster</source>]</p>

<p><ent>Strode</ent><br/
<hw>Strode</hw> <pr>(?)</pr>, <pos>n.</pos> <def>See <er>Strude</er>.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Strode</ent><br/
<hw>Strode</hw>, <def><pos>imp.</pos> of <er>Stride</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stroke</ent><br/
<hw>Stroke</hw> <pr>(?)</pr>, <mark>obs.</mark> <pos>imp.</pos> <mord>of <er>Strike</er></mord>. <def>Struck.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stroke</ent><br/
<hw>Stroke</hw>, <pos>n.</pos> <ety>[OE. <ets>strok</ets>, <ets>strook</ets>, <ets>strak</ets>, fr. <ets>striken</ets>. See <er>Strike</er>, <pos>v. t.</pos>]</ety> <sn>1.</sn> <def>The act of striking; a blow; a hit; a knock; esp., a violent or hostile attack made with the arm or hand, or with an instrument or weapon.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>His hand fetcheth a <qex>stroke</qex> with the ax to cut down the tree.</q> <rj><qau>Deut. xix. 5.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>A fool's lips enter into contention and his mouth calleth for <qex>strokes</qex>.</q> <rj><qau>Prov. xviii. 6.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>He entered and won the whole kingdom of Naples without striking a <qex>stroke</qex>.</q> <rj><qau>Bacon.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>The result of effect of a striking; injury or affliction; soreness.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>In the day that Lord bindeth up the breach of his people, and healeth the <qex>stroke</qex> of their wound.</q> <rj><qau>Isa. xxx. 26.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>The striking of the clock to tell the hour.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Well, but what's o'clock?<br/
- Upon the <qex>stroke</qex> of ten. -- Well, let is strike.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>A gentle, caressing touch or movement upon something; a stroking.</def>  <rj><au>Dryden.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <def>A mark or dash in writing or printing; a line; the touch of a pen or pencil; <as>as, an up <ex>stroke</ex>; a firm <ex>stroke</ex></as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>O, lasting as those colors may they shine,<br/
Free as thy <qex>stroke</qex>, yet faultless as thy line.</q> <rj><qau>Pope.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>6.</sn> <def>Hence, by extension, an addition or amandment to a written composition; a touch; <as>as, to give some finishing <ex>strokes</ex> to an essay</as>.</def>  <rj><au>Addison.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>7.</sn> <def>A sudden attack of disease; especially, a fatal attack; a severe disaster; any affliction or calamity, especially a sudden one; <as>as, a <ex>stroke</ex> of apoplexy; the <ex>stroke</ex> of death</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>At this one <qex>stroke</qex> the man looked dead in law.</q> <rj><qau>Harte.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>8.</sn> <def>A throb or beat, as of the heart.</def>  <rj><au>Tennyson.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>9.</sn> <def>One of a series of beats or movements against a resisting medium, by means of which movement through or upon it is accomplished; <as>as, the <ex>stroke</ex> of a bird's wing in flying, or an oar in rowing, of a skater, swimmer, etc.</as></def>; also: <fld>(Rowing)</fld> <sd>(a)</sd> <def>The rate of succession of stroke; <as>as, a quick <ex>stroke</ex></as>.</def> <sd>(b)</sd> <def>The oar nearest the stern of a boat, by which the other oars are guided; -- called also <altname>stroke oar</altname>.</def> <sd>(c)</sd> <def>The rower who pulls the stroke oar; the strokesman.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>10.</sn> <def>A powerful or sudden effort by which something is done, produced, or accomplished; also, something done or accomplished by such an effort; <as>as, a <ex>stroke</ex> of genius; a <ex>stroke</ex> of business; a master <ex>stroke</ex> of policy.</as></def><br/
[<source>1913 Webster</source>]</p>

<p><sn>11.</sn> <fld>(Mach.)</fld> <def>The movement, in either direction, of the piston plunger, piston rod, crosshead, etc., as of a steam engine or a pump, in which these parts have a reciprocating motion; <as>as, the forward <ex>stroke</ex> of a piston</as>; also, the entire distance passed through, as by a piston, in such a movement; <as>as, the piston is at half <ex>stroke</ex></as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ The respective strokes are distinguished as <xex>up</xex> and <xex>down</xex> strokes, <xex>outward</xex> and <xex>inward</xex> strokes, <xex>forward</xex> and <xex>back</xex> strokes, the forward stroke in stationary steam engines being toward the crosshead, but in locomotives toward the front of the vehicle.</note><br/
[<source>1913 Webster</source>]</p>

<p><sn>12.</sn> <def>Power; influence.</def> <mark>[Obs.]</mark> <ldquo/Where money beareth [hath] all the <xex>stroke</xex>.<rdquo/  <rj><au>Robynson (More's Utopia).</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>He has a great <qex>stroke</qex> with the reader.</q> <rj><qau>Dryden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>13.</sn> <def>Appetite.</def> <mark>[Obs.]</mark>  <rj><au>Swift.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>To keep stroke</b></col>, <cd>to make strokes in unison.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><q>The oars where silver,<br/
Which to the tune of flutes <qex>kept stroke</qex>.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stroke</ent><br/
<hw>Stroke</hw> <pr>(?)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Strokeed</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Strokeing</conjf>.]</vmorph> <ety>[OE. <ets>stroken</ets>, <ets>straken</ets>, AS. <ets>str<amac/cian</ets>, fr. <ets>str<imac/can</ets> to go over, pass. See <er>Strike</er>, <pos>v. t.</pos>, and cf. <er>Straggle</er>.]</ety> <sn>1.</sn> <def>To strike.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>Ye mote with the plat sword again<br/
<qex>Stroken</qex> him in the wound, and it will close.</q> <rj><qau>Chaucer.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To rib gently in one direction; especially, to pass the hand gently over by way of expressing kindness or tenderness; to caress; to soothe.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>He dried the falling drops, and, yet more kind,<br/
He <qex>stroked</qex> her cheeks.</q> <rj><qau>Dryden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>To make smooth by rubbing.</def>  <rj><au>Longfellow.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <fld>(Masonry)</fld> <def>To give a finely fluted surface to.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <def>To row the stroke oar of; <as>as, to <ex>stroke</ex> a boat</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stroker</ent><br/
<hw>Strok"er</hw> <pr>(?)</pr>, <pos>n.</pos> <def>One who strokes; also, one who pretends to cure by stroking.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Cures worked by Greatrix the <qex>stroker</qex>.</q> <rj><qau>Bp. Warburton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Strokesman</ent><br/
<hw>Strokes"man</hw> <pr>(?)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Strokesman</plw> <pr>(<?/)</pr>.</plu> <fld>(Rowing)</fld> <def>The man who rows the aftermost oar, and whose stroke is to be followed by the rest.</def>  <rj><au>Totten.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stroking</ent><br/
<hw>Strok"ing</hw> <pr>(?)</pr>, <pos>n.</pos> <sn>1.</sn> <def>The act of rubbing gently with the hand, or of smoothing; a stroke.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>I doubt not with one gentle <qex>stroking</qex> to wipe away ten thousand tears.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Needlework)</fld> <def>The act of laying small gathers in cloth in regular order.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <pluf>pl.</pluf> <def>See <er>Stripping</er>, 2.</def>  <rj><au>Smollett.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stroll</ent><br/
<hw>Stroll</hw> <pr>(?)</pr>, <pos>v. i.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Strolled</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Strolling</conjf>.]</vmorph> <ety>[Cf. Dan. <ets>stryge</ets> to stroll, Sw. <ets>stryka</ets> to stroke, to ramble, dial. Sw. <ets>strykel</ets> one who strolls about, Icel. <ets>strj<?/ka</ets> to stroke, D. <ets>struikelen</ets> to stumble, G. <ets>straucheln</ets>.  Cf. <er>Struggle</er>.]</ety> <def>To wander on foot; to ramble idly or leisurely; to rove.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>These mothers <qex>stroll</qex> to beg sustenance for their helpless infants.</q> <rj><qau>Swift.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><syn><b>Syn.</b> -- To rove; roam; range; stray.</syn><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stroll</ent><br/
<hw>Stroll</hw>, <pos>n.</pos> <def>A wandering on foot; an idle and leisurely walk; a ramble.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stroller</ent><br/
<hw>Stroll"er</hw> <pr>(?)</pr>, <pos>n.</pos> <def>One who strolls; a vagrant.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stroma</ent><br/
||<hw>Stro"ma</hw> <pr>(?)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Stromata</plw> <pr>(#)</pr>.</plu> <ety>[L., a bed covering, Gr. <?/ a couch or bed.]</ety> <sn>1.</sn> <fld>(Anat.)</fld> <sd>(a)</sd> <def>The connective tissue or supporting framework of an organ; <as>as, the <ex>stroma</ex> of the kidney</as>.</def> <sd>(b)</sd> <def>The spongy, colorless framework of a red blood corpuscle or other cell.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Bot.)</fld> <def>A layer or mass of cellular tissue, especially that part of the thallus of certain fungi which incloses the perithecia.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stromatic</ent><br/
<hw>Stro*mat"ic</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[Gr. <?/ coverlet of a bed, pl. <?/ patchwork (for such a coverlet), also applied to several <ets>miscellaneous</ets> writings, fr. <?/ anything spread out for resting upon, a bed, fr. <?/ to spread out.]</ety> <def>Miscellaneous; composed of different kinds.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stromatology</ent><br/
<hw>Stro`ma*tol"o*gy</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Gr. <?/, <?/, a bed + <ets>-logy</ets>.]</ety> <fld>(Geol.)</fld> <def>The history of the formation of stratified rocks.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stromb</ent><br/
<hw>Stromb</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Zool.)</fld> <def>Any marine univalve mollusk of the genus Strombus and allied genera. See <er>Conch</er>, and <er>Strombus</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Strombite</ent><br/
<hw>Strom"bite</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Paleon.)</fld> <def>A fossil shell of the genus Strombus.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stromboid</ent><br/
<hw>Strom"boid</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[<ets>Strombus</ets> + <ets>-oid</ets>.]</ety> <fld>(Zool.)</fld> <def>Of, pertaining to, or like, Strombus.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Strombuliform</ent><br/
<hw>Strom*bu"li*form</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[NL. <ets>strombulus</ets>, dim. of <ets>strombus + -form</ets>. See <er>Strombus</er>.]</ety> <sn>1.</sn> <fld>(Geol.)</fld> <def>Formed or shaped like a top.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Bot.)</fld> <def>Coiled into the shape of a screw or a helix.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Strombus</ent><br/
||<hw>Strom"bus</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L., fr. Gr. <?/.]</ety> <fld>(Zool.)</fld> <def>A genus of marine gastropods in which the shell has the outer lip dilated into a broad wing. It includes many large and handsome species commonly called <altname>conch shells</altname>, or <altname>conchs</altname>. See <er>Conch</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stromeyerite</ent><br/
<hw>Stro"mey`er*ite</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[So named from the German chemist Friedrich <ets>Stromeyer</ets>.]</ety> <fld>(Min.)</fld> <def>A steel-gray mineral of metallic luster. It is a sulphide of silver and copper.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Strond</ent><br/
<hw>Strond</hw> <pr>(?)</pr>, <pos>n.</pos> <def>Strand; beach.</def> <mark>[Obs.]</mark>  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><-- p. 1428 --></p>

<p><ent>Strong</ent><br/
<hw>Strong</hw> <pr>(?)</pr>, <pos>a.</pos> <amorph>[<pos>Compar.</pos> <adjf>Stronger</adjf> <pr>(?)</pr>; <pos>superl.</pos> <adjf>Strongest</adjf> <pr>(?)</pr>.]</amorph> <ety>[AS. <ets>strang</ets>, <ets>strong</ets>; akin to D. & G. <ets>streng</ets> strict, rigorous, OHG. <ets>strengi</ets> strong, brave, harsh, Icel. <ets>strangr</ets> strong, severe, Dan. <ets>streng</ets>, Sw. <ets>str<aum/ng</ets> strict, severe.  Cf. <er>Strength</er>, <er>Stretch</er>, <er>String</er>.]</ety> <sn>1.</sn> <def>Having active physical power, or great physical power to act; having a power of exerting great bodily force; vigorous.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>That our oxen may be <qex>strong</qex> to labor.</q> <rj><qau>Ps. cxliv. 14.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Orses the <qex>strong</qex> to greater strength must yield.</q> <rj><qau>Dryden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Having passive physical power; having ability to bear or endure; firm; hale; sound; robust; <as>as, a <ex>strong</ex> constitution; <ex>strong</ex> health.</as></def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>Solid; tough; not easily broken or injured; able to withstand violence; able to sustain attacks; not easily subdued or taken; <as>as, a <ex>strong</ex> beam; a <ex>strong</ex> rock; a <ex>strong</ex> fortress or town.</as></def><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>Having great military or naval force; powerful; <as>as, a <ex>strong</ex> army or fleet; a nation <ex>strong</ex> at sea.</as></def><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <def>Having great wealth, means, or resources; <as>as, a <ex>strong</ex> house, or company of merchants</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>6.</sn> <def>Reaching a certain degree or limit in respect to strength or numbers; <as>as, an army ten thousand <ex>strong</ex></as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>7.</sn> <def>Moving with rapidity or force; violent; forcible; impetuous; <as>as, a <ex>strong</ex> current of water or wind; the wind was <ex>strong</ex> from the northeast; a <ex>strong</ex> tide.</as></def><br/
[<source>1913 Webster</source>]</p>

<p><sn>8.</sn> <def>Adapted to make a deep or effectual impression on the mind or imagination; striking or superior of the kind; powerful; forcible; cogent; <as>as, a <ex>strong</ex> argument; <ex>strong</ex> reasons; <ex>strong</ex> evidence; a <ex>strong</ex> example; <ex>strong</ex> language.</as></def><br/
[<source>1913 Webster</source>]</p>

<p><sn>9.</sn> <def>Ardent; eager; zealous; earnestly engaged; <as>as, a <ex>strong</ex> partisan; a <ex>strong</ex> Whig or Tory.</as></def><br/
[<source>1913 Webster</source>]</p>

<p><q>Her mother, ever <qex>strong</qex> against that match.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>10.</sn> <def>Having virtues of great efficacy; or, having a particular quality in a great degree; <as>as, a <ex>strong</ex> powder or tincture; a <ex>strong</ex> decoction; <ex>strong</ex> tea or coffee.</as></def><br/
[<source>1913 Webster</source>]</p>

<p><sn>11.</sn> <def>Full of spirit; containing a large proportion of alcohol; intoxicating; <as>as, <ex>strong</ex> liquors</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>12.</sn> <def>Affecting any sense powerfully; <as>as, <ex>strong</ex> light, colors, etc.; a <ex>strong</ex> flavor of onions; a <ex>strong</ex> scent.</as></def><br/
[<source>1913 Webster</source>]</p>

<p><sn>13.</sn> <def>Solid; nourishing; <as>as, <ex>strong</ex> meat</as>.</def>  <rj><au>Heb. v. 12.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>14.</sn> <def>Well established; firm; not easily overthrown or altered; <as>as, a <ex>strong</ex> custom; a <ex>strong</ex> belief.</as></def><br/
[<source>1913 Webster</source>]</p>

<p><sn>15.</sn> <def>Violent; vehement; earnest; ardent.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>He had offered up prayers and supplications with <qex>strong</qex> crying and tears.</q> <rj><qau>Heb. v. 7.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>16.</sn> <def>Having great force, vigor, power, or the like, as the mind, intellect, or any faculty; <as>as, a man of a <ex>strong</ex> mind, memory, judgment, or imagination</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>I was <qex>stronger</qex> in prophecy than in criticism.</q> <rj><qau>Dryden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>17.</sn> <def>Vigorous; effective; forcible; powerful.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Like her sweet voice is thy harmonious song,<br/
As high, as sweet, as easy, and as <qex>strong</qex>.</q> <rj><qau>E. Smith.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>18.</sn> <fld>(Stock Exchange)</fld> <def>Tending to higher prices; rising; <as>as, a <ex>strong</ex> market</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>19.</sn> <fld>(Gram.)</fld> <sd>(a)</sd> <def>Pertaining to, or designating, a verb which forms its preterit (imperfect) by a variation in the root vowel, and the past participle (usually) by the addition of <xex>-en</xex> (with or without a change of the root vowel); as in the verbs <xex>strive</xex>, <xex>strove</xex>, <xex>striven</xex>; <xex>break</xex>, <xex>broke</xex>, <xex>broken</xex>; <xex>drink</xex>, <xex>drank</xex>, <xex>drunk</xex>. Opposed to <xex>weak</xex>, or <xex>regular</xex>. See <er>Weak</er>.</def> <sd>(b)</sd> <def>Applied to forms in Anglo-Saxon, etc., which retain the old declensional endings. In the Teutonic languages the vowel stems have held the original endings most firmly, and are called <xex>strong</xex>; the stems in <xex>-n</xex> are called <xex>weak</xex> other constant stems conform, or are irregular.</def>  <rj><au>F. A. March.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Strong conjugation</b></col> <fld>(Gram.)</fld>, <cd>the conjugation of a strong verb; -- called also <altname>old conjugation</altname>, or <altname>irregular conjugation</altname>, and distinguished from the <contr>weak conjugation</contr> or <contr>regular conjugation</contr>.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ <xex>Strong</xex> is often used in the formation of self-explaining compounds; as, <xex>strong</xex>-backed, <xex>strong</xex>-based, <xex>strong</xex>-bodied, <xex>strong</xex>-colored, <xex>strong</xex>-fisted, <xex>strong</xex>-handed, <xex>strong</xex>-ribbed, <xex>strong</xex>-smelling, <xex>strong</xex>-voiced, etc.</note><br/
[<source>1913 Webster</source>]</p>

<p><syn><b>Syn.</b> -- Vigorous; powerful; stout; solid; firm; hardy; muscular; forcible; cogent; valid. See <er>Robust</er>.</syn><br/
[<source>1913 Webster</source>]</p>

<p><ent>strongbox</ent><br/
<hw>strong"box`</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A box of rigid and durable construction fitted with a lock, used for the purpose of protecting valuable items, such as jewelry or money.</def><br/
<syn><b>Syn. --</b> lockbox.</syn><br/
[<source>PJC</source>]</p>

<p><ent>Stronghand</ent><br/
<hw>Strong"hand`</hw> <pr>(?)</pr>, <pos>n.</pos> <def>Violence; force; power.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>It was their meaning to take what they needed by <qex>stronghand</qex>.</q> <rj><qau>Sir W. Raleigh.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stronghold</ent><br/
<hw>Strong"hold`</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A fastness; a fort or fortress; fortfield place; a place of security.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Strongish</ent><br/
<hw>Strong"ish</hw>, <pos>a.</pos> <def>Somewhat strong.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Strongly</ent><br/
<hw>Strong"ly</hw>, <pos>adv.</pos> <def>In a strong manner; so as to be strong in action or in resistance; with strength; with great force; forcibly; powerfully; firmly; vehemently; <as>as, a town <ex>strongly</ex> fortified; he objected <ex>strongly</ex>.</as></def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Strong-minded</ent><br/
<hw>Strong"-mind`ed</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Having a vigorous mind; esp., having or affecting masculine qualities of mind; -- said of women.</def> -- <wordforms><wf>Strong"-mind`ed*ness</wf>, <pos>n.</pos></wordforms><br/
[<source>1913 Webster</source>]</p>

<p><ent>Strong-water</ent><br/
<hw>Strong"-wa`ter</hw> <pr>(?)</pr>, <pos>n.</pos> <sn>1.</sn> <def>An acid.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Distilled or ardent spirits; intoxicating liquor.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Strongylid</ent><br/
<hw>Stron"gy*lid</hw> <pr>(?)</pr>, <pos>a. & n.</pos> <fld>(Zool.)</fld> <def>Strongyloid.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Strongyloid</ent><br/
<hw>Stron"gy*loid</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[NL. <ets>Strongylus</ets> the genus (from Gr. <?/ round) + <ets>-oid</ets>.]</ety> <fld>(Zool.)</fld> <def>Like, or pertaining to, <gen>Strongylus</gen>, a genus of parasitic nematode worms of which many species infest domestic animals. Some of the species, especially those living in the kidneys, lungs, and bronchial tubes, are often very injurious.</def> -- <def2><pos>n.</pos> <def>A strongyloid worm.</def></def2><br/
[<source>1913 Webster</source>]</p>

<p><ent>Strontia</ent><br/
<hw>Stron"ti*a</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[NL. <ets>strontia</ets>, fr. <ets>Strontian</ets>, in Argyleshire, Scotland, where <ets>strontianite</ets> was first found.]</ety> <fld>(Chem.)</fld> <def>An earth of a white color resembling lime in appearance, and baryta in many of its properties. It is an oxide of the metal strontium.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Strontian</ent><br/
<hw>Stron"ti*an</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Min.)</fld> <def>Strontia.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Strontianite</ent><br/
<hw>Stron"ti*an*ite</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Min.)</fld> <def>Strontium carbonate, a mineral of a white, greenish, or yellowish color, usually occurring in fibrous massive forms, but sometimes in prismatic crystals.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Strontic</ent><br/
<hw>Stron"tic</hw> <pr>(?)</pr>, <pos>a.</pos> <fld>(Chem.)</fld> <def>Of or pertaining to strontium; containing, or designating the compounds of, strontium.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Strontitic</ent><br/
<hw>Stron*tit"ic</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Strontic.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Strontium</ent><br/
<hw>Stron"ti*um</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[NL. See <er>Strontia</er>.]</ety> <fld>(Chem.)</fld> <def>A metallic element of the calcium group, always naturally occurring combined, as in the minerals strontianite, celestite, etc. It is isolated as a yellowish metal, somewhat malleable but harder than calcium. It is chiefly employed (as in the nitrate) to color pyrotechnic flames red. Symbol Sr. Atomic weight 87.3.</def><br/
[<source>1913 Webster</source>]</p>

<p><-- Strontium-90.  <def>A radioactive isotope of strontium produced by certain nuclear reactions, and constituting one of the prominent harmful components of radioactive fallout from nuclear explosions; also called <altname>radiostrontium</altname>. It has a half-life of 28 years.</def>
 --><br/
[<source>1913 Webster</source>]</p>

<p><ent>Strook</ent><br/
<hw>Strook</hw> <pr>(?)</pr>, <mark>obs.</mark> <def><pos>imp.</pos> of <er>Strike</er>.</def>  <rj><au>Dryden.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Strook</ent><br/
<hw>Strook</hw>, <pos>n.</pos> <def>A stroke.</def> <mark>[Obs.]</mark>  <rj><au>Chaucer.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stroot</ent><br/
<hw>Stroot</hw> <pr>(?)</pr>, <pos>v. t.</pos> <ety>[Cf. <er>Strut</er>, <pos>v. i.</pos>]</ety> <def>To swell out; to strut.</def> <mark>[Obs.]</mark>  <rj><au>Chapman.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Strop</ent><br/
<hw>Strop</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[See <er>Strap</er>.]</ety> <def>A strap; specifically, same as <er>Strap</er>, 3.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Strop</ent><br/
<hw>Strop</hw>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Stropped</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Stropping</conjf>.]</vmorph> <def>To draw over, or rub upon, a strop with a view to sharpen; <as>as, to <ex>strop</ex> a razor</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Strop</ent><br/
<hw>Strop</hw>, <pos>n.</pos> <ety>[Cf. F. <ets>estrope</ets>, <ets><eacute/trope</ets>, fr. L. <ets>struppus</ets>. See <er>Strop</er> a strap.]</ety> <fld>(Naut.)</fld> <def>A piece of rope spliced into a circular wreath, and put round a block for hanging it.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Strophanthus</ent><br/
||<hw>Stro*phan"thus</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[NL., from Gr. <?/ a turning + <?/ a flower.]</ety> <fld>(Bot.)</fld> <def>A genus of tropical apocynaceous shrubs having singularly twisted flowers. One species (<spn>Strophanthus hispidus</spn>) is used medicinally as a cardiac sedative and stimulant.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Strophe</ent><br/
<hw>Stro"phe</hw> <pr>(?)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Strophes</plw> <pr>(#)</pr>.</plu> <ety>[NL., from Gr. <?/, fr. <?/ to twist, to turn; perh. akin to E. <ets>strap</ets>.]</ety> <def>In Greek choruses and dances, the movement of the chorus while turning from the right to the left of the orchestra; hence, the strain, or part of the choral ode, sung during this movement. Also sometimes used of a stanza of modern verse. See the Note under <er>Antistrophe</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Strophic</ent><br/
<hw>Stroph"ic</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Pertaining to, containing, or consisting of, strophes.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Strophiolated</ent><br/
<ent>Strophiolate</ent><br/
<mhw>{ <hw>Stro"phi*o*late</hw> <pr>(?)</pr>, <hw>Stro"phi*o*la`ted</hw> <pr>(?)</pr>, }</mhw> <pos>a.</pos> <fld>(Bot.)</fld> <def>Furnished with a strophiole, or caruncle, or that which resembles it.</def>  <rj><au>Gray.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Strophiole</ent><br/
<hw>Stro"phi*ole</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>strophiolum</ets> a little chaplet, dim. of <ets>strophium</ets> a band, Gr. <?/, dim. of <?/ a twisted band: cf. F. <ets>strophiole</ets>.]</ety> <fld>(Bot.)</fld> <def>A crestlike excrescence about the hilum of certain seeds; a caruncle.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Strophulus</ent><br/
||<hw>Stroph"u*lus</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[NL.]</ety> <fld>(Med.)</fld> <def>See <er>Red-gum</er>, 1.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stroud</ent><br/
<hw>Stroud</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A kind of coarse blanket or garment used by the North American Indians.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Strouding</ent><br/
<hw>Stroud"ing</hw>, <pos>n.</pos> <def>Material for strouds; a kind of coarse cloth used in trade with the North American Indians.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Strout</ent><br/
<hw>Strout</hw> <pr>(?)</pr>, <pos>v. i.</pos> <ety>[See <er>Strut</er>.]</ety> <def>To swell; to puff out; to project.</def> <mark>[Obs.]</mark>  <rj><au>Chaucer.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Strout</ent><br/
<hw>Strout</hw>, <pos>v. t.</pos> <def>To cause to project or swell out; to enlarge affectedly; to strut.</def> <mark>[Obs.]</mark>  <rj><au>Bacon.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Strove</ent><br/
<hw>Strove</hw> <pr>(?)</pr>, <def><pos>imp.</pos> of <er>Strive</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Strow</ent><br/
<hw>Strow</hw> <pr>(?)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp.</pos> <conjf>Strowed</conjf> <pr>(?)</pr>; <pos>p. p.</pos> <conjf>Strown</conjf> <pr>(?)</pr> <or/ <conjf>Strowed</conjf>.]</vmorph> <def>Same as <er>Strew</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Thick as autumnal leaves that <qex>strow</qex> the brooks<br/
In Vallombrosa.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>A manner turbid . . . and <qex>strown</qex> with blemished.</q> <rj><qau>M. Arnold.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Strowl</ent><br/
<hw>Strowl</hw> <pr>(?)</pr>, <pos>v. i.</pos> <def>To stroll.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Strown</ent><br/
<hw>Strown</hw> <pr>(?)</pr>, <def><pos>p. p.</pos> of <er>Strow</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stroy</ent><br/
<hw>Stroy</hw> <pr>(?)</pr>, <pos>v. i.</pos> <def>To destroy.</def> <mark>[Obs.]</mark>  <rj><au>Tusser.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Struck</ent><br/
<hw>Struck</hw> <pr>(?)</pr>, <def><pos>imp. & p. p.</pos> of <er>Strike</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Struck jury</b></col> <fld>(Law)</fld>, <cd>a special jury, composed of persons having special knowledge or qualifications, selected by striking from the panel of jurors a certain number for each party, leaving the number required by law to try the cause.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Strucken</ent><br/
<hw>Struck"en</hw> <pr>(?)</pr>, <mark>obs.</mark> <def><pos>p. p.</pos> of <er>Strike</er>.</def>  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Structural</ent><br/
<hw>Struc"tur*al</hw> <pr>(?)</pr>, <pos>a.</pos> <sn>1.</sn> <def>Of or pertaining to structure; affecting structure; <as>as, a <ex>structural</ex> error</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Biol.)</fld> <def>Of or pertaining to organit structure; <as>as, a <ex>structural</ex> element or cell; the <ex>structural</ex> peculiarities of an animal or a plant.</as></def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Structural formula</ent><br/
<hw>Struc"tur*al form"u*la</hw>. <fld>(Chem.)</fld> <def>a symbolic representation of the structure of one molecule of a chemical compound, showing the attachments of the atoms to each other, and representing their arrangement in space; the spatial representation is symbolic and approximate, and it may or may not depict the stereochemical relations of the bonds.  Distinguished from <cref>empirical formula</cref>. See also <cref>Rational formula</cref>, under <er>Formula</er>.</def><br/
[<source>PJC</source>]</p>

<p><ent>Structural shape</ent><br/
<hw>Struc"tur*al shape</hw>. <fld>(Engin. & Arch.)</fld> <def>The shape of a member especially adapted to structural purposes, esp. in giving the greatest strength with the least material.</def> <specif>Hence,</specif> <mark>Colloq.</mark>, <def>any steel or iron member of such shape, as channel irons, <universbold>I</universbold> beams, <universbold>T</universbold> beams, etc., or, sometimes, a column, girder, etc., built up with such members.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Structural steel</ent><br/
<hw>Structural steel</hw>. <sd>(a)</sd> <def>Rolled steel in structural shapes.</def> <sd>(b)</sd>  <def>A kind of strong mild steel, suitable for structural shapes.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Structure</ent><br/
<hw>Struc"ture</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>structura</ets>, from <ets>struere</ets>, <ets>structum</ets>, to arrange, build, construct; perhaps akin to E. <ets>strew</ets>: cf. F. <ets>structure</ets>.  Cf. <er>Construe</er>, <er>Destroy</er>, <er>Instrument</er>, <er>Obstruct</er>.]</ety> <sn>1.</sn> <def>The act of building; the practice of erecting buildings; construction.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>His son builds on, and never is content<br/
Till the last farthing is in <qex>structure</qex> spent.</q> <rj><qau>J. Dryden, Jr.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Manner of building; form; make; construction.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Want of insight into the <qex>structure</qex> and constitution of the terraqueous globe.</q> <rj><qau>Woodward.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>Arrangement of parts, of organs, or of constituent particles, in a substance or body; <as>as, the <ex>structure</ex> of a rock or a mineral; the <ex>structure</ex> of a sentence</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>It [basalt] has often a prismatic <qex>structure</qex>.</q> <rj><qau>Dana.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <fld>(Biol.)</fld> <def>Manner of organization; the arrangement of the different tissues or parts of animal and vegetable organisms; <as>as, organic <ex>structure</ex>, or the <ex>structure</ex> of animals and plants; cellular <ex>structure</ex>.</as></def><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <def>That which is built; a building; esp., a building of some size or magnificence; an edifice.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>There stands a <qex>structure</qex> of majestic frame.</q> <rj><qau>Pope.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Columnar structure</b></col>. <cd>See under <er>Columnar</er>.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Structured</ent><br/
<hw>Struc"tured</hw> <pr>(?)</pr>, <pos>a.</pos> <fld>(Biol.)</fld> <def>Having a definite organic structure; showing differentiation of parts.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The passage from a structureless state to a <qex>structured</qex> state is itself a vital process.</q> <rj><qau>H. Spencer.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Structureless</ent><br/
<hw>Struc"ture*less</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Without a definite structure, or arrangement of parts; without organization; devoid of cells; homogeneous; <as>as, a <ex>structureless</ex> membrane</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Structurist</ent><br/
<hw>Struc"tur*ist</hw> <pr>(?)</pr>, <pos>n.</pos> <def>One who forms structures; a builder; a constructor.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Strude</ent><br/
<hw>Strude</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A stock of breeding mares.</def> <altsp>[Written also <asp>strode</asp>.]</altsp> <mark>[Obs.]</mark>  <rj><au>Bailey.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Struggle</ent><br/
<hw>Strug"gle</hw> <pr>(?)</pr>, <pos>v. i.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Struggled</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Struggling</conjf> <pr>(?)</pr>.]</vmorph> <ety>[OE. <ets>strogelen</ets>; cf. Icel. <ets>strj<?/ka</ets> to stroke, to beat, to flog, Sw. <ets>stryka</ets> to stroke, to strike, Dan. <ets>stryge</ets>, G. <ets>straucheln</ets> to stumble.  Cf. <er>Stroll</er>.]</ety> <sn>1.</sn> <def>To strive, or to make efforts, with a twisting, or with contortions of the body.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To use great efforts; to labor hard; to strive; to contend forcibly; <as>as, to <ex>struggle</ex> to save one's life; to <ex>struggle</ex> with the waves; to <ex>struggle</ex> with adversity</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The brave men, living and dead, who <qex>struggled</qex> here, have consecrated it [Gettysburg] far above our power to add or detract.</q> <rj><qau>Lincoln.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>To labor in pain or anguish; to be in agony; to labor in any kind of difficulty or distress.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>'T is wisdom to beware,<br/
And better shun the bait than <qex>struggle</qex> in the snare.</q> <rj><qau>Dryden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><syn><b>Syn.</b> -- To strive; contend; labor; endeavor.</syn><br/
[<source>1913 Webster</source>]</p>

<p><ent>Struggle</ent><br/
<hw>Strug"gle</hw> <pr>(?)</pr>, <pos>n.</pos> <sn>1.</sn> <def>A violent effort or efforts with contortions of the body; agony; distress.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Great labor; forcible effort to obtain an object, or to avert an evil.</def>  <rj><au>Macaulay.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>Contest; contention; strife.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>An honest might look upon the <qex>struggle</qex> with indifference.</q> <rj><qau>Addison.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><syn><b>Syn.</b> -- Endeavor; effort; contest; labor; difficulty.</syn><br/
[<source>1913 Webster</source>]</p>

<p><ent>Struggler</ent><br/
<hw>Strug"gler</hw> <pr>(?)</pr>, <pos>n.</pos> <def>One who struggles.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Strull</ent><br/
<hw>Strull</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A bar so placed as to resist weight.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Strum</ent><br/
<hw>Strum</hw> <pr>(?)</pr>, <pos>v. t. & i.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Strummed</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Strumming</conjf>.]</vmorph> <ety>[Probably of imitative origin.  Cf. <er>Thrum</er>.]</ety> <def>To play on an instrument of music, or as on an instrument, in an unskillful or noisy way; to thrum; <as>as, to <ex>strum</ex> a piano</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Struma</ent><br/
||<hw>Stru"ma</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L., a scrofulous tumor.]</ety> <sn>1.</sn> <fld>(Med.)</fld> <def>Scrofula.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Bot.)</fld> <def>A cushionlike swelling on any organ; especially, that at the base of the capsule in many mosses.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Strumatic</ent><br/
<hw>Stru*mat"ic</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Scrofulous; strumous.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Strumose</ent><br/
<hw>Stru*mose"</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>strumosus</ets>: cf. F. <ets>strumeux</ets>.]</ety> <sn>1.</sn> <fld>(Med.)</fld> <def>Strumous.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Bot.)</fld> <def>Having a struma.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Strumous</ent><br/
<hw>Stru"mous</hw> <pr>(?)</pr>, <pos>a.</pos> <fld>(Med.)</fld> <def>Scrofulous; having struma.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Strumousness</ent><br/
<hw>Stru"mous*ness</hw>, <pos>n.</pos> <def>The state of being strumous.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Strumpet</ent><br/
<hw>Strum"pet</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[OE. <ets>strumpet</ets>, <ets>strompet</ets>; cf. OF. <ets>stupe</ets> debauchery, F. <ets>stupe</ets>, L. <ets>stuprare</ets>, <ets>stupratum</ets>, to debauch, <ets>stuprum</ets> debauchery, Gael. & Ir. <ets>striopach</ets> a prostitute.]</ety> <def>A prostitute; a harlot.</def>  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Strumpet</ent><br/
<hw>Strum"pet</hw>, <pos>a.</pos> <def>Of or pertaining to a strumpet; characteristic of a strumpet.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Out on thy more than <qex>strumpet</qex> impudence.</q> <rj><qau>B. Jonson.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Strumpet</ent><br/
<hw>Strum"pet</hw>, <pos>v. t.</pos> <sn>1.</sn> <def>To debauch.</def> <mark>[Obs.]</mark>  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To dishonor with the reputation of being a strumpet; hence, to belie; to slander.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>With his untrue reports, <qex>strumpet</qex> your fame.</q> <rj><qau>Massinger.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Strumstrum</ent><br/
<hw>Strum"strum</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A rude musical instrument somewhat like a cittern.</def> <mark>[R.]</mark>  <rj><au>Dampier.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Strung</ent><br/
<hw>Strung</hw> <pr>(?)</pr>, <def><pos>imp. & p. p.</pos> of <er>String</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Strunt</ent><br/
<hw>Strunt</hw> <pr>(?)</pr>, <pos>n.</pos> <def>Spirituous liquor.</def> <mark>[Scot.]</mark>  <rj><au>Burns.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Struntian</ent><br/
<hw>Strun"tian</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A kind of worsted braid, about an inch broad.</def> <mark>[Scot.]</mark>  <rj><au>Jamieson.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Struse</ent><br/
<hw>Struse</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Russ. <ets>strug'</ets>.]</ety> <fld>(Naut.)</fld> <def>A Russian river craft used for transporting freight.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Strut</ent><br/
<hw>Strut</hw> <pr>(?)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Strutted</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Strutting</conjf>.]</vmorph> <ety>[OE. <ets>struten</ets>, <ets>strouten</ets>, to swell; akin to G. <ets>strozen</ets> to be swelled, to be puffed up, to strut, Dan. <ets>strutte</ets>.]</ety> <sn>1.</sn> <def>To swell; to bulge out.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>The bellying canvas <qex>strutted</qex> with the gale.</q> <rj><qau>Dryden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To walk with a lofty, proud gait, and erect head; to walk with affected dignity.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Does he not hold up his head, . . . and <qex>strut</qex> in his gait?</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Strut</ent><br/
<hw>Strut</hw>, <pos>n.</pos> <ety>[For senses 2 & 3 cf. LG. <ets>strutt</ets> rigid.]</ety> <sn>1.</sn> <def>The act of strutting; a pompous step or walk.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Arch.)</fld> <def>In general, any piece of a frame which resists thrust or pressure in the direction of its own length. See <er>Brace</er>, and <xex>Illust.</xex> of <er>Frame</er>, and <er>Roof</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <fld>(Engin.)</fld> <def>Any part of a machine or structure, of which the principal function is to hold things apart; a brace subjected to compressive stress; -- the opposite of <xex>stay</xex>, and <xex>tie</xex>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Strut</ent><br/
<hw>Strut</hw>, <pos>v. t.</pos> <def>To hold apart.  Cf. <er>Strut</er>, <pos>n.</pos>, 3.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Strut</ent><br/
<hw>Strut</hw>, <pos>a.</pos> <def>Protuberant.</def> <mark>[Obs.]</mark>  <rj><au>Holland.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Struthian</ent><br/

<hw>Stru"thi*an</hw> <pr>(?)</pr>, <pos>a.</pos> <fld>(Zool.)</fld> <def>Struthious.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Struthio</ent><br/
||<hw>Stru"thi*o</hw> <pr>(?)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Struthiones</plw> <pr>(#)</pr>.</plu> <ety>[L., an ostrich, fr. Gr. <?/.]</ety> <fld>(Zool.)</fld> <def>A genus of birds including the African ostriches.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Struthioidea</ent><br/
||<hw>Stru`thi*oi"de*a</hw> <pr>(?)</pr>, <pos>n. pl.</pos> <ety>[NL. See <er>Struthio</er>, and <er>-oid</er>.]</ety> <fld>(Zool.)</fld> <def>Same as <er>Struthiones</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Struthiones</ent><br/
||<hw>Stru`thi*o"nes</hw> <pr>(?)</pr>, <pos>n. pl.</pos> <ety>[NL. See <er>Struthio</er>.]</ety> <fld>(Zool.)</fld> <sd>(a)</sd> <def>A division, or order, of birds, including only the African ostriches.</def> <sd>(b)</sd> <def>In a wider sense, an extensive group of birds including the ostriches, cassowaries, emus, moas, and allied birds incapable of flight. In this sense it is equivalent to <ord>Ratitae</ord>, or <ord>Dromaeognathae</ord>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Struthionine</ent><br/
<hw>Stru`thi*o"nine</hw> <pr>(?)</pr>, <pos>a.</pos> <fld>(Zool.)</fld> <def>Struthious.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Struthious</ent><br/
<hw>Stru"thi*ous</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>struthius</ets>, <ets>strutheus</ets>.]</ety> <fld>(Zool.)</fld> <def>Of or pertaining to the Struthiones, or Ostrich tribe.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Strutter</ent><br/
<hw>Strut"ter</hw> <pr>(?)</pr>, <pos>n.</pos> <def>One who struts.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Strutting</ent><br/
<hw>Strut"ting</hw>, <def><pos>a. & n.</pos> from <er>Strut</er>, <pos>v.</pos></def> -- <wordforms><wf>Strut"ting*ly</wf>, <pos>adv.</pos></wordforms><br/
[<source>1913 Webster</source>]</p>

<p><ent>Struvite</ent><br/
<hw>Struv"ite</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[After the Russian minister Von <ets>Struve</ets>.]</ety> <fld>(Min.)</fld> <def>A crystalline mineral found in guano. It is a hydrous phosphate of magnesia and ammonia.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Strychnia</ent><br/
<hw>Strych"ni*a</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[NL. See <er>Strychnine</er>.]</ety> <fld>(Chem.)</fld> <def>Strychnine.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Strychnic</ent><br/
<hw>Strych"nic</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Of or pertaining to strychnine; produced by strychnine; <as>as, <ex>strychnic</ex> compounds; <ex>strychnic</ex> poisoning</as></def>; specifically <fld>(Chem.)</fld>, <def>used to designate an acid, called also <xex>igasuric acid</xex>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Strychnine</ent><br/
<hw>Strych"nine</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>strychnos</ets> a kind of nightshade, Gr. <?/: cf. F. <ets>strychnine</ets>.]</ety> <fld>(Chem.)</fld> <def>A very poisonous alkaloid resembling brucine, obtained from various species of plants, especially from species of <fam>Loganiaceae</fam>, as from the seeds of the St. Ignatius bean (<spn>Strychnos Ignatia</spn>) and from nux vomica. It is obtained as a white crystalline substance, having a very bitter acrid taste, and is employed in medicine (chiefly in the form of the sulphate) as a powerful neurotic stimulant. Called also <altname>strychnia</altname>, and formerly <altname>strychnina</altname>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Strychnos</ent><br/
||<hw>Strych"nos</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L., a kind of nightshade, Gr. <?/.]</ety> <fld>(Bot.)</fld> <def>A genus of tropical trees and shrubs of the family <fam>Loganiaceae</fam>. See <er>Nux vomica</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><-- p. 1429 --></p>

<p><ent>Stryphnic</ent><br/
<hw>Stryph"nic</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[Gr. <?/ astringent.]</ety> <fld>(Chem.)</fld> <def>Pertaining to, or designating, a complex nitrogenous acid, obtained by the action of acetic acid and potassium nitrite on uric acid, as a yellow crystalline substance, with a bitter, astringent taste.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stub</ent><br/
<hw>Stub</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[OE. <ets>stubbe</ets>, AS. <ets>stub</ets>, <ets>styb</ets>; akin to D. <ets>stobbe</ets>, LG. <ets>stubbe</ets>, Dan. <ets>stub</ets>, Sw. <ets>stubbe</ets>, Icel. <ets>stubbr</ets>, <ets>stubbi</ets>; cf. Gr. <?/.]</ety> <sn>1.</sn> <def>The stump of a tree; that part of a tree or plant which remains fixed in the earth when the stem is cut down; -- applied especially to the stump of a small tree, or shrub.</def><br/
[<source>1913 Webster</source>]</p>

<p><q><qex>Stubs</qex> sharp and hideous to behold.</q> <rj><qau>Chaucer.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>And prickly <qex>stubs</qex> instead of trees are found.</q> <rj><qau>Dryden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A log; a block; a blockhead.</def> <mark>[Obs.]</mark>  <rj><au>Milton.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>The short blunt part of anything after larger part has been broken off or used up; hence, anything short and thick; <as>as, the <ex>stub</ex> of a pencil, candle, or cigar</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>A part of a leaf in a check book, after a check is torn out, on which the number, amount, and destination of the check are usually recorded.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <def>A pen with a short, blunt nib.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>6.</sn> <def>A stub nail; an old horseshoe nail; also, stub iron.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Stub end</b></col> <fld>(Mach.)</fld>, <cd>the enlarged end of a connecting rod, to which the strap is fastened.</cd> -- <col><b>Stub iron</b></col>, <cd>iron made from stub nails, or old horseshoe nails, -- used in making gun barrels.</cd> -- <col><b>Stub mortise</b></col> <fld>(Carp.)</fld>, <cd>a mortise passing only partly through the timber in which it is formed.</cd> -- <col><b>Stub nail</b></col>, <cd>an old horseshoe nail; a nail broken off; also, a short, thick nail.</cd> -- <mcol><col><b>Stub short</b></col>, <it>or</it> <col><b>Stub shot</b></col></mcol> <fld>(Lumber Manuf.)</fld>, <cd>the part of the end of a sawn log or plank which is beyond the place where the saw kerf ends, and which retains the plank in connection with the log, until it is split off.</cd> -- <col><b>Stub twist</b></col>, <cd>material for a gun barrel, made of a spirally welded ribbon of steel and stub iron combined.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stub</ent><br/
<hw>Stub</hw>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Stubbed</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Stubbing</conjf>.]</vmorph> <sn>1.</sn> <def>To grub up by the roots; to extirpate; <as>as, to <ex>stub</ex> up edible roots</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>What <qex>stubbing</qex>, plowing, digging, and harrowing is to a piece of land.</q> <rj><qau>Berkley.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To remove stubs from; <as>as, to <ex>stub</ex> land</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>To strike as the toes, against a stub, stone, or other fixed object.</def> <mark>[U. S.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stubbed</ent><br/
<hw>Stub"bed</hw> <pr>(?)</pr>, <pos>a.</pos> <sn>1.</sn> <def>Reduced to a stub; short and thick, like something truncated; blunt; obtuse.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Abounding in stubs; stubby.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>A bit of <qex>stubbed</qex> ground, once a wood.</q> <rj><qau>R. Browning.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>Not nice or delicate; hardy; rugged.</def> <ldquo/<xex>Stubbed</xex>, vulgar constitutions.<rdquo/  <rj><au>Berkley.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stubbedness</ent><br/
<hw>Stub"bed*ness</hw>, <pos>n.</pos> <def>The quality or state of being stubbed.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stubbiness</ent><br/
<hw>Stub"bi*ness</hw> <pr>(?)</pr>, <pos>n.</pos> <def>The state of being stubby.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stubble</ent><br/
<hw>Stub"ble</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[OE. <ets>stobil</ets>, <ets>stoble</ets>, OF. <ets>estouble</ets>, <ets>estuble</ets>, F. <ets><eacute/tuele</ets>, LL. <ets>stupla</ets>, <ets>stupula</ets>, L. <ets>stipula</ets> stubble, stalk; cf. D. & G. <ets>stopped</ets>, OHG. <ets>stupfila</ets>.  Cf. <er>Stipule</er>.]</ety> <def>The stumps of wheat, rye, barley, oats, or buckwheat, left in the ground; the part of the stalk left by the scythe or sickle.</def> <ldquo/After the first crop is off, they plow in the wheast <xex>stubble</xex>.<rdquo/  <rj><au>Mortimer.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Stubble goose</b></col> <fld>(Zool.)</fld>, <cd>the graylag goose.</cd> <mark>[Prov. Eng.]</mark> <au>Chaucer.</au> -- <col><b>Stubble rake</b></col>, <cd>a rake with long teeth for gleaning in stubble.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stubbled</ent><br/
<hw>Stub"bled</hw> <pr>(?)</pr>, <pos>a.</pos> <sn>1.</sn> <def>Covered with stubble.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>A crow was strutting o'er the <qex>stubbled</qex> plain.</q> <rj><qau>Gay.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Stubbed; <as>as, <ex>stubbled</ex> legs</as>.</def> <mark>[Obs.]</mark>  <rj><au>Skelton.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stubbly</ent><br/
<hw>Stub"bly</hw>, <pos>a.</pos> <def>Covered with stubble; stubbled.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stubborn</ent><br/
<hw>Stub"born</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[OE. <ets>stoburn</ets>, <ets>stiborn</ets>; probably fr. AS. <ets>styb</ets> a stub. See <er>Stub</er>.]</ety> <def>Firm as a stub or stump; stiff; unbending; unyielding; persistent; hence, unreasonably obstinate in will or opinion; not yielding to reason or persuasion; refractory; harsh; -- said of persons and things; <as>as, <ex>stubborn</ex> wills; <ex>stubborn</ex> ore; a <ex>stubborn</ex> oak; as <ex>stubborn</ex> as a mule.</as></def> <ldquo/Bow, <xex>stubborn</xex> knees.<rdquo/ <au>Shak.</au> <ldquo/<xex>Stubborn</xex> attention and more than common application.<rdquo/ <au>Locke.</au> <ldquo/<xex>Stubborn</xex> Stoics.<rdquo/ <au>Swift.</au><br/
[<source>1913 Webster</source>]</p>

<p><q>And I was young and full of ragerie [wantonness]<br/
<qex>Stubborn</qex> and strong, and jolly as a pie.</q> <rj><qau>Chaucer.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>These heretics be so stiff and <qex>stubborn</qex>.</q> <rj><qau>Sir T. More.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Your <qex>stubborn</qex> usage of the pope.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><syn><b>Syn.</b> -- Obstinate; inflexible; obdurate; headstrong; stiff; hardy; firm; refractory; intractable; rugged; contumacious; heady.</syn>  -- <usage><er>Stubborn</er>, <er>Obstinate</er>. <xex>Obstinate</xex> is used of either active or passive persistence in one's views or conduct, in spite of the wishes of others. <xex>Stubborn</xex> describes an extreme degree of passive <xex>obstinacy</xex>.</usage>
 -- <wordforms><wf>Stub"born*ly</wf>, <pos>adv.</pos> -- <wf>Stub"born*ness</wf>, <pos>n.</pos></wordforms><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stubby</ent><br/
<hw>Stub"by</hw> <pr>(?)</pr>, <pos>a.</pos> <sn>1.</sn> <def>Abounding with stubs.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Short and thick; short and strong, as bristles.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stucco</ent><br/
<hw>Stuc"co</hw> <pr>(?)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Stuccoes</plw> <pr>(#)</pr>, <plw>Stuccos</plw>.</plu> <ety>[It., fr. OHG. <ets>stucchi</ets> a crust, piece, G. <ets>st<uum/ck</ets> piece; akin to AS. <ets>stycce</ets>. See <er>Stock</er>.]</ety> <sn>1.</sn> <def>Plaster of any kind used as a coating for walls, especially, a fine plaster, composed of lime or gypsum with sand and pounded marble, used for internal decorations and fine work.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Work made of stucco; stuccowork.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stucco</ent><br/
<hw>Stuc"co</hw>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Stuccoed</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Stuccoing</conjf> <pr>(?)</pr>.]</vmorph> <def>To overlay or decorate with stucco, or fine plaster.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stuccoer</ent><br/
<hw>Stuc"co*er</hw> <pr>(?)</pr>, <pos>n.</pos> <def>One who stuccoes.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stuccowork</ent><br/
<hw>Stuc"co*work`</hw> <pr>(?)</pr>, <pos>n.</pos> <def>Work done in stucco.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stuck</ent><br/
<hw>Stuck</hw> <pr>(?)</pr>, <def><pos>imp. & p. p.</pos> of <er>Stick</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stuck</ent><br/
<hw>Stuck</hw>, <pos>n.</pos> <ety>[Cf. 1st <er>Stoccado</er>.]</ety> <def>A thrust.</def> <mark>[Obs.]</mark>  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stuckle</ent><br/
<hw>Stuc"kle</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[From <er>Stook</er>.]</ety> <def>A number of sheaves set together in the field; a stook.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stuck-up</ent><br/
<hw>Stuck"-up`</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Self-important and supercilious, <?/onceited; vain; arrogant.</def> <mark>[Colloq.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>The airs of small, <qex>stuck-up</qex>, men.</q> <rj><qau>A. K. H. Boyd.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stud</ent><br/
<hw>Stud</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[OE. <ets>stod</ets>, <ets>stood</ets>, AS. <ets>st<omac/d</ets>; akin to OHG. <ets>stuota</ets>, G. <ets>stute</ets> a mare, Icel. <ets>st<omac/<?/</ets> stud, Lith. <ets>stodas</ets> a herd, Russ. <ets>stado</ets>, and to E. <ets>stand</ets>. The sense is properly, a stand, an establishment. <root/163. See <er>Stand</er>, and cf. <er>Steed</er>.]</ety> <def>A collection of breeding horses and mares, or the place where they are kept; also, a number of horses kept for a racing, riding, etc.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>In the <qex>studs</qex> of Ireland, where care is taken, we see horses bred of excellent shape, vigor, and size.</q> <rj><qau>Sir W. Temple.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>He had the finest <qex>stud</qex> in England, and his delight was to win plates from Tories.</q> <rj><qau>Macaulay.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stud</ent><br/
<hw>Stud</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[AS. <ets>studu</ets> a post; akin to Sw. <ets>st<oum/d</ets> a prop, Icel. <ets>sto<?/</ets> a post, <ets>sty<?/ja</ets> to prop, and probably ultimately to E. <ets>stand</ets>; cf. D. <ets>stut</ets> a prop, G. <ets>st<uum/tze</ets>. See <er>Stand</er>.]</ety> <sn>1.</sn> <def>A stem; a trunk.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>Seest not this same hawthorn <qex>stud</qex>?</q> <rj><qau>Spenser.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Arch.)</fld> <def>An upright scanting, esp. one of the small uprights in the framing for lath and plaster partitions, and furring, and upon which the laths are nailed.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>A kind of nail with a large head, used chiefly for ornament; an ornamental knob; a boss.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>A belt of straw and ivy buds,<br/
With coral clasps and amber <qex>studs</qex>.</q> <rj><qau>Marlowe.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Crystal and myrrhine cups, embossed with gems<br/
And <qex>studs</qex> of pearl.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>An ornamental button of various forms, worn in a shirt front, collar, wristband, or the like, not sewed in place, but inserted through a buttonhole or eyelet, and transferable.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <fld>(Mach.)</fld> <sd>(a)</sd> <def>A short rod or pin, fixed in and projecting from something, and sometimes forming a journal.</def> <sd>(b)</sd> <def>A stud bolt.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>6.</sn> <def>An iron brace across the shorter diameter of the link of a chain cable.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Stud bolt</b></col>, <cd>a bolt with threads on both ends, to be screwed permanently into a fixed part at one end and receive a nut upon the other; -- called also <altname>standing bolt</altname>.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stud</ent><br/
<hw>Stud</hw>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Studded</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Studding</conjf>.]</vmorph> <sn>1.</sn> <def>To adorn with shining studs, or knobs.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Thy horses shall be trapped,<br/
Their harness <qex>studded</qex> all with gold and pearl.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To set with detached ornaments or prominent objects; to set thickly, as with studs.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The sloping sides and summits of our hills, and the extensive plains that stretch before our view, are <qex>studded</qex> with substantial, neat, and commodious dwellings of freemen.</q> <rj><qau>Bp. Hobart.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Studbook</ent><br/
<hw>Stud"book`</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A genealogical register of a particular breed or stud of horses, esp. thoroughbreds.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Studdery</ent><br/
<hw>Stud"der*y</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A stud, or collection of breeding horses and mares; also, a place for keeping a stud.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>King Henry the Eighth erected a noble <qex>studdery</qex>.</q> <rj><qau>Holinshed.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Studding</ent><br/
<hw>Stud"ding</hw> <pr>(?)</pr>, <pos>n.</pos> <def>Material for studs, or joists; studs, or joists, collectively; studs.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Studding sail</ent><br/
<hw>Stud"ding sail`</hw> <pr>(?)</pr>. <fld>(Naut.)</fld> <def>A light sail set at the side of a principal or square sail of a vessel in free winds, to increase her speed. Its head is bent to a small spar which is called the <xex>studding-sail boom</xex>. See <xex>Illust.</xex> of <er>Sail</er>.</def>  <rj><au>Toten.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Student</ent><br/
<hw>Stu"dent</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>studens</ets>, <ets>-entis</ets>, p. pr. of <ets>studere</ets> to study. See <er>Study</er>, <pos>n.</pos>]</ety> <sn>1.</sn> <def>A person engaged in study; one who is devoted to learning; a learner; a pupil; a scholar; especially, one who attends a school, or who seeks knowledge from professional teachers or from books; <as>as, the <ex>students</ex> of an academy, a college, or a university; a medical <ex>student</ex>; a hard <ex>student</ex>.</as></def><br/
[<source>1913 Webster</source>]</p>

<p><q>Keep a gamester from the dice, and a good <qex>student</qex> from his book.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>One who studies or examines in any manner; an attentive and systematic observer; <as>as, a <ex>student</ex> of human nature, or of physical nature</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Studentry</ent><br/
<hw>Stu"dent*ry</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A body of students.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Studentship</ent><br/
<hw>Stu"dent*ship</hw>, <pos>n.</pos> <def>The state of being a student.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Studfish</ent><br/
<hw>Stud"fish`</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Zool.)</fld> <def>Any one of several species of small American minnows of the genus <gen>Fundulus</gen>, as <spn>Fundulus catenatus</spn>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stud-horse</ent><br/
<hw>Stud"-horse`</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[AS. <ets>st<omac/d-hors</ets>.]</ety> <def>A stallion, esp. one kept for breeding.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Studied</ent><br/
<hw>Stud"ied</hw>, <pos>a.</pos> <sn>1.</sn> <def>Closely examined; read with diligence and attention; made the subject of study; well considered; <as>as, a <ex>studied</ex> lesson</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Well versed in any branch of learning; qualified by study; learned; <as>as, a man well <ex>studied</ex> in geometry</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>I shrewdly suspect that he is little <qex>studied</qex> of a theory of moral proportions.</q> <rj><qau>Burke.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>Premeditated; planned; designed; <as>as, a <ex>studied</ex> insult</as>.</def> <ldquo/<xex>Studied</xex> magnificence.<rdquo/  <rj><au>Hawthorne.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>Intent; inclined.</def> <mark>[Obs.]</mark>  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Studiedly</ent><br/
<hw>Stud"ied*ly</hw> <pr>(?)</pr>, <pos>adv.</pos> <def>In a studied manner.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Studier</ent><br/
<hw>Stud"i*er</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A student.</def> <mark>[R.]</mark>  <rj><au>W. Irving.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Lipsius was a great <qex>studier</qex> of the stoical philosophy.</q> <rj><qau>Tillotson.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Studio</ent><br/
<hw>Stu"di*o</hw> <pr>(?)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Studios</plw> <pr>(#)</pr>.</plu> <ety>[It. <ets>studio</ets>, properly, study. See <er>Study</er>.]</ety> <def>The working room of an artist.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Studious</ent><br/
<hw>Stu"di*ous</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>studious</ets>: cf. F. <ets>studieux</ets>. See <er>Study</er>.]</ety> <sn>1.</sn> <def>Given to study; devoted to the acquisition of knowledge from books; <as>as, a <ex>studious</ex> scholar</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Given to thought, or to the examination of subjects by contemplation; contemplative.</def>  <rj><au>Locke.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>Earnest in endeavors; aiming sedulously; attentive; observant; diligent; -- usually followed by an infinitive or by <xex>of</xex>; <as>as, be <ex>studious</ex> to please; <ex>studious</ex> to find new friends and allies</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>You that are so <qex>studious</qex><br/
Of my affairs, wholly neglect your own.</q> <rj><qau>Massinger.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>Planned with study; deliberate; studied.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>For the frigid villainy of <qex>studious</qex> lewdness, . . . with apology can be invented?</q> <rj><qau>Rambler.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <def>Favorable to study; suitable for thought and contemplation; <as>as, the <ex>studious</ex> shade</as>.</def> <mark>[Poetic]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>But let my due feet never fail<br/
To walk the <qex>studious</qex> cloister's pale.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p>-- <wordforms><wf>Stu"di*ous*ly</wf>, <pos>adv.</pos> -- <wf>Stu"di*ous*ness</wf>, <pos>n.</pos></wordforms><br/
[<source>1913 Webster</source>]</p>

<p><ent>Study</ent><br/
<hw>Stud"y</hw> <pr>(?)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Studies</plw> <pr>(#)</pr>.</plu> <ety>[OE. <ets>studie</ets>, L. <ets>studium</ets>, akin to <ets>studere</ets> to study; possibly akin to Gr. <?/ haste, zeal, <?/ to hasten; cf. OF. <ets>estudie</ets>, <ets>estude</ets>, F. <ets><eacute/tude</ets>.  Cf. <er>Etude</er>, <er>Student</er>, <er>Studio</er>, <er>Study</er>, <pos>v. i.</pos>]</ety> <sn>1.</sn> <def>A setting of the mind or thoughts upon a subject; hence, application of mind to books, arts, or science, or to any subject, for the purpose of acquiring knowledge.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Hammond . . . spent thirteen hours of the day in <qex>study</qex>.</q> <rj><qau>Bp. Fell.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q><qex>Study</qex> gives strength to the mind; conversation, grace.</q> <rj><qau>Sir W. Temple.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Mental occupation; absorbed or thoughtful attention; meditation; contemplation.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Just men they seemed, and all their <qex>study</qex> bent<br/
To worship God aright, and know his works.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>Any particular branch of learning that is studied; any object of attentive consideration.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The Holy Scriptures, especially the New Testament, are her daily <qex>study</qex>.</q> <rj><qau>Law.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>The proper <qex>study</qex> of mankind is man.</q> <rj><qau>Pope.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>A building or apartment devoted to study or to literary work.</def> <ldquo/His cheery little <xex>study</xex>.<rdquo/  <rj><au>Hawthorne.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <fld>(Fine Arts)</fld> <def>A representation or rendering of any object or scene intended, not for exhibition as an original work of art, but for the information, instruction, or assistance of the maker; <as>as, a <ex>study</ex> of heads or of hands for a figure picture</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>6.</sn> <fld>(Mus.)</fld> <def>A piece for special practice. See <er>Etude</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Study</ent><br/
<hw>Stud"y</hw> <pr>(?)</pr>, <pos>v. i.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Studied</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Studying</conjf> <pr>(?)</pr>.]</vmorph> <ety>[OE. <ets>studien</ets>, OF. <ets>estudier</ets>, F. <ets><eacute/tudier</ets>. See <er>Study</er>, <pos>n.</pos>]</ety> <sn>1.</sn> <def>To fix the mind closely upon a subject; to dwell upon anything in thought; to muse; to ponder.</def>  <rj><au>Chaucer.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>I found a moral first, and then <qex>studied</qex> for a fable.</q> <rj><qau>Swift.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To apply the mind to books or learning.</def>  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>To endeavor diligently; to be zealous.</def>  <rj><au>1 Thes. iv. 11.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Study</ent><br/
<hw>Stud"y</hw>, <pos>v. t.</pos> <sn>1.</sn> <def>To apply the mind to; to read and examine for the purpose of learning and understanding; <as>as, to <ex>study</ex> law or theology; to <ex>study</ex> languages</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To consider attentively; to examine closely; <as>as, to <ex>study</ex> the work of nature</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q><qex>Study</qex> thyself; what rank or what degree<br/
The wise Creator has ordained for thee.</q> <rj><qau>Dryden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>To form or arrange by previous thought; to con over, as in committing to memory; <as>as, to <ex>study</ex> a speech</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>To make an object of study; to aim at sedulously; to devote one's thoughts to; <as>as, to <ex>study</ex> the welfare of others; to <ex>study</ex> variety in composition</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>For their heart <qex>studieth</qex> destruction.</q> <rj><qau>Prov. xxiv. 2.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stufa</ent><br/
||<hw>Stu"fa</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[It. <ets>stufa</ets> a stove. See <er>Stove</er>.]</ety> <def>A jet of steam issuing from a fissure in the earth.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stuff</ent><br/
<hw>Stuff</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[OF. <ets>estoffe</ets>, F. <ets><eacute/toffe</ets>; of uncertain origin, perhaps of Teutonic origin and akin to E. <ets>stop</ets>, v.t.  Cf. <er>Stuff</er>, <pos>v. t.</pos>]</ety> <sn>1.</sn> <def>Material which is to be worked up in any process of manufacture.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>For the <qex>stuff</qex> they had was sufficient for all the work to make it, and too much.</q> <rj><qau>Ex. xxxvi. 7.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Ambitions should be made of sterner <qex>stuff</qex>.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>The workman on his <qex>stuff</qex> his skill doth show,<br/
And yet the <qex>stuff</qex> gives not the man his skill.</q> <rj><qau>Sir J. Davies.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>The fundamental material of which anything is made up; elemental part; essence.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Yet do I hold it very <qex>stuff</qex> o' the conscience<br/
To do no contrived murder.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>Woven material not made into garments; fabric of any kind; specifically, any one of various fabrics of wool or worsted; sometimes, worsted fiber.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>What <qex>stuff</qex> wilt have a kirtle of?</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>It [the arras] was of <qex>stuff</qex> and silk mixed, though, superior kinds were of silk exclusively.</q> <rj><qau>F. G. Lee.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>Furniture; goods; domestic vessels or utensils.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>He took away locks, and gave away the king's <qex>stuff</qex>.</q> <rj><qau>Hayward.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <def>A medicine or mixture; a potion.</def>  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>6.</sn> <def>Refuse or worthless matter; hence, also, foolish or irrational language; nonsense; trash.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Anger would indite<br/
Such woeful <qex>stuff</qex> as I or Shadwell write.</q> <rj><qau>Dryden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>7.</sn> <fld>(Naut.)</fld> <def>A melted mass of turpentine, tallow, etc., with which the masts, sides, and bottom of a ship are smeared for lubrication.</def>
 <rj><qau>Ham. Nav. Encyc.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>8.</sn> <def>Paper stock ground ready for use.</def><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ When partly ground, called <xex>half stuff</xex>.</note>  <rj><au>Knight.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Clear stuff</b></col>. <cd>See under <er>Clear</er>.</cd> -- <col><b>Small stuff</b></col> <fld>(Naut.)</fld>, <cd>all kinds of small cordage.</cd> <au>Ham. Nav. Encyc.</au> -- <col><b>Stuff gown</b></col>, <cd>the distinctive garb of a junior barrister; hence, a junior barrister himself. See <altname>Silk gown</altname>, under <er>Silk</er>.</cd></cs><-- <col><b>stuff and nonsense</b></col>. (See def. 6 for stuff) <cd>balderdash, twaddle, nonsense, foolishness.</cd> --><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stuff</ent><br/
<hw>Stuff</hw>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Stuffed</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Stuffing</conjf>.]</vmorph> <ety>[OE. <ets>stoffen</ets>; cf. OF. <ets>estoffer</ets>, F. <ets><eacute/toffer</ets>, to put stuff in, to stuff, to line, also, OF. <ets>estouffer</ets> to stifle, F. <ets><eacute/touffer</ets>; both perhaps of Teutonic origin, and akin to E. <ets>stop</ets>.  Cf. <er>Stop</er>, <pos>v. t.</pos>, <er>Stuff</er>, <pos>n.</pos>]</ety> <sn>1.</sn> <def>To fill by crowding something into; to cram with something; to load to excess; <as>as, to <ex>stuff</ex> a bedtick</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Sometimes this crook drew hazel bought adown,<br/
And <qex>stuffed</qex> her apron wide with nuts so brown.</q> <rj><qau>Gay.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Lest the gods, for sin,<br/
Should with a swelling dropsy <qex>stuff</qex> thy skin.</q> <rj><qau>Dryden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To thrust or crowd; to press; to pack.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Put roses into a glass with a narrow mouth, <qex>stuffing</qex> them close together . . . and they retain smell and color.</q> <rj><qau>Bacon.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>To fill by being pressed or packed into.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>With inward arms the dire machine they load,<br/
And iron bowels <qex>stuff</qex> the dark abode.</q> <rj><qau>Dryden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <fld>(Cookery)</fld> <def>To fill with a seasoning composition of bread, meat, condiments, etc.; <as>as, to <ex>stuff</ex> a turkey</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <def>To obstruct, as any of the organs; to affect with some obstruction in the organs of sense or respiration.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>I'm <qex>stuffed</qex>, cousin; I can not smell.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>6.</sn> <def>To fill the skin of, for the purpose of preserving as a specimen; -- said of birds or other animals.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>7.</sn> <def>To form or fashion by packing with the necessary material.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>An Eastern king put a judge to death for an iniquitous sentence, and ordered his hide to be <qex>stuffed</qex> into a cushion, and placed upon the tribunal.</q> <rj><qau>Swift.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>8.</sn> <def>To crowd with facts; to cram the mind of; sometimes, to crowd or fill with false or idle tales or fancies.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>9.</sn> <def>To put fraudulent votes into (a ballot box).</def> <mark>[U. S.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><-- p. 1430 --></p>

<p><ent>Stuff</ent><br/
<hw>Stuff</hw> <pr>(?)</pr>, <pos>v. i.</pos> <def>To feed gluttonously; to cram.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Taught harmless man to cram and <qex>stuff</qex>.</q> <rj><qau>Swift.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stuffer</ent><br/
<hw>Stuff"er</hw> <pr>(?)</pr>, <pos>n.</pos> <def>One who, or that which, stuffs.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stuffiness</ent><br/
<hw>Stuff"i*ness</hw> <pr>(?)</pr>, <pos>n.</pos> <def>The quality of being stuffy.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stuffing</ent><br/
<hw>Stuff"ing</hw>, <pos>n.</pos> <sn>1.</sn> <def>That which is used for filling anything; <as>as, the <ex>stuffing</ex> of a saddle or cushion</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Cookery)</fld> <def>Any seasoning preparation used to stuff meat; especially, a composition of bread, condiments, spices, etc.; forcemeat; dressing.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>A mixture of oil and tallow used in softening and dressing leather.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Stuffing box</b></col>, <cd>a device for rendering a joint impervious where there is a hole through which a movable cylindrical body, as the paston rod of a steam engine, or the plunger of a pump, slides back and forth, or in which a shaft turns. It usually consists of a box or chamber, made by an enlargement of part of the hole, forming a space around the rod or shaft for containing packing which is compressed and made to fill the space closely by means of a sleeve, called the <xex>gland</xex>, which fits loosely around the rod, and is pressed upon the packing by bolts or other means.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stuffy</ent><br/
<hw>Stuff"y</hw> <pr>(?)</pr>, <pos>a.</pos> <sn>1.</sn> <def>Stout; mettlesome; resolute.</def> <mark>[Scot.]</mark>  <rj><au>Jamieson.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Angry and obstinate; sulky.</def> <mark>[U. S.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>Ill-ventilated; close.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stuke</ent><br/
<hw>Stuke</hw> <pr>(?)</pr>, <pos>n.</pos> <def>Stucco.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stull</ent><br/
<hw>Stull</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[CF. <er>Stum</er>.]</ety> <def>A framework of timber covered with boards to support rubbish; also, a framework of boards to protect miners from falling stones.</def> <mark>[Prov. Eng.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stulm</ent><br/
<hw>Stulm</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Cf. G. <ets>stollen</ets> a post, a stulm, E. <ets>stall</ets>, <ets>stand</ets>.]</ety> <def>A shaft or gallery to drain a mine.</def> <mark>[Local, Eng.]</mark>  <rj><au>Bailey.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stulp</ent><br/
<hw>Stulp</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Cf. Icel. <ets>st<omac/lpi</ets>, Dan., Sw., & OD. <ets>stolpe</ets>.]</ety> <def>A short, stout post used for any purpose, a to mark a boundary.</def> <mark>[Prov. Eng.]</mark>  <rj><au>Halliwell.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stultification</ent><br/
<hw>Stul`ti*fi*ca"tion</hw> <pr>(?)</pr>, <pos>n.</pos> <def>The act of stultifying, or the state of being stultified.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stultifier</ent><br/
<hw>Stul"ti*fi`er</hw> <pr>(?)</pr>, <pos>n.</pos> <def>One who stultifies.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stultify</ent><br/
<hw>Stul"ti*fy</hw> <pr>(?)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Stultified</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Stultifying</conjf> <pr>(?)</pr>.]</vmorph> <ety>[L. <ets>stultus</ets> foolish + <ets>-fy</ets>.]</ety> <sn>1.</sn> <def>To make foolish; to make a fool of; <as>as, to <ex>stultify</ex> one by imposition; to <ex>stultify</ex> one's self by silly reasoning or conduct</as>.</def>  <rj><au>Burke.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To regard as a fool, or as foolish.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>The modern sciolist <qex>stultifies</qex> all understanding but his own, and that which he conceives like his own.</q>  <rj><au>Hazlitt.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <fld>(Law)</fld> <def>To allege or prove to be of unsound mind, so that the performance of some act may be avoided.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stultiloquence</ent><br/
<hw>Stul*til"o*quence</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>stultiloquentia</ets>; <ets>stultus</ets> foolish + <ets>loquentia</ets> a talking, fr. <ets>loquens</ets>, p. pr. of <ets>loqui</ets> to talk.]</ety> <def>Silly talk; babbling.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stultiloquent</ent><br/
<hw>Stul*til"o*quent</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[Cf. L. <ets>stultiloquus</ets>. See <er>Stultiloquence</er>.]</ety> <def>Given to, or characterized by, silly talk; babbling.</def> -- <wordforms><wf>Stul*til"o*quent*ly</wf>, <pos>adv.</pos></wordforms><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stultiloquy</ent><br/
<hw>Stul*til"o*quy</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>stultiloquium</ets>.]</ety> <def>Foolish talk; silly discource; babbling.</def>  <rj><au>Jer. Taylor.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stulty</ent><br/
<hw>Stul"ty</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>stultus</ets> foolish.]</ety> <def>Foolish; silly.</def> <mark>[Obs.]</mark>  <rj><au>Testament of Love.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stum</ent><br/
<hw>Stum</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[D. <ets>stom</ets> must, new wort, properly, dumb; cf. F. <ets>vin muet</ets> stum.  Cf. <er>Stammer</er>, <er>Stoom</er>.]</ety> <sn>1.</sn> <def>Unfermented grape juice or wine, often used to raise fermentation in dead or vapid wines; must.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Let our wines, without mixture of <qex>stum</qex>, be all fine.</q> <rj><qau>B. Jonson.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>And with thy <qex>stum</qex> ferment their fainting cause.</q> <rj><qau>Dryden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Wine revived by new fermentation, reulting from the admixture of must.</def>  <rj><au>Hudibras.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stum</ent><br/
<hw>Stum</hw>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Stummed</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Stumming</conjf>.]</vmorph> <def>To renew, as wine, by mixing must with it and raising a new fermentation.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>We <qex>stum</qex> our wines to renew their spirits.</q> <rj><qau>Floyer.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stumble</ent><br/
<hw>Stum"ble</hw> <pr>(?)</pr>, <pos>v. i.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Stumbled</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Stumbling</conjf> <pr>(?)</pr>.]</vmorph> <ety>[OE. <ets>stumblen</ets>, <ets>stomblen</ets>; freq. of a word akin to E. <ets>stammer</ets>. See <er>Stammer</er>.]</ety> <sn>1.</sn> <def>To trip in walking or in moving in any way with the legs; to strike the foot so as to fall, or to endanger a fall; to stagger because of a false step.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>There <qex>stumble</qex> steeds strong and down go all.</q> <rj><qau>Chaucer.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>The way of the wicked is as darkness: they know at what they <qex>stumble</qex>.</q> <rj><qau>Prov. iv. 19.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To walk in an unsteady or clumsy manner.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>He <qex>stumbled</qex> up the dark avenue.</q> <rj><qau>Sir W. Scott.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>To fall into a crime or an error; to err.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>He that loveth his brother abideth in the light, and there is none occasion og <qex>stumbling</qex> in him.</q> <rj><qau>1 John ii. 10.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>To strike or happen (upon a person or thing) without design; to fall or light by chance; -- with <xex>on</xex>, <xex>upon</xex>, or <xex>against</xex>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Ovid <qex>stumbled</qex>, by some inadvertency, upon Livia in a bath.</q> <rj><qau>Dryden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Forth as she waddled in the brake,<br/
A gray goose <qex>stumbled</qex> on a snake.</q> <rj><qau>C. Smart.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stumble</ent><br/
<hw>Stum"ble</hw>, <pos>v. t.</pos> <sn>1.</sn> <def>To cause to stumble or trip.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Fig.: To mislead; to confound; to perplex; to cause to err or to fall.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>False and dazzling fires to <qex>stumble</qex> men.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>One thing more <qex>stumbles</qex> me in the very foundation of this hypothesis.</q> <rj><qau>Locke.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stumble</ent><br/
<hw>Stum"ble</hw>, <pos>n.</pos> <sn>1.</sn> <def>A trip in walking or running.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A blunder; a failure; a fall from rectitude.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>One <qex>stumble</qex> is enough to deface the character of an honorable life.</q> <rj><qau>L'Estrange.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stumbler</ent><br/
<hw>Stum"bler</hw> <pr>(?)</pr>, <pos>n.</pos> <def>One who stumbles.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stumbling-block</ent><br/
<hw>Stum"bling-block`</hw> <pr>(?)</pr>, <pos>n.</pos> <def>Any cause of stumbling, perplexity, or error.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>We preach Christ crucified, unto the Jews a <qex>stumbling-block</qex>, and unto the Greeks foolishness.</q> <rj><qau>1 Cor. i. 23.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stumblingly</ent><br/
<hw>Stum"bling*ly</hw> <pr>(?)</pr>, <pos>adv.</pos> <def>In a stumbling manner.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stumbling-stone</ent><br/
<hw>Stum"bling-stone`</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A stumbling-block.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>This <qex>stumbling-stone</qex> we hope to take away.</q> <rj><qau>T. Burnet.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stump</ent><br/
<hw>Stump</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[OE. <ets>stumpe</ets>, <ets>stompe</ets>; akin to D. <ets>stomp</ets>, G. <ets>stumpf</ets>, Icel. <ets>stumpr</ets>, Dan. & Sw. <ets>stump</ets>, and perhaps also to E. <ets>stamp</ets>.]</ety> <sn>1.</sn> <def>The part of a tree or plant remaining in the earth after the stem or trunk is cut off; the stub.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>The part of a limb or other body remaining after a part is amputated or destroyed; a fixed or rooted remnant; a stub; <as>as, the <ex>stump</ex> of a leg, a finger, a tooth, or a broom</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <pluf>pl.</pluf> <def>The legs; <as>as, to stir one's <ex>stumps</ex></as>.</def> <mark>[Slang]</mark><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <fld>(Cricket)</fld> <def>One of the three pointed rods stuck in the ground to form a wicket and support the bails.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <def>A short, thick roll of leather or paper, cut to a point, or any similar implement, used to rub down the lines of a crayon or pencil drawing, in shading it, or for shading drawings by producing tints and gradations from crayon, etc., in powder.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>6.</sn> <def>A pin in a tumbler lock which forms an obstruction to throwing the bolt, except when the gates of the tumblers are properly arranged, as by the key; a fence; also, a pin or projection in a lock to form a guide for a movable piece.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Leg stump</b></col> <fld>(Cricket)</fld>, <cd>the stump nearest to the batsman.</cd> -- <col><b>Off stump</b></col> <fld>(Cricket)</fld>, <cd>the stump farthest from the batsman.</cd> -- <col><b>Stump tracery</b></col> <fld>(Arch.)</fld>, <cd>a term used to describe late German Gothic tracery, in which the molded bar seems to pass through itself in its convolutions, and is then cut off short, so that a section of the molding is seen at the end of each similar stump.</cd> -- <mcol><col><b>To go on the stump</b></col>, <it>or</it> <col><b>To take the stump</b></col></mcol>, <cd>to engage in making public addresses for electioneering purposes; -- a phrase derived from the practice of using a stump for a speaker's platform in newly-settled districts.  Hence also the phrases <xex>stump orator</xex>, <xex>stump speaker</xex>, <xex>stump speech</xex>, <xex>stump oratory</xex>, etc.</cd> <mark>[Colloq. U.S.]</mark></cs> -- <col><b>on the stump</b></col> <cd>campaigning for public office; running for election to office.</cd><br/
[<source>1913 Webster</source> <source>+PJC</source>]</p>

<p><ent>Stump</ent><br/
<hw>Stump</hw>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Stumped</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Stumping</conjf>.]</vmorph> <sn>1.</sn> <def>To cut off a part of; to reduce to a stump; to lop.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Around the <qex>stumped</qex> top soft moss did grow.</q> <rj><qau>Dr. H. More.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To strike, as the toes, against a stone or something fixed; to stub.</def> <mark>[Colloq.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>To challenge; also, to nonplus.</def> <mark>[Colloq.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>To travel over, delivering speeches for electioneering purposes; <as>as, to <ex>stump</ex> a State, or a district</as>. See <cref>To go on the stump</cref>, under <er>Stump</er>, <pos>n.</pos></def> <mark>[Colloq. U.S.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <fld>(Cricket)</fld> <sd>(a)</sd> <def>To put (a batsman) out of play by knocking off the bail, or knocking down the stumps of the wicket he is defending while he is off his allotted ground; -- sometimes with <xex>out</xex>.</def> <au>T. Hughes.</au> <sd>(b)</sd> <def>To bowl down the stumps of, as, of a wicket.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>A herd of boys with clamor bowled,<br/
And <qex>stumped</qex> the wicket.</q> <rj><qau>Tennyson.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>To stump it</b></col>. <sd>(a)</sd> <cd>To go afoot; hence, to run away; to escape</cd>. <mark>[Slang]</mark> <au>Ld. Lytton.</au> <sd>(b)</sd> <cd>To make electioneering speeches.</cd> <mark>[Colloq. U.S.]</mark></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stump</ent><br/
<hw>Stump</hw>, <pos>v. i.</pos> <def>To walk clumsily, as if on stumps.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>To stump up</b></col>, <cd>to pay cash.</cd> <mark>[Prov. Eng.]</mark> <au>Halliwell.</au></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stumpage</ent><br/
<hw>Stump"age</hw> <pr>(?)</pr>, <pos>n.</pos> <sn>1.</sn> <def>Timber in standing trees, -- often sold without the land at a fixed price per tree or per stump, the stumps being counted when the land is cleared.</def> <mark>[Local, U.S.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>Only trees above a certain size are allowed to be cut by loggers buying <qex>stumpage</qex> from the owners of land.</q> <rj><qau>C. S. Sargent.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A tax on the amount of timber cut, regulated by the price of lumber.</def> <mark>[Local, U.S.]</mark>  <rj><au>The Nation.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stumper</ent><br/
<hw>Stump"er</hw> <pr>(?)</pr>, <pos>n.</pos> <sn>1.</sn> <def>One who stumps.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A boastful person.</def> <mark>[Slang]</mark><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>A puzzling or incredible story.</def> <mark>[Slang, U.S.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stumpiness</ent><br/
<hw>Stump"i*ness</hw> <pr>(?)</pr>, <pos>n.</pos> <def>The state of being stumpy.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stump-tailed</ent><br/
<hw>Stump"-tailed`</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Having a short, thick tail.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Stump-tailed lizard</b></col> <fld>(Zool.)</fld>, <cd>a singular Australian scincoid lizard (<spn>Trachydosaurus rugosus</spn>) having a short, thick tail resembling its head in form; -- called also <altname>sleeping lizard</altname>.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stumpy</ent><br/
<hw>Stump"y</hw> <pr>(?)</pr>, <pos>a.</pos> <sn>1.</sn> <def>Full of stumps; hard; strong.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Short and thick; stubby.</def> <mark>[Colloq.]</mark> <ldquo/A <xex>stumpy</xex> little man.<rdquo/  <rj><au>J. C. Harris.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stun</ent><br/
<hw>Stun</hw> <pr>(?)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Stunned</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Stunning</conjf>.]</vmorph> <ety>[OE. <ets>stonien</ets>, <ets>stownien</ets>; either fr. AS. <ets>stunian</ets> to resound (cf. D. <ets>stenen</ets> to groan, G. <ets>st<oum/hnen</ets>, Icel. <ets>stynja</ets>, Gr. <?/, Skr. <ets>stan</ets> to thunder, and E. <ets>thunder</ets>), or from the same source as E. <ets>astonish</ets>. <root/168.]</ety> <sn>1.</sn> <def>To make senseless or dizzy by violence; to render senseless by a blow, as on the head.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>One hung a poleax at his saddlebow,<br/
And one a heavy mace to <qex>stun</qex> the foe.</q> <rj><qau>Dryden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To dull or deaden the sensibility of; to overcome; especially, to overpower one's sense of hearing.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>And <qex>stunned</qex> him with the music of the spheres.</q> <rj><qau>Pope.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>To astonish; to overpower; to bewilder.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>William was quite <qex>stunned</qex> at my discourse.</q> <rj><qau>De Foe.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stun</ent><br/
<hw>Stun</hw>, <pos>n.</pos> <def>The condition of being stunned.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stundist</ent><br/
||<hw>Stun"dist</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Russ. <ets>shtundist</ets>, prob. fr. G. <ets>stunde</ets> hour; -- from their meetings for Bible reading.]</ety> <fld>(Eccl. Hist.)</fld> <def>One of a large sect of Russian dissenters founded, about 1860, in the village of Osnova, near Odessa, by a peasant, Onishchenko, who had apparently been influenced by a German sect settled near there. They zealously practice Bible reading and reject priestly dominion and all external rites of worship.</def> -- <wordforms><wf>Stun"dism</wf> <pr>(#)</pr>, <pos>n.</pos></wordforms><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Stung</ent><br/
<hw>Stung</hw> <pr>(?)</pr>, <def><pos>imp. & p. p.</pos> of <er>Sting</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stunk</ent><br/
<hw>Stunk</hw> <pr>(?)</pr>, <def><pos>imp. & p. p.</pos> of <er>Stink</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stunner</ent><br/
<hw>Stun"ner</hw> <pr>(?)</pr>, <pos>n.</pos> <sn>1.</sn> <def>One who, or that which, stuns.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Something striking or amazing in quality; something of extraordinary excellence.</def> <mark>[Slang]</mark>  <rj><au>Thackeray.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stunning</ent><br/
<hw>Stun"ning</hw> <pr>(?)</pr>, <pos>a.</pos> <sn>1.</sn> <def>Overpowering consciousness; overpowering the senses; especially, overpowering the sense of hearing; confounding with noise.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Striking or overpowering with astonishment, especially on account of excellence; <as>as, <ex>stunning</ex> poetry</as>.</def> <mark>[Slang]</mark> <au>C. Kingsley.</au> -- <wordforms><wf>Stun"ning*ly</wf>, <pos>adv.</pos> <mark>[Slang]</mark></wordforms><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stunsail</ent><br/
<hw>Stun"sail</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Naut.)</fld> <def>A contraction of <er>Studding sail</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>With every rag set, <qex>stunsails</qex>, sky scrapers and all.</q> <rj><qau>Lowell.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stunt</ent><br/
<hw>Stunt</hw> <pr>(?)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Stunted</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Stunting</conjf>.]</vmorph> <ety>[See <er>Stint</er>.]</ety> <def>To hinder from growing to the natural size; to prevent the growth of; to stint, to dwarf; <as>as, to <ex>stunt</ex> a child; to <ex>stunt</ex> a plant</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>When, by a cold penury, I blast the abilities of a nation, and <qex>stunt</qex> the growth of its active energies, the ill or may do is beyond all calculation.</q> <rj><qau>Burke.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stunt</ent><br/
<hw>Stunt</hw> <pr>(?)</pr>, <pos>n.</pos> <sn>1.</sn> <def>A check in growth; also, that which has been checked in growth; a stunted animal or thing.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Specifically: A whale two years old, which, having been weaned, is lean, and yields but little blubber.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stunt</ent><br/
<hw>Stunt</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Cf. <er>Stint</er> a task.]</ety> <sn>1.</sn> <def>A feat hard to perform; an act which is striking for the skill, strength, or the like, required to do it; a feat.</def> <mark>[Colloq.]</mark></p>

<p><q>An extraordinary man does three or four different <ldquo/<qex>stunts</qex><rdquo/ with remarkable dexterity.</q>  <rj><qau>The Bookman.</qau></rj></p>

<p><q>He does not try to do <qex>stunts</qex>; and, above all, he does not care to go in swimming.</q>  <rj><qau>L. Hutton.</qau></rj><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><sn>2.</sn> <def>an unusual action performed to gain public attention; <as>as, a publicity <ex>stunt</ex></as>.</def><br/
[<source>PJC</source>]</p>

<p><ent>Stunted</ent><br/
<hw>Stunt"ed</hw>, <pos>a.</pos> <def>Dwarfed.</def> -- <wordforms><wf>Stunt"ed*ness</wf>, <pos>n.</pos></wordforms><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stuntness</ent><br/
<hw>Stunt"ness</hw>, <pos>n.</pos> <def>Stuntedness; brevity.</def> <mark>[R.]</mark>  <rj><au>Earle.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stupa</ent><br/
||<hw>Stu"pa</hw> <pr>(st<oomac/"p<adot/)</pr>, <pos>n.</pos> <ety>[Skr. <ets>st<umac/pa</ets>.]</ety> <def>A mound or monument commemorative of Buddha.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stupa</ent><br/
||<hw>Stu"pa</hw> <pr>(st<umac/"p<adot/)</pr>, <pos>n.</pos> <ety>[L.]</ety> <fld>(Med.)</fld> <def>See 1st <er>Stupe</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stupe</ent><br/
<hw>Stupe</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>stupa</ets>, or better <ets>stuppa</ets>, tow.  Cf. <er>Stop</er>, <pos>v. t.</pos>]</ety> <fld>(Med.)</fld> <def>Cloth or flax dipped in warm water or medicaments and applied to a hurt or sore.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stupe</ent><br/
<hw>Stupe</hw>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Stuped</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Stuping</conjf>.]</vmorph> <def>To foment with a stupe.</def>  <rj><au>Wiseman.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stupe</ent><br/
<hw>Stupe</hw>, <pos>n.</pos> <ety>[See <er>Stupid</er>.]</ety> <def>A stupid person.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stupefacient</ent><br/
<hw>Stu`pe*fa"cient</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>stupefaciens</ets>, p. pr. of <ets>stupefacere</ets> to stupefy; <ets>stupere</ets> to be stupefied + <ets>facere</ets> to make.  Cf. <er>Stupefy</er>.]</ety> <altsp>[Written also <asp>stupifacient</asp>.]</altsp> <def>Producing stupefaction; stupefactive.</def> -- <def2><pos>n.</pos> <fld>(Med.)</fld> <def>Anything promoting stupefaction; a narcotic.</def></def2><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stupefaction</ent><br/
<hw>Stu`pe*fac"tion</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Cf. F. <ets>stup<eacute/faction</ets>. See <er>Stupefacient</er>.]</ety> <def>The act of stupefying, or the state of being stupefied.</def> <altsp>[Written also <asp>stupifaction</asp>.]</altsp><br/
[<source>1913 Webster</source>]</p>

<p><q>Resistance of the dictates of conscience brings a hardness and <qex>stupefaction</qex> upon it.</q> <rj><qau>South.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stupefactive</ent><br/
<hw>Stu`pe*fac"tive</hw> <pr>(?)</pr>, <pos>a. & n.</pos> <ety>[Cf. F. <ets>stup<eacute/factif</ets>, LL. <ets>stupefactivus</ets>.]</ety> <def>Same as <er>Stupefacient</er>.</def> <altsp>[Written also <asp>stupifactive</asp>.]</altsp><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stupefied</ent><br/
<hw>Stu"pe*fied</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Having been made stupid.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stupefiedness</ent><br/
<hw>Stu"pe*fied`ness</hw>, <pos>n.</pos> <def>Quality of being stupid.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stupefier</ent><br/
<hw>Stu"pe*fi`er</hw> <pr>(?)</pr>, <pos>n.</pos> <def>One who, or that which, stupefies; a stupefying agent.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stupefy</ent><br/
<hw>Stu"pe*fy</hw> <pr>(?)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Stupefied</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Stupefying</conjf> <pr>(?)</pr>.]</vmorph> <ety>[F. <ets>stup<eacute/fier</ets>, fr. L. <ets>stupere</ets> to be stupefied + <ets>ficare</ets> (in comp.) to make, akin to <ets>facere</ets>. See <er>Stupid</er>, <er>Fact</er>, and cf. <er>Stupefacient</er>.]</ety> <altsp>[Written also <asp>stupify</asp>, especially in England.]</altsp> <sn>1.</sn> <def>To make stupid; to make dull; to blunt the faculty of perception or understanding in; to deprive of sensibility; to make torpid.</def><-- temporarily! as by excessive dullness or repetition --><br/
[<source>1913 Webster</source>]</p>

<p><q>The fumes of drink discompose and <qex>stupefy</qex> the brain.</q> <rj><qau>South.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To deprive of material mobility.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>It is not malleable; but yet is not fluent, but <qex>stupefied</qex>.</q> <rj><qau>Bacon.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stupendous</ent><br/
<hw>Stu*pen"dous</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>stupendus</ets> astonishing, p. future pass. of <ets>stupere</ets> to be astonished at.  Cf. <er>Stupid</er>.]</ety> <def>Astonishing; wonderful; amazing; especially, astonishing in magnitude or elevation; <as>as, a <ex>stupendous</ex> pile</as>.</def> <ldquo/A <xex>stupendous</xex> sum.<rdquo/  <rj><au>Macaulay.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>All are but parts of one <qex>stupendous</qex> whole.</q> <rj><qau>Pope.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p>-- <wordforms><wf>Stu*pen"dous*ly</wf>, <pos>adv.</pos> -- <wf>Stu*pen"dous*ness</wf>, <pos>n.</pos></wordforms><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stupeous</ent><br/
<hw>Stu"pe*ous</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>stupa</ets>, or better <ets>stuppa</ets>, tow; cf. L. <ets>stuppeus</ets> made of tow.  Cf. <er>Stupose</er>.]</ety> <def>Resembling tow; having long, loose scales, or matted filaments, like tow; stupose.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stupid</ent><br/
<hw>Stu"pid</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>stupidus</ets>, fr. <ets>stupere</ets> to be stupefied: cf. F. <ets>stupide</ets>.]</ety> <sn>1.</sn> <def>Very dull; insensible; senseless; wanting in understanding; heavy; sluggish; in a state of stupor; -- said of persons.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>O that men . . . should be so <qex>stupid</qex> grown . . . <br/
As to forsake the living God!</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>With wild surprise,<br/
A moment <qex>stupid</qex>, motionless he stood.</q> <rj><qau>Thomson.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Resulting from, or evincing, stupidity; formed without skill or genius; dull; heavy; -- said of things.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Observe what loads of <qex>stupid</qex> rhymes<br/
Oppress us in corrupted times.</q> <rj><qau>Swift.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><syn><b>Syn.</b> -- Simple; insensible; sluggish; senseless; doltish; sottish; dull; heavy; clodpated.</syn>
 -- <wordforms><wf>Stu"pid*ly</wf> <pr>(#)</pr>, <pos>adv.</pos> -- <wf>Stu"pid*ness</wf>, <pos>n.</pos></wordforms><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stupidity</ent><br/
<hw>Stu*pid"i*ty</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>stupiditas</ets>: cf. F. <ets>stupidit<eacute/</ets>.]</ety> <sn>1.</sn> <def>The quality or state of being stupid; extreme dullness of perception or understanding; insensibility; sluggishness.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Stupor; astonishment; stupefaction.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>A <qex>stupidity</qex><br/
Past admiration strikes me, joined with fear.</q> <rj><qau>Chapman.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stupify</ent><br/
<hw>Stu"pi*fy</hw> <pr>(?)</pr>, <pos>v. t.</pos> <def>See <er>Stupefy</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stupor</ent><br/
<hw>Stu"por</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L., from <ets>stupere</ets> to be struck senseless.]</ety> <sn>1.</sn> <def>Great diminution or suspension of sensibility; suppression of sense or feeling; lethargy.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Intellectual insensibility; moral stupidity; heedlessness or inattention to one's interests.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stupose</ent><br/
<hw>Stu*pose</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>stupa</ets>, or better <ets>stuppa</ets>, tow.  Cf. <er>Stupeous</er>.]</ety> <fld>(Bot.)</fld> <def>Composed of, or having, tufted or matted filaments like tow; stupeous.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stuprate</ent><br/
<hw>Stu"prate</hw> <pr>(?)</pr>, <pos>v. t.</pos> <ety>[L. <ets>stupratus</ets>, p. p. of <ets>stuprare</ets> to ravish, fr. <ets>stuprum</ets> defilement.]</ety> <def>To ravish; to debauch.</def> <mark>[R.]</mark>  <rj><au>Heywood.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stupration</ent><br/
<hw>Stu*pra"tion</hw> <pr>(?)</pr>, <pos>n.</pos> <def>Violation of chastity by force; rape.</def> <mark>[R.]</mark>  <rj><au>Sir T. Browne.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stuprum</ent><br/
||<hw>Stu"prum</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L.]</ety> <def>Stupration.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sturb</ent><br/
<hw>Sturb</hw> <pr>(?)</pr>, <pos>v. t.</pos> <def>To disturb.</def> <mark>[Obs.]</mark>  <rj><au>Chaucer.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sturdily</ent><br/
<hw>Stur"di*ly</hw> <pr>(?)</pr>, <pos>adv.</pos> <def>In a sturdy manner.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sturdiness</ent><br/
<hw>Stur"di*ness</hw>, <pos>n.</pos> <def>Quality of being sturdy.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sturdy</ent><br/
<hw>Stur"dy</hw> <pr>(?)</pr>, <pos>a.</pos> <amorph>[<pos>Compar.</pos> <adjf>Sturdier</adjf> <pr>(?)</pr>; <pos>superl.</pos> <adjf>Sturdiest</adjf>.]</amorph> <ety>[OE. <ets>sturdi</ets> inconsiderable, OF. <ets>estourdi</ets> stunned, giddy, thoughtless, rash, F. <ets><eacute/tourdi</ets>, p. p. of OF. <ets>estourdir</ets> to stun, to render giddy, to amaze, F. <ets><eacute/tourdir</ets>; of uncertain origin. The sense has probably been influenced by E. <ets>stout</ets>.]</ety> <sn>1.</sn> <def>Foolishly obstinate or resolute; stubborn; unrelenting; unfeeling; stern.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>This <qex>sturdy</qex> marquis gan his hearte dress<br/
To rue upon her wifely steadfastness.</q> <rj><qau>Chaucer.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>This must be done, and I would fain see<br/
Mortal so <qex>sturdy</qex> as to gainsay.</q> <rj><qau>Hudibras.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>A <qex>sturdy</qex>, hardened sinner shall advance to the utmost pitch of impiety with less reluctance than he took the first steps.</q> <rj><qau>Atterbury.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Resolute, in a good sense; or firm, unyielding quality; <as>as, a man of <ex>sturdy</ex> piety or patriotism</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>Characterized by physical strength or force; strong; lusty; violent; <as>as, a <ex>sturdy</ex> lout</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>How bowed the woods beneath their <qex>sturdy</qex> stroke!</q> <rj><qau>Gray.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>Stiff; stout; strong; <as>as, a <ex>sturdy</ex> oak</as>.</def>  <rj><au>Milton.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>He was not of any delicate contexture; his limbs rather <qex>sturdy</qex> than dainty.</q> <rj><qau>Sir H. Wotton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><syn><b>Syn.</b> -- Hardy; stout; strong; firm; robust; stiff.</syn><br/
[<source>1913 Webster</source>]</p>

<p><-- p. 1431 --></p>

<p><ent>Sturdy</ent><br/
<hw>Stur"dy</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[OF. <ets>estourdi</ets> giddiness, stupefaction.]</ety> <fld>(Vet.)</fld> <def>A disease in sheep and cattle, marked by great nervousness, or by dullness and stupor.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sturgeon</ent><br/
<hw>Stur"geon</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F. <ets>esturgeon</ets>, LL. <ets>sturio</ets>, <ets>sturgio</ets>, OHG. <ets>sturjo</ets>, G. <ets>st<oum/r</ets>; akin to AS. <ets>styria</ets>, <ets>styriga</ets>.]</ety> <fld>(Zool.)</fld> <def>Any one of numerous species of large cartilaginous ganoid fishes belonging to <gen>Acipenser</gen> and allied genera of the family <fam>Acipenseridae</fam>. They run up rivers to spawn, and are common on the coasts and in the large rivers and lakes of North America, Europe, and Asia.  Caviar is prepared from the roe, and isinglass from the air bladder.</def><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ The common North American species are <spn>Acipenser sturio</spn> of the Atlantic coast region, <spn>Acipenser transmontanus</spn> of the Pacific coast, and <spn>Acipenser rubicundus</spn> of the Mississippi River and its tributaries. In Europe, the common species is <spn>Acipenser sturio</spn>, and other well-known species are the sterlet and the huso.
    The sturgeons are included in the order <ord>Chondrostei</ord>. Their body is partially covered by five rows of large, carinated, bony plates, of which one row runs along the back. The tail is heterocercal. The toothless and protrusile mouth is beneath the head, and has four barbels in front.</note><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Shovel-nosed sturgeon</b></col>. <fld>(Zool.)</fld> <cd>See <er>Shovelnose</er> <sd>(d)</sd>.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sturiones</ent><br/
||<hw>Stu`ri*o"nes</hw> <pr>(?)</pr>, <pos>n. pl.</pos> <ety>[NL., from LL. <ets>sturio</ets>. See <er>Sturgeon</er>.]</ety> <fld>(Zool.)</fld> <def>An order of fishes including the sturgeons.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sturionian</ent><br/
<hw>Stu`ri*o"ni*an</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Zool.)</fld> <def>One of the family of fishes of which the sturgeon is the type.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sturk</ent><br/
<hw>Sturk</hw> <pr>(?)</pr>, <pos>n.</pos> <def>See <er>Stirk</er>.</def> <mark>[Prov. Eng. & Scot.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sturnoid</ent><br/
<hw>Stur"noid</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>sturnus</ets> a starling + <ets>-oid</ets>.]</ety> <fld>(Zool.)</fld> <def>Like or pertaining to the starlings.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sturt</ent><br/
<hw>Sturt</hw> <pr>(?)</pr>, <pos>v. t.</pos> <ety>[Cf. <er>Start</er>, <pos>v. i.</pos>]</ety> <def>To vex; to annoy; to startle.</def> <mark>[Obs. or Prov. Eng. & Scot.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sturt</ent><br/
<hw>Sturt</hw>, <pos>n.</pos> <sn>1.</sn> <def>Disturbance; annoyance; care.</def> <mark>[Obs. or Prov. Eng. & Scot.]</mark> <ldquo/<xex>Sturt</xex> and care.<rdquo/  <rj><au>J. Rolland.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Mining)</fld> <def>A bargain in tribute mining by which the tributor profits.</def>  <rj><au>Raymond.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sturtion</ent><br/
<hw>Stur"tion</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A corruption of <er>Nasturtion</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stut</ent><br/
<hw>Stut</hw> <pr>(?)</pr>, <pos>v. i.</pos> <def>To stutter.</def> <mark>[Obs.]</mark>  <rj><au>Skelton.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stutter</ent><br/
<hw>Stut"ter</hw> <pr>(?)</pr>, <pos>v. t. & i.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Stuttered</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Stuttering</conjf>.]</vmorph> <ety>[Freq. of <ets>stut</ets>, OE. <ets>stoten</ets>; probably of Dutch or Low German origin; cf. D. & LG. <ets>stotteren</ets>, G. <ets>stottern</ets>, D. <ets>stooten</ets> to push, to strike; akin to G. <ets>stossen</ets>, Icel. <ets>stauta</ets>, Sw. <ets>st<oum/ta</ets>, Dan. <ets>st<oum/de</ets>, Goth. <ets>stautan</ets>, L. <ets>tundere</ets>, Skr. <ets>tud</ets> to thrust.  Cf. <er>Contuse</er>, <er>Obtuse</er>.]</ety> <def>To hesitate or stumble in uttering words; to speak with spasmodic repetition or pauses; to stammer.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Trembling, <qex>stuttering</qex>, calling for his confessor.</q> <rj><qau>Macaulay.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stutter</ent><br/
<hw>Stut"ter</hw>, <pos>n.</pos> <sn>1.</sn> <def>The act of stuttering; a stammer. See <er>Stammer</er>, and <er>Stuttering</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>One who stutters; a stammerer.</def> <mark>[Obs.]</mark>  <rj><au>Bacon.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stutterer</ent><br/
<hw>Stut"ter*er</hw> <pr>(?)</pr>, <pos>n.</pos> <def>One who stutters; a stammerer.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stuttering</ent><br/
<hw>Stut"ter*ing</hw>, <pos>n.</pos> <def>The act of one who stutters; -- restricted by some physiologists to defective speech due to inability to form the proper sounds, the breathing being normal, as distinguished from <xex>stammering</xex>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stuttering</ent><br/
<hw>Stut"ter*ing</hw>, <pos>a.</pos> <def>Apt to stutter; hesitating; stammering.</def> -- <wordforms><wf>Stut"ter*ing*ly</wf>, <pos>adv.</pos></wordforms><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sty</ent><br/
<hw>Sty</hw> <pr>(st<imac/)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Sties</plw> <pr>(st<imac/z)</pr>.</plu> <altsp>[Written also <asp>stigh</asp>.]</altsp> <ety>[AS. <ets>stigu</ets>, fr. <ets>st<imac/gan</ets> to rise; originally, probably, a place into which animals climbed or went up. <root/164. See <er>Sty</er>, <pos>v. i.</pos>, and cf. <er>Steward</er>.]</ety> <sn>1.</sn> <def>A pen or inclosure for swine.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A place of bestial debauchery.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>To roll with pleasure in a sensual <qex>sty</qex>.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sty</ent><br/
<hw>Sty</hw>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Stied</conjf> <pr>(st<imac/d)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Stying</conjf> <pr>(st<imac/"<icr/ng)</pr>.]</vmorph> <def>To shut up in, or as in, a sty.</def>  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sty</ent><br/
<hw>Sty</hw>, <pos>v. i.</pos> <ety>[OE. <ets>stien</ets>, <ets>sti<yogh/en</ets>, AS. <ets>st<imac/gan</ets> to rise; akin to D. <ets>stijgen</ets>, OS. & OHG. <ets>st<imac/gan</ets>, G. <ets>steigen</ets>, Icel. <ets>st<imac/ga</ets>, Sw. <ets>stiga</ets>, Dan. <ets>stige</ets>, Goth. <ets>steigan</ets>, L. <ets>vestigium</ets> footstep, Gr. <?/ to walk, to go, Skr. <ets>stigh</ets> to mount.  Cf. <er>Distich</er>, <er>Stair</er> steps, <er>Stirrup</er>, <er>Sty</er> a boil, a pen for swine, <er>Vestige</er>.]</ety> <def>To soar; to ascend; to mount. See <er>Stirrup</er>.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>With bolder wing shall dare aloft to <qex>sty</qex>,<br/
To the last praises of this Faery Queene.</q> <rj><qau>Spenser.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sty</ent><br/
<hw>Sty</hw>, <pos>n.</pos> <ety>[For older <ets>styan</ets>, <ets>styanye</ets>, understood as <ets>sty on eye</ets>, AS. <ets>st<imac/gend</ets> (sc. <ets>e<aacute/ge</ets> eye), properly, rising, or swelling (eye), p. p. of <ets>st<imac/gan</ets> to rise. See <er>Sty</er>, <pos>v. i.</pos>]</ety> <fld>(Med.)</fld> <def>An inflamed swelling or boil on the edge of the eyelid.</def> <altsp>[Written also <asp>stye</asp>.]</altsp><br/
[<source>1913 Webster</source>]</p>

<p><ent>Styan</ent><br/
<hw>Sty"an</hw> <pr>(?)</pr>, <pos>n.</pos> <def>See <er>Sty</er>, a boil.</def> <mark>[R.]</mark>  <rj><au>De quincey.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Styca</ent><br/
<hw>Sty"ca</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[LL., fr. AS. <ets>stic</ets>, <ets>styc</ets>, <ets>stycge</ets>.]</ety> <def>An anglo-Saxon copper coin of the lowest value, being worth half a farthing.</def>  <rj><au>S. M. Leake.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stycerin</ent><br/
<hw>Sty"cer*in</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[<ets>Sty</ets>ryl + gly<ets>cerin</ets>.]</ety> <fld>(Chem.)</fld> <def>A triacid alcohol, related to glycerin, and obtained from certain styryl derivatives as a yellow, gummy, amorphous substance; -- called also <altname>phenyl glycerin</altname>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stye</ent><br/
<hw>Stye</hw>, <pos>n.</pos> <def>See <er>Sty</er>, a boil.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stygial</ent><br/
<hw>Styg"i*al</hw> <pr>(st<icr/j"<icr/*<ait/l)</pr>, <pos>a.</pos> <def>Stygian.</def> <mark>[R.]</mark>  <rj><au>Skelton.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stygian</ent><br/
<hw>Styg"i*an</hw> <pr>(st<icr/j"<icr/*<ait/n)</pr>, <pos>a.</pos> <ety>[L. <ets>Stygius</ets>, fr. <ets>Styx</ets>, <ets>Stygis</ets>, Gr. <grk>Sty`x</grk>, <grk>Stygo`s</grk>, the Styx.]</ety> <def>Of or pertaining to the river Styx; hence, hellish; infernal. See <er>Styx</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>At that so sudden blaze, the <qex>Stygian</qex> throng<br/
Bent their aspect.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stylagalmaic</ent><br/
<hw>Sty`la*gal*ma"ic</hw> <pr>(st<imac/`l<adot/*g<acr/l*m<amac/"<icr/k)</pr>, <pos>a.</pos> <ety>[Gr. <grk>sty^los</grk> a column + <grk>'a`galma</grk> an image.]</ety> <fld>(Arch.)</fld> <def>Performing the office of columns; <as>as, Atlantes and Caryatides are <ex>stylagalmaic</ex> figures or images</as>.</def> <altsp>[Written also <asp>stylogalmaic</asp>.]</altsp><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stylar</ent><br/
<hw>Sty"lar</hw> <pr>(?)</pr>, <pos>a.</pos> <def>See <er>Stilar</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stylaster</ent><br/
||<hw>Sty*las"ter</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[NL., from Gr. <grk>sty^los</grk> pillar + <grk>'asth`r</grk> star.]</ety> <fld>(Zool.)</fld> <def>Any one of numerous species of delicate, usually pink, calcareous hydroid corals of the genus <gen>Stylaster</gen>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Style</ent><br/
<hw>Style</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[OE. <ets>stile</ets>, F. <ets>style</ets>, Of. also <ets>stile</ets>, L. <ets>stilus</ets> a style or writing instrument, manner or writing, mode of expression; probably for <ets>stiglus</ets>, meaning, a pricking instrument, and akin to E. <ets>stick</ets>. See <er>Stick</er>, <pos>v. t.</pos>, and cf. <er>Stiletto</er>. The spelling with <it>y</it> is due to a supposed connection with Gr. <grk>sty^los</grk> a pillar.]</ety> <sn>1.</sn> <def>An instrument used by the ancients in writing on tablets covered with wax, having one of its ends sharp, and the other blunt, and somewhat expanded, for the purpose of making erasures by smoothing the wax.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Hence, anything resembling the ancient style in shape or use.</def> Specifically: --<br/
[<source>1913 Webster</source>]</p>

<p><sd>(a)</sd> <def>A pen; an author's pen.</def>  <rj><au>Dryden.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sd>(b)</sd> <def>A sharp-pointed tool used in engraving; a graver.</def><br/
[<source>1913 Webster</source>]</p>

<p><sd>(c)</sd> <def>A kind of blunt-pointed surgical instrument.</def><br/
[<source>1913 Webster</source>]</p>

<p><sd>(d)</sd> <fld>(Zool.)</fld> <def>A long, slender, bristlelike process, as the anal <xex>styles</xex> of insects.</def><br/
[<source>1913 Webster</source>]</p>

<p><sd>(e)</sd> <ety>[Perhaps fr. Gr. <grk>sty^los</grk> a pillar.]</ety> <def>The pin, or gnomon, of a dial, the shadow of which indicates the hour. See <er>Gnomon</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sd>(f)</sd> <ety>[Probably fr. Gr. <grk>sty^los</grk> a pillar.]</ety> <fld>(Bot.)</fld> <def>The elongated part of a pistil between the ovary and the stigma. See <xex>Illust.</xex> of <er>Stamen</er>, and of <er>Pistil</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>Mode of expressing thought in language, whether oral or written; especially, such use of language in the expression of thought as exhibits the spirit and faculty of an artist; choice or arrangement of words in discourse; rhetorical expression.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>High <qex>style</qex>, as when that men to kinges write.</q> <rj><qau>Chaucer.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q><qex>Style</qex> is the dress of thoughts.</q> <rj><qau>Chesterfield.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Proper words in proper places make the true definition of <qex>style</qex>.</q> <rj><qau>Swift.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>It is <qex>style</qex> alone by which posterity will judge of a great work.</q> <rj><qau>I. Disraeli.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>Mode of presentation, especially in music or any of the fine arts; a characteristic of peculiar mode of developing in idea or accomplishing a result.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The ornamental <qex>style</qex> also possesses its own peculiar merit.</q> <rj><qau>Sir J. Reynolds.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <def>Conformity to a recognized standard; manner which is deemed elegant and appropriate, especially in social demeanor; fashion.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>According to the usual <qex>style</qex> of dedications.</q> <rj><qau>C. Middleton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>6.</sn> <def>Mode or phrase by which anything is formally designated; the title; the official designation of any important body; mode of address; <as>as, the <ex>style</ex> of Majesty</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>One <qex>style</qex> to a gracious benefactor, another to a proud, insulting foe.</q> <rj><qau>Burke.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>7.</sn> <fld>(Chron.)</fld> <def>A mode of reckoning time, with regard to the Julian and Gregorian calendars.</def><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ <xex>Style</xex> is <xex>Old</xex> or <xex>New</xex>. The <xex>Old</xex> Style follows the Julian manner of computing the months and days, or the calendar as established by Julius Caesar, in which every fourth year consists of 366 days, and the other years of 365 days. This is about 11 minutes in a year too much. Pope Georgy XIII. reformed the calendar by retrenching 10 days in October, 1582, in order to bring back the vernal equinox to the same day as at the time of the Council of Nice, <sc>a. d.</sc> 325. This reformation was adopted by act of the British Parliament in 1751, by which act 11 days in September, 1752, were retrenched, and the third day was reckoned the fourteenth. This mode of reckoning is called <xex>New</xex> Style, according to which every year divisible by 4, unless it is divisible by 100 without being divisible by 400, has 366 days, and any other year 365 days.</note><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Style of court</b></col>, <cd>the practice or manner observed by a court in its proceedings.</cd> <au>Ayliffe.</au></cs><br/
[<source>1913 Webster</source>]</p>

<p><syn><b>Syn.</b> -- Diction; phraseology; manner; course; title. See <er>Diction</er>.</syn><br/
[<source>1913 Webster</source>]</p>

<p><ent>Style</ent><br/
<hw>Style</hw>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Styled</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Styling</conjf>.]</vmorph> <def>To entitle; to term, name, or call; to denominate.</def> <ldquo/<xex>Styled</xex> great conquerors.<rdquo/  <rj><au>Milton.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>How well his worth and brave adventures <qex>styled</qex>.</q> <rj><qau>Dryden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><syn><b>Syn.</b> -- To call; name; denominate; designate; term; characterize.</syn><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stylet</ent><br/
<hw>Sty"let</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F., dim. of <ets>style</ets>; cf. It. <ets>stiletto</ets>. See <er>Stiletto</er>.]</ety> <def>A small poniard; a stiletto.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Surg.)</fld> <sd>(a)</sd> <def>An instrument for examining wounds and fistulas, and for passing setons, and the like; a probe, -- called also <altname>specillum</altname>.</def> <sd>(b)</sd> <def>A stiff wire, inserted in catheters or other tubular instruments to maintain their shape and prevent clogging.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <fld>(Zool.)</fld> <def>Any small, more or less rigid, bristlelike organ; <as>as, the caudal <ex>stylets</ex> of certain insects; the ventral <ex>stylets</ex> of certain Infusoria</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Styliferous</ent><br/
<hw>Sty*lif"er*ous</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[<ets>Style</ets> + <ets>-ferous</ets>.]</ety> <fld>(Bot.)</fld> <def>Bearing one or more styles.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Styliform</ent><br/
<hw>Sty"li*form</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[<ets>Style</ets> + <ets>-form</ets>: cf. F. <ets>styliforme</ets>.]</ety> <def>Having the form of, or resembling, a style, pin, or pen; styloid.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stylish</ent><br/
<hw>Styl"ish</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Having style or artistic quality; given to, or fond of, the display of style; highly fashionable; modish; <as>as, a <ex>stylish</ex> dress, house, manner</as>.</def> -- <wordforms><wf>Styl"ish*ly</wf>, <pos>adv.</pos> -- <wf>Styl"ish*ness</wf>, <pos>n.</pos></wordforms><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stylist</ent><br/
<hw>Styl"ist</hw>, <pos>n.</pos> <def>One who is a master or a model of style, especially in writing or speaking; a critic of style.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Distinguished as a <qex>stylist</qex>, for ease.</q> <rj><qau>Fitzed. Hall.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stylistic</ent><br/
<hw>Sty*lis"tic</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Of or pertaining to style in language.</def> <mark>[R.]</mark> <ldquo/<xex>Stylistic</xex> trifles.<rdquo/  <rj><au>J. A. Symonds.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>The great <qex>stylistic</qex> differences in the works ascribed to him [Wyclif].</q> <rj><qau>G. P. Marsh.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stylite</ent><br/
<hw>Sty"lite</hw> <pr>(st<imac/"l<imac/t)</pr>, <pos>n.</pos> <ety>[Gr. <grk>styli`ths</grk>, fr. <grk>sty^los</grk> a pillar.]</ety> <fld>(Eccl. Hist.)</fld> <def>One of a sect of anchorites in the early church, who lived on the tops of pillars for the exercise of their patience; -- called also <altname>pillarist</altname> and <altname>pillar saint</altname>.</def><-- precursors of the flagpole-sitters! --><br/
[<source>1913 Webster</source>]</p>

<p><q>The two other holy men in Gregory's narrative had more exotic origins than the pair that has just been seen.  Gregory encountered one of them when on a journey to the north-eastern parts of the Frankish kingdom.  This was a Lombard, named Vulfolaic, who had spent some years in the arduous exercise of being a <ex>stylite</ex>, the Christian equivalent of a flagpole sitter; in other words, Vulfolaic was a monk whose main austerity consisted in living on top of a pillar.  By carrying out this feat in the rain, snow, and frost of the Moselle valley, Vulfolaic had convinced the local population to overthrow and abandon the idol of Diana to which they were addicted.</q> <rj><qau>Walter Goffart, FOREIGNERS IN THE HISTORIES OF GREGORY OF TOURS (http://www.arts.uwo.ca/florilegium/goffart.html).</qau></rj></p>

<p><ent>Stylo-</ent><br/
<hw>Sty"lo-</hw> <pr>(st<imac/"l<osl/-)</pr>. <def>A combining form used in anatomy to indicate <xex>connection with</xex>, or <xex>relation to</xex>, <xex>the styloid process of the temporal bone</xex>; <as>as, <ex>stylo</ex>hyal, <ex>stylo</ex>mastoid, <ex>stylo</ex>maxillary</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stylobate</ent><br/
<hw>Sty"lo*bate</hw> <pr>(st<imac/"l<osl/*b<asl/t)</pr>, <pos>n.</pos> <ety>[L. <ets>stylobates</ets>, <ets>stylobata</ets>, Gr. <grk>styloba`ths</grk>; <grk>sty^los</grk> a pillar + <grk>ba`ths</grk> one that treads, fr. <grk>bai`nein</grk> to go.]</ety> <fld>(Arch.)</fld> <def>The uninterrupted and continuous flat band, coping, or pavement upon which the bases of a row of columns are supported. See <er>Sub-base</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Styloglossal</ent><br/
<hw>Sty`lo*glos"sal</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[<ets>Stylo-</ets> + <ets>glossal</ets>.]</ety> <fld>(Anat.)</fld> <def>Of or pertaining to styloid process and the tongue.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stylograph</ent><br/
<hw>Sty"lo*graph</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A stylographic pen.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stylographic</ent><br/
<hw>Sty`lo*graph"ic</hw> <pr>(?)</pr>, <pos>a.</pos> <sn>1.</sn> <def>Of or pertaining to stylography; used in stylography; <as>as, <ex>stylographic</ex> tablets</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Pertaining to, or used in, stylographic pen; <as>as, <ex>stylographic</ex> ink</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Stylographic pen</b></col>, <cd>a pen with a conical point like that of a style, combined with a reservoir for supplying it with ink.</cd> -- <col><b>Stylographic pencil</b></col>, <cd>a pencil used in stylography.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stylographical</ent><br/
<hw>Sty`lo*graph"ic*al</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Same as <er>Stylographic</er>, 1.</def> -- <wordforms><wf>Sty`lo*graph"ic*al*ly</wf>, <pos>adv.</pos></wordforms><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stylography</ent><br/
<hw>Sty*log"ra*phy</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[<ets>Style</ets> + <ets>-graphy</ets>.]</ety> <def>A mode of writing or tracing lines by means of a style on cards or tablets.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stylohyal</ent><br/
<hw>Sty`lo*hy"al</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[<ets>Stylo-</ets> + the Gr. letter <?/.]</ety> <fld>(Anat.)</fld> <def>A segment in the hyoidean arch between the epihyal and tympanohyal.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stylohyoid</ent><br/
<hw>Sty`lo*hy"oid</hw> <pr>(?)</pr>, <pos>a.</pos> <fld>(Anat.)</fld> <def>Of or pertaining to the styloid process and the hyoid bone.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Styloid</ent><br/
<hw>Sty"loid</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[<ets>Style</ets> + <ets>-oid</ets>: cf. F. <ets>stylo<ium/de</ets>, Gr. <?/.]</ety> <sn>1.</sn> <def>Styliform; <as>as, the <ex>styloid</ex> process</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Anat.)</fld> <def>Of or pertaining to the styloid process.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Styloid process</b></col> <fld>(Anat.)</fld>, <cd>a long and slender process from the lower side of the temporal bone of man, corresponding to the tympanohyal and stylohyal of other animals.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stylomastoid</ent><br/
<hw>Sty`lo*mas"toid</hw> <pr>(?)</pr>, <pos>a.</pos> <fld>(Anat.)</fld> <def>Of or pertaining to the styloid and mastoid processes of the temporal bone.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stylomaxillary</ent><br/
<hw>Sty`lo*max"il*la*ry</hw> <pr>(?)</pr>, <pos>a.</pos> <fld>(Anat.)</fld> <def>Of or pertaining to the styloid process and the maxilla.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stylometer</ent><br/
<hw>Sty*lom"e*ter</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Gr. <?/ column + <ets>-meter</ets>.]</ety> <def>An instrument for measuring columns.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stylommata</ent><br/
||<hw>Sty*lom"ma*ta</hw> <pr>(?)</pr>, <pos>n. pl.</pos> <ety>[NL., fr. Gr. <?/ a pillar + <?/, <?/, the eye.]</ety> <def>Same as <er>Stylommatophora</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stylommatophora</ent><br/
||<hw>Sty*lom`ma*toph"o*ra</hw> <pr>(?)</pr>, <pos>n. pl.</pos> <ety>[NL., fr. Gr. <?/ column + <?/ eye + <grk>fe`rein</grk> to bear.]</ety> <fld>(Zool.)</fld> <def>A division of Pulmonata in which the eyes are situated at the tips of the tentacles. It includes the common land snails and slugs. See <xex>Illust.</xex> under <er>Snail</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stylommatophorous</ent><br/
<hw>Sty*lom`ma*toph"o*rous</hw> <pr>(?)</pr>, <pos>a.</pos> <fld>(Zool.)</fld> <def>Of or pertaining to Stylommatophora.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stylopodium</ent><br/
||<hw>Sty`lo*po"di*um</hw> <pr>(?)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Stylopodia</plw> <pr>(#)</pr>.</plu> <ety>[NL. See <er>Style</er>, and <er>Podium</er>.]</ety> <fld>(Bot.)</fld> <def>An expansion at the base of the style, as in umbelliferous plants.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stylops</ent><br/
||<hw>Sty"lops</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[NL., fr. Gr. <?/ a pillar + <?/, <?/, the eye.]</ety> <fld>(Zool.)</fld> <def>A genus of minute insects parasitic, in their larval state, on bees and wasps. It is the typical genus of the group Strepsiptera, formerly considered a distinct order, but now generally referred to the Coleoptera. See <er>Strepsiptera</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stylus</ent><br/
||<hw>Sty"lus</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>stylus</ets>, or better <ets>stilus</ets>.]</ety> <sn>1.</sn> <def>An instrument for writing. See <er>Style</er>, <pos>n.</pos>, 1.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>That needle-shaped part at the tip of the playing arm of phonograph which sits in the groove of a phonograph record while it is turning, to detect the undulations in the phonograph groove and convert them into vibrations which are transmitted to a system (since 1920 electronic) which converts the signal into sound; also called <altname>needle</altname>.  The <ex>stylus</ex> is frequently composed of a hard metal or of diamond.</def><br/
[<source>PJC</source>]</p>

<p><sn>3.</sn> <def>The needle-like device used to cut the grooves which record the sound on the original disc during recording of a phonograph record; it is moved by the vibrations given to the diaphragm by a sound, and produces the indented record.</def><br/
[<source>PJC</source>]</p>

<p><sn>4.</sn> <fld>(Computers)</fld> <def>A pen-shaped pointing device used to specify the cursor position on a graphics tablet.</def><br/
[<source>PJC</source>]</p>

<p><ent>Stimy</ent><br/
<ent>Stymie</ent><br/
<mhw>{ <hw>Sty"mie</hw> <pr>(?)</pr>, <pos>n.</pos> Also <hw>Sti"my</hw>  }</mhw>. <ety>[Orig. uncertain.]</ety> <fld>(Golf)</fld> <def>The position of two balls on the putting green such that, being more than six inches apart, one ball lies directly between the other and the hole at which the latter must be played; also, the act of bringing the balls into this position.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Stimy</ent><br/
<ent>Stymie</ent><br/
<mhw>{ <hw>Sty"mie</hw>, <pos>v. t.</pos> Also <hw>Sti"my</hw>  }</mhw>. <fld>(Golf)</fld> <def>To bring into the position of, or impede by, a stymie.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Styphnate</ent><br/
<hw>Styph"nate</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Chem.)</fld> <def>A salt of styphnic acid.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Styphnic</ent><br/
<hw>Styph"nic</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[Gr. (spurious) <grk>sty`fein</grk> to contract.]</ety> <fld>(Chem.)</fld> <def>Pertaining to, or designating, a yellow crystalline astringent acid, <chform>(NO2)3.C6H.(OH)2</chform>, obtained by the action of nitric acid on resorcin. <xex>Styphnic acid</xex> resembles picric acid, but is not bitter. It acts like a strong dibasic acid, having a series of well defined salts.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Styptic</ent><br/
<hw>Styp"tic</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>stypticus</ets>, Gr. <?/, fr. <?/ to contract.]</ety> <def>Producing contraction; stopping bleeding; having the quality of restraining hemorrhage when applied to the bleeding part; astringent.</def> <altsp>[Written also <asp>stiptic</asp>.]</altsp><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Styptic weed</b></col> <fld>(Bot.)</fld>, <cd>an American leguminous herb (<spn>Cassia occidentalis</spn>) closely related to the wild senna.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Styptic</ent><br/
<hw>Styp"tic</hw>, <pos>n.</pos> <fld>(Med.)</fld> <def>A styptic medicine.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Styptical</ent><br/
<hw>Styp"tic*al</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Styptic; astringent.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stypticity</ent><br/
<hw>Styp*tic"i*ty</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Cf. F. <ets>stypticit<eacute/</ets>.]</ety> <def>The quality or state of being styptic; astringency.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Styracin</ent><br/
<hw>Styr"a*cin</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[See <er>Styrax</er>.]</ety> <fld>(Chem.)</fld> <def>A white crystalline tasteless substance extracted from gum storax, and consisting of a salt of cinnamic acid with cinnamic alcohol.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Styrax</ent><br/
<hw>Sty"rax</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>styrax</ets>, <ets>storax</ets>, Gr. <?/. See <er>Storax</er>.]</ety> <sn>1.</sn> <fld>(Bot.)</fld> <def>A genus of shrubs and trees, mostly American or Asiatic, abounding in resinous and aromatic substances. <spn>Styrax officinalis</spn> yields storax, and <spn>Styrax Benzoin</spn> yields benzoin.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Same as <er>Storax</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Styrol</ent><br/
<hw>Sty"rol</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[<ets>Styr</ets>ax + L. <ets>ol</ets>eum oil.]</ety> <fld>(Chem.)</fld> <def>See <er>Styrolene</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Styrolene</ent><br/
<hw>Sty"ro*lene</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Chem.)</fld> <def>An unsaturated hydrocarbon, <chform>C8H8</chform>, obtained by the distillation of storax, by the decomposition of cinnamic acid, and by the condensation of acetylene, as a fragrant, aromatic, mobile liquid; -- called also <altname>phenyl ethylene</altname>, <altname>vinyl benzene</altname>, <altname>styrol</altname>, <altname>styrene</altname>, and <altname>cinnamene</altname>.</def><-- most commonly, styrene --><br/
[<source>1913 Webster</source>]</p>

<p><ent>Styrone</ent><br/
<hw>Sty"rone</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Chem.)</fld> <def>A white crystalline substance having a sweet taste and a hyacinthlike odor, obtained by the decomposition of styracin; -- properly called <altname>cinnamic alcohol</altname> <it>or</it>  <altname>styryl alcohol</altname>.</def><br/
[<source>1913 Webster</source>]</p>

<p><-- p. 1432 --></p>

<p><ent>Styryl</ent><br/
<hw>Sty"ryl</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[<ets>Styr</ets>ax + <ets>-yl</ets>.]</ety> <fld>(Chem.)</fld> <def>A hypothetical radical found in certain derivatives of styrolene and cinnamic acid; -- called also <altname>cinnyl</altname>, or <altname>cinnamyl</altname>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stythe</ent><br/
<hw>Stythe</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Mining)</fld> <def>Choke damp.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Stythy</ent><br/
<hw>Styth"y</hw> <pr>(?)</pr>, <pos>n. & v.</pos> <def>See <er>Stithy</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Styx</ent><br/
||<hw>Styx</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L., fr. Gr. <?/.]</ety> <fld>(Class. Myth.)</fld> <def>The principal river of the lower world, which had to be crossed in passing to the regions of the dead.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Suability</ent><br/
<hw>Su`a*bil"i*ty</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Law)</fld> <def>Liability to be sued; the state of being subjected by law to civil process.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Suable</ent><br/
<hw>Su"a*ble</hw> <pr>(?)</pr>, <pos>a.</pos> <fld>(Law)</fld> <def>Capable of being sued; subject by law to be called to answer in court.</def>  <rj><au>Story.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Suade</ent><br/
<hw>Suade</hw> <pr>(?)</pr>, <pos>v. t.</pos> <ety>[L. <ets>suadere</ets>.]</ety> <def>To persuade.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Suadible</ent><br/
<hw>Suad"i*ble</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>suadibilis</ets>.]</ety> <def>Suasible.</def> <mark>[Obs.]</mark>  <rj><au>Wyclif (James iii. 17).</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Suage</ent><br/
<hw>Suage</hw> <pr>(?)</pr>, <pos>v. t.</pos> <def>To assuage.</def> <mark>[Obs.]</mark>  <rj><au>Dryden.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Suant</ent><br/
<hw>Su"ant</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[Cf. <er>Sue</er> to pursue.]</ety> <def>Spread equally over the surface; uniform; even.</def> <altsp>[Written also <asp>suent</asp>.]</altsp> <mark>[Local, U.S. & Prov. Eng.]</mark> -- <wordforms><wf>Su"ant*ly</wf>, <pos>adv.</pos> <mark>[Local, U.S. & Prov. Eng.]</mark></wordforms><br/
[<source>1913 Webster</source>]</p>

<p><ent>Suasible</ent><br/
<hw>Sua"si*ble</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>suadere</ets>, <ets>suasum</ets>, to persuade.]</ety> <def>Capable of being persuaded; easily persuaded.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Suasion</ent><br/
<hw>Sua"sion</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>suasio</ets>, fr. <ets>suadere</ets>, <ets>suasum</ets>, to advise, persuade, fr. <ets>suadus</ets> persuading, persuasive; akin to <ets>suavis</ets> sweet: cf. OF. <ets>suasion</ets>. See <er>Suave</er>, and cf. <er>Dissuade</er>, <er>Persuade</er>.]</ety> <def>The act of persuading; persuasion; <as>as, moral <ex>suasion</ex></as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Suasive</ent><br/
<hw>Sua"sive</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Having power to persuade; persuasive; suasory.</def> <au>South.</au> <ldquo/Genial and <xex>suasive</xex> satire.<rdquo/ <au>Earle.</au> -- <wordforms><wf>Sua"sive*ly</wf>, <pos>adv.</pos></wordforms><br/
[<source>1913 Webster</source>]</p>

<p><ent>Suasory</ent><br/
<hw>Sua"so*ry</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>suasorius</ets>: cf. F. <ets>suasoire</ets>.]</ety> <def>Tending to persuade; suasive.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Suave</ent><br/
<hw>Suave</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>suavis</ets> sweet, pleasant: cf. F. <ets>suave</ets>. See <er>Sweet</er>, and cf. <er>Suasion</er>.]</ety> <def>Sweet; pleasant; delightful; gracious or agreeable in manner; bland.</def> -- <wordforms><wf>Suave"ly</wf>, <pos>adv.</pos></wordforms><br/
[<source>1913 Webster</source>]</p>

<p><ent>Suavify</ent><br/
<hw>Suav"i*fy</hw> <pr>(?)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Suavified</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Suavifying</conjf> <pr>(?)</pr>.]</vmorph> <ety>[<ets>Suave</ets> + <ets>-fy</ets>.]</ety> <def>To make affable or suave.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Suaviloquent</ent><br/
<hw>Sua*vil"o*quent</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>suaviloquens</ets>; <ets>suavis</ets> sweet + <ets>loquens</ets>, p. pr. of <ets>loqui</ets> to speak.]</ety> <def>Sweetly speaking; using agreeable speech.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Suaviloquy</ent><br/
<hw>Sua*vil"o*quy</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>suaviloquium</ets>.]</ety> <def>Sweetness of speech.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Suavity</ent><br/
<hw>Suav"i*ty</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>suavitas</ets>: cf. F. <ets>suavit<eacute/</ets>.]</ety> <sn>1.</sn> <def>Sweetness to the taste.</def> <mark>[Obs.]</mark>  <rj><au>Sir T. Browne.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>The quality of being sweet or pleasing to the mind; agreeableness; softness; pleasantness; gentleness; urbanity; <as>as, <ex>suavity</ex> of manners; <ex>suavity</ex> of language, conversation, or address</as>.</def>  <rj><au>Glanvill.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sub-</ent><br/
<hw>Sub-</hw> <pr>(?)</pr>. <ety>[L. <ets>sub</ets> under, below; akin to Gr. <?/, Skr. <ets>upa</ets> to, on, under, over.  Cf. <er>Hypo-</er>, <er>Super-</er>.]</ety> <sn>1.</sn> <def>A prefix signifying <xex>under</xex>, <xex>below</xex>, <xex>beneath</xex>, and hence often, <xex>in an inferior position</xex> or <xex>degree</xex>, <xex>in an imperfect</xex> or <xex>partial state</xex>, as in <xex>sub</xex>scribe, <xex>sub</xex>struct, <xex>sub</xex>serve, <xex>sub</xex>ject, <xex>sub</xex>ordinate, <xex>sub</xex>acid, <xex>sub</xex>astringent, <xex>sub</xex>granular, <xex>sub</xex>orn. <xex>Sub-</xex> in Latin compounds often becomes <xex>sum-</xex> before <xex>m</xex>, <xex>sur</xex> before <xex>r</xex>, and regularly becomes <xex>suc-</xex>, <xex>suf-</xex>, <xex>sug-</xex>, and <xex>sup-</xex> before <xex>c</xex>, <xex>f</xex>, <xex>g</xex>, and <xex>p</xex> respectively. Before <xex>c</xex>, <xex>p</xex>, and <xex>t</xex> it sometimes takes form <xex>sus-</xex> (by the dropping of <xex>b</xex> from a collateral form, <xex>subs-</xex>).</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Chem.)</fld> <def>A prefix denoting that the ingredient (of a compound) signified by the term to which it is prefixed,is present in <xex>only a small proportion</xex>, or <xex>less than the normal amount</xex>; <as>as, <ex>sub</ex>sulphide, <ex>sub</ex>oxide, etc.</as>  Prefixed to the name of a salt it is equivalent to <xex>basic</xex>; <as>as, <ex>sub</ex>acetate or <ex>basic</ex> acetate</as>.</def> <mark>[Obsoles.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sub</ent><br/
<hw>Sub</hw>, <pos>n.</pos> <sn>1.</sn> <def>A subordinate; a subaltern.</def> <mark>[Colloq.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>a shortened form of <er>submarine</er>, the boat.</def><br/
[<source>PJC</source>]</p>

<p><sn>3.</sn> <def>a shortened form of <er>submarine sandwich</er>; also called <altname>hero</altname>, <altname>hero sandwich</altname>, and <altname>grinder</altname>.</def><br/
[<source>PJC</source>]</p>

<p><ent>Subacetate</ent><br/
<hw>Sub*ac"e*tate</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Chem.)</fld> <def>An acetate containing an excess of the basic constituent.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Subacid</ent><br/
<hw>Sub*ac"id</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>subacidus</ets>. See <er>Sub-</er>, <er>Acid</er>.]</ety> <def>Moderately acid or sour; <as>as, some plants have <ex>subacid</ex> juices</as>.</def> -- <def2><pos>n.</pos> <def>A substance moderately acid.</def></def2><br/
[<source>1913 Webster</source>]</p>

<p><ent>Subacrid</ent><br/
<hw>Sub*ac"rid</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Moderalely acrid or harsh.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Subacromial</ent><br/
<hw>Sub`a*cro"mi*al</hw> <pr>(?)</pr>, <pos>a.</pos> <fld>(Anat.)</fld> <def>Situated beneath the acromial process of the scapula.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Subact</ent><br/
<hw>Sub*act"</hw> <pr>(?)</pr>, <pos>v. t.</pos> <ety>[L. <ets>subactus</ets>, p. p. of <ets>subigere</ets> to subdue; <ets>sub</ets> under + <ets>agere</ets> to lead, bring.]</ety> <def>To reduce; to subdue.</def> <mark>[Obs.]</mark>  <rj><au>Bacon.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Subaction</ent><br/
<hw>Sub*ac"tion</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Cf. L. <ets>subactio</ets> a working up, discipline.]</ety> <def>The act of reducing to any state, as of mixing two bodies combletely.</def> <mark>[Obs.]</mark>  <rj><au>Bacon.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Subacute</ent><br/
<hw>Sub`a*cute"</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Moderalely acute.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Subaduncate</ent><br/
<hw>Sub`a*dun"cate</hw> <pr>(?)</pr>, <pos>a.</pos> <fld>(Zool.)</fld> <def>Somewhat hooked or curved.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Subadvocate</ent><br/
<hw>Sub*ad"vo*cate</hw> <pr>(?)</pr>, <pos>n.</pos> <def>An under or subordinate advocate.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Subaerial</ent><br/
<hw>Sub`a*e"ri*al</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Beneath the sky; in the open air; specifically <fld>(Geol.)</fld>, taking place on the earth's surface, as opposed to <xex>subaqueous</xex>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Subagency</ent><br/
<hw>Sub*a"gen*cy</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A subordinate agency.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Subagent</ent><br/
<hw>Sub*a"gent</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Law)</fld> <def>A person employed by an agent to transact the whole, or a part, of the business intrusted to the latter.</def>  <rj><au>Bouvier. Chitty.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Subagitation</ent><br/
<hw>Sub*ag`i*ta"tion</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>subagitatio</ets>, fr. <ets>subagitare</ets> to lie with illicity.]</ety> <def>Unlawful sexual intercourse.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Subah</ent><br/
||<hw>Su"bah</hw> <pr>(s<oomac/"b<adot/)</pr>, <pos>n.</pos> <ety>[Per. & Hind. <ets>s<umac/bah</ets>.]</ety> <def>A province; a government, as of a viceroy; also, a subahdar.</def> <mark>[India]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Subahdar</ent><br/
<hw>Su"bah*dar</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Per. & Hind. <ets>s<umac/bah-d<amac/r</ets>, Per. <ets>s<umac/bah</ets> a province + <ets>d<amac/r</ets> holding, keeping.]</ety> <def>A viceroy; a governor of a subah; also, a native captain in the British native army.</def> <mark>[India]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Subahship</ent><br/
<ent>Subahdary</ent><br/
<mhw>{ <hw>Su"bah*dar`y</hw> <pr>(?)</pr>, <hw>Su"bah*ship</hw> <pr>(?)</pr>, }</mhw> <pos>n.</pos> <def>The office or jurisdiction of a subahdar.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Subaid</ent><br/
<hw>Sub*aid"</hw> <pr>(?)</pr>, <pos>v. t.</pos> <def>To aid secretly; to assist in a private manner, or indirectly.</def> <mark>[R.]</mark>  <rj><au>Daniel.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Subalmoner</ent><br/
<hw>Sub*al"mon*er</hw> <pr>(?)</pr>, <pos>n.</pos> <def>An under almoner.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Subalpine</ent><br/
<hw>Sub*al"pine</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>subalpinus</ets>.]</ety> <fld>(Bot. & Zool.)</fld> <def>Inhabiting the somewhat high slopes and summits of mountains, but considerably below the snow line.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Subaltern</ent><br/
<hw>Sub*al"tern</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[F. <ets>subalterne</ets>, LL. <ets>subalternus</ets>, fr. L. <ets>sub</ets> under + <ets>alter</ets> the one, the other of two. See <er>Alter</er>.]</ety> <sn>1.</sn> <def>Ranked or ranged below; subordinate; inferior; specifically <fld>(Mil.)</fld>, ranking as a junior officer; being below the rank of captain; <as>as, a <ex>subaltern</ex> officer</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Logic)</fld> <def>Asserting only a part of what is asserted in a related proposition.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Subaltern genus</b></col>. <fld>(Logic)</fld> <cd>See under <er>Genus</er>.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Subaltern</ent><br/
<hw>Sub*al"tern</hw> <pr>(?)</pr>, <pos>n.</pos> <sn>1.</sn> <def>A person holding a subordinate position; specifically, a commissioned military officer below the rank of captain.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Logic)</fld> <def>A subaltern proposition.</def>  <rj><au>Whately.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Subalternant</ent><br/
<hw>Sub`al*ter"nant</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Logic)</fld> <def>A universal proposition. See <er>Subaltern</er>, 2.</def>  <rj><au>Whately.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Subalternate</ent><br/
<hw>Sub`al*ter"nate</hw> <pr>(?)</pr>, <pos>a.</pos> <sn>1.</sn> <def>Succeeding by turns; successive.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Subordinate; subaltern; inferior.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>All their <qex>subalternate</qex> and several kinds.</q> <rj><qau>Evelyn.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Subalternate</ent><br/
<hw>Sub`al*ter"nate</hw>, <pos>n.</pos> <fld>(Logic)</fld> <def>A particular proposition, as opposed to a universal one. See <er>Subaltern</er>, 2.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Subalternating</ent><br/
<hw>Sub*al"ter*na`ting</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Subalternate; successive.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Subalternation</ent><br/
<hw>Sub*al"ter*na`tion</hw> <pr>(?)</pr>, <pos>n.</pos> <def>The state of being subalternate; succession of turns; subordination.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Subangular</ent><br/
<hw>Sub*an"gu*lar</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Slightly angular.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Subapennine</ent><br/
<hw>Sub*ap"en*nine</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Under, or at the foot of, the Apennine mountains; -- applied, in geology, to a series of Tertiary strata of the older Pliocene period.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Subapical</ent><br/
<hw>Sub*ap"ic*al</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Being under the apex; of or pertaining to the part just below the apex.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Subaquaneous</ent><br/
<hw>Sub`a*qua"ne*ous</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>subaquaneus</ets>; <ets>sub + aqua</ets> water.]</ety> <def>Subaqueous.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Subaqueous</ent><br/
<ent>Subaquatic</ent><br/
<mhw>{ <hw>Sub`a*quat"ic</hw> <pr>(?)</pr>, <hw>Sub*a"que*ous</hw> <pr>(?)</pr>, }</mhw> <pos>a.</pos> <sn>1.</sn> <def>Being under water, or beneath the surface of water; adapted for use under water; submarine; <as>as, a <ex>subaqueous</ex> helmet</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Geol.)</fld> <def>Formed in or under water; <as>as, <ex>subaqueous</ex> deposits</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Subarachnoidal</ent><br/
<ent>Subarachnoid</ent><br/
<mhw>{ <hw>Sub`a*rach"noid</hw> <pr>(?)</pr>, <hw>Sub*ar`ach*noid"al</hw> <pr>(?)</pr>, }</mhw> <pos>a.</pos> <fld>(Anat.)</fld> <def>Situated under the arachnoid membrane.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Subarctic</ent><br/
<hw>Sub*arc"tic</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Approximately arctic; belonging to a region just without the arctic circle.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Subarcuated</ent><br/
<ent>Subarcuate</ent><br/
<mhw>{ <hw>Sub*ar"cu*ate</hw> <pr>(?)</pr>, <hw>Sub*ar"cu*a`ted</hw> <pr>(?)</pr>, }</mhw> <pos>a.</pos> <def>Having a figure resembling that of a bow; somewhat curved or arched.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Subarration</ent><br/
<hw>Sub`ar*ra"tion</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Pref. <ets>sub-</ets> + L. <ets>arra</ets>, <ets>arrha</ets>, earnest money. See <er>Earnest</er> a pledge.]</ety> <def>The ancient custom of betrothing by the bestowal, on the part of the man, of marriage gifts or tokens, as money, rings, or other presents, upon the woman.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Subarytenoid</ent><br/
<hw>Sub`a*ryt"e*noid</hw> <pr>(?)</pr>, <pos>a.</pos> <fld>(Anat.)</fld> <def>Situated under the arytenoid cartilage of the larynx.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Subastral</ent><br/
<hw>Sub*as"tral</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Beneath the stars or heavens; terrestrial.</def>  <rj><au>Bp. Warburton.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Subastringent</ent><br/
<hw>Sub`as*trin"gent</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Somewhat astringent.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Subatom</ent><br/
<hw>Sub*at"om</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Chem.)</fld> <def>A hypothetical component of a chemical atom, on the theory that the elements themselves are complex substances; -- called also <altname>atomicule</altname>.</def>
<-- Add subatomic particles -- protons, neutrons, gluons, quarks, etc. --><br/
[<source>1913 Webster</source>]</p>

<p><ent>Subaud</ent><br/
<hw>Sub*aud"</hw> <pr>(?)</pr>, <pos>v. t.</pos> <ety>[L. <ets>subaudire</ets>, <ets>subauditum</ets>; <ets>sub</ets> under + <ets>audire</ets> to hear.]</ety> <def>To understand or supply in an ellipsis.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Subaudition</ent><br/
<hw>Sub`au*di"tion</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>subauditio</ets>.]</ety> <def>The act of understanding, or supplying, something not expressed; also, that which is so understood or supplied.</def>  <rj><au>Trench.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Subaxillary</ent><br/
<hw>Sub*ax"il*la*ry</hw> <pr>(?)</pr>, <pos>a.</pos> <sn>1.</sn> <fld>(Anat.)</fld> <def>Situated under the axilla, or armpit.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Bot.)</fld> <def>Placed under the axil, or angle formed by the branch of a plant with the stem, or a leaf with the branch.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Subbasal</ent><br/
<hw>Sub*ba"sal</hw> <pr>(?)</pr>, <pos>a.</pos> <fld>(Zool.)</fld> <def>Near the base.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sub-base</ent><br/
<hw>Sub"-base`</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Arch.)</fld> <def>The lowest member of a base when divided horizontally, or of a baseboard, pedestal, or the like.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sub-bass</ent><br/
<hw>Sub"-bass`</hw>, <pos>n.</pos> <fld>(Mus.)</fld> <def>The deepest pedal stop, or the lowest tones of an organ; the fundamental or ground bass.</def> <altsp>[Written also <asp>sub-base</asp>.]</altsp>  <rj><au>Ayliffe.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Subbeadle</ent><br/
<hw>Sub*bea"dle</hw> <pr>(?)</pr>, <pos>n.</pos> <def>An under beadle.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Subbrachial</ent><br/
<hw>Sub*brach"i*al</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Of or pertaining to the subbrachians.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Subbrachiales</ent><br/
||<hw>Sub*brach`i*a"les</hw> <pr>(?)</pr>, <pos>n. pl.</pos> <ety>[NL. See <er>Sub-</er>, and <er>Brachial</er>.]</ety> <fld>(Zool.)</fld> <def>A division of soft-finned fishes in which the ventral fins are situated beneath the pectorial fins, or nearly so.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Subbrachian</ent><br/
<hw>Sub*brach"i*an</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Pref. <ets>sub-</ets> + <ets>brachium</ets>.]</ety> <fld>(Zool.)</fld> <def>One of the Subbrachiales.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Subbreed</ent><br/
<hw>Sub"breed`</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Zool.)</fld> <def>A race or strain differing in certain characters from the parent breed; an incipient breed.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Subbronchial</ent><br/
<hw>Sub*bron"chi*al</hw> <pr>(?)</pr>, <pos>a.</pos> <fld>(Anat.)</fld> <def>Situated under, or on the ventral side of, the bronchi; <as>as, the <ex>subbronchial</ex> air sacs of birds</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Subcaliber</ent><br/
<hw>Sub*cal"i*ber</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Smaller than the caliber of a firearm.</def> <altsp>[Written also <asp>subcalibre</asp>.]</altsp><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Subcaliber projectile</b></col>, <cd>a projectile having a smaller diameter than the caliber of the arm from which it is fired, and to which it is fitted by means of a sabot.</cd> <au>Knight.</au></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Subcarbonate</ent><br/
<hw>Sub*car"bon*ate</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Chem.)</fld> <def>A carbonate containing an excess of the basic constituent.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Subcarboniferous</ent><br/
<hw>Sub*car`bon*if"er*ous</hw> <pr>(?)</pr>, <pos>a.</pos> <fld>(Geol.)</fld> <def>Of or pertaining to the lowest division of the Carboniferous formations underlying the proper coal measures. It was a marine formation characterized in general by beds of limestone.</def> -- <def2><pos>n.</pos> <def>The Subcarboniferous period or formation.</def></def2><br/
[<source>1913 Webster</source>]</p>

<p><ent>Subcarbureted</ent><br/
<hw>Sub*car"bu*ret`ed</hw> <pr>(?)</pr>, <pos>a.</pos> <fld>(Chem.)</fld> <def>United with, or containing, carbon in less than the normal proportion.</def> <altsp>[Written also <asp>subcarburetted</asp>.]</altsp> <mark>[Obsoles.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Subcartilaginous</ent><br/
<hw>Sub*car`ti*lag"i*nous</hw> <pr>(?)</pr>, <pos>a.</pos> <fld>(Anat.)</fld> <sd>(a)</sd> <def>Situated under or beneath a cartilage or cartilages.</def> <sd>(b)</sd> <def>Partially cartilaginous.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Subcaudal</ent><br/
<hw>Sub*cau"dal</hw> <pr>(?)</pr>, <pos>a.</pos> <fld>(Anat.)</fld> <def>Situated under, or on the ventral side of, the tail; <as>as, the <ex>subcaudal</ex>, or chevron, bones</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Subcelestial</ent><br/
<hw>Sub`ce*les"tial</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Being beneath the heavens; <as>as, <ex>subcelestial</ex> glories</as>.</def>  <rj><au>Barrow.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Subcellar</ent><br/
<hw>Sub*cel"lar</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A cellar beneath another story wholly or partly underground; usually, a cellar under a cellar.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Subcentral</ent><br/
<hw>Sub*cen"tral</hw> <pr>(?)</pr>, <pos>a.</pos> <sn>1.</sn> <def>Under the center.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Nearly central; not quite central.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Subchanter</ent><br/
<hw>Sub*chant"er</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Eccl.)</fld> <def>An underchanter; a precentor's deputy in a cathedral; a succentor.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Subcircular</ent><br/
<hw>Sub*cir"cu*lar</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Nearly circular.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Subclass</ent><br/
<hw>Sub"class`</hw> <pr>(?)</pr>, <pos>n.</pos> <def>One of the natural groups, more important than an order, into which some classes are divided; <as>as, the angiospermous <ex>subclass</ex> of exogens</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Subclavian</ent><br/
<hw>Sub*cla"vi*an</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[Pref. <ets>sub-</ets> + L. <ets>clavis</ets> a key. See <er>Clavicle</er>.]</ety> <fld>(Anat.)</fld> <def>Situated under the clavicle, or collar bone; <as>as, the <ex>subclavian</ex> arteries</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Subcolumnar</ent><br/
<hw>Sub`co*lum"nar</hw> <pr>(?)</pr>, <pos>a.</pos> <fld>(Geol.)</fld> <def>Having an imperfect or interrupted columnar structure.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Subcommittee</ent><br/
<hw>Sub`com*mit"tee</hw> <pr>(?)</pr>, <pos>n.</pos> <def>An under committee; a part or division of a committee.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Yet by their sequestrators and <qex>subcommittees</qex> abroad . . . those orders were commonly disobeyed.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Subcompressed</ent><br/
<hw>Sub`com*pressed"</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Not fully compressed; partially or somewhat compressed.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Subconcave</ent><br/
<hw>Sub*con"cave</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Slightly concave.</def>  <rj><au>Owen.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Subconformable</ent><br/
<hw>Sub`con*form"a*ble</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Partially conformable.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Subconical</ent><br/
<hw>Sub*con"ic*al</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Slightly conical.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Subconjunctival</ent><br/
<hw>Sub*con`junc*ti"val</hw> <pr>(?)</pr>, <pos>a.</pos> <fld>(Anat.)</fld> <def>Situated under the conjunctiva.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Subconscious</ent><br/
<hw>Sub*con"scious</hw> <pr>(?)</pr>, <pos>a.</pos> <sn>1.</sn> <def>Occurring without the possibility or the fact of an attendant consciousness; -- said of states of the soul.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Partially conscious; feebly conscious.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Subconsciousness</ent><br/
<hw>Sub*con"scious*ness</hw> <pr>(?)</pr>, <pos>n.</pos> <def>The state or quality of being subconscious; a state of mind in which perception and other mental processes occur without distinct consciousness.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Subconstellation</ent><br/
<hw>Sub*con`stel*la"tion</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Astron.)</fld> <def>A subordinate constellation.</def>  <rj><au>Sir T. Browne.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Subcontract</ent><br/
<hw>Sub*con"tract</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A contract under, or subordinate to, a previous contract.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Subcontracted</ent><br/
<hw>Sub`con*tract"ed</hw> <pr>(?)</pr>, <pos>a.</pos> <sn>1.</sn> <def>Contracted after a former contract.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Betrothed for the second time.</def> <mark>[Obs.]</mark>  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Subcontractor</ent><br/
<hw>Sub`con*tract"or</hw> <pr>(?)</pr>, <pos>n.</pos> <def>One who takes a portion of a contract, as for work, from the principal contractor.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Subcontrary</ent><br/
<hw>Sub*con"tra*ry</hw> <pr>(?)</pr>, <pos>a.</pos> <sn>1.</sn> <def>Contrary in an inferior degree.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Geom.)</fld> <def>Having, or being in, a contrary order; -- said of a section of an oblique cone having a circular base made by a plane not parallel to the base, but so inclined to the axis that the section is a circle; applied also to two similar triangles when so placed as to have a common angle at the vertex, the opposite sides not being parallel.</def>  <rj><au>Brande & C.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <fld>(Logic)</fld> <def>Denoting the relation of opposition between the <xex>particular affirmative</xex> and <xex>particular negative</xex>. Of these both may be true and only one can be false.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Subcontrary</ent><br/
<hw>Sub*con"tra*ry</hw>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Subcontraries</plw> <pr>(<?/)</pr>.</plu> <fld>(Logic)</fld> <def>A subcontrary proposition; a proposition inferior or contrary in a lower degree.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Subcoracoid</ent><br/
<hw>Sub*cor"a*coid</hw> <pr>(?)</pr>, <pos>a.</pos> <fld>(Anat.)</fld> <def>Situated under the coracoid process of the scapula; <as>as, the <ex>subcoracoid</ex> dislocation of the humerus</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Subcordate</ent><br/
<hw>Sub*cor"date</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Somewhat cordate; somewhat like a heart in shape.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Subcorneous</ent><br/
<hw>Sub*cor"ne*ous</hw> <pr>(?)</pr>, <pos>a.</pos> <fld>(Anat.)</fld> <sd>(a)</sd> <def>Situated under a horny part or layer.</def> <sd>(b)</sd> <def>Partially horny.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Subcostal</ent><br/
<hw>Sub*cos"tal</hw> <pr>(?)</pr>, <pos>a.</pos> <fld>(Anat. & Zool.)</fld> <def>Situated below the costas, or ribs; <as>as, the <ex>subcostal</ex> muscles</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ The <xex>subcostal</xex> muscles are distinct from, and within, the <xex>intercostal</xex>.</note><br/
[<source>1913 Webster</source>]</p>

<p><ent>Subcostal</ent><br/
<hw>Sub*cos"tal</hw>, <pos>n.</pos> <sn>1.</sn> <fld>(Anat.)</fld> <def>A subcostal muscle.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Zool.)</fld> <def>One of the principal nervures of the wings of an insect. It is situated next beneath or behind the costal. See <er>Nervure</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Subcranial</ent><br/
<hw>Sub*cra"ni*al</hw> <pr>(?)</pr>, <pos>a.</pos> <fld>(Anat.)</fld> <def>Situated under, or on the ventral side of, the cranium; facial.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Subcrustaceous</ent><br/
<hw>Sub`crus*ta"ceous</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Occurring beneath a crust or scab; <as>as, a <ex>subcrustaceous</ex> cicatrization</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Subcrystalline</ent><br/
<hw>Sub*crys"tal*line</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Imperfectly crystallized.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Subcultrated</ent><br/
<ent>Subcultrate</ent><br/
<mhw>{ <hw>Sub*cul"trate</hw> <pr>(?)</pr>, <hw>Sub*cul"tra*ted</hw> <pr>(?)</pr>, }</mhw> <pos>a.</pos> <fld>(Zool.)</fld> <def>Having a form resembling that of a colter, or straight on one side and curved on the other.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Subcutaneous</ent><br/
<hw>Sub`cu*ta"ne*ous</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Situated under the skin; hypodermic.</def> -- <wordforms><wf>Sub`cu*ta"ne*ous*ly</wf>, <pos>adv.</pos></wordforms><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Subcutaneous operation</b></col> <fld>(Surg.)</fld>, <cd>an operation performed without opening that part of the skin opposite to, or over, the internal section.</cd></cs>
<-- subcutaneous injection, (Med) an injection method in which the injected liquid is delivered directly under the skin, as contrasted with intramuscular or intravenous injection.  Also called subdermal injection. --><br/
[<source>1913 Webster</source>]</p>

<p><ent>Subcuticular</ent><br/
<hw>Sub`cu*tic"u*lar</hw> <pr>(?)</pr>, <pos>a.</pos> <fld>(Anat.)</fld> <def>Situated under the cuticle, or scarfskin.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Subcylindric</ent><br/
<ent>Subcylindrical</ent><br/
<mhw>{ <hw>Sub`cy*lin"dric*al</hw> <pr>(?)</pr>, <hw>Sub`cy*lin"dric</hw> <pr>(?)</pr>, }</mhw> <pos>a.</pos> <def>Imperfectly cylindrical; approximately cylindrical.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Subdeacon</ent><br/
<hw>Sub*dea"con</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Pref. <ets>sub-</ets> + <ets>deacon</ets>: cf. L. <ets>subdiaconus</ets>.]</ety> <fld>(Eccl.)</fld> <def>One belonging to an order in the Roman Catholic Church, next interior to the order of deacons; also, a member of a minor order in the Greek Church.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Subdeaconship</ent><br/
<ent>Subdeaconry</ent><br/
<mhw>{ <hw>Sub*dea"con*ry</hw> <pr>(?)</pr>, <hw>Sub*dea"con*ship</hw>, }</mhw> <pos>n.</pos> <fld>(Eccl.)</fld> <def>The order or office of subdeacon.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Subdean</ent><br/
<hw>Sub"dean`</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Pref. <ets>sub-</ets> + <ets>dean</ets>: cf. F. <ets>sousdoyen</ets>.]</ety> <def>An under dean; the deputy or substitute of a dean.</def>  <rj><au>Ayliffe.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Subdeanery</ent><br/
<hw>Sub*dean"er*y</hw> <pr>(?)</pr>, <pos>n.</pos> <def>Office or rank of subdean.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Subdecanal</ent><br/
<hw>Sub*dec"a*nal</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Of or pertaining to a subdean or subdeanery.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Subdecuple</ent><br/
<hw>Sub*dec"u*ple</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Containing one part of ten.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Subdelegate</ent><br/
<hw>Sub*del"e*gate</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A subordinate delegate, or one with inferior powers.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Subdelegate</ent><br/
<hw>Sub*del"e*gate</hw> <pr>(?)</pr>, <pos>v. t.</pos> <def>To appoint to act as subdelegate, or as a subordinate; to depete.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Subdented</ent><br/
<hw>Sub*dent"ed</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Indented beneath.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Subdepartment</ent><br/
<hw>Sub`de*part"ment</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A subordinate department; a bureau. See the Note under <er>Bureau</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><-- p. 1433 --></p>

<p><ent>Subdeposit</ent><br/
<hw>Sub`de*pos"it</hw> <pr>(?)</pr>, <pos>n.</pos> <def>That which is deposited beneath something else.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Subderisorious</ent><br/
<hw>Sub`der*i*so"ri*ous</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[Pref. <ets>sub-</ets> + L. <ets>derisorius</ets>. See <er>Derisory</er>.]</ety> <def>Ridiculing with moderation.</def> <mark>[R.]</mark>  <rj><au>Dr. H. More.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Subderivative</ent><br/
<hw>Sub`de*riv"a*tive</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A word derived from a derivative, and not directly from the root; <as>as, <ldquo/friendliness<rdquo/ is a <ex>subderivative</ex>, being derived from <ldquo/friendly<rdquo/, which is in turn a derivative from <ldquo/friend.<rdquo/</as></def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Subdiaconate</ent><br/
<hw>Sub`di*ac"o*nate</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Of or pertaining to a subdeacon, or to the office or rank of a subdeacon.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Subdiaconate</ent><br/
<hw>Sub`di*ac"o*nate</hw>, <pos>n.</pos> <def>The office or rank of a subdeacon.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Subdial</ent><br/
<hw>Sub*di"al</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>subdialis</ets> in the open air.]</ety> <def>Of or pertaining to the open air; being under the open sky.</def> <mark>[R.]</mark>  <rj><au>N. Bacon.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Subdialect</ent><br/
<hw>Sub*di"a*lect</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A subordinate dialect.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Subdichotomy</ent><br/
<hw>Sub`di*chot"o*my</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A subordinate, or inferior, division into parts; a subdivision.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>Many <qex>subdichatomies</qex> of petty schisms.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Subdilated</ent><br/
<hw>Sub`di*lat"ed</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Partially dilated.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Subdititious</ent><br/
<hw>Sub`di*ti"tious</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>subdititius</ets>, <ets>subditicius</ets>, fr. <ets>subdere</ets> to substitute.]</ety> <def>Put secretly in the place of something else; foisted in.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Subdiversify</ent><br/
<hw>Sub`di*ver"si*fy</hw> <pr>(?)</pr>, <pos>v. t.</pos> <def>To diversify aggain what is already diversified.</def> <mark>[R.]</mark>  <rj><au>Sir M. Hale.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Subdivide</ent><br/
<hw>Sub`di*vide"</hw> <pr>(?)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Subdivided</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Subdividing</conjf>.]</vmorph> <ety>[L. <ets>subdividere</ets>, <ets>sub</ets> under + <ets>dividere</ets> to divide. See <er>Divide</er>.]</ety> <def>To divide the parts of (anything) into more parts; to part into smaller divisions; to divide again, as what has already been divided.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The progenies of Cham and Japhet swarmed into colonies, and those colonies were <qex>subdivided</qex> into many others.</q> <rj><qau>Dryden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Subdivide</ent><br/
<hw>Sub`di*vide"</hw>, <pos>v. i.</pos> <def>To be, or to become, subdivided.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Subdivine</ent><br/
<hw>Sub`di*vine"</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Partaking of divinity; divine in a partial or lower degree.</def>  <rj><au>Bp. Hall.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Subdivisible</ent><br/
<hw>Sub`di*vis"i*ble</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Susceptible of subdivision.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Subdivision</ent><br/
<hw>Sub`di*vi"sion</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>subdivisio</ets>: cf. F. <ets>subdivision</ets>.]</ety> <sn>1.</sn> <def>The act of subdividing, or separating a part into smaller parts.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A part of a thing made by subdividing.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>In the decimal table, the <qex>subdivision</qex> of the cubit, as span, palm, and digit, are deduced from the shorter cubit.</q> <rj><qau>Arbuthnot.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Subdolous</ent><br/
<hw>Sub"do*lous</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>subdolus</ets>, <ets>sub + dolus</ets> deceit.]</ety> <def>Sly; crafty; cunning; artful.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Subdominant</ent><br/
<hw>Sub*dom"i*nant</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Mus.)</fld> <def>The fourth tone above, or fifth below, the tonic; -- so called as being under the dominant.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Subduable</ent><br/
<hw>Sub*du"a*ble</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Able to be subdued.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Subdual</ent><br/
<hw>Sub*du"al</hw> <pr>(?)</pr>, <pos>n.</pos> <def>Act of subduing.</def>  <rj><au>Bp. Warburton.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Subduct</ent><br/
<ent>Subduce</ent><br/
<mhw>{ <hw>Sub*duce"</hw> <pr>(?)</pr>, <hw>Sub*duct"</hw> <pr>(?)</pr>, }</mhw> <pos>v. t.</pos> <ety>[L. <ets>subducere</ets>, <ets>subductum</ets>; <ets>sub</ets> under + <ets>ducere</ets> to lead, to draw. See <er>Duke</er>, and cf. <er>Subdue</er>.]</ety> <sn>1.</sn> <def>To withdraw; to take away.</def>  <rj><au>Milton.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To subtract by arithmetical operation; to deduct.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>If, out of that infinite multitude of antecedent generations, we should <qex>subduce</qex> ten.</q> <rj><qau>Sir M. Hale.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Subduction</ent><br/
<hw>Sub*duc"tion</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>subductio</ets>.]</ety> <sn>1.</sn> <def>The act of subducting or taking away.</def>  <rj><au>Bp. Hall.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Arithmetical subtraction.</def>  <rj><au>Sir M. Hale.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Subdue</ent><br/
<hw>Sub*due"</hw> <pr>(?)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Subdued</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Subduing</conjf>.]</vmorph> <ety>[OE. <ets>soduen</ets>, OF. <ets>sosduire</ets> to seduce, L. <ets>subtus</ets> below (fr. <ets>sub</ets> under) + <ets>ducere</ets> to lead. See <er>Duke</er>, and cf. <er>Subduct</er>.]</ety> <sn>1.</sn> <def>To bring under; to conquer by force or the exertion of superior power, and bring into permanent subjection; to reduce under dominion; to vanquish.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>I will <qex>subdue</qex> all thine enemies.</q> <rj><qau>1 Chron. xvii. 10.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To overpower so as to disable from further resistance; to crush.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Nothing could have <qex>subdued</qex> nature<br/
To such a lowness, but his unkind daughters.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>If aught . . . were worthy to <qex>subdue</qex><br/
The soul of man.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>To destroy the force of; to overcome; <as>as, medicines <ex>subdue</ex> a fever</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>To render submissive; to bring under command; to reduce to mildness or obedience; to tame; <as>as, to <ex>subdue</ex> a stubborn child; to <ex>subdue</ex> the temper or passions</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <def>To overcome, as by persuasion or other mild means; <as>as, to <ex>subdue</ex> opposition by argument or entreaties</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>6.</sn> <def>To reduce to tenderness; to melt; to soften; <as>as, to <ex>subdue</ex> ferocity by tears</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>7.</sn> <def>To make mellow; to break, as land; also, to destroy, as weeds.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>8.</sn> <def>To reduce the intensity or degree of; to tone down; to soften; <as>as, to <ex>subdue</ex> the brilliancy of colors</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><syn><b>Syn.</b> -- To conquer; overpower; overcome; surmount; vanquish. See <er>Conquer</er>.</syn><br/
[<source>1913 Webster</source>]</p>

<p><ent>Subdued</ent><br/
<hw>Sub*dued"</hw> <pr>(?)</pr>, <pos>a.</pos> <sn>1.</sn> <def>Conquered; overpowered; crushed; submissive; mild.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Not glaring in color; soft in tone.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Subduement</ent><br/
<hw>Sub*due"ment</hw> <pr>(?)</pr>, <pos>n.</pos> <def>Subdual.</def> <mark>[Obs.]</mark>  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Subduer</ent><br/
<hw>Sub*du"er</hw> <pr>(?)</pr>, <pos>n.</pos> <def>One who, or that which, subdues; a conqueror.</def>  <rj><au>Spenser.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Subdulcid</ent><br/
<hw>Sub*dul"cid</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[Pref. <ets>sub</ets> + L. <ets>dulcis</ets> sweet.]</ety> <def>Somewhat sweet; sweetish.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Subduple</ent><br/
<hw>Sub"du*ple</hw> <pr>(?)</pr>, <pos>a.</pos> <fld>(Math.)</fld> <def>Indicating one part of two; in the ratio of one to two.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Subduple ratio</b></col>, <cd>the ratio of 1 to 2: thus, 3:6 is a <xex>subduple ratio</xex>, as 6:3 is a <xex>duple ratio</xex>.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Subduplicate</ent><br/
<hw>Sub*du"pli*cate</hw> <pr>(?)</pr>, <pos>a.</pos> <fld>(Math.)</fld> <def>Expressed by the square root; -- said of ratios.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Subduplicate ratio</b></col>, <cd>the ratio of the square roots, or the square root of a ratio; <as>thus, the <ex>subduplicate ratio</ex> of <it>a</it> to <it>b</it> is <root/<it>a</it> to <root/<it>b</it>, or <root/<fract><it>a</it>/<it>b</it></fract></as>.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Subdural</ent><br/
<hw>Sub*du"ral</hw> <pr>(?)</pr>, <pos>a.</pos> <fld>(Anat.)</fld> <def>Situated under the dura mater, or between the dura mater and the arachnoid membrane.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Subeditor</ent><br/
<hw>Sub*ed"i*tor</hw> <pr>(?)</pr>, <pos>n.</pos> <def>An assistant editor, as of a periodical or journal.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Subelongate</ent><br/
<hw>Sub`e*lon"gate</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Not fully elongated; somewhat elongated.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Subendocardial</ent><br/
<hw>Sub*en`do*car"di*al</hw> <pr>(?)</pr>, <pos>a.</pos> <fld>(Anat.)</fld> <def>Situated under the endocardium.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Subendymal</ent><br/
<hw>Sub*en"dy*mal</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[Pref. <ets>sub + endyma</ets>.]</ety> <def>Situated under the endyma.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Subepidermal</ent><br/
<hw>Sub*ep`i*der"mal</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Situated immediately below the epidermis.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Subepiglottic</ent><br/
<hw>Sub*ep`i*glot"tic</hw> <pr>(?)</pr>, <pos>a.</pos> <fld>(Anat.)</fld> <def>Situated under the epiglottis.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Subepithelial</ent><br/
<hw>Sub*ep`i*the"li*al</hw> <pr>(?)</pr>, <pos>a.</pos> <fld>(Anat.)</fld> <def>Situated under the epithelium.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Subequal</ent><br/
<hw>Sub*e"qual</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Nearly equal.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Suberate</ent><br/
<hw>Su"ber*ate</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Cf. F. <ets>sub<eacute/rate</ets>.]</ety> <fld>(Chem.)</fld> <def>A salt of suberic acid.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Subereous</ent><br/
<hw>Su*be"re*ous</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>subereus</ets> of the cork tree.]</ety> <def>Of or pertaining to cork; of the nature of cork; suberose.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Suberic</ent><br/
<hw>Su*ber"ic</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>suber</ets> the cork tree: cf. F. <ets>sub<eacute/reque</ets>.]</ety> <fld>(Chem.)</fld> <def>Of or pertaining to cork; specifically, designating an acid, <chform>C6H12.(CO2H)2</chform>, homologous with oxalic acid, and obtained from cork and certain fatty oils, as a white crystalline substance.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Suberin</ent><br/
<hw>Su"ber*in</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>suber</ets> the cork tree: cf. F. <ets>sub<eacute/rine</ets>.]</ety> <fld>(Bot.)</fld> <def>A material found in the cell walls of cork. It is a modification of lignin.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Suberite</ent><br/
<hw>Su"ber*ite</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>suber</ets> the cork tree.]</ety> <fld>(Zool.)</fld> <def>Any sponge of the genus <gen>Suberites</gen> and allied genera. These sponges have a fine and compact texture, and contain minute siliceous spicules.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Suberization</ent><br/
<hw>Su`ber*i*za"tion</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Bot.)</fld> <def>Conversion of the cell walls into cork tissue by development of suberin; -- commonly taking place in exposed tissues, as when a callus forms over a wound. Suberized cell walls are impervious to water.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Suberize</ent><br/
<hw>Su"ber*ize</hw> <pr>(?)</pr>, <pos>v. t.</pos>  <vmorph>[<pos>imp. & p. p.</pos> <conjf>-ized</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>-izing</conjf> <pr>(?)</pr>.]</vmorph> <ety>[L. <ets>suber</ets> cork.]</ety> <fld>(Bot.)</fld> <def>To effect suberization of.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Suberone</ent><br/
<hw>Su"ber*one</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Chem.)</fld> <sd>(a)</sd> <def>The hypothetical ketone of suberic acid.</def> <sd>(b)</sd> <def>A colorless liquid, analogous suberone proper, having a pleasant peppermint odor. It is obtained by the distillation of calcium suberate.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Suberous</ent><br/
<ent>Suberose</ent><br/
<mhw>{ <hw>Su"ber*ose`</hw> <pr>(?)</pr>, <hw>Su"ber*ous</hw> <pr>(?)</pr>, }</mhw> <pos>a.</pos> <ety>[L. <ets>suber</ets> the cork tree: cf. F. <ets>sub<eacute/reux</ets>.]</ety> <fld>(Bot.)</fld> <def>Having a corky texture.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Subesophageal</ent><br/
<hw>Sub*e`so*phag"e*al</hw> <pr>(?)</pr>, <pos>a.</pos> <fld>(Zool.)</fld> <def>Situated beneath the esophagus.</def> <altsp>[Written also <asp>suboesophageal</asp>.]</altsp><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Subesophageal ganglion</b></col> <fld>(Zool.)</fld>, <cd>a large special ganglion situated beneath the esophagus of arthropods, annelids, and some other invertebrates.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Subfamily</ent><br/
<hw>Sub*fam"i*ly</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Biol.)</fld> <def>One of the subdivisions, of more importance than genus, into which certain families are divided.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Subfibrous</ent><br/
<hw>Sub*fi"brous</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Somewhat fibrous.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>subfusc</ent><br/
<hw>sub*fusc"</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>subfuscus</ets>, <ets>suffuscus</ets>. See <er>Sub-</er>, and <er>Fuscous</er>.]</ety> <sn>1.</sn> <def>Same as <er>subfuscous</er>.</def><br/
[<source>PJC</source>]</p>

<p><sn>2.</sn> <def>Drab; dingy; dull.</def> <mark>[Chiefty Brit.]</mark> <br/
[<source>PJC</source>]</p>

<p><ent>Subfuscous</ent><br/
<hw>Sub*fus"cous</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>subfuscus</ets>, <ets>suffuscus</ets>. See <er>Sub-</er>, and <er>Fuscous</er>.]</ety> <def>Duskish; dusky; moderately dark; brownish; tawny.</def><br/
<syn><b>Syn. --</b> subfusc{1}.</syn><br/
[<source>1913 Webster</source>]</p>

<p><ent>Subfusk</ent><br/
<hw>Sub*fusk"</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Subfuscous.</def> <mark>[Obs.]</mark>  <rj><au>Tatler.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Subgelatinous</ent><br/
<hw>Sub`ge*lat"i*nous</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Imperfectly or partially gelatinous.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Subgeneric</ent><br/
<hw>Sub`ge*ner"ic</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Of or pertaining to a subgenus.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Subgenus</ent><br/
<hw>Sub*ge"nus</hw> <pr>(?)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Subgenera</plw> <pr>(<?/)</pr>.</plu> <fld>(Biol.)</fld> <def>A subdivision of a genus, comprising one or more species which differ from other species of the genus in some important character or characters; <as>as, the azaleas now constitute a <ex>subgenus</ex> of Rhododendron</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Subglacial</ent><br/
<hw>Sub*gla"cial</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Pertaining or belonging to the under side of a glacier; being beneath a glacier; <as>as, <ex>subglacial</ex> streams</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Subglobose</ent><br/
<hw>Sub`glo*bose"</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Not quite globose.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Subglobular</ent><br/
<hw>Sub*glob"u*lar</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Nearly globular.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Subglossal</ent><br/
<hw>Sub*glos"sal</hw> <pr>(?)</pr>, <pos>a.</pos> <fld>(Anat.)</fld> <def>Situated under the tongue; sublingual.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Subglottic</ent><br/
<hw>Sub*glot"tic</hw> <pr>(?)</pr>, <pos>a.</pos> <fld>(Anat.)</fld> <def>Situated below the glottis; -- applied to that part of the cavity of the larynx below the true vocal cords.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Subglumaceous</ent><br/
<hw>Sub`glu*ma"ceous</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Somewhat glumaceous.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Subgovernor</ent><br/
<hw>Sub*gov"ern*or</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A subordinate or assistant governor.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Subgranular</ent><br/
<hw>Sub*gran"u*lar</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Somewhat granular.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Subgroup</ent><br/
<hw>Sub"group`</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Biol.)</fld> <def>A subdivision of a group, as of animals.</def>  <rj><au>Darwin.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Subhastation</ent><br/
<hw>Sub`has*ta"tion</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>subhastatio</ets>.]</ety> <def>A public sale or auction.</def> <mark>[R.]</mark>  <rj><au>Bp. Burnet.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Subhepatic</ent><br/
<hw>Sub`he*pat"ic</hw> <pr>(?)</pr>, <pos>a.</pos> <fld>(Anat.)</fld> <def>Situated under, or on the ventral side of, the liver; -- applied to the interlobular branches of the portal vein.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Subhornblendic</ent><br/
<hw>Sub`horn*blend"ic</hw> <pr>(?)</pr>, <pos>a.</pos> <fld>(Min.)</fld> <def>Containing hornblende in a scattered state; of or relating to rocks containing disseminated hornblende.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Subhumerate</ent><br/
<hw>Sub*hu"mer*ate</hw> <pr>(?)</pr>, <pos>v. t.</pos> <ety>[See <er>Sub-</er>, <er>Humerus</er>.]</ety> <def>To place the shoulders under; to bear.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>Nothing surer ties a friend than freely to <qex>subhumerate</qex> the burden which was his.</q> <rj><qau>Feltham.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Subhyaloid</ent><br/
<hw>Sub*hy"a*loid</hw> <pr>(?)</pr>, <pos>a.</pos> <fld>(Anat.)</fld> <def>Situated under the hyaliod membrane.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Subhyoidean</ent><br/
<hw>Sub`hy*oid"e*an</hw> <pr>(?)</pr>, <pos>a.</pos> <fld>(Anat. & Med.)</fld> <def>Situated or performed beneath the hyoid bone; <as>as, <ex>subhyoidean</ex> laryngotomy</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Subimago</ent><br/
||<hw>Sub`i*ma"go</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[NL. See <er>Sub-</er>, and <er>Imago</er>.]</ety> <fld>(Zool.)</fld> <def>A stage in the development of certain insects, such as the May flies, intermediate between the pupa and imago. In this stage, the insect is able to fly, but subsequently sheds a skin before becoming mature. Called also <altname>pseudimago</altname>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Subincusation</ent><br/
<hw>Sub*in`cu*sa"tion</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Pref. <ets>sub</ets> + L. <ets>incusatio</ets> accusation, fr. <ets>incusare</ets> to accuse.]</ety> <def>A slight charge or accusation.</def> <mark>[Obs.]</mark>  <rj><au>Bp. Hall.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Subindex</ent><br/
<hw>Sub*in"dex</hw> <pr>(?)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Subindices</plw> <pr>(<?/)</pr>.</plu> <fld>(Math.)</fld> <def>A number or mark placed opposite the lower part of a letter or symbol to distinguish the symbol; <as>thus, <it>a</it><subs>0</subs>, <it>b</it><subs>1</subs>, <it>c</it><subs>2</subs>, <it>x</it><subs>n</subs>, have 0, 1, 2, and <it>n</it> as <ex>subindices</ex></as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Subindicate</ent><br/
<hw>Sub*in"di*cate</hw> <pr>(?)</pr>, <pos>v. t.</pos> <ety>[Pref. <ets>sub + indicate</ets>: cf. L. <ets>subindicare</ets>.]</ety> <def>To indicate by signs or hints; to indicate imperfectly.</def> <mark>[R.]</mark>  <rj><au>Dr. H. More.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Subindication</ent><br/
<hw>Sub*in`di*ca"tion</hw> <pr>(?)</pr>, <pos>n.</pos> <def>The act of indicating by signs; a slight indication.</def> <mark>[R.]</mark> <ldquo/The <xex>subindication</xex> and shadowing of heavenly things.<rdquo/  <rj><au>Barrow.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Subindividual</ent><br/
<hw>Sub*in`di*vid"u*al</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A division of that which is individual.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>An individual can not branch itself into <qex>subindividuals</qex>.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Subinduce</ent><br/
<hw>Sub`in*duce"</hw> <pr>(?)</pr>, <pos>v. t.</pos> <def>To insinuate; to offer indirectly.</def> <mark>[Obs.]</mark>  <rj><au>Sir E. Dering.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Subinfer</ent><br/
<hw>Sub`in*fer"</hw> <pr>(?)</pr>, <pos>v. t. & i.</pos> <def>To infer from an inference already made.</def> <mark>[Obs.]</mark>  <rj><au>Bp. Hall.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Subinfeudation</ent><br/
<hw>Sub*in`feu*da"tion</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Law)</fld> <sd>(a)</sd> <def>The granting of lands by inferior lords to their dependents, to be held by themselves by feudal tenure.</def> <au>Craig.</au> <sd>(b)</sd> <def>Subordinate tenancy; undertenancy.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The widow is immediate tenant to the heir, by a kind of <qex>subinfeudation</qex>, or undertenancy.</q> <rj><qau>Blackstone.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Subingression</ent><br/
<hw>Sub`in*gres"sion</hw> <pr>(?)</pr>, <pos>n.</pos> <def>Secret entrance.</def> <mark>[R.]</mark>  <rj><au>Boyle.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Subintestinal</ent><br/
<hw>Sub`in*tes"ti*nal</hw> <pr>(?)</pr>, <pos>a.</pos> <fld>(Anat.)</fld> <def>Situated under, or on the ventral side of, the intestine.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Subinvolution</ent><br/
<hw>Sub*in`vo*lu"tion</hw> <pr>(?)</pr>, <pos>n.</pos> <def>Partial or incomplete involution; <as>as, <ex>subinvolution</ex> of the uterus</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Subitaneous</ent><br/
<hw>Sub`i*ta"ne*ous</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>subitaneus</ets>. See <er>Sudden</er>.]</ety> <def>Sudden; hasty.</def> <mark>[Obs.]</mark> <au>Bullokar.</au> -- <wordforms><wf>Sub`i*ta"ne*ous*ness</wf>, <pos>n.</pos> <mark>[Obs.]</mark></wordforms><br/
[<source>1913 Webster</source>]</p>

<p><ent>Subitany</ent><br/
<hw>Sub"i*ta*ny</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Subitaneous; sudden; hasty.</def> <mark>[Obs.]</mark>  <rj><au>Hales.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Subito</ent><br/
||<hw>Su"bi*to</hw> <pr>(?)</pr>, <pos>adv.</pos> <ety>[It. & L.]</ety> <fld>(Mus.)</fld> <def>In haste; quickly; rapidly.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Subjacent</ent><br/
<hw>Sub*ja"cent</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>subjacens</ets>, p. pr. of <ets>subjacere</ets> to lie under; <ets>sub</ets> under + <ets>jacere</ets> to lie.]</ety> <sn>1.</sn> <def>Lying under or below.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Being in a lower situation, though not directly beneath; <as>as, hills and <ex>subjacent</ex> valleys</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Subject</ent><br/
<hw>Sub*ject"</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[OE. <ets>suget</ets>, OF. <ets>souzget</ets>, <ets>sougit</ets> (in which the first part is L. <ets>subtus</ets> below, fr. <ets>sub</ets> under), <ets>subgiet</ets>, <ets>subject</ets>, F. <ets>sujet</ets>, from L. <ets>subjectus</ets> lying under, subjected, p. p. of <ets>subjicere</ets>, <ets>subicere</ets>, to throw, lay, place, or bring under; <ets>sub</ets> under + <ets>jacere</ets> to throw. See <er>Jet</er> a shooting forth.]</ety> <sn>1.</sn> <def>Placed or situated under; lying below, or in a lower situation.</def> <mark>[Obs.]</mark>  <rj><au>Spenser.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Placed under the power of another; specifically <fld>(International Law)</fld>, owing allegiance to a particular sovereign or state; <as>as, Jamaica is <ex>subject</ex> to Great Britain</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Esau was never <qex>subject</qex> to Jacob.</q> <rj><qau>Locke.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>Exposed; liable; prone; disposed; <as>as, a country <ex>subject</ex> to extreme heat; men <ex>subject</ex> to temptation.</as></def><br/
[<source>1913 Webster</source>]</p>

<p><q>All human things are <qex>subject</qex> to decay.</q> <rj><qau>Dryden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>Obedient; submissive.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Put them in mind to be <qex>subject</qex> to principalities.</q> <rj><qau>Titus iii. 1.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><syn><b>Syn.</b> -- Liable; subordinate; inferior; obnoxious; exposed. See <er>Liable</er>.</syn><br/
[<source>1913 Webster</source>]</p>

<p><ent>Subject</ent><br/
<hw>Sub*ject"</hw>, <pos>n.</pos> <ety>[From L. <ets>subjectus</ets>, through an old form of F. <ets>sujet</ets>. See <er>Subject</er>, <pos>a.</pos>]</ety> <sn>1.</sn> <def>That which is placed under the authority, dominion, control, or influence of something else.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Specifically: One who is under the authority of a ruler and is governed by his laws; one who owes allegiance to a sovereign or a sovereign state; <as>as, a <ex>subject</ex> of Queen Victoria; a British <ex>subject</ex>; a <ex>subject</ex> of the United States.</as></def><br/
[<source>1913 Webster</source>]</p>

<p><q>Was never <qex>subject</qex> longed to be a king,<br/
As I do long and wish to be a <qex>subject</qex>.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>The <qex>subject</qex> must obey his prince, because God commands it, human laws require it.</q> <rj><qau>Swift.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ In international law, the term <xex>subject</xex> is convertible with <xex>citizen</xex>.</note><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>That which is subjected, or submitted to, any physical operation or process; specifically <fld>(Anat.)</fld>, a dead body used for the purpose of dissection.</def>
<-- also, an animal or person which is studied in a scientific experiment. --><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>That which is brought under thought or examination; that which is taken up for discussion, or concerning which anything is said or done.</def> <ldquo/This <xex>subject</xex> for heroic song.<rdquo/  <rj><au>Milton.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Make choice of a <qex>subject</qex>, beautiful and noble, which . . . shall afford an ample field of matter wherein to expatiate.</q> <rj><qau>Dryden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>The unhappy <qex>subject</qex> of these quarrels.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <def>The person who is treated of; the hero of a piece; the chief character.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Writers of particular lives . . . are apt to be prejudiced in favor of their <qex>subject</qex>.</q> <rj><qau>C. Middleton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>6.</sn> <fld>(Logic & Gram.)</fld> <def>That of which anything is affirmed or predicated; the theme of a proposition or discourse; that which is spoken of; <as>as, the nominative case is the <ex>subject</ex> of the verb</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The <qex>subject</qex> of a proposition is that concerning which anything is affirmed or denied.</q> <rj><qau>I. Watts.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>7.</sn> <def>That in which any quality, attribute, or relation, whether spiritual or material, inheres, or to which any of these appertain; substance; substratum.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>That which manifests its qualities -- in other words, that in which the appearing causes inhere, that to which they belong -- is called their <qex>subject</qex> or substance, or substratum.</q> <rj><qau>Sir W. Hamilton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>8.</sn> <def>Hence, that substance or being which is conscious of its own operations; the mind; the thinking agent or principal; the ego.  Cf. <er>Object</er>, <pos>n.</pos>, 2.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The philosophers of mind have, in a manner, usurped and appropriated this expression to themselves. Accordingly, in their hands, the phrases <qex>conscious</qex> or <qex>thinking subject</qex>, and <qex>subject</qex>, mean precisely the same thing.</q> <rj><qau>Sir W. Hamilton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>9.</sn> <fld>(Mus.)</fld> <def>The principal theme, or leading thought or phrase, on which a composition or a movement is based.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The earliest known form of <qex>subject</qex> is the ecclesiastical <qex>cantus firmus</qex>, or plain song.</q> <rj><qau>Rockstro.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>10.</sn> <fld>(Fine Arts)</fld> <def>The incident, scene, figure, group, etc., which it is the aim of the artist to represent.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Subject</ent><br/
<hw>Sub*ject"</hw> <pr>(?)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Subjected</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Subjecting</conjf>.]</vmorph> <sn>1.</sn> <def>To bring under control, power, or dominion; to make subject; to subordinate; to subdue.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Firmness of mind that <qex>subjects</qex> every gratification of sense to the rule of right reason.</q> <rj><qau>C. Middleton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>In one short view <qex>subjected</qex> to our eye,<br/
Gods, emperors, heroes, sages, beauties, lie.</q> <rj><qau>Pope.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>He is the most <qex>subjected</qex>, the most <?/nslaved, who is so in his understanding.</q> <rj><qau>Locke.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To expose; to make obnoxious or liable; <as>as, credulity <ex>subjects</ex> a person to impositions</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>To submit; to make accountable.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>God is not bound to <qex>subject</qex> his ways of operation to the scrutiny of our thoughts.</q> <rj><qau>Locke.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>To make subservient.</def><br/
[<source>1913 Webster</source>]</p>

<p><q><qex>Subjected</qex> to his service angel wings.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <def>To cause to undergo; <as>as, to <ex>subject</ex> a substance to a white heat; to <ex>subject</ex> a person to a rigid test</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><-- p. 1434 --></p>

<p><ent>Subjected</ent><br/
<hw>Sub*ject"ed</hw> <pr>(?)</pr>, <pos>a.</pos> <sn>1.</sn> <def>Subjacent.</def> <ldquo/Led them direct . . . to the <xex>subjected</xex> plain.<rdquo/ <mark>[Obs.]</mark>  <rj><au>Milton.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Reduced to subjection; brought under the dominion of another.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>Exposed; liable; subject; obnoxious.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Subjection</ent><br/
<hw>Sub*jec"tion</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>subjectio</ets>: cf. OF. <ets>subjection</ets>, F. <ets>subj<eacute/tion</ets>. See <er>Subject</er>, <pos>a.</pos>]</ety> <sn>1.</sn> <def>The act of subjecting, or of bringing under the dominion of another; the act of subduing.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The conquest of the kingdom, and <qex>subjection</qex> of the rebels.</q> <rj><qau>Sir M. Hale.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>The state of being subject, or under the power, control, and government of another; a state of obedience or submissiveness; <as>as, the safety of life, liberty, and property depends on our <ex>subjection</ex> to the laws</as>.</def> <ldquo/To be bound under <xex>subjection</xex>.<rdquo/  <rj><au>Chaucer.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Likewise, ye wives, be in <qex>subjection</qex> to your own husbands.</q> <rj><qau>1 Peter iii. 1.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Because the <qex>subjection</qex> of the body to the will is by natural necessity, the <qex>subjection</qex> of the will unto God voluntary, we stand in need of direction after what sort our wills and desires may be rightly conformed to His.</q> <rj><qau>Hooker.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Subjectist</ent><br/
<hw>Sub"ject*ist</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Metaph.)</fld> <def>One skilled in subjective philosophy; a subjectivist.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Subjective</ent><br/
<hw>Sub*jec"tive</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>subjectivus</ets>: cf. F. <ets>subjectif</ets>.]</ety> <sn>1.</sn> <def>Of or pertaining to a subject.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Especially, pertaining to, or derived from, one's own consciousness, in distinction from external observation; ralating to the mind, or intellectual world, in distinction from the outward or material excessively occupied with, or brooding over, one's own internal states.</def><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ In the philosophy of the mind, <xex>subjective</xex> denotes what is to be referred to the thinking subject, the <xex>ego</xex>; <xex>objective</xex>, what belongs to the object of thought, the <xex>non-ego</xex>. See <er>Objective</er>, <pos>a.</pos>, 2.</note>  <rj><au>Sir W. Hamilton.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <fld>(Lit. & Art)</fld> <def>Modified by, or making prominent, the individuality of a writer or an artist; <as>as, a <ex>subjective</ex> drama or painting; a <ex>subjective</ex> writer</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><syn><b>Syn.</b> -- See <er>Objective</er>.</syn><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Subjective sensation</b></col> <fld>(Physiol.)</fld>, <cd>one of the sensations occurring when stimuli due to internal causes excite the nervous apparatus of the sense organs, as when a person imagines he sees figures which have no objective reality.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p>-- <wordforms><wf>Sub*jec"tive*ly</wf>, <pos>adv.</pos> -- <wf>Sub*jec"tive*ness</wf>, <pos>n.</pos></wordforms><br/
[<source>1913 Webster</source>]</p>

<p><ent>Subjectivism</ent><br/
<hw>Sub*jec"tiv*ism</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Metaph.)</fld> <def>Any philosophical doctrine which refers all knowledge to, and founds it upon, any subjective states; egoism.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Subjectivist</ent><br/
<hw>Sub*jec"tiv*ist</hw>, <pos>n.</pos> <fld>(Metaph.)</fld> <def>One who holds to subjectivism; an egoist.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Subjectivity</ent><br/
<hw>Sub`jec*tiv"i*ty</hw> <pr>(?)</pr>, <pos>n.</pos> <def>The quality or state of being subjective; character of the subject.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Subjectless</ent><br/
<hw>Sub"ject*less</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Having no subject.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Subject-matter</ent><br/
<hw>Sub"ject-mat`ter</hw> <pr>(?)</pr>, <pos>n.</pos> <def>The matter or thought presented for consideration in some statement or discussion; that which is made the object of thought or study.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>As to the <qex>subject-matter</qex>, words are always to be understood as having a regard thereto.</q> <rj><qau>Blackstone.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>As science makes progress in any <qex>subject-matter</qex>, poetry recedes from it.</q> <rj><qau>J. H. Newman.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Subjectness</ent><br/
<hw>Sub"ject*ness</hw>, <pos>n.</pos> <def>Quality of being subject.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Subjicible</ent><br/
<hw>Sub*jic"i*ble</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Capable of being subjected.</def> <mark>[Obs.]</mark>  <rj><au>Jer. Taylor.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Subjoin</ent><br/
<hw>Sub*join"</hw> <pr>(?)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Subjoined</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Subjoining</conjf>.]</vmorph> <ety>[Cf. OF. <ets>subjoindre</ets>, L. <ets>subjungere</ets>. See <er>Sub-</er>, and <er>Join</er>, and cf. <er>Subjective</er>.]</ety> <def>To add after something else has been said or written; to ANNEX; <as>as, to <ex>subjoin</ex> an argument or reason</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><syn><b>Syn.</b> -- To add; annex; join; unite.</syn><br/
[<source>1913 Webster</source>]</p>

<p><ent>Subjoinder</ent><br/
<hw>Sub*join"der</hw> <pr>(?)</pr>, <pos>n.</pos> <def>An additional remark.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sub judice</ent><br/
||<hw>Sub ju"di*ce</hw> <pr>(?)</pr>. <ety>[L.]</ety> <def>Before the judge, or court; not yet decided; under judicial consideration.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Subjugate</ent><br/
<hw>Sub"ju*gate</hw> <pr>(?)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Subjugated</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Subjugating</conjf>.]</vmorph> <ety>[L. <ets>subjugatus</ets>, p. p. of <ets>subjugare</ets> to subjugate; <ets>sub</ets> under + <ets>jugum</ets> a yoke. See <er>Yoke</er>.]</ety> <def>To subdue, and bring under the yoke of power or dominion; to conquer by force, and compel to submit to the government or absolute control of another; to vanquish.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>He <qex>subjugated</qex> a king, and called him his <ldquo/vassal.<rdquo/</q> <rj><qau>Baker.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><syn><b>Syn.</b> -- To conquer; subdue; overcome. See <er>Conquer</er>.</syn><br/
[<source>1913 Webster</source>]</p>

<p><ent>Subjugation</ent><br/
<hw>Sub`ju*ga"tion</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Cf. F. <ets>subjugation</ets>, LL. <ets>subjugatio</ets>.]</ety> <def>The act of subjugating, or the state of being subjugated.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Subjugator</ent><br/
<hw>Sub"ju*ga`tor</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L.]</ety> <def>One who subjugates; a conqueror.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Subjunction</ent><br/
<hw>Sub*junc"tion</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[See <er>Subjunctive</er>.]</ety> <sn>1.</sn> <def>Act of subjoining, or state of being subjoined.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Something subjoined; <as>as, a <ex>subjunction</ex> to a sentence</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Subjunctive</ent><br/
<hw>Sub*junc"tive</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>subjunctivus</ets>, fr. <ets>subjungere</ets>, <ets>subjunctum</ets>, to subjoin: cf. F. <ets>subjonctif</ets>. See <er>Subjoin</er>.]</ety> <def>Subjoined or added to something before said or written.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Subjunctive mood</b></col> <fld>(Gram.)</fld>, <cd>that form of a verb which express the action or state not as a fact, but only as a conception of the mind still contingent and dependent. It is commonly subjoined, or added as subordinate, to some other verb, and in English is often connected with it by <xex>if</xex>, <xex>that</xex>, <xex>though</xex>, <xex>lest</xex>, <xex>unless</xex>, <xex>except</xex>, <xex>until</xex>, etc., as in the following sentence: <ldquo/<xex>If there were</xex> no honey, they [bees] <xex>would have</xex> no object in visiting the flower.<rdquo/ <au>Lubbock.</au> In some languages, as in Latin and Greek, the subjunctive is often independent of any other verb, being used in wishes, commands, exhortations, etc.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Subjunctive</ent><br/
<hw>Sub*junc"tive</hw>, <pos>n.</pos> <fld>(Gram.)</fld> <def>The subjunctive mood; also, a verb in the subjunctive mood.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Subkingdom</ent><br/
<hw>Sub*king"dom</hw> <pr>(?)</pr>, <pos>n.</pos> <def>One of the several primary divisions of either the animal, or vegetable kingdom, as, in Zoology, the Vertebrata, Tunicata, Mollusca, Articulata, Molluscoidea, Echinodermata, Coelentera, and the Protozoa; in botany, the Phanerogamia, and the Cryptogamia.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sublapsarian</ent><br/
<hw>Sub`lap*sa"ri*an</hw> <pr>(?)</pr>, <pos>n. & a.</pos> <ety>[Pref. <ets>sub + lapse</ets>: cf. F. <ets>sublapsarien</ets>, <ets>sublapsarie</ets>.]</ety> <fld>(Eccl. Hist.)</fld> <def>Same as <er>Infralapsarian</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sublapsarianism</ent><br/
<hw>Sub`lap*sa"ri*an*ism</hw> <pr>(?)</pr>, <pos>n.</pos> <def>Infralapsarianism.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sublapsary</ent><br/
<hw>Sub*lap"sa*ry</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Sublapsarian.</def>  <rj><au>Johnson.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sublate</ent><br/
<hw>Sub"late</hw> <pr>(?)</pr>, <pos>v. t.</pos> <ety>[From <ets>sublatus</ets>, used as p. p. of <ets>tollere</ets> to take away. See <er>Tolerate</er>.]</ety> <def>To take or carry away; to remove.</def> <mark>[R.]</mark>  <rj><au>E. Hall.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sublation</ent><br/
<hw>Sub*la"tion</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>sublatio</ets>, fr. <ets>sublatus</ets>, used as p. p. of <ets>tollere</ets> to take away.]</ety> <def>The act of taking or carrying away; removal.</def> <mark>[R.]</mark>  <rj><au>Bp. Hall.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sublative</ent><br/
<hw>Sub"la*tive</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Having power, or tending, to take away.</def> <mark>[R.]</mark>  <rj><au>Harris.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sublease</ent><br/
<hw>Sub"lease`</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Law)</fld> <def>A lease by a tenant or lessee to another person; an underlease.</def>  <rj><au>Bouvier.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sublessee</ent><br/
<hw>Sub`les*see"</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A holder of a sublease.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sublet</ent><br/
<hw>Sub*let"</hw> <pr>(?)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Sublet</conjf>; <pos>p. pr. & vb. n.</pos> <conjf>Subletting</conjf>.]</vmorph> <def>To underlet; to lease, as when a lessee leases to another person.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sublevation</ent><br/
<hw>Sub`le*va"tion</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>sublevare</ets> to lift up; <ets>sub</ets> under + <ets>levare</ets> to lift, raise: cf. L. <ets>sublevatio</ets> an allevation.]</ety> <sn>1.</sn> <def>The act of raising on high; elevation.</def>  <rj><au>Sir T. More.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>An uprising; an insurrection.</def> <mark>[R.]</mark>  <rj><au>Sir W. Temple.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sublibrarian</ent><br/
<hw>Sub`li*bra"ri*an</hw> <pr>(?)</pr>, <pos>n.</pos> <def>An under or assistant librarian.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sublieutenant</ent><br/
<hw>Sub`lieu*ten"ant</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Pref. <ets>sub</ets> + <ets>lieutenant</ets>: cf. F. <ets>sous-lieutenant</ets>.]</ety> <def>An inferior or second lieutenant; in the British service, a commissioned officer of the lowest rank.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Subligation</ent><br/
<hw>Sub`li*ga"tion</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>subligatio</ets>, from <ets>subligare</ets> to bind below; <ets>sub</ets> under + <ets>ligare</ets> to bind.]</ety> <def>The act of binding underneath.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sublimable</ent><br/
<hw>Sub*lim"a*ble</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[Cf. F. <ets>sublimable</ets>. See <er>Sublime</er>., <pos>v. t.</pos>]</ety> <def>Capable of being sublimed or sublimated.</def> -- <wordforms><wf>Sub*lim"a*ble*ness</wf>, <pos>n.</pos></wordforms>  <rj><au>Boyle.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sublimate</ent><br/
<hw>Sub"li*mate</hw> <pr>(?)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Sublimated</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Sublimating</conjf>.]</vmorph> <ety>[L. <ets>sublimatus</ets>, p. p. of <ets>sublimare</ets> to raise, elevate, fr. <ets>sublimis</ets> high: cf. F. <ets>sublimer</ets>. See <er>Sublime</er>, <pos>a.</pos>, and cf. <er>Surlime</er>, <pos>v. t.</pos>]</ety> <sn>1.</sn> <def>To bring by heat into the state of vapor, which, on cooling, returns again to the solid state; <as>as, to <ex>sublimate</ex> sulphur or camphor</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To refine and exalt; to heighten; to elevate.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The precepts of Christianity are . . . so apt to cleanse and <qex>sublimate</qex> the more gross and corrupt.</q> <rj><qau>Dr. H. More.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <fld>(Psychology)</fld> <def>To redirect the energy (of sexual or other biological drives) into a more socially acceptable or constructive form.</def><br/
[<source>PJC</source>]</p>

<p><ent>Sublimate</ent><br/
<hw>Sub"li*mate</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[LL. <ets>sublimatum</ets>.]</ety> <fld>(Chem.)</fld> <def>A product obtained by sublimation; hence, also, a purified product so obtained.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Corrosive sublimate</b></col>. <fld>(Chem.)</fld> <cd>mercuric chloride.  See <cref>Corrosive sublimate</cref> under <er>Corrosive</er>.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sublimate</ent><br/
<hw>Sub"li*mate</hw>, <pos>a.</pos> <ety>[LL. <ets>sublimatus</ets>.]</ety> <def>Brought into a state of vapor by heat, and again condensed as a solid.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sublimated</ent><br/
<hw>Sub"li*ma`ted</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Refined by, or as by, sublimation; exalted; purified.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>[Words] whose weight best suits a <qex>sublimated</qex> strain.</q> <rj><qau>Dryden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sublimation</ent><br/
<hw>Sub"li*ma`tion</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[LL. <ets>sublimatio</ets>: cf. F. <ets>sublimation</ets>.]</ety> <sn>1.</sn> <fld>(Chem.)</fld> <def>The act or process of subliming, or the state or result of being sublimed.</def>
<-- the process of vaporizing a solid and recondensing it into a solid, without it having first passed into the liquid state.  Certain solids, such as camphor, have a suffiently high vapor pressure in the solid phase to make this a practical method for purification. --><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>The act of heightening or improving; exaltation; elevation; purification.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>That which is sublimed; the product of a purifying process.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Religion is the perfection, refinement, and <qex>sublimation</qex> of morality.</q> <rj><qau>South.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sublimatory</ent><br/
<hw>Sub"li*ma*to*ry</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Used for sublimation; <as>as, <ex>sublimatory</ex> vessels</as>.</def>  <rj><au>Boyle.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sublimatory</ent><br/
<hw>Sub"li*ma*to*ry</hw>, <pos>n.</pos> <def>A vessel used for sublimation.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Vials, crosslets, and <qex>sublimatories</qex>.</q> <rj><qau>Chaucer.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sublime</ent><br/
<hw>Sub*lime"</hw> <pr>(?)</pr>, <pos>a.</pos> <amorph>[<pos>Compar.</pos> <adjf>Sublimer</adjf> <pr>(?)</pr>; <pos>superl.</pos> <adjf>Sublimest</adjf>.]</amorph> <ety>[L. <ets>sublimis</ets>; <ets>sub</ets> under + (perhaps) a word akin to <ets>limen</ets> lintel, sill, thus meaning, up to the lintel: cf. F. <ets>sublime</ets>.  Cf. <er>Eliminate</er>.]</ety> <sn>1.</sn> <def>Lifted up; high in place; exalted aloft; uplifted; lofty.</def><br/
[<source>1913 Webster</source>]</p>

<p><q><qex>Sublime</qex> on these a tower of steel is reared.</q> <rj><qau>Dryden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Distinguished by lofty or noble traits; eminent; -- said of persons.</def> <ldquo/The <xex>sublime</xex> Julian leader.<rdquo/  <rj><au>De Quincey.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>Awakening or expressing the emotion of awe, adoration, veneration, heroic resolve, etc.; dignified; grand; solemn; stately; -- said of an impressive object in nature, of an action, of a discourse, of a work of art, of a spectacle, etc.; <as>as, <ex>sublime</ex> scenery; a <ex>sublime</ex> deed</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Easy in words thy style, in sense <qex>sublime</qex>.</q> <rj><qau>Prior.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Know how <qex>sublime</qex> a thing it is<br/
To suffer and be strong.</q> <rj><qau>Longfellow.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>Elevated by joy; elate.</def> <mark>[Poetic]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>Their hearts were jocund and <qex>sublime</qex>,<br/
Drunk with idolatry, drunk with wine.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <def>Lofty of mien; haughty; proud.</def> <mark>[Poetic]</mark> <ldquo/Countenance <xex>sublime</xex> and insolent.<rdquo/  <rj><au>Spenser.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>His fair, large front and eye <qex>sublime</qex> declared<br/
Absolute rule.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><syn><b>Syn.</b> -- Exalted; lofty; noble; majestic. See <er>Grand</er>.</syn><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sublime</ent><br/
<hw>Sub*lime"</hw>, <pos>n.</pos> <def>That which is sublime; -- with the definite article</def>; as: <sd>(a)</sd> <def>A grand or lofty style in speaking or writing; a style that expresses lofty conceptions.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The <qex>sublime</qex> rises from the nobleness of thoughts, the magnificence of words, or the harmonious and lively turn of the phrase.</q> <rj><qau>Addison.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sd>(b)</sd> <def>That which is grand in nature or art, as distinguished from the merely beautiful.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sublime</ent><br/
<hw>Sub*lime"</hw>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Sublimed</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Subliming</conjf>.]</vmorph> <ety>[Cf. L. <ets>sublimare</ets>, F. <ets>sublimer</ets> to subject to sublimation. See <er>Sublime</er>, <pos>a.</pos>, and cf. <er>Sublimate</er>, <pos>v. t.</pos>]</ety> <sn>1.</sn> <def>To raise on high.</def> <mark>[Archaic]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>A soul <qex>sublimed</qex> by an idea above the region of vanity and conceit.</q> <rj><qau>E. P. Whipple.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Chem.)</fld> <def>To subject to the process of sublimation; to heat, volatilize, and condense in crystals or powder; to distill off, and condense in solid form; hence, also, to purify.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>To exalt; to heighten; to improve; to purify.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The sun . . . <br/
Which not alone the southern wit <qex>sublimes</qex>,<br/
But ripens spirits in cold, northern climes.</q> <rj><qau>Pope.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>To dignify; to ennoble.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>An ordinary gift can not <qex>sublime</qex> a person to a supernatural employment.</q> <rj><qau>Jer. Taylor.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sublime</ent><br/
<hw>Sub*lime"</hw> <pr>(?)</pr>, <pos>v. i.</pos> <fld>(Chem.)</fld> <def>To pass off in vapor, with immediate condensation; specifically, to evaporate or volatilize from the solid state without apparent melting; -- said of those substances, like arsenic, benzoic acid, etc., which do not exhibit a liquid form on heating, except under increased pressure.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sublimed</ent><br/
<hw>Sub*limed"</hw> <pr>(?)</pr>, <pos>a.</pos> <fld>(Chem.)</fld> <def>Having been subjected to the process of sublimation; hence, also, purified.</def> <ldquo/<xex>Sublimed</xex> mercurie.<rdquo/  <rj><au>Chaucer.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sublimely</ent><br/
<hw>Sub*lime"ly</hw> <pr>(?)</pr>, <pos>adv.</pos> <def>In a sublime manner.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sublimeness</ent><br/
<hw>Sub*lime"ness</hw>, <pos>n.</pos> <def>The quality or state of being sublime; sublimity.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sublimification</ent><br/
<hw>Sub*lim`i*fi*ca"tion</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>sublimis</ets> sublime + <ets>-ficare</ets> to make. See <er>-ry</er>.]</ety> <def>The act of making sublime, or state of being made sublime.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Subliminal</ent><br/
<hw>Sub*lim"i*nal</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[Pref. <ets>sub-</ets> + L. <ets>limen</ets> threshold.]</ety> <sn>1.</sn> <fld>(Philos.)</fld> <def>Existing in the mind, but below the surface or threshold of consciousness; that is, existing as feeling rather than as clear ideas.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><sn>2.</sn> <def>having or using an intensity of sensory stimulus insufficient to be perceived consciously, but having an effect on unconscious mental processes; <as>as, <ex>subliminal</ex> advertising</as>.</def> <note>The existence of such an effect is still not universally accepted.</note><br/
[<source>PJC</source>]</p>

<p><ent>Sublimity</ent><br/
<hw>Sub*lim"i*ty</hw> <pr>(?)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Sublimities</plw> <pr>(#)</pr>.</plu> <ety>[L. <ets>sublimitas</ets>: cf. F. <ets>sublimit<eacute/</ets>.]</ety> <sn>1.</sn> <def>The quality or state of being sublime (in any sense of the adjective).</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>That which is sublime; <as>as, the <ex>sublimities</ex> of nature</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><syn><b>Syn.</b> -- Grandeur; magnificence.</syn>  -- <usage><er>Sublimity</er>, <er>Grandeur</er>. The mental state indicated by these two words is the same, namely, a mingled emotion of astonishment and awe. In speaking of the quality which produces this emotion, we call it <xex>grandeur</xex> when it springs from what is vast in space, power, etc.; we call it <xex>sublimity</xex> when it springs from what is elevated far above the ordinary incidents of humanity. An immense plain is <xex>grand</xex>. The heavens are not only <xex>grand</xex>, but <xex>sublime</xex> (as the predominating emotion), from their immense height. Exalted intellect, and especially exalted virtue under severe trials, give us the sense of moral <xex>sublimity</xex>, as in the case of our Savior in his prayer for his murderers. We do not speak of Satan, when standing by the fiery gulf, with his <ldquo/unconquerable will and study of revenge,<rdquo/ as a <xex>sublime</xex> object; but there is a melancholy <xex>grandeur</xex> thrown around him, as of an <ldquo/archangel ruined.<rdquo/</usage><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sublineation</ent><br/
<hw>Sub*lin`e*a"tion</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A mark of a line or lines under a word in a sentence, or under another line; underlining.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sublingua</ent><br/
||<hw>Sub*lin"gua</hw> <pr>(?)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Sublinguae</plw> <pr>(#)</pr>.</plu> <ety>[NL.]</ety> <fld>(Anat.)</fld> <def>A process or fold below the tongue in some animals.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sublingual</ent><br/
<hw>Sub*lin"gual</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[Pref. <ets>sub + lingual</ets>: cf. F. <ets>sublingual</ets>.]</ety> <fld>(Anat.)</fld> <sd>(a)</sd> <def>Situated under the tongue; <as>as, the <ex>sublingual</ex> gland</as>.</def> <sd>(b)</sd> <def>Of or pertaining to the sublingual gland; <as>as, <ex>sublingual</ex> salvia</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sublition</ent><br/
<hw>Sub*li"tion</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>sublinere</ets>, <ets>sublitum</ets>, to smear, to lay on as a ground color.]</ety> <fld>(Paint.)</fld> <def>The act or process of laying the ground in a painting.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sublittoral</ent><br/
<hw>Sub*lit"to*ral</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Under the shore.</def>  <rj><au>Smart.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sublobular</ent><br/
<hw>Sub*lob"u*lar</hw> <pr>(?)</pr>, <pos>a.</pos> <fld>(Anat.)</fld> <def>Situated under, or at the bases of, the lobules of the liver.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sublumbar</ent><br/
<hw>Sub*lum"bar</hw> <pr>(?)</pr>, <pos>a.</pos> <fld>(Anat.)</fld> <def>Situated under, or on the ventral side of, the lumbar region of the vertebral column.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sublunary</ent><br/
<ent>Sublunar</ent><br/
<mhw>{ <hw>Sub*lu"nar</hw> <pr>(?)</pr>, <hw>Sub"lu*na*ry</hw> <pr>(?)</pr>, }</mhw> <pos>a.</pos> <ety>[Pref. <ets>sub + lunar</ets>, or <ets>lunary</ets>: cf. F. <ets>sublunaire</ets>.]</ety> <def>Situated beneath the moon; hence, of or pertaining to this world; terrestrial; earthly.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>All things <qex>sublunary</qex> are subject to change.</q> <rj><qau>Dryden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>All <qex>sublunary</qex> comforts imitate the changeableness, as well as feel the influence, of the planet they are under.</q> <rj><qau>South.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sublunary</ent><br/
<hw>Sub"lu*na*ry</hw>, <pos>n.</pos> <def>Any worldly thing.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Subluxation</ent><br/
<hw>Sub`lux*a"tion</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Pref. <ets>sub + luxation</ets>: cf. F. <ets>subluxation</ets>.]</ety> <fld>(Surg.)</fld> <def>An incomplete or partial dislocation.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Submammary</ent><br/
<hw>Sub*mam"ma*ry</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Situated under the mammae; <as>as, <ex>submammary</ex> inflammation</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Submarine</ent><br/
<hw>Sub`ma*rine"</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Being, acting, or growing, under water in the sea; <as>as, <ex>submarine</ex> navigators; <ex>submarine</ex> plants</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Submarine armor</b></col>, <cd>a waterproof dress of strong material, having a helmet into which air for breathing is pumped through a tube leading from above the surface to enable a diver to remain under water.</cd> -- <col><b>Submarine cable</b></col>. <cd>See <cref>Telegraph cable</cref>, under <er>Telegraph</er>.</cd> -- <col><b>Submarine mine</b></col>. <cd>See <er>Torpedo</er>, 2 <sd>(a)</sd>.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Submarine</ent><br/
<hw>Sub*ma*rine"</hw>, <pos>n.</pos> <def>A submarine plant or animal.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Submarine</ent><br/
<hw>Sub`ma*rine"</hw> <pr>(?)</pr>, <pos>n.</pos> <sn>1.</sn> <def>A submarine boat; a ship that can travel under the surface of the water.  Most such ships are ships of war, as part of a navy, but submarines are also used for oceanic research.  Also called <altname>sub</altname> and (from the German <ets>U-Boot</ets>) <altname>U-boat</altname>.</def> <specif>esp.,</specif> <fld>Nav.</fld>, <def>a submarine torpedo boat; -- called specif. <altname>submergible submarine</altname> when capable of operating at various depths and of traveling considerable distances under water, and <altname>submersible submarine</altname> when capable of being only partly submerged, i.e., so that the conning tower, etc., is still above water. The latter type and most of the former type are submerged as desired by regulating the amount of water admitted to the ballast tanks and sink on an even keel; some of the former type effect submersion while under way by means of horizontal rudders, in some cases also with admission of water to the ballast tanks.</def><br/
[<source>Webster 1913 Suppl.</source> <source>+PJC</source>]</p>

<p><sn>2.</sn> <def>A stowaway on a seagoing vessel.</def> <mark>[Colloq.]</mark><br/
[<source>PJC</source>]</p>

<p><sn>3.</sn> <def>A <er>submarine sandwich</er>.</def><br/
[<source>PJC</source>]</p>

<p><-- 2.     Nuclear submarine. A submarine powered by a nuclear reactor.
   Attack submarine.  A submarine designed to attack other ships, including other submarines.--></p>

<p><ent>submarine sandwich</ent><br/
<hw>sub`ma*rine" sand"wich</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A large sandwich on an elongated roll, usually incompletely cut into two halves, filed with various cold cuts, meatballs, lettuce, cheese, tomatoes, olives, etc., and spiced variously, and often having oil or other dressing applied; called also <altname>hoagie</altname>, <altname>hero</altname>, <altname>hero sandwich</altname>, <altname>grinder</altname>, <altname>sub</altname>, <altname>submarine</altname>, <altname>poor boy</altname>, and <altname>Italian sandwich</altname>.  A single such sandwich may consitute a substantial meal.  Very large variants are sometimes prepared for social gatherings and cut into pieces for individual consumption.</def><br/
[<source>PJC</source>]</p>

<p><ent>Submarshal</ent><br/
<hw>Sub*mar"shal</hw> <pr>(?)</pr>, <pos>n.</pos> <def>An under or deputy marshal.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Submaxillary</ent><br/
<hw>Sub*max"il*la*ry</hw> <pr>(?)</pr>, <pos>a.</pos> <fld>(Anat.)</fld> <sd>(a)</sd> <def>Situated under the maxilla, or lower jaw; inframaxillary; <as>as, the <ex>submaxillary</ex> gland</as>.</def> <sd>(b)</sd> <def>Of or pertaining to submaxillary gland; <as>as, <ex>submaxillary</ex> salvia</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Submedial</ent><br/
<hw>Sub*me"di*al</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Lying under the middle.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Submedian</ent><br/
<hw>Sub*me"di*an</hw> <pr>(?)</pr>, <pos>a.</pos> <fld>(Zool.)</fld> <def>Next to the median (on either side); <as>as, the <ex>submedian</ex> teeth of mollusks</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Submediant</ent><br/
<hw>Sub*me"di*ant</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Mus.)</fld> <def>The sixth tone of the scale; the under mediant, or third below the keynote; the superdominant.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Submental</ent><br/
<hw>Sub*men"tal</hw> <pr>(?)</pr>, <pos>a.</pos> <fld>(Anat.)</fld> <def>Situated under the chin; <as>as, the <ex>submental</ex> artery</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Submentum</ent><br/
<hw>Sub*men"tum</hw> <pr>(?)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Submenta</plw> <pr>(#)</pr>.</plu> <ety>[NL. See <er>Sub-</er>, and <er>Mentum</er>.]</ety> <fld>(Zool.)</fld> <def>The basal part of the labium of insects. It bears the mentum.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Submerge</ent><br/
<hw>Sub*merge"</hw> <pr>(?)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Submerged</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Submerging</conjf> <pr>(?)</pr>.]</vmorph> <ety>[L. <ets>submergere</ets>, <ets>submersum</ets>; <ets>sub</ets> under + <ets>mergere</ets> to plunge: cf. F. <ets>submerger</ets>. See <er>Merge</er>.]</ety> <sn>1.</sn> <def>To put under water; to plunge.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To cover or overflow with water; to inundate; to flood; to drown.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>I would thou didst,<br/
So half my Egypt were <qex>submerged</qex>.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><-- p. 1435 --></p>

<p><ent>Submerge</ent><br/
<hw>Sub*merge"</hw> <pr>(?)</pr>, <pos>v. i.</pos> <def>To plunge into water or other fluid; to be buried or covered, as by a fluid; to be merged; hence, to be completely included.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Some say swallows <qex>submerge</qex> in ponds.</q> <rj><qau>Gent. Mag.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Submergence</ent><br/
<hw>Sub*mer"gence</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[From L. <ets>submergens</ets>, p. pr.]</ety> <def>The act of submerging, or the state of being submerged; submersion.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Submerse</ent><br/
<hw>Sub*merse"</hw> <pr>(?)</pr>, <pos>a.</pos> <fld>(Bot.)</fld> <def>Submersed.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Submersed</ent><br/
<hw>Sub*mersed"</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>submersus</ets>, p. p. of <ets>submergere</ets>. See <er>Submerge</er>.]</ety> <def>Being or growing under water, as the leaves of aquatic plants.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Submersion</ent><br/
<hw>Sub*mer"sion</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>submersio</ets>: cf. F. <ets>submersion</ets>.]</ety> <sn>1.</sn> <def>The act of submerging, or putting under water or other fluid, or of causing to be overflowed; the act of plunging under water, or of drowning.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>The state of being put under water or other fluid, or of being overflowed or drowned.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Submetallic</ent><br/
<hw>Sub`me*tal"lic</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Imperfectly metallic; <as>as, a <ex>submetallic</ex> luster</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Subminister</ent><br/
<hw>Sub*min"is*ter</hw> <pr>(?)</pr>, <pos>v. t.</pos> <ety>[L. <ets>subministrare</ets>, <ets>subministratum</ets>. See <er>Sub-</er>, and <er>Ministre</er>, <pos>v. t.</pos>]</ety> <def>To supply; to afford.</def> <mark>[Obs.]</mark>  <rj><au>Sir M. Hale.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Subminister</ent><br/
<hw>Sub*min"is*ter</hw>, <pos>v. i.</pos> <def>To be subservient; to be useful.</def> <mark>[Obs.]</mark> <ldquo/Our passions . . . <xex>subminister</xex> to the best and worst purposes.<rdquo/  <rj><au>L'EStrange.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Subministrant</ent><br/
<hw>Sub*min"is*trant</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>subministrans</ets>, p. pr.]</ety> <def>Subordinate; subservient.</def> <mark>[Obs.]</mark>  <rj><au>Bacon.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Subministrate</ent><br/
<hw>Sub*min"is*trate</hw> <pr>(?)</pr>, <pos>v. t.</pos> <ety>[See <er>Subminister</er>.]</ety> <def>To supply; to afford; to subminister.</def> <mark>[Obs.]</mark>  <rj><au>Harvey.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Subministration</ent><br/
<hw>Sub*min`is*tra"tion</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>subministratio</ets>.]</ety> <def>The act of subministering.</def> <mark>[Obs.]</mark>  <rj><au>Sir H. Wotton.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Submiss</ent><br/
<hw>Sub*miss"</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>submissus</ets>, p. p. of <ets>submittere</ets> to let down, to lower. See <er>Submit</er>.]</ety> <sn>1.</sn> <def>Submissive; humble; obsequious.</def> <mark>[Archaic]</mark> <ldquo/Soft Silence and <xex>submiss</xex> Obedience.<rdquo/ <au>Spenser.</au> <ldquo/Stooping and <xex>submiss</xex>.<rdquo/  <rj><au>R. L. Stevenson.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Gentle; soft; calm; <as>as, <ex>submiss</ex> voices</as>.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Submission</ent><br/
<hw>Sub*mis"sion</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>submissio</ets> a letting down, lowering: cf. F. <ets>soumission</ets>.]</ety> <sn>1.</sn> <def>The act of submitting; the act of yielding to power or authority; surrender of the person and power to the control or government of another; obedience; compliance.</def><br/
[<source>1913 Webster</source>]</p>

<p><q><qex>Submission</qex>, dauphin! 't is a mere French word;<br/
We English warrious wot not what it means.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>The state of being submissive; acknowledgement of inferiority or dependence; humble or suppliant behavior; meekness; resignation.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>In all <qex>submission</qex> and humility<br/
York doth present himself unto your highness.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>No duty in religion is more justly required by God . . . than a perfect <qex>submission</qex> to his will in all things.</q> <rj><qau>Sir W. Temple.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>Acknowledgement of a fault; confession of error.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Be not as extreme in <qex>submission</qex><br/
As in offense.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <fld>(Law)</fld> <def>An agreement by which parties engage to submit any matter of controversy between them to the decision of arbitrators.</def>  <rj><au>Wharton (Law Dict.). Bouvier.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Submissive</ent><br/
<hw>Sub*mis"sive</hw> <pr>(?)</pr>, <pos>a.</pos> <sn>1.</sn> <def>Inclined or ready to submit; acknowledging one's inferiority; yielding; obedient; humble.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Not at his feet <qex>submissive</qex> in distress,<br/
Creature so fair his reconcilement seeking.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Showing a readiness to submit; expressing submission; <as>as, a <ex>submissive</ex> demeanor</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>With a <qex>submissive</qex> step I hasted down.</q> <rj><qau>Prior.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><syn><b>Syn.</b> -- Obedient; compliant; yielding; obsequious; subservient; humble; modest; passive.</syn><br/
[<source>1913 Webster</source>]</p>

<p>-- <wordforms><wf>Sub*mis"sive*ly</wf>, <pos>adv.</pos> -- <wf>Sub*mis"sive*ness</wf>, <pos>n.</pos></wordforms><br/
[<source>1913 Webster</source>]</p>

<p><ent>Submissly</ent><br/
<hw>Sub*miss"ly</hw> <pr>(?)</pr>, <pos>adv.</pos> <def>In a submissive manner; with a submission.</def> <mark>[Archaic]</mark>  <rj><au>Jer. Taylor.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Submissness</ent><br/
<hw>Sub*miss"ness</hw>, <pos>n.</pos> <def>Submissiveness.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Submit</ent><br/
<hw>Sub*mit"</hw> <pr>(?)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Submitted</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Submitting</conjf>.]</vmorph> <ety>[L. <ets>submittere</ets>; <ets>sub</ets> under + <ets>mittere</ets> to send: cf. F. <ets>soumettre</ets>. See <er>Missile</er>.]</ety> <sn>1.</sn> <def>To let down; to lower.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>Sometimes the hill <qex>submits</qex> itself a while.</q> <rj><qau>Dryden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To put or place under.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The bristled throat<br/
Of the <qex>submitted</qex> sacrifice with ruthless steel he cut.</q> <rj><qau>Chapman.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>To yield, resign, or surrender to power, will, or authority; -- often with the reflexive pronoun.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Ye ben <qex>submitted</qex> through your free assent.</q> <rj><qau>Chaucer.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>The angel of the Lord said unto her, Return to thy mistress, and <qex>submit</qex> thyself under her hands.</q> <rj><qau>Gen. xvi. 9.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Wives, <qex>submit</qex> yourselves unto your own husbands.</q> <rj><qau>Eph. v. 22.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>To leave or commit to the discretion or judgment of another or others; to refer; <as>as, to <ex>submit</ex> a controversy to arbitrators; to <ex>submit</ex> a question to the court</as>; -- often followed by a dependent proposition as the object.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Whether the condition of the clergy be able to bear a heavy burden, is <qex>submitted</qex> to the house.</q> <rj><qau>Swift.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>We <qex>submit</qex> that a wooden spoon of our day would not be justified in calling Galileo and Napier blockheads because they never heard of the differential calculus.</q> <rj><qau>Macaulay.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Submit</ent><br/
<hw>Sub*mit"</hw>, <pos>v. i.</pos> <sn>1.</sn> <def>To yield one's person to the power of another; to give up resistance; to surrender.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The revolted provinces presently <qex>submitted</qex>.</q> <rj><qau>C. Middleton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To yield one's opinion to the opinion of authority of another; to be subject; to acquiesce.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>To thy husband's will<br/
Thine shall <qex>submit</qex>.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>To be submissive or resigned; to yield without murmuring.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Our religion requires from us . . . to <qex>submit</qex> to pain, disgrace, and even death.</q> <rj><qau>Rogers.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Submitter</ent><br/
<hw>Sub*mit"ter</hw> <pr>(?)</pr>, <pos>n.</pos> <def>One who submits.</def>  <rj><au>Whitlock.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Submonish</ent><br/
<hw>Sub*mon"ish</hw> <pr>(?)</pr>, <pos>v. t.</pos> <ety>[L. <ets>submonere</ets>. See <er>Summon</er>, and <er>-ish</er>.]</ety> <def>To suggest; to prompt.</def> <mark>[R.]</mark> <ldquo/The <xex>submonishing</xex> inclinations of my senses.<rdquo/  <rj><au>T. Granger.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Submonition</ent><br/
<hw>Sub`mo*ni"tion</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[LL. <ets>submonitio</ets>.]</ety> <def>Suggestion; prompting.</def> <mark>[R.]</mark>  <rj><au>T. Granger.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Submucous</ent><br/
<hw>Sub*mu"cous</hw> <pr>(?)</pr>, <pos>a.</pos> <fld>(Anat.)</fld> <def>Situated under a mucous membrane.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Submultiple</ent><br/
<hw>Sub*mul"ti*ple</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Math.)</fld> <def>A number or quality which is contained in another an exact number of times, or is an aliquot part of it; thus, 7 is the <xex>submultiple</xex> of 56, being contained in it eight times.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Submultiple</ent><br/
<hw>Sub*mul"ti*ple</hw>, <pos>a.</pos> <fld>(Math.)</fld> <def>Of or pertaining to a submultiple; being a submultiple; <as>as, a <ex>submultiple</ex> number; <ex>submultiple</ex> ratio</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Submuscular</ent><br/
<hw>Sub*mus"cu*lar</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Situated underneath a muscle or muscles.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Subnarcotic</ent><br/
<hw>Sub`nar*cot"ic</hw> <pr>(?)</pr>, <pos>a.</pos> <fld>(Med.)</fld> <def>Moderately narcotic.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Subnasal</ent><br/
<hw>Sub*na"sal</hw> <pr>(?)</pr>, <pos>a.</pos> <fld>(Anat.)</fld> <def>Situated under the nose; <as>as, the <ex>subnasal</ex> point, or the middle point of the inferior border of the anterior nasal aperture</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Subnascent</ent><br/
<hw>Sub*nas"cent</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>subnascens</ets>, p. pr. of <ets>subnasci</ets> to grow under; <ets>sub</ets> under + <ets>nasci</ets> to be born.]</ety> <def>Growing underneath.</def> <mark>[R.]</mark>  <rj><au>Evelyn.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Subnect</ent><br/
<hw>Sub*nect"</hw> <pr>(?)</pr>, <pos>v. t.</pos> <ety>[L. <ets>subnectere</ets>, <ets>subnextum</ets>; <ets>sub</ets> under + <ets>nectere</ets> to tie.]</ety> <def>To tie or fasten beneath; to join beneath.</def> <mark>[R.]</mark>  <rj><au>Pope.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Subnex</ent><br/
<hw>Sub*nex"</hw> <pr>(?)</pr>, <pos>v. t.</pos> <ety>[See <er>Subnect</er>.]</ety> <def>To subjoin; to subnect.</def> <mark>[Obs.]</mark>  <rj><au>Holland.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Subnormal</ent><br/
<hw>Sub*nor"mal</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Geom.)</fld> <def>That part of the axis of a curved line which is intercepted between the ordinate and the normal.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Subnotation</ent><br/
<hw>Sub`no*ta"tion</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>subnotatio</ets> a signing underneath, fr. <ets>subnotare</ets> to subscribe; <ets>sub</ets> under + <ets>notare</ets> to note or mark.]</ety> <def>A rescript.</def>  <rj><au>Bouvier.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Subnotochordal</ent><br/
<hw>Sub*no`to*chor"dal</hw> <pr>(?)</pr>, <pos>a.</pos> <fld>(Anat.)</fld> <def>Situated on the ventral side of the notochord; <as>as, the <ex>subnotochordal</ex> rod</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Subnuvolar</ent><br/
<hw>Sub*nu"vo*lar</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[Pref. <ets>sub</ets> + It. <ets>nuvola</ets> cloud: cf. L. <ets>subnubilus</ets> somewhat cloudy.]</ety> <def>Under the clouds; attended or partly covered or obscured by clouds; somewhat cloudy.</def> <mark>[R. & Poetic]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q><qex>Subnuvolar</qex> lights of evening sharply slant.</q> <rj><qau>Milnes.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Subobscurely</ent><br/
<hw>Sub`ob*scure"ly</hw> <pr>(?)</pr>, <pos>adv.</pos> <def>Somewhat obscurely or darkly.</def> <mark>[R.]</mark>  <rj><au>Donne.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Subobtuse</ent><br/
<hw>Sub`ob*tuse"</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Partially obtuse.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Suboccipital</ent><br/
<hw>Sub`oc*cip"i*tal</hw> <pr>(?)</pr>, <pos>a.</pos> <fld>(Anat.)</fld> <def>Situated under, or posterior to, the occiput; <as>as, the <ex>suboccipital</ex>, or first cervical, nerve</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Suboctuple</ent><br/
<ent>Suboctave</ent><br/
<mhw>{ <hw>Sub*oc"tave</hw> <pr>(?)</pr>, <hw>Sub*oc"tu*ple</hw> <pr>(?)</pr>, }</mhw> <pos>a.</pos> <def>Containing one part of eight; having the ratio of one to eight.</def>  <rj><au>Bp. Wilkins.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Subocular</ent><br/
<hw>Sub*oc"u*lar</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[Pref. <ets>sub + ocular</ets>: cf. L. <ets>subocularis</ets>.]</ety> <fld>(Anat.)</fld> <def>Situated under, or on the ventral side of, the eye.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Subofficer</ent><br/
<hw>Sub*of"fi*cer</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Pref. <ets>sub + officer</ets>: cf. F. <ets>sous-officer</ets>.]</ety> <def>An under or subordinate officer.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Subopercular</ent><br/
<hw>Sub`o*per"cu*lar</hw> <pr>(?)</pr>, <pos>a.</pos> <fld>(Anat.)</fld> <def>Situated below the operculum; pertaining to the suboperculum.</def> -- <def2><pos>n.</pos> <def>The suboperculum.</def></def2><br/
[<source>1913 Webster</source>]</p>

<p><ent>Suboperculum</ent><br/
<hw>Sub`o*per"cu*lum</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[NL. See <er>Sub-</er>, <er>Operculum</er>.]</ety> <fld>(Anat.)</fld> <def>The lower opercular bone in fishes.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Suborbiculate</ent><br/
<ent>Suborbicular</ent><br/
<mhw>{ <hw>Sub`or*bic"u*lar</hw> <pr>(?)</pr>, <hw>Sub`or*bic"u*late</hw> <pr>(?)</pr>, }</mhw> <pos>a.</pos> <def>Almost orbiculate or orbicular.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Suborbitar</ent><br/
<ent>Suborbital</ent><br/
<mhw>{ <hw>Sub*or"bit*al</hw> <pr>(?)</pr>, <hw>Sub*or"bit*ar</hw> <pr>(?)</pr>, }</mhw> <pos>a.</pos> <fld>(Anat.)</fld> <def>Situated under or below the orbit.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Suborder</ent><br/
<hw>Sub*or"der</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Nat. Hist.)</fld> <def>A division of an order; a group of genera of a little lower rank than an order and of greater importance than a tribe or family; <as>as, cichoraceous plants form a <ex>suborder</ex> of Compositae</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Subordinacy</ent><br/
<hw>Sub*or"di*na*cy</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[See <er>Subordinate</er>.]</ety> <def>The quality or state of being subordinate, or subject to control; subordination, <as>as, to bring the imagination to act in <ex>subordinacy</ex> to reason</as>.</def>  <rj><au>Spectator.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Subordinancy</ent><br/
<ent>Subordinance</ent><br/
<mhw>{ <hw>Sub*or"di*nance</hw> <pr>(?)</pr>, <hw>Sub*or"di*nan*cy</hw> <pr>(?)</pr>, }</mhw> <pos>n.</pos> <ety>[Pref. <ets>sub</ets> + L. <ets>ordinans</ets>, p. pr. of <ets>ordinare</ets>. See <er>Subordinate</er>, <pos>a.</pos>]</ety> <def>Subordinacy; subordination.</def> <mark>[Obs.]</mark>  <rj><au>Dr. H. More. Sir W. Temple.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Subordinary</ent><br/
<hw>Sub*or"di*na*ry</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Her.)</fld> <def>One of several heraldic bearings somewhat less common than an ordinary. See <er>Ordinary</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ Different writers name different bearings as subordinaries, but the <xex>bar</xex>, <xex>bend</xex>, <xex>sinister</xex>, <xex>pile</xex>, <xex>inescutcheon bordure</xex>, <xex>gyron</xex>, and <xex>quarter</xex>, are always considered subordinaries by those who do not class them as ordinaries.</note><br/
[<source>1913 Webster</source>]</p>

<p><ent>Subordinate</ent><br/
<hw>Sub*or"di*nate</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[Pref. <ets>sub</ets> + L. <ets>ordinatus</ets>, p. p. of <ets>ordinare</ets> to set in order, to arrange. See <er>Ordain</er>.]</ety> <sn>1.</sn> <def>Placed in a lower order, class, or rank; holding a lower or inferior position.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The several kinds and <qex>subordinate</qex> species of each are easily distinguished.</q> <rj><qau>Woodward.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Inferior in order, nature, dignity, power, importance, or the like.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>It was <qex>subordinate</qex>, not enslaved, to the understanding.</q> <rj><qau>South.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Subordinate</ent><br/
<hw>Sub*or"di*nate</hw>, <pos>n.</pos> <def>One who stands in order or rank below another; -- distinguished from a <xex>principal</xex>.</def>  <rj><au>Milton.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Subordinate</ent><br/
<hw>Sub*or"di*nate</hw> <pr>(?)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Subordinated</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Subordinating</conjf>.]</vmorph> <sn>1.</sn> <def>To place in a lower order or class; to make or consider as of less value or importance; <as>as, to <ex>subordinate</ex> one creature to another</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To make subject; to subject or subdue; <as>as, to <ex>subordinate</ex> the passions to reason</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p>-- <wordforms><wf>Sub*or"di*nate*ly</wf>, <pos>adv.</pos> -- <wf>Sub*or"di*nate*ness</wf>, <pos>n.</pos></wordforms><br/
[<source>1913 Webster</source>]</p>

<p><ent>Subordination</ent><br/
<hw>Sub*or`di*na"tion</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Cf. F. <ets>subordination</ets>.]</ety> <sn>1.</sn> <def>The act of subordinating, placing in a lower order, or subjecting.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>The quality or state of being subordinate or inferior to an other; inferiority of rank or dignity; subjection.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Natural creature having a local <qex>subordination</qex>.</q> <rj><qau>Holyday.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>Place of inferior rank.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Persons who in their several <qex>subordinations</qex> would be obliged to follow the example of their superiors.</q> <rj><qau>Swift.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Subordinative</ent><br/
<hw>Sub*or"di*na*tive</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Tending to subordinate; expressing subordination; used to introduce a subordinate sentence; <as>as, a <ex>subordinative</ex> conjunction</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Suborn</ent><br/
<hw>Sub*orn"</hw> <pr>(?)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Suborned</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Suborning</conjf>.]</vmorph> <ety>[F. <ets>suborner</ets>, L. <ets>subornare</ets>; <ets>sub</ets> under, secretly + <ets>ornare</ets> to furnish, provide, equip, adorn. See <er>Ornament</er>.]</ety> <sn>1.</sn> <fld>(Law)</fld> <def>To procure or cause to take a false oath amounting to perjury, such oath being actually taken.</def>  <rj><au>Sir W. O. Russell.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To procure privately, or by collusion; to procure by indirect means; to incite secretly; to instigate.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Thou art <qex>suborned</qex> against his honor.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Those who by despair <qex>suborn</qex> their death.</q> <rj><qau>Dryden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Subornation</ent><br/
<hw>Sub`or*na"tion</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F. <ets>subornation</ets>.]</ety> <sn>1.</sn> <fld>(Law)</fld> <def>The act of suborning; the crime of procuring a person to take such a false oath as constitutes perjury.</def>  <rj><au>Blackstone.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>The sin or offense of procuring one to do a criminal or bad action, as by bribes or persuasion.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Foul <qex>subornation</qex> is predominant.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>The sort of chicanery attending the <qex>subornation</qex> of managers in the Leibnitz controversy.</q> <rj><qau>De Quinsey.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Suborner</ent><br/
<hw>Sub*orn"er</hw> <pr>(?)</pr>, <pos>n.</pos> <def>One who suborns or procures another to take, a false oath; one who procures another to do a bad action.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Suboval</ent><br/
<hw>Sub*o"val</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Somewhat oval; nearly oval.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Subovate</ent><br/
<hw>Sub*o"vate</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Nearly in the form of an egg, or of the section of an egg, but having the inferior extremity broadest; nearly ovate.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Subovated</ent><br/
<hw>Sub*o"va*ted</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Subovate.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Suboxide</ent><br/
<hw>Sub*ox"ide</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Chem.)</fld> <def>An oxide containing a relatively small amount of oxygen, and less than the normal proportion; <as>as, potassium <ex>suboxide</ex>, <chform>K4O</chform></as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Subpeduncular</ent><br/
<hw>Sub`pe*dun"cu*lar</hw> <pr>(?)</pr>, <pos>a.</pos> <fld>(Anat.)</fld> <def>Situated beneath the peduncle; <as>as, the <ex>subpeduncular</ex> lobe of the cerebellum</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Subpedunculate</ent><br/
<hw>Sub`pe*dun"cu*late</hw> <pr>(?)</pr>, <pos>a.</pos> <fld>(Bot. & Zool.)</fld> <def>Supported on, or growing from, a very short stem; having a short peduncle.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Subpellucid</ent><br/
<hw>Sub`pel*lu"cid</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Somewhat pellucid; nearly pellucid.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Subpena</ent><br/
<hw>Sub*pe"na</hw> <pr>(?)</pr>, <pos>n. & v. t.</pos> <def>See <er>Subpoena</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Subpentangular</ent><br/
<hw>Sub`pen*tan"gu*lar</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Nearly or approximately pentangular; almost pentangular.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Subpericardial</ent><br/
<hw>Sub*per`i*car"di*al</hw> <pr>(?)</pr>, <pos>a.</pos> <fld>(Anat.)</fld> <def>Situated under the cardiac pericardium.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Subperiosteal</ent><br/
<hw>Sub*per`i*os"te*al</hw> <pr>(?)</pr>, <pos>a.</pos> <fld>(Anat.)</fld> <def>Situated under the periosteum.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Subperiosteal operation</b></col> <fld>(Surg.)</fld>, <cd>a removal of bone effected without taking away the periosteum.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Subperitoneal</ent><br/
<hw>Sub*per`i*to"ne*al</hw> <pr>(?)</pr>, <pos>a.</pos> <fld>(Anat.)</fld> <def>Situated under the peritoneal membrane.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Subpetiolar</ent><br/
<hw>Sub*pet"i*o*lar</hw> <pr>(?)</pr>, <pos>a.</pos> <fld>(Bot.)</fld> <def>Concealed within the base of the petiole, as the leaf buds of the plane tree.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Subpleural</ent><br/
<hw>Sub*pleu"ral</hw> <pr>(?)</pr>, <pos>a.</pos> <fld>(Anat.)</fld> <def>Situated under the pleural membrane.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Subpodophyllous</ent><br/
<hw>Sub*pod`o*phyl"lous</hw> <pr>(?)</pr>, <pos>a.</pos> <fld>(Anat.)</fld> <def>Situated under the podophyllous tissue of the horse's foot.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Subpoena</ent><br/
<hw>Sub*poe"na</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[NL., fr. L. <ets>sub</ets> under + <ets>poena</ets> punishment. See <er>Pain</er>.]</ety> <fld>(Law)</fld> <def>A writ commanding the attendance in court, as a witness, of the person on whom it is served, under a penalty; the process by which a defendant in equity is commanded to appear and answer the plaintiff's bill.</def> <altsp>[Written also <asp>subpena</asp>.]</altsp><br/
[<source>1913 Webster</source>]</p>

<p><cs>||<col><b>Subpoena ad testificandum</b></col> <pr>(<?/)</pr>. <ety>[NL.]</ety> <cd>A writ used to procure the attendance of a witness for the purpose of testifying.</cd> -- ||<col><b>Subpoena duces tecum</b></col> <pr>(<?/)</pr>. <ety>[NL.]</ety> <cd>A writ which requires a witness to attend and bring certain documents.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Subpoena</ent><br/
<hw>Sub*poe"na</hw>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Subpoenaed</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Subpoenaing</conjf>.]</vmorph> <fld>(Law)</fld> <def>To serve with a writ of subpoena; to command attendance in court by a legal writ, under a penalty in case of disobedience.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Subpoenal</ent><br/
<hw>Sub*poe"nal</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Required or done under penalty.</def>  <rj><au>Gauden.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Subpolar</ent><br/
<hw>Sub*po"lar</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Situated below the poles.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Subpolygonal</ent><br/
<hw>Sub`po*lyg"o*nal</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Approximately polygonal; somewhat or almost polygonal.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Subprehensile</ent><br/
<hw>Sub`pre*hen"sile</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Somewhat prehensile; prehensile in an inferior degree.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Subprior</ent><br/
<hw>Sub*pri"or</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Pref. <ets>sub + prior</ets>: cf. F. <ets>sous-prieur</ets>.]</ety> <fld>(Eccl.)</fld> <def>The vicegerent of a prior; a claustral officer who assists the prior.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Subpubic</ent><br/
<hw>Sub*pu"bic</hw> <pr>(?)</pr>, <pos>a.</pos> <fld>(Anat.)</fld> <def>Situated under, or posterior to, the pubic bones.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Subpulmonary</ent><br/
<hw>Sub*pul"mo*na*ry</hw> <pr>(?)</pr>, <pos>a.</pos> <fld>(Anat.)</fld> <def>Situated under, or on the ventral side of, the lungs.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Subpurchaser</ent><br/
<hw>Sub*pur"chas*er</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A purchaser who buys from a purchaser; one who buys at second hand.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Subpyriform</ent><br/
<hw>Sub*pyr"i*form</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Somewhat pyriform.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Subquadrate</ent><br/
<hw>Sub*quad"rate</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Nearly or approximately square; almost square.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Subquadruple</ent><br/
<hw>Sub*quad"ru*ple</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Containing one part of four; in the ratio of one to four; <as>as, <ex>subquadruple</ex> proportion</as>.</def>  <rj><au>Bp. Wilkins.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Subquinquefid</ent><br/
<hw>Sub*quin"que*fid</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Almost quinquefid; nearly quinquefid.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Subquintuple</ent><br/
<hw>Sub*quin"tu*ple</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Having the ratio of one to five; <as>as, <ex>subquintuple</ex> proportion</as>.</def>  <rj><au>Bp. Wilkins.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Subreader</ent><br/
<hw>Sub*read"er</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Law)</fld> <def>An under reader in the inns of court, who reads the texts of law the reader is to discourse upon.</def> <mark>[Eng.]</mark>  <rj><au>Crabb.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Subrector</ent><br/
<hw>Sub*rec"tor</hw> <pr>(?)</pr>, <pos>n.</pos> <def>An assistant restor.</def> <mark>[Eng.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Subreligion</ent><br/
<hw>Sub`re*li"gion</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A secondary religion; a belief or principle held in a quasi religious veneration.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Loyalty is in the English a <qex>subreligion</qex>.</q> <rj><qau>Emerson.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Subreption</ent><br/
<hw>Sub*rep"tion</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>subreptio</ets>, fr. <ets>subripere</ets>, <ets>subreptum</ets>, to snatch or take away secretly: cf. F. <ets>subreption</ets>. See <er>Surreptitious</er>.]</ety> <def>The act of obtaining a favor by surprise, or by unfair representation through suppression or fraudulent concealment of facts.</def>  <rj><au>Bp. Hall.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Subreptitious</ent><br/
<hw>Sub`rep*ti"tious</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>subreptitius</ets>. See <er>Surreptitious</er>.]</ety> <def>Surreptitious.</def> <mark>[Obs.]</mark> -- <wordforms><wf>Sub`rep*ti"tious*ly</wf> <pr>(#)</pr>, <pos>adv.</pos> <mark>[Obs.]</mark></wordforms><br/
[<source>1913 Webster</source>]</p>

<p><-- p. 1436 --></p>

<p><ent>Subreptive</ent><br/
<hw>Sub*rep"tive</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>subreptivus</ets>.]</ety> <def>Surreptitious.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Subrigid</ent><br/
<hw>Sub*rig"id</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Somewhat rigid or stiff.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Subriguous</ent><br/
<hw>Sub*rig"u*ous</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>subriguus</ets>; <ets>sub</ets> under + <ets>riguus</ets> watered, akin to <ets>rigare</ets> to water.]</ety> <def>Watered or wet beneath; well-watered.</def> <mark>[Obs.]</mark>  <rj><au>Blount.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Subrogate</ent><br/
<hw>Sub"ro*gate</hw> <pr>(?)</pr>, <pos>v. t.</pos> <ety>[L. <ets>subrogatus</ets>, p. p. of <ets>subrogare</ets>. See <er>Surrogate</er>.]</ety> <def>To put in the place of another; to substitute.</def>  <rj><au>Barrow.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Subrogation</ent><br/
<hw>Sub`ro*ga"tion</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Cf. F. <ets>subrogation</ets>, LL. <ets>subrogatio</ets>.]</ety> <def>The act of subrogating.</def> Specifically: <fld>(Law)</fld> <def>The substitution of one person in the place of another as a creditor, the new creditor succeeding to the rights of the former; the mode by which a third person who pays a creditor succeeds to his rights against the debtor.</def>  <rj><au>Bouvier. Burrill. Abbott.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Subrotund</ent><br/
<hw>Sub`ro*tund"</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Somewhat rotund.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Subsacral</ent><br/
<hw>Sub*sa"cral</hw> <pr>(?)</pr>, <pos>a.</pos> <fld>(Anat.)</fld> <def>Situated under, or on the ventral side of, the sacrum.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Subsaline</ent><br/
<hw>Sub`sa*line"</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Moderately saline or salt.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Subsalt</ent><br/
<hw>Sub"salt`</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Chem.)</fld> <def>A basic salt. See the Note under <er>Salt</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Subsannation</ent><br/
<hw>Sub`san*na"tion</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>subsannatio</ets>, fr. <ets>subsannare</ets> to deride by mimicking gestures.]</ety> <def>Derision; mockery.</def> <mark>[Obs.]</mark>  <rj><au>Dr. H. More.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Subscapulary</ent><br/
<ent>Subscapular</ent><br/
<mhw>{ <hw>Sub*scap"u*lar</hw> <pr>(?)</pr>, <hw>Sub*scap"u*la*ry</hw> <pr>(?)</pr>, }</mhw> <pos>a.</pos> <fld>(Anat.)</fld> <def>Situated beneath the scapula; infrascapular; <as>as, the <ex>subscapular</ex> muscle</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Subscribable</ent><br/
<hw>Sub*scrib"a*ble</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Capable of being subscribed.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Subscribe</ent><br/
<hw>Sub*scribe"</hw> <pr>(?)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Subscribed</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Subscribing</conjf>.]</vmorph> <ety>[L. <ets>subscribere</ets>, <ets>subscriptum</ets>; <ets>sub</ets> under + <ets>scribere</ets> to write: cf. F. <ets>souscrire</ets>. See <er>Scribe</er>.]</ety> <sn>1.</sn> <def>To write underneath, as one's name; to sign (one's name) to a document.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>[They] <qex>subscribed</qex> their names under them.</q> <rj><qau>Sir T. More.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To sign with one's own hand; to give consent to, as something written, or to bind one's self to the terms of, by writing one's name beneath; <as>as, parties <ex>subscribe</ex> a covenant or contract; a man <ex>subscribes</ex> a bond</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>All the bishops <qex>subscribed</qex> the sentence.</q> <rj><qau>Milman.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>To attest by writing one's name beneath; <as>as, officers <ex>subscribe</ex> their official acts, and secretaries and clerks <ex>subscribe</ex> copies or records</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>To promise to give, by writing one's name with the amount; <as>as, each man <ex>subscribed</ex> ten dollars</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <def>To sign away; to yield; to surrender.</def> <mark>[Obs.]</mark>  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>6.</sn> <def>To declare over one's signature; to publish.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>Either or must shortly hear from him, or I will <qex>subscribe</qex> him a coward.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Subscribe</ent><br/
<hw>Sub*scribe"</hw>, <pos>v. i.</pos> <sn>1.</sn> <def>To sign one's name to a letter or other document.</def>  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To give consent to something written, by signing one's name; hence, to assent; to agree.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>So spake, so wished, much humbled Eve; but Fate<br/
<qex>Subscribed</qex> not.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>To become surely; -- with <xex>for</xex>.</def> <mark>[R.]</mark>  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>To yield; to admit one's self to be inferior or in the wrong.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>I will <qex>subscribe</qex>, and say I wronged the duke.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <def>To set one's name to a paper in token of promise to give a certain sum.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>6.</sn> <def>To enter one's name for a newspaper, a book, etc.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Subscriber</ent><br/
<hw>Sub*scrib"er</hw> <pr>(?)</pr>, <pos>n.</pos> <sn>1.</sn> <def>One who subscribes; one who contributes to an undertaking by subscribing.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>One who enters his name for a paper, book, map, or the like.</def>  <rj><au>Dryden.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Subscript</ent><br/
<hw>Sub"script</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>subscriptus</ets>, p. p. See <er>Subscribe</er>.]</ety> <def>Written below or underneath; <as>as, iota <ex>subscript</ex></as>. (See under <er>Iota</er>.) Specifically <fld>(Math.)</fld>, said of marks, figures, or letters (suffixes), written below and usually to the right of other letters to distinguish them; <as>as, <it>a</it>, <it>n</it>, 2, in the symbols <ex>X<subs>a</subs></ex>, <ex>A<subs>n</subs></ex>, <ex>Y<subs>2</subs></ex></as>. See <er>Suffix</er>, <pos>n.</pos>, 2, and <er>Subindex</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Subscript</ent><br/
<hw>Sub"script</hw>, <pos>n.</pos> <def>Anything written below.</def>  <rj><au>Bentley.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Subscription</ent><br/
<hw>Sub*scrip"tion</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>subscriptio</ets>: cf. F. <ets>souscription</ets>.]</ety> <sn>1.</sn> <def>The act of subscribing.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>That which is subscribed.</def> Specifically: <sd>(a)</sd> <def>A paper to which a signature is attached.</def> <sd>(b)</sd> <def>The signature attached to a paper.</def> <sd>(c)</sd> <def>Consent or attestation by underwriting the name.</def> <sd>(d)</sd> <def>Sum subscribed; amount of sums subscribed; <as>as, an individual <ex>subscription</ex> to a fund</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <fld>(Eccl.)</fld> <def>The acceptance of articles, or other tests tending to promote uniformity; esp. <fld>(Ch. of Eng.)</fld>, formal assent to the Thirty-nine Articles and the Book of Common Prayer, required before ordination.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>Submission; obedience.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>You owe me no <qex>subscription</qex>.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <fld>(Pharm.)</fld> <def>That part of a prescription which contains the direction to the apothecary.</def><br/
[<source>1913 Webster</source>]</p>

<p><-- 6. <def>A method of purchasing items produced periodically in a series, as newspapers or magazines, in which a certain number of the items are delivered as produced, without need for ordering each item individually; also, the purchase thus executed.</def> <br/
[<source>1913 Webster</source>]</p>

<p>   <note>The right to attend a series of public performances of ballet, opera, or music are also often sold by subscription.  The payment for a subscription may be made prior to delivery of any items (common with magazines and performances), or after a certain number of the items have been delivered (common with newspapers or works of art produced in a series).</note><br/
[<source>1913 Webster</source>]</p>

<p>   7. An application to purchase a certain number of securities to be delivered when they are newly issued. --><br/
[<source>1913 Webster</source>]</p>

<p><ent>Subscriptive</ent><br/
<hw>Sub*scrip"tive</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Of or pertaining to a subscription, or signature.</def> <ldquo/The <xex>subscriptive</xex> part.<rdquo/ <au>Richardson.</au> -- <wordforms><wf>Sub*scrip"tive*ly</wf>, <pos>adv.</pos></wordforms><br/
[<source>1913 Webster</source>]</p>

<p><ent>Subsecute</ent><br/
<hw>Sub"se*cute</hw> <pr>(?)</pr>, <pos>v. t.</pos> <ety>[L. <ets>subsecutus</ets>, p. p. of <ets>subsequi</ets>. See <er>Subsequent</er>.]</ety> <def>To follow closely, or so as to overtake; to pursue.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>To follow and detain him, if by any possibility he could be <qex>subsecuted</qex> and overtaken.</q> <rj><qau>E. Hall.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Subsecutive</ent><br/
<hw>Sub*sec"u*tive</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[Cf. F. <ets>subs<eacute/cutif</ets>.]</ety> <def>Following in a train or succession.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Subsellium</ent><br/
||<hw>Sub*sel"li*um</hw> <pr>(?)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Subsellia</plw> <pr>(#)</pr>.</plu> <ety>[L.]</ety> <fld>(Eccl. Arch.)</fld> <def>One of the stalls of the lower range where there are two ranges. See <xex>Illust.</xex> of <er>Stall</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Subsemitone</ent><br/
<hw>Sub*sem"i*tone</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Mus.)</fld> <def>The sensible or leading note, or sharp seventh, of any key; subtonic.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Subsensible</ent><br/
<hw>Sub*sen"si*ble</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Deeper than the reach of the senses.</def> <ldquo/That <xex>subsensible</xex> world.<rdquo/  <rj><au>Tyndall.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Subseptuple</ent><br/
<hw>Sub*sep"tu*ple</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Having the ratio of one to seven.</def>  <rj><au>Bp. Wilkins.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Subsequency</ent><br/
<ent>Subsequence</ent><br/
<mhw>{ <hw>Sub"se*quence</hw> <pr>(?)</pr>, <hw>Sub"se*quen*cy</hw> <pr>(?)</pr>, }</mhw> <pos>n.</pos> <def>The act or state of following; -- opposed to <xex>precedence</xex>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Subsequent</ent><br/
<hw>Sub"se*quent</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>subsequens</ets>, <ets>-entis</ets>, p. pr. of <ets>subsequi</ets> to follow, succeed: cf. F. <ets>subs<eacute/quent</ets>. See <er>Sue</er> to follow.]</ety> <sn>1.</sn> <def>Following in time; coming or being after something else at any time, indefinitely; <as>as, <ex>subsequent</ex> events; <ex>subsequent</ex> ages or years; a period long <ex>subsequent</ex> to the foundation of Rome</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Following in order of place; succeeding; <as>as, a <ex>subsequent</ex> clause in a treaty</as>.</def> <ldquo/The <xex>subsequent</xex> words come on before the precedent vanish.<rdquo/  <rj><au>Bacon.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Subsequently</ent><br/
<hw>Sub"se*quent*ly</hw>, <pos>adv.</pos> <def>At a later time; afterwards.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Subserous</ent><br/
<hw>Sub*se"rous</hw> <pr>(?)</pr>, <pos>a.</pos> <fld>(Anat.)</fld> <def>Situated under a serous membrane.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Subserve</ent><br/
<hw>Sub*serve"</hw> <pr>(?)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Subserved</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Subserving</conjf>.]</vmorph> <ety>[L. <ets>subservire</ets>; <ets>sub</ets> under + <ets>servire</ets> to serve. See <er>Serve</er>.]</ety> <def>To serve in subordination or instrumentally; to be subservient to; to help forward; to promote.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>It is a great credit to know the ways of captivating Nature, and making her <qex>subserve</qex> our purposes, than to have learned all the intrigues of policy.</q> <rj><qau>Glanvill.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Subserve</ent><br/
<hw>Sub*serve"</hw>, <pos>v. i.</pos> <def>To be subservient or subordinate; to serve in an inferior capacity.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Not made to rule,<br/
But to <qex>subserve</qex> where wisdom bears command.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Subserviency</ent><br/
<ent>Subservience</ent><br/
<mhw>{ <hw>Sub*serv"i*ence</hw> <pr>(?)</pr>, <hw>Sub*serv"i*en*cy</hw> <pr>(?)</pr>, }</mhw> <pos>n.</pos> <def>The quality or state of being subservient; instrumental fitness or use; hence, willingness to serve another's purposes; in a derogatory sense, servility.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The body wherein appears much fitness, use, and <qex>subserviency</qex> to infinite functions.</q> <rj><qau>Bentley.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>There is a regular subordination and <qex>subserviency</qex> among all the parts to beneficial ends.</q> <rj><qau>Cheyne.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Subservient</ent><br/
<hw>Sub*serv"i*ent</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>subserviens</ets>, <ets>-entis</ets>, p. pr. See <er>Subserve</er>.]</ety> <def>Fitted or disposed to subserve; useful in an inferior capacity; serving to promote some end; subordinate; hence, servile, truckling.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Scarce ever reading anything which he did not make <qex>subservient</qex> in one kind or other.</q> <rj><qau>Bp. Fell.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>These ranks of creatures are <qex>subservient</qex> one to another.</q> <rj><qau>Ray.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Their temporal ambition was wholly <qex>subservient</qex> to their proselytizing spirit.</q> <rj><qau>Burke.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Subserviently</ent><br/
<hw>Sub*serv"i*ent*ly</hw>, <pos>adv.</pos> <def>In a subservient manner.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Subsesqui-</ent><br/
<hw>Sub*ses"qui-</hw> <pr>(?)</pr>. <ety>[Pref. <ets>sub-</ets> + <ets>sesqui-</ets>.]</ety> <fld>(Chem.)</fld> <def>A prefix (also used adjectively) denoting the combination of constituents (especially electro-negative and electro-positive bodies) in the proportion of <xex>two</xex> to <xex>three</xex>; <as>as, a <ex>subsesqui</ex> acetate, <it>i. e.</it>, a salt having two equivalents of acetic acid to three of the base.</as></def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Subsextuple</ent><br/
<hw>Sub*sex"tu*ple</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Having the ratio of one to six; <as>as, a <ex>sub</ex>sextuple proportion</as>.</def>  <rj><au>Bp. Wilkins.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Subside</ent><br/
<hw>Sub*side"</hw> <pr>(?)</pr>, <pos>v. i.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Subsided</conjf>; <pos>p. pr. & vb. n.</pos> <conjf>Subsiding</conjf>.]</vmorph> <ety>[L. <ets>subsidere</ets>; <ets>sub</ets> under, below + <ets>sidere</ets> to sit down, to settle; akin to <ets>sedere</ets> to sit, E. <ets>sit</ets>. See <er>Sit</er>.]</ety> <sn>1.</sn> <def>To sink or fall to the bottom; to settle, as lees.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To tend downward; to become lower; to descend; to sink.</def> <ldquo/Heaven's <xex>subsiding</xex> hill.<rdquo/  <rj><au>Dryden.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>To fall into a state of quiet; to cease to rage; to be calmed; to settle down; to become tranquil; to abate; <as>as, the sea <ex>subsides</ex>; the tumults of war will <ex>subside</ex>; the fever has <ex>subsided</ex></as>.</def> <ldquo/In cases of danger, pride and envy naturally <xex>subside</xex>.<rdquo/  <rj><au>C. Middleton.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><syn><b>Syn.</b> -- See <er>Abate</er>.</syn><br/
[<source>1913 Webster</source>]</p>

<p><ent>Subsidency</ent><br/
<ent>Subsidence</ent><br/
<mhw>{ <hw>Sub*sid"ence</hw> <pr>(?)</pr>, <hw>Sub*sid"en*cy</hw> <pr>(?)</pr>, }</mhw> <pos>n.</pos> <ety>[L. <ets>subsidens</ets>, <ets>-entis</ets>, p. pr. of <ets>subsidere</ets>. See <er>Subside</er>.]</ety> <def>The act or process of subsiding.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The subdual or <qex>subsidence</qex> of the more violent passions.</q> <rj><qau>Bp. Warburton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Subsidiarily</ent><br/
<hw>Sub*sid"i*a*ri*ly</hw> <pr>(?)</pr>, <pos>adv.</pos> <def>In a subsidiary manner; so as to assist.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Subsidiary</ent><br/
<hw>Sub*sid"i*a*ry</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>subsidiarius</ets>: cf. F. <ets>subsidiaire</ets>. See <er>Subsidy</er>.]</ety> <sn>1.</sn> <def>Furnishing aid; assisting; auxiliary; helping; tributary; especially, aiding in an inferior position or capacity; <as>as, a <ex>subsidiary</ex> stream</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Chief ruler and principal head everywhere, not suffragant and <qex>subsidiary</qex>.</q> <rj><qau>Florio.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>They constituted a useful <qex>subsidiary</qex> testimony of another state of existence.</q> <rj><qau>Coleridge.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Of or pertaining to a subsidy; constituting a subsidy; being a part of, or of the nature of, a subsidy; <as>as, <ex>subsidiary</ex> payments to an ally</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>George the Second relied on his <qex>subsidiary</qex> treaties.</q> <rj><qau>Ld. Mahon.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Subsidiary</ent><br/
<hw>Sub*sid"i*a*ry</hw>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Subsidiaries</plw> <pr>(<?/)</pr>.</plu> <def>One who, or that which, contributes aid or additional supplies; an assistant; an auxiliary.</def>  <rj><au>Hammond.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Subsidize</ent><br/
<hw>Sub"si*dize</hw> <pr>(?)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Subsidized</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Subsidizing</conjf> <pr>(?)</pr>.]</vmorph> <ety>[From <er>Subsidy</er>.]</ety> <def>To furnish with a subsidy; to purchase the assistance of by the payment of a subsidy; to aid or promote, as a private enterprise, with public money; <as>as, to <ex>subsidize</ex> a steamship line</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>He employed the remittances from Spain to <qex>subsidize</qex> a large body of German mercenaries.</q> <rj><qau>Prescott.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Subsidy</ent><br/
<hw>Sub"si*dy</hw> <pr>(?)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Subsidies</plw> <pr>(#)</pr>.</plu> <ety>[L. <ets>subsidium</ets> the troops stationed in reserve in the third line of battlem reserve, support, help, fr. <ets>subsidere</ets> to sit down, lie in wait: cf. F. <ets>subside</ets>. See <er>Subside</er>.]</ety> <sn>1.</sn> <def>Support; aid; cooperation; esp., extraordinary aid in money rendered to the sovereign or to a friendly power.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>They advised the king to send speedy aids, and with much alacrity granted a great rate of <qex>subsidy</qex>.</q> <rj><qau>Bacon.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ <xex>Subsidies</xex> were taxes, not immediately on on property, but on persons in respect of their reputed estates, after the nominal rate of 4s. the pound for lands, and 2s. 8d. for goods.</note>  <rj><au>Blackstone.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Specifically: A sum of money paid by one sovereign or nation to another to purchase the cooperation or the neutrality of such sovereign or nation in war.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>A grant from the government, from a municipal corporation, or the like, to a private person or company to assist the establishment or support of an enterprise deemed advantageous to the public; a subvention; <as>as, a <ex>subsidy</ex> to the owners of a line of ocean steamships</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><syn><b>Syn.</b> -- Tribute; grant.</syn>  -- <usage><er>Subsidy</er>, <er>Tribute</er>. A <xex>subsidy</xex> is voluntary; a <xex>tribute</xex> is exacted.</usage><br/
[<source>1913 Webster</source>]</p>

<p><ent>Subsign</ent><br/
<hw>Sub*sign"</hw> <pr>(?)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Subsigned</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Subsigning</conjf>.]</vmorph> <ety>[L. <ets>subsignare</ets>; <ets>sub</ets> under + <ets>signare</ets> to mark: cf. F. <ets>soussigner</ets>. See <er>Sign</er>.]</ety> <def>To sign beneath; to subscribe.</def> <mark>[R.]</mark>  <rj><au>Camden.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Subsignation</ent><br/
<hw>Sub`sig*na"tion</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>subsignatio</ets>.]</ety> <def>The act of writing the name under something, as for attestation.</def> <mark>[R.]</mark>  <rj><au>Shelton.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Subsilicate</ent><br/
<hw>Sub*sil"i*cate</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A basic silicate.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Subsist</ent><br/
<hw>Sub*sist"</hw> <pr>(?)</pr>, <pos>v. i.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Subsisted</conjf>; <pos>p. pr. & vb. n.</pos> <conjf>Subsisting</conjf>.]</vmorph> <ety>[L. <ets>subsistere</ets> to stand still, stay, remain alive; <ets>sub</ets> under + <ets>sistere</ets> to stand, to cause to stand, from <ets>stare</ets> to stand: cf. F. <ets>subsister</ets>. See <er>Stand</er>.]</ety> <sn>1.</sn> <def>To be; to have existence; to inhere.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>And makes what happiness we justly call,<br/
<qex>Subsist</qex> not in the good of one, but all.</q> <rj><qau>Pope.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To continue; to retain a certain state.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Firm we <qex>subsist</qex>, yet possible to swerve.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>To be maintained with food and clothing; to be supported; to live.</def>  <rj><au>Milton.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>To <qex>subsist</qex> on other men's charity.</q> <rj><qau>Atterbury.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Subsist</ent><br/
<hw>Sub*sist"</hw>, <pos>v. t.</pos> <def>To support with provisions; to feed; to maintain; <as>as, to <ex>subsist</ex> one's family</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>He laid waste the adjacent country in order to render it more difficult for the enemy to <qex>subsist</qex> their army.</q> <rj><qau>Robertson.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Subsistence</ent><br/
<hw>Sub*sist"ence</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Cf. F. <ets>subsistance</ets>, L. <ets>subsistentia</ets>.]</ety> <sn>1.</sn> <def>Real being; existence.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Not only the things had <qex>subsistence</qex>, but the very images were of some creatures existing.</q> <rj><qau>Stillingfleet.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Inherency; <as>as, the <ex>subsistence</ex> of qualities in bodies</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>That which furnishes support to animal life; means of support; provisions, or that which produces provisions; livelihood; <as>as, a meager <ex>subsistence</ex></as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>His viceroy could only propose to himself a comfortable <qex>subsistence</qex> out of the plunder of his province.</q> <rj><qau>Addison.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <fld>(Theol.)</fld> <def>Same as <er>Hypostasis</er>, 2.</def>  <rj><au>Hooker.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Subsistence Department</ent><br/
<hw>Sub*sist"ence De*part"ment</hw>. <fld>(Mil.)</fld> <def>A staff department of the United States army charged, under the  supervision of the Chief of Staff, with the purchasing and issuing to the army of such supplies as make up the  ration. It also supplies, for authorized sales, certain articles of food and other minor stores. It is commanded by any officer of the rank of brigadier general, called commissary general, and the department is popularly called the <altname>Commissary Department</altname>.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Subsistency</ent><br/
<hw>Sub*sist"en*cy</hw> <pr>(?)</pr>, <pos>n.</pos> <def>Subsistence.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Subsistent</ent><br/
<hw>Sub*sist"ent</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>subsistens</ets>, p. pr. See <er>Subsist</er>.]</ety> <sn>1.</sn> <def>Having real being; <as>as, a <ex>subsistent</ex> spirit</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Inherent; <as>as, qualities <ex>subsistent</ex> in matter</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Subsizar</ent><br/
<hw>Sub*si"zar</hw> <pr>(?)</pr>, <pos>n.</pos> <def>An under sizar; a student of lower rank than a sizar.</def> <mark>[Cambridge Univ. Eng.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>Bid my <qex>subsizar</qex> carry my hackney to the buttery and give him his bever.</q> <rj><qau>J. Fletcher.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Subsoil</ent><br/
<hw>Sub"soil`</hw> <pr>(?)</pr>, <pos>n.</pos> <def>The bed, or stratum, of earth which lies immediately beneath the surface soil.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Subsoil plow</b></col>, <cd>a plow having a share and standard but no moldboard. It follows in the furrow made by an ordinary plow, and loosens the soil to an additional depth without bringing it to the surface.</cd> <au>Knight.</au></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Subsoil</ent><br/
<hw>Sub"soil`</hw>, <pos>v. t.</pos> <def>To turn up the subsoil of.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Subsolary</ent><br/
<hw>Sub*so"la*ry</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Being under the sun; hence, terrestrial; earthly; mundane.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Subspecies</ent><br/
<hw>Sub*spe"cies</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A group somewhat lessdistinct than speciesusually are, but based on characters more important than those which characterize ordinary varieties; often, a geographical variety or race.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Subsphenoidal</ent><br/
<hw>Sub`sphe*noid"al</hw> <pr>(?)</pr>, <pos>a.</pos> <fld>(Anat.)</fld> <def>Situated under, or on the ventral side of, the body of the sphenoid bone.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Subspherical</ent><br/
<hw>Sub*spher"ic*al</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Nearly spherical; having a figure resembling that of a sphere.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Subspinous</ent><br/
<hw>Sub*spi"nous</hw> <pr>(?)</pr>, <pos>a.</pos> <sd>(a)</sd> <fld>(Anat.)</fld> <def>Subvertebral.</def> <sd>(b)</sd> <fld>(Med.)</fld> <def>Situated beneath a spinous process, as that of the scapula; <as>as, <ex>subspinous</ex> dislocation of the humerus</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Substance</ent><br/
<hw>Sub"stance</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F., fr. L. <ets>substantia</ets>, fr. <ets>substare</ets> to be under or present, to stand firm; <ets>sub</ets> under + <ets>stare</ets> to stand. See <er>Stand</er>.]</ety> <sn>1.</sn> <def>That which underlies all outward manifestations; substratum; the permanent subject or cause of phenomena, whether material or spiritual; that in which properties inhere; that which is real, in distinction from that which is apparent; the abiding part of any existence, in distinction from any accident; that which constitutes anything what it is; real or existing essence.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>These cooks, how they stamp, and strain, and grind,<br/
And turn <qex>substance</qex> into accident!</q> <rj><qau>Chaucer.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Heroic virtue did his actions guide,<br/
And he the <qex>substance</qex>, not the appearance, chose.</q> <rj><qau>Dryden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>The most important element in any existence; the characteristic and essential components of anything; the main part; essential import; purport.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>This edition is the same in <qex>substance</qex> with the Latin.</q> <rj><qau>Bp. Burnet.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>It is insolent in words, in manner; but in <qex>substance</qex> it is not only insulting, but alarming.</q> <rj><qau>Burke.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>Body; matter; material of which a thing is made; hence, substantiality; solidity; firmness; <as>as, the <ex>substance</ex> of which a garment is made; some textile fabrics have little <ex>substance</ex>.</as></def><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>Material possessions; estate; property; resources.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>And there wasted his <qex>substance</qex> with riotous living.</q> <rj><qau>Luke xv. 13.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Thy <qex>substance</qex>, valued at the highest rate,<br/
Can not amount unto a hundred marks.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>We are destroying many thousand lives, and exhausting our <qex>substance</qex>, but not for our own interest.</q> <rj><qau>Swift.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <fld>(Theol.)</fld> <def>Same as <er>Hypostasis</er>, 2.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Substance</ent><br/
<hw>Sub"stance</hw>, <pos>v. t.</pos> <def>To furnish or endow with substance; to supply property to; to make rich.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Substanceless</ent><br/
<hw>Sub"stance*less</hw>, <pos>a.</pos> <def>Having no substance; unsubstantial.</def> <mark>[R.]</mark>  <rj><au>Coleridge.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Substant</ent><br/
<hw>Sub"stant</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>substans</ets>, <ets>-antis</ets>, p. pr. of <ets>substare</ets> to be firm.]</ety> <def>Substantial; firm.</def> <mark>[R.]</mark> <ldquo/[The glacier's] <xex>substant</xex> ice.<rdquo/  <rj><au>The Century.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Substantial</ent><br/
<hw>Sub*stan"tial</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[F. <ets>substantiel</ets>, L. <ets>substantialis</ets>.]</ety> <sn>1.</sn> <def>Belonging to substance; actually existing; real; <as>as, <ex>substantial</ex> life</as>.</def>  <rj><au>Milton.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>If this atheist would have his chance to be real and <qex>substantial</qex> agent, he is more stupid than the vulgar.</q> <rj><qau>Bentley.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Not seeming or imaginary; not illusive; real; solid; true; veritable.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>If happinessbe a <qex>substantial</qex> good.</q> <rj><qau>Denham.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>The <qex>substantial</qex> ornaments of virtue.</q> <rj><qau>L'Estrange.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>Corporeal; material; firm.</def> <ldquo/Most ponderous and <xex>substantial</xex> things.<rdquo/  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>The rainbow [appears to be] a large <qex>substantial</qex> arch.</q> <rj><qau>I. Watts.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>Having good substance; strong; stout; solid; firm; <as>as, <ex>substantial</ex> cloth; a <ex>substantial</ex> fence or wall</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <def>Possessed of goods or an estate; moderately wealthy; responsible; <as>as, a <ex>substantial</ex> freeholder</as>.</def> <ldquo/<xex>Substantial</xex> yeomen and burghers.<rdquo/  <rj><au>Sir W. Scott.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><-- p. 1437 --></p>

<p><ent>Substantiality</ent><br/
<hw>Sub*stan`ti*al"i*ty</hw> <pr>(?)</pr>, <pos>n.</pos> <def>The quality or state of being substantial; corporiety; materiality.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The soul is a stranger to such gross <qex>substantiality</qex>.</q> <rj><qau>Glanvill.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Substantialize</ent><br/
<hw>Sub*stan"tial*ize</hw> <pr>(?)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Substantialized</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Substantializing</conjf> <pr>(?)</pr>.]</vmorph> <def>To make substantial.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Substantially</ent><br/
<hw>Sub*stan"tial*ly</hw>, <pos>adv.</pos> <def>In a substantial manner; in substance; essentially.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>In him all his Father shone,<br/
<qex>Substantially</qex> expressed.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>The laws of this religion would make men, if they would truly observe them, <qex>substantially</qex> religious toward God, chastle, and temperate.</q> <rj><qau>Tillotson.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Substantialness</ent><br/
<hw>Sub*stan"tial*ness</hw>, <pos>n.</pos> <def>The quality or state of being substantial; <as>as, the <ex>substantialness</ex> of a wall or column</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Substantials</ent><br/
<hw>Sub*stan"tials</hw> <pr>(?)</pr>, <pos>n. pl.</pos> <def>Essential parts.</def>  <rj><au>Ayliffe.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Substantiate</ent><br/
<hw>Sub*stan"ti*ate</hw> <pr>(?)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Substantiated</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Substantiating</conjf>.]</vmorph> <sn>1.</sn> <def>To make to exist; to make real.</def>  <rj><au>Ayliffe.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To establish the existence or truth of by proof or competent evidence; to verify; <as>as, to <ex>substantiate</ex> a charge or allegation; to <ex>substantiate</ex> a declaration</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Observation is, in turn, wanted to direct and <qex>substantiate</qex> the course of experiment.</q> <rj><qau>Coleridge.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Substantiation</ent><br/
<hw>Sub*stan`ti*a"tion</hw> <pr>(?)</pr>, <pos>n.</pos> <def>The act of substantiating or proving; evidence; proof.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Substantival</ent><br/
<hw>Sub`stan*ti"val</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Of or pertaining to a substantive; of the nature of substantive.</def> -- <wordforms><wf>Sub`stan*ti"val*ly</wf>, <pos>adv.</pos></wordforms><br/
[<source>1913 Webster</source>]</p>

<p><ent>Substantive</ent><br/
<hw>Sub"stan*tive</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>substantivus</ets>: cf. F. <ets>substantif</ets>.]</ety> <sn>1.</sn> <def>Betokening or expressing existence; <as>as, the <ex>substantive</ex> verb, that is, the verb <ex>to be</ex></as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Depending on itself; independent.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>He considered how sufficient and <qex>substantive</qex> this land was to maintain itself without any aid of the foreigner.</q> <rj><qau>Bacon.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>Enduring; solid; firm; substantial.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Strength and magnitude are qualities which impress the imagination in a powerful and <qex>substantive</qex> manner.</q> <rj><qau>Hazlitt.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>Pertaining to, or constituting, the essential part or principles; <as>as, the law <ex>substantive</ex></as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Noun substantive</b></col> <fld>(Gram.)</fld>, <cd>a noun which designates an object, material or immaterial; a substantive.</cd> -- <col><b>Substantive color</b></col>, <cd>one which communicates its color without the aid of a mordant or base; -- opposed to <xex>adjective color</xex>.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Substantive</ent><br/
<hw>Sub"stan*tive</hw>, <pos>n.</pos> <ety>[Cf. F. <ets>substantif</ets>.]</ety> <fld>(Gram.)</fld> <def>A noun or name; the part of speech which designates something that exists, or some object of thought, either material or immaterial; <as>as, the words <ex>man</ex>, <ex>horse</ex>, <ex>city</ex>, <ex>goodness</ex>, <ex>excellence</ex>, are <ex>substantives</ex></as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Substantive</ent><br/
<hw>Sub"stan*tive</hw>, <pos>v. t.</pos> <def>To substantivize.</def> <mark>[R.]</mark>  <rj><au>Cudworth.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Substantively</ent><br/
<hw>Sub"stan*tive*ly</hw>, <pos>adv.</pos> <sn>1.</sn> <def>In a substantive manner; in substance; essentially.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Gram.)</fld> <def>As a substantive, name, or noun; <as>as, an adjective may be used <ex>substantively</ex></as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Substantiveness</ent><br/
<hw>Sub"stan*tive*ness</hw>, <pos>n.</pos> <def>The quality or state of being substantive.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Substantivize</ent><br/
<hw>Sub"stan*tiv*ize</hw> <pr>(?)</pr>, <pos>v. t.</pos> <def>To convert into a substantive; <as>as, to <ex>substantivize</ex> an adjective</as>.</def>  <rj><au>Fitzed. Hall.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Substile</ent><br/
<hw>Sub"stile`</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Dialing)</fld> <def>See <er>Substyle</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Substituent</ent><br/
<hw>Sub*stit"u*ent</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>substituens</ets>, p. pr. See <er>Substitute</er>.]</ety> <fld>(Chem.)</fld> <def>Any atom, group, or radical substituted for another, or entering a molecule in place of some other part which is removed.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Substitute</ent><br/
<hw>Sub"stit"ute</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>substitutus</ets>, p. p. of <ets>substituere</ets> to put under, put in the place of; <ets>sub</ets> under + <ets>statuere</ets> to put, place: cf. F. <ets>substitut</ets>. See <er>Statute</er>.]</ety> <def>One who, or that which, is substituted or put in the place of another; one who acts for another; that which stands in lieu of something else</def>; specifically <fld>(Mil.)</fld>, <def>a person who enlists for military service in the place of a conscript or drafted man.</def><-- archaic, no longer legal. --><br/
[<source>1913 Webster</source>]</p>

<p><q>Hast thou not made me here thy <qex>substitute</qex>?</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Ladies [in Shakespeare's age] . . . wore masks as the sole <qex>substitute</qex> known to our ancestors for the modern parasol.</q> <rj><qau>De Quincey.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Substitute</ent><br/
<hw>Sub"stit"ute</hw> <pr>(?)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Substituted</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Substituting</conjf> <pr>(?)</pr>.]</vmorph> <ety>[See <er>Substitute</er>, <pos>n.</pos>]</ety> <def>To put in the place of another person or thing; to exchange.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Some few verses are inserted or <qex>substituted</qex> in the room of others.</q> <rj><qau>Congreve.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Substituted</ent><br/
<hw>Sub"stit"uted</hw> <pr>(?)</pr>, <pos>a.</pos> <sn>1.</sn> <def>Exchanged; put in the place of another.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Chem.)</fld> <def>Containing substitutions or replacements; having been subjected to the process of substitution, or having some of its parts replaced; <as>as, alcohol is a <ex>substituted</ex> water; methyl amine is a <ex>substituted</ex> ammonia</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Substituted executor</b></col> <fld>(Law)</fld>, <cd>an executor appointed to act in place of one removed or resigned.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Substitution</ent><br/
<hw>Sub`sti*tu"tion</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>substitutio</ets>: cf. F. <ets>substitution</ets>.]</ety> <sn>1.</sn> <def>The act of substituting or putting one person or thing in the place of another; <as>as, the <ex>substitution</ex> of an agent, attorney, or representative to act for one in his absense; the <ex>substitution</ex> of bank notes for gold and silver as a circulating medium.</as></def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>The state of being substituted for another.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>The office or authority of one acting for another; delegated authority.</def> <mark>[R.]</mark>  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <fld>(Civil Law)</fld> <def>The designation of a person in a will to take a devise or legacy, either on failure of a former devisee or legatee by incapacity or unwillingness to accept, or after him.</def>  <rj><au>Burrill.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <fld>(Theol.)</fld> <def>The doctrine that Christ suffered vicariously, being substituted for the sinner, and that his sufferings were expiatory.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>6.</sn> <fld>(Chem.)</fld> <def>The act or process of substituting an atom or radical for another atom or radical; metathesis; also, the state of being so substituted. See <er>Metathesis</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Substitutional</ent><br/
<hw>Sub`sti*tu"tion*al</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Of or pertaining to substitution; standing in the place of another; substituted.</def> -- <wordforms><wf>Sub`sti*tu"tion*al*ly</wf>, <pos>adv.</pos></wordforms><br/
[<source>1913 Webster</source>]</p>

<p><ent>Substitutionary</ent><br/
<hw>Sub`sti*tu"tion*a*ry</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Of or pertaining to substitution; substitutional.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Substitutive</ent><br/
<hw>Sub"sti*tu`tive</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[Cf. F. <ets>substitutif</ets>, L. <ets>substitutivus</ets> conditional.]</ety> <def>Tending to afford or furnish a substitute; making substitution; capable of being substituted.</def>  <rj><au>Bp. Wilkins.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Substract</ent><br/
<hw>Sub*stract"</hw> <pr>(?)</pr>, <pos>v. t.</pos> <ety>[F. <ets>suostraire</ets>; L. <ets>subtus</ets> below (from <ets>sub</ets> under) + <ets>trahere</ets> to draw. See <er>Substract</er>.]</ety> <def>To subtract; to withdraw.</def> <mark>[Obs.]</mark>  <rj><au>Barrow.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Substraction</ent><br/
<hw>Sub*strac"tion</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[OF. <ets>substraction</ets>, F. <ets>soustraction</ets>. See <er>Subtract</er>.]</ety> <sn>1.</sn> <def>Subtraction; deduction.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Law)</fld> <def>See <er>Subtraction</er>, 3.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Substractor</ent><br/
<hw>Sub*stract"or</hw> <pr>(?)</pr>, <pos>n.</pos> <sn>1.</sn> <def>One who subtracts.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A detractor; a slanderer.</def> <mark>[Obs.]</mark>  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Substrate</ent><br/
<hw>Sub"strate</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A substratum.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Substrate</ent><br/
<hw>Sub"strate</hw>, <pos>a.</pos> <def>Having very slight furrows.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Substrate</ent><br/
<hw>Sub*strate"</hw> <pr>(?)</pr>, <pos>v. t.</pos> <ety>[L. <ets>substratus</ets>, p. p. of <ets>substrahere</ets>. See <er>Substratum</er>.]</ety> <def>To strew or lay under anything.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>The melted glass being supported by the <qex>substrated</qex> sand.</q> <rj><qau>Boyle.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Substratum</ent><br/
<hw>Sub*stra"tum</hw> <pr>(?)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Substrata</plw> <pr>(#)</pr>.</plu> <ety>[L. <ets>substratus</ets>, p. p. of <ets>substernere</ets> to strew under; <ets>sub</ets> under + <ets>sternere</ets> to strew. See <er>Stratum</er>.]</ety> <sn>1.</sn> <def>That which is laid or spread under; that which underlies something, as a layer of earth lying under another; specifically <fld>(Agric.)</fld>, the subsoil.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Metaph.)</fld> <def>The permanent subject of qualities or cause of phenomena; substance.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Substruct</ent><br/
<hw>Sub*struct"</hw> <pr>(?)</pr>, <pos>v. t.</pos> <ety>[See <er>Substruction</er>.]</ety> <def>To build beneath something; to lay as the foundation.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>He <qex>substructs</qex> the religion of Asia as the base.</q> <rj><qau>Emerson.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Substruction</ent><br/
<hw>Sub*struc"tion</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>substructio</ets>, fr. <ets>substruere</ets>, <ets>substructum</ets>, to build beneath; <ets>sub</ets> under + <ets>struere</ets> to build.]</ety> <fld>(Arch.)</fld> <def>Underbuilding; the foundation, or any preliminary structure intended to raise the lower floor or basement of a building above the natural level of the ground.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>It is a magnificent strong building, with a <qex>substruction</qex> very remarkable.</q> <rj><qau>Evelyn.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Substructure</ent><br/
<hw>Sub*struc"ture</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Pref. <ets>sub-</ets> + <ets>structure</ets>.]</ety> <sn>1.</sn> <fld>(Arch.)</fld> <def>Same as <er>Substruction</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>An under structure; a foundation; groundwork.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Substylar</ent><br/
<hw>Sub*sty"lar</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Pertaining to the substyle.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Substyle</ent><br/
<hw>Sub"style`</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Dialing)</fld> <def>A right line on which the style, or gnomon, of a dial is erected; being the common section of the face of the dial and a plane perpendicular to it passing through the style.</def> <altsp>[Written also <asp>substile</asp>.]</altsp>  <rj><au>Hutton.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Subsulphate</ent><br/
<hw>Sub*sul"phate</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Chem.)</fld> <def>A sulphate with an excess of the base.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Subsulphide</ent><br/
<hw>Sub*sul"phide</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Chem.)</fld> <def>A nonacid compound consisting of one equivalent of sulphur and more than one equivalent of some other body, as a metal.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Subsultive</ent><br/
<hw>Sub*sul"tive</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Subsultory.</def> <mark>[R.]</mark>  <rj><au>Berkley.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Subsultory</ent><br/
<hw>Sub*sul"to*ry</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>subsilire</ets>, <ets>subsultum</ets>, to spring up; <ets>sub</ets> under + <ets>salire</ets> to leap.]</ety> <def>Bounding; leaping; moving by sudden leaps or starts.</def> <mark>[R.]</mark> -- <wordforms><wf>Sub*sul"to*ri*ly</wf>, <pos>adv.</pos> <mark>[R.]</mark></wordforms><br/
[<source>1913 Webster</source>]</p>

<p><q>Flippancy opposed to solemnity, the <qex>subsultory</qex> to the continuous, -- these are the two frequent extremities to which the French manner betrays men.</q> <rj><qau>De Quincey.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Subsultus</ent><br/
||<hw>Sub*sul"tus</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[NL. See <er>Subsultory</er>.]</ety> <fld>(Med.)</fld> <def>A starting, twitching, or convulsive motion.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Subsumable</ent><br/
<hw>Sub*sum"a*ble</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Capable of being subsumed.</def>  <rj><au>J. B. Stallo.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Subsume</ent><br/
<hw>Sub*sume"</hw> <pr>(?)</pr>, <pos>v. t.</pos> <ety>[Pref. <ets>sub-</ets> + L. <ets>sumere</ets> to take.]</ety> <def>To take up into or under, as individual under species, species under genus, or particular under universal; to place (any one cognition) under another as belonging to it; to include under something else.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>To <qex>subsume</qex> one proposition under another.</q> <rj><qau>De Quincey.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>A principle under which one might <qex>subsume</qex> men's most strenuous efforts after righteousness.</q> <rj><qau>W. Pater.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Subsumption</ent><br/
<hw>Sub*sump"tion</hw> <pr>(?)</pr>, <pos>n.</pos> <sn>1.</sn> <def>The act of subsuming, or of including under another.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The first act of consciousness was a <qex>subsumption</qex> of that of which we were conscious under this notion.</q> <rj><qau>Sir W. Hamilton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>That which is subsumed, as the minor clause or premise of a syllogism.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>But whether you see cause to go against the rule, or the <qex>subsumption</qex> under the rule.</q> <rj><qau>De Quincey.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Subsumptive</ent><br/
<hw>Sub*sump"tive</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Relating to, or containing, a subsumption.</def>  <rj><au>Coleridge.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Subtangent</ent><br/
<hw>Sub*tan"gent</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Geom.)</fld> <def>The part of the axis contained between the ordinate and tangent drawn to the same point in a curve.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Subtartarean</ent><br/
<hw>Sub`tar*ta"re*an</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Being or living under Tartarus; infernal.</def> <ldquo/<xex>Subtartarean</xex> powers.<rdquo/  <rj><au>Pope.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Subtectacle</ent><br/
<hw>Sub*tec"ta*cle</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Pref. <ets>sub-</ets> + L. <ets>tectum</ets> a roof.]</ety> <def>A space under a roof; a tabernacle; a dwelling.</def> <mark>[Obs.]</mark>  <rj><au>Davies (Holy Roode).</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Subtegulaneous</ent><br/
<hw>Sub*teg`u*la"ne*ous</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>subtegulaneus</ets>; <ets>sub</ets> under + <ets>tegulare</ets> tiles for a roof.]</ety> <def>Under the roof or eaves; within doors.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Subtenant</ent><br/
<hw>Sub*ten"ant</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Law)</fld> <def>One who rents a tenement, or land, etc., of one who is also a tenant; an undertenant.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Subtend</ent><br/
<hw>Sub*tend"</hw> <pr>(?)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Subtended</conjf>; <pos>p. pr. & vb. n.</pos> <conjf>Subtending</conjf>.]</vmorph> <ety>[L. <ets>subtendere</ets>; <ets>sub</ets> under + <ets>tendere</ets> to stretch, extend. See <er>Tend</er>.]</ety> <def>To extend under, or be opposed to; <as>as, the line of a triangle which <ex>subtends</ex> the right angle; the chord <ex>subtends</ex> an arc</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Subtense</ent><br/
<hw>Sub*tense"</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>subtendere</ets>, <ets>subtentum</ets>. See <er>Subtend</er>, <er>Tense</er>, <pos>a.</pos>]</ety> <fld>(Geom.)</fld> <def>A line subtending, or stretching across; a chord; <as>as, the <ex>subtense</ex> of an arc</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Subtepid</ent><br/
<hw>Sub*tep"id</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Slightly tepid.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Subterete</ent><br/
<hw>Sub`te*rete"</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Somewhat terete.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Subterfluous</ent><br/
<ent>Subterfluent</ent><br/
<mhw>{ <hw>Sub*ter"flu*ent</hw> <pr>(?)</pr>, <hw>Sub*ter"flu*ous</hw> <pr>(?)</pr>, }</mhw> <pos>a.</pos> <ety>[L. <ets>subterfluens</ets>, p. pr. of <ets>subterfluere</ets> to flow beneath; <ets>subter</ets> under + <ets>fluere</ets> to flow.]</ety> <def>Running under or beneath.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Subterfuge</ent><br/
<hw>Sub"ter*fuge</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F., from LL. <ets>subterfugium</ets>, fr. L. <ets>subterfugere</ets> to flee secretly, to escape; <ets>subter</ets> under + <ets>fugere</ets> to flee. See <er>Fugitive</er>.]</ety> <def>That to which one resorts for escape or concealment; an artifice employed to escape censure or the force of an argument, or to justify opinions or conduct; a shift; an evasion.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Affect not little shifts and <qex>subterfuges</qex>, to avoid the force of an argument.</q> <rj><qau>I. Watts.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>By a miserable <qex>subterfuge</qex>, they hope to render this position safe by rendering it nugatory.</q> <rj><qau>Burke.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Subterrane</ent><br/
<hw>Sub"ter*rane</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Cf. L. <ets>subterraneum</ets>, F. <ets>souterrain</ets>. See <er>Subterranean</er>.]</ety> <def>A cave or room under ground.</def> <mark>[R.]</mark>  <rj><au>J. Bryant.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Subterraneal</ent><br/
<hw>Sub`ter*ra"ne*al</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Subterranean.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Subterraneous</ent><br/
<ent>Subterranean</ent><br/
<mhw>{ <hw>Sub`ter*ra"ne*an</hw> <pr>(?)</pr>, <hw>Sub`ter*ra"ne*ous</hw> <pr>(?)</pr>, }</mhw> <pos>a.</pos> <ety>[L. <ets>subterraneus</ets>; <ets>sub</ets> under + <ets>terra</ets> earth. See <er>Terrace</er>.]</ety> <def>Being or lying under the surface of the earth; situated within the earth, or under ground; <as>as, <ex>subterranean</ex> springs; a <ex>subterraneous</ex> passage.</as></def> -- <wordforms><wf>Sub`ter*ra"ne*ous*ly</wf>, <pos>adv.</pos></wordforms><br/
[<source>1913 Webster</source>]</p>

<p><ent>Subterranity</ent><br/
<hw>Sub`ter*ran"i*ty</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A place under ground; a subterrany.</def> <mark>[Obs.]</mark>  <rj><au>Sir T. Browne.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Subterrany</ent><br/
<hw>Sub"ter*ra*ny</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Subterranean.</def> <mark>[Obs.]</mark> <au>Bacon.</au> -- <def2><pos>n.</pos> <def>A subterranean place.</def></def2> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Subterrene</ent><br/
<hw>Sub`ter*rene"</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>subterrenus</ets>, equiv. to <ets>subterraneus</ets>.]</ety> <def>Subterraneous.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Subterrestrial</ent><br/
<hw>Sub`ter*res"tri*al</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Subterranean.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Subthalamic</ent><br/
<hw>Sub`tha*lam"ic</hw> <pr>(?)</pr>, <pos>a.</pos> <fld>(Anat.)</fld> <def>Situated under the optic thalamus.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Subtile</ent><br/
<hw>Sub"tile</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>subtilis</ets>. See <er>Subtile</er>.]</ety> <sn>1.</sn> <def>Thin; not dense or gross; rare; <as>as, <ex>subtile</ex> air; <ex>subtile</ex> vapor; a <ex>subtile</ex> medium</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Delicately constituted or constructed; nice; fine; delicate; tenuous; finely woven.</def> <ldquo/A <xex>sotil</xex> [subtile] twine's thread.<rdquo/  <rj><au>Chaucer.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>More <qex>subtile</qex> web Arachne can not spin.</q> <rj><qau>Spenser.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>I do distinguish plain<br/
Each <qex>subtile</qex> line of her immortal face.</q> <rj><qau>Sir J. Davies.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>Acute; piercing; searching.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The slow disease and <qex>subtile</qex> pain.</q> <rj><qau>Prior.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <def>Characterized by nicety of discrimination; discerning; delicate; refined; subtle.</def> <altsp>[In this sense now commonly written <asp>subtle</asp>.]</altsp><br/
[<source>1913 Webster</source>]</p>

<p><q>The genius of the Spanish people is exquisitely <qex>subtile</qex>, without being at all acute; hence there is so much humor and so little wit in their literature. The genius of the Italians, on the contrary, is acute, profound, and sensual, but not <qex>subtile</qex>; hence what they think to be humorous, is merely witty.</q> <rj><qau>Coleridge.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>The <qex>subtile</qex> influence of an intellect like Emerson's.</q> <rj><qau>Hawthorne.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <def>Sly; artful; cunning; crafty; subtle; <as>as, a <ex>subtile</ex> person; a <ex>subtile</ex> adversary; a <ex>subtile</ex> scheme.</as></def> <altsp>[In this sense now commonly written <asp>subtle</asp>.]</altsp><br/
[<source>1913 Webster</source>]</p>

<p><syn><b>Syn.</b> -- <er>Subtile</er>, <er>Acute</er>.</syn> <usage> In <xex>acute</xex> the image is that of a needle's point; in <xex>subtile</xex> that of a thread spun out to fineness. The <xex>acute</xex> intellect pierces to its aim; the <xex>subtile</xex> (or <xex>subtle</xex>) intellect winds its way through obstacles.</usage><br/
[<source>1913 Webster</source>]</p>

<p>-- <wordforms><wf>Sub"tile*ly</wf>, <pos>adv.</pos> -- <wf>Sub"tile*ness</wf>, <pos>n.</pos></wordforms><br/
[<source>1913 Webster</source>]</p>

<p><ent>Subtiliate</ent><br/
<hw>Sub*til"i*ate</hw> <pr>(?)</pr>, <pos>v. t.</pos> <ety>[LL. <ets>subtiliare</ets>.]</ety> <def>To make thin or rare.</def> <mark>[Obs.]</mark> <au>Harvey.</au> -- <wordforms><wf>Sub`til*i*a"tion</wf> <pr>(#)</pr>, <pos>n.</pos> <mark>[Obs.]</mark> <au>Boyle.</au></wordforms><br/
[<source>1913 Webster</source>]</p>

<p><ent>Subtilism</ent><br/
<hw>Sub"til*ism</hw> <pr>(?)</pr>, <pos>n.</pos> <def>The quality or state of being subtile; subtility; subtlety.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The high orthodox <qex>subtilism</qex> of Duns Scotus.</q> <rj><qau>Milman.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Subtility</ent><br/
<hw>Sub*til"i*ty</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>subtilitas</ets>: cf. F. <ets>subtilit<eacute/</ets>. See <er>Subtle</er>.]</ety> <def>Subtilty.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Subtilization</ent><br/
<hw>Sub`til*i*za"tion</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Cf. F. <ets>subtilization</ets>.]</ety> <sn>1.</sn> <def>The act of making subtile.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Old Chem.)</fld> <def>The operation of making so volatile as to rise in steam or vapor.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>Refinement; subtlety; extreme attenuation.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Subtilize</ent><br/
<hw>Sub"til*ize</hw> <pr>(?)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Subtilized</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Subtilizing</conjf> <pr>(?)</pr>.]</vmorph> <ety>[L. <ets>subtiliser</ets>.]</ety> <sn>1.</sn> <def>To make thin or fine; to make less gross or coarse.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To refine; to spin into niceties; <as>as, to <ex>subtilize</ex> arguments</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Nor as yet have we <qex>subtilized</qex> ourselves into savages.</q> <rj><qau>Burke.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Subtilize</ent><br/
<hw>Sub"til*ize</hw>, <pos>v. i.</pos> <def>To refine in argument; to make very nice distinctions.</def>  <rj><au>Milner.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Subtilizer</ent><br/
<hw>Sub"til*i`zer</hw> <pr>(?)</pr>, <pos>n.</pos> <def>One who subtilizes.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Subtilty</ent><br/
<hw>Sub"til*ty</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Contr. fr. <ets>subtility</ets>.]</ety> <sn>1.</sn> <def>The quality or state of being subtile; thinness; fineness; <as>as, the <ex>subtility</ex> of air or light</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Refinement; extreme acuteness; subtlety.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Intelligible discourses are spoiled by too much <qex>subtility</qex> in nice divisions.</q> <rj><qau>Locke.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>Cunning; skill; craft.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>To learn a lewd man this <qex>subtility</qex>.</q> <rj><qau>Chaucer.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>Slyness in design; artifice; guile; a cunning design or artifice; a trick; subtlety.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>O full of all <qex>subtility</qex> and all mischief.</q> <rj><qau>Acts xiii. 10.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ In senses 2, 3, and 4 the word is more commonly written <xex>subtlety</xex>.</note><br/
[<source>1913 Webster</source>]</p>

<p><ent>Subtle</ent><br/
<hw>Sub"tle</hw> <pr>(?)</pr>, <pos>a.</pos> <amorph>[<pos>Compar.</pos> <adjf>Subtler</adjf> <pr>(?)</pr>; <pos>superl.</pos> <adjf>Subtlest</adjf> <pr>(?)</pr>.]</amorph> <ety>[OE. <ets>sotil</ets>, <ets>subtil</ets>, OF. <ets>soutil</ets>, later <ets>subtil</ets>, F. <ets>subtil</ets>, L. <ets>subtilis</ets>; probably, originally, woven fine, and fr. <ets>sub</ets> under + <ets>tela</ets> a web, fr. <ets>texere</ets> to weave. See <er>Text</er>, and cf. <er>Subtile</er>.]</ety> <sn>1.</sn> <def>Sly in design; artful; cunning; insinuating; subtile; -- applied to persons; <as>as, a <ex>subtle</ex> foe</as>.</def> <ldquo/A <xex>subtle</xex> traitor.<rdquo/  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Cunningly devised; crafty; treacherous; <as>as, a <ex>subtle</ex> stratagem</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>Characterized by refinement and niceness in drawing distinctions; nicely discriminating; -- said of persons; <as>as, a <ex>subtle</ex> logician</as>; refined; tenuous; sinuous; insinuating; hence, penetrative or pervasive; -- said of the mind; its faculties, or its operations; <as>as, a <ex>subtle</ex> intellect; a <ex>subtle</ex> imagination; a <ex>subtle</ex> process of thought</as>; also, difficult of apprehension; elusive.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Things remote from use, obscure and <qex>subtle</qex>.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>Smooth and deceptive.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>Like to a bowl upon a <qex>subtle</qex> ground [bowling ground].</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><syn><b>Syn.</b> -- Artful; crafty; cunning; shrewd; sly; wily.</syn> <usage> <xex>Subtle</xex> is the most comprehensive of these epithets and implies the finest intellectual quality. See <er>Shrewd</er>, and <er>Cunning</er>.</usage><br/
[<source>1913 Webster</source>]</p>

<p><-- p. 1438 --></p>

<p><ent>Subtleness</ent><br/
<hw>Sub"tle*ness</hw> <pr>(?)</pr>, <pos>n.</pos> <def>The quality or state of being subtle; subtlety.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Subtlety</ent><br/
<hw>Sub"tle*ty</hw> <pr>(?)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Subtleties</plw> <pr>(#)</pr>.</plu> <ety>[OE. <ets>sotelte</ets>, <ets>sutilte</ets>, OF. <ets>sotillete</ets>, L. <ets>subtilitas</ets>. See <er>Subtle</er>, and cf. <er>Subtility</er>.]</ety> <sn>1.</sn> <def>The quality or state of being subtle, or sly; cunning; craftiness; artfulness.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The fox which lives by <qex>subtlety</qex>.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Nice discernment with delicacy of mental action; nicety of discrimination.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>Something that is sly, crafty, or delusive.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Unlearned in the world's false <qex>subtleties</qex>.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Subtly</ent><br/
<hw>Sub"tly</hw> <pr>(?)</pr>, <pos>adv.</pos> <def>In a subtle manner; slyly; artfully; cunningly.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Thou seest how <qex>subtly</qex> to detain thee I devise.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Nicely; delicately.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>In the nice bee what sense so <qex>subtly</qex> true.</q> <rj><qau>Pope.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q><qex>Subtly</qex> communicating itself to my sensibilities, but evading the analysis of my mind.</q> <rj><qau>Hawthorne.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>Deceitfully; delusively.</def> <mark>[Obs.]</mark>  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Subtonic</ent><br/
<hw>Sub*ton"ic</hw> <pr>(?)</pr>, <pos>a.</pos> <fld>(Phonetics)</fld> <def>Applied to, or distinguishing, a speech element consisting of tone, or proper vocal sound, not pure as in the vowels, but dimmed and otherwise modified by some kind of obstruction in the oral or the nasal passage, and in some cases with a mixture of breath sound; -- a term introduced by Dr. James Rush in 1833. See <xex>Guide to Pronunciation</xex>, <sect/<sect/155, 199-202.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Subtonic</ent><br/
<hw>Sub*ton"ic</hw>, <pos>n.</pos> <sn>1.</sn> <fld>(Phonetics)</fld> <def>A subtonic sound or element; a vocal consonant, as <it>b</it>, <it>d</it>, <it>g</it>, <it>n</it>, etc.; a subvocal.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Mus.)</fld> <def>The seventh tone of the scale, or that immediately below the tonic; -- called also <altname>subsemitone</altname>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Subtorrid</ent><br/
<hw>Sub*tor"rid</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Nearly torrid.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Subtract</ent><br/
<hw>Sub*tract"</hw> <pr>(?)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Subtracted</conjf>; <pos>p. pr. & vb. n.</pos> <conjf>Subtracting</conjf>.]</vmorph> <ety>[L. <ets>subtractus</ets>, p. p. of <ets>subtrahere</ets> to draw from beneath, withdraw, remove; <ets>sub</ets> under + <ets>trahere</ets> to draw. See <er>Trace</er>, <pos>v. t.</pos>, and cf. <er>Substract</er>.]</ety> <def>To withdraw, or take away, as a part from the whole; to deduct; <as>as, <ex>subtract</ex> 5 from 9, and the remainder is 4</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Subtracter</ent><br/
<hw>Sub*tract"er</hw> <pr>(?)</pr>, <pos>n.</pos> <sn>1.</sn> <def>One who subtracts.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>The subtrahend.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Subtraction</ent><br/
<hw>Sub*trac"tion</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>subtractio</ets> a drawing back. See <er>Subtract</er>, and cf. <er>Substraction</er>.]</ety> <sn>1.</sn> <def>The act or operation of subtracting or taking away a part.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Math.)</fld> <def>The taking of a lesser number or quantity from a greater of the same kind or denomination; an operation for finding the difference between two numbers or quantities.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <fld>(Law)</fld> <def>The withdrawing or withholding from a person of some right to which he is entitled by law.</def><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ Thus the <xex>subtraction</xex> of conjugal rights is when either the husband or wife withdraws from the other and lives separate without sufficient reason. The <xex>subtraction</xex> of a legacy is the withholding or detailing of it from the legatee by the executor. In like manner, the withholding of any service, rent, duty, or custom, is a <xex>subtraction</xex>, for which the law gives a remedy.</note>  <rj><au>Blackstone.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Subtractive</ent><br/
<hw>Sub*trac"tive</hw> <pr>(?)</pr>, <pos>a.</pos> <sn>1.</sn> <def>Tending, or having power, to subtract.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Math.)</fld> <def>Having the negative sign, or sign minus.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Subtrahend</ent><br/
<hw>Sub"tra*hend`</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>subtrahendus</ets> that is to be subtracted, p.fut.pess. of <ets>subtrahere</ets>. See <er>Subtract</er>.]</ety> <fld>(Math.)</fld> <def>The sum or number to be subtracted, or taken from another (the <contr>minuend</contr>) to find the <contr>difference</contr>.</def><br/
[<source>1913 Webster</source> + <source>PJC</source>]</p>

<p><ent>Subtranslucent</ent><br/
<hw>Sub`trans*lu"cent</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Not perfectly translucent.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Subtransparent</ent><br/
<hw>Sub`trans*pa"rent</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Not perfectly transparent.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Subtreasurer</ent><br/
<hw>Sub*treas"ur*er</hw> <pr>(?)</pr>, <pos>n.</pos> <def>The public officer who has charge of a subtreasury.</def> <mark>[U. S.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Subtreasury</ent><br/
<hw>Sub*treas"ur*y</hw> <pr>(?)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Subtreasuries</plw> <pr>(<?/)</pr>.</plu> <def>A subordinate treasury, or place of deposit; <as>as, the United States <ex>subtreasury</ex> at New York</as>.</def> <mark>[U. S.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Subtriangular</ent><br/
<hw>Sub`tri*an"gu*lar</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Nearly, but not perfectly, triangular.</def>  <rj><au>Darwin.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Subtribe</ent><br/
<hw>Sub"tribe`</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Bot. & Zool.)</fld> <def>A division of a tribe; a group of genera of a little lower rank than a tribe.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Subtrihedral</ent><br/
<hw>Sub`tri*he"dral</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Approaching the form of a three-sided pyramid; <as>as, the <ex>subtrihedral</ex> crown of a tooth</as>.</def>  <rj><au>Owen.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Subtriple</ent><br/
<hw>Sub*tri"ple</hw> <pr>(?)</pr>, <pos>a.</pos> <fld>(Math.)</fld> <def>Containing a third, or one part to three.</def>  <rj><au>Bp. Wilkins.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Subtriplicate</ent><br/
<hw>Sub*trip"li*cate</hw> <pr>(?)</pr>, <pos>a.</pos> <fld>(Math.)</fld> <def>Expressed by the cube root; -- said especially of ratios.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Subtriplicate ratio</b></col>, <cd>the ratio of the cube root; thus, the <xex>subtriplicate ratio</xex> of <it>a</it> to <it>b</it> is <cuberoot/<it>a</it> to <cuberoot/<it>b</it>, or <cuberoot/<fract><it>a</it>/<it>b</it></fract>.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Subtropical</ent><br/
<hw>Sub*trop"ic*al</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Nearly tropical.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Subtrude</ent><br/
<hw>Sub*trude"</hw> <pr>(?)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Subtruded</conjf>; <pos>p. pr. & vb. n.</pos> <conjf>Subtruding</conjf>.]</vmorph> <ety>[Pref. <ets>sub-</ets> + L. <ets>trudere</ets> to thrust.]</ety> <def>To place under; to insert.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Subturriculate</ent><br/
<hw>Sub`tur*ric"u*late</hw> <pr>(?)</pr>, <pos>a.</pos> <fld>(Zool.)</fld> <def>Somewhat turriculate.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Subtutor</ent><br/
<hw>Sub*tu"tor</hw> <pr>(?)</pr>, <pos>n.</pos> <def>An under tutor.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Subtypical</ent><br/
<hw>Sub*typ"ic*al</hw> <pr>(?)</pr>, <pos>a.</pos> <fld>(Zool.)</fld> <def>Deviating somewhat from the type of a species, genus, or other group; slightly aberrant.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Subulated</ent><br/
<ent>Subulate</ent><br/
<mhw>{ <hw>Su"bu*late</hw> <pr>(?)</pr>, <hw>Su"bu*la`ted</hw> <pr>(?)</pr>, }</mhw> <pos>a.</pos> <ety>[NL. <ets>subulatus</ets>, fr. L. <ets>subula</ets> an awl.]</ety> <def>Very narrow, and tapering gradually to a fine point from a broadish base; awl-shaped; linear.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Subulicornes</ent><br/
||<hw>Su`bu*li*cor"nes</hw> <pr>(?)</pr>, <pos>n. pl.</pos> <ety>[NL., from L. <ets>subula</ets> an awl + <ets>cornu</ets> horn.]</ety> <fld>(Zool.)</fld> <def>A division of insects having slender or subulate antennae. The dragon flies and May flies are examples.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Subuliform</ent><br/
<hw>Su"bu*li*form</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Subulate.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Subulipalp</ent><br/
<hw>Su"bu*li*palp`</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>subula</ets> an awl + E. <ets>palp</ets>.]</ety> <fld>(Zool.)</fld> <def>One of a group of carabid beetles having slender palpi.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Subumbonal</ent><br/
<hw>Sub`um*bo"nal</hw> <pr>(?)</pr>, <pos>a.</pos> <fld>(Zool.)</fld> <def>Beneath or forward of the umbos of a bivalve shell.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Subumbrella</ent><br/
<hw>Sub`um*brel"la</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Zool.)</fld> <def>The integument of the under surface of the bell, or disk-shaped body, of a jellyfish.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Subundation</ent><br/
<hw>Sub`un*da"tion</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Pref. <ets>sub-</ets> + L. <ets>unda</ets> a wave.]</ety> <def>A flood; a deluge.</def> <mark>[Obs.]</mark>  <rj><au>Huloet.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Subungual</ent><br/
<hw>Sub*un"gual</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Under the nail or hoof.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Suburb</ent><br/
<hw>Sub"urb</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>suburbium</ets>; <ets>sub</ets> under, below, near + <ets>urbs</ets> a city. See <er>Urban</er>.]</ety> <sn>1.</sn> <def>An outlying part of a city or town; a smaller place immediately adjacent to a city; in the plural, the region which is on the confines of any city or large town; <as>as, a house stands in the <ex>suburbs</ex>; a garden situated in the <ex>suburbs</ex> of Paris</as>.</def> <ldquo/In the <xex>suburbs</xex> of a town.<rdquo/  <rj><au>Chaucer.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>[London] could hardly have contained less than thirty or forty thousand souls within its walls; and the <qex>suburbs</qex> were very populous.</q> <rj><qau>Hallam.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Hence, the confines; the outer part; the environment.</def> <ldquo/The <xex>suburbs</xex> . . . of sorrow.<rdquo/  <rj><au>Jer. Taylor.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>The <qex>suburb</qex> of their straw-built citadel.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Suburb roister</b></col>, <cd>a rowdy; a loafer.</cd> <mark>[Obs.]</mark> <au>Milton.</au></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Suburban</ent><br/
<hw>Sub*ur"ban</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>suburbanus</ets>.]</ety> <def>Of or pertaining to suburbs; inhabiting, or being in, the suburbs of a city.</def> <ldquo/<xex>Suburban</xex> taverns.<rdquo/  <rj><au>Longfellow.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q><qex>Suburban</qex> villas, highway-side retreats, . . . <br/
Delight the citizen.</q> <rj><qau>Cowper.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Suburban</ent><br/
<hw>Sub*ur"ban</hw>, <pos>n.</pos> <def>One who dwells in the suburbs.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Suburbed</ent><br/
<hw>Sub"urbed</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Having a suburb or suburbs on its outer part.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Suburbian</ent><br/
<ent>Suburbial</ent><br/
<mhw>{ <hw>Sub*ur"bi*al</hw> <pr>(?)</pr>, <hw>Sub*ur"bi*an</hw> <pr>(?)</pr>, }</mhw> <pos>a.</pos> <def>Suburban.</def> <mark>[Obs.]</mark> <ldquo/<xex>Suburbial</xex> fields.<rdquo/ <au>Warton.</au> <ldquo/<xex>Suburbian</xex> muse.<rdquo/ <au>Dryden.</au><br/
[<source>1913 Webster</source>]</p>

<p><ent>Suburbicary</ent><br/
<ent>Suburbicarian</ent><br/
<mhw>{ <hw>Sub*ur`bi*ca"ri*an</hw> <pr>(?)</pr>, <hw>Sub*ur"bi*ca*ry</hw> <pr>(?)</pr>, }</mhw> <pos>a.</pos> <ety>[LL. <ets>suburbicarius</ets>, equiv. to L. <ets>suburbanus</ets>: cf. F. <ets>suburbicaire</ets>. See <er>Suburban</er>.]</ety> <def>Being in the suburbs; -- applied to the six dioceses in the suburbs of Rome subject to the pope as bishop of Rome.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The pope having stretched his authority beyond the bounds of his <qex>suburbicarian</qex> precincts.</q> <rj><qau>Barrow.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Suburethral</ent><br/
<hw>Sub`u*re"thral</hw> <pr>(?)</pr>, <pos>a.</pos> <fld>(Anat.)</fld> <def>Situated under the urethra, or under its orifice.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Subvaginal</ent><br/
<hw>Sub*vag"i*nal</hw> <pr>(?)</pr>, <pos>a.</pos> <fld>(Anat.)</fld> <def>Situated under or inside a sheath or vaginal membrane; <as>as, the <ex>subvaginal</ex>, or subdural, spaces about the optic nerve</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Subvariety</ent><br/
<hw>Sub`va*ri"e*ty</hw> <pr>(?)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>-ties</plw> <pr>(<?/)</pr>.</plu> <def>A subordinate variety, or a division of a variety.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Subvene</ent><br/
<hw>Sub*vene"</hw> <pr>(?)</pr>, <pos>v. i.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Subvened</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Subvening</conjf>.]</vmorph> <ety>[Pref. <ets>sub-</ets> + L. <ets>venire</ets> to come. See <er>Subvention</er>.]</ety> <def>To come under, as a support or stay; to happen.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>A future state must needs <qex>subvene</qex> to prevent the whole edifice from falling into ruin.</q> <rj><qau>Bp. Warburton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Subventaneous</ent><br/
<hw>Sub`ven*ta"ne*ous</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[Pref. <ets>sub-</ets> + L. <ets>ventus</ets> wind.]</ety> <def>Produced by the wind.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Subvention</ent><br/
<hw>Sub*ven"tion</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F., fr. LL. <ets>subventio</ets>, fr. L. <ets>subvenire</ets> to come up to one's assistance, to assist. See <er>Souvenir</er>, and cf. <er>Subvene</er>.]</ety> <sn>1.</sn> <def>The act of coming under.</def> <ldquo/The <xex>subvention</xex> of a cloud.<rdquo/  <rj><au>Stackhouse.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>The act of relieving, as of a burden; support; aid; assistance; help.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>A government aid or bounty.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Subvention</ent><br/
<hw>Sub*ven"tion</hw>, <pos>v. t.</pos> <def>To subventionize.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Subventionize</ent><br/
<hw>Sub*ven"tion*ize</hw> <pr>(?)</pr>, <pos>v. t.</pos> <def>To come to the aid of; to subsidize; to support.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Subventitious</ent><br/
<hw>Sub`ven*ti"tious</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Helping; aiding; supporting.</def>  <rj><au>Urquhart.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Subverse</ent><br/
<hw>Sub*verse"</hw> <pr>(?)</pr>, <pos>v. t.</pos> <ety>[L. <ets>subversus</ets>, p. p. of <ets>subvertere</ets>. See <er>Subvert</er>.]</ety> <def>To subvert.</def> <mark>[Obs.]</mark>  <rj><au>Spenser.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Subversion</ent><br/
<hw>Sub*ver"sion</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>subversio</ets>: cf. F. <ets>subversion</ets>. See <er>Subvert</er>.]</ety> <def>The act of overturning, or the state of being overturned; entire overthrow; an overthrow from the foundation; utter ruin; destruction; <as>as, the <ex>subversion</ex> of a government; the <ex>subversion</ex> of despotic power; the <ex>subversion</ex> of the constitution.</as></def><br/
[<source>1913 Webster</source>]</p>

<p><q>The <qex>subversion</qex> [by a storm] of woods and timber . . . through my whole estate.</q> <rj><qau>Evelyn.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Laws have been often abused to the oppression and <qex>subversion</qex> of that order they were intended to preserve.</q> <rj><qau>Rogers.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Subversionary</ent><br/
<hw>Sub*ver"sion*a*ry</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Promoting destruction.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Subversive</ent><br/
<hw>Sub*ver"sive</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[Cf. F. <ets>subversif</ets>.]</ety> <def>Tending to subvert; having a tendency to overthrow and ruin.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Lying is a vice <qex>subversive</qex> of the very ends and design of conversation.</q> <rj><qau>Rogers.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Subvert</ent><br/
<hw>Sub*vert"</hw> <pr>(?)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Subverted</conjf>; <pos>p. pr. & vb. n.</pos> <conjf>Subverting</conjf>.]</vmorph> <ety>[L. <ets>subvertere</ets>, <ets>subversum</ets>; <ets>sub</ets> under + <ets>vertere</ets> to turn: cf. F. <ets>subvertir</ets>. See <er>Verse</er>.]</ety> <sn>1.</sn> <def>To overturn from the foundation; to overthrow; to ruin utterly.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>These are his substance, sinews, arms, and strength,<br/
With which he yoketh your rebellious necks,<br/
Razeth your cities, and <qex>subverts</qex> your towns.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>This would <qex>subvert</qex> the principles of all knowledge.</q> <rj><qau>Locke.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To pervert, as the mind, and turn it from the truth; to corrupt; to confound.</def>  <rj><au>2 Tim. iii. 14.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><syn><b>Syn.</b> -- To overturn; overthrow; destroy; invert; reverse; extinguish.</syn><br/
[<source>1913 Webster</source>]</p>

<p><ent>Subvert</ent><br/
<hw>Sub*vert"</hw> <pr>(?)</pr>, <pos>v. i.</pos> <def>To overthrow anything from the foundation; to be subversive.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>They have a power given to them like that of the evil principle, to <qex>subvert</qex> and destroy.</q><br/
[<source>1913 Webster</source>]</p>

<p><ent>Subvertant</ent><br/
<hw>Sub*vert"ant</hw> <pr>(?)</pr>, <pos>a.</pos> <fld>(Her.)</fld> <def>Reversed.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Subvertebral</ent><br/
<hw>Sub*ver"te*bral</hw> <pr>(?)</pr>, <pos>a.</pos> <fld>(Anat.)</fld> <def>Situated beneath, or on the ventral side of, the vertebral column; situated beneath, or inside of, the endoskeleton; hypaxial; hyposkeletal.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Subverter</ent><br/
<hw>Sub*vert"er</hw> <pr>(?)</pr>, <pos>n.</pos> <def>One who, or that which, subverts; an overthrower.</def>  <rj><au>Sir T. More.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Subvertible</ent><br/
<hw>Sub*vert"i*ble</hw> <pr>(?)</pr>, <pos>a.</pos> <def>That may be subverted.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Subvitalized</ent><br/
<hw>Sub*vi"tal*ized</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Imperfectly vitalized; having naturally but little vital power or energy.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Subvocal</ent><br/
<hw>Sub*vo"cal</hw> <pr>(?)</pr>, <pos>a. & n.</pos> <def>Same as <er>Subtonic</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Subway</ent><br/
<hw>Sub"way`</hw> <pr>(?)</pr>, <pos>n.</pos> <sn>1.</sn> <def>An underground way or gallery; especially, a passage under a street, in which water mains, gas mains, telegraph wires, etc., are conducted.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>An underground railroad, usually having trains powered by electricity provided by an electric line running through the underground tunnel.  It is usually confined to the center portion of cities; -- called also <altname>tube</altname>, and in Britain, <altname>underground</altname>.  In certain other countries (as in France or Russia) it is called the <altname>metro</altname>.</def><br/
[<source>PJC</source>]</p>

<p><ent>Subworker</ent><br/
<hw>Sub*work"er</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A subordinate worker or helper.</def>  <rj><au>South.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Subzonal</ent><br/
<hw>Sub*zon"al</hw> <pr>(?)</pr>, <pos>a.</pos> <fld>(Anat.)</fld> <def>Situated under a zone, or zona; -- applied to a membrane between the zona radiata and the umbilical vesicle in the mammal embryo.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Subzygomatic</ent><br/
<hw>Sub*zyg`o*mat"ic</hw> <pr>(?)</pr>, <pos>a.</pos> <fld>(Anat.)</fld> <def>Situated under the zygoma or zygomatic process.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Succade</ent><br/
<hw>Suc"cade</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>succus</ets>, <ets>sucus</ets>, juice: cf. F. <ets>succade</ets> a sugarbox.  Cf. <er>Sucket</er>.]</ety> <sn>1.</sn> <def>A sweetmeat.</def> <mark>[Obs.]</mark>  <rj><au>Holland.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <pluf>pl.</pluf> <fld>(Com.)</fld> <def>Sweetmeats, or preserves in sugar, whether fruit, vegetables, or confections.</def>  <rj><au>Blakely.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Succade gourd</b></col>. <fld>(Bot.)</fld> <cd>Same as <cref>Vegetable marrow</cref>, under <er>Vegetable</er>.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Succedane</ent><br/
<hw>Suc"ce*dane</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A succedaneum.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Succedaneous</ent><br/
<hw>Suc`ce*da"ne*ous</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>succedaneus</ets>. See <er>Succeed</er>.]</ety> <def>Pertaining to, or acting as, a succedaneum; supplying the place of something else; being, or employed as, a substitute for another.</def>  <rj><au>Sir T. Browne.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Succedaneum</ent><br/
<hw>Suc`ce*da"ne*um</hw> <pr>(?)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Succedanea</plw> <pr>(#)</pr>.</plu> <ety>[NL. See <er>Succedaneous</er>.]</ety> <def>One who, or that which, succeeds to the place of another; that which is used for something else; a substitute;</def> <specif>specifically</specif> <fld>(Med.)</fld>, <def>a remedy used as a substitute for another.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>In lieu of me, you will have a very charming <qex>succedaneum</qex>, Lady Harriet Stanhope.</q> <rj><qau>Walpole.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Succeed</ent><br/
<hw>Suc*ceed"</hw> <pr>(?)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Succeeded</conjf>; <pos>p. pr. & vb. n.</pos> <conjf>Succeeding</conjf>.]</vmorph> <ety>[L. <ets>succedere</ets>, <ets>successum</ets>; <ets>sub</ets> under + <ets>cedere</ets> to go, to go along, approach, follow, succeed: cf. F. <ets>succ<eacute/der</ets>. See <er>Cede</er>, and cf. <er>Success</er>.]</ety> <sn>1.</sn> <def>To follow in order; to come next after; hence, to take the place of; <as>as, the king's eldest son <ex>succeeds</ex> his father on the throne; autumn <ex>succeeds</ex> summer</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>As he saw him nigh <qex>succeed</qex>.</q> <rj><qau>Spenser.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To fall heir to; to inherit.</def> <mark>[Obs. & R.]</mark>  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>To come after; to be subsequent or consequent to; to follow; to pursue.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Destructive effects . . . <qex>succeeded</qex> the curse.</q> <rj><qau>Sir T. Browne.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>To support; to prosper; to promote.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q><qex>Succeed</qex> my wish and second my design.</q> <rj><qau>Dryden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Succeed</ent><br/
<hw>Suc*ceed"</hw>, <pos>v. i.</pos> <sn>1.</sn> <def>To come in the place of another person, thing, or event; to come next in the usual, natural, or prescribed course of things; to follow; hence, to come next in the possession of anything; -- often with <xex>to</xex>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>If the father left only daughters, they equally <qex>succeeded</qex> to him in copartnership.</q> <rj><qau>Sir M. Hale.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Enjoy till I return<br/
Short pleasures; for long woes are to <qex>succeed</qex>!</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <specif>Specifically:</specif> <def>To ascend the throne after the removal the death of the occupant.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>No woman shall <qex>succeed</qex> in Salique land.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>To descend, as an estate or an heirloom, in the same family; to devolve.</def>  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>To obtain the object desired; to accomplish what is attempted or intended; to have a prosperous issue or termination; to be successful; <as>as, he <ex>succeeded</ex> in his plans; his plans <ex>succeeded</ex></as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>It is almost impossible for poets to <qex>succeed</qex> without ambition.</q> <rj><qau>Dryden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Spenser endeavored it in Shepherd's Kalendar; but neither will it <qex>succeed</qex> in English.</q> <rj><qau>Dryden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <def>To go under cover.</def> <mark>[A latinism. Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>Will you to the cooler cave <qex>succeed</qex>!</q> <rj><qau>Dryden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><syn><b>Syn.</b> -- To follow; pursue. See <er>Follow</er>.</syn><br/
[<source>1913 Webster</source>]</p>

<p><ent>Succeedant</ent><br/
<hw>Suc*ceed"ant</hw> <pr>(?)</pr>, <pos>a.</pos> <fld>(Her.)</fld> <def>Succeeding one another; following.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Succeeder</ent><br/
<hw>Suc*ceed"er</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A successor.</def>  <rj><au>Shak.</au>  <au>Tennyson.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Succeeding</ent><br/
<hw>Suc*ceed"ing</hw>, <pos>n.</pos> <def>The act of one who, or that which, succeeds; also, that which succeeds, or follows after; consequence.</def>  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Succentor</ent><br/
<hw>Suc"cen*tor</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[LL., an accompanier in singing, fr. <ets>succinere</ets> to sing, to accompany; <ets>sub</ets> under, after + <ets>canere</ets> to sing.]</ety> <fld>(Eccl.)</fld> <def>A subchanter.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Success</ent><br/
<hw>Suc*cess"</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>successus</ets>: cf. F. <ets>succ<egrave/s</ets>. See <er>Succeed</er>.]</ety> <sn>1.</sn> <def>Act of succeeding; succession.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>Then all the sons of these five brethren reigned<br/
By due <qex>success</qex>.</q> <rj><qau>Spenser.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>That which comes after; hence, consequence, issue, or result, of an endeavor or undertaking, whether good or bad; the outcome of effort.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Men . . . that are like to do that, that is committed to them, and to report back again faithfully the <qex>success</qex>.</q> <rj><qau>Bacon.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Perplexed and troubled at his bad <qex>success</qex><br/
The tempter stood.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>The favorable or prosperous termination of anything attempted; the attainment of a proposed object; prosperous issue.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Dream of <qex>success</qex> and happy victory!</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Or teach with more <qex>success</qex> her son<br/
The vices of the time to shun.</q> <rj><qau>Waller.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Military <qex>successes</qex>, above all others, elevate the minds of a people.</q> <rj><qau>Atterbury.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>That which meets with, or one who accomplishes, favorable results, as a play or a player.</def> <mark>[Colloq.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><-- p. 1439 --></p>

<p><ent>Successary</ent><br/
<hw>Suc"ces*sa*ry</hw> <pr>(?)</pr>, <pos>n.</pos> <def>Succession.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>My peculiar honors, not derived<br/
From <qex>successary</qex>, but purchased with my blood.</q> <rj><qau>Beau. & Fl.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Successful</ent><br/
<hw>Suc*cess"ful</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Resulting in success; assuring, or promotive of, success; accomplishing what was proposed; having the desired effect; hence, prosperous; fortunate; happy; <as>as, a <ex>successful</ex> use of medicine; a <ex>successful</ex> experiment; a <ex>successful</ex> enterprise</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Welcome, nephews, from <qex>successful</qex> wars.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><syn><b>Syn.</b> -- Happy; prosperous; fortunate; auspicious; lucky. See <er>Fortunate</er>.</syn><br/
[<source>1913 Webster</source>]</p>

<p>-- <wordforms><wf>Suc*cess"ful*ly</wf>, <pos>adv.</pos> -- <wf>Suc*cess"ful*ness</wf>, <pos>n.</pos></wordforms><br/
[<source>1913 Webster</source>]</p>

<p><ent>Succession</ent><br/
<hw>Suc*ces"sion</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>successio</ets>: cf. F. <ets>succession</ets>. See <er>Succeed</er>.]</ety> <sn>1.</sn> <def>The act of succeeding, or following after; a following of things in order of time or place, or a series of things so following; sequence; <as>as, a <ex>succession</ex> of good crops; a <ex>succession</ex> of disasters</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A series of persons or things according to some established rule of precedence; <as>as, a <ex>succession</ex> of kings, or of bishops; a <ex>succession</ex> of events in chronology</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>He was in the <qex>succession</qex> to an earldom.</q> <rj><qau>Macaulay.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>An order or series of descendants; lineage; race; descent.</def> <ldquo/A long <xex>succession</xex> must ensue.<rdquo/  <rj><au>Milton.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>The power or right of succeeding to the station or title of a father or other predecessor; the right to enter upon the office, rank, position, etc., held ny another; also, the entrance into the office, station, or rank of a predecessor; specifically, the succeeding, or right of succeeding, to a throne.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>You have the voice of the king himself for your <qex>succession</qex> in Denmark.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>The animosity of these factions did not really arise from the dispute about the <qex>succession</qex>.</q> <rj><qau>Macaulay.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <def>The right to enter upon the possession of the property of an ancestor, or one near of kin, or one preceding in an established order.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>6.</sn> <def>The person succeeding to rank or office; a successor or heir.</def> <mark>[R.]</mark>  <rj><au>Milton.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Apostolical succession</b></col>. <fld>(Theol.)</fld> <cd>See under <er>Apostolical</er>.</cd> -- <col><b>Succession duty</b></col>, <cd>a tax imposed on every succession to property, according to its value and the relation of the person who succeeds to the previous owner.</cd><-- = death duties? --> <mark>[Eng.]</mark> -- <col><b>Succession of crops</b></col>. <fld>(Agric.)</fld> <cd>See <cref>Rotation of crops</cref>, under <er>Rotation</er>.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Successional</ent><br/
<hw>Suc*ces"sion*al</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Of or pertaining to a succession; existing in a regular order; consecutive.</def> <ldquo/<xex>Successional</xex> teeth.<rdquo/ <au>Flower.</au> -- <wordforms><wf>Suc*ces"sion*al*ly</wf>, <pos>adv.</pos></wordforms><br/
[<source>1913 Webster</source>]</p>

<p><ent>Successionist</ent><br/
<hw>Suc*ces"sion*ist</hw>, <pos>n.</pos> <def>A person who insists on the importance of a regular succession of events, offices, etc.; especially <fld>(Eccl.)</fld>, one who insists that apostolic succession alone is valid.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Successive</ent><br/
<hw>Suc*ces"sive</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[Cf. F. <ets>successif</ets>. See <er>Succeed</er>.]</ety> <sn>1.</sn> <def>Following in order or in uninterrupted course; coming after without interruption or interval; following one after another in a line or series; consecutive; <as>as, the <ex>successive</ex> revolution of years; the <ex>successive</ex> kings of Egypt; <ex>successive</ex> strokes of a hammer.</as></def><br/
[<source>1913 Webster</source>]</p>

<p><q>Send the <qex>successive</qex> ills through ages down.</q> <rj><qau>Prior.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Having or giving the right of succeeding to an inheritance; inherited by succession; hereditary; <as>as, a <ex>successive</ex> title; a <ex>successive</ex> empire</as>.</def> <mark>[Obs.]</mark>  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Successive induction</b></col>. <fld>(Math.)</fld> <cd>See <er>Induction</er>, 5.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Successively</ent><br/
<hw>Suc*ces"sive*ly</hw>, <pos>adv.</pos> <def>In a successive manner.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The whiteness, at length, changed <qex>successively</qex> into blue, indigo, and violet.</q> <rj><qau>Sir I. Newton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Successiveness</ent><br/
<hw>Suc*ces"sive*ness</hw>, <pos>n.</pos> <def>The quality or state of being successive.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Successless</ent><br/
<hw>Suc*cess"less</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Having no success.</def><br/
[<source>1913 Webster</source>]</p>

<p><q><qex>Successless</qex> all her soft caresses prove.</q> <rj><qau>Pope.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p>-- <wordforms><wf>Suc*cess"less*ly</wf>, <pos>adv.</pos> -- <wf>Suc*cess"less*ness</wf>, <pos>n.</pos></wordforms><br/
[<source>1913 Webster</source>]</p>

<p><ent>Successor</ent><br/
<hw>Suc*ces"sor</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[OE. <ets>successour</ets>, OF. <ets>successur</ets>, <ets>successor</ets>, F. <ets>successeur</ets>, L. <ets>successor</ets>. See <er>Succeed</er>.]</ety> <def>One who succeeds or follows; one who takes the place which another has left, and sustains the like part or character; -- correlative to <xex>predecessor</xex>; <as>as, the <ex>successor</ex> of a deceased king</as>.</def>  <rj><au>Chaucer.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>A gift to a corporation, either of lands or of chattels, without naming their <qex>successors</qex>, vests an absolute property in them so lond as the corporation subsists.</q> <rj><qau>Blackstone.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Succiduous</ent><br/
<hw>Suc*cid"u*ous</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>succiduus</ets>, fr. <ets>succidere</ets> to fall under.]</ety> <def>Ready to fall; falling.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Succiferous</ent><br/
<hw>Suc*cif"er*ous</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>succus</ets>, <ets>sucus</ets>, juice, sap + <ets>-ferous</ets>.]</ety> <def>Producing or conveying sap.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Succinamate</ent><br/
<hw>Suc`cin*am"ate</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Chem.)</fld> <def>A salt of succinamic acid.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Succinamic</ent><br/
<hw>Suc`cin*am"ic</hw> <pr>(?)</pr>, <pos>a.</pos> <fld>(Chem.)</fld> <def>Of, pertaining to, or designating, an acid amide derivative of succinic acid, obtained as a white crystalline substance, and forming a series of salts.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Succinate</ent><br/
<hw>Suc"ci*nate</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>succinum</ets>, <ets>sucinum</ets>, amber, from <ets>succus</ets>, <ets>sucus</ets>, juice, sap: cf. F. <ets>succinate</ets>.]</ety> <fld>(Chem.)</fld> <def>A salt of succinic acid.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Succinct</ent><br/
<hw>Suc*cinct"</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>succinctus</ets>, p. p. of <ets>succingere</ets> to gird below or from below, to tuck up; <ets>sub + cingere</ets> to gird.  Cf. <er>Cincture</er>.]</ety> <sn>1.</sn> <def>Girded or tucked up; bound; drawn tightly together.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>His habit fit for speed <qex>succinct</qex>.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Compressed into a narrow compass; brief; concise.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Let all your precepts be <qex>succinct</qex> and clear.</q> <rj><qau>Roscommon.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>The shortest and most <qex>succinct</qex> model that ever grasped all the needs and necessities of mankind.</q> <rj><qau>South.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><syn><b>Syn.</b> -- Short; brief; concise; summary; compendious; laconic; terse.</syn><br/
[<source>1913 Webster</source>]</p>

<p>-- <wordforms><wf>Suc*cinct"ly</wf>, <pos>adv.</pos> -- <wf>Suc*cinct"ness</wf>, <pos>n.</pos></wordforms><br/
[<source>1913 Webster</source>]</p>

<p><ent>Succinic</ent><br/
<hw>Suc*cin"ic</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[Cf. F. <ets>succinique</ets>. See <er>Succinate</er>.]</ety> <fld>(Chem.)</fld> <def>Pertaining to, or derived from, amber; specif., designating a dibasic acid, C<?/H<?/.(CO<?/H)<?/, first obtained by the dry distillation of amber. It is found in a number of plants, as in lettuce and wormwood, and is also produced artificially as a white crystalline substance having a slightly acid taste.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Succinimide</ent><br/
<hw>Suc`cin*im"ide</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Chem.)</fld> <def>A white crystalline nitrogenous substance, <chform>C2H4.(CO)2.NH</chform>, obtained by treating succinic anhydride with ammonia gas. It is a typical imido acid, and forms a series of salts. See <cref>Imido acid</cref>, under <er>Imido</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Succinite</ent><br/
<hw>Suc"ci*nite</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Cf. F. <ets>succinite</ets>.]</ety> <fld>(Min.)</fld> <sd>(a)</sd> <def>Amber.</def> <sd>(b)</sd> <def>A garnet of an amber color.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Succinous</ent><br/
<hw>Suc"ci*nous</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[From L. <ets>succinum</ets> amber.]</ety> <def>Succinic.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Succinurate</ent><br/
<hw>Suc`cin*u"rate</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Chem.)</fld> <def>A salt of succinuric acid.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Succinuric</ent><br/
<hw>Suc`cin*u"ric</hw> <pr>(?)</pr>, <pos>a.</pos> <fld>(Chem.)</fld> <def>Pertaining to, or designating, an acid amide, analogous to succinamic acid, which is obtained as a white crystalline substance by heating urea with succinic anhydride. It is known also in its salts.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Succinyl</ent><br/
<hw>Suc"cin*yl</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[<ets>Succin</ets>ic + <ets>-yl</ets>.]</ety> <fld>(Chem.)</fld> <def>A hypothetical radical characteristic of succinic acid and certain of its derivatives.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Succise</ent><br/
<hw>Suc*cise"</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[See <er>Succision</er>.]</ety> <fld>(Bot.)</fld> <def>Appearing as if a part were cut off at the extremity.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Succision</ent><br/
<hw>Suc*ci"sion</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>succisio</ets>, fr. <ets>succidere</ets>, <ets>succisum</ets>, to cut away below, <ets>sub</ets> under + <ets>caedere</ets> to cut.]</ety> <def>The act of cutting down, as of trees; the act of cutting off.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Succor</ent><br/
<hw>Suc"cor</hw> <pr>(?)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Succored</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Succoring</conjf>.]</vmorph> <ety>[OE. <ets>socouren</ets>, OF. <ets>sucurre</ets>, <ets>soucourre</ets>, <ets>secorre</ets>, F. <ets>secourir</ets>, L. <ets>succurrere</ets>, <ets>succursum</ets>, to run under, run to the aid of, help, succor; <ets>sub</ets> under + <ets>currere</ets> to run. See <er>Current</er>.]</ety> <def>To run to, or run to support; hence, to help or relieve when in difficulty, want, or distress; to assist and deliver from suffering; to relieve; <as>as, to <ex>succor</ex> a besieged city</as>.</def> <altsp>[Written also <asp>succour</asp>.]</altsp><br/
[<source>1913 Webster</source>]</p>

<p><q>He is able to <qex>succor</qex> them that are tempted.</q> <rj><qau>Heb. ii. 18.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><syn><b>Syn.</b> -- To aid; assist; relieve; deliver; help; comfort.</syn><br/
[<source>1913 Webster</source>]</p>

<p><ent>Succor</ent><br/
<hw>Suc"cor</hw>, <pos>n.</pos> <ety>[OE. <ets>socours</ets>, <ets>sucurs</ets>, OF. <ets>sucurs</ets>, <ets>socors</ets>, <ets>secors</ets>, F. <ets>secours</ets>, L. <ets>succursus</ets>, fr. L. <ets>succurrere</ets>. See <er>Succor</er>, <pos>v. t.</pos>]</ety> <sn>1.</sn> <def>Aid; help; assistance; esp., assistance that relieves and delivers from difficulty, want, or distress.</def> <ldquo/We beseech mercy and <xex>succor</xex>.<rdquo/  <rj><au>Chaucer.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>My noble father . . . <br/
Flying for <qex>succor</qex> to his servant Bannister.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>The person or thing that brings relief.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>This mighty <qex>succor</qex>, which made glad the foe.</q> <rj><qau>Dryden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Succorable</ent><br/
<hw>Suc"cor*a*ble</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Capable of being succored or assisted; admitting of relief.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Succorer</ent><br/
<hw>Suc"cor*er</hw> <pr>(?)</pr>, <pos>n.</pos> <def>One who affords succor; a helper.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Succorless</ent><br/
<hw>Suc"cor*less</hw>, <pos>a.</pos> <def>Destitute of succor.</def>  <rj><au>Thomson.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Succory</ent><br/
<hw>Suc"co*ry</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Corrupted from <ets>chicory</ets>.]</ety> <fld>(Bot.)</fld> <def>A plant of the genus <gen>Cichorium</gen>. See <er>Chicory</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Succotash</ent><br/
<hw>Suc"co*tash</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Narragansett Indian <ets>m'sickquatash</ets> corn boiled whole.]</ety> <def>Green maize and beans boiled together. The dish is borrowed from the native Indians.</def> <altsp>[Written also <asp>suckatash</asp>.]</altsp><br/
[<source>1913 Webster</source>]</p>

<p><ent>Succoteague</ent><br/
<hw>Suc`co*teague"</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Zool.)</fld> <def>The squeteague.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Succuba</ent><br/
||<hw>Suc"cu*ba</hw> <pr>(?)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Succubae</plw> <pr>(#)</pr>.</plu> <ety>[NL., fr. L. <ets>succubare</ets> to lie under; <ets>sub</ets> under + <ets>cubare</ets> to lie down; cf. L. <ets>succuba</ets>, <ets>succubo</ets>, one who lies under another.]</ety> <def>A female demon or fiend. See <er>Succubus</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Though seeming in shape a woman natural<br/
Was a fiend of the kind that <qex>succubae</qex> some call.</q> <rj><qau>Mir. for Mag.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Succubine</ent><br/
<hw>Suc"cu*bine</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Of or pertaining to succuba.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Succubous</ent><br/
<hw>Suc"cu*bous</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[See <er>Succuba</er>.]</ety> <fld>(Bot.)</fld> <def>Having the leaves so placed that the upper part of each one is covered by the base of the next higher leaf, as in hepatic mosses of the genus <gen>Plagiochila</gen>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Succubus</ent><br/
||<hw>Suc"cu*bus</hw> <pr>(?)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Succubi</plw> <pr>(#)</pr>.</plu> <ety>[See <er>Succuba</er>.]</ety> <sn>1.</sn> <def>A demon or fiend; especially, a lascivious spirit supposed to have sexual intercourse with the men by night; a succuba.  Cf. <er>Incubus</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Med.)</fld> <def>The nightmare. See <er>Nightmare</er>, 2.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Succula</ent><br/
<hw>Suc"cu*la</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>sucula</ets> a winch, windlass, capstan.]</ety> <fld>(Mach.)</fld> <def>A bare axis or cylinder with staves or levers in it to turn it round, but without any drum.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Succulency</ent><br/
<ent>Succulence</ent><br/
<mhw>{ <hw>Suc"cu*lence</hw> <pr>(?)</pr>, <hw>Suc"cu*len*cy</hw> <pr>(?)</pr>, }</mhw> <pos>n.</pos> <ety>[See <er>Succulent</er>.]</ety> <def>The quality or condition of being succulent; juiciness; <as>as, the <ex>succulence</ex> of a peach</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Succulent</ent><br/
<hw>Suc"cu*lent</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>succulentus</ets>, <ets>suculentus</ets>, fr. <ets>succus</ets>, <ets>sucus</ets>, juice; perhaps akin to E. <ets>suck</ets>: cf. F. <ets>succulent</ets>.]</ety> <def>Full of juice; juicy.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Succulent</b></col><cd> plants <fld>(Bot.)</fld>, plants which have soft and juicy leaves or stems, as the houseleek, the live forever, and the species of Mesembryanthemum.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Succulently</ent><br/
<hw>Suc"cu*lent*ly</hw>, <pos>adv.</pos> <def>In a succulent manner.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Succulous</ent><br/
<hw>Suc"cu*lous</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Succulent; juicy.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Succumb</ent><br/
<hw>Suc*cumb"</hw> <pr>(?)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Succumbed</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Succumbing</conjf>.]</vmorph> <ety>[L. <ets>succumbere</ets>; <ets>sub</ets> under + <ets>cumbere</ets> (in comp.), akin to <ets>cubare</ets> to lie down. See <er>Incumbent</er>, <er>Cubit</er>.]</ety> <def>To yield; to submit; to give up unresistingly; <as>as, to <ex>succumb</ex> under calamities; to <ex>succumb</ex> to disease.</as></def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Succumbent</ent><br/
<hw>Suc*cum"bent</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>succumbens</ets>, p. pr.]</ety> <def>Submissive; yielding.</def> <mark>[R.]</mark>  <rj><au>Howell.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Succursal</ent><br/
<hw>Suc*cur"sal</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[Cf. F. <ets>succursale</ets>. See <er>Succor</er>, <pos>n.</pos> & <pos>v. t.</pos>]</ety> <def>Serving to aid or help; serving as a chapel of ease; tributary.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>Not a city was without its cathedral, surrounded by its <qex>succursal</qex> churches, its monasteries, and convents.</q> <rj><qau>Milman.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Succus</ent><br/
||<hw>Suc"cus</hw> <pr>(?)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Succi</plw> <pr>(<?/)</pr>.</plu> <fld>(Med.)</fld> <def>The expressed juice of a plant, for medicinal use.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs>||<col><b>Succus entericus</b></col> <pr>(<?/)</pr>. <ety>[NL., literally, juice of the intestines.]</ety> <fld>(Physiol.)</fld> <cd>A fluid secreted in small by certain glands (probably the glands of Lieberk<uum/hn) of the small intestines. Its exact action is somewhat doubtful.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Succussation</ent><br/
<hw>Suc`cus*sa"tion</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>succussare</ets> to jolt, v. intens. fr. <ets>succutere</ets>, <ets>succussum</ets>, to fling up from below, to toss up; <ets>sub</ets> under + <ets>quatere</ets> to shake.]</ety> <sn>1.</sn> <def>A trot or trotting.</def> <mark>[Obs.]</mark>  <rj><au>Sir T. Browne.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A shaking; succussion.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Succussion</ent><br/
<hw>Suc*cus"sion</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>succussio</ets>, from <ets>succutere</ets>: cf. F. <ets>succussion</ets>. See <er>Succussation</er>.]</ety> <def>The act of shaking; a shake; esp. <fld>(Med.)</fld>, a shaking of the body to ascertain if there be a liquid in the thorax.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Succussive</ent><br/
<hw>Suc*cus"sive</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Characterized by a shaking motion, especially an up and down movement, and not merely tremulous oscillation; <as>as, the <ex>succussive</ex> motion in earthquakes</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Such</ent><br/
<hw>Such</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[OE. <ets>such</ets>, <ets>sich</ets>, <ets>sech</ets>, <ets>sik</ets>, <ets>swich</ets>, <ets>swilch</ets>, <ets>swulch</ets>, <ets>swilc</ets>, <ets>swulc</ets>, AS. <ets>swelc</ets>, <ets>swilc</ets>, <ets>swylc</ets>; akin to OFries. <ets>selik</ets>, D. <ets>zulk</ets>, OS. <ets>sulic</ets>, OHG. <ets>sulih</ets>, <ets>solih</ets>, G. <ets>solch</ets>, Icel. <ets>sl<imac/kr</ets>, OSw. <ets>salik</ets>, Sw. <ets>slik</ets>, Dan. <ets>slig</ets>, Goth. <ets>swaleiks</ets>; originally meaning, so shaped. <root/192. See <er>So</er>, <er>Like</er>, <pos>a.</pos>, and cf. <er>Which</er>.]</ety> <sn>1.</sn> <def>Of that kind; of the like kind; like; resembling; similar; <as>as, we never saw <ex>such</ex> a day</as>; -- followed by <xex>that</xex> or <xex>as</xex> introducing the word or proposition which defines the similarity, or the standard of comparison; <as>as, the books are not <ex>such</ex> that I can recommend them, or, not <ex>such</ex> as I can recommend; these apples are not <ex>such</ex> as those we saw yesterday; give your children <ex>such</ex> precepts as tend to make them better</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>And in his time <qex>such</qex> a conqueror<br/
That greater was there none under the sun.</q> <rj><qau>Chaucer.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>His misery was <qex>such</qex> that none of the bystanders could refrain from weeping.</q> <rj><qau>Macaulay.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ The indefinite article <xex>a</xex> or <xex>an</xex> never precedes <xex>such</xex>, but is placed between it and the noun to which it refers; as, <xex>such a</xex> man; <xex>such an</xex> honor. The indefinite adjective <xex>some</xex>, <xex>several</xex>, <xex>one</xex>, <xex>few</xex>, <xex>many</xex>, <xex>all</xex>, etc., precede <xex>such</xex>; as, one <xex>such</xex> book is enough; all <xex>such</xex> people ought to be avoided; few <xex>such</xex> ideas were then held.</note><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Having the particular quality or character specified.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>That thou art happy, owe to God;<br/
That thou continuest <qex>such</qex>, owe to thyself.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>The same that; -- with <xex>as</xex>; <as>as, this was the state of the kingdom at <ex>such</ex> time as the enemy landed</as>.</def> <ldquo/[It] hath <xex>such</xex> senses as we have.<rdquo/  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>Certain; -- representing the object as already particularized in terms which are not mentioned.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>In rushed one and tells him <qex>such</qex> a knight<br/
Is new arrived.</q> <rj><qau>Daniel.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>To-day or to-morrow we will go into <qex>such</qex> a city, and continue there a year.</q> <rj><qau>James iv. 13.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ <xex>Such</xex> is used pronominally. <ldquo/He was the father of <xex>such</xex> as dwell in tents.<rdquo/ <au>Gen. iv. 20.</au> <ldquo/<xex>Such</xex> as I are free in spirit when our limbs are chained.<rdquo/ <au>Sir W. Scott.</au> <xex>Such</xex> is also used before adjectives joined to substantives; as, the fleet encountered <xex>such</xex> a terrible storm that it put back. <ldquo/Everything was managed with so much care, and <xex>such</xex> excellent order was observed.<rdquo/ <au>De Foe.</au><br/
[<source>1913 Webster</source>]</p>

<p><q>Temple sprung from a family which . . . long after his death produced so many eminent men, and formed <qex>such</qex> distinguished alliances, that, etc.</q> <rj><qau>Macaulay.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><xex>Such</xex> is used emphatically, without the correlative.<br/
[<source>1913 Webster</source>]</p>

<p><q>Now will he be mocking:<br/
I shall have <qex>such</qex> a life.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><xex>Such</xex> was formerly used with numerals in the sense of <xex>times as much</xex> or <xex>as many</xex>; as, <xex>such ten</xex>, or ten times as many.</note><br/
[<source>1913 Webster</source>]</p>

<p><cs><mcol><col><b>Such and such</b></col>, <it>or</it> <col><b>Such or such</b></col></mcol>, <cd>certain; some; -- used to represent the object indefinitely, as already particularized in one way or another, or as being of one kind or another.</cd> <ldquo/In <xex>such and such</xex> a place shall be my camp.<rdquo/ <au>2 Kings vi. 8.</au> <ldquo/Sovereign authority may enact a law commanding <xex>such and such</xex> an action.<rdquo/ <au>South.</au> -- <mcol><col><b>Such like</b></col> <it>or</it> <col><b>Such character</b></col></mcol>, <cd>of the like kind.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><q>And many other <qex>such like</qex> things ye do.</q> <rj><qau>Mark vii. 8.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Suchospondylous</ent><br/
<hw>Su`cho*spon"dy*lous</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[Gr. <?/ a crocodile + <?/ a vertebra.]</ety> <fld>(Zool.)</fld> <def>Having dorsal vertebrae with long and divided transverse processes; -- applied to certain reptiles.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Suchwise</ent><br/
<hw>Such"wise`</hw> <pr>(?)</pr>, <pos>adv.</pos> <def>In a such a manner; so.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Suck</ent><br/
<hw>Suck</hw> <pr>(s<ucr/k)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Sucked</conjf> <pr>(s<ucr/kt)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Sucking</conjf>.]</vmorph> <ety>[OE. <ets>suken</ets>, <ets>souken</ets>, AS. <ets>s<umac/can</ets>, <ets>s<umac/gan</ets>; akin to D. <ets>zuigen</ets>, G. <ets>saugen</ets>, OHG. <ets>s<umac/gan</ets>, Icel. <ets>s<umac/ga</ets>, <ets>sj<umac/ga</ets>, Sw. <ets>suga</ets>, Dan. <ets>suge</ets>, L. <ets>sugere</ets>.  Cf. <er>Honeysuckle</er>, <er>Soak</er>, <er>Succulent</er>, <er>Suction</er>.]</ety> <sn>1.</sn> <def>To draw, as a liquid, by the action of the mouth and tongue, which tends to produce a vacuum, and causes the liquid to rush in by atmospheric pressure; to draw, or apply force to, by exhausting the air.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To draw liquid from by the action of the mouth; <as>as, to <ex>suck</ex> an orange</as>; specifically, to draw milk from (the mother, the breast, etc.) with the mouth; <as>as, the young of an animal <ex>sucks</ex> the mother, or dam; an infant <ex>sucks</ex> the breast</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>To draw in, or imbibe, by any process resembles sucking; to inhale; to absorb; <as>as, to <ex>suck</ex> in air; the roots of plants <ex>suck</ex> water from the ground</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>To draw or drain.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Old ocean, <qex>sucked</qex> through the porous globe.</q> <rj><qau>Thomson.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <def>To draw in, as a whirlpool; to swallow up.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>As waters are by whirlpools <qex>sucked</qex> and drawn.</q> <rj><qau>Dryden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>To suck in</b></col>, <cd>to draw into the mouth; to imbibe; to absorb.</cd> -- <col><b>To suck out</b></col>, <cd>to draw out with the mouth; to empty by suction.</cd> -- <col><b>To suck up</b></col>, <cd>to draw into the mouth; to draw up by suction or absorption.</cd></cs>
<-- suck up (b) to be obsequious; to act as a toady. --><br/
[<source>1913 Webster</source>]</p>

<p><ent>Suck</ent><br/
<hw>Suck</hw>, <pos>v. i.</pos> <sn>1.</sn> <def>To draw, or attempt to draw, something by suction, as with the mouth, or through a tube.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Where the bee <qex>sucks</qex>, there <qex>suck</qex> I.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To draw milk from the breast or udder; <as>as, a child, or the young of an animal, is first nourished by <ex>sucking</ex></as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>To draw in; to imbibe; to partake.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The crown had <qex>sucked</qex> too hard, and now, being full, was like to draw less.</q> <rj><qau>Bacon.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>To be objectionable, of very poor quality, or offensive; <as>as, telemarketing calls really <ex>suck</ex>; he's a good actor, but his singing <ex>sucks</ex></as>.</def> <mark>[Colloq.]</mark><br/
[<source>PJC</source>]</p>

<p><ent>Suck</ent><br/
<hw>Suck</hw>, <pos>n.</pos> <sn>1.</sn> <def>The act of drawing with the mouth.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>That which is drawn into the mouth by sucking; specifically, mikl drawn from the breast.</def>  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>A small draught.</def> <mark>[Colloq.]</mark>  <rj><au>Massinger.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>Juice; succulence.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Suckanhock</ent><br/
<hw>Suck"an*hock</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Of American Indian origin.]</ety> <def>A kind of seawan. See Note under <er>Seawan</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Suckatash</ent><br/
<hw>Suck"a*tash</hw> <pr>(?)</pr>, <pos>n.</pos> <def>See <er>Succotash</er>.</def>  <rj><au>Bartlett.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sucken</ent><br/
<hw>Suck"en</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[See <er>Socome</er>, <er>Soc</er>.]</ety> <fld>(Scots Law)</fld> <def>The jurisdiction of a mill, or that extent of ground astricted to it, the tenants of which are bound to bring their grain thither to be ground.</def><br/
[<source>1913 Webster</source>]</p>

<p><-- p. 1440 --></p>

<p><ent>Sucker</ent><br/
<hw>Suck"er</hw> <pr>(s<ucr/k"<etil/r)</pr>, <pos>n.</pos> <sn>1.</sn> <def>One who, or that which, sucks; esp., one of the organs by which certain animals, as the octopus and remora, adhere to other bodies.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A suckling; a sucking animal.</def>  <rj><au>Beau. & Fl.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>The embolus, or bucket, of a pump; also, the valve of a pump basket.</def>  <rj><au>Boyle.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>A pipe through which anything is drawn.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <def>A small piece of leather, usually round, having a string attached to the center, which, when saturated with water and pressed upon a stone or other body having a smooth surface, adheres, by reason of the atmospheric pressure, with such force as to enable a considerable weight to be thus lifted by the string; -- used by children as a plaything.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>6.</sn> <fld>(Bot.)</fld> <def>A shoot from the roots or lower part of the stem of a plant; -- so called, perhaps, from diverting nourishment from the body of the plant.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>7.</sn> <fld>(Zool.)</fld> <sd>(a)</sd> <def>Any one of numerous species of North American fresh-water cyprinoid fishes of the family <fam>Catostomidae</fam>; so called because the lips are protrusile. The flesh is coarse, and they are of little value as food. The most common species of the Eastern United States are the northern sucker (<spn>Catostomus Commersoni</spn>), the white sucker (<spn>Catostomus teres</spn>), the hog sucker (<spn>Catostomus nigricans</spn>), and the chub, or sweet sucker (<spn>Erimyzon sucetta</spn>).  Some of the large Western species are called <stype>buffalo fish</stype>, <stype>red horse</stype>, <stype>black horse</stype>, and <stype>suckerel</stype>.</def> <sd>(b)</sd> <def>The remora.</def> <sd>(c)</sd> <def>The lumpfish.</def> <sd>(d)</sd> <def>The hagfish, or myxine.</def> <sd>(e)</sd> <def>A California food fish (<spn>Menticirrus undulatus</spn>) closely allied to the kingfish <sd>(a)</sd>; -- called also <altname>bagre</altname>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>8.</sn> <def>A parasite; a sponger. See def. 6, above.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>They who constantly converse with men far above their estates shall reap shame and loss thereby; if thou payest nothing, they will count thee a <qex>sucker</qex>, no branch.</q> <rj><qau>Fuller.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>9.</sn> <def>A hard drinker; a soaker.</def> <mark>[Slang]</mark><br/
[<source>1913 Webster</source>]</p>

<p><sn>10.</sn> <def>A greenhorn; someone easily cheated, gulled, or deceived.</def> <mark>[Slang, U.S.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><sn>11.</sn> <def>A nickname applied to a native of Illinois.</def> <mark>[U. S.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><sn>12.</sn> <def>A person strongly attracted to something; -- usually used with <ptcl>for</ptcl>; <as>as, he's a <ex>sucker</ex> for tall blondes</as>.</def><br/
[<source>PJC</source>]</p>

<p><sn>11.</sn> <def>Any thing or person; -- usually implying annoyance or dislike; <as>as, I went to change the blade and cut my finger on the <ex>sucker</ex></as>.</def> <mark>[Slang]</mark><br/
[<source>PJC</source>]</p>

<p><cs><mcol><col><b>Carp sucker</b></col>, <col><b>Cherry sucker</b></col></mcol>, <cd>etc. See under <er>Carp</er>, <er>Cherry</er>, etc.</cd> -- <col><b>Sucker fish</b></col>. <cd>See <cref>Sucking fish</cref>, under <er>Sucking</er>.</cd> -- <col><b>Sucker rod</b></col>, <cd>a pump rod. See under <er>Pump</er>.</cd> -- <col><b>Sucker tube</b></col> <fld>(Zool.)</fld>, <cd>one of the external ambulacral tubes of an echinoderm, -- usually terminated by a sucker and used for locomotion. Called also <altname>sucker foot</altname>. See <er>Spatangoid</er>.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sucker</ent><br/
<hw>Suck"er</hw> <pr>(?)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Suckered</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Suckering</conjf>.]</vmorph> <sn>1.</sn> <def>To strip off the suckers or shoots from; to deprive of suckers; <as>as, to <ex>sucker</ex> maize</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To cheat or deceive (a gullible person); to make a sucker of (someone).</def><br/
[<source>PJC</source>]</p>

<p><ent>Sucker</ent><br/
<hw>Suck"er</hw>, <pos>v. i.</pos> <def>To form suckers; <as>as, corn <ex>suckers</ex> abundantly</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sucker State</ent><br/
<hw>Suck"er State</hw>. <def>Illinois; -- a nickname.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Sucket</ent><br/
<hw>Suck"et</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Cf. <er>Suck</er>, <pos>v. t.</pos>, <er>Succades</er>.]</ety> <def>A sweetmeat; a dainty morsel.</def>  <rj><au>Jer. Taylor.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>suck face</ent><br/
<hw>suck face</hw>, <pos>v. i.</pos> <def>To engage in french kissing (soul-kissing).</def> <mark>[slang]</mark><br/
[<source>PJC</source>]</p>

<p><ent>Suckfish</ent><br/
<hw>Suck"fish`</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Zool.)</fld> <def>A sucker fish.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sucking</ent><br/
<hw>Suck"ing</hw>, <pos>a.</pos> <def>Drawing milk from the mother or dam; hence, colloquially, young, inexperienced, <as>as, a <ex>sucking</ex> infant; a <ex>sucking</ex> calf</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>I suppose you are a young barrister, <qex>sucking</qex> lawyer, or that sort of thing.</q> <rj><qau>Thackeray.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Sucking bottle</b></col>, <cd>a feeding bottle. See under <er>Bottle</er>.</cd> -- <col><b>Sucking fish</b></col> <fld>(Zool.)</fld>, <cd>the remora. See <er>Remora</er>.</cd> <au>Baird.</au> -- <col><b>Sucking pump</b></col>, <cd>a suction pump. See under <er>Suction</er>.</cd> -- <col><b>Sucking stomach</b></col> <fld>(Zool.)</fld>, <cd>the muscular first stomach of certain insects and other invertebrates which suck liquid food.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Suckle</ent><br/
<hw>Suc"kle</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A teat.</def> <mark>[Obs.]</mark>  <rj><au>Sir T. Herbert.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Suckle</ent><br/
<hw>Suc"kle</hw>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Suckled</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Suckling</conjf> <pr>(?)</pr>.]</vmorph> <ety>[Freq. of <ets>suck</ets>.]</ety> <def>To give suck to; to nurse at the breast.</def>  <rj><au>Addison.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>The breasts of Hecuba<br/
When she did <qex>suckle</qex> Hector, looked not lovelier.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>They are not weak, <qex>suckled</qex> by Wisdom.</q> <rj><qau>Landor.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Suckle</ent><br/
<hw>Suc"kle</hw>, <pos>v. i.</pos> <def>To nurse; to suck.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Suckler</ent><br/
<hw>Suc"kler</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Zool.)</fld> <def>An animal that suckles its young; a mammal.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Suckling</ent><br/
<hw>Suck"ling</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[OE. <ets>sokeling</ets>. See <er>Suck</er>, <pos>v. t.</pos>]</ety> <sn>1.</sn> <def>A young child or animal nursed at the breast.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A small kind of yellow clover (<spn>Trifolium filiforme</spn>) common in Southern Europe.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sucrate</ent><br/
<hw>Su"crate</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Chem.)</fld> <def>A compound of sucrose (or of some related carbohydrate) with some base, after the analogy of a salt; <as>as, sodium <ex>sucrate</ex></as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sucre</ent><br/
||<hw>Su"cre</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A silver coin of Ecuador, worth 68 cents (ca. 1900).</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sucrose</ent><br/
<hw>Su"crose`</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F. <ets>sucre</ets> sugar. See <er>Sugar</er>.]</ety> <fld>(Chem.)</fld> <def>A common variety of sugar found in the juices of many plants, as the sugar cane, sorghum, sugar maple, beet root, etc. It is extracted as a sweet, white crystalline substance which is valuable as a food product, and, being antiputrescent, is largely used in the preservation of fruit. Called also <altname>saccharose</altname>, <altname>cane sugar</altname>, etc.  At one time the term was used by extension, for any one of the class of isomeric substances (as <xex>lactose</xex>, <xex>maltose</xex>, etc.) of which sucrose proper is the type; however this usage is now archaic.</def><br/
[<source>1913 Webster</source> <source>+PJC</source>]</p>

<p><note><hand/ Sucrose proper is a dextrorotatory carbohydrate, <chform>C12H22O11</chform>. It does not reduce Fehling's solution, and though not directly fermentable, yet on standing with yeast it is changed by the diastase present to invert sugar (<xex>dextrose</xex> and <xex>levulose</xex>), which then breaks down to alcohol and carbon dioxide. It is also decomposed to invert sugar by heating with acids, whence it is also called a <altname>disaccharate</altname><-- disaccharide-->. Sucrose possesses at once the properties of an alcohol and a ketone, and also forms compounds (called <xex>sucrates</xex>) analogous to salts.  Cf. <er>Sugar</er>.</note><br/
[<source>1913 Webster</source>]</p>

<p><ent>Suction</ent><br/
<hw>Suc"tion</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>sugere</ets>, <ets>suctum</ets>, to suck; cf. OF. <ets>suction</ets>. See <er>Suck</er>, <pos>v. t.</pos>]</ety> <def>The act or process of sucking; the act of drawing, as fluids, by exhausting the air.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Suction chamber</b></col>, <cd>the chamber of a pump into which the suction pipe delivers.</cd> -- <mcol><col><b>Suction pipe</b></col>, <col><b>Suction valve</b></col></mcol>, <cd>the induction pipe, and induction valve, of a pump, respectively.</cd> -- <col><b>Suction pump</b></col>, <cd>the common pump, in which the water is raised into the barrel by atmospheric pressure.  See <xex>Illust.</xex> of <er>Pump</er>.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Suctoria</ent><br/
||<hw>Suc*to"ri*a</hw> <pr>(?)</pr>, <pos>n. pl.</pos> <ety>[NL. See <er>Suction</er>.]</ety> <fld>(Zool.)</fld> <sn>1.</sn> <def>An order of Infusoria having the body armed with somewhat stiff, tubular processes which they use as suckers in obtaining their food. They are usually stalked.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Same as <er>Rhizocephala</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Suctorial</ent><br/
<hw>Suc*to"ri*al</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>sugere</ets>, <ets>suctum</ets>, to suck.]</ety> <sn>1.</sn> <fld>(Zool.)</fld> <def>Adapted for sucking; living by sucking; <as>as, the humming birds are <ex>suctorial</ex> birds</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Zool.)</fld> <def>Capable of adhering by suction; <as>as, the <ex>suctorial</ex> fishes</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Suctorian</ent><br/
<hw>Suc*to"ri*an</hw> <pr>(?)</pr>, <pos>n.</pos> <sn>1.</sn> <fld>(Zool.)</fld> <def>A cartilaginous fish with a mouth adapted for suction, as the lampery.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Zool.)</fld> <def>One of the Suctoria.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Suctorious</ent><br/
<hw>Suc*to"ri*ous</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Suctorial.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sudamina</ent><br/
||<hw>Su*dam"i*na</hw> <pr>(?)</pr>, <pos>n. pl.</pos>, <sing>sing. <singw>Sudamen</singw> <pr>(<?/)</pr></sing>. <ety>[NL. <ets>sudamen</ets>, <ets>-inis</ets>, fr. <ets>sudare</ets> to sweat. See <er>Sweat</er>.]</ety> <fld>(Med.)</fld> <def>Minute vesicles surrounded by an area of reddened skin, produced by excessive sweating.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sudarium</ent><br/
||<hw>Su*da"ri*um</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L., a handkerchief.]</ety> <fld>(Eccl.)</fld> <def>The handkerchief upon which the Savior is said to have impressed his own portrait miraculously, when wiping his face with it, as he passed to the crucifixion.</def><-- = Veronica's veil. --><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sudary</ent><br/
<hw>Su"da*ry</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>sudarium</ets>, fr. <ets>sudare</ets> to sweat. See <er>Sweat</er>.]</ety> <def>A napkin or handkerchief.</def> <mark>[Obs. or R.]</mark>  <rj><au>Wyclif. R. Browning.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sudation</ent><br/
<hw>Su*da"tion</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>sudatio</ets>, fr. <ets>sudare</ets> to sweat: cf. F. <ets>sudation</ets>.]</ety> <def>A sweating.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sudatorium</ent><br/
||<hw>Su`da*to"ri*um</hw> <pr>(?)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Sudatoria</plw> <pr>(#)</pr>.</plu> <ety>[L.]</ety> <def>A sudatory.</def>  <rj><au>Dunglison.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sudatory</ent><br/
<hw>Su"da*to*ry</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>sudatorius</ets>, fr. <ets>sudare</ets> to sweat: cf. F. <ets>sudatoire</ets>. See <er>Sweat</er>.]</ety> <def>Sweating; perspiring.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sudatory</ent><br/
<hw>Su"da*to*ry</hw>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Sudatories</plw> <pr>(#)</pr>.</plu> <ety>[L. <ets>sudatorium</ets>.]</ety> <def>A bagnio; a sweating bath; a vapor bath.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>These <qex>sudatories</qex> are much in request for many infirmities.</q> <rj><qau>Evelyn.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sudd</ent><br/
<hw>Sudd</hw> <pr>(s<ucr/d)</pr>, <pos>n.</pos> <ety>[Ar. <ets>sadd</ets> barrier.]</ety> <def>A tangled mass of floating vegetal matter obstructing navigation.</def> <mark>[Central Africa]</mark><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Sudden</ent><br/
<hw>Sud"den</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[OE. <ets>sodian</ets>, <ets>sodein</ets>, OF. <ets>sodain</ets>, <ets>sudain</ets>, F. <ets>soudain</ets>, L. <ets>subitaneus</ets>, fr. <ets>subitus</ets> sudden, that has come unexpectedly, p. p. of <ets>subire</ets> to come on, to steal upon; <ets>sub</ets> under, secretly + <ets>ire</ets> to go. See <er>Issue</er>, and cf. <er>Subitaneous</er>.]</ety> <sn>1.</sn> <def>Happening without previous notice or with very brief notice; coming unexpectedly, or without the common preparation; immediate; instant; speedy.</def> <ldquo/O <xex>sudden</xex> wo!<rdquo/ <au>Chaucer.</au> <ldquo/For fear of <xex>sudden</xex> death.<rdquo/ <au>Shak.</au><br/
[<source>1913 Webster</source>]</p>

<p><q><qex>Sudden</qex> fear troubleth thee.</q> <rj><qau>Job xxii. 10.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Hastly prepared or employed; quick; rapid.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Never was such a <qex>sudden</qex> scholar made.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>The apples of Asphaltis, appearing goodly to the <qex>sudden</qex> eye.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>Hasty; violent; rash; precipitate.</def> <mark>[Obs.]</mark>  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><syn><b>Syn.</b> -- Unexpected; unusual; abrupt; unlooked-for.</syn><br/
[<source>1913 Webster</source>]</p>

<p>-- <wordforms><wf>Sud"den*ly</wf>, <pos>adv.</pos> -- <wf>Sud"den*ness</wf>, <pos>n.</pos></wordforms><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sudden</ent><br/
<hw>Sud"den</hw>, <pos>adv.</pos> <def>Suddenly; unexpectedly.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>Herbs of every leaf that <qex>sudden</qex> flowered.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sudden</ent><br/
<hw>Sud"den</hw>, <pos>n.</pos> <def>An unexpected occurrence; a surprise.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><mcol><col><b>All of a sudden</b></col>, <col><b>On a sudden</b></col>, <col><b>Of a sudden</b></col></mcol>, <cd>sooner than was expected; without the usual preparation; suddenly.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><q>How art thou lost! how <qex>on a sudden</qex> lost!</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>He withdrew his opposition <qex>all of a sudden</qex>.</q> <rj><qau>Thackeray.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Suddenty</ent><br/
<hw>Sud"den*ty</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Cf. F. <ets>soudainet<eacute/</ets>.]</ety> <def>Suddenness; a sudden.</def> <mark>[Scot.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>On a suddenty</b></col>, <cd>on a sudden.</cd> <mark>[Scot.]</mark> <au>Sir W. Scott.</au></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sudoral</ent><br/
<hw>Su"dor*al</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>sudor</ets>.]</ety> <def>Of or pertaining to sweat; <as>as, <ex>sudoral</ex> eruptions</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sudoriferous</ent><br/
<hw>Su`dor*if"er*ous</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>sudor</ets> sweat + <ets>-ferous</ets>.]</ety> <fld>(Physiol.)</fld> <def>Producing, or secreting, sweat; sudoriparous.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Sudoriferous glands</b></col> <fld>(Anat.)</fld>, <cd>small convoluted tubular glands which are situated in the subcutaneous tissues and discharge by minute orifices in the surface of the skin; the sweat glands.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sudorific</ent><br/
<hw>Su`dor*if"ic</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>sudor</ets> sweat (akin to E. <ets>sweat</ets>) + <ets>facere</ets> to make.]</ety> <def>Causing sweat; <as>as, <ex>sudorific</ex> herbs</as>.</def> -- <def2><pos>n.</pos> <def>A sudorific medicine.  Cf. <er>Diaphoretic</er>.</def></def2><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sudoriparous</ent><br/
<hw>Su`dor*ip"a*rous</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>sudor</ets> sweat + <ets>parere</ets> to produce.]</ety> <fld>(Physiol.)</fld> <def>Same as <er>Sudoriferous</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sudorous</ent><br/
<hw>Su"dor*ous</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>sudorus</ets>, fr. <ets>sudor</ets> sweat.]</ety> <def>Consisting of sweat.</def> <mark>[Obs.]</mark>  <rj><au>Sir T. Browne.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sudra</ent><br/
||<hw>Su"dra</hw> <pr>(s<oomac/"dr<adot/)</pr>, <pos>n.</pos> <ety>[Skr. <ets><cced/<umac/dra</ets>.]</ety> <def>The lowest of the four great castes among the Hindoos. See <er>Caste</er>.</def> <altsp>[Written also <asp>Soorah</asp>, <asp>Soodra</asp>, and <asp>Sooder</asp>.]</altsp><br/
[<source>1913 Webster</source>]</p>

<p><ent>Suds</ent><br/
<hw>Suds</hw> <pr>(?)</pr>, <pos>n. pl.</pos> <ety>[Akin to <ets>sodden</ets>, <ets>seethe</ets>. See <er>Seethe</er>.]</ety> <def>Water impregnated with soap, esp. when worked up into bubbles and froth.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>In the suds</b></col>, <cd>in turmoil or difficulty.</cd> <mark>[Colloq.]</mark> <au>Beau. & Fl.</au></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sue</ent><br/
<hw>Sue</hw> <pr>(?)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Sued</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Suing</conjf> <pr>(?)</pr>.]</vmorph> <ety>[OE. <ets>suen</ets>, <ets>sewen</ets>, <ets>siwen</ets>, OF. <ets>sivre</ets> (pres.ind. 3d sing. il <ets>siut</ets>, <ets>suit</ets>, he follows, nous <ets>sevons</ets> we follow), LL. <ets>sequere</ets>, for L. <ets>sequi</ets>, <ets>secutus</ets>; akin to Gr. <?/, Skr. <ets>sac</ets> to accompany, and probably to E. <ets>see</ets>, v.t. See <er>See</er>, <pos>v. t.</pos>, and cf. <er>Consequence</er>, <er>Ensue</er>, <er>Execute</er>, <er>Obsequious</er>, <er>Pursue</er>, <er>Second</er>, <er>Sect</er> in religion, <er>Sequence</er>, <er>Suit</er>.]</ety> <sn>1.</sn> <def>To follow up; to chase; to seek after; to endeavor to win; to woo.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>For yet there was no man that haddle him <qex>sued</qex>.</q> <rj><qau>Chaucer.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>I was beloved of many a gentle knight,<br/
And <qex>sued</qex> and sought with all the service due.</q> <rj><qau>Spenser.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q><qex>Sue</qex> me, and woo me, and flatter me.</q> <rj><qau>Tennyson.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Law)</fld> <sd>(a)</sd> <def>To seek justice or right from, by legal process; to institute process in law against; to bring an action against; to prosecute judicially.</def> <sd>(b)</sd> <def>To proceed with, as an action, and follow it up to its proper termination; to gain by legal process.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <fld>(Falconry)</fld> <def>To clean, as the beak; -- said of a hawk.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <fld>(Naut.)</fld> <def>To leave high and dry on shore; <as>as, to <ex>sue</ex> a ship</as>.</def>  <rj><au>R. H. Dana, Jr.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>To sue out</b></col> <fld>(Law)</fld>, <cd>to petition for and take out, or to apply for and obtain; <as>as, <ex>to sue out</ex> a writ in chancery; <ex>to sue out</ex> a pardon for a criminal</as>.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sue</ent><br/
<hw>Sue</hw> <pr>(?)</pr>, <pos>v. i.</pos> <sn>1.</sn> <def>To seek by request; to make application; to petition; to entreat; to plead.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>By adverse destiny constrained to <qex>sue</qex><br/
For counsel and redress, he <qex>sues</qex> to you.</q> <rj><qau>Pope.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Caesar came to Rome to <qex>sue</qex> for the double honor of a triumph and the consulship.</q> <rj><qau>C. Middleton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>The Indians were defeated and <qex>sued</qex> for peace.</q> <rj><qau>Jefferson.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Law)</fld> <def>To prosecute; to make legal claim; to seek (for something) in law; <as>as, to <ex>sue</ex> for damages</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>To woo; to pay addresses as a lover.</def>  <rj><au>Massinger.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <fld>(Naut.)</fld> <def>To be left high and dry on the shore, as a ship.</def>  <rj><au>R. H. Dana, Jr.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Suede</ent><br/
<mhw><hw>Suede</hw>, <hw>Su<egrave/de</hw></mhw> <pr>(sw<amac/d, sw<asl/d, <it>or</it> sw<acir/d)</pr>, <pos>n.</pos> <ety>[F., Sweden.]</ety> <def>Swedish glove leather, -- usually made from lambskins tanned with willow bark. Also used adjectively; <as>as, <ex>suede</ex> gloves; blue <ex>suede</ex> shoes</as>.</def><br/
[<source>Webster 1913 Suppl.</source> + <source>PJC</source>]</p>

<p><ent>Suent</ent><br/
<hw>Su"ent</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Uniformly or evenly distributed or spread; even; smooth. See <er>Suant</er>.</def>  <rj><au>Thoreau.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Suently</ent><br/
<hw>Su"ent*ly</hw>, <pos>adv.</pos> <def>Evenly; smoothly.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Suer</ent><br/
<hw>Su"er</hw> <pr>(?)</pr>, <pos>n.</pos> <def>One who sues; a suitor.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Suet</ent><br/
<hw>Su"et</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[OE. <ets>suet</ets>, dim. fr. OF. <ets>seu</ets>, <ets>suif</ets>, F. <ets>suif</ets>, L. <ets>sebum</ets>.  Cf. <er>Soap</er>, <er>Sebaceous</er>.]</ety> <def>The fat and fatty tissues of an animal, especially the harder fat about the kidneys and loins in beef and mutton, which, when melted and freed from the membranes, forms tallow.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Suety</ent><br/
<hw>Su"et*y</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Consisting of, or resembling, suet; <as>as, a <ex>suety</ex> substance</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Suf-</ent><br/
<hw>Suf-</hw> <pr>(?)</pr>. <def>A form of the prefix <er>Sub-</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Suffer</ent><br/
<hw>Suf"fer</hw> <pr>(?)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Suffered</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Suffering</conjf>.]</vmorph> <ety>[OE. <ets>suffren</ets>, <ets>soffren</ets>, OF. <ets>sufrir</ets>, <ets>sofrir</ets>, F. <ets>souffrir</ets>, (assumed) LL. <ets>sofferire</ets>, for L. <ets>sufferre</ets>; <ets>sub</ets> under + <ets>ferre</ets> to bear, akin to E. <ets>bear</ets>. See <er>Bear</er> to support.]</ety> <sn>1.</sn> <def>To feel, or endure, with pain, annoyance, etc.; to submit to with distress or grief; to undergo; <as>as, to <ex>suffer</ex> pain of body, or grief of mind</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To endure or undergo without sinking; to support; to sustain; to bear up under.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Our spirit and strength entire,<br/
Strongly to <qex>suffer</qex> and support our pains.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>To undergo; to be affected by; to sustain; to experience; <as>as, most substances <ex>suffer</ex> a change when long exposed to air and moisture; to <ex>suffer</ex> loss or damage.</as></def><br/
[<source>1913 Webster</source>]</p>

<p><q>If your more ponderous and settled project<br/
May <qex>suffer</qex> alteration.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>To allow; to permit; not to forbid or hinder; to tolerate.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Thou shalt in any wise rebuke thy neighbour, and not <qex>suffer</qex> sin upon him.</q> <rj><qau>Lev. xix. 17.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>I <qex>suffer</qex> them to enter and possess.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><syn><b>Syn.</b> -- To permit; bear; endure; support; sustain; allow; admit; tolerate. See <er>Permit</er>.</syn><br/
[<source>1913 Webster</source>]</p>

<p><ent>Suffer</ent><br/
<hw>Suf"fer</hw>, <pos>v. i.</pos> <sn>1.</sn> <def>To feel or undergo pain of body or mind; to bear what is inconvenient; <as>as, we <ex>suffer</ex> from pain, sickness, or sorrow; we <ex>suffer</ex> with anxiety</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>O well for him whose will is strong!<br/
He <qex>suffers</qex>, but he will not <qex>suffer</qex> long.</q> <rj><qau>Tennyson.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To undergo punishment; specifically, to undergo the penalty of death.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The father was first condemned to <qex>suffer</qex> upon a day appointed, and the son afterwards the day following.</q> <rj><qau>Clarendon.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>To be injured; to sustain loss or damage.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Public business <qex>suffers</qex> by private infirmities.</q> <rj><qau>Sir W. Temple.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sufferable</ent><br/
<hw>Suf"fer*a*ble</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[Cf. F. <ets>souffrable</ets>.]</ety> <sn>1.</sn> <def>Able to suffer or endure; patient.</def> <mark>[Obs.]</mark> <ldquo/Ye must be <xex>sufferable</xex>.<rdquo/  <rj><au>Chaucer.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>That may be suffered, tolerated, or permitted; allowable; tolerable.</def><br/
[<source>1913 Webster</source>]</p>

<p>-- <wordforms><wf>Suf"fer*a*ble*ness</wf>, <pos>n.</pos> -- <wf>Suf"fer*a*bly</wf>, <pos>adv.</pos></wordforms><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sufferance</ent><br/
<hw>Suf"fer*ance</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[OE. <ets>suffrance</ets>, OF. <ets>sufrance</ets>, <ets>soufrance</ets>, F. <ets>souffrance</ets>, L. <ets>sufferentia</ets>, from <ets>sufferens</ets>, <ets>-entis</ets>, p. pr. of <ets>sufferre</ets>. See <er>Suffer</er>.]</ety> <sn>1.</sn> <def>The state of suffering; the bearing of pain; endurance.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>He must not only die the death,<br/
But thy unkindness shall his death draw out<br/
To lingering <qex>sufferance</qex>.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Pain endured; misery; suffering; distress.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The seeming <qex>sufferances</qex> that you had borne.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>Loss; damage; injury.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>A grievous . . . <qex>sufferance</qex> on most part of their fleet.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>Submission under difficult or oppressive circumstances; patience; moderation.</def>  <rj><au>Chaucer.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>But hasty heat tempering with <qex>sufferance</qex> wise.</q> <rj><qau>Spenser.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <def>Negative consent by not forbidding or hindering; toleration; permission; allowance; leave.</def>  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>In their beginning they are weak and wan,<br/
But soon, through <qex>sufferance</qex>, grow to fearful end.</q> <rj><qau>Spenser.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Somewhiles by <qex>sufferance</qex>, and somewhiles by special leave and favor, they erected to themselves oratories.</q> <rj><qau>Hooker.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>6.</sn> <def>A permission granted by the customs authorities for the shipment of goods.</def> <mark>[Eng.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Estate of sufferance</b></col> <fld>(Law)</fld>, <cd>the holding by a tenant who came in by a lawful title, but remains, after his right has expired, without positive leave of the owner.</cd> <au>Blackstone.</au> -- <col><b>On sufferance</b></col>, <cd>by mere toleration; <as>as, to remain in a house <ex>on sufferance</ex></as>.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><syn><b>Syn.</b> -- Endurance; pain; misery; inconvenience; patience; moderation; toleration; permission.</syn><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sufferer</ent><br/
<hw>Suf"fer*er</hw> <pr>(?)</pr>, <pos>n.</pos> <sn>1.</sn> <def>One who suffers; one who endures or undergoes suffering; one who sustains inconvenience or loss; <as>as, <ex>sufferers</ex> by poverty or sickness; men are <ex>sufferers</ex> by fire or by losses at sea.</as></def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>One who permits or allows.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Suffering</ent><br/
<hw>Suf"fer*ing</hw>, <pos>n.</pos> <def>The bearing of pain, inconvenience, or loss; pain endured; distress, loss, or injury incurred; <as>as, <ex>sufferings</ex> by pain or sorrow; <ex>sufferings</ex> by want or by wrongs.</as></def> <ldquo/Souls in <xex>sufferings</xex> tried.<rdquo/  <rj><au>Keble.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Suffering</ent><br/
<hw>Suf"fer*ing</hw>, <pos>a.</pos> <def>Being in pain or grief; having loss, injury, distress, etc.</def> -- <wordforms><wf>Suf"fer*ing*ly</wf>, <pos>adv.</pos></wordforms><br/
[<source>1913 Webster</source>]</p>

<p><-- p. 1441 --></p>

<p><ent>Suffice</ent><br/
<hw>Suf*fice"</hw> <pr>(?)</pr>, <pos>v. i.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Sufficed</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Sufficing</conjf> <pr>(?)</pr>.]</vmorph> <ety>[OE. <ets>suffisen</ets>, OF. <ets>soufire</ets>, F. <ets>suffire</ets> (cf. <ets>suffisant</ets>, p. pr.), L. <ets>sufficere</ets> to put under, to substitute, to avail for, to suffice; <ets>sub</ets> under + <ets>facere</ets> to make. See <er>Fact</er>.]</ety> <def>To be enough, or sufficient; to meet the need (of anything); to be equal to the end proposed; to be adequate.</def>  <rj><au>Chaucer.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>To recount almighty works,<br/
What words or tongue of seraph can <qex>suffice</qex>?</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Suffice</ent><br/
<hw>Suf*fice"</hw>, <pos>v. t.</pos> <sn>1.</sn> <def>To satisfy; to content; to be equal to the wants or demands of.</def>  <rj><au>Spenser.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Let it <qex>suffice</qex> thee; speak no more unto me of this matter.</q> <rj><qau>Deut. iii. 26.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To furnish; to supply adequately.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>The power appeased, with winds <qex>sufficed</qex> the sail.</q> <rj><qau>Dryden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sufficience</ent><br/
<hw>Suf*fi"cience</hw> <pr>(?)</pr>, <pos>n.</pos> <def>Sufficiently.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sufficiency</ent><br/
<hw>Suf*fi"cien*cy</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>sufficientia</ets>: cf. F. <ets>suffisance</ets>. See <er>Suffice</er>.]</ety> <sn>1.</sn> <def>The quality or state of being sufficient, or adequate to the end proposed; adequacy.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>His <qex>sufficiency</qex> is such that he bestows and possesses, his plenty being unexhausted.</q> <rj><qau>Boyle.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Qualification for any purpose; ability; capacity.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>A substitute or most allowed <qex>sufficiency</qex>.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>I am not so confident of my own <qex>sufficiency</qex> as not willingly to admit the counsel of others.</q> <rj><qau>Eikon Basilike.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>Adequate substance or means; competence.</def> <ldquo/An elegant <xex>sufficiency</xex>.<rdquo/  <rj><au>Thomson.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>Supply equal to wants; ample stock or fund.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <def>Conceit; self-confidence; self-sufficiency.</def><br/
[<source>1913 Webster</source>]</p>

<p><q><qex>Sufficiency</qex> is a compound of vanity and ignorance.</q> <rj><qau>Sir W. Temple.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sufficient</ent><br/
<hw>Suf*fi"cient</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>sufficiens</ets>, <ets>-entis</ets>, p. pr. of <ets>sufficere</ets>: cf. F. <ets>suffisant</ets>. See <er>Suffice</er>.]</ety> <sn>1.</sn> <def>Equal to the end proposed; adequate to wants; enough; ample; competent; <as>as, provision <ex>sufficient</ex> for the family; an army <ex>sufficient</ex> to defend the country</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>My grace is <qex>sufficient</qex> for thee.</q> <rj><qau>2 Cor. xii. 9.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Possessing adequate talents or accomplishments; of competent power or ability; qualified; fit.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Who is <qex>sufficient</qex> for these things?</q> <rj><qau>2 Cor. ii. 16.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>Capable of meeting obligations; responsible.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The man is, notwithstanding, <qex>sufficient</qex> . . . I think I may take his bond.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>Self-sufficient; self-satisfied; content.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>Thou art the most <qex>sufficient</qex> (I'll say for thee),<br/
Not to believe a thing.</q> <rj><qau>Beau. & Fl.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><syn><b>Syn.</b> -- Enough; adequate; competent; full; satisfactory; ample.</syn><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sufficiently</ent><br/
<hw>Suf*fi"cient*ly</hw>, <pos>adv.</pos> <def>To a sufficient degree; to a degree that answers the purpose, or gives content; enough; <as>as, we are <ex>sufficiently</ex> supplied with food; a man <ex>sufficiently</ex> qualified for the discharge of his official duties</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sufficing</ent><br/
<hw>Suf*fi"cing</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Affording enough; satisfying.</def> -- <wordforms><wf>Suf*fi"cing*ly</wf>, <pos>adv.</pos> -- <wf>Suf*fi"cing*ness</wf>, <pos>n.</pos></wordforms><br/
[<source>1913 Webster</source>]</p>

<p><ent>Suffisance</ent><br/
<hw>Suf*fi"sance</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F. See <er>Sufficiency</er>.]</ety> <def>Sufficiency; plenty; abundance; contentment.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>He could in little thing have <qex>suffisaunce</qex>.</q> <rj><qau>Chaucer.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Suffisant</ent><br/
<hw>Suf*fi"sant</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Sufficient.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Suffix</ent><br/
<hw>Suf"fix</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>suffixus</ets>, p. p. of <ets>suffigere</ets> to fasten on, to affix; <ets>sub</ets> under + <ets>figere</ets> to fix: cf. F. <ets>suffixe</ets>. See <er>Fix</er>.]</ety> <sn>1.</sn> <def>A letter, letters, syllable, or syllables added or appended to the end of a word or a root to modify the meaning; a postfix.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Math.)</fld> <def>A subscript mark, number, or letter. See <er>Subscript</er>, <pos>a.</pos></def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Suffix</ent><br/
<hw>Suf*fix"</hw> <pr>(?)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Suffixed</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Suffixing</conjf>.]</vmorph> <def>To add or annex to the end, as a letter or syllable to a word; to append.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Suffixion</ent><br/
<hw>Suf*fix"ion</hw> <pr>(?)</pr>, <pos>n.</pos> <def>The act of suffixing, or the state of being suffixed.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Suffixment</ent><br/
<hw>Suf*fix"ment</hw> <pr>(?)</pr>, <pos>n.</pos> <def>Suffixion.</def> <mark>[R.]</mark>  <rj><au>Earle.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sufflaminate</ent><br/
<hw>Suf*flam"i*nate</hw> <pr>(?)</pr>, <pos>v. t.</pos> <ety>[L. <ets>sufflaminatus</ets>, p. p. of <ets>sufflaminare</ets> to hold back by a clog, from <ets>sufflamen</ets> a clog.]</ety> <sn>1.</sn> <def>To retard the motion of, as a carriage, by preventing one or more of its wheels from revolving, either by means of a chain or otherwise.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Hence, to stop; to impede.</def> <mark>[Obs.]</mark>  <rj><au>Barrow.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sufflate</ent><br/
<hw>Suf*flate"</hw> <pr>(?)</pr>, <pos>v. t.</pos> <ety>[L. <ets>sufflatus</ets>, p. p. of <ets>sufflare</ets> to blow up, inflate; <ets>sub</ets> under + <ets>flare</ets> to blow.]</ety> <def>To blow up; to inflate; to inspire.</def> <mark>[R.]</mark>  <rj><au>T. Ward.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sufflation</ent><br/
<hw>Suf*fla"tion</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>sufflatio</ets>.]</ety> <def>The act of blowing up or inflating.</def> <mark>[R.]</mark>  <rj><au>Coles.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Suffocate</ent><br/
<hw>Suf"fo*cate</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>suffocatus</ets>, p. p. of <ets>suffocare</ets> to choke; <ets>sub</ets> under + <ets>fauces</ets> the throat.  Cf. <er>Faucal</er>.]</ety> <def>Suffocated; choked.</def>  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Suffocate</ent><br/
<hw>Suf"fo*cate</hw> <pr>(?)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Suffocated</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Suffocating</conjf>.]</vmorph> <sn>1.</sn> <def>To choke or kill by stopping respiration; to stifle; to smother.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Let not hemp his windpipe <qex>suffocate</qex>.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To destroy; to extinguish; <as>as, to <ex>suffocate</ex> fire</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Suffocate</ent><br/
<hw>Suf"fo*cate</hw>, <pos>v. i.</pos> <def>To become choked, stifled, or smothered.</def> <ldquo/A swelling discontent is apt to <xex>suffocate</xex> and strangle without passage.<rdquo/  <rj><au>collier.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Suffocating</ent><br/
<hw>Suf"fo*ca`ting</hw>, <def><pos>a. & n.</pos> from <er>Suffocate</er>, <pos>v.</pos></def> -- <wordforms><wf>Suf"fo*ca`ting*ly</wf>, <pos>adv.</pos></wordforms><br/
[<source>1913 Webster</source>]</p>

<p><ent>Suffocation</ent><br/
<hw>Suf`fo*ca"tion</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>suffocatio</ets>: cf. F. <ets>suffocation</ets>.]</ety> <def>The act of suffocating, or the state of being suffocated; death caused by smothering or choking.</def><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ The term <xex>suffocation</xex> is sometimes employed synonymously with <xex>asphyxia</xex>. In the strict medico-legal sense it signifies <xex>asphyxia</xex> induced by obstruction of the respiration otherwise than by direct pressure on the neck (hanging, strangulation) or submersion (drowning).</note>  <rj><au>Quain.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Suffocative</ent><br/
<hw>Suf"fo*ca*tive</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Tending or able to choke or stifle.</def> <ldquo/<xex>Suffocative</xex> catarrhs.<rdquo/  <rj><au>Arbuthnot.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Suffossion</ent><br/
<hw>Suf*fos"sion</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>suffossio</ets>, from <ets>suffodere</ets>, <ets>suffossum</ets>, to dig under; <ets>sub</ets> under + <ets>fodere</ets> to dig.]</ety> <def>A digging under; an undermining.</def> <mark>[R.]</mark>  <rj><au>Bp. Hall.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Suffragan</ent><br/
<hw>Suf"fra*gan</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[F. <ets>suffragant</ets>, L. <ets>suffragans</ets>, p. pr. of <ets>suffragari</ets> to support with one's vote, to be favorable. See <er>Suffrage</er>.]</ety> <def>Assisting; assistant; <as>as, a <ex>suffragan</ex> bishop</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Suffragan</ent><br/
<hw>Suf"fra*gan</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F. <ets>suffragant</ets>: cf. LL. <ets>suffraganeus</ets>. See <er>Suffragan</er>, <pos>a.</pos>]</ety> <sn>1.</sn> <def>An assistant.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Eccl.)</fld> <def>A bishop considered as an assistant, or as subject, to his metropolitan; an assistant bishop.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Suffraganship</ent><br/
<hw>Suf"fra*gan*ship</hw>, <pos>n.</pos> <def>The office of a suffragan.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Suffragant</ent><br/
<hw>Suf"fra*gant</hw> <pr>(?)</pr>, <pos>a. & n.</pos> <def>Suffragan.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Suffragate</ent><br/
<hw>Suf"fra*gate</hw> <pr>(?)</pr>, <pos>v. i. & t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Suffragated</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Suffragating</conjf>.]</vmorph> <ety>[L. <ets>suffragatus</ets>, p. p. of <ets>suffragari</ets>. See <er>Suffragan</er>, <pos>a.</pos>]</ety> <def>To vote or vote with.</def> <mark>[Obs.]</mark> <ldquo/<xex>Suffragating</xex> tribes.<rdquo/  <rj><au>Dryden.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Suffragator</ent><br/
<hw>Suf"fra*ga`tor</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L.]</ety> <def>One who assists or favors by his vote.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Suffrage</ent><br/
<hw>Suf"frage</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F., fr. L. <ets>suffragium</ets>; perhaps originally, a broken piece, a potsherd, used in voting, and fr. <ets>sub</ets> under + the root of <ets>frangere</ets> to break. See <er>Break</er>.]</ety> <sn>1.</sn> <def>A vote given in deciding a controverted question, or in the choice of a man for an office or trust; the formal expression of an opinion; assent; vote.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>I ask your voices and your <qex>suffrages</qex>.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Testimony; attestation; witness; approval.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Lactantius and St. Austin confirm by their <qex>suffrage</qex> the observation made by heathen writers.</q> <rj><qau>Atterbury.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Every miracle is the <qex>suffrage</qex> of Heaven to the truth of a doctrine.</q> <rj><qau>South.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <fld>(Eccl.)</fld> <sd>(a)</sd> <def>A short petition, as those after the creed in matins and evensong.</def> <sd>(b)</sd> <def>A prayer in general, as one offered for the faithful departed.</def>  <rj><au>Shipley.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>I firmly believe that there is a purgatory, and that the souls therein detained are helped by the <qex>suffrages</qex> of the faithful.</q> <rj><qau>Creed of Pope Pius IV.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>Aid; assistance.</def> <mark>[A Latinism]</mark> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <def>The right to vote; franchise.</def><br/
[<source>PJC</source>]</p>

<p><ent>Suffrage</ent><br/
<hw>Suf"frage</hw>, <pos>v. t.</pos> <def>To vote for; to elect.</def> <mark>[Obs.]</mark>  <rj><au>Milton.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Suffragette</ent><br/
<hw>Suf`fra*gette"</hw> <pos>n.</pos> <def>A woman who advocates the right to vote for women; a woman suffragist.</def>  <note>This term was applied mostly to women in the United States prior to the adoption of the 19th amendment to the constitution in 1920, giving women the right to vote.  Modern use of this term usually refers to the women who advocated female suffrage{5} in the years prior to 1920.</note><br/
[<source>PJC</source>]</p>

<p><ent>Suffraginous</ent><br/
<hw>Suf*frag"i*nous</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>suffraginosus</ets> diseased in the hock, fr. <ets>suffrago</ets> the pastern, or hock.]</ety> <def>Of or pertaining to the hock of a beast.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Suffragist</ent><br/
<hw>Suf"fra*gist</hw> <pr>(?)</pr>, <pos>n.</pos> <sn>1.</sn> <def>One who possesses or exercises the political right of suffrage; a voter.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>One who has certain opinions or desires about the political right of suffrage; <as>as, a woman <ex>suffragist</ex></as>.</def><-- if female, usu. suffragette. --><br/
[<source>1913 Webster</source>]</p>

<p><q>It is curious that . . . Louisa Castelefort should be obliged after her marriage immediately to open her doors and turn ultra liberal, or an universal <qex>suffragist</qex>.</q> <rj><qau>Miss Edgeworth.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Suffrago</ent><br/
||<hw>Suf*fra"go</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L., the hock, from <ets>sub</ets> under + <ets>frangere</ets> to break.]</ety> <fld>(Zool.)</fld> <def>The heel joint.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Suffrance</ent><br/
<hw>Suf"france</hw> <pr>(?)</pr>, <pos>n.</pos> <def>Sufferance.</def> <mark>[Obs.]</mark>  <rj><au>Chaucer.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Suffrutescent</ent><br/
<hw>Suf`fru*tes"cent</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[Pref. <ets>suf-</ets> + <ets>frutescent</ets>.]</ety> <fld>(Bot.)</fld> <def>Slightly woody at the base.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Suffruticose</ent><br/
<hw>Suf*fru"ti*cose`</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[Pref. <ets>suf-</ets> + <ets>fruticose</ets>.]</ety> <fld>(Bot.)</fld> <def>Woody in the lower part of the stem, but with the yearly branches herbaceous, as sage, thyme, hyssop, and the like.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Suffruticous</ent><br/
<hw>Suf*fru"ti*cous</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Suffruticose.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Suffumigate</ent><br/
<hw>Suf*fu"mi*gate</hw> <pr>(?)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Suffumigated</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Suffumigating</conjf>.]</vmorph> <ety>[L. <ets>suffumigatus</ets>, p. p. of <ets>suffumigare</ets> to fumigate from below. See <er>Sub-</er>, and <er>Fumigate</er>.]</ety> <def>To apply fumes or smoke to the parts of, as to the body in medicine; to fumigate in part.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Suffumigation</ent><br/
<hw>Suf*fu`mi*ga"tion</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>suffumigatio</ets>: cf. F. <ets>suffumigation</ets>.]</ety> <def>The operation of suffumigating.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Suffumige</ent><br/
<hw>Suf*fu"mige</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[LL. <ets>suffumigium</ets>.]</ety> <def>A medical fume.</def> <mark>[Obs.]</mark>  <rj><au>Harvey.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Suffuse</ent><br/
<hw>Suf*fuse"</hw> <pr>(?)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Suffused</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Suffusing</conjf>.]</vmorph> <ety>[L. <ets>suffusus</ets>, p. p. of <ets>suffundere</ets> to overspread; <ets>sub</ets> under + <ets>fundere</ets> to pour. See <er>Fuse</er> to melt.]</ety> <def>To overspread, as with a fluid or tincture; to fill or cover, as with something fluid; <as>as, eyes <ex>suffused</ex> with tears; cheeks <ex>suffused</ex> with blushes</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>When purple light shall next <qex>suffuse</qex> the skies.</q> <rj><qau>Pope.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Suffusion</ent><br/
<hw>Suf*fu"sion</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>suffusio</ets>: cf. F. <ets>suffusion</ets>.]</ety> <sn>1.</sn> <def>The act or process of suffusing, or state of being suffused; an overspreading.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>To those that have the jaundice, or like <qex>suffusion</qex> of eyes, objects appear of that color.</q> <rj><qau>Ray.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>That with which a thing is suffused.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <fld>(Zool.)</fld> <def>A blending of one color into another; the spreading of one color over another, as on the feathers of birds.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sufi</ent><br/
<hw>Su"fi</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[From the name of a dynasty of Persian kings, <ets>Saf<imac/</ets>, <ets>Safav<imac/</ets>; said to come from name <ets>Saf<imac/-ud-d<imac/n</ets> of an ancestor of the family, confused with <ets>s<?/f<imac/</ets> pious.]</ety> <def>A title or surname of the king of Persia.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sufi</ent><br/
<hw>Su"fi</hw>, <pos>n.</pos> <ety>[Ar. & Per. <ets>s<?/f<imac/</ets>, wise, pious, devout.]</ety> <def>One of a certain order of religious men in Persia.</def> <altsp>[Written also <asp>sofi</asp>.]</altsp><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sufism</ent><br/
<hw>Su"fism</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A refined mysticism among certain classes of Mohammedans, particularly in Persia, who hold to a kind of pantheism and practice extreme asceticism in their lives.</def> <altsp>[Written also <asp>sofism</asp>.]</altsp><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sug</ent><br/
<hw>Sug</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A kind of worm or larva.</def>  <rj><au>Walton.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sugar</ent><br/
<hw>Sug"ar</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[OE. <ets>sugre</ets>, F. <ets>sucre</ets> (cf. It. <ets>zucchero</ets>, Sp. <ets>az<uacute/car</ets>), fr. Ar. <ets>sukkar</ets>, <ets>assukkar</ets>, fr. Skr. <ets><cced/arkar<amac/</ets> sugar, gravel; cf. Per. <ets>shakar</ets>.  Cf. <er>Saccharine</er>, <er>Sucrose</er>.]</ety> <sn>1.</sn> <def>A sweet white (or brownish yellow) crystalline substance, of a sandy or granular consistency, obtained by crystallizing the evaporated juice of certain plants, as the sugar cane, sorghum, beet root, sugar maple, etc. It is used for seasoning and preserving many kinds of food and drink. Ordinary sugar is essentially sucrose. See the Note below.</def><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ The term <xex>sugar</xex> includes several commercial grades, as the <xex>white</xex> or <xex>refined</xex>, <xex>granulated</xex>, <xex>loaf</xex> or <xex>lump</xex>, and the <xex>raw brown</xex> or <xex>muscovado</xex>. In a more general sense, it includes several distinct chemical compounds, as the <xex>glucoses</xex>, or <xex>grape sugars</xex> (including glucose proper, dextrose, and levulose), and the <xex>sucroses</xex>, or true sugars (as cane sugar). All sugars are carbohydrates. See <er>Carbohydrate</er>.
    The <xex>glucoses</xex>, or <xex>grape sugars</xex>, are ketone alcohols of the formula <chform>C6H12O6</chform>, and they turn the plane of polarization to the right or the left. They are produced from the amyloses and sucroses, as by the action of heat and acids of ferments, and are themselves decomposed by fermentation into alcohol and carbon dioxide. The only sugar (called <xex>acrose</xex>) as yet produced artificially belongs to this class. The <xex>sucroses</xex>, or <xex>cane sugars</xex>, are doubled glucose anhydrides of the formula <chform>C12H22O11</chform>. They are usually not fermentable as such (cf. <er>Sucrose</er>), and they act on polarized light.</note><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>By extension, anything resembling sugar in taste or appearance; <as>as, <ex>sugar</ex> of lead (lead acetate), a poisonous white crystalline substance having a sweet taste</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>Compliment or flattery used to disguise or render acceptable something obnoxious; honeyed or soothing words.</def> <mark>[Colloq.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Acorn sugar</b></col>. <cd>See <er>Quercite</er>.</cd> -- <col><b>Cane sugar</b></col>, <cd>sugar made from the sugar cane; sucrose, or an isomeric sugar. See <er>Sucrose</er>.</cd> -- <mcol><col><b>Diabetes sugar</b></col>, <it>or</it> <col><b>Diabetic sugar</b></col></mcol> <fld>(Med. Chem.)</fld>, <cd>a variety of sugar (grape sugar or dextrose) excreted in the urine in diabetes mellitus; -- the presence of such a sugar in the urine is used to diagnose the illness.</cd> -- <col><b>Fruit sugar</b></col>. <cd>See under <er>Fruit</er>, and <er>Fructose</er>.</cd> -- <col><b>Grape sugar</b></col>, <cd>a sirupy or white crystalline sugar (dextrose or glucose) found as a characteristic ingredient of ripe grapes, and also produced from many other sources. See <er>Dextrose</er>, and <er>Glucose</er>.</cd> -- <col><b>Invert sugar</b></col>. <cd>See under <er>Invert</er>.</cd> -- <col><b>Malt sugar</b></col>, <cd>a variety of sugar isomeric with sucrose, found in malt. See <er>Maltose</er>.</cd> -- <col><b>Manna sugar</b></col>, <cd>a substance found in manna, resembling, but distinct from, the sugars. See <er>Mannite</er>.</cd> -- <col><b>Milk sugar</b></col>, <cd>a variety of sugar characteristic of fresh milk, and isomeric with sucrose. See <er>Lactose</er>.</cd> -- <col><b>Muscle sugar</b></col>, <cd>a sweet white crystalline substance isomeric with, and formerly regarded to, the glucoses.  It is found in the tissue of muscle, the heart, liver, etc. Called also <altname>heart sugar</altname>. See <er>Inosite</er>.</cd> -- <col><b>Pine sugar</b></col>. <cd>See <er>Pinite</er>.</cd> -- <col><b>Starch sugar</b></col> <fld>(Com. Chem.)</fld>, <cd>a variety of dextrose made by the action of heat and acids on starch from corn, potatoes, etc.; -- called also <altname>potato sugar</altname>, <altname>corn sugar</altname>, and, inaccurately, <altname>invert sugar</altname>. See <er>Dextrose</er>, and <er>Glucose</er>.</cd> -- <col><b>Sugar barek</b></col>, <cd>one who refines sugar.</cd> -- <col><b>Sugar beet</b></col> <fld>(Bot.)</fld>, <cd>a variety of beet (<spn>Beta vulgaris</spn>) with very large white roots, extensively grown, esp. in Europe, for the sugar obtained from them.</cd> -- <col><b>Sugar berry</b></col> <fld>(Bot.)</fld>, <cd>the hackberry.</cd> -- <col><b>Sugar bird</b></col> <fld>(Zool.)</fld>, <cd>any one of several species of small South American singing birds of the genera <gen>Coereba</gen>, <gen>Dacnis</gen>, and allied genera belonging to the family <fam>Coerebidae</fam>. They are allied to the honey eaters.</cd> -- <col><b>Sugar bush</b></col>. <cd>See <er>Sugar orchard</er>.</cd> -- <col><b>Sugar camp</b></col>, <cd>a place in or near a sugar orchard, where maple sugar is made.</cd> -- <col><b>Sugar candian</b></col>, <cd>sugar candy.</cd> <mark>[Obs.]</mark> -- <col><b>Sugar candy</b></col>, <cd>sugar clarified and concreted or crystallized; candy made from sugar.</cd> -- <col><b>Sugar cane</b></col> <fld>(Bot.)</fld>, <cd>a tall perennial grass (<spn>Saccharum officinarium</spn>), with thick short-jointed stems. It has been cultivated for ages as the principal source of sugar.</cd> -- <col><b>Sugar loaf</b></col>. <sd>(a)</sd> <cd>A loaf or mass of refined sugar, usually in the form of a truncated cone.</cd> <sd>(b)</sd> <cd>A hat shaped like a sugar loaf.</cd><br/
[<source>1913 Webster</source>]</p>

<p><q>Why, do not or know you, grannam, and that <qex>sugar loaf</qex>?</q> <rj><qau>J. Webster.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p>-- <col><b>Sugar maple</b></col> <fld>(Bot.)</fld>, <cd>the rock maple (<spn>Acer saccharinum</spn>). See <er>Maple</er>.</cd> -- <col><b>Sugar mill</b></col>, <cd>a machine for pressing out the juice of the sugar cane, usually consisting of three or more rollers, between which the cane is passed.</cd> -- <col><b>Sugar mite</b></col>. <fld>(Zool.)</fld> <sd>(a)</sd> <cd>A small mite (<spn>Tyroglyphus sacchari</spn>), often found in great numbers in unrefined sugar.</cd> <sd>(b)</sd> <cd>The lepisma.</cd> -- <col><b>Sugar of lead</b></col>. <cd>See <er>Sugar</er>, 2, above.</cd> -- <col><b>Sugar of milk</b></col>. <cd>See under <er>Milk</er>.</cd> -- <col><b>Sugar orchard</b></col>, <cd>a collection of maple trees selected and preserved for purpose of obtaining sugar from them; -- called also, sometimes, <altname>sugar bush</altname>.</cd> <mark>[U.S.]</mark> <au>Bartlett.</au> -- <col><b>Sugar pine</b></col> <fld>(Bot.)</fld>, <cd>an immense coniferous tree (<spn>Pinus Lambertiana</spn>) of California and Oregon, furnishing a soft and easily worked timber. The resinous exudation from the stumps, etc., has a sweetish taste, and has been used as a substitute for sugar.</cd> -- <col><b>Sugar squirrel</b></col> <fld>(Zool.)</fld>, <cd>an Australian flying phalanger (<spn>Belideus sciureus</spn>), having a long bushy tail and a large parachute. It resembles a flying squirrel. See <xex>Illust.</xex> under <er>Phlanger</er>.</cd> -- <col><b>Sugar tongs</b></col>, <cd>small tongs, as of silver, used at table for taking lumps of sugar from a sugar bowl.</cd> -- <col><b>Sugar tree</b></col>. <fld>(Bot.)</fld> <cd>See <cref>Sugar maple</cref>, above.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sugar</ent><br/
<hw>Sug"ar</hw> <pr>(?)</pr>, <pos>v. i.</pos> <def>In making maple sugar, to complete the process of boiling down the sirup till it is thick enough to crystallize; to approach or reach the state of granulation; -- with the preposition <xex>off</xex>.</def> <mark>[Local, U.S.]</mark><-- field = sugar making --><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sugar</ent><br/
<hw>Sug"ar</hw>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Sugared</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Sugaring</conjf>.]</vmorph> <sn>1.</sn> <def>To impregnate, season, cover, or sprinkle with sugar; to mix sugar with.</def> <ldquo/When I <xex>sugar</xex> my liquor.<rdquo/  <rj><au>G. Eliot.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To cover with soft words; to disguise by flattery; to compliment; to sweeten; <as>as, to <ex>sugar</ex> reproof</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>With devotion's visage<br/
And pious action we do <qex>sugar</qex> o'er<br/
The devil himself.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sugared</ent><br/
<hw>Sug"ared</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Sweetened.</def> <ldquo/The <xex>sugared</xex> liquor.<rdquo/ <au>Spenser.</au> <def>Also used figuratively; <as>as, <ex>sugared</ex> kisses</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><-- p. 1442 --></p>

<p><ent>Sugar-house</ent><br/
<hw>Sug"ar-house`</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A building in which sugar is made or refined; a sugar manufactory.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sugariness</ent><br/
<hw>Sug"ar*i*ness</hw> <pr>(?)</pr>, <pos>n.</pos> <def>The quality or state of being sugary, or sweet.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sugaring</ent><br/
<hw>Sug"ar*ing</hw>, <pos>n.</pos> <sn>1.</sn> <def>The act of covering or sweetening with sugar; also, the sugar thus used.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>The act or process of making sugar.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sugarless</ent><br/
<hw>Sug"ar*less</hw>, <pos>a.</pos> <def>Without sugar; free from sugar.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sugarplum</ent><br/
<hw>Sug"ar*plum`</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A kind of candy or sweetneat made up in small balls or disks.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sugary</ent><br/
<hw>Sug"ar*y</hw> <pr>(?)</pr>, <pos>a.</pos> <sn>1.</sn> <def>Resembling or containing sugar; tasting of sugar; sweet.</def>  <rj><au>Spenser.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Fond of sugar or sweet things; <as>as, a <ex>sugary</ex> palate</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sugescent</ent><br/
<hw>Su*ges"cent</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>sugere</ets> to suck.]</ety> <def>Of or pertaining to sucking.</def> <mark>[R.]</mark>  <rj><au>Paley.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Suggest</ent><br/
<hw>Sug*gest"</hw> <pr>(?)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Suggested</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Suggesting</conjf>.]</vmorph> <ety>[L. <ets>suggestus</ets>, p. p. of <ets>suggerere</ets> to put under, furnish, suggest; <ets>sub</ets> under + <ets>gerere</ets> to carry, to bring. See <er>Jest</er>.]</ety> <sn>1.</sn> <def>To introduce indirectly to the thoughts; to cause to be thought of, usually by the agency of other objects.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Some ideas . . . are <qex>suggested</qex> to the mind by all the ways of sensation and reflection.</q> <rj><qau>Locke.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To propose with difference or modesty; to hint; to intimate; <as>as, to <ex>suggest</ex> a difficulty</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>To seduce; to prompt to evil; to tempt.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>Knowing that tender youth is soon <qex>suggested</qex>.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>To inform secretly.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><syn><b>Syn.</b> -- To hint; allude to; refer to; insinuate.</syn><br/
[<source>1913 Webster</source>]</p>

<p><ent>Suggest</ent><br/
<hw>Sug*gest"</hw>, <pos>v. i.</pos> <def>To make suggestions; to tempt.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>And ever weaker grows through acted crime,<br/
Or seeming-genial, venial fault,<br/
Recurring and <qex>suggesting</qex> still.</q> <rj><qau>Tennyson.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Suggester</ent><br/
<hw>Sug*gest"er</hw> <pr>(?)</pr>, <pos>n.</pos> <def>One who suggests.</def>  <rj><au>Beau. & Fl.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Suggestion</ent><br/
<hw>Sug*ges"tion</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F. <ets>suggestion</ets>, L. <ets>suggestio</ets>.]</ety> <sn>1.</sn> <def>The act of suggesting; presentation of an idea.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>That which is suggested; an intimation; an insinuation; a hint; a different proposal or mention; also, formerly, a secret incitement; temptation.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Why do I yield to that <qex>suggestion</qex>?</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>Charge; complaint; accusation.</def> <mark>[Obs.]</mark> <ldquo/A false <xex>suggestion</xex>.<rdquo/  <rj><au>Chaucer.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <fld>(Law)</fld> <def>Information without oath; an entry of a material fact or circumstance on the record for the information of the court, at the death or insolvency of a party.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <fld>(Physiol. & Metaph.)</fld> <def>The act or power of originating or recalling ideas or relations, distinguished as original and relative; -- a term much used by Scottish metaphysicians from Hutcherson to Thomas Brown.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>6.</sn> <fld>(Hypnotism)</fld> <def>The control of the mind of an hypnotic subject by ideas in the mind of the hypnotizer.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><syn><b>Syn.</b> -- Hint; allusion; intimation; insinuation.</syn>  -- <usage><er>Suggestion</er>, <er>Hint</er>. A <xex>hint</xex> is the briefest or most indirect mode of calling one's attention to a subject. A <xex>suggestion</xex> is a putting of something before the mind for consideration, an indirect or guarded mode of presenting argument or advice. A <xex>hint</xex> is usually something slight or covert, and may by merely negative in its character. A <xex>suggestion</xex> is ordinarily intended to furnish us with some practical assistance or direction. <ldquo/He gave me a <xex>hint</xex> of my danger, and added some <xex>suggestions</xex> as to the means of avoiding it.<rdquo/</usage><br/
[<source>1913 Webster</source>]</p>

<p><q>Willing to wound, and yet afraid to strike,<br/
Just <qex>hint</qex> a fault, and hesitate dislike.</q> <rj><qau>Pope.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Arthur, whom they say is killed to-night<br/
On your <qex>suggestion</qex>.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Suggestive</ent><br/
<hw>Sug*gest"ive</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Containing a suggestion, hint, or intimation.</def> -- <wordforms><wf>Sug*gest"ive*ly</wf>, <pos>adv.</pos> -- <wf>Sug*gest"ive*ness</wf>, <pos>n.</pos></wordforms><br/
[<source>1913 Webster</source>]</p>

<p><ent>Suggestive medicine</ent><br/
<hw>Sug*gest"ive med"i*cine</hw> <pr>(?)</pr>. <def>Treatment by commands or positive statements addressed to a more or less hypnotized patient.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Suggestment</ent><br/
<hw>Sug*gest"ment</hw> <pr>(?)</pr>, <pos>n.</pos> <def>Suggestion.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>They fancy that every thought must needs have an immediate outward <qex>suggestment</qex>.</q> <rj><qau>Hare.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Suggestress</ent><br/
<hw>Sug*gest"ress</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A woman who suggests.</def> <ldquo/The <xex>suggestress</xex> of suicides.<rdquo/  <rj><au>De Quincey.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Suggil</ent><br/
<hw>Sug"gil</hw> <pr>(?)</pr>, <pos>v. t.</pos> <ety>[L. <ets>suggillare</ets>, <ets>sugillare</ets>, <ets>suggillatum</ets>, <ets>sugillatum</ets>, literally, to beat black and blue.]</ety> <def>To defame.</def> <mark>[Obs.]</mark>  <rj><au>Abp. Parker.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Suggillate</ent><br/
<hw>Sug"gil*late</hw> <pr>(?)</pr>, <pos>v. t.</pos> <ety>[See <er>Suggil</er>.]</ety> <def>To beat livid, or black and blue.</def>  <rj><au>Wiseman.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Suggillation</ent><br/
<hw>Sug`gil*la"tion</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>suggillatio</ets>: cf. F. <ets>suggillation</ets>.]</ety> <def>A livid, or black and blue, mark; a blow; a bruise.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Suicidal</ent><br/
<hw>Su"i*ci`dal</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Partaking of, or of the nature of, the crime or suicide.</def> -- <wordforms><wf>Su"i*ci`dal*ly</wf>, <pos>adv.</pos></wordforms><br/
[<source>1913 Webster</source>]</p>

<p><ent>Suicide</ent><br/
<hw>Su"i*cide</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>sui</ets> of one's self (akin to <ets>suus</ets> one's own) + <ets>caedere</ets> to slay, to kill.  Cf. <er>So</er>, <pos>adv.</pos>, <er>Homicide</er>.]</ety> <sn>1.</sn> <def>The act of taking one's own life voluntary and intentionally; self-murder; specifically <fld>(Law)</fld>, the felonious killing of one's self; the deliberate and intentional destruction of one's own life by a person of years of discretion and of sound mind.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>One guilty of self-murder; a felo-de-se.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>Ruin of one's own interests.</def> <ldquo/Intestine war, which may be justly called political <xex>suicide</xex>.<rdquo/  <rj><au>V. Knox.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Suicidical</ent><br/
<hw>Su`i*cid"i*cal</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Suicidal.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Suicidism</ent><br/
<hw>Su"i*ci*dism</hw> <pr>(?)</pr>, <pos>n.</pos> <def>The quality or state of being suicidal, or self-murdering.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Suicism</ent><br/
<hw>Su"i*cism</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>suus</ets> one's own.]</ety> <def>Selfishness; egoism.</def> <mark>[R.]</mark>  <rj><au>Whitlock.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sui generis</ent><br/
||<hw>Su"i gen"e*ris</hw> <pr>(?)</pr>. <ety>[L.]</ety> <def>Of his or its own kind.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Suillage</ent><br/
<hw>Su"il*lage</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[OF. <ets>souillage</ets>, <ets>soillage</ets>, fr. <ets>souiller</ets>, <ets>soiller</ets>. See <er>Soil</er> to stain, and cf. <er>Sullage</er>.]</ety> <def>A drain or collection of filth.</def> <mark>[Obs.]</mark> <altsp>[Written also <asp>sulliage</asp>, and <asp>sullage</asp>.]</altsp>  <rj><au>Sir H. Wotton.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Suilline</ent><br/
<hw>Su"il*line</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>sus</ets> hog.]</ety> <fld>(Zool.)</fld> <def>Of or pertaining to a hog or the Hog family (<fam>Suidae</fam>).</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Suine</ent><br/
<hw>Su"ine</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Cf. <er>Suint</er>.]</ety> <def>A mixture of oleomargarine with lard or other fatty ingredients. It is used as a substitute for butter. See <er>Butterine</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Suing</ent><br/
<hw>Su"ing</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Cf. F. <ets>suer</ets> to sweat, L. <ets>sudare</ets>.]</ety> <def>The process of soaking through anything.</def> <mark>[Obs.]</mark>  <rj><au>Bacon.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Suingly</ent><br/
<hw>Su"ing*ly</hw>, <pos>adv.</pos> <ety>[See <er>Sue</er> to follow.]</ety> <def>In succession; afterwards.</def> <mark>[Obs.]</mark>  <rj><au>Sir T. More.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Suint</ent><br/
<hw>Su"int</hw> <pr>(s<umac/"<icr/nt)</pr>, <pos>n.</pos> <ety>[F.]</ety> <fld>(Chem.)</fld> <def>A peculiar substance obtained from the wool of sheep, consisting largely of potash mixed with fatty and earthy matters. It is used as a source of potash and also for the manufacture of gas.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Suiogoths</ent><br/
<hw>Su`i*o*goths"</hw> <pr>(?)</pr>, <pos>n. pl.</pos> <ety>[L. <ets>Suiones</ets> (a Teutonic tribe in what is now Sweeden) + E. <ets>Goth</ets>.]</ety> <def>The Scandinavian Goths. See the Note under <er>Goths</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Suist</ent><br/
<hw>Su"ist</hw>, <pos>n.</pos> <ety>[L. <ets>suus</ets> belinging to himself or to one's self.]</ety> <def>One who seeks for things which gratify merely himself; a selfish person; a selfist.</def> <mark>[R.]</mark>  <rj><au>Whitlock.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Suit</ent><br/
<hw>Suit</hw> <pr>(s<umac/t)</pr>, <pos>n.</pos> <ety>[OE. <ets>suite</ets>, F. <ets>suite</ets>, OF. <ets>suite</ets>, <ets>sieute</ets>, fr. <ets>suivre</ets> to follow, OF. <ets>sivre</ets>; perhaps influenced by L. <ets>secta</ets>. See <er>Sue</er> to follow, and cf. <er>Sect</er>, <er>Suite</er>.]</ety> <sn>1.</sn> <def>The act of following or pursuing, as game; pursuit.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>The act of suing; the process by which one endeavors to gain an end or an object; an attempt to attain a certain result; pursuit; endeavor.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Thenceforth the <qex>suit</qex> of earthly conquest shone.</q> <rj><qau>Spenser.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>The act of wooing in love; the solicitation of a woman in marriage; courtship.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Rebate your loves, each rival <qex>suit</qex> suspend,<br/
Till this funereal web my labors end.</q> <rj><qau>Pope.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <fld>(Law)</fld> <def>The attempt to gain an end by legal process; an action or process for the recovery of a right or claim; legal application to a court for justice; prosecution of right before any tribunal; <as>as, a civil <ex>suit</ex>; a criminal <ex>suit</ex>; a <ex>suit</ex> in chancery.</as></def><br/
[<source>1913 Webster</source>]</p>

<p><q>I arrest thee at the <qex>suit</qex> of Count Orsino.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>In England the several <qex>suits</qex>, or remedial instruments of justice, are distinguished into three kinds -- actions personal, real, and mixed.</q> <rj><qau>Blackstone.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <def>That which follows as a retinue; a company of attendants or followers; the assembly of persons who attend upon a prince, magistrate, or other person of distinction; -- often written <asp>suite</asp>, and pronounced <pr>sw<emac/t</pr>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>6.</sn> <def>Things that follow in a series or succession; the individual objects, collectively considered, which constitute a series, as of rooms, buildings, compositions, etc.; -- often written <asp>suite</asp>, and pronounced <pr>sw<emac/t</pr>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>7.</sn> <def>A number of things used together, and generally necessary to be united in order to answer their purpose; a number of things ordinarily classed or used together; a set; <as>as, a <ex>suit</ex> of curtains; a <ex>suit</ex> of armor; a <ex>suit</ex> of clothes; a three-piece business <ex>suit</ex>.</as></def> <ldquo/Two rogues in buckram <xex>suits</xex>.<rdquo/  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source> <source>+PJC</source>]</p>

<p><sn>8.</sn> <fld>(Playing Cards)</fld> <def>One of the four sets of cards which constitute a pack; -- each set consisting of thirteen cards bearing a particular emblem, as hearts, spades, clubs, or diamonds; also, the members of each such suit held by a player in certain games, such as bridge; <as>as, hearts were her long <ex>suit</ex></as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>To deal and shuffle, to divide and sort<br/
Her mingled <qex>suits</qex> and sequences.</q> <rj><qau>Cowper.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>9.</sn> <def>Regular order; succession.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>Every five and thirty years the same kind and <qex>suit</qex> of weather comes again.</q> <rj><qau>Bacon.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>10.</sn> <specif>Hence: (derived from def 7)</specif> <def>Someone who dresses in a business suit, as contrasted with more informal attire;</def> <specif>specifically,</specif> <def>a person, such as business executive, or government official, who is apt to view a situation formalistically, bureaucratically, or according to formal procedural criteria; -- used derogatively for one who is inflexible, esp. when a more humanistic or imaginative approach would be appropriate.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Out of suits</b></col>, <cd>having no correspondence.</cd> <mark>[Obs.]</mark> <au>Shak.</au> -- <col><b>Suit and service</b></col> <fld>(Feudal Law)</fld>, <cd>the duty of feudatories to attend the courts of their lords or superiors in time of peace, and in war to follow them and do military service; -- called also <altname>suit service</altname>.</cd> <au>Blackstone.</au> -- <col><b>Suit broker</b></col>, <cd>one who made a trade of obtaining the suits of petitioners at court.</cd> <mark>[Obs.]</mark> -- <col><b>Suit court</b></col> <fld>(O. Eng. Law)</fld>, <cd>the court in which tenants owe attendance to their lord.</cd> -- <col><b>Suit covenant</b></col> <fld>(O. Eng. Law)</fld>, <cd>a covenant to sue at a certain court.</cd> -- <col><b>Suit custom</b></col> <fld>(Law)</fld>, <cd>a service which is owed from time immemorial.</cd> -- <col><b>Suit service</b></col>. <fld>(Feudal Law)</fld> <cd>See <cref>Suit and service</cref>, above.</cd> -- <col><b>To bring suit</b></col>. <fld>(Law)</fld> <sd>(a)</sd> <cd>To bring <xex>secta</xex>, followers or witnesses, to prove the plaintiff's demand.</cd> <mark>[Obs.]</mark> <sd>(b)</sd> <cd>In modern usage, to institute an action.</cd> -- <col><b>To follow suit</b></col>. <sd>(a)</sd> <fld>(Card Playing)</fld> <cd>See under <er>Follow</er>, <pos>v. t.</pos></cd>  <sd>(b)</sd> <cd>To mimic the action of another person; to perform an action similar to what has preceded; <as>as, when she walked in, John left the room and his wife <ex>followed suit</ex></as>.</cd> -- <col>long suit</col> <sd>(a)</sd> <fld>(Card Playing)</fld> <cd>the suit{8} of which a player has the largest number of cards in his hand; <as>as, his <ex>long suit</ex> was clubs, but his partner insisted on making hearts trumps.</as>.</cd> <specif>Hence:</specif> <mark>[fig.]</mark> <cd>that quality or capability which is a person's best asset; <as>as, we could see from the mess in his room that neatness was not his <ex>long suit</ex></as>.</cd> -- <col>strong suit</col> <cd>same as <cref>long suit</cref>, <sd>(b)</sd>.</cd> <ldquo/I think our <qex>strong suit</qex> is that we can score from both the perimeter and the post.<rdquo/ <au>Bill Disbrow (basketball coach) 1998.</au>  <ldquo/Rigid ideological consistency has never been a <qex>strong suit</qex> of the Whole Earth Catalogue.<rdquo/ <au>Bruce Sterling (The Hacker Crackdown, 1994)</au></cs><br/
[<source>1913 Webster</source> <source>+PJC</source>]</p>

<p><ent>Suit</ent><br/
<hw>Suit</hw>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Suited</conjf>; <pos>p. pr. & vb. n.</pos> <conjf>Suiting</conjf>.]</vmorph> <sn>1.</sn> <def>To fit; to adapt; to make proper or suitable; <as>as, to <ex>suit</ex> the action to the word</as>.</def>  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To be fitted to; to accord with; to become; to befit.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Ill <qex>suits</qex> his cloth the praise of railing well.</q> <rj><qau>Dryden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Raise her notes to that sublime degree<br/
Which <qex>suits</qex> song of piety and thee.</q> <rj><qau>Prior.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>To dress; to clothe.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>So went he <qex>suited</qex> to his watery tomb.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>To please; to make content; <as>as, he is well <ex>suited</ex> with his place; to <ex>suit</ex> one's taste</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Suit</ent><br/
<hw>Suit</hw>, <pos>v. i.</pos> <def>To agree; to accord; to be fitted; to correspond; -- usually followed by <xex>with</xex> or <xex>to</xex>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The place itself was <qex>suiting</qex> to his care.</q> <rj><qau>Dryden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Give me not an office<br/
That <qex>suits</qex> with me so ill.</q> <rj><qau>Addison.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><syn><b>Syn.</b> -- To agree; accord; comport; tally; correspond; match; answer.</syn><br/
[<source>1913 Webster</source>]</p>

<p><ent>Suitability</ent><br/
<hw>Suit`a*bil"i*ty</hw> <pr>(?)</pr>, <pos>n.</pos> <def>The quality or state of being suitable; suitableness.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Suitable</ent><br/
<hw>Suit"a*ble</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Capable of suiting; fitting; accordant; proper; becoming; agreeable; adapted; <as>as, ornaments <ex>suitable</ex> to one's station; language <ex>suitable</ex> for the subject</as>.</def> -- <wordforms><wf>Suit"a*ble*ness</wf>, <pos>n.</pos> -- <wf>Suit"a*bly</wf>, <pos>adv.</pos></wordforms><br/
[<source>1913 Webster</source>]</p>

<p><syn><b>Syn.</b> -- Proper; fitting; becoming; accordant; agreeable; competent; correspondent; compatible; consonant; congruous; consistent.</syn><br/
[<source>1913 Webster</source>]</p>

<p><ent>Suite</ent><br/
<hw>Suite</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F. See <er>Suit</er>, <pos>n.</pos>]</ety> <sn>1.</sn> <def>A retinue or company of attendants, as of a distinguished personage; <as>as, the <ex>suite</ex> of an ambassador</as>. See <er>Suit</er>, <pos>n.</pos>, 5.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A connected series or succession of objects; a number of things used or clessed together; a set; <as>as, a <ex>suite</ex> of rooms; a <ex>suite</ex> of minerals</as>. See <er>Suit</er>, <pos>n.</pos>, 6.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Mr. Barnard took one of the candles that stood upon the king's table, and lighted his majesty through a <qex>suite</qex> of rooms till they came to a private door into the library.</q> <rj><qau>Boswell.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <fld>(Mus.)</fld> <def>One of the old musical forms, before the time of the more compact <xex>sonata</xex>, consisting of a string or series of pieces all in the same key, mostly in various dance rhythms, with sometimes an elaborate prelude. Some composers of the present day affect the <xex>suite</xex> form.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Suiting</ent><br/
<hw>Suit"ing</hw> <pr>(?)</pr>, <pos>n.</pos> <def>Among tailors, cloth suitable for making entire suits of clothes.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Suitor</ent><br/
<hw>Suit"or</hw> <pr>(?)</pr>, <pos>n.</pos> <sn>1.</sn> <def>One who sues, petitions, or entreats; a petitioner; an applicant.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>She hath been a <qex>suitor</qex> to me for her brother.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Especially, one who solicits a woman in marriage; a wooer; a lover.</def>  <rj><au>Sir P. Sidney.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <sd>(a)</sd> <fld>(Law)</fld> <def>One who sues or prosecutes a demand in court; a party to a suit, as a plaintiff, petitioner, etc.</def> <sd>(b)</sd> <fld>(O. Eng. Law)</fld> <def>One who attends a court as plaintiff, defendant, petitioner, appellant, witness, juror, or the like.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Suitress</ent><br/
<hw>Suit"ress</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A female supplicant.</def>  <rj><au>Rowe.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Suji</ent><br/
||<hw>Su"ji</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Hind. <ets>s<?/f<imac/</ets>.]</ety> <def>Indian wheat, granulated but not pulverized; a kind of semolina.</def> <altsp>[Written also <asp>soojee</asp>.]</altsp><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sula</ent><br/
||<hw>Su"la</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[NL., fr. Icel. <ets>s<?/la</ets> the gannet. See <er>Solan goose</er>.]</ety> <fld>(Zool.)</fld> <def>A genus of sea birds including the booby and the common gannet.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sulcated</ent><br/
<ent>Sulcate</ent><br/
<mhw>{ <hw>Sul"cate</hw> <pr>(?)</pr>, <hw>Sul"ca*ted</hw> <pr>(?)</pr>, }</mhw> <pos>a.</pos> <ety>[L. <ets>sulcatus</ets>, p. p. of <ets>sulcare</ets> to furrow, fr. <ets>sulcus</ets> a furrow.]</ety> <def>Scored with deep and regular furrows; furrowed or grooved; <as>as, a <ex>sulcated</ex> stem</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sulcation</ent><br/
<hw>Sul*ca"tion</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A channel or furrow.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sulciform</ent><br/
<hw>Sul"ci*form</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Having the form of a sulcus; <as>as, <ex>sulciform</ex> markings</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sulcus</ent><br/
||<hw>Sul"cus</hw> <pr>(?)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Sulci</plw> <pr>(#)</pr>.</plu> <ety>[L., a furrow.]</ety> <def>A furrow; a groove; a fissure.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Suleah fish</ent><br/
<hw>Su"le*ah fish`</hw> <pr>(?)</pr>. <fld>(Zool.)</fld> <def>A coarse fish of India, used in making a breakfast relish called <xex>burtah</xex>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sulk</ent><br/
<hw>Sulk</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>sulcus</ets>.]</ety> <def>A furrow.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sulk</ent><br/
<hw>Sulk</hw>, <pos>v. i.</pos> <ety>[See <er>Sulkiness</er>.]</ety> <def>To be silently sullen; to be morose or obstinate.</def>  <rj><au>T. Hook.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sulker</ent><br/
<hw>Sulk"er</hw> <pr>(?)</pr>, <pos>n.</pos> <def>One who sulks.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sulkily</ent><br/
<hw>Sulk"i*ly</hw> <pr>(?)</pr>, <pos>adv.</pos> <def>In a sulky manner.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sulkiness</ent><br/
<hw>Sulk"i*ness</hw>, <pos>n.</pos> <ety>[For <ets>sulkenness</ets>, fr. AS. <ets>solcen</ets> slothful, remiss, in <amac/<ets>solcen</ets>, be<ets>solcen</ets>, properly p. p. of <ets>sealcan</ets> in <amac/<ets>sealcan</ets> to be weak or slothful; of uncertain origin.]</ety> <def>The quality or state of being sulky; sullenness; moroseness; <as>as, <ex>sulkiness</ex> of disposition</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sulks</ent><br/
<hw>Sulks</hw> <pr>(?)</pr>, <pos>n. pl.</pos> <def>The condition of being sulky; a sulky mood or humor; <as>as, to be in the <ex>sulks</ex></as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sulky</ent><br/
<hw>Sulk"y</hw> <pr>(?)</pr>, <pos>a.</pos> <amorph>[<pos>Compar.</pos> <adjf>Sulkier</adjf> <pr>(?)</pr>; <pos>superl.</pos> <adjf>Sulkiest</adjf>.]</amorph> <ety>[See <er>Sulkiness</er>, and cf. <er>Sulky</er>, <pos>n.</pos>]</ety> <def>Moodly silent; sullen; sour; obstinate; morose; splenetic.</def><br/
[<source>1913 Webster</source>]</p>

<p><syn><b>Syn.</b> -- See <er>Sullen</er>.</syn><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sulky</ent><br/
<hw>Sulk"y</hw>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Sulkies</plw> <pr>(#)</pr>.</plu> <ety>[From <er>Sulky</er>, <pos>a.</pos>; -- so called from the owner's desire of riding alone.]</ety> <def>A light two-wheeled carriage for a single person.</def><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ <xex>Sulky</xex> is used adjectively in the names of several agricultural machines drawn by horses to denote that the machine is provided with wheels and a seat for the driver; as, <xex>sulky</xex> plow; <xex>sulky</xex> harrow; <xex>sulky</xex> rake, etc.</note><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sull</ent><br/
<hw>Sull</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[AS. <ets>suluh</ets>, <ets>sulh</ets>, a plow; cf. OHG. <ets>suohili</ets> a little plow.]</ety> <def>A plow.</def> <mark>[Obs.]</mark>  <rj><au>Ainsworth.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sullage</ent><br/
<hw>Sul"lage</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Cf. <er>Suillage</er>, <er>Sulliage</er>.]</ety> <sn>1.</sn> <def>Drainage of filth; filth collected from the street or highway; sewage.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>The streets were exceedingly large, well paved, having many vaults and conveyances under them for <qex>sullage</qex>.</q> <rj><qau>Evelyn.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>That which sullies or defiles.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>It is the privilege of the celestial luminaries to receive no tincture, <qex>sullage</qex>, or difilement from the most noisome sinks and dunghills here below.</q> <rj><qau>South.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <fld>(Founding)</fld> <def>The scoria on the surface of molten metal in the ladle.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <fld>(Hydraul. Engin.)</fld> <def>Silt; mud deposited by water.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Sullage piece</b></col> <fld>(Founding)</fld>, <cd>the sprue of a casting. See <er>Sprue</er>, <pos>n.</pos>, 1 <sd>(b)</sd>.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sullen</ent><br/
<hw>Sul"len</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[OE. <ets>solein</ets>, <ets>solain</ets>, lonely, sullen; through Old French fr. (assumed) LL. <ets>solanus</ets> solitary, fr. L. <ets>solus</ets> alone. See <er>Sole</er>, <pos>a.</pos>]</ety> <sn>1.</sn> <def>Lonely; solitary; desolate.</def> <mark>[Obs.]</mark>  <rj><au>Wyclif (Job iii. 14).</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Gloomy; dismal; foreboding.</def>  <rj><au>Milton.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Solemn hymns so <qex>sullen</qex> dirges change.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>Mischievous; malignant; unpropitious.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Such <qex>sullen</qex> planets at my birth did shine.</q> <rj><qau>Dryden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>Gloomily angry and silent; cross; sour; affected with ill humor; morose.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>And <qex>sullen</qex> I forsook the imperfect feast.</q> <rj><qau>Prior.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <def>Obstinate; intractable.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Things are as <qex>sullen</qex> as we are.</q> <rj><qau>Tillotson.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>6.</sn> <def>Heavy; dull; sluggish.</def> <ldquo/The larger stream was placid, and even <xex>sullen</xex>, in its course.<rdquo/  <rj><au>Sir W. Scott.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><syn><b>Syn.</b> -- Sulky; sour; cross; ill-natured; morose; peevish; fretful; ill-humored; petulant; gloomy; malign; intractable.</syn>  -- <usage><er>Sullen</er>, <er>Sulky</er>. Both <xex>sullen</xex> and <xex>sulky</xex> show themselves in the demeanor. <xex>Sullenness</xex> seems to be an habitual sulkiness, and <xex>sulkiness</xex> a temporary sullenness. The former may be an innate disposition; the latter, a disposition occasioned by recent injury. Thus we are in a <xex>sullen</xex> mood, and in a <xex>sulky</xex> fit.</usage><br/
[<source>1913 Webster</source>]</p>

<p><q>No cheerful breeze this <qex>sullen</qex> region knows;<br/
The dreaded east is all the wind that blows.</q> <rj><qau>Pope.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p>-- <wordforms><wf>Sul"len*ly</wf>, <pos>adv.</pos> -- <wf>Sul"len*ness</wf>, <pos>n.</pos></wordforms><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sullen</ent><br/
<hw>Sul"len</hw>, <pos>n.</pos> <sn>1.</sn> <def>One who is solitary, or lives alone; a hermit.</def> <mark>[Obs.]</mark>  <rj><au>Piers Plowman.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <pluf>pl.</pluf> <def>Sullen feelings or manners; sulks; moroseness; <as>as, to have the <ex>sullens</ex></as>.</def> <mark>[Obs.]</mark>  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sullen</ent><br/
<hw>Sul"len</hw>, <pos>v. t.</pos> <def>To make sullen or sluggish.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q><qex>Sullens</qex> the whole body with . . . laziness.</q> <rj><qau>Feltham.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sullevate</ent><br/
<hw>Sul"le*vate</hw> <pr>(?)</pr>, <pos>v. t.</pos> <ety>[L. <ets>sublevare</ets> to raise up.  Cf. <er>Sublevation</er>.]</ety> <def>To rouse; to excite.</def> <mark>[Obs.]</mark>  <rj><au>Daniel.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><-- p. 1443 --></p>

<p><ent>Sulliage</ent><br/
<hw>Sul"li*age</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Cf. <er>Sullage</er>, <er>Suillage</er>, or <er>Sully</er>, <pos>v. t.</pos>]</ety> <def>Foulness; filth.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>Though we wipe away with never so much care the dirt thrown at us, there will be left some <qex>sulliage</qex> behind.</q> <rj><qau>Gov. of Tongue.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sully</ent><br/
<hw>Sul"ly</hw> <pr>(?)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Sullied</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Sullying</conjf> <pr>(?)</pr>.]</vmorph> <ety>[OE. <ets>sulien</ets>, AS. <ets>sylian</ets>, fr. <ets>sol</ets> mire; akin to G. <ets>suhle</ets> mire, <ets>sich</ets>, <ets>s<uum/hlen</ets> to wallow, Sw. <ets>s<oum/la</ets> to bemire, Dan. <ets>s<oum/le</ets>, Goth. bi<ets>saulijan</ets> to defile.]</ety> <def>To soil; to dirty; to spot; to tarnish; to stain; to darken; -- used literally and figuratively; <as>as, to <ex>sully</ex> a sword; to <ex>sully</ex> a person's reputation</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Statues <qex>sullied</qex> yet with sacrilegious smoke.</q> <rj><qau>Roscommon.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>No spots to <qex>sully</qex> the brightness of this solemnity.</q> <rj><qau>Atterbury.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sully</ent><br/
<hw>Sul"ly</hw>, <pos>v. i.</pos> <def>To become soiled or tarnished.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Silvering will <qex>sully</qex> and canker more than gilding.</q> <rj><qau>Bacon.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sully</ent><br/
<hw>Sul"ly</hw>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Sullies</plw> <pr>(<?/)</pr>.</plu> <def>Soil; tarnish; stain.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>A noble and triumphant merit breaks through little spots and <qex>sullies</qex> in his reputation.</q> <rj><qau>Spectator.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sulphacid</ent><br/
<hw>Sulph*ac"id</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[<ets>Sulpho-</ets> + <ets>acid</ets>.]</ety> <fld>(Chem.)</fld> <def>An acid in which, to a greater or less extent, sulphur plays a part analogous to that of oxygen in an oxyacid; thus, thiosulphuric and sulpharsenic acids are <xex>sulphacids</xex>; -- called also <altname>sulphoacid</altname>. See the Note under <er>Acid</er>, <pos>n.</pos>, 2.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sulphamate</ent><br/
<hw>Sulph*am"ate</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Chem.)</fld> <def>A salt of sulphamic acid.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sulphamic</ent><br/
<hw>Sulph*am"ic</hw> <pr>(?)</pr>, <pos>a.</pos> <fld>(Chem.)</fld> <def>Of or pertaining to a sulphamide; derived from, or related to, a sulphamide; specifically, designating an amido acid derivative, <chform>NH2.SO2.OH</chform>, of sulphuric acid (analogous to sulphonic acid) which is not known in the free state, but is known in its salts.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sulphamide</ent><br/
<hw>Sulph*am"ide</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Chem.)</fld> <def>Any one of a series of amido compounds obtained by treating sulphuryl chloride with various amines.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sulphanilic</ent><br/
<hw>Sulph`a*nil"ic</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[From <ets>sulph</ets>uric + <ets>ani</ets>lene.]</ety> <fld>(Chem.)</fld> <def>Of, pertaining to, or designating, an anilene sulphonic acid which is obtained as a white crystalline substance.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sulphantimonate</ent><br/
<hw>Sulph*an`ti*mo"nate</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Chem.)</fld> <def>A salt of sulphantimonic acid.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sulphantimonic</ent><br/
<hw>Sulph*an`ti*mon"ic</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[<ets>Sulph</ets>o- + <ets>antimonic</ets>.]</ety> <fld>(Chem.)</fld> <def>Of, pertaining to, or designating, a hypothetical sulphacid of antimony (called also <xex>thioantimonic acid</xex>) analogous to sulpharsenic acid.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sulphantimonious</ent><br/
<hw>Sulph*an`ti*mo"ni*ous</hw> <pr>(?)</pr>, <pos>a.</pos> <fld>(Chem.)</fld> <def>Of, pertaining to, or designating, a hypothetical sulphacid of antimony (called also <xex>thioantimonious acid</xex>) analogous to sulpharsenious acid.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sulphantimonite</ent><br/
<hw>Sulph*an"ti*mo*nite`</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Chem.)</fld> <def>A salt of sulphantimonious acid.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sulpharsenate</ent><br/
<hw>Sulph*ar"se*nate</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Chem.)</fld> <def>A salt of sulpharsenic acid.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sulpharsenic</ent><br/
<hw>Sulph`ar*sen"ic</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[<ets>Sulph</ets>o- + <ets>arsenic</ets>.]</ety> <fld>(Chem.)</fld> <def>Of, pertaining to, or designating, a hypothetical sulphacid (called also <xex>thioarsenic acid</xex>) analogous to arsenic acid, and known only in its salts.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sulpharsenious</ent><br/
<hw>Sulph`ar*se"ni*ous</hw> <pr>(?)</pr>, <pos>a.</pos> <fld>(Chem.)</fld> <def>Of, pertaining to, or designating, a hypothetical sulphacid (called also <xex>thioarsenious acid</xex>) analogous to arsenious acid, and known only in its salts.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sulpharsenite</ent><br/
<hw>Sulph*ar"se*nite</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Chem.)</fld> <def>A salt of sulpharsenious acid.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sulphate</ent><br/
<hw>Sul"phate</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[NL. <ets>sulphas</ets>, <ets>sulphatis</ets>, fr. L. <ets>sulphur</ets>, <ets>sulfur</ets>, brimstone, sulphur: cf. F. <ets>sulfate</ets>.]</ety> <fld>(Chem.)</fld> <def>A salt of sulphuric acid.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sulphatic</ent><br/
<hw>Sul*phat"ic</hw> <pr>(?)</pr>, <pos>a.</pos> <fld>(Chem.)</fld> <def>Of, pertaining to, resembling, or containing, a sulphate or sulphates.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sulphato-</ent><br/
<hw>Sul"pha*to-</hw> <pr>(?)</pr>. <fld>(Chem.)</fld> <def>A combining form (also used adjectively) denoting a <xex>sulphate as an ingredient</xex> in certain double salts; <as>as, <ex>sulphato</ex>-carbonate</as>.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sulphaurate</ent><br/
<hw>Sulph*au"rate</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Chem.)</fld> <def>A salt of sulphauric acid.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sulphauric</ent><br/
<hw>Sulph*au"ric</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[<ets>Sulph</ets>o- + <ets>aurum</ets>.]</ety> <fld>(Chem.)</fld> <def>Of, pertaining to, or designating, a hypothetical sulphacid of gold (<xex>aurum</xex>), known only in its salts.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sulphide</ent><br/
<hw>Sul"phide</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Chem.)</fld> <def>A binary compound of sulphur, or one so regarded; -- formerly called <altname>sulphuret</altname>.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Double sulphide</b></col> <fld>(Chem.)</fld>, <cd>a compound of two sulphides.</cd> -- <col><b>Hydrogen sulphide</b></col>. <fld>(Chem.)</fld> <cd>See under <er>Hydrogen</er>.</cd> -- <col><b>Metallic sulphide</b></col>, <cd>a binary compound of sulphur with a metal.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sulphinate</ent><br/
<hw>Sul"phi*nate</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Chem.)</fld> <def>A salt of a sulphinic acid.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sulphindigotic</ent><br/
<hw>Sulph*in`di*got"ic</hw> <pr>(?)</pr>, <pos>a.</pos> <fld>(Chem.)</fld> <def>Of, pertaining to, or designating, a sulphonic acid obtained, as a blue solution, by dissolving indigo in sulphuric acid; -- formerly called also <altname>cerulic sulphuric</altname> acid, but properly called <altname>indigo-disulphonic acid</altname>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sulphine</ent><br/
<hw>Sul"phine</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Chem.)</fld> <def>Any one of a series of basic compounds which consist essentially of sulphur united with hydrocarbon radicals. In general they are oily or crystalline deliquescent substances having a peculiar odor; as, <stype>trimethyl sulphine</stype>, <chform>(CH3)3S.OH</chform>.  Cf. <er>Sulphonium</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sulphinic</ent><br/
<hw>Sul*phin"ic</hw> <pr>(?)</pr>, <pos>a.</pos> <fld>(Chem.)</fld> <def>Of, pertaining to, or designating, any one of a series of acids regarded as acid ethereal salts of hyposulphurous acid; <as>as, methyl <ex>sulphinic</ex> acid, <chform>CH3.SO.OH</chform>, a thick unstable liquid</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sulphinide</ent><br/
<hw>Sul"phi*nide</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[<ets>Sulph</ets>o- + am<ets>in</ets>e + anhydr<ets>ide</ets>.]</ety> <fld>(Chem.)</fld> <def>A white or yellowish crystalline substance, <chform>C6H4.(SO2.CO).NH</chform>, produced artificially by the oxidation of a sulphamic derivative of toluene. It is the sweetest substance known, having over two hundred times the sweetening power of sugar, and is known in commerce under the name of <altname>saccharine</altname>. It has acid properties and forms salts (which are inaccurately called <xex>saccharinates</xex>).</def>  <rj><au>I. Remsen.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sulphion</ent><br/
<hw>Sul"phi*on</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[<ets>Sulph</ets>o- + <ets>ion</ets>.]</ety> <fld>(Chem.)</fld> <def>A hypothetical radical, <chform>SO4</chform>, regarded as forming the acid or negative constituent of sulphuric acid and the sulphates in electrolytic decomposition; -- so called in accordance with the binary theory of salts.</def> <altsp>[Written also <asp>sulphione</asp>.]</altsp><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sulphionide</ent><br/
<hw>Sulph*i"on*ide</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Chem.)</fld> <def>A binary compound of sulphion, or one so regarded; <as>thus, sulphuric acid, <chform>H2SO4</chform>, is a <ex>sulphionide</ex></as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sulphite</ent><br/
<hw>Sul"phite</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Cf. F. <ets>sulfite</ets>. See <er>Sulphur</er>.]</ety> <sn>1.</sn> <fld>(Chem.)</fld> <def>A salt of sulphurous acid; -- called also <altname>sulfite</altname>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A person who is spontaneous and original in his habits of thought and conversation.</def> <mark>[Archaic Slang]</mark> -- <wordforms><wf>Sul*phit"ic</wf> <pr>(#)</pr>, <pos>a.</pos> <mark>[Slang]</mark></wordforms></p>

<p><q>A <qex>sulphite</qex> is a person who does his own thinking, he is a person who has surprises up his sleeve. He is explosive.</q>  <rj><qau>Gelett Burgess.</qau></rj><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Sulpho-</ent><br/
<hw>Sul"pho-</hw> <pr>(?)</pr>. <fld>(Chem.)</fld> <def>A prefix (also used adjectively) designating <xex>sulphur as an ingredient</xex> in certain compounds.  Cf. <er>Thio-</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sulphoarsenic</ent><br/
<hw>Sul`pho*ar*sen"ic</hw> <pr>(?)</pr>, <pos>a.</pos> <fld>(Chem.)</fld> <def>Of, pertaining to, or containing, sulphur and arsenic; -- said of an acid which is the same as arsenic acid with the substitution of sulphur for oxygen.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sulphocarbonate</ent><br/
<hw>Sul`pho*car"bon*ate</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Chem.)</fld> <def>A salt of sulphocarbonic acid; a thiocarbonate.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sulphocarbonic</ent><br/
<hw>Sul`pho*car*bon"ic</hw> <pr>(?)</pr>, <pos>a.</pos> <fld>(Chem.)</fld> <def>Of, pertaining to, or designating, a sulphacid, <chform>H2CSO2</chform> (called also <altname>thiocarbonic</altname> <it>acid</it>), or an acid, <chform>H2CS3</chform>, analogous to carbonic acid, obtained as a yellow oily liquid of a pungent odor, and forming salts.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sulphocyanate</ent><br/
<hw>Sul`pho*cy"a*nate</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Chem.)</fld> <def>A salt of sulphocyanic acid; -- also called <altname>thiocyanate</altname>, and formerly inaccurately <altname>sulphocyanide</altname>.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Ferric sulphocyanate</b></col> <fld>(Chem.)</fld>, <cd>a dark red crystalline substance usually obtained in a blood-red solution, and recognized as a test for ferric iron.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sulphocyanic</ent><br/
<hw>Sul`pho*cy*an"ic</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[See <er>Sulphur</er>, <er>Cyanic</er>.]</ety> <fld>(Chem.)</fld> <def>Of, pertaining to, derived from, or designating, a sulphacid, <chform>HSCN</chform>, analogous to cyanic acid, and obtained as a colorless deliquescent crystalline substance, having a bitter saline taste, and not poisonous.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sulphocyanide</ent><br/
<hw>Sul`pho*cy"a*nide</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Chem.)</fld> <def>See <er>Sulphocyanate</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sulphocyanogen</ent><br/
<hw>Sul`pho*cy*an"o*gen</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Chem.)</fld> <def>See <er>Persulphocyanogen</er>.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sulphonal</ent><br/
<hw>Sul"pho*nal</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Med.)</fld> <def>A substance employed as a hypnotic, produced by the union of mercaptan and acetone.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sulphonate</ent><br/
<hw>Sul"pho*nate</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Chem.)</fld> <def>A salt of sulphonic acid.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sulphone</ent><br/
<hw>Sul"phone</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Chem.)</fld> <def>Any one of a series of compounds analogous to the ketones, and consisting of the sulphuryl group united with two hydrocarbon radicals; <as>as, dimethyl <ex>sulphone</ex>, (CH<?/)<?/</as>.SO<?/.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sulphonic</ent><br/
<hw>Sul*phon"ic</hw> <pr>(?)</pr>, <pos>a.</pos> <fld>(Chem.)</fld> <def>Pertaining to, or derived from, a sulphone; -- used specifically to designate any one of a series of acids (regarded as acid ethereal salts of sulphurous acid) obtained by the oxidation of the mercaptans, or by treating sulphuric acid with certain aromatic bases (as benzene); <as>as, phenyl <ex>sulphonic</ex> acid, <chform>C6H5.SO2.OH</chform>, a stable colorless crystalline substance</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Sulphonic group</b></col> <fld>(Chem.)</fld>, <cd>the hypothetical radical, <chform>SO2.OH</chform>, the characteristic residue of sulphonic acids.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sulphonium</ent><br/
<hw>Sul*pho"ni*um</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[<ets>Sulph</ets>ur + amm<ets>onium</ets>.]</ety> <fld>(Chem.)</fld> <def>A hypothetical radical, <chform>SH3</chform>, regarded as the type and nucleus of the sulphines.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sulphophosphate</ent><br/
<hw>Sul`pho*phos"phate</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Chem.)</fld> <def>A salt of sulphophosphoric acid.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sulphophosphite</ent><br/
<hw>Sul`pho*phos"phite</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Chem.)</fld> <def>A salt of sulphophosphorous acid.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sulphophosphoric</ent><br/
<hw>Sul`pho*phos*phor"ic</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Of, pertaining to, or designating, a hypothetical sulphacid of phosphorus, analogous to phosphoric acid, and known in its salts.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sulphophosphorous</ent><br/
<hw>Sul`pho*phos"phor*ous</hw> <pr>(?)</pr>, <pos>a.</pos> <fld>(Chem.)</fld> <def>Of, pertaining to, or designating, a hypothetical acid of phosphorus, analogous to phosphorous acid, and known in its salts.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sulphosalt</ent><br/
<hw>Sul"pho*salt`</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Chem.)</fld> <def>A salt of a sulphacid.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sulphostannate</ent><br/
<hw>Sul`pho*stan"nate</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Chem.)</fld> <def>A salt of sulphostannic acid.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sulphostannic</ent><br/
<hw>Sul`pho*stan"nic</hw> <pr>(?)</pr>, <pos>a.</pos> <fld>(Chem.)</fld> <def>Of, pertaining to, or designating, a sulphacid of tin (more exactly called <xex>metasulphostannic acid</xex>), which is obtained as a dark brown amorphous substance, H<?/SnS<?/, forming a well-known series of salts.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sulphotungstate</ent><br/
<hw>Sul`pho*tung"state</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Chem.)</fld> <def>A salt of sulphotungstic acid.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sulphotungstic</ent><br/
<hw>Sul`pho*tung"stic</hw> <pr>(?)</pr>, <pos>a.</pos> <fld>(Chem.)</fld> <def>Of, pertaining to, or designating, hypothetical sulphacid of tungsten (called also <xex>sulphowolframic acid</xex>), analogous to sulphuric acid, and known in its salts.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sulphovinic</ent><br/
<hw>Sul`pho*vin"ic</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[<ets>Sulpho-</ets> + <ets>vinum</ets> wine: cf. F. <ets>sulfovinique</ets>.]</ety> <fld>(Chem.)</fld> <def>Of, pertaining to, and formerly designating, ethylsulphuric acid.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sulphur</ent><br/
<hw>Sul"phur</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L., better <ets>sulfur</ets>: cf. F. <ets>soufre</ets>.]</ety> <sn>1.</sn> <fld>(Chem.)</fld> <def>A nonmetallic element occurring naturally in large quantities, either combined as in the sulphides (as <xex>pyrites</xex>) and sulphates (as <xex>gypsum</xex>), or native in volcanic regions, in vast beds mixed with gypsum and various earthy materials, from which it is melted out. Symbol S. Atomic weight 32. The specific gravity of ordinary octohedral sulphur is 2.05; of prismatic sulphur, 1.96.</def><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ It is purified by distillation, and is obtained as a lemon-yellow powder (by sublimation), called <xex>flour</xex>, or <xex>flowers</xex>, <xex>of sulphur</xex>, or in cast sticks called <xex>roll sulphur</xex>, or <xex>brimstone</xex>. It burns with a blue flame and a peculiar suffocating odor. It is an ingredient of gunpowder, is used on friction matches, and in medicine (as a laxative and insecticide), but its chief use is in the manufacture of sulphuric acid. Sulphur can be obtained in two crystalline modifications, in orthorhombic octahedra, or in monoclinic prisms, the former of which is the more stable at ordinary temperatures. Sulphur is the type, in its chemical relations, of a group of elements, including <xex>selenium</xex> and <xex>tellurium</xex>, called collectively the <xex>sulphur group</xex>, or <xex>family</xex>. In many respects sulphur resembles oxygen.</note><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Zool.)</fld> <def>Any one of numerous species of yellow or orange butterflies of the subfamily <fam>Pierinae</fam>; <as>as, the clouded <ex>sulphur</ex> (<spn>Eurymus philodice</spn> syn. <spn>Colias philodice</spn>), which is the common yellow butterfly of the Eastern United States</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Amorphous sulphur</b></col> <fld>(Chem.)</fld>, <cd>an elastic variety of sulphur of a resinous appearance, obtained by pouring melted sulphur into water. On standing, it passes back into a brittle crystalline modification.</cd> -- <col><b>Liver of sulphur</b></col>. <fld>(Old Chem.)</fld> <cd>See <er>Hepar</er>.</cd> -- <col><b>Sulphur acid</b></col>. <fld>(Chem.)</fld> <cd>See <er>Sulphacid</er>.</cd> -- <col><b>Sulphur alcohol</b></col>. <fld>(Chem.)</fld> <cd>See <er>Mercaptan</er>.</cd> -- <col><b>Sulphur auratum</b></col> <ety>[L.]</ety> <fld>(Old Chem.)</fld>, <cd>a golden yellow powder, consisting of antimonic sulphide, <chform>Sb2S5</chform>, -- formerly a famous nostrum.</cd> -- <col><b>Sulphur base</b></col> <fld>(Chem.)</fld>, <cd>an alkaline sulphide capable of acting as a base in the formation of sulphur salts according to the old dual theory of salts.</cd> <mark>[Archaic]</mark> -- <col><b>Sulphur dioxide</b></col> <fld>(Chem.)</fld>, <cd>a colorless gas, <chform>SO2</chform>, of a pungent, suffocating odor, produced by the burning of sulphur. It is employed chiefly in the production of sulphuric acid, and as a reagent in bleaching; -- called also <altname>sulphurous anhydride</altname>, and formerly <altname>sulphurous acid</altname>.</cd> -- <col><b>Sulphur ether</b></col> <fld>(Chem.)</fld>, <cd>a sulphide of hydrocarbon radicals, formed like the ordinary ethers, which are oxides, but with sulphur in the place of oxygen.</cd> -- <col><b>Sulphur salt</b></col> <fld>(Chem.)</fld>, <cd>a salt of a sulphacid; a sulphosalt.</cd> -- <col><b>Sulphur showers</b></col>, <cd>showers of yellow pollen, resembling sulphur in appearance, often carried from pine forests by the wind to a great distance.</cd> -- <col><b>Sulphur trioxide</b></col> <fld>(Chem.)</fld>, <cd>a white crystalline solid, <chform>SO3</chform>, obtained by oxidation of sulphur dioxide. It dissolves in water with a hissing noise and the production of heat, forming sulphuric acid, and is employed as a dehydrating agent. Called also <altname>sulphuric anhydride</altname>, and formerly <altname>sulphuric acid</altname>.</cd> -- <col><b>Sulphur whale</b></col>. <fld>(Zool.)</fld> <cd>See <er>Sulphur-bottom</er>.</cd> -- <col><b>Vegetable sulphur</b></col> <fld>(Bot.)</fld>, <cd>lycopodium powder. See under <er>Lycopodium</er>.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sulphurate</ent><br/
<hw>Sul"phu*rate</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>sulphuratus</ets>, <ets>sulfuratus</ets>.]</ety> <def>Sulphureous.</def> <mark>[Poetic & R.]</mark>  <rj><au>Dr. H. More.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sulphurate</ent><br/
<hw>Sul"phu*rate</hw> <pr>(?)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Sulphurated</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Sulphurating</conjf>.]</vmorph> <fld>(Chem.)</fld> <def>To sulphurize.</def> <mark>[Archaic]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sulphuration</ent><br/
<hw>Sul`phu*ra"tion</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Cf. F. <ets>sulfuration</ets>, L. <ets>sulphuratio</ets>, <ets>sulfuratio</ets>, a vein of sulphur.]</ety> <def>The act or process of combining or impregnating with sulphur or its compounds; also, the state of being so combined or impregnated.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sulphurator</ent><br/
<hw>Sul"phu*ra`tor</hw> <pr>(?)</pr>, <pos>n.</pos> <def>An apparatus for impregnating with, or exposing to the action of, sulphur; especially, an apparatus for fumigating or bleaching by means of the fumes of burning sulphur.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sulphur-bottom</ent><br/
<hw>Sul`phur-bot"tom</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Zool.)</fld> <def>A very large whalebone whale of the genus <gen>Sibbaldius</gen>, having a yellowish belly; especially, <spn>Sibbaldius sulfureus</spn> of the North Pacific, and <spn>Sibbaldius borealis</spn> of the North Atlantic; -- called also <altname>sulphur whale</altname>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sulphureity</ent><br/
<hw>Sul`phu*re"i*ty</hw> <pr>(?)</pr>, <pos>n.</pos> <def>The quality or state of being sulphureous.</def> <mark>[Obs.]</mark>  <rj><au>B. Jonson.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sulphureous</ent><br/
<hw>Sul*phu"re*ous</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>sulphureus</ets>, <ets>sulfureus</ets>.]</ety> <def>Consisting of sulphur; having the qualities of sulphur, or brimstone; impregnated with sulphur.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Her snakes united, <qex>sulphureous</qex> waters drink.</q> <rj><qau>Pope.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p>-- <wordforms><wf>Sul*phu"re*ous*ly</wf>, <pos>adv.</pos> -- <wf>Sul*phu"re*ous*ness</wf>, <pos>n.</pos></wordforms><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sulphuret</ent><br/
<hw>Sul"phu*ret</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Chem.)</fld> <def>A sulphide; <as>as, a <ex>sulphuret</ex> of potassium</as>.</def> <mark>[Obsoles.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sulphureted</ent><br/
<hw>Sul"phu*ret`ed</hw>, <pos>a.</pos> <fld>(Chem.)</fld> <def>Combined or impregnated with sulphur; sulphurized.</def> <altsp>[Written also <asp>sulphuretted</asp>.]</altsp><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Sulphureted hydrogen</b></col>. <fld>(Chem.)</fld> <cd>See <cref>Hydrogen sulphide</cref>, under <er>Hydrogen</er>.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sulphuric</ent><br/
<hw>Sul*phu"ric</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[Cf. F. <ets>sulfurique</ets>.]</ety> <sn>1.</sn> <def>Of or pertaining to sulphur; <as>as, a <ex>sulphuric</ex> smell</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Chem.)</fld> <def>Derived from, or containing, sulphur; specifically, designating those compounds in which the element has a higher valence as contrasted with the <xex>sulphurous</xex> compounds; <as>as, <ex>sulphuric</ex> acid</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Sulphuric acid</b></col>. <sd>(a)</sd> <cd>Sulphur trioxide (see under <er>Sulphur</er>); -- formerly so called on the dualistic theory of salts.</cd> <mark>[Obs.]</mark> <sd>(b)</sd> <cd>A heavy, corrosive, oily liquid, <chform>H2SO4</chform>, colorless when pure, but usually yellowish or brownish, produced by the combined action of sulphur dioxide, oxygen (from the air), steam, and nitric fumes. It attacks and dissolves many metals and other intractable substances, sets free most acids from their salts, and is used in the manufacture of hydrochloric and nitric acids, of soda, of bleaching powders, etc. It is also powerful dehydrating agent, having a strong affinity for water, and eating and corroding paper, wood, clothing, etc.   It is thus used in the manufacture of ether, of imitation parchment, and of nitroglycerin. It is also used in etching iron, in removing iron scale from forgings, in petroleum refining, etc., and in general its manufacture is the most important and fundamental of all the chemical industries. Formerly called <altname>vitriolic acid</altname>, and now popularly <altname>vitriol</altname>, and <altname>oil of vitriol</altname>.</cd> -- <mcol><col><b>Fuming sulphuric acid</b></col>, <it>or</it> <col><b>Nordhausen sulphuric acid</b></col></mcol>. <cd>See <cref>Disulphuric acid</cref>, under <er>Disulphuric</er>.</cd> -- <col><b>Sulphuric anhydride</b></col>, <cd>sulphur trioxide. See under <er>Sulphur</er>.</cd> -- <col><b>Sulphuric ether</b></col>, <cd>common anaesthetic ether; -- so called because made by the catalytic action of sulphuric acid on alcohol. See <er>Ether</er>, 3 <sd>(a)</sd>.</cd><-- now usu. ether or ethyl ether. --></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sulphurine</ent><br/
<hw>Sul"phur*ine</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Sulphureous.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sulphuring</ent><br/
<hw>Sul"phur*ing</hw>, <pos>n.</pos> <def>Exposure to the fumes of burning sulphur, as in bleaching; the process of bleaching by exposure to the fumes of sulphur.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sulphurize</ent><br/
<hw>Sul"phur*ize</hw> <pr>(?)</pr>, <pos>v. t.</pos> <fld>(Chem.)</fld> <def>To combine or impregnate with sulphur or any of its compounds; <as>as, to <ex>sulphurize</ex> caoutchouc in vulcanizing</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><-- p. 1444 --></p>

<p><ent>Sulphurous</ent><br/
<hw>Sul"phur*ous</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>sulphurosus</ets>, <ets>sulfurosus</ets>: cf. F. <ets>sulfureux</ets>.]</ety> <sn>1.</sn> <def>Of or pertaining to sulphur.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Chem.)</fld> <sd>(a)</sd> <def>Derived from, or containing, sulphur; specifically, designating those compounds in which the element has a lower valence as contrasted with the <xex>sulphuric</xex> compounds.</def> <sd>(b)</sd> <def>Having the characteristic odor of sulphur dioxide, or of hydrogen sulphide, or of other sulphur compounds.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Sulphurous acid</b></col>. <sd>(a)</sd> <cd>Sulphur dioxide. See under <er>Sulphur</er>.</cd> <mark>[Obs.]</mark> <sd>(b)</sd> <cd>An acid, <chform>H2SO3</chform>, not known in the free state except as a solution of sulphur dioxide in water, but forming a well-known series of salts (the <xex>sulphites</xex>).</cd> -- <col><b>Sulphurous anhydride</b></col> <fld>(Chem.)</fld>, <cd>sulphur dioxide. See under <er>Sulphur</er>.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sulphurwort</ent><br/
<hw>Sul"phur*wort`</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Bot.)</fld> <def>The hog's fennel. See under <er>Fennel</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sulphury</ent><br/
<hw>Sul"phur*y</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Resembling, or partaking of the nature of, sulphur; having the qualities of sulphur.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sulphuryl</ent><br/
<hw>Sul"phur*yl</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[<ets>Sulphur</ets> + <ets>-yl</ets>.]</ety> <fld>(Chem.)</fld> <def>The hypothetical radical <chform>SO2</chform>; -- called also <altname>sulphon</altname>.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Sulphuryl chloride</b></col>, <cd>a chloride, pungent, fuming liquid, <chform>SO2.Cl2</chform>, obtained by the action of phosphorus pentachloride on sulphur trioxide. On treatment with water it decomposes into sulphuric and hydrochloric acids, and is hence called also <altname>sulphuric chloranhydride</altname>.</cd></cs><-- used in organic chemistry to prepare acyl chlorides fro organic acids. --><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sulphydrate</ent><br/
<hw>Sul*phy"drate</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Chem.)</fld> <def>A compound, analogous to a hydrate, regarded as a salt of sulphydric acid, or as a derivative of hydrogen sulphide in which one half of the hydrogen is replaced by a base (as potassium <xex>sulphydrate</xex>, <chform>KSH</chform>), or as a hydrate in which the oxygen has been wholly or partially replaced by sulphur.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sulphydric</ent><br/
<hw>Sul*phy"dric</hw> <pr>(?)</pr>, <pos>a.</pos> <fld>(Chem.)</fld> <def>Of, pertaining to, or designating, hydrogen sulphide, which is regarded as an acid, especially when in solution.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sulpician</ent><br/
<hw>Sul*pi"cian</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[So called after the parish of St. <ets>Sulpice</ets> in Paris, of which the founder, Jean Jacques Olier, was pastor in 1643.]</ety> <fld>(R. C. Ch.)</fld> <def>One of an order of priests established in France in 1642 to educate men for the ministry. The order was introduced soon afterwards into Canada, and in 1791 into the United States.</def> <altsp>[Written also <asp>Sulpitian</asp>.]</altsp><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sultan</ent><br/
<hw>Sul"tan</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F. <ets>sultan</ets> (cf. Sp. <ets>soldan</ets>, It. <ets>sultano</ets>, <ets>soldano</ets>), Ar. <ets>sult<amac/n</ets> sultan, dominion.  Cf. <er>Soldan</er>.]</ety> <def>A ruler, or sovereign, of a Mohammedan state; specifically, the ruler of the Turks; the Padishah, or Grand Seignior; -- officially so called.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Sultan flower</b></col>. <fld>(Bot.)</fld> <cd>See <cref>Sweet sultan</cref>, under <er>Sweet</er>.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sultana</ent><br/
<hw>Sul*ta"na</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[It.]</ety> <sn>1.</sn> <def>The wife of a sultan; a sultaness.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <pluf>pl.</pluf> <def>A kind of seedless raisin produced near <city>Smyrna</city> in Asiatic Turkey.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Sultana bird</b></col> <fld>(Zool.)</fld>, <cd>the hyacinthine, or purple, gallinule. See <xex>Illust.</xex> under <er>Gallinule</er>.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sultanate</ent><br/
<hw>Sul"tan*ate</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Cf. F. <ets>sultanat</ets>.]</ety> <sn>1.</sn> <def>The rule or dominion of a sultan; sultanship.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>The territory ruled by a sultan.</def><br/
<syn><b>Syn. --</b> soldanrie.</syn><br/
[<source>PJC</source>]</p>

<p><ent>Sultaness</ent><br/
<hw>Sul"tan*ess</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A sultana.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sultanic</ent><br/
<hw>Sul*tan"ic</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Pertaining to a sultan.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sultan-red</ent><br/
<hw>Sul"tan-red`</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Having a deep red color.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sultanry</ent><br/
<hw>Sul"tan*ry</hw> <pr>(?)</pr>, <pos>n.</pos> <def>The dominions of a sultan.</def>  <rj><au>Bacon.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sultanship</ent><br/
<hw>Sul"tan*ship</hw>, <pos>n.</pos> <def>The office or dignity of a sultan.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sultany</ent><br/
<hw>Sul"tan*y</hw> <pr>(?)</pr>, <pos>n.</pos> <def>Sultanry.</def> <mark>[Obs.]</mark>  <rj><au>Fuller.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sultrily</ent><br/
<hw>Sul"tri*ly</hw> <pr>(?)</pr>, <pos>adv.</pos> <def>In a sultry manner.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sultriness</ent><br/
<hw>Sul"tri*ness</hw>, <pos>n.</pos> <def>The quality or state of being sultry.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sultry</ent><br/
<hw>Sul"try</hw> <pr>(s<ucr/l"tr<ycr/)</pr>, <pos>a.</pos> <amorph>[<pos>Compar.</pos> <adjf>Sultrier</adjf> <pr>(s<ucr/l"tr<icr/*<etil/r)</pr>; <pos>superl.</pos> <adjf>Sultriest</adjf>.]</amorph> <ety>[From <er>Sweltry</er>.]</ety> <sn>1.</sn> <def>Very hot, burning, and oppressive; <as>as, Libya's <ex>sultry</ex> deserts</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Such as, born beneath the burning sky<br/
And <qex>sultry</qex> sun, betwixt the tropics lie.</q> <rj><qau>Dryden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Very hot and humid, or hot, close, stagnant, and oppressive, as air.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>When in the <qex>sultry</qex> glebe I faint,<br/
Or on the thirsty mountain plant.</q> <rj><qau>Addison.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sulu</ent><br/
<hw>Su*lu"</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Malay <ets>Suluk</ets>.]</ety> <def>A member of the most prominent tribe of the Moro tribes, occupying the Sulu Archipelago; also, their language.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Sum</ent><br/
<hw>Sum</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[OE. <ets>summe</ets>, <ets>somme</ets>, OF. <ets>sume</ets>, <ets>some</ets>, F. <ets>somme</ets>, L. <ets>summa</ets>, fr. <ets>summus</ets> highest, a superlative from <ets>sub</ets> under. See <er>Sub-</er>, and cf. <er>Supreme</er>.]</ety> <sn>1.</sn> <def>The aggregate of two or more numbers, magnitudes, quantities, or particulars; the amount or whole of any number of individuals or particulars added together; <as>as, the <ex>sum</ex> of 5 and 7 is 12</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Take ye the <qex>sum</qex> of all the congregation.</q> <rj><qau>Num. i. 2.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ <xex>Sum</xex> is now commonly applied to an aggregate of numbers, and <xex>number</xex> to an aggregate of persons or things.</note><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A quantity of money or currency; any amount, indefinitely; <as>as, a <ex>sum</ex> of money; a small <ex>sum</ex>, or a large <ex>sum</ex></as>.</def> <ldquo/The <xex>sum</xex> of forty pound.<rdquo/  <rj><au>Chaucer.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>With a great <qex>sum</qex> obtained I this freedom.</q> <rj><qau>Acts xxii. 28.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>The principal points or thoughts when viewed together; the amount; the substance; compendium; <as>as, this is the <ex>sum</ex> of all the evidence in the case; this is the <ex>sum</ex> and substance of his objections</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>Height; completion; utmost degree.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Thus have I told thee all my state, and brought<br/
My story to the <qex>sum</qex> of earthly bliss.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <fld>(Arith.)</fld> <def>A problem to be solved, or an example to be wrought out.</def>  <rj><au>Macaulay.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>A <qex>sum</qex> in arithmetic wherein a flaw discovered at a particular point is <qex>ipso facto</qex> fatal to the whole.</q> <rj><qau>Gladstone.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>A large sheet of paper . . . covered with long <qex>sums</qex>.</q> <rj><qau>Dickens.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Algebraic sum</b></col>, <cd>as distinguished from <xex>arithmetical sum</xex>, the aggregate of two or more numbers or quantities taken with regard to their signs, as + or -, according to the rules of addition in algebra; thus, the <xex>algebraic sum</xex> of -2, 8, and -1 is 5.</cd> -- <col><b>In sum</b></col>, <cd>in short; in brief.</cd> <mark>[Obs.]</mark> <ldquo/<xex>In sum</xex>, the gospel . . . prescribes every virtue to our conduct, and forbids every sin.<rdquo/ <au>Rogers.</au></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sum</ent><br/
<hw>Sum</hw>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Summed</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Summing</conjf>.]</vmorph> <ety>[Cf. F. <ets>sommer</ets>, LL. <ets>summare</ets>.]</ety> <sn>1.</sn> <def>To bring together into one whole; to collect into one amount; to cast up, as a column of figures; to ascertain the totality of; -- usually with <xex>up</xex>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The mind doth value every moment, and then the hour doth rather <qex>sum</qex> up the moments, than divide the day.</q> <rj><qau>Bacon.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To bring or collect into a small compass; to comprise in a few words; to condense; -- usually with <xex>up</xex>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q><ldquo/Go to the ant, thou sluggard,<rdquo/ in few words <qex>sums</qex> up the moral of this fable.</q> <rj><qau>L'Estrange.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>He <qex>sums</qex> their virtues in himself alone.</q> <rj><qau>Dryden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <fld>(Falconry)</fld> <def>To have (the feathers) full grown; to furnish with complete, or full-grown, plumage.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>But feathered soon and fledge<br/
They <qex>summed</qex> their pens [wings].</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Summing up</b></col>, <cd>a compendium or abridgment; a recapitulation; a r<eacute/sum<eacute/; a summary.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><syn><b>Syn.</b> -- To cast up; collect; comprise; condense; comprehend; compute.</syn><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sumach</ent><br/
<ent>Sumac</ent><br/
<mhw>{ <hw>Su"mac</hw>, <hw>Su"mach</hw>  }</mhw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F. <ets>sumac</ets>, formerly <ets>sumach</ets> (cf. Sp. <ets>zumaque</ets>), fr. Ar. <ets>summ<amac/q</ets>.]</ety> <altsp>[Written also <asp>shumac</asp>.]</altsp> <sn>1.</sn> <fld>(Bot.)</fld> <def>Any plant of the genus <gen>Rhus</gen>, shrubs or small trees with usually compound leaves and clusters of small flowers. Some of the species are used in tanning, some in dyeing, and some in medicine. One, the Japanese <spn>Rhus vernicifera</spn>, yields the celebrated Japan varnish, or lacquer.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>The powdered leaves, peduncles, and young branches of certain species of the sumac plant, used in tanning and dyeing.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Poison sumac</b></col>. <fld>(Bot.)</fld> <cd>See under <er>Poison</er>.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sumatra leaf</ent><br/
<hw>Su*ma"tra leaf</hw> <pr>(?)</pr>. <def>A thin, elastic, uniformly light-colored tobacco leaf, raised in Sumatra and extensively used for cigar wrappers.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Sumatran</ent><br/
<hw>Su*ma"tran</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Of or pertaining to Sumatra or its inhabitants.</def> -- <def2><pos>n.</pos> <def>A native of Sumatra.</def></def2><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sumbul</ent><br/
<hw>Sum"bul</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Pers.]</ety> <def>The musky root of an Asiatic umbelliferous plant, <spn>Ferula Sumbul</spn>. It is used in medicine as a stimulant.</def> <altsp>[Written also <asp>sumbal</asp>.]</altsp> -- <wordforms><wf>Sum*bul"ic</wf>, <pos>a.</pos></wordforms><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sumerian</ent><br/
<hw>Su*me"ri*an</hw> <pr>(?)</pr>, <pos>a.</pos> <altname>[Written also <asp>Sumirian</asp>.]</altname> <def>Of or pertaining to the region of lower Babylonia, which was anciently called <xex>Sumer</xex>, or its inhabitants or their language.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Sumerian</ent><br/
<hw>Su*me"ri*an</hw>, <pos>n.</pos> <altname>[Written also <asp>Sumirian</asp>.]</altname> <def>A native of lower Babylonia, anciently called Sumer.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Sumless</ent><br/
<hw>Sum"less</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Not to be summed up or computed; so great that the amount can not be ascertained; incalculable; inestimable.</def> <ldquo/Sumless treasure.<rdquo/  <rj><au>Pope.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Summarily</ent><br/
<hw>Sum"ma*ri*ly</hw> <pr>(?)</pr>, <pos>adv.</pos> <def>In a summary manner.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Summarist</ent><br/
<hw>Sum"ma*rist</hw> <pr>(?)</pr>, <pos>n.</pos> <def>One who summarized.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Summarize</ent><br/
<hw>Sum"ma*rize</hw> <pr>(?)</pr>, <pos>v. t.</pos> <def>To comprise in, or reduce to, a summary; to present briefly.</def>  <rj><au>Chambers.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Summary</ent><br/
<hw>Sum"ma*ry</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[Cf. F. <ets>sommaire</ets>. See <er>Sum</er>.]</ety> <sn>1.</sn> <def>Formed into a sum; summed up; reduced into a narrow compass, or into few words; short; brief; concise; compendious; <as>as, a <ex>summary</ex> statement of facts</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Hence, rapidly performed; quickly executed; <as>as, a <ex>summary</ex> process; to take <ex>summary</ex> vengeance</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><syn><b>Syn.</b> -- Short; brief; concise; compendious; succinct.</syn><br/
[<source>1913 Webster</source>]</p>

<p><ent>Summary</ent><br/
<hw>Sum"ma*ry</hw>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Summaries</plw> <pr>(#)</pr>.</plu> <ety>[F. <ets>sommaire</ets>, or L. <ets>summarium</ets>. See <er>Summary</er>, <pos>a.</pos>]</ety> <def>A general or comprehensive statement; an abridged account; an abstract, abridgment, or compendium, containing the sum or substance of a fuller account.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Summation</ent><br/
<hw>Sum*ma"tion</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Cf. F. <ets>sommation</ets>. See <er>Sum</er>, <pos>v. t.</pos>]</ety> <def>The act of summing, or forming a sum, or total amount; also, an aggregate.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Of this series no <qex>summation</qex> is possible to a finite intellect.</q> <rj><qau>De Quincey.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Summer</ent><br/
<hw>Sum"mer</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[From <er>Sum</er>, <pos>v.</pos>]</ety> <def>One who sums; one who casts up an account.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Summer</ent><br/
<hw>Sum"mer</hw>, <pos>n.</pos> <ety>[F. <ets>sommier</ets> a rafter, the same word as <ets>sommier</ets> a beast of burden. See <er>Sumpter</er>.]</ety> <fld>(Arch.)</fld> <def>A large stone or beam placed horizontally on columns, piers, posts, or the like, serving for various uses. Specifically: <sd>(a)</sd> The lintel of a door or window. <sd>(b)</sd> The commencement of a cross vault. <sd>(c)</sd> A central floor timber, as a girder, or a piece reaching from a wall to a girder. Called also <altname>summertree</altname>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Summer</ent><br/
<hw>Sum"mer</hw>, <pos>n.</pos> <ety>[OE. <ets>sumer</ets>, <ets>somer</ets>, AS. <ets>sumor</ets>, <ets>sumer</ets>; akin to OFries. <ets>sumur</ets>, D. <ets>zomer</ets>, OS. <ets>sumar</ets>, G. <ets>sommer</ets>, OHG. & Icel. <ets>sumar</ets>, Dan. <ets>sommer</ets>, Sw. <ets>sommar</ets>, W. <ets>haf</ets>, Zend <ets>hama</ets>, Skr. <ets>sam<amac/</ets> year. <root/292.]</ety> <def>The season of the year in which the sun shines most directly upon any region; the warmest period of the year.</def><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ North of the equator summer is popularly taken to include the months of June, July, and August. Astronomically it may be considered, in the northern hemisphere, to begin with the summer solstice, about June 21st, and to end with the autumnal equinox, about September 22d.</note><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Indian summer</b></col>, <cd>in North America, a period of warm weather late in autumn, usually characterized by a clear sky, and by a hazy or smoky appearance of the atmosphere, especially near the horizon. The name is derived probably from the custom of the Indians of using this time in preparation for winter by laying in stores of food.</cd> -- <col><b>Saint Martin's summer</b></col>. <cd>See under <er>Saint</er>.</cd> -- <col><b>Summer bird</b></col> <fld>(Zool.)</fld>, <cd>the wryneck.</cd> <mark>[Prov. Eng.]</mark> -- <col><b>Summer colt</b></col>, <cd>the undulating state of the air near the surface of the ground when heated.</cd> <mark>[Eng.]</mark> -- <col><b>Summer complaint</b></col> <fld>(Med.)</fld>, <cd>a popular term for any diarrheal disorder occurring in summer, especially when produced by heat and indigestion.</cd> -- <col><b>Summer coot</b></col> <fld>(Zool.)</fld>, <cd>the American gallinule.</cd> <mark>[Local, U.S.]</mark> -- <col><b>Summer cypress</b></col> <fld>(Bot.)</fld>, <cd>an annual plant (<spn>Kochia Scoparia</spn>) of the Goosefoot family. It has narrow, ciliate, crowded leaves, and is sometimes seen in gardens.</cd> -- <col><b>Summer duck</b></col>. <fld>(Zool.)</fld> <sd>(a)</sd> <cd>The wood duck.</cd> <sd>(b)</sd> <cd>The garganey, or summer teal. See <xex>Illust.</xex> of <cref>Wood duck</cref>, under <er>Wood</er>.</cd> -- <col><b>Summer fallow</b></col>, <cd>land uncropped and plowed, etc., during the summer, in order to pulverize the soil and kill the weeds.</cd> -- <col><b>Summer rash</b></col> <fld>(Med.)</fld>, <cd>prickly heat. See under <er>Prickly</er>.</cd> -- <col><b>Summer sheldrake</b></col> <fld>(Zool.)</fld>, <cd>the hooded merganser.</cd> <mark>[Local, U.S.]</mark> -- <col><b>Summer snipe</b></col>. <fld>(Zool.)</fld> <sd>(a)</sd> <cd>The dunlin.</cd> <sd>(b)</sd> <cd>The common European sandpiper.</cd> <sd>(c)</sd> <cd>The green sandpiper.</cd> -- <col><b>Summer tanager</b></col> <fld>(Zool.)</fld>, <cd>a singing bird (<spn>Piranga rubra</spn>) native of the Middle and Southern United States. The male is deep red, the female is yellowish olive above and yellow beneath. Called also <altname>summer redbird</altname>.</cd> -- <col><b>Summer teal</b></col> <fld>(Zool.)</fld>, the blue-winged teal. <mark>[Local, U.S.]</mark> -- <col><b>Summer wheat</b></col>, <cd>wheat that is sown in the spring, and matures during the summer following. See <er>Spring wheat</er>.</cd> -- <col><b>Summer yellowbird</b></col>. <fld>(Zool.)</fld> <cd>See <er>Yellowbird</er>.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Summer</ent><br/
<hw>Sum"mer</hw>, <pos>v. i.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Summered</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Summering</conjf>.]</vmorph> <def>To pass the summer; to spend the warm season; <as>as, to <ex>summer</ex> in Switzerland</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The fowls shall <qex>summer</qex> upon them.</q> <rj><qau>Isa. xviii. 6.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Summer</ent><br/
<hw>Sum"mer</hw>, <pos>v. t.</pos> <def>To keep or carry through the summer; to feed during the summer; <as>as, to <ex>summer</ex> stock</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Summer-fallow</ent><br/
<hw>Sum"mer-fal"low</hw> <pr>(?)</pr>, <pos>v. t.</pos> <def>To plow and work in summer, in order to prepare for wheat or other crop; to plow and let lie fallow.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Summerhouse</ent><br/
<hw>Sum"mer*house`</hw> <pr>(?)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Summerhouses</plw> <pr>(<?/)</pr>.</plu> <def>A rustic house or apartment in a garden or park, to be used as a pleasure resort in summer.</def>  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Summerliness</ent><br/
<hw>Sum"mer*li*ness</hw> <pr>(?)</pr>, <pos>n.</pos> <def>The quality or state of being like summer.</def> <mark>[R.]</mark>  <rj><au>Fuller.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Summerset</ent><br/
<ent>Summersault</ent><br/
<mhw>{ <hw>Sum"mer*sault</hw> <pr>(?)</pr>, <hw>Sum"mer*set</hw> <pr>(?)</pr>, }</mhw> <pos>n.</pos> <def>See <er>Somersault</er>, <er>Somerset</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Summerstir</ent><br/
<hw>Sum"mer*stir`</hw> <pr>(?)</pr>, <pos>v. t.</pos> <def>To summer-fallow.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Summertide</ent><br/
<hw>Sum"mer*tide`</hw> <pr>(?)</pr>, <pos>n.</pos> <def>Summer time.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Summertree</ent><br/
<hw>Sum"mer*tree`</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[<ets>Summer</ets> a beam + <ets>tree</ets>.]</ety> <fld>(Arch.)</fld> <def>A summer. See 2d <er>Summer</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Summery</ent><br/
<hw>Sum"mer*y</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Of or pertaining to summer; like summer; <as>as, a <ex>summery</ex> day</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Summist</ent><br/
<hw>Sum"mist</hw> <pr>(?)</pr>, <pos>n.</pos> <def>One who sums up; one who forms an abridgment or summary.</def>  <rj><au>Sir E. Dering.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Summit</ent><br/
<hw>Sum"mit</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F. <ets>sommet</ets>, dim. of OF. <ets>som</ets>, <ets>sum</ets>, top, from L. <ets>summum</ets>, from <ets>summus</ets> highest. See <er>Sum</er>, <pos>n.</pos>]</ety> <sn>1.</sn> <def>The top; the highest point.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Fixed on the <qex>summit</qex> of the highest mount.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>The highest degree; the utmost elevation; the acme; <as>as, the <ex>summit</ex> of human fame</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <fld>(Zool.)</fld> <def>The most elevated part of a bivalve shell, or the part in which the hinge is situated.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Summit level</b></col>, <cd>the highest level of a canal, a railroad, or the like, in surmounting an ascent.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Summitless</ent><br/
<hw>Sum"mit*less</hw>, <pos>a.</pos> <def>Having no summit.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Summity</ent><br/
<hw>Sum"mit*y</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>summitas</ets>, fr. <ets>summus</ets> highest: cf. F. <ets>sommit<eacute/</ets>. See <er>Sum</er>, <pos>n.</pos>]</ety> <sn>1.</sn> <def>The height or top of anything.</def> <mark>[Obs.]</mark>  <rj><au>Swift.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>The utmost degree; perfection.</def> <mark>[Obs.]</mark>  <rj><au>Hallywell.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Summon</ent><br/
<hw>Sum"mon</hw> <pr>(?)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Summoned</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Summoning</conjf>.]</vmorph> <ety>[OE. <ets>somonen</ets>, OF. <ets>sumundre</ets>, <ets>semondre</ets>, F. <ets>semondre</ets>, from (assumed) LL. <ets>summon<ecr/re</ets>, for L. <ets>summon<emac/re</ets> to give a hint; <ets>sub</ets> under + <ets>monere</ets> to admonish, to warn. See <er>Monition</er>, and cf. <er>Submonish</er>.]</ety> <sn>1.</sn> <def>To call, bid, or cite; to notify to come to appear; -- often with <xex>up</xex>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Stiffen the sinews, <qex>summon</qex> up the blood.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Trumpets <qex>summon</qex> him to war.</q> <rj><qau>Dryden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To give notice to, or command to appear, as in court; to cite by authority; <as>as, to <ex>summon</ex> witnesses</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <fld>(Mil.)</fld> <def>To call upon to surrender, as a fort.</def><br/
[<source>1913 Webster</source>]</p>

<p><syn><b>Syn.</b> -- To call; cite; notify; convene; convoke; excite; invite; bid. See <er>Call</er>.</syn><br/
[<source>1913 Webster</source>]</p>

<p><ent>Summoner</ent><br/
<hw>Sum"mon*er</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[OE. <ets>somner</ets>, <ets>sompnour</ets>, OF. <ets>semoneor</ets>, F. <ets>semonneur</ets>. See <er>Summon</er>, <pos>v. t.</pos>]</ety> <def>One who summons; one who cites by authority; specifically, a petty officer formerly employed to summon persons to appear in court; an apparitor.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Summons</ent><br/
<hw>Sum"mons</hw> <pr>(?)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Summonses</plw> <pr>(#)</pr>.</plu> <ety>[OE. <ets>somouns</ets>, OF. <ets>sumunse</ets>, <ets>semonse</ets>, <ets>semonce</ets>, F. <ets>semonce</ets>, <ets>semondre</ets> to summon, OF. p. p. <ets>semons</ets>. See <er>Summon</er>, <pos>v.</pos>]</ety> <sn>1.</sn> <def>The act of summoning; a call by authority, or by the command of a superior, to appear at a place named, or to attend to some duty.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Special <qex>summonses</qex> by the king.</q> <rj><qau>Hallam.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>This <qex>summons</qex> . . . unfit either to dispute or disobey.</q> <rj><qau>Bp. Fell.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>He sent to summon the seditious, and to offer pardon; but neither <qex>summons</qex> nor pardon was regarded.</q> <rj><qau>Sir J. Hayward.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Law)</fld> <def>A warning or citation to appear in court; a written notification signed by the proper officer, to be served on a person, warning him to appear in court at a day specified, to answer to the plaintiff, testify as a witness, or the like.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <fld>(Mil.)</fld> <def>A demand to surrender.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Summons</ent><br/
<hw>Sum"mons</hw>, <pos>v. t.</pos> <def>To summon.</def> <mark>[R. or Colloq.]</mark>  <rj><au>Swift.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Summum bonum</ent><br/
||<hw>Sum"mum bo"num</hw> <pr>(?)</pr>. <ety>[L.]</ety> <fld>(Philos.)</fld> <def>The supreme or highest good, -- referring to the object of human life.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Sumner</ent><br/
<hw>Sum"ner</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A summoner.</def> <mark>[Obs.]</mark>  <rj><au>Beau. & Fl.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sumoom</ent><br/
<hw>Su*moom"</hw> <pr>(?)</pr>, <pos>n.</pos> <def>See <er>Simoom</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sump</ent><br/
<hw>Sump</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Cf. G. <ets>sumpf</ets> a sump in a mine, a swamp, akin to LG. <ets>sump</ets>, D. <ets>somp</ets> a swamp, Dan. & Sw. <ets>sump</ets>, and perhaps to E. <ets>swamp</ets>.]</ety> <sn>1.</sn> <fld>(Metal.)</fld> <def>A round pit of stone, lined with clay, for receiving the metal on its first fusion.</def>  <rj><au>Ray.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>The cistern or reservoir made at the lowest point of a mine, from which is pumped the water which accumulates there.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>A pond of water for salt works.</def>  <rj><au>Knight.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>A puddle or dirty pool.</def> <mark>[Prov. Eng.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Sump fuse</b></col>, <cd>a fuse used in blasting under water.</cd> -- <col><b>Sump men</b></col> <fld>(Mining)</fld>, <cd>the men who sink the sump in a mine.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sumph</ent><br/
<hw>Sumph</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A dunce; a blockhead.</def> <mark>[Scot.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sumpitan</ent><br/
<hw>Sum"pi*tan</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A kind of blowgun for discharging arrows, -- used by the savages of Borneo and adjacent islands.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sumpter</ent><br/
<hw>Sump"ter</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[OF. <ets>sommetier</ets> the driver of a pack horse; akin to OF. & F. <ets>sommier</ets> a pack horse, L. <ets>sagmarius</ets>, fr. <ets>sagma</ets> a pack saddle, in LL., a load, Gr. <?/ a pack saddle, fr. <?/ to pack, load; cf. Skr. <ets>saj</ets>, <ets>sa<ntil/j</ets>, to hang on.  Cf. <er>Seam</er> a weight, <er>Summer</er> a beam.]</ety> <sn>1.</sn> <def>The driver of a pack horse.</def> <mark>[Obs.]</mark>  <rj><au>Skeat.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A pack; a burden.</def> <mark>[Obs.]</mark>  <rj><au>Beau. & Fl.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>An animal, especially a horse, that carries packs or burdens; a baggage horse.</def>  <rj><au>Holinshed.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sumpter</ent><br/
<hw>Sump"ter</hw>, <pos>a.</pos> <def>Carrying pack or burdens on the back; <as>as, a <ex>sumpter</ex> horse; a <ex>sumpter</ex> mule.</as></def>  <rj><au>Bacon.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sumption</ent><br/
<hw>Sump"tion</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>sumptio</ets>, fr. <ets>sumere</ets>, <ets>sumptum</ets>, to take.]</ety> <sn>1.</sn> <def>A taking.</def> <mark>[Obs.]</mark>  <rj><au>Jer. Taylor.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Logic)</fld> <def>The major premise of a syllogism.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sumptuary</ent><br/
<hw>Sump"tu*a*ry</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>sumptuarius</ets>, fr. <ets>sumptus</ets> expense, cost, fr. <ets>sumere</ets>, <ets>sumptum</ets>, to take, use, spend; <ets>sub</ets> under + <ets>emere</ets> to take, buy: cf. F. <ets>somptuaire</ets>. See <er>Redeem</er>.]</ety> <def>Relating to expense; regulating expense or expenditure.</def>  <rj><au>Bacon.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><mcol><col><b>Sumptuary laws</b></col> <it>or</it> <col><b>Sumptuary regulations</b></col></mcol>, <cd>laws intended to restrain or limit the expenditure of citizens in apparel, food, furniture, etc.; laws which regulate the prices of commodities and the wages of labor; laws which forbid or restrict the use of certain articles, as of luxurious apparel.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sumptuosity</ent><br/
<hw>Sump`tu*os"i*ty</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>sumptuositas</ets>: cf. F. <ets>somptuosit<eacute/</ets>.]</ety> <def>Expensiveness; costliness; sumptuousness.</def> <mark>[R.]</mark>  <rj><au>Sir W. Raleigh.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><-- p. 1445 --></p>

<p><ent>Sumptuous</ent><br/
<hw>Sump"tu*ous</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>sumptuosus</ets>, fr. <ets>sumptus</ets> expanse, cost: cf. F. <ets>somptueux</ets>. See <er>Sumptuary</er>.]</ety> <def>Involving large outlay or expense; costly; expensive; hence, luxurious; splendid; magnificient; <as>as, a <ex>sumptuous</ex> house or table; <ex>sumptuous</ex> apparel</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>We are too magnificient and <qex>sumptuous</qex> in our tables and attendance.</q> <rj><qau>Atterbury.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>She spoke, and turned her <qex>sumptuous</qex> head, with eyes<br/
Of shining expectation fixed on mine.</q> <rj><qau>Tennyson.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p>-- <wordforms><wf>Sump"tu*ous*ly</wf>, <pos>adv.</pos> -- <wf>Sump"tu*ous*ness</wf>, <pos>n.</pos></wordforms><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sun</ent><br/
<hw>Sun</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Bot.)</fld> <def>See <er>Sunn</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sun</ent><br/
<hw>Sun</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[OE. <ets>sunne</ets>, <ets>sonne</ets>, AS. <ets>sunne</ets>; akin to OFries. <ets>sunne</ets>, D. <ets>zon</ets>, OS. & OHG. <ets>sunna</ets>, G. <ets>sonne</ets>, Icel. <ets>sunna</ets>, Goth. <ets>sunna</ets>; perh. fr. same root as L. <ets>sol</ets>. <root/297.  Cf. <er>Solar</er>, <er>South</er>.]</ety> <sn>1.</sn> <def>The luminous orb, the light of which constitutes day, and its absence night; the central body round which the earth and planets revolve, by which they are held in their orbits, and from which they receive light and heat. Its mean distance from the earth is about 92,500,000 miles, and its diameter about 860,000.</def><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ Its mean apparent diameter as seen from the earth is 32<prime/ 4<sec/, and it revolves on its own axis once in 25<frac13/ days. Its mean density is about one fourth of that of the earth, or 1.41, that of water being unity.  Its luminous surface is called the <xex>photosphere</xex>, above which is an envelope consisting partly of hydrogen, called the <xex>chromosphere</xex>, which can be seen only through the spectroscope, or at the time of a total solar eclipse. Above the chromosphere, and sometimes extending out millions of miles, are luminous rays or streams of light which are visible only at the time of a total eclipse, forming the solar <xex>corona</xex>.</note><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Any heavenly body which forms the center of a system of orbs.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>The direct light or warmth of the sun; sunshine.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Lambs that did frisk in the <qex>sun</qex>.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>That which resembles the sun, as in splendor or importance; any source of light, warmth, or animation.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>For the Lord God is a <qex>sun</qex> and shield.</q> <rj><qau>Ps. lxxiv. 11.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>I will never consent to put out the <qex>sun</qex> of sovereignity to posterity.</q> <rj><qau>Eikon Basilike.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Sun and planet wheels</b></col> <fld>(Mach.)</fld>, <cd>an ingenious contrivance for converting reciprocating motion, as that of the working beam of a steam engine, into rotatory motion. It consists of a toothed wheel (called the <xex>sun wheel</xex>), firmly secured to the shaft it is desired to drive, and another wheel (called the <xex>planet wheel</xex>) secured to the end of a connecting rod. By the motion of the connecting rod, the planet wheel is made to circulate round the central wheel on the shaft, communicating to this latter a velocity of revolution the double of its own.</cd> <au>G. Francis.</au> -- <col><b>Sun angel</b></col> <fld>(Zool.)</fld>, <cd>a South American humming bird of the genus <gen>Heliangelos</gen>, noted for its beautiful colors and the brilliant luster of the feathers of its throat.</cd> -- <col><b>Sun animalcute</b></col>. <fld>(Zool.)</fld> <cd>See <er>Heliozoa</er>.</cd> -- <col><b>Sun bath</b></col> <fld>(Med.)</fld>, <cd>exposure of a patient to the sun's rays; insolation.</cd> -- <col><b>Sun bear</b></col> <fld>(Zool.)</fld>, <cd>a species of bear (<spn>Helarctos Malayanus</spn>) native of Southern Asia and Borneo. It has a small head and short neck, and fine short glossy fur, mostly black, but brownish on the nose. It is easily tamed. Called also <altname>bruang</altname>, and <altname>Malayan bear</altname>.</cd> -- <col><b>Sun beetle</b></col> <fld>(Zool.)</fld>, <cd>any small lustrous beetle of the genus <gen>Amara</gen>.</cd> -- <col><b>Sun bittern</b></col> <fld>(Zool.)</fld>, <cd>a singular South American bird (<spn>Eurypyga helias</spn>), in some respects related both to the rails and herons. It is beautifully variegated with white, brown, and black. Called also <altname>sunbird</altname>, and <altname>tiger bittern</altname>.</cd> -- <col><b>Sun fever</b></col> <fld>(Med.)</fld>, <cd>the condition of fever produced by sun stroke.</cd> -- <col><b>Sun gem</b></col> <fld>(Zool.)</fld>, <cd>a Brazilian humming bird (<spn>Heliactin cornutus</spn>). Its head is ornamented by two tufts of bright colored feathers, fiery crimson at the base and greenish yellow at the tip. Called also <altname>Horned hummer</altname>.</cd> -- <col><b>Sun grebe</b></col> <fld>(Zool.)</fld>, <cd>the finfoot.</cd> -- <col><b>Sun picture</b></col>, <cd>a picture taken by the agency of the sun's rays; a photograph.</cd> -- <col><b>Sun spots</b></col> <fld>(Astron.)</fld>, <cd>dark spots that appear on the sun's disk, consisting commonly of a black central portion with a surrounding border of lighter shade, and usually seen only by the telescope, but sometimes by the naked eye. They are very changeable in their figure and dimensions, and vary in size from mere apparent points to spaces of 50,000 miles in diameter. The term <xex>sun spots</xex> is often used to include bright spaces (called <xex>faculae</xex>) as well as dark spaces (called <xex>maculae</xex>). Called also <altname>solar spots</altname>. See <xex>Illustration</xex> in Appendix.</cd> -- <col><b>Sun star</b></col> <fld>(Zool.)</fld>, <cd>any one of several species of starfishes belonging to <gen>Solaster</gen>, <gen>Crossaster</gen>, and allied genera, having numerous rays.</cd> -- <col><b>Sun trout</b></col> <fld>(Zool.)</fld>, <cd>the squeteague.</cd> -- <col><b>Sun wheel</b></col>. <fld>(Mach.)</fld> <cd>See <cref>Sun and planet wheels</cref>, above.</cd> -- <col><b>Under the sun</b></col>, <cd>in the world; on earth.</cd> <ldquo/There is no new thing <xex>under the sun</xex>.<rdquo/ <au>Eccl. i. 9.</au></cs><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ <xex>Sun</xex> is often used in the formation of compound adjectives of obvious meaning; as, <xex>sun</xex>-bright, <xex>sun</xex>-dried, <xex>sun</xex>-gilt, <xex>sun</xex>like, <xex>sun</xex>-lit, <xex>sun</xex>-scorched, and the like.</note><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sun</ent><br/
<hw>Sun</hw>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Sunned</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Sunning</conjf>.]</vmorph> <def>To expose to the sun's rays; to warm or dry in the sun; <as>as, to <ex>sun</ex> cloth; to <ex>sun</ex> grain.</as></def><br/
[<source>1913 Webster</source>]</p>

<p><q>Then to <qex>sun</qex> thyself in open air.</q> <rj><qau>Dryden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sunbeam</ent><br/
<hw>Sun"beam`</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[AS. <ets>sunnebeam</ets>.]</ety> <def>A beam or ray of the sun.</def> <ldquo/Evening <xex>sunbeams</xex>.<rdquo/  <rj><au>Keble.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Thither came Uriel, gliding through the even<br/
On a <qex>sunbeam</qex>.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sunbird</ent><br/
<hw>Sun"bird`</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Zool.)</fld> <sd>(a)</sd> <def>Any one of numerous species of small brilliantly colored birds of the family <fam>Nectariniidae</fam>, native of Africa, Southern Asia, the East Indies, and Australia. In external appearance and habits they somewhat resemble humming birds, but they are true singing birds (Oscines).</def> <sd>(b)</sd> <def>The sun bittern.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sunblink</ent><br/
<hw>Sun"blink`</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A glimpse or flash of the sun.</def> <mark>[Scot.]</mark>  <rj><au>Sir W. Scott.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sunbonnet</ent><br/
<hw>Sun"bon"net</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A bonnet, generally made of some thin or light fabric, projecting beyond the face, and commonly having a cape, -- worn by women as a protection against the sun.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sunbow</ent><br/
<hw>Sun"bow`</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A rainbow; an iris.</def>  <rj><au>Byron.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sunburn</ent><br/
<hw>Sun"burn`</hw> <pr>(?)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Sunburned</conjf> <pr>(?)</pr> or <conjf>Sunburnt</conjf> <pr>(<?/)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Sunburning</conjf>.]</vmorph> <def>To burn or discolor by the sun; to tan.</def><br/
[<source>1913 Webster</source>]</p>

<p><q><qex>Sunburnt</qex> and swarthy though she be.</q> <rj><qau>Dryden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sunburn</ent><br/
<hw>Sun"burn`</hw>, <pos>n.</pos> <def>The burning or discoloration produced on the skin by the heat of the sun; tan.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sun-burner</ent><br/
<hw>Sun"-burn`er</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A circle or cluster of gas-burners for lighting and ventilating public buildings.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sunburning</ent><br/
<hw>Sun"burn`ing</hw>, <pos>n.</pos> <def>Sunburn; tan.</def>  <rj><au>Boyle.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sunburst</ent><br/
<hw>Sun"burst`</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A burst of sunlight.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Suncup</ent><br/
<hw>Sun"cup`</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A yellow flowered evening primrose (<spn>Taraxia ovata</spn>, syn. <spn>Oenothera ovata</spn>) native of California.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Sundart</ent><br/
<hw>Sun"dart`</hw> <pr>(?)</pr>, <pos>n.</pos> <def>Sunbeam.</def> <mark>[R.]</mark>  <rj><au>Mrs. Hemans.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sunday</ent><br/
<hw>Sun"day</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[AS. <ets>sunnandaeg</ets>; <ets>sunne</ets>, gen. <ets>sunnan</ets>, the sun + <ets>daeg</ets> day; akin to D. <ets>zondag</ets>, G. <ets>sonntag</ets>; -- so called because this day was anciently dedicated to the <ets>sun</ets>, or to its worship. See <er>Sun</er>, and <er>Day</er>.]</ety> <def>The first day of the week, -- consecrated among Christians to rest from secular employments, and to religious worship; the Christian Sabbath; the Lord's Day.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><mcol><col><b>Advent Sunday</b></col>, <col><b>Low Sunday</b></col>, <col><b>Passion Sunday</b></col></mcol>, <cd>etc. See under <er>Advent</er>, <er>Low</er>, etc.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><syn><b>Syn.</b> -- See <er>Sabbath</er>.</syn><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sunday</ent><br/
<hw>Sun"day</hw>, <pos>a.</pos> <def>Belonging to the Christian Sabbath.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Sunday letter</b></col>. <cd>See <cref>Dominical letter</cref>, under <er>Dominical</er>.</cd> -- <col><b>Sunday school</b></col>. <cd>See under <er>School</er>.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sunder</ent><br/
<hw>Sun"der</hw> <pr>(?)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Sundered</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Sundering</conjf>.]</vmorph> <ety>[OE. <ets>sundren</ets>, AS. <ets>sundrain</ets> (in <amac/<ets>sundrain</ets>, ge<ets>sundrain</ets>), from <ets>sundor</ets> asunder, separately, apart; akin to D. <ets>zonder</ets>, prep., without, G. <ets>sonder</ets> separate, as prep., without, <ets>sondern</ets> but, OHG. <ets>suntar</ets> separately, Icel. <ets>sundr</ets> asunder, Sw. & Dan. <ets>s<oum/nder</ets>, Goth. <ets>sundr<omac/</ets> alone, separately.]</ety> <def>To disunite in almost any manner, either by rending, cutting, or breaking; to part; to put or keep apart; to separate; to divide; to sever; <as>as, to <ex>sunder</ex> a rope; to <ex>sunder</ex> a limb; to <ex>sunder</ex> friends.</as></def><br/
[<source>1913 Webster</source>]</p>

<p><q>It is <qex>sundered</qex> from the main land by a sandy plain.</q> <rj><qau>Carew.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sunder</ent><br/
<hw>Sun"der</hw>, <pos>v. i.</pos> <def>To part; to separate.</def> <mark>[R.]</mark>  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sunder</ent><br/
<hw>Sun"der</hw>, <pos>n.</pos> <ety>[See <er>Sunder</er>, <pos>v. t.</pos>, and cf. <er>Asunder</er>.]</ety> <def>A separation into parts; a division or severance.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>In sunder</b></col>, <cd>into parts.</cd> <ldquo/He breaketh the bow, and cutteth the spear <xex>in sunder</xex>.<rdquo/ <au>Ps. xlvi. 9.</au></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sunder</ent><br/
<hw>Sun"der</hw>, <pos>v. t.</pos> <def>To expose to the sun and wind.</def> <mark>[Prov. Eng.]</mark>  <rj><au>Halliwell.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sundew</ent><br/
<hw>Sun"dew`</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Bot.)</fld> <def>Any plant of the genus Drosera, low bog plants whose leaves are beset with pediceled glands which secrete a viscid fluid that glitters like dewdrops and attracts and detains insects. After an insect is caught, the glands curve inward like tentacles and the leaf digests it. Called also <altname>lustwort</altname>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sundial</ent><br/
<hw>Sun"di`al</hw> <pr>(?)</pr>, <pos>n.</pos> <def>An instrument to show the time of day by means of the shadow of a gnomon, or style, on a plate.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Sundial shell</b></col> <fld>(Zool.)</fld>, <cd>any shell of the genus Solarium. See <er>Solarium</er>.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sundog</ent><br/
<hw>Sun"dog`</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Meteorol.)</fld> <sn>1.</sn> <def>A luminous spot occasionally seen a few degrees from the sun, supposed to be formed by the intersection of two or more halos, or in a manner similar to that of halos.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A fragmentary rainbow; a small rainbow near the horizon; -- called also <altname>dog</altname> and <altname>weathergaw</altname>.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Sundown</ent><br/
<hw>Sun"down`</hw> <pr>(?)</pr>, <pos>n.</pos> <sn>1.</sn> <def>The setting of the sun; sunset.</def> <ldquo/When <xex>sundown</xex> skirts the moor.<rdquo/  <rj><au>Tennyson.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A kind of broad-brimmed sun hat worn by women.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sundowner</ent><br/
<hw>Sun"down`er</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A tramp or vagabond in the Australian bush; -- so called from his coming to sheep stations at sunset of ask for supper and a bed, when it is too late to work; -- called also <altname>traveler</altname> and <altname>swagman</altname> (but not all <xex>swagmen</xex> are <ex>sundowners</ex>).</def></p>

<p><q><qex>Sundowners</qex>, -- men who loaf about till sunset, and then come in with the demand for unrefusable rations.</q>  <rj><qau>Francis Adams.</qau></rj><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Sun-dried</ent><br/
<hw>Sun"-dried`</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Dried by the heat of the sun; <as>as, <ex>sun-dried</ex> tomatoes</as>.</def> <ldquo/<xex>Sun-dried</xex> brick.<rdquo/  <rj><au>Sir T. Herbert.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sundries</ent><br/
<hw>Sun"dries</hw> <pr>(?)</pr>, <pos>n. pl.</pos> <def>Many different or small things; sundry things.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sundrily</ent><br/
<hw>Sun"dri*ly</hw> <pr>(?)</pr>, <pos>adv.</pos> <def>In sundry ways; variously.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sundrops</ent><br/
<hw>Sun"drops`</hw> <pr>(?)</pr>, <pos>n.</pos>  <ety>[<ets>Sun</ets> + <ets>drop</ets>.]</ety> <fld>(Bot.)</fld> <def>Any one of the several species of <gen>Kneiffia</gen>, esp. <spn>Kneiffia fruticosa</spn> (syn. <spn>Oenothera fruticosa</spn>), of the Evening-primrose family, having flowers that open by daylight.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Sundry</ent><br/
<hw>Sun"dry</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[OE. <ets>sundry</ets>, <ets>sondry</ets>, AS. <ets>syndrig</ets>, fr. <ets>sundor</ets> asunder. See <er>Sunder</er>, <pos>v. t.</pos>]</ety> <sn>1.</sn> <def>Several; divers; more than one or two; various.</def> <ldquo/<xex>Sundry</xex> wines.<rdquo/ <au>Chaucer.</au> <ldquo/<xex>Sundry</xex> weighty reasons.<rdquo/ <au>Shak.</au><br/
[<source>1913 Webster</source>]</p>

<p><q>With many a sound of <qex>sundry</qex> melody.</q> <rj><qau>Chaucer.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q><qex>Sundry</qex> foes the rural realm surround.</q> <rj><qau>Dryden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Separate; diverse.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>Every church almost had the Bible of a <qex>sundry</qex> translation.</q> <rj><qau>Coleridge.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>All and sundry</b></col>, <cd>all collectively, and each separately.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sundryman</ent><br/
<hw>Sun"dry*man</hw> <pr>(?)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Sundrymen</plw> <pr>(<?/)</pr>.</plu> <def>One who deals in sundries, or a variety of articles.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sunfish</ent><br/
<hw>Sun"fish`</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Zool.)</fld> <sd>(a)</sd> <def>A very large oceanic plectognath fish (<spn>Mola mola</spn>, <spn>Mola rotunda</spn>, or <spn>Orthagoriscus mola</spn>) having a broad body and a truncated tail.</def> <sd>(b)</sd> <def>Any one of numerous species of perch-like North American fresh-water fishes of the family <fam>Centrachidae</fam>. They have a broad, compressed body, and strong dorsal spines. Among the common species of the Eastern United States are <spn>Lepomis gibbosus</spn> (called also <altname>bream</altname>, <altname>pondfish</altname>, <altname>pumpkin seed</altname>, and <altname>sunny</altname>), the blue sunfish, or dollardee (<spn>Lepomis pallidus</spn>), and the long-eared sunfish (<spn>Lepomis auritus</spn>). Several of the species are called also <stype>pondfish</stype>.</def> <sd>(c)</sd> <def>The moonfish, or bluntnosed shiner.</def> <sd>(d)</sd> <def>The opah.</def> <sd>(e)</sd> <def>The basking, or liver, shark.</def> <sd>(f)</sd> <def>Any large jellyfish.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sunflower</ent><br/
<hw>Sun"flow`er</hw> <pr>(?)</pr>, <pos>n.</pos> <def>Any plant of the genus <gen>Helianthus</gen>; -- so called probably from the form and color of its flower, which is large disk with yellow rays. The commonly cultivated sunflower is <spn>Helianthus annuus</spn>, a native of America.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sunflower State</ent><br/
<hw>Sun"flow`er State</hw>. <def>Kansas; a nickname.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Sung</ent><br/
<hw>Sung</hw> <pr>(?)</pr>, <def><pos>imp. & p. p.</pos> of <er>Sing</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sunglass</ent><br/
<hw>Sun"glass`</hw> <pr>(?)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Sunglasses</plw> <pr>(<?/)</pr>.</plu> <def>A convex lens of glass for producing heat by converging the sun's rays into a focus.</def> <ldquo/Lighting a cigar with a <xex>sunglass</xex>.<rdquo/  <rj><au>Hawthorne.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sunglow</ent><br/
<hw>Sun"glow`</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A rosy flush in the sky seen after sunset.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sunk</ent><br/
<hw>Sunk</hw> <pr>(?)</pr>, <def><pos>imp. & p. p.</pos> of <er>Sink</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Sunk fence</b></col>, <cd>a ditch with a retaining wall, used to divide lands without defacing a landscape; a ha-ha.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sunken</ent><br/
<hw>Sunk"en</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Lying on the bottom of a river or other water; sunk.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sunless</ent><br/
<hw>Sun"less</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Destitute or deprived of the sun or its rays; shaded; shadowed.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The sunken glen whose <qex>sunless</qex> shrubs must weep.</q> <rj><qau>Byron.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sunlight</ent><br/
<hw>Sun"light`</hw> <pr>(?)</pr>, <pos>n.</pos> <def>The light of the sun.</def>  <rj><au>Milton.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sunlike</ent><br/
<hw>Sun"like`</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Like or resembling the sun.</def> <ldquo/A spot of <xex>sunlike</xex> brilliancy.<rdquo/  <rj><au>Tyndall.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sunlit</ent><br/
<hw>Sun"lit`</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Lighted by the sun.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sunn</ent><br/
<hw>Sunn</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Hind. <ets>san</ets>, fr. Skr. <ets><cced/ana</ets>.]</ety> <fld>(Bot.)</fld> <def>An East Indian leguminous plant (<spn>Crotalaria juncea</spn>) and its fiber, which is also called <altname>sunn hemp</altname>.</def> <altsp>[Written also <asp>sun</asp>.]</altsp><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sunna</ent><br/
||<hw>Sun"na</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Ar. <ets>sunnah</ets> rule, law.]</ety> <def>A collection of traditions received by the orthodox Mohammedans as of equal authority with the Koran.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sunniah</ent><br/
<hw>Sun"ni*ah</hw> <pr>(?)</pr>, <pos>n.</pos> <def>One of the sect of Sunnites.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sunniness</ent><br/
<hw>Sun"ni*ness</hw> <pr>(?)</pr>, <pos>n.</pos> <def>The quality or state of being sunny.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sunnite</ent><br/
<hw>Sun"nite</hw> <pr>(?)</pr>, <pos>n.</pos> <def>One of the orthodox Mohammedans who receive the Sunna as of equal importance with the Koran.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sunnud</ent><br/
<hw>Sun"nud</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Hind., fr. Ar. <ets>sanad</ets>.]</ety> <def>A charter or warrant; also, a deed of gift.</def> <mark>[India]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sunny</ent><br/
<hw>Sun"ny</hw> <pr>(?)</pr>, <pos>a.</pos> <amorph>[<pos>Compar.</pos> <adjf>Sunnier</adjf> <pr>(?)</pr>; <pos>superl.</pos> <adjf>Sunniest</adjf>.]</amorph> <sn>1.</sn> <def>Of or pertaining to the sun; proceeding from, or resembling the sun; hence, shining; bright; brilliant; radiant.</def> <ldquo/<xex>Sunny</xex> beams.<rdquo/ <au>Spenser.</au> <ldquo/<xex>Sunny</xex> locks.<rdquo/ <au>Shak.</au><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Exposed to the rays of the sun; brightened or warmed by the direct rays of the sun; <as>as, a <ex>sunny</ex> room; the <ex>sunny</ex> side of a hill</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Her blooming mountains and her <qex>sunny</qex> shores.</q> <rj><qau>Addison.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>Cheerful; genial; <as>as, a <ex>sunny</ex> disposition</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>My decayed fair<br/
A <qex>sunny</qex> look of his would soon repair.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sunny</ent><br/
<hw>Sun"ny</hw>, <pos>n.</pos> <fld>(Zool.)</fld> <def>See <er>Sunfish</er> <sd>(b)</sd>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sunproof</ent><br/
<hw>Sun"proof`</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Impervious to the rays of the sun.</def> <ldquo/Darksome yew, <xex>sunproof</xex>.<rdquo/  <rj><au>Marston.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sunrising</ent><br/
<ent>Sunrise</ent><br/
<mhw>{ <hw>Sun"rise`</hw> <pr>(?)</pr>, <hw>Sun"ris`ing</hw>, }</mhw> <pos>n.</pos> <sn>1.</sn> <def>The first appearance of the sun above the horizon in the morning; more generally, the time of such appearance, whether in fair or cloudy weather; <as>as, to begin work at <ex>sunrise</ex></as>.</def> <ldquo/The tide of <xex>sunrise</xex> swells.<rdquo/  <rj><au>Keble.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Hence, the region where the sun rises; the east.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Which were beyond Jordan toward the <qex>sunrising</qex>.</q> <rj><qau>Deut. iv. 47 (Rev. Ver.)</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Full hot and fast the Saxon rides, with rein of travel slack,<br/
And, bending o'ev his saddle, leaves the <qex>sunrise</qex> at his back.</q> <rj><qau>Whittier.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sunroof</ent><br/
<hw>Sun"roof`</hw> <pr>(?)</pr>, <pos>n.</pos> <def>a panel in the roof of an automobile, usually above the driver's seat, that can be opened.</def><br/
[<source>PJC</source>]</p>

<p><ent>Sunsetting</ent><br/
<ent>Sunset</ent><br/
<mhw>{ <hw>Sun"set"</hw> <pr>(?)</pr>, <hw>Sun"set`ting</hw>, }</mhw> <pos>n.</pos> <sn>1.</sn> <def>The descent of the sun below the horizon; also, the time when the sun sets; evening. Also used figuratively.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>'T is the <qex>sunset</qex> of life gives me mystical lore.</q> <rj><qau>Campbell.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Hence, the region where the sun sets; the west.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Sunset shell</b></col> <fld>(Zool.)</fld>, <cd>a West Indian marine bivalve (<spn>Tellina radiata</spn>) having a smooth shell marked with radiating bands of varied colors resembling those seen at sunset or before sunrise; -- called also <altname>rising sun</altname>.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sunshade</ent><br/
<hw>Sun"shade`</hw> <pr>(?)</pr>, <pos>n.</pos> <def>Anything used as a protection from the sun's rays.</def> Specifically: <sd>(a)</sd> <def>A small parasol.</def> <sd>(b)</sd> <def>An awning.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sunshine</ent><br/
<hw>Sun"shine`</hw> <pr>(?)</pr>, <pos>n.</pos> <sn>1.</sn> <def>The light of the sun, or the place where it shines; the direct rays of the sun, the place where they fall, or the warmth and light which they give.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>But all <qex>sunshine</qex>, as when his beams at noon<br/
Culminate from the equator.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Anything which has a warming and cheering influence like that of the rays of the sun; warmth; illumination; brightness.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>That man that sits within a monarch's heart,<br/
And ripens in the <qex>sunshine</qex> of his favor.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><-- p. 1446 --></p>

<p><ent>Sunshine</ent><br/
<hw>Sun"shine`</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Sunshiny; bright.</def> <au>Shak.</au> <ldquo/<xex>Sunshine</xex> hours.<rdquo/ <au>Keble.</au><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sunshiny</ent><br/
<hw>Sun"shin`y</hw> <pr>(?)</pr>, <pos>a.</pos> <sn>1.</sn> <def>Bright with the rays of the sun; clear, warm, or pleasant; <as>as, a <ex>sunshiny</ex> day</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Bright like the sun; resplendent.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Flashing beams of that <qex>sunshiny</qex> shield.</q> <rj><qau>Spenser.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>Beaming with good spirits; cheerful.</def> <ldquo/Her <xex>sunshiny</xex> face.<rdquo/  <rj><au>Spenser.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sunsquall</ent><br/
<hw>Sun"squall`</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Zool.)</fld> <def>Any large jellyfish.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sun star</ent><br/
<hw>Sun" star`</hw> <pr>(?)</pr>. <fld>(Zool.)</fld> <def>See <cref>Sun star</cref>, under <er>Sun</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sunsted</ent><br/
<hw>Sun"sted</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[<ets>Sun</ets> + <ets>stead</ets> a place.]</ety> <def>Solstice.</def> <mark>[Obs.]</mark> <ldquo/The summer <xex>sunsted</xex>.<rdquo/  <rj><au>Holland.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sunstone</ent><br/
<hw>Sun"stone`</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Med.)</fld> <def>Aventurine feldspar. See under <er>Aventurine</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sunstroke</ent><br/
<hw>Sun"stroke`</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Med.)</fld> <def>Any affection produced by the action of the sun on some part of the body; especially, a sudden prostration of the physical powers, with symptoms resembling those of apoplexy, occasioned by exposure to excessive heat, and often terminating fatally; coup de soleil.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sun-struck</ent><br/
<hw>Sun"-struck`</hw> <pr>(?)</pr>, <pos>a.</pos> <fld>(Med.)</fld> <def>Overcome by, or affected with, sunstroke; <as>as, <ex>sun-struck</ex> soldiers</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sunup</ent><br/
<hw>Sun"up`</hw> <pr>(?)</pr>, <pos>n.</pos> <def>Sunrise.</def> <mark>[Local, U.S.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>Such a horse as that might get over a good deal of ground atwixt <qex>sunup</qex> and sundown.</q> <rj><qau>Cooper.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sunward</ent><br/
<hw>Sun"ward</hw> <pr>(?)</pr>, <pos>adv.</pos> <def>Toward the sun.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sunwise</ent><br/
<hw>Sun"wise`</hw> <pr>(?)</pr>, <pos>adv.</pos> <def>In the direction of the sun's apparent motion, or from the east southward and westward, and so around the circle; also, in the same direction as the movement of the hands of a watch lying face upward.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sup</ent><br/
<hw>Sup</hw> <pr>(s<ucr/p)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Supped</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Supping</conjf>.]</vmorph> <ety>[OE. <ets>soupen</ets> to drink, AS. <ets>s<umac/pan</ets>; akin to D. <ets>zuipen</ets>, G. <ets>saufen</ets>, OHG. <ets>s<umac/fan</ets>, Icel. <ets>s<umac/pa</ets>, Sw. <ets>supa</ets>, Dan. <ets>s<oum/be</ets>.  Cf. <er>Sip</er>, <er>Sop</er>, <er>Soup</er>, <er>Supper</er>.]</ety> <def>To take into the mouth with the lips, as a liquid; to take or drink by a little at a time; to sip.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>There I'll <qex>sup</qex><br/
Balm and nectar in my cup.</q> <rj><qau>Crashaw.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sup</ent><br/
<hw>Sup</hw>, <pos>n.</pos> <def>A small mouthful, as of liquor or broth; a little taken with the lips; a sip.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Tom Thumb had got a little <qex>sup</qex>.</q> <rj><qau>Drayton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sup</ent><br/
<hw>Sup</hw>, <pos>v. i.</pos> <ety>[See <er>Supper</er>.]</ety> <def>To eat the evening meal; to take supper.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>I do entreat that we may <qex>sup</qex> together.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sup</ent><br/
<hw>Sup</hw>, <pos>v. t.</pos> <def>To treat with supper.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q><qex>Sup</qex> them well and look unto them all.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Supawn</ent><br/
<hw>Su*pawn"</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Of American Indian origin.]</ety> <def>Boiled Indian meal; hasty pudding; mush.</def> <altsp>[Written also <asp>sepawn</asp>, <asp>sepon</asp>, and <asp>suppawn</asp>.]</altsp> <mark>[Local, U.S.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Supe</ent><br/
<hw>Supe</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A super.</def> <mark>[Theatrical Cant]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Super-</ent><br/
<hw>Su"per-</hw> <pr>(?)</pr>. <ety>[L. <ets>super</ets> over, above; akin to Gr. <?/, L. <ets>sub</ets> under, and E. <ets>over</ets>. See <er>Over</er>, and cf. <er>Hyper-</er>, <er>Sub-</er>, <er>Supra-</er>, <er>Sur-</er>.]</ety> <sn>1.</sn> <def>A prefix signifying <xex>above</xex>, <xex>over</xex>, <xex>beyond</xex>, and hence often denoting <xex>in a superior position</xex>, <xex>in excess</xex>, <xex>over and above</xex>, <xex>in addition</xex>, <xex>exceedingly</xex>; as in <xex>super</xex>impose, <xex>super</xex>sede, <xex>super</xex>natural, <xex>super</xex>abundance.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Chem.)</fld> <def>A prefix formerly much used to denote that the ingredient to the name of which it was prefixed was present in a <xex>large</xex>, or <xex>unusually large</xex>, proportion as compared with the other ingredients; as in calcium <xex>super</xex>phosphate. It has been superseded by <xex>per-</xex>, <xex>bi-</xex>, <xex>di-</xex>, <xex>acid</xex>, etc. (as <xex>per</xex>oxide, <xex>bi</xex>carbonate, <xex>di</xex>sulphide, and <xex>acid</xex> sulphate), which retain the old meanings of <xex>super-</xex>, but with sharper definition.  Cf. <er>Acid</er>, <pos>a.</pos>, <er>Bi-</er>, <er>Di-</er>, and <er>Per-</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Super</ent><br/
<hw>Su"per</hw>, <pos>n.</pos> <def>A contraction of <er>Supernumerary</er>, in sense  2.</def> <mark>[Theatrical Cant]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Superable</ent><br/
<hw>Su"per*a*ble</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>superabilis</ets>, from <ets>superare</ets> to go over, to surmount, fr. <ets>super</ets> above, over.]</ety> <def>Capable of being overcome or conquered; surmountable.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Antipathies are generally <qex>superable</qex> by a single effort.</q> <rj><qau>Johnson.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p>-- <wordforms><wf>Su"per*a*ble*ness</wf>, <pos>n.</pos> -- <wf>Su"per*a*bly</wf>, <pos>adv.</pos></wordforms><br/
[<source>1913 Webster</source>]</p>

<p><ent>Superabound</ent><br/
<hw>Su`per*a*bound"</hw> <pr>(?)</pr>, <pos>v. i.</pos> <ety>[L. <ets>superabundare</ets>: cf. OF. <ets>superabonder</ets>. See <er>Super-</er>, and <er>Abound</er>.]</ety> <def>To be very abundant or exuberant; to be more than sufficient; <as>as, the country <ex>superabounds</ex> with corn</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Superabundance</ent><br/
<hw>Su`per*a*bun"dance</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>superabundantia</ets>: cf. OF. <ets>superabondance</ets>.]</ety> <def>The quality or state of being superabundant; a superabundant quantity; redundancy; excess.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Superabundant</ent><br/
<hw>Su`per*a*bun"dant</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>superabundans</ets>, p. pr. of <ets>superabundare</ets>. See <er>Superabound</er>.]</ety> <def>Abounding to excess; being more than is sufficient; redundant; <as>as, <ex>superabundant</ex> zeal</as>.</def> -- <wordforms><wf>Su`per*a*bun"dant*ly</wf>, <pos>adv.</pos></wordforms><br/
[<source>1913 Webster</source>]</p>

<p><ent>Superacidulated</ent><br/
<hw>Su`per*a*cid"u*la`ted</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Acidulated to excess.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Superadd</ent><br/
<hw>Su`per*add"</hw> <pr>(?)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Superadded</conjf>; <pos>p. pr. & vb. n.</pos> <conjf>Superadding</conjf>.]</vmorph> <ety>[L. <ets>superaddere</ets>. See <er>Super-</er>, and <er>Add</er>.]</ety> <def>To add over and above; to add to what has been added; to annex, as something extrinsic.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The strength of any living creature, in those external motion, is something distinct from, and <qex>superadded</qex> unto, its natural gravity.</q> <rj><qau>Bp. Wilkins.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>The peacock laid it extremely to heart that he had not the nightingale's voice <qex>superadded</qex> to the beauty of his plumes.</q> <rj><qau>L'Estrange.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Superaddition</ent><br/
<hw>Su`per*ad*di"tion</hw> <pr>(?)</pr>, <pos>n.</pos> <def>The act of adding something in excess or something extraneous; also, something which is added in excess or extraneously.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>This <qex>superaddition</qex> is nothing but fat.</q> <rj><qau>Arbuthnot.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Superadvenient</ent><br/
<hw>Su`per*ad*ven"ient</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Coming upon; coming in addition to, or in assistance of, something.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>He has done bravely by the <qex>superadvenient</qex> assistance of his God.</q> <rj><qau>Dr. H. More.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Superalimentation</ent><br/
<hw>Su`per*al`i*men*ta"tion</hw> <pr>(?)</pr>, <pos>n.</pos> <def>The act of overfeeding, or making one take food in excess of the natural appetite for it.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Superaltar</ent><br/
<hw>Su"per*al`tar</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Arch.)</fld> <def>A raised shelf or stand on the back of an altar, on which different objects can be placed; a predella or gradino.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Superangelic</ent><br/
<hw>Su`per*an*gel"ic</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Superior to the angels in nature or rank.</def> <mark>[R.]</mark>  <rj><au>Milman.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Superannuate</ent><br/
<hw>Su`per*an"nu*ate</hw> <pr>(?)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Superannuated</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Superannuating</conjf>.]</vmorph> <ety>[Pref. <ets>super-</ets> + L. <ets>annus</ets> a year.]</ety> <sn>1.</sn> <def>To impair or disquality on account of age or infirmity.</def>  <rj><au>Sir T. Browne.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To give a pension to, on account of old age or other infirmity; to cause to retire from service on a pension.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Superannuate</ent><br/
<hw>Su`per*an"nu*ate</hw> <pr>(?)</pr>, <pos>v. i.</pos> <def>To last beyond the year; -- said of annual plants.</def> <mark>[Obs.]</mark>  <rj><au>Bacon.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Superannuation</ent><br/
<hw>Su`per*an`nu*a"tion</hw> <pr>(?)</pr>, <pos>n.</pos> <def>The state of being superannuated, or too old for office or business; the state of being disqualified by old age; decrepitude.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The world itself is in a state of <qex>superannuation</qex>.</q> <rj><qau>Cowper.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Slyness blinking through the watery eye of <qex>superannuation</qex>.</q> <rj><qau>Coleridge.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Superb</ent><br/
<hw>Su*perb"</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[F. <ets>superbe</ets>, L. <ets>superbus</ets>, fr. <ets>super</ets> over. See <er>Super-</er>.]</ety> <sn>1.</sn> <def>Grand; magnificent; august; stately; <as>as, a <ex>superb</ex> edifice; a <ex>superb</ex> colonnade</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Rich; elegant; <as>as, <ex>superb</ex> furniture or decorations</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>Showy; excellent; grand; <as>as, a <ex>superb</ex> exhibition</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Superb paradise bird</b></col> <fld>(Zool.)</fld>, <cd>a bird of paradise (<spn>Paradisaea superba</spn> syn. <spn>Lophorina superba</spn>) having the scapulars erectile, and forming a large ornamental tuft on each shoulder, and a large gorget of brilliant feathers on the breast . The color is deep violet, or nearly black, with brilliant green reflections.  The gorget is bright metallic green.</cd> -- <col><b>Superb warber</b></col>. <fld>(Zool.)</fld> <cd>See <cref>Blue wren</cref>, under <er>Wren</er>.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p>-- <wordforms><wf>Su*perb"ly</wf>, <pos>adv.</pos> -- <wf>Su*perb"ness</wf>, <pos>n.</pos></wordforms><br/
[<source>1913 Webster</source>]</p>

<p><ent>Superbiate</ent><br/
<hw>Su*per"bi*ate</hw> <pr>(?)</pr>, <pos>v. t.</pos> <ety>[Cf. L. <ets>superbiare</ets>.]</ety> <def>To make (a person) haughty.</def> <mark>[Obs. & R.]</mark>  <rj><au>Feltham.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Supercarbonate</ent><br/
<hw>Su`per*car"bon*ate</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Chem.)</fld> <def>A bicarbonate.</def> <mark>[Obsoles.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Supercarbureted</ent><br/
<hw>Su`per*car"bu*ret`ed</hw> <pr>(?)</pr>, <pos>a.</pos> <fld>(Chem.)</fld> <def>Bicarbureted.</def> <altsp>[Written also <asp>supercarburetted</asp>.]</altsp> <mark>[Obsoles.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Supercargo</ent><br/
<hw>Su`per*car"go</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[<ets>Super-</ets> + <ets>cargo</ets>: cf. Sp. <ets>sobrecargo.</ets>  Cf. <er>Surcharge</er>.]</ety> <def>An officer or person in a merchant ship, whose duty is to manage the sales, and superintend the commercial concerns, of the voyage.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Supercarpal</ent><br/
<hw>Su`per*car"pal</hw> <pr>(?)</pr>, <pos>a.</pos> <fld>(Anat.)</fld> <def>Situated above, or in the upper part of, the carpus.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Supercelestial</ent><br/
<hw>Su`per*ce*les"tial</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[Pref. <ets>super-</ets> + <ets>celestial</ets>: cf. L. <ets>supercaelestis</ets>.]</ety> <sn>1.</sn> <def>Situated above the firmament, or great vault of heaven.</def>  <rj><au>Waterland.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Higher than celestial; superangelic.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Supercharge</ent><br/
<hw>Su`per*charge"</hw> <pr>(?)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Supercharged</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Supercharging</conjf> <pr>(?)</pr>.]</vmorph> <ety>[Pref. <ets>super-</ets> + <ets>charge</ets>.  Cf. <er>Surcharge</er>.]</ety> <fld>(Her.)</fld> <def>To charge (a bearing) upon another bearing; <as>as, to <ex>supercharge</ex> a rose upon a fess</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Supercharge</ent><br/
<hw>Su`per*charge"</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Her.)</fld> <def>A bearing charged upon another bearing.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Superchemical</ent><br/
<hw>Su`per*chem"ic*al</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Above or beyond chemistry; inexplicable by chemical laws.</def>  <rj><au>J. Le Conte.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Superchery</ent><br/
<hw>Su*perch"er*y</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F. <ets>supercherie</ets>.]</ety> <def>Deceit; fraud; imposition.</def> <mark>[Obs. & R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Superciliary</ent><br/
<hw>Su`per*cil"i*a*ry</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>supercilium</ets> an eyebrow. See <er>Supercilious</er>.]</ety> <sn>1.</sn> <def>Of or pertaining to the eyebrows; supraorbital.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Zool.)</fld> <def>Having a distinct streak of color above the eyes; <as>as, the <ex>superciliary</ex> woodpecker</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Supercilious</ent><br/
<hw>Su`per*cil"i*ous</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>superciliosus</ets>, fr. <ets>supercilium</ets> an eyebrow, pride; <ets>super</ets> over, + <ets>cilium</ets> an eyelid; probably akin to <ets>celare</ets> to conceal.  Cf. <er>Conceal</er>.]</ety> <def>Lofty with pride; haughty; dictatorial; overbearing; arrogant; <as>as, a <ex>supercilious</ex> officer; a<ex>supercilious</ex> air; <ex>supercilious</ex> behavior.</as></def> -- <wordforms><wf>Su`per*cil"i*ous*ly</wf>, <pos>adv.</pos> -- <wf>Su`per*cil"i*ous*ness</wf>, <pos>n.</pos></wordforms><br/
[<source>1913 Webster</source>]</p>

<p><ent>Supercilium</ent><br/
||<hw>Su`per*cil"i*um</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L.]</ety> <fld>(Zool.)</fld> <def>The eyebrow, or the region of the eyebrows.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Supercolumniation</ent><br/
<hw>Su`per*co*lum`ni*a"tion</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Arch.)</fld> <def>The putting of one order above another; also, an architectural work produced by this method; <as>as, the putting of the Doric order in the ground story, Ionic above it, and Corinthian or Composite above this</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Superconception</ent><br/
<hw>Su`per*con*cep"tion</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Physiol.)</fld> <def>Superfetation.</def> <mark>[R.]</mark>  <rj><au>Sir T. Browne.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Superconsequence</ent><br/
<hw>Su`per*con"se*quence</hw> <pr>(?)</pr>, <pos>n.</pos> <def>Remote consequence.</def> <mark>[Obs.]</mark>  <rj><au>Sir T. Browne.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Supercrescence</ent><br/
<hw>Su`per*cres"cence</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[See <er>Supercrescent</er>.]</ety> <def>That which grows upon another growing thing; a parasite.</def> <mark>[R.]</mark>  <rj><au>Sir T. Browne.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Supercrescent</ent><br/
<hw>Su`per*cres"cent</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>supercrescens</ets>, p. pr. of <ets>supercrescere</ets>; <ets>super</ets> above + <ets>crescere</ets> to grow.]</ety> <def>Growing on some other growing thing.</def> <mark>[R.]</mark>  <rj><au>Johnson.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Supercretaceous</ent><br/
<hw>Su`per*cre*ta"ceous</hw> <pr>(?)</pr>, <pos>a.</pos> <fld>(Geol.)</fld> <def>Same as <er>Supracretaceous</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><-- supercritical.  Being at a temperature and pressure above the critical temperature and pressure.  In a condition neither liquid nor gaseous.   Supercritical carbon dioxide is used as a solvent in separation chemistry. --><br/
[<source>1913 Webster</source>]</p>

<p><ent>Supercurious</ent><br/
<hw>Su`per*cu"ri*ous</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Excessively curious or inquisitive.</def>  <rj><au>Evelyn.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Superdominant</ent><br/
<hw>Su`per*dom"i*nant</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Mus.)</fld> <def>The sixth tone of the scale; that next above the dominant; -- called also <altname>submediant</altname>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Superdreadnought</ent><br/
<hw>Su`per*dread"nought`</hw> <pr>(?)</pr>, <pos>n.</pos> <def>See <er>Dreadnought</er>, above.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Supereminency</ent><br/
<ent>Supereminence</ent><br/
<mhw>{ <hw>Su`per*em"i*nence</hw> <pr>(?)</pr>, <hw>Su`per*em"i*nen*cy</hw> <pr>(?)</pr>, }</mhw> <pos>n.</pos> <ety>[L. <ets>supereminentia</ets>.]</ety> <def>The quality or state of being supereminent; distinguished eminence; <as>as, the <ex>supereminence</ex> of Cicero as an orator, or Lord Chatham as a statesman</as>.</def>  <rj><au>Ayliffe.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>He was not forever beset with the consciousness of his own <qex>supereminence</qex>.</q> <rj><qau>Prof. Wilson.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Supereminent</ent><br/
<hw>Su`per*em"i*nent</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>supereminens</ets>, p. pr. of <ets>supereminere</ets>. See <er>Super-</er>, and <er>Eminent</er>.]</ety> <def>Eminent in a superior degree; surpassing others in excellence; <as>as, a <ex>supereminent</ex> divine; the <ex>supereminent</ex> glory of Christ</as>.</def> -- <wordforms><wf>Su`per*em"i*nent*ly</wf>, <pos>adv.</pos></wordforms><br/
[<source>1913 Webster</source>]</p>

<p><ent>Supererogant</ent><br/
<hw>Su`per*er"o*gant</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>supererogans</ets>, p. pr. See <er>Supererogate</er>.]</ety> <def>Supererogatory.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Supererogate</ent><br/
<hw>Su`per*er"o*gate</hw> <pr>(?)</pr>, <pos>v. i.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Supererogated</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Supererogating</conjf>.]</vmorph> <ety>[L. <ets>supererogatus</ets>, p. p. of <ets>supererogare</ets> to spend or pay out over and above; <ets>super</ets> over, above + <ets>erogare</ets> to expend or pay out money from the public treasury after asking the consent of the people. See <er>Super-</er>, and <er>Erogate</er>, <er>Rogation</er>.]</ety> <def>To do more than duty requires; to perform works of supererogation; to atone (for a dificiency in another) by means of a surplus action or quality.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The fervency of one man in prayer can not <qex>supererogate</qex> for the coldness of another.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Supererogation</ent><br/
<hw>Su`per*er`o*ga"tion</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>supererogatio</ets> a payment in addition.]</ety> <def>The act of supererogating; performance of more than duty or necessity requires.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Works of supererogation</b></col> <fld>(R. C. Ch.)</fld>, <cd>those good deeds believed to have been performed by saints, or capable of being performed by men, over and above what is required for their own salvation.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Supererogative</ent><br/
<hw>Su`per*e*rog"a*tive</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Supererogatory.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Supererogatory</ent><br/
<hw>Su`per*e*rog"a*to*ry</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Performed to an extent not enjoined, or not required, by duty or necessity; <as>as, <ex>supererogatory</ex> services</as>.</def>  <rj><au>Howell.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Superessential</ent><br/
<hw>Su`per*es*sen"tial</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Essential above others, or above the constitution of a thing.</def>  <rj><au>J. Ellis.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Superethical</ent><br/
<hw>Su`per*eth"ic*al</hw> <pr>(?)</pr>, <pos>a.</pos> <def>More than ethical; above ethics.</def>  <rj><au>Bolingbroke.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Superexalt</ent><br/
<hw>Su`per*ex*alt"</hw> <pr>(?)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Superexalted</conjf>; <pos>p. pr. & vb. n.</pos> <conjf>Superexalting</conjf>.]</vmorph> <def>To exalt to a superior degree; to exalt above others.</def>  <rj><au>Barrow.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Superexaltation</ent><br/
<hw>Su`per*ex`al*ta"tion</hw> <pr>(?)</pr>, <pos>n.</pos> <def>Elevation above the common degree.</def>  <rj><au>Holyday.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Superexcellence</ent><br/
<hw>Su`per*ex"cel*lence</hw> <pr>(?)</pr>, <pos>n.</pos> <def>Superior excellence; extraordinary excellence.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Superexcellent</ent><br/
<hw>Su`per*ex"cel*lent</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[Pref. <ets>super-</ets> + <ets>excellent</ets>: cf. L. <ets>superexcellens</ets>.]</ety> <def>Excellent in an uncommon degree; very excellent.</def>  <rj><au>Drayton.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Superexcitation</ent><br/
<hw>Su`per*ex`ci*ta"tion</hw> <pr>(?)</pr>, <pos>n.</pos> <def>Excessive, or more than normal, excitation.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Superexcrescence</ent><br/
<hw>Su`per*ex*cres"cence</hw> <pr>(?)</pr>, <pos>n.</pos> <def>Something growing superfluously.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Superfamily</ent><br/
<hw>Su"per*fam`i*ly</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Zool.)</fld> <def>A group intermediate between a family and a suborder.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Superfecundation</ent><br/
<hw>Su`per*fec`un*da"tion</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Physiol.)</fld> <def>Fertilization of two ova, at the same menstruation, by two different acts of coition.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Superfecundity</ent><br/
<hw>Su`per*fe*cun"di*ty</hw> <pr>(?)</pr>, <pos>n.</pos> <def>Superabundant fecundity or multiplication of the species.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Superfetate</ent><br/
<hw>Su`per*fe"tate</hw> <pr>(?)</pr>, <pos>v. i.</pos> <ety>[L. <ets>superfetare</ets>; <ets>super</ets> above, over + <ets>fetare</ets> to bring forth.]</ety> <def>To conceive after a prior conception, but before the birth of the offspring.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The female . . . is said to <qex>superfetate</qex>.</q> <rj><qau>Grew.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Superfetation</ent><br/
<hw>Su`per*fe*ta"tion</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Cf. F. <ets>superf<eacute/tation</ets>.]</ety> <fld>(Physiol.)</fld> <def>The formation of a fetus at the result of an impregnation occurring after another impregnation but before the birth of the offspring produced by it. This is possible only when there is a double uterus, or where menstruation persists up to the time of the second impregnation.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>In then became a <qex>superfetation</qex> upon, and not an ingredient in, the national character.</q> <rj><qau>Coleridge.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Superfete</ent><br/
<hw>Su`per*fete"</hw> <pr>(?)</pr>, <pos>v. i.</pos> <def>To superfetate.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Superfete</ent><br/
<hw>Su`per*fete"</hw>, <pos>v. t.</pos> <def>To conceive (another fetus) after a former conception.</def> <mark>[Obs.]</mark>  <rj><au>Howell.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Superfice</ent><br/
<hw>Su"per*fice</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A superficies.</def> <mark>[Obs.]</mark>  <rj><au>Dryden.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Superficial</ent><br/
<hw>Su`per*fi"cial</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>superficialis</ets>: cf. F. <ets>superficiel</ets>. See <er>Superficies</er>.]</ety> <sn>1.</sn> <def>Of or pertaining to the superficies, or surface; lying on the surface; shallow; not deep; <as>as, a <ex>superficial</ex> color; a <ex>superficial</ex> covering; <ex>superficial</ex> measure or contents; <ex>superficial</ex> tillage</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Reaching or comprehending only what is obvious or apparent; not deep or profound; shallow; -- said especially in respect to study, learning, and the like; <as>as, a <ex>superficial</ex> scholar; <ex>superficial</ex> knowledge</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>This <qex>superficial</qex> tale<br/
Is but a preface of her worthy praise.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>He is a presumptuous and <qex>superficial</qex> writer.</q> <rj><qau>Burke.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>That <qex>superficial</qex> judgment, which happens to be right without deserving to be so.</q> <rj><qau>J. H. Newman.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p>-- <wordforms><wf>Su`per*fi"cial*ly</wf>, <pos>adv.</pos> -- <wf>Su`per*fi"cial*ness</wf>, <pos>n.</pos></wordforms><br/
[<source>1913 Webster</source>]</p>

<p><ent>Superficialist</ent><br/
<hw>Su`per*fi"cial*ist</hw>, <pos>n.</pos> <def>One who attends to anything superficially; a superficial or shallow person; a sciolist; a smatterer.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Superficiality</ent><br/
<hw>Su`per*fi`ci*al"i*ty</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Cf. F. <ets>superficialit<eacute/</ets>.]</ety> <def>The quality or state of being superficial; also, that which is superficial.</def>  <rj><au>Sir T. Browne.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Superficialize</ent><br/
<hw>Su`per*fi"cial*ize</hw> <pr>(?)</pr>, <pos>v. t.</pos> <def>To attend to, or to treat, superficially, or in a shallow or slighting way.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>It is a characteristic weakness of the day to <qex>superficialize</qex> evil.</q> <rj><qau>E. P. Whipple.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Superficiary</ent><br/
<hw>Su`per*fi"ci*a*ry</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Rom. Law)</fld> <def>One to whom a right of surface occupation is granted; one who pays quitrent for a house built upon another man's ground.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Superficiary</ent><br/
<hw>Su`per*fi"ci*a*ry</hw>, <pos>a.</pos> <sn>1.</sn> <def>Of or pertaining to the superficies, or surface; superficial.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Rom. Law)</fld> <def>Situated or built on another man's land, as a house.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Superficies</ent><br/
<hw>Su`per*fi"cies</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L., fr. <ets>super</ets> above, over + <ets>facies</ets> make, figure, shape. See <er>Surface</er>.]</ety> <sn>1.</sn> <def>The surface; the exterior part, superficial area, or face of a thing.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Civil Law)</fld> <sd>(a)</sd> <def>Everything on the surface of a piece of ground, or of a building, so closely connected by art or nature as to constitute a part of it, as houses, or other superstructures, fences, trees, vines, etc.</def> <sd>(b)</sd> <def>A real right consisting of a grant by a landed proprietor of a piece of ground, bearing a strong resemblance to the long building leases granted by landholders in England, in consideration of a rent, and under reservation of the ownership of the soil.</def>  <rj><au>Bouvier. Wharton.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Superfine</ent><br/
<hw>Su"per*fine</hw> <pr>(?)</pr>, <pos>a.</pos> <sn>1.</sn> <def>Very fine, or most fine; being of surpassing fineness; of extra nice or fine quality; <as>as, <ex>superfine</ex> cloth</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Excessively fine; too nice; over particular; <as>as, <ex>superfine</ex> distinctions; <ex>superfine</ex> tastes</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Superfineness</ent><br/
<hw>Su"per*fine`ness</hw>, <pos>n.</pos> <def>The state of being superfine.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Superfinical</ent><br/
<hw>Su`per*fin"i*cal</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Extremely finical.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Superfluence</ent><br/
<hw>Su*per"flu*ence</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>superfluens</ets>, p. pr. of <ets>superfluere</ets> to flow or run over. See <er>Superfluous</er>.]</ety> <def>Superfluity.</def> <mark>[Obs.]</mark>  <rj><au>Hammond.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Superfluitant</ent><br/
<hw>Su`per*flu"i*tant</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>super</ets> above + <ets>fluitans</ets>, p. pr. of <ets>fluitare</ets> intensive fr. <ets>fluere</ets> to flow.]</ety> <def>Floating above or on the surface.</def> <mark>[Obs.]</mark> <au>Sir T. Browne.</au> -- <wordforms><wf>Su`per*flu"i*tance</wf> <pr>(#)</pr>, <pos>n.</pos> <mark>[Obs.]</mark></wordforms><br/
[<source>1913 Webster</source>]</p>

<p><-- p. 1447 --></p>

<p><ent>Superfluity</ent><br/
<hw>Su`per*flu"i*ty</hw> <pr>(?)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Superfluities</plw> <pr>(#)</pr>.</plu> <ety>[L. <ets>superfluit<eacute/</ets>, L. <ets>superfluitas</ets>. See <er>Superfluous</er>.]</ety> <sn>1.</sn> <def>A greater quantity than is wanted; superabundance; <as>as, a <ex>superfluity</ex> of water; a <ex>superfluity</ex> of wealth.</as></def><br/
[<source>1913 Webster</source>]</p>

<p><q>A quiet mediocrity is still to be preferred before a troubled <qex>superfluity</qex>.</q> <rj><qau>Suckling.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>The state or quality of being superfluous; excess.</def> <ldquo/By a <xex>superfluity</xex> abominable.<rdquo/  <rj><au>Chaucer.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>Something beyond what is needed; something which serves for show or luxury.</def><br/
[<source>1913 Webster</source>]</p>

<p><syn><b>Syn.</b> -- Superabundance; excess; redundancy.</syn><br/
[<source>1913 Webster</source>]</p>

<p><ent>Superfluous</ent><br/
<hw>Su*per"flu*ous</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>superfluus</ets> overflowing; <ets>super</ets> over, above + <ets>fluere</ets> to flow. See <er>Super-</er>, and <er>Fluent</er>.]</ety> <def>More than is wanted or is sufficient; rendered unnecessary by superabundance; unnecessary; useless; excessive; <as>as, a <ex>superfluous</ex> price</as>.</def>  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>An authority which makes all further argument or illustration <qex>superfluous</qex>.</q> <rj><qau>E. Everett.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Superfluous interval</b></col> <fld>(Mus.)</fld>, <cd>an interval that exceeds a major or perfect interval by a semitone.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><syn><b>Syn.</b> -- Unnecessary; useless; exuberant; excessive; redundant; needless.</syn><br/
[<source>1913 Webster</source>]</p>

<p>-- <wordforms><wf>Su*per"flu*ous*ly</wf>, <pos>adv.</pos> -- <wf>Su*per"flu*ous*ness</wf>, <pos>n.</pos></wordforms><br/
[<source>1913 Webster</source>]</p>

<p><ent>Superflux</ent><br/
<hw>Su"per*flux</hw> <pr>(?)</pr>, <pos>n.</pos> <def>Superabundance; superfluity; an overflowing.</def> <mark>[R.]</mark>  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Superfoetation</ent><br/
<hw>Su`per*foe*ta"tion</hw> <pr>(?)</pr>, <pos>n.</pos> <def>Superfetation.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Superfoliation</ent><br/
<hw>Su`per*fo`li*a"tion</hw> <pr>(?)</pr>, <pos>n.</pos> <def>Excess of foliation.</def>  <rj><au>Sir T. Browne.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Superfrontal</ent><br/
<hw>Su`per*fron"tal</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Eccl.)</fld> <def>A cloth which is placed over the top of an altar, and often hangs down a few inches over the frontal.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Superfuse</ent><br/
<hw>Su`per*fuse"</hw> <pr>(?)</pr>, <pos>a.</pos> <def>To pour (something) over or on something else.</def> <mark>[Obs.]</mark>  <rj><au>Evelyn.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Superheat</ent><br/
<hw>Su`per*heat"</hw> <pr>(?)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Superheated</conjf>; <pos>p. pr. & vb. n.</pos> <conjf>Superheating</conjf>.]</vmorph> <sn>1.</sn> <def>To heat too much, to overheat; <as>as, to <ex>superheat</ex> an oven</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Steam Engine)</fld> <def>To heat, as steam, apart from contact with water, until it resembles a perfect gas.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>To heat (a liquid) above its boiling point without converting it into vapor.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Superheat</ent><br/
<hw>Su"per*heat`</hw>, <pos>n.</pos> <def>The increase of temperature communicated to steam by superheating it.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Superheated</ent><br/
<hw>Su"per*heat`ed</hw> <def>heated to a temperature above the boiling point at the ambient pressure; -- said of liquids.  In such an unstable condition, a small disturbance of the liquid can cause a rapid and sometimes violent ebullition of vapor.</def><br/
[<source>PJC</source>]</p>

<p><ent>Superheater</ent><br/
<hw>Su"per*heat`er</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Steam Engine)</fld> <def>An apparatus for superheating steam.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Superhive</ent><br/
<hw>Su"per*hive`</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A removable upper part of a hive. The word is sometimes contracted to <xex>super</xex>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Superhuman</ent><br/
<hw>Su`per*hu"man</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Above or beyond what is human; sometimes, divine; <as>as, <ex>superhuman</ex> strength; <ex>superhuman</ex> wisdom</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Superimpose</ent><br/
<hw>Su`per*im*pose"</hw> <pr>(?)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Superimposed</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Superimposing</conjf>.]</vmorph> <def>To lay or impose on something else; <as>as, a stratum of earth <ex>superimposed</ex> on another stratum</as>.</def> -- <wordforms><wf>Su`per*im`po*si"tion</wf> <pr>(#)</pr>, <pos>n.</pos></wordforms><br/
[<source>1913 Webster</source>]</p>

<p><ent>Superimpregnation</ent><br/
<hw>Su`per*im`preg*na"tion</hw> <pr>(?)</pr>, <pos>n.</pos> <def>The act of impregnating, or the state of being impregnated, in addition to a prior impregnation; superfetation.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Superincumbency</ent><br/
<ent>Superincumbence</ent><br/
<mhw>{ <hw>Su`per*in*cum"bence</hw> <pr>(?)</pr>, <hw>Su`per*in*cum"ben*cy</hw> <pr>(?)</pr>, }</mhw> <pos>n.</pos> <def>The quality or state of being superincumbent.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Superincumbent</ent><br/
<hw>Su`per*in*cum"bent</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>superincumbens</ets>, p. pr. of <ets>superincumbere</ets>. See <er>Super-</er>, and <er>Incumbent</er>.]</ety> <def>Lying or resting on something else.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Superinduce</ent><br/
<hw>Su`per*in*duce"</hw> <pr>(?)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Superinduced</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Superinducing</conjf> <pr>(?)</pr>.]</vmorph> <ety>[Pref. <ets>super-</ets> + <ets>induce</ets>: cf. L. <ets>superinducere</ets> to draw over.]</ety> <def>To bring in, or upon, as an addition to something.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Long custom of sinning <qex>superinduces</qex> upon the soul new and absurd desires.</q> <rj><qau>South.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Superinducement</ent><br/
<hw>Su`per*in*duce"ment</hw> <pr>(?)</pr>, <pos>n.</pos> <def>Superinduction.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Superinduction</ent><br/
<hw>Su`per*in*duc"tion</hw> <pr>(?)</pr>, <pos>n.</pos> <def>The act of superinducing, or the state of being superinduced.</def>  <rj><au>South.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Superinfuse</ent><br/
<hw>Su`per*in*fuse"</hw> <pr>(?)</pr>, <pos>v. t.</pos> <ety>[Pref. <ets>super-</ets> + <ets>infuse</ets>: cf. L. <ets>superinfundere</ets>, <ets>superinfusum</ets>, to pour over.]</ety> <def>To infuse over.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Superinjection</ent><br/
<hw>Su`per*in*jec"tion</hw> <pr>(?)</pr>, <pos>n.</pos> <def>An injection succeeding another.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Superinspect</ent><br/
<hw>Su`per*in*spect"</hw> <pr>(?)</pr>, <pos>v. t.</pos> <ety>[Pref. <ets>super-</ets> + <ets>inspect</ets>: cf. L. <ets>superinspicere</ets>, <ets>superinspectum</ets>.]</ety> <def>To over see; to superintend by inspection.</def> <mark>[R.]</mark>  <rj><au>Maydman.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Superinstitution</ent><br/
<hw>Su`per*in`sti*tu"tion</hw> <pr>(?)</pr>, <pos>n.</pos> <def>One institution upon another, as when A is instituted and admitted to a benefice upon a title, and B instituted and admitted upon the presentation of another.</def>  <rj><au>Bailey.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Superintellectual</ent><br/
<hw>Su`per*in`tel*lec"tu*al</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Being above intellect.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Superintend</ent><br/
<hw>Su`per*in*tend"</hw> <pr>(?)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Superintended</conjf>; <pos>p. pr. & vb. n.</pos> <conjf>Superintending</conjf>.]</vmorph> <ety>[L. <ets>superintendere</ets>. See <er>Super-</er>, and <er>Intend</er>.]</ety> <def>To have or exercise the charge and oversight of; to oversee with the power of direction; to take care of with authority; to supervise; <as>as, an officer <ex>superintends</ex> the building of a ship or the construction of a fort</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The king may appoint a council, who may <qex>superintend</qex> the works of this nature.</q> <rj><qau>Bacon.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><syn><b>Syn.</b> -- <er>Superintend</er>, <er>Supervise</er>.</syn> <usage> These words in general use are the synonymous. As sometimes used, <xex>supervise</xex> implies the more general, and <xex>superintend</xex>, the more particular and constant, inspection or direction. Among architects there is a disposition to use the word <xex>supervise</xex> in the sense of a general oversight of the main points of construction with reference to the design, etc., and to employ the word <xex>superintend</xex> to signify a constant, careful attention to all the details of construction. But this technical distinction is not firmly established.</usage><br/
[<source>1913 Webster</source>]</p>

<p><ent>Superintendence</ent><br/
<hw>Su`per*in*tend"ence</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Cf. F. <ets>superintendance</ets>.]</ety> <def>The act of superintending; care and oversight for the purpose of direction; supervision.</def>  <rj><au>Barrow.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><syn><b>Syn.</b> -- Inspection; oversight; care; direction; control; guidance.</syn><br/
[<source>1913 Webster</source>]</p>

<p><ent>Superintendency</ent><br/
<hw>Su`per*in*tend"en*cy</hw> <pr>(?)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>-cies</plw> <pr>(<?/)</pr>.</plu> <def>The act of superintending; superintendence.</def>  <rj><au>Boyle.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Superintendent</ent><br/
<hw>Su`per*in*tend"ent</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>superintendens</ets>, p. pr. See <er>Superintend</er>.]</ety> <def>Overseeing; superintending.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Superintendent</ent><br/
<hw>Su`per*in*tend"ent</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Cf. OF. <ets>superintendant</ets>, F. <ets>surintendant</ets>.  Cf. <er>Surintendant</er>.]</ety> <def>One who has the oversight and charge of some place, institution, or organization, affairs, etc., with the power of direction; <as>as, the <ex>superintendent</ex> of an almshouse; the <ex>superintendent</ex> of public works</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><syn><b>Syn.</b> -- Inspector; overseer; manager; director; curator; supervisor.</syn><br/
[<source>1913 Webster</source>]</p>

<p><ent>Superintender</ent><br/
<hw>Su`per*in*tend"er</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A superintendent.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Superinvestiture</ent><br/
<hw>Su`per*in*vest"i*ture</hw> <pr>(?)</pr>, <pos>n.</pos> <def>An outer vestment or garment.</def> <mark>[R.]</mark>  <rj><au>Bp. Horne.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Superior</ent><br/
<hw>Su*pe"ri*or</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L., compar. of <ets>superus</ets> being above, fr. <ets>super</ets> above, over: cf. F. <ets>sup<eacute/rieur</ets>. See <er>Super-</er>, and cf. <er>Supreme</er>.]</ety> <sn>1.</sn> <def>More elevated in place or position; higher; upper; <as>as, the <ex>superior</ex> limb of the sun; the <ex>superior</ex> part of an image</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Higher in rank or office; more exalted in dignity; <as>as, a <ex>superior</ex> officer; a <ex>superior</ex> degree of nobility</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>Higher or greater in excellence; surpassing others in the greatness, or value of any quality; greater in quality or degree; <as>as, a man of <ex>superior</ex> merit; or of <ex>superior</ex> bravery</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>Beyond the power or influence of; too great or firm to be subdued or affected by; -- with <xex>to</xex>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>There is not in earth a spectacle more worthy than a great man <qex>superior</qex> to his sufferings.</q> <rj><qau>Spectator.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <def>More comprehensive; as a term in classification; <as>as, a genus is <ex>superior</ex> to a species</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>6.</sn> <fld>(Bot.)</fld> <sd>(a)</sd> <def>Above the ovary; -- said of parts of the flower which, although normally below the ovary, adhere to it, and so appear to originate from its upper part; also of an ovary when the other floral organs are plainly below it in position, and free from it.</def> <sd>(b)</sd> <def>Belonging to the part of an axillary flower which is toward the main stem; posterior.</def> <sd>(c)</sd> <def>Pointing toward the apex of the fruit; ascending; -- said of the radicle.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><mcol><col><b>Superior conjunction</b></col>, <col><b>Superior planets</b></col></mcol>, <cd>etc. See <er>Conjunction</er>, <er>Planet</er>, etc.</cd> -- <mcol><col><b>Superior figure</b></col>, <col><b>Superior letter</b></col></mcol> <fld>(Print.)</fld>, <cd>a figure or letter printed above the line, as a reference to a note or an index of a power, etc; <as>as, in <mathex>x<exp>2</exp> + y<exp>n</exp></mathex>, 2 is a <ex>superior figure</ex>, <it>n</it> a <ex>superior letter</ex></as>.  Cf. <cref>Inferior figure</cref>, under <er>Inferior</er>.</cd><-- = superscripted number, letter --></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Superior</ent><br/
<hw>Su*pe"ri*or</hw>, <pos>n.</pos> <sn>1.</sn> <def>One who is above, or surpasses, another in rank, station, office, age, ability, or merit; one who surpasses in what is desirable; <as>as, Addison has no <ex>superior</ex> as a writer of pure English</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Eccl.)</fld> <def>The head of a monastery, convent, abbey, or the like.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Superioress</ent><br/
<hw>Su*pe"ri*or*ess</hw>, <pos>n.</pos> <fld>(Eccl.)</fld> <def>A woman who acts as chief in a convent, abbey, or nunnery; a lady superior.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Superiority</ent><br/
<hw>Su*pe`ri*or"i*ty</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Cf. F. <ets>sup<eacute/riorit<eacute/</ets>, LL. <ets>superioritas</ets>.]</ety> <def>The quality, state, or condition of being superior; <as>as, <ex>superiority</ex> of rank; <ex>superiority</ex> in merit.</as></def><br/
[<source>1913 Webster</source>]</p>

<p><syn><b>Syn.</b> -- Preeminence; excellence; predominancy; prevalence; ascendency; odds; advantage.</syn><br/
[<source>1913 Webster</source>]</p>

<p><ent>Superiorly</ent><br/
<hw>Su*pe"ri*or*ly</hw> <pr>(?)</pr>, <pos>adv.</pos> <def>In a superior position or manner.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Superjacent</ent><br/
<hw>Su`per*ja"cent</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>superjacens</ets>, p. pr. of <ets>superjacere</ets>; <ets>super</ets> above + <ets>jacere</ets> to lie.]</ety> <def>Situated immediately above; <as>as, <ex>superjacent</ex> rocks</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Superlation</ent><br/
<hw>Su`per*la"tion</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>superlatio</ets>. See <er>Superlative</er>.]</ety> <def>Exaltation of anything beyond truth or propriety.</def> <mark>[Obs.]</mark>  <rj><au>B. Jonson.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Superlative</ent><br/
<hw>Su`per*la"tive</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>superlativus</ets>, fr. <ets>superlatus</ets> excessive, used as p. p. of <ets>superiorferre</ets>, but from a different root: cf. F. <ets>superlatif</ets>. See <er>Elate</er>, <er>Tolerate</er>.]</ety> <sn>1.</sn> <def>Lifted up to the highest degree; most eminent; surpassing all other; supreme; <as>as, <ex>superlative</ex> wisdom or prudence; a woman of <ex>superlative</ex> beauty; the <ex>superlative</ex> glory of the divine character.</as></def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Gram.)</fld> <def>Expressing the highest or lowest degree of the quality, manner, etc., denoted by an adjective or an adverb. The superlative degree is formed from the positive by the use of <xex>-est</xex>, <xex>most</xex>, or <xex>least</xex>; <as>as, high<ex>est</ex>, <ex>most</ex> pleasant, <ex>least</ex> bright</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p>-- <wordforms><wf>Su`per*la"tive*ly</wf>, <pos>adv.</pos> -- <wf>Su`per*la"tive*ness</wf>, <pos>n.</pos></wordforms><br/
[<source>1913 Webster</source>]</p>

<p><ent>Superlative</ent><br/
<hw>Su`per*la"tive</hw>, <pos>n.</pos> <sn>1.</sn> <def>That which is highest or most eminent; the utmost degree.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Gram.)</fld> <sd>(a)</sd> <def>The superlative degree of adjectives and adverbs; also, a form or word by which the superlative degree is expressed; <as>as, <ex>strongest</ex>, <ex>wisest</ex>, <ex>most stormy</ex>, <ex>least windy</ex>, are all <ex>superlatives</ex></as>.</def>
<-- #sic. there is no definition (b)! --><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Absolute superlative</b></col>, <cd>a superlative in an absolute rather than in a comparative or exclusive sense. See <er>Elative</er>.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Superlucration</ent><br/
<hw>Su`per*lu*cra"tion</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Pref. <ets>super-</ets> + L. <ets>lucratio</ets> gain.]</ety> <def>Excessive or extraordinary gain.</def> <mark>[Obs.]</mark>  <rj><au>Davenant.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Superlunary</ent><br/
<ent>Superlunar</ent><br/
<mhw>{ <hw>Su`per*lu"nar</hw> <pr>(?)</pr>, <hw>Su`per*lu"na*ry</hw> <pr>(?)</pr>, }</mhw> <pos>a.</pos> <def>Being above the moon; not belonging to this world; -- opposed to <xex>sublunary</xex>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The head that turns at <qex>superlunar</qex> things.</q> <rj><qau>Pope.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Superman</ent><br/
<hw>Su"per*man`</hw> <pr>(?)</pr>, <pos>n.</pos> <sn>1.</sn> <def>same as <er>Overman</er>, above.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><sn>2.</sn> <def>a fictional character of cartoons, movies and television, with superhuman powers such as great strength, the ability to fly, and x-ray vision.  In the cartoon tale, he was born on the planet Krypton and sent by his parents into space before it exploded, and landed on earth, where he fights for "truth, justice, and the American way".  He works incognito as a reporter at the Daily Planet, and is constantly trying to avoid the uncovering of his secret identity by a co-worker, Lois Lane.</def><br/
[<source>PJC</source>]</p>

<p><ent>Supermaterial</ent><br/
<hw>Su`per*ma*te"ri*al</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Being above, or superior to, matter.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Supermaxilla</ent><br/
||<hw>Su`per*max*il"la</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[NL. See <er>Super-</er>, and <er>Maxilla</er>.]</ety> <fld>(Anat.)</fld> <def>The supermaxilla.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Supermaxillary</ent><br/
<hw>Su`per*max"il*la*ry</hw> <pr>(?)</pr>, <pos>a.</pos> <fld>(Anat.)</fld> <def>Supermaxillary.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Supermedial</ent><br/
<hw>Su`per*me"di*al</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Above the middle.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Supermundane</ent><br/
<hw>Su`per*mun"dane</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Being above the world; -- opposed to <xex>inframundane</xex>.</def>  <rj><au>Cudworth.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Supermundial</ent><br/
<hw>Su`per*mun"di*al</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Supermundane.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Supernacular</ent><br/
<hw>Su`per*nac"u*lar</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Like supernaculum; first-rate; <as>as, a <ex>supernacular</ex> wine</as>.</def> <mark>[R.]</mark>  <rj><au>Thackeray.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Supernaculum</ent><br/
<hw>Su`per*nac"u*lum</hw> <pr>(?)</pr>, <pos>adv. & n.</pos> <ety>[NL., from L. <ets>super</ets> over + G. <ets>nagel</ets>, a nail, as of the finger, or a corruption of L. <ets>super</ets> and <ets>ungulam</ets> claw.]</ety> <sn>1.</sn> <def>A kind of mock Latin term intended to mean, upon the nail; -- used formerly by topers.</def>  <rj><au>Nares.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Drinking <qex>super nagulum</qex> [<qex>supernaculum</qex>], a device of drinking, new come out of France, which is, after a man hath turned up the bottom of the cup, to drop it on his nail and make a pearl with that is left; which if it slide, and he can not make it stand on by reason there is too much, he must drink again for his penance.</q> <rj><qau>Nash.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Good liquor, of which not enough is left to wet one's nail.</def>  <rj><au>Grose.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Supernal</ent><br/
<hw>Su*per"nal</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>supernus</ets>, from <ets>super</ets> above: cf. F. <ets>supernel</ets>. See <er>Super-</er>.]</ety> <sn>1.</sn> <def>Being in a higher place or region; locally higher; <as>as, the <ex>supernal</ex> orbs; <ex>supernal</ex> regions</as>.</def> <ldquo/That <xex>supernal</xex> judge.<rdquo/  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Relating or belonging to things above; celestial; heavenly; <as>as, <ex>supernal</ex> grace</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Not by the sufferance of <qex>supernal</qex> power.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Supernatant</ent><br/
<hw>Su`per*na"tant</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>supernatanus</ets>, p. pr. of <ets>supernatare</ets> to swim above; <ets>super</ets> above + <ets>natare</ets> to swim.]</ety> <def>Swimming above; floating on the surface; <as>as, oil <ex>supernatant</ex> on water</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Supernatant</ent><br/
<--<hw>Su`per*na"tant</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Chem.)</fld> <def>The liquid remaining after solids suspended in the liquid have been sedimented by gravity or by centrifugation.  Contrasted with the solid <contr>sediment</contr>, or (in centrifugation) the <contr>pellet</contr>.</def> --><br/
[<source>1913 Webster</source>]</p>

<p><ent>Supernatation</ent><br/
<hw>Su`per*na*ta"tion</hw> <pr>(?)</pr>, <pos>n.</pos> <def>The act of floating on the surface of a fluid.</def>  <rj><au>Sir T. Browne.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Supernatural</ent><br/
<hw>Su`per*nat"u*ral</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[Pref. <ets>super-</ets> + <ets>natural</ets>: cf. OF. <ets>supernaturel</ets>, F. <ets>surnaturel</ets>.]</ety> <def>Being beyond, or exceeding, the power or laws of nature; miraculous.</def><br/
[<source>1913 Webster</source>]</p>

<p><syn><b>Syn.</b> -- Preternatural.</syn>  -- <usage><er>Supernatural</er>, <er>Preternatural</er>. <xex>Preternatural</xex> signifies <xex>beside</xex> nature, and <xex>supernatural</xex>, <xex>above</xex> or <xex>beyond</xex> nature. What is very greatly aside from the ordinary course of things is <xex>preternatural</xex>; what is above or beyond the established laws of the universe is <xex>supernatural</xex>. The dark day which terrified all Europe nearly a century ago was <xex>preternatural</xex>; the resurrection of the dead is <xex>supernatural</xex>. <ldquo/That form which the earth is under at present is <xex>preternatural</xex>, like a statue made and broken again.<rdquo/ <au>T. Burnet.</au> <ldquo/Cures wrought by medicines are natural operations; but the miraculous ones wrought by Christ and his apostles were <xex>supernatural</xex>.<rdquo/ <au>Boyle.</au></usage><br/
[<source>1913 Webster</source>]</p>

<p><q>That is <qex>supernatural</qex>, whether it be, that is either not in the chain of natural cause and effect, or which acts on the chain of cause and effect in nature, from without the chain.</q> <rj><qau>Bushnell.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>We must not view creation as <qex>supernatural</qex>, but we do look upon it as miraculous.</q> <rj><qau>McCosh.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>The supernatural</b></col>, <cd>whatever is above and beyond the scope, or the established course, of the laws of nature.</cd> <ldquo/Nature and the <xex>supernatural</xex>.<rdquo/ <au>H. Bushnell.</au></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Supernaturalism</ent><br/
<hw>Su`per*nat"u*ral*ism</hw> <pr>(?)</pr>, <pos>n.</pos> <sn>1.</sn> <def>The quality or state of being supernatural; supernaturalness.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Theol.)</fld> <def>The doctrine of a divine and supernatural agency in the production of the miracles and revelations recorded in the Bible, and in the grace which renews and sanctifies men, -- in opposition to the doctrine which denies the agency of any other than physical or natural causes in the case.</def> <altsp>[Written also <asp>supranaturalism</asp>.]</altsp><br/
[<source>1913 Webster</source>]</p>

<p><ent>Supernaturalist</ent><br/
<hw>Su`per*nat"u*ral*ist</hw>, <pos>n.</pos> <def>One who holds to the principles of supernaturalism.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Supernaturalistic</ent><br/
<hw>Su`per*nat`u*ral*is"tic</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Of or pertaining to supernaturalism.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Supernaturality</ent><br/
<hw>Su`per*nat`u*ral"i*ty</hw> <pr>(?)</pr>, <pos>n.</pos> <def>The quality or state of being supernatural.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Supernaturalize</ent><br/
<hw>Su`per*nat"u*ral*ize</hw> <pr>(?)</pr>, <pos>v. t.</pos> <def>To treat or regard as supernatural.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Supernaturally</ent><br/
<hw>Su`per*nat"u*ral*ly</hw>, <pos>adv.</pos> <def>In a supernatural manner.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Supernaturalness</ent><br/
<hw>Su`per*nat"u*ral*ness</hw>, <pos>n.</pos> <def>The quality or state of being supernatural.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Supernumerary</ent><br/
<hw>Su`per*nu"mer*a*ry</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>supernumerarius</ets>: cf. OF. <ets>supernum<eacute/raire</ets>, F. <ets>surnum<eacute/raire</ets>. See <er>Super-</er>, and <er>Numerary</er>, <er>Number</er>.]</ety> <sn>1.</sn> <def>Exceeding the number stated or prescribed; <as>as, a <ex>supernumerary</ex> officer in a regiment</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Exceeding a necessary, usual, or required number or quality; superfluous; <as>as, <ex>supernumerary</ex> addresses; <ex>supernumerary</ex> expense</as>.</def>  <rj><au>Addison.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Supernumerary</ent><br/
<hw>Su`per*nu"mer*a*ry</hw>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Supernumeraries</plw> <pr>(<?/)</pr>.</plu> <sn>1.</sn> <def>A person or thing beyond the number stated.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A person or thing beyond what is necessary or usual; especially, a person employed not for regular service, but only to fill the place of another in case of need; specifically, in theaters, a person who is not a regular actor, but is employed to appear in a stage spectacle.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Superoccipital</ent><br/
<hw>Su`per*oc*cip"i*tal</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Supraoccipital.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Superorder</ent><br/
<hw>Su`per*or"der</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Zool.)</fld> <def>A group intermediate in importance between an order and a subclass.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Superordination</ent><br/
<hw>Su`per*or`di*na"tion</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Pref. <ets>super-</ets> + <ets>ordination</ets>: cf. L. <ets>superordinatio</ets>.]</ety> <def>The ordination of a person to fill a station already occupied; especially, the ordination by an ecclesiastical official, during his lifetime, of his successor.</def>  <rj><au>Fuller.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Superoxide</ent><br/
<hw>Su`per*ox"ide</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Chem.)</fld> <def>See <er>Peroxide</er>.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Superparticular</ent><br/
<hw>Su`per*par*tic"u*lar</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>superparticularis</ets>. See <er>Super-</er>, and <er>Particular</er>.]</ety> <fld>(Math.)</fld> <def>Of or pertaining to a ratio when the excess of the greater term over the less is a unit, as the ratio of 1 to 2, or of 3 to 4.</def> <mark>[Obs.]</mark>  <rj><au>Hutton.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Superpartient</ent><br/
<hw>Su`per*par"tient</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>superpartiens</ets>; <ets>super</ets> over + <ets>partiens</ets>, p. pr. of <ets>partire</ets> to divide.]</ety> <fld>(Math.)</fld> <def>Of or pertaining to a ratio when the excess of the greater term over the less is more than a unit, as that of 3 to 5, or 7 to 10.</def> <mark>[Obs.]</mark>  <rj><au>Hutton.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Superphosphate</ent><br/
<hw>Su`per*phos"phate</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Chem.)</fld> <def>An acid phosphate.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Superphosphate of lime</b></col> <fld>(Com. Chem.)</fld>, <cd>a fertilizer obtained by trating bone dust, bone black, or phosphorite with sulphuric acid, whereby the insoluble neutral calcium phosphate, <chform>Ca3(PO4)2</chform>, is changed to the primary or acid calcium phosphate <chform>Ca(H2PO4)2</chform>, which is soluble and therefore available for the soil.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Superphysical</ent><br/
<hw>Su`per*phys"ic*al</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Above or beyond physics; not explainable by physical laws.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Something <qex>superphysical</qex> and superchemical.</q> <rj><qau>J. Le Conte.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Superplant</ent><br/
<hw>Su"per*plant`</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A plant growing on another, as the mistletoe; an epiphyte.</def> <mark>[Obs.]</mark>  <rj><au>Bacon.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Superplease</ent><br/
<hw>Su`per*please"</hw> <pr>(?)</pr>, <pos>v. t.</pos> <def>To please exceedingly.</def> <mark>[Obs.]</mark>  <rj><au>B. Jonson.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Superplus</ent><br/
<hw>Su"per*plus</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Pref. <ets>super-</ets> + L. <ets>plus</ets> more. See <er>Surplus</er>.]</ety> <def>Surplus.</def> <mark>[Obs.]</mark>  <rj><au>Goldsmith.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Superplusage</ent><br/
<hw>Su"per*plus`age</hw> <pr>(?)</pr>, <pos>n.</pos> <def>Surplusage.</def> <mark>[Obs.]</mark> <ldquo/There yet remained a <xex>superplusage</xex>.<rdquo/  <rj><au>Bp. Fell.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Superpolitic</ent><br/
<hw>Su`per*pol"i*tic</hw> <pr>(?)</pr>, <pos>a.</pos> <def>More than politic; above or exceeding policy.</def>  <rj><au>Milton.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><-- p. 1448 --></p>

<p><ent>Superponderate</ent><br/
<hw>Su`per*pon"der*ate</hw> <pr>(?)</pr>, <pos>v. t.</pos> <def>To wiegh over and above.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Superposable</ent><br/
<hw>Su`per*pos"a*ble</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Capable of being superposed, as one figure upon another.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Superpose</ent><br/
<hw>Su`per*pose"</hw> <pr>(?)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Superposed</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Superposing</conjf>.]</vmorph> <ety>[F. <ets>superposer</ets>. See <er>Super-</er>, and <er>Pose</er>.]</ety> <sn>1.</sn> <def>To lay upon, as one kind of rock on another.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Geom.)</fld> <def>To lay (a figure) upon another in such a manner that all the parts of the one coincide with the parts of the other; <as>as, to <ex>superpose</ex> one plane figure on another</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Superposition</ent><br/
<hw>Su`per*po*si"tion</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Cf. F. <ets>superposition</ets>. See <er>Super-</er>, and <er>Position</er>.]</ety> <def>The act of superposing, or the state of being superposed; <as>as, the <ex>superposition</ex> of rocks; the <ex>superposition</ex> of one plane figure on another, in geometry</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Superpraise</ent><br/
<hw>Su`per*praise"</hw> <pr>(?)</pr>, <pos>v. t.</pos> <def>To praise to excess.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>To vow, and swear, and <qex>superpraise</qex> my parts.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Superproportion</ent><br/
<hw>Su`per*pro*por"tion</hw> <pr>(?)</pr>, <pos>n.</pos> <def>Overplus or excess of proportion.</def>  <rj><au>Sir K. Digby.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Superpurgation</ent><br/
<hw>Su`per*pur*ga"tion</hw> <pr>(?)</pr>, <pos>n.</pos> <def>Excessive purgation.</def>  <rj><au>Wiseman.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Superreflection</ent><br/
<hw>Su`per*re*flec"tion</hw> <pr>(?)</pr>, <pos>n.</pos> <def>The reflection of a reflected image or sound.</def> <mark>[R.]</mark>  <rj><au>Bacon.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Superregal</ent><br/
<hw>Su`per*re"gal</hw> <pr>(?)</pr>, <pos>a.</pos> <def>More than regal; worthy of one greater than a king.</def>  <rj><au>Waterland.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Superreward</ent><br/
<hw>Su`per*re*ward"</hw> <pr>(?)</pr>, <pos>v. t.</pos> <def>To reward to an excessive degree.</def>  <rj><au>Bacon.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Superroyal</ent><br/
<hw>Su`per*roy"al</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Larger than royal; -- said of a particular size of printing and writing paper. See the Note under <er>Paper</er>, <pos>n.</pos></def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Supersacral</ent><br/
<hw>Su`per*sa"cral</hw> <pr>(?)</pr>, <pos>a.</pos> <fld>(Anat.)</fld> <def>Situated over, or on the dorsal side of, the sacrum.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Supersaliency</ent><br/
<hw>Su`per*sa"li*en*cy</hw> <pr>(?)</pr>, <pos>n.</pos> <def>The act of leaping on anything.</def> <mark>[Obs.]</mark>  <rj><au>Sir T. Browne.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Supersalient</ent><br/
<hw>Su`per*sa"li*ent</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[Pref. <ets>super-</ets> + L. <ets>saliens</ets> p. pr. of <ets>salire</ets> to leap.]</ety> <def>Leaping upon.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Supersalt</ent><br/
<hw>Su`per*salt"</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Chem.)</fld> <def>An acid salt. See <cref>Acid salt</cref> <sd>(a)</sd>, under <er>Salt</er>, <pos>n.</pos></def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Supersaturate</ent><br/
<hw>Su`per*sat"u*rate</hw> <pr>(?)</pr>, <pos>v. t.</pos> <def>To add to beyond saturation; <as>as, to <ex>supersaturate</ex> a solution</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Supersaturation</ent><br/
<hw>Su`per*sat`u*ra"tion</hw> <pr>(?)</pr>, <pos>n.</pos> <def>The operation of supersaturating, or the state of being supersaturated.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Superscribe</ent><br/
<hw>Su`per*scribe"</hw> <pr>(?)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Superscribed</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Superscribing</conjf>.]</vmorph> <ety>[L. <ets>superscribere</ets>, <ets>superscriptum</ets>; <ets>super</ets> over + <ets>scribere</ets> to write. See <er>Super-</er>, and <er>Scribe</er>.]</ety> <def>To write or engrave (a name, address, inscription, or the like) on the top or surface; to write a name, address, or the like, on the outside or cover of (anything); <as>as, to <ex>superscribe</ex> a letter</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Superscript</ent><br/
<hw>Su"per*script</hw> <pr>(?)</pr>, <pos>n.</pos> <def>Superscription.</def> <mark>[Obs.]</mark> <ldquo/I will overglance the <xex>superscript</xex>.<rdquo/  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Superscription</ent><br/
<hw>Su`per*scrip"tion</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>superscriptio</ets>. See <er>Superscribe</er>.]</ety> <sn>1.</sn> <def>The act of superscribing.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>That which is written or engraved on the surface, outside, or above something else; specifically, an address on a letter, envelope, or the like.</def>  <rj><au>Holland.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>The <qex>superscription</qex> of his accusation was written over, The King of the Jews.</q> <rj><qau>Mark xv. 26.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <fld>(Pharm.)</fld> <def>That part of a prescription which contains the Latin word <xex>recipe</xex> (Take) or the sign <?/.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Supersecular</ent><br/
<hw>Su`per*sec"u*lar</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Being above the world, or secular things.</def>  <rj><au>Bp. Hall.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Supersede</ent><br/
<hw>Su`per*sede"</hw> <pr>(?)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Superseded</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Superseding</conjf>.]</vmorph> <ety>[L. <ets>supersedere</ets>, <ets>supersessum</ets>, to sit above, be superior to, forbear, omit; <ets>super</ets> above + <ets>sedere</ets> to sit: cf. F. <ets>supers<eacute/der</ets>. See <er>Sit</er>, and cf. <er>Surcease</er>.]</ety> <sn>1.</sn> <def>To come, or be placed, in the room of; to replace.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To displace, or set aside, and put another in place of; <as>as, to <ex>supersede</ex> an officer</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>To make void, inefficacious, or useless, by superior power, or by coming in the place of; to set aside; to render unnecessary; to suspend; to stay.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Nothing is supposed that can <qex>supersede</qex> the known laws of natural motion.</q> <rj><qau>Bentley.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <fld>(Old Law)</fld> <def>To omit; to forbear.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Supersedeas</ent><br/
||<hw>Su`per*se"de*as</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L., suspend, set aside, stay, 2d pers. sing. present subjunctive of <ets>supersedere</ets>. See <er>Supersede</er>.]</ety> <fld>(Law)</fld> <def>A writ of command to suspend the powers of an officer in certain cases, or to stay proceedings under another writ.</def>  <rj><au>Blackstone.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Supersedure</ent><br/
<hw>Su*per*se"dure</hw> <pr>(?)</pr>, <pos>n.</pos> <def>The act of superseding, or setting aside; supersession; <as>as, the <ex>supersedure</ex> of trial by jury</as>.</def>  <rj><au>A. Hamilton.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Superseminate</ent><br/
<hw>Su`per*sem"i*nate</hw> <pr>(?)</pr>, <pos>v. t.</pos> <def>To sow, as seed, over something previously sown.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>That can not be done with joy, when it shall be indifferent to any man to <qex>superseminate</qex> what he please.</q> <rj><qau>Jer. Taylor.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Supersemination</ent><br/
<hw>Su`per*sem`i*na"tion</hw> <pr>(?)</pr>, <pos>n.</pos> <def>The sowing of seed over seed previously sown.</def> <mark>[Obs.]</mark>  <rj><au>Abp. Bramhall.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Supersensible</ent><br/
<hw>Su`per*sen"si*ble</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[Pref. <ets>super-</ets> + <ets>sensible</ets>: cf. F. <ets>supersensible</ets>.]</ety> <def>Beyond the reach of the senses; above the natural powers of perception.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Supersensitive</ent><br/
<hw>Su`per*sen"si*tive</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Excessively sensitive; morbidly sensitive.</def> -- <wordforms><wf>Su`per*sen"si*tive*ness</wf>, <pos>n.</pos></wordforms><br/
[<source>1913 Webster</source>]</p>

<p><ent>Supersensual</ent><br/
<hw>Su`per*sen"su*al</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Supersensible.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Supersensuous</ent><br/
<hw>Su`per*sen"su*ous</hw> <pr>(?)</pr>, <pos>a.</pos> <sn>1.</sn> <def>Supersensible.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Excessively sensuous.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Superserviceable</ent><br/
<hw>Su`per*serv"ice*a*ble</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Overofficious; doing more than is required or desired.</def> <ldquo/A <xex>superserviceable</xex>, finical rogue.<rdquo/  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Supersession</ent><br/
<hw>Su`per*ses"sion</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Cf. OF. <ets>supersession</ets>. See <er>Supersede</er>.]</ety> <def>The act of superseding, or the state of being superseded; supersedure.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The general law of diminishing return from land would have undergone, to that extent, a temporary <qex>supersession</qex>.</q> <rj><qau>J. S. Mill.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Supersolar</ent><br/
<hw>Su`per*so"lar</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Above the sun.</def>  <rj><au>Emerson.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>supersonic</ent><br/
<hw>su`per*son"ic</hw> <pr>(?)</pr>, <pos>a.</pos> <sn>1.</sn> <def>Moving at a speed greater than the speed of sound in air; -- this speed varies with altitude, since the speed of sound varies with altitude, as well as with temperature and humidity, being approximately 1129 feet per second at sea level and 984 feet per second at 16 miles altitude.</def><br/
[<source>PJC</source>]</p>

<p><sn>2.</sn> <def>Capable of moving at speeds higher than the speed of sound; -- of aircraft.</def><br/
[<source>PJC</source>]</p>

<p><ent>Supersphenoidal</ent><br/
<hw>Su`per*sphe*noid"al</hw> <pr>(?)</pr>, <pos>a.</pos> <fld>(Anat.)</fld> <def>Situated above, or on the dorsal side of, the body of the sphenoid bone.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Superspinous</ent><br/
<hw>Su`per*spi"nous</hw> <pr>(?)</pr>, <pos>a.</pos> <fld>(Anat.)</fld> <def>Supraspinuos.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Superstition</ent><br/
<hw>Su`per*sti"tion</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F. <ets>superstition</ets>, L. <ets>superstitio</ets>, originally, a standing still over or by a thing; hence, amazement, wonder, dread, especially of the divine or supernatural, fr. <ets>superstare</ets> to stand over; <ets>super</ets> over + <ets>stare</ets> to stand. See <er>Super-</er>, and <er>Stand</er>.]</ety> <sn>1.</sn> <def>An excessive reverence for, or fear of, that which is unknown or mysterious.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>An ignorant or irrational worship of the Supreme Deity; excessive exactness or rigor in religious opinions or practice; extreme and unnecessary scruples in the observance of religious rites not commanded, or of points of minor importance; also, a rite or practice proceeding from excess of sculptures in religion.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>And the truth<br/
With <qex>superstitions</qex> and traditions taint.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>The worship of a false god or gods; false religion; religious veneration for objects.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>[The accusers] had certain questions against him of their own <qex>superstition</qex>.</q> <rj><qau>Acts xxv. 19.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>Belief in the direct agency of superior powers in certain extraordinary or singular events, or in magic, omens, prognostics, or the like.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <def>Excessive nicety; scrupulous exactness.</def><br/
[<source>1913 Webster</source>]</p>

<p><syn><b>Syn.</b> -- Fanaticism.</syn>  -- <usage><er>Superstition</er>, <er>Fanaticism</er>. <xex>Superstition</xex> springs from religious feeling misdirected or unenlightened. <xex>Fanaticism</xex> arises from this same feeling in a state of high-wrought and self-confident excitement. The former leads in some cases to excessive rigor in religious opinions or practice; in others, to unfounded belief in extraordinary events or in charms, omens, and prognostics, hence producing weak fears, or excessive scrupulosity as to outward observances. The latter gives rise to an utter disregard of reason under the false assumption of enjoying a guidance directly inspired. <xex>Fanaticism</xex> has a secondary sense as applied to politics, etc., which corresponds to the primary.</usage><br/
[<source>1913 Webster</source>]</p>

<p><ent>Superstitionist</ent><br/
<hw>Su`per*sti"tion*ist</hw>, <pos>n.</pos> <def>One addicted to superstition.</def> <mark>[Obs.]</mark> <ldquo/Blind <xex>superstitionists</xex>.<rdquo/  <rj><au>Dr. H. More.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Superstitious</ent><br/
<hw>Su`per*sti"tious</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[F. <ets>superstitieux</ets>, L. <ets>superstitiosus</ets>.]</ety> <sn>1.</sn> <def>Of or pertaining to superstition; proceeding from, or manifesting, superstition; <as>as, <ex>superstitious</ex> rites; <ex>superstitious</ex> observances</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Evincing superstition; overscrupulous and rigid in religious observances; addicted to superstition; full of idle fancies and scruples in regard to religion.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Ye men of Athens, I perceive that in all things ye are too <qex>superstitious</qex>.</q> <rj><qau>Acts xvii. 22.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>Overexact; scrupulous beyond need.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Superstitious use</b></col> <fld>(Law)</fld>, <cd>the use of a gift or bequest, as of land, etc., for the maintenance of the rites of a religion not tolerated by the law.</cd> <mark>[Eng.]</mark> <au>Mozley & W.</au></cs><br/
[<source>1913 Webster</source>]</p>

<p>-- <wordforms><wf>Su`per*sti"tious*ly</wf>, <pos>adv.</pos> -- <wf>Su`per*sti"tious*ness</wf>, <pos>n.</pos></wordforms><br/
[<source>1913 Webster</source>]</p>

<p><ent>Superstrain</ent><br/
<hw>Su`per*strain"</hw> <pr>(?)</pr>, <pos>v. t.</pos> <def>To overstrain.</def>  <rj><au>Bacon.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Superstratum</ent><br/
<hw>Su`per*stra"tum</hw> <pr>(?)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Superstrata</plw> <pr>(<?/)</pr>.</plu> <ety>[NL.: cf. L. <ets>supersternere</ets>, <ets>superstratum</ets>, to spread upon. See <er>Super-</er>, and <er>Stratum</er>.]</ety> <def>A stratum, or layer, above another.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Superstruct</ent><br/
<hw>Su`per*struct"</hw> <pr>(?)</pr>, <pos>v. t.</pos> <ety>[L. <ets>superstructus</ets>, p. p. of <ets>superstruere</ets> to build upon; <ets>super</ets> over + <ets>struere</ets> to build. See <er>Super-</er>, and <er>Structure</er>.]</ety> <def>To build over or upon another structure; to erect upon a foundation.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>This is the only proper basis on which to <qex>superstruct</qex> first innocency and then virtue.</q> <rj><qau>Dr. H. More.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Superstruction</ent><br/
<hw>Su`per*struc"tion</hw> <pr>(?)</pr>, <pos>n.</pos> <sn>1.</sn> <def>The act of superstructing, or building upon.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>That which is superstructed, or built upon some foundation; an edifice; a superstructure.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>My own profession hath taught me not to erect new <qex>superstructions</qex> upon an old ruin.</q> <rj><qau>Denham.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Superstructive</ent><br/
<hw>Su`per*struct"ive</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Built or erected on something else.</def>  <rj><au>Hammond.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Superstructor</ent><br/
<hw>Su`per*struct"or</hw> <pr>(?)</pr>, <pos>n.</pos> <def>One who builds a superstructure.</def> <mark>[R.]</mark>  <rj><au>R. North.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Superstructure</ent><br/
<hw>Su`per*struc"ture</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Cf. F. <ets>superstructure</ets>.]</ety> <sn>1.</sn> <def>Any material structure or edifice built on something else; that which is raised on a foundation or basis</def>; esp. <fld>(Arch.)</fld>, <def>all that part of a building above the basement. Also used figuratively.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>You have added to your natural endowments the <qex>superstructure</qex> of study.</q> <rj><qau>Dryden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Railway Engin.)</fld> <def>The sleepers, and fastenings, in distinction from the roadbed.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Supersubstantial</ent><br/
<hw>Su`per*sub*stan"tial</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[Pref. <ets>super-</ets> + <ets>substantial</ets>: cf. F. <ets>supersubstantiel</ets>.]</ety> <def>More than substantial; spiritual.</def> <ldquo/The heavenly <xex>supersubstantial</xex> bread.<rdquo/  <rj><au>Jer. Taylor.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Supersubtle</ent><br/
<hw>Su`per*sub"tle</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Too subtle.</def>  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Supersulphate</ent><br/
<hw>Su`per*sul"phate</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Chem.)</fld> <def>An acid sulphate.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Supersulphureted</ent><br/
<hw>Su`per*sul"phu*ret`ed</hw> <pr>(?)</pr>, <pos>a.</pos> <fld>(Chem.)</fld> <def>Supersulphurized.</def> <mark>[Obs.]</mark> <altsp>[Written also <asp>-sulphuretted</asp>.]</altsp><br/
[<source>1913 Webster</source>]</p>

<p><ent>Supersulphurize</ent><br/
<hw>Su`per*sul"phur*ize</hw> <pr>(?)</pr>, <pos>v. t.</pos> <fld>(Chem.)</fld> <def>To impregnate or combine with an excess of sulphur.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Supertax</ent><br/
<hw>Su"per*tax`</hw> <pr>(?)</pr>, <pos>n.</pos>  <ety>[<ets>Super-</ets> + <ets>tax</ets>.]</ety> <def>A tax in addition to the usual or normal tax; specif., in the United Kingdom, an income tax of sixpence for every pound in addition to the normal income tax of one shilling and twopence for every pound, imposed, by the Finance Act of 1909-1910 (c. 8, ss 66, 72), on the amount by which the  income of any person exceeds <pound/3,000 when his total income exceeds <pound/5,000.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Supertemporal</ent><br/
<hw>Su`per*tem"po*ral</hw> <pr>(?)</pr>, <pos>n.</pos> <def>That which is more than temporal; that which is eternal.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Superterranean</ent><br/
<hw>Su`per*ter*ra"ne*an</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Being above ground.</def> <ldquo/<xex>Superterranean</xex> quarries.<rdquo/  <rj><au>Mrs. Trollope.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Superterrene</ent><br/
<hw>Su`per*ter*rene"</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[Pref. <ets>super-</ets> + <ets>terrene</ets>: cf. L. <ets>superterrenus</ets>.]</ety> <def>Being above ground, or above the earth.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Superterrestrial</ent><br/
<hw>Su`per*ter*res"tri*al</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Being above the earth, or above what belongs to the earth.</def>  <rj><au>Buckminster.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Supertonic</ent><br/
<hw>Su`per*ton"ic</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Mus.)</fld> <def>The note next above the keynote; the second of the scale.</def>  <rj><au>Busby.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Supertragical</ent><br/
<hw>Su`per*trag"ic*al</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Tragical to excess.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Supertuberation</ent><br/
<hw>Su`per*tu`ber*a"tion</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Pref. <ets>super-</ets> + <ets>tuber</ets>.]</ety> <fld>(Bot.)</fld> <def>The production of young tubers, as potatoes, from the old while still growing.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Supervacaneous</ent><br/
<hw>Su`per*va*ca"ne*ous</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>supervacaneus</ets>, <ets>supervacuus</ets>; <ets>super</ets> over + <ets>vacuus</ets> empty.]</ety> <def>Serving no purpose; superfluous; needless.</def> <mark>[Obs.]</mark>  <rj><au>Howell.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Supervene</ent><br/
<hw>Su`per*vene"</hw> <pr>(?)</pr>, <pos>v. i.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Supervened</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Supervening</conjf>.]</vmorph> <ety>[L. <ets>supervenire</ets>, <ets>superventum</ets>, to come over, to come upon; <ets>super</ets> over + <ets>venire</ets> to come. See <er>Super-</er>, and <er>Come</er>, and cf. <er>Overcome</er>.]</ety> <def>To come as something additional or extraneous; to occur with reference or relation to something else; to happen upon or after something else; to be added; to take place; to happen.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Such a mutual gravitation can never <qex>supervene</qex> to matter unless impressed by divine power.</q> <rj><qau>Bentley.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>A tyrany immediately <qex>supervened</qex>.</q> <rj><qau>Burke.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Supervenient</ent><br/
<hw>Su`per*ven"ient</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>superveniens</ets>, p. pr.]</ety> <def>Coming as something additional or extraneous; coming afterwards.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>That branch of belief was in him <qex>supervenient</qex> to Christian practice.</q> <rj><qau>Hammond.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Divorces can be granted, <qex>a mensa et toro</qex>, only for <qex>supervenient</qex> causes.</q> <rj><qau>Z. Swift.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Supervention</ent><br/
<hw>Su`per*ven"tion</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>superventio</ets>.]</ety> <def>The act of supervening.</def>  <rj><au>Bp. Hall.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Supervisal</ent><br/
<hw>Su`per*vis"al</hw> <pr>(?)</pr>, <pos>n.</pos> <def>Supervision.</def>  <rj><au>Walpole.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Supervise</ent><br/
<hw>Su`per*vise"</hw> <pr>(?)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Supervised</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Supervising</conjf>.]</vmorph> <ety>[Pref. <ets>super-</ets> + L. <ets>visere</ets> to look at attentively, to view, surely, intens. from <ets>videre</ets>, <ets>visum</ets>, to see.  Cf. <er>Survise</er>, and <er>Survey</er>.]</ety> <sn>1.</sn> <def>To oversee for direction; to superintend; to inspect with authority; <as>as, to <ex>supervise</ex> the construction of a steam engine, or the printing of a book</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To look over so as to read; to peruse.</def> <mark>[Obs.]</mark>  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><syn><b>Syn.</b> -- See <er>Superintend</er>.</syn><br/
[<source>1913 Webster</source>]</p>

<p><ent>Supervise</ent><br/
<hw>Su`per*vise"</hw>, <pos>n.</pos> <def>Supervision; inspection.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Supervision</ent><br/
<hw>Su`per*vi"sion</hw> <pr>(?)</pr>, <pos>n.</pos> <def>The act of overseeing; inspection; superintendence; oversight.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Supervisive</ent><br/
<hw>Su`per*vi"sive</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Supervisory.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Supervisor</ent><br/
<hw>Su`per*vis"or</hw> <pr>(?)</pr>, <pos>n.</pos> <sn>1.</sn> <def>One who supervises; an overseer; an inspector; a superintendent; <as>as, a <ex>supervisor</ex> of schools</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A spectator; a looker-on.</def> <mark>[Obs.]</mark>  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Supervisory</ent><br/
<hw>Su`per*vi"so*ry</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Of or pertaining to supervision; <as>as, <ex>supervisory</ex> powers</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Supervive</ent><br/
<hw>Su`per*vive"</hw> <pr>(?)</pr>, <pos>v. t.</pos> <ety>[L. <ets>supervivere</ets>. See <er>Survive</er>.]</ety> <def>To survive; to outlive.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Supervolute</ent><br/
<hw>Su`per*vo*lute"</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>supervolutus</ets>, p. p. of <ets>supervolvere</ets> to roll over; <ets>super</ets> over + <ets>volvere</ets> to roll.]</ety> <fld>(Bot.)</fld> <def>Having a plaited and convolute arrangement in the bud, as in the morning-glory.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Supination</ent><br/
<hw>Su`pi*na"tion</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>supinare</ets>, <ets>supinatum</ets>, to bend or lay backward, fr. <ets>supinus</ets> supine: cf. F. <ets>supination</ets>. See <er>Supine</er>.]</ety> <fld>(Physiol.)</fld> <sd>(a)</sd> <def>The act of turning the hand palm upward; also, position of the hand with the palm upward.</def> <sd>(b)</sd> <def>The act or state of lying with the face upward. Opposed to <ant>pronation</ant>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Supinator</ent><br/
<hw>Su`pi*na"tor</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[NL.]</ety> <fld>(Anat.)</fld> <def>A muscle which produces the motion of supination.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Supine</ent><br/
<hw>Su*pine"</hw> <pr>(s<usl/*p<imac/n")</pr>, <pos>a.</pos> <ety>[L. <ets>supinus</ets>, akin to <ets>sub</ets> under, <ets>super</ets> above.  Cf. <er>Sub-</er>, <er>Super-</er>.]</ety> <sn>1.</sn> <def>Lying on the back, or with the face upward; -- opposed to <ant>prone</ant>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Leaning backward, or inclining with exposure to the sun; sloping; inclined.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>If the vine<br/
On rising ground be placed, or hills <qex>supine</qex>.</q> <rj><qau>Dryden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>Negligent; heedless; indolent; listless.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>He became pusillanimous and <qex>supine</qex>, and openly exposed to any temptation.</q> <rj><qau>Woodward.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><syn><b>Syn.</b> -- Negligent; heedless; indolent; thoughtless; inattentive; listless; careless; drowsy.</syn><br/
[<source>1913 Webster</source>]</p>

<p>-- <wordforms><wf>Su*pine"ly</wf>, <pos>adv.</pos> -- <wf>Su*pine"ness</wf>, <pos>n.</pos></wordforms><br/
[<source>1913 Webster</source>]</p>

<p><ent>Supine</ent><br/
<hw>Su"pine</hw> <pr>(s<umac/"p<imac/n)</pr>, <pos>n.</pos> <ety>[L. <ets>supinum</ets> (sc. <ets>verbum</ets>), from <ets>supinus</ets> bent or thrown backward, perhaps so called because, although furnished with substantive case endings, it rests or falls back, as it were, on the verb: cf. F. <ets>supin</ets>.]</ety> <fld>(Lat. Gram.)</fld> <def>A verbal noun; or (according to C.F.Becker), a case of the infinitive mood ending in <xex>-um</xex> and <xex>-u</xex>, that in <xex>-um</xex> being sometimes called the <xex>former supine</xex>, and that in <xex>-u</xex> the <xex>latter supine</xex>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Supinity</ent><br/
<hw>Su*pin"i*ty</hw> <pr>(s<usl/*p<icr/n"<icr/*t<ycr/)</pr>, <pos>n.</pos> <ety>[L. <ets>supinitas</ets>.]</ety> <def>Supineness.</def> <mark>[Obs.]</mark>  <rj><au>Sir T. Browne.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Suppage</ent><br/
<hw>Sup"page</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[From <er>Sup</er>.]</ety> <def>What may be supped; pottage.</def> <mark>[Obs.]</mark>  <rj><au>Hooker.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Suppalpation</ent><br/
<hw>Sup`pal*pa"tion</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>suppalpari</ets> to caress a little; <ets>sub</ets> under, a little + <ets>palpare</ets> to caress.]</ety> <def>The act of enticing by soft words; enticement.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Supparasitation</ent><br/
<hw>Sup*par`a*si*ta"tion</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[See <er>Supparasite</er>.]</ety> <def>The act of flattering to gain favor; servile approbation.</def> <mark>[Obs.]</mark>  <rj><au>Bp. Hall.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Supparasite</ent><br/
<hw>Sup*par"a*site</hw> <pr>(?)</pr>, <pos>v. t.</pos> <ety>[L. <ets>supparasitari</ets>; <ets>sub</ets> under, a little + <ets>parasitus</ets> a parasite.]</ety> <def>To flatter; to cajole; to act the parasite.</def> <mark>[Obs.]</mark>  <rj><au>Dr. R. Clerke.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Suppawn</ent><br/
<hw>Sup*pawn"</hw> <pr>(?)</pr>, <pos>n.</pos> <def>See <er>Supawn</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Suppedaneous</ent><br/
<hw>Sup`pe*da"ne*ous</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[Pref. <ets>sub-</ets> + L. <ets>pes</ets>, <ets>pedis</ets>, a foot: cf. L. <ets>suppedaneum</ets> a footstool.]</ety> <def>Being under the feet.</def> <mark>[Obs.]</mark>  <rj><au>Sir T. Browne.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Suppeditate</ent><br/
<hw>Sup*ped"i*tate</hw> <pr>(?)</pr>, <pos>v. t.</pos> <ety>[L. <ets>suppeditatus</ets>, p. p. of <ets>suppeditare</ets> to supply.]</ety> <def>To supply; to furnish.</def> <mark>[Obs.]</mark>  <rj><au>Hammond.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Suppeditation</ent><br/
<hw>Sup*ped`i*ta"tion</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>suppeditatio</ets>.]</ety> <def>Supply; aid afforded.</def> <mark>[Obs.]</mark>  <rj><au>Bacon.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Supper</ent><br/
<hw>Sup"per</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[OE. <ets>soper</ets>, <ets>super</ets>, OF. <ets>super</ets>, <ets>soper</ets>, F. <ets>souper</ets>; originally an infinitive, to sup, take a meal. See <er>Soup</er>, and cf. <er>Sup</er> to take supper.]</ety> <def>A meal taken at the close of the day; the evening meal.</def><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ <xex>Supper</xex> is much used in an obvious sense, either adjectively or as the first part of a compound; as, <xex>supper</xex> time or <xex>supper</xex>-time, <xex>supper</xex> bell, <xex>supper</xex> hour, etc.</note><br/
[<source>1913 Webster</source>]</p>

<p><ent>Supper</ent><br/
<hw>Sup"per</hw>, <pos>v. i.</pos> <def>To take supper; to sup.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Supper</ent><br/
<hw>Sup"per</hw>, <pos>v. t.</pos> <def>To supply with supper.</def> <mark>[R.]</mark> <ldquo/Kester was <xex>suppering</xex> the horses.<rdquo/  <rj><au>Mrs. Gaskell.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Supperless</ent><br/
<hw>Sup"per*less</hw>, <pos>a.</pos> <def>Having no supper; deprived of supper; <as>as, to go <ex>supperless</ex> to bed</as>.</def>  <rj><au>Beau. & Fl.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Supping</ent><br/
<hw>Sup"ping</hw> <pr>(?)</pr>, <pos>n.</pos> <sn>1.</sn> <def>The act of one who sups; the act of taking supper.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>That which is supped; broth.</def> <mark>[Obs.]</mark>  <rj><au>Holland.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Supplace</ent><br/
<hw>Sup*place"</hw> <pr>(?)</pr>, <pos>v. t.</pos> <def>To replace.</def> <mark>[R.]</mark>  <rj><au>J. Bascom.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Supplant</ent><br/
<hw>Sup*plant"</hw> <pr>(?)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Supplanted</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Supplanting</conjf>.]</vmorph> <ety>[F. <ets>supplanter</ets>, L. <ets>supplantare</ets> to trip up one's heels, to throw down; <ets>sub</ets> under + <ets>planta</ets> the sole of the foot, also, a sucker, slip, sprout.  Cf. <er>Plant</er>, <pos>n.</pos>]</ety> <sn>1.</sn> <def>To trip up.</def> <mark>[Obs.]</mark> <ldquo/<xex>Supplanted</xex>, down he fell.<rdquo/  <rj><au>Milton.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To displace and take the place of; to supersede; to remove or displace by stratagem; <as>as, a rival <ex>supplants</ex> another in the favor of a mistress or a prince</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Suspecting that the courtier had <qex>supplanted</qex> the friend.</q> <rj><qau>Bp. Fell.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>To overthrow, undermine, or force away, in order to get a substitute in place of.</def> <mark>[obsolescent]</mark> <br/
[<source>1913 Webster</source>]</p>

<p><q>You never will <qex>supplant</qex> the received ideas of God.</q> <rj><qau>Landor.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>To remove (a thing) and replace it with something else.</def><br/
[<source>PJC</source>]</p>

<p><syn><b>Syn.</b> -- To remove; displace; overpower; undermine; overthrow; supersede.</syn><br/
[<source>1913 Webster</source>]</p>

<p><-- p. 1449 --></p>

<p><ent>Supplantation</ent><br/
<hw>Sup`plan*ta"tion</hw> <pr>(s<ucr/p`pl<acr/n*t<amac/"sh<ucr/n)</pr>, <pos>n.</pos> <ety>[Cf. F. <ets>supplantation</ets>, L. <ets>supplantatio</ets> hypocritical deceit.]</ety> <def>The act of supplanting or displacing.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Habitual <qex>supplantation</qex> of immediate selfishness.</q> <rj><qau>Cloeridge.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Supplanter</ent><br/
<hw>Sup*plan"ter</hw> <pr>(s<ucr/p*pl<acr/nt"<etil/r)</pr>, <pos>n.</pos> <def>One who supplants.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Supple</ent><br/
<hw>Sup"ple</hw> <pr>(s<ucr/p"p'l)</pr>, <pos>a.</pos> <ety>[OE. <ets>souple</ets>, F. <ets>souple</ets>, from L. <ets>supplex</ets> suppliant, perhaps originally, being the knees.  Cf. <er>Supplicate</er>.]</ety> <sn>1.</sn> <def>Pliant; flexible; easily bent; <as>as, <ex>supple</ex> joints; <ex>supple</ex> fingers</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Yielding; compliant; not obstinate; submissive to guidance; <as>as, a <ex>supple</ex> horse</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>If punishment . . . makes not the will <qex>supple</qex>, it hardens the offender.</q> <rj><qau>Locke.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>Bending to the humor of others; flattering; fawning; obsequious.</def>  <rj><au>Addison.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><syn><b>Syn.</b> -- Pliant; flexible; yielding; compliant; bending; flattering; fawning; soft.</syn><br/
[<source>1913 Webster</source>]</p>

<p><ent>Supple</ent><br/
<hw>Sup"ple</hw>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Suppled</conjf> <pr>(s<ucr/p"p'ld)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Suppling</conjf> <pr>(s<ucr/p"pl<icr/ng)</pr>.]</vmorph> <sn>1.</sn> <def>To make soft and pliant; to render flexible; <as>as, to <ex>supple</ex> leather</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The flesh therewith she <qex>suppled</qex> and did steep.</q> <rj><qau>Spenser.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To make compliant, submissive, or obedient.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>A mother persisting till she had bent her daughter's mind and <qex>suppled</qex> her will.</q> <rj><qau>Locke.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>They should <qex>supple</qex> our stiff willfulness.</q> <rj><qau>Barrow.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Supple</ent><br/
<hw>Sup"ple</hw>, <pos>v. i.</pos> <def>To become soft and pliant.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The stones . . . <br/
<qex>Suppled</qex> into softness as they fell.</q> <rj><qau>Dryden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Supple-chapped</ent><br/
<hw>Sup"ple-chapped`</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Having a limber tongue.</def> <mark>[R.]</mark> <ldquo/A <xex>supple-chapped</xex> flatterer.<rdquo/  <rj><au>Marston.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Supple-jack</ent><br/
<hw>Sup"ple-jack`</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Bot.)</fld> <sd>(a)</sd> <def>A climbing shrub (<spn>Berchemia volubilus</spn>) of the Southern United States, having a tough and pliable stem.</def> <sd>(b)</sd> <def>A somewhat similar tropical American plant (<spn>Paullinia Curassavica</spn>); also, a walking stick made from its stem.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>He was in form and spirit like a <qex>supple-jack</qex>, . . . yielding, but tough; though he bent, he never broke.</q> <rj><qau>W. Irving.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ This name is given to various plants of similar habit in different British colonies.</note><br/
[<source>1913 Webster</source>]</p>

<p><ent>Supplely</ent><br/
<hw>Sup"ple*ly</hw>, <pos>adv.</pos> <def>In a supple manner; softly; pliantly; mildly.</def>  <rj><au>Cotgrave.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Supplement</ent><br/
<hw>Sup"ple*ment</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F. <ets>suppl<eacute/ment</ets>, L. <ets>supplementum</ets>, fr. <ets>supplere</ets> to fill up. See <er>Supply</er>, <pos>v. t.</pos>]</ety> <sn>1.</sn> <def>That which supplies a deficiency, or meets a want; a store; a supply.</def> <mark>[Obs.]</mark>  <rj><au>Chapman.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>That which fills up, completes, or makes an addition to, something already organized, arranged, or set apart; specifically, a part added to, or issued as a continuation of, a book or paper, to make good its deficiencies or correct its errors.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <fld>(Trig.)</fld> <def>The number of degrees which, if added to a specified arc, make it 180<deg/; the quantity by which an arc or an angle falls short of 180 degrees, or an arc falls short of a semicircle.</def><br/
[<source>1913 Webster</source>]</p>

<p><syn><b>Syn.</b> -- Appendix.</syn>  -- <usage><er>Appendix</er>, <er>Supplement</er>. An <xex>appendix</xex> is that which is appended to something, but is not essential to its completeness; a <xex>supplement</xex> is that which supplements, or serves to complete or make perfect, that to which it is added.</usage><br/
[<source>1913 Webster</source>]</p>

<p><ent>Supplement</ent><br/
<hw>Sup"ple*ment</hw> <pr>(?)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Supplemented</conjf>; <pos>p. pr. & vb. n.</pos> <conjf>Supplementing</conjf>.]</vmorph> <def>To fill up or supply by addition; to add something to.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Causes of one kind must be <qex>supplemented</qex> by bringing to bear upon them a causation of another kind.</q> <rj><qau>I. Taylor.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Supplementary</ent><br/
<ent>Supplemental</ent><br/
<mhw>{ <hw>Sup`ple*men"tal</hw> <pr>(?)</pr>, <hw>Sup`ple*men"ta*ry</hw> <pr>(?)</pr>, }</mhw> <pos>a.</pos> <ety>[Cf. F. <ets>suppl<eacute/mentaire</ets>.]</ety> <def>Added to supply what is wanted; additional; being, or serving as, a supplement; <as>as, a <ex>supplemental</ex> law; a <ex>supplementary</ex> sheet or volume</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Supplemental air</b></col> <fld>(Physiol.)</fld>, <cd>the air which in addition to the residual air remains in the lungs after ordinary expiration, but which, unlike the residual air, can be expelled; reserve air.</cd> -- <col><b>Supplemental bill</b></col> <fld>(Equity)</fld>, <cd>a bill filed in aid of an original bill to supply some deffect in the latter, or to set forth new facts which can not be done by amendment.</cd> <au>Burrill.</au> <au>Daniel.</au> -- <col><b>Supplementary chords</b></col> <fld>(Math.)</fld>, <cd>in an ellipse or hyperbola, any two chords drawn through the extremities of a diameter, and intersecting on the curve.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Supplementation</ent><br/
<hw>Sup`ple*men*ta"tion</hw> <pr>(?)</pr>, <pos>n.</pos> <def>The act of supplementing.</def>  <rj><au>C. Kingsley.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Suppleness</ent><br/
<hw>Sup"ple*ness</hw> <pr>(?)</pr>, <pos>n.</pos> <def>The quality or state of being supple; flexibility; pliableness; pliancy.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Suppletory</ent><br/
<ent>Suppletive</ent><br/
<mhw>{ <hw>Sup"ple*tive</hw> <pr>(?)</pr>, <hw>Sup"ple*to*ry</hw> <pr>(?)</pr>, }</mhw> <pos>a.</pos> <ety>[Cf. F. <ets>suppl<eacute/tif</ets>, LL. <ets>suppletivus</ets>, from L. <ets>supplere</ets>, <ets>suppletum</ets>, to fill up. See <er>Supply</er>.]</ety> <def>Supplying deficiencies; supplementary; <as>as, a <ex>suppletory</ex> oath</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Suppletory</ent><br/
<hw>Sup"ple*to*ry</hw>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Suppletories</plw> <pr>(<?/)</pr>.</plu> <def>That which is to supply what is wanted.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Invent <qex>suppletories</qex> to excuse an evil man.</q> <rj><qau>Jer. Taylor.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Supplial</ent><br/
<hw>Sup*pli"al</hw> <pr>(?)</pr>, <pos>n.</pos> <def>The act of supplying; a supply.</def> <ldquo/The <xex>supplial</xex> of a preposition.<rdquo/  <rj><au>Fitzed. Hall.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Suppliance</ent><br/
<hw>Sup*pli"ance</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[From <er>Supply</er>.]</ety> <def>That which supplies a want; assistance; a gratification; satisfaction.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>The perfume and <qex>suppliance</qex> of a minute.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Suppliance</ent><br/
<hw>Sup*pli"ance</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[See <er>Suppliant</er>.]</ety> <def>Supplication; entreaty.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>When Greece her knee in <qex>suppliance</qex> bent.</q> <rj><qau>Halleck.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Suppliant</ent><br/
<hw>Sup"pli*ant</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[F., p. pr. of <ets>supplier</ets> to entreat, L. <ets>supplicare</ets>. See <er>Supplicate</er>, and cf. <er>Supplicant</er>.]</ety> <sn>1.</sn> <def>Asking earnestly and submissively; entreating; beseeching; supplicating.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The rich grow <qex>suppliant</qex>, and the poor grow proud.</q> <rj><qau>Dryden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Manifesting entreaty; expressive of supplication.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>To bow and sue for grace<br/
With <qex>suppliant</qex> knee.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><syn><b>Syn.</b> -- Entreating; beseeching; suing; begging; supplicating; imploring.</syn><br/
[<source>1913 Webster</source>]</p>

<p>-- <wordforms><wf>Sup"pli*ant*ly</wf>, <pos>adv.</pos> -- <wf>Sup"pli*ant*ness</wf>, <pos>n.</pos></wordforms><br/
[<source>1913 Webster</source>]</p>

<p><ent>Suppliant</ent><br/
<hw>Sup"pli*ant</hw>, <pos>n.</pos> <def>One who supplicates; a humble petitioner; one who entreats submissively.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Hear thy <qex>suppliant's</qex> prayer.</q> <rj><qau>Dryden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Supplicancy</ent><br/
<hw>Sup"pli*can*cy</hw> <pr>(?)</pr>, <pos>n.</pos> <def>Supplication.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Supplicant</ent><br/
<hw>Sup"pli*cant</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>supplicans</ets>, p. pr. See <er>Supplicate</er>, and cf. <er>Suppliant</er>.]</ety> <def>Entreating; asking submissively.</def> <au>Shak.</au> -- <wordforms><wf>Sup"pli*cant*ly</wf>, <pos>adv.</pos></wordforms><br/
[<source>1913 Webster</source>]</p>

<p><ent>Supplicant</ent><br/
<hw>Sup"pli*cant</hw>, <pos>n.</pos> <def>One who supplicates; a suppliant.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The wise <qex>supplicant</qex> . . . left the event to God.</q> <rj><qau>Rogers.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Supplicat</ent><br/
||<hw>Sup"pli*cat</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L., he supplicates.]</ety> <fld>(Eng. Universities)</fld> <def>A petition; esp., a written one, with a certificate that the conditions have been complied with.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Supplicate</ent><br/
<hw>Sup"pli*cate</hw> <pr>(?)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Supplicated</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Supplicating</conjf>.]</vmorph> <ety>[L. <ets>supplicatus</ets>, p. p. of <ets>supplicare</ets> to supplicate; of uncertain origin, cf. <ets>supplex</ets>, <ets>supplicis</ets>, humbly begging or entreating; perhaps fr. <ets>sub</ets> under + a word akin to <ets>placare</ets> to reconcile, appease (cf. <er>Placable</er>), or fr. <ets>sub</ets> under + <ets>plicare</ets> to fold, whence the idea of bending the knees (cf. <er>Ply</er>, <pos>v. t.</pos>).  Cf. <er>Supple</er>.]</ety> <sn>1.</sn> <def>To entreat for; to seek by earnest prayer; to ask for earnestly and humbly; <as>as, to <ex>supplicate</ex> blessings on Christian efforts to spread the gospel</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To address in prayer; to entreat as a supplicant; <as>as, to <ex>supplicate</ex> the Deity</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><syn><b>Syn.</b> -- To beseech; entreat; beg; petition; implore; importune; solicit; crave. See <er>Beseech</er>.</syn><br/
[<source>1913 Webster</source>]</p>

<p><ent>Supplicate</ent><br/
<hw>Sup"pli*cate</hw>, <pos>v. i.</pos> <def>To make petition with earnestness and submission; to implore.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>A man can not brook to <qex>supplicate</qex> or beg.</q> <rj><qau>Bacon.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Supplicatingly</ent><br/
<hw>Sup"pli*ca`ting*ly</hw>, <pos>adv.</pos> <def>In a supplicating manner.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Supplication</ent><br/
<hw>Sup`pli*ca"tion</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F. <ets>supplication</ets>, L.  <ets>supplicatio</ets>.]</ety> <sn>1.</sn> <def>The act of supplicating; humble and earnest prayer, as in worship.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A humble petition; an earnest request; an entreaty.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <fld>(Rom. Antiq.)</fld> <def>A religious solemnity observed in consequence of some military success, and also, in times of distress and danger, to avert the anger of the gods.</def><br/
[<source>1913 Webster</source>]</p>

<p><syn><b>Syn.</b> -- Entreaty; petition; solicitation; craving.</syn><br/
[<source>1913 Webster</source>]</p>

<p><ent>Supplicator</ent><br/
<hw>Sup"pli*ca`tor</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L.]</ety> <def>One who supplicates; a supplicant.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Supplicatory</ent><br/
<hw>Sup"pli*ca*to*ry</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[Cf. F. <ets>supplicatoire</ets>.]</ety> <def>Containing supplication; humble; earnest.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Supplier</ent><br/
<hw>Sup*pli"er</hw> <pr>(?)</pr>, <pos>n.</pos> <def>One who supplies.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Supply</ent><br/
<hw>Sup*ply"</hw> <pr>(?)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Supplied</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Supplying</conjf> <pr>(?)</pr>.]</vmorph> <ety>[For older <ets>supploy</ets>, F. <ets>suppl<eacute/er</ets>, OF. also <ets>supployer</ets>, (assumed) LL. <ets>suppletare</ets>, from L. <ets>supplere</ets>, <ets>suppletum</ets>; <ets>sub</ets> under + <ets>plere</ets> to fill, akin to <ets>plenus</ets> full. See <er>Plenty</er>.]</ety> <sn>1.</sn> <def>To fill up, or keep full; to furnish with what is wanted; to afford, or furnish with, a sufficiency; <as>as, rivers are <ex>supplied</ex> by smaller streams; an aqueduct <ex>supplies</ex> an artificial lake</as>; -- often followed by <xex>with</xex> before the thing furnished; <as>as, to <ex>supply</ex> a furnace with fuel; to <ex>supply</ex> soldiers with ammunition</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To serve instead of; to take the place of.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Burning ships the banished sun <qex>supply</qex>.</q> <rj><qau>Waller.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>The sun was set, and Vesper, to <qex>supply</qex><br/
His absent beams, had lighted up the sky.</q> <rj><qau>Dryden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>To fill temporarily; to serve as substitute for another in, as a vacant place or office; to occupy; to have possession of; <as>as, to <ex>supply</ex> a pulpit</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>To give; to bring or furnish; to provide; <as>as, to <ex>supply</ex> money for the war</as>.</def>  <rj><au>Prior.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><syn><b>Syn.</b> -- To furnish; provide; administer; minister; contribute; yield; accommodate.</syn><br/
[<source>1913 Webster</source>]</p>

<p><ent>Supply</ent><br/
<hw>Sup*ply"</hw>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Supplies</plw> <pr>(<?/)</pr>.</plu> <sn>1.</sn> <def>The act of supplying; supplial.</def>  <rj><au>A. Tucker.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>That which supplies a want; sufficiency of things for use or want.</def> Specifically: --<br/
[<source>1913 Webster</source>]</p>

<p><sd>(a)</sd> <def>Auxiliary troops or reenforcements.</def> <ldquo/My promised <xex>supply</xex> of horsemen.<rdquo/  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sd>(b)</sd> <def>The food, and the like, which meets the daily necessities of an army or other large body of men; store; -- used chiefly in the plural; <as>as, the army was discontented for lack of <ex>supplies</ex></as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sd>(c)</sd> <def>An amount of money provided, as by Parliament or Congress, to meet the annual national expenditures; generally in the plural; <as>as, to vote <ex>supplies</ex></as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sd>(d)</sd> <def>A person who fills a place for a time; one who supplies the place of another; a substitute; esp., a clergyman who supplies a vacant pulpit.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Stated supply</b></col> <fld>(Eccl.)</fld>, <cd>a clergyman employed to supply a pulpit for a definite time, but not settled as a pastor.</cd> <mark>[U.S.]</mark> -- <col><b>Supply and demand</b></col>. <fld>(Polit. Econ.)</fld> <cd><ldquo/<xex>Demand</xex> means the quantity of a given article which would be taken at a given price. <xex>Supply</xex> means the quantity of that article which could be had at that price.<rdquo/ <au>F. A. Walker.</au></cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Supply</ent><br/
<hw>Sup*ply"</hw>, <pos>a.</pos> <def>Serving to contain, deliver, or regulate a supply of anything; <as>as, a <ex>supply</ex> tank or valve</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Supply system</b></col> <fld>(Zool.)</fld>, <cd>the system of tubes and canals in sponges by means of which food and water are absorbed. See <xex>Illust.</xex> of <er>Spongiae</er>.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Supplyant</ent><br/
<hw>Sup*ply"ant</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Supplying or aiding; auxiliary; suppletory.</def> <mark>[Obs.]</mark>  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Supplyment</ent><br/
<hw>Sup*ply"ment</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A supplying or furnishing; supply.</def> <mark>[Obs.]</mark>  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Support</ent><br/
<hw>Sup*port"</hw> <pr>(?)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Supported</conjf>; <pos>p. pr. & vb. n.</pos> <conjf>Supporting</conjf>.]</vmorph> <ety>[F. <ets>supporter</ets>, L. <ets>supportare</ets> to carry on, to convey, in LL., to support, sustain; <ets>sub</ets> under + <ets>portare</ets> to carry. See <er>Port</er> demeanor.]</ety> <sn>1.</sn> <def>To bear by being under; to keep from falling; to uphold; to sustain, in a literal or physical sense; to prop up; to bear the weight of; <as>as, a pillar <ex>supports</ex> a structure; an abutment <ex>supports</ex> an arch; the trunk of a tree <ex>supports</ex> the branches.</as></def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To endure without being overcome, exhausted, or changed in character; to sustain; <as>as, to <ex>support</ex> pain, distress, or misfortunes</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>This fierce demeanor and his insolence<br/
The patience of a god could not <qex>support</qex>.</q> <rj><qau>Dryden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>To keep from failing or sinking; to solace under affictive circumstances; to assist; to encourage; to defend; <as>as, to <ex>support</ex> the courage or spirits</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>To assume and carry successfully, as the part of an actor; to represent or act; to sustain; <as>as, to <ex>support</ex> the character of King Lear</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <def>To furnish with the means of sustenance or livelihood; to maintain; to provide for; <as>as, to <ex>support</ex> a family; to <ex>support</ex> the ministers of the gospel.</as></def><br/
[<source>1913 Webster</source>]</p>

<p><sn>6.</sn> <def>To carry on; to enable to continue; to maintain; <as>as, to <ex>support</ex> a war or a contest; to <ex>support</ex> an argument or a debate</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>7.</sn> <def>To verify; to make good; to substantiate; to establish; to sustain; <as>as, the testimony is not sufficient to <ex>support</ex> the charges; the evidence will not <ex>support</ex> the statements or allegations.</as></def><br/
[<source>1913 Webster</source>]</p>

<p><q>To urge such arguments, as though they were sufficient to <qex>support</qex> and demonstrate a whole scheme of moral philosophy.</q> <rj><qau>J. Edwards.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>8.</sn> <def>To vindicate; to maintain; to defend successfully; <as>as, to be able to <ex>support</ex> one's own cause</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>9.</sn> <def>To uphold by aid or countenance; to aid; to help; to back up; <as>as, to <ex>support</ex> a friend or a party; to <ex>support</ex> the present administration.</as></def><br/
[<source>1913 Webster</source>]</p>

<p><q>Wherefore, bold pleasant,<br/
Darest thou <qex>support</qex> a published traitor?</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>10.</sn> <def>A attend as an honorary assistant; <as>as, a chairman <ex>supported</ex> by a vice chairman; O'Connell left the prison, <ex>supported</ex> by his two sons.</as></def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Support arms</b></col> <fld>(Mil.)</fld>, <cd>a command in the manual of arms in responce to which the piece is held vertically at the shoulder, with the hammer resting on the left forearm, which is passed horizontally across the body in front; also, the position assumed in response to this command.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><syn><b>Syn.</b> -- To maintain; endure; verify; substantiate; countenance; patronize; help; back; second; succor; relieve; uphold; encourage; favor; nurture; nourish; cherish; shield; defend; protect; stay; assist; forward.</syn><br/
[<source>1913 Webster</source>]</p>

<p><ent>Support</ent><br/
<hw>Sup*port"</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F.]</ety> <sn>1.</sn> <def>The act, state, or operation of supporting, upholding, or sustaining.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>That which upholds, sustains, or keeps from falling, as a prop, a pillar, or a foundation of any kind.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>That which maintains or preserves from being overcome, falling, yielding, sinking, giving way, or the like; subsistence; maintenance; assistance; reenforcement; <as>as, he gave his family a good <ex>support</ex>, the <ex>support</ex> of national credit; the assaulting column had the <ex>support</ex> of a battery.</as></def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Points of support</b></col> <fld>(Arch.)</fld>, <cd>the horizontal area of the solids of a building, walls, piers, and the like, as compared with the open or vacant spaces.</cd> -- <col><b>Right of support</b></col> <fld>(Law)</fld>, <cd>an easement or servitude by which the owner of a house has a right to rest his timber on the walls of his neighbor's house.</cd> <au>Kent.</au></cs><br/
[<source>1913 Webster</source>]</p>

<p><syn><b>Syn.</b> -- Stay; prop; maintenance; subsistence; assistance; favor; countenance; encouragement; patronage; aid; help; succor; nutriment; sustenance; food.</syn><br/
[<source>1913 Webster</source>]</p>

<p><ent>Supportable</ent><br/
<hw>Sup*port"a*ble</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[Cf. F. <ets>supportable</ets>.]</ety> <def>Capable of being supported, maintained, or endured; endurable.</def> -- <wordforms><wf>Sup*port"a*ble*ness</wf>, <pos>n.</pos> -- <wf>Sup*port"a*bly</wf>, <pos>adv.</pos></wordforms><br/
[<source>1913 Webster</source>]</p>

<p><ent>Supportance</ent><br/
<hw>Sup*port"ance</hw> <pr>(?)</pr>, <pos>n.</pos> <def>Support.</def> <mark>[Obs.]</mark>  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Supportation</ent><br/
<hw>Sup`por*ta"tion</hw> <pr>(?)</pr>, <pos>n.</pos> <def>Maintenance; support.</def> <mark>[Obs.]</mark>  <rj><au>Chaucer. Bacon.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Supporter</ent><br/
<hw>Sup*port"er</hw> <pr>(?)</pr>, <pos>n.</pos> <sn>1.</sn> <def>One who, or that which, supports; <as>as, oxygen is a <ex>supporter</ex> of life</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The sockets and <qex>supporters</qex> of flowers are figured.</q> <rj><qau>Bacon.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>The saints have a . . . <qex>supporter</qex> in all their miseries.</q> <rj><qau>South.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Especially, an adherent; one who sustains, advocates, and defends; <as>as, the <ex>supporter</ex> of a party, faction, or candidate</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <fld>(Shipbuilding)</fld> <def>A knee placed under the cathead.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <fld>(Her.)</fld> <def>A figure, sometimes of a man, but commonly of some animal, placed on either side of an escutcheon, and exterior to it. Usually, both supporters of an escutcheon are similar figures.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <fld>(Med.)</fld> <def>A broad band or truss for supporting the abdomen or some other part or organ.</def><br/
[<source>1913 Webster</source>]</p>

<p><-- athletic supporter.  A supprter (Def. 5), specifically to support the male testicles while engaged in vigorous athletics.  Also called a jock or a jockstrap. --><br/
[<source>1913 Webster</source>]</p>

<p><ent>Supportful</ent><br/
<hw>Sup*port"ful</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Abounding with support.</def> <mark>[Obs.]</mark>  <rj><au>Chapman.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Supportless</ent><br/
<hw>Sup*port"less</hw>, <pos>a.</pos> <def>Having no support.</def>  <rj><au>Milton.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Supportment</ent><br/
<hw>Sup*port"ment</hw> <pr>(?)</pr>, <pos>n.</pos> <def>Support.</def> <mark>[Obs.]</mark>  <rj><au>Sir H. Wotton.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Supportress</ent><br/
<hw>Sup*port"ress</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A female supporter.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>You are my gracious patroness and <qex>supportress</qex>.</q> <rj><qau>Massinger.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Supposable</ent><br/
<hw>Sup*pos"a*ble</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Capable of being supposed, or imagined to exist; <as>as, that is not a <ex>supposable</ex> case</as>.</def> -- <wordforms><wf>Sup*pos"a*ble*ness</wf>, <pos>n.</pos> -- <wf>Sup*pos"a*bly</wf>, <pos>adv.</pos></wordforms><br/
[<source>1913 Webster</source>]</p>

<p><ent>Supposal</ent><br/
<hw>Sup*pos"al</hw> <pr>(?)</pr>, <pos>n.</pos> <def>The act of supposing; also, that which is supposed; supposition; opinion.</def>  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Interest, with a Jew, never proceeds but upon <qex>supposal</qex>, at least, of a firm and sufficient bottom.</q> <rj><qau>South.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Suppose</ent><br/
<hw>Sup*pose"</hw> <pr>(?)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Supposed</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Supposing</conjf>.]</vmorph> <ety>[F. <ets>supposer</ets>; pref. <ets>sub-</ets> under + <ets>poser</ets> to place; -- corresponding in meaning to L. <ets>supponere</ets>, <ets>suppositum</ets>, to put under, to substitute, falsify, counterfeit. See <er>Pose</er>.]</ety> <sn>1.</sn> <def>To represent to one's self, or state to another, not as true or real, but as if so, and with a view to some consequence or application which the reality would involve or admit of; to imagine or admit to exist, for the sake of argument or illustration; to assume to be true; as, let us <xex>suppose</xex> the earth to be the center of the system, what would be the result?</def><br/
[<source>1913 Webster</source>]</p>

<p><q><qex>Suppose</qex> they take offence without a cause.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>When we have as great assurance that a thing is, as we could possibly, <qex>supposing</qex> it were, we ought not to make any doubt of its existence.</q> <rj><qau>Tillotson.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To imagine; to believe; to receive as true.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>How easy is a bush <qex>supposed</qex> a bear!</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Let not my lord <qex>suppose</qex> that they have slain all the young men, the king's sons; for Amnon only is dead.</q> <rj><qau>2 Sam. xiii. 32.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>To require to exist or to be true; to imply by the laws of thought or of nature; <as>as, purpose <ex>supposes</ex> foresight</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>One falsehood always <qex>supposes</qex> another, and renders all you can say suspected.</q> <rj><qau>Female Quixote.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>To put by fraud in the place of another.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><syn><b>Syn.</b> -- To imagine; believe; conclude; judge; consider; view; regard; conjecture; assume.</syn><br/
[<source>1913 Webster</source>]</p>

<p><-- p. 1450 --></p>

<p><ent>Suppose</ent><br/
<hw>Sup*pose"</hw> <pr>(?)</pr>, <pos>v. i.</pos> <def>To make supposition; to think; to be of opinion.</def>  <rj><au>Acts ii. 15.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Suppose</ent><br/
<hw>Sup*pose"</hw>, <pos>n.</pos> <def>Supposition.</def> <mark>[Obs.]</mark> <au>Shak.</au> <ldquo/A base <xex>suppose</xex> that he is honest.<rdquo/  <rj><au>Dryden.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Supposeer</ent><br/
<hw>Sup*pose"er</hw> <pr>(?)</pr>, <pos>n.</pos> <def>One who supposes.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Supposition</ent><br/
<hw>Sup`po*si"tion</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F. <ets>supposition</ets>, L. <ets>suppositio</ets> a placing under, a substitution, fr. <ets>supponere</ets>, <ets>suppositium</ets>, to put under, to substitute. The word has the meaning corresponding to <ets>suppose</ets>. See <er>Sub-</er>, and <er>Position</er>.]</ety> <sn>1.</sn> <def>The act of supposing, laying down, imagining, or considering as true or existing, what is known not to be true, or what is not proved.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>That which is supposed; hypothesis; conjecture; surmise; opinion or belief without sufficient evidence.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>This is only an infallibility upon <qex>supposition</qex> that if a thing be true, it is imposible to be false.</q> <rj><qau>Tillotson.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>He means are in <qex>supposition</qex>.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Suppositional</ent><br/
<hw>Sup`po*si"tion*al</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Resting on supposition; hypothetical; conjectural; supposed.</def>  <rj><au>South.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Supposititious</ent><br/
<hw>Sup*pos`i*ti"tious</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>suppositicus</ets>. See <er>Supposition</er>.]</ety> <sn>1.</sn> <def>Fraudulently substituted for something else; not being what is purports to be; not genuine; spurious; counterfeit; <as>as, a <ex>supposititious</ex> child; a <ex>supposititious</ex> writing</as>.</def>  <rj><au>Bacon.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Suppositional; hypothetical.</def> <mark>[R.]</mark>  <rj><au>Woodward.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p>-- <wordforms><wf>Sup*pos`i*ti"tious*ly</wf>, <pos>adv.</pos> -- <wf>Sup*pos`i*ti"tious*ness</wf>, <pos>n.</pos></wordforms><br/
[<source>1913 Webster</source>]</p>

<p><ent>Suppositive</ent><br/
<hw>Sup*pos"i*tive</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[Cf. F. <ets>suppositif</ets>.]</ety> <def>Including or implying supposition, or hypothesis; supposed.</def> -- <wordforms><wf>Sup*pos"i*tive*ly</wf>, <pos>adv.</pos> <au>Hammond.</au></wordforms><br/
[<source>1913 Webster</source>]</p>

<p><ent>Suppositive</ent><br/
<hw>Sup*pos"i*tive</hw>, <pos>n.</pos> <def>A word denoting or implying supposition, as the words <xex>if</xex>, <xex>granting</xex>, <xex>provided</xex>, etc.</def>  <rj><au>Harris.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Suppositor</ent><br/
<hw>Sup*pos"i*tor</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Med.)</fld> <def>An apparatus for the introduction of suppositories into the rectum.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Suppository</ent><br/
<hw>Sup*pos"i*to*ry</hw> <pr>(?)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Suppositories</plw> <pr>(#)</pr>.</plu> <ety>[LL. <ets>suppositorium</ets>, fr. L. <ets>suppositorius</ets> that is placed underneath: cf. F. <ets>suppositoire</ets>. See <er>Supposition</er>.]</ety> <fld>(Med.)</fld> <def>A pill or bolus for introduction into the rectum; esp., a cylinder or cone of medicated cacao butter.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Supposure</ent><br/
<hw>Sup*po"sure</hw> <pr>(?)</pr>, <pos>n.</pos> <def>Supposition; hypothesis; conjecture.</def> <mark>[Obs.]</mark>  <rj><au>Hudibras.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Suppress</ent><br/
<hw>Sup*press"</hw> <pr>(?)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Suppressed</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Suppressing</conjf>.]</vmorph> <ety>[L. <ets>suppressus</ets>, p. p. of <ets>supprimere</ets> to suppress; <ets>sub</ets> under + <ets>premere</ets>, <ets>pressum</ets>, to press. See <er>Sub-</er>, and <er>Press</er>.]</ety> <sn>1.</sn> <def>To overpower and crush; to subdue; to put down; to quell.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Every rebellion, when it is <qex>suppressed</qex>, doth make the subject weaker, and the prince stronger.</q> <rj><qau>Sir J. Davies.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To keep in; to restrain from utterance or vent; <as>as, to <ex>suppress</ex> the voice; to <ex>suppress</ex> a smile</as>.</def>  <rj><au>Sir W. Scott.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>To retain without disclosure; to conceal; not to reveal; to prevent publication of; <as>as, to <ex>suppress</ex> evidence; to <ex>suppress</ex> a pamphlet; to <ex>suppress</ex> the truth</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>She <qex>suppresses</qex> the name, and this keeps him in a pleasing suspense.</q> <rj><qau>Broome.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>To stop; to restrain; to arrest the discharges of; <as>as, to <ex>suppress</ex> a diarrhea, or a hemorrhage</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><syn><b>Syn.</b> -- To repress; restrain; put down; overthrow; overpower; overwhelm; conceal; stifle; stop; smother.</syn><br/
[<source>1913 Webster</source>]</p>

<p><ent>Suppressible</ent><br/
<hw>Sup*press"i*ble</hw> <pr>(?)</pr>, <pos>a.</pos> <def>That may be suppressed.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Suppression</ent><br/
<hw>Sup*pres"sion</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>suppressio</ets>: cf. F. <ets>suppression</ets>.]</ety> <sn>1.</sn> <def>The act of suppressing, or the state of being suppressed; repression; <as>as, the <ex>suppression</ex> of a riot, insurrection, or tumult; the <ex>suppression</ex> of truth, of reports, of evidence, and the like.</as></def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Med.)</fld> <def>Complete stoppage of a natural secretion or excretion; <as>as, <ex>suppression</ex> of urine; -- used in contradiction to <ex>retention</ex>, which signifies that the secretion or excretion is retained without expulsion</as>.</def>  <rj><au>Quain.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <fld>(Gram.)</fld> <def>Omission; <as>as, the <ex>suppression</ex> of a word</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><syn><b>Syn.</b> -- Overthrow; destruction; concealment; repression; detention; retention; obstruction.</syn><br/
[<source>1913 Webster</source>]</p>

<p><ent>Suppressive</ent><br/
<hw>Sup*press"ive</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Tending to suppress; subduing; concealing.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Suppressor</ent><br/
<hw>Sup*press"or</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L., hider.]</ety> <def>One who suppresses.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Supprise</ent><br/
<hw>Sup*prise"</hw> <pr>(?)</pr>, <pos>v. t.</pos> <def>To surprise.</def> <mark>[Obs.]</mark>  <rj><au>Chaucer.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Suppurant</ent><br/
<hw>Sup"pu*rant</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Med.)</fld> <def>A suppurative.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Suppurate</ent><br/
<hw>Sup"pu*rate</hw> <pr>(?)</pr>, <pos>v. i.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Suppurated</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Suppurating</conjf>.]</vmorph> <ety>[L. <ets>suppuratus</ets>, p. p. of <ets>suppurare</ets> to suppurate, cause to suppurate; <ets>sub</ets> under + <ets>pus</ets>, <ets>puris</ets>, matter. See <er>Pus</er>.]</ety> <def>To generate pus; <as>as, a boil or abscess <ex>suppurates</ex></as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Suppurate</ent><br/
<hw>Sup"pu*rate</hw>, <pos>v. t.</pos> <def>To cause to generate pus; <as>as, to <ex>suppurate</ex> a sore</as>.</def>  <rj><au>Arbuthnot.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Suppuration</ent><br/
<hw>Sup`pu*ra"tion</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>suppuratio</ets>: cf. F. <ets>suppuration</ets>.]</ety> <sn>1.</sn> <def>The act or process of suppurating.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>The matter produced by suppuration; pus.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Suppurative</ent><br/
<hw>Sup"pu*ra*tive</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[Cf. F. <ets>suppuratif</ets>.]</ety> <def>Tending to suppurate; promoting suppuration.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Suppurative fever</b></col> <fld>(Med.)</fld>, <cd>pyaemia.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Suppurative</ent><br/
<hw>Sup"pu*ra*tive</hw>, <pos>n.</pos> <fld>(Med.)</fld> <def>A suppurative medicine.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Supputate</ent><br/
<hw>Sup"pu*tate</hw> <pr>(?)</pr>, <pos>v. t.</pos> <ety>[L. <ets>supputatus</ets>, p. p. of <ets>supputare</ets>. See <er>Suppute</er>.]</ety> <def>To suppute.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Supputation</ent><br/
<hw>Sup`pu*ta"tion</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>supputatio</ets>: cf. F. <ets>supputation</ets>.]</ety> <def>Reckoning; account.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Suppute</ent><br/
<hw>Sup*pute"</hw> <pr>(?)</pr>, <pos>v. t.</pos> <ety>[F. <ets>supputer</ets>, or L. <ets>supputare</ets>; <ets>sub</ets> under + <ets>putare</ets> to reckon.]</ety> <def>To reckon; to compute; to suppose; to impute.</def> <mark>[Obs.]</mark>  <rj><au>Drayton.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Supra</ent><br/
<hw>Su"pra</hw> <pr>(?)</pr>, <pos>adv.</pos> <ety>[L.; akin to <ets>super</ets>. See <er>Super-</er>.]</ety> <def>Over; above; before; also, beyond; besides; -- much used as a prefix.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Supra-acromial</ent><br/
<hw>Su`pra-a*cro"mi*al</hw> <pr>(?)</pr>, <pos>a.</pos> <fld>(Anat.)</fld> <def>Situated above the acromial process of the scapula.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Supra-angular</ent><br/
<hw>Su`pra-an"gu*lar</hw> <pr>(?)</pr>, <pos>a.</pos> <fld>(Anat.)</fld> <def>See <er>Surangular</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Supra-auricular</ent><br/
<hw>Su`pra-au*ric"u*lar</hw> <pr>(?)</pr>, <pos>a.</pos> <fld>(Zool.)</fld> <def>Situated above the ear coverts, or auriculars; -- said of certain feathers of birds.</def> -- <def2><pos>n.</pos> <def>A supra-auricular feather.</def></def2><br/
[<source>1913 Webster</source>]</p>

<p><ent>Supra-axillary</ent><br/
<hw>Su"pra-ax"il*la*ry</hw> <pr>(?)</pr>, <pos>a.</pos> <fld>(Bot.)</fld> <def>Growing above the axil; inserted above the axil, as a peduncle. See <er>Suprafoliaceous</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Suprabranchial</ent><br/
<hw>Su`pra*bran"chi*al</hw> <pr>(?)</pr>, <pos>a.</pos> <fld>(Zool.)</fld> <def>Situated above the branchiae; -- applied especially to the upper division of the gill cavity of bivalve mollusks.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Suprachoroidal</ent><br/
<ent>Suprachoroid</ent><br/
<mhw>{ <hw>Su`pra*cho"roid</hw> <pr>(?)</pr>, <hw>Su`pra*cho*roid"al</hw> <pr>(?)</pr>, }</mhw> <pos>a.</pos> <fld>(Anat.)</fld> <def>Situated above the choroid; -- applied to the layer of the choroid coat of the eyeball next to the sclerotic.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Supraciliary</ent><br/
<hw>Su`pra*cil"i*a*ry</hw> <pr>(?)</pr>, <pos>a.</pos> <fld>(Anat.)</fld> <def>Superciliary.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Supraclavicle</ent><br/
<hw>Su`pra*clav"i*cle</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Anat.)</fld> <def>A bone which usually connects the clavicle with the post-temporal in the pectorial arch of fishes.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Supraclavicular</ent><br/
<hw>Su`pra*cla*vic"u*lar</hw> <pr>(?)</pr>, <pos>a.</pos> <fld>(Anat.)</fld> <sd>(a)</sd> <def>Situated above the clavicle.</def> <sd>(b)</sd> <def>Of or pertaining to the supraclavicle.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Supracondyloid</ent><br/
<ent>Supracondylar</ent><br/
<mhw>{ <hw>Su`pra*con"dy*lar</hw> <pr>(?)</pr>, <hw>Su`pra*con"dy*loid</hw> <pr>(?)</pr>, }</mhw> <pos>a.</pos> <fld>(Anat.)</fld> <def>Situated above a condyle or condyles.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Supracostal</ent><br/
<hw>Su`pra*cos"tal</hw> <pr>(?)</pr>, <pos>a.</pos> <fld>(Anat.)</fld> <def>Situated above, or on the outside of, the ribs.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Supracranial</ent><br/
<hw>Su`pra*cra"ni*al</hw> <pr>(?)</pr>, <pos>a.</pos> <fld>(Anat.)</fld> <def>Situated above, or in the roof of, the cranium.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Supracretaceous</ent><br/
<hw>Su`pra*cre*ta"ceous</hw> <pr>(?)</pr>, <pos>a.</pos> <fld>(Geol.)</fld> <def>Lying above the chalk; Supercretaceous.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Supradecompound</ent><br/
<hw>Su`pra*de*com"pound</hw> <pr>(?)</pr>, <pos>a.</pos> <fld>(Bot.)</fld> <def>More than decompound; divided many times.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Supra-esophageal</ent><br/
<ent>Supra-esophagal</ent><br/
<mhw>{ <hw>Su`pra-e*soph"a*gal</hw> <pr>(?)</pr>, <hw>Su`pra-e`so*phag"e*al</hw> <pr>(?)</pr>, }</mhw> <pos>a.</pos> <fld>(Bot. & Zool.)</fld> <def>Situated above, or on the dorsal side of, the esophagus; <as>as, the <ex>supra-esophageal</ex> ganglion of Crustacea</as>.</def> <altsp>[Written also <asp>supra-oesophagal</asp>, and <asp>supra-oesophageal</asp>.]</altsp><br/
[<source>1913 Webster</source>]</p>

<p><ent>Supra-ethmoid</ent><br/
<hw>Su`pra-eth"moid</hw> <pr>(?)</pr>, <pos>a.</pos> <fld>(Anat.)</fld> <def>Above, or on the dorsal side of, the ethmoid bone or cartilage.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Suprafoliaceous</ent><br/
<hw>Su`pra*fo`li*a"ceous</hw> <pr>(?)</pr>, <pos>a.</pos> <fld>(Bot.)</fld> <def>Inserted into the stem above the leaf, petiole, or axil, as a peduncle or flower.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Supraglotic</ent><br/
<hw>Su`pra*glot"ic</hw> <pr>(?)</pr>, <pos>a.</pos> <fld>(Anat.)</fld> <def>Situated above the glottis; -- applied to that part of the cavity of the larynx above the true vocal cords.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Suprahepatic</ent><br/
<hw>Su`pra*he*pat"ic</hw> <pr>(?)</pr>, <pos>a.</pos> <fld>(Anat.)</fld> <def>Situated over, or on the dorsal side of, the liver; -- applied to the branches of the hepatic veins.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Suprahyoid</ent><br/
<hw>Su`pra*hy"oid</hw> <pr>(?)</pr>, <pos>a.</pos> <fld>(Anat.)</fld> <def>Hyomental.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Supra-ilium</ent><br/
<hw>Su`pra-il"i*um</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Anat.)</fld> <def>The cartilaginous cap at the sacral end of the ilium of some animals.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Supralapsarian</ent><br/
<hw>Su`pra*lap*sa"ri*an</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[<ets>Supra-</ets> + <ets>lapse</ets>: cf. F. <ets>supralapsaire</ets>.]</ety> <fld>(Eccl. Hist.)</fld> <def>One of that class of Calvinists who believed that God's decree of election determined that man should fall, in order that the opportunity might be furnished of securing the redemption of a part of the race, the decree of salvation being conceived of as formed <xex>before</xex> or <xex>beyond</xex>, and not <xex>after</xex> or <xex>following</xex>, the lapse, or fall.  Cf. <er>Infralapsarian</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Supralapsarian</ent><br/
<hw>Su`pra*lap*sa"ri*an</hw>, <pos>a.</pos> <def>Of or pertaining to the Supralapsarians, or their doctrine.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Supralapsarianism</ent><br/
<hw>Su`pra*lap*sa"ri*an*ism</hw> <pr>(?)</pr>, <pos>n.</pos> <def>The doctrine, belief, or principles of the Supralapsarians.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Supralapsary</ent><br/
<hw>Su`pra*lap"sa*ry</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Supralapsarian.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Supralapsary</ent><br/
<hw>Su`pra*lap"sa*ry</hw>, <pos>n.</pos> <def>A Supralapsarian.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Supraloral</ent><br/
<hw>Su`pra*lo"ral</hw> <pr>(?)</pr>, <pos>a.</pos> <fld>(Zool.)</fld> <def>Situated above the lores; <as>as, the <ex>supraloral</ex> feathers of a bird</as>.</def> -- <def2><pos>n.</pos> <def>A supraloral feather.</def></def2><br/
[<source>1913 Webster</source>]</p>

<p><ent>Supralunary</ent><br/
<ent>Supralunar</ent><br/
<mhw>{ <hw>Su`pra*lu"nar</hw> <pr>(?)</pr>, <hw>Su`pra*lu"na*ry</hw> <pr>(?)</pr>, }</mhw> <pos>a.</pos> <def>Beyond the moon; hence, very lofty.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Supramaxilla</ent><br/
<hw>Su`pra*max"il*la</hw> <pr>(?)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Supramaxillae</plw> <pr>(<?/)</pr>.</plu> <fld>(Anat.)</fld> <def>The upper jaw or maxilla.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Supramaxillary</ent><br/
<hw>Su`pra*max"il*la*ry</hw> <pr>(?)</pr>, <pos>a.</pos> <fld>(Anat.)</fld> <sd>(a)</sd> <def>Situated over the lower jaw; <as>as, the <ex>supramaxillary</ex> nerve</as>.</def> <sd>(b)</sd> <def>Of or pertaining to the upper jaw.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Supramundane</ent><br/
<hw>Su`pra*mun"dane</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Being or situated above the world or above our system; celestial.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Supranaturalism</ent><br/
<hw>Su`pra*nat"u*ral*ism</hw> <pr>(?)</pr>, <pos>n.</pos> <def>The state of being supernatural; belief in supernatural agency or revelation; supernaturalism.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Supranaturalist</ent><br/
<hw>Su`pra*nat"u*ral*ist</hw>, <pos>n.</pos> <def>A supernaturalist.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Supranaturalistic</ent><br/
<ent>Supranaturalist</ent><br/
<mhw>{ <hw>Su`pra*nat"u*ral*ist</hw> <pr>(?)</pr>, <hw>Su`pra*nat`u*ral*is"tic</hw> <pr>(?)</pr>, }</mhw> <pos>a.</pos> <def>Of or pertaining to supernaturalism; supernaturalistic.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Supraoccipital</ent><br/
<hw>Su`pra*oc*cip"i*tal</hw> <pr>(?)</pr>, <pos>a.</pos> <fld>(Anat.)</fld> <def>Situated over, or in the upper part of, the occiput; of or pertaining to the supraoccipital bone.</def> -- <def2><pos>n.</pos> <def>The supraoccipital bone.</def></def2><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Supraoccipital bone</b></col> <fld>(Anat.)</fld>, <cd>a bone on the dorsal side of the great foramen of the skull, usually forming a part of the occipital in the adult, but distinct in the young.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Supraocular</ent><br/
<hw>Su`pra*oc"u*lar</hw> <pr>(?)</pr>, <pos>a.</pos> <fld>(Zool.)</fld> <def>Above the eyes; -- said of certain scales of fishes and reptiles.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Supra-oesophagal</ent><br/
<hw>Su`pra-oe*soph"a*gal</hw> <pr>(?)</pr>, <pos>a.</pos> <fld>(Anat.)</fld> <def>See <er>Supra-esophagal</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Supraorbitar</ent><br/
<ent>Supraorbital</ent><br/
<mhw>{ <hw>Su`pra*or"bit*al</hw> <pr>(?)</pr>, <hw>Su`pra*or"bit*ar</hw> <pr>(?)</pr>, }</mhw> <pos>a.</pos> <fld>(Anat.)</fld> <def>Situated above the orbit of the eye.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Supraorbital point</b></col> <fld>(Anat.)</fld>, <cd>the middle point of the supraorbital line, which is a line drawn across the narrowest part of the forehead, separating the face from the cranium; the ophryon.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Suprapedal</ent><br/
<hw>Su*prap"e*dal</hw> <pr>(?)</pr>, <pos>a.</pos> <fld>(Zool.)</fld> <def>Situated above the foot of a mollusk; <as>as, the <ex>suprapedal</ex> gland</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Supraprotest</ent><br/
<hw>Su`pra*pro"test</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Mercantile Law)</fld> <def>An acceptance of a bill by a third person after protest for nonacceptance by the drawee.</def>  <rj><au>Burrill.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Suprapubic</ent><br/
<ent>Suprapubian</ent><br/
<mhw>{ <hw>Su`pra*pu"bi*an</hw> <pr>(?)</pr>, <hw>Su`pra*pu"bic</hw> <pr>(?)</pr>, }</mhw> <pos>a.</pos> <fld>(Anat.)</fld> <def>Situated above, or anterior to, the pubic bone.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Suprarenal</ent><br/
<hw>Su`pra*re"nal</hw> <pr>(?)</pr>, <pos>a.</pos> <fld>(Anat.)</fld> <def>Situated above, or anterior to, the kidneys.</def> -- <def2><pos>n.</pos> <def>A suprarenal capsule.</def></def2><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Suprarenal capsules</b></col> <fld>(Anat.)</fld>, <cd>two small bodies of unknown function in front of, or near, the kidneys in most vertebrates. Also called <altname>renal capsules</altname>, and <altname>suprarenal bodies</altname>.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Suprascalpulary</ent><br/
<ent>Suprascalpular</ent><br/
<mhw>{ <hw>Su`pra*scalp"u*lar</hw> <pr>(?)</pr>, <hw>Su`pra*scalp"u*la*ry</hw> <pr>(?)</pr>, }</mhw> <pos>a.</pos> <fld>(Anat.)</fld> <def>Situated above, or on the anterior side of, the scapula.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Suprasphenoidal</ent><br/
<hw>Su`pra*sphe*noid"al</hw> <pr>(?)</pr>, <pos>a.</pos> <fld>(Anat.)</fld> <def>Situated above the sphenoidal bone; <as>as, the <ex>suprasphenoidal</ex> appendage, or pituitary body</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Supraspinal</ent><br/
<hw>Su`pra*spi"nal</hw>, <pos>a.</pos> <fld>(Anat.)</fld> <sd>(a)</sd> <def>Situated above the vertebral column.</def> <sd>(b)</sd> <def>Situated above a spine or spines; supraspinate; supraspinous.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Supraspinous</ent><br/
<ent>Supraspinate</ent><br/
<mhw>{ <hw>Su`pra*spi"nate</hw> <pr>(?)</pr>, <hw>Su`pra*spi"nous</hw> <pr>(?)</pr>, }</mhw> <pos>a.</pos> <fld>(Anat.)</fld> <def>Situated above a spine or spines; especially, situated above, or on the dorsal side of, the neural spines of the vertebral column, or above, or in front of, the spine of the scapula.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Suprastapedial</ent><br/
<hw>Su`pra*sta*pe"di*al</hw> <pr>(?)</pr>, <pos>a.</pos> <fld>(Anat.)</fld> <def>Of, pertaining to, or designating, that part of the columella of the ear which projects above the connection with the stapes, as in many animals.</def> -- <def2><pos>n.</pos> <def>The suprastapedial part of the columella.</def></def2><br/
[<source>1913 Webster</source>]</p>

<p><ent>Suprasternal</ent><br/
<hw>Su`pra*ster"nal</hw> <pr>(?)</pr>, <pos>a.</pos> <fld>(Anat.)</fld> <def>Situated above, or anterior to, the sternum.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Supratemporal</ent><br/
<hw>Su`pra*tem"po*ral</hw> <pr>(?)</pr>, <pos>a.</pos> <fld>(Anat.)</fld> <def>Situated above the temporal bone or temporal fossa.</def> -- <def2><pos>n.</pos> <def>A supratemporal bone.</def></def2><br/
[<source>1913 Webster</source>]</p>

<p><ent>Supratrochlear</ent><br/
<hw>Su`pra*troch"le*ar</hw> <pr>(?)</pr>, <pos>a.</pos> <fld>(Anat.)</fld> <def>Situated over or above a trochlea or trochlear surface; -- applied esp. to one of the subdivisions of the trigeminal nerve.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Supravaginal</ent><br/
<hw>Su`pra*vag"i*nal</hw> <pr>(?)</pr>, <pos>a.</pos> <fld>(Anat.)</fld> <def>Situated above or outside a sheath or vaginal membrane.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Supravision</ent><br/
<hw>Su`pra*vi"sion</hw> <pr>(?)</pr>, <pos>n.</pos> <def>Supervision.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Supravisor</ent><br/
<hw>Su`pra*vis"or</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A supervisor.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Supravulgar</ent><br/
<hw>Su`pra*vul"gar</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Being above the vulgar or common people.</def> <mark>[R.]</mark>  <rj><au>Collier.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Supremacy</ent><br/
<hw>Su*prem"a*cy</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Cf. F. <ets>supr<eacute/matie</ets>. See <er>Supreme</er>.]</ety> <def>The state of being supreme, or in the highest station of power; highest or supreme authority or power; <as>as, the <ex>supremacy</ex> of a king or a parliament</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The usurped power of the pope being destroyed, the crown was restored to its <qex>supremacy</qex> over spiritual men and causes.</q> <rj><qau>Blackstone.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Oath supremacy</b></col>, <cd>an oath which acknowledges the supremacy of the sovereign in spiritual affairs, and renounced or abjures the supremacy of the pope in ecclesiastical or temporal affairs.</cd> <mark>[Eng.]</mark> <au>Brande & C.</au></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Supreme</ent><br/
<hw>Su*preme"</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>supremus</ets>, superlative of <ets>superus</ets> that is above, upper, fr. <ets>super</ets> above: cf. F. <ets>supr<ecir/me</ets>. See <er>Super-</er>, and cf. <er>Sum</er>.]</ety> <sn>1.</sn> <def>Highest in authority; holding the highest place in authority, government, or power.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>He that is the <qex>supreme</qex> King of kings.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Highest; greatest; most excellent or most extreme; utmost; greatist possible (sometimes in a bad sense); <as>as, <ex>supreme</ex> love; <ex>supreme</ex> glory; <ex>supreme</ex> magnanimity; <ex>supreme</ex> folly</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Each would be <qex>supreme</qex> within its own sphere, and those spheres could not but clash.</q> <rj><qau>De Quincey.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <fld>(Bot.)</fld> <def>Situated at the highest part or point.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>The Supreme</b></col>, <cd>the Almighty; God.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Supremely</ent><br/
<hw>Su*preme"ly</hw>, <pos>adv.</pos> <def>In a supreme manner.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Supremity</ent><br/
<hw>Su*prem"i*ty</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Cf. LL. <ets>supremitas</ets>.]</ety> <def>Supremacy.</def> <mark>[Obs.]</mark>  <rj><au>Fuller.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sur-</ent><br/
<hw>Sur-</hw>. <ety>[F. <ets>sur</ets> over, above, contr. fr. L. <ets>super</ets>, <ets>supra</ets>. See <er>Super-</er>.]</ety> <def>A prefix signifying <xex>over</xex>, <xex>above</xex>, <xex>beyond</xex>, <xex>upon</xex>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sura</ent><br/
||<hw>Su"ra</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Ar., a step, a degree.]</ety> <def>One of the sections or chapters of the Koran, which are one hundred and fourteen in number.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Suradanni</ent><br/
<hw>Su`ra*dan"ni</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A valuable kind of wood obtained on the shores of the Demerara River in South America, much used for timbers, rails, naves and fellies of wheels, and the like.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Suraddition</ent><br/
<hw>Sur`ad*di"tion</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F.]</ety> <def>Something added or appended, as to a name.</def> <mark>[Obs.]</mark>  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Surah</ent><br/
<hw>Su"rah</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A soft twilled silk fabric much used for women's dresses; -- called also <altname>surah silk</altname>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sural</ent><br/
<hw>Su"ral</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>sura</ets> the calf of the leg: cf. F. <ets>sural</ets>.]</ety> <fld>(Anat.)</fld> <def>Of or pertaining to the calf of the leg; <as>as, the <ex>sural</ex> arteries</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Surance</ent><br/
<hw>Sur"ance</hw> <pr>(?)</pr>, <pos>n.</pos> <def>Assurance.</def> <mark>[Obs.]</mark>  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Surangular</ent><br/
<hw>Sur*an"gu*lar</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[Pref. <ets>sur-</ets> + <ets>angular</ets>.]</ety> <fld>(Anat.)</fld> <def>Above the angular bone; supra-angular; -- applied to a bone of the lower jaw in many reptiles and birds.</def> -- <def2><pos>n.</pos> <def>The surangular bone.</def></def2><br/
[<source>1913 Webster</source>]</p>

<p><ent>Surbase</ent><br/
<hw>Sur"base`</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Pref. <ets>sur-</ets> + <ets>base</ets>.]</ety> <sn>1.</sn> <fld>(Arch.)</fld> <def>A cornice, or series of moldings, on the top of the base of a pedestal, podium, etc. See <xex>Illust.</xex> of <er>Column</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A board or group of moldings running round a room on a level with the tops of the chair backs.</def>  <rj><au>Knight.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Surbased</ent><br/
<hw>Sur"based`</hw> <pr>(?)</pr>, <pos>a.</pos> <fld>(Arch.)</fld> <sd>(a)</sd> <def>Having a surbase, or molding above the base.</def> <sd>(b)</sd> <ety>[F. <ets>surbaiss<eacute/</ets>.]</ety> <def>Having the vertical height from springing line to crown less than the half span; -- said of an arch; <as>as, a segmental arch is <ex>surbased</ex></as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Surbate</ent><br/
<hw>Sur*bate"</hw> <pr>(?)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Surbated</conjf>; <pos>p. pr. & vb. n.</pos> <conjf>Surbating</conjf>.]</vmorph> <ety>[F. <ets>solbatu</ets>, p. p., bruised (said of a horse's foot); <ets>sole</ets> a sole (of a horse's foot) + <ets>battu</ets>, p. p. of <ets>battre</ets> to beat.]</ety> <sn>1.</sn> <def>To make sore or bruise, as the feet by travel.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>Lest they their fins should bruise, and <qex>surbate</qex> sore<br/
Their tender feet upon the stony ground.</q> <rj><qau>Spenser.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Chalky land <qex>surbates</qex> and spoils oxen's feet.</q> <rj><qau>Mortimer.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To harass; to fatigue.</def> <mark>[Obs.]</mark>  <rj><au>Clarendon.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><-- p. 1451 --></p>

<p><ent>Surbeat</ent><br/
<hw>Sur*beat"</hw> <pr>(?)</pr>, <pos>v. t.</pos> <def>Same as <er>Surbate</er>.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Surbed</ent><br/
<hw>Sur*bed"</hw> <pr>(?)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Surbedded</conjf>; <pos>p. pr. & vb. n.</pos> <conjf>Surbedding</conjf>.]</vmorph> <ety>[Pref. <ets>sur-</ets> + <ets>bed</ets>.]</ety> <def>To set edgewise, as a stone; that is, to set it in a position different from that which it had in the quarry.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>It . . . has something of a grain parallel with the horizon, and therefore should not be <qex>surbedded</qex>.</q> <rj><qau>Gilbert White.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Surbet</ent><br/
<hw>Sur*bet"</hw> <pr>(?)</pr>, <pos>v. t.</pos> <def>Same as <er>Surbate</er>.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Surbet</ent><br/
<hw>Sur*bet"</hw>, <pos>a.</pos> <def>Surbated; bruised.</def> <mark>[Obs.]</mark>  <rj><au>Spenser.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Surcease</ent><br/
<hw>Sur*cease"</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F. <ets>sursis</ets>, from <ets>sursis</ets>, p. p. of <ets>surseoir</ets> to suspend, postpone, defer, in OF., to delay, refrain from, forbear, L. <ets>supersedere</ets>. <ets>Surcease</ets> is not connected with E. <ets>cease</ets>. See <er>Supersede</er>.]</ety> <def>Cessation; stop; end.</def> <ldquo/Not desire, but its <xex>surcease</xex>.<rdquo/  <rj><au>Longfellow.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>It is time that there were an end and <qex>surcease</qex> made of this immodest and deformed manner of writing.</q> <rj><qau>Bacon.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Surcease</ent><br/
<hw>Sur*cease"</hw>, <pos>v. t.</pos> <def>To cause to cease; to end.</def> <mark>[Obs.]</mark> <ldquo/The waves . . . their range <xex>surceast</xex>.<rdquo/  <rj><au>Spenser.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>The nations, overawed, <qex>surceased</qex> the fight.</q> <rj><qau>Dryden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Surcease</ent><br/
<hw>Sur*cease"</hw>, <pos>v. i.</pos> <def>To cease.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Surceaseance</ent><br/
<hw>Sur*cease"ance</hw> <pr>(?)</pr>, <pos>n.</pos> <def>Cessation.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Surcharge</ent><br/
<hw>Sur*charge"</hw> <pr>(?)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Surcharged</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Surcharging</conjf> <pr>(?)</pr>.]</vmorph> <ety>[F. <ets>surcharger</ets>. See <er>Sur-</er>, and <er>Charge</er>, and cf. <er>Overcharge</er>, <er>Supercharge</er>, <er>Supercargo</er>.]</ety> <sn>1.</sn> <def>To overload; to overburden; to overmatch; to overcharge; <as>as, to <ex>surcharge</ex> a beast or a ship; to <ex>surcharge</ex> a cannon</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Four charged two, and two <qex>surcharged</qex> one.</q> <rj><qau>Spenser.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Your head reclined, as hiding grief from view,<br/
Droops like a rose <qex>surcharged</qex> with morning dew.</q> <rj><qau>Dryden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Law)</fld> <sd>(a)</sd> <def>To overstock; especially, to put more cattle into, as a common, than the person has a right to do, or more than the herbage will sustain. <xex>Blackstone</xex>.</def> <sd>(b)</sd> <fld>(Equity)</fld> <def>To show an omission in (an account) for which credit ought to have been given.</def>  <rj><au>Story. Daniel.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>To print or write a surcharge on (a postage stamp).</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Surcharge</ent><br/
<hw>Sur*charge"</hw>, <pos>n.</pos> <ety>[F.]</ety> <sn>1.</sn> <def>An overcharge; an excessive load or burden; a load greater than can well be borne.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>A numerous nobility causeth poverty and inconvenience in a state, for it is <qex>surcharge</qex> of expense.</q> <rj><qau>Bacon.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Law)</fld> <sd>(a)</sd> <def>The putting, by a commoner, of more beasts on the common than he has a right to.</def> <sd>(b)</sd> <fld>(Equity)</fld> <def>The showing an omission, as in an account, for which credit ought to have been given.</def>  <rj><au>Burrill.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <fld>(Railroads)</fld> <def>A charge over the usual or legal rates.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><sn>4.</sn>  <def>Something printed or written on a postage stamp to  give it a new legal effect, as a new valuation, a place, a date, etc.; also (<mark>Colloq.</mark>), a stamp with a surcharge.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Surchargement</ent><br/
<hw>Sur*charge"ment</hw> <pr>(?)</pr>, <pos>n.</pos> <def>The act of surcharging; also, surcharge, surplus.</def> <mark>[Obs.]</mark>  <rj><au>Daniel.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Surcharger</ent><br/
<hw>Sur*char"ger</hw> <pr>(?)</pr>, <pos>n.</pos> <def>One who surcharges.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Surcingle</ent><br/
<hw>Sur"cin`gle</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[OE. <ets>sursengle</ets>, OF. <ets>sursangle</ets>. See <er>Sur-</er>, and <er>Cingle</er>, <er>Shingles</er>.]</ety> <sn>1.</sn> <def>A belt, band, or girth which passes over a saddle, or over anything laid on a horse's back, to bind it fast.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Eccl.)</fld> <def>The girdle of a cassock, by which it is fastened round the waist.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Surcingled</ent><br/
<hw>Sur"cin`gled</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Bound with the surcingle.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Surcle</ent><br/
<hw>Sur"cle</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>surculus</ets>.]</ety> <def>A little shoot; a twig; a sucker.</def> <mark>[Obs.]</mark>  <rj><au>Sir T. Browne.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Surcloy</ent><br/
<hw>Sur"cloy</hw> <pr>(?)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Surcloyed</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Surcloying</conjf>.]</vmorph> <def>To surfeit.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Surcoat</ent><br/
<hw>Sur"coat`</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[OE. <ets>surcote</ets>, OF. <ets>surcote</ets>. See <er>Sur-</er>, and <er>Coat</er>, and cf. <er>Overcoat</er>.]</ety> <sn>1.</sn> <def>A coat worn over the other garments; especially, the long and flowing garment of knights, worn over the armor, and frequently emblazoned with the arms of the wearer.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>A long <qex>surcoat</qex> of pers upon he had..</q> <rj><qau>Chaucer.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>At night, or in the rain,<br/
He dons a <qex>surcoat</qex> which he doffs at morn.</q> <rj><qau>Emerson.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A name given to the outer garment of either sex at different epochs of the Middle Ages.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Surcrew</ent><br/
<hw>Sur"crew`</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[From F. <ets>surcro<icir/t</ets> increase, or <ets>surcr<ucir/</ets>, p. p. of <ets>surcro<icir/tre</ets> to overgrow.]</ety> <def>Increase; addition; surplus.</def> <mark>[Obs.]</mark>  <rj><au>Sir H. Wotton.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Surculate</ent><br/
<hw>Sur"cu*late</hw> <pr>(?)</pr>, <pos>v. t.</pos> <ety>[L. <ets>surculatus</ets>, p. p. of <ets>surculare</ets> to purne, from <ets>surculus</ets> a shoot, sprout. See <er>Surcle</er>.]</ety> <def>To purne; to trim.</def> <mark>[Obs.]</mark>  <rj><au>Cockeram.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Surculation</ent><br/
<hw>Sur`cu*la"tion</hw> <pr>(?)</pr>, <pos>n.</pos> <def>Act of purning.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Surculose</ent><br/
<hw>Sur"cu*lose`</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[CF. L. <ets>sucrulosus</ets> woody. See <er>Surcle</er>.]</ety> <fld>(Bot.)</fld> <def>Producing suckers, or shoots resembling suckers.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Surd</ent><br/
<hw>Surd</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>surdus</ets> deaf (whence the meaning, deaf to reason, irrational), perhaps akin to E. <ets>swart</ets>.  Cf. <er>Sordine</er>.]</ety> <sn>1.</sn> <def>Net having the sense of hearing; deaf.</def> <mark>[Obs.]</mark> <ldquo/A <xex>surd</xex> . . . generation.<rdquo/  <rj><au>Sir T. Browne.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Unheard.</def> <mark>[Obs.]</mark>  <rj><au>Kenrick.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <fld>(Math.)</fld> <def>Involving surds; not capable of being expressed in rational numbers; radical; irrational; <as>as, a <ex>surd</ex> expression or quantity; a <ex>surd</ex> number</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <fld>(Phonetics)</fld> <def>Uttered, as an element of speech, without tone, or proper vocal sound; voiceless; unintonated; nonvocal; atonic; whispered; aspirated; sharp; hard, as <xex>f</xex>, <xex>p</xex>, <xex>s</xex>, etc.; -- opposed to <xex>sonant</xex>. See <xex>Guide to Pronunciation</xex>, <sect/<sect/169, 179, 180.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Surd</ent><br/
<hw>Surd</hw>, <pos>n.</pos> <fld>(Math.)</fld> <sn>1.</sn> <def>A quantity which can not be expressed by rational numbers; <as>thus, <root/2 is a <ex>surd</ex></as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Phon.)</fld> <def>A surd element of speech. See <er>Surd</er>, <pos>a.</pos>, 4.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Surdal</ent><br/
<hw>Surd"al</hw> <pr>(?)</pr>, <pos>a.</pos> <fld>(Math.)</fld> <def>Same as <er>Surd</er>, <pos>a.</pos>, 3.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Surdiny</ent><br/
<hw>Surd"i*ny</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A sardine.</def> <mark>[Obs.]</mark>  <rj><au>Beau. & Fl.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Surdity</ent><br/
<hw>Surd"i*ty</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>surditas</ets>.]</ety> <def>Deafness.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sure</ent><br/
<hw>Sure</hw> <pr>(?)</pr>, <pos>a.</pos> <amorph>[<pos>Compar.</pos> <adjf>Surer</adjf> <pr>(?)</pr>; <pos>superl.</pos> <adjf>Surest</adjf>.]</amorph> <ety>[OE. <ets>sur</ets>, OF. <ets>se<uum/r</ets>, F. <ets>s<ucir/r</ets>, L. <ets>securus</ets>; <ets>se</ets> aside, without + <ets>cura</ets> care. See <er>Secure</er>, and cf. <er>Assure</er>, <er>Insure</er>, <er>Sicker</er> sure.]</ety> <sn>1.</sn> <def>Certainly knowing and believing; confident beyond doubt; implicity trusting; unquestioning; positive.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>We are <qex>sure</qex> that the judgment of God is according to truth against them which commit such things.</q> <rj><qau>Rom. ii. 2.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>I'm <qex>sure</qex> care 's an enemy of life.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Certain to find or retain; <as>as, to be <ex>sure</ex> of game; to be <ex>sure</ex> of success; to be <ex>sure</ex> of life or health</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>Fit or worthy to be depended on; certain not to fail or disappoint expectation; unfailing; strong; permanent; enduring.</def> <ldquo/His <xex>sure</xex> word.<rdquo/  <rj><au>Keble.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>The Lord will certainly make my lord a <qex>sure</qex> house; because my lord fighteth the battles of the Lord.</q> <rj><qau>1 Sam. xxv. 28.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>The testimony of the Lord is <qex>sure</qex>.</q> <rj><qau>Ps. xix. 7.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Which put in good <qex>sure</qex> leather sacks.</q> <rj><qau>Chapman.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>Betrothed; engaged to marry.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>The king was <qex>sure</qex> to Dame Elizabeth Lucy, and her husband before God.</q> <rj><qau>Sir T. More.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>I presume . . . that you had been <qex>sure</qex> as fast as faith could bind you, man and wife.</q> <rj><qau>Brome.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <def>Free from danger; safe; secure.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Fear not; the forest is not three leagues off;<br/
If we recover that we are <qex>sure</qex> enough.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p>-- <cs><mcol><col><b>To be sure</b></col>, <it>or</it> <col><b>Be sure</b></col></mcol>, <cd>certainly; without doubt; as, Shall you do? <xex>To be sure</xex> I shall.</cd> -- <col><b>To make sure</b></col>. <sd>(a)</sd> <cd>To make certain; to secure so that there can be no failure of the purpose or object.</cd> <ldquo/<xex>Make</xex> Cato <xex>sure</xex>.<rdquo/ <au>Addison.</au> <ldquo/A peace can not fail, provided we <xex>make sure</xex> of Spain.<rdquo/ <au>Sir W. Temple.</au> <sd>(b)</sd> <cd>To betroth.</cd> <mark>[Obs.]</mark></cs><br/
[<source>1913 Webster</source>]</p>

<p><q>She that's <qex>made sure</qex> to him she loves not well.</q> <rj><qau>Cotgrave.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><syn><b>Syn.</b> -- Certain; unfailing; infallible; safe; firm; permanent; steady; stable; strong; secure; indisputable; confident; positive.</syn><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sure</ent><br/
<hw>Sure</hw> <pr>(?)</pr>, <pos>adv.</pos> <def>In a sure manner; safely; certainly.</def> <ldquo/Great, <xex>sure</xex>, shall be thy meed.<rdquo/  <rj><au>Spenser.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>'T is pleasant, <qex>sure</qex>, to see one's name in print.</q> <rj><qau>Byron.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sure-footed</ent><br/
<hw>Sure"-foot`ed</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Not liable to stumble or fall; <as>as, a <ex>sure-footed</ex> horse</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Surely</ent><br/
<hw>Sure"ly</hw> <pr>(?)</pr>, <pos>adv.</pos> <sn>1.</sn> <def>In a sure or certain manner; certainly; infallibly; undoubtedly; assuredly.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>In the day that thou eatest thereof thou shalt <qex>surely</qex> die.</q> <rj><qau>Gen. ii. 17.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>He that created something out of nothing, <qex>surely</qex> can raise great things out of small.</q> <rj><qau>South.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Without danger; firmly; steadly; securely.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>He that walketh uprightly walketh <qex>surely</qex>.</q> <rj><qau>Prov. x. 9.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Surement</ent><br/
<hw>Sure"ment</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A making sure; surety.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>Every <qex>surement</qex> and every bond.</q> <rj><qau>Chaucer.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sureness</ent><br/
<hw>Sure"ness</hw>, <pos>n.</pos> <def>The state of being sure; certainty.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>For more <qex>sureness</qex> he repeats it.</q> <rj><qau>Woodward.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>The law holds with equal <qex>sureness</qex> for all right action.</q> <rj><qau>Emerson.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Suresby</ent><br/
<hw>Sures"by</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Etymol. uncertain. See <er>Rudesby</er>.]</ety> <def>One to be sure of, or to be relied on.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>There is one which is <qex>suresby</qex>, as they say, to serve, if anything will serve.</q> <rj><qau>Bradford.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Suretiship</ent><br/
<hw>Sure"ti*ship</hw> <pr>(?)</pr>, <pos>n.</pos> <def>Suretyship.</def>  <rj><au>Prov. xi. 15.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Surety</ent><br/
<hw>Sure"ty</hw> <pr>(?)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Sureties</plw> <pr>(#)</pr>.</plu> <ety>[OE. <ets>seurte</ets>, OF. <ets>se<uum/rt<eacute/</ets>, F. <ets>s<ucir/ret<eacute/</ets>. See <er>Sure</er>, <er>Security</er>.]</ety> <sn>1.</sn> <def>The state of being sure; certainty; security.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Know of a <qex>surety</qex>, that thy seed shall be a stranger in a land that is not theirs.</q> <rj><qau>Gen. xv. 13.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>For the more <qex>surety</qex> they looked round about.</q> <rj><qau>Sir P. Sidney.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>That which makes sure; that which confirms; ground of confidence or security.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>[We] our happy state<br/
Hold, as you yours, while our obedience holds;<br/
On other <qex>surety</qex> none.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>Security against loss or damage; security for payment, or for the performance of some act.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>There remains unpaid<br/
A hundred thousand more; in <qex>surety</qex> of the which<br/
One part of Aquitaine is bound to us.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <fld>(Law)</fld> <def>One who is bound with and for another who is primarily liable, and who is called the <xex>principal</xex>; one who engages to answer for another's appearance in court, or for his payment of a debt, or for performance of some act; a bondsman; a bail.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>He that is <qex>surety</qex> for a stranger shall smart for it.</q> <rj><qau>Prov. xi. 15.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <def>Hence, a substitute; a hostage.</def>  <rj><au>Cowper.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>6.</sn> <def>Evidence; confirmation; warrant.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>She called the saints to <qex>surety</qex>,<br/
That she would never put it from her finger,<br/
Unless she gave it to yourself.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Surety</ent><br/
<hw>Sure"ty</hw>, <pos>v. t.</pos> <def>To act as surety for.</def> <mark>[Obs.]</mark>  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Suretyship</ent><br/
<hw>Sure"ty*ship</hw>, <pos>n.</pos> <def>The state of being surety; the obligation of a person to answer for the debt, default, or miscarriage of another.</def>  <rj><au>Bouvier.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Surf</ent><br/
<hw>Surf</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Formerly spelled <ets>suffe</ets>, and probably the same word as E. <ets>sough</ets>.]</ety> <def>The swell of the sea which breaks upon the shore, esp. upon a sloping beach.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Surf bird</b></col> <fld>(Zool.)</fld>, <cd>a ploverlike bird of the genus <gen>Aphriza</gen>, allied to the turnstone.</cd> -- <col><b>Surf clam</b></col> <fld>(Zool.)</fld>, <cd>a large clam living on the open coast, especially <spn>Mactra solidissima</spn> (syn. <spn>Spisula solidissima</spn>). See <er>Mactra</er>.</cd> -- <col><b>Surf duck</b></col> <fld>(Zool.)</fld>, <cd>any one of several species of sea ducks of the genus <gen>Oidemia</gen>, especially <spn>Oidemia percpicillata</spn>; -- called also <altname>surf scoter</altname>. See the Note under <er>Scoter</er>.</cd> -- <col><b>Surf fish</b></col> <fld>(Zool.)</fld>, <cd>any one of numerous species of California embiotocoid fishes. See <er>Embiotocoid</er>.</cd> -- <col><b>Surf smelt</b></col>. <fld>(Zool.)</fld> <cd>See <er>Smelt</er>.</cd> -- <col><b>Surf whiting</b></col>. <fld>(Zool.)</fld> <cd>See under <er>Whiting</er>.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Surf</ent><br/
<hw>Surf</hw>, <pos>n.</pos> <def>The bottom of a drain.</def> <mark>[Prov. Eng.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Surface</ent><br/
<hw>Sur"face`</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F. See <er>Sur-</er>, and <er>Face</er>, and cf. <er>Superficial</er>.]</ety> <sn>1.</sn> <def>The exterior part of anything that has length and breadth; one of the limits that bound a solid, esp. the upper face; superficies; the outside; <as>as, the <ex>surface</ex> of the earth; the <ex>surface</ex> of a diamond; the <ex>surface</ex> of the body</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The bright <qex>surface</qex> of this ethereous mold.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Hence, outward or external appearance.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Vain and weak understandings, which penetrate no deeper than the <qex>surface</qex>.</q> <rj><qau>V. Knox.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <fld>(Geom.)</fld> <def>A magnitude that has length and breadth without thickness; superficies; <as>as, a plane <ex>surface</ex>; a spherical <ex>surface</ex></as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <fld>(Fort.)</fld> <def>That part of the side which is terminated by the flank prolonged, and the angle of the nearest bastion.</def>  <rj><au>Stocqueler.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><mcol><col><b>Caustic surface</b></col>, <col><b>Heating surface</b></col></mcol>, <cd>etc. See under <er>Caustic</er>, <er>Heating</er>, etc.</cd> -- <mcol><col><b>Surface condensation</b></col>, <col><b>Surface condenser</b></col></mcol>. <cd>See under <er>Condensation</er>, and <er>Condenser</er>.</cd> -- <col><b>Surface gauge</b></col> <fld>(Mach.)</fld>, <cd>an instrument consisting of a standard having a flat base and carrying an adjustable pointer, for gauging the evenness of a surface or its height, or for marking a line parallel with a surface.</cd> -- <col><b>Surface grub</b></col> <fld>(Zool.)</fld>, <cd>the larva of the great yellow underwing moth (<spn>Triphoena pronuba</spn>). It is often destructive to the roots of grasses and other plants.</cd> -- <col><b>Surface plate</b></col> <fld>(Mach.)</fld>, <cd>a plate having an accurately dressed flat surface, used as a standard of flatness by which to test other surfaces.</cd> -- <col><b>Surface printing</b></col>, <cd>printing from a surface in relief, as from type, in distinction from <xex>plate printing</xex>, in which the ink is contained in engraved lines.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Surface</ent><br/
<hw>Sur"face</hw> <pr>(?)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Surfaced</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Surfacing</conjf> <pr>(?)</pr>.]</vmorph> <sn>1.</sn> <def>To give a surface to; especially, to cause to have a smooth or plain surface; to make smooth or plain.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To work over the surface or soil of, as ground, in hunting for gold.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Surface</ent><br/
<hw>Sur"face</hw> <pr>(?)</pr>, <pos>v. i.</pos> <sn>1.</sn> <def>To rise from the depths of a liquid to the surface; <as>as, the submarine <ex>surfaced</ex> to recharge its batteries</as>.</def><br/
[<source>PJC</source>]</p>

<p><sn>2.</sn> <def>To become known or public; -- said of information.</def><br/
[<source>PJC</source>]</p>

<p><sn>3.</sn>  <def>To show up, as a person who was in hiding; <as>as, he absconded with the payroll and <ex>surfaced</ex> in Argentina</as>.</def><br/
[<source>PJC</source>]</p>

<p><ent>Surface loading</ent><br/
<hw>Sur"face load`ing</hw>. <fld>(Aeronautics)</fld> <def>The weight supported per square unit of surface; the quotient obtained by dividing the gross weight, in pounds, of a fully loaded flying machine, by the total area, in square feet, of its supporting surface.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Surfacer</ent><br/
<hw>Sur"fa*cer</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A form of machine for dressing the surface of wood, metal, stone, etc.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Surface tension</ent><br/
<hw>Sur"face ten"sion</hw>. <fld>(Physics)</fld> <def>That property, due to molecular forces, which exists in the <xex>surface film</xex> of all liquids and tends to bring the contained volume into a form having the least superficial area. The thickness of this film, amounting to less than a thousandth of a millimeter, is considered to equal the radius of the sphere of molecular action, that is, the greatest distance at which there is cohesion between two particles. Particles lying below this film, being equally acted on from all sides, are in equilibrium as to forces of cohesion, but those in the film are on the whole attracted inward, and tension results.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Surfboat</ent><br/
<hw>Surf"boat`</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Naut.)</fld> <def>A boat intended for use in heavy surf. It is built with a pronounced sheer, and with a view to resist the shock of waves and of contact with the beach.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Surfeit</ent><br/
<hw>Sur"feit</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[OE. <ets>surfet</ets>, OF. <ets>surfait</ets>, <ets>sorfait</ets>, excess, arrogance, crime, fr. <ets>surfaire</ets>, <ets>sorfaire</ets>, to augment, exaggerate, F. <ets>surfaire</ets> to overcharge; <ets>sur</ets> over + <ets>faire</ets> to make, do, L. <ets>facere</ets>. See <er>Sur-</er>, and <er>Fact</er>.]</ety> <sn>1.</sn> <def>Excess in eating and drinking.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Let not Sir <qex>Surfeit</qex> sit at thy board.</q> <rj><qau>Piers Plowman.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Now comes the sick hour that his <qex>surfeit</qex> made.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Fullness and oppression of the system, occasioned often by excessive eating and drinking.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>To prevent <qex>surfeit</qex> and other diseases that are incident to those that heat their blood by travels.</q> <rj><qau>Bunyan.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>Disgust caused by excess; satiety.</def>  <rj><au>Sir P. Sidney.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Matter and argument have been supplied abundantly, and even to <qex>surfeit</qex>.</q> <rj><qau>Burke.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Surfeit</ent><br/
<hw>Sur"feit</hw>, <pos>v. i.</pos> <sn>1.</sn> <def>To load the stomach with food, so that sickness or uneasiness ensues; to eat to excess.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>They are as sick that <qex>surfeit</qex> with too much as they that starve with nothing.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To indulge to satiety in any gratification.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Surfeit</ent><br/
<hw>Sur"feit</hw>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Surfeited</conjf>; <pos>p. pr. & vb. n.</pos> <conjf>Surfeiting</conjf>.]</vmorph> <sn>1.</sn> <def>To feed so as to oppress the stomach and derange the function of the system; to overfeed, and produce satiety, sickness, or uneasiness; -- often reflexive; <as>as, to <ex>surfeit</ex> one's self with sweets</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To fill to satiety and disgust; to cloy; <as>as, he <ex>surfeits</ex> us with compliments</as>.</def>  <rj><au>V. Knox.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Surfeiter</ent><br/
<hw>Sur"feit*er</hw> <pr>(?)</pr>, <pos>n.</pos> <def>One who surfeits.</def>  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Surfeit-water</ent><br/
<hw>Sur"feit-wa`ter</hw> <pr>(?)</pr>, <pos>n.</pos> <def>Water for the cure of surfeits.</def> <mark>[Obs.]</mark>  <rj><au>Locke.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Surfle</ent><br/
<ent>Surfel</ent><br/
<mhw>{ <hw>Sur"fel</hw>, <hw>Sur"fle</hw>  }</mhw> <pr>(?)</pr>, <pos>v. t.</pos> <ety>[Cf. <er>Sulphur</er>.]</ety> <def>To wash, as the face, with a cosmetic water, said by some to be prepared from the sulphur.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>She shall no oftener powder her hair, [or] <qex>surfel</qex> her cheeks, . . . but she shall as often gaze on my picture.</q> <rj><qau>Ford.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Surfer</ent><br/
<hw>Surf"er</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Zool.)</fld> <def>The surf duck.</def> <mark>[U. S.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Surfman</ent><br/
<hw>Surf"man</hw> <pr>(?)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Surmen</plw> <pr>(<?/)</pr>.</plu> <def>One who serves in a surfboat in the life-saving service.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Surfoot</ent><br/
<hw>Sur"foot`</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Tired or sore of foot from travel; lamed.</def> <mark>[Obs.]</mark>  <rj><au>Nares.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Surfy</ent><br/
<hw>Surf"y</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Consisting of, abounding in, or resembling, surf; <as>as, a <ex>surfy</ex> shore</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Scarce had they cleared the <qex>surfy</qex> waves<br/
That foam around those frightful caves.</q> <rj><qau>Moore.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Surge</ent><br/
<hw>Surge</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>surgere</ets>, <ets>surrectum</ets>, to raise, to rise; <ets>sub</ets> under + <ets>regere</ets> to direct: cf. OF. <ets>surgeon</ets>, <ets>sourgeon</ets>, fountain. See <er>Regent</er>, and cf. <er>Insurrection</er>, <er>Sortie</er>, <er>Source</er>.]</ety> <sn>1.</sn> <def>A spring; a fountain.</def> <mark>[Obs.]</mark> <ldquo/Divers <xex>surges</xex> and springs of water.<rdquo/  <rj><au>Ld. Berners.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A large wave or billow; a great, rolling swell of water, produced generally by a high wind.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>He that doubteth is like the <qex>surge</qex> of the sea driven by the wind and tossed.</q> <rj><qau>James i. 6 (Rev. Ver.)</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>He flies aloft, and, with impetuous roar,<br/
Pursues the foaming <qex>surges</qex> to the shore.</q> <rj><qau>Dryden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>The motion of, or produced by, a great wave.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>The tapered part of a windlass barrel or a capstan, upon which the cable surges, or slips.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Surge</ent><br/
<hw>Surge</hw>, <pos>v. i.</pos> <sn>1.</sn> <def>To swell; to rise hifg and roll.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The <qex>surging</qex> waters like a mountain rise.</q> <rj><qau>Spenser.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Naut.)</fld> <def>To slip along a windlass.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Surge</ent><br/
<hw>Surge</hw>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Surged</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Surging</conjf> <pr>(?)</pr>.]</vmorph> <ety>[Cf. F. <ets>surgir</ets> to cast anchor, to land.  Cf. <er>Surge</er>, <pos>n.</pos>]</ety> <fld>(Naut.)</fld> <def>To let go or slacken suddenly, as a rope; <as>as, to <ex>surge</ex> a hawser or messenger</as>; also, to slacken the rope about (a capstan).</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Surgeful</ent><br/
<hw>Surge"ful</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Abounding in surges; surgy.</def> <ldquo/Tossing the <xex>surgeful</xex> tides.<rdquo/  <rj><au>Drayton.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Surgeless</ent><br/
<hw>Surge"less</hw>, <pos>a.</pos> <def>Free from surges; smooth; calm.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Surgent</ent><br/
<hw>Sur"gent</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>surgens</ets>, p. pr.]</ety> <def>Rising; swelling, as a flood.</def> <mark>[R.]</mark>  <rj><au>Robert Greene.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Surgeon</ent><br/
<hw>Sur"geon</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[OE. <ets>surgien</ets>, OF. <ets>surgien</ets>, contr. fr. <ets>chirurgien</ets>. See <er>Chirurgeon</er>.]</ety> <sn>1.</sn> <def>One whose profession or occupation is to cure diseases or injuries of the body by manual operation; one whose occupation is to cure local injuries or disorders (such as wounds, dislocations, tumors, etc.), whether by manual operation, or by medication and constitutional treatment.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Zool.)</fld> <def>Any one of numerous species of chaetodont fishes of the family <fam>Teuthidae</fam>, or <fam>Acanthuridae</fam>, which have one or two sharp lancelike spines on each side of the base of the tail. Called also <altname>surgeon fish</altname>, <altname>doctor fish</altname>, <altname>lancet fish</altname>, and <altname>sea surgeon</altname>.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Surgeon apothecary</b></col>, <cd>one who unites the practice of surgery with that of the apothecary.</cd> <au>Dunglison.</au> -- <col><b>Surgeon dentist</b></col>, <cd>a dental surgeon; a dentist.</cd> -- <col><b>Surgeon fish</b></col>. <cd>See def. 2, above.</cd> -- <col><b>Surgeon general</b></col>. <sd>(a)</sd> <cd>In the United States army, the chief of the medical department.</cd> <sd>(b)</sd> <cd>In the British army, a surgeon ranking next below the chief of the medical department.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><-- p. 1452 --></p>

<p><ent>Surgeoncy</ent><br/
<hw>Sur"geon*cy</hw> <pr>(s<ucir/r"j<ucr/n*s<ycr/)</pr>, <pos>n.</pos> <def>The office or employment of a surgeon, as in the naval or military service.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Surgeonry</ent><br/
<hw>Sur"geon*ry</hw> <pr>(s<ucir/r"j<ucr/n*r<ycr/)</pr>, <pos>n.</pos> <def>Surgery.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Surgery</ent><br/
<hw>Sur"ge*ry</hw> <pr>(s<ucir/r"j<etil/r*<ycr/)</pr>, <pos>n.</pos> <ety>[OE. <ets>surgenrie</ets>, <ets>surgerie</ets>; cf. OF. <ets>cirurgie</ets>, F. <ets>chirurgie</ets>, L. <ets>chirurgia</ets>, Gr. <?/. See <er>Surgeon</er>.]</ety> <sn>1.</sn> <def>The art of healing by manual operation; that branch of medical science which treats of manual operations for the healing of diseases or injuries of the body; that branch of medical science which has for its object the cure of local injuries or diseases, as wounds or fractures, tumors, etc., whether by manual operation or by medicines and constitutional treatment.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A surgeon's operating room or laboratory.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Surgical</ent><br/
<hw>Sur"gi*cal</hw> <pr>(s<ucir/r"j<icr/*k<ait/l)</pr>, <pos>a.</pos> <def>Of or pertaining to surgeons or surgery; done by means of surgery; used in surgery; <as>as, a <ex>surgical</ex> operation; <ex>surgical</ex> instruments</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Surgical fever</b></col>. <fld>(Med.)</fld> <sd>(a)</sd> <cd>Pyaemia.</cd> <sd>(b)</sd> <cd>Traumatic fever, or the fever accompanying inflammation.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Surgically</ent><br/
<hw>Sur"gi*cal*ly</hw>, <pos>adv.</pos> <def>By means of surgery.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Surgy</ent><br/
<hw>Sur"gy</hw> <pr>(s<ucir/r"j<ycr/)</pr>, <pos>a.</pos> <def>Rising in surges or billows; full of surges; resembling surges in motion or appearance; swelling.</def> <ldquo/Over the <xex>surgy</xex> main.<rdquo/  <rj><au>Pope.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>suricate</ent><br/
<ent>suricat</ent><br/
<mhw><hw>su"ri*cat</hw>, <hw>su"ri*cate</hw></mhw> <pr>(s<umac/"r<icr/*k<acr/t)</pr>, <pos>n.</pos> <ety>[F. <ets>surikate</ets>, from the native name in South Africa.]</ety> <fld>(Zool.)</fld> <def>A South African mongooselike viverrine (<spn>Suricata suricatta</spn> or <spn>Suricata tetradactyla</spn>) having a lemurlike face and only four toes; called also <altname>mierkat</altname>, <altname>yellow mongoose</altname> and <altname>zenick</altname>.</def> <altsp>[Also spelled <asp>surikate</asp>.]</altsp> <note>The animal called <er>zenick</er> seems also to be related to or identical to the mierkat, but no reference can be found on the internet.</note><br/
<syn><b>Syn. --</b> meerkat.</syn><br/
[<source>1913 Webster</source> + <source>WordNet 1.5</source>]</p>

<p><ent>Surinam toad</ent><br/
<hw>Su`ri*nam" toad"</hw> <pr>(s<oomac/`r<icr/*n<aum/m" t<omac/d`)</pr>, <pos>prop. n.</pos>. <fld>(Zool.)</fld> <def>A species of toad native of Surinam.  See <er>Pipa</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Surintendant</ent><br/
<hw>Sur`in*tend"ant</hw> <pr>(s<ucir/r`<icr/n*t<ecr/nd"<ait/nt)</pr>, <pos>n.</pos> <ety>[F. See <er>Superintendent</er>.]</ety> <def>Superintendent.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Surlily</ent><br/
<hw>Sur"li*ly</hw> <pr>(s<ucir/r"l<icr/*l<ycr/)</pr>, <pos>adv.</pos> <def>In a surly manner.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Surliness</ent><br/
<hw>Sur"li*ness</hw>, <pos>n.</pos> <def>The quality or state of being surly.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Surling</ent><br/
<hw>Sur"ling</hw> <pr>(s<ucir/r"l<icr/ng)</pr>, <pos>n.</pos> <ety>[See <er>Surly</er>.]</ety> <def>A sour, morose fellow.</def> <mark>[Obs.]</mark>  <rj><au>Camden.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Surloin</ent><br/
<hw>Sur"loin`</hw> <pr>(s<ucir/r"loin`)</pr>, <pos>n.</pos> <ety>[F. <ets>surlonge</ets>; <ets>sur</ets> upon + <ets>longe</ets> loin. See <er>Sur-</er>, and <er>Loin</er>.]</ety> <def>A loin of beef, or the upper part of the loin.  See <er>Sirloin</er>, the more usual, but not etymologically preferable, orthography.</def> <altsp>[Also spelled <asp>sirloin</asp>.]</altsp><br/
[<source>1913 Webster</source>]</p>

<p><ent>Surly</ent><br/
<hw>Sur"ly</hw> <pr>(s<ucir/r"l<ycr/)</pr>, <pos>a.</pos> <amorph>[<pos>Compar.</pos> <adjf>Surlier</adjf> <pr>(s<ucir/r"l<icr/*<etil/r)</pr>; <pos>superl.</pos> <adjf>Surliest</adjf>.]</amorph> <ety>[Probably from <ets>sir</ets>, and originally meaning, <ets>sirlike</ets>, i.e., proud. See <er>Sir</er>, and <er>Like</er>, <pos>a.</pos>]</ety> <sn>1.</sn> <def>Arrogant; haughty.</def> <mark>[Obs.]</mark>  <rj><au>Cotgrave.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Gloomily morose; ill-natured, abrupt, and rude; severe; sour; crabbed; rough; sullen; gloomy; <as>as, a <ex>surly</ex> groom; a <ex>surly</ex> dog; <ex>surly</ex> language; a <ex>surly</ex> look</as>.</def> <ldquo/That <xex>surly</xex> spirit, melancholy.<rdquo/  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>Rough; dark; tempestuous.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Now softened into joy the <qex>surly</qex> storm.</q> <rj><qau>Thomson.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Surmark</ent><br/
<hw>Sur"mark`</hw> <pr>(s<ucir/r"m<aum/rk`)</pr>, <pos>n.</pos> <fld>(Shipbuilding)</fld> <def>A mark made on the molds of a ship, when building, to show where the angles of the timbers are to be placed.</def> <altsp>[Written also <asp>sirmark</asp>.]</altsp><br/
[<source>1913 Webster</source>]</p>

<p><ent>Surmisable</ent><br/
<hw>Sur*mis"a*ble</hw> <pr>(s<ucir/r*m<imac/z"<adot/*b'l)</pr>, <pos>a.</pos> <def>Capable of being surmised; <as>as, a <ex>surmisable</ex> result</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Surmisal</ent><br/
<hw>Sur*mis"al</hw> <pr>(s<ucir/r*m<imac/z"<ait/l)</pr>, <pos>n.</pos> <def>Surmise.</def> <mark>[R.]</mark>  <rj><au>Milton.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Surmise</ent><br/
<hw>Sur*mise"</hw> <pr>(s<ucir/r*m<imac/z")</pr>, <pos>n.</pos> <ety>[OF. <ets>surmise</ets> accusation, fr. <ets>surmettre</ets>, p. p. <ets>surmis</ets>, to impose, accuse; <ets>sur</ets> (see <er>Sur-</er>) + <ets>mettre</ets> to put, set, L. <ets>mittere</ets> to send. See <er>Mission</er>.]</ety> <sn>1.</sn> <def>A thought, imagination, or conjecture, which is based upon feeble or scanty evidence; suspicion; guess; <as>as, the <ex>surmises</ex> of jealousy or of envy</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>[We] double honor gain<br/
From his <qex>surmise</qex> proved false.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>No man ought to be charged with principles he actually disowns, unless his practicies contradict his profession; not upon small <qex>surmises</qex>.</q> <rj><qau>Swift.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Reflection; thought.</def> <mark>[Obs.]</mark>  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><syn><b>Syn.</b> -- Conjecture; supposition; suspicion; doubt.</syn><br/
[<source>1913 Webster</source>]</p>

<p><ent>Surmise</ent><br/
<hw>Sur*mise"</hw>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Surmised</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Surmising</conjf>.]</vmorph> <def>To imagine without certain knowledge; to infer on slight grounds; to suppose, conjecture, or suspect; to guess.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>It wafted nearer yet, and then she knew<br/
That what before she but <qex>surmised</qex>, was true.</q> <rj><qau>Dryden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>This change was not wrought by altering the form or position of the earth, as was <qex>surmised</qex> by a very learned man, but by dissolving it.</q> <rj><qau>Woodward.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Surmiser</ent><br/
<hw>Sur*mis"er</hw> <pr>(?)</pr>, <pos>n.</pos> <def>One who surmises.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Surmising</ent><br/
<hw>Sur*mis"ing</hw>, <def><pos>a. & n.</pos> from <er>Surmise</er>, <pos>v.</pos></def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Surmount</ent><br/
<hw>Sur*mount"</hw> <pr>(?)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Surmounted</conjf>; <pos>p. pr. & vb. n.</pos> <conjf>Surmounting</conjf>.]</vmorph> <ety>[OE. <ets>sourmounten</ets>, OF. <ets>surmonter</ets>, <ets>sormonter</ets>, F. <ets>surmonter</ets>; <ets>sur</ets> over + <ets>monter</ets> to mount. See <er>Sur-</er>, and <er>Mount</er>, <pos>v. i.</pos>]</ety> <sn>1.</sn> <def>To rise above; to be higher than; to overtop.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The mountains of Olympus, Athos, and Atlas, overreach and <qex>surmount</qex> all winds and clouds.</q> <rj><qau>Sir W. Raleigh.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To conquer; to overcome; <as>as, to <ex>surmount</ex> difficulties or obstacles</as>.</def>  <rj><au>Macaulay.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>To surpass; to exceed.</def>  <rj><au>Spenser.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>What <qex>surmounts</qex> the reach<br/
Of human sense I shall delineate.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><syn><b>Syn.</b> -- To conquer; overcome; vanquish; subdue; surpass; exceed.</syn><br/
[<source>1913 Webster</source>]</p>

<p><ent>Surmountable</ent><br/
<hw>Sur*mount"a*ble</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[Cf. F. <ets>surmontable</ets>.]</ety> <def>Capable of being surmounted or overcome; superable.</def> -- <wordforms><wf>Sur*mount"a*ble*ness</wf>, <pos>n.</pos></wordforms><br/
[<source>1913 Webster</source>]</p>

<p><ent>Surmounted</ent><br/
<hw>Sur*mount"ed</hw>, <pos>a.</pos> <sn>1.</sn> <fld>(Arch.)</fld> <def>Having its vertical height greater than the half span; -- said of an arch.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Her.)</fld> <def>Partly covered by another charge; -- said of an ordinary or other bearing.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Surmounter</ent><br/
<hw>Sur*mount"er</hw> <pr>(?)</pr>, <pos>n.</pos> <def>One who, or that which, surmounts.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Surmullet</ent><br/
<hw>Sur*mul"let</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F. <ets>surmulet</ets>; <ets>saur</ets>, <ets>saure</ets>, brownish yellow, red + <ets>mulet</ets> a mullet. See <er>Sorrel</er>, <pos>a.</pos>, and <er>Mullet</er>.]</ety> <fld>(Zool.)</fld> <def>Any one of various species of mullets of the family <fam>Millidae</fam>, esp. the European species (<spn>Millus surmulletus</spn>), which is highly prized as a food fish. See <er>Mullet</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Surmulot</ent><br/
<hw>Sur"mu*lot</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F.]</ety> <fld>(Zool.)</fld> <def>The brown, or Norway, rat.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Surname</ent><br/
<hw>Sur"name`</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Pref. <ets>sur + name</ets>; really a substitution for OE. <ets>sournoun</ets>, from F. <ets>surnom</ets>. See <er>Sur-</er>, and <er>Noun</er>, <er>Name</er>.]</ety> <sn>1.</sn> <def>A name or appellation which is added to, or over and above, the baptismal or Christian name, and becomes a family name.</def><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ Surnames originally designated occupation, estate, place of residence, or some particular thing or event that related to the person; thus, Edmund <xex>Ironsides</xex>; Robert <xex>Smith</xex>, or the <xex>smith</xex>; William <xex>Turner</xex>. Surnames are often also patronymics; as, John <xex>Johnson</xex>.</note><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>An appellation added to the original name; an agnomen.</def> <ldquo/My <xex>surname</xex>, Coriolanus.<rdquo/  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ This word has been sometimes written <xex>sirname</xex>, as if it signified <xex>sire</xex>-name, or the name derived from one's father.</note><br/
[<source>1913 Webster</source>]</p>

<p><ent>Surname</ent><br/
<hw>Sur*name"</hw> <pr>(?)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Surnamed</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Surnaming</conjf>.]</vmorph> <ety>[Cf. F. <ets>surnommer</ets>.]</ety> <def>To name or call by an appellation added to the original name; to give a surname to.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Another shall subscribe with his hand unto the Lord, and <qex>surname</qex> himself by the name of Israel.</q> <rj><qau>Isa. xliv. 5.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>And Simon he <qex>surnamed</qex> Peter.</q> <rj><qau>Mark iii. 16.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Surnominal</ent><br/
<hw>Sur*nom"i*nal</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Of or pertaining to a surname or surnames.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Suroxidate</ent><br/
<hw>Sur*ox"i*date</hw> <pr>(?)</pr>, <pos>v. t.</pos> <fld>(Chem.)</fld> <def>To combine with oxygen so as to form a suroxide or peroxide.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Suroxide</ent><br/
<hw>Sur*ox"ide</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Cf. F. <ets>suroxyde</ets>. See <er>Sur-</er>, and <er>Oxide</er>.]</ety> <fld>(Chem.)</fld> <def>A peroxide.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Surpass</ent><br/
<hw>Sur*pass"</hw> <pr>(?)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Surpassed</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Surpassing</conjf>.]</vmorph> <ety>[F. <ets>surpasser</ets>; <ets>sur</ets> over + <ets>passer</ets> to pass. See <er>Sur-</er>, and <er>Pass</er>.]</ety> <def>To go beyond in anything good or bad; to perform (an activity) better than; to exceed; to excel.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>This would <qex>surpass</qex><br/
Common revenge and interrupt his joy.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><syn><b>Syn.</b> -- To exceed; excel; outdo; outstrip.</syn><br/
[<source>1913 Webster</source>]</p>

<p><ent>Surpassable</ent><br/
<hw>Sur*pass"a*ble</hw> <pr>(?)</pr>, <pos>a.</pos> <def>That may be surpassed.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Surpassing</ent><br/
<hw>Sur*pass"ing</hw>, <pos>a.</pos> <def>Eminently excellent; exceeding others.</def> <ldquo/With <xex>surpassing</xex> glory crowned.<rdquo/ <au>Milton.</au> -- <wordforms><wf>Sur*pass"ing*ly</wf>, <pos>adv.</pos> -- <wf>Sur*pass"ing*ness</wf>, <pos>n.</pos></wordforms><br/
[<source>1913 Webster</source>]</p>

<p><ent>Surphul</ent><br/
<hw>Sur"phul</hw> <pr>(?)</pr>, <pos>v. t.</pos> <def>To surfel.</def> <mark>[Obs.]</mark>  <rj><au>Marston.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Surplice</ent><br/
<hw>Sur"plice</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F. <ets>surplis</ets>, OF. <ets>surpeiz</ets>, LL. <ets>superpellicium</ets>; <ets>super</ets> over + <ets>pellicium</ets>, <ets>pelliceum</ets>, a robe of fur, L. <ets>pellicius</ets> made of skins. See <er>Pelisse</er>.]</ety> <fld>(Eccl.)</fld> <def>A white garment worn over another dress by the clergy of the Roman Catholic, Episcopal, and certain other churches, in some of their ministrations.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Surplice fees</b></col> <fld>(Eccl.)</fld>, <cd>fees paid to the English clergy for occasional duties.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Surpliced</ent><br/
<hw>Sur"pliced</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Wearing a surplice.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Surplus</ent><br/
<hw>Sur"plus</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F., fr. <ets>sur</ets> over + <ets>plus</ets> more. See <er>Sur-</er>, and <er>Plus</er>, and cf. <er>Superplus</er>.]</ety> <sn>1.</sn> <def>That which remains when use or need is satisfied, or when a limit is reached; excess; overplus.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Specifically, an amount in the public treasury at any time greater than is required for the ordinary purposes of the government.</def><-- = budgetary surplus --><br/
[<source>1913 Webster</source>]</p>

<p><ent>Surplus</ent><br/
<hw>Sur"plus</hw>, <pos>a.</pos> <def>Being or constituting a surplus; more than sufficient; <as>as, <ex>surplus</ex> revenues; <ex>surplus</ex> population; <ex>surplus</ex> words.</as></def><br/
[<source>1913 Webster</source>]</p>

<p><q>When the price of corn falleth, men give over <qex>surplus</qex> tillage, and break no more ground.</q> <rj><qau>Carew.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Surplusage</ent><br/
<hw>Sur"plus*age</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[See <er>Surplus</er>, and cf. <er>Superplusage</er>.]</ety> <sn>1.</sn> <def>Surplus; excess; overplus; <as>as, <ex>surplusage</ex> of grain or goods beyond what is wanted</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Take what thou please of all this <qex>surplusage</qex>.</q> <rj><qau>Spenser.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>A <qex>surplusage</qex> given to one part is paid out of a reduction from another part of the same creature.</q> <rj><qau>Emerson.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Law)</fld> <def>Matter in pleading which is not necessary or relevant to the case, and which may be rejected.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <fld>(Accounts)</fld> <def>A greater disbursement than the charge of the accountant amounts to.</def> <mark>[Obs.]</mark>  <rj><au>Rees.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Surprisal</ent><br/
<hw>Sur*pris"al</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[See <er>Surprise</er>, <pos>n.</pos>]</ety> <def>The act of surprising, or state of being surprised; surprise.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>How to secure the lady from <qex>surprisal</qex>.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Because death is uncertain, let us prevent its <qex>surprisal</qex>.</q> <rj><qau>Barrow.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Surprise</ent><br/
<hw>Sur*prise"</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F. <ets>surprise</ets>, fr. <ets>surprendre</ets>, <ets>surpris</ets>; <ets>sur</ets> over + <ets>prendre</ets> to take, L. <ets>prehendere</ets>. See <er>Sur-</er>, and <er>Prehensile</er>.]</ety> <sn>1.</sn> <def>The act of coming upon, or taking, unawares; the act of seizing unexpectedly; surprisal; <as>as, the fort was taken by <ex>surprise</ex></as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>The state of being surprised, or taken unawares, by some act or event which could not reasonably be foreseen; emotion excited by what is sudden and strange; a suddenly excited feeling of wonder or astonishment.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Pure <qex>surprise</qex> and fear<br/
Made me to quit the house.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>Anything that causes such a state or emotion.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>A dish covered with a crust of raised paste, but with no other contents.</def> <mark>[Obs.]</mark>  <rj><au>King.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Surprise party</b></col>, <cd>a party of persons who assemble by mutual agreement, and without invitation, at the house of a common friend.</cd> <mark>[U.S.]</mark> <au>Bartlett.</au></cs><br/
[<source>1913 Webster</source>]</p>

<p><syn><b>Syn.</b> -- Wonder; astonishment; amazement.</syn><br/
[<source>1913 Webster</source>]</p>

<p><ent>Surprise</ent><br/
<hw>Sur*prise"</hw> <pr>(?)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Surprised</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Surprising</conjf>.]</vmorph> <ety>[From <er>Surprise</er>, <pos>n.</pos>: cf. F. <ets>surprendre</ets>, p. p. <ets>surpris</ets>.]</ety> <sn>1.</sn> <def>To come or fall suddenly and unexpectedly; to take unawares; to seize or capture by unexpected attack.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Fearfulness hath <qex>surprised</qex> the hypocrites.</q> <rj><qau>Isa. xxxiii. 14.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>The castle of Macduff I will <qex>surprise</qex>.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Who can speak<br/
The mingled passions that <qex>surprised</qex> his heart?</q> <rj><qau>Thomson.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To strike with wonder, astonishment, or confusion, by something sudden, unexpected, or remarkable; to confound; <as>as, his conduct <ex>surprised</ex> me</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>I am <qex>surprised</qex> with an uncouth fear.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Up he starts,<br/
Discovered and <qex>surprised</qex>.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>To lead (one) to do suddenly and without forethought; to bring (one) into some unexpected state; -- with <xex>into</xex>; <as>as, to be <ex>surprised</ex> into an indiscretion; to be <ex>surprised</ex> into generosity</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>To hold possession of; to hold.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>Not with me,<br/
That in my hands <qex>surprise</qex> the sovereignity.</q> <rj><qau>J. Webster.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><syn><b>Syn.</b> -- See <er>Astonish</er>.</syn><br/
[<source>1913 Webster</source>]</p>

<p><ent>Surprisement</ent><br/
<hw>Sur*prise"ment</hw> <pr>(?)</pr>, <pos>n.</pos> <def>Surprisal.</def> <mark>[Obs.]</mark>  <rj><au>Daniel.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Surpriser</ent><br/
<hw>Sur*pris"er</hw> <pr>(?)</pr>, <pos>n.</pos> <def>One who surprises.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Surprising</ent><br/
<hw>Sur*pris"ing</hw>, <pos>a.</pos> <def>Exciting surprise; extraordinary; of a nature to excite wonder and astonishment; <as>as, <ex>surprising</ex> bravery; a <ex>surprising</ex> escape from danger.</as></def> -- <wordforms><wf>Sur*pris"ing*ly</wf>, <pos>adv.</pos> -- <wf>Sur*pris"ing*ness</wf>, <pos>n.</pos></wordforms><br/
[<source>1913 Webster</source>]</p>

<p><syn><b>Syn.</b> -- Wonderful; extraordinary; unexpected; astonishing; striking.</syn><br/
[<source>1913 Webster</source>]</p>

<p><ent>Surquedrous</ent><br/
<ent>Surquedous</ent><br/
<mhw>{ <hw>Sur"que*dous</hw> <pr>(?)</pr>, <hw>Sur"que*drous</hw> <pr>(?)</pr>, }</mhw> <pos>a.</pos> <def>Having or exhibiting surquedry; arrogant; insolent.</def> <mark>[Obs.]</mark>  <rj><au>Gower. James II. of Scot.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Surquidry</ent><br/
<ent>Surquedry</ent><br/
<mhw>{ <hw>Sur"que*dry</hw> <pr>(?)</pr>, <hw>Sur"qui*dry</hw>  }</mhw>, <pos>n.</pos> <ety>[OF. <ets>surcuidier</ets> to presume; <ets>sur</ets> over + <ets>cuidier</ets> to think, L. <ets>cogitare</ets>. See <er>Sur-</er>, and <er>Cogitate</er>.]</ety> <def>Overweening pride; arrogance; presumption; insolence.</def> <mark>[Obs.]</mark>  <rj><au>Chaucer.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Then pay you the price of your <qex>surquedry</qex>.</q> <rj><qau>Spenser.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Surrebound</ent><br/
<hw>Sur`re*bound"</hw> <pr>(?)</pr>, <pos>v. i.</pos> <def>To give back echoes; to reecho.</def> <mark>[Obs.]</mark>  <rj><au>Chapman.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Surrebut</ent><br/
<hw>Sur`re*but"</hw> <pr>(?)</pr>, <pos>v. i.</pos> <ety>[Pref. <ets>sur + rebut</ets>.]</ety> <fld>(Law)</fld> <def>To reply, as a plaintiff to a defendant's rebutter.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Surrebuter</ent><br/
<hw>Sur`re*but"er</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Law)</fld> <def>The reply of a plaintiff to a defendant's rebutter.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Surrein</ent><br/
<hw>Sur"rein`</hw> <pr>(?)</pr>, <pos>v. t.</pos> <ety>[Pref. <ets>sur + rein</ets>.]</ety> <def>To override; to exhaust by riding.</def> <mark>[Obs.]</mark>  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Surrejoin</ent><br/
<hw>Sur`re*join"</hw> <pr>(?)</pr>, <pos>v. i.</pos> <ety>[Pref. <ets>sur + rejoin</ets>.]</ety> <fld>(Law)</fld> <def>To reply, as a plaintiff to a defendant's rejoinder.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Surrejoinder</ent><br/
<hw>Sur`re*join"der</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Law)</fld> <def>The answer of a plaintiff to a defendant's rejoinder.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Surrender</ent><br/
<hw>Sur*ren"der</hw> <pr>(?)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Surrendered</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Surrendering</conjf>.]</vmorph> <ety>[OF. <ets>surrendre</ets> to deliver; <ets>sur</ets> over + <ets>rendre</ets> to render. See <er>Sur-</er>, and <er>Render</er>.]</ety> <sn>1.</sn> <def>To yield to the power of another; to give or deliver up possession of (anything) upon compulsion or demand; <as>as, to <ex>surrender</ex> one's person to an enemy or to an officer; to <ex>surrender</ex> a fort or a ship</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To give up possession of; to yield; to resign; <as>as, to <ex>surrender</ex> a right, privilege, or advantage</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>To <qex>surrender</qex> up that right which otherwise their founders might have in them.</q> <rj><qau>Hooker.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>To yield to any influence, emotion, passion, or power; -- used reflexively; <as>as, to <ex>surrender</ex> one's self to grief, to despair, to indolence, or to sleep</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <fld>(Law)</fld> <def>To yield; to render or deliver up; to give up; <as>as, a principal <ex>surrendered</ex> by his bail, a fugitive from justice by a foreign state, or a particular estate by the tenant thereof to him in remainder or reversion</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Surrender</ent><br/
<hw>Sur*ren"der</hw>, <pos>v. i.</pos> <def>To give up one's self into the power of another; to yield; <as>as, the enemy, seeing no way of escape, <ex>surrendered</ex> at the first summons</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Surrender</ent><br/
<hw>Sur*ren"der</hw>, <pos>n.</pos> <sn>1.</sn> <def>The act of surrendering; the act of yielding, or resigning one's person, or the possession of something, into the power of another; <as>as, the <ex>surrender</ex> of a castle to an enemy; the <ex>surrender</ex> of a right</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>That he may secure some liberty he makes a <qex>surrender</qex> in trust of the whole of it.</q> <rj><qau>Burke.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Law)</fld> <sd>(a)</sd> <def>The yielding of a particular estate to him who has an immediate estate in remainder or reversion.</def> <sd>(b)</sd> <def>The giving up of a principal into lawful custody by his bail.</def> <sd>(c)</sd> <def>The delivery up of fugitives from justice by one government to another, as by a foreign state. See <er>Extradition</er>.</def>  <rj><au>Wharton.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <fld>(Insurance)</fld> <def>The voluntary cancellation of the legal liability of the company by the insured and beneficiary for a consideration (called the <col><b>surrender value</b></col>).</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Surrenderee</ent><br/
<hw>Sur*ren`der*ee"</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Law)</fld> <def>The person to whom a surrender is made.</def>  <rj><au>Mozley & W.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Surrenderer</ent><br/
<hw>Sur*ren"der*er</hw> <pr>(?)</pr>, <pos>n.</pos> <def>One who surrenders.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Surrenderor</ent><br/
<hw>Sur*ren`der*or"</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Law)</fld> <def>One who makes a surrender, as of an estate.</def>  <rj><au>Bouvier.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Surrendry</ent><br/
<hw>Sur*ren"dry</hw> <pr>(?)</pr>, <pos>n.</pos> <def>Surrender.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Surreption</ent><br/
<hw>Sur*rep"tion</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>surreptio</ets>, or <ets>subreptio</ets>.  Cf. <er>Subreption</er>.]</ety> <sn>1.</sn> <def>The act or process of getting in a surreptitious manner, or by craft or stealth.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Fame by <qex>surreption</qex> got<br/
May stead us for the time, but lasteth not.</q> <rj><qau>B. Jonson.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A coming unperceived or suddenly.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Surreptitious</ent><br/
<hw>Sur`rep*ti"tious</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>surreptitius</ets>, or <ets>subreptitius</ets>, fr. <ets>surripere</ets>, <ets>subripere</ets>, to snatch away, to withdraw privily; <ets>sub-</ets> under + <ets>rapere</ets> to snatch. See <er>Sub-</er>, and <er>Ravish</er>.]</ety> <def>Done or made by stealth, or without proper authority; made or introduced fraudulently; clandestine; stealthy; <as>as, a <ex>surreptitious</ex> passage in an old manuscript; a <ex>surreptitious</ex> removal of goods</as>.</def> -- <wordforms><wf>Sur`rep*ti"tious*ly</wf>, <pos>adv.</pos></wordforms><br/
[<source>1913 Webster</source>]</p>

<p><ent>Surrey</ent><br/
<hw>Sur"rey</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A four-wheeled pleasure carriage, (commonly two-seated) somewhat like a phaeton, but having a straight bottom.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Surrogate</ent><br/
<hw>Sur"ro*gate</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>surrogatus</ets>, p. p. of <ets>surrogare</ets>, <ets>subrogare</ets>, to put in another's place, to substitute; <ets>sub</ets> under + <ets>rogare</ets> to ask, ask for a vote, propose a law. See <er>Rogation</er>, and cf. <er>Subrogate</er>.]</ety> <sn>1.</sn> <def>A deputy; a delegate; a substitute.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>The deputy of an ecclesiastical judge, most commonly of a bishop or his chancellor, especially a deputy who grants marriage licenses.</def> <mark>[Eng.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>In some States of the United States, an officer who presides over the probate of wills and testaments and yield the settlement of estates.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>a <er>surrogate mother</er>.</def><br/
[<source>PJC</source>]</p>

<p><-- p. 1453 --></p>

<p><ent>Surrogate</ent><br/
<hw>Sur"ro*gate</hw> <pr>(?)</pr>, <pos>v. t.</pos> <def>To put in the place of another; to substitute.</def> <mark>[R.]</mark>  <rj><au>Dr. H. More.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Surrogate mother</ent><br/
<hw>Sur"ro*gate mother</hw> <pr>(?)</pr>, <pos>n.</pos> <sn>1.</sn> <def>a person or who serves in place of the biological mother for child, as for caring or nurturing.</def><br/
[<source>PJC</source>]</p>

<p><sn>2.</sn> <def>a female animal that raises and nurses the offspring of another animal.</def><br/
[<source>PJC</source>]</p>

<p><sn>3.</sn> <def>a woman who becomes pregnant by an artificial procedure (one other than copulation), and bears a child to be raised by another person. The person on whose behalf the child is gestated and born is usually one of the biological parents.  The embryo carried by the surrogate mother may be created by <it>in vitro</it> fertilization, and may have no genetic material from the surrogate mother, or the surrogate mother may be impregnated by artificial insemination, usually by sperm donated by the father who is to raise the child.</def>  <note>The procedure is common but controversial, and in some states of the United States, contracts for surrogacy are not considered enforceable.</note><br/
[<source>PJC</source>]</p>

<p><ent>Surrogateship</ent><br/
<hw>Sur"ro*gate*ship</hw>, <pos>n.</pos> <def>The office of a surrogate.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Surrogation</ent><br/
<hw>Sur`ro*ga"tion</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[See <er>Surrogate</er>, <pos>n.</pos>, and cf. <er>Subrogation</er>.]</ety> <def>The act of substituting one person in the place of another.</def> <mark>[R.]</mark>  <rj><au>Killingbeck.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Surround</ent><br/
<hw>Sur*round"</hw> <pr>(?)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Surrounded</conjf>; <pos>p. pr. & vb. n.</pos> <conjf>Surrounding</conjf>.]</vmorph> <ety>[OF. <ets>suronder</ets> to overflow, LL. <ets>superundare</ets>; fr. L. <ets>super</ets> over + <ets>undare</ets> to rise in waves, overflow, fr. <ets>unda</ets> wave. The English sense is due to the influence of E. <ets>round</ets>. See <er>Super-</er>, and <er>Undulate</er>, and cf. <er>Abound</er>.]</ety> <sn>1.</sn> <def>To inclose on all sides; to encompass; to environ.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To lie or be on all sides of; to encircle; <as>as, a wall <ex>surrounds</ex> the city</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>But could instead, and ever-during dark<br/
<qex>Surrounds</qex> me.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>To pass around; to travel about; to circumnavigate; <as>as, to <ex>surround</ex> the world</as>.</def> <mark>[Obs.]</mark>  <rj><au>Fuller.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <fld>(Mil.)</fld> <def>To inclose, as a body of troops, between hostile forces, so as to cut off means of communication or retreat; to invest, as a city.</def><br/
[<source>1913 Webster</source>]</p>

<p><syn><b>Syn.</b> -- To encompass; encircle; environ; invest; hem in; fence about.</syn><br/
[<source>1913 Webster</source>]</p>

<p><ent>Surround</ent><br/
<hw>Sur*round"</hw>, <pos>n.</pos> <def>A method of hunting some animals, as the buffalo, by surrounding a herd, and driving them over a precipice, into a ravine, etc.</def> <mark>[U.S.]</mark>  <rj><au>Baird.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Surrounding</ent><br/
<hw>Sur*round"ing</hw>, <pos>a.</pos> <def>Inclosing; encircling.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Surrounding</ent><br/
<hw>Sur*round"ing</hw>, <pos>n.</pos> <sn>1.</sn> <def>An encompassing.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <pluf>pl.</pluf> <def>The things which surround or environ; external or attending circumstances or conditions.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Surroyal</ent><br/
<hw>Sur*roy"al</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Pref. <ets>sur-</ets> + <ets>royal</ets>.]</ety> <fld>(Zool.)</fld> <def>One of the terminal branches or divisions of the beam of the antler of the stag or other large deer.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sursanure</ent><br/
<hw>Sur"sa*nure</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[(Assumed) OF. <ets>sursane<uum/re</ets>. See <er>Sur-</er>, and <er>Sane</er>.]</ety> <def>A wound healed or healing outwardly only.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>Of a <qex>sursanure</qex><br/
In surgery is perilous the cure.</q> <rj><qau>Chaucer.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Surseance</ent><br/
<hw>Sur"se*ance</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[OF., fr. OF. & F. <ets>surseoir</ets>. See <er>Surcease</er>.]</ety> <def>Peace; quiet.</def> <mark>[Obs.]</mark>  <rj><au>Bacon.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sursolid</ent><br/
<hw>Sur*sol"id</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F. <ets>sursolide</ets>. See <er>Sur-</er>, and <er>Solid</er>.]</ety> <fld>(Math.)</fld> <def>The fifth power of a number; <as>as, <ex>a<?/</ex> is the <ex>sursolid</ex> of <ex>a</ex>, or 32 that of 2</as>.</def> <mark>[R.]</mark>  <rj><au>Hutton.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Surstyle</ent><br/
<hw>Sur*style"</hw> <pr>(?)</pr>, <pos>v. t.</pos> <def>To surname.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sursum corda</ent><br/
||<hw>Sur"sum cor"da</hw> <pr>(?)</pr>. <ety>[L. <ets>sursum</ets> upward + <ets>corda</ets> hearts.]</ety> <fld>(Eccl.)</fld> <def>In the Eucharist, the versicles immediately before the preface, inviting the people to join in the service by <ldquo/lifting up the heart<rdquo/ to God.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Surtax</ent><br/
<hw>Sur"tax</hw> <pr>(?)</pr>, <pos>n.</pos> <def>An additional or extra tax.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Surtax</ent><br/
<hw>Sur*tax"</hw> <pr>(?)</pr>, <pos>v. t.</pos> <def>To impose an additional tax on.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Surtout</ent><br/
<hw>Sur*tout"</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F., fr. <ets>sur</ets> over + <ets>tout</ets> all.]</ety> <def>A man's coat to be worn over his other garments; an overcoat, especially when long, and fitting closely like a body coat.</def>  <rj><au>Gay.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Surturbrand</ent><br/
<hw>Sur"tur*brand</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Icel. <ets>surtarbrandr</ets>; <ets>svartr</ets> black + <ets>brandr</ets> a firebrand.]</ety> <def>A fibrous brown coal or bituminous wood.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Surucucu</ent><br/
<hw>Su`ru*cu"cu</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Zool.)</fld> <def>See <cref>Bush master</cref>, under <er>Bush</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Surveillance</ent><br/
<hw>Sur*veil"lance</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F., fr. <ets>surveiller</ets> to watch over; <ets>sur</ets> over + <ets>veiller</ets> to watch, L. <ets>vigilare</ets>. See <er>Sur-</er>, and <er>Vigil</er>.]</ety> <def>Oversight; watch; inspection; supervision.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>That sort of <qex>surveillance</qex> of which . . . the young have accused the old.</q> <rj><qau>Sir W. Scott.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Surveillant</ent><br/
<hw>Sur*veil"lant</hw> <pr>(?)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Surveillants</plw> <pr>(#)</pr>.</plu> <ety>[F., fr. <ets>surveiller</ets> to watch over. See <er>Surveillance</er>.]</ety> <def>One who watches over another; an overseer; a spy; a supervisor.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Surveillant</ent><br/
<hw>Sur*veil"lant</hw>, <pos>a.</pos> <def>Overseeing; watchful.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Survene</ent><br/
<hw>Sur*vene"</hw> <pr>(?)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Survened</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Survening</conjf>.]</vmorph> <ety>[F. <ets>survenir</ets>. See <er>Supervene</er>.]</ety> <def>To supervene upon; to come as an addition to.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>A suppuration that <qex>survenes</qex> lethargies.</q> <rj><qau>Harvey.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Survenue</ent><br/
<hw>Sur"ve*nue</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[OF. See <er>Survene</er>.]</ety> <def>A sudden or unexpected coming or stepping on.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Survey</ent><br/
<hw>Sur*vey"</hw> <pr>(?)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Surveyed</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Surveying</conjf>.]</vmorph> <ety>[OF. <ets>surveoir</ets>, <ets>surveer</ets>; <ets>sur</ets>, <ets>sor</ets>, over, E. <ets>sur + veoir</ets>, <ets>veeir</ets>, to see, F. <ets>voir</ets>, L. <ets>videre</ets>. See <er>Sur-</er>, and <er>Vision</er>, and cf. <er>Supervise</er>.]</ety> <sn>1.</sn> <def>To inspect, or take a view of; to view with attention, as from a high place; to overlook; <as>as, to stand on a hill, and <ex>survey</ex> the surrounding country</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Round he <qex>surveys</qex> and well might, where he stood,<br/
So high above.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To view with a scrutinizing eye; to examine.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>With such altered looks, . . . <br/
All pale and speechless, he <qex>surveyed</qex> me round.</q> <rj><qau>Dryden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>To examine with reference to condition, situation, value, etc.; to examine and ascertain the state of; <as>as, to <ex>survey</ex> a building in order to determine its value and exposure to loss by fire</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>To determine the form, extent, position, etc., of, as a tract of land, a coast, harbor, or the like, by means of linear and angular measurments, and the application of the principles of geometry and trigonometry; <as>as, to <ex>survey</ex> land or a coast</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <def>To examine and ascertain, as the boundaries and royalties of a manor, the tenure of the tenants, and the rent and value of the same.</def> <mark>[Eng.]</mark>  <rj><au>Jacob (Law Dict.).</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Survey</ent><br/
<hw>Sur"vey</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Formerly accentuated universally on the last syllable, and still so accented by many speakers.]</ety> <sn>1.</sn> <def>The act of surveying; a general view, as from above.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Under his proud <qex>survey</qex> the city lies.</q> <rj><qau>Sir J. Denham.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A particular view; an examination, especially an official examination, of all the parts or particulars of a thing, with a design to ascertain the condition, quantity, or quality; <as>as, a <ex>survey</ex> of the stores of a ship; a <ex>survey</ex> of roads and bridges; a <ex>survey</ex> of buildings</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>The operation of finding the contour, dimensions, position, or other particulars of, as any part of the earth's surface, whether land or water; also, a measured plan and description of any portion of country, or of a road or line through it.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Survey of dogs</b></col>. <cd>See <cref>Court of regard</cref>, under <er>Regard</er>.</cd> -- <col><b>Trigonometrical survey</b></col>, <cd>a survey of a portion of country by measuring a single base, and connecting it with various points in the tract surveyed by a series of triangles, the angles of which are carefully measured, the relative positions and distances of all parts being computed from these data.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><syn><b>Syn.</b> -- Review; retrospect; examination; prospect.</syn><br/
[<source>1913 Webster</source>]</p>

<p><ent>Surveyal</ent><br/
<hw>Sur*vey"al</hw> <pr>(?)</pr>, <pos>n.</pos> <def>Survey.</def> <mark>[R.]</mark>  <rj><au>Barrow.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Surveyance</ent><br/
<hw>Sur*vey"ance</hw> <pr>(?)</pr>, <pos>n.</pos> <def>Survey; inspection.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Surveying</ent><br/
<hw>Sur*vey"ing</hw>, <pos>n.</pos> <def>That branch of applied mathematics which teaches the art of determining the area of any portion of the earth's surface, the length and directions of the bounding lines, the contour of the surface, etc., with an accurate delineation of the whole on paper; the act or occupation of making surveys.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Geodetic surveying</b></col>, <cd>geodesy.</cd> -- <mcol><col><b>Maritime surveying</b></col>, <it>or</it> <col><b>Nautical surveying</b></col></mcol>, <cd>that branch of surveying which determines the forms of coasts and harbors, the entrances of rivers, with the position of islands, rocks, and shoals, the depth of water, etc.</cd> -- <col><b>Plane surveying</b></col>. <cd>See under <er>Plane</er>, <pos>a.</pos></cd> -- <col><b>Topographical surveying</b></col>, <cd>that branch of surveying which involves the process of ascertaining and representing upon a plane surface the contour, physical features, etc., of any portion of the surface of the earth.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Surveyor</ent><br/
<hw>Sur*vey"or</hw> <pr>(?)</pr>, <pos>n.</pos> <sn>1.</sn> <def>One placed to superintend others; an overseer; an inspector.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Were 't not madness then,<br/
To make the fox <qex>surveyor</qex> of the fold?</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>One who views and examines for the purpose of ascertaining the condition, quantity, or quality of anything; <as>as, a <ex>surveyor</ex> of highways, ordnance, etc.</as></def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>One who surveys or measures land; one who practices the art of surveying.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <fld>(Customs)</fld> <sd>(a)</sd> <def>An officer who ascertains the contents of casks, and the quantity of liquors subject to duty; a gauger.</def> <sd>(b)</sd> <def>In the United States, an officer whose duties include the various measures to be taken for ascertaining the quantity, condition, and value of merchandise brought into a port.</def> <au>Abbot.</au><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Surveyor general</b></col>. <sd>(a)</sd> <cd>A principal surveyor; <as>as, the <ex>surveyor general</ex> of the king's manors, or of woods and parks</as>.</cd> <mark>[Eng.]</mark> <sd>(b)</sd> <cd>An officer having charge of the survey of the public lands of a land district.</cd> <mark>[U.S.]</mark> <au>Davies & Peck (Math. Dict.).</au> -- <col><b>Surveyor's compass</b></col>. <cd>See <er>Circumferentor</er>.</cd> -- <col><b>Surveyor's level</b></col>. <cd>See under <er>Level</er>.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Surveyorship</ent><br/
<hw>Sur*vey"or*ship</hw>, <pos>n.</pos> <def>The office of a surveyor.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Surview</ent><br/
<hw>Sur*view"</hw> <pr>(?)</pr>, <pos>v. t.</pos> <ety>[Pref. <ets>sur-</ets> + <ets>view</ets>.  Cf. <er>Survey</er>.]</ety> <def>To survey; to make a survey of.</def> <mark>[Obs.]</mark> <ldquo/To <xex>surview</xex> his ground.<rdquo/  <rj><au>Spenser.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Surview</ent><br/
<hw>Sur*view"</hw>, <pos>n.</pos> <def>A survey.</def> <mark>[Obs.]</mark>  <rj><au>Bp. Sanderson.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Survise</ent><br/
<hw>Sur*vise"</hw> <pr>(?)</pr>, <pos>v. t.</pos> <ety>[See <er>Supervise</er>, and <er>Survey</er>.]</ety> <def>To look over; to supervise.</def> <mark>[Obs.]</mark>  <rj><au>B. Jonson.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Survival</ent><br/
<hw>Sur*viv"al</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[From <er>Survive</er>.]</ety> <sn>1.</sn> <def>A living or continuing longer than, or beyond the existence of, another person, thing, or event; an outliving.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Arhaeol. & Ethnol.)</fld> <def>Any habit, usage, or belief, remaining from ancient times, the origin of which is often unknown, or imperfectly known.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The close bearing of the doctrine of <qex>survival</qex> on the study of manners and customs.</q> <rj><qau>Tylor.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Survival of the fittest</b></col>. <fld>(Biol.)</fld> <cd>See <cref>Natural selection</cref>, under <er>Natural</er>.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Survivancy</ent><br/
<ent>Survivance</ent><br/
<mhw>{ <hw>Sur*viv"ance</hw> <pr>(?)</pr>, <hw>Sur*viv"an*cy</hw> <pr>(?)</pr>, }</mhw> <pos>n.</pos> <ety>[F. <ets>survivance</ets>.]</ety> <def>Survivorship.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>His son had the <qex>survivance</qex> of the stadtholdership.</q> <rj><qau>Bp. Burnet.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Survive</ent><br/
<hw>Sur*vive"</hw> <pr>(?)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Survived</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Surviving</conjf>.]</vmorph> <ety>[F. <ets>survivre</ets>, L. <ets>supervivere</ets>; <ets>super</ets> over + <ets>vivere</ets> to live. See <er>Super-</er>, and <er>Victuals</er>.]</ety> <def>To live beyond the life or existence of; to live longer than; to outlive; to outlast; <as>as, to <ex>survive</ex> a person or an event</as>.</def>  <rj><au>Cowper.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p>
<q>I'll assure her of<br/
Her widowhood, be it that she <qex>survive</qex> me,<br/
In all my lands and leases whatsoever.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Survive</ent><br/
<hw>Sur*vive"</hw>, <pos>v. i.</pos> <def>To remain alive; to continue to live.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Thy pleasure,<br/
Which, when no other enemy <qex>survives</qex>,<br/
Still conquers all the conquerors.</q> <rj><qau>Sir J. Denham.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Alike are life and death,<br/
When life in death <qex>survives</qex>.</q> <rj><qau>Longfellow.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Survivency</ent><br/
<hw>Sur*viv"en*cy</hw> <pr>(?)</pr>, <pos>n.</pos> <def>Survivorship.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Surviver</ent><br/
<hw>Sur*viv"er</hw> <pr>(?)</pr>, <pos>n.</pos> <def>One who survives; a survivor.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Surviving</ent><br/
<hw>Sur*viv"ing</hw>, <pos>a.</pos> <def>Remaining alive; yet living or existing; <as>as, <ex>surviving</ex> friends; <ex>surviving</ex> customs</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Survivor</ent><br/
<hw>Sur*viv"or</hw> <pr>(?)</pr>, <pos>n.</pos> <sn>1.</sn> <def>One who survives or outlives another person, or any time, event, or thing.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The <qex>survivor</qex> bound<br/
In filial obligation for some term<br/
To do obsequious sorrow.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Law)</fld> <def>The longer liver of two joint tenants, or two persons having a joint interest in anything.</def>  <rj><au>Blackstone.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Survivorship</ent><br/
<hw>Sur*viv"or*ship</hw>, <pos>n.</pos> <sn>1.</sn> <def>The state of being a survivor.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>1.</sn> <fld>(Law)</fld> <def>The right of a joint tenant, or other person who has a joint interest in an estate, to take the whole estate upon the death of other.</def>  <rj><au>Blackstone.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Chance of survivorship</b></col>, <cd>the chance that a person of a given age has of surviving another of a giving age; thus, by the Carlisle tables of mortality the chances of survivorship for two persons, aged 25 and 65, are 89 and 11 respectively, or about 8 to 1 that the elder die first.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Susceptibility</ent><br/
<hw>Sus*cep`ti*bil"i*ty</hw> <pr>(?)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Susceptibilities</plw> <pr>(#)</pr>.</plu> <ety>[Cf. F. <ets>susceptibilit<eacute/</ets>.]</ety> <sn>1.</sn> <def>The state or quality of being susceptible; the capability of receiving impressions, or of being affected.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Specifically, capacity for deep feeling or emotional excitement; sensibility, in its broadest acceptation; impressibility; sensitiveness.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Magnetic susceptibility</b></col> <fld>(Physics)</fld>, <cd>the intensity of magnetization of a body placed in a uniform megnetic field of unit strength.</cd> <au>Sir W. Thomson.</au></cs><br/
[<source>1913 Webster</source>]</p>

<p><syn><b>Syn.</b> -- Capability; sensibility; feeling; emotion.</syn><br/
[<source>1913 Webster</source>]</p>

<p><ent>Susceptible</ent><br/
<hw>Sus*cep"ti*ble</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[F., from L. <ets>suscipere</ets>, <ets>susceptum</ets>, to take up, to support, undertake, recognize, admit; pref. <ets>sus</ets> (see <er>Sub-</er>) + <ets>capere</ets> to take. See <er>Capable</er>.]</ety> <sn>1.</sn> <def>Capable of admitting anything additional, or any change, affection, or influence; readily acted upon; <as>as, a body <ex>susceptible</ex> of color or of alteration</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>It sheds on souls <qex>susceptible</qex> of light,<br/
The glorious dawn of our eternal day.</q> <rj><qau>Young.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Capable of impression; having nice sensibility; impressible; tender; sensitive; <as>as, children are more <ex>susceptible</ex> than adults; a man of a <ex>susceptible</ex> heart.</as></def><-- = impressionable --><br/
[<source>1913 Webster</source>]</p>

<p><q>Candidates are . . . not very <qex>susceptible</qex> of affronts.</q> <rj><qau>Cowper.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>I am constitutionally <qex>susceptible</qex> of noises.</q> <rj><qau>Lamb.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p>-- <wordforms><wf>Sus*cep"ti*ble*ness</wf>, <pos>n.</pos> -- <wf>Sus*cep"ti*bly</wf>, <pos>adv.</pos></wordforms><br/
[<source>1913 Webster</source>]</p>

<p><ent>Susception</ent><br/
<hw>Sus*cep"tion</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>susceptio</ets>: cf. F. <ets>susception</ets>. See <er>Susceptible</er>.]</ety> <def>The act of taking; reception.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Susceptive</ent><br/
<hw>Sus*cep"tive</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Susceptible.</def> <au>I. Watts.</au> -- <wordforms><wf>Sus*cep"tive*ness</wf>, <pos>n.</pos></wordforms><br/
[<source>1913 Webster</source>]</p>

<p><ent>Susceptivity</ent><br/
<hw>Sus`cep*tiv"i*ty</hw> <pr>(?)</pr>, <pos>n.</pos> <def>Capacity for receiving; susceptibility.</def> <mark>[R.]</mark>  <rj><au>Wollaston.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Susceptor</ent><br/
<hw>Sus*cep"tor</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. See <er>Susceptible</er>.]</ety> <def>One who undertakes anything; specifically, a godfather; a sponsor; a guardian.</def>  <rj><au>Puller. Shipley.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Suscipiency</ent><br/
<hw>Sus*cip"i*en*cy</hw> <pr>(?)</pr>, <pos>n.</pos> <def>Admission.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Suscipient</ent><br/
<hw>Sus*cip"i*ent</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>suscipiens</ets>, p. pr. of <ets>suscipere</ets>. See <er>Susceptible</er>.]</ety> <def>Receiving; admitting.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Suscipient</ent><br/
<hw>Sus*cip"i*ent</hw>, <pos>n.</pos> <def>One who takes or admits; one who receives.</def> <mark>[R.]</mark>  <rj><au>Jer. Taylor.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Suscitability</ent><br/
<hw>Sus`ci*ta*bil"i*ty</hw> <pr>(?)</pr>, <pos>n.</pos> <def>Capability of being suscitated; excitability.</def> <mark>[Obs.]</mark>  <rj><au>B. Jonson.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Suscitate</ent><br/
<hw>Sus"ci*tate</hw> <pr>(?)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Suscitated</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Suscitating</conjf>.]</vmorph> <ety>[L. <ets>suscitatus</ets>, p. p. of <ets>suscitare</ets> to lift up, to rouse; pref. <ets>sus-</ets> (see <er>Sub-</er>) + <ets>citare</ets> to rouse, excite.  Cf. <er>Excite</er>, <er>Incite</er>.]</ety> <def>To rouse; to excite; to call into life and action.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Suscitation</ent><br/
<hw>Sus`ci*ta"tion</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>suscitatio</ets>: cf. F. <ets>suscitation</ets>.]</ety> <def>The act of raising or exciting.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>A mere <qex>suscitation</qex> or production of a thing.</q> <rj><qau>South.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Suslik</ent><br/
<hw>Sus"lik</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Russ. <ets>s<uacute/slik'</ets>.]</ety> <fld>(Zool.)</fld> <def>A ground squirrel (<spn>Spermophilus citillus</spn>) of Europe and Asia. It has large cheek pouches.</def> <altsp>[Written also <asp>souslik</asp>.]</altsp><br/
[<source>1913 Webster</source>]</p>

<p><ent>Suspect</ent><br/
<hw>Sus*pect"</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>suspectus</ets>, p. p. of <ets>suspicere</ets> to look up, admire, esteem, to look at secretly or askance, to mistrust; <ets>sub</ets> under + <ets>specere</ets> to look: cf. F. <ets>suspect</ets> suspected, suspicious. See <er>Spy</er>, and cf. <er>Suspicion</er>.]</ety> <sn>1.</sn> <def>Suspicious; inspiring distrust.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q><qex>Suspect</qex> [was] his face, <qex>suspect</qex> his word also.</q> <rj><qau>Chaucer.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Suspected; distrusted.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>What I can do or offer is <qex>suspect</qex>.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Suspect</ent><br/
<hw>Sus*pect"</hw>, <pos>n.</pos> <ety>[LL. <ets>suspectus</ets>. See <er>Suspect</er>, <pos>a.</pos>]</ety> <sn>1.</sn> <def>Suspicion.</def> <mark>[Obs.]</mark>  <rj><au>Chaucer.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>So with <qex>suspect</qex>, with fear and grief, dismayed.</q> <rj><qau>Fairfax.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>One who, or that which, is suspected; an object of suspicion; -- formerly applied to persons and things; now, only to persons suspected of crime.</def>  <rj><au>Bacon.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Suspect</ent><br/
<hw>Sus*pect"</hw>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Suspected</conjf>; <pos>p. pr. & vb. n.</pos> <conjf>Suspecting</conjf>.]</vmorph> <sn>1.</sn> <def>To imagine to exist; to have a slight or vague opinion of the existence of, without proof, and often upon weak evidence or no evidence; to mistrust; to surmise; -- commonly used regarding something unfavorable, hurtful, or wrong; <as>as, to <ex>suspect</ex> the presence of disease</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Nothing makes a man <qex>suspect</qex> much, more than to know little; and therefore men should remedy suspicion by procuring to know more.</q> <rj><qau>Bacon.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>From her hand I could <qex>suspect</qex> no ill.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To imagine to be guilty, upon slight evidence, or without proof; <as>as, to <ex>suspect</ex> one of equivocation</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>To hold to be uncertain; to doubt; to mistrust; to distruct; <as>as, to <ex>suspect</ex> the truth of a story</as>.</def>  <rj><au>Addison.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>To look up to; to respect.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><syn><b>Syn.</b> -- To mistrust; distrust; surmise; doubt.</syn><br/
[<source>1913 Webster</source>]</p>

<p><ent>Suspect</ent><br/
<hw>Sus*pect"</hw>, <pos>v. i.</pos> <def>To imagine guilt; to have a suspicion or suspicions; to be suspicious.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>If I <qex>suspect</qex> without cause, why then make sport at me.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Suspectable</ent><br/
<hw>Sus*pect"a*ble</hw> <pr>(?)</pr>, <pos>a.</pos> <def>That may be suspected.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Suspected</ent><br/
<hw>Sus*pect"ed</hw>, <pos>a.</pos> <def>Distrusted; doubted.</def> -- <wordforms><wf>Sus*pect"ed*ly</wf>, <pos>adv.</pos> -- <wf>Sus*pect"ed*ness</wf>, <pos>n.</pos></wordforms><br/
[<source>1913 Webster</source>]</p>

<p><ent>Suspecter</ent><br/
<hw>Sus*pect"er</hw> <pr>(?)</pr>, <pos>n.</pos> <def>One who suspects.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Suspectful</ent><br/
<hw>Sus*pect"ful</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Apt to suspect or mistrust; full of suspicion; suspicious; <as>as, to be <ex>suspectful</ex> of the motives of others</as>.</def> <au>Milton.</au> -- <wordforms><wf>Sus*pect"ful*ness</wf>, <pos>n.</pos></wordforms><br/
[<source>1913 Webster</source>]</p>

<p><ent>Suspection</ent><br/
<hw>Sus*pec"tion</hw> <pr>(?)</pr>, <pos>n.</pos> <def>Suspicion.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Suspectiousness</ent><br/
<hw>Sus*pec"tious*ness</hw> <pr>(?)</pr>, <pos>n.</pos> <def>Suspiciousness; cause for suspicion.</def> <mark>[Obs. & R.]</mark>  <rj><au>Ld. Berners.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Suspectless</ent><br/
<hw>Sus*pect"less</hw> <pr>(?)</pr>, <pos>a.</pos> <sn>1.</sn> <def>Not suspecting; having no suspicion.</def> <mark>[R.]</mark>  <rj><au>Sir T. Herbert.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Not suspected; not mistrusted.</def> <mark>[R.]</mark>  <rj><au>Beau. & Fl.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Suspend</ent><br/
<hw>Sus*pend"</hw> <pr>(?)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Suspended</conjf>; <pos>p. pr. & vb. n.</pos> <conjf>Suspending</conjf>.]</vmorph> <ety>[F. <ets>suspendre</ets>, or OF. <ets>souspendre</ets> (where the prefix is L. <ets>subtus</ets> below, from <ets>sub</ets> under), L. <ets>suspendere</ets>, <ets>suspensum</ets>; pref. <ets>sus-</ets> (see <er>Sub-</er>) + <ets>pendere</ets> to hang. See <er>Pedant</er>, and cf. <er>Suspense</er>, <pos>n.</pos>]</ety> <sn>1.</sn> <def>To attach to something above; to hang; <as>as, to <ex>suspend</ex> a ball by a thread; to <ex>suspend</ex> a needle by a loadstone</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To make to depend; <as>as, God hath <ex>suspended</ex> the promise of eternal life on the condition of obedience and holiness of life</as>.</def> <mark>[Archaic]</mark>  <rj><au>Tillotson.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>To cause to cease for a time; to hinder from proceeding; to interrupt; to delay; to stay.</def><br/
[<source>1913 Webster</source>]</p>

<p><q><qex>Suspend</qex> your indignation against my brother.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>The guard nor fights nor fies; their fate so near<br/
At once <qex>suspends</qex> their courage and their fear.</q> <rj><qau>Denham.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>To hold in an undetermined or undecided state; <as>as, to <ex>suspend</ex> one's judgment or opinion</as>.</def>  <rj><au>Locke.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <def>To debar, or cause to withdraw temporarily, from any privilege, from the execution of an office, from the enjoyment of income, etc.; <as>as, to <ex>suspend</ex> a student from college; to <ex>suspend</ex> a member of a club.</as></def><br/
[<source>1913 Webster</source>]</p>

<p><q>Good men should not be <qex>suspended</qex> from the exercise of their ministry and deprived of their livelihood for ceremonies which are on all hands acknowledged indifferent.</q> <rj><qau>Bp. Sanderson.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>6.</sn> <def>To cause to cease for a time from operation or effect; <as>as, to <ex>suspend</ex> the habeas corpus act; to <ex>suspend</ex> the rules of a legislative body.</as></def><br/
[<source>1913 Webster</source>]</p>

<p><sn>7.</sn> <fld>(Chem.)</fld> <def>To support in a liquid, as an insoluble powder, by stirring, to facilitate chemical action.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>To suspend payment</b></col> <fld>(Com.)</fld>, <cd>to cease paying debts or obligations; to fail; -- said of a merchant, a bank, etc.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><syn><b>Syn.</b> -- To hang; interrupt; delay; intermit; stay; hinder; debar.</syn><br/
[<source>1913 Webster</source>]</p>

<p><-- p. 1454 --></p>

<p><ent>Suspend</ent><br/
<hw>Sus*pend"</hw> <pr>(?)</pr>, <pos>v. i.</pos> <def>To cease from operation or activity; esp., to stop payment, or be unable to meet obligations or engagements (said of a commercial firm or a bank).</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Suspender</ent><br/
<hw>Sus*pend"er</hw> <pr>(?)</pr>, <pos>n.</pos> <def>One who, or that which, suspends; esp., one of a pair of straps or braces worn over the shoulders, for holding up the trousers.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Suspensation</ent><br/
<hw>Sus`pen*sa"tion</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Cf. LL. <ets>suspensatio</ets> suspension from a charge or benefice.]</ety> <def>The act of suspending, or the state of being suspended, especially for a short time; temporary suspension.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Suspense</ent><br/
<hw>Sus*pense"</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[F. <ets>suspens</ets>, L. <ets>suspensus</ets>, p. p. of <ets>suspendere</ets>. See <er>Suspend</er>.]</ety> <sn>1.</sn> <def>Held or lifted up; held or prevented from proceeding.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>[The great light of day] <qex>suspense</qex> in heaven.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Expressing, or proceeding from, suspense or doubt.</def> <mark>[Obs.]</mark> <ldquo/Expectation held his look <xex>suspense</xex>.<rdquo/  <rj><au>Milton.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Suspense</ent><br/
<hw>Sus*pense"</hw>, <pos>n.</pos> <ety>[From F. <ets>suspens</ets>, a. See <er>Suspense</er>, <pos>a.</pos>]</ety> <sn>1.</sn> <def>The state of being suspended; specifically, a state of uncertainty and expectation, with anxiety or apprehension; indetermination; indecision; <as>as, the <ex>suspense</ex> of a person waiting for the verdict of a jury</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Ten days the prophet in <qex>suspense</qex> remained.</q> <rj><qau>Denham.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Upon the ticklish balance of <qex>suspense</qex>.</q> <rj><qau>Cowper.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Cessation for a time; stop; pause.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>A cool <qex>suspense</qex> from pleasure and from pain.</q> <rj><qau>Pope.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <ety>[Cf. F. <ets>suspense</ets>.]</ety> <fld>(Law)</fld> <def>A temporary cessation of one's right; suspension, as when the rent or other profits of land cease by unity of possession of land and rent.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Suspense account</b></col> <fld>(Bookkeeping)</fld>, <cd>an account in which receipts or disbursements are temporarily entered until their proper position in the books is determined.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Suspensely</ent><br/
<hw>Sus*pense"ly</hw>, <pos>adv.</pos> <def>In suspense.</def> <mark>[Obs.]</mark>  <rj><au>Hales.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Suspensibility</ent><br/
<hw>Sus*pen`si*bil"i*ty</hw> <pr>(?)</pr>, <pos>n.</pos> <def>The quality or state of being suspensible.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Suspensible</ent><br/
<hw>Sus*pen"si*ble</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Capable of being suspended; capable of being held from sinking.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Suspension</ent><br/
<hw>Sus*pen"sion</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Cf. F. <ets>suspension</ets>, L. <ets>suspensio</ets> arched work, imperfect pronunciation. See <er>Suspend</er>.]</ety> <sn>1.</sn> <def>The act of suspending, or the state of being suspended; pendency; <as>as, <ex>suspension</ex> from a hook</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Especially, temporary delay, interruption, or cessation</def>; as: <sd>(a)</sd> <def>Of labor, study, pain, etc.</def> <sd>(b)</sd> <def>Of decision, determination, judgment, etc.; <as>as, to ask a <ex>suspension</ex> of judgment or opinion in view of evidence to be produced</as>.</def> <sd>(c)</sd> <def>Of the payment of what is due; <as>as, the <ex>suspension</ex> of a mercantile firm or of a bank</as>.</def> <sd>(d)</sd> <def>Of punishment, or sentence of punishment.</def> <sd>(e)</sd> <def>Of a person in respect of the exercise of his office, powers, prerogative, etc.; <as>as, the <ex>suspension</ex> of a student or of a clergyman</as>.</def> <sd>(f)</sd> <def>Of the action or execution of law, etc.; <as>as, the <ex>suspension</ex> of the habeas corpus act</as>.</def>
<-- # each of the above lettered definitions is elliptical; needs special handling for analysis. --><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>A conditional withholding, interruption, or delay; <as>as, the <ex>suspension</ex> of a payment on the performance of a condition</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>The state of a solid when its particles are mixed with, but undissolved in, a fluid, and are capable of separation by straining; also, any substance in this state.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <fld>(Rhet.)</fld> <def>A keeping of the hearer in doubt and in attentive expectation of what is to follow, or of what is to be the inference or conclusion from the arguments or observations employed.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>6.</sn> <fld>(Scots Law)</fld> <def>A stay or postponement of execution of a sentence condemnatory by means of letters of suspension granted on application to the lord ordinary.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>7.</sn> <fld>(Mus.)</fld> <def>The prolongation of one or more tones of a chord into the chord which follows, thus producing a momentary discord, suspending the concord which the ear expects.  Cf. <er>Retardation</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Pleas in suspension</b></col> <fld>(Law)</fld>, <cd>pleas which temporarily abate or suspend a suit.</cd> -- <col><b>Points of suspension</b></col> <fld>(Mech.)</fld>, <cd>the points, as in the axis or beam of a balance, at which the weights act, or from which they are suspended.</cd> -- <col><b>Suspension bridge</b></col>, <cd>a bridge supported by chains, ropes, or wires, which usually pass over high piers or columns at each end, and are secured in the ground beyond.</cd> -- <col><b>Suspension of arms</b></col> <fld>(Mil.)</fld>, <cd>a short truce or cessation of operations agreed on by the commanders of contending armies, as for burying the dead, making proposal for surrender or for peace, etc.</cd> -- <col><b>Suspension scale</b></col>, <cd>a scale in which the platform hangs suspended from the weighing apparatus instead of resting upon it.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><syn><b>Syn.</b> -- Delay; interruption; intermission; stop.</syn><br/
[<source>1913 Webster</source>]</p>

<p><ent>Suspensive</ent><br/
<hw>Sus*pen"sive</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[Cf. F. <ets>suspensif</ets>. See <er>Suspend</er>.]</ety> <def>Tending to suspend, or to keep in suspense; causing interruption or delay; uncertain; doubtful.</def> <ldquo/In <xex>suspensive</xex> thoughts.<rdquo/ <au>Beaumont.</au> <ldquo/A <xex>suspensive</xex> veto.<rdquo/ <au>Macaulay.</au><br/
[<source>1913 Webster</source>]</p>

<p><q>The provisional and <qex>suspensive</qex> attitude.</q> <rj><qau>J. Morley.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Suspensive</b></col><cd> condition <fld>(Scots Law)</fld>, a condition precedent, or a condition without the performance of which the contract can not be completed.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Suspensor</ent><br/
<hw>Sus*pen"sor</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[NL.]</ety> <sn>1.</sn> <def>A suspensory.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Bot.)</fld> <def>The cord which suspends the embryo; and which is attached to the radicle in the young state; the proembryo.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Suspensorium</ent><br/
||<hw>Sus`pen*so"ri*um</hw> <pr>(?)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Suspensoria</plw> <pr>(#)</pr>.</plu> <ety>[NL.]</ety> <fld>(Anat.)</fld> <def>Anything which suspends or holds up a part: especially, the mandibular suspensorium (a series of bones, or of cartilages representing them) which connects the base of the lower jaw with the skull in most vertebrates below mammals.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Suspensory</ent><br/
<hw>Sus*pen"so*ry</hw> <pr>(?)</pr>, <pos>a.</pos> <sn>1.</sn> <def>Suspended; hanging; depending.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Fitted or serving to suspend; suspending; <as>as, a <ex>suspensory</ex> muscle</as>.</def>  <rj><au>Ray.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <fld>(Anat.)</fld> <def>Of or pertaining to a suspensorium.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Suspensory</ent><br/
<hw>Sus*pen"so*ry</hw>, <pos>n.</pos> <ety>[Cf. F. <ets>suspensoir</ets>, <ets>suspensoire</ets>.]</ety> <def>That which suspends, or holds up, as a truss</def>; specifically <fld>(Med.)</fld>, <def>a bandage or bag for supporting the scrotum.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Suspicable</ent><br/
<hw>Sus"pi*ca*ble</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>suspacabilis</ets>, fr. <ets>suspicari</ets> to suspect, akin to <ets>suspicere</ets>. See <er>Suspect</er>, <pos>v. t.</pos>]</ety> <def>Liable to suspicion; suspicious.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>It is a very <qex>suspicable</qex> business.</q> <rj><qau>Dr. H. more.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Suspiciency</ent><br/
<hw>Sus*pi"cien*cy</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[From L. <ets>suspiciens</ets>, p. pr. of <ets>suspicere</ets>. See <er>Suspect</er>, <pos>v. t.</pos>]</ety> <def>Suspiciousness; suspicion.</def> <mark>[Obs.]</mark>  <rj><au>Hopkins.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Suspicion</ent><br/
<hw>Sus*pi"cion</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[OE. <ets>suspecioun</ets>, OF. <ets>souspe<cced/on</ets>, F. <ets>soup<cced/on</ets>, L. <ets>suspectio</ets> a looking up to, an esteeming highly, suspicion, fr. <ets>suspicere</ets> to look up, to esteem, to mistrust. The modern form <ets>suspicion</ets> in English and French is in imitation of L. <ets>suspicio</ets> mistrust, suspicion. See <er>Suspect</er>, and cf. <er>Suspicious</er>.]</ety> <sn>1.</sn> <def>The act of suspecting; the imagination or apprehension of the existence of something (esp. something wrong or hurtful) without proof, or upon very slight evidence, or upon no evidence.</def><br/
[<source>1913 Webster</source>]</p>

<p><q><qex>Suspicions</qex> among thoughts are like bats among birds, they ever fly by twilight.</q> <rj><qau>Bacon.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Slight degree; suggestion; hint.</def> <mark>[Colloq.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>The features are mild but expressive, with just a <qex>suspicion</qex> . . . of saturnine or sarcastic humor.</q> <rj><qau>A. W. Ward.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><syn><b>Syn.</b> -- Jealousy; distrust; mistrust; diffidence; doubt.</syn><br/
[<source>1913 Webster</source>]</p>

<p><ent>Suspicion</ent><br/
<hw>Sus*pi"cion</hw>, <pos>v. t.</pos> <def>To view with suspicion; to suspect; to doubt.</def> <mark>[Obs. or Low]</mark>  <rj><au>South.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Suspicious</ent><br/
<hw>Sus*pi"cious</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[OE. <ets>suspecious</ets>; cf. L. <ets>suspiciosus</ets>. See <er>Suspicion</er>.]</ety> <sn>1.</sn> <def>Inclined to suspect; given or prone to suspicion; apt to imagine without proof.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Nature itself, after it has done an injury, will ever be <qex>suspicious</qex>; and no man can love the person he suspects.</q> <rj><qau>South.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Many mischievous insects are daily at work to make men of merit <qex>suspicious</qex> of each other.</q> <rj><qau>Pope.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Indicating suspicion, mistrust, or fear.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>We have a <qex>suspicious</qex>, fearful, constrained countenance.</q> <rj><qau>Swift.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>Liable to suspicion; adapted to raise suspicion; giving reason to imagine ill; questionable; <as>as, an author of <ex>suspicious</ex> innovations; <ex>suspicious</ex> circumstances</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>I spy a black, <qex>suspicious</qex>, threatening could.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><syn><b>Syn.</b> -- Jealous; distrustful; mistrustful; doubtful; questionable. See <er>Jealous</er>.</syn><br/
[<source>1913 Webster</source>]</p>

<p>-- <wordforms><wf>Sus*pi"cious*ly</wf>, <pos>adv.</pos> -- <wf>Sus*pi"cious*ness</wf>, <pos>n.</pos></wordforms><br/
[<source>1913 Webster</source>]</p>

<p><ent>Suspiral</ent><br/
<hw>Sus*pir"al</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[From <er>Suspire</er>.]</ety> <sn>1.</sn> <def>A breathing hole; a vent or ventiduct.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A spring of water passing under ground toward a cistern or conduit.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Suspiration</ent><br/
<hw>Sus`pi*ra"tion</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>suspiratio</ets>. See <er>Suspire</er>.]</ety> <def>The act of sighing, or fetching a long and deep breath; a deep respiration; a sigh.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Windy <qex>suspiration</qex> of forced breath.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Suspire</ent><br/
<hw>Sus*pire"</hw> <pr>(?)</pr>, <pos>v. i.</pos> <ety>[L. <ets>suspirare</ets> to breathe out, to sigh; <ets>sub</ets> under + <ets>spirare</ets> to breathe: cf. F. <ets>souspirer</ets>, OF. <ets>souspirer</ets>.]</ety> <def>To fetch a long, deep breath; to sigh; to breathe.</def>  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Fireflies that <qex>suspire</qex><br/
In short, soft lapses of transported flame.</q> <rj><qau>Mrs. Browning.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Suspire</ent><br/
<hw>Sus*pire"</hw>, <pos>n.</pos> <ety>[Cf. L. <ets>suspirium</ets>.]</ety> <def>A long, deep breath; a sigh.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Suspired</ent><br/
<hw>Sus*pired"</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Ardently desired or longed for; earnestly coveted.</def> <mark>[Obs.]</mark>  <rj><au>Sir H. Wotton.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sustain</ent><br/
<hw>Sus*tain"</hw> <pr>(?)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Sustained</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Sustaining</conjf>.]</vmorph> <ety>[OE. <ets>sustenen</ets>, <ets>susteinen</ets>, OF. <ets>sustenir</ets>, <ets>sostenir</ets>, F. <ets>soutenir</ets> (the French prefix is properly fr. L. <ets>subtus</ets> below, fr. <ets>sub</ets> under), L. <ets>sustinere</ets>; pref. <ets>sus-</ets> (see <er>Sub-</er>) + <ets>tenere</ets> to hold. See <er>Tenable</er>, and cf. <er>Sustenance</er>.]</ety> <sn>1.</sn> <def>To keep from falling; to bear; to uphold; to support; <as>as, a foundation <ex>sustains</ex> the superstructure; a beast <ex>sustains</ex> a load; a rope <ex>sustains</ex> a weight</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Every pillar the temple to <qex>sustain</qex>.</q> <rj><qau>Chaucer.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Hence, to keep from sinking, as in despondence, or the like; to support.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>No comfortable expectations of another life to <qex>sustain</qex> him under the evils in this world.</q> <rj><qau>Tillotson.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>To maintain; to keep alive; to support; to subsist; to nourish; <as>as, provisions to <ex>sustain</ex> an army</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>To aid, comfort, or relieve; to vindicate.</def>  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>His sons, who seek the tyrant to <qex>sustain</qex>.</q> <rj><qau>Dryden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <def>To endure without failing or yielding; to bear up under; <as>as, to <ex>sustain</ex> defeat and disappointment</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>6.</sn> <def>To suffer; to bear; to undergo.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Shall Turnus, then, such endless toil <qex>sustain</qex>?</q> <rj><qau>Dryden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>You shall <qex>sustain</qex> more new disgraces.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>7.</sn> <def>To allow the prosecution of; to admit as valid; to sanction; to continue; not to dismiss or abate; <as>as, the court <ex>sustained</ex> the action or suit</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>8.</sn> <def>To prove; to establish by evidence; to corroborate or confirm; to be conclusive of; <as>as, to <ex>sustain</ex> a charge, an accusation, or a proposition</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><syn><b>Syn.</b> -- To support; uphold; subsist; assist; relieve; suffer; undergo.</syn><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sustain</ent><br/
<hw>Sus*tain"</hw> <pr>(?)</pr>, <pos>n.</pos> <def>One who, or that which, upholds or sustains; a sustainer.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>I waked again, for my <qex>sustain</qex> was the Lord.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sustainable</ent><br/
<hw>Sus*tain"a*ble</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[Cf. F. <ets>soutenable</ets>, OF. <ets>soustenable</ets>.]</ety> <def>Capable of being sustained or maintained; <as>as, the action is not <ex>sustainable</ex></as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sustained</ent><br/
<hw>Sus*tained"</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Held up to a certain pitch, degree, or level; uniform; <as>as, <ex>sustained</ex> pasion; a <ex>sustained</ex> style of writing; a <ex>sustained</ex> note in music</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sustainer</ent><br/
<hw>Sus*tain"er</hw> <pr>(?)</pr>, <pos>n.</pos> <def>One who, or that which, sustains.</def>  <rj><au>Waterland.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sustainment</ent><br/
<hw>Sus*tain"ment</hw> <pr>(?)</pr>, <pos>n.</pos> <def>The act of sustaining; maintenance; support.</def>  <rj><au>Milton. Lowell.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sustaltic</ent><br/
<hw>Sus*tal"tic</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[Gr. <?/ contractile, fr. <?/ to draw together, to moderate; <?/ together + <?/ to place.]</ety> <def>Mournful; -- said of a species of music among the ancient Greeks.</def>  <rj><au>Busby.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sustenance</ent><br/
<hw>Sus"te*nance</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[OF. <ets>sustenance</ets>, <ets>sostenance</ets>, <ets>soustenance</ets>: cf. L. <ets>sustenentia</ets> endurance. See <er>Sustain</er>.]</ety> <sn>1.</sn> <def>The act of sustaining; support; maintenance; subsistence; <as>as, the <ex>sustenance</ex> of the body; the <ex>sustenance</ex> of life</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>That which supports life; food; victuals; provisions; means of living; <as>as, the city has ample <ex>sustenance</ex></as>.</def> <ldquo/A man of little <xex>sustenance</xex>.<rdquo/  <rj><au>Chaucer.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>For lying is thy <qex>sustenance</qex>, thy food.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sustentacle</ent><br/
<hw>Sus*ten"ta*cle</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>sustentaculum</ets>. See <er>Sustentation</er>.]</ety> <def>Sustenance.</def> <mark>[Obs.]</mark>  <rj><au>Dr. H. More.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sustentacular</ent><br/
<hw>Sus`ten*tac"u*lar</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[See <er>Sustenance</er>.]</ety> <fld>(Anat.)</fld> <def>Supporting; sustaining; <as>as, a <ex>sustentacular</ex> tissue</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sustentate</ent><br/
<hw>Sus"ten*tate</hw> <pr>(?)</pr>, <pos>v. t.</pos> <def>To sustain.</def> <mark>[R.]</mark>  <rj><au>C. Reade.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sustentation</ent><br/
<hw>Sus`ten*ta"tion</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>sustentatio</ets> sustenance, maintenance, fr. <ets>sustentare</ets> to support, maintain, v. intens. fr. <ets>sustinere</ets> to sustain: cf. F. <ets>sustentation</ets>. See <er>Sustain</er>.]</ety> <sn>1.</sn> <def>The act of sustaining, or the state of being sustained; preservation from falling; support; sustenance; maintenance.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Physiol.)</fld> <def>The aggregate of the functions by which a living organism is maintained in a normal condition of weight and growth.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Sustentation fund</b></col> <fld>(Eccl.)</fld>, <cd>a fund of a religious body for support of its ministers, chapels, etc.; <as>as, the <ex>sustentation fund</ex> of the Free Church of Scotland</as>.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sustentative</ent><br/
<hw>Sus"ten*ta*tive</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Adapted to sustain, strengthen, or corroborate; <as>as, <ex>sustentative</ex> citations or quotations</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Sustentative functions</b></col> <fld>(Physiol.)</fld>, <cd>those functions of the body which affect its material composition and thus determine its mass.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sustention</ent><br/
<hw>Sus*ten"tion</hw> <pr>(?)</pr>, <pos>n.</pos> <def>Sustentation.</def> <mark>[R. or Colloq.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>In fine images, in <qex>sustention</qex>, in irony, they surpass anything that Burke ever wrote.</q> <rj><qau>J. Morley.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sustre</ent><br/
<ent>Suster</ent><br/
<mhw>{ <hw>Sus"ter</hw>, <hw>Sus"tre</hw> <pr>(?)</pr>, }</mhw> <pos>n.</pos>; <plu><it>pl.</it> <plw>Susters</plw> <pr>(<?/)</pr>, <plw>Sustres</plw>, <it>or</it> <plw>Sustren</plw> <pr>(<?/)</pr>.</plu> <def>Sister.</def> <mark>[Obs.]</mark>  <rj><au>Chaucer.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>There are seven <qex>sustren</qex>, that serve truth ever.</q> <rj><qau>Piers Plowman.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Susu</ent><br/
<hw>Su"su</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Zool.)</fld> <def>See <er>Soosoo</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Susurrant</ent><br/
<hw>Su*sur"rant</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>susurrans</ets>, p. pr. from <ets>susurrare</ets> to whisper.]</ety> <def>Whispering.</def> <mark>[R.]</mark> <ldquo/The soft <xex>susurrant</xex> sigh.<rdquo/  <rj><au>Poetry of Anti-Jacobin.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Susurration</ent><br/
<hw>Su`sur*ra"tion</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>susurratio</ets>, fr. <ets>susurrare</ets> to whisper: cf. F. <ets>susurration</ets>.]</ety> <def>A whispering; a soft murmur.</def> <ldquo/Soft <xex>susurrations</xex> of the trees.<rdquo/  <rj><au>Howell.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Susurringly</ent><br/
<hw>Su*sur"ring*ly</hw> <pr>(?)</pr>, <pos>adv.</pos> <def>In the manner of a whisper.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Susurrous</ent><br/
<hw>Su*sur"rous</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>susurrus</ets>.]</ety> <def>Whispering; rustling; full of whispering sounds.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Susurrus</ent><br/
||<hw>Su*sur"rus</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L.]</ety> <def>The act of whispering; a whisper; a murmur.</def>  <rj><au>De Quincey.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>The soft <qex>susurrus</qex> and sighs of the branches.</q> <rj><qau>Longfellow.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sutile</ent><br/
<hw>Su"tile</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>sutilis</ets>, fr. <ets>suere</ets> to sew: cf. F. <ets>sutile</ets>.]</ety> <def>Done by stitching.</def> <mark>[R.]</mark>  <rj><au>Boswell.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sutler</ent><br/
<hw>Sut"ler</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[D. <ets>zoetelaar</ets>, OD. <ets>soetelaar</ets>, a small trader, especially in camps, fr. <ets>soetelen</ets> to undertake low offices; cf. G. <ets>sudeln</ets> to do dirty work, to sully, soil, E. <ets>suds</ets>.]</ety> <def>A person who follows an army, and sells to the troops provisions, liquors, and the like.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sutlership</ent><br/
<hw>Sut"ler*ship</hw>, <pos>n.</pos> <def>The condition or occupation of a sutler.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sutling</ent><br/
<hw>Sut"ling</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Belonging to sutlers; engaged in the occupation of a sutler.</def>  <rj><au>Addison.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sutor</ent><br/
<hw>Su"tor</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A kind of sirup made by the Indians of Arizona from the fruit of some cactaceous plant (probably the <xex>Cereus giganteus</xex>).</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sutra</ent><br/
||<hw>Su"tra</hw> <pr>(?)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Sutras</plw> <pr>(#)</pr>.</plu> <ety>[Skr. <ets>s<?/tra</ets> a thread, a string of rules; an aphorism; fr. <ets>siv</ets> to sew.]</ety> <sn>1.</sn> <sd>(a)</sd> <def>A precept; an aphorism; a brief rule.</def> <sd>(b)</sd> <def>A collection of such aphorisms.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <pluf>pl.</pluf> <def>A body of Hindoo literature containing aphorisms on grammar, meter, law, and philosophy, and forming a connecting link between the Vedic and later Sanscrit literature.</def>  <rj><au>Balfour (Cyc. of India).</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Suttee</ent><br/
<hw>Sut*tee"</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Skr. <ets>sat<imac/</ets> a faithful wife, fem. of <ets>sant</ets> existing, real, true, good, p. pr. of <ets>as</ets> to be.  Cf. <er>Sooth</er>.]</ety> <sn>1.</sn> <def>A Hindoo widow who immolates herself, or is immolated, on the funeral pile of her husband; -- so called because this act of self-immolation is regarded as envincing excellence of wifely character.</def> <mark>[India]</mark><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>The act of burning a widow on the funeral pile of her husband.</def> <mark>[India]</mark><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ The practice, though abolished in British India law in 1829, is not wholly prevented.</note><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sutteeism</ent><br/
<hw>Sut*tee"ism</hw> <pr>(?)</pr>, <pos>n.</pos> <def>The practice of self-immolation of widows in Hindostan.</def><br/
[<source>1913 Webster</source>]</p>

<p><-- p. 1455 --></p>

<p><ent>Suttle</ent><br/
<hw>Sut"tle</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Etymol. uncertain.]</ety> <fld>(Com.)</fld> <def>The weight when the tare has been deducted, and tret is yet to be allowed.</def>  <rj><au>M<?/Culloch.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Suttle</ent><br/
<hw>Sut"tle</hw>, <pos>v. i.</pos> <ety>[See <er>Sutler</er>.]</ety> <def>To act as sutler; to supply provisions and other articles to troops.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sutural</ent><br/
<hw>Su"tur*al</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[Cf. F. <ets>sutural</ets>, NL. <ets>suturals</ets>.]</ety> <sn>1.</sn> <def>Of or pertaining to a suture, or seam.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Bot.)</fld> <def>Taking place at a suture; <as>as, a <ex>sutural</ex> de<?/iscence</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Suturally</ent><br/
<hw>Su"tur*al*ly</hw>, <pos>adv.</pos> <def>In a sutural manner.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Suturated</ent><br/
<hw>Su"tur*a`ted</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Sewed or knit together; united by a suture; stitched.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Suture</ent><br/
<hw>Su"ture</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>sutura</ets>, fr. <ets>suere</ets>, <ets>sutum</ets>, to sew or stitch: cf. F. <ets>suture</ets>. See <er>Sew</er> to unite with thread.]</ety> <sn>1.</sn> <def>The act of sewing; also, the line along which two things or parts are sewed together, or are united so as to form a seam, or that which resembles a seam.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Surg.)</fld> <sd>(a)</sd> <def>The uniting of the parts of a wound by stitching.</def> <sd>(b)</sd> <def>The stitch by which the parts are united.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <fld>(Anat.)</fld> <def>The line of union, or seam, in an immovable articulation, like those between the bones of the skull; also, such an articulation itself; synarthrosis. See <cref>Harmonic suture</cref>, under <er>Harmonic</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <fld>(Bot.)</fld> <sd>(a)</sd> <def>The line, or seam, formed by the union of two margins in any part of a plant; <as>as, the ventral <ex>suture</ex> of a legume</as>.</def> <sd>(b)</sd> <def>A line resembling a seam; <as>as, the dorsal <ex>suture</ex> of a legume, which really corresponds to a midrib</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <fld>(Zool.)</fld> <sd>(a)</sd> <def>The line at which the elytra of a beetle meet and are sometimes confluent.</def> <sd>(b)</sd> <def>A seam, or impressed line, as between the segments of a crustacean, or between the whorls of a univalve shell.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><mcol><col><b>Glover's suture</b></col>, <col><b>Harmonic suture</b></col></mcol>, <cd>etc. See under <er>Glover</er>, <er>Harmonic</er>, etc.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sutured</ent><br/
<hw>Su"tured</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Having a suture or sutures; knit or united together.</def>  <rj><au>Pennant.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Suwarrow</ent><br/
<hw>Su*war"row</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Bot.)</fld> <def>The giant cactus (<spn>Cereus giganteus</spn>); -- so named by the Indians of Arizona.  Called also <altname>saguaro</altname>.</def>
<-- Saguaro is the most common now --><br/
[<source>1913 Webster</source>]</p>

<p><ent>Suzerain</ent><br/
<hw>Su"ze*rain</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F., formed fr. <ets>sus</ets> above, L. <ets>susum</ets>, <ets>sursum</ets> (fr. <ets>sub</ets> under + <ets>versum</ets>, p. p. of <ets>vertere</ets> to turn), after the analogy of <ets>souverain</ets>, E. <ets>sovereign</ets>. See <er>Sub-</er>, and <er>Verse</er>.]</ety> <def>A superior lord, to whom fealty is due; a feudal lord; a lord paramount.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Suzerainty</ent><br/
<hw>Su"ze*rain*ty</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F. <ets>suzerainet<eacute/</ets>.]</ety> <def>The dominion or authority of a suzerain; paramount authority.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Swa</ent><br/
<hw>Swa</hw> <pr>(sw<aum/)</pr>, <pos>adv.</pos> <ety>[See <er>So</er>.]</ety> <def>So.</def> <mark>[Obs.]</mark>  <rj><au>Chaucer.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Swab</ent><br/
<hw>Swab</hw> <pr>(?)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Swabbed</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Swabbing</conjf>.]</vmorph> <ety>[See <er>Swabber</er>, <pos>n.</pos>]</ety> <def>To clean with a mop or swab; to wipe when very wet, as after washing; <as>as, to <ex>swab</ex> the desk of a ship</as>.</def> <altsp>[Spelt also <asp>swob</asp>.]</altsp><br/
[<source>1913 Webster</source>]</p>

<p><ent>Swab</ent><br/
<hw>Swab</hw>, <pos>n.</pos> <ety>[Written also <ets>swob</ets>.]</ety> <sn>1.</sn> <def>A kind of mop for cleaning floors, the desks of vessels, etc., esp. one made of rope-yarns or threads.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A bit of sponge, cloth, or the like, fastened to a handle, for cleansing the mouth of a sick person, applying medicaments to deep-seated parts, etc.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <fld>(Naut.)</fld> <def>An epaulet.</def> <mark>[Sailor's Slang]</mark>  <rj><au>Marryat.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>A cod, or pod, as of beans or pease.</def> <mark>[Obs.]</mark>  <rj><au>Bailey.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <def>A sponge, or other suitable substance, attached to a long rod or handle, for cleaning the bore of a firearm.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Swabber</ent><br/
<hw>Swab"ber</hw> <pr>(?)</pr>, <pos>v. t.</pos> <def>To swab.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Swabber</ent><br/
<hw>Swab"ber</hw>, <pos>n.</pos> <ety>[D. <ets>zwabber</ets>; cf.D. <ets>zwabberen</ets> to swab, G. <ets>schwabbern</ets>, Dan. <ets>svabre</ets>, Sw. <ets>svab</ets> a swab, <ets>svabla</ets> to swab.]</ety> <sn>1.</sn> <def>One who swabs a floor or desk.</def>  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Naut.)</fld> <def>Formerly, an interior officer on board of British ships of war, whose business it was to see that the ship was kept clean.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>Same as <er>Swobber</er>, 2.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Swad</ent><br/
<hw>Swad</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Probably fr. AS. <ets>swe<?/ian</ets> to bind.]</ety> <altsp>[Written also <asp>swod</asp>.]</altsp> <sn>1.</sn> <def>A cod, or pod, as of beans or pease.</def> <mark>[Prov. Eng.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q><qex>Swad</qex>, in the north, is a peascod shell -- thence used for an empty, shallow-headed fellow.</q> <rj><qau>Blount.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A clown; a country bumpkin.</def> <mark>[Obs. or Prov. Eng.]</mark> <ldquo/Country swains, and silly <xex>swads</xex>.<rdquo/  <rj><au>Greene.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>There was one busy fellow was their leader,<br/
A blunt, squat <qex>swad</qex>, but lower than yourself.</q> <rj><qau>B. Jonson.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>A lump of mass; also, a crowd.</def> <mark>[Low, U.S.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <fld>(Coal Mining)</fld> <def>A thin layer of refuse at the bottom of a seam.</def>  <rj><au>Raymond.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Swaddle</ent><br/
<hw>Swad"dle</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[AS. <ets>swe<?/il</ets>, <ets>swe<?/el</ets>, fr. <ets>swe<?/ain</ets> to bind. See <er>Swathe</er>.]</ety> <def>Anything used to swaddle with, as a cloth or band; a swaddling band.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>They put me in bed in all my swaddles.</q> <rj><qau>Addison.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Swaddle</ent><br/
<hw>Swad"dle</hw>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Swaddled</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Swaddling</conjf> <pr>(?)</pr>.]</vmorph> <sn>1.</sn> <def>To bind as with a bandage; to bind or warp tightly with clothes; to swathe; -- used esp. of infants; <as>as, to <ex>swaddle</ex> a baby</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>They <qex>swaddled</qex> me up in my nightgown with long pieces of linen.</q> <rj><qau>Addison.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To beat; to cudgel.</def> <mark>[Obs.]</mark>  <rj><au>Hudibras.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Swaddlebill</ent><br/
<hw>Swad"dle*bill`</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Zool.)</fld> <def>The shoveler.</def> <mark>[Local, U.S.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Swaddler</ent><br/
<hw>Swad"dler</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A term of contempt for an Irish Methodist.</def>  <rj><au>Shipley.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Swaddling</ent><br/
<hw>Swad"dling</hw> <pr>(?)</pr>, <def><pos>a. & n.</pos> from <er>Swaddle</er>, <pos>v.</pos></def><br/
[<source>1913 Webster</source>]</p>

<p><cs><mcol><col><b>Swaddling band</b></col>, <col><b>Swaddling cloth</b></col>, <it>or</it> <col><b>Swaddling clout</b></col></mcol>, <cd>a band or cloth wrapped round an infant, especially round a newborn infant.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><q>Ye shall find the babe wrapped in <qex>swaddling clothes</qex>, lying in a manger.</q> <rj><qau>Luke ii. 12.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Swag</ent><br/
<hw>Swag</hw> <pr>(?)</pr>, <pos>v. i.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Swagged</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Swagging</conjf> <pr>(?)</pr>.]</vmorph> <ety>[Cf. Icel. <ets>sveggja</ets>, <ets>sveigja</ets> to bend, to sway, Norw. <ets>svaga</ets> to sway. See <er>Sway</er>.]</ety> <sn>1.</sn> <def>To hang or move, as something loose and heavy; to sway; to swing.</def> <mark>[Prov. Eng.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To sink down by its weight; to sag.</def>  <rj><au>Sir H. Wotton.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>I <qex>swag</qex> as a fat person's belly <qex>swaggeth</qex> as he goeth.</q> <rj><qau>Palsgrave.</qau></rj><br/
[<source>1913 Webster</source>]</p>


<p><sn>3.</sn> <def>To tramp carrying a swag.</def> <mark>[Australia]</mark><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Swag</ent><br/
<hw>Swag</hw>, <pos>n.</pos> <sn>1.</sn> <def>A swaying, irregular motion.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A burglar's or thief's booty; boodle.</def> <mark>[Cant or Slang]</mark>  <rj><au>Charles Reade.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <mark>[Australia]</mark> <sd>(a)</sd> <def>A tramping bushman's luggage, rolled up either in canvas or in a blanket so as to form a long bundle, and carried on the back or over the shoulder; -- called also a <altname>bluey</altname>, or a <altname>drum</altname>.</def> <sd>(b)</sd> <def>Any bundle of luggage similarly rolled up; hence, luggage in general.</def></p>

<p><q>He tramped for years till the <qex>swag</qex> he bore seemed part of himself.</q>  <rj><qau>Lawson.</qau></rj><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Swag-bellied</ent><br/
<hw>Swag"-bel`lied</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Having a prominent, overhanging belly.</def>  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Swagbelly</ent><br/
<hw>Swag"bel`ly</hw> <pr>(?)</pr>, <pos>n.</pos> <sn>1.</sn> <def>A prominent, overhanging belly.</def>  <rj><au>Smollett.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Med.)</fld> <def>Any large tumor developed in the abdomen, and neither fluctuating nor sonorous.</def>  <rj><au>Dunglison.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Swage</ent><br/
<hw>Swage</hw> <pr>(?)</pr>, <pos>v. t. & i.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Swaged</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Swaging</conjf> <pr>(?)</pr>.]</vmorph> <ety>[Equiv. to <ets>suage</ets>, abbrev. fr. <ets>assuage</ets>.]</ety> <def>See <er>Assuage</er>.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Swage</ent><br/
<hw>Swage</hw>, <pos>n.</pos> <def>A tool, variously shaped or grooved on the end or face, used by blacksmiths and other workers in metals, for shaping their work, whether sheet metal or forging, by holding the swage upon the work, or the work upon the swage, and striking with a sledge.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Swage block</b></col>, <cd>a perforated block of iron, having grooved sides and adapted for use in heading bolts and swaging objects of large size.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Swage</ent><br/
<hw>Swage</hw>, <pos>v. t.</pos> <def>To shape by means of a swage; to fashion, as a piece of iron, by forcing it into a groove or mold having the required shape.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Swagger</ent><br/
<hw>Swag"ger</hw> <pr>(?)</pr>, <pos>v. i.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Swaggered</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Swaggering</conjf>.]</vmorph> <ety>[Freq. of <ets>swag</ets>.]</ety> <sn>1.</sn> <def>To walk with a swaying motion; hence, to walk and act in a pompous, consequential manner.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>A man who <qex>swaggers</qex> about London clubs.</q> <rj><qau>Beaconsfield.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To boast or brag noisily; to be ostentatiously proud or vainglorious; to bluster; to bully.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>What a pleasant it is . . . to <qex>swagger</qex> at the bar!</q> <rj><qau>Arbuthnot.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>To be great is not . . . to <qex>swagger</qex> at our footmen.</q> <rj><qau>Colier.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Swagger</ent><br/
<hw>Swag"ger</hw>, <pos>v. t.</pos> <def>To bully.</def> <mark>[R.]</mark>  <rj><au>Swift.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Swagger</ent><br/
<hw>Swag"ger</hw>, <pos>n.</pos> <def>The act or manner of a swaggerer.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>He gave a half <qex>swagger</qex>, half leer, as he stepped forth to receive us.</q> <rj><qau>W. Irving.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Swagger</ent><br/
<hw>Swag"ger</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A swagman.</def> <mark>[Australia]</mark><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Swaggerer</ent><br/
<hw>Swag"ger*er</hw> <pr>(?)</pr>, <pos>n.</pos> <def>One who swaggers; a blusterer; a bully; a boastful, noisy fellow.</def>  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Swaggie</ent><br/
<hw>Swag"gie</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A swagman.</def> <mark>[Australia]</mark><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Swaggy</ent><br/
<hw>Swag"gy</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Inclined to swag; sinking, hanging, or leaning by its weight.</def>  <rj><au>Sir T. Browne.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Swagman</ent><br/
<hw>Swag"man</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A bushman carrying a swag and traveling on foot; -- called also <altname>swagsman</altname>, <altname>swagger</altname>, and <altname>swaggie</altname>.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><q>Once a jolly <qex>swagman</qex> sat beside a billabong<br/
Under the shade of a coolibah tree.<br/
And he sang as he sat and watched his billy boiling,<br/
`Who'll come a-waltzing Matilda with me?'</q> <rj><qau>[Waltzing Matilda, an Australian tune.]</qau></rj><br/
[<source>PJC</source>]</p>

<p><ent>Swagsman</ent><br/
<hw>Swags"man</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A swagman.</def> <mark>[Australia]</mark><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Swain</ent><br/
<hw>Swain</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[OE. <ets>swain</ets>, <ets>swein</ets>, Icel. <ets>sveinn</ets> a boy, servant; akin to Sw. <ets>sven</ets>, Dan. <ets>svend</ets>, AS. <ets>sw<amac/n</ets>, OHG. <ets>swein</ets>.]</ety> <sn>1.</sn> <def>A servant.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>Him behoves serve himself that has no <qex>swain</qex>.</q> <rj><qau>Chaucer.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A young man dwelling in the country; a rustic; esp., a cuntry gallant or lover; -- chiefly in poetry.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>It were a happy life<br/
To be no better than a homely <qex>swain</qex>.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Blest <qex>swains</qex>! whose nymphs in every grace excel.</q> <rj><qau>Pope.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Swainish</ent><br/
<hw>Swain"ish</hw>, <pos>a.</pos> <def>Pertaining to, or resembling, a swain; rustic; ignorant.</def> <ldquo/An ungentle and <xex>swainish</xex> beast.<rdquo/ <au>Milton.</au> -- <wordforms><wf>Swain"ish*ness</wf>, <pos>n.</pos> <au>Emerson.</au></wordforms><br/
[<source>1913 Webster</source>]</p>

<p><ent>Swainling</ent><br/
<hw>Swain"ling</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A little swain.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Swainmote</ent><br/
<hw>Swain"mote`</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[<ets>Swain</ets> + <ets>mote</ets> meeting: cf. LL. <ets>swanimotum</ets>.]</ety> <fld>(Eng. Forest Law)</fld> <def>A court held before the verders of the forest as judges, by the steward of the court, thrice every year, the swains, or freeholders, within the forest composing the jury.</def> <altsp>[Written also <asp>swanimote</asp>, and <asp>sweinmote</asp>.]</altsp>  <rj><au>Blackstone.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Swainship</ent><br/
<hw>Swain"ship</hw>, <pos>n.</pos> <def>The condition of a swain.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Swaip</ent><br/
<hw>Swaip</hw> <pr>(?)</pr>, <pos>v. i.</pos> <ety>[Cf. <er>Sweep</er>.]</ety> <def>To walk proudly; to sweep along.</def> <mark>[Prov. Eng.]</mark>  <rj><au>Todd.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Swal</ent><br/
<hw>Swal</hw> <pr>(?)</pr>, <mark>obs.</mark> <pos>imp.</pos> <mord>of <er>Swell</er></mord>. <def>Swelled.</def>  <rj><au>Chaucer.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Swale</ent><br/
<hw>Swale</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Cf. Icel. <ets>svalr</ets> cool, <ets>svala</ets> to cool.]</ety> <def>A valley or low place; a tract of low, and usually wet, land; a moor; a fen.</def> <mark>[Prov. Eng. & Local, U.S.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Swale</ent><br/
<hw>Swale</hw>, <pos>v. i. & t.</pos> <def>To melt and waste away; to singe. See <er>Sweal</er>, <pos>v.</pos></def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Swale</ent><br/
<hw>Swale</hw>, <pos>n.</pos> <def>A gutter in a candle.</def> <mark>[Prov. Eng.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Swallet</ent><br/
<hw>Swal"let</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Cf. G. <ets>schwall</ets> a sea swell, from <ets>schwellen</ets> to swell, E. <ets>swell</ets>.]</ety> <def>Water breaking in upon the miners at their work; -- so called among tin miners.</def> <mark>[Prov. Eng.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Swallow</ent><br/
<hw>Swal"low</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[OE. <ets>swalowe</ets>, AS. <ets>swalewe</ets>, <ets>swealwe</ets>; akin to D. <ets>zwaluw</ets>, OHG. <ets>swalawa</ets>, G. <ets>schwalbe</ets>, Icel. & Sw. <ets>svala</ets>, Dan. <ets>svale</ets>.]</ety> <sn>1.</sn> <fld>(Zool.)</fld> <def>Any one of numerous species of passerine birds of the family <fam>Hirundinidae</fam>, especially one of those species in which the tail is deeply forked. They have long, pointed wings, and are noted for the swiftness and gracefulness of their flight.</def><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ The most common North American species are the barn swallow (see under <er>Barn</er>), the cliff, or eaves, swallow (see under <er>Cliff</er>), the white-bellied, or tree, swallow (<spn>Tachycineta bicolor</spn>), and the bank swallow (see under <er>Bank</er>). The common European swallow (<spn>Chelidon rustica</spn>), and the window swallow, or martin (<spn>Chelidon urbica</spn>), are familiar species.</note><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Zool.)</fld> <def>Any one of numerous species of swifts which resemble the true swallows in form and habits, as the common American chimney swallow, or swift.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <fld>(Naut.)</fld> <def>The aperture in a block through which the rope reeves.</def>  <rj><au>Ham. Nav. Encyc.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Swallow plover</b></col> <fld>(Zool.)</fld>, <cd>any one of several species of fork-tailed ploverlike birds of the genus <gen>Glareola</gen>, as <spn>Glareola orientalis</spn> of India; a pratincole.</cd> -- <col><b>Swallow shrike</b></col> <fld>(Zool.)</fld>, <cd>any one of several species of East Indian and Asiatic birds of the family <fam>Artamiidae</fam>, allied to the shrikes but similar to swallows in appearance and habits. The ashy swallow shrike (<spn>Artamus fuscus</spn>) is common in India.</cd> -- <col><b>Swallow warbler</b></col> <fld>(Zool.)</fld>, <cd>any one of numerous species of East Indian and Australian singing birds of the genus <gen>Dicaeum</gen>. They are allied to the honeysuckers.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Swallow</ent><br/
<hw>Swal"low</hw> <pr>(?)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Swallowed</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Swallowing</conjf>.]</vmorph> <ety>[OE. <ets>swolewen</ets>, <ets>swolwen</ets>, <ets>swolhen</ets>, AS. <ets>swelgan</ets>; akin to D. <ets>zwelgen</ets>, OHG. <ets>swelahan</ets>, <ets>swelgan</ets>, G. <ets>schwelgen</ets> to feast, to revel, Icel. <ets>svelgia</ets> to swallow, SW. <ets>sv<aum/lja</ets>, Dan. <ets>svaelge</ets>.  Cf. <er>Groundsel</er> a plant.]</ety> <sn>1.</sn> <def>To take into the stomach; to receive through the gullet, or esophagus, into the stomach; <as>as, to <ex>swallow</ex> food or drink</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>As if I had <qex>swallowed</qex> snowballs for pills.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To draw into an abyss or gulf; to ingulf; to absorb -- usually followed by <xex>up</xex>.</def>  <rj><au>Milton.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>The earth opened her mouth, and <qex>swallowed</qex> them up, and their houses.</q> <rj><qau>Num. xvi. 32.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>To receive or embrace, as opinions or belief, without examination or scruple; to receive implicitly.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Though that story . . . be not so readily <qex>swallowed</qex>.</q> <rj><qau>Sir T. Browne.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>To engross; to appropriate; -- usually with <xex>up</xex>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Homer excels . . . in this, that he <qex>swallowed</qex> up the honor of those who succeeded him.</q> <rj><qau>Pope.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <def>To occupy; to take up; to employ.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The necessary provision of the life <qex>swallows</qex> the greatest part of their time.</q> <rj><qau>Locke.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>6.</sn> <def>To seize and waste; to exhaust; to consume.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Corruption <qex>swallowed</qex> what the liberal hand<br/
Of bounty scattered.</q> <rj><qau>Thomson.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>7.</sn> <def>To retract; to recant; <as>as, to <ex>swallow</ex> one's opinions</as>.</def> <ldquo/<xex>Swallowed</xex> his vows whole.<rdquo/  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>8.</sn> <def>To put up with; to bear patiently or without retaliation; <as>as, to <ex>swallow</ex> an affront or insult</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><syn><b>Syn.</b> -- To absorb; imbibe; ingulf; engross; consume. See <er>Absorb</er>.</syn><br/
[<source>1913 Webster</source>]</p>

<p><ent>Swallow</ent><br/
<hw>Swal"low</hw>, <pos>v. i.</pos> <def>To perform the act of swallowing; <as>as, his cold is so severe he is unable to <ex>swallow</ex></as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Swallow</ent><br/
<hw>Swal"low</hw>, <pos>n.</pos> <sn>1.</sn> <def>The act of swallowing.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>The gullet, or esophagus; the throat.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>Taste; relish; inclination; liking.</def> <mark>[Colloq.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>I have no <qex>swallow</qex> for it.</q> <rj><qau>Massinger.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>Capacity for swallowing; voracity.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>There being nothing too gross for the <qex>swallow</qex> of political rancor.</q> <rj><qau>Prof. Wilson.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <def>As much as is, or can be, swallowed at once; <as>as, a <ex>swallow</ex> of water</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>6.</sn> <def>That which ingulfs; a whirlpool.</def> <mark>[Obs.]</mark>  <rj><au>Fabyan.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Swallower</ent><br/
<hw>Swal"low*er</hw> <pr>(?)</pr>, <pos>n.</pos> <def>One who swallows; also, a glutton.</def>  <rj><au>Tatler.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Swallowfish</ent><br/
<hw>Swal"low*fish`</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Zool.)</fld> <def>The European sapphirine gurnard (<spn>Trigla hirundo</spn>). It has large pectoral fins.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Swallowtail</ent><br/
<hw>Swal"low*tail`</hw> <pr>(?)</pr>, <pos>n.</pos> <sn>1.</sn> <fld>(Carp.)</fld> <def>A kind of tenon or tongue used in making joints. See <er>Dovetail</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Bot.)</fld> <def>A species of willow.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <fld>(Fort.)</fld> <def>An outwork with converging sides, its head or front forming a reentrant angle; -- so called from its form.  Called also <altname>priestcap</altname>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>A swallow-tailed coat.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>This Stultz coat, a blue <qex>swallowtail</qex>, with yellow buttons.</q> <rj><qau>Thackeray.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <def>An arrow.</def>  <rj><au>Sir W. Scott.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>6.</sn> <fld>(Zool.)</fld> <def>Any one of numerous species of large and handsome butterflies, belonging to Papilio and allied genera, in which the posterior border of each hind wing is prolongated in the form of a long lobe.</def><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ The black swallowtail, or asterias (see <er>Papilio</er>), the blue swallowtail, or philenor, the tiger swallowtail, or turnus (see <er>Turnus</er>), and the zebra swallowtail, or ajax (see under <er>Zebra</er>) are common American species.  See also <er>Troilus</er>.</note><br/
[<source>1913 Webster</source>]</p>

<p><ent>Swallow-tailed</ent><br/
<hw>Swal"low-tailed`</hw> <pr>(?)</pr>, <pos>a.</pos> <sn>1.</sn> <def>Having a tail like that of a swallow; hence, like a swallow's tail in form; having narrow and tapering or pointed skirts; <as>as, a <ex>swallow-tailed</ex> coat</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Carp.)</fld> <def>United by dovetailing; dovetailed.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Swallow-tailed duck</b></col> <fld>(Zool.)</fld>, <cd>the old squaw.</cd> -- <col><b>Swallow-tailed gull</b></col> <fld>(Zool.)</fld>, <cd>an Arctic gull (<spn>Xema furcata</spn>), which has a deeply forked tail.</cd> -- <mcol><col><b>Swallow-tailed hawk</b></col> <it>or</it> <col><b>Swallow-tailed kite</b></col></mcol> <fld>(Zool.)</fld>, <cd>the fork-tailed kite.</cd> -- <col><b>Swallow-tailed moth</b></col> <fld>(Zool.)</fld>, <cd>a European moth (<spn>Urapteryx sambucaria</spn>) having tail-like lobes on the hind wings.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Swallowwort</ent><br/
<hw>Swal"low*wort`</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Bot.)</fld> <sd>(a)</sd> <def>See <er>Celandine</er>.</def> <sd>(b)</sd> <def>A poisonous plant (<spn>Vincetoxicum officinale</spn>) of the Milkweed family, at one time used in medicine; -- also called <altname>white swallowwort</altname>.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>African swallowwort</b></col>, <cd>a plant of the genus Stapelia.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Swam</ent><br/
<hw>Swam</hw> <pr>(?)</pr>, <def><pos>imp.</pos> of <er>Swim</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Swamp</ent><br/
<hw>Swamp</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Cf. AS. <ets>swam</ets> a fungus, OD. <ets>swam</ets> a sponge, D. <ets>zwam</ets> a fungus, G. <ets>schwamm</ets> a sponge, Icel. <ets>sv<oum/ppr</ets>, Dan. & Sw. <ets>swamp</ets>, Goth. <ets>swamms</ets>, Gr. <grk>somfo`s</grk> porous, spongy.]</ety> <def>Wet, spongy land; soft, low ground saturated with water, but not usually covered with it; marshy ground away from the seashore.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Gray <qex>swamps</qex> and pools, waste places of the hern.</q> <rj><qau>Tennyson.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>A <qex>swamp</qex> differs from a bog and a marsh in producing trees and shrubs, while the latter produce only herbage, plants, and mosses.</q> <rj><qau>Farming Encyc. (E. Edwards, Words).</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Swamp blackbird</b></col>. <fld>(Zool.)</fld> <cd>See <er>Redwing</er> <sd>(b)</sd>.</cd> -- <col><b>Swamp cabbage</b></col> <fld>(Bot.)</fld>, <cd>skunk cabbage.</cd> -- <col><b>Swamp deer</b></col> <fld>(Zool.)</fld>, <cd>an Asiatic deer (<spn>Rucervus Duvaucelli</spn>) of India.</cd> -- <col><b>Swamp hen</b></col>. <fld>(Zool.)</fld> <sd>(a)</sd> <cd>An Australian azure-breasted bird (<spn>Porphyrio bellus</spn>); -- called also <altname>goollema</altname>.</cd> <sd>(b)</sd> <cd>An Australian water crake, or rail (<spn>Porzana Tabuensis</spn>); -- called also <altname>little swamp hen</altname>.</cd> <sd>(c)</sd> <cd>The European purple gallinule.</cd> -- <col><b>Swamp honeysuckle</b></col> <fld>(Bot.)</fld>, <cd>an American shrub (<spn>Azalea viscosa</spn> syn. <spn>Rhododendron viscosa</spn> or <spn>Rhododendron viscosum</spn>) growing in swampy places, with fragrant flowers of a white color, or white tinged with rose; -- called also <altname>swamp pink</altname> and <altname>white swamp honeysuckle</altname>.</cd> -- <col><b>Swamp hook</b></col>, <cd>a hook and chain used by lumbermen in handling logs.  Cf. <er>Cant hook</er>.</cd> -- <col><b>Swamp itch</b></col>. <fld>(Med.)</fld> <cd>See <cref>Prairie itch</cref>, under <er>Prairie</er>.</cd> -- <col><b>Swamp laurel</b></col> <fld>(Bot.)</fld>, <cd>a shrub (<spn>Kalmia glauca</spn>) having small leaves with the lower surface glaucous.</cd> -- <col><b>Swamp maple</b></col> <fld>(Bot.)</fld>, <cd>red maple. See <er>Maple</er>.</cd> -- <col><b>Swamp oak</b></col> <fld>(Bot.)</fld>, <cd>a name given to several kinds of oak which grow in swampy places, as swamp Spanish oak (<spn>Quercus palustris</spn>), swamp white oak (<spn>Quercus bicolor</spn>), swamp post oak (<spn>Quercus lyrata</spn>).</cd> -- <col><b>Swamp ore</b></col><cd> <fld>(Min.)</fld>, bog ore; limonite.</cd> -- <col><b>Swamp partridge</b></col> <fld>(Zool.)</fld>, <cd>any one of several Australian game birds of the genera <gen>Synoicus</gen> and <gen>Excalfatoria</gen>, allied to the European partridges.</cd> -- <col><b>Swamp robin</b></col> <fld>(Zool.)</fld>, <cd>the chewink.</cd> -- <col><b>Swamp sassafras</b></col> <fld>(Bot.)</fld>, <cd>a small North American tree of the genus <gen>Magnolia</gen> (<spn>Magnolia glauca</spn>) with aromatic leaves and fragrant creamy-white blossoms; -- called also <altname>sweet bay</altname>.</cd> -- <col><b>Swamp sparrow</b></col> <fld>(Zool.)</fld>, <cd>a common North American sparrow (<spn>Melospiza Georgiana</spn>, or <spn>Melospiza palustris</spn>), closely resembling the song sparrow. It lives in low, swampy places.</cd> -- <col><b>Swamp willow</b></col>. <fld>(Bot.)</fld> <cd>See <cref>Pussy willow</cref>, under <er>Pussy</er>.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><-- p. 1456 --></p>

<p><ent>Swamp</ent><br/
<hw>Swamp</hw> <pr>(?)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Swamped</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Swamping</conjf>.]</vmorph> <sn>1.</sn> <def>To plunge or sink into a swamp.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Naut.)</fld> <def>To cause (a boat) to become filled with water; to capsize or sink by whelming with water.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>Fig.: To plunge into difficulties and perils; to overwhelm; to ruin; to wreck.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The Whig majority of the house of Lords was <qex>swamped</qex> by the creation of twelve Tory peers.</q> <rj><qau>J. R. Green.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Having <qex>swamped</qex> himself in following the ignis fatuus of a theory.</q> <rj><qau>Sir W. Hamilton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Swamp</ent><br/
<hw>Swamp</hw>, <pos>v. i.</pos> <sn>1.</sn> <def>To sink or stick in a swamp; figuratively, to become involved in insuperable difficulties.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To become filled with water, as a boat; to founder; to capsize or sink; figuratively, to be ruined; to be wrecked.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Swampy</ent><br/
<hw>Swamp"y</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Consisting of swamp; like a swamp; low, wet, and spongy; <as>as, <ex>swampy</ex> land</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Swan</ent><br/
<hw>Swan</hw> <pr>(sw<ocr/n)</pr>, <pos>n.</pos> <ety>[AS. <ets>swan</ets>; akin to D. <ets>zwaan</ets>, OHG. <ets>swan</ets>, G. <ets>schwan</ets>, Icel. <ets>svanr</ets>, Sw. <ets>svan</ets>, Dan. <ets>svane</ets>; and perhaps to E. <ets>sound</ets> something audible.]</ety> <sn>1.</sn> <fld>(Zool.)</fld> <def>Any one of numerous species of large aquatic birds belonging to <gen>Cygnus</gen>, <gen>Olor</gen>, and allied genera of the subfamily <fam>Cygninae</fam>. They have a large and strong beak and a long neck, and are noted for their graceful movements when swimming. Most of the northern species are white. In literature the swan was fabled to sing a melodious song, especially at the time of its death.</def><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ The European white, or mute, swan (<spn>Cygnus gibbus</spn>), which is most commonly domesticated, bends its neck in an S-shaped curve. The whistling, or trumpeting, swans of the genus <gen>Olor</gen> do not bend the neck in an S-shaped curve, and are noted for their loud and sonorous cry, due to complex convolutions of the windpipe. To this genus belong the European whooper, or whistling swan (<spn>Olor cygnus</spn>), the American whistling swan (<spn>Olor Columbianus</spn>), and the trumpeter swan (<spn>Olor buccinator</spn>). The Australian black swan (<spn>Chenopis atrata</spn>) is dull black with white on the wings, and has the bill carmine, crossed with a white band. It is a very graceful species and is often domesticated.  The South American black-necked swan (<spn>Sthenelides melancorypha</spn>) is a very beautiful and graceful species, entirely white, except the head and neck, which are dark velvety seal-brown. Its bill has a double bright rose-colored knob.</note><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Fig.: An appellation for a sweet singer, or a poet noted for grace and melody; <as>as Shakespeare is called the <ex>swan</ex> of Avon</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <fld>(Astron.)</fld> <def>The constellation Cygnus.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Swan goose</b></col> <fld>(Zool.)</fld>, <cd>a bird of India (<spn>Cygnopsis cygnoides</spn>) resembling both the swan and the goose.</cd> -- <col><b>Swan shot</b></col>, <cd>a large size of shot used in fowling.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Swang</ent><br/
<hw>Swang</hw> <pr>(?)</pr>, <mark>obs.</mark> <def><pos>imp.</pos> of <er>Swing</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Swang</ent><br/
<hw>Swang</hw>, <pos>n.</pos> <ety>[Cf. <er>Swamp</er>.]</ety> <def>A swamp.</def> <mark>[Prov. Eng.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Swanherd</ent><br/
<hw>Swan"herd`</hw> <pr>(?)</pr>, <pos>n.</pos> <def>One who tends or marks swans; <as>as, the royal <ex>swanherd</ex> of England</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Swan-hopping</ent><br/
<hw>Swan"-hop`ping</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A corruption of <er>Swan-upping</er>.</def> <mark>[Eng.]</mark>  <rj><au>Encyc. Brit.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Swanimote</ent><br/
<hw>Swan"i*mote</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Eng. Forest Law)</fld> <def>See <er>Swainmote</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Swanky</ent><br/
<ent>Swankie</ent><br/
<mhw>{ <hw>Swank"ie</hw>, <hw>Swank"y</hw>  }</mhw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Cf. G. <ets>schwank</ets> flexible, pliant.]</ety> <def>An active and clever young fellow.</def> <mark>[Scot.]</mark>  <rj><au>Sir W. Scott.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Swanlike</ent><br/
<hw>Swan"like`</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Resembling a swan.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Swanmark</ent><br/
<hw>Swan"mark`</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A mark of ownership cut on the bill or swan.</def> <mark>[Eng.]</mark>  <rj><au>Encyc. Brit.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Swannery</ent><br/
<hw>Swan"ner*y</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A place where swans are bred.</def> <ldquo/The largest <xex>swannery</xex> in England.<rdquo/  <rj><au>Encyc. Brit.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Swanny</ent><br/
<hw>Swan"ny</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Swanlike; <as>as, a <ex>swanny</ex> glossiness of the neck</as>.</def>  <rj><au>Richardson.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Swanpan</ent><br/
<hw>Swan"pan</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Cf. <er>Schwanpan</er>.]</ety> <def>The Chinese abacus; a schwanpan.</def><-- also, suan-pan -->  <rj><au>S. W. Williams.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Swans-down</ent><br/
<ent>Swan's-down</ent><br/
<mhw>{ <hw>Swan's"-down`</hw> <pr>(?)</pr>, <it>or</it> <hw>Swans"-down`</hw>  }</mhw>, <pos>n.</pos> <sn>1.</sn> <def>The down, or fine, soft feathers, of the swan, used on various articles of dress.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A fine, soft, thick cloth of wool mixed with silk or cotton; a sort of twilled fustian, like moleskin.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Swan's-down cotton</b></col>. <cd>See <cref>Cotton flannel</cref>, under <er>Cotton</er>.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Swanskin</ent><br/
<hw>Swan"skin`</hw> <pr>(?)</pr>, <pos>n.</pos> <sn>1.</sn> <def>The act of a swan with the down or the feathers on.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A species of soft flannel, thick and warm.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Swan-upping</ent><br/
<hw>Swan"-up`ping</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A yearly expedition on the Thames to take up young swans and mark them, as by Companies of Dyers and Vintners; -- called also <altname>swan-hopping</altname>.</def> <mark>[Eng.]</mark>  <rj><au>Encyc. Brit.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Swap</ent><br/
<hw>Swap</hw> <pr>(?)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Swapped</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Swapping</conjf>.]</vmorph> <ety>[OE. <ets>swappen</ets> to strike; cf. E. to <ets>strike</ets> a bargain; perh. akin to E. <ets>sweep</ets>.  Cf. <er>Swap</er> a blow, <er>Swap</er>, <pos>v. i.</pos>]</ety> <altsp>[Written also <asp>swop</asp>.]</altsp> <sn>1.</sn> <def>To strike; -- with <xex>off</xex>.</def> <mark>[Obs. or Prov. Eng.]</mark> <ldquo/<xex>Swap</xex> off his head!<rdquo/  <rj><au>Chaucer.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To exchange (usually two things of the same kind); to swop.</def> <mark>[Colloq.]</mark>  <rj><au>Miss Edgeworth.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Swap</ent><br/
<hw>Swap</hw>, <pos>v. i.</pos> <ety>[Cf. <er>Swap</er>, <pos>v. t.</pos>]</ety> <sn>1.</sn> <def>To fall or descend; to rush hastily or violently.</def>  <rj><au>C. Richardson (Dict.).</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>All suddenly she <qex>swapt</qex> adown to ground.</q> <rj><qau>Chaucer.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To beat the air, or ply the wings, with a sweeping motion or noise; to flap.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Swap</ent><br/
<hw>Swap</hw>, <pos>n.</pos> <ety>[Cf. G. <ets>schwapp</ets>, n., a slap, swap, <ets>schwapp</ets>, <ets>schwapps</ets>, interj., slap! smack! and E. <ets>swap</ets>, v.t.]</ety> <sn>1.</sn> <def>A blow; a stroke.</def> <mark>[Obs. or Prov. Eng.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>An exchange; a barter.</def> <mark>[Colloq.]</mark>  <rj><au>Sir W. Scott.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Swap</ent><br/
<hw>Swap</hw>, <pos>adv.</pos> <ety>[See <er>Swap</er>, <pos>n.</pos>]</ety> <def>Hastily.</def> <mark>[Prov. Eng.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Swape</ent><br/
<hw>Swape</hw> <pr>(?)</pr>, <pos>n.</pos> <def>See <er>Sweep</er>, <pos>n.</pos>, 12.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sward</ent><br/
<hw>Sward</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[AS. <ets>sweard</ets> skin, covering; akin to OFries. <ets>swarge</ets>, D. <ets>zwoord</ets>, G. <ets>schwarte</ets>, Icel. <ets>sv<oum/r<?/r</ets> skin, sward of the earth.]</ety> <sn>1.</sn> <def>Skin; covering.</def> <mark>[Obs. or Prov. Eng.]</mark>  <rj><au>Halliwell.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>The grassy surface of land; that part of the soil which is filled with the roots of grass; turf.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The <qex>sward</qex> was trim as any garden lawn.</q> <rj><qau>Tennyson.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Sward pork</b></col>, <cd>bacon in large fitches.</cd> <mark>[Prov. Eng.]</mark></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sward</ent><br/
<hw>Sward</hw>, <pos>v. t. & i.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Swarded</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Swarding</conjf>.]</vmorph> <def>To produce sward upon; to cover, or be covered, with sward.</def>  <rj><au>Mortimer.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sward-cutter</ent><br/
<hw>Sward"-cut`ter</hw> <pr>(?)</pr>, <pos>n.</pos> <sd>(a)</sd> <def>A plow for turning up grass land.</def> <sd>(b)</sd> <def>A lawn mower.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Swarded</ent><br/
<hw>Sward"ed</hw>, <pos>a.</pos> <def>Covered with sward.</def>  <rj><au>Mrs. Browning.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Swardy</ent><br/
<hw>Sward"y</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Covered with sward or grass.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sware</ent><br/
<hw>Sware</hw> <pr>(?)</pr>, <def><pos>imp.</pos> of <er>Swear</er>.</def> <mark>[Obs. or Poetic]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>Cophetua <qex>sware</qex> a royal oath.</q> <rj><qau>Tennyson.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Swarf</ent><br/
<hw>Swarf</hw> <pr>(?)</pr>, <pos>v. i.</pos> <ety>[Cf. <er>Swerve</er>.]</ety> <def>To grow languid; to faint.</def> <mark>[Scot.]</mark> <ldquo/To <xex>swarf</xex> for very hunger.<rdquo/  <rj><au>Sir W. Scott.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Swarf</ent><br/
<hw>Swarf</hw>, <pos>n.</pos> <ety>[Cf. <er>Swerve</er>.]</ety> <def>The grit worn away from grindstones in grinding cutlery wet.</def> <mark>[Prov. Eng.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Swarm</ent><br/
<hw>Swarm</hw> <pr>(?)</pr>, <pos>v. i.</pos> <ety>[Cf. <er>Swerve</er>.]</ety> <def>To climb a tree, pole, or the like, by embracing it with the arms and legs alternately. See <er>Shin</er>.</def> <mark>[Colloq.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>At the top was placed a piece of money, as a prize for those who could <qex>swarm</qex> up and seize it.</q> <rj><qau>W. Coxe.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Swarm</ent><br/
<hw>Swarm</hw>, <pos>n.</pos> <ety>[OE. <ets>swarm</ets>, AS. <ets>swearm</ets>; akin to D. <ets>zwerm</ets>, G. <ets>schwarm</ets>, OHG. <ets>swaram</ets>, Icel. <ets>svarmr</ets> a tumult, Sw. <ets>sv<aum/rm</ets> a swarm, Dan. <ets>svaerm</ets>, and G. <ets>schwirren</ets> to whiz, to buzz, Skr. <ets>svar</ets> to sound, and perhaps to E. <ets>swear</ets>. <root/177.  Cf. <er>Swerve</er>, <er>Swirl</er>.]</ety> <sn>1.</sn> <def>A large number or mass of small animals or insects, especially when in motion.</def> <ldquo/A deadly <xex>swarm</xex> of hornets.<rdquo/  <rj><au>Milton.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Especially, a great number of honeybees which emigrate from a hive at once, and seek new lodgings under the direction of a queen; a like body of bees settled permanently in a hive.</def> <ldquo/A <xex>swarm</xex> of bees.<rdquo/  <rj><au>Chaucer.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>Hence, any great number or multitude, as of people in motion, or sometimes of inanimate objects; <as>as, a <ex>swarm</ex> of meteorites</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Those prodigious <qex>swarms</qex> that had settled themselves in every part of it [Italy].</q> <rj><qau>Addison.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><syn><b>Syn.</b> -- Multitude; crowd; throng.</syn><br/
[<source>1913 Webster</source>]</p>

<p><ent>Swarm</ent><br/
<hw>Swarm</hw>, <pos>v. i.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Swarmed</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Swarming</conjf>.]</vmorph> <sn>1.</sn> <def>To collect, and depart from a hive by flight in a body; -- said of bees; <as>as, bees <ex>swarm</ex> in warm, clear days in summer</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To appear or collect in a crowd; to throng together; to congregate in a multitude.</def>  <rj><au>Chaucer.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>To be crowded; to be thronged with a multitude of beings in motion.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Every place <qex>swarms</qex> with soldiers.</q> <rj><qau>Spenser.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>To abound; to be filled (with).</def>  <rj><au>Atterbury.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <def>To breed multitudes.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Not so thick <qex>swarmed</qex> once the soil<br/
Bedropped with blood of Gorgon.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Swarm</ent><br/
<hw>Swarm</hw>, <pos>v. t.</pos> <def>To crowd or throng.</def>  <rj><au>Fanshawe.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Swarmspore</ent><br/
<hw>Swarm"spore`</hw> <pr>(?)</pr>, <pos>n.</pos> <sn>1.</sn> <fld>(Bot.)</fld> <def>One of innumerable minute, motile, reproductive bodies, produced asexually by certain algae and fungi; a zoospore.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Zool.)</fld> <def>One of the minute flagellate germs produced by the sporulation of a protozoan; -- called also <altname>zoospore</altname>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Swart</ent><br/
<hw>Swart</hw> <pr>(?)</pr>, <pos>n.</pos> <def>Sward.</def> <mark>[Obs.]</mark>  <rj><au>Holinshed.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Swart</ent><br/
<hw>Swart</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[OE. <ets>swart</ets>, AS. <ets>sweart</ets> black; akin to OFries, OS. & LG. <ets>swart</ets>, D. <ets>zwart</ets>, G. <ets>schwartz</ets>, OHG. <ets>swarz</ets>, Icel. <ets>svarir</ets>, Sw. <ets>svart</ets>, Dan. <ets>sort</ets>, Goth. <ets>swarts</ets>; cf. L. <ets>sordes</ets> dirt, <ets>sordere</ets> to be dirty.  Cf. <er>Sordid</er>, <er>Surd</er>.]</ety> <sn>1.</sn> <def>Of a dark hue; moderately black; swarthy; tawny.</def> <ldquo/<xex>Swart</xex> attendants.<rdquo/ <au>Trench.</au> <ldquo/<xex>Swart</xex> savage maids.<rdquo/ <au>Hawthorne.</au><br/
[<source>1913 Webster</source>]</p>

<p><q>A nation strange, with visage <qex>swart</qex>.</q> <rj><qau>Spenser.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Gloomy; malignant.</def> <mark>[Obs.]</mark>  <rj><au>Milton.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Swart star</b></col>, <cd>the Dog Star; -- so called from its appearing during the hot weather of summer, which makes swart the countenance.</cd> <mark>[R.]</mark> <au>Milton.</au></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Swart</ent><br/
<hw>Swart</hw>, <pos>v. t.</pos> <def>To make swart or tawny; <as>as, to <ex>swart</ex> a living part</as>.</def>  <rj><au>Sir T. Browne.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Swartback</ent><br/
<hw>Swart"back`</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Zool.)</fld> <def>The black-backed gull (<spn>Larus marinus</spn>); -- called also <altname>swarbie</altname>.</def> <mark>[Prov. Eng.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Swarth</ent><br/
<hw>Swarth</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Swart; swarthy.</def> <ldquo/A <xex>swarth</xex> complexion.<rdquo/  <rj><au>Chapman.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Swarth</ent><br/
<hw>Swarth</hw> <pr>(?)</pr>, <pos>n.</pos> <def>An apparition of a person about to die; a wraith.</def> <mark>[Prov. Eng.]</mark>  <rj><au>Grose.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Swarth</ent><br/
<hw>Swarth</hw>, <pos>n.</pos> <ety>[See <er>Sward</er>.]</ety> <def>Sward; short grass.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Grassy <qex>swarth</qex>, close cropped by nibbling sheep.</q> <rj><qau>Cowper.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Swarth</ent><br/
<hw>Swarth</hw>, <pos>n.</pos> <def>See <er>Swath</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Swarthily</ent><br/
<hw>Swarth"i*ly</hw> <pr>(?)</pr>, <pos>adv.</pos> <def>In a swarthy manner; with a tawny hue; duskily.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Swarthiness</ent><br/
<hw>Swarth"i*ness</hw>, <pos>n.</pos> <def>The quality or state of being swarthy; a dusky or dark complexion; tawniness.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Swarthness</ent><br/
<hw>Swarth"ness</hw>, <pos>n.</pos> <def>Swarthiness.</def> <mark>[R.]</mark>  <rj><au>Dr. R. Clerke.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Swarthy</ent><br/
<hw>Swarth"y</hw> <pr>(?)</pr>, <pos>a.</pos> <amorph>[<pos>Compar.</pos> <adjf>Swarthier</adjf> <pr>(?)</pr>; <pos>superl.</pos> <adjf>Swarthiest</adjf>.]</amorph> <ety>[See <er>Swart</er>, <pos>a.</pos>]</ety> <def>Being of a dark hue or dusky complexion; tawny; swart; <as>as, <ex>swarthy</ex> faces</as>.</def> <ldquo/A <xex>swarthy</xex> Ethiope.<rdquo/  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Their <qex>swarthy</qex> hosts would darken all our plains.</q> <rj><qau>Addison.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Swarthy</ent><br/
<hw>Swarth"y</hw>, <pos>v. t.</pos> <def>To make swarthy.</def> <mark>[Obs.]</mark>  <rj><au>Cowley.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Swartiness</ent><br/
<hw>Swart"i*ness</hw> <pr>(?)</pr>, <pos>n.</pos> <def>Swarthiness.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Swartish</ent><br/
<hw>Swart"ish</hw>, <pos>a.</pos> <def>Somewhat swart, dark, or tawny.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Swartness</ent><br/
<hw>Swart"ness</hw>, <pos>n.</pos> <def>The quality or state of being swart.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Swarty</ent><br/
<hw>Swart"y</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Swarthy; tawny.</def> <mark>[Obs.]</mark>  <rj><au>Burton.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Swarve</ent><br/
<hw>Swarve</hw> <pr>(?)</pr>, <pos>v. i.</pos> <ety>[See <er>Swerve</er>.]</ety> <sn>1.</sn> <def>To swerve.</def> <mark>[Obs. or Scot.]</mark>  <rj><au>Spenser. Jamieson.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To climb.</def> <mark>[Prov. Eng.]</mark>  <rj><au>Halliwell.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Swash</ent><br/
<hw>Swash</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Cf. <er>Swash</er>, <pos>v. i.</pos>, <er>Squash</er>, <pos>v. t.</pos>]</ety> <fld>(Arch.)</fld> <def>An oval figure, whose moldings are oblique to the axis of the work.</def>  <rj><au>Moxon.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Swash plate</b></col> <fld>(Mach.)</fld>, <cd>a revolving circular plate, set obliquely on its shaft, and acting as a cam to give a reciprocating motion to a rod in a direction parallel to the shaft.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Swash</ent><br/
<hw>Swash</hw>, <pos>a.</pos> <ety>[Cf. <er>Swash</er>, <pos>v. i.</pos>, <er>Squash</er>, <pos>v. t.</pos>]</ety> <def>Soft, like fruit too ripe; swashy.</def> <mark>[Prov. Eng.]</mark>  <rj><au>Pegge.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Swash</ent><br/
<hw>Swash</hw>, <pos>v. i.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Swashed</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Swashing</conjf>.]</vmorph> <ety>[Probably of imitative origin; cf. Sw. <ets>svasska</ets> to splash, and, for sense 3, Sw. <ets>svassa</ets> to bully, to rodomontade.]</ety> <sn>1.</sn> <def>To dash or flow noisily, as water; to splash; <as>as, water <ex>swashing</ex> on a shallow place</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To fall violently or noisily.</def> <mark>[Obs.]</mark>  <rj><au>Holinshed.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>To bluster; to make a great noise; to vapor or brag.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Swash</ent><br/
<hw>Swash</hw>, <pos>n.</pos> <sn>1.</sn> <def>Impulse of water flowing with violence; a dashing or splashing of water.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A narrow sound or channel of water lying within a sand bank, or between a sand bank and the shore, or a bar over which the sea washes.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>Liquid filth; wash; hog mash.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>A blustering noise; a swaggering behavior.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <def>A swaggering fellow; a swasher.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Swashbuckler</ent><br/
<hw>Swash"buc`kler</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A bully or braggadocio; a swaggering, boastful fellow; a swaggerer.</def>  <rj><au>Milton.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Swasher</ent><br/
<hw>Swash"er</hw> <pr>(?)</pr>, <pos>n.</pos> <def>One who makes a blustering show of valor or force of arms.</def>  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Swashing</ent><br/
<hw>Swash"ing</hw>, <pos>a.</pos> <sn>1.</sn> <def>Swaggering; hectoring.</def> <ldquo/A <xex>swashing</xex> and martial outside.<rdquo/  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Resounding; crushing.</def> <ldquo/<xex>Swashing</xex> blow.<rdquo/  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Swashway</ent><br/
<hw>Swash"way`</hw> <pr>(?)</pr>, <pos>n.</pos> <def>Same as 4th <er>Swash</er>, 2.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Swashy</ent><br/
<hw>Swash"y</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Soft, like fruit that is too ripe; quashy; swash.</def> <mark>[Prov. Eng.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Swastica</ent><br/
<ent>Swastika</ent><br/
<mhw>{ <hw>Swas"ti*ka</hw>, <hw>Swas"ti*ca</hw> }</mhw> <pr>(?)</pr>, <pos>n.</pos> <altsp>[Also <asp>suastica</asp>, <asp>svastika</asp>, etc.]</altsp> <ety>[Skr. <ets>svastika</ets>, fr. <ets>svasti</ets> walfare; <ets>su</ets> well + <ets>asti</ets> being.]</ety> <def>A symbol or ornament in the form of a Greek cross with the ends of the arms at right angles all in the same direction, and each prolonged to the height of the parallel arm of the cross.  A great many modified forms exist, ogee and volute as well as rectilinear, while various decorative designs, as Greek fret or meander, are derived from or closely associated with it. The swastika is found in remains from the Bronze Age in various parts of Europe, esp. at Hissarlik (Troy), and was in frequent use as late as the 10th century. It is found in ancient Persia, in India, where both Jains and  Buddhists used (or still use) it as religious symbol, in China and Japan, and among Indian tribes of North, Central, and South America. It is usually thought to be a charm, talisman, or religious token, esp. a sign of good luck or benediction.  <person>Max M<uum/ller</person> distinguished from the <ex>swastika</ex>, with arms prolonged to the right, the <contr>suavastika</contr>, with arms prolonged to the left, but this distinction is not commonly recognized.  Other names for the swastika are <altname>fylfot</altname> and <altname>gammadion</altname>.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p> <note>The <ex>swastika</ex> with arms bent to the right came to be used as a symbol of Aryan supremacy by the Nazi party in Germany, 1933 - 1945; hence, it is now associated in the United States and European countries with Nazism or antisemitism.  It is sometimes used by neo-nazis, or by antisemites as an antisemitic symbol.</note><br/
[<source>PJC</source>]</p>

<p><ent>Swat</ent><br/
<hw>Swat</hw> <pr>(?)</pr>, <mark>obs.</mark> <def><pos>imp.</pos> of <er>Sweat</er>.</def>  <rj><au>Chaucer.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Swat</ent><br/
<hw>Swat</hw> <pr>(sw<ocr/t)</pr>, <pos>v. t.</pos> <sn>1.</sn> <def>to hit, especially with an instrument in the hand; to smack.</def></p>

<p><sn>2.</sn> <fld>(Baseball)</fld> <def>to hit (a baseball) powerfully, and for a long distance.</def></p>

<p><ent>Swat</ent><br/
<hw>Swat</hw> <pr>(sw<ocr/t)</pr>, <pos>n.</pos> <altsp>[Also spelled <asp>swot</asp>.]</altsp> <sn>1.</sn> <def>a sharp blow, especially one made with an instrument in the hand.</def></p>

<p><sn>2.</sn> <fld>(Baseball)</fld> <def>a powerful hit, especially a home run.</def></p>

<p><ent>Swat</ent><br/
<hw>Swat</hw> <pr>(sw<ocr/t)</pr>, <pos>n.</pos> <def>a former state in Northwest India, now a part of Pakistan.  It was ruled by a prince called a <it>Wali</it>.</def></p>

<p><ent>Swat team</ent><br/
<ent>SWAT team</ent><br/
<mhw><hw>SWAT" team`</hw>, <hw>Swat" team`</hw></mhw> <pr>(sw<ocr/t" t<emac/m)</pr>, <pos>n.</pos> <ety>[acronym: <ets>S</ets>pecial <ets>W</ets>eapons <ets>a</ets>nd <ets>T</ets>raining.]</ety> <def>a special group of police trained to deal with unusually dangerous or violent situations, and having special weapons, such as rifles more powerful than those carried by regular police officers.  They are employed, for example, in situations when hostages are being held, or heavily armed persons need to be captured.  Called also <altname>SWAT</altname> or <altname>S.W.A.T.</altname>.</def></p>

<p><ent>Swatch</ent><br/
<hw>Swatch</hw> <pr>(?)</pr>, <pos>n.</pos> <sn>1.</sn> <def>A swath.</def> <mark>[Obs.]</mark>  <rj><au>Tusser.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A piece, pattern, or sample, generally of cloth.</def>  <rj><au>Halliwell.</au>  <au>Jamieson.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Swate</ent><br/
<hw>Swate</hw> <pr>(sw<amac/t)</pr>, <mark>obs.</mark> <def><pos>imp.</pos> of <er>Sweat</er>.</def>  <rj><au>Thomson.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Swath</ent><br/
<hw>Swath</hw> <pr>(sw<add/th; 277)</pr>, <pos>n.</pos> <ety>[AS. <ets>swa<edh/u</ets> a track, trace; akin to D. <ets>zwaad</ets>, <ets>zwad</ets>, <ets>zwade</ets>, a swath of grass, G. <ets>schwad</ets>, <ets>schwaden</ets>; perhaps, originally, a shred.  Cf. <er>Swathe</er>, <pos>v. t.</pos>]</ety> <sn>1.</sn> <def>A line of grass or grain cut and thrown together by the scythe in mowing or cradling.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>The whole sweep of a scythe, or the whole breadth from which grass or grain is cut by a scythe or a machine, in mowing or cradling; <as>as, to cut a wide <ex>swath</ex></as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>A band or fillet; a swathe.</def>  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Swath bank</b></col>, <cd>a row of new-mown grass.</cd> <mark>[Prov. Eng.]</mark></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Swathe</ent><br/
<hw>Swathe</hw> <pr>(sw<amac/<th/)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Swathed</conjf> <pr>(sw<amac/<th/d)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Swathing</conjf>.]</vmorph> <ety>[OE. <ets>swathen</ets>, AS. <ets>swe<edh/ain</ets>. See <er>Swath</er>, <pos>n.</pos>, and cf. <er>Swaddle</er>.]</ety> <def>To bind with a swathe, band, bandage, or rollers.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Their children are never <qex>swathed</qex> or bound about with any thing when they are first born.</q> <rj><qau>Abp. Abbot.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Swathe</ent><br/
<hw>Swathe</hw>, <pos>n.</pos> <def>A bandage; a band; a swath.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Wrapped me in above an hundred yards of <qex>swathe</qex>.</q> <rj><qau>Addison.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Milk and a <qex>swathe</qex>, at first, his whole demand.</q> <rj><qau>Young.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>The solemn glory of the afternoon, with its long <qex>swathes</qex> of light between the far off rows of limes.</q> <rj><qau>G. Eliot.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Swather</ent><br/
<hw>Swath"er</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[See <er>Swath</er>, <pos>n.</pos>]</ety> <fld>(Agric.)</fld> <def>A device attached to a mowing machine for raising the uncut fallen grain and marking the limit of the swath.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Swatte</ent><br/
<hw>Swat"te</hw> <pr>(?)</pr>, <mark>obs.</mark> <def><pos>imp.</pos> of <er>Sweat</er>.</def>  <rj><au>Chaucer.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sway</ent><br/
<hw>Sway</hw> <pr>(sw<amac/)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Swayed</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Swaying</conjf>.]</vmorph> <ety>[OE. <ets>sweyen</ets>, Icel. <ets>sveigja</ets>, akin to E. <ets>swing</ets>; cf. D. <ets>zwaaijen</ets> to wield, swing. See <er>Swing</er>, and cf. <er>Swag</er>, <pos>v. i.</pos>]</ety> <sn>1.</sn> <def>To move or wield with the hand; to swing; to wield; <as>as, to <ex>sway</ex> the scepter</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>As sparkles from the anvil rise,<br/
When heavy hammers on the wedge are <qex>swayed</qex>.</q> <rj><qau>Spenser.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To influence or direct by power and authority; by persuasion, or by moral force; to rule; to govern; to guide.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The will of man is by his reason <qex>swayed</qex>.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>She could not <qex>sway</qex> her house.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>This was the race<br/
To <qex>sway</qex> the world, and land and sea subdue.</q> <rj><qau>Dryden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>To cause to incline or swing to one side, or backward and forward; to bias; to turn; to bend; warp; <as>as, reeds <ex>swayed</ex> by wind; judgment <ex>swayed</ex> by passion</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>As bowls run true by being made<br/
On purpose false, and to be <qex>swayed</qex>.</q> <rj><qau>Hudibras.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Let not temporal and little advantages <qex>sway</qex> you against a more durable interest.</q> <rj><qau>Tillotson.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <fld>(Naut.)</fld> <def>To hoist; <as>as, to <ex>sway</ex> up the yards</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><syn><b>Syn.</b> -- To bias; rule; govern; direct; influence; swing; move; wave; wield.</syn><br/
[<source>1913 Webster</source>]</p>

<p><-- p. 1457 --></p>

<p><ent>Sway</ent><br/
<hw>Sway</hw> <pr>(sw<amac/)</pr>, <pos>v. i.</pos> <sn>1.</sn> <def>To be drawn to one side by weight or influence; to lean; to incline.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The balance <qex>sways</qex> on our part.</q> <rj><qau>Bacon.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To move or swing from side to side; or backward and forward.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>To have weight or influence.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The example of sundry churches . . . doth <qex>sway</qex> much.</q> <rj><qau>Hooker.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>To bear sway; to rule; to govern.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Hadst thou <qex>swayed</qex> as kings should do.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sway</ent><br/
<hw>Sway</hw>, <pos>n.</pos> <sn>1.</sn> <def>The act of swaying; a swaying motion; the swing or sweep of a weapon.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>With huge two-handed <qex>sway</qex> brandished aloft.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Influence, weight, or authority that inclines to one side; <as>as, the <ex>sway</ex> of desires</as>.</def>  <rj><au>A. Tucker.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>Preponderance; turn or cast of balance.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Expert<br/
When to advance, or stand, or turn the <qex>sway</qex><br/
Of battle.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>Rule; dominion; control.</def>  <rj><au>Cowper.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>When vice prevails, and impious men bear <qex>sway</qex>,<br/
The post of honor is a private station.</q> <rj><qau>Addison.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <def>A switch or rod used by thatchers to bind their work.</def> <mark>[Prov. Eng.]</mark>  <rj><au>Halliwell.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><syn><b>Syn.</b> -- Rule; dominion; power; empire; control; influence; direction; preponderance; ascendency.</syn><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sway-backed</ent><br/
<hw>Sway"-backed`</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Having the back hollow or sagged, whether naturally or as the result of injury or weakness; -- said of horses and other animals.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sway bar</ent><br/
<hw>Sway bar</hw>. <fld>(Vehicles)</fld> <sd>(a)</sd> <def>A bar attached to the hounds, in the rear of the front axle, so as to slide on the reach as the axle is swung in turning the vehicle.</def> <sd>(b)</sd> <def>Either of the two bars used on couplong the oront ano rear sleds of a logging sled; also, the bar used to couple two logging cars.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Sway-bracing</ent><br/
<hw>Sway"-bra`cing</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Engin.)</fld> <def>The horizontal bracing of a bridge, which prevents its swaying.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Swayed</ent><br/
<hw>Swayed</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Bent down, and hollow in the back; sway-backed; -- said of a horse.</def>  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Swayful</ent><br/
<hw>Sway"ful</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Able to sway.</def> <mark>[R.]</mark>  <rj><au>Rush.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Swaying</ent><br/
<hw>Sway"ing</hw>, <pos>n.</pos> <def>An injury caused by violent strains or by overloading; -- said of the backs of horses.</def>  <rj><au>Crabb.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sweal</ent><br/
<hw>Sweal</hw> <pr>(?)</pr>, <pos>v. i.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Swealed</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Swealing</conjf>.]</vmorph> <ety>[OE. <ets>swelen</ets> to burn, AS. <ets>swelan</ets>; akin to G. <ets>schwelen</ets> to burn slowly, <ets>schw<uum/l</ets> sultry, Icel. <ets>svaela</ets> a thick smoke.]</ety> <def>To melt and run down, as the tallow of a candle; to waste away without feeding the flame.</def> <altsp>[Written also <asp>swale</asp>.]</altsp>  <rj><au>Sir W. Scott.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sweal</ent><br/
<hw>Sweal</hw>, <pos>v. t.</pos> <def>To singe; to scorch; to swale; <as>as, to <ex>sweal</ex> a pig by singeing off the hair</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Swear</ent><br/
<hw>Swear</hw> <pr>(?)</pr>, <pos>v. i.</pos> <vmorph>[<pos>imp.</pos> <conjf>Swore</conjf> <pr>(?)</pr>, formerly <conjf>Sware</conjf> <pr>(<?/)</pr>; <pos>p. p.</pos> <conjf>Sworn</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Swearing</conjf>.]</vmorph> <ety>[OE. <ets>swerien</ets>, AS. <ets>swerian</ets>; akin to D. <ets>zweren</ets>, OS. <ets>swerian</ets>, OHG. <ets>swerien</ets>, G. <ets>schw<oum/ren</ets>, Icel. <ets>sverja</ets>, Sw. <ets>sv<aum/rja</ets>, Dan. <ets>svaerge</ets>, Icel. & Sw. <ets>svara</ets> to answer, Dan. <ets>svare</ets>, Dan. & Sw. <ets>svar</ets> an answer, Goth. <ets>swaran</ets> to swear, and perhaps to E. <ets>swarm</ets>. <root/177.  Cf. <er>Answer</er>.]</ety> <sn>1.</sn> <def>To affirm or utter a solemn declaration, with an appeal to God for the truth of what is affirmed; to make a promise, threat, or resolve on oath; also, to affirm solemnly by some sacred object, or one regarded as sacred, as the Bible, the Koran, etc.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Ye shall <qex>swear</qex> by my name falsely.</q> <rj><qau>Lev. xix. 12.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>I <qex>swear</qex> by all the Roman gods.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Law)</fld> <def>To give evidence on oath; <as>as, to <ex>swear</ex> to the truth of a statement; he <ex>swore</ex> against the prisoner</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>To make an appeal to God in an irreverant manner; to use the name of God or sacred things profanely; to call upon God in imprecation; to curse.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>[I] <qex>swore</qex> little; diced not above seven times a week.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>To swear by</b></col>, <cd>to place great confidence in a person or thing; to trust implicitly as an authority.</cd>  <ldquo/I simply meant to ask if you are one of those who <xex>swear by</xex> Lord Verulam.<rdquo/ <au>Miss Edgeworth.</au> -- <col><b>To swear off</b></col>, <cd>to make a solemn vow, or a serious resolution, to abstain from something; <as>as, <ex>to swear off</ex> smoking</as>.</cd> <mark>[Slang]</mark></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Swear</ent><br/
<hw>Swear</hw>, <pos>v. t.</pos> <sn>1.</sn> <def>To utter or affirm with a solemn appeal to God for the truth of the declaration; to make (a promise, threat, or resolve) under oath.</def><br/
[<source>1913 Webster</source>]</p>

<p><q><qex>Swear</qex> unto me here by God, that thou wilt not deal falsely with me.</q> <rj><qau>Gen. xxi. 23.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>He <qex>swore</qex> consent to your succession.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Law)</fld> <def>To put to an oath; to cause to take an oath; to administer an oath to; -- ofetn followed by <xex>in</xex> or <xex>into</xex>; <as>as, to <ex>swear</ex> witnesses; to <ex>swear</ex> a jury; to <ex>swear</ex> in an officer; he was <ex>sworn</ex> into office</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>To declare or charge upon oath; <as>as, he <ex>swore</ex> treason against his friend</as>.</def>  <rj><au>Johnson.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>To appeal to by an oath.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Now, by Apollo, king,<br/
Thou <qex>swear'st</qex> thy gods in vain.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>To swear the peace against one</b></col>, <cd>to make oath that one is under the actual fear of death or bodily harm from the person, in which case the person must find sureties that he will keep the peace.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Swearer</ent><br/
<hw>Swear"er</hw> <pr>(?)</pr>, <pos>n.</pos> <sn>1.</sn> <def>One who swears; one who calls God to witness for the truth of his declaration.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A profane person; one who uses profane language.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Then the liars and <qex>swearers</qex> are fools.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Swearing</ent><br/
<hw>Swear"ing</hw>, <def><pos>a. & n.</pos> from <er>Swear</er>, <pos>v.</pos></def><br/
[<source>1913 Webster</source>]</p>

<p><q>Idle <qex>swearing</qex> is a cursedness.</q> <rj><qau>Chaucer.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sweat</ent><br/
<hw>Sweat</hw> <pr>(?)</pr>, <pos>v. i.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Sweat</conjf> or <conjf>Sweated</conjf> (<mark>Obs.</mark> <conjf>Swat</conjf> <pr>(<?/)</pr>); <pos>p. pr. & vb. n.</pos> <conjf>Sweating</conjf>.]</vmorph> <ety>[OE. <ets>sweten</ets>, AS. <ets>swaetan</ets>, fr. <ets>sw<amac/t</ets>, n., sweat; akin to OFries. & OS. <ets>sw<emac/t</ets>, D. <ets>zweet</ets>, OHG. <ets>sweiz</ets>, G. <ets>schweiss</ets>, Icel. <ets>sviti</ets>, <ets>sveiti</ets>, Sw. <ets>svett</ets>, Dan. <ets>sved</ets>, L. <ets>sudor</ets> sweat, <ets>sudare</ets> to sweat, Gr. <?/, <?/, sweat, <?/ to sweat, Skr. <ets>sv<emac/da</ets> sweat, <ets>svid</ets> to sweat. <root/178.  Cf. <er>Exude</er>, <er>Sudary</er>, <er>Sudorific</er>.]</ety> <sn>1.</sn> <def>To excrete sensible moisture from the pores of the skin; to perspire.</def>  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Fig.: To perspire in toil; to work hard; to drudge.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>He 'd have the poets <qex>sweat</qex>.</q> <rj><qau>Waller.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>To emit moisture, as green plants in a heap.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sweat</ent><br/
<hw>Sweat</hw>, <pos>v. t.</pos> <sn>1.</sn> <def>To cause to excrete moisture from the skin; to cause to perspire; <as>as, his physicians attempted to <ex>sweat</ex> him by most powerful sudorifics</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To emit or suffer to flow from the pores; to exude.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>It made her not a drop for <qex>sweat</qex>.</q> <rj><qau>Chaucer.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>With exercise she <qex>sweat</qex> ill humors out.</q> <rj><qau>Dryden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>To unite by heating, after the application of soldier.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>To get something advantageous, as money, property, or labor from (any one), by exaction or oppression; <as>as, to <ex>sweat</ex> a spendthrift; to <ex>sweat</ex> laborers</as>.</def> <mark>[Colloq.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>To sweat coin</b></col>, <cd>to remove a portion of a piece of coin, as by shaking it with others in a bag, so that the friction wears off a small quantity of the metal.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><q>The only use of it [money] which is interdicted is to put it in circulation again after having diminished its weight by <ldquo/<qex>sweating</qex><rdquo/, or otherwise, because the quantity of metal contains is no longer consistent with its impression.</q> <rj><qau>R. Cobden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sweat</ent><br/
<hw>Sweat</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Cf. OE. <ets>swot</ets>, AS. <ets>sw<amac/t</ets>. See <er>Sweat</er>, <pos>v. i.</pos>]</ety> <sn>1.</sn> <fld>(Physiol.)</fld> <def>The fluid which is excreted from the skin of an animal; the fluid secreted by the sudoriferous glands; a transparent, colorless, acid liquid with a peculiar odor, containing some fatty acids and mineral matter; perspiration. See <er>Perspiration</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>In the <qex>sweat</qex> of thy face shalt thou eat bread.</q> <rj><qau>Gen. iii. 19.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>The act of sweating; or the state of one who sweats; hence, labor; toil; drudgery.</def>  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>Moisture issuing from any substance; <as>as, the <ex>sweat</ex> of hay or grain in a mow or stack</as>.</def>  <rj><au>Mortimer.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>The sweating sickness.</def> <mark>[Obs.]</mark>  <rj><au>Holinshed.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <fld>(Man.)</fld> <def>A short run by a race horse in exercise.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Sweat box</b></col> <fld>(Naut.)</fld>, <cd>a small closet in which refractory men are confined.</cd> -- <col><b>Sweat glands</b></col> <fld>(Anat.)</fld>, <cd>sudoriferous glands. See under <er>Sudoriferous</er>.</cd></cs>
<-- sweat suit.  A suit comprising a top and trousers, having full arms and legs, used while performing physical exercises, esp. out-of-doors.
  Sweat equity.  The rights to a portion of ownership or profit, hypothetically owned by a worker who participated in producing a product, such as in improving a piece of real estate.
 --><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sweater</ent><br/
<hw>Sweat"er</hw> <pr>(?)</pr>, <pos>n.</pos> <sn>1.</sn> <def>One who sweats.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>One who, or that which, causes to sweat</def>; as: <sd>(a)</sd> <def>A sudorific.</def> <sd>(b)</sd> <def>A woolen jacket or jersey worn by athletes.</def> <sd>(c)</sd> <def>An employer who oppresses his workmen by paying low wages.</def> <mark>[Slang]</mark>
<-- jersey worn by lots of people, not just athletes.  Needs expansion. --><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sweatily</ent><br/
<hw>Sweat"i*ly</hw> <pr>(?)</pr>, <pos>adv.</pos> <def>In a sweaty manner.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sweatiness</ent><br/
<hw>Sweat"i*ness</hw>, <pos>n.</pos> <def>Quality or state of being sweaty.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sweating</ent><br/
<hw>Sweat"ing</hw>, <def><pos>a. & n.</pos> from <er>Sweat</er>, <pos>v.</pos></def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Sweating bath</b></col>, <cd>a bath producing sensible sweat; a stove or sudatory.</cd> -- <col><b>Sweating house</b></col>, <cd>a house for sweating persons in sickness.</cd> -- <col><b>Sweating iron</b></col>, <cd>a kind of knife, or a piece of iron, used to scrape off sweat, especially from horses; a horse scraper.</cd> -- <col><b>Sweating room</b></col>. <sd>(a)</sd> <cd>A room for sweating persons.</cd> <sd>(b)</sd> <fld>(Dairying)</fld> <cd>A room for sweating cheese and carrying off the superfluous juices.</cd> -- <col><b>Sweating sickness</b></col> <fld>(Med.)</fld>, <cd>a febrile epidemic disease which prevailed in some countries of Europe, but particularly in England, in the fifteenth and sixteenth centuries, characterized by profuse sweating. Death often occured in a few hours.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sweaty</ent><br/
<hw>Sweat"y</hw> <pr>(?)</pr>, <pos>a.</pos> <amorph>[<pos>Compar.</pos> <adjf>Sweatier</adjf> <pr>(?)</pr>; <pos>superl.</pos> <adjf>Sweatiest</adjf>.]</amorph> <sn>1.</sn> <def>Moist with sweat; <as>as, a <ex>sweaty</ex> skin; a <ex>sweaty</ex> garment.</as></def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Consisting of sweat; of the nature of sweat.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>No noisome whiffs or <qex>sweaty</qex> streams.</q> <rj><qau>Swift.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>Causing sweat; hence, laborious; toilsome; difficult.</def> <ldquo/The <xex>sweaty</xex> forge.<rdquo/  <rj><au>Prior.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Swede</ent><br/
<hw>Swede</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Cf. G. <ets>Schwede</ets>.]</ety> <sn>1.</sn> <def>A native or inhabitant of Sweden.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Bot.)</fld> <def>A Swedish turnip. See under <er>Turnip</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Swedenborgian</ent><br/
<hw>Swe`den*bor"gi*an</hw> <pr>(?)</pr>, <pos>n.</pos> <def>One who holds the doctrines of the New Jerusalem church, as taught by Emanuel <xex>Swedenborg</xex>, a Swedish philosopher and religious writer, who was born <sc>a. d.</sc> 1688 and died 1772. Swedenborg claimed to have intercourse with the spiritual world, through the opening of his spiritual senses in 1745. He taught that the Lord Jesus Christ, as comprehending in himself all the fullness of the Godhead, is the one only God, and that there is a spiritual sense to the Scriptures, which he (Swedenborg) was able to reveal, because he saw the correspondence between natural and spiritual things.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Swedenborgian</ent><br/
<hw>Swe`den*bor"gi*an</hw>, <pos>a.</pos> <def>Of or pertaining to Swedenborg or his views.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Swedenborgianism</ent><br/
<hw>Swe`den*bor"gi*an*ism</hw> <pr>(?)</pr>, <pos>n.</pos> <def>The doctrines of the Swedenborgians.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Swedish</ent><br/
<hw>Swed"ish</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[Cf. G. <ets>schwedisch</ets>, Sw. <ets>svensk</ets>.]</ety> <def>Of or pertaining to Sweden or its inhabitants.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Swedish turnip</b></col>. <fld>(Bot.)</fld> <cd>See under <er>Turnip</er>.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Swedish</ent><br/
<hw>Swed"ish</hw>, <pos>n.</pos> <def>The language of Swedes.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sweeny</ent><br/
<hw>Swee"ny</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Far.)</fld> <def>An atrophy of the muscles of the shoulder in horses; also, atrophy of any muscle in horses.</def> <altsp>[Written also <asp>swinney</asp>.]</altsp><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sweep</ent><br/
<hw>Sweep</hw> <pr>(?)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Swept</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Sweeping</conjf>.]</vmorph> <ety>[OE. <ets>swepen</ets>; akin to AS. <ets>sw<amac/pan</ets>. See <er>Swoop</er>, <pos>v. i.</pos>]</ety> <sn>1.</sn> <def>To pass a broom across (a surface) so as to remove loose dirt, dust, etc.; to brush, or rub over, with a broom for the purpose of cleaning; <as>as, to <ex>sweep</ex> a floor, the street, or a chimney</as>. Used also figuratively.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>I will <qex>sweep</qex> it with the besom of destruction.</q> <rj><qau>Isa. xiv. 23.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To drive or carry along or off with a broom or a brush, or as if with a broom; to remove by, or as if by, brushing; <as>as, to <ex>sweep</ex> dirt from a floor; the wind <ex>sweeps</ex> the snow from the hills; a freshet <ex>sweeps</ex> away a dam, timber, or rubbish; a pestilence <ex>sweeps</ex> off multitudes.</as></def><br/
[<source>1913 Webster</source>]</p>

<p><q>The hail shall <qex>sweep</qex> away the refuge of lies.</q> <rj><qau>Isa. xxviii. 17.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>I have already <qex>swept</qex> the stakes.</q> <rj><qau>Dryden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>To brush against or over; to rub lightly along.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Their long descending train,<br/
With rubies edged and sapphires, <qex>swept</qex> the plain.</q> <rj><qau>Dryden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>To carry with a long, swinging, or dragging motion; hence, to carry in a stately or proud fashion.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>And like a peacock <qex>sweep</qex> along his tail.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <def>To strike with a long stroke.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Wake into voice each silent string,<br/
And <qex>sweep</qex> the sounding lyre.</q> <rj><qau>Pope.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>6.</sn> <fld>(Naut.)</fld> <def>To draw or drag something over; <as>as, to <ex>sweep</ex> the bottom of a river with a net</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>7.</sn> <def>To pass over, or traverse, with the eye or with an instrument of observation; <as>as, to <ex>sweep</ex> the heavens with a telescope</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><mcol><col><b>To sweep a mold</b></col> <it>or</it> <col><b>To sweep up a mold</b></col></mcol> <fld>(Founding)</fld>, <cd>to form the sand into a mold by a templet, instead of compressing it around the pattern.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sweep</ent><br/
<hw>Sweep</hw> <pr>(?)</pr>, <pos>v. i.</pos> <sn>1.</sn> <def>To clean rooms, yards, etc., or to clear away dust, dirt, litter, etc., with a broom, brush, or the like.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To brush swiftly over the surface of anything; to pass with switness and force, as if brushing the surface of anything; to move in a stately manner; <as>as, the wind <ex>sweeps</ex> across the plain; a woman <ex>sweeps</ex> through a drawing-room.</as></def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>To pass over anything comprehensively; to range through with rapidity; <as>as, his eye <ex>sweeps</ex> through space</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sweep</ent><br/
<hw>Sweep</hw>, <pos>n.</pos> <sn>1.</sn> <def>The act of sweeping.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>The compass or range of a stroke; <as>as, a long <ex>sweep</ex></as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>The compass of any turning body or of any motion; <as>as, the <ex>sweep</ex> of a door; the <ex>sweep</ex> of the eye</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>The compass of anything flowing or brushing; <as>as, the flood carried away everything within its <ex>sweep</ex></as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <def>Violent and general destruction; <as>as, the <ex>sweep</ex> of an epidemic disease</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>6.</sn> <def>Direction and extent of any motion not rectlinear; <as>as, the <ex>sweep</ex> of a compass</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>7.</sn> <def>Direction or departure of a curve, a road, an arch, or the like, away from a rectlinear line.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The road which makes a small <qex>sweep</qex>.</q> <rj><qau>Sir W. Scott.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>8.</sn> <def>One who sweeps; a sweeper; specifically, a chimney sweeper.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>9.</sn> <fld>(Founding)</fld> <def>A movable templet for making molds, in loam molding.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>10.</sn> <fld>(Naut.)</fld> <sd>(a)</sd> <def>The mold of a ship when she begins to curve in at the rungheads; any part of a ship shaped in a segment of a circle.</def> <sd>(b)</sd> <def>A large oar used in small vessels, partly to propel them and partly to steer them.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>11.</sn> <fld>(Refining)</fld> <def>The almond furnace.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><sn>12.</sn> <def>A long pole, or piece of timber, moved on a horizontal fulcrum fixed to a tall post and used to raise and lower a bucket in a well for drawing water.</def> <altsp>[Variously written <asp>swape</asp>, <asp>sweep</asp>, <asp>swepe</asp>, and <asp>swipe</asp>.]</altsp><br/
[<source>1913 Webster</source>]</p>

<p><sn>13.</sn> <fld>(Card Playing)</fld> <def>In the game of casino, a pairing or combining of all the cards on the board, and so removing them all; in whist, the winning of all the tricks (thirteen) in a hand; a slam.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>14.</sn> <pluf>pl.</pluf> <def>The sweeping of workshops where precious metals are worked, containing filings, etc.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Sweep net</b></col>, <cd>a net for drawing over a large compass.</cd> -- <col><b>Sweep of the tiller</b></col> <fld>(Naut.)</fld>, <cd>a circular frame on which the tiller traverses.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sweepage</ent><br/
<hw>Sweep"age</hw> <pr>(?)</pr>, <pos>n.</pos> <def>The crop of hay got in a meadow.</def> <mark>[Prov. Eng.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sweeper</ent><br/
<hw>Sweep"er</hw> <pr>(?)</pr>, <pos>n.</pos> <def>One who, or that which, sweeps, or cleans by sweeping; a sweep; <as>as, a carpet <ex>sweeper</ex></as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>It is oxygen which is the great <qex>sweeper</qex> of the economy.</q> <rj><qau>Huxley.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sweeping</ent><br/
<hw>Sweep"ing</hw>, <pos>a.</pos> <def>Cleaning off surfaces, or cleaning away dust, dirt, or litter, as a broom does; moving with swiftness and force; carrying everything before it; including in its scope many persons or things; <as>as, a <ex>sweeping</ex> flood; a <ex>sweeping</ex> majority; a <ex>sweeping</ex> accusation.</as></def> -- <wordforms><wf>Sweep"ing*ly</wf>, <pos>adv.</pos> -<wf>Sweep"ing*ness</wf>, <pos>n.</pos></wordforms>
<-- should be separate senses --><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sweepings</ent><br/
<hw>Sweep"ings</hw> <pr>(?)</pr>, <pos>n. pl.</pos> <def>Things collected by sweeping; rubbish; <as>as, the <ex>sweepings</ex> of a street</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sweep-saw</ent><br/
<hw>Sweep"-saw`</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A bow-saw.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sweepstake</ent><br/
<hw>Sweep"stake`</hw> <pr>(?)</pr>, <pos>n.</pos> <sn>1.</sn> <def>A winning of all the stakes or prizes.</def>  <rj><au>Heylin.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A complete removal or carrying away; a clean sweep.</def> <mark>[Obs.]</mark>  <rj><au>Bp. Hacket.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sweepstakes</ent><br/
<hw>Sweep"stakes`</hw> <pr>(?)</pr>, <pos>n.</pos> <sn>1.</sn> <def>A winning of all the stakes or prizes; a sweepstake.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <pos>sing. <or/ pl.</pos> <def>The whole money or other things staked at a horse race, a given sum being put up for each horse, all of which goes to the winner, or is divided among several, as may be previously agreed.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>A race for all the sums staked or prizes offered.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sweepwasher</ent><br/
<hw>Sweep"wash`er</hw> <pr>(?)</pr>, <pos>n.</pos> <def>One who extracts the residuum of precious metals from the sweepings, potsherds, etc., of refineries of gold and silver, or places where these metals are used.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sweepy</ent><br/
<hw>Sweep"y</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Moving with a sweeping motion.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The branches bend before their <qex>sweepy</qex> away.</q> <rj><qau>Dryden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><-- p. 1458 --></p>

<p><ent>Sweet</ent><br/
<hw>Sweet</hw> <pr>(?)</pr>, <pos>a.</pos> <amorph>[<pos>Compar.</pos> <adjf>Sweeter</adjf> <pr>(?)</pr>; <pos>superl.</pos> <adjf>Sweetest</adjf>.]</amorph> <ety>[OE. <ets>swete</ets>, <ets>swote</ets>, <ets>sote</ets>, AS. <ets>sw<emac/te</ets>; akin to OFries. <ets>sw<emac/te</ets>, OS. <ets>sw<omac/ti</ets>, D. <ets>zoet</ets>, G. <ets>s<uum/ss</ets>, OHG. <ets>suozi</ets>, Icel. <ets>saetr</ets>, <ets>soetr</ets>, Sw. <ets>s<oum/t</ets>, Dan. <ets>s<oum/d</ets>, Goth. <ets>suts</ets>, L. <ets>suavis</ets>, for <ets>suadvis</ets>, Gr. <?/, Skr. <ets>sv<amac/du</ets> sweet, <ets>svad</ets>, <ets>sv<amac/d</ets>, to sweeten. <root/175.  Cf. <er>Assuage</er>, <er>Suave</er>, <er>Suasion</er>.]</ety> <sn>1.</sn> <def>Having an agreeable taste or flavor such as that of sugar; saccharine; -- opposed to <xex>sour</xex> and <xex>bitter</xex>; <as>as, a <ex>sweet</ex> beverage; <ex>sweet</ex> fruits; <ex>sweet</ex> oranges.</as></def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Pleasing to the smell; fragrant; redolent; balmy; <as>as, a <ex>sweet</ex> rose; <ex>sweet</ex> odor; <ex>sweet</ex> incense.</as></def><br/
[<source>1913 Webster</source>]</p>

<p><q>The breath of these flowers is <qex>sweet</qex> to me.</q> <rj><qau>Longfellow.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>Pleasing to the ear; soft; melodious; harmonious; <as>as, the <ex>sweet</ex> notes of a flute or an organ; <ex>sweet</ex> music; a <ex>sweet</ex> voice; a <ex>sweet</ex> singer.</as></def><br/
[<source>1913 Webster</source>]</p>

<p><q>To make his English <qex>sweet</qex> upon his tongue.</q> <rj><qau>Chaucer.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>A voice <qex>sweet</qex>, tremulous, but powerful.</q> <rj><qau>Hawthorne.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>Pleasing to the eye; beautiful; mild and attractive; fair; <as>as, a <ex>sweet</ex> face; a <ex>sweet</ex> color or complexion.</as></def><br/
[<source>1913 Webster</source>]</p>

<p><q><qex>Sweet</qex> interchange<br/
Of hill and valley, rivers, woods, and plains.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <def>Fresh; not salt or brackish; <as>as, <ex>sweet</ex> water</as>.</def>  <rj><au>Bacon.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>6.</sn> <def>Not changed from a sound or wholesome state. Specifically: <sd>(a)</sd> Not sour; <as>as, <ex>sweet</ex> milk or bread</as>. <sd>(b)</sd> Not state; not putrescent or putrid; not rancid; <as>as, <ex>sweet</ex> butter; <ex>sweet</ex> meat or fish</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>7.</sn> <def>Plaesing to the mind; mild; gentle; calm; amiable; winning; presuasive; <as>as, <ex>sweet</ex> manners</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Canst thou bind the <qex>sweet</qex> influence of Pleiades?</q> <rj><qau>Job xxxviii. 31.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Mildness and <qex>sweet</qex> reasonableness is the one established rule of Christian working.</q> <rj><qau>M. Arnold.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ <xex>Sweet</xex> is often used in the formation of self-explaining compounds; as, <xex>sweet</xex>-blossomed, <xex>sweet</xex>-featured, <xex>sweet</xex>-smelling, <xex>sweet</xex>-tempered, <xex>sweet</xex>-toned, etc.</note><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Sweet alyssum</b></col>. <fld>(Bot.)</fld> <cd>See <er>Alyssum</er>.</cd> -- <col><b>Sweet apple</b></col>. <fld>(Bot.)</fld> <sd>(a)</sd> <cd>Any apple of sweet flavor.</cd> <sd>(b)</sd> <cd>See <er>Sweet-sop</er>.</cd> -- <col><b>Sweet bay</b></col>. <fld>(Bot.)</fld> <sd>(a)</sd> <cd>The laurel (<spn>Laurus nobilis</spn>).</cd> <sd>(b)</sd> <cd>Swamp sassafras.</cd> -- <col><b>Sweet calabash</b></col> <fld>(Bot.)</fld>, <cd>a plant of the genus <gen>Passiflora</gen> (<spn>Passiflora maliformis</spn>) growing in the West Indies, and producing a roundish, edible fruit, the size of an apple.</cd> -- <col><b>Sweet cicely</b></col>. <fld>(Bot.)</fld> <sd>(a)</sd> <cd>Either of the North American plants of the umbelliferous genus <gen>Osmorrhiza</gen> having aromatic roots and seeds, and white flowers.</cd> <au>Gray.</au> <sd>(b)</sd> <cd>A plant of the genus <gen>Myrrhis</gen> (<spn>Myrrhis odorata</spn>) growing in England.</cd> -- <mcol><col><b>Sweet calamus</b></col>, <it>or</it> <col><b>Sweet cane</b></col></mcol>. <fld>(Bot.)</fld> <cd>Same as <cref>Sweet flag</cref>, below.</cd> -- <col><b>Sweet Cistus</b></col> <fld>(Bot.)</fld>, <cd>an evergreen shrub (<spn>Cistus Ladanum</spn>) from which the gum ladanum is obtained.</cd> -- <col><b>Sweet clover</b></col>. <fld>(Bot.)</fld> <cd>See <er>Melilot</er>.</cd> -- <col><b>Sweet coltsfoot</b></col> <fld>(Bot.)</fld>, <cd>a kind of butterbur (<spn>Petasites sagittata</spn>) found in Western North America.</cd> -- <col><b>Sweet corn</b></col> <fld>(Bot.)</fld>, <cd>a variety of the maize of a sweet taste. See the Note under <er>Corn</er>.</cd> -- <col><b>Sweet fern</b></col> <fld>(Bot.)</fld>, <cd>a small North American shrub (<spn>Comptonia asplenifolia</spn> syn. <spn>Myrica asplenifolia</spn>) having sweet-scented or aromatic leaves resembling fern leaves.</cd> -- <col><b>Sweet flag</b></col> <fld>(Bot.)</fld>, <cd>an endogenous plant (<spn>Acorus Calamus</spn>) having long flaglike leaves and a rootstock of a pungent aromatic taste. It is found in wet places in Europe and America. See <er>Calamus</er>, 2.</cd> -- <col><b>Sweet gale</b></col> <fld>(Bot.)</fld>, <cd>a shrub (<spn>Myrica Gale</spn>) having bitter fragrant leaves; -- also called <altname>sweet willow</altname>, and <altname>Dutch myrtle</altname>. See 5th <er>Gale</er>.</cd> -- <col><b>Sweet grass</b></col> <fld>(Bot.)</fld>, <cd>holy, or Seneca, grass.</cd> -- <col><b>Sweet gum</b></col> <fld>(Bot.)</fld>, <cd>an American tree (<spn>Liquidambar styraciflua</spn>). See <er>Liquidambar</er>.</cd> -- <col><b>Sweet herbs</b></col>, <cd>fragrant herbs cultivated for culinary purposes.</cd> -- <col><b>Sweet John</b></col> <fld>(Bot.)</fld>, <cd>a variety of the sweet William.</cd> -- <col><b>Sweet leaf</b></col> <fld>(Bot.)</fld>, <cd>horse sugar.  See under <er>Horse</er>.</cd> -- <col><b>Sweet marjoram</b></col>. <fld>(Bot.)</fld> <cd>See <er>Marjoram</er>.</cd> -- <col><b>Sweet marten</b></col> <fld>(Zool.)</fld>, <cd>the pine marten.</cd> -- <col><b>Sweet maudlin</b></col> <fld>(Bot.)</fld>, <cd>a composite plant (<spn>Achillea Ageratum</spn>) allied to milfoil.</cd> -- <col><b>Sweet oil</b></col>, <cd>olive oil.</cd> -- <col><b>Sweet pea</b></col>. <fld>(Bot.)</fld> <cd>See under <er>Pea</er>.</cd> -- <col><b>Sweet potato</b></col>. <fld>(Bot.)</fld> <cd>See under <er>Potato</er>.</cd> -- <col><b>Sweet rush</b></col> <fld>(Bot.)</fld>, <cd>sweet flag.</cd> -- <col><b>Sweet spirits of niter</b></col> <fld>(Med. Chem.)</fld> <cd>See <cref>Spirit of nitrous ether</cref>, under <er>Spirit</er>.</cd> -- <col><b>Sweet sultan</b></col> <fld>(Bot.)</fld>, <cd>an annual composite plant (<spn>Centaurea moschata</spn>), also, the yellow-flowered (<spn>Centaurea odorata</spn>); -- called also <altname>sultan flower</altname>.</cd> -- <col><b>Sweet tooth</b></col>, <cd>an especial fondness for sweet things or for sweetmeats.</cd> <mark>[Colloq.]</mark> -- <col><b>Sweet William</b></col>. <sd>(a)</sd> <fld>(Bot.)</fld> <cd>A species of pink (<spn>Dianthus barbatus</spn>) of many varieties.</cd> <sd>(b)</sd> <fld>(Zool.)</fld> <cd>The willow warbler.</cd> <sd>(c)</sd> <fld>(Zool.)</fld> <cd>The European goldfinch; -- called also <altname>sweet Billy</altname>.</cd> <mark>[Prov. Eng.]</mark> -- <col><b>Sweet willow</b></col> <fld>(Bot.)</fld>, <cd>sweet gale.</cd> -- <col><b>Sweet wine</b></col>. <cd>See <cref>Dry wine</cref>, under <er>Dry</er>.</cd> -- <col><b>To be sweet on</b></col>, <cd>to have a particular fondness for, or special interest in, as a young man for a young woman.</cd> <mark>[Colloq.]</mark> <au>Thackeray.</au></cs><br/
[<source>1913 Webster</source>]</p>

<p><syn><b>Syn.</b> -- Sugary; saccharine; dulcet; luscious.</syn><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sweet</ent><br/
<hw>Sweet</hw> <pr>(?)</pr>, <pos>n.</pos> <sn>1.</sn> <def>That which is sweet to the taste; -- used chiefly in the plural.</def> Specifically: <sd>(a)</sd> <def>Confectionery, sweetmeats, preserves, etc.</def> <sd>(b)</sd> <def>Home-made wines, cordials, metheglin, etc.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>That which is sweet or pleasant in odor; a perfume.</def> <ldquo/A wilderness of <xex>sweets</xex>.<rdquo/  <rj><au>Milton.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>That which is pleasing or grateful to the mind; <as>as, the <ex>sweets</ex> of domestic life</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>A little bitter mingled in our cup leaves no relish of the <qex>sweet</qex>.</q> <rj><qau>Locke.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>One who is dear to another; a darling; -- a term of endearment.</def> <ldquo/Wherefore frowns my <xex>sweet</xex>?<rdquo/  <rj><au>B. Jonson.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sweet</ent><br/
<hw>Sweet</hw>, <pos>adv.</pos> <def>Sweetly.</def>  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sweet</ent><br/
<hw>Sweet</hw>, <pos>v. t.</pos> <def>To sweeten.</def> <mark>[Obs.]</mark>  <rj><au>Udall.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sweetbread</ent><br/
<hw>Sweet"bread`</hw> <pr>(sw<emac/t"br<ecr/d`)</pr>, <pos>n.</pos> <sn>1.</sn> <def>Either the thymus gland or the pancreas, the former being called <stype>neck sweetbread</stype> or <stype>throat sweetbread</stype>, the latter <stype>belly sweetbread</stype>.  The sweetbreads of ruminants, esp. of the calf, are highly esteemed as food.  See <er>Pancreas</er>, and <er>Thymus</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Anat.)</fld> <def>The pancreas.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sweet-breasted</ent><br/
<hw>Sweet"-breast`ed</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Having a sweet, musical voice, as the nightingale.  Cf. <er>Breast</er>, <pos>n.</pos>, 6.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sweetbrier</ent><br/
<hw>Sweet"bri`er</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Bot.)</fld> <def>A kind of rose (<spn>Rosa rubiginosa</spn>) with minutely glandular and fragrant foliage. The small-flowered sweetbrier is <spn>Rosa micrantha</spn>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sweeten</ent><br/
<hw>Sweet"en</hw> <pr>(?)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Sweetened</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Sweetening</conjf>.]</vmorph> <ety>[See <er>Sweet</er>, <pos>a.</pos>]</ety> <sn>1.</sn> <def>To make sweet to the taste; <as>as, to <ex>sweeten</ex> tea</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To make pleasing or grateful to the mind or feelings; <as>as, to <ex>sweeten</ex> life; to <ex>sweeten</ex> friendship</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>To make mild or kind; to soften; <as>as, to <ex>sweeten</ex> the temper</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>To make less painful or laborious; to relieve; <as>as, to <ex>sweeten</ex> the cares of life</as>.</def>  <rj><au>Dryden.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>And <qex>sweeten</qex> every secret tear.</q> <rj><qau>Keble.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <def>To soften to the eye; to make delicate.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Correggio has made his memory immortal by the strength he has given to his figures, and by <qex>sweetening</qex> his lights and shadows, and melting them into each other.</q> <rj><qau>Dryden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>6.</sn> <def>To make pure and salubrious by destroying noxious matter; <as>as, to <ex>sweeten</ex> rooms or apartments that have been infected; to <ex>sweeten</ex> the air.</as></def><br/
[<source>1913 Webster</source>]</p>

<p><sn>7.</sn> <def>To make warm and fertile; -- opposed to <xex>sour</xex>; <as>as, to dry and <ex>sweeten</ex> soils</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>8.</sn> <def>To restore to purity; to free from taint; <as>as, to <ex>sweeten</ex> water, butter, or meat</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><-- 9. To make more attractive; -- said of offers in negotiations, as, to sweeten the deal by increasing the price offered. --><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sweeten</ent><br/
<hw>Sweet"en</hw>, <pos>v. i.</pos> <def>To become sweet.</def>  <rj><au>Bacon.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sweetener</ent><br/
<hw>Sweet"en*er</hw> <pr>(?)</pr>, <pos>n.</pos> <def>One who, or that which, sweetens; one who palliates; that which moderates acrimony.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sweetening</ent><br/
<hw>Sweet"en*ing</hw>, <pos>n.</pos> <sn>1.</sn> <def>The act of making sweet.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>That which sweetens.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sweetheart</ent><br/
<hw>Sweet"heart`</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A lover of mistress.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sweethearting</ent><br/
<hw>Sweet"heart`ing</hw>, <pos>n.</pos> <def>Making love.</def> <ldquo/To play at <xex>sweethearting</xex>.<rdquo/  <rj><au>W. Black.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sweeting</ent><br/
<hw>Sweet"ing</hw>, <pos>n.</pos> <sn>1.</sn> <def>A sweet apple.</def>  <rj><au>Ascham.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A darling; -- a word of endearment.</def>  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sweetish</ent><br/
<hw>Sweet"ish</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Somewhat sweet.</def> -- <wordforms><wf>Sweet"ish*ness</wf>, <pos>n.</pos></wordforms><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sweetly</ent><br/
<hw>Sweet"ly</hw>, <pos>adv.</pos> <ety>[AS. <ets>sw<emac/tlice</ets>.]</ety> <def>In a sweet manner.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sweetmeat</ent><br/
<hw>Sweet"meat`</hw> <pr>(?)</pr>, <pos>n.</pos> <sn>1.</sn> <def>Fruit preserved with sugar, as peaches, pears, melons, nuts, orange peel, etc.; -- usually in the plural; a confect; a confection.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>The paint used in making patent leather.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <fld>(Zool.)</fld> <def>A boat shell (<spn>Crepidula fornicata</spn>) of the American coast.</def> <mark>[Local, U.S.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sweetness</ent><br/
<hw>Sweet"ness</hw>, <pos>n.</pos> <ety>[AS. <ets>sw<emac/tness</ets>.]</ety> <def>The quality or state of being sweet (in any sense of the adjective); gratefulness to the taste or to the smell; agreeableness.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sweetroot</ent><br/
<hw>Sweet"root`</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Bot.)</fld> <def>Licorice.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sweet-scented</ent><br/
<hw>Sweet"-scent`ed</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Having a sweet scent or smell; fragrant.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Sweet-scented shrub</b></col> <fld>(Bot.)</fld>, <cd>a shrub of the genus <gen>Calycanthus</gen>, the flowers of which, when crushed, have a fragrance resembling that of strawberries.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sweet-sop</ent><br/
<hw>Sweet"-sop`</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Bot.)</fld> <def>A kind of custard apple (<spn>Anona squamosa</spn>). See under <er>Custard</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sweetwater</ent><br/
<hw>Sweet"wa`ter</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Bot.)</fld> <def>A variety of white grape, having a sweet watery juice; -- also called <altname>white sweetwater</altname>, and <altname>white muscadine</altname>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sweetweed</ent><br/
<hw>Sweet"weed`</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Bot.)</fld> <def>A name for two tropical American weeds (<spn>Capraria biflora</spn>, and <spn>Scoparia dulcis</spn>) of the Figwort family.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sweetwood</ent><br/
<hw>Sweet"wood`</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Bot.)</fld> <sd>(a)</sd> <def>The true laurel (<spn>Laurus nobilis</spn>.)</def> <sd>(b)</sd> <def>The timber of the tree <spn>Oreodaphne Leucoxylon</spn>, growing in Jamaica. The name is also applied to the timber of several other related trees.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sweetwort</ent><br/
<hw>Sweet"wort`</hw> <pr>(?)</pr>, <pos>n.</pos> <def>Any plant of a sweet taste.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sweigh</ent><br/
<hw>Sweigh</hw> <pr>(?)</pr>, <pos>n.</pos> <def>Sway; movement.</def> <mark>[Obs.]</mark>  <rj><au>Chaucer.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sweinmote</ent><br/
<hw>Swein"mote`</hw> <pr>(?)</pr>, <pos>n.</pos> <def>See <er>Swainmote</er>.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Swell</ent><br/
<hw>Swell</hw> <pr>(?)</pr>, <pos>v. i.</pos> <vmorph>[<pos>imp.</pos> <conjf>Swelled</conjf> <pr>(?)</pr>; <pos>p. p.</pos> <conjf>Swelled</conjf> or <conjf>Swollen</conjf> <pr>(<?/)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Swelling</conjf>.]</vmorph> <ety>[AS. <ets>swellan</ets>; akin to D. <ets>zwellen</ets>, OS. & OHG. <ets>swellan</ets>, G. <ets>schwellen</ets>, Icel. <ets>svella</ets>, Sw. <ets>sv<aum/lla</ets>.]</ety> <sn>1.</sn> <def>To grow larger; to dilate or extend the exterior surface or dimensions, by matter added within, or by expansion of the inclosed substance; <as>as, the legs <ex>swell</ex> in dropsy; a bruised part <ex>swells</ex>; a bladder <ex>swells</ex> by inflation.</as></def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To increase in size or extent by any addition; to increase in volume or force; <as>as, a river <ex>swells</ex>, and overflows its banks; sounds <ex>swell</ex> or diminish.</as></def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>To rise or be driven into waves or billows; to heave; <as>as, in tempest, the ocean <ex>swells</ex> into waves</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>To be puffed up or bloated; <as>as, to <ex>swell</ex> with pride</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>You <qex>swell</qex> at the tartan, as the bull is said to do at scarlet.</q> <rj><qau>Sir W. Scott.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <def>To be inflated; to belly; <as>as, the sails <ex>swell</ex></as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>6.</sn> <def>To be turgid, bombastic, or extravagant; <as>as, <ex>swelling</ex> words; a <ex>swelling</ex> style.</as></def><br/
[<source>1913 Webster</source>]</p>

<p><sn>7.</sn> <def>To protuberate; to bulge out; <as>as, a cask <ex>swells</ex> in the middle</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>8.</sn> <def>To be elated; to rise arrogantly.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Your equal mind yet <qex>swells</qex> not into state.</q> <rj><qau>Dryden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>9.</sn> <def>To grow upon the view; to become larger; to expand.</def> <ldquo/Monarchs to behold the <xex>swelling</xex> scene!<rdquo/  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>10.</sn> <def>To become larger in amount; <as>as, many little debts added, <ex>swell</ex> to a great amount</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>11.</sn> <def>To act in a pompous, ostentatious, or arrogant manner; to strut; to look big.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Here he comes, <qex>swelling</qex> like a turkey cock.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Swell</ent><br/
<hw>Swell</hw>, <pos>v. t.</pos> <sn>1.</sn> <def>To increase the size, bulk, or dimensions of; to cause to rise, dilate, or increase; <as>as, rains and dissolving snow <ex>swell</ex> the rivers in spring; immigration <ex>swells</ex> the population</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>[The Church] <qex>swells</qex> her high, heart-cheering tone.</q> <rj><qau>Keble.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To aggravate; to heighten.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>It is low ebb with his accuser when such peccadilloes are put to <qex>swell</qex> the charge.</q> <rj><qau>Atterbury.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>To raise to arrogance; to puff up; to inflate; <as>as, to be <ex>swelled</ex> with pride or haughtiness</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <fld>(Mus.)</fld> <def>To augment gradually in force or loudness, as the sound of a note.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Swell</ent><br/
<hw>Swell</hw>, <pos>n.</pos> <sn>1.</sn> <def>The act of swelling.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Gradual increase.</def> Specifically: <sd>(a)</sd> <def>Increase or augmentation in bulk; protuberance.</def> <sd>(b)</sd> <def>Increase in height; elevation; rise.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Little River affords navigation during a <qex>swell</qex> to within three miles of the Miami.</q> <rj><qau>Jefferson.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sd>(c)</sd> <def>Increase of force, intensity, or volume of sound.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Music arose with its voluptuous <qex>swell</qex>.</q> <rj><qau>Byron.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sd>(d)</sd> <def>Increase of power in style, or of rhetorical force.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The <qex>swell</qex> and subsidence of his periods.</q> <rj><qau>Landor.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>A gradual ascent, or rounded elevation, of land; <as>as, an extensive plain abounding with little <ex>swells</ex></as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>A wave, or billow; especially, a succession of large waves; the roll of the sea after a storm; <as>as, a heavy <ex>swell</ex> sets into the harbor</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The <qex>swell</qex><br/
Of the long waves that roll in yonder bay.</q> <rj><qau>Tennyson.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>The gigantic <qex>swells</qex> and billows of the snow.</q> <rj><qau>Hawthorne.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <fld>(Mus.)</fld> <def>A gradual increase and decrease of the volume of sound; the crescendo and diminuendo combined; -- generally indicated by the sign.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>6.</sn> <def>A showy, dashing person; a dandy.</def> <mark>[Slang]</mark><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Ground swell</b></col>. <cd>See under <er>Ground</er>.</cd> -- <col><b>Organ swell</b></col> <fld>(Mus.)</fld>, <cd>a certain number of pipes inclosed in a box, the uncovering of which by means of a pedal produces increased sound.</cd> -- <col><b>Swell shark</b></col> <fld>(Zool.)</fld>, <cd>a small shark (<spn>Scyllium ventricosum</spn>) of the west coast of North America, which takes in air when caught, and swells up like a swellfish.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Swell</ent><br/
<hw>Swell</hw>, <pos>a.</pos> <def>Having the characteristics of a person of rank and importance; showy; dandified; distinguished; <as>as, a <ex>swell</ex> person; a <ex>swell</ex> neighborhood.</as></def> <mark>[Slang]</mark><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Swell mob</b></col>. <cd>See under <er>Mob</er>.</cd> <mark>[Slang]</mark></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Swelldom</ent><br/
<hw>Swell"dom</hw> <pr>(?)</pr>, <pos>n.</pos> <def>People of rank and fashion; the class of swells, collectively.</def> <mark>[Jocose]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Swellfish</ent><br/
<hw>Swell"fish`</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Zool.)</fld> <def>Any plectognath fish that dilates itself, as the bur fish, puffer, or diodon.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Swelling</ent><br/
<hw>Swell"ing</hw>, <pos>n.</pos> <sn>1.</sn> <def>The act of that which swells; <as>as, the <ex>swelling</ex> of rivers in spring; the <ex>swelling</ex> of the breast with pride.</as></def><br/
[<source>1913 Webster</source>]</p>

<p><q>Rise to the <qex>swelling</qex> of the voiceless sea.</q> <rj><qau>Coleridge.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A protuberance; a prominence</def>; especially <fld>(Med.)</fld>, <def>an unnatural prominence or protuberance; <as>as, a scrofulous <ex>swelling</ex></as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The superficies of such plates are not even, but have many cavities and <qex>swellings</qex>.</q> <rj><qau>Sir I. Newton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Swellish</ent><br/
<hw>Swell"ish</hw>, <pos>a.</pos> <def>Dandified; stylish.</def> <mark>[Slang]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Swelltoad</ent><br/
<hw>Swell"toad`</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Zool.)</fld> <def>A swellfish.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Swelt</ent><br/
<hw>Swelt</hw> <pr>(?)</pr>, <mark>obs.</mark> <def><pos>imp.</pos> of <er>Swell</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Swelt</ent><br/
<hw>Swelt</hw>, <pos>v. i.</pos> <ety>[OE. <ets>swelten</ets> to die, to swoon or faint, AS. <ets>sweltan</ets> to die; akin to OD. <ets>swelten</ets> to hunger, to fail, OS. <ets>sweltan</ets> to die, Icel. <ets>svelta</ets> to die, to hunger, Sw. <ets>sv<aum/lta</ets> to hunger, Dan. <ets>sulte</ets>, Goth. <ets>sviltan</ets> to die.  Cf. <er>Swelter</er>, <er>Sweltry</er>.]</ety> <sn>1.</sn> <def>To die; to perish.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To faint; to swoon.</def> <mark>[Obs.]</mark>  <rj><au>Chaucer.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Night she <qex>swelt</qex> for passing joy.</q> <rj><qau>Spenser.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Swelt</ent><br/
<hw>Swelt</hw>, <pos>v. t.</pos> <def>To overpower, as with heat; to cause to faint; to swelter.</def> <mark>[Obs. or Prov. Eng.]</mark>  <rj><au>Bp. Hall.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Swelter</ent><br/
<hw>Swel"ter</hw> <pr>(?)</pr>, <pos>v. i.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Sweltered</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Sweltering</conjf>.]</vmorph> <ety>[From <er>Swelt</er>, <pos>v. i.</pos>]</ety> <sn>1.</sn> <def>To be overcome and faint with heat; to be ready to perish with heat.</def> <ldquo/<xex>Sweltered</xex> cattle.<rdquo/  <rj><au>Coleridge.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To welter; to soak.</def> <mark>[Obs.]</mark>  <rj><au>Drayton.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Swelter</ent><br/
<hw>Swel"ter</hw>, <pos>v. t.</pos> <sn>1.</sn> <def>To oppress with heat.</def>  <rj><au>Bentley.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To exude, like sweat.</def> <mark>[R.]</mark>  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sweltry</ent><br/
<hw>Swel"try</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[See <er>Swelter</er>, <er>Swelt</er>, <pos>v. i.</pos>, and cf. <er>Sultry</er>.]</ety> <def>Suffocating with heat; oppressively hot; sultry.</def> <mark>[R.]</mark>  <rj><au>Evelyn.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Swelve</ent><br/
<hw>Swel"ve</hw> <pr>(?)</pr>, <pos>v. t.</pos> <def>To swallow.</def> <mark>[Obs.]</mark>  <rj><au>Chaucer.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Swept</ent><br/
<hw>Swept</hw> <pr>(?)</pr>, <def><pos>imp. & p. p.</pos> of <er>Sweep</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Swerd</ent><br/
<hw>Swerd</hw> <pr>(?)</pr>, <pos>n. & v.</pos> <def>See <er>Sward</er>, <pos>n. & v.</pos></def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Swerd</ent><br/
<hw>Swerd</hw>, <pos>n.</pos> <def>Sword.</def> <mark>[Obs.]</mark>  <rj><au>Chaucer.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Swerve</ent><br/
<hw>Swerve</hw> <pr>(?)</pr>, <pos>v. i.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Swerved</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Swerving</conjf>.]</vmorph> <ety>[OE. <ets>swerven</ets>, AS. <ets>sweorfan</ets> to wipe off, to file, to polish; akin to OFries. <ets>swerva</ets> to creep, D. <ets>zwerven</ets> to swerve, to rope, OS. <ets>swerban</ets> to wipe off, MHG. <ets>swerben</ets> to be whirled, OHG. <ets>swerban</ets> to wipe off, Icel. <ets>sverfa</ets> to file, Goth. <ets>swa<iacute/rban</ets> (in comp.) to wipe, and perhaps to E. <ets>swarm</ets>.  Cf. <er>Swarm</er>.]</ety> <sn>1.</sn> <def>To stray; to wander; to rope.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>A maid thitherward did run,<br/
To catch her sparrow which from her did <qex>swerve</qex>.</q> <rj><qau>Sir P. Sidney.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To go out of a straight line; to deflect.</def> <ldquo/The point [of the sword] <xex>swerved</xex>.<rdquo/  <rj><au>Sir P. Sidney.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>To wander from any line prescribed, or from a rule or duty; to depart from what is established by law, duty, custom, or the like; to deviate.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>I <qex>swerve</qex> not from thy commandments.</q> <rj><qau>Bk. of Com. Prayer.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>They <qex>swerve</qex> from the strict letter of the law.</q> <rj><qau>Clarendon.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Many who, through the contagion of evil example, <qex>swerve</qex> exceedingly from the rules of their holy religion.</q> <rj><qau>Atterbury.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>To bend; to incline.</def> <ldquo/The battle <xex>swerved</xex>.<rdquo/  <rj><au>Milton.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <def>To climb or move upward by winding or turning.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The tree was high;<br/
Yet nimbly up from bough to bough I <qex>swerved</qex>.</q> <rj><qau>Dryden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Swerve</ent><br/
<hw>Swerve</hw>, <pos>v. t.</pos> <def>To turn aside.</def>  <rj><au>Gauden.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sweven</ent><br/
<hw>Swe"ven</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[AS. <ets>swefen</ets> sleep, dream; akin to <ets>swebban</ets>, <ets>swefian</ets>, to put to sleep, to kill. <root/176. See <er>Somnolent</er>.]</ety> <def>A vision seen in sleep; a dream.</def> <mark>[Obs.]</mark>  <rj><au>Wycliff (Acts ii. 17).</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>I defy both <qex>sweven</qex> and dream.</q> <rj><qau>Chaucer.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Swich</ent><br/
<hw>Swich</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[See <er>Such</er>.]</ety> <def>Such.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q><qex>Swich</qex> things as that I know I will declare.</q> <rj><qau>Chaucer.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Swietenia</ent><br/
||<hw>Swie*te"ni*a</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[NL. Named after Gerard Van <ets>Sweiten</ets>, physician to Maria Theresa of Austria.]</ety> <fld>(Bot.)</fld> <def>A genus of meliaceous trees consisting of one species (<spn>Sweitenia Mahogoni</spn>), the mahogany tree.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Swift</ent><br/
<hw>Swift</hw> <pr>(sw<icr/ft)</pr>, <pos>a.</pos> <amorph>[<pos>Compar.</pos> <adjf>Swifter</adjf> <pr>(sw<icr/ft"<etil/r)</pr>; <pos>superl.</pos> <adjf>Swiftest</adjf>.]</amorph> <ety>[AS. <ets>swift</ets>; akin to <ets>sw<amac/pan</ets> to sweep, <ets>swipu</ets> a whip; cf. <ets>sw<imac/fan</ets> to move quickly, to revolve. See <er>Swoop</er>, <pos>v. i.</pos>, and cf. <er>Swivel</er>, <er>Squib</er>.]</ety> <sn>1.</sn> <def>Moving a great distance in a short time; moving with celerity or velocity; fleet; rapid; quick; speedy; prompt.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>My beloved brethren, let every man be <qex>swift</qex> to hear, slow to speak, slow to wrath.</q> <rj><qau>James i. 19.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q><qex>Swift</qex> of dispatch and easy of access.</q> <rj><qau>Dryden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>And bring upon themselves <qex>swift</qex> destruction.</q> <rj><qau>2 Pet. ii. 1.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Of short continuance; passing away quickly.</def>  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ <xex>Swift</xex> is often used in the formation of compounds which are generally self-explaining; as, <xex>swift</xex>-darting, <xex>swift</xex>-footed, <xex>swift</xex>-winged, etc.</note><br/
[<source>1913 Webster</source>]</p>

<p><syn><b>Syn.</b> -- Quick; fleet; speedy; rapid; expeditious.</syn><br/
[<source>1913 Webster</source>]</p>

<p><ent>Swift</ent><br/
<hw>Swift</hw>, <pos>adv.</pos> <def>Swiftly.</def> <mark>[Obs. or Poetic]</mark>  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Ply <qex>swift</qex> and strong the oar.</q> <rj><qau>Southey.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Swift</ent><br/
<hw>Swift</hw>, <pos>n.</pos> <sn>1.</sn> <def>The current of a stream.</def> <mark>[R.]</mark>  <rj><au>Walton.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Zool.)</fld> <def>Any one of numerous species of small, long-winged, insectivorous birds of the family <fam>Micropodidae</fam>. In form and habits the swifts resemble swallows, but they are destitute of complex vocal muscles and are not singing birds, but belong to a widely different group allied to the humming birds.</def><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ The common European swift (<spn>Cypselus apus</spn> syn. <spn>Micropus apus</spn>) nests in church steeples and under the tiles of roofs, and is noted for its rapid flight and shrill screams.  It is called also <altname>black martin</altname>, <altname>black swift</altname>, <altname>hawk swallow</altname>, <altname>devil bird</altname>, <altname>swingdevil</altname>, <altname>screech martin</altname>, and <altname>shriek owl</altname>.  The common American, or chimney, swift (<spn>Chaetura pelagica</spn>) has sharp rigid tips to the tail feathers. It attaches its nest to the inner walls of chimneys, and is called also <altname>chimney swallow</altname>.  The Australian swift (<spn>Chaetura caudacuta</spn>) also has sharp naked tips to the tail quills.  The European Alpine swift (<spn>Cypselus melba</spn>) is whitish beneath, with a white band across the breast. The common Indian swift is <spn>Cypselus affinis</spn>. See also <cref>Palm swift</cref>, under <er>Palm</er>, and <cref>Tree swift</cref>, under <er>Tree</er>.</note><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <fld>(Zool.)</fld> <def>Any one of several species of lizards, as the pine lizard.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <fld>(Zool.)</fld> <def>The ghost moth. See under <er>Ghost</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <ety>[Cf. <er>Swivel</er>.]</ety> <def>A reel, or turning instrument, for winding yarn, thread, etc.; -- used chiefly in the plural.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>6.</sn> <def>The main card cylinder of a flax-carding machine.</def><br/
[<source>1913 Webster</source>]</p>

<p><-- p. 1459 --></p>

<p><ent>Swifter</ent><br/
<hw>Swift"er</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Cf. <er>Swivel</er>.]</ety> <fld>(Naut.)</fld> <sd>(a)</sd> <def>A rope used to retain the bars of the capstan in their sockets while men are turning it.</def> <sd>(b)</sd> <def>A rope used to encircle a boat longitudinally, to strengthen and defend her sides.</def> <sd>(c)</sd> <def>The forward shroud of a lower mast.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Swifter</ent><br/
<hw>Swift"er</hw>, <pos>v. t.</pos> <fld>(Naut.)</fld> <def>To tighten, as slack standing rigging, by bringing the opposite shrouds nearer.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Swiftfoot</ent><br/
<hw>Swift"foot`</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Nimble; fleet.</def>  <rj><au>Mir. for Mag.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Swiftfoot</ent><br/
<hw>Swift"foot`</hw>, <pos>n.</pos> <fld>(Zool.)</fld> <def>The courser.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Swiftlet</ent><br/
<hw>Swift"let</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Zool.)</fld> <def>Any one of numerous species of small East Indian and Asiatic swifts of the genus <gen>Collocalia</gen>. Some of the species are noted for furnishing the edible bird's nest. See <xex>Illust.</xex> under <er>Edible</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Swiftly</ent><br/
<hw>Swift"ly</hw>, <pos>adv.</pos> <def>In a swift manner; with quick motion or velocity; fleetly.</def>  <rj><au>Wyclif.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Swiftness</ent><br/
<hw>Swift"ness</hw>, <pos>n.</pos> <def>The quality or state of being swift; speed; quickness; celerity; velocity; rapidity; <as>as, the <ex>swiftness</ex> of a bird; the <ex>swiftness</ex> of a stream; <ex>swiftness</ex> of descent in a falling body; <ex>swiftness</ex> of thought, etc.</as></def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Swig</ent><br/
<hw>Swig</hw> <pr>(?)</pr>, <pos>v. t.</pos> <ety>[Cf. D. <ets>zwelgen</ets> to swallow, E. <ets>swallow</ets>, v.t.]</ety> <sn>1.</sn> <def>To drink in long draughts; to gulp; <as>as, to <ex>swig</ex> cider</as>.</def> <mark>[Colloq.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To suck.</def> <mark>[Obs. or Archaic]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>The lambkins <qex>swig</qex> the teat.</q> <rj><qau>Creech.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Swig</ent><br/
<hw>Swig</hw>, <pos>n.</pos> <sn>1.</sn> <def>A long draught.</def> <mark>[Colloq.]</mark>  <rj><au>Marryat.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Naut.)</fld> <def>A tackle with ropes which are not parallel.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>A beverage consisting of warm beer flavored with spices, lemon, etc.</def> <mark>[Prov. Eng.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Swig</ent><br/
<hw>Swig</hw>, <pos>v. t.</pos> <ety>[Cf. Prov. E. <ets>swig</ets> to leak out, AS. <ets>sw<imac/jian</ets> to be silent, <ets>sw<imac/can</ets> to evade, escape.]</ety> <sn>1.</sn> <def>To castrate, as a ram, by binding the testicles tightly with a string, so that they mortify and slough off.</def> <mark>[Prov. Eng.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Naut.)</fld> <def>To pull upon (a tackle) by throwing the weight of the body upon the fall between the block and a cleat.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Swill</ent><br/
<hw>Swill</hw> <pr>(?)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Swilled</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Swilling</conjf>.]</vmorph> <ety>[OE. <ets>swilen</ets> to wash, AS. <ets>swilian</ets>.]</ety> <sn>1.</sn> <def>To wash; to drench.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>As fearfully as doth a galled rock<br/
O'erhang and jutty his confounded base,<br/
<qex>Swilled</qex> with the wild and wasteful ocean.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <ety>[Properly, to drink like a pig. See <er>Swill</er>, <pos>n.</pos>]</ety> <def>To drink in great draughts; to swallow greedily.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Well-dressed people, of both sexes, . . . devouring sliced beef, and <qex>swilling</qex> pork, and punch, and cider.</q> <rj><qau>Smollett.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>To inebriate; to fill with drink.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>I should be loth<br/
To meet the rudeness and <qex>swilled</qex> insolence<br/
Of such late wassailers.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Swill</ent><br/
<hw>Swill</hw>, <pos>v. i.</pos> <def>To drink greedily or swinishly; to drink to excess.</def>  <rj><au>South.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Swill</ent><br/
<hw>Swill</hw>, <pos>n.</pos> <sn>1.</sn> <def>The wash, or mixture of liquid substances, given to swine; hogwash; -- called also <altname>swillings</altname>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Large draughts of liquor; drink taken in excessive quantities.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Swiller</ent><br/
<hw>Swill"er</hw> <pr>(?)</pr>, <pos>n.</pos> <def>One who swills.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Swillings</ent><br/
<hw>Swill"ings</hw> <pr>(?)</pr>, <pos>n. pl.</pos> <def>See <er>Swill</er>, <pos>n.</pos>, 1.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Swim</ent><br/
<hw>Swim</hw> <pr>(?)</pr>, <pos>v. i.</pos> <vmorph>[<pos>imp.</pos> <conjf>Swam</conjf> <pr>(?)</pr> or <conjf>Swum</conjf> <pr>(<?/)</pr>; <pos>p. p.</pos> <conjf>Swum</conjf>; <pos>p. pr. & vb. n.</pos> <conjf>Swimming</conjf>.]</vmorph> <ety>[AS. <ets>swimman</ets>; akin to D. <ets>zwemmen</ets>, OHG. <ets>swimman</ets>, G. <ets>schwimmen</ets>, Icel. <ets>svimma</ets>, Dan. <ets>sw<oum/mme</ets>, Sw. <ets>simma</ets>.  Cf. <er>Sound</er> an air bladder, a strait.]</ety> <sn>1.</sn> <def>To be supported by water or other fluid; not to sink; to float; <as>as, any substance will <ex>swim</ex>, whose specific gravity is less than that of the fluid in which it is immersed</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To move progressively in water by means of strokes with the hands and feet, or the fins or the tail.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Leap in with me into this angry flood,<br/
And <qex>swim</qex> to yonder point.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>To be overflowed or drenched.</def>  <rj><au>Ps. vi. 6.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Sudden the ditches swell, the meadows <qex>swim</qex>.</q> <rj><qau>Thomson.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>Fig.: To be as if borne or floating in a fluid.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>[They] now <qex>swim</qex> in joy.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <def>To be filled with swimming animals.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>[Streams] that <qex>swim</qex> full of small fishes.</q> <rj><qau>Chaucer.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Swim</ent><br/
<hw>Swim</hw>, <pos>v. t.</pos> <sn>1.</sn> <def>To pass or move over or on by swimming; <as>as, to <ex>swim</ex> a stream</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Sometimes he thought to <qex>swim</qex> the stormy main.</q> <rj><qau>Dryden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To cause or compel to swim; to make to float; <as>as, to <ex>swim</ex> a horse across a river</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>To immerse in water that the lighter parts may float; <as>as, to <ex>swim</ex> wheat in order to select seed</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Swim</ent><br/
<hw>Swim</hw>, <pos>n.</pos> <sn>1.</sn> <def>The act of swimming; a gliding motion, like that of one swimming.</def>  <rj><au>B. Jonson.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>The sound, or air bladder, of a fish.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>A part of a stream much frequented by fish.</def> <mark>[Eng.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Swim bladder</b></col>, <cd>an air bladder of a fish.</cd> -- <col><b>To be in the swim</b></col>, <cd>to be in a favored position; to be associated with others in active affairs.</cd> <mark>[Colloq.]</mark></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Swim</ent><br/
<hw>Swim</hw>, <pos>v. i.</pos> <ety>[OE. <ets>swime</ets> dizziness, vertigo, AS. <ets>sw<imac/ma</ets>; akin to D. <ets>zwijm</ets>, Icel. <ets>svimi</ets> dizziness, <ets>svina</ets> to subside, <ets>sv<imac/a</ets> to abate, G. <ets>schwindel</ets> dizziness, <ets>schwinden</ets> to disappear, to dwindle, OHG. <ets>sw<imac/nan</ets> to dwindle.  Cf. <er>Squemish</er>, <er>Swindler</er>.]</ety> <def>To be dizzy; to have an unsteady or reeling sensation; <as>as, the head <ex>swims</ex></as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Swimbel</ent><br/
<hw>Swim"bel</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Etymol. uncertain.]</ety> <def>A moaning or sighing sound or noise; a sough.</def> <mark>[Obs.]</mark>  <rj><au>Chaucer.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Swimmer</ent><br/
<hw>Swim"mer</hw> <pr>(?)</pr>, <pos>n.</pos> <sn>1.</sn> <def>One who swims.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Far.)</fld> <def>A protuberance on the leg of a horse.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <fld>(Zool.)</fld> <def>A swimming bird; one of the natatores.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Little swimmer</b></col> <fld>(Zool.)</fld>, <cd>a phalarope.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Swimmeret</ent><br/
<hw>Swim"mer*et</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Zool.)</fld> <def>One of a series of flat, fringed, and usually bilobed, appendages, of which several pairs occur on the abdominal somites of many crustaceans. They are used as fins in swimming.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Swimming</ent><br/
<hw>Swim"ming</hw> <pr>(?)</pr>, <pos>a.</pos> <sn>1.</sn> <def>That swims; capable of swimming; adapted to, or used in, swimming; <as>as, a <ex>swimming</ex> bird; a <ex>swimming</ex> motion</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Suffused with moisture; <as>as, <ex>swimming</ex> eyes</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Swimming bell</b></col> <fld>(Zool.)</fld>, <cd>a nectocalyx. See <xex>Illust.</xex> under <er>Siphonophora</er>.</cd> -- <col><b>Swimming crab</b></col> <fld>(Zool.)</fld>, <cd>any one of numerous species of marine crabs, as those of the family <fam>Protunidae</fam>, which have some of the joints of one or more pairs of legs flattened so as to serve as fins.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Swimming</ent><br/
<hw>Swim"ming</hw>, <pos>n.</pos> <def>The act of one who swims.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Swimming</ent><br/
<hw>Swim"ming</hw>, <pos>a.</pos> <ety>[From <er>Swim</er> to be dizzy.]</ety> <def>Being in a state of vertigo or dizziness; <as>as, a <ex>swimming</ex> brain</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Swimming</ent><br/
<hw>Swim"ming</hw>, <pos>n.</pos> <def>Vertigo; dizziness; <as>as, a <ex>swimming</ex> in the head</as>.</def>  <rj><au>Dryden.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Swimmingly</ent><br/
<hw>Swim"ming*ly</hw>, <pos>adv.</pos> <def>In an easy, gliding manner, as if swimming; smoothly; successfully; prosperously.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Swimmingness</ent><br/
<hw>Swim"ming*ness</hw>, <pos>n.</pos> <def>Act or state of swimming; suffusion.</def> <ldquo/A <xex>swimmingness</xex> in the eye.<rdquo/  <rj><au>Congreve.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Swinck</ent><br/
<hw>Swinck</hw> <pr>(?)</pr>, <pos>v. & n.</pos> <def>See <er>Swink</er>.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Swindle</ent><br/
<hw>Swin"dle</hw> <pr>(?)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Swindled</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Swindling</conjf> <pr>(?)</pr>.]</vmorph> <ety>[See <er>Swindler</er>.]</ety> <def>To cheat defraud grossly, or with deliberate artifice; <as>as, to <ex>swindle</ex> a man out of his property</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Lammote . . . has <qex>swindled</qex> one of them out of three hundred livres.</q> <rj><qau>Carlyle.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Swindle</ent><br/
<hw>Swin"dle</hw>, <pos>n.</pos> <def>The act or process of swindling; a cheat.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Swindler</ent><br/
<hw>Swin"dler</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[G. <ets>schwindler</ets>, fr. <ets>schwindlen</ets> to be dizzy, to act thoughtlessly, to cheat, fr. <ets>schwindel</ets> dizziness, fr. <ets>schwinden</ets> to vanish, to disappear, to dwindle. See <er>Swim</er> to be dizzy.]</ety> <def>One who swindles, or defrauds grossly; one who makes a practice of defrauding others by imposition or deliberate artifice; a cheat.</def><br/
[<source>1913 Webster</source>]</p>

<p><syn><b>Syn.</b> -- Sharper; rogue.</syn>  -- <usage><er>Swindler</er>, <er>Sharper</er>. These words agree in describing persons who take unfair advantages. A <xex>swindler</xex> is one who obtains money or goods under false pretenses. A <xex>sharper</xex> is one who cheats by sharp practice, as in playing at cards or staking what he can not pay.</usage><br/
[<source>1913 Webster</source>]</p>

<p><q>Fraud and injustice soon follow, and the dignity of the British merchant is sunk in the scandalous appellation of a <qex>swindler</qex>.</q> <rj><qau>V. Knox.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Perhaps you 'll think I act the same<br/
As a sly <qex>sharper</qex> plays his game.</q> <rj><qau>Cotton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Swindlery</ent><br/
<hw>Swin"dler*y</hw> <pr>(?)</pr>, <pos>n.</pos> <def>Swindling; rougery.</def> <mark>[R.]</mark> <ldquo/<xex>Swindlery</xex> and blackguardism.<rdquo/  <rj><au>Carlyle.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Swine</ent><br/
<hw>Swine</hw> <pr>(?)</pr>, <pos>n. sing. & pl.</pos> <ety>[OE. <ets>swin</ets>, AS. <ets>sw<imac/n</ets>; akin to OFries. & OS. <ets>swin</ets>, D. <ets>zwijn</ets>, G. <ets>schwein</ets>, OHG. <ets>sw<imac/n</ets>, Icel. <ets>sv<imac/n</ets>, Sw. <ets>svin</ets>, Dan. <ets>sviin</ets>, Goth. <ets>swein</ets>; originally a diminutive corresponding to E. <ets>sow</ets>. See <er>Sow</er>, <pos>n.</pos>]</ety> <fld>(Zool.)</fld> <def>Any animal of the hog kind, especially one of the domestical species. Swine secrete a large amount of subcutaneous fat, which, when extracted, is known as <xex>lard</xex>. The male is specifically called <xex>boar</xex>, the female, <xex>sow</xex>, and the young, <xex>pig</xex>. See <er>Hog</er>.</def> <ldquo/A great herd of <xex>swine</xex>.<rdquo/  <rj><au>Mark v. 11.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Swine grass</b></col> <fld>(Bot.)</fld>, <cd>knotgrass (<spn>Polygonum aviculare</spn>); -- so called because eaten by swine.</cd> -- <col><b>Swine oat</b></col> <fld>(Bot.)</fld>, <cd>a kind of oat sometimes grown for swine.</cd> -- <col><b>Swine's cress</b></col> <fld>(Bot.)</fld>, <cd>a species of cress of the genus <gen>Senebiera</gen> (<spn>Senebiera Coronopus</spn>).</cd> -- <col><b>Swine's head</b></col>, <cd>a dolt; a blockhead.</cd> <mark>[Obs.]</mark> <au>Chaucer.</au> -- <col><b>Swine thistle</b></col> <fld>(Bot.)</fld>, <cd>the sow thistle.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Swinebread</ent><br/
<hw>Swine"bread`</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Bot.)</fld> <def>The truffle.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Swinecase</ent><br/
<hw>Swine"case`</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A hogsty.</def> <mark>[Prov. Eng.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Swinecote</ent><br/
<hw>Swine"cote`</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A hogsty.</def> <mark>[Prov. Eng.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Swinecrue</ent><br/
<hw>Swine"crue`</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[<ets>Swine</ets> + Prov. E. <ets>crue</ets> a coop.]</ety> <def>A hogsty.</def> <mark>[Prov. Eng.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Swinefish</ent><br/
<hw>Swine"fish`</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Zool.)</fld> <def>The wolf fish.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Swineherd</ent><br/
<hw>Swine"herd`</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A keeper of swine.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Swinepipe</ent><br/
<hw>Swine"pipe`</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Zool.)</fld> <def>The European redwing.</def> <mark>[Prov. Eng.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Swine-pox</ent><br/
<hw>Swine"-pox`</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Med.)</fld> <def>A variety of the chicken pox, with acuminated vesicles containing a watery fluid; the water pox.</def>  <rj><au>Pepys.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Swinery</ent><br/
<hw>Swin"er*y</hw> <pr>(sw<imac/n"<etil/r*<ycr/)</pr>, <pos>n.</pos> <def>Same as <er>Piggery</er>.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Swinestone</ent><br/
<hw>Swine"stone`</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Min.)</fld> <def>See <er>Stinkstone</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Swinesty</ent><br/
<hw>Swine"sty`</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A sty, or pen, for swine.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Swing</ent><br/
<hw>Swing</hw> <pr>(?)</pr>, <pos>v. i.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Swung</conjf> <pr>(?)</pr>; <pos>Archaic imp.</pos> <conjf>Swang</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Swinging</conjf>.]</vmorph> <ety>[OE. <ets>swingen</ets>, AS. <ets>swingan</ets> to scourge, to fly, to flutter; akin to G. <ets>schwingen</ets> to winnow, to swingle, oscillate, <ets>sich schwingen</ets> to leap, to soar, OHG. <ets>swingan</ets> to throw, to scourge, to soar, Sw. <ets>svinga</ets> to swing, to whirl, Dan. <ets>svinge</ets>.  Cf. <er>Swagger</er>, <er>Sway</er>, <er>Swinge</er>, <er>Swink</er>.]</ety> <sn>1.</sn> <def>To move to and fro, as a body suspended in the air; to wave; to vibrate; to oscillate.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>I tried if a pendulum would <qex>swing</qex> faster, or continue <qex>swinging</qex> longer, in case of exsuction of the air.</q> <rj><qau>Boyle.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To sway or move from one side or direction to another; <as>as, the door <ex>swung</ex> open</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>To use a swing; <as>as, a boy <ex>swings</ex> for exercise or pleasure</as>. See <er>Swing</er>, <pos>n.</pos>, 3.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <fld>(Naut.)</fld> <def>To turn round by action of wind or tide when at anchor; <as>as, a ship <ex>swings</ex> with the tide</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <def>To be hanged.</def> <mark>[Colloq.]</mark>  <rj><au>D. Webster.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>To swing round the circle</b></col>, <cd>to make a complete circuit.</cd> <mark>[Colloq.]</mark></cs><br/
[<source>1913 Webster</source>]</p>

<p><q>He had <qex>swung round the circle</qex> of theories and systems in which his age abounded, without finding relief.</q> <rj><qau>A. V. G. Allen.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Swing</ent><br/
<hw>Swing</hw>, <pos>v. t.</pos> <sn>1.</sn> <def>To cause to swing or vibrate; to cause to move backward and forward, or from one side to the other.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>He <qex>swings</qex> his tail, and swiftly turns his round.</q> <rj><qau>Dryden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>They get on ropes, as you must have seen the children, and are <qex>swung</qex> by their men visitants.</q> <rj><qau>Spectator.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To give a circular movement to; to whirl; to brandish; <as>as, to <ex>swing</ex> a sword; to <ex>swing</ex> a club</as>; hence, colloquially, to manage; <as>as, to <ex>swing</ex> a business</as>.</def><-- or, to accomplish --><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <fld>(Mach.)</fld> <def>To admit or turn (anything) for the purpose of shaping it; -- said of a lathe; <as>as, the lathe can <ex>swing</ex> a pulley of 12 inches diameter</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><mcol><col><b>To swing a door</b></col>, <col><b>gate</b></col>, etc.</mcol> <fld>(Carp.)</fld>, <cd>to put it on hinges so that it can swing or turn.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Swing</ent><br/
<hw>Swing</hw> <pr>(?)</pr>, <pos>n.</pos> <sn>1.</sn> <def>The act of swinging; a waving, oscillating, or vibratory motion of a hanging or pivoted object; oscillation; <as>as, the <ex>swing</ex> of a pendulum</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Swaying motion from one side or direction to the other; <as>as, some men walk with a <ex>swing</ex></as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>A line, cord, or other thing suspended and hanging loose, upon which anything may swing; especially, an apparatus for recreation by swinging, commonly consisting of a rope, the two ends of which are attached overhead, as to the bough of a tree, a seat being placed in the loop at the bottom; also, any contrivance by which a similar motion is produced for amusement or exercise.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>Influence of power of a body put in swaying motion.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The ram that batters down the wall,<br/
For the great <qex>swing</qex> and rudeness of his poise,<br/
They place before his hand that made the engine.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <def>Capacity of a turning lathe, as determined by the diameter of the largest object that can be turned in it.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>6.</sn> <def>Free course; unrestrained liberty or license; tendency.</def> <ldquo/Take thy <xex>swing</xex>.<rdquo/  <rj><au>Dryden.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>To prevent anything which may prove an obstacle to the full <qex>swing</qex> of his genius.</q> <rj><qau>Burke.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Full swing</b></col>. <cd>See under <er>Full</er>.</cd> -- <col><b>Swing beam</b></col> <fld>(Railway Mach.)</fld>, <cd>a crosspiece sustaining the car body, and so suspended from the framing of a truck that it may have an independent lateral motion.</cd> -- <col><b>Swing bridge</b></col>, <cd>a form of drawbridge which swings horizontally, as on a vertical pivot.</cd> -- <mcol><col><b>Swing plow</b></col>, <it>or</it> <col><b>Swing plough</b></col></mcol>. <sd>(a)</sd> <cd>A plow without a fore wheel under the beam.</cd> <sd>(b)</sd> <cd>A reversible or sidehill plow.</cd> -- <col><b>Swing wheel</b></col>. <sd>(a)</sd> <cd>The scape-wheel in a clock, which drives the pendulum.</cd> <sd>(b)</sd> <cd>The balance of a watch.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Swingdevil</ent><br/
<hw>Swing"dev`il</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Zool.)</fld> <ety>[So named from its swift flight and dark color, which give it an uncanny appearance.]</ety> <def>The European swift.</def> <mark>[Prov. Eng.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Swinge</ent><br/
<hw>Swinge</hw> <pr>(sw<icr/nj)</pr>, <pos>v. & n.</pos> <def>See <er>Singe</er>.</def> <mark>[Obs.]</mark>  <rj><au>Spenser.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Swinge</ent><br/
<hw>Swinge</hw>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Swinged</conjf> <pr>(sw<icr/njd)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Swingeing</conjf> <pr>(sw<icr/nj"<icr/ng)</pr>.]</vmorph> <ety>[OE. <ets>swengen</ets>, AS. <ets>swengan</ets> to shake, causative of <ets>swingan</ets>. See <er>Swing</er>.]</ety> <sn>1.</sn> <def>To beat soundly; to whip; to chastise; to punish.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>I had <qex>swinged</qex> him soundly.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>And <qex>swinges</qex> his own vices in his son.</q> <rj><qau>C. Dryden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To move as a lash; to lash.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q><qex>Swinges</qex> the scaly horror of his folded tail.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Swinge</ent><br/
<hw>Swinge</hw>, <pos>n.</pos> <sn>1.</sn> <def>The sweep of anything in motion; a swinging blow; a swing.</def> <mark>[Obs.]</mark>  <rj><au>Waller.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Power; sway; influence.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Swingebuckler</ent><br/
<hw>Swinge"buc`kler</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A swashbuckler; a bully; a roisterer.</def> <mark>[Obs.]</mark>  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Swingeing</ent><br/
<hw>Swinge"ing</hw>, <pos>a.</pos> <def>Huge; very large.</def> <mark>[Colloq.]</mark> <au>Arbuthnot.</au> <au>Byron.</au> -- <wordforms><wf>Swinge"ing*ly</wf>, <pos>adv.</pos></wordforms>  <rj><au>Dryden.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Swingel</ent><br/
<hw>Swin"gel</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[AS. <ets>swingele</ets> whip, scourge. See <er>Swing</er>.]</ety> <def>The swinging part of a flail which falls on the grain in thrashing; the swiple.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Swinger</ent><br/
<hw>Swing"er</hw> <pr>(?)</pr>, <pos>n.</pos> <def>One who swings or whirls.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Swinger</ent><br/
<hw>Swin"ger</hw> <pr>(?)</pr>, <pos>n.</pos> <sn>1.</sn> <def>One who swinges.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Anything very large, forcible, or astonishing.</def> <mark>[Obs. or Colloq.]</mark>  <rj><au>Herrick.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><-- 3. [Colloq.] <def>A person who engages frequently in lively and fashionable pursuits, such as attending night clubs or discos.</def><br/
[<source>1913 Webster</source>]</p>

<p>   4. <def>A person who engages freely in sexual intercourse.</def>
--><br/
[<source>1913 Webster</source>]</p>

<p><ent>Swingle</ent><br/
<hw>Swin"gle</hw> <pr>(?)</pr>, <pos>v. i.</pos> <ety>[Freq. of <ets>swing</ets>.]</ety> <sn>1.</sn> <def>To dangle; to wave hanging.</def> <mark>[Obs.]</mark>  <rj><au>Johnson.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To swing for pleasure.</def> <mark>[Obs. or Prov. Eng.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Swingle</ent><br/
<hw>Swin"gle</hw>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Swingled</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Swingling</conjf> <pr>(?)</pr>.]</vmorph> <ety>[See <er>Swingel</er>.]</ety> <sn>1.</sn> <def>To clean, as flax, by beating it with a swingle, so as to separate the coarse parts and the woody substance from it; to scutch.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To beat off the tops of without pulling up the roots; -- said of weeds.</def> <mark>[Prov. Eng.]</mark>  <rj><au>Forby.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Swingle</ent><br/
<hw>Swin"gle</hw>, <pos>n.</pos> <def>A wooden instrument like a large knife, about two feet long, with one thin edge, used for beating and cleaning flax; a scutcher; -- called also <altname>swingling knife</altname>, <altname>swingling staff</altname>, and <altname>swingling wand</altname>.</def><br/
[<source>1913 Webster</source>]</p>

<-- 2.  contraction for <it>swinging single</it>, i .e. an unmarried swinger[sense 3 or 4] [Colloq.] -->

<p><ent>Swinglebar</ent><br/
<hw>Swin"gle*bar`</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A swingletree.</def>  <rj><au>De Quincey.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Swingletail</ent><br/
<hw>Swin"gle*tail`</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Zool.)</fld> <def>The thrasher, or fox shark. See <er>Thrasher</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Swingletree</ent><br/
<hw>Swin"gle*tree`</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[So named in allusion to its <ets>swinging</ets>. See <er>Swingle</er>, <pos>v. i.</pos>, and cf. <er>Swingtree</er>.]</ety> <def>A whiffletree, or whippletree. See <er>Singletree</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Swingling</ent><br/
<hw>Swin"gling</hw> <pr>(?)</pr>, <def><pos>a. & n.</pos> from <er>Swingle</er>, <pos>v. t.</pos></def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Swingling tow</b></col>, <cd>the coarse part of flax, separated from the finer by swingling and hatcheling.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Swingtree</ent><br/
<hw>Swing"tree`</hw> <pr>(?)</pr>, <pos>n.</pos> <def>The bar of a carriage to which the traces are fastened; the whiffletree.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Swinish</ent><br/
<hw>Swin"ish</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Of or pertaining to swine; befitting swine; like swine; hoggish; gross; beasty; <as>as, a <ex>swinish</ex> drunkard or sot</as>.</def> <ldquo/<xex>Swinish</xex> gluttony.<rdquo/  <rj><au>Milton.</au></rj> -- <wordforms><wf>Swin"ish*ly</wf>, <pos>adv.</pos> -- <wf>Swin"ish*ness</wf>, <pos>n.</pos></wordforms><br/
[<source>1913 Webster</source>]</p>

<p><ent>Swink</ent><br/
<hw>Swink</hw> <pr>(?)</pr>, <pos>v. i.</pos> <vmorph>[<pos>imp.</pos> <conjf>Swank</conjf> <pr>(?)</pr>, <conjf>Swonk</conjf> <pr>(<?/)</pr>; <pos>p. p.</pos> <conjf>Swonken</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Swinking</conjf>.]</vmorph> <ety>[AS. <ets>swincan</ets>, akin to <ets>swingan</ets>. See <er>Swing</er>.]</ety> <def>To labor; to toil; to salve.</def> <mark>[Obs. or Archaic]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>Or <qex>swink</qex> with his hands and labor.</q> <rj><qau>Chaucer.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>For which men <qex>swink</qex> and sweat incessantly.</q> <rj><qau>Spenser.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>The <qex>swinking</qex> crowd at every stroke pant <ldquo/Ho.<rdquo/</q> <rj><qau>Sir Samuel Freguson.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Swink</ent><br/
<hw>Swink</hw>, <pos>v. t.</pos> <sn>1.</sn> <def>To cause to toil or drudge; to tire or exhaust with labor.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>And the <qex>swinked</qex> hedger at his supper sat.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To acquire by labor.</def> <mark>[Obs.]</mark>  <rj><au>Piers Plowman.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>To devour all that others <qex>swink</qex>.</q> <rj><qau>Chaucer.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Swink</ent><br/
<hw>Swink</hw>, <pos>n.</pos> <ety>[As. <ets>swinc</ets>, ge<ets>swinc</ets>.]</ety> <def>Labor; toil; drudgery.</def> <mark>[Obs.]</mark>  <rj><au>Chaucer. Spenser.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Swinker</ent><br/
<hw>Swink"er</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A laborer.</def> <mark>[Obs.]</mark>  <rj><au>Chaucer.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Swinney</ent><br/
<hw>Swin"ney</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Far.)</fld> <def>See <er>Sweeny</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Swipe</ent><br/
<hw>Swipe</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Cf. <er>Sweep</er>, <er>Swiple</er>.]</ety> <sn>1.</sn> <def>A swape or sweep. See <er>Sweep</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A strong blow given with a sweeping motion, as with a bat or club.</def><br/
[<source>1913 Webster</source>]</p>

<p><q><qex>Swipes</qex> [in cricket] over the blower's head, and over either of the long fields.</q> <rj><qau>R. A. Proctor.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <pluf>pl.</pluf> <def>Poor, weak beer; small beer.</def> <mark>[Slang, Eng.]</mark> <altsp>[Written also <asp>swypes</asp>.]</altsp>  <rj><au>Craig.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><-- p. 1460 --></p>

<p><ent>Swipe</ent><br/
<hw>Swipe</hw> <pr>(?)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Swiped</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Swiping</conjf>.]</vmorph> <sn>1.</sn> <def>To give a swipe to; to strike forcibly with a sweeping motion, as a ball.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Loose balls may be <qex>swiped</qex> almost <it>ad libitum</it>.</q> <rj><qau>R. A. Proctor.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To pluck; to snatch; to steal.</def> <mark>[Slang, U.S.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Swiple</ent><br/
<hw>Swi"ple</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[See <er>Swipe</er>.]</ety> <def>That part of a flail which strikes the grain in thrashing; a swingel.</def> <altsp>[Written also <asp>swipel</asp>, and <asp>swipple</asp>.]</altsp><br/
[<source>1913 Webster</source>]</p>

<p><ent>Swipper</ent><br/
<hw>Swip"per</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[From AS. <ets>swipian</ets> to whip, shake, whirl; akin to <ets>sw<amac/pan</ets> to sweep. See <er>Swoop</er>.]</ety> <def>Nimble; quick.</def> <mark>[Obs. or Prov. Eng. & Slang]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Swirl</ent><br/
<hw>Swirl</hw> <pr>(?)</pr>, <pos>v. t. & i.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Swirled</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Swirling</conjf>.]</vmorph> <ety>[Akin to Norw. <ets>svirla</ets> to whirl, freq. of <ets>sverra</ets> to whirl, Dan. <ets>svirre</ets>, G. <ets>schwirren</ets> to whiz, to buzz. <root/177. See <er>Swarm</er>, <pos>n.</pos>]</ety> <def>To whirl, or cause to whirl, as in an eddy.</def> <ldquo/The river <xex>swirled</xex> along.<rdquo/  <rj><au>C. Kingsley.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Swirl</ent><br/
<hw>Swirl</hw>, <pos>n.</pos> <def>A whirling motion; an eddy, as of water; a whirl.</def> <ldquo/The silent <xex>swirl</xex> of bats.<rdquo/  <rj><au>Mrs. Browning.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Swish</ent><br/
<hw>Swish</hw> <pr>(?)</pr>, <pos>v. t.</pos> <ety>[From the sound.  Cf. <er>Swash</er>.]</ety> <sn>1.</sn> <def>To flourish, so as to make the sound <xex>swish</xex>.</def>  <rj><au>Coleridge.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To flog; to lash.</def> <mark>[Slang]</mark>  <rj><au>Thackeray.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Swish</ent><br/
<hw>Swish</hw>, <pos>v. i.</pos> <def>To dash; to swash.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Swish</ent><br/
<hw>Swish</hw>, <pos>n.</pos> <sn>1.</sn> <def>A sound of quick movement, as of something whirled through the air.</def> <mark>[Colloq.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Naut.)</fld> <def>Light driven spray.</def> <mark>[Eng.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Swiss</ent><br/
<hw>Swiss</hw> <pr>(?)</pr>, <pos>n. sing. & pl.</pos> <ety>[F. <ets>Suisse</ets>, of German origin.  Cf. <er>Switzer</er>.]</ety> <def>A native or inhabitant of Switzerland; a Switzer; the people of Switzerland.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Swiss</ent><br/
<hw>Swiss</hw>, <pos>a.</pos> <def>Of or pertaining to Switzerland, or the people of Switzerland.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Switch</ent><br/
<hw>Switch</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Cf. OD. <ets>swick</ets> a scourage, a whip.  Cf. <er>Swink</er>, <er>Swing</er>.]</ety> <sn>1.</sn> <def>A small, flexible twig or rod.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Mauritania, on the fifth medal, leads a horse with something like a thread; in her other hand she holds a <qex>switch</qex>.</q> <rj><qau>Addison.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Railways)</fld> <def>A movable part of a rail; or of opposite rails, for transferring cars from one track to another.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>A separate mass or trees of hair, or of some substance (at jute) made to resemble hair, worn on the head by women.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <fld>(Elec.)</fld> <def>A device for shifting an electric current to another circuit, or for making and breaking a circuit.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><cs><col><b>Safety switch</b></col> <fld>(Railways)</fld>, <cd>a form of switch contrived to prevent or lessen the danger of derailment of trains.</cd> -- <col><b>Switch back</b></col> <fld>(Railways)</fld>, <cd>an arrangement of tracks whereby elevations otherwise insurmountable are passed. The track ascends by a series of zigzags, the engine running alternately forward and back, until the summit is reached.</cd> -- <col><b>Switch board</b></col> <fld>(Elec.)</fld>, <cd>a collection of switches in one piece of apparatus, so arranged that a number of circuits may be connected or combined in any desired manner.</cd><-- need "switchboard" for telephony --> -- <col><b>Switch grass</b></col>. <fld>(Bot.)</fld> <cd>See under <er>Grass</er>.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Switch</ent><br/
<hw>Switch</hw>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Switched</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Switching</conjf>.]</vmorph> <sn>1.</sn> <def>To strike with a switch or small flexible rod; to whip.</def>  <rj><au>Chapman.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To swing or whisk; <as>as, to <ex>switch</ex> a cane</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>To trim, <as>as, a hedge</as>.</def> <mark>[Prov. Eng.]</mark>  <rj><au>Halliwell.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>To turn from one railway track to another; to transfer by a switch; -- generally with <xex>off</xex>, <xex>from</xex>, etc.; <as>as, to <ex>switch</ex> off a train; to <ex>switch</ex> a car from one track to another</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <fld>(Eccl.)</fld> <def>To shift to another circuit.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Switch</ent><br/
<hw>Switch</hw>, <pos>v. i.</pos> <def>To walk with a jerk.</def> <mark>[Prov. Eng.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>switchboard</ent><br/
<hw>switch"board`</hw> <pos>n.</pos> <sn>1.</sn> <def>an elctrical apparatus consisting predominantly of a panel on which are switches or other means of completing electrical circuits; -- used especially for the devices used in telephone exchanges.  See sense 2.</def><br/
[<source>PJC</source>]</p>

<p><sn>2.</sn> <fld>(Telephone)</fld>  <def>An apparatus containing switches by means of which a connection may be made from an incoming telephone line to any one of numerous outgoing lines.  The switches may be either mechanical or electronic, and the switching action may be automatic, controlled by signals in the incoming call, or manual, controlled by a switchboard operator.  In older style manual switchboards, the circuits were connected by the use of patchcords, inserted into plugs.</def><br/
<syn><b>Syn. --</b> patchboard, plugboard, telephone exchange.</syn><br/
[<source>PJC</source>]</p>

<p><ent>Switchel</ent><br/
<hw>Switch"el</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[See <er>Sweet</er>.]</ety> <def>A beverage of molasses and water, seasoned with vinegar and ginger.</def> <mark>[U. S.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>switcheroo</ent><br/
<hw>switch"er*oo</hw> <pos>n.</pos> <def>a sudden and unexpected change or reversal of position, attitude, or action.</def><br/
[<source>PJC</source>]</p>

<p><ent>Switching</ent><br/
<hw>Switch"ing</hw>, <def><pos>a. & n.</pos> from <er>Switch</er>, <pos>v.</pos></def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Switching engine</b></col>, <cd>a locomotive for switching cars from one track to another, and making up trains; -- called also <altname>switch engine</altname>.</cd> <mark>[U.S.]</mark></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Switching power supply</ent><br/
<hw>Switch"ing power supply</hw>, <pos>n.</pos> <def>a device used as part of an electronic device, which transforms electrical current from an AC line circuit to DC for use in electronic devices, and which can use either 110 volt or 220 volt AC line curent.</def><br/
[<source>PJC</source>]</p>

<p><ent>Switchman</ent><br/
<hw>Switch"man</hw> <pr>(?)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Switchmen</plw> <pr>(<?/)</pr>.</plu> <def>One who tends a switch on a railway.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Switchy</ent><br/
<hw>Switch"y</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Whisking.</def> <mark>[Colloq.]</mark>  <rj><au>Coombe.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Swithe</ent><br/
<hw>Swithe</hw> <pr>(?)</pr>, <pos>adv.</pos> <ety>[AS. <ets>sw<imac/<?/e</ets> strongly, violently.]</ety> <def>Instantly; quickly; speedily; rapidly.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>That thou doest, do thou <qex>swithe</qex>.</q> <rj><qau>Wyclif (John xiii. 27).</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Switzer</ent><br/
<hw>Switz"er</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Cf. G. <ets>schweizer</ets>.  Cf. <er>Swiss</er>.]</ety> <def>A native or inhabitant of Switzerland; a Swiss.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Swive</ent><br/
<hw>Swive</hw> <pr>(?)</pr>, <pos>v. t.</pos> <ety>[OE. <ets>swiven</ets>, fr. AS. <ets>sw<imac/fan</ets>. See <er>Swivel</er>.]</ety> <def>To copulate with (a woman).</def> <mark>[Obs.]</mark>  <rj><au>Chaucer.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Swivel</ent><br/
<hw>Swiv"el</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[AS. <ets>sw<imac/fan</ets> to move quickly, to remove; akin to Icel. <ets>sveifla</ets> to whirl, shake, <ets>sv<imac/fa</ets> to ramble, to turn. See <er>Swoop</er>, and cf. <er>Swift</er> a reel, <er>Swift</er>, <pos>a.</pos>]</ety> <sn>1.</sn> <fld>(Mech.)</fld> <def>A piece, as a ring or hook, attached to another piece by a pin, in such a manner as to permit rotation about the pin as an axis.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Mil.)</fld> <def>A small piece of ordnance, turning on a point or swivel; -- called also <altname>swivel gun</altname>.</def>  <rj><au>Wilhelm.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Swivel bridge</b></col>, <cd>a kind of drawbridge that turns round on a vertical axis; a swing bridge.</cd> -- <col><b>Swivel hook</b></col>, <cd>a hook connected with the iron strap of a pulley block by a swivel joint, for readily taking the turns out of a tackle.</cd> -- <col><b>Swivel joint</b></col>, <cd>a joint, the two pieces composing which turn round, with respect to each other, on a longitudinal pin or axis, as in a chain, to prevent twisting.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Swivel</ent><br/
<hw>Swiv"el</hw>, <pos>v. i.</pos> <def>To swing or turn, as on a pin or pivot.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Swivel-eyed</ent><br/
<hw>Swiv"el-eyed`</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Squint-eyed.</def> <mark>[Prov. Eng.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Swizzle</ent><br/
<hw>Swiz"zle</hw> <pr>(?)</pr>, <pos>v. t.</pos> <def>To drink; to swill.</def>  <rj><au>Halliwell.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Swizzle</ent><br/
<hw>Swiz"zle</hw>, <pos>n.</pos> <def>Ale and beer mixed; also, drink generally.</def> <mark>[Prov. Eng.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Swob</ent><br/
<hw>Swob</hw> <pr>(?)</pr>, <pos>n. & v.</pos> <def>See <er>Swab</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Swobber</ent><br/
<hw>Swob"ber</hw> <pr>(?)</pr>, <pos>n.</pos> <sn>1.</sn> <def>See <er>Swabber</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <pluf>pl.</pluf> <def>Four privileged cards, formerly used in betting at the game of whist.</def> <altsp>[Written also <asp>swabber</asp>.]</altsp>  <rj><au>Swift.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Swollen</ent><br/
<hw>Swoll"en</hw> <pr>(?)</pr>, <def><pos>p. p.</pos> of <er>Swell</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Swollen</ent><br/
<hw>Swoll"en</hw>, <pos>a.</pos> <def>Enlarged by swelling; immoderately increased; <as>as, <ex>swollen</ex> eyes; <ex>swollen</ex> streams.</as></def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Swoln</ent><br/
<hw>Swoln</hw> <pr>(?)</pr>. <def>Contraction of <er>Swollen</er>, <pos>p. p.</pos></def>  <rj><au>Milton.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Swom</ent><br/
<hw>Swom</hw> <pr>(?)</pr>, <mark>obs.</mark> <def><pos>imp.</pos> of <er>Swim</er>.</def>  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Swoon</ent><br/
<hw>Swoon</hw> <pr>(?)</pr>, <pos>v. i.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Swooned</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Swooning</conjf>.]</vmorph> <ety>[OE. <ets>swounen</ets>, <ets>swoghenen</ets>, for <ets>swo<?/nien</ets>, fr. <ets>swo<?/en</ets> to sigh deeply, to droop, AS. <ets>sw<omac/gan</ets> to sough, sigh; cf. ge<ets>sw<omac/gen</ets> senseless, swooned, ge<ets>sw<omac/wung</ets> a swooning.  Cf. <er>Sough</er>.]</ety> <def>To sink into a fainting fit, in which there is an apparent suspension of the vital functions and mental powers; to faint; -- often with <xex>away</xex>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The sucklings <qex>swoon</qex> in the streets of the city.</q> <rj><qau>Lam. ii. 11.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>The most in years . . . <qex>swooned</qex> first away for pain.</q> <rj><qau>Dryden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>He seemed ready to <qex>swoon</qex> away in the surprise of joy.</q> <rj><qau>Tatler.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Swoon</ent><br/
<hw>Swoon</hw>, <pos>n.</pos> <def>A fainting fit; syncope.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Swooning</ent><br/
<hw>Swoon"ing</hw>, <def><pos>a. & n.</pos> from <er>Swoon</er>, <pos>v.</pos></def> -- <wordforms><wf>Swoon"ing*ly</wf>, <pos>adv.</pos></wordforms><br/
[<source>1913 Webster</source>]</p>

<p><ent>Swoop</ent><br/
<hw>Swoop</hw> <pr>(?)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Swooped</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Swooping</conjf>.]</vmorph> <ety>[OE. <ets>swopen</ets>, usually, to sweep, As. <ets>sw<amac/pan</ets> to sweep, to rush; akin to G. <ets>schweifen</ets> to rove, to ramble, to curve, OHG. <ets>sweifan</ets> to whirl, Icel. <ets>sveipa</ets> to sweep; also to AS. <ets>sw<imac/fan</ets> to move quickly.  Cf. <er>Sweep</er>, <er>Swift</er>, <pos>a.</pos> & <pos>n.</pos>, <er>Swipe</er>, <er>Swivel</er>.]</ety> <sn>1.</sn> <def>To fall on at once and seize; to catch while on the wing; <as>as, a hawk <ex>swoops</ex> a chicken</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To seize; to catch up; to take with a sweep.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>And now at last you came to <qex>swoop</qex> it all.</q> <rj><qau>Dryden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>The grazing ox which <qex>swoops</qex> it [the medicinal herb] in with the common grass.</q> <rj><qau>Glanvill.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Swoop</ent><br/
<hw>Swoop</hw>, <pos>v. i.</pos> <sn>1.</sn> <def>To descend with closed wings from a height upon prey, as a hawk; to stoop.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To pass with pomp; to sweep.</def> <mark>[Obs.]</mark>  <rj><au>Drayton.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Swoop</ent><br/
<hw>Swoop</hw>, <pos>n.</pos> <def>A falling on and seizing, as the prey of a rapacious bird; the act of swooping.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The eagle fell, . . . and carried away a whole litter of cubs at a <qex>swoop</qex>.</q> <rj><qau>L'Estrange.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Swoopstake</ent><br/
<hw>Swoop"stake`</hw> <pr>(?)</pr>, <pos>n.</pos> <def>See <er>Sweepstake</er>.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Swoopstake</ent><br/
<hw>Swoop"stake`</hw>, <pos>adv.</pos> <def>Altogether; indiscriminately.</def> <mark>[R.]</mark>  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Swop</ent><br/
<hw>Swop</hw> <pr>(?)</pr>, <pos>v. & n.</pos> <def>Same as <er>Swap</er>.</def>  <rj><au>Dryden.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sword</ent><br/
<hw>Sword</hw> <pr>(s<omac/rd)</pr>, <pos>n.</pos> <ety>[OE. <ets>swerd</ets>, AS. <ets>sweord</ets>; akin to OFries. <ets>swerd</ets>, <ets>swird</ets>, D. <ets>zwaard</ets>, OS. <ets>swerd</ets>, OHG. <ets>swert</ets>, G. <ets>schwert</ets>, Icel. <ets>sver<edh/</ets>, Sw. <ets>sv<aum/rd</ets>, Dan. <ets>svaerd</ets>; of uncertain origin.]</ety> <sn>1.</sn> <def>An offensive weapon, having a long and usually sharp-pointed blade with a cutting edge or edges. It is the general term, including the small sword, rapier, saber, scimiter, and many other varieties.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Hence, the emblem of judicial vengeance or punishment, or of authority and power.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>He [the ruler] beareth not the <qex>sword</qex> in vain.</q> <rj><qau>Rom. xiii. 4.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>She quits the balance, and resigns the <qex>sword</qex>.</q> <rj><qau>Dryden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>Destruction by the sword, or in battle; war; dissension.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>I came not to send peace, but a <qex>sword</qex>.</q> <rj><qau>Matt. x. 34.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>The military power of a country.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>He hath no more authority over the <qex>sword</qex> than over the law.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <fld>(Weaving)</fld> <def>One of the end bars by which the lay of a hand loom is suspended.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Sword arm</b></col>, <cd>the right arm.</cd> -- <col><b>Sword bayonet</b></col>, <cd>a bayonet shaped somewhat like a sword, and which can be used as a sword.</cd> -- <col><b>Sword bearer</b></col>, <cd>one who carries his master's sword; an officer in London who carries a sword before the lord mayor when he goes abroad.</cd> -- <col><b>Sword belt</b></col>, <cd>a belt by which a sword is suspended, and borne at the side.</cd> -- <col><b>Sword blade</b></col>, <cd>the blade, or cutting part, of a sword.</cd> -- <col><b>Sword cane</b></col>, <cd>a cane which conceals the blade of a sword or dagger, as in a sheath.</cd> -- <col><b>Sword dance</b></col>. <sd>(a)</sd> <cd>A dance in which swords are brandished and clashed together by the male dancers.</cd> <au>Sir W. Scott.</au> <sd>(b)</sd> <cd>A dance performed over swords laid on the ground, but without touching them.</cd> -- <col><b>Sword fight</b></col>, <cd>fencing; a combat or trial of skill with swords; swordplay.</cd> -- <col><b>Sword grass</b></col>. <fld>(Bot.)</fld> <cd>See <er>Gladen</er>.</cd> -- <col><b>Sword knot</b></col>, <cd>a ribbon tied to the hilt of a sword.</cd> -- <col><b>Sword law</b></col>, <cd>government by the sword, or by force; violence.</cd> <au>Milton.</au> -- <col><b>Sword lily</b></col>. <fld>(Bot.)</fld> <cd>See <er>Gladiolus</er>.</cd> -- <col><b>Sword mat</b></col> <fld>(Naut.)</fld>, <cd>a mat closely woven of yarns; -- so called from a wooden implement used in its manufacture.</cd> -- <col><b>Sword shrimp</b></col> <fld>(Zool.)</fld>, <cd>a European shrimp (<spn>Pasiphaea sivado</spn>) having a very thin, compressed body.</cd> -- <col><b>Sword stick</b></col>, <cd>a sword cane.</cd> -- <col><b>To measure swords with one</b></col>. <cd>See under <er>Measure</er>, <pos>v. t.</pos></cd> -- <col><b>To put to the sword</b></col>. <cd>See under <er>Put</er>.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Swordbill</ent><br/
<hw>Sword"bill`</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Zool.)</fld> <def>A humming bird (<spn>Docimastes ensiferus</spn>) having a very long, slender bill, exceeding the length of the body of the bird.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sworded</ent><br/
<hw>Sword"ed</hw>, <pos>a.</pos> <ety>[Cf. AS. ge<ets>swurdod</ets>.]</ety> <def>Girded with a sword.</def>  <rj><au>Milton.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sworder</ent><br/
<hw>Sword"er</hw> <pr>(?)</pr>, <pos>n.</pos> <def>One who uses, or fights with, a sword; a swordsman; a soldier; a cutthroat.</def> <mark>[Obs.]</mark>  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Swordfish</ent><br/
<hw>Sword"fish`</hw> <pr>(?)</pr>, <pos>n.</pos> <sn>1.</sn> <fld>(Zool.)</fld> <sd>(a)</sd> <def>A very large oceanic fish (<spn>Xiphias gladius</spn>), the only representative of the family <fam>Xiphiidae</fam>. It is highly valued as a food fish. The bones of the upper jaw are consolidated, and form a long, rigid, swordlike beak; the dorsal fin is high and without distinct spines; the ventral fins are absent. The adult is destitute of teeth. It becomes sixteen feet or more long.</def> <sd>(b)</sd> <def>The gar pike.</def> <sd>(c)</sd> <def>The cutlass fish.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Astron.)</fld> <def>A southern constellation. See <er>Dorado</er>, 1.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Swordfish sucker</b></col> <fld>(Zool.)</fld>, <cd>a remora (<spn>Remora brachyptera</spn>) which attaches itself to the swordfish.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Swordick</ent><br/
<hw>Sword"ick</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Zool.)</fld> <def>The spotted gunnel (<spn>Muraenoides gunnellus</spn>).</def> <mark>[Prov. Eng.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Swording</ent><br/
<hw>Sword"ing</hw> <pr>(?)</pr>, <pos>n.</pos> <def>Slashing with a sword.</def>  <rj><au>Tennyson.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Swordless</ent><br/
<hw>Sword"less</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Destitute of a sword.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Swordman</ent><br/
<hw>Sword"man</hw> <pr>(?)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Swordmen</plw> <pr>(<?/)</pr>.</plu> <def>A swordsman.</def> <ldquo/Sinewy <xex>swordmen</xex>.<rdquo/  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Swordplay</ent><br/
<hw>Sword"play`</hw> <pr>(?)</pr>, <pos>n.</pos> <def>Fencing; a sword fight.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Swordplayer</ent><br/
<hw>Sword"play`er</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A fencer; a gladiator; one who exhibits his skill in the use of the sword.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sword-shaped</ent><br/
<hw>Sword"-shaped`</hw> <pr>(?)</pr>, <pos>a.</pos> <fld>(Bot.)</fld> <def>Shaped like a sword; ensiform, as the long, flat leaves of the Iris, cattail, and the like.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Swordsman</ent><br/
<hw>Swords"man</hw> <pr>(?)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Swordsmen</plw> <pr>(<?/)</pr>.</plu> <sn>1.</sn> <def>A soldier; a fighting man.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>One skilled of a use of the sword; a professor of the science of fencing; a fencer.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Swordsmanship</ent><br/
<hw>Swords"man*ship</hw>, <pos>n.</pos> <def>The state of being a swordsman; skill in the use of the sword.</def>  <rj><au>Cowper.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Swordtail</ent><br/
<hw>Sword"tail`</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Zool.)</fld> <sd>(a)</sd> <def>The limulus.</def> <sd>(b)</sd> <def>Any hemipterous insect of the genus <gen>Uroxiphus</gen>, found upon forest trees.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Swore</ent><br/
<hw>Swore</hw> <pr>(?)</pr>, <def><pos>imp.</pos> of <er>Swear</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sworn</ent><br/
<hw>Sworn</hw> <pr>(?)</pr>, <def><pos>p. p.</pos> of <er>Swear</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Sworn brothers</b></col>, <cd>originally, companions in arms who took an oath to share together good and bad fortune; hence, faithful friends.</cd> -- <col><b>Sworn enemies</b></col>, <cd>determined or irreconcilable enemies.</cd> -- <col><b>Sworn friends</b></col>, <cd>close friends.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Swough</ent><br/
<hw>Swough</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[See <er>Swoon</er>.]</ety> <sn>1.</sn> <def>A sound; a groan; a moan; a sough.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>He sigheth with full many a sorry <qex>swough</qex>.</q> <rj><qau>Chaucer.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A swoon.</def> <mark>[Obs.]</mark>  <rj><au>Chaucer.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Swound</ent><br/
<hw>Swound</hw> <pr>(?)</pr>, <pos>v. & n.</pos> <def>See <er>Swoon</er>, <pos>v. & n.</pos></def> <mark>[Prov. Eng. or Archaic]</mark>  <rj><au>Shak. Dryden.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>The landlord stirred<br/
As one awaking from a <qex>swound</qex>.</q> <rj><qau>Longfellow.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>'Swounds</ent><br/
<hw>'Swounds</hw> <pr>(?)</pr>, <pos>interj.</pos> <ety>[Cf. <er>Zounds</er>.]</ety> <def>An exclamation contracted from <xex>God's wounds</xex>; -- used as an oath.</def> <mark>[Obs. or Archaic]</mark>  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Swown</ent><br/
<hw>Swown</hw> <pr>(?)</pr>, <pos>v. & n.</pos> <def>Swoon.</def> <mark>[Obs.]</mark>  <rj><au>Chaucer.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Swum</ent><br/
<hw>Swum</hw> <pr>(?)</pr>, <def><pos>imp. & p. p.</pos> of <er>Swim</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Swung</ent><br/
<hw>Swung</hw> <pr>(?)</pr>, <def><pos>imp. & p. p.</pos> of <er>Swing</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Swythe</ent><br/
<hw>Swythe</hw> <pr>(?)</pr>, <pos>adv.</pos> <def>Quickly. See <er>Swithe</er>.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sy</ent><br/
<hw>Sy</hw> <pr>(?)</pr>, <mark>obs.</mark> <pos>imp.</pos> <mord>of <er>See</er></mord>. <def>Saw.</def>  <rj><au>Chaucer.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Syb</ent><br/
<hw>Syb</hw> <pr>(?)</pr>, <pos>a.</pos> <def>See <er>Sib</er>.</def> <mark>[Obs. or Scot.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sybarite</ent><br/
<hw>Syb"a*rite</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>Sybarita</ets>, Gr. <?/, fr. <?/, a city in Italy, noted for the effeminacy and voluptuousness of its inhabitants; cf. F. <ets>Sybarite</ets>.]</ety> <def>A person devoted to luxury and pleasure; a voluptuary.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sybaritical</ent><br/
<ent>Sybaritic</ent><br/
<mhw>{ <hw>Syb`a*rit"ic</hw> <pr>(?)</pr>, <hw>Syb`a*rit"ic*al</hw> <pr>(?)</pr>, }</mhw> <pos>a.</pos> <ety>[L. <ets>Sybariticus</ets>, Gr. <?/.]</ety> <def>Of or pertaining to the Sybarites; resembling the Sybarites; luxurious; wanton; effeminate.</def> <ldquo/<xex>Sybaritic</xex> dinners.<rdquo/ <au>Bp. Warburton.</au> <ldquo/<xex>Sybaritical</xex> cloistres.<rdquo/ <au>Bp. Hall.</au><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sybaritism</ent><br/
<hw>Syb"a*rit*ism</hw> <pr>(?)</pr>, <pos>n.</pos> <def>Luxuriousness; effeminacy; wantonness; voluptuousness.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sycamine</ent><br/
<hw>Syc"a*mine</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>sycaminus</ets>, Gr. <?/; perhaps of Semitic origin.]</ety> <def>See <er>Sycamore</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sycamore</ent><br/
<hw>Syc"a*more</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>sycomorus</ets>, Gr. <?/ the fig mulberry; <?/ a fig + <?/ the black mulberry; or perhaps of Semitic origin: cf. F. <ets>sycomore</ets>.  Cf. <er>Mulberry</er>.]</ety> <fld>(Bot.)</fld> <sd>(a)</sd> <def>A large tree (<spn>Ficus Sycomorus</spn>) allied to the common fig.  It is found in Egypt and Syria, and is the sycamore, or sycamine, of Scripture.</def> <sd>(b)</sd> <def>The American plane tree, or buttonwood.</def> <sd>(c)</sd> <def>A large European species of maple (<spn>Acer Pseudo-Platanus</spn>).</def> <altsp>[Written sometimes <asp>sycomore</asp>.]</altsp><br/
[<source>1913 Webster</source>]</p>

<p><ent>Syce</ent><br/
<hw>Syce</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Ar. <ets>s<amac/<ium/s</ets>.]</ety> <def>A groom.</def> <mark>[India]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sycee</ent><br/
<hw>Sy*cee"</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Said to be from a Chinese word, <ets>se-tze</ets> or <ets>se-sze</ets>, meaning, fine silk, and to be so called because if pure it may be drawn out into fine threads.]</ety> <def>Silver, pounded into ingots of the shape of a shoe, and used as currency. The most common weight is about one pound troy.</def> <mark>[China]</mark>  <rj><au>McElrath.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sychnocarpous</ent><br/
<hw>Sych`no*car"pous</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[Gr. <?/ much or frequent + <?/ fruit.]</ety> <fld>(Bot.)</fld> <def>Having the capacity of bearing several successive crops of fruit without perishing; <as>as, <ex>sychnocarpous</ex> plants</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sycite</ent><br/
<hw>Sy"cite</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Gr. <?/ figlike, fr. <?/ a fig.]</ety> <fld>(Min.)</fld> <def>A nodule of flint, or a pebble, which resembles a fig.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sycoceric</ent><br/
<hw>Syc`o*cer"ic</hw> <pr>(?)</pr>, <pos>a.</pos> <fld>(Chem.)</fld> <def>Of, pertaining to, or designating, an acid obtained by the oxidation of sycoceryl alcohol.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sycoceryl</ent><br/
<hw>Syc`o*ce"ryl</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Gr. <?/ a fig + <?/ wax + <ets>-yl</ets>.]</ety> <fld>(Chem.)</fld> <def>A radical, of the aromatic series, regarded as an essential ingredient of certain compounds found in the waxy resin of an Australian species of fig.</def><br/
[<source>1913 Webster</source>]</p>

<p><-- p. 1461 --></p>

<p><ent>Sycock</ent><br/
<hw>Sy"cock</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Zool.)</fld> <def>The missel thrush.</def> <mark>[Prov. Eng.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sycones</ent><br/
||<hw>Sy*co"nes</hw> <pr>(?)</pr>, <pos>n. pl.</pos> <ety>[NL., fr. Gr. <?/ a fig.]</ety> <fld>(Zool.)</fld> <def>A division of calcareous sponges.</def><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ They usually resemble a fig, being vase-shaped with a fringed opening at the summit. The feeding cells are in ampullae connected with radial tubes in the thickened walls of the body.</note><br/
[<source>1913 Webster</source>]</p>

<p><ent>Syconus</ent><br/
<ent>Syconium</ent><br/
<mhw>{ ||<hw>Sy*co"ni*um</hw> <pr>(?)</pr>, ||<hw>Sy*co"nus</hw> <pr>(?)</pr>, }</mhw> <pos>n.</pos> <ety>[NL., fr. Gr. <?/ the fig.]</ety> <fld>(Bot.)</fld> <def>A collective fleshy fruit, in which the ovaries are hidden within a hollow receptacle, as in the fig.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sycophancy</ent><br/
<hw>Syc"o*phan*cy</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Cf. L. <ets>sycophantia</ets> deceit, Gr. <?/ false accusation.]</ety> <def>The character or characteristic of a sycophant.</def> Hence: -<br/
[<source>1913 Webster</source>]</p>

<p><sd>(a)</sd> <def>False accusation; calumniation; talebearing.</def> <mark>[Obs.]</mark>  <rj><au>Bp. Hall.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sd>(b)</sd> <def>Obsequious flattery; servility.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The <qex>sycophancy</qex> of A.Philips had prejudiced Mr. Addison against Pope.</q> <rj><qau>Bp. Warburton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sycophant</ent><br/
<hw>Syc"o*phant</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>sycophanta</ets> a slanderer, deceiver, parasite, Gr. <?/ a false accuser, false adviser, literally, a fig shower; <?/ a fig + <?/ to show: cf. F. <ets>sycophante</ets>. The reason for the name is not certainly known. See <er>Phenomenon</er>.]</ety> <sn>1.</sn> <def>An informer; a talebearer.</def> <mark>[Obs.]</mark> <ldquo/Accusing <xex>sycophants</xex>, of all men, did best sort to his nature.<rdquo/  <rj><au>Sir P. Sidney.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A base parasite; a mean or servile flatterer; especially, a flatterer of princes and great men.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>A <qex>sycophant</qex> will everything admire:<br/
Each verse, each sentence, sets his soul on fire.</q> <rj><qau>Dryden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sycophant</ent><br/
<hw>Syc"o*phant</hw> <pr>(?)</pr>, <pos>v. t.</pos> <ety>[CF. L. <ets>sycophantari</ets> to deceive, to trick, Gr. <?/.]</ety> <sn>1.</sn> <def>To inform against; hence, to calumniate.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q><qex>Sycophanting</qex> and misnaming the work of his adversary.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To play the sycophant toward; to flatter obsequiously.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sycophant</ent><br/
<hw>Syc"o*phant</hw>, <pos>v. i.</pos> <def>To play the sycophant.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sycophantcy</ent><br/
<hw>Syc"o*phant*cy</hw> <pr>(?)</pr>, <pos>n.</pos> <def>Sycophancy.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sycophantical</ent><br/
<ent>Sycophantic</ent><br/
<mhw>{ <hw>Syc`o*phan"tic</hw> <pr>(?)</pr>, <hw>Syc`o*phan"tic*al</hw> <pr>(?)</pr>, }</mhw> <pos>a.</pos> <ety>[Cf. Gr. <?/ slanderous.]</ety> <def>Of or pertaining to a sycophant; characteristic of a sycophant; meanly or obsequiously flattering; courting favor by mean adulation; parasitic.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>To be cheated and ruined by a <qex>sycophantical</qex> parasite.</q> <rj><qau>South.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q><qex>Sycophantic</qex> servants to the King of Spain.</q> <rj><qau>De Quincey.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sycophantish</ent><br/
<hw>Syc"o*phant`ish</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Like a sycophant; obsequiously flattering.</def> -- <wordforms><wf>Syc"o*phant`ish*ly</wf>, <pos>adv.</pos></wordforms><br/
[<source>1913 Webster</source>]</p>

<p><q><qex>Sycophantish</qex> satirists that forever humor the prevailing folly.</q> <rj><qau>De Quincey.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sycophantism</ent><br/
<hw>Syc"o*phant*ism</hw> <pr>(?)</pr>, <pos>n.</pos> <def>Sycophancy.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sycophantize</ent><br/
<hw>Syc"o*phant*ize</hw> <pr>(?)</pr>, <pos>v. i.</pos> <def>To play the sycophant.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sycophantry</ent><br/
<hw>Syc"o*phant*ry</hw> <pr>(?)</pr>, <pos>n.</pos> <def>Sycophancy.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sycosis</ent><br/
||<hw>Sy*co"sis</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[NL., fr. Gr. <?/, fr. <?/ a fig.]</ety> <fld>(Med.)</fld> <def>A pustular eruption upon the scalp, or the beared part of the face, whether due to ringworm, acne, or impetigo.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Syderolite</ent><br/
<hw>Syd"er*o*lite</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A kind of Bohemian earthenware resembling the Wedgwood ware.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sye</ent><br/
<hw>Sye</hw> <pr>(?)</pr>, <mark>obs.</mark> <pos>imp.</pos> <mord>of <er>See</er></mord>. <def>Saw.</def>  <rj><au>Chaucer.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Syenite</ent><br/
<hw>Sy"e*nite</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>Syenites</ets> (sc. <ets>lapis</ets>), from <ets>Syene</ets>, Gr. <?/.]</ety> <fld>(Min.)</fld> <sd>(a)</sd> <def>Orig., a rock composed of quartz, hornblende, and feldspar, anciently quarried at <xex>Syene</xex>, in Upper Egypt, and now called <altname>granite</altname>.</def> <sd>(b)</sd> <def>A granular, crystalline, ingeous rock composed of orthoclase and hornblende, the latter often replaced or accompanied by pyroxene or mica. Syenite sometimes contains nephelite (elaeolite) or leucite, and is then called <altname>nephelite (elaeolite) syenite</altname> or <altname>leucite syenite</altname>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Syenitic</ent><br/
<hw>Sy`e*nit"ic</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[Written also <ets>sienitic</ets>.]</ety> <sn>1.</sn> <def>Relating to Syene; <as>as, <ex>Syenitic</ex> inscriptions</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Relating to, or like, syenite; <as>as, <ex>syenitic</ex> granite</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Syke</ent><br/
<hw>Syke</hw> <pr>(?)</pr>, <pos>n. & v.</pos> <def>See <er>Sike</er>.</def> <mark>[Obs.]</mark>  <rj><au>Chaucer.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Syker</ent><br/
<hw>Syk"er</hw> <pr>(?)</pr>, <pos>a. & adv.</pos> <def>See <er>Sicker</er>.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Syle</ent><br/
<hw>Syle</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[See <er>Sile</er> a young herring.]</ety> <fld>(Zool.)</fld> <def>A young herring (<spn>Clupea harengus</spn>).</def> <altsp>[Also written <asp>sile</asp>.]</altsp><br/
[<source>1913 Webster</source>]</p>

<p><q>But our folk call them <qex>syle</qex>, and nought but <qex>syle</qex>,<br/
And when they're grown, why then we call them herring.</q> <rj><qau>J. Ingelow.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Syllabarium</ent><br/
||<hw>Syl`la*ba"ri*um</hw> <pr>(?)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Syllabaria</plw> <pr>(#)</pr>.</plu> <ety>[NL.]</ety> <def>A syllabary.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Syllabary</ent><br/
<hw>Syl"la*ba*ry</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A table of syllables; more especially, a table of the indivisible syllabic symbols used in certain languages, as the Japanese and Cherokee, instead of letters.</def>  <rj><au>S. W. Williams.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Syllabe</ent><br/
<hw>Syl"labe</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F.]</ety> <def>Syllable.</def> <mark>[R.]</mark>  <rj><au>B. Jonson.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Syllabical</ent><br/
<ent>Syllabic</ent><br/
<mhw>{ <hw>Syl*lab"ic</hw> <pr>(?)</pr>, <hw>Syl*lab"ic*al</hw> <pr>(?)</pr>, }</mhw> <pos>a.</pos> <ety>[Gr. <?/: cf. F. <ets>syllabique</ets>.]</ety> <sn>1.</sn> <def>Of or pertaining to a syllable or syllables; <as>as, <ex>syllabic</ex> accent</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Consisting of a syllable or syllables; <as>as, a <ex>syllabic</ex> augment</as>.</def> <ldquo/The <xex>syllabic</xex> stage of writing.<rdquo/  <rj><au>Earle.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Syllabically</ent><br/
<hw>Syl*lab"ic*al*ly</hw>, <pos>adv.</pos> <def>In a syllabic manner.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Syllabicate</ent><br/
<hw>Syl*lab"i*cate</hw> <pr>(?)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Syllabicated</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Syllabicating</conjf>.]</vmorph> <def>To form or divide into syllables; to syllabify.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Syllabication</ent><br/
<hw>Syl*lab`i*ca"tion</hw> <pr>(?)</pr>, <pos>n.</pos> <def>The act of forming syllables; the act or method of dividing words into syllables. See <xex>Guide to Pron.</xex>, <sect/275.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Syllabification</ent><br/
<hw>Syl*lab`i*fi*ca"tion</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[See <er>Syllabify</er>.]</ety> <def>Same as <er>Syllabication</er>.</def>  <rj><au>Rush.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q><qex>Syllabification</qex> depends not on mere force, but on discontinuity of force.</q>  <rj><au>H. Sweet.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Syllabify</ent><br/
<hw>Syl*lab"i*fy</hw> <pr>(?)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Syllabified</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Syllabifying</conjf> <pr>(?)</pr>.]</vmorph> <ety>[L. <ets>syllaba</ets> syllable + <ets>-fy</ets>.]</ety> <def>To form or divide into syllables.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Syllabism</ent><br/
<hw>Syl"la*bism</hw> <pr>(?)</pr>, <pos>n.</pos> <def>The expressing of the sounds of a language by syllables, rather than by an alphabet or by signs for words.</def>  <rj><au>I. Taylor (The Alphabet).</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Syllabist</ent><br/
<hw>Syl"la*bist</hw> <pr>(?)</pr>, <pos>n.</pos> <def>One who forms or divides words into syllables, or is skilled in doing this.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Syllabize</ent><br/
<hw>Syl"la*bize</hw> <pr>(?)</pr>, <pos>v. t.</pos> <def>To syllabify.</def>  <rj><au>Howell.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Syllable</ent><br/
<hw>Syl"la*ble</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[OE. <ets>sillable</ets>, OF. <ets>sillabe</ets>, F. <ets>syllabe</ets>, L. <ets>syllaba</ets>, Gr. <?/ that which is held together, several letters taken together so as to form one sound, a syllable, fr. <?/ to take together; <?/ with + <?/ to take; cf. Skr. <ets>labh</ets>, <ets>rabh</ets>.  Cf. <er>Lemma</er>, <er>Dilemma</er>.]</ety> <sn>1.</sn> <def>An elementary sound, or a combination of elementary sounds, uttered together, or with a single effort or impulse of the voice, and constituting a word or a part of a word.  In other terms, it is a vowel or a diphtong, either by itself or flanked by one or more consonants, the whole produced by a single impulse or utterance.  One of the liquids, <xex>l</xex>, <xex>m</xex>, <xex>n</xex>, may fill the place of a vowel in a syllable. Adjoining syllables in a word or phrase need not to be marked off by a pause, but only by such an abatement and renewal, or reenforcement, of the stress as to give the feeling of separate impulses. See <xex>Guide to Pronunciation</xex>, <sect/275.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>In writing and printing, a part of a word, separated from the rest, and capable of being pronounced by a single impulse of the voice. It may or may not correspond to a syllable in the spoken language.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Withouten vice [<it>i. e.</it> mistake] of <qex>syllable</qex> or letter.</q> <rj><qau>Chaucer.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>A small part of a sentence or discourse; anything concise or short; a particle.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Before any <qex>syllable</qex> of the law of God was written.</q> <rj><qau>Hooker.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Who dare speak<br/
One <qex>syllable</qex> against him?</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Syllable</ent><br/
<hw>Syl"la*ble</hw>, <pos>v. t.</pos> <def>To pronounce the syllables of; to utter; to articulate.</def>  <rj><au>Milton.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Syllabub</ent><br/
<hw>Syl"la*bub</hw> <pr>(?)</pr>, <pos>n.</pos> <def>Same as <er>Syllabub</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Syllabus</ent><br/
<hw>Syl"la*bus</hw> <pr>(?)</pr>, <pos>n.</pos>; <plu><it>pl.</it> E. <plw>Syllabuses</plw> <pr>(#)</pr>, L. <plw>Syllabi</plw> <pr>(#)</pr>.</plu> <ety>[L., fr. the same source as E. <ets>syllable</ets>.]</ety> <sn>1.</sn> <def>A compendium containing the heads of a discourse, and the like; an abstract.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Law)</fld> <def>The headnote of a reported case; the brief statement of the points of law determined prefixed to a reported case. The opinion controls the syllabus, the latter being merely explanatory of the former.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Syllepsis</ent><br/
||<hw>Syl*lep"sis</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L., fr. Gr. <grk>sy`llhpsis</grk> a taking together, from <?/. See <er>syllable</er>, <pos>n.</pos>]</ety> <sn>1.</sn> <fld>(Rhet.)</fld> <def>A figure of speech by which a word is used in a literal and metaphorical sense at the same time.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Gram.)</fld> <def>The agreement of a verb or adjective with one, rather than another, of two nouns, with either of which it might agree in gender, number, etc.; as, <it>rex</it> et regina <it>beati.</it></def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sylleptical</ent><br/
<ent>Sylleptic</ent><br/
<mhw>{ <hw>Syl*lep"tic</hw> <pr>(?)</pr>, <hw>Syl*lep"tic*al</hw> <pr>(?)</pr>, }</mhw> <pos>a.</pos> <ety>[Gr. <?/ collective.]</ety> <def>Of or pertaining to a syllepsis; containing syllepsis.</def> -- <wordforms><wf>Syl*lep"tic*al*ly</wf>, <pos>adv.</pos></wordforms><br/
[<source>1913 Webster</source>]</p>

<p><ent>Syllidian</ent><br/
<hw>Syl*lid"i*an</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[From NL. <ets>Syllis</ets>, the typical genus.]</ety> <fld>(Zool.)</fld> <def>Any one of numerous species of marine annelids of the family <fam>Syllidae</fam>.</def><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ Many of the species are phosphorescent; others are remarkable for undergoing strobilation or fission and for their polymorphism. The egg, in such species, develops into an asexual individual. When mature, a number of its posterior segments gradually develop into one or more sexual individuals which finally break away and swim free in the sea. The males, females, and neuters usually differ greatly in form and structure.</note><br/
[<source>1913 Webster</source>]</p>

<p><ent>Syllogism</ent><br/
<hw>Syl"lo*gism</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[OE. <ets>silogisme</ets>, OF. <ets>silogime</ets>, <ets>sillogisme</ets>, F. <ets>syllogisme</ets>, L. <ets>syllogismus</ets>, Gr. <grk>syllogismo`s</grk> a reckoning all together, a reasoning, syllogism, fr. <grk>syllogi`zesqai</grk> to reckon all together, to bring at once before the mind, to infer, conclude; <grk>sy`n</grk> with, together + <grk>logi`zesqai</grk> to reckon, to conclude by reasoning. See <er>Syn-</er>, and <er>Logistic</er>, <er>Logic</er>.]</ety> <fld>(Logic)</fld> <def>The regular logical form of every argument, consisting of three propositions, of which the first two are called the <xex>premises</xex>, and the last, the <xex>conclusion</xex>. The conclusion necessarily follows from the premises; so that, if these are true, the conclusion must be true, and the argument amounts to demonstration</def>; <note>as in the following example:<br/
[<source>1913 Webster</source>]</p>

<p>Every virtue is laudable;
Kindness is a virtue;
Therefore kindness is laudable.<br/
[<source>1913 Webster</source>]</p>

<p>These propositions are denominated respectively the <xex>major premise</xex>, the <xex>minor premise</xex>, and the <xex>conclusion</xex>.</note><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ If the premises are not true and the syllogism is regular, the reasoning is valid, and the conclusion, whether true or false, is correctly derived.</note><br/
[<source>1913 Webster</source>]</p>

<p><ent>Syllogistical</ent><br/
<ent>Syllogistic</ent><br/
<mhw>{ <hw>Syl`lo*gis"tic</hw> <pr>(?)</pr>, <hw>Syl`lo*gis"tic*al</hw> <pr>(?)</pr>, }</mhw> <pos>a.</pos> <ety>[L. <ets>syllogisticus</ets>, Gr. <?/: cf. F. <ets>syllogistique</ets>.]</ety> <def>Of or pertaining to a syllogism; consisting of a syllogism, or of the form of reasoning by syllogisms; <as>as, <ex>syllogistic</ex> arguments or reasoning</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Syllogistically</ent><br/
<hw>Syl`lo*gis"tic*al*ly</hw>, <pos>adv.</pos> <def>In a syllogistic manner.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Syllogization</ent><br/
<hw>Syl`lo*gi*za"tion</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A reasoning by syllogisms.</def> <mark>[Obs. or R.]</mark>  <rj><au>Harris.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Syllogize</ent><br/
<hw>Syl"lo*gize</hw> <pr>(?)</pr>, <pos>v. i.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Syllogized</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Syllogizing</conjf> <pr>(?)</pr>.]</vmorph> <ety>[Gr. <?/: cf. F. <ets>syllogiser</ets>.]</ety> <def>To reason by means of syllogisms.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Men have endeavored . . . to teach boys to <qex>syllogize</qex>, or frame arguments and refute them, without any real inward knowledge of the question.</q> <rj><qau>I. Watts.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Syllogizer</ent><br/
<hw>Syl"lo*gi`zer</hw> <pr>(?)</pr>, <pos>n.</pos> <def>One who syllogizes.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sylph</ent><br/
<hw>Sylph</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F. <ets>sylphe</ets>, m., fr. Gr. <?/ a kind of grub, beetle, or moth; -- so called by Paracelsus.]</ety> <sn>1.</sn> <def>An imaginary being inhabiting the air; a fairy.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Fig.: A slender, graceful woman.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <fld>(Zool.)</fld> <def>Any one of several species of very brilliant South American humming birds, having a very long and deeply-forked tail; <as>as, the blue-tailed <ex>sylph</ex> (<spn>Cynanthus cyanurus</spn>)</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sylphid</ent><br/
<hw>Sylph"id</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F. <ets>sylphide</ets>, fem. See <er>Sylph</er>.]</ety> <def>A little sylph; a young or diminutive sylph.</def> <ldquo/The place of the <xex>sylphid</xex> queen.<rdquo/  <rj><au>J. R. Drake.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Ye sylphs and <qex>sylphids</qex>, to your chief give ear,<br/
Fays, fairies, genii, elves, and demons, hear.</q> <rj><qau>Pope.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sylphine</ent><br/
<hw>Sylph"ine</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Like a sylph.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sylphish</ent><br/
<hw>Sylph"ish</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Sylphlike.</def>  <rj><au>Carlyle.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sylphlike</ent><br/
<hw>Sylph"like`</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Like a sylph; airy; graceful.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Sometimes a dance . . . <br/
Displayed some <qex>sylphlike</qex> figures in its maze.</q> <rj><qau>Byron.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sylva</ent><br/
||<hw>Syl"va</hw> <pr>(?)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Sylvae</plw> <pr>(#)</pr>.</plu> <ety>[L. <ets>sylva</ets>, better <ets>silva</ets>, a wood. See <er>Silva</er>.]</ety> <fld>(Bot.)</fld> <def>Same as <er>Silva</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sylvan</ent><br/
<hw>Syl"van</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[See <er>Silvan</er>, <pos>a.</pos>]</ety> <sn>1.</sn> <def>Of or pertaining to a sylva; forestlike; hence, rural; rustic.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The traditional memory of a rural and a <qex>sylvan</qex> region . . . is usually exact as well as tenacious.</q> <rj><qau>De Quincey.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Abounding in forests or in trees; woody.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sylvan</ent><br/
<hw>Syl"van</hw>, <pos>n.</pos> <ety>[L. <ets>Sylvanus</ets>, better <ets>Silvanus</ets>. See <er>Silvan</er>, <pos>a.</pos>]</ety> <def>A fabled deity of the wood; a satyr; a faun; sometimes, a rustic.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Her private orchards, walled on every side,<br/
To lawless <qex>sylvans</qex> all access denied.</q> <rj><qau>Pope.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sylvan</ent><br/
<hw>Syl"van</hw>, <pos>n.</pos> <ety>[<ets>Sylva</ets> + furfur<ets>an</ets>.]</ety> <fld>(Chem.)</fld> <def>A liquid hydrocarbon obtained together with furfuran (tetrol) by the distillation of pine wood; -- called also <altname>methyl tetrol</altname>, or <altname>methyl furfuran</altname>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sylvanite</ent><br/
<hw>Syl"van*ite</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Fr. Tran<ets>sylvania</ets>, where first found.]</ety> <fld>(Min.)</fld> <def>A telluride of gold and silver, <chform>(Au, Ag)Te2</chform>, of a steel gray, silver white, or brass yellow. It often occurs in implanted crystals resembling written characters, and hence is called <altname>graphic tellurium</altname>. H., 1.5-2. Sp.gr., 7.9-8.3.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Sylvanium</ent><br/
<hw>Syl*va"ni*um</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[NL., so called from Tran<ets>sylvania</ets>, where it was first found.]</ety> <fld>(Chem.)</fld> <def>An old name for tellurium.</def> <altsp>[Written also <asp>silvanium</asp>.]</altsp><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sylvate</ent><br/
<hw>Syl"vate</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Chem.)</fld> <def>A salt of sylvic acid.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sylvatic</ent><br/
<hw>Syl*vat"ic</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>sylvaticus</ets>, better <ets>silvaticus</ets>. See <er>Silvan</er>, <pos>a.</pos>]</ety> <def>Sylvan.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sylvestrian</ent><br/
<hw>Syl*ves"tri*an</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>sylvestris</ets>, better <ets>silvestris</ets>.]</ety> <def>Sylvan.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sylvic</ent><br/
<hw>Syl"vic</hw> <pr>(?)</pr>, <pos>a.</pos> <fld>(Chem.)</fld> <def>Of, pertaining to, or resembling, pine or its products; specifically, designating an acid called also <xex>abeitic acid</xex>, which is the chief ingredient of common resin (obtained from <spn>Pinus sylvestris</spn>, and other species).</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sylvicoline</ent><br/
<hw>Syl*vic"o*line</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>sylva</ets>, <ets>silva</ets>, forest + <ets>colere</ets> to inhabit.]</ety> <fld>(Zool.)</fld> <def>Of or pertaining to the family of warblers (<fam>Sylvicolidae</fam>). See <er>Warbler</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sylviculture</ent><br/
<hw>Syl"vi*cul`ture</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>sylva</ets>, <ets>silva</ets>, forest + E. <ets>culture</ets>.]</ety> <def>The cultivation of forest trees for timber or other purposes; forestry; arboriculture.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sylviculturist</ent><br/
<hw>Syl`vi*cul"tur*ist</hw> <pr>(?)</pr>, <pos>n.</pos> <def>One who cultivates forest trees, especially as a business.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sylvite</ent><br/
<ent>Sylvine</ent><br/
<mhw>{ <hw>Syl"vine</hw> <pr>(?)</pr>, <hw>Syl"vite</hw> <pr>(?)</pr>, }</mhw> <pos>n.</pos> <ety>[So called from NL. sal digestivus <ets>sylvii</ets> potassium chloride.]</ety> <fld>(Min.)</fld> <def>Native potassium chloride.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sym-</ent><br/
<hw>Sym-</hw> <pr>(?)</pr>. <def>See <er>Syn-</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Symarr</ent><br/
<ent>Symar</ent><br/
<mhw>{ <hw>Sy*mar"</hw> <pr>(?)</pr>, <hw>Sy"marr</hw>  }</mhw>, <pos>n.</pos> <def>See <er>Simar</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Symbal</ent><br/
<hw>Sym"bal</hw> <pr>(?)</pr>, <pos>n.</pos> <def>See <er>Cimbal</er>.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Symbiosis</ent><br/
||<hw>Sym`bi*o"sis</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[NL., fr. Gr. <grk>symbi`wsis</grk> a living together, <grk>symbioy^n</grk> to live together; <grk>sy`n</grk> with + <?/ to live.]</ety> <fld>(Biol.)</fld> <def>The living together in more or less imitative association or even close union of two dissimilar organisms. In a broad sense the term includes parasitism, or <col><b>antagonistic symbiosis</b></col> <it>or</it> <col><b>antipathetic symbiosis</b></col>, in which the association is disadvantageous or destructive to one of the organisms, but ordinarily it is used of cases where the association is advantageous, or often necessary, to one or both, and not harmful to either. When there is bodily union (in extreme cases so close that the two form practically a single body, as in the union of algae and fungi to form lichens, and in the inclusion of algae in radiolarians) it is called <col><b>conjunctive symbiosis</b></col>; if there is no actual union of the organisms (as in the association of ants with myrmecophytes), <col><b>disjunctive symbiosis</b></col>.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Symbiotic</ent><br/
<hw>Sym`bi*ot"ic</hw> <pr>(<icr/m`b<icr/*<ocr/t"<icr/k)</pr>, <pos>a.</pos> <ety>[Gr. <grk>symbiwtiko`s</grk>.]</ety> <fld>(Biol.)</fld> <def>Pertaining to, or characterized by, or living in, a state of symbiosis.</def> -- <wordforms><wf>Sym`bi*ot"ic*al</wf> <pr>(#)</pr>, <pos>a.</pos> -- <wf>Sym`bi*ot"ic*al*ly</wf> <pr>(#)</pr>, <pos>adv.</pos></wordforms><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Symbol</ent><br/
<hw>Sym"bol</hw> <pr>(s<icr/m"b<ocr/l)</pr>, <pos>n.</pos> <ety>[L. <ets>symbolus</ets>, <ets>symbolum</ets>, Gr. <grk>sy`mbolon</grk> a sign by which one knows or infers a thing, from <grk>symba`llein</grk> to throw or put together, to compare; <grk>sy`n</grk> with + <grk>ba`llein</grk> to throw: cf. F. <ets>symbole</ets>.  Cf. <er>Emblem</er>, <er>Parable</er>.]</ety> <sn>1.</sn> <def>A visible sign or representation of an idea; anything which suggests an idea or quality, or another thing, as by resemblance or by convention; an emblem; a representation; a type; a figure; <as>as, the lion is the <ex>symbol</ex> of courage; the lamb is the <ex>symbol</ex> of meekness or patience</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>A <qex>symbol</qex> is a sign included in the idea which it represents, <it>e. g.</it>, an actual part chosen to represent the whole, or a lower form or species used as the representative of a higher in the same kind.</q> <rj><qau>Coleridge.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Math.)</fld> <def>Any character used to represent a quantity, an operation, a relation, or an abbreviation.</def><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ In crystallography, the <xex>symbol</xex> of a plane is the numerical expression which defines its position relatively to the assumed axes.</note><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <fld>(Theol.)</fld> <def>An abstract or compendium of faith or doctrine; a creed, or a summary of the articles of religion.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <ety>[Gr. <?/ contributions.]</ety> <def>That which is thrown into a common fund; hence, an appointed or accustomed duty.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>They do their work in the days of peace . . . and come to pay their <qex>symbol</qex> in a war or in a plague.</q> <rj><qau>Jer. Taylor.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <def>Share; allotment.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>The persons who are to be judged . . . shall all appear to receive their <qex>symbol</qex>.</q> <rj><qau>Jer. Taylor.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>6.</sn> <fld>(Chem.)</fld> <def>An abbreviation standing for the name of an element and consisting of the initial letter of the Latin or New Latin name, or sometimes of the initial letter with a following one; <as>as, <chform>C</chform> for carbon, <chform>Na</chform> for sodium (Natrium), <chform>Fe</chform> for iron (Ferrum), <chform>Sn</chform> for tin (Stannum), <chform>Sb</chform> for antimony (Stibium), etc.</as>  See the list of names and symbols under <er>Element</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ In pure and organic chemistry there are symbols not only for the elements, but also for their grouping in formulas, radicals, or residues, as evidenced by their composition, reactions, synthesis, etc. See the diagram of <cref>Benzene nucleus</cref>, under <er>Benzene</er>.</note><br/
[<source>1913 Webster</source>]</p>

<p><syn><b>Syn.</b> -- Emblem; figure; type. See <er>Emblem</er>.</syn><br/
[<source>1913 Webster</source>]</p>

<p><ent>Symbol</ent><br/
<hw>Sym"bol</hw>, <pos>v. t.</pos> <def>To symbolize.</def> <mark>[R.]</mark>  <rj><au>Tennyson.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Symbolic</ent><br/
<hw>Sym*bol"ic</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Cf. F. <ets>symbolique</ets>. See <er>Symbolic</er>, <pos>a.</pos>]</ety> <fld>(Theol.)</fld> <def>See <er>Symbolics</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Symbolical</ent><br/
<ent>Symbolic</ent><br/
<mhw>{ <hw>Sym*bol"ic</hw> <pr>(?)</pr>, <hw>Sym*bol"ic*al</hw> <pr>(?)</pr>, }</mhw> <pos>a.</pos> <ety>[L. <ets>symbolicus</ets>, Gr. <grk>symboliko`s</grk>: cf. F. <ets>symbolique</ets>.]</ety> <def>Of or pertaining to a symbol or symbols; of the nature of a symbol; exhibiting or expressing by resemblance or signs; representative; <as>as, the figure of an eye is <ex>symbolic</ex> of sight and knowledge</as>.</def> -- <wordforms><wf>Sym*bol"ic*al*ly</wf>, <pos>adv.</pos> -- <wf>Sym*bol"ic*al*ness</wf>, <pos>n.</pos></wordforms><br/
[<source>1913 Webster</source>]</p>

<p><q>The sacrament is a representation of Christ's death by such <qex>symbolical</qex> actions as he himself appointed.</q> <rj><qau>Jer. Taylor.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Symbolical delivery</b></col> <fld>(Law)</fld>, <cd>the delivery of property sold by delivering something else as a symbol, token, or representative of it.</cd> <au>Bouvier.  Chitty.</au> -- <col><b>Symbolical philosophy</b></col>, <cd>the philosophy expressed by hieroglyphics.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><-- p. 1462 --></p>

<p><ent>Symbolics</ent><br/
<hw>Sym*bol"ics</hw> <pr>(?)</pr>, <pos>n.</pos> <def>The study of ancient symbols</def>; esp. <fld>(Theol.)</fld>, <def>that branch of historic theology which treats of creeds and confessions of faith; symbolism; -- called also <altname>symbolic</altname>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Symbolism</ent><br/
<hw>Sym"bol*ism</hw> <pr>(?)</pr>, <pos>n.</pos> <sn>1.</sn> <def>The act of symbolizing, or the state of being symbolized; <as>as, <ex>symbolism</ex> in Christian art is the representation of truth, virtues, vices, etc., by emblematic colors, signs, and forms</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A system of symbols or representations.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <fld>(Chem.)</fld> <sd>(a)</sd> <def>The practice of using symbols, or the system of notation developed thereby.</def> <sd>(b)</sd> <def>A combining together of parts or ingredients.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <fld>(Theol.)</fld> <def>The science of creeds; symbolics.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Symbolist</ent><br/
<hw>Sym"bol*ist</hw>, <pos>n.</pos> <def>One who employs symbols.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Symbolistical</ent><br/
<ent>Symbolistic</ent><br/
<mhw>{ <hw>Sym`bol*is"tic</hw> <pr>(?)</pr>, <hw>Sym`bol*is"tic*al</hw> <pr>(?)</pr>, }</mhw> <pos>a.</pos> <def>Characterized by the use of symbols; <as>as, <ex>symbolistic</ex> poetry</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Symbolization</ent><br/
<hw>Sym`bol*i*za"tion</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Cf. F. <ets>symbolisation</ets>.]</ety> <def>The act of symbolizing; symbolical representation.</def>  <rj><au>Sir T. Browne.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Symbolize</ent><br/
<hw>Sym"bol*ize</hw> <pr>(?)</pr>, <pos>v. i.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Symbolized</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Symbolizing</conjf> <pr>(?)</pr>.]</vmorph> <ety>[Cf. F. <ets>symboliser</ets>.]</ety> <sn>1.</sn> <def>To have a resemblance of qualities or properties; to correspond; to harmonize.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The pleasing of color <qex>symbolizeth</qex> with the pleasing of any single tone to the ear; but the pleasing of order doth <qex>symbolize</qex> with harmony.</q> <rj><qau>Bacon.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>They both <qex>symbolize</qex> in this, that they love to look upon themselves through multiplying glasses.</q> <rj><qau>Howell.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To hold the same faith; to agree.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>The believers in pretended miracles have always previously <qex>symbolized</qex> with the performers of them.</q> <rj><qau>G. S. Faber.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>To use symbols; to represent ideas symbolically.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Symbolize</ent><br/
<hw>Sym"bol*ize</hw>, <pos>v. t.</pos> <sn>1.</sn> <def>To make to agree in properties or qualities.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To make representative of something; to regard or treat as symbolic.</def> <ldquo/Some <xex>symbolize</xex> the same from the mystery of its colors.<rdquo/  <rj><au>Sir T. Browne.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>To represent by a symbol or symbols.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Symbolizer</ent><br/
<hw>Sym"bol*i`zer</hw> <pr>(?)</pr>, <pos>n.</pos> <def>One who symbolizes.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Symbological</ent><br/
<hw>Sym`bo*log"i*cal</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Pertaining to a symbology; versed in, or characterized by, symbology.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Symbologist</ent><br/
<hw>Sym*bol"o*gist</hw> <pr>(?)</pr>, <pos>n.</pos> <def>One who practices, or who is versed in, symbology.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Symbology</ent><br/
<hw>Sym*bol"o*gy</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[<ets>Symbol</ets> + <ets>-logy</ets>.]</ety> <def>The art of expressing by symbols.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Symbranchii</ent><br/
||<hw>Sym*bran"chi*i</hw> <pr>(?)</pr>, <pos>n. pl.</pos> <ety>[NL., fr. Gr. <grk>sy`n</grk> with + <?/ a gill.]</ety> <fld>(Zool.)</fld> <def>An order of slender eel-like fishes having the gill openings confluent beneath the neck. The pectoral arch is generally attached to the skull, and the entire margin of the upper jaw is formed by the premaxillary. Called also <altname>Symbranchia</altname>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Symmetral</ent><br/
<hw>Sym"me*tral</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Commensurable; symmetrical.</def> <mark>[Obs.]</mark>  <rj><au>Dr. H. More.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Symmetrian</ent><br/
<hw>Sym*me"tri*an</hw> <pr>(?)</pr>, <pos>n.</pos> <def>One eminently studious of symmetry of parts.</def> <mark>[R.]</mark>  <rj><au>Sir P. Sidney.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Symmetric</ent><br/
<hw>Sym*met"ric</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Symmetrical.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Symmetrical</ent><br/
<hw>Sym*met"ric*al</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[Cf. F. <ets>sym<eacute/trique</ets>. See <er>Symmetry</er>.]</ety> <sn>1.</sn> <def>Involving or exhibiting symmetry; proportional in parts; having its parts in due proportion as to dimensions; <as>as, a <ex>symmetrical</ex> body or building</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Biol.)</fld> <def>Having the organs or parts of one side corresponding with those of the other; having the parts in two or more series of organs the same in number; exhibiting a symmetry.  See <er>Symmetry</er>, 2.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <fld>(Bot.)</fld> <sd>(a)</sd> <def>Having an equal number of parts in the successive circles of floral organs; -- said of flowers.</def> <sd>(b)</sd> <def>Having a likeness in the form and size of floral organs of the same kind; regular.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <fld>(Math.)</fld> <def>Having a common measure; commensurable.</def> <sd>(b)</sd> <def>Having corresponding parts or relations.</def><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ A curve or a plane figure is <xex>symmetrical</xex> with respect to a given line, and a line, surface, or solid with respect to a plane, when for each point on one side of the line or plane there is a corresponding point on the other side, so situated that the line joining the two corresponding points is perpendicular to the line or plane and is bisected by it. Two solids are <xex>symmetrical</xex> when they are so situated with respect to an intervening plane that the several points of their surfaces thus correspond to each other in position and distance. In analysis, an expression is <xex>symmetrical</xex> with respect to several letters when any two of them may change places without affecting the expression; as, the expression <mathex>a<exp>2</exp>b + ab<exp>2</exp> + a<exp>2</exp>c + ac<exp>2</exp> + b<exp>2</exp>c + bc<exp>2</exp></mathex>, is <xex>symmetrical</xex> with respect to the letters <it>a</it>, <it>b</it>, <it>c</it>.</note><br/
[<source>1913 Webster</source>]</p>

<p>-- <wordforms><wf>Sym*met"ric*al*ly</wf>, <pos>adv.</pos> -- <wf>Sym*met"ric*al*ness</wf>, <pos>n.</pos></wordforms><br/
[<source>1913 Webster</source>]</p>

<p><ent>Symmetrician</ent><br/
<hw>Sym`me*tri"cian</hw> <pr>(?)</pr>, <pos>n.</pos> <def>Same as <er>Symmetrian</er>.</def> <mark>[R.]</mark>  <rj><au>Holinshed.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Symmetrist</ent><br/
<hw>Sym"me*trist</hw> <pr>(?)</pr>, <pos>n.</pos> <def>One eminently studious of symmetry of parts.</def>  <rj><au>Sir H. Wotton.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Symmetrize</ent><br/
<hw>Sym"me*trize</hw> <pr>(?)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Symmetrized</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Symmetrizing</conjf> <pr>(?)</pr>.]</vmorph> <ety>[Cf. F. <ets>sym<eacute/triser</ets>.]</ety> <def>To make proportional in its parts; to reduce to symmetry.</def>  <rj><au>Burke.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Symmetry</ent><br/
<hw>Sym"me*try</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>symmetria</ets>, Gr. <?/; <grk>sy`n</grk> with, together + <?/ a measure: cf. F. <ets>sym<eacute/trie</ets>. See <er>Syn-</er>, and <er>Meter</er> rhythm.]</ety> <sn>1.</sn> <def>A due proportion of the several parts of a body to each other; adaptation of the form or dimensions of the several parts of a thing to each other; the union and conformity of the members of a work to the whole.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Biol.)</fld> <def>The law of likeness; similarity of structure; regularity in form and arrangement; orderly and similar distribution of parts, such that an animal may be divided into parts which are structurally symmetrical.</def><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ <xex>Bilateral symmetry</xex>, or <xex>two-sidedness</xex>, in vertebrates, etc., is that in which the body can be divided into symmetrical halves by a vertical plane passing through the middle; <xex>radial symmetry</xex>, as in echinoderms, is that in which the individual parts are arranged symmetrically around a central axis; <xex>serial symmetry</xex>, or <xex>zonal symmetry</xex>, as in earthworms, is that in which the segments or metameres of the body are disposed in a zonal manner one after the other in a longitudinal axis. This last is sometimes called <xex>metamerism</xex>.</note><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <fld>(Bot.)</fld> <sd>(a)</sd> <def>Equality in the number of parts of the successive circles in a flower.</def> <sd>(b)</sd> <def>Likeness in the form and size of floral organs of the same kind; regularity.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Axis of symmetry</b></col>. <fld>(Geom.)</fld> <cd>See under <er>Axis</er>.</cd> -- <col><b>Respective symmetry</b></col>, <cd>that disposition of parts in which only the opposite sides are equal to each other.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sympathetic</ent><br/
<hw>Sym`pa*thet"ic</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[See <er>Sympathy</er>, and cf. <er>Pathetic</er>.]</ety> <sn>1.</sn> <def>Inclined to sympathy; sympathizing.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Far wiser he, whose <qex>sympathetic</qex> mind<br/
Exults in all the good of all mankind.</q> <rj><qau>Goldsmith.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Produced by, or expressive of, sympathy.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Ope the sacred source of <qex>sympathetic</qex> tears.</q> <rj><qau>Gray.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <fld>(Physiol.)</fld> <sd>(a)</sd> <def>Produced by sympathy; -- applied particularly to symptoms or affections. See <er>Sympathy</er>.</def> <sd>(b)</sd> <def>Of or relating to the sympathetic nervous system or some of its branches; produced by stimulation on the sympathetic nervious system or some part of it; <as>as, the <ex>sympathetic</ex> saliva, a modified form of saliva, produced from some of the salivary glands by stimulation of a sympathetic nerve fiber</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Sympathetic ink</b></col>. <fld>(Chem.)</fld> <cd>See under <er>Ink</er>.</cd> -- <col><b>Sympathetic nerve</b></col> <fld>(Anat.)</fld>, <cd>any nerve of the sympathetic system; especially, the axial chain of ganglions and nerves belonging to the sympathetic system.</cd> -- <col><b>Sympathetic powder</b></col> <fld>(Alchemy)</fld>, <cd>a kind of powder long supposed to be able to cure a wound if applied to the weapon that inflicted it, or even to a portion of the bloody clothes.</cd> <au>Dunglison.</au> -- <col><b>Sympathetic sounds</b></col> <fld>(Physics)</fld>, <cd>sounds produced from solid bodies by means of vibrations which have been communicated to them from some other sounding body, by means of the air or an intervening solid.</cd> -- <col><b>Sympathetic system</b></col> <fld>(Anat.)</fld>, <cd>a system of nerves and nerve ganglions connected with the alimentary canal, the vascular system, and the glandular organs of most vertebrates, and controlling more or less their actions. The axial part of the system and its principal ganglions and nerves are situated in the body cavity and form a chain of ganglions on each side of the vertebral column connected with numerous other ganglions and nerve plexuses.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sympathetical</ent><br/
<hw>Sym`pa*thet"ic*al</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Sympathetic.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sympathetically</ent><br/
<hw>Sym`pa*thet"ic*al*ly</hw>, <pos>adv.</pos> <def>In a sympathetic manner.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sympathist</ent><br/
<hw>Sym"pa*thist</hw> <pr>(?)</pr>, <pos>n.</pos> <def>One who sympathizes; a sympathizer.</def> <mark>[R.]</mark>  <rj><au>Coleridge.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sympathize</ent><br/
<hw>Sym"pa*thize</hw> <pr>(?)</pr>, <pos>v. i.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Sympathized</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Sympathizing</conjf> <pr>(?)</pr>.]</vmorph> <ety>[F. <ets>sympathiser</ets>. See <er>Sympathy</er>.]</ety> <sn>1.</sn> <def>To have a common feeling, as of bodily pleasure or pain.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The mind will <qex>sympathize</qex> so much with the anguish and debility of the body, that it will be too distracted to fix itself in meditation.</q> <rj><qau>Buckminster.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To feel in consequence of what another feels; to be affected by feelings similar to those of another, in consequence of knowing the person to be thus affected.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Their countrymen . . . <qex>sympathized</qex> with their heroes in all their adventures.</q> <rj><qau>Addison.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>To agree; to be in accord; to harmonize.</def>  <rj><au>Dryden.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sympathize</ent><br/
<hw>Sym"pa*thize</hw>, <pos>v. t.</pos> <sn>1.</sn> <def>To experience together.</def> <mark>[Obs.]</mark> <ldquo/This <xex>sympathized</xex> . . . error.<rdquo/  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To ansew to; to correspond to.</def> <mark>[Obs.]</mark>  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sympathizer</ent><br/
<hw>Sym"pa*thi`zer</hw> <pr>(?)</pr>, <pos>n.</pos> <def>One who sympathizes.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sympathy</ent><br/
<hw>Sym"pa*thy</hw> <pr>(?)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Sympathies</plw> <pr>(#)</pr>.</plu> <ety>[F. <ets>sympathie</ets>, L. <ets>sympathia</ets>, Gr. <?/; <grk>sy`n</grk> with + <?/ suffering, passion, fr. <?/, <?/, to suffer. See <er>Syn-</er>, and <er>Pathos</er>.]</ety> <sn>1.</sn> <def>Feeling corresponding to that which another feels; the quality of being affected by the affection of another, with feelings correspondent in kind, if not in degree; fellow-feeling.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>They saw, but other sight instead -- a crowd<br/
Of ugly serpents! Horror on them fell,<br/
And horrid <qex>sympathy</qex>.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>An agreement of affections or inclinations, or a conformity of natural temperament, which causes persons to be pleased, or in accord, with one another; <as>as, there is perfect <ex>sympathy</ex> between them</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>Kindness of feeling toward one who suffers; pity; commiseration; compassion.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>I value myself upon <qex>sympathy</qex>, I hate and despise myself for envy.</q> <rj><qau>Kames.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <fld>(Physiol. & Med.)</fld> <sd>(a)</sd> <def>The reciprocal influence exercised by organs or parts on one another, as shown in the effects of a diseased condition of one part on another part or organ, as in the vomiting produced by a tumor of the brain.</def> <sd>(b)</sd> <def>The influence of a certain psychological state in one person in producing a like state in another.</def> <note>In the original 1890 work, sense <it>(b)</it> was described as: <ldquo/That relation which exists between different persons by which one of them produces in the others a state or condition like that of himself. This is shown in the tendency to yawn which a person often feels on seeing another yawn, or the strong inclination to become hysteric experienced by many women on seeing another person suffering with hysteria.<rdquo/</note><br/
[<source>Webster 1913 Suppl.</source> <source>+PJC</source>]</p>

<p><sn>5.</sn> <def>A tendency of inanimate things to unite, or to act on each other; <as>as, the <ex>sympathy</ex> between the loadstone and iron</as>.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><sn>6.</sn> <def>Similarity of function, use office, or the like.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The adverb has most <qex>sympathy</qex> with the verb.</q> <rj><qau>Earle.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><syn><b>Syn.</b> -- Pity; fellow-feeling; compassion; commiseration; tenderness; condolence; agreement.</syn>  -- <usage><er>Sympathy</er>, <er>Commiseration</er>. <xex>Sympathy</xex> is literally a fellow-feeling with others in their varied conditions of joy or of grief. This term, however, is now more commonly applied to a fellow-feeling with others under affliction, and then coincides very nearly with <xex>commiseration</xex>. In this case it is commonly followed by <xex>for</xex>; <as>as, to feel <ex>sympathy</ex> for a friend when we see him distressed</as>. The verb <xex>sympathize</xex> is followed by <xex>with</xex>; <as>as, to <ex>sympathize</ex> with a friend in his distresses or enjoyments</as>. <ldquo/Every man would be a distinct species to himself, were there no <xex>sympathy</xex> among individuals.<rdquo/ <au>South.</au> See <er>Pity</er>.</usage><br/
[<source>1913 Webster</source>]</p>

<p><q>Fault,<br/
Acknowledged and deplored, in Adam wrought<br/
<qex>Commiseration</qex>.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sympetalous</ent><br/
<hw>Sym*pet"al*ous</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[Pref. <ets>sym-</ets> + <ets>petal</ets>.]</ety> <fld>(Bot.)</fld> <def>Having the petals united; gamopetalous.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Symphonic</ent><br/
<hw>Sym*phon"ic</hw> <pr>(?)</pr>, <pos>a.</pos> <sn>1.</sn> <def>Symphonious.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Mus.)</fld> <def>Relating to, or in the manner of, symphony; <as>as, the <ex>symphonic</ex> form or style of composition</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Symphonious</ent><br/
<hw>Sym*pho"ni*ous</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[From <er>Symphony</er>.]</ety> <sn>1.</sn> <def>Agreeing in sound; accordant; harmonious.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Followed with acclamation and the sound<br/
<qex>Symphonious</qex> of ten thousand harps.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Mus.)</fld> <def>Symphonic.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Symphonist</ent><br/
<hw>Sym"pho*nist</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Cf. F. <ets>symphoniste</ets>.]</ety> <def>A composer of symphonies.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Symphonize</ent><br/
<hw>Sym"pho*nize</hw> <pr>(?)</pr>, <pos>v. i.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Symphonized</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Symphonizing</conjf> <pr>(?)</pr>.]</vmorph> <def>To agree; to be in harmony.</def> <mark>[R.]</mark>  <rj><au>Boyle.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Symphony</ent><br/
<hw>Sym"pho*ny</hw> <pr>(?)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Symphonies</plw> <pr>(#)</pr>.</plu> <ety>[F. <ets>symphonie</ets> (cf. It. <ets>sinfonia</ets>), L. <ets>symphonia</ets>, Gr. <?/; <grk>sy`n</grk> with + <?/ a sound, the voice. See <er>Phonetic</er>.]</ety> <sn>1.</sn> <def>A consonance or harmony of sounds, agreeable to the ear, whether the sounds are vocal or instrumental, or both.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The trumpets sound,<br/
And warlike <qex>symphony</qex> in heard around.</q> <rj><qau>Dryden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A stringed instrument formerly in use, somewhat resembling the virginal.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>With harp and pipe and <qex>symphony</qex>.</q> <rj><qau>Chaucer.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <fld>(Mus.)</fld> <sd>(a)</sd> <def>An elaborate instrumental composition for a full orchestra, consisting usually, like the sonata, of three or four contrasted yet inwardly related movements, as the allegro, the adagio, the minuet and trio, or scherzo, and the finale in quick time. The term has recently been applied to large orchestral works in freer form, with arguments or programmes to explain their meaning, such as the <ldquo/symphonic poems<rdquo/ of Liszt. The term was formerly applied to any composition for an orchestra, as overtures, etc., and still earlier, to certain compositions partly vocal, partly instrumental.</def> <sd>(b)</sd> <def>An instrumental passage at the beginning or end, or in the course of, a vocal composition; a prelude, interlude, or postude; a ritornello.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Symphyla</ent><br/
||<hw>Sym*phy"la</hw> <pr>(?)</pr>, <pos>n. pl.</pos> <ety>[NL., fr. Gr. <grk>sy`n</grk> with + <?/ a clan.]</ety> <fld>(Zool.)</fld> <def>An order of small apterous insects having an elongated body, with three pairs of thoracic and about nine pairs of abdominal legs. They are, in many respects, intermediate between myriapods and true insects.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Symphyseal</ent><br/
<hw>Sym*phys"e*al</hw> <pr>(?)</pr>, <pos>a.</pos> <fld>(Anat.)</fld> <def>Of or pertaining to to symphysis.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Symphyseotomy</ent><br/
<hw>Sym`phy*se*ot"o*my</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[NL. <ets>symphysis pubis</ets> + Gr. <?/ to cut.]</ety> <fld>(Surg.)</fld> <def>The operation of dividing the symphysis pubis for the purpose of facilitating labor; -- formerly called the <xex>Sigualtian section</xex>.</def> <altsp>[Written also <asp>symphysotomy</asp>.]</altsp>  <rj><au>Dunglison.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Symphysis</ent><br/
||<hw>Sym"phy*sis</hw> <pr>(?)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Symphyses</plw> <pr>(#)</pr>.</plu> <ety>[NL., fr. Gr. <?/, fr. <?/ to make to grow together; <grk>sy`n</grk> with + <?/ to cause to grow; to grow.]</ety> <fld>(Anat.)</fld> <sd>(a)</sd> <def>An articulation formed by intervening cartilage; <as>as, the pubic <ex>symphysis</ex></as>.</def> <sd>(b)</sd> <def>The union or coalescence of bones; also, the place of union or coalescence; <as>as, the <ex>symphysis</ex> of the lower jaw</as>.  Cf. <er>Articulation</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Symphysotomy</ent><br/
<hw>Sym`phy*sot"o*my</hw> <pr>(?)</pr>, <pos>n.</pos> <def>Symphyseotomy.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Symphytism</ent><br/
<hw>Sym"phy*tism</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Gr. <?/ grown together.]</ety> <def>Coalescence; a growing into one with another word.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>Some of the phrasal adverbs have assumed the form of single words, by that <qex>symphytism</qex> which naturally attaches these light elements to each other.</q> <rj><qau>Earle.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sympiesometer</ent><br/
<hw>Sym`pi*e*som"e*ter</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Gr. <?/ compression (fr. <?/ to press together; <grk>sy`n</grk> with + <?/ to press, squeeze) + <ets>-meter</ets>.]</ety> <def>A sensitive kind of barometer, in which the pressure of the atmosphere, acting upon a liquid, as oil, in the lower portion of the instrument, compresses an elastic gas in the upper part.</def><br/
[<source>1913 Webster</source>]</p>

<p><-- Figure of a sympiesometer, with labeled parts. -->
<note><hand/ The column of oil of a lower part <xex>BC</xex> of a glass tube compresses hydrogen gas in the upper part <xex>AB</xex>, and is thus measured on the scale <xex>pq</xex> by the position of a surface of the oil in the tube. The scale <xex>pq</xex> is adjustable, and its index must be set to the division on the scale <xex>rs</xex> corresponding to the temperature indicated by the termometer <xex>t</xex>, in order to correct for the effects of temperature on the gas. It is sensitive, and convenient for use at sea, but inferior in accuracy to the mercurial barometer.</note><br/
[<source>1913 Webster</source>]</p>

<p><ent>Symplectic</ent><br/
<hw>Sym*plec"tic</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[Gr. <?/ plaiting together, fr. <?/ to plait together.]</ety> <fld>(Anat.)</fld> <def>Plaiting or joining together; -- said of a bone next above the quadrate in the mandibular suspensorium of many fishes, which unites together the other bones of the suspensorium.</def> -- <def2><pos>n.</pos> <def>The symplectic bone.</def></def2><br/
[<source>1913 Webster</source>]</p>

<p><ent>Symploce</ent><br/
||<hw>Sym"plo*ce</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L., fr. Gr. <?/ an interweaving, fr. <?/ to twine together; <?/ + <?/ to twine.]</ety> <fld>(Rhet.)</fld> <def>The repetition of a word or phrase at the beginning and another at the end of successive clauses; as, <xex>Justice</xex> came down from heaven to view <xex>the earth</xex>; <xex>Justice</xex> returned to heaven, and left <xex>the earth</xex>.</def><br/
[<source>1913 Webster</source>]</p>

<p><-- p. 1463 --></p>

<p><ent>Sympode</ent><br/
<hw>Sym"pode</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Bot.)</fld> <def>A sympodium.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sympodial</ent><br/
<hw>Sym*po"di*al</hw> <pr>(?)</pr>, <pos>a.</pos> <fld>(Bot.)</fld> <def>Composed of superposed branches in such a way as to imitate a simple axis; <as>as, a <ex>sympodial</ex> stem</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sympodium</ent><br/
||<hw>Sym*po"di*um</hw> <pr>(?)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Sympodia</plw> <pr>(#)</pr>.</plu> <ety>[NL., fr. Gr. <grk>sy`n</grk> with + <?/, dim. of <?/, <?/, foot.]</ety> <fld>(Bot.)</fld> <def>An axis or stem produced by dichotomous branching in which one of the branches is regularly developed at the expense of the other, as in the grapevine.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Symposiac</ent><br/
<hw>Sym*po"si*ac</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>symposiacus</ets>, Gr. <?/.]</ety> <def>Of or pertaining to compotations and merrymaking; happening where company is drinking together; <as>as, <ex>symposiac</ex> meetings</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q><qex>Symposiac</qex> disputations amongst my acquaintance.</q> <rj><qau>Arbuthnot.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Symposiac</ent><br/
<hw>Sym*po"si*ac</hw>, <pos>n.</pos> <def>A conference or conversation of philosophers at a banquet; hence, any similar gathering.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Symposiarch</ent><br/
<hw>Sym*po"si*arch</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Gr. <?/, <?/; <?/ a symposium + <?/ to be first, to rule.]</ety> <fld>(Gr. Antiq.)</fld> <def>The master of a feast.</def>
<-- = M.C. --><br/
[<source>1913 Webster</source>]</p>

<p><ent>Symposiast</ent><br/
<hw>Sym*po"si*ast</hw> <pr>(?)</pr>, <pos>n.</pos> <def>One engaged with others at a banquet or merrymaking.</def>  <rj><au>Sydney Smith.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Symposion</ent><br/
||<hw>Sym*po"si*on</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[NL.]</ety> <def>A drinking together; a symposium.</def> <ldquo/Our <xex>symposion</xex> last night.<rdquo/  <rj><au>Sir W. Scott.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Symposium</ent><br/
<hw>Sym*po"si*um</hw> <pr>(?)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Symposia</plw> <pr>(#)</pr>.</plu> <ety>[L., fr. Gr. <grk>sympo`sion</grk> a drinking party, feast; <grk>sy`n</grk> with + <grk>po`sis</grk> a drinking. See <er>Syn-</er>, and cf. <er>Potable</er>.]</ety> <sn>1.</sn> <def>A drinking together; a merry feast.</def>  <rj><au>T. Warton.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A collection of short essays by different authors on a common topic; -- so called from the appellation given to the philosophical dialogue by the Greeks.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Symptom</ent><br/
<hw>Symp"tom</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F. <ets>sympt<ocir/me</ets>, Gr. <?/ anything that has befallen one, a chance, causality, symptom, fr. <?/ to fall together; <grk>sy`n</grk> with + <?/ to fall; akin to Skr. <ets>pat</ets> to fly, to fall. See <er>Syn-</er>, and cf. <er>Asymptote</er>, <er>Feather</er>.]</ety> <sn>1.</sn> <fld>(Med.)</fld> <def>Any affection which accompanies disease; a perceptible change in the body or its functions, which indicates disease, or the kind or phases of disease; <as>as, the causes of disease often lie beyond our sight, but we learn their nature by the <ex>symptoms</ex> exhibited</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Like the sick man, we are expiring with all sorts of good <qex>symptoms</qex>.</q> <rj><qau>Swift.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A sign or token; that which indicates the existence of something else; <as>as, corruption in elections is a <ex>symptom</ex> of the decay of public virtue</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><syn><b>Syn.</b> -- Mark; note; sign; token; indication.</syn><br/
[<source>1913 Webster</source>]</p>

<p><ent>Symptomatical</ent><br/
<ent>Symptomatic</ent><br/
<mhw>{ <hw>Symp`tom*at"ic</hw> <pr>(?)</pr>, <hw>Symp`tom*at"ic*al</hw> <pr>(?)</pr>, }</mhw> <pos>a.</pos> <ety>[Cf. F. <ets>symptomatique</ets>, Gr. <?/ causal.]</ety> <sn>1.</sn> <def>Of or pertaining to symptoms; happening in concurrence with something; being a symptom; indicating the existence of something else.</def><br/
[<source>1913 Webster</source>]</p>

<p><q><qex>Symptomatic</qex> of a shallow understanding and an unamiable temper.</q> <rj><qau>Macaulay.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>According to symptoms; <as>as, a <ex>symptomatical</ex> classification of diseases</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p>-- <wordforms><wf>Symp`tom*at"ic*al*ly</wf>, <pos>adv.</pos></wordforms><br/
[<source>1913 Webster</source>]</p>

<p><ent>Symptomatology</ent><br/
<hw>Symp`tom*a*tol"o*gy</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Gr. <?/, <?/, symptom + <ets>-logy</ets>: cf. F. <ets>symptomatologie</ets>.]</ety> <fld>(Med.)</fld> <def>The doctrine of symptoms; that part of the science of medicine which treats of the symptoms of diseases; semeiology.</def><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ It includes <xex>diagnosis</xex>, or the determination of the disease from its symptoms; and <xex>prognosis</xex>, or the determination of its probable course and event.</note><br/
[<source>1913 Webster</source>]</p>

<p><ent>Syn-</ent><br/
<hw>Syn-</hw> <pr>(?)</pr>. <ety>[Gr. <grk>sy`n</grk> with.]</ety> <def>A prefix meaning <xex>with</xex>, <xex>along with</xex>, <xex>together</xex>, <xex>at the same time</xex>. <xex>Syn-</xex> becomes <xex>sym-</xex> before <xex>p</xex>, <xex>b</xex>, and <xex>m</xex>, and <xex>syl-</xex> before <xex>l</xex>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Synacmy</ent><br/
<ent>Synacme</ent><br/
<mhw>{ <hw>Syn*ac"me</hw> <pr>(?)</pr>, <hw>Syn*ac"my</hw> <pr>(?)</pr>, }</mhw> <pos>n.</pos> <ety>[NL. <ets>synacme</ets>. See <er>Syn-</er>, and <er>Acme</er>.]</ety> <fld>(Bot.)</fld> <def>Same as <er>Synanthesis</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Syneresis</ent><br/
<ent>Synaeresis</ent><br/
<mhw>{ <hw>Syn*aer"e*sis</hw>, <hw>Syn*er"e*sis</hw>  }</mhw> <pr>(?)</pr>, <pos>n.</pos> <ety>[NL., fr. Gr. <?/ a taking or drawing together, fr. <?/ to take together; <grk>sy`n</grk> with + <?/ to take, to grasp. See <er>Syn-</er>, and <er>Heresy</er>.]</ety> <fld>(Gram.)</fld> <def>The union, or drawing together into one syllable, of two vowels that are ordinarily separated in syllabification; synecphonesis; -- the opposite of <xex>diaeresis</xex>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Synagogical</ent><br/
<hw>Syn`a*gog"ic*al</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Of or pertaining to a synagogue.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Synagogue</ent><br/
<hw>Syn"a*gogue</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F., from L. <ets>synagoga</ets>, Gr. <?/ a bringing together, an assembly, a synagogue, fr. <?/ to bring together; <grk>sy`n</grk> with + <?/ to lead. See <er>Syn-</er>, and <er>Agent</er>.]</ety> <sn>1.</sn> <def>A congregation or assembly of Jews met for the purpose of worship, or the performance of religious rites.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>The building or place appropriated to the religious worship of the Jews.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>The council of, probably, 120 members among the Jews, first appointed after the return from the Babylonish captivity; -- called also the <altname>Great Synagogue</altname>, and sometimes, though erroneously, the <altname>Sanhedrin</altname>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>A congregation in the early Christian church.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>My brethren, . . . if there come into your <qex>synagogue</qex> a man with a gold ring.</q> <rj><qau>James ii. 1,2 (Rev. Ver.).</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <def>Any assembly of men.</def> <mark>[Obs. or R.]</mark>  <rj><au>Milton.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Synalepha</ent><br/
<hw>Syn`a*le"pha</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[NL., fr. L. <ets>synaloepha</ets>, Gr. <?/, from <?/ to melt together; <grk>sy`n</grk> with + <?/ to besmear.]</ety> <fld>(Gram.)</fld> <def>A contraction of syllables by suppressing some vowel or diphthong at the end of a word, before another vowel or diphthong; <as>as, <ex>th' army</ex>, for <ex>the army</ex></as>.</def> <altsp>[Written also <asp>synaloepha</asp>.]</altsp><br/
[<source>1913 Webster</source>]</p>

<p><ent>Synallagmatic</ent><br/
<hw>Syn`al*lag*mat"ic</hw>, <pos>a.</pos> <ety>[Gr. <?/, from <?/ a mutual agreement, contract, fr. <?/ to exchange, negotiate with; <grk>sy`n</grk> with + <?/ to change.]</ety> <fld>(Law)</fld> <def>Imposing reciprocal obligations upon the parties; <as>as, a <ex>synallagmatic</ex> contract</as>.</def>  <rj><au>Bouvier.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Synallaxine</ent><br/
<hw>Syn`al*lax"ine</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[From Gr. <?/ to associate with.]</ety> <fld>(Zool.)</fld> <def>Having the outer and middle toes partially united; -- said of certain birds related to the creepers.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Synaloepha</ent><br/
||<hw>Syn`a*loe"pha</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L.]</ety> <def>Same as <er>Synalepha</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Synangium</ent><br/
||<hw>Syn*an"gi*um</hw> <pr>(?)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Synangia</plw> <pr>(#)</pr>.</plu> <ety>[NL., fr. Gr. <?/ + <?/ a hollow vessel.]</ety> <fld>(Anat.)</fld> <def>The divided part beyond the pylangium in the aortic trunk of the amphibian heart.</def> -- <wordforms><wf>Syn*an"gi*al</wf> <pr>(#)</pr>, <pos>a.</pos></wordforms><br/
[<source>1913 Webster</source>]</p>

<p><ent>Synantherous</ent><br/
<hw>Syn*an"ther*ous</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[Pref. <ets>syn-</ets> + anther.]</ety> <fld>(Bot.)</fld> <def>Having the stamens united by their anthers; <as>as, <ex>synantherous</ex> flowers</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Synanthesis</ent><br/
||<hw>Syn`an*the"sis</hw> <pr>(s<icr/n`<acr/n*th<emac/"s<icr/s)</pr>, <pos>n.</pos> <ety>[NL., fr. Gr. <grk>sy`n</grk> with + Gr. <grk>'a`nqhsis</grk> bloom.]</ety> <fld>(Bot.)</fld> <def>The simultaneous maturity of the anthers and stigmas of a blossom.</def>  <rj><au>Gray.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Synanthous</ent><br/
<hw>Syn*an"thous</hw> <pr>(s<icr/n*<acr/n"th<ucr/s)</pr>, <pos>a.</pos> <ety>[Pref. <ets>syn-</ets> + Gr. <grk>'a`nqos</grk> flower.]</ety> <fld>(Bot.)</fld> <def>Having flowers and leaves which appear at the same time; -- said of certain plants.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Synanthrose</ent><br/
<hw>Syn*an"throse"</hw> <pr>(s<icr/n*<acr/n"thr<omac/s)</pr>, <pos>n.</pos> <ety>[From NL. <ets>Synantherae</ets> the Compositae; Gr. <grk>sy`n</grk> with + <grk>'anqh`ros</grk> blooming.]</ety> <fld>(Chem.)</fld> <def>A variety of sugar, isomeric with sucrose, found in the tubers of the Jerusalem artichoke (<spn>Helianthus tuberosus</spn>), in the dahlia, and other Compositae.</def><-- ?? not in Merck I. --><br/
[<source>1913 Webster</source>]</p>

<p><ent>Synapta</ent><br/
||<hw>Syn*ap"ta</hw> <pr>(s<icr/n*<acr/p"t<adot/)</pr>, <pos>n.</pos> <ety>[NL., fr. Gr. <grk>synapto`s</grk> fastened together; <grk>sy`n</grk> with + <grk>'a`ptein</grk> to fasten.]</ety> <fld>(Zool.)</fld> <def>A genus of slender, transparent holothurians which have delicate calcareous anchors attached to the dermal plates. See <xex>Illustration</xex> in Appendix.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Synaptase</ent><br/
<hw>Syn*ap"tase</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Gr. <?/ fastened together + dias<ets>tase</ets>.]</ety> <fld>(Chem.)</fld> <def>A ferment resembling diastase, found in bitter almonds.  Cf. <er>Amygdalin</er>, and <er>Emulsin</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Synapticula</ent><br/
||<hw>Syn`ap*tic"u*la</hw> <pr>(?)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Synapticulae</plw> <pr>(#)</pr>.</plu> <ety>[NL., dim. from Gr. <?/ fastened together.]</ety> <fld>(Zool.)</fld> <def>One of numerous calcareous processes which extend between, and unite, the adjacent septa of certain corals, especially of the fungian corals.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Synarchy</ent><br/
<hw>Syn"ar*chy</hw> <pr>(s<icr/n"<adot/r*k<ycr/)</pr>, <pos>n.</pos> <ety>[Gr. <grk>synarchi`a</grk>, fr. <grk>syna`rchein</grk> to rule jointly with; <grk>sy`n</grk> with + <grk>'a`rchein</grk> to rule.]</ety> <def>Joint rule or sovereignty.</def> <mark>[R.]</mark>  <rj><au>Stackhouse.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Synartesis</ent><br/
<hw>Syn`ar*te"sis</hw> <pr>(s<icr/n`<aum/r*t<emac/"s<icr/s)</pr>, <pos>n.</pos> <ety>[NL., fr. Gr. <?/ a fastening together, fr. <?/ to fasten together.]</ety> <def>A fastening or knitting together; the state of being closely jointed; close union.</def> <mark>[R.]</mark>  <rj><au>Coleridge.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Synarthrodia</ent><br/
||<hw>Syn`ar*thro"di*a</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[NL.]</ety> <fld>(Anat.)</fld> <def>Synarthrosis.</def> -- <wordforms><wf>Syn`ar*thro"di*al</wf> <pr>(#)</pr>, <pos>a.</pos></wordforms>  <rj><au>Dunglison.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Synarthrosis</ent><br/
||<hw>Syn`ar*thro"sis</hw> <pr>(?)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Synarthroses</plw> <pr>(#)</pr>.</plu> <ety>[NL., fr. Gr. <?/ a being jointed together, fr. <?/ to link or joint together; <grk>sy`n</grk> with + <?/ a joint.]</ety> <fld>(Anat.)</fld> <def>Immovable articulation by close union, as in sutures. It sometimes includes symphysial articulations also. See the Note under <er>Articulation</er>, <pos>n.</pos>, 1.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Synastry</ent><br/
<hw>Syn"as*try</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Pref. <ets>syn-</ets> + Gr. <?/ a star.]</ety> <def>Concurrence of starry position or influence; hence, similarity of condition, fortune, etc., as prefigured by astrological calculation.</def> <mark>[R.]</mark>  <rj><au>Motley.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Synaxis</ent><br/
<hw>Syn*ax"is</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L., fr. Gr. <?/, fr. <?/ to bring together. See <er>Synagogue</er>.]</ety> <def>A congregation; also, formerly, the Lord's Supper.</def>  <rj><au>Jer. Taylor.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Syncarp</ent><br/
<hw>Syn"carp</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[NL. <ets>syncarpium</ets>. See <er>Syncarpous</er>.]</ety> <fld>(Bot.)</fld> <def>A kind of aggregate fruit in which the ovaries cohere in a solid mass, with a slender receptacle, as in the magnolia; also, a similar multiple fruit, as a mulberry.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Syncarpium</ent><br/
||<hw>Syn*car"pi*um</hw> <pr>(?)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Syncarpia</plw> <pr>(#)</pr>.</plu> <ety>[NL.]</ety> <fld>(Bot.)</fld> <def>Same as <er>Syncarp</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Syncarpous</ent><br/
<hw>Syn*car"pous</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[Pref. <ets>syn-</ets> + Gr. <?/ a fruit.]</ety> <fld>(Bot.)</fld> <def>Composed of several carpels consolidated into one ovary.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Syncategorematic</ent><br/
<hw>Syn*cat`e*gor`e*mat"ic</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[Gr. <?/; <grk>sy`n</grk> with + <?/ a predicate. See <er>Syn-</er>, and <er>Categorematic</er>.]</ety> <fld>(Logic)</fld> <def>Not capable of being used as a term by itself; -- said of words, as an adverb or preposition.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Synchondrosis</ent><br/
||<hw>Syn`chon*dro"sis</hw> <pr>(?)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Synchondroses</plw> <pr>(#)</pr>.</plu> <ety>[NL., fr. Gr. <?/; <grk>sy`n</grk> with + <?/ cartilage.]</ety> <fld>(Anat.)</fld> <def>An immovable articulation in which the union is formed by cartilage.</def> -- <wordforms><wf>Syn`chon*dro"si*al</wf>, <pos>a.</pos></wordforms><br/
[<source>1913 Webster</source>]</p>

<p><ent>Synchondrotomy</ent><br/
<hw>Syn`chon*drot"o*my</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Gr. <?/ union by cartilage + <?/ to cut.]</ety> <fld>(Surg.)</fld> <def>Symphyseotomy.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Synchoresis</ent><br/
<hw>Syn`cho*re"sis</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[NL., fr. Gr. <?/; <grk>sy`n</grk> with + <?/ a going.]</ety> <fld>(Rhet.)</fld> <def>A concession made for the purpose of retorting with greater force.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Synchronal</ent><br/
<hw>Syn"chro*nal</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[See <er>Synchronous</er>.]</ety> <def>Happening at, or belonging to, the same time; synchronous; simultaneous.</def>  <rj><au>Dr. H. More.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Synchronal</ent><br/
<hw>Syn"chro*nal</hw>, <pos>n.</pos> <def>A synchronal thing or event.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Synchronical</ent><br/
<hw>Syn*chron"ic*al</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[Cf. F. <ets>synchronique</ets>.]</ety> <def>Happening at the same time; synchronous.</def> <au>Boyle.</au> -- <wordforms><wf>Syn*chron"ic*al*ly</wf>, <pos>adv.</pos></wordforms><br/
[<source>1913 Webster</source>]</p>

<p><ent>Synchronism</ent><br/
<hw>Syn"chro*nism</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Gr. <?/, fr. <?/ to be contemporary with, from <?/ synchronous. See <er>Synchronous</er>.]</ety> <sn>1.</sn> <def>The concurrence of events in time; simultaneousness.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>The tabular arrangement of historical events and personages, according to their dates.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <fld>(Paint.)</fld> <def>A representation, in the same picture, of two or events which occured at different times.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Synchronistic</ent><br/
<hw>Syn`chro*nis"tic</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Of or pertaining to synchronism; arranged according to correspondence in time; <as>as, <ex>synchronistic</ex> tables</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Synchronization</ent><br/
<hw>Syn`chro*ni*za"tion</hw> <pr>(?)</pr>, <pos>n.</pos> <def>The act of synchronizing; concurrence of events in respect to time.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Synchronize</ent><br/
<hw>Syn"chro*nize</hw> <pr>(?)</pr>, <pos>v. i.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Synchronized</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Synchronizing</conjf> <pr>(?)</pr>.]</vmorph> <ety>[Gr. <?/.]</ety> <def>To agree in time; to be simultaneous.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The path of this great empire, through its arch of progress, <qex>synchronized</qex> with that of Christianity.</q> <rj><qau>De Quincey.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Synchronize</ent><br/
<hw>Syn"chro*nize</hw>, <pos>v. t.</pos> <sn>1.</sn> <def>To assign to the same date or period of time; <as>as, to <ex>synchronize</ex> two events of Greek and Roman history</as>.</def> <ldquo/Josephus <xex>synchronizes</xex> Nisan with the Egyptian Pharmus.<rdquo/  <rj><au>W. L. Bevan.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To cause to agree in time; <as>as, to <ex>synchronize</ex> the movements of different machines; to <ex>synchronize</ex> clocks.</as></def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Synchronology</ent><br/
<hw>Syn`chro*nol"o*gy</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Pref. <ets>syn-</ets> + Gr. <?/ time + <ets>-logy</ets>.]</ety> <def>Contemporaneous chronology.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Synchronous</ent><br/
<hw>Syn"chro*nous</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[Gr. <?/; <grk>sy`n</grk> with + <?/ time.  Cf. <er>Chronicle</er>.]</ety> <def>Happening at the same time; simultaneous.</def> -- <wordforms><wf>Syn"chro*nous*ly</wf>, <pos>adv.</pos></wordforms><br/
[<source>1913 Webster</source>]</p>

<p><ent>Synchrony</ent><br/
<hw>Syn"chro*ny</hw> <pr>(?)</pr>, <pos>n.</pos> <def>The concurrence of events in time; synchronism.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>Geological contemporaneity is the same as chronological <qex>synchrony</qex>.</q> <rj><qau>Huxley.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Synchysis</ent><br/
||<hw>Syn"chy*sis</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[NL., fr. Gr. <?/, fr. <?/ to confound; <grk>sy`n</grk> with + <?/ to pour.]</ety> <def>A derangement or confusion of any kind, as of words in a sentence, or of humors in the eye.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Sparkling synchysis</b></col> <fld>(Med.)</fld>, <cd>a condition in which the vitreous humor is softened and contains sparkling scales of cholesterin.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Synclastic</ent><br/
<hw>Syn*clas"tic</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[Pref. <ets>syn-</ets> + Gr. <grk>kla^n</grk> to break.]</ety> <fld>(Math. Physics)</fld> <def>Curved toward the same side in all directions; -- said of surfaces which in all directions around any point bend away from a tangent plane toward the same side, as the surface of a sphere; -- opposed to anticlastic.</def>  <rj><au>Sir W. Thomson.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Synclinal</ent><br/
<hw>Syn*cli"nal</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[Gr. <?/ to incline together; <grk>sy`n</grk> with + <?/ to incline.]</ety> <sn>1.</sn> <def>Inclined downward from opposite directions, so as to meet in a common point or line.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Geol.)</fld> <def>Formed by strata dipping toward a common line or plane; <as>as, a <ex>synclinal</ex> trough or valley; a <ex>synclinal</ex> fold; -- opposed to <ex>anticlinal</ex>.</as></def><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ A downward flexure in the case of folded rocks makes a <xex>synclinal</xex> axis, and the alternating upward flexure an <xex>anticlinal</xex> axis.</note><br/
[<source>1913 Webster</source>]</p>

<p><ent>Synclinal</ent><br/
<hw>Syn*cli"nal</hw>, <pos>n.</pos> <fld>(Geol.)</fld> <def>A synclinal fold.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Syncline</ent><br/
<hw>Syn*cline"</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Geol.)</fld> <def>A synclinal fold.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Synclinical</ent><br/
<hw>Syn*clin"ic*al</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Synclinal.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Synclinorium</ent><br/
||<hw>Syn`cli*no"ri*um</hw> <pr>(?)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Synclinoria</plw> <pr>(#)</pr>.</plu> <ety>[NL., fr. Gr. <?/ to lay together + <?/ mountain.]</ety> <fld>(Geol.)</fld> <def>A mountain range owing its origin to the progress of a geosynclinal, and ending in a catastrophe of displacement and upturning.</def>  <rj><au>Dana.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Syncopal</ent><br/
<hw>Syn"co*pal</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Of or pertaining to syncope; resembling syncope.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Syncopate</ent><br/
<hw>Syn"co*pate</hw> <pr>(?)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Syncopated</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Syncopating</conjf>.]</vmorph> <ety>[LL. <ets>syncopatus</ets>, p. p. of <ets>syncopare</ets> to syncopate, to swoon. See <er>Syncope</er>.]</ety> <sn>1.</sn> <fld>(Gram.)</fld> <def>To contract, as a word, by taking one or more letters or syllables from the middle; <as>as, <ldquo/Gloster<rdquo/ is a <ex>syncopated</ex> form of <ldquo/Gloucester</as>.<rdquo/</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Mus.)</fld> <def>To commence, as a tone, on an unaccented part of a measure, and continue it into the following accented part, so that the accent is driven back upon the weak part and the rhythm drags.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Syncopation</ent><br/
<hw>Syn`co*pa"tion</hw> <pr>(?)</pr>, <pos>n.</pos> <sn>1.</sn> <fld>(Gram.)</fld> <def>The act of syncopating; the contraction of a word by taking one or more letters or syllables from the middle; syncope.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Mus.)</fld> <def>The act of syncopating; a peculiar figure of rhythm, or rhythmical alteration, which consists in welding into one tone the second half of one beat with the first half of the beat which follows.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Syncope</ent><br/
<hw>Syn"co*pe</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>syncope</ets>, <ets>syncopa</ets>, Gr. <?/ a cutting up, a syncope; akin to <?/ to beat together, to cut up, cut short, weavy; <grk>sy`n</grk> with + <?/ to strike, cut.]</ety> <sn>1.</sn> <fld>(Gram.)</fld> <def>An elision or retrenchment of one or more letters or syllables from the middle of a word; as, <xex>ne'er</xex> for <xex>never</xex>, <xex>ev'ry</xex> for <xex>every</xex>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Mus.)</fld> <def>Same as <er>Syncopation</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <fld>(Med.)</fld> <def>A fainting, or swooning. See <er>Fainting</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>A pause or cessation; suspension.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>Revely, and dance, and show,<br/
Suffer a <qex>syncope</qex> and solemn pause.</q> <rj><qau>Cowper.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Syncopist</ent><br/
<hw>Syn"co*pist</hw> <pr>(?)</pr>, <pos>n.</pos> <def>One who syncopates.</def>  <rj><au>Addison.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Syncopize</ent><br/
<hw>Syn"co*pize</hw> <pr>(?)</pr>, <pos>v. t.</pos> <def>To syncopate.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Syncotyledonous</ent><br/
<hw>Syn*cot`y*led"on*ous</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[Pref. <ets>syn-</ets> + <ets>cotyledonous</ets>.]</ety> <fld>(Bot.)</fld> <def>Having united cotyledonous.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Syncretic</ent><br/
<hw>Syn*cret"ic</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Uniting and blending together different systems, as of philosophy, morals, or religion.</def>  <rj><au>Smart.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Syncretism</ent><br/
<hw>Syn"cre*tism</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Gr. <?/, fr. <?/ to make two parties join against a third: cf. F. <ets>syncr<eacute/tisme</ets>.]</ety> <sn>1.</sn> <def>Attempted union of principles or parties irreconcilably at variance with each other.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>He is plotting a carnal <qex>syncretism</qex>, and attempting the reconcilement of Christ and Belial.</q> <rj><qau>Baxter.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q><qex>Syncretism</qex> is opposed to eclecticism in philosophy.</q> <rj><qau>Krauth-Fleming.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Philol.)</fld> <def>The union or fusion into one of two or more originally different inflectional forms, as of two cases.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Syncretist</ent><br/
<hw>Syn"cre*tist</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Cf. F. <ets>syncr<eacute/tiste</ets>.]</ety> <def>One who attempts to unite principles or parties which are irreconcilably at variance;</def> specifically <fld>(Eccl. Hist.)</fld>, <def>an adherent of George Calixtus and other Germans of the seventeenth century, who sought to unite or reconcile the Protestant sects with each other and with the Roman Catholics, and thus occasioned a long and violent controversy in the Lutheran church.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Syncretistic</ent><br/
<hw>Syn`cre*tis"tic</hw> <pr>(?)</pr>, <pos>a.</pos> <sn>1.</sn> <def>Pertaining to, or characterized by, syncretism; <as>as, a <ex>syncretistic</ex> mixture of the service of Jehovah and the worship of idols</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Of or pertaining to Syncretists.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Syncrisis</ent><br/
||<hw>Syn"cri*sis</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[NL., fr. Gr. <?/ a comparison; <?/ together + <?/ to judge.]</ety> <fld>(Rhet.)</fld> <def>A figure of speech in which opposite things or persons are compared.</def>  <rj><au>Crabb.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><-- p. 1464 --></p>

<p><ent>Syncytium</ent><br/
||<hw>Syn*cy"ti*um</hw> <pr>(?)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Syncitia</plw> <pr>(#)</pr>.</plu> <ety>[NL., from Gr. <?/ together + <?/ a hollow vessel.]</ety> <sn>1.</sn> <fld>(Biol.)</fld> <def>Tissue in which the cell or partition walls are wholly wanting and the cell bodies fused together, so that the tissue consists of a continuous mass of protoplasm in which nuclei are imbedded, as in ordinary striped muscle.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Zool.)</fld> <def>The ectoderm of a sponge.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Syndactyle</ent><br/
<ent>Syndactyl</ent><br/
<mhw>{ <hw>Syn*dac"tyl</hw>, <hw>Syn*dac"tyle</hw> }</mhw> <pr>(?)</pr>, <pos>a.</pos>  <ety>[<ets>Syn-</ets> + Gr. <?/ finger, toe.]</ety> <fld>(Zool. & Med.)</fld> <def>Having two or more digits wholly or partly united. See <er>Syndactylism</er>.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Syndactyle</ent><br/
<hw>Syn*dac"tyle</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Pref. <ets>syn-</ets> + Gr. <?/ finger, toe: cf. F. <ets>syndactyle</ets>.]</ety> <fld>(Zool.)</fld> <def>Any bird having syndactilous feet.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Syndactylic</ent><br/
<hw>Syn*dac*tyl"ic</hw> <pr>(?)</pr>, <pos>a.</pos> <fld>(Zool.)</fld> <def>Syndactilous.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Syndactylous</ent><br/
<hw>Syn*dac"tyl*ous</hw> <pr>(?)</pr>, <pos>a.</pos> <fld>(Zool.)</fld> <def>Having the toes firmly united together for some distance, and without an intermediate web, as the kingfishers; gressorial.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Syndesmography</ent><br/
<hw>Syn`des*mog"ra*phy</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Gr. <?/ band, bond + <ets>-graphy</ets>.]</ety> <def>A description of the ligaments; syndesmology.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Syndesmology</ent><br/
<hw>Syn`des*mol"o*gy</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Gr. <?/ band, bond + <ets>-logy</ets>.]</ety> <def>That part of anatomy which treats of ligaments.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Syndesmosis</ent><br/
||<hw>Syn`des*mo"sis</hw> <pr>(?)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Syndesmoses</plw> <pr>(#)</pr>.</plu> <ety>[NL., fr. Gr. <?/ a bond; <?/ together + <?/ a bond, fr. <?/ to bind.]</ety> <fld>(Anat.)</fld> <def>An articulation formed by means of ligaments.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Syndetical</ent><br/
<ent>Syndetic</ent><br/
<mhw>{ <hw>Syn*det"ic</hw> <pr>(?)</pr>, <hw>Syn*det"ic*al</hw> <pr>(?)</pr>, }</mhw> <pos>a.</pos> <ety>[Gr. <?/, from <?/ to bind together; <grk>sy`n</grk> with + <?/ to bind; cf. <er>Asyndetic</er>.]</ety> <def>Connecting; conjunctive; <as>as, <ex>syndetic</ex> words or connectives; <ex>syndetic</ex> references in a dictionary.</as></def> -- <wordforms><wf>Syn*det"ic*al*ly</wf>, <pos>adv.</pos></wordforms><br/
[<source>1913 Webster</source>]</p>

<p><q>With the <qex>syndetic</qex> juxtaposition of distinct members, the article is not often repeated.</q> <rj><qau>C. J. Grece (Trans. Maetzner's Gram.).</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Syndic</ent><br/
<hw>Syn"dic</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>syndictus</ets>, Gr. <?/ helping in a court of justice, advocate; <grk>sy`n</grk> with + <?/ justice, akin to <?/ to show: cf. F. <ets>syndic</ets>. See <er>Teach</er>.]</ety> <sn>1.</sn> <def>An officer of government, invested with different powers in different countries; a magistrate.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Law)</fld> <def>An agent of a corporation, or of any body of men engaged in a business enterprise; an advocate or patron; an assignee.</def><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ In France, <xex>syndics</xex> are appointed by the creditors of a bankrupt to manage the property. Almost all the companies in Paris, the university, and the like, have their <xex>syndics</xex>. The university of Cambridge, Eng., has its <xex>syndics</xex>, who are chosen from the senate to transact special business, such as the regulation of fees, the framing of laws, etc.</note><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <fld>(Civil Law)</fld> <def>One appointed to manage an estate, essentially as a trustee, under English law.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Syndical</ent><br/
<hw>Syn"dic*al</hw> <pr>(?)</pr>, <pos>a.</pos> <sn>1.</sn> <def>Consisting of, or pert. to, a syndic.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><sn>2.</sn>  <def>Of or pertaining to, or of the nature of, syndicalism.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Syndicalism</ent><br/
<hw>Syn"dic*al*ism</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F. <ets>syndicalisme</ets>.]</ety> <def>The theory, plan, or practice of trade-union action (originally as advocated and practiced by the French Conf<eacute/d<eacute/ration G<eacute/n<eacute/rale du Travail) which aims to abolish the present political and social system by means of the general strike (as distinguished from the local or sectional strike) and direct action of whatever kind (as distinguished from action which takes effect only through the medium of political action) -- <xex>direct action</xex> including any kind of action that is directly effective, whether it be a simple strike, a peaceful public demonstration, sabotage, or revolutionary violence. By the general strike and direct action syndicalism aims to establish a social system in which the means and processes of production are in the control of local organizations of workers, who are manage them for the common good.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Syndicalist</ent><br/
<hw>Syn"dic*al*ist</hw>, <pos>n.</pos> <def>One who advocates or practices syndicalism.</def> -- <wordforms><wf>Syn`dic*al*is"tic</wf> <pr>(#)</pr>, <pos>a.</pos></wordforms><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Syndicate</ent><br/
<hw>Syn"di*cate</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Cf. F. <ets>syndicat</ets>, LL. <ets>syndicatus</ets>.]</ety> <sn>1.</sn> <def>The office or jurisdiction of a syndic; a council, or body of syndics.</def>  <rj><au>Bp. Burnet.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>An association of persons officially authorized to undertake some duty or to negotiate some business; also, an association of persons who combine to carry out, on their own account, a financial or industrial project; <as>as, a <ex>syndicate</ex> of bankers formed to take up and dispose of an entire issue of government bonds</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>A more or less organized association of criminals controlling some aspects of criminal activity, in a specific area or country-wide; -- used loosely as a synonym for <altname>organized crime</altname> or the <altname>mafia</altname>.</def><br/
[<source>PJC</source>]</p>

<p><sn>4.</sn> <fld>(Journalism)</fld> <def>a commercial organization that purchases various journalistic items, such as articles, columns, or comic strips, from their individual creators, and resells them to newspapers or other periodicals for simultaneous publication over a wide area.</def><br/
[<source>PJC</source>]</p>

<p><ent>Syndicate</ent><br/
<hw>Syn"di*cate</hw> <pr>(?)</pr>, <pos>v. t.</pos> <ety>[LL. <ets>syndicatus</ets>, p. p. of <ets>syndicare</ets> to censure.]</ety> <def>To judge; to censure.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Syndicate</ent><br/
<hw>Syn"di*cate</hw> <pr>(?)</pr>, <pos>v. t.</pos>  <vmorph>[<pos>imp. & p. p.</pos> <conjf>syndicated</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>syndicating</conjf> <pr>(?)</pr>.]</vmorph> <sn>1.</sn> <def>To combine or form into, or manage as, a syndicate.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><sn>2.</sn> <def>To acquire or control for or by, or to subject to the management of, a syndicate; <as>as, <ex>syndicated</ex> newspapers</as>.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><sn>3.</sn> <fld>(Journalism)</fld> <def>to purchase various journalistic items, such as articles, columns, or comic strips, from their individual creators, and resell them to numerous periodicals for simultaneous publication over a wide area; <as>a <ex>syndicated</ex> columnist</as>.</def><br/
[<source>PJC</source>]</p>

<p><ent>Syndicate</ent><br/
<hw>Syn"di*cate</hw>, <pos>v. i.</pos> <def>To unite to form a syndicate.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Syndication</ent><br/
<hw>Syn`di*ca"tion</hw> <pr>(?)</pr>, <pos>n.</pos> <def>Act or process of syndicating or forming a syndicate.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Syndrome</ent><br/
||<hw>Syn"dro*me</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[NL., from Gr. <?/; <grk>sy`n</grk> with + <?/ a course, a running.]</ety> <sn>1.</sn> <def>Concurrence.</def> <mark>[R.]</mark>  <rj><au>Glanvill.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Med.)</fld> <def>A group of symptoms occurring together that are characteristic and indicative of some underlying cause, such as a disease.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>generally, a pattern of characteristics or behaviors occurring together, that are sufficiently common and distinctive to indicate that they are due to a known cause.</def><br/
[<source>PJC</source>]</p>

<p><ent>Syndyasmian</ent><br/
<hw>Syn`dy*as"mi*an</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[Gr. <grk>syndyasmo`s</grk> a pairing, fr. <grk>syndya`zein</grk> to pair.]</ety> <def>Pertaining to the state of pairing together sexually; -- said of animals during periods of procreation and while rearing their offspring.</def>  <rj><au>Morgan.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Syne</ent><br/
<hw>Syne</hw> <pr>(?)</pr>, <pos>adv.</pos> <ety>[See <er>Since</er>.]</ety> <sn>1.</sn> <def>Afterwards; since; ago.</def> <mark>[Obs. or Scot.]</mark>  <rj><au>R. of Brunne.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Late, -- as opposed to <xex>soon</xex>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>[Each rogue] shall be discovered either soon or <qex>syne</qex>.</q> <rj><qau>W. Hamilton (Life of Wallace).</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Syne</ent><br/
<hw>Syne</hw>, <pos>conj.</pos> <def>Since; seeing.</def> <mark>[Scot.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Synecdoche</ent><br/
<hw>Syn*ec"do*che</hw> <pr>(s<icr/n*<ecr/k"d<osl/*k<esl/)</pr>, <pos>n.</pos> <ety>[L. <ets>synecdoche</ets>, Gr. <grk>synekdochh`</grk>, fr.  to receive jointly; <grk>sy`n</grk> with + <?/ to receive; <?/ out + <?/ to receive.]</ety> <fld>(Rhet.)</fld> <def>A figure or trope by which a part of a thing is put for the whole (as, fifty <xex>sail</xex> for fifty <xex>ships</xex>), or the whole for a part (as, the smiling <xex>year</xex> for <xex>spring</xex>), the species for the genus (as, <xex>cutthroat</xex> for <xex>assassin</xex>), the genus for the species (as, a <xex>creature</xex> for a <xex>man</xex>), the name of the material for the thing made, etc.</def>  <rj><au>Bain.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Synecdochical</ent><br/
<hw>Syn`ec*doch"ic*al</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Expressed by synecdoche; implying a synecdoche.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Isis is used for Themesis by a <qex>synecdochical</qex> kind of speech, or by a poetical liberty, in using one for another.</q> <rj><qau>Drayton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Synecdochically</ent><br/
<hw>Syn`ec*doch"ic*al*ly</hw>, <pos>adv.</pos> <def>By synecdoche.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Synechia</ent><br/
||<hw>Syn*e"chi*a</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[NL., fr. Gr. <?/, fr. <?/ to hold together; <grk>sy`n</grk> with + <?/ to hold.]</ety> <fld>(Med.)</fld> <def>A disease of the eye, in which the iris adheres to the cornea or to the capsule of the crystalline lens.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Synecphonesis</ent><br/
||<hw>Syn*ec`pho*ne"sis</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[NL., fr. Gr. <?/, fr. <?/ to utter together.]</ety> <fld>(Gram.)</fld> <def>A contraction of two syllables into one; synizesis.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Synedral</ent><br/
<hw>Syn*e"dral</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[Gr. <?/ sitting with; <grk>sy`n</grk> with + <grk>"e`dra</grk> seat.]</ety> <fld>(Bot.)</fld> <def>Growing on the angles of a stem, as the leaves in some species of Selaginella.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Synentognathi</ent><br/
||<hw>Syn`en*tog"na*thi</hw> <pr>(?)</pr>, <pos>n. pl.</pos> <ety>[NL., fr. Gr. <grk>sy`n</grk> with + <grk>'ento`s</grk> within + <grk>gna`qos</grk> jaw.]</ety> <fld>(Zool.)</fld> <def>An order of fishes, resembling the Physoclisti, without spines in the dorsal, anal, and ventral fins. It includes the true flying fishes.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Synepy</ent><br/
<hw>Syn"e*py</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Gr. <?/; <grk>sy`n</grk> with + <?/ a word.]</ety> <fld>(Rhet.)</fld> <def>The interjunction, or joining, of words in uttering the clauses of sentences.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Syneresis</ent><br/
<hw>Syn*er"e*sis</hw> <pr>(?)</pr>, <pos>n.</pos> <def>Same as <er>Synaeresis</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Synergetic</ent><br/
<hw>Syn`er*get"ic</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[Gr. <?/, fr. <?/ to work together; <grk>sy`n</grk> with + <grk>'e`rgon</grk> work.]</ety> <def>Working together; cooperating; <as>as, <ex>synergetic</ex> muscles</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Synergism</ent><br/
<hw>Syn"er*gism</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[See <er>Synergetic</er>.]</ety> <fld>(Theol.)</fld> <def>The doctrine or theory, attributed to Melanchthon, that in the regeneration of a human soul there is a cooperation, or joint agency, on the part both of God and of man.</def><br/
[<source>1913 Webster</source>]</p>

<p><-- 2. Same as synergy, 2. --><br/
[<source>1913 Webster</source>]</p>

<p><ent>Synergist</ent><br/
<hw>Syn"er*gist</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Cf. F. <ets>synergiste</ets>.]</ety> <sn>1.</sn> <def>One who holds the doctrine of synergism.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Med.)</fld> <def>A remedy which has an action similar to that of another remedy, and hence increases the efficiency of that remedy when combined with it.</def><br/
[<source>1913 Webster</source>]</p>

<p><-- 3. (Biochemistry) A chemical compound which exhibits a synergistic effect on some biochemical or physiological action, in combination with another compound.  [A supertype of def. 2.] 
--><br/
[<source>1913 Webster</source>]</p>

<p><ent>Synergistic</ent><br/
<hw>Syn`er*gis"tic</hw> <pr>(?)</pr>, <pos>a.</pos> <sn>1.</sn> <def>Of or pertaining to synergism.</def> <ldquo/A <xex>synergistic</xex> view of regeneration.<rdquo/  <rj><au>Shedd.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Cooperating; synergetic.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Synergy</ent><br/
<hw>Syn"er*gy</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Gr. <?/. See <er>Synergetic</er>.]</ety> <sn>2.</sn> <def>Combined action</def>; especially <fld>(Med.)</fld>, <def>the combined healthy action of every organ of a particular system; <as>as, the digestive <ex>synergy</ex></as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>An effect of the interaction of the actions of two agents such that the result of the combined action is greater than expected as a simple additive combination of the two agents acting separately; -- also called <altname>synergism.</altname>.  Opposite to <ant>antagonism</ant>.</def><br/
[<source>PJC</source>]</p>

<p><ent>Synesis</ent><br/
<hw>Syn"e*sis</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Gr. <?/ intelligence.]</ety> <fld>(Gram.)</fld> <def>A construction in which adherence to some element in the sense causes a departure from strict syntax, as in <ldquo/Philip went down to Samaria and preached Christ unto <xex>them</xex>.<rdquo/</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Syngenesia</ent><br/
||<hw>Syn`ge*ne"si*a</hw> <pr>(?)</pr>, <pos>n. pl.</pos> <ety>[NL., fr. Gr. <grk>sy`n</grk> with, together + <?/ generation, birth.]</ety> <fld>(Bot.)</fld> <def>A Linnaean class of plants in which the stamens are united by the anthers.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Syngenesious</ent><br/
<ent>Syngenesian</ent><br/
<mhw>{ <hw>Syn`ge*ne"sian</hw> <pr>(?)</pr>, <hw>Syn`ge*ne"sious</hw> <pr>(?)</pr>, }</mhw> <pos>a.</pos> <fld>(Bot.)</fld> <def>Having the stamens united by the anthers; of or pertaining to the Syngenesia.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Syngenesis</ent><br/
<hw>Syn*gen"e*sis</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Pref. <ets>syn-</ets> + <ets>genesis</ets>.]</ety> <fld>(Biol.)</fld> <def>A theory of generation in which each germ is supposed to contain the germs of all subsequent generations; -- the opposite of <contr>epigenesis</contr>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Syngnathi</ent><br/
||<hw>Syng"na*thi</hw> <pr>(?)</pr>, <pos>n. pl.</pos> <ety>[NL., fr. Gr. <grk>sy`n</grk> with + <?/ jaw.]</ety> <fld>(Zool.)</fld> <def>A suborder of lophobranch fishes which have an elongated snout and lack the ventral and first dorsal fins.  The pipefishes and sea horses are examples.</def> -- <wordforms><wf>Syng"na*thous</wf> <pr>(#)</pr>, <pos>a.</pos></wordforms><br/
[<source>1913 Webster</source>]</p>

<p><ent>Syngraph</ent><br/
<hw>Syn"graph</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>syngrapha</ets>, Gr. <?/; <grk>sy`n</grk> with + <?/ to write.]</ety> <fld>(Law)</fld> <def>A writing signed by both or all the parties to a contract or bond.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Synizesis</ent><br/
||<hw>Syn`i*ze"sis</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L., fr. Gr. <?/, fr. <?/ to sit together; <grk>sy`n</grk> with + <?/ to sit.]</ety> <sn>1.</sn> <fld>(Med.)</fld> <def>An obliteration of the pupil of the eye.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Gram.)</fld> <def>A contraction of two syllables into one; synecphonesis.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Synneorosis</ent><br/
||<hw>Syn`neo*ro"sis</hw> <pr>(?)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Synneuroses</plw> <pr>(#)</pr>.</plu> <ety>[NL., fr. Gr. <?/; <grk>sy`n</grk> with + <?/ a sinew, ligament.]</ety> <fld>(Anat.)</fld> <def>Syndesmosis.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Synocha</ent><br/
||<hw>Syn"o*cha</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[NL., fr. Gr. <?/ a holding together. See <er>Synechia</er>.]</ety> <fld>(Med.)</fld> <def>See <er>Synochus</er>.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Synochal</ent><br/
<hw>Syn"o*chal</hw> <pr>(?)</pr>, <pos>a.</pos> <fld>(Med.)</fld> <def>Of or pertaining to synocha; like synocha.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Synochus</ent><br/
||<hw>Syn"o*chus</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[NL., from Gr. <?/ joined together.]</ety> <fld>(Med.)</fld> <def>A continuous fever.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ <xex>Synocha</xex> and <xex>synochus</xex> were used as epithets of two distinct types of fever, but in different senses at different periods. The same disease is placed under synocha by one author, under synochus by another.</note>  <rj><au>Quain.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Synocil</ent><br/
<hw>Syn"o*cil</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Pref. <ets>syn-</ets> + <ets>cil</ets>ium.]</ety> <fld>(Zool.)</fld> <def>A sense organ found in certain sponges. It consists of several filaments, each of which arises from a single cell.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Synod</ent><br/
<hw>Syn"od</hw> <pr>(s<icr/n"<ucr/d)</pr>, <pos>n.</pos> <ety>[L. <ets>synodus</ets>, Gr. <grk>sy`nodos</grk> a meeting; <grk>sy`n</grk> with + <grk>"odo`s</grk> a way; cf. AS. <ets>sino<edh/</ets>, <ets>seno<edh/</ets>, F. <ets>synode</ets>, both from the Latin.]</ety> <sn>1.</sn> <fld>(Eccl. Hist.)</fld> <def>An ecclesiastic council or meeting to consult on church matters.</def><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ <xex>Synods</xex> are of four kinds: 1. <xex>General</xex>, or <xex>ecumenical</xex>, which are composed of bishops from different nations; -- commonly called <xex>general council</xex>. 2. <xex>National</xex>, composed of bishops of one nation only. 3. <xex>Provincial</xex>, in which the bishops of only one province meet; -- called also <xex>convocations</xex>. 4. <xex>Diocesan</xex>, a synod in which the bishop of the diocese or his representative presides.
    Among Presbyterians, a <xex>synod</xex> is composed of several adjoining presbyteries. The members are the ministers and a ruling elder from each parish.</note><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>An assembly or council having civil authority; a legislative body.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>It hath in solemn <qex>synods</qex> been decreed,<br/
Both by the Syracusians and ourselves,<br/
To admit no traffic to our adverse towns.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Parent of gods and men, propitious Jove!<br/
And you, bright <qex>synod</qex> of the powers above.</q> <rj><qau>Dryden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <fld>(Astron.)</fld> <def>A conjunction of two or more of the heavenly bodies.</def> <mark>[R.]</mark>  <rj><au>Milton.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Synodal</ent><br/
<hw>Syn"od*al</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>synodalis</ets>: cf. F. <ets>synodal</ets>.]</ety> <def>Synodical.</def>  <rj><au>Milton.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Synodal</ent><br/
<hw>Syn"od*al</hw>, <pos>n.</pos> <sn>1.</sn> <fld>(Ch. of Eng.)</fld> <def>A tribute in money formerly paid to the bishop or archdeacon, at the time of his Easter visitation, by every parish priest, now made to the ecclesiastical commissioners; a procuration.</def><br/
[<source>1913 Webster</source>]</p>

<p><q><qex>Synodals</qex> are due, of common right, to the bishop only.</q> <rj><qau>Gibson.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A constitution made in a provincial or diocesan synod.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Synodical</ent><br/
<ent>Synodic</ent><br/
<mhw>{ <hw>Syn*od"ic</hw> <pr>(?)</pr>, <hw>Syn*od"ic*al</hw> <pr>(?)</pr>, }</mhw> <pos>a.</pos> <ety>[L. <ets>synodicus</ets>, Gr. <?/: cf. F. <ets>synodique</ets>.]</ety> <sn>1.</sn> <fld>(Eccl.)</fld> <def>Of or pertaining to a synod; transacted in, or authorized by, a synod; <as>as, <ex>synodical</ex> proceedings or forms</as>.</def> <ldquo/A <xex>synodical</xex> epistle.<rdquo/  <rj><au>Bp. Stillingfleet.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Astron.)</fld> <def>Pertaining to conjunction, especially to the period between two successive conjunctions; extending from one conjunction, as of the moon or a planet with the sun, to the next; <as>as, a <ex>synodical</ex> month (see <ex>Lunar month</ex>, under <er>Month</er>); the <ex>synodical</ex> revolution of the moon or a planet.</as></def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Synodically</ent><br/
<hw>Syn*od"ic*al*ly</hw>, <pos>adv.</pos> <def>In a synodical manner; in a synod; by the authority of a synod.</def> <ldquo/<xex>Synodically</xex> agreed upon.<rdquo/  <rj><au>R. Nelson.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Synodist</ent><br/
<hw>Syn"od*ist</hw> <pr>(?)</pr>, <pos>n.</pos> <def>An adherent to a synod.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>These <qex>synodists</qex> thought fit in Latin as yet to veil their decrees from vulgar eyes.</q> <rj><qau>Fuller.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Synoecious</ent><br/
<hw>Syn*oe"cious</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[Pref. <ets>syn-</ets> + Gr. <?/ house.]</ety> <fld>(Bot.)</fld> <def>Having stamens and pistil in the same head, or, in mosses, having antheridia and archegonia on the same receptacle.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Synomocy</ent><br/
<hw>Syn*om"o*cy</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Gr. <?/, fr. <?/ to swear with or together; <grk>sy`n</grk> with + <?/ to swear.]</ety> <def>Sworn brotherhood; a society in ancient Greece nearly resembling a modern political club.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Synonym</ent><br/
<hw>Syn"o*nym</hw> <pr>(s<icr/n"<osl/*n<icr/m)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Synonyms</plw> <pr>(s<icr/n"<osl/*n<icr/mz)</pr>.</plu> <ety>[F. <ets>synonyme</ets>, L. <ets>synonyma</ets>, pl. of <ets>synonymum</ets>, Gr. <grk>synw`nymon</grk>. See <er>Synonymous</er>.]</ety> <sn>1.</sn> <def>One of two or more words (commonly words of the same language) which are equivalents of each other; one of two or more words which have very nearly the same signification, and therefore may often be used interchangeably. See under <er>Synonymous</er>.</def> <altsp>[Written also <asp>synonyme</asp>.]</altsp><br/
[<source>1913 Webster</source>]</p>

<p><q>All languages tend to clear themselves of <qex>synonyms</qex> as intellectual culture advances, the superfluous words being taken up and appropriated by new shades and combinations of thought evolved in the progress of society.</q> <rj><qau>De Quincey.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>His name has thus become, throughout all civilized countries, a <qex>synonym</qex> for probity and philanthropy.</q> <rj><qau>Macaulay.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>In popular literary acceptation, and as employed in special dictionaries of such words, <qex>synonyms</qex> are words sufficiently alike in general signification to be liable to be confounded, but yet so different in special definition as to require to be distinguished.</q>  <rj><au>G. P. Marsh.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>An incorrect or incorrectly applied scientific name, as a new name applied to a species or genus already properly named, or a specific name preoccupied by that of another species of the same genus; -- so used in the system of nomenclature (which see) in which the correct scientific names of certain natural groups (usually genera, species, and subspecies) are regarded as determined by priority.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><sn>3.</sn>  <def>One of two or more words corresponding in meaning but of different languages; a heteronym.</def> <mark>[Rare]</mark><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Synonyma</ent><br/
||<hw>Syn*on"y*ma</hw> <pr>(s<icr/n*<ocr/n"<icr/*m<adot/)</pr>, <pos>n. pl.</pos> <ety>[L.]</ety> <def>Synonyms.</def> <mark>[Obs.]</mark>  <rj><au>Fuller.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Synonymal</ent><br/
<hw>Syn*on"y*mal</hw> <pr>(s<icr/n*<ocr/n"<icr/*m<ait/l)</pr>, <pos>a.</pos> <def>Synonymous.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Synonymally</ent><br/
<hw>Syn*on"y*mal*ly</hw>, <pos>adv.</pos> <def>Synonymously.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Synonyme</ent><br/
<hw>Syn"o*nyme</hw> <pr>(?)</pr>, <pos>n.</pos> <def>Same as <er>Synonym</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Synonymic</ent><br/
<hw>Syn`o*nym"ic</hw> <pr>(s<icr/n`<ocr/*n<icr/m"<icr/k)</pr>, <pos>n.</pos> <ety>[Cf. G. <ets>synonymik</ets>. See <er>Synonymous</er>.]</ety> <fld>(Gram.)</fld> <def>The science, or the scientific treatment, of synonymous words.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Synonymical</ent><br/
<ent>Synonymic</ent><br/
<mhw>{ <hw>Syn`o*nym"ic</hw> <pr>(?)</pr>, <hw>Syn`o*nym"ic*al</hw> <pr>(?)</pr>, }</mhw> <pos>a.</pos> <def>Of or pertaining to synonyms, or synonymic; synonymous.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Synonymicon</ent><br/
||<hw>Syn`o*nym"i*con</hw> <pr>(s<icr/n`<ocr/*n<icr/m"<icr/*k<ocr/n)</pr>, <pos>n.</pos> <ety>[NL.]</ety> <def>A dictionary of synonyms.</def>  <rj><au>C. J. Smith.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Synonymist</ent><br/
<hw>Syn*on"y*mist</hw> <pr>(s<icr/n*<ocr/n"<icr/*m<icr/st)</pr>, <pos>n.</pos> <ety>[Cf. F. <ets>synonymiste</ets>.]</ety> <def>One who collects or explains synonyms.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Synonymize</ent><br/
<hw>Syn*on"y*mize</hw> <pr>(?)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Synonymized</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Synonymizing</conjf> <pr>(?)</pr>.]</vmorph> <def>To express by a synonym or synonyms; to give the synonym or synonyms corresponding to.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>This word <ldquo/fortis<rdquo/ we may <qex>synonymize</qex> after all these fashions: stout, hardy, valiant, doughty, courageous, adventurous, brave, bold, daring, intrepid.</q> <rj><qau>Camden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Synonymous</ent><br/
<hw>Syn*on"y*mous</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[Gr. <?/; <grk>sy`n</grk> with, together + <?/, <?/, name. See <er>Syn-</er>, and <er>Name</er>.]</ety> <def>Having the character of a synonym; expressing the same thing; conveying the same, or approximately the same, idea.</def> -- <wordforms><wf>Syn*on"y*mous*ly</wf>, <pos>adv.</pos></wordforms><br/
[<source>1913 Webster</source>]</p>

<p><q>These words consist of two propositions, which are not distinct in sense, but one and the same thing variously expressed; for wisdom and understanding are <qex>synonymous</qex> words here.</q> <rj><qau>Tillotson.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><syn><b>Syn.</b> -- Identical; interchangeable. -- <er>Synonymous</er>, <er>Identical</er>. If no words are <xex>synonymous</xex> except those which are <xex>identical</xex> in use and meaning, so that the one can in all cases be substituted for the other, we have scarcely ten such words in our language. But the term more properly denotes that the words in question approach so near to each other, that, in many or most cases, they can be used interchangeably. 1. Words may thus coincide in <xex>certain</xex> connections, and so be interchanged, when they can not be interchanged in other connections; thus we may speak either <xex>strength</xex> of mind or of <xex>force</xex> of mind, but we say the <xex>force</xex> (not <xex>strength</xex>) of gravitation. 2. Two words may differ slightly, but this difference may be unimportant to the speaker's object, so that he may freely interchange them; thus it makes but little difference, in most cases, whether we speak of a man's having <xex>secured</xex> his object or having <xex>attained</xex> his object. For these and other causes we have numerous words which may, in many cases or connections, be used interchangeably, and these are properly called <xex>synonyms</xex>. <xex>Synonymous</xex> words <ldquo/are words which, with great and essential resemblances of meaning, have, at the same time, small, subordinate, and partial differences, -- these differences being such as either originally and on the ground of their etymology inhered in them; or differences which they have by usage acquired in the eyes of all; or such as, though nearly latent now, they are capable of receiving at the hands of wise and discreet masters of the tongue. <xex>Synonyms</xex> are words of like significance in the main, but with a certain unlikeness as well.<rdquo/ <au>Trench.</au></syn><br/
[<source>1913 Webster</source>]</p>

<p><ent>Synonymy</ent><br/
<hw>Syn*on"y*my</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>synonymia</ets>, Gr. <?/ a synonym: cf. F. <ets>synonymie</ets>.]</ety> <sn>1.</sn> <def>The quality of being synonymous; sameness of meaning.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A system of synonyms.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <fld>(Rhet.)</fld> <def>A figure by which synonymous words are used to amplify a discourse.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Synopsis</ent><br/
<hw>Syn*op"sis</hw> <pr>(?)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Synopses</plw> <pr>(#)</pr>.</plu> <ety>[L., from Gr. <?/; <grk>sy`n</grk> with, together + <?/ a sight, view, from the root seen in E. <ets>optic</ets>.]</ety> <def>A general view, or a collection of heads or parts so arranged as to exhibit a general view of the whole; an abstract or summary of a discourse; a syllabus; a conspectus.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>That the reader may see in one view the exactness of the method, as well as force of the argument, I shall here draw up a short <qex>synopsis</qex> of this epistle.</q> <rj><qau>Bp. Warburton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><syn><b>Syn.</b> -- Abridgment; compendium; epitome; abstract; summary; syllabus; conspectus. See <er>Abridgment</er>.</syn><br/
[<source>1913 Webster</source>]</p>

<p><ent>Synoptical</ent><br/
<ent>Synoptic</ent><br/
<mhw>{ <hw>Syn*op"tic</hw> <pr>(?)</pr>, <hw>Syn*op"tic*al</hw> <pr>(?)</pr>, }</mhw> <pos>a.</pos> <ety>[Gr. <?/: cf. F. <ets>synoptique</ets>. See <er>Synopsis</er>.]</ety> <def>Affording a general view of the whole, or of the principal parts of a thing; <as>as, a <ex>synoptic</ex> table; a <ex>synoptical</ex> statement of an argument</as>.</def> <ldquo/The <xex>synoptic</xex> Gospels.<rdquo/ <au>Alford.</au> -- <wordforms><wf>Syn*op"tic*al*ly</wf>, <pos>adv.</pos></wordforms><br/
[<source>1913 Webster</source>]</p>

<p><ent>Synoptic</ent><br/
<hw>Syn*op"tic</hw>, <pos>n.</pos> <def>One of the first three Gospels of the New Testament. See <er>Synoptist</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Synoptist</ent><br/
<hw>Syn*op"tist</hw> <pr>(?)</pr>, <pos>n.</pos> <def>Any one of the authors of the three synoptic Gospels, which give a history of our Lord's life and ministry, in distinction from the writer of John's Gospel, which gives a fuller record of his teachings.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Synosteology</ent><br/
<hw>Syn*os`te*ol"o*gy</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Pref. <ets>syn-</ets> + Gr. <?/ bone + <ets>-logy</ets>.]</ety> <def>That part of anatomy which treats of joints; arthrology.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Synosteosis</ent><br/
||<hw>Syn*os`te*o"sis</hw> <pr>(?)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Synosteoses</plw> <pr>(#)</pr>.</plu> <ety>[NL., fr. Gr. <grk>sy`n</grk> with + <?/ bone.]</ety> <fld>(Anat.)</fld> <def>Union by means of bone; the complete closing up and obliteration of sutures.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Synostosis</ent><br/
||<hw>Syn`os*to"sis</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[NL.]</ety> <def>Same as <er>Synosteosis</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Synovia</ent><br/
||<hw>Syn*o"vi*a</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[NL., perhaps fr. Gr. <grk>sy`n</grk> with + L. <ets>ovum</ets> egg: cf. F. <ets>synovie</ets>.]</ety> <fld>(Anat.)</fld> <def>A transparent, viscid, lubricating fluid which contains mucin and secreted by synovial membranes; synovial fluid.</def><br/
[<source>1913 Webster</source>]</p>

<p><-- p. 1465 --></p>

<p><ent>Synovial</ent><br/
<hw>Syn*o"vi*al</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[Cf. F. <ets>synovial</ets>.]</ety> <fld>(Anat.)</fld> <def>Of or pertaining to synovia; secreting synovia.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Synovial capsule</b></col>, <cd>a closed sac of synovial membrane situated between the articular surfaces at diarthrodial joints.</cd> -- <col><b>Synovial fluid</b></col>, <cd>synovia.</cd> -- <col><b>Synovial membrane</b></col>, <cd>the dense and very smooth connective tissue membrane which secretes synovia and surrounds synovial capsules and other synovial cavities.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Synovitis</ent><br/
||<hw>Syn`o*vi"tis</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[NL. See <er>Synovia</er>, <er>-itis</er>.]</ety> <fld>(Med.)</fld> <def>Inflammation of the synovial membrane.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Synpelmous</ent><br/
<hw>Syn*pel"mous</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[Pref. <ets>syn-</ets> + <?/ the sole of the foot.]</ety> <fld>(Zool.)</fld> <def>Having the two main flexor tendons of the toes blended together.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Synsepalous</ent><br/
<hw>Syn*sep"al*ous</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[Pref. <ets>syn-</ets> + <ets>sepal</ets>.]</ety> <fld>(Bot.)</fld> <def>Having united sepals; gamosepalous.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Syntactical</ent><br/
<ent>Syntactic</ent><br/
<mhw>{ <hw>Syn*tac"tic</hw> <pr>(?)</pr>, <hw>Syn*tac"tic*al</hw> <pr>(?)</pr>, }</mhw> <pos>a.</pos> <ety>[Cf. G. <?/ putting together. See <er>Syntax</er>.]</ety> <def>Of or pertaining to syntax; according to the rules of syntax, or construction.</def> -- <wordforms><wf>Syn*tac"tic*al*ly</wf>, <pos>adv.</pos></wordforms><br/
[<source>1913 Webster</source>]</p>

<p><ent>Syntax</ent><br/
<hw>Syn"tax</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>syntaxis</ets>, Gr. <?/, fr. <?/ to put together in order; <grk>sy`n</grk> with + <?/ to put in order; cf. F. <ets>syntaxe</ets>. See <er>Syn-</er>, and <er>Tactics</er>.]</ety> <sn>1.</sn> <def>Connected system or order; union of things; a number of things jointed together; organism.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>They owe no other dependence to the first than what is common to the whole <qex>syntax</qex> of beings.</q> <rj><qau>Glanvill.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>That part of grammar which treats of the construction of sentences; the due arrangement of words in sentences in their necessary relations, according to established usage in any language.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Syntaxis</ent><br/
||<hw>Syn*tax"is</hw> <pr>(?)</pr>, <pos>n.</pos> <def>Syntax.</def> <mark>[R.]</mark>  <rj><au>B. Jonson.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Synteresis</ent><br/
<hw>Syn`te*re"sis</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[NL., fr. Gr. <?/ preservation, fr. <?/ to preserve; <grk>sy`n</grk> with + <?/ to guard.]</ety> <sn>1.</sn> <fld>(Med.)</fld> <def>Prophylaxis.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Metaph.)</fld> <def>Conscience viewed as the internal repository of the laws of duty.</def>  <rj><au>Whewell.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Synteretic</ent><br/
<hw>Syn`te*ret"ic</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[Gr. <?/.]</ety> <fld>(Med.)</fld> <def>Preserving health; prophylactic.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Synteretics</ent><br/
<hw>Syn`te*ret"ics</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Med.)</fld> <def>That department of medicine which relates to the preservation of health; prophylaxis.</def> <mark>[Obs.]</mark><-- = hygeine? --><br/
[<source>1913 Webster</source>]</p>

<p><ent>Synthermal</ent><br/
<hw>Syn*ther"mal</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[Pref. <ets>syn-</ets> + <ets>thermal</ets>.]</ety> <def>Having the same degree of heat.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Synthesis</ent><br/
<hw>Syn"the*sis</hw> <pr>(?)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Syntheses</plw> <pr>(#)</pr>.</plu> <ety>[L., a mixture, properly, a putting together, Gr. <?/, fr. <?/ to place or put together; <grk>sy`n</grk> with + <?/ to place. See <er>Thesis</er>.]</ety> <sn>1.</sn> <def>Composition, or the putting of two or more things together, as in compounding medicines.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Chem.)</fld> <def>The art or process of making a compound by putting the ingredients together, as contrasted with <xex>analysis</xex>; thus, water is made by <xex>synthesis</xex> from hydrogen and oxygen; hence, specifically, the building up of complex compounds by special reactions, whereby their component radicals are so grouped that the resulting substances are identical in every respect with the natural articles when such occur; thus, artificial alcohol, urea, indigo blue, alizarin, etc., are made by <xex>synthesis</xex>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <fld>(Logic)</fld> <def>The combination of separate elements of thought into a whole, as of simple into complex conceptions, species into genera, individual propositions into systems; -- the opposite of <ant>analysis</ant>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Analysis and <qex>synthesis</qex>, though commonly treated as two different methods, are, if properly understood, only the two necessary parts of the same method. Each is the relative and correlative of the other.</q> <rj><qau>Sir W. Hamilton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Synthesist</ent><br/
<hw>Syn"the*sist</hw> <pr>(?)</pr>, <pos>n.</pos> <def>One who employs synthesis, or who follows synthetic methods.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Synthesize</ent><br/
<hw>Syn"the*size</hw> <pr>(?)</pr>, <pos>v. t.</pos> <sn>1.</sn> <def>To combine by synthesis; to unite.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To produce by synthesis; <as>as, to <ex>synthesize</ex> albumin</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Synthetical</ent><br/
<ent>Synthetic</ent><br/
<mhw>{ <hw>Syn*thet"ic</hw> <pr>(?)</pr>, <hw>Syn*thet"ic*al</hw> <pr>(?)</pr>, }</mhw> <pos>a.</pos> <ety>[Gr. <?/: cf. F. <ets>synth<eacute/tique</ets>.]</ety> <sn>1.</sn> <def>Of or pertaining to synthesis; consisting in synthesis or composition; <as>as, the <ex>synthetic</ex> method of reasoning, as opposed to <ex>analytical</ex></as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Philosophers hasten too much from the analytic to the <qex>synthetic</qex> method; that is, they draw general conclusions from too small a number of particular observations and experiments.</q> <rj><qau>Bolingbroke.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Chem.)</fld> <def>Artificial.  Cf. <er>Synthesis</er>, 2.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <fld>(Zool.)</fld> <def>Comprising within itself structural or other characters which are usually found only in two or more diverse groups; -- said of species, genera, and higher groups. See the Note under <er>Comprehensive</er>, 3.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><mcol><col><b>Synthetic language</b></col>, <it>or</it> <col><b>Synthetical language</b></col></mcol>, <cd>an inflectional language, or one characterized by grammatical endings; -- opposed to <contr>analytic language</contr>.</cd> <au>R. Morris.</au></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Synthetically</ent><br/
<hw>Syn*thet"ic*al*ly</hw>, <pos>adv.</pos> <def>In a synthetic manner.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Synthetize</ent><br/
<hw>Syn"the*tize</hw> <pr>(?)</pr>, <pos>v. t.</pos> <ety>[Cf. Gr. <?/.]</ety> <def>To combine; to unite in regular structure.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Syntomy</ent><br/
<hw>Syn"to*my</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Gr. <?/, fr. <?/ to cut short; <grk>sy`n</grk> with + <?/ to cut.]</ety> <def>Brevity; conciseness.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Syntonic</ent><br/
<hw>Syn*ton"ic</hw> <pr>(?)</pr>, <pos>a.</pos> <fld>(Physics)</fld> <def>Of or pert. to syntony; specif., designating, or pert. to, a system of wireless telegraphy in which the transmitting and receiving apparatus are in syntony with, and only with, one another.</def> -- <wordforms><wf>Syn*ton"ic*al</wf> <pr>(#)</pr>, <pos>a.</pos> -- <wf>Syn*ton"ic*al*ly</wf>, <pos>adv.</pos></wordforms><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Syntonin</ent><br/
<hw>Syn"to*nin</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Cf. Gr. <?/ stretched tight, intense.]</ety> <fld>(Physiol. Chem.)</fld> <def>A proteid substance (acid albumin) formed from the albuminous matter of muscle by the action of dilute acids; -- formerly called <altname>musculin</altname>. See <cref>Acid albumin</cref>, under <er>Albumin</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Syntonize</ent><br/
<hw>Syn"to*nize</hw> <pr>(?)</pr>, <pos>v. t.</pos>  <vmorph>[<pos>imp. & p. p.</pos> <conjf>-nized</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>-nizing</conjf> <pr>(?)</pr>.]</vmorph> <ety>[See <er>Syntony</er>.]</ety> <fld>(Physics)</fld> <def>To adjust or devise so as to emit or respond to electric oscillations of a certain wave length; to tune; specif., to put (two or more instruments or systems of wireless telegraphy) in syntony with each other.</def> -- <wordforms><wf>Syn`to*ni*za"tion</wf> <pr>(#)</pr>, <pos>n.</pos></wordforms><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Syntonizer</ent><br/
<hw>Syn"to*niz`er</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Physics)</fld> <def>One that syntonizes; specif., a device consisting essentially of a variable inductance coil and condenser with a pair of adjustable spark balls, for attuning the time periods of antennae in wireless telegraphy (called also <altname><col><b>syntonizing coil</b></col></altname>).</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Syntony</ent><br/
<hw>Syn"to*ny</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Cf. Gr. <?/ agreement. See <er>Syn-</er>; <er>Tone</er>.]</ety> <fld>(Physics)</fld> <def>State of being adjusted to a certain wave length; agreement or tuning between the time period of an apparatus emitting electric oscillations and that of a receiving apparatus, esp. in wireless telegraphy.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Syphering</ent><br/
<hw>Sy"pher*ing</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Etymol. uncertain.]</ety> <fld>(Carp.)</fld> <def>The lapping of chamfered edges of planks to make a smooth surface, as for a bulkhead.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Syphilide</ent><br/
<hw>Syph"i*lide</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F.]</ety> <fld>(Med.)</fld> <def>A cutaneous eruption due to syphilis.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Syphilis</ent><br/
<hw>Syph"i*lis</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[NL., fr. <ets>Syphilus</ets>, the name of a shepherd in the Latin poem of Fracastoro, <ldquo/<ets>Syphilus</ets>, sive Morbus Gallicus,<rdquo/ which was published in 1530; Gr. <?/ hog, swine + <?/ dear, loving. The term was introduced into nosology by Sauvages.]</ety> <fld>(Med.)</fld> <def>The pox, or venereal disease; a chronic, specific, infectious disease, usually communicated by sexual intercourse or by hereditary transmission, and occurring in three stages known as <xex>primary</xex>, <xex>secondary</xex>, and <xex>tertiary syphilis</xex>. See under <er>Primary</er>, <er>Secondary</er>, and <er>Tertiary</er>.</def><-- a bacterial infection caused by <spn>Treponema pallidum</spn>.  Usu. tretable with penicillin or other beta-lactam antibiotics. --><br/
[<source>1913 Webster</source>]</p>

<p><ent>Syphilitic</ent><br/
<hw>Syph`i*lit"ic</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[Cf. F. <ets>syphilitique</ets>.]</ety> <fld>(Med.)</fld> <def>Of or pertaining to syphilis; of the nature of syphilis; affected with syphilis.</def> -- <def2><pos>n.</pos> <def>A syphilitic patient.</def></def2><br/
[<source>1913 Webster</source>]</p>

<p><ent>Syphilitically</ent><br/
<hw>Syph`i*lit"ic*al*ly</hw> <pr>(?)</pr>, <pos>adv.</pos> <fld>(Med.)</fld> <def>In a syphilitic manner; with venereal disease.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Syphilization</ent><br/
<hw>Syph`i*li*za"tion</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Med.)</fld> <def>Inoculation with the syphilitic virus, especially when employed as a preventive measure, like vaccination.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Syphilize</ent><br/
<hw>Syph"i*lize</hw> <pr>(?)</pr>, <pos>v. t.</pos> <fld>(Med.)</fld> <def>To inoculate with syphilis.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Syphiloderm</ent><br/
<hw>Syph"i*lo*derm</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[See <er>Syphilis</er>, and <er>Derm</er>.]</ety> <fld>(Med.)</fld> <def>A cutaneous affection due to syphilis.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Syphilodermatous</ent><br/
<hw>Syph`i*lo*der"ma*tous</hw> <pr>(?)</pr>, <pos>a.</pos> <fld>(Med.)</fld> <def>Of or pertaining to the cutaneous manifestations of syphilis.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Syphiloid</ent><br/
<hw>Syph"i*loid</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[<ets>Syphil</ets>is + <ets>-oid</ets>.]</ety> <fld>(Med.)</fld> <def>Resembling syphilis.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Syphilologist</ent><br/
<hw>Syph`i*lol"o*gist</hw> <pr>(?)</pr>, <pos>n.</pos> <def>One skilled in syphilology.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Syphilology</ent><br/
<hw>Syph`i*lol"o*gy</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[<ets>Syphil</ets>is + <ets>-logy</ets>.]</ety> <def>That branch of medicine which treats of syphilis.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Syphon</ent><br/
<hw>Sy"phon</hw> <pr>(?)</pr>, <pos>n.</pos> <def>See <er>Syphon</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Syracuse</ent><br/
<hw>Syr"a*cuse</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A red wine of Italy.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Syren</ent><br/
<hw>Sy"ren</hw> <pr>(?)</pr>, <pos>n.</pos> <def>See <er>Siren</er>.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Syriac</ent><br/
<hw>Syr"i*ac</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>Syriacus</ets>, from <ets>Syria</ets>: cf. F. <ets>syriaque</ets>.]</ety> <def>Of or pertaining to Syria, or its language; <as>as, the <ex>Syriac</ex> version of the Pentateuch</as>.</def> -- <def2><pos>n.</pos> <def>The language of Syria; especially, the ancient language of that country.</def></def2><br/
[<source>1913 Webster</source>]</p>

<p><ent>Syriacism</ent><br/
<hw>Syr"i*a*cism</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A Syrian idiom; a Syrianism.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Syrian</ent><br/
<hw>Syr"i*an</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>Syrius</ets>: cf. F. <ets>Syrien</ets>.]</ety> <def>Of or pertaining to Syria; Syriac.</def> -- <def2><pos>n.</pos> <def>A native of Syria.</def></def2><br/
[<source>1913 Webster</source>]</p>

<p><ent>Syrianism</ent><br/
<hw>Syr"i*an*ism</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A Syrian idiom, or a peculiarity of the Syrian language; a Syriacism.</def>  <rj><au>Paley.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Syriasm</ent><br/
<hw>Syr"i*asm</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A Syrian idiom; a Syrianism; a Syriacism.</def>  <rj><au>M. Stuart.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>The Scripture Greek is observed to be full of <qex>Syriasms</qex> and Hebraisms.</q> <rj><qau>Bp. Warburton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Syringa</ent><br/
<hw>Sy*rin"ga</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[NL., fr. Gr. <?/, <?/, a shepherd's pipe, tube.  Cf. <er>Syringe</er>.]</ety> <fld>(Bot.)</fld> <sd>(a)</sd> <def>A genus of plants; the lilac.</def> <sd>(b)</sd> <def>The mock orange; -- popularly so called because its stems were formerly used as pipestems.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Syringe</ent><br/
<hw>Syr"inge</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F. <ets>seringue</ets> (cf. Pr. <ets>siringua</ets>, Sp. <ets>jeringa</ets>, It. <ets>sciringa</ets>, <ets>scilinga</ets>), fg. Gr. <?/, <?/, a pipe or tube; cf. Skr. <ets>svar</ets> to sound, and E. <ets>swarum</ets>.  Cf. <er>Syringa</er>.]</ety> <def>A kind of small hand-pump for throwing a stream of liquid, or for purposes of aspiration. It consists of a small cylindrical barrel and piston, or a bulb of soft elastic material, with or without valves, and with a nozzle which is sometimes at the end of a flexible tube; -- used for injecting animal bodies, cleansing wounds, etc.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Garden syringe</b></col>. <cd>See <er>Garden</er>.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Syringe</ent><br/
<hw>Syr"inge</hw>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Syringed</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Syringing</conjf> <pr>(?)</pr>.]</vmorph> <sn>1.</sn> <def>To inject by means of a syringe; <as>as, to <ex>syringe</ex> warm water into a vein</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To wash and clean by injection from a syringe.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Syringeal</ent><br/
<hw>Sy*rin"ge*al</hw> <pr>(?)</pr>, <pos>a.</pos> <fld>(Anat.)</fld> <def>Of or pertaining to the syrinx; <as>as, the <ex>syringeal</ex> muscle</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Syringin</ent><br/
<hw>Sy*rin"gin</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Chem.)</fld> <def>A glucoside found in the bark of the lilac (<gen>Syringa</gen>) and extracted as a white crystalline substance; -- formerly called also <altname>lilacin</altname>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Syringocoele</ent><br/
||<hw>Sy*rin"go*coele</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[<ets>Syrinx</ets> + Gr. <?/ hollow.]</ety> <fld>(Anat.)</fld> <def>The central canal of the spinal cord.</def>  <rj><au>B. G. Wilder.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Syringotome</ent><br/
<hw>Sy*rin"go*tome</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Cf. F. <ets>syringotome</ets>. See <er>Syringotomy</er>.]</ety> <fld>(Surg. & Anat.)</fld> <def>A small blunt-pointed bistoury, -- used in syringotomy.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Syringotomy</ent><br/
<hw>Syr`in*got"o*my</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Gr. <?/ a tube, a hollow sore + <?/ to cut: cf. F. <ets>syringotomie</ets>.]</ety> <fld>(Surg.)</fld> <def>The operation of cutting for anal fistula.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Syrinx</ent><br/
<hw>Syr"inx</hw> <pr>(?)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Syringes</plw> <pr>(#)</pr>.</plu> <ety>[NL., from Gr. <?/ a pipe.]</ety> <sn>1.</sn> <fld>(Mus.)</fld> <def>A wind instrument made of reeds tied together; -- called also <altname>pandean pipes</altname>.</def><-- pipes of Pan --><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Anat.)</fld> <def>The lower larynx in birds.</def><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ In birds there are two laringes, an upper or true, but voiceless, larynx in the usual position behind the tongue, and a lower one, at or near the junction of the trachea and bronchi, which is the true organ of the voice.</note><br/
[<source>1913 Webster</source>]</p>

<p><ent>Syrma</ent><br/
||<hw>Syr"ma</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L., fr. Gr. <?/, fr. <?/ to drag.]</ety> <fld>(Class. Antiq.)</fld> <def>A long dress, trailing on the floor, worn by tragic actors in Greek and Roman theaters.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Syrphian</ent><br/
<hw>Syr"phi*an</hw> <pr>(?)</pr>, <pos>a.</pos> <fld>(Zool.)</fld> <def>Of or pertaining to the syrphus flies.</def> -- <def2><pos>n.</pos> <fld>(Zool.)</fld> <def>A syrphus fly.</def></def2><br/
[<source>1913 Webster</source>]</p>

<p><ent>Syrphus fly</ent><br/
<hw>Syr"phus fly`</hw> <pr>(?)</pr>. <ety>[NL. <ets>Syrphus</ets>, the generic name, fr. Gr. <?/, <?/, a kind of winged insect.]</ety> <fld>(Zool.)</fld> <def>Any one of numerous species of dipterous flies of the genus <gen>Syrphus</gen> and allied genera. They are usually bright-colored, with yellow bands, and hover around plants. The larvae feed upon plant lice, and are, therefore, very beneficial to agriculture.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Syrt</ent><br/
<hw>Syrt</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>syrtis</ets> a sand bank in the sea, Gr. <?/: cf. F. <ets>syrte</ets>.]</ety> <def>A quicksand; a bog.</def> <mark>[R.]</mark>  <rj><au>Young.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Syrtic</ent><br/
<hw>Syr"tic</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Of or pertaining to a syrt; resembling syrt, or quicksand.</def> <mark>[R.]</mark>  <rj><au>Ed. Rev.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Syrtis</ent><br/
||<hw>Syr"tis</hw> <pr>(?)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Syrtes</plw> <pr>(#)</pr>.</plu> <ety>[See <er>Syrt</er>.]</ety> <def>A quicksand.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Quenched in a boggy <qex>syrtis</qex>, neither sea<br/
Nor good dry land.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Syrupy</ent><br/
<ent>Syrup</ent><br/
<mhw>{ <hw>Syr"up</hw> <pr>(?)</pr>, <pos>n.</pos>, <hw>Syr"up*y</hw> <pr>(?)</pr>, }</mhw> <pos>a.</pos> <ety>[See <er>Sirup</er>.]</ety> <def>Same as <er>Sirup</er>, <er>Sirupy</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Syssarcosis</ent><br/
||<hw>Sys`sar*co"sis</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[NL., fr. Gr. <?/, fr. <?/ to unite by flesh, to cover over with flesh; <grk>sy`n</grk> with + <?/, <?/, flesh.]</ety> <fld>(Anat.)</fld> <def>The junction of bones by intervening muscles.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Systaltic</ent><br/
<hw>Sys*tal"tic</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>systalticus</ets> drawing together, Gr. <?/, from <?/ to draw together.  Cf. <er>Sustaltic</er>, <er>Systole</er>.]</ety> <fld>(Physiol.)</fld> <def>Capable of, or taking place by, alternate contraction and dilatation; <as>as, the <ex>systaltic</ex> action of the heart</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Systasis</ent><br/
||<hw>Sys"ta*sis</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[NL., fr. Gr. <?/, fr. <?/ to stand together. See under <er>System</er>.]</ety> <def>A political union, confederation, or league.</def> <mark>[R.]</mark>  <rj><au>Burke.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>System</ent><br/
<hw>Sys"tem</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>systema</ets>, Gr. <?/, fr. <?/ to place together; <grk>sy`n</grk> with + <?/ to place: cf. F. <ets>syst<egrave/me</ets>. See <er>Stand</er>.]</ety> <sn>1.</sn> <def>An assemblage of objects arranged in regular subordination, or after some distinct method, usually logical or scientific; a complete whole of objects related by some common law, principle, or end; a complete exhibition of essential principles or facts, arranged in a rational dependence or connection; a regular union of principles or parts forming one entire thing; <as>as, a <ex>system</ex> of philosophy; a <ex>system</ex> of government; a <ex>system</ex> of divinity; a <ex>system</ex> of botany or chemistry; a military <ex>system</ex>; the solar <ex>system</ex>.</as></def>
<-- Specifically, a computer system. --><br/
[<source>1913 Webster</source>]</p>

<p><q>The best way to learn any science, is to begin with a regular <qex>system</qex>, or a short and plain scheme of that science well drawn up into a narrow compass.</q> <rj><qau>I. Watts.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Hence, the whole scheme of created things regarded as forming one complete plan of whole; the universe.</def> <ldquo/The great <xex>system</xex> of the world.<rdquo/  <rj><au>Boyle.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>Regular method or order; formal arrangement; plan; <as>as, to have a <ex>system</ex> in one's business</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <fld>(Mus.)</fld> <def>The collection of staves which form a full score. See <er>Score</er>, <pos>n.</pos></def><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <fld>(Biol.)</fld> <def>An assemblage of parts or organs, either in animal or plant, essential to the performance of some particular function or functions which as a rule are of greater complexity than those manifested by a single organ; <as>as, the capillary <ex>system</ex>, the muscular <ex>system</ex>, the digestive <ex>system</ex>, etc.</as>; hence, the whole body as a functional unity.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>6.</sn> <fld>(Zool.)</fld> <def>One of the stellate or irregular clusters of intimately united zooids which are imbedded in, or scattered over, the surface of the common tissue of many compound ascidians.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><mcol><col><b>Block system</b></col>, <col><b>Conservative system</b></col></mcol>, <cd>etc. See under <er>Block</er>, <er>Conservative</er>, etc.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Systematical</ent><br/
<ent>Systematic</ent><br/
<mhw>{ <hw>Sys`tem*at"ic</hw> <pr>(?)</pr>, <hw>Sys`tem*at"ic*al</hw> <pr>(?)</pr>, }</mhw> <pos>a.</pos> <altsp>[Gr. <?/: cf. F. <ets>syst<eacute/matique</ets>.]</altsp> <sn>1.</sn> <def>Of or pertaining to system; consisting in system; methodical; formed with regular connection and adaptation or subordination of parts to each other, and to the design of the whole; <as>as, a <ex>systematic</ex> arrangement of plants or animals; a <ex>systematic</ex> course of study.</as></def><br/
[<source>1913 Webster</source>]</p>

<p><q>Now we deal much in essays, and unreasonably despise <qex>systematical</qex> learning; whereas our fathers had a just value for regularity and systems.</q> <rj><qau>I. Watts.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>A representation of phenomena, in order to answer the purposes of science, must be <qex>systematic</qex>.</q> <rj><qau>Whewell.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Proceeding according to system, or regular method; <as>as, a <ex>systematic</ex> writer; <ex>systematic</ex> benevolence.</as></def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>Pertaining to the system of the world; cosmical.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>These ends may be called cosmical, or <qex>systematical</qex>.</q> <rj><qau>Boyle.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <fld>(Med.)</fld> <def>Affecting successively the different parts of the system or set of nervous fibres; <as>as, <ex>systematic</ex> degeneration</as>.</def>
<-- affecting the whole body, as contrasted with local. --><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Systematic theology</b></col>. <cd>See under <er>Theology</er>.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Systematically</ent><br/
<hw>Sys`tem*at"ic*al*ly</hw>, <pos>adv.</pos> <def>In a systematic manner; methodically.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Systematism</ent><br/
<hw>Sys"tem*a*tism</hw> <pr>(?)</pr>, <pos>n.</pos> <def>The reduction of facts or principles to a system.</def>  <rj><au>Dunglison.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Systematist</ent><br/
<hw>Sys"tem*a*tist</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Cf. F. <ets>syst<eacute/matiste</ets>.]</ety> <sn>1.</sn> <def>One who forms a system, or reduces to system.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>One who adheres to a system.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Systematization</ent><br/
<hw>Sys`tem*a*ti*za"tion</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Cf. F. <ets>syst<eacute/matization</ets>.]</ety> <def>The act or operation of systematizing.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Systematize</ent><br/
<hw>Sys"tem*a*tize</hw> <pr>(?)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Systematized</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Systematizing</conjf> <pr>(?)</pr>.]</vmorph> <ety>[Cf. F. <ets>syst<eacute/matiser</ets>.  Cf. <er>Systemize</er>.]</ety> <def>To reduce to system or regular method; to arrange methodically; to methodize; <as>as, to <ex>systematize</ex> a collection of plants or minerals; to <ex>systematize</ex> one's work; to <ex>systematize</ex> one's ideas</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Diseases were healed, and buildings erected, before medicine and architecture were <qex>systematized</qex> into arts.</q> <rj><qau>Harris.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Systematizer</ent><br/
<hw>Sys"tem*a*ti`zer</hw> <pr>(?)</pr>, <pos>n.</pos> <def>One who systematizes.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Aristotle may be called the <qex>systematizer</qex> of his master's doctrines.</q> <rj><qau>Harris.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Systematology</ent><br/
<hw>Sys`tem*a*tol"o*gy</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Gr. <?/, <?/, system + <ets>-logy</ets>.]</ety> <def>The doctrine of, or a treatise upon, systems.</def>  <rj><au>Dunglison.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Systemic</ent><br/
<hw>Sys*tem"ic</hw> <pr>(?)</pr>, <pos>a.</pos> <sn>1.</sn> <def>Of or relating to a system; common to a system; <as>as, the <ex>systemic</ex> circulation of the blood</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Anat. & Physiol.)</fld> <def>Of or pertaining to the general system, or the body as a whole; <as>as, <ex>systemic</ex> death, in distinction from <xex>local</xex> death; <ex>systemic</ex> circulation, in distinction from <xex>pulmonic</xex> circulation; <ex>systemic</ex> diseases</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Systemic death</b></col>. <cd>See the Note under <er>Death</er>, <pos>n.</pos>, 1.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Systemization</ent><br/
<hw>Sys`tem*i*za"tion</hw> <pr>(?)</pr>, <pos>n.</pos> <def>The act or process of systematizing; systematization.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Systemize</ent><br/
<hw>Sys"tem*ize</hw> <pr>(s<icr/s"t<ecr/m*<imac/z)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Systemized</conjf> <pr>(s<icr/s"t<ecr/m*<imac/zd)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Systemizing</conjf> <pr>(s<icr/s"t<ecr/m*<imac/`z<icr/ng)</pr>.]</vmorph> <ety>[Cf. <er>Systematize</er>.]</ety> <def>To reduce to system; to systematize.</def><br/
[<source>1913 Webster</source>]</p>

<p><-- p. 1466 --></p>

<p><ent>Systemizer</ent><br/
<hw>Sys"tem*i`zer</hw> <pr>(s<icr/s"t<ecr/m*<imac/`z<etil/r)</pr>, <pos>n.</pos> <def>One who systemizes, or reduces to system; a systematizer.</def></p>

<p><ent>Systemless</ent><br/
<hw>Sys"tem*less</hw>, <pos>a.</pos> <sn>1.</sn> <def>Being without system.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Nat. Hist.)</fld> <def>Not agreeing with some artificial system of classification.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <fld>(Biol.)</fld> <def>Not having any of the distinct systems or types of structure, as the radiate, articulate, etc., characteristic of organic nature; <as>as, all unicellular organisms are <ex>systemless</ex></as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Systole</ent><br/
<hw>Sys"to*le</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[NL., fr. Gr. <?/, fr. <?/ to contract; <grk>sy`n</grk> with + <?/ to set, place.]</ety> <sn>1.</sn> <fld>(Gram.)</fld> <def>The shortening of the long syllable.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Physiol. & Biol.)</fld> <def>The contraction of the heart and arteries by which the blood is forced onward and the circulation kept up; also, the contraction of a rhythmically pulsating contractile vacuole; -- correlative to <contr>diastole</contr>.</def> -- <wordforms><wf>Sys*tol"ic</wf> <pr>(#)</pr>, <pos>a.</pos></wordforms><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Systolic</ent><br/
<hw>Sys*tol"ic</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Of or pertaining to systole, or contraction; contracting; esp., relating to the systole of the heart; <as>as, <ex>systolic</ex> murmur</as>.</def>  <rj><au>Dunglison.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Systyle</ent><br/
<hw>Sys"tyle</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>systylos</ets>, Gr. <grk>sy`n</grk> with columns standing close; <grk>sy`n</grk> with + <?/ a column: cf. F. <ets>systyle</ets>.]</ety> <fld>(Arch.)</fld> <def>Having a space equal to two diameters or four modules between two columns; -- said of a portico or building. See <er>Intercolumniation</er>.</def> -- <def2><pos>n.</pos> <def>A systyle temple or other edifice.</def></def2><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sythe</ent><br/
<ent>Syth</ent><br/
<mhw>{ <hw>Syth</hw> <pr>(?)</pr>, <hw>Sythe</hw> <pr>(?)</pr>, }</mhw> <pos>prep., adv., conj. & n.</pos> <def>See <er>Sith</er>, <er>Sithe</er>.</def> <mark>[Obs.]</mark>  <rj><au>Chaucer.</au>  <au>Piers Plowman.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Sythe</ent><br/
<hw>Sythe</hw> <pr>(?)</pr>, <pos>n.</pos> <def>Scythe.</def> <mark>[Obs. or R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Syzygial</ent><br/
<hw>Sy*zyg"i*al</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Pertaining to a syzygy.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Syzygy</ent><br/
<hw>Syz"y*gy</hw> <pr>(s<icr/z"<icr/*j<ycr/)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Syzygies</plw> <pr>(-j<icr/z)</pr>.</plu> <ety>[L. <ets>syzygia</ets> a joining together, conjunction, Gr. <grk>syzygi`a</grk>; <grk>sy`n</grk> with + <grk>zeygny`nai</grk> to join, <grk>zygo`n</grk> yoke: cf. F. <ets>syzygie</ets>. See <er>Yoke</er>, <pos>n.</pos>]</ety> <sn>1.</sn> <fld>(Astron.)</fld> <def>The point of an orbit, as of the moon or a planet, at which it is in conjunction or opposition; -- commonly used in the plural.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Gr. & L. Pros.)</fld> <def>The coupling together of different feet; <as>as, in Greek verse, an iambic <ex>syzygy</ex></as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <fld>(Zool.)</fld> <sd>(a)</sd> <def>Any one of the segments of an arm of a crinoid composed of two joints so closely united that the line of union is obliterated on the outer, though visible on the inner, side.</def> <sd>(b)</sd> <def>The immovable union of two joints of a crinoidal arm.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>The intimately united and apparently fused condition of certain low organisms during conjugation.</def></p>

<p><cs><col><b>Line of syzygies</b></col> <fld>(Astron.)</fld>, <cd>the straight line connecting the earth, the sun, and the moon or a planet, when the latter is in conjunction or opposition; -- used chiefly of the moon.</cd></cs><br/
[<source>1913 Webster</source>]</p>

