<p><-- Begin file 20 of 26:  Letter T (Version 0.48) 
        
           This file is part 20 of the GNU version of
     The Collaborative International Dictionary of English
               Also referred to as GCIDE
  * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * 

GCIDE is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2, or (at your option)
any later version.

GCIDE is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this copy of GCIDE; see the file COPYING.  If not, write 
to the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
Boston, MA 02111-1307, USA.
           * * * * * * * * * * * * * * * * * * * * * 

           This dictionary was derived from the
         Webster's Revised Unabridged Dictionary
                 Version published 1913
               by the  C. & G. Merriam Co.
                   Springfield, Mass.
                 Under the direction of
                Noah Porter, D.D., LL.D.

                        and from
           WordNet, a semantic network created by
              the Cognitive Science Department
                 of Princeton University
                  under the direction of
                   Prof. George Miller

             and is being updated and supplemented by
         an open coalition of volunteer collaborators from
                       around the world.

     This electronic dictionary is the starting point for an 
ongoing project to develop a modern on-line comprehensive encyclopedic
dictionary, by the efforts of all individuals willing to help build a
large and freely available knowledge base.  Contributions of data,
time, and effort are requested from any person willing to assist creation
of a comprehensive and organized knowledge base for free access on the
internet.  Anyone willing to assist in any way in constructing such a
knowledge base should contact:

     Patrick Cassidy          pc@worldsoul.org
     735 Belvidere Ave.       Office: (908)668-5252
     Plainfield, NJ 07062
     (908) 561-3416

    Last edit October 6, 2002.

 --></p>

<p><centered><point26>T.</point26></centered></p>

<p><ent>T</ent><br/
<hw>T</hw> <pr>(t<emac/)</pr>, <def>the twentieth letter of the English alphabet, is a nonvocal consonant. With the letter <xex>h</xex> it forms the digraph <xex>th</xex>, which has two distinct sounds, as in <xex>th</xex>in, <xex>th</xex>en. See <xex>Guide to Pronunciation</xex>, <sect/<sect/262-264, and also <sect/<sect/153, 156, 169, 172, 176, 178-180.</def><br/
[<source>1913 Webster</source>]</p>

<p>The letter derives its name and form from the Latin, the form of the Latin letter being further derived through the Greek from the Ph<oe/nician. The ultimate origin is probably Egyptian. It is etymologically most nearly related to <xex>d</xex>, <xex>s</xex>, <xex>th</xex>; as in <xex>t</xex>ug, <xex>d</xex>uke; <xex>t</xex>wo, <xex>d</xex>ual, L. <xex>d</xex>uo; re<xex>s</xex>in, L. re<xex>s</xex>ina, Gr. <grk>"rhti`nh</grk>, ten<xex>t</xex>, ten<xex>s</xex>e, a., <xex>t</xex>enuous, <xex>th</xex>in; nos<xex>t</xex>ril, <xex>th</xex>rill. See <er>D</er>, <er>S</er>.<br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>T bandage</b></col> <fld>(Surg.)</fld>, <cd>a bandage shaped like the letter <universbold>T</universbold>, and used principally for application to the groin, or perineum.</cd> -- <col><b>T cart</b></col>, <cd>a kind of fashionable two seated wagon for pleasure driving.</cd> -- <col><b>T iron</b></col>. <sd>(a)</sd> <cd>A rod with a short crosspiece at the end, -- used as a hook.</cd> <sd>(b)</sd> <cd>Iron in bars, having a cross section formed like the letter <universbold>T</universbold>, -- used in structures.</cd> -- <col><b>T rail</b></col>, <cd>a kind of rail for railroad tracks, having no flange at the bottom so that a section resembles the letter <universbold>T</universbold>.</cd> -- <col><b>T square</b></col>, <cd>a ruler having a crosspiece or head at one end, for the purpose of making parallel lines; -- so called from its shape. It is laid on a drawing board and guided by the crosspiece, which is pressed against the straight edge of the board. Sometimes the head is arranged to be set at different angles.</cd> -- <col><b>To a T</b></col>, <cd>exactly, perfectly; <as>as, to suit <ex>to a T</ex></as>.</cd> <mark>[Colloq.]</mark></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Ta</ent><br/
<hw>Ta</hw> <pr>(?)</pr>, <pos>v. t.</pos> <def>To take.</def> <mark>[Obs. or Scot.]</mark>  <rj><au>Cursor Mundi.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ Used by Chaucer to represent a peculiarity of the Northern dialect.</note><br/
[<source>1913 Webster</source>]</p>

<p><ent>Taas</ent><br/
<hw>Taas</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A heap. See <er>Tas</er>.</def> <mark>[Obs.]</mark>  <rj><au>Chaucer.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tab</ent><br/
<hw>Tab</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Etymol. uncertain.]</ety> <sn>1.</sn> <def>The flap or latchet of a shoe fastened with a string or a buckle.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A tag. See <er>Tag</er>, 2.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>A loop for pulling or lifting something.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>A border of lace or other material, worn on the inner front edge of ladies' bonnets.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <def>A loose pendent part of a lady's garment; esp., one of a series of pendent squares forming an edge or border.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>6.</sn> <def>a small projecting piece of a file folder, file card, or similar sheet used in a filing system, on which a notation is written to permit convenient search for the folder, card, etc.</def><br/
[<source>PJC</source>]</p>

<p><sn>7.</sn> <def>a bill or check for some purchase, as in a restaurant; <as>as, the salesman will pick up the <ex>tab</ex></as>.</def><br/
[<source>PJC</source>]</p>

<p><sn>8.</sn> <def>a key on a typewriter or computer keyboard which advances the carriage or curser to the next (preset) tab position; -- used especially to type or print text or numbers in columns.</def><br/
[<source>PJC</source>]</p>

<p><ent>Tabacco</ent><br/
<hw>Ta*bac"co</hw> <pr>(?)</pr>, <pos>n.</pos> <def>Tobacco.</def> <mark>[Obs.]</mark>  <rj><au>B. Jonson.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tabanus</ent><br/
||<hw>Ta*ba"nus</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L., horsefly.]</ety> <fld>(Zool.)</fld> <def>A genus of blood sucking flies, including the horseflies.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tabard</ent><br/
<hw>Tab"ard</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[OE. <ets>tabard</ets>, <ets>tabart</ets>; cf. Sp. & Pg. <ets>tabardo</ets>, It. <ets>tabarro</ets>, W. <ets>tabar</ets>, LGr. <grk>tampa`rion</grk>, LL. <ets>tabardum</ets>.]</ety> <def>A sort of tunic or mantle formerly worn for protection from the weather. When worn over the armor it was commonly emblazoned with the arms of the wearer, and from this the name was given to the garment adopted for heralds.</def> <altsp>[Spelt also <asp>taberd</asp>.]</altsp><br/
[<source>1913 Webster</source>]</p>

<p><q>In a <qex>tabard</qex> he [the Plowman] rode upon a mare.</q> <rj><qau>Chaucer.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tabarder</ent><br/
<hw>Tab"ard*er</hw> <pr>(?)</pr>, <pos>n.</pos> <sn>1.</sn> <def>One who wears a tabard.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A scholar on the foundation of Queen's College, Oxford, England, whose original dress was a tabard.</def>  <rj><au>Nares.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tabaret</ent><br/
<hw>Tab"a*ret</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Cf. <er>Tabby</er>.]</ety> <def>A stout silk having satin stripes, -- used for furniture.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tabasco sauce</ent><br/
<hw>Ta*bas"co sauce</hw> <pr>(?)</pr>. <ety>[So named after <etsep>Tabasco</etsep>, a river and state of Mexico.]</ety> <def>A kind of very pungent sauce made from red peppers.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>tabasheer</ent><br/
<hw>tab`a*sheer"</hw> <pr>(t<acr/b`<adot/*sh<emac/r")</pr>, <pos>n.</pos> <ety>[Per. <ets>tab<amac/sh<imac/r</ets>: cf. Skr. <ets>tvakksh<imac/r<amac/</ets>, <ets>tvaksh<imac/r<amac/</ets>.]</ety> <def>A concretion in the joints of the bamboo, which consists largely or chiefly of pure silica.  It is highly valued in the East Indies as a medicine for the cure of bilious vomitings, bloody flux, piles, and various other diseases.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tabbinet</ent><br/
<hw>Tab"bi*net</hw> <pr>(t<acr/b"b<icr/*n<ecr/t)</pr>, <pos>n.</pos> <ety>[Cf. <er>Tabby</er>.]</ety> <def>A fabric like poplin, with a watered surface.</def> <altsp>[Written also <asp>tabinet</asp>.]</altsp><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tabby</ent><br/
<hw>Tab"by</hw> <pr>(t<acr/b"b<ycr/)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Tabbies</plw> <pr>(t<acr/b"b<icr/z)</pr>.</plu> <ety>[F. <ets>tabis</ets> (cf. It. <ets>tab<igrave/</ets>, Sp. & Pg. <ets>tab<iacute/</ets>, LL. <ets>attabi</ets>), fr. Ar. <ets>'att<amac/b<imac/</ets>, properly the name of a quarter of Bagdad where it was made, the quarter being named from the prince <etsep>Attab</etsep>, great grandson of Omeyya.  Cf. <er>Tobine</er>.]</ety> <sn>1.</sn> <def>A kind of waved silk, usually called <altname>watered silk</altname>, manufactured like taffeta, but thicker and stronger.  The watering is given to it by calendering.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A mixture of lime with shells, gravel, or stones, in equal proportions, with an equal proportion of water. When dry, this becomes as hard as rock.</def>  <rj><au>Weale.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>A brindled cat; hence, popularly, any cat.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>An old maid or gossip.</def> <mark>[Colloq.]</mark>  <rj><au>Byron.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tabby</ent><br/
<hw>Tab"by</hw> <pr>(?)</pr>, <pos>a.</pos> <sn>1.</sn> <def>Having a wavy or watered appearance; <as>as, a <ex>tabby</ex> waistcoat</as>.</def>  <rj><au>Pepys.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Brindled; diversified in color; <as>as, a <ex>tabby</ex> cat</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Tabby moth</b></col> <fld>(Zool.)</fld>, <cd>the grease moth. See under <er>Grease</er>.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tabby</ent><br/
<hw>Tab"by</hw>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Tabbied</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Tabbying</conjf> <pr>(?)</pr>.]</vmorph> <def>To water; to cause to look wavy, by the process of calendering; to calender; <as>as, to <ex>tabby</ex> silk, mohair, ribbon, etc.</as></def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tabefaction</ent><br/
<hw>Tab`e*fac"tion</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[See <er>Tabefy</er>.]</ety> <def>A wasting away; a gradual losing of flesh by disease.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tabefy</ent><br/
<hw>Tab"e*fy</hw> <pr>(?)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Tabefied</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Tabefying</conjf> <pr>(?)</pr>.]</vmorph> <ety>[L. <ets>tabere</ets> to waste away + <ets>-fy</ets>: cf. L. <ets>tabefacere</ets> to melt.]</ety> <def>To cause to waste gradually, to emaciate.</def> <mark>[R.]</mark>  <rj><au>Harvey.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tabellion</ent><br/
<hw>Ta*bel"lion</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>tabellio</ets>, fr. <ets>tabella</ets> a tablet, a writing, document, dim. of <ets>tabula</ets> a board: cf. F. <ets>tabellion</ets>. See <er>Table</er>.]</ety> <def>A secretary or notary under the Roman empire; also, a similar officer in France during the old monarchy.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Taber</ent><br/
<hw>Ta"ber</hw> <pr>(?)</pr>, <pos>v. i.</pos> <def>Same as <er>Tabor</er>.</def>  <rj><au>Nahum ii. 7.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Taberd</ent><br/
<hw>Tab"erd</hw> <pr>(?)</pr>, <pos>n.</pos> <def>See <er>Tabard</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tabernacle</ent><br/
<hw>Tab"er*na*cle</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F., fr. L. <ets>tabernaculum</ets>, dim. of <ets>taberna</ets> nut. See <er>Tabern</er>.]</ety> <sn>1.</sn> <def>A slightly built or temporary habitation; especially, a tent.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Dwelling in <qex>tabernacles</qex> with Isaac and Jacob.</q> <rj><qau>Heb. xi. 9.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Orange trees planted in the ground, and secured in winter with a wooden <qex>tabernacle</qex> and stoves.</q> <rj><qau>Evelyn.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Jewish Antiq.)</fld> <def>A portable structure of wooden framework covered with curtains, which was carried through the wilderness in the Israelitish exodus, as a place of sacrifice and worship.</def>  <rj><au>Ex. xxvi.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>Hence, the Jewish temple; sometimes, any other place for worship.</def>  <rj><au>Acts xv. 16.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>Figuratively: The human body, as the temporary abode of the soul.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Shortly I must put off this my <qex>tabernacle</qex>.</q> <rj><qau>2 Pet. i. 14.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <def>Any small cell, or like place, in which some holy or precious things was deposited or kept.</def> Specifically: --<br/
[<source>1913 Webster</source>]</p>

<p><sd>(a)</sd> <def>The ornamental receptacle for the pyx, or for the consecrated elements, whether a part of a building or movable.</def><br/
[<source>1913 Webster</source>]</p>

<p><sd>(b)</sd> <def>A niche for the image of a saint, or for any sacred painting or sculpture.</def><br/
[<source>1913 Webster</source>]</p>

<p><sd>(c)</sd> <def>Hence, a work of art of sacred subject, having a partially architectural character, as a solid frame resting on a bracket, or the like.</def><br/
[<source>1913 Webster</source>]</p>

<p><sd>(d)</sd> <def>A tryptich for sacred imagery.</def><br/
[<source>1913 Webster</source>]</p>

<p><sd>(e)</sd> <def>A seat or stall in a choir, with its canopy.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>6.</sn> <fld>(Naut.)</fld> <def>A boxlike step for a mast with the after side open, so that the mast can be lowered to pass under bridges, etc.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Feast of Tabernacles</b></col> <fld>(Jewish Antiq.)</fld>, <cd>one of the three principal festivals of the Jews, lasting seven days, during which the people dwelt in booths formed of the boughs of trees, in commemoration of the habitation of their ancestors in similar dwellings during their pilgrimage in the wilderness.</cd> -- <col><b>Tabernacle work</b></col>, <cd>rich canopy work like that over the head of niches, used over seats or stalls, or over sepulchral monuments.</cd> <au>Oxf. Gloss.</au></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tabernacle</ent><br/
<hw>Tab"er*na*cle</hw>, <pos>v. i.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Tabernacled</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Tabernacling</conjf> <pr>(?)</pr>.]</vmorph> <def>To dwell or reside for a time; to be temporary housed.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>He assumed our nature, and <qex>tabernacled</qex> among us in the flesh.</q> <rj><qau>Dr. J. Scott.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tabernacular</ent><br/
<hw>Tab`er*nac"u*lar</hw> <pr>(?)</pr>, <pos>a.</pos> <sn>1.</sn> <def>Of or pertaining to a tabernacle, especially the Jewish tabernacle.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Formed in latticework; latticed.</def>  <rj><au>T. Warton.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>Of or pertaining to huts or booths; hence, common; low.</def> <ldquo/Horribly <xex>tabernacular</xex>.<rdquo/  <rj><au>De Quincey.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tabes</ent><br/
||<hw>Ta"bes</hw> <pr>(t<amac/"b<emac/z)</pr>, <pos>n.</pos> <ety>[L., a wasting disease.]</ety> <fld>(Med.)</fld> <def>Progressive emaciation of the body, accompanied with hectic fever, with no well-marked local symptoms.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs>||<col><b>Tabes dorsalis</b></col> <pr>(t<amac/"b<emac/z d<ocir/r*s<amac/"l<icr/s)</pr> <ety>[NL., tabes of the back]</ety>, <cd>locomotor ataxia; -- sometimes called simply <xex>tabes</xex>.</cd> -- ||<col><b>Tabes mesenterica</b></col> <pr>(<?/)</pr> <ety>[NL., mesenteric tabes]</ety>, <cd>a wasting disease of childhood characterized by chronic inflammation of the lymphatic glands of the mesentery, attended with caseous degeneration.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tabescent</ent><br/
<hw>Ta*bes"cent</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>tabescens</ets> wasting, p. pr. of <ets>tabescere</ets>.]</ety> <def>Withering, or wasting away.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tabetic</ent><br/
<hw>Ta*bet"ic</hw> <pr>(?)</pr>, <pos>a.</pos> <fld>(Med.)</fld> <def>Of or pertaining to tabes; of the nature of tabes; affected with tabes; tabid.</def> -- <def2><pos>n.</pos> <def>One affected with tabes.</def></def2><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tabid</ent><br/
<hw>Tab"id</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>tabidus</ets>: cf. F. <ets>tabide</ets>. See <er>Tabes</er>.]</ety> <fld>(Med.)</fld> <def>Affected by tabes; tabetic.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>In <qex>tabid</qex> persons, milk is the bset restorative.</q> <rj><qau>Arbuthnot.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p>-- <wordforms><wf>Tab"id*ly</wf>, <pos>adv.</pos> -- <wf>Tab"id*ness</wf>, <pos>n.</pos></wordforms><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tabifical</ent><br/
<ent>Tabific</ent><br/
<mhw>{ <hw>Ta*bif"ic</hw> <pr>(?)</pr>, <hw>Ta*bif"ic*al</hw> <pr>(?)</pr>, }</mhw> <pos>a.</pos> <ety>[<ets>Tabes</ets> + L. <ets>facere</ets> to make.]</ety> <fld>(Med.)</fld> <def>Producing tabes; wasting; tabefying.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tabinet</ent><br/
<hw>Tab"inet</hw> <pr>(?)</pr>, <pos>n.</pos> <def>See <er>Tabbinet</er>.</def>  <rj><au>Thackeray.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tablature</ent><br/
<hw>Tab"la*ture</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Cf. F. <ets>tablature</ets> ancient mode of musical notation. See <er>Table</er>.]</ety> <sn>1.</sn> <fld>(Paint.)</fld> <def>A painting on a wall or ceiling; a single piece comprehended in one view, and formed according to one design; hence, a picture in general.</def>  <rj><au>Shaftesbury.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Mus.)</fld> <def>An ancient mode of indicating musical sounds by letters and other signs instead of by notes.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The chimes of bells are so rarely managed that I went up to that of Sir Nicholas, where I found who played all sorts of compositions from the <qex>tablature</qex> before him as if he had fingered an organ.</q> <rj><qau>Evelyn.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <fld>(Anat.)</fld> <def>Division into plates or tables with intervening spaces; <as>as, the <ex>tablature</ex> of the cranial bones</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Table</ent><br/
<hw>Ta"ble</hw> <pr>(t<amac/"'l)</pr>, <pos>n.</pos> <ety>[F., fr. L. <ets>tabula</ets> a board, tablet, a painting.  Cf. <er>Tabular</er>, <er>Taffrail</er>, <er>Tavern</er>.]</ety> <sn>1.</sn> <def>A smooth, flat surface, like the side of a board; a thin, flat, smooth piece of anything; a slab.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>A bagnio paved with fair <qex>tables</qex> of marble.</q> <rj><qau>Sandys.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A thin, flat piece of wood, stone, metal, or other material, on which anything is cut, traced, written, or painted; a tablet</def>; <pluf>pl.</pluf> <def>a memorandum book.</def> <ldquo/The names . . . written on his <xex>tables</xex>.<rdquo/  <rj><au>Chaucer.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>And the Lord said unto Moses, Hew thee two <qex>tables</qex> of stone like unto the first, and I will write upon these <qex>tables</qex> the words that were in the first <qex>tables</qex>, which thou brakest.</q> <rj><qau>Ex. xxxiv. 1.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>And stand there with your <qex>tables</qex> to glean<br/
The golden sentences.</q> <rj><qau>Beau. & Fl.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>Any smooth, flat surface upon which an inscription, a drawing, or the like, may be produced.</def> <ldquo/Painted in a <xex>table</xex> plain.<rdquo/  <rj><au>Spenser.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>The opposite walls are painted by Rubens, which, with that other of the Infanta taking leave of Don Philip, is a most incomparable <qex>table</qex>.</q> <rj><qau>Evelyn.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>St. Antony has a <qex>table</qex> that hangs up to him from a poor peasant.</q> <rj><qau>Addison.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>Hence, in a great variety of applications: A condensed statement which may be comprehended by the eye in a single view; a methodical or systematic synopsis; the presentation of many items or particulars in one group; a scheme; a schedule.</def> Specifically: --<br/
[<source>1913 Webster</source>]</p>

<p><sd>(a)</sd> <fld>(Bibliog.)</fld> <def>A view of the contents of a work; a statement of the principal topics discussed; an index; a syllabus; a synopsis; <as>as, a <ex>table</ex> of contents</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sd>(b)</sd> <fld>(Chem.)</fld> <def>A list of substances and their properties; especially, the a list of the elementary substances with their atomic weights, densities, symbols, etc.; the periodic table of the elements.</def><br/
[<source>1913 Webster</source>]</p>

<p><sd>(c)</sd> <fld>(Mathematics, Science and Technology)</fld>  <def>Any collection and arrangement in a condensed form of many particulars or values, for ready reference, as of weights, measures, currency, specific gravities, etc.; also, a series of numbers following some law, and expressing particular values corresponding to certain other numbers on which they depend, and by means of which they are taken out for use in computations; <as>as, <ex>tables</ex> of logarithms, sines, tangents, squares, cubes, etc.; annuity <ex>tables</ex>; interest <ex>tables</ex>; astronomical <ex>tables</ex>; a <ex>table</ex> of logarithms, etc.</as></def><br/
[<source>1913 Webster</source>]</p>

<p><sd>(d)</sd> <fld>(Palmistry)</fld> <def>The arrangement or disposition of the lines which appear on the inside of the hand.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Mistress of a fairer <qex>table</qex><br/
Hath not history for fable.</q> <rj><qau>B. Jonson.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <def>An article of furniture, consisting of a flat slab, board, or the like, having a smooth surface, fixed horizontally on legs, and used for a great variety of purposes, as in eating, writing, or working.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>We may again<br/
Give to our <qex>tables</qex> meat.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>The nymph the <qex>table</qex> spread.</q> <rj><qau>Pope.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>6.</sn> <def>Hence, food placed on a table to be partaken of; fare; entertainment; <as>as, to set a good <ex>table</ex></as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>7.</sn> <def>The company assembled round a table.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>I drink the general joy of the whole <qex>table</qex>.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>8.</sn> <fld>(Anat.)</fld> <def>One of the two, external and internal, layers of compact bone, separated by diploe, in the walls of the cranium.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>9.</sn> <fld>(Arch.)</fld> <def>A stringcourse which includes an offset; esp., a band of stone, or the like, set where an offset is required, so as to make it decorative. See <er>Water table</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>10.</sn> <fld>(Games)</fld> <sd>(a)</sd> <def>The board on the opposite sides of which backgammon and draughts are played.</def> <sd>(b)</sd> <def>One of the divisions of a backgammon board; <as>as, to play into the right-hand <ex>table</ex></as>.</def> <sd>(c)</sd> <pluf>pl.</pluf> <def>The games of backgammon and of draughts.</def> <mark>[Obs.]</mark>  <rj><au>Chaucer.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>This is the ape of form, monsieur the nice,<br/
That, when he plays at <qex>tables</qex>, chides the dice.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>11.</sn> <fld>(Glass Manuf.)</fld> <def>A circular plate of crown glass.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>A circular plate or <qex>table</qex> of about five feet diameter weighs on an average nine pounds.</q> <rj><qau>Ure.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>12.</sn> <fld>(Jewelry)</fld> <def>The upper flat surface of a diamond or other precious stone, the sides of which are cut in angles.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>13.</sn> <fld>(Persp.)</fld> <def>A plane surface, supposed to be transparent and perpendicular to the horizon; -- called also <altname>perspective plane</altname>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>14.</sn> <fld>(Mach.)</fld> <def>The part of a machine tool on which the work rests and is fastened.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><mcol><col><b>Bench table</b></col>, <col><b>Card table</b></col>, <col><b>Communion table</b></col>, <col><b>Lord's table</b></col></mcol>, <cd>etc. See under <er>Bench</er>, <er>Card</er>, etc.</cd> -- <col><b>Raised table</b></col> <fld>(Arch. & Sculp.)</fld>, <cd>a raised or projecting member of a flat surface, large in proportion to the projection, and usually rectangular, -- especially intended to receive an inscription or the like.</cd> -- <col><b>Roller table</b></col> <fld>(Horology)</fld>, <cd>a flat disk on the arbor of the balance of a watch, holding the jewel which rolls in and out of the fork at the end of the lever of the escapement.</cd> -- <col><b>Round table</b></col>. <cd>See Dictionary of Noted Names in Fiction.</cd> -- <col><b>Table anvil</b></col>, <cd>a small anvil to be fastened to a table for use in making slight repairs.</cd> -- <col><b>Table base</b></col>. <fld>(Arch.)</fld> <cd>Same as <er>Water table</er>.</cd> -- <col><b>Table bed</b></col>, <cd>a bed in the form of a table.</cd> -- <col><b>Table beer</b></col>, <cd>beer for table, or for common use; small beer.</cd> -- <col><b>Table bell</b></col>, <cd>a small bell to be used at table for calling servants.</cd> -- <col><b>Table cover</b></col>, <cd>a cloth for covering a table, especially at other than mealtimes.</cd> -- <col><b>Table diamond</b></col>, <cd>a thin diamond cut with a flat upper surface.</cd> -- <col><b>Table linen</b></col>, <cd>linen tablecloth, napkins, and the like.</cd> -- <col><b>Table money</b></col> <fld>(Mil. or Naut.)</fld>, <cd>an allowance sometimes made to officers over and above their pay, for table expenses.</cd> -- <col><b>Table rent</b></col> <fld>(O. Eng. Law)</fld>, <cd>rent paid to a bishop or religious, reserved or appropriated to his table or housekeeping.</cd> <au>Burrill.</au> -- <col><b>Table shore</b></col> <fld>(Naut.)</fld>, <cd>a low, level shore.</cd> -- <col><b>Table talk</b></col>, <cd>conversation at table, or at meals.</cd> -- <col><b>Table talker</b></col>, <cd>one who talks at table.</cd> -- <mcol><col><b>Table tipping</b></col>, <col><b>Table turning</b></col></mcol>, <cd>certain movements of tables, etc., attributed by some to the agency of departed spirits, and by others to the development of latent vital or spriritual forces, but more commonly ascribed to the muscular force of persons in connection with the objects moved, or to physical force applied otherwise.</cd> -- <mcol><col><b>Tables of a girder</b></col> <it>or</it> <col><b>Tables of a chord</b></col></mcol> <fld>(Engin.)</fld>, <cd>the upper and lower horizontal members.</cd> -- <col><b>To lay on the table</b></col>, <cd>in parliamentary usage, to lay, as a report, motion, etc., on the table of the presiding officer, -- that is, to postpone the consideration of, by a vote; -- also called to <altname>table</altname> .  It is a tactic often used with the intention of postponing consideration of a motion indefinitely, that is, to kill the motion.</cd> -- <col><b>To serve tables</b></col> <fld>(Script.)</fld>, <cd>to provide for the poor, or to distribute provisions for their wants.</cd> <au>Acts vi. 2.</au> -- <col><b>To turn the tables</b></col>, <cd>to change the condition or fortune of contending parties; -- a metaphorical expression taken from the vicissitudes of fortune in gaming.</cd> -- <col><b>Twelve tables</b></col> <fld>(Rom. Antiq.)</fld>, <cd>a celebrated body of Roman laws, framed by decemvirs appointed 450 years before Christ, on the return of deputies or commissioners who had been sent to Greece to examine into foreign laws and institutions. They consisted partly of laws transcribed from the institutions of other nations, partly of such as were altered and accommodated to the manners of the Romans, partly of new provisions, and mainly, perhaps, of laws and usages under their ancient kings.</cd> <au>Burrill.</au></cs><br/
[<source>1913 Webster</source>]</p>

<p><-- p. 1467 --><br/
[<source>1913 Webster</source>]</p>

<p><ent>Table</ent><br/
<hw>Ta"ble</hw> <pr>(t<amac/"b'l)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Tabled</conjf> <pr>(t<amac/"b'ld)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Tabling</conjf> <pr>(t<amac/"bling)</pr>.]</vmorph> <sn>1.</sn> <def>To form into a table or catalogue; to tabulate; <as>as, to <ex>table</ex> fines</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To delineate, as on a table; to represent, as in a picture.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q><qex>Tabled</qex> and pictured in the chambers of meditation.</q> <rj><qau>Bacon.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>To supply with food; to feed.</def> <mark>[Obs.]</mark>  <rj><au>Milton.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <fld>(Carp.)</fld> <def>To insert, as one piece of timber into another, by alternate scores or projections from the middle, to prevent slipping; to scarf.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <def>To lay or place on a table, as money.</def>  <rj><au>Carlyle.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>6.</sn> <def>In parliamentary usage, to lay on the table; to postpone, by a formal vote, the consideration of (a bill, motion, or the like) till called for, or indefinitely.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>7.</sn> <def>To enter upon the docket; <as>as, to <ex>table</ex> charges against some one</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>8.</sn> <fld>(Naut.)</fld> <def>To make broad hems in the skirts and bottoms of (sails) in order to strengthen them in the part attached to the boltrope.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Table</ent><br/
<hw>Ta"ble</hw>, <pos>v. i.</pos> <def>To live at the table of another; to board; to eat.</def> <mark>[Obs.]</mark> <ldquo/He . . . was driven from the society of men to <xex>table</xex> with the beasts.<rdquo/  <rj><au>South.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tableau</ent><br/
||<hw>Ta`bleau"</hw> <pr>(t<adot/`bl<omac/" E. t<acr/b"l<omac/`; 277)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Tableaux</plw> <pr>(t<adot/`bl<omac/" E. t<acr/b"l<omac/z`)</pr>.</plu> <ety>[F., dim. fr. L. <ets>tabula</ets> a painting. See <er>Table</er>.]</ety> <sn>1.</sn> <def>A striking and vivid representation; a picture.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A representation of some scene by means of persons grouped in the proper manner, placed in appropriate postures, and remaining silent and motionless.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <fld>(Solitaire)</fld> <def>The arrangement, or layout, of cards.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Tableau vivant</ent><br/
||<hw>Ta`bleau" vi`vant"</hw> <pr>(t<adot/`bl<omac/" v<esl/`v<aum/N")</pr>; <plu><it>pl.</it> <plw>Tableaux vivants</plw> <pr>(t<adot/`bl<omac/" v<esl/`v<aum/N")</pr>.</plu> <ety>[F.]</ety> <def>Same as <er>Tableau</er>, <pos>n.</pos>, 2.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tablebook</ent><br/
<hw>Ta"ble*book`</hw> <pr>(t<amac/"b'l b<oocr/k`)</pr>, <pos>n.</pos> <def>A tablet; a notebook.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Put into your <qex>tablebook</qex> whatever you judge worthy.</q> <rj><qau>Dryden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tablecloth</ent><br/
<hw>Ta"ble*cloth`</hw> <pr>(t<amac/"b'l kl<ocr/th`; 115)</pr>, <pos>n.</pos> <def>A cloth for covering a table, especially one with which a table is covered before the dishes, etc., are set on for meals.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Table d'hote</ent><br/
||<hw>Ta"ble d'h<ocir/te"</hw> <pr>(t<adot/"bl' d<omac/t`)</pr>; <plu><it>pl.</it> <plw>Tables d'h<ocir/te</plw> <pr>(#)</pr>.</plu> <ety>[F., literally, table of the landlord.]</ety> <sn>1.</sn> <def>A common table for guests at a hotel; an ordinary.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Now, commonly, a meal, usually of several preselected and fixed courses, in a restaurant, hotel, or the like, for which one pays a fixed price.  Sometimes, a meal with optional courses for which one pays a fixed price irrespective of what one orders; but the latter is usuallyt referred to as a <contr>pris fixe</contr> meal or a <contr>a la carte</contr> meal. Often used adjectively; <as>as, a <ex>table-d'h<ocir/te</ex> meal</as>.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Table-land</ent><br/
<hw>Ta"ble-land`</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A broad, level, elevated area of land; a plateau.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The toppling crags of Duty scaled,<br/
Are close upon the shining <qex>table-lands</qex><br/
To which our God himself is moon and sun.</q> <rj><qau>Tennyson.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tableman</ent><br/
<hw>Ta"ble*man</hw> <pr>(?)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Tablemen</plw> <pr>(<?/)</pr>.</plu> <def>A man at draughts; a piece used in playing games at tables. See <er>Table</er>, <pos>n.</pos>, 10.</def> <mark>[R.]</mark>  <rj><au>Bacon.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tablement</ent><br/
<hw>Ta"ble*ment</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Arch.)</fld> <def>A table.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q><qex>Tablements</qex> and chapters of pillars.</q> <rj><qau>Holland.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tabler</ent><br/
<hw>Ta"bler</hw> <pr>(?)</pr>, <pos>n.</pos> <sn>1.</sn> <def>One who boards.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>One who boards others for hire.</def> <mark>[Obs.]</mark>  <rj><au>B. Jonson.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tablespoon</ent><br/
<hw>Ta"ble*spoon`</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A spoon of the largest size commonly used at the table; -- distinguished from <xex>teaspoon</xex>, <xex>dessert spoon</xex>, etc.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tablespoonful</ent><br/
<hw>Ta"ble*spoon`ful</hw> <pr>(?)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Tablespoonfuls</plw> <pr>(<?/)</pr>.</plu> <def>As much as a tablespoon will hold; enough to fill a tablespoon. It is usually reckoned as one half of a fluid ounce, or four fluid drams.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tablet</ent><br/
<hw>Ta"blet</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F. <ets>tablette</ets>, dim. of <ets>table</ets>. See <er>Table</er>.]</ety> <sn>1.</sn> <def>A small table or flat surface.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A flat piece of any material on which to write, paint, draw, or engrave; also, such a piece containing an inscription or a picture.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>Hence, a small picture; a miniature.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <pluf>pl.</pluf> <def>A kind of pocket memorandum book.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <def>A flattish cake or piece; <as>as, <ex>tablets</ex> of arsenic were formerly worn as a preservative against the plague</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>6.</sn> <fld>(Pharm.)</fld> <def>A solid kind of electuary or confection, commonly made of dry ingredients with sugar, and usually formed into little flat squares; -- called also <altname>lozenge</altname>, and <altname>troche</altname>, especially when of a round or rounded form.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tableware</ent><br/
<hw>Ta"ble*ware`</hw> <pr>(?)</pr>, <pos>n.</pos> <def>Ware, or articles collectively, for use during meals, including, for example, <examp>dishes</examp>, <examp>plates</examp>, <examp>bowls</examp>, <examp>knives</examp>, <examp>forks</examp>, and <examp>spoons</examp>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Table work</ent><br/
<hw>Table work</hw>. <fld>(Print.)</fld> <def>Typesetting of tabular nmatter, or the type matter set in tabular form.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Tabling</ent><br/
<hw>Ta"bling</hw> <pr>(?)</pr>, <pos>n.</pos> <sn>1.</sn> <def>A forming into tables; a setting down in order.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Carp.)</fld> <def>The letting of one timber into another by alternate scores or projections, as in shipbuilding.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <fld>(Naut.)</fld> <def>A broad hem on the edge of a sail.</def>  <rj><au>Totten.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>Board; support.</def> <mark>[Obs.]</mark>  <rj><au>Trence in English (1614).</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <def>Act of playing at tables. See <er>Table</er>, <pos>n.</pos>, 10.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Tabling house</b></col>, <cd>a gambling house.</cd> <mark>[Obs.]</mark> <au>Northbrooke.</au></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tabloid</ent><br/
<hw>Tab"loid</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[A table-mark.]</ety> <sn>1.</sn> <def>A compressed portion of one or more drugs or chemicals, or of food, etc.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><sn>2.</sn> <def>a newspaper with pages about half the size of a standard-sized newspaper, especially one that has relatively short or condensed articles and a large porortion of pictorial matter.</def><br/
[<source>PJC</source>]</p>

<p><ent>Tabloid</ent><br/
<hw>Tab"loid</hw> <pr>(t<acr/b"loid)</pr>, <pos>a.</pos> <sn>1.</sn> <def>Compressed or condensed, as into a tabloid; administrated in or as in tabloids, or small condensed bits; <as>as, a <ex>tabloid</ex> form of imparting information</as>.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><sn>2.</sn> <def>of or pertaining to a tabloid newspaper or the type of story typically contained in one, such as lurid or sensationalistic stories of scandal, crime, or violence.</def><br/
[<source>PJC</source>]</p>

<p><ent>Taboo</ent><br/
<hw>Ta*boo"</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A total prohibition of intercourse with, use of, or approach to, a given person or thing under pain of death, -- an interdict of religious origin and authority, formerly common in the islands of Polynesia; interdiction.</def> <altsp>[Written also <asp>tabu</asp>.]</altsp><br/
[<source>1913 Webster</source>]</p>

<p><ent>Taboo</ent><br/
<hw>Ta*boo"</hw>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Tabooed</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Tabooing</conjf>.]</vmorph> <def>To put under taboo; to forbid, or to forbid the use of; to interdict approach to, or use of; <as>as, to <ex>taboo</ex> the ground set apart as a sanctuary for criminals</as>.</def> <altsp>[Written also <asp>tabu</asp>.]</altsp><br/
[<source>1913 Webster</source>]</p>

<p><ent>Taboo</ent><br/
<hw>Ta*boo"</hw> <pr>(?)</pr>, <pos>a.</pos> <altsp>[Written also <asp>tabu</asp> and <asp>tapu</asp>.]</altsp> <ety>[Polynesian <ets>tabu</ets>, <ets>tapu</ets>, sacred, under restriction, a prohibition.]</ety> <def>Set apart or sacred by religious custom among certain races of Polynesia, New Zealand, etc., and forbidden to certain persons or uses; hence, prohibited under severe penalties; interdicted; <as>as, food, places, words, customs, etc., may be <ex>taboo</ex></as>.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Tabor</ent><br/
<hw>Ta"bor</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[OF. <ets>tabor</ets>, <ets>tabour</ets>, F. <ets>tambour</ets>; cf. Pr. <ets>tabor</ets>, <ets>tanbor</ets>, Sp. & Pg. <ets>tambor</ets>, <ets>atambor</ets>, It. <ets>tamburo</ets>; all fr. Ar. & Per. <ets>tamb<?/r</ets> a kind of lute, or giutar, or Per. <ets>tab<imac/r</ets> a drum.  Cf. <er>Tabouret</er>, <er>Tambour</er>.]</ety> <fld>(Mus.)</fld> <def>A small drum used as an accompaniment to a pipe or fife, both being played by the same person.</def> <altsp>[Written also <asp>tabour</asp>, and <asp>taber</asp>.]</altsp><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tabor</ent><br/
<hw>Ta"bor</hw>, <pos>v. i.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Tabored</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Taboring</conjf>.]</vmorph> <ety>[Cf. OF. <ets>taborer</ets>.]</ety> <altsp>[Written also <asp>tabour</asp>.]</altsp> <sn>1.</sn> <def>To play on a tabor, or little drum.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To strike lightly and frequently.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tabor</ent><br/
<hw>Ta"bor</hw>, <pos>v. t.</pos> <def>To make (a sound) with a tabor.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Taborer</ent><br/
<hw>Ta"bor*er</hw> <pr>(?)</pr>, <pos>n.</pos> <def>One who plays on the tabor.</def>  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Taboret</ent><br/
<hw>Tab"o*ret</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Dim. of <ets>tabor</ets>.  Cf. <er>Tabret</er>.]</ety> <fld>(Mus.)</fld> <def>A small tabor.</def> <altsp>[Written also <asp>tabouret</asp>.]</altsp><br/
[<source>1913 Webster</source>]</p>

<p><ent>Taborine</ent><br/
<hw>Tab"o*rine</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[OF. <ets>tabourin</ets>, F. <ets>tambourin</ets>. See <er>Tabor</er>, and cf. <er>Tambourine</er>.]</ety> <fld>(Mus.)</fld> <def>A small, shallow drum; a tabor.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Taborite</ent><br/
<hw>Ta"bor*ite</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Eccl. Hist.)</fld> <def>One of certain Bohemian reformers who suffered persecution in the fifteenth century; -- so called from <xex>Tabor</xex>, a hill or fortress where they encamped during a part of their struggles.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tabour</ent><br/
<hw>Ta"bour</hw> <pr>(?)</pr>, <pos>n. & v.</pos> <def>See <er>Tabor</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tabouret</ent><br/
<hw>Tab"ou*ret</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F., dim. of OF. <ets>tabor</ets>, <ets>tabour</ets>, drum. See <er>Tabor</er>.]</ety> <sn>1.</sn> <def>Same as <er>Taboret</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A seat without arms or back, cushioned and stuffed: a high stool; -- so called from its resemblance to a drum.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>An embroidery frame.</def>  <rj><au>Knight.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Right of the tabouret</b></col>, <cd>the privilege of sitting on a tabouret in the presence of the severeign, formerly granted to certain ladies of high rank at the French court.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tabrere</ent><br/
<hw>Tab"rere</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A taborer.</def> <mark>[Obs.]</mark>  <rj><au>Spenser.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tabret</ent><br/
<hw>Tab"ret</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A taboret.</def>  <rj><au>Young.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tabu</ent><br/
<hw>Ta*bu"</hw> <pr>(?)</pr>, <pos>n. & v.</pos> <def>See <er>Taboo</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tabula</ent><br/
||<hw>Tab"u*la</hw> <pr>(?)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Tabulae</plw> <pr>(#)</pr>.</plu> <ety>[L.]</ety> <sn>1.</sn> <def>A table; a tablet.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Zool.)</fld> <def>One of the transverse plants found in the calicles of certain corals and hydroids.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Tabula rasa</b></col> <pr>(<?/)</pr> <ety>[L.]</ety>, <cd>a smoothed tablet; hence, figuratively, the mind in its earliest state, before receiving impressions from without; -- a term used by Hobbes, Locke, and others, in maintaining a theory opposed to the doctrine of <xex>innate ideas</xex>.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tabular</ent><br/
<hw>Tab"u*lar</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>tabularis</ets>, fr. <ets>tabula</ets> a board, table. See <er>Table</er>.]</ety> <def>Having the form of, or pertaining to, a table (in any of the uses of the word).</def> Specifically: --<br/
[<source>1913 Webster</source>]</p>

<p><sd>(a)</sd> <def>Having a flat surface; <as>as, a <ex>tabular</ex> rock</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sd>(b)</sd> <def>Formed into a succession of flakes; laminated.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Nodules . . . that are <qex>tabular</qex> and plated.</q> <rj><qau>Woodward.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sd>(c)</sd> <def>Set in squares.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><sd>(d)</sd> <def>Arranged in a schedule; <as>as, <ex>tabular</ex> statistics</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sd>(e)</sd> <def>Derived from, or computed by, the use of tables; <as>as, <ex>tabular</ex> right ascension</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Tabular difference</b></col> <fld>(Math.)</fld>, <cd>the difference between two consecutive numbers in a table, sometimes printed in its proper place in the table.</cd> -- <col><b>Tabular spar</b></col> <fld>(Min.)</fld>, <cd>wollastonite.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tabularization</ent><br/
<hw>Tab`u*lar*i*za"tion</hw> <pr>(?)</pr>, <pos>n.</pos> <def>The act of tabularizing, or the state of being tabularized; formation into tables; tabulation.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tabularize</ent><br/
<hw>Tab"u*lar*ize</hw> <pr>(?)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Tabularized</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Tabularizing</conjf> <pr>(?)</pr>.]</vmorph> <def>To tabulate.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tabulata</ent><br/
||<hw>Tab`u*la"ta</hw> <pr>(?)</pr>, <pos>n. pl.</pos> <ety>[NL., fr. L. <ets>tabulatus</ets> floored.]</ety> <fld>(Zool.)</fld> <def>An artificial group of stony corals including those which have transverse septa in the calicles. The genera <gen>Pocillopora</gen> and Favosites are examples.</def>
<-- ## note that Pocillopora is italicised but not listed separately in this dictionary.  Favosites is not italicised, and has an entry as a headword.  Is that the difference between italicisation  or not for genus names? --><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tabulate</ent><br/
<hw>Tab"u*late</hw> <pr>(?)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Tabulated</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Tabulating</conjf>.]</vmorph> <ety>[L. <ets>tabula</ets> a table. See <er>Tabular</er>.]</ety> <sn>1.</sn> <def>To form into a table or tables; to reduce to tables or synopses.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>A philosophy is not worth the having, unless its results may be <qex>tabulated</qex>, and put in figures.</q> <rj><qau>I. Taylor.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To shape with a flat surface.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tabulation</ent><br/
<hw>Tab`u*la"tion</hw> <pr>(?)</pr>, <pos>n.</pos> <def>The act of forming into a table or tables; <as>as, the <ex>tabulation</ex> of statistics</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tac</ent><br/
<hw>Tac</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Cf. <er>Tack</er>, <pos>n.</pos>, 4.]</ety> <fld>(O. Eng. Law)</fld> <def>A kind of customary payment by a tenant; -- a word used in old records.</def>  <rj><au>Cowell.</au>  <au>Burrill.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tacamahaca</ent><br/
<ent>Tacamahac</ent><br/
<mhw>{ <hw>Tac"a*ma*hac`</hw> <pr>(?)</pr>, <hw>Tac`a*ma*ha"ca</hw> <pr>(?)</pr>, }</mhw> <pos>n.</pos> <sn>1.</sn> <def>A bitter balsamic resin obtained from tropical American trees of the genus <gen>Elaphrium</gen> (<spn>Elaphrium tomentosum</spn> and <spn>Elaphrium Tacamahaca</spn>), and also from East Indian trees of the genus <gen>Calophyllum</gen>; also, the resinous exhudation of the balsam poplar.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Bot.)</fld> <def>Any tree yielding tacamahac resin, especially, in North America, the balsam poplar, or balm of Gilead (<spn>Populus balsamifera</spn>).</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tacaud</ent><br/
<hw>Ta*caud"</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Cf. F. <ets>tacaud</ets>. See <er>Tomcod</er>.]</ety> <fld>(Zool.)</fld> <def>The bib, or whiting pout.</def> <mark>[Prov. Eng.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tac-au-tac</ent><br/
||<hw>Tac`-au-tac"</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F., fr. <ets>riposter du tac au tac</ets> to parry (where <ets>tac</ets> imitates the sound made by the steel).]</ety> <fld>(Fencing)</fld> <def>The parry which is connected with a riposte; also, a series of quick attacks and parries in which neither fencer gains a point.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Tace</ent><br/
<hw>Tace</hw> <pr>(?)</pr>, <pos>n.</pos> <def>The cross, or church, of St. Antony. See <xex>Illust.</xex> (6), under <er>Cross</er>, <pos>n.</pos></def>  <rj><au>Mollett.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tace</ent><br/
<hw>Tace</hw>, <pos>n.</pos> <def>See <er>Tasse</er>.</def>  <rj><au>Fairholt.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tacet</ent><br/
||<hw>Ta"cet</hw> <pr>(?)</pr>, <pos>v. impers.</pos> <ety>[L., it is silent, 3d pers.pr. of <ets>tacere</ets> to be silent.]</ety> <fld>(Mus.)</fld> <def>It is silent; -- a direction for a vocal or instrumental part to be silent during a whole movement.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tache</ent><br/
<hw>Tache</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[See <er>Tack</er> a kind of nail.]</ety> <def>Something used for taking hold or holding; a catch; a loop; a button.</def> <mark>[Obs.]</mark>  <rj><au>Ex. xxvi. 6.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tache</ent><br/
<hw>Tache</hw>, <pos>n.</pos> <ety>[F. <ets>tache</ets> spot. See <er>Techy</er>.]</ety> <def>A spot, stain, or blemish.</def> <mark>[Obs.]</mark>  <rj><au>Warner.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tachhydrite</ent><br/
<hw>Tach*hy"drite</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Gr. <grk>tachy`s</grk> quick + <grk>"y`dwr</grk> water.  So named from its ready deliquescence.]</ety> <fld>(Min.)</fld> <def>A hydrous chloride of calcium and magnesium occurring in yellowish masses which rapidly deliquesce upon exposure.  It is found in the salt mines at <city>Stassfurt</city>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tachina</ent><br/
||<hw>Tach"i*na</hw> <pr>(?)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Tachinae</plw> <pr>(#)</pr>.</plu> <ety>[NL., fr. Gr. <grk>tachino`s</grk>, for <grk>tachy`s</grk> swift.]</ety> <fld>(Zool.)</fld> <def>Any one of numerous species of Diptera belonging to <gen>Tachina</gen> and allied genera.  Their larvae are external parasites of other insects.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tachistoscope</ent><br/
<hw>Ta*chis"to*scope</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Gr. <grk>ta`chistos</grk>, superl. of <grk>tachy`s</grk> swift + <ets>-scope</ets>.]</ety> <fld>(Physiol.)</fld> <def>An apparatus for exposing briefly to view a screen bearing letters or figures.  It is used in studying the range of attention, or the power of distinguishing separate objects in a single impression.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Tachograph</ent><br/
<hw>Tach"o*graph</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Gr. <grk>ta`chos</grk> speed + <ets>-graph</ets>.]</ety> <def>A recording or registering tachometer; also, its autographic record.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Tachometer</ent><br/
<hw>Ta*chom"e*ter</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Gr. <grk>ta`chos</grk> swiftness, speed (fr. <grk>tachy`s</grk> quick) + <ets>-meter</ets>: cf. F. <ets>tachom<egrave/tre</ets>.]</ety> <def>An instrument for measuring the velocity, or indicating changes in the velocity, of a moving body or substance.</def> Specifically: --<br/
[<source>1913 Webster</source>]</p>

<p><sd>(a)</sd> <def>An instrument for measuring the velocity of running water in a river or canal, consisting of a wheel with inclined vanes, which is turned by the current.  The rotations of the wheel are recorded by clockwork.</def><br/
[<source>1913 Webster</source>]</p>

<p><sd>(b)</sd> <def>An instrument for showing at any moment the speed of a revolving shaft, consisting of a delicate revolving conical pendulum which is driven by the shaft, and the action of which by change of speed moves a pointer which indicates the speed on a graduated dial.</def><br/
[<source>1913 Webster</source>]</p>

<p><sd>(c)</sd> <fld>(Physiol.)</fld> <def>An instrument for measuring the velocity of the blood; a haematachometer.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tachometry</ent><br/
<hw>Ta*chom"e*try</hw> <pr>(?)</pr>, <pos>n.</pos> <def>Measurement by a tachometer; the science or use of tachometers.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Tachydidaxy</ent><br/
<hw>Tach"y*di*dax`y</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Gr. <grk>tachy`s</grk> quick + <grk>di`daxis</grk> teaching.]</ety> <def>A short or rapid method of instructing.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tachyglossa</ent><br/
||<hw>Tach`y*glos"sa</hw> <pr>(?)</pr>, <pos>n. pl.</pos> <ety>[NL., fr. Gr. <grk>tachy`s</grk> quick + <grk>glw^ssa</grk> tongue.]</ety> <fld>(Zool.)</fld> <def>A division of monotremes which comprises the spiny ant-eaters of Australia and New Guinea. See <xex>Illust.</xex> under <er>Echidna</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tachygraph</ent><br/
<hw>Tach"y*graph</hw> <pr>(?)</pr>, <pos>n.</pos> <def>An example of tachygraphy; esp., an ancient Greek or Roman tachygraphic manuscript.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Tachygrapher</ent><br/
<hw>Ta*chyg"ra*pher</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Gr. <?/ a fast writer.]</ety> <def>One who writes shorthand; a stenographer; esp., an ancient Greek or Roman notary.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Tachygraphical</ent><br/
<ent>Tachygraphic</ent><br/
<mhw>{ <hw>Tach`y*graph"ic</hw> <pr>(?)</pr>, <hw>Tach`y*graph"ic*al</hw> <pr>(?)</pr>, }</mhw> <pos>a.</pos> <ety>[Cf. F. <ets>tachygraphique</ets>.]</ety> <def>Of or pertaining to tachygraphy; written in shorthand.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tachygraphy</ent><br/
<hw>Ta*chyg"ra*phy</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Gr. <grk>tachy`s</grk> quick + <ets>-graphy</ets>: cf. F. <ets>tachygraphie</ets>.]</ety> <def>The art or practice of rapid writing; shorthand writing; stenography.</def>  <rj><au>I. Taylor (The Alphabet).</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tachylyte</ent><br/
<hw>Tach"y*lyte</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Gr. <grk>tachy`s</grk> quick + <grk>ly`ein</grk> to dissolve.]</ety> <fld>(Min.)</fld> <def>A vitreous form of basalt; -- so called because decomposable by acids and readily fusible.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tachymeter</ent><br/
<hw>Ta*chym"e*ter</hw> <pr>(?)</pr>, <pos>n.</pos>  <ety>[<ets>Tachy-</ets> + <ets>-meter</ets>.]</ety> <sn>1.</sn> <fld>(Surveying)</fld> <def>An instrument, esp. a transit or theodolite with stadia wires, for determining quickly the distances, bearings, and elevations of distant objects.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><sn>2.</sn>  <def>A speed indicator; a tachometer.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Tachymetry</ent><br/
<hw>Ta*chym"e*try</hw> <pr>(?)</pr>, <pos>n.</pos> <def>The science or use of the tachymeter.</def> -- <wordforms><wf>Ta`chy*met"ric</wf> <pr>(#)</pr>, <pos>a.</pos></wordforms><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Tachyscope</ent><br/
<hw>Tach"y*scope</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Gr. <grk>tachy`s</grk> quick + <ets>-scope</ets>.]</ety> <def>An early form of animated-picture machine, devised in 1889 by <person>Otto Ansch<uum/tz</person> of <city>Berlin</city>, in which the chronophotographs were mounted upon the periphery of a rotating wheel.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Tacit</ent><br/
<hw>Tac"it</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>tacitus</ets>, p. p. of <ets>tacere</ets> to be silent, to pass over in silence; akin to Goth. <ets><thorn/ahan</ets> to be silent, Icel. <ets><thorn/egja</ets>, OHG. <ets>dag<emac/n</ets>: cf. F. <ets>tacite</ets>.  Cf. <er>Reticent</er>.]</ety> <def>Done or made in silence; implied, but not expressed; silent; <as>as, <ex>tacit</ex> consent is consent by silence, or by not interposing an objection</as>.</def> -- <wordforms><wf>Tac"it*ly</wf>, <pos>adv.</pos></wordforms><br/
[<source>1913 Webster</source>]</p>

<p><q>The <qex>tacit</qex> and secret theft of abusing our brother in civil contracts.</q> <rj><qau>Jer. Taylor.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Taciturn</ent><br/
<hw>Tac"i*turn</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>taciturnus</ets>: cf. F. <ets>taciturne</ets>. See <er>Tacit</er>.]</ety> <def>Habitually silent; not given to converse; not apt to talk or speak.</def> -- <wordforms><wf>Tac"i*turn*ly</wf>, <pos>adv.</pos></wordforms><br/
[<source>1913 Webster</source>]</p>

<p><syn><b>Syn.</b> -- Silent; reserved.</syn> <usage> <er>Taciturn</er>, <er>Silent</er>. <xex>Silent</xex> has reference to the act; <xex>taciturn</xex>, to the habit.  A man may be <xex>silent</xex> from circumstances; he is <xex>taciturn</xex> from disposition.  The loquacious man is at times <xex>silent</xex>; one who is <xex>taciturn</xex> may now and then make an effort at conversation.</usage><br/
[<source>1913 Webster</source>]</p>

<p><ent>Taciturnity</ent><br/
<hw>Tac`i*tur"ni*ty</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>taciturnitas</ets>: cf. F. <ets>taciturnit<eacute/</ets>.]</ety> <def>Habitual silence, or reserve in speaking.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The cause of Addison's <qex>taciturnity</qex> was a natural diffidence in the company of strangers.</q> <rj><qau>V. Knox.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>The <qex>taciturnity</qex> and the short answers which gave so much offense.</q> <rj><qau>Macaulay.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tack</ent><br/
<hw>Tack</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[From an old or dialectal form of F. <ets>tache</ets>. See <er>Techy</er>.]</ety> <sn>1.</sn> <def>A stain; a tache.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <ety>[Cf. L. <ets>tactus</ets>.]</ety> <def>A peculiar flavor or taint; <as>as, a musty <ex>tack</ex></as>.</def> <mark>[Obs. or Colloq.]</mark>  <rj><au>Drayton.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tack</ent><br/
<hw>Tack</hw>, <pos>n.</pos> <ety>[OE. <ets>tak</ets>, <ets>takke</ets>, a fastening; akin to D. <ets>tak</ets> a branch, twig, G. <ets>zacke</ets> a twig, prong, spike, Dan. <ets>takke</ets> a tack, spike; cf. also Sw. <ets>tagg</ets> prickle, point, Icel. <ets>t<amac/g</ets> a willow twig, Ir. <ets>taca</ets> a peg, nail, fastening, Gael. <ets>tacaid</ets>, Armor. & Corn. <ets>tach</ets>; perhaps akin to E. <ets>take</ets>.  Cf. <er>Attach</er>, <er>Attack</er>, <er>Detach</er>, <er>Tag</er> an end, <er>Zigzag</er>.]</ety> <sn>1.</sn> <def>A small, short, sharp-pointed nail, usually having a broad, flat head.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>That which is attached; a supplement; an appendix. See <er>Tack</er>, <pos>v. t.</pos>, 3.</def>  <rj><au>Macaulay.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Some <qex>tacks</qex> had been made to money bills in King Charles's time.</q> <rj><qau>Bp. Burnet.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><-- p. 1468 --><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <fld>(Naut.)</fld> <sd>(a)</sd> <def>A rope used to hold in place the foremost lower corners of the courses when the vessel is closehauled (see <xex>Illust.</xex> of <er>Ship</er>); also, a rope employed to pull the lower corner of a studding sail to the boom.</def> <sd>(b)</sd> <def>The part of a sail to which the tack is usually fastened; the foremost lower corner of fore-and-aft sails, as of schooners (see <xex>Illust.</xex> of <er>Sail</er>).</def> <sd>(c)</sd> <def>The direction of a vessel in regard to the trim of her sails; <as>as, the starboard <ex>tack</ex>, or port <ex>tack</ex></as>; -- the former when she is closehauled with the wind on her starboard side; hence, the run of a vessel on one tack; also, a change of direction; <as>as, to take a different <ex>tack</ex>; -- often used metaphorically</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <fld>(Scots Law)</fld> <def>A contract by which the use of a thing is set, or let, for hire; a lease.</def>  <rj><au>Burrill.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <def>Confidence; reliance.</def> <mark>[Prov. Eng.]</mark>  <rj><au>Halliwell.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Tack of a flag</b></col> <fld>(Naut.)</fld>, <cd>a line spliced into the eye at the foot of the hoist for securing the flag to the halyards.</cd> -- <col><b>Tack pins</b></col> <fld>(Naut.)</fld>, <cd>belaying pins; -- also called <altname>jack pins</altname>.</cd> -- <col><b>To haul the tacks aboard</b></col> <fld>(Naut.)</fld>, <cd>to set the courses.</cd> -- <col><b>To hold tack</b></col>, <cd>to last or hold out.</cd> <au>Milton.</au></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tack</ent><br/
<hw>Tack</hw> <pr>(?)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Tacked</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Tacking</conjf>.]</vmorph> <ety>[Cf. OD. <ets>tacken</ets> to touch, take, seize, fix, akin to E. <ets>take</ets>. See <er>Tack</er> a small nail.]</ety> <sn>1.</sn> <def>To fasten or attach.</def> <ldquo/In hopes of getting some commendam <xex>tacked</xex> to their sees.<rdquo/  <rj><au>Swift.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>And <qex>tacks</qex> the center to the sphere.</q> <rj><qau>Herbert.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Especially, to attach or secure in a slight or hasty manner, as by stitching or nailing; <as>as, to <ex>tack</ex> together the sheets of a book; to <ex>tack</ex> one piece of cloth to another; to <ex>tack</ex> on a board or shingle; to <ex>tack</ex> one piece of metal to another by drops of solder.</as></def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>In parliamentary usage, to add (a supplement) to a bill; to append; -- often with <xex>on</xex> or <xex>to</xex>; <as>as, to <ex>tack</ex> on a non-germane appropriation to a bill</as>.</def>  <rj><au>Macaulay.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <fld>(Naut.)</fld> <def>To change the direction of (a vessel) when sailing closehauled, by putting the helm alee and shifting the tacks and sails so that she will proceed to windward nearly at right angles to her former course.</def><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ In tacking, a vessel is brought to point at first directly to windward, and then so that the wind will blow against the other side.</note><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tack</ent><br/
<hw>Tack</hw>, <pos>v. i.</pos> <fld>(Naut.)</fld> <def>To change the direction of a vessel by shifting the position of the helm and sails; also (as said of a vessel), to have her direction changed through the shifting of the helm and sails. See <er>Tack</er>, <pos>v. t.</pos>, 4.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Monk, . . . when he wanted his ship to <qex>tack</qex> to larboard, moved the mirth of his crew by calling out, <ldquo/Wheel to the left.<rdquo/</q> <rj><qau>Macaulay.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tacker</ent><br/
<hw>Tack"er</hw> <pr>(?)</pr>, <pos>n.</pos> <def>One who tacks.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tacket</ent><br/
<hw>Tack"et</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Dim. of <ets>tack</ets> a small nail.]</ety> <def>A small, broad-headed nail.</def> <mark>[Scot.]</mark>  <rj><au>Jamieson.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tackey</ent><br/
<hw>Tack"ey</hw> <pr>(?)</pr>, <pos>a. & n.</pos> <def>See <er>Tacky</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tacking</ent><br/
<hw>Tack"ing</hw>, <pos>n.</pos> <fld>(Law)</fld> <def>A union of securities given at different times, all of which must be redeemed before an intermediate purchaser can interpose his claim.</def>  <rj><au>Bouvier.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ The doctrine of <xex>tacking</xex> is not recognized in American law.</note>  <rj><au>Kent.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tackle</ent><br/
<hw>Tac"kle</hw> <pr>(?; sometimes improperly pronounced ?, especially by seamen)</pr>, <pos>n.</pos> <ety>[OE. <ets>takel</ets>, akin to LG. & D. <ets>takel</ets>, Dan. <ets>takkel</ets>, Sw. <ets>tackel</ets>; perhaps akin to E. <ets>taw</ets>, v. t., or to <ets>take</ets>.]</ety> <sn>1.</sn> <def>Apparatus for raising or lowering heavy weights, consisting of a rope and pulley blocks; sometimes, the rope and attachments, as distinct from the block, in which case the full appratus is referred to as a <altname>block and tackle</altname>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Any instruments of action; an apparatus by which an object is moved or operated; gear; <as>as, fishing <ex>tackle</ex>, hunting <ex>tackle</ex></as>; formerly, specifically, weapons.</def> <ldquo/She to her <xex>tackle</xex> fell.<rdquo/  <rj><au>Hudibras.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ In Chaucer, it denotes usually an arrow or arrows.</note><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <fld>(Naut.)</fld> <def>The rigging and apparatus of a ship; also, any purchase where more than one block is used.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Fall and tackle</b></col>. <cd>See the Note under <er>Pulley</er>.</cd> -- <col><b>Fishing tackle</b></col>. <cd>See under <er>Fishing</er>, <pos>a.</pos></cd> -- <col><b>Ground tackle</b></col> <fld>(Naut.)</fld>, <cd>anchors, cables, etc.</cd> -- <col><b>Gun tackle</b></col>, <cd>the apparatus or appliances for hauling cannon in or out.</cd> -- <col><b>Tackle fall</b></col>, <cd>the rope, or rather the end of the rope, of a tackle, to which the power is applied.</cd> -- <col><b>Tack tackle</b></col> <fld>(Naut.)</fld>, <cd>a small tackle to pull down the tacks of the principal sails.</cd> -- <mcol><col><b>Tackle board</b></col>, <col><b>Tackle post</b></col></mcol> <fld>(Ropemaking)</fld>, <cd>a board, frame, or post, at the end of a ropewalk, for supporting the spindels, or whirls, for twisting the yarns.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>tackle</ent><br/
<hw>tac"kle</hw> <pos>n.</pos> <fld>(Football)</fld> <sn>1.</sn> <def>An act of tackling{4}; <as>as, brought down by a <ex>tackle</ex> by a lineman</as>.</def><br/
[<source>PJC</source>]</p>

<p><sn>2.</sn>  <fld>(Football)</fld> <def>One of two linemen on a football team, occupying a position between the guard and an end; also, the position played by such a tackle.</def><br/
[<source>PJC</source>]</p>

<p><ent>Tackle</ent><br/
<hw>Tac"kle</hw>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Tackled</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Tackling</conjf>.]</vmorph> <ety>[Cf. LG. <ets>takeln</ets> to equip. See <er>Tackle</er>, <pos>n.</pos>]</ety> <sn>1.</sn> <def>To supply with tackle.</def>  <rj><au>Beau. & Fl.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To fasten or attach, as with a tackle; to harness; <as>as, to <ex>tackle</ex> a horse into a coach or wagon</as>.</def> <mark>[Colloq.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>To seize; to lay hold of; to grapple; <as>as, a wrestler <ex>tackles</ex> his antagonist; a dog <ex>tackles</ex> the game.</as></def><br/
[<source>1913 Webster</source>]</p>

<p><q>The greatest poetess of our day has wasted her time and strength in <qex>tackling</qex> windmills under conditions the most fitted to insure her defeat.</q> <rj><qau>Dublin Univ. Mag.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <fld>(Football)</fld> <def>To cause the ball carrier to fall to the ground, thus ending the forward motion of the ball and the play.</def><br/
[<source>PJC</source>]</p>

<p><sn>5.</sn> <def>To begin to deal with; <as>as, to <ex>tackle</ex> the problem</as>.</def><br/
[<source>PJC</source>]</p>

<p><ent>Tackled</ent><br/
<hw>Tac"kled</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Made of ropes tacked together.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>My man shall be with thee,<br/
And bring thee cords made like a <qex>tackled</qex> stair.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tackling</ent><br/
<hw>Tac"kling</hw>, <pos>n.</pos> <fld>(Naut.)</fld> <sn>1.</sn> <def>Furniture of the masts and yards of a vessel, as cordage, sails, etc.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Instruments of action; <as>as, fishing <ex>tackling</ex></as>.</def>  <rj><au>Walton.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>The straps and fixures adjusted to an animal, by which he draws a carriage, or the like; harness.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tacksman</ent><br/
<hw>Tacks"man</hw> <pr>(?)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Tacksmen</plw> <pr>(?)</pr>.</plu> <fld>(Scots Law)</fld> <def>One who holds a tack or lease from another; a tenant, or lessee.</def>  <rj><au>Sir W. Scott.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>The <qex>tacksmen</qex>, who formed what may be called the <ldquo/peerage<rdquo/ of the little community, must be the captains.</q> <rj><qau>Macaulay.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tacky</ent><br/
<hw>Tack"y</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[Cf. <er>Techy</er>, <er>Tack</er> a spot.]</ety> <def>Sticky; somewhat adhesive;; -- said of paint, varnish, etc., when not well dried.</def> <mark>[U. S.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tacky</ent><br/
<hw>Tack"y</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[Etymol. uncert.]</ety> <sn>1.</sn> <def>Dowdy, shabby, or neglected in appearance; unkempt.</def> <mark>[Local, U. S.]</mark><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><sn>2.</sn> <def>In poor taste; appearing cheap; gaudy; unstylish.  Broadly used to describe objects whose style is disapproved of by the speaker.</def><br/
[<source>PJC</source>]</p>

<p><sn>3.</sn> <def>Tactless; in poor taste; -- used to describe behavior.</def><br/
[<source>PJC</source>]</p>

<p><ent>Tacky</ent><br/
<hw>Tack"y</hw>, <pos>n.</pos> <altsp>[Written also <asp>tackey</asp>.]</altsp> <def>An ill-conditioned, ill-fed, or neglected horse; also, a person in a like condition.</def> <mark>[Southern U. S.]</mark><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Taconic</ent><br/
<hw>Ta*con"ic</hw> <pr>(?)</pr>, <pos>a.</pos> <fld>(Geol.)</fld> <def>Designating, or pertaining to, the series of rocks forming the <etsep>Taconic</etsep> mountains in Western New England.  They were once supposed to be older than the Cambrian, but later proved to belong to the Lower Silurian and Cambrian.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tact</ent><br/
<hw>Tact</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>tactus</ets> a touching, touch, fr. <ets>tangere</ets>, <ets>tactum</ets>, to touch: cf. F. <ets>tact</ets>. See <er>Tangent</er>.]</ety> <sn>1.</sn> <def>The sense of touch; feeling.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Did you suppose that I could not make myself sensible to <qex>tact</qex> as well as sight?</q> <rj><qau>Southey.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Now, sight is a very refined <qex>tact</qex>.</q> <rj><qau>J. Le Conte.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Mus.)</fld> <def>The stroke in beating time.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>Sensitive mental touch; peculiar skill or faculty; nice perception or discernment; ready power of appreciating and doing what is required by circumstances.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>He had formed plans not inferior in grandeur and boldness to those of Richelieu, and had carried them into effect with a <qex>tact</qex> and wariness worthy of Mazarin.</q> <rj><qau>Macaulay.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>A <qex>tact</qex> which surpassed the <qex>tact</qex> of her sex as much as the <qex>tact</qex> of her sex surpassed the <qex>tact</qex> of ours.</q> <rj><qau>Macaulay.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tactable</ent><br/
<hw>Tac"ta*ble</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Capable of being touched; tangible.</def> <mark>[R.]</mark> <ldquo/They [women] being created to be both tractable and <xex>tactable</xex>.<rdquo/  <rj><au>Massinger.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tactful</ent><br/
<hw>Tact"ful</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Full of tact; characterized by a discerning sense of what is right, proper, or judicious.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Tactical</ent><br/
<ent>Tactic</ent><br/
<mhw>{ <hw>Tac"tic</hw> <pr>(?)</pr>, <hw>Tac"tic*al</hw> <pr>(?)</pr>, }</mhw> <pos>a.</pos> <ety>[Gr. <grk>taktiko`s</grk>. See <er>tactics</er>.]</ety> <def>Of or pertaining to military or naval tactics; hence, pertaining to, or characterized by, planning or maneuvering for the short term; -- contrasted with <contr>strategic</contr>, planning for the long term.</def><br/ -- <wordforms><wf>Tac"tic*al*ly</wf>, <pos>adv.</pos></wordforms><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Tactic</ent><br/
<hw>Tac"tic</hw> <pr>(?)</pr>, <pos>n.</pos> <def>See <er>Tactics</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tactician</ent><br/
<hw>Tac*ti"cian</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Cf. F. <ets>tacticien</ets>.]</ety> <def>One versed in tactics; hence, a skillful maneuverer; an adroit manager.</def>
<-- as, a skilled parliamentary tactician. --><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tactics</ent><br/
<hw>Tac"tics</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Gr. <grk>taktika`</grk> pl., and <grk>taktikh`</grk> (sc. <grk>te`chnh</grk>, sing., fr. <grk>taktiko`s</grk> fit for ordering or arranging, fr. <grk>ta`ssein</grk>, <grk>ta`ttein</grk>, to put in order, to arrange: cf. F. <ets>tactique</ets>.]</ety> <sn>1.</sn> <def>The science and art of disposing military and naval forces in order for battle, and performing military and naval evolutions. It is divided into <xex>grand tactics</xex>, or the tactics of battles, and <xex>elementary tactics</xex>, or the tactics of instruction.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Hence, any system or method of procedure.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tactile</ent><br/
<hw>Tac"tile</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>tactilis</ets>, fr. <ets>tangere</ets>, <ets>tactum</ets>, to touch: cf. F. <ets>tactile</ets>.]</ety> <def>Of or pertaining to the organs, or the sense, of touch; perceiving, or perceptible, by the touch; capable of being touched; <as>as, <ex>tactile</ex> corpuscles; <ex>tactile</ex> sensations.</as></def> <ldquo/<xex>Tactile</xex> sweets.<rdquo/ <au>Beaumont.</au> <ldquo/<xex>Tactile</xex> qualities.<rdquo/ <au>Sir M. Hale.</au><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Tactile</b></col><cd> sense <fld>(Physiol.)</fld>, the sense of touch, or pressure sense. See <er>Touch</er>.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><q>The delicacy of the <qex>tactile</qex> sense varies on different parts of the skin; it is geatest on the forehead, temples and back of the forearm.</q> <rj><qau>H. N. Martin.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tactility</ent><br/
<hw>Tac*til"i*ty</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Cf. F. <ets>tactilit<eacute/</ets>.]</ety> <def>The quality or state of being tactile; perceptibility by touch; tangibleness.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Taction</ent><br/
<hw>Tac"tion</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>tactio</ets>, from <ets>tangere</ets>, <ets>tactum</ets>, to touch.]</ety> <def>The act of touching; touch; contact; tangency.</def> <ldquo/External <xex>taction</xex>.<rdquo/  <rj><au>Chesterfield.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tactless</ent><br/
<hw>Tact"less</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Destitute of tact.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tactual</ent><br/
<hw>Tac"tu*al</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[See <er>Tact</er>.]</ety> <fld>(Physiol.)</fld> <def>Of or pertaining to the sense, or the organs, of touch; derived from touch.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>In the lowest organisms we have a kind of <qex>tactual</qex> sense diffused over the entire body.</q> <rj><qau>Tyndall.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tadpole</ent><br/
<hw>Tad"pole`</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[OE. <ets>tadde</ets> toad (AS. <ets>t<amac/die</ets>, <ets>t<amac/dige</ets>) + <ets>poll</ets>; properly, a toad that is or seems all head. See <er>Toad</er>, and <er>Poll</er>.]</ety> <sn>1.</sn> <fld>(Zool.)</fld> <def>The young aquatic larva of any amphibian. In this stage it breathes by means of external or internal gills, is at first destitute of legs, and has a finlike tail. Called also <altname>polliwig</altname>, <altname>polliwog</altname>, <altname>porwiggle</altname>, or <altname>purwiggy</altname>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Zool.)</fld> <def>The hooded merganser.</def> <mark>[Local, U. S.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Tadpole fish</b></col>. <fld>(Zool.)</fld> <cd>See <er>Forkbeard</er> <sd>(a)</sd>.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Taedium</ent><br/
||<hw>Tae"di*um</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L.]</ety> <def>See <er>Tedium</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tael</ent><br/
<hw>Tael</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Malay <ets>ta<icr/l</ets>, a certain weight, probably fr. Hind. <ets>tola</ets>, Skr. <ets>tul<amac/</ets> a balance, weight, <ets>tul</ets> to weigh.]</ety> <def>A denomination of money, in China, worth nearly six shillings sterling, or about a dollar and forty cents; also, a weight of one ounce and a third.</def> <altsp>[Written also <asp>tale</asp>.]</altsp><br/
[<source>1913 Webster</source>]</p>

<p><ent>Ta'en</ent><br/
<ent>Taen</ent><br/
<mhw>{ <hw>Taen</hw> <pr>(?)</pr>, <it>or</it> <hw>Ta'en</hw>  }</mhw>, <def><pos>p. p.</pos> of <er>Ta</er>, to take, or a contraction of <er>Taken</er>.</def> <mark>[Poetic & Scot.]</mark>  <rj><au>Burns.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Taenia</ent><br/
||<hw>Tae"ni*a</hw> <pr>(?)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Taeniae</plw> <pr>(#)</pr>.</plu> <ety>[L., a ribbon, a tapeworm.]</ety> <sn>1.</sn> <fld>(Zool.)</fld> <def>A genus of intestinal worms which includes the common tapeworms of man. See <er>Tapeworm</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Anat.)</fld> <def>A band; a structural line; -- applied to several bands and lines of nervous matter in the brain.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <fld>(Arch.)</fld> <def>The fillet, or band, at the bottom of a Doric frieze, separating it from the architrave.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Teniacide</ent><br/
<ent>Taeniacide</ent><br/
<mhw>{ <hw>Tae"ni*a*cide`</hw> <pr>(?)</pr>, <pos>n.</pos> Also <hw>Te"ni*a*cide`</hw>  }</mhw>. <ety>[<ets>Taenia</ets> + <ets>-cide</ets>.]</ety> <fld>(Med.)</fld> <def>A remedy to destroy tapeworms.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Taeniada</ent><br/
||<hw>Tae*ni"a*da</hw> <pr>(?)</pr>, <pos>n. pl.</pos> <ety>[NL.]</ety> <fld>(Zool.)</fld> <def>Same as <er>Taenioidea</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Teniafuge</ent><br/
<ent>Taeniafuge</ent><br/
<mhw>{ <hw>Tae"ni*a*fuge`</hw> <pr>(?)</pr>, <pos>n.</pos> Also <hw>Te"ni*a*fuge`</hw> }</mhw>. <ety>[<ets>Taenia</ets> + L. <ets>fugare</ets> to drive away.]</ety> <fld>(Med.)</fld> <def>A remedy to expel tapeworms.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Teniasis</ent><br/
<ent>Taeniasis</ent><br/
<mhw>{ ||<hw>Tae*ni"a*sis</hw> <pr>(?)</pr>, <pos>n.</pos> Also ||<hw>Te*ni"a*sis</hw> }</mhw>. <ety>[NL. See <er>Taenia</er>.]</ety> <fld>(Med.)</fld> <def>Ill health due to taenia, or tapeworms.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Taeniata</ent><br/
||<hw>Tae`ni*a"ta</hw> <pr>(?)</pr>, <pos>n. pl.</pos> <ety>[NL., fr. L. <ets>taenia</ets> a ribbon.]</ety> <fld>(Zool.)</fld> <def>A division of Ctenophora including those which have a long, ribbonlike body. The Venus's girdle is the most familiar example.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Taenidium</ent><br/
||<hw>Tae*nid"i*um</hw> <pr>(?)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Taenidia</plw> <pr>(#)</pr>.</plu> <ety>[NL., dim. fr. L. <ets>taenia</ets> a ribbon.]</ety> <fld>(Zool.)</fld> <def>The chitinous fiber forming the spiral thread of the tracheae of insects. See <xex>Illust.</xex> of <er>Trachea</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Taenioglossa</ent><br/
||<hw>Tae`ni*o*glos"sa</hw> <pr>(?)</pr>, <pos>n. pl.</pos> <ety>[NL., fr. Gr. <grk>taini`a</grk> a ribbon + <grk>glw^ssa</grk> a tongue.]</ety> <fld>(Zool.)</fld> <def>An extensive division of gastropod mollusks in which the odontophore is long and narrow, and usually bears seven rows of teeth.  It includes a large number of families both marine and fresh-water.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Taenioglossate</ent><br/
<hw>Tae`ni*o*glos"sate</hw> <pr>(?)</pr>, <pos>a.</pos> <fld>(Zool.)</fld> <def>Of or pertaining to the Taenioglossa.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Taenioid</ent><br/
<hw>Tae"ni*oid</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[<ets>Taenia</ets> + <ets>-oid</ets>.]</ety> <sn>1.</sn> <def>Ribbonlike; shaped like a ribbon.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Zool.)</fld> <def>Like or pertaining to Taenia.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Taenioidea</ent><br/
||<hw>Tae`ni*oi"de*a</hw> <pr>(?)</pr>, <pos>n. pl.</pos> <fld>(Zool.)</fld> <def>The division of cestode worms which comprises the tapeworms. See <er>Tapeworm</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Taeniola</ent><br/
||<hw>Tae*ni"o*la</hw> <pr>(?)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Taeniolae</plw> <pr>(#)</pr>.</plu> <ety>[L., dim. of <ets>taenia</ets> a ribbon.]</ety> <fld>(Zool.)</fld> <def>One of the radial partitions which separate the internal cavities of certain medusae.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Taeniosomi</ent><br/
||<hw>Tae`ni*o*so"mi</hw> <pr>(?)</pr>, <pos>n. pl.</pos> <ety>[NL., fr. Gr. <grk>taini`a</grk> ribbon + <grk>sw^ma</grk> body.]</ety> <fld>(Zool.)</fld> <def>An order of fishes remarkable for their long and compressed form. The ribbon fishes are examples. See <cref>Ribbon fish</cref>, under <er>Ribbon</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tafferer</ent><br/
<hw>Taf"fer*er</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Naut.)</fld> <def>See <er>Taffrail</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Taffety</ent><br/
<ent>Taffeta</ent><br/
<mhw>{ <hw>Taf"fe*ta</hw> <pr>(?)</pr>, <hw>Taf"fe*ty</hw> <pr>(?)</pr>, }</mhw> <pos>n.</pos> <ety>[F. <ets>taffetas</ets>, It. <ets>taffet<agrave/</ets>, from Per. <ets>t<amac/ftah</ets>, originally, twisted, woven, from <ets>t<amac/ftan</ets> to twist, to spin.]</ety> <def>A fine, smooth stuff of silk, having usually the wavy luster called <xex>watering</xex>. The term has also been applied to different kinds of silk goods, from the 16th century to modern times.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Lined with <qex>taffeta</qex> and with sendal.</q> <rj><qau>Chaucer.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Taffrail</ent><br/
<hw>Taff"rail</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[D. <ets>tafereel</ets> a panel, picture, fr. <ets>tafel</ets> table, fr. L. <ets>tabula</ets>. See <er>Table</er>.]</ety> <fld>(Naut.)</fld> <def>The upper part of a ship's stern, which is flat like a table on the top, and sometimes ornamented with carved work; the rail around a ship's stern.</def> <altsp>[Written also <asp>tafferel</asp>.]</altsp><br/
[<source>1913 Webster</source>]</p>

<p><ent>Taffy</ent><br/
<hw>Taf"fy</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Prov. E. <ets>taffy</ets> toffy.]</ety> <sn>1.</sn> <def>A kind of candy made of molasses or brown sugar boiled down and poured out in shallow pans.</def> <altsp>[Written also, in England, <asp>toffy</asp>.]</altsp><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Flattery; soft phrases.</def> <mark>[Slang]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tafia</ent><br/
<hw>Taf"i*a</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Cf. F. & Sp. <ets>tafia</ets>, It. <ets>taffia</ets>; fr. Malay <ets>t<amac/f<imac/a</ets> a spirit distilled from molasses.  Cf. <er>Ratafia</er>.]</ety> <def>A variety of rum.</def> <mark>[West Indies]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tag</ent><br/
<hw>Tag</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Probably akin to <ets>tack</ets> a small nail; cf. Sw. <ets>tagg</ets> a prickle, point, tooth.]</ety> <sn>1.</sn> <def>Any slight appendage, as to an article of dress; something slight hanging loosely; specifically, a direction card, or label.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A metallic binding, tube, or point, at the end of a string, or lace, to stiffen it.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>The end, or catchword, of an actor's speech; cue.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>Something mean and paltry; the rabble.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Tag and rag</b></col>, <cd>the lowest sort; the rabble.</cd> <au>Holinshed.</au></cs><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <def>A sheep of the first year.</def> <mark>[Prov. Eng.]</mark>  <rj><au>Halliwell.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tag</ent><br/
<hw>Tag</hw>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Tagged</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Tagging</conjf> <pr>(?)</pr>.]</vmorph> <sn>1.</sn> <def>To fit with, or as with, a tag or tags.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>He learned to make long-<qex>tagged</qex> thread laces.</q> <rj><qau>Macaulay.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>His courteous host . . . <br/
<qex>Tags</qex> every sentence with some fawning word.</q> <rj><qau>Dryden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To join; to fasten; to attach.</def>  <rj><au>Bolingbroke.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>To follow closely after; esp., to follow and touch in the game of tag. See <er>Tag</er>, a play.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tag</ent><br/
<hw>Tag</hw>, <pos>v. i.</pos> <def>To follow closely, as it were an appendage; -- often with <xex>after</xex>; <as>as, to <ex>tag</ex> after a person</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tag</ent><br/
<hw>Tag</hw>, <pos>n.</pos> <ety>[From <er>Tag</er>, <pos>v.</pos>; cf. <er>Tag</er>, an end.]</ety> <def>A child's play in which one runs after and touches another, and then runs away to avoid being touched.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tagal</ent><br/
<hw>Ta*gal"</hw> <pr>(?)</pr>, <pos>n.</pos> <sn>1.</sn> <def>One of a Malayan race, mainly of central Luzon, next to the Visayans the most numerous of the native peoples of the Philippines. Nearly all are Christians and many are highly educated.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><sn>2.</sn>  <def>The language of the Tagals; Tagalog.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Tagalog</ent><br/
<hw>Ta*ga"log</hw> <pr>(?)</pr>, <pos>n.</pos> <sn>1.</sn> <fld>(Ethnol.)</fld> <def>Any member of a certain tribe which is one of the leading and most civilized of those native of the Philippine Islands.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><sn>2.</sn>  <def>The language of the Tagalogs. It belongs to the Malay family of languages and is one of the most highly developed members of the family.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Tagbelt</ent><br/
<hw>Tag"belt`</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Far.)</fld> <def>Same as <er>Tagsore</er>.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tag day</ent><br/
<hw>Tag day</hw>. <def>A day on which contributions to some public or private charity or fund are solicited promiscuously on the street, and tags given to contributors to wear as an evidence of their having contributed. Such solicitation is now subject to legal restriction in various places.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Tagger</ent><br/
<hw>Tag"ger</hw> <pr>(?)</pr>, <pos>n.</pos> <sn>1.</sn> <def>One who, or that which, appends or joins one thing to another.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>That which is pointed like a tag.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Hedgehogs' or procupines' small <qex>taggers</qex>.</q> <rj><qau>Cotton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <pluf>pl.</pluf> <def>Sheets of tin or other plate which run below the gauge.</def>  <rj><au>Knight.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>A device for removing taglocks from sheep.</def>  <rj><au>Knight.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><-- [Colloq.] One who spray-paints a distinctive logo on a wall or other property not his own. --><br/
[<source>1913 Webster</source>]</p>

<p><ent>Taglet</ent><br/
<hw>Tag"let</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A little tag.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Taglia</ent><br/
||<hw>Tagl"ia</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[It., a cutting, a pulley, from <ets>tagliare</ets> to cut. See <er>Tailor</er>.]</ety> <fld>(Mech.)</fld> <def>A peculiar combination of pulleys.</def>  <rj><au>Brande & C.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tagliacotain</ent><br/
<hw>Tagl`ia*co"tain</hw> <pr>(?)</pr>, <pos>a.</pos> <fld>(Surg.)</fld> <def>Of or pertaining to <xex>Tagliacozzi</xex>, a Venetian surgeon; <as>as, the <ex>Tagliacotian</ex> operation, a method of rhinoplasty described by him</as>.</def> <altsp>[Also <asp>Taliacotian</asp>, and <asp>Tagliacozzian</asp>.]</altsp><br/
[<source>1913 Webster</source>]</p>

<p><ent>Taglioni</ent><br/
<hw>Tagl*io"ni</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A kind of outer coat, or overcoat; -- said to be so named after a celebrated Italian family of professional dancers.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>He ought certainly to exchange his <qex>taglioni</qex>, or comfortable greatcoat, for a cuirass of steel.</q> <rj><qau>Sir W. Scott.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Taglock</ent><br/
<hw>Tag"lock`</hw> <pr>(?)</pr>, <pos>n.</pos> <def>An entangled lock, as of hair or wool.</def>  <rj><au>Nares.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tagnicate</ent><br/
<hw>Tag"ni*cate</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Zool.)</fld> <def>The white-lipped peccary.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tag-rag</ent><br/
<hw>Tag"-rag`</hw> <pr>(?)</pr>, <pos>n. & a.</pos> <ety>[See <er>Tag</er> an end, and <er>Rag</er>.]</ety> <def>The lowest class of people; the rabble.  Cf. <cref>Rag, tag, and bobtail</cref>, under <er>Bobtail</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>If the <qex>tag-rag</qex> people did not clap him and hiss him, I am no true man.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tag sale</ent><br/
<hw>Tag sale</hw> <pr>(?)</pr>, <pos>n.</pos>  <ety>[From the price <ets>tag</ets> usually attached to each item.]</ety> <def>A sale of usually used items (such as furniture, clothing, household items or bric-a-brac), conducted by one or a small group of individuals, at a location which is not a normal retail establishment.</def> <note>Frequently it is held in the private home or in a yard attached to a private home belonging to the seller.  Similar to a <cref>yard sale</cref> or <cref>garage sale</cref>.  Compare <contr>flea market</contr>, where used items are sold by many individuals in a place rented for the purpose.</note><br/
[<source>PJC</source>]</p>

<p><ent>Tagsore</ent><br/
<hw>Tag"sore`</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Far.)</fld> <def>Adhesion of the tail of a sheep to the wool from excoriation produced by contact with the feces; -- called also <altname>tagbelt</altname>.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tagtail</ent><br/
<hw>Tag"tail`</hw> <pr>(?)</pr>, <pos>n.</pos> <sn>1.</sn> <def>A worm which has its tail conspicuously colored.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A person who attaches himself to another against the will of the latter; a hanger-on.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Taguan</ent><br/
<hw>Tag"u*an</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[From the native name in the East Indies.]</ety> <fld>(Zool.)</fld> <def>A large flying squirrel (<spn>Pteromys petuarista</spn>). Its body becomes two feet long, with a large bushy tail nearly as long.</def><br/
[<source>1913 Webster</source>]</p>

<p><-- p. 1469 --><br/
[<source>1913 Webster</source>]</p>

<p><ent>Taguicati</ent><br/
<hw>Ta`gui*ca"ti</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[From the native name.]</ety> <fld>(Zool.)</fld> <def>The white-lipped peccary.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Taha</ent><br/
<hw>Ta"ha</hw> <pr>(?)</pr>, <pos>n.</pos> <def>The African rufous-necked weaver bird (<spn>Hyphantornis texor</spn>).</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tahaleb</ent><br/
<hw>Ta*ha"leb</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[From the native name.]</ety> <fld>(Zool.)</fld> <def>A fox (<spn>Vulpes Niloticus</spn>) of Northern Africa.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tahitian</ent><br/
<hw>Ta*hi"ti*an</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Of or pertaining to Tahiti, an island in the Pacific Ocean.</def> -- <def2><pos>n.</pos> <def>A native inhabitant of Tahiti.</def></def2><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tahr</ent><br/
<hw>Tahr</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Zool.)</fld> <def>Same as <er>Thar</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tai</ent><br/
<hw>Tai</hw> <pr>(t<aum/"<esl/)</pr>, <pos>a.</pos> <def>Designating, or pertaining to, the chief linguistic stock of Indo-China, including the peoples of Siamese and Shan speech.  It includes the Thai language.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Tai</ent><br/
<hw>Tai</hw>, <pos>n.</pos> <def>A member of one of the tribes of the Tai stock.</def></p>

<p><q>The <qex>Tais</qex> first appeared in history in Yunnan, and from thence they migrated into Upper Burma. The earliest swarms appear to have entered that tract about two thousand years ago, and were small in number.</q>  <rj><qau>Census of India, 1901.</qau></rj><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Tail</ent><br/
<hw>Tail</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F. <ets>taille</ets> a cutting. See <er>Entail</er>, <er>Tally</er>.]</ety> <fld>(Law)</fld> <def>Limitation; abridgment.</def>  <rj><au>Burrill.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Estate in tail</b></col>, <cd>a limited, abridged, or reduced fee; an estate limited to certain heirs, and from which the other heirs are precluded; -- called also <altname>estate tail</altname>.</cd> <au>Blackstone.</au></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tail</ent><br/
<hw>Tail</hw>, <pos>a.</pos> <fld>(Law)</fld> <def>Limited; abridged; reduced; curtailed; <as>as, estate <ex>tail</ex></as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tail</ent><br/
<hw>Tail</hw>, <pos>n.</pos> <ety>[AS. <ets>taegel</ets>, <ets>taegl</ets>; akin to G. <ets>zagel</ets>, Icel. <ets>tagl</ets>, Sw. <ets>tagel</ets>, Goth. <ets>tagl</ets> hair. <root/59.]</ety> <sn>1.</sn> <fld>(Zool.)</fld> <def>The terminal, and usually flexible, posterior appendage of an animal.</def><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ The tail of mammals and reptiles contains a series of movable vertebrae, and is covered with flesh and hairs or scales like those of other parts of the body. The tail of existing birds consists of several more or less consolidated vertebrae which supports a fanlike group of quills to which the term <xex>tail</xex> is more particularly applied. The tail of fishes consists of the tapering hind portion of the body ending in a caudal fin. The term <xex>tail</xex> is sometimes applied to the entire abdomen of a crustacean or insect, and sometimes to the terminal piece or pygidium alone.</note><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Any long, flexible terminal appendage; whatever resembles, in shape or position, the tail of an animal, as a catkin.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Doretus writes a great praise of the distilled waters of those <qex>tails</qex> that hang on willow trees.</q> <rj><qau>Harvey.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>Hence, the back, last, lower, or inferior part of anything, -- as opposed to the <ant>head</ant>, or the superior part.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The Lord will make thee the head, and not the <qex>tail</qex>.</q> <rj><qau>Deut. xxviii. 13.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>A train or company of attendants; a retinue.</def><br/
[<source>1913 Webster</source>]</p>

<p><q><ldquo/Ah,<rdquo/ said he, <ldquo/if you saw but the chief with his <qex>tail</qex> on.<rdquo/</q> <rj><qau>Sir W. Scott.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <def>The side of a coin opposite to that which bears the head, effigy, or date; the reverse; -- rarely used except in the expression <ldquo/heads or tails,<rdquo/ employed when a coin is thrown up for the purpose of deciding some point by its fall.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>6.</sn> <fld>(Anat.)</fld> <def>The distal tendon of a muscle.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>7.</sn> <fld>(Bot.)</fld> <def>A downy or feathery appendage to certain achenes. It is formed of the permanent elongated style.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>8.</sn> <fld>(Surg.)</fld> <sd>(a)</sd> <def>A portion of an incision, at its beginning or end, which does not go through the whole thickness of the skin, and is more painful than a complete incision; -- called also <altname>tailing</altname>.</def> <sd>(b)</sd> <def>One of the strips at the end of a bandage formed by splitting the bandage one or more times.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>9.</sn> <fld>(Naut.)</fld> <def>A rope spliced to the strap of a block, by which it may be lashed to anything.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>10.</sn> <fld>(Mus.)</fld> <def>The part of a note which runs perpendicularly upward or downward from the head; the stem.</def>  <rj><au>Moore (Encyc. of Music).</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>11.</sn> <pluf>pl.</pluf> <def>Same as <er>Tailing</er>, 4.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>12.</sn> <fld>(Arch.)</fld> <def>The bottom or lower portion of a member or part, as a slate or tile.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>13.</sn> <pluf>pl.</pluf> <fld>(Mining)</fld> <def>See <er>Tailing</er>, <pos>n.</pos>, 5.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>14.</sn> <fld>(Astronomy)</fld> <def>the long visible stream of gases, ions, or dust particles extending from the head of a comet in the direction opposite to the sun.</def><br/
[<source>PJC</source>]</p>

<p><sn>15.</sn> <pluf>pl.</pluf> <fld>(Rope Making)</fld> <def>In some forms of rope-laying machine, pieces of rope attached to the iron bar passing through the grooven wooden top containing the strands, for wrapping around the rope to be laid.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><sn>16.</sn>  <pluf>pl.</pluf> <def>A tailed coat; a tail coat.</def> <mark>[Colloq. or Dial.]</mark><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><sn>17.</sn> <fld>(Aeronautics)</fld> <def>In airplanes, an airfoil or group of airfoils used at the rear to confer stability.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><sn>18.</sn> <def>the buttocks.</def> <mark>[slang or vulgar]</mark><br/
[<source>PJC</source>]</p>

<p><sn>19.</sn> <def>sexual intercourse, or a woman used for sexual intercourse; <as>as, to get some <ex>tail</ex>; to find a piece of <ex>tail</ex></as>.  See also <er>tailing{3}</er>.</def> <mark>[slang and vulgar]</mark><br/
[<source>PJC</source>]</p>

<p><cs><col><b>Tail beam</b></col>. <fld>(Arch.)</fld> <cd>Same as <er>Tailpiece</er>.</cd> -- <col><b>Tail coverts</b></col> <fld>(Zool.)</fld>, <cd>the feathers which cover the bases of the tail quills. They are sometimes much longer than the quills, and form elegant plumes. Those above the quills are called the <stype>upper tail coverts</stype>, and those below, the <stype>under tail coverts</stype>.</cd> -- <col><b>Tail end</b></col>, <cd>the latter end; the termination; <as>as, the <ex>tail end</ex> of a contest</as>.</cd> <mark>[Colloq.]</mark> -- <col><b>Tail joist</b></col>. <fld>(Arch.)</fld> <cd>Same as <er>Tailpiece</er>.</cd> -- <col><b>Tail of a comet</b></col> <fld>(Astron.)</fld>, <cd>a luminous train extending from the nucleus or body, often to a great distance, and usually in a direction opposite to the sun.</cd> -- <col><b>Tail of a gale</b></col> <fld>(Naut.)</fld>, <cd>the latter part of it, when the wind has greatly abated.</cd> <au>Totten.</au> -- <col><b>Tail of a lock</b></col> (on a canal), <cd>the lower end, or entrance into the lower pond.</cd> -- <col><b>Tail of the trenches</b></col> <fld>(Fort.)</fld>, <cd>the post where the besiegers begin to break ground, and cover themselves from the fire of the place, in advancing the lines of approach.</cd> -- <col><b>Tail spindle</b></col>, <cd>the spindle of the tailstock of a turning lathe; -- called also <altname>dead spindle</altname>.</cd> -- <col><b>To turn tail</b></col>, <cd>to run away; to flee.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><q>Would she <qex>turn tail</qex> to the heron, and fly quite out another way; but all was to return in a higher pitch.</q> <rj><qau>Sir P. Sidney.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tail</ent><br/
<hw>Tail</hw>, <pos>v. t.</pos> <sn>1.</sn> <def>To follow or hang to, like a tail; to be attached closely to, as that which can not be evaded.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>Nevertheless his bond of two thousand pounds, wherewith he was <qex>tailed</qex>, continued uncanceled, and was called on the next Parliament.</q> <rj><qau>Fuller.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To pull or draw by the tail.</def> <mark>[R.]</mark>  <rj><au>Hudibras.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><mcol><col><b>To tail in</b></col> <it>or</it> <col><b>To tail on</b></col></mcol> <fld>(Arch.)</fld>, <cd>to fasten by one of the ends into a wall or some other support; <as>as, <ex>to tail in</ex> a timber</as>.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tail</ent><br/
<hw>Tail</hw>, <pos>v. i.</pos> <sn>1.</sn> <fld>(Arch.)</fld> <def>To hold by the end; -- said of a timber when it rests upon a wall or other support; -- with <xex>in</xex> or <xex>into</xex>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Naut.)</fld> <def>To swing with the stern in a certain direction; -- said of a vessel at anchor; <as>as, this vessel <ex>tails</ex> down stream</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Tail on</b></col>. <fld>(Naut.)</fld> <cd>See <cref>Tally on</cref>, under <er>Tally</er>.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tailage</ent><br/
<hw>Tail"age</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(O. Eng. Law)</fld> <def>See <er>Tallage</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tail-bay</ent><br/
<hw>Tail"-bay`</hw> <pr>(?)</pr>, <pos>n.</pos> <sn>1.</sn> <fld>(Arch.)</fld> <def>One of the joists which rest one end on the wall and the other on a girder; also, the space between a wall and the nearest girder of a floor.  Cf. <er>Case-bay</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>The part of a canal lock below the lower gates.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tailblock</ent><br/
<hw>Tail"block`</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Naut.)</fld> <def>A block with a tail. See <er>Tail</er>, 9.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tailboard</ent><br/
<hw>Tail"board`</hw> <pr>(?)</pr>, <pos>n.</pos> <def>The board at the rear end of a cart or wagon, which can be removed or let down, for convenience in loading or unloading.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tailed</ent><br/
<hw>Tailed</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Having a tail; having (such) a tail or (so many) tails; -- chiefly used in composition; <as>as, bob<ex>tailed</ex>, long<ex>tailed</ex>, etc.</as></def><br/
[<source>1913 Webster</source>]</p>

<p><q>Snouted and <qex>tailed</qex> like a boar.</q> <rj><qau>Grew.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tailing</ent><br/
<hw>Tail"ing</hw> <pr>(?)</pr>, <pos>n.</pos> <sn>1.</sn> <fld>(Arch.)</fld> <def>The part of a projecting stone or brick inserted in a wall.</def>  <rj><au>Gwilt.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Surg.)</fld> <def>Same as <er>Tail</er>, <pos>n.</pos>, 8 <sd>(a)</sd>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>Sexual intercourse.</def> <mark>[Obs.]</mark>  <rj><au>Chaucer.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <pluf>pl.</pluf> <def>The lighter parts of grain separated from the seed threshing and winnowing; chaff.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <pluf>pl.</pluf> <fld>(Mining)</fld> <def>The refuse part of stamped ore, thrown behind the tail of the buddle or washing apparatus. It is dressed over again to secure whatever metal may exist in it.  Called also <altname>tails</altname>.</def>  <rj><au>Pryce.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>6.</sn> <fld>(Elec.)</fld> <def>A prolongation of current in a telegraph line, due to capacity in the line and causing signals to run together.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Taille</ent><br/
<hw>Taille</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F. See <er>Tally</er>, <er>Tailor</er>.]</ety> <sn>1.</sn> <def>A tally; an account scored on a piece of wood.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>Whether that he paid or took by <qex>taille</qex>.</q> <rj><qau>Chaucer.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(O. F. Law)</fld> <def>Any imposition levied by the king, or any other lord, upon his subjects.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The <qex>taille</qex>, as it still subsists in France, may serve as an example of those ancient tallages. It was a tax upon the profits of the farmer, which they estimate by the stock that he has upon the farm.</q> <rj><qau>A. Smith.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <fld>(Mus.)</fld> <def>The French name for the tenor voice or part; also, for the tenor viol or viola.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tailless</ent><br/
<hw>Tail"less</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Having no tail.</def>  <rj><au>H. Spencer.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Taillie</ent><br/
<hw>Tail"lie</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Scots Law)</fld> <def>Same as <er>Tailzie</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tailor</ent><br/
<hw>Tai"lor</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[OF. <ets>tailleor</ets>, F. <ets>tailleur</ets>, fr. OF. <ets>taillier</ets>, F. <ets>tailler</ets> to cut, fr. L. <ets>talea</ets> a rod, stick, a cutting, layer for planting.  Cf. <er>Detail</er>, <er>Entail</er>, <er>Retail</er>, <er>Tally</er>, <pos>n.</pos>]</ety> <sn>1.</sn> <def>One whose occupation is to cut out and make men's garments; also, one who cuts out and makes ladies' outer garments.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Well said, good woman's <qex>tailor</qex> . . . I would thou wert a man's <qex>tailor</qex>.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Zool.)</fld> <sd>(a)</sd> <def>The mattowacca; -- called also <altname>tailor herring</altname>.</def> <sd>(b)</sd> <def>The silversides.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <fld>(Zool.)</fld> <def>The goldfish.</def> <mark>[Prov. Eng.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Salt-water tailor</b></col> <fld>(Zool.)</fld>, <cd>the bluefish.</cd> <mark>[Local, U. S.]</mark> <au>Bartlett.</au> -- <col><b>Tailor bird</b></col> <fld>(Zool.)</fld>, <cd>any one of numerous species of small Asiatic and East Indian singing birds belonging to <gen>Orthotomus</gen>, <gen>Prinia</gen>, and allied genera.  They are noted for the skill with which they sew leaves together to form nests.  The common Indian species are <spn>Orthotomus longicauda</spn>, which has the back, scapulars, and upper tail coverts yellowish green, and the under parts white; and the golden-headed tailor bird (<spn>Orthotomus coronatus</spn>), which has the top of the head golden yellow and the back and wings pale olive-green.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tailor</ent><br/
<hw>Tai"lor</hw>, <pos>v. i.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Tailored</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Tailoring</conjf>.]</vmorph> <def>To practice making men's clothes; to follow the business of a tailor.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>These <qex>tailoring</qex> artists for our lays<br/
Invent cramped rules.</q> <rj><qau>M. Green.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tailoress</ent><br/
<hw>Tai"lor*ess</hw>, <pos>n.</pos> <def>A female tailor.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tailoring</ent><br/
<hw>Tai"lor*ing</hw>, <pos>adv.</pos> <def>The business or the work of a tailor or a tailoress.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tailor-made</ent><br/
<hw>Tai"lor-made`</hw>, <pos>a.</pos> <sn>1.</sn> <def>Made by a tailor or according to a tailor's fashion; -- said specif. of women's garments made with certain closeness of fit, simplicity of ornament, etc.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><sn>2.</sn> <def>made or as if made specifically for the particular purpose at hand; --  used metaphorically.</def><br/
[<source>PJC</source>]</p>

<p><ent>Tailpiece</ent><br/
<hw>Tail"piece`</hw> <pr>(?)</pr>, <pos>n.</pos> <sn>1.</sn> <def>A piece at the end; an appendage.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Arch.)</fld> <def>One of the timbers which tail into a header, in floor framing. See <xex>Illust.</xex> of <er>Header</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <fld>(Print.)</fld> <def>An ornament placed at the bottom of a short page to fill up the space, or at the end of a book.</def>  <rj><au>Savage.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>A piece of ebony or other material attached to the lower end of a violin or similar instrument, to which the strings are fastened.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <fld>(Locks)</fld> <def>A piece for transmitting motion from the hub of a lock to the latch bolt.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><sn>6.</sn>  <def>The part of a telescope containing the adjusting device for the eyepiece, etc.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Tailpin</ent><br/
<hw>Tail"pin"</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Mach.)</fld> <def>The center in the spindle of a turning lathe.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tailrace</ent><br/
<hw>Tail"race`</hw> <pr>(?)</pr>, <pos>n.</pos> <sn>1.</sn> <def>See <er>Race</er>, <pos>n.</pos>, 6.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Mining)</fld> <def>The channel in which tailings, suspended in water, are conducted away.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tailstock</ent><br/
<hw>Tail"stock`</hw> <pr>(?)</pr>, <pos>n.</pos> <def>The sliding block or support, in a lathe, which carries the dead spindle, or adjustable center. The <contr>headstock</contr> supports the live spindle.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tail-water</ent><br/
<hw>Tail"-wa`ter</hw> <pr>(?)</pr>, <pos>n.</pos> <def>Water in a tailrace.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tailzie</ent><br/
<hw>Tail"zie</hw> <pr>(-z<icr/ <or/ -y<icr/)</pr>, <pos>n.</pos> <ety>[F. <ets>tailler</ets> to cut. See <er>Tail</er> a limitation.]</ety> <fld>(Scots Law)</fld> <def>An entailment or deed whereby the legal course of succession is cut off, and an arbitrary one substituted.</def> <altsp>[Written also <asp>tailzee</asp>.]</altsp><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tain</ent><br/
<hw>Tain</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[OE. <ets>tein</ets>, <ets>teyne</ets>; cf. Icel. <ets>teinn</ets> a twig, akin to AS. <ets>t<amac/n</ets>, Goth. <ets>tains</ets>.]</ety> <def>Thin tin plate; also, tin foil for mirrors.</def>  <rj><au>Knight.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Taint</ent><br/
<hw>Taint</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Cf. F. <ets>atteinte</ets> a blow, bit, stroke. See <er>Attaint</er>.]</ety> <sn>1.</sn> <def>A thrust with a lance, which fails of its intended effect.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>This <qex>taint</qex> he followed with his sword drawn from a silver sheath.</q> <rj><qau>Chapman.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>An injury done to a lance in an encounter, without its being broken; also, a breaking of a lance in an encounter in a dishonorable or unscientific manner.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Taint</ent><br/
<hw>Taint</hw>, <pos>v. i.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Tainted</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Tainting</conjf>.]</vmorph> <def>To thrust ineffectually with a lance.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Taint</ent><br/
<hw>Taint</hw>, <pos>v. t.</pos> <sn>1.</sn> <def>To injure, as a lance, without breaking it; also, to break, as a lance, but usually in an unknightly or unscientific manner.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>Do not fear; I have<br/
A staff to <qex>taint</qex>, and bravely.</q> <rj><qau>Massinger.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To hit or touch lightly, in tilting.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>They <qex>tainted</qex> each other on the helms and passed by.</q> <rj><qau>Ld. Berners.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Taint</ent><br/
<hw>Taint</hw>, <pos>v. t.</pos> <ety>[F. <ets>teint</ets>, p. p. of <ets>teindre</ets> to dye, tinge, fr. L. <ets>tingere</ets>, <ets>tinctum</ets>. See <er>Tinge</er>, and cf. <er>Tint</er>.]</ety> <sn>1.</sn> <def>To imbue or impregnate with something extraneous, especially with something odious, noxious, or poisonous; hence, to corrupt; to infect; to poison; <as>as, putrid substance <ex>taint</ex> the air</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Fig.: To stain; to sully; to tarnish.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>His unkindness may defeat my life,<br/
But never <qex>taint</qex> my love.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><syn><b>Syn.</b> -- To contaminate; defile; pollute; corrupt; infect; disease; vitiate; poison.</syn><br/
[<source>1913 Webster</source>]</p>

<p><ent>Taint</ent><br/
<hw>Taint</hw> <pr>(?)</pr>, <pos>v. i.</pos> <sn>1.</sn> <def>To be infected or corrupted; to be touched with something corrupting.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>I can not <qex>taint</qex> with fear.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To be affected with incipient putrefaction; <as>as, meat soon <ex>taints</ex> in warm weather</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Taint</ent><br/
<hw>Taint</hw>, <pos>n.</pos> <sn>1.</sn> <def>Tincture; hue; color; tinge.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Infection; corruption; deprivation.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>He had inherited from his parents a scrofulous <qex>taint</qex>, which it was beyond the power of medicine to remove.</q> <rj><qau>Macaulay.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>A blemish on reputation; stain; spot; disgrace.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Taint</ent><br/
<hw>Taint</hw> <pr>(?)</pr>, <pos>v. t.</pos> <def>Aphetic form of <er>Attaint</er>.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Taintless</ent><br/
<hw>Taint"less</hw>, <pos>a.</pos> <def>Free from taint or infection; pure.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Taintlessly</ent><br/
<hw>Taint"less*ly</hw>, <pos>adv.</pos> <def>In a taintless manner.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tainture</ent><br/
<hw>Tain"ture</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F. <ets>teinture</ets>. See <er>Taint</er> to stain, and cf. <er>Tincture</er>.]</ety> <def>Taint; tinge; difilement; stain; spot.</def> <mark>[R.]</mark>  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Taintworm</ent><br/
<hw>Taint"worm`</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Zool.)</fld> <def>A destructive parasitic worm or insect larva.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Taeping</ent><br/
<ent>Taiping</ent><br/
<mhw>{ <hw>Tai"ping"</hw>, <it>or</it> <hw>Tae"ping"</hw> }</mhw> <pr>(?)</pr>, <pos>a.</pos> <ety>[Chin. <ets>t'aip'ing</ets> great peace.]</ety> <fld>(Chinese Hist.)</fld> <def>Pertaining to or designating a dynasty with which one Hung-Siu-Chuen, a half-religious, half-political enthusiast, attempted to supplant the Manchu dynasty by the <col><b>Taiping rebellion</b></col>, incited by him in 1850 and suppressed by General Gordon about 1864.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Taira</ent><br/
<hw>Tai"ra</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Zool.)</fld> <def>Same as <er>Tayra</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tairn</ent><br/
<hw>Tairn</hw> <pr>(?)</pr>, <pos>n.</pos> <def>See <er>Tarn</er>.</def>  <rj><au>Coleridge.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tait</ent><br/
<hw>Tait</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Zool.)</fld> <def>A small nocturnal and arboreal Australian marsupial (<spn>Tarsipes rostratus</spn>) about the size of a mouse. It has a long muzzle, a long tongue, and very few teeth, and feeds upon honey and insects. Called also <altname>noolbenger</altname>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tajassu</ent><br/
<ent>Tajacu</ent><br/
<mhw>{ <hw>Ta*ja<cced/"u</hw>, <hw>Ta*jas"su</hw>  }</mhw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Pg. <ets>taja<cced/<uacute/</ets>, from Braz. <ets>taya<cced/<uacute/</ets> a hog or swine.]</ety> <fld>(Zool.)</fld> <def>The common, or collared, peccary (<spn>Tayassu tajacu</spn>).  Called also <altname>javelina</altname> and <altname>tayaussa</altname>.</def><br/
[<source>1913 Webster</source> + <source>PJC</source>]</p>

<p><q><title>Javelina</title><br/
Common Names: Collared Peccary, Musk Hog, Tayaussa<br/
Genus: Tayassu<br/
Species: tajacu<br/
<br/
The <a href="images/javelina.gif">javelina</a> is the only wild pig in North America. It looks exactly like a real pig except a little bigger.  It is a grizzled blackish-gray color with a yellowish band which runs under the neck.  The name <altname>collared peccary</altname> comes from the ruff of hair around its neck.  It has small, round ears and beady eyes. Its body is barrel-shaped with short legs.  Its head is pointed, ending in a disc-like nose. Javelinas have 3 toes on each hind foot. The upper tusks (1.5 inches long) are pointed down, instead of up like some other wild pigs. The javelina grows from 46 to 60 inches long, and can weigh up to 60 pounds.<br/
<br/
Javelinas live in the canyon area of the desert. They live there because it is bushy and there are water holes everywhere. They need the bushes for the shade and they need lots of water to live.<br/
<br/
The javelina eats cacti, grass, bulbs, berries, flowers, mushrooms, and fruit, which is easy for them to get in their habitat.<br/
<br/
The javelina always travel and live in groups. The female gives birth to twins about once a year. Her young travel with the group their whole life until they die. No new members are ever accepted into the group unless they are born into it.<br/
<br/
The javelinas have a very good nose. That can have its advantages if there is a predator around. They also look for food in groups so if they're attacked they can fight back in numbers.<br/
<br/
The javelina's niche is the water holes and bushes in the canyon area where they live. They get shade under the bushes, so they won't over heat. They can't survive long without water, so it's good to have water close by if you are a javelina.<br/
<br/
Javelinas' status in the wild is very good right now. There are lots of them around and they're not dying off or going extinct.</q>  <qau>Tristan A.  (from http://www.blueplanetbiomes.org/javelina.htm, 2000.)</qau><br/
[<source>PJC</source>]</p>


<p><ent>Taj Mahal</ent><br/
<hw>Taj Ma*hal"</hw> <pr>(t<aum/j m<adot/*h<aum/l")</pr>, <pos>prop. n.</pos> <ety>[Corruption of Per. <ets>Mumt<amac/z-i-Ma<hsdot/al</ets>, lit., the distinguished one of the palace, fr. Ar.]</ety> <def>A marble mausoleum built at <city>Agra</city>, <country>India</country>, by the Mogul Emperor <person>Shah Jahan</person>, in memory of his favorite wife.  In beauty of design and rich decorative detail it is one of the best examples of Saracenic architecture, and specifically of Mogul architecture.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Take</ent><br/
<hw>Take</hw> <pr>(t<amac/k)</pr>, <mark>obs.</mark> <pos>p. p.</pos> <mord>of <er>Take</er></mord>. <def>Taken.</def>  <rj><au>Chaucer.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><-- p. 1470 --><br/
[<source>1913 Webster</source>]</p>

<p><ent>Take</ent><br/
<hw>Take</hw>, <pos>v. t.</pos> <vmorph>[<pos>imp.</pos> <conjf>Took</conjf> <pr>(t<oocr/k)</pr>; <pos>p. p.</pos> <conjf>Taken</conjf> <pr>(t<amac/k'n)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Taking</conjf>.]</vmorph> <ety>[Icel. <ets>taka</ets>; akin to Sw. <ets>taga</ets>, Dan. <ets>tage</ets>, Goth. <ets>t<emac/kan</ets> to touch; of uncertain origin.]</ety> <sn>1.</sn> <def>In an active sense; To lay hold of; to seize with the hands, or otherwise; to grasp; to get into one's hold or possession; to procure; to seize and carry away; to convey.</def> <specif>Hence, specifically:</specif> --<br/
[<source>1913 Webster</source>]</p>

<p><sd>(a)</sd> <def>To obtain possession of by force or artifice; to get the custody or control of; to reduce into subjection to one's power or will; to capture; to seize; to make prisoner; <as>as, to <ex>take</ex> an army, a city, or a ship</as>; also, to come upon or befall; to fasten on; to attack; to seize; -- said of a disease, misfortune, or the like.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>This man was <qex>taken</qex> of the Jews.</q> <rj><qau>Acts xxiii. 27.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Men in their loose, unguarded hours they <qex>take</qex>;<br/
Not that themselves are wise, but others weak.</q> <rj><qau>Pope.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>They that come abroad after these showers are commonly <qex>taken</qex> with sickness.</q> <rj><qau>Bacon.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>There he blasts the tree and <qex>takes</qex> the cattle<br/
And makes milch kine yield blood.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sd>(b)</sd> <def>To gain or secure the interest or affection of; to captivate; to engage; to interest; to charm.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Neither let her <qex>take</qex> thee with her eyelids.</q> <rj><qau>Prov. vi. 25.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Cleombroutus was so <qex>taken</qex> with this prospect, that he had no patience.</q> <rj><qau>Wake.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>I know not why, but there was a something in those half-seen features, -- a charm in the very shadow that hung over their imagined beauty, -- which <qex>took</qex> me more than all the outshining loveliness of her companions.</q> <rj><qau>Moore.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sd>(c)</sd> <def>To make selection of; to choose; also, to turn to; to have recourse to; <as>as, to <ex>take</ex> the road to the right</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Saul said, Cast lots between me and Jonathan my son.  And Jonathan was <qex>taken</qex>.</q> <rj><qau>1 Sam. xiv. 42.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>The violence of storming is the course which God is forced to <qex>take</qex> for the destroying . . . of sinners.</q> <rj><qau>Hammond.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sd>(d)</sd> <def>To employ; to use; to occupy; hence, to demand; to require; <as>as, it <ex>takes</ex> so much cloth to make a coat; it <ex>takes</ex> five hours to get to Boston from New York by car</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>This man always <qex>takes</qex> time . . . before he passes his judgments.</q> <rj><qau>I. Watts.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sd>(e)</sd> <def>To form a likeness of; to copy; to delineate; to picture; <as>as, to <ex>take</ex> a picture of a person</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Beauty alone could beauty <qex>take</qex> so right.</q> <rj><qau>Dryden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sd>(f)</sd> <def>To draw; to deduce; to derive.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>The firm belief of a future judgment is the most forcible motive to a good life, because <qex>taken</qex> from this consideration of the most lasting happiness and misery.</q> <rj><qau>Tillotson.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sd>(g)</sd> <def>To assume; to adopt; to acquire, as shape; to permit to one's self; to indulge or engage in; to yield to; to have or feel; to enjoy or experience, as rest, revenge, delight, shame; to form and adopt, as a resolution; -- used in general senses, limited by a following complement, in many idiomatic phrases; <as>as, to <ex>take</ex> a resolution; I <ex>take</ex> the liberty to say</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sd>(h)</sd> <def>To lead; to conduct; <as>as, to <ex>take</ex> a child to church</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sd>(i)</sd> <def>To carry; to convey; to deliver to another; to hand over; <as>as, he <ex>took</ex> the book to the bindery; he <ex>took</ex> a dictionary with him</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>He <qex>took</qex> me certain gold, I wot it well.</q> <rj><qau>Chaucer.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sd>(k)</sd> <def>To remove; to withdraw; to deduct; -- with <xex>from</xex>; <as>as, to <ex>take</ex> the breath from one; to <ex>take</ex> two from four</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>In a somewhat passive sense, to receive; to bear; to endure; to acknowledge; to accept.</def> <specif>Specifically:</specif> --<br/
[<source>1913 Webster</source>]</p>

<p><sd>(a)</sd> <def>To accept, as something offered; to receive; not to refuse or reject; to admit.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Ye shall <qex>take</qex> no satisfaction for the life of a murderer.</q> <rj><qau>Num. xxxv. 31.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Let not a widow be <qex>taken</qex> into the number under threescore.</q> <rj><qau>1 Tim. v. 10.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sd>(b)</sd> <def>To receive as something to be eaten or drunk; to partake of; to swallow; <as>as, to <ex>take</ex> food or wine</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sd>(c)</sd> <def>Not to refuse or balk at; to undertake readily; to clear; <as>as, to <ex>take</ex> a hedge or fence</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sd>(d)</sd> <def>To bear without ill humor or resentment; to submit to; to tolerate; to endure; <as>as, to <ex>take</ex> a joke; he will <ex>take</ex> an affront from no man</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sd>(e)</sd> <def>To admit, as, something presented to the mind; not to dispute; to allow; to accept; to receive in thought; to entertain in opinion; to understand; to interpret; to regard or look upon; to consider; to suppose; <as>as, to <ex>take</ex> a thing for granted; this I <ex>take</ex> to be man's motive; to <ex>take</ex> men for spies</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>You <qex>take</qex> me right.</q> <rj><qau>Bacon.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Charity, <qex>taken</qex> in its largest extent, is nothing else but the science love of God and our neighbor.</q> <rj><qau>Wake.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>[He] <qex>took</qex> that for virtue and affection which was nothing but vice in a disguise.</q> <rj><qau>South.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>You'd doubt his sex, and <qex>take</qex> him for a girl.</q> <rj><qau>Tate.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sd>(f)</sd> <def>To accept the word or offer of; to receive and accept; to bear; to submit to; to enter into agreement with; -- used in general senses; <as>as, to <ex>take</ex> a form or shape</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>I <qex>take</qex> thee at thy word.</q> <rj><qau>Rowe.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Yet thy moist clay is pliant to command; . . . <br/
Not <qex>take</qex> the mold.</q> <rj><qau>Dryden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>To make a picture, photograph, or the like, of; <as>as, to <ex>take</ex> a group or a scene</as>.</def> <mark>[Colloq.]</mark><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><sn>4.</sn>  <def>To give or deliver (a blow to); to strike; hit; <as>as, he <ex>took</ex> me in the face; he <ex>took</ex> me a blow on the head.</as></def> <mark>[Obs. exc. Slang or Dial.]</mark><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><cs><mcol><col><b>To be taken aback</b></col>, <col><b>To take advantage of</b></col>, <col><b>To take air</b></col>, etc.</mcol> <cd>See under <er>Aback</er>, <er>Advantage</er>, etc.</cd> -- <col><b>To take aim</b></col>, <cd>to direct the eye or weapon; to aim.</cd> -- <col><b>To take along</b></col>, <cd>to carry, lead, or convey.</cd> -- <col><b>To take arms</b></col>, <cd>to commence war or hostilities.</cd> -- <col><b>To take away</b></col>, <cd>to carry off; to remove; to cause deprivation of; to do away with; <as>as, a bill for <ex>taking away</ex> the votes of bishops</as>.</cd> <ldquo/By your own law, I <xex>take</xex> your life <xex>away</xex>.<rdquo/ <au>Dryden.</au> -- <col><b>To take breath</b></col>, <cd>to stop, as from labor, in order to breathe or rest; to recruit or refresh one's self.</cd> -- <col><b>To take care</b></col>, <cd>to exercise care or vigilance; to be solicitous.</cd> <ldquo/Doth God <xex>take care</xex> for oxen?<rdquo/ <au>1 Cor. ix. 9.</au> -- <col><b>To take care of</b></col>, <cd>to have the charge or care of; to care for; to superintend or oversee.</cd> -- <col><b>To take down</b></col>. <sd>(a)</sd> <cd>To reduce; to bring down, as from a high, or higher, place; <as>as, <ex>to take down</ex> a book</as>; hence, to bring lower; to depress; to abase or humble; <as>as, <ex>to take down</ex> pride, or the proud</as>.</cd> <ldquo/I never attempted to be impudent yet, that I was not <xex>taken down</xex>.<rdquo/ <au>Goldsmith.</au> <sd>(b)</sd> <cd>To swallow; <as>as, <ex>to take down</ex> a potion</as>.</cd> <sd>(c)</sd> <cd>To pull down; to pull to pieces; <as>as, <ex>to take down</ex> a house or a scaffold</as>.</cd> <sd>(d)</sd> <cd>To record; to write down; <as>as, <ex>to take down</ex> a man's words at the time he utters them</as>.</cd> -- <mcol><col><b>To take effect</b></col>, <col><b>To take fire</b></col></mcol>. <cd>See under <er>Effect</er>, and <er>Fire</er>.</cd> -- <mcol><col><b>To take ground to the right</b></col> <it>or</it> <col><b>To take ground to the left</b></col></mcol> <fld>(Mil.)</fld>, <cd>to extend the line to the right or left; to move, as troops, to the right or left.</cd> -- <col><b>To take heart</b></col>, <cd>to gain confidence or courage; to be encouraged.</cd> -- <col><b>To take heed</b></col>, <cd>to be careful or cautious.</cd> <ldquo/<xex>Take heed</xex> what doom against yourself you give.<rdquo/ <au>Dryden.</au> -- <col><b>To take heed to</b></col>, <cd>to attend with care, <as>as, <ex>take heed to</ex> thy ways</as>.</cd> -- <col><b>To take hold of</b></col>, <cd>to seize; to fix on.</cd> -- <col><b>To take horse</b></col>, <cd>to mount and ride a horse.</cd> -- <col><b>To take in</b></col>. <sd>(a)</sd> <cd>To inclose; to fence.</cd> <sd>(b)</sd> <cd>To encompass or embrace; to comprise; to comprehend.</cd> <sd>(c)</sd> <cd>To draw into a smaller compass; to contract; to brail or furl; <as>as, <ex>to take in</ex> sail</as>.</cd> <sd>(d)</sd> <cd>To cheat; to circumvent; to gull; to deceive.</cd> <mark>[Colloq.]</mark> <sd>(e)</sd> <cd>To admit; to receive; <as>as, a leaky vessel will <ex>take in</ex> water</as>.</cd> <sd>(f)</sd> <cd>To win by conquest.</cd> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>For now Troy's broad-wayed town<br/
He shall <qex>take in</qex>.</q> <rj><qau>Chapman.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sd>(g)</sd> <cd>To receive into the mind or understanding.</cd> <ldquo/Some bright genius can <xex>take in</xex> a long train of propositions.<rdquo/ <au>I. Watts.</au> <sd>(h)</sd> <cd>To receive regularly, as a periodical work or newspaper; to take.</cd> <mark>[Eng.]</mark> -- <col><b>To take in hand</b></col>. <cd>See under <er>Hand</er>.</cd> -- <col><b>To take in vain</b></col>, <cd>to employ or utter as in an oath.</cd> <ldquo/Thou shalt not <xex>take</xex> the name of the Lord thy God <xex>in vain</xex>.<rdquo/ <au>Ex. xx. 7.</au> -- <col><b>To take issue</b></col>. <cd>See under <er>Issue</er>.</cd> -- <col><b>To take leave</b></col>. <cd>See <er>Leave</er>, <pos>n.</pos>, 2.</cd> -- <mcol><col><b>To take a newspaper</b></col>, <col><b>magazine</b></col>, or the like</mcol>, <cd>to receive it regularly, as on paying the price of subscription.</cd> -- <col><b>To take notice</b></col>, <cd>to observe, or to observe with particular attention.</cd> -- <col><b>To take notice of</b></col>. <cd>See under <er>Notice</er>.</cd> -- <col><b>To take oath</b></col>, <cd>to swear with solemnity, or in a judicial manner.</cd>  -- <col><b>To take on</b></col>, <cd>to assume; to take upon one's self; <as>as, <ex>to take on</ex> a character or responsibility</as>.</cd> -- <col><b>To take one's own course</b></col>, <cd>to act one's pleasure; to pursue the measures of one's own choice.</cd> -- <col><b>To take order for</b></col>. <cd>See under <er>Order</er>.</cd> -- <col><b>To take order with</b></col>, <cd>to check; to hinder; to repress.</cd> <mark>[Obs.]</mark> <au>Bacon.</au> -- <col><b>To take orders</b></col>. <sd>(a)</sd> <cd>To receive directions or commands.</cd> <sd>(b)</sd> <fld>(Eccl.)</fld> <cd>To enter some grade of the ministry. See <er>Order</er>, <pos>n.</pos>, 10.</cd> -- <col><b>To take out</b></col>. <sd>(a)</sd> <cd>To remove from within a place; to separate; to deduct.</cd> <sd>(b)</sd> <cd>To draw out; to remove; to clear or cleanse from; <as>as, <ex>to take out</ex> a stain or spot from cloth</as>.</cd> <sd>(c)</sd> <cd>To produce for one's self; <as>as, <ex>to take out</ex> a patent</as>.</cd> <-- "produce"?? better, "obtain" --> <sd>(d)</sd> <cd>To put an end to; <as>as, <ex>to take</ex> the conceit <ex>out</ex> of a man</as>.</cd> <sd>(e)</sd> <cd>To escort; <as>as, <ex>to take out</ex> to dinner</as>.</cd><-- usu. paying the expenses --> -- <col><b>To take over</b></col>, <cd>to undertake; to take the management of.</cd> <mark>[Eng.]</mark> <au>Cross (Life of G. Eliot).</au> -- <col><b>To take part</b></col>, <cd>to share; <as>as, they <ex>take part</ex> in our rejoicing</as>.</cd> -- <col><b>To take part with</b></col>, <cd>to unite with; to join with.</cd><-- take part in = participate in --> -- <mcol><col><b>To take place</b></col>, <col><b>root</b></col>, <col><b>sides</b></col>, <col><b>stock</b></col>, etc.</mcol> <cd>See under <er>Place</er>, <er>Root</er>, <er>Side</er>, etc.</cd> -- <col><b>To take the air</b></col>. <sd>(a)</sd> <fld>(Falconry)</fld> <cd>To seek to escape by trying to rise higher than the falcon; -- said of a bird.</cd> <sd>(b)</sd> <cd>See under <er>Air</er>.</cd> -- <col><b>To take the field</b></col>. <fld>(Mil.)</fld> <cd>See under <er>Field</er>.</cd> -- <col><b>To take thought</b></col>, <cd>to be concerned or anxious; to be solicitous.</cd> <au>Matt. vi. 25, 27.</au> -- <col><b>To take to heart</b></col>. <cd>See under <er>Heart</er>.</cd> -- <col><b>To take to task</b></col>, <cd>to reprove; to censure.</cd> -- <-- <col><b>to take to the air</b></col>, <cd>to take off.</cd> --> <col><b>To take up</b></col>. <sd>(a)</sd> <cd>To lift; to raise.</cd> <au>Hood.</au> <sd>(b)</sd> <cd>To buy or borrow; <as>as, <ex>to take up</ex> goods to a large amount; <ex>to take up</ex> money at the bank</as>.</cd> <sd>(c)</sd> <cd>To begin; <as>as, <ex>to take up</ex> a lamentation</as>.</cd> <au>Ezek. xix. 1.</au> <sd>(d)</sd> <cd>To gather together; to bind up; to fasten or to replace; <as>as, <ex>to take up</ex> raveled stitches</as></cd>; specifically <fld>(Surg.)</fld>, <cd>to fasten with a ligature.</cd> <sd>(e)</sd> <cd>To engross; to employ; to occupy or fill; <as>as, <ex>to take up</ex> the time; <ex>to take up</ex> a great deal of room</as>.</cd> <sd>(f)</sd> <cd>To take permanently.</cd> <ldquo/Arnobius asserts that men of the finest parts . . . <xex>took up</xex> their rest in the Christian religion.<rdquo/ <au>Addison.</au> <sd>(g)</sd> <cd>To seize; to catch; to arrest; <as>as, <ex>to take up</ex> a thief; <ex>to take up</ex> vagabonds</as>.</cd> <sd>(h)</sd> <cd>To admit; to believe; to receive.</cd> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>The ancients <qex>took up</qex> experiments upon credit.</q> <rj><qau>Bacon.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sd>(i)</sd> <cd>To answer by reproof; to reprimand; to berate.</cd><br/
[<source>1913 Webster</source>]</p>

<p><q>One of his relations <qex>took</qex> him <qex>up</qex> roundly.</q> <rj><qau>L'Estrange.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sd>(k)</sd> <cd>To begin where another left off; to keep up in continuous succession; to take up (a topic, an activity).</cd><br/
[<source>1913 Webster</source>]</p>

<p><q>Soon as the evening shades prevail,<br/
The moon <qex>takes up</qex> the wondrous tale.</q> <rj><qau>Addison.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><-- The second volume takes up where the first left off. --><br/
[<source>1913 Webster</source>]</p>

<p><sd>(l)</sd> <cd>To assume; to adopt as one's own; to carry on or manage; <as>as, <ex>to take up</ex> the quarrels of our neighbors; <ex>to take up</ex> current opinions</as>.</cd> <ldquo/They <xex>take up</xex> our old trade of conquering.<rdquo/ <au>Dryden.</au> <sd>(m)</sd> <cd>To comprise; to include.</cd> <ldquo/The noble poem of Palemon and Arcite . . . <xex>takes up</xex> seven years.<rdquo/ <au>Dryden.</au> <sd>(n)</sd> <cd>To receive, accept, or adopt for the purpose of assisting; to espouse the cause of; to favor.</cd> <au>Ps. xxvii. 10.</au> <sd>(o)</sd> <cd>To collect; to exact, as a tax; to levy; <as>as, <ex>to take up</ex> a contribution</as>.</cd> <ldquo/<xex>Take up</xex> commodities upon our bills.<rdquo/ <au>Shak.</au> <sd>(p)</sd> <cd>To pay and receive; <as>as, <ex>to take up</ex> a note at the bank</as>.</cd> <sd>(q)</sd> <fld>(Mach.)</fld> <cd>To remove, as by an adjustment of parts; <as>as, <ex>to take up</ex> lost motion, as in a bearing</as>; also, to make tight, as by winding, or drawing; <as>as, <ex>to take up</ex> slack thread in sewing</as>.</cd> <sd>(r)</sd> <cd>To make up; to compose; to settle; <as>as, <ex>to take up</ex> a quarrel</as>.</cd> <mark>[Obs.]</mark> <au>Shak.</au> -- (s) <cd>To accept from someone, as a wager or a challenge; <as>as, J. <ex>took</ex> M. <ex>up</ex> on his challenge</as>.</cd> -- <col><b>To take up arms</b></col>. <cd>Same as <cref>To take arms</cref>, above.</cd> -- <col><b>To take upon one's self</b></col>. <sd>(a)</sd> <cd>To assume; to undertake; <as>as, he <ex>takes upon himself</ex> to assert that the fact is capable of proof</as>.</cd> <sd>(b)</sd> <cd>To appropriate to one's self; to allow to be imputed to, or inflicted upon, one's self; <as>as, <ex>to take upon one's self</ex> a punishment</as>.</cd> -- <col><b>To take up the gauntlet</b></col>. <cd>See under <er>Gauntlet</er>.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Take</ent><br/
<hw>Take</hw> <pr>(?)</pr>, <pos>v. i.</pos> <sn>1.</sn> <def>To take hold; to fix upon anything; to have the natural or intended effect; to accomplish a purpose; <as>as, he was inoculated, but the virus did not <ex>take</ex></as>.</def>  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>When flame <qex>taketh</qex> and openeth, it giveth a noise.</q> <rj><qau>Bacon.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>In impressions from mind to mind, the impression <qex>taketh</qex>, but is overcome . . . before it work any manifest effect.</q> <rj><qau>Bacon.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To please; to gain reception; to succeed.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Each wit may praise it for his own dear sake,<br/
And hint he writ it, if the thing should <qex>take</qex>.</q> <rj><qau>Addison.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>To move or direct the course; to resort; to betake one's self; to proceed; to go; -- usually with <xex>to</xex>; <as>as, the fox, being hard pressed, <ex>took</ex> to the hedge</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>To admit of being pictured, as in a photograph; <as>as, his face does not <ex>take</ex> well</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>To take after</b></col>. <sd>(a)</sd> <cd>To learn to follow; to copy; to imitate; <as>as, he <ex>takes after</ex> a good pattern</as>.</cd> <sd>(b)</sd> <cd>To resemble; <as>as, the son <ex>takes after</ex> his father</as>.</cd> -- <col><b>To take in with</b></col>, <cd>to resort to.</cd> <mark>[Obs.]</mark> <au>Bacon.</au> -- <col><b>To take on</b></col>, <cd>to be violently affected; to express grief or pain in a violent manner.</cd> -- <col><b>To take to</b></col>. <sd>(a)</sd> <cd>To apply one's self to; to be fond of; to become attached to; <as>as, <ex>to take to</ex> evil practices</as>.</cd> <ldquo/If he does but <xex>take to</xex> you, . . . you will contract a great friendship with him.<rdquo/ <au>Walpole.</au> <sd>(b)</sd> <cd>To resort to; to betake one's self to.</cd> <ldquo/Men of learning, who <xex>take to</xex> business, discharge it generally with greater honesty than men of the world.<rdquo/ <au>Addison.</au> -- <col><b>To take up</b></col>. <sd>(a)</sd> <cd>To stop.</cd> <mark>[Obs.]</mark> <ldquo/Sinners at last <xex>take up</xex> and settle in a contempt of religion.<rdquo/ <au>Tillotson.</au> <sd>(b)</sd> <cd>To reform.</cd> <mark>[Obs.]</mark> <au>Locke.</au> -- <col><b>To take up with</b></col>. <sd>(a)</sd> <cd>To be contended to receive; to receive without opposition; to put up with; <as>as, <ex>to take up with</ex> plain fare</as>.</cd> <ldquo/In affairs which may have an extensive influence on our future happiness, we should not <xex>take up with</xex> probabilities.<rdquo/ <au>I. Watts.</au> <sd>(b)</sd> <cd>To lodge with; to dwell with.</cd> <mark>[Obs.]</mark> <au>L'Estrange.</au> -- <col><b>To take with</b></col>, <cd>to please.</cd> <au>Bacon.</au></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Take</ent><br/
<hw>Take</hw>, <pos>n.</pos> <sn>1.</sn> <def>That which is taken, such as the quantity of fish captured at one haul or catch, or the amouont of money collected during one event; <as>as, the box-office <ex>take</ex></as>.</def><br/
[<source>1913 Webster</source> <source>+PJC</source>]</p>

<p><sn>2.</sn> <fld>(Print.)</fld> <def>The quantity or copy given to a compositor at one time.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Take-in</ent><br/
<hw>Take"-in`</hw> <pr>(?)</pr>, <pos>n.</pos> <def>Imposition; fraud.</def> <mark>[Colloq.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Taken</ent><br/
<hw>Tak"en</hw> <pr>(t<amac/k"'n)</pr>, <def><pos>p. p.</pos> of <er>Take</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Take-off</ent><br/
<ent>Takeoff</ent><br/
<mhw><hw>Take"off`</hw>, <hw>Take"-off`</hw></mhw> <pr>(?)</pr>, <pos>n.</pos> <sn>1.</sn> <def>An imitation, especially in the way of caricature; -- used with <ptcl>of</ptcl> or <ptcl>on</ptcl>; <as>as, the comedian did a hilarious <ex>takeoff</ex> on the president</as>.</def><br/
[<source>1913 Webster</source> <source>+PJC</source>]</p>

<p><sn>2.</sn> <def>The spot at which one takes off; specif., the place from which a jumper rises in leaping.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><q>The <qex>take-off</qex> should be selected with great care, and a pit of large dimensions provided on the landing side.</q>  <rj><qau>Encyc. of Sport.</qau></rj><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><sn>3.</sn> <def>The beginning of a leap from a surface or a flight into the air, especially the process or event of an airplane leaving the ground and beginning its flight; <as>as, the <ex>takeoff</ex> of flight CA123 was scheduled for 3:00 PM</as>.</def><br/
[<source>PJC</source>]</p>

<p><ent>Take off</ent><br/
<hw>Take" off`</hw> <pr>(?)</pr>, <pos>v. i.</pos> <sn>1.</sn> <def>to begin a leap from a surface or a flight into the air; especially, (of a bird or an airplane) to leave the ground and begin to fly; <as>as, flight CA123 <ex>took off</ex> on schedule at 3:00 PM</as>.</def><br/
[<source>PJC</source>]</p>

<p><sn>2.</sn> <def>To begin a period of accelerating growth or development; <as>as, the economy <ex>took off</ex> in the third quarter</as>.</def><br/
[<source>PJC</source>]</p>

<p><sn>3.</sn> <def>To begin a journey; to depart.</def><br/
[<source>PJC</source>]</p>

<p><ent>Take off</ent><br/
<hw>Take" off`</hw> <pr>(?)</pr>, <pos>v. t.</pos> <sn>1.</sn> <def>To remove, as from the surface or outside; to remove from the top of anything; <as>as, <ex>to take off</ex> a load; <ex>to take off</ex> one's hat, coat or other article of clothing; to <ex>take off</ex> a coat of paint from a surface</as>.</def><br/
[<source>1913 Webster</source> <source>+PJC</source>]</p>

<p><sn>2.</sn> <def>To cut off; <as>as, <ex>to take off</ex> the head, or a limb</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>To destroy; <as>as, <ex>to take off</ex> life</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>To remove; to invalidate; <as>as, <ex>to take off</ex> the force of an argument</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <def>To withdraw; to call or draw away; <as>as, the director <ex>took</ex> him <ex>off</ex> the project</as>.</def> <au>Locke.</au><br/
[<source>1913 Webster</source> + <source>PJC</source>]</p>

<p><sn>6.</sn> <def>To swallow; <as>as, <ex>to take off</ex> a glass of wine</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>7.</sn> <def>To purchase; to take in trade.</def> <ldquo/The Spaniards having no commodities that we will <xex>take off</xex>.<rdquo/ <au>Locke.</au><br/
[<source>1913 Webster</source>]</p>

<p><sn>8.</sn> <def>To copy; to reproduce.</def> <ldquo/<xex>Take off</xex> all their models in wood.<rdquo/ <au>Addison.</au><br/
[<source>1913 Webster</source>]</p>

<p><sn>9.</sn> <def>To imitate; to mimic; to personate.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>10.</sn> <def>To find place for; to dispose of; <as>as, more scholars than preferments can <ex>take off</ex>.</as></def> <mark>[R.]</mark> <au>Bacon.</au><br/
[<source>1913 Webster</source>]</p>

<p><sn>11.</sn> <def>To discount or deduct (from a price); <as>the dealer <ex>took off</ex> twenty percent on remaining toys</as>.</def><br/
[<source>PJC</source>]</p>

<p><ent>takeover</ent><br/
<hw>take"o*ver</hw> <pos>n.</pos> <fld>(Business, Finance)</fld> <def>The acquisition of ownership of one company by another company, usually by purchasing a controlling percentage of its stock or by exchanging stock of the purchasing company for that of the purchased company.  It is a <stype>hostile takeover</stype> if the management of the company being taken over is opposed to the deal.  A hostile <ex>takeover</ex> is sometimes organized by a <er>corporate raider</er>.</def><br/
<syn><b>Syn. --</b> acquisition, buyout</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Taker</ent><br/
<hw>Tak"er</hw> <pr>(t<amac/k"<etil/r)</pr>, <pos>n.</pos> <def>One who takes or receives; one who catches or apprehends.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>take the heat</ent><br/
<hw>take` the heat"</hw> <pr>(t<amac/k`<th/<ucr/*h<emac/t")</pr>, <pos>phr.</pos> <sn>1.</sn> <def>To be assigned or to accept the blame for some misdeed; <as>as, Mary broke the vase, but she acted innocent and young Johnny had to <ex>take the heat</ex></as>.</def><br/
[<source>PJC</source>]</p>

<p><sn>2.</sn> <def>To withstand high temperature; <as>as, the new radios can <ex>take the heat</ex> of a black car parked in the sun in Death Valley</as>.</def><br/
[<source>PJC</source>]</p>

<p><sn>3.</sn> <specif>Hence:</specif> <def>To endure stressful conditions, expecially without signs of difficulty or complaint; <as>as, if you can't <ex>take the heat</ex>, stay out of the kitchen</as>,</def><br/
[<source>PJC</source>]</p>

<p><ent>Take-up</ent><br/
<hw>Take"-up`</hw> <pr>(t<amac/k"<ucr/p`)</pr>, <pos>n.</pos> <fld>(Mach.)</fld> <def>That which takes up or tightens; specifically, a device in a sewing machine for drawing up the slack thread as the needle rises, in completing a stitch.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Taking</ent><br/
<hw>Tak"ing</hw> <pr>(?)</pr>, <pos>a.</pos> <sn>1.</sn> <def>Apt to take; alluring; attracting.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Subtile in making his temptations most <qex>taking</qex>.</q> <rj><qau>Fuller.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Infectious; contageous.</def> <mark>[Obs.]</mark>  <rj><au>Beau. & Fl.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p>-- <wordforms><wf>Tak"ing*ly</wf>, <pos>adv.</pos> -- <wf>Tak"ing*ness</wf>, <pos>n.</pos></wordforms><br/
[<source>1913 Webster</source>]</p>

<p><ent>Taking</ent><br/
<hw>Tak"ing</hw>, <pos>n.</pos> <sn>1.</sn> <def>The act of gaining possession; a seizing; seizure; apprehension.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Agitation; excitement; distress of mind.</def> <mark>[Colloq.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>What a <qex>taking</qex> was he in, when your husband asked who was in the basket!</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>Malign influence; infection.</def> <mark>[Obs.]</mark>  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Taking-off</ent><br/
<hw>Tak"ing-off`</hw> <pr>(?)</pr>, <pos>n.</pos> <sn>1.</sn> <def>Removal; murder. See <cref>To take off</cref> <sd>(c)</sd>, under <er>Take</er>, <pos>v. t.</pos></def><br/
[<source>1913 Webster</source>]</p>

<p><q>The deep damnation of his <qex>taking-off</qex>.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Print.)</fld> <def>The removal of sheets from the press.</def> <mark>[Eng.]</mark><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><sn>3.</sn>  <def>Act of presenting a take-off, or burlesque imitation.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Talapoin</ent><br/
<hw>Tal"a*poin</hw> <pr>(t<acr/l"<adot/*poin)</pr>, <pos>n.</pos> <fld>(Zool.)</fld> <def>A small African monkey (<spn>Cercopithecus talapoin</spn> or <spn>Miopithecus talapoin</spn>) -- called also <altname>melarhine</altname>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Talapoin</ent><br/
||<hw>Tal"a*poin</hw> <pr>(t<acr/l"<adot/*poin)</pr>, <pos>n.</pos> <ety>[Pg. <ets>talapoi</ets>, <ets>talapoin</ets>, name for Buddhist priest, fr. Siamese <ets>t<lsquo/ama p<lsquo/r<acr/</ets>; <ets>t<lsquo/ama</ets>, honorific title + <ets>p<lsquo/r<acr/</ets> priest.]</ety> <def>A Buddhist monk or priest.</def> <mark>[Ceylon & Indo-China]</mark><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Talaria</ent><br/
||<hw>Ta*la"ri*a</hw> <pr>(t<adot/*l<amac/"r<icr/*<adot/)</pr>, <pos>n. pl.</pos> <ety>[L., from <ets>talaris</ets> pertaining to the ankles, fr. <ets>talus</ets> ankle.]</ety> <fld>(Class. Myth.)</fld> <def>Small wings or winged shoes represented as fastened to the ankles, -- chiefly used as an attribute of Mercury.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Talbot</ent><br/
<hw>Tal"bot</hw> <pr>(t<add/l"b<ucr/t)</pr>, <pos>n.</pos> <def>A sort of dog, noted for quick scent and eager pursuit of game.</def> <mark>[Obs.]</mark>  <rj><au>Wase (1654).</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ The figure of a dog is borne in the arms of the <xex>Talbot</xex> family, whence, perhaps, the name.</note><br/
[<source>1913 Webster</source>]</p>

<p><ent>Talbotype</ent><br/
<hw>Tal"bo*type</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Photog.)</fld> <def>Same as <er>Calotype</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Talc</ent><br/
<hw>Talc</hw> <pr>(t<acr/lk)</pr>, <pos>n.</pos> <ety>[F. <ets>talc</ets>; cf. Sp. & It. <ets>talco</ets>, LL. <ets>talcus</ets>; all fr. Ar. <ets>talq</ets>.]</ety> <fld>(Min.)</fld> <def>A soft mineral of a soapy feel and a greenish, whitish, or grayish color, usually occurring in foliated masses. It is hydrous silicate of magnesia. <stype>Steatite</stype>, or <stype>soapstone</stype>, is a compact granular variety.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Indurated talc</b></col>, <cd>an impure, slaty talc, with a nearly compact texture, and greater hardness than common talc; -- called also <altname>talc slate</altname>.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Talcous</ent><br/
<ent>Talcose</ent><br/
<mhw>{ <hw>Tal*cose"</hw> <pr>(t<acr/l*k<omac/s" or t<acr/lk"<omac/s`)</pr>, <hw>Talc"ous</hw> <pr>(t<acr/lk"<ucr/s)</pr>, }</mhw> <pos>a.</pos> <ety>[Cf. F. <ets>talqueux</ets>.]</ety> <fld>(Min.)</fld> <def>Of or pertaining to talc; composed of, or resembling, talc.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Talcum</ent><br/
||<hw>Tal"cum</hw> <pr>(t<acr/l"k<ucr/m)</pr>, <pos>n.</pos> <ety>[NL.]</ety> <fld>(Min.)</fld> <def>Same as <er>Talc</er>.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Tale</ent><br/
<hw>Tale</hw> <pr>(t<amac/l)</pr>, <pos>n.</pos> <def>See <er>Tael</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tale</ent><br/
<hw>Tale</hw>, <pos>n.</pos> <ety>[AS. <ets>talu</ets> number, speech, narrative; akin to D. <ets>taal</ets> speech, language, G. <ets>zahl</ets> number, OHG. <ets>zala</ets>, Icel. <ets>tal</ets>, <ets>tala</ets>, number, speech, Sw. <ets>tal</ets>, Dan. <ets>tal</ets> number, <ets>tale</ets> speech, Goth. <ets>talzjan</ets> to instruct.  Cf. <er>Tell</er>, <pos>v. t.</pos>, <er>Toll</er> a tax, also <er>Talk</er>, <pos>v. i.</pos>]</ety> <sn>1.</sn> <def>That which is told; an oral relation or recital; any rehearsal of what has occured; narrative; discourse; statement; history; story.</def> <ldquo/The <xex>tale</xex> of Troy divine.<rdquo/ <au>Milton.</au> <ldquo/In such manner rime is Dante's <xex>tale</xex>.<rdquo/ <au>Chaucer.</au><br/
[<source>1913 Webster</source>]</p>

<p><q>We spend our years as a <qex>tale</qex> that is told.</q> <rj><qau>Ps. xc. 9.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A number told or counted off; a reckoning by count; an enumeration; a count, in distinction from measure or weight; a number reckoned or stated.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The ignorant, . . . who measure by <qex>tale</qex>, and not by weight.</q> <rj><qau>Hooker.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>And every shepherd tells his <qex>tale</qex>,<br/
Under the hawthorn in the dale.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>In packing, they keep a just <qex>tale</qex> of the number.</q> <rj><qau>Carew.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <fld>(Law)</fld> <def>A count or declaration.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>To tell tale of</b></col>, <cd>to make account of.</cd> <mark>[Obs.]</mark></cs><br/
[<source>1913 Webster</source>]</p>

<p><q>Therefore little <qex>tale</qex> hath he <qex>told</qex><br/
<qex>Of</qex> any dream, so holy was his heart.</q> <rj><qau>Chaucer.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><syn><b>Syn.</b> -- Anecdote; story; fable; incident; memoir; relation; account; legend; narrative.</syn><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tale</ent><br/
<hw>Tale</hw> <pr>(t<amac/l)</pr>, <pos>v. i.</pos> <def>To tell stories.</def> <mark>[Obs.]</mark>  <rj><au>Chaucer.</au>  <au>Gower.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Talebearer</ent><br/
<hw>Tale"bear`er</hw> <pr>(t<amac/l"b<acir/r`<etil/r)</pr>, <pos>n.</pos> <def>One who officiously tells tales; one who impertinently or maliciously communicates intelligence, scandal, etc., and makes mischief.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Spies and <qex>talebearers</qex>, encouraged by her father, did their best to inflame her resentment.</q> <rj><qau>Macaulay.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Talebearing</ent><br/
<hw>Tale"bear`ing</hw>, <pos>a.</pos> <def>Telling tales officiously.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Talebearing</ent><br/
<hw>Tale"bear`ing</hw>, <pos>n.</pos> <def>The act of informing officiously; communication of secrets, scandal, etc., maliciously.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Taled</ent><br/
<hw>Ta"led</hw> <pr>(t<amac/"l<ecr/d)</pr>, <pos>n.</pos> <fld>(Jewish Antiq.)</fld> <def>A kind of quadrangular piece of cloth put on by the Jews when repeating prayers in the synagogues.</def>  <rj><au>Crabb.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Taleful</ent><br/
<hw>Tale"ful</hw> <pr>(t<amac/l"f<usdot/l)</pr>, <pos>a.</pos> <def>Full of stories.</def> <mark>[R.]</mark>  <rj><au>Thomson.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Talegalla</ent><br/
||<hw>Tal`e*gal"la</hw> <pr>(t<amac/l`<esl/*g<acr/l"l<adot/)</pr>, <pos>prop. n.</pos> <ety>[NL.]</ety> <fld>(Zool.)</fld> <def>A genus of Australian birds which includes the brush turkey. See <er>Brush turkey</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Talent</ent><br/
<hw>Tal"ent</hw> <pr>(t<acr/l"<eit/nt)</pr>, <pos>n.</pos> <ety>[F., fr. L. <ets>talentum</ets> a talent (in sense 1), Gr. <grk>ta`lanton</grk> a balance, anything weighed, a definite weight, a talent; akin to <grk>tlh^nai</grk> to bear, endure, <grk>tolma^n</grk>, L. <ets>tolerare</ets>, <ets>tollere</ets>, to lift up, sustain, endure. See <er>Thole</er>, <pos>v. t.</pos>, <er>Tolerate</er>.]</ety> <sn>1.</sn> <def>Among the ancient Greeks, a weight and a denomination of money equal to 60 minae or 6,000 drachmae.  The Attic talent, as a weight, was about 57 lbs. avoirdupois; as a denomination of silver money, its value was <pound/243 15s. sterling, or about $1,180 (using 1900 values).</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Rowing vessel whose burden does not exceed five hundred <qex>talents</qex>.</q> <rj><qau>Jowett (Thucid.).</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Among the Hebrews, a weight and denomination of money. For silver it was equivalent to 3,000 shekels, and in weight was equal to about 93<frac34/ lbs. avoirdupois; as a denomination of silver, it has been variously estimated at from <pound/340 to <pound/396 sterling, or about $1,645 to $1,916 (ca. 1900).  For gold it was equal to 10,000 gold shekels.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>Inclination; will; disposition; desire.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>They rather counseled you to your <qex>talent</qex> than to your profit.</q> <rj><qau>Chaucer.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>Intellectual ability, natural or acquired; mental endowment or capacity; skill in accomplishing; a special gift, particularly in business, art, or the like; faculty; a use of the word probably originating in the Scripture parable of the talents (<au>Matt. xxv. 14-30</au>).</def><br/
[<source>1913 Webster</source>]</p>

<p><q>He is chiefly to be considered in his three different <qex>talents</qex>, as a critic, a satirist, and a writer of odes.</q> <rj><qau>Dryden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>His <qex>talents</qex>, his accomplishments, his graceful manners, made him generally popular.</q> <rj><qau>Macaulay.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><syn><b>Syn.</b> -- Ability; faculty; gift; endowment. See <er>Genius</er>.</syn><br/
[<source>1913 Webster</source>]</p>

<p><ent>Talented</ent><br/
<hw>Tal"ent*ed</hw>, <pos>a.</pos> <def>Furnished with talents; possessing skill or talent; mentally gifted.</def>  <rj><au>Abp. Abbot (1663).</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ This word has been strongly objected to by Coleridge and some other critics, but, as it would seem, upon not very good grounds, as the use of <xex>talent</xex> or <xex>talents</xex> to signify mental ability, although at first merely metaphorical, is now fully established, and <xex>talented</xex>, as a formative, is just as analogical and legitimate as <xex>gifted</xex>, <xex>bigoted</xex>, <xex>moneyed</xex>, <xex>landed</xex>, <xex>lilied</xex>, <xex>honeyed</xex>, and numerous other adjectives having a participal form, but derived directly from nouns and not from verbs.</note><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tales</ent><br/
||<hw>Ta"les</hw> <pr>(t<amac/"l<emac/z)</pr>, <pos>n.</pos> <ety>[L., pl. of <ets>talis</ets> such (persons).]</ety> <fld>(Law)</fld> <sd>(a)</sd> <pluf>pl.</pluf> <def>Persons added to a jury, commonly from those in or about the courthouse, to make up any deficiency in the number of jurors regularly summoned, being like, or <xex>such</xex> as, the latter.</def> <au>Blount.</au>  <au>Blackstone.</au> <sd>(b)</sd> <pos>syntactically sing.</pos> <def>The writ by which such persons are summoned.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Tales book</b></col>, <cd>a book containing the names of such as are admitted of the tales.</cd> <au>Blount.</au> <au>Craig.</au> -- ||<col><b>Tales de circumstantibus</b></col> <ety>[L.]</ety>, <cd>such, or the like, from those standing about.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Talesman</ent><br/
<hw>Tales"man</hw> <pr>(?)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Talesmen</plw> <pr>(<?/)</pr>.</plu> <fld>(Law)</fld> <def>A person called to make up a deficiency in the number of jurors when a tales is awarded.</def>  <rj><au>Wharton.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Taleteller</ent><br/
<hw>Tale"tell`er</hw> <pr>(?)</pr>, <pos>n.</pos> <def>One who tells tales or stories, especially in a mischievous or officious manner; a talebearer; a telltale; a tattler.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Talewise</ent><br/
<hw>Tale"wise`</hw> <pr>(?)</pr>, <pos>adv.</pos> <def>In a way of a tale or story.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Taliacotian</ent><br/
<hw>Tal"ia*co`tian</hw> <pr>(?)</pr>, <pos>a.</pos> <def>See <er>Tagliacotian</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Taliation</ent><br/
<hw>Tal`i*a"tion</hw> <pr>(?)</pr>, <pos>n.</pos> <def>Retaliation.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>Just heav'n this <qex>taliation</qex> did decree.</q> <rj><qau>Beaumont.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Talion</ent><br/
<hw>Ta"li*on</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F., fr. L. <ets>talio</ets>, perh. fr. <ets>talis</ets> such.  Cf. <er>Retaliation</er>.]</ety> <def>Retaliation.</def> <mark>[R.]</mark>  <rj><au>Holinshed.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Talipes</ent><br/
||<hw>Tal"i*pes</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[NL., fr. L. <ets>talus</ets> an ankle + <ets>pes</ets>, <ets>pedis</ets>, a foot; cf. L. <ets>talipedare</ets> to be weak in the feet, properly, to walk on the ankles.]</ety> <fld>(Surg.)</fld> <def>The deformity called <altname>clubfoot</altname>. See <er>Clubfoot</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ Several varieties are distinguished; as, <stype>Talipes varus</stype>, in which the foot is drawn up and bent inward; <stype>T. valgus</stype>, in which the foot is bent outward; <stype>T. equinus</stype>, in which the sole faces backward and the patient walks upon the balls of the toes; and <stype>T. calcaneus</stype> (called also <stype>talus</stype>), in which the sole faces forward and the patient walks upon the heel.</note><br/
[<source>1913 Webster</source>]</p>

<p><ent>Talipot</ent><br/
<hw>Tal"i*pot</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Hind. <ets>t<amac/lp<amac/t</ets> the leaf of the tree.]</ety> <fld>(Bot.)</fld> <def>A beautiful tropical palm tree (<spn>Corypha umbraculifera</spn>), a native of Ceylon and the Malabar coast. It has a trunk sixty or seventy feet high, bearing a crown of gigantic fan-shaped leaves which are used as umbrellas and as fans in ceremonial processions, and, when cut into strips, as a substitute for writing paper.</def><br/
[<source>1913 Webster</source>]</p>

<p><-- p. 1471 --><br/
[<source>1913 Webster</source>]</p>

<p><ent>Talisman</ent><br/
<hw>Tal"is*man</hw> <pr>(?)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Talismans</plw> <pr>(#)</pr>.</plu> <ety>[Sp., from Ar. <ets>tilism</ets>, <ets>tilsam</ets>, a magical image, pl. <ets>tilsam<amac/n</ets>, fr. Gr. <?/ tribute, tax, LGr., an initiation, incantation, from <?/ to complete, perform, to play taxes, to make perfect, to initiate, especially in the mysteries, fr. <?/ completion, end.]</ety> <sn>1.</sn> <def>A magical figure cut or engraved under certain superstitious observances of the configuration of the heavens, to which wonderful effects are ascribed; the seal, figure, character, or image, of a heavenly sign, constellation, or planet, engraved on a sympathetic stone, or on a metal corresponding to the star, in order to receive its influence.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Hence, something that produces extraordinary effects, esp. in averting or repelling evil; an amulet; a charm; <as>as, a <ex>talisman</ex> to avert diseases</as>.</def>  <rj><au>Swift.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Talismanical</ent><br/
<ent>Talismanic</ent><br/
<mhw>{ <hw>Tal`is*man"ic</hw> <pr>(?)</pr>, <hw>Tal`is*man"ic*al</hw> <pr>(?)</pr>, }</mhw> <pos>a.</pos> <ety>[Cf. F. <ets>talismanique</ets>.]</ety> <def>Of or pertaining to a talisman; having the properties of a talisman, or preservative against evils by occult influence; magical.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>talk</ent><br/
<hw>talk</hw> <pr>(t<add/k)</pr>, <pos>v. i.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>talked</conjf> <pr>(t<add/kt)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>talking</conjf>.]</vmorph> <ety>[Cf. LG. <ets>talk</ets> talk, gabble, Prov. G. <ets>talken</ets> to speak indistinctly; or OD. <ets>tolken</ets> to interpret, MHG. <ets>tolkan</ets> to interpret, to tell, to speak indistinctly, Dan. <ets>tolke</ets> to interpret, Sw. <ets>tolka</ets>, Icel. <ets>t<umac/lka</ets> to interpret, <ets>t<umac/lkr</ets> an interpreter, Lith. <ets>tulkas</ets> an interpreter, <ets>tulkanti</ets>, <ets>tulk<omac/ti</ets>, to interpret, Russ. <ets>tolkovate</ets> to interpret, to talk about; or perhaps fr. OE. <ets>talien</ets> to speak (see <er>tale</er>, <pos>v. i.</pos> & <pos>n.</pos>).]</ety> <sn>1.</sn> <def>To utter words; esp., to converse familiarly; to speak, as in familiar discourse, when two or more persons interchange thoughts.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>I will buy with you, sell with you, <qex>talk</qex> with you, walk with you, and so following, but I will not eat with you.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To confer; to reason; to consult.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Let me <qex>talk</qex> with thee of thy judgments.</q> <rj><qau>Jer. xii. 1.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>To prate; to speak impertinently.</def> <mark>[Colloq.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>To talk of</b></col>, <cd>to relate; to tell; to give an account of; <as>as, authors <ex>talk of</ex> the wonderful remains of Palmyra</as>.</cd> <ldquo/The natural histories of Switzerland <xex>talk</xex> much <xex>of</xex> the fall of these rocks, and the great damage done.<rdquo/ <au>Addison.</au> -- <col><b>To talk to</b></col>, <cd>to advise or exhort, or to reprove gently; <as>as, I will <ex>talk to</ex> my son respecting his conduct</as>.</cd> <mark>[Colloq.]</mark></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Talk</ent><br/
<hw>Talk</hw>, <pos>v. t.</pos> <sn>1.</sn> <def>To speak freely; to use for conversing or communicating; <as>as, to <ex>talk</ex> French</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To deliver in talking; to speak; to utter; to make a subject of conversation; <as>as, to <ex>talk</ex> nonsense; to <ex>talk</ex> politics</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>To consume or spend in talking; -- often followed by <xex>away</xex>; <as>as, to <ex>talk</ex> away an evening</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>To cause to be or become by talking.</def> <ldquo/They would <xex>talk</xex> themselves mad.<rdquo/  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>To talk over</b></col>. <sd>(a)</sd> <cd>To talk about; to have conference respecting; to deliberate upon; to discuss; <as>as, to <ex>talk over</ex> a matter or plan</as>.</cd> <sd>(b)</sd> <cd>To change the mind or opinion of by talking; to convince; <as>as, to <ex>talk over</ex> an opponent</as>.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Talk</ent><br/
<hw>Talk</hw>, <pos>n.</pos> <sn>1.</sn> <def>The act of talking; especially, familiar converse; mutual discourse; that which is uttered, especially in familiar conversation, or the mutual converse of two or more.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>In various <qex>talk</qex> the instructive hours they passed.</q> <rj><qau>Pope.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Their <qex>talk</qex>, when it was not made up of nautical phrases, was too commonly made up of oaths and curses.</q> <rj><qau>Macaulay.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Report; rumor; <as>as, to hear <ex>talk</ex> of war</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>I hear a <qex>talk</qex> up and down of raising our money.</q> <rj><qau>Locke.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>Subject of discourse; <as>as, his achievment is the <ex>talk</ex> of the town</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><syn><b>Syn.</b> -- Conversation; colloquy; discourse; chat; dialogue; conference; communication. See <er>Conversation</er>.</syn><br/
[<source>1913 Webster</source>]</p>

<p><ent>Talkative</ent><br/
<hw>Talk"a*tive</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Given to much talking.</def><br/
[<source>1913 Webster</source>]</p>

<p><syn><b>Syn.</b> -- Garrulous; loquacious. See <er>Garrulous</er>.</syn><br/
[<source>1913 Webster</source>]</p>

<p>-- <wordforms><wf>Talk"a*tive*ly</wf>, <pos>adv.</pos> -- <wf>Talk"a*tive*ness</wf>, <pos>n.</pos></wordforms><br/
[<source>1913 Webster</source>]</p>

<p><ent>Talker</ent><br/
<hw>Talk"er</hw> <pr>(?)</pr>, <pos>n.</pos> <sn>1.</sn> <def>One who talks; especially, one who is noted for his power of conversing readily or agreeably; a conversationist.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>There probably were never four <qex>talkers</qex> more admirable in four different ways than Johnson, Burke, Beauclerk, and Garrick.</q> <rj><qau>Macaulay.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A loquacious person, male or female; a prattler; a babbler; also, a boaster; a braggart; -- used in contempt or reproach.</def>  <rj><au>Jer. Taylor.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Talking</ent><br/
<hw>Talk"ing</hw>, <pos>a.</pos> <sn>1.</sn> <def>That talks; able to utter words; <as>as, a <ex>talking</ex> parrot</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Given to talk; loquacious.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The hawthorn bush, with seats beneath the shade,<br/
For <qex>talking</qex> age and whispering lovers made.</q> <rj><qau>Goldsmith.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tall</ent><br/
<hw>Tall</hw> <pr>(?)</pr>, <pos>a.</pos> <amorph>[<pos>Compar.</pos> <adjf>Taller</adjf> <pr>(?)</pr>; <pos>superl.</pos> <adjf>Tallest</adjf>.]</amorph> <ety>[OE. <ets>tal</ets> seemly, elegant, docile (?); of uncertain origin; cf. AS. un-<ets>tala</ets>, un-<ets>tale</ets>, bad, Goth. un<ets>tals</ets> indocile, disobedient, uninstructed, or W. & Corn. <ets>tal</ets> high, Ir. <ets>talla</ets> meet, fit, proper, just.]</ety> <sn>1.</sn> <def>High in stature; having a considerable, or an unusual, extension upward; long and comparatively slender; having the diameter or lateral extent small in proportion to the height; <as>as, a <ex>tall</ex> person, tree, or mast</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Two of far nobler shape, erect and <qex>tall</qex>.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Brave; bold; courageous.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>As <qex>tall</qex> a trencherman<br/
As e'er demolished a pye fortification.</q> <rj><qau>Massinger.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>His companions, being almost in despair of victory, were suddenly recomforted by Sir William Stanley, which came to succors with three thousand <qex>tall</qex> men.</q> <rj><qau>Grafton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>Fine; splendid; excellent; also, extravagant; excessive.</def> <mark>[Obs. or Slang]</mark>  <rj><au>B. Jonson.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><syn><b>Syn.</b> -- High; lofty.</syn> <usage> -- <er>Tall</er>, <er>High</er>, <er>Lofty</er>. <xex>High</xex> is the generic term, and is applied to anything which is elevated or raised above another thing. <xex>Tall</xex> specifically describes that which has a small diameter in proportion to its height; hence, we speak of a <xex>tall</xex> man, a <xex>tall</xex> steeple, a <xex>tall</xex> mast, etc., but not of a <xex>tall</xex> hill. <xex>Lofty</xex> has a special reference to the expanse above us, and denotes an imposing height; <as>as, a <ex>lofty</ex> mountain; a <ex>lofty</ex> room</as>. <xex>Tall</xex> is now properly applied only to physical objects; <xex>high</xex> and <xex>lofty</xex> have a moral acceptation; <as>as, <ex>high</ex> thought, purpose, etc.</as>; <xex>lofty</xex> aspirations; a <xex>lofty</xex> genius. <xex>Lofty</xex> is the stronger word, and is usually coupled with the grand or admirable.</usage><br/
[<source>1913 Webster</source>]</p>

<p><ent>Talliage</ent><br/
<ent>Tallage</ent><br/
<mhw>{ <hw>Tal"lage</hw> <pr>(?)</pr>, <hw>Tal"li*age</hw> <pr>(?)</pr>, }</mhw> <pos>n.</pos> <ety>[F. <ets>taillage</ets>. See <er>Taille</er>, and cf. <er>Tailage</er>.]</ety> <fld>(O. Eng. Law)</fld> <def>A certain rate or tax paid by barons, knights, and inferior tenants, toward the public expenses.</def> <altsp>[Written also <asp>tailage</asp>, <asp>taillage</asp>.]</altsp><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ When paid out of knight's fees, it was called <stype>scutage</stype>; when by cities and burghs, <stype>tallage</stype>; when upon lands not held by military tenure, <stype>hidage</stype>.</note>  <rj><au>Blackstone.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tallage</ent><br/
<hw>Tal"lage</hw>, <pos>v. t.</pos> <def>To lay an impost upon; to cause to pay tallage.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tallboy</ent><br/
<hw>Tall"boy`</hw> <pr>(?)</pr>, <pos>n.</pos> <sn>1.</sn> <def>A kind of long-stemmed wineglass or cup.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><sn>2.</sn>  <def>A piece of household furniture common in the eighteenth century, usually in two separate parts, with larger drawers above and smaller ones below and raised on legs fifteen inches or more in height; -- called also <altname>highboy</altname>.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><sn>3.</sn>  <def>A long sheet-metal pipe for a chimney top.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Tallier</ent><br/
<hw>Tal"li*er</hw> <pr>(?)</pr>, <pos>n.</pos> <def>One who keeps tally.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tallis</ent><br/
||<hw>Tal"lis</hw> <pr>(?)</pr>, <pos>n.</pos> <def>Same as <er>Tallith</er>.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Tallith</ent><br/
||<hw>Tal"lith</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[NHeb. <ets>tall<imac/th</ets>.]</ety> <fld>(Jewish Costume)</fld> <sd>(a)</sd> <def>An undergarment worn by orthodox Jews, covering the chest and the upper part of the back. It has an opening for the head, and has tassels, called <part>zizith</part>, on its four corners.</def> <sd>(b)</sd> <def>A tasseled shawl or scarf worn over the head or thrown round the shoulders while at prayer.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Tallness</ent><br/
<hw>Tall"ness</hw> <pr>(?)</pr>, <pos>n.</pos> <def>The quality or state of being tall; height of stature.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tallow</ent><br/
<hw>Tal"low</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[OE. <ets>taluh</ets>, <ets>talugh</ets>; akin to OD. <ets>talgh</ets>, D. <ets>talk</ets>, G., Dan. and Sw. <ets>talg</ets>, Icel. <ets>t<omac/lgr</ets>, <ets>t<omac/lg</ets>, <ets>t<omac/lk</ets>; and perhaps to Goth. <ets>tulgus</ets> firm.]</ety> <sn>1.</sn> <def>The suet or fat of animals of the sheep and ox kinds, separated from membranous and fibrous matter by melting.</def><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ The solid consistency of tallow is due to the large amount of stearin it contains. See <er>Fat</er>.</note><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>The fat of some other animals, or the fat obtained from certain plants, or from other sources, resembling the fat of animals of the sheep and ox kinds.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Tallow candle</b></col>, <cd>a candle made of tallow.</cd> -- <col><b>Tallow catch</b></col>, <cd>a keech.</cd> See <er>Keech</er>. <mark>[Obs.]</mark> -- <col><b>Tallow chandler</b></col>, <cd>one whose occupation is to make, or to sell, tallow candles.</cd> -- <col><b>Tallow chandlery</b></col>, <cd>the trade of a tallow chandler; also, the place where his business is carried on.</cd> -- <col><b>Tallow tree</b></col> <fld>(Bot.)</fld>, <cd>a tree (<spn>Stillingia sebifera</spn>) growing in China, the seeds of which are covered with a substance which resembles tallow and is applied to the same purposes.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tallow</ent><br/
<hw>Tal"low</hw>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Tallowed</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Tallowing</conjf>.]</vmorph> <sn>1.</sn> <def>To grease or smear with tallow.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To cause to have a large quantity of tallow; to fatten; <as>as, <ex>tallow</ex> sheep</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tallower</ent><br/
<hw>Tal"low*er</hw> <pr>(?)</pr>, <pos>n.</pos> <def>An animal which produces tallow.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tallow-face</ent><br/
<hw>Tal"low-face`</hw> <pr>(?)</pr>, <pos>n.</pos> <def>One who has a sickly, pale complexion.</def>  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tallow-faced</ent><br/
<hw>Tal"low-faced`</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Having a sickly complexion; pale.</def>  <rj><au>Burton.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tallowing</ent><br/
<hw>Tal"low*ing</hw>, <pos>n.</pos> <def>The act, or art, of causing animals to produce tallow; also, the property in animals of producing tallow.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tallowish</ent><br/
<hw>Tal"low*ish</hw>, <pos>a.</pos> <def>Having the qualities of tallow.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tallowy</ent><br/
<hw>Tal"low*y</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Of the nature of tallow; resembling tallow; greasy.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tallwood</ent><br/
<hw>Tall"wood`</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Cf. <er>Tally</er>.]</ety> <def>Firewood cut into billets of a certain length.</def> <mark>[Obs.]</mark> <mark>[Eng.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tally</ent><br/
<hw>Tal"ly</hw> <pr>(?)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Tallies</plw> <pr>(#)</pr>.</plu> <ety>[OE. <ets>taile</ets>, <ets>taille</ets>, F. <ets>taille</ets> a cutting, cut tally, fr. <ets>tailler</ets> to cut, but influenced probably by <ets>taill<eacute/</ets>, p. p. of <ets>tailler</ets>. See <er>Tailor</er>, and cf. <er>Tail</er> a limitation, <er>Taille</er>, <er>Tallage</er>.]</ety> <sn>1.</sn> <def>Originally, a piece of wood on which notches or scores were cut, as the marks of number; later, one of two books, sheets of paper, etc., on which corresponding accounts were kept.</def><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ In purshasing and selling, it was once customary for traders to have two sticks, or one stick cleft into two parts, and to mark with a score or notch, on each, the number or quantity of goods delivered, -- the seller keeping one stick, and the purchaser the other. Before the use of writing, this, or something like it, was the only method of keeping accounts; and <xex>tallies</xex> were received as evidence in courts of justice. In the English exchequer were <xex>tallies</xex> of loans, one part being kept in the exchequer, the other being given to the creditor in lieu of an obligation for money lent to government.</note><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Hence, any account or score kept by notches or marks, whether on wood or paper, or in a book; especially, one kept in duplicate.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>One thing made to suit another; a match; a mate.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>They were framed the <qex>tallies</qex> for each other.</q> <rj><qau>Dryden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>A notch, mark, or score made on or in a tally; <as>as, to make or earn a <ex>tally</ex> in a game</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <def>A tally shop. See <cref>Tally shop</cref>, below.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Tally shop</b></col>, <cd>a shop at which goods or articles are sold to customers on account, the account being kept in corresponding books, one called the <xex>tally</xex>, kept by the buyer, the other the <xex>counter tally</xex>, kept by the seller, and the payments being made weekly or otherwise by agreement. The trade thus regulated is called <xex>tally trade</xex>.</cd> <au>Eng. Encyc.</au> -- <col><b>To strike tallies</b></col>, <cd>to act in correspondence, or alike.</cd> <mark>[Obs.]</mark> <au>Fuller.</au></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tally</ent><br/
<hw>Tal"ly</hw>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Tallied</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Tallying</conjf>.]</vmorph> <ety>[Cf. F. <ets>tialler</ets> to cut. See <er>Tally</er>, <pos>n.</pos>]</ety> <sn>1.</sn> <def>To score with correspondent notches; hence, to make to correspond; to cause to fit or suit.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>They are not so well <qex>tallied</qex> to the present juncture.</q> <rj><qau>Pope.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Naut.)</fld> <def>To check off, as parcels of freight going inboard or outboard.</def>  <rj><au>W. C. Russell.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Tally on</b></col> <fld>(Naut.)</fld>, <cd>to dovetail together.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tally</ent><br/
<hw>Tal"ly</hw> <pr>(?)</pr>, <pos>v. i.</pos> <sn>1.</sn> <def>To be fitted; to suit; to correspond; to match.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>I found pieces of tiles that exactly <qex>tallied</qex> with the channel.</q> <rj><qau>Addison.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Your idea . . . <qex>tallies</qex> exactly with mine.</q> <rj><qau>Walpole.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To make a tally; to score; <as>as, to <ex>tally</ex> in a game</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Tally on</b></col> <fld>(Naut.)</fld>, <cd>to man a rope for hauling, the men standing in a line or tail.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tally</ent><br/
<hw>Tal"ly</hw> <pr>(?)</pr>, <pos>adv.</pos> <ety>[See <er>Tall</er>, <pos>a.</pos>]</ety> <def>Stoutly; with spirit.</def> <mark>[Obs.]</mark>  <rj><au>Beau. & Fl.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tallyho</ent><br/
<hw>Tal"ly*ho`</hw> <pr>(?)</pr>, <pos>interj. & n.</pos> <sn>1.</sn> <def>The huntsman's cry to incite or urge on his hounds.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A tallyho coach.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Tallyho coach</b></col>, <cd>a pleasure coach. See under <er>Coach</er>.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tallyman</ent><br/
<hw>Tal"ly*man</hw> <pr>(?)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Tallymen</plw> <pr>(<?/)</pr>.</plu> <sn>1.</sn> <def>One who keeps the tally, or marks the sticks.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>One who keeps a tally shop, or conducts his business as tally trade.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Talma</ent><br/
<hw>Tal"ma</hw> <pr>(?)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Talmas</plw> <pr>(#)</pr>.</plu> <ety>[Prob. so called from <ets>Talma</ets>, a French actor.]</ety> <sd>(a)</sd> <def>A kind of large cape, or short, full cloak, forming part of the dress of ladies.</def> <sd>(b)</sd> <def>A similar garment worn formerly by gentlemen.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Talmud</ent><br/
<hw>Tal"mud</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Chald. <ets>talm<umac/d</ets> instruction, doctrine, fr. <ets>lamad</ets> to learn, <ets>limmad</ets> to teach.]</ety> <def>The body of the Jewish civil and canonical law not comprised in the Pentateuch.</def><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ The <xex>Talmud</xex> consists of two parts, the <xex>Mishna</xex>, or text, and the <xex>Gemara</xex>, or commentary. Sometimes, however, the name <xex>Talmud</xex> is restricted, especially by Jewish writers, to the Gemara. There are two Talmuds, the <xex>Palestinian</xex>, commonly, but incorrectly, called the <xex>Talmud of Jerusalem</xex>, and the <xex>Babylonian Talmud</xex>. They contain the same Mishna, but different Gemaras. The Babylonian Talmud is about three times as large as the other, and is more highly esteemed by the Jews.</note><br/
[<source>1913 Webster</source>]</p>

<p><ent>Talmudical</ent><br/
<ent>Talmudic</ent><br/
<mhw>{ <hw>Tal*mud"ic</hw> <pr>(?)</pr>, <hw>Tal*mud"ic*al</hw> <pr>(?)</pr>, }</mhw> <pos>a.</pos> <ety>[Cf. F. <ets>talmudique</ets>.]</ety> <def>Of or pertaining to the Talmud; contained in the Talmud; <as>as, <ex>Talmudic</ex> Greek; <ex>Talmudical</ex> phrases.</as></def>  <rj><au>Lightfoot.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Talmudism</ent><br/
<hw>Tal"mud*ism</hw> <pr>(?)</pr>, <pos>n.</pos>  <pr>(<?/)</pr>, <pos>n.</pos> <def>The teachings of the Talmud, or adherence to them.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Talmudist</ent><br/
<hw>Tal"mud*ist</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Cf. F. <ets>talmudiste</ets>.]</ety> <def>One versed in the Talmud; one who adheres to the teachings of the Talmud.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Talmudistic</ent><br/
<hw>Tal`mud*is"tic</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Resembling the Talmud; Talmudic.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Talon</ent><br/
<hw>Tal"on</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F., heel, spur, LL. <ets>talo</ets>, fr. L. <ets>talus</ets> the ankle, heel.]</ety> <sn>1.</sn> <def>The claw of a predaceous bird or animal, especially the claw of a bird of prey.</def>  <rj><au>Bacon.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Zool.)</fld> <def>One of certain small prominences on the hind part of the face of an elephant's tooth.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <fld>(Arch.)</fld> <def>A kind of molding, concave at the bottom and convex at the top; -- usually called an <altname>ogee</altname>.</def><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ When the concave part is at the top, it is called an <stype>inverted talon</stype>.</note><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>The shoulder of the bolt of a lock on which the key acts to shoot the bolt.</def>  <rj><au>Knight.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Taluk</ent><br/
<ent>Talook</ent><br/
<mhw>{ <hw>Ta*look"</hw>, <hw>Ta*luk"</hw>  }</mhw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Ar. <ets>ta'lluq</ets>.]</ety> <def>A large estate; esp., one constituting a revenue district or dependency the native proprietor of which is responsible for the collection and payment of the public revenue due from it.</def> <mark>[India]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Talukdar</ent><br/
<ent>Talookdar</ent><br/
<mhw>{ <hw>Ta*look"dar</hw>, <hw>Ta*luk"dar</hw>  }</mhw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Hind., fr. Per. <ets>ta'lluqd<amac/r</ets>.]</ety> <def>A proprietor of a talook.</def> <mark>[India]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Talpa</ent><br/
||<hw>Tal"pa</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L., mole.]</ety> <fld>(Zool.)</fld> <def>A genus of small insectivores including the common European mole.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Talus</ent><br/
||<hw>Ta"lus</hw> <pr>(?)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Tali</plw> <pr>(#)</pr>.</plu> <ety>[L., the ankle, the ankle bone.]</ety> <sn>1.</sn> <fld>(Anat.)</fld> <def>The astragalus.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Surg.)</fld> <def>A variety of clubfoot (<spn>Talipes calcaneus</spn>). See the Note under <er>Talipes</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Talus</ent><br/
<hw>Ta"lus</hw>, <pos>n.</pos> <ety>[F.]</ety> <sn>1.</sn> <fld>(Fort.)</fld> <def>A slope; the inclination of the face of a work.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Geol.)</fld> <def>A sloping heap of fragments of rock lying at the foot of a precipice.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tamability</ent><br/
<hw>Tam`a*bil"i*ty</hw> <pr>(?)</pr>, <pos>n.</pos> <def>The quality or state of being tamable; tamableness.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tamable</ent><br/
<hw>Tam"a*ble</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Capable of being tamed, subdued, or reclaimed from wildness or savage ferociousness.</def> -- <wordforms><wf>Tam"a*ble*ness</wf>, <pos>n.</pos></wordforms><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tamale</ent><br/
<hw>Ta*ma"le</hw> <pr>(?)</pr>, <pos>n.</pos> <altsp>[Written also <asp>tamal</asp>, <asp>tomale</asp>.]</altsp> <ety>[Amer. Sp. <ets>tamal</ets>, of Mex. origin.]</ety> <def>A Mexican dish made of crushed corn (cornmeal) mixed with minced meat, seasoned with red pepper, dipped in oil, and steamed.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Tamandu</ent><br/
<hw>Ta*man"du</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Sp., from the native name: cf. F. <ets>tamandua</ets>.]</ety> <fld>(Zool.)</fld> <def>A small ant-eater (<spn>Tamandua tetradactyla</spn>) native of the tropical parts of South America.</def><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ It has five toes on the fore feet, an elongated snout, small ears, and short woolly hair. Its tail is stout and hairy at the base, tapering, and covered with minute scales, and is somewhat prehensile at the end. Called also <altname>tamandua</altname>, <altname>little ant-bear</altname>, <altname>fourmilier</altname>, and <altname>cagouare</altname>.
    The collared, or striped, tamandu (<stype><spn>Tamandua bivittata</spn></stype>) is considered a distinct species by some writers, but by others is regarded as only a variety.</note><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tamanoir</ent><br/
<hw>Ta`ma*noir"</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Zool.)</fld> <def>The ant-bear.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tamarack</ent><br/
<hw>Tam"a*rack</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Bot.)</fld> <sd>(a)</sd> <def>The American larch; also, the larch of Oregon and British Columbia (<spn>Larix occidentalis</spn>). See <er>Hackmatack</er>, and <er>Larch</er>.</def> <sd>(b)</sd> <def>The black pine (<spn>Pinus Murrayana</spn>) of Alaska, California, etc. It is a small tree with fine-grained wood.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tamaric</ent><br/
<hw>Tam"a*ric</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>tamarice</ets>. See <er>Tamarisk</er>.]</ety> <def>A shrub or tree supposed to be the tamarisk, or perhaps some kind of heath.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>He shall be like <qex>tamaric</qex> in the desert, and he shall not see when good shall come.</q> <rj><qau>Jer. xvii. 6 (Douay version).</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tamarin</ent><br/
<hw>Tam"a*rin</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[From the native name in Cayenne.]</ety> <fld>(Zool.)</fld> <def>Any one of several species of small squirrel-like South American monkeys of the genus <gen>Midas</gen>, especially <spn>Midas ursulus</spn>.</def><br/
[<source>1913 Webster</source>]</p>

<p><-- p. 1472 --><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tamarind</ent><br/
<hw>Tam"a*rind</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[It. <ets>tamarindo</ets>, or Sp. <ets>tamarindo</ets>, or Pg. <ets>tamarindo</ets>, <ets>tamarinho</ets>, from Ar. <ets>tamarhind<imac/</ets>, literally, Indian date; <ets>tamar</ets> a dried date + <ets>Hind</ets> India: cf. F. <ets>tamarin</ets>.  Cf. <er>Hindu</er>.]</ety> <fld>(Bot.)</fld> <sn>1.</sn> <def>A leguminous tree (<spn>Tamarindus Indica</spn>) cultivated both the Indies, and the other tropical countries, for the sake of its shade, and for its fruit. The trunk of the tree is lofty and large, with wide-spreading branches; the flowers are in racemes at the ends of the branches. The leaves are small and finely pinnated.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>One of the preserved seed pods of the tamarind, which contain an acid pulp, and are used medicinally and for preparing a pleasant drink.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Tamarind fish</b></col>, <cd>a preparation of a variety of East Indian fish with the acid pulp of the tamarind fruit.</cd> -- <col><b>Velvet tamarind</b></col>. <sd>(a)</sd> <cd>A West African leguminous tree (<spn>Codarium acutifolium</spn>).</cd> <sd>(b)</sd> <cd>One of the small black velvety pods, which are used for food in Sierra Leone.</cd> -- <col><b>Wild tamarind</b></col> <fld>(Bot.)</fld>, <cd>a name given to certain trees somewhat resembling the tamarind, as the <spn>Lysiloma latisiliqua</spn> of Southern Florida, and the <spn>Pithecolobium filicifolium</spn> of the West Indies.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tamarisk</ent><br/
<hw>Tam"a*risk</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>tamariscus</ets>, also <ets>tamarix</ets>, <ets>tamarice</ets>, Skr. <ets>tam<amac/la</ets>, <ets>tam<amac/laka</ets>, a tree with a very dark bark; cf. <ets>tamas</ets> darkness: cf. F. <ets>tamarisc</ets>, <ets>tamarix</ets>, <ets>tamaris</ets>.]</ety> <fld>(Bot.)</fld> <def>Any shrub or tree of the genus <gen>Tamarix</gen>, the species of which are European and Asiatic.  They have minute scalelike leaves, and small flowers in spikes.  An Arabian species (<spn>Tamarix mannifera</spn>) is the source of one kind of manna.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Tamarisk salt tree</b></col>, <cd>an East Indian tree (<spn>Tamarix orientalis</spn>) which produces an incrustation of salt.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tambac</ent><br/
<hw>Tam"bac</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Metal.)</fld> <def>See <er>Tombac</er>.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tambour</ent><br/
<hw>Tam"bour</hw> <pr>(?)</pr>, <pos>n.</pos> <sn>1.</sn> <fld>(Mus.)</fld> <def>A kind of small flat drum; a tambourine.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A small frame, commonly circular, and somewhat resembling a tambourine, used for stretching, and firmly holding, a portion of cloth that is to be embroidered; also, the embroidery done upon such a frame; -- called also, in the latter sense, <altname>tambour work</altname>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <fld>(Arch.)</fld> <def>Same as <er>Drum</er>, <pos>n.</pos>, 2<sd>(d)</sd>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <fld>(Fort.)</fld> <def>A work usually in the form of a redan, to inclose a space before a door or staircase, or at the gorge of a larger work. It is arranged like a stockade.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <fld>(Physiol.)</fld> <def>A shallow metallic cup or drum, with a thin elastic membrane supporting a writing lever. Two or more of these are connected by an India rubber tube, and used to transmit and register the movements of the pulse or of any pulsating artery.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tambour</ent><br/
<hw>Tam"bour</hw>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Tamboured</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Tambouring</conjf>.]</vmorph> <def>To embroider on a tambour.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tamboura</ent><br/
<hw>Tam*bour"a</hw> <pr>(?)</pr>, <pos>n.</pos> <sn>1.</sn> <fld>(Mus.)</fld> <def>A stringed musical instrument resembling a lute but lacking frets, with a small round body and a long neck, used to produce an accompaniment for singing; -- called also <altname>tambur</altname>, <altname>tambour</altname>, and <altname>tampur</altname>.</def> <altsp>[Also spelled <asp>tambura</asp>.]</altsp><br/
[<source>PJC</source>]</p>

<p><ent>Tambourin</ent><br/
<hw>Tam`bou`rin"</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F. See <er>Tambourine</er>.]</ety> <sn>1.</sn> <def>A tambourine.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Mus.)</fld> <def>An old Proven<cced/al dance of a lively character, common on the stage.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tambourine</ent><br/
<hw>Tam`bour*ine"</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F. <ets>tambourin</ets>; cf. It. <ets>tamburino</ets>. See <er>Tambour</er>, and cf. <er>Tamborine</er>.]</ety> <def>A small drum, especially a shallow drum with only one skin, played on with the hand, and having bells at the sides; a timbrel.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tambourine</ent><br/
<hw>Tam`bour*ine"</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A South American wild dove (<spn>Tympanistria tympanistria</spn>), mostly white, with black-tiped wings and tail. Its resonant note is said to be ventriloquous.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Tambreet</ent><br/
<hw>Tam"breet</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Zool.)</fld> <def>The duck mole.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tamburin</ent><br/
<hw>Tam`bu*rin"</hw> <pr>(?)</pr>, <pos>n.</pos> <def>See <er>Tambourine</er>.</def>  <rj><au>Spenser.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tame</ent><br/
<hw>Tame</hw> <pr>(?)</pr>, <pos>v. t.</pos> <ety>[Cf. F. <ets>entamer</ets> to cut into, to broach.]</ety> <def>To broach or enter upon; to taste, as a liquor; to divide; to distribute; to deal out.</def> <mark>[Obs. or Prov. Eng.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>In the time of famine he is the Joseph of the country, and keeps the poor from starving. Then he <qex>tameth</qex> his stacks of corn, which not his covetousness, but providence, hath reserved for time of need.</q> <rj><qau>Fuller.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tame</ent><br/
<hw>Tame</hw>, <pos>a.</pos> <amorph>[<pos>Compar.</pos> <adjf>Tamer</adjf> <pr>(?)</pr>; <pos>superl.</pos> <adjf>Tamest</adjf>.]</amorph> <ety>[AS. <ets>tam</ets>; akin to D. <ets>tam</ets>, G. <ets>zahm</ets>, OHG. <ets>zam</ets>, Dan. & Sw. <ets>tam</ets>, Icel. <ets>tamr</ets>, L. <ets>domare</ets> to tame, Gr. <?/, Skr. <ets>dam</ets> to be tame, to tame, and perhaps to E. <ets>beteem</ets>. <root/61.  Cf. <er>Adamant</er>, <er>Diamond</er>, <er>Dame</er>, <er>Daunt</er>, <er>Indomitable</er>.]</ety> <sn>1.</sn> <def>Reduced from a state of native wildness and shyness; accustomed to man; domesticated; domestic; <as>as, a <ex>tame</ex> deer, a <ex>tame</ex> bird</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Crushed; subdued; depressed; spiritless.</def><br/
[<source>1913 Webster</source>]</p>

<p><q><qex>Tame</qex> slaves of the laborious plow.</q> <rj><qau>Roscommon.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>Deficient in spirit or animation; spiritless; dull; flat; insipid; <as>as, a <ex>tame</ex> poem; <ex>tame</ex> scenery</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><syn><b>Syn.</b> -- Gentle; mild; meek. See <er>Gentle</er>.</syn><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tame</ent><br/
<hw>Tame</hw>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Tamed</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Taming</conjf>.]</vmorph> <ety>[AS. <ets>tamian</ets>, <ets>temian</ets>, akin to D. <ets>tammen</ets>, <ets>temmen</ets>, G. <ets>z<aum/hmen</ets>, OHG. <ets>zemmen</ets>, Icel. <ets>temja</ets>, Goth. ga<ets>tamjan</ets>. See <er>Tame</er>, <pos>a.</pos>]</ety> <sn>1.</sn> <def>To reduce from a wild to a domestic state; to make gentle and familiar; to reclaim; to domesticate; <as>as, to <ex>tame</ex> a wild beast</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>They had not been <qex>tamed</qex> into submission, but baited into savegeness and stubbornness.</q> <rj><qau>Macaulay.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To subdue; to conquer; to repress; <as>as, to <ex>tame</ex> the pride or passions of youth</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tameable</ent><br/
<hw>Tame"a*ble</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Tamable.</def>  <rj><au>Bp. Wilkins.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tameless</ent><br/
<hw>Tame"less</hw>, <pos>a.</pos> <def>Incapable of being tamed; wild; untamed; untamable.</def> <au>Bp. Hall.</au> -- <wordforms><wf>Tame"less*ness</wf>, <pos>n.</pos></wordforms><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tamely</ent><br/
<hw>Tame"ly</hw>, <pos>adv.</pos> <def>In a tame manner.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tameness</ent><br/
<hw>Tame"ness</hw>, <pos>n.</pos> <def>The quality or state of being tame.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tamer</ent><br/
<hw>Tam"er</hw> <pr>(?)</pr>, <pos>n.</pos> <def>One who tames or subdues.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tamerlane</ent><br/
<hw>Ta*mer*lane"</hw> <pr>(t<acr/*m<etil/r*l<amac/n")</pr>, <pos>prop. n.</pos> <def>A Tatar conquerer, also called <altname>Timur</altname> or <altname>Timour</altname> (t<emac/*m<ocir/r") or <altname>Timur Bey</altname>, also <altname>Timur-Leng</altname> or <altname>Timur-i-Leng</altname> ('Timur the Lame'), which was corrupted to <ex>Tamerlane</ex>.  He was born in Central Asia, 1333, a member of the Barslas, a Turkish Mongol tribe which had converted to Islam.  He died 1405.  Though he claimed descent from Jenghiz Khan, it is believed that he was in fact descended from a follower of the Khan.  By 1370, Tamerlane, a renowned warrior, began consolidating his power among the various nomadic tribes of Central Asia by conquering the entire region. He became a ruler about 1370 of a realm whose capital was Samarkand; conquered Persia, Central Asia, and in 1398 a great part of India, including Delhi; waged war with the Turkish <person>Sultan Bajazet I.</person> (Beyazid), whom he defeated at Ankara in 1402 and took prisoner; and died while preparing to invade China.  By the end of his life in 1405, after 35 years of campaigns and wars that left hundreds of thousands dead and enslaved, he had successfully defeated Ottoman Turks, Hindus, The Golden Horde, and other peoples and controlled an empire stretching from the Aegean to the River Ganges and threatened the trembling Kingdoms of Europe and the Eastern Roman Empire.  He is the <asp>Tamerlaine</asp> of the plays.</def><br/
[<source>Century Dict. 1906</source> + <source>PJC</source>]</p>

<p><q>Just at the moment when the Sultan (Bajazet) seemed to have attained the pinnacle of his ambition, when his authority was unquestioningly obeyed over the greater part of the Byzantine Empire in Europe and Asia, when the Christian states were regarding him with terror as the scourge of the world, another and greater scourge came to quell him, and at one stroke all the vast fabric of empire which Bayezid (Beyazid or B<amac/yez<imac/d) had so triumphantly erected was shattered to the ground.  This terrible conquerer was Tim<umac/r the Tatar, or as we call him, <ldquo/Tamerlane<rdquo/.  Tim<umac/r was of Turkish race, and was born near Samarkand in 1333.  He was consequently an old man of 70 when he came to encounter B<amac/yez<imac/d in 1402.  It had taken him many years to establish his authority over a portion of the numerous divisions into which the immense empire of Chingiz Khan had fallen after the death of that stupendous conqueror.  Tim<umac/r was but a petty chief among many others: but at last he won his way and became ruler of Samarkand and the whole province of Transoxiana, or 'Beyond the River' (M<amac/-war<amac/-n-nahr) as the Arabs called the country north of the Oxus.  Once fairly established in this province, Tim<umac/r began to overrun the surrounding lands, and during thirty years his ruthless armies spread over the provinces of Asia, from Dehli to Damascus, and from the Sea of Aral to the Persian Gulf.  The subdivision of the Moslem Empire into numerous petty kingdoms rendered it powerless to meet the overwhelming hordes which Tim<umac/r brought down from Central Asia.  One and all, the kings and princes of Persia and Syria succumbed, and Tim<umac/r carried his banners triumphantly as far as the frontier of Egypt, where the brave Mamluk Sultans still dared to defy him.  He had so far left B<amac/yez<imac/d unmolested; partly because he was too powerful to be rashly provoked, and partly because Tim<umac/r respected the Sultan's valorous deeds against the Christians: for Tim<umac/r, though a wholesale butcher, was very conscientious in matters of religion, and held that B<amac/yez<imac/d's fighting for the Faith rightly covered a multitude of sins.</q> <rj><qau>Poole, Story of Turkey, p. 63</qau></rj><br/
[<source>Century Dict. 1906</source>]</p>

<p><note>Timour (t<imac/*m<oomac/r"), Timur, or TAMERLANE, was the second of the great conquerers whom central Asia sent forth in the middle ages, and was born at Kesh, about 40 miles southeast of Samarkand, April 9, 1336. His father was a Turkish chieftain and his mother claimed descent from the great Genghis-Khan.   When he became tribal chieftain, Timour helped the Amir Hussein to drive out the Kalmucks.  Turkestan was thereupon divided between them, but soon war broke out between the two chiefs, and the death of Hussein in battle made Timour master of all Turkestan.  He now began his career of conquest, overcoming the Getes, Khiva and Khorassin, after storming Herat.  His ever-widening circle of possessions soon embraced Persia, Mesopotamia, Georgia, and the Mongol state, Kiptchak.  He threatened Moscow, burned Azoo, captured Delhi, overran Syria, and stormed Bagdad, which had revolted.  At last, July 20, 1402, Timour met the Sultan Bajazet of the Ottoman Turks, on the plains of Ankara, captured him and routed his army, thus becoming master of the Turkish empire.  He took but a short rest at his capital, Samarkand, and in his eagerness to conquer China, led his army of 200,000 across the Jaxartes on the ice, and pushed rapidly on for 300 miles, when his death, Feb. 18, 1405, saved the independence of China.  Though notorious for his acts of cruelty -- he may have slaughtered 80,000 in Delhi -- he was a patron of the arts. In his reign of 35 years, this chief of a small tribe, dependent on the Kalmucks, became the ruler of the vast territory stretching from Moscow to the Ganges.  A number of writings said to have been written by Timour have been preserved in Persian, one of which, the <booki>Institutions</booki>, has been translated into English.</note>  <au>The Student's Cyclopedia, 1897.</au><br/
[<source>PJC</source>]</p>

<p><note>  There is a story about an incident when an archaeologist opened Timur's tomb at the Gur-Amir mausoleum in Samarkand, which was erected in 1404.  Timur and several of his descendants, including Ulugh Beg, are interred in that magnificent structure in the south-western side of Samarkand.  In the mausoleum, mosaics made out of light- and dark-blue glazed bricks decorate the walls and the drum, and the tiled geometrical designs of the cupola shine brightly in the sun.  Restoration work was started in 1967; the exterior cupola and glazed decorations were restored before that, in the 1950s.  The mausoleum holds tombstones made of marble and onyx, the tombstone of Timur is carved from a slab of nephrite. The burials proper are placed in a crypt under the mausoleum.<br/
   In 1941, a distinguished Soviet scientist, M. Gerasimov, received permission to exhume Tamerlane's body.  On June 22, 1941, working in the Samarkand crypt, he opened the sarcophagus to study the body and found the inscription: "Whoever opens this will be defeated by an enemy more fearsome than I." Hours later, Hitler invaded Russia. Five weeks after the great Emir was reinterred in 1942, the Germans surrendered at Stalingrad.<br/
   Examination of the remains in Timur's tomb confirmed that the body was tall, as was reported in the histories, and had been wounded in the leg and arm.<br/
    The actual inscription on the tomb has been reported variously:<br/
   "He whomsoever shall disturb the earthly resting place of Timur-i-Lenk (Tamerlane), then his country shall suffer such terrible retribution as the Hand of Allah shall visit upon it."<br/
   "When I rise, the World will Tremble".</note><br/
[<source>PJC</source>]</p>

<p><q><title>Timur's Legacy: The Architecture of Samarkand</title><br/
<br/
Let he who doubt Our power and munificence look upon Our buildings<br/
                      Amir Timur, 1379 AD<br/
<br/
Timur, better known in the West as Tamerlane from his nickname Timur-i-leng or "Timur the Lame", was the last of the great nomadic warriors to sweep out of Central Asia and shake the world. As befits a man styled "World Conqueror", we know a lot about him -- and not all of it good. In 1336, at Shakhrisabz in present-day Uzbekistan, the wife of a minor chief of the Mongol Barlas clan gave birth to a son with blood-filled palms, a sure omen that the infant was predestined to cause the death of many. He was given an appropriate name -- Timur means "iron" in Turkish -- and raised in the Turkic-Islamic tradition of the surrounding steppe as a rider, archer and swordsman.<br/
<br/
Even by the harsh standards of the Mongol hordes, Timur excelled. Before he was twenty years old he had attracted a band of followers with whom he ranged across the steppe raiding caravans and rustling horses. In 1360 his skills as a commander were rewarded when he was recognised as chief of the Barlas clan. Over the next ten years he steadily extended his influence over Transoxiana -- the region between the Oxus and Jaxartes Rivers centred on present-day Uzbekistan -- acquiring wounds to his right arm and leg in the process, and hence his nickname. In 1370 he conquered Turkistan, the last surviving Mongol Khanate, and declared himself Amir or "Commander". He made the Silk Road city of Samarkand his capital, and then embarked on a series of military conquests that rocked Asia and Europe to their very foundations.<br/
<br/
For 35 years Timur's forces ranged far and wide, repeatedly sweeping across Central Asia, Iran, Turkey and northern India. In 1405 Timur was preparing his greatest expedition ever, aimed at conquering China, when he was struck down by fever. Despite the best efforts of his doctors, to the sound of massive thunderclaps and "foaming like a camel dragged backwards by the rein", Timur finally succumbed. The Ming Emperor must have breathed a heartfelt sigh of relief when he eventually heard the news.<br/
<br/
Historians estimate that Timur, who personally led his forces as far afield as Moscow and Delhi, may have been responsible for the death of as many as 15 million people. Yet he made little attempt to consolidate his conquests, preferring to mount regular, devastating attacks against his neighbours before returning to his native Transoxiana. As a consequence, the dynasty he established proved to be short-lived, though in 1526 Timur's great, great, great grandson Babur restored the family fortunes by conquering Delhi and founding the resplendent Mogul Empire.<br/
<br/
Timur must have been an enigma to his contemporaries. Brutal and utterly ruthless, he was nevertheless a man of culture. He is said to have been illiterate, but fluent in Turkish and Persian. Sources speak of his sharp wit and hunger for knowledge. When not out and about slaughtering his neighbours, he indulged in passionate debate with scholars of history, medicine and astronomy. He enjoyed playing chess. Above all, he seems to have loved his capital, Samarkand, and he spent much time between campaigns embellishing this previously undistinguished city. To help in this great enterprise, he plundered cities like Damascus, Baghdad, Isfahan and Delhi not just for the loot, but for their skilled artisans, who were brought back to make Samarkand a city worthy of the "World Conqueror". As a consequence the warlike Timur's most lasting and unlikely legacy remains the unsurpassed architectural jewel of Central Asia.<br/
<br/
With Timur's death Transoxiana began a long period of decline, culminating in gradual Russian conquest during the 19th century. Samarkand had long been inaccessible to outsiders because of the xenophobia and religious bigotry of the ruling amirs. This situation was compounded in 1920, when the Red Army seized control of the region and began a process of Sovietisation. In 1924 Samarkand was included within the frontiers of the new Uzbek Soviet Socialist Republic, and a curtain of silence fell across the region with Westerners, in particular, being rigorously excluded.<br/
<br/
Only in the 1980s did the veil begin to rise, and then within a few short years the former USSR disintegrated, resulting in the birth of independent Uzbekistan in 1991. Although ruled by a suspicious and innately cautious former Soviet aparatchik, Islam Karimov, Uzbekistan is today slowly opening to foreign tourism. It should do well. The cities of Bukhara and Khiva, together with Timur's capital at Samarkand, are truly magnificent. In places, it's as though time stood still. It didn't of course. The Soviets worked long and hard to restore what remained of Timurid Samarkand, and Uzbekistan stands to benefit greatly as a result. Moreover, the process continues apace, both in spiritual terms -- Timur is now an Uzbek national hero -- and at a more mundane level. Everywhere the chip of stonemasons' hammers is to be heard, and a whole new generation of skilled craftsmen is being trained to restore the architectural legacy of the "Iron Limper".<br/
<br/
The historic heart of Samarkand is the Registan, an open square dominated by three great madrassa , or Islamic colleges. <person>George Curzon</person>, later to become Viceroy of India, visited in 1899 and was moved enough to describe the Registan as "the noblest public square in the world". He continues: "No European spectacle can be adequately compared to it, in our inability to point to an open space in any western city that is commanded on three of its four sides by Gothic cathedrals of the finest order". The architecture is distinctively Timurid, being characterised by an extraordinarily lavish use of colour, especially emerald, azure, deep blue and gold. The great domes are fluted, the vast porticoes richly decorated with corkscrew columns and intricately-patterned glazed tiles. Astonishingly, the façade of the Shir Dor Madrassa on the east side of the square is decorated with half-tiger, half-lion creatures stalking deer, whilst a blazing sun with a human face rises behind the beast of prey's back. In Islam, such representational art is generally forbidden, and it is wonderful that these clearly heretical images have survived through the long centuries since they were created.<br/
<br/
Samarkand -- let alone Uzbekistan -- has too many Timurid gems to describe in one short article, but after the Registan, the monumental Bibi Khanum Mosque is perhaps the most extraordinary sight in the city. Built for Timur's chief wife, <person>Saray Mulk Khanum</person>, this magnificent building was financed by the plunder brought back from Delhi in 1398; it is said that 95 elephants were used in hauling marble for the mosque. On Bibi Khanum's completion a chronicler was moved to write: "Its dome would have been unique had it not been for the heavens, and unique would have been its portal had it not been for the Milky Way".  Even so, historians have shown that in his plans for the Bibi Khanum, Timur's vision exceeded the architectural possibilities of the time.  Quite simply, the lofty iwan (portico) and the towering minarets were too ambitious for the technology of the time -- especially in a land prone to violent earthquakes. By all accounts, parts of the giant mosque began to collapse within months of its consecration.  Today all three massive azure domes have been restored, and work still continues, though this time with ferro-concrete supports hidden behind the elaborate glazed tilework, on the lofty iwan and minarets. When the restoration is complete in around 2002, Uzbekistan will have yet another architectural marvel to draw visitors.<br/
<br/
Finally and fittingly we turn to the Gur-i Amir, or "Tomb of the Ruler", Timur's own last resting place. This fabulous structure, which was completed in 1404, is dominated by the octagonal mausoleum and its peerless fluted dome, azure in colour, with 64 separate ribs. Within lie the remains not only of Timur, but also of various members of his family, including his grandson the scholar-king Ulugh Beg. Timur's tomb is protected by a single slab of jade, said to be the largest in the world. Brought back by Ulugh Beg from Mongolia in 1425, it was broken in half in the 18th century by the Persian ruler Nadir Shah, who tried to remove it from the chamber. Carved into the jade is an inscription in Arabic: "When I rise, the World will Tremble".<br/
<br/
Coincidence, no doubt, but on the night of June 22, 1941, the Russian Scientist M. Gerasimov began his exhumation of Timur's remains. Within hours Hitler's armies crashed across the Soviet frontier signalling the beginning of the Nazi invasion. Gerasimov's investigations showed that Timur had been a tall man for his race and time, lame, as recorded, in his right leg, and with a wound to his right arm.  Surprisingly, red hair still clung to the skull from which Gerasimov reconstructed a bronze bust. Eventually Timur's remains were reinterred with full Muslim burial rites, giving truth to the message thundered in Arabic script three metres high from the cylindrical drum of the great conqueror's mausoleum: "Only God is Immortal".</q><br/
<rj><qau>Andrew Forbes/CPA</qau> (Text copyright   2001.)<br/
(from http://www.cpamedia.com/articles/20010215/)</rj><br/
[<source>PJC</source>]</p>

<p><ent>Tamias</ent><br/
||<hw>Ta"mi*as</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[NL., fr. Gr. <?/ a distributer.]</ety> <fld>(Zool.)</fld> <def>A genus of ground squirrels, including the chipmunk.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tamil</ent><br/
<hw>Ta"mil</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Of or pertaining to the Tamils, or to their language.</def> <altsp>[Written also <asp>Tamul</asp>.]</altsp><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tamil</ent><br/
<hw>Ta"mil</hw>, <pos>n.</pos> <ety>[Written also <ets>Tamul</ets>.]</ety> <sn>1.</sn> <fld>(Ethnol.)</fld> <def>One of a Dravidian race of men native of Northern Ceylon and Southern India.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>The Tamil language, the most important of the Dravidian languages. See <er>Dravidian</er>, <pos>a.</pos></def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tamilian</ent><br/
<hw>Ta*mil"i*an</hw> <pr>(?)</pr>, <pos>a. & n.</pos> <def>Tamil.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Taminy</ent><br/
<ent>Tamine</ent><br/
<mhw>{ <hw>Tam"ine</hw> <pr>(?)</pr>, <hw>Tam"i*ny</hw> <pr>(?)</pr>, }</mhw> <pos>n.</pos> <ety>[Cf. F. <ets>tamis</ets> a sort of sieve.  Cf. <er>Stamin</er>, <er>Temse</er>.]</ety> <def>A kind of woolen cloth; tammy.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tamis</ent><br/
<hw>Tam"is</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F., a kind of sieve.]</ety> <sn>1.</sn> <def>A sieve, or strainer, made of a kind of woolen cloth.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>The cloth itself; tammy.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Tamis bird</b></col> <fld>(Zool.)</fld>, <cd>a Guinea fowl.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tamkin</ent><br/
<hw>Tam"kin</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A tampion.</def>  <rj><au>Johnson (Dict.).</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tammy</ent><br/
<hw>Tam"my</hw> <pr>(?)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Tammies</plw> <pr>(<?/)</pr>.</plu> <sn>1.</sn> <def>A kind of woolen, or woolen and cotton, cloth, often highly glazed, -- used for curtains, sieves, strainers, etc.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A sieve, or strainer, made of this material; a tamis.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tam-o'-shanter</ent><br/
<hw>Tam`-o'-shan"ter</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[So named after <etsep>Tam o'Shanter</etsep>, a character in Burns's poem of the same name.]</ety> <def>A kind of Scotch cap of wool, worsted, or the like, having a round, flattish top much wider than the band which fits the head, and usually having a tassel in the center.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Tamoxifen</ent><br/
<hw>Ta*mox"i*fen</hw> <pr>(t<acr/*m<ocr/k"s<icr/*f<ecr/n)</pr>, <pos>n.</pos> <def>a chemical compound (<chform>C26H29NO</chform>) which is non-steroidal but physiogically active as an estrogen antagonist.  It is used to treat postmenopausal breast cancer.  Chemically it is <chname>1-p-dimethylaminoethoxyphenyl-trans-1,2-diphenyl-but-1-ene</chname>. It can be obtained as a white crystalline powder.</def> <au>[MI11]</au><br/
<syn><b>Syn. --</b> Kessar; Noltam; Nolvadex; Tamofen; Tamoxasta; Terimon; Xynoplex.</syn>
[<source>PJC</source>]</p>

<p><ent>Tamp</ent><br/
<hw>Tamp</hw> <pr>(?)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Tamped</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Tamping</conjf>.]</vmorph> <ety>[Cf. F. <ets>tamponner</ets> to plug or stop. See <er>Tampion</er>.]</ety> <sn>1.</sn> <def>In blasting, to plug up with clay, earth, dry sand, sod, or other material, as a hole bored in a rock, in order to prevent the force of the explosion from being misdirected.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To drive in or down by frequent gentle strokes; <as>as, to <ex>tamp</ex> earth so as to make a smooth place</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tampan</ent><br/
<hw>Tam"pan</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Zool.)</fld> <def>A venomous South African tick.</def>  <rj><au>Livingstone.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tampeon</ent><br/
<hw>Tam"pe*on</hw> <pr>(?)</pr>, <pos>n.</pos> <def>See <er>Tampion</er>.</def>  <rj><au>Farrow.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tamper</ent><br/
<hw>Tamp"er</hw> <pr>(?)</pr>, <pos>n.</pos> <sn>1.</sn> <def>One who tamps; specifically, one who prepares for blasting, by filling the hole in which the charge is placed.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>An instrument used in tamping; a tamping iron.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tamper</ent><br/
<hw>Tam"per</hw> <pr>(?)</pr>, <pos>v. i.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Tampered</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Tampering</conjf>.]</vmorph> <ety>[A corruption of <ets>temper</ets>.]</ety> <sn>1.</sn> <def>To meddle; to be busy; to try little experiments; <as>as, to <ex>tamper</ex> with a disease</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>'T is dangerous <qex>tampering</qex> with a muse.</q> <rj><qau>Roscommon.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To meddle so as to alter, injure, or vitiate a thing.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>To deal unfairly; to practice secretly; to use bribery.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Others <qex>tampered</qex><br/
For Fleetwood, Desborough, and Lambert.</q> <rj><qau>Hudibras.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tamperer</ent><br/
<hw>Tam"per*er</hw> <pr>(?)</pr>, <pos>n.</pos> <def>One who tampers; one who deals unfairly.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tampico fibre</ent><br/
<ent>Tampico fiber</ent><br/
<mhw>{ <hw>Tam*pi"co fi"ber</hw> <it>or</it> <hw>Tam*pi"co fi"bre</hw> <pr>(?)</pr> }</mhw>. <def>A tough vegetable fiber used as a substitute for bristles in making brushes. The piassava and the ixtle are both used under this name.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tamping</ent><br/
<hw>Tamp"ing</hw> <pr>(?)</pr>, <pos>n.</pos> <sn>1.</sn> <def>The act of one who tamps; specifically, the act of filling up a hole in a rock, or the branch of a mine, for the purpose of blasting the rock or exploding the mine.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>The material used in tamping. See <er>Tamp</er>, <pos>v. t.</pos>, 1.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Tamping iron</b></col>, <cd>an iron rod for beating down the earthy substance in tamping for blasting.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tampion</ent><br/
<hw>Tam"pi*on</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F. <ets>tampon</ets>, <ets>tapon</ets>, <ets>tape</ets>, of Dutch or German origin. See <er>Tap</er> a pipe or plug, and cf. <er>Tamp</er>, <er>Tampop</er>, <er>Tompion</er>.]</ety> <altsp>[Written also <asp>tampeon</asp>, and <asp>tompion</asp>.]</altsp> <sn>1.</sn> <def>A wooden stopper, or plug, as for a cannon or other piece of ordnance, when not in use.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Mus.)</fld> <def>A plug for upper end of an organ pipe.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tampoe</ent><br/
<hw>Tam"poe</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Bot.)</fld> <def>The edible fruit of an East Indian tree (<spn>Baccaurea Malayana</spn>) of the Spurge family. It somewhat resembles an apple.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tampon</ent><br/
<hw>Tam"pon</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F. See <er>Tampion</er>.]</ety> <fld>(Surg.)</fld> <def>A plug introduced into a natural or artificial cavity of the body in order to arrest hemorrhage, absorb secretions (as from menstruation), or for the application of medicine.</def><br/
[<source>1913 Webster</source> <source>+PJC</source>]</p>

<p><ent>Tampon</ent><br/
<hw>Tam"pon</hw>, <pos>v. t.</pos> <fld>(Surg.)</fld> <def>To plug with a tampon.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tampoon</ent><br/
<hw>Tam"poon</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[See <er>Tampion</er>.]</ety> <def>The stopper of a barrel; a bung.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tam-tam</ent><br/
<hw>Tam"-tam`</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Hind.; of imitative origin.]</ety> <fld>(Mus.)</fld> <sd>(a)</sd> <def>A kind of drum used in the East Indies and other Oriental countries; -- called also <altname>tom-tom</altname>.</def> <sd>(b)</sd> <def>A gong. See <er>Gong</er>, <pos>n.</pos>, 1.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tamul</ent><br/
<hw>Ta"mul</hw> <pr>(?)</pr>, <pos>a. & n.</pos> <def>Tamil.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tamworth</ent><br/
<hw>Tam"worth</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[From <etsep>Tamworth</etsep>, Staffordshire, England.]</ety> <def>One of a long-established English breed of large pigs. They are red, often spotted with black, with a long snout and erect or forwardly pointed ears, and are valued as bacon producers.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Tan</ent><br/
<hw>Tan</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Chin.]</ety> <def>See <er>Picul</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tan</ent><br/
<hw>Tan</hw>, <pos>n.</pos> <ety>[F. <ets>tan</ets>, perhaps fr. Armor. <ets>tann</ets> an oak, oak bar; or of Teutonic origin; cf. G. <ets>tanne</ets> a fir, OHG. <ets>tanna</ets> a fir, oak, MHG. <ets>tan</ets> a forest.  Cf. <er>Tawny</er>.]</ety> <sn>1.</sn> <def>The bark of the oak, and some other trees, bruised and broken by a mill, for tanning hides; -- so called both before and after it has been used. Called also <altname>tan bark</altname>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A yellowish-brown color, like that of tan.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>A brown color imparted to the skin by exposure to the sun; <as>as, hands covered with <ex>tan</ex></as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Tan bed</b></col> <fld>(Hort.)</fld>, <cd>a bed made of tan; a bark bed.</cd> -- <col><b>Tan pickle</b></col>, <cd>the liquor used in tanning leather.</cd> -- <col><b>Tan spud</b></col>, <cd>a spud used in stripping bark for tan from trees.</cd> -- <col><b>Tan stove</b></col>. <cd>See <cref>Bark stove</cref>, under <er>Bark</er>.</cd> -- <col><b>Tan vat</b></col>, <cd>a vat in which hides are steeped in liquor with tan.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tan</ent><br/
<hw>Tan</hw>, <pos>a.</pos> <def>Of the color of tan; yellowish-brown.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Black and tan</b></col>. <cd>See under <er>Black</er>, <pos>a.</pos></cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tan</ent><br/
<hw>Tan</hw>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Tanned</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Tanning</conjf>.]</vmorph> <ety>[F. <ets>tanner</ets>, LL. <ets>tannare</ets>. See <er>Tan</er>, <pos>n.</pos>]</ety> <sn>1.</sn> <def>To convert (the skin of an animal) into leather, as by usual process of steeping it in an infusion of oak or some other bark, whereby it is impregnated with tannin, or tannic acid (which exists in several species of bark), and is thus rendered firm, durable, and in some degree impervious to water.</def><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ The essential result in tanning is due to the fact that the tannins form, with gelatins and albuminoids, a series of insoluble compounds which constitute leather. Similar results may be produced by the use of other reagents in place of tannin, as alum, and some acids or chlorides, which are employed in certain processes of tanning.</note><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To make brown; to imbrown, as by exposure to the rays of the sun; <as>as, to <ex>tan</ex> the skin</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>To thrash or beat; to flog; to switch; <as>as, to <ex>tan</ex> a disobedient child's hide</as>.</def> <mark>[Colloq.]</mark><br/
[<source>Webster 1913 Suppl.</source> <source>+PJC</source>]</p>

<p><ent>Tan</ent><br/
<hw>Tan</hw> <pr>(?)</pr>, <pos>v. i.</pos> <def>To get or become tanned.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tana</ent><br/
<hw>Ta"na</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Zool.)</fld> <def>Same as <er>Banxring</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tanager</ent><br/
<hw>Tan"a*ger</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[NL. <ets>tanagra</ets>, probably fr. Brazilian <ets>tangara</ets>.]</ety> <fld>(Zool.)</fld> <def>Any one of numerous species of bright-colored singing birds belonging to <gen>Tanagra</gen>, <gen>Piranga</gen>, and allied genera. The scarlet tanager (<spn>Piranga erythromelas</spn>) and the summer redbird (<spn>Piranga rubra</spn>) are common species of the United States.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tanagrine</ent><br/
<hw>Tan"a*grine</hw> <pr>(?)</pr>, <pos>a.</pos> <fld>(Zool.)</fld> <def>Of or pertaining to the tanagers.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tanagroid</ent><br/
<hw>Tan"a*groid</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[<ets>Tanager</ets> + <ets>-oid</ets>.]</ety> <fld>(Zool.)</fld> <def>Tanagrine.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tanakh</ent><br/
<ent>Tanak</ent><br/
<mhw><hw>Tanak</hw>, <hw>Tanakh</hw></mhw> <pr>(?)</pr>, <pos>n.</pos> <ety>[From the initial (Hebrew) letters of <ets>T</ets>orah, <ets>N</ets>evi'im, and  <ets>K</ets>ethubim.]</ety> <def>a term used among Jews for the Hebrew Bible; the Old Testament.</def> <altsp>[Also spelled <asp>Tanach</asp>.]</altsp> <note>Although Christians use the term <ldquo/Old Testament<rdquo/, this term implies the superseding force of the <ldquo/New Testament<rdquo/, not recognized as revelation by the Jewish faith.</note><br/
[<source>PJC</source>]</p>

<p><q>The Hebrew Bible is divided into three parts: (1) The <er>Torah</er>, <ldquo/Law,<rdquo/ or Pentateuch. (2) The Prophets . . . (3) The Kethubim, or the <ldquo/Writings,<rdquo/ generally termed <er>Hagiographa</er>.</q>  <rj><qau>C. H. H. Wright.</qau></rj><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Tanate</ent><br/
||<hw>Ta*na"te</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Zool.)</fld> <def>An Asiatic wild dog (<spn>Canis procyonoides</spn>), native of Japan and adjacent countries. It has a short, bushy tail. Called also <altname>raccoon dog</altname>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tandem</ent><br/
<hw>Tan"dem</hw> <pr>(?)</pr>, <pos>adv. & a.</pos> <ety>[L. <ets>tandem</ets> at length (of time only), punningly taken as meaning, lengthwise.]</ety> <def>One after another; -- said especially of horses harnessed and driven one before another, instead of abreast.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tandem</ent><br/
<hw>Tan"dem</hw>, <pos>n.</pos> <sn>1.</sn> <def>A team of horses harnessed one before the other.</def> <ldquo/He drove <xex>tandems</xex>.<rdquo/  <rj><au>Thackeray.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A tandem bicycle or other vehicle.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><cs><mcol><col><b>Tandem bicycle</b></col> <it>or</it>  <col><b>Tandem tricycle</b></col></mcol>, <cd>one for two persons in which one rider sits before the other.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tandem cart</ent><br/
<hw>Tan"dem cart</hw>. <def>A kind of two-wheeled vehicle with seats back to back, the front one somewhat elevated.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Tandem engine</ent><br/
<hw>Tandem engine</hw>. <def>A steam engine having two or more steam cylinders in line, with a common piston rod.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Tandem system</ent><br/
<hw>Tandem system</hw>. <fld>(Elec.)</fld> <def>same as <er>Cascade system</er>.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Tang</ent><br/
<hw>Tang</hw> <pr>(t<acr/ng)</pr>, <pos>n.</pos> <ety>[Of Scand. origin; cf. Dan. <ets>tang</ets> seaweed, Sw. <ets>t<aring/ng</ets>, Icel. <ets><thorn/ang</ets>.  Cf. <er>Tangle</er>.]</ety> <fld>(Bot.)</fld> <def>A coarse blackish seaweed (<spn>Fuscus nodosus</spn>).</def>  <rj><au>Dr. Prior.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Tang sparrow</b></col> <fld>(Zool.)</fld>, <cd>the rock pipit.</cd> <mark>[Prov. Eng.]</mark></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tang</ent><br/
<hw>Tang</hw>, <pos>n.</pos> <ety>[Probably fr. OD. <ets>tanger</ets> sharp, tart, literally, pinching; akin to E. <ets>tongs</ets>. <root/59. See <er>Tong</er>.]</ety> <sn>1.</sn> <def>A strong or offensive taste; especially, a taste of something extraneous to the thing itself; <as>as, wine or cider has a <ex>tang</ex> of the cask</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Fig.: A sharp, specific flavor or tinge.  Cf. <er>Tang</er> a twang.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Such proceedings had a strong <qex>tang</qex> of tyranny.</q> <rj><qau>Fuller.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>A cant of philosophism, and a <qex>tang</qex> of party politics.</q> <rj><qau>Jeffrey.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <ety>[Probably of Scand. origin; cf. Icel. <ets>tangi</ets> a projecting point; akin to E. <ets>tongs</ets>. See <er>Tongs</er>.]</ety> <def>A projecting part of an object by means of which it is secured to a handle, or to some other part; anything resembling a tongue in form or position.</def> Specifically: --<br/
[<source>1913 Webster</source>]</p>

<p><sd>(a)</sd> <def>The part of a knife, fork, file, or other small instrument, which is inserted into the handle.</def><br/
[<source>1913 Webster</source>]</p>

<p><sd>(b)</sd> <def>The projecting part of the breech of a musket barrel, by which the barrel is secured to the stock.</def><br/
[<source>1913 Webster</source>]</p>

<p><sd>(c)</sd> <def>The part of a sword blade to which the handle is fastened.</def><br/
[<source>1913 Webster</source>]</p>

<p><sd>(d)</sd> <def>The tongue of a buckle.</def> <mark>[Prov. Eng.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tang</ent><br/
<hw>Tang</hw>, <pos>n.</pos> <ety>[Of imitative origin.  Cf. <er>Twang</er>. This word has become confused with <ets>tang</ets> tatse, flavor.]</ety> <def>A sharp, twanging sound; an unpleasant tone; a twang.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tang</ent><br/
<hw>Tang</hw> <pr>(t<aum/ng)</pr>, <pos>n.</pos> <ety>[Chin. <ets>T`ang</ets>.]</ety> <def>A dynasty in Chinese history, from <sc>a. d.</sc> 618 to 905, distinguished by the founding of the Imperial Academy (the Hanlin), by the invention of printing, and as marking a golden age of literature.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Tang</ent><br/
<hw>Tang</hw>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Tanged</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Tanging</conjf>.]</vmorph> <def>To cause to ring or sound loudly; to ring.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Let thy tongue <qex>tang</qex> arguments of state.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>To tang bees</b></col>, <cd>to cause a swarm of bees to settle, by beating metal to make a din.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tang</ent><br/
<hw>Tang</hw>, <pos>v. i.</pos> <def>To make a ringing sound; to ring.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Let thy tongue <qex>tang</qex> arguments of state.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tangalung</ent><br/
<hw>Tan"ga*lung</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Zool.)</fld> <def>An East Indian civet (<spn>Viverra tangalunga</spn>).</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tangelo</ent><br/
<hw>Tan"ge*lo</hw> <pr>(t<acr/n"j<esl/*l<omac/)</pr>, <pos>n.</pos>  <ety>[<ets>Tang</ets>erine + pom<ets>elo</ets>.]</ety> <def>A hybrid between the tangerine orange and the grapefruit, or pomelo; also, the fruit.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Tangence</ent><br/
<hw>Tan"gence</hw> <pr>(?)</pr>, <pos>n.</pos> <def>Tangency.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tangency</ent><br/
<hw>Tan"gen*cy</hw> <pr>(?)</pr>, <pos>n.</pos> <def>The quality or state of being tangent; a contact or touching.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tangent</ent><br/
<hw>Tan"gent</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>tangens</ets>, <ets>-entis</ets>, p. pr. of <ets>tangere</ets> to touch; akin to Gr. <?/ having seized: cf. F. <ets>tangente</ets>.  Cf. <er>Attain</er>, <er>Contaminate</er>, <er>Contingent</er>, <er>Entire</er>, <er>Tact</er>, <er>Taste</er>, <er>Tax</er>, <pos>v. t.</pos>]</ety> <fld>(Geom.)</fld> <def>A tangent line curve, or surface; specifically, that portion of the straight line tangent to a curve that is between the point of tangency and a given line, the given line being, for example, the axis of abscissas, or a radius of a circle produced. See <cref>Trigonometrical function</cref>, under <er>Function</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><mcol><col><b>Artificial tangent</b></col>, <it>or</it> <col><b>Logarithmic tangent</b></col></mcol>, <cd>the logarithm of the natural tangent of an arc.</cd> -- <col><b>Natural tangent</b></col>, <cd>a decimal expressing the length of the tangent of an arc, the radius being reckoned unity.</cd> -- <col><b>Tangent galvanometer</b></col> <fld>(Elec.)</fld>, <cd>a form of galvanometer having a circular coil and a short needle, in which the tangent of the angle of deflection of the needle is proportional to the strength of the current.</cd> -- <col><b>Tangent of an angle</b></col>, <cd>the natural tangent of the arc subtending or measuring the angle.</cd> -- <col><b>Tangent of an arc</b></col>, <cd>a right line, as <it>ta</it>, touching the arc of a circle at one extremity <it>a</it>, and terminated by a line <it>ct</it>, passing from the center through the other extremity <it>o</it>.</cd></cs>
<-- references are to a figure showing the tangent of an arc --><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tangent</ent><br/
<hw>Tan"gent</hw>, <pos>a.</pos> <ety>[L. <ets>tangens</ets>, <ets>-entis</ets>, p. pr.]</ety> <def>Touching; touching at a single point</def>; specifically <fld>(Geom.)</fld> <def>meeting a curve or surface at a point and having at that point the same direction as the curve or surface; -- said of a straight line, curve, or surface; <as>as, a line <ex>tangent</ex> to a curve; a curve <ex>tangent</ex> to a surface; <ex>tangent</ex> surfaces.</as></def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Tangent plane</b></col> <fld>(Geom.)</fld>, <cd>a plane which touches a surface in a point or line.</cd> -- <col><b>Tangent scale</b></col> <fld>(Gun.)</fld>, <cd>a kind of breech sight for a cannon.</cd> -- <col><b>Tangent screw</b></col> <fld>(Mach.)</fld>, <cd>an endless screw; a worm.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><-- p. 1473 --><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tangental</ent><br/
<hw>Tan*gen"tal</hw> <pr>(?)</pr>, <pos>a.</pos> <fld>(Geom.)</fld> <def>Tangential.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tangential</ent><br/
<hw>Tan*gen"tial</hw> <pr>(?)</pr>, <pos>a.</pos> <fld>(Geom.)</fld> <def>Of or pertaining to a tangent; in the direction of a tangent.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Tangential force</b></col> <fld>(Mech.)</fld>, <cd>a force which acts on a moving body in the direction of a tangent to the path of the body, its effect being to increase or diminish the velocity; -- distinguished from a <xex>normal force</xex>, which acts at right angles to the tangent and changes the direction of the motion without changing the velocity.</cd> -- <col><b>Tangential stress</b></col>. <fld>(Engin.)</fld> <cd>See <er>Shear</er>, <pos>n.</pos>, 3.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tangentially</ent><br/
<hw>Tan*gen"tial*ly</hw>, <pos>adv.</pos> <def>In the direction of a tangent.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tangent spoke</ent><br/
<hw>Tangent spoke</hw>. <def>A tension spoke of a bicycle or similar wheel, secured tangentially to the hub.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Tangent wheel</ent><br/
<hw>Tangent wheel</hw>. <sd>(a)</sd> <def>A worm or worm wheel; a tangent screw.</def> <sd>(b)</sd> <def>A wheel with tangent spokes.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Tangerine</ent><br/
<hw>Tan"ger*ine`</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Etymol. uncertain.]</ety> <fld>(Bot.)</fld> <def>A kind of orange, much like the mandarin, but of deeper color and higher flavor. It is said to have been produced in America from the mandarin.</def> <altsp>[Written also <asp>tangierine</asp>.]</altsp><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tangfish</ent><br/
<hw>Tang"fish`</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Zool.)</fld> <def>The common harbor seal.</def> <mark>[Prov. Eng.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tanghinia</ent><br/
||<hw>Tan*ghin"i*a</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[NL.]</ety> <fld>(Bot.)</fld> <def>The ordeal tree. See under <er>Ordeal</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tangibility</ent><br/
<hw>Tan`gi*bil"i*ty</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Cf. F. <ets>tanggibilit<eacute/</ets>.]</ety> <def>The quality or state of being tangible.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tangible</ent><br/
<hw>Tan"gi*ble</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>tangibilis</ets>, fr. <ets>tangere</ets> to touch: cf. F. <ets>tangible</ets>. See <er>Tangent</er>.]</ety> <sn>1.</sn> <def>Perceptible to the touch; tactile; palpable.</def>  <rj><au>Bacon.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Capable of being possessed or realized; readily apprehensible by the mind; real; substantial; evident.</def> <ldquo/A <xex>tangible</xex> blunder.<rdquo/  <rj><au>Byron.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Direct and <qex>tangible</qex> benefit to ourselves and others.</q> <rj><qau>Southey.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p>-- <wordforms><wf>Tan"gi*ble*ness</wf>, <pos>n.</pos> -- <wf>Tan"gi*bly</wf>, <pos>adv.</pos></wordforms><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tangle</ent><br/
<hw>Tan"gle</hw> <pr>(?)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Tangled</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Tangling</conjf> <pr>(?)</pr>.]</vmorph> <ety>[A frequentative fr. <ets>tang</ets> seaweed; hence, to twist like seaweed. See <er>Tang</er> seaweed, and cf. <er>Tangle</er>, <pos>n.</pos>]</ety> <sn>1.</sn> <def>To unite or knit together confusedly; to interweave or interlock, as threads, so as to make it difficult to unravel the knot; to entangle; to ravel.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To involve; to insnare; to entrap; <as>as, to be <ex>tangled</ex> in lies</as>.</def> <ldquo/<xex>Tangled</xex> in amorous nets.<rdquo/  <rj><au>Milton.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>When my simple weakness strays,<br/
<qex>Tangled</qex> in forbidden ways.</q> <rj><qau>Crashaw.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tangle</ent><br/
<hw>Tan"gle</hw>, <pos>v. i.</pos> <def>To be entangled or united confusedly; to get in a tangle.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tangle</ent><br/
<hw>Tan"gle</hw>, <pos>n.</pos> <sn>1.</sn> <ety>[Cf. Icel. <ets><thorn/<oum/ngull</ets>. See <er>Tang</er> seaweed.]</ety> <fld>(Bot.)</fld> <def>Any large blackish seaweed, especially the <spn>Laminaria saccharina</spn>. See <er>Kelp</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Coral and sea fan and <qex>tangle</qex>, the blooms and the palms of the ocean.</q> <rj><qau>C. Kingsley.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <ety>[From <er>Tangle</er>, <pos>v.</pos>]</ety> <def>A knot of threads, or other thing, united confusedly, or so interwoven as not to be easily disengaged; a snarl; <as>as, hair or yarn in <ex>tangles</ex>; a <ex>tangle</ex> of vines and briers. Used also figuratively.</as></def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <pluf>pl.</pluf> <def>An instrument consisting essentially of an iron bar to which are attached swabs, or bundles of frayed rope, or other similar substances, -- used to capture starfishes, sea urchins, and other similar creatures living at the bottom of the sea.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Blue tangle</b></col>. <fld>(Bot.)</fld>S<cd>ee <er>Dangleberry</er>.</cd> -- <col><b>Tangle picker</b></col> <fld>(Zool.)</fld>, <cd>the turnstone.</cd> <mark>[Prov. Eng.]</mark></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tanglefish</ent><br/
<hw>Tan"gle*fish`</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Zool.)</fld> <def>The sea adder, or great pipefish of Europe.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tanglingly</ent><br/
<hw>Tan"gling*ly</hw> <pr>(?)</pr>, <pos>adv.</pos> <def>In a tangling manner.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tangly</ent><br/
<hw>Tan"gly</hw> <pr>(?)</pr>, <pos>a.</pos> <sn>1.</sn> <def>Entangled; intricate.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Covered with tangle, or seaweed.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Prone, helpless, on the <qex>tangly</qex> beach he lay.</q> <rj><qau>Falconer.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tango</ent><br/
<hw>Tan"go</hw> <pr>(t<acr/<nsm/"g<omac/; <it>Sp.</it> t<aum/<nsm/"g<omac/)</pr>, <pos>n.</pos>; <plu>pl. <plw>Tangos</plw> <pr>(-g<omac/z)</pr></plu>. <ety>[Sp., a certain dance.]</ety> <sd>(a)</sd> <def>A difficult dance in two-four time characterized by graceful posturing, frequent pointing positions, and a great variety of steps, including the cross step and turning steps. The dance is of Spanish origin, and is believed to have been in its original form a part of the fandango.</def> <sd>(b)</sd> <def>Any of various popular forms derived from this.</def> <sd>(c)</sd> <def>a musical tune appropriate for this dance.</def><br/
[<source>Webster 1913 Suppl.</source> <source>+PJC</source>]</p>

<p><ent>Tangram</ent><br/
<hw>Tan"gram</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Cf. <er>Trangram</er>.]</ety> <def>A Chinese toy made by cutting a square of thin wood, or other suitable material, into seven pieces, as shown in the cut, these pieces being capable of combination in various ways, so as to form a great number of different figures. It is now often used in primary schools as a means of instruction.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tangue</ent><br/
<hw>Tangue</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Zool.)</fld> <def>The tenrec.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tangun</ent><br/
<hw>Tan"gun</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Zool.)</fld> <def>A piebald variety of the horse, native of Thibet.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tangwhaup</ent><br/
<hw>Tang"whaup</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Zool.)</fld> <def>The whimbrel.</def> <mark>[Prov. Eng.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tanier</ent><br/
<hw>Tan"i*er</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Bot.)</fld> <def>An aroid plant (<spn>Caladium sagittaefolium</spn>), the leaves of which are boiled and eaten in the West Indies.</def> <altsp>[Written also <asp>tannier</asp>.]</altsp><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tanist</ent><br/
<hw>Tan"ist</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Ir. <ets>tanaiste</ets>, <ets>tanaise</ets>, second, the second person in rank, the presumptive or apparent heir to a prince.]</ety> <def>In Ireland, a lord or proprietor of a tract of land or of a castle, elected by a family, under the system of tanistry.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>This family [the O'Hanlons] were <qex>tanists</qex> of a large territory within the present county of Armagh.</q> <rj><qau>M. A. Lower.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tanistry</ent><br/
<hw>Tan"ist*ry</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[See <er>Tanist</er>.]</ety> <def>In Ireland, a tenure of family lands by which the proprietor had only a life estate, to which he was admitted by election.</def><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ The primitive intention seems to have been that the inheritance should descend to the oldest or most worthy of the blood and name of the deceased. This was, in reality, giving it to the strongest; and the practice often occasioned bloody feuds in families, for which reason it was abolished under James I.</note><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tanite</ent><br/
<hw>Ta"nite</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A firm composition of emery and a certain kind of cement, used for making grinding wheels, slabs, etc.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tank</ent><br/
<hw>Tank</hw> <pr>(t<acr/<nsm/k)</pr>, <pos>n.</pos> <def>A small Indian dry measure, averaging 240 grains in weight; also, a Bombay weight of 72 grains, for pearls.</def>  <rj><au>Simmonds.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tank</ent><br/
<hw>Tank</hw> <pr>(t<acr/<nsm/k)</pr>, <pos>n.</pos> <ety>[Pg. <ets>tanque</ets>, L. <ets>stangum</ets> a pool; or perhaps of East Indian origin.  Cf. <er>Stank</er>, <pos>n.</pos>]</ety> <sn>1.</sn> <def>A large basin or cistern; an artificial receptacle for liquids.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A pond, pool, or small lake, natural or artificial.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><q>We stood in the afterglow on the bank of the <qex>tank</qex> and saw the ducks come home.</q>  <rj><qau>F. Remington.</qau></rj></p>

<p><q>The <qex>tanks</qex> are full and the grass is high.</q>  <rj><qau>Lawson.</qau></rj><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><sn>3.</sn> <fld>(Mil.)</fld> <def>a heavily armored combat vehicle which moves on caterpillar treads, rather than wheels.  It typically carries a cannon and a heavy machine, and sometimes other weapons.  It is the main distinguishing weapon of an <partof>armored division</partof>.</def><br/
[<source>PJC</source>]</p>

<p><sn>4.</sn> <def>a jail cell for temporarily holding prisoners, as in a police station.</def><br/
[<source>PJC</source>]</p>

<p><cs><col><b>Tank engine</b></col>, <cd>a locomotive which carries the water and fuel it requires, thus dispensing with a tender.</cd> -- <col><b>Tank iron</b></col>, <cd>plate iron thinner than boiler plate, and thicker than sheet iron or stovepipe iron.</cd> -- <col><b>Tank worm</b></col> <fld>(Zool.)</fld>, <cd>a small nematoid worm found in the water tanks of India, supposed by some to be the young of the Guinea worm.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tanka</ent><br/
<hw>Tan"ka</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Naut.)</fld> <def>A kind of boat used in Canton. It is about 25 feet long and is often rowed by women. Called also <altname>tankia</altname>.</def>  <rj><au>S. W. Williams.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tankage</ent><br/
<hw>Tank"age</hw> <pr>(?)</pr>, <pos>n.</pos> <sn>1.</sn> <def>The act or process of putting or storing in tanks.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><sn>2.</sn>  <def>Fees charged for storage in tanks.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><sn>3.</sn>  <def>The capacity or contents of a tank or tanks.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><sn>4.</sn>  <fld>(Agric.)</fld> <def>Waste matter from tanks; esp., the dried nitrogenous residue from tanks in which fat has been rendered, used as a fertilizer.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Tankard</ent><br/
<hw>Tank"ard</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[OF. <ets>tanquart</ets>; cf. OD. <ets>tanckaert</ets>; of uncertain origin.]</ety> <def>A large drinking vessel, especially one with a cover.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Marius was the first who drank out of a silver <qex>tankard</qex>, after the manner of Bacchus.</q> <rj><qau>Arbuthnot.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tankia</ent><br/
<hw>Tan"ki*a</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Naut.)</fld> <def>See <er>Tanka</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tankling</ent><br/
<hw>Tank"ling</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A tinkling.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tank vessel</ent><br/
<ent>Tank ship</ent><br/
<mhw>{ <hw>Tank ship</hw>, <hw>Tank vessel</hw>  }</mhw>. <fld>(Naut.)</fld> <def>A vessel fitted with tanks for the carrying of oil or other liquid in bulk; -- called also <altname>tanker</altname>.  A tank ship of very large capacity is called a <stype>supertanker</stype>.</def><br/
[<source>Webster 1913 Suppl.</source> <source>+PJC</source>]</p>

<p><ent>Tank top</ent><br/
<hw>Tank" top`</hw>, <pos>n.</pos> <def>a sleeveless and collarless shirt with wide shoulder straps and no front opening, often close-fitting and low-cut.</def><br/
[<source>PJC</source>]</p>

<p><ent>Tank warfare</ent><br/
<hw>Tank war"fare</hw>, <pos>n.</pos> <def>combat between tanks of opposing armies.</def><br/
[<source>PJC</source>]</p>

<p><ent>Tanling</ent><br/
<hw>Tan"ling</hw> <pr>(?)</pr>, <pos>n.</pos> <def>One tanned by the sun.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>Hot summer's <qex>tanlings</qex> and<br/
The shrinking slaves of winter.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tannable</ent><br/
<hw>Tan"na*ble</hw> <pr>(?)</pr>, <pos>a.</pos> <def>That may be tanned.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tannage</ent><br/
<hw>Tan"nage</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A tanning; the act, operation, or result of tanning.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>They should have got his cheek fresh <qex>tannage</qex>.</q> <rj><qau>R. Browning.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tannate</ent><br/
<hw>Tan"nate</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Cf. F. <ets>tannate</ets>.]</ety> <fld>(Chem.)</fld> <def>A salt of tannic acid.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tanner</ent><br/
<hw>Tan"ner</hw> <pr>(?)</pr>, <pos>n.</pos> <def>One whose occupation is to tan hides, or convert them into leather by the use of tan.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tanner</ent><br/
<hw>Tan"ner</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Etym. uncertain.]</ety> <def>A sixpence.</def> <mark>[Slang, Eng.]</mark><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Tannery</ent><br/
<hw>Tan"ner*y</hw> <pr>(?)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Tanneries</plw> <pr>(#)</pr>.</plu> <ety>[Cf. F. <ets>tannerie</ets>.]</ety> <sn>1.</sn> <def>A place where the work of tanning is carried on.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>The art or process of tanning.</def> <mark>[R.]</mark>  <rj><au>Carlyle.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tannic</ent><br/
<hw>Tan"nic</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Of or pertaining to tan; derived from, or resembling, tan; <as>as, <ex>tannic</ex> acid</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Tannic acid</b></col>. <fld>(Chem.)</fld> <sd>(a)</sd> <cd>An acid obtained from nutgalls as a yellow amorphous substance, <chform>C14H10O9</chform>, having an astringent taste, and forming with ferric salts a bluish-black compound, which is the basis of common ink. Called also <altname>tannin</altname>, and <altname>gallotannic acid</altname>.</cd> <sd>(b)</sd> <cd>By extension, any one of a series of astringent substances resembling tannin proper, widely diffused through the vegetable kingdom, as in oak bark, willow, catechu, tea, coffee, etc.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tannier</ent><br/
<hw>Tan"ni*er</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Bot.)</fld> <def>See <er>Tanier</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tannigen</ent><br/
<hw>Tan"ni*gen</hw> <pr>(?)</pr>, <pos>n.</pos>  <ety>[<ets>Tann</ets>in + <ets>-gen</ets>.]</ety> <fld>(Pharm.)</fld> <def>A compound obtained as a yellowish gray powder by the action of acetyl chloride or acetic anhydride or ordinary tannic acid. It is used as an intestinal astringent, and locally in rhinitis and pharyngitis.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Tannin</ent><br/
<hw>Tan"nin</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Cf. F. <ets>tannin</ets>.]</ety> <fld>(Chem.)</fld> <def>Same as <cref>Tannic acid</cref>, under <er>Tannic</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tanning</ent><br/
<hw>Tan"ning</hw>, <pos>n.</pos> <def>The art or process of converting skins into leather. See <er>Tan</er>, <pos>v. t.</pos>, 1.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tanrec</ent><br/
<hw>Tan"rec</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Zool.)</fld> <def>Same as <er>Tenrec</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tansy</ent><br/
<hw>Tan"sy</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[OE. <ets>tansaye</ets>, F. <ets>tanaise</ets>; cf. It. & Sp. <ets>tanaceto</ets>, NL. <ets>tanacetum</ets>, Pg. <ets>atanasia</ets>, <ets>athanasia</ets>, Gr. <grk>'aqanasi`a</grk> immortality, fr. <grk>'aqa`natos</grk> immortal; <grk>'a</grk> priv. + <grk>qa`natos</grk> death.]</ety> <sn>1.</sn> <fld>(Bot.)</fld> <def>Any plant of the composite genus <gen>Tanacetum</gen>. The common tansy (<spn>Tanacetum vulgare</spn>) has finely divided leaves, a strong aromatic odor, and a very bitter taste. It is used for medicinal and culinary purposes.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A dish common in the seventeenth century, made of eggs, sugar, rose water, cream, and the juice of herbs, baked with butter in a shallow dish.</def> <mark>[Obs.]</mark>  <rj><au>Pepys.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Double tansy</b></col> <fld>(Bot.)</fld>, <cd>a variety of the common tansy with the leaves more dissected than usual.</cd> -- <col><b>Tansy mustard</b></col> <fld>(Bot.)</fld>, <cd>a plant (<spn>Sisymbrium canescens</spn>) of the Mustard family, with tansylike leaves.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tant</ent><br/
<hw>Tant</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Cf. <er>Taint</er> tincture.]</ety> <fld>(Zool.)</fld> <def>A small scarlet arachnid.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tantalate</ent><br/
<hw>Tan"ta*late</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Chem.)</fld> <def>A salt of tantalic acid.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tantalic</ent><br/
<hw>Tan*tal"ic</hw> <pr>(?)</pr>, <pos>a.</pos> <fld>(Chem.)</fld> <def>Of or pertaining to tantalum; derived from, or containing, tantalum; specifically, designating any one of a series of acids analogous to nitric acid and the polyacid compounds of phosphorus.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tantalism</ent><br/
<hw>Tan"ta*lism</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[See <er>Tantalize</er>.]</ety> <def>A punishment like that of Tantalus; a teasing or tormenting by the hope or near approach of good which is not attainable; tantalization.</def>  <rj><au>Addison.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Is not such a provision like <qex>tantalism</qex> to this people?</q> <rj><qau>Josiah Quincy.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tantalite</ent><br/
<hw>Tan"ta*lite</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Cf. F. <ets>tantalite</ets>.]</ety> <fld>(Min.)</fld> <def>A heavy mineral of an iron-black color and submetallic luster.  It is essentially a tantalate of iron.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tantalization</ent><br/
<hw>Tan`ta*li*za"tion</hw> <pr>(?)</pr>, <pos>n.</pos> <def>The act of tantalizing, or state of being tantalized.</def>  <rj><au>Gayton.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tantalize</ent><br/
<hw>Tan"ta*lize</hw> <pr>(?)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Tantalized</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Tantalizing</conjf> <pr>(?)</pr>.]</vmorph> <ety>[From <er>Tantalus</er>: cf. F. <ets>tantaliser</ets>.]</ety> <def>To tease or torment by presenting some good to the view and exciting desire, but continually frustrating the expectations by keeping that good out of reach; to tease; to torment.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Thy vain desires, at strife<br/
Within themselves, have <qex>tantalized</qex> thy life.</q> <rj><qau>Dryden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><syn><b>Syn.</b> -- To tease; vex; irritate; provoke.</syn> <usage> -- <er>Tantalize</er>, <er>Disappoint</er>. To <xex>disappoint</xex> is literally to <xex>do away</xex> with what was (or was taken to be) <xex>appointed</xex>; hence the peculiar pain from hopes thus dashed to the ground. To <xex>tantalize</xex>, a much stronger term, describes a most distressing form of disappointment, as in the case of Tantalus, the Phrygian king. To <xex>tantalize</xex> is to visit with the bitterest disappointment -- to torment by exciting hopes or expectations which can never be realized.</usage><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tantalizer</ent><br/
<hw>Tan"ta*li`zer</hw> <pr>(?)</pr>, <pos>n.</pos> <def>One who tantalizes.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tantalizingly</ent><br/
<hw>Tan"ta*li`zing*ly</hw> <pr>(?)</pr>, <pos>adv.</pos> <def>In a tantalizing or teasing manner.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tantalum</ent><br/
<hw>Tan"ta*lum</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[NL. So named on account of the perplexity and difficulty encounterd by its discoverer (Ekeberg) in isolating it. See <er>Tantalus</er>.]</ety> <fld>(Chem.)</fld> <def>A rare nonmetallic element found in certain minerals, as tantalite, samarskite, and fergusonite, and isolated as a dark powder which becomes steel-gray by burnishing. Symbol Ta. Atomic weight 182.0. Formerly called also <altname>tantalium</altname>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tantalus</ent><br/
<hw>Tan"ta*lus</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L., from Gr. <grk>Ta`ntalos</grk>.]</ety> <fld>(Gr. Myth.)</fld> <sn>1.</sn> <def>A Phrygian king who was punished in the lower world by being placed in the midst of a lake whose waters reached to his chin but receded whenever he attempted to allay his thirst, while over his head hung branches laden with choice fruit which likewise receded whenever he stretched out his hand to grasp them.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Zool.)</fld> <def>A genus of wading birds comprising the wood ibises.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Tantalus's cup</b></col> <fld>(Physics)</fld>, <cd>a philosophical toy, consisting of a cup, within which is the figure of a man, and within the figure a siphon, the longer arm of which passes down through the bottom of the cup, and allows the escape of any liquid that may be poured in, when it reaches as high as the bend of the siphon, which is just below the level of the mouth of the figure in the cup.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tantamount</ent><br/
<hw>Tan"ta*mount`</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[F. <ets>tant</ets> so much (L. <ets>tantus</ets>) + E. <ets>amount</ets>.]</ety> <def>Equivalent in value, signification, or effect.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>A usage nearly <qex>tantamount</qex> to constitutional right.</q> <rj><qau>Hallam.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>The certainty that delay, under these circumstances, was <qex>tantamount</qex> to ruin.</q> <rj><qau>De Quincey.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tantamount</ent><br/
<hw>Tan"ta*mount`</hw>, <pos>v. i.</pos> <def>To be tantamount or equivalent; to amount.</def> <mark>[Obs.]</mark>  <rj><au>Jer. Taylor.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tantivy</ent><br/
<hw>Tan*tiv"y</hw> <pr>(?)</pr>, <pos>adv.</pos> <ety>[Said to be from the note of a hunting horn.]</ety> <def>Swiftly; speedily; rapidly; -- a fox-hunting term; <as>as, to ride <ex>tantivy</ex></as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tantivy</ent><br/
<hw>Tan*tiv"y</hw>, <pos>n.</pos> <def>A rapid, violent gallop; an impetuous rush.</def>  <rj><au>Cleverland.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tantivy</ent><br/
<hw>Tan*tiv"y</hw>, <pos>v. i.</pos> <def>To go away in haste.</def> <mark>[Colloq.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tantra</ent><br/
<hw>Tan"tra</hw> <pr>(t<acr/n"tr<adot/; t<ucr/n"tr<adot/)</pr>, <pos>n.</pos> <ety>[Skr.]</ety> <fld>(Hinduism)</fld> <def>A ceremonial treatise related to Puranic and magic literature; esp., one of the sacred works of the worshipers of Sakti.</def> -- <wordforms><wf>Tan"tric</wf> <pr>(t<acr/n"tr<icr/k)</pr>, <pos>a.</pos></wordforms><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Tantrism</ent><br/
<hw>Tan"trism</hw> <pr>(?)</pr>, <pos>n.</pos> <def>The system of doctrines and  rites taught in the tantras.</def> -- <wordforms><wf>Tan"trist</wf> <pr>(#)</pr>, <pos>n.</pos></wordforms><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Tantrum</ent><br/
<hw>Tan"trum</hw> <pr>(?)</pr>, <pos>n.</pos> <sn>1.</sn> <def>A whim; an affected air.</def> <mark>[Colloq. and archaic]</mark>  <rj><au>Thackeray.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A display of ill-humor, especially a demonstration of rage or frustration by shouting or violent physical movements, such as the stamping of feet; called also <altname>temper tantrum</altname>.  It is usually associated with children, but is sometimes seen in adults.</def><br/
[<source>PJC</source>]</p>

<p><sn>3.</sn> <mark>[fig.]</mark> <def>A display of anger expressed by irrationally striking out at innocent targets or inanimate objects; <as>as, the governor was so insulted by the article, he threw a temper <ex>tantrum</ex> and cancelled the ceremony</as>.</def><br/
[<source>PJC</source>]</p>

<p><ent>Tanyard</ent><br/
<hw>Tan"yard`</hw> <pr>(?)</pr>, <pos>n.</pos> <def>An inclosure where the tanning of leather is carried on; a tannery.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tanystomata</ent><br/
||<hw>Tan`y*stom"a*ta</hw> <pr>(?)</pr>, <pos>n. pl.</pos> <ety>[NL., fr. Gr. <?/ to stretch + <?/, <?/, mouth.]</ety> <fld>(Zool.)</fld> <def>A division of dipterous insects in which the proboscis is large and contains lancelike mandibles and maxillae. The horseflies and robber flies are examples.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Taoism</ent><br/
<hw>Ta"o*ism</hw> <pr>(?)</pr>, <pos>n.</pos> <def>One of the popular religions of China, sanctioned by the state.</def> -- <wordforms><wf>Ta"o*ist</wf>, <pos>a. & n.</pos></wordforms><br/
[<source>1913 Webster</source>]</p>

<p><ent>Taotai</ent><br/
||<hw>Tao`tai"</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Chin. <ets>tao</ets> circuit + <ets>t'ai</ets>, a title of respect.]</ety> <def>In China, an official at the head of the civil and military affairs of a circuit, which consists of two or more <xex>fu</xex>, or territorial departments; -- called also, by foreigners, <altname>intendant of circuit</altname>. Foreign consuls and commissioners associated with taotais as superintendants of trade at the treaty ports are ranked with the taotai.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Tap</ent><br/
<hw>Tap</hw> <pr>(?)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Tapped</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Tapping</conjf>.]</vmorph> <ety>[F. <ets>taper</ets> to strike; of Teutonic origin; cf. dial. G. <ets>tapp</ets>, <ets>tapps</ets>, a blow, <ets>tappe</ets> a paw, fist, G. <ets>tappen</ets> to grope.]</ety> <sn>1.</sn> <def>To strike with a slight or gentle blow; to touch gently; to rap lightly; to pat; <as>as, to <ex>tap</ex> one with the hand or a cane</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To put a new sole or heel on; <as>as, to <ex>tap</ex> shoes</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tap</ent><br/
<hw>Tap</hw>, <pos>n.</pos> <ety>[Cf. F. <ets>tape</ets>. See <er>Tap</er> to strike.]</ety> <sn>1.</sn> <def>A gentle or slight blow; a light rap; a pat.</def>  <rj><au>Addison.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A piece of leather fastened upon the bottom of a boot or shoe in repairing or renewing the sole or heel.</def>
<-- a piece of metal so fastened, used to reduce wear on the shoe, or for the purpose of tap dancing. --><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <pluf>pl.</pluf> <fld>(Mil.)</fld> <def>A signal, by drum or trumpet, for extinguishing all lights in soldiers' quarters and retiring to bed, -- usually given about a quarter of an hour after tattoo.</def>  <rj><au>Wilhelm.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tap</ent><br/
<hw>Tap</hw>, <pos>v. i.</pos> <def>To strike a gentle blow.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tap</ent><br/
<hw>Tap</hw>, <pos>n.</pos> <ety>[AS. <ets>t<ae/ppa</ets>, akin to D. <ets>tap</ets>, G. <ets>zapfen</ets>, OHG. <ets>zapfo</ets>, Dan. <ets>tap</ets>, Sw. <ets>tapp</ets>, Icel. <ets>tappi</ets>.  Cf. <er>Tampion</er>, <er>Tip</er>.]</ety> <sn>1.</sn> <def>A hole or pipe through which liquor is drawn.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A plug or spile for stopping a hole pierced in a cask, or the like; a faucet.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>Liquor drawn through a tap; hence, a certain kind or quality of liquor; <as>as, a liquor of the same <ex>tap</ex></as>.</def> <mark>[Colloq.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>A place where liquor is drawn for drinking; a taproom; a bar.</def> <mark>[Colloq.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <fld>(Mech.)</fld> <def>A tool for forming an internal screw, as in a nut, consisting of a hardened steel male screw grooved longitudinally so as to have cutting edges.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>On tap</b></col>. <sd>(a)</sd> <cd>Ready to be drawn; <as>as, ale <ex>on tap</ex></as>.</cd> <sd>(b)</sd> <cd>Broached, or furnished with a tap; <as>as, a barrel <ex>on tap</ex></as>.</cd> -- <col><b>Plug tap</b></col> <fld>(Mech.)</fld>, <cd>a screw-cutting tap with a slightly tapering end.</cd> -- <col><b>Tap bolt</b></col>, <cd>a bolt with a head on one end and a thread on the other end, to be screwed into some fixed part, instead of passing through the part and receiving a nut. See <xex>Illust.</xex> under <er>Bolt</er>.</cd> -- <col><b>Tap cinder</b></col> <fld>(Metal.)</fld>, <cd>the slag of a puddling furnace.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tap</ent><br/
<hw>Tap</hw>, <pos>v. t.</pos> <sn>1.</sn> <def>To pierce so as to let out, or draw off, a fluid; <as>as, to <ex>tap</ex> a cask, a tree, a tumor,  a keg of beer, etc.</as></def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Hence, to draw resources from (a reservoir) in any analogous way; <as>as, to <ex>tap</ex> someone's knowledge of the Unix system; to <ex>tap</ex> the treasury.</as></def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>To draw, or cause to flow, by piercing.</def>  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>He has been <qex>tapping</qex> his liquors.</q> <rj><qau>Addison.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <fld>(Mech.)</fld> <def>To form an internal screw in (anything) by means of a tool called a <xex>tap</xex>; <as>as, to <ex>tap</ex> a nut, a pipe, or tubing</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <def>to connect a listening device to (a telephone or telegraph line) secretly, for the purpose of hearing private conversations; also, to obtain or record (information) by tapping; -- a technique used by law enforcement agencies investigating suspected criminals.  In the United States it is illegal without a court order permitting it.</def><br/
[<source>PJC</source>]</p>

<p><ent>Tapa</ent><br/
||<hw>Ta"pa</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A kind of cloth prepared by the Polynesians from the inner bark of the paper mulberry; -- sometimes called also <altname>kapa</altname>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tapadero</ent><br/
<ent>Tapadera</ent><br/
<mhw>{ ||<hw>Tap`a*de"ra</hw> <pr>(?)</pr>, ||<hw>Tap`a*de"ro</hw> <pr>(?)</pr> }</mhw>, <pos>n.</pos>} <altsp>[Also <asp>tapidero</asp>.]</altsp> <ety>[Sp. <ets>tapadera</ets> lid, cover.]</ety> <def>One of the leather hoods which cover the stirrups of a Mexican saddle.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Tapayaxin</ent><br/
||<hw>Ta`pa*yax"in</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Zool.)</fld> <def>A Mexican spinous lizard (<spn>Phrynosoma orbiculare</spn>) having a head somewhat like that of a toad; -- called also <altname>horned toad</altname>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tape</ent><br/
<hw>Tape</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[AS. <ets>t<ae/ppe</ets> a fillet.  Cf. <er>Tapestry</er>, <er>Tippet</er>.]</ety> <sn>1.</sn> <def>A narrow fillet or band of cotton or linen; a narrow woven fabric used for strings and the like; <as>as, curtains tied with <ex>tape</ex></as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A tapeline; also, a metallic ribbon so marked as to serve as a tapeline; <as>as, a steel <ex>tape</ex></as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Red tape</b></col>. <cd>See under <er>Red</er>.</cd> -- <col><b>Tape grass</b></col> <fld>(Bot.)</fld>, <cd>a plant (<spn>Vallisneria spiralis</spn>) with long ribbonlike leaves, growing in fresh or brackish water; -- called also <altname>fresh-water eelgrass</altname>, and, in Maryland, <altname>wild celery</altname>.</cd> -- <col><b>Tape needle</b></col>.  <cd>See <er>Bodkin</er>, <pos>n.</pos>, 4.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tape</ent><br/
<hw>Tape</hw> <pr>(?)</pr>, <pos>v. t.</pos>  <vmorph>[<pos>imp. & p. p.</pos> <conjf>Taped</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Taping</conjf>.]</vmorph> <sn>1.</sn> <def>To furnish with tape; to fasten, tie, bind, or the like, with tape;</def> <specif>specif.</specif> <fld>(Elec.)</fld>, <def>to cover (a wire) with insulating tape.</def></p>

<p><sn>2.</sn> <def>to record on audio tape or video tape; -- either directly, at the scene of the action tape, or indirectly, as from a broadcast of the action.</def> <ldquo/I was busy when that episode was on TV, but I <ex>taped</ex> it and watched it later.<rdquo/<br/
[<source>PJC</source>]</p>

<p><ent>Tape deck</ent><br/
<hw>Tape deck</hw> <pr>(?)</pr>, <pos>n.</pos> <def>an electroinic device for recording and playing back sounds on magnetic tape; usually it needs to be connected to an amplifier system for playback.</def><br/
[<source>PJC</source>]</p>

<p><ent>Tapeline</ent><br/
<hw>Tape"line`</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A painted tape, marked with linear dimensions, as inches, feet, etc., and often inclosed in a case, -- used for measuring.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Taper</ent><br/
<hw>Ta"per</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[AS. <ets>tapur</ets>, <ets>tapor</ets>, <ets>taper</ets>; cf. Ir. <ets>tapar</ets>, W. <ets>tampr</ets>.]</ety> <sn>1.</sn> <def>A small wax candle; a small lighted wax candle; hence, a small light.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Get me a <qex>taper</qex> in my study, Lucius.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A tapering form; gradual diminution of thickness in an elongated object; <as>as, the <ex>taper</ex> of a spire</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><-- p. 1474 --><br/
[<source>1913 Webster</source>]</p>

<p><ent>Taper</ent><br/
<hw>Ta"per</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[Supposed to be from <ets>taper</ets>, n., in allusion to its form.]</ety> <def>Regularly narrowed toward the point; becoming small toward one end; conical; pyramidical; <as>as, <ex>taper</ex> fingers</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Taper</ent><br/
<hw>Ta"per</hw>, <pos>v. i.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Tapered</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Tapering</conjf>.]</vmorph> <def>To become gradually smaller toward one end; <as>as, a sugar loaf <ex>tapers</ex> toward one end</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Taper</ent><br/
<hw>Ta"per</hw>, <pos>v. t.</pos> <def>To make or cause to taper.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tape recorder</ent><br/
<hw>Tape recorder</hw> <pr>(?)</pr>, <pos>n.</pos> <def>an electroinic device for recording and playing back sounds on magnetic tape; it often has an integrated microphone, amplifier, and speaker, and in such cases requires no additional equipment other than the magnetic tape for recording or playback.</def><br/
[<source>PJC</source>]</p>

<p><ent>Tape recording</ent><br/
<hw>Tape recording</hw> <pr>(?)</pr>, <pos>n.</pos> <sn>1.</sn> <def>a recording of sound or video on magnetic tape.</def><br/
[<source>PJC</source>]</p>

<p><sn>2.</sn> <def>the process of recording sound or video on magnetic tape.</def><br/
[<source>PJC</source>]</p>

<p><ent>Tapered</ent><br/
<hw>Ta"pered</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Lighted with a taper or tapers; <as>as, a <ex>tapered</ex> choir</as>.</def> <mark>[R.]</mark>  <rj><au>T. Warton.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tapering</ent><br/
<hw>Ta"per*ing</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Becoming gradually smaller toward one end.</def> -- <wordforms><wf>Ta"per*ing*ly</wf>, <pos>adv.</pos></wordforms><br/
[<source>1913 Webster</source>]</p>

<p><ent>Taperness</ent><br/
<hw>Ta"per*ness</hw>, <pos>n.</pos> <def>The quality or state of being taper; tapering form; taper.</def>  <rj><au>Shenstone.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tapestry</ent><br/
<hw>Tap"es*try</hw> <pr>(?)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Tapestries</plw> <pr>(#)</pr>.</plu> <ety>[F. <ets>tapissere</ets>, fr. <ets>tapisser</ets> to carpet, to hang, or cover with tapestry, fr. <ets>tapis</ets> a carpet, carpeting, LL. <ets>tapecius</ets>, fr. L. <ets>tapete</ets> carpet, tapestry, Gr. <?/, <?/.  Cf. <er>Tapis</er>, <er>Tippet</er>.]</ety> <def>A fabric, usually of worsted, worked upon a warp of linen or other thread by hand, the designs being usually more or less pictorial and the stuff employed for wall hangings and the like. The term is also applied to different kinds of embroidery.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Tapestry carpet</b></col>, <cd>a kind of carpet, somewhat resembling Brussels, in which the warp is printed before weaving, so as to produce the figure in the cloth.</cd> -- <col><b>Tapestry moth</b></col>. <fld>(Zool.)</fld> <cd>Same as <cref>Carpet moth</cref>, under <er>Carpet</er>.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tapestry</ent><br/
<hw>Tap"es*try</hw>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Tapestried</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Tapestrying</conjf>.]</vmorph> <def>To adorn with tapestry, or as with tapestry.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The Trosachs wound, as now, between gigantic walls of rock <qex>tapestried</qex> with broom and wild roses.</q> <rj><qau>Macaulay.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tapestry beetle</ent><br/
<hw>Tap"es*try bee"tle</hw>. <def>A small black dermestoid beetle (<spn>Attagenus piceus</spn>) whose larva feeds on tapestry, carpets, silk, fur, flour, and various other goods.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Tapet</ent><br/
<hw>Tap"et</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>tapete</ets>. See <er>Tapestry</er>.]</ety> <def>Worked or figured stuff; tapestry.</def> <mark>[R.]</mark>  <rj><au>Spenser.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tapeti</ent><br/
<hw>Tap"e*ti</hw> <pr>(?)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Tapetis</plw> <pr>(#)</pr>.</plu> <ety>[Braz.]</ety> <fld>(Zool.)</fld> <def>A small South American hare (<spn>Lepus Braziliensis</spn>).</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tapetum</ent><br/
||<hw>Ta*pe"tum</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[NL., from L. <ets>tapete</ets> a carpet, a tapestry.]</ety> <fld>(Anat.)</fld> <def>An area in the pigmented layer of the choroid coat of the eye in many animals, which has an iridescent or metallic luster and helps to make the eye visible in the dark. Sometimes applied to the whole layer of pigmented epithelium of the choroid.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tapeworm</ent><br/
<hw>Tape"worm`</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Zool.)</fld> <def>Any one of numerous species of cestode worms belonging to Taenia and many allied genera. The body is long, flat, and composed of numerous segments or proglottids varying in shape, those toward the end of the body being much larger and longer than the anterior ones, and containing the fully developed sexual organs. The head is small, destitute of a mouth, but furnished with two or more suckers (which vary greatly in shape in different genera), and sometimes, also, with hooks for adhesion to the walls of the intestines of the animals in which they are parasitic. The larvae (see <er>Cysticercus</er>) live in the flesh of various creatures, and when swallowed by another animal of the right species develop into the mature tapeworm in its intestine. See <xex>Illustration</xex> in Appendix.</def><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ Three species are common parasites of man: the <stype>pork tapeworm</stype> (<spn>Taenia solium</spn>), the larva of which is found in pork; the <stype>beef tapeworm</stype> (<spn>Taenia mediocanellata</spn>), the larva of which lives in the flesh of young cattle; and the <stype>broad tapeworm</stype> (<spn>Bothriocephalus latus</spn>) which is found chiefly in the inhabitants of the mountainous regions of Europe and Asia. See also <er>Echinococcus</er>, <er>Cysticercus</er>, <er>Proglottis</er>, and 2d <er>Measles</er>, 4.</note><br/
[<source>1913 Webster</source>]</p>

<p><ent>Taphouse</ent><br/
<hw>Tap"house`</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A house where liquors are retailed.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Taphrenchyma</ent><br/
||<hw>Taph*ren"chy*ma</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Gr. <?/ a trench + <ets>enchyma</ets>, as in <ets>parenchyma</ets>.]</ety> <fld>(Bot.)</fld> <def>Same as <er>Bothrenchyma</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tapinage</ent><br/
<hw>Tap"i*nage</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[See <er>Tapish</er>.]</ety> <def>A lurking or skulking.</def> <mark>[Obs.]</mark>  <rj><au>Gower.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tapioca</ent><br/
<hw>Tap`i*o"ca</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Braz. <ets>tapioka</ets>: cf. Pg., Sp. & F. <ets>tapioca</ets>.]</ety> <def>A coarsely granular substance obtained by heating, and thus partly changing, the moistened starch obtained from the roots of the cassava. It is much used in puddings and as a thickening for soups. See <er>Cassava</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tapir</ent><br/
<hw>Ta"pir</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Braz. <ets>tapy'ra</ets>: cf. F. <ets>tapir</ets>.]</ety> <fld>(Zool.)</fld> <def>Any one of several species of large odd-toed ungulates belonging to <gen>Tapirus</gen>, <gen>Elasmognathus</gen>, and allied genera. They have a long prehensile upper lip, short ears, short and stout legs, a short, thick tail, and short, close hair. They have three toes on the hind feet, and four toes on the fore feet, but the outermost toe is of little use.</def><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ The best-known species are the Indian tapir (<spn>Tapirus Indicus</spn>), native of the <geog>East Indies</geog> and <geog>Malacca</geog>, which is black with a broad band of white around the middle, and the common American tapir (<spn>Tapirus Americanus</spn>), which, when adult, is dull brown. Several others species inhabit the Andes and Central America.</note><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Tapir tiger</b></col> <fld>(Zool.)</fld>, <cd>the wallah.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tapiroid</ent><br/
<hw>Ta"pir*oid</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[<ets>Tapir</ets> + <ets>-oid</ets>.]</ety> <fld>(Zool.)</fld> <def>Allied to the tapir, or the Tapir family.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tapis</ent><br/
<hw>Ta"pis</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F. See <er>Tapestry</er>.]</ety> <def>Tapestry; formerly, the cover of a council table.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><mcol><col><b>On the tapis</b></col>, <it>or</it> <col><b>Upon the tapis</b></col></mcol>, <cd>on the table, or under consideration; <as>as, to lay a motion in Parliament <ex>on the tapis</ex></as>.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tapis</ent><br/
<hw>Tap"is</hw> <pr>(?)</pr>, <pos>v. t.</pos> <def>To cover or work with figures like tapestry.</def> <mark>[R.]</mark>  <rj><au>Holland.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tapiser</ent><br/
<hw>Tap"is*er</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F. <ets>tapissier</ets>.]</ety> <def>A maker of tapestry; an upholsterer.</def> <mark>[R.]</mark>  <rj><au>Chaucer.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tapish</ent><br/
<hw>Tap"ish</hw> <pr>(?)</pr>, <pos>v. i.</pos> <ety>[F. se <ets>tapir</ets> to squat.]</ety> <def>To lie close to the ground, so as to be concealed; to squat; to crouch; hence, to hide one's self.</def> <altsp>[Written also <asp>tappis</asp>, <asp>tappish</asp>, <asp>tappice</asp>.]</altsp> <mark>[Obs. or Prov. Eng.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>As a hound that, having roused a hart,<br/
Although he <qex>tappish</qex> ne'er so soft.</q> <rj><qau>Chapman.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Taplash</ent><br/
<hw>Tap"lash`</hw> <pr>(?)</pr>, <pos>n.</pos> <def>Bad small beer; also, the refuse or dregs of liquor.</def> <mark>[Obs. or Prov. Eng.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>The <qex>taplash</qex> of strong ale and wine.</q> <rj><qau>Taylor (1630).</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Taplings</ent><br/
<hw>Tap"lings</hw> <pr>(?)</pr>, <pos>n. pl.</pos> <def>The strong double leathers by which the two parts of a flail are united.</def>  <rj><au>Halliwell.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tapoa tafa</ent><br/
||<hw>Ta*po"a ta"fa</hw> <pr>(?)</pr>. <fld>(Zool.)</fld> <def>A small carnivorous marsupial (<spn>Phascogale penicillata</spn>) having long, soft fur, and a very long tail with a tuft of long hairs at the end; -- called also <altname>brush-tailed phascogale</altname>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tappen</ent><br/
<hw>Tap"pen</hw> <pr>(?)</pr>, <pos>n.</pos> <def>An obstruction, or indigestible mass, found in the intestine of bears and other animals during hibernation.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tapper</ent><br/
<hw>Tap"per</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Zool.)</fld> <def>The lesser spotted woodpecker (<spn>Dendrocopus minor</spn>); -- called also <altname>tapperer</altname>, <altname>tabberer</altname>, <altname>little wood pie</altname>, <altname>barred woodpecker</altname>, <altname>wood tapper</altname>, <altname>hickwall</altname>, and <altname>pump borer</altname>.</def> <mark>[Prov. Eng.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tappester</ent><br/
<hw>Tap"pes*ter</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[See <er>Tapster</er>.]</ety> <def>A female tapster.</def> <mark>[Obs.]</mark>  <rj><au>Chaucer.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tappet</ent><br/
<hw>Tap"pet</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Mach.)</fld> <def>A lever or projection moved by some other piece, as a cam, or intended to tap or touch something else, with a view to produce change or regulate motion.</def>  <rj><au>G. Francis.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Tappet motion</b></col>, <cd>a valve motion worked by tappets from a reciprocating part, without an eccentric or cam, -- used in steam pumps, etc.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tappet rod</ent><br/
<hw>Tap"pet rod</hw>. <fld>(Mech.)</fld> <def>A rod carrying a tappet or tappets, as one for closing the valves in a Cornish pumping engine.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Tappis</ent><br/
<ent>Tappice</ent><br/
<mhw>{ <hw>Tap"pice</hw> <pr>(?)</pr>, <hw>Tap"pis</hw> <pr>(?)</pr> }</mhw>, <pos>v. i.</pos> <def>See <er>Tapish</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tappit hen</ent><br/
<hw>Tap"pit hen`</hw> <pr>(?)</pr>. <sn>1.</sn> <def>A hen having a tuft of feathers on her head.</def> <mark>[Scot.]</mark>  <rj><au>Jamieson.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A measuring pot holding one quart (according to some, three quarts); -- so called from a knob on the lid, thought to resemble a crested hen.</def> <mark>[Scot.]</mark>  <rj><au>Jamieson.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tappoon</ent><br/
<hw>Tap*poon"</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Sp. <ets>tamp<oacute/n</ets> a stopper.]</ety> <fld>(Irrigation)</fld> <def>A piece of wood or sheet metal fitted into a ditch to dam up the water so as to overflow a field.</def> <mark>[U. S.]</mark><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Taproom</ent><br/
<hw>Tap"room`</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A room where liquors are kept on tap; a barroom.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The ambassador was put one night into a miserable <qex>taproom</qex>, full of soldiers smoking.</q> <rj><qau>Macaulay.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Taproot</ent><br/
<hw>Tap"root`</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Bot.)</fld> <def>The root of a plant which penetrates the earth directly downward to a considerable depth without dividing.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tapster</ent><br/
<hw>Tap"ster</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[AS. <ets>t<ae/ppestre</ets> a female tapster. See <er>Tap</er> a plug, pipe, and <er>-ster</er>.]</ety> <def>One whose business is to tap or draw ale or other liquor.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Taqua-nut</ent><br/
<hw>Ta"qua-nut`</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Bot.)</fld> <def>A Central American name for the ivory nut.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tar</ent><br/
<hw>Tar</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Abbrev. from <ets>tarpaulin</ets>.]</ety> <def>A sailor; a seaman.</def> <mark>[Colloq.]</mark>  <rj><au>Swift.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tar</ent><br/
<hw>Tar</hw>, <pos>n.</pos> <ety>[OE. <ets>terre</ets>, <ets>tarre</ets>, AS. <ets>teru</ets>, <ets>teoru</ets>; akin to D. <ets>teer</ets>, G. <ets>teer</ets>, <ets>theer</ets>, Icel. <ets>tjara</ets>, Sw. <ets>tj<aum/ra</ets>, Dan. <ets>ti<ae/re</ets>, and to E. <ets>tree</ets>. <root/63. See <er>Tree</er>.]</ety> <def>A thick, black, viscous liquid obtained by the distillation of wood, coal, etc., and having a varied composition according to the temperature and material employed in obtaining it.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Coal tar</b></col>. <cd>See in the Vocabulary.</cd> -- <col><b>Mineral tar</b></col> <fld>(Min.)</fld>, <cd>a kind of soft native bitumen.</cd> -- <col><b>Tar board</b></col>, <cd>a strong quality of millboard made from junk and old tarred rope.</cd> <au>Knight.</au> -- <col><b>Tar water</b></col>. <sd>(a)</sd> <cd>A cold infusion of tar in water, used as a medicine.</cd> <sd>(b)</sd> <cd>The ammoniacal water of gas works.</cd> -- <col><b>Wood tar</b></col>, <cd>tar obtained from wood. It is usually obtained by the distillation of the wood of the pine, spruce, or fir, and is used in varnishes, cements, and to render ropes, oakum, etc., impervious to water.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tar</ent><br/
<hw>Tar</hw>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Tarred</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Tarring</conjf>.]</vmorph> <def>To smear with tar, or as with tar; <as>as, to <ex>tar</ex> ropes; to <ex>tar</ex> cloth.</as></def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>To tar and feather a person</b></col>. <cd>See under <er>Feather</er>, <pos>v. t.</pos></cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Taranis</ent><br/
<hw>Tar"a*nis</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>taranis</ets>, from the Celtic; cf. W. & Corn. <ets>taran</ets> thunder.]</ety> <fld>(Myth.)</fld> <def>A Celtic divinity, regarded as the evil principle, but confounded by the Romans with Jupiter.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tarantass</ent><br/
<hw>Tar`an*tass"</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Russ. <ets>tarantas'</ets>.]</ety> <def>A low four-wheeled carriage used in Russia. The carriage box rests on two long, springy poles which run from the fore to the hind axletree. When snow falls, the wheels are taken off, and the body is mounted on a sledge.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tarantella</ent><br/
<hw>Tar`an*tel"la</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[It.]</ety> <fld>(Mus.)</fld> <sd>(a)</sd> <def>A rapid and delirious sort of Neapolitan dance in 6-8 time, which moves in whirling triplets; -- so called from a popular notion of its being a remedy against the poisonous bite of the <xex>tarantula</xex>. Some derive its name from Taranto in Apulia.</def> <sd>(b)</sd> <def>Music suited to such a dance.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tarantism</ent><br/
<hw>Tar"ant*ism</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[It. <ets>tarantismo</ets>: cf. F. <ets>tarentisme</ets>. See <er>Tarantula</er>.]</ety> <fld>(Med.)</fld> <def>A nervous affection producing melancholy, stupor, and an uncontrollable desire to dance. It was supposed to be produced by the bite of the tarantula, and considered to be incapable of cure except by protracted dancing to appropriate music.</def> <altsp>[Written also <asp>tarentism</asp>.]</altsp><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tarantula</ent><br/
<hw>Ta*ran"tu*la</hw> <pr>(?)</pr>, <pos>n.</pos>; <plu><it>pl.</it> E. <plw>Tarantulas</plw> <pr>(#)</pr>, L. <plw>Tarantulae</plw> <pr>(#)</pr>.</plu> <ety>[NL., fr. It. <ets>tarantola</ets>, fr. L. <ets>Tarentum</ets>, now <ets>Taranto</ets>, in the south of Italy.]</ety> <fld>(Zool.)</fld> <def>Any one of several species of large spiders, popularly supposed to be very venomous, especially the European species (<spn>Tarantula apuliae</spn>). The tarantulas of Texas and adjacent countries are large species of Mygale.</def> <altsp>[Written also <asp>tarentula</asp>.]</altsp><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Tarantula killer</b></col>, <cd>a very large wasp (<spn>Pompilus formosus</spn>), which captures the Texan tarantula (<spn>Mygale Hentzii</spn>) and places it in its nest as food for its young, after paralyzing it by a sting.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tarantulated</ent><br/
<hw>Ta*ran"tu*la`ted</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Bitten by a tarantula; affected with tarantism.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tarbogan</ent><br/
<hw>Tar*bog"an</hw> <pr>(?)</pr>, <pos>n. & v.</pos> <def>See <er>Toboggan</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tarboosh</ent><br/
<hw>Tar*boosh"</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Ar. <ets>tarb<?/sh</ets>; perhaps from Per. <ets>sar-posh</ets> headdress: cf. F. <ets>tarbouch</ets>.]</ety> <def>A red cap worn by Turks and other Eastern nations, sometimes alone and sometimes swathed with linen or other stuff to make a turban. See <er>Fez</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tardation</ent><br/
<hw>Tar*da"tion</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>tardatio</ets>, fr. <ets>tardare</ets>, <ets>tardatum</ets>, to retard, delay, fr. <ets>tardus</ets> slow.]</ety> <def>The act of retarding, or delaying; retardation.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tardigrada</ent><br/
||<hw>Tar`di*gra"da</hw> <pr>(?)</pr>, <pos>n. pl.</pos> <ety>[NL. See <er>Tardigrade</er>, <pos>a.</pos>]</ety> <sn>1.</sn> <fld>(Zool.)</fld> <def>A tribe of edentates comprising the sloths. They are noted for the slowness of their movements when on the ground. See <er>Sloth</er>, 3.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Zool.)</fld> <def>An order of minute aquatic arachnids; -- called also <altname>bear animalcules</altname>, <altname>sloth animalcules</altname>, and <altname>water bears</altname>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tardigrade</ent><br/
<hw>Tar"di*grade</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>tardigradus</ets>; <ets>tardus</ets> slow + <ets>gradi</ets> to step: cf. F. <ets>tardigrade</ets>.]</ety> <sn>1.</sn> <def>Moving or stepping slowly; slow-paced.</def> <mark>[R.]</mark>  <rj><au>G. Eliot.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Zool.)</fld> <def>Of or pertaining to the Tardigrada.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tardigrade</ent><br/
<hw>Tar"di*grade</hw>, <pos>n.</pos> <fld>(Zool.)</fld> <def>One of the Tardigrada.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tardigradous</ent><br/
<hw>Tar"di*gra`dous</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Moving slowly; slow-paced.</def> <mark>[Obs.]</mark>  <rj><au>Sir T. Browne.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tardily</ent><br/
<hw>Tar"di*ly</hw> <pr>(?)</pr>, <pos>adv.</pos> <def>In a tardy manner; slowly.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tardiness</ent><br/
<hw>Tar"di*ness</hw>, <pos>n.</pos> <def>The quality or state of being tardy.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tarditation</ent><br/
<hw>Tar`di*ta"tion</hw> <pr>(?)</pr>, <pos>n.</pos> <def>Tardiness.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>To instruct them to avoid all snares of <qex>tarditation</qex>, in the Lord's affairs.</q> <rj><qau>Herrick.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tardity</ent><br/
<hw>Tar"di*ty</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>tarditas</ets>.]</ety> <def>Slowness; tardiness.</def> <mark>[R.]</mark>  <rj><au>Sir K. Digby.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tardo</ent><br/
||<hw>Tar"do</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[It.]</ety> <fld>(Mus.)</fld> <def>Slow; -- a direction to perform a passage slowly.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tardo</ent><br/
||<hw>Tar"do</hw>, <pos>n.</pos> <ety>[Sp., slow, L. <ets>tardus</ets>.]</ety> <fld>(Zool.)</fld> <def>A sloth.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tardy</ent><br/
<hw>Tar"dy</hw> <pr>(?)</pr>, <pos>a.</pos> <amorph>[<pos>Compar.</pos> <adjf>Tardier</adjf> <pr>(?)</pr>; <pos>superl.</pos> <adjf>Tardiest</adjf>.]</amorph> <ety>[F. <ets>tardif</ets>, fr. (assumed) LL. <ets>tardivus</ets>, fr. L. <ets>tardus</ets> slow.]</ety> <sn>1.</sn> <def>Moving with a slow pace or motion; slow; not swift.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>And check the <qex>tardy</qex> flight of time.</q> <rj><qau>Sandys.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q><qex>Tardy</qex> to vengeance, and with mercy brave.</q> <rj><qau>Prior.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Not being inseason; late; dilatory; -- opposed to <xex>prompt</xex>; <as>as, to be <ex>tardy</ex> in one's payments</as>.</def>  <rj><au>Arbuthnot.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>The <qex>tardy</qex> plants in our cold orchards placed.</q> <rj><qau>Waller.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>Unwary; unready.</def> <mark>[Obs.]</mark>  <rj><au>Hudibras.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>Criminal; guilty.</def> <mark>[Obs.]</mark>  <rj><au>Collier.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><syn><b>Syn.</b> -- Slow; dilatory; tedious; reluctant. See <er>Slow</er>.</syn><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tardy</ent><br/
<hw>Tar"dy</hw>, <pos>v. t.</pos> <def>To make tardy.</def> <mark>[Obs.]</mark>  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tare</ent><br/
<hw>Tare</hw> <pr>(?)</pr>, <mark>obs.</mark> <pos>imp.</pos> <mord>of <er>Tear</er></mord>. <def>Tore.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tare</ent><br/
<hw>Tare</hw>, <pos>n.</pos> <ety>[Cf. Prov. E. <ets>tare</ets> brisk, eager, OE. <ets>tarefitch</ets> the wild vetch.]</ety> <sn>1.</sn> <def>A weed that grows among wheat and other grain; -- alleged by modern naturalists to be the <spn>Lolium temulentum</spn>, or darnel.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Didst not thou sow good seed in thy field? From whence then hath it <qex>tares</qex>?</q> <rj><qau>Matt. xiii. 27.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>The <ldquo/darnel<rdquo/ is said to be the <qex>tares</qex> of Scripture, and is the only deleterious species belonging to the whole order.</q> <rj><qau>Baird.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Bot.)</fld> <def>A name of several climbing or diffuse leguminous herbs of the genus <gen>Vicia</gen>; especially, the <spn>Vicia sativa</spn>, sometimes grown for fodder.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tare</ent><br/
<hw>Tare</hw>, <pos>n.</pos> <ety>[F. <ets>tare</ets>; cf. Pr., Sp., Pg., & It. <ets>tara</ets>; all fr. Ar. <ets>tarah</ets> thrown away, removed, fr. <ets>taraha</ets> to reject, remove.]</ety> <fld>(Com.)</fld> <def>Deficientcy in the weight or quantity of goods by reason of the weight of the cask, bag, or whatever contains the commodity, and is weighed with it; hence, the allowance or abatement of a certain weight or quantity which the seller makes to the buyer on account of the weight of such cask, bag, etc.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tare</ent><br/
<hw>Tare</hw>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Tared</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Taring</conjf>.]</vmorph> <def>To ascertain or mark the tare of (goods).</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tared</ent><br/
<hw>Tared</hw> <pr>(?)</pr>, <pos>a.</pos> <fld>(Chem.)</fld> <def>Weighed; determined; reduced to equal or standard weight; <as>as, <ex>tared</ex> filter papers, used in weighing precipitates</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tarente</ent><br/
<hw>Ta*ren"te</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Cf. F. <ets>tarente</ets>.]</ety> <fld>(Zool.)</fld> <def>A harmless lizard of the Gecko family (<spn>Platydactylus Mauritianicus</spn>) found in Southern Europe and adjacent countries, especially among old walls and ruins.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tarentism</ent><br/
<hw>Tar"ent*ism</hw> <pr>(?)</pr>, <pos>n.</pos> <def>See <er>Tarantism</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tarentula</ent><br/
<hw>Ta*ren"tu*la</hw> <pr>(?)</pr>, <pos>n.</pos> <def>See <er>Tarantula</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Targe</ent><br/
<hw>Targe</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F.  Cf. <er>Target</er>.]</ety> <def>A shield or target.</def> <mark>[Obs. or Poetic]</mark> <ldquo/A buckler on a <xex>targe</xex>.<rdquo/  <rj><au>Chaucer.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Target</ent><br/
<hw>Tar"get</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[OF. <ets>targette</ets>, dim. of OF. & F. <ets>targe</ets>, of Teutonic origin; cf. AS. <ets>targe</ets>, OD. <ets>targie</ets>, G. <ets>zarge</ets> a frame, case, border, OHG. <ets>zarga</ets>, Icel. <ets>targa</ets> shield.]</ety> <sn>1.</sn> <def>A kind of small shield or buckler, used as a defensive weapon in war.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <sd>(a)</sd> <def>A butt or mark to shoot at, as for practice, or to test the accuracy of a firearm, or the force of a projectile.</def> <sd>(b)</sd> <def>The pattern or arrangement of a series of hits made by a marksman on a butt or mark; <as>as, he made a good <ex>target</ex></as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <fld>(Surveying)</fld> <def>The sliding crosspiece, or vane, on a leveling staff.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <fld>(Railroad)</fld> <def>A conspicuous disk attached to a switch lever to show its position, or for use as a signal.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <def>A thin cut; a slice; specif., of lamb, a piece consisting of the neck and breast joints.</def> <mark>[Eng.]</mark><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><sn>6.</sn>  <def>A tassel or pendent; also, a shred; tatter.</def> <mark>[Obs. Scot.]</mark><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><sn>7.</sn>  <def>A goal for an activity; <as>as, the <ex>target</ex> of this year's fundraising drive is 2 million dollars</as>.</def><br/
[<source>PJC</source>]</p>

<p><sn>8.</sn>  <def>A metallic object toward which a beam of electrons is aimed in a tube designed to generate X-rays; when the electrons strike the <ex>target,</ex> the impact causes emission of X-rays.</def><br/
[<source>PJC</source>]</p>

<p><sn>9.</sn>  <def>Any object toward which a beam of photons, a laser beam, an electron beam, or a beam of atomic or subatomic particles is aimed.</def><br/
[<source>PJC</source>]</p>

<p><sn>10.</sn>  <def>A person who is the subject of criticism or ridicule.</def><br/
[<source>PJC</source>]</p>

<p><ent>Target date</ent><br/
<hw>Tar"get date</hw> <pr>(?)</pr>, <pos>n.</pos> <def>The date set as a goal for completion of some activity.</def><br/
[<source>PJC</source>]</p>

<p><ent>Target language</ent><br/
<hw>Tar"get language</hw> <pr>(?)</pr>, <pos>n.</pos> <def>the language into which a text is to be translated; -- correlative of <inv>source language</inv>.</def><br/
[<source>PJC</source>]</p>

<p><-- p. 1475 --><br/
[<source>1913 Webster</source>]</p>

<p><ent>Targeted</ent><br/
<hw>Tar"get*ed</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Furnished, armed, or protected, with a target.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Targeteer</ent><br/
<hw>Tar`get*eer"</hw> <pr>(?)</pr>, <pos>n.</pos> <def>One who is armed with a target or shield.</def> <altsp>[Written also <asp>targetier</asp>.]</altsp><br/
[<source>1913 Webster</source>]</p>

<p><ent>Targum</ent><br/
<hw>Tar"gum</hw> <pr>(?)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Targums</plw> <pr>(#)</pr>. Heb. <plw>Targumim</plw> <pr>(#)</pr>.</plu> <ety>[Chald. <ets>targ<umac/m</ets> interpretation, fr. <ets>targ<emac/m</ets> to interpret.  Cf. <er>Truchman</er>, and <er>Dragoman</er>.]</ety> <def>A translation or paraphrase of some portion of the Old Testament Scriptures in the Chaldee or Aramaic language or dialect.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Targumist</ent><br/
<hw>Tar"gum*ist</hw>, <pos>n.</pos> <def>The writer of a Targum; one versed in the Targums.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tarheel</ent><br/
<hw>Tar"heel</hw> <pr>(?)</pr>, <pos>n.</pos> <def>a native or resident of North Carolina; -- used as a nickname.</def><br/
[<source>PJC</source>]</p>

<p><ent>Tarheel state</ent><br/
<hw>Tar"heel state</hw> <pr>(?)</pr>, <pos>n.</pos> <def>North Carolina; -- used as a nickname.</def><br/
[<source>PJC</source>]</p>

<p><ent>Tariff</ent><br/
<hw>Tar"iff</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F. <ets>tarif</ets>; cf. Sp. & Pg. <ets>tarifa</ets>, It. <ets>tariffa</ets>; all fr. Ar. <ets>ta'r<imac/f</ets> information, explanation, definition, from <ets>'arafa</ets>, to know, to inform, explain.]</ety> <sn>1.</sn> <def>A schedule, system, or scheme of duties imposed by the government of a country upon goods imported or exported; <as>as, a revenue <ex>tariff</ex>; a protective <ex>tariff</ex>; Clay's compromise <ex>tariff</ex>. (U. S. 1833).</as></def><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ The United States and Great Britain impose no duties on exports; hence, in these countries the <ex>tariff</ex> refers only to imports.</note><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ A tariff may be imposed solely for, and with reference to, the production of revenue (called a <col><b>revenue tariff</b></col>, or <col><b>tariff for revenue</b></col>, or for the artificial fostering of home industries (<col><b>a projective tariff</b></col>), or as a means of coercing foreign governments, as in case of <col><b>retaliatory tariff</b></col>.</note><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><sn>2.</sn> <def>The duty, or rate of duty, so imposed; <as>as, the <ex>tariff</ex> on wool; a <ex>tariff</ex> of two cents a pound.</as></def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>Any schedule or system of rates, changes, etc.; <as>as, a <ex>tariff</ex> of fees, or of railroad fares</as>.</def>  <rj><au>Bolingbroke.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tariff</ent><br/
<hw>Tar"iff</hw>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Tariffed</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Tariffing</conjf>.]</vmorph> <def>To make a list of duties on, as goods.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tarin</ent><br/
<hw>Tar"in</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F.]</ety> <fld>(Zool.)</fld> <def>The siskin.</def> <mark>[Prov.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Taring</ent><br/
<hw>Tar"ing</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Zool.)</fld> <def>The common tern; -- called also <altname>tarret</altname>, and <altname>tarrock</altname>.</def> <mark>[Prov. Eng.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tarlatan</ent><br/
<hw>Tar"la*tan</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A kind of thin, transparent muslin, used for dresses.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tarn</ent><br/
<hw>Tarn</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[OE. <ets>terne</ets>, Icel. <ets>tj<oum/rn</ets>.]</ety> <def>A mountain lake or pool.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>A lofty precipice in front,<br/
A silent <qex>tarn</qex> below.</q> <rj><qau>Wordsworth.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tarnish</ent><br/
<hw>Tar"nish</hw> <pr>(?)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Tarnished</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Tarnishing</conjf>.]</vmorph> <ety>[F. <ets>ternir</ets>, fr. OHG. <ets>tarnen</ets> to darken, to conceal, hide; akin to OS. <ets>dernian</ets> to hide, AS. <ets>dernan</ets>, <ets>dyrnan</ets>, OHG. <ets>tarni</ets> hidden, OS. <ets>derni</ets>, AS. <ets>derne</ets>, <ets>dyrne</ets>.  Cf. <er>Dern</er>, <pos>a.</pos>, and see <er>-ish</er>.]</ety> <def>To soil, or change the appearance of, especially by an alternation induced by the air, or by dust, or the like; to diminish, dull, or destroy the luster of; to sully; <as>as, to <ex>tarnish</ex> a metal; to <ex>tarnish</ex> gilding; to <ex>tarnish</ex> the purity of color</as>.</def> <ldquo/<xex>Tarnished</xex> lace.<rdquo/ <au>Fuller.</au> Used also figuratively; <as>as, to <ex>tarnish</ex> one's honor</as>.<br/
[<source>1913 Webster</source>]</p>

<p><syn><b>Syn.</b> -- To sully; stain; dim.</syn><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tarnish</ent><br/
<hw>Tar"nish</hw>, <pos>v. i.</pos> <def>To lose luster; to become dull; <as>as, gilding will <ex>tarnish</ex> in a foul air</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Till thy fresh glories, which now shine so bright,<br/
Grow stale and <qex>tarnish</qex> with our daily sight.</q> <rj><qau>Dryden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tarnish</ent><br/
<hw>Tar"nish</hw>, <pos>n.</pos> <sn>1.</sn> <def>The quality or state of being tarnished; stain; soil; blemish.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Min.)</fld> <def>A thin film on the surface of a metal, usually due to a slight alteration of the original color; <as>as, the steel <ex>tarnish</ex> in columbite</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tarnisher</ent><br/
<hw>Tar"nish*er</hw> <pr>(?)</pr>, <pos>n.</pos> <def>One who, or that which, tarnishes.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Taro</ent><br/
<hw>Ta"ro</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[From the Polynesian name.]</ety> <fld>(Bot.)</fld> <def>A name for several aroid plants (<spn>Colocasia antiquorum</spn>, var. <varn>esculenta</varn>, <spn>Colocasia macrorhiza</spn>, etc.), and their rootstocks. They have large ovate-sagittate leaves and large fleshy tuberous rootstocks, which are cooked and used for food in tropical countries.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tarot</ent><br/
<hw>Tar"ot</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F.; cf. It. <ets>tarocco</ets>.]</ety> <sn>1.</sn> <def>A game of cards; -- called also <altname>taroc</altname>.</def>  <rj><au>Hoyle.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>any of a set of 22 playing cards which bear allegorical images representing various objects or influences affecting human life, and widely used in fortunetelling; they are also used as trumps in the game of  taroc.  Various images are used by different artists to represent the themes of each card.</def><br/
[<source>PJC</source>]</p>

<p><ent>Tarpan</ent><br/
<hw>Tar"pan</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[From the native name.]</ety> <fld>(Zool.)</fld> <def>A wild horse found in the region of the Caspian Sea.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tarpaulin</ent><br/
<hw>Tar*pau"lin</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[<ets>Tar</ets> + <ets>palling</ets> a covering, pall to cover. See <er>Pall</er> a covering.]</ety> <sn>1.</sn> <def>A piece of canvas covered with tar or a waterproof composition, used for covering the hatches of a ship, hammocks, boats, etc.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A hat made of, or covered with, painted or tarred cloth, worn by sailors and others.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>Hence, a sailor; a seaman; a tar.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>To a landsman, these <qex>tarpaulins</qex>, as they were called, seemed a strange and half-savage race.</q> <rj><qau>Macaulay.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tarpeian</ent><br/
<hw>Tar*pe"ian</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>Tarpeius</ets>, prop., pertaining to <etsep>Tarpeia</etsep>.]</ety> <def>Pertaining to or designating a rock or peak of the Capitoline hill, Rome, from which condemned criminals were hurled.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Tarpon</ent><br/
<hw>Tar"pon</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Zool.)</fld> <def>Same as <er>Tarpum</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tarpum</ent><br/
<hw>Tar"pum</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Zool.)</fld> <def>A very large marine fish (<spn>Megapolis Atlanticus</spn>) of the Southern United States and the West Indies. It often becomes six or more feet in length, and has large silvery scales. The scales are a staple article of trade, and are used in fancywork. Called also <altname>tarpon</altname>, <altname>sabalo</altname>, <altname>savanilla</altname>, <altname>silverfish</altname>, and <altname>jewfish</altname>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tarquinish</ent><br/
<hw>Tar"quin*ish</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Like a Tarquin, a king of ancient Rome; proud; haughty; overbearing.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tarrace</ent><br/
<hw>Tar"race</hw> <pr>(?)</pr>, <pos>n.</pos> <def>See <er>Trass</er>.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tarragon</ent><br/
<hw>Tar"ra*gon</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Sp. <ets>taragona</ets>, Ar. <ets>tarkh<?/n</ets>; perhaps fr. Gr. <?/ a dragon, or L. <ets>draco</ets>; cf. L. <ets>dracunculus</ets> tarragon.  Cf. <er>Dragon</er>.]</ety> <fld>(Bot.)</fld> <def>A plant of the genus <gen>Artemisa</gen> (<spn>Artemisa dracunculus</spn>), much used in France for flavoring vinegar.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tarras</ent><br/
<hw>Tar"ras</hw> <pr>(?)</pr>, <pos>n.</pos> <def>See <er>Trass</er>.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tarre</ent><br/
<hw>Tarre</hw> <pr>(?)</pr>, <pos>v. t.</pos> <ety>[OE. <ets>tarien</ets>, <ets>terien</ets>, to irritate, provoke, AS. <ets>tergan</ets> to pull, pluck, torment; probably akin to E. <ets>tear</ets>, v. t.  <root/63.  Cf. <er>Tarry</er>, <pos>v.</pos>]</ety> <def>To set on, as a dog; to incite.</def> <mark>[Obs.]</mark>  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tarriance</ent><br/
<hw>Tar"ri*ance</hw> <pr>(?)</pr>, <pos>n.</pos> <def>The act or time of tarrying; delay; lateness.</def> <mark>[Archaic]</mark>  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>And after two days' <qex>tarriance</qex> there, returned.</q> <rj><qau>Tennyson.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tarrier</ent><br/
<hw>Tar"ri*er</hw> <pr>(?)</pr>, <pos>n.</pos> <def>One who, or that which, tarries.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tarrier</ent><br/
<hw>Tar"ri*er</hw>, <pos>n.</pos> <fld>(Zool.)</fld> <def>A kind of dig; a terrier.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tarrock</ent><br/
<hw>Tar"rock</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Greenland <ets>tattarock</ets>.]</ety> <fld>(Zool.)</fld> <sd>(a)</sd> <def>The young of the kittiwake gull before the first molt.</def> <sd>(b)</sd> <def>The common guillemot.</def> <mark>[Prov. Eng.]</mark> <sd>(c)</sd> <def>The common tern.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tarry</ent><br/
<hw>Tar"ry</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[From <er>Tar</er>, <pos>n.</pos>]</ety> <def>Consisting of, or covered with, tar; like tar.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tarry</ent><br/
<hw>Tar"ry</hw> <pr>(?)</pr>, <pos>v. i.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Tarried</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Tarrying</conjf>.]</vmorph> <ety>[OE. <ets>tarien</ets> to irritate (see <er>Tarre</er>); but with a change of sense probably due to confusion with OE. <ets>targen</ets> to delay, OF. <ets>targier</ets>, fr. (assumed) LL. <ets>tardicare</ets>, fr. L. <ets>tardare</ets> to make slow, to tarry, fr. <ets>tardus</ets> slow.  Cf. <er>Tardy</er>.]</ety> <sn>1.</sn> <def>To stay or remain behind; to wait.</def><br/
[<source>1913 Webster</source>]</p>

<p><q><qex>Tarry</qex> ye for us, until we come again.</q> <rj><qau>Ex. xxiv. 14.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To delay; to put off going or coming; to loiter.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Come down unto me, <qex>tarry</qex> not.</q> <rj><qau>Gen. xic. 9.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>One <qex>tarried</qex> here, there hurried one.</q> <rj><qau>Emerson.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>To stay; to abide; to continue; to lodge.</def><br/
[<source>1913 Webster</source>]</p>

<p><q><qex>Tarry</qex> all night, and wash your feet.</q> <rj><qau>Gen. xix. 2.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><syn><b>Syn.</b> -- To abide; continue; lodge; await; loiter.</syn><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tarry</ent><br/
<hw>Tar"ry</hw>, <pos>v. t.</pos> <sn>1.</sn> <def>To delay; to defer; to put off.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q><qex>Tarry</qex> us here no longer than to-morrow.</q> <rj><qau>Chaucer.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To wait for; to stay or stop for.</def> <mark>[Archaic]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>He that will have a cake out of the wheat must needs <qex>tarry</qex> the grinding.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>He plodded on, . . . <qex>tarrying</qex> no further question.</q> <rj><qau>Sir W. Scott.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tarry</ent><br/
<hw>Tar"ry</hw>, <pos>n.</pos> <def>Stay; stop; delay.</def> <mark>[Obs.]</mark>  <rj><au>E. Lodge.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tarsal</ent><br/
<hw>Tar"sal</hw> <pr>(?)</pr>, <pos>a.</pos> <fld>(Anat.)</fld> <def>Of or pertaining to the tarsus (either of the foot or eye).</def> -- <def2><pos>n.</pos> <def>A tarsal bone or cartilage; a tarsale.</def></def2><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Tarsal tetter</b></col> <fld>(Med.)</fld>, <cd>an eruptive disease of the edges of the eyelids; a kind of bleareye.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tarsal</ent><br/
<hw>Tar"sal</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Zool.)</fld> <def>Same as <er>Tercel</er>.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tarsale</ent><br/
||<hw>Tar*sa"le</hw> <pr>(?)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Tarsalia</plw> <pr>(#)</pr>.</plu> <ety>[NL.]</ety> <fld>(Anat.)</fld> <def>One of the bones or cartilages of the tarsus; esp., one of the series articulating with the metatarsals.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tarse</ent><br/
<hw>Tarse</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Cf. <er>Tassel</er>, <er>Tiercel</er>.]</ety> <fld>(Falconry)</fld> <def>The male falcon.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tarse</ent><br/
<hw>Tarse</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Cf. F. <ets>tarse</ets>.]</ety> <fld>(Anat.)</fld> <def>tarsus.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tarsectomy</ent><br/
<hw>Tar*sec"to*my</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[<ets>Tarsus</ets> + Gr. <?/ to cut out.]</ety> <fld>(Surg.)</fld> <def>The operation of excising one or more of the bones of the tarsus.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tarsel</ent><br/
<hw>Tar"sel</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A male hawk. See <er>Tercel</er>.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tarsi</ent><br/
||<hw>Tar"si</hw> <pr>(?)</pr>, <pos>n.</pos>, <def><pos>pl.</pos> of <er>Tarsus</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tarsiatura</ent><br/
<ent>Tarsia</ent><br/
<mhw>{ ||<hw>Tar"si*a</hw> <pr>(?)</pr>, ||<hw>Tar`si*a*tu"ra</hw> <pr>(?)</pr>, }</mhw> <pos>n.</pos> <ety>[It.]</ety> <def>A kind of mosaic in woodwork, much employed in Italy in the fifteenth century and later, in which scrolls and arabesques, and sometimes architectural scenes, landscapes, fruits, flowers, and the like, were produced by inlaying pieces of wood of different colors and shades into panels usually of walnut wood.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tarsier</ent><br/
<hw>Tar"si*er</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Cf. F. <ets>tarsier</ets>.]</ety> <def>See <er>Tarsius</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tarsius</ent><br/
||<hw>Tar"si*us</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[NL. See <er>Tarsus</er>.]</ety> <fld>(Zool.)</fld> <def>A genus of nocturnal lemurine mammals having very large eyes and ears, a long tail, and very long proximal tarsal bones; -- called also <altname>malmag</altname>, <altname>spectral lemur</altname>, <altname>podji</altname>, and <altname>tarsier</altname>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tarso-</ent><br/
<hw>Tar"so-</hw> <pr>(?)</pr>. <def>A combining form used in anatomy to indicate <xex>connection with</xex>, or <xex>relation to</xex>, <xex>the tarsus</xex>; <as>as, <ex>tarso</ex>metatarsus</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tarsometatarsal</ent><br/
<hw>Tar`so*met`a*tar"sal</hw> <pr>(?)</pr>, <pos>a.</pos> <fld>(Anat.)</fld> <sd>(a)</sd> <def>Of or pertaining to both the tarsus and metatarsus; <as>as, the <ex>tarsometatarsal</ex> articulations</as>.</def> <sd>(b)</sd> <def>Of or pertaining to the tarsometatarsus.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tarsometatarsus</ent><br/
||<hw>Tar`so*met`a*tar"sus</hw> <pr>(?)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Tarsometatarsi</plw> <pr>(#)</pr>.</plu> <ety>[NL.]</ety> <fld>(Anat.)</fld> <def>The large bone next the foot in the leg of a bird. It is formed by the union of the distal part of the tarsus with the metatarsus.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tarsorrhaphy</ent><br/
<hw>Tar*sor"rha*phy</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[<ets>Tarsus</ets> + Gr. <grk>"rafh`</grk> seam, fr. <?/ to sew.]</ety> <fld>(Surg.)</fld> <def>An operation to diminish the size of the opening between eyelids when enlarged by surrounding cicatrices.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tarsotomy</ent><br/
<hw>Tar*sot"o*my</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[<ets>Tarsus</ets> + Gr. <?/ to cut.]</ety> <fld>(Surg.)</fld> <def>The operation of cutting or removing the tarsal cartilages.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tarsus</ent><br/
<hw>Tar"sus</hw> <pr>(?)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Tarsi</plw> <pr>(#)</pr>.</plu> <ety>[NL., fr. Gr. <?/ the flat of the foot, the edge of the eyelid.  Cf. 2d <er>Tarse</er>.]</ety> <sn>1.</sn> <fld>(Anat.)</fld> <sd>(a)</sd> <def>The ankle; the bones or cartilages of the part of the foot between the metatarsus and the leg, consisting in man of seven short bones.</def> <sd>(b)</sd> <def>A plate of dense connective tissue or cartilage in the eyelid of man and many animals; -- called also <altname>tarsal cartilage</altname>, and <altname>tarsal plate</altname>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Zool.)</fld> <def>The foot of an insect or a crustacean. It usually consists of form two to five joints.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tart</ent><br/
<hw>Tart</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[AS. <ets>teart</ets>. <root/63.  Cf. <er>Tear</er>, <pos>v. t.</pos>]</ety> <sn>1.</sn> <def>Sharp to the taste; acid; sour; <as>as, a <ex>tart</ex> apple</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Fig.: Sharp; keen; severe; <as>as, a <ex>tart</ex> reply; <ex>tart</ex> language; a <ex>tart</ex> rebuke.</as></def><br/
[<source>1913 Webster</source>]</p>

<p><q>Why art thou <qex>tart</qex>, my brother?</q> <rj><qau>Bunyan.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tart</ent><br/
<hw>Tart</hw>, <pos>n.</pos> <ety>[OE. <ets>tarte</ets>, F. <ets>tarte</ets>; perhaps originally the same word as <ets>tourte</ets>, LL. <ets>torta</ets>, fr. L. <ets>tortus</ets>, p. p. of <ets>torquere</ets> to twist, bend, wind, because tarts were originally made of a twisted shape.  Cf. <er>Torture</er>, <pos>n.</pos>]</ety> <def>A species of small open pie, or piece of pastry, containing jelly or conserve; a sort of fruit pie.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tartan</ent><br/
<hw>Tar"tan</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F. <ets>tiretane</ets> linsey-woolsey, akin to Sp. <ets>tirita<ntil/a</ets> a sort of thin silk; cf. Sp. <ets>tiritar</ets> to shiver or shake with cold.]</ety> <def>Woolen cloth, checkered or crossbarred with narrow bands of various colors, much worn in the Highlands of Scotland; hence, any pattern of tartan; also, other material of a similar pattern.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>MacCullummore's heart will be as cold as death can make it, when it does not warm to the <qex>tartan</qex>.</q> <rj><qau>Sir W. Scott.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>The sight of the <qex>tartan</qex> inflamed the populace of London with hatred.</q> <rj><qau>Macaulay.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tartan</ent><br/
<hw>Tar"tan</hw>, <pos>n.</pos> <ety>[F. <ets>tartane</ets>, or Sp., Pg., or It. <ets>tartana</ets>; all perhaps of Arabic origin.]</ety> <fld>(Naut.)</fld> <def>A small coasting vessel, used in the Mediterranean, having one mast carrying large leteen sail, and a bowsprit with staysail or jib.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tartar</ent><br/
<hw>Tar"tar</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F. <ets>tartre</ets> (cf. Pr. <ets>tartari</ets>, Sp., Pg., & It. <ets>tartaro</ets>, LL. <ets>tartarum</ets>, LGr. <?/); perhaps of Arabic origin.]</ety> <sn>1.</sn> <fld>(Chem.)</fld> <def>A reddish crust or sediment in wine casks, consisting essentially of crude cream of tartar, and used in marking pure cream of tartar, tartaric acid, potassium carbonate, black flux, etc., and, in dyeing, as a mordant for woolen goods; -- called also <altname>argol</altname>, <altname>wine stone</altname>, etc.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A correction which often incrusts the teeth, consisting of salivary mucus, animal matter, and phosphate of lime.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Cream of tartar</b></col>. <fld>(Chem.)</fld> <cd>See under <er>Cream</er>.</cd> -- <col><b>Tartar emetic</b></col> <fld>(Med. Chem.)</fld>, <cd>a double tartrate of potassium and basic antimony. It is a poisonous white crystalline substance having a sweetish metallic taste, and used in medicine as a sudorific and emetic.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tartar</ent><br/
<hw>Tar"tar</hw> <pr>(?)</pr>, <pos>n.</pos> <sn>1.</sn> <ety>[Per. <ets>T<amac/t<amac/r</ets>, of Tartar origin.]</ety> <def>A native or inhabitant of Tartary in Asia; a member of any one of numerous tribes, chiefly Moslem, of Turkish origin, inhabiting the Russian Europe; -- written also, more correctly but less usually, <altname>Tatar</altname>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A person of a keen, irritable temper.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>To catch a tartar</b></col>, <cd>to lay hold of, or encounter, a person who proves too strong for the assailant.</cd> <mark>[Colloq.]</mark></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tartar</ent><br/
<hw>Tar"tar</hw>, <pos>a.</pos> <def>Of or pertaining to Tartary in Asia, or the Tartars.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tartar</ent><br/
<hw>Tar"tar</hw>, <pos>n.</pos> <ety>[Cf. F. <ets>tartare</ets>.]</ety> <def>See <er>Tartarus</er>.</def>  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tartarated</ent><br/
<hw>Tar"tar*a`ted</hw> <pr>(?)</pr>, <pos>a.</pos> <fld>(Chem.)</fld> <def>Tartrated.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tartareous</ent><br/
<ent>Tartarean</ent><br/
<mhw>{ <hw>Tar*ta"re*an</hw> <pr>(?)</pr>, <hw>Tar*ta"re*ous</hw> <pr>(?)</pr>, }</mhw> <pos>a.</pos> <ety>[L. <ets>tartareus</ets>: cf. F. <ets>tartar<eacute/en</ets>.]</ety> <def>Of or pertaining to Tartarus; hellish.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tartareous</ent><br/
<hw>Tar*ta"re*ous</hw>, <pos>a.</pos> <ety>[Cf. 1st <er>Tartarous</er>.]</ety> <sn>1.</sn> <def>Consisting of tartar; of the nature of tartar.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Bot.)</fld> <def>Having the surface rough and crumbling; <as>as, many lichens are <ex>tartareous</ex></as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tartaric</ent><br/
<ent>Tartarian</ent><br/
<mhw>{ <hw>Tar*ta"ri*an</hw> <pr>(?)</pr>, <hw>Tar*tar"ic</hw> <pr>(?)</pr>, }</mhw> <pos>a.</pos> <def>Of or pertaining to Tartary in Asia, or the Tartars.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Tartarian lamb</b></col> <fld>(Bot.)</fld>, <cd>Scythian lamb. See <er>Barometz</er>.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tartarian</ent><br/
<hw>Tar*ta"ri*an</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Bot.)</fld> <def>The name of some kinds of cherries, <as>as the Black <ex>Tartarian</ex>, or the White <ex>Tartarian</ex></as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tartaric</ent><br/
<hw>Tar*tar"ic</hw> <pr>(?)</pr>, <pos>a.</pos> <fld>(Chem.)</fld> <def>Of or pertaining to tartar; derived from, or resembling, tartar.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Tartaric acid</b></col>. <fld>(a)</fld> <cd>An acid widely diffused throughout the vegetable kingdom, as in grapes, mountain-ash berries, etc., and obtained from tartar as a white crystalline substance, <chform>C2H2(OH)2.(CO2H)2</chform>, having a strong pure acid taste. It is used in medicine, in dyeing, calico printing, photography, etc., and also as a substitute for lemon juice. Called also <altname>dextro-tartaric acid</altname>.</cd> <sd>(b)</sd> <cd>By extension, any one of the series of isomeric acids (racemic acid, levotartaric acid, inactive tartaric acid) of which tartaric acid proper is the type.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tartarine</ent><br/
<hw>Tar"tar*ine</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Old Chem.)</fld> <def>Potassium carbonate, obtained by the incineration of tartar.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tartarize</ent><br/
<hw>Tar"tar*ize</hw> <pr>(?)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Tartarized</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Tartarizing</conjf> <pr>(?)</pr>.]</vmorph> <ety>[Cf. F. <ets>tartariser</ets>.]</ety> <fld>(Chem.)</fld> <def>To impregnate with, or subject to the action of, tartar.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Tartarized antimony</b></col> <fld>(Med. Chem.)</fld>, <cd>tartar emetic.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tartarize</ent><br/
<hw>Tar"tar*ize</hw> <pr>(?)</pr>, <pos>v. t.</pos> <def>To cause to resemble the Tartars and their civilization, as by conquest.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tartarous</ent><br/
<hw>Tar"tar*ous</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[Cf. F. <ets>tartareux</ets>.]</ety> <def>Containing tartar; consisting of tartar, or partaking of its qualities; tartareous.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tartarous</ent><br/
<hw>Tar"tar*ous</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Resembling, or characteristic of, a Tartar; ill-natured; irritable.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The <qex>Tartarous</qex> moods of common men.</q> <rj><qau>B. Jonson.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tartarum</ent><br/
<hw>Tar"ta*rum</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Chem.)</fld> <def>See 1st <er>Tartar</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tartarus</ent><br/
<hw>Tar"ta*rus</hw> <pr>(t<aum/r"t<adot/*r<ucr/s)</pr>, <pos>prop. n.</pos> <ety>[L., from Gr. <grk>Ta`rtaros</grk>.]</ety> <fld>(Class. Myth.)</fld> <def>The infernal regions, described in the Iliad as situated as far below Hades as heaven is above the earth, and by later writers as the place of punishment for the spirits of the wicked. By the later poets, also, the name is often used synonymously with <altname>Hades</altname>, or the Lower World in general.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tartary</ent><br/
<hw>Tar"ta*ry</hw> <pr>(?)</pr>, <pos>n.</pos> <def>Tartarus.</def> <mark>[Obs.]</mark>  <rj><au>Spenser.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><-- p. 1476 --><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tartini's tones</ent><br/
<hw>Tar*ti"ni's tones`</hw> <pr>(?)</pr>. <ety>[From <ets>Tartini</ets>, an Italian violinist, who discovered them in 1754.]</ety> <def>See the Note under <er>Tone</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tartish</ent><br/
<hw>Tart"ish</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Somewhat tart.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tartlet</ent><br/
<hw>Tart"let</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A small tart.</def>  <rj><au>V. Knox.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tartly</ent><br/
<hw>Tart"ly</hw>, <pos>adv.</pos> <def>In a tart manner; with acidity.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tartness</ent><br/
<hw>Tart"ness</hw>, <pos>n.</pos> <def>The quality or state of being tart.</def><br/
[<source>1913 Webster</source>]</p>

<p><syn><b>Syn.</b> -- Acrimony; sourness; keenness; poignancy; severity; asperity; acerbity; harshness. See <er>Acrimony</er>.</syn><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tartralic</ent><br/
<hw>Tar*tral"ic</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[From <er>Tartar</er> the chemical compound.]</ety> <fld>(Chem.)</fld> <def>Pertaining to, or designating, an acid obtained as a white amorphous deliquescent substance, <chform>C8H10O11</chform>; -- called also <altname>ditartaric</altname>, <altname>tartrilic</altname>, or <altname>tartrylic acid</altname>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tartramate</ent><br/
<hw>Tar*tram"ate</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Chem.)</fld> <def>A salt of tartramic acid.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tartramic</ent><br/
<hw>Tar*tram"ic</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[<ets>Tarto-</ets> + <ets>amic</ets>.]</ety> <fld>(Chem.)</fld> <def>Of, pertaining to, or designating, an acid which is the primary acid amide derivative of tartaric acid.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tartramide</ent><br/
<hw>Tar*tram"ide</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[<ets>Tarto-</ets> + <ets>amide</ets>.]</ety> <fld>(Chem.)</fld> <def>An acid amide derivative of tartaric acid, obtained as a white crystalline substance.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tartrate</ent><br/
<hw>Tar"trate</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Cf. F. <ets>tartrate</ets>.]</ety> <fld>(Chem.)</fld> <def>A salt of tartaric acid.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tartrated</ent><br/
<hw>Tar"tra`ted</hw> <pr>(?)</pr>, <pos>a.</pos> <fld>(Med. Chem.)</fld> <def>Containing, or derived from, tartar; combined with tartaric acid.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tartrazine</ent><br/
<hw>Tar"tra*zine</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[<ets>Tart</ets>aric + hyd<ets>razine</ets>.]</ety> <fld>(Chem.)</fld> <def>An artificial dyestuff obtained as an orange-yellow powder, and regarded as a phenyl hydrazine derivative of tartaric and sulphonic acids.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tartrelic</ent><br/
<hw>Tar*trel"ic</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[From <er>Tartar</er> the chemical compound.]</ety> <fld>(Chem.)</fld> <def>Of, pertaining to, or designating, an anhydride, <chform>C4H4O5</chform>, of tartaric acid, obtained as a white crystalline deliquescent substance.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tartro-</ent><br/
<hw>Tar"tro-</hw>. <def>A combining form (also used adjectively) used in chemistry to denote <xex>the presence of tartar</xex> or <xex>of some of its compounds</xex> or <xex>derivatives</xex>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tartronate</ent><br/
<hw>Tar"tro*nate</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Chem.)</fld> <def>A salt of tartronic acid.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tartronic</ent><br/
<hw>Tar*tron"ic</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[<ets>Tartro-</ets> + mal<ets>onic</ets>.]</ety> <fld>(Chem.)</fld> <def>Of, pertaining to, or designating, an organic acid (called also <xex>hydroxy malonic acid</xex>) obtained, by reducing mesoxalic acid, as a white crystalline substance.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tartronyl</ent><br/
<hw>Tar"tro*nyl</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[<ets>Tartron</ets>ic + <ets>-yl</ets>.]</ety> <fld>(Chem.)</fld> <def>A hypothetical radical constituting the characteristic residue of tartronic acid and certain of its derivatives.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tartrovinic</ent><br/
<hw>Tar`tro*vin"ic</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[<ets>Tartro-</ets> + <ets>vinic</ets>.]</ety> <fld>(Chem.)</fld> <def>Of, pertaining to, or designating, a certain acid composed of tartaric acid in combination with ethyl, and now called <xex>ethyltartaric acid</xex>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tartufe</ent><br/
<ent>Tartuffe</ent><br/
<mhw>{ <hw>Tar*tuffe"</hw>, <hw>Tar*tufe"</hw>  }</mhw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F. <ets>tartufe</ets>.]</ety> <def>A hypocritical devotee. See the Dictionary of Noted Names in Fiction.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tartufish</ent><br/
<ent>Tartuffish</ent><br/
<mhw>{ <hw>Tar*tuff"ish</hw>, <hw>Tar*tuf"ish</hw>, }</mhw> <pos>a.</pos> <def>Like a tartuffe; precise; hypocritical.</def>  <rj><au>Sterne.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tarweed</ent><br/
<hw>Tar"weed`</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Bot.)</fld> <def>A name given to several resinous-glandular composite plants of California, esp. to the species of <gen>Grindelia</gen>, <gen>Hemizonia</gen>, and <gen>Madia</gen>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tas</ent><br/
<hw>Tas</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F.]</ety> <def>A heap.</def> <mark>[Obs.]</mark> <ldquo/The <xex>tas</xex> of bodies slain.<rdquo/  <rj><au>Chaucer.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tas</ent><br/
<hw>Tas</hw>, <pos>v. t.</pos> <def>To tassel.</def> <mark>[Obs.]</mark> <ldquo/A purse of leather <xex>tassed</xex> with silk.<rdquo/  <rj><au>Chaucer.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tasco</ent><br/
<hw>Tas"co</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Cf. Sp. <ets>tasconio</ets>.]</ety> <def>A kind of clay for making melting pots.</def>  <rj><au>Percy Smith.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tasimer</ent><br/
<hw>Ta*sim"er</hw> <pr>(t<adot/*s<icr/m"<esl/*t<etil/r)</pr>, <pos>n.</pos> <ety>[Gr. <grk>ta`sis</grk> stretching, extension (from <grk>tei`nein</grk> to stretch) + <ets>-meter</ets>.]</ety> <fld>(Physics)</fld> <def>An instrument for detecting or measuring minute extensions or movements of solid bodies. It consists essentially of a small rod, disk, or button of carbon, forming part of an electrical circuit, the resistance of which, being varied by the changes of pressure produced by the movements of the object to be measured, causes variations in the strength of the current, which variations are indicated by a sensitive galvanometer. It is also used for measuring minute changes of temperature.</def>  <rj><au>T. A. Edison.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Task</ent><br/
<hw>Task</hw> <pr>(t<adot/sk)</pr>, <pos>n.</pos> <ety>[OE. <ets>taske</ets>, OF. <ets>tasque</ets>, F. <ets>t<acir/che</ets>, for <ets>tasche</ets>, LL. <ets>tasca</ets>, <ets>taxa</ets>, fr. L. <ets>taxare</ets> to rate, appraise, estimate. See <er>Tax</er>, <pos>n.</pos> & <pos>v.</pos>]</ety> <sn>1.</sn> <def>Labor or study imposed by another, often in a definite quantity or amount.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Ma <qex>task</qex> of servile toil.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Each morning sees some <qex>task</qex> begin,<br/
Each evening sees it close.</q> <rj><qau>Longfellow.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Business; employment; undertaking; labor.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>His mental powers were equal to greater <qex>tasks</qex>.</q> <rj><qau>Atterbury.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>To take to task</b></col>. <cd>See under <er>Take</er>.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><syn><b>Syn.</b> -- Work; labor; employment; business; toil; drudgery; study; lesson; stint.</syn><br/
[<source>1913 Webster</source>]</p>

<p><ent>Task</ent><br/
<hw>Task</hw>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Tasked</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Tasking</conjf>.]</vmorph> <sn>1.</sn> <def>To impose a task upon; to assign a definite amount of business, labor, or duty to.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>There <qex>task</qex> thy maids, and exercise the loom.</q> <rj><qau>Dryden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To oppress with severe or excessive burdens; to tax.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>To charge; to tax, as with a fault.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Too impudent to <qex>task</qex> me with those errors.</q> <rj><qau>Beau. & Fl.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tasker</ent><br/
<hw>Task"er</hw> <pr>(?)</pr>, <pos>n.</pos> <sn>1.</sn> <def>One who imposes a task.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>One who performs a task, as a day-laborer.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>A laborer who receives his wages in kind.</def> <mark>[Scot.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Taskmaster</ent><br/
<hw>Task"mas`ter</hw> <pr>(?)</pr>, <pos>n.</pos> <def>One who imposes a task, or burdens another with labor; one whose duty is to assign tasks; an overseer.</def>  <rj><au>Ex. i. 11.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>All is, if I have grace to use it so,<br/
As ever in my great <qex>Taskmaster's</qex> eye.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Task wage</ent><br/
<hw>Task wage</hw>. <fld>(Polit. Econ.)</fld> <def>A wage paid by the day, or some fixed period, on condition that a minimum task be performed. When the workman is paid in proportion for excess over the minimum, the wage is one for piece-work.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Taskwork</ent><br/
<hw>Task"work`</hw> <pr>(?)</pr>, <pos>n.</pos> <def>Work done as a task; also, work done by the job; piecework.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Taslet</ent><br/
<hw>Tas"let</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[See <er>Tasse</er> a piece of armor.]</ety> <def>A piece of armor formerly worn to guard the thighs; a tasse.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tasmanian</ent><br/
<hw>Tas*ma"ni*an</hw> <pr>(t<acr/z*m<amac/"n<icr/*<ait/n)</pr>, <pos>a.</pos> <def>Of or pertaining to Tasmania, or Van Diemen's Land.</def> -- <def2><pos>n.</pos> <def>A native or inhabitant of Tasmania;</def> <specif>specifically</specif> <fld>(Ethnol.)</fld>, <def>in the plural, the race of men that formerly inhabited Tasmania, but is now extinct.</def></def2><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Tasmanian cider tree</b></col>. <fld>(Bot.)</fld> <cd>See the Note under <er>Eucalyptus</er>.</cd> -- <col><b>Tasmanian devil</b></col>. <fld>(Zool.)</fld> <cd>See under <er>Devil</er>.</cd> -- <col><b>Tasmanian wolf</b></col> <fld>(Zool.)</fld>, <cd>a savage carnivorous marsupial; -- called also <altname>zebra wolf</altname>. See <cref>Zebra wolf</cref>, under <er>Wolf</er>.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tasse</ent><br/
<hw>Tasse</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[OF. <ets>tassette</ets>.]</ety> <def>A piece of armor for the thighs, forming an appendage to the ancient corselet.</def><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ Usually the <xex>tasse</xex> was a plate of iron swinging from the cuirass, but the skirts of sliding splints were also called by this name.</note><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tassel</ent><br/
<hw>Tas"sel</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Falconry)</fld> <def>A male hawk. See <er>Tercel</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tassel</ent><br/
<hw>Tas"sel</hw>, <pos>n.</pos> <ety>[See <er>Teasel</er>.]</ety> <def>A kind of bur used in dressing cloth; a teasel.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tassel</ent><br/
<hw>Tas"sel</hw>, <pos>n.</pos> <ety>[OE., a fastening of a mantle, OF. <ets>tassel</ets> a fastening, clasp, F. <ets>tasseau</ets> a bracket, Fr. L. <ets>taxillus</ets> a little die, dim. of <ets>talus</ets> a die of a longish shape, rounded on two sides and marked only on the other four, a knuckle bone.]</ety> <sn>1.</sn> <def>A pendent ornament, attached to the corners of cushions, to curtains, and the like, ending in a tuft of loose threads or cords.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>The flower or head of some plants, esp. when pendent.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>And the maize field grew and ripened, Till it stood in all the splendor<br/
Of its garments green and yellow,<br/
Of its <qex>tassels</qex> and its plumage.</q> <rj><qau>Longfellow.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>A narrow silk ribbon, or the like, sewed to a book to be put between the leaves.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <fld>(Arch.)</fld> <def>A piece of board that is laid upon a wall as a sort of plate, to give a level surface to the ends of floor timbers; -- rarely used in the United States.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Tassel flower</b></col> <fld>(Bot.)</fld>, <cd>a name of several composite plants of the genus <gen>Cineraria</gen>, especially the <spn>Cineraria sconchifolia</spn>, and of the blossoms which they bear.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tassel</ent><br/
<hw>Tas"sel</hw>, <pos>v. i.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Tasseled</conjf> <pr>(?)</pr> or <conjf>Tasselled</conjf>; <pos>p. pr. & vb. n.</pos> <conjf>Tasseling</conjf> or <conjf>Tasselling</conjf>.]</vmorph> <def>To put forth a tassel or flower; <as>as, maize <ex>tassels</ex></as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tassel</ent><br/
<hw>Tas"sel</hw>, <pos>v. t.</pos> <def>To adorn with tassels.</def>  <rj><au>Chaucer.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tasset</ent><br/
<hw>Tas"set</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[See <er>Tasse</er>.]</ety> <def>A defense for the front of the thigh, consisting of one or more iron plates hanging from the belt on the lower edge of the corselet.</def><-- same as tasse? --><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tastable</ent><br/
<hw>Tast"a*ble</hw> <pr>(t<amac/st"<adot/*b'l)</pr>, <pos>a.</pos> <def>Capable of worthy of being tasted; savory; relishing.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Taste</ent><br/
<hw>Taste</hw> <pr>(t<amac/st)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Tasted</conjf>; <pos>p. pr. & vb. n.</pos> <conjf>Tasting</conjf>.]</vmorph> <ety>[OE. <ets>tasten</ets> to feel, to taste, OF. <ets>taster</ets>, F. <ets>tater</ets> to feel, to try by the touch, to try, to taste, (assumed) LL. <ets>taxitare</ets>, fr. L. <ets>taxare</ets> to touch sharply, to estimate. See <er>Tax</er>, <pos>v. t.</pos>]</ety> <sn>1.</sn> <def>To try by the touch; to handle; <as>as, to <ex>taste</ex> a bow</as>.</def> <mark>[Obs.]</mark>  <rj><au>Chapman.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q><qex>Taste</qex> it well and stone thou shalt it find.</q> <rj><qau>Chaucer.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To try by the touch of the tongue; to perceive the relish or flavor of (anything) by taking a small quantity into a mouth. Also used figuratively.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>When the ruler of the feast had <qex>tasted</qex> the water that was made wine.</q> <rj><qau>John ii. 9.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>When Commodus had once <qex>tasted</qex> human blood, he became incapable of pity or remorse.</q> <rj><qau>Gibbon.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>To try by eating a little; to eat a small quantity of.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>I <qex>tasted</qex> a little of this honey.</q> <rj><qau>1 Sam. xiv. 29.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>To become acquainted with by actual trial; to essay; to experience; to undergo.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>He . . . should <qex>taste</qex> death for every man.</q> <rj><qau>Heb. ii. 9.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <def>To partake of; to participate in; -- usually with an implied sense of relish or pleasure.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Thou . . . wilt <qex>taste</qex><br/
No pleasure, though in pleasure, solitary.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Taste</ent><br/
<hw>Taste</hw>, <pos>v. i.</pos> <sn>1.</sn> <def>To try food with the mouth; to eat or drink a little only; to try the flavor of anything; <as>as, to <ex>taste</ex> of each kind of wine</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To have a smack; to excite a particular sensation, by which the specific quality or flavor is distinguished; to have a particular quality or character; <as>as, this water <ex>tastes</ex> brackish; the milk <ex>tastes</ex> of garlic</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Yea, every idle, nice, and wanton reason<br/
Shall to the king <qex>taste</qex> of this action.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>To take sparingly.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>For age but <qex>tastes</qex> of pleasures, youth devours.</q> <rj><qau>Dryden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>To have perception, experience, or enjoyment; to partake; <as>as, to <ex>taste</ex> of nature's bounty</as>.</def>  <rj><au>Waller.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>The valiant never <qex>taste</qex> of death but once.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Taste</ent><br/
<hw>Taste</hw>, <pos>n.</pos> <sn>1.</sn> <def>The act of tasting; gustation.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A particular sensation excited by the application of a substance to the tongue; the quality or savor of any substance as perceived by means of the tongue; flavor; <as>as, the <ex>taste</ex> of an orange or an apple; a bitter <ex>taste</ex>; an acid <ex>taste</ex>; a sweet <ex>taste</ex>.</as></def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <fld>(Physiol.)</fld> <def>The one of the five senses by which certain properties of bodies (called their <xex>taste</xex>, <xex>savor</xex>, <xex>flavor</xex>) are ascertained by contact with the organs of taste.</def><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ Taste depends mainly on the contact of soluble matter with the terminal organs (connected with branches of the glossopharyngeal and other nerves) in the papillae on the surface of the tongue. The base of the tongue is considered most sensitive to bitter substances, the point to sweet and acid substances.</note><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>Intellectual relish; liking; fondness; -- formerly with <xex>of</xex>, now with <xex>for</xex>; <as>as, he had no <ex>taste</ex> for study</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>I have no <qex>taste</qex><br/
Of popular applause.</q> <rj><qau>Dryden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <def>The power of perceiving and relishing excellence in human performances; the faculty of discerning beauty, order, congruity, proportion, symmetry, or whatever constitutes excellence, particularly in the fine arts and belles-letters; critical judgment; discernment.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>6.</sn> <def>Manner, with respect to what is pleasing, refined, or in accordance with good usage; style; <as>as, music composed in good <ex>taste</ex>; an epitaph in bad <ex>taste</ex>.</as></def><br/
[<source>1913 Webster</source>]</p>

<p><sn>7.</sn> <def>Essay; trial; experience; experiment.</def>  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>8.</sn> <def>A small portion given as a specimen; a little piece tasted or eaten; a bit.</def>  <rj><au>Bacon.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>9.</sn> <def>A kind of narrow and thin silk ribbon.</def><br/
[<source>1913 Webster</source>]</p>

<p><syn><b>Syn.</b> -- Savor; relish; flavor; sensibility; gout.</syn> <usage> -- <er>Taste</er>, <er>Sensibility</er>, <er>Judgment</er>. Some consider <xex>taste</xex> as a mere <xex>sensibility</xex>, and others as a simple exercise of <xex>judgment</xex>; but a union of both is requisite to the existence of anything which deserves the name. An original sense of the beautiful is just as necessary to aesthetic judgments, as a sense of right and wrong to the formation of any just conclusions on moral subjects. But this <ldquo/sense of the beautiful<rdquo/ is not an arbitrary principle. It is under the guidance of reason; it grows in delicacy and correctness with the progress of the individual and of society at large; it has its laws, which are seated in the nature of man; and it is in the development of these laws that we find the true <ldquo/standard of taste.<rdquo/</usage><br/
[<source>1913 Webster</source>]</p>

<p><q>What, then, is <qex>taste</qex>, but those internal powers,<br/
Active and strong, and feelingly alive<br/
To each fine impulse? a discerning sense<br/
Of decent and sublime, with quick disgust<br/
From things deformed, or disarranged, or gross<br/
In species? This, nor gems, nor stores of gold,<br/
Nor purple state, nor culture, can bestow,<br/
But God alone, when first his active hand<br/
Imprints the secret bias of the soul.</q> <rj><qau>Akenside.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><mcol><col><b>Taste buds</b></col>, <it>or</it> <col><b>Taste goblets</b></col></mcol> <fld>(Anat.)</fld>, <cd>the flask-shaped end organs of taste in the epithelium of the tongue.  They are made up of modified epithelial cells arranged somewhat like leaves in a bud.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tasteful</ent><br/
<hw>Taste"ful</hw> <pr>(t<amac/st"f<usdot/l)</pr>, <pos>a.</pos> <sn>1.</sn> <def>Having a high relish; savory.</def> <ldquo/<xex>Tasteful</xex> herbs.<rdquo/  <rj><au>Pope.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Having or exhibiting good taste; in accordance with good taste; tasty; <as>as, a <ex>tasteful</ex> drapery</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p>-- <wordforms><wf>Taste"ful*ly</wf>, <pos>adv.</pos> -- <wf>Taste"ful*ness</wf>, <pos>n.</pos></wordforms><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tasteless</ent><br/
<hw>Taste"less</hw>, <pos>a.</pos> <sn>1.</sn> <def>Having no taste; insipid; flat; <as>as, <ex>tasteless</ex> fruit</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Destitute of the sense of taste; or of good taste; <as>as, a <ex>tasteless</ex> age</as>.</def>  <rj><au>Orrery.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>Not in accordance with good taste; <as>as, a <ex>tasteless</ex> arrangement of drapery; a <ex>tasteless</ex> remark</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p>-- <wordforms><wf>Taste"less*ly</wf>, <pos>adv.</pos> -- <wf>Taste"less*ness</wf>, <pos>n.</pos></wordforms><br/
[<source>1913 Webster</source>]</p>

<p><ent>Taster</ent><br/
<hw>Tast"er</hw> <pr>(t<amac/st"<etil/r)</pr>, <pos>n.</pos> <sn>1.</sn> <def>One who tastes; especially, one who first tastes food or drink to ascertain its quality.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Thy tutor be thy <qex>taster</qex>, ere thou eat.</q> <rj><qau>Dryden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>That in which, or by which, anything is tasted, as, a dram cup, a cheese taster, or the like.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <fld>(Zool.)</fld> <def>One of a peculiar kind of zooids situated on the polyp-stem of certain Siphonophora. They somewhat resemble the feeding zooids, but are destitute of mouths. See <er>Siphonophora</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tastily</ent><br/
<hw>Tast"i*ly</hw> <pr>(t<amac/st"<icr/*l<ycr/)</pr>, <pos>adv.</pos> <def>In a tasty manner.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tasting</ent><br/
<hw>Tast"ing</hw>, <pos>n.</pos> <def>The act of perceiving or tasting by the organs of taste; the faculty or sense by which we perceive or distinguish savors.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tasto</ent><br/
||<hw>Tas"to</hw> <pr>(t<aum/s"t<osl/)</pr>, <pos>n.</pos> <ety>[It.]</ety> <fld>(Mus.)</fld> <def>A key or thing touched to produce a tone.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs>||<col><b>Tasto solo</b></col>, <cd>single touch; -- in old music, a direction denoting that the notes in the bass over or under which it is written should be performed alone, or with no other chords than unisons and octaves.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tasty</ent><br/
<hw>Tast"y</hw> <pr>(t<amac/st"<ycr/)</pr>, <pos>a.</pos> <amorph>[<pos>Compar.</pos> <adjf>Tastier</adjf> <pr>(t<amac/st"<icr/*<etil/r)</pr>; <pos>superl.</pos> <adjf>Tastiest</adjf>.]</amorph> <sn>1.</sn> <def>Having a good taste; -- applied to persons; <as>as, a <ex>tasty</ex> woman</as>. See <er>Taste</er>, <pos>n.</pos>, 5.</def><-- not used in that sense now. --><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Being in conformity to the principles of good taste; elegant; <as>as, <ex>tasty</ex> furniture; a <ex>tasty</ex> dress.</as></def><br/
<syn><b>Syn. --</b> tasteful{2}.</syn><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tat</ent><br/
<hw>Tat</hw> <pr>(t<acr/t)</pr>, <pos>n.</pos> <ety>[Hind. <ets><tsdo/<amac/<tsdo/</ets>.]</ety> <def>Gunny cloth made from the fiber of the <spn>Corchorus olitorius</spn>, or jute.</def> <mark>[India]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tat</ent><br/
<hw>Tat</hw>, <pos>n.</pos> <ety>[Hind. <ets><tsdot/a<tsdot/<tsdot/<umac/</ets>.]</ety> <fld>(Zool.)</fld> <def>A pony.</def> <mark>[India]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tataupa</ent><br/
<hw>Ta*tau"pa</hw> <pr>(t<adot/*t<add/"p<adot/)</pr>, <pos>n.</pos> <ety>[From the native name.]</ety> <fld>(Zool.)</fld> <def>A South American tinamou (<spn>Crypturus tataupa</spn>).</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tatch</ent><br/
<hw>Tatch</hw> <pr>(t<acr/ch)</pr>, <pos>n.</pos> <ety>[F. <ets>tache</ets> spot. See <er>Techy</er>.]</ety> <def>A spot or stain; also, a trick.</def> <mark>[Obs.]</mark>  <rj><au>Sir T. Elyot.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tath</ent><br/
<hw>Tath</hw> <pr>(t<acr/th)</pr>, <mark>obs.</mark> <def><pos>3d pers. sing. pres.</pos> of <er>Ta</er>, to take.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tath</ent><br/
<hw>Tath</hw>, <pos>n.</pos> <ety>[Prov. E.; of Scand. origin; cf. Icel. <ets>ta<edh/</ets> dung, <ets>ta<edh/a</ets> the grass of a manured pasture, <ets>te<edh/ja</ets> to manure.  <root/58.  Cf. <er>Ted</er>.]</ety> <sn>1.</sn> <def>Dung, or droppings of cattle.</def> <mark>[Prov. Eng. & Scot.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>The luxuriant grass growing about the droppings of cattle in a pasture.</def> <mark>[Prov. Eng. & Scot.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tath</ent><br/
<hw>Tath</hw>, <pos>v. t.</pos> <def>To manure (land) by pasturing cattle on it, or causing them to lie upon it.</def> <mark>[Prov. Eng. & Scot.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tatou</ent><br/
<hw>Ta*tou"</hw> <pr>(t<adot/*t<oomac/")</pr>, <pos>n.</pos> <ety>[Cf. <er>Tatouay</er>.]</ety> <fld>(Zool.)</fld> <def>The giant armadillo (<spn>Priodontes gigas</spn>) of tropical South America. It becomes nearly five feet long including the tail. It is noted for its burrowing powers, feeds largely upon dead animals, and sometimes invades human graves.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tatouay</ent><br/
<hw>Tat"ou*ay</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Of Brazilian origin; cf. Pg. <ets>tatu</ets>, F. <ets>tatou</ets>.]</ety> <fld>(Zool.)</fld> <def>An armadillo (<spn>Xenurus unicinctus</spn>), native of the tropical parts of South America.  It has about thirteen movable bands composed of small, nearly square, scales.  The head is long; the tail is round and tapered, and nearly destitute of scales; the claws of the fore feet are very large.  Called also <altname>tatouary</altname>, and <altname>broad-banded armadillo</altname>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tatouhou</ent><br/
<hw>Tat"ou*hou</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Cf. <er>Tatouay</er>.]</ety> <fld>(Zool.)</fld> <def>The peba.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tatt</ent><br/
<hw>Tatt</hw> <pr>(t<acr/t)</pr>, <pos>v. t. & i.</pos> <def>To make (anything) by tatting; to work at tatting; <as>as, <ex>tatted</ex> edging</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><-- p. 1477 --><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tatta</ent><br/
||<hw>Tat"ta</hw> <pr>(t<acr/t"t<adot/)</pr>, <pos>n.</pos> <ety>[Hind. <ets><tsdot/a<tsdot/<tsdot/<imac/</ets>, <ets>t<amac/t<imac/</ets>.]</ety> <def>A bamboo frame or trellis hung at a door or window of a house, over which water is suffered to trickle, in order to moisten and cool the air as it enters.</def> <mark>[India]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tatter</ent><br/
<hw>Tat"ter</hw> <pr>(?)</pr>, <pos>n.</pos> <def>One who makes tatting.</def>  <rj><au>Caulfield & S. (Doct. of Needlework).</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tatter</ent><br/
<hw>Tat"ter</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Icel. <ets>t<oum/tur</ets>, <ets>t<oum/ttur</ets>, pl. <ets>t<oum/trar</ets>, <ets><?/<oum/ttrar</ets>; cf. Norw. <ets>totra</ets>, pl. <ets>totror</ets>, LG. <ets>taltern</ets> tatters. <root/240.]</ety> <def>A rag, or a part torn and hanging; -- chiefly used in the plural.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Tear a passion to <qex>tatters</qex>, to very rags.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tatter</ent><br/
<hw>Tat"ter</hw>, <pos>v. t.</pos> <vmorph>[<pos>p. p.</pos> <conjf>Tattered</conjf> <pr>(?)</pr>.]</vmorph> <def>To rend or tear into rags; -- used chiefly in the past participle as an adjective.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Where waved the <qex>tattered</qex> ensigns of Ragfair.</q> <rj><qau>Pope.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tatterdemalion</ent><br/
<hw>Tat`ter*de*mal"ion</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[<ets>Tatter</ets> + OF. <ets>desmaillier</ets> to break the meshes of, to tear: cf. OF. <ets>maillon</ets> long clothes, swadding clothes, F. <ets>maillot.</ets> See <er>Tatter</er>, and <er>Mail</er> armor.]</ety> <def>A ragged fellow; a ragamuffin.</def>  <rj><au>L'Estrange.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tattersall's</ent><br/
<hw>Tat"ter*sall's</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A famous horse market in London, established in 1766 by Richard Tattersall, also used as the headquarters of credit betting on English horse races; hence, a large horse market elsewhere.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Tatting</ent><br/
<hw>Tat"ting</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A kind of lace made from common sewing thread, with a peculiar stitch.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Tatting shuttle</b></col>, <cd>the shuttle on which the thread used in tatting is wound.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tattle</ent><br/
<hw>Tat"tle</hw> <pr>(?)</pr>, <pos>v. i.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Tattled</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Tattling</conjf> <pr>(?)</pr>.]</vmorph> <ety>[Akin to OE. <ets>tateren</ets>, LG. <ets>tateln</ets>, D. <ets>tateren</ets> to stammer, and perhaps to E. <ets>titter</ets>.]</ety> <sn>1.</sn> <def>To prate; to talk idly; to use many words with little meaning; to chat.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The <qex>tattling</qex> quality of age, which is always narrative.</q> <rj><qau>Dryden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To tell tales; to communicate secrets; to be a talebearer; <as>as, a <ex>tattling</ex> girl</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tattle</ent><br/
<hw>Tat"tle</hw>, <pos>n.</pos> <def>Idle talk or chat; trifling talk; prate.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>[They] told the <qex>tattle</qex> of the day.</q> <rj><qau>Swift.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tattler</ent><br/
<hw>Tat"tler</hw> <pr>(?)</pr>, <pos>n.</pos> <sn>1.</sn> <def>One who tattles; an idle talker; one who tells tales.</def>  <rj><au>Jer. Taylor.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Zool.)</fld> <def>Any one of several species of large, long-legged sandpipers belonging to the genus <gen>Totanus</gen>.</def><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ The common American species are the greater tattler, or telltale (<spn>Totanus melanoleucus</spn>), the smaller tattler, or lesser yellowlegs (<spn>Totanus flavipes</spn>), the solitary tattler (<spn>Totanus solitarius</spn>), and the semipalmated tattler, or willet. The first two are called also <altname>telltale</altname>, <altname>telltale spine</altname>, <altname>telltale tattler</altname>, <altname>yellowlegs</altname>, <altname>yellowshanks</altname>, and <altname>yelper.</altname></note><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tattlery</ent><br/
<hw>Tat"tler*y</hw> <pr>(?)</pr>, <pos>n.</pos> <def>Idle talk or chat; tittle-tattle.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tattling</ent><br/
<hw>Tat"tling</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Given to idle talk; apt to tell tales.</def> -- <wordforms><wf>Tat"tling*ly</wf>, <pos>adv.</pos></wordforms><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tattoo</ent><br/
<hw>Tat*too"</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Earlier <ets>taptoo</ets>, D. <ets>taptoe</ets>; <ets>tap</ets> a tap, faucet + <ets>toe</ets> to, shut (<it>i. e.</it>, the taps, or drinking houses, shut from the soldiers).]</ety> <fld>(Mil.)</fld> <def>A beat of drum, or sound of a trumpet or bugle, at night, giving notice to soldiers to retreat, or to repair to their quarters in garrison, or to their tents in camp.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>The Devil's tattoo</b></col>. <cd>See under <er>Devil</er>.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tattoo</ent><br/
<hw>Tat*too"</hw>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Tattooed</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Tattooing</conjf>.]</vmorph> <ety>[Of Polynesian origin; cf. New Zealand <ets>ta</ets> to tattoo, <ets>tatu</ets> puncturation (in Otaheite).]</ety> <def>To color, as the flesh, by pricking in coloring matter, so as to form marks or figures which can not be washed out.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tattoo</ent><br/
<hw>Tat*too"</hw>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Tattoos</plw> <pr>(<?/)</pr>.</plu> <def>An indelible mark or figure made by puncturing the skin and introducing some pigment into the punctures; -- a mode of ornamentation practiced by various barbarous races, both in ancient and modern times, and also by some among civilized nations, especially by sailors.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tatty</ent><br/
<hw>Tat"ty</hw> <pr>(?)</pr>, <pos>n.</pos>; <plu>pl. <plw>Tatties</plw> <pr>(#)</pr></plu>. <ety>[Hind. <ets><tsdot/a<tsdot/<tsdot/<imac/</ets>.]</ety> <def>A mat or screen of fibers, as of the kuskus grass, hung at a door or window and kept wet to moisten and cool the air as it enters.</def> <mark>[India]</mark><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Tatu</ent><br/
<hw>Ta*tu"</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Zool.)</fld> <def>Same as <er>Tatou</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tatusiid</ent><br/
<hw>Ta*tu"si*id</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Zool.)</fld> <def>Any armadillo of the family <fam>Tatusiidae</fam>, of which the peba and mule armadillo are examples. Also used adjectively.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tau</ent><br/
<hw>Tau</hw> <pr>(tou)</pr>, <pos>n.</pos> <ety>[Gr. <grk>tay^</grk>.]</ety> <def>The nineteenth letter (<TAU/, <tau/) of the Greek alphabet, equivalent to English <it>t</it>.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Tau</ent><br/
<hw>Tau</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Gr. <grk>tay^</grk> the letter <tau/ (English <er>T</er>).]</ety> <fld>(Zool.)</fld> <def>The common American toadfish; -- so called from a marking resembling the Greek letter <ex>tau</ex> (<tau/).</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Tau cross</b></col>. <cd>See <xex>Illust.</xex> 6, of <er>Cross</er>.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Taught</ent><br/
<hw>Taught</hw> <pr>(?)</pr>, <pos>a.</pos> <def>See <er>Taut</er>.</def>  <rj><au>Totten.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Taught</ent><br/
<hw>Taught</hw>, <def><pos>imp. & p. p.</pos> of <er>Teach</er>.</def> <ety>[AS. imp. <ets>t<aemac/hte</ets>, p. p. <ets>get<aemac/ht</ets>.]</ety> <note>See <er>Teach</er>.</note><br/
[<source>1913 Webster</source>]</p>

<p><ent>Taunt</ent><br/
<hw>Taunt</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[Cf. OF. <ets>tant</ets> so great, F. <ets>tant</ets> so much, L. <ets>tantus</ets> of such size, so great, so much.]</ety> <fld>(Naut.)</fld> <def>Very high or tall; <as>as, a ship with <ex>taunt</ex> masts</as>.</def>  <rj><au>Totten.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Taunt</ent><br/
<hw>Taunt</hw> <pr>(?)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Taunted</conjf>; <pos>p. pr. & vb. n.</pos> <conjf>Taunting</conjf>.]</vmorph> <ety>[Earlier, to tease; probably fr. OF. <ets>tanter</ets> to tempt, to try, for <ets>tenter</ets>. See <er>Tempt</er>.]</ety> <def>To reproach with severe or insulting words; to revile; to upbraid; to jeer at; to flout.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>When I had at my pleasure <qex>taunted</qex> her.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><syn><b>Syn.</b> -- To deride; ridicule; mock; jeer; flout; revile. See <er>Deride</er>.</syn><br/
[<source>1913 Webster</source>]</p>

<p><ent>Taunt</ent><br/
<hw>Taunt</hw>, <pos>n.</pos> <def>Upbraiding language; bitter or sarcastic reproach; insulting invective.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>With scoffs, and scorns, and contemelious <qex>taunts</qex>.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>With sacrilegious <qex>taunt</qex> and impious jest.</q> <rj><qau>Prior.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Taunter</ent><br/
<hw>Taunt"er</hw> <pr>(?)</pr>, <pos>n.</pos> <def>One who taunts.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Taunting</ent><br/
<hw>Taunt"ing</hw>, <def><pos>a. & n.</pos> from <er>Taunt</er>, <pos>v.</pos></def><br/
[<source>1913 Webster</source>]</p>

<p><q>Every kind of insolent and <qex>taunting</qex> reflection.</q> <rj><qau>Burke.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tauntingly</ent><br/
<hw>Taunt"ing*ly</hw>, <pos>adv.</pos> <def>In a taunting manner.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tauntress</ent><br/
<hw>Taunt"ress</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A woman who taunts.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tawpie</ent><br/
<ent>Taupie</ent><br/
<mhw>{ <hw>Tau"pie</hw>, <hw>Taw"pie</hw>  }</mhw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Cf. Icel. <ets>t<omac/pi</ets> fool, Dan. <ets>taabe</ets>, Sw. <ets>t<aring/p</ets>.]</ety> <def>A foolish or thoughtless young person, esp. a slothful or slovenly woman.</def> <mark>[Scot.]</mark>  <rj><au>Burns.</au></rj><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Taur</ent><br/
<hw>Taur</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>Taurus</ets>.]</ety> <def>The constellation Taurus.</def> <mark>[Obs.]</mark>  <rj><au>Chaucer.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tauricornous</ent><br/
<hw>Tau`ri*cor"nous</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>tauricornis</ets>; <ets>taurus</ets> a bull + <ets>cornu</ets> a horn.]</ety> <fld>(Zool.)</fld> <def>Having horns like those of a bull.</def>  <rj><au>Sir T. Browne.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Taurid</ent><br/
<hw>Tau"rid</hw> <pr>(?)</pr>, <pos>n.</pos>  <ety>[<ets>Taurus</ets> + 1st <ets>-id</ets>.]</ety> <fld>(Astron.)</fld> <def>Any of a group of meteors appearing November 20-23; -- so called because they appear to radiate from a point in Taurus.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Tauridor</ent><br/
<hw>Tau`ri*dor"</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[See <er>Toreador</er>.]</ety> <def>A bullfighter; a toreador.</def>  <rj><au>Sir W. Scott.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tauriform</ent><br/
<hw>Tau"ri*form</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>tauriformis</ets>; <ets>taurus</ets> a bull + <ets>-form</ets>: cf. F. <ets>tauriforme</ets>.]</ety> <def>Having the form of a bull.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Taurine</ent><br/
<hw>Tau"rine</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>taurinus</ets>, fr. <ets>taurus</ets> a bull. See <er>Taurus</er>.]</ety> <fld>(Zool.)</fld> <def>Of or pertaining to the genus Taurus, or cattle.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Taurine</ent><br/
<hw>Tau"rine</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[So named because it was discovered in the bile of the ox. See <er>Taurus</er>.]</ety> <fld>(Physiol. Chem.)</fld> <def>A chemical compound occurring in small quantity in the juices of muscle, in the lungs, and elsewhere, but especially in the bile, where it is found as a component part of taurocholic acid, from which it can be prepared by decomposition of the acid. It crystallizes in colorless, regular six-sided prisms, and is especially characterized by containing both nitrogen and sulphur, being chemically amido-isethionic acid, <chform>C2H7NSO3</chform>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Taurocholate</ent><br/
<hw>Tau`ro*cho"late</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Physiol. Chem.)</fld> <def>A salt of taurocholic acid; <as>as, sodium <ex>taurocholate</ex>, which occurs in human bile</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Taurocholic</ent><br/
<hw>Tau`ro*chol"ic</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[<ets>Taur</ets>ine + <ets>cholic</ets>.]</ety> <fld>(Physiol. Chem.)</fld> <def>Pertaining to, or designating, a conjugate acid (called <xex>taurocholic acid</xex>) composed of taurine and cholic acid, present abundantly in human bile and in that of carnivora. It is exceedingly deliquescent, and hence appears generally as a thick, gummy mass, easily soluble in water and alcohol. It has a bitter taste.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Taurocolla</ent><br/
<ent>Taurocol</ent><br/
<mhw>{ <hw>Tau"ro*col</hw> <pr>(?)</pr>, <hw>Tau`ro*col"la</hw> <pr>(?)</pr>, }</mhw> <pos>n.</pos> <ety>[NL. <ets>taurocolla</ets>, fr. Gr. <grk>tayro`kolla</grk>; <grk>tay^ros</grk> a bull + <grk>ko`lla</grk> glue: cf. F. <ets>taurocolle</ets>.]</ety> <def>Glue made from a bull's hide.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tauromachian</ent><br/
<hw>Tau`ro*ma"chi*an</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[See <er>Tauromachy</er>.]</ety> <def>Of or pertaining to bullfights.</def> -- <def2><pos>n.</pos> <def>A bullfighter.</def></def2><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tauromachy</ent><br/
<hw>Tau*rom"a*chy</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Gr. <grk>tayromachi`a</grk>; <grk>tay^ros</grk> bull + <grk>ma`chh</grk> fight.]</ety> <def>Bullfighting.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Taurus</ent><br/
||<hw>Tau"rus</hw> <pr>(t<add/"r<ucr/s)</pr>, <pos>n.</pos> <ety>[L., akin to Gr. <grk>tay^ros</grk>, and E. <ets>steer</ets>. See <er>Steer</er> a young ox.]</ety> <sn>1.</sn> <fld>(Astron.)</fld> <sd>(a)</sd> <def>The Bull; the second in order of the twelve signs of the zodiac, which the sun enters about the 20th of April; -- marked thus [<taurus/] in almanacs.</def> <sd>(b)</sd> <def>A zodiacal constellation, containing the well-known clusters called the Pleiades and the Hyades, in the latter of which is situated the remarkably bright Aldebaran.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Zool.)</fld> <def>A genus of ruminants comprising the common domestic cattle.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Taurylic</ent><br/
<hw>Tau*ryl"ic</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>taurus</ets> a bull + E. phen<ets>ylic</ets>.]</ety> <fld>(Chem.)</fld> <def>Pertaining to, or designating, an acid found of a urine of neat cattle, and probably identical with <xex>cresol</xex>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Taut</ent><br/
<hw>Taut</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[Dan. <ets>t<ae/t</ets>; akin to E. <ets>tight</ets>. See <er>Tight</er>.]</ety> <sn>1.</sn> <fld>(Naut.)</fld> <def>Tight; stretched; not slack; -- said esp. of a rope that is tightly strained.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Snug; close; firm; secure.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Taut hand</b></col> <fld>(Naut.)</fld>, <cd>a sailor's term for an officer who is severe in discipline.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tautaug</ent><br/
<hw>Tau*taug"</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Zool.)</fld> <def>Same as <er>Tautog</er>.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Tautegorical</ent><br/
<hw>Tau`te*gor"ic*al</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[Gr. <?/, for <?/ <?/ the same + <?/ to speak.  Cf. <er>Allegory</er>.]</ety> <def>Expressing the same thing with different words; -- opposed to <xex>allegorical</xex>.</def> <mark>[R.]</mark>  <rj><au>Coleridge.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tautochrone</ent><br/
<hw>Tau"to*chrone</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Gr. <?/, for <?/ <?/ the same + <?/ time: cf. F. <ets>tautochrone</ets>.]</ety> <fld>(Math.)</fld> <def>A curved line, such that a heavy body, descending along it by the action of gravity, will always arrive at the lowest point in the same time, wherever in the curve it may begin to fall; <as>as, an inverted cycloid with its base horizontal is a <ex>tautochrone</ex></as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tautochronous</ent><br/
<hw>Tau*toch"ro*nous</hw> <pr>(?)</pr>, <pos>a.</pos> <fld>(Math.)</fld> <def>Occupying the same time; pertaining to, or having the properties of, a tautochrone.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tautog</ent><br/
<hw>Tau*tog"</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[The pl. of <ets>taut</ets>, the American Indian name, translated by Roger Williams <ets>sheep's heads</ets>, and written by him <ets>tauta<uacute/og</ets>.]</ety> <fld>(Zool.)</fld> <def>An edible labroid fish (<spn>Haitula onitis</spn>, or <spn>Tautoga onitis</spn>) of the Atlantic coast of the United States. When adult it is nearly black, more or less irregularly barred, with greenish gray. Called also <altname>blackfish</altname>, <altname>oyster fish</altname>, <altname>salt-water chub</altname>, and <altname>moll</altname>.</def> <altsp>[Written also <asp>tautaug</asp>.]</altsp><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tautologic</ent><br/
<hw>Tau`to*log"ic</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Tautological.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tautological</ent><br/
<hw>Tau`to*log"ic*al</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[Cf. F. <ets>tautologique</ets>.]</ety> <def>Involving tautology; having the same signification; <as>as, <ex>tautological</ex> expression</as>.</def> -- <wordforms><wf>Tau`to*log"ic*al*ly</wf>, <pos>adv.</pos></wordforms><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Tautological echo</b></col>, <cd>an echo that repeats the same sound or syllable many times.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tautologist</ent><br/
<hw>Tau*tol"o*gist</hw> <pr>(?)</pr>, <pos>n.</pos> <def>One who uses tautological words or phrases.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tautologize</ent><br/
<hw>Tau*tol"o*gize</hw> <pr>(?)</pr>, <pos>v. i.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Tautologized</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Tautologizing</conjf> <pr>(?)</pr>.]</vmorph> <def>To repeat the same thing in different words.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tautologous</ent><br/
<hw>Tau*tol"o*gous</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[Gr. <?/; <?/, for <?/ <?/ the same + <?/ to speak.]</ety> <def>Repeating the same thing in different words; tautological.</def> <mark>[R.]</mark>  <rj><au>Tooke.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tautology</ent><br/
<hw>Tau*tol"o*gy</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>tautologia</ets>, Gr. <?/: cf. F. <ets>tautologie</ets>.]</ety> <fld>(Rhet.)</fld> <def>A repetition of the same meaning in different words; needless repetition of an idea in different words or phrases; a representation of anything as the cause, condition, or consequence of itself, as in the following lines: --<br/
<br/
<q>The dawn is overcast, the morning lowers,<br/
And heavily in clouds brings on the day.</q> <rj><qau>Addison.</qau></rj></def><br/
[<source>1913 Webster</source>]</p>

<p><syn><b>Syn.</b> -- Repetition.</syn> <usage> -- <er>Tautology</er>, <er>Repetition</er>. There may be frequent <xex>repetitions</xex> (as in legal instruments) which are warranted either by necessity or convenience; but <xex>tautology</xex> is always a fault, being a sameness of expression which adds nothing to the sense or the sound.</usage><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tautomeric</ent><br/
<hw>Tau`to*mer"ic</hw> <pr>(?)</pr>, <pos>a.</pos> <fld>(Chem.)</fld> <def>Relating to, or characterized by, tautomerism.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tautomerism</ent><br/
<hw>Tau*tom"er*ism</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Gr. <?/; <?/, for <?/ <?/ the same + <?/ part.]</ety> <fld>(Chem.)</fld> <def>The condition, quality, or relation of metameric substances, or their respective derivatives, which are more or less interchangeable, according as one form or the other is the more stable. It is a special case of metamerism; thus, the lactam and the lactim compounds exhibit <xex>tautomerism</xex>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tautoousious</ent><br/
<ent>Tautoousian</ent><br/
<mhw>{ <hw>Tau`to*ou"si*an</hw> <pr>(?)</pr>, <hw>Tau`to*ou"si*ous</hw> <pr>(?)</pr>, }</mhw> <pos>a.</pos> <ety>[Gr. <?/; <?/, for <?/ <?/ the same + <?/ being, essence.]</ety> <def>Having the same essence; being identically of the same nature.</def> <mark>[R.]</mark>  <rj><au>Cudworth.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tautophonical</ent><br/
<hw>Tau`to*phon"ic*al</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Pertaining to, or characterized by, tautophony; repeating the same sound.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tautophony</ent><br/
<hw>Tau*toph"o*ny</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Gr. <?/; <?/, for <?/ <?/ the same + <?/ voice.]</ety> <def>Repetition of the same sound.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tautozonal</ent><br/
<hw>Tau`to*zon"al</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[Gr. <?/; <?/, for <?/ <?/ the same + E. <ets>zonal</ets>.]</ety> <fld>(Crystallog.)</fld> <def>Belonging to the same zone; <as>as, <ex>tautozonal</ex> planes</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tavern</ent><br/
<hw>Tav"ern</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[OE. <ets>taverne</ets>, F. <ets>taverne</ets>, from L. <ets>taberna</ets> a hut, booth, tavern.  Cf. <er>Table</er>, <er>Tabernacle</er>.]</ety> <def>A public house where travelers and other transient guests are accomodated with rooms and meals; an inn; a hotel; especially, in modern times, a public house licensed to sell liquor in small quantities.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Taverner</ent><br/
<hw>Tav"ern*er</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F. <ets>tavernier</ets>, L. <ets>tabernarius</ets>.]</ety> <def>One who keeps a tavern.</def>  <rj><au>Chaucer. Camden.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Taverning</ent><br/
<hw>Tav"ern*ing</hw>, <pos>n.</pos> <def>A feasting at taverns.</def> <mark>[Obs.]</mark> <ldquo/The misrule of our <xex>tavernings</xex>.<rdquo/  <rj><au>Bp. Hall.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tavernman</ent><br/
<hw>Tav"ern*man</hw> <pr>(?)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Tavernmen</plw> <pr>(<?/)</pr>.</plu> <def>The keeper of a tavern; also, a tippler.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Taw</ent><br/
<hw>Taw</hw> <pr>(?)</pr>, <pos>n.</pos> <def>Tow.</def> <mark>[Obs.]</mark>  <rj><au>Chaucer.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Taw</ent><br/
<hw>Taw</hw>, <pos>v. t.</pos> <ety>[Cf. <er>Tew</er> to tow, <er>Tow</er>, <pos>v. t.</pos>]</ety> <def>To push; to tug; to tow.</def> <mark>[Obs.]</mark>  <rj><au>Drayton.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Taw</ent><br/
<hw>Taw</hw> <pr>(?)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Tawed</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Tawing</conjf>.]</vmorph> <ety>[OE. <ets>tawen</ets>, <ets>tewen</ets>, AS. <ets>t<amac/wian</ets> to prepare; cf. D. <ets>touwen</ets>, Goth. <ets>t<emac/wa</ets> order, <ets>taujan</ets> to do, and E. <ets>tool</ets>. <root/64.  Cf. 1st <er>Tew</er>, <er>Tow</er> the coarse part of flax.]</ety> <sn>1.</sn> <def>To prepare or dress, as hemp, by beating; to tew; hence, to beat; to scourge.</def> <mark>[Obs.]</mark>  <rj><au>Beau. & Fl.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To dress and prepare, as the skins of sheep, lambs, goats, and kids, for gloves, and the like, by imbuing them with alum, salt, and other agents, for softening and bleaching them.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Taw</ent><br/
<hw>Taw</hw>, <pos>n.</pos> <ety>[Cf. AS. <ets>t<amac/w</ets> instrument.]</ety> <sn>1.</sn> <def>A large marble to be played with; also, a game at marbles.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A line or mark from which the players begin a game of marbles.</def> <mark>[Colloq. U. S.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tawdrily</ent><br/
<hw>Taw"dri*ly</hw> <pr>(?)</pr>, <pos>adv.</pos> <def>In a tawdry manner.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tawdriness</ent><br/
<hw>Taw"dri*ness</hw>, <pos>n.</pos> <def>Quality or state of being tawdry.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>A clumsy person makes his ungracefulness more ungraceful by <qex>tawdriness</qex> of dress.</q> <rj><qau>Richardson.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tawdry</ent><br/
<hw>Taw"dry</hw> <pr>(?)</pr>, <pos>a.</pos> <amorph>[<pos>Compar.</pos> <adjf>Tawdrier</adjf> <pr>(?)</pr>; <pos>superl.</pos> <adjf>Tawdriest</adjf>.]</amorph> <ety>[Said to be corrupted from <ets>Saint Audrey</ets>, or <ets>Auldrey</ets>, meaning <ets>Saint Ethelreda</ets>, implying therefore, originally, bought at the fair of St. Audrey, where laces and gay toys of all sorts were sold. This fair was held in Isle Ely, and probably at other places, on the day of the saint, which was the 17th of October.]</ety> <sn>1.</sn> <def>Bought at the festival of St. Audrey.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>And gird in your waist,<br/
For more fineness, with a <qex>tawdry</qex> lace.</q> <rj><qau>Spenser.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Very fine and showy in colors, without taste or elegance; having an excess of showy ornaments without grace; cheap and gaudy; <as>as, a <ex>tawdry</ex> dress; <ex>tawdry</ex> feathers; <ex>tawdry</ex> colors.</as></def><-- tacky? --><br/
[<source>1913 Webster</source>]</p>

<p><q>He rails from morning to night at essenced fops and <qex>tawdry</qex> courtiers.</q> <rj><qau>Spectator.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tawdry</ent><br/
<hw>Taw"dry</hw>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Tawdries</plw> <pr>(<?/)</pr>.</plu> <def>A necklace of a rural fashion, bought at St. Audrey's fair; hence, a necklace in general.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>Of which the Naiads and the blue Nereids make<br/
Them <qex>tawdries</qex> for their necks.</q> <rj><qau>Drayton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tawer</ent><br/
<hw>Taw"er</hw> <pr>(?)</pr>, <pos>n.</pos> <def>One who taws; a dresser of white leather.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tawery</ent><br/
<hw>Taw"er*y</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A place where skins are tawed.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tawniness</ent><br/
<hw>Taw"ni*ness</hw> <pr>(?)</pr>, <pos>n.</pos> <def>The quality or state of being tawny.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tawny</ent><br/
<hw>Taw"ny</hw> <pr>(?)</pr>, <pos>a.</pos> <amorph>[<pos>Compar.</pos> <adjf>Tawnier</adjf> <pr>(?)</pr>; <pos>superl.</pos> <adjf>Tawniest</adjf>.]</amorph> <ety>[F. <ets>tann<eacute/</ets>, p. p. of <ets>tanner</ets> to tan. See <er>Tan</er>, <pos>v. t.</pos> & <pos>n.</pos>  Cf. <er>Tenn<eacute/</er>.]</ety> <def>Of a dull yellowish brown color, like things tanned, or persons who are sunburnt; <as>as, <ex>tawny</ex> Moor or Spaniard; the <ex>tawny</ex> lion</as>.</def> <ldquo/A leopard's <xex>tawny</xex> and spotted hide.<rdquo/  <rj><au>Longfellow.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Taws</ent><br/
<hw>Taws</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[See <er>Taw</er> to beat.]</ety> <def>A leather lash, or other instrument of punishment, used by a schoolmaster.</def> <altsp>[Written also <asp>tawes</asp>, <asp>tawis</asp>, and <asp>tawse</asp>.]</altsp> <mark>[Scot.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>Never use the <qex>taws</qex> when a gloom can do the turn.</q> <rj><qau>Ramsay.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tax</ent><br/
<hw>Tax</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F. <ets>taxe</ets>, fr. <ets>taxer</ets> to tax, L. <ets>taxare</ets> to touch, sharply, to feel, handle, to censure, value, estimate, fr. <ets>tangere</ets>, <ets>tactum</ets>, to touch. See <er>Tangent</er>, and cf. <er>Task</er>, <er>Taste</er>.]</ety> <sn>1.</sn> <def>A charge, especially a pecuniary burden which is imposed by authority.</def> Specifically: --<br/
[<source>1913 Webster</source>]</p>

<p><sd>(a)</sd> <def>A charge or burden laid upon persons or property for the support of a government.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>A farmer of <qex>taxes</qex> is, of all creditors, proverbially the most rapacious.</q> <rj><qau>Macaulay.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sd>(b)</sd> <def>Especially, the sum laid upon specific things, as upon polls, lands, houses, income, etc.; <as>as, a land <ex>tax</ex>; a window <ex>tax</ex>; a <ex>tax</ex> on carriages, and the like</as>.</def> <note>Taxes are <stype>annual</stype> or <stype>perpetual</stype>, <stype>direct</stype> or <stype>indirect</stype>, etc.</note><br/
[<source>1913 Webster</source>]</p>

<p><sd>(c)</sd> <def>A sum imposed or levied upon the members of a society to defray its expenses.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A task exacted from one who is under control; a contribution or service, the rendering of which is imposed upon a subject.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>A disagreeable or burdensome duty or charge; <as>as, a heavy <ex>tax</ex> on time or health</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>Charge; censure.</def> <mark>[Obs.]</mark>  <rj><au>Clarendon.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <def>A lesson to be learned; a task.</def> <mark>[Obs.]</mark>  <rj><au>Johnson.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Tax cart</b></col>, <cd>a spring cart subject to a low tax.</cd> <mark>[Eng.]</mark></cs><br/
[<source>1913 Webster</source>]</p>

<p><syn><b>Syn.</b> -- Impost; tribute; contribution; duty; toll; rate; assessment; exaction; custom; demand.</syn><br/
[<source>1913 Webster</source>]</p>

<p><-- p. 1478 --><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tax</ent><br/
<hw>Tax</hw> <pr>(t<acr/ks)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Taxed</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Taxing</conjf>.]</vmorph> <ety>[Cf. F. <ets>taxer</ets>. See <er>Tax</er>, <pos>n.</pos>]</ety> <sn>1.</sn> <def>To subject to the payment of a tax or taxes; to impose a tax upon; to lay a burden upon; especially, to exact money from for the support of government.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>We are more heavily <qex>taxed</qex> by our idleness, pride, and folly than we are <qex>taxed</qex> by government.</q> <rj><qau>Franklin.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Law)</fld> <def>To assess, fix, or determine judicially, the amount of; <as>as, to <ex>tax</ex> the cost of an action in court</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>To charge; to accuse; also, to censure; -- often followed by <xex>with</xex>, rarely by <xex>of</xex> before an indirect object; <as>as, to <ex>tax</ex> a man with pride</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>I <qex>tax</qex> you, you elements, with unkindness.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Men's virtues I have commended as freely as I have <qex>taxed</qex> their crimes.</q> <rj><qau>Dryden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Fear not now that men should <qex>tax</qex> thine honor.</q> <rj><qau>M. Arnold.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Taxability</ent><br/
<hw>Tax`a*bil"i*ty</hw> <pr>(?)</pr>, <pos>n.</pos> <def>The quality or state of being taxable; taxableness.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Taxable</ent><br/
<hw>Tax"a*ble</hw> <pr>(?)</pr>, <pos>a.</pos> <sn>1.</sn> <def>Capable of being taxed; liable by law to the assessment of taxes; <as>as, <ex>taxable</ex> estate; <ex>taxable</ex> commodities</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Law)</fld> <def>That may be legally charged by a court against the plaintiff of defendant in a suit; <as>as, <ex>taxable</ex> costs</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p>-- <wordforms><wf>Tax"a*ble*ness</wf>, <pos>n.</pos> -- <wf>Tax"a*bly</wf>, <pos>adv.</pos></wordforms><br/
[<source>1913 Webster</source>]</p>

<p><ent>Taxaspidean</ent><br/
<hw>Tax`as*pid"e*an</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[Gr. <?/ an arrangement + <?/, <?/, shield.]</ety> <fld>(Zool.)</fld> <def>Having the posterior tarsal scales, or scutella, rectangular and arranged in regular rows; -- said of certain birds.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Taxation</ent><br/
<hw>Tax*a"tion</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F. <ets>taxation</ets>, L. <ets>taxatio</ets> a valuing, estimation, from L. <ets>taxare</ets>. See <er>Tax</er>.]</ety> <sn>1.</sn> <def>The act of laying a tax, or of imposing taxes, as on the subjects of a state, by government, or on the members of a corporation or company, by the proper authority; the raising of revenue; also, a system of raising revenue.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Law)</fld> <def>The act of taxing, or assessing a bill of cost.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>Tax; sum imposed.</def> <mark>[R.]</mark>  <rj><au>Daniel.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>Charge; accusation.</def> <mark>[Obs.]</mark>  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tax certificate</ent><br/
<hw>Tax certificate</hw>. <fld>(Law)</fld> <def>The certificate issued to the purchaser of land at a tax sale certifying to the sale and the payment of the consideration thereof, and entitling the purchaser upon certain conditions and at a certain time thereafter to a deed or instrument of conveyance (called a <xex>tax deed</xex>) of the land, to be executed by the proper officer.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Taxel</ent><br/
<hw>Tax"el</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Zool.)</fld> <def>The American badger.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Taxeopoda</ent><br/
||<hw>Tax`e*op"o*da</hw> <pr>(?)</pr>, <pos>n. pl.</pos> <ety>[NL., fr. Gr. <?/ (?) + <ets>-poda</ets>.]</ety> <fld>(Paleon.)</fld> <def>An order of extinct Mammalia found in the Tertiary formations.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Taxer</ent><br/
<hw>Tax"er</hw> <pr>(?)</pr>, <pos>n.</pos> <sn>1.</sn> <def>One who taxes.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>One of two officers chosen yearly to regulate the assize of bread, and to see the true gauge of weights and measures is observed.</def> <mark>[Camb. Univ., Eng.]</mark> <altsp>[Written also <asp>taxor</asp>.]</altsp><br/
[<source>1913 Webster</source>]</p>

<p><ent>Taxgatherer</ent><br/
<hw>Tax"gath`er*er</hw> <pr>(?)</pr>, <pos>n.</pos> <def>One who collects taxes or revenues.</def> -- <wordforms><wf>Tax"gath`er*ing</wf>, <pos>n.</pos></wordforms><br/
[<source>1913 Webster</source>]</p>

<p><ent>Taxiarch</ent><br/
<hw>Tax"i*arch</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Gr. <?/ and <?/; <?/ a division of an army, a brigade (from <?/ to arrange, array) + to rule.]</ety> <fld>(Gr. Antiq.)</fld> <def>An Athenian military officer commanding a certain division of an army.</def>  <rj><au>Milford.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Taxicorn</ent><br/
<hw>Tax"i*corn</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>taxus</ets> a yew + <ets>cornu</ets> a horn: cf. F. <ets>taxicorne</ets>.]</ety> <fld>(Zool.)</fld> <def>One of a family of beetles (<fam>Taxicornes</fam>) whose antennae are largest at the tip. Also used adjectively.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Taxidermic</ent><br/
<hw>Tax`i*der"mic</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[Cf. F. <ets>taxidermique</ets>.]</ety> <def>Of or pertaining to the art of preparing and preserving the skins of animals.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Taxidermist</ent><br/
<hw>Tax"i*der`mist</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A person skilled in taxidermy.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Taxidermy</ent><br/
<hw>Tax"i*der`my</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Gr. <?/ an arranging, arrangement (fr. <?/ to arrange) + <?/ a skin, from <?/ to skin: cf. F. <ets>taxidermie</ets>. See <er>Tactics</er>, <er>Tear</er>, <pos>v. t.</pos>]</ety> <def>The art of preparing, preserving, and mounting the skins of animals so as to represent their natural appearance, as for cabinets.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>taximeter</ent><br/
<hw>taximeter</hw> <pos>n.</pos> <sn>1.</sn> <def>a meter in a taxi that registers the fare (based on the length of the ride).</def><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Taxine</ent><br/
<hw>Tax"ine</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>taxus</ets> a yew.]</ety> <fld>(Chem.)</fld> <def>A poisonous alkaloid of bitter taste extracted from the leaves and seeds of the European yew (<spn>Taxus baccata</spn>).  Called also <altname>taxia</altname>.  The usual preparation is a mixture of compounds.  Taxine A has the chemical formula: <chform>C35H47NO10</chform>.</def><br/
[<source>1913 Webster</source> <source>+PJC</source>]</p>

<p><ent>Taxis</ent><br/
||<hw>Tax"is</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[NL., fr. Gr. <grk>ta`xis</grk> a division or arrangement, fr. <grk>ta`ssein</grk> to arrange.]</ety> <sn>1.</sn> <fld>(Surg.)</fld> <def>Manipulation applied to a hernial tumor, or to an intestinal obstruction, for the purpose of reducing it.</def>  <rj><au>Dunglison.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>In technical uses, as in architecture, biology, grammar, etc., arrangement; order; ordonnance.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><sn>3.</sn> <def>a reflexive movement by a motile organism by which it moves or orients itself in relation to some source of stimulation; <as>as, chemo<ex>taxis</ex>, the motion toward or away from gradients of certain chemical compounds</as>.</def><br/
[<source>PJC</source>]</p>

<p><ent>-taxis</ent><br/
<hw>-tax"is</hw> <pr>(?)</pr>, <pos>n.</pos> <def>combining form for <er>taxis{3}</er>.</def><br/
[<source>PJC</source>]</p>

<p><ent>taxi</ent><br/
<hw>tax"i</hw> <pr>(?)</pr>, <pos>n.</pos> <sn>1.</sn> <def>same as <er>taxicab</er>.</def><br/
[<source>PJC</source>]</p>

<p><sn>2.</sn> <def>any vehicle that carries passengers for a fare, as a water <ex>taxi</ex>.</def><br/
[<source>PJC</source>]</p>

<p><ent>taxicab</ent><br/
<hw>tax"i*cab</hw> <pr>(?)</pr>, <pos>n.</pos> <def>an automobile with a professional driver which can be hired to carry passengers; -- also called a <altname>taxi</altname>, and informally called a <altname>cab</altname> or a <altname>hack</altname>.  The driver of a taxicab is referred to as a <er>cab driver</er> or <er>cabbie</er>, and sometimes as a <er>chauffeur</er> or <er>hackie</er>.</def> <note>Taxicabs may be engaged by a prior appointment made, e.g. by telephone, or they may <perf>cruise</perf> for passengers, i.e. they may drive in city streets and stop to <perf>pick up pasengers</perf> when they are signalled by a prospective passenger.  The act of signalling a taxicab (usually by a wave of the arm) is often called <col><b>to hail a cab</b></col> or <col><b>to flag down a cab</b></col>.</note><br/
[<source>PJC</source>]</p>

<p><ent>Taxless</ent><br/
<hw>Tax"less</hw>, <pos>a.</pos> <def>Free from taxation.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Taxology</ent><br/
<hw>Tax*ol"o*gy</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Gr. <grk>ta`xis</grk> arrangement + <ets>-logy</ets>.]</ety> <fld>(Biol.)</fld> <def>Same as <er>Taxonomy</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Taxon</ent><br/
<hw>Tax"on</hw> <pr>(t<acr/ks"<ocr/n)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>taxa</plw> or <plw>taxons</plw>.</plu> <def>a taxonomic group, or the name of a taxonomic grouping.</def><br/
[<source>PJC</source>]</p>

<p><ent>Taxonomic</ent><br/
<hw>Tax`o*nom"ic</hw> <pr>(t<acr/ks`<ocr/*n<ocr/m"<icr/k)</pr>, <pos>a.</pos> <def>Pertaining to, or involving, taxonomy, or the laws and principles of classification; classificatory.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Taxonomist</ent><br/
<hw>Tax*on"o*mist</hw> <pr>(t<acr/ks*<ocr/n"<osl/*m<icr/st)</pr>, <pos>n.</pos> <def>One skilled in taxonomy.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Taxonomy</ent><br/
<hw>Tax*on"o*my</hw> <pr>(t<acr/ks*<ocr/n"<osl/*m<ycr/)</pr>, <pos>n.</pos> <ety>[Gr. <grk>ta`xis</grk> an arrangement, order + <grk>no`mos</grk> a law.]</ety> <sn>1.</sn> <def>That division of the natural sciences which treats of the classification of animals and plants, primarily by consideration of their natural relationships with respect to their structure or genetic origin; the laws or principles of classification; systematics.</def><br/
[<source>1913 Webster</source> <source>+PJC</source>]</p>

<p><sn>2.</sn> <def>A systematic arrangement of objects or concepts showing the relations between them, especially one including a hierarchical arrangement of types in which categories of objects are classified as subtypes of more abstract categories, starting from one or a small number of top categories, and descending to more specific types through an arbitrary number of levels.  An <partof><er>ontology</er></partof> usually contains a <ex>taxonomy</ex> as one of the important principles of organization.</def><br/
[<source>1913 Webster</source> <source>+PJC</source>]</p>

<p><ent>Taxor</ent><br/
<hw>Tax"or</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[NL.]</ety> <def>Same as <er>Taxer</er>, <pos>n.</pos>, 2.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Taxpayer</ent><br/
<hw>Tax"pay`er</hw> <pr>(?)</pr>, <pos>n.</pos> <def>One who is assessed and pays a tax.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Taylor-White process</ent><br/
<hw>Tay"lor-White" proc`ess</hw>. <fld>(Metal.)</fld> <def>A process (invented about 1899 by <person>Frederick W. Taylor</person> and <person>Maunsel B. White</person>) for giving toughness to self-hardening steels. The steel is heated almost to fusion, cooled to a temperature of from 700<deg/ to 850<deg/ C. in molten lead, further cooled in oil, reheated to between 370<deg/ and 670<deg/ C., and cooled in air.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Tayra</ent><br/
<hw>Tay"ra</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[From the native name.]</ety> <fld>(Zool.)</fld> <def>A South American carnivore (<spn>Galera barbara</spn>) allied to the grison. The tail is long and thick. The length, including the tail, is about three feet.</def> <altsp>[Written also <asp>taira</asp>.]</altsp><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tay-Sachs Disease</ent><br/
<hw>Tay"-Sachs Disease</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Med.)</fld> <def>A rare hereditary disease affecting lipid metabolism in humans, due to a deficiency of hexosaminidase.  It occurs in infants and children, and causes death before the onset of adulthood.  It occurs most commonly of people of Jewish origin from easter Europe.  It is characterized by accumulation of lipids in nervous tissue, causes a red spot on the retina, and eventual blindness and paralysis before death.</def><br/
[<source>PJC</source>]</p>

<p><ent>Tazel</ent><br/
<hw>Ta"zel</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Bot.)</fld> <def>The teasel.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tazza</ent><br/
||<hw>Taz"za</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[It.]</ety> <def>An ornamental cup or vase with a large, flat, shallow bowl, resting on a pedestal and often having handles.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>T cart</ent><br/
<hw>T" cart`</hw> <pr>(?)</pr>. <def>See under <er>T</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>T cell</ent><br/
<hw>T" cell`</hw> <pr>(t<emac/"s<ecr/l`)</pr> <ety>[From <ets>T</ets>hymus, the site of maturation of T-cells.]</ety> <def>A type of white blood cell that circulates in the blood and lymph, and provides cell-mediated immunity for the organism, protecting against infecting cells or the body's own malignant cells; also called <altname>T lymphocyte</altname>.  There are several types of T cells.  They develop, as do B cells, from progenitor cells in the bone marrow, but are distinguished from B-cells (B-lymphocytes) by their site of differentiation; T-cells mature in the thymus and B-cells in the bone marrow (in birds in the Bursa of Fabricius).  They also have different antigen receptors from those of B-cells. T-cells differentiate into cells that can directly kill infecting cells (cell-mediated immunity, cytotoxity) or activate other cells of the immune system (helper T cells), whereas B-cells differentiate on activation into antibody-secreting plasma cells.   Helper T cells interact with B-cells by secreting lymphokines that stimulate the B cell which have detected a foreign antigen to enter the cell cycle and develop, by repeated mitosis, into a clone of cells with identical receptors, and then to secrete antibodies to that specific antigen.</def><br/
[<source>PJC</source>]</p>

<p><ent>Tchawytcha</ent><br/
||<hw>Tcha*wy"tcha</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Zool.)</fld> <def>The quinnat salmon.</def> <mark>[Local, U. S.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tchick</ent><br/
<hw>Tchick</hw> <pr>(ch<icr/k)</pr>, <pos>n.</pos> <ety>[Imitative.]</ety> <def>A slight sound such as that made by pressing the tongue against the roof of the mouth and explosively sucking out the air at one side, as in urging on a horse.</def> -- <def2><pos>v. i.</pos> <def>To make a tchick.</def></def2><-- = cluck? click? --><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>tchotchke</ent><br/
<hw>tchotch"ke</hw> <pos>n.</pos> <sn>1.</sn> <ety>[Yiddish <ets>tshatshke</ets>, trinket, from Pol. <ets>czaczko</ets>. <au>MW10.</au>]</ety> <def>a knickknack or trinket; a decorative item or souvenir of little value.</def> <altsp>[Also spelled <asp>chotchke</asp>.]</altsp><br/
[<source>PJC</source>]</p>

<p><ent>T connection</ent><br/
<hw>T connection</hw>. <fld>(Elec.)</fld> <def>The connection of two coils diagrammatically as a letter <universbold>T</universbold>, chiefly used as a connection for passing transformers. When the three free ends are connected to a source of three-phase current, two-phase current may be derived from the secondary circuits. The reverse arrangement may be used to transform from two-phase.</def> -- <wordforms><wf>T"-connected</wf>, <pos>a.</pos></wordforms><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Tea</ent><br/
<hw>Tea</hw> <pr>(t<emac/)</pr>, <pos>n.</pos> <ety>[Chin. <ets>tsh<amac/</ets>, Prov. Chin. <ets>te</ets>: cf. F. <ets>th<eacute/</ets>.]</ety> <sn>1.</sn> <def>The prepared leaves of a shrub, or small tree (<spn>Thea Chinensis</spn> <it>or</it>  <spn>Camellia Chinensis</spn>). The shrub is a native of China, but has been introduced to some extent into some other countries.</def><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ <xex>Teas</xex> are classed as <xex>green</xex> or <xex>black</xex>, according to their color or appearance, the kinds being distinguished also by various other characteristic differences, as of taste, odor, and the like. The color, flavor, and quality are dependent upon the treatment which the leaves receive after being gathered. The leaves for green tea are heated, or roasted slightly, in shallow pans over a wood fire, almost immediately after being gathered, after which they are rolled with the hands upon a table, to free them from a portion of their moisture, and to twist them, and are then quickly dried. Those intended for black tea are spread out in the air for some time after being gathered, and then tossed about with the hands until they become soft and flaccid, when they are roasted for a few minutes, and rolled, and having then been exposed to the air for a few hours in a soft and moist state, are finally dried slowly over a charcoal fire. The operation of roasting and rolling is sometimes repeated several times, until the leaves have become of the proper color. The principal sorts of green tea are Twankay, the poorest kind; Hyson skin, the refuse of Hyson; Hyson, Imperial, and Gunpowder, fine varieties; and Young Hyson, a choice kind made from young leaves gathered early in the spring. Those of black tea are Bohea, the poorest kind; Congou; Oolong; Souchong, one of the finest varieties; and Pekoe, a fine-flavored kind, made chiefly from young spring buds. See <er>Bohea</er>, <er>Congou</er>, <cref>Gunpowder tea</cref>, under <er>Gunpowder</er>, <er>Hyson</er>, <er>Oolong</er>, and <er>Souchong</er>.</note>  <rj><au>K. Johnson.</au>  <au>Tomlinson.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ <ldquo/No knowledge of . . . [tea] appears to have reached Europe till after the establishment of intercourse between Portugal and China in 1517. The Portuguese, however, did little towards the introduction of the herb into Europe, and it was not till the Dutch established themselves at Bantam early in 17th century, that these adventurers learned from the Chinese the habit of tea drinking, and brought it to Europe.<rdquo/</note>  <rj><au>Encyc. Brit.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A decoction or infusion of tea leaves in boiling water; <as>as, <ex>tea</ex> is a common beverage</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>Any infusion or decoction, especially when made of the dried leaves of plants; <as>as, sage <ex>tea</ex>; chamomile <ex>tea</ex>; catnip <ex>tea</ex>.</as></def><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>The evening meal, at which tea is usually served; supper.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Arabian tea</b></col>, <cd>the leaves of <spn>Catha edulis</spn>; also <fld>(Bot.)</fld>, the plant itself. See <er>Kat</er>.</cd> -- <col><b>Assam tea</b></col>, <cd>tea grown in Assam, in India, originally brought there from China about the year 1850.</cd> -- <mcol><col><b>Australian tea</b></col>, <it>or</it> <col><b>Botany Bay tea</b></col></mcol> <fld>(Bot.)</fld>, <cd>a woody climbing plant (<spn>Smilax glycyphylla</spn>).</cd> -- <col><b>Brazilian tea</b></col>. <sd>(a)</sd> <cd>The dried leaves of <spn>Lantana pseodothea</spn>, used in Brazil as a substitute for tea.</cd> <sd>(b)</sd> <cd>The dried leaves of <spn>Stachytarpheta mutabilis</spn>, used for adulterating tea, and also, in <country>Austria</country>, for preparing a beverage.</cd> -- <col><b>Labrador tea</b></col>. <fld>(Bot.)</fld> <cd>See under <er>Labrador</er>.</cd> -- <col><b>New Jersey tea</b></col> <fld>(Bot.)</fld>, <cd>an American shrub, the leaves of which were formerly used as a substitute for tea; redroot. See <er>Redroot</er>.</cd> -- <col><b>New Zealand tea</b></col>. <fld>(Bot.)</fld> <cd>See under <er>New Zealand</er>.</cd> -- <col><b>Oswego tea</b></col>. <fld>(Bot.)</fld> <cd>See <er>Oswego tea</er>.</cd> -- <col><b>Paraguay tea</b></col>, <cd>mate. See 1st <er>Mate</er>.</cd> -- <col><b>Tea board</b></col>, <cd>a board or tray for holding a tea set.</cd> -- <col><b>Tea bug</b></col> <fld>(Zool.)</fld>, <cd>an hemipterous insect which injures the tea plant by sucking the juice of the tender leaves.</cd> -- <col><b>Tea caddy</b></col>, <cd>a small box for holding tea.</cd> -- <col><b>Tea chest</b></col>, <cd>a small, square wooden case, usually lined with sheet lead or tin, in which tea is imported from China.</cd> -- <col><b>Tea clam</b></col> <fld>(Zool.)</fld>, <cd>a small quahaug.</cd> <mark>[Local, U. S.]</mark> -- <col><b>Tea garden</b></col>, <cd>a public garden where tea and other refreshments are served.</cd> -- <col><b>Tea plant</b></col> <fld>(Bot.)</fld>, <cd>any plant, the leaves of which are used in making a beverage by infusion; specifically, <spn>Thea Chinensis</spn>, from which the tea of commerce is obtained.</cd> -- <col><b>Tea rose</b></col> <fld>(Bot.)</fld>, <cd>a delicate and graceful variety of the rose (<spn>Rosa Indica</spn>, var. <varn>odorata</varn>), introduced from China, and so named from its scent. Many varieties are now cultivated.</cd> -- <col><b>Tea service</b></col>, <cd>the appurtenances or utensils required for a tea table, -- when of silver, usually comprising only the teapot, milk pitcher, and sugar dish.</cd> -- <col><b>Tea set</b></col>, <cd>a tea service.</cd> -- <col><b>Tea table</b></col>, <cd>a table on which tea furniture is set, or at which tea is drunk.</cd> -- <col><b>Tea taster</b></col>, <cd>one who tests or ascertains the quality of tea by tasting.</cd> -- <col><b>Tea tree</b></col> <fld>(Bot.)</fld>, <cd>the tea plant of China. See <spn>Tea plant</spn>, above.</cd><-- In Australia and New Zealand, tea tree refers to a tree or tall shrib, Leptospermum scoparium, having white bell-shaped flowers.  The leaves are used to prepare an infusion; an oil, tea tree oil, is also derived, and claimed to have therapeutic properties, as for healing burns of the skin. --> -- <col><b>Tea urn</b></col>, <cd>a vessel generally in the form of an urn or vase, for supplying hot water for steeping, or infusing, tea.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tea</ent><br/
<hw>Tea</hw>, <pos>v. i.</pos> <def>To take or drink tea.</def> <mark>[Colloq.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Teaberry</ent><br/
<hw>Tea"ber`ry</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Bot.)</fld> <def>The checkerberry.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Teach</ent><br/
<hw>Teach</hw> <pr>(t<emac/ch)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Taught</conjf> <pr>(t<add/t)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Teaching</conjf>.]</vmorph> <ety>[OE. <ets>techen</ets>, imp. <ets>taughte</ets>, <ets>tahte</ets>, AS. <ets>t<aemac/cean</ets>, imp. <ets>t<aemac/hte</ets>, to show, teach, akin to <ets>t<amac/cn</ets> token. See <er>Token</er>.]</ety> <sn>1.</sn> <def>To impart the knowledge of; to give intelligence concerning; to impart, as knowledge before unknown, or rules for practice; to inculcate as true or important; to exhibit impressively; <as>as, to <ex>teach</ex> arithmetic, dancing, music, or the like; to <ex>teach</ex> morals.</as></def><br/
[<source>1913 Webster</source>]</p>

<p><q>If some men <qex>teach</qex> wicked things, it must be that others should practice them.</q> <rj><qau>South.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To direct, as an instructor; to manage, as a preceptor; to guide the studies of; to instruct; to inform; to conduct through a course of studies; <as>as, to <ex>teach</ex> a child or a class</as>.</def> <ldquo/He <xex>taught</xex> his disciples.<rdquo/  <rj><au>Mark ix. 31.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>The village master <qex>taught</qex> his little school.</q> <rj><qau>Goldsmith.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>To accustom; to guide; to show; to admonish.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>I shall myself to herbs <qex>teach</qex> you.</q> <rj><qau>Chaucer.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>They have <qex>taught</qex> their tongue to speak lies.</q> <rj><qau>Jer. ix. 5.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ This verb is often used with two objects, one of the person, the other of the thing; <as>as, he <ex>taught</ex> me Latin grammar</as>. In the passive construction, either of these objects may be retained in the objective case, while the other becomes the subject; <as>as, I was <ex>taught</ex> Latin grammar by him; Latin grammar was <ex>taught</ex> me by him</as>.</note><br/
[<source>1913 Webster</source>]</p>

<p><syn><b>Syn.</b> -- To instruct; inform; inculcate; tell; guide; counsel; admonish. See the Note under <er>Learn</er>.</syn><br/
[<source>1913 Webster</source>]</p>

<p><ent>Teach</ent><br/
<hw>Teach</hw> <pr>(?)</pr>, <pos>v. i.</pos> <def>To give instruction; to follow the business, or to perform the duties, of a preceptor.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>And gladly would he learn, and gladly <qex>teach</qex>.</q> <rj><qau>Chaucer.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>The priests thereof <qex>teach</qex> for hire.</q> <rj><qau>Micah iii. 11.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Teachable</ent><br/
<hw>Teach"a*ble</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Capable of being taught; apt to learn; also, willing to receive instruction; docile.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>We ought to bring our minds free, unbiased, and <qex>teachable</qex>, to learn our religion from the Word of God.</q> <rj><qau>I. Watts.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Teachableness</ent><br/
<hw>Teach"a*ble*ness</hw>, <pos>n.</pos> <def>Willingness to be taught.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Teache</ent><br/
<hw>Teache</hw> <pr>(t<emac/ch)</pr>, <pos>n.</pos> <ety>[Cf. Amer. Sp. <ets>tacha</ets>, <ets>tacho</ets>.]</ety> <fld>(Sugar Manuf.)</fld> <def>Any one of the series of boilers or evaporating pans in which the cane juice is concentrated in making sugar; especially, the last boiler of the series.</def>  <rj><au>Ure.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><note>The 1890 edition had the following etymology for <ex>teache</ex>: <ety>[Cf. Ir. <ets>teaghaim</ets>, Gael. <ets>teasaich</ets>, to heat.]</ety>.  Presumably a speculation which proved incorrect?</note></p>

<p><ent>Teacher</ent><br/
<hw>Teach"er</hw> <pr>(?)</pr>, <pos>n.</pos> <sn>1.</sn> <def>One who teaches or instructs; one whose business or occupation is to instruct others; an instructor; a tutor.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>One who instructs others in religion; a preacher; a minister of the gospel; sometimes, one who preaches without regular ordination.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The <qex>teachers</qex> in all the churches assembled.</q> <rj><qau>Sir W. Raleigh.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Teaching</ent><br/
<hw>Teach"ing</hw>, <pos>n.</pos> <def>The act or business of instructing; also, that which is taught; instruction.</def><br/
[<source>1913 Webster</source>]</p>

<p><syn><b>Syn.</b> -- Education; instruction; breeding. See <er>Education</er>.</syn><br/
[<source>1913 Webster</source>]</p>

<p><ent>Teachless</ent><br/
<hw>Teach"less</hw>, <pos>a.</pos> <def>Not teachable.</def> <mark>[R.]</mark>  <rj><au>Shelley.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Teacup</ent><br/
<hw>Tea"cup`</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A small cup from which to drink tea.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Teacupful</ent><br/
<hw>Tea"cup`ful</hw> <pr>(?)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Teacupfuls</plw> <pr>(<?/)</pr>.</plu> <def>As much as a teacup can hold; enough to fill a teacup.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Teade</ent><br/
<ent>Tead</ent><br/
<mhw>{ <hw>Tead</hw>, <hw>Teade</hw>  }</mhw> <pr>(t<emac/d)</pr>, <pos>n.</pos> <ety>[L. <ets>taeda</ets>, <ets>teda</ets>.]</ety> <def>A torch.</def> <mark>[Obs.]</mark> <ldquo/A burning <xex>teade</xex>.<rdquo/  <rj><au>Spenser.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Teagle</ent><br/
<hw>Tea"gle</hw> <pr>(t<emac/"g'l)</pr>, <pos>n.</pos> <ety>[Cf. <er>Tackle</er>.]</ety> <def>A hoisting apparatus; an elevator; a crane; a lift.</def> <mark>[Prov. Eng.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Teague</ent><br/
<hw>Teague</hw> <pr>(t<emac/g)</pr>, <pos>n.</pos> <ety>[Cf. W. <ets>taeog</ets>, <ets>taeawg</ets>, <ets>taiawg</ets>, adj., rustic, rude, n., a vassal, villain, peasant, clown, Ir. <ets>thuatach</ets> rural, boorish.]</ety> <def>An Irishman; -- a term used in contempt.</def>  <rj><au>Johnson.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Teak</ent><br/
<hw>Teak</hw> <pr>(t<emac/k)</pr>, <pos>n.</pos> <ety>[Malayalam <ets>tekku</ets>.]</ety> <fld>(Bot.)</fld> <def>A tree of East Indies (<spn>Tectona grandis</spn>) which furnishes an extremely strong and durable timber highly valued for shipbuilding and other purposes; also, the timber of the tree.</def> <altsp>[Written also <asp>teek</asp>.]</altsp><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>African teak</b></col>, <cd>a tree (<spn>Oldfieldia Africana</spn>) of Sierra Leone; also, its very heavy and durable wood; -- called also <altname>African oak</altname>.</cd> -- <col><b>New Zeland teak</b></col>, <cd>a large tree (<spn>Vitex littoralis</spn>) of New Zeland; also, its hard, durable timber.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Teakettle</ent><br/
<hw>Tea"ket`tle</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A kettle in which water is boiled for making tea, coffee, etc.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Teal</ent><br/
<hw>Teal</hw> <pr>(t<emac/l)</pr>, <pos>n.</pos> <ety>[OE. <ets>tele</ets>; akin to D. <ets>teling</ets> a generation, production, teal, <ets>telen</ets> to breed, produce, and E. <ets>till</ets> to cultivate. The English word probably once meant, a brood or flock. See <er>Till</er> to cultivate.]</ety> <fld>(Zool.)</fld> <def>Any one of several species of small fresh-water ducks of the genus <gen>Anas</gen> and the subgenera <gen>Querquedula</gen> and <gen>Nettion</gen>. The male is handsomely colored, and has a bright green or blue speculum on the wings.</def><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ The common European teal (<spn>Anas crecca</spn>) and the European blue-winged teal, or garganey (<spn>Anas querquedula</spn> or <spn>Anas circia</spn>), are well-known species. In America the blue-winged teal (<spn>Anas discors</spn>), the green-winged teal (<spn>Anas Carolinensis</spn>), and the cinnamon teal (<spn>Anas cyanoptera</spn>) are common species, valued as game birds. See <er>Garganey</er>.</note><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Goose teal</b></col>, <cd>a goslet. See <er>Goslet</er>.</cd> -- <col><b>Teal duck</b></col>, <cd>the common European teal.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Team</ent><br/
<hw>Team</hw> <pr>(t<emac/m)</pr>, <pos>n.</pos> <ety>[OE. <ets>tem</ets>, <ets>team</ets>, AS. <ets>te<aacute/m</ets>, offspring, progeny, race of descendants, family; akin to D. <ets>toom</ets> a bridle, LG. <ets>toom</ets> progeny, team, bridle, G. <ets>zaum</ets> a bridle, <ets>zeugen</ets> to beget, Icel. <ets>taumr</ets> a rein, bridle, Dan. <ets>t<oum/mme</ets>, Sw. <ets>t<oum/m</ets>, and also to E. <ets>tow</ets> to drag, <ets>tug</ets> to draw.  <root/64.  See <er>Tug</er>, and cf. <er>Teem</er> to bear.]</ety> <sn>1.</sn> <def>A group of young animals, especially of young ducks; a brood; a litter.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>A <qex>team</qex> of ducklings about her.</q> <rj><qau>Holland.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Hence, a number of animals moving together.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>A long <qex>team</qex> of snowy swans on high.</q> <rj><qau>Dryden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>Two or more horses, oxen, or other beasts harnessed to the same vehicle for drawing, as to a coach, wagon, sled, or the like.</def> <ldquo/A <xex>team</xex> of dolphins.<rdquo/  <rj><au>Spenser.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>To take his <qex>team</qex> and till the earth.</q> <rj><qau>Piers Plowman.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>It happened almost every day that coaches stuck fast, until a <qex>team</qex> of cattle could be procured from some neighboring farm to tug them out of the slough.</q> <rj><qau>Macaulay.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>A number of persons associated together in any work; a gang; especially, a number of persons selected to contend on one side in a match, or a series of matches, in a cricket, football, rowing, etc.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <fld>(Zool.)</fld> <def>A flock of wild ducks.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>6.</sn> <fld>(O. Eng. Law)</fld> <def>A royalty or privilege granted by royal charter to a lord of a manor, of having, keeping, and judging in his court, his bondmen, neifes, and villains, and their offspring, or suit, that is, goods and chattels, and appurtenances thereto.</def>  <rj><au>Burrill.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><-- p. 1479 --><br/
[<source>1913 Webster</source>]</p>

<p><ent>Team</ent><br/
<hw>Team</hw> <pr>(?)</pr>, <pos>v. i.</pos> <def>To engage in the occupation of driving a team of horses, cattle, or the like, as in conveying or hauling lumber, goods, etc.; to be a teamster.</def><br/
[<source>1913 Webster</source>]</p>

<p><-- <col><b>team up</b></col>, <cd>to form one or more teams, either for a common endeavor, or to compete in a contest.</cd> --><br/
[<source>1913 Webster</source>]</p>

<p><ent>Team</ent><br/
<hw>Team</hw>, <pos>v. t.</pos> <def>To convey or haul with a team; <as>as, to <ex>team</ex> lumber</as>.</def> <mark>[R.]</mark>  <rj><au>Thoreau.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Teamed</ent><br/
<hw>Teamed</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Yoked in, or as in, a team.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>Let their <qex>teamed</qex> fishes softly swim.</q> <rj><qau>Spenser.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Teaming</ent><br/
<hw>Team"ing</hw> <pr>(?)</pr>, <pos>n.</pos> <sn>1.</sn> <def>The act or occupation of driving a team, or of hauling or carrying, as logs, goods, or the like, with a team.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Manuf.)</fld> <def>Contract work.</def> <mark>[R.]</mark>  <rj><au>Knight.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Teamster</ent><br/
<hw>Team"ster</hw> <pr>(?)</pr>, <pos>n.</pos> <def>One who drives a team.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Teamwork</ent><br/
<hw>Team"work`</hw> <pr>(?)</pr>, <pos>n.</pos> <sn>1.</sn> <def>Work done by a team, as distinguished from that done by personal labor.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Work done by a number of associates, usually each doing a clearly defined portion, but all subordinating personal prominence to the efficiency of the whole; <as>as, the <ex>teamwork</ex> of a football eleven or a gun crew</as>.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><q>Is the <qex>teamwork</qex> system employed, or does one workman make the whole cigar?</q>  <rj><qau>U. S. Consular Repts.</qau></rj><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Teapot</ent><br/
<hw>Tea"pot`</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A vessel with a spout, in which tea is made, and from which it is poured into teacups.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Teapoy</ent><br/
<hw>Tea"poy</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Hind. <ets>tip<amac/i</ets>; Hind. <ets>tin</ets> there + Per. <ets>p<amac/e</ets> foot.]</ety> <def>An ornamental stand, usually with three legs, having caddies for holding tea.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tear</ent><br/
<hw>Tear</hw> <pr>(t<emac/r)</pr>, <pos>n.</pos> <ety>[AS. <ets>te<aacute/r</ets>; akin to G. <ets>z<aum/rhe</ets>, OHG. <ets>zahar</ets>, OFries. & Icel. <ets>t<amac/r</ets>, Sw. <ets>t<aring/r</ets>, Dan. <ets>taare</ets>, Goth. <ets>tagr</ets>, OIr. <ets>d<emac/r</ets>, W. <ets>dagr</ets>, OW. <ets>dacr</ets>, L. <ets>lacrima</ets>, <ets>lacruma</ets>, for older <ets>dacruma</ets>, Gr. <grk>da`kry</grk>, <grk>da`kryon</grk>, <grk>da`kryma</grk>. <root/59.  Cf. <er>Lachrymose</er>.]</ety> <sn>1.</sn> <fld>(Physiol.)</fld> <def>A drop of the limpid, saline fluid secreted, normally in small amount, by the lachrymal gland, and diffused between the eye and the eyelids to moisten the parts and facilitate their motion. Ordinarily the secretion passes through the lachrymal duct into the nose, but when it is increased by emotion or other causes, it overflows the lids.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>And yet for thee ne wept she never a <qex>tear</qex>.</q> <rj><qau>Chaucer.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Something in the form of a transparent drop of fluid matter; also, a solid, transparent, tear-shaped drop, as of some balsams or resins.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Let Araby extol her happy coast,<br/
Her fragrant flowers, her trees with precious <qex>tears</qex>.</q> <rj><qau>Dryden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>That which causes or accompanies tears; a lament; a dirge.</def> <mark>[R.]</mark> <ldquo/Some melodous <xex>tear</xex>.<rdquo/  <rj><au>Milton.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <fld>(Glass Manuf.)</fld> <def>A partially vitrified bit of clay in glass.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><note><hand/ <xex>Tear</xex> is sometimes used in the formation of self-explaining compounds; as, <xex>tear</xex>-distilling, <xex>tear</xex>-drop, <xex>tear</xex>-filled, <xex>tear</xex>-stained, and the like.</note><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Tears of St. Lawrence</b></col>, <cd>the Perseid shower of meteors, seen every year on or about the eve of <person>St. Lawrence</person>, August 9th.</cd> -- <col><b>Tears of wine</b></col>, <cd>drops which form and roll down a glass above the surface of strong wine. The phenomenon is due to the evaporation of alcohol from the surface layer, which, becoming more watery, increases in surface tension and creeps up the sides until its weight causes it to break.</cd></cs><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Tear</ent><br/
<hw>Tear</hw> <pr>(t<acir/r)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp.</pos> <conjf>Tore</conjf> <pr>(t<omac/r)</pr>, ((<mark>Obs.</mark> <conjf>Tare</conjf>) <pr>(t<acir/r)</pr>; <pos>p. p.</pos> <conjf>Torn</conjf> <pr>(t<omac/rn)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Tearing</conjf>.]</vmorph> <ety>[OE. <ets>teren</ets>, AS. <ets>teran</ets>; akin to OS. far<ets>terian</ets> to destroy, D. <ets>teren</ets> to consume, G. <ets>zerren</ets> to pull, to tear, <ets>zehren</ets> to consume, Icel. <ets>t>ae/ra</ets>, Goth. <ets>gata<iacute/ran</ets> to destroy, Lith. <ets>dirti</ets> to flay, Russ. <ets>drate</ets> to pull, to tear, Gr. <grk>de`rein</grk> to flay, Skr. <ets>dar</ets> to burst. <root/63.  Cf. <er>Darn</er>, <er>Epidermis</er>, <er>Tarre</er>, <er>Tirade</er>.]</ety> <sn>1.</sn> <def>To separate by violence; to pull apart by force; to rend; to lacerate; <as>as, to <ex>tear</ex> cloth; to <ex>tear</ex> a garment; to <ex>tear</ex> the skin or flesh.</as></def><br/
[<source>1913 Webster</source>]</p>

<p><q><qex>Tear</qex> him to pieces; he's a conspirator.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Hence, to divide by violent measures; to disrupt; to rend; <as>as, a party or government <ex>torn</ex> by factions</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>To rend away; to force away; to remove by force; to sunder; <as>as, a child <ex>torn</ex> from its home</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The hand of fate<br/
Hath <qex>torn</qex> thee from me.</q> <rj><qau>Addison.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>To pull with violence; <as>as, to <ex>tear</ex> the hair</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <def>To move violently; to agitate.</def> <ldquo/Once I loved <xex>torn</xex> ocean's roar.<rdquo/  <rj><au>Byron.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>To tear a cat</b></col>, <cd>to rant violently; to rave; -- especially applied to theatrical ranting.</cd> <mark>[Obs.]</mark> <au>Shak.</au> -- <col><b>To tear down</b></col>, <cd>to demolish violently; to pull or pluck down.</cd> -- <col><b>To tear off</b></col>, <cd>to pull off by violence; to strip.</cd> -- <col><b>To tear out</b></col>, <cd>to pull or draw out by violence; <as>as, <ex>to tear out</ex> the eyes</as>.</cd> -- <col><b>To tear up</b></col>, <cd>to rip up; to remove from a fixed state by violence; <as>as, <ex>to tear up</ex> a floor; <ex>to tear up</ex> the foundation of government or order</as>.</cd></cs><-- tear sheet, (a) a sheet usu. with performations, intended to be torn from a book or booklet to be used for some purpose. (b) any sheet torn from a publication. --><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tear</ent><br/
<hw>Tear</hw> <pr>(?)</pr>, <pos>v. i.</pos> <sn>1.</sn> <def>To divide or separate on being pulled; to be rent; <as>as, this cloth <ex>tears</ex> easily</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To move and act with turbulent violence; to rush with violence; hence, to rage; to rave.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tear</ent><br/
<hw>Tear</hw> <pr>(?)</pr>, <pos>n.</pos> <def>The act of tearing, or the state of being torn; a rent; a fissure.</def>  <rj><au>Macaulay.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Wear and tear</b></col>. <cd>See under <er>Wear</er>, <pos>n.</pos></cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tearer</ent><br/
<hw>Tear"er</hw> <pr>(?)</pr>, <pos>n.</pos> <def>One who tears or rends anything; also, one who rages or raves with violence.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tear-falling</ent><br/
<hw>Tear"-fall`ing</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Shedding tears; tender.</def> <mark>[Poetic]</mark> <ldquo/<xex>Tear-falling</xex> pity.<rdquo/  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tearful</ent><br/
<hw>Tear"ful</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Abounding with tears; weeping; shedding tears; <as>as, <ex>tearful</ex> eyes</as>.</def> -- <wordforms><wf>Tear"ful*ly</wf>, <pos>adv.</pos> -- <wf>Tear"ful*ness</wf>, <pos>n.</pos></wordforms><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tearless</ent><br/
<hw>Tear"less</hw>, <pos>a.</pos> <def>Shedding no tears; free from tears; unfeeling.</def> -- <wordforms><wf>Tear"less*ly</wf>, <pos>adv.</pos> -- <wf>Tear"less*ness</wf>, <pos>n.</pos></wordforms><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tearpit</ent><br/
<hw>Tear"pit`</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Anat.)</fld> <def>A cavity or pouch beneath the lower eyelid of most deer and antelope; the lachrymal sinus; larmier. It is capable of being opened at pleasure and secretes a waxy substance.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tear-thumb</ent><br/
<hw>Tear"-thumb`</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Bot.)</fld> <def>A name given to several species of plants of the genus Polygonum, having angular stems beset with minute reflexed prickles.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Teary</ent><br/
<hw>Tear"y</hw> <pr>(?)</pr>, <pos>a.</pos> <sn>1.</sn> <def>Wet with tears; tearful.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Consisting of tears, or drops like tears.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tea-saucer</ent><br/
<hw>Tea"-sau`cer</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A small saucer in which a teacup is set.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tease</ent><br/
<hw>Tease</hw> <pr>(t<emac/z)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Teased</conjf> <pr>(t<emac/zd)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Teasing</conjf>.]</vmorph> <ety>[AS. <ets>t<?/san</ets> to pluck, tease; akin to OD. <ets>teesen</ets>, MHG. <ets>zeisen</ets>, Dan. <ets>t<ae/se</ets>, <ets>t<ae/sse</ets>. <root/58.  Cf. <er>Touse</er>.]</ety> <sn>1.</sn> <def>To comb or card, as wool or flax.</def> <ldquo/<xex>Teasing</xex> matted wool.<rdquo/  <rj><au>Wordsworth.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To stratch, as cloth, for the purpose of raising a nap; teasel.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <fld>(Anat.)</fld> <def>To tear or separate into minute shreds, as with needles or similar instruments.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>To vex with importunity or impertinence; to harass, annoy, disturb, or irritate by petty requests, or by jests and raillery; to plague.</def>  <rj><au>Cowper.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>He . . . suffered them to <qex>tease</qex> him into acts directly opposed to his strongest inclinations.</q> <rj><qau>Macaulay.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><syn><b>Syn.</b> -- To vex; harass: annoy; disturb; irritate; plague; torment; mortify; tantalize; chagrin.</syn> <usage> -- <er>Tease</er>, <er>Vex</er>. To <xex>tease</xex> is literally to pull or scratch, and implies a prolonged annoyance in respect to little things, which is often more irritating, and harder to bear, than severe pain. <xex>Vex</xex> meant originally to seize and bear away hither and thither, and hence, to disturb; <as>as, to <ex>vex</ex> the ocean with storms</as>. This sense of the term now rarely occurs; but <xex>vex</xex> is still a stronger word than <xex>tease</xex>, denoting the disturbance or anger created by minor provocations, losses, disappointments, etc.  We are <xex>teased</xex> by the buzzing of a fly in our eyes; we are <xex>vexed</xex> by the carelessness or stupidity of our servants.</usage><br/
[<source>1913 Webster</source>]</p>

<p><q>Not by the force of carnal reason,<br/
But indefatigable <qex>teasing</qex>.</q> <rj><qau>Hudibras.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>In disappointments, where the affections have been strongly placed, and the expectations sanguine, particularly where the agency of others is concerned, sorrow may degenerate into <qex>vexation</qex> and chagrin.</q> <rj><qau>Cogan.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Tease tenon</b></col> <fld>(Joinery)</fld>, <cd>a long tenon at the top of a post to receive two beams crossing each other one above the other.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tease</ent><br/
<hw>Tease</hw> <pr>(?)</pr>, <pos>n.</pos> <def>One who teases or plagues.</def> <mark>[Colloq.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Teasel</ent><br/
<hw>Tea"sel</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[OE. <ets>tesel</ets>, AS. <ets>t<aemac/sel</ets>, <ets>t<aemac/sl</ets>, the fuller's herb. See <er>Tease</er>.]</ety> <altsp>[Written also <asp>tassel</asp>, <asp>tazel</asp>, <asp>teasle</asp>, <asp>teazel</asp>, and <asp>teazle</asp>.]</altsp> <sn>1.</sn> <fld>(Bot.)</fld> <def>A plant of the genus <gen>Dipsacus</gen>, of which one species (<spn>Dipsacus fullonum</spn>) bears a large flower head covered with stiff, prickly, hooked bracts. This flower head, when dried, is used for raising a nap on woolen cloth.</def><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ Small teasel is <spn>Dipsacus pilosus</spn>, wild teasel is <spn>Dipsacus sylvestris</spn>.</note><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A bur of this plant.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>Any contrivance intended as a substitute for teasels in dressing cloth.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Teasel frame</b></col>, <cd>a frame or set of iron bars in which teasel heads are fixed for raising the nap on woolen cloth.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Teasel</ent><br/
<hw>Tea"sel</hw>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Teaseled</conjf> <pr>(?)</pr> or <conjf>Teaselled</conjf>; <pos>p. pr. & vb. n.</pos> <conjf>Teaseling</conjf> or <conjf>Teaselling</conjf>.]</vmorph> <def>To subject, as woolen cloth, to the action of teasels, or any substitute for them which has an effect to raise a nap.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Teaseler</ent><br/
<hw>Tea"sel*er</hw> <pr>(?)</pr>, <pos>n.</pos> <def>One who uses teasels for raising a nap on cloth.</def> <altsp>[Written also <asp>teaseller</asp>, <asp>teasler</asp>.]</altsp><br/
[<source>1913 Webster</source>]</p>

<p><ent>Teaseling</ent><br/
<hw>Tea"sel*ing</hw>, <pos>n.</pos> <def>The cutting and gathering of teasels; the use of teasels.</def> <altsp>[Written also <asp>teaselling</asp>, <asp>teazling</asp>.]</altsp><br/
[<source>1913 Webster</source>]</p>

<p><ent>Teaser</ent><br/
<hw>Teas"er</hw> <pr>(?)</pr>, <pos>n.</pos> <sn>1.</sn> <def>One who teases or vexes.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Zool.)</fld> <def>A jager gull.</def> <mark>[Prov. Eng.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <fld>(Elec.)</fld> <def>A shunt winding on field magnets for maintaining their magnetism when the main circuit is open.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Teasle</ent><br/
<hw>Tea"sle</hw> <pr>(?)</pr>, <pos>n. & v. t.</pos> <def>See <er>Teasel</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Teaspoon</ent><br/
<hw>Tea"spoon`</hw> <pr>(t<emac/"sp<oomac/n`)</pr>, <pos>n.</pos> <sn>1.</sn> <def>A small spoon used in stirring and sipping tea, coffee, etc., and for other purposes.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>same as <er>teaspoonful</er>.</def><br/
[<source>PJC</source>]</p>

<p><ent>Teaspoonful</ent><br/
<hw>Tea"spoon`ful</hw> <pr>(t<emac/"sp<oomac/n`f<usdot/l)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Teaspoonfuls</plw> <pr>(t<emac/"sp<oomac/n`f<usdot/lz)</pr> or <plw>Teaspoonsful</plw>.</plu> <def>As much as teaspoon will hold; enough to fill a teaspoon.  In cooking, it is usually estimated as 4.9 milliliters, which is <frac16/ of a fluid ounce, or <frac13/ of a tablespoonful.  In cooking recipes it may be abbreviated as <abbr>t.</abbr></def> <note>In the 1890 dictionary, it was defined as a fluid dram or one quarter of a tablespoonful.</note><br/
[<source>1913 Webster</source> <source>+PJC</source>]</p>

<p><ent>Teat</ent><br/
<hw>Teat</hw> <pr>(t<emac/t)</pr>, <pos>n.</pos> <ety>[OE. <ets>tete</ets>, <ets>titte</ets>, AS. <ets>tit</ets>, <ets>titt</ets>; akin to LG. & OD. <ets>titte</ets>, D. <ets>tet</ets>, G. <ets>zitze</ets>: cf. F. <ets>tette</ets>, probably of Teutonic origin.]</ety> <sn>1.</sn> <def>The protuberance through which milk is drawn from the udder or breast of a mammal; a nipple; a pap; a mammilla; a dug; a tit.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Mach.)</fld> <def>A small protuberance or nozzle resembling the teat of an animal.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Teated</ent><br/
<hw>Teat"ed</hw>, <pos>a.</pos> <def>Having protuberances resembling the teat of an animal.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Teathe</ent><br/
<hw>Teathe</hw> <pr>(?)</pr>, <pos>n. & v.</pos> <def>See <er>Tath</er>.</def> <mark>[Prov. Eng.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Teatish</ent><br/
<hw>Teat"ish</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Peevish; tettish; fretful; -- said of a child. See <er>Tettish</er>.</def> <mark>[Obs.]</mark>  <rj><au>Beau. & Fl.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Teaze-hole</ent><br/
<hw>Teaze"-hole`</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Corrupted fr. F. <ets>tisard</ets> fire door.]</ety> <fld>(Glass Works)</fld> <def>The opening in the furnaces through which fuel is introduced.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Teazel</ent><br/
<hw>Tea"zel</hw> <pr>(?)</pr>, <pos>n. & v. t.</pos> <def>See <er>Teasel</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Teazer</ent><br/
<hw>Tea"zer</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Corrupted fr. F. <ets>tiser</ets> to feed a fire.]</ety> <def>The stoker or fireman of a furnace, as in glass works.</def>  <rj><au>Tomlinson.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Teazle</ent><br/
<hw>Tea"zle</hw> <pr>(?)</pr>, <pos>n. & v. t.</pos> <def>See <er>Teasel</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tebeth</ent><br/
<hw>Te"beth</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Heb.]</ety> <def>The tenth month of the Jewish ecclesiastical year, answering to a part of December with a part of January.</def>  <rj><au>Esther ii. 16.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Techily</ent><br/
<hw>Tech"i*ly</hw> <pr>(?)</pr>, <pos>adv.</pos> <def>In a techy manner.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Techiness</ent><br/
<hw>Tech"i*ness</hw>, <pos>n.</pos> <def>The quality or state of being techy.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Technic</ent><br/
<hw>Tech"nic</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Technical.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Technic</ent><br/
<hw>Tech"nic</hw>, <pos>n.</pos> <ety>[See <er>Technical</er>, <pos>a.</pos>]</ety> <sn>1.</sn> <def>The method of performance in any art; technical skill; artistic execution; technique.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>They illustrate the method of nature, not the <qex>technic</qex> of a manlike Artificer.</q> <rj><qau>Tyndall.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <pluf>pl.</pluf> <def>Technical terms or objects; things pertaining to the practice of an art or science.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Technical</ent><br/
<hw>Tech"nic*al</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[Gr. <?/, fr. <?/ an art, probably from the same root as <?/, <?/, to bring forth, produce, and perhaps akin to E. <ets>text</ets>: cf. F. <ets>technique</ets>.]</ety> <def>Of or pertaining to the useful or mechanic arts, or to any science, business, or the like; specially appropriate to any art, science, or business; <as>as, the words of an indictment must be <ex>technical</ex></as>.</def>  <rj><au>Blackstone.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Technicality</ent><br/
<hw>Tech`ni*cal"i*ty</hw> <pr>(?)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Technicalities</plw> <pr>(<?/)</pr>.</plu> <sn>1.</sn> <def>The quality or state of being technical; technicalness.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>That which is technical, or peculiar to any trade, profession, sect, or the like.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The <qex>technicalities</qex> of the sect.</q> <rj><qau>Palfrey.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Technically</ent><br/
<hw>Tech"nic*al*ly</hw> <pr>(?)</pr>, <pos>adv.</pos> <def>In a technical manner; according to the signification of terms as used in any art, business, or profession.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Technicalness</ent><br/
<hw>Tech"nic*al*ness</hw>, <pos>n.</pos> <def>The quality or state of being technical; technicality.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Technicals</ent><br/
<hw>Tech"nic*als</hw> <pr>(?)</pr>, <pos>n. pl.</pos> <def>Those things which pertain to the practical part of an art, science, or profession; technical terms; technics.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Technician</ent><br/
<hw>Tech*ni"cian</hw> <pr>(?)</pr>, <pos>n.</pos> <def>a person trained or skilled in the technical details of a particular art or science, especially one skilled at operating, maintaining, or repairing equipment, in contrast to the theory or informational content of a craft; -- formerly also called a <er>technicist</er>.</def> <note>In computer software companies, individuals skilled at the details of using programs and employed to help customers to install or use software or troubleshoot software problems for  are also called <ex>technicians</ex>.</note><br/
[<source>Webster 1913 Suppl.</source> <source>+PJC</source>]</p>

<p><ent>Technicist</ent><br/
<hw>Tech"ni*cist</hw> <pr>(?)</pr>, <pos>n.</pos> <def>One skilled in technics or in one or more of the practical arts.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Technicological</ent><br/
<hw>Tech`ni*co*log"ic*al</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Technological; technical.</def> <mark>[R.]</mark>  <rj><au>Dr. J. Scott.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Technicology</ent><br/
<hw>Tech`ni*col"o*gy</hw> <pr>(?)</pr>, <pos>n.</pos> <def>Technology.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Technicolor</ent><br/
<hw>Tech`ni*col"or</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[a treadmark]</ety> <def>the name of one process used for color cinematography; -- also used attributively.</def> <mark>[trademark]</mark> <br/
[<source>PJC</source>]</p>

<p><ent>Technics</ent><br/
<hw>Tech"nics</hw> <pr>(?)</pr>, <pos>n.</pos> <sn>1.</sn> <def>The doctrine of arts in general; such branches of learning as respect the arts.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>The study of a particular art.</def><br/
[<source>PJC</source>]</p>

<p><ent>Techniphone</ent><br/
<hw>Tech"ni*phone</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Gr. <?/ art + <ets>-phone</ets>.]</ety> <fld>(Music)</fld> <def>A dumb gymnastic apparatus for  training the hands of pianists and organists, as to a legato touch.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Technique</ent><br/
<hw>Tech`nique"</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F.]</ety> <sn>1.</sn> <def>The method or manner of performance in any art; -- also called <altname>technic</altname>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>The body of technical methods and procedures used in a science or craft.</def><br/
[<source>PJC</source>]</p>

<p><sn>3.</sn> <def>the detailed movements used for executing an artistic performance; technical skill; artistic execution; <as>as, a pianist's fingering <ex>technique</ex></as>.</def><br/
[<source>1913 Webster</source> <source>+PJC</source>]</p>

<p><ent>Technism</ent><br/
<hw>Tech"nism</hw> <pr>(?)</pr>, <pos>n.</pos> <def>Technicality.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Techno</ent><br/
<hw>Tech"no</hw> <pr>(?)</pr>, <pos>n.</pos> <def>a form of music primarily created by computer sound synthizers rather than by musicians playing instruments.</def> <mark>[slang]</mark> <br/
[<source>PJC</source>]</p>

<p><ent>Technobabble</ent><br/
<hw>Tech"no*bab`ble</hw> <pr>(?)</pr>, <pos>n.</pos> <def>technical jargon incomprehensible to non-specialists; -- sometimes used derogatorily of discussions using unnecessarily technical terminology and intended to impress or confuse, rather than inform, the listener.</def><br/
[<source>PJC</source>]</p>

<p><ent>technocracy</ent><br/
<hw>tech*noc"ra*cy</hw> <pr>(?)</pr>, <pos>n.</pos> <def>government by technical specialists.</def><br/
[<source>PJC</source>]</p>

<p><ent>technocrat</ent><br/
<hw>tech"no*crat</hw> <pr>(?)</pr>, <pos>n.</pos> <sn>1.</sn> <def>a technical specialist exercising governmental or managerial authority.</def><br/
[<source>PJC</source>]</p>

<p><sn>2.</sn> <def>a proponent of technocracy.</def><br/
[<source>PJC</source>]</p>

<p> -- <wordforms><wf>technocratic</wf>, <pos>a.</pos></wordforms><br/
[<source>PJC</source>]</p>

<p><ent>Technography</ent><br/
<hw>Tech*nog"ra*phy</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Gr. <?/ art, skill, craft + <ets>graph</ets>.]</ety> <def>Description of the arts and  crafts of tribes and peoples.</def> -- <wordforms><wf>Tech`no*graph"ic</wf>, <wf>Tech`no*graph"ic*al</wf> <pr>(#)</pr>, <pos>a.</pos></wordforms><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Technologic</ent><br/
<hw>Tech`no*log"ic</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Technological.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Technological</ent><br/
<hw>Tech`no*log"ic*al</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[Cf. F. <ets>technologique</ets>.]</ety> <def>Of or pertaining to technology.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Technologist</ent><br/
<hw>Tech*nol"o*gist</hw> <pr>(?)</pr>, <pos>n.</pos> <def>One skilled in technology; one who treats of arts, or of the terms of arts.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Technology</ent><br/
<hw>Tech*nol"o*gy</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Gr. <?/ an art + <ets>-logy</ets>; cf. Gr. <?/ systematic treatment: cf. F. <ets>technologie</ets>.]</ety> <def>Industrial science; the science of systematic knowledge of the industrial arts, especially of the more important manufactures, as spinning, weaving, metallurgy, etc.</def><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ <xex>Technology</xex> is not an independent science, having a set of doctrines of its own, but consists of applications of the principles established in the various physical sciences (chemistry, mechanics, mineralogy, etc.) to manufacturing processes.  <rj><au>Internat. Cyc.</au></rj>
</note><br/
[<source>1913 Webster</source>]</p>

<p><ent>Techy</ent><br/
<hw>Tech"y</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[From OE. <ets>tecche</ets>, <ets>tache</ets>, a habit, bad habit, vice, OF. <ets>tache</ets>, <ets>teche</ets>, a spot, stain, blemish, habit, vice, F. <ets>tache</ets> a spot, blemish; probably akin to E. <ets>tack</ets> a small nail. See <er>Tack</er> a small nail, and cf. <er>Touchy</er>.]</ety> <def>Peevish; fretful; irritable.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tectibranch</ent><br/
<hw>Tec`ti*branch</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Zool.)</fld> <def>One of the Tectibranchiata. Also used adjectively.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tectibranchia</ent><br/
||<hw>Tec`ti*bran"chi*a</hw> <pr>(?)</pr>, <pos>n. pl.</pos> <ety>[NL.]</ety> <def>Same as <er>Tectibranchiata</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tectibranchiata</ent><br/
||<hw>Tec`ti*bran`chi*a"ta</hw> <pr>(?)</pr>, <pos>n. pl.</pos> <ety>[NL., fr. L. <ets>tectus</ets> (p. p. of <ets>tegere</ets> to cover) + Gr. <?/ a gill.]</ety> <fld>(Zool.)</fld> <def>An order, or suborder, of gastropod Mollusca in which the gills are usually situated on one side of the back, and protected by a fold of the mantle.  When there is a shell, it is usually thin and delicate and often rudimentary. The aplysias and the bubble shells are examples.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tectibranchiate</ent><br/
<hw>Tec`ti*bran"chi*ate</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>tectus</ets> (p. p. of <ets>tegere</ets> to cover) + E. <ets>branchiate</ets>.]</ety> <fld>(Zool.)</fld> <def>Having the gills covered by the mantle; of or pertaining to the Tectibranchiata.</def> -- <def2><pos>n.</pos> <def>A tectibranchiate mollusk.</def></def2><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tectly</ent><br/
<hw>Tect"ly</hw> <pr>(?)</pr>, <pos>adv.</pos> <ety>[L. <ets>tectus</ets> covered, fr. <ets>tegere</ets> to cover.]</ety> <def>Covertly; privately; secretly.</def> <mark>[Obs.]</mark>  <rj><au>Holinshed.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tectology</ent><br/
<hw>Tec*tol"o*gy</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Gr. <?/ a carpenter + <ets>-logy</ets>.]</ety> <fld>(Biol.)</fld> <def>A division of morphology created by Haeckel; the science of organic individuality constituting the purely structural portion of morphology, in which the organism is regarded as composed of organic individuals of different orders, each organ being considered an individual. See <er>Promorphology</er>, and <er>Morphon</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tectonic</ent><br/
<hw>Tec*ton"ic</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>tectonicus</ets>, Gr. <?/, fr. <?/, <?/, a carpenter, builder.]</ety> <sn>1.</sn> <def>Of or pertaining to building or construction; architectural.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Biol.)</fld> <def>Structural.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><sn>3.</sn>  <fld>(Geol. & Phys. Geog.)</fld> <def>Of, pert. to, or designating, the rock structures and external forms resulting from the deformation of the earth's crust; <as>as, <ex>tectonic</ex> arches or valleys; <ex>tectonic</ex> plates</as>.</def><br/
[<source>Webster 1913 Suppl.</source> <source>+PJC</source>]</p>

<p><ent>Tectonics</ent><br/
<hw>Tec*ton"ics</hw> <pr>(?)</pr>, <pos>n.</pos> <sn>1.</sn> <def>The science, or the art, by which implements, vessels, dwellings, or other edifices, are constructed, both agreeably to the end for which they are designed, and in conformity with artistic sentiments and ideas.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn>  <fld>(Geol. & Phys. Geog.)</fld> <def>the branch of geology concerned with the rock structures and external forms resulting from the deformation of the earth's crust; also, similar studies of other planets.  Also called <altname>structural geology</altname>.</def><br/
[<source>PJC</source>]</p>

<p><cs><col><b>plate tectonics</b></col> <cd>a geological theory which considers the earth's crust as divided into a number of large relatively rigid plates, which move relatively independently on the more plastic asthenosphere under the influence of magmatic upwellings, so as to drift apart, slide past, or collide with each other, causing the formation, breakup, or merging of continents, and causing volcanism, the building of mountain ranges, and the subduction of one plate beneath another. In recent decades a large body of data have accumulated to support the theory and provide some details of the mechanisms at work.  One set of supporting observations consists of data showing that the continents have slowly moved relative to each other over long periods of time, a phenomenon called <er>continental drift</er>.  Africa and South America, for example, have apparently moved apart from a connected configuration at about 2 to 3 cm per year over tens of millions of years.</cd></cs><br/
[<source>PJC</source>]</p>

<p><ent>Tectorial</ent><br/
<hw>Tec*to"ri*al</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>tectorius</ets>.]</ety> <fld>(Anat.)</fld> <def>Of or pertaining to covering; -- applied to a membrane immediately over the organ of Corti in the internal ear.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tectrices</ent><br/
||<hw>Tec"tri*ces</hw> <pr>(?)</pr>, <pos>n. pl.</pos> <ety>[NL., fr. L. <ets>tegere</ets>, <ets>tectum</ets>, to cover.]</ety> <fld>(Zool.)</fld> <def>The wing coverts of a bird. See <er>Covert</er>, and <xex>Illust.</xex> of <er>Bird</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tecum</ent><br/
<hw>Te"cum</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Bot.)</fld> <def>See <er>Tucum</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Ted</ent><br/
<hw>Ted</hw> <pr>(?)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Tedded</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Tedding</conjf>.]</vmorph> <ety>[Prob. fr. Icel. <ets>te<?/ja</ets> to spread manure, fr. <ets>ta<?/</ets> manure; akin to MHG. <ets>zetten</ets> to scatter, spread. <root/58.  Cf. <er>Teathe</er>.]</ety> <def>To spread, or turn from the swath, and scatter for drying, as new-mowed grass; -- chiefly used in the past participle.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The smell of grain or <qex>tedded</qex> grass.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>The <qex>tedded</qex> hay and corn sheaved in one field.</q> <rj><qau>Coleridge.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tedder</ent><br/
<hw>Ted"der</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A machine for stirring and spreading hay, to expedite its drying.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tedder</ent><br/
<hw>Ted"der</hw>, <pos>n.</pos> <ety>[OE. <root/64. See <er>Tether</er>.]</ety> <def>Same as <er>Tether</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tedder</ent><br/
<hw>Ted"der</hw>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Teddered</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Teddering</conjf>.]</vmorph> <def>Same as <er>Tether</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tedesco</ent><br/
||<hw>Te*des"co</hw> <pr>(?)</pr>, <pos>a.</pos>; <plu>pl. <plw>Tedeschi</plw> <pr>(#)</pr></plu>. <ety>[It., of Germanic origin. See <er>Dutch</er>.]</ety> <def>German; -- used chiefly of art, literature, etc.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Te Deum</ent><br/
||<hw>Te` De"um</hw> <pr>(?)</pr>. <ety>[L., from <ets>te</ets> (accus. of <ets>tu</ets> thou) + <ets>Deum</ets>, accus. of <ets>Deus</ets> God. See <er>Thou</er>, and <er>Deity</er>.]</ety> <sn>1.</sn> <def>An ancient and celebrated Christian hymn, of uncertain authorship, but often ascribed to St. Ambrose; -- so called from the first words <ldquo/<xex>Te Deum laudamus</xex>.<rdquo/ It forms part of the daily matins of the Roman Catholic breviary, and is sung on all occasions of thanksgiving. In its English form, commencing with words, <ldquo/We praise thee, O God,<rdquo/ it forms a part of the regular morning service of the Church of England and the Protestant Episcopal Church in America.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A religious service in which the singing of the hymn forms a principal part.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>A musical setting of the <er>Te Deum{1}</er>.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Tedge</ent><br/
<hw>Tedge</hw> <pr>(t<ecr/j)</pr>, <pos>n.</pos> <fld>(Founding)</fld> <def>The gate of a mold, through which the melted metal is poured; runner, geat.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tediosity</ent><br/
<hw>Te`di*os"i*ty</hw> <pr>(?)</pr>, <pos>n.</pos> <def>Tediousness.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tedious</ent><br/
<hw>Te"di*ous</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>taediosus</ets>, fr. <ets>taedium</ets>. See <er>Tedium</er>.]</ety> <def>Involving tedium; tiresome from continuance, prolixity, slowness, or the like; wearisome.</def> -- <wordforms><wf>Te"di*ous*ly</wf>, <pos>adv.</pos> -- <wf>Te"di*ous*ness</wf>, <pos>n.</pos></wordforms><br/
[<source>1913 Webster</source>]</p>

<p><q>I see a man's life is a <qex>tedious</qex> one.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>I would not be <qex>tedious</qex> to the court.</q> <rj><qau>Bunyan.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><syn><b>Syn.</b> -- Wearisome; fatiguing. See <er>Irksome</er>.</syn><br/
[<source>1913 Webster</source>]</p>

<p><-- p. 1480 --><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tedium</ent><br/
<hw>Te"di*um</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>taedium</ets>, fr. <ets>taedet</ets> it disgusts, it wearies one.]</ety> <def>Irksomeness; wearisomeness; tediousness.</def> <altsp>[Written also <asp>taedium</asp>.]</altsp>  <rj><au>Cowper.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>To relieve the <qex>tedium</qex>, he kept plying them with all manner of bams.</q> <rj><qau>Prof. Wilson.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>The <qex>tedium</qex> of his office reminded him more strongly of the willing scholar, and his thoughts were rambling.</q> <rj><qau>Dickens.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tee</ent><br/
<hw>Tee</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Cf. Icel. <ets>tj<amac/</ets> to show, mark.]</ety> <sd>(a)</sd> <def>The mark aimed at in curling and in quoits.</def> <sd>(b)</sd> <def>The nodule of earth, or a short peg stuck into the ground, from which the ball is struck at the beginning of play for each hole in golf.</def><br/
[<source>1913 Webster</source> <source>+PJC</source>]</p>

<p><ent>Tee</ent><br/
<hw>Tee</hw>, <pos>n.</pos> <sn>1.</sn> <def>A short piece of pipe having a lateral outlet, used to connect a line of pipe with a pipe at a right angle with the line; -- so called because it resembles the letter <er>T</er> in shape.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>The letter T, t; also, something shaped like, or resembling in form, the letter <universbold>T</universbold>.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Tee</ent><br/
<hw>Tee</hw> <pr>(?)</pr>, <pos>v. t.</pos>  <vmorph>[<pos>imp. & p. p.</pos> <conjf>Teed</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Teeing</conjf>.]</vmorph> <fld>(Golf)</fld> <def>To place (the ball) on a tee; also called to <altname>tee up</altname>.</def><-- also . See tee off. --><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Teeing ground</ent><br/
<hw>Teeing ground</hw>. <fld>(Golf)</fld> <def>The space from within which the ball must be struck in beginning the play for each hole.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Tee iron</ent><br/
<hw>Tee" i`ron</hw> <pr>(?)</pr>. <def>See <cref>T iron</cref>, under <er>T</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Teek</ent><br/
<hw>Teek</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Bot.)</fld> <def>See <er>Teak</er>.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Teel</ent><br/
<hw>Teel</hw> <pr>(?)</pr>, <pos>n.</pos> <def>Sesame.</def> <altsp>[Sometimes written <asp>til</asp>.]</altsp><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Teel oil</b></col>, <cd>sesame oil.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Teelseed</ent><br/
<hw>Teel"seed`</hw> <pr>(?)</pr>, <pos>n.</pos> <def>The seed of sesame.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Teem</ent><br/
<hw>Teem</hw> <pr>(?)</pr>, <pos>v. t.</pos> <ety>[Icel. <ets>t<ae/ma</ets> to empty, from <ets>t<omac/mr</ets> empty; akin to Dan. <ets>t<oum/mme</ets> to empty, Sw. <ets>t<oum/mma</ets>. See <er>Toom</er> to empty.]</ety> <sn>1.</sn> <def>To pour; -- commonly followed by <xex>out</xex>; <as>as, to <ex>teem</ex> out ale</as>.</def> <mark>[Obs. or Prov. Eng.]</mark>  <rj><au>Swift.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Steel Manuf.)</fld> <def>To pour, as steel, from a melting pot; to fill, as a mold, with molten metal.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Teem</ent><br/
<hw>Teem</hw>, <pos>v. t.</pos> <ety>[See <er>Tame</er>, <pos>a.</pos>, and cf. <er>Beteem</er>.]</ety> <def>To think fit.</def> <mark>[Obs. or R.]</mark>  <rj><au>G. Gifford.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Teem</ent><br/
<hw>Teem</hw>, <pos>v. i.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Teemed</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Teeming</conjf>.]</vmorph> <ety>[OE. <ets>temen</ets>, AS. <ets>t<emac/man</ets>, <ets>t<?/man</ets>, from <ets>te<aacute/m</ets>. See <er>Team</er>.]</ety> <sn>1.</sn> <def>To bring forth young, as an animal; to produce fruit, as a plant; to bear; to be pregnant; to conceive; to multiply.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>If she must <qex>teem</qex>,<br/
Create her child of spleen.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To be full, or ready to bring forth; to be stocked to overflowing; to be prolific; to abound.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>His mind <qex>teeming</qex> with schemes of future deceit to cover former villainy.</q> <rj><qau>Sir W. Scott.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>The young, brimful of the hopes and feeling which <qex>teem</qex> in our time.</q> <rj><qau>F. Harrison.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Teem</ent><br/
<hw>Teem</hw>, <pos>v. t.</pos> <def>To produce; to bring forth.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>That [grief] of an hour's age doth hiss the speaker;<br/
Each minute <qex>teems</qex> a new one.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Teemer</ent><br/
<hw>Teem"er</hw> <pr>(?)</pr>, <pos>n.</pos> <def>One who teems, or brings forth.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Teemful</ent><br/
<hw>Teem"ful</hw> <pr>(?)</pr>, <pos>a.</pos> <sn>1.</sn> <def>Pregnant; prolific.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Brimful.</def> <mark>[Obs.]</mark>  <rj><au>Ainsworth.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Teeming</ent><br/
<hw>Teem"ing</hw>, <pos>a.</pos> <def>Prolific; productive.</def><br/
[<source>1913 Webster</source>]</p>

<p><q><qex>Teeming</qex> buds and cheerful appear.</q> <rj><qau>Dryden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Teemless</ent><br/
<hw>Teem"less</hw>, <pos>a.</pos> <def>Not fruitful or prolific; barren; <as>as, a <ex>teemless</ex> earth</as>.</def> <mark>[Poetic]</mark>  <rj><au>Dryden.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Teen</ent><br/
<hw>Teen</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[OE. <ets>tene</ets>, AS. <ets>te<oacute/na</ets> reproach, wrong, fr. <ets>te<oacute/n</ets> to accuse; akin to G. <ets>zeihen</ets>, Goth. ga<ets>teihan</ets> to tell, announce, L. <ets>dicere</ets> to say. See <er>Token</er>.]</ety> <def>Grief; sorrow; affiction; pain.</def> <mark>[Archaic]</mark>  <rj><au>Chaucer. Spenser.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>With public toil and private <qex>teen</qex><br/
Thou sank'st alone.</q> <rj><qau>M. Arnold.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Teen</ent><br/
<hw>Teen</hw>, <pos>v. t.</pos> <ety>[AS. <ets>te<oacute/nian</ets>, <ets>t<?/nan</ets>, to slander, vex. <root/64. See <er>Teen</er>, <pos>n.</pos>]</ety> <def>To excite; to provoke; to vex; to affict; to injure.</def> <mark>[Obs.]</mark>  <rj><au>Piers Plowman.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Teen</ent><br/
<hw>Teen</hw>, <pos>v. t.</pos> <ety>[See <er>Tine</er> to shut.]</ety> <def>To hedge or fence in; to inclose.</def> <mark>[Prov. Eng.]</mark>  <rj><au>Halliwell.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Teen</ent><br/
<hw>Teen</hw>, <pos>n.</pos> <def>a <er>teenager</er>.</def><br/
[<source>PJC</source>]</p>

<p><ent>Teenage</ent><br/
<hw>Teen"age</hw> <pr>(?)</pr>, <pos>n.</pos> <def>The longer wood for making or mending fences.</def> <mark>[Prov. Eng.]</mark>  <rj><au>Halliwell.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Teenage</ent><br/
<hw>Teen"age`</hw> <pr>(t<emac/n"<amac/j)</pr>, <pos>n.</pos> <def>of or pertaining to a teenager; being in one's teens; <as>as, a busload of <ex>teenage</ex> football fans; <ex>teenage</ex> inexperience</as>.</def><br/
[<source>PJC</source>]</p>

<p><ent>Teenager</ent><br/
<hw>Teen"ag`er</hw> <pr>(t<emac/n"<amac/j`<etil/r)</pr>, <pos>n.</pos> <def>a person whose age is in the teens, i.e. one between the ages of 13 to 19 inclusive.</def><br/
[<source>PJC</source>]</p>

<p><ent>Teend</ent><br/
<hw>Teend</hw> <pr>(?)</pr>, <pos>v. t. & i.</pos> <ety>[See <er>Tinder</er>.]</ety> <def>To kindle; to burn.</def> <mark>[Obs.]</mark>  <rj><au>Herrick.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Teenful</ent><br/
<hw>Teen"ful</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Full of teen; harmful; grievous; grieving; afflicted.</def> <mark>[Obs.]</mark>  <rj><au>Piers Plowman.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Teens</ent><br/
<hw>Teens</hw> <pr>(?)</pr>, <pos>n. pl.</pos> <ety>[See <er>Ten</er>.]</ety> <def>The years of one's age having the termination <xex>-teen</xex>, beginning with thirteen and ending with nineteen; <as>as, a girl in her <ex>teens</ex></as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Teeny</ent><br/
<hw>Tee"ny</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Very small; tiny.</def> <mark>[Colloq.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Teeny</ent><br/
<hw>Teen"y</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[See <er>Teen</er> grief.]</ety> <def>Fretful; peevish; pettish; cross.</def> <mark>[Prov. Eng.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Teeong</ent><br/
<hw>Tee*ong"</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Zool.)</fld> <def>The mino bird.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Teest</ent><br/
<hw>Teest</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A tinsmith's stake, or small anvil.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Teeswater</ent><br/
<hw>Tees"wa`ter</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[From the river <ets>Tees</ets>, northern England.]</ety> <sn>1.</sn> <def>A breed of cattle formerly bred in England, but supposed to have originated in Holland and to have been the principal stock from which the shorthorns were derived.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><sn>2.</sn>  <def>An old English breed of sheep allied to the Leicester.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Teetan</ent><br/
<hw>Tee"tan</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Zool.)</fld> <def>A pipit.</def> <mark>[Prov. Eng.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Teetee</ent><br/
<hw>Tee"tee</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Sp. <ets>tit<iacute/</ets>.]</ety> <sn>1.</sn> <fld>(Zool.)</fld> <def>Any one of several species of small, soft-furred South American monkeys belonging to <gen>Callithrix</gen>, <gen>Chrysothrix</gen>, and allied genera; <as>as, the collared <ex>teetee</ex> (<spn>Callithrix torquatus</spn>), and the squirrel <ex>teetee</ex> (<spn>Chrysothrix sciurea</spn>)</as>. Called also <altname>pinche</altname>, <altname>titi</altname>, and <altname>saimiri</altname>. See <cref>Squirrel monkey</cref>, under <er>Squirrel</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Zool.)</fld> <def>A diving petrel of Australia (<spn>Halodroma wrinatrix</spn>).</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Teeter</ent><br/
<hw>Tee"ter</hw> <pr>(?)</pr>, <pos>v. i. & t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Teetered</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Teetering</conjf>.]</vmorph> <ety>[Prov. E. <ets>titter</ets> to tremble, to seesaw; cf. Icel. <ets>titra</ets> to tremble, OHG. <ets>zittar<omac/n</ets>, G. <ets>zittern</ets>.]</ety> <def>To move up and down on the ends of a balanced plank, or the like, as children do for sport; to seesaw; to titter; to titter-totter.</def> <mark>[U. S.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>[The bobolink] alit upon the flower, and <qex>teetered</qex> up and down.</q> <rj><qau>H. W. Beecher.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Teeter-tail</ent><br/
<hw>Tee"ter-tail`</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Zool.)</fld> <def>The spotted sandpiper. See the Note under <er>Sandpiper</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Teeth</ent><br/
<hw>Teeth</hw> <pr>(?)</pr>, <pos>n.</pos>, <def><pos>pl.</pos> of <er>Tooth</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Teeth</ent><br/
<hw>Teeth</hw> <pr>(?)</pr>, <pos>v. i.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Teethed</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Teething</conjf>.]</vmorph> <def>To breed, or grow, teeth.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Teething</ent><br/
<hw>Teeth"ing</hw> <pr>(?)</pr>, <pos>n.</pos> <def>The process of the first growth of teeth, or the phenomena attending their issue through the gums; dentition.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Teetotal</ent><br/
<hw>Tee*to"tal</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Entire; total.</def> <mark>[Colloq.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Teetotaler</ent><br/
<hw>Tee*to"tal*er</hw> <pr>(?)</pr>, <pos>n.</pos> <def>One pledged to entire abstinence from all intoxicating drinks.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Teetotalism</ent><br/
<hw>Tee*to"tal*ism</hw> <pr>(?)</pr>, <pos>n.</pos> <def>The principle or practice of entire abstinence, esp. from intoxicating drinks.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Teetotally</ent><br/
<hw>Tee*to"tal*ly</hw> <pr>(?)</pr>, <pos>adv.</pos> <def>Entirely; totally.</def> <mark>[Colloq.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Teetotum</ent><br/
<hw>Tee*to"tum</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[For T-<ets>totum</ets>. It was used for playing games of chance, and was four-sided, one side having the letter <b>T</b> on it, standing for Latin <ets>totum</ets> all, meaning, take all that is staked, whence the name. The other three sides each had a letter indicating an English or Latin word; as <b>P</b> meaning put down, <b>N</b> nothing or L. <ets>nil</ets>, <b>H</b> half. See <er>Total</er>.]</ety> <def>A child's toy, somewhat resembling a top, and twirled by the fingers.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The staggerings of the gentleman . . . were like those of a <qex>teetotum</qex> nearly spent.</q> <rj><qau>Dickens.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tee-to-tum</ent><br/
<hw>Tee-to"-tum</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Cf. <er>Teetotaler</er>.]</ety> <def>A workingmen's resort conducted under religious influences as a counteractant to the drinking saloon.</def> <mark>[Colloq. or Cant]</mark><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Teetuck</ent><br/
<hw>Tee"tuck</hw> <pr>(?)</pr>, <pos>n.</pos> <def>The rock pipit.</def> <mark>[Prov. Eng.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Teeuck</ent><br/
<hw>Tee"uck</hw> <pr>(?)</pr>, <pos>n.</pos> <def>The lapwing.</def> <mark>[Prov. Eng.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Teewit</ent><br/
<hw>Tee"wit</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Zool.)</fld> <def>The pewit.</def> <mark>[Prov. Eng.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Teg</ent><br/
<hw>Teg</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A sheep in its second year; also, a doe in its second year.</def> <mark>[Prov. Eng.]</mark>  <rj><au>Halliwell.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tegmen</ent><br/
||<hw>Teg"men</hw> <pr>(?)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Tegmina</plw> <pr>(#)</pr>.</plu> <ety>[L., fr. <ets>tegere</ets>, <ets>tectum</ets>, to cover.]</ety> <sn>1.</sn> <def>A tegument or covering.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Bot.)</fld> <def>The inner layer of the coating of a seed, usually thin and delicate; the endopleura.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <fld>(Zool.)</fld> <def>One of the elytra of an insect, especially of certain Orthoptera.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <pluf>pl.</pluf> <fld>(Zool.)</fld> <def>Same as <er>Tectrices</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tegmental</ent><br/
<hw>Teg*men"tal</hw> <pr>(?)</pr>, <pos>a.</pos> <fld>(Biol.)</fld> <def>Of or pertaining to a tegument or tegmentum; <as>as, the <ex>tegmental</ex> layer of the epiblast; the <ex>tegmental</ex> cells of the taste buds.</as></def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tegmentum</ent><br/
||<hw>Teg*men"tum</hw> <pr>(?)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Tegmenta</plw> <pr>(#)</pr>.</plu> <ety>[L., a covering.]</ety> <fld>(Anat.)</fld> <def>A covering; -- applied especially to the bundles of longitudinal fibers in the upper part of the crura of the cerebrum.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Teguexin</ent><br/
<hw>Te*guex"in</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Zool.)</fld> <def>A large South American lizard (<spn>Tejus teguexin</spn>). It becomes three or four feet long, and is blackish above, marked with yellowish spots of various sizes. It feeds upon fruits, insects, reptiles, young birds, and birds' eggs. The closely allied species <spn>Tejus rufescens</spn> is called <altname>red teguexin</altname>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tegula</ent><br/
||<hw>Teg"u*la</hw> <pr>(?)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Tegulae</plw> <pr>(#)</pr>.</plu> <ety>[L., a tile, dim. fr. <ets>tegere</ets> to cover.]</ety> <fld>(Zool.)</fld> <def>A small appendage situated above the base of the wings of Hymenoptera and attached to the mesonotum.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tegular</ent><br/
<hw>Teg"u*lar</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[LL. <ets>tegularis</ets>, from L. <ets>tegula</ets> a tile. See <er>Tile</er>.]</ety> <def>Of or pertaining to a tile; resembling a tile, or arranged like tiles; consisting of tiles; <as>as, a <ex>tegular</ex> pavement</as>.</def> -- <wordforms><wf>Teg"u*lar*ly</wf>, <pos>adv.</pos></wordforms><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tegulated</ent><br/
<hw>Teg`u*la"ted</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Composed of small plates, as of horn or metal, overlapping like tiles; -- said of a kind of ancient armor.</def>  <rj><au>Fairholt.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tegument</ent><br/
<hw>Teg"u*ment</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>tegumentum</ets>, from <ets>tegere</ets> to cover. See <er>Thatch</er>, <pos>n.</pos>, and cf. <er>Detect</er>, <er>Protect</er>.]</ety> <sn>1.</sn> <def>A cover or covering; an integument.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Especially, the covering of a living body, or of some part or organ of such a body; skin; hide.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tegumentary</ent><br/
<hw>Teg`u*men"ta*ry</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[Cf. F. <ets>t<eacute/gumentaire</ets>.]</ety> <def>Of or pertaining to a tegument or teguments; consisting of teguments; serving as a tegument or covering.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Te-hee</ent><br/
<hw>Te-hee"</hw> <pr>(?)</pr>, <pos>n. & interj.</pos> <def>A tittering laugh; a titter.</def> <ldquo/'<xex>Te-hee</xex>,' quoth she.<rdquo/  <rj><au>Chaucer.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Te-hee</ent><br/
<hw>Te-hee"</hw>, <pos>v. i.</pos> <def>To titter; to laugh derisively.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>She cried, <ldquo/Come, come; you must not look grave upon me.<rdquo/ Upon this, I <qex>te-heed</qex>.</q> <rj><qau>Madame D'Arblay.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Teil</ent><br/
<hw>Teil</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[OF. <ets>teil</ets>, <ets>til</ets>, L. <ets>tilia</ets>.]</ety> <fld>(Bot.)</fld> <def>The lime tree, or linden; -- called also <altname>teil tree</altname>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Teind</ent><br/
<hw>Teind</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Cf. Icel. <ets>t<imac/und</ets>. See <er>Tithe</er>.]</ety> <def>A tithe.</def> <mark>[Scot.]</mark>  <rj><au>Jamieson.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Teine</ent><br/
<hw>Teine</hw> <pr>(?)</pr>, <pos>n.</pos> <def>See <er>Teyne</er>.</def> <mark>[Obs.]</mark>  <rj><au>Chaucer.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Teinland</ent><br/
<hw>Tein"land</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(O. Eng. Law)</fld> <def>Land granted by the crown to a thane or lord.</def>  <rj><au>Burrill.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Teinoscope</ent><br/
<hw>Tei"no*scope</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Gr. <?/ to extend + <ets>-scope</ets>.]</ety> <fld>(Physics)</fld> <def>An instrument formed by combining prisms so as to correct the chromatic aberration of the light while linear dimensions of objects seen through the prisms are increased or diminished; -- called also <altname>prism telescope</altname>.</def>  <rj><au>Sir D. Brewster.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Teint</ent><br/
<hw>Teint</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F. <ets>teint</ets>, <ets>teinte</ets>. See <er>Tint</er>.]</ety> <def>Tint; color; tinge, See <er>Tint</er>.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>Time shall . . . embrown the <qex>teint</qex>.</q> <rj><qau>Dryden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Teinture</ent><br/
<hw>Tein"ture</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F. See <er>Tincture</er>.]</ety> <def>Color; tinge; tincture.</def> <mark>[Obs.]</mark>  <rj><au>Holland.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tek</ent><br/
<hw>Tek</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Zool.)</fld> <def>A Siberian ibex.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Telamones</ent><br/
||<hw>Tel`a*mo"nes</hw> <pr>(?)</pr>, <pos>n. pl.</pos> <ety>[L., pl. of <ets>telamo</ets> or <ets>telamon</ets>, Gr. <?/ a bearer, fr. <?/ to bear.]</ety> <fld>(Arch.)</fld> <def>Same as <er>Atlantes</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Telangiectasis</ent><br/
||<hw>Tel*an`gi*ec"ta*sis</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[NL., fr. Gr. <?/ end + <?/ vessel + <?/ extension.]</ety> <fld>(Med.)</fld> <def>Dilatation of the capillary vessels.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Telangiectasy</ent><br/
<hw>Tel*an`gi*ec"ta*sy</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Med.)</fld> <def>Telangiectasis.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Telarly</ent><br/
<hw>Te"lar*ly</hw> <pr>(?)</pr>, <pos>adv.</pos> <def>In a weblike manner.</def> <mark>[Obs.]</mark> <ldquo/<xex>Telarly</xex> interwoven.<rdquo/  <rj><au>Sir T. Browne.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Telary</ent><br/
<hw>Te"la*ry</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[LL. <ets>telaris</ets>, fr. L. <ets>tela</ets> a web. See <er>Toil</er> a snare.]</ety> <def>Of or pertaining to a web; hence, spinning webs; retiary.</def> <ldquo/Pictures of <xex>telary</xex> spiders.<rdquo/  <rj><au>Sir T. Browne.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Telautogram</ent><br/
<hw>Tel*au"to*gram</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A message transmitted and recorded by a teleautograph.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Telautograph</ent><br/
<hw>Tel*au"to*graph</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Gr. <grk>th^le</grk> far + <ets>autograph</ets>.]</ety> <def>A facsimile telegraph for reproducing writing, pictures, maps, etc. In the transmitter the motions of the pencil are communicated by levers to two rotary shafts, by which variations in current are produced in two separate circuits. In the receiver these variations are utilized by electromagnetic devices and levers to move a pen as the pencil moves.</def> -- <wordforms><wf>Tel`au*tog"ra*phist</wf> <pr>(#)</pr>, <pos>n.</pos><-- superseded by FAX --></wordforms><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Telechirograph</ent><br/
<hw>Tel`e*chi"ro*graph</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Gr. <grk>th^le</grk> far + <grk>chei`r</grk>, <grk>cheiro`s</grk>, hand + <ets>-graph</ets>.]</ety> <def>An instrument for  telegraphically transmitting and receiving handwritten messages, as photographically by a beam of light from a mirror.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Teledu</ent><br/
<hw>Tel"e*du</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Zool.)</fld> <def>An East Indian carnivore (<spn>Mydaus meliceps</spn>) allied to the badger, and noted for the very offensive odor that it emits, somewhat resembling that of a skunk. It is a native of the high mountains of Java and Sumatra, and has long, silky fur. Called also <altname>stinking badger</altname>, and <altname>stinkard</altname>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Telega</ent><br/
<hw>Te*le"ga</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Russ. <ets>telyega</ets>.]</ety> <def>A rude four-wheeled, springless wagon, used among the Russians.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Telegony</ent><br/
<hw>Te*leg"o*ny</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Gr. <grk>th^le</grk> far + root of  Gr. <?/ to be born.]</ety> <fld>(Biol.)</fld> <def>The supposed influence of a father upon offspring subsequent to his own, begotten of the same mother by another father.</def> -- <wordforms><wf>Te*leg"o*nous</wf> <pr>(#)</pr>, <pos>a.</pos></wordforms><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Telegram</ent><br/
<hw>Tel"e*gram</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Gr. <?/ far + <ets>-gram</ets>.]</ety> <def>A message sent by telegraph; a telegraphic dispatch.</def><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ <ldquo/A friend desires us to give notice that he will ask leave, at some convenient time, to introduce a new word into the vocabulary. It is <xex>telegram</xex>, instead of <xex>telegraphic dispatch</xex>, or <xex>telegraphic communication</xex>.<rdquo/  <rj><au>Albany [N. Y.] Evening Journal (April 6, 1852).</au></rj>
</note><br/
[<source>1913 Webster</source>]</p>

<p><ent>Telegrammic</ent><br/
<hw>Tel`e*gram*mic</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Pertaining to, or resembling, a telegram; laconic; concise; brief.</def> <mark>[R.]</mark><br/
<syn><b>Syn. --</b> telegraphic.</syn><br/
[<source>1913 Webster</source>]</p>

<p><ent>Telegraph</ent><br/
<hw>Tel"e*graph</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Gr. <?/ far, far off (cf. Lith. <ets>toli</ets>) + <ets>-graph</ets>: cf. F. <ets>t<eacute/l<eacute/graphe</ets>. See <er>Graphic</er>.]</ety> <def>An apparatus, or a process, for communicating intelligence rapidly between distant points, especially by means of preconcerted visible or audible signals representing words or ideas, or by means of words and signs, transmitted by electrical action.</def><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ The instruments used are classed as <xex>indicator</xex>, <xex>type-printing</xex>, <xex>symbol-printing</xex>, or <xex>chemical-printing telegraphs</xex>, according as the intelligence is given by the movements of a pointer or indicator, as in Cooke & Wheatstone's (the form commonly used in England), or by impressing, on a fillet of paper, letters from types, as in House's and Hughe's, or dots and marks from a sharp point moved by a magnet, as in Morse's, or symbols produced by electro-chemical action, as in Bain's. In the offices in the United States the recording instrument is now little used, the receiving operator reading by ear the combinations of long and short intervals of sound produced by the armature of an electro-magnet as it is put in motion by the opening and breaking of the circuit, which motion, in registering instruments, traces upon a ribbon of paper the lines and dots used to represent the letters of the alphabet. See <xex>Illustration</xex> in Appendix, and <er>Morse code</er>.</note><br/
[<source>1913 Webster</source>]</p>

<p><note>In 1837,  <person>Samuel F. B. Morse</person>, an American artist, devised a working electric telegraph, based on a rough knowledge of electrical circuits, electromagnetic induction coils, and a scheme to encode alphabetic letters.  He  and his collaborators and backers campaigned for years before persuading the federal government to fund a demonstration.  Finally, on May 24, 1844, they sent the first official long-distance telegraphic message in Morse code, "What hath God wrought," through a copper wire strung between Washington, D.C., to Baltimore, Maryland.   The phrase was taken from the Bible, Numbers 23:23.  It  had been suggested to <persfn>Morse</persfn> by <person>Annie Ellworth</person>, the young daughter of a friend.</note> <au>Library of Congress, American Memories series (http://memory.loc.gov/ammem/today/may24.html).</au><br/
[<source>PJC</source>]</p>

<p><cs><col><b>Acoustic telegraph</b></col>. <cd>See under <er>Acoustic</er>.</cd> -- <col><b>Dial telegraph</b></col>, <cd>a telegraph in which letters of the alphabet and numbers or other symbols are placed upon the border of a circular dial plate at each station, the apparatus being so arranged that the needle or index of the dial at the receiving station accurately copies the movements of that at the sending station.</cd> -- <mcol><col><b>Electric telegraph</b></col>, <it>or</it> <col><b>Electro-magnetic telegraph</b></col></mcol>, <cd>a telegraph in which an operator at one station causes words or signs to be made at another by means of a current of electricity, generated by a battery and transmitted over an intervening wire.</cd> -- <col><b>Facsimile telegraph</b></col>. <cd>See under <er>Facsimile</er>.</cd> -- <col><b>Indicator telegraph</b></col>. <cd>See under <er>Indicator</er>.</cd> -- <col><b>Pan-telegraph</b></col>, <cd>an electric telegraph by means of which a drawing or writing, as an autographic message, may be exactly reproduced at a distant station.</cd> -- <col><b>Printing telegraph</b></col>, <cd>an electric telegraph which automatically prints the message as it is received at a distant station, in letters, not signs.</cd> -- <col><b>Signal telegraph</b></col>, <cd>a telegraph in which preconcerted signals, made by a machine, or otherwise, at one station, are seen or heard and interpreted at another; a semaphore.</cd> -- <col><b>Submarine telegraph cable</b></col>, <cd>a telegraph cable laid under water to connect stations separated by a body of water.</cd> -- <col><b>Telegraph cable</b></col>, <cd>a telegraphic cable consisting of several conducting wires, inclosed by an insulating and protecting material, so as to bring the wires into compact compass for use on poles, or to form a strong cable impervious to water, to be laid under ground, as in a town or city, or under water, as in the ocean.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Telegraph</ent><br/
<hw>Tel"e*graph</hw> <pr>(?)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Telegraphed</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Telegraphing</conjf> <pr>(?)</pr>.]</vmorph> <ety>[F. <ets>t<eacute/l<eacute/graphier</ets>.]</ety> <def>To convey or announce by telegraph.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Telegrapher</ent><br/
<hw>Te*leg"ra*pher</hw> <pr>(?)</pr>, <pos>n.</pos> <def>One who sends telegraphic messages; a telegraphic operator; a telegraphist.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>telegraphese</ent><br/
<hw>telegraphese</hw> <pos>n.</pos> <sn>1.</sn> <def>language characterized by terseness and ellipsis as in telegrams.</def><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Telegraphic</ent><br/
<hw>Tel`e*graph"ic</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[Cf. F. <ets>t<eacute/l<eacute/graphique</ets>.]</ety> <sn>1.</sn> <def>Of or pertaining to the telegraph; made or communicated by a telegraph; <as>as, <ex>telegraphic</ex> signals; <ex>telegraphic</ex> art; <ex>telegraphic</ex> intelligence.</as></def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Having only the essential information; brief; concise; terse; -- of communications, by analogy with the style of telegrams, which are short to avoid unnecessary expense.</def> <note>a <ex>telegraphic</ex> communication should have enough information to allow comprehension of the content, though it may leave out normally included words.  If so much is left out that the communication becomes difficult or impossible to understand, it may be called <contr>cryptic</contr>. <ldquo/Sighted sub. Sank same.<rdquo/ is a <ex>telegraphic</ex> message.</note><br/
[<source>PJC</source>]</p>

<p><ent>Telegraphical</ent><br/
<hw>Tel`e*graph"ic*al</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Telegraphic.</def> -- <wordforms><wf>Tel`e*graph"ic*al*ly</wf>, <pos>adv.</pos></wordforms><br/
[<source>1913 Webster</source>]</p>

<p><ent>Telegraphist</ent><br/
<hw>Te*leg"ra*phist</hw> <pr>(?)</pr>, <pos>n.</pos> <def>One skilled in telegraphy; a telegrapher.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Telegraph plant</ent><br/
<hw>Telegraph plant</hw> <fld>(Bot.)</fld>, <def>A tick trefoil (<spn>Meibomia gyrans</spn> formerly <spn>Desmodium gyrans</spn>), native of the East Indies; it is a leguminous plant whose lateral leaflets jerk up and down like the arms of a semaphore, and also rotate on their axes.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Telegraphone</ent><br/
<hw>Te*leg"ra*phone</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Gr. <grk>th^le</grk> far + <ets>-graph</ets> + <?/ sound.]</ety> <def>An instrument for recording and reproducing sound by local magnetization of a steel wire, disk, or ribbon, moved against the pole of a magnet connected electrically with a telephone receiver, or the like.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Telegraphoscope</ent><br/
<hw>Tel`e*graph"o*scope</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Gr. <grk>th^le</grk>  far + <ets>-graph</ets> + <ets>-scope</ets>.]</ety> <def>An instrument for telegraphically transmitting a picture and reproducing its image as a positive or negative.  The transmitter includes a camera obscura and a row of minute selenium cells.  The receiver includes an oscillograph, relay, equilibrator, and an induction coil the sparks from which perforate a paper with tiny holes that form the image.  It is now (1999) obsolete, having been replaced by telefax and internet transmission of images.</def><br/
[<source>Webster 1913 Suppl.</source> <source>+PJC</source>]</p>

<p><ent>Telegraphy</ent><br/
<hw>Te*leg"ra*phy</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Cf. F. <ets>t<eacute/l<eacute/graphie</ets>.]</ety> <def>The science or art of constructing, or of communicating by means of, telegraphs; <as>as, submarine <ex>telegraphy</ex></as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Telegu</ent><br/
<hw>Tel`e*gu"</hw>, <pos>prop. a. & n.</pos> <def>same as <er>Telugu</er>.</def><br/
[<source>PJC</source>]</p>

<p><ent>Telehydrobarometer</ent><br/
<hw>Tel`e*hy`dro*ba*rom"e*ter</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Gr. <grk>th^le</grk> far + <ets>hydrobarometer</ets>.]</ety> <def>An instrument for indicating the level of water in a distant tank or reservior.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Tele-iconograph</ent><br/
<hw>Tel`e-i*con"o*graph</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Gr. <grk>th^le</grk> far + <ets>iconograph</ets>.]</ety> <sn>1.</sn> <def>An instrument essentially the same as the telemetrograph.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><sn>2.</sn>  <def>A form of facsimile telegraph.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>telekinesis</ent><br/
<hw>tel`e*ki*ne"sis</hw> <pr>(t<ecr/l`<ucr/*k<icr/*n<emac/"s<icr/s)</pr>, <pos>n.</pos> <def>the ability to move objects by means of thought alone, without physical means; -- an ability claimed by certain persons, such as mediums.</def> <note>the existence of this ability is disbelieved by most scientists.</note><br/
<syn><b>Syn. --</b> psychokinesis</syn><br/
[<source>PJC</source>]</p>

<p><ent>Tel-el-Amarna</ent><br/
<hw>Tel`-el-A*mar"na</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Ar., hill of Amarna.]</ety> <def>A station on the Nile in Egypt, midway between Thebes and Memphis, forming the site of the ancient city of Akhetaton, capital of <person>Amenophis IV.</person> (<persfn>Akhenaton</persfn>, or <person>Amenhotep IV.</person>, of the 18th dynasty, king 1353-1336 B. C.), whose archive chamber was discovered there during extensive excavations in 1887-1888.  A collection of about 300 clay tablets (called the <mcol><col><b>Tel-el-Amarna tablets</b></col>, <it>or</it> the <col><b>Amarna tablets</b></col></mcol>) was found here, forming the diplomatic correspondence (<col><b>Tel-el-Amarna letters</b></col>) of <person>Amenophis IV.</person> and his father, <person>Amenophis III.</person>, with the kings of Asiatic countries (such as Babylonia, Assyria, and Palestine), written in cuneiform characters.  It is an important source of our knowledge of Asia from about 1400 to 1370 <sc>b. c.</sc>.  The name of the site is also spelled <asp>Tell-el-Amarna</asp>, <asp>Tell el Amarna</asp>, and <asp>Tel Amarna</asp>.</def><br/
[<source>Webster 1913 Suppl.</source> <source>+PJC</source>]</p>

<p><ent>Telelectric</ent><br/
<hw>Tel`e*lec"tric</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[Gr. <grk>th^le</grk> far + <ets>electric</ets>.]</ety> <fld>(Elec.)</fld> <def>Of or pertaining to transmission, as of music, to a distance by electricity.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Telelectroscope</ent><br/
<hw>Tel`e*lec"tro*scope</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Gr. <grk>th^le</grk> far + <ets>electro-</ets> + <ets>-scope</ets>.]</ety> <def>Any apparatus for making distant objects visible by the aid of electric transmission.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>telemarketing</ent><br/
<hw>telemarketing</hw> <pos>n.</pos> <def>the use of the telephone as an interactive medium for promotion; calling potential customers by telephone for the purpose of selling something; -- applied especially to calls made to persons who have not previously contacted the seller.</def><br/
<syn><b>Syn. --</b> teleselling.</syn><br/
[<source>WordNet 1.5</source> <source>+PJC</source>]</p>

<p><ent>Telemechanic</ent><br/
<hw>Tel`e*me*chan"ic</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[Gr. <grk>th^le</grk> far + <ets>mechanic</ets>.]</ety> <def>Designating, or pert. to, any device for operating mechanisms at a distance.</def> -- <wordforms><wf>Tel`e*mech"a*nism</wf> <pr>(#)</pr>, <pos>n.</pos></wordforms><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Telemachus</ent><br/
<hw>Tel`e*ma"chus</hw> <pr>(?)</pr>, <pos>prop. n.</pos> <def>The son of Odysseus and Penelope, as told in Homer's Oddysey.</def><br/
[<source>PJC</source>]</p>

<p><ent>Telemeteorograph</ent><br/
<hw>Tel`e*me`te*or"o*graph</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Gr. <grk>th^le</grk> far + <ets>meteorograph</ets>.]</ety> <def>Any apparatus recording meteorological phenomena at a distance from the measuring apparatus, as by electricity or by compressed air; esp., an apparatus recording conditions at many distant stations at a central office.</def> -- <wordforms><wf>Tel`e*me`te*or*o*graph"ic</wf> <pr>(#)</pr>, <pos>a.</pos></wordforms><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Telemeter</ent><br/
<hw>Te*lem"e*ter</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Gr. <?/ far + <ets>-meter</ets>.]</ety> <sn>1.</sn> <def>An instrument used for measuring the distance of an object from an observer; <as>as, a telescope with a micrometer for measuring the apparent diameter of an object whose real dimensions are known</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A measuring instrument which sends the information obtained from its sensors by radio to a distant station, usually to be recorded there; also, the complete system including measuring instrument, transmitter, and receiver.  Such instruments are used, for example, to measure conditions in space or in other locations difficult of access for humans observers, or merely to allow one observer to monitor conditions in many places simultaneously.</def><br/
[<source>PJC</source>]</p>

<p><ent>Telemetrograph</ent><br/
<hw>Tel`e*met"ro*graph</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Gr. <grk>th^le</grk> far + <grk>me`tron</grk> measure + <ets>-graph</ets>.]</ety> <def>A combination of the camera lucida and telescope for drawing and measuring distant objects.</def> -- <wordforms><wf>Tel`e*me*trog"ra*phy</wf> <pr>(#)</pr>, <pos>n.</pos> -- <wf>Tel`e*met`ro*graph"ic</wf> <pr>(#)</pr>, <pos>a.</pos></wordforms><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Telemetry</ent><br/
<hw>Te*lem"e*try</hw> <pr>(?)</pr>, <pos>n.</pos> <def>The science or process of making remote measurements and sending the data by radio; the use of a telemeter.</def><br/
[<source>PJC</source>]</p>

<p><ent>Telemotor</ent><br/
<hw>Tel`e*mo"tor</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Gr. <grk>th^le</grk> far + <ets>motor</ets>.]</ety> <fld>(Naut.)</fld> <def>A hydraulic device by which the movement of the wheel on the bridge operates the steering gear at the stern.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>telencephalon</ent><br/
<hw>telencephalon</hw> <pos>n.</pos> <sn>1.</sn> <def>the anterior division of the forebrain; the cerebrum and related parts of the hypothalamus.</def><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Telenergy</ent><br/
<hw>Tel*en"er*gy</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Gr. <grk>th^le</grk> far + <ets>energy</ets>.]</ety> <def>Display of force or energy at a distance, or without contact; now more commonly called <altname>telekinesis</altname>; -- applied to mediumistic phenomena.</def> -- <wordforms><wf>Tel`en*er"gic</wf> <pr>(#)</pr>, <pos>a.</pos></wordforms><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Telengiscope</ent><br/
<hw>Te*len"gi*scope</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Gr. <grk>th^le</grk> far + <?/ near + <ets>-scope</ets>.]</ety> <fld>(Optics)</fld> <def>An instrument of such focal length that it may be used as an observing telescope for objects close at hand or as a long-focused microscope.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Teleocephali</ent><br/
||<hw>Te`le*o*ceph"a*li</hw> <pr>(t<emac/`l<esl/*<osl/*s<ecr/f"<adot/*l<imac/ <it>or</it> t<ecr/`l<esl/*<osl/*s<ecr/f"<adot/*l<imac/)</pr>, <pos>n. pl.</pos> <ety>[NL., fr. Gr. <grk>te`leos</grk> complete + <grk>kefalh`</grk> head.]</ety> <fld>(Zool.)</fld> <def>An extensive order of bony fishes including most of the common market species, as bass, salmon, cod, perch, etc.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Teleological</ent><br/
<hw>Te`le*o*log"ic*al</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[Cf. F. <ets>t<eacute/l<eacute/ologique</ets>.]</ety> <sn>1.</sn> <fld>(Biol.)</fld> <def>Of or pertaining to teleology, or the doctrine of design.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>showing evidence of design or purpose, especially in natural phenomena.</def><br/
[<source>PJC</source>]</p>

<p> -- <wordforms><wf>Te`le*o*log"ic*al*ly</wf>, <pos>adv.</pos></wordforms><br/
[<source>1913 Webster</source>]</p>

<p><ent>Teleologist</ent><br/
<hw>Te`le*ol"o*gist</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Biol.)</fld> <def>One versed in teleology.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Teleology</ent><br/
<hw>Te`le*ol"o*gy</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Gr. <?/, <grk>teleos</grk>, the end or issue + <ets>-logy</ets>: cf. F. <ets>t<eacute/l<eacute/ologie</ets>.]</ety> <def>The doctrine of the final causes of things</def>; specif. <fld>(Biol.)</fld>, <def>the doctrine of design, which assumes that the phenomena of organic life, particularly those of evolution, are explicable only by purposive causes, and that they in no way admit of a mechanical explanation or one based entirely on biological science; the doctrine of adaptation to purpose.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Teleophore</ent><br/
<hw>Te"le*o*phore`</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Gr. <grk>teleos</grk> complete + <grk>fe`rein</grk> to bear.]</ety> <fld>(Zool.)</fld> <def>Same as <er>Gonotheca</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Teleorganic</ent><br/
<hw>Te`le*or*gan"ic</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[Gr. <grk>teleos</grk> complete + E. <ets>organic</ets>.]</ety> <fld>(Physiol.)</fld> <def>Vital; <as>as, <ex>teleorganic</ex> functions</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Teleosaur</ent><br/
<hw>Te`le*o*saur"</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Paleon.)</fld> <def>Any one of several species of fossil suarians belonging to Teleosaurus and allied genera. These reptiles are related to the crocodiles, but have biconcave vertebrae.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Teleosaurus</ent><br/
||<hw>Te`le*o*sau"rus</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[NL., fr. Gr. <?/ complete, perfect + <?/ a lizard.]</ety> <fld>(Paleon.)</fld> <def>A genus of extinct crocodilian reptiles of the Jurassic period, having a long and slender snout.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Teleost</ent><br/
<hw>Te"le*ost</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Gr. <?/ complete + <?/ bone.]</ety> <fld>(Zool.)</fld> <def>One of the Teleosti. Also used adjectively.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Teleostean</ent><br/
<hw>Te`le*os"te*an</hw> <pr>(?)</pr>, <pos>a.</pos> <fld>(Zool.)</fld> <def>Of or pertaining to the teleosts.</def> -- <def2><pos>n.</pos> <def>A teleostean fish.</def></def2><br/
[<source>1913 Webster</source>]</p>

<p><ent>Teleostei</ent><br/
||<hw>Te`le*os"te*i</hw> <pr>(?)</pr>, <pos>n. pl.</pos> <ety>[NL., fr. Gr. <?/ complete + <?/ bone.]</ety> <fld>(Zool.)</fld> <def>A subclass of fishes including all the ordinary bony fishes as distinguished from the ganoids.</def><br/
[<source>1913 Webster</source>]</p>

<p><-- p. 1481 --><br/
[<source>1913 Webster</source>]</p>

<p><ent>Teleostomi</ent><br/
||<hw>Te"le*os`to*mi</hw> <pr>(?)</pr>, <pos>n. pl.</pos> <ety>[NL., fr. Gr. <?/ complete + <?/ mouth.]</ety> <fld>(Zool.)</fld> <def>An extensive division of fishes including the ordinary fishes (Teleostei) and the ganoids.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Teleozoic</ent><br/
<hw>Te`le*o*zo"ic</hw> <pr>(?)</pr>, <pos>a.</pos> <fld>(Zool.)</fld> <def>Having tissued composed of cells.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Teleozoon</ent><br/
<hw>Te*le*o*zo"on</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Zool.)</fld> <def>A metazoan.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Telepathy</ent><br/
<hw>Te*lep"a*thy</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Gr. <?/ far off + <?/, <?/, to suffer.]</ety> <def>The sympathetic affection of one mind by the thoughts, feelings, or emotions of another at a distance, without communication through the ordinary channels of sensation.</def> <note>The existence of this ability has not been proven scientifically.</note>-- <wordforms><wf>Tel`e*path"ic</wf>, <pos>a.</pos> -- <wf>Te*lep"a*thist</wf>, <pos>n.</pos></wordforms><br/
[<source>1913 Webster</source> <source>+PJC</source>]</p>

<p><ent>Telepheme</ent><br/
<hw>Tel"e*pheme</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Gr. <?/ afar + <?/ a saying.]</ety> <def>A message by a telephone.</def> <mark>[Recent]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Telephone</ent><br/
<hw>Tel"e*phone</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Gr. <?/ far off + <?/ sound.]</ety> <fld>(Physics)</fld> <def>An instrument for reproducing sounds, especially articulate speech, at a distance.</def><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ The ordinary telephone consists essentially of a device by which currents of electricity, produced by sounds through the agency of certain mechanical devices and exactly corresponding in duration and intensity to the vibrations of the air which attend them, are transmitted to a distant station, and there, acting on suitable mechanism, reproduce similar sounds by repeating the vibrations. The necessary variations in the electrical currents are usually produced by means of a microphone attached to a thin diaphragm upon which the voice acts, and are intensified by means of an induction coil. In the <xex>magnetic telephone</xex>, or <xex>magneto-telephone</xex>, the diaphragm is of soft iron placed close to the pole of a magnet upon which is wound a coil of fine wire, and its vibrations produce corresponding vibrable currents in the wire by induction. The <xex>mechanical</xex>, or <xex>string</xex>, <xex>telephone</xex> is a device in which the voice or sound causes vibrations in a thin diaphragm, which are directly transmitted along a wire or string connecting it to a similar diaphragm at the remote station, thus reproducing the sound. It does not employ electricity.</note><br/
[<source>1913 Webster</source>]</p>

<p><ent>Telephone</ent><br/
<hw>Tel"e*phone</hw>, <pos>v. t.</pos> <def>To convey or announce by telephone.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Telephone exchange</ent><br/
<hw>Tel`e*phone ex*change"</hw>. <def>A central office in which the  wires of telephones from local subscribers may be connected by switches to other local telephones or to long-distance lines, to permit transmission of conversation or data.</def> <note>In the late 1990's the traditional copper wires connecting local telephones to the telephone exchange have begun to be replaced with optical fiber connections.</note><br/
[<source>Webster 1913 Suppl.</source> <source>+PJC</source>]</p>

<p><ent>Telephonic</ent><br/
<hw>Tel`e*phon"ic</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[Cf. F. <ets>t<eacute/l<eacute/phonique</ets>. See <er>Telephone</er>.]</ety> <sn>1.</sn> <def>Conveying sound to a great distance.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Of or pertaining to the telephone; by the telephone.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Telephonically</ent><br/
<hw>Tel`e*phon"ic*al*ly</hw> <pr>(?)</pr>, <pos>adv.</pos> <def>By telephonic means or processes; by the use of the telephone.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Telephony</ent><br/
<hw>Te*leph"o*ny</hw> <pr>(?)</pr>, <pos>n.</pos> <def>The art or process of reproducing sounds at a distance, as with the telephone.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Telephote</ent><br/
<hw>Tel"e*phote</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Gr. <grk>th^le</grk> far + <grk>fw^s</grk>, <grk>fwto`s</grk>, light.]</ety> <def>A telelectric apparatus for producing images of visible objects at a distance.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Telephoto</ent><br/
<hw>Tel`e*pho"to</hw> <pr>(?)</pr>, <pos>a.</pos> <sn>1.</sn> <def>of or relating to photography with a <er>telephoto lens</er>.</def><br/
[<source>PJC</source>]</p>

<p><sn>2.</sn> <def>of or pertaining to a <er>telephoto lens</er>.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>telephoto</ent><br/
<hw>telephoto</hw> <pos>n.</pos> <sn>1.</sn> <def>a <er>telephoto lens</er>.</def><br/
[<source>PJC</source>]</p>

<p><sn>2.</sn> <def>a photograph taken using a <er>telephoto lens</er>.</def><br/
[<source>PJC</source>]</p>

<p><ent>Telephotograph</ent><br/
<hw>Tel`e*pho"to*graph</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Gr. <grk>th^le</grk> far + <ets>photograph</ets>.]</ety> <def>a photograph transmitted and reproduced by telephotography.</def><br/
[<source>Webster 1913 Suppl.</source> + <source>WordNet 1.5</source>]</p>

<p><sn>2.</sn> <def>a photograph made with a telephoto lens.</def><br/
<syn><b>Syn. --</b> telephoto.</syn><br/
[<source>Webster 1913 Suppl.</source> + <source>WordNet 1.5</source>]</p>

<p><ent>Telephotographic</ent><br/
<hw>Tel`e*pho`to*graph"ic</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Designating, or pertaining to, the process of telephotography.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Telephotography</ent><br/
<hw>Tel`e*pho*tog"ra*phy</hw> <pr>(?)</pr>, <pos>n.</pos> <sn>1.</sn> <def>The photography of distant objects in more enlarged form than is possible by the ordinary means, usually by a camera provided with a <er>telephoto lens</er> or mounted in place of the eyepiece of a telescope, so that the real or a magnified image falls on the sensitive plate.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><sn>2.</sn>  <def>Art or process of electrically transmitting and reproducing photographic or other pictures at a distance, especially by methods similar to those used in electric telegraphy.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><sn>3.</sn>  <def>Less properly, phototelegraphy.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Telephoto lens</ent><br/
<hw>Tel`e*pho"to lens</hw> <pos>n.</pos> <def>a special compound camera lens with a long effective focal length but used in a camera with a short focal length, allowing large images to be obtained of distant objects when used in a camera in place of an ordinary lens; -- called also <altname>telephotographic lens</altname>.</def><br/
[<source>PJC</source>]</p>

<p><ent>Telepolariscope</ent><br/
<hw>Tel`e*po*lar"i*scope</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Gr. <?/ far off + E. <ets>polariscope</ets>.]</ety> <fld>(Opt.)</fld> <def>A polariscope arranged to be attached to a telescope.</def>  <rj><au>Lockyer.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Telerythin</ent><br/
<hw>Tel`e*ryth"in</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Gr. <?/ end + E. <ets>erythrin</ets>.]</ety> <fld>(Chem.)</fld> <def>A red crystalline compound related to, or produced from, erythrin. So called because regarded as the end of the series of erythrin compounds.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Telescope</ent><br/
<hw>Tel"e*scope</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Gr. <?/ viewing afar, farseeing; <?/ far, far off + <?/ a watcher, akin to <?/ to view: cf. F. <ets>t<eacute/lescope</ets>. See <er>Telegraph</er>, and <er>-scope</er>.]</ety> <def>An optical instrument used in viewing distant objects, as the heavenly bodies.</def><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ A telescope assists the eye chiefly in two ways; first, by enlarging the visual angle under which a distant object is seen, and thus magnifying that object; and, secondly, by collecting, and conveying to the eye, a larger beam of light than would enter the naked organ, thus rendering objects distinct and visible which would otherwise be indistinct and or invisible. Its essential parts are the <xex>object glass</xex>, or <xex>concave mirror</xex>, which collects the beam of light, and forms an image of the object, and the <xex>eyeglass</xex>, which is a microscope, by which the image is magnified.</note><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Achromatic telescope</b></col>. <cd>See under <er>Achromatic</er>.</cd> -- <col><b>Aplanatic telescope</b></col>, <cd>a telescope having an aplanatic eyepiece.</cd> -- <col><b>Astronomical telescope</b></col>, <cd>a telescope which has a simple eyepiece so constructed or used as not to reverse the image formed by the object glass, and consequently exhibits objects inverted, which is not a hindrance in astronomical observations.</cd> -- <col><b>Cassegrainian telescope</b></col>, <cd>a reflecting telescope invented by <etsep>Cassegrain</etsep>, which differs from the Gregorian only in having the secondary speculum convex instead of concave, and placed nearer the large speculum. The Cassegrainian represents objects inverted; the Gregorian, in their natural position. The Melbourne telescope (see <xex>Illust.</xex> under <cref>Reflecting telescope</cref>, below) is a Cassegrainian telescope.</cd> -- <col><b>Dialytic telescope</b></col>. <cd>See under <er>Dialytic</er>.</cd> -- <col><b>Equatorial telescope</b></col>. <cd>See the Note under <er>Equatorial</er>.</cd> -- <col><b>Galilean telescope</b></col>, <cd>a refracting telescope in which the eyeglass is a concave instead of a convex lens, as in the common opera glass. This was the construction originally adopted by <etsep>Galileo</etsep>, the inventor of the instrument. It exhibits the objects erect, that is, in their natural positions.</cd> -- <col><b>Gregorian telescope</b></col>, <cd>a form of reflecting telescope. See under <er>Gregorian</er>.</cd> -- <col><b>Herschelian telescope</b></col>, <cd>a reflecting telescope of the form invented by Sir William <etsep>Herschel</etsep>, in which only one speculum is employed, by means of which an image of the object is formed near one side of the open end of the tube, and to this the eyeglass is applied directly.</cd> -- <col><b>Newtonian telescope</b></col>, <cd>a form of reflecting telescope. See under <er>Newtonian</er>.</cd> -- <col><b>Photographic telescope</b></col>, <cd>a telescope specially constructed to make photographs of the heavenly bodies.</cd> -- <col><b>Prism telescope</b></col>. <cd>See <er>Teinoscope</er>.</cd> -- <col><b>Reflecting telescope</b></col>, <cd>a telescope in which the image is formed by a speculum or mirror (or usually by two speculums, a large one at the lower end of the telescope, and the smaller one near the open end) instead of an object glass. See <cref>Gregorian, Cassegrainian, Herschelian, <and/ Newtonian, telescopes</cref>, above.</cd> -- <col><b>Refracting telescope</b></col>, <cd>a telescope in which the image is formed by refraction through an object glass.</cd> -- <col><b>Telescope carp</b></col> <fld>(Zool.)</fld>, <cd>the telescope fish.</cd> -- <col><b>Telescope fish</b></col> <fld>(Zool.)</fld>, <cd>a monstrous variety of the goldfish having very protuberant eyes.</cd> -- <col><b>Telescope fly</b></col> <fld>(Zool.)</fld>, <cd>any two-winged fly of the genus <gen>Diopsis</gen>, native of Africa and Asia. The telescope flies are remarkable for having the eyes raised on very long stalks.</cd> -- <col><b>Telescope shell</b></col> <fld>(Zool.)</fld>, <cd>an elongated gastropod (<spn>Cerithium telescopium</spn>) having numerous flattened whorls.</cd> -- <col><b>Telescope sight</b></col> <fld>(Firearms)</fld>, <cd>a slender telescope attached to the barrel, having cross wires in the eyepiece and used as a sight.</cd> -- <col><b>Terrestrial telescope</b></col>, <cd>a telescope whose eyepiece has one or two lenses more than the astronomical, for the purpose of inverting the image, and exhibiting objects erect.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Telescope</ent><br/
<hw>Tel"e*scope</hw> <pr>(?)</pr>, <pos>v. i.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Telescoped</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Telescoping</conjf> <pr>(?)</pr>.]</vmorph> <def>To slide or pass one within another, after the manner of the sections of a small telescope or spyglass; to come into collision, as railway cars, in such a manner that one runs into another; to become compressed in the manner of a telescope, due to a collision or other force.</def> <mark>[Recent]</mark><br/
[<source>1913 Webster</source> <source>+PJC</source>]</p>

<p><ent>Telescope</ent><br/
<hw>Tel"e*scope</hw>, <pos>v. t.</pos> <sn>1.</sn> <def>To cause to come into collision, so as to telescope.</def> <mark>[Recent]</mark><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>to shorten or abridge significantly; <as>as, to <ex>telescope</ex> a whole semester's lectures into one week</as>.</def><br/
[<source>PJC</source>]</p>

<p><ent>Telescope</ent><br/
<hw>Tel"e*scope</hw> <pr>(t<ecr/l"<esl/*sk<omac/p)</pr>, <pos>a.</pos> <def>Capable of being extended or compacted, like a telescope, by the sliding of joints or parts one within the other; telescopic; <as>as, a <ex>telescope</ex> bag; <ex>telescope</ex> table, etc.; -- now more commonly replaced by the term <altname>telescoping</altname>.</as></def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Telescope bag</ent><br/
<hw>Telescope bag</hw>. <def>An adjustable traveling bag consisting of two cases, the larger slipping over the other.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Telescopical</ent><br/
<ent>Telescopic</ent><br/
<mhw>{ <hw>Tel`e*scop"ic</hw> <pr>(?)</pr>, <hw>Tel`e*scop"ic*al</hw> <pr>(?)</pr>, }</mhw> <pos>a.</pos> <ety>[Cf. F. <ets>t<eacute/lescopique</ets>.]</ety> <sn>1.</sn> <def>Of or pertaining to a telescope; performed by a telescope.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Seen or discoverable only by a telescope; <as>as, <ex>telescopic</ex> stars</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>Able to discern objects at a distance; farseeing; far-reaching; <as>as, a <ex>telescopic</ex> eye; <ex>telescopic</ex> vision</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>Having the power of extension by joints sliding one within another, like the tube of a small telescope or a spyglass; especially <fld>(Mach.)</fld>, constructed of concentric tubes, either stationary, as in the <xex>telescopic</xex> boiler, or movable, as in the <xex>telescopic</xex> chimney of a war vessel, which may be put out of sight by being lowered endwise.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Telescopically</ent><br/
<hw>Tel`e*scop"ic*al*ly</hw>, <pos>adv.</pos> <def>In a telescopical manner; by or with the telescope.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Telescopic sight</ent><br/
<hw>Tel`e*scop"ic sight`</hw>. <def>A sight consisting of a small telescope, as on a compass or rifle.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Telescoping</ent><br/
<hw>Tel"e*scop`ing</hw> <pr>(t<ecr/l"<esl/*sk<omac/p`<icr/ng)</pr>, <pos>a.</pos> <def>Capable of being extended or compacted, like a telescope, by the sliding of sections or parts one within the other; telescopic; <as>as, <ex>telescoping</ex> tripod legs; a <ex>telescoping</ex> table, etc.</as>; -- a term replacing the formerly used <altname>telescope</altname>.</def><br/
[<source>Webster 1913 Suppl.</source> <source>+PJC</source>]</p>

<p><ent>Telescopist</ent><br/
<hw>Te*les"co*pist</hw> <pr>(?)</pr>, <pos>n.</pos> <def>One who uses a telescope.</def>  <rj><au>R. A. Proctor.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Telescopy</ent><br/
<hw>Te*les"co*py</hw> <pr>(?)</pr>, <pos>n.</pos> <def>The art or practice of using or making telescopes.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Teleseism</ent><br/
<hw>Tel"e*seism</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Gr. <grk>th^le</grk> far + <?/ shock.]</ety> <def>A seismic movement or shock far from the  recording instrument.</def> -- <wordforms><wf>Tel`e*seis"mic</wf> <pr>(#)</pr>, <pos>a.</pos></wordforms><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Teleseme</ent><br/
<hw>Tel"e*seme</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Gr. <grk>th^le</grk> far + <?/ sign.]</ety> <def>A system of apparatus for electric signals providing for automatic transmission of a definite number of different signals or calls, as in connection with hotel annunciators.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Telesm</ent><br/
<hw>Tel"esm</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Ar. <ets>tilism</ets>. See <er>Talisman</er>.]</ety> <def>A kind of amulet or magical charm.</def> <mark>[Obs.]</mark>  <rj><au>J. Gregory.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Telesmatical</ent><br/
<ent>Telesmatic</ent><br/
<mhw>{ <hw>Tel`es*mat"ic</hw> <pr>(?)</pr>, <hw>Tel`es*mat"ic*al</hw> <pr>(?)</pr>, }</mhw> <pos>a.</pos> <def>Of or pertaining to telesms; magical.</def>  <rj><au>J. Gregory.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Telespectroscope</ent><br/
<hw>Tel`e*spec"tro*scope</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Gr. <?/ far off + E. <ets>spectroscope</ets>.]</ety> <fld>(Astron.)</fld> <def>A spectroscope arranged to be attached to a telescope for observation of distant objects, as the sun or stars.</def>  <rj><au>Lockyer.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Telestereograph</ent><br/
<hw>Tel`e*ste"re*o*graph`</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Gr. <grk>th^le</grk> far + <ets>stereograph</ets>.]</ety> <def>An instrument for telegraphically reproducing a photograph.</def> -- <wordforms><wf>Tel`e*ste`re*og"ra*phy</wf> <pr>(#)</pr>, <pos>n.</pos></wordforms><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Telestereoscope</ent><br/
<hw>Tel`e*ste"re*o*scope</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Gr. <?/ far off + E. <ets>stereoscope</ets>.]</ety> <fld>(Opt.)</fld> <def>A stereoscope adapted to view distant natural objects or landscapes; a telescopic stereoscope.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Telestic</ent><br/
<hw>Te*les"tic</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[Gr. <?/ fit for finishing, from <?/ to finish.]</ety> <def>Tending or relating to a purpose or an end.</def> <mark>[R.]</mark>  <rj><au>Cudworth.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Telestich</ent><br/
<hw>Te*les"tich</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Gr. <?/ the end + <?/ a line, verse.]</ety> <def>A poem in which the final letters of the lines, taken consequently, make a name.  Cf. <er>Acrostic</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Telethermograph</ent><br/
<hw>Tel`e*ther"mo*graph</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Gr. <grk>th^le</grk> far + <ets>thermo-</ets> + <ets>-graph</ets>.]</ety> <fld>(Physics)</fld> <sd>(a)</sd> <def>A record of fluctuations of temperature made automatically at a distant station.</def> <sd>(b)</sd> <def>An instrument, usually electrical, making such records.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Telethermometer</ent><br/
<hw>Tel`e*ther*mom"e*ter</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Gr. <?/ far off + E. <ets>thermometer</ets>.]</ety> <fld>(Physics)</fld> <def>An apparatus for determining the temperature of a distant point, as by a thermoelectric circuit or otherwise.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Teleutospore</ent><br/
<hw>Te*leu"to*spore</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Gr. <?/ completion + E. <ets>spore</ets>.]</ety> <fld>(Bot.)</fld> <def>The thick-celled winter or resting spore of the rusts (order <ord>Uredinales</ord>), produced in late summer. See <xex>Illust.</xex> of <er>Uredospore</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Telford</ent><br/
<hw>Tel"ford</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[After Thomas <etsep>Telford</etsep>, a Scotch road engineer.]</ety> <def>Designating, or pert. to, a road pavement having a surface of small stone rolled hard and  smooth, distinguished from macadam road by its firm foundation of large stones with fragments of stone wedged tightly, in the interstices; <as>as, <ex>telford</ex> pavement, road, etc.</as></def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Telfordize</ent><br/
<hw>Tel"ford*ize</hw> <pr>(?)</pr>, <pos>v. t.</pos>  <vmorph>[<pos>imp. & p. p.</pos> <conjf>Telfordized</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Telfordizing</conjf> <pr>(?)</pr>.]</vmorph> <def>To furnish (a road) with a telford pavement.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Telharmonic</ent><br/
<hw>Tel`har*mon"ic</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Of or pertaining to telharmonium.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Telharmony</ent><br/
<ent>Telharmonium</ent><br/
<hw>Tel`har*mo"ni*um</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Gr. <grk>th^le</grk> far + <ets>harmolium</ets>.]</ety> <def>An instrument for producing music (<hw>Tel*har"mo*ny</hw> [<pr><?/</pr>]), at a distant point or points by means of alternating currents of electricity controlled by an operator who plays on a keyboard. The music is produced by a receiving instrument similar or analogous to the telephone, but not held to the ear. The pitch corresponds with frequency of alternation of current.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Telic</ent><br/
<hw>Tel"ic</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[Gr. <?/, from <?/ the end.]</ety> <fld>(Gram.)</fld> <def>Denoting the final end or purpose, as distinguished from <contr>ecbatic</contr>. See <er>Ecbatic</er>.</def>  <rj><au>Gibbs.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tell</ent><br/
<hw>Tell</hw> <pr>(t<ecr/l)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Told</conjf> <pr>(t<omac/ld)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Telling</conjf>.]</vmorph> <ety>[AS. <ets>tellan</ets>, from <ets>talu</ets> tale, number, speech; akin to D. <ets>tellen</ets> to count, G. <ets>z<aum/hlen</ets>, OHG. <ets>zellen</ets> to count, tell, say, Icel. <ets>telja</ets>, Dan. <ets>tale</ets> to speak, <ets>t<ae/lle</ets> to count. See <er>Tale</er> that which is told.]</ety> <sn>1.</sn> <def>To mention one by one, or piece by piece; to recount; to enumerate; to reckon; to number; to count; <as>as, to <ex>tell</ex> money</as>.</def> <ldquo/An heap of coin he <xex>told</xex>.<rdquo/  <rj><au>Spenser.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>He <qex>telleth</qex> the number of the stars.</q> <rj><qau>Ps. cxlvii. 4.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q><qex>Tell</qex> the joints of the body.</q> <rj><qau>Jer. Taylor.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To utter or recite in detail; to give an account of; to narrate.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Of which I shall <qex>tell</qex> all the array.</q> <rj><qau>Chaucer.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>And not a man appears to <qex>tell</qex> their fate.</q> <rj><qau>Pope.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>To make known; to publish; to disclose; to divulge.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Why didst thou not <qex>tell</qex> me that she was thy wife?</q> <rj><qau>Gen. xii. 18.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>To give instruction to; to make report to; to acquaint; to teach; to inform.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>A secret pilgrimage,<br/
That you to-day promised to <qex>tell</qex> me of?</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <def>To order; to request; to command.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>He <qex>told</qex> her not to be frightened.</q> <rj><qau>Dickens.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>6.</sn> <def>To discern so as to report; to ascertain by observing; to find out; to discover; <as>as, I can not <ex>tell</ex> where one color ends and the other begins</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>7.</sn> <def>To make account of; to regard; to reckon; to value; to estimate.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>I ne <qex>told</qex> no dainity of her love.</q> <rj><qau>Chaucer.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ <xex>Tell</xex>, though equivalent in some respect to <xex>speak</xex> and <xex>say</xex>, has not always the same application. We say, to <xex>tell</xex> truth or falsehood, to <xex>tell</xex> a number, to <xex>tell</xex> the reasons, to <xex>tell</xex> something or nothing; but we never say, to <xex>tell</xex> a speech, discourse, or oration, or to <xex>tell</xex> an argument or a lesson. It is much used in commands; <as>as, <ex>tell</ex> me the whole story; <ex>tell</ex> me all you know</as>.</note><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>To tell off</b></col>, <cd>to count; to divide.</cd> <au>Sir W. Scott.</au></cs><br/
[<source>1913 Webster</source>]</p>

<p><syn><b>Syn.</b> -- To communicate; impart; reveal; disclose; inform; acquaint; report; repeat; rehearse; recite.</syn><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tell</ent><br/
<hw>Tell</hw>, <pos>v. i.</pos> <sn>1.</sn> <def>To give an account; to make report.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>That I may publish with the voice of thankgiving, and <qex>tell</qex> of all thy wondrous works.</q> <rj><qau>Ps. xxvi. 7.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To take effect; to produce a marked effect; <as>as, every shot <ex>tells</ex>; every expression <ex>tells</ex></as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>To tell of</b></col>. <sd>(a)</sd> <cd>To speak of; to mention; to narrate or describe.</cd> <sd>(b)</sd> <cd>To inform against; to disclose some fault of.</cd> -- <col><b>To tell on</b></col>, <cd>to inform against.</cd> <mark>[Archaic & Colloq.]</mark></cs><br/
[<source>1913 Webster</source>]</p>

<p><q>Lest they should <qex>tell on</qex> us, saying, So did David.</q> <rj><qau>1 Sam. xxvii. 11.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tell</ent><br/
<hw>Tell</hw>, <pos>n.</pos> <def>That which is told; tale; account.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>I am at the end of my <qex>tell</qex>.</q> <rj><qau>Walpole.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tell</ent><br/
<hw>Tell</hw>, <pos>n.</pos> <ety>[Ar.]</ety> <def>A hill or mound.</def>  <rj><au>W. M. Thomson.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tellable</ent><br/
<hw>Tell"a*ble</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Capable of being told.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tellen</ent><br/
<hw>Tel"len</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Zool.)</fld> <def>Any species of Tellina.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Teller</ent><br/
<hw>Tell"er</hw> <pr>(?)</pr>, <pos>n.</pos> <sn>1.</sn> <def>One who tells, relates, or communicates; an informer, narrator, or describer.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>One of four officers of the English Exchequer, formerly appointed to receive moneys due to the king and to pay moneys payable by the king.</def>  <rj><au>Cowell.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>An employee of a bank who receives money paid in, and pays money out, and makes records of such transactions.</def><br/
[<source>1913 Webster</source> <source>+PJC</source>]</p>

<p><sn>4.</sn> <def>One who is appointed to count the votes given in a legislative body, public meeting, assembly, etc.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tellership</ent><br/
<hw>Tell"er*ship</hw>, <pos>n.</pos> <def>The office or employment of a teller.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tellina</ent><br/
||<hw>Tel*li"na</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[NL., fr. Gr. <?/ a kind of shellfish.]</ety> <fld>(Zool.)</fld> <def>A genus of marine bivalve mollusks having thin, delicate, and often handsomely colored shells.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Telling</ent><br/
<hw>Tell"ing</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Operating with great effect; effective; <as>as, a <ex>telling</ex> speech</as>.</def> -- <wordforms><wf>Tell"ing*ly</wf>, <pos>adv.</pos></wordforms><br/
[<source>1913 Webster</source>]</p>

<p><ent>Telltale</ent><br/
<hw>Tell"tale`</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Telling tales; babbling.</def> <ldquo/The <xex>telltale</xex> heart.<rdquo/  <rj><au>Poe.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Telltale</ent><br/
<hw>Tell"tale`</hw>, <pos>n.</pos> <sn>1.</sn> <def>One who officiously communicates information of the private concerns of others; one who tells that which prudence should suppress.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Mus.)</fld> <def>A movable piece of ivory, lead, or other material, connected with the bellows of an organ, that gives notice, by its position, when the wind is exhausted.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <fld>(Naut.)</fld> <sd>(a)</sd> <def>A mechanical attachment to the steering wheel, which, in the absence of a tiller, shows the position of the helm.</def> <sd>(b)</sd> <def>A compass in the cabin of a vessel, usually placed where the captain can see it at all hours, and thus inform himself of the vessel's course.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <fld>(Mach.)</fld> <def>A machine or contrivance for indicating or recording something, particularly for keeping a check upon employees, as factory hands, watchmen, drivers, check takers, and the like, by revealing to their employers what they have done or omitted.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <fld>(Zool.)</fld> <def>The tattler. See <er>Tattler</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>6.</sn> <def>A thing that serves to disclose something or give information; a hint or indication.</def></p>

<p><q>It supplies many useful links and <qex>telltales</qex>.</q>  <rj><qau>Saintsbury.</qau></rj><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><sn>7.</sn>  <fld>(Railroads)</fld> <def>An arrangement consisting of long strips, as of rope, wire, or leather, hanging from a bar over railroad tracks, in such a position as to warn freight brakemen of their approach to a low overhead bridge.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><-- p. 1482 --><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tellural</ent><br/
<hw>Tel*lu"ral</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>tellus</ets>, <ets>-uris</ets>, the earth.]</ety> <def>Of or pertaining to the earth.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tellurate</ent><br/
<hw>Tel"lu*rate</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Cf. F. <ets>tellurate</ets>. See <er>Tellurium</er>.]</ety> <fld>(Chem.)</fld> <def>A salt of telluric acid.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Telluret</ent><br/
<hw>Tel"lu*ret</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Chem.)</fld> <def>A telluride.</def> <mark>[Obsoles.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tellureted</ent><br/
<hw>Tel"lu*ret`ed</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Chem.)</fld> <def>Combined or impregnated with tellurium; tellurized.</def> <altsp>[Written also <asp>telluretted</asp>.]</altsp> <mark>[Obsoles.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Tellureted hydrogen</b></col> <fld>(Chem.)</fld>, <cd>hydrogen telluride, <chform>H2Te</chform>, a gaseous substance analogous to hydrogen sulphide; -- called also <altname>tellurhydric acid</altname>.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tellurhydric</ent><br/
<hw>Tel`lur*hy"dric</hw> <pr>(?)</pr>, <pos>a.</pos> <fld>(Chem.)</fld> <def>Of, pertaining to, or designating, hydrogen telluride, which is regarded as an acid, especially when in solution.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tellurian</ent><br/
<hw>Tel*lu"ri*an</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>tellus</ets>, <ets>-uris</ets>, the earth.]</ety> <def>Of or pertaining to the earth.</def>  <rj><au>De Quincey.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tellurian</ent><br/
<hw>Tel*lu"ri*an</hw>, <pos>n.</pos> <sn>1.</sn> <def>A dweller on the earth.</def>  <rj><au>De Quincey.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>An instrument for showing the operation of the causes which produce the succession of day and night, and the changes of the seasons.</def> <altsp>[Written also <asp>tellurion</asp>.]</altsp><br/
[<source>1913 Webster</source>]</p>

<p><ent>Telluric</ent><br/
<hw>Tel*lu"ric</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>tellus</ets>, <ets>-uris</ets>, the earth: cf. F. <ets>tellurique</ets>.]</ety> <sn>1.</sn> <def>Of or pertaining to the earth; proceeding from the earth.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Amid these hot, <qex>telluric</qex> flames.</q> <rj><qau>Carlyle.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Chem.)</fld> <def>Of or pertaining to tellurium; derived from, or resembling, tellurium; specifically, designating those compounds in which the element has a higher valence as contrasted with <contr>tellurous</contr> compounds; <as>as, <ex>telluric acid</ex>, which is analogous to sulphuric acid</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Telluric bismuth</b></col> <fld>(Min.)</fld>, <cd>tetradymite.</cd> -- <col><b>Telluric silver</b></col> <fld>(Min.)</fld>, <cd>hessite.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Telluride</ent><br/
<hw>Tel"lu*ride</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Chem.)</fld> <def>A compound of tellurium with a more positive element or radical; -- formerly called <altname>telluret</altname>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tellurism</ent><br/
<hw>Tel"lu*rism</hw> <pr>(?)</pr>, <pos>n.</pos> <def>An hypothesis of animal magnetism propounded by Dr. Keiser, in Germany, in which the phenomena are ascribed to the agency of a telluric spirit or influence.</def> <mark>[R.]</mark>  <rj><au>S. Thompson.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tellurite</ent><br/
<hw>Tel"lu*rite</hw> <pr>(?)</pr>, <pos>n.</pos> <sn>1.</sn> <fld>(Chem.)</fld> <def>A salt of tellurous acid.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Min.)</fld> <def>Oxide of tellurium. It occurs sparingly in tufts of white or yellowish crystals.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tellurium</ent><br/
<hw>Tel*lu"ri*um</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[NL., from L. <ets>tellus</ets>, <ets>-uris</ets>, the earth.]</ety> <fld>(Chem.)</fld> <def>A rare nonmetallic element, analogous to sulphur and selenium, occasionally found native as a substance of a silver-white metallic luster, but usually combined with metals, as with gold and silver in the mineral sylvanite, with mercury in Coloradoite, etc. Symbol Te. Atomic weight 125.2.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Graphic tellurium</b></col>. <fld>(Min.)</fld> <cd>See <er>Sylvanite</er>.</cd> -- <col><b>Tellurium glance</b></col> <fld>(Min.)</fld>, <cd>nagyagite; -- called also <altname>black tellurium</altname>.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tellurize</ent><br/
<hw>Tel"lu*rize</hw> <pr>(?)</pr>, <pos>v. t.</pos> <fld>(Chem.)</fld> <def>To impregnate with, or to subject to the action of, tellurium; -- chiefly used adjectively in the past participle; <as>as, <ex>tellurized</ex> ores</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tellurous</ent><br/
<hw>Tel"lu*rous</hw> <pr>(?)</pr>, <pos>a.</pos> <fld>(Chem.)</fld> <def>Of or pertaining to tellurium; derived from, or containing, tellurium; specifically, designating those compounds in which the element has a lower valence as contrasted with <contr>telluric</contr> compounds; <as>as, <ex>tellurous acid</ex>, which is analogous to sulphurous acid</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Telodynamic</ent><br/
<hw>Tel`o*dy*nam"ic</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[Gr. <?/ far + E. <ets>dynamic</ets>.]</ety> <def>Relating to a system for transmitting power to a distance by means of swiftly moving ropes or cables driving grooved pulleys of large diameter.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Teloogoo</ent><br/
<hw>Tel`oo*goo"</hw> <pr>(?)</pr>, <pos>n.</pos> <def>See <er>Telugu</er>.</def>  <rj><au>D. O. Allen.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Telotrocha</ent><br/
||<hw>Te*lot"ro*cha</hw> <pr>(?)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Telotrochae</plw> <pr>(#)</pr>.</plu> <ety>[NL. See <er>Telotrochal</er>.]</ety> <fld>(Zool.)</fld> <def>An annelid larva having telotrochal bands of cilia.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Telotrochous</ent><br/
<ent>Telotrochal</ent><br/
<mhw>{ <hw>Te*lot"ro*chal</hw> <pr>(?)</pr>, <hw>Te*lot"ro*chous</hw> <pr>(?)</pr>, }</mhw> <pos>a.</pos> <ety>[Gr. <?/ complete + <?/ wheel, hoop.]</ety> <fld>(Zool.)</fld> <def>Having both a preoral and a posterior band of cilla; -- applied to the larvae of certain annelids.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Telotype</ent><br/
<hw>Tel"o*type</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Gr. <?/ far off + <ets>-type</ets>.]</ety> <def>An electric telegraph which prints the messages in letters and not in signs.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Telpher</ent><br/
<hw>Tel"pher</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Gr. <?/ far, far off + <grk>fe`rein</grk> to bear.]</ety> <sn>1.</sn> <fld>(Elec.)</fld> <def>A contrivance for the conveyance of vehicles or loads by means of electricity.</def>  <rj><au>Fleeming Jenkin.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Elec.)</fld> <def>Specif., the equipment or apparatus used in a system of electric transportation by means of carriages which are suspended on an overhead conductor, as of wire.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><cs><mcol><col><b>Telpher line</b></col>, <it>or</it> <col><b>Telpher road</b></col></mcol>, <cd>an electric line or road over which vehicles for carrying loads are moved by electric engines actuated by a current conveyed by the line.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Telpherage</ent><br/
<hw>Tel"pher*age</hw> <pr>(?)</pr>, <pos>n.</pos> <sn>1.</sn> <def>The conveyance of vehicles or loads by means of electricity.</def>  <rj><au>Fleeming Jenkin.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Elec.)</fld> <def>Specif., electric transportation of goods by means of carriages suspended on overhead conductors, as of wire, the power being conveyed to the motor carriage by the wires on which it runs. <ex>Telpherage</ex> and <xex>telpher</xex> are sometimes applied to such systems in which the motive power is not electricity.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Telson</ent><br/
<hw>Tel"son</hw> <pr>(?)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Telsons</plw> <pr>(#)</pr>.</plu> <ety>[NL., fr. Gr. <?/ a boundary, limit.]</ety> <fld>(Zool.)</fld> <def>The terminal joint or movable piece at the end of the abdomen of Crustacea and other articulates. See <er>Thoracostraca</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Telugu</ent><br/
<hw>Tel`u*gu"</hw> <pr>(?)</pr>, <pos>n.</pos> <sn>1.</sn> <def>A Darvidian language spoken in the northern parts of the Madras presidency. In extent of use it is the next language after Hindustani (in its various forms) and Bengali.</def> <altsp>[Spelt also <asp>Teloogoo</asp>.]</altsp><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>One of the people speaking the Telugu language.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Telugu</ent><br/
<hw>Tel`u*gu"</hw>, <pos>a.</pos> <def>Of or pertaining to the Telugu language, or the Telugus.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Temblor</ent><br/
||<hw>Tem*blor"</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Sp.]</ety> <def>An earthquake.</def> <mark>[Western U. S.]</mark><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Temerarious</ent><br/
<hw>Tem`er*a"ri*ous</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>temerarius</ets>. See <er>Temerity</er>.]</ety> <def>Unreasonably adventurous; despising danger; rash; headstrong; audacious; reckless; heedless.</def> -- <wordforms><wf>Tem`er*a"ri*ous*ly</wf>, <pos>adv.</pos></wordforms><br/
[<source>1913 Webster</source>]</p>

<p><q>I spake against <qex>temerarious</qex> judgment.</q> <rj><qau>Latimer.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Temeration</ent><br/
<hw>Tem`er*a"tion</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>temerare</ets> to defile.]</ety> <def>Temerity.</def> <mark>[Obs.]</mark>  <rj><au>Jer. Taylor.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Temerity</ent><br/
<hw>Te*mer"i*ty</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>temeritas</ets>, from <ets>temere</ets> by chance, rashly; perhaps akin to Skr. <ets>tamas</ets> darkness: cf. F. <ets>t<eacute/m<eacute/rit<eacute/</ets>.]</ety> <def>Unreasonable contempt of danger; extreme venturesomeness; rashness; <as>as, the <ex>temerity</ex> of a commander in war</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><syn><b>Syn.</b> -- Rashness; precipitancy; heedlessness; venturesomeness.</syn> <usage> -- <er>Temerity</er>, <er>Rashness</er>. These words are closely allied in sense, but have a slight difference in their use and application. <xex>Temerity</xex> is Latin, and <xex>rashness</xex> is Anglo-Saxon. As in many such cases, the Latin term is more select and dignified; the Anglo-Saxon more familiar and energetic. We show <xex>temerity</xex> in hasty decisions, and the conduct to which they lead. We show <xex>rashness</xex> in particular actions, as dictated by sudden impulse. It is an exhibition of <xex>temerity</xex> to approach the verge of a precipice; it is an act of <xex>rashness</xex> to jump into a river without being able to swim. <xex>Temerity</xex>, then, is an unreasonable contempt of danger; <xex>rashness</xex> is a rushing into danger from thoughtlessness or excited feeling.</usage><br/
[<source>1913 Webster</source>]</p>

<p><q>It is notorious <qex>temerity</qex> to pass sentence upon grounds uncapable of evidence.</q> <rj><qau>Barrow.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Her <qex>rush</qex> hand in evil hour<br/
Forth reaching to the fruit, she plucked, she eat.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Temerous</ent><br/
<hw>Tem"er*ous</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Temerarious.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tempean</ent><br/
<hw>Tem*pe"an</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Of or pertaining to Temple, a valley in Thessaly, celebrated by Greek poets on account of its beautiful scenery; resembling Temple; hence, beautiful; delightful; charming.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Temper</ent><br/
<hw>Tem"per</hw> <pr>(?)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Tempered</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Tempering</conjf>.]</vmorph> <ety>[AS. <ets>temprian</ets> or OF. <ets>temper</ets>, F. <ets>temp<eacute/rer</ets>, and (in sense 3) <ets>temper</ets>, L. <ets>temperare</ets>, akin to <ets>tempus</ets> time.  Cf. <er>Temporal</er>, <er>Distemper</er>, <er>Tamper</er>.]</ety> <sn>1.</sn> <def>To mingle in due proportion; to prepare by combining; to modify, as by adding some new element; to qualify, as by an ingredient; hence, to soften; to mollify; to assuage; to soothe; to calm.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Puritan austerity was so <qex>tempered</qex> by Dutch indifference, that mercy itself could not have dictated a milder system.</q> <rj><qau>Bancroft.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Woman! lovely woman! nature made thee<br/
To <qex>temper</qex> man: we had been brutes without you.</q> <rj><qau>Otway.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>But thy fire<br/
Shall be more <qex>tempered</qex>, and thy hope far higher.</q> <rj><qau>Byron.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>She [the Goddess of Justice] threw darkness and clouds about her, that <qex>tempered</qex> the light into a thousand beautiful shades and colors.</q> <rj><qau>Addison.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To fit together; to adjust; to accomodate.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Thy sustenance . . . serving to the appetite of the eater, <qex>tempered</qex> itself to every man's liking.</q> <rj><qau>Wisdom xvi. 21.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <fld>(Metal.)</fld> <def>To bring to a proper degree of hardness; <as>as, to <ex>temper</ex> iron or steel</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The <qex>tempered</qex> metals clash, and yield a silver sound.</q> <rj><qau>Dryden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>To govern; to manage.</def> <mark>[A Latinism & Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>With which the damned ghosts he governeth,<br/
And furies rules, and Tartare <qex>tempereth</qex>.</q> <rj><qau>Spenser.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <def>To moisten to a proper consistency and stir thoroughly, as clay for making brick, loam for molding, etc.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>6.</sn> <fld>(Mus.)</fld> <def>To adjust, as the mathematical scale to the actual scale, or to that in actual use.</def><br/
[<source>1913 Webster</source>]</p>

<p><syn><b>Syn.</b> -- To soften; mollify; assuage; soothe; calm.</syn><br/
[<source>1913 Webster</source>]</p>

<p><ent>Temper</ent><br/
<hw>Tem"per</hw>, <pos>n.</pos> <sn>1.</sn> <def>The state of any compound substance which results from the mixture of various ingredients; due mixture of different qualities; just combination; <as>as, the <ex>temper</ex> of mortar</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Constitution of body; temperament; in old writers, the mixture or relative proportion of the four humors, blood, choler, phlegm, and melancholy.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The exquisiteness of his [Christ's] bodily <qex>temper</qex> increased the exquisiteness of his torment.</q> <rj><qau>Fuller.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>Disposition of mind; the constitution of the mind, particularly with regard to the passions and affections; <as>as, a calm <ex>temper</ex>; a hasty <ex>temper</ex>; a fretful <ex>temper</ex>.</as></def><br/
[<source>1913 Webster</source>]</p>

<p><q>Remember with what mild<br/
And gracious <qex>temper</qex> he both heared and judged.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>The consequents of a certain ethical <qex>temper</qex>.</q> <rj><qau>J. H. Newman.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>Calmness of mind; moderation; equanimity; composure; <as>as, to keep one's <ex>temper</ex></as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>To fall with dignity, with <qex>temper</qex> rise.</q> <rj><qau>Pope.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Restore yourselves to your <qex>tempers</qex>, fathers.</q> <rj><qau>B. Jonson.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <def>Heat of mind or passion; irritation; proneness to anger; -- in a reproachful sense.</def> <mark>[Colloq.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><sn>6.</sn> <def>The state of a metal or other substance, especially as to its hardness, produced by some process of heating or cooling; <as>as, the <ex>temper</ex> of iron or steel</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>7.</sn> <def>Middle state or course; mean; medium.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>The perfect lawgiver is a just <qex>temper</qex> between the mere man of theory, who can see nothing but general principles, and the mere man of business, who can see nothing but particular circumstances.</q> <rj><qau>Macaulay.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>8.</sn> <fld>(Sugar Works)</fld> <def>Milk of lime, or other substance, employed in the process formerly used to clarify sugar.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Temper screw</b></col>, <cd>in deep well boring, an adjusting screw connecting the working beam with the rope carrying the tools, for lowering the tools as the drilling progresses.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><syn><b>Syn.</b> -- Disposition; temperament; frame; humor; mood. See <er>Disposition</er>.</syn><br/
[<source>1913 Webster</source>]</p>

<p><ent>Temper</ent><br/
<hw>Tem"per</hw>, <pos>v. i.</pos> <sn>1.</sn> <def>To accord; to agree; to act and think in conformity.</def> <mark>[Obs.]</mark>  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To have or get a proper or desired state or quality; to grow soft and pliable.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>I have him already <qex>tempering</qex> between my finger and my thumb, and shortly will I seal with him.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tempera</ent><br/
||<hw>Tem"pe*ra</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[It.]</ety> <fld>(Paint.)</fld> <def>A mode or process of painting; distemper.</def><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ The term is applied especially to early Italian painting, common vehicles of which were yolk of egg, yolk and white of egg mixed together, the white juice of the fig tree, and the like.</note><br/
[<source>1913 Webster</source>]</p>

<p><ent>Temperable</ent><br/
<hw>Tem"per*a*ble</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Capable of being tempered.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The fusible, hard, and <qex>temperable</qex> texture of metals.</q> <rj><qau>Emerson.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Temperament</ent><br/
<hw>Tem"per*a*ment</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>temperamentum</ets> a mixing in due proportion, proper measure, temperament: cf. F. <ets>temp<eacute/rament</ets>. See <er>Temper</er>, <pos>v. t.</pos>]</ety> <sn>1.</sn> <def>Internal constitution; state with respect to the relative proportion of different qualities, or constituent parts.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The common law . . . has reduced the kingdom to its just state and <qex>temperament</qex>.</q> <rj><qau>Sir M. Hale.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Due mixture of qualities; a condition brought about by mutual compromises or concessions.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>However, I forejudge not any probable expedient, any <qex>temperament</qex> that can be found in things of this nature, so disputable on their side.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>The act of tempering or modifying; adjustment, as of clashing rules, interests, passions, or the like; also, the means by which such adjustment is effected.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Wholesome <qex>temperaments</qex> of the rashness of popular assemblies.</q> <rj><qau>Sir J. Mackintosh.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>Condition with regard to heat or cold; temperature.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>Bodies are denominated <ldquo/hot<rdquo/ and <ldquo/cold<rdquo/ in proportion to the present <qex>temperament</qex> of that part of our body to which they are applied.</q> <rj><qau>Locke.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <fld>(Mus.)</fld> <def>A system of compromises in the tuning of organs, pianofortes, and the like, whereby the tones generated with the vibrations of a ground tone are mutually modified and in part canceled, until their number reduced to the actual practicable scale of twelve tones to the octave. This scale, although in so far artificial, is yet closely suggestive of its origin in nature, and this system of tuning, although not mathematically true, yet satisfies the ear, while it has the convenience that the same twelve fixed tones answer for every key or scale, C<sharp/ becoming identical with D<flat/, and so on.</def><-- = tempering --><br/
[<source>1913 Webster</source>]</p>

<p><sn>6.</sn> <fld>(Physiol.)</fld> <def>The peculiar physical and mental character of an individual, in olden times erroneously supposed to be due to individual variation in the relations and proportions of the constituent parts of the body, especially of the fluids, as the bile, blood, lymph, etc. Hence the phrases, bilious or choleric <xex>temperament</xex>, sanguine <xex>temperament</xex>, etc., implying a predominance of one of these fluids and a corresponding influence on the temperament.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Equal temperament</b></col> <fld>(Mus.)</fld>, <cd>that in which the variations from mathematically true pitch are distributed among all the keys alike.</cd> -- <col><b>Unequal temperament</b></col> <fld>(Mus.)</fld>, <cd>that in which the variations are thrown into the keys least used.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Temperamental</ent><br/
<hw>Tem`per*a*men"tal</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Of or pertaining to temperament; constitutional.</def> <mark>[R.]</mark>  <rj><au>Sir T. Browne.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Temperance</ent><br/
<hw>Tem"per*ance</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>temperantia</ets>: cf. F. <ets>temp<eacute/rance</ets>. See <er>Temper</er>, <pos>v. t.</pos>]</ety> <sn>1.</sn> <def>Habitual moderation in regard to the indulgence of the natural appetites and passions; restrained or moderate indulgence; moderation; <as>as, <ex>temperance</ex> in eating and drinking; <ex>temperance</ex> in the indulgence of joy or mirth</as>; specifically, moderation, and sometimes abstinence, in respect to using intoxicating liquors.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Moderation of passion; patience; calmness; sedateness.</def> <mark>[R.]</mark> <ldquo/A gentleman of all <xex>temperance</xex>.<rdquo/  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>He calmed his wrath with goodly <qex>temperance</qex>.</q> <rj><qau>Spenser.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>State with regard to heat or cold; temperature.</def> <mark>[Obs.]</mark> <ldquo/Tender and delicate <xex>temperance</xex>.<rdquo/  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Temperance society</b></col>, <cd>an association formed for the purpose of diminishing or stopping the use of alcoholic liquors as a beverage.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Temperancy</ent><br/
<hw>Tem"per*an*cy</hw> <pr>(?)</pr>, <pos>n.</pos> <def>Temperance.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Temperate</ent><br/
<hw>Tem"per*ate</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>temperatus</ets>, p. p. of <ets>temperare</ets>. See <er>Temper</er>, <pos>v. t.</pos>]</ety> <sn>1.</sn> <def>Moderate; not excessive; <as>as, <ex>temperate</ex> heat; a <ex>temperate</ex> climate</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Not marked with passion; not violent; cool; calm; <as>as, <ex>temperate</ex> language</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>She is not hot, but <qex>temperate</qex> as the morn.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>That sober freedom out of which there springs<br/
Our loyal passion for our <qex>temperate</qex> kings.</q> <rj><qau>Tennyson.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>Moderate in the indulgence of the natural appetites or passions; <as>as, <ex>temperate</ex> in eating and drinking</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Be sober and <qex>temperate</qex>, and you will be healthy.</q> <rj><qau>Franklin.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>Proceeding from temperance.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>The <qex>temperate</qex> sleeps, and spirits light as air.</q> <rj><qau>Pope.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Temperate zone</b></col> <fld>(Geog.)</fld>, <cd>that part of the earth which lies between either tropic and the corresponding polar circle; -- so called because the heat is less than in the torrid zone, and the cold less than in the frigid zones.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><syn><b>Syn.</b> -- Abstemious; sober; calm; cool; sedate.</syn><br/
[<source>1913 Webster</source>]</p>

<p><ent>Temperate</ent><br/
<hw>Tem"per*ate</hw> <pr>(?)</pr>, <pos>v. t.</pos> <def>To render temperate; to moderate; to soften; to temper.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>It inflames temperance, and <qex>temperates</qex> wrath.</q> <rj><qau>Marston.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Temperately</ent><br/
<hw>Tem"per*ate*ly</hw> <pr>(?)</pr>, <pos>adv.</pos> <def>In a temperate manner.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Temperateness</ent><br/
<hw>Tem"per*ate*ness</hw>, <pos>n.</pos> <def>The quality or state of being temperate; moderateness; temperance.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Temperative</ent><br/
<hw>Tem"per*a*tive</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[Cf. L. <ets>temperativus</ets> soothing.]</ety> <def>Having power to temper.</def> <mark>[R.]</mark>  <rj><au>T. Granger.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Temperature</ent><br/
<hw>Tem"per*a*ture</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F. <ets>temp<eacute/rature</ets>, L. <ets>temperatura</ets> due measure, proportion, temper, temperament.]</ety> <sn>1.</sn> <def>Constitution; state; degree of any quality.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The best composition and <qex>temperature</qex> is, to have openness in fame and opinion, secrecy in habit, dissimulation in seasonable use, and a power to feign, if there be no remedy.</q> <rj><qau>Bacon.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Memory depends upon the consistence and the <qex>temperature</qex> of the brain.</q> <rj><qau>I. Watts.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Freedom from passion; moderation.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>In that proud port, which her so goodly graceth,<br/
Most goodly <qex>temperature</qex> you may descry.</q> <rj><qau>Spenser.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <fld>(Physics)</fld> <def>Condition with respect to heat or cold, especially as indicated by the sensation produced, or by the thermometer or pyrometer; degree of heat or cold; <as>as, the <ex>temperature</ex> of the air; high <ex>temperature</ex>; low <ex>temperature</ex>; <ex>temperature</ex> of freezing or of boiling.</as></def> <note>The temperature of a liquid or a solid body as measured by a thermometer is a measure of the average kinetic energy of the consituent atoms or molecules of the body.  For other states of matter such as plasma, electromagnetic radiation, or subatomic particles, an analogous measure of the average kinetic energy may be expressed as a <ex>temperature</ex>, although it could never be measured by a traditional thermometer, let alone by sensing with the skin.</note><br/
[<source>1913 Webster</source> <source>+PJC</source>]</p>

<p><sn>4.</sn> <def>Mixture; compound.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>Made a <qex>temperature</qex> of brass and iron together.</q> <rj><qau>Holland.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <fld>(Physiol. & Med.)</fld> <def>The degree of heat of the body of a living being, esp. of the human body; also <mark>(Colloq.)</mark>, loosely, the excess of this over the  normal (of the human body 98<deg/-99.5<deg/ F., in the mouth of an adult about 98.4<deg/).</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><cs><col><b>Absolute temperature</b></col>. <fld>(Physics)</fld> <cd>See under <er>Absolute</er>.</cd> -- <col><b>Animal temperature</b></col> <fld>(Physiol.)</fld>, <cd>the nearly constant temperature maintained in the bodies of warm-blooded (<xex>homoiothermal</xex>) animals during life. The ultimate source of the heat is to be found in the potential energy of the food and the oxygen which is absorbed from the air during respiration. See <er>Homoiothermal</er>.</cd> -- <col><b>Temperature sense</b></col> <fld>(Physiol.)</fld>, <cd>the faculty of perceiving cold and warmth, and so of perceiving differences of temperature in external objects.</cd> <au>H. N. Martin.</au></cs><br/
[<source>1913 Webster</source>]</p>

<p><-- p. 1483 --><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tempered</ent><br/
<hw>Tem"pered</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Brought to a proper temper; <as>as, <ex>tempered</ex> steel</as>; having (such) a temper; -- chiefly used in composition; <as>as, a good-<ex>tempered</ex> or bad-<ex>tempered</ex> man; a well-<ex>tempered</ex> sword.</as></def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Temperer</ent><br/
<hw>Tem"per*er</hw> <pr>(?)</pr>, <pos>n.</pos> <def>One who, or that which, tempers; specifically, a machine in which lime, cement, stone, etc., are mixed with water.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tempering</ent><br/
<hw>Tem"per*ing</hw>, <pos>n.</pos> <fld>(Metal.)</fld> <def>The process of giving the requisite degree of hardness or softness to a substance, as iron and steel; especially, the process of giving to steel the degree of hardness required for various purposes, consisting usually in first plunging the article, when heated to redness, in cold water or other liquid, to give an excess of hardness, and then reheating it gradually until the hardness is reduced or drawn down to the degree required, as indicated by the color produced on a polished portion, or by the burning of oil.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Tempering color</b></col>, <cd>the shade of color that indicates the degree of temper in tempering steel, as pale straw yellow for lancets, razors, and tools for metal; dark straw yellow for penknives, screw taps, etc.; brown yellow for axes, chisels, and plane irons; yellow tinged with purple for table knives and shears; purple for swords and watch springs; blue for springs and saws; and very pale blue tinged with green, too soft for steel instruments.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Temper screw</ent><br/
<hw>Tem"per screw</hw>. <sn>1.</sn> <def>A screw link, to which is attached the rope of a rope-drilling apparatus, for feeding and slightly turning the drill jar at each stroke.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><sn>2.</sn>  <def>A set screw used for adjusting.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Tempest</ent><br/
<hw>Tem"pest</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[OF. <ets>tempeste</ets>, F. <ets>temp<ecir/te</ets>, (assumed) LL. <ets>tempesta</ets>, fr. L. <ets>tempestas</ets> a portion of time, a season, weather, storm, akin to <ets>tempus</ets> time. See <er>Temporal</er> of time.]</ety> <sn>1.</sn> <def>An extensive current of wind, rushing with great velocity and violence, and commonly attended with rain, hail, or snow; a furious storm.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>[We] caught in a fiery <qex>tempest</qex>, shall be hurled,<br/
Each on his rock transfixed.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Fig.: Any violent tumult or commotion; <as>as, a political <ex>tempest</ex>; a <ex>tempest</ex> of war, or of the passions</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>A fashionable assembly; a drum. See the Note under <er>Drum</er>, <pos>n.</pos>, 4.</def> <mark>[Archaic]</mark>  <rj><au>Smollett.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ <ex>Tempest</ex> is sometimes used in the formation of self-explaining compounds; <as>as, <ex>tempest</ex>-beaten, <ex>tempest</ex>-loving, <ex>tempest</ex>-tossed, <ex>tempest</ex>-winged, and the like</as>.</note><br/
[<source>1913 Webster</source>]</p>

<p><syn><b>Syn.</b> -- Storm; agitation; perturbation. See <er>Storm</er>.</syn><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tempest</ent><br/
<hw>Tem"pest</hw>, <pos>v. t.</pos> <ety>[Cf. OF. <ets>tempester</ets>, F. <ets>temp<ecir/ter</ets> to rage.]</ety> <def>To disturb as by a tempest.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>Part huge of bulk<br/
Wallowing unwieldy, enormous in their gait,<br/
<qex>Tempest</qex> the ocean.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tempest</ent><br/
<hw>Tem"pest</hw>, <pos>v. i.</pos> <def>To storm.</def> <mark>[Obs.]</mark>  <rj><au>B. Jonson.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tempestive</ent><br/
<hw>Tem*pes"tive</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>tempestivus</ets>.]</ety> <def>Seasonable; timely; <as>as, <ex>tempestive</ex> showers</as>.</def> <mark>[Obs.]</mark> <au>Heywood.</au> -- <wordforms><wf>Tem*pes"tive*ly</wf>, <pos>adv.</pos> <mark>[Obs.]</mark></wordforms><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tempestivily</ent><br/
<hw>Tem`pes*tiv"i*ly</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>tempestivitas</ets>.]</ety> <def>The quality, or state, of being tempestive; seasonableness.</def> <mark>[Obs.]</mark>  <rj><au>Sir T. Browne.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tempestuous</ent><br/
<hw>Tem*pes"tu*ous</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>tempestuous</ets>: cf. OF. <ets>tempestueux</ets>, F. <ets>temp<ecir/tueux</ets>.]</ety> <def>Of or pertaining to a tempest; involving or resembling a tempest; turbulent; violent; stormy; <as>as, <ex>tempestuous</ex> weather; a <ex>tempestuous</ex> night; a <ex>tempestuous</ex> debate.</as></def> -- <wordforms><wf>Tem*pes"tu*ous*ly</wf>, <pos>adv.</pos> -- <wf>Tem*pes"tu*ous*ness</wf>, <pos>n.</pos></wordforms><br/
[<source>1913 Webster</source>]</p>

<p><q>They saw the Hebrew leader,<br/
Waiting, and clutching his <qex>tempestuous</qex> beard.</q> <rj><qau>Longfellow.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Templar</ent><br/
<hw>Tem"plar</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[OE. <ets>templere</ets>, F. <ets>templier</ets>, LL. <ets>templarius</ets>. See <er>Temple</er> a church.]</ety> <sn>1.</sn> <def>One of a religious and military order first established at Jerusalem, in the early part of the 12th century, for the protection of pilgrims and of the Holy Sepulcher. These <membof>Knights Templars</membof>, or <membof>Knights of the Temple</membof>, were so named because they occupied an apartment of the palace of Bladwin II. in Jerusalem, near the Temple.</def><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ The order was first limited in numbers, and its members were bound by vows of chastity and poverty. After the conquest of Palestine by the Saracens, the Templars spread over Europe, and, by reason of their reputation for valor and piety, they were enriched by numerous donations of money and lands. The extravagances and vices of the later Templars, however, finally led to the suppression of the order by the Council of Vienne in 1312.</note><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A student of law, so called from having apartments in the Temple at London, the original buildings having belonged to the Knights Templars. See <cref>Inner Temple</cref>, and <cref>Middle Temple</cref>, under <er>Temple</er>.</def> <mark>[Eng.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>One belonged to a certain order or degree among the Freemasons, called <membof>Knights Templars</membof>. Also, one of an order among temperance men, styled <membof>Good Templars</membof>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Templar</ent><br/
<hw>Tem"plar</hw>, <pos>a.</pos> <def>Of or pertaining to a temple.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>Solitary, family, and <qex>templar</qex> devotion.</q> <rj><qau>Coleridge.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Template</ent><br/
<hw>Tem"plate</hw> <pr>(?)</pr>, <pos>n.</pos> <def>Same as <er>Templet</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Temple</ent><br/
<hw>Tem"ple</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Cf. <er>Templet</er>.]</ety> <fld>(Weaving)</fld> <def>A contrivence used in a loom for keeping the web stretched transversely.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Temple</ent><br/
<hw>Tem"ple</hw>, <pos>n.</pos> <ety>[OF. <ets>temple</ets>, F. <ets>tempe</ets>, from L. <ets>tempora</ets>, <ets>tempus</ets>; perhaps originally, the right place, the fatal spot, supposed to be the same word as <ets>tempus</ets>, <ets>temporis</ets>, the fitting or appointed time. See <er>Temporal</er> of time, and cf. <er>Tempo</er>, <er>Tense</er>, <pos>n.</pos>]</ety> <sn>1.</sn> <fld>(Anat.)</fld> <def>The space, on either side of the head, back of the eye and forehead, above the zygomatic arch and in front of the ear.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>One of the side bars of a pair of spectacles, jointed to the bows, and passing one on either side of the head to hold the spectacles in place.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Temple</ent><br/
<hw>Tem"ple</hw>, <pos>n.</pos> <ety>[AS. <ets>tempel</ets>, from L. <ets>templum</ets> a space marked out, sanctuary, temple; cf. Gr. <?/ a piece of land marked off, land dedicated to a god: cf. F. <ets>t<eacute/mple</ets>, from the Latin.  Cf. <er>Contemplate</er>.]</ety> <sn>1.</sn> <def>A place or edifice dedicated to the worship of some deity; <as>as, the <ex>temple</ex> of Jupiter at Athens, or of Juggernaut in India</as>.</def> <ldquo/The <xex>temple</xex> of mighty Mars.<rdquo/  <rj><au>Chaucer.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Jewish Antiq.)</fld> <def>The edifice erected at Jerusalem for the worship of Jehovah.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Jesus walked in the <qex>temple</qex> in Solomon's porch.</q> <rj><qau>John x. 23.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>Hence, among Christians, an edifice erected as a place of public worship; a church.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Can he whose life is a perpetual insult to the authority of God enter with any pleasure a <qex>temple</qex> consecrated to devotion and sanctified by prayer?</q> <rj><qau>Buckminster.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>Fig.: Any place in which the divine presence specially resides.</def> <ldquo/The <xex>temple</xex> of his body.<rdquo/  <rj><au>John ii. 21.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Know ye not that ye are the <qex>temple</qex> of God, and that the spirit of God dwelleth in you?</q> <rj><qau>1 Cor. iii. 16.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>The groves were God's first <qex>temples</qex>.</q> <rj><qau>Bryant.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <fld>(Mormon Ch.)</fld> <def>A building dedicated to the administration of ordinances.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><sn>6.</sn>  <def>A local organization of Odd Fellows.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><cs><mcol><col><b>Inner Temple</b></col>, <it>and</it> <col><b>Middle Temple</b></col></mcol>, <cd>two buildings, or ranges of buildings, occupied by two inns of court in London, on the site of a monastic establishment of the Knights Templars, called <xex>the Temple</xex>.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Temple</ent><br/
<hw>Tem"ple</hw> <pr>(?)</pr>, <pos>v. t.</pos> <def>To build a temple for; to appropriate a temple to; <as>as, to <ex>temple</ex> a god</as>.</def> <mark>[R.]</mark>  <rj><au>Feltham.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Templed</ent><br/
<hw>Tem"pled</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Supplied with a temple or temples, or with churches; inclosed in a temple.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>I love thy rocks and rills,<br/
Thy woods and <qex>templed</qex> hills.</q> <rj><qau>S. F. Smith.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Templet</ent><br/
<hw>Tem"plet</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[LL. <ets>templatus</ets> vaulted, from L. <ets>templum</ets> a small timber.]</ety> <altsp>[Spelt also <asp>template</asp>.]</altsp> <sn>1.</sn> <def>A gauge, pattern, or mold, commonly a thin plate or board, used as a guide to the form of the work to be executed; <as>as, a mason's or a wheelwright's <ex>templet</ex></as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Arch.)</fld> <def>A short piece of timber, iron, or stone, placed in a wall under a girder or other beam, to distribute the weight or pressure.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tempo</ent><br/
||<hw>Tem"po</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[It., fr. L. <ets>tempus</ets>. See <er>Tense</er>, <pos>n.</pos>]</ety> <fld>(Mus.)</fld> <def>The rate or degree of movement in time.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs>||<col><b>A tempo giusto</b></col> <pr>(j<oomac/s"t<osl/)</pr> <ety>[It.]</ety>, <cd>in exact time; -- sometimes, directing a return to strict time after a tempo rubato.</cd> -- <col><b>Tempo rubato</b></col>. <cd>See under <er>Rubato</er>.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Temporal</ent><br/
<hw>Tem"po*ral</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>temporalis</ets>, fr. <ets>tempora</ets> the temples: cf. F. <ets>temporal</ets>. See <er>Temple</er> a part of the head.]</ety> <fld>(Anat.)</fld> <def>Of or pertaining to the temple or temples; <as>as, the <ex>temporal</ex> bone; a <ex>temporal</ex> artery.</as></def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Temporal bone</b></col>, <cd>a very complex bone situated in the side of the skull of most mammals and containing the organ of hearing. It consists of an expanded <xex>squamosal</xex> portion above the ear, corresponding to the squamosal and zygoma of the lower vertebrates, and a thickened basal <xex>petrosal</xex> and <xex>mastoid</xex> portion, corresponding to the periotic and tympanic bones of the lower vertebrates.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Temporal</ent><br/
<hw>Tem"po*ral</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>temporalis</ets>, fr. <ets>tempus</ets>, <ets>temporis</ets>, time, portion of time, the fitting or appointed time: cf. F. <ets>temporel</ets>.  Cf. <er>Contemporaneous</er>, <er>Extempore</er>, <er>Temper</er>, <pos>v. t.</pos>, <er>Tempest</er>, <er>Temple</er> a part of the head, <er>Tense</er>, <pos>n.</pos>, <er>Thing</er>.]</ety> <sn>1.</sn> <def>Of or pertaining to time, that is, to the present life, or this world; secular, as distinguished from <xex>sacred</xex> or <xex>eternal</xex>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The things which are seen are <qex>temporal</qex>, but the things which are not seen are eternal.</q> <rj><qau>2 Cor. iv. 18.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Is this an hour for <qex>temporal</qex> affairs?</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Civil or political, as distinguished from <xex>ecclesiastical</xex>; <as>as, <ex>temporal</ex> power; <ex>temporal</ex> courts</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Lords temporal</b></col>. <cd>See under <er>Lord</er>, <pos>n.</pos></cd> -- <col><b>Temporal augment</b></col>. <cd>See the Note under <er>Augment</er>, <pos>n.</pos></cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><syn><b>Syn.</b> -- Transient; fleeting; transitory.</syn><br/
[<source>1913 Webster</source>]</p>

<p><ent>Temporal</ent><br/
<hw>Tem"po*ral</hw>, <pos>n.</pos> <def>Anything temporal or secular; a temporality; -- used chiefly in the plural.</def>  <rj><au>Dryden.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>He assigns supremacy to the pope in spirituals, and to the emperor or <qex>temporals</qex>.</q> <rj><qau>Lowell.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Temporality</ent><br/
<hw>Tem`po*ral"i*ty</hw> <pr>(?)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Temporalities</plw> <pr>(#)</pr>.</plu> <ety>[L. <ets>temporalitas</ets>, in LL., possessions of the church: cf. F. <ets>temporalit<eacute/</ets>.]</ety> <sn>1.</sn> <def>The state or quality of being temporary; -- opposed to <xex>perpetuity</xex>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>The laity; temporality.</def> <mark>[Obs.]</mark>  <rj><au>Sir T. More.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>That which pertains to temporal welfare; material interests; especially, the revenue of an ecclesiastic proceeding from lands, tenements, or lay fees, tithes, and the like; -- chiefly used in the plural.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Supreme head, . . . under God, of the spirituality and <qex>temporality</qex> of the same church.</q> <rj><qau>Fuller.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Temporally</ent><br/
<hw>Tem"po*ral*ly</hw> <pr>(?)</pr>, <pos>adv.</pos> <def>In a temporal manner; secularly.</def> <mark>[R.]</mark>  <rj><au>South.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Temporalness</ent><br/
<hw>Tem"po*ral*ness</hw>, <pos>n.</pos> <def>Worldliness.</def> <mark>[R.]</mark>  <rj><au>Cotgrave.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Temporalty</ent><br/
<hw>Tem"po*ral*ty</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[See <er>Temporality</er>.]</ety> <sn>1.</sn> <def>The laity; secular people.</def> <mark>[Obs.]</mark>  <rj><au>Abp. Abbot.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A secular possession; a temporality.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Temporaneous</ent><br/
<hw>Tem`po*ra"ne*ous</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>temporaneus</ets> happening at the right time, fr. <ets>tempus</ets>, <ets>temporis</ets>, time.]</ety> <def>Temporarity.</def> <mark>[Obs.]</mark>  <rj><au>Hallywell.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Temporarily</ent><br/
<hw>Tem"po*ra*ri*ly</hw> <pr>(?)</pr>, <pos>adv.</pos> <def>In a temporary manner; for a time.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Temporariness</ent><br/
<hw>Tem"po*ra*ri*ness</hw>, <pos>n.</pos> <def>The quality or state of being temporary; -- opposed to <xex>perpetuity</xex>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Temporary</ent><br/
<hw>Tem"po*ra*ry</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>temporarius</ets>, fr. <ets>tempus</ets>, <ets>temporis</ets>, time: cf. F. <ets>temporaire</ets>.]</ety> <def>Lasting for a time only; existing or continuing for a limited time; not permanent; <as>as, the patient has obtained <ex>temporary</ex> relief</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q><qex>Temporary</qex> government of the city.</q> <rj><qau>Motley.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Temporary star</b></col>. <fld>(Astron.)</fld> <cd>See under <er>Star</er>.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Temporist</ent><br/
<hw>Tem"po*rist</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A temporizer.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>Why, turn a <qex>temporist</qex>, row with the tide.</q> <rj><qau>Marston.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Temporization</ent><br/
<hw>Tem`po*ri*za"tion</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Cf. F. <ets>temporisation</ets>.]</ety> <def>The act of temporizing.</def>  <rj><au>Johnson.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Temporize</ent><br/
<hw>Tem"po*rize</hw> <pr>(?)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Temporized</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Temporizing</conjf> <pr>(?)</pr>.]</vmorph> <ety>[F. <ets>temporiser</ets>. See <er>Temporal</er> of time.]</ety> <sn>1.</sn> <def>To comply with the time or occasion; to humor, or yield to, the current of opinion or circumstances; also, to trim, as between two parties.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>They might their grievance inwardly complain,<br/
But outwardly they needs must <qex>temporize</qex>.</q> <rj><qau>Daniel.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To delay; to procrastinate.</def> <mark>[R.]</mark>  <rj><au>Bacon.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>To comply; to agree.</def> <mark>[Obs.]</mark>  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Temporizer</ent><br/
<hw>Tem"po*ri`zer</hw> <pr>(?)</pr>, <pos>n.</pos> <def>One who temporizes; one who yields to the time, or complies with the prevailing opinions, fashions, or occasions; a trimmer.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>A sort of <qex>temporizers</qex>, ready to embrace and maintain all that is, or shall be, proposed, in hope of preferment.</q> <rj><qau>Burton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Temporizingly</ent><br/
<hw>Tem"po*ri`zing*ly</hw> <pr>(?)</pr>, <pos>adv.</pos> <def>In a temporizing or yielding manner.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Temporo-</ent><br/
<hw>Tem"po*ro-</hw> <pr>(?)</pr>. <def>A combining form used in anatomy to indicate <xex>connection with</xex>, or <xex>relation to</xex>, <xex>the temple</xex>, or <xex>temporal bone</xex>; <as>as, <ex>temporo</ex>facial</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Temporo-auricular</ent><br/
<hw>Tem`po*ro-au*ric"u*lar</hw> <pr>(?)</pr>, <pos>a.</pos> <fld>(Anat.)</fld> <def>Of or pertaining to both the temple and the ear; <as>as, the <ex>temporo-auricular</ex> nerve</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Temporofacial</ent><br/
<hw>Tem`po*ro*fa"cial</hw> <pr>(?)</pr>, <pos>a.</pos> <fld>(Anat.)</fld> <def>Of or pertaining to both the temple and the face.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Temporomalar</ent><br/
<hw>Tem`po*ro*ma"lar</hw> <pr>(?)</pr>, <pos>a.</pos> <fld>(Anat.)</fld> <def>Of or pertaining to both the temple and the region of the malar bone; <as>as, the <ex>temporomalar</ex> nerve</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Temporomaxillary</ent><br/
<hw>Tem`po*ro*max"il*la*ry</hw> <pr>(?)</pr>, <pos>a.</pos> <fld>(Anat.)</fld> <def>Of or pertaining to both the temple or the temporal bone and the maxilla.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Temps</ent><br/
<hw>Temps</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[OF. & F., fr. L. <ets>tempus</ets>. See <er>Temporal</er> of time.]</ety> <def>Time.</def> <mark>[Obs.]</mark>  <rj><au>Chaucer.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tempse</ent><br/
<hw>Tempse</hw> <pr>(?)</pr>, <pos>n.</pos> <def>See <er>Temse</er>.</def> <mark>[Obs. or Prov. Eng.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tempt</ent><br/
<hw>Tempt</hw> <pr>(?)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Tempted</conjf>; <pos>p. pr. & vb. n.</pos> <conjf>Tempting</conjf>.]</vmorph> <ety>[OE. <ets>tempten</ets>, <ets>tenten</ets>, from OF. <ets>tempter</ets>, <ets>tenter</ets>, F. <ets>tenter</ets>, fr. L. <ets>tentare</ets>, <ets>temptare</ets>, to handle, feel, attack, to try, put to the test, urge, freq. from <ets>tendere</ets>, <ets>tentum</ets>, and <ets>tensum</ets>, to stretch. See <er>Thin</er>, and cf. <er>Attempt</er>, <er>Tend</er>, <er>Taunt</er>, <er>Tent</er> a pavilion, <er>Tent</er> to probe.]</ety> <sn>1.</sn> <def>To put to trial; to prove; to test; to try.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>God did <qex>tempt</qex> Abraham.</q> <rj><qau>Gen. xxii. 1.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Ye shall not <qex>tempt</qex> the Lord your God.</q> <rj><qau>Deut. vi. 16.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To lead, or endeavor to lead, into evil; to entice to what is wrong; to seduce.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Every man is <qex>tempted</qex> when he is drawn away of his own lust, and enticed.</q> <rj><qau>James i. 14.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>To endeavor to persuade; to induce; to invite; to incite; to provoke; to instigate.</def><br/
[<source>1913 Webster</source>]</p>

<p><q><qex>Tempt</qex> not the brave and needy to despair.</q> <rj><qau>Dryden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Nor <qex>tempt</qex> the wrath of heaven's avenging Sire.</q> <rj><qau>Pope.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>To endeavor to accomplish or reach; to attempt.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Ere leave be given to <qex>tempt</qex> the nether skies.</q> <rj><qau>Dryden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><syn><b>Syn.</b> -- To entice; allure; attract; decoy; seduce.</syn><br/
[<source>1913 Webster</source>]</p>

<p><ent>Temptability</ent><br/
<hw>Tempt`a*bil"i*ty</hw> <pr>(?)</pr>, <pos>n.</pos> <def>The quality or state of being temptable; lability to temptation.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Temptable</ent><br/
<hw>Tempt"a*ble</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Capable of being tempted; liable to be tempted.</def>  <rj><au>Cudworth.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Temptation</ent><br/
<hw>Temp*ta"tion</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[OF. <ets>temptation</ets>, <ets>tentation</ets>, F. <ets>tentation</ets>, L. <ets>tentatio</ets>.]</ety> <sn>1.</sn> <def>The act of tempting, or enticing to evil; seduction.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>When the devil had ended all the <qex>temptation</qex>, he departed from him for a season.</q> <rj><qau>Luke iv. 13.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>The state of being tempted, or enticed to evil.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Lead us not into <qex>temptation</qex>.</q> <rj><qau>Luke xi. 4.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>That which tempts; an inducement; an allurement, especially to something evil.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Dare to be great, without a guilty crown;<br/
View it, and lay the bright <qex>temptation</qex> down.</q> <rj><qau>Dryden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Temptationless</ent><br/
<hw>Temp*ta"tion*less</hw>, <pos>a.</pos> <def>Having no temptation or motive; <as>as, a <ex>temptationless</ex> sin</as>.</def> <mark>[R.]</mark>  <rj><au>Hammond.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Temptatious</ent><br/
<hw>Temp*ta"tious</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Tempting.</def> <mark>[Prov. Eng.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tempter</ent><br/
<hw>Tempt"er</hw> <pr>(?)</pr>, <pos>n.</pos> <def>One who tempts or entices; especially, Satan, or the Devil, regarded as the great enticer to evil.</def> <ldquo/Those who are bent to do wickedly will never want <xex>tempters</xex> to urge them on.<rdquo/  <rj><au>Tillotson.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>So glozed the <qex>Tempter</qex>, and his proem tuned.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tempting</ent><br/
<hw>Tempt"ing</hw>, <pos>a.</pos> <def>Adapted to entice or allure; attractive; alluring; seductive; enticing; <as>as, <ex>tempting</ex> pleasures</as>.</def> -- <wordforms><wf>Tempt"ing*ly</wf>, <pos>adv.</pos> -- <wf>Tempt"ing*ness</wf>, <pos>n.</pos></wordforms><br/
[<source>1913 Webster</source>]</p>

<p><ent>Temptress</ent><br/
<hw>Tempt"ress</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A woman who entices.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>She was my <qex>temptress</qex>, the foul provoker.</q> <rj><qau>Sir W. Scott.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Temse</ent><br/
<hw>Temse</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F. <ets>tamis</ets>, or D. <ets>tems</ets>, <ets>teems</ets>.  Cf. <er>Tamine</er>.]</ety> <def>A sieve.</def> <altsp>[Written also <asp>tems</asp>, and <asp>tempse</asp>.]</altsp> <mark>[Prov. Eng.]</mark>  <rj><au>Halliwell.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><mcol><col><b>Temse bread</b></col>, <col><b>Temsed bread</b></col>, <col><b>Temse loaf</b></col></mcol>, <cd>bread made of flour better sifted than common fluor.</cd> <mark>[Prov. Eng.]</mark></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Temulency</ent><br/
<ent>Temulence</ent><br/
<mhw>{ <hw>Tem"u*lence</hw> <pr>(?)</pr>, <hw>Tem"u*len*cy</hw> <pr>(?)</pr>, }</mhw> <pos>n.</pos> <ety>[L. <ets>temulentia</ets>.]</ety> <def>Intoxication; inebriation; drunkenness.</def> <mark>[R.]</mark> <ldquo/Their <xex>temulency</xex>.<rdquo/  <rj><au>Jer. Taylor.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Temulent</ent><br/
<hw>Tem"u*lent</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>temulentus</ets>.]</ety> <def>Intoxicated; drunken.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Temulentive</ent><br/
<hw>Tem"u*lent*ive</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Somewhat temulent; addicted to drink.</def> <mark>[R.]</mark>  <rj><au>R. Junius.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Ten</ent><br/
<hw>Ten</hw> <pr>(t<ecr/n)</pr>, <pos>a.</pos> <ety>[AS. <ets>t<emac/n</ets>, <ets>ti<eacute/n</ets>, <ets>t<ymac/n</ets>, <ets>t<emac/ne</ets>; akin to OFries. <ets>tian</ets>, OS. <ets>tehan</ets>, D. <ets>tien</ets>, G. <ets>zehn</ets>, OHG. <ets>zehan</ets>, Icel. <ets>t<imac/u</ets>, Sw. <ets>tio</ets>, Dan. <ets>ti</ets>, Goth. <ets>ta<iacute/hun</ets>, Lith. <ets>deszimt</ets>, Russ. <ets>desiate</ets>, W. <ets>deg</ets>, Ir. & Gael. <ets>deich</ets>, L. <ets>decem</ets>, Gr. <grk>de`ka</grk>, Skr. <ets>da<cced/an</ets>. <root/308.  Cf. <er>Dean</er>, <er>Decade</er>, <er>Decimal</er>, <er>December</er>, <er>Eighteen</er>, <er>Eighty</er>, <er>Teens</er>, <er>Tithe</er>.]</ety> <def>One more than nine; twice five.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>With twice <qex>ten</qex> sail I crossed the Phrygian Sea.</q> <rj><qau>Dryden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ <xex>Ten</xex> is often used, indefinitely, for <xex>several</xex>, <xex>many</xex>, and other like words.</note><br/
[<source>1913 Webster</source>]</p>

<p><q>There 's a proud modesty in merit,<br/
Averse from begging, and resolved to pay<br/
<qex>Ten</qex> times the gift it asks.</q> <rj><qau>Dryden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><-- p. 1484 --><br/
[<source>1913 Webster</source>]</p>

<p><ent>Ten</ent><br/
<hw>Ten</hw> <pr>(t<ecr/n)</pr>, <pos>n.</pos> <sn>1.</sn> <def>The number greater by one than nine; the sum of five and five; ten units of objects.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>I will not destroy it for <qex>ten's</qex> sake.</q> <rj><qau>Gen. xviii. 32.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A symbol representing ten units, as 10, <er>x</er>, or <er>X</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tenability</ent><br/
<hw>Ten`a*bil"i*ty</hw> <pr>(t<ecr/n`<adot/*b<icr/l"<icr/*t<ycr/)</pr>, <pos>n.</pos> <def>The quality or state of being tenable; tenableness.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tenable</ent><br/
<hw>Ten"a*ble</hw> <pr>(t<ecr/n"<adot/*b'l)</pr>, <pos>a.</pos> <ety>[F. <ets>tenable</ets>, fr. <ets>tenir</ets> to hold, L. <ets>tenere</ets>. See <er>Thin</er>, and cf. <er>Continue</er>, <er>Continent</er>, <er>Entertain</er>, <er>Maintain</er>, <er>Tenant</er>, <er>Tent</er>.]</ety> <def>Capable of being held, maintained, or defended, as against an assailant or objector, or against attempts to take or process; <as>as, a <ex>tenable</ex> fortress, a <ex>tenable</ex> argument</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>If you have hitherto concealed his sight,<br/
Let it be <qex>tenable</qex> in your silence still.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>I would be the last man in the world to give up his cause when it was <qex>tenable</qex>.</q> <rj><qau>Sir W. Scott.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tenableness</ent><br/
<hw>Ten`a*ble*ness</hw>, <pos>n.</pos> <def>Same as <er>Tenability</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tenace</ent><br/
<hw>Ten"ace</hw> <pr>(t<ecr/n"<asl/s)</pr>, <pos>n.</pos> <ety>[F. <ets>tenace</ets> tenacious, demeurer <ets>tenace</ets> to hold the best and third best cards and take both tricks, the adversary having to lead. See <er>Tenacious</er>.]</ety> <fld>(Whist)</fld> <def>The holding by the fourth hand of the best and third best cards of a suit led; also, sometimes, the combination of best with third best card of a suit in any hand.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tenacious</ent><br/
<hw>Te*na"cious</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>tenax</ets>, <ets>-acis</ets>, from <ets>tenere</ets> to hold. See <er>Tenable</er>, and cf. <er>Tenace</er>.]</ety> <sn>1.</sn> <def>Holding fast, or inclined to hold fast; inclined to retain what is in possession; <as>as, men <ex>tenacious</ex> of their just rights</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Apt to retain; retentive; <as>as, a <ex>tenacious</ex> memory</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>Having parts apt to adhere to each other; cohesive; tough; <as>as, steel is a <ex>tenacious</ex> metal; tar is more <ex>tenacious</ex> than oil</as>.</def>  <rj><au>Sir I. Newton.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>Apt to adhere to another substance; glutinous; viscous; sticking; adhesive.</def> <ldquo/Female feet, too weak to struggle with <xex>tenacious</xex> clay.<rdquo/  <rj><au>Cowper.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <def>Niggardly; closefisted; miserly.</def>  <rj><au>Ainsworth.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>6.</sn> <def>Holding stoutly to one's opinion or purpose; obstinate; stubborn.</def><br/
[<source>1913 Webster</source>]</p>

<p>-- <wordforms><wf>Te*na"cious*ly</wf>, <pos>adv.</pos> -- <wf>Te*na"cious*ness</wf>, <pos>n.</pos></wordforms><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tenacity</ent><br/
<hw>Te*nac"i*ty</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>tenacitas</ets>: cf. F. <ets>t<eacute/nacit<eacute/</ets>. See <er>Tenacious</er>.]</ety> <sn>1.</sn> <def>The quality or state of being tenacious; <as>as, <ex>tenacity</ex>, or retentiveness, of memory; <ex>tenacity</ex>, or persistency, of purpose</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>That quality of bodies which keeps them from parting without considerable force; cohesiveness; the effect of attraction; -- as distinguished from <xex>brittleness</xex>, <xex>fragility</xex>, <xex>mobility</xex>, etc.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>That quality of bodies which makes them adhere to other bodies; adhesiveness; viscosity.</def>  <rj><au>Holland.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <fld>(Physics)</fld> <def>The greatest longitudinal stress a substance can bear without tearing asunder, -- usually expressed with reference to a unit area of the cross section of the substance, as the number of pounds per square inch, or kilograms per square centimeter, necessary to produce rupture.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tenaculum</ent><br/
||<hw>Te*nac"u*lum</hw> <pr>(?)</pr>, <pos>n.</pos>; <plu><it>pl.</it> L. <plw>Tenacula</plw> <pr>(#)</pr>; E. <plw>Tenaculums</plw> <pr>(#)</pr>.</plu> <ety>[L., a holder, fr. <ets>tenere</ets> to hold.  Cf. <er>Tenaille</er>.]</ety> <fld>(Surg.)</fld> <def>An instrument consisting of a fine, sharp hook attached to a handle, and used mainly for taking up arteries, and the like.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tenacy</ent><br/
<hw>Ten"a*cy</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>tenacia</ets> obstinacy. See <er>Tenacious</er>.]</ety> <def>Tenaciousness; obstinacy.</def> <mark>[Obs.]</mark>  <rj><au>Barrow.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tenaille</ent><br/
<hw>Te*naille"</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F., a pair of pincers or tongs, a tenaille, fr. L. <ets>tenaculum</ets>. See <er>Tenaculum</er>.]</ety> <fld>(Fort.)</fld> <def>An outwork in the main ditch, in front of the curtain, between two bastions. See <xex>Illust.</xex> of <er>Ravelin</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tenaillon</ent><br/
<hw>Te*nail"lon</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F. See <er>Tenaille</er>.]</ety> <fld>(Fort.)</fld> <def>A work constructed on each side of the ravelins, to increase their strength, procure additional ground beyond the ditch, or cover the shoulders of the bastions.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tenancy</ent><br/
<hw>Ten"an*cy</hw> <pr>(?)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Tenacies</plw> <pr>(#)</pr>.</plu> <ety>[Cf. OF. <ets>tenace</ets>, LL. <ets>tenentia</ets>. See <er>Tenant</er>.]</ety> <fld>(Law)</fld> <sd>(a)</sd> <def>A holding, or a mode of holding, an estate; tenure; the temporary possession of what belongs to another.</def> <sd>(b)</sd> <fld>(O. Eng. Law)</fld> <def>A house for habitation, or place to live in, held of another.</def>  <rj><au>Blount. Blackstone. Wharton.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tenant</ent><br/
<hw>Ten"ant</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F. <ets>tenant</ets>, p. pr. of <ets>tenir</ets> to hold. See <er>Tenable</er>, and cf. <er>Lieutenant</er>.]</ety> <sn>1.</sn> <fld>(Law)</fld> <def>One who holds or possesses lands, or other real estate, by any kind of right, whether in fee simple, in common, in severalty, for life, for years, or at will; also, one who has the occupation or temporary possession of lands or tenements the title of which is in another; -- correlative to <xex>landlord</xex>. See Citation from <au>Blackstone</au>, under <er>Tenement</er>, 2.</def>  <rj><au>Blount. Wharton.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>One who has possession of any place; a dweller; an occupant.</def> <ldquo/Sweet <xex>tenants</xex> of this grove.<rdquo/  <rj><au>Cowper.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>The hhappy <qex>tenant</qex> of your shade.</q> <rj><qau>Cowley.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>The sister <qex>tenants</qex> of the middle deep.</q> <rj><qau>Byron.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><mcol><col><b>Tenant in capite</b></col> <ety>[L. <ets>in</ets> in + <ets>capite</ets>, abl. of <ets>caput</ets> head, chief.]</ety>, <it>or</it> <col><b>Tenant in chief</b></col></mcol>, <cd>by the laws of England, one who holds immediately of the king.  According to the feudal system, all lands in England are considered as held immediately or mediately of the king, who is styled <xex>lord paramount</xex>. Such tenants, however, are considered as having the fee of the lands and permanent possession.</cd> <au>Blackstone.</au> -- <col><b>Tenant in common</b></col>. <cd>See under <er>Common</er>.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tenant</ent><br/
<hw>Ten"ant</hw>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Tenanted</conjf>; <pos>p. pr. & vb. n.</pos> <conjf>Tenanting</conjf>.]</vmorph> <def>To hold, occupy, or possess as a tenant.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Sir Roger's estate is <qex>tenanted</qex> by persons who have served him or his ancestors.</q> <rj><qau>Addison.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tenantable</ent><br/
<hw>Ten"ant*a*ble</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Fit to be rented; in a condition suitable for a tenant.</def> -- <wordforms><wf>Ten"ant*a*ble*ness</wf>, <pos>n.</pos></wordforms><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tenantless</ent><br/
<hw>Ten"ant*less</hw>, <pos>a.</pos> <def>Having no tenants; unoccupied; <as>as, a <ex>tenantless</ex> mansion</as>.</def>  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tenantry</ent><br/
<hw>Ten"ant*ry</hw> <pr>(?)</pr>, <pos>n.</pos> <sn>1.</sn> <def>The body of tenants; <as>as, the <ex>tenantry</ex> of a manor or a kingdom</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Tenancy.</def> <mark>[Obs.]</mark>  <rj><au>Ridley.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tenant saw</ent><br/
<hw>Ten"ant saw`</hw> <pr>(?)</pr>. <def>See <cref>Tenon saw</cref>, under <er>Tenon</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tench</ent><br/
<hw>Tench</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[OF. <ets>tenche</ets>, F. <ets>tanche</ets>, L. <ets>tinca</ets>.]</ety> <fld>(Zool.)</fld> <def>A European fresh-water fish (<spn>Tinca tinca</spn>, or <spn>Tinca vulgaris</spn>) allied to the carp. It is noted for its tenacity of life.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tend</ent><br/
<hw>Tend</hw> <pr>(?)</pr>, <pos>v. t.</pos> <ety>[See <er>Tender</er> to offer.]</ety> <fld>(O. Eng. Law)</fld> <def>To make a tender of; to offer or tender.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tend</ent><br/
<hw>Tend</hw>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Tended</conjf>; <pos>p. pr. & vb. n.</pos> <conjf>Tending</conjf>.]</vmorph> <ety>[Aphetic form of <ets>attend</ets>. See <er>Attend</er>, <er>Tend</er> to move, and cf. <er>Tender</er> one that tends or attends.]</ety> <sn>1.</sn> <def>To accompany as an assistant or protector; to care for the wants of; to look after; to watch; to guard; <as>as, shepherds <ex>tend</ex> their flocks</as>.</def>  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>And flaming ministers to watch and <qex>tend</qex><br/
Their earthly charge.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>There 's not a sparrow or a wren,<br/
There 's not a blade of autumn grain,<br/
Which the four seasons do not <qex>tend</qex><br/
And tides of life and increase lend.</q> <rj><qau>Emerson.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To be attentive to; to note carefully; to attend to.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Being to descend<br/
A ladder much in height, I did not <qex>tend</qex><br/
My way well down.</q> <rj><qau>Chapman.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>To tend a vessel</b></col> <fld>(Naut.)</fld>, <cd>to manage an anchored vessel when the tide turns, so that in swinging she shall not entangle the cable.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tend</ent><br/
<hw>Tend</hw>, <pos>v. i.</pos> <sn>1.</sn> <def>To wait, as attendants or servants; to serve; to attend; -- with <xex>on</xex> or <xex>upon</xex>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Was he not companion with the riotous knights<br/
That <qex>tend</qex> upon my father?</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <ety>[F. <ets>attendre</ets>.]</ety> <def>To await; to expect.</def> <mark>[Obs.]</mark>  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tend</ent><br/
<hw>Tend</hw>, <pos>v. i.</pos> <ety>[F. <ets>tendre</ets>, L. <ets>tendere</ets>, <ets>tensum</ets> and <ets>tentum</ets>, to stretch, extend, direct one's course, tend; akin to Gr. <?/ to stretch, Skr. <ets>tan</ets>. See <er>Thin</er>, and cf. <er>Tend</er> to attend, <er>Contend</er>, <er>Intense</er>, <er>Ostensible</er>, <er>Portent</er>, <er>Tempt</er>, <er>Tender</er> to offer, <er>Tense</er>, <pos>a.</pos>]</ety> <sn>1.</sn> <def>To move in a certain direction; -- usually with <xex>to</xex> or <xex>towards</xex>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Two gentlemen <qex>tending</qex> towards that sight.</q> <rj><qau>Sir H. Wotton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Thus will this latter, as the former world,<br/
Still <qex>tend</qex> from bad to worse.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>The clouds above me to the white Alps <qex>tend</qex>.</q> <rj><qau>Byron.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To be directed, as to any end, object, or purpose; to aim; to have or give a leaning; to exert activity or influence; to serve as a means; to contribute; <as>as, our petitions, if granted, might <ex>tend</ex> to our destruction</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The thoughts of the diligent <qex>tend</qex> only to plenteousness; but of every one that is hasty only to want.</q> <rj><qau>Prov. xxi. 5.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>The laws of our religion <qex>tend</qex> to the universal happiness of mankind.</q> <rj><qau>Tillotson.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tendance</ent><br/
<hw>Tend"ance</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[See <er>Tend</er> to attend, and cf. <er>Attendance</er>.]</ety> <sn>1.</sn> <def>The act of attending or waiting; attendance.</def> <mark>[Archaic]</mark>  <rj><au>Spenser.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>The breath<br/
Of her sweet <qex>tendance</qex> hovering over him.</q> <rj><qau>Tennyson.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Persons in attendance; attendants.</def> <mark>[Obs.]</mark>  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tendence</ent><br/
<hw>Tend"ence</hw> <pr>(?)</pr>, <pos>n.</pos> <def>Tendency.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tendency</ent><br/
<hw>Tend"en*cy</hw> <pr>(?)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Tendencies</plw> <pr>(#)</pr>.</plu> <ety>[L. <ets>tendents</ets>, <ets>-entis</ets>, p. pr. of <ets>tendere</ets>: cf. F. <ets>tendance</ets>. See <er>Tend</er> to move.]</ety> <def>Direction or course toward any place, object, effect, or result; drift; causal or efficient influence to bring about an effect or result.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Writings of this kind, if conducted with candor, have a more particular <qex>tendency</qex> to the good of their country.</q> <rj><qau>Addison.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>In every experimental science, there is a <qex>tendency</qex> toward perfection.</q> <rj><qau>Macaulay.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><syn><b>Syn.</b> -- Disposition; inclination; proneness; drift; scope; aim.</syn><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tender</ent><br/
<hw>Tend"er</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[From <er>Tend</er> to attend.  Cf. <er>Attender</er>.]</ety> <sn>1.</sn> <def>One who tends; one who takes care of any person or thing; a nurse.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Naut.)</fld> <def>A vessel employed to attend other vessels, to supply them with provisions and other stores, to convey intelligence, or the like.</def><br/
[<source>1913 Webster</source>]</p>

<p><-- submarine tender, a ship which provides supplies and logistic support to submarines.  A specialization of def. 2. --><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>A car attached to a locomotive, for carrying a supply of fuel and water.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tender</ent><br/
<hw>Ten"der</hw> <pr>(?)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Tendered</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Tendering</conjf>.]</vmorph> <ety>[F. <ets>tendre</ets> to stretch, stretch out, reach, L. <ets>tendere</ets>. See <er>Tend</er> to move.]</ety> <sn>1.</sn> <fld>(Law)</fld> <def>To offer in payment or satisfaction of a demand, in order to save a penalty or forfeiture; <as>as, to <ex>tender</ex> the amount of rent or debt</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To offer in words; to present for acceptance.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>You see how all conditions, how all minds, . . . <qex>tender</qex> down<br/
Their services to Lord Timon.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tender</ent><br/
<hw>Ten"der</hw>, <pos>n.</pos> <sn>1.</sn> <fld>(Law)</fld> <def>An offer, either of money to pay a debt, or of service to be performed, in order to save a penalty or forfeiture, which would be incurred by nonpayment or nonperformance; <as>as, the <ex>tender</ex> of rent due, or of the amount of a note, with interest</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ To constitute a legal tender, such money must be offered as the law prescribes. So also the tender must be at the time and place where the rent or debt ought to be paid, and it must be to the full amount due.</note><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Any offer or proposal made for acceptance; <as>as, a <ex>tender</ex> of a loan, of service, or of friendship; a <ex>tender</ex> of a bid for a contract.</as></def><br/
[<source>1913 Webster</source>]</p>

<p><q>A free, unlimited <qex>tender</qex> of the gospel.</q> <rj><qau>South.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>The thing offered; especially, money offered in payment of an obligation.</def>  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><-- 4. (Finance) An offer to buy a certain number of shares of stock of a publicly-traded company at a fixed price, usu. in an attempt to gain control of the company. --><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Legal tender</b></col>. <cd>See under <er>Legal</er>.</cd> -- <col><b>Tender of issue</b></col> <fld>(Law)</fld>, <cd>a form of words in a pleading, by which a party offers to refer the question raised upon it to the appropriate mode of decision.</cd> <au>Burrill.</au></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tender</ent><br/
<hw>Ten"der</hw>, <pos>a.</pos> <amorph>[<pos>Compar.</pos> <adjf>Tenderer</adjf> <pr>(?)</pr>; <pos>superl.</pos> <adjf>Tenderest</adjf>.]</amorph> <ety>[F. <ets>tendre</ets>, L. <ets>tener</ets>; probably akin to <ets>tenuis</ets> thin. See <er>Thin</er>.]</ety> <sn>1.</sn> <def>Easily impressed, broken, bruised, or injured; not firm or hard; delicate; <as>as, <ex>tender</ex> plants; <ex>tender</ex> flesh; <ex>tender</ex> fruit</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Sensible to impression and pain; easily pained.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Our bodies are not naturally more <qex>tender</qex> than our faces.</q> <rj><qau>L'Estrange.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>Physically weak; not hardly or able to endure hardship; immature; effeminate.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The <qex>tender</qex> and delicate woman among you.</q> <rj><qau>Deut. xxviii. 56.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>Susceptible of the softer passions, as love, compassion, kindness; compassionate; pitiful; anxious for another's good; easily excited to pity, forgiveness, or favor; sympathetic.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The Lord is very pitiful, and of <qex>tender</qex> mercy.</q> <rj><qau>James v. 11.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>I am choleric by my nature, and <qex>tender</qex> by my temper.</q> <rj><qau>Fuller.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <def>Exciting kind concern; dear; precious.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>I love Valentine,<br/
Whose life's as <qex>tender</qex> to me as my soul!</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>6.</sn> <def>Careful to save inviolate, or not to injure; -- with <xex>of</xex>.</def> <ldquo/<xex>Tender</xex> of property.<rdquo/  <rj><au>Burke.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>The civil authority should be <qex>tender</qex> of the honor of God and religion.</q> <rj><qau>Tillotson.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>7.</sn> <def>Unwilling to cause pain; gentle; mild.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>You, that are thus so <qex>tender</qex> o'er his follies,<br/
Will never do him good.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>8.</sn> <def>Adapted to excite feeling or sympathy; expressive of the softer passions; pathetic; <as>as, <ex>tender</ex> expressions; <ex>tender</ex> expostulations; a <ex>tender</ex> strain</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>9.</sn> <def>Apt to give pain; causing grief or pain; delicate; <as>as, a <ex>tender</ex> subject</as>.</def> <ldquo/Things that are <xex>tender</xex> and unpleasing.<rdquo/  <rj><au>Bacon.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>10.</sn> <fld>(Naut.)</fld> <def>Heeling over too easily when under sail; -- said of a vessel.</def><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ <ex>Tender</ex> is sometimes used in the formation of self-explaining compounds; <as>as, <ex>tender</ex>-footed, <ex>tender</ex>-looking, <ex>tender</ex>-minded, <ex>tender</ex>-mouthed, and the like</as>.</note><br/
[<source>1913 Webster</source>]</p>

<p><syn><b>Syn.</b> -- Delicate; effeminate; soft; sensitive; compassionate; kind; humane; merciful; pitiful.</syn><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tender</ent><br/
<hw>Ten"der</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Cf. F. <ets>tendre</ets>.]</ety> <def>Regard; care; kind concern.</def> <mark>[Obs.]</mark>  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tender</ent><br/
<hw>Ten"der</hw>, <pos>v. t.</pos> <def>To have a care of; to be tender toward; hence, to regard; to esteem; to value.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>For first, next after life, he <qex>tendered</qex> her good.</q> <rj><qau>Spenser.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q><qex>Tender</qex> yourself more dearly.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>To see a prince in want would move a miser's charity. Our western princes <qex>tendered</qex> his case, which they counted might be their own.</q> <rj><qau>Fuller.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tenderfoot</ent><br/
<hw>Ten"der*foot`</hw> <pr>(?)</pr>, <pos>n.</pos> <sn>1.</sn> <def>A delicate person; one not inured to the hardship and rudeness of pioneer life.</def> <mark>[Slang, Western U. S.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>See <er>Boy scout</er>.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Tender-hearted</ent><br/
<hw>Ten"der-heart`ed</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Having great sensibility; susceptible of impressions or influence; affectionate; pitying; sensitive.</def> -- <wordforms><wf>Ten"der-heart`ed*ly</wf>, <pos>adv.</pos> -- <wf>Ten"der-heart`ed*ness</wf>, <pos>n.</pos></wordforms><br/
[<source>1913 Webster</source>]</p>

<p><q>Rehoboam was young and <qex>tender-hearted</qex>, and could not withstand them.</q> <rj><qau>2 Chron. xiii. 7.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Be ye kind one to another, <qex>tender-hearted</qex>.</q> <rj><qau>Eph. iv. 32.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tender-hefted</ent><br/
<hw>Ten"der-heft`ed</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Having great tenderness; easily moved.</def> <mark>[Obs.]</mark>  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tenderling</ent><br/
<hw>Ten"der*ling</hw> <pr>(?)</pr>, <pos>n.</pos> <sn>1.</sn> <def>One made tender by too much kindness; a fondling.</def> <mark>[R.]</mark>  <rj><au>W. Harrison (1586).</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Zool.)</fld> <def>One of the first antlers of a deer.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tenderloin</ent><br/
<hw>Ten"der*loin`</hw> <pr>(?)</pr>, <pos>n.</pos> <sn>1.</sn> <def>A strip of tender flesh on either side of the vertebral column under the short ribs, in the hind quarter of beef and pork. It consists of the psoas muscles.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn>  <def>In New York City, the region which is the center of the night life of fashionable amusement, including the  majority of the theaters, etc., centering on Broadway. The term orig. designates the old twenty-ninth police precinct, in this region, which afforded the police great opportunities for profit through conniving at vice and lawbreaking, one captain being reported to have said on being transferred there that whereas he had been eating chuck steak he would now eat tenderlion. Hence, in some other cities, a district largely devoted to night amusement, or, sometimes, to vice.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Tenderly</ent><br/
<hw>Ten"der*ly</hw>, <pos>adv.</pos> <def>In a tender manner; with tenderness; mildly; gently; softly; in a manner not to injure or give pain; with pity or affection; kindly.</def>  <rj><au>Chaucer.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tenderness</ent><br/
<hw>Ten"der*ness</hw>, <pos>n.</pos> <def>The quality or state of being tender (in any sense of the adjective).</def><br/
[<source>1913 Webster</source>]</p>

<p><syn><b>Syn.</b> -- Benignity; humanity; sensibility; benevolence; kindness; pity; clemency; mildness; mercy.</syn><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tendinous</ent><br/
<hw>Ten"di*nous</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[Cf. F. <ets>tendineux</ets>.]</ety> <sn>1.</sn> <def>Pertaining to a tendon; of the nature of tendon.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Full of tendons; sinewy; <as>as, nervous and <ex>tendinous</ex> parts of the body</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tendment</ent><br/
<hw>Tend"ment</hw> <pr>(?)</pr>, <pos>n.</pos> <def>Attendance; care.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tendon</ent><br/
<hw>Ten"don</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F., fr. L. <ets>tendere</ets> to stretch, extend. See <er>Tend</er> to move.]</ety> <fld>(Anat.)</fld> <def>A tough insensible cord, bundle, or band of fibrous connective tissue uniting a muscle with some other part; a sinew.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Tendon reflex</b></col> <fld>(Physiol.)</fld>, <cd>a kind of reflex act in which a muscle is made to contract by a blow upon its tendon. Its absence is generally a sign of disease. See <cref>Knee jerk</cref>, under <er>Knee</er>.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tendonous</ent><br/
<hw>Ten"don*ous</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Tendinous.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tendosynovitis</ent><br/
||<hw>Ten`do*syn`o*vi"tis</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[NL. See <er>Tendon</er>, and <er>Synovitis</er>.]</ety> <def>See <er>Tenosynovitis</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tendrac</ent><br/
<hw>Ten"drac</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[See <er>Tenrec</er>.]</ety> <fld>(Zool.)</fld> <def>Any one of several species of small insectivores of the family <fam>Centetidae</fam>, belonging to <gen>Ericulus</gen>, <gen>Echinope</gen>, and related genera, native of Madagascar. They are more or less spinose and resemble the hedgehog in habits. The rice tendrac (<spn>Oryzorictes hora</spn>) is very injurious to rice crops. Some of the species are called also <stype>tenrec</stype>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tendre</ent><br/
||<hw>Ten"dre</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F.]</ety> <def>Tender feeling or fondness; affection.</def></p>

<p><q>You poor friendless creatures are always having some foolish <qex>tendre</qex>.</q>  <rj><qau>Thackeray.</qau></rj><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Tendresse</ent><br/
<hw>Ten*dresse"</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F.]</ety> <def>Tender feeling; fondness.</def> <mark>[Obs., except as a French word]</mark><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Tendril</ent><br/
<hw>Ten"dril</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Shortened fr. OF. <ets>tendrillon</ets>, fr. F. <ets>tendre</ets> tender; hence, properly, the tender branch or spring of a plant: cf. F. <ets>tendrille</ets>. See <er>Tender</er>, <pos>a.</pos>, and cf. <er>Tendron</er>.]</ety> <fld>(Bot.)</fld> <def>A slender, leafless portion of a plant by which it becomes attached to a supporting body, after which the tendril usually contracts by coiling spirally.</def><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ Tendrils may represent the end of a stem, as in the grapevine; an axillary branch, as in the passion flower; stipules, as in the genus Smilax; or the end of a leaf, as in the pea.</note><br/
[<source>1913 Webster</source>]</p>

<p><-- p. 1485 --><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tendril</ent><br/
<hw>Ten"dril</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Clasping; climbing as a tendril.</def> <mark>[R.]</mark>  <rj><au>Dyer.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tendrilled</ent><br/
<ent>Tendriled</ent><br/
<mhw>{ <hw>Ten"driled</hw>, <hw>Ten"drilled</hw>  }</mhw> <pr>(?)</pr>, <pos>a.</pos> <fld>(Bot.)</fld> <def>Furnished with tendrils, or with such or so many, tendrils.</def> <ldquo/The <xex>thousand tendriled</xex> vine.<rdquo/  <rj><au>Southey.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tendron</ent><br/
<hw>Ten"dron</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F.  Cf. <er>Tendril</er>.]</ety> <def>A tendril.</def> <mark>[Obs.]</mark>  <rj><au>Holland.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tendry</ent><br/
<hw>Ten"dry</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A tender; an offer.</def> <mark>[Obs.]</mark>  <rj><au>Heylin.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tene</ent><br/
<hw>Tene</hw> <pr>(?)</pr>, <pos>n. & v.</pos> <def>See 1st and 2d <er>Teen</er>.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tenebrae</ent><br/
||<hw>Ten"e*brae</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L., pl., darkness.]</ety> <fld>(R. C. Ch.)</fld> <def>The matins and lauds for the last three days of Holy Week, commemorating the sufferings and death of Christ, -- usually sung on the afternoon or evening of Wednesday, Thursday, and Friday, instead of on the following days.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tenebricose</ent><br/
<hw>Te*neb"ri*cose`</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>tenebricosus</ets>.]</ety> <def>Tenebrous; dark; gloomy.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tenebrific</ent><br/
<hw>Ten`e*brif"ic</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>tenebrae</ets> darkness + <ets>facere</ets> to make.]</ety> <def>Rendering dark or gloomy; tenebrous; gloomy.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>It lightens, it brightens,<br/
The <qex>tenebrific</qex> scene.</q> <rj><qau>Burns.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Where light<br/
Lay fitful in a <qex>tenebrific</qex> time.</q> <rj><qau>R. Browning.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tenebrificous</ent><br/
<hw>Ten`e*brif"ic*ous</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Tenebrific.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Authors who are <qex>tenebrificous</qex> stars.</q> <rj><qau>Addison.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tenebrious</ent><br/
<hw>Te*ne"bri*ous</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Tenebrous.</def>  <rj><au>Young.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tenebrose</ent><br/
<hw>Ten"e*brose`</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Characterized by darkness or gloom; tenebrous.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tenebrosity</ent><br/
<hw>Ten`e*bros"i*ty</hw> <pr>(?)</pr>, <pos>n.</pos> <def>The quality or state of being tenebrous; tenebrousness.</def>  <rj><au>Burton.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tenebrous</ent><br/
<hw>Ten"e*brous</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>tenebrosus</ets>, fr. <ets>tenebrae</ets> darkness: cf. F. <ets>t<eacute/n<eacute/breux</ets>.]</ety> <def>Dark; gloomy; dusky; tenebrious.</def> -- <wordforms><wf>Ten"e*brous*ness</wf>, <pos>n.</pos></wordforms><br/
[<source>1913 Webster</source>]</p>

<p><q>The most dark, <qex>tenebrous</qex> night.</q> <rj><qau>J. Hall (1565).</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>The towering and <qex>tenebrous</qex> boughts of the cypress.</q> <rj><qau>Longfellow.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tenement</ent><br/
<hw>Ten"e*ment</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[OF. <ets>tenement</ets> a holding, a fief, F. <ets>t<egrave/nement</ets>, LL. <ets>tenementum</ets>, fr. L. <ets>tenere</ets> to hold. See <er>Tenant</er>.]</ety> <sn>1.</sn> <fld>(Feud. Law)</fld> <def>That which is held of another by service; property which one holds of a lord or proprietor in consideration of some military or pecuniary service; fief; fee.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Common Law)</fld> <def>Any species of permanent property that may be held, so as to create a tenancy, as lands, houses, rents, commons, an office, an advowson, a franchise, a right of common, a peerage, and the like; -- called also <altname>free tenements</altname> <it>or</it> <altname>frank tenements</altname>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The thing held is a <qex>tenement</qex>, the possessor of it a <ldquo/tenant,<rdquo/ and the manner of possession is called <ldquo/tenure.<rdquo/</q> <rj><qau>Blackstone.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>A dwelling house; a building for a habitation; also, an apartment, or suite of rooms, in a building, used by one family; often, a house erected to be rented.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>Fig.: Dwelling; abode; habitation.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Who has informed us that a rational soul can inhabit no <qex>tenement</qex>, unless it has just such a sort of frontispiece?</q> <rj><qau>Locke.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <def>A <cref>tenement house</cref>.</def><br/
[<source>PJC</source>]</p>

<p><cs><col><b>Tenement house</b></col>, <cd>commonly, a dwelling house erected for the purpose of being rented, and divided into separate apartments or tenements for families.  The term is often applied to apartment houses occupied by poor families, often overcrowded and in poor condition.</cd></cs><br/
[<source>1913 Webster</source> <source>+PJC</source>]</p>

<p><syn><b>Syn.</b> -- House; dwelling; habitation.</syn> <usage> -- <er>Tenement</er>, <er>House</er>.  There may be many <xex>houses</xex> under one roof, but they are completely separated from each other by party walls. A <xex>tenement</xex> may be detached by itself, or it may be part of a house divided off for the use of a family.  In modern usage, a <ex>tenement</ex> or <ex>tenement house</ex> most commonly refers to the meaning given for <cref>tenement house</cref>, above.</usage><br/
[<source>1913 Webster</source> <source>+PJC</source>]</p>

<p><ent>Tenemental</ent><br/
<hw>Ten`e*men"tal</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Of or pertaining to a tenement; capable of being held by tenants.</def>  <rj><au>Blackstone.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tenementary</ent><br/
<hw>Ten`e*men"ta*ry</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Capable of being leased; held by tenants.</def>  <rj><au>Spelman.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tenent</ent><br/
<hw>Ten"ent</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>tenent</ets> they hold, 3d pers. pl. pres. of <ets>tenere</ets>.]</ety> <def>A tenet.</def> <mark>[Obs.]</mark>  <rj><au>Bp. Sanderson.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Teneral</ent><br/
<hw>Ten"er*al</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>tener</ets>, <ets>-eris</ets>, tender, delicate.]</ety> <fld>(Zool.)</fld> <def>Of, pertaining to, or designating, a condition assumed by the imago of certain Neuroptera, after exclusion from the pupa. In this state the insect is soft, and has not fully attained its mature coloring.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Teneriffe</ent><br/
<hw>Ten`er*iffe"</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A white wine resembling Madeira in taste, but more tart, produced in Teneriffe, one of the Canary Islands; -- called also <altname>Vidonia</altname>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tenerity</ent><br/
<hw>Te*ner"i*ty</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>teneritas</ets>. See <er>Tender</er>, <pos>a.</pos>]</ety> <def>Tenderness.</def> <mark>[Obs.]</mark>  <rj><au>Ainsworth.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tenesmic</ent><br/
<hw>Te*nes"mic</hw> <pr>(?)</pr>, <pos>a.</pos> <fld>(Med.)</fld> <def>Of or pertaining to tenesmus; characterized by tenesmus.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tenesmus</ent><br/
||<hw>Te*nes"mus</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[NL., fr. Gr. <?/, fr. <?/ to stretch: cf. L. <ets>tenesmos</ets>.]</ety> <fld>(Med.)</fld> <def>An urgent and distressing sensation, as if a discharge from the intestines must take place, although none can be effected; -- always referred to the lower extremity of the rectum.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Vesical tenesmus</b></col>, <cd>a similar sensation as to the evacuation of urine, referred to the region of the bladder.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tenet</ent><br/
<hw>Ten"et</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>tenet</ets> he holds, fr. <ets>tenere</ets> to hold. See <er>Tenable</er>.]</ety> <def>Any opinion, principle, dogma, belief, or doctrine, which a person holds or maintains as true; <as>as, the <ex>tenets</ex> of Plato or of Cicero</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>That al animals of the land are in their kind in the sea, . . . is a <qex>tenet</qex> very questionable.</q> <rj><qau>Sir T. Browne.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>The religious <qex>tenets</qex> of his family he had early renounced with contempt.</q> <rj><qau>Macaulay.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><syn><b>Syn.</b> -- Dogma; doctrine; opinion; principle; position. See <er>Dogma</er>.</syn><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tenfold</ent><br/
<hw>Ten"fold`</hw> <pr>(?)</pr>, <pos>a. & adv.</pos> <def>In tens; consisting of ten in one; ten times repeated.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The grisly Terror . . . grew <qex>tenfold</qex><br/
More dreadful and deform.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tenia</ent><br/
||<hw>Te"ni*a</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[NL.]</ety> <def>See <er>Taenia</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tenioid</ent><br/
<hw>Te"ni*oid</hw> <pr>(?)</pr>, <pos>a.</pos> <def>See <er>Taenoid</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tennantite</ent><br/
<hw>Ten"nant*ite</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Named after <person>Smithson <etsep>Tennant</etsep></person>, an English chemist.]</ety> <fld>(Min.)</fld> <def>A blackish lead-gray mineral, closely related to tetrahedrite.  It is essentially a sulphide of arsenic and copper.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tenne</ent><br/
||<hw>Ten`n<eacute/"</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Cf. <er>Tawny</er>.]</ety> <fld>(Her.)</fld> <def>A tincture, rarely employed, which is considered as an orange color or bright brown. It is represented by diagonal lines from sinister to dexter, crossed by vertical lines.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tennis</ent><br/
<hw>Ten"nis</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[OE. <ets>tennes</ets>, <ets>tenies</ets>, <ets>tenyse</ets>; of uncertain origin, perhaps fr. F. <ets>tenez</ets> hold or take it, fr. <ets>tenir</ets> to hold (see <er>Tenable</er>).]</ety> <def>A play in which a ball is driven to and fro, or kept in motion by striking it with a racket or with the open hand.</def>  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>His easy bow, his good stories, his style of dancing and playing <qex>tennis</qex>, . . . were familiar to all London.</q> <rj><qau>Macaulay.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Court tennis</b></col>, <cd>the old game of tennis as played within walled courts of peculiar construction; -- distinguished from <contr>lawn tennis</contr>.</cd> -- <col><b>Lawn tennis</b></col>. <cd>See under <er>Lawn</er>, <pos>n.</pos></cd> -- <col><b>Tennis court</b></col>, <cd>a place or court for playing the game of tennis.</cd> <au>Shak.</au></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tennis</ent><br/
<hw>Ten"nis</hw>, <pos>v. t.</pos> <def>To drive backward and forward, as a ball in playing tennis.</def> <mark>[R.]</mark>  <rj><au>Spenser.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tenno</ent><br/
||<hw>Ten*no"</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Jap. <ets>tenn<omac/</ets>, fr. Chin. <ets>t'ien</ets> heaven + <ets>wang</ets> king.]</ety> <def>Lit., King of Heaven; -- a title of the emperor of Japan as the head of the Shinto religion.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Tennu</ent><br/
<hw>Ten"nu</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Zool.)</fld> <def>The tapir.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tennysonian</ent><br/
<hw>Ten`ny*so"ni*an</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Of or pertaining to <person>Alfred (Lord) Tennyson</person>, the English poet (1809-92); resembling, or having some of the characteristics of, his poetry, as simplicity, pictorial quality, sensuousness, etc.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Ten-o'clock</ent><br/
<hw>Ten"-o'*clock`</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Bot.)</fld> <def>A plant, the star-of-Bethlehem. See under <er>Star</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tenon</ent><br/
<hw>Ten"on</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F., fr. <ets>tenir</ets> to hold. See <er>Tenable</er>.]</ety> <fld>(Carp. & Join.)</fld> <def>A projecting member left by cutting away the wood around it, and made to insert into a mortise, and in this way secure together the parts of a frame; especially, such a member when it passes entirely through the thickness of the piece in which the mortise is cut, and shows on the other side.  Cf. <er>Tooth</er>, <er>Tusk</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Tenon saw</b></col>, <cd>a saw with a thin blade, usually stiffened by a brass or steel back, for cutting tenons.</cd> <altsp>[Corruptly written <asp>tenant saw</asp>.]</altsp> <au>Gwilt.</au></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tenon</ent><br/
<hw>Ten"on</hw>, <pos>v. t.</pos> <def>To cut or fit for insertion into a mortise, as the end of a piece of timber.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tenonian</ent><br/
<hw>Te*no"ni*an</hw> <pr>(?)</pr>, <pos>a.</pos> <fld>(Anat.)</fld> <def>Discovered or described by M. <etsep>Tenon</etsep>, a French anatomist.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Tenonian capsule</b></col> <fld>(Anat.)</fld>, <cd>a lymphatic space inclosed by a delicate membrane or fascia (the <xex>fascia of Tenon</xex>) between the eyeball and the fat of the orbit; -- called also <altname>capsule of Tenon</altname>.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tenonitis</ent><br/
||<hw>Ten`o*ni"tis</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[NL., Gr. <grk>te`nwn</grk> tendon + <ets>-itis</ets>.]</ety> <fld>(Med.)</fld> <def>Inflammation of a tendon.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Tenonitis</ent><br/
||<hw>Ten`o*ni"tis</hw>, <pos>n.</pos> <ety>[NL. See <er>Tenonian</er>, <er>-itis</er>.]</ety> <def>Inflammation of the Tenonian capsule.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Tenor</ent><br/
<hw>Ten"or</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L., from <ets>tenere</ets> to hold; hence, properly, a holding on in a continued course: cf. F. <ets>teneur</ets>. See <er>Tenable</er>, and cf. <er>Tenor</er> a kind of voice.]</ety> <sn>1.</sn> <def>A state of holding on in a continuous course; manner of continuity; constant mode; general tendency; course; career.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Along the cool sequestered vale of life<br/
They kept the noiseless <qex>tenor</qex> of their away.</q> <rj><qau>Gray.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>That course of thought which holds on through a discourse; the general drift or course of thought; purport; intent; meaning; understanding.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>When it [the bond] is paid according to the <qex>tenor</qex>.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Does not the whole <qex>tenor</qex> of the divine law positively require humility and meekness to all men?</q> <rj><qau>Spart.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>Stamp; character; nature.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>This success would look like chance, if it were perpetual, and always of the same <qex>tenor</qex>.</q> <rj><qau>Dryden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <fld>(Law)</fld> <def>An exact copy of a writing, set forth in the words and figures of it. It differs from <xex>purport</xex>, which is only the substance or general import of the instrument.</def>  <rj><au>Bouvier.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <ety>[F. <ets>t<eacute/nor</ets>, L. <ets>tenor</ets>, properly, a holding; -- so called because the tenor was the voice which took and held the principal part, the plain song, air, or tune, to which the other voices supplied a harmony above and below: cf. It. <ets>tenore</ets>.]</ety> <fld>(Mus.)</fld> <sd>(a)</sd> <def>The higher of the two kinds of voices usually belonging to adult males; hence, the part in the harmony adapted to this voice; the second of the four parts in the scale of sounds, reckoning from the base, and originally the air, to which the other parts were auxillary.</def> <sd>(b)</sd> <def>A person who sings the tenor, or the instrument that play it.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><mcol><col><b>Old Tenor</b></col>, <col><b>New Tenor</b></col>, <col><b>Middle Tenor</b></col></mcol>, <cd>different descriptions of paper money, issued at different periods, by the American colonial governments in the last century.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tenorrhaphy</ent><br/
<hw>Te*nor"rha*phy</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Gr. <grk>te`nwn</grk> tendon + <ets>-rhaphy</ets>.]</ety> <fld>(Surg.)</fld> <def>Suture of a tendon.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Tenositis</ent><br/
||<hw>Ten`o*si"tis</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[NL., fr. Gr. <grk>te`nwn</grk> tendon + <ets>-itis</ets>.]</ety> <fld>(Med.)</fld> <def>Inflammation of a tendon.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Tenosynovitis</ent><br/
||<hw>Ten`o*syn`o*vi"tis</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[NL., fr. Gr. <grk>te`nwn</grk> a tendon + E. <ets>synovitis</ets>.]</ety> <fld>(Med.)</fld> <def>Inflammation of the synovial sheath of a tendon.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tenotome</ent><br/
<hw>Ten"o*tome</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Surg.)</fld> <def>A slender knife for use in the operation of tenotomy.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tenotomy</ent><br/
<hw>Te*not"o*my</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Gr. <grk>te`nwn</grk> a tendon + <grk>te`mnein</grk> to cut.]</ety> <fld>(Surg.)</fld> <def>The division of a tendon, or the act of dividing a tendon.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tenpenny</ent><br/
<hw>Ten"pen*ny</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Valued or sold at ten pence; <as>as, a <ex>tenpenny</ex> cake</as>. See 2d <er>Penny</er>, <pos>n.</pos></def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tenpenny</ent><br/
<hw>Ten"pen*ny</hw>, <pos>a.</pos> <def>Denoting a size of nails. See 1st <er>Penny</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tenpins</ent><br/
<hw>Ten"pins</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A game resembling ninepins, but played with ten pins. See <er>Ninepins</er>.</def> <mark>[U. S.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Ten-pounder</ent><br/
<hw>Ten"-pound`er</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Zool.)</fld> <def>A large oceanic fish (<spn>Elops saurus</spn>) found in the tropical parts of all the oceans. It is used chiefly for bait.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tenrec</ent><br/
<hw>Ten"rec</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[From the native name: cf. F. <ets>tanrac</ets>, <ets>tanrec</ets>, <ets>tandrec</ets>.]</ety> <fld>(Zool.)</fld> <def>A small insectivore (<spn>Centetes ecaudatus</spn>), native of Madagascar, but introduced also into the islands of Bourbon and Mauritius; -- called also <altname>tanrec</altname>. The name is applied to other allied genera. See <er>Tendrac</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tense</ent><br/
<hw>Tense</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[OF. <ets>tens</ets>, properly, time, F. <ets>temps</ets> time, tense. See <er>Temporal</er> of time, and cf. <er>Thing</er>.]</ety> <fld>(Gram.)</fld> <def>One of the forms which a verb takes by inflection or by adding auxiliary words, so as to indicate the time of the action or event signified; the modification which verbs undergo for the indication of time.</def><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ The primary simple tenses are three: those which express time <xex>past</xex>, <xex>present</xex>, and <xex>future</xex>; but these admit of modifications, which differ in different languages.</note><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tense</ent><br/
<hw>Tense</hw>, <pos>a.</pos> <ety>[L. <ets>tensus</ets>, p. p. of <ets>tendere</ets> to stretch. See <er>Tend</er> to move, and cf. <er>Toise</er>.]</ety> <def>Stretched tightly; strained to stiffness; rigid; not lax; <as>as, a <ex>tense</ex> fiber</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The temples were sunk, her forehead was <qex>tense</qex>, and a fatal paleness was upon her.</q> <rj><qau>Goldsmith.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p>-- <wordforms><wf>Tense"ly</wf>, <pos>adv.</pos> -- <wf>Tense"ness</wf>, <pos>n.</pos></wordforms><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tensibility</ent><br/
<hw>Ten`si*bil"i*ty</hw> <pr>(?)</pr>, <pos>n.</pos> <def>The quality or state of being tensible; tensility.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tensible</ent><br/
<hw>Ten"si*ble</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[See <er>Tense</er>, <pos>a.</pos>]</ety> <def>Capable of being extended or drawn out; ductile; tensible.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Gold . . . is likewise the most flexible and <qex>tensible</qex>.</q> <rj><qau>Bacon.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tensile</ent><br/
<hw>Ten"sile</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[See <er>Tense</er>, <pos>a.</pos>]</ety> <sn>1.</sn> <def>Of or pertaining to extension; <as>as, <ex>tensile</ex> strength</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Capable of extension; ductile; tensible.</def>  <rj><au>Bacon.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tensiled</ent><br/
<hw>Ten"siled</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Made tensile.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tensility</ent><br/
<hw>Ten*sil"i*ty</hw> <pr>(?)</pr>, <pos>n.</pos> <def>The quality or state of being tensile, or capable of extension; tensibility; <as>as, the <ex>tensility</ex> of the muscles</as>.</def>  <rj><au>Dr. H. Mere.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tension</ent><br/
<hw>Ten"sion</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>tensio</ets>, from <ets>tendere</ets>, <ets>tensum</ets>, to stretch: cf. F. <ets>tension</ets>. See <er>Tense</er>, <pos>a.</pos>]</ety> <sn>1.</sn> <def>The act of stretching or straining; the state of being stretched or strained to stiffness; the state of being bent strained; <as>as, the <ex>tension</ex> of the muscles, <ex>tension</ex> of the larynx</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Fig.: Extreme strain of mind or excitement of feeling; intense effort.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>The degree of stretching to which a wire, cord, piece of timber, or the like, is strained by drawing it in the direction of its length; strain.</def>  <rj><au>Gwilt.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <fld>(Mech.)</fld> <def>The force by which a part is pulled when forming part of any system in equilibrium or in motion; <as>as, the <ex>tension</ex> of a srting supporting a weight equals that weight</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <def>A device for checking the delivery of the thread in a sewing machine, so as to give the stitch the required degree of tightness.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>6.</sn> <fld>(Physics)</fld> <def>Expansive force; the force with which the particles of a body, as a gas, tend to recede from each other and occupy a larger space; elastic force; elasticity; <as>as, the <ex>tension</ex> of vapor; the <ex>tension</ex> of air</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>7.</sn> <fld>(Elec.)</fld> <def>The quality in consequence of which an electric charge tends to discharge itself, as into the air by a spark, or to pass from a body of greater to one of less electrical potential. It varies as the quantity of electricity upon a given area.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><mcol><col><b>Tension brace</b></col>, <it>or</it> <col><b>Tension member</b></col></mcol> <fld>(Engin.)</fld>, <cd>a brace or member designed to resist tension, or subjected to tension, in a structure.</cd> -- <col><b>Tension rod</b></col> <fld>(Engin.)</fld>, <cd>an iron rod used as a tension member to strengthen timber or metal framework, roofs, or the like.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tensioned</ent><br/
<hw>Ten"sioned</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Extended or drawn out; subjected to tension.</def> <ldquo/A highly <xex>tensioned</xex> string.<rdquo/  <rj><au>Tyndall.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tensity</ent><br/
<hw>Ten"si*ty</hw> <pr>(?)</pr>, <pos>n.</pos> <def>The quality or state of being tense, or strained to stiffness; tension; tenseness.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tensive</ent><br/
<hw>Ten"sive</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[Cf. F. <ets>tensif</ets>. See <er>Tense</er>, <pos>a.</pos>]</ety> <def>Giving the sensation of tension, stiffness, or contraction.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>A <qex>tensive</qex> pain from distension of the parts.</q> <rj><qau>Floyer.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tensor</ent><br/
<hw>Ten"sor</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[NL. See <er>Tension</er>.]</ety> <sn>1.</sn> <fld>(Anat.)</fld> <def>A muscle that stretches a part, or renders it tense.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Geom.)</fld> <def>The ratio of one vector to another in length, no regard being had to the direction of the two vectors; -- so called because considered as a <xex>stretching</xex> factor in changing one vector into another. See <er>Versor</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Ten-strike</ent><br/
<hw>Ten"-strike`</hw> <pr>(?)</pr>, <pos>n.</pos> <sn>1.</sn> <fld>(Tenpins)</fld> <def>A knocking down of all ten pins at one delivery of the ball<-- also, strike-->.</def> <mark>[U. S.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Any quick, decisive stroke or act.</def> <mark>[Colloq. U. S.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tensure</ent><br/
<hw>Ten"sure</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>tensura</ets>. See <er>Tension</er>.]</ety> <def>Tension.</def> <mark>[Obs.]</mark>  <rj><au>Bacon.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tent</ent><br/
<hw>Tent</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Sp. <ets>tinto</ets>, properly, deep-colored, fr. L. <ets>tinctus</ets>, p. p. of <ets>tingere</ets> to dye. See <er>Tinge</er>, and cf. <er>Tint</er>, <er>Tinto</er>.]</ety> <def>A kind of wine of a deep red color, chiefly from Galicia or Malaga in Spain; -- called also <altname>tent wine</altname>, and <altname>tinta</altname>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tent</ent><br/
<hw>Tent</hw>, <pos>n.</pos> <ety>[Cf. <er>Attent</er>, <pos>n.</pos>]</ety> <sn>1.</sn> <def>Attention; regard, care.</def> <mark>[Obs. or Prov. Eng. & Scot.]</mark>  <rj><au>Lydgate.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Intention; design.</def> <mark>[Prov. Eng.]</mark>  <rj><au>Halliwell.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tent</ent><br/
<hw>Tent</hw>, <pos>v. t.</pos> <def>To attend to; to heed; hence, to guard; to hinder.</def> <mark>[Prov. Eng. & Scot.]</mark>  <rj><au>Halliwell.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tent</ent><br/
<hw>Tent</hw>, <pos>v. t.</pos> <ety>[OF. <ets>tenter</ets>. See <er>Tempt</er>.]</ety> <def>To probe or to search with a tent; to keep open with a tent; <as>as, to <ex>tent</ex> a wound</as>. Used also figuratively.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>I'll <qex>tent</qex> him to the quick.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tent</ent><br/
<hw>Tent</hw>, <pos>n.</pos> <ety>[F. <ets>tente</ets>. See <er>Tent</er> to probe.]</ety> <fld>(Surg.)</fld> <sd>(a)</sd> <def>A roll of lint or linen, or a conical or cylindrical piece of sponge or other absorbent, used chiefly to dilate a natural canal, to keep open the orifice of a wound, or to absorb discharges.</def> <sd>(b)</sd> <def>A probe for searching a wound.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The <qex>tent</qex> that searches<br/
To the bottom of the worst.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><-- p. 1486 --><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tent</ent><br/
<hw>Tent</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[OE. <ets>tente</ets>, F. <ets>tente</ets>, LL. <ets>tenta</ets>, fr. L. <ets>tendere</ets>, <ets>tentum</ets>, to stretch. See <er>Tend</er> to move, and cf. <er>Tent</er> a roll of lint.]</ety> <sn>1.</sn> <def>A pavilion or portable lodge consisting of skins, canvas, or some strong cloth, stretched and sustained by poles, -- used for sheltering persons from the weather, especially soldiers in camp.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Within his <qex>tent</qex>, large as is a barn.</q> <rj><qau>Chaucer.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Her.)</fld> <def>The representation of a tent used as a bearing.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Tent bed</b></col>, <cd>a high-post bedstead curtained with a tentlike canopy.</cd> -- <col><b>Tent caterpillar</b></col> <fld>(Zool.)</fld>, <cd>any one of several species of gregarious caterpillars which construct on trees large silken webs into which they retreat when at rest. Some of the species are very destructive to fruit trees. The most common American species is the larva of a bombycid moth (<spn>Clisiocampa Americana</spn>). Called also <altname>lackery caterpillar</altname>, and <altname>webworm</altname>.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tent</ent><br/
<hw>Tent</hw>, <pos>v. i.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Tented</conjf>; <pos>p. pr. & vb. n.</pos> <conjf>Tenting</conjf>.]</vmorph> <def>To lodge as a tent; to tabernacle.</def>  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>We 're <qex>tenting</qex> to-night on the old camp ground.</q> <rj><qau>W. Kittredge.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tentacle</ent><br/
<hw>Ten"ta*cle</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[NL. <ets>tentaculum</ets>, from L. <ets>tentare</ets> to handle, feel: cf. F. <ets>tentacule</ets>. See <er>Tempt</er>.]</ety> <fld>(Zool.)</fld> <def>A more or less elongated process or organ, simple or branched, proceeding from the head or cephalic region of invertebrate animals, being either an organ of sense, prehension, or motion.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Tentacle sheath</b></col> <fld>(Zool.)</fld>, <cd>a sheathlike structure around the base of the tentacles of many mollusks.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tentacled</ent><br/
<hw>Ten"ta*cled</hw> <pr>(?)</pr>, <pos>a.</pos> <fld>(Zool.)</fld> <def>Having tentacles.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tentacular</ent><br/
<hw>Ten*tac"u*lar</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[Cf. F. <ets>tentaculaire</ets>.]</ety> <fld>(Zool.)</fld> <def>Of or pertaining to a tentacle or tentacles.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tentaculata</ent><br/
||<hw>Ten*tac`u*la"ta</hw> <pr>(?)</pr>, <pos>n. pl.</pos> <ety>[NL.]</ety> <fld>(Zool.)</fld> <def>A division of Ctenophora including those which have two long tentacles.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tentaculated</ent><br/
<ent>Tentaculate</ent><br/
<mhw>{ <hw>Ten*tac"u*late</hw> <pr>(?)</pr>, <hw>Ten*tac"u*la`ted</hw> <pr>(?)</pr>, }</mhw> <pos>a.</pos> <fld>(Zool.)</fld> <def>Having tentacles, or organs like tentacles; tentacled.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tentaculifera</ent><br/
||<hw>Ten`ta*cu*lif"e*ra</hw> <pr>(?)</pr>, <pos>n. pl.</pos> <ety>[NL.]</ety> <fld>(Zool.)</fld> <def>Same as <er>Suctoria</er>, 1.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tentaculiferous</ent><br/
<hw>Ten`ta*cu*lif"er*ous</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[<ets>Tentaculum</ets> + <ets>-ferous</ets>.]</ety> <fld>(Zool.)</fld> <def>Producing or bearing tentacles.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tentaculiform</ent><br/
<hw>Ten`ta*cu"li*form</hw> <pr>(?)</pr>, <pos>a.</pos> <fld>(Zool.)</fld> <def>Shaped like a tentacle.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tentaculite</ent><br/
<hw>Ten*tac"u*lite</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Paleon.)</fld> <def>Any one of numerous species of small, conical fossil shells found in Paleozoic rocks. They are supposed to be pteropods.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tentaculocyst</ent><br/
<hw>Ten*tac"u*lo*cyst</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[<ets>Tentaculum</ets> + <ets>cyst</ets>.]</ety> <fld>(Zool.)</fld> <def>One of the auditory organs of certain medusae; -- called also <altname>auditory tentacle</altname>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tentaculum</ent><br/
||<hw>Ten*tac"u*lum</hw> <pr>(?)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Tentacula</plw> <pr>(#)</pr>.</plu> <ety>[NL. See <er>Tentacle</er>.]</ety> <sn>1.</sn> <fld>(Zool.)</fld> <def>A tentacle.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Anat.)</fld> <def>One of the stiff hairs situated about the mouth, or on the face, of many animals, and supposed to be tactile organs; a tactile hair.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tentage</ent><br/
<hw>Tent"age</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[From <er>Tent</er> a pavilion.]</ety> <def>A collection of tents; an encampment.</def> <mark>[Obs.]</mark>  <rj><au>Drayton.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tentation</ent><br/
<hw>Ten*ta"tion</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>tentatio</ets>: cf. F. <ets>tentation</ets>. See <er>Temptation</er>.]</ety> <sn>1.</sn> <def>Trial; temptation.</def> <mark>[Obs.]</mark>  <rj><au>Sir T. Browne.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Mech.)</fld> <def>A mode of adjusting or operating by repeated trials or experiments.</def>  <rj><au>Knight.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tentative</ent><br/
<hw>Ten*ta"tive</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>tentare</ets> to try: cf. F. <ets>tentatif</ets>. See <er>Tempt</er>.]</ety> <def>Of or pertaining to a trial or trials; essaying; experimental.</def> <ldquo/A slow, <xex>tentative</xex> manner.<rdquo/ <au>Carlyle.</au> -- <wordforms><wf>Ten*ta"tive*ly</wf>, <pos>adv.</pos></wordforms><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tentative</ent><br/
<hw>Ten*ta"tive</hw>, <pos>n.</pos> <ety>[Cf. F. <ets>tentative</ets>.]</ety> <def>An essay; a trial; an experiment.</def>  <rj><au>Berkley.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tented</ent><br/
<hw>Tent"ed</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Covered with tents.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tenter</ent><br/
<hw>Ten"ter</hw> <pr>(?)</pr>, <pos>n.</pos> <sn>1.</sn> <def>One who takes care of, or tends, machines in a factory; a kind of assistant foreman.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Mach.)</fld> <def>A kind of governor.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tenter</ent><br/
<hw>Ten"ter</hw>, <pos>n.</pos> <ety>[OE. <ets>tenture</ets>, <ets>tentoure</ets>, OF. <ets>tenture</ets> a stretching, spreading, F. <ets>tenture</ets> hangings, tapestry, from L. <ets>tendere</ets>, <ets>tentum</ets>, to stretch. See <er>Tend</er> to move.]</ety> <def>A machine or frame for stretching cloth by means of hooks, called <xex>tenter-hooks</xex>, so that it may dry even and square.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Tenter ground</b></col>, <cd>a place where tenters are erected.</cd> -- <col><b>Tenter-hook</b></col>, <cd>a sharp, hooked nail used for fastening cloth on a tenter.</cd> -- <mcol><col><b>To be on the tenters</b></col>, <col><b>To be on the tenter-hooks</b></col> <it>or</it> <col><b>To be on tenterhooks</b></col></mcol>, <cd>to be on the stretch; to be in distress, uneasiness, or suspense.</cd> <au>Hudibras.</au></cs><br/
[<source>1913 Webster</source> <source>+PJC</source>]</p>

<p><ent>Tenter</ent><br/
<hw>Ten"ter</hw>, <pos>v. i.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Tentered</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Tentering</conjf>.]</vmorph> <def>To admit extension.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Woolen cloth will <qex>tenter</qex>, linen scarcely.</q> <rj><qau>Bacon.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tenter</ent><br/
<hw>Ten"ter</hw>, <pos>v. t.</pos> <def>To hang or stretch on, or as on, tenters.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tentful</ent><br/
<hw>Tent"ful</hw> <pr>(?)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Tentfuls</plw> <pr>(<?/)</pr>.</plu> <def>As much, or as many, as a tent will hold.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tenth</ent><br/
<hw>Tenth</hw> <pr>(t<ecr/nth)</pr>, <pos>a.</pos> <ety>[From <er>Ten</er>: cf. OE. <ets>tethe</ets>, AS. <ets>te<oacute/<edh/a</ets>. See <er>Ten</er>, and cf. <er>Tithe</er>.]</ety> <sn>1.</sn> <def>Next in order after the ninth; coming after nine others.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Constituting or being one of ten equal parts into which anything is divided.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tenth</ent><br/
<hw>Tenth</hw> <pr>(?)</pr>, <pos>n.</pos> <sn>1.</sn> <def>The next in order after the ninth; one coming after nine others.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>The quotient of a unit divided by ten; one of ten equal parts into which anything is divided.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>The tenth part of annual produce, income, increase, or the like; a tithe.</def>  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <fld>(Mus.)</fld> <def>The interval between any tone and the tone represented on the tenth degree of the staff above it, as between one of the scale and three of the octave above; the octave of the third.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <pluf>pl.</pluf> <fld>(Eng. Law)</fld> <sd>(a)</sd> <def>A temporary aid issuing out of personal property, and granted to the king by Parliament; formerly, the real tenth part of all the movables belonging to the subject.</def> <sd>(b)</sd> <fld>(Eccl. Law)</fld> <def>The tenth part of the annual profit of every living in the kingdom, formerly paid to the pope, but afterward transferred to the crown. It now forms a part of the fund called <xex>Queen Anne's Bounty</xex>.</def>
<-- (b) sic. = tithe? what kind of "living"?? prob. living, n. 5 = "the benefice of a clergyman" -->  <rj><au>Burrill.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tenthly</ent><br/
<hw>Tenth"ly</hw>, <pos>adv.</pos> <def>In a tenth manner.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tenthmetre</ent><br/
<ent>Tenthmeter</ent><br/
<mhw>{ <hw>Tenth"me`ter</hw>, <hw>Tenth"me`tre</hw>  }</mhw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Physics)</fld> <def>A unit for the measurement of many small lengths, such that 10<exp>10</exp> of these units make one meter; the ten millionth part of a millimeter.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tenthredinides</ent><br/
||<hw>Ten`thre*din"i*des</hw> <pr>(?)</pr>, <pos>n. pl.</pos> <ety>[NL., fr. Gr. <?/ a kind of wasp.]</ety> <fld>(Zool.)</fld> <def>A group of Hymneoptera comprising the sawflies.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tentif</ent><br/
<hw>Ten"tif</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Attentive.</def> <mark>[Obs.]</mark>  <rj><au>Chaucer.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tentifly</ent><br/
<hw>Ten"tif*ly</hw>, <pos>adv.</pos> <def>Attentively.</def> <mark>[Obs.]</mark>  <rj><au>Chaucer.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tentiginous</ent><br/
<hw>Ten*tig"i*nous</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>tentigo</ets>, <ets>-inis</ets>, a tension, lecherousness, fr. <ets>tendere</ets>, <ets>tentum</ets>, to stretch.]</ety> <sn>1.</sn> <def>Stiff; stretched; strained.</def> <mark>[Obs.]</mark>  <rj><au>Johnson.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Lustful, or pertaining to lust.</def> <mark>[Obs.]</mark>  <rj><au>B. Jonson.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tentmaker</ent><br/
<hw>Tent"mak`er</hw> <pr>(?)</pr>, <pos>n.</pos> <def>One whose occupation it is to make tents.</def>  <rj><au>Acts xviii. 3.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tentorium</ent><br/
||<hw>Ten*to"ri*um</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L., a tent.]</ety> <fld>(Anat.)</fld> <def>A fold of the dura mater which separates the cerebellum from the cerebrum and often incloses a process or plate of the skull called the <xex>bony tentorium</xex>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tentory</ent><br/
<hw>Tent"o*ry</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>tentorium</ets> a tent.]</ety> <def>The awning or covering of a tent.</def> <mark>[Obs.]</mark>  <rj><au>Evelyn.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tentwort</ent><br/
<hw>Tent"wort`</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Bot.)</fld> <def>A kind of small fern, the wall rue. See under <er>Wall</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tenuate</ent><br/
<hw>Ten"u*ate</hw> <pr>(?)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Tenuated</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Tenuating</conjf>.]</vmorph> <ety>[L. <ets>tenuatus</ets>, p. p. of <ets>tenuare</ets> to make thin, fr. <ets>tenuis</ets> thin. See <er>Tenuous</er>.]</ety> <def>To make thin; to attenuate.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tenuifolious</ent><br/
<hw>Ten`u*i*fo"li*ous</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>tenuis</ets> thin + <ets>folium</ets> a leaf.]</ety> <fld>(Bot.)</fld> <def>Having thin or narrow leaves.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tenuious</ent><br/
<hw>Te*nu"i*ous</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[See <er>Tenuous</er>.]</ety> <def>Rare or subtile; tenuous; -- opposed to <xex>dense</xex>.</def> <mark>[Obs.]</mark>  <rj><au>Glanvill.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tenuiroster</ent><br/
<hw>Ten`u*i*ros"ter</hw> <pr>(?)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Tenuirosters</plw> <pr>(#)</pr>.</plu> <ety>[NL., fr. L. <ets>tenuis</ets> thin + <ets>rostrum</ets> a beak.]</ety> <fld>(Zool.)</fld> <def>One of the Tenuirostres.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tenuirostral</ent><br/
<hw>Ten`u*i*ros"tral</hw> <pr>(?)</pr>, <pos>a.</pos> <fld>(Zool.)</fld> <def>Thin-billed; -- applied to birds with a slender bill, as the humming birds.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tenuirostres</ent><br/
||<hw>Ten`u*i*ros"tres</hw> <pr>(?)</pr>, <pos>n. pl.</pos> <ety>[NL.]</ety> <fld>(Zool.)</fld> <def>An artificial group of passerine birds having slender bills, as the humming birds.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tenuis</ent><br/
||<hw>Ten"u*is</hw> <pr>(?)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Tenues</plw> <pr>(#)</pr>.</plu> <ety>[NL., fr. L. <ets>tenuis</ets> fine, thin. See <er>Tenuous</er>.]</ety> <fld>(Gr. Gram.)</fld> <def>One of the three surd mutes <kappa/, <pi/, <tau/; -- so called in relation to their respective middle letters, or medials, <gamma/, <beta/, <delta/, and their aspirates, <chi/, <phi/, <theta/. The term is also applied to the corresponding letters and articulate elements in other languages.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tenuity</ent><br/
<hw>Te*nu"i*ty</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>tenuitas</ets>, from <ets>tenuis</ets> thin: cf. F. <ets>t<eacute/nuit<eacute/</ets>. See <er>Tenuous</er>.]</ety> <sn>1.</sn> <def>The quality or state of being tenuous; thinness, applied to a broad substance; slenderness, applied to anything that is long; <as>as, the <ex>tenuity</ex> of a leaf; the <ex>tenuity</ex> of a hair.</as></def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Rarily; rareness; thinness, as of a fluid; <as>as, the <ex>tenuity</ex> of the air; the <ex>tenuity</ex> of the blood</as>.</def>  <rj><au>Bacon.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>Poverty; indigence.</def> <mark>[Obs.]</mark>  <rj><au>Eikon Basilike.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>Refinement; delicacy.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tenuous</ent><br/
<hw>Ten"u*ous</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>tenuis</ets> thin. See <er>Thin</er>, and cf. <er>Tenuis</er>.]</ety> <sn>1.</sn> <def>Thin; slender; small; minute.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Rare; subtile; not dense; -- said of fluids.</def><br/
[<source>1913 Webster</source>]</p>

<p><-- 3. Fig. <def>Lacking substance, as a tenuous argument.</def> --><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tenure</ent><br/
<hw>Ten"ure</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F. <ets>tenure</ets>, OF. <ets>teneure</ets>, fr. F. <ets>tenir</ets> to hold. See <er>Tenable</er>.]</ety> <sn>1.</sn> <def>The act or right of holding, as property, especially real estate.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>That the <qex>tenure</qex> of estates might rest on equity, the Indian title to lands was in all cases to be quieted.</q> <rj><qau>Bancroft.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Eng. Law)</fld> <def>The manner of holding lands and tenements of a superior.</def><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ Tenure is inseparable from the idea of property in land, according to the theory of the English law; and this idea of tenure pervades, to a considerable extent, the law of real property in the United States, where the title to land is essentially allodial, and almost all lands are held in fee simple, not of a superior, but the whole right and title to the property being vested in the owner. Tenure, in general, then, is the particular manner of holding real estate, as by exclusive title or ownership, by fee simple, by fee tail, by courtesy, in dower, by copyhold, by lease, at will, etc.</note><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>The consideration, condition, or service which the occupier of land gives to his lord or superior for the use of his land.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>Manner of holding, in general; <as>as, in absolute governments, men hold their rights by a precarious <ex>tenure</ex></as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>All that seems thine own,<br/
Held by the <qex>tenure</qex> of his will alone.</q> <rj><qau>Cowper.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Tenure by fee alms</b></col>. <fld>(Law)</fld> <cd>See <er>Frankalmoigne</er>.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Teocalli</ent><br/
<hw>Te`o*cal"li</hw> <pr>(?)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Teocallis</plw> <pr>(#)</pr>.</plu> <ety>[Mexican.]</ety> <def>Literally, God's house; a temple, usually of pyramidal form, such as were built by the aborigines of Mexico, Yucatan, etc.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>And Aztec priests upon their <qex>teocallis</qex><br/
Beat the wild war-drums made of serpent's skin.</q> <rj><qau>Longfellow.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Teosinte</ent><br/
<hw>Te`o*sin"te</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Bot.)</fld> <def>A large grass (<spn>Euchlaena luxurians</spn>) closely related to maize. It is native of Mexico and Central America, but is now cultivated for fodder in the Southern United States and in many warm countries. Called also <altname>Guatemala grass</altname>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tepal</ent><br/
<hw>Tep"al</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F. <ets>t<eacute/pale</ets>, fr. <ets>p<eacute/tale</ets>, by transposition.]</ety> <fld>(Bot.)</fld> <def>A division of a perianth.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tepee</ent><br/
<hw>Tep*ee"</hw> <pr>(?)</pr>, <pos>n.</pos> <def>An Indian wigwam or tent.</def><-- also teepee --><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tepefaction</ent><br/
<hw>Tep`e*fac"tion</hw> <pr>(?)</pr>, <pos>n.</pos> <def>Act of tepefying.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tepefy</ent><br/
<hw>Tep"e*fy</hw> <pr>(?)</pr>, <pos>v. t. & i.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Tepefied</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Tepefying</conjf> <pr>(?)</pr>.]</vmorph> <ety>[L. <ets>tepere</ets> to be tepid + <ets>-fy</ets>; cf. L. <ets>tepefacere</ets>. See <er>Tepid</er>.]</ety> <def>To make or become tepid, or moderately warm.</def>  <rj><au>Goldsmith.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tephramancy</ent><br/
<hw>Teph"ra*man`cy</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Gr. <?/ ashes + <ets>-mancy</ets>.]</ety> <def>Divination by the ashes of the altar on which a victim had been consumed in sacrifice.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tephrite</ent><br/
<hw>Teph"rite</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Gr. <?/ ashes.]</ety> <fld>(Geol.)</fld> <def>An igneous rock consisting essentially of plagioclase and either leucite or nephelite, or both.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tephroite</ent><br/
<hw>Teph"ro*ite</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[See <er>Tephrosia</er>.]</ety> <fld>(Min.)</fld> <def>A silicate of manganese of an ash-gray color.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tephrosia</ent><br/
||<hw>Te*phro"si*a</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[NL., fr. Gr. <?/ ash-colored, from <?/ ashes.]</ety> <fld>(Bot.)</fld> <def>A genus of leguminous shrubby plants and herbs, mostly found in tropical countries, a few herbaceous species being North American. The foliage is often ashy-pubescent, whence the name.</def><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ The <xex>Tephrosia toxicaria</xex> is used in the West Indies and in Polynesia for stupefying fish. <xex>T</xex>. <xex>purpurea</xex> is used medicinally in the East Indies. <xex>T</xex>. <xex>Virginia</xex> is the goat's rue of the United States.</note><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tepid</ent><br/
<hw>Tep"id</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>tepidus</ets>, fr. <ets>tepere</ets> to be warm; akin to Skr. <ets>tap</ets> to be warm, <ets>tapas</ets> heat.]</ety> <def>Moderately warm; lukewarm; <as>as, a <ex>tepid</ex> bath; <ex>tepid</ex> rays; <ex>tepid</ex> vapors.</as></def> -- <wordforms><wf>Tep"id*ness</wf>, <pos>n.</pos></wordforms><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tepidity</ent><br/
<hw>Te*pid"i*ty</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Cf. F. <ets>t<eacute/pidit<eacute/</ets>.]</ety> <def>The quality or state of being tepid; moderate warmth; lukewarmness; tepidness.</def>  <rj><au>Jer. Taylor.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tepor</ent><br/
<hw>Te"por</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L., fr. <ets>tepere</ets> to be tepid.]</ety> <def>Gentle heat; moderate warmth; tepidness.</def>  <rj><au>Arbuthnot.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tequila</ent><br/
<hw>Te*qui"la</hw> <pr>(?)</pr>, <pos>n.</pos> <def>An intoxicating liquor made from the maguey in the district of Tequila, Mexico.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Ter-</ent><br/
<hw>Ter-</hw> <pr>(?)</pr>. <def>A combining form from L. <xex>ter</xex> signifying <xex>three times</xex>, <xex>thrice</xex>. See <er>Tri-</er>, 2.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Teraconic</ent><br/
<hw>Ter`a*con"ic</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[<ets>Ter</ets>ebic + citr<ets>aconic</ets>.]</ety> <fld>(Chem.)</fld> <def>Pertaining to, or designating, an acid obtained by the distillation of terebic acid, and homologous with citraconic acid.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Teracrylic</ent><br/
<hw>Ter`a*cryl"ic</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[<ets>Ter</ets>pene + <ets>acrylic</ets>.]</ety> <fld>(Chem.)</fld> <def>Of, pertaining to, or designating, an acid of the acrylic series, obtained by the distillation of terpenylic acid, as an only substance having a peculiar cheesy odor.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Teraph</ent><br/
<hw>Ter"aph</hw> <pr>(?)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Teraphs</plw> <pr>(<?/)</pr>.</plu> <def>See <er>Teraphim</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Teraphim</ent><br/
<hw>Ter"a*phim</hw> <pr>(?)</pr>, <pos>n. pl.</pos> <ety>[Heb. <ets>ter<amac/ph<imac/m</ets>.]</ety> <def>Images connected with the magical rites used by those Israelites who added corrupt practices to the patriarchal religion. Teraphim were consulted by the Israelites for oracular answers.</def>  <rj><au>Dr. W. Smith (Bib. Dict.).</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Terapin</ent><br/
<hw>Ter"a*pin</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Zool.)</fld> <def>See <er>Terrapin</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Teratical</ent><br/
<hw>Te*rat"ic*al</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[Gr. <?/ a wonder.]</ety> <def>Wonderful; ominous; prodigious.</def> <mark>[Obs.]</mark>  <rj><au>Wollaston.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Teratogeny</ent><br/
<hw>Ter`a*tog"e*ny</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Gr. <?/, <?/, a wonder, monster + the root of <?/ to be born.]</ety> <fld>(Med.)</fld> <def>The formation of monsters.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Teratoid</ent><br/
<hw>Ter"a*toid</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[Gr. <?/, <?/, monster + <ets>-oid</ets>.]</ety> <def>Resembling a monster; abnormal; of a pathological growth, exceedingly complex or highly organized.</def>  <rj><au>S. D. Gross.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Teratological</ent><br/
<hw>Ter`a*to*log"ic*al</hw> <pr>(?)</pr>, <pos>a.</pos> <fld>(Biol.)</fld> <def>Of or pertaining to teratology; <as>as, <ex>teratological</ex> changes</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Teratology</ent><br/
<hw>Ter`a*tol"o*gy</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Gr. <?/, <?/, a wonder, monster + <ets>-logy</ets>: cf. Gr. <?/ a telling of wonders, and F. <ets>t<eacute/ratologie</ets>.]</ety> <sn>1.</sn> <def>That branch of biological science which treats of monstrosities, malformations, or deviations from the normal type of structure, either in plants or animals.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Affectation of sublimity; bombast.</def> <mark>[Obs.]</mark>  <rj><au>Bailey.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Teratoma</ent><br/
<hw>Ter`a*to"ma</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[NL., fr. Gr. <?/, <?/, monster + <ets>-oma</ets>.]</ety> <fld>(Med.)</fld> <def>A tumor, sometimes found in newborn children, which is made up of a heterigenous mixture of tissues, as of bone, cartilage and muscle.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Terbic</ent><br/
<hw>Ter"bic</hw> <pr>(?)</pr>, <pos>a.</pos> <fld>(Chem.)</fld> <def>Of, pertaining to, or containing, terbium; also, designating certain of its compounds.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Terbium</ent><br/
<hw>Ter"bi*um</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[NL., fr. Yt<ets>terby</ets>, in Sweden. See <er>Erbium</er>.]</ety> <fld>(Chem.)</fld> <def>A rare metallic element, found in certain minerals, as gadolinite and samarskite, with other rare earths such as ytterbium. Symbol Tb. Atomic number 65. Atomic weight 158.925.</def><br/
[<source>1913 Webster</source> <source>+PJC</source>]</p>

<p><ent>Terce</ent><br/
<hw>Terce</hw> <pr>(?)</pr>, <pos>n.</pos> <def>See <er>Tierce</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tercel</ent><br/
<hw>Ter"cel</hw> <pr>(?)</pr>, <pos>n.</pos> <def>See <er>Tiercel</er>. Called also <altname>tarsel</altname>, <altname>tassel</altname>.</def>  <rj><au>Chaucer.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tercelet</ent><br/
<hw>Terce"let</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Zool.)</fld> <def>A male hawk or eagle; a tiercelet.</def>  <rj><au>Chaucer.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tercellene</ent><br/
<hw>Ter"cel*lene</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Zool.)</fld> <def>A small male hawk.</def> <mark>[R.]</mark>  <rj><au>Sir T. Browne.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tercentenary</ent><br/
<hw>Ter*cen"te*na*ry</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>ter</ets> thirce + E. <ets>centenary</ets>.]</ety> <def>Including, or relating to, an interval of three hundred years.</def> -- <def2><pos>n.</pos> <def>The three hundredth anniversary of any event; also, a celebration of such an anniversary.</def></def2><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tercet</ent><br/
<hw>Ter"cet</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F., fr. It. <ets>terzetto</ets>, dim. of <ets>terzo</ets>, third, L. <ets>tertius</ets>. See <er>Tierce</er>, and cf. <er>Terzetto</er>.]</ety> <sn>1.</sn> <fld>(Mus.)</fld> <def>A triplet.</def>  <rj><au>Hiles.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Poetry)</fld> <def>A triplet; a group of three lines.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tercine</ent><br/
<hw>Ter"cine</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F., from L. <ets>tertius</ets> the third.]</ety> <fld>(Bot.)</fld> <def>A cellular layer derived from the nucleus of an ovule and surrounding the embryo sac.  Cf. <er>Quintine</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Terebate</ent><br/
<hw>Ter"e*bate</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A salt of terebic acid.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Terebene</ent><br/
<hw>Ter"e*bene</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Chem.)</fld> <def>A polymeric modification of terpene, obtained as a white crystalline camphorlike substance; -- called also <altname>camphene</altname>. By extension, any one of a group of related substances.</def><br/
[<source>1913 Webster</source>]</p>

<p><-- p. 1487 --><br/
[<source>1913 Webster</source>]</p>

<p><ent>Terebenthene</ent><br/
<hw>Ter`e*ben"thene</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Chem.)</fld> <def>Oil of turpentine. See <er>Turpentine</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Terebic</ent><br/
<hw>Te*reb"ic</hw> <pr>(?)</pr>, <pos>a.</pos> <fld>(Chem.)</fld> <def>Pertaining to, or obtained from, terbenthene (oil of turpentine); specifically, designating an acid, <chform>C7H10O4</chform>, obtained by the oxidation of terbenthene with nitric acid, as a white crystalline substance.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Terebilenic</ent><br/
<hw>Ter`e*bi*len"ic</hw> <pr>(?)</pr>, <pos>a.</pos> <fld>(Chem.)</fld> <def>Of, pertaining to, or designating, a complex acid, <chform>C7H8O4</chform>, obtained as a white crystalline substance by a modified oxidation of terebic acid.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Terebinth</ent><br/
<hw>Ter"e*binth</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>terbinthus</ets>, Gr. <?/: cf. F. <ets>t<eacute/r<eacute/binthe</ets>.  Cf. <er>Turpentine</er>.]</ety> <fld>(Bot.)</fld> <def>The turpentine tree.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Terebinthic</ent><br/
<hw>Ter`e*bin"thic</hw> <pr>(?)</pr>, <pos>a.</pos> <fld>(Chem.)</fld> <def>Of or pertaining to turpentine; resembling turpentine; terbinthine; <as>as, <ex>terbinthic</ex> qualities</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Terebinthinate</ent><br/
<hw>Ter`e*bin"thi*nate</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Impregnating with the qualities of turpentine; terbinthine.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Terebinthine</ent><br/
<hw>Ter`e*bin"thine</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>terbinthinus</ets>, Gr. <?/.]</ety> <def>Of or pertaining to turpentine; consisting of turpentine, or partaking of its qualities.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Terebra</ent><br/
||<hw>Ter"e*bra</hw> <pr>(?)</pr>, <pos>n.</pos>; <plu><it>pl.</it> E. <plw>Terebras</plw> <pr>(#)</pr>, L. <plw>Terebrae</plw> <pr>(#)</pr>.</plu> <ety>[L., a borer.]</ety> <sn>1.</sn> <fld>(Zool.)</fld> <def>A genus of marine gastropods having a long, tapering spire. They belong to the Toxoglossa. Called also <altname>auger shell</altname>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Zool.)</fld> <def>The boring ovipositor of a hymenopterous insect.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Terebrant</ent><br/
<hw>Ter"e*brant</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>terebrans</ets>, <ets>-antis</ets>, p. pr.]</ety> <fld>(Zool.)</fld> <def>Boring, or adapted for boring; -- said of certain Hymenoptera, as the sawflies.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Terebrantia</ent><br/
||<hw>Ter`e*bran"ti*a</hw> <pr>(?)</pr>, <pos>n. pl.</pos> <ety>[NL.]</ety> <fld>(Zool.)</fld> <def>A division of Hymenoptera including those which have an ovipositor adapted for perforating plants. It includes the sawflies.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Terebrate</ent><br/
<hw>Ter"e*brate</hw> <pr>(?)</pr>, <pos>v. t.</pos> <ety>[L. <ets>terebratus</ets>, p. p. of <ets>terebrare</ets>, from <ets>terebra</ets> a borer, <ets>terere</ets> to rub.]</ety> <def>To perforate; to bore; to pierce.</def> <mark>[R.]</mark>  <rj><au>Sir T. Browne.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Terebrating</ent><br/
<hw>Ter"e*bra`ting</hw> <pr>(?)</pr>, <pos>a.</pos> <sn>1.</sn> <fld>(Zool.)</fld> <def>Boring; perforating; -- applied to molluskas which form holes in rocks, wood, etc.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Med.)</fld> <def>Boring; piercing; -- applied to certain kinds of pain, especially to those of locomotor ataxia.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Terebration</ent><br/
<hw>Ter`e*bra"tion</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>terebratio</ets>.]</ety> <def>The act of terebrating, or boring.</def> <mark>[R.]</mark>  <rj><au>Bacon.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Terebratula</ent><br/
||<hw>Ter`e*brat"u*la</hw> <pr>(?)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Terebratulae</plw> <pr>(#)</pr>.</plu> <ety>[Nl., dim. fr. <ets>terebratus</ets>, p. p., perforated.]</ety> <fld>(Zool.)</fld> <def>A genus of brachiopods which includes many living and some fossil species. The larger valve has a perforated beak, through which projects a short peduncle for attachment. Called also <altname>lamp shell</altname>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Terebratulid</ent><br/
<hw>Ter`e*brat"u*lid</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Zool.)</fld> <def>Any species of Terebratula or allied genera. Used also adjectively.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Terebratuliform</ent><br/
<hw>Ter`e*bra*tu"li*form</hw> <pr>(?)</pr>, <pos>a.</pos> <fld>(Zool.)</fld> <def>Having the general form of a terebratula shell.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Teredine</ent><br/
<hw>Ter"e*dine</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F. <ets>t<eacute/r<eacute/dine</ets>.]</ety> <fld>(Zool.)</fld> <def>A borer; the teredo.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Teredo</ent><br/
<hw>Te*re"do</hw> <pr>(?)</pr>, <pos>n.</pos>; <plu><it>pl.</it> E. <plw>Teredos</plw> <pr>(#)</pr>, L. <plw>Teredines</plw> <pr>(#)</pr>.</plu> <ety>[L., a worm that gnaws wood, clothes, etc.; akin to Gr. <?/, L. <ets>terere</ets> to rub.]</ety> <fld>(Zool.)</fld> <def>A genus of long, slender, wormlike bivalve mollusks which bore into submerged wood, such as the piles of wharves, bottoms of ships, etc.; -- called also <altname>shipworm</altname>. See <er>Shipworm</er>. See <xex>Illust.</xex> in Appendix.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Terek</ent><br/
<hw>Ter"ek</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Because found on the <etsep>Terek</etsep> River in the Caucasus.]</ety> <def>A sandpiper (<spn>Terekia cinerea</spn>) of the Old World, breeding in the far north of eastern Europe and Asia and migrating to South Africa and Australia. It frequents rivers.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Terephthalate</ent><br/
<hw>Ter*eph"tha*late</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Chem.)</fld> <def>A salt of terephthalic acid.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Terephthalic</ent><br/
<hw>Ter`eph*thal"ic</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[<ets>Tere</ets>bene + <ets>phthalic</ets>.]</ety> <fld>(Chem.)</fld> <def>Of, pertaining to, or designating, a dibasic acid of the aromatic series, metameric with phthalic acid, and obtained, as a tasteless white crystalline powder, by the oxidation of oil of turpentine; -- called also <altname>paraphthalic acid</altname>.  Cf. <er>Phthalic</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Teret</ent><br/
<hw>Ter"et</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Round; terete.</def> <mark>[Obs.]</mark>  <rj><au>Fotherby.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Terete</ent><br/
<hw>Te*rete"</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>teres</ets>, <ets>-etis</ets>, rounded off, properly, rubbed off, fr. <ets>terere</ets> to rub.]</ety> <def>Cylindrical and slightly tapering; columnar, as some stems of plants.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Teretial</ent><br/
<hw>Te*re"tial</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[See <er>Terete</er>.]</ety> <fld>(Anat.)</fld> <def>Rounded; <as>as, the <ex>teretial</ex> tracts in the floor of the fourth ventricle of the brain of some fishes</as>.</def>  <rj><au>Owen.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Teretous</ent><br/
<hw>Ter"e*tous</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Terete.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tergal</ent><br/
<hw>Ter"gal</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>tergum</ets> the back.]</ety> <fld>(Anat. & Zool.)</fld> <def>Of or pertaining to back, or tergum. See <er>Dorsal</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tergant</ent><br/
<hw>Ter"gant</hw> <pr>(?)</pr>, <pos>a.</pos> <fld>(Her.)</fld> <def>Showing the back; <as>as, the eagle <ex>tergant</ex></as>.</def> <altsp>[Written also <asp>tergiant</asp>.]</altsp><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tergeminate</ent><br/
<ent>Tergeminal</ent><br/
<mhw>{ <hw>Ter*gem"i*nal</hw> <pr>(?)</pr>, <hw>Ter*gem"i*nate</hw> <pr>(?)</pr>, }</mhw> <pos>a.</pos> <ety>[See <er>Tergeminous</er>.]</ety> <fld>(Bot.)</fld> <def>Thrice twin; having three pairs of leaflets.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tergeminous</ent><br/
<hw>Ter*gem"i*nous</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>tergeminus</ets>; <ets>ter</ets> thrice + <ets>geminus</ets> doubled at birth, twin-born.  Cf. <er>Trigeminous</er>.]</ety> <def>Threefold; thrice-paired.</def>  <rj><au>Blount.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tergiferous</ent><br/
<hw>Ter*gif"er*ous</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>tergum</ets> the back + <ets>-ferous</ets>.]</ety> <def>Carrying or bearing upon the back.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Tergiferous plants</b></col> <fld>(Bot.)</fld>, <cd>plants which bear their seeds on the back of their leaves, as ferns.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tergite</ent><br/
<hw>Ter"gite</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Zool.)</fld> <def>The dorsal portion of an arthromere or somite of an articulate animal. See <xex>Illust.</xex> under <er>Coleoptera</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tergiversate</ent><br/
<hw>Ter"gi*ver*sate</hw> <pr>(?)</pr>, <pos>v. i.</pos> <ety>[L. <ets>tergiversatus</ets>, p. p. of <ets>tergiversari</ets> to turn one's back, to shift; <ets>tergum</ets> back + <ets>versare</ets>, freq. of <ets>vertere</ets> to turn. See <er>Verse</er>.]</ety> <def>To shift; to practice evasion; to use subterfuges; to shuffle.</def> <mark>[R.]</mark>  <rj><au>Bailey.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tergiversation</ent><br/
<hw>Ter`gi*ver*sa"tion</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>tergiversario</ets>: cf. F. <ets>tergiversation</ets>.]</ety> <sn>1.</sn> <def>The act of tergiversating; a shifting; shift; subterfuge; evasion.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Writing is to be preferred before verbal conferences, as being freer from passions and <qex>tergiversations</qex>.</q> <rj><qau>Abp. Bramhall.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Fickleness of conduct; inconstancy; change.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The colonel, after all his <qex>tergiversations</qex>, lost his life in the king's service.</q> <rj><qau>Clarendon.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tergiversator</ent><br/
<hw>Ter"gi*ver*sa`tor</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L.]</ety> <def>One who tergiversates; one who suffles, or practices evasion.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tergum</ent><br/
||<hw>Ter"gum</hw> <pr>(?)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Terga</plw> <pr>(#)</pr>.</plu> <ety>[L., the back.]</ety> <fld>(Zool.)</fld> <sd>(a)</sd> <def>The back of an animal.</def> <sd>(b)</sd> <def>The dorsal piece of a somite of an articulate animal.</def> <sd>(c)</sd> <def>One of the dorsal plates of the operculum of a cirriped.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Terin</ent><br/
<hw>Te"rin</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F. <ets>tarin</ets>, Prov. F. <ets>tairin</ets>, <ets>t<eacute/rin</ets>, probably from the Picard <ets>t<egrave/re</ets> tender.]</ety> <fld>(Zool.)</fld> <def>A small yellow singing bird, with an ash-colored head; the European siskin. Called also <altname>tarin</altname>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Term</ent><br/
<hw>Term</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F. <ets>terme</ets>, L. <ets>termen</ets>, <ets>-inis</ets>, <ets>terminus</ets>, a boundary limit, end; akin to Gr. <?/, <?/. See <er>Thrum</er> a tuft, and cf. <er>Terminus</er>, <er>Determine</er>, <er>Exterminate</er>.]</ety> <sn>1.</sn> <def>That which limits the extent of anything; limit; extremity; bound; boundary.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Corruption is a reciprocal to generation, and they two are as nature's two <qex>terms</qex>, or boundaries.</q> <rj><qau>Bacon.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>The time for which anything lasts; any limited time; <as>as, a <ex>term</ex> of five years; the <ex>term</ex> of life</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>In universities, schools, etc., a definite continuous period during which instruction is regularly given to students; <as>as, the school year is divided into three <ex>terms</ex></as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <fld>(Geom.)</fld> <def>A point, line, or superficies, that limits; <as>as, a line is the <ex>term</ex> of a superficies, and a superficies is the <ex>term</ex> of a solid</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <fld>(Law)</fld> <def>A fixed period of time; a prescribed duration</def>; as: <sd>(a)</sd> <def>The limitation of an estate; or rather, the whole time for which an estate is granted, as for the term of a life or lives, or for a term of years.</def> <sd>(b)</sd> <def>A space of time granted to a debtor for discharging his obligation.</def> <sd>(c)</sd> <def>The time in which a court is held or is open for the trial of causes.</def>  <rj><au>Bouvier.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ In England, there were formerly four terms in the year, during which the superior courts were open: Hilary term, beginning on the 11th and ending on the 31st of January; Easter term, beginning on the 15th of April, and ending on the 8th of May; Trinity term, beginning on the 22d day of May, and ending on the 12th of June; Michaelmas term, beginning on the 2d and ending on the 25th day of November. The rest of the year was called <xex>vacation</xex>. But this division has been practically abolished by the Judicature Acts of 1873, 1875, which provide for the more convenient arrangement of the terms and vacations.<br/
    In the United States, the terms to be observed by the tribunals of justice are prescribed by the statutes of Congress and of the several States.</note><br/
[<source>1913 Webster</source>]</p>

<p><sn>6.</sn> <fld>(Logic)</fld> <def>The subject or the predicate of a proposition; one of the three component parts of a syllogism, each one of which is used twice.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The subject and predicate of a proposition are, after Aristotle, together called its <qex>terms</qex> or extremes.</q> <rj><qau>Sir W. Hamilton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ The predicate of the conclusion is called the <xex>major</xex> term, because it is the most general, and the subject of the conclusion is called the <xex>minor</xex> term, because it is less general. These are called the <xex>extermes</xex>; and the third term, introduced as a common measure between them, is called the <xex>mean</xex> or <xex>middle</xex> term. Thus in the following syllogism, --<br/
[<source>1913 Webster</source>]</p>

<p>Every vegetable is combustible;
Every tree is a vegetable;
Therefore every tree is combustible, -<br/
[<source>1913 Webster</source>]</p>

<p><xex>combustible</xex>, the predicate of the conclusion, is the major term; <xex>tree</xex> is the minor term; <xex>vegetable</xex> is the middle term.</note><br/
[<source>1913 Webster</source>]</p>

<p><sn>7.</sn> <def>A word or expression; specifically, one that has a precisely limited meaning in certain relations and uses, or is peculiar to a science, art, profession, or the like; <as>as, a technical <ex>term</ex></as>.</def> <ldquo/<xex>Terms</xex> quaint of law.<rdquo/  <rj><au>Chaucer.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>In painting, the greatest beauties can not always be expressed for want of <qex>terms</qex>.</q> <rj><qau>Dryden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>8.</sn> <fld>(Arch.)</fld> <def>A quadrangular pillar, adorned on the top with the figure of a head, as of a man, woman, or satyr; -- called also <altname>terminal figure</altname>. See <er>Terminus</er>, <pos>n.</pos>, 2 and 3.</def><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ The pillar part frequently tapers downward, or is narrowest at the base. <xex>Terms</xex> rudely carved were formerly used for landmarks or boundaries. <au>Gwilt.</au></note><br/
[<source>1913 Webster</source>]</p>

<p><sn>9.</sn> <fld>(Alg.)</fld> <def>A member of a compound quantity; <as>as, <it>a</it> or <it>b</it> in <mathex>a + b</mathex>; <it>ab</it> or <it>cd</it> in <mathex>ab - cd</mathex></as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>10.</sn> <pluf>pl.</pluf> <fld>(Med.)</fld> <def>The menses.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>11.</sn> <pluf>pl.</pluf> <fld>(Law)</fld> <def>Propositions or promises, as in contracts, which, when assented to or accepted by another, settle the contract and bind the parties; conditions.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>12.</sn> <fld>(Law)</fld> <def>In Scotland, the time fixed for the payment of rents.</def><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ <xex>Terms legal and conventional</xex> in Scotland correspond to <xex>quarter days</xex> in England and Ireland. There are two <xex>legal terms</xex> -- Whitsunday, May 15, and Martinmas, Nov. 11; and two <xex>conventional terms</xex> -- Candlemas, Feb. 2, and Lammas day, Aug. 1. <au>Mozley & W.</au></note><br/
[<source>1913 Webster</source>]</p>

<p><sn>13.</sn> <fld>(Naut.)</fld> <def>A piece of carved work placed under each end of the taffrail.</def>  <rj><au>J. Knowels.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>In term</b></col>, <cd>in set terms; in formal phrase.</cd> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>I can not speak <qex>in term</qex>.</q> <rj><qau>Chaucer.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p>-- <col><b>Term fee</b></col> <fld>(Law)</fld> <sd>(a)</sd>, <cd>a fee by the term, chargeable to a suitor, or by law fixed and taxable in the costs of a cause for each or any term it is in court.</cd> -- <col><b>Terms of a proportion</b></col> <fld>(Math.)</fld>, <cd>the four members of which it is composed.</cd> -- <col><b>To bring to terms</b></col>, <cd>to compel (one) to agree, assent, or submit; to force (one) to come to terms.</cd> -- <col><b>To make terms</b></col>, <cd>to come to terms; to make an agreement: to agree.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><syn><b>Syn.</b> -- Limit; bound; boundary; condition; stipulation; word; expression.</syn> <usage> -- <er>Term</er>, <er>Word</er>. These are more frequently interchanged than almost any other vocables that occur of the language. There is, however, a difference between them which is worthy of being kept in mind. <xex>Word</xex> is generic; it denotes an utterance which represents or expresses our thoughts and feelings. <xex>Term</xex> originally denoted one of the two essential members of a proposition in logic, and hence signifies a word of specific meaning, and applicable to a definite class of objects. Thus, we may speak of a scientific or a technical <xex>term</xex>, and of stating things in distinct <xex>terms</xex>. Thus we say, <ldquo/the <xex>term</xex> minister literally denotes servant;<rdquo/ <ldquo/an exact definition of <xex>terms</xex> is essential to clearness of thought;<rdquo/ <ldquo/no <xex>term</xex> of reproach can sufficiently express my indignation;<rdquo/ <ldquo/every art has its peculiar and distinctive <xex>terms</xex>,<rdquo/ etc. So also we say, <ldquo/purity of style depends on the choice of <xex>words</xex>, and precision of style on a clear understanding of the <xex>terms</xex> used.<rdquo/ <xex>Term</xex> is chiefly applied to verbs, nouns, and adjectives, these being capable of standing as terms in a logical proposition; while prepositions and conjunctions, which can never be so employed, are rarely spoken of as <xex>terms</xex>, but simply as <xex>words</xex>.</usage><br/
[<source>1913 Webster</source>]</p>

<p><ent>Term</ent><br/
<hw>Term</hw> <pr>(?)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Termed</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Terming</conjf>.]</vmorph> <ety>[See <er>Term</er>, <pos>n.</pos>, and cf. <er>Terminate</er>.]</ety> <def>To apply a term to; to name; to call; to denominate.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Men <qex>term</qex> what is beyond the limits of the universe <ldquo/imaginary space.<rdquo/</q> <rj><qau>Locke.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Terma</ent><br/
||<hw>Ter"ma</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[NL. See <er>Term</er>, <pos>n.</pos>]</ety> <fld>(Anat.)</fld> <def>The terminal lamina, or thin ventral part, of the anterior wall of the third ventricle of the brain.</def>  <rj><au>B. G. Wilder.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Termagancy</ent><br/
<hw>Ter"ma*gan*cy</hw> <pr>(?)</pr>, <pos>n.</pos> <def>The quality or state of being termagant; turbulence; tumultuousness; <as>as, a violent <ex>termagancy</ex> of temper</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Termagant</ent><br/
<hw>Ter"ma*gant</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[OE. <ets>Trivigant</ets>, <ets>Termagant</ets>, Termagant (in sense 1), OF. <ets>Tervagan</ets>; cf. It. <ets>Trivigante</ets>.]</ety> <sn>1.</sn> <def>An imaginary being supposed by the Christians to be a Moslem deity or false god.  He is represented in the ancient moralities, farces, and puppet shows as extremely vociferous and tumultous.</def> <mark>[Obs.]</mark> <au>Chaucer.</au> <ldquo/And oftentimes by <xex>Termagant</xex> and Mahound [Mahomet] swore.<rdquo/ <au>Spenser.</au><br/
[<source>1913 Webster</source>]</p>

<p><q>The lesser part on Christ believed well,<br/
On <qex>Termagant</qex> the more, and on Mahound.</q> <rj><qau>Fairfax.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A boisterous, brawling, turbulent person; -- formerly applied to both sexes, now only to women.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>This terrible <qex>termagant</qex>, this Nero, this Pharaoh.</q> <rj><qau>Bale (1543).</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>The slave of an imperious and reckless <qex>termagant</qex>.</q> <rj><qau>Macaulay.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Termagant</ent><br/
<hw>Ter"ma*gant</hw>, <pos>a.</pos> <def>Tumultuous; turbulent; boisterous; furious; quarrelsome; scolding.</def> -- <wordforms><wf>Ter"ma*gant*ly</wf>, <pos>adv.</pos></wordforms><br/
[<source>1913 Webster</source>]</p>

<p><q>A <qex>termagant</qex>, imperious, prodigal, profligate wench.</q> <rj><qau>Arbuthnot.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Termatarium</ent><br/
||<hw>Ter`ma*ta"ri*um</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[NL. See <er>Termes</er>.]</ety> <fld>(Zool.)</fld> <def>Any nest or dwelling of termes, or white ants.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Termatary</ent><br/
<hw>Ter"ma*ta*ry</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Zool.)</fld> <def>Same as <er>Termatarium</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Term day</ent><br/
<hw>Term day</hw>. <def>A day which is a term (as for payment of rent), or is a day in a term, as of the sitting of a court; esp., one of a series of special days, designated by scientists of different nations or stations, for making synoptic magnetic, meteorological, or other physical observations.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Termer</ent><br/
<hw>Term"er</hw> <pr>(?)</pr>, <pos>n.</pos> <sn>1.</sn> <def>One who resorted to London during the law term only, in order to practice tricks, to carry on intrigues, or the like.</def> <mark>[Obs.]</mark> <altsp>[Written also <asp>termor</asp>.]</altsp>  <rj><au>B. Jonson.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Law)</fld> <def>One who has an estate for a term of years or for life.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Termes</ent><br/
||<hw>Ter"mes</hw> <pr>(t<etil/r"m<emac/z)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Termites</plw> <pr>(-m<icr/*t<emac/z)</pr>.</plu> <ety>[L. <ets>termes</ets>, <ets>tarmes</ets>, <ets>-itis</ets>, a woodworm.  Cf. <er>Termite</er>.]</ety> <fld>(Zool.)</fld> <def>A genus of Pseudoneuroptera including the white ants, or termites. See <er>Termite</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Terminable</ent><br/
<hw>Ter"mi*na*ble</hw> <pr>(-m<icr/n*<adot/*b'l)</pr>, <pos>a.</pos> <ety>[See <er>Terminate</er>.]</ety> <def>Capable of being terminated or bounded; limitable.</def> -- <wordforms><wf>Ter"mi*na*ble*ness</wf>, <pos>n.</pos></wordforms><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Terminable annuity</b></col>, <cd>an annuity for a stated, definite number of years; -- distinguished from <contr>life annuity</contr>, and <contr>perpetual annuity</contr>.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Terminal</ent><br/
<hw>Ter"mi*nal</hw> <pr>(-n<ait/l)</pr>, <pos>a.</pos> <ety>[L. <ets>terminals</ets>: cf. F. <ets>terminal</ets>. See <er>Term</er>, <pos>n.</pos>]</ety> <sn>1.</sn> <def>Of or pertaining to the end or extremity; forming the extremity; <as>as, a <ex>terminal</ex> edge</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Bot.)</fld> <def>Growing at the end of a branch or stem; terminating; <as>as, a <ex>terminal</ex> bud, flower, or spike</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <fld>(Railroads)</fld> <def>Pertaining to a railroad terminal; connected with the receipt or delivery of freight; <as>as, <ex>terminal</ex> charges</as>.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><cs><col><b>Terminal moraine</b></col>. <cd>See the Note under <er>Moraine</er>.</cd> -- <col><b>Terminal statue</b></col>. <cd>See <er>Terminus</er>, <pos>n.</pos>, 2 and 3.</cd> -- <col><b>Terminal velocity</b></col>. <sd>(a)</sd> <cd>The velocity acquired at the end of a body's motion.</cd> <sd>(b)</sd> <cd>The limit toward which the velocity of a body approaches, as of a body falling through the air.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Terminal</ent><br/
<hw>Ter"mi*nal</hw>, <pos>n.</pos> <sn>1.</sn> <def>That which terminates or ends; termination; extremity.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Eccl.)</fld> <def>Either of the ends of the conducting circuit of an electrical apparatus, as an inductorium, dynamo, or electric motor, usually provided with binding screws for the attachment of wires by which a current may be conveyed into or from the machine; a pole.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <fld>(Railroads)</fld> <sd>(a)</sd> <def>The end of a line of railroad, with the switches, stations, sheds, and other appliances pertaining thereto.</def> <sd>(b)</sd> <def>Any station for the delivery or receipt of freight lying too far from the main line to be served by mere <xex>sidings</xex>.</def> <sd>(c)</sd> <def>A rate charged on all freight, independent of the distance, and supposed to cover the expenses of station service, as distinct from <contr>mileage rate</contr>, generally proportionate to the distance and intended to cover movement expenses; a terminal charge.</def> <sd>(d)</sd> <def>A town lying at the end of a railroad,  in which the terminal is located; -- more properly called a <altname>terminus</altname>.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><sn>4.</sn> <def>The station at either end of a bus line line which transports freight or passengers.</def><br/
[<source>PJC</source>]</p>

<p><sn>5.</sn> <def>A station where passenger buses start or end a trip; -- also called <ecol><b>bus terminal</b></ecol>.</def><br/
[<source>PJC</source>]</p>

<p><sn>6.</sn> <def>The structure at an airport where passengers board or debark, and where ticket purchases and baggage pickup is performed; -- also called <ecol><b>airline terminal</b></ecol>.</def><br/
[<source>PJC</source>]</p>

<p><sn>7.</sn> <fld>(Computers)</fld> <def>An electronic device where data may be entered into a computer, and information received from it, usually consisting of a keyboard and video display unit (monitor); the terminal may be integrated or connected directly to a computer, or connected by a communications circuit with a computer at a remote location; -- also called <ecol><b>computer terminal</b></ecol>.</def><br/
[<source>PJC</source>]</p>

<p><cs><col><b>freight terminal</b></col>, <cd>a terminal used for loading or unloading of freight.</cd></cs><br/
[<source>PJC</source>]</p>

<p><ent>Terminalia</ent><br/
||<hw>Ter`mi*na"li*a</hw> <pr>(?)</pr>, <pos>n. pl.</pos> <ety>[L.]</ety> <fld>(Rom. Antiq.)</fld> <def>A festival celebrated annually by the Romans on February 23 in honor of <etsep>Terminus</etsep>, the god of boundaries.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Terminant</ent><br/
<hw>Ter"mi*nant</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>terminans</ets>, p. pr. of <ets>terminare</ets>.]</ety> <def>Termination; ending.</def> <mark>[R.]</mark>  <rj><au>Puttenham.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Terminate</ent><br/
<hw>Ter"mi*nate</hw> <pr>(?)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Terminated</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Terminating</conjf>.]</vmorph> <ety>[L. <ets>terminatus</ets>, p. p. of <ets>terminare</ets>. See <er>Term</er>.]</ety> <sn>1.</sn> <def>To set a term or limit to; to form the extreme point or side of; to bound; to limit; <as>as, to <ex>terminate</ex> a surface by a line</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To put an end to; to make to cease; <as>as, to <ex>terminate</ex> an effort, or a controversy</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>Hence, to put the finishing touch to; to bring to completion; to perfect.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>During this interval of calm and prosperity, he [Michael Angelo] <qex>terminated</qex> two figures of slaves, destined for the tomb, in an incomparable style of art.</q> <rj><qau>J. S. Harford.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><-- p. 1488 --><br/
[<source>1913 Webster</source>]</p>

<p><ent>Terminate</ent><br/
<hw>Ter"mi*nate</hw> <pr>(?)</pr>, <pos>v. i.</pos> <sn>1.</sn> <def>To be limited in space by a point, line, or surface; to stop short; to end; to cease; <as>as, the torrid zone <ex>terminates</ex> at the tropics</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To come to a limit in time; to end; to close.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The wisdom of this world, its designs and efficacy, <qex>terminate</qex> on zhis side heaven.</q> <rj><qau>South.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Termination</ent><br/
<hw>Ter`mi*na"tion</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>terminatio</ets> a bounding, fixing, determining: cf. F. <ets>terminasion</ets>, OF. also <ets>termination</ets>. See <er>Term</er>.]</ety> <sn>1.</sn> <def>The act of terminating, or of limiting or setting bounds; the act of ending or concluding; <as>as, a voluntary <ex>termination</ex> of hostilities</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>That which ends or bounds; limit in space or extent; bound; end; <as>as, the <ex>termination</ex> of a line</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>End in time or existence; <as>as, the <ex>termination</ex> of the year, or of life; the <ex>termination</ex> of happiness</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>End; conclusion; result.</def>  <rj><au>Hallam.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <def>Last purpose of design.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><sn>6.</sn> <def>A word; a term.</def> <mark>[R. & Obs.]</mark>  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>7.</sn> <fld>(Gram.)</fld> <def>The ending of a word; a final syllable or letter; the part added to a stem in inflection.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Terminational</ent><br/
<hw>Ter`mi*na"tion*al</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Of or pertaining to termination; forming a termination.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Terminative</ent><br/
<hw>Ter"mi*na*tive</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Tending or serving to terminate; terminating; determining; definitive.</def> <au>Bp. Rust.</au> -- <wordforms><wf>Ter"mi*na*tive*ly</wf>, <pos>adv.</pos> <au>Jer. Taylor.</au></wordforms><br/
[<source>1913 Webster</source>]</p>

<p><ent>Terminator</ent><br/
<hw>Ter"mi*na`tor</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L., he who limits or sets bounds.]</ety> <sn>1.</sn> <def>One who, or that which, terminates.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Astron.)</fld> <def>The dividing line between the illuminated and the unilluminated part of the moon.</def><br/
[<source>1913 Webster</source>]</p>

<p><-- The Terminator. Arnold Schwarzenegger. --><br/
[<source>1913 Webster</source>]</p>

<p><ent>Terminatory</ent><br/
<hw>Ter"mi*na*to*ry</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Terminative.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Termine</ent><br/
<hw>Ter"mine</hw> <pr>(?)</pr>, <pos>v. t.</pos> <ety>[Cf. F. <ets>terminer</ets>.]</ety> <def>To terminate.</def> <mark>[Obs.]</mark>  <rj><au>Bp. Hall.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Terminer</ent><br/
<hw>Ter"mi*ner</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F. <ets>terminer</ets> to bound, limit, end. See <er>Terminate</er>.]</ety> <fld>(Law)</fld> <def>A determining; <as>as, in oyer and <ex>terminer</ex></as>. See <er>Oyer</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Terminism</ent><br/
<hw>Ter"mi*nism</hw> <pr>(?)</pr>, <pos>n.</pos> <def>The doctrine held by the Terminists.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Terminist</ent><br/
<hw>Ter"mi*nist</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Cf. F. <ets>terministe</ets>.]</ety> <fld>(Theol.)</fld> <def>One of a class of theologians who maintain that God has fixed a certain term for the probation of individual persons, during which period, and no longer, they have the offer to grace.</def>  <rj><au>Murdock.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Terminological</ent><br/
<hw>Ter`mi*no*log"ic*al</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Of or pertaining to terminology.</def> -- <wordforms><wf>Ter`mi*no*log"ic*al*ly</wf>, <pos>adv.</pos></wordforms><br/
[<source>1913 Webster</source>]</p>

<p><ent>Terminology</ent><br/
<hw>Ter`mi*nol"o*gy</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>terminus</ets> term + <ets>-logy</ets>: cf. F. <ets>terminologie</ets>.]</ety> <sn>1.</sn> <def>The doctrine of terms; a theory of terms or appellations; a treatise on terms.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>The terms actually used in any business, art, science, or the like; nomenclature; technical terms; <as>as, the <ex>terminology</ex> of chemistry</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The barbarous effect produced by a German structure of sentence, and a <qex>terminology</qex> altogether new.</q> <rj><qau>De Quincey.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Term insurance</ent><br/
<hw>Term insurance</hw>. <def>Insurance for a specified term providing for no payment to the insured except upon losses during the term, and becoming void upon its expiration.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Terminus</ent><br/
<hw>Ter"mi*nus</hw> <pr>(?)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Termini</plw> <pr>(#)</pr>.</plu> <ety>[L. See <er>Term</er>.]</ety> <sn>1.</sn> <def>Literally, a boundary; a border; a limit.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Myth.)</fld> <def>The Roman divinity who presided over boundaries, whose statue was properly a short pillar terminating in the bust of a man, woman, satyr, or the like, but often merely a post or stone stuck in the ground on a boundary line.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>Hence, any post or stone marking a boundary; a term. See <er>Term</er>, 8.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>Either end of a railroad line; also, the station house, or the town or city, at that place.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Termite</ent><br/
<hw>Ter"mite</hw> <pr>(?)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Termites</plw> <pr>(#)</pr>.</plu> <ety>[F. See <er>Termes</er>.]</ety> <fld>(Zool.)</fld> <def>Any one of numerous species of pseudoneoropterous insects belonging to <gen>Termes</gen> and allied genera; -- called also <altname>white ant</altname>. See <xex>Illust.</xex> of <er>White ant</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ They are very abundant in tropical countries, and are noted for their destructive habits, their large nests, their remarkable social instincts, and their division of labor among the polymorphic individuals of several kinds. Besides the males and females, each nest has ordinary workers, and large-headed individuals called <xex>soldiers</xex>.</note><br/
[<source>1913 Webster</source>]</p>

<p><ent>Termless</ent><br/
<hw>Term"less</hw> <pr>(?)</pr>, <pos>a.</pos> <sn>1.</sn> <def>Having no term or end; unlimited; boundless; unending; <as>as, <ex>termless</ex> time</as>.</def> <mark>[R.]</mark> <ldquo/<xex>Termless</xex> joys.<rdquo/  <rj><au>Sir W. Raleigh.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Inexpressible; indescribable.</def> <mark>[R.]</mark>  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Termly</ent><br/
<hw>Term"ly</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Occurring every term; <as>as, a <ex>termly</ex> fee</as>.</def> <mark>[R.]</mark>  <rj><au>Bacon.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Termly</ent><br/
<hw>Term"ly</hw>, <pos>adv.</pos> <def>Term by term; every term.</def> <mark>[R.]</mark> <ldquo/Fees . . . that are <xex>termly</xex> given.<rdquo/  <rj><au>Bacon.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Termonology</ent><br/
<hw>Ter`mo*nol"o*gy</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Gr. <?/, <?/, boundary, end + <ets>-logy</ets>.]</ety> <def>Terminology.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Termor</ent><br/
<hw>Term"or</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Law)</fld> <def>Same as <er>Termer</er>, 2.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Term policy</ent><br/
<hw>Term policy</hw>. <def>A policy of term insurance.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Tern</ent><br/
<hw>Tern</hw> <pr>(t<etil/rn)</pr>, <pos>n.</pos> <ety>[Dan. <ets>terne</ets>, <ets>t<ae/rne</ets>; akin to Sw. <ets>t<aum/rna</ets>, Icel. <ets><thorn/erna</ets>; cf. NL. <ets>sterna</ets>.]</ety> <fld>(Zool.)</fld> <def>Any one of numerous species of long-winged aquatic birds, allied to the gulls, and belonging to <gen>Sterna</gen> and various allied genera.</def><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ Terns differ from gulls chiefly in their graceful form, in their weaker and more slender bills and feet, and their longer and more pointed wings. The tail is usually forked. Most of the species are white with the back and wings pale gray, and often with a dark head. The common European tern (<spn>Sterna hirundo</spn>) is found also in Asia and America. Among other American species are the arctic tern (<spn>Sterna paradisaea</spn>), the roseate tern (<spn>Sterna Dougalli</spn>), the least tern (<spn>Sterna Antillarum</spn>), the royal tern (<spn>Sterna maxima</spn>), and the sooty tern (<spn>Sterna fuliginosa</spn>).</note><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Hooded tern</b></col>. <cd>See <cref>Fairy bird</cref>, under <er>Fairy</er>.</cd> -- <col><b>Marsh tern</b></col>, <cd>any tern of the genus <gen>Hydrochelidon</gen>. They frequent marshes and rivers and feed largely upon insects.</cd> -- <col><b>River tern</b></col>, <cd>any tern belonging to <gen>Seena</gen> or allied genera which frequent rivers.</cd> -- <col><b>Sea tern</b></col>, <cd>any tern of the genus <gen>Thalasseus</gen>. Terns of this genus have very long, pointed wings, and chiefly frequent seas and the mouths of large rivers.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tern</ent><br/
<hw>Tern</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. pl. <ets>terni</ets> three each, three; akin to <ets>tres</ets> three. See <er>Three</er>, and cf. <er>Trine</er>.]</ety> <def>Threefold; triple; consisting of three; ternate.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Tern flowers</b></col> <fld>(Bot.)</fld>, <cd>flowers growing three and three together.</cd> -- <col><b>Tern leaves</b></col> <fld>(Bot.)</fld>, <cd>leaves arranged in threes, or three by three, or having three in each whorl or set.</cd> -- <col><b>Tern peduncles</b></col> <fld>(Bot.)</fld>, <cd>three peduncles growing together from the same axis.</cd> -- <col><b>Tern schooner</b></col> <fld>(Naut.)</fld>, <cd>a three-masted schooner.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tern</ent><br/
<hw>Tern</hw>, <pos>n.</pos> <ety>[F. <ets>terne</ets>. See <er>Tern</er>, <pos>a.</pos>]</ety> <def>That which consists of, or pertains to, three things or numbers together; especially, a prize in a lottery resulting from the favorable combination of three numbers in the drawing; also, the three numbers themselves.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>She'd win a <qex>tern</qex> in Thursday's lottery.</q> <rj><qau>Mrs. Browning.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Ternary</ent><br/
<hw>Ter"na*ry</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>ternarius</ets>, fr. <ets>terni</ets>. See <er>Tern</er>, <pos>a.</pos>]</ety> <sn>1.</sn> <def>Proceeding by threes; consisting of three; <as>as, the <ex>ternary</ex> number was anciently esteemed a symbol of perfection, and held in great veneration</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Chem.)</fld> <def>Containing, or consisting of, three different parts, as elements, atoms, groups, or radicals, which are regarded as having different functions or relations in the molecule; <as>thus, sodic hydroxide, <chform>NaOH</chform>, is a <ex>ternary</ex> compound</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Ternary</ent><br/
<hw>Ter"na*ry</hw>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Ternaries</plw> <pr>(<?/)</pr>.</plu> <def>A ternion; the number three; three things taken together; a triad.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Some in <qex>ternaries</qex>, some in pairs, and some single.</q> <rj><qau>Holder.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Ternate</ent><br/
<hw>Ter"nate</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[NL. <ets>ternatus</ets>, fr. L. <ets>terni</ets> three each. See <er>Tern</er>, <pos>a.</pos>]</ety> <def>Having the parts arranged by threes; <as>as, <ex>ternate</ex> branches, leaves, or flowers</as>.</def> -- <wordforms><wf>Ter"nate*ly</wf>, <pos>adv.</pos></wordforms><br/
[<source>1913 Webster</source>]</p>

<p><ent>Terneplate</ent><br/
<hw>Terne"plate`</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[See <er>Tern</er>, <pos>a.</pos>, and <er>Plate</er>.]</ety> <def>Thin iron sheets coated with an alloy of lead and tin; -- so called because made up of <xex>three</xex> metals.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Ternion</ent><br/
<hw>Ter"ni*on</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>ternio</ets>, fr. <ets>terni</ets> three each. See <er>Tern</er>, <pos>a.</pos>]</ety> <def>The number three; three things together; a ternary.</def>  <rj><au>Bp. Hall.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Terpene</ent><br/
<hw>Ter"pene</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[See <er>Turpentine</er>.]</ety> <fld>(Chem.)</fld> <def>Any one of a series of isomeric hydrocarbons of pleasant aromatic odor, occurring especially in coniferous plants and represented by oil of turpentine, but including also certain hydrocarbons found in some essential oils.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Terpentic</ent><br/
<hw>Ter*pen"tic</hw> <pr>(?)</pr>, <pos>a.</pos> <fld>(Chem.)</fld> <def>Terpenylic.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Terpenylic</ent><br/
<hw>Ter`pe*nyl"ic</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[<ets>Terpene</ets> + <ets>-yl</ets> + <ets>-ic</ets>.]</ety> <fld>(Chem.)</fld> <def>Of, pertaining to, or designating, an acid, <chform>C8H12O4</chform> (called also <xex>terpentic acid</xex>), homologous with terebic acid, and obtained as a white crystalline substance by the oxidation of oil of turpentine with chromic acid.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Terpilene</ent><br/
<hw>Ter"pi*lene</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Chem.)</fld> <def>A polymeric form of terpene, resembling terbene.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Terpin</ent><br/
<hw>Ter"pin</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Chem.)</fld> <def>A white crystalline substance regarded as a hydrate of oil of turpentine.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Terpinol</ent><br/
<hw>Ter"pin*ol</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[<ets>Terpin</ets> + L. <ets>ol</ets>eum oil.]</ety> <fld>(Chem.)</fld> <def>Any oil substance having a hyacinthine odor, obtained by the action of acids on terpin, and regarded as a related hydrate.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Terpsichore</ent><br/
<hw>Terp*sich"o*re</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L., fr. Gr. <?/; <?/ enjoyment (fr. <?/ to gladden) + <?/ dance, dancing.]</ety> <fld>(Gr. Myth.)</fld> <def>The Muse who presided over the choral song and the dance, especially the latter.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Terpsichorean</ent><br/
<hw>Terp`sich*o*re"an</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Of or pertaining to Terpsichore; of or pertaining to dancing.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Terra</ent><br/
||<hw>Ter"ra</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[It. & L. See <er>Terrace</er>.]</ety> <def>The earth; earth.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Terra alba</b></col> <ety>[L., white earth]</ety> <fld>(Com.)</fld>, <cd>a white amorphous earthy substance consisting of burnt gypsum, aluminium silicate (kaolin), or some similar ingredient, as magnesia. It is sometimes used to adulterate certain foods, spices, candies, paints, etc.</cd> -- <col><b>Terra cotta</b></col>. <ety>[It., fr. <ets>terra</ets> earth + <ets>cotta</ets>, fem. of <ets>cotto</ets> cooked, L. <ets>coctus</ets>, p. p. of <ets>coquere</ets> to cook. See <er>Cook</er>, <pos>n.</pos>]</ety> <cd>Baked clay; a kind of hard pottery used for statues, architectural decorations, figures, vases, and the like.</cd> -- <col><b>Terrae filius</b></col> <ety>[L., son of the earth]</ety>, <cd>formerly, one appointed to write a satirical Latin poem at the public acts in the University of Oxford; -- not unlike the <it>prevaricator</it> at Cambridge, England.</cd> -- <col><b>Terra firma</b></col> <ety>[L.]</ety>, <cd>firm or solid earth, as opposed to <contr>water</contr>.</cd> -- <col><b>Terra Japonica</b></col>. <ety>[NL.]</ety> <cd>Same as <er>Gambier</er>. It was formerly supposed to be a kind of earth from Japan.</cd> -- <col><b>Terra Lemnia</b></col> <ety>[L., Lemnian earth]</ety>, <cd>Lemnian earth. See under <er>Lemnian</er>.</cd> -- <col><b>Terra ponderosa</b></col> <ety>[L., ponderous earth]</ety> <fld>(Min.)</fld>, <cd>barite, or heavy spar.</cd> -- <col><b>Terra di Sienna</b></col>. <cd>See <er>Sienna</er>.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Terrace</ent><br/
<hw>Ter"race</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F. <ets>terrasse</ets> (cf. Sp. <ets>terraza</ets>, It. <ets>terrazza</ets>), fr. L. <ets>terra</ets> the earth, probably for <ets>tersa</ets>, originally meaning, dry land, and akin to <ets>torrere</ets> to parch, E. <ets>torrid</ets>, and <ets>thirst</ets>. See <er>Thirst</er>, and cf. <er>Fumitory</er>, <er>Inter</er>, <pos>v.</pos>, <er>Patterre</er>, <er>Terrier</er>, <er>Trass</er>, <er>Tureen</er>, <er>Turmeric</er>.]</ety> <sn>1.</sn> <def>A raised level space, shelf, or platform of earth, supported on one or more sides by a wall, a bank of tuft, or the like, whether designed for use or pleasure.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A balcony, especially a large and uncovered one.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>A flat roof to a house; <as>as, the buildings of the Oriental nations are covered with <ex>terraces</ex></as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>A street, or a row of houses, on a bank or the side of a hill; hence, any street, or row of houses.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <fld>(Geol.)</fld> <def>A level plain, usually with a steep front, bordering a river, a lake, or sometimes the sea.</def><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ Many rivers are bordered by a series of terraces at different levels, indicating the flood plains at successive periods in their history.</note><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Terrace epoch</b></col>. <fld>(Geol.)</fld> <cd>See <cref>Drift epoch</cref>, under <er>Drift</er>, <pos>a.</pos></cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Terrace</ent><br/
<hw>Ter"race</hw>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Terraced</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Terracing</conjf> <pr>(?)</pr>.]</vmorph> <def>To form into a terrace or terraces; to furnish with a terrace or terraces, <as>as, to <ex>terrace</ex> a garden, or a building</as>.</def>  <rj><au>Sir H. Wotton.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Clermont's <qex>terraced</qex> height, and Esher's groves.</q> <rj><qau>Thomson.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Terraculture</ent><br/
<hw>Ter"ra*cul`ture</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>terra</ets> the earth + <ets>cultura</ets>.]</ety> <def>Cultivation on the earth; agriculture.</def> <mark>[R.]</mark> -- <wordforms><wf>Ter`ra*cul"tur*al</wf> <pr>(#)</pr>, <pos>a.</pos> <mark>[R.]</mark></wordforms><br/
[<source>1913 Webster</source>]</p>

<p><ent>Terra incognita</ent><br/
||<hw>Ter"ra in*cog"ni*ta</hw> <pr>(?)</pr>. <ety>[L.]</ety> <def>An unknown land; unexplored country.</def></p>

<p><q>The enormous tracts lying outside China proper, still almost <qex>terrae incognitae</qex>.</q>  <rj><qau>A. R. Colquhoun.</qau></rj><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Terrane</ent><br/
<hw>Ter"rane</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F. <ets>terrain</ets>, from L. <ets>terra</ets> earth.]</ety> <sn>1.</sn> <fld>(Geol.)</fld> <def>A group of rocks having a common age or origin; -- nearly equivalent to <xex>formation</xex>, but used somewhat less comprehensively.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Terrane</ent><br/
<hw>Ter"rane</hw>, <pos>n.</pos> <fld>(Geog.)</fld> <sn>2.</sn> <def>A region or limited area considered with reference to some special feature; <as>as, the <ex>terrane</ex> of a river, that is, its drainage basin</as>.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Terrapin</ent><br/
<hw>Ter"ra*pin</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Probably of American Indian origin.]</ety> <fld>(Zool.)</fld> <def>Any one of numerous species of tortoises living in fresh and brackish waters. Many of them are valued for food.</def> <altsp>[Written also <asp>terapin</asp>, <asp>terrapen</asp>, <asp>terrapene</asp>, <asp>turpen</asp>, and <asp>turapen</asp>.]</altsp><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ The yellow-bellied terrapin (<spn>Pseudemys scabra</spn>) of the Southern United States, the red-bellied terrapin (<spn>Pseudemys rugosa</spn> or <spn>Chrysemys rubriventris</spn>), native of the tributaries Chesapeake Bay (called also <altname>potter</altname>, <altname>slider</altname>, and <altname>redfender</altname>), and the diamond-back or salt-marsh terrapin (<spn>Malaclemmys palustris</spn>), are the most important American species. The diamond-back terrapin is native of nearly the whole of the Atlantic coast of the United States.</note><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Alligator terrapin</b></col>, <cd>the snapping turtle.</cd> -- <col><b>Mud terrapin</b></col>, <cd>any one of numerous species of American tortoises of the genus <gen>Cinosternon</gen>.</cd> -- <col><b>Painted terrapin</b></col>, <cd>the painted turtle. See under <er>Painted</er>.</cd> -- <col><b>Speckled terrapin</b></col>, <cd>a small fresh-water American terrapin (<spn>Chelopus guttatus</spn>) having the carapace black with round yellow spots; -- called also <altname>spotted turtle</altname>.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Terraqueous</ent><br/
<hw>Ter*ra"que*ous</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>terra</ets> the earth + E. <ets>aqueous</ets>.]</ety> <def>Consisting of land and water; <as>as, the earth is a <ex>terraqueous</ex> globe</as>.</def>  <rj><au>Cudworth.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>The grand <qex>terraqueous</qex> spectacle<br/
From center to circumference unveiled.</q> <rj><qau>Wordsworth.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Terrar</ent><br/
<hw>Ter"rar</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[LL. <ets>terrarius liber</ets>. See <er>Terrier</er> a collection of acknowledgments.]</ety> <fld>(O. Eng. Law)</fld> <def>See 2d <er>Terrier</er>, 2.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Terras</ent><br/
<hw>Ter"ras</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Min.)</fld> <def>See <er><?/rass</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Terreen</ent><br/
<hw>Ter*reen"</hw> <pr>(?)</pr>, <pos>n.</pos> <def>See <er>Turren</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Terreity</ent><br/
<hw>Ter*re"i*ty</hw> <pr>(?)</pr>, <pos>n.</pos> <def>Quality of being earthy; earthiness.</def> <mark>[Obs.]</mark>  <rj><au>B. Jonson.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Terrel</ent><br/
<hw>Ter"rel</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[NL. <ets>terrella</ets>, from L. <ets>terra</ets> the earth.]</ety> <def>A spherical magnet so placed that its poles, equator, etc., correspond to those of the earth.</def> <mark>[Obs.]</mark>  <rj><au>Chambers.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Terremote</ent><br/
<hw>Terre"mote`</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[OF. <ets>terremote</ets>, <ets>terremoete</ets>, fr. L. <ets>terra</ets> the earth + <ets>movere</ets>, <ets>motum</ets>, to move.]</ety> <def>An earthquake.</def> <mark>[Obs.]</mark>  <rj><au>Gower.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Terrene</ent><br/
<hw>Ter*rene"</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A tureen.</def> <mark>[Obs.]</mark>  <rj><au>Walpole.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Terrene</ent><br/
<hw>Ter*rene"</hw>, <pos>a.</pos> <ety>[L. <ets>terrenus</ets>, fr. <ets>terra</ets> the earth. See <er>Terrace</er>.]</ety> <sn>1.</sn> <def>Of or pertaining to the earth; earthy; <as>as, <ex>terrene</ex> substance</as>.</def>  <rj><au>Holland.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Earthy; terrestrial.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>God set before him a mortal and immortal life, a nature celestial and <qex>terrene</qex>.</q> <rj><qau>Sir W. Raleigh.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Be true and faithful to the king and his heirs, and truth and faith to bear of life and limb, and <qex>terrene</qex> honor.</q> <rj><qau>O. Eng. Oath of Allegiance, quoted by Blackstone.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Common conceptions of the matters which lie at the basis of our <qex>terrene</qex> experience.</q> <rj><qau>Hickok.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Terrene</ent><br/
<hw>Ter*rene"</hw>, <pos>n.</pos> <ety>[L. <ets>terrenum</ets> land, ground: cf. F. <ets>terrain</ets>.]</ety> <sn>1.</sn> <def>The earth's surface; the earth.</def> <mark>[Poetic]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>Tenfold the length of this <qex>terrene</qex>.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Surv.)</fld> <def>The surface of the ground.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Terrenity</ent><br/
<hw>Ter*ren"i*ty</hw> <pr>(?)</pr>, <pos>n.</pos> <def>Earthiness; worldliness.</def> <mark>[Obs.]</mark> <ldquo/A dull and low <xex>terrenity</xex>.<rdquo/  <rj><au>Feltham.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Terreous</ent><br/
<hw>Ter"re*ous</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>terreus</ets>, fr. <ets>terra</ets> the earth. See <er>Terrace</er>.]</ety> <def>Consisting of earth; earthy; <as>as, <ex>terreous</ex> substances; <ex>terreous</ex> particles</as>.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Terreplein</ent><br/
<hw>Terre"plein`</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F., fr. L. <ets>terra</ets> earth + <ets>planus</ets> even, level, plain.]</ety> <sn>1.</sn> <fld>(Fort.)</fld> <def>The top, platform, or horizontal surface, of a rampart, on which the cannon are placed. See <xex>Illust.</xex> of <er>Casemate</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Civ. Engin.)</fld> <def>An embankment of earth with a broad level top, which is sometimes excavated to form a continuation of an elevated canal across a valley.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Terrestre</ent><br/
<hw>Ter*res"tre</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[OE., from OF. & F. <ets>terrestre</ets>.]</ety> <def>Terrestrial; earthly.</def> <mark>[Obs.]</mark> <ldquo/His paradise <xex>terrestre</xex>.<rdquo/  <rj><au>Chaucer.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Terrestrial</ent><br/
<hw>Ter*res"tri*al</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>terrestris</ets>, from <ets>terra</ets> the earth. See <er>Terrace</er>.]</ety> <sn>1.</sn> <def>Of or pertaining to the earth; existing on the earth; earthly; <as>as, <ex>terrestrial</ex> animals</as>.</def> <ldquo/Bodies <xex>terrestrial</xex>.<rdquo/  <rj><au>1 Cor. xv. 40.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Representing, or consisting of, the earth; <as>as, a <ex>terrestrial</ex> globe</as>.</def> <ldquo/The dark <xex>terrestrial</xex> ball.<rdquo/  <rj><au>Addison.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>Of or pertaining to the world, or to the present state; sublunary; mundane.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Vain labors of <qex>terrestrial</qex> wit.</q> <rj><qau>Spenser.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>A genius bright and base,<br/
Of towering talents, and <qex>terrestrial</qex> aims.</q> <rj><qau>Young.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>Consisting of land, in distinction from water; belonging to, or inhabiting, the land or ground, in distinction from trees, water, or the like; <as>as, <ex>terrestrial</ex> serpents</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The <qex>terrestrial</qex> parts of the globe.</q> <rj><qau>Woodward.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <def>Adapted for the observation of objects on land and on the earth; <as>as, a <ex>terrestrial</ex> telescope, in distinction from an <ex>astronomical</ex> telescope</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p>-- <wordforms><wf>Ter*res"tri*al*ly</wf>, <pos>adv.</pos> -- <wf>Ter*res"tri*al*ness</wf>, <pos>n.</pos></wordforms><br/
[<source>1913 Webster</source>]</p>

<p><ent>Terrestrial</ent><br/
<hw>Ter*res"tri*al</hw>, <pos>n.</pos> <def>An inhabitant of the earth.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Terrestrify</ent><br/
<hw>Ter*res"tri*fy</hw> <pr>(?)</pr>, <pos>v. t.</pos> <ety>[L. <ets>terrestris</ets> terrestrial + <ets>-fy</ets>.]</ety> <def>To convert or reduce into a condition like that of the earth; to make earthy.</def> <mark>[Obs.]</mark>  <rj><au>Sir T. Browne.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Terrestrious</ent><br/
<hw>Ter*res"tri*ous</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[See <er>Terrestrial</er>.]</ety> <def>Terrestrial.</def> <mark>[Obs.]</mark>  <rj><au>Sir T. Browne.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Terret</ent><br/
<hw>Ter"ret</hw> <pr>(?)</pr>, <pos>n.</pos> <def>One of the rings on the top of the saddle of a harness, through which the reins pass.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Terre-tenant</ent><br/
<hw>Terre"-ten`ant</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F. <ets>terre</ets> earth, land + <ets>tenant</ets>, p. pr. of <ets>tenir</ets> to hold.]</ety> <fld>(Law)</fld> <def>One who has the actual possession of land; the occupant.</def> <altsp>[Written also <asp>ter-tenant</asp>.]</altsp><br/
[<source>1913 Webster</source>]</p>

<p><-- p. 1489 --><br/
[<source>1913 Webster</source>]</p>

<p><ent>Terre-verte</ent><br/
<hw>Terre"-verte`</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F., fr. <ets>terre</ets> earth + <ets>vert</ets>, <ets>verte</ets>, green.]</ety> <def>An olive-green earth used as a pigment. See <er>Glauconite</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Terrible</ent><br/
<hw>Ter"ri*ble</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[F., fr. L. <ets>terribilis</ets>, fr. <ets>terrere</ets> to frighten. See <er>Terror</er>.]</ety> <sn>1.</sn> <def>Adapted or likely to excite terror, awe, or dread; dreadful; formidable.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Prudent in peace, and <qex>terrible</qex> in war.</q> <rj><qau>Prior.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Thou shalt not be affrighted at them; for the Lord thy God is among you, a mighty God and <qex>terrible</qex>.</q> <rj><qau>Deut. vii. 21.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Excessive; extreme; severe.</def> <mark>[Colloq.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>The <qex>terrible</qex> coldness of the season.</q> <rj><qau>Clarendon.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><syn><b>Syn.</b> -- Terrific; fearful; frightful; formidable; dreadful; horrible; shocking; awful.</syn><br/
[<source>1913 Webster</source>]</p>

<p>-- <wordforms><wf>Ter"ri*ble*ness</wf>, <pos>n.</pos> -- <wf>Ter"ri*bly</wf>, <pos>adv.</pos></wordforms><br/
[<source>1913 Webster</source>]</p>

<p><ent>Terricolae</ent><br/
||<hw>Ter*ric"o*lae</hw> <pr>(?)</pr>, <pos>n. pl.</pos> <ety>[NL., fr. L. <ets>terra + colere</ets> to inhabit.]</ety> <fld>(Zool.)</fld> <def>A division of annelids including the common earthworms and allied species.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Terrienniak</ent><br/
<hw>Ter`ri*en"ni*ak</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Zool.)</fld> <def>The arctic fox.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Terrier</ent><br/
<hw>Ter"ri*er</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[CF. L. <ets>terere</ets> to rub, to rub away, <ets>terebra</ets> a borer.]</ety> <def>An auger or borer.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Terrier</ent><br/
<hw>Ter"ri*er</hw>, <pos>n.</pos> <sn>1.</sn> <ety>[F. <ets>terrier</ets>, <ets>chien terrier</ets>, from <ets>terre</ets> the earth, L. <ets>terra</ets>; cf. F. <ets>terrier</ets> a burrow, LL. <ets>terrarium</ets> a hillock (hence the sense, a mound thrown up in making a burrow, a burrow). See <er>Terrace</er>, and cf. <er>Terrier</er>, 2.]</ety> <fld>(Zool.)</fld> <def>One of a breed of small dogs, which includes several distinct subbreeds, some of which, such as the Skye terrier and Yorkshire terrier, have long hair and drooping ears, while others, at the English and the black-and-tan terriers, have short, close, smooth hair and upright ears.</def><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ Most kinds of terriers are noted for their courage, the acuteness of their sense of smell, their propensity to hunt burrowing animals, and their activity in destroying rats, etc. See <cref>Fox terrier</cref>, under <er>Fox</er>.</note><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <ety>[F. <ets>terrier</ets>, <ets>papier terrier</ets>, LL. <ets>terrarius liber</ets>, i.e., a book belonging or pertaining to land or landed estates. See <er>Terrier</er>, 1, and cf. <er>Terrar</er>.]</ety> <fld>(Law)</fld> <sd>(a)</sd> <def>Formerly, a collection of acknowledgments of the vassals or tenants of a lordship, containing the rents and services they owed to the lord, and the like.</def> <sd>(b)</sd> <def>In modern usage, a book or roll in which the lands of private persons or corporations are described by their site, boundaries, number of acres, or the like.</def> <altsp>[Written also <asp>terrar</asp>.]</altsp><br/
[<source>1913 Webster</source>]</p>

<p><ent>Terrific</ent><br/
<hw>Ter*rif"ic</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>terrificus</ets>; fr. <ets>terrere</ets> to frighten + <ets>facere</ets> to make. See <er>Terror</er>, and <er>Fact</er>.]</ety> <def>Causing terror; adapted to excite great fear or dread; terrible; <as>as, a <ex>terrific</ex> form; a <ex>terrific</ex> sight.</as></def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Terrifical</ent><br/
<hw>Ter*rif"ic*al</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Terrific.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Terrifically</ent><br/
<hw>Ter*rif"ic*al*ly</hw>, <pos>adv.</pos> <def>In a terrific manner.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Terrify</ent><br/
<hw>Ter"ri*fy</hw> <pr>(?)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Terrified</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Terrifying</conjf> <pr>(?)</pr>.]</vmorph> <ety>[L. <ets>terrere</ets> to frighten + <ets>-fy</ets>: cf. F. <ets>terrifier</ets>, L. <ets>terrificare</ets>. See <er>Terrific</er>, and <er>-fy</er>.]</ety> <sn>1.</sn> <def>To make terrible.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>If the law, instead of aggravating and <qex>terrifying</qex> sin, shall give out license, it foils itself.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To alarm or shock with fear; to frighten.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>When ye shall hear of wars . . . be not <qex>terrified</qex>.</q> <rj><qau>Luke xxi. 9.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Terrigenous</ent><br/
<hw>Ter*rig"e*nous</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>terrigena</ets>, <ets>terrigenus</ets>; <ets>terra</ets> the earth + <ets>genere</ets>, <ets>gignere</ets>, to bring forth.]</ety> <def>Earthborn; produced by the earth.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Terrine</ent><br/
<hw>Ter*rine"</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F. See <er>Tureen</er>.]</ety> <sn>1.</sn> <def>A dish or pan, originally of earthenware, such as those in which  various dishes are cooked and served; esp., an earthenware jar containing some table delicacy and sold with its contents.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><sn>2.</sn>  <fld>(Cookery)</fld> <def>A kind of ragout formerly cooked and served in the same dish; also, a dish consisting of several meats braised together and served in a terrine.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><sn>3.</sn>  <def>A soup tureen.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Territorial</ent><br/
<hw>Ter`ri*to"ri*al</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>territorialis</ets>: cf. F. <ets>territorial</ets>.]</ety> <sn>1.</sn> <def>Of or pertaining to territory or land; <as>as, <ex>territorial</ex> limits; <ex>territorial</ex> jurisdiction.</as></def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Limited to a certain district; <as>as, right may be personal or <ex>territorial</ex></as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>Of or pertaining to all or any of the Territories of the United States, or to any district similarly organized elsewhere; <as>as, <ex>Territorial</ex> governments</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <fld>(Zool.)</fld> <def>exhibiting <er>territoriality</er>; -- of individual animals or species.</def><br/
[<source>PJC</source>]</p>

<p><ent>Territoriality</ent><br/
<hw>Ter`ri*to`ri*al"i*ty</hw> <pos>n.</pos> <fld>(Zool.)</fld>  <def>a pattern of behavior characterized by the marking and defense of a home territory, exhibited by certain male animals.</def><br/
[<source>WordNet 1.5</source> <source>+PJC</source>]</p>

<p><ent>Territorialize</ent><br/
<hw>Ter`ri*to"ri*al*ize</hw> <pr>(?)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Territorialized</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Territorializing</conjf> <pr>(?)</pr>.]</vmorph> <sn>1.</sn> <def>To enlarge by extension of territory.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To reduce to the condition of a territory.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Territorially</ent><br/
<hw>Ter`ri*to"ri*al*ly</hw>, <pos>adv.</pos> <def>In regard to territory; by means of territory.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Territorial waters</ent><br/
<hw>Ter`ri*to"ri*al wa"ters</hw>. <fld>(Internat. Law)</fld> <def>The waters under the territorial jurisdiction of a state; specif., the  belt (often called the <col><b>marine belt</b></col> or <col><b>territorial sea</b></col>) of sea subject to such jurisdiction, and subject only to the right of innocent passage by the vessels of other states.</def></p>

<p><q>Perhaps it may be said without impropriety that a state has theoretically the right to extend its <qex>territorial waters</qex> from time to time at its will with the increased range of guns. Whether it would in practice be judicious to do so . . . is a widely different matter . . . . In any case the custom of regulating a line three miles from land as defining the boundary of marginal <qex>territorial waters</qex> is so far fixed that a state must be supposed to accept it in absence of express notice.</q>  <rj><qau>W. E. Hall.</qau></rj><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Territoried</ent><br/
<hw>Ter"ri*to*ried</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Possessed of territory.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Territory</ent><br/
<hw>Ter"ri*to*ry</hw> <pr>(?)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Territories</plw> <pr>(#)</pr>.</plu> <ety>[L. <ets>territorium</ets>, from <ets>terra</ets> the earth: cf. F. <ets>territoire</ets>. See <er>Terrace</er>.]</ety> <sn>1.</sn> <def>A large extent or tract of land; a region; a country; a district.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>He looked, and saw wide <qex>territory</qex> spread<br/
Before him -- towns, and rural works between.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>The extent of land belonging to, or under the dominion of, a prince, state, or other form of government; often, a tract of land lying at a distance from the parent country or from the seat of government; <as>as, the <ex>territory</ex> of a State; the <ex>territories</ex> of the East India Company</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>In the United States, a portion of the country not included within the limits of any State, and not yet admitted as a State into the Union, but organized with a separate legislature, under a Territorial governor and other officers appointed by the President and Senate of the United States. In Canada, a similarly organized portion of the country not yet formed into a Province.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Terror</ent><br/
<hw>Ter"ror</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>terror</ets>, akin to <ets>terrere</ets> to frighten, for <ets>tersere</ets>; akin to Gr. <?/ to flee away, dread, Skr. <ets>tras</ets> to tremble, to be afraid, Russ. <ets>triasti</ets> to shake: cf. F. <ets>terreur</ets>.  Cf. <er>Deter</er>.]</ety> <sn>1.</sn> <def>Extreme fear; fear that agitates body and mind; violent dread; fright.</def><br/
[<source>1913 Webster</source>]</p>

<p><q><qex>Terror</qex> seized the rebel host.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>That which excites dread; a cause of extreme fear.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Those enormous <qex>terrors</qex> of the Nile.</q> <rj><qau>Prior.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Rulers are not a <qex>terror</qex> to good works.</q> <rj><qau>Rom. xiii. 3.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>There is no <qex>terror</qex>, Cassius, in your threats.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ <xex>Terror</xex> is used in the formation of compounds which are generally self-explaining: as, <xex>terror</xex>-fraught, <xex>terror</xex>-giving, <xex>terror</xex>-smitten, <xex>terror</xex>-stricken, <xex>terror</xex>-struck, and the like.</note><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>King of terrors</b></col>, <cd>death.</cd> <au>Job xviii. 14.</au> -- <col><b>Reign of Terror</b></col>. <fld>(French Hist.)</fld> <cd>See in Dictionary of Noted Names in Fiction.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><syn><b>Syn.</b> -- Alarm; fright; consternation; dread; dismay. See <er>Alarm</er>.</syn><br/
[<source>1913 Webster</source>]</p>

<p><ent>Terrorism</ent><br/
<hw>Ter"ror*ism</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Cf. F. <ets>terrorisme</ets>.]</ety> <sn>1.</sn> <def>The act of terrorizing, or state of being terrorized; a mode of government by terror or intimidation.</def>  <rj><au>Jefferson.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>The practise of coercing governments to accede to political demands by committing violence on civilian targets; any similar use of violence to achieve goals.</def><br/
[<source>PJC</source>]</p>

<p><ent>Terrorist</ent><br/
<hw>Ter"ror*ist</hw>, <pos>n.</pos> <ety>[F. <ets>terroriste</ets>.]</ety> <sn>1.</sn> <def>One who governs by terrorism or intimidation; specifically, an agent or partisan of the revolutionary tribunal during the Reign of Terror in France.</def>  <rj><au>Burke.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>One who commits terrorism{2}.</def><br/
[<source>PJC</source>]</p>

<p><ent>Terrorize</ent><br/
<hw>Ter"ror*ize</hw> <pr>(?)</pr>, <pos>v. t.</pos> <ety>[Cf. F. <ets>terroriser</ets>.]</ety> <def>To impress with terror; to coerce by intimidation.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Humiliated by the tyranny of foreign despotism, and <qex>terrorized</qex> by ecclesiastical authority.</q> <rj><qau>J. A. Symonds.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Terrorless</ent><br/
<hw>Ter"ror*less</hw>, <pos>a.</pos> <def>Free from terror.</def>  <rj><au>Poe.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Terry</ent><br/
<hw>Ter"ry</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A kind of heavy colored fabric, either all silk, or silk and worsted, or silk and cotton, often called <xex>terry velvet</xex>, used for upholstery and trimmings.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tersanctus</ent><br/
||<hw>Ter*sanc"tus</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>ter</ets> thrice + <ets>sanctus</ets> holy.]</ety> <fld>(Eccl.)</fld> <def>An ancient ascription of praise (containing the word <ldquo/Holy<rdquo/ -- in its Latin form, <ldquo/<xex>Sanctus</xex><rdquo/ -- thrice repeated), used in the Mass of the Roman Catholic Church and before the prayer of consecration in the communion service of the Church of England and the Protestant Episcopal Church.  Cf. <er>Trisagion</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Terse</ent><br/
<hw>Terse</hw> <pr>(?)</pr>, <pos>a.</pos> <amorph>[<pos>Compar.</pos> <adjf>Terser</adjf> <pr>(?)</pr>; <pos>superl.</pos> <adjf>Tersest</adjf>.]</amorph> <ety>[L. <ets>tersus</ets>, p. p. of <ets>tergere</ets> to rub or wipe off.]</ety> <sn>1.</sn> <def>Appearing as if rubbed or wiped off; rubbed; smooth; polished.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>Many stones, . . . although <qex>terse</qex> and smooth, have not this power attractive.</q> <rj><qau>Sir T. Browne.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Refined; accomplished; -- said of persons.</def> <mark>[R. & Obs.]</mark> <ldquo/Your polite and <xex>terse</xex> gallants.<rdquo/  <rj><au>Massinger.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>Elegantly concise; free of superfluous words; polished to smoothness; <as>as, <ex>terse</ex> language; a <ex>terse</ex> style</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q><qex>Terse</qex>, luminous, and dignified eloquence.</q> <rj><qau>Macaulay.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>A poet, too, was there, whose verse<br/
Was tender, musical, and <qex>terse</qex>.</q> <rj><qau>Longfellow.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><syn><b>Syn.</b> -- Neat; concise; compact.</syn> <usage> <er>Terse</er>, <er>Concise</er>. <xex>Terse</xex> was defined by Johnson <ldquo/cleanly written<rdquo/, <it>i. e.</it>, free from blemishes, neat or smooth. Its present sense is <ldquo/free from excrescences,<rdquo/ and hence, <xex>compact</xex>, with smoothness, grace, or elegance, as in the following lones of Whitehead: -<br/
[<source>1913 Webster</source>]</p>

<p><q><ldquo/In eight <qex>terse</qex> lines has Phaedrus told<br/
(So frugal were the bards of old)<br/
A tale of goats; and closed with grace,<br/
Plan, moral, all, in that short space.<rdquo/</q><br/
[<source>1913 Webster</source>]</p>

<p>  It differs from <xex>concise</xex> in not implying, perhaps, quite as much condensation, but chiefly in the additional idea of <ldquo/grace or elegance.<rdquo/</usage><br/
[<source>1913 Webster</source>]</p>

<p>-- <wordforms><wf>Terse"ly</wf>, <pos>adv.</pos> -- <wf>Terse"ness</wf>, <pos>n.</pos></wordforms><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tersulphide</ent><br/
<hw>Ter*sul"phide</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Pref. <ets>ter-</ets> + <ets>sulphide</ets>.]</ety> <fld>(Chem.)</fld> <def>A trisulphide.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tersulphuret</ent><br/
<hw>Ter*sul"phu*ret</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Pref. <ets>ter-</ets> + <ets>sulphuret</ets>.]</ety> <fld>(Chem.)</fld> <def>A trisulphide.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Ter-tenant</ent><br/
<hw>Ter"-ten`ant</hw> <pr>(?)</pr>, <pos>n.</pos> <def>See <er>Terre-tenant</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tertial</ent><br/
<hw>Ter"tial</hw> <pr>(?)</pr>, <pos>a. & n.</pos> <ety>[From L. <ets>tertius</ets> third, the tertial feathers being feathers of the third row. See <er>Tierce</er>.]</ety> <fld>(Zool.)</fld> <def>Same as <er>Tertiary</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tertian</ent><br/
<hw>Ter"tian</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>tertianus</ets>, from <ets>tertius</ets> the third. See <er>Tierce</er>.]</ety> <fld>(Med.)</fld> <def>Occurring every third day; <as>as, a <ex>tertian</ex> fever</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tertian</ent><br/
<hw>Ter"tian</hw>, <pos>n.</pos> <ety>[L. <ets>tertiana</ets> (sc. <ets>febris</ets>): cf. OF. <ets>tertiane</ets>.]</ety> <sn>1.</sn> <fld>(Med.)</fld> <def>A disease, especially an intermittent fever, which returns every third day, reckoning inclusively, or in which the intermission lasts one day.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A liquid measure formerly used for wine, equal to seventy imperial, or eighty-four wine, gallons, being one third of a tun.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tertiary</ent><br/
<hw>Ter"ti*a*ry</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>tertiarius</ets> containing a third part, fr. <ets>tertius</ets> third: cf. F. <ets>tertiaire</ets>. See <er>Tierce</er>.]</ety> <sn>1.</sn> <def>Being of the third formation, order, or rank; third; <as>as, a <ex>tertiary</ex> use of a word</as>.</def>  <rj><au>Trench.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Chem.)</fld> <def>Possessing some quality in the third degree; having been subjected to the substitution of three atoms or radicals; <as>as, a <ex>tertiary</ex> alcohol, amine, or salt</as>.  Cf. <er>Primary</er>, and <er>Secondary</er>.</def>
<-- specifically, an organic compound in which the carbon atom attached to the eponymic functional group has three carbon atoms attached to it; <as>as, <ex>tertiary</ex> butyl alcohol, <chform>(CH3)3C.OH</chform></as>. --><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <fld>(Geol.)</fld> <def>Later than, or subsequent to, the Secondary.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <fld>(Zool.)</fld> <def>Growing on the innermost joint of a bird's wing; tertial; -- said of quills.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Tertiary age</b></col>. <fld>(Geol.)</fld> <cd>See under <er>Age</er>, 8.</cd> -- <col><b>Tertiary color</b></col>, <cd>a color produced by the mixture of two secondaries.</cd> <ldquo/The so-called <xex>tertiary colors</xex> are <it>citrine</it>, <it>russet</it>, and <it>olive.</it><rdquo/ <au>Fairholt.</au> -- <col><b>Tertiary period</b></col>. <fld>(Geol.)</fld> <sd>(a)</sd> <cd>The first period of the age of mammals, or of the Cenozoic era.</cd> <sd>(b)</sd> <cd>The rock formation of that period; -- called also <altname>Tertiary formation</altname>. See the <xex>Chart</xex> of <er>Geology</er>.</cd> -- <col><b>Tertiary syphilis</b></col> <fld>(Med.)</fld>, <cd>the third and last stage of syphilis, in which it invades the bones and internal organs.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tertiary</ent><br/
<hw>Ter"ti*a*ry</hw>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Tertiaries</plw> <pr>(<?/)</pr>.</plu> <sn>1.</sn> <fld>(R. C. Ch.)</fld> <def>A member of the Third Order in any monastic system; <as>as, the Franciscan <ex>tertiaries</ex>; the Dominican <ex>tertiaries</ex>; the Carmelite <ex>tertiaries</ex></as>. See <cref>Third Order</cref>, under <er>Third</er>.</def>  <rj><au>Addis & Arnold.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Geol.)</fld> <def>The Tertiary era, period, or formation.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <fld>(Zool.)</fld> <def>One of the quill feathers which are borne upon the basal joint of the wing of a bird. See <xex>Illust.</xex> of <er>Bird</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tertiate</ent><br/
<hw>Ter"ti*ate</hw> <pr>(?)</pr>, <pos>v. t.</pos> <ety>[L. <ets>tertiatus</ets>, p. p. of <ets>tertiare</ets> to do for the third time, fr. <ets>tertius</ets> the third.]</ety> <sn>1.</sn> <def>To do or perform for the third time.</def> <mark>[Obs. & R.]</mark>  <rj><au>Johnson.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Gun.)</fld> <def>To examine, as the thickness of the metal at the muzzle of a gun; or, in general, to examine the thickness of, as ordnance, in order to ascertain its strength.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tertium quid</ent><br/
||<hw>Ter"ti*um quid</hw> <pr>(?)</pr>. <ety>[L.]</ety> <def>A third somewhat; something mediating, or regarded as being, between two diverse or incompatible substances, natures, or positions.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Terutero</ent><br/
||<hw>Ter`u*ter"o</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Probably so named from its city.]</ety> <fld>(Zool.)</fld> <def>The South American lapwing (<spn>Vanellus Cayennensis</spn>). Its wings are furnished with short spurs. Called also <altname>Cayenne lapwing</altname>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Terza rima</ent><br/
||<hw>Ter"za ri"ma</hw> <pr>(?)</pr>. <ety>[It., a third or triple rhyme.]</ety> <def>A peculiar and complicated system of versification, borrowed by the early Italian poets from the Troubadours.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Terzetto</ent><br/
||<hw>Ter*zet"to</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[It., dim. of <ets>terzo</ets> the third, L. <ets>tertius</ets>. See <er>Tierce</er>.]</ety> <fld>(Mus.)</fld> <def>A composition in three voice parts; a vocal (rarely an instrumental) trio.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tesla transformer</ent><br/
<ent>Tesla coil</ent><br/
<mhw>{ <hw>Tes"la coil</hw>, <hw>Tes"la trans*form"er</hw> }</mhw> <pr>(?)</pr>. <ety>[After N. <etsep>Tesla</etsep>, American electrician.]</ety> <fld>(Elec.)</fld> <def>A transformer without iron, for high frequency alternating or oscillating currents; an oscillation transformer.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Tesselar</ent><br/
<hw>Tes"sel*ar</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>tessella</ets> a small square piece, a little cube, dim. of <ets>tessera</ets> a square piece of stone, wood, etc., a die.]</ety> <def>Formed of tesserae, as a mosaic.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tessellata</ent><br/
||<hw>Tes`sel*la"ta</hw> <pr>(?)</pr>, <pos>n. pl.</pos> <ety>[NL. See <er>Tessellate</er>.]</ety> <fld>(Zool.)</fld> <def>A division of Crinoidea including numerous fossil species in which the body is covered with tessellated plates.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tessellate</ent><br/
<hw>Tes"sel*late</hw> <pr>(?)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Tessellated</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Tessellating</conjf>.]</vmorph> <ety>[L. <ets>tessellatus</ets> tessellated. See <er>Tessellar</er>.]</ety> <def>To form into squares or checkers; to lay with checkered work.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The floors are sometimes of wood, <qex>tessellated</qex> after the fashion of France.</q> <rj><qau>Macaulay.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tessellate</ent><br/
<hw>Tes"sel*late</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>tesselatus</ets>.]</ety> <def>Tessellated.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tessellated</ent><br/
<hw>Tes"sel*la`ted</hw> <pr>(?)</pr>, <pos>a.</pos> <sn>1.</sn> <def>Formed of little squares, as mosaic work; checkered; <as>as, a <ex>tessellated</ex> pavement</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Bot. & Zool.)</fld> <def>Marked like a checkerboard; <as>as, a <ex>tessellated</ex> leaf</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tessellation</ent><br/
<hw>Tes`sel*la"tion</hw> <pr>(?)</pr>, <pos>n.</pos> <def>The act of tessellating; also, the mosaic work so formed.</def>  <rj><au>J. Forsyth.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tessera</ent><br/
||<hw>Tes"se*ra</hw> <pr>(?)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Tesserae</plw> <pr>(#)</pr>.</plu> <ety>[L., a square piece, a die. See <er>Tessellar</er>.]</ety> <def>A small piece of marble, glass, earthenware, or the like, having a square, or nearly square, face, used by the ancients for mosaic, as for making pavements, for ornamenting walls, and like purposes; also, a similar piece of ivory, bone, wood, etc., used as a ticket of admission to theaters, or as a certificate for successful gladiators, and as a token for various other purposes.</def>  <rj><au>Fairholt.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tesseraic</ent><br/
<hw>Tes`se*ra"ic</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Diversified by squares; done in mosaic; tessellated.</def> <mark>[Obs.]</mark>  <rj><au>Sir R. Atkyns (1712).</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tesseral</ent><br/
<hw>Tes"se*ral</hw> <pr>(?)</pr>, <pos>a.</pos> <sn>1.</sn> <def>Of, pertaining to, or containing, tesserae.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Crystallog.)</fld> <def>Isometric.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tessular</ent><br/
<hw>Tes"su*lar</hw> <pr>(?)</pr>, <pos>a.</pos> <fld>(Crystallog.)</fld> <def>Tesseral.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Test</ent><br/
<hw>Test</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[OE. <ets>test</ets> test, or cupel, potsherd, F. <ets>t<ecir/t</ets>, from L. <ets>testum</ets> an earthen vessel; akin to <ets>testa</ets> a piece of burned clay, an earthen pot, a potsherd, perhaps for <ets>tersta</ets>, and akin to <ets>torrere</ets> to patch, <ets>terra</ets> earth (cf. <er>Thirst</er>, and <er>Terrace</er>), but cf. Zend <ets>tasta</ets> cup.  Cf. <er>Test</er> a shell, <er>Testaceous</er>, <er>Tester</er> a covering, a coin, <er>Testy</er>, <er>T<ecir/te-<agrave/-t<ecir/te</er>.]</ety> <sn>1.</sn> <fld>(Metal.)</fld> <def>A cupel or cupelling hearth in which precious metals are melted for trial and refinement.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Our ingots, <qex>tests</qex>, and many mo.</q> <rj><qau>Chaucer.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Examination or trial by the cupel; hence, any critical examination or decisive trial; <as>as, to put a man's assertions to a <ex>test</ex></as>.</def> <ldquo/Bring me to the <xex>test</xex>.<rdquo/  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>Means of trial; <as>as, absence is a <ex>test</ex> of love</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Each <qex>test</qex> every light her muse will bear.</q> <rj><qau>Dryden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>That with which anything is compared for proof of its genuineness; a touchstone; a standard.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Life, force, and beauty must to all impart,<br/
At once the source, and end, and <qex>test</qex> of art.</q> <rj><qau>Pope.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <def>Discriminative characteristic; standard of judgment; ground of admission or exclusion.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Our <qex>test</qex> excludes your tribe from benefit.</q> <rj><qau>Dryden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>6.</sn> <def>Judgment; distinction; discrimination.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Who would excel, when few can make a <qex>test</qex><br/
Betwixt indifferent writing and the best?</q> <rj><qau>Dryden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>7.</sn> <fld>(Chem.)</fld> <def>A reaction employed to recognize or distinguish any particular substance or constituent of a compound, as the production of some characteristic precipitate; also, the reagent employed to produce such reaction; thus, the ordinary <xex>test</xex> for sulphuric acid is the production of a white insoluble precipitate of barium sulphate by means of some soluble barium salt.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>8.</sn> <def>A set of questions to be answered or problems to be solved, used as a means to measure a person's knowledge, aptitude, skill, intelligence, etc.; in school settings, synonymous with <altname>examination</altname> or <altname>exam</altname>; <as>as, an intelligence <ex>test</ex></as>.  Also used attributively; <as>as a <ex>test</ex> score, <ex>test</ex> results</as>.</def><br/
[<source>PJC</source>]</p>

<p><cs><col><b>Test act</b></col> <fld>(Eng. Law)</fld>, <cd>an act of the English Parliament prescribing a form of oath and declaration against transubstantiation, which all officers, civil and military, were formerly obliged to take within six months after their admission to office. They were obliged also to receive the sacrament according to the usage of the Church of England.</cd> <au>Blackstone.</au> -- <col><b>Test object</b></col> <fld>(Optics)</fld>, <cd>an object which tests the power or quality of a microscope or telescope, by requiring a certain degree of excellence in the instrument to determine its existence or its peculiar texture or markings.</cd> -- <col><b>Test paper</b></col>. <sd>(a)</sd> <fld>(Chem.)</fld> <cd>Paper prepared for use in testing for certain substances by being saturated with a reagent which changes color in some specific way when acted upon by those substances; thus, litmus paper is turned red by acids, and blue by alkalies, turmeric paper is turned brown by alkalies, etc.</cd> <sd>(b)</sd> <fld>(Law)</fld> <cd>An instrument admitted as a standard or comparison of handwriting in those jurisdictions in which comparison of hands is permitted as a mode of proving handwriting.</cd> -- <col><b>Test tube</b></col>. <fld>(Chem.)</fld> <sd>(a)</sd> <cd>A simple tube of thin glass, closed at one end, for heating solutions and for performing ordinary reactions.</cd> <sd>(b)</sd> <cd>A graduated tube.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><syn><b>Syn.</b> -- Criterion; standard; experience; proof; experiment; trial.</syn> <usage> -- <er>Test</er>, <er>Trial</er>. <xex>Trial</xex> is the wider term; <xex>test</xex> is a searching and decisive <xex>trial</xex>. It is derived from the Latin <xex>testa</xex> (earthen pot), which term was early applied to the <xex>fining pot</xex>, or <xex>crucible</xex>, in which metals are melted for trial and refinement. Hence the peculiar force of the word, as indicating a trial or criterion of the most decisive kind.</usage><br/
[<source>1913 Webster</source>]</p>

<p><q>I leave him to your gracious acceptance, whose <qex>trial</qex> shall better publish his commediation.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Thy virtue, prince, has stood the <qex>test</qex> of fortune,<br/
Like purest gold, that tortured in the furnace,<br/
Comes out more bright, and brings forth all its weight.</q> <rj><qau>Addison.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Test</ent><br/
<hw>Test</hw>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Tested</conjf>; <pos>p. pr. & vb. n.</pos> <conjf>Testing</conjf>.]</vmorph> <sn>1.</sn> <fld>(Metal.)</fld> <def>To refine, as gold or silver, in a test, or cupel; to subject to cupellation.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To put to the proof; to prove the truth, genuineness, or quality of by experiment, or by some principle or standard; to try; <as>as, to <ex>test</ex> the soundness of a principle; to <ex>test</ex> the validity of an argument.</as></def><br/
[<source>1913 Webster</source>]</p>

<p><q>Experience is the surest standard by which to <qex>test</qex> the real tendency of the existing constitution.</q> <rj><qau>Washington.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <fld>(Chem.)</fld> <def>To examine or try, as by the use of some reagent; <as>as, to <ex>test</ex> a solution by litmus paper</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>To administer a test{8} to (someone) for the purpose of ascertaining a person's knowledge or skill; especially, in academic settings, to determine how well a student has learned the subject matter of a course of instruction.</def><br/
[<source>PJC</source>]</p>

<p><-- p. 1490 --><br/
[<source>1913 Webster</source>]</p>

<p><ent>Test</ent><br/
<hw>Test</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>testis</ets>.  Cf. <er>Testament</er>, <er>Testify</er>.]</ety> <def>A witness.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>Prelates and great lords of England, who were for the more surety <qex>tests</qex> of that deed.</q> <rj><qau>Ld. Berners.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Test</ent><br/
<hw>Test</hw>, <pos>v. i.</pos> <ety>[L. <ets>testari</ets>. See <er>Testament</er>.]</ety> <def>To make a testament, or will.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Testa</ent><br/
<ent>Test</ent><br/
<mhw>{ <hw>Test</hw> <pr>(?)</pr>, ||<hw>Tes"ta</hw> <pr>(?)</pr>, }</mhw> <pos>n.</pos>; <plu><it>pl.</it> E. <plw>Tests</plw> <pr>(#)</pr>, L. <plw>Testae</plw> <pr>(#)</pr>.</plu> <ety>[L. <ets>testa</ets> a piece of burned clay, a broken piece of earthenware, a shell. See <er>Test</er> a cupel.]</ety> <sn>1.</sn> <fld>(Zool.)</fld> <def>The external hard or firm covering of many invertebrate animals.</def><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ The <xex>test</xex> of crustaceans and insects is composed largely of chitin; in mollusks it is composed chiefly of calcium carbonate, and is called the <xex>shell</xex>.</note><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Bot.)</fld> <def>The outer integument of a seed; the episperm, or spermoderm.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Testable</ent><br/
<hw>Test"a*ble</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[See <er>Testament</er>.]</ety> <sn>1.</sn> <def>Capable of being tested or proved.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Capable of being devised, or given by will.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Testacea</ent><br/
||<hw>Tes*ta"ce*a</hw> <pr>(?)</pr>, <pos>n. pl.</pos> <ety>[NL., fr. L. <ets>testaceum</ets> a shelled anumal. See <er>Testaceous</er>.]</ety> <fld>(Zool.)</fld> <def>Invertebrate animals covered with shells, especially mollusks; shellfish.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Testacean</ent><br/
<hw>Tes*ta"cean</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Zool.)</fld> <def>Onr of the Testacea.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Testaceography</ent><br/
<hw>Tes*ta`ce*og"ra*phy</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[<ets>Testacea</ets> + <ets>-graphy</ets>: cf. F. <ets>testac<eacute/ographie</ets>.]</ety> <def>The science which treats of testaceans, or shellfish; the description of shellfish.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Testaceology</ent><br/
<hw>Tes*ta`ce*ol"o*gy</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[<ets>Testacea</ets> + <ets>-logy</ets>: cf. F. <ets>testac<eacute/ologie</ets>.]</ety> <def>The science of testaceous mollusks; conchology.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Testaceous</ent><br/
<hw>Tes*ta"ceous</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>testaceus</ets>, fr. <ets>testa</ets> a shell. See <er>Testa</er>.]</ety> <sn>1.</sn> <def>Of or pertaining to shells; consisted of a hard shell, or having a hard shell.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Bot. & Zool.)</fld> <def>Having a dull red brick color or a brownish yellow color.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Testaceous animals</b></col> <fld>(Zool.)</fld>, <cd>animals having a firm, calcareous shell, as oysters and clams, thus distinguished from <contr>crustaceous animals</contr>, whose shells are more thin and soft, and consist of several joints, or articulations, as lobsters and crabs.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Testacy</ent><br/
<hw>Tes"ta*cy</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[See <er>Testate</er>.]</ety> <fld>(Law)</fld> <def>The state or circumstance of being testate, or of leaving a valid will, or testament, at death.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Testament</ent><br/
<hw>Tes"ta*ment</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F., fr. L. <ets>testamentum</ets>, fr. <ets>testari</ets> to be a witness, to make one's last will, akin to <ets>testis</ets> a witness.  Cf. <er>Intestate</er>, <er>Testify</er>.]</ety> <sn>1.</sn> <fld>(Law)</fld> <def>A solemn, authentic instrument in writing, by which a person declares his will as to disposal of his estate and effects after his death.</def><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ This is otherwise called a <xex>will</xex>, and sometimes a <xex>last will and testament</xex>. A <xex>testament</xex>, to be valid, must be made by a person of sound mind; and it must be executed and published in due form of law. A man, in certain cases, may make a valid will by word of mouth only. See <cref>Nuncupative will</cref>, under <er>Nuncupative</er>.</note><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>One of the two distinct revelations of God's purposes toward man; a covenant; also, one of the two general divisions of the canonical books of the sacred Scriptures, in which the covenants are respectively revealed; <as>as, the Old <ex>Testament</ex>; the New <ex>Testament</ex>; -- often limited, in colloquial language, to the latter.</as></def><br/
[<source>1913 Webster</source>]</p>

<p><q>He is the mediator of the new <qex>testament</qex> . . . for the redemption of the transgressions that were under the first <qex>testament</qex>.</q> <rj><qau>Heb. ix. 15.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Holographic testament</b></col>, <cd>a testament written wholly by the testator himself.</cd> <au>Bouvier.</au></cs>
<-- also holographic will.  "Written" means, in handwriting. --><br/
[<source>1913 Webster</source>]</p>

<p><ent>Testamental</ent><br/
<hw>Tes`ta*men"tal</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>testamentalis</ets>.]</ety> <def>Of or pertaining to a testament; testamentary.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Thy <qex>testamental</qex> cup I take,<br/
And thus remember thee.</q> <rj><qau>J. Montgomery.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Testamentary</ent><br/
<hw>Tes`ta*men"ta*ry</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>testamentarius</ets>: cf. F. <ets>testamentaire</ets>.]</ety> <sn>1.</sn> <def>Of or pertaining to a will, or testament; <as>as, letters <ex>testamentary</ex></as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Bequeathed by will; given by testament.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>How many <qex>testamentary</qex> charities have been defeated by the negligence or fraud of executors!</q> <rj><qau>Atterbury.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>Done, appointed by, or founded on, a testament, or will; <as>as, a <ex>testamentary</ex> guardian of a minor, who may be appointed by the will of a father to act in that capacity until the child becomes of age</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Testamentation</ent><br/
<hw>Tes`ta*men*ta"tion</hw> <pr>(?)</pr>, <pos>n.</pos> <def>The act or power of giving by testament, or will.</def> <mark>[R.]</mark>  <rj><au>Burke.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Testamentize</ent><br/
<hw>Tes"ta*men*tize</hw> <pr>(?)</pr>, <pos>v. i.</pos> <def>To make a will.</def> <mark>[Obs.]</mark>  <rj><au>Fuller.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Testamur</ent><br/
||<hw>Tes*ta"mur</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L., we testify, fr. <ets>testari</ets> to testify.]</ety> <fld>(Eng. Universities)</fld> <def>A certificate of merit or proficiency; -- so called from the Latin words, <xex>Ita testamur</xex>, with which it commences.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Testate</ent><br/
<hw>Tes"tate</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>testatus</ets>, p. p. of <ets>testari</ets>. See <er>Testament</er>.]</ety> <fld>(Law)</fld> <def>Having made and left a will; <as>as, a person is said to die <ex>testate</ex></as>.</def>  <rj><au>Ayliffe.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Testate</ent><br/
<hw>Tes"tate</hw>, <pos>n.</pos> <fld>(Law)</fld> <def>One who leaves a valid will at death; a testate person.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Testation</ent><br/
<hw>Tes*ta"tion</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>testatio</ets>.]</ety> <def>A witnessing or witness.</def> <mark>[Obs.]</mark>  <rj><au>Bp. Hall.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Testator</ent><br/
<hw>Tes*ta"tor</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L.: cf. F. <ets>testateur</ets>.]</ety> <fld>(Law)</fld> <def>A man who makes and leaves a will, or testament, at death.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Testatrix</ent><br/
<hw>Tes*ta"trix</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L.]</ety> <fld>(Law)</fld> <def>A woman who makes and leaves a will at death; a female testator.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Teste</ent><br/
<hw>Tes"te</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[So called fr. L. <ets>teste</ets>, abl. of <ets>testis</ets> a witness, because this was formerly the initial word in the clause.]</ety> <fld>(Law)</fld> <sd>(a)</sd> <def>A witness.</def> <sd>(b)</sd> <def>The witnessing or concluding clause, duty attached; -- said of a writ, deed, or the like.</def>  <rj><au>Burrill.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tester</ent><br/
<hw>Tes"ter</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[OE. <ets>testere</ets> a headpiece, helmet, OF. <ets>testiere</ets>, F. <ets>t<ecir/ti<egrave/re</ets> a head covering, fr. OF. <ets>teste</ets> the head, F. <ets>t<ecir/te</ets>, fr. L. <ets>testa</ets> an earthen pot, the skull. See <er>Test</er> a cupel, and cf. <er>Testi<egrave/re</er>.]</ety> <sn>1.</sn> <def>A headpiece; a helmet.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>The shields bright, <qex>testers</qex>, and trappures.</q> <rj><qau>Chaucer.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A flat canopy, as over a pulpit or tomb.</def>  <rj><au>Oxf. Gross.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>A canopy over a bed, supported by the bedposts.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>No <qex>testers</qex> to the bed, and the saddles and portmanteaus heaped on me to keep off the cold.</q> <rj><qau>Walpole.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tester</ent><br/
<hw>Tes"ter</hw>, <pos>n.</pos> <ety>[For <ets>testern</ets>, <ets>teston</ets>, fr. F. <ets>teston</ets>, fr. OF. <ets>teste</ets> the head, the head of the king being impressed upon the coin. See <er>Tester</er> a covering, and cf. <er>Testone</er>, <er>Testoon</er>.]</ety> <def>An old French silver coin, originally of the value of about eighteen pence, subsequently reduced to ninepence, and later to sixpence, sterling. Hence, in modern English slang, a sixpence; -- often contracted to <altname>tizzy</altname>. Called also <altname>teston</altname>.</def>  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Testern</ent><br/
<hw>Tes"tern</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A sixpence; a tester.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Testern</ent><br/
<hw>Tes"tern</hw>, <pos>v. t.</pos> <def>To present with a tester.</def> <mark>[Obs.]</mark>  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Testes</ent><br/
||<hw>Tes"tes</hw> <pr>(?)</pr>, <pos>n.</pos>, <def><pos>pl.</pos> of <er>Teste</er>, or of <er>Testis</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Testicardines</ent><br/
||<hw>Tes`ti*car"di*nes</hw> <pr>(?)</pr>, <pos>n. pl.</pos> <ety>[NL. See <er>Test</er> a shell, and <er>Cardo</er>.]</ety> <fld>(Zool.)</fld> <def>A division of brachiopods including those which have a calcareous shell furnished with a hinge and hinge teeth. Terebratula and Spirifer are examples.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Testicle</ent><br/
<hw>Tes"ti*cle</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>testiculus</ets>, dim. of <ets>testis</ets> a testicle, probably the same word as <ets>testis</ets> a witness, as being a witness to manhood.  Cf. <er>Test</er> a witness.]</ety> <fld>(Anat.)</fld> <def>One of the essential male genital glands which secrete the semen.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Testicond</ent><br/
<hw>Tes"ti*cond</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>testis</ets> testis + <ets>condere</ets> to hide.]</ety> <fld>(Zool.)</fld> <def>Having the testicles naturally concealed, as in the case of the cetaceans.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Testicular</ent><br/
<hw>Tes*tic"u*lar</hw> <pr>(?)</pr>, <pos>a.</pos> <fld>(Anat.)</fld> <def>Of or pertaining to the testicle.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Testiculate</ent><br/
<hw>Tes*tic"u*late</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[NL. <ets>testiculatus</ets>.]</ety> <fld>(Bot.)</fld> <sd>(a)</sd> <def>Shaped like a testicle, ovate and solid.</def> <sd>(b)</sd> <def>Having two tubers resembling testicles in form, as some species of orchis.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Testiere</ent><br/
||<hw>Tes`ti*<egrave/re"</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[OF. <ets>testiere</ets>. See <er>Tester</er> a headpiece.]</ety> <def>A piece of plate armor for the head of a war horse; a tester.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Testif</ent><br/
<hw>Tes"tif</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[See <er>Testy</er>.]</ety> <def>Testy; headstrong; obstinate.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q><qex>Testif</qex> they were and lusty for to play.</q> <rj><qau>Chaucer.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Testification</ent><br/
<hw>Tes`ti*fi*ca"tion</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>testificatio</ets>: cf. OF. <ets>testification</ets>. See <er>Testify</er>.]</ety> <def>The act of testifying, or giving testimony or evidence; <as>as, a direct <ex>testification</ex> of our homage to God</as>.</def>  <rj><au>South.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Testificator</ent><br/
<hw>Tes"ti*fi*ca`tor</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[NL.]</ety> <def>A testifier.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Testifier</ent><br/
<hw>Tes"ti*fi`er</hw> <pr>(?)</pr>, <pos>n.</pos> <def>One who testifies; one who gives testimony, or bears witness to prove anything; a witness.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Testify</ent><br/
<hw>Tes"ti*fy</hw> <pr>(?)</pr>, <pos>v. i.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Testified</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Testifying</conjf> <pr>(?)</pr>.]</vmorph> <ety>[OF. <ets>testifier</ets>, L. <ets>testificari</ets>; <ets>testis</ets> a witness + <ets>-ficare</ets> (in comp.) to make. See <er>-fy</er>, and cf. <er>Attest</er>, <er>Contest</er>, <er>Detest</er>, <er>Protest</er>, <er>Testament</er>.]</ety> <sn>1.</sn> <def>To make a solemn declaration, verbal or written, to establish some fact; to give testimony for the purpose of communicating to others a knowledge of something not known to them.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Jesus . . . needed not that any should <qex>testify</qex> of man, for he knew what was in man.</q> <rj><qau>John ii. 25.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Law)</fld> <def>To make a solemn declaration under oath or affirmation, for the purpose of establishing, or making proof of, some fact to a court; to give testimony in a cause depending before a tribunal.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>One witness shall not <qex>testify</qex> against any person to cause him to die.</q> <rj><qau>Num. xxxv. 30.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>To declare a charge; to protest; to give information; to bear witness; -- with <xex>against</xex>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>O Israel, . . . I will <qex>testify</qex> against thee.</q> <rj><qau>Ps. l. 7.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>I <qex>testified</qex> against them in the day wherein they sold victuals.</q> <rj><qau>Neh. xiii. 15.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Testify</ent><br/
<hw>Tes"ti*fy</hw>, <pos>v. t.</pos> <sn>1.</sn> <def>To bear witness to; to support the truth of by testimony; to affirm or declare solemny.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>We speak that we do know, and <qex>testify</qex> that we have seen; and ye receive not our witness.</q> <rj><qau>John iii. 11.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Law)</fld> <def>To affirm or declare under oath or affirmation before a tribunal, in order to prove some fact.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Testify</ent><br/
<hw>Tes"ti*fy</hw>, <pos>adv.</pos> <def>In a testy manner; fretfully; peevishly; with petulance.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Testimonial</ent><br/
<hw>Tes`ti*mo"ni*al</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Cf. OF. <ets>testimoniale</ets>, LL. <ets>testimonialis</ets>, L. <ets>testimoniales</ets> (sc. <ets>litterae</ets>). See <er>Testimonial</er>, <pos>a.</pos>]</ety> <sn>1.</sn> <def>A writing or certificate which bears testimony in favor of one's character, good conduct, ability, etc., or of the value of a thing.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Something, as money or plate, presented to a preson as a token of respect, or of obligation for services rendered.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Testimonial</ent><br/
<hw>Tes`ti*mo"ni*al</hw>, <pos>a.</pos> <ety>[L. <ets>testimonialis</ets>: cf. F. <ets>testimonial</ets>.]</ety> <def>Relating to, or containing, testimony.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Testimony</ent><br/
<hw>Tes"ti*mo*ny</hw> <pr>(?)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Testimonies</plw> <pr>(#)</pr>.</plu> <ety>[L. <ets>testimonium</ets>, from <ets>testis</ets> a witness: cf. OF. <ets>testimoine</ets>, <ets>testemoine</ets>, <ets>testimonie</ets>. See <er>Testify</er>.]</ety> <sn>1.</sn> <def>A solemn declaration or affirmation made for the purpose of establishing or proving some fact.</def><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ Such declaration, in judicial proceedings, may be verbal or written, but must be under oath or affirmation.</note><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Affirmation; declaration; <as>as, these doctrines are supported by the uniform <ex>testimony</ex> of the fathers; the belief of past facts must depend on the evidence of human <ex>testimony</ex>, or the <ex>testimony</ex> of historians</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>Open attestation; profession.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>[Thou] for the <qex>testimony</qex> of truth, hast borne<br/
Universal reproach.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>Witness; evidence; proof of some fact.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>When ye depart thence, shake off the dust under your feet for a <qex>testimony</qex> against them.</q> <rj><qau>Mark vi. 11.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <fld>(Jewish Antiq.)</fld> <def>The two tables of the law.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Thou shalt put into the ark the <qex>testimony</qex> which I shall give thee.</q> <rj><qau>Ex. xxv. 16.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>6.</sn> <def>Hence, the whole divine revelation; the sacre<?/ Scriptures.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The <qex>testimony</qex> of the Lord is sure, making wise the simple.</q> <rj><qau>Ps. xix. 7.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><syn><b>Syn.</b> -- Proof; evidence; attestation; witness; affirmation; confirmation; averment.</syn> <usage> -- <er>Testimony</er>, <er>Proof</er>, <er>Evidence</er>. <xex>Proof</xex> is the most familiar, and is used more frequently (though not exclusively) of facts and things which occur in the ordinary concerns of life. <xex>Evidence</xex> is a word of more dignity, and is more generally applied to that which is moral or intellectual; <as>as, the <ex>evidences</ex> of Christianity, etc.</as> <xex>Testimony</xex> is what is deposed to by a witness on oath or affirmation. When used figuratively or in a wider sense, the word <xex>testimony</xex> has still a reference to some living agent as its author, as when we speak of the <xex>testimony</xex> of conscience, or of doing a thing in <xex>testimony</xex> of our affection, etc. <xex>Testimony</xex> refers rather to the thing declared, <xex>evidence</xex> to its value or effect. <ldquo/To conform our language more to common use, we ought to divide arguments into demonstrations, <xex>proofs</xex>, and probabilities; ba <xex>proofs</xex>, meaning such arguments from experience as leave no room for doubt or opposition.<rdquo/ <au>Hume.</au> <ldquo/The <xex>evidence</xex> of sense is the first and highest kind of <xex>evidence</xex> of which human nature is capable.<rdquo/ <au>Bp. Wilkins.</au> <ldquo/The <xex>proof</xex> of everything must be by the <xex>testimony</xex> of such as the parties produce.<rdquo/ <au>Spenser.</au></usage><br/
[<source>1913 Webster</source>]</p>

<p><ent>Testimony</ent><br/
<hw>Tes"ti*mo*ny</hw> <pr>(?)</pr>, <pos>v. t.</pos> <def>To witness; to attest; to prove by testimony.</def> <mark>[Obs.]</mark>  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Testiness</ent><br/
<hw>Tes"ti*ness</hw> <pr>(?)</pr>, <pos>n.</pos> <def>The quality or state of being testy; fretfulness; petulance.</def><br/
[<source>1913 Webster</source>]</p>

<p><q><qex>Testiness</qex> is a disposition or aptness to be angry.</q> <rj><qau>Locke.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Testing</ent><br/
<hw>Test"ing</hw> <pr>(?)</pr>, <pos>n.</pos> <sn>1.</sn> <def>The act of testing or proving; trial; proof.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Metal.)</fld> <def>The operation of refining gold or silver in a <xex>test</xex>, or cupel; cupellation.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Testing machine</b></col> <fld>(Engin.)</fld>, <cd>a machine used in the determination of the strength of materials, as iron, stone, etc., and their behavior under strains of various kinds, as elongation, bending, crushing, etc.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Testis</ent><br/
||<hw>Tes"tis</hw> <pr>(?)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Testes</plw> <pr>(#)</pr>.</plu> <ety>[L.]</ety> <fld>(Anat.)</fld> <def>A testicle.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Test-market</ent><br/
<hw>Test"-mark`et</hw> <pr>(?)</pr>, <pos>v. t.</pos> <def>to market (a product) in a limited area for a period of time to determine the probable demand.</def><br/
[<source>PJC</source>]</p>

<p><ent>Teston</ent><br/
<hw>Tes"ton</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A tester; a sixpence.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Testone</ent><br/
<hw>Tes*tone"</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Cf. Pg. <ets>test<atil/o</ets>, <ets>tost<atil/o</ets>. See <er>Testoon</er>.]</ety> <def>A silver coin of Portugal, worth about sixpence sterling, or about eleven cents.</def>  <rj><au>Homans.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Testoon</ent><br/
<hw>Tes*toon"</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[It. <ets>testone</ets>. See <er>Tester</er> a coin.]</ety> <def>An Italian silver coin. The testoon of Rome is worth 1s. 3d. sterling, or about thirty cents.</def>  <rj><au>Homans.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Testosterone</ent><br/
<hw>Tes*tos"ter*one</hw> <pr>(t<ecr/s*t<ocr/s"t<etil/r*<omac/n)</pr>, <pos>n.</pos> <ety>[<ets>test</ets>es + o + <ets>sterone</ets> a steroid ketone.]</ety> <def>a steroid hormone (<chform>C19H28O2</chform>) produced mostly in the testes, which is responsible for producing the secondary sexual characteristics of males.</def><br/
[<source>PJC</source>]</p>

<p><ent>Testudinal</ent><br/
<hw>Tes*tu"di*nal</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[See <er>Testudo</er>.]</ety> <fld>(Zool.)</fld> <def>Of, pertaining to, or resembling, a tortoise.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Testudinarious</ent><br/
<hw>Tes*tu`di*na"ri*ous</hw> <pr>(?)</pr>, <pos>a.</pos> <fld>(Zool.)</fld> <def>Of or pertaining to the shell of a tortoise; resembling a tortoise shell; having the color or markings of a tortoise shell.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Testudinata</ent><br/
||<hw>Tes*tu`di*na"ta</hw> <pr>(?)</pr>, <pos>n. pl.</pos> <ety>[Nl. See <er>Testudo</er>.]</ety> <fld>(Zool.)</fld> <def>An order of reptiles which includes the turtles and tortoises. The body is covered by a shell consisting of an upper or dorsal shell, called the <part>carapace</part>, and a lower or ventral shell, called the <part>plastron</part>, each of which consists of several plates.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Testudinated</ent><br/
<ent>Testudinate</ent><br/
<mhw>{ <hw>Tes*tu"di*nate</hw> <pr>(?)</pr>, <hw>Tes*tu"di*na`ted</hw> <pr>(?)</pr>, }</mhw> <pos>a.</pos> <ety>[L. <ets>testudinatus</ets>, fr. <ets>testudo</ets>, <ets>-inis</ets>, a tortoise, an arch or vault.]</ety> <def>Resembling a tortoise shell in appearance or structure; roofed; arched; vaulted.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Testudineous</ent><br/
<hw>Tes`tu*din"e*ous</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>testudineus</ets>.]</ety> <def>Resembling the shell of a tortoise.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Testudo</ent><br/
||<hw>Tes*tu"do</hw> <pr>(?)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Testudines</plw> <pr>(#)</pr>.</plu> <ety>[L., from <ets>testa</ets> the shell of shellfish, or of testaceous animals.]</ety> <sn>1.</sn> <fld>(Zool.)</fld> <def>A genus of tortoises which formerly included a large number of diverse forms, but is now restricted to certain terrestrial species, such as the European land tortoise (<spn>Testudo Graeca</spn>) and the gopher of the Southern United States.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Rom. Antiq.)</fld> <def>A cover or screen which a body of troops formed with their shields or targets, by holding them over their heads when standing close to each other. This cover resembled the back of a tortoise, and served to shelter the men from darts, stones, and other missiles. A similar defense was sometimes formed of boards, and moved on wheels.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <fld>(Mus.)</fld> <def>A kind of musical instrument. a species of lyre; -- so called in allusion to the lyre of Mercury, fabled to have been made of the shell of a tortoise.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Testy</ent><br/
<hw>Tes"ty</hw> <pr>(?)</pr>, <pos>a.</pos> <amorph>[<pos>Compar.</pos> <adjf>Testier</adjf> <pr>(?)</pr>; <pos>superl.</pos> <adjf>Testiest</adjf>.]</amorph> <ety>[OF. <ets>testu</ets> obstinate, headstrong, F. <ets>t<ecir/tu</ets>, fr. OF. <ets>teste</ets> the head, F. <ets>t<ecir/te</ets>. See <er>Test</er> a cupel.]</ety> <def>Fretful; peevish; petulant; easily irritated.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Must I observe you? must I stand and crouch<br/
Under your <qex>testy</qex> humor?</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>I was displeased with myself; I was <qex>testy</qex>.</q> <rj><qau>Latimer.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tetanic</ent><br/
<hw>Te*tan"ic</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[Cf. L. <ets>tetanicus</ets> suffering from tetanus, Gr. <?/, F. <ets>t<eacute/tanique</ets>.]</ety> <sn>1.</sn> <fld>(Physiol.)</fld> <def>Of or pertaining to tetanus; having the character of tetanus; <as>as, a <ex>tetanic</ex> state; <ex>tetanic</ex> contraction.</as></def><br/
[<source>1913 Webster</source>]</p>

<p><q>This condition of muscle, this fusion of a number of simple spasms into an apparently smooth, continuous effort, is known as tetanus, or <qex>tetanic</qex> contraction.</q> <rj><qau>Foster.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Physiol. & Med.)</fld> <def>Producing, or tending to produce, tetanus, or tonic contraction of the muscles; <as>as, a <ex>tetanic</ex> remedy</as>. See <er>Tetanic</er>, <pos>n.</pos></def><br/
[<source>1913 Webster</source>]</p>

<p><-- p. 1491 --><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tetanic</ent><br/
<hw>Te*tan"ic</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Physiol. & Med.)</fld> <def>A substance (notably nux vomica, strychnine, and brucine) which, either as a remedy or a poison, acts primarily on the spinal cord, and which, when taken in comparatively large quantity, produces tetanic spasms or convulsions.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tetanin</ent><br/
<hw>Tet"a*nin</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Physiol. Chem.)</fld> <def>A poisonous base (ptomaine) formed in meat broth through the agency of a peculiar microbe from the wound of a person who has died of tetanus; -- so called because it produces tetanus as one of its prominent effects.</def>
<-- ?? not in Merck INdex -- tetanospasmin?  The neurotoxin of Clostridium tetani. --><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tetanization</ent><br/
<hw>Tet`a*ni*za"tion</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Physiol.)</fld> <def>The production or condition of tetanus.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tetanize</ent><br/
<hw>Tet"a*nize</hw> <pr>(?)</pr>, <pos>v. t.</pos> <fld>(Physiol.)</fld> <def>To throw, as a muscle, into a state of permanent contraction; to cause tetanus in. See <er>Tetanus</er>, <pos>n.</pos>, 2.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tetanoid</ent><br/
<hw>Tet"a*noid</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[<ets>Tetanus</ets> + <ets>-oid</ets>.]</ety> <fld>(Med. & Physiol.)</fld> <def>Resembling tetanus.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tetanomotor</ent><br/
<hw>Tet`a*no*mo"tor</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Physiol.)</fld> <def>An instrument from tetanizing a muscle by irritating its nerve by successive mechanical shocks.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tetanus</ent><br/
||<hw>Tet"a*nus</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L., fr. Gr. <?/, fr. <?/ stretched, <?/ to stretch.]</ety> <sn>1.</sn> <fld>(Med.)</fld> <def>A painful and usually fatal disease, resulting generally from a wound, and having as its principal symptom persistent spasm of the voluntary muscles. When the muscles of the lower jaw are affected, it is called <altname>locked-jaw</altname>, or <altname>lickjaw</altname>, and it takes various names from the various incurvations of the body resulting from the spasm.</def><-- caused by the anaerobic bacterium Clostridium tetani. --><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Physiol.)</fld> <def>That condition of a muscle in which it is in a state of continued vibratory contraction, as when stimulated by a series of induction shocks.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tetany</ent><br/
<hw>Tet"a*ny</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Med.)</fld> <def>A morbid condition resembling tetanus, but distinguished from it by being less severe and having intermittent spasms.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tetard</ent><br/
<hw>Te*tard"</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Zool.)</fld> <def>A gobioid fish (<spn>Eleotris gyrinus</spn>) of the Southern United States; -- called also <altname>sleeper</altname>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tetartohedral</ent><br/
<hw>Te*tar`to*he"dral</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[Gr. <?/ fourth + <?/ base.]</ety> <fld>(Crystallog.)</fld> <def>Having one fourth the number of planes which are requisite to complete symmetry.</def> -- <wordforms><wf>Te*tar`to*he"dral*ly</wf>, <pos>adv.</pos></wordforms><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tetartohedrism</ent><br/
<hw>Te*tar`to*he"drism</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Crystallog.)</fld> <def>The property of being tetartohedral.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tetaug</ent><br/
<hw>Te*taug"</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Zool.)</fld> <def>See <er>Tautog</er>.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tetchiness</ent><br/
<hw>Tetch"i*ness</hw>, <pos>n.</pos> <def>See <er>Techiness</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tetchy</ent><br/
<hw>Tetch"y</hw> <pr>(?)</pr>, <pos>a.</pos> <def>See <er>Techy</er>.</def>  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tete</ent><br/
||<hw>T<ecir/te</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F., the head. See <er>Tester</er> a covering.]</ety> <def>A kind of wig; false hair.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tete-a-tete</ent><br/
||<hw>T<ecir/te`-<agrave/-t<ecir/te"</hw> <pr>(t<amac/t`<adot/*t<amac/t")</pr>, <pos>n.</pos> <ety>[F., head to head. See <er>Tester</er> a covering, <er>Test</er> a cupel.]</ety> <sn>1.</sn> <def>Private conversation; familiar interview or conference of two persons.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A short sofa intended to accomodate two persons.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tete-a-tete</ent><br/
||<hw>T<ecir/te`-<agrave/-t<ecir/te"</hw>, <pos>a.</pos> <def>Private; confidential; familiar.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>She avoided <qex>t<ecir/te-<agrave/-t<ecir/te</qex> walks with him.</q> <rj><qau>C. Kingsley.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tete-a-tete</ent><br/
||<hw>T<ecir/te`-<agrave/-t<ecir/te"</hw>, <pos>adv.</pos> <def>Face to face; privately or confidentially; familiarly.</def>  <rj><au>Prior.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tete-de-pont</ent><br/
||<hw>T<ecir/te`-de-pont"</hw> <pr>(t<amac/t`d<eit/*p<ocir/N")</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>T<ecir/tes-de-pont</plw> <pr>(#)</pr>.</plu> <ety>[F., head of a bridge.]</ety> <fld>(Mil.)</fld> <def>A work thrown up at the end of a bridge nearest the enemy, for covering the communications across a river; a bridgehead.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tetel</ent><br/
<hw>Te*tel"</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Zool.)</fld> <def>A large African antelope (<spn>Alcelaphus tora</spn>). It has widely divergent, strongly ringed horns.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tether</ent><br/
<hw>Teth"er</hw> <pr>(t<ect/<th/"<etil/r)</pr>, <pos>n.</pos> <ety>[Formerly <ets>tedder</ets>, OE. <ets>tedir</ets>; akin to LG. <ets>tider</ets>, <ets>tier</ets>, Icel. <ets>tj<omac/<edh/r</ets>, Sw. <ets>tj<umac/der</ets>, Dan. <ets>t<oum/ir</ets>. <root/64.]</ety> <def>A long rope or chain by which an animal is fastened, as to a stake, so that it can range or feed only within certain limits.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tether</ent><br/
<hw>Teth"er</hw>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Tethered</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Tethering</conjf>.]</vmorph> <def>To confine, as an animal, with a long rope or chain, as for feeding within certain limits.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>And by a slender cord was <qex>tethered</qex> to a stone.</q> <rj><qau>Wordsworth.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tether-ball</ent><br/
<hw>Teth"er-ball`</hw>, <pos>n.</pos> <def>A game played with rackets and a ball suspended by a string from an upright pole, the object of each side being to wrap the string around the pole by striking the ball in a direction opposite to the other.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Tethydan</ent><br/
<hw>Te*thy"dan</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[See <er>Tethys</er>.]</ety> <fld>(Zool.)</fld> <def>A tunicate.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tethyodea</ent><br/
||<hw>Te`thy*o"de*a</hw> <pr>(?)</pr>, <pos>n. pl.</pos> <ety>[NL., fr. <ets>Tethys</ets> + Gr. <grk>e'i^dos</grk> form.]</ety> <fld>(Zool.)</fld> <def>A division of Tunicata including the common attached ascidians, both simple and compound. Called also <altname>Tethioidea</altname>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tethys</ent><br/
<hw>Te"thys</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[NL., fr. Gr. <?/ an oyster, or <?/ a kind of ascidian.]</ety> <fld>(Zool.)</fld> <def>A genus of a large naked mollusks having a very large, broad, fringed cephalic disk, and branched dorsal gills. Some of the species become a foot long and are brilliantly colored.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tetra-</ent><br/
<hw>Tet"ra-</hw> <pr>(?)</pr>. <ety>[Gr. <grk>te`tra-</grk>, from <grk>te`sares</grk>, <grk>te`ttares</grk>, four. See <er>Four</er>.]</ety> <sn>1.</sn> <def>A combining form or prefix signifying <xex>four</xex>, as in <xex>tetra</xex>basic, <xex>tetra</xex>petalous.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Chem.)</fld> <def>A combining form (also used adjectively) denoting <xex>four proportional</xex> or <xex>combining parts</xex> of the substance or ingredient denoted by the term to which it is prefixed, as in <xex>tetra</xex>-chloride, <xex>tetr</xex>oxide.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tetrabasic</ent><br/
<hw>Tet`ra*bas"ic</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[<ets>Tetra-</ets> + <ets>basic</ets>.]</ety> <fld>(Chem.)</fld> <def>Capable of neutralizing four molecules of a monacid base; having four hydrogen atoms capable of replacement by bases; quadribasic; -- said of certain acids; thus, normal silicic acid, <chform>Si(OH)4</chform>, is a <xex>tetrabasic</xex> acid.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tetraboric</ent><br/
<hw>Tet`ra*bor"ic</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[<ets>Tetra-</ets> + <ets>boric</ets>.]</ety> <fld>(Chem.)</fld> <def>Same as <er>Pyroboric</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tetrabranchiata</ent><br/
||<hw>Tet`ra*bran`chi*a"ta</hw> <pr>(?)</pr>, <pos>n. pl.</pos> <ety>[NL. See <er>Tetra-</er>, and <er>Branchia</er>.]</ety> <fld>(Zool.)</fld> <def>An order of Cephalopoda having four gills. Among living species it includes only the pearly nautilus. Numerous genera and species are found in the fossil state, such as Ammonites, Baculites, Orthoceras, etc.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tetrabranchiate</ent><br/
<hw>Tet`ra*bran`chi*ate</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[<ets>Tetra</ets> + <ets>branchiate</ets>.]</ety> <fld>(Zool.)</fld> <def>Of or pertaining to the Tetrabranchiata.</def> -- <def2><pos>n.</pos> <def>One of the Tetrabranchiata.</def></def2><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tetracarpel</ent><br/
<hw>Tet`ra*car"pel</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[<ets>Tetra-</ets> + <ets>carpellary</ets>.]</ety> <fld>(Bot.)</fld> <def>Composed of four carpels.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tetrachord</ent><br/
<hw>Tet"ra*chord</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>tetrachordon</ets>, Gr. <?/, from <?/ four-stringed; <grk>te`tra-</grk> (see <er>Tetra-</er>) + <?/ a chord: cf. F. <ets>t<eacute/trachorde</ets>.]</ety> <fld>(Anc. Mus.)</fld> <def>A scale series of four sounds, of which the extremes, or first and last, constituted a fourth. These extremes were immutable; the two middle sounds were changeable.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tetrachotomous</ent><br/
<hw>Tet`ra*chot"o*mous</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[Gr. <grk>te`tracha</grk> in four parts + <grk>te`mnein</grk> to cut.]</ety> <fld>(Bot.)</fld> <def>Having a division by fours; separated into four parts or series, or into series of fours.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tetracid</ent><br/
<hw>Tet*rac"id</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[<ets>Tetra</ets> + <ets>acid</ets>.]</ety> <fld>(Chem.)</fld> <def>Capable of neutralizing four molecules of a monobasic acid; having four hydrogen atoms capable of replacement ba acids or acid atoms; -- said of certain bases; <as>thus, erythrine, <chform>C4H6(OH)4</chform>, is a <ex>tetracid</ex> alcohol</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tetracoccous</ent><br/
<hw>Tet`ra*coc"cous</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[See <er>Tetra-</er>, and <er>Coccus</er>.]</ety> <fld>(Bot.)</fld> <def>Having four cocci, or carpels.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tetracolon</ent><br/
<hw>Tet`ra*co"lon</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Gr. <?/ with four members; <grk>te`tra-</grk> (see <er>Tetra-</er>) + <?/ limb, member.]</ety> <fld>(Pros.)</fld> <def>A stanza or division in lyric poetry, consisting of four verses or lines.</def>  <rj><au>Crabb.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tetracoralla</ent><br/
||<hw>Te`tra*co*ral"la</hw> <pr>(?)</pr>, <pos>n. pl.</pos> <ety>[NL. See <er>Tetra-</er>, and <er>Corallum</er>.]</ety> <fld>(Paleon.)</fld> <def>Same as <er>Rugosa</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tetractinellid</ent><br/
<hw>Te*trac`ti*nel"lid</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Zool.)</fld> <def>Any species of sponge of the division Tetractinellida. Also used adjectively.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tetractinellida</ent><br/
||<hw>Te*trac`ti*nel"li*da</hw> <pr>(?)</pr>, <pos>n. pl.</pos> <ety>[NL., from Gr. <grk>te`tra-</grk> tetra- + <?/, <?/, ray, spoke.]</ety> <fld>(Zool.)</fld> <def>A division of Spongiae in which the spicules are siliceous and have four branches diverging at right angles. Called also <altname>Tetractinellinae</altname>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tetrad</ent><br/
<hw>Tet"rad</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>tetras</ets>, <ets>-adis</ets>, Gr. <?/, <?/: cf. F. <ets>t<eacute/trade</ets>.]</ety> <sn>1.</sn> <def>The number four; a collection of four things; a quaternion.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Chem.)</fld> <def>A tetravalent or quadrivalent atom or radical; <as>as, carbon is a <ex>tetrad</ex></as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tetradactyle</ent><br/
<ent>Tetradactyl</ent><br/
<mhw>{ <hw>Tet`ra*dac"tyl</hw>, <hw>Tet`ra*dac"tyle</hw>  }</mhw> <pr>(?)</pr>, <pos>a.</pos> <ety>[Cf. F. <ets>t<eacute/tradactyle</ets>.]</ety> <fld>(Zool.)</fld> <def>Tetradactylous.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tetradactylous</ent><br/
<hw>Tet`ra*dac"tyl*ous</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[Gr. <?/; <grk>te`tra-</grk> (see <er>Tetra-</er>) + <?/ finger, toe.]</ety> <fld>(Zool.)</fld> <def>Having, or characterized by, four digits to the foot or hand.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tetradecane</ent><br/
<hw>Tet`ra*dec"ane</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[<ets>Tetra-</ets> + Gr. <?/ ten.]</ety> <fld>(Chem.)</fld> <def>A light oily hydrocarbon, <chform>C14H30</chform>, of the marsh-gas series; -- so called from the fourteen carbon atoms in the molecule.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tetradecapoda</ent><br/
||<hw>Tet`ra*de*cap"o*da</hw> <pr>(?)</pr>, <pos>n. pl.</pos> <ety>[NL. See <er>Tetra-</er>, and <er>Decapoda</er>.]</ety> <fld>(Zool.)</fld> <def>Same as <er>Arthrostraca</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tetradic</ent><br/
<hw>Tet*rad"ic</hw> <pr>(?)</pr>, <pos>a.</pos> <fld>(Chem.)</fld> <def>Of or pertaining to a tetrad; possessing or having the characteristics of a tetrad; <as>as, a carbon is a <ex>tetradic</ex> element</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tetradite</ent><br/
<hw>Tet"ra*dite</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[See <er>Tetrad</er>.]</ety> <def>A person in some way remarkable with regard to the number four, as one born on the <xex>fourth</xex> day of the month, or one who reverenced <xex>four</xex> persons in the Godhead.</def>  <rj><au>Smart.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tetradon</ent><br/
<hw>Tet"ra*don</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Zool.)</fld> <def>See <er>Tetrodon</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tetradont</ent><br/
<hw>Tet"ra*dont</hw> <pr>(?)</pr>, <pos>a. & n.</pos> <fld>(Zool.)</fld> <def>See <er>Tetrodont</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tetradrachma</ent><br/
<ent>Tetradrachm</ent><br/
<mhw>{ <hw>Tet"ra*drachm</hw> <pr>(?)</pr>, ||<hw>Tet`ra*drach"ma</hw> <pr>(?)</pr>, }</mhw> <pos>n.</pos> <ety>[NL. <ets>tetradrachma</ets>, fr. Gr. <grk>tetra`drachmon</grk>; <grk>te`tra-</grk> (see <er>Tetra-</er>) + <grk>drachmh`</grk> drachm, drachma.]</ety> <def>A silver coin among the ancient Greeks, of the value of four drachms.</def> <note>The Attic <ex>tetradrachm</ex> was equal to 3s. 3d. sterling, or about 76 cents.</note><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tetradymite</ent><br/
<hw>Tet*rad"y*mite</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Gr. <grk>tetra`dymos</grk> fourfold. So named from its occurrence in compound twin crystals, or fourlings.]</ety> <fld>(Min.)</fld> <def>A telluride of bismuth. It is of a pale steel-gray color and metallic luster, and usually occurs in foliated masses. Called also <altname>telluric bismuth</altname>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tetradynamia</ent><br/
||<hw>Tet`ra*dy*na"mi*a</hw> <pr>(?)</pr>, <pos>n. pl.</pos> <ety>[NL., fr. Gr. <grk>te`tra-</grk> (see <er>Tetra-</er>) + <?/ power.]</ety> <fld>(Bot.)</fld> <def>A Linnaean class of plants having six stamens, four of which are longer than the others.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tetradynamian</ent><br/
<hw>Tet`ra*dy*na"mi*an</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Bot.)</fld> <def>A plant of the order Tetradynamia.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tetradynamous</ent><br/
<ent>Tetradynamian</ent><br/
<mhw>{ <hw>Tet`ra*dy*na"mi*an</hw> <pr>(?)</pr>, <hw>Tet`ra*dyn"a*mous</hw> <pr>(?)</pr>, }</mhw> <pos>a.</pos> <fld>(Bot.)</fld> <def>Belonging to the order Tetradynamia; having six stamens, four of which are uniformly longer than the others.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tetragon</ent><br/
<hw>Tet"ra*gon</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>tetragonum</ets>, Gr. <grk>tetra`gwnon</grk>; <grk>te`tra-</grk> (see <er>Tetra-</er>) + <grk>gwni`a</grk> corner, angle: cf. F. <ets>t<eacute/tragone</ets>.]</ety> <sn>1.</sn> <fld>(Geom.)</fld> <def>A plane figure having four sides and angles; a quadrangle, as a square, a rhombus, etc.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Astrol.)</fld> <def>An aspect of two planets with regard to the earth when they are distant from each other ninety degrees, or the fourth of a circle.</def>  <rj><au>Hutton.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tetragonal</ent><br/
<hw>Te*trag"o*nal</hw> <pr>(?)</pr>, <pos>a.</pos> <sn>1.</sn> <fld>(Geom.)</fld> <def>Of or pertaining to a tetragon; having four angles or sides; thus, the square, the parallelogram, the rhombus, and the trapezium are <xex>tetragonal</xex> fingers.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Bot.)</fld> <def>Having four prominent longitudinal angles.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <fld>(Crystallog.)</fld> <def>Designating, or belonging to, a certain system of crystallization; dimetric. See <cref>Tetragonal system</cref>, under <er>Crystallization</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tetragrammaton</ent><br/
||<hw>Tet`ra*gram"ma*ton</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[NL., fr. Gr. <?/; <grk>te`tra-</grk> (see <er>Tetra-</er>) + <?/ a letter.]</ety> <def>The mystic number four, which was often symbolized to represent the Deity, whose name was expressed by four letters among some ancient nations; <as>as, the Hebrew <ex>JeHoVaH</ex>, Greek <grk>qeo`s</grk>, Latin <ex>deus</ex>, etc.</as></def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tetragynia</ent><br/
||<hw>Tet`ra*gyn"i*a</hw> <pr>(?)</pr>, <pos>n. pl.</pos> <ety>[NL., fr. Gr. <grk>te`tra-</grk> (see <er>Tetra-</er>) + <grk>gynh`</grk> a woman, female.]</ety> <fld>(Bot.)</fld> <def>A Linnaean order of plants having four styles.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tetragynous</ent><br/
<ent>Tetragynian</ent><br/
<mhw>{ <hw>Tet`ra*gyn"i*an</hw> <pr>(?)</pr>, <hw>Te*trag"y*nous</hw> <pr>(?)</pr>, }</mhw> <pos>a.</pos> <fld>(Bot.)</fld> <def>Belonging to the order <ord>Tetragynia</ord>; having four styles.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tetrahedral</ent><br/
<hw>Tet`ra*he"dral</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[See <er>Tetrahedron</er>.]</ety> <sn>1.</sn> <def>Having, or composed of, four sides.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Crystallog.)</fld> <sd>(a)</sd> <def>Having the form of the regular tetrahedron.</def> <sd>(b)</sd> <def>Pertaining or related to a tetrahedron, or to the system of hemihedral forms to which the tetrahedron belongs.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Tetrahedral angle</b></col> <fld>(Geom.)</fld>, <cd>a solid angle bounded or inclosed by four plane angles.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tetrahedrally</ent><br/
<hw>Tet`ra*he"dral*ly</hw>, <pos>adv.</pos> <def>In a tetrahedral manner.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tetrahedrite</ent><br/
<hw>Tet`ra*he"drite</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[So called because the crystals of the species are commonly <ets>tetrahedrons</ets>.]</ety> <fld>(Min.)</fld> <def>A sulphide of antimony and copper, with small quantities of other metals. It is a very common ore of copper, and some varieties yield a considerable presentage of silver. Called also <altname>gray copper ore</altname>, <altname>fahlore</altname>, and <altname>panabase</altname>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tetrahedron</ent><br/
<hw>Tet`ra*he"dron</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[<ets>Tetra-</ets> + Gr. <?/ seat, base, fr. <?/ to sit.]</ety> <fld>(Geom.)</fld> <def>A solid figure inclosed or bounded by four triangles.</def><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ In crystallography, the regular tetrahedron is regarded as the hemihedral form of the regular octahedron.</note><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Regular tetrahedron</b></col> <fld>(Geom.)</fld>, <cd>a solid bounded by four equal equilateral triangles; one of the five regular solids.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tetrahexahedral</ent><br/
<hw>Tet`ra*hex`a*he"dral</hw> <pr>(?)</pr>, <pos>a.</pos> <fld>(Crystallog.)</fld> <def>Pertaining to a tetrahexahedron.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tetrahexahedron</ent><br/
<hw>Tet`ra*hex`a*he"dron</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[<ets>Tetra-</ets> + <ets>hexahedron</ets>.]</ety> <fld>(Crystallog.)</fld> <def>A solid in the isometric system, bounded by twenty-four equal triangular faces, four corresponding to each face of the cube.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>tetrahydrocannabinol</ent><br/
<hw>tet`ra*hy`dro*can*nab"i*nol</hw> <pr>(t<ecr/t`r<adot/*h<imac/`dr<osl/*k<acr/n*n<acr/b"<icr/n<add/l)</pr>, <pos>n.</pos> <fld>(Chem. & Med.)</fld> <def>A chemical substance (<chform>C21H20O2</chform>) which is produced by the hemp plant (Cannabis sativa), and is the physiologically active agent of dried preparations of that plant, called variously marijuana, hashish, ganja, hemp, etc.; also called <altname>THC</altname>.  It causes the euphoric effect for which the preparations are smoked or chewed.  It is used in medicine in a purified form as an antiemetic (an antinausea agent), especially in conjunction with chemotherapy of cancer.  It occurs primarily as the <DELTA/<sup>1</sup>-3,4-trans isomer, also called <altname><DELTA/<sup>9</sup>-THC</altname>, with small amounts of the <DELTA/<sup>6</sup>-3,4-trans isomer detectable at about 1%.  It is a controlled substance, classified as a hallucinogen, and its possession or distribution is illegal in almost all states of the United States.</def> <au>MI11</au><br/
[<source>PJC</source>]</p>

<p><ent>Tetrakishexahedron</ent><br/
<hw>Tet`ra*kis*hex`a*he"dron</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Gr. <?/ four times + E. <ets>hexahedron</ets>.]</ety> <fld>(Crystallog.)</fld> <def>A tetrahexahedron.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tetrakosane</ent><br/
<hw>Tet"ra*ko*sane`</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[<ets>Tetra-</ets> + Gr. <?/ twenty.]</ety> <fld>(Chem.)</fld> <def>A hydrocarbon, <chform>C24H50</chform>, resembling paraffin, and like it belonging to the marsh-gas series; -- so called from having <xex>twenty-four</xex> atoms of carbon in the molecule.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tetralogy</ent><br/
<hw>Te*tral"o*gy</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Gr. <?/; <grk>te`tra-</grk> (see <er>Tetra-</er>) + <?/ a speech, discourse: cf. F. <ets>t<eacute/tralogie</ets>.]</ety> <fld>(Gr. Drama)</fld> <def>A group or series of four dramatic pieces, three tragedies and one satyric, or comic, piece (or sometimes four tragedies), represented consequently on the Attic stage at the Dionysiac festival.</def><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ A group or series of three tragedies, exhibited together without a fourth piese, was called a <xex>trilogy</xex>.</note><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tetramera</ent><br/
||<hw>Te*tram"e*ra</hw> <pr>(?)</pr>, <pos>n. pl.</pos> <ety>[NL. See <er>Tetramerous</er>.]</ety> <fld>(Zool.)</fld> <def>A division of Coleoptera having, apparently, only four tarsal joints, one joint being rudimentary.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tetramerous</ent><br/
<hw>Te*tram"er*ous</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[<ets>Tetra-</ets> + Gr. <?/ part.]</ety> <sn>1.</sn> <fld>(Bot.)</fld> <def>Having the parts arranged in sets of four; <as>as, a <ex>tetramerous</ex> flower</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Zool.)</fld> <def>Having four joints in each of the tarsi; -- said of certain insects.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tetrameter</ent><br/
<hw>Te*tram"e*ter</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>tetrametrus</ets>, Gr. <?/; <grk>te`tra-</grk> (see <er>Tetra-</er>) + <?/ a measure: cf. F. <ets>t<eacute/tram<egrave/tre</ets>.]</ety> <fld>(GR. & Latin Pros.)</fld> <def>A verse or line consisting of four measures, that is, in iambic, trochaic, and anapestic verse, of eight feet; in other kinds of verse, of four feet.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tetramethylene</ent><br/
<hw>Tet`ra*meth"yl*ene</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[<ets>Tetra-</ets> + <ets>methylene</ets>.]</ety> <fld>(Chem.)</fld> <sd>(a)</sd> <def>A hypothetical hydrocarbon, <chform>C4H8</chform>, analogous to trimethylene, and regarded as the base of well-known series or derivatives.</def> <sd>(b)</sd> <def>Sometimes, an isomeric radical used to designate certain compounds which are really related to butylene.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tetramorph</ent><br/
<hw>Tet"ra*morph</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[<ets>Tetra-</ets> + Gr. <grk>morfh`</grk> form, figure: cf. Gr. <?/ fourfold.]</ety> <fld>(Christian Art)</fld> <def>The union of the four attributes of the Evangelists in one figure, which is represented as winged, and standing on winged fiery wheels, the wings being covered with eyes. The representations of it are evidently suggested by the vision of Ezekiel (ch. i.)</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tetrandria</ent><br/
||<hw>Te*tran"dri*a</hw> <pr>(?)</pr>, <pos>n. pl.</pos> <ety>[NL., from Gr. <grk>te`tra-</grk> (see <er>Tetra-</er>) + <?/, <?/, a man, male.]</ety> <fld>(Bot.)</fld> <def>A Linnaean class of plants having four stamens.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tetrandrous</ent><br/
<ent>Tetrandrian</ent><br/
<mhw>{ <hw>Te*tran"dri*an</hw> <pr>(?)</pr>, <hw>Te*tran"drous</hw> <pr>(?)</pr>, }</mhw> <pos>a.</pos> <fld>(Bot.)</fld> <def>Belonging to the class Tetrandria.</def><br/
[<source>1913 Webster</source>]</p>

<p><-- p. 1492 --><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tetraonid</ent><br/
<hw>Te*tra"o*nid</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>tetrao</ets> a heath cock, grouse, Gr. <?/: cf. F. <ets>t<eacute/traonide</ets>.]</ety> <fld>(Zool.)</fld> <def>A bird belonging to the tribe of which the genus <gen>Tetrao</gen> is the type, as the grouse, partridge, quail, and the like. Used also adjectively.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tetrapetalous</ent><br/
<hw>Tet`ra*pet"al*ous</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[<ets>Tetra-</ets> + <ets>petal</ets>.]</ety> <fld>(Bot.)</fld> <def>Containing four distinct petals, or flower leaves; <as>as, a <ex>tetrapetalous</ex> corolla</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tetrapharmacum</ent><br/
<ent>Tetrapharmacom</ent><br/
<mhw>{ <hw>Tet`ra*phar"ma*com</hw> <pr>(?)</pr>, <hw>Tet`ra*phar"ma*cum</hw> <pr>(?)</pr> }</mhw>, <pos>n.</pos> <ety>[NL. <ets>tetrapharmacon</ets>, L. <ets>tetrapharmacum</ets>, Gr. <?/; <grk>te`tra-</grk> (see <er>Tetra-</er>) + <?/ a drug.]</ety> <fld>(Med.)</fld> <def>A combination of wax, resin, lard, and pitch, composing an ointment.</def>  <rj><au>Brande & C.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tetraphenol</ent><br/
<hw>Tet`ra*phe"nol</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[<ets>Tetra-</ets> + <ets>phenol</ets>.]</ety> <fld>(Chem.)</fld> <def>Furfuran.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tetraphyllous</ent><br/
<hw>Te*traph"yl*lous</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[<ets>Tetra-</ets> + Gr. <?/ a leaf.]</ety> <fld>(Bot.)</fld> <def>Having four leaves; consisting of four distinct leaves or leaflets.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tetrapla</ent><br/
||<hw>Tet"ra*pla</hw> <pr>(?)</pr>, <pos>n.</pos>; <note>etymologically <pos>pl.</pos>, but syntactically <pos>sing.</pos></note> <ety>[NL., fr. Gr. <grk>tetraplo`os</grk>, <grk>tetraploy^s</grk>, fourfold.]</ety> <def>A Bible consisting of four different Greek versions arranged in four columns by <persfn>Origen</persfn>; hence, any version in four languages or four columns.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tetrapneumona</ent><br/
||<hw>Tet`rap*neu"mo*na</hw> <pr>(?)</pr>, <pos>n. pl.</pos> <ety>[NL. See <er>Tetra-</er>, and <er>Pneumo-</er>.]</ety> <fld>(Zool.)</fld> <def>A division of Arachnida including those spiders which have four lungs, or pulmonary sacs. It includes the bird spiders (<gen>Mygale</gen>) and the trapdoor spiders. See <er>Mygale</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tetrapnuemonian</ent><br/
<hw>Tet`rap*nue*mo"ni*an</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Zool.)</fld> <def>One of the Tetrapneumona.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tetrapod</ent><br/
<hw>Tet"ra*pod</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Gr. <?/ fourfooted; <grk>te`tra-</grk> (see <er>Tetra-</er>) + <?/, <?/, foot.]</ety> <fld>(Zool.)</fld> <def>An insect characterized by having but four perfect legs, as certain of the butterflies.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tetrapody</ent><br/
<hw>Te*trap"o*dy</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Gr. <?/.]</ety> <def>A set of four feet; a measure or distance of four feet.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tetrapteran</ent><br/
<hw>Te*trap"ter*an</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[See <er>Tetrapterous</er>.]</ety> <fld>(Zool.)</fld> <def>An insect having four wings.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tetrapterous</ent><br/
<hw>Te*trap"ter*ous</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[Gr. <?/; <grk>te`tra-</grk> (see <er>Tetra-</er>) + <?/ wing.]</ety> <fld>(Zool.)</fld> <def>Having four wings.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tetraptote</ent><br/
<hw>Tet"rap*tote</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>tetraptotum</ets>, Gr. <?/.]</ety> <fld>(Gram.)</fld> <def>A noun that has four cases only.</def>  <rj><au>Andrews.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tetrarch</ent><br/
<hw>Te"trarch</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>tetrarches</ets>, Gr. <?/, <?/; <grk>te`tra-</grk> (see <er>Tetra-</er>) + <?/ a ruler, <?/ to lead; rule: cf. F. <ets>t<eacute/trarque</ets>. See <er>Arch</er>, <pos>a.</pos>]</ety> <fld>(Rom. Antiq.)</fld> <def>A Roman governor of the fourth part of a province; hence, any subordinate or dependent prince; also, a petty king or sovereign.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tetrarch</ent><br/
<hw>Te"trarch</hw>, <pos>a.</pos> <def>Four.</def> <mark>[Obs.]</mark>  <rj><au>Fuller.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tetrarchate</ent><br/
<hw>Te*trarch"ate</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Cf. F. <ets>t<eacute/trarchat</ets>.]</ety> <fld>(Rom. Antiq.)</fld> <def>A tetrarchy.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tetrarchical</ent><br/
<hw>Te*trarch"ic*al</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[Gr. <?/.]</ety> <def>Of or pertaining to a tetrarch or tetrarchy.</def>  <rj><au>Bolingbroke.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tetrarchy</ent><br/
<hw>Tet"rarch*y</hw> <pr>(?)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Tetrarchies</plw> <pr>(#)</pr>.</plu> <ety>[L. <ets>tetrarchia</ets>, Gr. <?/: cf. F. <ets>t<eacute/trarchie</ets>.]</ety> <fld>(Rom. Antiq.)</fld> <def>The district under a Roman tetrarch; the office or jurisdiction of a tetrarch; a tetrarchate.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tetraschistic</ent><br/
<hw>Tet`ra*schis"tic</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[Gr. <?/ divided into four parts; <grk>te`tra-</grk> tetra- + <?/ to split.]</ety> <fld>(Biol.)</fld> <def>Characterized by division into four parts.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tetrasepalous</ent><br/
<hw>Tet`ra*sep"al*ous</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[<ets>Tetra-</ets> + <ets>sepal</ets>.]</ety> <fld>(Bot.)</fld> <def>Having four sepals.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tetraspaston</ent><br/
<hw>Tet`ra*spas"ton</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[NL., fr. Gr. <grk>te`tra-</grk> (see <er>Tetra-</er>) + <?/ to draw, pull.]</ety> <fld>(Mach.)</fld> <def>A machine in which four pulleys act together.</def>  <rj><au>Brande & C.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tetraspermous</ent><br/
<hw>Tet`ra*sper"mous</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[<ets>Tetra-</ets> + Gr. <?/ a seed.]</ety> <fld>(Bot.)</fld> <def>Having four seeds.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Tetraspermous plant</b></col>, <cd>a plant which produces four seeds in each flower.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tetraspore</ent><br/
<hw>Tet"ra*spore</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[<ets>Tetra-</ets> + <ets>spore</ets>.]</ety> <fld>(Bot.)</fld> <def>A nonsexual spore, one of a group of four regularly occurring in red seaweeds.</def> -- <wordforms><wf>Tet`ra*spor"ic</wf> <pr>(#)</pr>, <pos>a.</pos></wordforms><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tetrastich</ent><br/
<hw>Te*tras"tich</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>tetrastichon</ets>, Gr. <?/; <grk>te`tra-</grk> (see <er>Tetra-</er>) + <?/ a row, verse.]</ety> <def>A stanza, epigram, or poem, consisting of four verses or lines.</def>  <rj><au>Pope.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tetrastyle</ent><br/
<hw>Tet"ra*style</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>tetrastylon</ets>, Gr. <?/ with four pillars in front; <grk>te`tra-</grk> (see <er>Tetra-</er>) + <?/ a column.]</ety> <fld>(Arch.)</fld> <def>Having four columns in front; -- said of a temple, portico, or colonnade.</def> -- <def2><pos>n.</pos> <def>A tetrastyle building.</def></def2><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tetrasyllabical</ent><br/
<ent>Tetrasyllabic</ent><br/
<mhw>{ <hw>Tet`ra*syl*lab"ic</hw> <pr>(?)</pr>, <hw>Tet`ra*syl*lab"ic*al</hw> <pr>(?)</pr>, }</mhw> <pos>a.</pos> <ety>[Cf. F. <ets>t<eacute/trasyllabique</ets>.]</ety> <def>Consisting of, or having, four syllables; quadrisyllabic.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tetrasyllable</ent><br/
<hw>Tet"ra*syl`la*ble</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[<ets>Tetra-</ets> + <ets>syllable</ets>: cf. Gr. <?/ of four syllables.]</ety> <def>A word consisting of four syllables; a quadrisyllable.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tetrathecal</ent><br/
<hw>Tet`ra*the"cal</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[<ets>Tetra-</ets> + <ets>thecal</ets>.]</ety> <fld>(Bot.)</fld> <def>Having four loculaments, or thecae.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tetrathionate</ent><br/
<hw>Tet`ra*thi"on*ate</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Chem.)</fld> <def>A salt of tetrathionic acid.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tetrathionic</ent><br/
<hw>Tet`ra*thi*on"ic</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[<ets>Tetra-</ets> + <ets>thionic</ets>.]</ety> <fld>(Chem.)</fld> <def>Of, pertaining to, or designating, a thionic derivative, <chform>H2S4O6</chform>, of sulphuric acid, obtained as a colorless, odorless liquid.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tetratomic</ent><br/
<hw>Tet`ra*tom"ic</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[<ets>Tetra-</ets> + <ets>atomic</ets>.]</ety> <fld>(Chem.)</fld> <sd>(a)</sd> <def>Consisting of four atoms; having four atoms in the molecule, as phosphorus and arsenic.</def> <sd>(b)</sd> <def>Having a valence of four; quadrivalent; tetravalent; sometimes, in a specific sense, having four hydroxyl groups, whether acid or basic.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tetravalence</ent><br/
<hw>Te*trav"a*lence</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Chem.)</fld> <def>The quality or state of being tetravalent; quadrivalence.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tetravalent</ent><br/
<hw>Te*trav"a*lent</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[<ets>Tetra-</ets> + L. <ets>valens</ets>, <ets>-entis</ets>, p. pr.]</ety> <fld>(Chem.)</fld> <def>Having a valence of four; tetratomic; quadrivalent.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tetraxile</ent><br/
<hw>Te*trax"ile</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[<ets>Tetra-</ets> + <ets>axile</ets>.]</ety> <fld>(Zool.)</fld> <def>Having four branches diverging at right angles; -- said of certain spicules of sponges.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>-in</ent><br/
<ent>Tetrazine</ent><br/
<mhw>{ <hw>Tet*raz"ine</hw> <pr>(?)</pr>, <pos>n.</pos> Also <hw>-in</hw>  }</mhw>. <ety>[<ets>Tetrazo-</ets> + <ets>-ine</ets>.]</ety> <fld>(Chem.)</fld> <def>A hypothetical compound, <chform>C2H2N4</chform> which  may be regarded as benzene with four <chform>CH</chform> groups replaced by nitrogen atoms; also, any of various derivatives of the same. There are three isomeric varieties.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Tetrazo-</ent><br/
<hw>Tet*raz"o-</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[<ets>Tetra-</ets> + <ets>azo-</ets>.]</ety> <fld>(Chem.)</fld> <def>A combining form (also used adjectively), designating any one of a series of double derivatives of the azo and diazo compounds <xex>containing four atoms of nitrogen</xex>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tetrazole</ent><br/
<hw>Tet*raz"ole</hw> <pr>(?)</pr>, <pos>n.</pos>  <ety>[<ets>Tetrazo-</ets> + <ets>-ole</ets>.]</ety> <fld>(Org. Chem.)</fld> <def>A crystalline acid substance, <chform>CH2N4</chform>, which may be regarded as pyrrol in which nitrogen atoms replace three <chform>CH</chform> groups; also, any of various derivatives of the same.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Tetrazone</ent><br/
<hw>Tet"ra*zone</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Chem.)</fld> <def>Any one of a certain series of basic compounds containing a chain of four nitrogen atoms; <as>for example, ethyl <ex>tetrazone</ex>, <chform>(C2H5)2N.N2.N(C2H5)2</chform>, a colorless liquid having an odor of leeks</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tetrical</ent><br/
<ent>Tetric</ent><br/
<mhw>{ <hw>Tet"ric</hw> <pr>(?)</pr>, <hw>Tet"ri*cal</hw> <pr>(?)</pr>, }</mhw> <pos>a.</pos> <ety>[L. <ets>tetricus</ets>, taetricus, from <ets>teter</ets>, <ets>taeter</ets>, offensive, foul.]</ety> <def>Forward; perverse; harsh; sour; rugged.</def> <mark>[Obs.]</mark> -- <wordforms><wf>Tet"ric*al*ness</wf>, <pos>n.</pos></wordforms><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tetricity</ent><br/
<hw>Te*tric"i*ty</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>tetricitas</ets>, <ets>taetricitas</ets>.]</ety> <def>Crabbedness; perverseness.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tetricous</ent><br/
<hw>Tet"ric*ous</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Tetric.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tetrinic</ent><br/
<hw>Te*trin"ic</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[See <er>Tetra-</er>.]</ety> <fld>(Chem.)</fld> <def>Of, pertaining to, or designating, a complex ketonic acid, <chform>C5H6O3</chform>, obtained as a white crystalline substance; -- so called because once supposed to contain a peculiar radical of <xex>four</xex> carbon atoms. Called also <altname>acetyl-acrylic acid</altname>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tetrodon</ent><br/
<hw>Tet"ro*don</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[<ets>Tetra-</ets> + Gr. <?/, <?/, tooth.]</ety> <fld>(Zool.)</fld> <def>Any one of numerous species of plectognath fishes belonging to <gen>Tetrodon</gen> and allied genera. Each jaw is furnished with two large, thick, beaklike, bony teeth.</def> <altsp>[Written also <asp>tetradon</asp>.]</altsp><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ The skin is usually spinous, and the belly is capable of being greatly distended by air or water. It includes the swellfish, puffer <sd>(a)</sd>, and similar species.</note><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tetrodont</ent><br/
<hw>Tet"ro*dont</hw> <pr>(?)</pr>, <pos>a.</pos> <fld>(Zool.)</fld> <def>Of or pertaining to the tetrodons.</def> -- <def2><pos>n.</pos> <def>A tetrodon.</def></def2> <altsp>[Written also <asp>tetradont</asp>, and <asp>tetraodont</asp>.]</altsp><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tetrol</ent><br/
<hw>Tet"rol</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[<ets>Tetra-</ets> + benz<ets>ol</ets>.]</ety> <fld>(Chem.)</fld> <def>A hypothetical hydrocarbon, <chform>C4H4</chform>, analogous to benzene; -- so called from the <xex>four</xex> carbon atoms in the molecule.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Tetrol phenol</b></col>, <cd>furfuran.</cd> <mark>[Obs.]</mark></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tetrolic</ent><br/
<hw>Tet*rol"ic</hw> <pr>(?)</pr>, <pos>a.</pos> <fld>(Chem.)</fld> <def>Of, pertaining to, or designating, an acid, <chform>C3H3.CO2H</chform>, of the acetylene series, homologous with propiolic acid, obtained as a white crystalline substance.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tetrose</ent><br/
<hw>Tet"rose</hw> <pr>(?)</pr>, <pos>n.</pos>  <ety>[<ets>Tetra-</ets> + <ets>-ose</ets>.]</ety> <fld>(Chem.)</fld> <def>A monosaccharide derived from a certain alcohol.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Tetroxide</ent><br/
<hw>Tet*rox"ide</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[<ets>Tetr</ets>a- + <ets>oxide</ets>.]</ety> <fld>(Chem.)</fld> <def>An oxide having four atoms of oxygen in the molecule; a quadroxide; <as>as, osmium <ex>tetroxide</ex>, <chform>OsO4</chform></as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tetryl</ent><br/
<hw>Tet"ryl</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[<ets>Tetr</ets>a- + <ets>-yl</ets>.]</ety> <fld>(Chem.)</fld> <def>Butyl; -- so called from the <xex>four</xex> carbon atoms in the molecule.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tetrylene</ent><br/
<hw>Tet"ryl*ene</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[<ets>Tetr</ets>a- + eth<ets>ylene</ets>.]</ety> <fld>(Chem.)</fld> <def>Butylene; -- so called from the <xex>four</xex> carbon atoms in the molecule.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tetter</ent><br/
<hw>Tet"ter</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[OE. <ets>teter</ets>, AS. <ets>teter</ets>, <ets>tetr</ets>; akin to G. <ets>zitter</ets>, <ets>zitter</ets>mal, OHG. <ets>zittar</ets>och, Skr. <ets>dadru</ets>, <ets>dadruka</ets>, a sort of skin disease. <root/63, 240.]</ety> <fld>(Med.)</fld> <def>A vesicular disease of the skin; herpes. See <er>Herpes</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Honeycomb tetter</b></col> <fld>(Med.)</fld>, <cd>favus.</cd> -- <col><b>Moist tetter</b></col> <fld>(Med.)</fld>, <cd>eczema.</cd> -- <col><b>Scaly tetter</b></col> <fld>(Med.)</fld>, <cd>psoriasis.</cd> -- <col><b>Tetter berry</b></col> <fld>(Bot.)</fld>, <cd>the white bryony.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tetter</ent><br/
<hw>Tet"ter</hw>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Tettered</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Tettering</conjf>.]</vmorph> <def>To affect with tetter.</def>  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tetterous</ent><br/
<hw>Tet"ter*ous</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Having the character of, or pertaining to, tetter.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tetter-totter</ent><br/
<hw>Tet"ter-tot`ter</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[See <er>Teeter</er>.]</ety> <def>A certain game of children; seesaw; -- called also <altname>titter-totter</altname>, and <altname>titter-cum-totter</altname>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tetterwort</ent><br/
<hw>Tet"ter*wort`</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Bot.)</fld> <def>A plant used as a remedy for tetter, -- in England the calendine, in America the bloodroot.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tettigonian</ent><br/
<hw>Tet`ti*go"ni*an</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Gr. <?/, dim. of <?/ a kind of grasshopper.]</ety> <fld>(Zool.)</fld> <def>Any one of numerous species of Hemiptera belonging to <gen>Tettigonia</gen> and allied genera; a leaf hopper.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tettish</ent><br/
<hw>Tet"tish</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[Cf. <er>Testy</er>.]</ety> <def>Captious; testy.</def> <altsp>[Written also <asp>teatish</asp>.]</altsp> <mark>[Obs.]</mark>  <rj><au>Beau. & Fl.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tettix</ent><br/
||<hw>Tet"tix</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[NL., fr. Gr. <?/ a kind of grasshopper.]</ety> <sn>1.</sn> <fld>(Zool.)</fld> <def>The cicada.</def> <mark>[Obs. or R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Zool.)</fld> <def>A genus of small grasshoppers.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tetty</ent><br/
<hw>Tet"ty</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Testy; irritable.</def> <mark>[Obs.]</mark>  <rj><au>Burton.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Teufit</ent><br/
<hw>Teu"fit</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Zool.)</fld> <def>The lapwing; -- called also <altname>teuchit</altname>.</def> <mark>[Prov. Eng.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Teuk</ent><br/
<hw>Teuk</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Zool.)</fld> <def>The redshank.</def> <mark>[Prov. Eng.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Teuton</ent><br/
<hw>Teu"ton</hw> <pr>(?)</pr>, <pos>n.</pos>; <plu><it>pl.</it> E. <plw>Teutons</plw> <pr>(#)</pr>, L. <plw>Teutones</plw> <pr>(#)</pr>.</plu> <ety>[L. <ets>Teutones</ets>, <ets>Teutoni</ets>, the name of a Germanic people, probably akin to E. <ets>Dutch</ets>.  Cf. <er>Dutch</er>.]</ety> <sn>1.</sn> <def>One of an ancient German tribe; later, a name applied to any member of the Germanic race in Europe; now used to designate a German, Dutchman, Scandinavian, etc., in distinction from a Celt or one of a Latin race.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A member of the Teutonic branch of the Indo-European, or Aryan, family.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Teutonic</ent><br/
<hw>Teu*ton"ic</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>Teutonicus</ets>, from <ets>Teutoni</ets>, or <ets>Teutones</ets>. See <er>Teuton</er>.]</ety> <sn>1.</sn> <def>Of or pertaining to the Teutons, esp. the ancient Teutons; Germanic.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Of or pertaining to any of the Teutonic languages, or the peoples who speak these languages.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Teutonic languages</b></col>, <cd>a group of languages forming a division of the Indo-European, or Aryan, family, and embracing the High German, Low German, Gothic, and Scandinavian dialects and languages.</cd> -- <col><b>Teutonic order</b></col>, <cd>a military religious order of knights, established toward the close of the twelfth century, in imitation of the Templars and Hospitalers, and composed chiefly of Teutons, or Germans. The order rapidly increased in numbers and strength till it became master of all Prussia, Livonia, and Pomerania. In its decay it was abolished by Napoleon; but it has been revived as an honorary order.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Teutonic</ent><br/
<hw>Teu*ton"ic</hw> <pr>(?)</pr>, <pos>n.</pos> <def>The language of the ancient Germans; the Teutonic languages, collectively.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Teutonicism</ent><br/
<hw>Teu*ton"i*cism</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A mode of speech peculiar to the Teutons; a Teutonic idiom, phrase, or expression; a Teutonic mode or custom; a Germanism.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tew</ent><br/
<hw>Tew</hw> <pr>(?)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Tewed</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Tewing</conjf>.]</vmorph> <ety>[OE. <ets>tewen</ets>, <ets>tawen</ets>. <root/64. See <er>Taw</er>, <pos>v.</pos>]</ety> <sn>1.</sn> <def>To prepare by beating or working, as leather or hemp; to taw.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Hence, to beat; to scourge; also, to pull about; to maul; to tease; to vex.</def> <mark>[Obs. or Prov. Eng. & Scot.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tew</ent><br/
<hw>Tew</hw>, <pos>v. i.</pos> <def>To work hard; to strive; to fuse.</def> <mark>[Local]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tew</ent><br/
<hw>Tew</hw>, <pos>v. t.</pos> <ety>[Cf. <er>Taw</er> to tow, <er>Tow</er>, <pos>v. t.</pos>]</ety> <def>To tow along, as a vessel.</def> <mark>[Obs.]</mark>  <rj><au>Drayton.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tew</ent><br/
<hw>Tew</hw>, <pos>n.</pos> <def>A rope or chain for towing a boat; also, a cord; a string.</def> <mark>[Obs. or Prov. Eng.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tewan</ent><br/
<hw>Te"wan</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Ethnol.)</fld> <def>A tribe of American Indians including many of the Pueblos of New Mexico and adjacent regions.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tewed</ent><br/
<hw>Tewed</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Fatigued; worn with labor or hardship.</def> <mark>[Obs. or Local]</mark>  <rj><au>Mir. for Mag.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tewel</ent><br/
<hw>Tew"el</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[OE. <ets>tuel</ets>, OF. <ets>tuiel</ets>, <ets>tuel</ets>, F. <ets>tuyau</ets>; of Teutonic origin; cf. Dan. <ets>tud</ets>, D. <ets>tuit</ets>, Prov. G. <ets>zaute</ets>.  Cf. <er>Tuy<egrave/re</er>.]</ety> <sn>1.</sn> <def>A pipe, funnel, or chimney, as for smoke.</def>  <rj><au>Chaucer.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>The tuy<egrave/re of a furnace.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tewhit</ent><br/
<hw>Te"whit</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Zool.)</fld> <def>The lapwing; -- called also <altname>teewheep</altname>.</def> <mark>[Prov. Eng.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tewtaw</ent><br/
<hw>Tew"taw</hw> <pr>(?)</pr>, <pos>v. t.</pos> <ety>[See <er>Tew</er>, <pos>v. t.</pos>]</ety> <def>To beat; to break, as flax or hemp.</def> <mark>[Obs.]</mark>  <rj><au>Mortimer.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Texas</ent><br/
<hw>Tex"as</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A structure on the hurricane deck of a steamer, containing the pilot house, officers' cabins, etc.</def> <mark>[Western U. S.]</mark>  <rj><au>Knight.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Texas Leaguer</ent><br/
<hw>Tex"as Leagu"er</hw> <pr>(?)</pr>. <ety>[From the Texas (baseball) League.]</ety> <fld>(Baseball)</fld> <def>A short fly that falls too far out to be handled by an infielder and too close in to be caught by an outfielder; -- called also <altname>Texas league single</altname>.</def> <mark>[Cant]</mark><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Text</ent><br/
<hw>Text</hw> <pr>(t<ecr/kst)</pr>, <pos>n.</pos> <ety>[F. <ets>texte</ets>, L. <ets>textus</ets>, texture, structure, context, fr. <ets>texere</ets>, <ets>textum</ets>, to weave, construct, compose; cf. Gr. <grk>te`ktwn</grk> carpenter, Skr. <ets>taksh</ets> to cut, carve, make.  Cf. <er>Context</er>, <er>Mantle</er>, <pos>n.</pos>, <er>Pretext</er>, <er>Tissue</er>, <er>Toil</er> a snare.]</ety> <sn>1.</sn> <def>A discourse or composition on which a note or commentary is written; the original words of an author, in distinction from a paraphrase, annotation, or commentary.</def>  <rj><au>Chaucer.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(O. Eng. Law)</fld> <def>The four Gospels, by way of distinction or eminence.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>A verse or passage of Scripture, especially one chosen as the subject of a sermon, or in proof of a doctrine.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>How oft, when Paul has served us with a <qex>text</qex>,<br/
Has Epictetus, Plato, Tully, preached!</q> <rj><qau>Cowper.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>Hence, anything chosen as the subject of an argument, literary composition, or the like; topic; theme.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <def>A style of writing in large characters; text-hand also, a kind of type used in printing; <as>as, German <ex>text</ex></as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>6.</sn> <def>That part of a document (printed or electronic) comprising the words, especially the main body of expository words, in contrast to the illustrations, pictures, charts, tables, or other formatted material which contain graphic elements as a major component.</def><br/
[<source>PJC</source>]</p>

<p><sn>7.</sn>  <def>Any communication composed of words.</def><br/
[<source>PJC</source>]</p>

<p><sn>8.</sn>  <def>a textbook.</def><br/
[<source>PJC</source>]</p>

<p><cs><col><b>Text blindness</b></col>. <fld>(Physiol.)</fld> <cd>See <cref>Word blindness</cref>, under <er>Word</er>.</cd> -- <col><b>Text letter</b></col>, <cd>a large or capital letter.</cd> <mark>[Obs.]</mark> -- <col><b>Text pen</b></col>, <cd>a kind of metallic pen used in engrossing, or in writing text-hand.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Text</ent><br/
<hw>Text</hw>, <pos>v. t.</pos> <def>To write in large characters, as in text hand.</def> <mark>[Obs.]</mark>  <rj><au>Beau. & Fl.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Text-book</ent><br/
<hw>Text"-book`</hw> <pr>(?)</pr>, <pos>n.</pos> <sn>1.</sn> <def>A book with wide spaces between the lines, to give room for notes.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A volume, as of some classical author, on which a teacher lectures or comments; hence, any manual of instruction; a schoolbook.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Text-hand</ent><br/
<ent>Text hand</ent><br/
<mhw><hw>Text hand</hw>, <hw>Text"-hand`</hw></mhw> <pr>(?)</pr>, <pos>n.</pos> <def>A large hand in writing; -- so called because it was the practice to write the text of a book in a large hand and the notes in a smaller hand.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Textile</ent><br/
<hw>Tex"tile</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>textilis</ets>, fr. <ets>texere</ets> to weave: cf. F. <ets>textile</ets>. See <er>Text</er>.]</ety> <def>Pertaining to weaving or to woven fabrics; <as>as, <ex>textile</ex> arts</as>; woven, capable of being woven; formed by weaving; <as>as, <ex>textile</ex> fabrics</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Textile cone</b></col> <fld>(Zool.)</fld>, <cd>a beautiful cone shell (<spn>Conus textilis</spn>) in which the colors are arranged so that they resemble certain kinds of cloth.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Textile</ent><br/
<hw>Tex"tile</hw>, <pos>n.</pos> <def>That which is, or may be, woven; a fabric made by weaving.</def>  <rj><au>Bacon.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Textman</ent><br/
<hw>Text"man</hw> <pr>(?)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Textmen</plw> <pr>(<?/)</pr>.</plu> <def>One ready in quoting texts.</def> <mark>[R.]</mark>  <rj><au>Bp. Sanderston.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Textorial</ent><br/
<hw>Tex*to"ri*al</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>textorius</ets>, fr. <ets>textor</ets> a weaver, fr. <ets>texere</ets>, <ets>textum</ets>, to weave.]</ety> <def>Of or pertaining to weaving.</def>  <rj><au>T. Warton.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Textrine</ent><br/
<hw>Tex"trine</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>textrinus</ets>, for <ets>textorinus</ets>, fr. <ets>textor</ets> a weaver.]</ety> <def>Of or pertaining to weaving, textorial; <as>as, the <ex>textrine</ex> art</as>.</def>  <rj><au>Denham.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Textual</ent><br/
<hw>Tex"tu*al</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[OE. <ets>textuel</ets>, F. <ets>textuel</ets>.]</ety> <sn>1.</sn> <def>Of, pertaining to, or contained in, the text; <as>as, <ex>textual</ex> criticism; a <ex>textual</ex> reading</as>.</def>  <rj><au>Milton.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Serving for, or depending on, texts.</def>  <rj><au>Bp. Hall.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>Familiar with texts or authorities so as to cite them accurately.</def> <ldquo/I am not <xex>textuel</xex>.<rdquo/  <rj><au>Chaucer.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Textualist</ent><br/
<hw>Tex"tu*al*ist</hw>, <pos>n.</pos> <def>A textman; a textuary.</def>  <rj><au>Lightfoot.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Textually</ent><br/
<hw>Tex"tu*al*ly</hw>, <pos>adv.</pos> <def>In a textual manner; in the text or body of a work; in accordance with the text.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Textuarist</ent><br/
<hw>Tex"tu*a*rist</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A textuary.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Textuary</ent><br/
<hw>Tex"tu*a*ry</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[Cf. F. <ets>textuaire</ets>.]</ety> <sn>1.</sn> <def>Contained in the text; textual.</def>  <rj><au>Sir T. Browne.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Serving as a text; authoritative.</def>  <rj><au>Glanvill.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Textuary</ent><br/
<hw>Tex"tu*a*ry</hw>, <pos>n.</pos> <ety>[Cf. F. <ets>textuaire</ets>.]</ety> <sn>1.</sn> <def>One who is well versed in the Scriptures; a textman.</def>  <rj><au>Bp. Bull.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>One who adheres strictly or rigidly to the text.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Textuel</ent><br/
<hw>Tex"tu*el</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Textual.</def> <mark>[Obs.]</mark>  <rj><au>Chaucer.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Textuist</ent><br/
<hw>Tex"tu*ist</hw>, <pos>n.</pos> <def>A textualist; a textman.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>The crabbed <qex>textualists</qex> of his time.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Textural</ent><br/
<hw>Tex"tur*al</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Of or pertaining to texture.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Texture</ent><br/
<hw>Tex"ture</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>textura</ets>, fr. <ets>texere</ets>, <ets>textum</ets>, to weave: cf. F. <ets>texture</ets>. See <er>Text</er>.]</ety> <sn>1.</sn> <def>The act or art of weaving.</def> <mark>[R.]</mark>  <rj><au>Sir T. Browne.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>That which woven; a woven fabric; a web.</def>  <rj><au>Milton.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Others, apart far in the grassy dale,<br/
Or roughening waste, their humble <qex>texture</qex> weave.</q> <rj><qau>Thomson.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>The disposition or connection of threads, filaments, or other slender bodies, interwoven; <as>as, the <ex>texture</ex> of cloth or of a spider's web</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>The disposition of the several parts of any body in connection with each other, or the manner in which the constituent parts are united; structure; <as>as, the <ex>texture</ex> of earthy substances or minerals; the <ex>texture</ex> of a plant or a bone; the <ex>texture</ex> of paper; a loose or compact <ex>texture</ex>.</as></def><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <fld>(Biol.)</fld> <def>A tissue. See <er>Tissue</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><-- p. 1493 --><br/
[<source>1913 Webster</source>]</p>

<p><ent>Texture</ent><br/
<hw>Tex"ture</hw> <pr>(?)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Textured</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Texturing</conjf>.]</vmorph> <def>To form a texture of or with; to interweave.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Textury</ent><br/
<hw>Tex"tur*y</hw> <pr>(?)</pr>, <pos>n.</pos> <def>The art or process of weaving; texture.</def> <mark>[Obs.]</mark>  <rj><au>Sir T. Browne.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Teyne</ent><br/
<hw>Teyne</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[See <er>Tain</er>.]</ety> <def>A thin plate of metal.</def> <mark>[Obs.]</mark> <ldquo/A <xex>teyne</xex> of silver.<rdquo/  <rj><au>Chaucer.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Th</ent><br/
<hw>Th</hw>. <def>In Old English, the article <xex>the</xex>, when the following word began with a vowel, was often written with elision as if a part of the word. Thus in Chaucer, the forms <xex>thabsence</xex>, <xex>tharray</xex>, <xex>thegle</xex>, <xex>thend</xex>, <xex>thingot</xex>, etc., are found for <xex>the absence</xex>, <xex>the array</xex>, <xex>the eagle</xex>, <xex>the end</xex>, etc.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Thacker</ent><br/
<ent>Thack</ent><br/
<mhw>{ <hw>Thack</hw> <pr>(?)</pr>, <hw>Thack"er</hw> <pr>(?)</pr> }</mhw>. <def>See <er>Thatch</er>, <er>Thatcher</er>.</def> <mark>[Obs. or Prov. Eng. & Scot.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Thai</ent><br/
<hw>Thai</hw> <pr>(t<imac/)</pr>, <pos>n.</pos> <def>The language of Thailand (Siam); it is a member of the Tai group of languages.</def><br/
[<source>PJC</source>]</p>

<p><ent>Thai</ent><br/
<hw>Thai</hw> <pr>(t<imac/)</pr>, <pos>a.</pos> <def>Of or pertaining to the Thai language, a member of the Tai group of languages.</def><br/
[<source>PJC</source>]</p>

<p><sn>2.</sn> <def>Of or pertaining to Thailand or its people.</def><br/
[<source>PJC</source>]</p>

<p><ent>Thak</ent><br/
<hw>Thak</hw> <pr>(?)</pr>, <pos>v. t.</pos> <def>To thwack.</def> <mark>[Obs.]</mark>  <rj><au>Chaucer.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Thalamencephalon</ent><br/
||<hw>Thal`a*men*ceph"a*lon</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[NL. See <er>Thalamus</er>, and <er>Encephalon</er>.]</ety> <fld>(Anat.)</fld> <def>The segment of the brain next in front of the midbrain, including the thalami, pineal gland, and pituitary body; the diencephalon; the interbrain.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Thalamic</ent><br/
<hw>Tha*lam"ic</hw> <pr>(?)</pr>, <pos>a.</pos> <fld>(Anat.)</fld> <def>Of or pertaining to a thalamus or to thalami.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Thalamiflorous</ent><br/
<ent>Thalamifloral</ent><br/
<mhw>{ <hw>Thal`a*mi*flo"ral</hw> <pr>(?)</pr>, <hw>Thal`a*mi*flo"rous</hw> <pr>(?)</pr>, }</mhw> <pos>a.</pos> <ety>[See <er>Thalamus</er>, and <er>Floral</er>.]</ety> <fld>(Bot.)</fld> <def>Bearing the stamens directly on the receptacle; -- said of a subclass of polypetalous dicotyledonous plants in the system of De Candolle.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Thalamocoele</ent><br/
<hw>Thal"a*mo*coele`</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[<ets>Thalam</ets>ic + Gr. <grk>koi^los</grk> hollow.]</ety> <fld>(Anat.)</fld> <def>The cavity or ventricle of the thalamencephalon; the third ventricle.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Thalamophora</ent><br/
||<hw>Thal`a*moph"o*ra</hw> <pr>(?)</pr>, <pos>n. pl.</pos> <ety>[NL., fr. Gr. <grk>qa`lamos</grk> chamber + <grk>fe`rein</grk> to bear.]</ety> <fld>(Zool.)</fld> <def>Same as <er>Foraminifera</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Thalamus</ent><br/
||<hw>Thal"a*mus</hw> <pr>(?)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Thalami</plw> <pr>(#)</pr>.</plu> <ety>[L. <ets>thalamus</ets> chamber, Gr. <grk>qa`lamos</grk>.]</ety> <sn>1.</sn> <fld>(Anat.)</fld> <def>A mass of nervous matter on either side of the third ventricle of the brain; -- called also <altname>optic thalamus</altname>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Bot.)</fld> <sd>(a)</sd> <def>Same as <er>Thallus</er>.</def> <sd>(b)</sd> <def>The receptacle of a flower; a torus.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Thalassian</ent><br/
<hw>Tha*las"si*an</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[From Gr. <grk>qa`lassa</grk> the sea.]</ety> <fld>(Zool.)</fld> <def>Any sea tortoise.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Thalassic</ent><br/
<hw>Tha*las"sic</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[Gr. <grk>qa`lassa</grk> the sea.]</ety> <fld>(Geol.)</fld> <def>Of or pertaining to the sea; -- sometimes applied to rocks formed from sediments deposited upon the sea bottom.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Thalassinian</ent><br/
<hw>Thal`as*sin"i*an</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Zool.)</fld> <def>Any species of <fam>Thalassinidae</fam>, a family of burrowing macrurous Crustacea, having a long and soft abdomen.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Thalassography</ent><br/
<hw>Thal`as*sog"ra*phy</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Gr. <grk>qa`lassa</grk> sea + <ets>-graphy</ets>.]</ety> <def>The study or science of the life of marine organisms.</def>  <rj><au>Agassiz.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>thaler</ent><br/
||<hw>tha"ler</hw> <pr>(t<aum/"l<etil/r)</pr>, <pos>n.</pos> <ety>[G. See <er>Dollar</er>.]</ety> <def>A former German silver coin worth about three shillings sterling, or about 73 cents, around 1900.</def><br/
[<source>1913 Webster</source> <source>+PJC</source>]</p>

<p><ent>Thalia</ent><br/
<hw>Tha*li"a</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L., fr. Gr. <grk>Qa`leia</grk>, originally, blooming, luxuriant, akin to <grk>qa`llein</grk> to be luxuriant.]</ety> <fld>(Class. Myth.)</fld> <sd>(a)</sd> <def>That one of the nine Muses who presided over comedy.</def> <sd>(b)</sd> <def>One of the three Graces.</def> <sd>(c)</sd> <def>One of the Nereids.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Thaliacea</ent><br/
||<hw>Tha`li*a"ce*a</hw> <pr>(?)</pr>, <pos>n. pl.</pos> <ety>[NL. See <er>Thalia</er>.]</ety> <fld>(Zool.)</fld> <def>A division of Tunicata comprising the free-swimming species, such as Salpa and Doliolum.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Thalian</ent><br/
<hw>Tha*li"an</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Of or pertaining to Thalia; hence, of or pertaining to comedy; comic.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Thallate</ent><br/
<hw>Thal"late</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Chem.)</fld> <def>A salt of a hypothetical thallic acid.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Thallene</ent><br/
<hw>Thal"lene</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Chem.)</fld> <def>A hydrocarbon obtained from coal-tar residues, and remarkable for its intense yellowish green fluorescence.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Thallic</ent><br/
<hw>Thal"lic</hw> <pr>(?)</pr>, <pos>a.</pos> <fld>(Chem.)</fld> <def>Of or pertaining to thallium; derived from, or containing, thallium; specifically, designating those compounds in which the element has a higher valence as contrasted with the <contr>thallous</contr> compounds; <as>as, <ex>thallic</ex> oxide</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Thalline</ent><br/
<hw>Thal"line</hw> <pr>(?)</pr>, <pos>a.</pos> <fld>(Bot.)</fld> <def>Consisting of a thallus.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Thalline</ent><br/
<hw>Thal"line</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Gr. <?/ a young shoot or branch.]</ety> <fld>(Chem.)</fld> <def>An artificial alkaloid of the quinoline series, obtained as a white crystalline substance, <chform>C10H13NO</chform>, whose salts are valuable as antipyretics; -- so called from the <xex>green</xex> color produced in its solution by certain oxidizing agents.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Thallious</ent><br/
<hw>Thal"li*ous</hw> <pr>(?)</pr>, <pos>a.</pos> <fld>(Chem.)</fld> <def>See <er>Thallous</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Thallium</ent><br/
<hw>Thal"li*um</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[NL., fr. Gr. <?/ young or green shoot or branch, twig. So called from a characteristic bright <ets>green</ets> line in its spectrum.]</ety> <fld>(Chem.)</fld> <def>A rare metallic element of the aluminium group found in some minerals, as certain pyrites, and also in the lead-chamber deposit in the manufacture of sulphuric acid. It is isolated as a heavy, soft, bluish white metal, easily oxidized in moist air, but preserved by keeping under water. Symbol Tl. Atomic weight 203.7.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Thallogen</ent><br/
<hw>Thal"lo*gen</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Gr. <?/ young shoot or branch, frond + <ets>-gen</ets>.]</ety> <fld>(Bot.)</fld> <def>One of a large class or division of the vegetable kingdom, which includes those flowerless plants, such as fungi, algae, and lichens, that consist of a thallus only, composed of cellular tissue, or of a congeries of cells, or even of separate cells, and never show a distinction into root, stem, and leaf.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Thalloid</ent><br/
<hw>Thal"loid</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[<ets>Thallus</ets> + <ets>-oid</ets>.]</ety> <fld>(Bot.)</fld> <def>Resembling, or consisting of, thallus.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Thallophyta</ent><br/
||<hw>Thal*loph"y*ta</hw> <pr>(?)</pr>, <pos>n. pl.</pos> <ety>[NL. See <er>Thallophyte</er>.]</ety> <fld>(Bot.)</fld> <def>A phylum of plants of very diverse habit and structure, including the algae, fungi, and lichens. The simpler forms, as many blue-green algae, yeasts, etc., are unicellular and reproduce vegetatively or  by means of asexual spores; in the higher forms the plant body is a <xex>thallus</xex>, which may be filamentous or may consist of plates of cells; it is commonly undifferentiated into stem, leaves, and roots, and shows no distinct tissue systems; the fronds of many algae, however, are modified to serve many of the functions of the above-named organs. Both asexual and sexual reproduction, often of a complex type, occur in these forms. The Thallophyta exist almost exclusively as gametophytes, the sporophyte being absent or rudimentary. By those who do not separate the Myxophyta from the Tallophyta as a distinct phylum the latter is treated as the lowermost group in the vegetable kingdom.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Thallophyte</ent><br/
<hw>Thal"lo*phyte</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Gr. <?/ young shoot + <ets>-phyte</ets>.]</ety> <sn>1.</sn> <fld>(Bot.)</fld> <def>A plant belonging to the Thallophyta.</def> -- <wordforms><wf>Thal`lo*phyt"ic</wf> <pr>(#)</pr>, <pos>a.</pos></wordforms><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><sn>2.</sn> <fld>(Bot.)</fld> <def>Same as <er>Thallogen</er>.</def><br/
[<source>1913 Webster</source>]<-- ##is this identical to thallophyte??
 --></p>

<p><ent>Thallous</ent><br/
<hw>Thal"lous</hw> <pr>(?)</pr>, <pos>a.</pos> <fld>(Chem.)</fld> <def>Of or pertaining to thallium; derived from, or containing, thallium; specifically, designating those compounds in which the element has a lower valence as contrasted with the <contr>thallic</contr> compounds.</def> <altsp>[Written also <asp>thallious</asp>.]</altsp><br/
[<source>1913 Webster</source>]</p>

<p><ent>Thallus</ent><br/
||<hw>Thal"lus</hw> <pr>(?)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Thalli</plw> <pr>(#)</pr>.</plu> <ety>[NL., fr. Gr. <?/ young shoot or branch, frond.]</ety> <fld>(Bot.)</fld> <def>A solid mass of cellular tissue, consisting of one or more layers, usually in the form of a flat stratum or expansion, but sometimes erect or pendulous, and elongated and branching, and forming the substance of the thallogens.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Thalweg</ent><br/
||<hw>Thal"weg`</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[G., fr. <ets>thal</ets> valley + <ets>weg</ets> way. See <er>Dale</er>; <er>Way</er>.]</ety> <fld>(Physiography)</fld> <sd>(a)</sd> <def>A line following the lowest part of a valley, whether under water or not.</def> <sd>(b)</sd> <def>The line of continuous maximum descent from any point on a land surface, or that cutting all contours and angles.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Tammuz</ent><br/
<ent>Thammuz</ent><br/
<mhw>{ <hw>Tham"muz</hw> <pr>(?)</pr>, <hw>Tam"muz</hw> <pr>(?)</pr>, }</mhw> <pos>n.</pos> <ety>[Heb. <ets>thamm<umac/z</ets>.]</ety> <sn>1.</sn> <def>A deity among the ancient Syrians, in honor of whom the Hebrew idolatresses held an annual lamentation. This deity has been conjectured to be the same with the Phoenician Adon, or Adonis.</def>  <rj><au>Milton.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>The fourth month of the Jewish ecclesiastical year, -- supposed to correspond nearly with our month of July.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Thamnophile</ent><br/
<hw>Tham"no*phile</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Gr. <grk>qa`mnos</grk> a bush + <grk>fi`los</grk> loving.]</ety> <fld>(Zool.)</fld> <def>A bush shrike.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Thamyn</ent><br/
||<hw>Tha"myn</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Zool.)</fld> <def>An Asiatic deer (<spn>Rucervus Eldi</spn>) resembling the swamp deer; -- called also <altname>Eld's deer</altname>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Than</ent><br/
<hw>Than</hw> <pr>(<th/<acr/n)</pr>, <pos>conj.</pos> <ety>[OE. <ets>than</ets>, <ets>thon</ets>, <ets>then</ets>, <ets>thanne</ets>, <ets>thonne</ets>, <ets>thenne</ets>, than, then, AS. <ets><edh/anne</ets>, <ets><edh/onne</ets>, <ets><edh/aenne</ets>; akin to D. <ets>dan</ets>, OHG. <ets>danne</ets>, G. <ets>dann</ets> then, <ets>denn</ets> than, for, Goth. <ets><thorn/an</ets> then, and to E. <ets>the</ets>, <ets>there</ets>, <ets>that</ets>. See <er>That</er>, and cf. <er>Then</er>.]</ety> <def>A particle expressing comparison, used after certain adjectives and adverbs which express comparison or diversity, as <xex>more</xex>, <xex>better</xex>, <xex>other</xex>, <xex>otherwise</xex>, and the like. It is usually followed by the object compared in the nominative case. Sometimes, however, the object compared is placed in the objective case, and <xex>than</xex> is then considered by some grammarians as a preposition. Sometimes the object is expressed in a sentence, usually introduced by <xex>that</xex>; <as>as, I would rather suffer <ex>than</ex> that you should want</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Behold, a greater <qex>than</qex> Solomon is here.</q> <rj><qau>Matt. xii. 42.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Which when Beelzebub perceived, <qex>than</qex> whom,<br/
Satan except, none higher sat.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>It's wiser being good <qex>than</qex> bad;<br/
It's safer being meek <qex>than</qex> fierce;<br/
It's fitter being sane <qex>than</qex> mad.</q> <rj><qau>R. Browning.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Than</ent><br/
<hw>Than</hw>, <pos>adv.</pos> <def>Then. See <er>Then</er>.</def> <mark>[Obs.]</mark>  <rj><au>Gower.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q><qex>Thanne</qex> longen folk to gon on pilgrimages.</q> <rj><qau>Chaucer.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Thana</ent><br/
||<hw>Tha"na</hw> <pr>(t<aum/"n<aum/)</pr>, <pos>n.</pos> <altname>[Written also <asp>tana</asp>, <asp>tanna</asp>.]</altname> <ety>[Hind. <ets>th<amac/n<amac/</ets>.]</ety> <def>A police station.</def> <mark>[India]</mark>  <rj><au>Kipling.</au></rj><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Thanage</ent><br/
<hw>Than"age</hw> <pr>(?)</pr>, <pos>n.</pos> <def>The district in which a thane anciently had jurisdiction; thanedom.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Thanatoid</ent><br/
<hw>Than"a*toid</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[Gr. <grk>qa`natos</grk> death + <ets>-oid</ets>.]</ety> <def>Deathlike; resembling death.</def>  <rj><au>Dunglison.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Thanatology</ent><br/
<hw>Than`a*tol"o*gy</hw> <pr>(th<acr/n`<adot/*t<ocr/l"<osl/*j<ycr/)</pr>, <pos>n.</pos> <ety>[Gr. <grk>qa`natos</grk> + <ets>-logy</ets>.]</ety> <def>A description, or the doctrine, of death.</def>  <rj><au>Dunglison.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>thanatopsis</ent><br/
<hw>than`a*top"sis</hw> <pr>(th<acr/n`<adot/*t<ocr/p"s<icr/s)</pr>, <pos>n.</pos> <ety>[NL., fr. Gr. <grk>qa`natos</grk> death + <grk>'o`psis</grk> view.]</ety>  <def>A view of death; a meditation on the subject of death.</def>  <rj><au>Bryant.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Thanatopsis</ent><br/
<hw>Than`a*top"sis</hw> <pr>(th<acr/n`<adot/*t<ocr/p"s<icr/s)</pr>, <pos>prop. n.</pos> <ety>[NL., fr. Gr. <grk>qa`natos</grk> death + <grk>'o`psis</grk> view.]</ety>  <def>The title of a poem by <person>William Cullen Bryant</person> (1794-1878), meditating on the subject of death.  One of Bryant's best-remembered poems, it was written in 1811 and was discovered and rushed to publication in 1817 (in the <it>North American Review</it>) by <persfn>Bryant's</persfn> father, originally without the poet's knowledge.  A revised version was published in 1821.  In this elegy <persfn>Bryant</persfn> reflects that death comes to all men, common and great, and that all eventually shall rest together in the "mighty sepulchre" of the earth.</def><br/
[<source>PJC</source>]</p>

<p><note>The text of the poem is as follows:<br/
<br/
To him who in the love of nature holds<br/
Communion with her visible forms, she speaks<br/
A various language; for his gayer hours<br/
She has a voice of gladness, and a smile<br/
And eloquence of beauty; and she glides<br/
Into his darker musings, with a mild<br/
And healing sympathy that steals away<br/
Their sharpness ere he is aware.  When thoughts<br/
Of the last bitter hour come like a blight<br/
Over thy spirit, and sad images<br/
Of the stern agony, and shroud, and pall,<br/
And breathless darkness, and the narrow house,<br/
Make thee to shudder, and grow sick at heart; --<br/
Go forth, under the open sky, and list<br/
To Nature's teachings, while from all around --<br/
Earth and her waters, and the depths of air --<br/
Comes a still voice.  Yet a few days, and thee<br/
The all-beholding sun shall see no more<br/
In all his course; nor yet in the cold ground,<br/
Where thy pale form was laid, with many tears,<br/
Nor in the embrace of ocean, shall exist<br/
Thy image.  Earth, that nourished thee, shall claim<br/
Thy growth, to be resolved to earth again,<br/
And, lost each human trace, surrendering up<br/
Thine individual being, shalt thou go <br/
To mix forever with the elements,<br/
To be a brother to the insensible rock<br/
And to the sluggish clod, which the rude swain<br/
Turns with his share, and treads upon.  The oak<br/
Shall send his roots abroad, and pierce thy mold.<br/
Yet not to thine eternal resting-place<br/
Shalt thou retire alone, nor couldst thou wish<br/
Couch more magnificent.  Thou shalt lie down<br/
With patriarchs of the infant world -- with kings,<br/
The powerful of the earth -- the wise, the good,<br/
Fair forms, and hoary seers of ages past,<br/
All in one mighty sepulchre.  -- The hills<br/
Rock-ribbed and ancient as the sun, -- the vales<br/
Stretching in pensive quietness between;<br/
The venerable woods -- rivers that move<br/
In majesty, and the complaining brooks<br/
That make the meadows green; and, poured round all,<br/
Old Ocean's gray and melancholy waste,--<br/
Are but the solemn decorations all<br/
Of the great tomb of man.  The golden sun,<br/
The planets, all the infinite host of heaven,<br/
Are shining on the sad abodes of death<br/
Through the still lapse of ages.  All that tread<br/
The globe are but a handful to the tribes<br/
That slumber in its bosom. -- Take the wings<br/
Of morning, pierce the Barcan wilderness,<br/
Or lose thyself in the continuous woods<br/
Where rolls the Oregon, and hears no sound,<br/
Save his own dashings -- yet the dead are there:<br/
And millions in those solitudes, since first<br/
The flight of years began, have laid them down<br/
In their last sleep -- the dead reign there alone.<br/
So shalt thou rest -- and what if thou withdraw<br/
In silence from the living, and no friend<br/
Take note of thy departure?  All that breathe<br/
Will share thy destiny.  The gay will laugh<br/
When thou art gone, the solemn brood of care<br/
Plod on, and each one as before will chase<br/
His favorite phantom; yet all these shall leave<br/
Their mirth and their employments, and shall come<br/
And make their bed with thee.  As the long train<br/
Of ages glides away, the sons of men--<br/
The youth in life's fresh spring, and he who goes<br/
In the full strength of years, matron and maid,<br/
The speechless babe, and the gray-headed man--<br/
Shall one by one be gathered to thy side,<br/
By those, who in their turn, shall follow them.<br/
So live, that when thy summons comes to join<br/
The innumerable caravan, which moves<br/
To that mysterious realm, where each shall take<br/
His chamber in the silent halls of death,<br/
Thou go not, like the quarry-slave at night,<br/
Scourged to his dungeon, but, sustained and soothed<br/
By an unfaltering trust, approach thy grave<br/
Like one who wraps the drapery of his couch<br/
About him, and lies down to pleasant dreams.<br/
</note><br/
[<source>PJC</source>]</p>

<p><ent>Thane</ent><br/
<hw>Thane</hw> <pr>(th<amac/n)</pr>, <pos>n.</pos> <ety>[OE. <ets>thein</ets>, <ets><thorn/ein</ets>, AS. <ets><thorn/egen</ets>, <ets><thorn/egn</ets>; akin to OHG. <ets>degan</ets> a follower, warrior, boy, MHG. <ets>degen</ets> a hero, G. <ets>degen</ets> hero, soldier, Icel. <ets><thorn/egn</ets> a thane, a freeman; probably akin to Gr. <grk>te`knon</grk> a child, <grk>ti`ktein</grk> to bear, beget, or perhaps to Goth. <ets><thorn/ius</ets> servant, AS. <ets><thorn/e<oacute/w</ets>, G. <ets>dienen</ets> to serve.]</ety> <def>A dignitary under the Anglo-Saxons and Danes in England.  Of these there were two orders, the king's thanes, who attended the kings in their courts and held lands immediately of them, and the ordinary thanes, who were lords of manors and who had particular jurisdiction within their limits.  After the Conquest, this title was disused, and <altname>baron</altname> took its place.</def><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ Among the ancient Scots, <ex>thane</ex> was a title of honor, which seems gradually to have declined in its significance.</note>  <rj><au>Jamieson.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>thanedom</ent><br/
<hw>thane"dom</hw> <pr>(th<amac/n"d<ucr/m)</pr>, <pos>n.</pos> <def>The property or jurisdiction of a thane; thanage.</def>  <rj><au>Sir W. Scott.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>thanehood</ent><br/
<hw>thane"hood</hw> <pr>(th<amac/n"h<oocr/d)</pr>, <pos>n.</pos> <def>The character or dignity of a thane; also, thanes, collectively.</def>  <rj><au>J. R. Green.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>thaneship</ent><br/
<hw>thane"ship</hw>, <pos>n.</pos> <def>The state or dignity of a thane; thanehood; also, the seignioralty of a thane.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>thank</ent><br/
<hw>thank</hw> <pr>(th<acr/<nsm/k)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>thanks</plw> <pr>(th<acr/<nsm/ks)</pr>.</plu> <ety>[AS. <ets><thorn/anc</ets>, <ets><thorn/onc</ets>, thanks, favor, thought; akin to OS. <ets>thank</ets> favor, pleasure, thanks, D. & G. <ets>dank</ets> thanks, Icel. <ets><thorn/<oum/kk</ets>, Dan. <ets>tak</ets>, Sw. <ets>tack</ets>, Goth. <ets><thorn/agks</ets> thanks; -- originally, a thought, a thinking. See <er>Think</er>.]</ety> <def>A expression of gratitude; an acknowledgment expressive of a sense of favor or kindness received; obligation, claim, or desert, or gratitude; -- now generally used in the plural.</def> <ldquo/This ceremonial <xex>thanks</xex>.<rdquo/  <rj><au>Massinger.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>If ye do good to them which do good to you, what <qex>thank</qex> have ye? for sinners also do even the same.</q> <rj><qau>Luke vi. 33.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>What great <qex>thank</qex>, then, if any man, reputed wise and constant, will neither do, nor permit others under his charge to do, that which he approves not, especially in matter of sin?</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q><qex>Thanks</qex>, <qex>thanks</qex> to thee, most worthy friend,<br/
For the lesson thou hast taught.</q> <rj><qau>Longfellow.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><mcol><col><b>His thanks</b></col>, <col><b>Her thanks</b></col>, etc.</mcol>, <cd>of his or her own accord; with his or her good will; voluntary.</cd> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>Full sooth is said that love ne lordship,<br/
Will not, <qex>his thanks</qex>, have no fellowship.</q> <rj><qau>Chaucer.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p>-- <col><b>In thank</b></col>, <cd>with thanks or thankfulness.</cd> <mark>[Obs.]</mark> -- <col><b>Thank offering</b></col>, <cd>an offering made as an expression of thanks.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Thank</ent><br/
<hw>Thank</hw> <pr>(?)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Thanked</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Thanking</conjf>.]</vmorph> <ety>[AS. <ets><thorn/ancian</ets>. See <er>Thank</er>, <pos>n.</pos>]</ety> <def>To express gratitude to (anyone) for a favor; to make acknowledgments to (anyone) for kindness bestowed; -- used also ironically for <xex>blame</xex>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q><ldquo/Graunt mercy, lord, that <qex>thank</qex> I you,<rdquo/ quod she.</q> <rj><qau>Chaucer.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>I <qex>thank</qex> thee for thine honest care.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Weigh the danger with the doubtful bliss,<br/
And <qex>thank</qex> yourself if aught should fall amiss.</q> <rj><qau>Dryden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Thankful</ent><br/
<hw>Thank"ful</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[AS. <ets><thorn/ancfull</ets>.]</ety> <sn>1.</sn> <def>Obtaining or deserving thanks; thankworthy.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>Ladies, look here; this is the <qex>thankful</qex> glass<br/
That mends the looker's eyes; this is the well<br/
That washes what it shows.</q> <rj><qau>Herbert.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Impressed with a sense of kindness received, and ready to acknowledge it; grateful.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Be <qex>thankful</qex> unto him, and bless his name.</q> <rj><qau>Ps. c. 4.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p>-- <wordforms><wf>Thank"ful*ly</wf>, <pos>adv.</pos> -- <wf>Thank"ful*ness</wf>, <pos>n.</pos></wordforms><br/
[<source>1913 Webster</source>]</p>

<p><ent>Thankless</ent><br/
<hw>Thank"less</hw>, <pos>a.</pos> <sn>1.</sn> <def>Not acknowledging favors; not expressing thankfulness; unthankful; ungrateful.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>That she may feel<br/
How sharper than a serpent's tooth it is<br/
To have a <qex>thankless</qex> child!</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Not obtaining or deserving thanks; unacceptable; <as>as, a <ex>thankless</ex> task</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>To shepherd <qex>thankless</qex>, but by thieves that love the night allowed.</q> <rj><qau>Chapman.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p>-- <wordforms><wf>Thank"less*ly</wf> <pr>(#)</pr>, <pos>adv.</pos> -- <wf>Thank"less*ness</wf>, <pos>n.</pos></wordforms><br/
[<source>1913 Webster</source>]</p>

<p><ent>Thankly</ent><br/
<hw>Thank"ly</hw>, <pos>adv.</pos> <def>Thankfully.</def> <mark>[Obs.]</mark>  <rj><au>Sylvester (Du Bartas).</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Thanksgive</ent><br/
<hw>Thanks"give</hw> <pr>(?)</pr>, <pos>v. t.</pos> <def>To give or dedicate in token of thanks.</def> <mark>[Obs. or R.]</mark>  <rj><au>Mede.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Thanksgiver</ent><br/
<hw>Thanks"giv`er</hw> <pr>(?)</pr>, <pos>n.</pos> <def>One who gives thanks, or acknowledges a kindness.</def>  <rj><au>Barrow.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Thanksgiving</ent><br/
<hw>Thanks"giv`ing</hw> <pr>(?)</pr>, <pos>n.</pos> <sn>1.</sn> <def>The act of rending thanks, or expressing gratitude for favors or mercies.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Every creature of God is good, and nothing to be refused, if it be received with <qex>thanksgiving</qex>.</q> <rj><qau>1 Tim. iv. 4.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>In the <qex>thanksgiving</qex> before meat.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>And taught by thee the Church prolongs<br/
Her hymns of high <qex>thanksgiving</qex> still.</q> <rj><qau>Keble.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A public acknowledgment or celebration of divine goodness; also, a day set apart for religious services, specially to acknowledge the goodness of God, either in any remarkable deliverance from calamities or danger, or in the ordinary dispensation of his bounties.</def><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ In the United States it is now customary for the President by proclamation to appoint annually a day (usually the last Thursday in November) of thanksgiving and praise to God for the mercies of the past year. This is an extension of the custom long prevailing in several States in which an annual Thanksgiving day has been appointed by proclamation of the governor.</note><br/
[<source>1913 Webster</source>]</p>

<p><ent>Thankworthiness</ent><br/
<hw>Thank"wor`thi*ness</hw> <pr>(?)</pr>, <pos>n.</pos> <def>The quality or state of being thankworthy.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Thankworthy</ent><br/
<hw>Thank"wor`thy</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Deserving thanks; worthy of gratitude; mreitorious.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>For this <qex>thankworthy</qex>, if a man, for conscience toward God, endure grief, suffering wrongfully.</q> <rj><qau>1 Pet. ii. 19.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Thar</ent><br/
<hw>Thar</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Zool.)</fld> <def>A goatlike animal (<spn>Capra Jemlaica</spn>) native of the Himalayas. It has small, flattened horns, curved directly backward. The hair of the neck, shoulders, and chest of the male is very long, reaching to the knees. Called also <altname>serow</altname>, and <altname>imo</altname>.</def> <altsp>[Written also <asp>thaar</asp>, and <asp>tahr</asp>.]</altsp><br/
[<source>1913 Webster</source>]</p>

<p><ent>Thar</ent><br/
<hw>Thar</hw>, <pos>v. impersonal, pres.</pos> <ety>[OE. <ets>thar</ets>, <ets><thorn/arf</ets>, AS. <ets><thorn/earf</ets>, infin. <ets><thorn/urfan</ets> to need; akin to OHG. <ets>durfan</ets>, G. <ets>d<uum/rfen</ets> to be allowed, Icel. <ets><thorn/urfa</ets> to need, Goth. <ets><thorn/a<uacute/rban</ets>.]</ety> <def>It needs; need.</def> <mark>[Obs.]</mark>  <rj><au>Piers Plowman.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>What <qex>thar</qex> thee reck or care?</q> <rj><qau>Chaucer.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tharms</ent><br/
<hw>Tharms</hw> <pr>(?)</pr>, <pos>n. pl.</pos> <ety>[AS. <ets><thorn/earm</ets> a gut; akin to D. & G. <ets>darm</ets>, Icel. <ets><thorn/armr</ets>, Sw. & Dan. <ets>tarm</ets>. <root/53.]</ety> <def>Twisted guts.</def> <mark>[Obs. or Prov. Eng.]</mark>  <rj><au>Ascham.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tharos</ent><br/
<hw>Tha"ros</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Zool.)</fld> <def>A small American butterfly (<spn>Phycoides tharos</spn>) having the upper surface of the wings variegated with orange and black, the outer margins black with small white crescents; -- called also <altname>pearl crescent</altname>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>That</ent><br/
<hw>That</hw> <pr>(?)</pr>, <pos>pron., a., conj., & adv.</pos> <ety>[AS. <ets><edh/aet</ets>, neuter nom. & acc. sing. of the article (originally a demonstrative pronoun). The nom. masc. <ets>s<emac/</ets>, and the nom. fem. <ets>se<oacute/</ets> are from a different root. AS. <ets><edh/aet</ets> is akin to D. <ets>dat</ets>, G. <ets>das</ets>, OHG. <ets>daz</ets>, Sw. & Dan. <ets>det</ets>, Icel. <ets><thorn/at</ets> (masc. <ets>s<amac/</ets>, fem. <ets>s<omac/</ets>), Goth. <ets><thorn/ata</ets> (masc. <ets>sa</ets>, fem. <ets>s<omac/</ets>), Gr. <?/ (masc. <?/, fem. <?/), Skr. <ets>tat</ets> (for <ets>tad</ets>, masc. <ets>sas</ets>, fem. <ets>s<amac/</ets>); cf. L. is<ets>tud</ets> that. <root/184.  Cf. <er>The</er>, <er>Their</er>, <er>They</er>, <er>Them</er>, <er>This</er>, <er>Than</er>, <er>Since</er>.]</ety> <sn>1.</sn> <def>As a demonstrative pronoun (<pluf>pl.</pluf> <er>Those</er>), <xex>that</xex> usually points out, or refers to, a person or thing previously mentioned, or supposed to be understood. <xex>That</xex>, as a demonstrative, may precede the noun to which it refers; <as>as, <ex>that</ex> which he has said is true; <ex>those</ex> in the basket are good apples.</as></def><br/
[<source>1913 Webster</source>]</p>

<p><q>The early fame of Gratian was equal to <qex>that</qex> of the most celebrated princes.</q> <rj><qau>Gibbon.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ <xex>That</xex> may refer to an entire sentence or paragraph, and not merely to a word. It usually follows, but sometimes precedes, the sentence referred to.</note><br/
[<source>1913 Webster</source>]</p>

<p><q><qex>That</qex> be far from thee, to do after this manner, to slay the righteous with the wicked.</q> <rj><qau>Gen. xviii. 25.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>And when Moses heard <qex>that</qex>, he was content.</q> <rj><qau>Lev. x. 20.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>I will know your business, Harry, <qex>that</qex> I will.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ <xex>That</xex> is often used in opposition to <xex>this</xex>, or by way of distinction, and in such cases <xex>this</xex>, like the Latin <xex>hic</xex> and French <xex>ceci</xex>, generally refers to that which is nearer, and <xex>that</xex>, like Latin <xex>ille</xex> and French <xex>cela</xex>, to that which is more remote. When they refer to foreign words or phrases, <xex>this</xex> generally refers to the latter, and <xex>that</xex> to the former.</note><br/
[<source>1913 Webster</source>]</p>

<p><q>Two principles in human nature reign;<br/
Self-love, to urge, and Reason, to restrain;<br/
Nor <qex>this</qex> a good, nor <qex>that</qex> a bad we call.</q> <rj><qau>Pope.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>If the Lord will, we shall live, and do <qex>this</qex> or <qex>that</qex>.</q> <rj><qau>James iv. 16.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>As an adjective, <xex>that</xex> has the same demonstrative force as the pronoun, but is followed by a noun.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>It shall be more tolerable for Sodom and Gomorrah in the day of judgment, than for <qex>that</qex> city.</q> <rj><qau>Matt. x. 15.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>The woman was made whole from <qex>that</qex> hour.</q> <rj><qau>Matt. ix. 22.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ <xex>That</xex> was formerly sometimes used with the force of the article <xex>the</xex>, especially in the phrases <xex>that one</xex>, <xex>that other</xex>, which were subsequently corrupted into <xex>th'tone</xex>, <xex>th'tother</xex> (now written <xex>t'other</xex>).</note><br/
[<source>1913 Webster</source>]</p>

<p><q>Upon a day out riden knightes two . . . <br/
<qex>That one</qex> of them came home, <qex>that other</qex> not.</q> <rj><qau>Chaucer.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>As a relative pronoun, <xex>that</xex> is equivalent to <xex>who</xex> or <xex>which</xex>, serving to point out, and make definite, a person or thing spoken of, or alluded to, before, and may be either singular or plural.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>He <qex>that</qex> reproveth a scorner getteth to himself shame.</q> <rj><qau>Prov. ix. 7.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>A judgment <qex>that</qex> is equal and impartial must incline to the greater probabilities.</q> <rj><qau>Bp. Wilkins.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><-- p. 1494 --><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ If the relative clause simply conveys an additional idea, and is not properly explanatory or restrictive, <xex>who</xex> or <xex>which</xex> (rarely <xex>that</xex>) is employed; as, the king <xex>that</xex> (or <xex>who</xex>) rules well is generally popular; Victoria, <xex>who</xex> (not <xex>that</xex>) rules well, enjoys the confidence of her subjects. Ambiguity may in some cases be avoided in the use of <xex>that</xex> (which is restrictive) instead of <xex>who</xex> or <xex>which</xex>, likely to be understood in a coordinating sense. <au>Bain.</au><br/
[<source>1913 Webster</source>]</p>

<p><xex>That</xex> was formerly used for <xex>that which</xex>, as <xex>what</xex> is now; but such use is now archaic.<br/
[<source>1913 Webster</source>]</p>

<p><q>We speak <qex>that</qex> we do know, and testify <qex>that</qex> we have seen.</q> <rj><qau>John iii. 11.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q><qex>That</qex> I have done it is thyself to wite [blame].</q> <rj><qau>Chaucer.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><xex>That</xex>, as a relative pronoun, cannot be governed by a preposition preceding it, but may be governed by one at the end of the sentence which it commences.<br/
[<source>1913 Webster</source>]</p>

<p><q>The ship <qex>that</qex> somebody was sailing in.</q> <rj><qau>Sir W. Scott.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p>In Old English, <xex>that</xex> was often used with the demonstratives <xex>he</xex>, <xex>his</xex>, <xex>him</xex>, etc., and the two together had the force of a relative pronoun; thus, <xex>that he</xex> = who; <xex>that his</xex> = whose; <xex>that him</xex> = whom.<br/
[<source>1913 Webster</source>]</p>

<p><q>I saw to-day a corpse yborn to church<br/
<qex>That</qex> now on Monday last I saw <qex>him</qex> wirche [work].</q> <rj><qau>Chaucer.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p>Formerly, <xex>that</xex> was used, where we now commonly use <xex>which</xex>, as a relative pronoun with the demonstrative pronoun <xex>that</xex> as its antecedent.<br/
[<source>1913 Webster</source>]</p>

<p><q>That <qex>that</qex> dieth, let it die; and that <qex>that</qex> is to cut off, let it be cut off.</q> <rj><qau>Zech. xi. 9.</qau></rj>
</note><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>As a conjunction, <xex>that</xex> retains much of its force as a demonstrative pronoun.</def> It is used, specifically: --<br/
[<source>1913 Webster</source>]</p>

<p><sd>(a)</sd> <def>To introduce a clause employed as the object of the preceding verb, or as the subject or predicate nominative of a verb.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>She tells them 't is a causeless fantasy,<br/
And childish error, <qex>that</qex> they are afraid.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>I have shewed before, <qex>that</qex> a mere possibility to the contrary, can by no means hinder a thing from being highly credible.</q> <rj><qau>Bp. Wilkins.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sd>(b)</sd> <def>To introduce, a reason or cause; -- equivalent to <xex>for that</xex>, <xex>in that</xex>, <xex>for the reason that</xex>, <xex>because</xex>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>He does hear me;<br/
And <qex>that</qex> he does, I weep.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sd>(c)</sd> <def>To introduce a purpose; -- usually followed by <xex>may</xex>, or <xex>might</xex>, and frequently preceded by <xex>so</xex>, <xex>in order</xex>, <xex>to the end</xex>, etc.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>These things I say, <qex>that</qex> ye might be saved.</q> <rj><qau>John v. 34.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>To the end <qex>that</qex> he may prolong his days.</q> <rj><qau>Deut. xvii. 20.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sd>(d)</sd> <def>To introduce a consequence, result, or effect; -- usually preceded by <xex>so</xex> or <xex>such</xex>, sometimes by <xex>that</xex>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The birds their notes renew, and bleating herds<br/
Attest their joy, <qex>that</qex> hill and valley rings.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>He gazed so long<br/
<qex>That</qex> both his eyes were dazzled.</q> <rj><qau>Tennyson.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sd>(e)</sd> To introduce a clause denoting time; -- equivalent to <xex>in which time</xex>, <xex>at which time</xex>, <xex>when</xex>.<br/
[<source>1913 Webster</source>]</p>

<p><q>So wept Duessa until eventide,<br/
<qex>That</qex> shining lamps in Jove's high course were lit.</q> <rj><qau>Spenser.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Is not this the day<br/
<qex>That</qex> Hermia should give answer of her choice?</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sd>(f)</sd> <def>In an elliptical sentence to introduce a dependent sentence expressing a wish, or a cause of surprise, indignation, or the like.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Ha, cousin Silence, <qex>that</qex> thou hadst seen that that this knight and I have seen!</q> <rj><qau>Shak.</qau></rj>
<-- = if only . . . = if --><br/
[<source>1913 Webster</source>]</p>

<p><q>O God, <qex>that</qex> right should thus overcome might!</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ <xex>That</xex> was formerly added to other conjunctions or to adverbs to make them emphatic.<br/
[<source>1913 Webster</source>]</p>

<p><q>To try if <qex>that</qex> our own be ours or no.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><xex>That</xex> is sometimes used to connect a clause with a preceding conjunction on which it depends.<br/
[<source>1913 Webster</source>]</p>

<p><q>When he had carried Rome and <qex>that</qex> we looked<br/
For no less spoil than glory.</q> <rj><qau>Shak.</qau></rj>
</note><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <def>As adverb: To such a degree; so; <as>as, he was <ex>that</ex> frightened he could say nothing</as>.</def> <mark>[Archaic or in illiteral use.]</mark><-- = so --><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>All that</b></col>, <cd>everything of that kind; all that sort.</cd><br/
[<source>1913 Webster</source>]</p>

<p><q>With singing, laughing, ogling, and <qex>all that</qex>.</q> <rj><qau>Pope.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>The rank is but the guinea's stamp,<br/
The man's the gowd [gold] for <qex>a'that</qex>.</q> <rj><qau>Burns.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p>-- <col><b>For that</b></col>. <cd>See under <er>For</er>, <pos>prep.</pos></cd> -- <col><b>In that</b></col>. <cd>See under <er>In</er>, <pos>prep.</pos></cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Thatch</ent><br/
<hw>Thatch</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[OE. <ets>thak</ets>, AS. <ets><thorn/<ae/c</ets> a roof; akin to <ets><thorn/eccean</ets> to cover, D. <ets>dak</ets> a roof, <ets>dekken</ets> to cover, G. <ets>dach</ets> a roof, <ets>decken</ets> 8cover, Icel. <ets><thorn/ak</ets> a roof, Sw. <ets>tak</ets>, Dan. <ets>tag</ets>, Lith. <ets>st<omac/gas</ets>, Ir. <ets>teagh</ets> a house, Gael. <ets>teach</ets>, <ets>tigh</ets>, W. <ets>ty</ets>, L. <ets>tegere</ets> to cover, <ets>toga</ets> a toga, Gr. <?/, <?/, a roof, <?/ to cover, Skr. <ets>sthag</ets>.  Cf. <er>Deck</er>, <er>Integument</er>, <er>Tile</er>, <er>Toga</er>.]</ety> <sn>1.</sn> <def>Straw, rushes, or the like, used for making or covering the roofs of buildings, or of stacks of hay or grain.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Bot.)</fld> <def>A name in the West Indies for several kinds of palm, the leaves of which are used for thatching.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Thatch sparrow</b></col>, <cd>the house sparrow.</cd> <mark>[Prov. Eng.]</mark></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Thatch</ent><br/
<hw>Thatch</hw>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Thatched</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Thatching</conjf>.]</vmorph> <ety>[From <er>Thatch</er>, <pos>n.</pos>: cf. OE. <ets>thecchen</ets>, AS. <ets><?/eccean</ets> to cover.]</ety> <def>To cover with, or with a roof of, straw, reeds, or some similar substance; <as>as, to <ex>thatch</ex> a roof, a stable, or a stack of grain</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Thatcher</ent><br/
<hw>Thatch"er</hw> <pr>(?)</pr>, <pos>n.</pos> <def>One who thatches.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Thatching</ent><br/
<hw>Thatch"ing</hw>, <pos>n.</pos> <sn>1.</sn> <def>The act or art of covering buildings with thatch; so as to keep out rain, snow, etc.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>The materials used for this purpose; thatch.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Thaught</ent><br/
<hw>Thaught</hw> <pr>(th<add/t)</pr>, <pos>n.</pos> <fld>(Naut.)</fld> <def>See <er>Thwart</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Thaumatolatry</ent><br/
<hw>Thau`ma*tol"a*try</hw> <pr>(th<add/`m<adot/*t<ocr/l"<adot/*tr<ycr/)</pr>, <pos>n.</pos> <ety>[Gr. <grk>qay^ma</grk>, <grk>qay`matos</grk>, a wonder + <grk>latrei`a</grk> worship.]</ety> <def>Worship or undue admiration of wonderful or miraculous things.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>The <qex>thaumatolatry</qex> by which our theology has been debased for more than a century.</q> <rj><qau>Hare.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Thaumatrope</ent><br/
<hw>Thau"ma*trope</hw> <pr>(th<add/"m<adot/*tr<omac/p)</pr>, <pos>n.</pos> <ety>[Gr. <grk>qay^ma</grk> a wonder + <grk>tre`pein</grk> to turn.]</ety> <fld>(Opt.)</fld> <def>An optical instrument or toy for showing the persistence of an impression upon the eyes after the luminous object is withdrawn.</def><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ It consists of a card having on its opposite faces figures of two different objects, or halves of the same object, as a bird and a cage, which, when the card is whirled rapidly round a diameter by the strings that hold it, appear to the eye combined in a single picture, as of a bird in its cage.</note><br/
[<source>1913 Webster</source>]</p>

<p><ent>thaumaturge</ent><br/
<hw>thau"ma*turge</hw> <pr>(th<add/"m<adot/*t<ucir/rj)</pr>, <pos>n.</pos> <ety>[See <er>thaumaturgus</er>.]</ety> <def>A magician; a wonder worker.</def>  <rj><au>Lowell.</au></rj><br/
<syn><b>Syn. --</b> thaumaturgist.</syn><br/
[<source>1913 Webster</source>]</p>

<p><ent>thaumaturgical</ent><br/
<ent>thaumaturgic</ent><br/
<mhw>{ <hw>thau`ma*tur"gic</hw> <pr>(th<add/`m<adot/*t<ucir/r"j<icr/k)</pr>, <hw>thau`ma*tur"gic*al</hw> <pr>(th<add/`m<adot/*t<ucir/r"j<icr/*k<ait/l)</pr>, }</mhw> <pos>a.</pos> <def>Of or pertaining to thaumaturgy; magical; wonderful.</def>  <rj><au>Burton.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>thaumaturgics</ent><br/
<hw>thau`ma*tur"gics</hw> <pr>(th<add/`m<adot/*t<ucir/r"j<icr/ks)</pr>, <pos>n.</pos> <def>Feats of legerdemain, or magical performances.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>thaumaturgist</ent><br/
<hw>thau`ma*tur"gist</hw> <pr>(th<add/`m<adot/*t<ucir/r"j<icr/st)</pr>, <pos>n.</pos> <def>One who deals in wonders, or believes in them; a wonder worker; a thaumaturge.</def>  <rj><au>Carlyle.</au></rj><br/
[<source>1913 Webster</source> <source>+PJC</source>]</p>

<p><ent>thaumaturgus</ent><br/
||<hw>thau`ma*tur"gus</hw> <pr>(th<add/`m<adot/*t<ucir/r"g<ucr/s)</pr>, <pos>n.</pos> <ety>[NL., from Gr. <grk>qaymatourgo`s</grk> wonder-working; <grk>qay^ma</grk> a wonder + <grk>'e`rgon</grk> work.]</ety> <def>A miracle worker; -- a title given by the Roman Catholics to some saints.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>thaumaturgy</ent><br/
<hw>thau"ma*tur`gy</hw> <pr>(th<add/`m<adot/*t<ucir/r"j<ycr/)</pr>, <pos>n.</pos> <ety>[Gr. <grk>qaymatourgi`a</grk>.]</ety> <def>The act or art of performing something wonderful; magic; legerdemain.</def>  <rj><au>T. Warton.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>thave</ent><br/
<hw>thave</hw> <pr>(th<amac/v)</pr>, <pos>n.</pos> <def>Same as <er>theave</er>.</def> <mark>[Prov. Eng.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>thaw</ent><br/
<hw>thaw</hw> <pr>(th<add/)</pr>, <pos>v. i.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Thawed</conjf> <pr>(th<add/d)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Thawing</conjf>.]</vmorph> <ety>[AS. <ets><thorn/<amac/wian</ets>, <ets><thorn/<amac/wan</ets>; akin to D. <ets>dovijen</ets>, G. <ets>tauen</ets>, <ets>thauen</ets> (cf. also ver<ets>dauen</ets> to digest, OHG. <ets>douwen</ets>, fir<ets>douwen</ets>), Icel. <ets><thorn/eyja</ets>, Sw. <ets>t<oum/a</ets>, Dan. <ets>t<oum/e</ets>, and perhaps to Gr. <grk>th`kein</grk> to melt. <root/56.]</ety> <sn>1.</sn> <def>To melt, dissolve, or become fluid; to soften; -- said of that which is frozen; <as>as, the ice <ex>thaws</ex></as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To become so warm as to melt ice and snow; -- said in reference to the weather, and used impersonally.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <specif>Fig.:</specif> <def>To grow gentle or genial.  Compare <er>cold</er>{4}, <pos>a.</pos> and <er>hard</er>{6}, <pos>a.</pos></def><br/
[<source>1913 Webster</source> <source>+PJC</source>]</p>

<p><ent>thaw</ent><br/
<hw>thaw</hw>, <pos>v. t.</pos> <def>To cause (frozen things, as earth, snow, ice) to melt, soften, or dissolve.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>thaw</ent><br/
<hw>thaw</hw>, <pos>n.</pos> <def>The melting of ice, snow, or other congealed matter; the resolution of ice, or the like, into the state of a fluid; liquefaction by heat of anything congealed by frost; also, a warmth of weather sufficient to melt that which is congealed.</def>  <rj><au>Dryden.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>thawy</ent><br/
<hw>thaw"y</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Liquefying by heat after having been frozen; thawing; melting.</def> <mark>[archaic]</mark><br/
[<source>1913 Webster</source> <source>+PJC</source>]</p>

<p><ent>The</ent><br/
<hw>The</hw> <pr>(<th/<emac/)</pr>, <pos>v. i.</pos> <def>See <er>Thee</er>.</def> <mark>[Obs.]</mark>  <rj><au>Chaucer.</au>  <au>Milton.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>The</ent><br/
<hw>The</hw> <pr>(<th/<emac/, <it>when emphatic or alone</it>; <th/<esl/, <it>obscure before a vowel</it>; <th/<eit/, <it>obscure before a consonant</it>; 37)</pr>, <pos>definite article.</pos> <ety>[AS. <ets><edh/<emac/</ets>, a later form for earlier nom. sing. masc. <ets>s<emac/</ets>, formed under the influence of the oblique cases.  See <er>That</er>, pron.]</ety> <def>A word placed before nouns to limit or individualize their meaning.</def><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ <ex>The</ex> was originally a demonstrative pronoun, being a weakened form of <ex>that</ex>.  When placed before adjectives and participles, it converts them into abstract nouns; as, <ex>the</ex> sublime and <ex>the</ex> beautiful. <au>Burke.</au>  <ex>The</ex> is used regularly before many proper names, as of rivers, oceans, ships, etc.; as, <ex>the</ex> Nile, <ex>the</ex> Atlantic, <ex>the</ex> Great Eastern, <ex>the</ex> West Indies, <ex>The</ex> Hague.  <ex>The</ex> with an epithet or ordinal number often follows a proper name; as, Alexander <ex>the</ex> Great; Napoleon <ex>the</ex> Third.  <ex>The</ex> may be employed to individualize a particular kind or species; as, <ex>the</ex> grasshopper shall be a burden. <au>Eccl. xii. 5.</au></note><br/
[<source>1913 Webster</source>]</p>

<p><ent>The</ent><br/
<hw>The</hw>, <pos>adv.</pos> <ety>[AS. <ets><edh/<emac/</ets>, <ets><edh/<ymac/</ets>, instrumental case of <ets>s<emac/</ets>, <ets>se<oacute/</ets>, <ets><edh/<ae/t</ets>, the definite article.  See 2d <er>The</er>.]</ety> <def>By that; by how much; by so much; on that account; -- used before comparatives; <as>as, <ex>the</ex> longer we continue in sin, <ex>the</ex> more difficult it is to reform</as>.</def> <ldquo/Yet not <xex>the</xex> more cease I.<rdquo/  <rj><au>Milton.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>So much <qex>the</qex> rather thou, Celestial Light,<br/
Shine inward, and the mind through all her powers<br/
Irradiate.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Thea</ent><br/
||<hw>The"a</hw> <pr>(?)</pr>, <pos>prop. n.</pos> <ety>[NL. See <er>Tea</er>.]</ety> <fld>(Bot.)</fld> <def>A genus of plants found in China and Japan; the tea plant.</def><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ It is now commonly referred to the genus <gen>Camellia</gen>.</note><br/
[<source>1913 Webster</source>]</p>

<p><ent>theandric</ent><br/
<hw>the*an"dric</hw> <pr>(th<esl/*<acr/n"dr<icr/k)</pr>, <pos>a.</pos> <ety>[Gr. <grk>qeandriko`s</grk>; <grk>qeo`s</grk> god + <grk>anh`r</grk>, <grk>andro`s</grk>, a man.]</ety> <def>Relating to, or existing by, the union of divine and human operation in Christ, or the joint agency of the divine and human nature.</def>  <rj><au>Murdock.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Theanthropical</ent><br/
<ent>theanthropic</ent><br/
<mhw>{ <hw>the`an*throp"ic</hw> <pr>(th<emac/`<acr/n*thr<ocr/p"<icr/k)</pr>, <hw>The`an*throp"ic*al</hw> <pr>(th<emac/`<acr/n*thr<ocr/p"<icr/*k<ait/l)</pr>, }</mhw> <pos>a.</pos> <def>Partaking of, or combining, both divinity and humanity.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>The gorgeous and imposing figures of his [Homer's] <qex>theanthropic</qex> sytem.</q> <rj><qau>Gladstone.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Theanthropism</ent><br/
<hw>The*an"thro*pism</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Gr. <?/ god + <?/ man.]</ety> <sn>1.</sn> <def>A state of being God and man.</def> <mark>[R.]</mark>  <rj><au>Coleridge.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>The ascription of human atributes to the Deity, or to a polytheistic deity; anthropomorphism.</def>  <rj><au>Gladstone.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Theanthropist</ent><br/
<hw>The*an"thro*pist</hw> <pr>(?)</pr>, <pos>n.</pos> <def>One who advocates, or believes in, theanthropism.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Theanthropy</ent><br/
<hw>The*an"thro*py</hw> <pr>(?)</pr>, <pos>n.</pos> <def>Theanthropism.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Thearchic</ent><br/
<hw>The*ar"chic</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[Gr. <?/. See <er>Thearchy</er>.]</ety> <def>Divinely sovereign or supreme.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>He [Jesus] is the <qex>thearchic</qex> Intelligence.</q> <rj><qau>Milman.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Thearchy</ent><br/
<hw>The"ar*chy</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Gr. <?/ god + <ets>-archy</ets>: cf. Gr. <?/ the supreme deity.]</ety> <def>Government by God; divine sovereignty; theocracy.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Theatre</ent><br/
<ent>Theater</ent><br/
<mhw>{ <hw>The"a*ter</hw>, <hw>The"a*tre</hw>  }</mhw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F. <ets>th<eacute/<acir/tre</ets>, L. <ets>theatrum</ets>, Gr. <?/, fr. <?/  to see, view; cf. Skr. <ets>dhy<amac/</ets> to meditate, think.  Cf. <er>Theory</er>.]</ety> <sn>1.</sn> <def>An edifice in which dramatic performances or spectacles are exhibited for the amusement of spectators; anciently uncovered, except the stage, but in modern times roofed.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Any room adapted to the exhibition of any performances before an assembly, as public lectures, scholastic exercises, anatomical demonstrations, surgical operations, etc.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>That which resembles a theater in form, use, or the like; a place rising by steps or gradations, like the seats of a theater.</def>  <rj><au>Burns.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Shade above shade, a woody <qex>theater</qex><br/
Of stateliest view.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>A sphere or scheme of operation.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>For if a man can be partaker of God's <qex>theater</qex>, he shall likewise be partaker of God's rest.</q> <rj><qau>Bacon.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <def>A place or region where great events are enacted; <as>as, the <ex>theater</ex> of war</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Theatine</ent><br/
<ent>Theatin</ent><br/
<mhw>{ <hw>The"a*tin</hw>, <hw>The"a*tine</hw>  }</mhw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F. <ets>th<eacute/atin</ets>, It. <ets>theatino</ets>.]</ety> <fld>(R. C. Ch.)</fld> <sn>1.</sn> <def>One of an order of Italian monks, established in 1524, expressly to oppose Reformation, and to raise the tone of piety among Roman Catholics. They hold no property, nor do they beg, but depend on what Providence sends. Their chief employment is preaching and giving religious instruction.</def><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ Their name is derived from <xex>Theate</xex>, or <xex>Chieti</xex>, a city of Naples, the archbishop of which was a principal founder of the order; but they bore various names; as, <xex>Regular Clerks of the Community</xex>, <xex>Pauline Monks</xex>, <xex>Apostolic Clerks</xex>, and <xex>Regular Clerks of the Divine Providence</xex>. The order never flourished much out of Italy.</note><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(R. C. Ch.)</fld> <def>One of an order of nuns founded by <person>Ursula Benincasa</person>, who died in 1618.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Theatral</ent><br/
<hw>The"a*tral</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>theatralis</ets>: cf. F. <ets>th<eacute/atral</ets>.]</ety> <def>Of or pertaining to a theater; theatrical.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Theatric</ent><br/
<hw>The*at"ric</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Theatrical.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Woods over woods in gay, <qex>theatric</qex> pride.</q> <rj><qau>Goldsmith.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Theatrical</ent><br/
<hw>The*at"ric*al</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>theatricus</ets>, Gr. <?/.]</ety> <def>Of or pertaining to a theater, or to the scenic representations; resembling the manner of dramatic performers; histrionic; hence, artificial; <as>as, <ex>theatrical</ex> performances; <ex>theatrical</ex> gestures</as>.</def> -- <wordforms><wf>The*at`ri*cal"i*ty</wf> <pr>(#)</pr>, <pos>n.</pos> -- <wf>The*at"ric*al*ly</wf> <pr>(#)</pr>, <pos>adv.</pos></wordforms><br/
[<source>1913 Webster</source>]</p>

<p><q>No meretricious aid whatever has been called in -- no trick, no illusion of the eye, nothing <qex>theatrical</qex>.</q> <rj><qau>R. Jefferies.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Theatricals</ent><br/
<hw>The*at"ric*als</hw> <pr>(?)</pr>, <pos>n. pl.</pos> <def>Dramatic performances; especially, those produced by amateurs.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Such fashionable cant terms as <lsquo/<qex>theatricals</qex>,' and <lsquo/musicals,' invented by the flippant Topham, still survive among his confraternity of frivolity.</q> <rj><qau>I. Disraeli.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Theave</ent><br/
<hw>Theave</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Cf. W. <ets>dafad</ets> a sheep, ewe.]</ety> <def>A ewe lamb of the first year; also, a sheep three years old.</def> <altsp>[Written also <asp>thave</asp>.]</altsp> <mark>[Prov. Eng.]</mark>  <rj><au>Halliwell.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Thebaic</ent><br/
<hw>The*ba"ic</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>thebaicus</ets>, Gr. <?/.]</ety> <def>Of or pertaining to Thebes in Egypt; specifically, designating a version of the Bible preserved by the Copts, and esteemed of great value by biblical scholars. This version is also called the <xex>Sahidic version</xex>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Thebaid</ent><br/
<hw>The"ba*id</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>Thebais</ets>, <ets>-idis</ets>.]</ety> <def>A Latin epic poem by Statius about Thebes in Boeotia.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Thebaine</ent><br/
<hw>The*ba"ine</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[So called from a kind of Egyptian opium produced at <ets>Thebes</ets>.]</ety> <fld>(Chem.)</fld> <def>A poisonous alkaloid, <chform>C19H21NO3</chform>, found in opium in small quantities, having a sharp, astringent taste, and a tetanic action resembling that of strychnine.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Theban</ent><br/
<hw>The"ban</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>Thebanus</ets>.]</ety> <def>Of or pertaining to Thebes.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Theban year</b></col> <fld>(Anc. Chron.)</fld>, <cd>the Egyptian year of 365 days and 6 hours.</cd> <au>J. Bryant.</au></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Theban</ent><br/
<hw>The"ban</hw>, <pos>n.</pos> <def>A native or inhabitant of Thebes; also, a wise man.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>I'll talk a word with this same learned <qex>Theban</qex>.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Theca</ent><br/
||<hw>The"ca</hw> <pr>(?)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Thecae</plw> <pr>(#)</pr>.</plu> <ety>[L., fr. Gr. <?/ a case to put anything in. See <er>Tick</er> a cover.]</ety> <sn>1.</sn> <def>A sheath; a case; <as>as, the <ex>theca</ex>, or cell, of an anther; the <ex>theca</ex>, or spore case, of a fungus; the <ex>theca</ex> of the spinal cord.</as></def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Zool.)</fld> <sd>(a)</sd> <def>The chitinous cup which protects the hydranths of certain hydroids.</def> <sd>(b)</sd> <def>The more or less cuplike calicle of a coral.</def> <sd>(c)</sd> <def>The wall forming a calicle of a coral.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Thecal</ent><br/
<hw>The"cal</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Of or pertaining to a theca; <as>as, a <ex>thecal</ex> abscess</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Thecaphore</ent><br/
<hw>The"ca*phore</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[<ets>Theca</ets> + Gr. <grk>fe`rein</grk> to bear: cf. F. <ets>th<eacute/caphore</ets>.]</ety> <fld>(Bot.)</fld> <sd>(a)</sd> <def>A surface or organ bearing a theca, or covered with thecae.</def> <sd>(b)</sd> <def>See <er>Basigynium</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Thecasporous</ent><br/
<hw>The*cas"po*rous</hw> <pr>(?)</pr>, <pos>a.</pos> <fld>(Bot.)</fld> <def>Having the spores in thecae, or cases.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Thecata</ent><br/
||<hw>The*ca"ta</hw> <pr>(?)</pr>, <pos>n. pl.</pos> <ety>[NL., from Gr. <?/ a case.]</ety> <fld>(Zool.)</fld> <def>Same as <er>Thecophora</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Thecla</ent><br/
<hw>Thec"la</hw> <pr>(?)</pr>, <pos>n.</pos> <def>Any one of many species of small delicately colored butterflies belonging to <gen>Thecla</gen> and allied genera; -- called also <altname>hairstreak</altname>, and <altname>elfin</altname>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Thecodactyl</ent><br/
<hw>The`co*dac"tyl</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[<?/ case + <?/ finger.]</ety> <fld>(Zool.)</fld> <def>Any one of a group of lizards of the Gecko tribe, having the toes broad, and furnished with a groove in which the claws can be concealed.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Thecodont</ent><br/
<hw>The"co*dont</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[Gr. <?/ a case + <?/, <?/, a tooth.]</ety> <sn>1.</sn> <fld>(Anat.)</fld> <def>Having the teeth inserted in sockets in the alveoli of the jaws.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Paleon.)</fld> <def>Of or pertaining to the thecodonts.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Thecodont</ent><br/
<hw>The"co*dont</hw>, <pos>n.</pos> <fld>(Paleon.)</fld> <def>One of the Thecodontia.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Thecodontia</ent><br/
||<hw>The`co*don"ti*a</hw> <pr>(?)</pr>, <pos>n. pl.</pos> <ety>[NL.]</ety> <fld>(Paleon.)</fld> <def>A group of fossil saurians having biconcave vertebrae and the teeth implanted in sockets.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Thecophora</ent><br/
||<hw>The*coph"o*ra</hw> <pr>(?)</pr>, <pos>n. pl.</pos> <ety>[NL., from Gr. <?/ a case + <grk>fe`rein</grk> to bear.]</ety> <fld>(Zool.)</fld> <def>A division of hydroids comprising those which have the hydranths in thecae and the gonophores in capsules. The campanularians and sertularians are examples. Called also <altname>Thecata</altname>. See <xex>Illust.</xex> under <er>Hydroidea</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Thecosomata</ent><br/
||<hw>The`co*so"ma*ta</hw> <pr>(?)</pr>, <pos>n. pl.</pos> <ety>[NL. See <er>Theca</er>, and <er>Soma</er>.]</ety> <fld>(Zool.)</fld> <def>An order of Pteropoda comprising those species which have a shell. See <er>Pteropoda</er>.</def> -- <wordforms><wf>The`co*so"ma*tous</wf> <pr>(#)</pr>, <pos>a.</pos></wordforms><br/
[<source>1913 Webster</source>]</p>

<p><ent>Thedom</ent><br/
<hw>The"dom</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[<ets>Thee</ets> to prosper + <ets>-dom</ets>.]</ety> <def>Success; fortune; luck; chance.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>Evil <qex>thedom</qex> on his monk's snout.</q> <rj><qau>Chaucer.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Thee</ent><br/
<hw>Thee</hw> <pr>(th<emac/)</pr>, <pos>v. i.</pos> <ety>[AS. <ets><thorn/e<oacute/n</ets>; akin to OS. <ets>th<imac/han</ets>, D. ge<ets>dijen</ets>, G. ge<ets>deihen</ets>, OHG. gi<ets>dihan</ets>, Goth. <ets><thorn/eihan</ets>, Lith. <ets>tekti</ets> to fall to the lot of.  Cf. <er>Tight</er>, <pos>a.</pos>]</ety> <def>To thrive; to prosper.</def> <mark>[Obs.]</mark> <ldquo/He shall never <xex>thee</xex>.<rdquo/ <au>Chaucer.</au><br/
[<source>1913 Webster</source>]</p>

<p><q>Well mote <qex>thee</qex>, as well can wish your thought.</q> <rj><qau>Spenser.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Thee</ent><br/
<hw>Thee</hw> <pr>(<th/<emac/)</pr>, <pos>pron.</pos> <ety>[AS. <ets><edh/<emac/</ets>, acc. & dat. of <ets><edh/<umac/</ets> thou. See <er>Thou</er>.]</ety> <def>The objective case of <xex>thou</xex>. See <er>Thou</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ Thee is poetically used for <xex>thyself</xex>, as <xex>him</xex> for <xex>himself</xex>, etc.</note><br/
[<source>1913 Webster</source>]</p>

<p><q>This sword hath ended him; so shall it thee,<br/
Unless thou yield <qex>thee</qex> as my prisoner.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><-- p. 1495 --><br/
[<source>1913 Webster</source>]</p>

<p><ent>Theft</ent><br/
<hw>Theft</hw> <pr>(th<ecr/ft)</pr>, <pos>n.</pos> <ety>[OE. <ets>thefte</ets>, AS. <ets><thorn/i<eacute/f<edh/e</ets>, <ets><thorn/<ymac/f<edh/e</ets>, <ets><thorn/e<oacute/f<edh/e</ets>. See <er>Thief</er>.]</ety> <sn>1.</sn> <fld>(Law)</fld> <def>The act of stealing; specifically, the felonious taking and removing of personal property, with an intent to deprive the rightful owner of the same; larceny.</def><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ To constitute theft there must be a taking without the owner's consent, and it must be unlawful or felonious; every part of the property stolen must be removed, however slightly, from its former position; and it must be, at least momentarily, in the complete possession of the thief. See <er>Larceny</er>, and the Note under <er>Robbery</er>.</note><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>The thing stolen.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>If the <qex>theft</qex> be certainly found in his hand alive, . . . he shall restore double.</q> <rj><qau>Ex. xxii. 4.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Theftbote</ent><br/
<hw>Theft"bote`</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[<ets>Theft</ets> + <ets>bote</ets> compensation.]</ety> <fld>(Law)</fld> <def>The receiving of a man's goods again from a thief, or a compensation for them, by way of composition, with the intent that the thief shall escape punishment.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Thegn</ent><br/
<hw>Thegn</hw> <pr>(?)</pr>, <pos>n.</pos> <def>Thane. See <er>Thane</er>.</def>  <rj><au>E. A. Freeman.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Thegnhood</ent><br/
<hw>Thegn"hood</hw> <pr>(?)</pr>, <pos>n.</pos> <def>Thanehood.</def>  <rj><au>E. A. Freeman.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Theiform</ent><br/
<hw>The"i*form</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[NL. <ets>thea</ets> tea, the tea plant + <ets>-form</ets>: cf. F. <ets>th<eacute/iforme</ets>.]</ety> <def>Having the form of tea.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Theine</ent><br/
<hw>The"ine</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F. <ets>th<eacute/ine</ets>, fr. NL. <ets>thea</ets>. See <er>Theiform</er>.]</ety> <fld>(Chem.)</fld> <def>See <er>Caffeine</er>. Called also <altname>theina</altname>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Their</ent><br/
<hw>Their</hw> <pr>(?)</pr>, <pos>pron. & a.</pos> <ety>[OE. <ets>thair</ets>, fr. Icel. <ets><thorn/eirra</ets>, <ets><thorn/eira</ets>, of them, but properly gen. pl. of the definite article; akin to AS. <ets><edh/<amac/ra</ets>, <ets><edh/<aemac/ra</ets>, gen. pl. of the definite article, or fr. AS. <ets><edh/<aemac/ra</ets>, influenced by the Scandinavian use. See <er>That</er>.]</ety> <def>The possessive case of the personal pronoun <xex>they</xex>; <as>as, <ex>their</ex> houses; <ex>their</ex> country</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ The possessive takes the form <xex>theirs</xex> (<?/) when the noun to which it refers is not expressed, but implied or understood; as, our land is richest, but <xex>theirs</xex> is best cultivated.</note><br/
[<source>1913 Webster</source>]</p>

<p><q>Nothing but the name of zeal appears<br/
'Twixt our best actions and the worst of <qex>theirs</qex>.</q> <rj><qau>Denham.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Theism</ent><br/
<hw>The"ism</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[From Gr. <?/ God; probably akin to <?/ to pray for, <?/ spoken by God, decreed: cf. F. <ets>th<eacute/isme</ets>.  Cf. <er>Enthusiasm</er>, <er>Pantheon</er>, <er>Theology</er>.]</ety> <def>The belief or acknowledgment of the existence of a God, as opposed to <contr>atheism</contr>, <contr>pantheism</contr>, or <contr>polytheism</contr>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Theism</ent><br/
<hw>The"ism</hw> <pr>(th<emac/"<icr/z'm)</pr>, <pos>n.</pos> <ety>[NL. & E. <ets>thea</ets> tea + <ets>-ism</ets>.]</ety> <fld>(Med.)</fld> <def>The morbid condition resulting from the excessive use of tea.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Theist</ent><br/
<hw>The"ist</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Cf. F. <ets>th<eacute/iste</ets>. See <er>Theism</er>.]</ety> <def>One who believes in the existence of a God; especially, one who believes in a personal God; -- opposed to <contr>atheist</contr>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Theistical</ent><br/
<ent>Theistic</ent><br/
<mhw>{ <hw>The*is"tic</hw> <pr>(?)</pr>, <hw>The*is"tic*al</hw> <pr>(?)</pr>, }</mhw> <pos>a.</pos> <def>Of or pertaining to theism, or a theist; according to the doctrine of theists.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Thelphusian</ent><br/
<hw>Thel*phu"si*an</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Gr. <?/ nipple + <?/ to blow, to puff.]</ety> <fld>(Zool.)</fld> <def>One of a tribe of fresh-water crabs which live in or on the banks of rivers in tropical countries.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Thelytokous</ent><br/
<hw>The*lyt"o*kous</hw> <pr>(th<esl/*l<icr/t"<osl/*k<ucr/s)</pr>, <pos>a.</pos> <ety>[Gr. <grk>qh^lys</grk> female + <grk>to`kos</grk> a bringing forth.]</ety> <fld>(Zool.)</fld> <def>Producing females only; -- said of certain female insects.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Them</ent><br/
<hw>Them</hw> <pr>(<th/<ecr/m)</pr>, <pos>pron.</pos> <ety>[AS. <ets><edh/<aemac/m</ets>, dat. pl. of the article, but influenced by the Scand. use of the corresponding form <ets><thorn/eim</ets> as a personal pronoun. See <er>They</er>.]</ety> <def>The objective case of they. See <er>They</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Go ye rather to <qex>them</qex> that sell, and buy for yourselves.</q> <rj><qau>Matt. xxv. 9.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Then shall the King say unto <qex>them</qex> on his right hand, Come, ye blessed of my Father.</q> <rj><qau>Matt. xxv. 34.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ <xex>Them</xex> is poetically used for <xex>themselves</xex>, as <xex>him</xex> for <xex>himself</xex>, etc.</note><br/
[<source>1913 Webster</source>]</p>

<p><q>Little stars may hide <qex>them</qex> when they list.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Thematic</ent><br/
<hw>The*mat"ic</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[Gr. <?/: cf. F. <ets>th<eacute/matique</ets>.]</ety> <sn>1.</sn> <fld>(Gram.)</fld> <def>Of or pertaining to the theme of a word. See <er>Theme</er>, <pos>n.</pos>, 4.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Mus.)</fld> <def>Of or pertaining to a theme, or subject.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Thematic catalogue</b></col> <fld>(Mus.)</fld>, <cd>a catalogue of musical works which, besides the title and other particulars, gives in notes the theme, or first few measures, of the whole work or of its several movements.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Theme</ent><br/
<hw>Theme</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[OE. <ets>teme</ets>, OF. <ets>teme</ets>, F. <ets>th<egrave/me</ets>, L. <ets>thema</ets>, Gr. <?/, fr. <?/ to set, place. See <er>Do</er>, and cf. <er>Thesis</er>.]</ety> <sn>1.</sn> <def>A subject or topic on which a person writes or speaks; a proposition for discussion or argument; a text.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>My <qex>theme</qex> is alway one and ever was.</q> <rj><qau>Chaucer.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>And when a soldier was the <qex>theme</qex>, my name<br/
Was not far off.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Discourse on a certain subject.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Then ran repentance and rehearsed his <qex>theme</qex>.</q> <rj><qau>Piers Plowman.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>It was the subject of my <qex>theme</qex>.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>A composition or essay required of a pupil.</def>  <rj><au>Locke.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <fld>(Gram.)</fld> <def>A noun or verb, not modified by inflections; also, that part of a noun or verb which remains unchanged (except by euphonic variations) in declension or conjugation; stem.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <def>That by means of which a thing is done; means; instrument.</def> <mark>[Obs.]</mark>  <rj><au>Swift.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>6.</sn> <fld>(Mus.)</fld> <def>The leading subject of a composition or a movement.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Themis</ent><br/
<hw>The"mis</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L., fr. Gr. <?/, fr. <?/ that which is laid down or established by usage, law, prob. fr. <?/ to set, place.]</ety> <fld>(Gr. Myth.)</fld> <def>The goddess of law and order; the patroness of existing rights.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Themselves</ent><br/
<hw>Them*selves"</hw> <pr>(?)</pr>, <pos>pron.</pos> <def>The plural of <xex>himself</xex>, <xex>herself</xex>, and <xex>itself</xex>. See <er>Himself</er>, <er>Herself</er>, <er>Itself</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Then</ent><br/
<hw>Then</hw> <pr>(<th/<ecr/n)</pr>, <pos>adv.</pos> <ety>[Originally the same word as <ets>than</ets>. See <er>Than</er>.]</ety> <sn>1.</sn> <def>At that time (referring to a time specified, either past or future).</def><br/
[<source>1913 Webster</source>]</p>

<p><q>And the Canaanite was <qex>then</qex> in the land.</q> <rj><qau>Gen. xii. 6.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Now I know in part; but <qex>then</qex> shall I know even as also I am known.</q> <rj><qau>1 Cor. xiii. 12.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Soon afterward, or immediately; next; afterward.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>First be reconciled to thy brother, and <qex>then</qex> come and offer thy gift.</q> <rj><qau>Matt. v. 24.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>At another time; later; again.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>One while the master is not aware of what is done, and <qex>then</qex> in other cases it may fall out to be own act.</q> <rj><qau>L'Estrange.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>By then</b></col>. <sd>(a)</sd> <cd>By that time.</cd> <sd>(b)</sd> <cd>By the time that.</cd> <mark>[Obs.]</mark></cs><br/
[<source>1913 Webster</source>]</p>

<p><q>But that opinion, I trust, <qex>by then</qex> this following argument hath been well read, will be left for one of the mysteries of an indulgent Antichrist.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Now and then</b></col>. <cd>See under <er>Now</er>, <pos>adv.</pos></cd> -- <col><b>Till then</b></col>, <cd>until that time; until the time mentioned.</cd> <au>Milton.</au></cs><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ <xex>Then</xex> is often used elliptically, like an adjective, for <xex>then existing</xex>; as, the <xex>then</xex> administration.</note><br/
[<source>1913 Webster</source>]</p>

<p><ent>Then</ent><br/
<hw>Then</hw> <pr>(?)</pr>, <pos>conj.</pos> <sn>1.</sn> <def>Than.</def> <mark>[Obs.]</mark>  <rj><au>Spenser.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>In that case; in consequence; as a consequence; therefore; for this reason.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>If all this be so, <qex>then</qex> man has a natural freedom.</q> <rj><qau>Locke.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Now, <qex>then</qex>, be all thy weighty cares away.</q> <rj><qau>Dryden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><syn><b>Syn.</b> -- Therefore.</syn> <usage> <er>Then</er>, <er>Therefore</er>. Both these words are used in reasoning; but <xex>therefore</xex> takes the lead, while <xex>then</xex> is rather subordinate or incidental. <xex>Therefore</xex> states reasons and draws inferences in form; <xex>then</xex>, to a great extent, takes the point as proved, and passes on to the general conclusion. <ldquo/<xex>Therefore</xex> being justified by faith, we have peace with God.<rdquo/ <au>Rom. v. 1.</au> <ldquo/So <xex>then</xex> faith cometh by hearing, and hearing by the word of God.<rdquo/ <au>Rom. x. 17.</au></usage><br/
[<source>1913 Webster</source>]</p>

<p><ent>Thenadays</ent><br/
<hw>Then"a*days</hw> <pr>(?)</pr>, <pos>adv.</pos> <def>At that time; then; in those days; -- correlative to <xex>nowadays</xex>.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Thenar</ent><br/
<ent>Thenal</ent><br/
<mhw>{ <hw>The"nal</hw> <pr>(?)</pr>, <hw>The"nar</hw> <pr>(?)</pr>, }</mhw> <pos>a.</pos> <ety>[NL., fr. Gr. <?/.]</ety> <fld>(Anat.)</fld> <def>Of or pertaining to the thenar; corresponding to thenar; palmar.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Thenar</ent><br/
<hw>The"nar</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Anat.)</fld> <sd>(a)</sd> <def>The palm of the hand.</def> <sd>(b)</sd> <def>The prominence of the palm above the base of the thumb; the thenar eminence; the ball of the thumb. Sometimes applied to the corresponding part of the foot.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Thenardite</ent><br/
<hw>The*nard"ite</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Named after the French chemist, L. J. <etsep>Th<eacute/nard</etsep>.]</ety> <fld>(Min.)</fld> <def>Anhydrous sodium sulphate, a mineral of a white or brown color and vitreous luster.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Thence</ent><br/
<hw>Thence</hw> <pr>(?)</pr>, <pos>adv.</pos> <ety>[OE. <ets>thenne</ets>, <ets>thanne</ets>, and (with the adverbal <ets>-s</ets>; see <er>-wards</er>) <ets>thennes</ets>, <ets>thannes</ets> (hence <ets>thens</ets>, now written <ets>thence</ets>), AS. <ets><edh/anon</ets>, <ets><edh/anan</ets>, <ets><edh/onan</ets>; akin to OHG. <ets>dannana</ets>, <ets>dann<amac/n</ets>, <ets>dan<amac/n</ets>, and G. von <ets>dannen</ets>, E. <ets>that</ets>, <ets>there</ets>. See <er>That</er>.]</ety> <sn>1.</sn> <def>From that place.</def> <ldquo/Bid him <xex>thence</xex> go.<rdquo/  <rj><au>Chaucer.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>When ye depart <qex>thence</qex>, shake off the dust under your feet for a testimony against them.</q> <rj><qau>Mark vi. 11.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ It is not unusual, though pleonastic, to use <xex>from</xex> before <xex>thence</xex>.  Cf. <er>Hence</er>, <er>Whence</er>.</note><br/
[<source>1913 Webster</source>]</p>

<p><q>Then I will send, and fetch thee <qex>from thence</qex>.</q> <rj><qau>Gen. xxvii. 45.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>From that time; thenceforth; thereafter.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>There shall be no more <qex>thence</qex> an infant of days.</q> <rj><qau>Isa. lxv. 20.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>For that reason; therefore.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Not to sit idle with so great a gift<br/
Useless, and <qex>thence</qex> ridiculous, about him.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>Not there; elsewhere; absent.</def> <mark>[Poetic]</mark>  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Thenceforth</ent><br/
<hw>Thence`forth"</hw> <pr>(?)</pr>, <pos>adv.</pos> <def>From that time; thereafter.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>If the salt have lost his savor, wherewith shall it be salted? it is <qex>thenceforth</qex> good for nothing.</q> <rj><qau>Matt. v. 13.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><note>    This word is sometimes preceded by <xex>from</xex>, -- a redundancy sanctioned by custom.  <rj><au>Chaucer.</au> <au>John. xix. 12.</au></rj>
</note><br/
[<source>1913 Webster</source>]</p>

<p><ent>Thenceforward</ent><br/
<hw>Thence`for"ward</hw> <pr>(?)</pr>, <pos>adv.</pos> <def>From that time onward; thenceforth.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Thencefrom</ent><br/
<hw>Thence`from"</hw> <pr>(?)</pr>, <pos>adv.</pos> <def>From that place.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Theobroma</ent><br/
<hw>The`o*bro"ma</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[NL., fr. Gr. <?/ a god + <?/ food, fr. <?/ to eat: cf. F. <ets>th<eacute/obrome</ets>.]</ety> <fld>(Bot.)</fld> <def>A genus of small trees. See <er>Cacao</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Theobromic</ent><br/
<hw>The`o*bro"mic</hw> <pr>(?)</pr>, <pos>a.</pos> <fld>(Chem.)</fld> <def>Of, pertaining to, or designating, an acid extracted from cacao butter (from the <xex>Theobroma Cacao</xex>), peanut oil (from <xex>Arachis hypogaea</xex>), etc., as a white waxy crystalline substance.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Theobromine</ent><br/
<hw>The`o*bro"mine</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Chem.)</fld> <def>An alkaloidal ureide, <chform>C7H8N4O2</chform>, homologous with and resembling caffeine, produced artificially, and also extracted from cacao and chocolate (from <spn>Theobroma Cacao</spn>) as a bitter white crystalline substance; -- called also <altname>dimethyl xanthine</altname>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Theochristic</ent><br/
<hw>The`o*chris"tic</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[Gr. <?/; <?/ God + <?/ anointed, fr. <?/ to anoint.]</ety> <def>Anointed by God.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Theocracy</ent><br/
<hw>The*oc"ra*cy</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Gr. <?/; <?/ God + <?/ to be strong, to rule, fr. <?/ strength: cf. F. <ets>th<eacute/ocratie</ets>. See <er>Theism</er>, and cf. <er>Democracy</er>.]</ety> <sn>1.</sn> <def>Government of a state by the immediate direction or administration of God; hence, the exercise of political authority by priests as representing the Deity.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>The state thus governed, as the Hebrew commonwealth before it became a kingdom.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Theocrasy</ent><br/
<hw>The*oc"ra*sy</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Gr. <?/ union of the soul with God; <?/ God + <?/ a mixing, akin to <?/ to mix.]</ety> <sn>1.</sn> <def>A mixture of the worship of different gods, as of Jehovah and idols.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>This syncretistic <qex>theocracy</qex> by no means excludes in him [Solomon] the proper service of idols.</q> <rj><qau>J. Murphy.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Philos.)</fld> <def>An intimate union of the soul with God in contemplation, -- an ideal of the Neoplatonists and of some Oriental mystics.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Theocrat</ent><br/
<hw>The"o*crat</hw> <pr>(?)</pr>, <pos>n.</pos> <def>One who lives under a theocratic form of government; one who in civil affairs conforms to divine law.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Theocratical</ent><br/
<ent>Theocratic</ent><br/
<mhw>{ <hw>The`o*crat"ic</hw> <pr>(?)</pr>, <hw>The`o*crat"ic*al</hw> <pr>(?)</pr>, }</mhw> <pos>a.</pos> <ety>[Cf. F. <ets>th<eacute/ocratique</ets>.]</ety> <def>Of or pertaining to a theocracy; administred by the immediate direction of God; <as>as, the <ex>theocratical</ex> state of the Israelites</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Theodicy</ent><br/
<hw>The*od"i*cy</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[NL. <ets>theodicaea</ets>, fr. Gr. <?/ God + <?/ right, justice: cf. F. <ets>th<eacute/odic<eacute/e</ets>.]</ety> <sn>1.</sn> <def>A vindication of the justice of God in ordaining or permitting natural and moral evil.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>That department of philosophy which treats of the being, perfections, and government of God, and the immortality of the soul.</def>  <rj><au>Krauth-Fleming.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Theodolite</ent><br/
<hw>The*od"o*lite</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Probably a corruption of <ets>the alidade</ets>. See <er>Alidade</er>.]</ety> <def>An instrument used, especially in trigonometrical surveying, for the accurate measurement of horizontal angles, and also usually of vertical angles. It is variously constructed.</def><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ The theodolite consists principally of a telescope, with cross wires in the focus of its object glass, clamped in Y's attached to a frame that is mounted so as to turn both on vertical and horizontal axes, the former carrying a vernier plate on a horizontal graduated plate or circle for azimuthal angles, and the latter a vertical graduated arc or semicircle for altitudes. The whole is furnished with levels and adjusting screws and mounted on a tripod.</note><br/
[<source>1913 Webster</source>]</p>

<p><ent>Theodolitic</ent><br/
<hw>The*od`o*lit"ic</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Of or pertaining to a theodolite; made by means of a theodolite; <as>as, <ex>theodolitic</ex> observations</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Theogonic</ent><br/
<hw>The`o*gon"ic</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Of or relating to theogony.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Theogonism</ent><br/
<hw>The*og"o*nism</hw> <pr>(?)</pr>, <pos>n.</pos> <def>Theogony.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Theogonist</ent><br/
<hw>The*og"o*nist</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A writer on theogony.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Theogony</ent><br/
<hw>The*og"o*ny</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>theogonia</ets>, Gr. <?/; <?/ a god + the root of <?/ to be born. See <er>Theism</er>, and <er>Genus</er>.]</ety> <def>The generation or genealogy of the gods; that branch of heathen theology which deals with the origin and descent of the deities; also, a poem treating of such genealogies; <as>as, the <ex>Theogony</ex> of Hesiod</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Theologaster</ent><br/
<hw>The*ol"o*gas`ter</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Formed like <ets>poetaster</ets>: cf. F. <ets>th<eacute/ologastre</ets>.]</ety> <def>A pretender or quack in theology.</def> <mark>[R.]</mark>  <rj><au>Burton.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Theologer</ent><br/
<hw>The*ol"o*ger</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A theologian.</def>  <rj><au>Cudworth.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Theologian</ent><br/
<hw>The`o*lo"gi*an</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Cf. F. <ets>th<eacute/ologien</ets>, L. <ets>theologus</ets>, Gr. <?/. See <er>Theology</er>.]</ety> <def>A person well versed in theology; a professor of theology or divinity; a divine.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Theologic</ent><br/
<hw>The`o*log"ic</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Theological.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Theological</ent><br/
<hw>The`o*log"ic*al</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>theologicus</ets>, Gr. <?/: cf. F. <ets>th<eacute/ologique</ets>.]</ety> <def>Of or pertaining to theology, or the science of God and of divine things; <as>as, a <ex>theological</ex> treatise</as>.</def> -- <wordforms><wf>The`o*log"ic*al*ly</wf>, <pos>adv.</pos></wordforms><br/
[<source>1913 Webster</source>]</p>

<p><ent>Theologics</ent><br/
<hw>The`o*log"ics</hw> <pr>(?)</pr>, <pos>n.</pos> <def>Theology.</def>  <rj><au>Young.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Theologist</ent><br/
<hw>The*ol"o*gist</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A theologian.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Theologize</ent><br/
<hw>The*ol"o*gize</hw> <pr>(?)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Theologized</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Theologizing</conjf> <pr>(?)</pr>.]</vmorph> <ety>[Cf. F. <ets>th<eacute/ologiser</ets>.]</ety> <def>To render theological; to apply to divinity; to reduce to a system of theology.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>School divinity was but Aristotle's philosophy <qex>theologized</qex>.</q> <rj><qau>Glanvill.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Theologize</ent><br/
<hw>The*ol"o*gize</hw>, <pos>v. i.</pos> <def>To frame a system of theology; to theorize or speculate upon theological subjects.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Theologizer</ent><br/
<hw>The*ol"o*gi`zer</hw> <pr>(?)</pr>, <pos>n.</pos> <def>One who theologizes; a theologian.</def> <mark>[R.]</mark>  <rj><au>Boyle.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Theologue</ent><br/
<hw>The"o*logue</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Cf. L. <ets>theologus</ets>, Gr. <?/, and E. <ets>philologue</ets>.]</ety> <sn>1.</sn> <def>A theologian.</def>  <rj><au>Dryden.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Ye gentle <qex>theologues</qex> of calmer kind.</q> <rj><qau>Young.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>He [Jerome] was the <qex>theologue</qex> -- and the word is designation enough.</q> <rj><qau>I. Taylor.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A student in a theological seminary.</def> <altsp>[Written also <asp>theolog</asp>.]</altsp> <mark>[Colloq. U. S.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Theology</ent><br/
<hw>The*ol"o*gy</hw> <pr>(?)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Theologies</plw> <pr>(#)</pr>.</plu> <ety>[L. <ets>theologia</ets>, Gr. <?/; <?/ God + <?/ discourse: cf. F. <ets>th<eacute/ologie</ets>. See <er>Theism</er>, and <er>Logic</er>.]</ety> <def>The science of God or of religion; the science which treats of the existence, character, and attributes of God, his laws and government, the doctrines we are to believe, and the duties we are to practice; divinity; (as more commonly understood) <ldquo/the knowledge derivable from the Scriptures, the systematic exhibition of revealed truth, the science of Christian faith and life.<rdquo/</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Many speak of <qex>theology</qex> as a science of religion [instead of <ldquo/science of God<rdquo/] because they disbelieve that there is any knowledge of God to be attained.</q> <rj><qau>Prof. R. Flint (Enc. Brit.).</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q><qex>Theology</qex> is ordered knowledge; representing in the region of the intellect what religion represents in the heart and life of man.</q> <rj><qau>Gladstone.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><mcol><col><b>Ascetic theology</b></col>, <col><b>Natural theology</b></col></mcol>. <cd>See <er>Ascetic</er>, <er>Natural</er>.</cd> -- <col><b>Moral theology</b></col>, <cd>that phase of theology which is concerned with moral character and conduct.</cd> -- <col><b>Revealed theology</b></col>, <cd>theology which is to be learned only from revelation.</cd> -- <col><b>Scholastic theology</b></col>, <cd>theology as taught by the scholastics, or as prosecuted after their principles and methods.</cd> -- <col><b>Speculative theology</b></col>, <cd>theology as founded upon, or influenced by, speculation or metaphysical philosophy.</cd> -- <col><b>Systematic theology</b></col>, <cd>that branch of theology of which the aim is to reduce all revealed truth to a series of statements that together shall constitute an organized whole.</cd> <au>E. G. Robinson (Johnson's Cyc.).</au></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Theomachist</ent><br/
<hw>The*om"a*chist</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Cf. Gr. <?/.]</ety> <def>One who fights against the gods; one who resists God of the divine will.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Theomachy</ent><br/
<hw>The*om"a*chy</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Gr. <?/; <?/ a god + <?/ a battle.]</ety> <sn>1.</sn> <def>A fighting against the gods, as the battle of the gaints with the gods.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A battle or strife among the gods.</def>  <rj><au>Gladstone.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>Opposition to God or the divine will.</def>  <rj><au>Bacon.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Theomancy</ent><br/
<hw>The"o*man`cy</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Gr. <?/ a god + <ets>-mancy</ets>: cf. F. <ets>th<eacute/omancie</ets>, Gr. <?/ a spirit of prophecy,.]</ety> <def>A kind of divination drawn from the responses of oracles among heathen nations.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Theopathic</ent><br/
<ent>Theopathetic</ent><br/
<mhw>{ <hw>The`o*pa*thet"ic</hw> <pr>(?)</pr>, <hw>The`o*path"ic</hw> <pr>(?)</pr>, }</mhw> <pos>a.</pos> <def>Of or pertaining to a theopathy.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Theopathy</ent><br/
<hw>The*op"a*thy</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Gr. <?/ God + <?/, <?/, to suffer, feel.]</ety> <def>Capacity for religious affections or worship.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Theophanic</ent><br/
<hw>The`o*phan"ic</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Of or pertaining to a theopany; appearing to man, as a god.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Theophany</ent><br/
<hw>The*oph"a*ny</hw> <pr>(?)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>-nies</plw> <pr>(#)</pr>.</plu> <ety>[Gr. <?/; <?/ God + <?/ to appear.]</ety> <def>A manifestation of God to man by actual appearance, usually as an incarnation.</def><br/
[<source>1913 Webster</source>]</p>

<p><-- p. 1496 --><br/
[<source>1913 Webster</source>]</p>

<p><ent>Theophilanthropic</ent><br/
<hw>The`o*phil`an*throp"ic</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Pertaining to theophilanthropy or the theophilanthropists.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Theophilanthropism</ent><br/
<hw>The`o*phi*lan"thro*pism</hw> <pr>(?)</pr>, <pos>n.</pos> <def>The doctrine of the theophilanthropists; theophilanthropy.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Theophilanthropist</ent><br/
<hw>The`o*phi*lan"thro*pist</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Cf. F. <ets>th<eacute/ophilanthrope</ets>.]</ety> <fld>(Eccl. Hist.)</fld> <def>A member of a deistical society established at Paris during the French revolution.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Theophilanthropy</ent><br/
<hw>The`o*phi*lan"thro*py</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Gr. <?/ God + E. <ets>philanthropy</ets>.]</ety> <def>Theophilanthropism.</def>  <rj><au>Macaulay.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Theophilosophic</ent><br/
<hw>The`o*phil`o*soph"ic</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[Gr. <?/ God + E. <ets>philosophic</ets>.]</ety> <def>Combining theism and philosophy, or pertaining to the combination of theism and philosophy.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Theopneusted</ent><br/
<hw>The`op*neus"ted</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Divinely inspired; theopneustic.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Theopneustic</ent><br/
<hw>The`op*neus"tic</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[Gr. <?/ inspired of God; <?/ God + <?/ to blow, to breathe.]</ety> <def>Given by the inspiration of the Spirit of God.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Theopneusty</ent><br/
<hw>The"op*neus`ty</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Gr. <?/.]</ety> <def>Divine inspiration; the supernatural influence of the Divine Spirit in qualifying men to receive and communicate revealed truth.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Theorbist</ent><br/
<hw>The*or"bist</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Mus.)</fld> <def>One who plays on a theorbo.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Theorbo</ent><br/
<hw>The*or"bo</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F. <ets>th<eacute/orbe</ets>, <ets>t<eacute/orbe</ets>, formerly <ets>tuorbe</ets>, <ets>tiorbe</ets>, It. <ets>tiorba</ets>.]</ety> <fld>(Mus.)</fld> <def>An instrument made like large lute, but having two necks, with two sets of pegs, the lower set holding the strings governed by frets, while to the upper set were attached the long bass strings used as open notes.</def><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ A larger form of <xex>theorbo</xex> was also called the <altname>archlute</altname>, and was used chiefly, if not only, as an accompaniment to the voice. Both have long fallen into disuse.</note><br/
[<source>1913 Webster</source>]</p>

<p><ent>Theorem</ent><br/
<hw>The"o*rem</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>theorema</ets>, Gr. <?/ a sight, speculation, theory, theorem, fr. <?/ to look at, <?/ a spectator: cf. F. <ets>th<eacute/or<egrave/me</ets>. See <er>Theory</er>.]</ety> <sn>1.</sn> <def>That which is considered and established as a principle; hence, sometimes, a rule.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Not theories, but <qex>theorems</qex> (<?/), the intelligible products of contemplation, intellectual objects in the mind, and of and for the mind exclusively.</q> <rj><qau>Coleridge.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>By the <qex>theorems</qex>,<br/
Which your polite and terser gallants practice,<br/
I re-refine the court, and civilize<br/
Their barbarous natures.</q> <rj><qau>Massinger.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Math.)</fld> <def>A statement of a principle to be demonstrated.</def><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ A <xex>theorem</xex> is something to be proved, and is thus distinguished from a <xex>problem</xex>, which is something to be solved. In analysis, the term is sometimes applied to a rule, especially a rule or statement of relations expressed in a formula or by symbols; <as>as, the binomial <ex>theorem</ex>; Taylor's <ex>theorem</ex></as>. See the Note under <er>Proposition</er>, <pos>n.</pos>, 5.</note><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Binomial theorem</b></col>. <fld>(Math.)</fld> <cd>See under <er>Binomial</er>.</cd> -- <col><b>Negative theorem</b></col>, <cd>a theorem which expresses the impossibility of any assertion.</cd> -- <col><b>Particular theorem</b></col> <fld>(Math.)</fld>, <cd>a theorem which extends only to a particular quantity.</cd> -- <col><b>Theorem of Pappus</b></col>. <fld>(Math.)</fld> <cd>See <cref>Centrobaric method</cref>, under <er>Centrobaric</er>.</cd> -- <col><b>Universal theorem</b></col> <fld>(Math.)</fld>, <cd>a theorem which extends to any quantity without restriction.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Theorem</ent><br/
<hw>The"o*rem</hw>, <pos>v. t.</pos> <def>To formulate into a theorem.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Theorematical</ent><br/
<ent>Theorematic</ent><br/
<mhw>{ <hw>The`o*re*mat"ic</hw> <pr>(?)</pr>, <hw>The`o*re*mat"ic*al</hw> <pr>(?)</pr>, }</mhw> <pos>a.</pos> <ety>[Cf. Gr. <?/.]</ety> <def>Of or pertaining to a theorem or theorems; comprised in a theorem; consisting of theorems.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Theorematist</ent><br/
<hw>The`o*rem"a*tist</hw> <pr>(?)</pr>, <pos>n.</pos> <def>One who constructs theorems.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Theoremic</ent><br/
<hw>The`o*rem"ic</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Theorematic.</def>  <rj><au>Grew.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Theoretical</ent><br/
<ent>Theoretic</ent><br/
<mhw>{ <hw>The`o*ret"ic</hw> <pr>(?)</pr>, <hw>The`o*ret"ic*al</hw> <pr>(?)</pr>, }</mhw> <pos>a.</pos> <ety>[Gr. <?/: cf. L. <ets>theoreticus</ets>, F. <ets>th<eacute/or<eacute/tique</ets>.]</ety> <def>Pertaining to theory; depending on, or confined to, theory or speculation; speculative; terminating in theory or speculation: not practical; <as>as, <ex>theoretical</ex> learning; <ex>theoretic</ex> sciences.</as></def> -- <wordforms><wf>The`o*ret"ic*al*ly</wf>, <pos>adv.</pos></wordforms><br/
[<source>1913 Webster</source>]</p>

<p><ent>Theoretics</ent><br/
<hw>The`o*ret"ics</hw> <pr>(?)</pr>, <pos>n.</pos> <def>The speculative part of a science; speculation.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>At the very first, with our Lord himself, and his apostles, as represented to us in the New Testament, morals come before contemplation, ethics before <qex>theoretics</qex>.</q> <rj><qau>H. B. Wilson.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Theoric</ent><br/
<hw>The*or"ic</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[Cf. F. <ets>th<eacute/orique</ets>. See <er>Theory</er>.]</ety> <sn>1.</sn> <def>Of or pertaining to the theorica.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <pr>(<it>pron.</it> <?/)</pr> <def>Relating to, or skilled in, theory; theoretically skilled.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>A man but young,<br/
Yet old in judgment, <qex>theoric</qex> and practic<br/
In all humanity.</q> <rj><qau>Massinger.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Theoric</ent><br/
<hw>The"o*ric</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[OF. <ets>theorique</ets>; cf. L. <ets>theorice</ets>.]</ety> <def>Speculation; theory.</def> <mark>[Obs.]</mark>  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Theorica</ent><br/
||<hw>The*or"i*ca</hw> <pr>(?)</pr>, <pos>n. pl.</pos> <ety>[NL., fr. Gr. <?/ (sc. <?/), fr. <?/ belonging to <?/ a public spectacle. See <er>Theory</er>.]</ety> <fld>(Gr. Antiq.)</fld> <def>Public moneys expended at Athens on festivals, sacrifices, and public entertainments (especially theatrical performances), and in gifts to the people; -- also called <altname>theoric fund</altname>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Theorical</ent><br/
<hw>The*or"ic*al</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Theoretic.</def> <mark>[Obs.]</mark>  <rj><au>Bp. Hall.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Theorically</ent><br/
<hw>The*or"ic*al*ly</hw>, <pos>adv.</pos> <def>In a theoretic manner.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Theorist</ent><br/
<hw>The"o*rist</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Cf. F. <ets>th<eacute/oriste</ets>.]</ety> <def>One who forms theories; one given to theory and speculation; a speculatist.</def>  <rj><au>Cowper.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>The greatest <qex>theoretists</qex> have given the preference to such a government as that which obtains in this kingdom.</q> <rj><qau>Addison.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><-- 2. A scientist who forms theories about natural phenomena, based on the data gathered by others, rather than himself performing experiments to test the theories.  Contrasted with experimentalist.  --><br/
[<source>1913 Webster</source>]</p>

<p><ent>Theorization</ent><br/
<hw>The`o*ri*za"tion</hw> <pr>(?)</pr>, <pos>n.</pos> <def>The act or product of theorizing; the formation of a theory or theories; speculation.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Theorize</ent><br/
<hw>The"o*rize</hw> <pr>(?)</pr>, <pos>v. i.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Theorized</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Theorizing</conjf> <pr>(?)</pr>.]</vmorph> <ety>[Cf. F. <ets>th<eacute/oriser</ets>.]</ety> <def>To form a theory or theories; to form opinions solely by theory; to speculate.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Theorizer</ent><br/
<hw>The"o*ri`zer</hw> <pr>(?)</pr>, <pos>n.</pos> <def>One who theorizes or speculates; a theorist.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Theory</ent><br/
<hw>The"o*ry</hw> <pr>(?)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Theories</plw> <pr>(#)</pr>.</plu> <ety>[F. <ets>th<eacute/orie</ets>, L. <ets>theoria</ets>, Gr. <?/ a beholding, spectacle, contemplation, speculation, fr. <?/ a spectator, <?/ to see, view. See <er>Theater</er>.]</ety> <sn>1.</sn> <def>A doctrine, or scheme of things, which terminates in speculation or contemplation, without a view to practice; hypothesis; speculation.</def><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ <ldquo/This word is employed by English writers in a very loose and improper sense. It is with them usually convertible into <xex>hypothesis</xex>, and <xex>hypothesis</xex> is commonly used as another term for <xex>conjecture</xex>. The terms <xex>theory</xex> and <xex>theoretical</xex> are properly used in opposition to the terms <xex>practice</xex> and <xex>practical</xex>. In this sense, they were exclusively employed by the ancients; and in this sense, they are almost exclusively employed by the Continental philosophers.<rdquo/  <rj><au>Sir W. Hamilton.</au></rj>
</note><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>An exposition of the general or abstract principles of any science; <as>as, the <ex>theory</ex> of music</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>The science, as distinguished from the art; <as>as, the <ex>theory</ex> and practice of medicine</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>The philosophical explanation of phenomena, either physical or moral; <as>as, Lavoisier's <ex>theory</ex> of combustion; Adam Smith's <ex>theory</ex> of moral sentiments.</as></def><br/
[<source>1913 Webster</source>]</p>

<p><cs><mcol><col><b>Atomic theory</b></col>, <col><b>Binary theory</b></col></mcol>, <cd>etc. See under <er>Atomic</er>, <er>Binary</er>, etc.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><syn><b>Syn.</b> -- Hypothesis, speculation.</syn> <usage> -- <er>Theory</er>, <er>Hypothesis</er>. A <xex>theory</xex> is a scheme of the relations subsisting between the parts of a systematic whole; an <xex>hypothesis</xex> is a tentative conjecture respecting a cause of phenomena.</usage><br/
[<source>1913 Webster</source>]</p>

<p><ent>Theosopher</ent><br/
<ent>Theosoph</ent><br/
<mhw>{ <hw>The"o*soph</hw> <pr>(?)</pr>, <hw>The*os"o*pher</hw> <pr>(?)</pr>, }</mhw> <pos>n.</pos> <def>A theosophist.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Theosophical</ent><br/
<ent>Theosophic</ent><br/
<mhw>{ <hw>The`o*soph"ic</hw> <pr>(?)</pr>, <hw>The`o*soph"ic*al</hw> <pr>(?)</pr>, }</mhw> <pos>a.</pos> <ety>[Cf. F. <ets>th<eacute/osophique</ets>.]</ety> <def>Of or pertaining to theosophy.</def> -- <wordforms><wf>The`o*soph"ic*al*ly</wf>, <pos>adv.</pos></wordforms><br/
[<source>1913 Webster</source>]</p>

<p><ent>Theosophism</ent><br/
<hw>The*os"o*phism</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Cf. F. <ets>th<eacute/osophisme</ets>.]</ety> <def>Belief in theosophy.</def>  <rj><au>Murdock.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Theosophist</ent><br/
<hw>The*os"o*phist</hw> <pr>(?)</pr>, <pos>n.</pos> <def>One addicted to theosophy.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The <qex>theosophist</qex> is one who gives you a theory of God, or of the works of God, which has not reason, but an inspiration of his own, for its basis.</q> <rj><qau>R. A. Vaughan.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Theosophistical</ent><br/
<hw>The*os`o*phis"tic*al</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Of or pertaining to theosophy; theosophical.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Theosophize</ent><br/
<hw>The*os"o*phize</hw> <pr>(?)</pr>, <pos>v. i.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Theosophized</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Theosophizing</conjf>.]</vmorph> <def>To practice theosophy.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Theosophy</ent><br/
<hw>The*os"o*phy</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Gr. <?/ knowledge of things divine, fr. <?/ wise in the things of God; <?/ God + <?/ wise: cf. F. <ets>th<eacute/osophie</ets>.]</ety> <def>Any system of philosophy or mysticism which proposes to attain intercourse with God and superior spirits, and consequent superhuman knowledge, by physical processes, as by the theurgic operations of some ancient Platonists, or by the chemical processes of the German fire philosophers; also, a direct, as distinguished from a revealed, knowledge of God, supposed to be attained by extraordinary illumination; especially, a direct insight into the processes of the divine mind, and the interior relations of the divine nature.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Therapeutae</ent><br/
||<hw>Ther`a*peu"tae</hw> <pr>(?)</pr>, <pos>n. pl.</pos> <ety>[NL., fr. Gr. <?/ (pl. <?/) an attendant, servant, physician. See <er>Therapeutic</er>.]</ety> <fld>(Eccl. Hist.)</fld> <def>A name given to certain ascetics said to have anciently dwelt in the neighborhood of Alexandria. They are described in a work attributed to Philo, the genuineness and credibility of which are now much discredited.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Therapeutical</ent><br/
<ent>Therapeutic</ent><br/
<mhw>{ <hw>Ther`a*peu"tic</hw> <pr>(?)</pr>, <hw>Ther`a*peu"tic*al</hw> <pr>(?)</pr>, }</mhw> <pos>a.</pos> <ety>[F. <ets>th<eacute/rapeutique</ets>, Gr. <?/, from <?/ attendant, servant, <?/ to serve, take care of, treat medically, <?/ attendant, servant.]</ety> <fld>(Med.)</fld> <def>Of or pertaining to the healing art; concerned in discovering and applying remedies for diseases; curative.</def> <ldquo/<xex>Therapeutic</xex> or curative physic.<rdquo/  <rj><au>Sir T. Browne.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Medicine is justly distributed into <ldquo/prophylactic,<rdquo/ or the art of preserving health, and <qex>therapeutic</qex>, or the art of restoring it.</q> <rj><qau>I. Watts.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Therapeutic</ent><br/
<hw>Ther`a*peu"tic</hw>, <pos>n.</pos> <def>One of the Therapeutae.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Therapeutics</ent><br/
<hw>Ther`a*peu"tics</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Cf. F. <ets>th<eacute/rapeutique</ets>.]</ety> <def>That part of medical science which treats of the discovery and application of remedies for diseases.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Therapeutist</ent><br/
<hw>Ther`a*peu"tist</hw> <pr>(?)</pr>, <pos>n.</pos> <def>One versed in therapeutics, or the discovery and application of remedies.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Therapy</ent><br/
<hw>Ther"a*py</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Gr. <?/.]</ety> <def>Therapeutics.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>There</ent><br/
<hw>There</hw> <pr>(?)</pr>, <pos>adv.</pos> <ety>[OE. <ets>ther</ets>, AS. <ets><edh/<aemac/r</ets>; akin to D. <ets>daar</ets>, G. <ets>da</ets>, OHG. <ets>d<amac/r</ets>, Sw. & Dan. <ets>der</ets>, Icel. & Goth. <ets><thorn/ar</ets>, Skr. <ets>tarhi</ets> then, and E. <ets>that</ets>. <root/184. See <er>That</er>, <pos>pron.</pos>]</ety> <sn>1.</sn> <def>In or at that place.</def> <ldquo/[They] <xex>there</xex> left me and my man, both bound together.<rdquo/  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>The Lord God planted a garden eastward in Eden; and <qex>there</qex> he put the man whom he had formed.</q> <rj><qau>Ge. ii. 8.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ In distinction from <xex>here</xex>, <xex>there</xex> usually signifies a place farther off. <ldquo/Darkness <xex>there</xex> might well seem twilight <xex>here</xex>.<rdquo/ <au>Milton.</au></note><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>In that matter, relation, etc.; at that point, stage, etc., regarded as a distinct place; <as>as, he did not stop <ex>there</ex>, but continued his speech</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The law that theaten'd death becomes thy friend<br/
And turns it to exile; <qex>there</qex> art thou happy.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>To or into that place; thither.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The rarest that e'er came <qex>there</qex>.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ <ex>There</ex> is sometimes used by way of exclamation, calling the attention to something, especially to something distant; as, <ex>there</ex>, <ex>there</ex>! see <ex>there</ex>! look <ex>there</ex>! <ex>There</ex> is often used as an expletive, and in this use, when it introduces a sentence or clause, the verb precedes its subject.</note><br/
[<source>1913 Webster</source>]</p>

<p><q>A knight <qex>there</qex> was, and that a worthy man.</q> <rj><qau>Chaucer.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q><qex>There</qex> is a path which no fowl knoweth.</q> <rj><qau>Job xxviii. 7.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Wherever <qex>there</qex> is a sense or perception, <qex>there</qex> some idea is actually produced.</q> <rj><qau>Locke.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q><qex>There</qex> have been that have delivered themselves from their ills by their good fortune or virtue.</q> <rj><qau>Suckling.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ <xex>There</xex> is much used in composition, and often has the sense of a pronoun. See <er>Thereabout</er>, <er>Thereafter</er>, <er>Therefrom</er>, etc.</note><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ <xex>There</xex> was formerly used in the sense of <xex>where</xex>.</note><br/
[<source>1913 Webster</source>]</p>

<p><q>Spend their good <qex>there</qex> it is reasonable.</q> <rj><qau>Chaucer.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Here and there</b></col>, <cd>in one place and another.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><syn><b>Syn.</b> -- See <er>Thither</er>.</syn><br/
[<source>1913 Webster</source>]</p>

<p><ent>Thereabouts</ent><br/
<ent>Thereabout</ent><br/
<mhw>{ <hw>There"a*bout`</hw> <pr>(?)</pr>, <hw>There"a*bouts`</hw> <pr>(?)</pr>, }</mhw> <pos>adv.</pos> <ety>[The latter spelling is less proper, but more commonly used.]</ety> <sn>1.</sn> <def>Near that place.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Near that number, degree, or quantity; nearly; <as>as, ten men, or <ex>thereabouts</ex></as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Five or six thousand horse . . . or <qex>thereabouts</qex>.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Some three months since, or <qex>thereabout</qex>.</q> <rj><qau>Suckling.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>Concerning that; about that.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>What will ye dine? I will go <qex>thereabout</qex>.</q> <rj><qau>Chaucer.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>They were much perplexed <qex>thereabout</qex>.</q> <rj><qau>Luke xxiv. 4.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Thereafter</ent><br/
<hw>There*af"ter</hw> <pr>(?)</pr>, <pos>adv.</pos> <ety>[AS. <ets><edh/<aemac/r<ae/fter</ets> after that. See <er>There</er>, and <er>After</er>.]</ety> <sn>1.</sn> <def>After that; afterward.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>According to that; accordingly.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>I deny not but that it is of greatest concernment in the church and commonwealth to have a vigilant eye how books demean themselves as well as men; and <qex>thereafter</qex> to confine, imprison, and do sharpest justice on them as malefactors.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>Of that sort.</def> <mark>[Obs.]</mark> <ldquo/My audience is not <xex>thereafter</xex>.<rdquo/  <rj><au>Latimer.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Thereagain</ent><br/
<hw>There"a*gain`</hw> <pr>(?)</pr>, <pos>adv.</pos> <def>In opposition; against one's course.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>If that him list to stand <qex>thereagain</qex>.</q> <rj><qau>Chaucer.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>There-anent</ent><br/
<hw>There"-a*nent`</hw> <pr>(?)</pr>, <pos>adv.</pos> <def>Concerning that.</def> <mark>[Scot.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Thereat</ent><br/
<hw>There*at"</hw> <pr>(?)</pr>, <pos>adv.</pos> <sn>1.</sn> <def>At that place; there.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Wide is the gate, and broad is the way, that leadeth to destruction, and many there be which go in <qex>thereat</qex>.</q> <rj><qau>Matt. vii. 13.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>At that occurrence or event; on that account.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Every error is a stain to the beauty of nature; for which cause it blusheth <qex>thereat</qex>.</q> <rj><qau>Hooker.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Therebiforn</ent><br/
<ent>Therebefore</ent><br/
<mhw>{ <hw>There`be*fore"</hw> <pr>(?)</pr>, <hw>There`bi*forn"</hw> <pr>(?)</pr>, }</mhw> <pos>adv.</pos> <def>Before that time; beforehand.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>Many a winter <qex>therebiforn</qex>.</q> <rj><qau>Chaucer.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Thereby</ent><br/
<hw>There*by"</hw> <pr>(?)</pr>, <pos>adv.</pos> <sn>1.</sn> <def>By that; by that means; in consequence of that.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Acquaint now thyself with him, and be at peace; <qex>thereby</qex> good shall come unto thee.</q> <rj><qau>Job xxii. 21.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Annexed to that.</def> <ldquo/<xex>Thereby</xex> hangs a tale.<rdquo/  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>Thereabout; -- said of place, number, etc.</def>  <rj><au>Chaucer.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Therefor</ent><br/
<hw>There*for"</hw> <pr>(?)</pr>, <pos>adv.</pos> <ety>[<ets>There</ets> + <ets>for.</ets>  Cf. <er>Therefore</er>.]</ety> <def>For that, or this; for it.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>With certain officers ordained <qex>therefore</qex>.</q> <rj><qau>Chaucer.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Therefore</ent><br/
<hw>There"fore</hw> <pr>(?)</pr>, <pos>conj. & adv.</pos> <ety>[OE. <ets>therfore</ets>. See <er>There</er>, and <er>Fore</er>, <pos>adv.</pos>, <er>For</er>, and cf. <er>Therefor</er>.]</ety> <sn>1.</sn> <def>For that or this reason, referring to something previously stated; for that.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>I have married a wife, and <qex>therefore</qex> I can not come.</q> <rj><qau>Luke xiv. 20.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Behold, we have forsaken all, and followed thee; what shall we have <qex>therefore</qex>?</q> <rj><qau>Matt. xix. 27.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Consequently; by consequence.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>He blushes; <qex>therefore</qex> he is guilty.</q> <rj><qau>Spectator.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><syn><b>Syn.</b> -- See <er>Then</er>.</syn><br/
[<source>1913 Webster</source>]</p>

<p><ent>Therefrom</ent><br/
<hw>There*from"</hw> <pr>(?)</pr>, <pos>adv.</pos> <def>From this or that.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Turn not aside <qex>therefrom</qex> to the right hand or to the left.</q> <rj><qau>John. xxiii. 6.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Therein</ent><br/
<hw>There*in"</hw> <pr>(?)</pr>, <pos>adv.</pos> <def>In that or this place, time, or thing; in that particular or respect.</def>  <rj><au>Wyclif.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>He pricketh through a fair forest,<br/
<qex>Therein</qex> is many a wild beast.</q> <rj><qau>Chaucer.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Bring forth abundantly in the earth, and multiply <qex>therein</qex>.</q> <rj><qau>Gen. ix. 7.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q><qex>Therein</qex> our letters do not well agree.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Thereinto</ent><br/
<hw>There`in*to"</hw> <pr>(?)</pr>, <pos>adv.</pos> <def>Into that or this, or into that place.</def>  <rj><au>Bacon.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Let not them . . . enter <qex>thereinto</qex>.</q> <rj><qau>Luke xxi. 21.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Thereof</ent><br/
<hw>There*of"</hw> <pr>(?)</pr>, <pos>adv.</pos> <def>Of that or this.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>In the day that thou eatest <qex>thereof</qex>, thou shalt surely die.</q> <rj><qau>Gen. ii. 17.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Thereology</ent><br/
<hw>The`re*ol"o*gy</hw> <pr>(?)</pr>, <pos>n.</pos> <def>Therapeutios.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Thereon</ent><br/
<hw>There*on"</hw> <pr>(?)</pr>, <pos>adv.</pos> <ety>[AS. <ets><?/<?/ron</ets>. See <er>There</er>, and <er>On</er>.]</ety> <def>On that or this.</def>  <rj><au>Chaucer.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Then the king said, Hang him <qex>thereon</qex>.</q> <rj><qau>Esther vii. 9.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Thereout</ent><br/
<hw>There*out"</hw> <pr>(?)</pr>, <pos>adv.</pos> <sn>1.</sn> <def>Out of that or this.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>He shall take <qex>thereout</qex> his handful of the flour.</q> <rj><qau>Lev. ii. 2.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>On the outside; out of doors.</def> <mark>[Obs.]</mark>  <rj><au>Chaucer.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Thereto</ent><br/
<hw>There*to"</hw> <pr>(?)</pr>, <pos>adv.</pos> <sn>1.</sn> <def>To that or this.</def>  <rj><au>Chaucer.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Besides; moreover.</def> <mark>[Obs.]</mark>  <rj><au>Spenser.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Her mouth full small, and <qex>thereto</qex> soft and red.</q> <rj><qau>Chaucer.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Theretofore</ent><br/
<hw>There`to*fore"</hw> <pr>(?)</pr>, <pos>adv.</pos> <def>Up to that time; before then; -- correlative with <xex>heretofore</xex>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Thereunder</ent><br/
<hw>There*un"der</hw> <pr>(?)</pr>, <pos>adv.</pos> <def>Under that or this.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Thereunto</ent><br/
<hw>There`un*to"</hw> <pr>(?)</pr>, <pos>adv.</pos> <def>Unto that or this; thereto; besides.</def>  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Thereupon</ent><br/
<hw>There`up*on"</hw> <pr>(?)</pr>, <pos>adv.</pos> <sn>1.</sn> <def>Upon that or this; thereon.</def> <ldquo/They shall feed <xex>thereupon</xex>.<rdquo/  <rj><au>Zeph. ii. 7.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>On account, or in consequence, of that; therefore.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>[He] hopes to find you forward, . . . <br/
And <qex>thereupon</qex> he sends you this good news.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>Immediately; at once; without delay.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Therewhile</ent><br/
<hw>There*while"</hw> <pr>(?)</pr>, <pos>adv.</pos> <def>At that time; at the same time.</def> <mark>[Obs.]</mark>  <rj><au>Laud.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Therewith</ent><br/
<hw>There*with"</hw> <pr>(?)</pr>, <pos>adv.</pos> <sn>1.</sn> <def>With that or this.</def> <ldquo/I have learned in whatsoever state I am, <xex>therewith</xex> to be content.<rdquo/  <rj><au>Phil. iv. 11.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>In addition; besides; moreover.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>To speak of strength and <qex>therewith</qex> hardiness.</q> <rj><qau>Chaucer.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>At the same time; forthwith.</def> <mark>[Obs.]</mark>  <rj><au>Johnson.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Therewithal</ent><br/
<hw>There`with*al"</hw> <pr>(?)</pr>, <pos>adv.</pos> <sn>1.</sn> <def>Over and above; besides; moreover.</def> <mark>[Obs.]</mark>  <rj><au>Daniel.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>And <qex>therewithal</qex> it was full poor and bad.</q> <rj><qau>Chaucer.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>With that or this; therewith; at the same time.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Thy slanders I forgive; and <qex>therewithal</qex><br/
Remit thy other forfeits.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>And <qex>therewithal</qex> one came and seized on her,<br/
And Enid started waking.</q> <rj><qau>Tennyson.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Therf</ent><br/
<hw>Therf</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[AS. <ets><?/eorf</ets>; akin to OHG. <ets>derb</ets>, Icel. <ets><?/jarfr</ets>.]</ety> <def>Not fermented; unleavened; -- said of bread, loaves, etc.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>Pask and the feast of <qex>therf</qex> loaves.</q> <rj><qau>Wyclif.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><-- p. 1497 --><br/
[<source>1913 Webster</source>]</p>

<p><ent>Theriaca</ent><br/
<ent>Theriac</ent><br/
<mhw>{ <hw>The"ri*ac</hw> <pr>(?)</pr>, ||<hw>The*ri"a*ca</hw> <pr>(?)</pr>, }</mhw> <pos>n.</pos> <ety>[L. <ets>theriaca</ets> an antidote against the bite of serpents, Gr. <?/: cf. F. <ets>th<eacute/riaque</ets>. See <er>Treacle</er>.]</ety> <sn>1.</sn> <fld>(Old Med.)</fld> <def>An ancient composition esteemed efficacious against the effects of poison; especially, a certain compound of sixty-four drugs, prepared, pulverized, and reduced by means of honey to an electuary; -- called also <altname>theriaca Andromachi</altname>, and <altname>Venice treacle</altname>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Treacle; molasses.</def>  <rj><au>British Pharm.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Theriacal</ent><br/
<ent>Theriac</ent><br/

<mhw>{ <hw>The"ri*ac</hw> <pr>(?)</pr>, <hw>The*ri"a*cal</hw> <pr>(?)</pr>, }</mhw> <pos>a.</pos> <ety>[Cf. F. <ets>th<eacute/riacal</ets>.]</ety> <def>Of or pertaining to theriac; medicinal.</def> <ldquo/<xex>Theriacal</xex> herbs.<rdquo/  <rj><au>Bacon.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Therial</ent><br/
<hw>The"ri*al</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Theriac.</def> <mark>[R.]</mark>  <rj><au>Holland.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Theriodont</ent><br/
<hw>The"ri*o*dont</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Paleon.)</fld> <def>One of the Theriodontia. Used also adjectively.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Theriodonta</ent><br/
||<hw>The`ri*o*don"ta</hw> <pr>(?)</pr>, <pos>n. pl.</pos> <ety>[NL.]</ety> <fld>(Paleon.)</fld> <def>Same as <er>Theriodontia</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Theriodontia</ent><br/
||<hw>The`ri*o*don"ti*a</hw> <pr>(?)</pr>, <pos>n. pl.</pos> <ety>[NL., fr. Gr. <?/ (dim. of <?/ a beast) + <?/, <?/, a tooth.]</ety> <fld>(Paleon.)</fld> <def>An extinct order of reptiles found in the Permian and Triassic formations in South Africa. In some respects they resembled carnivorous mammals. Called also <altname>Theromorpha</altname>.</def><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ They had biconcave vertebrae, ambulatory limbs, and a well-developed pelvis and shoulder girdle. Some of the species had large maxillary teeth. The head somewhat resembled that of a turtle. The Dicynodont is one of the best-known examples. See <er>Dicynodont</er>.</note><br/
[<source>1913 Webster</source>]</p>

<p><ent>Theriotomy</ent><br/
<hw>The`ri*ot"o*my</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Gr. <?/ wild beast + <?/ to cut.]</ety> <def>Zootomy.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Therm</ent><br/
<hw>Therm</hw> <pr>(th<etil/rm)</pr>, <pos>n.</pos> <def>one of several units of heat, especially one equivalent to 1000 large calories, or 100,000 British thermal units.</def><br/
[<source>PJC</source>]</p>

<p><ent>Thermae</ent><br/
||<hw>Ther"mae</hw> <pr>(?)</pr>, <pos>n. pl.</pos> <ety>[L. See <er>Thermal</er>.]</ety> <def>Springs or baths of warm or hot water.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Thermal</ent><br/
<hw>Ther"mal</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>thermae</ets> hot springs, fr. Gr. <?/, pl. of <?/ heat, fr. <?/ hot, warm, <?/ to warm, make hot; perhaps akin to L. <ets>formus</ets> warm, and E. <ets>forceps</ets>.]</ety> <sn>1.</sn> <def>Of or pertaining to heat; warm; hot; <as>as, the <ex>thermal</ex> unit; <ex>thermal</ex> waters</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The <qex>thermal</qex> condition of the earth.</q> <rj><qau>J. D. Forbes.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>caused by or affected by heat; <as>as, <ex>thermal</ex> springs</as>.</def><br/
[<source>PJC</source>]</p>

<p><sn>3.</sn> <def>designed to retain heat; <as>as, <ex>thermal</ex> underwear</as>.</def><br/
[<source>PJC</source>]</p>

<p><cs><mcol><col><b>Thermal conductivity</b></col>, <col><b>Thermal spectrum</b></col></mcol>. <cd>See under <er>Conductivity</er>, and <er>Spectrum</er>.</cd> -- <col><b>Thermal unit</b></col> <fld>(Physics)</fld>, <cd>a unit chosen for the comparison or calculation of quantities of heat. The unit most commonly employed is the amount of heat necessary to raise the temperature of one gram or one pound of water from zero to one degree Centigrade. See <er>Calorie</er>, and under <er>Unit</er>.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Thermally</ent><br/
<hw>Ther"mal*ly</hw>, <pos>adv.</pos> <def>In a thermal manner.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Thermantidote</ent><br/
<hw>Ther*man"ti*dote</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Gr. <?/  heat + E. <ets>antidote</ets>.]</ety> <def>A device for circulating and cooling the air, consisting essentially of a kind of roasting fan fitted in a window and incased in wet tatties.</def> <mark>[India]</mark></p>

<p><q>Will you bring me to book on the mountains, or where the <qex>thermantidotes</qex> play?</q>  <rj><qau>Kipling.</qau></rj><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Thermetograph</ent><br/
<hw>Ther*met"o*graph</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Gr. <?/ heat + <?/ measure + <ets>-graph</ets>.]</ety> <def>A self-registering thermometer, especially one that registers the maximum and minimum during long periods.</def>  <rj><au>Nichol.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Thermic</ent><br/
<hw>Ther"mic</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[Gr. <?/ heat.]</ety> <def>Of or pertaining to heat; due to heat; thermal; <as>as, <ex>thermic</ex> lines</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Thermic balance</b></col>. <cd>See <er>Bolometer</er>.</cd> -- <col><b>Thermic fever</b></col> <fld>(Med.)</fld>, <cd>the condition of fever produced by sunstroke. See <er>Sunstroke</er>.</cd> -- <col><b>Thermic weight</b></col>. <fld>(Mech.)</fld> <cd>Same as <cref>Heat weight</cref>, under <er>Heat</er>.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Thermidor</ent><br/
||<hw>Ther`mi`dor"</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F., fr. Gr. <?/ warm, hot.]</ety> <def>The eleventh month of the French republican calendar, -- commencing July 19, and ending August 17. See the Note under <er>Vend<eacute/miaire</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Thermifugine</ent><br/
<hw>Ther*mif"u*gine</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Gr. <?/ heat + L. <ets>fugere</ets> to flee.]</ety> <fld>(Chem.)</fld> <def>An artificial alkaloid of complex composition, resembling thalline and used as an antipyretic, -- whence its name.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Thermo-</ent><br/
<hw>Ther"mo-</hw> <pr>(?)</pr>. <def>A combining form from Gr. <grk>qe`rmh</grk> <xex>heat</xex>, <grk>qermo`s</grk> <xex>hot</xex>, <xex>warm</xex>; as in <xex>thermo</xex>chemistry, <xex>thermo</xex>dynamic.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Thermoanesthesia</ent><br/
<ent>Thermoanaesthesia</ent><br/
<mhw>{ <hw>Ther`mo*an`aes*the"si*a</hw> <it>or</it> <hw>Ther`mo*an`es*the"si*a</hw>  }</mhw> <pr>(?)</pr>, <pos>n.</pos> <ety>[NL.]</ety> <fld>(Med.)</fld> <def>Loss of power to distinguish heat or cold by touch.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Thermobarograph</ent><br/
<hw>Ther`mo*bar"o*graph</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Physics)</fld> <def>An instrument for recording simultaneously the pressure and temperature of a gas; a combined thermograph and barograph.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Thermobarometer</ent><br/
<hw>Ther`mo*ba*rom"e*ter</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[<ets>Thermo-</ets> + <ets>barometer</ets>.]</ety> <sn>1.</sn> <fld>(Physics)</fld> <def>An instrument for determining altitudes by the boiling point of water.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A siphon barometer adapted to be used also as a thermometer.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Thermobattery</ent><br/
<hw>Ther`mo*bat"ter*y</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[<ets>Thermo-</ets> + <ets>battery</ets>.]</ety> <def>A thermoelectric battery; a thermopile.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Thermocautery</ent><br/
<hw>Ther`mo*cau"ter*y</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[<ets>Thermo-</ets> + <ets>cautery</ets>.]</ety> <fld>(Surg.)</fld> <def>Cautery by the application of heat.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Paquelin's thermocautery</b></col>, <cd>thermocautery by means of a hollow platinum point, which is kept constantly hot by the passage through it of benzine vapor.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Thermochemical</ent><br/
<ent>Thermochemic</ent><br/
<mhw>{ <hw>Ther`mo*chem"ic</hw> <pr>(?)</pr>, <hw>Ther`mo*chem"ic*al</hw> <pr>(?)</pr>, }</mhw> <pos>a.</pos> <fld>(Chem. Physics)</fld> <def>Of or pertaining to thermochemistry; obtained by, or employed in, thermochemistry.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Thermochemistry</ent><br/
<hw>Ther`mo*chem"is*try</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[<ets>Thermo-</ets> + <ets>chemistry</ets>.]</ety> <def>That branch of chemical science which includes the investigation of the various relations existing between chemical action and that manifestation of force termed heat, or the determination of the heat evolved by, or employed in, chemical actions.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Thermochroic</ent><br/
<hw>Ther`mo*chro"ic</hw> <pr>(?)</pr>, <pos>a.</pos>  <ety>[<ets>Thermo-</ets> + Gr. <?/, <?/, color.]</ety> <def>Pertaining to or designating heat rays that have undergone selective absorption and are therefore analogous to colored light rays.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Thermochrosy</ent><br/
<hw>Ther*moch"ro*sy</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[<ets>Thermo-</ets> + Gr. <?/ coloring.]</ety> <fld>(Physics)</fld> <def>The property possessed by heat of being composed, like light, of rays of different degrees of refrangibility, which are unequal in rate or degree of transmission through diathermic substances.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Thermocouple</ent><br/
<hw>Ther"mo*cou`ple</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A thermoelectric couple.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Thermocurrent</ent><br/
<hw>Ther"mo*cur`rent</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[<ets>Thermo-</ets> + <ets>current</ets>.]</ety> <fld>(Physics)</fld> <def>A current developed or set in motion by heat; specif., an electric current, in a heterogeneous circuit, due to differences of temperature between the junctions of the substances of which the circuit is composed.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Thermodin</ent><br/
<hw>Ther"mo*din</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[G., fr. Gr. <?/ heat.]</ety> <fld>(Pharm.)</fld> <def>A white crystalline substance derived from urethane, used in medicine as an antipyretic, etc.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Thermodynamic</ent><br/
<hw>Ther`mo*dy*nam"ic</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[<ets>Thermo-</ets> + <ets>dynamic</ets>.]</ety> <fld>(Physics)</fld> <def>Relating to thermodynamics; caused or operated by force due to the application of heat.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Thermodynamic function</b></col>. <cd>See <cref>Heat weight</cref>, under <er>Heat</er>.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Thermodynamics</ent><br/
<hw>Ther`mo*dy*nam"ics</hw> <pr>(?)</pr>, <pos>n.</pos> <def>The science which treats of the mechanical action or relations of heat.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Thermoelectric</ent><br/
<hw>Ther`mo*e*lec"tric</hw> <pr>(?)</pr>, <pos>a.</pos> <fld>(Physics)</fld> <def>Pertaining to thermoelectricity; <as>as, <ex>thermoelectric</ex> currents</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Thermoelectric pair</ent><br/
<ent>Thermoelectric couple</ent><br/
<mhw>{ <hw>Ther`mo*e*lec"tric couple</hw> <it>or</it>  <hw>Ther`mo*e*lec"tric pair</hw> }</mhw> <pr>(?)</pr>. <fld>(Elec.)</fld> <def>A union of two conductors, as bars or wires of dissimilar metals joined at their extremities, for producing a thermoelectric current.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Thermoelectricity</ent><br/
<hw>Ther`mo*e`lec*tric"i*ty</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[<ets>Thermo-</ets> + <ets>electricity</ets>: cf. F. <ets>thermo<eacute/lectricit<eacute/</ets>.]</ety> <fld>(Physics)</fld> <def>Electricity developed in the action of heat. See the Note under <er>Electricity</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Thermoelectrometer</ent><br/
<hw>Ther`mo*e`lec*trom"e*ter</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[<ets>Thermo-</ets> + <ets>electrometer</ets>.]</ety> <fld>(Physics)</fld> <def>An instrument for measuring the strength of an electric current in the heat which it produces, or for determining the heat developed by such a current.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Thermogen</ent><br/
<hw>Ther"mo*gen</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[<ets>Thermo-</ets> + <ets>-gen</ets>.]</ety> <fld>(Old Chem.)</fld> <def>Caloric; heat; regarded as a material but imponderable substance.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Thermogenic</ent><br/
<hw>Ther`mo*gen"ic</hw> <pr>(?)</pr>, <pos>a.</pos> <fld>(Physiol.)</fld> <def>Relating to heat, or to the production of heat; producing heat; thermogenous; <as>as, the <ex>thermogenic</ex> tissues</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Thermogenous</ent><br/
<hw>Ther*mog"e*nous</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[<ets>Thermo-</ets> + <ets>-genous</ets>.]</ety> <fld>(Physiol.)</fld> <def>Producing heat; thermogenic.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Thermogram</ent><br/
<hw>Ther"mo*gram</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Physics)</fld> <def>The trace or record made by means of a thermograph.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Thermograph</ent><br/
<hw>Ther"mo*graph</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[<ets>Thermo-</ets> + <ets>-graph</ets>.]</ety> <fld>(Physics)</fld> <def>An instrument for automatically recording indications of the variation of temperature.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Thermography</ent><br/
<hw>Ther*mog"ra*phy</hw> <pr>(?)</pr>, <pos>n.</pos>  <ety>[<ets>Thermo-</ets> + <ets>-graphy</ets>.]</ety> <def>Any process of writing involving the use of heat.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Thermojunction</ent><br/
<hw>Ther`mo*junc"tion</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Elec.)</fld> <def>A junction of two dissimilar conductors used to produce a thermoelectric current, as in one form of pyrometer; a thermocouple.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Thermology</ent><br/
<hw>Ther*mol"o*gy</hw> <pr>(th<etil/r"m<ocr/l"<osl/*j<ycr/)</pr>, <pos>n.</pos> <ety>[<ets>Thermo-</ets> + <ets>-logy</ets>.]</ety> <def>A discourse on, or an account of, heat.</def>  <rj><au>Whewell.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Thermoluminescence</ent><br/
<hw>Ther`mo*lu`mi*nes"cence</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Physics)</fld> <def>Luminescence exhibited by a substance on being moderately heated. It is shown esp. by certain substances that have been exposed to the action of light or to X-rays.</def><-- ##?? used in dating of pottery? --> -- <wordforms><wf>Ther`mo*lu`mi*nes"cent</wf> <pr>(#)</pr>, <pos>a.</pos></wordforms><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Thermolysin</ent><br/
<hw>Ther`mo*ly"sin</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[<ets>Thermo-</ets> + Gr. <grk>ly`ein</grk> to loose + <ets>-in</ets>.]</ety> <fld>(Biochem.)</fld> <def>a proteolytic enzyme obtained from the bacterium <spn>Bacillus thermoproteolyticus</spn>, which hydrolyses the N-terminal amide bonds of hydrophobic amino acid residues in proteins.  It is used in studies of protein structure.  It has a molecular weight of about 37,500 and contains zinc and calcium in its active configuration.</def> <au>[MI11]</au><br/
[<source>PJC</source>]</p>

<p><ent>Thermolysis</ent><br/
<hw>Ther*mol"y*sis</hw> <pr>(th<etil/r"m<ocr/l"<icr/*s<icr/s)</pr>, <pos>n.</pos> <ety>[<ets>Thermo-</ets> + Gr. <grk>ly`ein</grk> to loose.]</ety> <fld>(Chem.)</fld> <def>The resolution of a compound into parts by heat; dissociation by heat.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Thermolytic</ent><br/
<hw>Ther`mo*lyt"ic</hw> <pr>(th<etil/r"m<osl/*l<icr/t"<icr/k)</pr>, <pos>a.</pos> <def>Of or pertaining to thermolysis.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Thermolyze</ent><br/
<hw>Ther"mo*lyze</hw> <pr>(th<etil/r"m<osl/*l<imac/z)</pr>, <pos>v. t.</pos> <fld>(Chem.)</fld> <def>To subject to thermolysis; to dissociate by heat.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Thermomagnetism</ent><br/
<hw>Ther`mo*mag"net*ism</hw> <pr>(th<etil/r"m<osl/*m<acr/g"n<ecr/t*<icr/z'm)</pr>, <pos>n.</pos> <ety>[<ets>Thermo-</ets> + <ets>magnetism</ets>.]</ety> <def>Magnetism as affected or caused by the action of heat; the relation of heat to magnetism.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Thermometer</ent><br/
<hw>Ther*mom"e*ter</hw> <pr>(th<etil/r*m<ocr/m"<esl/*t<etil/r)</pr>, <pos>n.</pos> <ety>[<ets>Thermo-</ets> + <ets>-meter</ets>: cf. F. thermom<egrave/tre. See <er>Thermal</er>.]</ety> <fld>(Physics)</fld> <def>An instrument for measuring temperature, founded on the principle that changes of temperature in bodies are accompanied by proportional changes in their volumes or dimensions.</def><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ The <xex>thermometer</xex> usually consists of a glass tube of capillary bore, terminating in a bulb, and containing mercury or alcohol, which expanding or contracting according to the temperature to which it is exposed, indicates the degree of heat or cold by the amount of space occupied, as shown by the position of the top of the liquid column on a graduated scale. See <er>Centigrade</er>, <er>Fahrenheit</er>, and <er>R<eacute/aumur</er>.<br/
[<source>1913 Webster</source>]</p>

<p>    To reduce degrees Fahrenheit to degrees Centigrade, subtract 32<deg/ and multiply by <frac59/; to reduce degrees Centigrade to degrees Fahrenheit, multiply by <frac95/ and add 32<deg/.</note><br/
[<source>1913 Webster</source>]</p>

<p><cs><mcol><col><b>Air thermometer</b></col>, <col><b>Balance thermometer</b></col></mcol>, <cd>etc. See under <er>Air</er>, <er>Balance</er>, etc.</cd> -- <col><b>Metallic thermometer</b></col>, <cd>a form of thermometer indicating changes of temperature by the expansion or contraction of rods or strips of metal.</cd> -- <mcol><col><b>Register thermometer</b></col>, <it>or</it> <col><b>Self-registering thermometer</b></col></mcol>, <cd>a thermometer that registers the maximum and minimum of temperature occurring in the interval of time between two consecutive settings of the instrument.  A common form contains a bit of steel wire to be pushed before the column and left at the point of maximum temperature, or a slide of enamel, which is drawn back by the liquid, and left within it at the point of minimum temperature.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Thermometrical</ent><br/
<ent>Thermometric</ent><br/
<mhw>{ <hw>Ther`mo*met"ric</hw> <pr>(?)</pr>, <hw>Ther`mo*met"ric*al</hw> <pr>(?)</pr>, }</mhw> <pos>a.</pos> <ety>[Cf. F. <ets>thermom<eacute/trique</ets>.]</ety> <sn>1.</sn> <def>Of or pertaining to a thermometer; <as>as, the <ex>thermometrical</ex> scale or tube</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Made, or ascertained, by means of a thermometer; <as>as, <ex>thermometrical</ex> observations</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Thermometrically</ent><br/
<hw>Ther`mo*met"ric*al*ly</hw> <pr>(?)</pr>, <pos>adv.</pos> <def>In a thermometrical manner; by means of a thermometer.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Thermometrograph</ent><br/
<hw>Ther`mo*met"ro*graph</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[<ets>Thermo-</ets> + Gr. <?/ measure + <ets>-graph</ets>.]</ety> <fld>(Physics)</fld> <def>An instrument for recording graphically the variations of temperature, or the indications of a thermometer.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Thermometry</ent><br/
<hw>Ther*mom"e*try</hw> <pr>(?)</pr>, <pos>n.</pos> <def>The estimation of temperature by the use of a thermometric apparatus.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Thermomotor</ent><br/
<hw>Ther`mo*mo"tor</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A heat engine; a hot-air engine.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Thermomultiplier</ent><br/
<hw>Ther`mo*mul"ti*pli`er</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[<ets>Thermo-</ets> + <ets>multiplier</ets>.]</ety> <def>Same as <er>Thermopile</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Thermoneurosis</ent><br/
<hw>Ther`mo*neu*ro"sis</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[NL.]</ety> <fld>(Med.)</fld> <sd>(a)</sd> <def>A neurosis caused by exposure to heat.</def> <sd>(b)</sd> <def>A neurosis causing a rise or fall of a body's temperature.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Thermoneutrality</ent><br/
<hw>Ther`mo*neu*tral"i*ty</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Chem.)</fld> <def>Neutrality as regards heat effects.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Thermophilic</ent><br/
<hw>Ther`mo*phil"ic</hw> <pr>(?)</pr>, <pos>a.</pos>  <ety>[<ets>Thermo-</ets> + Gr. <?/ loving + <ets>-ic</ets>.]</ety> <def>Heat-loving; -- applied especially to microorganisms such as certain bacteria, fungi and algae, which grow best at temperatures above 40<deg/ C (e.g. between 50<deg/ and 60<deg/), and in some cases at temperatures that would kill ordinary microorganisms.  They are found in naturally hot locations, such as at hot springs or the thermal vents at the ocean bottom.</def><br/
[<source>Webster 1913 Suppl.</source> <source>+PJC</source>]</p>

<p><ent>Thermophone</ent><br/
<hw>Ther"mo*phone</hw> <pr>(?)</pr>, <pos>n.</pos>  <ety>[<ets>Thermo-</ets> + <ets>phone</ets>.]</ety> <sn>1.</sn> <def>A portable form of telethermometer, using a telephone in connection with a differential thermometer.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><sn>2.</sn>  <def>A telephone involving heat effects, as changes in temperature (hence in length) due to pulsations of the line current in a fine wire connected with the receiver diaphragm.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Thermophore</ent><br/
<hw>Ther"mo*phore</hw> <pr>(?)</pr>, <pos>n.</pos>  <ety>[<ets>Thermo-</ets> + Gr. <?/ -bearing, fr. <grk>fe`rein</grk> to bear.]</ety> <def>An apparatus for conveying heat, as a case containing material which retains its heat for a considerable period.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Thermopile</ent><br/
<hw>Ther"mo*pile</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[<ets>Thermo-</ets> + <ets>pile</ets> a heap.]</ety> <fld>(Physics)</fld> <def>An instrument of extreme sensibility, used to determine slight differences and degrees of heat. It is composed of alternate bars of antimony and bismuth, or any two metals having different capacities for the conduction of heat, connected with an astatic galvanometer, which is very sensibly affected by the electric current induced in the system of bars when exposed even to the feeblest degrees of heat.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Thermoregulator</ent><br/
<hw>Ther`mo*reg"u*la`tor</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Physics)</fld> <def>A device for the automatic regulation of temperature; a thermostat.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Thermoscope</ent><br/
<hw>Ther"mo*scope</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[<ets>Thermo-</ets> + <ets>-scope</ets>.]</ety> <fld>(Physics)</fld> <def>An instrument for indicating changes of temperature without indicating the degree of heat by which it is affected; especially, an instrument contrived by <person>Count Rumford</person> which, as modified by <person>Professor Leslie</person>, was afterward called the <altname>differential thermometer</altname>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Thermoscopic</ent><br/
<hw>Ther`mo*scop"ic</hw> <pr>(?)</pr>, <pos>a.</pos> <fld>(Physics)</fld> <def>Of or pertaining to the thermoscope; made by means of the thermoscope; <as>as, <ex>thermoscopic</ex> observations</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Thermosiphon</ent><br/
<hw>Ther`mo*si"phon</hw> <pr>(?)</pr>, <pos>n.</pos> <def>An arrangement of siphon tubes for assisting circulation in a liquid.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Thermostable</ent><br/
<hw>Ther`mo*sta"ble</hw> <pr>(?)</pr>, <pos>a.</pos>  <ety>[<ets>Thermo-</ets> + <ets>stable</ets> fixed.]</ety> <fld>(Physiol. Chem.)</fld> <def>Capable of being heated to or somewhat above 55<deg/ C. without loss of special properties; -- said of proteins, immune substances, etc.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Thermostat</ent><br/
<hw>Ther"mo*stat</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[<ets>Thermo-</ets> + Gr. <?/ to make to stand.]</ety> <fld>(Physics)</fld> <def>A device which automatically regulates temperature, or provides a signal used by another device to regulate temperature.  The temperature-sensitive signal may be electronic, as that produced by a thermocouple.  The signal may also be caused mechanically, as by the unequal expansion of different metals, liquids, or gases by heat, which can then cause the opening or closing of the damper of a stove, or the like, as the heat becomes greater or less than is desired.</def><br/
[<source>1913 Webster</source> <source>+PJC</source>]</p>

<p><ent>Thermostatic</ent><br/
<hw>Ther`mo*stat"ic</hw> <pr>(?)</pr>, <pos>a.</pos> <fld>(Physics)</fld> <def>Of or pertaining to the thermostat; made or effected by means of the thermostat.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Thermosystaltic</ent><br/
<hw>Ther`mo*sys*tal"tic</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[<ets>Thermo-</ets> + <ets>systaltic</ets>.]</ety> <fld>(Physiol.)</fld> <def>Influenced in its contraction by heat or cold; -- said of a muscle.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Thermotactic</ent><br/
<hw>Ther`mo*tac"tic</hw> <pr>(?)</pr>, <pos>a.</pos> <fld>(Physiol.)</fld> <def>Of or retaining to thermotaxis.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Thermotank</ent><br/
<hw>Ther"mo*tank</hw> <pr>(?)</pr>, <pos>n.</pos>  <ety>[<ets>Thermo-</ets> + <ets>ank</ets>.]</ety> <def>A tank containing pipes through which circulates steam, water, air, or the like, for heating or cooling; -- used in some heating and ventilation systems.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Thermotaxic</ent><br/
<hw>Ther`mo*tax"ic</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[<ets>Thermo-</ets> + Gr. <?/ arrangement.]</ety> <fld>(Physiol.)</fld> <def>Pertaining to, or connected with, the regulation of temperature in the animal body; <as>as, the <ex>thermotaxic</ex> nervous system</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Thermotaxis</ent><br/
<hw>Ther`mo*tax"is</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[NL.; <ets>thermo-</ets> + Gr. <?/ an arranging.]</ety> <fld>(Physiol.)</fld> <sd>(a)</sd> <def>The property possessed by protoplasm of moving under the influence of heat.</def> <sd>(b)</sd> <def>Determination of the direction of locomotion by heat.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Thermotensile</ent><br/
<hw>Ther`mo*ten"sile</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Pertaining to the variation of tensile strength with the temperature.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Thermotension</ent><br/
<hw>Ther`mo*ten"sion</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[<ets>Thermo-</ets> + <ets>tension</ets>.]</ety> <def>A process of increasing the strength of wrought iron by heating it to a determinate temperature, and giving to it, while in that state, a mechanical strain or tension in the direction in which the strength is afterward to be exerted.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Thermotherapy</ent><br/
<hw>Ther`mo*ther"a*py</hw> <pr>(?)</pr>, <pos>n.</pos>  <ety>[<ets>Thermo-</ets> + <ets>therapy</ets>.]</ety> <fld>(Med.)</fld> <def>Treatment of disease by heat, esp. by hot air.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Thermotical</ent><br/
<ent>Thermotic</ent><br/
<mhw>{ <hw>Ther*mot"ic</hw> <pr>(?)</pr>, <hw>Ther*mot"ic*al</hw> <pr>(?)</pr>, }</mhw> <pos>a.</pos> <ety>[Gr. <?/ heat, fr. <?/ hot.]</ety> <def>Of or pertaining to heat; produced by heat; <as>as, <ex>thermotical</ex> phenomena</as>.</def>  <rj><au>Whewell.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Thermotics</ent><br/
<hw>Ther*mot"ics</hw> <pr>(?)</pr>, <pos>n.</pos> <def>The science of heat.</def>  <rj><au>Whewell.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Thermotonus</ent><br/
<hw>Ther*mot"o*nus</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[NL.; <ets>thermo-</ets> + <ets>tonus</ets>.]</ety> <fld>(Plant Physiol.)</fld> <def>A condition of tonicity with respect to temperature.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Thermotropic</ent><br/
<hw>Ther`mo*trop"ic</hw> <pr>(?)</pr>, <pos>a.</pos> <fld>(Bot.)</fld> <def>Manifesting thermotropism.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Thermotropism</ent><br/
<hw>Ther*mot"ro*pism</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[<ets>Thermo-</ets> + Gr. <?/ to turn.]</ety> <fld>(Bot.)</fld> <def>The phenomenon of turning towards a source of warmth, seen in the growing parts of some plants.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Thermotype</ent><br/
<hw>Ther"mo*type</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[<ets>Thermo-</ets> + <ets>-type</ets>.]</ety> <def>A picture (as of a slice of wood) obtained by first wetting the object slightly with hydrochloric or dilute sulphuric acid, then taking an impression with a press, and next strongly heating this impression.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Thermotypy</ent><br/
<hw>Ther*mot"y*py</hw> <pr>(?)</pr>, <pos>n.</pos> <def>The art or process of obtaining thermotypes.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Thermovoltaic</ent><br/
<hw>Ther`mo*vol*ta"ic</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[<ets>Thermo-</ets> + <ets>voltaic</ets>.]</ety> <fld>(Physics)</fld> <def>Of or relating to heat and electricity; especially, relating to thermal effects produced by voltaic action.</def>  <rj><au>Faraday.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Theroid</ent><br/
<hw>The"roid</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[Gr. <grk>qh`r</grk>, <grk>qhro`s</grk>, wild beast + <ets>-oid</ets>.]</ety> <fld>(Med.)</fld> <def>Resembling a beast in nature or habit; marked by animal characteristics; <as>as, <ex>theroid</ex> idiocy</as>.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Theromorpha</ent><br/
||<hw>The`ro*mor"pha</hw> <pr>(?)</pr>, <pos>n. pl.</pos> <ety>[NL.: Gr. <grk>qh`r</grk> beast + <grk>morfh`</grk> form.]</ety> <fld>(Paleon.)</fld> <def>See <er>Theriodonta</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Theropoda</ent><br/
||<hw>The*rop"o*da</hw> <pr>(?)</pr>, <pos>n. pl.</pos> <ety>[NL., fr. Gr. <grk>qh`r</grk> a beast + <?/, <?/, foot.]</ety> <fld>(Paleon.)</fld> <def>An order of carnivorous dinosaurs in which the feet are less birdlike, and hence more like those of an ordinary quadruped, than in the Ornithopoda. It includes the rapacious genera <gen>Megalosaurus</gen>, <gen>Creosaurus</gen>, and their allies.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Thesaurus</ent><br/
||<hw>The*sau"rus</hw> <pr>(?)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Thesauri</plw> <pr>(#)</pr>.</plu> <ety>[L. See <er>Treasure</er>.]</ety> <def>A treasury or storehouse; hence, a repository, especially of knowledge; -- often applied to a comprehensive work, like a dictionary or cyclopedia.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>These</ent><br/
<hw>These</hw> <pr>(<th/<emac/z)</pr>, <pos>pron.</pos> <ety>[OE. <ets><thorn/es</ets>, <ets><thorn/aes</ets>, a variant of <ets><thorn/as</ets>, pl. of <ets><thorn/es</ets>, <ets>thes</ets>, this. See <er>This</er>, and cf. <er>Those</er>.]</ety> <def>The plural of <xex>this</xex>. See <er>This</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Thesicle</ent><br/
<hw>Thes"i*cle</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Dim. of <ets>thesis</ets>.]</ety> <def>A little or subordinate thesis; a proposition.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Thesis</ent><br/
<hw>The"sis</hw> <pr>(?)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Theses</plw> <pr>(#)</pr>.</plu> <ety>[L., fr. Gr. <?/, fr. <?/ to place, set. See <er>Do</er>, and cf. <er>Anathema</er>, <er>Apothecary</er>, <er>Epithet</er>, <er>Hypothesis</er>, <er>Parenthesis</er>, <er>Theme</er>, <er>Tick</er> a cover.]</ety> <sn>1.</sn> <def>A position or proposition which a person advances and offers to maintain, or which is actually maintained by argument.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Hence, an essay or dissertation written upon specific or definite theme; especially, an essay presented by a candidate for a diploma or degree.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>I told them of the grave, becoming, and sublime deportment they should assume upon this mystical occasion, and read them two homilies and a <qex>thesis</qex> of my own composing, to prepare them.</q> <rj><qau>Goldsmith.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <fld>(Logic)</fld> <def>An affirmation, or distinction from a supposition or hypothesis.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <fld>(Mus.)</fld> <def>The accented part of the measure, expressed by the downward beat; -- the opposite of <xex>arsis</xex>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <fld>(Pros.)</fld> <sd>(a)</sd> <def>The depression of the voice in pronouncing the syllables of a word.</def> <sd>(b)</sd> <def>The part of the foot upon which such a depression falls.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Thesmothete</ent><br/
<hw>Thes"mo*thete</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Gr. <?/; <?/ that which is established, a law (fr. <?/ to set) + <?/ a giver (also fr. <?/).]</ety> <fld>(Gr. Antiq.)</fld> <def>A lawgiver; a legislator; one of the six junior archons at Athens.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Thespian</ent><br/
<hw>Thes"pi*an</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[From L. <ets>Thespis</ets>, Gr. <?/, the founder of the Greek drama.]</ety> <def>Of or pertaining to Thespis; hence, relating to the drama; dramatic; <as>as, the <ex>Thespian</ex> art</as>.</def> -- <def2><pos>n.</pos> <def>An actor.</def></def2><br/
[<source>1913 Webster</source>]</p>

<p><ent>Thessalian</ent><br/
<hw>Thes*sa"li*an</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[Cf. L. <ets>Thessalius</ets>.]</ety> <def>Of or pertaining to Thessaly in Greece.</def> <au>Shak.</au> -- <def2><pos>n.</pos> <def>A native or inhabitant of Thessaly.</def></def2><br/
[<source>1913 Webster</source>]</p>

<p><ent>Thessalonian</ent><br/
<hw>Thes`sa*lo"ni*an</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Of or pertaining to Thessalonica, a city of Macedonia.</def> -- <def2><pos>n.</pos> <def>A native or inhabitant of Thessalonica.</def></def2><br/
[<source>1913 Webster</source>]</p>

<p><ent>Theta</ent><br/
<hw>The"ta</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L., fr. Gr. <grk>qh^ta</grk>, the Greek letter <theta/, <THETA/.]</ety> <def>A letter of the Greek alphabet corresponding to <it>th</it> in English; -- sometimes called the <xex>unlucky</xex> letter, from being used by the judges on their ballots in passing condemnation on a prisoner, it being the first letter of the Greek <grk>qa`natos</grk>, death.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Theta function</b></col> <fld>(Math.)</fld>, <cd>one of a group of functions used in developing the properties of elliptic functions.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Thetical</ent><br/
<hw>Thet"ic*al</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[Gr. <?/ fit for placing, fr. <?/ to set, lay down. See <er>Thesis</er>.]</ety> <def>Laid down; absolute or positive, as a law.</def>  <rj><au>Dr. H. More.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Thetine</ent><br/
<hw>The"tine</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[<ets>Th</ets>io + <ets>et</ets>her + sulph<ets>ine</ets>.]</ety> <fld>(Chem.)</fld> <def>Any one of a series of complex basic sulphur compounds analogous to the sulphines.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Theurgical</ent><br/
<ent>Theurgic</ent><br/
<mhw>{ <hw>The*ur"gic</hw> <pr>(?)</pr>, <hw>The*ur"gic*al</hw> <pr>(?)</pr>, }</mhw> <pos>a.</pos> <ety>[L. <ets>theurgicus</ets>, Gr. <?/: cf. F. <ets>th<eacute/urgique</ets>.]</ety> <def>Of or pertaining to theurgy; magical.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Theurgic hymns</b></col>, <cd>songs of incantation.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Theurgist</ent><br/
<hw>The"ur*gist</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Cf. F. <ets>th<eacute/urgiste</ets>.]</ety> <def>One who pretends to, or is addicted to, theurgy.</def>  <rj><au>Hallywell.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><-- p. 1498 --><br/
[<source>1913 Webster</source>]</p>

<p><ent>Theurgy</ent><br/
<hw>The"ur*gy</hw> <pr>(th<emac/"<ucr/r*j<ycr/)</pr>, <pos>n.</pos> <ety>[L. <ets>theurgia</ets>, Gr. <grk>qeoyrgi`a</grk>, fr. <grk>qeoyrgo`s</grk> doing the works of God; <grk>qeo`s</grk> God + <grk>'e`rgon</grk> work: cf. F. <ets>th<eacute/urgie</ets>. See <er>Theism</er>, and <er>Work</er>.]</ety> <sn>1.</sn> <def>A divine work; a miracle; hence, magic; sorcery.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A kind of magical science or art developed in Alexandria among the Neoplatonists, and supposed to enable man to influence the will of the gods by means of purification and other sacramental rites.</def>  <rj><au>Schaff-Herzog Encyc.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>In later or modern magic, that species of magic in which effects are claimed to be produced by supernatural agency, in distinction from <xex>natural</xex> magic.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Thew</ent><br/
<hw>Thew</hw> <pr>(th<umac/)</pr>, <pos>n.</pos> <note>[Chiefly used in the plural <er>Thews</er> <pr>(th<umac/z)</pr>.]</note> <ety>[OE. <ets>thew</ets>, <ets><thorn/eau</ets>, manner, habit, strength, AS. <ets><thorn/e<aacute/w</ets> manner, habit (cf. <ets><thorn/<ymac/wan</ets> to drive); akin to OS. <ets>thau</ets> custom, habit, OHG. <ets>dou</ets>. <root/56.]</ety> <sn>1.</sn> <def>Manner; custom; habit; form of behavior; qualities of mind; disposition; specifically, good qualities; virtues.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>For her great light<br/
Of sapience, and for her <qex>thews</qex> clear.</q> <rj><qau>Chaucer.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Evil speeches destroy good <qex>thews</qex>.</q> <rj><qau>Wyclif (1 Cor. xv. 33).</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>To be upbrought in gentle <qex>thews</qex> and martial might.</q> <rj><qau>Spenser.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Muscle or strength; nerve; brawn; sinew.</def>  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>And I myself, who sat apart<br/
And watched them, waxed in every limb;<br/
I felt the <qex>thews</qex> of Anakim,<br/
The pules of a Titan's heart.</q> <rj><qau>Tennyson.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Thewed</ent><br/
<hw>Thewed</hw> <pr>(th<umac/d)</pr>, <pos>a.</pos> <sn>1.</sn> <def>Furnished with thews or muscles; <as>as, a well-<ex>thewed</ex> limb</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Accustomed; mannered.</def> <mark>[Obs.]</mark>  <rj><au>John Skelton.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Yet would not seem so rude and <qex>thewed</qex> ill.</q> <rj><qau>Spenser.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Thewy</ent><br/
<hw>Thew"y</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Having strong or large thews or muscles; muscular; sinewy; strong.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>They</ent><br/
<hw>They</hw> <pr>(<th/<amac/)</pr>, <pos>pron. pl.</pos>; <wordforms><pos>poss.</pos> <er>Theirs</er>; <pos>obj.</pos> <er>Them</er></wordforms>. <ety>[Icel. <ets><thorn/eir</ets> they, properly nom. pl. masc. of <ets>s<amac/</ets>, <ets>s<umac/</ets>, <ets><thorn/at</ets>, a demonstrative pronoun, akin to the English definite article, AS. <ets>s<emac/</ets>, <ets>se<oacute/</ets>, <ets><edh/aet</ets>, nom. pl. <ets><edh/<amac/</ets>. See <er>That</er>.]</ety> <def>The plural of <xex>he</xex>, <xex>she</xex>, or <xex>it</xex>. <ex>They</ex> is never used adjectively, but always as a pronoun proper, and sometimes refers to persons without an antecedent expressed.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Jolif and glad <qex>they</qex> went unto <qex>here</qex> [their] rest<br/
And casten <qex>hem</qex> [them] full early for to sail.</q> <rj><qau>Chaucer.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q><qex>They</qex> of Italy salute you.</q> <rj><qau>Heb. xiii. 24.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Blessed are <qex>they</qex> which do hunger and thirst after righteousness.</q> <rj><qau>Matt. v. 6.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ <ex>They</ex> is used indefinitely, as our ancestors used <xex>man</xex>, and as the French use <xex>on</xex>; as, <ex>they</ex> say (French <xex>on dit</xex>), that is, it is said by persons not specified.</note><br/
[<source>1913 Webster</source>]</p>

<p><ent>Thialdine</ent><br/
<hw>Thi*al"dine</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[<ets>Thio-</ets> + <ets>al</ets>dehyde + <ets>-ine</ets>.]</ety> <fld>(Chem.)</fld> <def>A weak nitrogenous sulphur base, <chform>C6H13NS2</chform>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Thialol</ent><br/
<hw>Thi"al*ol</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[<ets>Thio-</ets> + <ets>al</ets>cohol + L. <ets>ol</ets>eum oil.]</ety> <fld>(Chem.)</fld> <def>A colorless oily liquid, <chform>(C2H5)2S2</chform>, having a strong garlic odor; -- called also <altname>ethyl disulphide</altname>. By extension, any one of the series of related compounds.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Thibetan</ent><br/
<hw>Thib"e*tan</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Of or pertaining to Thibet.</def> -- <def2><pos>n.</pos> <def>A native or inhabitant of Thibet.</def></def2><-- now usu. Tibetan and Tibet. --><br/
[<source>1913 Webster</source>]</p>

<p><ent>Thibet cloth</ent><br/
<hw>Thib"et cloth`</hw> <pr>(?)</pr>. <sd>(a)</sd> <def>A fabric made of coarse goat's hair; a kind of camlet.</def> <sd>(b)</sd> <def>A kind of fine woolen cloth, used for dresses, cloaks, etc.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Thibetian</ent><br/
<hw>Thi*be"tian</hw> <pr>(?)</pr>, <pos>a. & n.</pos> <def>Same as <er>Thibetan</er>.</def><-- = Tibetan --><br/
[<source>1913 Webster</source>]</p>

<p><ent>Thible</ent><br/
<hw>Thi"ble</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A slice; a skimmer; a spatula; a pudding stick.</def> <mark>[Obs. or Prov. Eng.]</mark>  <rj><au>Ainsworth.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Thick</ent><br/
<hw>Thick</hw> <pr>(th<icr/k)</pr>, <pos>a.</pos> <amorph>[<pos>Compar.</pos> <adjf>Thicker</adjf> <pr>(-<etil/r)</pr>; <pos>superl.</pos> <adjf>Thickest</adjf>.]</amorph> <ety>[OE. <ets>thicke</ets>, AS. <ets><thorn/icce</ets>; akin to D. <ets>dik</ets>, OS. <ets>thikki</ets>, OHG. <ets>dicchi</ets> thick, dense, G. <ets>dick</ets> thick, Icel. <ets><thorn/ykkr</ets>, <ets><thorn/j<oum/kkr</ets>, and probably to Gael. & Ir. <ets>tiugh</ets>.  Cf. <er>Tight</er>.]</ety> <sn>1.</sn> <def>Measuring in the third dimension other than length and breadth, or in general dimension other than length; -- said of a solid body; <as>as, a timber seven inches <ex>thick</ex></as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Were it as <qex>thick</qex> as is a branched oak.</q> <rj><qau>Chaucer.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>My little finger shall be <qex>thicker</qex> than my father's loins.</q> <rj><qau>1 Kings xii. 10.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Having more depth or extent from one surface to its opposite than usual; not thin or slender; <as>as, a <ex>thick</ex> plank; <ex>thick</ex> cloth; <ex>thick</ex> paper; <ex>thick</ex> neck</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>Dense; not thin; inspissated; <as>as, <ex>thick</ex> vapors</as>. Also used figuratively; <as>as, <ex>thick</ex> darkness</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Make the gruel <qex>thick</qex> and slab.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>Not transparent or clear; hence, turbid, muddy, or misty; <as>as, the water of a river is apt to be <ex>thick</ex> after a rain</as>.</def> <ldquo/In a <xex>thick</xex>, misty day.<rdquo/  <rj><au>Sir W. Scott.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <def>Abundant, close, or crowded in space; closely set; following in quick succession; frequently recurring.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The people were gathered <qex>thick</qex> together.</q> <rj><qau>Luke xi. 29.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Black was the forest; <qex>thick</qex> with beech it stood.</q> <rj><qau>Dryden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>6.</sn> <def>Not having due distinction of syllables, or good articulation; indistinct; <as>as, a <ex>thick</ex> utterance</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>7.</sn> <def>Deep; profound; <as>as, <ex>thick</ex> sleep</as>.</def> <mark>[R.]</mark>  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>8.</sn> <def>Dull; not quick; <as>as, <ex>thick</ex> of fearing</as>.</def>  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>His dimensions to any <qex>thick</qex> sight were invincible.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>9.</sn> <def>Intimate; very friendly; familiar.</def> <mark>[Colloq.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>We have been <qex>thick</qex> ever since.</q> <rj><qau>T. Hughes.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ <ex>Thick</ex> is often used in the formation of compounds, most of which are self-explaining; as, <ex>thick</ex>-barred, <ex>thick</ex>-bodied, <ex>thick</ex>-coming, <ex>thick</ex>-cut, <ex>thick</ex>-flying, <ex>thick</ex>-growing, <ex>thick</ex>-leaved, <ex>thick</ex>-lipped, <ex>thick</ex>-necked, <ex>thick</ex>-planted, <ex>thick</ex>-ribbed, <ex>thick</ex>-shelled, <ex>thick</ex>-woven, and the like.</note><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Thick register</b></col>. <fld>(Phon.)</fld> <cd>See the Note under <er>Register</er>, <pos>n.</pos>, 7.</cd> -- <col><b>Thick stuff</b></col> <fld>(Naut.)</fld>, <cd>all plank that is more than four inches thick and less than twelve.</cd> <au>J. Knowles.</au></cs>
<-- -- <col><b>Thick-skulled</b></col>, thick-headed. Stupid, slow to learn. [derogatory] --><br/
[<source>1913 Webster</source>]</p>

<p><syn><b>Syn.</b> -- Dense; close; compact; solid; gross; coarse.</syn><br/
[<source>1913 Webster</source>]</p>

<p><ent>Thick</ent><br/
<hw>Thick</hw>, <pos>n.</pos> <sn>1.</sn> <def>The thickest part, or the time when anything is thickest.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>In the <qex>thick</qex> of the dust and smoke.</q> <rj><qau>Knolles.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A thicket; <as>as, gloomy <ex>thicks</ex></as>.</def> <mark>[Obs.]</mark>  <rj><au>Drayton.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Through the <qex>thick</qex> they heard one rudely rush.</q> <rj><qau>Spenser.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>He through a little window cast his sight<br/
Through <qex>thick</qex> of bars, that gave a scanty light.</q> <rj><qau>Dryden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Thick-and-thin block</b></col> <fld>(Naut.)</fld>, <cd>a fiddle block. See under <er>Fiddle</er>.</cd> -- <col><b>Through thick and thin</b></col>, <cd>through all obstacles and difficulties, both great and small.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><q><qex>Through thick and thin</qex> she followed him.</q> <rj><qau>Hudibras.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>He became the panegyrist, <qex>through thick and thin</qex>, of a military frenzy.</q> <rj><qau>Coleridge.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Thick</ent><br/
<hw>Thick</hw> <pr>(th<icr/k)</pr>, <pos>adv.</pos> <ety>[AS. <ets><thorn/icce</ets>.]</ety> <sn>1.</sn> <def>Frequently; fast; quick.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Closely; <as>as, a plat of ground <ex>thick</ex> sown</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>To a great depth, or to a greater depth than usual; <as>as, land covered <ex>thick</ex> with manure</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Thick and threefold</b></col>, <cd>in quick succession, or in great numbers.</cd> <mark>[Obs.]</mark> <au>L'Estrange.</au></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Thick</ent><br/
<hw>Thick</hw>, <pos>v. t. & i.</pos> <ety>[Cf. AS. <ets><thorn/iccian</ets>.]</ety> <def>To thicken.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>The nightmare Life-in-death was she,<br/
Who <qex>thicks</qex> man's blood with cold.</q> <rj><qau>Coleridge.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Thickbill</ent><br/
<hw>Thick"bill`</hw> <pr>(?)</pr>, <pos>n.</pos> <def>The bullfinch.</def> <mark>[Prov. Eng.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Thicken</ent><br/
<hw>Thick"en</hw> <pr>(?)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Thickened</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Thickening</conjf>.]</vmorph> <def>To make thick (in any sense of the word).</def> <specif>Specifically:</specif> --<br/
[<source>1913 Webster</source>]</p>

<p><sd>(a)</sd> <def>To render dense; to inspissate; <as>as, to <ex>thicken</ex> paint</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sd>(b)</sd> <def>To make close; to fill up interstices in; <as>as, to <ex>thicken</ex> cloth; to <ex>thicken</ex> ranks of trees or men</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sd>(c)</sd> <def>To strengthen; to confirm.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>And this may to <qex>thicken</qex> other proofs.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sd>(d)</sd> <def>To make more frequent; <as>as, to <ex>thicken</ex> blows</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Thicken</ent><br/
<hw>Thick"en</hw>, <pos>v. i.</pos> <def>To become thick.</def> <ldquo/Thy luster <xex>thickens</xex> when he shines by.<rdquo/  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>The press of people <qex>thickens</qex> to the court.</q> <rj><qau>Dryden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>The combat <qex>thickens</qex>, like the storm that flies.</q> <rj><qau>Dryden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Thickening</ent><br/
<hw>Thick"en*ing</hw>, <pos>n.</pos> <def>Something put into a liquid or mass to make it thicker.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Thicket</ent><br/
<hw>Thick"et</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[AS. <ets><thorn/iccet</ets>. See <er>Thick</er>, <pos>a.</pos>]</ety> <def>A wood or a collection of trees, shrubs, etc., closely set; <as>as, a ram caught in a <ex>thicket</ex></as>.</def>  <rj><au>Gen. xxii. 13.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Thickhead</ent><br/
<hw>Thick"head`</hw> <pr>(?)</pr>, <pos>n.</pos> <sn>1.</sn> <def>A thick-headed or stupid person.</def> <mark>[Colloq.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Zool.)</fld> <def>Any one of several species of Australian singing birds of the genus <gen>Pachycephala</gen>. The males of some of the species are bright-colored. Some of the species are popularly called <altname>thrushes</altname>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Thick-headed</ent><br/
<hw>Thick"-head`ed</hw>, <pos>a.</pos> <def>Having a thick skull; stupid.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Thickish</ent><br/
<hw>Thick"ish</hw>, <pos>a.</pos> <def>Somewhat thick.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Thick-knee</ent><br/
<hw>Thick"-knee`</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Zool.)</fld> <def>A stone curlew. See under <er>Stone</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Thickly</ent><br/
<hw>Thick"ly</hw>, <pos>adv.</pos> <def>In a thick manner; deeply; closely.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Thickness</ent><br/
<hw>Thick"ness</hw>, <pos>n.</pos> <ety>[AS. <ets><?/icnes</ets>.]</ety> <def>The quality or state of being thick (in any of the senses of the adjective).</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Thickset</ent><br/
<hw>Thick"set`</hw> <pr>(?)</pr>, <pos>a.</pos> <sn>1.</sn> <def>Close planted; <as>as, a <ex>thickset</ex> wood; a <ex>thickset</ex> hedge</as>.</def>  <rj><au>Dryden.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Having a short, thick body; stout.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Thickset</ent><br/
<hw>Thick"set`</hw>, <pos>n.</pos> <sn>1.</sn> <def>A close or thick hedge.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A stout, twilled cotton cloth; a fustian corduroy, or velveteen.</def>  <rj><au>McElrath.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Thickskin</ent><br/
<hw>Thick"skin`</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A coarse, gross person; a person void of sensibility or sinsitiveness; a dullard.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Thick-skinned</ent><br/
<hw>Thick"-skinned`</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Having a thick skin; hence, not sensitive; dull; obtuse.</def>  <rj><au>Holland.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Thickskull</ent><br/
<hw>Thick"skull`</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A dullard, or dull person; a blockhead; a numskull.</def>  <rj><au>Entick.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Thick-skulled</ent><br/
<hw>Thick"-skulled`</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Having a thick skull; hence, dull; heavy; stupid; slow to learn.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Thick wind</ent><br/
<hw>Thick" wind`</hw> <pr>(?)</pr>. <fld>(Far.)</fld> <def>A defect of respiration in a horse, that is unassociated with noise in breathing or with the signs of emphysema.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Thick-winded</ent><br/
<hw>Thick"-wind`ed</hw>, <pos>a.</pos> <fld>(Far.)</fld> <def>Affected with thick wind.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Thider</ent><br/
<hw>Thid"er</hw> <pr>(?)</pr>, <pos>adv.</pos> <def>Thither.</def> <mark>[Obs.]</mark>  <rj><au>Chaucer.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Thiderward</ent><br/
<hw>Thid"er*ward</hw> <pr>(?)</pr>, <pos>adv.</pos> <def>Thitherward.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Thief</ent><br/
<hw>Thief</hw> <pr>(th<emac/f)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Thieves</plw> <pr>(th<emac/vz)</pr>.</plu> <ety>[OE. <ets>thef</ets>, <ets>theef</ets>, AS. <ets><thorn/e<oacute/f</ets>; akin to OFries. <ets>thiaf</ets>, OS. <ets>theof</ets>, <ets>thiof</ets>, D. <ets>dief</ets>, G. <ets>dieb</ets>, OHG. <ets>diob</ets>, Icel. <ets><thorn/j<omac/fr</ets>, Sw. <ets>tjuf</ets>, Dan. <ets>tyv</ets>, Goth. <ets><thorn/iufs</ets>, <ets><thorn/iubs</ets>, and perhaps to Lith. <ets>tupeti</ets> to squat or crouch down.  Cf. <er>Theft</er>.]</ety> <sn>1.</sn> <def>One who steals; one who commits theft or larceny. See <er>Theft</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>There came a privy <qex>thief</qex>, men clepeth death.</q> <rj><qau>Chaucer.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Where <qex>thieves</qex> break through and steal.</q> <rj><qau>Matt. vi. 19.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A waster in the snuff of a candle.</def>  <rj><au>Bp. Hall.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Thief catcher</b></col>. <cd>Same as <er>Thief taker</er>.</cd> -- <col><b>Thief leader</b></col>, <cd>one who leads or takes away a thief.</cd> <au>L'Estrange.</au> -- <col><b>Thief taker</b></col>, <cd>one whose business is to find and capture thieves and bring them to justice.</cd> -- <col><b>Thief tube</b></col>, <cd>a tube for withdrawing a sample of a liquid from a cask.</cd> -- <col><b>Thieves' vinegar</b></col>, <cd>a kind of aromatic vinegar for the sick room, taking its name from the story that thieves, by using it, were enabled to plunder, with impunity to health, in the great plague at London.</cd> <mark>[Eng.]</mark></cs><br/
[<source>1913 Webster</source>]</p>

<p><syn><b>Syn.</b> -- Robber; pilferer.</syn> <usage> -- <er>Thief</er>, <er>Robber</er>. A <xex>thief</xex> takes our property by stealth; a <xex>robber</xex> attacks us openly, and strips us by main force.</usage><br/
[<source>1913 Webster</source>]</p>

<p><q>Take heed, have open eye, for <qex>thieves</qex> do foot by night.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Some roving <qex>robber</qex> calling to his fellows.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Thiefly</ent><br/
<hw>Thief"ly</hw>, <pos>a. & adv.</pos> <def>Like a thief; thievish; thievishly.</def> <mark>[Obs.]</mark>  <rj><au>Chaucer.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Thienone</ent><br/
<hw>Thi"e*none</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[<ets>Thien</ets>yl + ket<ets>one</ets>.]</ety> <fld>(Chem.)</fld> <def>A ketone derivative of thiophene obtained as a white crystalline substance, <chform>(C4H3S)2.CO</chform>, by the action of aluminum chloride and carbonyl chloride on thiophene.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Thienyl</ent><br/
<hw>Thi"e*nyl</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[<ets>Thi</ets>oph<ets>ene</ets> + <ets>-yl</ets>.]</ety> <fld>(Chem.)</fld> <def>The hypothetical radical <chform>C4H3S.</chform>, regarded as the essential residue of thiophene and certain of its derivatives.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Thieve</ent><br/
<hw>Thieve</hw> <pr>(?)</pr>, <pos>v. t. & i.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Thieved</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Thieving</conjf>.]</vmorph> <ety>[AS. ge<ets><thorn/e<oacute/fian</ets>.]</ety> <def>To practice theft; to steal.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Thievery</ent><br/
<hw>Thiev"er*y</hw> <pr>(?)</pr>, <pos>n.</pos> <sn>1.</sn> <def>The practice of stealing; theft; thievishness.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Among the Spartans, <qex>thievery</qex> was a practice morally good and honest.</q> <rj><qau>South.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>That which is stolen.</def> <mark>[Obs.]</mark>  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Thievish</ent><br/
<hw>Thiev"ish</hw>, <pos>a.</pos> <sn>1.</sn> <def>Given to stealing; addicted to theft; <as>as, a <ex>thievish</ex> boy, a <ex>thievish</ex> magpie</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Like a thief; acting by stealth; sly; secret.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Time's <qex>thievish</qex> progress to eternity.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>Partaking of the nature of theft; accomplished by stealing; dishonest; <as>as, a <ex>thievish</ex> practice</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Or with a base and biosterous sword enforce<br/
A <qex>thievish</qex> living on the common road.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p>-- <wordforms><wf>Thiev"ish*ly</wf>, <pos>adv.</pos> -- <wf>Thiev"ish*ness</wf>, <pos>n.</pos></wordforms><br/
[<source>1913 Webster</source>]</p>

<p><ent>Thigh</ent><br/
<hw>Thigh</hw> <pr>(th<imac/)</pr>, <pos>n.</pos> <ety>[OE. <ets>thi</ets>, <ets><thorn/ih</ets>, <ets><thorn/eh</ets>, AS. <ets><thorn/e<oacute/h</ets>; akin to OFries. <ets>thiach</ets>, D. <ets>dij</ets>, <ets>dije</ets>, OHG. <ets>dioh</ets>, <ets>thioh</ets>, Icel. <ets><thorn/j<omac/</ets> thigh, rump, and probably to Lith. <ets>taukas</ets> fat of animals, <ets>tukti</ets> to become fat, Russ. <ets>tuke</ets> fat of animals. <root/56.]</ety> <sn>1.</sn> <fld>(Anat.)</fld> <def>The proximal segment of the hind limb between the knee and the trunk. See <er>Femur</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Zool.)</fld> <def>The coxa, or femur, of an insect.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Thigh bone</b></col> <fld>(Anat.)</fld>, <cd>the femur.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Thigmotactic</ent><br/
<hw>Thig`mo*tac"tic</hw> <pr>(?)</pr>, <pos>a.</pos> <fld>(Physiol.)</fld> <def>Of or pertaining to thigmotaxis.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Thigmotaxis</ent><br/
||<hw>Thig`mo*tax"is</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[NL., fr. Gr. <?/ touch + <?/ an arranging.]</ety> <fld>(Physiol.)</fld> <def>The property possessed by living protoplasm of contracting, and thus moving, when touched by a solid or fluid substance.</def></p>

<p><note><hand/ When the movement is away from the touching body, it is <stype>negative thigmotaxis</stype>; when towards it, <stype>positive thigmotaxis</stype>.</note><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Thilk</ent><br/
<hw>Thilk</hw> <pr>(?)</pr>, <pos>pron.</pos> <ety>[Cf. <er>Ilk</er> same.]</ety> <def>That same; this; that.</def> <mark>[Obs.]</mark> <ldquo/I love <xex>thilk</xex> lass.<rdquo/  <rj><au>Spenser.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Thou spake right now of <qex>thilke</qex> traitor death.</q> <rj><qau>Chaucer.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Thill</ent><br/
<hw>Thill</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[OE. <ets>thille</ets>, AS. <ets><?/ille</ets> a board, plank, beam, thill; akin to <ets><?/el</ets> a plank, D. <ets>deel</ets> a plank, floor, G. <ets>diele</ets>, OHG. <ets>dili</ets>, <ets>dilla</ets>, Icel. <ets><?/ilja</ets> a plank, planking, a thwart, <ets><?/ili</ets> a wainscot, plank; cf. Skr. <ets>tala</ets> a level surface. <root/236.  Cf. <er>Fill</er> a thill, <er>Deal</er> a plank.]</ety> <sn>1.</sn> <def>One of the two long pieces of wood, extending before a vehicle, between which a horse is hitched; a shaft.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Mining)</fld> <def>The floor of a coal mine.</def>  <rj><au>Raymond.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Thill coupling</b></col>, <cd>a device for connecting the thill of a vehicle to the axle.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Thiller</ent><br/
<hw>Thill"er</hw> <pr>(?)</pr>, <pos>n.</pos> <def>The horse which goes between the thills, or shafts, and supports them; also, the last horse in a team; -- called also <altname>thill horse</altname>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Thimble</ent><br/
<hw>Thim"ble</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[OE. <ets>thimbil</ets>, AS. <ets><?/<?/mel</ets>, fr. <ets><?/<?/ma</ets> a thumb. <root/56. See <er>Thumb</er>.]</ety> <sn>1.</sn> <def>A kind of cap or cover, or sometimes a broad ring, for the end of the finger, used in sewing to protect the finger when pushing the needle through the material. It is usually made of metal, and has upon the outer surface numerous small pits to catch the head of the needle.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Mech.)</fld> <def>Any thimble-shaped appendage or fixure.</def> Specifically: -- <sd>(a)</sd> <def>A tubular piece, generally a strut, through which a bolt or pin passes.</def> <sd>(b)</sd> <def>A fixed or movable ring, tube, or lining placed in a hole.</def> <sd>(c)</sd> <def>A tubular cone for expanding a flue; -- called <altname>ferrule</altname> in England.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <fld>(Naut.)</fld> <def>A ring of thin metal formed with a grooved circumference so as to fit within an eye-spice, or the like, and protect it from chafing.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Thimbleberry</ent><br/
<hw>Thim"ble*ber`ry</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Bot.)</fld> <def>A kind of black raspberry (<spn>Rubus occidentalis</spn>), common in America.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Thimbleeye</ent><br/
<hw>Thim"ble*eye`</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Zool.)</fld> <def>The chub mackerel. See under <er>Chub</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Thimbleful</ent><br/
<hw>Thim"ble*ful</hw> <pr>(?)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Thimblefuls</plw> <pr>(<?/)</pr>.</plu> <def>As much as a thimble will hold; a very small quantity.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>For a <qex>thimbleful</qex> of golf, a <qex>thimbleful</qex> of love.</q> <rj><qau>Dryden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Thimblerig</ent><br/
<hw>Thim"ble*rig`</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A sleight-of-hand trick played with three small cups, shaped like thimbles, and a small ball or little pea.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Thimblerig</ent><br/
<hw>Thim"ble*rig`</hw>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Thimblerigged</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Thimblerigging</conjf>.]</vmorph> <def>To swindle by means of small cups or thimbles, and a pea or small ball placed under one of them and quickly shifted to another, the victim laying a wager that he knows under which cup it is; hence, to cheat by any trick.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Thimblerigger</ent><br/
<hw>Thim"ble*rig`ger</hw> <pr>(?)</pr>, <pos>n.</pos> <def>One who cheats by thimblerigging, or tricks of legerdemain.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Thimbleweed</ent><br/
<hw>Thim"ble*weed`</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Bot.)</fld> <def>Any plant of the composite genus <gen>Rudbeckia</gen>, coarse herbs somewhat resembling the sunflower; -- so called from their conical receptacles.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Thin</ent><br/
<hw>Thin</hw> <pr>(?)</pr>, <pos>a.</pos> <amorph>[<pos>Compar.</pos> <adjf>Thiner</adjf> <pr>(?)</pr>; <pos>superl.</pos> <adjf>Thinest</adjf>.]</amorph> <ety>[OE. <ets>thinne</ets>, <ets>thenne</ets>, <ets>thunne</ets>, AS. <ets><thorn/ynne</ets>; akin to D. <ets>dun</ets>, G. <ets>d<uum/nn</ets>, OHG. <ets>dunni</ets>, Icel. <ets><thorn/unnr</ets>, Sw. <ets>tunn</ets>, Dan. <ets>tynd</ets>, Gael. & Ir. <ets>tana</ets>, W. <ets>teneu</ets>, L. <ets>tenuis</ets>, Gr. <?/ (in comp.) stretched out, <?/ stretched, stretched out, long, Skr. <ets>tanu</ets> thin, slender; also to AS. <ets><?/enian</ets> to extend, G. <ets>dehnen</ets>, Icel. <ets><?/enja</ets>, Goth. <ets><?/anjan</ets> (in comp.), L. <ets>tendere</ets> to stretch, <ets>tenere</ets> to hold, Gr. <?/ to stretch, Skr. <ets>tan</ets>. <root/51 & 237.   Cf. <er>Attenuate</er>, <er>Dance</er>, <er>Tempt</er>, <er>Tenable</er>, <er>Tend</er> to move, <er>Tenous</er>, <er>Thunder</er>, <er>Tone</er>.]</ety> <sn>1.</sn> <def>Having little thickness or extent from one surface to its opposite; <as>as, a <ex>thin</ex> plate of metal; <ex>thin</ex> paper; a <ex>thin</ex> board; a <ex>thin</ex> covering.</as></def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Rare; not dense or thick; -- applied to fluids or soft mixtures; <as>as, <ex>thin</ex> blood; <ex>thin</ex> broth; <ex>thin</ex> air</as>.</def>  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>In the day, when the air is more <qex>thin</qex>.</q> <rj><qau>Bacon.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Satan, bowing low<br/
His gray dissimulation, disappeared,<br/
Into <qex>thin</qex> air diffused.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>Not close; not crowded; not filling the space; not having the individuals of which the thing is composed in a close or compact state; hence, not abundant; <as>as, the trees of a forest are <ex>thin</ex>; the corn or grass is <ex>thin</ex>.</as></def><br/
[<source>1913 Webster</source>]</p>

<p><q>Ferrara is very large, but extremely <qex>thin</qex> of people.</q> <rj><qau>Addison.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>Not full or well grown; wanting in plumpness.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Seven <qex>thin</qex> ears . . . blasted with the east wind.</q> <rj><qau>Gen. xli. 6.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <def>Not stout; slim; slender; lean; gaunt; <as>as, a person becomes <ex>thin</ex> by disease</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>6.</sn> <def>Wanting in body or volume; small; feeble; not full.</def><br/
[<source>1913 Webster</source>]</p>

<p><q><qex>Thin</qex>, hollow sounds, and lamentable screams.</q> <rj><qau>Dryden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>7.</sn> <def>Slight; small; slender; flimsy; wanting substance or depth or force; superficial; inadequate; not sufficient for a covering; <as>as, a <ex>thin</ex> disguise</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>My tale is done, for my wit is but <qex>thin</qex>.</q> <rj><qau>Chaucer.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ <ex>Thin</ex> is used in the formation of compounds which are mostly self-explaining; as, <ex>thin</ex>-faced, <ex>thin</ex>-lipped, <ex>thin</ex>-peopled, <ex>thin</ex>-shelled, and the like.</note><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Thin section</b></col>. <cd>See under <er>Section</er>.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Thin</ent><br/
<hw>Thin</hw>, <pos>adv.</pos> <def>Not thickly or closely; in a seattered state; <as>as, seed sown <ex>thin</ex></as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Spain is <qex>thin</qex> sown of people.</q> <rj><qau>Bacon.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Thin</ent><br/
<hw>Thin</hw>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Thinned</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Thinning</conjf>.]</vmorph> <ety>[Cf. AS. ge<ets><thorn/ynnian</ets>.]</ety> <def>To make thin (in any of the senses of the adjective).</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Thin</ent><br/
<hw>Thin</hw>, <pos>v. i.</pos> <def>To grow or become thin; -- used with some adverbs, as <xex>out</xex>, <xex>away</xex>, etc.; <as>as, geological strata <ex>thin</ex> out, <it>i. e.</it>, gradually diminish in thickness until they disappear</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Thine</ent><br/
<hw>Thine</hw> <pr>(<th/<imac/n)</pr>, <pos>pron. & a.</pos> <ety>[OE. <ets>thin</ets>, AS. <ets><edh/<imac/n</ets>, originally gen. of <ets><edh/u</ets>, <ets><edh/<umac/</ets>, thou; akin to G. <ets>dein</ets> thine, Icel. <ets><thorn/inn</ets>, possessive pron., <ets><thorn/<imac/n</ets>, gen. of <thorn/<umac/ thou, Goth. <ets><thorn/eins</ets>, possessive pron., <ets><thorn/eina</ets>, gen. of <ets><thorn/u</ets> thou. See <er>Thou</er>, and cf. <er>Thy</er>.]</ety> <def>A form of the possessive case of the pronoun <xex>thou</xex>, now superseded in common discourse by <xex>your</xex>, the possessive of <xex>you</xex>, but maintaining a place in solemn discourse, in poetry, and in the usual language of the Friends, or Quakers.</def><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ In the old style, <xex>thine</xex> was commonly shortened to <xex>thi</xex> (<xex>thy</xex>) when used attributively before words beginning with a consonant; now, <xex>thy</xex> is used also before vowels. <xex>Thine</xex> is often used absolutely, the thing possessed being understood.</note><br/
[<source>1913 Webster</source>]</p>

<p><-- p. 1499 --><br/
[<source>1913 Webster</source>]</p>

<p><ent>Thing</ent><br/
<hw>Thing</hw> <pr>(th<icr/ng)</pr>, <pos>n.</pos> <ety>[AS. <ets><thorn/ing</ets> a thing, cause, assembly, judicial assembly; akin to <ets><thorn/ingan</ets> to negotiate, <ets><thorn/ingian</ets> to reconcile, conciliate, D. <ets>ding</ets> a thing, OS. <ets>thing</ets> thing, assembly, judicial assembly, G. <ets>ding</ets> a thing, formerly also, an assembly, court, Icel. <ets><thorn/ing</ets> a thing, assembly, court, Sw. & Dan. <ets>ting</ets>; perhaps originally used of the transaction of or before a popular assembly, or the time appointed for such an assembly; cf. G. <ets>dingen</ets> to bargain, hire, MHG. <ets>dingen</ets> to hold court, speak before a court, negotiate, Goth. <ets><thorn/eihs</ets> time, perhaps akin to L. <ets>tempus</ets> time.  Cf. <er>Hustings</er>, and <er>Temporal</er> of time.]</ety> <sn>1.</sn> <def>Whatever exists, or is conceived to exist, as a separate entity, whether animate or inanimate; any separable or distinguishable object of thought.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>God made . . . every <qex>thing</qex> that creepeth upon the earth after his kind.</q> <rj><qau>Gen. i. 25.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>He sent after this manner; ten asses laden with the good <qex>things</qex> of Egypt.</q> <rj><qau>Gen. xiv. 23.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>A <qex>thing</qex> of beauty is a joy forever.</q> <rj><qau>Keats.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>An inanimate object, in distinction from a living being; any lifeless material.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Ye meads and groves, unconscious <qex>things</qex>!</q> <rj><qau>Cowper.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>A transaction or occurrence; an event; a deed.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>[And Jacob said] All these <qex>things</qex> are against me.</q> <rj><qau>Gen. xlii. 36.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Which if ye tell me, I in like wise will tell you by what authority I do these <qex>things</qex>.</q> <rj><qau>Matt. xxi. 24.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>A portion or part; something.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Wicked men who understand any <qex>thing</qex> of wisdom.</q> <rj><qau>Tillotson.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <def>A diminutive or slighted object; any object viewed as merely existing; -- often used in pity or contempt.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>See, sons, what <qex>things</qex> you are!</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>The poor <qex>thing</qex> sighed, and . . . turned from me.</q> <rj><qau>Addison.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>I'll be this abject <qex>thing</qex> no more.</q> <rj><qau>Granville.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>I have a <qex>thing</qex> in prose.</q> <rj><qau>Swift.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>6.</sn> <pluf>pl.</pluf> <def>Clothes; furniture; appurtenances; luggage; <as>as, to pack or store one's <ex>things</ex></as>.</def> <mark>[Colloq.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ Formerly, the singular was sometimes used in a plural or collective sense.</note><br/
[<source>1913 Webster</source>]</p>

<p><q>And them she gave her moebles and her <qex>thing</qex>.</q> <rj><qau>Chaucer.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ <xex>Thing</xex> was used in a very general sense in Old English, and is still heard colloquially where some more definite term would be used in careful composition.</note><br/
[<source>1913 Webster</source>]</p>

<p><q>In the garden [he] walketh to and fro,<br/
And hath his <qex>things</qex> [<it>i. e.</it>, prayers, devotions] said full courteously.</q> <rj><qau>Chaucer.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Hearkening his minstrels their <qex>things</qex> play.</q> <rj><qau>Chaucer.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>7.</sn> <fld>(Law)</fld> <def>Whatever may be possessed or owned; a property; -- distinguished from <xex>person</xex>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>8.</sn> <pr>[<it>In this sense pronounced</it> t<icr/ng.]</pr> <def>In Scandinavian countries, a legislative or judicial assembly.</def>  <rj><au>Longfellow.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Things personal</b></col>. <fld>(Law)</fld> <cd>Same as <cref>Personal property</cref>, under <er>Personal</er>.</cd> -- <col><b>Things real</b></col>. <cd>Same as <cref>Real property</cref>, under <er>Real</er>.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Ting</ent><br/
<ent>Thing</ent><br/
<mhw>{ <hw>Thing</hw>, ||<hw>Ting</hw>  }</mhw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Dan. <ets>thing</ets>, <ets>ting</ets>, Norw. <ets>ting</ets>, or Sw. <ets>ting</ets>.]</ety> <def>In Scandinavian countries, a legislative or judicial assembly; -- used, esp. in composition, in titles of such bodies. See <er>Legislature</er>, <xex>Norway</xex>.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Think</ent><br/
<hw>Think</hw> <pr>(?)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Thought</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Thinking</conjf>.]</vmorph> <ety>[OE. <ets>thinken</ets>, properly, to seem, from AS. <ets><thorn/yncean</ets> (cf. <er>Methinks</er>), but confounded with OE. <ets>thenken</ets> to think, fr. AS. <ets><thorn/encean</ets> (imp. <ets><thorn/<omac/hte</ets>); akin to D. <ets>denken</ets>, <ets>dunken</ets>, OS. <ets>thenkian</ets>, <ets>thunkian</ets>, G. <ets>denken</ets>, <ets>d<uum/nken</ets>, Icel. <ets><thorn/ekkja</ets> to perceive, to know, <ets><thorn/ykkja</ets> to seem, Goth. <ets><thorn/agkjan</ets>, <ets><thorn/aggkjan</ets>, to think, <ets><thorn/ygkjan</ets> to think, to seem, OL. <ets>tongere</ets> to know.  Cf. <er>Thank</er>, <er>Thought</er>.]</ety> <sn>1.</sn> <def>To seem or appear; -- used chiefly in the expressions <xex>methinketh</xex> or <xex>methinks</xex>, and <xex>methought</xex>.</def>
<-- structurally similar to Russ. mne kazhetsya --><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ These are genuine Anglo-Saxon expressions, equivalent to <xex>it seems to me</xex>, <xex>it seemed to me</xex>. In these expressions <xex>me</xex> is in the dative case.</note><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To employ any of the intellectual powers except that of simple perception through the senses; to exercise the higher intellectual faculties.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>For that I am<br/
I know, because I <qex>think</qex>.</q> <rj><qau>Dryden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> Specifically: -- <sd>(a)</sd> <def>To call anything to mind; to remember; <as>as, I would have sent the books, but I did not <ex>think</ex> of it</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Well <qex>thought</qex> upon; I have it here.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sd>(b)</sd> <def>To reflect upon any subject; to muse; to meditate; to ponder; to consider; to deliberate.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>And when he <qex>thought</qex> thereon, he wept.</q> <rj><qau>Mark xiv. 72.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>He <qex>thought</qex> within himself, saying, What shall I do, because I have no room where to bestow my fruits?</q> <rj><qau>Luke xii. 17.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sd>(c)</sd> <def>To form an opinion by reasoning; to judge; to conclude; to believe; <as>as, I <ex>think</ex> it will rain to-morrow</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Let them marry to whom they <qex>think</qex> best.</q> <rj><qau>Num. xxxvi. 6.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sd>(d)</sd> <def>To purpose; to intend; to design; to mean.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>I <qex>thought</qex> to promote thee unto great honor.</q> <rj><qau>Num. xxiv. 11.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Thou <qex>thought'st</qex> to help me.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sd>(e)</sd> <def>To presume; to venture.</def><br/
[<source>1913 Webster</source>]</p>

<p><q><qex>Think</qex> not to say within yourselves, We have Abraham to our father.</q> <rj><qau>Matt. iii. 9.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ <xex>To think</xex>, in a philosophical use as yet somewhat limited, designates the higher intellectual acts, the acts preeminently rational; to judge; to compare; to reason. Thinking is employed by Hamilton as <ldquo/comprehending all our collective energies.<rdquo/ It is defined by Mansel as <ldquo/the act of knowing or judging by means of concepts,<rdquo/by Lotze as <ldquo/the reaction of the mind on the material supplied by external influences.<rdquo/ See <er>Thought</er>.</note><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>To think better of</b></col>. <cd>See under <er>Better</er>.</cd> -- <mcol><col><b>To think much of</b></col>, <it>or</it> <col><b>To think well of</b></col></mcol>, <cd>to hold in esteem; to esteem highly.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><syn><b>Syn.</b> -- To expect; guess; cogitate; reflect; ponder; contemplate; meditate; muse; imagine; suppose; believe. See <er>Expect</er>, <er>Guess</er>.</syn><br/
[<source>1913 Webster</source>]</p>

<p><ent>Think</ent><br/
<hw>Think</hw>, <pos>v. t.</pos> <sn>1.</sn> <def>To conceive; to imagine.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Charity . . . <qex>thinketh</qex> no evil.</q> <rj><qau>1 Cor. xiii. 4,5.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To plan or design; to plot; to compass.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>So little womanhood<br/
And natural goodness, as to <qex>think</qex> the death<br/
Of her own son.</q> <rj><qau>Beau. & Fl.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>To believe; to consider; to esteem.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Nor <qex>think</qex> superfluous other's aid.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>To think much</b></col>, <cd>to esteem a great matter; to grudge.</cd> <mark>[Obs.]</mark> <ldquo/[He] <xex>thought</xex> not <xex>much</xex> to clothe his enemies.<rdquo/ <au>Milton.</au> -- <col><b>To think scorn</b></col>. <sd>(a)</sd> <cd>To disdain.</cd> <mark>[Obs.]</mark> <ldquo/He <xex>thought scorn</xex> to lay hands on Mordecai alone.<rdquo/ <au>Esther iii. 6.</au> <sd>(b)</sd> <cd>To feel indignation.</cd> <mark>[Obs.]</mark></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Think</ent><br/
<hw>Think</hw>, <pos>n.</pos> <def>Act of thinking; a thought.</def> <ldquo/If you think that I'm finished, you've got another <ex>think</ex> coming!<rdquo/ <mark>[Obs. or Colloq.]</mark><br/
[<source>Webster 1913 Suppl.</source> <source>+PJC</source>]</p>

<p><ent>Thinkable</ent><br/
<hw>Think"a*ble</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Capable of being thought or conceived; cogitable.</def>  <rj><au>Sir W. Hamilton.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Thinker</ent><br/
<hw>Think"er</hw> <pr>(?)</pr>, <pos>n.</pos> <def>One who thinks; especially and chiefly, one who thinks in a particular manner; <as>as, a close <ex>thinker</ex>; a deep <ex>thinker</ex>; a coherent <ex>thinker</ex>.</as></def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Thinking</ent><br/
<hw>Think"ing</hw>, <pos>a.</pos> <def>Having the faculty of thought; cogitative; capable of a regular train of ideas; <as>as, man is a <ex>thinking</ex> being</as>.</def> -- <wordforms><wf>Think"ing*ly</wf>, <pos>adv.</pos></wordforms><br/
[<source>1913 Webster</source>]</p>

<p><ent>Thinking</ent><br/
<hw>Think"ing</hw>, <pos>n.</pos> <def>The act of thinking; mode of thinking; imagination; cogitation; judgment.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>I heard a bird so sing,<br/
Whose music, to my <qex>thinking</qex>, pleased the king.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Thinly</ent><br/
<hw>Thin"ly</hw> <pr>(?)</pr>, <pos>a.</pos> <def>In a thin manner; in a loose, scattered manner; scantily; not thickly; <as>as, ground <ex>thinly</ex> planted with trees; a country <ex>thinly</ex> inhabited.</as></def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Thinner</ent><br/
<hw>Thin"ner</hw> <pr>(?)</pr>, <pos>n.</pos> <def>One who thins, or makes thinner.</def><br/
[<source>1913 Webster</source>]</p>

<p><-- 2. A solvent used to thin a viscous liquid, as a paint thinner. --><br/
[<source>1913 Webster</source>]</p>

<p><ent>Thinness</ent><br/
<hw>Thin"ness</hw>, <pos>n.</pos> <def>The quality or state of being thin (in any of the senses of the word).</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Thinnish</ent><br/
<hw>Thin"nish</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Somewhat thin.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Thinolite</ent><br/
<hw>Thin"o*lite</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Gr. <?/, <?/, shore + <ets>-lite</ets>.]</ety> <fld>(Min.)</fld> <def>A calcareous tufa, in part crystalline, occurring on a large scale as a shore deposit about the Quaternary lake basins of Nevada.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Thin-skinned</ent><br/
<hw>Thin"-skinned`</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Having a thin skin; hence, sensitive; irritable.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Thio-</ent><br/
<hw>Thi"o-</hw> <pr>(?)</pr>. <ety>[Gr. <?/ brimstone, sulphur.]</ety> <fld>(Chem.)</fld> <def>A combining form (also used adjectively) denoting <xex>the presence of sulphur</xex>. See <er>Sulpho-</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Thiocarbonate</ent><br/
<hw>Thi`o*car"bon*ate</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Chem.)</fld> <def>A sulphocarbonate.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Thiocarbonic</ent><br/
<hw>Thi`o*car*bon"ic</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[<ets>Thio-</ets> + <ets>carbonic</ets>.]</ety> <fld>(Chem.)</fld> <def>Same as <er>Sulphocarbonic</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Thiocyanate</ent><br/
<hw>Thi`o*cy"a*nate</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Chem.)</fld> <def>Same as <er>Sulphocyanate</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Thiocyanic</ent><br/
<hw>Thi`o*cy*an"ic</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[<ets>Thio-</ets> + <ets>cyanic</ets>.]</ety> <fld>(Chem.)</fld> <def>Same as <er>Sulphocyanic</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Thionaphthene</ent><br/
<hw>Thi`o*naph"thene</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[<ets>Thio</ets>phene + <ets>naphth</ets>al<ets>ene</ets>.]</ety> <fld>(Chem.)</fld> <def>A double benzene and thiophene nucleus, <chform>C8H6S</chform>, analogous to naphthalene, and like it the base of a large series of derivatives.</def> <altsp>[Written also <asp>thionaphtene</asp>.]</altsp><br/
[<source>1913 Webster</source>]</p>

<p><ent>Thionic</ent><br/
<hw>Thi*on"ic</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[Gr. <?/ brimstone, sulphur.]</ety> <fld>(Chem.)</fld> <def>Of or pertaining to sulphur; containing or resembling sulphur; specifically, designating certain of the thio compounds; <as>as, the <ex>thionic</ex> acids</as>.  Cf. <er>Dithionic</er>, <er>Trithionic</er>, <er>Tetrathionic</er>, etc.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Thionine</ent><br/
<hw>Thi"on*ine</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Gr. <?/ brimstone, sulphur.]</ety> <fld>(Chem.)</fld> <def>An artificial red or violet dyestuff consisting of a complex sulphur derivative of certain aromatic diamines, and obtained as a dark crystalline powder; -- called also <altname>phenylene violet</altname>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Thionol</ent><br/
<hw>Thi"on*ol</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[<ets>Thion</ets>ine + <ets>-ol</ets>.]</ety> <fld>(Chem.)</fld> <def>A red or violet dyestuff having a greenish metallic luster. It is produced artificially, by the chemical dehydration of thionine, as a brown amorphous powder.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Thionoline</ent><br/
<hw>Thi*on"o*line</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Chem.)</fld> <def>A beautiful fluorescent crystalline substance, intermediate in composition between thionol and thionine.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Thionyl</ent><br/
<hw>Thi"on*yl</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[<ets>Thion</ets>ic + <ets>-yl</ets>.]</ety> <fld>(Chem.)</fld> <def>The hypothetical radical <chform>SO</chform>, regarded as an essential constituent of certain sulphurous compounds; <as>as, <ex>thionyl</ex> chloride</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Thiophene</ent><br/
<hw>Thi"o*phene</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[<ets>Thio-</ets> + <ets>ph</ets>enyl + <ets>-ene</ets>.]</ety> <fld>(Chem.)</fld> <def>A sulphur hydrocarbon, <chform>C4H4S</chform>, analogous to furfuran and benzene, and acting as the base of a large number of substances which closely resemble the corresponding aromatic derivatives.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Thiophenic</ent><br/
<hw>Thi`o*phen"ic</hw> <pr>(?)</pr>, <pos>a.</pos> <fld>(Chem.)</fld> <def>Of, pertaining to, or derived from, thiophene; specifically, designating a certain acid analogous to benzoic acid.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Thiophenol</ent><br/
<hw>Thi`o*phe"nol</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[<ets>Thio-</ets> + <ets>phenol</ets>.]</ety> <fld>(Chem.)</fld> <def>A colorless mobile liquid, <chform>C6H5.SH</chform>, of an offensive odor, and analogous to phenol; -- called also <altname>phenyl sulphydrate</altname>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Thiophthene</ent><br/
<hw>Thi*oph"thene</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Abbreviated from <ets>thio</ets>na<ets>phthene</ets>.]</ety> <fld>(Chem.)</fld> <def>A double thiophene nucleus, <chform>C6H4S2</chform>, analogous to thionaphthene, and the base of a large series of compounds.</def> <altsp>[Written also <asp>thiophtene</asp>.]</altsp><br/
[<source>1913 Webster</source>]</p>

<p><ent>Thiosulphate</ent><br/
<hw>Thi`o*sul"phate</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Chem.)</fld> <def>A salt of thiosulphuric acid; -- formerly called <altname>hyposulphite</altname>.</def><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ The sodium salt called in photography by the name <xex>sodium hyposulphite</xex>, being used as a solvent for the excess of unchanged silver chloride, bromide, and iodide on the sensitive plate.</note><br/
[<source>1913 Webster</source>]</p>

<p><ent>Thiosulphuric</ent><br/
<hw>Thi`o*sul*phur"ic</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[<ets>Thio-</ets> + <ets>sulphuric</ets>.]</ety> <fld>(Chem.)</fld> <def>Of, pertaining to, or designating, an unstable acid, <chform>H2S2O3</chform>, analogous to sulphuric acid, and formerly called <altname>hyposulphurous acid</altname>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Thiotolene</ent><br/
<hw>Thi`o*to"lene</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[<ets>Thio-</ets> + <ets>tol</ets>u<ets>ene</ets>.]</ety> <fld>(Chem.)</fld> <def>A colorless oily liquid, <chform>C4H3S.CH3</chform>, analogous to, and resembling, toluene; -- called also <altname>methyl thiophene</altname>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Thioxene</ent><br/
<hw>Thi*ox"ene</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[<ets>Thio</ets>phene + <ets>x</ets>yl<ets>ene</ets>.]</ety> <fld>(Chem.)</fld> <def>Any one of three possible metameric substances, which are dimethyl derivatives of thiophene, like the xylenes from benzene.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Third</ent><br/
<hw>Third</hw> <pr>(th<etil/rd)</pr>, <pos>a.</pos> <ety>[OE. <ets>thirde</ets>, AS. <ets><thorn/ridda</ets>, fr. <ets><thorn/r<imac/</ets>, <ets><thorn/re<oacute/</ets>, three; akin to D. <ets>derde</ets> third, G. <ets>dritte</ets>, Icel. <ets><thorn/ri<edh/i</ets>, Goth. <ets><thorn/ridja</ets>, L. <ets>tertius</ets>, Gr. <grk>tri`tos</grk>, Skr. <ets>t<rsdot/t<imac/ya</ets>. See <er>Three</er>, and cf. <er>Riding</er> a jurisdiction, <er>Tierce</er>.]</ety> <sn>1.</sn> <def>Next after the second; coming after two others; -- the ordinal of three; <as>as, the <ex>third</ex> hour in the day</as>.</def> <ldquo/The <xex>third</xex> night.<rdquo/  <rj><au>Chaucer.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Constituting or being one of three equal parts into which anything is divided; <as>as, the <ex>third</ex> part of a day</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Third estate</b></col>. <sd>(a)</sd> <cd>In England, the commons, or the commonalty, who are represented in Parliament by the House of Commons.</cd> <sd>(b)</sd> <cd>In France, the tiers <eacute/tat.</cd> See <er>Tiers <eacute/tat</er>. <col><b>Third order</b></col> <fld>(R. C. Ch.)</fld>, <cd>an order attached to a monastic order, and comprising men and women devoted to a rule of pious living, called the third rule, by a simple vow if they remain seculars, and by more solemn vows if they become regulars. See <er>Tertiary</er>, <pos>n.</pos>, 1.</cd> -- <col><b>Third person</b></col> <fld>(Gram.)</fld>, <cd>the person spoken of. See <er>Person</er>, <pos>n.</pos>, 7.</cd> -- <col><b>Third sound</b></col>. <fld>(Mus.)</fld> <cd>See <er>Third</er>, <pos>n.</pos>, 3.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Third</ent><br/
<hw>Third</hw> <pr>(?)</pr>, <pos>n.</pos> <sn>1.</sn> <def>The quotient of a unit divided by three; one of three equal parts into which anything is divided.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>The sixtieth part of a second of time.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <fld>(Mus.)</fld> <def>The third tone of the scale; the mediant.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <pluf>pl.</pluf> <fld>(Law)</fld> <def>The third part of the estate of a deceased husband, which, by some local laws, the widow is entitled to enjoy during her life.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Major third</b></col> <fld>(Mus.)</fld>, <cd>an interval of two tones.</cd> -- <col><b>Minor third</b></col> <fld>(Mus.)</fld>, <cd>an interval of a tone and a half.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Third-borough</ent><br/
<hw>Third"-bor`ough</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(O. Eng. Law)</fld> <def>An under constable.</def>  <rj><au>Shak. Johnson.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Thirdings</ent><br/
<hw>Third"ings</hw> <pr>(?)</pr>, <pos>n. pl.</pos> <fld>(Eng. Law)</fld> <def>The third part of the corn or grain growing on the ground at the tenant's death, due to the lord for a heriot, as within the manor of Turfat in Herefordshire.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Thirdly</ent><br/
<hw>Third"ly</hw>, <pos>adv.</pos> <def>In the third place.</def>  <rj><au>Bacon.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Third-penny</ent><br/
<hw>Third"-pen`ny</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(A.S. Law)</fld> <def>A third part of the profits of fines and penalties imposed at the country court, which was among the perquisites enjoyed by the earl.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Third rail</ent><br/
<hw>Third rail</hw>. <fld>(Electric Railways)</fld> <sd>(a)</sd> <def>The third rail used in the third-rail system.</def> <sd>(b)</sd> <def>An electric railway using such a rail.</def> <mark>[Colloq.]</mark><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Third-rail system</ent><br/
<hw>Third-rail system</hw>. <fld>(Electric Railways)</fld> <def>A system in which a third rail is used for carrying the current for operating the motors, the rail being insulated from the ground and the current being taken off by means of contact brushes or other devices.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Thirl</ent><br/
<hw>Thirl</hw> <pr>(?)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Thirled</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Thirling</conjf>.]</vmorph> <ety>[See <er>Thrill</er>.]</ety> <def>To bore; to drill or thrill. See <er>Thrill</er>.</def> <mark>[Obs. or Prov.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>That with a spear was <qex>thirled</qex> his breast bone.</q> <rj><qau>Chaucer.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Thirlage</ent><br/
<hw>Thirl"age</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Cf. <er>Thrall</er>.]</ety> <fld>(Scots Law)</fld> <def>The right which the owner of a mill possesses, by contract or law, to compel the tenants of a certain district, or of his sucken, to bring all their grain to his mill for grinding.</def>  <rj><au>Erskine.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Thirst</ent><br/
<hw>Thirst</hw> <pr>(th<etil/rst)</pr>, <pos>n.</pos> <ety>[OE. <ets>thirst</ets>, <ets><thorn/urst</ets>, AS. <ets><thorn/urst</ets>, <ets><thorn/yrst</ets>; akin to D. <ets>dorst</ets>, OS. <ets>thurst</ets>, G. <ets>durst</ets>, Icel. <ets><thorn/orsti</ets>, Sw. & Dan. <ets>t<oum/rst</ets>, Goth. <ets><thorn/a<uacute/rstei</ets> thirst, <ets><thorn/a<uacute/rsus</ets> dry, withered, <ets><thorn/a<uacute/rsie<thorn/ mik</ets> I thirst, ga<ets><thorn/a<iacute/rsan</ets> to wither, L. <ets>torrere</ets> to parch, Gr. <grk>te`rsesqai</grk> to become dry, <grk>tesai`nein</grk> to dry up, Skr. <ets>t<rsdot/sh</ets> to thirst. <root/54.  Cf. <er>Torrid</er>.]</ety> <sn>1.</sn> <def>A sensation of dryness in the throat associated with a craving for liquids, produced by deprivation of drink, or by some other cause (as fear, excitement, etc.) which arrests the secretion of the pharyngeal mucous membrane; hence, the condition producing this sensation.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Wherefore is this that thou hast brought us up out of Egypt, to kill us, and our children . . . with <qex>thirst</qex>?</q> <rj><qau>Ex. xvii. 3.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>With <qex>thirst</qex>, with cold, with hunger so confounded.</q> <rj><qau>Chaucer.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Fig.: A want and eager desire after anything; a craving or longing; -- usually with <xex>for</xex>, <xex>of</xex>, or <xex>after</xex>; <as>as, the <ex>thirst</ex> for gold</as>.</def> <ldquo/<xex>Thirst</xex> of worldy good.<rdquo/ <au>Fairfax.</au> <ldquo/The <xex>thirst</xex> I had of knowledge.<rdquo/ <au>Milton.</au><br/
[<source>1913 Webster</source>]</p>

<p><ent>Thirst</ent><br/
<hw>Thirst</hw>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Thirsted</conjf>; <pos>p. pr. & vb. n.</pos> <conjf>Thirsting</conjf>.]</vmorph> <ety>[AS. <ets><thorn/yrstan</ets>. See <er>Thirst</er>, <pos>n.</pos>]</ety> <sn>1.</sn> <def>To feel thirst; to experience a painful or uneasy sensation of the throat or fauces, as for want of drink.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The people <qex>thirsted</qex> there for water.</q> <rj><qau>Ex. xvii. 3.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To have a vehement desire.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>My soul <qex>thirsteth</qex> for . . . the living God.</q> <rj><qau>Ps. xlii. 2.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Thirst</ent><br/
<hw>Thirst</hw>, <pos>v. t.</pos> <def>To have a thirst for.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>He seeks his keeper's flesh, and <qex>thirsts</qex> his blood.</q> <rj><qau>Prior.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Thirster</ent><br/
<hw>Thirst"er</hw> <pr>(?)</pr>, <pos>n.</pos> <def>One who thirsts.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Thirstily</ent><br/
<hw>Thirst"i*ly</hw> <pr>(?)</pr>, <pos>adv.</pos> <def>In a thirsty manner.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Thirstiness</ent><br/
<hw>Thirst"i*ness</hw>, <pos>n.</pos> <def>The state of being thirsty; thirst.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Thirstle</ent><br/
<hw>Thirs"tle</hw> <pr>(?)</pr>, <pos>n.</pos> <def>The throstle.</def> <mark>[Prov. Eng.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Thirsty</ent><br/
<hw>Thirst"y</hw> <pr>(?)</pr>, <pos>a.</pos> <amorph>[<pos>Compar.</pos> <adjf>Thirstier</adjf> <pr>(?)</pr>; <pos>superl.</pos> <adjf>Thirstiest</adjf>.]</amorph> <ety>[AS. <ets><?/urstig</ets>. See <er>Thirst</er>, <pos>n.</pos>]</ety> <sn>1.</sn> <def>Feeling thirst; having a painful or distressing sensation from want of drink; hence, having an eager desire.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Give me, I pray thee, a little water to drink, for I am <qex>thirsty</qex>.</q> <rj><qau>Judges iv. 19.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Deficient in moisture; dry; parched.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>A dry and <qex>thirsty</qex> land, where no water is.</q> <rj><qau>Ps. lxiii. 1.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>When in the sultry glebe I faint,<br/
Or on the <qex>thirsty</qex> mountain pant.</q> <rj><qau>Addison.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Thirteen</ent><br/
<hw>Thir"teen`</hw> <pr>(th<etil/r"t<emac/n`)</pr>, <pos>a.</pos> <ety>[OE. <ets>threttene</ets>, AS. <ets><thorn/re<oacute/t<emac/ne</ets>, <ets><thorn/re<oacute/tyne</ets>. See <er>Three</er>, and <er>Ten</er>, and cf. <er>Thirty</er>.]</ety> <def>One more than twelve; ten and three; <as>as, <ex>thirteen</ex> ounces or pounds</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Thirteen</ent><br/
<hw>Thir"teen`</hw>, <pos>n.</pos> <sn>1.</sn> <def>The number greater by one than twelve; the sum of ten and three; thirteen units or objects.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A symbol representing thirteen units, as 13 or xiii.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Thirteenth</ent><br/
<hw>Thir"teenth`</hw> <pr>(th<etil/r"t<emac/nth`)</pr>, <pos>a.</pos> <ety>[From <er>Thirteen</er>: cf. AS. <ets><thorn/re<oacute/te<oacute/<edh/a</ets>.]</ety> <sn>1.</sn> <def>Next in order after the twelfth; the third after the tenth; -- the ordinal of <xex>thirteen</xex>; <as>as, the <ex>thirteenth</ex> day of the month</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Constituting or being one of thirteen equal parts into which anything is divided.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Thirteenth</ent><br/
<hw>Thir"teenth`</hw>, <pos>n.</pos> <sn>1.</sn> <def>The quotient of a unit divided by thirteen; one of thirteen equal parts into which anything is divided.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>The next in order after the twelfth.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <fld>(Mus.)</fld> <def>The interval comprising an octave and a sixth.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Thirtieth</ent><br/
<hw>Thir"ti*eth</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[From <er>Thirty</er>: cf. AS. <ets><thorn/r<imac/tig<omac/<edh/a</ets>.]</ety> <sn>1.</sn> <def>Next in order after the twenty-ninth; the tenth after the twentieth; -- the ordinal of <xex>thirty</xex>; <as>as, the <ex>thirtieth</ex> day of the month</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Constituting or being one of thirty equal parts into which anything is divided.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Thirtieth</ent><br/
<hw>Thir"ti*eth</hw>, <pos>n.</pos> <def>The quotient of a unit divided by thirty; one of thirty equal parts.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Thirty</ent><br/
<hw>Thir"ty</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[OE. <ets>thritty</ets>, AS. <ets><thorn/r<imac/tig</ets>, <ets><thorn/rittig</ets>; akin to D. <ets>dertig</ets>, G. <ets>dreissig</ets>, Icel. <ets><thorn/rj<amac/t<imac/u</ets>, <ets><thorn/rj<amac/tigi</ets>, <ets><thorn/rir teger</ets>, Goth. <ets><thorn/reis tigjus</ets>, i.e., three tens. See <er>Three</er>, and Ten, and cf. <er>Thirteen</er>.]</ety> <def>Being three times ten; consisting of one more than twenty-nine; twenty and ten; <as>as, the month of June consists of <ex>thirty</ex> days</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><-- p. 1500 --><br/
[<source>1913 Webster</source>]</p>

<p><ent>Thirty</ent><br/
<hw>Thir"ty</hw> <pr>(?)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Thirties</plw> <pr>(<?/)</pr>.</plu> <sn>1.</sn> <def>The sum of three tens, or twenty and ten; thirty units or objects.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A symbol expressing thirty, as 30, or XXX.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Thirty-second</ent><br/
<hw>Thir"ty-sec`ond</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Being one of thirty-two equal parts into which anything is divided.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Thirty-second note</b></col> <fld>(Mus.)</fld>, <cd>the thirty-second part of a whole note; a demi-semiquaver.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>This</ent><br/
<hw>This</hw> <pr>(<th/<icr/s)</pr>, <pos>pron. & a.</pos>; <plu><it>pl.</it> <plw>These</plw> <pr>(<th/<emac/z)</pr>.</plu> <ety>[OE. <ets>this</ets>, <ets>thes</ets>, AS. <ets><edh/<emac/s</ets>, masc., <ets><edh/e<oacute/s</ets>, fem., <ets><edh/is</ets>, neut.; akin to OS. <ets>these</ets>, D. <ets>deze</ets>, G. <ets>dieser</ets>, OHG. <ets>diser</ets>, <ets>deser</ets>, Icel. <ets><thorn/essi</ets>; originally from the definite article + a particle <ets>-se</ets>, <ets>-si</ets>; cf. Goth. <ets>sai</ets> behold. See <er>The</er>, <er>That</er>, and cf. <er>These</er>, <er>Those</er>.]</ety> <sn>1.</sn> <def>As a demonstrative pronoun, <xex>this</xex> denotes something that is present or near in place or time, or something just mentioned, or that is just about to be mentioned.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>When they heard <qex>this</qex>, they were pricked in their heart.</q> <rj><qau>Acts ii. 37.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>But know <qex>this</qex>, that if the good man of the house had known in what watch the thief would come, he would have watched.</q> <rj><qau>Matt. xxiv. 43.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>As an adjective, <xex>this</xex> has the same demonstrative force as the pronoun, but is followed by a noun; <as>as, <ex>this</ex> book; <ex>this</ex> way to town</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ <xex>This</xex> may be used as opposed or correlative to <xex>that</xex>, and sometimes as opposed to <xex>other</xex> or to a second <xex>this</xex>. See the Note under <er>That</er>, 1.</note><br/
[<source>1913 Webster</source>]</p>

<p><q><qex>This</qex> way and <qex>that</qex> wavering sails they bend.</q> <rj><qau>Pope.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>A body of <qex>this</qex> or <qex>that</qex> denomination is produced.</q> <rj><qau>Boyle.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Their judgment in <qex>this</qex> we may not, and in <qex>that</qex> we need not, follow.</q> <rj><qau>Hooker.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Consider the arguments which the author had to write <qex>this</qex>, or to design the <qex>other</qex>, before you arraign him.</q> <rj><qau>Dryden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Thy crimes . . . soon by <qex>this</qex> or <qex>this</qex> will end.</q> <rj><qau>Addison.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ <xex>This</xex>, like <xex>a</xex>, <xex>every</xex>, <xex>that</xex>, etc., may refer to a number, as of years, persons, etc., taken collectively or as a whole.</note><br/
[<source>1913 Webster</source>]</p>

<p><q><qex>This</qex> twenty years have I been with thee..</q> <rj><qau>Gen. xxxi. 38.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>I have not wept <qex>this</qex> years; but now<br/
My mother comes afresh into my eyes.</q> <rj><qau>Dryden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Thistle</ent><br/
<hw>This"tle</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[OE. <ets>thistil</ets>, AS. <ets><thorn/istel</ets>; akin to D. & G. <ets>distel</ets>, OHG. <ets>distila</ets>, <ets>distil</ets>, Icel. <ets><thorn/istill</ets>, Sw. <ets>tistel</ets>, Dan. <ets>tidsel</ets>; of uncertain origin.]</ety> <fld>(Bot.)</fld> <def>Any one of several prickly composite plants, especially those of the genera <gen>Cnicus</gen>, <gen>Craduus</gen>, and <gen>Onopordon</gen>. The name is often also applied to other prickly plants.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Blessed thistle</b></col>, <cd><spn>Carduus benedictus</spn>, so named because it was formerly considered an antidote to the bite of venomous creatures.</cd> -- <col><b>Bull thistle</b></col>, <cd><spn>Cnicus lanceolatus</spn>, the common large thistle of neglected pastures.</cd> -- <col><b>Canada thistle</b></col>, <cd><spn>Cnicus arvensis</spn>, a native of Europe, but introduced into the United States from Canada.</cd> -- <col><b>Cotton thistle</b></col>, <cd><spn>Onopordon Acanthium</spn>.</cd> -- <col><b>Fuller's thistle</b></col>, <cd>the teasel.</cd> -- <mcol><col><b>Globe thistle</b></col>, <col><b>Melon thistle</b></col></mcol>, <cd>etc. See under <er>Globe</er>, <er>Melon</er>, etc.</cd> -- <col><b>Pine thistle</b></col>, <cd><spn>Atractylis gummifera</spn>, a native of the Mediterranean region. A vicid gum resin flows from the involucre.</cd> -- <col><b>Scotch thistle</b></col>, <cd>either the cotton thistle, or the musk thistle, or the spear thistle; -- all used national emblems of Scotland.</cd> -- <col><b>Sow thistle</b></col>, <cd><spn>Sonchus oleraceus</spn>.</cd> -- <col><b>Spear thistle</b></col>. <cd>Same as <er>Bull thistle</er>.</cd> -- <col><b>Star thistle</b></col>, <cd>a species of <gen>Centaurea</gen>. See <er>Centaurea</er>.</cd> -- <col><b>Torch thistle</b></col>, <cd>a candelabra-shaped plant of the genus Cereus. See <er>Cereus</er>.</cd> -- <col><b>Yellow thistle</b></col>, <cd><spn>Cincus horridulus</spn>.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Thistle bird</b></col> <fld>(Zool.)</fld>, <cd>the American goldfinch, or yellow-bird (<spn>Spinus tristis</spn>); -- so called on account of its feeding on the seeds of thistles. See <xex>Illust.</xex> under <er>Goldfinch</er>.</cd> -- <col><b>Thistle butterfly</b></col> <fld>(Zool.)</fld>, <cd>a handsomely colored American butterfly (<spn>Vanessa cardui</spn>) whose larva feeds upon thistles; -- called also <altname>painted lady</altname>.</cd> -- <col><b>Thistle cock</b></col> <fld>(Zool.)</fld>, <cd>the corn bunting (<spn>Emberiza militaria</spn>).</cd> <mark>[Prov. Eng.]</mark> -- <col><b>Thistle crown</b></col>, <cd>a gold coin of England of the reign of James I., worth four shillings.</cd> -- <col><b>Thistle finch</b></col> <fld>(Zool.)</fld>, <cd>the goldfinch; -- so called from its fondness for thistle seeds.</cd> <mark>[Prov. Eng.]</mark> -- <col><b>Thistle funnel</b></col>, <cd>a funnel having a bulging body and flaring mouth.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Thistly</ent><br/
<hw>This"tly</hw> <pr>(?)</pr>, <pos>a.</pos> <sn>1.</sn> <def>Overgrown with thistles; <as>as, <ex>thistly</ex> ground</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Fig.: Resembling a thistle or thistles; sharp; pricking.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>In such a world, so thorny, and where none<br/
Finds happiness unblighted, or, if found,<br/
Without some <qex>thistly</qex> sorrow at its side.</q> <rj><qau>Cowper.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Thither</ent><br/
<hw>Thith"er</hw> <pr>(?)</pr>, <pos>adv.</pos> <ety>[OE. <ets>thider</ets>, AS. <ets><edh/ider</ets>; akin to E. <ets>that</ets>; cf. Icel. <ets><thorn/a<edh/ra</ets> there, Goth. <ets><thorn/a<thorn/r<omac/</ets> thence. See <er>That</er>, and <er>The</er>.]</ety> <sn>1.</sn> <def>To that place; -- opposed to <ant>hither</ant>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>This city is near; . . . O, let me escape <qex>thither</qex>.</q> <rj><qau>Gen. xix. 20.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Where I am, <qex>thither</qex> ye can not come.</q> <rj><qau>John vii. 34.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To that point, end, or result; <as>as, the argument tended <ex>thither</ex></as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Hither and thither</b></col>, <cd>to this place and to that; one way and another.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><syn><b>Syn.</b> -- There.</syn> <usage> <er>Thither</er>, <er>There</er>. <xex>Thither</xex> properly denotes motion toward a place; <xex>there</xex> denotes rest in a place; <as>as, I am going <ex>thither</ex>, and shall meet you <ex>there</ex></as>. But <xex>thither</xex> has now become obsolete, except in poetry, or a style purposely conformed to the past, and <xex>there</xex> is now used in both senses; <as>as, I shall go <ex>there</ex> to-morrow; we shall go <ex>there</ex> together</as>.</usage><br/
[<source>1913 Webster</source>]</p>

<p><ent>Thither</ent><br/
<hw>Thith"er</hw> <pr>(?)</pr>, <pos>a.</pos> <sn>1.</sn> <def>Being on the farther side from the person speaking; farther; -- a correlative of <xex>hither</xex>; <as>as, on the <ex>thither</ex> side of the water</as>.</def>  <rj><au>W. D. Howells.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Applied to time: On the thither side of, older than; of more years than. See <er>Hither</er>, <pos>a.</pos></def>  <rj><au>Huxley.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Thitherto</ent><br/
<hw>Thith"er*to`</hw> <pr>(?)</pr>, <pos>adv.</pos> <def>To that point; so far.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Thitherward</ent><br/
<hw>Thith"er*ward</hw> <pr>(?)</pr>, <pos>adv.</pos> <def>To ward that place; in that direction.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>They shall ask the way to Zion, with their faces <qex>thitherward</qex>.</q> <rj><qau>Jer. l. 5.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Thitsee</ent><br/
<hw>Thit"see</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Written also <ets>theesee</ets>, and <ets>thietsie</ets>.]</ety> <sn>1.</sn> <fld>(Bot.)</fld> <def>The varnish tree of Burmah (<spn>Melanorrhoea usitatissima</spn>).</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A black varnish obtained from the tree.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Thlipsis</ent><br/
||<hw>Thlip"sis</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[NL., fr. Gr. <?/ pressure, fr. <?/ to press.]</ety> <fld>(Med.)</fld> <def>Compression, especially constriction of vessels by an external cause.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tho</ent><br/
<hw>Tho</hw> <pr>(<th/<omac/)</pr>, <pos>def. art.</pos> <def>The.</def> <mark>[Obs.]</mark>  <rj><au>Spenser.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tho</ent><br/
<hw>Tho</hw>, <pos>pron. pl.</pos> <def>Those.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>This knowen <qex>tho</qex> that be to wives bound.</q> <rj><qau>Chaucer.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tho</ent><br/
<hw>Tho</hw>, <pos>adv.</pos> <ety>[AS. <ets><thorn/<amac/</ets>.]</ety> <def>Then.</def> <mark>[Obs.]</mark>  <rj><au>Spenser.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>To do obsequies as was <qex>tho</qex> the guise.</q> <rj><qau>Chaucer.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tho</ent><br/
<hw>Tho</hw>, <pos>conj.</pos> <def>Though.</def> <mark>[Reformed spelling.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Thole</ent><br/
<hw>Thole</hw> <pr>(?)</pr>, <pos>n.</pos> <altsp>[Written also <asp>thowel</asp>, and <asp>thowl</asp>.]</altsp> <ety>[OE. <ets>thol</ets>, AS. <ets><thorn/ol</ets>; akin to D. <ets>dol</ets>, Icel. <ets><thorn/ollr</ets> a fir tree, a young fir, a tree, a thole.]</ety> <sn>1.</sn> <def>A wooden or metal pin, set in the gunwale of a boat, to serve as a fulcrum for the oar in rowing.</def>  <rj><au>Longfellow.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>The pin, or handle, of a scythe snath.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Thole pin</b></col>. <cd>Same as <er>Thole</er>.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Thole</ent><br/
<hw>Thole</hw>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Tholed</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Tholing</conjf>.]</vmorph> <ety>[OE. <ets><thorn/olen</ets>, <ets><thorn/olien</ets>, AS. <ets><thorn/olian</ets>; akin to OS. <ets>thol<omac/n</ets>, OHG. <ets>dol<emac/n</ets>, G. ge<ets>duld</ets> patience, <ets>dulden</ets> to endure, Icel. <ets><thorn/ola</ets>, Sw. <ets>t<aring/la</ets>, Dan. <ets>taale</ets>, Goth. <ets><thorn/ulan</ets>, L. <ets>tolerate</ets>, <ets>tulisse</ets>, to endure, bear, <ets>tollere</ets> to lift, bear, Gr. <?/ to bear, Skr. <ets>tul</ets> to lift. <root/55.  Cf. <er>Tolerate</er>.]</ety> <def>To bear; to endure; to undergo.</def> <mark>[Obs. or Scot.]</mark>  <rj><au>Gower.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>So much woe as I have with you <qex>tholed</qex>.</q> <rj><qau>Chaucer.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>To <qex>thole</qex> the winter's steely dribble.</q> <rj><qau>Burns.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Thole</ent><br/
<hw>Thole</hw>, <pos>v. i.</pos> <def>To wait.</def> <mark>[Prov. Eng. & Scot.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Thomean</ent><br/
<ent>Thomaean</ent><br/
<mhw>{ <hw>Tho*mae"an</hw>, <hw>Tho*me"an</hw>}</mhw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Eccl. Hist.)</fld> <def>A member of the ancient church of Christians established on the Malabar coast of India, which some suppose to have been originally founded by the Apostle Thomas.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Thomas slag</ent><br/
<ent>Thomas phosphate</ent><br/
<mhw>{ <hw>Thom"as phos"phate</hw> <it>or</it>  <hw>Thom"as slag</hw>  }</mhw>. <def>Same as <er>Basic slag</er>, above.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Thomas process</ent><br/
<hw>Thom"as proc"ess</hw> <pr>(?)</pr>. <fld>(Iron Metal.)</fld> <def>Same as <er>Basic process</er>, above.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Thomaism</ent><br/
<ent>Thomism</ent><br/
<mhw>{ <hw>Tho"mism</hw> <pr>(?)</pr>, <hw>Tho"ma*ism</hw> <pr>(?)</pr>, }</mhw> <pos>n.</pos> <fld>(Eccl. Hist.)</fld> <def>The doctrine of <person>Thomas Aquinas</person>, esp. with respect to predestination and grace.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Thomist</ent><br/
<hw>Tho"mist</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Eccl. Hist.)</fld> <def>A follower of <person>Thomas Aquinas</person>. See <er>Scotist</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Thomite</ent><br/
<hw>Tho"mite</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Eccl. Hist.)</fld> <def>A Thomaean.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Thomsenolite</ent><br/
<hw>Thom"sen*o*lite</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Named after <person>Dr. J. <etsep>Thomsen</etsep></person> of <city>Copenhagen</city>. See <er>-lite</er>.]</ety> <fld>(Min.)</fld> <def>A fluoride of aluminium, calcium, and sodium occurring with the cryolite of Greenland.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Thomsen's disease</ent><br/
<hw>Thom"sen's dis*ease"</hw> <pr>(?)</pr>. <ety>[From <etsep>Thomsen</etsep>, a physician of Sleswick.]</ety> <fld>(Med.)</fld> <def>An affection apparently congenital, consisting in tonic contraction and stiffness of the voluntary muscles occurring after a period of muscular inaction.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Thomsonian</ent><br/
<hw>Thom*so"ni*an</hw> <pr>(?)</pr>, <pos>a.</pos> <fld>(Med.)</fld> <def>Of or pertaining to Thomsonianism.</def> -- <def2><pos>n.</pos> <def>A believer in Thomsonianism; one who practices Thomsonianism.</def></def2><br/
[<source>1913 Webster</source>]</p>

<p><ent>Thomsonianism</ent><br/
<hw>Thom*so"ni*an*ism</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Med.)</fld> <def>An empirical system which assumes that the human body is composed of four elements, earth, air, fire, and water, and that vegetable medicines alone should be used; -- from the founder, Dr. Samuel <etsep>Thomson</etsep>, of Massachusetts.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Thomsonite</ent><br/
<hw>Thom"son*ite</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[From R. D. <etsep>Thomson</etsep>, of Glasgow.]</ety> <fld>(Min.)</fld> <def>A zeolitic mineral, occurring generally in masses of a radiated structure. It is a hydrous silicate of aluminia, lime, and soda. Called also <altname>mesole</altname>, and <altname>comptonite</altname>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Thomson process</ent><br/
<hw>Thomson process</hw>. <ety>[After Elihu <etsep>Thomson</etsep>, American inventor.]</ety> <def>A process of electric welding in which heat is developed by a large current passing through the metal.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Thong</ent><br/
<hw>Thong</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[OE. <ets>thong</ets>, <ets><thorn/wong</ets>, <ets>thwang</ets>, AS. <ets><thorn/wang</ets>; akin to Icel. <ets><thorn/vengr</ets> a thong, latchet. <root/57.  Cf. <er>Twinge</er>.]</ety> <def>A strap of leather; especially, one used for fastening anything.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>And nails for loosened spears, and <qex>thongs</qex> for shields, provide.</q> <rj><qau>Dryden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Thong seal</b></col> <fld>(Zool.)</fld>, <cd>the bearded seal. See the Note under <er>Seal</er>.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Thooid</ent><br/
<hw>Tho"oid</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[Gr. <?/, <?/, the jacal + <ets>-oid</ets>.]</ety> <fld>(Zool.)</fld> <def>Of or pertaining to a group of carnivores, including the wovels and the dogs.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Thor</ent><br/
<hw>Thor</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Icel. <ets><thorn/<omac/rs</ets>.  Cf. <er>Thursday</er>.]</ety> <fld>(Scand. Myth.)</fld> <def>The god of thunder, and son of Odin.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Thoracentesis</ent><br/
||<hw>Tho`ra*cen*te"sis</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[NL., fr. Gr. <?/ thorax + <?/ pricking, from <?/ to prick, stab.]</ety> <fld>(Surg.)</fld> <def>The operation of puncturing the chest wall so as to let out liquids contained in the cavity of the chest.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Thoracic</ent><br/
<hw>Tho*rac"ic</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[Cf. F. <ets>thoracique</ets>.]</ety> <fld>(Anat.)</fld> <def>Of or pertaining to the thorax, or chest.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Thoracic duct</b></col> <fld>(Anat.)</fld>, <cd>the great trunk of the lymphatic vessels, situated on the ventral side of the vertebral column in the thorax and abdomen. See <xex>Illust.</xex> of <er>Lacteal</er>.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Thoracic</ent><br/
<hw>Tho*rac"ic</hw>, <pos>n.</pos> <ety>[Cf. F. <ets>thoracique</ets>.]</ety> <fld>(Zool.)</fld> <def>One of a group of fishes having the ventral fins placed beneath the thorax or beneath the pectorial fins.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Thoracica</ent><br/
||<hw>Tho*rac"i*ca</hw> <pr>(?)</pr>, <pos>n. pl.</pos> <ety>[NL.]</ety> <fld>(Zool.)</fld> <def>A division of cirripeds including those which have six thoracic segments, usually bearing six pairs of cirri. The common barnacles are examples.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Thoracometer</ent><br/
<hw>Tho`ra*com"e*ter</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Physiol.)</fld> <def>Same as <er>Stethometer</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Thoracoplasty</ent><br/
<hw>Tho`ra*co*plas"ty</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[<ets>Thorax</ets> + <ets>plasty</ets>.]</ety> <fld>(Med.)</fld> <def>A remodeling or reshaping of the thorax; especially, the operation of removing the ribs, so as to obliterate the pleural cavity in cases of empyema.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Thoracostraca</ent><br/
||<hw>Tho`ra*cos"tra*ca</hw> <pr>(?)</pr>, <pos>n. pl.</pos> <ety>[NL. See <er>Thorax</er>, and <er>Ostracoid</er>, <pos>a.</pos>]</ety> <fld>(Zool.)</fld> <def>An extensive division of Crustacea, having a dorsal shield or carapec<?/ <?/<?/niting all, or nearly all, of the thoracic somites to the head. It includes the crabs, lobsters, shrimps, and similar species.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Thoracotomy</ent><br/
<hw>Tho`ra*cot"o*my</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Gr. <?/, <?/, thorax + <?/ to cut.]</ety> <fld>(Surg.)</fld> <def>The operation of opening the pleural cavity by incision.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Thoral</ent><br/
<hw>Tho"ral</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>torus</ets> a couch, bed.]</ety> <def>Of or pertaining to a bed.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Thorax</ent><br/
<hw>Tho"rax</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L., fr. Gr. <?/.]</ety> <sn>1.</sn> <fld>(Anat.)</fld> <def>The part of the trunk between the neck and the abdomen, containing that part of the body cavity the walls of which are supported by the dorsal vertebrae, the ribs, and the sternum, and which the heart and lungs are situated; the chest.</def><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ In mammals the thoracic cavity is completely separated from the abdominal by the diaphragm, but in birds and many reptiles the separation is incomplete, while in other reptiles, and in amphibians and fishes, there is no marked separation and no true thorax.</note><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Zool.)</fld> <sd>(a)</sd> <def>The middle region of the body of an insect, or that region which bears the legs and wings. It is composed of three united somites, each of which is composed of several distinct parts. See <xex>Illust.</xex> in Appendix. and <xex>Illust.</xex> of <er>Coleoptera</er>.</def> <sd>(b)</sd> <def>The second, or middle, region of the body of a crustacean, arachnid, or other articulate animal. In the case of decapod Crustacea, some writers include under the term <xex>thorax</xex> only the three segments bearing the maxillipeds; others include also the five segments bearing the legs. See <xex>Illust.</xex> in Appendix.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <fld>(Antiq.)</fld> <def>A breastplate, cuirass, or corselet; especially, the breastplate worn by the ancient Greeks.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Thoria</ent><br/
<hw>Tho"ri*a</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[NL. See <er>Thorite</er>.]</ety> <fld>(Chem.)</fld> <def>A rare white earthy substance, consisting of the oxide of thorium; -- formerly called also <altname>thorina</altname>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Thoric</ent><br/
<hw>Thor"ic</hw> <pr>(?)</pr>, <pos>a.</pos> <fld>(Chem.)</fld> <def>Of or pertaining to thorium; designating the compounds of thorium.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Thorite</ent><br/
<hw>Tho"rite</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[So called by Berzelius from the Scandinavian god <ets>Thor</ets>. See <er>Thor</er>.]</ety> <fld>(Min.)</fld> <def>A mineral of a brown to black color, or, as in the variety <xex>orangite</xex>, orange-yellow. It is essentially a silicate of thorium.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Thorium</ent><br/
<hw>Tho"ri*um</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[NL. See <er>Thorite</er>.]</ety> <fld>(Chem.)</fld> <def>A metallic element found in certain rare minerals, as thorite, pyrochlore, monazite, etc., and isolated as an infusible gray metallic powder which burns in the air and forms thoria; -- formerly called also <altname>thorinum</altname>. Symbol Th. Atomic weight 232.0.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Thorn</ent><br/
<hw>Thorn</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[AS. <ets><thorn/orn</ets>; akin to OS. & OFries. <ets>thorn</ets>, D. <ets>doorn</ets>, G. <ets>dorn</ets>, Dan. <ets>torn</ets>, Sw. <ets>t<oum/rne</ets>, Icel. <ets><thorn/orn</ets>, Goth. <ets><thorn/a<uacute/rnus</ets>; cf. Pol. <ets>tarn</ets>, Russ. <ets>tern'</ets> the blackthorn, <ets>ternie</ets> thorns, Skr. <ets>t<rsdot/<nsdot/a</ets> grass, blade of grass. <root/53.]</ety> <sn>1.</sn> <def>A hard and sharp-pointed projection from a woody stem; usually, a branch so transformed; a spine.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Bot.)</fld> <def>Any shrub or small tree which bears thorns; especially, any species of the genus Crataegus, as the haw<xex>thorn</xex>, white<xex>thorn</xex>, cockspur <xex>thorn</xex>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>Fig.: That which pricks or annoys as a thorn; anything troublesome; trouble; care.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>There was given to me a <qex>thorn</qex> in the flesh, the messenger of Satan to buffet me.</q> <rj><qau>2 Cor. xii. 7.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>The guilt of empire, all its <qex>thorns</qex> and cares,<br/
Be only mine.</q> <rj><qau>Southern.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>The name of the Anglo-Saxon letter <?/, capital form <?/. It was used to represent both of the sounds of English <xex>th</xex>, as in <xex>thin</xex>, <xex>then</xex>. So called because it was the initial letter of <xex>thorn</xex>, a spine.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Thorn apple</b></col> <fld>(Bot.)</fld>, <cd>Jamestown weed.</cd> -- <col><b>Thorn broom</b></col> <fld>(Bot.)</fld>, <cd>a shrub that produces thorns.</cd> -- <col><b>Thorn hedge</b></col>, <cd>a hedge of thorn-bearing trees or bushes.</cd> -- <col><b>Thorn devil</b></col>. <fld>(Zool.)</fld> <cd>See <er>Moloch</er>, 2.</cd> -- <col><b>Thorn hopper</b></col> <fld>(Zool.)</fld>, <cd>a tree hopper (<spn>Thelia crataegi</spn>) which lives on the thorn bush, apple tree, and allied trees.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Thorn</ent><br/
<hw>Thorn</hw>, <pos>v. t.</pos> <def>To prick, as with a thorn.</def> <mark>[Poetic]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>I am the only rose of all the stock<br/
That never <qex>thorn'd</qex> him.</q> <rj><qau>Tennyson.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Thornback</ent><br/
<hw>Thorn"back`</hw> <pr>(?)</pr>, <pos>n.</pos> <sn>1.</sn> <fld>(Zool.)</fld> <def>A European skate (<spn>Raia clavata</spn>) having thornlike spines on its back.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Zool.)</fld> <def>The large European spider crab or king crab (<spn>Maia squinado</spn>).</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Thornbill</ent><br/
<hw>Thorn"bill`</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Zool.)</fld> <def>Any one of several species of small, brilliantly colored American birds of the genus <gen>Rhamphomicron</gen>. They have a long, slender, sharp bill, and feed upon honey, insects, and the juice of the sugar cane.</def><br/
[<source>1913 Webster</source>]</p>

<p><-- p. 1501 --><br/
[<source>1913 Webster</source>]</p>

<p><ent>Thornbird</ent><br/
<hw>Thorn"bird`</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Zool.)</fld> <def>A small South American bird (<spn>Anumbius anumbii</spn>) allied to the ovenbirds of the genus <gen>Furnarius</gen>). It builds a very large and complex nest of twigs and thorns in a bush or tree.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Thornbut</ent><br/
<hw>Thorn"but</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[<ets>Thorn</ets> + <ets>-but</ets> as in <ets>halibut</ets>; cf. G. <ets>dornbutt</ets>.]</ety> <fld>(Zool.)</fld> <def>The turbot.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Thorn-headed</ent><br/
<hw>Thorn"-head`ed</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Having a head armed with thorns or spines.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Thorn-headed worm</b></col> <fld>(Zool.)</fld>, <cd>any worm of the order Acanthocephala; -- called also <altname>thornhead</altname>.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Thornless</ent><br/
<hw>Thorn"less</hw>, <pos>a.</pos> <def>Destitute of, or free from, thorns.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Thornset</ent><br/
<hw>Thorn"set`</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Set with thorns.</def>  <rj><au>Dyer.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Thorntail</ent><br/
<hw>Thorn"tail`</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Zool.)</fld> <def>A beautiful South American humming bird (<spn>Gouldia Popelairii</spn>), having the six outer tail feathers long, slender, and pointed. The head is ornamented with a long, pointed crest.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Thorny</ent><br/
<hw>Thorn"y</hw> <pr>(?)</pr>, <pos>a.</pos> <amorph>[<pos>Compar.</pos> <adjf>Thornier</adjf> <pr>(?)</pr>; <pos>superl.</pos> <adjf>Thorniest</adjf>.]</amorph> <ety>[Cf. AS. <ets><thorn/orniht</ets>.]</ety> <sn>1.</sn> <def>Full of thorns or spines; rough with thorns; spiny; <as>as, a <ex>thorny</ex> wood; a <ex>thorny</ex> tree; a <ex>thorny</ex> crown</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Like a thorn or thorns; hence, figuratively, troublesome; vexatious; harassing; perplexing.</def> <ldquo/The <xex>thorny</xex> point of bare distress.<rdquo/  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>The steep and <qex>thorny</qex> way to heaven.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Thorny rest-harrow</b></col> <fld>(Bot.)</fld>, <cd>rest-harrow.</cd> -- <col><b>Thorny trefoil</b></col>, <cd>a prickly plant of the genus <gen>Fagonia</gen> (<spn>Fagonia Cretica</spn>, etc.).</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Thoro</ent><br/
<hw>Thor"o</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Thorough.</def> <mark>[Reformed spelling.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Thorough</ent><br/
<hw>Thor"ough</hw> <pr>(?)</pr>, <pos>prep.</pos> <ety>[See <er>Through</er>.]</ety> <def>Through.</def> <mark>[Obs.]</mark>  <rj><au>Spenser. Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Thorough</ent><br/
<hw>Thor"ough</hw>, <pos>a.</pos> <sn>1.</sn> <def>Passing through; <as>as, <ex>thorough</ex> lights in a house</as>.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Passing through or to the end; hence, complete; perfect; <as>as, a <ex>thorough</ex> reformation; <ex>thorough</ex> work; a <ex>thorough</ex> translator; a <ex>thorough</ex> poet</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Thorough</ent><br/
<hw>Thor"ough</hw>, <pos>adv.</pos> <sn>1.</sn> <def>Thoroughly.</def> <mark>[Obs. or Colloq.]</mark>  <rj><au>Chaucer.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Through.</def> <mark>[Obs.]</mark>  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Thorough</ent><br/
<hw>Thor"ough</hw>, <pos>n.</pos> <def>A furrow between two ridges, to drain off the surface water.</def> <mark>[Prov. Eng.]</mark>  <rj><au>Halliwell.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Thorough bass</ent><br/
<hw>Thor"ough bass`</hw> <pr>(?)</pr>. <fld>(Mus.)</fld> <def>The representation of chords by figures placed under the base; figured bass; basso continuo; -- sometimes used as synonymous with <altname>harmony</altname>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Thorough-brace</ent><br/
<hw>Thor"ough-brace`</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A leather strap supporting the body of a carriage, and attached to springs, or serving as a spring. See <xex>Illust.</xex> of <er>Chaise</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Thoroughbred</ent><br/
<hw>Thor"ough*bred`</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Bred from the best blood through a long line; pure-blooded; -- said of stock, as horses. Hence, having the characteristics of such breeding; mettlesome; courageous; of elegant form, or the like.</def> -- <def2><pos>n.</pos> <def>A thoroughbred animal, especially a horse.</def></def2><br/
[<source>1913 Webster</source>]</p>

<p><ent>Thoroughfare</ent><br/
<hw>Thor"ough*fare`</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[AS. <ets><thorn/urhfaru</ets>.]</ety> <sn>1.</sn> <def>A passage through; a passage from one street or opening to another; an unobstructed way open to the public; a public road; hence, a frequented street.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>A large and splendid <qex>thoroughfare</qex>.</q> <rj><qau>Motley.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A passing or going through; passage.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>[Made] Hell and this world -- one realm, one continent<br/
Of easy <qex>thoroughfare</qex>.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Thoroughgoing</ent><br/
<hw>Thor"ough*go`ing</hw> <pr>(?)</pr>, <pos>a.</pos> <sn>1.</sn> <def>Going through, or to the end or bottom; very thorough; complete.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Going all lengths; extreme; thoroughplaced; -- less common in this sense.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Thorough-lighted</ent><br/
<hw>Thor"ough-light`ed</hw> <pr>(?)</pr>, <pos>a.</pos> <fld>(Arch.)</fld> <def>Provided with thorough lights or windows at opposite sides, as a room or building.</def>  <rj><au>Gwilt.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Thoroughly</ent><br/
<hw>Thor"ough*ly</hw>, <pos>adv.</pos> <def>In a thorough manner; fully; entirely; completely.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Thoroughness</ent><br/
<hw>Thor"ough*ness</hw>, <pos>n.</pos> <def>The quality or state of being thorough; completeness.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Thoroughpaced</ent><br/
<hw>Thor"ough*paced`</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Perfect in what is undertaken; complete; going all lengths; <as>as, a <ex>thoroughplaced</ex> Tory or Whig</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>If she be a <qex>thoroughplaced</qex> impostor.</q> <rj><qau>Sir W. Scott.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Thoroughpin</ent><br/
<hw>Thor"ough*pin`</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Far.)</fld> <def>A disease of the hock (sometimes of the knee) of a horse, caused by inflammation of the synovial membrane and a consequent excessive secretion of the synovial fluid; -- probably so called because there is usually an oval swelling on each side of the leg, appearing somewhat as if a pin had been thrust through.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Thoroughsped</ent><br/
<hw>Thor"ough*sped`</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Fully accomplished; thoroughplaced.</def> <mark>[R.]</mark>  <rj><au>Swift.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Thoroughstitch</ent><br/
<hw>Thor"ough*stitch`</hw> <pr>(?)</pr>, <pos>adv.</pos> <def>So as to go the whole length of any business; fully; completely.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>Preservance alone can carry us <qex>thoroughstitch</qex>.</q> <rj><qau>L'Estrange.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Thoroughwax</ent><br/
<hw>Thor"ough*wax`</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Bot.)</fld> <sd>(a)</sd> <def>An umbelliferous plant (<spn>Bupleurum rotundifolium</spn>) with perfoliate leaves.</def> <sd>(b)</sd> <def>Thoroughwort.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Thoroughwort</ent><br/
<hw>Thor"ough*wort`</hw> <pr>(?)</pr>, <pos>n.</pos> <def>Same as <er>Boneset</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Thorow</ent><br/
<hw>Thor"ow</hw> <pr>(?)</pr>, <pos>prep.</pos> <def>Through.</def> <mark>[Obs.]</mark> <ldquo/<xex>Thorow</xex> bramble, pits, and floods.<rdquo/  <rj><au>Beau. & Fl.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Thorow</ent><br/
<hw>Thor"ow</hw>, <pos>a.</pos> <def>Thorough.</def> <mark>[Obs.]</mark>  <rj><au>Hakluyt.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Thorpe</ent><br/
<ent>Thorp</ent><br/
<mhw>{ <hw>Thorp</hw>, <hw>Thorpe</hw>}</mhw> <pr>(th<ocir/rp)</pr>, <pos>n.</pos> <ety>[AS. <ets><thorn/orp</ets>; akin to OS. & OFries. <ets>thorp</ets>, D. <ets>dorp</ets>, G. <ets>dorf</ets>, Icel. <ets><thorn/orp</ets>, Dan. <ets>torp</ets>, Sw. <ets>torp</ets> a cottage, a little farm, Goth. <ets><thorn/a<uacute/rp</ets> a field, and probably to Lith. <ets>troba</ets> a building, a house, W. <ets>tref</ets> a hamlet, Ir. <ets>treabh</ets> a farmed village, a tribe, clan, Gael. <ets>treabhair</ets> houses, and perhaps to L. <ets>turba</ets> a crowd, mult.  Cf. <er>Dorp</er>.]</ety> <def>A group of houses in the country; a small village; a hamlet; a dorp; -- now chiefly occurring in names of places and persons; <as>as, Al<ex>thorp</ex>, Mable<ex>thorpe</ex></as>.</def> <ldquo/Within a little <xex>thorp</xex> I staid.<rdquo/  <rj><au>Fairfax.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Then <qex>thorpe</qex> and byre arose in fire.</q> <rj><qau>Tennyson.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Those</ent><br/
<hw>Those</hw> <pr>(?)</pr>, <pos>pron.</pos> <ety>[OE. <ets><thorn/os</ets>, <ets><thorn/as</ets>, AS. <ets><edh/<amac/s</ets>, nom. and acc. pl. of <ets><edh/<emac/s</ets> this. See <er>This</er>, and cf. <er>These</er>.]</ety> <def>The plural of <xex>that</xex>. See <er>That</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Thoth</ent><br/
||<hw>Thoth</hw> <pr>(?)</pr>, <pos>n.</pos> <sn>1.</sn> <fld>(Myth.)</fld> <def>The god of eloquence and letters among the ancient Egyptians, and supposed to be the inventor of writing and philosophy. He corresponded to the <xex>Mercury</xex> of the Romans, and was usually represented as a human figure with the head of an ibis or a lamb.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Zool.)</fld> <def>The Egyptian sacred baboon.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>thou</ent><br/
<hw>thou</hw> <pr>(<th/ou)</pr>, <pos>pron.</pos> <nmorph>[<pos>Sing.</pos>: <pos>nom.</pos> <decf>Thou</decf>; <pos>poss.</pos> <decf>Thy</decf> <pr>(<th/<imac/)</pr> or <decf>Thine</decf> <pr>(<th/<imac/n)</pr>; <pos>obj.</pos> <decf>Thee</decf> <pr>(<th/<emac/)</pr>. <pos>Pl.</pos>: <pos>nom.</pos> <decf>You</decf> <pr>(y<oomac/)</pr>; <pos>poss.</pos> <decf>Your</decf> <pr>(y<oomac/r)</pr> or <decf>Yours</decf> <pr>(y<oomac/rz)</pr>; <pos>obj.</pos> <decf>You</decf>.]</nmorph> <ety>[OE. <ets>thou</ets>, <ets><thorn/u</ets>, AS. <ets><edh/<umac/</ets>, <ets><edh/u</ets>; akin to OS. & OFries. <ets>thu</ets>, G., Dan. & Sw. <ets>du</ets>, Icel. <ets><thorn/<umac/</ets>, Goth. <ets><thorn/u</ets>, Russ. <ets>tui</ets>, Ir. & Gael. <ets>tu</ets>, W. <ets>ti</ets>, L. <ets>tu</ets>, Gr. <grk>sy`</grk>, Dor. <ets><grk>ty`</grk></ets>, Skr. <ets>tvam</ets>.  <root/185.  Cf. <er>Thee</er>, <er>Thine</er>, <er>Te Deum</er>.]</ety> <def>The second personal pronoun, in the singular number, denoting the person addressed; thyself; the pronoun which is used in addressing persons in the solemn or poetical style.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Art <qex>thou</qex> he that should come?</q> <rj><qau>Matt. xi. 3.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ <ldquo/In Old English, generally, <xex>thou</xex> is the language of a lord to a servant, of an equal to an equal, and expresses also companionship, love, permission, defiance, scorn, threatening: whilst <xex>ye</xex> is the language of a servant to a lord, and of compliment, and further expresses honor, submission, or entreaty.<rdquo/</note>  <rj><au>Skeat.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ <xex>Thou</xex> is now sometimes used by the Friends, or Quakers, in familiar discourse, though most of them corruptly say <xex>thee</xex> instead of <xex>thou</xex>.</note><br/
[<source>1913 Webster</source>]</p>

<p><ent>Thou</ent><br/
<hw>Thou</hw>, <pos>v. t.</pos> <def>To address as <xex>thou</xex>, esp. to do so in order to treat with insolent familiarity or contempt.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>If thou <qex>thouest</qex> him some thrice, it shall not be amiss.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Thou</ent><br/
<hw>Thou</hw>, <pos>v. i.</pos> <def>To use the words <xex>thou</xex> and <xex>thee</xex> in discourse after the manner of the Friends.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Though</ent><br/
<hw>Though</hw> <pr>(<th/<omac/)</pr>, <pos>conj.</pos> <ety>[OE. <ets>thogh</ets>, <ets><thorn/ah</ets>, AS. <ets><edh/e<aacute/h</ets>, <ets><edh/<aemac/h</ets>, <ets><edh/<emac/h</ets>; akin to OS. <ets>th<omac/h</ets>, OFries. <ets>thach</ets>, D. & G. <ets>doch</ets> but, yet, OHG. <ets>doh</ets> but, yet though, Icel. <ets><thorn/<omac/</ets> yet, nevertheless, Sw. <ets>dock</ets>, Dan. <ets>dog</ets>, Goth. <ets><thorn/<aacute/uh</ets>, <ets><thorn/<aacute/u</ets>, than, or, yet; of uncertain origin.  <root/184.]</ety> <def>Granting, admitting, or supposing that; notwithstanding that; if.</def><br/
[<source>1913 Webster</source>]</p>

<p><q><qex>Though</qex> he slay me, yet will I trust in him.</q> <rj><qau>Job xiii. 15.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Not that I so affirm, <qex>though</qex> so it seem.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ It is compounded with <xex>all</xex> in <xex>although</xex>. See <er>Although</er>.</note><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>As though</b></col>, <cd>as if.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><q>In the vine were three branches; and it was <qex>as though</qex> it budded.</q> <rj><qau>Gen. xl. 10.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Though</ent><br/
<hw>Though</hw>, <pos>adv.</pos> <def>However; nevertheless; notwithstanding; -- used in familiar language, and in the middle or at the end of a sentence.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>I would not be as sick <qex>though</qex> for his place.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>A good cause would do well, <qex>though</qex>.</q> <rj><qau>Dryden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Thought</ent><br/
<hw>Thought</hw> <pr>(?)</pr>, <def><pos>imp. & p. p.</pos> of <er>Think</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Thought</ent><br/
<hw>Thought</hw>, <pos>n.</pos> <ety>[OE. <ets><thorn/oght</ets>, <ets><thorn/ouht</ets>, AS. <ets><thorn/<omac/ht</ets>, ge<ets><thorn/<omac/ht</ets>, fr. <ets><thorn/encean</ets> to think; akin to D. ge<ets>dachte</ets> thought, MHG. <ets>d<amac/ht</ets>, ge<ets>d<amac/ht</ets>, Icel. <ets><thorn/<omac/ttr</ets>, <ets><thorn/<omac/tti</ets>. See <er>Think</er>.]</ety> <sn>1.</sn> <def>The act of thinking; the exercise of the mind in any of its higher forms; reflection; cogitation.</def><br/
[<source>1913 Webster</source>]</p>

<p><q><qex>Thought</qex> can not be superadded to matter, so as in any sense to render it true that matter can become cogitative.</q> <rj><qau>Dr. T. Dwight.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Meditation; serious consideration.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Pride, of all others the most dangerous fault,<br/
Proceeds from want of sense or want of <qex>thought</qex>.</q> <rj><qau>Roscommon.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>That which is thought; an idea; a mental conception, whether an opinion, judgment, fancy, purpose, or intention.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Thus Bethel spoke, who always speaks his <qex>thought</qex>.</q> <rj><qau>Pope.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Why do you keep alone, . . . <br/
Using those <qex>thoughts</qex> which should indeed have died<br/
With them they think on?</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q><qex>Thoughts</qex> come crowding in so fast upon me, that my only difficulty is to choose or to reject.</q> <rj><qau>Dryden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>All their <qex>thoughts</qex> are against me for evil.</q> <rj><qau>Ps. lvi. 5.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>Solicitude; anxious care; concern.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Hawis was put in trouble, and died with <qex>thought</qex> and anguish before his business came to an end.</q> <rj><qau>Bacon.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Take no <qex>thought</qex> for your life, what ye shall eat, or what ye shall drink.</q> <rj><qau>Matt. vi. 25.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <def>A small degree or quantity; a trifle; <as>as, a <ex>thought</ex> longer; a <ex>thought</ex> better.</as></def> <mark>[Colloq.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>If the hair were a <qex>thought</qex> browner.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ <xex>Thought</xex>, in philosophical usage now somewhat current, denotes the capacity for, or the exercise of, the very highest intellectual functions, especially those usually comprehended under judgment.</note><br/
[<source>1913 Webster</source>]</p>

<p><q>This [faculty], to which I gave the name of the <ldquo/elaborative faculty,<rdquo/ -- the faculty of relations or comparison, -- constitutes what is properly denominated <qex>thought</qex>.</q> <rj><qau>Sir W. Hamilton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><syn><b>Syn.</b> -- Idea; conception; imagination; fancy; conceit; notion; supposition; reflection; consideration; meditation; contemplation; cogitation; deliberation.</syn><br/
[<source>1913 Webster</source>]</p>

<p><ent>Thoughtful</ent><br/
<hw>Thought"ful</hw> <pr>(?)</pr>, <pos>a.</pos> <sn>1.</sn> <def>Full of thought; employed in meditation; contemplative; <as>as, a man of <ex>thoughtful</ex> mind</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>War, horrid war, your <qex>thoughtful</qex> walks invades.</q> <rj><qau>Pope.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Attentive; careful; exercising the judgment; having the mind directed to an object; <as>as, <ex>thoughtful</ex> of gain; <ex>thoughtful</ex> in seeking truth</as>.</def>  <rj><au>Glanvill.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>Anxious; solicitous; concerned.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Around her crowd distrust, and doubt, and fear,<br/
And <qex>thoughtful</qex> foresight, and tormenting care.</q> <rj><qau>Prior.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><syn><b>Syn.</b> -- Considerate; deliberate; contemplative; attentive; careful; wary; circumspect; reflective; discreet.</syn> <usage> -- <er>Thoughtful</er>, <er>Considerate</er>. He who is habitually <xex>thoughtful</xex> rarely neglects his duty or his true interest; he who is <xex>considerate</xex> pauses to reflect and guard himself against error. One who is not <xex>thoughtful</xex> by nature, if he can be made <xex>considerate</xex>, will usually be guarded against serious mistakes. <ldquo/He who is <xex>thoughtful</xex> does not forget his duty; he who is <xex>considerate</xex> pauses, and <xex>considers</xex> properly what is his duty. It is a recommendation to a subordinate person to be <xex>thoughtful</xex> in doing what is wished of him; it is the recommendation of a confidential person to be <xex>considerate</xex>, as he has often to judge according to his own discretion.<rdquo/  <rj><au>Crabb.</au></rj></usage><br/
[<source>1913 Webster</source>]</p>

<p>-- <wordforms><wf>Thought"ful*ly</wf>, <pos>adv.</pos> -- <wf>Thought"ful*ness</wf>, <pos>n.</pos></wordforms><br/
[<source>1913 Webster</source>]</p>

<p><ent>Thoughtless</ent><br/
<hw>Thought"less</hw>, <pos>adv.</pos> <sn>1.</sn> <def>Lacking thought; careless; inconsiderate; rash; <as>as, a <ex>thoughtless</ex> person, or act</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Giddy; gay; dissipated.</def> <mark>[R.]</mark>  <rj><au>Johnson.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>Deficient in reasoning power; stupid; dull.</def><br/
[<source>1913 Webster</source>]</p>

<p><q><qex>Thoughtless</qex> as monarch oaks that shade the plain.</q> <rj><qau>Dryden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p>-- <wordforms><wf>Thought"less*ly</wf>, <pos>adv.</pos> -- <wf>Thought"less*ness</wf>, <pos>n.</pos></wordforms><br/
[<source>1913 Webster</source>]</p>

<p><ent>Thought transference</ent><br/
<hw>Thought transference</hw>. <def>Telepathy.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Thousand</ent><br/
<hw>Thou"sand</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[OE. <ets><thorn/ousend</ets>, <ets><thorn/usend</ets>, AS. <ets><thorn/<umac/send</ets>; akin to OS. <ets>th<umac/sundig</ets>, <ets>th<umac/sind</ets>, OFries. <ets>thusend</ets>, D. <ets>duizend</ets>, G. <ets>tausend</ets>, OHG. <ets>t<umac/sunt</ets>, <ets>d<umac/sunt</ets>, Icel. <ets><thorn/<umac/sund</ets>, <ets><thorn/<umac/shund</ets>, Sw. <ets>tusen</ets>, <ets>Dan</ets>. <ets>tusind</ets>, Goth. <ets><thorn/<umac/sundi</ets>, Lith. <ets>tukstantis</ets>, Russ. <ets>tuisiacha</ets>; of uncertain origin.]</ety> <sn>1.</sn> <def>The number of ten hundred; a collection or sum consisting of ten times one hundred units or objects.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Hence, indefinitely, a great number.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>A <qex>thousand</qex> shall fall at thy side, and ten <qex>thousand</qex> at thy right hand.</q> <rj><qau>Ps. xci. 7.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ The word <xex>thousand</xex> often takes a plural form. See the Note under <er>Hundred</er>.</note><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>A symbol representing one thousand units; <as>as, 1,000, M or CI<Crev/</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Thousand</ent><br/
<hw>Thou"sand</hw>, <pos>a.</pos> <sn>1.</sn> <def>Consisting of ten hundred; being ten times one hundred.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Hence, consisting of a great number indefinitely.</def> <ldquo/Perplexed with a <xex>thousand</xex> cares.<rdquo/  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Thousandfold</ent><br/
<hw>Thou"sand*fold`</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Multiplied by a thousand.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Thousand legs</ent><br/
<hw>Thou"sand legs`</hw> <pr>(?)</pr>. <fld>(Zool.)</fld> <def>A millepid, or galleyworm; -- called also <altname>thousand-legged worm</altname>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Thousandth</ent><br/
<hw>Thou"sandth</hw> <pr>(?)</pr>, <pos>a.</pos> <sn>1.</sn> <def>Next in order after nine hundred and ninty-nine; coming last of a thousand successive individuals or units; -- the ordinal of <xex>thousand</xex>; <as>as, the <ex>thousandth</ex> part of a thing</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Constituting, or being one of, a thousand equal parts into which anything is divided; the tenth of a hundredth.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>Occurring as being one of, or the last one of, a very great number; very small; minute; -- used hyperbolically; <as>as, to do a thing for the <ex>thousandth</ex> time</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Thousandth</ent><br/
<hw>Thou"sandth</hw>, <pos>n.</pos> <def>The quotient of a unit divided by a thousand; one of a thousand equal parts into which a unit is divided.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Thowl</ent><br/
<ent>Thowel</ent><br/
<mhw>{ <hw>Thow"el</hw> <pr>(?)</pr>, <hw>Thowl</hw> <pr>(?)</pr>, }</mhw> <pos>n.</pos> <ety>[See <er>Thole</er>.]</ety> <fld>(Naut.)</fld> <sd>(a)</sd> <def>A thole pin.</def> <sd>(b)</sd> <def>A rowlock.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>I would sit impatiently thinking with what an unusual amount of noise the oars worked in the <qex>thowels</qex>.</q> <rj><qau>Dickens.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Thracian</ent><br/
<hw>Thra"cian</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Of or pertaining to Thrace, or its people.</def> -- <def2><pos>n.</pos>  <def>A native or inhabitant of Thrace.</def></def2><br/
[<source>1913 Webster</source>]</p>

<p><ent>Thrack</ent><br/
<hw>Thrack</hw> <pr>(?)</pr>, <pos>v. t.</pos> <def>To load or burden; <as>as, to <ex>thrack</ex> a man with property</as>.</def> <mark>[Obs.]</mark>  <rj><au>South.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Thrackscat</ent><br/
<hw>Thrack"scat</hw> <pr>(?)</pr>, <pos>n.</pos> <def>Metal still in the mine.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Thraldom</ent><br/
<hw>Thral"dom</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Icel. <ets><?/r<ae/ld<omac/mr</ets>. See <er>Thrall</er>, and <er>-dom</er>.]</ety> <def>The condition of a thrall; slavery; bondage; state of servitude.</def> <altsp>[Written also <asp>thralldom</asp>.]</altsp><br/
[<source>1913 Webster</source>]</p>

<p><q>Women are born to <qex>thraldom</qex> and penance<br/
And to be under man's governance.</q> <rj><qau>Chaucer.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>He shall rule, and she in <qex>thraldom</qex> live.</q> <rj><qau>Dryden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Thrall</ent><br/
<hw>Thrall</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[OE. <ets>thral</ets>, <ets><thorn/ral</ets>, Icel. <ets><thorn/r<ae/ll</ets>, perhaps through AS. <ets><thorn/r<aemac/l</ets>; akin to Sw. <ets>tr<aum/l</ets>, Dan. <ets>tr<ae/l</ets>, and probably to AS. <ets><thorn/r<ae/gian</ets> to run, Goth. <ets><thorn/ragjan</ets>, Gr. <grk>tre`chein</grk>; cf. OHG. <ets>dregil</ets>, <ets>drigil</ets>, a servant.]</ety> <sn>1.</sn> <def>A slave; a bondman.</def>  <rj><au>Chaucer.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Gurth, the born <qex>thrall</qex> of Cedric.</q> <rj><qau>Sir W. Scott.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Slavery; bondage; servitude; thraldom.</def>  <rj><au>Tennyson.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>He still in <qex>thrall</qex><br/
Of all-subdoing sleep.</q> <rj><qau>Chapman.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>A shelf; a stand for barrels, etc.</def> <mark>[Prov. Eng.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Thrall</ent><br/
<hw>Thrall</hw>, <pos>a.</pos> <def>Of or pertaining to a thrall; in the condition of a thrall; bond; enslaved.</def> <mark>[Obs.]</mark>  <rj><au>Spenser.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>The fiend that would make you <qex>thrall</qex> and bond.</q> <rj><qau>Chaucer.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Thrall</ent><br/
<hw>Thrall</hw>, <pos>v. t.</pos> <def>To enslave.</def> <mark>[Obs. or Poetic]</mark>  <rj><au>Spenser.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Thralldom</ent><br/
<hw>Thrall"dom</hw> <pr>(?)</pr>, <pos>n.</pos> <def>Thraldom.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Thrall-less</ent><br/
<hw>Thrall"-less</hw>, <pos>a.</pos> <sd>(a)</sd> <def>Having no thralls.</def> <sd>(b)</sd> <def>Not enslaved; not subject to bonds.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Thrall-like</ent><br/
<hw>Thrall"-like`</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Resembling a thrall, or his condition, feelings, or the like; slavish.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Servile and <qex>thrall-like</qex> fear.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Thranite</ent><br/
<hw>Thra"nite</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Gr. <?/, from <?/ a bench, form, especially the topmost of the three benches in a trireme.]</ety> <fld>(Gr. Antiq.)</fld> <def>One of the rowers on the topmost of the three benches in a trireme.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Thrapple</ent><br/
<hw>Thrap"ple</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Also <ets>thropple</ets>, corrupted fr. <ets>throttle</ets>.]</ety> <def>Windpipe; throttle.</def> <mark>[Prov. Eng. & Scot.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Thresh</ent><br/
<ent>Thrash</ent><br/
<mhw>{ <hw>Thrash</hw> <pr>(?)</pr>, <hw>Thresh</hw> <pr>(?)</pr> }</mhw>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Thrashed</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Thrashing</conjf>.]</vmorph> <ety>[OE. <ets><thorn/reschen</ets>, <ets><thorn/reshen</ets>, to beat, AS. <ets><thorn/erscan</ets>, <ets><thorn/rescan</ets>; akin to D. <ets>dorschen</ets>, OD. <ets>derschen</ets>, G. <ets>dreschen</ets>, OHG. <ets>dreskan</ets>, Icel. <ets><thorn/reskja</ets>, Sw. <ets>tr<oum/ska</ets>, Dan. <ets>t<ae/rske</ets>, Goth. <ets><thorn/riskan</ets>, Lith. <ets>traszketi</ets> to rattle, Russ. <ets>treskate</ets> to burst, crackle, <ets>tresk'</ets> a crash, OSlav. <ets>troska</ets> a stroke of lighting.  Cf. <er>Thresh</er>.]</ety> <sn>1.</sn> <def>To beat out grain from, as straw or husks; to beat the straw or husk of (grain) with a flail; to beat off, as the kernels of grain; <as>as, to <ex>thrash</ex> wheat, rye, or oats; to <ex>thrash</ex> over the old straw</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The wheat was reaped, <qex>thrashed</qex>, and winnowed by machines.</q> <rj><qau>H. Spencer.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To beat soundly, as with a stick or whip; to drub.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Thresh</ent><br/
<ent>Thrash</ent><br/
<mhw>{ <hw>Thrash</hw>, <hw>Thresh</hw>}</mhw>, <pos>v. t.</pos> <sn>1.</sn> <def>To practice thrashing grain or the like; to perform the business of beating grain from straw; <as>as, a man who <ex>thrashes</ex> well</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Hence, to labor; to toil; also, to move violently.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>I rather would be Maevius, <qex>thrash</qex> for rhymes,<br/
Like his, the scorn and scandal of the times.</q> <rj><qau>Dryden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Thrashel</ent><br/
<hw>Thrash"el</hw> <pr>(?)</pr>, <pos>n.</pos> <def>An instrument to thrash with; a flail.</def> <mark>[Obs. or Prov. Eng.]</mark>  <rj><au>Halliwell.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Thresher</ent><br/
<ent>Thrasher</ent><br/
<mhw>{ <hw>Thrash"er</hw> <pr>(?)</pr>, <hw>Thresh"er</hw> <pr>(?)</pr> }</mhw>, <pos>n.</pos> <sn>1.</sn> <def>One who, or that which, thrashes grain; a thrashing machine.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Zool.)</fld> <def>A large and voracious shark (<spn>Alopias vulpes</spn>), remarkable for the great length of the upper lobe of its tail, with which it beats, or thrashes, its prey. It is found both upon the American and the European coasts. Called also <altname>fox shark</altname>, <altname>sea ape</altname>, <altname>sea fox</altname>, <altname>slasher</altname>, <altname>swingle-tail</altname>, and <altname>thrasher shark</altname>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <fld>(Zool.)</fld> <def>A name given to the brown thrush and other allied species. See <er>Brown thrush</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Sage thrasher</b></col>. <fld>(Zool.)</fld> <cd>See under <er>Sage</er>.</cd> -- <col><b>Thrasher whale</b></col> <fld>(Zool.)</fld>, <cd>the common killer of the Atlantic.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><-- p. 1502 --><br/
[<source>1913 Webster</source>]</p>

<p><ent>Thrashing</ent><br/
<hw>Thrash"ing</hw> <pr>(?)</pr>, <def><pos>a. & n.</pos> from <er>Thrash</er>, <pos>v.</pos></def><br/
[<source>1913 Webster</source>]</p>

<p><cs><mcol><col><b>Thrashing floor</b></col>, <col><b>Threshing-floor</b></col>, <it>or</it> <col><b>Threshing floor</b></col></mcol>, <cd>a floor or area on which grain is beaten out.</cd> -- <col><b>Thrashing machine</b></col>, <cd>a machine for separating grain from the straw.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Thrasonical</ent><br/
<hw>Thra*son"ic*al</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[From <ets>Thrso</ets>, the name of a braggart soldier in Terence's <ldquo/Eunuch:<rdquo/ cf. L. <ets>Thrasonianus</ets>.]</ety> <def>Of or pertaining to Thraso; like, or becoming to, Thraso; bragging; boastful; vainglorious.</def> -- <wordforms><wf>Thra*son"ic*al*ly</wf>, <pos>adv.</pos></wordforms><br/
[<source>1913 Webster</source>]</p>

<p><q>Caesar's <qex>thrasonical</qex> brag of 'I came, saw, and overcame.'</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Thraste</ent><br/
<hw>Thraste</hw> <pr>(thr<aum/st)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp.</pos> <conjf>Thraste</conjf>; <pos>p. p.</pos> <conjf>Thrast</conjf>.]</vmorph> <def>To thrust.</def> <mark>[Obs.]</mark>  <rj><au>Chaucer.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Thrave</ent><br/
<hw>Thrave</hw> <pr>(thr<amac/v)</pr>, <pos>n.</pos> <ety>[OE. <ets><thorn/rave</ets>, <ets><thorn/reve</ets>, Icel. <ets><thorn/refi</ets>; akin to Dan. <ets>trave</ets>; cf. Icel. <ets><thorn/r<imac/fa</ets> to grasp.]</ety> <sn>1.</sn> <def>Twenty-four (in some places, twelve) sheaves of wheat; a shock, or stook.</def> <mark>[Prov. Eng.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>The number of two dozen; also, an indefinite number; a bunch; a company; a throng.</def> <ldquo/The worst of a <xex>thrave</xex>.<rdquo/ <mark>[Obs.]</mark>  <rj><au>Landsdowne MS.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>He sends forth <qex>thraves</qex> of ballads to the sale.</q> <rj><qau>Bp. Hall.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Thraw</ent><br/
<hw>Thraw</hw> <pr>(thr<add/)</pr>, <pos>n. & v.</pos> <def>See <er>Throse</er>.</def> <mark>[Scot.]</mark>  <rj><au>Burns.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Thread</ent><br/
<hw>Thread</hw> <pr>(thr<ecr/d)</pr>, <pos>n.</pos> <ety>[OE. <ets>threed</ets>, <ets><thorn/red</ets>, AS. <ets><thorn/r<aemac/d</ets>; akin to D. <ets>draad</ets>, G. <ets>draht</ets> wire, thread, OHG. <ets>dr<amac/t</ets>, Icel. <ets><thorn/r<amac/<edh/r</ets> a thread, Sw. <ets>tr<aring/d</ets>, Dan. <ets>traad</ets>, and AS. <ets><thorn/r<amac/wan</ets> to twist. See <er>Throw</er>, and cf. <er>Third</er>.]</ety> <sn>1.</sn> <def>A very small twist of flax, wool, cotton, silk, or other fibrous substance, drawn out to considerable length; a compound cord consisting of two or more single yarns doubled, or joined together, and twisted; also, one fiber of a cord composed of multiple fibers.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A filament of any substance, as of glass, gold or silver; a filamentous part of an object, such as a flower; a component fiber of any or of any fibrous substance, as of bark.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>The prominent part of the spiral of a screw or nut; the rib. See <er>Screw</er>, <pos>n.</pos>, 1.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <mark>(Fig.)</mark> <def>Something continued in a long course or tenor; a recurrent theme or related sequence of events in a larger story; <as>as the <ex>thread</ex> of a story, or of life, or of a discourse</as>.</def>  <rj><au>Bp. Burnet.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <def>Fig.: Composition; quality; fineness.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>A neat courtier,<br/
Of a most elegant <qex>thread</qex>.</q> <rj><qau>B. Jonson.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>6.</sn> <fld>(Computers)</fld> <def>A related sequence of instructions or actions within a program that runs at least in part independent of other actions within the program; -- such <ex>threads</ex> are capable of being executed only in oprating systems permittnig multitasking.</def><br/
[<source>PJC</source>]</p>

<p><sn>7.</sn> <fld>(Computers)</fld> <def>A sequence of messages posted to an on-line newsgroup or discussion group, dealing with the same topic; -- messages in such a <ex>thread</ex> typically refer to a previous posting, thus allowing their identification as part of the <ex>thread</ex>.  Some news-reading programs allow a user to follow a single such <ex>thread</ex> independent of the other postings to that newsgroup.</def><br/
[<source>PJC</source>]</p>

<p><cs><col><b>Air thread</b></col>, <cd>the fine white filaments which are seen floating in the air in summer, the production of spiders; gossamer.</cd> -- <col><b>Thread and thrum</b></col>, <cd>the good and bad together.</cd> <mark>[Obs.]</mark> <au>Shak.</au> -- <col><b>Thread cell</b></col> <fld>(Zool.)</fld>, <cd>a lasso cell. See under <er>Lasso</er>.</cd> -- <col><b>Thread herring</b></col> <fld>(Zool.)</fld>, <cd>the gizzard shad. See under <er>Gizzard</er>.</cd> -- <col><b>Thread lace</b></col>, <cd>lace made of linen thread.</cd> -- <col><b>Thread needle</b></col>, <cd>a game in which children stand in a row, joining hands, and in which the outer one, still holding his neighbor, runs between the others; -- called also <altname>thread the needle</altname>.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Thread</ent><br/
<hw>Thread</hw>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Threaded</conjf>; <pos>p. pr. & vb. n.</pos> <conjf>Threading</conjf>.]</vmorph> <sn>1.</sn> <def>To pass a thread through the eye of; <as>as, to <ex>thread</ex> a needle</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To pass or pierce through as a narrow way; also, to effect or make, as one's way, through or between obstacles; to thrid.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Heavy trading ships . . . <qex>threading</qex> the Bosphorus.</q> <rj><qau>Mitford.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>They would not <qex>thread</qex> the gates.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>To form a thread, or spiral rib, on or in; <as>as, to <ex>thread</ex> a screw or nut</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Threadbare</ent><br/
<hw>Thread"bare`</hw> <pr>(?)</pr>, <pos>a.</pos> <sn>1.</sn> <def>Worn to the naked thread; having the nap worn off; <xex>threadbare</xex> clothes.</def> <ldquo/A <xex>threadbare</xex> cope.<rdquo/  <rj><au>Chaucer.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Fig.: Worn out; <as>as, a <ex>threadbare</ex> subject; stale topics and <ex>threadbare</ex> quotations.</as></def>  <rj><au>Swift.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Threadbareness</ent><br/
<hw>Thread"bare`ness</hw>, <pos>n.</pos> <def>The state of being threadbare.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Threaden</ent><br/
<hw>Thread"en</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Made of thread; <as>as, <ex>threaden</ex> sails; a <ex>threaden</ex> fillet.</as></def> <mark>[Obs.]</mark>  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Threader</ent><br/
<hw>Thread"er</hw> <pr>(?)</pr>, <pos>n.</pos> <sn>1.</sn> <def>A device for assisting in threading a needle.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A tool or machine for forming a thread on a screw or in a nut.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Threadfin</ent><br/
<hw>Thread"fin`</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Zool.)</fld> <def>Any one of several species of fishes belonging to <gen>Polynemus</gen> and allied genera. They have numerous long pectoral filaments.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Threadfish</ent><br/
<hw>Thread"fish`</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Zool.)</fld> <sd>(a)</sd> <def>The cutlass fish.</def> <sd>(b)</sd> <def>A carangoid fish (<spn>Caranx gallus</spn>, or <spn>Caranx crinitus</spn>) having the anterior rays of the soft dorsal and anal fins prolonged in the form of long threads.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Threadiness</ent><br/
<hw>Thread"i*ness</hw> <pr>(?)</pr>, <pos>n.</pos> <def>Quality of being thready.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>threads</ent><br/
<hw>threads</hw> <pr>(thr<ecr/dz)</pr>, <pos>n. pl.</pos> <def>Clothes; clothing; <as>as, he was wearing his new <ex>threads</ex> at the party</as>.</def> <mark>[Slang]</mark><br/
[<source>PJC</source>]</p>

<p><ent>Thread-shaped</ent><br/
<hw>Thread"-shaped`</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Having the form of a thread; filiform.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Threadworm</ent><br/
<hw>Thread"worm`</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Zool.)</fld> <def>Any long, slender nematode worm, especially the pinworm and filaria.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Thready</ent><br/
<hw>Thread"y</hw> <pr>(?)</pr>, <pos>a.</pos> <sn>1.</sn> <def>Like thread or filaments; slender; <as>as, the <ex>thready</ex> roots of a shrub</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Containing, or consisting of, thread.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Threap</ent><br/
<hw>Threap</hw> <pr>(thr<emac/p)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Threaped</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Threaping</conjf>.]</vmorph> <ety>[AS. <ets><thorn/re<aacute/pian</ets> to reprove.]</ety> <altsp>[Written also <asp>threpe</asp>, and <asp>threip</asp>.]</altsp> <sn>1.</sn> <def>To call; to name.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To maintain obstinately against denial or contradiction; also, to contend or argue against (another) with obstinacy; to chide; <as>as, he <ex>threaped</ex> me down that it was so</as>.</def> <mark>[Prov. Eng. & Scot.]</mark>  <rj><au>Burns.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>To beat, or thrash.</def> <mark>[Prov. Eng.]</mark>  <rj><au>Halliwell.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>To cozen, or cheat.</def> <mark>[Prov. Eng.]</mark>  <rj><au>Halliwell.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Threap</ent><br/
<hw>Threap</hw>, <pos>v. i.</pos> <def>To contend obstinately; to be pertinacious.</def> <mark>[Prov. Eng. & Scot.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>It's not for a man with a woman to <qex>threap</qex>.</q> <rj><qau>Percy's Reliques.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Threap</ent><br/
<hw>Threap</hw> <pr>(?)</pr>, <pos>n.</pos> <def>An obstinate decision or determination; a pertinacious affirmation.</def> <mark>[Prov. Eng. & Scot.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>He was taken a <qex>threap</qex> that he would have it finished before the year was done.</q> <rj><qau>Carlyle.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Threat</ent><br/
<hw>Threat</hw> <pr>(thr<ecr/t)</pr>, <pos>n.</pos> <ety>[AS. <ets><thorn/re<aacute/t</ets>, akin to <amac/<ets><thorn/re<oacute/tan</ets> to vex, G. ver<ets>driessen</ets>, OHG. ir<ets>driozan</ets>, Icel. <ets><thorn/rj<omac/ta</ets> to fail, want, lack, Goth. us<ets><thorn/riutan</ets> to vex, to trouble, Russ. <ets>trudite</ets> to impose a task, irritate, vex, L. <ets>trudere</ets> to push.  Cf. <er>Abstruse</er>, <er>Intrude</er>, <er>Obstrude</er>, <er>Protrude</er>.]</ety> <def>The expression of an intention to inflict evil or injury on another; the declaration of an evil, loss, or pain to come; menace; threatening; denunciation.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>There is no terror, Cassius, in your <qex>threats</qex>.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Threat</ent><br/
<hw>Threat</hw>, <pos>v. t. & i.</pos> <ety>[OE. <ets><thorn/reten</ets>, AS. <ets><thorn/re<aacute/tian</ets>. See <er>Threat</er>, <pos>n.</pos>]</ety> <def>To threaten.</def> <mark>[Obs. or Poetic]</mark>  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Of all his <qex>threating</qex> reck not a mite.</q> <rj><qau>Chaucer.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Our dreaded admiral from far they <qex>threat</qex>.</q> <rj><qau>Dryden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Threaten</ent><br/
<hw>Threat"en</hw> <pr>(?)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Threatened</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Threatening</conjf>.]</vmorph> <ety>[OE. <ets><thorn/retenen</ets>. See <er>Threat</er>, <pos>v. t.</pos>]</ety> <sn>1.</sn> <def>To utter threats against; to menace; to inspire with apprehension; to alarm, or attempt to alarm, as with the promise of something evil or disagreeable; to warn.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Let us straitly <qex>threaten</qex> them, that they speak henceforth to no man in this name.</q> <rj><qau>Acts iv. 17.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To exhibit the appearance of (something evil or unpleasant) as approaching; to indicate as impending; to announce the conditional infliction of; <as>as, to <ex>threaten</ex> war; to <ex>threaten</ex> death</as>.</def>  <rj><au>Milton.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>The skies look grimly<br/
And <qex>threaten</qex> present blusters.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><syn><b>Syn.</b> -- To menace.</syn> <usage> -- <er>Threaten</er>, <er>Menace</er>. <xex>Threaten</xex> is Anglo-Saxon, and <xex>menace</xex> is Latin. As often happens, the former is the more familiar term; the latter is more employed in formal style. We are <xex>threatened</xex> with a drought; the country is <xex>menaced</xex> with war.</usage><br/
[<source>1913 Webster</source>]</p>

<p><q>By turns put on the suppliant and the lord:<br/
<qex>Threatened</qex> this moment, and the next implored.</q> <rj><qau>Prior.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Of the sharp ax<br/
Regardless, that o'er his devoted head<br/
Hangs <qex>menacing</qex>.</q> <rj><qau>Somerville.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Threaten</ent><br/
<hw>Threat"en</hw>, <pos>v. i.</pos> <def>To use threats, or menaces; also, to have a threatening appearance.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Though the seas <qex>threaten</qex>, they are merciful.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Threatener</ent><br/
<hw>Threat"en*er</hw> <pr>(?)</pr>, <pos>n.</pos> <def>One who threatens.</def>  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Threatening</ent><br/
<hw>Threat"en*ing</hw>, <def><pos>a. & n.</pos> from <er>Threaten</er>, <pos>v.</pos></def> -- <wordforms><wf>Threat"en*ing*ly</wf>, <pos>adv.</pos></wordforms><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Threatening letters</b></col> <fld>(Law)</fld>, <cd>letters containing threats, especially those designed to extort money, or to obtain other property, by menaces; blackmailing letters.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Threatful</ent><br/
<hw>Threat"ful</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Full of threats; having a menacing appearance.</def> <au>Spenser.</au> -- <wordforms><wf>Threat"ful*ly</wf>, <pos>adv.</pos></wordforms><br/
[<source>1913 Webster</source>]</p>

<p><ent>Threave</ent><br/
<hw>Threave</hw> <pr>(?)</pr>, <pos>n.</pos> <def>Same as <er>Thrave</er>.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Three</ent><br/
<hw>Three</hw> <pr>(thr<emac/)</pr>, <pos>a.</pos> <ety>[OE. <ets><thorn/re</ets>, <ets><thorn/reo</ets>, <ets><thorn/ri</ets>, AS. <ets><thorn/r<imac/</ets>, masc., <ets><thorn/re<oacute/</ets>, fem. and neut.; akin to OFries. <ets>thre</ets>, OS. <ets>thria</ets>, <ets>threa</ets>, D. <ets>drie</ets>, G. <ets>drei</ets>, OHG. <ets>dr<imac/</ets>, Icel. <ets><thorn/r<imac/r</ets>, Dan. & Sw. <ets>tre</ets>, Goth. <ets><thorn/reis</ets>, Lith. <ets>trys</ets>, Ir., Gael. & W. <ets>tri</ets>, Russ. <ets>tri</ets>, L. <ets>tres</ets>, Gr. <grk>trei^s</grk>, Skr. <ets>tri</ets>. <root/301.  Cf. 3d <er>Drilling</er>, <er>Tern</er>, <pos>a.</pos>, <er>Third</er>, <er>Thirteen</er>, <er>Thirty</er>, <er>Tierce</er>, <er>Trey</er>, <er>Tri-</er>, <er>Triad</er>, <er>Trinity</er>, <er>Tripod</er>.]</ety> <def>One more than two; two and one.</def> <ldquo/I offer thee <xex>three</xex> things.<rdquo/  <rj><au>2 Sam. xxiv. 12.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q><qex>Three</qex> solemn aisles approach the shrine.</q> <rj><qau>Keble.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ <ex>Three</ex> is often joined with other words, forming compounds signifying <xex>divided into</xex>, <xex>composed of</xex>, or <xex>containing</xex>, <xex>three parts</xex>, <xex>portions</xex>, <xex>organs</xex>, or the like; as, <ex>three</ex>-branched, <ex>three</ex>-capsuled, <ex>three</ex>-celled, <ex>three</ex>-cleft, <ex>three</ex>-edged, <ex>three</ex>-foot, <ex>three</ex>-footed, <ex>three</ex>-forked, <ex>three</ex>-grained, <ex>three</ex>-headed, <ex>three</ex>-legged, <ex>three</ex>-mouthed, <ex>three</ex>-nooked, <ex>three</ex>-petaled, <ex>three</ex>-pronged, <ex>three</ex>-ribbed, <ex>three</ex>-seeded, <ex>three</ex>-stringed, <ex>three</ex>-toed, and the like.</note><br/
[<source>1913 Webster</source>]</p>

<p><ent>Three</ent><br/
<hw>Three</hw>, <pos>n.</pos> <sn>1.</sn> <def>The number greater by a unit than two; three units or objects.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A symbol representing three units, as 3 or iii.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Rule of three</b></col>. <fld>(Arith.)</fld> <cd>See under <er>Rule</er>, <pos>n.</pos></cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Three-coat</ent><br/
<hw>Three"-coat`</hw> <pr>(?)</pr>, <pos>a.</pos> <fld>(Arch.)</fld> <def>Having or consisting of three coats; -- applied to plastering which consists of pricking-up, floating, and a finishing coat; or, as called in the United States, a scratch coat, browning, and finishing coat.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Three-color</ent><br/
<hw>Three"-col`or</hw>, <pos>a.</pos> <def>Designating, or pert. to, a photomechanical process employing printings in three colors, as red, yellow, and blue.</def><br/
[<source>Webster 1913 Suppl.</source>]<br/
[<source>1913 Webster</source>]</p>

<p><ent>Three-cornered</ent><br/
<hw>Three"-cor`nered</hw> <pr>(?)</pr>, <pos>a.</pos> <sn>1.</sn> <def>Having three corners, or angles; <as>as, a <ex>three-cornered</ex> hat</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Bot.)</fld> <def>Having three prominent longitudinal angles; <as>as, a <ex>three-cornered</ex> stem</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Three-decker</ent><br/
<hw>Three"-deck`er</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Naut.)</fld> <def>A vessel of war carrying guns on three decks.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Three-flowered</ent><br/
<hw>Three"-flow`ered</hw> <pr>(?)</pr>, <pos>a.</pos> <fld>(Bot.)</fld> <def>Bearing three flowers together, or only three flowers.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Threefold</ent><br/
<hw>Three"fold`</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[OE. <ets><thorn/reofald</ets>; cf. AS. <ets><thorn/r<imac/feald</ets>.]</ety> <def>Consisting of three, or thrice repeated; triple; <as>as, <ex>threefold</ex> justice</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>A <qex>threefold</qex> cord is not quickly broken.</q> <rj><qau>Eccl. iv. 12.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Three-handed</ent><br/
<hw>Three"-hand`ed</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Said of games or contests where three persons play against each other, or two against one; <as>as, a <ex>three-handed</ex> game of cards</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Three-leaved</ent><br/
<ent>Three-leafed</ent><br/
<mhw>{ <hw>Three"-leafed`</hw> <pr>(?)</pr>, <hw>Three"-leaved`</hw> <pr>(?)</pr>, }</mhw> <pos>a.</pos> <fld>(Bot.)</fld> <sd>(a)</sd> <def>Producing three leaves; <as>as, <ex>three-leaved</ex> nightshade</as>.</def> <sd>(b)</sd> <def>Consisting of three distinct leaflets; having the leaflets arranged in threes.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Three-leaved nightshade</b></col>. <cd>See <er>Trillium</er>.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Three-lobed</ent><br/
<hw>Three"-lobed`</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Having three lobes.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Three-lobed leaf</b></col> <fld>(Bot.)</fld>, <cd>a leaf divided into three parts, the sinuses extending not more than half way to the middle, and either the parts of the sinuses being rounded.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Three-mile</ent><br/
<hw>Three"-mile`</hw>, <pos>a.</pos> <def>Of or pertaining to three miles; <as>as, <col><b>the three-mile limit</b></col>, or the limit of the marine belt (the <mcol><col><b>three-mile belt</b></col> <it>or</it> <col><b>three-mile zone</b></col></mcol>) of three miles included in territorial waters (which see) of a state</as>.</def><br/
[<source>Webster 1913 Suppl.</source>]<br/
[<source>1913 Webster</source>]</p>

<p><ent>Three-nerved</ent><br/
<hw>Three"-nerved`</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Having three nerves.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Three-nerved leaf</b></col> <fld>(Bot.)</fld>, <cd>a leaf having three distinct and prominent ribs, or nerves, extending from the base.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Three-parted</ent><br/
<hw>Three"-part`ed</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Divided into, or consisting of, three parts; tripartite.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Three-parted leaf</b></col> <fld>(Bot.)</fld>, <cd>a leaf divided into three parts down to the base, but not entirely separate.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Threepence</ent><br/
<hw>Three"pence</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A small silver coin of three times the value of a penny.</def> <mark>[Eng.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Threepenny</ent><br/
<hw>Three"pen*ny</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Costing or worth three pence; hence, worth but little; poor; mean.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Three-pile</ent><br/
<hw>Three"-pile`</hw> <pr>(?)</pr>, <pos>n.</pos> <def>An old name for the finest and most costly kind of velvet, having a fine, thick pile.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>I have served Prince Florizel and in my time wore <qex>three-pile</qex>.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Three-piled</ent><br/
<hw>Three"-piled`</hw> <pr>(?)</pr>, <pos>a.</pos> <sn>1.</sn> <def>Having the quality of three-pile; best; most costly.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>Thou art good velvet; thou 'rt <qex>three-piled</qex> piece.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Fig.: Extravagant; exaggerated; high-flown.</def> <ldquo/<xex>Three-piled</xex> hyperboles.<rdquo/  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>Accustomed to wearing three-pile; hence, of high rank, or wealth.</def> <mark>[Obs.]</mark> <ldquo/<xex>Three-piled</xex> people.<rdquo/  <rj><au>Beau. & Fl.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Three-ply</ent><br/
<hw>Three"-ply`</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Consisting of three distinct webs inwrought together in weaving, as cloth or carpeting; having three strands; threefold.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Three-pointed</ent><br/
<hw>Three"-point`ed</hw> <pr>(?)</pr>, <pos>a.</pos> <fld>(Bot.)</fld> <def>Having three acute or setigerous points; tricuspidate.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Three-port</ent><br/
<hw>Three"-port`</hw>, <pos>a.</pos> <def>Having three ports; specif.: Designating a type of two-cycle internal-combustion engine in which the mixture enters the crank case through a port uncovered by the piston near the end of its stroke.</def><br/
[<source>Webster 1913 Suppl.</source>]<br/
[<source>1913 Webster</source>]</p>

<p><ent>Three-quarter</ent><br/
<hw>Three"-quar`ter</hw> <pr>(?)</pr>, <pos>a.</pos> <fld>(Paint.)</fld> <def>Measuring thirty inches by twenty-five; -- said of portraitures.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Three-quarter length</b></col>, <cd>a portrait showing the figure to the hips only.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Three-score</ent><br/
<hw>Three"-score`</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Thrice twenty; sixty.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Three-sided</ent><br/
<hw>Three"-sid`ed</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Having three sides, especially three plane sides; <as>as, a <ex>three-sided</ex> stem, leaf, petiole, peduncle, scape, or pericarp</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Three-square</ent><br/
<hw>Three"-square`</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Having a cross section in the form of an equilateral triangle; -- said especially of a kind of file.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Three-torque system of control</ent><br/
<hw>Three"-torque` system of control</hw>. <fld>(Aeronautics)</fld> <def>Any system of rudders by which the pilot can exert a turning moment about each of the three rectangular axes of an aeroplane or airship.</def><br/
[<source>Webster 1913 Suppl.</source>]<br/
[<source>1913 Webster</source>]</p>

<p><ent>Three-valved</ent><br/
<hw>Three"-valved`</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Consisting of, or having, three valves; opening with three valves; <as>as, a <ex>three-valved</ex> pericarp</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Three-way</ent><br/
<hw>Three"-way`</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Connected with, or serving to connect, three channels or pipes; <as>as, a <ex>three-way</ex> cock or valve</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Threne</ent><br/
<hw>Threne</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>threnus</ets>, Gr. <?/.  Cf. <er>Drone</er>.]</ety> <def>Lamentation; threnody; a dirge.</def>  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>The <qex>threns</qex> . . . of the prophet Jeremiah.</q> <rj><qau>Jer. Taylor.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Threnetical</ent><br/
<ent>Threnetic</ent><br/
<mhw>{ <hw>Thre*net"ic</hw> <pr>(?)</pr>, <hw>Thre*net"ic*al</hw> <pr>(?)</pr>, }</mhw> <pos>a.</pos> <ety>[Gr. <?/. See <er>Threne</er>.]</ety> <def>Pertaining to a threne; sorrowful; mournful.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Threnode</ent><br/
<hw>Thren"ode</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A threne, or threnody; a dirge; a funeral song.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Threnodist</ent><br/
<hw>Thren"o*dist</hw> <pr>(?)</pr>, <pos>n.</pos> <def>One who composes, delivers, or utters, a threnode, or threnody.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Threnody</ent><br/
<hw>Thren"o*dy</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Gr. <?/; <?/ a dirge + <?/ a song. See <er>Threne</er>, and <er>Ode</er>.]</ety> <def>A song of lamentation; a threnode.</def>  <rj><au>Sir T. Herbert.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Threpe</ent><br/
<hw>Threpe</hw> <pr>(?)</pr>, <pos>v. t.</pos> <ety>[See <er>Threap</er>.]</ety> <def>To call; to term.</def> <mark>[Obs.]</mark> <ldquo/Luna silver we <xex>threpe</xex>.<rdquo/  <rj><au>Chaucer.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Threpsology</ent><br/
<hw>Threp*sol"o*gy</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Gr. <?/ nourishment + <ets>-logy</ets>.]</ety> <fld>(Med.)</fld> <def>The doctrine of nutrition; a treatise on nutrition.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Thresh</ent><br/
<hw>Thresh</hw> <pr>(?)</pr>, <pos>v. t. & i.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Threshed</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Threshing</conjf>.]</vmorph> <def>Same as <er>Thrash</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>He would <qex>thresh</qex>, and thereto dike and delve.</q> <rj><qau>Chaucer.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Thresher</ent><br/
<hw>Thresh"er</hw> <pr>(?)</pr>, <pos>n.</pos> <def>Same as <er>Thrasher</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Thresh-fold</ent><br/
<hw>Thresh"-fold`</hw> <pr>(?)</pr>, <pos>n.</pos> <def>Threshold.</def> <mark>[Obs.]</mark>  <rj><au>Chaucer.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Threshold</ent><br/
<hw>Thresh"old</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[OE. <ets>threswold</ets>, <ets><thorn/reshwold</ets>, AS. <ets><thorn/rescwald</ets>, <ets><thorn/erscwald</ets>, <ets><thorn/erscold</ets>, <ets><thorn/rescold</ets>, fr. <ets><thorn/rescan</ets>, <ets><thorn/erscan</ets>, to thresh; akin to Icel. <ets><thorn/reskj<oum/de</ets>, <ets><thorn/r<oum/skuldr</ets>, Sw. <ets>tr<oum/skel</ets>, Dan. <ets>t<ae/rskel</ets>. See <er>Thrash</er>.]</ety> <sn>1.</sn> <def>The plank, stone, or piece of timber, which lies under a door, especially of a dwelling house, church, temple, or the like; the doorsill; hence, entrance; gate; door.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Fig.: The place or point of entering or beginning, entrance; outset; <as>as, the <ex>threshold</ex> of life</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Threshwold</ent><br/
<hw>Thresh"wold`</hw> <pr>(?)</pr>, <pos>n.</pos> <def>Threshold.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Threste</ent><br/
<hw>Threste</hw> <pr>(thr<ecr/st)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp.</pos> <conjf>Threste</conjf>; <pos>p. p.</pos> <conjf>Threst</conjf>.]</vmorph> <def>To thrust.</def> <mark>[Obs.]</mark>  <rj><au>Chaucer.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Thretteen</ent><br/
<hw>Thret"teen`</hw> <pr>(thr<ecr/t"t<emac/n`)</pr>, <pos>a.</pos> <def>Thirteen.</def> <mark>[Obs. or Scot.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Thretty</ent><br/
<hw>Thret"ty</hw> <pr>(thr<ecr/t"t<ycr/)</pr>, <pos>a.</pos> <def>Thirty.</def> <mark>[Obs. or Scot.]</mark>  <rj><au>Burns.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Threw</ent><br/
<hw>Threw</hw> <pr>(thr<udd/)</pr>, <def><pos>imp.</pos> of <er>Throw</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Thribble</ent><br/
<hw>Thrib"ble</hw> <pr>(thr<icr/b"b'l)</pr>, <pos>a.</pos> <def>Triple; treble; threefold.</def> <mark>[Prov. Eng. or Colloq.]</mark>  <rj><au>Halliwell.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Thrice</ent><br/
<hw>Thrice</hw> <pr>(thr<imac/s)</pr>, <pos>adv.</pos> <ety>[OE. <ets>thries</ets>; <ets>thrie</ets> thrice (AS. <ets><thorn/riga</ets>, <ets><thorn/riwa</ets>) + <ets>-s</ets>, the adverbal suffix. See <er>Three</er>, and <er>-wards</er>.]</ety> <sn>1.</sn> <def>Three times.</def> <ldquo/<xex>Thrice</xex> in vain.<rdquo/  <rj><au>Spenser.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Verily I say unto thee.  That this night, before the cock crow, thou shalt deny me <qex>thrice</qex>.</q> <rj><qau>Matt. xxvi. 34.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>In a threefold manner or degree; repeatedly; very.</def><br/
[<source>1913 Webster</source>]</p>

<p><q><qex>Thrice</qex> noble lord, let me entreat of you<br/
To pardon me.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q><qex>Thrice</qex> is he armed that hath his quarrel just.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ <ex>Thrice</ex> is often used, generally with an intensive force, to form compounds which are usually of obvious meaning; as, in <ex>thrice</ex>-blessed, <ex>thrice</ex>-favored, <ex>thrice</ex>-hallowed, <ex>thrice</ex>-happy, <ex>thrice</ex>-told, and the like.</note><br/
[<source>1913 Webster</source>]</p>

<p><ent>Thricecock</ent><br/
<hw>Thrice"cock`</hw> <pr>(thr<imac/s"k<ocr/k`)</pr>, <pos>n.</pos> <fld>(Zool.)</fld> <def>The missel thrush.</def> <mark>[Prov. Eng.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Thrid</ent><br/
<hw>Thrid</hw> <pr>(thr<icr/d)</pr>, <pos>a.</pos> <def>Third.</def> <mark>[Obs.]</mark>  <rj><au>Chaucer.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Thrid</ent><br/
<hw>Thrid</hw>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Thridded</conjf>; <pos>p. pr. & vb. n.</pos> <conjf>Thridding</conjf>.]</vmorph> <ety>[A variant of <ets>thread</ets>.]</ety> <sn>1.</sn> <def>To pass through in the manner of a thread or a needle; to make or find a course through; to thread.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Some <qex>thrid</qex> the mazy ringlets of her hair.</q> <rj><qau>Pope.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>And now he <qex>thrids</qex> the bramble bush.</q> <rj><qau>J. R. Drake.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>I began<br/
To <qex>thrid</qex> the musky-circled mazes.</q> <rj><qau>Tennyson.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To make or effect (a way or course) through something; <as>as, to <ex>thrid</ex> one's way through a wood</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Thrid</ent><br/
<hw>Thrid</hw>, <pos>n.</pos> <def>Thread; continuous line.</def> <mark>[Archaic]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>I resume the <qex>thrid</qex> of my discourse.</q> <rj><qau>Dryden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Thrifallow</ent><br/
<hw>Thri"fal`low</hw> <pr>(thr<imac/"f<acr/l`l<osl/)</pr>, <pos>v. t.</pos> <def>See <er>Thryfallow</er>, and <er>Trifallow</er>.</def> <mark>[R.]</mark>  <rj><au>Tusser.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Thrift</ent><br/
<hw>Thrift</hw> <pr>(thr<icr/ft)</pr>, <pos>n.</pos> <ety>[Icel. <ets><thorn/rift</ets>. See <er>Thrive</er>.]</ety> <sn>1.</sn> <def>A thriving state; good husbandry; economical management in regard to property; frugality.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The rest, . . . willing to fall to <qex>thrift</qex>, prove very good husbands.</q> <rj><qau>Spenser.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Success and advance in the acquisition of property; increase of worldly goods; gain; prosperity.</def>  <ldquo/Your <xex>thrift</xex> is gone full clean.<rdquo/  <rj><au>Chaucer.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>I have a mind presages me such <qex>thrift</qex>.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><-- p. 1503 --><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>Vigorous growth, as of a plant.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <fld>(Bot.)</fld> <def>One of several species of flowering plants of the genera <gen>Statice</gen> and <gen>Armeria</gen>.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Common thrift</b></col> <fld>(Bot.)</fld>, <cd><spn>Armeria vulgaris</spn>; -- also called <altname>sea pink</altname>.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><syn><b>Syn.</b> -- Frugality; economy; prosperity; gain; profit.</syn><br/
[<source>1913 Webster</source>]</p>

<p><ent>Thriftily</ent><br/
<hw>Thrift"i*ly</hw> <pr>(thr<icr/ft"<icr/*l<ycr/)</pr>, <pos>adv.</pos> <sn>1.</sn> <def>In a thrifty manner.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Carefully; properly; becomingly.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>A young clerk . . . in Latin <qex>thriftily</qex> them gret [greeted].</q> <rj><qau>Chaucer.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Thriftiness</ent><br/
<hw>Thrift"i*ness</hw>, <pos>n.</pos> <def>The quality or state of being thrifty; thrift.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Thriftless</ent><br/
<hw>Thrift"less</hw>, <pos>a.</pos> <def>Without thrift; not prudent or prosperous in money affairs.</def> -- <wordforms><wf>Thrift"less*ly</wf>, <pos>adv.</pos> -- <wf>Thrift"less*ness</wf>, <pos>n.</pos></wordforms><br/
[<source>1913 Webster</source>]</p>

<p><ent>Thrifty</ent><br/
<hw>Thrift"y</hw> <pr>(thr<icr/ft"<ycr/)</pr>, <pos>a.</pos> <amorph>[<pos>Compar.</pos> <adjf>Thriftier</adjf> <pr>(?)</pr>; <pos>superl.</pos> <adjf>Thriftiest</adjf>.]</amorph> <sn>1.</sn> <def>Given to, or evincing, thrift; characterized by economy and good menegement of property; sparing; frugal.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Her chaffer was so <qex>thrifty</qex> and so new.</q> <rj><qau>Chaucer.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>I am glad he hath so much youth and vigor left, of which he hath not been <qex>thrifty</qex>.</q> <rj><qau>Swift.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Thriving by industry and frugality; prosperous in the acquisition of worldly goods; increasing in wealth; <as>as, a <ex>thrifty</ex> farmer or mechanic</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>Growing rapidly or vigorously; thriving; <as>as, a <ex>thrifty</ex> plant or colt</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>Secured by thrift; well husbanded.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>I have five hundred crowns,<br/
The <qex>thrifty</qex> hire I saved under your father.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <def>Well appearing; looking or being in good condition; becoming.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>I sit at home, I have no <qex>thrifty</qex> cloth.</q> <rj><qau>Chaucer.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><syn><b>Syn.</b> -- Frugal; sparing; economical; saving; careful.</syn><br/
[<source>1913 Webster</source>]</p>

<p><ent>Thrill</ent><br/
<hw>Thrill</hw> <pr>(thr<icr/l)</pr>, <pos>n.</pos> <ety>[See <er>Trill</er>.]</ety> <def>A warbling; a trill.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Thrill</ent><br/
<hw>Thrill</hw>, <pos>n.</pos> <ety>[AS. <ets><thorn/yrel</ets> an aperture. See <er>Thrill</er>, <pos>v. t.</pos>]</ety> <def>A breathing place or hole; a nostril, as of a bird.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Thrill</ent><br/
<hw>Thrill</hw>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Thrilled</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Thrilling</conjf>.]</vmorph> <ety>[OE. <ets>thrillen</ets>, <ets><thorn/irlen</ets>, <ets><thorn/urlen</ets>, to pierce; all probably fr. AS. <ets><thorn/yrlian</ets>, <ets><thorn/yrelian</ets>, Fr. <ets><thorn/yrel</ets> pierced; as a noun, a hole, fr. <ets><thorn/urh</ets> through; probably akin to D. <ets>drillen</ets> to drill, to bore. <root/53. See <er>Through</er>, and cf. <er>Drill</er> to bore, <er>Nostril</er>, <er>Trill</er> to trickle.]</ety> <sn>1.</sn> <def>To perforate by a pointed instrument; to bore; to transfix; to drill.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>He pierced through his chafed chest<br/
With <qex>thrilling</qex> point of deadly iron brand.</q> <rj><qau>Spenser.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Hence, to affect, as if by something that pierces or pricks; to cause to have a shivering, throbbing, tingling, or exquisite sensation; to pierce; to penetrate.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>To bathe in flery floods, or to reside<br/
In <qex>thrilling</qex> region of thick-ribbed ice.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Vivid and picturesque turns of expression which <qex>thrill</qex> the <?/eader with sudden delight.</q> <rj><qau>M. Arnold.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>The cruel word her tender heart so <qex>thrilled</qex>,<br/
That sudden cold did run through every vein.</q> <rj><qau>Spenser.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>To hurl; to throw; to cast.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>I'll <qex>thrill</qex> my javelin.</q> <rj><qau>Heywood.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Thrill</ent><br/
<hw>Thrill</hw>, <pos>v. i.</pos> <sn>1.</sn> <def>To pierce, as something sharp; to penetrate; especially, to cause a tingling sensation that runs through the system with a slight shivering; <as>as, a sharp sound <ex>thrills</ex> through the whole frame</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>I have a faint cold fear <qex>thrills</qex> through my veins.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To feel a sharp, shivering, tingling, or exquisite sensation, running through the body.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>To seek sweet safety out<br/
In vaults and prisons, and to <qex>thrill</qex> and shake.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Thrill</ent><br/
<hw>Thrill</hw>, <pos>n.</pos> <sn>1.</sn> <def>A drill. See 3d <er>Drill</er>, 1.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A sensation as of being thrilled; a tremulous excitement; <as>as, a <ex>thrill</ex> of horror; a <ex>thrill</ex> of joy</as>.</def>  <rj><au>Burns.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Thrillant</ent><br/
<hw>Thrill"ant</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Piercing; sharp; thrilling.</def> <mark>[Obs.]</mark> <ldquo/His <xex>thrillant</xex> spear.<rdquo/  <rj><au>Spenser.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Thrilling</ent><br/
<hw>Thrill"ing</hw>, <pos>a.</pos> <def>Causing a thrill; causing tremulous excitement; deeply moving; <as>as, a <ex>thrilling</ex> romance</as>.</def> -- <wordforms><wf>Thrill"ing*ly</wf>, <pos>adv.</pos> -- <wf>Thrill"ing*ness</wf>, <pos>n.</pos></wordforms><br/
[<source>1913 Webster</source>]</p>

<p><ent>Thring</ent><br/
<hw>Thring</hw> <pr>(?)</pr>, <pos>v. t. & i.</pos> <vmorph>[<pos>imp.</pos> <conjf>Throng</conjf> <pr>(?)</pr>.]</vmorph> <ety>[AS. <ets><thorn/ringan</ets>. See <er>Throng</er>.]</ety> <def>To press, crowd, or throng.</def> <mark>[Obs.]</mark>  <rj><au>Chaucer.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Thrips</ent><br/
||<hw>Thrips</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L., a woodworm, Gr. <?/.]</ety> <fld>(Zool.)</fld> <def>Any one of numerous small species of Thysanoptera, especially those which attack useful plants, as the grain thrips (<spn>Thrips cerealium</spn>).</def><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ The term is also popularly applied to various other small injurious insects.</note><br/
[<source>1913 Webster</source>]</p>

<p><ent>Thrist</ent><br/
<hw>Thrist</hw> <pr>(?)</pr>, <pos>n.</pos> <def>Thrist.</def> <mark>[Obs.]</mark>  <rj><au>Spenser.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Thrittene</ent><br/
<hw>Thrit"tene`</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Thirteen.</def> <mark>[Obs.]</mark>  <rj><au>Chaucer.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Thrive</ent><br/
<hw>Thrive</hw> <pr>(thr<imac/v)</pr>, <pos>v. i.</pos> <vmorph>[<pos>imp.</pos> <conjf>Throve</conjf> <pr>(thr<omac/v)</pr> or <conjf>Thrived</conjf> <pr>(thr<imac/vd)</pr>; <pos>p. p.</pos> <conjf>Thrived</conjf> or <conjf>Thriven</conjf> <pr>(thr<icr/v"'n)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Thriving</conjf>.]</vmorph> <ety>[OE. <ets><thorn/riven</ets>, Icel. <ets><thorn/r<imac/fask</ets>; probably originally, to grasp for one's self, from <ets><thorn/r<imac/fa</ets> to grasp; akin to Dan. <ets>trives</ets> to thrive, Sw. <ets>trifvas</ets>.  Cf. <er>Thrift</er>.]</ety> <sn>1.</sn> <def>To prosper by industry, economy, and good management of property; to increase in goods and estate; <as>as, a farmer <ex>thrives</ex> by good husbandry</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Diligence and humility is the way to <qex>thrive</qex> in the riches of the understanding, as well as in gold.</q> <rj><qau>I. Watts.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To prosper in any business; to have increase or success.</def> <ldquo/They by vices <xex>thrive</xex>.<rdquo/  <rj><au>Sandys.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>O son, why sit we here, each other viewing<br/
Idly, while Satan, our great author, <qex>thrives</qex>?</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>And so she <qex>throve</qex> and prospered.</q> <rj><qau>Tennyson.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>To increase in bulk or stature; to grow vigorously or luxuriantly, as a plant; to flourish; <as>as, young cattle <ex>thrive</ex> in rich pastures; trees <ex>thrive</ex> in a good soil</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Thriven</ent><br/
<hw>Thriv"en</hw> <pr>(?)</pr>, <def><pos>p. p.</pos> of <er>Thrive</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Thriver</ent><br/
<hw>Thriv"er</hw> <pr>(?)</pr>, <pos>n.</pos> <def>One who thrives, or prospers.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Thrivingly</ent><br/
<hw>Thriv"ing*ly</hw>, <pos>adv.</pos> <def>In a thriving manner.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Thrivingness</ent><br/
<hw>Thriv"ing*ness</hw>, <pos>n.</pos> <def>The quality or condition of one who thrives; prosperity; growth; increase.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Thro'</ent><br/
<hw>Thro'</hw> <pr>(?)</pr>. <def>A contraction of <er>Through</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Throat</ent><br/
<hw>Throat</hw> <pr>(thr<omac/t)</pr>, <pos>n.</pos> <ety>[OE. <ets>throte</ets>, AS. <ets><thorn/rote</ets>, <ets><thorn/rotu</ets>; akin to OHG. <ets>drozza</ets>, G. <ets>drossel</ets>; cf. OFries. & D. <ets>stort</ets>.  Cf. <er>Throttle</er>.]</ety> <sn>1.</sn> <fld>(Anat.)</fld> <sd>(a)</sd> <def>The part of the neck in front of, or ventral to, the vertebral column.</def> <sd>(b)</sd> <def>Hence, the passage through it to the stomach and lungs; the pharynx; -- sometimes restricted to the fauces.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>I can vent clamor from my <qex>throat</qex>.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A contracted portion of a vessel, or of a passage way; <as>as, the <ex>throat</ex> of a pitcher or vase</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <fld>(Arch.)</fld> <def>The part of a chimney between the gathering, or portion of the funnel which contracts in ascending, and the flue.</def>  <rj><au>Gwilt.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <fld>(Naut.)</fld> <sd>(a)</sd> <def>The upper fore corner of a boom-and-gaff sail, or of a staysail.</def> <sd>(b)</sd> <def>That end of a gaff which is next the mast.</def> <sd>(c)</sd> <def>The angle where the arm of an anchor is joined to the shank.</def>  <rj><au>Totten.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <fld>(Shipbuilding)</fld> <def>The inside of a timber knee.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>6.</sn> <fld>(Bot.)</fld> <def>The orifice of a tubular organ; the outer end of the tube of a monopetalous corolla; the faux, or fauces.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Throat brails</b></col> <fld>(Naut.)</fld>, <cd>brails attached to the gaff close to the mast.</cd> -- <col><b>Throat halyards</b></col> <fld>(Naut.)</fld>, <cd>halyards that raise the throat of the gaff.</cd> -- <col><b>Throat pipe</b></col> <fld>(Anat.)</fld>, <cd>the windpipe, or trachea.</cd> -- <col><b>To give one the lie in his throat</b></col>, <cd>to accuse one pointedly of lying abominably.</cd> -- <col><b>To lie in one's throat</b></col>, <cd>to lie flatly or abominably.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Throat</ent><br/
<hw>Throat</hw>, <pos>v. t.</pos> <sn>1.</sn> <def>To utter in the throat; to mutter; <as>as, to <ex>throat</ex> threats</as>.</def> <mark>[Obs.]</mark>  <rj><au>Chapman.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To mow, as beans, in a direction against their bending.</def> <mark>[Prov. Eng.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Throatband</ent><br/
<hw>Throat"band`</hw> <pr>(?)</pr>, <pos>n.</pos> <def>Same as <er>Throatlatch</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Throatboll</ent><br/
<hw>Throat"boll`</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[<ets>Throat</ets> + <ets>boll</ets> a ball.]</ety> <def>The Adam's apple in the neck.</def> <mark>[Obs. or R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>By the <qex>throatboll</qex> he caught Aleyn.</q> <rj><qau>Chaucer.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Throating</ent><br/
<hw>Throat"ing</hw>, <pos>n.</pos> <fld>(Arch.)</fld> <def>A drip, or drip molding.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Throatlatch</ent><br/
<hw>Throat"latch`</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A strap of a bridle, halter, or the like, passing under a horse's throat.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Throatwort</ent><br/
<hw>Throat"wort`</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Bot.)</fld> <def>A plant (<spn>Campanula Trachelium</spn>) formerly considered a remedy for sore throats because of its throat-shaped corolla.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Throaty</ent><br/
<hw>Throat"y</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Guttural; hoarse; having a guttural voice.</def> <ldquo/Hard, <xex>throaty</xex> words.<rdquo/  <rj><au>Howell.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Throb</ent><br/
<hw>Throb</hw> <pr>(?)</pr>, <pos>v. i.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Throbbed</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Throbbing</conjf>.]</vmorph> <ety>[OE. <ets><thorn/robben</ets>; of uncertain origin; cf. Russ. <ets>trepete</ets> a trembling, and E. <ets>trepidation</ets>.]</ety> <def>To beat, or pulsate, with more than usual force or rapidity; to beat in consequence of agitation; to palpitate; -- said of the heart, pulse, etc.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>My heart<br/
<qex>Throbs</qex> to know one thing.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Here may his head lie on my <qex>throbbing</qex> breast.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Throb</ent><br/
<hw>Throb</hw>, <pos>n.</pos> <def>A beat, or strong pulsation, as of the heart and arteries; a violent beating; a papitation:</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The impatient <qex>throbs</qex> and longings of a soul<br/
That pants and reaches after distant good.</q> <rj><qau>Addison.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Throdden</ent><br/
<hw>Throd"den</hw> <pr>(?)</pr>, <pos>v. i.</pos> <ety>[Prov. E. <ets>throdden</ets>, <ets>throddle</ets>, fat, thriving; cf. Icel. <ets>throask</ets> to grow.]</ety> <def>To grow; to thrive.</def> <mark>[Prov. Eng.]</mark>  <rj><au>Grose.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Throe</ent><br/
<hw>Throe</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[OE. <ets><thorn/rowe</ets>, <ets><thorn/rawe</ets>, AS. <ets><thorn/re<aacute/</ets> a threatening, oppression, suffering, perhaps influenced by Icel. <ets><thorn/r<amac/</ets> a throe, a pang, a longing; cf. AS. <ets><thorn/reowian</ets> to suffer.]</ety> <sn>1.</sn> <def>Extreme pain; violent pang; anguish; agony; especially, one of the pangs of travail in childbirth, or purturition.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Prodogious motion felt, and rueful <qex>throes</qex>.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A tool for splitting wood into shingles; a frow.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Throe</ent><br/
<hw>Throe</hw>, <pos>v. i.</pos> <def>To struggle in extreme pain; to be in agony; to agonize.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Throe</ent><br/
<hw>Throe</hw>, <pos>v. t.</pos> <def>To put in agony.</def> <mark>[R.]</mark>  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Thrombin</ent><br/
<hw>Throm"bin</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[See <er>Thrombus</er>.]</ety> <fld>(Biochem.)</fld> <def>The enzyme which converts fibrinogen into fibrin.</def><br/
[<source>Webster 1913 Suppl.</source> <source>+PJC</source>]<br/
[<source>1913 Webster</source>]</p>

<p><ent>Thrombosis</ent><br/
||<hw>Throm*bo"sis</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[NL. See <er>Thrombus</er>.]</ety> <fld>(Med.)</fld> <def>The obstruction of a blood vessel by a clot formed at the site of obstruction; -- distinguished from <contr>embolism</contr>, which is produced by a clot or foreign body brought from a distance.</def> -- <wordforms><wf>Throm*bot"ic</wf> <pr>(#)</pr>, <pos>a.</pos></wordforms><br/
[<source>1913 Webster</source>]</p>

<p><ent>Thrombus</ent><br/
||<hw>Throm"bus</hw> <pr>(?)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Thrombi</plw> <pr>(#)</pr>.</plu> <ety>[NL., fr. Gr. <?/ a lump, a clot of blood.]</ety> <fld>(Med.)</fld> <sd>(a)</sd> <def>A clot of blood formed of a passage of a vessel and remaining at the site of coagulation.</def> <sd>(b)</sd> <def>A tumor produced by the escape of blood into the subcutaneous cellular tissue.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Throne</ent><br/
<hw>Throne</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[OE. <ets>trone</ets>, F. <ets>tr<ocir/ne</ets>, L. <ets>thronus</ets>, Gr. <?/; cf. <?/ a bench, <?/ a footstool, <?/ to set one's self, to sit, Skr. <ets>dhara<nsdot/a</ets> supporting, <ets>dh<rsdot/</ets> to hold fast, carry, and E. <ets>firm</ets>, a.]</ety> <sn>1.</sn> <def>A chair of state, commonly a royal seat, but sometimes the seat of a prince, bishop, or other high dignitary.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The noble king is set up in his <qex>throne</qex>.</q> <rj><qau>Chaucer.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>High on a <qex>throne</qex> of royal state.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Hence, sovereign power and dignity; also, the one who occupies a throne, or is invested with sovereign authority; an exalted or dignified personage.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Only in the <qex>throne</qex> will I be greater than thou.</q> <rj><qau>Gen. xli. 40.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>To mold a mighty state's decrees,<br/
And shape the whisper of the <qex>throne</qex>.</q> <rj><qau>Tennyson.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <pluf>pl.</pluf> <def>A high order of angels in the celestial hierarchy; -- a meaning given by the schoolmen.</def>  <rj><au>Milton.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Great Sire! whom <qex>thrones</qex> celestial ceaseless sing.</q> <rj><qau>Young.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Throne</ent><br/
<hw>Throne</hw>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Throned</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Throning</conjf>.]</vmorph> <sn>1.</sn> <def>To place on a royal seat; to enthrone.</def>  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To place in an elevated position; to give sovereignty or dominion to; to exalt.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>True image of the Father, whether <qex>throned</qex><br/
In the bosom of bliss, and light of light.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Throne</ent><br/
<hw>Throne</hw> <pr>(?)</pr>, <pos>v. i.</pos> <def>To be in, or sit upon, a throne; to be placed as if upon a throne.</def>  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Throneless</ent><br/
<hw>Throne"less</hw>, <pos>a.</pos> <def>Having no throne.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Throng</ent><br/
<hw>Throng</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[OE. <ets><thorn/rong</ets>, <ets><thorn/rang</ets>, AS. <ets>ge<thorn/rang</ets>, fr. <ets><thorn/ringan</ets> to crowd, to press; akin to OS. <ets>thringan</ets>, D. & G. <ets>dringen</ets>, OHG. <ets>dringan</ets>, Icel. <ets><thorn/ryngva</ets>, <ets><thorn/r<oum/ngva</ets>, Goth. <ets><thorn/riehan</ets>, D. & G. <ets>drang</ets> a throng, press, Icel. <ets><thorn/r<oum/ng</ets> a throng, Lith. <ets>trenkti</ets> to jolt, <ets>tranksmas</ets> a tumult.  Cf. <er>Thring</er>.]</ety> <sn>1.</sn> <def>A multitude of persons or of living beings pressing or pressed into a close body or assemblage; a crowd.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A great multitude; <as>as, the heavenly <ex>throng</ex></as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><syn><b>Syn.</b> -- <er>Throng</er>, <er>Multitude</er>, <er>Crowd</er>.</syn> <usage> Any great number of persons form a <xex>multitude</xex>; a <xex>throng</xex> is a large number of persons who are gathered or are moving together in a collective body; a <xex>crowd</xex> is composed of a large or small number of persons who press together so as to bring their bodies into immediate or inconvenient contact. A dispersed <xex>multitude</xex>; the <xex>throngs</xex> in the streets of a city; the <xex>crowd</xex> at a fair or a street fight. But these distinctions are not carefully observed.</usage><br/
[<source>1913 Webster</source>]</p>

<p><q>So, with this bold opposer rushes on<br/
This many-headed monster, <qex>multitude</qex>.</q> <rj><qau>Daniel.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Not to know me argues yourselves unknown,<br/
The lowest of your <qex>throng</qex>.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>I come from empty noise, and tasteless pomp,<br/
From <qex>crowds</qex> that hide a monarch from himself.</q> <rj><qau>Johnson.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Throng</ent><br/
<hw>Throng</hw>, <pos>v. i.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Thronged</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Thronging</conjf>.]</vmorph> <def>To crowd together; to press together into a close body, as a multitude of persons; to gather or move in multitudes.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>I have seen the dumb men <qex>throng</qex> to see him.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Throng</ent><br/
<hw>Throng</hw>, <pos>v. t.</pos> <sn>1.</sn> <def>To crowd, or press, as persons; to oppress or annoy with a crowd of living beings.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Much people followed him, and <qex>thronged</qex> him.</q> <rj><qau>Mark v. 24.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To crowd into; to fill closely by crowding or pressing into, as a hall or a street.</def>  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Throng</ent><br/
<hw>Throng</hw>, <pos>a.</pos> <def>Thronged; crowded; also, much occupied; busy.</def> <mark>[Obs. or Prov. Eng.]</mark>  <rj><au>Bp. Sanderson.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>To the intent the sick . . . should not lie too <qex>throng</qex>.</q> <rj><qau>Robynson (More's Utopia).</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Throngly</ent><br/
<hw>Throng"ly</hw>, <pos>adv.</pos> <def>In throngs or crowds.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Throp</ent><br/
<hw>Throp</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A thorp.</def> <mark>[Obs.]</mark>  <rj><au>Chaucer.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Thropple</ent><br/
<hw>Throp"ple</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Cf. <er>Thrapple</er>, and see <er>Throttle</er>.]</ety> <def>The windpipe.</def> <mark>[Prov. Eng.]</mark>  <rj><au>Halliwell.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Thropple</ent><br/
<hw>Throp"ple</hw>, <pos>v. t.</pos> <def>To throttle.</def> <mark>[Prov. Eng.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Throstle</ent><br/
<hw>Thros"tle</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[OE. <ets>throsel</ets>, AS. <ets><thorn/rostle</ets>, <ets><thorn/rosle</ets>; akin to MHG. <ets>trostel</ets>, G. <ets>drossel</ets>, Icel. <ets><thorn/r<oum/str</ets>, Sw. <ets>trast</ets>, Lith. <ets>strazdas</ets>, L. <ets>turdus</ets>. <root/238.   Cf. <er>Thrush</er> the bird.]</ety> <sn>1.</sn> <fld>(Zool.)</fld> <def>The song thrush. See under <er>Song</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A machine for spinning wool, cotton, etc., from the rove, consisting of a set of drawing rollers with bobbins and flyers, and differing from the mule in having the twisting apparatus stationary and the processes continuous; -- so called because it makes a singing noise.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Throstle cock</b></col>, <cd>the missel thrush.</cd> <mark>[Prov. Eng.]</mark></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Throstling</ent><br/
<hw>Thros"tling</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Cf. <er>Throttle</er>.]</ety> <def>A disease of bovine cattle, consisting of a swelling under the throat, which, unless checked, causes strangulation.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Throttle</ent><br/
<hw>Throt"tle</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Dim. of <ets>throat</ets>. See <er>Throat</er>.]</ety> <sn>1.</sn> <def>The windpipe, or trachea; the weasand.</def>  <rj><au>Sir W. Scott.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Steam Engine)</fld> <def>The throttle valve.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Throttle lever</b></col> <fld>(Steam Engine)</fld>, <cd>the hand lever by which a throttle valve is moved, especially in a locomotive.</cd> -- <col><b>Throttle valve</b></col> <fld>(Steam Engine)</fld>, <cd>a valve moved by hand or by a governor for regulating the supply of steam to the steam chest. In one form it consists of a disk turning on a transverse axis.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Throttle</ent><br/
<hw>Throt"tle</hw>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Throttled</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Throttling</conjf> <pr>(?)</pr>.]</vmorph> <sn>1.</sn> <def>To compress the throat of; to choke; to strangle.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Grant him this, and the Parliament hath no more freedom than if it sat in his noose, which, when he pleases to draw together with one twitch of his negative, shall <qex>throttle</qex> a whole nation, to the wish of Caligula, in one neck.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To utter with breaks and interruption, in the manner of a person half suffocated.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q><qex>Throttle</qex> their practiced accent in their fears.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>To shut off, or reduce flow of, as steam to an engine.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Throttle</ent><br/
<hw>Throt"tle</hw>, <pos>v. i.</pos> <sn>1.</sn> <def>To have the throat obstructed so as to be in danger of suffocation; to choke; to suffocate.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To breathe hard, as when nearly suffocated.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Throttler</ent><br/
<hw>Throt"tler</hw> <pr>(?)</pr>, <pos>n.</pos> <sn>1.</sn> <def>One who, or that which, throttles, or chokes.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Zool.)</fld> <def>See <er>Flasher</er>, 3 <sd>(b)</sd>.</def> <mark>[Prov. Eng.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Through</ent><br/
<hw>Through</hw> <pr>(?)</pr>, <pos>prep.</pos> <ety>[OE. <ets>thurgh</ets>, <ets><thorn/urh</ets>, <ets><thorn/uruh</ets>, <ets><thorn/oruh</ets>, AS. <ets><thorn/urh</ets>; akin to OS. <ets>thurh</ets>, <ets>thuru</ets>, OFries. <ets>thruch</ets>, D. <ets>door</ets>, OHG. <ets>durh</ets>, <ets>duruh</ets>, G. <ets>durch</ets>, Goth. <ets><thorn/a<iacute/rh</ets>; cf. Ir. <ets>tri</ets>, <ets>tre</ets>, W. <ets>trwy</ets>. <root/53.   Cf. <er>Nostril</er>, <er>Thorough</er>, <er>Thrill</er>.]</ety> <sn>1.</sn> <def>From end to end of, or from side to side of; from one surface or limit of, to the opposite; into and out of at the opposite, or at another, point; <as>as, to bore <ex>through</ex> a piece of timber, or <ex>through</ex> a board; a ball passes <ex>through</ex> the side of a ship</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Between the sides or walls of; within; <as>as, to pass <ex>through</ex> a door; to go <ex>through</ex> an avenue</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q><qex>Through</qex> the gate of ivory he dismissed<br/
His valiant offspring.</q> <rj><qau>Dryden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>By means of; by the agency of.</def><br/
[<source>1913 Webster</source>]</p>

<p><q><qex>Through</qex> these hands this science has passed with great applause.</q> <rj><qau>Sir W. Temple.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Material things are presented only <qex>through</qex> their senses.</q> <rj><qau>Cheyne.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>Over the whole surface or extent of; <as>as, to ride <ex>through</ex> the country; to look <ex>through</ex> an account</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <def>Among or in the midst of; -- used to denote passage; <as>as, a fish swims <ex>through</ex> the water; the light glimmers <ex>through</ex> a thicket</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>6.</sn> <def>From the beginning to the end of; to the end or conclusion of; <as>as, <ex>through</ex> life; <ex>through</ex> the year</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Through</ent><br/
<hw>Through</hw>, <pos>adv.</pos> <sn>1.</sn> <def>From one end or side to the other; <as>as, to pierce a thing <ex>through</ex></as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>From beginning to end; <as>as, to read a letter <ex>through</ex></as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>To the end; to a conclusion; to the ultimate purpose; <as>as, to carry a project <ex>through</ex></as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ <ex>Through</ex> was formerly used to form compound adjectives where we now use <ex>thorough</ex>; as, <ex>through</ex>-bred; <ex>through</ex>-lighted; <ex>through</ex>-placed, etc.</note><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>To drop through</b></col>, <cd>to fall through; to come to naught; to fail.</cd> -- <col><b>To fall through</b></col>. <cd>See under <er>Fall</er>, <pos>v. i.</pos></cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><-- p. 1504 --><br/
[<source>1913 Webster</source>]</p>

<p><ent>Through</ent><br/
<hw>Through</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Going or extending through; going, extending, or serving from the beginning to the end; thorough; complete; <as>as, a <ex>through</ex> line; a <ex>through</ex> ticket; a <ex>through</ex> train</as>.  Also, admitting of passage through; <as>as, a <ex>through</ex> bridge</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Through bolt</b></col>, <cd>a bolt which passes through all the thickness or layers of that which it fastens, or in which it is fixed.</cd> -- <col><b>Through bridge</b></col>, <cd>a bridge in which the floor is supported by the lower chords of the tissues instead of the upper, so that travel is between the trusses and not over them.   Cf. <cref>Deck bridge</cref>, under <er>Deck</er>.</cd> -- <col><b>Through cold</b></col>, <cd>a deep-seated cold.</cd> <mark>[Obs.]</mark> <au>Holland.</au> -- <col><b>Through stone</b></col>, <cd>a flat gravestone.</cd> <mark>[Scot.]</mark> <altsp>[Written also <asp>through stane</asp>.]</altsp> <au>Sir W. Scott.</au> -- <col><b>Through ticket</b></col>, <cd>a ticket for the whole journey.</cd> -- <col><b>Through train</b></col>, <cd>a train which goes the whole length of a railway, or of a long route.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Throughly</ent><br/
<hw>Through"ly</hw>, <pos>adv.</pos> <def>Thoroughly.</def> <mark>[Obs.]</mark>  <rj><au>Bacon.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Wash me <qex>throughly</qex> from mine iniquity.</q> <rj><qau>Ps. li. 2.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>To dare in fields is valor; but how few<br/
Dare to be <qex>throughly</qex> valiant to be true?</q> <rj><qau>Dryden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Throughout</ent><br/
<hw>Through*out"</hw> <pr>(?)</pr>, <pos>prep.</pos> <def>Quite through; from one extremity to the other of; also, every part of; <as>as, to search <ex>throughout</ex> the house</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Nor to their idle orbs doth sight appear<br/
Of sun, or moon, or star, <qex>throughout</qex> the year.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Throughout</ent><br/
<hw>Through*out"</hw>, <pos>adv.</pos> <def>In every part; <as>as, the cloth was of a piece <ex>throughout</ex></as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Throve</ent><br/
<hw>Throve</hw> <pr>(?)</pr>, <def><pos>imp.</pos> of <er>Thrive</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Throw</ent><br/
<hw>Throw</hw> <pr>(thr<omac/)</pr>, <pos>n.</pos> <ety>[See <er>Throe</er>.]</ety> <def>Pain; especially, pain of travail; throe.</def> <mark>[Obs.]</mark>  <rj><au>Spenser. Dryden.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Throw</ent><br/
<hw>Throw</hw>, <pos>n.</pos> <ety>[AS. <ets><thorn/r<amac/h</ets>, <ets><thorn/r<amac/g</ets>.]</ety> <def>Time; while; space of time; moment; trice.</def> <mark>[Obs.]</mark>  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>I will with Thomas speak a little <qex>throw</qex>.</q> <rj><qau>Chaucer.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Throw</ent><br/
<hw>Throw</hw>, <pos>v. t.</pos> <vmorph>[<pos>imp.</pos> <conjf>Threw</conjf> <pr>(thr<udd/)</pr>; <pos>p. p.</pos> <conjf>Thrown</conjf> <pr>(thr<omac/n)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Throwing</conjf>.]</vmorph> <ety>[OE. <ets><thorn/rowen</ets>, <ets><thorn/rawen</ets>, to throw, to twist, AS. <ets><thorn/r<amac/wan</ets> to twist, to whirl; akin to D. <ets>draaijen</ets>, G. <ets>drehen</ets>, OHG. <ets>dr<amac/jan</ets>, L. <ets>terebra</ets> an auger, gimlet, Gr. <?/ to bore, to turn, <?/ to pierce, <?/ a hole.  Cf. <er>Thread</er>, <er>Trite</er>, <er>Turn</er>, <pos>v. t.</pos>]</ety> <sn>1.</sn> <def>To fling, cast, or hurl with a certain whirling motion of the arm, to <xex>throw</xex> a ball; -- distinguished from to <xex>toss</xex>, or to <xex>bowl</xex>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To fling or cast in any manner; to drive to a distance from the hand or from an engine; to propel; to send; <as>as, to <ex>throw</ex> stones or dust with the hand; a cannon <ex>throws</ex> a ball; a fire engine <ex>throws</ex> a stream of water to extinguish flames</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>To drive by violence; <as>as, a vessel or sailors may be <ex>thrown</ex> upon a rock</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <fld>(Mil.)</fld> <def>To cause to take a strategic position; <as>as, he <ex>threw</ex> a detachment of his army across the river</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <def>To overturn; to prostrate in wrestling; <as>as, a man <ex>throws</ex> his antagonist</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>6.</sn> <def>To cast, as dice; to venture at dice.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Set less than thou <qex>throwest</qex>.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>7.</sn> <def>To put on hastily; to spread carelessly.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>O'er his fair limbs a flowery vest he <qex>threw</qex>.</q> <rj><qau>Pope.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>8.</sn> <def>To divest or strip one's self of; to put off.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>There the snake <qex>throws</qex> her enameled skin.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>9.</sn> <fld>(Pottery)</fld> <def>To form or shape roughly on a throwing engine, or potter's wheel, as earthen vessels.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>10.</sn> <def>To give forcible utterance to; to cast; to vent.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>I have <qex>thrown</qex><br/
A brave defiance in King Henry's teeth.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>11.</sn> <def>To bring forth; to produce, as young; to bear; -- said especially of rabbits.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>12.</sn> <def>To twist two or more filaments of, as silk, so as to form one thread; to twist together, as singles, in a direction contrary to the twist of the singles themselves; -- sometimes applied to the whole class of operations by which silk is prepared for the weaver.</def>  <rj><au>Tomlinson.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>To throw away</b></col>. <sd>(a)</sd> <cd>To lose by neglect or folly; to spend in vain; to bestow without a compensation; <as>as, <ex>to throw away</ex> time; <ex>to throw away</ex> money</as>.</cd> <sd>(b)</sd> <cd>To reject; <as>as, <ex>to throw away</ex> a good book, or a good offer</as>.</cd> -- <col><b>To throw back</b></col>. <sd>(a)</sd> <cd>To retort; to cast back, as a reply.</cd> <sd>(b)</sd> <cd>To reject; to refuse.</cd> <sd>(c)</sd> <cd>To reflect, as light.</cd> -- <col><b>To throw by</b></col>, <cd>to lay aside; to discard; to neglect as useless; <as>as, <ex>to throw by</ex> a garment</as>.</cd> -- <col><b>To throw down</b></col>, <cd>to subvert; to overthrow; to destroy; <as>as, <ex>to throw down</ex> a fence or wall</as>.</cd> -- <col><b>To throw in</b></col>. <sd>(a)</sd> <cd>To inject, as a fluid.</cd> <sd>(b)</sd> <cd>To put in; to deposit with others; to contribute; <as>as, <ex>to throw in</ex> a few dollars to help make up a fund; <ex>to throw in</ex> an occasional comment</as>.</cd> <sd>(c)</sd> <cd>To add without enumeration or valuation, as something extra to clinch a bargain.</cd> -- <col><b>To throw off</b></col>. <sd>(a)</sd> <cd>To expel; to free one's self from; <as>as, <ex>to throw off</ex> a disease</as>.</cd> <sd>(b)</sd> <cd>To reject; to discard; to abandon; <as>as, <ex>to throw off</ex> all sense of shame; <ex>to throw off</ex> a dependent</as>.</cd> <sd>(c)</sd> <cd>To make a start in a hunt or race.</cd> <mark>[Eng.]</mark><-- (d) To emit. Same as throw out (e).  <sd>(e)</sd> To disconcert or confuse. Same as to throw out (f). --> -- <col><b>To throw on</b></col>, <cd>to cast on; to load.</cd> -- <col><b>To throw one's self down</b></col>, <cd>to lie down neglectively or suddenly.</cd> -- <mcol><col><b>To throw one's self on</b></col> <it>or</it> <col><b>To throw one's self upon</b></col></mcol>. <sd>(a)</sd> <cd>To fall upon.</cd> <sd>(b)</sd> <cd>To resign one's self to the favor, clemency, or sustain power of (another); to repose upon.</cd> -- <col><b>To throw out</b></col>. <sd>(a)</sd> <cd>To cast out; to reject or discard; to expel.</cd> <ldquo/The other two, whom they had <xex>thrown out</xex>, they were content should enjoy their exile.<rdquo/ <au>Swift.</au> <ldquo/The bill was <xex>thrown out</xex>.<rdquo/ <au>Swift.</au> <sd>(b)</sd> <cd>To utter; to give utterance to; to speak; <as>as, <ex>to throw out</ex> insinuation or observation</as>.</cd> <ldquo/She <xex>throws out</xex> thrilling shrieks.<rdquo/ <au>Spenser.</au> <sd>(c)</sd> <cd>To distance; to leave behind.</cd> <au>Addison.</au> <sd>(d)</sd> <cd>To cause to project; <as>as, <ex>to throw out</ex> a pier or an abutment</as>.</cd> <sd>(e)</sd> <cd>To give forth; to emit; <as>as, an electric lamp <ex>throws out</ex> a brilliant light</as>.</cd> <sd>(f)</sd> <cd>To put out; to confuse; <as>as, a sudden question often <ex>throws out</ex> an orator</as>.</cd> -- <col><b>To throw over</b></col>, <cd>to abandon the cause of; to desert; to discard; <as>as, <ex>to throw over</ex> a friend in difficulties</as>.</cd> -- <col><b>To throw up</b></col>. <sd>(a)</sd> <cd>To resign; to give up; to demit; <as>as, <ex>to throw up</ex> a commission</as>.</cd> <ldquo/Experienced gamesters <xex>throw up</xex> their cards when they know that the game is in the enemy's hand.<rdquo/ <au>Addison.</au> <sd>(b)</sd> <cd>To reject from the stomach; to vomit.</cd> <sd>(c)</sd> <cd>To construct hastily; <as>as, <ex>to throw up</ex> a breastwork of earth</as>.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Throw</ent><br/
<hw>Throw</hw> <pr>(?)</pr>, <pos>v. i.</pos> <def>To perform the act of throwing or casting; to cast; specifically, to cast dice.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>To throw about</b></col>, <cd>to cast about; to try expedients.</cd> <mark>[R.]</mark></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Throw</ent><br/
<hw>Throw</hw>, <pos>n.</pos> <sn>1.</sn> <def>The act of hurling or flinging; a driving or propelling from the hand or an engine; a cast.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>He heaved a stone, and, rising to the <qex>throw</qex>,<br/
He sent it in a whirlwind at the foe.</q> <rj><qau>Addison.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A stroke; a blow.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>Nor shield defend the thunder of his <qex>throws</qex>.</q> <rj><qau>Spenser.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>The distance which a missile is, or may be, thrown; <as>as, a stone's <ex>throw</ex></as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>A cast of dice; the manner in which dice fall when cast; <as>as, a good <ex>throw</ex></as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <def>An effort; a violent sally.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>Your youth admires<br/
The <qex>throws</qex> and swellings of a Roman soul.</q> <rj><qau>Addison.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>6.</sn> <fld>(Mach.)</fld> <def>The extreme movement given to a sliding or vibrating reciprocating piece by a cam, crank, eccentric, or the like; travel; stroke; <as>as, the <ex>throw</ex> of a slide valve</as>. Also, frequently, the length of the radius of a crank, or the eccentricity of an eccentric; <as>as, the <ex>throw</ex> of the crank of a steam engine is equal to half the stroke of the piston</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>7.</sn> <fld>(Pottery)</fld> <def>A potter's wheel or table; a jigger. See 2d <er>Jigger</er>, 2 <sd>(a)</sd>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>8.</sn> <def>A turner's lathe; a throwe.</def> <mark>[Prov. Eng.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><sn>9.</sn> <fld>(Mining)</fld> <def>The amount of vertical displacement produced by a fault; -- according to the direction it is designated as an <xex>upthrow</xex>, or a <xex>downthrow</xex>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>throw away</ent><br/
<hw>throw away</hw> <pos>v. t.</pos> <sn>1.</sn> <def>to discard.</def><br/
[<source>PJC</source>]</p>

<p><sn>2.</sn> <def>to waste or squander.</def><br/
[<source>PJC</source>]<br/
[<source>1913 Webster</source>]</p>

<p><ent>throwaway</ent><br/
<hw>throw"a*way</hw> <pos>a.</pos> <sn>1.</sn> <def>designed to be discarded after a single use; disposable.</def><br/
[<source>PJC</source>]</p>

<p><sn>2.</sn> <def>spoken with deliberate underemphasis; <as>as, a <ex>throwaway</ex> line in a play</as>.</def><br/
[<source>PJC</source>]<br/
[<source>1913 Webster</source>]</p>

<p><ent>throwaway</ent><br/
<hw>throw"a*way</hw> <pos>n.</pos> <sn>1.</sn> <def>an advertisement (usually printed on a page or in a leaflet) intended for wide distribution.</def><br/
<syn><b>Syn. --</b> circular, handbill, bill, broadside, broadsheet, flier, flyer</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><sn>2.</sn> <def>words spoken in a casual way with conscious underemphasis.</def><br/
[<source>WordNet 1.5</source>]<br/
[<source>1913 Webster</source>]</p>

<p><ent>throwback</ent><br/
<hw>throw"back</hw>, <pos>n.</pos> <sn>1.</sn> <def>the reappearance in an organism of characteristics of an earlier ancestral type; atavism.</def><br/
[<source>PJC</source>]</p>

<p><sn>2.</sn> <def>an organisms having characteristics of an earlier ancestral type.</def><br/
[<source>PJC</source>]<br/
[<source>1913 Webster</source>]</p>

<p><ent>Throw back</ent><br/
<hw>Throw back</hw>, <pos>v. i.</pos> <def>to revert to an ancestral type or character.</def> <ldquo/A large proportion of the steerage passengers <xex>throw back</xex> to their Darwinian ancestry.<rdquo/ <rj><au>The Century.</au></rj><br/
[<source>Webster 1913 Suppl.</source>]<br/
[<source>1913 Webster</source>]</p>

<p><ent>Throw-crook</ent><br/
<hw>Throw"-crook`</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Agric.)</fld> <def>An instrument used for twisting ropes out of straw.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Throwe</ent><br/
<hw>Throwe</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A turning lathe.</def> <mark>[Prov. Eng.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Thrower</ent><br/
<hw>Throw"er</hw> <pr>(thr<omac/"<etil/r)</pr>, <pos>n.</pos> <def>One who throws.</def> <specif>Specifically:</specif> <sd>(a)</sd> <def>One who throws or twists silk; a throwster.</def> <sd>(b)</sd> <def>One who shapes vessels on a throwing engine.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Throwing</ent><br/
<hw>Throw"ing</hw>, <def><pos>a. & n.</pos> from <er>Throw</er>, <pos>v.</pos></def><br/
[<source>1913 Webster</source>]</p>

<p><cs><mcol><col><b>Throwing engine</b></col>, <col><b>Throwing mill</b></col>, <col><b>Throwing table</b></col>, <it>or</it> <col><b>Throwing wheel</b></col></mcol> <fld>(Pottery)</fld>, <cd>a machine on which earthenware is first rudely shaped by the hand of the potter from a mass of clay revolving rapidly on a disk or table carried by a vertical spindle; a potter's wheel.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Throwing stick</ent><br/
<hw>Throw"ing stick`</hw>. <fld>(Anthropol.)</fld> <def>An instrument  used by various savage races for throwing a spear; -- called also <altname>throw stick</altname> and <altname>spear thrower</altname>.  One end of the stick receives the butt of the spear, as upon a hook or thong, and the other end is grasped with the hand, which also holds the spear, toward the middle, above it with the finger and thumb, the effect being to bring the place of support nearer the center of the spear, and practically lengthen the arm in the act of throwing.  In Mexico, one such device is called the <stype>atlatl</stype>.</def><br/
[<source>Webster 1913 Suppl.</source> <source>+PJC</source>]<br/
[<source>1913 Webster</source>]</p>

<p><ent>Thrown</ent><br/
<hw>Thrown</hw> <pr>(thr<omac/n)</pr>, <def><pos>a. & p. p.</pos> from <er>Throw</er>, <pos>v.</pos></def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Thrown silk</b></col>, <cd>silk thread consisting of two or more singles twisted together like a rope, in a direction contrary to that in which the singles of which it is composed are twisted.</cd> <au>M'Culloch.</au> -- <col><b>Thrown singles</b></col>, <cd>silk thread or cord made by three processes of twisting, first into <xex>singles</xex>, two or more of which are twisted together making <xex>dumb singles</xex>, and several of these twisted together to make <xex>thrown singles</xex>.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Throw-off</ent><br/
<hw>Throw"-off`</hw> <pr>(thr<omac/"<ocr/f`; 115)</pr>, <pos>n.</pos> <def>A start in a hunt or a race.</def> <mark>[Eng.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Throwster</ent><br/
<hw>Throw"ster</hw> <pr>(thr<omac/"st<etil/r)</pr>, <pos>n.</pos> <ety>[<ets>Throw</ets> + <ets>-ster</ets>.]</ety> <def>One who throws or twists silk; a thrower.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Thru</ent><br/
<hw>Thru</hw> <pr>(thr<udd/)</pr>, <pos>prep., adv. & a.</pos> <def>Through.</def> <mark>[Ref. spelling.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Thrum</ent><br/
<hw>Thrum</hw> <pr>(thr<ucr/m)</pr>, <pos>n.</pos> <ety>[OE. <ets>thrum</ets>, <ets>throm</ets>; akin to OD. <ets>drom</ets>, D. <ets>dreum</ets>, G. <ets>trumm</ets>, lump, end, fragment, OHG. <ets>drum</ets> end, Icel. <ets><thorn/r<oum/mr</ets> edge, brim, and L. <ets>terminus</ets> a limit, term.  Cf. <er>Term</er>.]</ety> <altsp>[Written also <asp>thrumb</asp>.]</altsp> <sn>1.</sn> <def>One of the ends of weaver's threads; hence, any soft, short threads or tufts resembling these.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Any coarse yarn; an unraveled strand of rope.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <fld>(Bot.)</fld> <def>A threadlike part of a flower; a stamen.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <fld>(Mining)</fld> <def>A shove out of place; a small displacement or fault along a seam.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <fld>(Naut.)</fld> <def>A mat made of canvas and tufts of yarn.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Thrum cap</b></col>, <cd>a knitted cap.</cd> <au>Halliwell.</au> -- <col><b>Thrum hat</b></col>, <cd>a hat made of coarse woolen cloth.</cd> <au>Minsheu.</au></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Thrum</ent><br/
<hw>Thrum</hw>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Thrummed</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Thrumming</conjf>.]</vmorph> <sn>1.</sn> <def>To furnish with thrums; to insert tufts in; to fringe.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Are we born to <qex>thrum</qex> caps or pick straw?</q> <rj><qau>Quarles.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Naut.)</fld> <def>To insert short pieces of rope-yarn or spun yarn in; <as>as, to <ex>thrum</ex> a piece of canvas, or a mat, thus making a rough or tufted surface</as>.</def>  <rj><au>Totten.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Thrum</ent><br/
<hw>Thrum</hw>, <pos>v. i.</pos> <ety>[CF. Icel. <ets><?/ruma</ets> to rattle, to thunder, and E. <ets>drum</ets>.]</ety> <sn>1.</sn> <def>To play rudely or monotonously on a stringed instrument with the fingers; to strum.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Hence, to make a monotonous drumming noise; <as>as, to <ex>thrum</ex> on a table</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Thrum</ent><br/
<hw>Thrum</hw>, <pos>v. t.</pos> <sn>1.</sn> <def>To play, as a stringed instrument, in a rude or monotonous manner.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Hence, to drum on; to strike in a monotonous manner; to <xex>thrum</xex> the table.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Thrum-eyed</ent><br/
<hw>Thrum"-eyed`</hw> <pr>(?)</pr>, <pos>a.</pos> <fld>(Bot.)</fld> <def>Having the anthers raised above the stigma, and visible at the throat of the corolla, as in long-stamened primroses; -- the reverse of <xex>pin-eyed</xex>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Thrummy</ent><br/
<hw>Thrum"my</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Like thrums; made of, furnished with, or characterized by, thrums.</def>  <rj><au>Dampier.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>On her head <qex>thrummy</qex> cap she had.</q> <rj><qau>Chalkhill.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Thrumwort</ent><br/
<hw>Thrum"wort`</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Bot.)</fld> <def>A kind of amaranth (<spn>Amarantus caudatus</spn>).</def>  <rj><au>Dr. Prior.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Thruout</ent><br/
<hw>Thru*out"</hw> <pr>(?)</pr>. <def>Throughout.</def> <mark>[Ref. spelling.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Thrush</ent><br/
<hw>Thrush</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[OE. <ets><thorn/rusche</ets>, AS. <ets><thorn/rysce</ets>; akin to OHG. <ets>drosca</ets>, <ets>droscea</ets>, <ets>droscela</ets>, and E. <ets>throstle</ets>.  Cf. <er>Throstle</er>.]</ety> <sn>1.</sn> <fld>(Zool.)</fld> <def>Any one of numerous species of singing birds belonging to <gen>Turdus</gen> and allied genera. They are noted for the sweetness of their songs.</def><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ Among the best-known European species are the song thrush or throstle (<spn>Turdus musicus</spn>), the missel thrush (see under <er>Missel</er>), the European redwing, and the blackbird. The most important American species are the wood thrush (<spn>Turdus mustelinus</spn>), Wilson's thrush (<spn>Turdus fuscescens</spn>), the hermit thrush (see under <er>Hermit</er>), Swainson's thrush (<spn>Turdus Aliciae</spn>), and the migratory thrush, or American robin (see <er>Robin</er>).</note><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Zool.)</fld> <def>Any one of numerous species of singing birds more or less resembling the true thrushes in appearance or habits; as the thunderbird and the American brown thrush (or thrasher). See <er>Brown thrush</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Ant thrush</b></col>. <cd>See <er>Ant thrush</er>, <er>Breve</er>, and <er>Pitta</er>.</cd> -- <col><b>Babbling thrush</b></col>, <cd>any one of numerous species of Asiatic timaline birds; -- called also <altname>babbler</altname>.</cd> -- <col><b>Fruit thrush</b></col>, <cd>any species of bulbul.</cd> -- <col><b>Shrike thrush</b></col>. <cd>See under <er>Shrike</er>.</cd> -- <col><b>Stone thrush</b></col>, <cd>the missel thrush; -- said to be so called from its marbled breast.</cd> -- <col><b>Thrush nightingale</b></col>. <cd>See <er>Nightingale</er>, 2.</cd> -- <col><b>Thrush tit</b></col>, <cd>any one of several species of Asiatic singing birds of the genus <gen>Cochoa</gen>. They are beautifully colored birds allied to the tits, but resembling thrushes in size and habits.</cd> -- <col><b>Water thrush</b></col>. <sd>(a)</sd> <cd>The European dipper.</cd> <sd>(b)</sd> <cd>An American warbler (<spn>Seiurus Noveboracensis</spn>).</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Thrush</ent><br/
<hw>Thrush</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Akin to Dan. <ets>tr<oum/ske</ets>, Sw. <ets>trosk</ets>; cf. Dan. <ets>t<oum/r</ets> dry, Sw. <ets>torr</ets>, Icel. <ets><thorn/urr</ets>, AS. <ets><thorn/yrr</ets>, OE. <ets>thrust</ets> thrist, E. <ets>thrist</ets>.]</ety> <sn>1.</sn> <fld>(Med.)</fld> <def>An affection of the mouth, fauces, etc., common in newly born children, characterized by minute ulcers called <xex>aphthae</xex>. See <er>Aphthae</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Far.)</fld> <def>An inflammatory and suppurative affection of the feet in certain animals. In the horse it is in the frog.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Thrushel</ent><br/
<hw>Thrush"el</hw> <pr>(?)</pr>, <pos>n.</pos> <def>The song thrush.</def> <mark>[Prov. Eng.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Thrusher</ent><br/
<hw>Thrush"er</hw> <pr>(?)</pr>, <pos>n.</pos> <def>The song thrush.</def> <mark>[Prov. Eng.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Thrust</ent><br/
<hw>Thrust</hw> <pr>(?)</pr>, <pos>n. & v.</pos> <def>Thrist.</def> <mark>[Obs.]</mark>  <rj><au>Spenser.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Thrust</ent><br/
<hw>Thrust</hw>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Thrust</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Thrusting</conjf>.]</vmorph> <ety>[OE. <ets><?/rusten</ets>, <ets><?/risten</ets>, <ets><?/resten</ets>, Icel. <ets><?/r<?/st<?/</ets> to thrust, press, force, compel; perhaps akin to E. <ets>threat</ets>.]</ety> <sn>1.</sn> <def>To push or drive with force; to drive, force, or impel; to shove; <as>as, to <ex>thrust</ex> anything with the hand or foot, or with an instrument</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Into a dungeon <qex>thrust</qex>, to work with slaves.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To stab; to pierce; -- usually with <xex>through</xex>.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><mcol><col><b>To thrust away</b></col> <it>or</it> <col><b>To thrust from</b></col></mcol>, <cd>to push away; to reject.</cd> -- <col><b>To thrust in</b></col>, <cd>to push or drive in.</cd> -- <col><b>To thrust off</b></col>, <cd>to push away.</cd> -- <col><b>To thrust on</b></col>, <cd>to impel; to urge.</cd> -- <mcol><col><b>To thrust one's self in</b></col> <it>or</it> <col><b>To thrust one's self into</b></col></mcol>, <cd>to obtrude upon, to intrude, as into a room; to enter (a place) where one is not invited or not welcome.</cd> -- <col><b>To thrust out</b></col>, <cd>to drive out or away; to expel.</cd> -- <col><b>To thrust through</b></col>, <cd>to pierce; to stab.</cd> <ldquo/I am eight times <xex>thrust through</xex> the doublet.<rdquo/ <au>Shak.</au> -- <col><b>To thrust together</b></col>, <cd>to compress.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Thrust</ent><br/
<hw>Thrust</hw>, <pos>v. i.</pos> <sn>1.</sn> <def>To make a push; to attack with a pointed weapon; <as>as, a fencer <ex>thrusts</ex> at his antagonist</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To enter by pushing; to squeeze in.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>And <qex>thrust</qex> between my father and the god.</q> <rj><qau>Dryden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>To push forward; to come with force; to press on; to intrude.</def> <ldquo/Young, old, <xex>thrust</xex> there in mighty concourse.<rdquo/  <rj><au>Chapman.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>To thrust to</b></col>, <cd>to rush upon.</cd> <mark>[Obs.]</mark></cs><br/
[<source>1913 Webster</source>]</p>

<p><q>As doth an eager hound<br/
<qex>Thrust to</qex> an hind within some covert glade.</q> <rj><qau>Spenser.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Thrust</ent><br/
<hw>Thrust</hw>, <pos>n.</pos> <sn>1.</sn> <def>A violent push or driving, as with a pointed weapon moved in the direction of its length, or with the hand or foot, or with any instrument; a stab; -- a word much used as a term of fencing.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>[Polites] Pyrrhus with his lance pursues,<br/
And often reaches, and his <qex>thrusts</qex> renews.</q> <rj><qau>Dryden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>An attack; an assault.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>One <qex>thrust</qex> at your pure, pretended mechanism.</q> <rj><qau>Dr. H. More.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <fld>(Mech.)</fld> <def>The force or pressure of one part of a construction against other parts; especially <fld>(Arch.)</fld>, a horizontal or diagonal outward pressure, as of an arch against its abutments, or of rafters against the wall which support them.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <fld>(Mining)</fld> <def>The breaking down of the roof of a gallery under its superincumbent weight.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Thrust bearing</b></col> <fld>(Screw Steamers)</fld>, <cd>a bearing arranged to receive the thrust or endwise pressure of the screw shaft.</cd> -- <col><b>Thrust plane</b></col> <fld>(Geol.)</fld>, <cd>the surface along which dislocation has taken place in the case of a reversed fault.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><syn><b>Syn.</b> -- Push; shove; assault; attack.</syn> <usage> <er>Thrust</er>, <er>Push</er>, <er>Shove</er>. <xex>Push</xex> and <xex>shove</xex> usually imply the application of force by a body already in contact with the body to be impelled. <xex>Thrust</xex>, often, but not always, implies the impulse or application of force by a body which is in motion before it reaches the body to be impelled.</usage><br/
[<source>1913 Webster</source>]</p>

<p><ent>Thruster</ent><br/
<hw>Thrust"er</hw> <pr>(?)</pr>, <pos>n.</pos> <def>One who thrusts or stabs.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Thrusting</ent><br/
<hw>Thrust"ing</hw>, <pos>n.</pos> <sn>1.</sn> <def>The act of pushing with force.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Dairies)</fld> <sd>(a)</sd> <def>The act of squeezing curd with the hand, to expel the whey.</def> <sd>(b)</sd> <pluf>pl.</pluf> <def>The white whey, or that which is last pressed out of the curd by the hand, and of which butter is sometimes made.</def> <altsp>[Written also <asp>thrutchthings</asp>.]</altsp> <mark>[Prov. Eng.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Thrusting screw</b></col>, <cd>the screw of a screw press, as for pressing curd in making cheese.</cd> <mark>[R.]</mark></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Thrustle</ent><br/
<hw>Thrus"tle</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Zool.)</fld> <def>The throstle, or song thrust.</def> <mark>[Obs. or Prov. Eng.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>When he heard the <qex>thrustel</qex> sing.</q> <rj><qau>Chaucer.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Thryes</ent><br/
<hw>Thryes</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Thrice.</def> <mark>[Obs.]</mark>  <rj><au>Chaucer.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Thryfallow</ent><br/
<hw>Thry"fal`low</hw> <pr>(?)</pr>, <pos>v. t.</pos> <ety>[Perhaps fr. <ets>thrice + fallow</ets>.  Cf. <er>Trifallow</er>.]</ety> <def>To plow for the third time in summer; to trifallow.</def> <mark>[R.]</mark> <altsp>[Written also <asp>thrifallow</asp>.]</altsp>  <rj><au>Tusser.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Thud</ent><br/
<hw>Thud</hw> <pr>(th<ucr/d)</pr>, <pos>n.</pos> <ety>[Cf. AS. <ets><thorn/<omac/den</ets> a whirlwind, violent wind, or E. <ets>thump</ets>.]</ety> <def>A dull sound without resonance, like that produced by striking with, or striking against, some comparatively soft substance; also, the stroke or blow producing such sound; <as>as, the <ex>thrud</ex> of a cannon ball striking the earth</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>At every new <qex>thud</qex> of the blast, a sob arose.</q> <rj><qau>Jeffrey.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>At intervals there came some tremendous <qex>thud</qex> on the side of the steamer.</q> <rj><qau>C. Mackay.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Thud</ent><br/
<hw>Thud</hw> <pr>(th<ucr/d)</pr>, <pos>v. i. & t.</pos> <def>To make, or strike so as to make, a dull sound, or thud.</def></p>

<p><q>Hardly the softest <qex>thudding</qex> of velvety pads.</q>  <rj><qau>A. C. Doyle.</qau></rj></p>

<p><q>The waves break into spray, dash and rumble and <qex>thud</qex> below your feet.</q>  <rj><qau>H. F. Brown.</qau></rj><br/
[<source>Webster 1913 Suppl.</source>]<br/
[<source>1913 Webster</source>]</p>

<p><ent>Thug</ent><br/
<hw>Thug</hw> <pr>(th<ucr/g; 277)</pr>, <pos>n.</pos> <ety>[Hind. <ets>thag</ets> a deceiver, robber.]</ety> <sn>1.</sn> <def>One of an association of robbers and murderers in India who practiced murder by stealthy approaches, and from religious motives. They have been nearly exterminated by the British government.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>An assassin; a ruffian; a rough.</def> <ldquo/<xex>Thugs</xex> and midnight rounders.<rdquo/  <rj><au>The Century.</au></rj><br/
[<source>Webster 1913 Suppl.</source>]<br/
[<source>1913 Webster</source>]</p>

<p><ent>Thuggee</ent><br/
<hw>Thug*gee"</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Hind. <ets><tsdot/hag<imac/</ets>.]</ety> <def>The practice of secret or stealthy murder by Thugs.</def> <ldquo/One of the suppressors of <xex>Thuggee</xex>.<rdquo/  <rj><au>J. D. Hooker.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Thuggism</ent><br/
<ent>Thuggery</ent><br/
<mhw>{ <hw>Thug"ger*y</hw> <pr>(?)</pr>, <hw>Thug"gism</hw> <pr>(?)</pr>, }</mhw> <pos>n.</pos> <def>Thuggee.</def><br/
[<source>1913 Webster</source>]</p>

<p><-- p. 1505 --><br/
[<source>1913 Webster</source>]</p>

<p><ent>Thuja</ent><br/
||<hw>Thu"ja</hw> <pr>(th<umac/"j<adot/)</pr>, <pos>n.</pos> <ety>[NL., from Gr. <grk>qyi`a</grk> an African tree with sweet-smelling wood.]</ety> <fld>(Bot.)</fld> <def>A genus of evergreen trees, thickly branched, remarkable for the distichous arrangement of their branches, and having scalelike, closely imbricated, or compressed leaves.</def> <altsp>[Written also <asp>thuya</asp>.]</altsp> See <er>Thyine wood</er>.<br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ <spn>Thuja occidentalis</spn> is the <stype><spn>Arbor vitae</spn></stype> of the Eastern and Northern United States. <spn>Thuja gigantea</spn> of North-western America is a very large tree, there called <stype>red cedar</stype>, and <stype>canoe cedar</stype>, and furnishes a useful timber.</note><br/
[<source>1913 Webster</source>]</p>

<p><ent>thujone</ent><br/
||<hw>thu"jone</hw> <pr>(th<umac/"j<omac/n)</pr>, <pos>n.</pos> <def>An oil, <chform>C10H16O</chform>, the chief constituent of cedar leaf oil (thuja oil), which is obtained from the northern white cedar (arbor vitae, <spn>Thuja occidentalis</spn>) or the western Red Cedar (<spn>Thuja plicatis</spn>).  It is a stimulant similar to camphor.  It is also called <altname>thujol</altname>, <altname>thuyol</altname>, <altname>absinthol</altname>, <altname>thuyone</altname>, <altname>tanacetol</altname>, and <altname>tanacetone</altname>.</def> <au>Stedman.</au><br/
[<source>PJC</source>]</p>

<p><ent>Thule</ent><br/
||<hw>Thu"le</hw> <pr>(th<umac/"l<esl/)</pr>, <pos>prop. n.</pos> <ety>[L. <ets>Thule</ets>, <ets>Thyle</ets>, Gr. <grk>Qoy`lh</grk>, <grk>Qy`lh</grk>.]</ety> <def>The name given by ancient geographers to the northernmost part of the habitable world.  According to some, this land was Norway, according to others, Iceland, or more probably Mainland, the largest of the Shetland islands; hence, the Latin phrase <xex>ultima Thule</xex>, farthest Thule.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Thulia</ent><br/
<hw>Thu"li*a</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[NL.]</ety> <fld>(Chem.)</fld> <def>Oxide of thulium.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Thulium</ent><br/
<hw>Thu"li*um</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[NL. See <er>Thule</er>.]</ety> <fld>(Chem.)</fld> <def>A rare metallic element of the lanthanide group having atomic number 69, and atomic weight 168.93.  It is found in the mineral gadolinite and other minerals, together with other rare earths.  For more information see the data from <a href="more\thulium.htm">ChemGlobe</a>. </def><br/
[<source>1913 Webster</source> + <source>PJC</source>]</p>

<p><ent>Thumb</ent><br/
<hw>Thumb</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[OE. <ets>thombe</ets>, <ets>thoumbe</ets>, <ets><thorn/ume</ets>, AS. <ets><thorn/<umac/ma</ets>; akin to OFries. <ets>th<umac/ma</ets>, D. <ets>duim</ets>, G. <ets>daumen</ets>, OHG. <ets>d<umac/mo</ets>, Icel. <ets><thorn/umall</ets>, Dan. <ets>tommel</ets>finger, Sw. <ets>tumme</ets>, and perhaps to L. <ets>tumere</ets> to swell. <root/56.  Cf. <er>Thimble</er>, <er>Tumid</er>.]</ety> <def>The short, thick first digit of the human hand, differing from the other fingers in having but two phalanges; the pollex. See <er>Pollex</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Upon his <qex>thumb</qex> he had of gold a ring.</q> <rj><qau>Chaucer.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Thumb band</b></col>, <cd>a twist of anything as thick as the thumb.</cd> <au>Mortimer.</au> -- <col><b>Thumb blue</b></col>, <cd>indigo in the form of small balls or lumps, used by washerwomen to blue linen, and the like.</cd> -- <col><b>Thumb latch</b></col>, <cd>a door latch having a lever formed to be pressed by the thumb.</cd> -- <col><b>Thumb mark</b></col>. <sd>(a)</sd> <cd>The mark left by the impression of a thumb, as on the leaves of a book.</cd> <au>Longfellow.</au> <sd>(b)</sd> <cd>The dark spot over each foot in finely bred black and tan terriers.</cd> -- <col><b>Thumb nut</b></col>, <cd>a nut for a screw, having wings to grasp between the thumb and fingers in turning it; also, a nut with a knurled rim for the same perpose.</cd> -- <col><b>Thumb ring</b></col>, <cd>a ring worn on the thumb.</cd> <au>Shak.</au> -- <col><b>Thumb stall</b></col>. <sd>(a)</sd> <cd>A kind of thimble or ferrule of iron, or leather, for protecting the thumb in making sails, and in other work.</cd> <sd>(b)</sd> <fld>(Mil.)</fld> <cd>A buckskin cushion worn on the thumb, and used to close the vent of a cannon while it is sponged, or loaded.</cd> -- <col><b>Under one's thumb</b></col>, <cd>completely under one's power or influence; in a condition of subservience.</cd> <mark>[Colloq.]</mark></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Thumb</ent><br/
<hw>Thumb</hw>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Thumbed</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Thumbing</conjf> <pr>(?)</pr>.]</vmorph> <sn>1.</sn> <def>To handle awkwardly.</def>  <rj><au>Johnson.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To play with the thumbs, or with the thumbs and fingers; <as>as, to <ex>thumb</ex> over a tune</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>To soil or wear with the thumb or the fingers; to soil, or wear out, by frequent handling; also, to cover with the thumb; <as>as, to <ex>thumb</ex> the touch-hole of a cannon</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>He gravely informed the enemy that all his cards had been <qex>thumbed</qex> to pieces, and begged them to let him have a few more packs.</q> <rj><qau>Macaulay.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Thumb</ent><br/
<hw>Thumb</hw>, <pos>v. i.</pos> <def>To play with the thumb or thumbs; to play clumsily; to thrum.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Thumbbird</ent><br/
<hw>Thumb"bird`</hw> <pr>(?)</pr>, <pos>n.</pos> <def>The goldcrest.</def> <mark>[Prov. Eng.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Thumbed</ent><br/
<hw>Thumbed</hw> <pr>(?)</pr>, <pos>a.</pos> <sn>1.</sn> <def>Having thumbs.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Soiled by handling.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Thumbkin</ent><br/
<hw>Thumb"kin</hw> <pr>(?)</pr>, <pos>n.</pos> <def>An instrument of torture for compressing the thumb; a thumbscrew.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Thumbless</ent><br/
<hw>Thumb"less</hw>, <pos>a.</pos> <def>Without a thumb.</def>  <rj><au>Darwin.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Thumbscrew</ent><br/
<hw>Thumb"screw`</hw> <pr>(?)</pr>, <pos>n.</pos> <sn>1.</sn> <def>A screw having a flat-sided or knurled head, so that it may be turned by the thumb and forefinger.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>An old instrument of torture for compressing the thumb by a screw; a thumbkin.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Thummie</ent><br/
<hw>Thum"mie</hw> <pr>(?)</pr>, <pos>n.</pos> <def>The chiff-chaff.</def> <mark>[Prov. Eng.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Thummim</ent><br/
<hw>Thum"mim</hw> <pr>(?)</pr>, <pos>n. pl.</pos> <ety>[Heb., pl. of <ets>th<omac/m</ets> perfection.]</ety> <def>A mysterious part or decoration of the breastplate of the Jewish high priest. See the note under <er>Urim</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Thump</ent><br/
<hw>Thump</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Probably of imitative origin; perhaps influenced by <ets>dump</ets>, v. t.]</ety> <sn>1.</sn> <def>The sound made by the sudden fall or blow of a heavy body, as of a hammer, or the like.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The distant forge's swinging <qex>thump</qex> profound.</q> <rj><qau>Wordsworth.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>With heavy <qex>thump</qex>, a lifeless lump,<br/
They dropped down, one by one.</q> <rj><qau>Coleridge.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A blow or knock, as with something blunt or heavy; a heavy fall.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The watchman gave so great a <qex>thump</qex> at my door, that I awaked at the knock.</q> <rj><qau>Tatler.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Thump</ent><br/
<hw>Thump</hw>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Thumped</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Thumping</conjf>.]</vmorph> <def>To strike or beat with something thick or heavy, or so as to cause a dull sound.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>These bastard Bretons; whom our hathers<br/
Have in their own land beaten, bobbed, and <qex>thumped</qex>.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Thump</ent><br/
<hw>Thump</hw>, <pos>v. i.</pos> <def>To give a thump or thumps; to strike or fall with a heavy blow; to pound.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>A watchman at midnight <qex>thumps</qex> with his pole.</q> <rj><qau>Swift.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Thumper</ent><br/
<hw>Thump"er</hw> <pr>(?)</pr>, <pos>n.</pos> <def>One who, or that which, thumps.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Thumping</ent><br/
<hw>Thump"ing</hw>, <pos>a.</pos> <def>Heavy; large.</def> <mark>[Colloq.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Thunder</ent><br/
<hw>Thun"der</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[OE. <ets><thorn/under</ets>, <ets><thorn/onder</ets>, <ets><thorn/oner</ets>, AS. <ets><thorn/unor</ets>; akin to <ets><thorn/unian</ets> to stretch, to thunder, D. <ets>donder</ets> thunder, G. <ets>donner</ets>, OHG. <ets>donar</ets>, Icel. <ets><thorn/<omac/rr</ets> Thor, L. <ets>tonare</ets> to thunder, <ets>tonitrus</ets> thunder, Gr. <grk>to`nos</grk> a stretching, straining, Skr. <ets>tan</ets> to stretch. <root/52. See <er>Thin</er>, and cf. <er>Astonish</er>, <er>Detonate</er>, <er>Intone</er>, <er>Thursday</er>, <er>Tone</er>.]</ety> <sn>1.</sn> <def>The sound which follows a flash of lightning; the report of a discharge of atmospheric electricity.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>The discharge of electricity; a thunderbolt.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>The revenging gods<br/
'Gainst parricides did all their <qex>thunders</qex> bend.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>Any loud noise; <as>as, the <ex>thunder</ex> of cannon</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>An alarming or statrling threat or denunciation.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The <qex>thunders</qex> of the Vatican could no longer strike into the heart of princes.</q> <rj><qau>Prescott.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Thunder pumper</b></col>. <fld>(Zool.)</fld> <sd>(a)</sd> <cd>The croaker (<spn>Haploidontus grunniens</spn>).</cd> <sd>(b)</sd> <cd>The American bittern or stake-driver.</cd> -- <col><b>Thunder rod</b></col>, <cd>a lightning rod.</cd> <mark>[R.]</mark> -- <col><b>Thunder snake</b></col>. <fld>(Zool.)</fld> <sd>(a)</sd> <cd>The chicken, or milk, snake.</cd> <sd>(b)</sd> <cd>A small reddish ground snake (<spn>Carphophis amoena</spn> syn. <spn>Celuta amoena</spn>) native to the Eastern United States; -- called also <altname>worm snake</altname>.</cd> -- <col><b>Thunder tube</b></col>, <cd>a fulgurite. See <er>Fulgurite</er>.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Thunder</ent><br/
<hw>Thun"der</hw> <pr>(?)</pr>, <pos>v. i.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Thundered</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Thundering</conjf>.]</vmorph> <ety>[AS. <ets><thorn/unrian</ets>. See <er>Thunder</er>, <pos>n.</pos>]</ety> <sn>1.</sn> <def>To produce thunder; to sound, rattle, or roar, as a discharge of atmospheric electricity; -- often used impersonally; <as>as, it <ex>thundered</ex> continuously</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Canst thou <qex>thunder</qex> with a voice like him?</q> <rj><qau>Job xl. 9.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Fig.: To make a loud noise; esp. a heavy sound, of some continuance.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>His dreadful voice no more<br/
Would <qex>thunder</qex> in my ears.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>To utter violent denunciation.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Thunder</ent><br/
<hw>Thun"der</hw>, <pos>v. t.</pos> <def>To emit with noise and terror; to utter vehemently; to publish, as a threat or denunciation.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Oracles severe<br/
Were daily <qex>thundered</qex> in our general's ear.</q> <rj><qau>Dryden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>An archdeacon, as being a prelate, may <qex>thunder</qex> out an ecclesiastical censure.</q> <rj><qau>Ayliffe.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Thunderbird</ent><br/
<hw>Thun"der*bird`</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Zool.)</fld> <def>An Australian insectivorous singing bird (<spn>Pachycephala gutturalis</spn>). The male is conspicuously marked with black and yellow, and has a black crescent on the breast. Called also <altname>white-throated thickhead</altname>, <altname>orange-breasted thrust</altname>, <altname>black-crowned thrush</altname>, <altname>guttural thrush</altname>, and <altname>black-breasted flycatcher</altname>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Thunderbolt</ent><br/
<hw>Thun"der*bolt`</hw> <pr>(?)</pr>, <pos>n.</pos> <sn>1.</sn> <def>A shaft of lightning; a brilliant stream of electricity passing from one part of the heavens to another, or from the clouds to the earth.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Something resembling lightning in suddenness and effectiveness.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The Scipios' worth, those <qex>thunderbolts</qex> of war.</q> <rj><qau>Dryden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>Vehement threatening or censure; especially, ecclesiastical denunciation; fulmination.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>He severely threatens such with the <qex>thunderbolt</qex> of excommunication.</q> <rj><qau>Hakewill.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <fld>(Paleon.)</fld> <def>A belemnite, or thunderstone.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Thunderbolt beetle</b></col> <fld>(Zool.)</fld>, <cd>a long-horned beetle (<spn>Arhopalus fulminans</spn>) whose larva bores in the trunk of oak and chestnut trees. It is brownish and bluish-black, with W-shaped whitish or silvery markings on the elytra.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Thunderburst</ent><br/
<hw>Thun"der*burst`</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A burst of thunder.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Thunderclap</ent><br/
<hw>Thun"der*clap`</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A sharp burst of thunder; a sudden report of a discharge of atmospheric electricity.</def> <ldquo/<xex>Thunderclaps</xex> that make them quake.<rdquo/  <rj><au>Spenser.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>When suddenly the <qex>thunderclap</qex> was heard.</q> <rj><qau>Dryden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Thundercloud</ent><br/
<hw>Thun"der*cloud`</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A cloud charged with electricity, and producing lightning and thunder.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Thunderer</ent><br/
<hw>Thun"der*er</hw> <pr>(?)</pr>, <pos>n.</pos> <def>One who thunders; -- used especially as a translation of L. <xex>tonans</xex>, an epithet applied by the Romans to several of their gods, esp. to <persfn>Jupiter</persfn>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>That dreadful oath which binds the <qex>Thunderer</qex>.</q> <rj><qau>Pope.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Thunderfish</ent><br/
<hw>Thun"der*fish`</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Zool.)</fld> <def>A large European loach (<spn>Misgurnus fossilis</spn>).</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Thunderhead</ent><br/
<hw>Thun"der*head`</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A rounded mass of cloud, with shining white edges; a cumulus, -- often appearing before a thunderstorm.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Thundering</ent><br/
<hw>Thun"der*ing</hw>, <pos>a.</pos> <sn>1.</sn> <def>Emitting thunder.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Roll the <qex>thundering</qex> chariot o'er the ground.</q> <rj><qau>J. Trumbull.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Very great; -- often adverbially.</def> <mark>[Slang]</mark><br/
[<source>1913 Webster</source>]</p>

<p>-- <wordforms><wf>Thun"der*ing*ly</wf>, <pos>adv.</pos></wordforms><br/
[<source>1913 Webster</source>]</p>

<p><ent>Thundering</ent><br/
<hw>Thun"der*ing</hw>, <pos>n.</pos> <def>Thunder.</def>  <rj><au>Rev. iv. 5.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Thunderless</ent><br/
<hw>Thun"der*less</hw>, <pos>a.</pos> <def>Without thunder or noise.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Thunderous</ent><br/
<hw>Thun"der*ous</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[Written also <ets>thundrous</ets>.]</ety> <sn>1.</sn> <def>Producing thunder.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>How he before the <qex>thunderous</qex> throne doth lie.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Making a noise like thunder; sounding loud and deep; sonorous.</def><br/
[<source>1913 Webster</source>]</p>

<p>-- <wordforms><wf>Thun"der*ous*ly</wf>, <pos>adv.</pos></wordforms><br/
[<source>1913 Webster</source>]</p>

<p><ent>Thunderproof</ent><br/
<hw>Thun"der*proof`</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Secure against the effects of thunder or lightning.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Thundershower</ent><br/
<hw>Thun"der*show`er</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A shower accompanied with lightning and thunder.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Thunderstone</ent><br/
<hw>Thun"der*stone`</hw> <pr>(?)</pr>, <pos>n.</pos> <sn>1.</sn> <def>A thunderbolt, -- formerly believed to be a stone.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Fear no more the lightning flash,<br/
Nor the all-dreaded <qex>thunderstone</qex>.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Paleon.)</fld> <def>A belemnite. See <er>Belemnite</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Thunderstorm</ent><br/
<hw>Thun"der*storm`</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A storm accompanied with lightning and thunder.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Thunderstrike</ent><br/
<hw>Thun"der*strike`</hw> <pr>(?)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp.</pos> <conjf>Thunderstruck</conjf> <pr>(?)</pr>; <pos>p. p.</pos> <conjf>Thunderstruck</conjf>, <conjf>-strucken</conjf> <pr>(<?/)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Thunderstriking</conjf>.]</vmorph> <sn>1.</sn> <def>To strike, blast, or injure by, or as by, lightning.</def> <mark>[R.]</mark>  <rj><au>Sir P. Sidney.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To astonish, or strike dumb, as with something terrible; -- rarely used except in the past participle.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>drove before him, <qex>thunderstruck</qex>.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Thunderworm</ent><br/
<hw>Thun"der*worm`</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Zool.)</fld> <def>A small, footless, burrowing, snakelike lizard (<spn>Rhineura Floridana</spn>) allied to Amphisbaena, native of Florida; -- so called because it leaves its burrows after a thundershower.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Thundery</ent>
<hw>Thun"der*y</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Accompanied with thunder; thunderous.</def> <mark>[R.]</mark> <ldquo/<xex>Thundery</xex> weather.<rdquo/  <rj><au>Pennant.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Thundrous</ent><br/
<hw>Thun"drous</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Thunderous; sonorous.</def> <ldquo/Scraps of <xex>thunderous</xex> epic.<rdquo/  <rj><au>Tennyson.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Thunny</ent><br/
<hw>Thun"ny</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Zool.)</fld> <def>The tunny.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Thurgh</ent><br/
<hw>Thurgh</hw> <pr>(?)</pr>, <pos>prep.</pos> <def>Through.</def> <mark>[Obs.]</mark>  <rj><au>Chaucer.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Thurghfare</ent><br/
<hw>Thurgh"fare`</hw> <pr>(?)</pr>, <pos>n.</pos> <def>Thoroughfare.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>This world is but a <qex>thurghfare</qex> full of woe.</q> <rj><qau>Chaucer.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Thurible</ent><br/
<hw>Thu"ri*ble</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>thuribulum</ets>, <ets>turibulum</ets>, from <ets>thus</ets>, <ets>thuris</ets>, or better <ets>tus</ets>, <ets>turis</ets>, frankincense, fr. Gr. <?/ a sacrifice, an offering, from <?/ to sacrifice.]</ety> <fld>(R. C. Ch.)</fld> <def>A censer of metal, for burning incense, having various forms, held in the hand or suspended by chains; -- used especially at mass, vespers, and other solemn services.</def>  <rj><au>Fairholt.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Thuriferous</ent><br/
<hw>Thu*rif"er*ous</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>thurifer</ets>, <ets>turifer</ets>; <ets>thus</ets> frankincense + <ets>-ferre</ets> to bear.]</ety> <def>Producing or bearing frankincense.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Thurification</ent><br/
<hw>Thu`ri*fi*ca"tion</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>thus</ets> incense + <ets>-ficare</ets> (in comp.) to make. See <er>-fy</er>.]</ety> <def>The act of fuming with incense, or the act of burning incense.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Thuringian</ent><br/
<hw>Thu*rin"gi*an</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Of or pertaining to Thuringia, a country in Germany, or its people.</def> -- <def2><pos>n.</pos>  <def>A native, or inhabitant of Thuringia.</def></def2><br/
[<source>1913 Webster</source>]</p>

<p><ent>Thuringite</ent><br/
<hw>Thu*rin"gite</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[From <etsep>Thuringia</etsep>, where it is found.]</ety> <fld>(Min.)</fld> <def>A mineral occurring as an aggregation of minute scales having an olive-green color and pearly luster. It is a hydrous silicate of aluminia and iron.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Thurl</ent><br/
<hw>Thurl</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[AS. <ets><thorn/yrel</ets> a hole. <root/53. See <er>Thirl</er>, <er>Thrill</er>.]</ety> <sn>1.</sn> <def>A hole; an aperture.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Mining)</fld> <sd>(a)</sd> <def>A short communication between adits in a mine.</def> <sd>(b)</sd> <def>A long adit in a coalpit.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Thurl</ent><br/
<hw>Thurl</hw>, <pos>v. t.</pos> <ety>[See <er>Thrill</er>.]</ety> <sn>1.</sn> <def>To cut through; to pierce.</def> <mark>[Obs.]</mark>  <rj><au>Piers Plowman.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Mining)</fld> <def>To cut through, as a partition between one working and another.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Thurling</ent><br/
<hw>Thurl"ing</hw>, <pos>n.</pos> <fld>(Mining)</fld> <def>Same as <er>Thurl</er>, <pos>n.</pos>, 2 <sd>(a)</sd>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Thurrok</ent><br/
<hw>Thur"rok</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[AS. <ets><thorn/urruc</ets> a boat.]</ety> <def>The hold of a ship; a sink.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>Small drops of water that enter through a little crevice into the <qex>thurrok</qex> and into the bottom of a ship.</q> <rj><qau>Chaucer.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Thursday</ent><br/
<hw>Thurs"day</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[OE. <ets><thorn/ursdei</ets>, <ets><thorn/orsday</ets>, from the Scand. name <ets>Thor</ets> + E. <ets>day</ets>. Icel. <ets><thorn/<omac/rr</ets> Thor, the god of thunder, is akin to AS. <ets><thorn/unor</ets> thunder; D. <ets>Donderdag</ets> Thursday, G. <ets>Donnerstag</ets>, Icel. <ets><thorn/<omac/rsdagr</ets>, Sw. & Dan. <ets>Torsdag</ets>. <root/52. See <er>Thor</er>, <er>Thunder</er>, and <er>Day</er>.]</ety> <def>The fifth day of the week, following Wednesday and preceding Friday.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Holy Thursday</b></col>. <cd>See under <er>Holy</er>.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Thurst</ent><br/
<hw>Thurst</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Coal Mining)</fld> <def>The ruins of the fallen roof resulting from the removal of the pillars and stalls.</def>  <rj><au>Raymond.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Thus</ent><br/
||<hw>Thus</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>thus</ets>, better <ets>tus</ets>, frankincense. See <er>Thurible</er>.]</ety> <def>The commoner kind of frankincense, or that obtained from the Norway spruce, the long-leaved pine, and other conifers.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Thus</ent><br/
<hw>Thus</hw> <pr>(<th/us)</pr>, <pos>adv.</pos> <ety>[OE. <ets>thus</ets>, AS. <ets><edh/us</ets>; akin to OFries. & OS. <ets>thus</ets>, D. <ets>dus</ets>, and E. <ets>that</ets>; cf. OHG. <ets>sus</ets>. See <er>That</er>.]</ety> <sn>1.</sn> <def>In this or that manner; on this wise.</def><br/
[<source>1913 Webster</source>]</p>

<p><q><qex>Thus</qex> did Noah; according to all that God commanded him, so did he.</q> <rj><qau>Gen. vi. 22.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q><qex>Thus</qex> God the heaven created, <qex>thus</qex> the earth.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To this degree or extent; so far; so; <as>as, <ex>thus</ex> wise; <ex>thus</ex> peaceble; <ex>thus</ex> bold</as>.</def>  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q><qex>Thus</qex> far extend, <qex>thus</qex> far thy bounds.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Thussock</ent><br/
<hw>Thus"sock</hw> <pr>(?)</pr>, <pos>n.</pos> <def>See <er>Tussock</er>.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Thuya</ent><br/
<hw>Thu"ya</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Bot.)</fld> <def>Same as <er>Thuja</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Thuyin</ent><br/
<hw>Thu"yin</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Chem.)</fld> <def>A substance extracted from trees of the genus <gen>Thuja</gen>, or <gen>Thuya</gen>, and probably identical with quercitrin.</def> <altsp>[Written also <asp>thujin</asp>.]</altsp><br/
[<source>1913 Webster</source>]</p>

<p><ent>Thwack</ent><br/
<hw>Thwack</hw> <pr>(thw<acr/k)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Thwacked</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Thwacking</conjf>.]</vmorph> <ety>[Cf. OE. <ets>thakken</ets> to stroke, AS. <ets><thorn/accian</ets>, E. <ets>whack</ets>.]</ety> <sn>1.</sn> <def>To strike with something flat or heavy; to bang, or thrash: to thump.</def> <ldquo/A distant <xex>thwacking</xex> sound.<rdquo/  <rj><au>W. Irving.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To fill to overflow.</def> <mark>[Obs.]</mark>  <rj><au>Stanyhurst.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Thwack</ent><br/
<hw>Thwack</hw>, <pos>n.</pos> <def>A heavy blow with something flat or heavy; a thump.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>With many a stiff <qex>thwack</qex>, many a bang,<br/
Hard crab tree and old iron rang.</q> <rj><qau>Hudibras.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Thwaite</ent><br/
<hw>Thwaite</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Zool.)</fld> <def>The twaite.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Thwaite</ent><br/
<hw>Thwaite</hw>, <pos>n.</pos> <ety>[CF. Icel. <ets><thorn/veit</ets> a piece of land, fr. <ets><thorn/v<imac/ta</ets> to cut. See <er>Thwite</er>, and cf. <er>Doit</er>, and <er>Twaite</er> land cleared of woods.]</ety> <def>Forest land cleared, and converted to tillage; an assart.</def> <mark>[Obs. or Prov. Eng.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ <xex>Thwaite</xex> occurs in composition as the last element in many names of places in the north of England; as, in Ros<xex>thwaite</xex>, Stone<xex>thwaite</xex>.</note><br/
[<source>1913 Webster</source>]</p>

<p><ent>Thwart</ent><br/
<hw>Thwart</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[OE. <ets><thorn/wart</ets>, <ets><thorn/wert</ets>, a. and adv., Icel. <ets><thorn/vert</ets>, neut. of <ets><thorn/verr</ets> athwart, transverse, across; akin to AS. <ets><thorn/weorh</ets> perverse, transverse, cross, D. <ets>dwars</ets>, OHG. <ets>dwerah</ets>, <ets>twerh</ets>, G. <ets>zwerch</ets>, <ets>quer</ets>, Dan. & Sw. <ets>tver</ets> athwart, transverse, Sw. <ets>tv<aum/r</ets> cross, unfriendly, Goth. <ets><thorn/wa<iacute/rhs</ets> angry.  Cf. <er>Queer</er>.]</ety> <sn>1.</sn> <def>Situated or placed across something else; transverse; oblique.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Moved contrary with <qex>thwart</qex> obliquities.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Fig.: Perverse; crossgrained.</def> <mark>[Obs.]</mark>  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Thwart</ent><br/
<hw>Thwart</hw>, <pos>adv.</pos> <ety>[See <er>Thwart</er>, <pos>a.</pos>]</ety> <def>Thwartly; obliquely; transversely; athwart.</def> <mark>[Obs.]</mark>  <rj><au>Milton.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Thwart</ent><br/
<hw>Thwart</hw>, <pos>prep.</pos> <def>Across; athwart.</def>  <rj><au>Spenser.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Thwart ships</b></col>. <cd>See <cref>Athwart ships</cref>, under <er>Athwart</er>.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Thwart</ent><br/
<hw>Thwart</hw>, <pos>n.</pos> <fld>(Naut.)</fld> <def>A seat in an open boat reaching from one side to the other, or athwart the boat.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Thwart</ent><br/
<hw>Thwart</hw>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Thwarted</conjf>; <pos>p. pr. & vb. n.</pos> <conjf>Thwarting</conjf>.]</vmorph> <sn>1.</sn> <def>To move across or counter to; to cross; <as>as, an arrow <ex>thwarts</ex> the air</as>.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>Swift as a shooting star<br/
In autumn <qex>thwarts</qex> the night.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To cross, as a purpose; to oppose; to run counter to; to contravene; hence, to frustrate or defeat.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>If crooked fortune had not <qex>thwarted</qex> me.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>The proposals of the one never <qex>thwarted</qex> the inclinations of the other.</q> <rj><qau>South.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Thwart</ent><br/
<hw>Thwart</hw>, <pos>v. i.</pos> <sn>1.</sn> <def>To move or go in an oblique or crosswise manner.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Hence, to be in opposition; to clash.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>Any proposition . . . that shall at all <qex>thwart</qex> with internal <qex>oracles</qex>.</q> <rj><qau>Locke.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Thwarter</ent><br/
<hw>Thwart"er</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Far.)</fld> <def>A disease in sheep, indicated by shaking, trembling, or convulsive motions.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Thwartingly</ent><br/
<hw>Thwart"ing*ly</hw>, <pos>adv.</pos> <def>In a thwarting or obstructing manner; so as to thwart.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Thwartly</ent><br/
<hw>Thwart"ly</hw>, <pos>adv.</pos> <def>Transversely; obliquely.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Thwartness</ent><br/
<hw>Thwart"ness</hw>, <pos>n.</pos> <def>The quality or state of being thwart; obliquity; perverseness.</def><br/
[<source>1913 Webster</source>]</p>

<p><-- p. 1506 --><br/
[<source>1913 Webster</source>]</p>

<p><ent>Thwite</ent><br/
<hw>Thwite</hw> <pr>(?)</pr>, <pos>v. t.</pos> <ety>[AS. <ets><thorn/w<imac/tan</ets>. See <er>Whittle</er>, and cf. <er>Thwaite</er> a piece of land.]</ety> <def>To cut or clip with a knife; to whittle.</def> <mark>[Obs. or Prov. Eng.]</mark>  <rj><au>Chaucer.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Thwittle</ent><br/
<hw>Thwit"tle</hw> <pr>(?)</pr>, <pos>v. t.</pos> <ety>[See <er>Thwite</er>, and <er>Whittle</er>.]</ety> <def>To cut or whittle.</def> <mark>[Obs. or Prov. Eng.]</mark>  <rj><au>Palsgrave.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Thwittle</ent><br/
<hw>Thwit"tle</hw>, <pos>n.</pos> <def>A small knife; a whittle.</def> <altsp>[Written also <asp>thwitel</asp>.]</altsp> <mark>[Obs.]</mark> <ldquo/A Sheffield <xex>thwittle</xex>.<rdquo/  <rj><au>Chaucer.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Thy</ent><br/
<hw>Thy</hw> <pr>(?)</pr>, <pos>pron.</pos> <ety>[OE. <ets>thi</ets>, shortened from <ets>thin</ets>. See <er>Thine</er>, <er>Thou</er>.]</ety> <def>Of thee, or belonging to thee; the more common form of <xex>thine</xex>, possessive case of <xex>thou</xex>; -- used always attributively, and chiefly in the solemn or grave style, and in poetry. <xex>Thine</xex> is used in the predicate; <as>as, the knife is <ex>thine</ex></as>. See <er>Thine</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Our father which art in heaven, Hallowed be <qex>thy</qex> name. <qex>Thy</qex> kingdom come. <qex>Thy</qex> will be done.</q> <rj><qau>Matt. vi. 9,10.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>These are <qex>thy</qex> glorious works, Parent of good.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Thyine wood</ent><br/
<hw>Thy"ine wood`</hw> <pr>(?)</pr>. <ety>[Gr. <?/ <?/, fr. <?/, adj., pertaining to the tree <?/ or <?/, an African tree with sweet-smelling wood.]</ety> <fld>(Bot.)</fld> <def>The fragrant and beautiful wood of a North African tree (<spn>Callitris quadrivalvis</spn>), formerly called <spn>Thuja articulata</spn>. The tree is of the Cedar family, and furnishes a balsamic resin called <xex>sandarach</xex>.</def>  <rj><au>Rev. xviii. 12.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Thylacine</ent><br/
<hw>Thy"la*cine</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Gr. <?/ a sack.]</ety> <fld>(Zool.)</fld> <def>The zebra wolf. See under <er>Wolf</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Thymate</ent><br/
<hw>Thym"ate</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Chem.)</fld> <def>A compound of thymol analogous to a salt; <as>as, sodium <ex>thymate</ex></as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Thyme</ent><br/
<hw>Thyme</hw> <pr>(t<imac/m)</pr>, <pos>n.</pos> <ety>[OE. <ets>tyme</ets>, L. <ets>thymum</ets>, Gr. <grk>qy`mon</grk>, <grk>qy`mos</grk>; cf. <grk>qy`ein</grk>, to sacrifice, <grk>qy`os</grk> a sacrifice, offering, incense: cf. F. <ets>thym</ets>; -- perhaps so named because of its sweet smell.  Cf. <er>Fume</er>, <pos>n.</pos>]</ety> <fld>(Bot.)</fld> <def>Any plant of the labiate genus <gen>Thymus</gen>. The garden thyme (<spn>Thymus vulgaris</spn>) is a warm, pungent aromatic, much used to give a relish to seasoning and soups.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Ankle deep in moss and flowery <qex>thyme</qex>.</q> <rj><qau>Cowper.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Cat thyme</b></col>, <cd>a labiate plant (<spn>Teucrium Marum</spn>) of the Mediterranean religion. Cats are said to be fond of rolling on it.</cd> <au>J. Smith (Dict. Econ. Plants).</au> -- <col><b>Wild thyme</b></col>, <cd><spn>Thymus Serpyllum</spn>, common on banks and hillsides in Europe.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><q>I know a bank where the <qex>wild thyme</qex> blows.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Thymene</ent><br/
<hw>Thym"ene</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Chem.)</fld> <def>A liquid terpene obtained from oil of thyme.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Thymiatechny</ent><br/
<hw>Thym"i*a*tech`ny</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Gr. <?/ incense + <grk>te`chnh</grk> art.]</ety> <fld>(Med.)</fld> <def>The art of employing perfumes in medicine.</def> <mark>[R.]</mark>  <rj><au>Dunglison.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Thymic</ent><br/
<hw>Thym"ic</hw> <pr>(?)</pr>, <pos>a.</pos> <fld>(Anat.)</fld> <def>Of or pertaining to the thymus gland.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Thymic</ent><br/
<hw>Thy"mic</hw> <pr>(?)</pr>, <pos>a.</pos> <fld>(Med. Chem.)</fld> <def>Pertaining to, or derived from, thyme; <as>as, <ex>thymic</ex> acid</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Thymol</ent><br/
<hw>Thym"ol</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[<ets>Thym</ets>e + <ets>-ol</ets>.]</ety> <fld>(Chem.)</fld> <def>A phenol derivative of cymene, <chform>C10H13.OH</chform>, isomeric with carvacrol, found in oil of thyme, and extracted as a white crystalline substance of a pleasant aromatic odor and strong antiseptic properties; -- called also <altname>hydroxy cymene</altname>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>thymus</ent><br/
||<hw>thy"mus</hw> <pr>(th<imac/"m<ucr/s)</pr>, <pos>a.</pos> <ety>[NL., fr. Gr. <grk>qy`mos</grk>.]</ety> <fld>(Anat.)</fld> <def>Of, pertaining to, or designating, the thymus gland.</def> -- <def2><pos>n.</pos> <def>The thymus gland.</def></def2><br/
[<source>1913 Webster</source>]</p>

<p><cs><mcol><col><b>Thymus gland</b></col>, <it>or</it> <col><b>Thymus body</b></col></mcol>, <cd>a ductless gland in the throat, or in the neighboring region, of nearly all vertebrates.  In man and other mammals it is the throat, or neck, sweetbread, which lies in the upper part of the thorax and lower part of the throat.  It is largest in fetal and early life, and disappears or becomes rudimentary in the adult.  The thymus gland functions as the site of maturation of T-lymphocytes (T-cells), which confer cell-mediated immunity on the host organism;  thus, removal or malfunciton of the thymus can lead to absence of cell-mediated immunity, and a consequent loss of resistance to infection.</cd></cs><br/
[<source>1913 Webster</source> <source>+PJC</source>]</p>

<p><ent>Thymy</ent><br/
<hw>Thym"y</hw> <pr>(t<imac/m"<ycr/)</pr>, <pos>a.</pos> <def>Abounding with thyme; fragrant; <as>as, a <ex>thymy</ex> vale</as>.</def>  <rj><au>Akenside.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Where'er a <qex>thymy</qex> bank he found,<br/
He rolled upon the fragrant ground.</q> <rj><qau>Gay.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><-- thyratron<br/
[<source>1913 Webster</source>]</p>

<p>    thyristor.  --><br/
[<source>1913 Webster</source>]</p>

<p><ent>Thyro-</ent><br/
<hw>Thy"ro-</hw> <pr>(?)</pr>. <def>A combining form used in anatomy to indicate <xex>connection with</xex>, or <xex>relation to</xex>, <xex>the thyroid body</xex> or <xex>the thyroid cartilage</xex>; <as>as, <ex>thyro</ex>hyal</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Thyroarytenoid</ent><br/
<hw>Thy`ro*a*ryt"e*noid</hw> <pr>(?)</pr>, <pos>a.</pos> <fld>(Anat.)</fld> <def>Of or pertaining to both the thyroid and arytenoid cartilages of the larynx.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Thyrohyal</ent><br/
<hw>Thy`ro*hy"al</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Anat.)</fld> <def>One of the lower segments in the hyoid arch, often consolidated with the body of the hyoid bone and forming one of its great horns, as in man.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Thyrohyoid</ent><br/
<hw>Thy`ro*hy"oid</hw> <pr>(?)</pr>, <pos>a.</pos> <fld>(Anat.)</fld> <def>Of or pertaining to the thyroid cartilage of the larynx and the hyoid arch.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Thyroid</ent><br/
<hw>Thy"roid</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[Gr. <grk>qyreoeidh`s</grk> shield-shaped; <grk>qyreo`s</grk> a large, oblong shield (from <grk>qy`ra</grk> a door) + <grk>e'i^dos</grk> form: cf. F. <ets>thyro<ium/de</ets>, <ets>thyr<eacute/o<ium/de</ets>.]</ety> <sn>1.</sn> <def>Shaped like an oblong shield; shield-shaped; <as>as, the <ex>thyroid</ex> cartilage</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Anat.)</fld> <def>Of or pertaining to the thyroid body, thyroid cartilage, or thyroid artery; thyroideal.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Thyroid cartilage</b></col>. <cd>See under <er>Larynx</er>.</cd> -- <mcol><col><b>Thyroid body</b></col>, <it>or</it> <col><b>Thyroid gland</b></col></mcol> <fld>(Anat.)</fld>, <cd>an endocrine gland having two lobes, located in the floor of the mouth or the region of the larynx, which secretes two hormones (<prod>thyroxine</prod> and <prod>triiodothyronine</prod>) that regulate the rates of growth and metabloism.  In man and most mammals it is a highly vascular organ, partly surrounding the base of the larynx and the upper part of the trachea.  Cases of hypothyroidism, where the gland is insufficiently active, can be treated by administration of thyroxine or a combination of thyroxine and triiodothyronine.</cd> -- <col><b>Thyroid dislocation</b></col> <fld>(Surg.)</fld>, <cd>dislocation of the thigh bone into the thyroid foramen.</cd> -- <col><b>Thyroid foramen</b></col>, <cd>the obturator foramen.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Thyroideal</ent><br/
<hw>Thy*roid"e*al</hw> <pr>(?)</pr>, <pos>a.</pos> <fld>(Anat.)</fld> <def>Thyroid.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Thyrotomy</ent><br/
<hw>Thy*rot"o*my</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[<ets>Thyro-</ets> + Gr. <?/ to cut.]</ety> <fld>(Surg.)</fld> <def>The operation of cutting into the thyroid cartilage.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Thyrse</ent><br/
<hw>Thyrse</hw> <pr>(th<etil/rs)</pr>, <pos>n.</pos> <ety>[Cf. F. <ets>thyrse</ets>.]</ety> <def>A thyrsus.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Thyrsoidal</ent><br/
<ent>Thyrsoid</ent><br/
<mhw>{ <hw>Thyr"soid</hw> <pr>(th<etil/r"soid)</pr>, <hw>Thyr*soid"al</hw> <pr>(th<etil/r*soid"<ait/l)</pr>, }</mhw> <pos>a.</pos> <ety>[Gr. <?/; <?/ thyrsus + <grk>e'i^dos</grk> form, shape: cf. F. <ets>thyrso<ium/de</ets>.]</ety> <def>Having somewhat the form of a thyrsus.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Thyrsus</ent><br/
||<hw>Thyr"sus</hw> <pr>(?)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Thyrsi</plw> <pr>(#)</pr>.</plu> <ety>[L., fr. Gr. <?/.  Cf. <er>Torso</er>.]</ety> <sn>1.</sn> <def>A staff entwined with ivy, and surmounted by a pine cone, or by a bunch of vine or ivy leaves with grapes or berries. It is an attribute of Bacchus, and of the satyrs and others engaging in Bacchic rites.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>A good to grow on graves<br/
As twist about a <qex>thyrsus</qex>.</q> <rj><qau>Mrs. Browning.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>In my hand I bear<br/
The <qex>thyrsus</qex>, tipped with fragrant cones of pine.</q> <rj><qau>Longfellow.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Bot.)</fld> <def>A species of inflorescence; a dense panicle, as in the lilac and horse-chestnut.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Thysanopter</ent><br/
||<hw>Thy`sa*nop"ter</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Zool.)</fld> <def>One of the Thysanoptera.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Thysanoptera</ent><br/
||<hw>Thy`sa*nop"te*ra</hw> <pr>(?)</pr>, <pos>n. pl.</pos> <ety>[NL., from Gr. <?/ a fringe + <?/ a wing.]</ety> <fld>(Zool.)</fld> <def>A division of insects, considered by some writers a distinct order, but regarded by others as belonging to the Hemiptera. They are all of small size, and have narrow, broadly fringed wings with rudimentary nervures. Most of the species feed upon the juices of plants, and some, as those which attack grain, are very injurious to crops. Called also <altname>Physopoda</altname>.  See <er>Thrips</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Thysanopteran</ent><br/
<hw>Thy`sa*nop"ter*an</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Zool.)</fld> <def>One of the Thysanoptera.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Thysanopterous</ent><br/
<hw>Thy`sa*nop"ter*ous</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Of or pertaining to the Thysanoptera.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Thysanura</ent><br/
||<hw>Thys`a*nu"ra</hw> <pr>(?)</pr>, <pos>n. pl.</pos> <ety>[NL., fr. Gr. <?/ fringe + <?/ tail.]</ety> <fld>(Zool.)</fld> <def>An order of wingless hexapod insects which have setiform caudal appendages, either bent beneath the body to form a spring, or projecting as bristles. It comprises the Cinura, or bristletails, and the Collembola, or springtails. Called also <altname>Thysanoura</altname>. See <er>Lepisma</er>, and <er>Podura</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Thysanuran</ent><br/
<hw>Thys`a*nu"ran</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Zool.)</fld> <def>One of the Thysanura. Also used adjectively.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Thysanurous</ent><br/
<hw>Thys`a*nu"rous</hw> <pr>(?)</pr>, <pos>a.</pos> <fld>(Zool.)</fld> <def>Of or pertaining to the Thysanura.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Thysbe</ent><br/
<hw>Thys"be</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[NL., fr. L. <ets>Thisbe</ets> maiden beloved by Pyramus, Gr. <?/.]</ety> <fld>(Zool.)</fld> <def>A common clearwing moth (<spn>Hemaris thysbe</spn>).</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Thyself</ent><br/
<hw>Thy*self"</hw> <pr>(?)</pr>, <pos>pron.</pos> <def>An emphasized form of the personal pronoun of the second person; -- used as a subject commonly with <xex>thou</xex>; <as>as, thou <ex>thyself</ex> shalt go</as>; that is, thou shalt go, and no other. It is sometimes used, especially in the predicate, without <xex>thou</xex>, and in the nominative as well as in the objective case.</def><br/
[<source>1913 Webster</source>]</p>

<p><q><qex>Thyself</qex> shalt see the act.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Ere I do thee, thou to <qex>thyself</qex> wast cruel.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tiar</ent><br/
<hw>Ti"ar</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Cf. F. <ets>tiare</ets>. See <er>Tiara</er>.]</ety> <def>A tiara.</def> <mark>[Poetic]</mark>  <rj><au>Milton. Tennyson.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tiara</ent><br/
<hw>Ti*a"ra</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L., from Gr. <?/, <?/; of Persian origin.]</ety> <sn>1.</sn> <def>A form of headdress worn by the ancient Persians. According to Xenophon, the royal tiara was encircled with a diadem, and was high and erect, while those of the people were flexible, or had rims turned over.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>The pope's triple crown. It was at first a round, high cap, but was afterward encompassed with a crown, subsequently with a second, and finally with a third. Fig.: The papal dignity.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tiaraed</ent><br/
<hw>Ti*a"raed</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Adorned with, or wearing, a tiara.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tib-cat</ent><br/
<hw>Tib"-cat`</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A female cat.</def> <mark>[Prov. Eng.]</mark>  <rj><au>Halliwell.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tibia</ent><br/
||<hw>Tib"i*a</hw> <pr>(?)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Tibiae</plw> <pr>(#)</pr>.</plu> <ety>[L.]</ety> <sn>1.</sn> <fld>(Anat.)</fld> <def>The inner, or preaxial, and usually the larger, of the two bones of the leg or hind limb below the knee.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Zool.)</fld> <def>The fourth joint of the leg of an insect. See <xex>Illust.</xex> under <er>Coleoptera</er>, and under <er>Hexapoda</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <fld>(Antiq.)</fld> <def>A musical instrument of the flute kind, originally made of the leg bone of an animal.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tibial</ent><br/
<hw>Tib"i*al</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>tibialis</ets>, fr. <ets>tibia</ets> the shin bone; also, a pipe or flute, originally made of a bone: cf. F. <ets>tibial</ets>.]</ety> <sn>1.</sn> <def>Of or pertaining to a tibia.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Of or pertaining to a pipe or flute.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Tibial spur</b></col> <fld>(Zool.)</fld>, <cd>a spine frequently borne on the tibia of insects. See <xex>Illust.</xex> under <er>Coleoptera</er>.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tibial</ent><br/
<hw>Tib"i*al</hw>, <pos>n.</pos> <fld>(Anat.)</fld> <def>A tibial bone; a tibiale.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tibiale</ent><br/
||<hw>Tib`i*a"le</hw> <pr>(?)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Tibialia</plw> <pr>(#)</pr>.</plu> <ety>[NL.]</ety> <fld>(Anat.)</fld> <def>The bone or cartilage of the tarsus which articulates with the tibia and corresponds to a part of the astragalus in man and most mammals.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tibicinate</ent><br/
<hw>Ti*bic"i*nate</hw> <pr>(?)</pr>, <pos>v. i.</pos> <ety>[L. <ets>tibicinare</ets>.]</ety> <def>To play on a tibia, or pipe.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tibio-</ent><br/
<hw>Tib"i*o-</hw> <pr>(?)</pr>. <def>A combining form used in anatomy to indicate <xex>connection with</xex>, or <xex>relation to</xex>, <xex>the tibia</xex>; <as>as, <ex>tibio</ex>tarsus, <ex>tibio</ex>fibular</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tibiotarsal</ent><br/
<hw>Tib`i*o*tar"sal</hw> <pr>(?)</pr>, <pos>a.</pos> <fld>(Anat.)</fld> <sd>(a)</sd> <def>Of or pertaining to both to the tibia and the tarsus; <as>as, the <ex>tibiotarsal</ex> articulation</as>.</def> <sd>(b)</sd> <def>Of or pertaining to the tibiotarsus.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tibiotarsus</ent><br/
<hw>Tib`i*o*tar"sus</hw> <pr>(?)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Tibiotarsi</plw> <pr>(<?/)</pr>.</plu> <fld>(Anat.)</fld> <def>The large bone between the femur and tarsometatarsus in the leg of a bird. It is formed by the union of the proximal part of the tarsus with the tibia.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tibrie</ent><br/
<hw>Tib"rie</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Zool.)</fld> <def>The pollack.</def> <mark>[Prov. Eng.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tic</ent><br/
<hw>Tic</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F.]</ety> <fld>(Med.)</fld> <def>A local and habitual convulsive motion of certain muscles; especially, such a motion of some of the muscles of the face; twitching; velication; -- called also <altname>spasmodic tic</altname>.</def>  <rj><au>Dunglison.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Tic douloureux</b></col> <pr>(<?/)</pr>. <ety>[F., fr. <ets>tic</ets> a knack, a twitching + <ets>douloureux</ets> painful.]</ety> <fld>(Med.)</fld> <cd>Neuralgia in the face; face ague. See under <er>Face</er>.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tical</ent><br/
||<hw>Ti*cal"</hw> <pr>(?)</pr>, <pos>n.</pos> <sn>1.</sn> <def>A bean-shaped coin of Siam, worth about sixty cents; also, a weight equal to 236 grains troy.</def>  <rj><au>Malcom.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A money of account in China, reckoning at about $1.60; also, a weight of about four ounces avoirdupois.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tice</ent><br/
<hw>Tice</hw> <pr>(?)</pr>, <pos>v. t.</pos> <ety>[Aphetic form of <ets>entice</ets>.]</ety> <def>To entice.</def> <mark>[Obs.]</mark>  <rj><au>The Coronation.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tice</ent><br/
<hw>Tice</hw>, <pos>n.</pos> <fld>(Cricket)</fld> <def>A ball bowled to strike the ground about a bat's length in front of the wicket.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Ticement</ent><br/
<hw>Tice"ment</hw> <pr>(?)</pr>, <pos>n.</pos> <def>Enticement.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tichorrhine</ent><br/
<hw>Tich"or*rhine</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Gr. <?/ a wall + <?/, <?/, the nose.]</ety> <fld>(Paleon.)</fld> <def>A fossil rhinoceros with a vertical bony medial septum supporting the nose; the hairy rhinoceros.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tick</ent><br/
<hw>Tick</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Abbrev. from <ets>ticket</ets>.]</ety> <def>Credit; trust; <as>as, to buy on, or upon, <ex>tick</ex></as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tick</ent><br/
<hw>Tick</hw>, <pos>v. i.</pos> <sn>1.</sn> <def>To go on trust, or credit.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To give tick; to trust.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tick</ent><br/
<hw>Tick</hw>, <pos>n.</pos> <ety>[OE. <ets>tike</ets>, <ets>teke</ets>; akin to D. <ets>teek</ets>, G. <ets>zecke</ets>.  Cf. <er>Tike</er> a tick.]</ety> <fld>(Zool.)</fld> <sd>(a)</sd> <def>Any one of numerous species of large parasitic mites which attach themselves to, and suck the blood of, cattle, dogs, and many other animals. When filled with blood they become ovate, much swollen, and usually livid red in color. Some of the species often attach themselves to the human body. The young are active and have at first but six legs.</def> <sd>(b)</sd> <def>Any one of several species of dipterous insects having a flattened and usually wingless body, as the bird ticks (see under <er>Bird</er>) and sheep tick (see under <er>Sheep</er>).</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Tick bean</b></col>, <cd>a small bean used for feeding horses and other animals.</cd> -- <col><b>Tick trefoil</b></col> <fld>(Bot.)</fld>, <cd>a name given to many plants of the leguminous genus <gen>Desmodium</gen>, which have trifoliate leaves, and joined pods roughened with minute hooked hairs by which the joints adhere to clothing and to the fleece of sheep.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tick</ent><br/
<hw>Tick</hw>, <pos>n.</pos> <ety>[LL. <ets>techa</ets>, <ets>teca</ets>, L. <ets>theca</ets> case, Gr. <?/, fr. <?/ to put. See <er>Thesis</er>.]</ety> <sn>1.</sn> <def>The cover, or case, of a bed, mattress, etc., which contains the straw, feathers, hair, or other filling.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Ticking. See <er>Ticking</er>, <pos>n.</pos></def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tick</ent><br/
<hw>Tick</hw>, <pos>v. i.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Ticked</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Ticking</conjf>.]</vmorph> <ety>[Probably of imitative origin; cf. D. <ets>tikken</ets>, LG. <ets>ticken</ets>.]</ety> <sn>1.</sn> <def>To make a small or repeating noise by beating or otherwise, as a watch does; to beat.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To strike gently; to pat.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Stand not <qex>ticking</qex> and toying at the branches.</q> <rj><qau>Latimer.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tick</ent><br/
<hw>Tick</hw>, <pos>n.</pos> <sn>1.</sn> <def>A quick, audible beat, as of a clock.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Any small mark intended to direct attention to something, or to serve as a check.</def>  <rj><au>Dickens.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <fld>(Zool.)</fld> <def>The whinchat; -- so called from its note.</def> <mark>[Prov. Eng.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Death tick</b></col>. <fld>(Zool.)</fld> <cd>See <er>Deathwatch</er>.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tick</ent><br/
<hw>Tick</hw>, <pos>v. t.</pos> <def>To check off by means of a tick or any small mark; to score.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>When I had got all my responsibilities down upon my list, I compared each with the bill and <qex>ticked</qex> it off.</q> <rj><qau>Dickens.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Ticken</ent><br/
<hw>Tick"en</hw> <pr>(?)</pr>, <pos>n.</pos> <def>See <er>Ticking</er>.</def> <mark>[R.]</mark>  <rj><au>R. Browning.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Ticker</ent><br/
<hw>Tick"er</hw> <pr>(t<icr/k"<etil/r)</pr>, <pos>n.</pos> <ety>[See <er>Tick</er>.]</ety> <sn>1.</sn> <def>One who, or that which, ticks, or produces a ticking sound, as a watch or clock, a telegraphic sounder, etc.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A telegraphic receiving instrument that automatically prints off stock quotations (<altname><col><b>stock ticker</b></col></altname>), market report, or other news on a paper ribbon or <ldquo/tape.<rdquo/</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><sn>3.</sn> <def>an electronic instrument receiving information by transmision from a remote source and displaying it in readable fashion, not necessarily on paper tape (e.g. on a video display terminal or moving ribbon of electronically controlled lights).</def><br/
[<source>PJC</source>]</p>

<p><sn>4.</sn> <def>The heart.</def> <mark>[Colloq.]</mark><br/
[<source>PJC</source>]</p>

<p><col><b>Ticker tape</b></col> <cd>Tape from or designed to be used in a stock ticker, usu. of paper and being narrow but long.</cd> -- <col><b>Stock ticker</b></col>, <cd>an electro-mechanical information receiving device connected by telegraphic wire to a stock exchange, and which prints out the latest transactions or news on stock exchanges, commonly found in the offices of stock brokers.  By  1980 such devices were largely superseded by electronic stock quotation devices.</cd><br/
[<source>1913 Webster</source> <source>+PJC</source>]</p>

<p><ent>ticker tape parade</ent><br/
<hw>ticker tape parade</hw> <def>A parade to honor a person or persons, held in New York City, during which people in the tall buildings of Manhattan throw large quantities of paper, confetti, paper ribbons, or the like onto the parading group.  The name comes form the ticker tape originally thrown onto the parade when it passed stockbrokers' offices in lower Manhattan, before stock tickers became obsolete; in subsequent years other types of waste paper were used to serve the honorary function, as well as paper tape distributed specifically for the purpose of being thrown in such a parade.</def><br/
[<source>PJC</source>]</p>

<p><ent>Ticket</ent><br/
<hw>Tick"et</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F. <ets><eacute/tiquette</ets> a label, ticket, fr. OF. <ets>estiquette</ets>, or OF. <ets>etiquet</ets>, <ets>estiquet</ets>; both of Teutonic origin, and akin to E. <ets>stick</ets>. See <er>Stick</er>, <pos>n.</pos> & <pos>v.</pos>, and cf. <er>Etiquette</er>, <er>Tick</er> credit.]</ety> <def>A small piece of paper, cardboard, or the like, serving as a notice, certificate, or distinguishing token of something.</def> Specifically: --<br/
[<source>1913 Webster</source>]</p>

<p><sd>(a)</sd> <def>A little note or notice.</def> <mark>[Obs. or Local]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>He constantly read his lectures twice a week for above forty years, giving notice of the time to his auditors in a <qex>ticket</qex> on the school doors.</q> <rj><qau>Fuller.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sd>(b)</sd> <def>A tradesman's bill or account.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ Hence the phrase <xex>on ticket</xex>, on account; whence, by abbreviation, came the phrase <xex>on tick</xex>. See 1st <er>Tick</er>.</note><br/
[<source>1913 Webster</source>]</p>

<p><q>Your courtier is mad to take up silks and velvets<br/
<qex>On ticket</qex> for his mistress.</q> <rj><qau>J. Cotgrave.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sd>(c)</sd> <def>A certificate or token of right of admission to a place of assembly, or of passage in a public conveyance; <as>as, a theater <ex>ticket</ex>; a railroad or steamboat <ex>ticket</ex></as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sd>(d)</sd> <def>A label to show the character or price of goods.</def><br/
[<source>1913 Webster</source>]</p>

<p><sd>(e)</sd> <def>A certificate or token of a share in a lottery or other scheme for distributing money, goods, or the like.</def><br/
[<source>1913 Webster</source>]</p>

<p><sd>(f)</sd> <fld>(Politics)</fld> <def>A printed list of candidates to be voted for at an election; a set of nominations by one party for election; a ballot.</def> <mark>[U. S.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>The old <qex>ticket</qex> forever! We have it by thirty-four votes.</q> <rj><qau>Sarah Franklin (1766).</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Scratched ticket</b></col>, <cd>a ticket from which the names of one or more of the candidates are scratched out.</cd> -- <col><b>Split ticket</b></col>, <cd>a ticket representing different divisions of a party, or containing candidates selected from two or more parties.</cd> -- <col><b>Straight ticket</b></col>, <cd>a ticket containing the regular nominations of a party, without change.</cd> -- <col><b>Ticket day</b></col> <fld>(Com.)</fld>, <cd>the day before the settling or pay day on the stock exchange, when the names of the actual purchasers are rendered in by one stockbroker to another.</cd> <mark>[Eng.]</mark> <au>Simmonds.</au> -- <col><b>Ticket of leave</b></col>, <cd>a license or permit given to a convict, or prisoner of the crown, to go at large, and to labor for himself before the expiration of his sentence, subject to certain specific conditions.</cd> <mark>[Eng.]</mark> <au>Simmonds.</au> -- <col><b>Ticket porter</b></col>, <cd>a licensed porter wearing a badge by which he may be identified.</cd> <mark>[Eng.]</mark></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Ticket</ent><br/
<hw>Tick"et</hw>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Ticketed</conjf>; <pos>p. pr. & vb. n.</pos> <conjf>Ticketing</conjf>.]</vmorph> <sn>1.</sn> <def>To distinguish by a ticket; to put a ticket on; <as>as, to <ex>ticket</ex> goods</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To furnish with a tickets; to book; <as>as, to <ex>ticket</ex> passengers to California</as>.</def> <mark>[U. S.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><-- Ticketed.  having a ticket, esp. a ticket for travel on a carrier sucha as an airline.  A term used to distinguish those who have made a reservation for travel, but have not yet paid and received their ticket, from those who have.  "You have a reservation, but you have not yet been ticketed." --><br/
[<source>1913 Webster</source>]</p>

<p><ent>Ticketing</ent><br/
<hw>Tick"et*ing</hw>, <pos>n.</pos> <def>A periodical sale of ore in the English mining districts; -- so called from the tickets upon which are written the bids of the buyers.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Ticking</ent><br/
<hw>Tick"ing</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[From <er>Tick</er> a bed cover.  Cf. <er>Ticken</er>.]</ety> <def>A strong, closely woven linen or cotton fabric, of which ticks for beds are made. It is usually twilled, and woven in stripes of different colors, as white and blue; -- called also <altname>ticken</altname>.</def><br/
[<source>1913 Webster</source>]</p>

<p><-- p. 1507 --><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tickle</ent><br/
<hw>Tic"kle</hw> <pr>(?)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Tickled</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Tickling</conjf> <pr>(?)</pr>.]</vmorph> <ety>[Perhaps freq. of <ets>tick</ets> to beat; pat; but cf. also AS. <ets>citelian</ets> to tickle, D. <ets>kittelen</ets>, G. <ets>kitzlen</ets>, OHG. <ets>chizzil<omac/n</ets>, <ets>chuzzil<omac/n</ets>, Icel. <ets>kitla</ets>.  Cf. <er>Kittle</er>, <pos>v. t.</pos>]</ety> <sn>1.</sn> <def>To touch lightly, so as to produce a peculiar thrilling sensation, which commonly causes laughter, and a kind of spasm which become dangerous if too long protracted.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>If you <qex>tickle</qex> us, do we not laugh?</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To please; to gratify; to make joyous.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Pleased with a rattle, <qex>tickled</qex> with a straw.</q> <rj><qau>Pope.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Such a nature<br/
<qex>Tickled</qex> with good success, disdains the shadow<br/
Which he treads on at noon.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tickle</ent><br/
<hw>Tic"kle</hw>, <pos>v. i.</pos> <sn>1.</sn> <def>To feel titillation.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>He with secret joy therefore<br/
Did <qex>tickle</qex> inwardly in every vein.</q> <rj><qau>Spenser.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To excite the sensation of titillation.</def>  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tickle</ent><br/
<hw>Tic"kle</hw>, <pos>a.</pos> <sn>1.</sn> <def>Ticklish; easily tickled.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Liable to change; uncertain; inconstant.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>The world is now full <qex>tickle</qex>, sikerly.</q> <rj><qau>Chaucer.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>So <qex>tickle</qex> is the state of earthy things.</q> <rj><qau>Spenser.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>Wavering, or liable to waver and fall at the slightest touch; unstable; easily overthrown.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>Thy head stands so <qex>tickle</qex> on thy shoulders, that a milkmaid, if she be in love, may sigh it off.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tickle-footed</ent><br/
<hw>Tic"kle-foot`ed</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Uncertain; inconstant; slippery.</def> <mark>[Obs. & R.]</mark>  <rj><au>Beau. & Fl.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Ticklenburg</ent><br/
<hw>Tick"len*burg</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A coarse, mixed linen fabric made to be sold in the West Indies.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tickleness</ent><br/
<hw>Tic"kle*ness</hw> <pr>(?)</pr>, <pos>n.</pos> <def>Unsteadiness.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>For hoard hath hate, and climbing <qex>tickleness</qex>.</q> <rj><qau>Chaucer.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tickler</ent><br/
<hw>Tic"kler</hw> <pr>(?)</pr>, <pos>n.</pos> <sn>1.</sn> <def>One who, or that which, tickles.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Something puzzling or difficult.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>A book containing a memorandum of notes and debts arranged in the order of their maturity.</def> <mark>[Com. Cant, U. S.]</mark>  <rj><au>Bartlett.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>A prong used by coopers to extract bungs from casks.</def> <mark>[Eng.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Ticklish</ent><br/
<hw>Tic"klish</hw> <pr>(?)</pr>, <pos>a.</pos> <sn>1.</sn> <def>Sensible to slight touches; easily tickled; <as>as, the sole of the foot is very <ex>ticklish</ex>; the hardened palm of the hand is not <ex>ticklish</ex></as>.</def>  <rj><au>Bacon.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Standing so as to be liable to totter and fall at the slightest touch; unfixed; easily affected; unstable.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Can any man with comfort lodge in a condition so dismally <qex>ticklish</qex>?</q> <rj><qau>Barrow.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>Difficult; nice; critical; <as>as, a <ex>ticklish</ex> business</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Surely princes had need, in tender matters and <qex>ticklish</qex> times, to beware what they say.</q> <rj><qau>Bacon.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p>-- <wordforms><wf>Tic"klish*ly</wf>, <pos>adv.</pos> -- <wf>Tic"klish*ness</wf>, <pos>n.</pos></wordforms><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tickseed</ent><br/
<hw>Tick"seed`</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[<ets>Tick</ets> the insect + <ets>seed</ets>; cf. G. <ets>wanzensamen</ets>, literally, bug seed.]</ety> <sn>1.</sn> <def>A seed or fruit resembling in shape an insect, as that of certain plants.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Bot.)</fld> <sd>(a)</sd> <def>Same as <er>Coreopsis</er>.</def> <sd>(b)</sd> <def>Any plant of the genus <gen>Corispermum</gen>, plants of the Goosefoot family.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Ticktack</ent><br/
<hw>Tick"tack`</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[See <er>Tick</er> to beat, to pat, and (for sense 2) cf. <er>Tricktrack</er>.]</ety> <sn>1.</sn> <def>A noise like that made by a clock or a watch.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A kind of backgammon played both with men and pegs; tricktrack.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>A game at <qex>ticktack</qex> with words.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Ticktack</ent><br/
<hw>Tick"tack`</hw>, <pos>adv.</pos> <def>With a ticking noise, like that of a watch.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Ticpolonga</ent><br/
<hw>Tic`po*lon"ga</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Native name.]</ety> <fld>(Zool.)</fld> <def>A very venomous viper (<spn>Daboia Russellii</spn>), native of Ceylon and India; -- called also <altname>cobra monil</altname>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tid</ent><br/
<hw>Tid</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[Cf. AS. <ets>tedre</ets>, <ets>tydere</ets>, weak, tender.]</ety> <def>Tender; soft; nice; -- now only used in <xex>tid</xex>bit.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tidal</ent><br/
<hw>Tid"al</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Of or pertaining to tides; caused by tides; having tides; periodically rising and falling, or following and ebbing; <as>as, <ex>tidal</ex> waters</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The <qex>tidal</qex> wave of deeper souls<br/
Into our inmost being rolls,<br/
And lifts us unawares<br/
Out of all meaner cares.</q> <rj><qau>Longfellow.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Tidal air</b></col> <fld>(Physiol.)</fld>, <cd>the air which passes in and out of the lungs in ordinary breathing. It varies from twenty to thirty cubic inches.</cd> -- <col><b>Tidal basin</b></col>, <cd>a dock that is filled at the rising of the tide.</cd> -- <col><b>Tidal wave</b></col>. <sd>(a)</sd> <cd>See <cref>Tide wave</cref>, under <er>Tide</er>.  Cf. 4th <er>Bore</er>.</cd> <sd>(b)</sd> <cd>A vast, swift wave caused by an earthquake or some extraordinary combination of natural causes. It rises far above high-water mark and is often very destructive upon low-lying coasts.</cd></cs>
<-- called in Japan tsunami. --><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tidal wave</ent><br/
<hw>Tid"al wave</hw> <pr>(?)</pr>, <pos>n.</pos> <sn>1.</sn> <def>an unusually high wave from the sea, sometimes reaching far inland and causing great destruction, and usually caused by some event, such as an earthquake, far from the shore.  In Japan, such a wave is called a <altname>tsunami</altname>.</def><br/
[<source>PJC</source>]</p>

<p><sn>2.</sn> <mark>[fig.]</mark> <def>an unusually large quantity of items or events requiring attention and causing strain on the capacity to handle them; <as>as, a <ex>tidal wave</ex> of orders for a new product; a <ex>tidal wave</ex> of tourists</as>.</def><br/
[<source>PJC</source>]</p>

<p><ent>Tidbit</ent><br/
<hw>Tid"bit`</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[<ets>Tid</ets> + <ets>bit</ets>.]</ety> <def>A delicate or tender piece of anything eatable; a delicious morsel.</def> <altsp>[Written also <asp>titbit</asp>.]</altsp><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tidde</ent><br/
<hw>Tid"de</hw> <pr>(?)</pr>, <mark>obs.</mark> <def><pos>imp.</pos> of <er>Tide</er>, <pos>v. i.</pos></def>  <rj><au>Chaucer.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tiddle</ent><br/
<ent>Tidder</ent><br/
<mhw>{ <hw>Tid"der</hw> <pr>(?)</pr>, <hw>Tid"dle</hw> <pr>(?)</pr>, }</mhw> <pos>v. t.</pos> <ety>[Cf. AS. <ets>tyderian</ets> to grow tender. See <er>Tid</er>.]</ety> <def>To use with tenderness; to fondle.</def> <mark>[Obs. or Prov. Eng.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tiddledywinks</ent><br/
<hw>Tid"dle*dy*winks`</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A game in which the object is to snap small disks of plastic, bone, ivory, or the like, from a flat surface, as of a table, into a small cup or basket; -- called also <altname>tiddlywinks</altname>.</def> <mark>[U. S.]</mark><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Tiddlywinks</ent><br/
<hw>Tid"dly*winks`</hw> <pr>(?)</pr>, <pos>n.</pos> <def>Same as <er>Tiddledywinks</er>.</def>  <rj><au>Kipling.</au></rj><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Tide</ent><br/
<hw>Tide</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[AS. <ets>t<imac/d</ets> time; akin to OS. & OFries. <ets>t<imac/d</ets>, D. <ets>tijd</ets>, G. <ets>zeit</ets>, OHG. <ets>z<imac/t</ets>, Icel. <ets>t<imac/<?/</ets>, Sw. & Dan. <ets>tid</ets>, and probably to Skr. <ets>aditi</ets> unlimited, endless, where <ets>a-</ets> is a negative prefix. <root/58.  Cf. <er>Tidings</er>, <er>Tidy</er>, <er>Till</er>, <pos>prep.</pos>, <er>Time</er>.]</ety> <sn>1.</sn> <def>Time; period; season.</def> <mark>[Obsoles.]</mark> <ldquo/This lusty summer's <xex>tide</xex>.<rdquo/  <rj><au>Chaucer.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>And rest their weary limbs a <qex>tide</qex>.</q> <rj><qau>Spenser.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Which, at the appointed <qex>tide</qex>,<br/
Each one did make his bride.</q> <rj><qau>Spenser.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>At the <qex>tide</qex> of Christ his birth.</q> <rj><qau>Fuller.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>The alternate rising and falling of the waters of the ocean, and of bays, rivers, etc., connected therewith. The tide ebbs and flows twice in each lunar day, or the space of a little more than twenty-four hours. It is occasioned by the attraction of the sun and moon (the influence of the latter being three times that of the former), acting unequally on the waters in different parts of the earth, thus disturbing their equilibrium. A high tide upon one side of the earth is accompanied by a high tide upon the opposite side. Hence, when the sun and moon are in conjunction or opposition, as at new moon and full moon, their action is such as to produce a greater than the usual tide, called the <stype>spring tide</stype>, as represented in the cut. When the moon is in the first or third quarter, the sun's attraction in part counteracts the effect of the moon's attraction, thus producing under the moon a smaller tide than usual, called the <stype>neap tide</stype>.</def><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ The flow or rising of the water is called <xex>flood tide</xex>, and the reflux, <xex>ebb tide</xex>.</note><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>A stream; current; flood; <as>as, a <ex>tide</ex> of blood</as>.</def> <ldquo/Let in the <xex>tide</xex> of knaves once more; my cook and I'll provide.<rdquo/  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>Tendency or direction of causes, influences, or events; course; current.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>There is a <qex>tide</qex> in the affairs of men,<br/
Which, taken at the flood, leads on to fortune.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <def>Violent confluence.</def> <mark>[Obs.]</mark>  <rj><au>Bacon.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>6.</sn> <fld>(Mining)</fld> <def>The period of twelve hours.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Atmospheric tides</b></col>, <cd>tidal movements of the atmosphere similar to those of the ocean, and produced in the same manner by the attractive forces of the sun and moon.</cd> -- <col><b>Inferior tide</b></col>. <cd>See under <er>Inferior</er>, <pos>a.</pos></cd> -- <col><b>To work double tides</b></col>. See under <er>Work</er>, <pos>v. t.</pos> -- <col><b>Tide day</b></col>, <cd>the interval between the occurrences of two consecutive maxima of the resultant wave at the same place. Its length varies as the components of sun and moon waves approach to, or recede from, one another. A retardation from this cause is called the <xex>lagging</xex> of the tide, while the acceleration of the recurrence of high water is termed the <xex>priming</xex> of the tide. See <cref>Lag of the tide</cref>, under 2d <er>Lag</er>.</cd> -- <col><b>Tide dial</b></col>, <cd>a dial to exhibit the state of the tides at any time.</cd> -- <col><b>Tide gate</b></col>. <sd>(a)</sd> <cd>An opening through which water may flow freely when the tide sets in one direction, but which closes automatically and prevents the water from flowing in the other direction.</cd> <sd>(b)</sd> <fld>(Naut.)</fld> <cd>A place where the tide runs with great velocity, as through a gate.</cd> -- <col><b>Tide gauge</b></col>, <cd>a gauge for showing the height of the tide; especially, a contrivance for registering the state of the tide continuously at every instant of time.</cd> <au>Brande & C.</au> -- <col><b>Tide lock</b></col>, <cd>a lock situated between an inclosed basin, or a canal, and the tide water of a harbor or river, when they are on different levels, so that craft can pass either way at all times of the tide; -- called also <altname>guard lock</altname>.</cd> -- <col><b>Tide mill</b></col>. <fld>(a)</fld> <cd>A mill operated by the tidal currents.</cd> <sd>(b)</sd> <cd>A mill for clearing lands from tide water.</cd> -- <col><b>Tide rip</b></col>, <cd>a body of water made rough by the conflict of opposing tides or currents.</cd> -- <col><b>Tide table</b></col>, <cd>a table giving the time of the rise and fall of the tide at any place.</cd> -- <col><b>Tide water</b></col>, <cd>water affected by the flow of the tide; hence, broadly, the seaboard.</cd> -- <mcol><col><b>Tide wave</b></col>, <it>or</it>  <col><b>Tidal wave</b></col></mcol>, <cd>the swell of water as the tide moves. That of the ocean is called <xex>primitive</xex>; that of bays or channels <xex>derivative</xex>.  See also <er>tidal wave</er> in the vocabulary.</cd> <au>Whewell.</au> -- <col><b>Tide wheel</b></col>, <cd>a water wheel so constructed as to be moved by the ebb or flow of the tide.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tide</ent><br/
<hw>Tide</hw> <pr>(t<imac/d)</pr>, <pos>v. t.</pos> <def>To cause to float with the tide; to drive or carry with the tide or stream.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>They are <qex>tided</qex> down the stream.</q> <rj><qau>Feltham.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tide</ent><br/
<hw>Tide</hw>, <pos>v. i.</pos> <ety>[AS. <ets>t<imac/dan</ets> to happen. See <er>Tide</er>, <pos>n.</pos>]</ety> <sn>1.</sn> <def>To betide; to happen.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>What should us <qex>tide</qex> of this new law?</q> <rj><qau>Chaucer.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To pour a tide or flood.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <fld>(Naut.)</fld> <def>To work into or out of a river or harbor by drifting with the tide and anchoring when it becomes adverse.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tided</ent><br/
<hw>Tid"ed</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Affected by the tide; having a tide.</def> <ldquo/The <xex>tided</xex> Thames.<rdquo/  <rj><au>Bp. Hall.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tideland</ent><br/
<hw>Tide"land</hw> <pr>(?)</pr>, <pos>n.</pos> <def>Land that is overflowed by tide water; hence, land near the sea.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Tideless</ent><br/
<hw>Tide"less</hw>, <pos>a.</pos> <def>Having no tide.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tidemark</ent><br/
<hw>Tide"mark</hw>, <pos>n.</pos> <sn>1.</sn> <def>a mark on a shore line indicating the normal level of the water at high tide, caused by the action of the flowing water; sometimes also used for the mark left at the point of the normal low tide; -- called also <altname>high water mark</altname>.</def><br/
[<source>PJC</source>]</p>

<p><sn>2.</sn> <mark>[fig.]</mark> <def>the maximum level which some activity has reached and from which it has declined; -- called also <altname>high water mark</altname>.</def><br/
[<source>PJC</source>]</p>

<p><ent>Tide-rode</ent><br/
<hw>Tide"-rode`</hw> <pr>(?)</pr>, <pos>a.</pos> <fld>(Naut.)</fld> <def>Swung by the tide when at anchor; -- opposed to <xex>wind-rode</xex>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tidesman</ent><br/
<hw>Tides"man</hw> <pr>(?)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Tidesmen</plw> <pr>(<?/)</pr>.</plu> <def>A customhouse officer who goes on board of a merchant ship to secure payment of the duties; a tidewaiter.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tidewaiter</ent><br/
<hw>Tide"wait`er</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A customhouse officer who watches the landing of goods from merchant vessels, in order to secure payment of duties.</def>  <rj><au>Swift.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tideway</ent><br/
<hw>Tide"way`</hw> <pr>(?)</pr>, <pos>n.</pos> <def>Channel in which the tide sets.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tidife</ent><br/
<hw>Tid"ife</hw> <pr>(?)</pr>, <pos>n.</pos> <def>The blue titmouse.</def> <mark>[Prov. Eng.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ The <ldquo/tidif<rdquo/ mentioned in Chaucer is by some supposed to be the titmouse, by others the wren.</note><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tidily</ent><br/
<hw>Ti"di*ly</hw> <pr>(?)</pr>, <pos>adv.</pos> <def>In a tidy manner.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tidiness</ent><br/
<hw>Ti"di*ness</hw>, <pos>n.</pos> <def>The quality or state of being tidy.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tiding</ent><br/
<hw>Ti"ding</hw> <pr>(?)</pr>, <pos>n.</pos> <def>Tidings.</def> <mark>[Obs.]</mark>  <rj><au>Chaucer.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tidings</ent><br/
<hw>Ti"dings</hw> <pr>(?)</pr>, <pos>n. pl.</pos> <ety>[OE. <ets>tidinge</ets>, <ets>ti<?/inge</ets>, <ets>tidinde</ets>, from or influenced by Icel. <ets>t<imac/<?/indi</ets>; akin to Dan. <ets>tidende</ets>, Sw. <ets>tidning</ets>, G. <ets>zeung</ets>, AS. <ets>t<imac/dan</ets> to happen, E. be<ets>tide</ets>, <ets>tide</ets>. See <er>Tide</er>, <pos>v. i.</pos> & <pos>n.</pos>]</ety> <def>Account of what has taken place, and was not before known; news.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>I shall make my master glad with these <qex>tidings</qex>.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Full well the busy whisper, circling round,<br/
Conveyed the dismal <qex>tidings</qex> when he frowned.</q> <rj><qau>Goldsmith.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ Although <xex>tidings</xex> is plural in form, it has been used also as a singular. By Shakespeare it was used indiscriminately as a singular or plural.</note><br/
[<source>1913 Webster</source>]</p>

<p><q>Now near the <qex>tidings</qex> of our comfort is.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q><qex>Tidings</qex> to the contrary<br/
Are brought your eyes.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><syn><b>Syn.</b> -- News; advice; information; intelligence.</syn> <usage> -- <er>Tidings</er>, <er>News</er>. The term <xex>news</xex> denotes recent intelligence from any quarter; the term <xex>tidings</xex> denotes intelligence expected from a particular quarter, showing what has there <xex>betided</xex>. We may be indifferent as to <xex>news</xex>, but are always more or less interested in <xex>tidings</xex>. We read the <xex>news</xex> daily; we wait for <xex>tidings</xex> respecting an absent friend or an impending battle. We may be curious to hear the <xex>news</xex>; we are always anxious for <xex>tidings</xex>.</usage><br/
[<source>1913 Webster</source>]</p>

<p><q>Evil <qex>news</qex> rides post, while good <qex>news</qex> baits.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>What <qex>tidings</qex> dost thou bring?</q> <rj><qau>Addison.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tidley</ent><br/
<hw>Tid"ley</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Zool.)</fld> <sd>(a)</sd> <def>The wren.</def> <sd>(b)</sd> <def>The goldcrest.</def> <mark>[Prov. Eng.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tidology</ent><br/
<hw>Tid*ol"o*gy</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[<ets>Tide</ets> + <ets>-logy</ets>.]</ety> <def>A discourse or treatise upon the tides; that part of science which treats of tides.</def>  <rj><au>J. S. Mill.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tidy</ent><br/
<hw>Ti"dy</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Zool.)</fld> <def>The wren; -- called also <altname>tiddy</altname>.</def> <mark>[Prov. Eng.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>The <qex>tidy</qex> for her notes as delicate as they.</q> <rj><qau>Drayton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ This name is probably applied also to other small singing birds, as the goldcrest.</note><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tidy</ent><br/
<hw>Ti"dy</hw>, <pos>a.</pos> <amorph>[<pos>Compar.</pos> <adjf>Tidier</adjf> <pr>(?)</pr>; <pos>superl.</pos> <adjf>Tidiest</adjf>.]</amorph> <ety>[From <er>Tide</er> time, season; cf. D. <ets>tijdig</ets> timely, G. <ets>zeitig</ets>, Dan. & Sw. <ets>tidig</ets>.]</ety> <sn>1.</sn> <def>Being in proper time; timely; seasonable; favorable; <as>as, <ex>tidy</ex> weather</as>.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>If weather be fair and <qex>tidy</qex>.</q> <rj><qau>Tusser.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Arranged in good order; orderly; appropriate; neat; kept in proper and becoming neatness, or habitually keeping things so; <as>as, a <ex>tidy</ex> lass; their dress is <ex>tidy</ex>; the apartments are well furnished and <ex>tidy</ex></as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>A <qex>tidy</qex> man, that tened [injured] me never.</q> <rj><qau>Piers Plowman.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tidy</ent><br/
<hw>Ti"dy</hw>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Tidies</plw> <pr>(<?/)</pr>.</plu> <sn>1.</sn> <def>A cover, often of tatting, drawn work, or other ornamental work, for the back of a chair, the arms of a sofa, or the like.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A child's pinafore.</def> <mark>[Prov. Eng.]</mark>  <rj><au>Wright.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tidy</ent><br/
<hw>Ti"dy</hw>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Tidied</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Tidying</conjf>.]</vmorph> <def>To put in proper order; to make neat; <as>as, to <ex>tidy</ex> a room; to <ex>tidy</ex> one's dress</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tidy</ent><br/
<hw>Ti"dy</hw>, <pos>v. i.</pos> <def>To make things tidy.</def> <mark>[Colloq.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>I have <qex>tidied</qex> and <qex>tidied</qex> over and over again.</q> <rj><qau>Dickens.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tidytips</ent><br/
<hw>Ti"dy*tips`</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Bot.)</fld> <def>A California composite plant (<spn>Layia platyglossa</spn>), the flower of which has yellow rays tipped with white.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tie</ent><br/
<hw>Tie</hw> <pr>(?)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Ties</plw> <pr>(#)</pr>.</plu> <ety>[AS. <ets>t<emac/ge</ets>, <ets>t<?/ge</ets>, <ets>t<imac/ge</ets>. <root/64. See <er>Tie</er>, <pos>v. t.</pos>]</ety> <sn>1.</sn> <def>A knot; a fastening.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A bond; an obligation, moral or legal; <as>as, the sacred <ex>ties</ex> of friendship or of duty; the <ex>ties</ex> of allegiance</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>No distance breaks the <qex>tie</qex> of blood.</q> <rj><qau>Young.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>A knot of hair, as at the back of a wig.</def>  <rj><au>Young.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>An equality in numbers, as of votes, scores, etc., which prevents either party from being victorious; equality in any contest, as a race.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <fld>(Arch. & Engin.)</fld> <def>A beam or rod for holding two parts together; in railways, one of the transverse timbers which support the track and keep it in place.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>6.</sn> <fld>(Mus.)</fld> <def>A line, usually straight, drawn across the stems of notes, or a curved line written over or under the notes, signifying that they are to be slurred, or closely united in the performance, or that two notes of the same pitch are to be sounded as one; a bind; a ligature.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>7.</sn> <pluf>pl.</pluf> <def>Low shoes fastened with lacings.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Bale tie</b></col>, <cd>a fastening for the ends of a hoop for a bale.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tie</ent><br/
<hw>Tie</hw>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Tied</conjf> <pr>(?)</pr> (<mark>Obs</mark>. <conjf>Tight</conjf> <pr>(<?/)</pr>); <pos>p. pr. & vb. n.</pos> <conjf>Tying</conjf> <pr>(?)</pr>.]</vmorph> <ety>[OE. <ets>ti<?/en</ets>, <ets>teyen</ets>, AS. <ets>t<imac/gan</ets>, <ets>ti<eacute/gan</ets>, fr. <ets>te<aacute/g</ets>, <ets>te<aacute/h</ets>, a rope; akin to Icel. <ets>taug</ets>, and AS. <ets>te<oacute/n</ets> to draw, to pull. See <er>Tug</er>, <pos>v. t.</pos>, and cf. <er>Tow</er> to drag.]</ety> <sn>1.</sn> <def>To fasten with a band or cord and knot; to bind.</def> <ldquo/<xex>Tie</xex> the kine to the cart.<rdquo/  <rj><au>1 Sam. vi. 7.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>My son, keep thy father's commandment, and forsake not the law of thy mother: bind them continually upon thine heart, and <qex>tie</qex> them about thy neck.</q> <rj><qau>Prov. vi. 20,21.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To form, as a knot, by interlacing or complicating a cord; also, to interlace, or form a knot in; <as>as, to <ex>tie</ex> a cord to a tree</as>; to knit; to knot.</def> <ldquo/We do not <xex>tie</xex> this knot with an intention to puzzle the argument.<rdquo/  <rj><au>Bp. Burnet.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>To unite firmly; to fasten; to hold.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>In bond of virtuous love together <qex>tied</qex>.</q> <rj><qau>Fairfax.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>To hold or constrain by authority or moral influence, as by knotted cords; to oblige; to constrain; to restrain; to confine.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Not <qex>tied</qex> to rules of policy, you find<br/
Revenge less sweet than a forgiving mind.</q> <rj><qau>Dryden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <fld>(Mus.)</fld> <def>To unite, as notes, by a cross line, or by a curved line, or slur, drawn over or under them.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>6.</sn> <def>To make an equal score with, in a contest; to be even with.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>To ride and tie</b></col>. <cd>See under <er>Ride</er>.</cd> -- <col><b>To tie down</b></col>. <sd>(a)</sd> <cd>To fasten so as to prevent from rising.</cd> <sd>(b)</sd> <cd>To restrain; to confine; to hinder from action.</cd> -- <col><b>To tie up</b></col>, <cd>to confine; to restrain; to hinder from motion or action.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tie</ent><br/
<hw>Tie</hw>, <pos>v. i.</pos> <def>To make a tie; to make an equal score.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tiebar</ent><br/
<hw>Tie"bar`</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A flat bar used as a tie.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tiebeam</ent><br/
<hw>Tie"beam`</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Arch.)</fld> <def>A beam acting as a tie, as at the bottom of a pair of principal rafters, to prevent them from thrusting out the wall. See <xex>Illust.</xex> of <xex>Timbers</xex>, under <er>Roof</er>.</def>  <rj><au>Gwilt.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tien an men</ent><br/
<hw>Tien an men</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Chinese, square of heavenly peace.]</ety> <def>a large plaza in Beijing, China.</def><br/
[<source>PJC</source>]</p>

<p><ent>Tienda</ent><br/
||<hw>Ti*en"da</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Sp., prop., tent, awning.]</ety> <def>In Cuba, Mexico, etc., a booth, stall, or shop where merchandise is sold.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Tier</ent><br/
<hw>Ti"er</hw> <pr>(?)</pr>, <pos>n.</pos> <def>One who, or that which, ties.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tier</ent><br/
<hw>Ti"er</hw>, <pos>n.</pos> <ety>[See <er>Tire</er> a headdress.]</ety> <def>A chold's apron covering the upper part of the body, and tied with tape or cord; a pinafore.</def> <altsp>[Written also <asp>tire</asp>.]</altsp><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tier</ent><br/
<hw>Tier</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Perhaps fr. OF. <ets>tire</ets>, F. <ets>tire</ets>; probably of Teutonic origin; cf. OHG. <ets>ziar<imac/</ets> ornament, G. <ets>zier</ets>, AS. <ets>t<imac/r</ets> glory, <ets>ti<eacute/r</ets> row, rank. But cf. also F. <ets>tirer</ets> to draw, pull; of Teutonic origin.  Cf. <er>Attire</er>, <pos>v. t.</pos>, <er>Tire</er> a headdress, but also <er>Tirade</er>.]</ety> <def>A row or rank, especially one of two or more rows placed one above, or higher than, another; <as>as, a <ex>tier</ex> of seats in a theater</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Tiers of a cable</b></col>, <cd>the ranges of fakes, or windings, of a cable, laid one within another when coiled.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tierce</ent><br/
<hw>Tierce</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F. <ets>tierce</ets> a third, from <ets>tiers</ets>, <ets>tierce</ets>, third, fr. L. <ets>tertius</ets> the third; akin to <ets>tres</ets> three. See <er>Third</er>, <er>Three</er>, and cf. <er>Terce</er>, <er>Tercet</er>, <er>Tertiary</er>.]</ety> <sn>1.</sn> <def>A cask whose content is one third of a pipe; that is, forty-two wine gallons; also, a liquid measure of forty-two wine, or thirty-five imperial, gallons.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A cask larger than a barrel, and smaller than a hogshead or a puncheon, in which salt provisions, rice, etc., are packed for shipment.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <fld>(Mus.)</fld> <def>The third tone of the scale. See <er>Mediant</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>A sequence of three playing cards of the same suit. Tierce of ace, king, queen, is called <xex>tierce-major</xex>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <fld>(Fencing)</fld> <def>A position in thrusting or parrying in which the wrist and nails are turned downward.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>6.</sn> <fld>(R. C. Ch.)</fld> <def>The third hour of the day, or nine <sc>a. m,</sc>; one of the canonical hours; also, the service appointed for that hour.</def><br/
[<source>1913 Webster</source>]</p>

<p><-- p. 1508 --><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tierce</ent><br/
<hw>Tier*c<eacute/"</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[F.]</ety> <fld>(Her.)</fld> <def>Divided into three equal parts of three different tinctures; -- said of an escutcheon.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tiercelet</ent><br/
<ent>Tiercel</ent><br/
<mhw>{ <hw>Tier"cel</hw> <pr>(?)</pr>, <hw>Tierce"let</hw> <pr>(?)</pr>, }</mhw> <pos>n.</pos> <ety>[OE. <ets>tercel</ets>, <ets>tercelet</ets>, F. <ets>tiercelet</ets>, a dim. of (assumed) <ets>tiercel</ets>, or LL. <ets>tertiolus</ets>, dim. fr. L. <ets>tertius</ets> the third; -- so called, according to some, because every third bird in the nest is a male, or, according to others, because the male is the third part less than female.  Cf. <er>Tercel</er>.]</ety> <fld>(Falconry)</fld> <def>The male of various falcons, esp. of the peregrine; also, the male of the goshawk.</def>  <rj><au>Encyc. Brit.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tierce-major</ent><br/
<hw>Tierce"-ma`jor</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Cf. F. <ets>tierce majeure</ets>.]</ety> <fld>(Card Playing)</fld> <def>See <er>Tierce</er>, 4.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tiercet</ent><br/
<hw>Tier"cet</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F. <ets>tercet</ets>. See <er>Tercet</er>.]</ety> <fld>(Pros.)</fld> <def>A triplet; three lines, or three lines rhyming together.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tie-rod</ent><br/
<hw>Tie"-rod</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A rod used as a tie. See <er>Tie</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tiers etat</ent><br/
||<hw>Tiers` <eacute/`tat"</hw> <pr>(?)</pr>. <ety>[F.]</ety> <def>The third estate, or commonalty, in France, answering to the commons in Great Britain; -- so called in distinction from, and as inferior to, the nobles and clergy.</def><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ The refusal of the clergy and nobility to give the <xex>tiers <eacute/tat</xex> a representation in the States-general proportioned to their actual numbers had an important influence in bringing on the French Revolution of 1789. Since that time the term has been purely historical.</note><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tietick</ent><br/
<hw>Tie"tick</hw> <pr>(?)</pr>, <pos>n.</pos> <def>The meadow pipit.</def> <mark>[Prov. Eng]</mark>.<br/
[<source>1913 Webster</source>]</p>

<p><ent>Tiewig</ent><br/
<hw>Tie"wig`</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A wig having a tie or ties, or one having some of the curls tied up; also, a wig tied upon the head.</def>  <rj><au>Wright. V. Knox.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tiff</ent><br/
<hw>Tiff</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Originally, a sniff, sniffing; cf. Icel. <ets><?/efr</ets> a smell, <ets><?/efa</ets> to sniff, Norw. <ets>tev</ets> a drawing in of the breath, <ets>teva</ets> to sniff, smell, dial. Sw. <ets>t<uum/v</ets> smell, scent, taste.]</ety> <sn>1.</sn> <def>Liquor; especially, a small draught of liquor.</def> <ldquo/Sipping his <xex>tiff</xex> of brandy punch.<rdquo/  <rj><au>Sir W. Scott.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A fit of anger or peevishness; a slight altercation or contention. See <er>Tift</er>.</def>  <rj><au>Thackeray.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tiff</ent><br/
<hw>Tiff</hw>, <pos>v. i.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Tiffed</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Tiffing</conjf>.]</vmorph> <def>To be in a pet.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>She <qex>tiffed</qex> with Tim, she ran from Ralph.</q> <rj><qau>Landor.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tiff</ent><br/
<hw>Tiff</hw>, <pos>v. t.</pos> <ety>[OE. <ets>tiffen</ets>, OF. <ets>tiffer</ets>, <ets>tifer</ets>, to bedizen; cf. D. <ets>tippen</ets> to clip the points or ends of the hair, E. <ets>tip</ets>, n.]</ety> <def>To deck out; to dress.</def> <mark>[Obs.]</mark>  <rj><au>A. Tucker.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tiffany</ent><br/
<hw>Tif"fa*ny</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[OE. <ets>tiffenay</ets>; cf. OF. <ets>tiffe</ets> ornament, <ets>tiffer</ets> to adjust, adorn. See <er>Tiff</er> to dress.]</ety> <def>A species of gause, or very silk.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The smoke of sulphur . . . is commonly used by women to whiten <qex>tiffanies</qex>.</q> <rj><qau>Sir T. Browne.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tiffany</ent><br/
<hw>Tif"fa*ny</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[After <person>Louis Comfort <etsep>Tiffany</etsep></person>.]</ety> <sn>1.</sn> <def>made of stained glass by, or in the style of objects made by, <person>Louis C. <etsep>Tiffany</etsep></person>; <as>as, a <ex>Tiffany</ex> lamp</as>.</def><br/
[<source>PJC</source>]</p>

<p><sn>2.</sn> <def>Made by or purchased from the <corpn><etsep>Tiffany</etsep> corporation</corpn> (with headquarters in New York City); <as>as, <ex>Tiffany</ex> jewelry</as>.</def><br/
[<source>PJC</source>]</p>

<p><ent>Tiffin</ent><br/
<hw>Tif"fin</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Properly, <ets>tiffing</ets> a quaffing, a drinking. See <er>Tiff</er>, <pos>n.</pos>]</ety> <def>A lunch, or slight repast between breakfast and dinner; -- originally, a Provincial English word, but introduced into India, and brought back to England in a special sense.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tiffish</ent><br/
<hw>Tiff"ish</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Inclined to tiffs; peevish; petulant.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tift</ent><br/
<hw>Tift</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Cf. Norw. <ets>teft</ets> a scent. See <er>Tiff</er>, <pos>n.</pos>]</ety> <def>A fit of pettishness, or slight anger; a tiff.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>After all your fatigue you seem as ready for a <qex>tift</qex> with me as if you had newly come from church.</q> <rj><qau>Blackwood's Mag.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tig</ent><br/
<hw>Tig</hw> <pr>(?)</pr>, <pos>n.</pos> <sn>1.</sn> <def>A game among children. See <er>Tag</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A capacious, flat-bottomed drinking cup, generally with four handles, formerly used for passing around the table at convivial entertainment.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tigella</ent><br/
||<hw>Ti*gel"la</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[NL., from F. <ets>tige</ets> stem or stock.]</ety> <fld>(Bot.)</fld> <def>That part of an embryo which represents the young stem; the caulicle or radicle.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tigelle</ent><br/
<hw>Ti*gelle"</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F.]</ety> <fld>(Bot.)</fld> <def>Same as <er>Tigella</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tiger</ent><br/
<hw>Ti"ger</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[OE. <ets>tigre</ets>, F. <ets>tigre</ets>, L. <ets>tigris</ets>, Gr. <grk>ti`gris</grk>; probably of Persian origin; cf. Zend <ets>tighra</ets> pointed, <ets>tighri</ets> an arrow, Per. <ets>t<imac/r</ets>; perhaps akin to E. <ets>stick</ets>, v. t.; -- probably so named from its quickness.]</ety> <sn>1.</sn> <def>A very large and powerful carnivore (<spn>Felis tigris</spn>) native of Southern Asia and the East Indies. Its back and sides are tawny or rufous yellow, transversely striped with black, the tail is ringed with black, the throat and belly are nearly white. When full grown, it equals or exceeds the lion in size and strength. Called also <altname>royal tiger</altname>, and <altname>Bengal tiger</altname>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Fig.: A ferocious, bloodthirsty person.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>As for heinous <qex>tiger</qex>, Tamora.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>A servant in livery, who rides with his master or mistress.</def>  <rj><au>Dickens.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>A kind of growl or screech, after cheering; <as>as, three cheers and a <ex>tiger</ex></as>.</def> <mark>[Colloq. U. S.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <def>A pneumatic box or pan used in refining sugar.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>American tiger</b></col>. <fld>(Zool.)</fld> <sd>(a)</sd> <cd>The puma.</cd> <sd>(b)</sd> <cd>The jaguar.</cd> -- <col><b>Clouded tiger</b></col> <fld>(Zool.)</fld>, <cd>a handsome striped and spotted carnivore (<spn>Felis macrocelis</spn> or <spn>Felis marmorata</spn>) native of the <geog>East Indies</geog> and <geog>Southern Asia</geog>.  Its body is about three and a half feet long, and its tail about three feet long. Its ground color is brownish gray, and the dark markings are irregular stripes, spots, and rings, but there are always two dark bands on the face, one extending back from the eye, and one from the angle of the mouth. Called also <altname>tortoise-shell tiger</altname>.</cd> -- <col><b>Mexican tiger</b></col> <fld>(Zool.)</fld>, <cd>the jaguar.</cd> -- <col><b>Tiger beetle</b></col> <fld>(Zool.)</fld>, <cd>any one of numerous species of active carnivorous beetles of the family <fam>Cicindelidae</fam>. They usually inhabit dry or sandy places, and fly rapidly.</cd> -- <col><b>Tiger bittern</b></col>. <fld>(Zool.)</fld> <cd>See <cref>Sun bittern</cref>, under <er>Sun</er>.</cd> -- <col><b>Tiger cat</b></col> <fld>(Zool.)</fld>, <cd>any one of several species of wild cats of moderate size with dark transverse bars or stripes somewhat resembling those of the tiger.</cd> -- <col><b>Tiger flower</b></col> <fld>(Bot.)</fld>, <cd>an iridaceous plant of the genus <gen>Tigridia</gen> (as <spn>Tigridia conchiflora</spn>, <spn>Tigridia grandiflora</spn>, etc.) having showy flowers, spotted or streaked somewhat like the skin of a tiger.</cd> -- <col><b>Tiger grass</b></col> <fld>(Bot.)</fld>, <cd>a low East Indian fan palm (<spn>Chamaerops Ritchieana</spn>).  It is used in many ways by the natives.</cd> <au>J. Smith (Dict. Econ. Plants).</au> -- <col><b>Tiger lily</b></col>. <fld>(Bot.)</fld> <cd>See under <er>Lily</er>.</cd> -- <col><b>Tiger moth</b></col> <fld>(Zool.)</fld>, <cd>any one of numerous species of moths of the family <fam>Arctiadae</fam> which are striped or barred with black and white or with other conspicuous colors. The larvae are called <altname>woolly bears</altname>.</cd> -- <col><b>Tiger shark</b></col> <fld>(Zool.)</fld>, <cd>a voracious shark (<spn>Galeocerdo tigrinus</spn> syn. <spn>Galeocerdo maculatus</spn>) more or less barred or spotted with yellow.  It is found in both the <geog>Atlantic</geog> and <geog>Indian Ocean</geog>.  Called also <altname>zebra shark</altname>.</cd> -- <col><b>Tiger shell</b></col> <fld>(Zool.)</fld>, <cd>a large and conspicuously spotted cowrie (<spn>Cypraea tigris</spn>); -- so called from its fancied resemblance to a tiger in color and markings. Called also <altname>tiger cowrie</altname>.</cd> -- <col><b>Tiger snake</b></col> <fld>(Zool.)</fld>, <cd>either of two very venomous snakes of Tasmania and Australia, <spn>Notechis scutatis</spn> and <spn>Notechis ater</spn>, which grow up to 5 feet in length.</cd> -- <col><b>Tiger wolf</b></col> <fld>(Zool.)</fld>, <cd>the spotted hyena (<spn>Hyaena crocuta</spn>).</cd> -- <col><b>Tiger wood</b></col>, <cd>the variegated heartwood of a tree (<spn>Machaerium Schomburgkii</spn>) found in Guiana.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tiger-eye</ent><br/
<hw>Ti"ger-eye`</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Min.)</fld> <def>A siliceous stone of a yellow color and chatoyant luster, obtained in South Africa and much used for ornament. It is an altered form of the mineral crocidolite. See <er>Crocidolite</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tiger-foot</ent><br/
<hw>Ti"ger-foot`</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Bot.)</fld> <def>Same as <er>Tiger's-foot</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tiger-footed</ent><br/
<hw>Ti"ger-foot`ed</hw>, <pos>a.</pos> <def>Hastening to devour; furious.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tigerine</ent><br/
<hw>Ti"ger*ine</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Tigerish; tigrine.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tigerish</ent><br/
<hw>Ti"ger*ish</hw>, <pos>a.</pos> <def>Like a tiger; tigrish.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tiger's-foot</ent><br/
<hw>Ti"ger's-foot`</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Bot.)</fld> <def>A name given to some species of morning-glory (<gen>Ipomoea</gen>) having the leaves lobed in pedate fashion.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tigh</ent><br/
<hw>Tigh</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Perhaps akin to <ets>tight</ets>.]</ety> <def>A close, or inclosure; a croft.</def> <mark>[Obs.]</mark>  <rj><au>Cowell.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tight</ent><br/
<hw>Tight</hw> <pr>(t<imac/t)</pr>, <mark>obs.</mark> <def><pos>p. p.</pos> of <er>Tie</er>.</def>  <rj><au>Spenser.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tight</ent><br/
<hw>Tight</hw>, <pos>a.</pos> <amorph>[<pos>Compar.</pos> <adjf>Tighter</adjf> <pr>(t<imac/t"<etil/r)</pr>; <pos>superl.</pos> <adjf>Tightest</adjf>.]</amorph> <ety>[OE. <ets>tight</ets>, <ets>thiht</ets>; probably of Scand. origin; cf. Icel. <ets><thorn/<emac/ttr</ets>, Dan. <ets>t<ae/t</ets>, Sw. <ets>t<aum/t</ets>: akin to D. & G. <ets>dicht</ets> thick, tight, and perhaps to E. <ets>thee</ets> to thrive, or to <ets>thick</ets>.  Cf. <er>Taut</er>.]</ety> <sn>1.</sn> <def>Firmly held together; compact; not loose or open; <as>as, <ex>tight</ex> cloth; a <ex>tight</ex> knot.</as></def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Close, so as not to admit the passage of a liquid or other fluid; not leaky; <as>as, a <ex>tight</ex> ship; a <ex>tight</ex> cask; a <ex>tight</ex> room</as>; -- often used in this sense as the second member of a compound; <as>as, water-<ex>tight</ex>; air-<ex>tight</ex>.</as></def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>Fitting close, or too close, to the body; <as>as, a <ex>tight</ex> coat or other garment</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>Not ragged; whole; neat; tidy.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Clad very plain, but clean and <qex>tight</qex>.</q> <rj><qau>Evelyn.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>I'll spin and card, and keep our children <qex>tight</qex>.</q> <rj><qau>Gay.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <def>Close; parsimonious; saving; <as>as, a man <ex>tight</ex> in his dealings</as>.</def> <mark>[Colloq.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><sn>6.</sn> <def>Not slack or loose; firmly stretched; taut; -- applied to a rope, chain, or the like, extended or stretched out.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>7.</sn> <def>Handy; adroit; brisk.</def> <mark>[Obs.]</mark>  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>8.</sn> <def>Somewhat intoxicated; tipsy.</def> <mark>[Slang]</mark><br/
[<source>1913 Webster</source>]</p>

<p><sn>9.</sn> <fld>(Com.)</fld> <def>Pressing; stringent; not easy; firmly held; dear; -- said of money or the money market.  Cf. <er>Easy</er>, 7.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tight</ent><br/
<hw>Tight</hw>, <pos>v. t.</pos> <def>To tighten.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tighten</ent><br/
<hw>Tight"en</hw> <pr>(?)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Tightened</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Tightening</conjf>.]</vmorph> <def>To draw tighter; to straiten; to make more close in any manner.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Just where I please, with <qex>tightened</qex> rein<br/
I'll urge thee round the dusty plain.</q> <rj><qau>Fawkes.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Tightening pulley</b></col> <fld>(Mach.)</fld>, <cd>a pulley which rests, or is forced, against a driving belt to tighten it.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tightener</ent><br/
<hw>Tight"en*er</hw> <pr>(?)</pr>, <pos>n.</pos> <def>That which tightens; specifically <fld>(Mach.)</fld>, a tightening pulley.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tighter</ent><br/
<hw>Tight"er</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A ribbon or string used to draw clothes closer.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tightly</ent><br/
<hw>Tight"ly</hw>, <pos>adv.</pos> <def>In a tight manner; closely; nearly.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tightness</ent><br/
<hw>Tight"ness</hw>, <pos>n.</pos> <def>The quality or condition of being tight.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tights</ent><br/
<hw>Tights</hw> <pr>(?)</pr>, <pos>n. pl.</pos> <def>Close-fitting garments, especially for the lower part of the body and the legs.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tiglic</ent><br/
<hw>Tig"lic</hw> <pr>(?)</pr>, <pos>a.</pos> <fld>(Chem.)</fld> <def>Of, pertaining to, or designating, an acid, <chform>C4H7CO2H</chform> (called also <xex>methyl crotonic acid</xex>), homologous with crotonic acid, and obtained from croton oil (from <spn>Croton Tiglium</spn>) as a white crystalline substance.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tigress</ent><br/
<hw>Ti"gress</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[From <er>Tiger</er>: cf. F. <ets>tigresse</ets>.]</ety> <fld>(Zool.)</fld> <def>The female of the tiger.</def>  <rj><au>Holland.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tigrine</ent><br/
<hw>Ti"grine</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>tigrinus</ets>, fr. <ets>tigris</ets> a tiger.]</ety> <sn>1.</sn> <def>Of or pertaining to a tiger; like a tiger.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Zool.)</fld> <def>Resembling the tiger in color; <as>as, the <ex>tigrine</ex> cat (<spn>Felis tigrina</spn>) of South America</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tigrish</ent><br/
<hw>Ti"grish</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Resembling a tiger; tigerish.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tike</ent><br/
<hw>Tike</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Zool.)</fld> <def>A tick. See 2d <er>Tick</er>.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tike</ent><br/
<hw>Tike</hw>, <pos>n.</pos> <ety>[Icel. <ets>t<imac/k</ets> a bitch; akin to Sw. <ets>tik</ets>.]</ety> <sn>1.</sn> <def>A dog; a cur.</def> <ldquo/Bobtail <xex>tike</xex> or trundle-tail.<rdquo/  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A countryman or clown; a boorish, clumsy, or eccentric  person; -- also spelled <asp>tyke</asp>.</def><br/
[<source>1913 Webster</source> <source>+PJC</source>]</p>

<p><sn>3.</sn> <def>a small child; -- variant of <er>tyke</er>.</def><br/
[<source>PJC</source>]</p>

<p><ent>Tikor</ent><br/
<hw>Tik"or</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Hind. <ets>tikhur</ets>.]</ety> <def>A starch or arrow-root made from the tubes of an East Indian zinziberaceous plant (<spn>Curcuma angustifolia</spn>); also, the plant itself.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Tikoor</ent><br/
<ent>Tikur</ent><br/
<mhw>{ <hw>Ti*kur"</hw>, <hw>Ti*koor"</hw> }</mhw> <pr>(t<esl/*t<oomac/r")</pr>, <pos>n.</pos> <ety>[Hind. <ets>tik<umac/r</ets>.]</ety> <def>An East Indian tree (<spn>Garcinia pedunculata</spn>) having a large yellow fleshy fruit with a pleasant acid flavor.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Tikus</ent><br/
<hw>Ti"kus</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Zool.)</fld> <def>The bulau.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Til</ent><br/
<hw>Til</hw> <pr>(?)</pr>, <pos>prep. & conj.</pos> <def>See <er>Till</er>.</def> <mark>[Obs.]</mark>  <rj><au>Chaucer.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tilbury</ent><br/
<hw>Til"bu*ry</hw> <pr>(?)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Tilburies</plw> <pr>(#)</pr>.</plu> <ety>[Probably from <ets>Tilbury</ets>fort, in the Country of Essex, in England.]</ety> <def>A kind of gig or two-wheeled carriage, without a top or cover.</def> <altsp>[Written also <asp>tilburgh</asp>.]</altsp><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tilde</ent><br/
||<hw>Til"de</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Sp., fr. L. <ets>titulus</ets> a superscription, title, token, sign. See <er>Title</er>, <pos>n.</pos>]</ety> <def>The accentual mark placed over <it>n</it>, and sometimes over <it>l</it>, in Spanish words [thus, <it><ntil/</it>, <it><ltil/</it>], indicating that, in pronunciation, the sound of the following vowel is to be preceded by that of the initial, or consonantal, <it>y</it>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tile</ent><br/
<hw>Tile</hw> <pr>(?)</pr>, <pos>v. t.</pos> <ety>[See 2d <er>Tiler</er>.]</ety> <def>To protect from the intrusion of the uninitiated; <as>as, to <ex>tile</ex> a Masonic lodge</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tile</ent><br/
<hw>Tile</hw>, <pos>n.</pos> <ety>[OE. <ets>tile</ets>, <ets>tigel</ets>, AS. <ets>tigel</ets>, <ets>tigol</ets>, fr. L. <ets>tegula</ets>, from <ets>tegere</ets> to cover. See <er>Thatch</er>, and cf. <er>Tegular</er>.]</ety> <sn>1.</sn> <def>A plate, or thin piece, of baked clay, used for covering the roofs of buildings, for floors, for drains, and often for ornamental mantel works.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Arch.)</fld> <sd>(a)</sd> <def>A small slab of marble or other material used for flooring.</def> <sd>(b)</sd> <def>A plate of metal used for roofing.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <fld>(Metal.)</fld> <def>A small, flat piece of dried earth or earthenware, used to cover vessels in which metals are fused.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>A draintile.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <def>A stiff hat.</def> <mark>[Colloq.]</mark>  <rj><au>Dickens.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Tile drain</b></col>, <cd>a drain made of tiles.</cd> -- <col><b>Tile earth</b></col>, <cd>a species of strong, clayey earth; stiff and stubborn land.</cd> <mark>[Prov. Eng.]</mark> -- <col><b>Tile kiln</b></col>, <cd>a kiln in which tiles are burnt; a tilery.</cd> -- <col><b>Tile ore</b></col> <fld>(Min.)</fld>, <cd>an earthy variety of cuprite.</cd> -- <col><b>Tile red</b></col>, <cd>light red like the color of tiles or bricks.</cd> -- <col><b>Tile tea</b></col>, <cd>a kind of hard, flat brick tea. See <cref>Brick tea</cref>, under <er>Brick</er>.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tile</ent><br/
<hw>Tile</hw>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Tiled</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Tiling</conjf>.]</vmorph> <sn>1.</sn> <def>To cover with tiles; <as>as, to <ex>tile</ex> a house</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Fig.: To cover, as if with tiles.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The muscle, sinew, and vein,<br/
Which <qex>tile</qex> this house, will come again.</q> <rj><qau>Donne.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tile-drain</ent><br/
<hw>Tile"-drain`</hw> <pr>(?)</pr>, <pos>v. t.</pos> <def>To drain by means of tiles; to furnish with a tile drain.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tilefish</ent><br/
<hw>Tile"fish`</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Zool.)</fld> <def>A large, edible, deep-water food fish (<spn>Lopholatilus chamaeleonticeps</spn>) more or less thickly covered with large, round, yellow spots.</def><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ It was discovered off the Eastern coast of the United States in 1880, and was abundant in 1881, but is believed to have become extinct in 1882.</note><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tiler</ent><br/
<hw>Til"er</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A man whose occupation is to cover buildings with tiles.</def>  <rj><au>Bancroft.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tiler</ent><br/
<hw>Til"er</hw>, <pos>n.</pos> <ety>[Of uncertain origin, but probably from E. <ets>tile</ets>, n.]</ety> <def>A doorkeeper or attendant at a lodge of Freemasons.</def> <altsp>[Written also <asp>tyler</asp>.]</altsp><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tilery</ent><br/
<hw>Til"er*y</hw> <pr>(?)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Tileries</plw> <pr>(#)</pr>.</plu> <ety>[From <er>Tile</er>; cf. F. <ets>tuilerie</ets>, fr. <ets>tuile</ets> a tile, L. <ets>tegula</ets>.]</ety> <def>A place where tiles are made or burned; a tile kiln.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tileseed</ent><br/
<hw>Tile"seed`</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Bot.)</fld> <def>Any plant of the genus <gen>Geissois</gen>, having seeds overlapping like tiles on a roof.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Tilestone</ent><br/
<hw>Tile"stone`</hw> <pr>(?)</pr>, <pos>n.</pos> <sn>1.</sn> <fld>(Geol.)</fld> <def>A kind of laminated shale or sandstone belonging to some of the layers of the Upper Silurian.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A tile of stone.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tilia</ent><br/
||<hw>Til"i*a</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L., linden. Cf. <er>Teil</er>.]</ety> <fld>(Bot.)</fld> <def>A genus of trees, the lindens, the type of the family <fam>Tiliaceae</fam>, distinguished by the winglike bract coalescent with the peduncle, and by the indehiscent fruit having one or two seeds. There are about twenty species, natives of temperate regions.  Many species are planted as ornamental shade trees, and the tough fibrous inner bark is a valuable article of commerce. Also, a plant of this genus.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Tiliaceous</ent><br/
<hw>Til`i*a"ceous</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[OE. <ets>tilia</ets> the linden tree.]</ety> <fld>(Bot.)</fld> <def>Of, pertaining to, or resembling, a natural family of plants (<fam>Tiliaceae</fam>) of which the linden (<gen>Tilia</gen>) is the type. The family includes many plants which furnish a valuable fiber, as the jute.</def><br/
[<source>1913 Webster</source> <source>+PJC</source>]</p>

<p><ent>Tiling</ent><br/
<hw>Til"ing</hw> <pr>(?)</pr>, <pos>n.</pos> <sn>1.</sn> <def>A surface covered with tiles, or composed of tiles.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>They . . . let him down through the <qex>tiling</qex>.</q> <rj><qau>Luke v. 19.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Tiles, collectively.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Till</ent><br/
<hw>Till</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Abbrev. from <ets>lentil</ets>.]</ety> <def>A vetch; a tare.</def> <mark>[Prov. Eng.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Till</ent><br/
<hw>Till</hw>, <pos>n.</pos> <ety>[Properly, a drawer, from OE. <ets>tillen</ets> to draw. See <er>Tiller</er> the lever of a rudder.]</ety> <def>A drawer.</def> Specifically: <sd>(a)</sd> <def>A tray or drawer in a chest.</def> <sd>(b)</sd> <def>A money drawer in a shop or store.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Till alarm</b></col>, <cd>a device for sounding an alarm when a money drawer is opened or tampered with.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Till</ent><br/
<hw>Till</hw>, <pos>n.</pos> <sn>1.</sn> <fld>(Geol.)</fld> <def>A deposit of clay, sand, and gravel, without lamination, formed in a glacier valley by means of the waters derived from the melting glaciers; -- sometimes applied to alluvium of an upper river terrace, when not laminated, and appearing as if formed in the same manner.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A kind of coarse, obdurate land.</def>  <rj><au>Loudon.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Till</ent><br/
<hw>Till</hw>, <pos>prep.</pos> <ety>[OE. <ets>til</ets>, Icel. <ets>til</ets>; akin to Dan. <ets>til</ets>, Sw. <ets>till</ets>, OFries. <ets>til</ets>, also to AS. <ets>til</ets> good, excellent, G. <ets>ziel</ets> end, limit, object, OHG. <ets>zil</ets>, Goth. <ets>tils</ets>, ga<ets>tils</ets>, fit, convenient, and E. <ets>till</ets> to cultivate. See <er>Till</er>, <pos>v. t.</pos>]</ety> <def>To; unto; up to; as far as; until; -- now used only in respect to time, but formerly, also, of place, degree, etc., and still so used in Scotland and in parts of England and Ireland; <as>as, I worked <ex>till</ex> four o'clock; I will wait <ex>till</ex> next week.</as></def><br/
[<source>1913 Webster</source>]</p>

<p><q>He . . . came <qex>till</qex> an house.</q> <rj><qau>Chaucer.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Women, up <qex>till</qex> this<br/
Cramped under worse than South-sea-isle taboo.</q> <rj><qau>Tennyson.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Similar sentiments will recur to every one familiar with his writings -- all through them <qex>till</qex> the very end.</q> <rj><qau>Prof. Wilson.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Till now</b></col>, <cd>to the present time.</cd> -- <col><b>Till then</b></col>, <cd>to that time.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><-- p. 1509 --><br/
[<source>1913 Webster</source>]</p>

<p><ent>Till</ent><br/
<hw>Till</hw> <pr>(?)</pr>, <pos>conj.</pos> <def>As far as; up to the place or degree that; especially, up to the time that; that is, to the time specified in the sentence or clause following; until.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>And said unto them, Occupy <qex>till</qex> I come.</q> <rj><qau>Luke xix. 13.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Mediate so long <qex>till</qex> you make some act of prayer to God.</q> <rj><qau>Jer. Taylor.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>There was no outbreak <qex>till</qex> the regiment arrived.</q> <rj><qau>Macaulay.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ This use may be explained by supposing an ellipsis of <xex>when</xex>, or <xex>the time when</xex>, the proper conjunction or conjunctive adverb begin <xex>when</xex>.</note><br/
[<source>1913 Webster</source>]</p>

<p><ent>Till</ent><br/
<hw>Till</hw>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Tilled</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Tilling</conjf>.]</vmorph> <ety>[OE. <ets>tilen</ets>, <ets>tilien</ets>, AS. <ets>tilian</ets>, <ets>teolian</ets>, to aim, strive for, till; akin to OS. <ets>tilian</ets> to get, D. <ets>telen</ets> to propagate, G. <ets>zielen</ets> to aim, <ets>ziel</ets> an end, object, and perhaps also to E. <ets>tide</ets>, <ets>time</ets>, from the idea of something fixed or definite.  Cf. <er>Teal</er>, <er>Till</er>, <pos>prep.</pos>.]</ety> <sn>1.</sn> <def>To plow and prepare for seed, and to sow, dress, raise crops from, etc., to cultivate; <as>as, to <ex>till</ex> the earth, a field, a farm</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>No field nolde [would not] <qex>tilye</qex>.</q> <rj><qau>P. Plowman.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>the Lord God sent him forth from the garden of Eden, to <qex>till</qex> the ground from whence he was taken.</q> <rj><qau>Gen. iii. 23.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To prepare; to get.</def> <mark>[Obs.]</mark>  <rj><au>W. Browne.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Till</ent><br/
<hw>Till</hw>, <pos>v. i.</pos> <def>To cultivate land.</def>  <rj><au>Piers Plowman.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tillable</ent><br/
<hw>Till"a*ble</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Capable of being tilled; fit for the plow; arable.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tillage</ent><br/
<hw>Till"age</hw> <pr>(?)</pr>, <pos>n.</pos> <sn>1.</sn> <def>The operation, practice, or art of tilling or preparing land for seed, and keeping the ground in a proper state for the growth of crops.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A place tilled or cultivated; cultivated land.</def><br/
[<source>1913 Webster</source>]</p>

<p><syn><b>Syn.</b> -- Cultivation; culture; husbandry; farming; agriculture.</syn><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tillandsia</ent><br/
||<hw>Til*land"si*a</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[NL., after Prof. <etsep>Tillands</etsep>, of Abo, in Finland.]</ety> <fld>(Bot.)</fld> <def>An immense genus of epiphytic bromeliaceous plants confined to tropical and subtropical America. They usually bear a rosette of narrow overlapping basal leaves, which often hold a considerable quantity of water. The spicate or paniculate flowers have free perianth segments, and are often subtended by colored bracts. Also, a plant of this genus.</def>  <note><spn>Tillandsia usneoides</spn>, called <stype>Spanish moss</stype>, <stype>long moss</stype>, <stype>black moss</stype>, and <stype>Florida moss</stype>, has a very slender pendulous branching stem, and forms great hanging tufts on the branches of trees in the Southeastern United States and south to Argentina. It is often used for stuffing mattresses</note><br/
[<source>1913 Webster</source> + <source>Webster 1913 Suppl.</source>]</p>

<p><ent>Tiller</ent><br/
<hw>Till"er</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[From <er>Till</er>, <pos>v. t.</pos>]</ety> <def>One who tills; a husbandman; a cultivator; a plowman.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tiller</ent><br/
<hw>Till"er</hw>, <pos>n.</pos> <ety>[AS. <ets>telgor</ets> a small branch.  Cf. <er>Till</er> to cultivate.]</ety> <sn>1.</sn> <fld>(Bot.)</fld> <sd>(a)</sd> <def>A shoot of a plant, springing from the root or bottom of the original stalk; a sucker.</def> <sd>(b)</sd> <def>A sprout or young tree that springs from a root or stump.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A young timber tree.</def> <mark>[Prov. Eng.]</mark>  <rj><au>Evelyn.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tiller</ent><br/
<hw>Till"er</hw>, <pos>v. i.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Tillered</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Tillering</conjf>.]</vmorph> <def>To put forth new shoots from the root, or round the bottom of the original stalk; <as>as, wheat or rye <ex>tillers</ex>; some spread plants by <ex>tillering</ex>.</as></def> <altsp>[Sometimes written <asp>tillow</asp>.]</altsp><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tiller</ent><br/
<hw>Till"er</hw>, <pos>n.</pos> <ety>[From OE. <ets>tillen</ets>, <ets>tullen</ets>, to draw, pull; probably fr. AS. <ets>tyllan</ets> in for<ets>tyllan</ets> to lead astray; or cf. D. <ets>tillen</ets> to lift up.  Cf. <er>Till</er> a drawer.]</ety> <sn>1.</sn> <fld>(Naut.)</fld> <def>A lever of wood or metal fitted to the rudder head and used for turning side to side in steering. In small boats hand power is used; in large vessels, the tiller is moved by means of mechanical appliances. See <xex>Illust.</xex> of <er>Rudder</er>.  Cf. 2d <er>Helm</er>, 1.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>The stalk, or handle, of a crossbow; also, sometimes, the bow itself.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>You can shoot in a <qex>tiller</qex>.</q> <rj><qau>Beau. & Fl.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>The handle of anything.</def> <mark>[Prov. Eng.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>A small drawer; a till.</def>  <rj><au>Dryden.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Tiller rope</b></col> <fld>(Naut.)</fld>, <cd>a rope for turning a tiller. In a large vessel it forms the connection between the fore end of the tiller and the steering wheel.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tilley seed</ent><br/
<ent>Tilley</ent><br/
<mhw>{ <hw>Til"ley</hw> <pr>(?)</pr>, <pos>n.</pos>, <it>or</it> <hw>Til"ley seed`</hw> <pr>(?)</pr> }</mhw>. <fld>(Bot.)</fld> <def>The seeds of a small tree (<spn>Croton Pavana</spn>) common in the Malay Archipelago. These seeds furnish croton oil, like those of <spn>Croton Tiglium</spn>.</def> <altsp>[Written also <asp>tilly</asp>.]</altsp><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tillman</ent><br/
<hw>Till"man</hw> <pr>(?)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Tillmen</plw> <pr>(<?/)</pr>.</plu> <def>A man who tills the earth; a husbandman.</def> <mark>[Obs.]</mark>  <rj><au>Tusser.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tillodont</ent><br/
<hw>Til"lo*dont</hw> <pr>(?)</pr>, <pos>n.</pos> <def>One of the Tillodontia.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tillodontia</ent><br/
||<hw>Til`lo*don"ti*a</hw> <pr>(?)</pr>, <pos>n. pl.</pos> <fld>(Paleon.)</fld> <def>An extinct group of Mammalia found fossil in the Eocene formation. The species are related to the carnivores, ungulates, and rodents. Called also <altname>Tillodonta</altname>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tillot</ent><br/
<hw>Til"lot</hw> <pr>(t<icr/l"l<ocr/t)</pr>, <pos>n.</pos> <def>A bag made of thin glazed muslin, used as a wrapper for dress goods.</def>  <rj><au>McElrath.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tillow</ent><br/
<hw>Til"low</hw> <pr>(?)</pr>, <pos>v. i.</pos> <def>See 3d <er>Tiller</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tilly-vally</ent><br/
<hw>Til"ly-val`ly</hw> <pr>(?)</pr>, <pos>interj., adv., or a.</pos> <def>A word of unknown origin and signification, formerly used as expressive of contempt, or when anything said was rejected as trifling or impertinent.</def> <altsp>[Written also <asp>tille-vally</asp>, <asp>tilly-fally</asp>, <asp>tille-fally</asp>, and otherwise.]</altsp>  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tilmus</ent><br/
||<hw>Til"mus</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[NL., fr. Gr. <grk>tilmo`s</grk>, fr. <grk>ti`llein</grk> to pluck, pull.]</ety> <fld>(Med.)</fld> <def>Floccillation.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Til seed</ent><br/
<hw>Til seed</hw> <pr>(t<icr/l; t<emac/l)</pr>. <sd>(a)</sd> <def>The seed of sesame.</def> <sd>(b)</sd> <def>The seed of an African asteraceous plant (<spn>Guizotia abyssinica</spn>), yielding a bland fixed oil used in medicine.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Tilt</ent><br/
<hw>Tilt</hw> <pr>(t<icr/lt)</pr>, <pos>n.</pos> <ety>[OE. <ets>telt</ets> (perhaps from the Danish), <ets>teld</ets>, AS. <ets>teld</ets>, ge<ets>teld</ets>; akin to OD. <ets>telde</ets>, G. <ets>zelt</ets>, Icel. <ets>tjald</ets>, Sw. <ets>t<aum/lt</ets>, <ets>tj<aum/ll</ets>, Dan. <ets>telt</ets>, and AS. be<ets>teldan</ets> to cover.]</ety> <sn>1.</sn> <def>A covering overhead; especially, a tent.</def>  <rj><au>Denham.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>The cloth covering of a cart or a wagon.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <fld>(Naut.)</fld> <def>A cloth cover of a boat; a small canopy or awning extended over the sternsheets of a boat.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Tilt boat</b></col> <fld>(Naut.)</fld>, <cd>a boat covered with canvas or other cloth.</cd> -- <col><b>Tilt roof</b></col> <fld>(Arch.)</fld>, <cd>a round-headed roof, like the canopy of a wagon.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tilt</ent><br/
<hw>Tilt</hw>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Tilted</conjf>; <pos>p. pr. & vb. n.</pos> <conjf>Tilting</conjf>.]</vmorph> <def>To cover with a tilt, or awning.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tilt</ent><br/
<hw>Tilt</hw>, <pos>v. t.</pos> <ety>[OE. <ets>tilten</ets>, <ets>tulten</ets>, to totter, fall, AS. <ets>tealt</ets> unstable, precarious; akin to <ets>tealtrian</ets> to totter, to vacillate, D. <ets>tel</ets> amble, ambling pace, G. <ets>zelt</ets>, Icel. <ets>t<oum/lt</ets> an ambling pace, <ets>t<oum/lta</ets> to amble.  Cf. <er>Totter</er>.]</ety> <sn>1.</sn> <def>To incline; to tip; to raise one end of for discharging liquor; <as>as, to <ex>tilt</ex> a barrel</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To point or thrust, as a lance.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Sons against fathers <qex>tilt</qex> the fatal lance.</q> <rj><qau>J. Philips.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>To point or thrust a weapon at.</def> <mark>[Obs.]</mark>  <rj><au>Beau. & Fl.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>To hammer or forge with a tilt hammer; <as>as, to <ex>tilt</ex> steel in order to render it more ductile</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tilt</ent><br/
<hw>Tilt</hw>, <pos>v. i.</pos> <sn>1.</sn> <def>To run or ride, and thrust with a lance; to practice the military game or exercise of thrusting with a lance, as a combatant on horseback; to joust; also, figuratively, to engage in any combat or movement resembling that of horsemen tilting with lances.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>He <qex>tilts</qex><br/
With piercing steel at bold Mercutio's breast.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Swords out, and <qex>tilting</qex> one at other's breast.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>But in this tournament can no man <qex>tilt</qex>.</q> <rj><qau>Tennyson.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>The fleet, swift <qex>tilting</qex>, o'er the <?/urges flew.</q> <rj><qau>Pope.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To lean; to fall partly over; to tip.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The trunk of the body is kept from <qex>tilting</qex> forward by the muscles of the back.</q> <rj><qau>Grew.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tilt</ent><br/
<hw>Tilt</hw> <pr>(?)</pr>, <pos>n.</pos> <sn>1.</sn> <def>A thrust, as with a lance.</def>  <rj><au>Addison.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A military exercise on horseback, in which the combatants attacked each other with lances; a tournament.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>See <er>Tilt hammer</er>, in the Vocabulary.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>Inclination forward; <as>as, the <ex>tilt</ex> of a cask</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Full tilt</b></col>, <cd>with full force.</cd>  <rj><au>Dampier.</au></rj></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tilter</ent><br/
<hw>Tilt"er</hw> <pr>(?)</pr>, <pos>n.</pos> <sn>1.</sn> <def>One who tilts, or jousts; hence, one who fights.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Let me alone to match your <qex>tilter</qex>.</q> <rj><qau>Glanville.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>One who operates a tilt hammer.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tilth</ent><br/
<hw>Tilth</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[AS. <ets>til<?/</ets>, fr. <ets>tilian</ets> to till. See <er>Till</er> to cultivate.]</ety> <sn>1.</sn> <def>The state of being tilled, or prepared for a crop; culture; <as>as, land is good <ex>tilth</ex></as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The <qex>tilth</qex> and rank fertility of its golden youth.</q> <rj><qau>De Quincey.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>That which is tilled; tillage ground.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>And so by <qex>tilth</qex> and grange . . . <br/
We gained the mother city.</q> <rj><qau>Tennyson.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tilt hammer</ent><br/
<hw>Tilt" ham`mer</hw> <pr>(?)</pr>. <def>A tilted hammer; a heavy hammer, used in iron works, which is lifted or tilted by projections or wipers on a revolving shaft; a trip hammer.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tilting</ent><br/
<hw>Tilt"ing</hw> <pr>(?)</pr>, <pos>n.</pos> <sn>1.</sn> <def>The act of one who tilts; a tilt.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>The process by which blister steel is rendered ductile by being forged with a tilt hammer.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Tilting helmet</b></col>, <cd>a helmet of large size and unusual weight and strength, worn at tilts.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tilt-mill</ent><br/
<hw>Tilt"-mill`</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A mill where a tilt hammer is used, or where the process of tilting is carried on.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Til tree</ent><br/
<hw>Til tree</hw> <pr>(t<icr/l; t<emac/l)</pr>. <fld>(Bot.)</fld> <sd>(a)</sd> <def>Var. of <er>Teil tree</er>.</def> <sd>(b)</sd> <def>An ill-smelling lauraceous tree (<spn>Ocotea foetens</spn>) of the Canary Islands; -- sometimes distinguished as <altname>Canary Island til tree</altname>.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Tilt-up</ent><br/
<hw>Tilt"-up`</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Zool.)</fld> <def>Same as <er>Tip-up</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tilt-yard</ent><br/
<hw>Tilt"-yard`</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A yard or place for tilting.</def> <ldquo/The <xex>tilt-yard</xex> of Templestowe.<rdquo/  <rj><au>Sir W. Scott.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Timal</ent><br/
<hw>Ti"mal</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Zool.)</fld> <def>The blue titmouse.</def> <mark>[Prov. Eng.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Timaline</ent><br/
<hw>Tim"a*line</hw> <pr>(?)</pr>, <pos>a.</pos> <fld>(Zool.)</fld> <def>Of or pertaining to the genus <gen>Timalus</gen> or family <fam>Timalidae</fam>, which includes the babblers thrushes, and bulbuls.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Timbal</ent><br/
<hw>Tim"bal</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A kettledrum. See <er>Tymbal</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Timbale</ent><br/
||<hw>Tim`bale"</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F., prop., a kettledrum; -- so named from the form of the mold used. Cf. <er>Timbal</er>.]</ety> <fld>(Cookery)</fld> <def>A seasoned preparation, as of chicken, lobster, cheese, or fish, cooked in a drum-shaped mold; also, a pastry case, usually small, filled with a cooked mixture.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Timber</ent><br/
<hw>Tim"ber</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Probably the same word as <ets>timber</ets> sort of wood; cf. Sw. <ets>timber</ets>, LG. <ets>timmer</ets>, MHG. <ets>zimber</ets>, G. <ets>zimmer</ets>, F. <ets>timbre</ets>, LL. <ets>timbrium</ets>.  Cf. <er>Timmer</er>.]</ety> <fld>(Com.)</fld> <def>A certain quantity of fur skins, as of martens, ermines, sables, etc., packed between boards; being in some cases forty skins, in others one hundred and twenty; -- called also <altname>timmer</altname>.</def> <altsp>[Written also <asp>timbre</asp>.]</altsp><br/
[<source>1913 Webster</source>]</p>

<p><ent>Timber</ent><br/
<hw>Tim"ber</hw>, <pos>n.</pos> <ety>[F. <ets>timbre</ets>. See <er>Timbre</er>.]</ety> <fld>(Her.)</fld> <def>The crest on a coat of arms.</def> <altsp>[Written also <asp>timbre</asp>.]</altsp><br/
[<source>1913 Webster</source>]</p>

<p><ent>Timber</ent><br/
<hw>Tim"ber</hw>, <pos>v. t.</pos> <def>To surmount as a timber does.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Timber</ent><br/
<hw>Tim"ber</hw>, <pos>n.</pos> <ety>[AS. <ets>timbor</ets>, <ets>timber</ets>, wood, building; akin to OFries. <ets>timber</ets>, D. <ets>timmer</ets> a room, G. <ets>zimmer</ets>, OHG. <ets>zimbar</ets> timber, a dwelling, room, Icel. <ets>timbr</ets> timber, Sw. <ets>timmer</ets>, Dan. <ets>t<oum/mmer</ets>, Goth. <ets>timrjan</ets> to build, <ets>timrja</ets> a builder, L. <ets>domus</ets> a house, Gr. <?/ house, <?/ to build, Skr. <ets>dama</ets> a house. <root/62.  Cf. <er>Dome</er>, <er>Domestic</er>.]</ety> <sn>1.</sn> <def>That sort of wood which is proper for buildings or for tools, utensils, furniture, carriages, fences, ships, and the like; -- usually said of felled trees, but sometimes of those standing.  Cf. <er>Lumber</er>, 3.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>And ta'en my fiddle to the gate, . . . <br/
And fiddled in the <qex>timber</qex>!</q> <rj><qau>Tennyson.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>The body, stem, or trunk of a tree.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>Fig.: Material for any structure.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Such dispositions are the very errors of human nature; and yet they are the fittest <qex>timber</qex> to make politics of.</q> <rj><qau>Bacon.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>A single piece or squared stick of wood intended for building, or already framed; collectively, the larger pieces or sticks of wood, forming the framework of a house, ship, or other structure, in distinction from the covering or boarding.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>So they prepared <qex>timber</qex> . . . to build the house.</q> <rj><qau>1 Kings v. 18.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Many of the <qex>timbers</qex> were decayed.</q> <rj><qau>W. Coxe.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <def>Woods or forest; wooden land.</def> <mark>[Western U. S.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><sn>6.</sn> <fld>(Shipbuilding)</fld> <def>A rib, or a curving piece of wood, branching outward from the keel and bending upward in a vertical direction. One <xex>timber</xex> is composed of several pieces united.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Timber and room</b></col>. <fld>(Shipbuilding)</fld> <cd>Same as <cref>Room and space</cref>. See under <er>Room</er>.</cd> -- <col><b>Timber beetle</b></col> <fld>(Zool.)</fld>, <cd>any one of numerous species of beetles the larvae of which bore in timber; <as>as, the silky <ex>timber beetle</ex> (<spn>Lymexylon sericeum</spn>)</as>.</cd> -- <col><b>Timber doodle</b></col> <fld>(Zool.)</fld>, <cd>the American woodcock.</cd> <mark>[Local, U. S.]</mark> -- <col><b>Timber grouse</b></col> <fld>(Zool.)</fld>, <cd>any species of grouse that inhabits woods, as the ruffed grouse and spruce partridge; -- distinguished from <xex>prairie grouse</xex>.</cd> -- <col><b>Timber hitch</b></col> <fld>(Naut.)</fld>, <cd>a kind of hitch used for temporarily marking fast a rope to a spar. See <xex>Illust.</xex> under <er>Hitch</er>.</cd> -- <col><b>Timber mare</b></col>, <cd>a kind of instrument upon which soldiers were formerly compelled to ride for punishment.</cd> <au>Johnson.</au> -- <col><b>Timber scribe</b></col>, <cd>a metal tool or pointed instrument for marking timber.</cd> <au>Simmonds.</au> -- <col><b>Timber sow</b></col>. <fld>(Zool.)</fld> <cd>Same as <cref>Timber worm</cref>, below.</cd> <au>Bacon.</au> -- <col><b>Timber tree</b></col>, <cd>a tree suitable for timber.</cd> -- <col><b>Timber worm</b></col> <fld>(Zool.)</fld>, <cd>any larval insect which burrows in timber.</cd> -- <col><b>Timber yard</b></col>, <cd>a yard or place where timber is deposited.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Timber</ent><br/
<hw>Tim"ber</hw> <pr>(?)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Timbered</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Timbering</conjf>.]</vmorph> <def>To furnish with timber; -- chiefly used in the past participle.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>His bark is stoutly <qex>timbered</qex>.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Timber</ent><br/
<hw>Tim"ber</hw>, <pos>v. i.</pos> <sn>1.</sn> <def>To light on a tree.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Falconry)</fld> <def>To make a nest.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Timbered</ent><br/
<hw>Tim"bered</hw> <pr>(?)</pr>, <pos>a.</pos> <sn>1.</sn> <def>Furnished with timber; -- often compounded; <as>as, a well-<ex>timbered</ex> house; a low-<ex>timbered</ex> house</as>.</def>  <rj><au>L'Estrange.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Built; formed; contrived.</def> <mark>[R.]</mark>  <rj><au>Sir H. Wotton.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>Massive, like timber.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>His <qex>timbered</qex> bones all broken, rudely rumbled.</q> <rj><qau>Spenser.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>Covered with growth timber; wooden; <as>as, well-<ex>timbered</ex> land</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Timberhead</ent><br/
<hw>Tim"ber*head`</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Naut.)</fld> <def>The top end of a timber, rising above the gunwale, and serving for belaying ropes, etc.; -- called also <altname>kevel head</altname>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Timbering</ent><br/
<hw>Tim"ber*ing</hw>, <pos>n.</pos> <def>The act of furnishing with timber; also, timbers, collectively; timberwork; timber.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Timberling</ent><br/
<hw>Tim"ber*ling</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[<ets>Timber</ets> + <ets>-ling</ets>.]</ety> <def>A small tree.</def> <mark>[Eng.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Timberman</ent><br/
<hw>Tim"ber*man</hw> <pr>(?)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Timbermen</plw> <pr>(<?/)</pr>.</plu> <fld>(Mining)</fld> <def>A man employed in placing supports of timber in a mine.</def>  <rj><au>Weale.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Timberwork</ent><br/
<hw>Tim"ber*work`</hw> <pr>(?)</pr>, <pos>n.</pos> <def>Work made of timbers.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Timbre</ent><br/
<hw>Tim"bre</hw> <pr>(?)</pr>, <pos>n.</pos> <def>See 1st <er>Timber</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Timbre</ent><br/
<hw>Tim"bre</hw>, <pos>n.</pos> <ety>[F., a bell to be struck with a hammer, sound, tone, stamp, crest, in OF., a timbrel.  Cf. <er>Timbrel</er>.]</ety> <sn>1.</sn> <fld>(Her.)</fld> <def>The crest on a coat of arms.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Mus.)</fld> <def>The quality or tone distinguishing voices or instruments; tone color; clang tint; <as>as, the <ex>timbre</ex> of the voice; the <ex>timbre</ex> of a violin.</as>  See <er>Tone</er>, and <cref>Partial tones</cref>, under <er>Partial</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Timbrel</ent><br/
<hw>Tim"brel</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Dim. of OE. <ets>timbre</ets>, OF. <ets>timbre</ets>; probably fr. L. <ets>typmanum</ets>, Gr. <?/ a kettledrum, but influenced perhaps by Ar. <ets>tabl</ets> a drum; cf. Per. <ets>tambal</ets> a drum. See <er>Tympanum</er>, and cf. 2d <er>Timbre</er>, <er>Tymbal</er>.]</ety> <fld>(Mus.)</fld> <def>A kind of drum, tabor, or tabret, in use from the highest antiquity.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Miriam . . . took a <qex>timbrel</qex> in her hand, and all the women went out after her with <qex>timbrels</qex> and with dances.</q> <rj><qau>Ex. xv. 20.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Timbrelled</ent><br/
<ent>Timbreled</ent><br/
<mhw>{ <hw>Tim"breled</hw>, <hw>Tim"brelled</hw>}</mhw> <pr>(?)</pr>, <pos>a.</pos> <def>Sung to the sound of the timbrel.</def> <ldquo/In vain with <xex>timbreled</xex> anthems dark.<rdquo/  <rj><au>Milton.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Timbuktoo</ent><br/
<ent>Timbuctoo</ent><br/
<mhw><hw>Timbuctoo</hw>, <hw>Timbuktoo</hw></mhw> <pos>prop. n.</pos>, <def>A city on the southern edge of the Sahara, in central Africa, some nine miles from the Niger.  It is about three miles around, and was formerly surrounded by a clay wall.  Timbuctoo has a large caravan trade, gold dust being the most important export.  The people are negroes, Tuariks, Mandingoes, Arabs, Foolahs, etc.  The city was founded in the 12th century, but was first seen by a white man in 1826.  Timbuctoo now belongs to France, and a railroad is proposed to connect Algiers, Timbuctoo and Senegambia.  Population, 13,000 (1893), greatly increased during the trading season from November to January.</def> <au>Student's Cyclopedia, 1897.</au><br/
[<source>PJC</source>]</p>

<p><ent>Timburine</ent><br/
<hw>Tim`bu*rine"</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A tambourine.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Time</ent><br/
<hw>Time</hw> <pr>(?)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Times</plw> <pr>(#)</pr>.</plu> <ety>[OE. <ets>time</ets>, AS. <ets>t<imac/ma</ets>, akin to <ets>t<imac/d</ets> time, and to Icel. <ets>t<imac/mi</ets>, Dan. <ets>time</ets> an hour, Sw. <ets>timme</ets>. <root/58. See <er>Tide</er>, <pos>n.</pos>]</ety> <sn>1.</sn> <def>Duration, considered independently of any system of measurement or any employment of terms which designate limited portions thereof.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The <qex>time</qex> wasteth [<it>i. e.</it> passes away] night and day.</q> <rj><qau>Chaucer.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>I know of no ideas . . . that have a better claim to be accounted simple and original than those of space and <qex>time</qex>.</q> <rj><qau>Reid.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A particular period or part of duration, whether past, present, or future; a point or portion of duration; <as>as, the <ex>time</ex> was, or has been; the <ex>time</ex> is, or will be</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>God, who at sundry <qex>times</qex> and in divers manners spake in <qex>time</qex> past unto the fathers by the prophets.</q> <rj><qau>Heb. i. 1.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>The period at which any definite event occurred, or person lived; age; period; era; <as>as, the Spanish Armada was destroyed in the <ex>time</ex> of Queen Elizabeth</as>; -- often in the plural; <as>as, ancient <ex>times</ex>; modern <ex>times</ex>.</as></def><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>The duration of one's life; the hours and days which a person has at his disposal.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Believe me, your <qex>time</qex> is not your own; it belongs to God, to religion, to mankind.</q> <rj><qau>Buckminster.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <def>A proper time; a season; an opportunity.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>There is . . . a <qex>time</qex> to every purpose.</q> <rj><qau>Eccl. iii. 1.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>The <qex>time</qex> of figs was not yet.</q> <rj><qau>Mark xi. 13.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>6.</sn> <def>Hour of travail, delivery, or parturition.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>She was within one month of her <qex>time</qex>.</q> <rj><qau>Clarendon.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>7.</sn> <def>Performance or occurrence of an action or event, considered with reference to repetition; addition of a number to itself; repetition; <as>as, to double cloth four <ex>times</ex>; four <ex>times</ex> four, or sixteen</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Summers three <qex>times</qex> eight save one.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>8.</sn> <def>The present life; existence in this world as contrasted with immortal life; definite, as contrasted with infinite, duration.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Till <qex>time</qex> and sin together cease.</q> <rj><qau>Keble.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>9.</sn> <fld>(Gram.)</fld> <def>Tense.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>10.</sn> <fld>(Mus.)</fld> <def>The measured duration of sounds; measure; tempo; rate of movement; rhythmical division; <as>as, common or triple <ex>time</ex>; the musician keeps good <ex>time</ex></as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Some few lines set unto a solemn <qex>time</qex>.</q> <rj><qau>Beau. & Fl.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ <xex>Time</xex> is often used in the formation of compounds, mostly self-explaining; as, <xex>time</xex>-battered, <xex>time</xex>-beguiling, <xex>time</xex>-consecrated, <xex>time</xex>-consuming, <xex>time</xex>-enduring, <xex>time</xex>-killing, <xex>time</xex>-sanctioned, <xex>time</xex>-scorner, <xex>time</xex>-wasting, <xex>time</xex>-worn, etc.</note><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Absolute time</b></col>, <cd>time irrespective of local standards or epochs; as, all spectators see a lunar eclipse at the same instant of <xex>absolute time</xex>.</cd> -- <col><b>Apparent time</b></col>, <cd>the time of day reckoned by the sun, or so that 12 o'clock at the place is the instant of the transit of the sun's center over the meridian.</cd> -- <col><b>Astronomical time</b></col>, <cd>mean solar time reckoned by counting the hours continuously up to twenty-four from one noon to the next.</cd> -- <col><b>At times</b></col>, <cd>at distinct intervals of duration; now and then; <as>as, <ex>at times</ex> he reads, <ex>at</ex> other <ex>times</ex> he rides</as>.</cd> -- <col><b>Civil time</b></col>, <cd>time as reckoned for the purposes of common life in distinct periods, as years, months, days, hours, etc., the latter, among most modern nations, being divided into two series of twelve each, and reckoned, the first series from midnight to noon, the second, from noon to midnight.</cd> -- <col><b>Common time</b></col> <fld>(Mil.)</fld>, <cd>the ordinary time of marching, in which ninety steps, each twenty-eight inches in length, are taken in one minute.</cd> -- <col><b>Equation of time</b></col>. <cd>See under <er>Equation</er>, <pos>n.</pos></cd> -- <col><b>In time</b></col>. <sd>(a)</sd> <cd>In good season; sufficiently early; <as>as, he arrived <ex>in time</ex> to see the exhibition</as>.</cd> <sd>(b)</sd> <cd>After a considerable space of duration; eventually; finally; <as>as, you will <ex>in time</ex> recover your health and strength</as>.</cd> -- <col><b>Mean time</b></col>. <cd>See under 4th <er>Mean</er>.</cd> -- <col><b>Quick time</b></col> <fld>(Mil.)</fld>, <cd>time of marching, in which one hundred and twenty steps, each thirty inches in length, are taken in one minute.</cd> -- <col><b>Sidereal time</b></col>. <cd>See under <er>Sidereal</er>.</cd> -- <col><b>Standard time</b></col>, <cd>the civil time that has been established by law or by general usage over a region or country. In England the standard time is Greenwich mean solar time. In the United States and Canada four kinds of standard time have been adopted by the railroads and accepted by the people, viz., <xex>Eastern</xex>, <xex>Central</xex>, <xex>Mountain</xex>, and <xex>Pacific</xex> time, corresponding severally to the mean local times of the 75th, 90th, 105th, and 120th meridians west from Greenwich, and being therefore five, six, seven, and eight hours slower than Greenwich time.</cd> -- <col><b>Time ball</b></col>, <cd>a ball arranged to drop from the summit of a pole, to indicate true midday time, as at Greenwich Observatory, England.</cd> <au>Nichol.</au> -- <col><b>Time bargain</b></col> <fld>(Com.)</fld>, <cd>a contract made for the sale or purchase of merchandise, or of stock in the public funds, at a certain time in the future.</cd><-- = a futures contract? --> -- <col><b>Time bill</b></col>. <cd>Same as <er>Time-table</er>.</cd> <mark>[Eng.]</mark> -- <col><b>Time book</b></col>, <cd>a book in which is kept a record of the time persons have worked.</cd> -- <col><b>Time detector</b></col>, <cd>a timepiece provided with a device for registering and indicating the exact time when a watchman visits certain stations in his beat.</cd> -- <col><b>Time enough</b></col>, <cd>in season; early enough.</cd> <ldquo/Stanly at Bosworth field, . . . came <xex>time enough</xex> to save his life.<rdquo/ <au>Bacon.</au> -- <col><b>Time fuse</b></col>, <cd>a fuse, as for an explosive projectile, which can be so arranged as to ignite the charge at a certain definite interval after being itself ignited.</cd> -- <mcol><col><b>Time immemorial</b></col>, <it>or</it> <col><b>Time out of mind</b></col></mcol>. <fld>(Eng. Law)</fld> <cd>See under <er>Immemorial</er>.</cd> -- <col><b>Time lock</b></col>, <cd>a lock having clockwork attached, which, when wound up, prevents the bolt from being withdrawn when locked, until a certain interval of time has elapsed.</cd> -- <col><b>Time of day</b></col>, <cd>salutation appropriate to the times of the day, as <ldquo/good morning,<rdquo/ <ldquo/good evening,<rdquo/ and the like; greeting.</cd> -- <col><b>To kill time</b></col>. <cd>See under <er>Kill</er>, <pos>v. t.</pos></cd> -- <col><b>To make time</b></col>. <sd>(a)</sd> <cd>To gain time.</cd> <sd>(b)</sd> <cd>To occupy or use (a certain) time in doing something; <as>as, the trotting horse <ex>made</ex> fast <ex>time</ex></as>.</cd> -- <mcol><col><b>To move against time</b></col>, <col><b>To run against time</b></col>, <it>or</it> <col><b>To go against time</b></col></mcol>, <cd>to move, run, or go a given distance without a competitor, in the quickest possible time; or, to accomplish the greatest distance which can be passed over in a given time; <as>as, the horse is <ex>to run against time</ex></as>.</cd> -- <col><b>True time</b></col>. <sd>(a)</sd> <cd>Mean time as kept by a clock going uniformly.</cd> <sd>(b)</sd> <fld>(Astron.)</fld> <cd>Apparent time as reckoned from the transit of the sun's center over the meridian.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><-- p. 1510 --><br/
[<source>1913 Webster</source>]</p>

<p><ent>Time</ent><br/
<hw>Time</hw> <pr>(t<imac/m)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Timed</conjf> <pr>(t<imac/md)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Timing</conjf>.]</vmorph> <sn>1.</sn> <def>To appoint the time for; to bring, begin, or perform at the proper season or time; <as>as, he <ex>timed</ex> his appearance rightly</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>There is no greater wisdom than well to <qex>time</qex> the beginnings and onsets of things.</q> <rj><qau>Bacon.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To regulate as to time; to accompany, or agree with, in time of movement.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Who overlooked the oars, and <qex>timed</qex> the stroke.</q> <rj><qau>Addison.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>He was a thing of blood, whose every motion<br/
Was <qex>timed</qex> with dying cries.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>To ascertain or record the time, duration, or rate of; <as>as, to <ex>time</ex> the speed of horses, or hours for workmen</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>To measure, as in music or harmony.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Time</ent><br/
<hw>Time</hw>, <pos>v. i.</pos> <sn>1.</sn> <def>To keep or beat time; to proceed or move in time.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>With oar strokes <qex>timing</qex> to their song.</q> <rj><qau>Whittier.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To pass time; to delay.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Timeful</ent><br/
<hw>Time"ful</hw> <pr>(t<imac/m"f<usdot/l)</pr>, <pos>a.</pos> <def>Seasonable; timely; sufficiently early.</def> <mark>[Obs.]</mark>  <rj><au>Sir W. Raleigh.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Time-honored</ent><br/
<hw>Time"-hon`ored</hw> <pr>(t<imac/m"<ocr/n`<etil/rd)</pr>, <pos>a.</pos> <def>Honored for a long time; venerable, and worthy of honor, by reason of antiquity, or long continuance.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Timekeeper</ent><br/
<hw>Time"keep`er</hw> <pr>(t<imac/m"k<emac/p`<etil/r)</pr>, <pos>n.</pos> <sn>1.</sn> <def>A clock, watch, or other chronometer; a timepiece.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A person who keeps, marks, regulates, or determines the time.</def> Specifically: --<br/
[<source>1913 Webster</source>]</p>

<p><sd>(a)</sd> <def>A person who keeps a record of the time spent by workmen at their work.</def><br/
[<source>1913 Webster</source>]</p>

<p><sd>(b)</sd> <def>One who gives the time for the departure of conveyances.</def><br/
[<source>1913 Webster</source>]</p>

<p><sd>(c)</sd> <def>One who marks the time in musical performances.</def><br/
[<source>1913 Webster</source>]</p>

<p><sd>(d)</sd> <def>One appointed to mark and declare the time of participants in races or other contests.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Timeless</ent><br/
<hw>Time"less</hw> <pr>(t<imac/m"l<ecr/s)</pr>, <pos>a.</pos> <sn>1.</sn> <def>Done at an improper time; unseasonable; untimely.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>Nor fits it to prolong the heavenly feast<br/
<qex>Timeless</qex>, indecent.</q> <rj><qau>Pope.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Done or occurring before the proper time; premature; immature; <as>as, a <ex>timeless</ex> grave</as>.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>Must I behold thy <qex>timeless</qex>, cruel death?</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>Having no end; interminable; unending.</def> <ldquo/<xex>Timeless</xex> night and chaos.<rdquo/  <rj><au>Young.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Timelessly</ent><br/
<hw>Time"less*ly</hw>, <pos>adv.</pos> <def>In a timeless manner; unseasonably.</def> <mark>[R.]</mark>  <rj><au>Milton.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Timeliness</ent><br/
<hw>Time"li*ness</hw> <pr>(t<imac/m"l<emac/*n<ecr/s)</pr>, <pos>n.</pos> <def>The quality or state of being timely; seasonableness; opportuneness.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Timeling</ent><br/
<hw>Time"ling</hw> <pr>(t<imac/m"l<icr/ng)</pr>, <pos>n.</pos> <def>A timeserver.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Timely</ent><br/
<hw>Time"ly</hw> <pr>(t<imac/m"l<emac/)</pr>, <pos>a.</pos> <amorph>[<pos>Compar.</pos> <adjf>Timelier</adjf> <pr>(t<imac/m"l<emac/*<etil/r)</pr>; <pos>superl.</pos> <adjf>Timeliest</adjf>.]</amorph> <sn>1.</sn> <def>Being or occurring in good time; sufficiently early; seasonable.</def> <ldquo/The <xex>timely</xex> dew of sleep.<rdquo/  <rj><au>Milton.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Keeping time or measure.</def>  <rj><au>Spenser.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Timely</ent><br/
<hw>Time"ly</hw>, <pos>adv.</pos> <def>Early; soon; in good season.</def><br/
[<source>1913 Webster</source>]</p>

<p><q><qex>Timely</qex> advised, the coming evil shun.</q> <rj><qau>Prior.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Thanks to you,<br/
That called me <qex>timelier</qex> than my purpose hither,<br/
For I have gained by it.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Timenoguy</ent><br/
<hw>Ti*men"o*guy</hw> <pr>(t<isl/*m<ecr/n"<osl/*g<imac/)</pr>, <pos>n.</pos> <fld>(Naut.)</fld> <def>A rope carried taut between or over obstacles likely to engage or foul the running rigging in working a ship.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Timeous</ent><br/
<hw>Time"ous</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Timely; seasonable.</def> <mark>[R. or Scot.]</mark> -- <wordforms><wf>Time"ous*ly</wf>, <pos>adv.</pos> <mark>[R. or Scot.]</mark></wordforms><br/
[<source>1913 Webster</source>]</p>

<p><ent>Timepiece</ent><br/
<hw>Time"piece`</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A clock, watch, or other instrument, to measure or show the progress of time; a chronometer.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Timepleaser</ent><br/
<hw>Time"pleas`er</hw> <pr>(?)</pr>, <pos>n.</pos> <def>One who complies with prevailing opinions, whatever they may be; a timeserver.</def><br/
[<source>1913 Webster</source>]</p>

<p><q><qex>Timepleasers</qex>, flatterers, foes to nobleness.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Time policy</ent><br/
<hw>Time policy</hw>. <fld>(Insurance)</fld> <def>A policy limited to become void at a specified time; -- often contrasted with <contr>voyage policy</contr>.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Timer</ent><br/
<hw>Tim"er</hw> <pr>(?)</pr>, <pos>n.</pos> <sn>1.</sn> <def>A timekeeper; especially, a watch by which small intervals of time can be measured; a kind of stop watch. It is used for timing the speed of horses, machinery, etc.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Internal-combustion Engines)</fld> <def>In electric ignition, an adjustable device for automatically timing the spark.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Timesaving</ent><br/
<hw>Time"sav`ing</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Saving time; <as>as, a <ex>timesaving</ex> expedient</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Timeserver</ent><br/
<hw>Time"serv`er</hw> <pr>(?)</pr>, <pos>n.</pos> <def>One who adapts his opinions and manners to the times; one who obsequiously compiles with the ruling power; -- now used only in a bad sense.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Timeserving</ent><br/
<hw>Time"serv`ing</hw>, <pos>a.</pos> <def>Obsequiously complying with the spirit of the times, or the humors of those in power.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Timeserving</ent><br/
<hw>Time"serv`ing</hw>, <pos>n.</pos> <def>An obsequious compliance with the spirit of the times, or the humors of those in power, which implies a surrender of one's independence, and sometimes of one's integrity.</def><br/
[<source>1913 Webster</source>]</p>

<p><syn><b>Syn.</b> -- Temporizing.</syn> <usage> -- <er>Timeserving</er>, <er>Temporizing</er>. Both these words are applied to the conduct of one who adapts himself servilely to times and seasons. A <xex>timeserver</xex> is rather active, and a <xex>temporizer</xex>, passive. One whose policy is <xex>timeserving</xex> comes forward to act upon principles or opinions which may promote his advancement; one who is <xex>temporizing</xex> yields to the current of public sentiment or prejudice, and shrinks from a course of action which might injure him with others. The former is dishonest; the latter is weak; and both are contemptible.</usage><br/
[<source>1913 Webster</source>]</p>

<p><q>Trimming and <qex>timeserving</qex>, which are but two words for the same thing, . . . produce confusion.</q> <rj><qau>South.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>[I] pronounce thee . . . a hovering <qex>temporizer</qex>, that<br/
Canst with thine eyes at once see good and evil,<br/
Inclining to them both.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Time signature</ent><br/
<hw>Time signature</hw>. <fld>(Music)</fld> <def>A sign at the beginning of a composition or movement, placed after the key signature, to indicate its time or meter. Also called <altname>rhythmical signature</altname>. It is in the form of a fraction, of which the denominator indicates the kind of note taken as time unit for the beat, and the numerator, the number of these to the measure.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Time-table</ent><br/
<hw>Time"-ta`ble</hw> <pr>(?)</pr>, <pos>n.</pos> <sn>1.</sn> <def>A tabular statement of the time at which, or within which, several things are to take place, as the recitations in a school, the departure and arrival of railroad trains or other public conveyances, the rise and fall of the tides, etc.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Railroad)</fld> <def>A plane surface divided in one direction with lines representing hours and minutes, and in the other with lines representing miles, and having diagonals (usually movable strings) representing the speed and position of various trains.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <fld>(Mus.)</fld> <def>A table showing the notation, length, or duration of the several notes.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Timid</ent><br/
<hw>Tim"id</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>timidus</ets>, fr. <ets>timere</ets> to fear; cf. Skr. <ets>tam</ets> to become breathless, to become stupefief: cf. F. <ets>timide</ets>.]</ety> <def>Wanting courage to meet danger; easily frightened; timorous; not bold; fearful; shy.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Poor is the triumph o'er the <qex>timid</qex> hare.</q> <rj><qau>Thomson.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><syn><b>Syn.</b> -- Fearful; timorous; afraid; cowardly; pusillanimous; faint-hearted; shrinking; retiring.</syn><br/
[<source>1913 Webster</source>]</p>

<p>-- <wordforms><wf>Tim"id*ly</wf>, <pos>adv.</pos> -- <wf>Tim"id*ness</wf>, <pos>n.</pos></wordforms><br/
[<source>1913 Webster</source>]</p>

<p><ent>Timidity</ent><br/
<hw>Ti*mid"i*ty</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>timiditas</ets>: cf. F. <ets>timidit<eacute/</ets>.]</ety> <def>The quality or state of being timid; timorousness; timidness.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Timidous</ent><br/
<hw>Tim"id*ous</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Timid.</def> <mark>[Obs.]</mark>  <rj><au>Hudibras.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Timist</ent><br/
<hw>Tim"ist</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Written also <ets>timeist</ets>.]</ety> <sn>1.</sn> <fld>(Mus.)</fld> <def>A performer who keeps good time.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A timeserver.</def> <mark>[Obs.]</mark>  <rj><au>Overbury.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Timmer</ent><br/
<hw>Tim"mer</hw> <pr>(?)</pr>, <pos>n.</pos> <def>Same as 1st <er>Timber</er>.</def> <mark>[Scot.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Timocracy</ent><br/
<hw>Ti*moc"ra*cy</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Gr. <?/; <?/ honor, worth (fr. <?/ to honor) + <?/ to govern: cf. F. <ets>timocratie</ets>.]</ety> <fld>(Gr. Antiq.)</fld> <sd>(a)</sd> <def>A state in which the love of honor is the ruling motive.</def> <sd>(b)</sd> <def>A state in which honors are distributed according to a rating of property.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Timocratic</ent><br/
<hw>Ti`mo*crat"ic</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Belonging to, or constituted by, timocracy.</def>  <rj><au>Sir G. C. Lewis.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Timoneer</ent><br/
<hw>Tim`o*neer"</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F. <ets>timonier</ets>, fr. <ets>timon</ets> a helm, fr. L. <ets>temo</ets>, <ets>-onis</ets>, a pole.]</ety> <def>A helmsman.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Timorous</ent><br/
<hw>Tim"or*ous</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[LL. <ets>timorosus</ets>, from L. <ets>timor</ets> fear; akin to <ets>timere</ets> to fear. See <er>Timid</er>.]</ety> <sn>1.</sn> <def>Fearful of danger; timid; deficient in courage.</def>  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Indicating, or caused by, fear; <as>as, <ex>timorous</ex> doubts</as>.</def> <ldquo/The <xex>timorous</xex> apostasy of chuchmen.<rdquo/  <rj><au>Milman.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p>-- <wordforms><wf>Tim"or*ous*ly</wf>, <pos>adv.</pos> -- <wf>Tim"or*ous*ness</wf>, <pos>n.</pos></wordforms><br/
[<source>1913 Webster</source>]</p>

<p><ent>Timorsome</ent><br/
<hw>Tim"or*some</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Easily frightened; timorous.</def> <altsp>[Written also <asp>timersome</asp>.]</altsp> <mark>[Scot.]</mark>  <rj><au>Sir W. Scott.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Timothy grass</ent><br/
<ent>Timothy</ent><br/
<mhw>{ <hw>Tim"o*thy</hw> <pr>(?)</pr>, <pos>n.</pos>, <it>or</it> <hw>Tim"o*thy grass`</hw> <pr>(?)</pr> }</mhw>. <ety>[From <ets>Timothy</ets> Hanson, who carried the seed from New England to Maryland about 1720.]</ety> <fld>(Bot.)</fld> <def>A kind of grass (<spn>Phleum pratense</spn>) with long cylindrical spikes; -- called also <altname>herd's grass</altname>, in England, <altname>cat's-tail grass</altname>, and <altname>meadow cat's-tail grass</altname>. It is much prized for fodder. See <xex>Illustration</xex> in Appendix.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Timothy</ent><br/
<hw>Tim"o*thy</hw>  <fld>(Bible) </fld> <def>A disciple and companion of <person>St. Paul</person>.  He was the son of a Greek and a Jewess, and his home was either at <city>Derbe</city>, or <city>Lystra</city> in Lycaonia.  <persfn>Paul</persfn> set him apart as a minister of the new gospel, and after preaching in Macedonia and Achaia, he went, at <persfn>Paul's</persfn> request to <city>Ephesus</city>, and accompanied the apostle to <city>Jerusalem</city>.  It was to him that the two epistles to <ex>Timothy</ex> were addressed by the apostle <persfn>Paul</persfn>. According to tradition, <ex>Timothy</ex> suffered martyrdom under <persfn>Domitian</persfn>.</def> <au>The Student's Cyclopedia, 1897.</au><br/
[<source>PJC</source>]</p>

<p><ent>Timour</ent><br/
<hw>Ti*mour"</hw> <pr>(t<emac/*m<oomac/r")</pr>, <pos>prop. n.</pos> <def>See <er>Tamerlane</er>.</def><br/
<syn><b>Syn. --</b> Tamerlane, Timur.</syn><br/
[<source>PJC</source>]</p>

<p><ent>Timous</ent><br/
<hw>Tim"ous</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[Cf. <er>Timeous</er>.]</ety> <def>Timely; seasonable.</def> <mark>[Obs.]</mark> <au>Bacon.</au> -- <wordforms><wf>Tim"ous*ly</wf>, <pos>adv.</pos> <mark>[Obs.]</mark></wordforms><br/
[<source>1913 Webster</source>]</p>

<p><ent>Timpano</ent><br/
||<hw>Tim"pa*no</hw> <pr>(?)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Timpani</plw> <pr>(#)</pr>.</plu> <ety>[It.]</ety> <fld>(Mus.)</fld> <def>See <er>Tympano</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Timur-leng</ent><br/
<ent>Timur</ent><br/
<mhw><hw>Timur</hw>, <hw>Timur-leng</hw></mhw> <pos>prop. n.</pos> <def>See <er>Tamerlane</er>.</def><br/
<syn><b>Syn. --</b> Tamerlane, Timour.</syn><br/
[<source>PJC</source>]</p>

<p><ent>Tim-whiskey</ent><br/
<hw>Tim"-whis`key</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A kind of carriage. See <er>Whiskey</er>.</def>  <rj><au>Southery.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tin</ent><br/
<hw>Tin</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[As. <ets>tin</ets>; akin to D. <ets>tin</ets>, G. <ets>zinn</ets>, OHG. <ets>zin</ets>, Icel. & Dan. <ets>tin</ets>, Sw. <ets>tenn</ets>; of unknown origin.]</ety> <sn>1.</sn> <fld>(Chem.)</fld> <def>An elementary substance found as an oxide in the mineral cassiterite, and reduced as a soft silvery-white crystalline metal, with a tinge of yellowish-blue, and a high luster.  It is malleable at ordinary temperatures, but brittle when heated. It is softer than gold and can be beaten out into very thin strips called tinfoil. It is ductile at 2120, when it can be drawn out into wire which is not very tenacious; it melts at 4420, and at a higher temperature burns with a brilliant white light. Air and moisture act on tin very slightly. The peculiar properties of tin, especially its malleability, its brilliancy and the slowness with which it rusts make it very serviceable.  With other metals it forms valuable alloys, as bronze, gun metal, bell metal, pewter and solder.  It is not easily oxidized in the air, and is used chiefly to coat iron to protect it from rusting, in the form of tin foil with mercury to form the reflective surface of mirrors, and in solder, bronze, speculum metal, and other alloys. Its compounds are designated as <xex>stannous</xex>, or <xex>stannic</xex>. Symbol Sn (<xex>Stannum</xex>). Atomic weight 117.4.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Thin plates of iron covered with tin; tin plate.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>Money.</def> <mark>[Cant]</mark>  <rj><au>Beaconsfield.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Block tin</b></col> <fld>(Metal.)</fld>, <cd>commercial tin, cast into blocks, and partially refined, but containing small quantities of various impurities, as copper, lead, iron, arsenic, etc.; solid tin as distinguished from tin plate; -- called also <altname>bar tin</altname>.</cd> -- <col><b>Butter of tin</b></col>. <fld>(Old Chem.)</fld> <cd>See <cref>Fuming liquor of Libavius</cref>, under <er>Fuming</er>.</cd> -- <col><b>Grain tin</b></col>. <fld>(Metal.)</fld> <cd>See under <er>Grain</er>.</cd> -- <col><b>Salt of tin</b></col> <fld>(Dyeing)</fld>, <cd>stannous chloride, especially so called when used as a mordant.</cd> -- <col><b>Stream tin</b></col>. <cd>See under <er>Stream</er>.</cd> -- <col><b>Tin cry</b></col> <fld>(Chem.)</fld>, <cd>the peculiar creaking noise made when a bar of tin is bent. It is produced by the grating of the crystal granules on each other.</cd> -- <col><b>Tin foil</b></col>, <cd>tin reduced to a thin leaf.</cd> -- <col><b>Tin frame</b></col> <fld>(Mining)</fld>, <cd>a kind of buddle used in washing tin ore.</cd> -- <mcol><col><b>Tin liquor</b></col>, <col><b>Tin mordant</b></col></mcol> <fld>(Dyeing)</fld>, <cd>stannous chloride, used as a mordant in dyeing and calico printing.</cd> -- <col><b>Tin penny</b></col>, <cd>a customary duty in England, formerly paid to tithingmen for liberty to dig in tin mines.</cd> <mark>[Obs.]</mark> <au>Bailey.</au> -- <col><b>Tin plate</b></col>, <cd>thin sheet iron coated with tin.</cd> -- <col><b>Tin pyrites</b></col>. <cd>See <er>Stannite</er>.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tin</ent><br/
<hw>Tin</hw> <pr>(?)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Tinned</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Tinning</conjf>.]</vmorph> <def>To cover with tin or tinned iron, or to overlay with tin foil.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tinamides</ent><br/
||<hw>Ti*nam"i*des</hw> <pr>(?)</pr>, <pos>n. pl.</pos> <ety>[NL.]</ety> <fld>(Zool.)</fld> <def>A division of struthious birds, including the tinamous.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tinamou</ent><br/
<hw>Tin"a*mou</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[From the native name: cf. F. <ets>tinamous</ets>.]</ety> <fld>(Zool.)</fld> <def>Any one of several species of South American birds belonging to <gen>Tinamus</gen> and allied genera.</def><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ In general appearance and habits they resemble grouse and partridges, but in anatomical characters they are allied to the ostriches and other struthious birds. Their wings are of moderate length, and they are able to fly a considerable distance.</note><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tincal</ent><br/
<hw>Tin"cal</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Ar., Per. & Hind. <ets>tink<amac/r</ets>; cf. Malay <ets>tingkal</ets>; all fr. Skr. <ets><?/a<?/ka<?/a</ets>.  Cf. <er>Altincar</er>.]</ety> <fld>(Chem.)</fld> <def>Crude native borax, formerly imported from Thibet. It was once the chief source of boric compounds.  Cf. <er>Borax</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tinchel</ent><br/
<hw>Tin"chel</hw> <pr>(?)</pr>, <pos>n.</pos> <altsp>[Written also <asp>tinchill</asp>.]</altsp> <ety>[Gael. <ets>timchioll</ets> a circuit, compass.]</ety> <def>A circle of sportsmen, who, by surrounding an extensive space and gradually closing in, bring a number of deer and game within a narrow compass.</def> <mark>[Scot.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>We'll quell the savage mountaineer,<br/
As their <qex>tinchel</qex> cows the game!</q> <rj><qau>Sir W. Scott.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tinct</ent><br/
<hw>Tinct</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>tinctus</ets>, p. p. of <ets>tingere</ets> to tinge. See <er>Tinge</er>.]</ety> <def>Tined; tinged.</def> <mark>[Archaic]</mark>  <rj><au>Spenser.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tinct</ent><br/
<hw>Tinct</hw>, <pos>n.</pos> <ety>[See <er>Tint</er>.]</ety> <def>Color; tinge; tincture; tint.</def> <mark>[Archaic]</mark> <ldquo/Blue of heaven's own <xex>tinct</xex>.<rdquo/  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>All the devices blazoned on the shield,<br/
In their own <qex>tinct</qex>.</q> <rj><qau>Tennyson.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tinct</ent><br/
<hw>Tinct</hw>, <pos>v. t.</pos> <ety>[See <er>Tinge</er>.]</ety> <def>To color or stain; to imblue; to tint.</def> <mark>[Archaic]</mark>  <rj><au>Bacon.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tinctorial</ent><br/
<hw>Tinc*to"ri*al</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>tinctorius</ets>, from <ets>tinctor</ets> a dyer, <ets>tingere</ets>, <ets>tinctum</ets>, to dye: cf. F. <ets>tinctorial</ets>. See <er>Tinge</er>.]</ety> <def>Of or relating to color or colors; imparting a color; <as>as, <ex>tinctorial</ex> matter</as>.</def>  <rj><au>Ure.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tincture</ent><br/
<hw>Tinc"ture</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>tinctura</ets> a dyeing, from <ets>tingere</ets>, <ets>tinctum</ets>, to tinge, dye: cf. OE. <ets>tainture</ets>, <ets>teinture</ets>, F. <ets>teinture</ets>, L. <ets>tinctura</ets>. See <er>Tinge</er>.]</ety> <sn>1.</sn> <def>A tinge or shade of color; a tint; <as>as, a <ex>tincture</ex> of red</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Her.)</fld> <def>One of the metals, colors, or furs used in armory.</def><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ There are two metals: gold, called <xex>or</xex>, and represented in engraving by a white surface covered with small dots; and silver, called <xex>argent</xex>, and represented by a plain white surface. The colors and their representations are as follows: red, called <xex>gules</xex>, or a shading of vertical lines; blue, called <xex>azure</xex>, or horizontal lines; black, called <xex>sable</xex>, or horizontal and vertical lines crossing; green, called <xex>vert</xex>, or diagonal lines from dexter chief corner; purple, called <xex>purpure</xex>, or diagonal lines from sinister chief corner. The furs are <xex>ermine</xex>, <xex>ermines</xex>, <xex>erminois</xex>, <xex>pean</xex>, <xex>vair</xex>, <xex>counter vair</xex>, <xex>potent</xex>, and <xex>counter potent</xex>. See <xex>Illustration</xex> in Appendix.</note><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>The finer and more volatile parts of a substance, separated by a solvent; an extract of a part of the substance of a body communicated to the solvent.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <fld>(Med.)</fld> <def>A solution (commonly colored) of medicinal substance in alcohol, usually more or less diluted; spirit containing medicinal substances in solution.</def><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ According to the United States Pharmacopoeia, the term <xex>tincture</xex> (also called <xex>alcoholic tincture</xex>, and <xex>spirituous tincture</xex>) is reserved for the alcoholic solutions of nonvolatile substances, alcoholic solutions of volatile substances being called <xex>spirits</xex>.</note><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Ethereal tincture</b></col>, <cd>a solution of medicinal substance in ether.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <def>A slight taste superadded to any substance; <as>as, a <ex>tincture</ex> of orange peel</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>6.</sn> <def>A slight quality added to anything; a tinge; <as>as, a <ex>tincture</ex> of French manners</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>All manners take a <qex>tincture</qex> from our own.</q> <rj><qau>Pope.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Every man had a slight <qex>tincture</qex> of soldiership, and scarcely any man more than a slight <qex>tincture</qex>.</q> <rj><qau>Macaulay.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tincture</ent><br/
<hw>Tinc"ture</hw>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Tinctured</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Tincturing</conjf>.]</vmorph> <sn>1.</sn> <def>To communicate a slight foreign color to; to tinge; to impregnate with some extraneous matter.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>A little black paint will <qex>tincture</qex> and spoil twenty gay colors.</q> <rj><qau>I. Watts.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To imbue the mind of; to communicate a portion of anything foreign to; to tinge.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The stain of habitual sin may thoroughly <qex>tincture</qex> all our soul.</q> <rj><qau>Barrow.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tind</ent><br/
<hw>Tind</hw> <pr>(?)</pr>, <pos>v. t.</pos> <ety>[OE. <ets>tenden</ets>, AS. <ets>tendan</ets>; akin to G. <ets>z<uum/nden</ets>, OHG. <ets>zunten</ets>, Icel. <ets>tendra</ets>, Sw. <ets>t<aum/nda</ets>, Dan. <ets>t<ae/nde</ets>, Goth. <ets>tandjan</ets> to kindle, <ets>tundnan</ets> to be kindled, to burn.  Cf. <er>Tinder</er>.]</ety> <def>To kindle.</def> <mark>[Obs.]</mark>  <rj><au>Bp. Sanderson.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tindal</ent><br/
<hw>Tin"dal</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[From the native name: cf. Malayalam <ets>ta<?/<?/al</ets>.]</ety> <sn>1.</sn> <def>A petty officer among lascars, or native East Indian sailors; a boatswain's mate; a cockswain.</def> <mark>[India]</mark>  <rj><au>Malcom.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>An attendant on an army.</def> <mark>[India]</mark>  <rj><au>Simmonds.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tinder</ent><br/
<hw>Tin"der</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[OE. <ets>tinder</ets>, <ets>tunder</ets>, AS. <ets>tynder</ets>, <ets>tyndre</ets>; akin to <ets>tendan</ets> to kindle, D. <ets>tonder</ets> tinder, G. <ets>zunder</ets>, OHG. <ets>zuntara</ets>, <ets>zuntra</ets>, Icel. <ets>tundr</ets>, Sw. <ets>tunder</ets>, Dan. <ets>t<oum/nder</ets>. See <er>Tind</er>.]</ety> <def>Something very inflammable, used for kindling fire from a spark, as scorched linen.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>German tinder</b></col>. <cd>Same as <er>Amadou</er>.</cd> -- <col><b>Tinder box</b></col>, <cd>a box in which tinder is kept.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><-- p. 1511 --><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tine</ent><br/
<hw>Tine</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[See <er>Teen</er> affliction.]</ety> <def>Trouble; distress; teen.</def> <mark>[Obs.]</mark> <ldquo/Cruel winter's <xex>tine</xex>.<rdquo/  <rj><au>Spenser.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tine</ent><br/
<hw>Tine</hw>, <pos>v. t.</pos> <ety>[See <er>Tind</er>.]</ety> <def>To kindle; to set on fire.</def> <mark>[Obs.]</mark> <see>See <er>Tind</er>.</see> <ldquo/To <xex>tine</xex> the cloven wood.<rdquo/  <rj><au>Dryden.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Coals of contention and hot vengeance <qex>tind</qex>.</q> <rj><qau>Spenser.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tine</ent><br/
<hw>Tine</hw>, <pos>v. i.</pos> <ety>[Cf. <er>Tine</er> distress, or <er>Tine</er> to kindle.]</ety> <def>To kindle; to rage; to smart.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>Ne was there slave, ne was there medicine<br/
That mote recure their wounds; so inly they did <qex>tine</qex>.</q> <rj><qau>Spenser.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tine</ent><br/
<hw>Tine</hw>, <pos>v. t.</pos> <ety>[AS. <ets>t<?/nan</ets>, from <ets>t<?/n</ets> an inclosure. See <er>Town</er>.]</ety> <def>To shut in, or inclose.</def> <mark>[Prov. Eng.]</mark>  <rj><au>Halliwell.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tine</ent><br/
<hw>Tine</hw>, <pos>n.</pos> <ety>[OE. <ets>tind</ets>, AS. <ets>tind</ets>; akin to MHG. <ets>zint</ets>, Icel. <ets>tindr</ets>, Sw. <ets>tinne</ets>, and probably to G. <ets>zinne</ets> a pinnacle, OHG. <ets>zinna</ets>, and E. <ets>tooth</ets>. See <er>Tooth</er>.]</ety> <def>A tooth, or spike, as of a fork; a prong, as of an antler.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tinea</ent><br/
||<hw>Tin"e*a</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L., a worm, a moth.]</ety> <sn>1.</sn> <fld>(Med.)</fld> <def>A name applied to various skin diseases, but especially to ringworm. See <er>Ringworm</er>, and <er>Sycosis</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Zool.)</fld> <def>A genus of small Lepidoptera, including the clothes moths and carpet moths.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tinean</ent><br/
<hw>Tin"e*an</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Zool.)</fld> <def>Any species of Tinea, or of the family <fam>Tineidae</fam>, which includes numerous small moths, many of which are injurious to woolen and fur goods and to cultivated plants. Also used adjectively.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tined</ent><br/
<hw>Tined</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Furnished with tines; <as>as, a three-<ex>tined</ex> fork</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tineid</ent><br/
<hw>Tin"e*id</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Zool.)</fld> <def>Same as <er>Tinean</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tineman</ent><br/
<hw>Tine"man</hw> <pr>(?)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Tinemen</plw> <pr>(#)</pr>.</plu> <ety>[Probably akin to <ets>tine</ets> to shut or inclose.]</ety> <fld>(O. Eng. Forest Law)</fld> <def>An officer of the forest who had the care of vert and venison by night.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tinet</ent><br/
<hw>Ti"net</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[From <er>Tine</er> to shut in, inclose.]</ety> <def>Brushwood and thorns for making and repairing hedges.</def> <mark>[Obs. Eng.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Ting</ent><br/
<hw>Ting</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[An imitative word.  Cf. <er>Tink</er>.]</ety> <def>A sharp sound, as of a bell; a tinkling.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Ting</ent><br/
<hw>Ting</hw>, <pos>v. i.</pos> <def>To sound or ring, as a bell; to tinkle.</def> <mark>[R.]</mark>  <rj><au>Holland.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Ting</ent><br/
||<hw>Ting</hw>, <pos>n.</pos> <def>The apartment in a Chinese temple where the idol is kept.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tinge</ent><br/
<hw>Tinge</hw> <pr>(?)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Tinged</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Tingeing</conjf> <pr>(?)</pr>.]</vmorph> <ety>[L. <ets>tingere</ets>, <ets>tinctum</ets>, to dye, stain, wet; akin to Gr. <?/, and perhaps to G. <ets>tunken</ets> to dip, OHG. <ets>tunch<omac/n</ets>, <ets>dunch<omac/n</ets>, <ets>thunk<omac/n</ets>.  Cf. <er>Distain</er>, <er>Dunker</er>, <er>Stain</er>, <er>Taint</er> a stain, to stain, <er>Tincture</er>, <er>Tint</er>.]</ety> <def>To imbue or impregnate with something different or foreign; <as>as, to <ex>tinge</ex> a decoction with a bitter taste</as>; to affect in some degree with the qualities of another substance, either by mixture, or by application to the surface; especially, to color slightly; to stain; <as>as, to <ex>tinge</ex> a blue color with red; an infusion <ex>tinged</ex> with a yellow color by saffron.</as></def><br/
[<source>1913 Webster</source>]</p>

<p><q>His [Sir Roger's] virtues, as well as imperfections, are <qex>tinged</qex> by a certain extravagance.</q> <rj><qau>Addison.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><syn><b>Syn.</b> -- To color; dye; stain.</syn><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tinge</ent><br/
<hw>Tinge</hw>, <pos>n.</pos> <def>A degree, usually a slight degree, of some color, taste, or something foreign, infused into another substance or mixture, or added to it; tincture; color; dye; hue; shade; taste.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>His notions, too, respecting the government of the state, took a <qex>tinge</qex> from his notions respecting the government of the church.</q> <rj><qau>Macaulay.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tingent</ent><br/
<hw>Tin"gent</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>tingens</ets>, p. pr. of <ets>tingere</ets> to tinge. See <er>Tinge</er>.]</ety> <def>Having the power to tinge.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>As for the white part, it appears much less enriched with the <qex>tingent</qex> property.</q> <rj><qau>Boyle.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tinger</ent><br/
<hw>Tin"ger</hw> <pr>(?)</pr>, <pos>n.</pos> <def>One who, or that which, tinges.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tingid</ent><br/
<hw>Tin"gid</hw> <pr>(?)</pr>, <pos>a.</pos> <fld>(Zool.)</fld> <def>Of or pertaining to the genus Tingis.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tingis</ent><br/
||<hw>Tin"gis</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[NL.]</ety> <fld>(Zool.)</fld> <def>A genus of small hemipterous insects which injure trees by sucking the sap from the leaves. See <xex>Illustration</xex> in Appendix.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tingle</ent><br/
<hw>Tin"gle</hw> <pr>(?)</pr>, <pos>v. i.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Tingled</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Tingling</conjf> <pr>(?)</pr>.]</vmorph> <ety>[Freq. of <ets>ting</ets>.  Cf. <er>Tinkle</er>.]</ety> <sn>1.</sn> <def>To feel a kind of thrilling sensation, as in hearing a shrill sound.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>At which both the ears of every one that heareth it shall tingle.</q> <rj><qau>1 Sam. iii. 11.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To feel a sharp, thrilling pain.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The pale boy senator yet <qex>tingling</qex> stands.</q> <rj><qau>Pope.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>To have, or to cause, a sharp, thrilling sensation, or a slight pricking sensation.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>They suck pollution through their <qex>tingling</qex> vein.</q> <rj><qau>Tickell.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tink</ent><br/
<hw>Tink</hw> <pr>(?)</pr>, <pos>v. i.</pos> <ety>[OE. <ets>tinken</ets>; of imitative origin.  Cf. <er>Ting</er> a tinkling, <er>Tinker</er>.]</ety> <def>To make a sharp, shrill noise; to tinkle.</def>  <rj><au>Wyclif (1 Cor. xiii. 1).</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tink</ent><br/
<hw>Tink</hw>, <pos>n.</pos> <def>A sharp, quick sound; a tinkle.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tinker</ent><br/
<hw>Tink"er</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[From <er>Tink</er>, because the tinker's way of proclaiming his trade is to beat a kettle, or because in his work he makes a <ets>tinkling</ets> noise. <ets>Johnson</ets>.]</ety> <sn>1.</sn> <def>A mender of brass kettles, pans, and other metal ware.</def> <ldquo/Tailors and <xex>tinkers</xex>.<rdquo/  <rj><au>Piers Plowman.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>One skilled in a variety of small mechanical work.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <fld>(Ordnance)</fld> <def>A small mortar on the end of a staff.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <fld>(Zool.)</fld> <sd>(a)</sd> <def>A young mackerel about two years old.</def> <sd>(b)</sd> <def>The chub mackerel.</def> <sd>(c)</sd> <def>The silversides.</def> <sd>(d)</sd> <def>A skate.</def> <mark>[Prov. Eng.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <fld>(Zool.)</fld> <def>The razor-billed auk.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tinker</ent><br/
<hw>Tink"er</hw>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Tinkered</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Tinkering</conjf>.]</vmorph> <def>To mend or solder, as metal wares; hence, more generally, to mend.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tinker</ent><br/
<hw>Tink"er</hw>, <pos>v. i.</pos> <def>To busy one's self in mending old kettles, pans, etc.; to play the tinker; to be occupied with small mechanical works.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tinkering</ent><br/
<hw>Tink"er*ing</hw>, <pos>n.</pos> <def>The act or work of a tinker.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tinkerly</ent><br/
<hw>Tink"er*ly</hw>, <pos>a.</pos> <def>After the manner of a tinker.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tinkle</ent><br/
<ent>Tinkershire</ent><br/
<mhw>{ <hw>Tink"er*shire</hw> <pr>(?)</pr>, <hw>Tin"kle</hw> <pr>(?)</pr> }</mhw>, <pos>n.</pos> <fld>(Zool.)</fld> <def>The common guillemot.</def> <mark>[Prov. Eng.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tinkle</ent><br/
<hw>Tin"kle</hw> <pr>(?)</pr>, <pos>v. i.</pos> <ety>[Freq. of <ets>tink</ets>. See <er>Tink</er>, <er>Tingle</er>.]</ety> <sn>1.</sn> <def>To make, or give forth, small, quick, sharp sounds, as a piece of metal does when struck; to clink.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>As sounding brass, or a <qex>tinkling</qex> cymbal.</q> <rj><qau>1 Cor. xiii. 1.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>The sprightly horse<br/
Moves to the music of his <qex>tinkling</qex> bells.</q> <rj><qau>Dodsley.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To hear, or resound with, a small, sharp sound.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>And his ears <qex>tinkled</qex>, and the color fled.</q> <rj><qau>Dryden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tinkle</ent><br/
<hw>Tin"kle</hw>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Tinkled</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Tinkling</conjf>.]</vmorph> <def>To cause to clonk, or make small, sharp, quick sounds.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tinkle</ent><br/
<hw>Tin"kle</hw>, <pos>n.</pos> <def>A small, sharp, quick sound, as that made by striking metal.</def>  <rj><au>Cowper.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tinkler</ent><br/
<hw>Tin"kler</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A tinker.</def> <mark>[Prov. Eng.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tinkling</ent><br/
<hw>Tin"kling</hw> <pr>(?)</pr>, <pos>n.</pos> <sn>1.</sn> <def>A tinkle, or succession of tinkles.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Drowsy <qex>tinklings</qex> lull the distant folds.</q> <rj><qau>Gray.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Zool.)</fld> <def>A grackle (<spn>Quiscalus crassirostris</spn>) native of Jamaica. It often associates with domestic cattle, and rids them of insects.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tinman</ent><br/
<hw>Tin"man</hw> <pr>(?)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Tinmen</plw> <pr>(<?/)</pr>.</plu> <def>A manufacturer of tin vessels; a dealer in tinware.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tinmouth</ent><br/
<hw>Tin"mouth`</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Zool.)</fld> <def>The crappie.</def> <mark>[U. S.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tinned</ent><br/
<hw>Tinned</hw> <pr>(?)</pr>, <pos>a.</pos> <sn>1.</sn> <def>Covered, or plated, with tin; <as>as, a <ex>tinned</ex> roof; <ex>tinned</ex> iron</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Packed in tin cases; canned; <as>as, <ex>tinned</ex> meats</as>.</def>  <rj><au>Cassell (Dict. of Cookery).</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tinnen</ent><br/
<hw>Tin"nen</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Made or consisting of tin.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tinner</ent><br/
<hw>Tin"ner</hw> <pr>(?)</pr>, <pos>n.</pos> <sn>1.</sn> <def>One who works in a tin mine.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>One who makes, or works in, tinware; a tinman.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tinnient</ent><br/
<hw>Tin"ni*ent</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>tinniens</ets>, p. pr. of <ets>tinnire</ets> to ring, tinkle.]</ety> <def>Emitting a clear sound.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tinning</ent><br/
<hw>Tin"ning</hw> <pr>(?)</pr>, <pos>n.</pos> <sn>1.</sn> <def>The act, art, or process of covering or coating anything with melted tin, or with tin foil, as kitchen utensils, locks, and the like.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>The covering or lining of tin thus put on.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tinnitus</ent><br/
||<hw>Tin*ni"tus</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L., fr. <ets>tinnire</ets> to jingle.]</ety> <fld>(Med.)</fld> <def>A ringing, whistling, or other imaginary noise perceived in the ears; -- called also <altname>tinnitus aurium</altname>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tinnock</ent><br/
<hw>Tin"nock</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Zool.)</fld> <def>The blue titmouse.</def> <mark>[Prov. Eng.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tinny</ent><br/
<hw>Tin"ny</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Pertaining to, abounding with, or resembling, tin.</def> <ldquo/The <xex>tinny</xex> strand.<rdquo/  <rj><au>Drayton.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tinsel</ent><br/
<hw>Tin"sel</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F. <ets><eacute/tincelle</ets> a spark, OF. <ets>estincelle</ets>, L. <ets>scintilla</ets>.  Cf. <er>Scintillate</er>, <er>Stencil</er>.]</ety> <sn>1.</sn> <def>A shining material used for ornamental purposes; especially, a very thin, gauzelike cloth with much gold or silver woven into it; also, very thin metal overlaid with a thin coating of gold or silver, brass foil, or the like.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Who can discern the <qex>tinsel</qex> from the gold?</q> <rj><qau>Dryden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Something shining and gaudy; something superficially shining and showy, or having a false luster, and more gay than valuable.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>O happy peasant! O unhappy bard!<br/
His the mere <qex>tinsel</qex>, hers the rich reward.</q> <rj><qau>Cowper.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tinsel</ent><br/
<hw>Tin"sel</hw>, <pos>a.</pos> <def>Showy to excess; gaudy; specious; superficial.</def> <ldquo/<xex>Tinsel</xex> trappings.<rdquo/  <rj><au>Milton.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tinsel</ent><br/
<hw>Tin"sel</hw>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Tinseled</conjf> <pr>(?)</pr> or <conjf>Tinselled</conjf>; <pos>p. pr. & vb. n.</pos> <conjf>Tinseling</conjf> or <conjf>Tinselling</conjf>.]</vmorph> <def>To adorn with tinsel; to deck out with cheap but showy ornaments; to make gaudy.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>She, <qex>tinseled</qex> o'er in robes of varying hues.</q> <rj><qau>Pope.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tinselly</ent><br/
<hw>Tin"sel*ly</hw>, <pos>a.</pos> <def>Like tinsel; gaudy; showy, but cheap.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tinselly</ent><br/
<hw>Tin"sel*ly</hw>, <pos>adv.</pos> <def>In a showy and cheap manner.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tinsmith</ent><br/
<hw>Tin"smith`</hw> <pr>(?)</pr>, <pos>n.</pos> <def>One who works in tin; a tinner.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tinstone</ent><br/
<hw>Tin"stone`</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Min.)</fld> <def>Cassiterite.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tint</ent><br/
<hw>Tint</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[For older <ets>tinct</ets>, fr. L. <ets>tinctus</ets>, p. p. of <ets>tingere</ets> to dye: cf. F. <ets>teinte</ets>, <ets>teint</ets>, It. <ets>tinta</ets>, <ets>tinto</ets>. See <er>Tinge</er>, and cf. <er>Taint</er> to stain, a stain, <er>Tent</er> a kind of wine, <er>Tinto</er>.]</ety> <def>A slight coloring.</def> Specifically: --<br/
[<source>1913 Webster</source>]</p>

<p><sd>(a)</sd> <def>A pale or faint tinge of any color.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Or blend in beauteous <qex>tints</qex> the colored mass.</q> <rj><qau>Pope.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Their vigor sickens, and their <qex>tints</qex> decline.</q> <rj><qau>Harte.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sd>(b)</sd> <def>A color considered with reference to other very similar colors; <as>as, red and blue are different colors, but two shades of scarlet are different <ex>tints</ex></as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sd>(c)</sd> <fld>(Engraving)</fld> <def>A shaded effect produced by the juxtaposition of many fine parallel lines.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Tint tool</b></col> <fld>(Eng.)</fld>, <cd>a species of graver used for cutting the parallel lines which produce tints in engraving.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tint</ent><br/
<hw>Tint</hw>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Tinted</conjf>; <pos>p. pr. & vb. n.</pos> <conjf>Tinting</conjf>.]</vmorph> <def>To give a slight coloring to; to tinge.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tintamar</ent><br/
<hw>Tin`ta*mar"</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F. <ets>tintamarre</ets>.]</ety> <def>A hideous or confused noise; an uproar.</def> <mark>[Obs.]</mark>  <rj><au>Howell.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tinternell</ent><br/
<hw>Tin"ter*nell</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A certain old dance.</def> <mark>[Obs.]</mark>  <rj><au>Halliwell.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tintie</ent><br/
<hw>Tin"tie</hw> <pr>(t<icr/n"t<icr/)</pr>, <pos>n.</pos> <fld>(Zool.)</fld> <def>The wren.</def> <mark>[Prov. Eng.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tintinnabulary</ent><br/
<ent>Tintinnabular</ent><br/
<mhw>{ <hw>Tin`tin*nab"u*lar</hw> <pr>(?)</pr>, <hw>Tin`tin*nab"u*la*ry</hw> <pr>(?)</pr>, }</mhw> <pos>a.</pos> <ety>[L. <ets>tintinnabulum</ets> a little bell, fr. <ets>tintinnare</ets> to ring, to jingle, <ets>tinnire</ets> to jingle.]</ety> <def>Having or making the sound of a bell; tinkling.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tintinnabulation</ent><br/
<hw>Tin`tin*nab`u*la"tion</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A tinkling sound, as of a bell or bells.</def>  <rj><au>Poe.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tintinnabulous</ent><br/
<hw>Tin`tin*nab"u*lous</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Of, pertaining to, or resembling, the tinkling of a bell; having a tinkling sound; tintinnabular.</def>  <rj><au>De Quincey.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tintinnabulum</ent><br/
||<hw>Tin`tin*nab"u*lum</hw> <pr>(?)</pr>, <pos>n.</pos>; <plu>pl. <plw>Tintinnabula</plw> <pr>(#)</pr></plu>. <ety>[L., a bell. See <er>Tintinnabular</er>.]</ety> <def>A bell; also, a set or combination of bells or metal plates used as a musical instrument or as a toy.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Tinto</ent><br/
<hw>Tin"to</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Pg., tinged, fr. L. <ets>tinctus</ets>, p. p. of <ets>tingere</ets> to tinge. See <er>Tint</er>, <pos>n.</pos>]</ety> <def>A red Madeira wine, wanting the high aroma of the white sorts, and, when old, resembling tawny port.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tintometer</ent><br/
<hw>Tint*om"e*ter</hw> <pr>(?)</pr>, <pos>n.</pos>  <ety>[<ets>Tint</ets> + <ets>-meter</ets>.]</ety> <fld>(Physics)</fld> <def>An apparatus for the determination of colors by comparison with arbitrary standards; a colorimeter.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Tintype</ent><br/
<hw>Tin"type`</hw> <pr>(?)</pr>, <pos>n.</pos> <def>Same as <er>Ferrotype</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tinware</ent><br/
<hw>Tin"ware`</hw> <pr>(?)</pr>, <pos>n.</pos> <def>Articles made of tinned iron.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tiny</ent><br/
<hw>Ti"ny</hw> <pr>(?)</pr>, <pos>a.</pos> <amorph>[<pos>Compar.</pos> <adjf>Tinier</adjf> <pr>(?)</pr>; <pos>superl.</pos> <adjf>Tiniest</adjf>.]</amorph> <ety>[Probably fr. <ets>tine</ets>, <ets>teen</ets>, trouble, distress, vexation.]</ety> <def>Very small; little; puny.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>When that I was and a little <qex>tiny</qex> boy.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tip</ent><br/
<hw>Tip</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Akin to D. & Dan. <ets>tip</ets>, LG. & Sw. <ets>tipp</ets>, G. <ets>zipfel</ets>, and probably to E. <ets>tap</ets> a plug, a pipe.]</ety> <sn>1.</sn> <def>The point or extremity of anything; a pointed or somewhat sharply rounded end; the end; <as>as, the <ex>tip</ex> of the finger; the <ex>tip</ex> of a spear</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>To the very <qex>tip</qex> of the nose.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>An end piece or part; a piece, as a cap, nozzle, ferrule, or point, applied to the extreme end of anything; <as>as, a <ex>tip</ex> for an umbrella, a shoe, a gas burner, etc.</as></def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <fld>(Hat Manuf.)</fld> <def>A piece of stiffened lining pasted on the inside of a hat crown.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>A thin, boarded brush made of camel's hair, used by gilders in lifting gold leaf.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <def>Rubbish thrown from a quarry.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tip</ent><br/
<hw>Tip</hw> <pr>(?)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Tipped</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Tipping</conjf>.]</vmorph> <def>To form a point upon; to cover the tip, top, or end of; <as>as, to <ex>tip</ex> anything with gold or silver</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>With truncheon <qex>tipped</qex> with iron head.</q> <rj><qau>Hudibras.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q><qex>Tipped</qex> with jet,<br/
Fair ermines spotless as the snows they press.</q> <rj><qau>Thomson.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tip</ent><br/
<hw>Tip</hw>, <pos>v. t.</pos> <ety>[Cf. LG. <ets>tippen</ets> to tap, Sw. <ets>tippa</ets>, and E. <ets>tap</ets> to strike gently.]</ety> <sn>1.</sn> <def>To strike slightly; to tap.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>A third rogue <qex>tips</qex> me by the elbow.</q> <rj><qau>Swift.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To bestow a gift, or douceur, upon; to give a present to; <as>as, to <ex>tip</ex> a servant</as>.</def> <mark>[Colloq.]</mark>  <rj><au>Thackeray.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>To lower one end of, or to throw upon the end; to tilt; <as>as, to <ex>tip</ex> a cask; to <ex>tip</ex> a cart</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>To tip off</b></col>, <cd>to pour out, as liquor.</cd> -- <col><b>To tip over</b></col>, <cd>to overturn.</cd> -- <col><b>To tip the wink</b></col>, <cd>to direct a wink; to give a hint or suggestion by, or as by, a wink.</cd> <mark>[Slang]</mark> <au>Pope.</au> -- <col><b>To tip up</b></col>, <cd>to turn partly over by raising one end.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tip</ent><br/
<hw>Tip</hw>, <pos>v. i.</pos> <def>To fall on, or incline to, one side.</def>  <rj><au>Bunyan.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>To tip off</b></col>, <cd>to fall off by tipping.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tip</ent><br/
<hw>Tip</hw>, <pos>n.</pos> <ety>[See <er>Tip</er> to strike slightly, and cf. <er>Tap</er> a slight blow.]</ety> <sn>1.</sn> <def>A light touch or blow; a tap.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A gift; a douceur; a fee.</def> <mark>[Colloq.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>A hint, or secret intimation, as to the chances in a horse race, or the like.</def> <mark>[Sporting Cant]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tipcart</ent><br/
<hw>Tip"cart`</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A cart so constructed that the body can be easily tipped, in order to dump the load.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tipcat</ent><br/
<hw>Tip"cat`</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A game in which a small piece of wood pointed at both ends, called a <xex>cat</xex>, is tipped, or struck with a stick or bat, so as to fly into the air.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>In the middle of a game at <qex>tipcat</qex>, he paused, and stood staring wildly upward with his stick in his hand.</q> <rj><qau>Macaulay.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tipper</ent><br/
<hw>Tip"per</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A kind of ale brewed with brackish water obtained from a particular well; -- so called from the first brewer of it, one Thomas <xex>Tipper</xex>.</def> <mark>[Eng.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tippet</ent><br/
<hw>Tip"pet</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[OE. <ets>tipet</ets>, <ets>tepet</ets>, AS. <ets>t<ae/ppet</ets>, probably fr. L. <ets>tapete</ets> tapestry, hangings.  Cf. <er>Tape</er>, <er>Tapestry</er>, <er>Tapet</er>.]</ety> <sn>1.</sn> <def>A cape, or scarflike garment for covering the neck, or the neck and shoulders, -- usually made of fur, cloth, or other warm material.</def>  <rj><au>Chaucer. Bacon.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A length of twisted hair or gut in a fish line.</def> <mark>[Scot.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>A handful of straw bound together at one end, and used for thatching.</def> <mark>[Scot.]</mark>  <rj><au>Jamieson.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Tippet grebe</b></col> <fld>(Zool.)</fld>, <cd>the great crested grebe, or one of several similar species.</cd> -- <col><b>Tippet grouse</b></col> <fld>(Zool.)</fld>, <cd>the ruffed grouse.</cd> -- <col><b>To turn tippet</b></col>, <cd>to change.</cd> <mark>[Obs.]</mark>  <rj><au>B. Jonson.</au></rj></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tipping</ent><br/
<hw>Tip"ping</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Mus.)</fld> <def>A distinct articulation given in playing quick notes on the flute, by striking the tongue against the roof of the mouth; double-tonguing.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tipple</ent><br/
<hw>Tip"ple</hw> <pr>(?)</pr>, <pos>v. i.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Tippled</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Tippling</conjf> <pr>(?)</pr>.]</vmorph> <ety>[From <ets>tip</ets> a small end, or a word akin to it; cf. Norw. <ets>tipla</ets> to tipple, to drip, Prov. E. <ets>tip</ets>, <ets>tiff</ets>, <ets>tift</ets>, a draught of liquor, dial. G. <ets>zipfeln</ets> to eat and drink in small parts. See <er>Tip</er> a point, and cf. <er>Tipsy</er>.]</ety> <def>To drink spirituous or strong liquors habitually; to indulge in the frequent and improper used of spirituous liquors; especially, to drink frequently in small quantities, but without absolute drunkeness.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Few of those who were summoned left their homes, and those few generally found it more agreeable to <qex>tipple</qex> in alehouses than to pace the streets.</q> <rj><qau>Macaulay.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tipple</ent><br/
<hw>Tip"ple</hw>, <pos>v. t.</pos> <sn>1.</sn> <def>To drink, as strong liquors, frequently or in excess.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Himself, for saving charges,<br/
A peeled, sliced onions eats, and <qex>tipples</qex> verjuice.</q> <rj><qau>Dryden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To put up in bundles in order to dry, as hay.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tipple</ent><br/
<hw>Tip"ple</hw>, <pos>n.</pos> <def>Liquor taken in tippling; drink.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Pulque, the national <qex>tipple</qex> of Mexico.</q> <rj><qau>S. B. Griffin.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tipple</ent><br/
<hw>Tip"ple</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Cf. 3d <er>Tip</er>.]</ety> <def>An apparatus by which loaded cars are emptied by tipping; also, the place where such tipping is done.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Tippled</ent><br/
<hw>Tip"pled</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Intoxicated; inebriated; tipsy; drunk.</def> <mark>[R.]</mark>  <rj><au>Dryden.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tippler</ent><br/
<hw>Tip"pler</hw> <pr>(?)</pr>, <pos>n.</pos> <sn>1.</sn> <def>One who keeps a tippling-house.</def> <mark>[Obs.]</mark>  <rj><au>Latimer.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>One who habitually indulges in the excessive use of spirituous liquors, whether he becomes intoxicated or not.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tippling-house</ent><br/
<hw>Tip"pling-house`</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A house in which liquors are sold in drams or small quantities, to be drunk on the premises.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tipsify</ent><br/
<hw>Tip"si*fy</hw> <pr>(?)</pr>, <pos>v. t.</pos> <ety>[<ets>Tipsy</ets> + <ets>-fy</ets>.]</ety> <def>To make tipsy.</def> <mark>[Colloq.]</mark>  <rj><au>Thackeray.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tipsily</ent><br/
<hw>Tip"si*ly</hw>, <pos>adv.</pos> <def>In a tipsy manner; like one tipsy.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tipsiness</ent><br/
<hw>Tip"si*ness</hw>, <pos>n.</pos> <def>The state of being tipsy.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tipstaff</ent><br/
<hw>Tip"staff`</hw> <pr>(?)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Tipstaff</plw> <pr>(<?/)</pr>.</plu> <sn>1.</sn> <def>A staff tipped with metal.</def>  <rj><au>Bacon.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>An officer who bears a staff tipped with metal; a constable.</def>  <rj><au>Macaulay.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tipster</ent><br/
<hw>Tip"ster</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[<ets>Tip</ets> a hint + <ets>-ster</ets>.]</ety> <def>One who makes a practice of giving or selling tips, or private hints or information, esp. for use in gambling upon the probable outcome of events, as horse races.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Tipstock</ent><br/
<hw>Tip"stock`</hw> <pr>(?)</pr>, <pos>n.</pos> <def>The detachable or movable fore part of a gunstock, lying beneath the barrel or barrels, and forming a hold for the left hand.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Tipsy</ent><br/
<hw>Tip"sy</hw> <pr>(?)</pr>, <pos>a.</pos> <amorph>[<pos>Compar.</pos> <adjf>Tipsier</adjf> <pr>(?)</pr>; <pos>superl.</pos> <adjf>Tipsiest</adjf>.]</amorph> <ety>[Akin to <ets>tipple</ets>; cf. Prov. G. <ets>tips</ets> drunkenness, be<ets>tipst</ets> drunk, tipsy. See <er>Tipple</er>.]</ety> <sn>1.</sn> <def>Being under the influence of strong drink; rendered weak or foolish by liquor, but not absolutely or completely drunk; fuddled; intoxicated.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Staggering, as if from intoxication; reeling.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Midnight shout and revelry,<br/
<qex>Tipsy</qex> dance and jollity.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tiptoe</ent><br/
<hw>Tip"toe`</hw> <pr>(?)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Tiptoes</plw> <pr>(<?/)</pr>.</plu> <def>The end, or tip, of the toe.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>He must . . . stand on his <qex>typtoon</qex> [tiptoes].</q> <rj><qau>Chaucer.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Upon his <qex>tiptoes</qex> stalketh stately by.</q> <rj><qau>Spenser.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><mcol><col><b>To be a tiptoe</b></col>, <col><b>To stand a tiptoe</b></col>, <col><b>To stand on tiptoe</b></col> <it>or</it> <col><b>To be on tiptoe</b></col></mcol>, <cd>to be awake or alive to anything; to be roused; to be eager or alert; <as>as, <ex>to be a tiptoe</ex> with expectation</as>.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><-- p. 1512 --><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tiptoe</ent><br/
<hw>Tip"toe`</hw> <pr>(?)</pr>, <pos>a.</pos> <sn>1.</sn> <def>Being on tiptoe, or as on tiptoe; hence, raised as high as possible; lifted up; exalted; also, alert.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Night's candles are burnt out, and jocund day<br/
Stands <qex>tiptoe</qex> on the misty mountain tops.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Above the <qex>tiptoe</qex> pinnacle of glory.</q> <rj><qau>Byron.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Noiseless; stealthy.</def> <ldquo/With <xex>tiptoe</xex> step.<rdquo/  <rj><au>Cowper.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Tiptoe mirth</b></col>, <cd>the highest degree of mirth.</cd>  <rj><au>Sir W. Scott.</au></rj></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tiptoe</ent><br/
<hw>Tip"toe`</hw>, <pos>v. i.</pos> <def>To step or walk on tiptoe.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tiptop</ent><br/
<hw>Tip"top`</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[<ets>Tip</ets> end + <ets>top</ets>.]</ety> <def>The highest or utmost degree; the best of anything.</def> <mark>[Colloq.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tiptop</ent><br/
<hw>Tip"top`</hw>, <pos>a.</pos> <def>Very excellent; most excellent; perfect.</def> <mark>[Colloq.]</mark> <ldquo/Four <xex>tiptop</xex> voices.<rdquo/ <au>Gray.</au> <ldquo/Sung in a <xex>tiptop</xex> manner.<rdquo/ <au>Goldsmith.</au><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tipula</ent><br/
||<hw>Tip"u*la</hw> <pr>(?)</pr>, <pos>n.</pos>; <plu><it>pl.</it> L. <plw>Tipulae</plw> <pr>(#)</pr>, E. <plw>Tipulas</plw> <pr>(#)</pr>.</plu> <ety>[L., the water spider, or water spinner.]</ety> <fld>(Zool.)</fld> <def>Any one of many species of long-legged dipterous insects belonging to <gen>Tipula</gen> and allied genera. They have long and slender bodies. See <cref>Crane fly</cref>, under <er>Crane</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tipulary</ent><br/
<hw>Tip"u*la*ry</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[Cf. F. <ets>tipulaire</ets>.]</ety> <fld>(Zool.)</fld> <def>Of or pertaining to the tipulas.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tip-up</ent><br/
<hw>Tip"-up`</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Zool.)</fld> <def>The spotted sandpiper; -- called also <altname>teeter-tail</altname>. See under <er>Sandpiper</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tirade</ent><br/
<hw>Ti*rade"</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F., fr. It. <ets>tirada</ets>, properly, a pulling; hence, a lengthening out, a long speech, a tirade, fr. <ets>tirare</ets> to draw; of Teutonic origin, and akin to E. <ets>tear</ets> to redn. See <er>Tear</er> to rend, and cf. <er>Tire</er> to tear.]</ety> <def>A declamatory strain or flight of censure or abuse; a rambling invective; an oration or harangue abounding in censorious and bitter language.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Here he delivers a violent <qex>tirade</qex> against persons who profess to know anything about angels.</q> <rj><qau>Quarterly Review.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tirailleur</ent><br/
||<hw>Ti`rail`leur"</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F., from <ets>tirailler</ets> to skirmish, wrest, from <ets>tirer</ets> to draw.]</ety> <fld>(Mil.)</fld> <def>Formerly, a member of an independent body of marksmen in the French army. They were used sometimes in front of the army to annoy the enemy, sometimes in the rear to check his pursuit. The term is now applied to all troops acting as skirmishers.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tire</ent><br/
<hw>Tire</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A tier, row, or rank. See <er>Tier</er>.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>In posture to displode their second <qex>tire</qex><br/
Of thunder.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tire</ent><br/
<hw>Tire</hw>, <pos>n.</pos> <ety>[Aphetic form of <ets>attire</ets>; OE. <ets>tir</ets>, <ets>a tir</ets>. See <er>Attire</er>.]</ety> <sn>1.</sn> <def>Attire; apparel.</def> <mark>[Archaic]</mark> <ldquo/Having rich <xex>tire</xex> about you.<rdquo/  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A covering for the head; a headdress.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>On her head she wore a <qex>tire</qex> of gold.</q> <rj><qau>Spenser.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>A child's apron, covering the breast and having no sleeves; a pinafore; a tier.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>Furniture; apparatus; equipment.</def> <mark>[Obs.]</mark> <ldquo/The <xex>tire</xex> of war.<rdquo/  <rj><au>Philips.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <ety>[Probably the same word, and so called as being an attire or covering for the wheel.]</ety> <def>A ring, hoop or band, as of rubber or metal, on the circumference of the wheel of a vehicle, to impart strength and receive the wear.  In Britain, spelled <asp>tyre</asp>.</def><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ The iron <xex>tire</xex> of a wagon wheel or cart wheel binds the fellies together. The <xex>tire</xex> of a locomotive or railroad-car wheel is a heavy hoop of iron or steel shrunk tightly upon an iron central part. The wheel of a bicycle or road vehicle (automobile, motorcyle, truck) has a <xex>tire</xex> of rubber, which is typically hollow inside and inflated with air to lessen the shocks from bumps on uneven roads.</note><br/
[<source>1913 Webster</source> <source>+PJC</source>]</p>

<p><ent>Tire</ent><br/
<hw>Tire</hw>, <pos>v. t.</pos> <def>To adorn; to attire; to dress.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>[Jezebel] painted her face, and <qex>tired</qex> her head.</q> <rj><qau>2 Kings ix. 30.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tire</ent><br/
<hw>Tire</hw>, <pos>v. i.</pos> <ety>[F. <ets>tirer</ets> to draw or pull; of Teutonic origin, and akin to E. <ets>tear</ets> to rend. See <er>Tirade</er>.]</ety> <sn>1.</sn> <def>To seize, pull, and tear prey, as a hawk does.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>Even as an empty eagle, sharp by fast,<br/
<qex>Tires</qex> with her beak on feathers, flesh, and bone.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Ye dregs of baseness, vultures among men,<br/
That <qex>tire</qex> upon the hearts of generous spirits.</q> <rj><qau>B. Jonson.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To seize, rend, or tear something as prey; to be fixed upon, or engaged with, anything.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>Thus made she her remove,<br/
And left wrath <qex>tiring</qex> on her son.</q> <rj><qau>Chapman.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Upon that were my thoughts <qex>tiring</qex>.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tire</ent><br/
<hw>Tire</hw>, <pos>v. i.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Tired</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Tiring</conjf>.]</vmorph> <ety>[OE. <ets>teorien</ets> to become weary, to fail, AS. <ets>teorian</ets> to be tired, be weary, to tire, exhaust; perhaps akin to E. <ets>tear</ets> to rend, the intermediate sense being, perhaps, to wear out; or cf. E. <ets>tarry</ets>.]</ety> <def>To become weary; to be fatigued; to have the strength fail; to have the patience exhausted; <as>as, a feeble person soon <ex>tires</ex></as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tire</ent><br/
<hw>Tire</hw>, <pos>v. t.</pos> <def>To exhaust the strength of, as by toil or labor; to exhaust the patience of; to wear out (one's interest, attention, or the like); to weary; to fatigue; to jade.</def>  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q><qex>Tired</qex> with toil, all hopes of safety past.</q> <rj><qau>Dryden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>To tire out</b></col>, <cd>to weary or fatigue to exhaustion; to harass.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><syn><b>Syn.</b> -- To jade; weary; exhaust; harass. See <er>Jade</er>.</syn><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tired</ent><br/
<hw>Tired</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Weary; fatigued; exhausted.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tiredness</ent><br/
<hw>Tired"ness</hw>, <pos>n.</pos> <def>The state of being tired, or weary.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tire iron</ent><br/
<hw>Tire" iron</hw> <pr>(?)</pr>, <pos>n.</pos> <def>a short steel bar with one end flattened so as to form a blade, used to remove rubber tires from the rim of a wheel.</def><br/
[<source>PJC</source>]</p>

<p><ent>Tireless</ent><br/
<hw>Tire"less</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Untiring.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tireling</ent><br/
<hw>Tire"ling</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Tired; fatigued.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tiresome</ent><br/
<hw>Tire"some</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Fitted or tending to tire; exhausted; wearisome; fatiguing; tedious; <as>as, a <ex>tiresome</ex> journey; a <ex>tiresome</ex> discourse.</as></def> -- <wordforms><wf>Tire"some*ly</wf>, <pos>adv.</pos> -- <wf>Tire"some*ness</wf>, <pos>n.</pos></wordforms><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tire-woman</ent><br/
<hw>Tire"-wom`an</hw> <pr>(?)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Tire-women</plw> <pr>(#)</pr>.</plu> <ety>[See <er>Tire</er> attire, <er>Attire</er>.]</ety> <sn>1.</sn> <def>A lady's maid.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Fashionableness of the <qex>tire-woman's</qex> making.</q> <rj><qau>Locke.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A dresser in a theater.</def>  <rj><au>Simmonds.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tiring-house</ent><br/
<hw>Tir"ing-house`</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[For <ets>attiring house</ets>.]</ety> <def>A tiring-room.</def> <mark>[Obs.]</mark>  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tiring-room</ent><br/
<hw>Tir"ing-room`</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[For <ets>attiring room</ets>.]</ety> <def>The room or place where players dress for the stage.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tirl</ent><br/
<hw>Tirl</hw> <pr>(?)</pr>, <pos>v. i.</pos> <ety>[Cf. <er>Twirl</er>, <er>Thirl</er>.]</ety> <mark>[Scot. & Prov. Eng.]</mark> <sn>1.</sn> <def>To quiver; to vibrate; to veer about.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><sn>2.</sn>  <def>To make a ratting or clattering sound by twirling or shaking; <as>as, to <ex>tirl</ex> at the pin, or latch, of a door</as>.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Tirma</ent><br/
<hw>Tir"ma</hw> <pr>(?)</pr>, <pos>n.</pos> <def>The oyster catcher.</def> <mark>[Prov. Eng.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tiro</ent><br/
<hw>Ti"ro</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L.]</ety> <def>Same as <er>Tyro</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>T iron</ent><br/
<hw>T" i`ron</hw> <pr>(?)</pr>. <def>See under <er>T</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tironian</ent><br/
<hw>Ti*ro"ni*an</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>Tironianus</ets>, fr. <ets>Tiro</ets>, the learned freedman and amanuensis of Cicero.]</ety> <def>Of or pertaining to Tiro, or a system of shorthand said to have been introduced by him into ancient Rome.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tirralirra</ent><br/
<hw>Tir"ra*lir`ra</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A verbal imitation of a musical sound, as of the note of a lark or a horn.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The lark, that <qex>tirra lyra</qex> chants.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q><ldquo/<qex>Tirralira</qex>, <rdquo/ by the river,<br/
Sang Sir Lancelot.</q> <rj><qau>Tennyson.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tirrit</ent><br/
<hw>Tir"rit</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A word from the vocabulary of Mrs. Quickly, the hostess in Shakespeare's Henry IV., probably meaning <xex>terror</xex>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tirwit</ent><br/
<hw>Tir"wit</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Cf. <er>Pewit</er>.]</ety> <fld>(Zool.)</fld> <def>The lapwing.</def> <mark>[Prov. Eng.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>'T is</ent><br/
<hw>'T is</hw> <pr>(?)</pr>. <def>A common contraction of <xex>it is</xex>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tisane</ent><br/
<hw>Ti*sane"</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F.]</ety> <fld>(Med.)</fld> <def>See <er>Ptisan</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tisar</ent><br/
<hw>Ti"sar</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F. <ets>tisard</ets>.]</ety> <fld>(Glass Manuf.)</fld> <def>The fireplace at the side of an annealing oven.</def>  <rj><au>Knight.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tisical</ent><br/
<ent>Tisic</ent><br/
<mhw>{ <hw>Tis"ic</hw> <pr>(?)</pr>, <hw>Tis"ic*al</hw> <pr>(?)</pr>, }</mhw> <pos>a.</pos> <ety>[For <ets>phthisic</ets>, <ets>phthisical</ets>.]</ety> <def>Consumptive, phthisical.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tisic</ent><br/
<hw>Tis"ic</hw>, <pos>n.</pos> <def>Consumption; phthisis. See <er>Phthisis</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tisicky</ent><br/
<hw>Tis"ick*y</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Consumptive, phthisical.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tisri</ent><br/
<hw>Tis"ri</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Heb. <ets>tishr<imac/</ets>, fr. Chald. <ets>sher<amac/'</ets> to open, to begin.]</ety> <def>The seventh month of the Jewish ecclesiastical year, answering to a part of September with a part of October.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tissue</ent><br/
<hw>Tis"sue</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F. <ets>tissu</ets>, fr. <ets>tissu</ets>, p. p. of <ets>tisser</ets>, <ets>tistre</ets>, to weave, fr. L. <ets>texere</ets>. See <er>Text</er>.]</ety> <sn>1.</sn> <def>A woven fabric.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A fine transparent silk stuff, used for veils, etc.; specifically, cloth interwoven with gold or silver threads, or embossed with figures.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>A robe of <qex>tissue</qex>, stiff with golden wire.</q> <rj><qau>Dryden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>In their glittering <qex>tissues</qex> bear emblazed<br/
Holy memorials.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <fld>(Biol.)</fld> <def>One of the elementary materials or fibres, having a uniform structure and a specialized function, of which ordinary animals and plants are composed; a texture; <as>as, epithelial <ex>tissue</ex>; connective <ex>tissue</ex></as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ The term <xex>tissue</xex> is also often applied in a wider sense to all the materials or elementary tissues, differing in structure and function, which go to make up an organ; as, vascular <xex>tissue</xex>, tegumentary <xex>tissue</xex>, etc.</note><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>Fig.: Web; texture; complicated fabrication; connected series; <as>as, a <ex>tissue</ex> of forgeries, or of falsehood</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Unwilling to leave the dry bones of Agnosticism wholly unclothed with any living <qex>tissue</qex> of religious emotion.</q> <rj><qau>A. J. Balfour.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Tissue paper</b></col>, <cd>very thin, gauzelike paper, used for protecting engravings in books, for wrapping up delicate articles, etc.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tissue</ent><br/
<hw>Tis"sue</hw>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Tissued</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Tissuing</conjf>.]</vmorph> <def>To form tissue of; to interweave.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Covered with cloth of gold <qex>tissued</qex> upon blue.</q> <rj><qau>Bacon.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tissued</ent><br/
<hw>Tis"sued</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Clothed in, or adorned with, tissue; also, variegated; <as>as, <ex>tissued</ex> flowers</as>.</def>  <rj><au>Cowper.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>And crested chiefs and <qex>tissued</qex> dames<br/
Assembled at the clarion's call.</q> <rj><qau>T. Warton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tit</ent><br/
<hw>Tit</hw> <pr>(?)</pr>, <pos>n.</pos> <sn>1.</sn> <def>A small horse.</def>  <rj><au>Tusser.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A woman; -- used in contempt.</def>  <rj><au>Burton.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>A morsel; a bit.</def>  <rj><au>Halliwell.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <ety>[OE.; cf. Icel. <ets>titter</ets> a tit or small bird. The word probably meant originally, something small, and is perhaps the same as <ets>teat</ets>.  Cf. <er>Titmouse</er>, <er>Tittle</er>.]</ety> <fld>(Zool.)</fld> <sd>(a)</sd> <def>Any one of numerous species of small singing birds belonging to the families <fam>Paridae</fam> and <fam>Leiotrichidae</fam>; a titmouse.</def> <sd>(b)</sd> <def>The European meadow pipit; a titlark.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Ground tit</b></col>. <fld>(Zool.)</fld> <cd>See <cref>Wren tit</cref>, under <er>Wren</er>.</cd> -- <col><b>Hill tit</b></col> <fld>(Zool.)</fld>, <cd>any one of numerous species of Asiatic singing birds belonging to <gen>Siva</gen>, <gen>Milna</gen>, and allied genera.</cd> -- <col><b>Tit babbler</b></col> <fld>(Zool.)</fld>, <cd>any one of several species of small East Indian and Asiatic timaline birds of the genus <gen>Trichastoma</gen>.</cd> -- <col><b>Tit for tat</b></col>. <ety>[Probably for <ets>tip for tap</ets>. See <er>Tip</er> a slight blow.]</ety> <cd>An equivalent; retaliation.</cd> -- <col><b>Tit thrush</b></col> <fld>(Zool.)</fld>, <cd>any one of numerous species of Asiatic and East Indian birds belonging to <gen>Suthora</gen> and allied genera. In some respects they are intermediate between the thrushes and titmice.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Titan</ent><br/
<hw>Ti"tan</hw> <pr>(t<imac/"t<ait/n)</pr>, <pos>a.</pos> <def>Titanic.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The <qex>Titan</qex> physical difficulties of his enterprise.</q> <rj><qau>I. Taylor.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>titanate</ent><br/
<hw>ti"tan*ate</hw> <pr>(t<imac/"t<ait/n*<asl/t)</pr>, <pos>n.</pos> <fld>(Chem.)</fld> <def>A salt of titanic acid.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Titan crane</ent><br/
<hw>Ti"tan crane</hw>. <fld>(Mach.)</fld> <def>A massive crane with an overhanging counterbalanced arm carrying a traveler and lifting crab, the whole supported by a carriage mounted on track rails.  It is used esp. for setting heavy masonry blocks for piers, breakwaters, etc.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>titanic</ent><br/
<hw>ti*tan"ic</hw> <pr>(t<isl/*t<acr/n"<icr/k)</pr>, <pos>a.</pos> <def>Of or relating to Titans, or fabled giants of ancient mythology; hence, enormous in size or strength; <as>as, <ex>Titanic</ex> structures</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Titanic</ent><br/
<hw>Ti*tan"ic</hw> <pr>(?)</pr>, <pos>prop. n.</pos> <def>The name of a large ocean liner which hit an iceberg and sank on its maiden voyage from England to New York in 1912, with the loss of hundreds of lives.  Also, the name of several movies made about the incident.</def><br/
[<source>PJC</source>]</p>

<p><ent>titanic</ent><br/
<hw>ti*tan"ic</hw> <pr>(t<isl/*t<acr/n"<icr/k)</pr>, <pos>a.</pos> <ety>[Cf. F. <ets>titanique</ets>.]</ety> <fld>(Chem.)</fld> <def>Of or pertaining to titanium; derived from, or containing, titanium; specifically, designating those compounds of titanium in which it has a higher valence as contrasted with the <contr>titanous</contr> compounds.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>titanic acid</b></col> <fld>(Chem.)</fld>, <cd>a white amorphous powder, <chform>Ti(OH)4</chform>, obtained by decomposing certain titanates; -- called also <altname>normal titanic acid</altname>. By extension, any one of a series of derived acids, called also <stype>metatitanic acid</stype>, <stype>polytitanic acid</stype>, etc.</cd> -- <col><b>Titanic iron ore</b></col>. <fld>(Min.)</fld> <cd>See <er>Menaccanite</er>.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Titaniferous</ent><br/
<hw>Ti`tan*if"er*ous</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[<ets>Titanium</ets> + <ets>-ferous</ets>: cf. F. <ets>titanif<egrave/re</ets>.]</ety> <def>Containing or affording titanium; <as>as, <ex>titaniferous</ex> magnetite</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Titanite</ent><br/
<hw>Ti"tan*ite</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Cf. F. <ets>titanite</ets>; -- so called from containing titanic acid.]</ety> <fld>(Min.)</fld> <def>See <er>Sphene</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Titanitic</ent><br/
<hw>Ti`tan*it"ic</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Pertaining to, or containing, titanium; <as>as, a <ex>titanitic</ex> mineral</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Titanium</ent><br/
<hw>Ti*ta"ni*um</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[NL., fr. L. <ets>Titani</ets> or <ets>Titanes</ets>, Gr. <?/, the sons of the earth.]</ety> <fld>(Chem.)</fld> <def>An elementary substance found combined in the minerals manaccanite, rutile, sphene, etc., and isolated as an infusible iron-gray amorphous powder, having a metallic luster. It burns when heated in the air. Symbol Ti. Atomic weight 48.1.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Titano-</ent><br/
<hw>Ti"tan*o-</hw> <pr>(?)</pr>. <fld>(Chem.)</fld> <def>A combining form (also used adjectively) designating certain <xex>double compounds of titanium</xex> with some other elements; <as>as, <ex>titano</ex>-cyanide, <ex>titano</ex>-fluoride, <ex>titano</ex>-silicate, etc</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Titanotherium</ent><br/
||<hw>Ti`tan*o*the"ri*um</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[NL., fr. Gr. <?/ a Titan + <?/, dim. of <?/ a beast.]</ety> <fld>(Paleon.)</fld> <def>A large American Miocene mammal, allied to the rhinoceros, and more nearly to the extinct Brontotherium.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Titanous</ent><br/
<hw>Ti"tan*ous</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Designating certain compounds of titanium in which that element has a lower valence as contrasted with <contr>titanic</contr> compounds.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Titbit</ent><br/
<hw>Tit"bit`</hw> <pr>(?)</pr>, <pos>n.</pos> <def>Same as <er>Tidbit</er>.</def> <mark>[Chiefly Brit.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tith</ent><br/
<hw>Tith</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[See <er>Tight</er>, <pos>a.</pos>]</ety> <def>Tight; nimble.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>Of a good stirring strain too, she goes <qex>tith</qex>.</q> <rj><qau>Beau. & Fl.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tithable</ent><br/
<hw>Tith"a*ble</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Subject to the payment of tithes; <as>as, <ex>tithable</ex> lands</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tithe</ent><br/
<hw>Tithe</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[OE. <ets>tithe</ets>, <ets>tethe</ets>, properly an adj., tenth, AS. <ets>te<oacute/<?/a</ets> the tenth; akin to <ets>ti<eacute/n</ets>, <ets>t<?/n</ets>, <ets>t<emac/n</ets>, ten, G. <ets>zehnte</ets>, adj., tenth, n., a tithe, Icel. <ets>t<imac/und</ets> the tenth; tithe, Goth. <ets>ta<iacute/hunda</ets> tenth. See <er>Ten</er>, and cf. <er>Tenth</er>, <er>Teind</er>.]</ety> <sn>1.</sn> <def>A tenth; the tenth part of anything; specifically, the tenthpart of the increase arising from the profits of land and stock, allotted to the clergy for their support, as in England, or devoted to religious or charitable uses. Almost all the <xex>tithes</xex> of England and Wales are commuted by law into rent charges.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The <qex>tithes</qex> of the corn, the new wine, and the oil.</q> <rj><qau>Neh. xiii. 5.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ <xex>Tithes</xex> are called <xex>personal</xex> when accuring from labor, art, trade, and navigation; <xex>predial</xex>, when issuing from the earth, as hay, wood, and fruit; and <xex>mixed</xex>, when accuring from beaste fed from the ground.</note>  <rj><au>Blackstone.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Hence, a small part or proportion.</def>  <rj><au>Bacon.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Great tithes</b></col>, <cd>tithes of corn, hay, and wood.</cd> -- <col><b>Mixed tithes</b></col>, <cd>tithes of wool, milk, pigs, etc.</cd> -- <col><b>Small tithes</b></col>, <cd>personal and mixed tithes.</cd> -- <col><b>Tithe commissioner</b></col>, <cd>one of a board of officers appointed by the government for arranging propositions for commuting, or compounding for, tithes.</cd> <mark>[Eng.]</mark> <au>Simmonds.</au></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tithe</ent><br/
<hw>Tithe</hw>, <pos>a.</pos> <def>Tenth.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>Every <qex>tithe</qex> soul, 'mongst many thousand.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tithe</ent><br/
<hw>Tithe</hw>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Tithed</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Tithing</conjf>.]</vmorph> <ety>[As. <ets>te<oacute/<?/ian</ets>.]</ety> <def>To levy a tenth part on; to tax to the amount of a tenth; to pay tithes on.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Ye <qex>tithe</qex> mint and rue.</q> <rj><qau>Luke xi. 42.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tithe</ent><br/
<hw>Tithe</hw>, <pos>v. i.</pos> <def>Tp pay tithes.</def> <mark>[R.]</mark>  <rj><au>Tusser.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tither</ent><br/
<hw>Tith"er</hw> <pr>(?)</pr>, <pos>n.</pos> <sn>1.</sn> <def>One who collects tithes.</def>  <rj><au>Milton.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>One who pays tithes.</def> <mark>[R.]</mark>  <rj><au>Chaucer.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tithing</ent><br/
<hw>Tith"ing</hw>, <pos>n.</pos> <ety>[AS. <ets>te<oacute/<?/ung</ets>.]</ety> <sn>1.</sn> <def>The act of levying or taking tithes; that which is taken as tithe; a tithe.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>To take <qex>tithing</qex> of their blood and sweat.</q> <rj><qau>Motley.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(O. Eng. Law)</fld> <def>A number or company of ten householders who, dwelling near each other, were sureties or frankpledges to the king for the good behavior of each other; a decennary.</def>  <rj><au>Blackstone.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tithingman</ent><br/
<hw>Tith"ing*man</hw> <pr>(?)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Tithingmen</plw> <pr>(<?/)</pr>.</plu> <sn>1.</sn> <fld>(O. Eng. Law)</fld> <def>The chief man of a tithing; a headborough; one elected to preside over the tithing.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Law)</fld> <def>A peace officer; an under constable.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>A parish officer elected annually to preserve good order in the church during divine service, to make complaint of any disorderly conduct, and to enforce the observance of the Sabbath.</def> <mark>[Local, U. S.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tithly</ent><br/
<hw>Tith"ly</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[From <er>Tith</er>.]</ety> <def>Tightly; nimbly.</def> <mark>[Obs.]</mark> <ldquo/I have seen him trip it <xex>tithly</xex>.<rdquo/  <rj><au>Beau. & Fl.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tithonic</ent><br/
<hw>Ti*thon"ic</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>Tithonius</ets> belonging to <ets>Tithonus</ets>, the consort of Aurora, Gr. <?/.]</ety> <def>Of, pertaining to, or denoting, those rays of light which produce chemical effects; actinic.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tithonicity</ent><br/
<hw>Tith`o*nic"i*ty</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Chem. & Physics)</fld> <def>The state or property of being tithonic; actinism.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tithonographic</ent><br/
<hw>Ti*thon`o*graph"ic</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[<ets>Tithon</ets>ic + <ets>-graph</ets> + <ets>-ic</ets>.]</ety> <def>Of, relating to, or produced by, the chemical action of rays of light; photographic.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tithonometer</ent><br/
<hw>Tith`o*nom"e*ter</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[<ets>Tithon</ets>ic + <ets>-meter</ets>.]</ety> <def>An instrument or apparatus for measuring or detecting tithonicity; an actinometer.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tithymal</ent><br/
<hw>Tith"y*mal</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>tithymalus</ets> a plant with a milklike sap, Gr. <?/: cf. F. <ets>tithymale</ets>.]</ety> <fld>(Bot.)</fld> <def>Any kind of spurge, esp. <spn>Euphorbia Cyparissias</spn>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Titi</ent><br/
<hw>Ti"ti</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Zool.)</fld> <def>Same as <er>Teetee</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Titi</ent><br/
<hw>Ti"ti</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Orig. uncert.]</ety> <sn>1.</sn> <def>A tree of the southern United States (<spn>Cliftonia monophylla</spn>) having glossy leaves and racemes of fragrant white flowers succeeded by one-seeded drupes; -- called also <altname>black titi</altname>, <altname>buckwheat tree</altname>, and <altname>ironwood</altname>.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><sn>2.</sn>  <def>Any related tree of the genus <gen>Cyrilla</gen>, often disting. as <altname>white titi</altname>.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Titillate</ent><br/
<hw>Tit"il*late</hw> <pr>(t<icr/t"<icr/l*l<amac/t)</pr>, <pos>v. t. & i.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Titillated</conjf> <pr>(t<icr/t"<icr/l*l<amac/`t<ecr/d)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Titillating</conjf>.]</vmorph> <ety>[L. <ets>titillatus</ets>, p. p. of <ets>titillare</ets>.]</ety> <sn>1.</sn> <def>To tickle; <as>as, to <ex>titillate</ex> the nose with a feather</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The pungent grains of <qex>titillating</qex> dust.</q> <rj><qau>Pope.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To arouse an agreeable sensation in; <as>as, to <ex>titillate</ex> one's imagination; to <ex>titillate</ex> one's audience; to <ex>titillate</ex> the senses</as>.</def><br/
[<source>PJC</source>]</p>

<p><ent>titillation</ent><br/
<hw>tit`il*la"tion</hw> <pr>(t<icr/t`<icr/l*l<amac/"sh<ucr/n)</pr>, <pos>n.</pos> <ety>[L. <ets>titillatio</ets>: cf. F. <ets>titillation</ets>.]</ety> <sn>1.</sn> <def>The act of tickling, or the state of being tickled; a tickling sensation.</def>  <rj><au>A. Tucker.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Any pleasurable sensation.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Those <qex>titillations</qex> that reach no higher than the senses.</q> <rj><qau>Glanvill.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Titillative</ent><br/
<hw>Tit"il*la*tive</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Tending or serving to titillate, or tickle; tickling.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tittivate</ent><br/
<ent>Titivate</ent><br/
<mhw>{ <hw>Tit"i*vate</hw>, <hw>Tit"ti*vate</hw>  }</mhw> <pr>(t<icr/t"<icr/*v<amac/t)</pr>, <pos>v. t. & i.</pos>  <vmorph>[<pos>imp. & p. p.</pos> <conjf>Titivated</conjf> <pr>(t<icr/t"<icr/*v<amac/t`<ecr/d)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Titivating</conjf> <pr>(t<icr/t"<icr/*v<amac/t`<icr/ng)</pr>.]</vmorph> <ety>[Formed from an uncertain source, in imitation of words in <ets>-ate</ets> fr. L.]</ety> <def>To dress or smarten up; to spruce.</def> -- <wordforms><wf>Tit`i*va"tion</wf>, <wf>Tit`ti*va"tion</wf> <pr>(#)</pr>, <pos>n.</pos></wordforms> <mark>[Both Humorous]</mark></p>

<p><q><ldquo/Come here, an' let me <qex>titivate</qex> you.<rdquo/ He sat down beside her, and submitted to be dusted.</q>  <rj><qau>Quiller-Couch.</qau></rj><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Titlark</ent><br/
<hw>Tit"lark`</hw> <pr>(t<icr/t"l<aum/rk`)</pr>, <pos>n.</pos> <ety>[<ets>Tit</ets> a small bird + <ets>lark</ets>.]</ety> <fld>(Zool.)</fld> <def>Any one of numerous small spring birds belonging to <gen>Anthus</gen>, <gen>Corydalla</gen>, and allied genera, which resemble the true larks in color and in having a very long hind claw; especially, the European meadow pipit (<spn>Anthus pratensis</spn>).</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Title</ent><br/
<hw>Ti"tle</hw> <pr>(t<imac/"t'l)</pr>, <pos>n.</pos> <ety>[OF. <ets>title</ets>, F. <ets>titre</ets>, L. <ets>titulus</ets> an inscription, label, title, sign, token.  Cf. <er>Tilde</er>, <er>Titrate</er>, <er>Titular</er>.]</ety> <sn>1.</sn> <def>An inscription put over or upon anything as a name by which it is known.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>The inscription in the beginning of a book, usually containing the subject of the work, the author's and publisher's names, the date, etc.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <fld>(Bookbindng)</fld> <def>The panel for the name, between the bands of the back of a book.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>A section or division of a subject, as of a law, a book, specif. <fld>(Roman & Canon Laws)</fld>, a chapter or division of a law book.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <def>An appellation of dignity, distinction, or preeminence (hereditary or acquired), given to persons, as duke marquis, honorable, esquire, etc.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>With his former <qex>title</qex> greet Macbeth.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>6.</sn> <def>A name; an appellation; a designation.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>7.</sn> <fld>(Law)</fld> <sd>(a)</sd> <def>That which constitutes a just cause of exclusive possession; that which is the foundation of ownership of property, real or personal; a right; <as>as, a good <ex>title</ex> to an estate, or an imperfect <ex>title</ex></as>.</def> <sd>(b)</sd> <def>The instrument which is evidence of a right.</def> <sd>(c)</sd> <fld>(Canon Law)</fld> <def>That by which a beneficiary holds a benefice.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>8.</sn> <fld>(Anc. Church Records)</fld> <def>A church to which a priest was ordained, and where he was to reside.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Title deeds</b></col> <fld>(Law)</fld>, <cd>the muniments or evidences of ownership; as, the <xex>title deeds</xex> to an estate.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><syn><b>Syn.</b> -- Epithet; name; appellation; denomination. See <er>epithet</er>, and <er>Name</er>.</syn><br/
[<source>1913 Webster</source>]</p>

<p><-- p. 1513 --><br/
[<source>1913 Webster</source>]</p>

<p><ent>Title</ent><br/
<hw>Ti"tle</hw> <pr>(?)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Titled</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Titling</conjf> <pr>(?)</pr>.]</vmorph> <ety>[Cf. L. <ets>titulare</ets>, F. <ets>titrer</ets>. See <er>Title</er>, <pos>n.</pos>]</ety> <def>To call by a title; to name; to entitle.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Hadrian, having quieted the island, took it for honor to be <qex>titled</qex> on his coin, <ldquo/The Restorer of Britain.<rdquo/</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Titled</ent><br/
<hw>Ti"tled</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Having or bearing a title.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Titleless</ent><br/
<hw>Ti"tle*less</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Not having a title or name; without legitimate title.</def> <ldquo/A <xex>titleless</xex> tyrant.<rdquo/  <rj><au>Chaucer.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Title-page</ent><br/
<hw>Ti"tle-page`</hw> <pr>(?)</pr>, <pos>n.</pos> <def>The page of a book which contains it title.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The world's all <qex>title-page</qex>; there's no contents.</q> <rj><qau>Young.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Titler</ent><br/
<hw>Tit"ler</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A large truncated cone of refined sugar.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Titling</ent><br/
<hw>Tit"ling</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Icel. <ets>titlingr</ets> a tit sparrow. See <er>Tit</er> a small bird.]</ety> <sn>1.</sn> <fld>(Zool.)</fld> <sd>(a)</sd> <def>The hedge sparrow; -- called also <altname>titlene</altname>. Its nest often chosen by the cuckoo as a place for depositing its own eggs.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The <qex>titling</qex>, . . . being thus deceived, hatcheth the egg, and bringeth up the chick of another bird.</q> <rj><qau>Holland.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sd>(b)</sd> <def>The meadow pipit.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Stockfish; -- formerly so called in customhouses.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Titmal</ent><br/
<hw>Tit"mal</hw> <pr>(?)</pr>, <pos>n.</pos> <def>The blue titmouse.</def> <mark>[Prov. Eng.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Titmouse</ent><br/
<hw>Tit"mouse`</hw> <pr>(?)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Titmice</plw> <pr>(#)</pr>.</plu> <ety>[OE. <ets>titemose</ets>, <ets>titmase</ets>; <ets>tit</ets> small, or a small bird + AS. <ets>m<amac/se</ets> a kind of small bird; akin to D. <ets>mees</ets> a titmouse, G. <ets>meise</ets>, OHG. <ets>meisa</ets>, Icel. <ets>meisingr</ets>. The English form has been influenced by the unrelated word <ets>mouse</ets>.  Cf. <er>Tit</er> a small bird.]</ety> <fld>(Zool.)</fld> <def>Any one of numerous species of small insectivorous singing birds belonging to <gen>Parus</gen> and allied genera; -- called also <altname>tit</altname>, and <altname>tomtit</altname>.</def><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ The blue titmouse (<spn>Parus coeruleus</spn>), the marsh titmouse (<spn>Parus palustris</spn>), the crested titmouse (<spn>Parus cristatus</spn>), the great titmouse (<spn>Parus major</spn>), and the long tailed titmouse (<spn>Aegithalos caudatus</spn>), are the best-known European species. See <er>Chickadee</er>.</note><br/
[<source>1913 Webster</source>]</p>

<p><ent>Titrate</ent><br/
<hw>Ti"trate</hw> <pr>(?)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Titrated</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Titrating</conjf>.]</vmorph> <ety>[F. <ets>titrer</ets>, from <ets>titre</ets> standard, title. See <er>Title</er>, <pos>n.</pos>]</ety> <fld>(Chem.)</fld> <def>To analyse, or determine the strength of, by means of standard solutions.  Cf. <cref>Standardized solution</cref>, under <er>Solution</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Titrated</ent><br/
<hw>Ti"tra*ted</hw> <pr>(?)</pr>, <pos>a.</pos> <fld>(Chem.)</fld> <def>Standardized; determined or analyzed by titration; <as>as, <ex>titrated</ex> solutions</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Titration</ent><br/
<hw>Ti*tra"tion</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Chem.)</fld> <def>The act or process of titrating; a substance obtained by titrating.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Titter</ent><br/
<hw>Tit"ter</hw> <pr>(?)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Tittered</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Tittering</conjf>.]</vmorph> <ety>[Probably of imitative origin.]</ety> <def>To laugh with the tongue striking against the root of the upper teeth; to laugh with restraint, or without much noise; to giggle.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>A group of <qex>tittering</qex> pages ran before.</q> <rj><qau>Longfellow.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Titter</ent><br/
<hw>Tit"ter</hw>, <pos>n.</pos> <def>A restrained laugh.</def> <ldquo/There was a <xex>titter</xex> of . . . delight on his countenance.<rdquo/  <rj><au>Coleridge.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Titter</ent><br/
<hw>Tit"ter</hw>, <pos>v. i.</pos> <def>To seesaw. See <er>Teeter</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Titterel</ent><br/
<hw>Tit"ter*el</hw> <pr>(?)</pr>, <pos>n.</pos> <def>The whimbrel.</def> <mark>[Prov. Eng.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Titter-totter</ent><br/
<hw>Tit"ter-tot`ter</hw> <pr>(?)</pr>, <pos>v. i.</pos> <def>See <er>Teeter</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tittimouse</ent><br/
<hw>Tit"ti*mouse`</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Zool.)</fld> <def>Titmouse.</def> <mark>[Prov. Eng.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tittle</ent><br/
<hw>Tit"tle</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[OE. <ets>titel</ets>, <ets>titil</ets>, apparently a dim. of <ets>tit</ets>, in the sense of small; cf. G. <ets>t<uum/ttel</ets> a tittle, dim. of OHG. <ets>tutta</ets> teat. Perhaps, however, the same word as <ets>title</ets>, n.]</ety> <def>A particle; a minute part; a jot; an iota.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>It is easier for heaven and earth to pass, than one <qex>tittle</qex> of the law to fail.</q> <rj><qau>Luke xvi. 17.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Every <qex>tittle</qex> of this prophecy is most exactly verified.</q> <rj><qau>South.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tittlebat</ent><br/
<hw>Tit"tle*bat</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Zool.)</fld> <def>The three-spined stickleback.</def> <mark>[Prov. Eng.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tittle-tattle</ent><br/
<hw>Tit"tle-tat`tle</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[A reduplication of <ets>tattle</ets>.]</ety> <sn>1.</sn> <def>Idle, trifling talk; empty prattle.</def>  <rj><au>Arbuthnot.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>An idle, trifling talker; a gossip.</def> <mark>[R.]</mark>  <rj><au>Tatler.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tittle-tattle</ent><br/
<hw>Tit"tle-tat`tle</hw>, <pos>v. i.</pos> <def>To talk idly; to prate.</def>  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tittle-tattling</ent><br/
<hw>Tit"tle-tat`tling</hw> <pr>(?)</pr>, <pos>n.</pos> <def>The act or habit of parting idly or gossiping.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tittup</ent><br/
<hw>Tit"tup</hw> <pr>(?)</pr>, <pos>v. i.</pos>  <vmorph>[<pos>imp. & p. p.</pos> <conjf>Tittuped</conjf> <pr>(?)</pr> or <conjf>Tittupped</conjf>; <pos>p. pr. & vb. n.</pos> <conjf>Tittuping</conjf> or <conjf>Tittupping</conjf>.]</vmorph> <altsp>[Written also <asp>titup</asp>.]</altsp> <ety>[Cf. <er>Teeter</er>.]</ety> <def>To behave or move in a lively or restless manner, as an impatient horse; to caper; to prance; to frisk.</def>  <rj><au>Kipling.</au></rj><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Tittup</ent><br/
<hw>Tit"tup</hw>, <pos>n.</pos> <def>The act of tittuping; lively, gay, or restless behavior or gait; a prance or caper.</def> <altsp>[Written also <asp>titup</asp>.]</altsp><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Tittuppy</ent><br/
<hw>Tit"tup*py</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Given to tittuping; gay; lively; prancing; also, shaky; unsteady.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Titty</ent><br/
<hw>Tit"ty</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A little teat; a nipple.</def> <mark>[Familiar]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Titubate</ent><br/
<hw>Tit"u*bate</hw> <pr>(?)</pr>, <pos>v. i.</pos> <ety>[L. <ets>titubatus</ets>, p. p. of <ets>titubare</ets> to stagger, totter.]</ety> <sn>1.</sn> <def>To stumble.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To rock or roll, as a curved body on a plane.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Titubation</ent><br/
<hw>Tit`u*ba"tion</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>titubatio</ets>: cf. F. <ets>titubation</ets>.]</ety> <def>The act of stumbling, rocking, or rolling; a reeling.</def>  <rj><au>Quain.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Titular</ent><br/
<hw>Tit"u*lar</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[F. <ets>titulaire</ets>, fr. L. <ets>titulus</ets>. See <er>Title</er>.]</ety> <def>Existing in title or name only; nominal; having the title to an office or dignity without discharging its appropriate duties; <as>as, a <ex>titular</ex> prince</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>If these magnificent titles yet remain<br/
Not merely <qex>titular</qex>.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Titular bishop</b></col>. <cd>See under <er>Bishop</er>.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Titular</ent><br/
<hw>Tit"u*lar</hw>, <pos>n.</pos> <def>A titulary.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Titularity</ent><br/
<hw>Tit`u*lar"i*ty</hw> <pr>(?)</pr>, <pos>n.</pos> <def>The quality or state of being titular.</def> <mark>[R.]</mark>  <rj><au>Sir T. Browne.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Titularly</ent><br/
<hw>Tit"u*lar*ly</hw> <pr>(?)</pr>, <pos>adv.</pos> <def>In a titular manner; nominally; by title only.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Titulary</ent><br/
<hw>Tit"u*la*ry</hw> <pr>(?)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Titularies</plw> <pr>(#)</pr>.</plu> <ety>[Cf. F. <ets>titulaire</ets>.]</ety> <def>A person invested with a title, in virtue of which he holds an office or benefice, whether he performs the duties of it or not.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Titulary</ent><br/
<hw>Tit"u*la*ry</hw>, <pos>a.</pos> <sn>1.</sn> <def>Consisting in a title; titular.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Of or pertaining to a title.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tituled</ent><br/
<hw>Tit"uled</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Having a title.</def> <mark>[Obs.]</mark>  <rj><au>Fuller.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tiver</ent><br/
<hw>Tiv"er</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[AS. <ets>te<aacute/for</ets>, <ets>te<aacute/fur</ets>.]</ety> <def>A kind of ocher which is used in some parts of England in marking sheep.</def> <mark>[Prov. Eng.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tiver</ent><br/
<hw>Tiv"er</hw>, <pos>v. t.</pos> <def>To mark with tiver.</def> <mark>[Prov. Eng.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tivoli</ent><br/
<hw>Tiv"o*li</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Prob. fr. <ets>Tivoli</ets> in Italy, a pleasure resort not far from Rome.]</ety> <def>A game resembling bagatelle, played on a special oblong board or table (<mcol><col><b>Tivoli board</b></col> <it>or</it>  <col><b>Tivoli table</b></col></mcol>), which has a curved upper end, a set of numbered compartments at the lower end, side alleys, and the surface studded with pins and sometimes furnished with numbered depressions or cups.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Tivy</ent><br/
<hw>Tiv"y</hw> <pr>(?)</pr>, <pos>adv.</pos> <ety>[See <er>Tantivy</er>.]</ety> <def>With great speed; -- a huntsman's word or sound.</def>  <rj><au>Dryden.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tiza</ent><br/
<hw>Ti"za</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[CF. Sp. <ets>tiza</ets> whitening, a kind of chalk or pipe clay.]</ety> <fld>(Chem.)</fld> <def>See <er>Ulexite</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tlinkit</ent><br/
<hw>Tlin"kit</hw> <pr>(?)</pr>, <pos>n. pl.</pos> <def>The Indians of a seafaring group of tribes of southern Alaska comprising the Koluschan stock. Previous to deterioration from contact with the whites they were the foremost traders of the northwest. They built substantial houses of cedar adorned with  totem poles, and were expert stone carvers and copper workers. Slavery, the potlatch, and the use of immense labrets were characteristic. Many now work in the salmon industry.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>T lymphocyte</ent><br/
<hw>T` lym"pho*cyte</hw> <def>Same as <er>T cell</er>.</def><br/
[<source>PJC</source>]</p>

<p><ent>Tmesis</ent><br/
<hw>Tme"sis</hw> <pr>(m<emac/"s<icr/s <it>or</it> t'm<emac/"s<icr/s; 277)</pr>, <pos>n.</pos> <ety>[L., from Gr. <grk>tmh^sis</grk> a cutting, fr. <grk>te`mnein</grk> to cut.]</ety> <fld>(Gram.)</fld> <def>The separation of the parts of a compound word by the intervention of one or more words; <as>as, <ex>in what place soever</ex>, for <ex>whatsoever place</ex></as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>To-</ent><br/
<hw>To-</hw> <pr>(?, see <er>To</er>, <pos>prep.</pos>)</pr>, <ety>[AS. <ets>to-</ets> asunder; akin to G. <ets>zer-</ets>, and perhaps to L. <ets>dis-</ets>, or Gr. <?/.]</ety> <def>An obsolete intensive prefix used in the formation of compound verbs; as in <xex>to</xex>-beat, <xex>to</xex>-break, <xex>to</xex>-hew, <xex>to</xex>-rend, <xex>to</xex>-tear. See these words in the Vocabulary. See the Note on <cref>All to</cref>, or <cref>All-to</cref>, under <er>All</er>, <pos>adv.</pos></def><br/
[<source>1913 Webster</source>]</p>

<p><ent>To</ent><br/
<hw>To</hw> <pr>(<?/, <it>emphatic or alone</it>, <?/, <it>obscure or unemphatic</it>)</pr>, <pos>prep.</pos> <ety>[AS. <ets>t<omac/</ets>; akin to OS. & OFries. <ets>t<omac/</ets>, D. <ets>toe</ets>, G. <ets>zu</ets>, OHG. <ets>zuo</ets>, <ets>zua</ets>, <ets>z<omac/</ets>, Russ. <ets>do</ets>, Ir. & Gael. <ets>do</ets>, OL. <ets>-do</ets>, <ets>-du</ets>, as in <ets>endo</ets>, <ets>indu</ets>, in, Gr. <?/, as in <?/ homeward. <root/200.   Cf. <er>Too</er>, <er>Tatoo</er> a beat of drums.]</ety> <sn>1.</sn> <def>The preposition <xex>to</xex> primarily indicates approach and arrival, motion made in the direction of a place or thing and attaining it, access; and also, motion or tendency without arrival; movement toward; -- opposed to <ant>from</ant>.</def> <ldquo/<xex>To</xex> Canterbury they wend.<rdquo/  <rj><au>Chaucer.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Stay with us, go not <qex>to</qex> Wittenberg.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>So <qex>to</qex> the sylvan lodge<br/
They came, that like Pomona's arbor smiled.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>I'll <qex>to</qex> him again, . . . <br/
He'll tell me all his purpose.<br/
She stretched her arms <qex>to</qex> heaven.</q> <rj><qau>Dryden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Hence, it indicates motion, course, or tendency toward a time, a state or condition, an aim, or anything capable of being regarded as a limit to a tendency, movement, or action; <as>as, he is going <ex>to</ex> a trade; he is rising <ex>to</ex> wealth and honor</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ Formerly, by omission of the verb denoting motion, <xex>to</xex> sometimes followed a form of <xex>be</xex>, with the sense of <xex>at</xex>, or <xex>in</xex>. <ldquo/When the sun was [gone or declined] <xex>to</xex> rest.<rdquo/ <au>Chaucer.</au></note><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>In a very general way, and with innumerable varieties of application, <xex>to</xex> connects transitive verbs with their remoter or indirect object, and adjectives, nouns, and neuter or passive verbs with a following noun which limits their action. Its sphere verges upon that of <xex>for</xex>, but it contains less the idea of design or appropriation; <as>as, these remarks were addressed <ex>to</ex> a large audience; let us keep this seat <ex>to</ex> ourselves; a substance sweet <ex>to</ex> the taste; an event painful <ex>to</ex> the mind; duty <ex>to</ex> God and <ex>to</ex> our parents; a dislike <ex>to</ex> spirituous liquor</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Marks and points out each man of us <qex>to</qex> slaughter.</q> <rj><qau>B. Jonson.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Whilst they, distilled<br/
Almost <qex>to</qex> jelly with the act of fear,<br/
Stand dumb and speak not <qex>to</qex> him.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Add <qex>to</qex> your faith virtue; and <qex>to</qex> virtue knowledge; and <qex>to</qex> knowledge temperance; and <qex>to</qex> temperance patience; and <qex>to</qex> patience godliness; and <qex>to</qex> godliness brotherly kindness; and <qex>to</qex> brotherly kindness charity.</q> <rj><qau>2 Pet. i. 5,6,7.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>I have a king's oath <qex>to</qex> the contrary.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Numbers were crowded <qex>to</qex> death.</q> <rj><qau>Clarendon.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Fate and the dooming gods are deaf <qex>to</qex> tears.</q> <rj><qau>Dryden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Go, buckle <qex>to</qex> the law.</q> <rj><qau>Dryden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>As sign of the infinitive, <xex>to</xex> had originally the use of last defined, governing the infinitive as a verbal noun, and connecting it as indirect object with a preceding verb or adjective; thus, ready <xex>to</xex> go, <xex>i</xex>.<xex>e</xex>., ready unto going; good <xex>to</xex> eat, <xex>i</xex>.<xex>e</xex>., good for eating; I do my utmost <xex>to</xex> lead my life pleasantly. But it has come to be the almost constant prefix to the infinitive, even in situations where it has no prepositional meaning, as where the infinitive is direct object or subject; thus, I love <xex>to</xex> learn, <xex>i</xex>.<xex>e</xex>., I love learning; <xex>to</xex> die for one's country is noble, <xex>i</xex>.<xex>e</xex>., the dying for one's country. Where the infinitive denotes the design or purpose, good usage formerly allowed the prefixing of <xex>for</xex> to the <xex>to</xex>; <as>as, what went ye out <ex>for</ex> see? (<au>Matt. xi. 8</au>)</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Then longen folk <qex>to</qex> go on pilgrimages,<br/
And palmers <qex>for to</qex> seeken strange stranders.</q> <rj><qau>Chaucer.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><note>Such usage is now obsolete or illiterate. In colloquial usage, <xex>to</xex> often stands for, and supplies, an infinitive already mentioned; thus, he commands me <xex>to</xex> go with him, but I do not wish <xex>to</xex>.</note><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <def>In many phrases, and in connection with many other words, <xex>to</xex> has a pregnant meaning, or is used elliptically.</def> Thus, it denotes or implies: <sd>(a)</sd> <def>Extent; limit; degree of comprehension; inclusion as far as; <as>as, they met us <ex>to</ex> the number of three hundred</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>We ready are to try our fortunes<br/
<qex>To</qex> the last man.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Few of the Esquimaux can count <qex>to</qex> ten.</q> <rj><qau>Quant. Rev.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sd>(b)</sd> <def>Effect; end; consequence; <as>as, the prince was flattered <ex>to</ex> his ruin; he engaged in a war <ex>to</ex> his cost; violent factions exist <ex>to</ex> the prejudice of the state</as>.</def> <sd>(c)</sd> <def>Apposition; connection; antithesis; opposition; as, they engaged hand <xex>to</xex> hand.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Now we see through a glass, darkly; but then face <qex>to</qex> face.</q> <rj><qau>1 Cor. xiii. 12.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sd>(d)</sd> <def>Accord; adaptation; <as>as, an occupation <ex>to</ex> his taste; she has a husband <ex>to</ex> her mind</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>He <qex>to</qex> God's image, she <qex>to</qex> his was made.</q> <rj><qau>Dryden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sd>(e)</sd> <def>Comparison; <as>as, three is <ex>to</ex> nine as nine is <ex>to</ex> twenty-seven; it is ten <ex>to</ex> one that you will offend him</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>All that they did was piety <qex>to</qex> this.</q> <rj><qau>B. Jonson.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sd>(f)</sd> <def>Addition; union; accumulation.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Wisdom he has, and <qex>to</qex> his wisdom, courage.</q> <rj><qau>Denham.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sd>(g)</sd> <def>Accompaniment; <as>as, she sang <ex>to</ex> his guitar; they danced <ex>to</ex> the music of a piano</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Anon they move<br/
In perfect phalanx <qex>to</qex> the Dorian mood<br/
Of flutes and soft recorders.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sd>(h)</sd> <def>Character; condition of being; purpose subserved or office filled.</def> <mark>[In this sense archaic]</mark> <ldquo/I have a king here <xex>to</xex> my flatterer.<rdquo/  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Made his masters and others . . . to consider him <qex>to</qex> a little wonder.</q> <rj><qau>Walton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ <xex>To</xex> in <xex>to-day</xex>, <xex>to-night</xex>, and <xex>to-morrow</xex> has the sense or force of <xex>for</xex> or <xex>on</xex>; <xex>for</xex>, or <xex>on</xex>, (this) day, <xex>for</xex>, or <xex>on</xex>, (this) night, <xex>for</xex>, or <xex>on</xex>, (the) morrow. <xex>To-day</xex>, <xex>to-night</xex>, <xex>to-morrow</xex> may be considered as compounds, and usually as adverbs; but they are sometimes used as nouns; as, <xex>to-day</xex> is ours.<br/
[<source>1913 Webster</source>]</p>

<p><q><qex>To-morrow</qex>, and <qex>to-morrow</qex>, and <qex>to-morrow</qex>;<br/
Creeps in this petty pace from day to day.</q> <rj><qau>Shak.</qau></rj>
</note><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>To and again</b></col>, <cd>to and fro.</cd> <mark>[R.]</mark> -- <col><b>To and fro</b></col>, <cd>forward and back. In this phrase, <xex>to</xex> is adverbial.</cd><br/
[<source>1913 Webster</source>]</p>

<p><q>There was great showing both <qex>to and fro</qex>.</q> <rj><qau>Chaucer.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p>-- <col><b>To-and-fro</b></col>, <cd>a pacing backward and forward; as, to commence a <xex>to-and-fro</xex>.</cd> <au>Tennyson.</au> -- <col><b>To the face</b></col>, <cd>in front of; in behind; hence, in the presence of.</cd> -- <col><b>To wit</b></col>, <cd>to know; namely. See <er>Wit</er>, <pos>v. i.</pos></cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ <xex>To</xex>, without an object expressed, is used adverbially; as, put <xex>to</xex> the door, <it>i. e.</it>, put the door to its frame, close it; and in the nautical expressions, to heave <xex>to</xex>, to come <xex>to</xex>, meaning to a certain position. <xex>To</xex>, like <xex>on</xex>, is sometimes used as a command, <xex>forward</xex>, <xex>set to</xex>. <ldquo/<xex>To</xex>, Achilles! <xex>to</xex>, Ajax! <xex>to</xex>!<rdquo/ <au>Shak.</au></note><br/
[<source>1913 Webster</source>]</p>

<p><ent>Toad</ent><br/
<hw>Toad</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[OE. <ets>tode</ets>, <ets>tade</ets>, AS. <ets>t<amac/die</ets>, <ets>t<amac/dige</ets>; of unknown origin.  Cf. <er>Tadpole</er>.]</ety> <fld>(Zool.)</fld> <def>Any one of numerous species of batrachians belonging to the genus <gen>Bufo</gen> and allied genera, especially those of the family <fam>Bufonidae</fam>. Toads are generally terrestrial in their habits except during the breeding season, when they seek the water. Most of the species burrow beneath the earth in the daytime and come forth to feed on insects at night. Most toads have a rough, warty skin in which are glands that secrete an acrid fluid.</def><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ The common toad (<spn>Bufo vulgaris</spn>) and the natterjack are familiar European species. The common American toad (<spn>Bufo lentiginosus</spn>) is similar to the European toad, but is less warty and is more active, moving chiefly by leaping.</note><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Obstetrical toad</b></col>. <fld>(Zool.)</fld> <cd>See under <er>Obstetrical</er>.</cd> -- <col><b>Surinam toad</b></col>. <fld>(Zool.)</fld> <cd>See <er>Pita</er>.</cd> -- <col><b>Toad lizard</b></col> <fld>(Zool.)</fld>, <cd>a horned toad.</cd> -- <col><b>Toad pipe</b></col> <fld>(Bot.)</fld>, <cd>a hollow-stemmed plant (<spn>Equisetum limosum</spn>) growing in muddy places.</cd> <au>Dr. Prior.</au> -- <col><b>Toad rush</b></col> <fld>(Bot.)</fld>, <cd>a low-growing kind of rush (<spn>Juncus bufonius</spn>).</cd> -- <col><b>Toad snatcher</b></col> <fld>(Zool.)</fld>, <cd>the reed bunting.</cd> <mark>[Prov. Eng.]</mark> -- <col><b>Toad spittle</b></col>. <fld>(Zool.)</fld> <cd>See <cref>Cuckoo spit</cref>, under <er>Cuckoo</er>.</cd> -- <col><b>Tree toad</b></col>. <fld>(Zool.)</fld> <cd>See under <er>Tree</er>.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Toadeater</ent><br/
<hw>Toad"eat`er</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Said to be so called in allusion to an old alleged practice among mountebanks' boys of eating toads (popularly supposed to be poisonous), in order that their masters might have an opportunity of pretending to effect a cure. The French equivalent expression is <ets>un avaleur de couleuvres</ets>.  Cf. <er>Toady</er>.]</ety> <def>A fawning, obsequious parasite; a mean sycophant; a flatterer; a toady.</def>  <rj><au>V. Knox.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>You had nearly imposed upon me, but you have lost your labor. You're too zealous a <qex>toadeater</qex>, and betray yourself.</q> <rj><qau>Dickens.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Toadfish</ent><br/
<hw>Toad"fish`</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Zool.)</fld> <sd>(a)</sd> <def>Any marine fish of the genus <gen>Batrachus</gen>, having a large, thick head and a wide mouth, and bearing some resemblance to a toad. The American species (<spn>Batrachus tau</spn>) is very common in shallow water. Called also <altname>oyster fish</altname>, and <altname>sapo</altname>.</def> <sd>(b)</sd> <def>The angler.</def> <sd>(c)</sd> <def>A swellfish.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Toadflax</ent><br/
<hw>Toad"flax`</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Bot.)</fld> <def>An herb (<spn>Linaria vulgaris</spn>) of the Figwort family, having narrow leaves and showy orange and yellow flowers; -- called also <altname>butter and eggs</altname>, <altname>flaxweed</altname>, and <altname>ramsted</altname>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Toadhead</ent><br/
<hw>Toad"head`</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Zool.)</fld> <def>The golden plover.</def> <mark>[Local, U. S.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Toadish</ent><br/
<hw>Toad"ish</hw>, <pos>a.</pos> <def>Like a toad.</def> <mark>[Obs.]</mark>  <rj><au>A. Stafford.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Toadlet</ent><br/
<hw>Toad"let</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A small toad.</def> <mark>[R.]</mark>  <rj><au>Coleridge.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Toadstone</ent><br/
<hw>Toad"stone`</hw> <pr>(?)</pr>, <pos>n.</pos> <sn>1.</sn> <fld>(Min.)</fld> <def>A local name for the igneous rocks of Derbyshire, England; -- said by some to be derived from the German <ets>todter stein</ets>, meaning <ets>dead stone</ets>, that is, stone which contains no ores.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Bufonite, formerly regarded as a precious stone, and worn as a jewel. See <er>Bufonite</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Toadstool</ent><br/
<hw>Toad"stool`</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Bot.)</fld> <def>A name given to many umbrella-shaped fungi, mostly of the genus <gen>Agaricus</gen>. The species are almost numberless. They grow on decaying organic matter.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Toady</ent><br/
<hw>Toad"y</hw> <pr>(?)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Toadies</plw> <pr>(#)</pr>.</plu> <ety>[Shortened from <ets>toadeater</ets>.]</ety> <sn>1.</sn> <def>A mean flatterer; a toadeater; a sycophant.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Before I had been standing at the window five minutes, they somehow conveyed to me that they were all <qex>toadies</qex> and humbugs.</q> <rj><qau>Dickens.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A coarse, rustic woman.</def> <mark>[R.]</mark>  <rj><au>Sir W. Scott.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><-- p. 1514 --><br/
[<source>1913 Webster</source>]</p>

<p><ent>Toady</ent><br/
<hw>Toad"y</hw> <pr>(?)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Toadied</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Toadying</conjf>.]</vmorph> <def>To fawn upon with mean sycophancy.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Toadyism</ent><br/
<hw>Toad"y*ism</hw> <pr>(?)</pr>, <pos>n.</pos> <def>The practice of meanly fawning on another; base sycophancy; servile adulation.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Toast</ent><br/
<hw>Toast</hw> <pr>(?)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Toasted</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Toasting</conjf>.]</vmorph> <ety>[OF. <ets>toster</ets> to roast, toast, fr. L. <ets>torrere</ets>, <ets>tostum</ets>, to parch, roast. See <er>Torrid</er>.]</ety> <sn>1.</sn> <def>To dry and brown by the heat of a fire; <as>as, to <ex>toast</ex> bread</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To warm thoroughly; <as>as, to <ex>toast</ex> the feet</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>To name when a health is proposed to be drunk; to drink to the health, or in honor, of; <as>as, to <ex>toast</ex> a lady</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Toast</ent><br/
<hw>Toast</hw>, <pos>n.</pos> <ety>[OF. <ets>toste</ets>, or <ets>tost<eacute/e</ets>, toasted bread. See <er>Toast</er>, <pos>v.</pos>]</ety> <sn>1.</sn> <def>Bread dried and browned before a fire, usually in slices; also, a kind of food prepared by putting slices of toasted bread into milk, gravy, etc.</def>
<-- now usu. prepared in an electrical toaster. See <er>toaster.</er> --><br/
[<source>1913 Webster</source>]</p>

<p><q>My sober evening let the tankard bless,<br/
With <qex>toast</qex> embrowned, and fragrant nutmeg fraught.</q> <rj><qau>T. Warton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A lady in honor of whom persons or a company are invited to drink; -- so called because toasts were formerly put into the liquor, as a great delicacy.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>It now came to the time of Mr. Jones to give a <qex>toast</qex> . . . who could not refrain from mentioning his dear Sophia.</q> <rj><qau>Fielding.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>Hence, any person, especially a person of distinction, in honor of whom a health is drunk; hence, also, anything so commemorated; a sentiment, as <ldquo/The land we live in,<rdquo/  <ldquo/The day we celebrate,<rdquo/ etc.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Toast rack</b></col>, <cd>a small rack or stand for a table, having partitions for holding slices of dry toast.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Toaster</ent><br/
<hw>Toast"er</hw> <pr>(?)</pr>, <pos>n.</pos> <sn>1.</sn> <def>One who toasts.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A kitchen utensil for toasting bread, cheese, etc.</def><-- since 1950, usu. operated by electricity, with heating coils arranged so as to brown a slice of bread evenly over both surfaces.  --><br/
[<source>1913 Webster</source>]</p>

<p><-- <col><b>Toaster oven</b></col>.  an electrical toaster. --><br/
[<source>1913 Webster</source>]</p>

<p><ent>Toasting</ent><br/
<hw>Toast"ing</hw>, <def><pos>a. & n.</pos> from <er>Toast</er>, <pos>v.</pos></def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Toasting fork</b></col>, <cd>a long-handled fork for toasting bread, cheese, or the like, by the fire.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Toastmaster</ent><br/
<hw>Toast"mas`ter</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A person who presides at a public dinner or banquet, and announces the toasts.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Toat</ent><br/
<hw>Toat</hw> <pr>(?)</pr>, <pos>n.</pos> <def>The handle of a joiner's plane.</def>  <rj><au>Knight.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tobacco</ent><br/
<hw>To*bac"co</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Sp. <ets>tabaco</ets>, fr. the Indian <ets>tabaco</ets> the tube or pipe in which the Indians or Caribbees smoked this plant. Some derive the word from <ets>Tabaco</ets>, a province of Yucatan, where it was said to be first found by the Spaniards; others from the island of <ets>Tobago</ets>, one of the Caribbees. But these derivations are very doubtful.]</ety> <sn>1.</sn> <fld>(Bot.)</fld> <def>An American plant (<spn>Nicotiana Tabacum</spn>) of the Nightshade family, much used for smoking and chewing, and as snuff. As a medicine, it is narcotic, emetic, and cathartic. Tobacco has a strong, peculiar smell, and an acrid taste.</def><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ The name is extended to other species of the genus, and to some unrelated plants, as Indian tobacco (<spn>Nicotiana rustica</spn>, and also <spn>Lobelia inflata</spn>), mountain tobacco (<spn>Arnica montana</spn>), and Shiraz tobacco (<spn>Nicotiana Persica</spn>).</note><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>The leaves of the plant prepared for smoking, chewing, etc., by being dried, cured, and manufactured in various ways.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Tobacco box</b></col> <fld>(Zool.)</fld>, <cd>the common American skate.</cd> -- <col><b>Tobacco camphor</b></col>. <fld>(Chem.)</fld> <cd>See <er>Nicotianine</er>.</cd> -- <col><b>Tobacco man</b></col>, <cd>a tobacconist.</cd> <mark>[R.]</mark> -- <col><b>Tobacco pipe</b></col>. <sd>(a)</sd> <cd>A pipe used for smoking, made of baked clay, wood, or other material.</cd> <sd>(b)</sd> <fld>(Bot.)</fld> <cd>Same as <cref>Indian pipe</cref>, under <er>Indian</er>.</cd> -- <col><b>Tobacco-pipe clay</b></col> <fld>(Min.)</fld>, <cd>a species of clay used in making tobacco pipes; -- called also <altname>cimolite</altname>.</cd> -- <col><b>Tobacco-pipe fish</b></col>. <fld>(Zool.)</fld> <cd>See <er>Pipemouth</er>.</cd> -- <col><b>Tobacco stopper</b></col>, <cd>a small plug for pressing down the tobacco in a pipe as it is smoked.</cd> -- <col><b>Tobacco worm</b></col> <fld>(Zool.)</fld>, <cd>the larva of a large hawk moth (<spn>Sphinx Carolina</spn> syn. <spn>Phlegethontius Carolina</spn>).  It is dark green, with seven oblique white stripes bordered above with dark brown on each side of the body.  It feeds upon the leaves of tobacco and tomato plants, and is often very injurious to the tobacco crop. See <xex>Illust.</xex> of <er>Hawk moth</er>.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tobacconing</ent><br/
<hw>To*bac"co*ning</hw> <pr>(?)</pr>, <pos>n.</pos> <def>Smoking tobacco.</def> <mark>[Obs.]</mark> <ldquo/<xex>Tobacconing</xex> is but a smoky play.<rdquo/ <mark>[Obs.]</mark>  <rj><au>Sylvester.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tobacconist</ent><br/
<hw>To*bac"co*nist</hw> <pr>(?)</pr>, <pos>n.</pos> <sn>1.</sn> <def>A dealer in tobacco; also, a manufacturer of tobacco.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A smoker of tobacco.</def> <mark>[Obs.]</mark>  <rj><au>Sylvester.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>To-beat</ent><br/
<hw>To-beat"</hw> <pr>(?)</pr>, <pos>v. t.</pos> <ety>[Pref. <ets>to-</ets> + <ets>beat</ets>.]</ety> <def>To beat thoroughly or severely.</def> <mark>[Obs.]</mark>  <rj><au>Layamon.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tobias fish</ent><br/
<hw>To*bi"as fish`</hw> <pr>(?)</pr>. <ety>[See the Note under <er>Asmodeus</er>, in the Dictionary of Noted Names in Fiction.]</ety> <fld>(Zool.)</fld> <def>The lant, or sand eel.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tobie</ent><br/
<hw>To"bie</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Cf. <er>Toby</er>.]</ety> <def>A kind of inferior cigar of a long slender shape, tapered at one end.</def> <mark>[Local, U. S.]</mark><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Tobine</ent><br/
<hw>To"bine</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Cf. G. <ets>tobin</ets>, D. <ets>tabijn</ets>. See <er>Tabby</er>.]</ety> <def>A stout twilled silk used for dresses.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tobit</ent><br/
<hw>To"bit</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A book of the Apocrypha.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Toboggan</ent><br/
<hw>To*bog"gan</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Corruption of American Indian <ets>odabagan</ets> a sled.]</ety> <def>A kind of sledge made of pliable board, turned up at one or both ends, used for coasting down hills or prepared inclined planes; also, a sleigh or sledge, to be drawn by dogs, or by hand, over soft and deep snow.</def> <altsp>[Written also <asp>tobogan</asp>, and <asp>tarbogan</asp>.]</altsp><br/
[<source>1913 Webster</source>]</p>

<p><ent>Toboggan</ent><br/
<hw>To*bog"gan</hw> <pr>(?)</pr>, <pos>v. i.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Tobogganed</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Tobogganing</conjf>.]</vmorph> <def>To slide down hill over the snow or ice on a toboggan.</def>  <rj><au>Barilett.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tobogganist</ent><br/
<ent>Tobogganer</ent><br/
<mhw>{ <hw>To*bog"gan*er</hw> <pr>(?)</pr>, <hw>To*bog"gan*ist</hw> <pr>(?)</pr>, }</mhw> <pos>n.</pos> <def>One who practices tobogganing.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>To-break</ent><br/
<hw>To-break"</hw> <pr>(?)</pr>, <pos>v. t.</pos> <ety>[Pref. <ets>to-</ets> + <ets>break</ets>.]</ety> <def>To break completely; to break in pieces.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>With nose and mouth <qex>to-broke</qex>.</q> <rj><qau>Chaucer.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>To-brest</ent><br/
<hw>To-brest"</hw> <pr>(?)</pr>, <pos>v. t.</pos> <ety>[Pref. <ets>to-</ets> + <ets>brest</ets>.]</ety> <def>To burst or break in pieces.</def> <mark>[Obs.]</mark>  <rj><au>Chaucer.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Toby</ent><br/
<hw>To"by</hw> <pr>(?)</pr>, <pos>n.</pos>; <plu>pl. <plw>-bies</plw> <pr>(#)</pr></plu>. <ety>[Perh. from the proper name.]</ety> <def>A small jug, pitcher, or mug, generally used for ale, shaped somewhat like a stout man, with a cocked hat forming the brim.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Toccata</ent><br/
||<hw>Toc*ca"ta</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[It., fr. <ets>toccare</ets> to touch. See <er>Touch</er>.]</ety> <fld>(Mus.)</fld> <def>An old form of piece for the organ or harpsichord, somewhat in the free and brilliant style of the <xex>prelude</xex>, <xex>fantasia</xex>, or <xex>capriccio</xex>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Toccatina</ent><br/
<ent>Toccatella</ent><br/
<mhw>{ ||<hw>Toc`ca*tel"la</hw> <pr>(?)</pr>, ||<hw>Toc`ca*ti"na</hw> <pr>(?)</pr> }</mhw>, <pos>n.</pos>} <ety>[It.]</ety> <fld>(Music)</fld> <def>A short or simple toccata.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Tocher</ent><br/
<hw>Toch"er</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Gael. <ets>tochradh</ets>.]</ety> <def>Dowry brought by a bride to her husband.</def> <mark>[Scot.]</mark>  <rj><au>Burns.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tockay</ent><br/
<hw>Tock"ay</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Zool.)</fld> <def>A spotted lizard native of India.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Toco</ent><br/
<hw>To"co</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Zool.)</fld> <def>A toucan (<spn>Ramphastos toco</spn>) having a very large beak. See <xex>Illust.</xex> under <er>Toucan</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tocology</ent><br/
<hw>To*col"o*gy</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Gr. <?/ a birth + <ets>-logy</ets>.]</ety> <def>The science of obstetrics, or midwifery; that department of medicine which treats of parturition.</def> <altsp>[Written also <asp>tokology</asp>.]</altsp><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tocororo</ent><br/
||<hw>To*co*ro"ro</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Probably from the native name through the Spanish: cf. Sp. <ets>tocororo</ets>.]</ety> <fld>(Zool.)</fld> <def>A cuban trogon (<spn>Priotelus temnurus</spn>) having a serrated bill and a tail concave at the end.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tocsin</ent><br/
<hw>Toc"sin</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F., fr. OF. <ets>toquier</ets> to touch, F. <ets>toquer</ets> (originally, a dialectic form of F. <ets>toucher</ets>) + <ets>seint</ets> (for <ets>sein</ets>) a bell, LL. <ets>signum</ets>, fr. L. <ets>signum</ets> a sign, signal. See <er>Touch</er>, and <er>Sign</er>.]</ety> <def>An alarm bell, or the ringing of a bell for the purpose of alarm.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The loud <qex>tocsin</qex> tolled their last alarm.</q> <rj><qau>Campbell.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tod</ent><br/
<hw>Tod</hw> <pr>(t<ocr/d)</pr>, <pos>n.</pos> <ety>[Akin to D. <ets>todde</ets> a rag, G. <ets>zotte</ets> shag, rag, a tuft of hair, Icel. <ets>toddi</ets> a piece of a thing, a tod of wool.]</ety> <sn>1.</sn> <def>A bush; a thick shrub; a bushy clump.</def> <mark>[R.]</mark> <ldquo/An ivy <xex>todde</xex>.<rdquo/  <rj><au>Spenser.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>The ivy <qex>tod</qex> is heavy with snow.</q> <rj><qau>Coleridge.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>An old weight used in weighing wool, being usually twenty-eight pounds.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>A fox; -- probably so named from its bushy tail.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The wolf, the <qex>tod</qex>, the brock.</q> <rj><qau>B. Jonson.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Tod stove</b></col>, <cd>a close stove adapted for burning small round wood, twigs, etc.</cd> <mark>[U. S.]</mark>  <rj><au>Knight.</au></rj></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tod</ent><br/
<hw>Tod</hw>, <pos>v. t. & i.</pos> <def>To weigh; to yield in tods.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>To-day</ent><br/
<hw>To-day"</hw> <pr>(?)</pr>, <pos>adv.</pos> <ety>[AS. <ets>t<omac/ d<ae/g</ets>. See <er>To</er>, <pos>prep.</pos>, and <er>Day</er>.]</ety> <def>On this day; on the present day.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Worcester's horse came but <qex>to-day</qex>.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>To-day</ent><br/
<hw>To-day"</hw>, <pos>n.</pos> <def>The present day.</def>
<-- usu. spelt <asp>today.</asp> --><br/
[<source>1913 Webster</source>]</p>

<p><q>On <qex>to-day</qex><br/
Is worth for me a thousand yesterdays.</q> <rj><qau>Longfellow.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><-- today adj. modern, recent. --><br/
[<source>1913 Webster</source>]</p>

<p><ent>Toddle</ent><br/
<hw>Tod"dle</hw> <pr>(?)</pr>, <pos>v. i.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Toddled</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Toddling</conjf> <pr>(?)</pr>.]</vmorph> <ety>[Akin to <ets>tottle</ets>, <ets>totter</ets>.]</ety> <def>To walk with short, tottering steps, as a child.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Toddle</ent><br/
<hw>Tod"dle</hw>, <pos>n.</pos> <def>A toddling walk.</def>  <rj><au>Trollope.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Toddler</ent><br/
<hw>Tod"dler</hw> <pr>(?)</pr>, <pos>n.</pos> <def>One who toddles; especially, a young child.</def>  <rj><au>Mrs. Gaskell.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Toddy</ent><br/
<hw>Tod"dy</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Formed from Hind. <ets>t<amac/<?/<imac/</ets> the juice of the palmyra tree, popularly, <ets>toddy</ets>, fr. <ets>t<amac/<?/</ets> the palmyra tree, Skr. <ets>t<amac/la</ets>.]</ety> <sn>1.</sn> <def>A juice drawn from various kinds of palms in the East Indies; or, a spirituous liquor procured from it by fermentation.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A mixture of spirit and hot water sweetened.</def><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ <xex>Toddy</xex> differs from <xex>grog</xex> in having a less proportion of spirit, and is being made hot and sweetened.</note><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Toddy bird</b></col> <fld>(Zool.)</fld>, <cd>a weaver bird of the East Indies and India: -- so called from its fondness for the juice of the palm.</cd> -- <col><b>Toddy cat</b></col> <fld>(Zool.)</fld>, <cd>the common paradoxure; the palm cat.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>To-do</ent><br/
<hw>To-do"</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[<ets>To</ets> + <ets>do</ets>.   Cf. <er>Ado</er>.]</ety> <def>Bustle; stir; commotion; ado.</def> <mark>[Colloq.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tody</ent><br/
<hw>To"dy</hw> <pr>(?)</pr>, <pos>n.</pos>; <plu><it>pl.</it> Todies <pr>(#)</pr>.</plu> <ety>[Cf. NL. <ets>todus</ets>, F. <ets>todier</ets>, G. <ets>todvogel</ets>.]</ety> <fld>(Zool.)</fld> <def>Any one of several species of small insectivorous West Indian birds of the genus <gen>Todus</gen>. They are allied to the kingfishers.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Toe</ent><br/
<hw>Toe</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[OE. <ets>too</ets>, <ets>taa</ets>, AS. <ets>t<amac/</ets>; akin to D. <ets>teen</ets>, G. <ets>zehe</ets>, OHG. <ets>z<emac/ha</ets>, Icel. <ets>t<amac/</ets>, Sw. <ets>t<aring/</ets>, Dan. <ets>taa</ets>; of uncertain origin. <root/60.]</ety> <sn>1.</sn> <fld>(Anat.)</fld> <def>One of the terminal members, or digits, of the foot of a man or an animal.</def> <ldquo/Each one, tripping on his <xex>toe</xex>.<rdquo/  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Zool.)</fld> <def>The fore part of the hoof or foot of an animal.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>Anything, or any part, corresponding to the toe of the foot; <as>as, the <ex>toe</ex> of a boot; the <ex>toe</ex> of a skate</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <fld>(Mach.)</fld> <sd>(a)</sd> <def>The journal, or pivot, at the lower end of a revolving shaft or spindle, which rests in a step.</def> <sd>(b)</sd> <def>A lateral projection at one end, or between the ends, of a piece, as a rod or bolt, by means of which it is moved.</def> <sd>(c)</sd> <def>A projection from the periphery of a revolving piece, acting as a cam to lift another piece.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Toe biter</b></col> <fld>(Zool.)</fld>, <cd>a tadpole; a polliwig.</cd> -- <col><b>Toe drop</b></col> <fld>(Med.)</fld>, <cd>a morbid condition of the foot in which the toe is depressed and the heel elevated, as in <xex>talipes equinus</xex>. See <er>Talipes</er>.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Toe</ent><br/
<hw>Toe</hw>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Toed</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Toeing</conjf>.]</vmorph> <def>To touch or reach with the toes; to come fully up to; <as>as, to <ex>toe</ex> the mark</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Toe</ent><br/
<hw>Toe</hw>, <pos>v. i.</pos> <def>To hold or carry the toes (in a certain way).</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>To toe in</b></col>, <cd>to stand or carry the feet in such a way that the toes of either foot incline toward the other.</cd> -- <col><b>To toe out</b></col>, <cd>to have the toes of each foot, in standing or walking, incline from the other foot.</cd></cs>
<-- (Automobiles) <col><b>toe in</b></col>, to align the front wheels so that they point slightly toward each other. --><br/
[<source>1913 Webster</source>]</p>

<p><ent>Toed</ent><br/
<hw>Toed</hw> <pr>(?)</pr>, <pos>a.</pos> <sn>1.</sn> <def>Having (such or so many) toes; -- chiefly used in composition; <as>as, narrow-<ex>toed</ex>, four-<ex>toed</ex></as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Carp.)</fld> <def>Having the end secured by nails driven obliquely, said of a board, plank, or joist serving as a brace, and in general of any part of a frame secured to other parts by diagonal nailing.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Toe drop</ent><br/
<hw>Toe drop</hw>. <fld>(Med.)</fld> <def>A morbid condition of the foot in which the toe is depressed and the heel elevated.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Toe hold</ent><br/
<hw>Toe hold</hw>. <fld>(Wrestling)</fld> <def>A hold in which the agressor bends back his opponent's foot.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>To-fall</ent><br/
<hw>To-fall"</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Arch.)</fld> <def>A lean-to. See <er>Lean-to</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Toff</ent><br/
<hw>Toff</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Etym. uncertain.]</ety> <def>A fop; a beau; a swell.</def> <mark>[Slang, Eng.]</mark>  <rj><au>Kipling.</au></rj><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Toffy</ent><br/
<ent>Toffee</ent><br/
<mhw>{ <hw>Tof"fee</hw> <pr>(?)</pr>, <hw>Tof"fy</hw> <pr>(?)</pr> }</mhw>, <pos>n.</pos> <def>Taffy.</def> <mark>[Eng.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Toforn</ent><br/
<ent>Tofore</ent><br/
<mhw>{ <hw>To*fore"</hw> <pr>(?)</pr>, <hw>To*forn"</hw> <pr>(?)</pr> }</mhw>, <pos>prep. & adv.</pos> <ety>[AS. <ets>t<omac/foran</ets>. See <er>To</er>, <pos>prep.</pos>, <er>Fore</er>.]</ety> <def>Before.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q><qex>Toforn</qex> him goeth the loud minstrelsy.</q> <rj><qau>Chaucer.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Would thou wert as thou <qex>tofore</qex> hast been!</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Toft</ent><br/
<hw>Toft</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[OE. <ets>toft</ets> a knoll; akin to LG. <ets>toft</ets> a field hedged in, not far from a house, Icel. <ets>topt</ets> a green knoll, grassy place, place marked out for a house, Dan. <ets>toft</ets>.]</ety> <sn>1.</sn> <def>A knoll or hill.</def> <mark>[Obs.]</mark> <ldquo/A tower on a <xex>toft</xex>.<rdquo/  <rj><au>Piers Plowman.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A grove of trees; also, a plain.</def> <mark>[Prov. Eng.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <fld>(O. Eng. Law)</fld> <def>A place where a messuage has once stood; the site of a burnt or decayed house.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Toftman</ent><br/
<hw>Toft"man</hw> <pr>(?)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Toftmen</plw> <pr>(<?/)</pr>.</plu> <def>The owner of a toft. See <er>Toft</er>, 3.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tofus</ent><br/
||<hw>To"fus</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L., tufa.]</ety> <sn>1.</sn> <def>Tophus.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Min.)</fld> <def>Tufa. See under <er>Tufa</er>, and <er>Toph</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tog</ent><br/
<hw>Tog</hw> <pr>(?)</pr>, <pos>v. t. & i.</pos> <def>To put toggery, or togs, on; to dress; -- usually with <xex>out</xex>, implying care, elaborateness, or the like.</def> <mark>[Colloq. or Slang]</mark>  <rj><au>Harper's Weekly.</au></rj><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Toga</ent><br/
||<hw>To"ga</hw> <pr>(?)</pr>, <pos>n.</pos>; <plu><it>pl.</it> E. <plw>Togas</plw> <pr>(#)</pr>, L. <plw>Togae</plw> <pr>(#)</pr>.</plu> <ety>[L., akin to <ets>tegere</ets> to cover. See <er>Thatch</er>.]</ety> <fld>(Rom. Antiq.)</fld> <def>The loose outer garment worn by the ancient Romans, consisting of a single broad piece of woolen cloth of a shape approaching a semicircle. It was of undyed wool, except the border of the toga praetexta.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs>||<col><b>Toga praetexta</b></col>. <ety>[L.]</ety>, <cd>a toga with a broad purple border, worn by children of both sexes, by magistrates, and by persons engaged in sacred rites.</cd> -- ||<col><b>Toga virilis</b></col> <ety>[L.]</ety>, <cd>the manly gown; the common toga. This was assumed by Roman boys about the time of completing their fourteenth year.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Togated</ent><br/
<hw>To"ga*ted</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>togatus</ets>, from <ets>toga</ets> a toga.]</ety> <def>Dressed in a toga or gown; wearing a gown; gowned.</def> <mark>[R.]</mark>  <rj><au>Sir M. Sandys.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Toged</ent><br/
<hw>To"ged</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Togated.</def> <mark>[Obs. or R.]</mark>  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Together</ent><br/
<hw>To*geth"er</hw> <pr>(?)</pr>, <pos>adv.</pos> <ety>[OE. <ets>togedere</ets>, <ets>togidere</ets>, AS. <ets>t<omac/g<ae/dere</ets>, <ets>t<omac/g<ae/dre</ets>, <ets>t<omac/gadere</ets>; <ets>t<omac/</ets> to + <ets>gador</ets> together. <root/29. See <er>To</er>, <pos>prep.</pos>, and <er>Gather</er>.]</ety> <sn>1.</sn> <def>In company or association with respect to place or time; <as>as, to live <ex>together</ex> in one house; to live <ex>together</ex> in the same age; they walked <ex>together</ex> to the town.</as></def><br/
[<source>1913 Webster</source>]</p>

<p><q>Soldiers can never stand idle long <qex>together</qex>.</q> <rj><qau>Landor.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>In or into union; into junction; <as>as, to sew, knit, or fasten two things <ex>together</ex>; to mix things <ex>together</ex>.</as></def><br/
[<source>1913 Webster</source>]</p>

<p><q>The king joined humanity and policy <qex>together</qex>.</q> <rj><qau>Bacon.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>In concert; with mutual cooperation; <as>as, the allies made war upon France <ex>together</ex></as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Together with</b></col>, <cd>in union with; in company or mixture with; along with.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><q>Take the bad <qex>together with</qex> the good.</q> <rj><qau>Dryden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Toggery</ent><br/
<hw>Tog"ger*y</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Cf. <er>Togated</er>.]</ety> <def>Clothes; garments; dress; <as>as, fishing <ex>toggery</ex></as>.</def> <mark>[Colloq.]</mark>
<-- now the same idea is expressed as <er>togs</er> --><br/
[<source>1913 Webster</source>]</p>

<p><ent>Toggle</ent><br/
<hw>Tog"gle</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Cf. <er>Tug</er>.]</ety> <altsp>[Written also <asp>toggel</asp>.]</altsp> <sn>1.</sn> <fld>(Naut.)</fld> <def>A wooden pin tapering toward both ends with a groove around its middle, fixed transversely in the eye of a rope to be secured to any other loop or bight or ring; a kind of button or frog capable of being readily engaged and disengaged for temporary purposes.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Mach.)</fld> <def>Two rods or plates connected by a toggle joint.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>A toggle switch.</def><br/
[<source>PJC</source>]</p>

<p><ent>Toggle</ent><br/
<hw>Tog"gle</hw> <pr>(?)</pr>, <pos>v. t.</pos> <fld>(Computer programming)</fld> <def>To change the value of (a program variable) by activating a toggle switch; <as>as, to <ex>toggle</ex> the view from character to graphic mode; to <ex>toggle</ex> the keyboard input from insert to overtype mode</as>.</def><br/
[<source>PJC</source>]</p>

<p><cs><col><b>Toggle iron</b></col>, <cd>a harpoon with a pivoted crosspiece in a mortise near the point to prevent it from being drawn out when a whale, shark, or other animal, is harpooned.</cd> -- <col><b>Toggle joint</b></col>, <cd>an elbow or knee joint, consisting of two bars so connected that they may be brought quite or nearly into a straight line, and made to produce great endwise pressure, when any force is applied to bring them into this position.</cd></cs></p>

<p><ent>Toggle switch</ent><br/
<hw>Tog"gle switch`</hw> <pr>(?)</pr>, <pos>n.</pos> <sn>1.</sn> <fld>(Elec.)</fld>  <def>an electrical switch operated by pushing a lever through a small angle of deflection.  The lever has a spring which returns it to its original position after the pressure applied by the operator is released.</def><br/
[<source>PJC</source>]</p>

<p><sn>2.</sn> <fld>(Computer programming)</fld> <def>A mechanism for acquiring input from an operator, such that taking some action (such as pressing a function key on a keyboard) will cause a program variable to take a new value. The values are usually changed in cyclic fashion, so that a certain number of activations of the toggle returns the variable to its initial value.  When there are two values to the variable, each activation of the toggle causes the variable to assume the alternate value.</def><br/
[<source>PJC</source>]</p>

<p><-- Illustrations here of toggle iron and togle joint. --><br/
[<source>1913 Webster</source>]</p>

<p><ent>Toght</ent><br/
<hw>Toght</hw> <pr>(t<add/t)</pr>, <pos>a.</pos> <def>Taut.</def> <mark>[Obs.]</mark>  <rj><au>Chaucer.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Togidres</ent><br/
<ent>Togider</ent><br/
<mhw>{ <hw>To*gid"er</hw> <pr>(?)</pr>, <hw>To*gid"res</hw> <pr>(?)</pr> }</mhw>, <pos>adv.</pos> <def>Together.</def> <mark>[Obs.]</mark>  <rj><au>Chaucer.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Togs</ent><br/
<hw>Togs</hw> <pr>(?)</pr>, <pos>n. pl.</pos> <ety>[See <er>Toggery</er>.]</ety> <def>Clothes; garments; toggery.</def> <mark>[Colloq. or Slang]</mark><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Togue</ent><br/
<hw>Togue</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[From the American Indian name.]</ety> <fld>(Zool.)</fld> <def>The namaycush.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tohew</ent><br/
<hw>To*hew"</hw> <pr>(?)</pr>, <pos>v. t.</pos> <ety>[Pref. <ets>to-</ets> + <ets>hew</ets>.]</ety> <def>To hew in pieces.</def> <mark>[Obs.]</mark>  <rj><au>Chaucer.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tohubohu</ent><br/
||<hw>To"hu*bo"hu</hw> <pr>(t<omac/"h<oomac/-b<omac/"h<oomac/)</pr>, <pos>n.</pos> <ety>[F. <ets>tohu-bohu</ets>, fr. Heb. <ets>toh<umac/ wa boh<umac/</ets> without form and void (<au>Gen. i. 2</au>).]</ety> <def>Chaos; confusion.</def> <mark>[Rare]</mark> <br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><q>Was ever such a <qex>tohubohu</qex> of people as there assembles?</q>  <rj><qau>Thackeray.</qau></rj><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Toil</ent><br/
<hw>Toil</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F. <ets>toiles</ets>, pl., toils, nets, fr. <ets>toile</ets> cloth, canvas, spider web, fr. L. <ets>tela</ets> any woven stuff, a web, fr. <ets>texere</ets> to weave. See <er>Text</er>, and cf. <er>Toilet</er>.]</ety> <def>A net or snare; any thread, web, or string spread for taking prey; -- usually in the plural.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>As a Numidian lion, when first caught,<br/
Endures the <qex>toil</qex> that holds him.</q> <rj><qau>Denham.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Then <qex>toils</qex> for beasts, and lime for birds, were found.</q> <rj><qau>Dryden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Toil</ent><br/
<hw>Toil</hw>, <pos>v. i.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Toiled</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Toiling</conjf>.]</vmorph> <ety>[OE. <ets>toilen</ets> to pull about, to toil; of uncertain origin; cf. OD. <ets>teulen</ets>, <ets>tuylen</ets>, to labor, till, or OF. <ets>tooillier</ets>, <ets>toailler</ets>, to wash, rub (cf. <er>Towel</er>); or perhaps ultimately from the same root as E. <ets>tug</ets>.]</ety> <def>To exert strength with pain and fatigue of body or mind, especially of the body, with efforts of some continuance or duration; to labor; to work.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Toil</ent><br/
<hw>Toil</hw>, <pos>v. t.</pos> <sn>1.</sn> <def>To weary; to overlabor.</def> <mark>[Obs.]</mark> <ldquo/<xex>Toiled</xex> with works of war.<rdquo/  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To labor; to work; -- often with <xex>out</xex>.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>Places well <qex>toiled</qex> and husbanded.</q> <rj><qau>Holland.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>[I] <qex>toiled</qex> out my uncouth passage.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><-- p. 1515 --><br/
[<source>1913 Webster</source>]</p>

<p><ent>Toil</ent><br/
<hw>Toil</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[OE. <ets>toil</ets> turmoil, struggle; cf. OD. <ets>tuyl</ets> labor, work. See <er>Toil</er>, <pos>v.</pos>]</ety> <def>Labor with pain and fatigue; labor that oppresses the body or mind, esp. the body.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>My task of servile <qex>toil</qex>.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>After such bloody <qex>toil</qex>, we bid good night.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ <xex>Toil</xex> is used in the formation of compounds which are generally of obvious signification; as, <xex>toil</xex>-strung, <xex>toil</xex>-wasted, <xex>toil</xex>-worn, and the like.</note><br/
[<source>1913 Webster</source>]</p>

<p><syn><b>Syn.</b> -- Labor; drudgery; work; exertion; occupation; employment; task; travail.</syn> <usage> -- <er>Toil</er>, <er>Labor</er>, <er>Drudgery</er>. <xex>Labor</xex> implies strenuous exertion, but not necessary such as overtasks the faculties; <xex>toil</xex> denotes a severity of labor which is painful and exhausting; <xex>drudgery</xex> implies mean and degrading work, or, at least, work which wearies or disgusts from its minuteness or dull uniformity.</usage><br/
[<source>1913 Webster</source>]</p>

<p><q>You do not know the heavy grievances,<br/
The <qex>toils</qex>, the <qex>labors</qex>, weary <qex>drudgeries</qex>,<br/
Which they impose.</q> <rj><qau>Southern.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>How often have I blessed the coming day,<br/
When <qex>toil</qex> remitting lent its turn to play.</q> <rj><qau>Goldsmith.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Toiler</ent><br/
<hw>Toil"er</hw> <pr>(?)</pr>, <pos>n.</pos> <def>One who toils, or labors painfully.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Toilet</ent><br/
<hw>Toi"let</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F. <ets>toilette</ets>, dim. of <ets>toile</ets> cloth. See <er>Toil</er> a net.]</ety> <sn>1.</sn> <def>A covering of linen, silk, or tapestry, spread over a table in a chamber or a dressing room.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A dressing table.</def>  <rj><au>Pope.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>Act or mode of dressing, or that which is arranged in dressing; attire; dress; <as>as, her <ex>toilet</ex> is perfect</as>.</def> <altsp>[Written also <asp>toilette</asp>.]</altsp><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Toilet glass</b></col>, <cd>a looking-glass for a toilet table or for a dressing room.</cd> -- <mcol><col><b>Toilet service</b></col>, <col><b>Toilet set</b></col></mcol>, <cd>earthenware, glass, and other utensils for a dressing room.</cd> -- <col><b>Toilet table</b></col>, <cd>a dressing table; a toilet. See def. 2 above.</cd> -- <col><b>To snake one's toilet</b></col>, <cd>to dress one's self; especially, to dress one's self carefully.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Toilette</ent><br/
<hw>Toi*lette"</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F.]</ety> <def>See <er>Toilet</er>, 3.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Toilful</ent><br/
<hw>Toil"ful</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Producing or involving much toil; laborious; toilsome; <as>as, <ex>toilful</ex> care</as>.</def>  <rj><au>Mickle.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Toilinette</ent><br/
<hw>Toi`li*nette"</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F. <ets>toilinet</ets>. See <er>Toil</er> a net.]</ety> <def>A cloth, the weft of which is of woolen yarn, and the warp of cotton and silk, -- used for waistcoats.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Toilless</ent><br/
<hw>Toil"less</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Free from toil.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Toilsome</ent><br/
<hw>Toil"some</hw> <pr>(toil"s<ucr/m)</pr>, <pos>a.</pos> <def>Attended with toil, or fatigue and pain; laborious; wearisome; <as>as, <ex>toilsome</ex> work</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>What can be <qex>toilsome</qex> in these pleasant walks?</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p>-- <wordforms><wf>Toil"some*ly</wf>, <pos>adv.</pos> -- <wf>Toil"some*ness</wf>, <pos>n.</pos></wordforms><br/
[<source>1913 Webster</source>]</p>

<p><ent>Toise</ent><br/
<hw>Toise</hw> <pr>(toiz)</pr>, <pos>n.</pos> <ety>[F., fr. LL. <ets>tesa</ets>, fr. L. <ets>tensus</ets>, fem. <ets>tensa</ets>, p. p. of <ets>tendere</ets> to stretch, extend. See <er>Tense</er>, <pos>a.</pos>]</ety> <def>An old measure of length in France, containing six French feet, or about 6.3946 French feet.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Toison</ent><br/
||<hw>Toi`son"</hw> <pr>(tw<adot/`z<ocir/n")</pr>, <pos>n.</pos> <ety>[F.]</ety> <def>A sheep's fleece.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Toison d'or</ent><br/
||<hw>Toi`son" d'or"</hw> <pr>(d<osl/r")</pr>. <ety>[F.]</ety> <def>Lit., the golden fleece; specif., the order of the Golden Fleece, or its jewel.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Tokay</ent><br/
<hw>To*kay"</hw> <pr>(t<osl/*k<amac/")</pr>, <pos>n.</pos> <ety>[Named fr. <etsep>Tokay</etsep> in Hungary.]</ety> <sn>1.</sn> <fld>(Bot.)</fld> <def>A grape of an oval shape and whitish color.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A rich Hungarian wine made from Tokay grapes.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Token</ent><br/
<hw>To"ken</hw> <pr>(t<omac/"k'n)</pr>, <pos>n.</pos> <ety>[OE. <ets>token</ets>, <ets>taken</ets>, AS. <ets>t<amac/cen</ets>; akin to OFries. <ets>t<emac/ken</ets>, OS. <ets>t<emac/kan</ets>, D. <ets>teeken</ets>, G. <ets>zeichen</ets>, OHG. <ets>Zeihhan</ets>, Icel. <ets>t<amac/kan</ets>, <ets>teiken</ets>, Sw. <ets>tecken</ets>, Dan. <ets>tegn</ets>, Goth. <ets>taikns</ets> sign, token, ga<ets>teihan</ets> to tell, show, AS. <ets>te<oacute/n</ets> to accuse, G. <ets>zeihen</ets>, OHG. <ets>z<imac/han</ets>, G. <ets>zeigen</ets> to show, OHG. <ets>zeig<omac/n</ets>, Icel. <ets>tj<amac/</ets>, L. <ets>dicere</ets> to say, Gr. <grk>deikny`nai</grk> to show, Skr. <ets>di<cced/</ets>.  Cf. <er>Diction</er>, <er>Teach</er>.]</ety> <sn>1.</sn> <def>Something intended or supposed to represent or indicate another thing or an event; a sign; a symbol; <as>as, the rainbow is a <ex>token</ex> of God's covenant established with Noah</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A memorial of friendship; something by which the friendship of another person is to be kept in mind; a memento; a souvenir.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>This is some <qex>token</qex> from a never friend.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>Something given or shown as a symbol or guarantee of authority or right; a sign of authenticity, of power, good faith, etc.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Say, by this <qex>token</qex>, I desire his company.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>A piece of metal intended for currency, and issued by a private party, usually bearing the name of the issuer, and redeemable in lawful money. Also, a coin issued by government, esp. when its use as lawful money is limited and its intrinsic value is much below its nominal value.</def><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ It is now made unlawful for private persons to issue tokens.</note><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <fld>(Med.)</fld> <def>A livid spot upon the body, indicating, or supposed to indicate, the approach of death.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>Like the fearful <qex>tokens</qex> of the plague,<br/
Are mere forerunners of their ends.</q> <rj><qau>Beau. & Fl.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>6.</sn> <fld>(Print.)</fld> <def>Ten and a half quires, or, commonly, 250 sheets, of paper printed on both sides; also, in some cases, the same number of sheets printed on one side, or half the number printed on both sides.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>7.</sn> <fld>(Ch. of Scot.)</fld> <def>A piece of metal given beforehand to each person in the congregation who is permitted to partake of the Lord's Supper.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>8.</sn> <fld>(Mining)</fld> <def>A bit of leather having a peculiar mark designating a particular miner. Each hewer sends one of these with each corf or tub he has hewn.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>9.</sn> <fld>(Weaving)</fld> <def>In a Jacquard loom, a colored signal to show the weaver which shuttle to use.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><cs><col><b>Token money</b></col>, <cd>money which is lawfully current for more than its real value. See <er>Token</er>, <pos>n.</pos>, 4.</cd> -- <col><b>Token sheet</b></col> <fld>(Print.)</fld>, <cd>the last sheet of each token.</cd>  <rj><au>W. Savage.</au></rj></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Token</ent><br/
<hw>To"ken</hw>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Tokened</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Tokening</conjf>.]</vmorph> <ety>[AS. <ets>t<amac/cnian</ets>, fr. <ets>t<amac/cen</ets> token. See <er>Token</er>, <pos>n.</pos>]</ety> <def>To betoken.</def> <mark>[Obs.]</mark>  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tokened</ent><br/
<hw>To"kened</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Marked by tokens, or spots; <as>as, the <ex>tokened</ex> pestilence</as>.</def> <mark>[Obs.]</mark>  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tokenless</ent><br/
<hw>To"ken*less</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Without a token.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tokin</ent><br/
<hw>Tok"in</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A tocsin.</def> <mark>[Obs.]</mark>  <rj><au>Halliwell.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tol</ent><br/
<hw>Tol</hw> <pr>(t<omac/l)</pr>, <pos>v. t.</pos> <fld>(Law)</fld> <def>To take away. See <er>Toll</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tola</ent><br/
||<hw>To"la</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Hind., from Skr. <ets>tul<amac/</ets> a balance.]</ety> <def>A weight of British India. The standard tola is equal to 180 grains.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tolane</ent><br/
<hw>To*lane"</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[From <er>Toluene</er>.]</ety> <fld>(Chem.)</fld> <def>A hydrocarbon, <chform>C14H10</chform>, related both to the acetylene and the aromatic series, and produced artificially as a white crystalline substance; -- called also <altname>diphenyl acetylene</altname>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tolbooth</ent><br/
<hw>Tol"booth`</hw> <pr>(?)</pr>, <pos>n.</pos> <def>See <er>Tollbooth</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Told</ent><br/
<hw>Told</hw> <pr>(t<omac/ld)</pr>, <def><pos>imp. & p. p.</pos> of <er>Tell</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tole</ent><br/
<hw>Tole</hw> <pr>(t<omac/l)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Toled</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Toling</conjf>.]</vmorph> <ety>[OE. <ets>tollen</ets> to draw, to entice; of uncertain origin.  Cf. <er>Toll</er> to ring a bell.]</ety> <def>To draw, or cause to follow, by displaying something pleasing or desirable; to allure by some bait.</def> <altsp>[Written also <asp>toll</asp>.]</altsp><br/
[<source>1913 Webster</source>]</p>

<p><q>Whatever you observe him to be more frighted at then he should, <qex>tole</qex> him on to by insensible degrees, till at last he masters the difficulty.</q><br/
[<source>1913 Webster</source>]</p>

<p><ent>Toledo</ent><br/
<hw>To*le"do</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A sword or sword blade made at <etsep>Toledo</etsep> in Spain, which city was famous in the 16th and 17th centuries for the excellence of its weapons.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tolerability</ent><br/
<hw>Tol`er*a*bil"i*ty</hw> <pr>(?)</pr>, <pos>n.</pos> <def>The quality or state of being tolerable.</def> <mark>[R.]</mark>  <rj><au>Fuller. Wordsworth.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tolerable</ent><br/
<hw>Tol"er*a*ble</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>tolerabilis</ets>: cf. F. <ets>tol<eacute/rable</ets>. See <er>Tolerate</er>.]</ety> <sn>1.</sn> <def>Capable of being borne or endured; supportable, either physically or mentally.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>As may affect the earth with cold and heat<br/
Scarce <qex>tolerable</qex>.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Moderately good or agreeable; not contemptible; not very excellent or pleasing, but such as can be borne or received without disgust, resentment, or opposition; passable; <as>as, a <ex>tolerable</ex> administration; a <ex>tolerable</ex> entertainment; a <ex>tolerable</ex> translation</as>.</def>  <rj><au>Dryden.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p>-- <wordforms><wf>Tol"er*a*ble*ness</wf>, <pos>n.</pos> -- <wf>Tol"er*a*bly</wf>, <pos>adv.</pos></wordforms><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tolerance</ent><br/
<hw>Tol"er*ance</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>tolerantia</ets>: cf. F. <ets>tol<eacute/rance</ets>.]</ety> <sn>1.</sn> <def>The power or capacity of enduring; the act of enduring; endurance.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Diogenes, one frosty morning, came into the market place, shaking, to show his <qex>tolerance</qex>.</q> <rj><qau>Bacon.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>The endurance of the presence or actions of objectionable persons, or of the expression of offensive opinions; toleration.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <fld>(Med.)</fld> <def>The power possessed or acquired by some persons of bearing doses of medicine which in ordinary cases would prove injurious or fatal.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <fld>(Forestry)</fld> <def>Capability of growth in more or less shade.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><sn>5.</sn> <def>the allowed amount of variation from the standard or from exact conformity to the specified dimensions, weight, hardness, voltage etc., in various mechanical or electrical devices or operations; -- caklled also <altname>allowance</altname></def> <specif>specif.:</specif> <fld>(Coinage)</fld> <def>The amount which coins, either singly or in lots, are legally allowed to vary above or below the standard of weight or fineness.</def><br/
[<source>Webster 1913 Suppl.</source> <source>+PJC</source>]</p>

<p><sn>6.</sn> <fld>(Biochemistry)</fld> <def>the capacity to resist the deleterious action of a chemical agent normally harmful to the organism; <as>as, the acquired <ex>tolerance</ex> of bacteria to anitbiotics</as>.</def><br/
[<source>PJC</source>]</p>

<p><sn>7.</sn> <fld>(Immunology)</fld> <def>the acquired inability to respond with an immune reaction to an antigen to which the organism normally responds; -- called also <altname>immunotolerance</altname>, <altname>immunological tolerance</altname>, or <altname>immune tolerance</altname>.  Such tolerance may be induced by exposing an animal to the antigen at a very early stage of life, prior to maturation of the immune system, or, in adults, by exposing the animal to repeated low doses of a weak protein antigen (<stype>low-zone tolerance</stype>), or to a large amount of an antigen (<stype>high-zone tolerance</stype>).</def><br/
[<source>PJC</source>]</p>

<p><cs><col><b>Tolerance of the mint</b></col>. <fld>(Coinage)</fld> <cd>Same as <cref>Remedy of the mint</cref>. See under <er>Remedy</er>.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tolerant</ent><br/
<hw>Tol"er*ant</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>tolerans</ets>, p. pr. of <ets>tolerare</ets> to tolerate: cf. F. <ets>tol<eacute/rant</ets>. See <er>Tolerate</er>.]</ety> <def>Inclined to tolerate; favoring toleration; forbearing; indulgent.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tolerate</ent><br/
<hw>Tol"er*ate</hw> <pr>(?)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Tolerated</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Tolerating</conjf>.]</vmorph> <ety>[L. <ets>toleratus</ets>, p. p. of <ets>tolerare</ets>, fr. the same root as <ets>tollere</ets> to lift up, <ets>tuli</ets>, used as perfect of <ets>ferre</ets> to bear, <ets>latus</ets> (for <ets>tlatus</ets>), used as p. p. of <ets>ferre</ets> to bear, and E. <ets>thole</ets>. See <er>Thole</er>, and cf. <er>Atlas</er>, <er>Collation</er>, <er>Delay</er>, <er>Elate</er>, <er>Extol</er>, <er>Legislate</er>, <er>Oblate</er>, <er>Prelate</er>, <er>Relate</er>, <er>Superlative</er>, <er>Talent</er>, <er>Toll</er> to take away, <er>Translate</er>.]</ety> <def>To suffer to be, or to be done, without prohibition or hindrance; to allow or permit negatively, by not preventing; not to restrain; to put up with; <as>as, to <ex>tolerate</ex> doubtful practices</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Crying should not be <qex>tolerated</qex> in children.</q> <rj><qau>Locke.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>We <qex>tolerate</qex> them because property and liberty, to a degree, require that toleration.</q> <rj><qau>Burke.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><syn><b>Syn.</b> -- See <er>Permit</er>.</syn><br/
[<source>1913 Webster</source>]</p>

<p><ent>Toleration</ent><br/
<hw>Tol`er*a"tion</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>toleratio</ets>: cf. OF. <ets>toleration</ets>.]</ety> <sn>1.</sn> <def>The act of tolerating; the allowance of that which is not wholly approved.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Specifically, the allowance of religious opinions and modes of worship in a state when contrary to, or different from, those of the established church or belief.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>Hence, freedom from bigotry and severity in judgment of the opinions or belief of others, especially in respect to religious matters.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Toll</ent><br/
<hw>Toll</hw> <pr>(?)</pr>, <pos>v. t.</pos> <ety>[L. <ets>tollere</ets>. See <er>Tolerate</er>.]</ety> <fld>(O. Eng. Law)</fld> <def>To take away; to vacate; to annul.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Toll</ent><br/
<hw>Toll</hw>, <pos>v. t.</pos> <ety>[See <er>Tole</er>.]</ety> <sn>1.</sn> <def>To draw; to entice; to allure. See <er>Tole</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <ety>[Probably the same word as <ets>toll</ets> to draw, and at first meaning, to ring in order to draw people to church.]</ety> <def>To cause to sound, as a bell, with strokes slowly and uniformly repeated; <as>as, to <ex>toll</ex> the funeral bell</as>.</def> <ldquo/The sexton <xex>tolled</xex> the bell.<rdquo/  <rj><au>Hood.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>To strike, or to indicate by striking, as the hour; to ring a toll for; <as>as, to <ex>toll</ex> a departed friend</as>.</def>  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Slow <qex>tolls</qex> the village clock the drowsy hour.</q> <rj><qau>Beattie.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>To call, summon, or notify, by tolling or ringing.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>When hollow murmurs of their evening bells<br/
Dismiss the sleepy swains, and <qex>toll</qex> them to their cells.</q> <rj><qau>Dryden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Toll</ent><br/
<hw>Toll</hw>, <pos>v. i.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Tolled</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Tolling</conjf>.]</vmorph> <def>To sound or ring, as a bell, with strokes uniformly repeated at intervals, as at funerals, or in calling assemblies, or to announce the death of a person.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The country cocks do crow, the clocks do <qex>toll</qex>.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Now sink in sorrows with a <qex>tolling</qex> bell.</q> <rj><qau>Pope.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Toll</ent><br/
<hw>Toll</hw>, <pos>n.</pos> <def>The sound of a bell produced by strokes slowly and uniformly repeated.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Toll</ent><br/
<hw>Toll</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[OE. <ets>tol</ets>, AS. <ets>toll</ets>; akin to OS. & D. <ets>tol</ets>, G. <ets>zoll</ets>, OHG. <ets>zol</ets>, Icel. <ets>tollr</ets>, Sw. <ets>tull</ets>, Dan. <ets>told</ets>, and also to E. <ets>tale</ets>; -- originally, that which is counted out in payment. See <er>Tale</er> number.]</ety> <sn>1.</sn> <def>A tax paid for some liberty or privilege, particularly for the privilege of passing over a bridge or on a highway, or for that of vending goods in a fair, market, or the like.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Sax. & O. Eng. Law)</fld> <def>A liberty to buy and sell within the bounds of a manor.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>A portion of grain taken by a miller as a compensation for grinding.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Toll and team</b></col> <fld>(O. Eng. Law)</fld>, <cd>the privilege of having a market, and jurisdiction of villeins.</cd> <au>Burrill.</au> -- <col><b>Toll bar</b></col>, <cd>a bar or beam used on a canal for stopping boats at the tollhouse, or on a road for stopping passengers.</cd> -- <col><b>Toll bridge</b></col>, <cd>a bridge where toll is paid for passing over it.</cd> -- <col><b>Toll corn</b></col>, <cd>corn taken as pay for grinding at a mill.</cd> -- <col><b>Toll dish</b></col>, <cd>a dish for measuring toll in mills.</cd> -- <col><b>Toll gatherer</b></col>, <cd>a man who takes, or gathers, toll.</cd> -- <col><b>Toll hop</b></col>, <cd>a toll dish.</cd> <mark>[Obs.]</mark> <au>Crabb.</au> -- <col><b>Toll thorough</b></col> <fld>(Eng. Law)</fld>, <cd>toll taken by a town for beasts driven through it, or over a bridge or ferry maintained at its cost.</cd> <au>Brande & C.</au> -- <col><b>Toll traverse</b></col> <fld>(Eng. Law)</fld>, <cd>toll taken by an individual for beasts driven across his ground; toll paid by a person for passing over the private ground, bridge, ferry, or the like, of another.</cd> -- <col><b>Toll turn</b></col> <fld>(Eng. Law)</fld>, <cd>a toll paid at the return of beasts from market, though they were not sold.</cd> <au>Burrill.</au></cs><br/
[<source>1913 Webster</source>]</p>

<p><syn><b>Syn.</b> -- Tax; custom; duty; impost.</syn><br/
[<source>1913 Webster</source>]</p>

<p><ent>Toll</ent><br/
<hw>Toll</hw> <pr>(?)</pr>, <pos>v. i.</pos> <sn>1.</sn> <def>To pay toll or tallage.</def> <mark>[R.]</mark>  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To take toll; to raise a tax.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>Well could he [the miller] steal corn and <qex>toll</qex> thrice.</q> <rj><qau>Chaucer.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>No Italian priest<br/
Shall tithe or <qex>toll</qex> in our dominions.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Toll</ent><br/
<hw>Toll</hw>, <pos>v. t.</pos> <def>To collect, as a toll.</def>  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tollable</ent><br/
<hw>Toll"a*ble</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Subject to the payment of toll; <as>as, <ex>tollable</ex> goods</as>.</def>  <rj><au>Wright.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tollage</ent><br/
<hw>Toll"age</hw> <pr>(?)</pr>, <pos>n.</pos> <def>Payment of toll; also, the amount or quantity paid as toll.</def>  <rj><au>Drayton.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tollbooth</ent><br/
<hw>Toll"booth`</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[<ets>Toll</ets> a tax + <ets>booth</ets>.]</ety> <ety>[Written also <ets>tolbooth</ets>.]</ety> <sn>1.</sn> <def>A place where goods are weighed to ascertain the duties or toll.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>He saw Levy . . . sitting at the <qex>tollbooth</qex>.</q> <rj><qau>Wyclif (Mark ii. 14).</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>In Scotland, a burgh jail; hence, any prison, especially a town jail.</def>  <rj><au>Sir W. Scott.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tollbooth</ent><br/
<hw>Toll"booth`</hw>, <pos>v. t.</pos> <def>To imprison in a tollbooth.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>That they might <qex>tollbooth</qex> Oxford men.</q> <rj><qau>Bp. Corbet.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Toller</ent><br/
<hw>Toll"er</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[AS. <ets>tollere</ets>.]</ety> <def>A toll gatherer.</def> <ldquo/<xex>Tollers</xex> in markets.<rdquo/  <rj><au>Piers Plowman.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Toller</ent><br/
<hw>Toll"er</hw>, <pos>n.</pos> <def>One who tolls a bell.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tolletane</ent><br/
<hw>Tol"le*tane</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>Toletanus</ets>.]</ety> <def>Of or pertaining to Toledo in Spain; made in Toledo.</def> <mark>[Obs.]</mark> <ldquo/Tables <xex>Tolletanes</xex>.<rdquo/  <rj><au>Chaucer.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tollgate</ent><br/
<hw>Toll"gate`</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A gate where toll is taken.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tollhouse</ent><br/
<hw>Toll"house`</hw> <pr>(?)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Tollhouses</plw> <pr>(<?/)</pr>.</plu> <def>A house occupied by a receiver of tolls.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tollman</ent><br/
<hw>Toll"man</hw> <pr>(?)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Tollmen</plw> <pr>(<?/)</pr>.</plu> <def>One who receives or collects toll; a toll gatherer.</def>  <rj><au>Cowper.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tolmen</ent><br/
<hw>Tol"men</hw> <pr>(?)</pr>, <pos>n.</pos> <def>See <er>Dolmen</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tolsester</ent><br/
<hw>Tol"ses*ter</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[LL. <ets>tolsestrum</ets>.  Cf. <er>Toll</er> a tax.]</ety> <fld>(O. Eng. Law)</fld> <def>A toll or tribute of a sextary of ale, paid to the lords of some manors by their tenants, for liberty to brew and sell ale.</def>  <rj><au>Cowell.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tolsey</ent><br/
<hw>Tol"sey</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A tollbooth; also, a merchants' meeting place, or exchange.</def> <mark>[Obs.]</mark>  <rj><au>Halliwell.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tolstoian</ent><br/
<ent>Tolstoyan</ent><br/
<mhw>{ <hw>Tol*stoy"an</hw>, <hw>Tol*stoi"an</hw>  }</mhw> <pr>(?)</pr>, <pos>a.</pos> <def>Of or pertaining to <etsep>Tolstoy</etsep> (1828-1910).</def> -- <def2><pos>n.</pos>  <def>A follower of Tolstoy, who advocates and practices manual labor, simplicity of living, nonresistance, etc., holds that possession of wealth and ownership of property are sinful, and in religion rejects all teachings not coming from Christ himself.</def></def2><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Tolt</ent><br/
<hw>Tolt</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[LL. <ets>tolta</ets>, fr. L. <ets>tollere</ets> to take away.]</ety> <fld>(O. Eng. Law)</fld> <def>A writ by which a cause pending in a court baron was removed into a country court.</def>  <rj><au>Cowell.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Toltec</ent><br/
<hw>Tol"tec</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Ethnol.)</fld> <def>One of a race which formerly occupied Mexico.</def> -- <wordforms><wf>Tol"te*can</wf> <pr>(#)</pr>, <pos>a.</pos></wordforms><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tolu</ent><br/
<hw>To*lu"</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A fragrant balsam said to have been first brought from Santiago de Tolu, in New Granada. See <cref>Balsam of Tolu</cref>, under <er>Balsam</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Tolu tree</b></col> <fld>(Bot.)</fld>, <cd>a large tree (<spn>Myroxylon toluiferum</spn>), the wood of which is red in the center, and has an aromatic rose odor. It affords the balsam called <xex>tolu</xex>.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Toluate</ent><br/
<hw>Tol"u*ate</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Chem.)</fld> <def>A salt of any one of the toluic acids.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Toluene</ent><br/
<hw>Tol"u*ene</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[<ets>Tolu</ets> + benz<ets>ene</ets>.]</ety> <fld>(Chem.)</fld> <def>A hydrocarbon, <chform>C6H5.CH3</chform>, of the aromatic series, homologous with benzene, and obtained as a light mobile colorless liquid, by distilling tolu balsam, coal tar, etc.; -- called also <altname>methyl benzene</altname>, <altname>phenyl methane</altname>, etc.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Toluenyl</ent><br/
<hw>Tol`u*e"nyl</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[<ets>Toluene</ets> + <ets>-yl</ets>.]</ety> <fld>(Chem.)</fld> <def>Tolyl.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Toluic</ent><br/
<hw>To*lu"ic</hw> <pr>(?)</pr>, <pos>a.</pos> <fld>(Chem.)</fld> <def>Pertaining to, or designating, one of three metameric acids, <chform>CH3.C6H4.CO2H</chform>, which are related to toluene and analogous to benzoic acids. They are white crystalline substances, and are called respectively <stype>orthotoluic</stype> <it>acid</it>, <stype>metatoluic</stype> <it>acid</it>, and <stype>paratoluic</stype> <it>acid.</it></def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Toluid</ent><br/
<hw>Tol"u*id</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Chem.)</fld> <def>A complex double tolyl and toluidine derivative of glycocoll, obtained as a white crystalline substance.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Toluidine</ent><br/
<hw>To*lu"i*dine</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Chem.)</fld> <def>Any one of three metameric amido derivatives of toluene analogous to aniline, and called respectively <stype>orthtoluidine</stype>, <stype>metatoluidine</stype>, and <stype>paratoluidine</stype>; especially, the commonest one, or <stype>paratoluidine</stype>, which is obtained as a white crystalline substance.</def><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ It is used in the aniline dye industry, and constitutes the essential nucleus or radical of those dyes.</note><br/
[<source>1913 Webster</source>]</p>

<p><ent>Toluole</ent><br/
<ent>Toluol</ent><br/
<mhw>{ <hw>Tol"u*ol</hw>, <hw>Tol"u*ole</hw>}</mhw> <pr>(?)</pr>, <pos>n.</pos> <ety>[<ets>Tolu</ets> + benz<ets>ol</ets>.]</ety> <fld>(Chem.)</fld> <def>Same as <er>Toluene</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Toluric</ent><br/
<hw>To*lu"ric</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[<ets>Tol</ets>uic + <ets>uric</ets>.]</ety> <fld>(Chem.)</fld> <def>Of, pertaining to, or designating, any one of three isomeric crystalline acids, <chform>C9H10ON.CO2H</chform>, which are toluyl derivatives of glycocoll.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tolutation</ent><br/
<hw>Tol`u*ta"tion</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>tolutim</ets> on a trot, properly, lifting up the feet, akin to <ets>tollere</ets> to lift up.]</ety> <def>A pacing or ambling.</def> <mark>[Obs.]</mark>  <rj><au>Sir T. Browne.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Toluyl</ent><br/
<hw>Tol"u*yl</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[<ets>Tolu</ets>ic + <ets>-yl</ets>.]</ety> <fld>(Chem.)</fld> <def>Any one of the three hypothetical radicals corresponding to the three toluic acids.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Toluylene</ent><br/
<hw>Tol"u*yl*ene</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Chem.)</fld> <sd>(a)</sd> <def>Same as <er>Stilbene</er>.</def> <sd>(b)</sd> <def>Sometimes, but less properly, tolylene.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tolyl</ent><br/
<hw>Tol"yl</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[<ets>Tol</ets>uic + <ets>-yl</ets>.]</ety> <fld>(Chem.)</fld> <def>The hydrocarbon radical, <chform>CH3.C6H4</chform>, regarded as characteristic of certain compounds of the aromatic series related to toluene; <as>as, <ex>tolyl</ex> carbinol</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tolylene</ent><br/
<hw>Tol"yl*ene</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Chem.)</fld> <def>A hydrocarbon radical, <chform>C6H4.(CH2)2</chform>, regarded as characteristic of certain toluene derivatives.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tolypeutine</ent><br/
<hw>Tol`y*peu"tine</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Zool.)</fld> <def>The apar.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tom</ent><br/
<hw>Tom</hw> <pr>(?)</pr>, <pos>n.</pos> <def>The knave of trumps at gleek.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tom</ent><br/
<hw>Tom</hw>, <pos>n.</pos> <sn>1.</sn> <def>A familiar contraction of <altname>Thomas</altname>, a proper name of a man.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><sn>2.</sn>  <def>The male of certain animals; -- often used adjectively or in composition; <as>as, <ex>tom</ex> turkey, <ex>tom</ex>cat, etc.</as></def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Tomahawk</ent><br/
<hw>Tom"a*hawk</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Of American Indian origin; cf. Algonkin <ets>tomehagen</ets>, Mohegan <ets>tumnahegan</ets>, Delaware <ets>tamoihecan</ets>.]</ety> <def>A kind of war hatchet used by the American Indians. It was originally made of stone, but afterwards of iron.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tomahawk</ent><br/
<hw>Tom"a*hawk</hw>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Tomahawked</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Tomahawking</conjf>.]</vmorph> <def>To cut, strike, or kill, with a tomahawk.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tomaley</ent><br/
<hw>Tom"a`ley</hw> <pr>(?)</pr>, <pos>n.</pos> <def>The liver of the lobster, which becomes green when boiled; -- called also <altname>tomalline</altname>.</def><br/
[<source>1913 Webster</source>]</p>

<p><-- p. 1516 --><br/
[<source>1913 Webster</source>]</p>

<p><ent>Toman</ent><br/
||<hw>To*man"</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Per. <ets>t<omac/m<amac/n</ets>; from a Mongol word signifying, ten thousand.]</ety> <def>A money of account in Persia, whose value varies greatly at different times and places. Its average value may be reckoned at about two and a half dollars.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tom and Jerry</ent><br/
<hw>Tom and Jerry</hw>. <def>A hot sweetened drink of rum and water spiced with cinnamon, cloves, etc., and beaten up with eggs.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Tomato</ent><br/
<hw>To*ma"to</hw> <pr>(?)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Tomatoes</plw> <pr>(#)</pr>.</plu> <ety>[Sp. or Pg. <ets>tomate</ets>, of American Indian origin; cf. Mexican <ets>tomail</ets>.]</ety> <fld>(Bot.)</fld> <def>The fruit of a plant of the Nightshade family (<spn>Lycopersicum esculentun</spn>); also, the plant itself. The fruit, which is called also <altname>love apple</altname>, is usually of a rounded, flattened form, but often irregular in shape. It is of a bright red or yellow color, and is eaten either cooked or uncooked.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Tomato gall</b></col> <fld>(Zool.)</fld>, <cd>a large gall consisting of a mass of irregular swellings on the stems and leaves of grapevines.  They are yellowish green, somewhat tinged with red, and produced by the larva of a small two-winged fly (<spn>Lasioptera vitis</spn>).</cd> -- <col><b>Tomato sphinx</b></col> <fld>(Zool.)</fld>, <cd>the adult or imago of the <cref>tomato worm</cref>.  It closely resembles the tobacco hawk moth.  Called also <altname>tomato hawk moth</altname>. See <xex>Illust.</xex> of <er>Hawk moth</er>.</cd> -- <col><b>Tomato worm</b></col> <fld>(Zool.)</fld>, <cd>the larva of a large hawk moth (<spn>Manduca quinquemaculata</spn>, <spn>Protoparce quinquemaculata</spn>, <spn>Sphinx quinquemaculata</spn>, or <spn>Macrosila quinquemaculata</spn>) which feeds upon the leaves of the tomato and potato plants, often doing considerable damage.  Called also <altname>tomato hornworm</altname> and <altname>potato worm</altname>, and in the Southern U. S. <altname>tobacco fly</altname>.</cd></cs><br/
[<source>1913 Webster</source> <source>+PJC</source>]</p>

<p><ent>Tomb</ent><br/
<hw>Tomb</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[OE. <ets>tombe</ets>, <ets>toumbe</ets>, F. <ets>tombe</ets>, LL. <ets>tumba</ets>, fr. Gr. <?/ a tomb, grave; perhaps akin to L. <ets>tumulus</ets> a mound.  Cf. <er>Tumulus</er>.]</ety> <sn>1.</sn> <def>A pit in which the dead body of a human being is deposited; a grave; a sepulcher.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>As one dead in the bottom of a <qex>tomb</qex>.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A house or vault, formed wholly or partly in the earth, with walls and a roof, for the reception of the dead.</def> <ldquo/In <xex>tomb</xex> of marble stones.<rdquo/  <rj><au>Chaucer.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>A monument erected to inclose the body and preserve the name and memory of the dead.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Hang her an epitaph upon her <qex>tomb</qex>.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Tomb bat</b></col> <fld>(Zool.)</fld>, <cd>any one of species of Old World bats of the genus <gen>Taphozous</gen> which inhabit tombs, especially the Egyptian species (<spn>Taphozous perforatus</spn>).</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tomb</ent><br/
<hw>Tomb</hw>,, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Tombed</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Tombing</conjf>.]</vmorph> <def>To place in a tomb; to bury; to inter; to entomb.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>I <qex>tombed</qex> my brother that I might be blessed.</q> <rj><qau>Chapman.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tombac</ent><br/
<hw>Tom"bac</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Pg. <ets>tambaca</ets>,<ets>tambaque</ets>, fr. Malay <ets>tamb<amac/ga</ets> copper; cf. Skr. <ets>t<amac/mraka</ets>; cf. F. <ets>tombac</ets>.]</ety> <fld>(Metal.)</fld> <def>An alloy of copper and zinc, resembling brass, and containing about 84 per cent of copper; -- called also <altname>German brass</altname> <it>or</it> <altname>Dutch brass</altname>.  It is very malleable and ductile, and when beaten into thin leaves is sometimes called <altname>Dutch metal</altname>.  The addition of arsenic makes <stype>white tombac</stype>.</def> <altsp>[Written also <asp>tombak</asp>, and <asp>tambac</asp>.]</altsp><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tombester</ent><br/
<hw>Tom"bes*ter</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[See <er>Tumble</er>, and <er>-ster</er>.]</ety> <def>A female dancer.</def> <mark>[Obs.]</mark>  <rj><au>Chaucer.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tombless</ent><br/
<hw>Tomb"less</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Destitute of a tomb.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tomboy</ent><br/
<hw>Tom"boy`</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[<ets>Tom</ets> (for Thomas, L. <ets>Thomas</ets>, fr. Gr. <?/ )+ <ets>boy</ets>.]</ety> <def>A romping girl; a hoiden.</def> <mark>[Colloq.]</mark>  <rj><au>J. Fletcher.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tombstone</ent><br/
<hw>Tomb"stone`</hw> <pr>(t<oomac/m"st<omac/n`)</pr>, <pos>n.</pos> <def>A stone or small stone monument erected over a grave, bearing the name of the dead person interred there, to preserve the memory of the deceased.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tomcat</ent><br/
<hw>Tom"cat`</hw> <pr>(t<ocr/m"k<acr/t`)</pr>, <pos>n.</pos> <ety>[<ets>Tom</ets> (see <er>Tomboy</er>) + <ets>cat</ets>.]</ety> <def>A male cat, especially when full grown or of large size.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tomcod</ent><br/
<hw>Tom"cod`</hw> <pr>(t<ocr/m"k<ocr/d`)</pr>, <pos>n.</pos> <ety>[<ets>Tom</ets> (see <er>Tomboy</er>) + <ets>cod</ets>: cf. F. <ets>tacaud</ets> whiting pout, American Indian <ets>tacaud</ets>, literally, plenty fish.]</ety> <fld>(Zool.)</fld> <sd>(a)</sd> <def>A small edible American fish (<spn>Microgadus tomcod</spn>) of the Codfish family, very abundant in autumn on the Atlantic coast of the Northen United States; -- called also <altname>frostfish</altname>. See <xex>Illust.</xex> under <er>Frostfish</er>.</def> <sd>(b)</sd> <def>The kingfish. See <er>Kingfish</er> <sd>(a)</sd>.</def> <sd>(c)</sd> <def>The jack. See 2d <er>Jack</er>, 8. <sd>(c)</sd>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tome</ent><br/
<hw>Tome</hw> <pr>(t<omac/m)</pr>, <pos>n.</pos> <ety>[F. <ets>tome</ets> (cf. It., Sp., & Pg. <ets>tomo</ets>), L. <ets>tomus</ets>, fr. Gr. <?/ a piece cut off, a part of a book, a volume, akin to <?/ to cup, and perhaps to L. <ets>tondere</ets> to shear, E. <ets>tonsure</ets>.  Cf. <er>Anatomy</er>, <er>Atom</er>, <er>Entomology</er>, <er>Epitome</er>. ]</ety> <def>As many writings as are bound in a volume, forming part of a larger work; a book; -- usually applied to a ponderous volume.</def><br/
[<source>1913 Webster</source>]</p>

<p><q><qex>Tomes</qex> of fable and of dream.</q> <rj><qau>Cowper.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>A more childish expedient than that to which he now resorted is not to be found in all the <qex>tomes</qex> of the casuists.</q> <rj><qau>Macaulay.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tomelet</ent><br/
<hw>Tome"let</hw> <pr>(?)</pr>, <pos>n.</pos> <def>All small tome, or volume.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tomentose</ent><br/
<hw>To"men*tose`</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>tomentum</ets> a stuffing of wool, hair, or feathers: cf. F. <ets>tomenteux</ets>.]</ety> <fld>(Bot. & Zool.)</fld> <def>Covered with matted woolly hairs; <as>as, a <ex>tomentose</ex> leaf; a <ex>tomentose</ex> leaf; a <ex>tomentose</ex> membrane</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tomentous</ent><br/
<hw>To*men"tous</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Tomentose.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tomentum</ent><br/
||<hw>To*men"tum</hw> <pr>(?)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Tomenta</plw> <pr>(#)</pr>.</plu> <ety>[L. See <er>Tomentose</er>. ]</ety> <fld>(Bot.)</fld> <def>The closely matted hair or downy nap covering the leaves or stems of some plants.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tomfool</ent><br/
<hw>Tom"fool`</hw> <pr>(t<ocr/m"f<oomac/l`)</pr>, <pos>n.</pos> <ety>[<ets>Tom</ets> (see <er>Tomboy</er>) + <ets>fool</ets>.]</ety> <def>A great fool; a trifler.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tomfoolery</ent><br/
<hw>Tom`fool"er*y</hw> <pr>(?)</pr>, <pos>n.</pos> <def>Trifling or silly behavior done mostly for amusement; foolishness; fooling around.</def><br/
<syn><b>Syn. --</b> tomfoolishness.</syn><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tomfoolishness</ent><br/
<hw>Tom`fool"ish*ness</hw> <pr>(?)</pr>, <pos>n.</pos> <def>Same as <er>tomfoolery</er>.</def><br/
[<source>PJC</source>]</p>

<p><ent>Tomium</ent><br/
||<hw>To"mi*um</hw> <pr>(t<omac/"m<icr/*<ucr/m)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Tomia</plw> <pr>(t<omac/"m<icr/*<adot/)</pr></plu> <ety>[NL., fr. Gr. <grk>te`mnein</grk> to cut.]</ety> <fld>(Zool.)</fld> <def>The cutting edge of the bill of a bird.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tomjohn</ent><br/
<hw>Tom"john`</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Probably of East Indian origin.]</ety> <def>A kind of open sedan used in Ceylon, carried by a single pole on men's shoulders.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tommy</ent><br/
<hw>Tom"my</hw> <pr>(?)</pr>, <pos>n.</pos> <sn>1.</sn> <def>Bread, -- generally a penny roll; the supply of food carried by workmen as their daily allowance.</def> <mark>[Slang,Eng.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A truck, or barter; the exchange of labor for goods, not money.</def> <mark>[Slang, Eng.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ <xex>Tommy</xex> is used adjectively or in compounds; as, <xex>tommy</xex> master, <xex>tommy</xex>-store,<xex>tommy</xex>-shop,etc.</note><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>Same as <er>Tommy Atkins</er>; -- a shortened form.</def> <mark>[Colloq.]</mark> <br/
[<source>PJC</source>]</p>

<p><ent>Tommy Atkins</ent><br/
<hw>Tom"my At"kins</hw> <pr>(?)</pr>, <pos>prop. n.</pos> <def>Any white regular soldier of the British army; also, such soldiers collectively; -- said to be fictitious name inserted in the models given to soldiers to guide them in filling out account blanks, etc.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Tomnoddy</ent><br/
<hw>Tom"nod`dy</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[<ets>Tom</ets> (see <er>Tomboy</er>) + <ets>noddy</ets>.]</ety> <sn>1.</sn> <fld>(Zool.)</fld> <def>A sea bird, the puffin.</def> <mark>[Prov.Eng.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A fool; a dunce; a noddy.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tom o' Bedlam</ent><br/
<hw>Tom o' Bed"lam</hw> <pr>(?)</pr>. <def>Formerly, a wandering mendicant discharged as incurable from Bethlehem Hospital, England; hence, a wandering mendicant, either mad or feigning to be so; a madman; a bedlamite.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Tomopteris</ent><br/
||<hw>To*mop"te*ris</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[NL., fr. Gr. <grk>to`mos</grk> a cut + <grk>ptero`n</grk> wing (but taken to mean, fin).]</ety> <fld>(Zool.)</fld> <def>A genus of transparent marine annelids which swim actively at the surface of the sea. They have deeply divided or forked finlike organs (parapodia). This genus is the type of the order, or suborder, Gymnocopa.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tomorn</ent><br/
<hw>To*morn"</hw> <pr>(?)</pr>, <pos>adv.</pos> <ety>[Prep. <ets>to + morn</ets>.]</ety> <def>To-morrow.</def> <mark>[Obs.]</mark>  <rj><au>Chaucer.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tomorrow</ent><br/
<hw>To*mor"row</hw> <pr>(?)</pr>, <pos>adv.</pos> <ety>[Prep. <ets>to + morrow</ets>.]</ety> <def>On the day after the present day; on the next day; on the morrow.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Summon him <qex>to-morrow</qex> to the Tower.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tomorrow</ent><br/
<hw>To*mor"row</hw> <pr>(?)</pr>, <pos>n.</pos> <def>The day after the present; the morrow.</def><ldquo/<xex>To-morrow</xex> is our wedding day.<rdquo/  <rj><au> Cowper.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>One today is worth two <qex>to-morrows</qex>.</q> <rj><qau>Franklin.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tompion</ent><br/
<hw>Tom"pi*on</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[See <er>Tampios</er>]</ety> <sn>1.</sn> <def>A stopper of a cannon or a musket. See <er>Tampion</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Mus.)</fld> <def>A plug in a flute or an organ pipe, to modulate the tone.</def>  <rj><au>Knight.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>The iron bottom to which grapeshot are fixed.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tompon</ent><br/
<hw>Tom"pon</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F. <ets>tampon</ets>. See <er>Tampion</er>.]</ety> <def>An inking pad used in lithographic printing.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tomrig</ent><br/
<hw>Tom"rig`</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[<ets>Tom</ets> (see <er>Tomboy</er>) + <ets>rig</ets>.)]</ety> <def>A rude, wild, wanton girl; a hoiden; a tomboy.</def> <au>Dennis.</au><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tomtate</ent><br/
<hw>Tom"tate</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A Florida and West Indian grunt (<spn>Bathystoma rimator</spn> <it>or</it>  <spn>Haemulon rimator</spn>); also, any of various allied species.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Tomtit</ent><br/
<hw>Tom"tit`</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[<ets>Tom</ets> (see <er>Tomboy</er>) + <ets>tit</ets> the bird.]</ety> <fld>(Zool.)</fld> <sd>(a)</sd> <def>A titmouse, especially the blue titmouse.</def> <mark>[Prov. Eng.]</mark> <sd>(b)</sd> <def>The wren.</def> <mark>[Prov. Eng.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tom-tom</ent><br/
<hw>Tom"-tom`</hw> <pr>(t<ocr/m"t<ocr/m`)</pr>, <pos>n.</pos> <def>See <er>Tam-tam</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Ton</ent><br/
<hw>Ton</hw> <pr>(t<ocr/n)</pr>, <mark>obs.</mark> <def><pos>pl.</pos> of <er>Toe</er>.</def>  <rj><au>Chaucer.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Ton</ent><br/
<hw>Ton</hw> <pr>(t<ucr/n)</pr>, <pos>n.</pos> <ety>[Cf. <er>Tunny</er>.]</ety> <fld>(Zool.)</fld> <def>The common tunny, or horse mackerel.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Ton</ent><br/
||<hw>Ton</hw> <pr>(t<ocir/n)</pr>, <pos>n.</pos> <ety>[F. See <er>Tone</er>.]</ety> <def>The prevailing fashion or mode; vogue; <as>as, things of <ex>ton</ex></as>.</def>  <rj><au>Byron.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>If our people of <qex>ton</qex> are selfish, at any rate they show they are selfish.</q> <rj><qau>Thackeray.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Bon ton</b></col>. <cd>See in the Vocabulary.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Ton</ent><br/
<hw>Ton</hw> <pr>(t<ucr/n)</pr>, <pos>n.</pos> <ety>[OE. <ets>tonne</ets>, <ets>tunne</ets>, a tun, <er>AS</er>. <ets>tunne</ets> a tun, tub, a large vessel; akin to G. & F. <ets>tonne</ets> a ton, tun, LL. <ets>tunna</ets> a tun; all perhaps of Celtic origin; cf. Ir. & Gael. <ets>tunna</ets> a tun.  Cf. <er>Tun</er>,<er>Tunnel</er>.]</ety> <fld>(Com.)</fld> <def>A measure of weight or quantity.</def> Specifically: --<br/
[<source>1913 Webster</source>]</p>

<p><sd>(a)</sd> <def>The weight of twenty hundredweight.</def><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ In England, the <xex>ton</xex> is 2,240 pounds. In the United States the ton is commonly estimated at 2,000 pounds, this being sometimes called the <xex>short ton</xex>, while that of 2,240 pounds is called the <xex>long ton</xex>.</note><br/
[<source>1913 Webster</source>]</p>

<p><sd>(b)</sd> <fld>(Naut. & Com.)</fld> <def>Forty cubic feet of space, being the unit of measurement of the burden, or carrying capacity, of a vessel; as a vessel of 300 <xex>tons</xex> burden.</def> See the Note under <er>Tonnage</er>.<br/
[<source>1913 Webster</source>]</p>

<p><sd>(c)</sd> <fld>(Naut. & Com.)</fld> <def>A certain weight or quantity of merchandise, with reference to transportation as freight; as, six hundred weight of ship bread in casks, seven hundred weight in bags, eight hundred weight in bulk; ten bushels of potatoes; eight sacks, or ten barrels, of flour; forty cubic feet of rough, or fifty cubic feet of hewn, timber, etc.</def><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ <xex>Ton</xex> and <xex>tun</xex> have the same etymology, and were formerly used interchangeably; but now <xex>ton</xex> generally designates the weight, and <xex>tun</xex> the cask. See <er>Tun</er>.</note><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tonality</ent><br/
<hw>To*nal"i*ty</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Cf. F. <ets>tonalit<eacute/</ets>.]</ety> <fld>(Mus.)</fld> <def>The principle of key in music; the character which a composition has by virtue of the key in which it is written, or through the family relationship of all its tones and chords to the keynote, or tonic, of the whole.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The predominance of the tonic as the link which connects all the tones of a piece, we may, with F<eacute/tis, term the principle of <qex>tonality</qex>.</q> <rj><qau>Helmholtz.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>To-name</ent><br/
<hw>To"-name`</hw> <pr>(t<oomac/"n<amac/m`)</pr>, <pos>n.</pos> <ety>[prep. <ets>to</ets> + <ets>name</ets>.]</ety> <def>A name added, for the sake of distinction, to one's surname, or used instead of it.</def> <mark>[Scot.]</mark>  <rj><au>Jamieson.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tonca bean</ent><br/
<hw>Ton"ca bean`</hw> <pr>(?)</pr>. <fld>(Bot.)</fld> <def>See <er>Tonka bean</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tone</ent><br/
<hw>Tone</hw> <pr>(t<omac/n)</pr>, <pos>n.</pos> <ety>[F. <ets>ton</ets>, L. <ets>tonus</ets> a sound, tone, fr. Gr. <grk>to`nos</grk> a stretching, straining, raising of the voice, pitch, accent, measure or meter, in pl., modes or keys differing in pitch; akin to <grk>tei`nein</grk> to stretch or strain. See <er>Thin</er>, and cf. <er>Monotonous</er>, <er>Thunder</er>, <er>Ton</er> fashion, <er>Tune</er>.]</ety> <sn>1.</sn> <def>Sound, or the character of a sound, or a sound considered as of this or that character; <as>as, a low, high, loud, grave, acute, sweet, or harsh <ex>tone</ex></as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>[Harmony divine] smooths her charming <qex>tones</qex>.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q><qex>Tones</qex> that with seraph hymns might blend.</q> <rj><qau>Keble.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Rhet.)</fld> <def>Accent, or inflection or modulation of the voice, as adapted to express emotion or passion.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Eager his <qex>tone</qex>, and ardent were his eyes.</q> <rj><qau>Dryden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>A whining style of speaking; a kind of mournful or artificial strain of voice; an affected speaking with a measured rhythm ahd a regular rise and fall of the voice; <as>as, children often read with a <ex>tone</ex></as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <fld>(Mus.)</fld> <sd>(a)</sd> <def>A sound considered as to pitch; <as>as, the seven <ex>tones</ex> of the octave; she has good high <ex>tones</ex>.</as></def> <sd>(b)</sd> <def>The larger kind of interval between contiguous sounds in the diatonic scale, the smaller being called a <xex>semitone</xex> as, a whole <xex>tone</xex> too flat; raise it a <xex>tone</xex>.</def> <sd>(c)</sd> <def>The peculiar quality of sound in any voice or instrument; <as>as, a rich <ex>tone</ex>, <ex>a reedy tone</ex></as>.</def> <sd>(d)</sd> <def>A mode or tune or plain chant; <as>as, the Gregorian <ex>tones</ex></as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ The use of the word <xex>tone</xex>, both for a sound and for the interval between two sounds or tones, is confusing, but is common -- almost universal.</note><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ Nearly every musical sound is composite, consisting of several simultaneous <xex>tones</xex> having different rates of vibration according to fixed laws, which depend upon the nature of the vibrating body and the mode of excitation. The components (of a composite sound) are called <xex>partial tones</xex>; that one having the lowest rate of vibration is the <xex>fundamental tone</xex>, and the other <xex>partial tones</xex> are called <xex>harmonics</xex>, or <xex>overtones</xex>.  The vibration ratios of the <xex>partial tones</xex> composing any sound are expressed by all, or by a part, of the numbers in the series 1, 2, 3, 4, 5, etc.; and the quality of any sound (the <xex>tone color</xex>) is due in part to the presence or absence of <xex>overtones</xex> as represented in this series, and in part to the greater or less intensity of those present as compared with the <xex>fundamental tone</xex> and with one another. <xex>Resultant tones</xex>, <xex>combination tones</xex>, <xex>summation tones</xex>, <xex>difference tones</xex>, <xex>Tartini's tones</xex> (terms only in part synonymous) are produced by the simultaneous sounding of two or more primary (simple or composite) tones.</note><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <fld>(Med.)</fld> <def>That state of a body, or of any of its organs or parts, in which the animal functions are healthy and performed with due vigor.</def><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ In this sense, the word is metaphorically applied to character or faculties, intellectual and moral; as, his mind has lost its <xex>tone</xex>.</note><br/
[<source>1913 Webster</source>]</p>

<p><sn>6.</sn> <fld>(Physiol.)</fld> <def>Tonicity; <as>as, arterial <ex>tone</ex></as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>7.</sn> <def>State of mind; temper; mood.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The strange situation I am in and the melancholy state of public affairs, . . . drag the mind down . . . from a philosophical <qex>tone</qex> or temper, to the drudgery of private and public business.</q> <rj><qau>Bolingbroke.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Their <qex>tone</qex> was dissatisfied, almost menacing.</q> <rj><qau>W. C. Bryant.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>8.</sn> <def>Tenor; character; spirit; drift; <as>as, the <ex>tone</ex> of his remarks was commendatory</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>9.</sn> <def>General or prevailing character or style, as of morals, manners, or sentiment, in reference to a scale of high and low; <as>as, a low <ex>tone</ex> of morals; a <ex>tone</ex> of elevated sentiment; a courtly <ex>tone</ex> of manners</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>10.</sn> <def>The general effect of a picture produced by the combination of light and shade, together with color in the case of a painting; -- commonly used in a favorable sense; <as>as, this picture has <ex>tone</ex></as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>11.</sn> <fld>(Physiol.)</fld> <def>Quality, with respect to attendant feeling; the more or less variable complex of emotion accompanying and characterizing a sensation or a conceptual state; <as>as, feeling <ex>tone</ex>; color <ex>tone</ex></as>.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><sn>12.</sn>  <def>Color quality proper; -- called also <altname>hue</altname>. Also, a gradation of color, either a hue, or a tint or shade.</def></p>

<p><q>She was dressed in a soft cloth of a gray <qex>tone</qex>.</q>  <rj><qau>Sir G. Parker.</qau></rj><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><sn>13.</sn>  <fld>(Plant Physiol.)</fld> <def>The condition of normal balance of a healthy plant in its relations to light, heat, and moisture.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><cs><col><b>Tone color</b></col>. <fld>(Mus.)</fld> <cd>see the Note under def. 4, above.</cd> -- <col><b>Tone syllable</b></col>, <cd>an accented syllable.</cd>  <au>M. Stuart.</au></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tone</ent><br/
<hw>Tone</hw> <pr>(?)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Toned</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Toning</conjf>.]</vmorph> <sn>1.</sn> <def>To utter with an affected tone.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To give tone, or a particular tone, to; to tune. See <er>Tune</er>, <pos>v. t.</pos></def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <fld>(Photog.)</fld> <def>To bring, as a print, to a certain required shade of color, as by chemical treatment.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>To tone down</b></col>. <sd>(a)</sd> <cd>To cause to give lower tone or sound; to give a lower tone to.</cd> <sd>(b)</sd> <fld>(Paint.)</fld> <cd>To modify, as color, by making it less brilliant or less crude; to modify, as a composition of color, by making it more harmonius.</cd><br/
[<source>1913 Webster</source>]</p>

<p><q>Its thousand hues <qex>toned down</qex> harmoniusly.</q> <rj><qau>C. Kingsley.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sd>(c)</sd> Fig.: To moderate or relax; to diminish or weaken the striking characteristics of; to soften.<br/
[<source>1913 Webster</source>]</p>

<p><q>The best method for the purpose in hand was to employ some one of a character and position suited to get possession of their confidence, and then use it <qex>to tone down</qex> their religious strictures.</q> <rj><qau>Palfrey.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p> -- <col><b>To tone up</b></col>, <cd>to cause to give a higher tone or sound; to give a higher tone to; to make more intense; to heighten; to strengthen.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Toned</ent><br/
<hw>Toned</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Having (such) a tone; -- chiefly used in composition; <as>as, high-<ex>toned</ex>; sweet-<ex>toned</ex></as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Toned paper</b></col>, <cd>paper having a slight tint, in distinction from paper which is quite white.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Toneless</ent><br/
<hw>Tone"less</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Having no tone; unmusical.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tonge</ent><br/
<ent>Tong</ent><br/
<mhw>{ <hw>Tong</hw> <pr>(?)</pr>, <hw>Tonge</hw>}</mhw>, <pos>n.</pos> <def>Tongue.</def> <mark>[Obs.]</mark> <au>Chaucer.</au><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tong</ent><br/
<hw>Tong</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Chinese <ets>t'ang</ets>, lit., hall.]</ety> <def>In China, an association, secret society, or organization of any kind; in the United States, usually, a secret association of Chinese such as that of the highbinders; in the U. S. the <ex>tongs</ex> have been frequently associated with criminal activity and gang warfare.</def><br/
[<source>Webster 1913 Suppl.</source> <source>+PJC</source>]</p>

<p><ent>Tonga</ent><br/
<hw>Ton"ga</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Med.)</fld> <def>A drug useful in neuralgia, derived from a Fijian plant supposed to be of the aroid genus <gen>Epipremnum</gen>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tonga</ent><br/
<hw>Ton"ga</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Hind. <ets>t<amac/ng<amac/</ets>, Skr. <ets>tama<ndot/gaka</ets>.]</ety> <def>A kind of light two-wheeled vehicle, usually for four persons, drawn by ponies or bullocks.</def> <mark>[India]</mark><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Tongkang</ent><br/
<hw>Tong"kang</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Naut.)</fld> <def>A kind of boat or junk used in the seas of the Malay Archipelago.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tongo</ent><br/
<hw>Ton"go</hw> <pr>(?)</pr>, <pos>n.</pos> <def>The mangrove; -- so called in the Pacific Islands.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tongs</ent><br/
<hw>Tongs</hw> <pr>(?)</pr>, <pos>n. pl.</pos> <ety>[OE. <ets>tonge</ets>, <ets>tange</ets>, AS. <ets>tange</ets>; akin to D. <ets>tang</ets>, G. <ets>zanga</ets>, OHG. <ets>zanga</ets>, Don. <ets>tang</ets>, Sw. <ets>t<aring/ng</ets>, Icel. <ets>t<oum/ng</ets>, Gr. <grk>da`knein</grk> to bite, Skr. <ets>da<ntil/i<cced/</ets>, <ets>da<cced/</ets>. <root/59.  Cf. <er>Tang</er> a strong taste, anything projecting.]</ety> <def>An instrument, usually of metal, consisting of two parts, or long shafts, jointed together at or near one end, or united by an elastic bow, used for handling things, especially hot coals or metals; -- often called a <altname>pair of tongs</altname>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tongue</ent><br/
<hw>Tongue</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[OE. <ets>tunge</ets>, <ets>tonge</ets>, AS. <ets>tunge</ets>; akin to OFries. <ets>tunge</ets>, D. <ets>tong</ets>, OS. <ets>tunga</ets>, G. <ets>zunge</ets>, OHG. <ets>zunga</ets>, Icel. & Sw. <ets>tunga</ets>, Dan <ets>tunge</ets>, Goth. <ets>tugg<omac/</ets>, OL. <ets>dingua</ets>, L. <ets>lingua</ets>. <root/243  Cf.<er>Language</er>, <er>Lingo</er>. ]</ety><br/
[<source>1913 Webster</source>]</p>

<p><sn>1.</sn> <fld>(Anat.)</fld> <def>an organ situated in the floor of the mouth of most vertebrates and connected with the hyoid arch.</def><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ The tongue is usually muscular, mobile, and free at one extremity, and in man other mammals is the principal organ of taste, aids in the prehension of food, in swallowing, and in modifying the voice as in speech.</note><br/
[<source>1913 Webster</source>]</p>

<p><q>To make his English sweet upon his <qex>tongue</qex>.</q> <rj><qau>Chaucer.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>The power of articulate utterance; speech.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Parrots imitating human <qex>tongue</qex>.</q> <rj><qau>Dryden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>Discourse; fluency of speech or expression.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Much <qex>tongue</qex> and much judgment seldom go together.</q> <rj><qau>L. Estrange.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>Honorable discourse; eulogy.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>She was born noble; let that title find her a private grave, but neither <qex>tongue</qex> nor honor.</q> <rj><qau>Beau. & Fl.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <def>A language; the whole sum of words used by a particular nation; <as>as, the English <ex>tongue</ex></as>.</def>  <rj><au>Chaucer.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Whose <qex>tongue</qex> thou shalt not understand.</q> <rj><qau>Deut. xxviii. 49.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>To speak all <qex>tongues</qex>.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>6.</sn> <def>Speech; words or declarations only; -- opposed to <xex>thoughts</xex> or <xex>actions</xex>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>My little children, let us love in word, neither in <qex>tongue</qex>, but in deed and in truth.</q> <rj><qau>1 John iii. 18.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>7.</sn> <def>A people having a distinct language.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>A will gather all nations and <qex>tongues</qex>.</q> <rj><qau>Isa. lxvi. 18.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>8.</sn> <fld>(Zool.)</fld> <sd>(a)</sd> <def>The lingual ribbon, or odontophore, of a mollusk.</def> <sd>(b)</sd> <def>The proboscis of a moth or a butterfly.</def> <sd>(c)</sd> <def>The lingua of an insect.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>9.</sn> <fld>(Zool.)</fld> <def>Any small sole.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>10.</sn> <def>That which is considered as resembing an animal's tongue, in position or form.</def> Specifically: --<br/
[<source>1913 Webster</source>]</p>

<p><sd>(a)</sd> <def>A projection, or slender appendage or fixture; as, the <xex>tongue</xex> of a buckle, or of a balance.</def><br/
[<source>1913 Webster</source>]</p>

<p><-- p. 1517 --><br/
[<source>1913 Webster</source>]</p>

<p><sd>(b)</sd> <def>A projection on the side, as of a board, which fits into a groove.</def><br/
[<source>1913 Webster</source>]</p>

<p><sd>(c)</sd> <def>A point, or long, narrow strip of land, projecting from the mainland into a sea or a lake.</def><br/
[<source>1913 Webster</source>]</p>

<p><sd>(d)</sd> <def>The pole of a vehicle; especially, the pole of an ox cart, to the end of which the oxen are yoked.</def><br/
[<source>1913 Webster</source>]</p>

<p><sd>(e)</sd> <def>The clapper of a bell.</def><br/
[<source>1913 Webster</source>]</p>

<p><sd>(f)</sd> <fld>(Naut.)</fld> <def>A short piece of rope spliced into the upper part of standing backstays, etc.; also. the upper main piece of a mast composed of several pieces.</def><br/
[<source>1913 Webster</source>]</p>

<p><sd>(g)</sd> <fld>(Mus.)</fld> <def>Same as <er>Reed</er>, <pos>n.</pos>, 5.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>To hold the tongue</b></col>, <cd>to be silent.</cd> -- <col><b>Tongue bone</b></col> <fld>(Anat.)</fld>, <cd>the hyoid bone.</cd> -- <col><b>Tongue grafting</b></col>. <cd>See under <er>Grafting</er>.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><syn><b>Syn.</b> -- Language; speech; expression. See <er>Language</er>.</syn><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tongue</ent><br/
<hw>Tongue</hw> <pr>(?)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Tongued</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Tonguing</conjf>.]</vmorph> <sn>1.</sn> <def>To speak; to utter.</def> <ldquo/Such stuff as madmen <xex>tongue</xex>.<rdquo/ <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To chide; to scold.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>How might she <qex>tongue</qex> me.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <fld>(Mus.)</fld> <def>To modulate or modify with the tongue, as notes, in playing the flute and some other wind instruments.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>To join means of a tongue and grove; <as>as, to <ex>tongue</ex> boards together</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tongue</ent><br/
<hw>Tongue</hw>, <pos>v. i.</pos> <sn>1.</sn> <def>To talk; to prate.</def>  <rj><au>Dryden.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Mus.)</fld> <def>To use the tongue in forming the notes, as in playing the flute and some other wind instruments.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tonguebird</ent><br/
<hw>Tongue"bird`</hw> <pr>(?)</pr>, <pos>n.</pos> <def>The wryneck.</def><mark>[Prov. Eng.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tongued</ent><br/
<hw>Tongued</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Having a tongue.</def><br/
[<source>1913 Webster</source>]</p>

<p><q><qex>Tongued</qex> like the night crow.</q> <rj><qau>Donne.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tonguefish</ent><br/
<hw>Tongue"fish`</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Zool.)</fld> <def>A flounder (<spn>Symphurus plagiusa</spn>) native of the southern coast of the United States.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tongueless</ent><br/
<hw>Tongue"less</hw> <pr>(?)</pr>, <pos>a.</pos> <sn>1.</sn> <def>Having no tongue.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Hence, speechless; mute.</def> <ldquo/What <xex>tongueless</xex> blocks were they! would they not speak?<rdquo/  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>Unnamed; not spoken of.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>One good deed dying <qex>tongueless</qex>.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tonguelet</ent><br/
<hw>Tongue"let</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A little tongue.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tongue-pad</ent><br/
<hw>Tongue"-pad`</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A great talker.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tongue-shaped</ent><br/
<hw>Tongue"-shaped`</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Shaped like a tongue; specifically <fld>(Bot.)</fld>, linear or oblong, and fleshy, blunt at the end, and convex beneath; <as>as, a <ex>tongue-shaped</ex> leaf</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tongue-shell</ent><br/
<hw>Tongue"-shell`</hw> <pr>(?)</pr>, <pos>n.</pos> <def>Any species of Lingula.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tonguester</ent><br/
<hw>Tongue"ster</hw> <pr>(?)</pr>, <pos>n.</pos> <def>One who uses his tongue; a talker; a story-teller; a gossip.</def> <mark>[Poetic.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>Step by step we rose to greatness; through the <qex>tonguesters</qex> we may fall.</q> <rj><qau>Tennyson.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tongue-tie</ent><br/
<hw>Tongue"-tie`</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Med.)</fld> <def>Impeded motion of the tongue because of the shortness of the fraenum, or of the adhesion of its margins to the gums.</def>  <rj><au>Dunglison.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tongue-tie</ent><br/
<hw>Tongue"-tie`</hw>, <pos>v. t.</pos> <def>To deprive of speech or the power of speech, or of distinct articulation.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tongue-tied</ent><br/
<hw>Tongue"-tied`</hw> <pr>(?)</pr>, <pos>a.</pos> <sn>1.</sn> <def>Destitute of the power of distinct articulation; having an impediment in the speech, esp. when caused by a short fraenum.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Unable to speak freely, from whatever cause.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Love, therefore, and <qex>tongue-tied</qex> simplicity.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tongueworm</ent><br/
<hw>Tongue"worm`</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Zool.)</fld> <def>Any species of Linguatulina.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tonguing</ent><br/
<hw>Tongu"ing</hw> <pr>(?)</pr>, <pos>vb. n.</pos> <fld>(Music)</fld> <def>Modification of tone for a rapid staccato effect by the performer's tongue, in playing a wind instrument, as a flute. In <stype>single tonguing</stype> only one kind of stroke is used, the tongue articulating a rapid <ldquo/t;<rdquo/ in <stype>double tonguing</stype>, two strokes, as for <ldquo/t<rdquo/ and <ldquo/k,<rdquo/ are alternated; in <stype>triple tonguing</stype>, <ldquo/t, k, t,<rdquo/ etc.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Tonguy</ent><br/
<hw>Tongu"y</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Ready or voluble in speaking; <as>as, a <ex>tonguy</ex> speaker</as>.</def> <altsp>[Written also <asp>tonguey</asp>.]</altsp> <mark>[Colloq.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tonic</ent><br/
<hw>Ton"ic</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[Cf. F. <ets>tonigue</ets>, Gr. <?/. See <er>Tone</er>.]</ety> <sn>1.</sn> <def>Of or relating to tones or sounds; specifically <fld>(Phon.)</fld>, applied to, or distingshing, a speech sound made with tone unmixed and undimmed by obstruction, such sounds, namely, the vowels and diphthongs, being so called by Dr. James Rush (1833) <ldquo/ from their forming the purest and most plastic material of intonation.<rdquo/</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Of or pertaining to tension; increasing tension; hence, increasing strength; <as>as, <ex>tonic</ex> power</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <fld>(Med.)</fld> <def>Increasing strength, or the tone of the animal system; obviating the effects of debility, and restoring healthy functions.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <fld>(Med.)</fld> <def>Characterized by continuous muscular contraction; <as>as, <ex>tonic</ex> convulsions</as>.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><cs><col><b>Tonic spasm</b></col>. <fld>(Med.)</fld> <cd>See the Note under <er>Spasm</er>.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tonic</ent><br/
<hw>Ton"ic</hw>, <pos>n.</pos> <ety>[Cf. F. <ets>tonique</ets>, NL. <ets>tonicum</ets>.]</ety> <sn>1.</sn> <fld>(Phon.)</fld> <def>A tonic element or letter; a vowel or a diphthong.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Mus.)</fld> <def>The key tone, or first tone of any scale.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <fld>(Med.)</fld> <def>A medicine that increases the strength, and gives vigor of action to the system.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Tonic sol-fa</b></col> <fld>(Mus.)</fld>, <cd>the name of the most popular among letter systems of notation (at least in England), based on key relationship, and hence called <ldquo/tonic.<rdquo/  Instead of the five lines, clefs, signature, etc., of the usual notation, it employs letters and the syllables <xex>do</xex>, <xex>re</xex>, <xex>mi</xex>, etc., variously modified, with other simple signs of duration, of upper or lower octave, etc. See <er>Sol-fa</er>.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tonical</ent><br/
<hw>Ton"ic*al</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Tonic.</def> <mark>[R.]</mark> <au>Sir T. Browne.</au><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tonicity</ent><br/
<hw>To*nic"i*ty</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Physiol.)</fld> <def>The state of healthy tension or partial contraction of muscle fibers while at rest; tone; tonus.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tonight</ent><br/
<hw>To*night"</hw> <pr>(?)</pr>, <pos>adv.</pos> <ety>[Prep. <ets>to</ets> + <ets>night</ets>]</ety> <sn>1.</sn> <def>On this present or coming night.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>On the last night past.</def> <mark>[Obs.]</mark>  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tonight</ent><br/
<hw>To*night"</hw>, <pos>n.</pos> <def>The present or the coming night; the night after the present day.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tonite</ent><br/
<hw>Ton"ite</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Cf.L. <ets>tonare</ets> to thunder.]</ety> <def>An explosive compound; a preparation of gun cotton.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tonka bean</ent><br/
<hw>Ton"ka bean`</hw> <pr>(?)</pr>. <ety>[Cf. F. <ets>onca</ets>, <ets>tonka</ets>.]</ety> <fld>(Bot.)</fld> <def>The seed of a leguminous tree (<spn>Dipteryx odorata</spn>), native of Guiana. It has a peculiarly agreeable smell, and is employed in the scenting of snuff. Called also <altname>tonquin bean</altname>.</def> <altsp>[Written also <asp>tonca bean</asp>, <asp>tonga bean</asp>.]</altsp><br/
[<source>1913 Webster</source>]</p>

<p><ent>Ton mile</ent><br/
<hw>Ton mile</hw>. <fld>(Railroads)</fld> <def>A unit of measurement of the freight transportation performed by a railroad during a given period, usually a year, the total of which consists of the sum of the products obtained by multiplying the aggregate weight of each shipment in tons during the given period by the number of miles for which it is carried.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Ton mileage</ent><br/
<hw>Ton mileage</hw>. <fld>(Railroads)</fld> <def>Ton miles collectively; esp., the total ton miles performed by a railroad in a given period.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Tonnage</ent><br/
<hw>Ton"nage</hw> <pr>(?; 48)</pr>, <pos>n.</pos> <ety>[From <er>Ton</er> a measure.]</ety><br/
[<source>1913 Webster</source>]</p>

<p><sn>1.</sn> <def>The weight of goods carried in a boat or a ship.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>The cubical content or burden of a vessel, or vessels, in tons; or, the amount of weight which one or several vessels may carry. See <er>Ton</er>, <pos>n.</pos> <sd>(b)</sd>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>A fleet . . . with an aggregate <qex>tonnage</qex> of 60,000 seemed sufficient to conquer the world.</q> <rj><qau>Motley.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>A duty or impost on vessels, estimated per ton, or, a duty, toll, or rate payable on goods per ton transported on canals.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>The whole amount of shipping estimated by tons; <as>as, the <ex>tonnage</ex> of the United States</as>. See <er>Ton</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ There are in common use the following terms relating to tonnage: (<xex>a</xex>) Displacement. (<xex>b</xex>) Register tonnage, gross and net. (<xex>c</xex>) Freight tonnage. (<xex>d</xex>) Builders' measurement. (<xex>e</xex>) Yacht measurement. The first is mainly used for war vessels, where the total weight is likely to be nearly constant. The second is the most important, being that used for commercial purposes. The third and fourth are different rules for ascertaining the actual burden-carrying power of a vessel, and the fifth is for the proper classification of pleasure craft. <xex>Gross tonnage</xex> expresses the total cubical interior of a vessel; <xex>net tonnage</xex>, the cubical space actually available for freight-carrying purposes. Rules for ascertaining these measurements are established by law.</note><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tonne</ent><br/
<hw>Tonne</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A tun.</def> <mark>[Obs.]</mark>  <rj><au>Chaucer.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tonne</ent><br/
||<hw>Tonne</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F.]</ety> <def>A metric ton.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Tonneau</ent><br/
||<hw>Ton`neau"</hw> <pr>(?)</pr>, <pos>n.</pos>; <plu>pl. <plw>Tonneaux</plw> <pr>(#)</pr></plu>. <ety>[F.]</ety> <sn>1.</sn> <def>In France, a light-wheeled vehicle with square or rounded body and rear entrance.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><sn>2.</sn>  <fld>(Automobiles)</fld> <def>Orig., the after part of the body with  entrance at the rear (as in vehicle in def. 1); now, one with sides closing in the seat or seats and entered by a door usually at the side, also, the entire body of an automobile having such an after part.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><sn>3.</sn>  <def>same as <er>Tonne</er>.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Tonnihood</ent><br/
<hw>Ton"ni*hood</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Zool.)</fld> <def>The female of the bullfinch; -- called also <altname>tonyhoop</altname>.</def> <mark>[Prov. Eng.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tonnish</ent><br/
<hw>Ton"nish</hw> <pr>(t<ocr/n"n<icr/sh)</pr>, <pos>a.</pos> <def>In the ton; fashionable; modish.</def> -- <wordforms><wf>Ton"nish*ness</wf>, <pos>n.</pos></wordforms><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tonometer</ent><br/
<hw>To*nom"e*ter</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Gr. <?/ a stretching, a tone + <ets>-meter</ets>.]</ety> <sn>1.</sn> <fld>(Physics.)</fld> <def>An instrument for determining the rate of vibrations in tones.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Physiol.)</fld> <sd>(a)</sd> <def>An apparatus for studying and registering the action of various fluids and drugs on the excised heart of lower animals.</def> <sd>(b)</sd> <def>An instrument for measuring tension, esp. that of the eyeball.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tonometry</ent><br/
<hw>To*nom"e*try</hw> <pr>(?)</pr>, <pos>n.</pos> <def>The act of measuring with a tonometer;</def> <specif>specifically</specif> <fld>(Med.)</fld>, <def>measurement of tension, esp. the tension of the eyeball.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tonophant</ent><br/
<hw>Ton"o*phant</hw> <pr>(t<omac/n"<osl/*f<ait/nt)</pr>, <pos>n.</pos> <ety>[Gr. <grk>to`nos</grk> a tone + <grk>fai`nein</grk> to show.]</ety> <fld>(Physics.)</fld> <def>A modification of the kaleidophon, for showing composition of acoustic vibrations. It consists of two thin slips of steel welded together, their length being adjustable by a screw socket.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tonous</ent><br/
<hw>Ton"ous</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Abounding in tone or sound.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tonquin bean</ent><br/
<hw>Ton"quin bean`</hw> <pr>(t<ocr/<nsm/"k<icr/n b<emac/n)</pr>. <def>See <er>Tonka bean</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tonsil</ent><br/
<hw>Ton"sil</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>tonsill</ets><?/, pl.: cf. F. <ets>tonsille</ets>. ]</ety> <fld>(Anat.)</fld> <def>One of the two glandular organs situated in the throat at the sides of the fauces. The tonsils are sometimes called the <altname>almonds</altname>, from their shape.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tonsilar</ent><br/
<hw>Ton"sil*ar</hw> <pr>(?)</pr>, <pos>a.</pos> <fld>(Anat.)</fld> <def>Of or pertaining to the tonsils; tonsilitic.</def> <altsp>[Written also <asp>tonsillar</asp>.]</altsp><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tonsile</ent><br/
<hw>Ton"sile</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>tonsilis</ets>, fr. <ets>tondere</ets>, <ets>tonsum</ets>, to shear, clip. See <er>Tonsure</er>. ]</ety> <def>Capable of being clipped.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tonsilitic</ent><br/
<hw>Ton`sil*it"ic</hw> <pr>(?)</pr>, <pos>a.</pos> <fld>(Anat.)</fld> <def>Tonsilar.</def> <altsp>[Written also <asp>tonsillitic</asp>.]</altsp><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tonsilitis</ent><br/
||<hw>Ton`sil*i"tis</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[NL. See <er>Tonsil</er>, and <er>-itis</er>.]</ety> <fld>(Med.)</fld> <def>Inflammation of the tonsils; quinsy.</def> <altsp>[Written also, and more usually, <asp>tonsillitis</asp>.]</altsp><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tonsillectomy</ent><br/
<hw>Ton`sil*lec"to*my</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Surg.)</fld> <def>The operation of removing the tonsils; -- formerly also called <altname>tonsilotomy</altname>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tonsillitis</ent><br/
<hw>Ton`sil*li"tis</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Med.)</fld> <def>Inflammation of the tonsils; --- same as <er>tonsilitis</er>.</def><br/
[<source>PJC</source>]</p>

<p><ent>Tonsilotome</ent><br/
<hw>Ton*sil"o*tome</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[<ets>Tonsil</ets> + Gr. <grk>te`mnein</grk> to cut.]</ety> <fld>(Surg.)</fld> <def>An instrument for removing the tonsils.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tonsilotomy</ent><br/
<hw>Ton`sil*ot"o*my</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Surg.)</fld> <def>The operation of removing the tonsils, or a portion thereof; -- called more commonly <altname>tonsillectomy</altname>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tonsor</ent><br/
<hw>Ton"sor</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L.]</ety> <def>A barber.</def> <au>Sir W. Scott.</au><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tonsorial</ent><br/
<hw>Ton*so"ri*al</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>tonsorius</ets>, fr. <ets>tonsor</ets> a shearer, barber, fr. <ets>tondere</ets>, <ets>tonsum</ets>, to shear. See <er>Tonsure</er>.]</ety> <def>Of or pertaining to a barber, or shaving.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tonsure</ent><br/
<hw>Ton"sure</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F., fr. L. <ets>tonsura</ets> a shearing, clipping, from <ets>tondere</ets>, <ets>tonsum</ets>, to shear, shave; cf. Gr. <?/ to gnaw; perhaps akin to Gr. <?/ to cut, and E. <ets>tome</ets>.]</ety> <sn>1.</sn> <def>The act of clipping the hair, or of shaving the crown of the head; also, the state of being shorn.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(R. C. Ch.)</fld> <sd>(a)</sd> <def>The first ceremony used for devoting a person to the service of God and the church; the first degree of the clericate, given by a bishop, abbot, or cardinal priest, consisting in cutting off the hair from a circular space at the back of the head, with prayers and benedictions; hence, entrance or admission into minor orders.</def> <sd>(b)</sd> <def>The shaven corona, or crown, which priests wear as a mark of their order and of their rank.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tonsured</ent><br/
<hw>Ton"sured</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Having the tonsure; shaven; shorn; clipped; hence, bald.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>A <qex>tonsured</qex> head in middle age forlorn.</q> <rj><qau>Tennyson.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tontine</ent><br/
<hw>Ton*tine"</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F., from It. <ets>tontina</ets>; -- so called from its inventor, <etsep>Tonti</etsep>, an Italian, of the 17th century.]</ety> <def>An annuity, with the benefit of survivorship, or a loan raised on life annuities with the benefit of survivorship. Thus, an annuity is shared among a number, on the principle that the share of each, at his death, is enjoyed by the survivors, until at last the whole goes to the last survivor, or to the last two or three, according to the terms on which the money is advanced. Used also adjectively; <as>as, <ex>tontine</ex> insurance</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q> Too many of the financiers by professions are apt to see nothing in revenue but banks, and circulations, and annuities on lives, and <qex>tontines</qex>, and perpetual rents, and all the small wares of the shop.</q> <rj><qau>Burke.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tontine insurance</ent><br/
<hw>Ton*tine" in*su"rance</hw>. <fld>(Life Insurance)</fld> <def>Insurance in which the benefits of the insurance are distributed upon the tontine principle. Under the old, or <col><b>full tontine</b></col>, plan, all benefits were forfeited on lapsed policies, on the policies of those who died within the tontine period only the face of the policy was paid without any share of the surplus, and the survivor at the end of the tontine period received the entire surplus. This plan of tontine insurance has been replaced in the United States by the <col><b>semitontine</b></col> plan, in which the surplus is divided among the holders of policies in force at the termination of the tontine period, but the reverse for the paid-up value is paid on lapsed policies, and on the policies of those that have died the face is paid. Other modified forms are called <stype>free tontine</stype>, <stype>deferred dividend</stype>, etc., according to the nature of the tontine arrangement.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Tonus</ent><br/
||<hw>To"nus</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. a sound, tone. See <er>Tone</er>.]</ety> <fld>(Physiol.)</fld> <def>Tonicity, or tone; <as>as, muscular <ex>tonus</ex></as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tony</ent><br/
<hw>To"ny</hw> <pr>(?)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Tonies</plw> <pr>(#)</pr>.</plu> <ety>[Abbrev. from <ets>Anthony</ets>.]</ety> <def>A simpleton.</def>  <rj><au>L'Estrange.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>A pattern and companion fit<br/
For all the keeping <qex>tonies</qex> of the pit.</q> <rj><qau>Dryden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Too</ent><br/
<hw>Too</hw> <pr>(?)</pr>, <pos>adv.</pos> <ety>[The same word as <ets>to</ets>, prep. See <er>To</er>.]</ety><br/
[<source>1913 Webster</source>]</p>

<p><sn>1.</sn> <def>Over; more than enough; -- noting excess; <as>as, a thing is <ex>too</ex> long, <ex>too</ex> short, or <ex>too</ex> wide; <ex>too</ex> high; <ex>too</ex> many; <ex>too</ex> much</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>His will, <qex>too</qex> strong to bend, <qex>too</qex> proud to learn.</q> <rj><qau>Cowley.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Likewise; also; in addition.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>An honest courtier, yet a patriot <qex>too</qex>.</q> <rj><qau>Pope.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Let those eyes that view<br/
The daring crime, behold the vengeance <qex>too</qex>.</q> <rj><qau>Pope.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Too too</b></col>, <cd>a duplication used to signify great excess.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><q>O that this <qex>too too</qex> solid flesh would melt.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Such is not Charles his <qex>too too</qex> active age.</q> <rj><qau>Dryden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><syn><b>Syn.</b> -- Also; likewise. See <er>Also</er>.</syn><br/
[<source>1913 Webster</source>]</p>

<p><ent>Took</ent><br/
<hw>Took</hw> <pr>(t<oocr/k)</pr>, <def><pos>imp.</pos> of <er>Take</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tool</ent><br/
<hw>Tool</hw> <pr>(t<oomac/l)</pr>, <pos>n.</pos> <ety>[OE. <ets>tol</ets>,<ets>tool</ets>. AS. <ets>t<omac/l</ets>; akin to Icel. <ets>t<omac/l</ets>, Goth. <ets>taijan</ets> to do, to make, <ets>taui</ets> deed, work, and perhaps to E. <ets>taw</ets> to dress leather. <root/64.]</ety> <sn>1.</sn> <def>An instrument such as a hammer, saw, plane, file, and the like, used in the manual arts, to facilitate mechanical operations; any instrument used by a craftsman or laborer at his work; an implement; <as>as, the <ex>tools</ex> of a joiner, smith, shoe-maker, etc.</as>; also, a cutter, chisel, or other part of an instrument or machine that dresses work.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A machine for cutting or shaping materials; -- also called <altname>machine tool</altname>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>Hence, any instrument of use or service.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>That angry fool . . . <br/
Whipping her horse, did with his smarting <qex>tool</qex><br/
Oft whip her dainty self.</q> <rj><qau>Spenser.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>A weapon.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>Him that is aghast of every <qex>tool</qex>.</q> <rj><qau>Chaucer.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <def>A person used as an instrument by another person; -- a word of reproach; <as>as, men of intrigue have their <ex>tools</ex>, by whose agency they accomplish their purposes</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>I was not made for a minion or a <qex>tool</qex>.</q> <rj><qau>Burks.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tool</ent><br/
<hw>Tool</hw> <pr>(?)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>tooled</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>tooling</conjf>.]</vmorph> <sn>1.</sn> <def>To shape, form, or finish with a tool.</def> <ldquo/Elaborately <xex>tooled</xex>.<rdquo/  <rj><au>Ld. Lytton.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To drive, as a coach.</def> <mark>[Slang, Eng.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tool</ent><br/
<hw>Tool</hw> <pr>(t<oomac/l)</pr>, <pos>v. i.</pos> <ety>[Cf. <er>Tool</er>, <pos>v. t.</pos>, 2.]</ety> <def>To travel in a vehicle; to ride or drive.</def> <mark>[Colloq.]</mark></p>

<p><q>Boys on their bicycles <qex>tooling</qex> along the well-kept roads.</q>  <rj><qau>Illust. American.</qau></rj><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Tooling</ent><br/
<hw>Tool"ing</hw>, <pos>n.</pos> <def>Work performed with a tool.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The fine <qex>tooling</qex> and delicate tracery of the cabinet artist is lost upon a building of colossal proportions.</q> <rj><qau>De Quincey.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tool-stock</ent><br/
<ent>Tool-post</ent><br/
<mhw>{ <hw>Tool"-post`</hw> <pr>(?)</pr>, <hw>Tool"-stock`</hw> <pr>(?)</pr>, }</mhw> <pos>n.</pos> <fld>(Mach.)</fld> <def>The part of a tool-rest in which a cutting tool is clamped.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tool-rest</ent><br/
<hw>Tool"-rest`</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Mach.)</fld> <def>the part that supports a tool-post or a tool.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Toolshed</ent><br/
<hw>Tool"shed</hw>. <def>a small closed structure in which tools are stored, often found in the back yard of a residence.</def><br/
[<source>{JC</source>]</p>

<p><ent>Tool steel</ent><br/
<hw>Tool steel</hw>. <def>Hard steel, usually crucible steel, capable of being tempered so as to be suitable for tools.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Toom</ent><br/
<hw>Toom</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[OE. <ets>tom</ets>, fr. Icel. <ets>t<omac/mr</ets>; akin to Dan. & Sw. <ets>tom</ets>, As. <ets>t<omac/me</ets>, adv.  Cf. <er>Teem</er> to pour.]</ety> <def>Empty.</def> <mark>[Obs. or Prov. Eng. & Scot.]</mark>  <rj><au>Wyclif.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Toom</ent><br/
<hw>Toom</hw>, <pos>v. t.</pos> <def>To empty.</def> <mark>[Obs. or Prov.Eng. & Scot.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Toon</ent><br/
<hw>Toon</hw> <pr>(?)</pr>, <mark>obs.</mark> <def><pos>pl.</pos> of <er>Toe</er>.</def>  <rj><au>Chaucer.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Toon</ent><br/
<hw>Toon</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Hind. <ets>tun</ets>, <ets>t<umac/n</ets>, Skr. <ets>tunna</ets>.]</ety> <fld>(Bot.)</fld> <def>The reddish brown wood of an East Indian tree (<spn>Cedrela Toona</spn>) closely resembling the Spanish cedar; also. the tree itself.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Toonwood</ent><br/
<hw>Toon"wood`</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Bot.)</fld> <def>Same as <er>Toon</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Toot</ent><br/
<hw>Toot</hw> <pr>(?)</pr>, <pos>v. i.</pos> <ety>[OE. <ets>toten</ets>, AS. <ets>totian</ets> to project; hence, to peep out.]</ety> <altsp>[Written also <asp>tout</asp>.]</altsp> <sn>1.</sn> <def>To stand out, or be prominent.</def> <mark>[Obs.]</mark>  <rj><au>Howell.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To peep; to look narrowly.</def> <mark>[Obs.]</mark>  <rj><au>Latimer.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>For birds in bushes <qex>tooting</qex>.</q> <rj><qau>Spenser.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Toot</ent><br/
<hw>Toot</hw>, <pos>v. t.</pos> <def>To see; to spy.</def> <mark>[Obs.]</mark>  <rj><au>P. Plowman.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Toot</ent><br/
<hw>Toot</hw>, <pos>v. i.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Tooted</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Tooting</conjf>.]</vmorph> <ety>[Cf. D. <ets>toeten</ets> to blow a horn, G. <ets>tuten</ets>, Sw. <ets>tuta</ets>, Dan. <ets>tude</ets>; probably of imitative origin.]</ety> <def>To blow or sound a horn; to make similar noise by contact of the tongue with the root of the upper teeth at the beginning and end of the sound; also, to give forth such a sound, as a horn when blown.</def> <ldquo/A <xex>tooting</xex> horn.<rdquo/  <rj><au>Howell.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q><qex>Tooting</qex> horns and rattling teams of mail coaches.</q> <rj><qau>Thackeray.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Toot</ent><br/
<hw>Toot</hw>, <pos>v. t.</pos> <def>To cause to sound, as a horn, the note being modified at the beginning and end as if by pronouncing the letter <it>t</it>; to blow; to sound.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tooter</ent><br/
<hw>Toot"er</hw> <pr>(?)</pr>, <pos>n.</pos> <def>One who toots; one who plays upon a pipe or horn.</def>  <rj><au>B. Jonson.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tooth</ent><br/
<hw>Tooth</hw> <pr>(t<oomac/th)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Teeth</plw> <pr>(t<emac/th)</pr>.</plu> <ety>[OE. <ets>toth</ets>,<ets>tooth</ets>, AS. <ets>t<omac/<edh/</ets>; akin to OFries. <ets>t<omac/th</ets>, OS. & D. <ets>tand</ets>, OHG. <ets>zang</ets>, <ets>zan</ets>, G. <ets>zahn</ets>, Icel. <ets>t<oum/nn</ets>, Sw. & Dan. <ets>tand</ets>, Goth. <ets>tumpus</ets>, Lith. <ets>dantis</ets>, W. <ets>dant</ets>, L. <ets>dens</ets>, <ets>dentis</ets>, Gr. <grk>'odoy`s</grk>, <grk>'odo`ntos</grk>, Skr. <ets>danta</ets>; probably originally the p. pr. of the verb to <ets>eat</ets>. <root/239.  Cf. <er>Eat</er>, <er>Dandelion</er>, <er>Dent</er> the tooth of a wheel, <er>Dental</er>, <er>Dentist</er>, <er>Indent</er>, <er>Tine</er> of a fork, <er>Tusk</er>. ]</ety> <sn>1.</sn> <fld>(Anat.)</fld> <def>One of the hard, bony appendages which are borne on the jaws, or on other bones in the walls of the mouth or pharynx of most vertebrates, and which usually aid in the prehension and mastication of food.</def><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ The hard parts of teeth are principally made up of <xex>dentine</xex>, or ivory, and a very hard substance called <xex>enamel</xex>. These are variously combined in different animals.  Each tooth consist of three parts, a <xex>crown</xex>, or body, projecting above the gum, one or more <xex>fangs</xex> imbedded in the jaw, and the <xex>neck</xex>, or intermediate part. In some animals one or more of the teeth are modified into tusks which project from the mouth, as in both sexes of the elephant and of the walrus, and in the male narwhal.<br/
   In adult man there are thirty-two teeth, composed largely of dentine, but the crowns are covered with enamel, and the fangs with a layer of bone called <xex>cementum</xex>. Of the eight teeth on each half of each jaw, the two in front are <xex>incisors</xex>, then come one <xex>canine</xex>, cuspid, or dog tooth, two <xex>bicuspids</xex>, or false molars, and three <xex>molars</xex>, or grinding teeth. The milk, or temporary, teeth are only twenty in number, there being two incisors, one canine, and two molars on each half of each jaw. The last molars, or wisdom teeth, usually appear long after the others, and occasionally do not appear above the jaw at all.</note><br/
[<source>1913 Webster</source>]</p>

<p><q>How sharper than a serpent's <qex>tooth</qex> it is<br/
To have a thankless child!</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Fig.: Taste; palate.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>These are not dishes for thy dainty <qex>tooth</qex>.</q> <rj><qau>Dryden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>Any projection corresponding to the tooth of an animal, in shape, position, or office; <as>as, the <ex>teeth</ex>, or cogs, of a cogwheel; a <ex>tooth</ex>, prong, or tine, of a fork; a <ex>tooth</ex>, or the <ex>teeth</ex>, of a rake, a saw, a file, a card</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <sd>(a)</sd> <def>A projecting member resembling a tenon, but fitting into a mortise that is only sunk, not pierced through.</def> <sd>(b)</sd> <def>One of several steps, or offsets, in a <xex>tusk</xex>. See <er>Tusk</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><-- p. 1518 --><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <fld>(Nat. Hist.)</fld> <def>An angular or prominence on any edge; <as>as, a <ex>tooth</ex> on the scale of a fish, or on a leaf of a plant</as></def>; specifically <fld>(Bot.)</fld>, <def>one of the appendages at the mouth of the capsule of a moss. See <er>Peristome</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>6.</sn> <fld>(Zool.)</fld> <def>Any hard calcareous or chitinous organ found in the mouth of various invertebrates and used in feeding or procuring food; <as>as, the <ex>teeth</ex> of a mollusk or a starfish</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>In spite of the teeth</b></col>, <cd>in defiance of opposition; in opposition to every effort.</cd> -- <col><b>In the teeth</b></col>, <cd>directly; in direct opposition; in front.</cd> <ldquo/Nor strive with all the tempest in my <xex>teeth</xex>.<rdquo/ <au>Pope.</au> -- <col><b>To cast in the teeth</b></col>, <cd>to report reproachfully; to taunt or insult one with.</cd> -- <col><b>Tooth and nail</b></col>, <cd>as if by biting and scratching; with one's utmost power; by all possible means.</cd>  <au>L'Estrange.</au>  <ldquo/I shall fight <xex>tooth and nail</xex> for international copyright.<rdquo/ <au>Charles Reade.</au> -- <col><b>Tooth coralline</b></col> <fld>(Zool.)</fld>, <cd>any sertularian hydroid.</cd> -- <col><b>Tooth edge</b></col>, <cd>the sensation excited in the teeth by grating sounds, and by the touch of certain substances, as keen acids.</cd> -- <col><b>Tooth key</b></col>, <cd>an instrument used to extract teeth by a motion resembling that of turning a key.</cd> -- <col><b>Tooth net</b></col>, <cd>a large fishing net anchored.</cd> <mark>[Scot.]</mark> <au>Jamieson.</au> -- <col><b>Tooth ornament</b></col>. <fld>(Arch.)</fld> <cd>Same as <er>Dogtooth</er>, <pos>n.</pos>, 2.</cd><-- <col><b>Tooth paste</b></col>, <cd>a paste for cleaning the teeth; a dentifrice.</cd> --> -- <col><b>Tooth powder</b></col>, <cd>a powder for cleaning the teeth; a dentifrice.</cd> -- <col><b>Tooth rash</b></col>. <fld>(Med.)</fld> <cd>See <er>Red-gum</er>, 1.</cd> -- <col><b>To show the teeth</b></col>, <cd>to threaten.</cd> <ldquo/When the Law <xex>shows</xex> her <xex>teeth</xex>, but dares not bite.<rdquo/  <au>Young.</au> -- <col><b>To the teeth</b></col>, <cd>in open opposition; directly to one's face.</cd> <ldquo/That I shall live, and tell him <xex>to</xex> his <xex>teeth</xex> .<rdquo/ <au>Shak.</au></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tooth</ent><br/
<hw>Tooth</hw> <pr>(?)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Toothed</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Toothing</conjf>.]</vmorph> <sn>1.</sn> <def>To furnish with teeth.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The twin cards <qex>toothed</qex> with glittering wire.</q> <rj><qau>Wordsworth.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To indent; to jag; <as>as, to <ex>tooth</ex> a saw</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>To lock into each other. See <er>Tooth</er>, <pos>n.</pos>, 4.</def>  <rj><au>Moxon.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Toothache</ent><br/
<hw>Tooth"ache`</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Med.)</fld> <def>Pain in a tooth or in the teeth; odontalgia.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Toothache grass</b></col> <fld>(Bot.)</fld>, <cd>a kind of grass (<spn>Ctenium Americanum</spn>) having a very pungent taste.</cd> -- <col><b>Toothache tree</b></col>. <fld>(Bot.)</fld> <sd>(a)</sd> <cd>The prickly ash.</cd> <sd>(b)</sd> <cd>A shrub of the genus <gen>Aralia</gen> (<spn>Aralia spinosa</spn>).</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Toothback</ent><br/
<hw>Tooth"back`</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Zool.)</fld> <def>Any notodontian.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Toothbill</ent><br/
<hw>Tooth"bill`</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Zool.)</fld> <def>A peculiar fruit-eating ground pigeon (<spn>Didunculus strigiostris</spn>) native of the Samoan Islands, and noted for its resemblance, in several characteristics, to the extinct dodo. Its beak is stout and strongly hooked, and the mandible has two or three strong teeth toward the end.  Its color is chocolate red. Called also <altname>toothbilled pigeon</altname>, and <altname>manu-mea</altname>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Toothbrush</ent><br/
<hw>Tooth"brush`</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A brush for cleaning the teeth.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Toothdrawer</ent><br/
<hw>Tooth"draw`er</hw> <pr>(?)</pr>, <pos>n.</pos> <def>One whose business it is to extract teeth with instruments; a dentist.</def>  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Toothed</ent><br/
<hw>Toothed</hw> <pr>(?)</pr>, <pos>a.</pos> <sn>1.</sn> <def>Having teeth; furnished with teeth.</def> <ldquo/Ruby-lipped and <xex>toothed</xex> with pearl.<rdquo/  <rj><au>Herrick.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Bot. & Zool.)</fld> <def>Having marginal projecting points; dentate.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Toothed whale</b></col> <fld>(Zool.)</fld>, <cd>any whale of the order Denticete. See <er>Denticete</er>.</cd> -- <col><b>Toothed wheel</b></col>, <cd>a wheel with teeth or projections cut or set on its edge or circumference, for transmitting motion by their action on the engaging teeth of another wheel.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Toothful</ent><br/
<hw>Tooth"ful</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Toothsome.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Toothing</ent><br/
<hw>Tooth"ing</hw>, <pos>n.</pos> <sn>1.</sn> <def>The act or process of indenting or furnishing with teeth.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Masonry)</fld> <def>Bricks alternately projecting at the end of a wall, in order that they may be bonded into a continuation of it when the remainder is carried up.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Toothing plane</b></col>, <cd>a plane of which the iron is formed into a series of small teeth, for the purpose of roughening surfaces, as of veneers.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Toothless</ent><br/
<hw>Tooth"less</hw>, <pos>a.</pos> <def>Having no teeth.</def>  <rj><au>Cowper.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Toothlet</ent><br/
<hw>Tooth"let</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A little tooth, or like projection.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Toothleted</ent><br/
<hw>Tooth"let*ed</hw>, <pos>a.</pos> <def>Having a toothlet or toothlets; <as>as, a <ex>toothleted</ex> leaf</as>.</def> <altsp>[Written also <asp>toothletted</asp>.]</altsp><br/
[<source>1913 Webster</source>]</p>

<p><ent>Toothpick</ent><br/
<hw>Tooth"pick`</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A pointed instrument for clearing the teeth of substances lodged between them.</def><-- esp., a slim sliver of wood, about two inches in length, tapering to a point at both ends, and used for removing food particles from between the teeth after a meal. --><br/
[<source>1913 Webster</source>]</p>

<p><ent>Toothpicker</ent><br/
<hw>Tooth"pick`er</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A toothpick.</def> <mark>[Obs.]</mark>  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Toothshell</ent><br/
<hw>Tooth"shell"</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Zool.)</fld> <def>Any species of <gen>Dentalium</gen> and allied genera having a tooth-shaped shell. See <er>Dentalium</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Toothsome</ent><br/
<hw>Tooth"some</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Grateful to the taste; palatable.</def> -- <wordforms><wf>Tooth"some*ly</wf>, <pos>adv.</pos> -- <wf>Tooth"some*ness</wf>, <pos>n.</pos></wordforms><br/
[<source>1913 Webster</source>]</p>

<p><q>Though less <qex>toothsome</qex> to me, they were more wholesome for me.</q> <rj><qau>Fuller.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Toothwort</ent><br/
<hw>Tooth"wort`</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Bot.)</fld> <def>A plant whose roots are fancied to resemble teeth, as certain plants of the genus <gen>Lathraea</gen>, and various species of <gen>Dentaria</gen>. See <er>Coralwort</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Toothy</ent><br/
<hw>Tooth"y</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Toothed; with teeth.</def> <mark>[R]</mark> <au>Croxall.</au><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tootle</ent><br/
<hw>Too"tle</hw> <pr>(?)</pr>, <pos>v. i.</pos>  <vmorph>[<pos>imp. & p. p.</pos> <conjf>Tootled</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Tootling</conjf> <pr>(?)</pr>.]</vmorph> <ety>[Freq. of <ets>toot</ets>.]</ety> <def>To toot gently, repeatedly, or continuously, on a wind instrument, as a flute; also, to make a similar noise by any means.</def> <ldquo/The <xex>tootling</xex> robin.<rdquo/  <rj><au>John Clare.</au></rj><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Toozoo</ent><br/
<hw>Too*zoo"</hw> <pr>(?)</pr>, <pos>n.</pos> <def>The ringdove.</def> <mark>[Prov. Eng.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Top</ent><br/
<hw>Top</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[CF. OD. <ets>dop</ets>, <ets>top</ets>, OHG., MNG., & dial. G. <ets>topf</ets>; perhaps akin to G. <ets>topf</ets> a pot.]</ety> <sn>1.</sn> <def>A child's toy, commonly in the form of a conoid or pear, made to spin on its point, usually by drawing off a string wound round its surface or stem, the motion being sometimes continued by means of a whip.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Rope Making)</fld> <def>A plug, or conical block of wood, with longitudital grooves on its surface, in which the strands of the rope slide in the process of twisting.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Top</ent><br/
<hw>Top</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[AS. <ets>top</ets>; akin to OFries. <ets>top</ets> a tuft, D. <ets>top</ets> top, OHG. <ets>zopf</ets> end, tip, tuft of hair, G. <ets>zopf</ets> tuft of hair, pigtail, top of a tree, Icel. <ets>toppr</ets> a tuft of hair, crest, top, Dan. <ets>top</ets>, Sw. <ets>topp</ets> pinnacle, top; of uncertain origin.  Cf. <er>Tuft</er>.]</ety> <sn>1.</sn> <def>The highest part of anything; the upper end, edge, or extremity; the upper side or surface; summit; apex; vertex; cover; lid; <as>as, the <ex>top</ex> of a spire; the <ex>top</ex> of a house; the <ex>top</ex> of a mountain; the <ex>top</ex> of the ground.</as></def><br/
[<source>1913 Webster</source>]</p>

<p><q>The star that bids the shepherd fold,<br/
Now the <qex>top</qex> of heaven doth hold.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>The utmost degree; the acme; the summit.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The <qex>top</qex> of my ambition is to contribute to that work.</q> <rj><qau>Pope.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>The highest rank; the most honorable position; the utmost attainable place; <as>as, to be at the <ex>top</ex> of one's class, or at the <ex>top</ex> of the school</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>And wears upon his baby brow the round<br/
And <qex>top</qex> of sovereignty.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>The chief person; the most prominent one.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Other . . . aspired to be the <qex>top</qex> of zealots.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <def>The crown of the head, or the hair upon it; the head.</def> <ldquo/From <xex>top</xex> to toe<rdquo/  <rj><au>Spenser.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>All the stored vengeance of Heaven fall<br/
On her ungrateful <qex>top</qex> !</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>6.</sn> <def>The head, or upper part, of a plant.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The buds . . . are called heads, or <qex>tops</qex>, as cabbageheads.</q> <rj><qau>I. Watts.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>7.</sn> <fld>(Naut.)</fld> <def>A platform surrounding the head of the lower mast and projecting on all sudes. It serves to spead the topmast rigging, thus strengheningthe mast, and also furnishes a convenient standing place for the men aloft.</def>  <rj><au>Totten.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>8.</sn> <fld>(Wool Manuf.)</fld> <def>A bundle or ball of slivers of comkbed wool, from which the noils, or dust, have been taken out.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>9.</sn> <def>Eve; verge; point.</def> <mark>[R.]</mark> <ldquo/He was upon the <xex>top</xex> of his marriage with Magdaleine.<rdquo/  <rj><au>Knolles.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>10.</sn> <def>The part of a cut gem between the girdle, or circumference, and the table, or flat upper surface.</def>  <rj><au>Knight.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>11.</sn> <pluf>pl.</pluf> <def>Top-boots.</def> <mark>[Slang]</mark>  <rj><au>Dickens.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>12.</sn> <fld>(Golf)</fld> <sd>(a)</sd> <def>A stroke on the top of the ball.</def> <sd>(b)</sd> <def>A forward spin given to the ball by hitting it on or near the top.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><note><hand/ <xex>Top</xex> is often used adjectively or as the first part of compound words, usually self-explaining; as, <xex>top</xex> stone, or <xex>top</xex>stone; <xex>top</xex>-boots, or <xex>top</xex> boots; <xex>top</xex> soil, or <xex>top</xex>-soil.</note><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Top and but</b></col> <fld>(Shipbuilding)</fld>, <cd>a phrase used to denote a method of working long tapering planks by bringing the but of one plank to the top of the other to make up a constant breadth in two layers.</cd> -- <col><b>Top minnow</b></col> <fld>(Zool.)</fld>, <cd>a small viviparous fresh-water fish (<spn>Gambusia patruelis</spn>) abundant in the Southern United States. Also applied to other similar species.</cd> --  <col><b>From top to toe</b></col>, <cd>from head to foot; altogether.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Top</ent><br/
<hw>Top</hw>, <pos>v. i.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Topped</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Topping</conjf>.]</vmorph> <sn>1.</sn> <def>To rise aloft; to be eminent; to tower; <as>as, lofty ridges and <ex>topping</ex> mountains</as>.</def>  <rj><au>Derham.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To predominate; <as>as, <ex>topping</ex> passions</as>.</def> <ldquo/Influenced by <xex>topping</xex> uneasiness.<rdquo/  <rj><au>Locke.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>To excel; to rise above others.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>But write thy, and <qex>top</qex>.</q> <rj><qau>Dryden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <fld>(Golf)</fld> <def>To strike a ball above the center.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><sn>5.</sn>  <fld>(Naut.)</fld> <def>To rise at one end, as a yard; -- usually with <xex>up</xex>.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Top</ent><br/
<hw>Top</hw>, <pos>v. t.</pos> <sn>1.</sn> <def>To cover on the top; to tip; to cap; -- chiefly used in the past participle.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Like moving mountains <qex>topped</qex> with snow.</q> <rj><qau>Waller.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>A mount<br/
Of alabaster, <qex>topped</qex> with golden spires.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To rise above; to excel; to outgo; to surpass.</def><br/
[<source>1913 Webster</source>]</p>

<p><q><qex>Topping</qex> all others in boasting.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Edmund the base shall <qex>top</qex> the legitimate.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>To rise to the top of; to go over the top of.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>But wind about till thou hast <qex>topped</qex> the hill.</q> <rj><qau>Denham.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>To take off the or upper part of; to crop.</def><br/
[<source>1913 Webster</source>]</p>

<p><q><qex>Top</qex> your rose trees a little with your knife.</q> <rj><qau>Evelyn.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <def>To perform eminently, or better than before.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>From endeavoring universally to <qex>top</qex> their parts, they will go universally beyond them.</q> <rj><qau>Jeffrey.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>6.</sn> <fld>(Naut.)</fld> <def>To raise one end of, as a yard, so that that end becomes higher than the other.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>7.</sn> <fld>(Dyeing)</fld> <def>To cover with another dye; <as>as, to <ex>top</ex> aniline black with methyl violet to prevent greening and crocking</as>.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><sn>8.</sn> <def>To put a stiffening piece or back on (a saw blade).</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><sn>9.</sn>  <def>To arrange, as fruit, with the best on top.</def> <mark>[Cant]</mark><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><sn>10.</sn>  <def>To strike the top of, as a wall, with the hind feet, in jumping, so as to gain new impetus; -- said of a horse.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><sn>11.</sn>  <def>To improve (domestic animals, esp. sheep) by crossing certain individuals or breeds with other superior.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><sn>12.</sn>  <fld>(Naut.)</fld> <def>To raise one end of, as a yard, so that that end becomes higher than the other.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><sn>13.</sn>  <def>To cut, break, or otherwise take off the top of (a steel ingot) to remove unsound metal.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><sn>14.</sn>  <fld>(Golf)</fld> <def>To strike (the ball) above the center; also, to make (as a stroke) by hitting the ball in this way.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><cs><col><b>To top off</b></col>, <sd>(a)</sd> <cd>to complete by putting on, or finishing, the top or uppermost part of; as, <xex>to top off</xex> a stack of hay; hence, to complete; to finish; to adorn.</cd> <sd>(b)</sd> <cd>to completely fill (an almost full tank) by adding more of the liquid it already contains.</cd></cs><br/
[<source>1913 Webster</source> <source>+PJC</source>]</p>

<p><ent>Toparch</ent><br/
<hw>To"parch</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>toparcha</ets>, Gr. <?/; <?/ a place + <?/ to govern, to rule.]</ety> <def>The ruler or principal man in a place or country; the governor of a toparchy.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The prince and <qex>toparch</qex> of that country.</q> <rj><qau>Fuller.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Toparchy</ent><br/
<hw>To"parch*y</hw> <pr>(?)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Toparchies</plw> <pr>(#)</pr>.</plu> <ety>[L. <ets>toparchia</ets>, Gr. <?/. See <er>Toparch</er>.]</ety> <def>A small state, consisting of a few cities or towns; a petty country governed by a toparch; <as>as, Judea was formerly divided into ten <ex>toparchies</ex></as>.</def>  <rj><au>Fuller.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Top-armor</ent><br/
<hw>Top"-ar`mor</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Naut.)</fld> <def>A top railing supported by stanchions and equipped with netting.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Topau</ent><br/
<hw>To"pau</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Zool.)</fld> <def>The rhinocerous bird <sd>(a)</sd>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Topaz</ent><br/
<hw>To"paz</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[OE. <ets>topas</ets>, F. <ets>topaze</ets>, L. <ets>topazos</ets>, or <ets>topazion</ets>, a kind of precious stone, Gr. <grk>to`pazos</grk>, <grk>topa`zion</grk>; possibly akin to Skr. <ets>tap</ets> to glow (cf. <er>Tepid</er>). According to some, the name is from <ets>Topazos</ets>, a small island in the Red Sea, where the Romans obtained a stone which they called by this name, but which is the chrysolite of the moderns.]</ety> <sn>1.</sn> <fld>(Min.)</fld> <def>A mineral occurring in rhombic prisms, generally yellowish and pellucid, also colorless, and of greenesh, bluish, or brownish shades. It sometimes occurs massive and opaque. It is a fluosilicate of alumina, and is used as a gem.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Zool.)</fld> <def>Either one of two species of large, brilliantly colored humming birds of the genus <gen>Topaza</gen> (<spn>Topaza pella</spn> or <spn>Topaza pyra</spn>), of South America and the West Indies.</def><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ The two tail feathers next to the central ones are much longer that the rest, curved, and crossed. The Throat is metallic yellowish-green, with a tint like topaz in the center, the belly is bright crimson, the back bright red. Called also <altname>topaz hummer</altname>.</note><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>False topaz</b></col>. <fld>(Min.)</fld> <cd>See the Note under <er>Quartz</er>.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Topazolite</ent><br/
<hw>To*paz"o*lite</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[<ets>Topaz</ets> + <ets>-lite</ets>; cf. F. <ets>topazolithe</ets>.]</ety> <fld>(Min.)</fld> <def>A topaz-yellow variety of garnet.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Top billing</ent><br/
<hw>Top" bil*ling</hw> <pr>(?)</pr>, <pos>n.</pos> <def>the most prominent place in an advertisement or program for a theatrical performance, movie, fundraising affair, etc.; the place where the star's name usually is located.</def><br/
[<source>PJC</source>]</p>

<p><ent>Top-block</ent><br/
<hw>Top"-block`</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Naut.)</fld> <def>A large ironbound block strapped with a hook, and, when used, hung to an eyebolt in the cap, -- used in swaying and lowering the topmast.</def>  <rj><au>Totten.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Top-boots</ent><br/
<hw>Top"-boots</hw> <pr>(?)</pr>, <pos>n. pl.</pos> <def>High boots, having generally a band of some kind of light-colored leather around the upper part of the leg; riding boots.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Top-chain</ent><br/
<hw>Top"-chain`</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Naut.)</fld> <def>A chain for slinging the lower yards, in time of action, to prevent their falling, if the ropes by which they are hung are shot away.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Top-cloth</ent><br/
<hw>Top"-cloth</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Naut.)</fld> <def>A piece of canvas used to cover the hammocks which are lashed to the top in action to protect the topmen.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Topcoat</ent><br/
<hw>Top"coat`</hw> <pr>(?)</pr>, <pos>n.</pos> <def>An outer coat; an overcoat.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Top-drain</ent><br/
<hw>Top"-drain`</hw> <pr>(?)</pr>, <pos>v. t.</pos> <def>To drain the surface of, as land; <as>as, to <ex>top-drain</ex> a field or farm</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Top-draining</ent><br/
<hw>Top"-drain`ing</hw>, <pos>n.</pos> <def>The act or practice of drining the surface of land.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Top-dress</ent><br/
<hw>Top"-dress`</hw> <pr>(?)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Top-dressed</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Top-dressing</conjf>.]</vmorph> <def>To apply a surface dressing of manureto,as land.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Top-dressing</ent><br/
<hw>Top"-dress`ing</hw>, <pos>n.</pos> <def>The act of applying a dressing of manure to the surface of land; also, manure so applied.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tope</ent><br/
<hw>Tope</hw> <pr>(t<omac/p)</pr>, <pos>n.</pos> <ety>[Probably from Skr. <ets>st<umac/pa</ets> a tope, a stupa, through Prakrit <ets>th<umac/po</ets>.]</ety> <def>A moundlike Buddhist sepulcher, or memorial monument, often erected over a Buddhist relic.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tope</ent><br/
<hw>Tope</hw>, <pos>n.</pos> <ety>[Tamil <ets>t<omac/ppu</ets>.]</ety> <def>A grove or clump of trees; <as>as, a toddy <ex>tope</ex></as>.</def> <mark>[India]</mark>  <rj><au>Whitworth.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tope</ent><br/
<hw>Tope</hw>, <pos>n.</pos> <sn>1.</sn> <fld>(Zool.)</fld> <def>A small shark or dogfish (<spn>Galeorhinus galeus</spn> syn. <spn>Galeus galeus</spn>), native of <geog>Europe</geog>, but found also on the coasts of <state>California</state> and <geog>Tasmania</geog>; -- called also <altname>toper</altname>, <altname>oil shark</altname>, <altname>miller's dog</altname>, and <altname>penny dog</altname>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Zool.)</fld> <def>The wren.</def> <mark>[Prov. Eng.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tope</ent><br/
<hw>Tope</hw>, <pos>v. i.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Toped</conjf> <pr>(t<omac/pt)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Toping</conjf>.]</vmorph> <ety>[F. <ets>t<ocir/per</ets> to cover a stake in playing at dice, to accept an offer, <ets>t<ocir/pe</ets> agreed!; -- perhaps imitative of the sound of striking hands on concluding a bargain.  From being used in English as a drinking term, probably at first in accepting a toast.]</ety> <def>To drink hard or frequently; to drink strong or spiritous liquors to excess.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>If you <qex>tope</qex> in form, and treat.</q> <rj><qau>Dryden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Topek</ent><br/
<hw>To"pek</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Eskimo <ets>tupek</ets>.]</ety> <def>An Eskimo house made of material other than snow, esp. one having walls of turf, driftwood, rock, or skin, and a roof of skins of the walrus or seal.  In Alaska it is often partially underground and covered with timber and turf.  Topeks are also used by Indians of the lower <geog>Yukon</geog> region.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Toper</ent><br/
<hw>To"per</hw> <pr>(t<omac/"p<etil/r)</pr>, <pos>n.</pos> <def>One who topes, or drinks frequently or to excess; a drunkard; a sot.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Topet</ent><br/
<hw>Top"et</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F. <ets>toupet</ets> tuft. See <er>Touper</er>.]</ety> <fld>(Zool.)</fld> <def>The European crested titmouse.</def> <mark>[Prov. Eng.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Top fermentation</ent><br/
<hw>Top fermentation</hw>. <def>An alcoholic fermentation during which the yeast cells are carried to the top of the fermenting liquid. It proceeds with some violence and requires a temperature of 14-30<deg/ C. (58-86<deg/ F.). It is used in the production of ale, porter, etc., and of wines high in alcohol, and in distilling.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Topflight</ent><br/
<hw>Top"flight</hw>, <pos>a.</pos> <def>unexcelled; of the highest grade; of greatest skill; exhibiting excellence; <as>a <ex>topflight</ex> performance</as>.</def><br/
[<source>PJC</source>]</p>

<p><ent>Topful</ent><br/
<hw>Top"ful</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Full to the top, ore brim; brimfull.</def> <ldquo/<xex>Topful</xex> of direst cruelty.<rdquo/  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>[He] was so <qex>topful</qex> of himself, that he let it spill on all the company.</q> <rj><qau>I. Watts.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Topgallant</ent><br/
<hw>Top`gal"lant</hw> <pr>(?)</pr>, <pos>a.</pos><br/
[<source>1913 Webster</source>]</p>

<p><sn>1.</sn> <fld>(Naut.)</fld> <def>Situated above the topmast and below the royal mast; designatb, or pertaining to, the third spars in order from the deck; <as>as, the <ex>topgallant</ex> mast, yards, braces, and the like</as>. See <xex>Illustration</xex> of <er>Ship</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Fig.: Highest; elevated; splendid.</def> <ldquo/The consciences of <xex>topgallant</xex> sparks.<rdquo/  <rj><au>L'Estrange.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Topgallant breeze</b></col>, <cd>a breeze in which the topgallant sails may properly be carried.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Topgallant</ent><br/
<hw>Top`gal"lant</hw>, <pos>n.</pos> <sn>1.</sn> <fld>(Naut.)</fld> <def>A topgallant mast or sail.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Fig.: Anything elevated or splendid.</def>  <rj><au>Bacon.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Toph</ent><br/
<hw>Toph</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>tophus</ets>, <ets>tofus</ets>, tufa, or tuft.  Cf. <er>Tufa</er>, <er>Tofus</er>, <er>Tophus</er>.]</ety> <fld>(Min.)</fld> <def>kind of sandstone.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tophaceous</ent><br/
<hw>To*pha"ceous</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>tophaceus</ets>, <ets>tofaceus</ets>.]</ety> <def>Gritty; sandy; rough; stony.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Top-hamper</ent><br/
<hw>Top"-ham`per</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Naut.)</fld> <def>The upper rigging, spars, etc., of a ship.</def> <altsp>[Written also <asp>top hamper</asp>.]</altsp><br/
[<source>1913 Webster</source>]</p>

<p><q>All the ships of the fleet . . . were so encumbered with <qex>tophamper</qex>, so overweighted in proportion to their draught of water, that they could bear but little canvas, even with smooth seas and light and favorable winds.</q> <rj><qau>Motley.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Top-heavy</ent><br/
<hw>Top"-heav`y</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Having the top or upper part too heavy for the lower part.</def>  <rj><au>Sir H. Wotton.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tophet</ent><br/
<hw>To"phet</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Heb. <ets>t<omac/phet</ets>, literally, a place to be spit upon, an abominable place, fr. <ets>t<umac/ph</ets> to spit out.]</ety> <def>A place lying east or southeast of Jerusalem, in the valley of Hinnom.</def> <altsp>[Written also <asp>Topheth</asp>.]</altsp><br/
[<source>1913 Webster</source>]</p>

<p><q>And he defiled <qex>Topheth</qex>, which is in the valley of the children of Hinnom.</q> <rj><qau>2 Kings xxiii. 10.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ It seems to have been at first part of the royal garden, but it was afterwards defiled and polluted by the sacrifices of Baal and the fires of Moloch, and resounded with the cries of burning infants. At a later period, its altars and high places were thrown down, and all the filth of the city poured into it, until it became the abhorrence of Jerusalem, and, in symbol, the place where are wailing and gnashing of teeth.</note><br/
[<source>1913 Webster</source>]</p>

<p><q>The pleasant valley of Hinnom, <qex>Tophet</qex> thence<br/
And black Gehenna called, the type of hell.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tophin</ent><br/
<hw>Toph"in</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Min.)</fld> <def>Same as <er>Toph</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><-- p. 1519 --><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tophus</ent><br/
<hw>To"phus</hw> <pr>(?)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Tophi</plw> <pr>(#)</pr>.</plu>  <ety>[NL.: cf. F. <ets>tophus</ets> a mineral concretion in the joint. See <er>Toph</er>.]</ety> <altsp>[Written also <asp>tofus</asp>.]</altsp> <sn>1.</sn> <fld>(Med.)</fld> <def>One of the mineral concretions about the joints, and in other situations, occurring chiefly in gouty persons. They consist usually of urate of sodium; when occurring in the internal organs they are also composed of phosphate of calcium.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Min.)</fld> <def>Calcareous tufa.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Topi</ent><br/
<hw>To"pi</hw> <pr>(?)</pr>, <pos>n.</pos> <def>An antelope (<spn>Damaliscus corrigum jimela</spn>) having a glossy purplish brown coat. It is related to the blesbok and is native of East Africa. Also, any of various related varieties of other districts south of the Sahara.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Topiarian</ent><br/
<hw>Top`i*a"ri*an</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[See <er>Toplary</er>.]</ety> <def>Of or pertaining to the ornamental cutting and trimming of trees, hedges, etc.; practicing ornamental gardening.</def> <mark>[R.]</mark> <ldquo/The <xex>topiarian</xex> artist.<rdquo/  <rj><au>Sir W. Scott.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>All the pedantries of the <qex>topiarian</qex> art.</q> <rj><qau>C. Kingsley.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Topiary</ent><br/
<hw>Top"i*a*ry</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>topiarius</ets> belonging to ornamental gardening, fr. <ets>topia</ets> (sc. <ets>opera</ets>) ornamental gardening, fr. Gr. <?/ a place.]</ety> <def>Of or pertaining to ornamental gardening; produced by cutting, trimming, etc.; topiarian.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Topiary work</b></col>, <cd>arbors, shrubbery, hedges, or the like, cut and trimmed into fanciful forms, as of animals, buildings, etc.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Topic</ent><br/
<hw>Top"ic</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F. <ets>topiques</ets>, pl., L. <ets>topica</ets> the title of a work of Aristotle, Gr. <grk>topika`</grk>, fr. <grk>topiko`s</grk> of or for place, concerning <grk>to`poi</grk>, or commonplaces, fr. <grk>to`pos</grk> a place.]</ety> <sd>(a)</sd> <def>One of the various general forms of argument employed in probable as distinguished from demonstrative reasoning, -- denominated by Aristotle <grk>to`poi</grk> (literally, places), as being the places or sources from which arguments may be derived, or to which they may be referred; also, a prepared form of argument, applicable to a great variety of cases, with a supply of which the ancient rhetoricians and orators provided themselves; a commonplace of argument or oratory.</def> <sd>(b)</sd> <pluf>pl.</pluf> <def>A treatise on forms of argument; a system or scheme of forms or commonplaces of argument or oratory; <as>as, the <ex>Topics</ex> of Aristotle</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>These <qex>topics</qex>, or loci, were no other than general ideas applicable to a great many different subjects, which the orator was directed to consult.</q> <rj><qau>Blair.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>In this question by [reason] I do not mean a distinct <qex>topic</qex>, but a transcendent that runs through all <qex>topics</qex>.</q> <rj><qau>Jer. Taylor.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>An argument or reason.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>Contumacious persons, who are not to be fixed by any principles, whom no <qex>topics</qex> can work upon.</q> <rj><qau>Bp. Wilkins.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>The subject of any distinct portion of a discourse, or argument, or literary composition; also, the general or main subject of the whole; a matter treated of; a subject, as of conversation or of thought; a matter; a point; a head.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <fld>(Med.)</fld> <def>An external local application or remedy, as a plaster, a blister, etc.</def> <mark>[Obsoles.]</mark>  <rj><au>Wiseman.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Topic</ent><br/
<hw>Top"ic</hw>, <pos>a.</pos> <def>Topical.</def>  <rj><au>Drayton. Holland.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Topical</ent><br/
<hw>Top"ic*al</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[Cf. F. <ets>topique</ets>, LL. <ets>topicus</ets>, Gr. <?/. See <er>Topic</er>, <pos>n.</pos>]</ety> <sn>1.</sn> <def>Of or pertaining to a place; limited; logical application; <as>as, a <ex>topical</ex> remedy; a <ex>topical</ex> claim or privilege</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Rhet. & logic)</fld> <def>Pertaining to, or consisting of, a topic or topics; according to topics.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>Resembling a topic, or general maxim; hence, not demonstrative, but merely probable, as an argument.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Evidences of fact can be no more than <qex>topical</qex> and probable.</q>  <rj><au>Sir M. Hale.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Topically</ent><br/
<hw>Top"ic*al*ly</hw>, <pos>adv.</pos> <def>In a topical manner; with application to, or limitation of, a particular place or topic.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Topknot</ent><br/
<hw>Top"knot`</hw> <pr>(?)</pr>, <pos>n.</pos> <sn>1.</sn> <def>A crest or knot of feathers upon the head or top, as of a bird; also, an ornamental knot worn on top of the head, as by women.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>A great, stout servant girl, with cheeks as red as her <qex>topknot</qex>.</q> <rj><qau>Sir W. Scott.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Zool.)</fld> <def>A small Europen flounder (<spn>Rhoumbus punctatus</spn>). The name is also applied to allied species.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Topless</ent><br/
<hw>Top"less</hw>, <pos>a.</pos> <sn>1.</sn> <def>Having no top, or no visble fop; hence, fig.: very lofty; supreme; unequaled.</def> <ldquo/ The <xex>topless</xex> Apennines.<rdquo/  <ldquo/<xex>Topless</xex> fortunes.<rdquo/  <rj><au>Beau. & Fl.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>wearing no clothes above the waist, said especially of women.</def><br/
[<source>PJC</source>]</p>

<p><sn>3.</sn> <def>featuring entertainment by women wearing no clothes above the waist; <as>as, a <ex>topless</ex> go-go joint</as>.</def><br/
[<source>PJC</source>]</p>

<p><ent>Top-light</ent><br/
<hw>Top"-light`</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Naut.)</fld> <def>A lantern or light on the top of a vessel.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Topman</ent><br/
<hw>Top"man</hw> <pr>(?)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Topmem</plw> <pr>(<?/)</pr>.</plu> <sn>1.</sn> <def>See <er>Topsman</er>, 2.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Naut.)</fld> <def>A man stationed in the top.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Topmast</ent><br/
<hw>Top"mast</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Naut.)</fld> <def>The second mast, or that which is next above the lower mast, and below the topgallant mast.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Topmost</ent><br/
<hw>Top"most`</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Highest; uppermost; <as>as, the <ex>topmost</ex> cliff; the <ex>topmost</ex> branch of a tree</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The nightngale may claim the <qex>topmost</qex> bough.</q> <rj><qau>Cowper.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Topographer</ent><br/
<hw>To*pog"ra*pher</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Cf. F. <ets>topographe</ets>, Cr. <?/ .]</ety> <def>One who is skilled in the science of topography; one who describes a particular place, town, city, or tract of land.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Dante is the one authorized <qex>topographer</qex> of the mediaeval hell.</q> <rj><qau>Milman.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Topographical</ent><br/
<ent>Topographic</ent><br/
<mhw>{ <hw>Top`o*graph"ic</hw> <pr>(?)</pr>, <pos>a.</pos> <hw>Top`o*graph"ic*al</hw> <pr>(?)</pr>, }</mhw><ety>[Cf. F. <ets>topographique</ets>.]</ety> <def>Of or pertaining to topography; descriptive of a place.</def> -- <wordforms><wf>Top`o*graph"ic*al*ly</wf>, <pos>adv.</pos></wordforms><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Topographical map</b></col>. <cd>See under <er>Cadastral</er>. -- Topographical surveying. See under <er>Surveying</er>.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Topographist</ent><br/
<hw>To*pog"ra*phist</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A topographer.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Topography</ent><br/
<hw>To*pog"ra*phy</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F. <ets>topographie</ets>, Gr. <?/; <?/ a place + <?/ to write.]</ety> <def>The description of a particular place, town, manor, parish, or tract of land; especially, the exact and scientific delineation and description in minute detail of any place or region.</def><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ <xex>Topography</xex>, as the description of particular places, is distinguished from <xex>chorography</xex>, the description of a region or a district, and for <xex>geography</xex>, the description of the earth or of countries.</note>  <rj><au>Brande & C.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Topologist</ent><br/
<hw>To*pol"o*gist</hw> <pr>(?)</pr>, <pos>n.</pos> <def>a mathematician specializing in topology.</def> <ldquo/A <ex>topologist</ex> is someone who doesn't know the difference between a donut and a coffee cup.<rdquo/<br/
[<source></source>]</p>

<p><ent>Topology</ent><br/
<hw>To*pol"o*gy</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Gr. <?/ place + <ets>-logy</ets>.]</ety> <def>The art of, or method for, assisting the memory by associating the thing or subject to be remembered with some place.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>a branch of mathematics which studies the properties of geometrical forms which retain their identity under certain transformations, such as stretching or twisting, which are homeomorphic.  See also <er>topologist</er>.</def><br/
[<source>PJC</source>]</p>

<p><sn>3.</sn> <def>configuration, especially in three dimensions; -- used, e. g. of the configurations taken by macromolecules, such as superhelical DNA.</def><br/
[<source>PJC</source>]</p>

<p><ent>Toponomy</ent><br/
<hw>To*pon"o*my</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Gr. <?/ place + <?/ to distribute.]</ety> <def>The designation of position and direction.</def>  <rj><au>B. G. Wilder.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Toponym</ent><br/
<hw>Top"o*nym</hw> <pr>(?)</pr>, <pos>n.</pos> <sn>1.</sn> <def>A name of a place.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><sn>2.</sn> <def>a name, as in the binomial name of a plant, based on, or derived from, a place name, or based on the location of the thing named.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Toponymy</ent><br/
<hw>To*pon"y*my</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A system of toponyms; the use of toponyms.</def> -- <wordforms><wf>To*pon"y*mal</wf> <pr>(#)</pr>, <wf>Top`o*nym"ic</wf> <pr>(#)</pr>, <wf>Top`o*nym"ic*al</wf> <pr>(#)</pr>, <pos>a.</pos></wordforms><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Topophone</ent><br/
<hw>Top"o*phone</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Gr. <?/ place + Gr. <?/ sound.]</ety> <def>A double ear trumpet for estimating the direction from which sounds proceed, esp. for the use of navigators.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Top out</ent><br/
<hw>Top out</hw>. <fld>(Building)</fld> <def>To top off; to finish by putting on a cap of top (uppermost) course (called a <col><b>top`ping-out" course</b></col>).</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Topper</ent><br/
<hw>Top"per</hw> <pr>(?)</pr>, <pos>n.</pos> <sn>1.</sn> <def>One that tops, in any sense of the verb;</def> <specif>specif.:</specif> <sd>(a)</sd> <def>A cover of a top layer or part.</def> <mark>[Colloq.]</mark> <sd>(b)</sd> <def>One that excels, surpasses, or is extraordinary of its kind.</def> <mark>[Slang]</mark> <sd>(c)</sd> <def>Any device for cutting off tops; <as>as, a turnip <ex>topper</ex></as>.</def> <sd>(d)</sd> <def>One who tops steel ingots.</def> <sd>(e)</sd> <def>A three-square float (file) used by comb makers.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><sn>2.</sn>  <def>A top hat.</def> <mark>[Slang or Colloq.]</mark><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><sn>3.</sn>  <def>Tobacco left in the bottom of a pipe bowl; -- so called from its being often taken out and placed on top of the newly filled bowl. Also, a cigar stump.</def> <mark>[Slang]</mark><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Toppiece</ent><br/
<hw>Top"piece`</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A small wig for the top of the head; a toupee.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Topping</ent><br/
<hw>Top"ping</hw> <pr>(?)</pr>, <pos>a.</pos> <sn>1.</sn> <def>Rising above; surpassing.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Hence, assuming superiority; proud.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The great and flourishing condition of some of the <qex>topping</qex> sinners of the world.</q> <rj><qau>South.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>Fine; gallant.</def> <mark>[Slang]</mark>  <rj><au>Johnson.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Topping</ent><br/
<hw>Top"ping</hw>, <pos>n.</pos> <sn>1.</sn> <def>The act of one who tops; the act of cutting off the top.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Naut.)</fld> <def>The act of raising one extremity of a spar higher than the other.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <pluf>pl.</pluf> <def>That which comes from hemp in the process of hatcheling.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <fld>(Angling)</fld> <def>The tail of an artificial fly.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><sn>5.</sn> <def>something placed on top of another thing, such as a sauce or garnish on food; <as>as, a three-scoop banana split with three different <ex>toppings</ex></as>; an ice-cream cone with chocolate fudge <ex>topping</ex>.</def><br/
[<source>PJC</source>]</p>

<p><cs><col><b>Topping lift</b></col> <fld>(Naut.)</fld>, <cd>a large, strong tackle employed to raise or top the end of a gaff, or of a boom.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Toppingly</ent><br/
<hw>Top"ping*ly</hw>, <pos>adv.</pos> <def>In a topping or proud manner.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Toppingly</ent><br/
<hw>Top"ping*ly</hw>, <pos>a.</pos> <def>Same as <er>Topping</er>, <pos>a.</pos>, 3.</def> <mark>[Obs.]</mark>  <ldquo/<xex>Topping</xex> quests.<rdquo/  <rj><au>Tusser.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Topple</ent><br/
<hw>Top"ple</hw> <pr>(?)</pr>, <pos>v. i.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Toppled</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Toppling</conjf>.]</vmorph> <ety>[From <er>Top</er> summit.]</ety> <def>To fall forward; to pitch or tumble down.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Though castles <qex>topple</qex> on their warders' heads.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Topple</ent><br/
<hw>Top"ple</hw>, <pos>v. t.</pos> <def>To throw down; to overturn.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>He <qex>topple</qex> crags from the precipice.</q> <rj><qau>Longfellow.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Top-proud</ent><br/
<hw>Top"-proud`</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Proud to the highest degree.</def> <mark>[R.]</mark> <ldquo/This <xex>top-proud</xex> fellow.<rdquo/  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Top rake</ent><br/
<hw>Top rake</hw>. <fld>(Mech.)</fld> <def>The angle that the front edge of the point of a tool is set back from the normal to the surface being cut.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Top-rope</ent><br/
<hw>Top"-rope`</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Naut.)</fld> <def>A rope used for hoisting and lowering a topmast, and for other purposes.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Topsail</ent><br/
<hw>Top"sail`</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Naut.)</fld> <def>In a square-rigged vessel, the sail next above the lowermost sail on a mast. This sail is the one most frequently reefed or furled in working the ship. In a fore-and-aft rigged vessel, the sail set upon and above the gaff. See <er>Cutter</er>, <er>Schooner</er>, <er>Sail</er>, and <er>Ship</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Topsail schooner</b></col>. <fld>(Naut.)</fld> <cd>See <er>Schooner</er>, and <xex>Illustration</xex> in Appendix.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tops-and-bottoms</ent><br/
<hw>Tops"-and-bot`toms</hw> <pr>(?)</pr>, <pos>n. pl.</pos> <def>Small rolls of dough, baked, cut in halves, and then browned in an oven, -- used as food for infants.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>'T is said that her <qex>top-and-bottoms</qex> were gilt.</q> <rj><qau>Hood.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Top secret</ent><br/
<hw>Top" sec`ret</hw> <pr>(?)</pr>, <pos>a.</pos> <sn>1.</sn> <fld>(Mil.)</fld> <def>containing information whose disclosure could result in grave danger to the national security; -- the highest of the three commonly known levels of national security classification, the others being <contr>confidential</contr> and <contr>secret</contr>.</def> <note>The names of the higher levels of secrecy classification are themselves classified secrets.</note><br/
[<source>PJC</source>]</p>

<p><sn>2.</sn> <def>kept secret by very careful and strict rules for disclosure.</def><br/
[<source>PJC</source>]</p>

<p><ent>Top sergeant</ent><br/
<hw>Top" ser`geant</hw> <pr>(?)</pr>, <pos>n.</pos> <def>same as First Sergeant.</def><br/
[<source>PJC</source>]</p>

<p><ent>Top-shaped</ent><br/
<hw>Top"-shaped`</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Having the shape of a top;</def> <fld>(Bot.)</fld> <def>cone-shaped, with the apex downward; turbinate.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Top-shell</ent><br/
<hw>Top"-shell`</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Zool.)</fld> <def>Any one of numerous species of marine top-shaped shells of the genus <gen>Trochus</gen>, or family <fam>Trochidae</fam>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Topsman</ent><br/
<hw>Tops"man</hw> <pr>(?)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Topsmen</plw> <pr>(<?/)</pr>.</plu> <sn>1.</sn> <def>The chief drover of those who drive a herd of cattle.</def>  <rj><au>P. Cyc.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>The uppermost sawyer in a saw pit; a topman.</def>  <rj><au>Simmonds.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Topsoil</ent><br/
<hw>Top"soil`</hw> <pr>(?)</pr>, <pos>n.</pos> <def>The upper layer of soil; surface soil.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Topsoiling</ent><br/
<hw>Top"soil`ing</hw>, <pos>n.</pos> <fld>(Engin.)</fld> <def>The act or art of taking off the top soil of land before an excavation or embankment is begun.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Topstone</ent><br/
<hw>Top"stone`</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A stone that is placed on the top, or which forms the top.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Topsy-turvy</ent><br/
<hw>Top"sy-tur"vy</hw> <pr>(?)</pr>, <pos>adv.</pos> <ety>[Earlier <ets>topside-turvey</ets>, <ets>topsy-tervy</ets>; probably for <ets>top so turvy</ets>; that is, the top as turvy, as it were turvy; where <ets>turvy</ets> probably means, overturned, fr. <er>AS</er>. <ets>torfian</ets> to throw.]</ety> <def>In an inverted posture; with the top or head downward; upside down; <as>as, to turn a carriage <ex>topsy-turvy</ex></as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Top-tackle</ent><br/
<hw>Top"-tac`kle</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Naut.)</fld> <def>A tackle used in hoisting and lowering the topmast.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Top-timbers</ent><br/
<hw>Top"-tim`bers</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Naut.)</fld> <def>The highest timbers on the side of a vessel, being those above the futtocks.</def>  <rj><au>R. H. Dana, Jr.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Top-tool</ent><br/
<hw>Top"-tool`</hw> <pr>(-t<oomac/l`)</pr>, <pos>n.</pos> <fld>(Blacksmithing.)</fld> <def>A tool applied to the top of the work, in distinction from a tool inserted in the anvil and on which the work is placed.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Toque</ent><br/
<hw>Toque</hw> <pr>(t<omac/k)</pr>, <pos>n.</pos> <ety>[F. <ets>toque</ets>; of Celtic origin; cf. W. <ets>toc</ets>.]</ety> <sn>1.</sn> <def>A kind of cap worn in the 16th century, and copied in modern fashions; -- called also <altname>toquet</altname>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>His velvet <qex>toque</qex> stuck as airily as ever upon the side of his head.</q> <rj><qau>Motley.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Zool.)</fld> <def>A variety of the bonnet monkey.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Toquet</ent><br/
<hw>To*quet"</hw> <pr>(?)</pr>, <pos>n.</pos> <def>See <er>Toque</er>, 1.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tor</ent><br/
<hw>Tor</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[AS. <ets>torr</ets>; cf. Gael. <ets>torr</ets>.  Cf. <er>Tower</er>.]</ety><br/
[<source>1913 Webster</source>]</p>

<p><sn>1.</sn> <def>A tower; a turret.</def> <mark>[R.]</mark>  <rj><au>Ray.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>High-pointed hill; a rocky pinnacle.</def> <mark>[Prov. Eng.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>A rolling range of dreary moors, unbroken by <qex>tor</qex> or tree.</q> <rj><qau>C. Kingsley.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Torase</ent><br/
<ent>Torace</ent><br/
<mhw>{ <hw>To*race"</hw> <pr>(?)</pr>, <hw>To*rase"</hw>  }</mhw>, <pos>v. t.</pos> <ety>[Pref. <ets>to-</ets> + <er>OE</er>. <ets>r<?/sen</ets> to rage.]</ety> <def>To scratch to pieces.</def> <mark>[Obs.]</mark>  <rj><au>Chaucer.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tora</ent><br/
<ent>Torah</ent><br/
<mhw>{ ||<hw>To"rah</hw>, ||<hw>To"ra</hw>  }</mhw> <pr>(?)</pr>, <pos>n.</pos>; <plu>pl. <plw>Toroth</plw> <pr>(#)</pr></plu>. <ety>[Heb. <ets>t<omac/r<amac/h</ets>.]</ety> <fld>(Jewish Lit.)</fld> <sd>(a)</sd> <def>A law; a precept.</def></p>

<p><q>A considerable body of priestly <qex>Toroth</qex>.</q>  <rj><qau>S. R. Driver.</qau></rj></p>

<p><sd>(b)</sd> <def>Divine instruction; revelation.</def></p>

<p><q><qex>Tora</qex>, . . . before the time of Malachi, is generally used of the revelations of God's will made through the prophets.</q>  <rj><qau>T. K. Cheyne.</qau></rj></p>

<p><sd>(c)</sd> <def>The Pentateuch or <ldquo/Law of Moses.<rdquo/</def></p>

<p><q>The Hebrew Bible is divided into three parts: (1) The <qex>Torah</qex>, <ldquo/Law,<rdquo/ or Pentateuch. (2) The Prophets (Nevi'im in Hebrew) . . . (3) The Kethubim, or the <ldquo/Writings,<rdquo/ generally termed Hagiographa.  From the first letters of these three parts, the word <ldquo/Tanakh<rdquo/ is derived, and used by Jews as the name of their Bible, the Christian Old Testament.</q>  <rj><qau>C. H. H. Wright.</qau></rj><br/
[<source>Webster 1913 Suppl.</source> <source>+PJC</source>]</p>

<p><ent>Torana</ent><br/
<ent>Toran</ent><br/
<mhw>{ <hw>To"ran</hw> <pr>(?)</pr>, ||<hw>To"ra*na</hw> <pr>(?)</pr> }</mhw>, <pos>n.</pos> <ety>[Skr. <ets>t<omac/ra<nsdot/a</ets> an arch, a gate.]</ety> <fld>(Indian Arch.)</fld> <def>A gateway, commonly of wood, but sometimes of stone, consisting of two upright pillars carrying one to three transverse lintels. It is often minutely carved with symbolic sculpture, and serves as a monumental approach to a Buddhist temple.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Torbernite</ent><br/
<hw>Tor"bern*ite</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[So named after <etsep>Torber</etsep> Bergmann, a Swedish chemist.]</ety> <fld>(Min.)</fld> <def>A mineral occurring in emerald-green tabular crystals having a micaceous structure.  It is a hydrous phosphate of uranium and copper. Called also <altname>copper uranite</altname>, and <altname>chalcolite</altname>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Torc</ent><br/
<hw>Torc</hw> <pr>(t<ocir/rk)</pr>, <pos>n.</pos> <def>Same as <er>Torque</er>, 1.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Torch</ent><br/
<hw>Torch</hw> <pr>(t<ocir/rch)</pr>, <pos>n.</pos> <ety>[OE. <ets>torche</ets>, F. <ets>torche</ets> a torch, rag, wisp, pad; probably from a derivative of L. <ets>torquere</ets>, <ets>tortum</ets>, to twist, because twisted like a rope; cf. F. <ets>torcher</ets> to rub, wipe, It. <ets>topcia</ets> a torch, <ets>torciare</ets> to wrap, twist, OF. <ets>torse</ets> a torse.  Cf. <er>Torture</er>.]</ety> <def>A light or luminary formed of some combustible substance, as of resinous wood; a large candle or flambeau, or a lamp giving a large, flaring flame.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>They light the nuptial <qex>torch</qex>.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><-- <sn>2.</sn> <def>A flashlight.</def> <mark>[Brit.]</mark> --><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Torch thistle</b></col>. <fld>(Bot.)</fld> <cd>See under <er>Thistle</er>.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Torchbearer</ent><br/
<hw>Torch"bear`er</hw> <pr>(?)</pr>, <pos>n.</pos> <def>One whose office it is to carry a torch.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Torcher</ent><br/
<hw>Torch"er</hw> <pr>(?)</pr>, <pos>n.</pos> <def>One who gives light with a torch, or as if with a torch.</def> <mark>[Obs.]</mark>  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Torchlight</ent><br/
<hw>Torch"light`</hw> <pr>(?)</pr>, <pos>n.</pos> <def>The light of a torch, or of torches.  Also adjectively; <as>as, a <ex>torchlight</ex> procession</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Torchon lace</ent><br/
<hw>Tor"chon lace`</hw> <pr>(?)</pr>. <ety>[F. <ets>torchon</ets> a kind of coarse napkin.]</ety> <def>a simple thread lace worked upon a pillow with coarse thread; also, a similar lace made by machinery.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Torchon paper</ent><br/
<hw>Tor"chon pa"per</hw> <pr>(?)</pr>. <ety>[F. <ets>papier torchon</ets>.]</ety> <def>Paper with a rough surface; esp., handmade paper of great hardness for the use of painters in water colors.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Torch race</ent><br/
<hw>Torch race</hw>. <def>A race by men carrying torches, as in ancient Greece.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Torch singer</ent><br/
<hw>Torch" sing`er</hw> <pr>(?)</pr>, <pos>n.</pos> <def>one who sings torch songs.</def><br/
[<source>PJC</source>]</p>

<p><ent>Torch song</ent><br/
<hw>Torch" song`</hw> <pr>(?)</pr>, <pos>n.</pos> <def>a popular song with a sentimental theme of unrequited love or other form of unhappiness.</def><br/
[<source>PJC</source>]</p>

<p><ent>Torchwood</ent><br/
<hw>Torch"wood`</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Bot.)</fld> <def>The inflammable wood of certain trees (<spn>Amyris balsamifera</spn>, <spn>Amyris Floridana</spn>, etc.); also, the trees themselves.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Torchwort</ent><br/
<hw>Torch"wort`</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Bot.)</fld> <def>The common mullein, the stalks of which, dipped in suet, anciently served for torches. Called also <altname>torch</altname>, and <altname>hig-taper</altname>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tore</ent><br/
<hw>Tore</hw> <pr>(?)</pr>, <def><pos>imp.</pos> of <er>Tear</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tore</ent><br/
<hw>Tore</hw>, <pos>n.</pos> <ety>[Probably from the root of <ets>tear</ets>; cf. W. <ets>t<oacute/r</ets> a break, cut, <ets>t<oacute/ri</ets> to break, cut.]</ety> <def>The dead grass that remains on mowing land in winter and spring.</def> <mark>[Prov. Eng.]</mark>  <rj><au>Mortimer.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tore</ent><br/
<hw>Tore</hw>, <pos>n.</pos> <ety>[See <er>Torus</er>.]</ety> <sn>1.</sn> <fld>(Arch.)</fld> <def>Same as <er>Torus</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Geom.)</fld> <def>same as <er>torus</er>.</def><br/
[<source>PJC</source>]</p>

<p><ent>Toreador</ent><br/
<hw>To"re*a*dor`</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Sp., fr. <ets>torear</ets> to fight bulls, fr.L. <ets>taurus</ets> a bull.]</ety> <def>A bullfighter.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>To-rend</ent><br/
<hw>To-rend"</hw> <pr>(?)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>To-rent</conjf>.]</vmorph> <ety>[Pref. <ets>to-</ets> + <ets>rend</ets>.]</ety> <def>To rend in pieces.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>The wolf hath many a sheep and lamb <qex>to-rent</qex>.</q> <rj><qau>Chaucer.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Toret</ent><br/
<hw>Tor"et</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Probably dim. fr. <ets>tore</ets>, <ets>torus</ets>.]</ety> <def>A Turret.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Toret</ent><br/
<hw>Tor"et</hw>, <pos>n.</pos> <def>A ring for fastening a hawk's leash to the jesses; also, a ring affixed to the collar of a dog, etc.</def> <mark>[Obs.]</mark>  <rj><au>Chaucer.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Toreumatography</ent><br/
<hw>To"reu`ma*tog"ra*phy</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Gr. <?/, <?/, embossed work, work in relief (from <?/ to bore through, to work in relief) + <ets>-graphy</ets>.]</ety> <def>A description of sculpture such as bas-relief in metal.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Toreumatology</ent><br/
<hw>To*reu`ma*tol"o*gy</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Gr. <?/ embossed work + <ets>-logy</ets>.]</ety> <def>The art or the description of scupture such as bas-relief in metal; toreumatography.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Toreutic</ent><br/
<hw>To*reu"tic</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[Gr. <?/ belonging to work in relief.]</ety> <fld>(Sculp.)</fld> <def>In relief; pertaining to sculpture in relief, especially of metal; also, pertaining to chasing such as surface ornamentation in metal.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Torgoch</ent><br/
<hw>Tor"goch</hw> <pr>(?)</pr>, <pos>n.</pos> <def>The saibling.</def> <mark>[Prov. Eng.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Torilto</ent><br/
||<hw>To*ril"to</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Cf. Sp. <ets>torillo</ets> a little bull.]</ety> <fld>(Zool.)</fld> <def>A species of Turnix (<spn>Turnix sylvatica</spn>) native of Spain and Northen Africa.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Torinese</ent><br/
<hw>To`rin*ese"</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[It.]</ety> <def>Of or pertaining to Turin.</def> -- <def2><pos>n. sing. & pl.</pos> <def>A native or inhabitant of Turin; collectively, the people of Turin.</def></def2><br/
[<source>1913 Webster</source>]</p>

<p><ent>Torment</ent><br/
<hw>Tor"ment</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[OF. <ets>torment</ets>, F. <ets>tourment</ets>, fr. L. <ets>tormentum</ets> an engine for hurling missiles, an instrument of torture, a rack, torture, fr. <ets>torquere</ets> to turn, to twist, hurl. See <er>Turture</er>.]</ety> <sn>1.</sn> <fld>(Mil. Antiq.)</fld> <def>An engine for casting stones.</def> <mark>[Obs.]</mark>  <rj><au>Sir T. Elyot.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Extreme pain; anguish; torture; the utmost degree of misery, either of body or mind.</def>  <rj><au>Chaucer.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>The more I see<br/
Pleasures about me, so much more I feel<br/
<qex>Torment</qex> within me.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>That which gives pain, vexation, or misery.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>They brought unto him all sick people that were taken with divers diseases and <qex>torments</qex>.</q> <rj><qau>Matt. iv. 24.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Torment</ent><br/
<hw>Tor*ment"</hw> <pr>(?)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>tormented</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>tormenting</conjf>.]</vmorph> <ety>[OF. <ets>tormenter</ets>, F. <ets>tourmenter</ets>.]</ety> <sn>1.</sn> <def>To put to extreme pain or anguish; to inflict excruciating misery upon, either of body or mind; to torture.</def> <ldquo/ Art thou come hither to <xex>torment</xex> us before our time? <rdquo/  <rj><au>Matt. viii. 29.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To pain; to distress; to afflict.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Lord, my servant lieth at home sick of the palsy, grievously <qex>tormented</qex>.</q> <rj><qau>Matt. viii. 6.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>To tease; to vex; to harass; <as>as, to be <ex>tormented</ex> with importunities, or with petty annoyances</as>.</def> <mark>[Colloq.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>To put into great agitation.</def> <mark>[R.]</mark> <ldquo/[They], soaring on main wing, <xex>tormented</xex> all the air.<rdquo/  <rj><au>Milton.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tormenter</ent><br/
<hw>Tor*ment"er</hw> <pr>(?)</pr>, <pos>n.</pos> <sn>1.</sn> <def>One who, or that which, torments; a tormentor.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>An executioner.</def> <mark>[Obs.]</mark>  <rj><au>Chaucer.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tormentful</ent><br/
<hw>Tor*ment"ful</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Full of torment; causing, or accompanied by, torment; excruciating.</def> <mark>[R.]</mark>  <rj><au>Tillotson.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tormentil</ent><br/
<hw>Tor"men*til</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F. <ets>tormentille</ets>; cf. Pr., It., & NL. <ets>tormentilla</ets>, Sp. <ets>tormentila</ets>; all fr. L. <ets>tormentum</ets> pain. So called because it is said to allay pain. See <er>Torment</er>.]</ety> <fld>(Bot.)</fld> <def>A rosaceous herb (<spn>Potentilla Tormentilla</spn>), the root of which is used as a powerful astringent, and for alleviating gripes, or tormina, in diarrhea.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tormenting</ent><br/
<hw>Tor*ment"ing</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Causing torment; <as>as, a <ex>tormenting</ex> dream</as>.</def> -- <wordforms><wf>Tor*ment"ing*ly</wf>, <pos>adv.</pos></wordforms><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tormentise</ent><br/
<hw>Tor"ment*ise</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[See <er>Torment</er>.]</ety> <def>Torture; torment.</def> <mark>[Obs.]</mark>  <rj><au>Chaucer.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tormentor</ent><br/
<hw>Tor*ment"or</hw> <pr>(?)</pr>, <pos>n.</pos> <sn>1.</sn> <def>One who, or that which, torments; one who inflicts penal anguish or tortures.</def>  <rj><au>Jer. Taylor.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Thoughts, my <qex>tormentors</qex>, armed with deadly stings.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Agric.)</fld> <def>An implement for reducing a stiff soil, resembling a harrow, but running upon wheels.</def>  <rj><au>Hebert.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tormentress</ent><br/
<hw>Tor*ment"ress</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A woman who torments.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Fortune ordinarily cometh after to whip and punish them, as the scourge and <qex>tormentress</qex> of glory and honor.</q> <rj><qau>Holland.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tormentry</ent><br/
<hw>Tor"ment*ry</hw> <pr>(?)</pr>, <pos>n.</pos> <def>Anything producing torment, annoyance, or pain.</def> <mark>[Obs.]</mark>  <rj><au>Chaucer.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><-- p. 1520 --><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tormina</ent><br/
||<hw>Tor"mi*na</hw> <pr>(?)</pr>, <pos>n. pl.</pos> <ety>[L., a griping in the belly.]</ety> <fld>(Med.)</fld> <def>acute, colicky pains; gripes.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Torminous</ent><br/
<hw>Tor"mi*nous</hw> <pr>(?)</pr>, <pos>a.</pos> <fld>(Med.)</fld> <def>Affected with tormina; griping.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Torn</ent><br/
<hw>Torn</hw> <pr>(?)</pr>, <def><pos>p. p.</pos> of <er>Tear</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tornado</ent><br/
<hw>Tor*na"do</hw> <pr>(?)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Tornadoes</plw> <pr>(#)</pr>.</plu> <ety>[From Sp. or Pg. <ets>tornar</ets> to turn, return, L. <ets>tornare</ets> to turn, hence, a whirling wind. The Sp. & Pg. <ets>tornada</ets> is a return. See <er>Turn</er>.]</ety> <def>A violent whirling wind; specifically <fld>(Meteorol.)</fld>, a tempest distinguished by a rapid whirling and slow progressive motion, usually accompaned with severe thunder, lightning, and torrents of rain, and commonly of short duration and small breadth; a small cyclone<-- twister -->.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tornaria</ent><br/
||<hw>Tor*na"ri*a</hw> <pr>(?)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Tornariae</plw>  <pr>(#)</pr>.</plu> <ety>[NL., fr. L. <ets>tornare</ets> to turn.]</ety> <fld>(Zool.)</fld> <def>The peculiar free swimming larva of Balanoglossus. See <xex>Illust.</xex> in Append.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Torose</ent><br/
<hw>To*rose"</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>torosus</ets> full of muscle, brawny, fleshy. See <er>Torus</er>.]</ety> <def>Cylindrical with alternate swellings and contractions; having the surface covered with rounded prominences.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Torosity</ent><br/
<hw>To*ros"i*ty</hw> <pr>(?)</pr>, <pos>n.</pos> <def>The quality or state of being torose.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Torous</ent><br/
<hw>Torous</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Torose.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Torpedinous</ent><br/
<hw>Tor*ped"i*nous</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Of or pertaining to a torpedo; resembling a torpedo; exerting a benumbing influence; stupefying; dull; torpid.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Fishy were his eyes; <qex>torpedinous</qex> was his manner.</q> <rj><qau>De Quincey.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Torpedo</ent><br/
<hw>Tor*pe"do</hw> <pr>(?)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Torpedoes</plw> <pr>(#)</pr>.</plu> <ety>[L. <ets>torpedo</ets>, <ets>-inis</ets>, from <ets>torpere</ets> to be stiff, numb, or torpid. See <er>Torpid</er>.]</ety><br/
[<source>1913 Webster</source>]</p>

<p><sn>1.</sn> <fld>(Zool.)</fld> <def>Any one of numerous species of elasmobranch fishes belonging to <gen>Torpedo</gen> and allied genera. They are related to the rays, but have the power of giving electrical shocks. Called also <altname>crampfish</altname>, and <altname>numbfish</altname>. See <cref>Electrical fish</cref>, under <er>Electrical</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ The common European torpedo (<spn>Torpedo vulgaris</spn>) and the American species (<spn>Torpedo occidentalis</spn>) are the best known.</note><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>An engine or machine for destroying ships by blowing them up; a mine{4}.</def> Specifically: --<br/
[<source>1913 Webster</source> <source>+PJC</source>]</p>

<p><sd>(a)</sd> <def>A quantity of explosives anchored in a channel, beneath the water, or set adrift in a current, and so designed that they will explode when touched or approached by a vessel, or when an electric circuit is closed by an operator on shore; now called <altname>marine mine</altname>.</def>  <mark>[obsolete]</mark><br/
[<source>1913 Webster</source> <source>+PJC</source>]</p>

<p><q>Damn the <qex>torpedoes</qex> -- full speed ahead!</q> <rj><qau>Adm. David Glasgow Farragut (At the battle of Mobile Bay, 1864).</qau></rj></p>

<p><sd>(b)</sd> <def>A kind of small submarine boat carrying an explosive charge, and projected from a ship against another ship at a distance, or made self-propelling, and otherwise automatic in its action against a distant ship.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <fld>(Mil.)</fld> <def>A kind of shell or cartridge buried in earth, to be exploded by electricity or by stepping on it; now called <altname>land mine</altname>.</def> <mark>[obsolete]</mark><br/
[<source>1913 Webster</source> <source>+PJC</source>]</p>

<p><sn>4.</sn> <fld>(Railroad)</fld> <def>A kind of detonating cartridge or shell placed on a rail, and exploded when crushed under the locomotive wheels, -- used as an alarm signal.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <def>An explosive cartridge or shell lowered or dropped into a bored oil well, and there exploded, to clear the well of obstructions or to open communication with a source of supply of oil.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>6.</sn> <def>A kind of firework in the form of a small ball, or pellet, which explodes when thrown upon a hard object.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>7.</sn> <def>An automobile with a <er>torpedo body</er>.</def> <mark>[Archaic Cant]</mark><br/
[<source>Webster 1913 Suppl.</source> <source>+PJC</source>]</p>

<p><cs><col><b>Fish torpedo</b></col>, <cd>a spindle-shaped, or fish-shaped, self-propelling submarine torpedo.</cd> -- <col><b>Spar torpedo</b></col>, <cd>a canister or other vessel containing an explosive charge, and attached to the end of a long spar which projects from a ship or boat and is thrust against an enemy's ship, exploding the torpedo.</cd> -- <col><b>Torpedo boat</b></col>, <cd>a vessel adapted for carrying, launching, operating, or otherwise making use of, torpedoes against an enemy's ship., especially, a small, fast boat with tubes for launching torpedoes.</cd> -- <col><b>Torpedo nettings</b></col>, <cd>nettings made of chains or bars, which can be suspended around a vessel and allowed to sink beneath the surface of the water, as a protection against torpedoes.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Torpedo</ent><br/
<hw>Tor*pe"do</hw>, <pos>v. t.</pos> <sn>1.</sn> <def>to destroy by, or subject to the action of, a torpedo.</def>  <rj><au>London Spectator.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <mark>[Fig.]</mark> <def>To destroy, cause to halt, or prevent from being accomplished; -- used esp. with reference to a plan or an enterprise, halted by some action before the plan is put into execution.</def><br/
[<source>PJC</source>]</p>

<p><ent>Torpedo-boat destroyer</ent><br/
<hw>Tor*pe"do-boat` de*stroy"er</hw>. <def>A larger, swifter, and more powerful armed type of torpedo boat, originally intended principally for the destruction of torpedo boats, but later used also as a more formidable torpedo boat.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Torpedo body</ent><br/
<hw>Tor*pe"do body</hw>. <def>An automobile body which is built so that the side surfaces are flush.</def> <mark>[Cant]</mark><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Torpedo boom</ent><br/
<hw>Tor*pe"do boom</hw>. <def>A spar formerly carried by men-of-war, having a torpedo on its end.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Torpedo catcher</ent><br/
<hw>Tor*pe"do catch"er</hw>. <def>A small fast vessel for pursuing and destroying torpedo boats.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Torpedoist</ent><br/
<hw>Tor*pe"do*ist</hw>, <pos>n.</pos> <fld>(Nav.)</fld> <def>One skilled in the theory or use of torpedoes; also, one who favors the use of torpedoes.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Torpedo shell</ent><br/
<hw>Torpedo shell</hw>. <fld>(Ordnance)</fld> <def>A shell longer than a deck-piercing shell, with thinner walls and a larger cavity for the bursting charge, which consists of about 130 pounds of high explosive. It has no soft cap, and is intended to effect its damage by the powerful explosion which follows on slight resistance. It is used chiefly in 12-inch mortars.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Torpedo station</ent><br/
<hw>Torpedo station</hw>. <def>A headquarters for torpedo vessels and their supplies, usually having facilities for repairs and for instruction and experiments. The principal torpedo station of the United States is at Newport, R.I.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Torpedo stern</ent><br/
<hw>Torpedo stern</hw>. <def>A broad stern without overhang, flattened on the bottom, used in some torpedo and fast power boats. It prevents settling in the water at high speed.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Torpedo tube</ent><br/
<hw>Torpedo tube</hw>. <fld>(Nav.)</fld> <def>A tube fixed below or near the water line through which a torpedo is fired, usually by a small charge of gunpowder. On torpedo vessels the tubes are on deck and usually in broadside, on larger vessels usually submerged in broadside and fitted with a movable shield which is pushed out from the vessel's side to protect the torpedo until clear, but formerly sometimes in the bow. In submarine torpedo boats they are in the bow.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Torpent</ent><br/
<hw>Tor"pent</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>torpens</ets>, p. pr. of <ets>torpere</ets> to be numb.]</ety> <def>Having no motion or activity; incapable of motion; benumbed; torpid.</def> <mark>[Obs.]</mark>  <rj><au>Evelyn.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Torpescence</ent><br/
<hw>Tor*pes"cence</hw> <pr>(?)</pr>, <pos>n.</pos> <def>The quality or state or being torpescent; torpidness; numbness; stupidity.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Torpescent</ent><br/
<hw>Tor*pes"cent</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>torpescens</ets>, p. pr. of <ets>torpescere</ets> to grow stiff, numb, or torpid, incho. fr. <ets>torpere</ets>. See <er>Torpid</er>.]</ety> <def>Becoming torpid or numb.</def>  <rj><au>Shenstone.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Torpid</ent><br/
<hw>Tor"pid</hw> <pr>(t<ocir/r"p<icr/d)</pr>, <pos>a.</pos> <ety>[L. <ets>torpidus</ets>, fr. <ets>torpere</ets> to be stiff, numb, or torpid; of uncertain origin.]</ety> <sn>1.</sn> <def>Having lost motion, or the power of exertion and feeling; numb; benumbed; <as>as, a <ex>torpid</ex> limb</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Without heat all things would be <qex>torpid</qex>.</q> <rj><qau>Ray.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Dull; stupid; sluggish; inactive.</def>  <rj><au>Sir M. Hale.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Torpid</ent><br/
<hw>Tor"pid</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[See <er>Torpid</er>, <pos>a.</pos>]</ety> <mark>[Slang, Oxford University, Eng.]</mark> <sn>1.</sn> <def>An inferior racing boat, or one who rows in such a boat.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><sn>2.</sn>  <pluf>pl.</pluf> <def>The Lenten rowing races.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Torpidity</ent><br/
<hw>Tor*pid"i*ty</hw> <pr>(?)</pr>, <pos>n.</pos> <def>Same as <er>Torpidness</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Torpidly</ent><br/
<hw>Tor"pid*ly</hw> <pr>(?)</pr>, <pos>adv.</pos> <def>In a torpid manner.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Torpidness</ent><br/
<hw>Tor"pid*ness</hw>, <pos>n.</pos> <def>The qualityy or state of being torpid.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Torpify</ent><br/
<hw>Tor"pi*fy</hw> <pr>(?)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Torpified</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Torpifying</conjf>. <pr>(<?/)</pr>.]</vmorph> <ety>[L. <ets>torpere</ets> to be torpid + <ets>-fy</ets>.]</ety> <def>To make torpid; to numb, or benumb.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Torpitude</ent><br/
<hw>Tor"pi*tude</hw> <pr>(?)</pr>, <pos>n.</pos> <def>Torpidness.</def> <mark>[Obs.]</mark> <ldquo/In a kind of <xex>torpitude</xex>, or sleeping state.<rdquo/  <rj><au>Derham.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Torpor</ent><br/
<hw>Tor"por</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L., from <ets>torpere</ets>, to be torpid.]</ety><br/
[<source>1913 Webster</source>]</p>

<p><sn>1.</sn> <def>Loss of motion, or of the motion; a state of inactivity with partial or total insensibility; numbness.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Dullness; sluggishness; inactivity; <as>as, a <ex>torpor</ex> of the mental faculties</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Torporific</ent><br/
<hw>Tor`por*if"ic</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>torpor</ets> torpor + <ets>facere</ets> to make.]</ety> <def>Tending to produce torpor.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Torquate</ent><br/
<hw>Tor"quate</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>torquatus</ets> wearing a collar.]</ety> <fld>(Zool.)</fld> <def>Collared; having a torques, or distinct colored ring around the neck.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>torquated</ent><br/
<hw>tor"qua*ted</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>Torqyatus</ets>.]</ety> <def>Having or wearing a torque, or neck chain.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Torque</ent><br/
<hw>Torque</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>torques</ets> a twisted neck chain, fr. <ets>torquere</ets> to twist.]</ety> <sn>1.</sn> <def>A collar or neck chain, usually twisted, especially as worn by ancient barbaric nations, as the Gauls, Germans, and Britons.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <ety>[L. <ets>torquere</ets> to twist.]</ety> <fld>(Mech.)</fld> <def>That which tends to produce torsion; a couple of forces.</def>  <rj><au>J. Thomson.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <fld>(Phys. Science)</fld> <def>A turning or twisting; tendency to turn, or cause to turn, about an axis.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Torqued</ent><br/
<hw>Torqued</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>torquere</ets> to twist, to turn, to wind.]</ety> <sn>1.</sn> <def>Wreathed; twisted.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Her.)</fld> <def>Twisted; bent; -- said of a dolphin haurient, which forms a figure like the letter S.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Torques</ent><br/
||<hw>Tor"ques</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L., a necklace. See <er>Torque</er>, 1.]</ety> <fld>(Zool.)</fld> <def>A cervical ring of hair or feathers, distinguished by its color or structure; a collar.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Torrefaction</ent><br/
<hw>Tor`re*fac"tion</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>torrefacere</ets>,<ets>torrefactum</ets>, to torrefy: cf. F. <ets>torr</ets><eacute/<ets>faction</ets>. See <er>Torrefy</er>.]</ety> <def>The act or process of torrefying, or the state of being torrefied.</def>  <rj><au>Bp. Hall.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Torrefy</ent><br/
<hw>Tor"re*fy</hw> <pr>(?)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Torrefied</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Torrefying</conjf>.]</vmorph> <ety>[L. <ets>torrere</ets> to parch + <ets>-fy</ets>: cf. F. <ets>torr</ets><eacute/<ets>fier</ets>, L. <ets>torrefacere</ets>.]</ety> <altsp>[Written also <asp>torrify</asp>.]</altsp> <sn>1.</sn> <def>To dry by a fire.</def>  <rj><au>Sir T. Browne.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Metal.)</fld> <def>To subject to scorching heat, so as to drive off volatile ingredients; to roast, as ores.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <fld>(Pharm.)</fld> <def>To dry or parch, as drugs, on a metallic plate till they are friable, or are reduced to the state desired.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Torrens system</ent><br/
<hw>Tor"rens sys`tem</hw> <pr>(?)</pr>. <def>A system of registration of titles to land (as distinct from registration of deeds) introduced into South Australia by the Real Property (or Torrens) Act (act 15 of 1857-58), drafted by Sir Robert Torrens (1814-84). Its essential feature is the guaranty by the government of properly registered titles. The system has been generally adopted in Australia and British Columbia, and in its original or a modified form in some other countries, including some States of the United States. Hence <col><b>Torrens title</b></col>, etc.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Torrent</ent><br/
<hw>Tor"rent</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F., fr. L. <ets>torrens</ets>, <ets>-entis</ets>, fr. <ets>torrens</ets> burning, roaring, boiling, p. pr. of <ets>torrere</ets> to dry by heat, to burn. See <er>Torrid</er>.]</ety> <sn>1.</sn> <def>A violent stream, as of water, lava, or the like; a stream suddenly raised and running rapidly, as down a precipice.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The roaring <qex>torrent</qex> is deep and wide.</q> <rj><qau>Longfellow.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Fig.: A violent or rapid flow; a strong current; a flood; <as>as, a <ex>torrent</ex> of vices; a <ex>torrent</ex> of eloquence</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>At length, Erasmus, that great injured name, . . .<br/
Stemmed the wild <qex>torrent</qex> of a barbarous age.</q> <rj><qau>Pope.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Torrent</ent><br/
<hw>Tor"rent</hw>, <pos>a.</pos> <ety>[See <er>Torrent</er>, <pos>n.</pos>]</ety> <def>Rolling or rushing in a rapid stream.</def> <ldquo/Waves of <xex>torrent</xex> fire.<rdquo/  <rj><au>Milton.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Torrentine</ent><br/
<ent>Torrential</ent><br/
<mhw>{ <hw>Tor*ren"tial</hw> <pr>(?)</pr>, <hw>Tor*ren"tine</hw> <pr>(?)</pr>, }</mhw> <pos>a.</pos> <def>Of or pertaining to a torrent; having the character of a torrent; caused by a torrent .</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Torricellian</ent><br/
<hw>Tor`ri*cel"li*an</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Of or pertaining to <xex>Torricelli</xex>, an Italian philosopher and mathematician, who, in 1643, discovered that the rise of a liquid in a tube, as in the barometer, is due to atmospheric pressure. See <er>Barometer</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Torricellian tube</b></col>, <cd>a glass tube thirty or more inches in length, open at the lower end and hermetically sealed at the upper, such as is used in the barometer.</cd> -- <col><b>Torricellian vacuum</b></col> <fld>(Physics)</fld>, <cd>a vacuum produced by filling with a fluid, as mercury, a tube hermetically closed at one end, and, after immersing the other end in a vessel of the same fluid, allowing the inclosed fluid to descend till it is counterbalanced by the pressure of the atmosphere, as in the barometer.</cd>  <rj><au>Hutton.</au></rj></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Torrid</ent><br/
<hw>Tor"rid</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>torridus</ets>, fr. <ets>torrere</ets> to parch, to burn, akin to E. <ets>Thist</ets>: cf. F. <ets>torride</ets>. See <er>Thirst</er>.]</ety> <sn>1.</sn> <def>Parched; dried with heat; <as>as, a <ex>torrid</ex> plain or desert</as>.</def> <ldquo/Barca or Cyrene's <xex>torrid</xex> soil.<rdquo/  <rj><au>Milton.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Violenty hot; drying or scorching with heat; burning; parching.</def> <ldquo/<xex>Torrid</xex> heat.<rdquo/  <rj><au>Milton.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Torrid zone</b></col> <fld>(Geog.)</fld>, <cd>that space or board belt of the earth, included between the tropics, over which the sun is vertical at some period of every year, and the heat is always great.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Torridity</ent><br/
<hw>Tor*rid"i*ty</hw> <pr>(?)</pr>, <pos>n.</pos> <def>Torridness.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Torridness</ent><br/
<hw>Tor"rid*ness</hw> <pr>(?)</pr>, <pos>n.</pos> <def>The quality or state of being torrid or parched.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Torril</ent><br/
<hw>Tor"ril</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A worthless woman; also, a worthless horse.</def> <mark>[Prov. Eng.]</mark>  <rj><au>Halliwell.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Torrock</ent><br/
<hw>Tor"rock</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Zool.)</fld> <def>A gull.</def> <mark>[Prov. Eng.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Torsade</ent><br/
<hw>Tor*sade"</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F.]</ety> <def>A twisted cord; also, a molded or worked ornament of similar form.</def></p>

<p><q>The crown decked with <qex>torsades</qex> of pearls.</q>  <rj><qau>Harper's Mag.</qau></rj><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Torsal</ent><br/
<hw>Tor"sal</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Carp.)</fld> <def>A torsel.</def>  <rj><au>Knight.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Torse</ent><br/
<hw>Torse</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[OF., fr. OF. & F. <ets>tors</ets>, <ets>torse</ets>, twisted, wreathed, p. p. of <ets>tordre</ets> to twist, L. <ets>torquere</ets>. See <er>Torture</er>.]</ety><br/
[<source>1913 Webster</source>]</p>

<p><sn>1.</sn> <fld>(Her.)</fld> <def>A wreath.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <ety>[F. <ets>tors</ets>, <ets>torse</ets>, twisted.]</ety> <fld>(Geom.)</fld> <def>A developable surface. See under <er>Developable</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Torsel</ent><br/
<hw>Tor"sel</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Carp.)</fld> <def>A plate of timber for the end of a beam or joist to rest on.</def>  <rj><au>Gwilt</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Torsibillty</ent><br/
<hw>Tor`si*bil"l*ty</hw> <pr>(?)</pr>, <pos>n.</pos> <def>The tendency, as of a rope, to untwist after being twisted.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Torsion</ent><br/
<hw>Tor"sion</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F., fr. LL. <ets>torsio</ets>, fr. L. <ets>torquere</ets>, <ets>tortum</ets>, to twist. See <er>Torture</er>.]</ety> <sn>1.</sn> <def>The act of turning or twisting, or the state of being twisted; the twisting or wrenching of a body by the exertion of a lateral force tending to turn one end or part of it about a longitudinal axis, while the other is held fast or turned in the opposite direction.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Mech.)</fld> <def>That force with which a thread, wire, or rod of any material, returns, or tends to return, to a state of rest after it has been twisted; torsibility.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Angle of torsion</b></col><cd> (of a curve) <fld>(Geom.)</fld>, the indefinitely small angle between two consecutive osculating planes of a curve of double curvature.</cd> -- <col><b>Moment of torsion</b></col> <fld>(Mech.)</fld> <cd>the moment of a pair of equal and opposite couples which tend to twist a body.</cd> -- <col><b>Torsion balance</b></col> <fld>(Physics.)</fld>, <cd>an instrument for estimating very minute forces, as electric or magnetic attractions and repulsions, by the torsion of a very slender wire or fiber having at its lower extremity a horizontal bar or needle, upon which the forces act.</cd> -- <col><b>Torsion scale</b></col>, <cd>a scale for weighing in which the fulcra of the levers or beams are strained wires or strips acting by torsion.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Torsional</ent><br/
<hw>Tor"sion*al</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Of or pertaining to torsion; resulting from torsion, or the force with which a thread or wire returns to a state of rest after having been twisted round its axis; <as>as, <ex>torsional</ex> force</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Torsion electrometer</ent><br/
<hw>Tor"sion e*lec*trom"e*ter</hw>. <fld>(Elec.)</fld> <def>A torsion balance used for measuring electric attraction or repulsion.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Torsion galvanometer</ent><br/
<hw>Torsion galvanometer</hw>. <fld>(Elec.)</fld> <def>A galvanometer in which current is measured by torsion.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Torsion head</ent><br/
<hw>Torsion head</hw>. <def>That part of a torsion balance from  which the wire or filament is suspended.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Torsion indicator</ent><br/
<hw>Torsion indicator</hw>. <def>An autographic torsion meter.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Torsion meter</ent><br/
<hw>Torsion meter</hw>. <fld>(Mech.)</fld> <def>An instrument for determining the torque on a shaft, and hence the horse power of an engine, esp. of a marine engine of high power, by measuring the amount of twist of a given length of the shaft. Called also <altname>torsimeter</altname>, <altname>torsiometer</altname>, <altname>torsometer</altname>.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Torsk</ent><br/
<hw>Torsk</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Dan.; akin to Icel. <ets><thorn/orskr</ets> a codfish, G. <ets>dorsch</ets>.]</ety> <fld>(Zool.)</fld> <sd>(a)</sd> <def>The cusk. See <er>Cusk</er>.</def> <sd>(b)</sd> <def>The codfish. Called also <altname>tusk</altname>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Torso</ent><br/
<hw>Tor"so</hw> <pr>(?)</pr>, <pos>n.</pos>; <plu><it>pl.</it> E. <plw>Torsos</plw> <pr>(#)</pr>, It. <plw>Torsi</plw> <pr>(#)</pr>.</plu> <ety>[It. <ets>torso</ets>, probably fr. L. <ets>thyrsus</ets> a stalk, stem, thyrsus, Gr. <?/; cf. OHG. <ets>torso</ets>, <ets>turso</ets>, a stalk, stem, G. <ets>dorsche</ets> a cabbage stalk.  Cf. <er>Thyrsus</er>, <er>Truss</er>.]</ety> <def>The human body, as distinguished from the head and limbs; in sculpture, the trunk of a statue, mutilated of head and limbs; <as>as, the <ex>torso</ex> of Hercules</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tort</ent><br/
<hw>Tort</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F., from LL. <ets>tortum</ets>, fr. L. <ets>tortus</ets> twisted, crooked, p. p. of <ets>torqure</ets> to twist, bend. See <er>Torture</er>.]</ety> <sn>1.</sn> <def>Mischief; injury; calamity.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>That had them long opprest with <qex>tort</qex>.</q> <rj><qau>Spenser.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Law)</fld> <def>Any civil wrong or injury; a wrongful act (not involving a breach of contract) for which an action will lie; a form of action, in some parts of the United States, for a wrong or injury.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs>||<col><b>Executor de son tort</b></col>.  <cd>See under <er>Executor</er>.</cd> -- <col><b>Tort feasor</b></col> <fld>(Law)</fld>, <cd>a wrongdoer; a trespasser.</cd>  <rj><au>Wharton.</au></rj></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tort</ent><br/
<hw>Tort</hw>, <pos>a.</pos> <def>Stretched tight; taut.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>Yet holds he them with <qex>tortest</qex> rein.</q> <rj><qau>Emerson.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Torta</ent><br/
<hw>Tor"ta</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Cf. Sp. <ets>torta</ets> a cake.]</ety> <fld>(Metal.)</fld> <def>a flat heap of moist, crushed silver ore, prepared for the patio process.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Torteau</ent><br/
<hw>Tor"teau</hw> <pr>(?)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Torteaus</plw> <pr>(#)</pr>.</plu> <ety>[Of. <ets>torteau</ets>, <ets>tortel</ets>, from L. <ets>tortus</ets> twisted. See <er>Tort</er>.]</ety> <fld>(Her.)</fld> <def>A roundel of a red color.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Torticollis</ent><br/
<hw>Tor`ti*col"lis</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F. <ets>toricolis</ets>; L. <ets>torquere</ets>, <ets>tortum</ets>, to twist + <ets>collum</ets> the neck.]</ety> <fld>(Med.)</fld> <def>See <er>Wryneck</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tortile</ent><br/
<hw>Tor"tile</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>tortilis</ets>, fr. <ets>torquere</ets>, <ets>tortum</ets>, to twist: cf. F. <ets>tortile</ets>.]</ety> <def>Twisted; wreathed; coiled.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tortility</ent><br/
<hw>Tor*til"i*ty</hw> <pr>(?)</pr>, <pos>n.</pos> <def>The quality or state of being tortile, twisted, or wreathed.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tortilla</ent><br/
||<hw>Tor*til"la</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Sp.]</ety> <def>An unleavened cake, as of maize flour, baked on a heated iron or stone.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tortion</ent><br/
<hw>Tor"tion</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[LL. <ets>tortio</ets>. See <er>Torsion</er>.]</ety> <def>Torment; pain.</def> <mark>[Obs.]</mark>  <rj><au>Bacon.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tortious</ent><br/
<hw>Tor"tious</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[From <er>Tort</er>.]</ety> <sn>1.</sn> <def>Injurious; wrongful.</def> <mark>[Obs.]</mark> <ldquo/<xex>Tortious</xex> power.<rdquo/  <rj><au>Spenser.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Law)</fld> <def>Imploying tort, or privat injury for which the law gives damages; involing tort.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tortiously</ent><br/
<hw>Tor"tious*ly</hw>, <pos>adv.</pos> <fld>(Law)</fld> <def>In a tortous manner.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tortive</ent><br/
<hw>Tor"tive</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>tortus</ets>, p. p. of <ets>torquere</ets> to twist, wind.]</ety> <def>Twisted; wreathed.</def>  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tortoise</ent><br/
<hw>Tor"toise</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[OE. <ets>tortuce</ets>, fr. OF. <ets>tortis</ets> crooked, fr. L. <ets>tortus</ets> twisted, crooked, contorted, p. p. of <ets>torquere</ets>, <ets>tortum</ets>, to wind; cf. F. <ets>tortue</ets> tortoise, LL. <ets>tortuca</ets>, <ets>tartuca</ets>, Pr. <ets>tortesa</ets> crookedness, <ets>tortis</ets> crooked. so called in allusion to its crooked feet. See <er>Torture</er>.]</ety> <sn>1.</sn> <fld>(Zool.)</fld> <def>Any one of numerous species of reptiles of the order <ord>Testudinata</ord>.</def><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ The term is applied especially to the land and fresh-water species, while the marine species are generally called <xex>turtles</xex>, but the terms <xex>tortoise</xex> and <xex>turtle</xex> are used synonymously by many writers.  See <er>Testudinata</er>, <er>Terrapin</er>, and <er>Turtle</er>.</note><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Rom. Antiq.)</fld> <def>Same as <er>Testudo</er>, 2.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><mcol><col><b>Box tortoise</b></col>, <col><b>Land tortoise</b></col>, etc.</mcol> <cd>See under <er>Box</er>, <er>Land</er>, etc.</cd> -- <col><b>Painted tortoise</b></col>. <fld>(Zool.)</fld> <cd>See <cref>Painted turtle</cref>, under <er>Painted</er>.</cd> -- <col><b>Soft-shell tortoise</b></col>. <fld>(Zool.)</fld> <cd>See <er>Trionyx</er>.</cd> -- <col><b>Spotted tortoise</b></col>. <fld>(Zool.)</fld> <cd>A small American fresh-water tortoise (<spn>Chelopus guttatus</spn> or <spn>Nanemys guttatus</spn>) having a blackish carapace on which are scattered round yellow spots.</cd> -- <col><b>Tortoise beetle</b></col> <fld>(Zool.)</fld>, <cd>any one of numerous species of small tortoise-shaped beetles.  Many of them have a brilliant metallic luster.  The larvae feed upon the leaves of various plants, and protect themselves beneath a mass of dried excrement held over the back by means of the caudal spines.  The golden tortoise beetle (<spn>Cassida aurichalcea</spn>) is found on the morning-glory vine and allied plants.</cd> -- <col><b>Tortoise plant</b></col>. <fld>(Bot.)</fld> <cd>See <cref>Elephant's foot</cref>, under <er>Elephant</er>.</cd> -- <col><b>Tortoise shell</b></col>, <cd>the substance of the shell or horny plates of several species of sea turtles, especially of the hawkbill turtle. It is used in inlaying and in the manufacture of various ornamental articles.</cd> -- <col><b>Tortoise-shell butterfly</b></col> <fld>(Zool.)</fld>, <cd>any one of several species of handsomely colored butterflies of the genus <gen>Aglais</gen>, as <spn>Aglais Milberti</spn>, and <spn>Aglais urticae</spn>, both of which, in the larva state, feed upon nettles.</cd> -- <col><b>Tortoise-shell turtle</b></col> <fld>(Zool.)</fld>, <cd>the hawkbill turtle. See <er>Hawkbill</er>.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>tortoise-shell</ent><br/
<hw>tortoise-shell</hw>, <pos>adj.</pos> <def>Having a color like that of a tortoise's shell, black with white and orange spots; --  used mostly to describe cats of that color.</def> <def2><pos>n.</pos> <def>a tortoise-shell cat.</def></def2></p>

<p><ent>Tortricid</ent><br/
<hw>Tor"tri*cid</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[See <er>Tortrix</er>.]</ety> <fld>(Zool.)</fld> <def>Of or pertaining to Tortix, or the family <fam>Tortricidae</fam>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tortrix</ent><br/
||<hw>Tor"trix</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[NL., fr. L. <ets>torquere</ets>, <ets>tortum</ets>, to twist.]</ety> <sn>1.</sn> <fld>(Zool.)</fld> <def>Any one of numerous species of small moths of the family <fam>Tortricidae</fam>, the larvae of which usually roll up the leaves of plants on which they live; -- also called <altname>leaf roller</altname>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Zool.)</fld> <def>A genus of tropical short-tailed snakes, which are not venomous.  One species (<spn>Tortrix scytalae</spn>) is handsomely banded with black, and is sometimes worn alive by the natives of Brazil for a necklace.</def><br/
[<source>1913 Webster</source>]</p>

<p><-- p. 1521 --><br/</p>

<p><ent>Tortulous</ent><br/
<hw>Tor"tu*lous</hw> <pr>(t<ocir/r"t<usl/*l<ucr/s; 135)</pr>, <pos>a.</pos> <fld>(Nat. Hist.)</fld> <def>Swelled out at intervals like a knotted cord.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tortuose</ent><br/
<hw>Tor"tu*ose`</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[See Tortuous.]</ety> <def>Wreathed; twisted; winding.</def>  <rj><au>Loudon</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tortuoslty</ent><br/
<hw>Tor`tu*os"l*ty</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>tortuositas</ets>: cf. F. <ets>tortuosite</ets>.]</ety> <def>the quality or state of being tortuous.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tortuous</ent><br/
<hw>Tor"tu*ous</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[OE. <ets>tortuos</ets>, L. <ets>tortuosus</ets>, fr. <ets>tortus</ets> a twisting, winding, fr. <ets>torquere</ets>, <ets>tortum</ets>, to twist: cf. F. <ets>tortueux</ets>. See <er>Torture</er>.]</ety> <sn>1.</sn> <def>Bent in different directions; wreathed; twisted; winding; <as>as, a <ex>tortuous</ex> train; a <ex>tortuous</ex> leaf or corolla</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The badger made his dark and <qex>tortuous</qex> hole on the side of every hill where the copsewood grew thick.</q> <rj><qau>Macaulay.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Fig.: Deviating from rectitude; indirect; erroneous; deceitful.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>That course became somewhat less<qex>tortuous</qex>, when the battle of the Boyne had cowed the spirit of the Jakobites.</q> <rj><qau>Macaulay.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>Injurious: tortious.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <fld>(Astrol.)</fld> <def>Oblique; -- applied to the six signs of the zodiac (from Capricorn to Gemini) which ascend most rapidly and obliquely.</def> <mark>[Obs.]</mark>  <rj><au>Skeat.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Infortunate ascendent <qex>tortuous</qex>.</q> <rj><qau>Chaucer.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p>--<wordforms><wf>Tor"tu*ous*ly</wf>, <pos>adv.</pos> -- <wf>Tor"tu*ous*ness</wf>, <pos>n.</pos></wordforms><br/
[<source>1913 Webster</source>]</p>

<p><ent>Torturable</ent><br/
<hw>Tor"tur*a*ble</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Capable of being tortured.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Torture</ent><br/
<hw>Tor"ture</hw> <pr>(t<ocir/r"t<usl/r; 135)</pr>, <pos>n.</pos> <ety>[F., fr. L. <ets>tortura</ets>, fr. <ets>torquere</ets>, <ets>tortum</ets>, to twist, rack, torture; probably akin to Gr. <grk>tre`pein</grk> to turn, G. <ets>drechseln</ets> to turn on a lathe, and perhaps to E. <ets>queer</ets>.  Cf. <er>Contort</er>, <er>Distort</er>, <er>Extort</er>, <er>Retort</er>, <er>Tart</er>, <pos>n.</pos>, <er>Torch</er>, <er>Torment</er>, <er>Tortion</er>, <er>Tort</er>, <er>Trope.</er>]</ety> <sn>1.</sn> <def>Extreme pain; anguish of body or mind; pang; agony; torment; <as>as, <ex>torture</ex> of mind</as>.</def>  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Ghastly spasm or racking <qex>torture</qex>.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Especially, severe pain inflicted judicially, either as punishment for a crime, or for the purpose of extorting a confession from an accused person, as by water or fire, by the boot or thumbkin, or by the rack or wheel.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>The act or process of torturing.</def><br/
[<source>1913 Webster</source>]</p>

<p><q><qex>Torture</qex>, which had always been deciared illegal, and which had recently been declared illegal even by the servile judges of that age, was inflicted for the last time in England in the month of May, 1640.</q> <rj><qau>Macaulay.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>torture</ent><br/
<hw>tor"ture</hw>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>tortured</conjf> <pr>(t<ocir/r"t<usl/rd; 135)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>tTorturing.</conjf>]</vmorph> <ety>[Cf. F. <ets>Torturer</ets>. ]</ety> <sn>1.</sn> <def>To put to torture; to pain extremely; to harass; to vex.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To punish with torture; to put to the rack; <as>as, to <ex>torture</ex> an accused person</as>.</def>  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>To wrest from the proper meaning; to distort.</def>  <rj><au>Jar. Taylor.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>To keep on the stretch, as a bow.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>The bow <qex>tortureth</qex> the string.</q> <rj><qau>Bacon.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Torturer</ent><br/
<hw>Tor"tur*er</hw> <pr>(t<ocir/r"t<usl/r*<etil/r)</pr>, <pos>n.</pos> <def>One who tortures; a tormentor.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Torturingly</ent><br/
<hw>Tor"tur*ing*ly</hw>, <pos>adv.</pos> <def>So as to torture.</def>  <rj><au>Beau. & Fl.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Torturous</ent><br/
<hw>Tor"tur*ous</hw> <pr>(t<ocir/r"t<usl/r*<ucr/s)</pr>, <pos>a.</pos> <def>Involving, or pertaining to, torture.</def> <mark>[R.]</mark> <ldquo/<xex>The torturous crucifixion</xex>.<rdquo/  <rj><au>I. Disraeli.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Torula</ent><br/
||<hw>Tor"u*la</hw> <pr>(?)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Torulae</plw> <pr>(<?/)</pr></plu> <ety>[NL., dim. of L. <ets>torus</ets> a semicircular molding.]</ety> <fld>(Biol.)</fld> <sd>(a)</sd> <def>A chain of special bacteria.</def> <sd>(b)</sd> <def>A genus of budding fungi.  Same as <er>Saccharomyces</er>.  Also used adjectively.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Torulaform</ent><br/
<hw>Tor"u*la*form`</hw> <pr>(?)</pr>, <pos>a.</pos> <fld>(Biol.)</fld> <def>Having the appearance of a torula; in the form of a little chain; <as>as, a <ex>torulaform</ex> string of micrococci</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Torulose</ent><br/
<hw>Tor"u*lose</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>torulus</ets>, dim. of <ets>torus</ets>: cf. F. <ets>toruleux</ets>. See <er>Torus</er>]</ety> <fld>(Bot.)</fld> <def>Same as <er>Torose.</er></def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Torulous</ent><br/
<hw>Tor"u*lous</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Same as <er>Torose.</er></def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Torus</ent><br/
||<hw>To"rus</hw> <pr>(t<omac/"r<ucr/s)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Tori</plw> <pr>(t<omac/"r<imac/)</pr>.</plu> <ety>[L., a round, swelling, or bulging place, an elevation.  Cf. 3d <er>Tore</er>.]</ety><br/
[<source>1913 Webster</source>]</p>

<p><sn>1.</sn> <fld>(Arch.)</fld> <def>A large molding used in the bases of columns.  Its profile is semicircular. See <xex>Illust.</xex> of <er>Molding.</er></def>  <rj><au>Brande & C.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Zool.)</fld> <def>One of the ventral parapodia of tubicolous annelids. It usually has the form of an oblong thickening or elevation of the integument with rows of uncini or hooks along the center. See <xex>Illust.</xex> under <er>Tubicolae</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <fld>(Bot.)</fld> <def>The receptacle, or part of the flower on which the carpels stand.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <fld>(Geom.)</fld> <sd>(a)</sd> <def>The surface described by the circumference of a circle revolving about a straight line in its own plane.</def> <sd>(b)</sd> <def>The solid inclosed by such a surface; -- sometimes called an <altname>anchor ring</altname>.</def><br/
<syn><b>Syn. --</b>3d Tore{2}.</syn><br/
[<source>1913 Webster</source>]</p>

<p><ent>Torved</ent><br/
<hw>Torved</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Stern; grim. See <er>Torvous.</er></def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>But yesterday his breath<br/
Awed Rome, and his least <qex>torved</qex> frown was death.</q> <rj><qau>J. Webster (1654).</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Torvity</ent><br/
<hw>Tor"vi*ty</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>torvitas</ets>. See <er>Torvous</er>.]</ety> <def>Sourness or severity of countenance; sternness.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Torvous</ent><br/
<hw>Tor"vous</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>torvus</ets>. ]</ety> <def>Sour of aspect; of a severe countenance; stern; grim.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>That <qex>torvous</qex>, sour look produced by anger.</q> <rj><qau>Derham.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tory</ent><br/
<hw>To"ry</hw> <pr>(?)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Tories</plw> <pr>(#)</pr>.</plu> <ety>[ Properly used of the Irish bogtrotters who robbed and plundered during the English civil wars, professing to be in sympathy with the royal cause; hence transferred to those who sought to maintain the extreme prerogatives of the crown; probably from Ir. <ets>toiridhe</ets>, <ets>tor</ets>, a pursuer; akin to Ir. & Gael. <ets>toir</ets> a pursuit.]</ety> <sn>1.</sn> <fld>(Eng. Politics)</fld> <def>A member of the conservative party, as opposed to the progressive party which was formerly called the Whig, and is now called the Liberal, party; an earnest supporter of existing royal and ecclesiastical authority.</def><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ The word <xex>Tory</xex> first occurs in English history in 1679, during the struggle in Parliament occasioned by the introduction of the bill for the exclusion of the duke of York from the line of succession, and was applied by the advocates of the bill to its opponents as a title of obloquy or contempt.  The Tories subsequently took a broader ground, and their leading principle became the maintenance of things as they were.  The name, however, has for several years ceased to designate an existing party, but is rather applied to certain traditional maxims of public policy.  The political successors of the Tories are now commonly known as <xex>Conservatives</xex>.</note>  <rj><au>New Am. Cyc.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Amer. Hist.)</fld> <def>One who, in the time of the Revolution, favored submitting to the claims of Great Britain against the colonies; an adherent to the crown.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tory</ent><br/
<hw>To"ry</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Of or pertaining to the Tories.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Toryism</ent><br/
<hw>To"ry*ism</hw> <pr>(?)</pr>, <pos>n.</pos> <def>The principles of the Tories.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Toscatter</ent><br/
<hw>To*scat"ter</hw> <pr>(?)</pr>, <pos>v. t.</pos> <ety>[Pref. <ets>to-</ets> + <ets>scatter</ets>.]</ety> <def>To scatter in pieces; to divide.</def> <mark>[Obs.]</mark>  <rj><au>Chaucer.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tose</ent><br/
<hw>Tose</hw> <pr>(?)</pr>, <pos>v. t.</pos> <ety>[ See <er>Touse</er> ]</ety> <def>To tease, or comb, as wool.</def> <mark>[Obs.or Prov. Eng.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tosh</ent><br/
<hw>Tosh</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[Cf. OF. <ets>tonce shorn</ets>, <ets>clipped</ets>, and E. <ets>tonsure</ets>.]</ety> <def>Neat; trim.</def> <mark>[Scot.]</mark>  <rj><au>Jomieson.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Toshred</ent><br/
<hw>To*shred"</hw> <pr>(?)</pr>, <pos>v. t.</pos> <ety>[Pref. <ets>to-</ets> + <ets>shred</ets>. ]</ety> <def>To cut into shreds or pieces.</def> <mark>[Obs.]</mark>  <rj><au>Chaucer.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Toss</ent><br/
<hw>Toss</hw> <pr>(?)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Tossed</conjf>  <pr>(<?/)</pr>; (less properly <conjf>Tost</conjf> ); <pos>p. pr. & vb. n.</pos> <conjf>Tossing</conjf>.]</vmorph> <ety>[ W. <ets>tosiaw</ets>, <ets>tosio</ets>, to jerk, toss, snatch, <ets>tos</ets>a quick jerk, a toss, a snatch. ]</ety> <sn>1.</sn> <def>To throw with the hand; especially, to throw with the palm of the hand upward, or to throw upward; <as>as, to <ex>toss</ex> a ball</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To lift or throw up with a sudden or violent motion; <as>as, to <ex>toss</ex> the head</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>He <qex>tossed</qex> his arm aloft, and proudly told me,<br/
He would not stay.</q> <rj><qau>Addison.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>To cause to rise and fall; <as>as, a ship <ex>tossed</ex> on the waves in a storm</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>We being exceedingly <qex>tossed</qex> with a tempest.</q> <rj><qau>Act xxvii. 18.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>To agitate; to make restless.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Calm region once,<br/
And full of peace, now <qex>tossed</qex> and turbulent.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <def>Hence, to try; to harass.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Whom devils fly, thus is he <qex>tossed</qex> of men.</q> <rj><qau>Herbert.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>6.</sn> <def>To keep in play; to tumble over; <as>as, to spend four years in <ex>tossing</ex> the rules of grammar</as>.</def> <mark>[Obs.]</mark>  <rj><au>Ascham.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>To toss off</b></col>, <sd>(a)</sd> <cd>to drink hastily.</cd> <sd>(b)</sd> <cd>to accomplish easily or quickly.</cd> <sd>(c)</sd> <cd>to say in an offhand manner; <as>as, to <ex>toss off</ex> a comment</as>.</cd> <sd>(d)</sd> <cd>to masturbate; -- British slang.</cd> -- <col><b>To toss the cars</b></col>.<cd>See under Oar, <pos>n.</pos></cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Toss</ent><br/
<hw>Toss</hw>, <pos>v. i.</pos> <sn>1.</sn> <def>To roll and tumble; to be in violent commotion; to write; to fling.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>To <qex>toss</qex> and fling, and to be restless, only frets and enrages our pain.</q> <rj><qau>Tillotson.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To be tossed, as a fleet on the ocean.</def>  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>To toss for</b></col>, <cd>to throw dice or a coin to determine the possession of; to gamble for.</cd> -- <col><b>To toss up</b></col>, <cd>to throw a coin into the air, and wager on which side it will fall, or determine a question by its fall.</cd> <au>Bramsion.</au></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Toss</ent><br/
<hw>Toss</hw>, <pos>n.</pos> <sn>1.</sn> <def>A throwing upward, or with a jerk; the act of tossing; <as>as, the <ex>toss</ex> of a ball</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A throwing up of the head; a particular manner of raising the head with a jerk.</def>  <rj><au>Swift.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tossel</ent><br/
<hw>Tos"sel</hw> <pr>(?)</pr>, <pos>n.</pos> <def>See <er>Tassel</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tosser</ent><br/
<hw>Toss"er</hw> <pr>(?)</pr>, <pos>n.</pos> <def>One who tosses.</def>  <rj><au>J. Fletcher.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tossily</ent><br/
<hw>Toss"i*ly</hw> <pr>(?)</pr>, <pos>adv.</pos> <def>In a tossy manner.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tossing</ent><br/
<hw>Toss"ing</hw>, <pos>n.</pos> <sn>1.</sn> <def>The act of throwing upward; a rising and falling suddenly; a rolling and tumbling.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Mining)</fld> <sd>(a)</sd> <def>A process which consists in washing ores by violent agitation in water, in order to separate the lighter or earthy particles; -- called also <altname>tozing</altname>, and <altname>treloobing</altname>, in Cornwall.</def> <au>Pryce.</au> <sd>(b)</sd> <def>A process for refining tin by dropping it through the air while melted.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tosspot</ent><br/
<hw>Toss"pot`</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A toper; one habitually given to strong drink; a drunkard.</def>  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tossy</ent><br/
<hw>Toss"y</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Tossing the head, as in scorn or pride; hence, proud; contemptuous; scornful; affectedly indifferent; <as>as, a <ex>tossy</ex> commonplace</as>.</def> <mark>[R.]</mark>  <rj><au>C. Kingsley.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tost</ent><br/
<hw>Tost</hw> <pr>(?)</pr>, <def><pos>imp. & p. p.</pos> of Toss.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tosto</ent><br/
||<hw>Tos"to</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[It.]</ety> <fld>(Mus.)</fld> <def>Quick; rapid.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs>||<col><b>Pui tosto</b></col> <pr>(<?/)</pr> <ety>[It.]</ety> <fld>(Mus.)</fld>, <cd>faster; more rapid.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Toswink</ent><br/
<hw>To*swink"</hw> <pr>(?)</pr>, <pos>v. i.</pos> <ety>[Pref. <ets>to-</ets> + <ets>swink</ets>.]</ety> <def>To labor excessively.</def> <mark>[Obs.]</mark>  <rj><au>Chaucer.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tot</ent><br/
<hw>Tot</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Cf. <er>Toddle</er>, <er>Tottle</er>, <er>Totter</er>.]</ety> <sn>1.</sn> <def>Anything small; -- frequently applied as a term of endearment to a little child.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A drinking cup of small size, holding about half a pint.</def> <mark>[Prov. Eng.]</mark>  <rj><au>Halliwell.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>A foolish fellow.</def> <mark>[Prov. Eng.]</mark>  <rj><au>Halliwell.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tot</ent><br/
<hw>Tot</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L.]</ety> <def>Lit., so much; -- a term used in the English exchequer to indicate that a debt was good or collectible for the amount specified, and often written opposite the item.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Tot</ent><br/
<hw>Tot</hw>, <pos>v. t.</pos>  <vmorph>[<pos>imp. & p. p.</pos> <conjf>Totted</conjf>; <pos>p. pr. & vb. n.</pos> <conjf>Totting</conjf>.]</vmorph> <sn>1.</sn> <def>To mark with the word <ldquo/tot<rdquo/; <as>as, a <ex>totted</ex> debt</as>. See <er>Tot</er>, <pos>n.</pos></def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><sn>2.</sn>  <ety>[Cf. <er>Total</er>.]</ety> <def>To add; to count; to make up the  sum of; to total; -- often with <xex>up</xex>.</def> <mark>[Colloq., Eng.]</mark></p>

<p><q>The last two <qex>tot</qex> up the bill.</q>  <rj><qau>Thackeray.</qau></rj><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Tota</ent><br/
<hw>To"ta</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[From the native name in Egypt.]</ety> <fld>(Zool.)</fld> <def>The grivet.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Total</ent><br/
<hw>To"tal</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[F., fr. LL. <ets>totalis</ets>, fr. L. <ets>tolus</ets> all,whole.  Cf. <er>Factotum</er>, <er>Surtout</er>, <er>Teetotum</er>.]</ety> <def>Whole; not divided; entire; full; complete; absolute; <as>as, a <ex>total</ex> departure from the evidence; a <ex>total</ex> loss.</as></def> <ldquo/ <xex>Total</xex> darkness.<rdquo/  <ldquo/To undergo myself the <xex>total</xex> crime.<rdquo/  <rj><au>Milton.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Total abstinence</b></col>. <cd>See <er>Abstinence</er>, <pos>n.</pos>, 1.</cd> -- <col><b>Total depravity</b></col>. <fld>(Theol.)</fld> <cd>See <cref>Original sin</cref>, under <er>Original</er>.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><syn>Whole; entire; complete. See <er>Whole</er>.</syn><br/
[<source>1913 Webster</source>]</p>

<p><ent>Total</ent><br/
<hw>To"tal</hw>, <pos>n.</pos> <def>The whole; the whole sum or amount; <as>as, these sums added make the grand <ex>total</ex> of five millions</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Total</ent><br/
<hw>To"tal</hw> <pr>(?)</pr>, <pos>v. t.</pos>  <vmorph>[<pos>imp. & p. p.</pos> <conjf>Totaled</conjf> <pr>(?)</pr> or <conjf>Totalled</conjf>; <pos>p. pr. & vb. n.</pos> <conjf>Totaling</conjf> or <conjf>Totalling</conjf>.]</vmorph> <sn>1.</sn> <def>To bring to a total; also, to reach as a total; to amount to.</def> <mark>[Colloq.]</mark><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><sn>2.</sn> <def>to determine the total of (a set of numbers); to add; -- often used with <ptcl>up</ptcl>; <as>as, to <ex>total</ex> up the bill</as>.</def><br/
[<source>PJC</source>]</p>

<p><sn>3.</sn> <def>To damage beyond repair; -- used especially of vehicles damaged in an accident; <as>as, he skid on an ice patch and <ex>totaled</ex> his Mercedes against a tree</as>.  From <ets>total loss</ets>.</def> <mark>[colloq.]</mark> <br/
[<source>PJC</source>]</p>

<p><ent>Totalis</ent><br/
<hw>To"tal*is</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[See <er>Total</er>, <pos>a.</pos>]</ety> <def>The total.</def></p>

<p><q>I look on nothing but <qex>totalis</qex>.</q>  <rj><qau>B. Jonson.</qau></rj><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Totalisator</ent><br/
<hw>To"tal*i*sa`tor</hw> <pr>(?)</pr>, <pos>n.</pos> <def>Same as <er>Totalizator</er>.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Totality</ent><br/
<hw>To*tal"i*ty</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Cf. F. <ets>totalite</ets>, LL. <ets>totalitas</ets>.]</ety> <sn>1.</sn> <def>The quality or state of being total; <as>as, the <ex>totality</ex> of an eclipse</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>The whole sum; the whole quantity or amount; the entirety; <as>as, the <ex>totality</ex> of human knowledge</as>.</def>  <rj><au>Buckle.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>The <qex>totality</qex> of a sentence or passage.</q> <rj><qau>Coleridge.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Totalization</ent><br/
<hw>To`tal*i*za"tion</hw> <pr>(?)</pr>, <pos>n.</pos> <def>Act of totalizing, or state of being totalized.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Totalizator</ent><br/
<hw>To"tal*i*za`tor</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[From <er>Totalize</er>: cf. F. <ets>totalisateur</ets>.]</ety> <def>A machine for registering and indicating the number and nature of bets made on horse races. Called also <altname>totalizer</altname>.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Totalize</ent><br/
<hw>To"tal*ize</hw> <pr>(?)</pr>, <pos>v. t.</pos> <def>To make total, or complete; to reduce to completeness.</def>  <rj><au>Coleridge.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Totalize</ent><br/
<hw>To"tal*ize</hw>, <pos>v. i.</pos> <def>To use a totalizator.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Totalizer</ent><br/
<hw>To"tal*i`zer</hw> <pr>(?)</pr>, <pos>n.</pos> <def>Same as <er>Totalizator</er>.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Totally</ent><br/
<hw>To"tal*ly</hw>, <pos>adv.</pos> <def>In a total manner; wholly; entirely.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Totalness</ent><br/
<hw>To"tal*ness</hw>, <pos>n.</pos> <def>The quality or state of being total; entireness; totality.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Totara</ent><br/
<hw>To"ta*ra</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Maori.]</ety> <def>A coniferous tree (<spn>Podocarpus totara</spn>), next to the kauri the most valuable timber tree of New Zealand.  Its hard reddish wood is used for furniture and building, esp. in wharves, bridges, etc.  Also <altname>mahogany pine</altname>.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Tote</ent><br/
<hw>Tote</hw> <pr>(?)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Toted</conjf>; <pos>p. pr. & vb. n.</pos> <conjf>Toting</conjf>.]</vmorph> <ety>[Said to be of African origin.]</ety> <def>To carry or bear; <as>as, to <ex>tote</ex> a child over a stream; to <ex>tote</ex> a gun on one's hip</as>; -- a colloquial word originating in the Southern States, and used there esp. by negroes, now common throughout the U. S.</def><br/
[<source>1913 Webster</source> <source>+PJC</source>]</p>

<p><ent>Tote</ent><br/
<hw>Tote</hw>, <pos>n.</pos> <ety>[L. <ets>totum</ets>, fr. <ets>totus</ets> all, whole.]</ety> <def>The entire body, or all; <as>as, the whole <ex>tote</ex></as>.</def> <mark>[Colloq.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Totear</ent><br/
<hw>To*tear"</hw> <pr>(?)</pr>, <pos>v. t.</pos> <ety>[Pref. <ets>to-</ets> + <ets>tear</ets>. ]</ety> <def>To tear or rend in pieces.</def> <mark>[Obs.]</mark>  <rj><au>Chaucer.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Totem</ent><br/
<hw>To"tem</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Massachusetts Indian <ets>wutohtimoin</ets> that to which a person or place belongs.]</ety> <sn>1.</sn> <def>A rude picture, as of a bird, beast, or the like, used by the North American Indians as a symbolic designation, as of a family or a clan; also, the object or animal itself, considered as an symbol of the family.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>And they painted on the grave posts<br/
Of the graves, yet unforgotten,<br/
Each his own ancestral <qex>totem</qex><br/
Each the symbol of his household;<br/
Figures of the bear and reindeer,<br/
Of the turtle, crane, and beaver.</q> <rj><qau>Longfellow.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>The <qex>totem</qex>, the clan deity, the beast or bird who in some supernatural way attends to the clan and watches over it.</q> <rj><qau>Bagehot.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Anything which serves as a venerated or mystic symbol or emblem.</def><br/
[<source>PJC</source>]</p>

<p><ent>Totemic</ent><br/
<hw>To*tem"ic</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Of or pertaining to a totem, or totemism.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Totemism</ent><br/
<hw>To"tem*ism</hw> <pr>(?)</pr>, <pos>n.</pos> <sn>1.</sn> <def>The system of distinguishing families, clans, etc., in a tribe by the totem.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Superstitious regard for a totem; the worship of any real or imaginary object; nature worship.</def>  <rj><au>Tylor.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Totemist</ent><br/
<hw>To"tem*ist</hw>, <pos>n.</pos> <def>One belonging to a clan or tribe having a totem.</def> -- <wordforms><wf>To`tem*is"tic</wf> <pr>(#)</pr>, <pos>a.</pos></wordforms><br/
[<source>1913 Webster</source>]</p>

<p><ent>Totem post</ent><br/
<ent>Totem pole</ent><br/
<mhw>{ <hw>To"tem pole</hw> <it>or</it>  <hw>To"tem post</hw> }</mhw>. <def>A pole or pillar, carved and  painted with a series of totemic symbols, set up  before the house of certain Indian tribes of the northwest coast of North America, esp. Indians of the Koluschan stock.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Toter</ent><br/
<hw>Tot"er</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[See <er>Tote</er> to carry.]</ety> <fld>(Zool.)</fld> <def>The stone roller. See <cref>Stone roller</cref> <sd>(a)</sd>, under <er>Stone</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>T'other</ent><br/
<hw>T'oth"er</hw> <pr>(?)</pr>. <def>A colloquial contraction of <sig>the other</sig>, and formerly a contraction for <xex>that other</xex>. See the Note under <er>That</er>, 2.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The <qex>tothir</qex> that was crucifield with him.</q> <rj><qau>Wyclif(John xix. 32)</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Totipalmate</ent><br/
<hw>To`ti*pal"mate</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>totus</ets> all, whole + E. <ets>palmate</ets>.]</ety> <fld>(Zool.)</fld> <def>Having all four toes united by a web; -- said of certain sea birds, as the pelican and the gannet. See <xex>Illust.</xex> under <er>Aves</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Totipalmi</ent><br/
||<hw>To`ti*pal"mi</hw> <pr>(?)</pr>, <pos>n. pl.</pos> <ety>[NL., from L. <ets>totus</ets> all, whole + <ets>palmus</ets> palm.]</ety> <fld>(Zool.)</fld> <def>A division of swimming birds including those that have totipalmate feet.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Totipresence</ent><br/
<hw>To`ti*pres"ence</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>totus</ets> all, whole + E. <ets>presence</ets>.]</ety> <def>Omnipresence.</def> <mark>[Obs.]</mark>  <rj><au>A. Tucker.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Totipresent</ent><br/
<hw>To`ti*pres"ent</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>totus</ets> all, whole + E. <ets>present</ets>.]</ety> <def>Omnipresent.</def> <mark>[Obs.]</mark>  <rj><au>A. Tucker.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Totter</ent><br/
<hw>Tot"ter</hw> <pr>(?)</pr>, <pos>v. i.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Tottered</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Tottering</conjf>.]</vmorph> <ety>[Probably for older <ets>tolter</ets>; cf. AS. <ets>tealtrian</ets> to totter, vacillate.  Cf.<er>Tilt</er> to incline, <er>Toddle</er>, <er>Tottle</er>, <er>Totty</er>.]</ety> <sn>1.</sn> <def>To shake so as to threaten a fall; to vacillate; to be unsteady; to stagger; <as>as, an old man <ex>totters</ex> with age</as>.</def> <ldquo/As a bowing wall shall ye be, and as a <xex>tottering</xex> fence.<rdquo/  <rj><au>Ps. lxii. 3.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To shake; to reel; to lean; to waver.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Troy nods from high, and <qex>totters</qex> to her fall.</q> <rj><qau>Dryden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Totterer</ent><br/
<hw>Tot"ter*er</hw> <pr>(?)</pr>, <pos>n.</pos> <def>One who totters.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Totteringly</ent><br/
<hw>Tot"ter*ing*ly</hw>, <pos>adv.</pos> <def>In a tottering manner.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tottery</ent><br/
<hw>Tot"ter*y</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Trembling or vaccilating, as if about to fall; unsteady; shaking.</def>  <rj><au>Johnson.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tottle</ent><br/
<hw>Tot"tle</hw> <pr>(t<ocr/t"t'l)</pr>, <pos>v. i.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Tottled</conjf> <pr>(t<ocr/t"t'ld)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Tottling</conjf> <pr>(t<ocr/t"tl<icr/ng)</pr>.]</vmorph> <ety>[See <er>Toddle</er>, <er>Totter</er>.]</ety> <def>To walk in a wavering, unsteady manner; to toddle; to topple.</def> <mark>[Colloq.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tottlish</ent><br/
<hw>Tot"tlish</hw> <pr>(t<ocr/t"tl<icr/sh)</pr>, <pos>a.</pos> <def>Trembling or tottering, as if about to fall; unsteady.</def> <mark>[Colloq. U. S.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Totty</ent><br/
<hw>Tot"ty</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[OE. <ets>toti</ets>.  Cf. <er>Totter</er>.]</ety> <def>Unsteady; dizzy; tottery.</def> <mark>[Obs. or Prov. Eng.]</mark>  <rj><au>Sir W. Scott.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>For yet his noule [head] was <qex>totty</qex> of the must.</q> <rj><qau>Spenser.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Toty</ent><br/
<hw>Tot"y</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Totty.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>My head is <qex>toty</qex> of my swink to-night.</q> <rj><qau>Chaucer.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Toty</ent><br/
<hw>To"ty</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A sailor or fisherman; -- so called in some parts of the Pacific.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Toucan</ent><br/
<hw>Tou"can</hw> <pr>(t<oomac/"k<acr/n; 277)</pr>, <pos>n.</pos> <ety>[F., fr. Pg. <ets>tucano</ets>; from Brazilian name. ]</ety> <sn>1.</sn> <fld>(Zool.)</fld> <def>Any one of numerous species of fruit-eating birds of tropical America belonging to <gen>Ramphastos</gen>, <gen>Pteroglossus</gen>, and allied genera of the family <fam>Ramphastidae</fam>.  They have a very large, but light and thin, beak, often nearly as long as the body itself.  Most of the species are brilliantly colored with red, yellow, white, and black in striking contrast.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Astronom.)</fld> <def>A modern constellation of the southern hemisphere.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Toucanet</ent><br/
<hw>Tou"can*et</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Zool.)</fld> <def>A small toucan.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Touch</ent><br/
<hw>Touch</hw> <pr>(?)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Touched</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Touching</conjf>.]</vmorph> <ety>[F. <ets>toucher</ets>, OF. <ets>touchier</ets>, <ets>tuchier</ets>; of Teutonic origin; cf. OHG. <ets>zucchen</ets>, <ets>zukken</ets>, to twitch, pluck, draw, G. <ets>zukken</ets>, <ets>zukken</ets>, v. intens. fr. OHG. <ets>ziohan</ets> to draw, G. <ets>ziehen</ets>, akin to E. <ets>tug</ets>. See <er>Tuck</er>, <pos>v. t.</pos>, <er>Tug</er>, and cf. <er>Tocsin</er>, <er>Toccata</er>.]</ety> <sn>1.</sn> <def>To come in contact with; to hit or strike lightly against; to extend the hand, foot, or the like, so as to reach or rest on.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Him thus intent Ithuriel with his spear<br/
<qex>Touched</qex> lightly.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To perceive by the sense of feeling.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Nothing but body can be <qex>touched</qex> or <qex>touch</qex>.</q> <rj><qau>Greech.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>To come to; to reach; to attain to.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The god, vindictive, doomed them never more-<br/
Ah, men unblessed! -- to <qex>touch</qex> their natal shore.</q> <rj><qau>Pope.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>To try; to prove, as with a touchstone.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>Wherein I mean to <qex>touch</qex> your love indeed.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <def>To relate to; to concern; to affect.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The quarrel <qex>toucheth</qex> none but us alone.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>6.</sn> <def>To handle, speak of, or deal with; to treat of.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Storial thing that <qex>toucheth</qex> gentilesse.</q> <rj><qau>Chaucer.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>7.</sn> <def>To meddle or interfere with; <as>as, I have not <ex>touched</ex> the books</as>.</def>  <rj><au>Pope.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>8.</sn> <def>To affect the senses or the sensibility of; to move; to melt; to soften; especially, to cause feelings of pity, compassion, sympathy, or gratitude in.</def><br/
[<source>1913 Webster</source> <source>+PJC</source>]</p>

<p><q>What of sweet before<br/
Hath <qex>touched</qex> my sense, flat seems to this and harsh.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>The tender sire was <qex>touched</qex> with what he said.</q> <rj><qau>Addison.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>9.</sn> <def>To mark or delineate with touches; to add a slight stroke to with the pencil or brush.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The lines, though <qex>touched</qex> but faintly, are drawn right.</q> <rj><qau>Pope.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>10.</sn> <def>To infect; to affect slightly.</def>  <rj><au>Bacon.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>11.</sn> <def>To make an impression on; to have effect upon.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Its face . . . so hard that a file will not <qex>touch</qex> it.</q> <rj><qau>Moxon.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>12.</sn> <def>To strike; to manipulate; to play on; <as>as, to <ex>touch</ex> an instrument of music</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>[They] <qex>touched</qex> their golden harps.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>13.</sn> <def>To perform, as a tune; to play.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>A person is the royal retinue <qex>touched</qex> a light and lively air on the flageolet.</q> <rj><qau>Sir W. Scott.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>14.</sn> <def>To influence by impulse; to impel forcibly.</def> <ldquo/ No decree of mine, . . . [to] <xex>touch</xex> with lightest moment of impulse his free will,<rdquo/  <rj><au>Milton.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>15.</sn> <def>To harm, afflict, or distress.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Let us make a covenant with thee, that thou wilt do us no hurt, as we have not <qex>touched</qex> thee.</q> <rj><qau>Gen. xxvi. 28, 29.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>16.</sn> <def>To affect with insanity, especially in a slight degree; to make partially insane; -- rarely used except in the past participle.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>She feared his head was a little <qex>touched</qex>.</q> <rj><qau>Ld. Lytton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>17.</sn> <fld>(Geom.)</fld> <def>To be tangent to. See <er>Tangent</er>, <pos>a.</pos></def><br/
[<source>1913 Webster</source>]</p>

<p><sn>18.</sn> <def>To lay a hand upon for curing disease.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>19.</sn> <def>To compare with; to be equal to; -- usually with a negative; <as>as, he held that for good cheer nothing could <ex>touch</ex> an open fire</as>.</def> <mark>[Colloq.]</mark><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><sn>20.</sn>  <def>To induce to give or lend; to borrow from; <as>as, to <ex>touch</ex> one for a loan</as>; hence, to steal from.</def> <mark>[Slang]</mark><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><cs><col><b>To touch a sail</b></col> <fld>(Naut.)</fld>, <cd>to bring it so close to the wind that its weather leech shakes.</cd> -- <col><b>To touch the wind</b></col> <fld>(Naut.)</fld>, <cd>to keep the ship as near the wind as possible.</cd> -- <col><b>To touch up</b></col>, <cd>to repair; to improve by touches or emendation.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><-- p. 1522 --><br/
[<source>1913 Webster</source>]</p>

<p><ent>Touch</ent><br/
<hw>Touch</hw> <pr>(?)</pr>, <pos>v. i.</pos> <sn>1.</sn> <def>To be in contact; to be in a state of junction, so that no space is between; <as>as, two spheres <ex>touch</ex> only at points</as>.</def>  <rj><au>Johnson.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To fasten; to take effect; to make impression.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>Strong waters pierce metals, and will <qex>touch</qex> upon gold, that will not <qex>touch</qex> upon silver.</q> <rj><qau>Bacon.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>To treat anything in discourse, especially in a slight or casual manner; -- often with <xex>on</xex> or <xex>upon</xex>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>If the antiquaries have <qex>touched</qex> upon it, they immediately<br/
quitted it.</q> <rj><qau>Addison.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <fld>(Naut)</fld> <def>To be brought, as a sail, so close to the wind that its weather leech shakes.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>To touch and go</b></col> <fld>(Naut.)</fld>, <cd>to touch bottom lightly and without damage, as a vessel in motion.</cd> -- <col><b>To touch at</b></col>, <cd>to come or go to, without tarrying; <as>as, the ship <ex>touched at</ex> Lisbon</as>.</cd> -- <mcol><col><b>To touch on</b></col> <it>or</it> <col><b>To touch upon</b></col></mcol>, <sd>(a)</sd> <cd>to come or go to for a short time.</cd> <mark>[R.]</mark>  <br/
[<source>1913 Webster</source>]<br/
<br/
<q>I made a little voyage round the lake, and <qex>touched on</qex> the several towns that lie on its coasts.</q> <rj><qau>Addison.</qau></rj><br/
[<source>1913 Webster</source>]<br/
<br/
<sd>(b)</sd> <cd>to discuss briefly, as only a small part of a discourse.</cd></cs><br/
[<source>PJC</source>]</p>


<p><ent>Touch</ent><br/
<hw>Touch</hw>, <pos>n.</pos> <ety>[Cf. F. <ets>touche</ets>. See <er>Touch</er>, <pos>v.</pos> ]</ety> <sn>1.</sn> <def>The act of touching, or the state of being touched; contact.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Their <qex>touch</qex> affrights me as a serpent's sting.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Physiol.)</fld> <def>The sense by which pressure or traction exerted on the skin is recognized; the sense by which the properties of bodies are determined by contact; the tactile sense. See <cref>Tactile sense</cref>, under <er>Tactile</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The spider's <qex>touch</qex>, how exquisitely fine.</q> <rj><qau>Pope.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ Pure tactile feelings are necessarily rare, since temperature sensations and muscular sensations are more or less combined with them. The organs of touch are found chiefly in the epidermis of the skin and certain underlying nervous structures.</note><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>Act or power of exciting emotion.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Not alone<br/
The death of Fulvia, with more urgent <qex>touches</qex>,<br/
Do strongly speak to us.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>An emotion or affection.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>A true, natural, and a sensible <qex>touch</qex> of mercy.</q> <rj><qau>Hooker.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <def>Personal reference or application.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>Speech of <qex>touch</qex> toward others should be sparingly used.</q> <rj><qau>Bacon.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>6.</sn> <def>A stroke; <as>as, a <ex>touch</ex> of raillery; a satiric <ex>touch</ex>; hence, animadversion; censure; reproof</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>I never bare any <qex>touch</qex> of conscience with greater regret.</q> <rj><qau>Eikon Basilike.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>7.</sn> <def>A single stroke on a drawing or a picture.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Never give the least <qex>touch</qex> with your pencil till you have well examined your design.</q> <rj><qau>Dryden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>8.</sn> <def>Feature; lineament; trait.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Of many faces, eyes, and hearts,<br/
To have the <qex>touches</qex> dearest prized.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>9.</sn> <def>The act of the hand on a musical instrument; bence, in the plural, musical notes.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Soft stillness and the night<br/
Become the <qex>touches</qex> of sweet harmony.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>10.</sn> <def>A small quantity intermixed; a little; a dash.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Eyes La <qex>touch</qex> of Sir Peter Lely in them.</q> <rj><qau>Hazlitt.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Madam, I have a <qex>touch</qex> of your condition.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>11.</sn> <def>A hint; a suggestion; slight notice.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>A small <qex>touch</qex> will put him in mind of them.</q> <rj><qau>Bacon.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>12.</sn> <def>A slight and brief essay.</def> <mark>[Colloq.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>Print my preface in such form as, in the booksellers' phrase, will make a sixpenny <qex>touch</qex>.</q> <rj><qau>Swift.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>13.</sn> <def>A touchstone; hence, stone of the sort used for touchstone.</def> <mark>[Obs.]</mark> <ldquo/ Now do I play the <xex>touch</xex>.<rdquo/  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>A neat new monument of <qex>touch</qex> and alabaster.</q> <rj><qau>Fuller.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>14.</sn> <def>Hence, examination or trial by some decisive standard; test; proof; tried quality.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Equity, the true <qex>touch</qex> of all laws.</q> <rj><qau>Carew.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Friends of noble <qex>touch</qex> .</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>15.</sn> <fld>(Mus.)</fld> <def>The particular or characteristic mode of action, or the resistance of the keys of an instrument to the fingers; <as>as, a heavy <ex>touch</ex>, or a light <ex>touch</ex></as>; also, the manner of touching, striking, or pressing the keys of a piano; <as>as, a legato <ex>touch</ex>; a staccato <ex>touch</ex></as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>16.</sn> <fld>(Shipbilding)</fld> <def>The broadest part of a plank worked top and but (see <cref>Top and but</cref>, under <er>Top</er>, <pos>n.</pos>), or of one worked anchor-stock fashion (that is, tapered from the middle to both ends); also, the angles of the stern timbers at the counters.</def>  <rj><au>J. Knowles.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>17.</sn> <fld>(Football)</fld> <def>That part of the field which is beyond the line of flags on either side.</def>  <rj><au>Encyc. of Rural Sports.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>18.</sn> <def>A boys' game; tag.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>19.</sn> <fld>(Change Ringing)</fld> <def>A set of changes less than the total possible on seven bells, that is, less than 5,040.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><sn>20.</sn>  <def>An act of borrowing or stealing.</def> <mark>[Slang]</mark><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><sn>21.</sn>  <def>Tallow; -- a plumber's term.</def> <mark>[Eng.]</mark><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><cs><col><b>In touch</b></col> <sd>(a)</sd> <fld>(Football)</fld>, <cd>outside of bounds.</cd> <au>T. Hughes.</au> <sd>(b)</sd> <cd>in communication; communicating, once or repeatedly.</cd> -- <col><b>To be in touch</b></col>, <sd>(a)</sd> <cd>to be in contact, communication, or in sympathy.</cd> <sd>(b)</sd> <cd>to be aware of current events.</cd> -- <col><b>To keep touch</b></col>. <sd>(a)</sd> <cd>To be true or punctual to a promise or engagement <mark>[Obs.]</mark>; hence, to fulfill duly a function.</cd><br/
[<source>1913 Webster</source>]</p>

<p><q>My mind and senses <qex>keep touch</qex> and time.</q> <rj><qau>Sir W. Scott.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sd>(b)</sd> <cd>To keep in contact; to maintain connection or sympathy; -- with <xex>with</xex> or <xex>of</xex>.  Also <col>to keep in touch</col>.</cd> --  <col><b>Touch and go</b></col>, <cd>a phrase descriptive of a narrow escape.</cd> -- <col><b>True as touch</b></col>  (<it>i. e.</it>, <xex>touchstone</xex>), <cd>quite true.</cd> <mark>[Obs.]</mark></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Touchable</ent><br/
<hw>Touch"a*ble</hw> <pr>(t<ucr/ch"<adot/*b'l)</pr>, <pos>a.</pos> <def>Capable of being touched; tangible.</def> -- <wordforms><wf>Touch"a*ble*ness</wf>, <pos>n.</pos></wordforms><br/
[<source>1913 Webster</source>]</p>

<p><ent>Touchback</ent><br/
<hw>Touch"back`</hw> <pr>(t<ucr/ch"b<acr/k`)</pr>, <pos>n.</pos> <fld>(Football)</fld> <def>The act of touching the football down by a player behind his own goal line when it received its last impulse from an opponent; -- distinguished from <contr>safety</contr> touchdown.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Touch-box</ent><br/
<hw>Touch"-box`</hw> <pr>(t<ucr/ch"b<ocr/ks`)</pr>, <pos>n.</pos> <def>A box containing lighted tinder, formerly carried by soldiers who used matchlocks, to kindle the match.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Touchdown</ent><br/
<hw>Touch"down`</hw> <pr>(t<ucr/ch"doun`)</pr>, <pos>n.</pos> <fld>(Football)</fld> <def>The act of touching the football down behind the opponents' goal; also the score (6 points) resulting from such a play.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Safety touchdown</b></col>. <cd>See under <er>Safety</er>.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Touchhole</ent><br/
<hw>Touch"hole`</hw> <pr>(t<ucr/ch"h<omac/l`)</pr>, <pos>n.</pos> <def>The vent of a cannon or other firearm, by which fire is communicateed to the powder of the charge.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Touchily</ent><br/
<hw>Touch"i*ly</hw> <pr>(t<ucr/ch"<icr/*l<ycr/)</pr>, <pos>adv.</pos> <def>In a touchy manner.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Touchiness</ent><br/
<hw>Touch"i*ness</hw> <pr>(t<ucr/ch"<icr/*n<ecr/s)</pr>, <pos>n.</pos> <def>The quality or state of being touchy; peevishness; irritability; irascibility.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Touching</ent><br/
<hw>Touch"ing</hw> <pr>(t<ucr/ch"<icr/ng)</pr>, <pos>a.</pos> <def>Affecting; moving; pathetic; <as>as, a <ex>touching</ex> tale</as>.</def> -- <wordforms><wf>Touch"ing*ly</wf> <pr>(#)</pr>, <pos>adv.</pos></wordforms><br/
[<source>1913 Webster</source>]</p>

<p><ent>Touching</ent><br/
<hw>Touch"ing</hw>, <pos>prep.</pos> <def>Concerning; with respect to.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Now, as <qex>touching</qex> things offered unto idols.</q> <rj><qau>1 Cor. viii. 1.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Touching</ent><br/
<hw>Touch"ing</hw>, <pos>n.</pos> <def>The sense or act of feeling; touch.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Touch-me-not</ent><br/
<hw>Touch"-me-not`</hw> <pr>(t<ucr/ch"m<esl/*n<ocr/t`)</pr>, <pos>n.</pos> <fld>(Bot.)</fld> <sd>(a)</sd> <def>See <er>Impatiens</er>.</def> <sd>(b)</sd> <def>Squirting cucumber. See under <er>Cucumber</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Touch-needle</ent><br/
<hw>Touch"-nee`dle</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Metal.)</fld> <def>A small bar of gold and silver, either pure, or alloyed in some known proportion with copper, for trying the purity of articles of gold or silver by comparison of the streaks made by the article and the bar on a touchstone.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Touch-paper</ent><br/
<hw>Touch"-pa`per</hw> <pr>(?)</pr>, <pos>n.</pos> <def>Paper steeped in saltpeter, which burns slowly, and is used as a match for firing gunpowder, and the like.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Touchstone</ent><br/
<hw>Touch"stone`</hw> <pr>(?)</pr>, <pos>n.</pos> <sn>1.</sn> <fld>(Min.)</fld> <def>Lydian stone; basanite; -- so called because used to test the purity of gold and silver by the streak which is left upon the stone when it is rubbed by the metal. See <er>Basanite</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> Fig.: <def>Any test or criterion by which the qualities of a thing are tried.</def>  <rj><au>Hooker.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>The foregoing doctrine affords us also a <qex>touchstone</qex> for the trial of spirits.</q> <rj><qau>South.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Irish touchstone</b></col> <fld>(Min.)</fld>, <cd>basalt, the stone which composes the Giant's Causeway.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>touch on</ent><br/
<hw>touch" on</hw> <pr>(?)</pr>, <pos>v. t.</pos> <def>To mention briefly, or in passing.</def><br/
[<source>PJC</source>]</p>

<p><ent>touch up</ent><br/
<hw>touch" up`</hw> <pr>(?)</pr>, <pos>n.</pos> <def>To make minor improvements in, especially in the appearance.</def><br/
[<source>PJC</source>]</p>

<p><ent>touch up paint</ent><br/
<hw>touch" up paint`</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A paint applied to small areas of a painted surface, to cover small blemishes, especially on an automobile.</def><br/
[<source>PJC</source>]</p>

<p><ent>Touchwood</ent><br/
<hw>Touch"wood`</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Probably for <ets>tachwood</ets>; OE. <ets>tache</ets> tinder (of uncertain origin) + <ets>wood</ets>.]</ety><br/
[<source>1913 Webster</source>]</p>

<p><sn>1.</sn> <def>Wood so decayed as to serve for tinder; spunk, or punk.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Dried fungi used as tinder; especially, the <spn>Polyporus igniarius</spn>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Touchy</ent><br/
<hw>Touch"y</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[For <ets>techy</ets>, <ets>tetchy</ets>.]</ety> <def>Peevish; irritable; irascible; techy; apt to take fire.</def> <mark>[Colloq.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>It may be said of Dryden that he was at no time <qex>touchy</qex> about personal attacks.</q> <rj><qau>Saintsbury.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tough</ent><br/
<hw>Tough</hw> <pr>(?)</pr>, <pos>a.</pos> <amorph>[<pos>Compar.</pos> <adjf>Tougher</adjf> <pr>(?)</pr>; <pos>superl.</pos> <adjf>Toughest</adjf>.]</amorph> <ety>[OE. <ets>tough</ets>, AS. <ets>t<omac/h</ets>, akin to D. <ets>taai</ets>, LG. <ets>taa</ets>, <ets>tage</ets>, <ets>tau</ets>, OHG. <ets>z<amac/hi</ets>, G.  <ets>z<aum/he</ets>, and also to AS. ge<ets>tenge</ets> near to, close to, oppressive, OS. bi<ets>tengi</ets>.]</ety> <sn>1.</sn> <def>Having the quality of flexibility without brittleness; yielding to force without breaking; capable of resisting great strain; <as>as, the ligaments of animals are remarkably <ex>tough</ex></as>.</def> <ldquo/<xex>Tough</xex> roots and stubs. <rdquo/  <rj><au>Milton.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Not easily broken; able to endure hardship; firm; strong; -- of objects and people; <as>as, <ex>tough</ex> sinews</as>.</def>  <rj><au>Cowper.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>A body made of brass, the crone demands, . . . <br/
<qex>Tough</qex> to the last, and with no toil to tire.</q> <rj><qau>Dryden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>The basis of his character was caution combined with <qex>tough</qex> tenacity of purpose.</q> <rj><qau>J. A. Symonds.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>Not easily separated; viscous; clammy; tenacious; <as>as, <ex>tough</ex> phlegm</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>Stiff; rigid; not flexible; stubborn; <as>as, a <ex>tough</ex> bow</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>So <qex>tough</qex> a frame she could not bend.</q> <rj><qau>Dryden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <def>Severe; violent; <as>as, a <ex>tough</ex> storm</as>.</def> <mark>[Colloq.]</mark> <ldquo/ A <xex>tough</xex> debate. <rdquo/  <rj><au>Fuller.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>6.</sn> <def>Difficult to do, perform, or accomplish; <as>as, a <ex>tough</ex> job</as>.</def><br/
[<source>PJC</source>]</p>

<p><sn>7.</sn> <def>Prone to aggressive or violent behavior; rowdyish; -- of people, or groups; <as>as, a <ex>tough</ex> neighborhood; a <ex>tough</ex> character</as>.</def><br/
[<source>PJC</source>]</p>

<p><cs><col><b>To make it tough</b></col>, <cd>to make it a matter of difficulty; to make it a hard matter.</cd> <mark>[Obs.]</mark>  <rj><au>Chaucer.</au></rj></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>tough</ent><br/
<hw>tough</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A person who is tough{7}; a ruffian; a thug; <as>as, a cluster of neighborhood <ex>toughs</ex> hanging out on the corner</as>.</def><br/
[<source>PJC</source>]</p>

<p><ent>Tough-cake</ent><br/
<hw>Tough"-cake`</hw> <pr>(?)</pr>, <pos>n.</pos> <def>See <er>Tough-pitch</er> <sd>(b)</sd>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Toughen</ent><br/
<hw>Tough"en</hw> <pr>(?)</pr>, <pos>v. i. & t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Toughened</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Toughening</conjf>.]</vmorph> <def>To grow or make tough, or tougher.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tough-head</ent><br/
<hw>Tough"-head`</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Zool.)</fld> <def>The ruddy duck.</def> <mark>[ Local U. S. ]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Toughish</ent><br/
<hw>Tough"ish</hw>, <pos>a.</pos> <def>Tough in a slight degree.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Toughly</ent><br/
<hw>Tough"ly</hw>, <pos>adv.</pos> <def>In a tough manner.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Toughness</ent><br/
<hw>Tough"ness</hw>, <pos>n.</pos> <def>The quality or state of being tough.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tough-pitch</ent><br/
<hw>Tough"-pitch`</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Metal.)</fld> <sd>(a)</sd> <def>The exact state or quality of texture and consistency of well reduced and refined copper.</def> <sd>(b)</sd> <def>Copper so reduced; -- called also <altname>tough-cake</altname>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Touite</ent><br/
<hw>Tou"ite</hw> <pr>(?)</pr>, <pos>n.</pos> <def>The wood warbler.</def> <mark>[Prov. Eng.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Toupet</ent><br/
<ent>Toupee</ent><br/
<mhw>{ <hw>Tou*pee"</hw> <pr>(?; 277)</pr>, <hw>Tou*pet"</hw> <pr>(?; 277)</pr> }</mhw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F. <ets>toupet</ets>, dim. of OF. <ets>top</ets> a tuft; of Teutonic origin, and akin to E. <ets>top</ets>. See <er>Top</er> apex, and cf. <er>Topet</er>.]</ety><br/
[<source>1913 Webster</source>]</p>

<p><sn>1.</sn> <def>A little tuft; a curl or artificial lock of hair.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A small wig, or a toppiece of a wig.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Her powdered hair is turned backward over a <qex>toupee</qex>.</q> <rj><qau>G. Eliot.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Toupettit</ent><br/
<hw>Tou"pet*tit</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[See <er>Topet</er>, <er>toupee</er>.]</ety> <fld>(Zool.)</fld> <def>The crested titmouse.</def> <mark>[Prov. Eng.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tour</ent><br/
<hw>Tour</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F. <ets>tour</ets>. See <er>Tower</er>.]</ety> <def>A tower.</def> <mark>[Obs.]</mark>  <rj><au>Chaucer.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tour</ent><br/
<hw>Tour</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F. <ets>tour</ets>. See <er>Turn</er>, <pos>v. t.</pos>]</ety> <sn>1.</sn> <def>A going round; a circuit; hence, a journey in a circuit; a prolonged circuitous journey; a comprehensive excursion; <as>as, the <ex>tour</ex> of Europe; the <ex>tour</ex> of France or England</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The bird of Jove stooped from his airy <qex>tour</qex>.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A turn; a revolution; <as>as, the <ex>tours</ex> of the heavenly bodies</as>.</def> <mark>[Obs.]</mark>  <rj><au>Blackmore.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <fld>(Mil.)</fld> <def>anything done successively, or by regular order; a turn; <as>as, a <ex>tour</ex> of duty</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><syn><b>Syn.</b> -- Journey; excursion. See <er>Journey</er>.</syn><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tour</ent><br/
<hw>Tour</hw> <pr>(?)</pr>, <pos>v. i.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Toured</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Touring</conjf>.]</vmorph> <def>To make a tourm; <as>as, to <ex>tour</ex> throught a country</as>.</def>  <rj><au>T. Hughes.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Touraco</ent><br/
<hw>Tou*ra"co</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Zool.)</fld> <def>Same as <er>Turacou</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tourbillion</ent><br/
<hw>Tour*bil"lion</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F. <ets>torbillion</ets> a whirlwind, tourbillion, fr. L. <ets>turbo</ets>, -<ets>inis</ets>, a whirl, whirlwind.]</ety> <def>An ornamental firework which turns round, when in the air, so as to form a scroll of fire.</def>  <rj><au>G. Francis.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Touring car</ent><br/
<hw>Tour"ing car</hw>. <def>An automobile designed for touring; specif., a roomy car, not a limousine, for five or more passengers.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Tourist</ent><br/
<hw>Tour"ist</hw> <pr>(?)</pr>, <pos>n.</pos> <def>One who makes a tour, or performs a journey, especially for pleasure.</def><br/
[<source>1913 Webster</source> <source>+PJC</source>]</p>

<p><ent>Tourmaline</ent><br/
<hw>Tour"ma*line</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F. <ets>tourmaline</ets>, cf. It. <ets>turmalina</ets>, <ets>tormalina</ets>, NL. <ets>turmalina</ets>, <ets>turmalinus</ets>; all fr. <ets>tournamal</ets>, a name given to this stone in Ceylon.]</ety> <fld>(Min.)</fld> <def>A mineral occurring usually in three-sided or six-sided prisms terminated by rhombohedral or scalenohedral planes. Black tourmaline (schorl) is the most common variety, but there are also other varieties, as the blue (indicolite), red (rubellite), also green, brown, and white. The red and green varieties when transparent are valued as jewels.</def> <altsp>[Written also <asp>turmaline</asp> .]</altsp><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ Crystals of tourmaline when heated exhibit electric polarity (see <er>Pyroelectric</er>, <pos>n.</pos>).  Tourmaline is also used in the form of a polariscope called <xex>tourmaline tongs</xex>.</note><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tourn</ent><br/
<hw>Tourn</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[See <er>Turn</er>]</ety> <sn>1.</sn> <def>A spinning wheel.</def> <mark>[Prov. Eng.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(O.Eng.Law)</fld> <def>The sheriff's turn, or court.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tournament</ent><br/
<hw>Tour"na*ment</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[OE. <ets>turnement</ets>, <ets>tornement</ets>, OF. <ets>torneiement</ets>, <ets>tornoiement</ets>, F. <ets>tournoiement</ets> a turning or wheeling round. See <er>Tourney</er>.]</ety> <sn>1.</sn> <def>A mock fight, or warlike game, formerly in great favor, in which a number of combatants were engaged, as an exhibition of their address and bravery; hence, figuratively, a real battle.</def> <ldquo/In battle and in <xex>tourneyment</xex>.<rdquo/  <rj><au>Chaucer.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>With cruel <qex>tournament</qex> the squadrons join.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ It different from the <xex>joust</xex>, which was a trial of skill between one man and another.</note><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Any contest of skill in which there are many contestents for championship; <as>as, a chess <ex>tournament</ex></as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tournery</ent><br/
<hw>Tourn"er*y</hw> <pr>(?)</pr>, <pos>n.</pos> <def>Work turned on a lathe; turnery.</def> <mark>[Obs.]</mark> <see>See <er>Turnery</er>.</see>  <rj><au>Evelyn.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tourney</ent><br/
<hw>Tour"ney</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[OF. <ets>tornei</ets>, <ets>tornoi</ets>, F. <ets>tournoi</ets>, fr. OF. <ets>torneier</ets>, <ets>tornoier</ets>, <ets>tournoier</ets>, to tit, to tourney, F. <ets>tournoyer</ets> to turn round and round. See <er>Turn</er>, <pos>v. t.</pos>]</ety> <def>A tournament.</def>  <rj><au>Bacon.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>At tilt or <qex>tourney</qex> or like warlike game.</q> <rj><qau>Spenser.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>We hold a <qex>tourney</qex> here to-morrow morn,<br/
And there is scantly time for half the work.</q> <rj><qau>Tennyson.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tourney</ent><br/
<hw>Tour"ney</hw>, <pos>v. i.</pos> <ety>[Cf.OF. <ets>torneier</ets>. See <er>Tourney</er>, <pos>n.</pos> ]</ety> <def>To perform in tournaments; to tilt.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Well could he <qex>tourney</qex>, and in lists debate.</q> <rj><qau>Spenser.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tourniquet</ent><br/
<hw>Tour"ni*quet</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F., fr. <ets>tourner</ets> to turn.]</ety> <fld>(Surg.)</fld> <def>An instrument for arresting hemorrhage. It consists essentially of a pad or compress upon which pressure is made by a band which is tightened by a screw or other means.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tournois</ent><br/
||<hw>Tour`nois"</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F., belonging to <ets>Tours</ets> in France.]</ety> <def>A former French money of account worth 20 sous, or a franc. It was thus called in distinction from the Paris <xex>livre</xex>, which contained 25 sous.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tournure</ent><br/
||<hw>Tour*nure"</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F., fr. <ets>tourner</ets> to turn.]</ety><br/
[<source>1913 Webster</source>]</p>

<p><sn>1.</sn> <def>Turn; contour; figure.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Any device used by women to expand the skirt of a dress below the waist; a bustle.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Touze</ent><br/
<ent>Touse</ent><br/
<mhw>{ <hw>Touse</hw>, <hw>Touze</hw> <pr>(touz)</pr> }</mhw>, <pos>v. t. & i.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Toused</conjf> <pr>(touzd)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Tousing</conjf>.]</vmorph> <ety>[OE. <ets>tosen</ets>.  <root/64. See <er>tease</er>, and cf. <er>Tose</er>, <er>Toze</er>.]</ety> <def>To pull; to haul; to tear; to worry.</def> <mark>[Prov. Eng.]</mark>  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>As a bear, whom angry curs have <qex>touzed</qex>.</q> <rj><qau>Spenser.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Touse</ent><br/
<hw>Touse</hw> <pr>(touz)</pr>, <pos>n.</pos> <def>A pulling; a disturbance.</def> <mark>[Prov. Eng.]</mark>  <rj><au>Halliwell.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tousel</ent><br/
<hw>Tous"el</hw> <pr>(?)</pr>, <pos>v. t.</pos> <def>Same as <er>Tousle</er>.</def> <mark>[Colloq.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Touser</ent><br/
<hw>Tous"er</hw> <pr>(?)</pr>, <pos>n.</pos> <def>One who touses.</def> <mark>[Prov. Eng.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tousle</ent><br/
<hw>Tou"sle</hw> <pr>(?)</pr>, <pos>v. t.</pos> <ety>[Freq. of <ets>touse</ets>.  Cf.<er>Tossle</er>.]</ety> <def>To put into disorder; to tumble; to touse.</def> <mark>[Colloq.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tous-les-mois</ent><br/
<hw>Tous`-les`-mois"</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F., all the months, <it>i. e.</it>, every month.]</ety> <def>A kind of starch with very large, oval, flattened grains, often sold as arrowroot, and extensively used for adulterating cocoa. It is made from the rootstocks of a species of <gen>Canna</gen>, probably <spn>Canna edulis</spn>, the tubers of which are edible every month in the year.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tousy</ent><br/
<hw>Tou"sy</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[See <er>Touse</er>, <pos>n. & v.</pos>]</ety> <def>Tousled; tangled; rough; shaggy.</def> <mark>[Colloq.]</mark><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Tout</ent><br/
<hw>Tout</hw> <pr>(t<oomac/t)</pr>, <pos>v. i.</pos> <ety>[See 1st <er>Toot</er>.]</ety> <sn>1.</sn> <def>To act as a tout. See 2d <er>Tout</er>.</def> <mark>[Cant. Eng.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To ply or seek for customers.</def> <mark>[Prov. Eng.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tout</ent><br/
<hw>Tout</hw> <pr>(tout; t<oomac/t; <it>Scot. & dial.</it> t<oomac/t)</pr>, <pos>v. i.</pos>  <vmorph>[<pos>imp. & p. p.</pos> <conjf>Touted</conjf>; <pos>p. pr. & vb. n.</pos> <conjf>Touting</conjf>.]</vmorph> <sn>1.</sn> <def>To look narrowly; spy.</def> <mark>[Scot. & Dial. Eng.]</mark><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><sn>2.</sn>  <fld>(Horse Racing)</fld> <sd>(a)</sd> <def>To spy out the movements of race horses at their trials, or to get by stealth or other improper means the secrets of the stable, for betting purposes.</def> <mark>[Cant, Eng.]</mark> <sd>(b)</sd> <def>To act as a tout; to tout, or  give a tip on, a race horse.</def> <mark>[Cant, U. S.]</mark><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Tout</ent><br/
<hw>Tout</hw>, <pos>n.</pos> <def>One who secretly watches race horses which are in course of training, to get information about their capabilities, for use in betting.</def> <mark>[Cant. Eng.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>One who gives a tip on a race horses for an expected compensation, esp. in hopes of a share in any winnings; -- usually contemptuous.</def> <mark>[Cant, U. S.]</mark><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><sn>3.</sn>  <def>One who solicits custom, as a runner for a hotel, cab, gambling place.</def> <mark>[Colloq.]</mark><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><sn>4.</sn>  <def>A spy for a smuggler, thief, or the like.</def> <mark>[Colloq.]</mark><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Tout</ent><br/
<hw>Tout</hw> <pr>(t<oomac/t)</pr>, <pos>n.</pos> <ety>[Prob. fr. F. <ets>tout</ets> all.]</ety> <def>In the game of solo, a proposal to win all eight tricks.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Tout</ent><br/
<hw>Tout</hw>, <pos>v. i.</pos> <ety>[See 3d <er>Toot</er>. ]</ety> <def>To toot a horn.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tout</ent><br/
<hw>Tout</hw>, <pos>v. t.</pos> <fld>(Horse Racing)</fld> <sd>(a)</sd> <def>To spy out information about, as a racing stable or horse.</def> <mark>[Cant, Eng.]</mark> <sd>(b)</sd> <def>To give a tip on (a race horse) to a better with the  expectation of sharing in the latter's winnings.</def> <mark>[Cant, U. S.]</mark><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Tout</ent><br/
<hw>Tout</hw>, <pos>n.</pos> <def>The anus.</def> <mark>[Obs.]</mark>  <rj><au>Chaucer.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tout-ensemble</ent><br/
||<hw>Tout`-en`sem"ble</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F.]</ety> <def>All together; hence, in costume, the fine arts, etc., the general effect of a work as a whole, without regard to the execution of the separate perts.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Touter</ent><br/
<hw>Tout"er</hw> <pr>(?)</pr>, <pos>n.</pos> <def>One who seeks customers, as for an inn, a public conveyance, shops, and the like: hence, an obtrusive candidate for office.</def> <mark>[Colloq.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>The prey of ring droppers, . . . duffers, <qex>touters</qex>, or any of those bloodless sharpers who are, perhaps, better known to the police.</q> <rj><qau>Dickens.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Touze</ent><br/
<hw>Touze</hw> <pr>(touz)</pr>, <pos>v. t. & i.</pos> <def>See <er>Touse</er>.</def> <mark>[Prov. Eng.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tow</ent><br/
<hw>Tow</hw> <pr>(t<omac/)</pr>, <pos>n.</pos> <ety>[OE. <ets>tow</ets>, AS. <ets>tow</ets>, akin to OD. <ets>touw</ets>, Icel.  <ets>t<omac/</ets> a tuft of wool for spinning; cf. E. <ets>taw</ets>, v. t.]</ety> <def>The coarse and broken part of flax or hemp, separated from the finer part by the hatchel or swingle.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tow</ent><br/
<hw>Tow</hw>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Towed</conjf> <pr>(t<omac/d)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Towing</conjf>.]</vmorph> <ety>[OE. <ets>towen</ets>, to<yogh/en; akin to OFries. <ets>toga</ets> to pull about, OHG. zog<omac/n, Icel. <ets>toga</ets>, AS. <ets>toh</ets>line a towline, and AS. <ets>te<oacute/n</ets> to draw, p. p. <ets>getogen</ets>. See <er>Tug</er>.]</ety> <def>To draw or pull through the water, as a vessel of any kind, by means of a rope.</def><br/
[<source>1913 Webster</source>]</p>

<p><-- p. 1523 --><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tow</ent><br/
<hw>Tow</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Cf. Icel. <ets>taug</ets> a rope, from the same root as E. <ets>tow</ets>, <pos>v. t.</pos>]</ety> <sn>1.</sn> <def>A rope by which anything is towed; a towline, or towrope.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>The act of towing, or the state of being towed; -- chiefly used in the phrase, <xex>to take in tow</xex>, that is to tow.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>That which is towed, or drawn by a towline, as a barge, raft, collection of boats, ect.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Towage</ent><br/
<hw>Tow"age</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[From <er>Tow</er>, <pos>v.</pos>.cf. F. <ets>touage</ets>.]</ety> <sn>1.</sn> <def>The act of towing.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>The price paid for towing.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Towall</ent><br/
<hw>Tow"all</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A towel.</def> <mark>[Obs.]</mark>  <rj><au>Chaucer.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Towards</ent><br/
<ent>Toward</ent><br/
<mhw>{ <hw>To"ward</hw>, <hw>To"wards</hw>  }</mhw> <pr>(?)</pr>, <pos>prep.</pos><ety>[AS. <?/ impending, imminent, future, toward, <?/ towards. See <er>To</er>, and -<er>ward</er>, <er>wards</er>.]</ety> <sn>1.</sn> <def>In the direction of; to.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>He set his face <qex>toward</qex> the wilderness.</q> <rj><qau>Num. xxiv. 1.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>The waves make <qex>towards</qex> the pebbled shore.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>With direction to, in a moral sense; with respect or reference to; regarding; concerning.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>His eye shall be evil <qex>toward</qex> his brother.</q> <rj><qau>Deut. xxviii. 54.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Herein do I exercise myself, to have always a conscience void of offense <qex>toward</qex> God, and <qex>toward</qex> men.</q> <rj><qau>Acts xxiv. 16.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>Tending to; in the direction of; in behalf of.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>This was the first alarm England received <qex>towards</qex> any trouble.</q> <rj><qau>Clarendom.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>Near; about; approaching to.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>I am <qex>toward</qex> nine years older since I left you.</q> <rj><qau>Swift.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Towards</ent><br/
<ent>Toward</ent><br/
<mhw>{ <hw>To"ward</hw>, <hw>To"wards</hw>  }</mhw> <pos>adv.</pos> <def>Near; at hand; in state of preparation.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Do you hear sught, sir, of a battle <qex>toward</qex> ?</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>We have a trifling foolish banquet <qex>Towards</qex>.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Toward</ent><br/
<hw>To"ward</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[AS. <?/. See <er>Toward</er>, <pos>prep.</pos>]</ety> <sn>1.</sn> <def>Approaching; coming near.</def> <ldquo/His <xex>toward</xex> peril.<rdquo/  <rj><au>Spenser.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Readly to do or learn; compliant with duty; not froward; apt; docile; tractable; <as>as, a <ex>toward</ex> youth</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>Ready to act; forward; bold; valiant.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Why, that is spoken like a <qex>toward</qex> prince.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Towardliness</ent><br/
<hw>To"ward*li*ness</hw> <pr>(?)</pr>, <pos>n.</pos> <def>The quality or state of being towardly; docility; tractableness.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The beauty and <qex>towardliness</qex> of these children moved her brethren to envy.</q> <rj><qau>Sir W. Raleigh.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Towardly</ent><br/
<hw>To"ward*ly</hw>, <pos>a.</pos> <def>Same as <er>Toward</er>, <pos>a.</pos>, 2.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>He's <qex>towardly</qex> and will come on apace.</q> <rj><qau>Dryden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Towardness</ent><br/
<hw>To"ward*ness</hw>, <pos>n.</pos> <def>Quality or state of being toward.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Towards</ent><br/
<hw>To"wards</hw> <pr>(<?/)</pr>, <pos>prep. & adv.</pos> <def>See <er>Toward</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Towboat</ent><br/
<hw>Tow"boat`</hw> <pr>(?)</pr>, <pos>n.</pos> <sn>1.</sn> <def>A vessel constructed for being towed, as a canal boat.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A steamer used for towing other vessels; a tug.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Towel</ent><br/
<hw>Tow"el</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[OE. <ets>towaille</ets>, <ets>towail</ets>, F. <ets>touaille</ets>, LL. <ets>toacula</ets>, of Teutonic origin; cf. OHG. <ets>dwahila</ets>, <ets>swahilla</ets>, G. <ets>zwehle</ets>, fr. OHG. <ets>dwahan</ets> to wash; akin to D. <ets>dwaal</ets> a towel, AS. <ets><thorn/we<aacute/n</ets> to wash, OS. <ets>thwahan</ets>, Icel. <ets><thorn/v<amac/</ets>, Sw. <ets>tv<aring/</ets>, Dan. <ets>toe</ets>, Goth. <ets><thorn/wahan</ets>.   Cf. <er>Doily</er>.]</ety> <def>A cloth used for wiping, especially one used for drying anything wet, as the person after a bath.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Towel gourd</b></col> <fld>(Bot.)</fld>, <cd>the fruit of the cucurbitaceous plant <spn>Luffa Aegyptiaca</spn>; also, the plant itself. The fruit is very fibrous, and, when separated from its rind and seeds, is used as a sponge or towel. Called also <altname>Egyptian bath sponge</altname>, and <altname>dishcloth</altname>.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Towel</ent><br/
<hw>Tow"el</hw>, <pos>v. t.</pos> <def>To beat with a stick.</def> <mark>[Prov. Eng.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Toweling</ent><br/
<hw>Tow"el*ing</hw>, <pos>n.</pos> <def>Cloth for towels, especially such as is woven in long pieces to be cut at will, as distinguished from that woven in towel lengths with borders, etc.</def> <altsp>[Written also <asp>towelling</asp>.]</altsp><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tower</ent><br/
<hw>Tow"er</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[OE. <ets>tour</ets>,<ets>tor</ets>,<ets>tur</ets>, F. <ets>tour</ets>, L. <ets>turris</ets>; akin to Gr. <?/; cf. W. <ets>twr</ets> a tower, Ir. <ets>tor</ets> a castle, Gael. <ets>torr</ets> a tower, castle.  Cf. <er>Tor</er>, <er>Turret</er>.]</ety> <sn>1.</sn> <fld>(Arch.)</fld> <sd>(a)</sd> <def>A mass of building standing alone and insulated, usually higher than its diameter, but when of great size not always of that proportion.</def> <sd>(b)</sd> <def>A projection from a line of wall, as a fortification, for purposes of defense, as a flanker, either or the same height as the curtain wall or higher.</def> <sd>(c)</sd> <def>A structure appended to a larger edifice for a special purpose, as for a belfry, and then usually high in proportion to its width and to the height of the rest of the edifice; <as>as, a church <ex>tower</ex></as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A citadel; a fortress; hence, a defense.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Thou hast been a shelter for me, and a strong <qex>tower</qex> from the enemy.</q> <rj><qau>Ps. lxi. 3.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>A headdress of a high or towerlike form, fashionable about the end of the seventeenth century and until 1715; also, any high headdress.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Lay trains of amorous intrigues<br/
In <qex>towers</qex>, and curls, and periwigs.</q> <rj><qau>Hudibras.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>High flight; elevation.</def> <mark>[Obs.]</mark>  <rj><au>Johnson.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Gay Lussac's tower</b></col> <fld>(Chem.)</fld>, <cd>a large tower or chamber used in the sulphuric acid process, to absorb (by means of concentrated acid) the spent nitrous fumes that they may be returned to the Glover's tower to be reemployed. See <cref>Sulphuric acid</cref>, under <er>Sulphuric</er>, and <cref>Glover's tower</cref>, below.</cd> -- <col><b>Glover's tower</b></col> <fld>(Chem.)</fld>, <cd>a large tower or chamber used in the manufacture of sulphuric acid, to condense the crude acid and to deliver concentrated acid charged with nitrous fumes. These fumes, as a catalytic, effect the conversion of sulphurous to sulphuric acid. See <cref>Sulphuric acid</cref>, under <er>Sulphuric</er>, and <cref>Gay Lussac's tower</cref>, above.</cd> -- <col><b>Round tower</b></col>. <cd>See under <er>Round</er>, <pos>a.</pos></cd> -- <col><b>Shot tower</b></col>. <cd>See under <er>Shot</er>.</cd> -- <col><b>Tower bastion</b></col> <fld>(Fort.)</fld>, <cd>a bastion of masonry, often with chambers beneath, built at an angle of the interior polygon of some works.</cd> -- <col><b>Tower mustard</b></col> <fld>(Bot.)</fld>, <cd>the cruciferous plant <spn>Arabis perfoliata</spn>.</cd> -- <col><b>Tower of London</b></col>, <cd>a collection of buildings in the eastern part of London, formerly containing a state prison, and now used as an arsenal and repository of various objects of public interest.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tower</ent><br/
<hw>Tow"er</hw> <pr>(?)</pr>, <pos>v. i.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>towered</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>towering</conjf>.]</vmorph> <def>To rise and overtop other objects; to be lofty or very high; hence, to soar.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>On the other side an high rock <qex>towered</qex> still.</q> <rj><qau>Spenser.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>My lord protector's hawks do <qex>tower</qex> so well.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tower</ent><br/
<hw>Tow"er</hw>, <pos>v. t.</pos> <def>To soar into.</def> <mark>[Obs.]</mark>  <rj><au>Milton.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Towered</ent><br/
<hw>Tow"ered</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Adorned or defended by towers.</def><br/
[<source>1913 Webster</source>]</p>

<p><q><qex>Towered</qex> cities please us then.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Towering</ent><br/
<hw>Tow"er*ing</hw> <pr>(?)</pr>, <pos>a.</pos> <sn>1.</sn> <def>Very high; elevated; rising aloft; <as>as, a <ex>towering</ex> height</as>.</def>  <rj><au>Pope.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Hence, extreme; violent; surpassing.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>A man agitated by a <qex>towering</qex> passion.</q> <rj><qau>Sir W. Scott.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Towery</ent><br/
<hw>Tow"er*y</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Having towers; adorned or defended by towers.</def> <mark>[R.]</mark> <ldquo/<xex>Towery cities</xex>.<rdquo/  <rj><au>Pope.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tow-head</ent><br/
<hw>Tow"-head`</hw> <pr>(?)</pr>, <pos>n.</pos> <sn>1.</sn> <def>An urchin who has soft, whitish hair.</def> <mark>[Colloq.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Zool.)</fld> <def>The hooded merganser.</def> <mark>[ Local, U. S. ]</mark><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>A low alluvial island or shoal in a river.</def> <mark>[Local, U. S.]</mark><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Towhee</ent><br/
<hw>To*whee"</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Zool.)</fld> <def>The chewink.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Towilly</ent><br/
<hw>To*wil"ly</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Zool.)</fld> <def>The sanderling; -- so called from its cry.</def> <mark>[Prov. Eng.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Towline</ent><br/
<hw>Tow"line`</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[AS. <ets>tohline</ets>. See <er>Tow</er>, <pos>v. t.</pos>, <ets>and</ets> <er>Line</er>. ]</ety> <fld>(Naut.)</fld> <def>A line used to tow vessels; a towrope.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Town</ent><br/
<hw>Town</hw> <pr>(toun)</pr>, <pos>n.</pos> <ety>[OE. <ets>toun</ets>, <ets>tun</ets>, AS. <ets>tun</ets> inclosure, fence, village, town; akin to D. <ets>tuin</ets> a garden, G. <ets>zaun</ets> a hadge, fence, OHG. <ets>zun</ets>, Icel. <ets>tun</ets> an inclosure, homestead, house, Ir. & Gael. <ets>dun</ets> a fortress, W. <ets>din</ets>.  Cf. <er>Down</er>, <pos>adv. & prep.</pos>, <er>Dune</er>, <er>tine</er> to inclose.]</ety> <sn>1.</sn> <def>Formerly: <sd>(a)</sd> An inclosure which surrounded the mere homestead or dwelling of the lord of the manor. <mark>[Obs.]</mark> <sd>(b)</sd> The whole of the land which constituted the domain. <mark>[Obs.]</mark> <sd>(c)</sd> A collection of houses inclosed by fences or walls.</def> <mark>[Obs.]</mark>  <rj><au>Palsgrave.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Any number or collection of houses to which belongs a regular market, and which is not a city or the see of a bishop.</def> <mark>[Eng.]</mark>  <rj><au>Johnson.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>Any collection of houses larger than a village, and not incorporated as a city; also, loosely, any large, closely populated place, whether incorporated or not, in distinction from the country, or from rural communities.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>God made the country, and man made the <qex>town</qex>.</q> <rj><qau>Cowper.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>The body of inhabitants resident in a town; <as>as, the <ex>town</ex> voted to send two representatives to the legislature; the <ex>town</ex> voted to lay a tax for repairing the highways</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <def>A township; the whole territory within certain limits, less than those of a country.</def> <mark>[U. S.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><sn>6.</sn> <def>The court end of London; -- commonly with <xex>the</xex>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>7.</sn> <def>The metropolis or its inhabitants; <as>as, in winter the gentleman lives in <ex>town</ex>; in summer, in the country</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Always hankering after the diversions of the <qex>town</qex>.</q> <rj><qau>Addison.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Stunned with his giddy larum half the <qex>town</qex>.</q> <rj><qau>Pope.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ The same form of expressions is used in regard to other populous towns.</note><br/
[<source>1913 Webster</source>]</p>

<p><sn>8.</sn> <def>A farm or farmstead; also, a court or farmyard.</def> <mark>[Prov. Eng. & Scot.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ <xex>Town</xex> is often used adjectively or in combination with other words; as, <xex>town</xex> clerk, or <xex>town</xex>-clerk; <xex>town</xex>-crier, or <xex>town</xex> crier; <xex>town</xex>hall, <xex>town</xex>-hall, or <xex>town</xex> hall; <xex>town</xex>house, <xex>town</xex> house, or <xex>town</xex>-house.</note><br/
[<source>1913 Webster</source>]</p>

<p><syn><b>Syn.</b> -- Village; hamlet. See <er>Village</er>.</syn><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Town clerk</b></col>, <cd>an office who keeps the records of a town, and enters its official proceedings. See <er>Clerk</er>.</cd> -- <col><b>Town cress</b></col> <fld>(Bot.)</fld>, <cd>the garden cress, or peppergrass.</cd> <au>Dr. Prior.</au> -- <col><b>Town house</b></col>. <sd>(a)</sd> <cd>A house in town, in distinction from a house in the country.</cd> <sd>(b)</sd> <cd>See <er>Townhouse</er>.</cd> -- <col><b>Town meeting</b></col>, <cd>a legal meeting of the inhabitants of a town entitled to vote, for the transaction of public bisiness.</cd> <mark>[U. S.]</mark> -- <col><b>Town talk</b></col>, <cd>the common talk of a place; the subject or topic of common conversation.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>town and gown</ent><br/
<hw>town" and gown"</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Of or pertaining to interactions between a college or university and the residents of the town in which the institution is located; <as>as, a <ex>town and gown</ex> dispute</as>.</def><br/
[<source>PJC</source>]</p>

<p><ent>Town-crier</ent><br/
<hw>Town"-cri`er</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A town officer who makes proclamations to the people; the public crier of a town.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Towned</ent><br/
<hw>Towned</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Having towns; containing many towns.</def> <mark>[Obs.]</mark>  <rj><au>Hakluyt.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Townhall</ent><br/
<hw>Town"hall`</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A public hall or building, belonging to a town, where the public offices are established, the town council meets, the people assemble in town meeting, etc.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Townhouse</ent><br/
<hw>Town"house`</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A building devoted to the public used of a town; a townhall.</def><br/
[<source>1913 Webster</source>]</p>

<p><-- 2. a house in the city, usu. said of a second residence belonging to one who has a permanent residence elsewhere, as in the countryside. = town house (b) <br/
[<source>1913 Webster</source>]</p>

<p> 3. Row House. --><br/
[<source>1913 Webster</source>]</p>

<p><ent>Townish</ent><br/
<hw>Town"ish</hw>, <pos>a.</pos> <def>Of or pertaining to the inhabitants of a town; like the town.</def> <mark>[R.]</mark>  <rj><au>Turbervile.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Townless</ent><br/
<hw>Town"less</hw>, <pos>a.</pos> <def>Having no town.</def>  <rj><au>Howell.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Townlet</ent><br/
<hw>Town"let</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A small town.</def>  <rj><au>North Brit. Rev.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Townsfolk</ent><br/
<hw>Towns"folk`</hw> <pr>(?)</pr>, <pos>n.</pos> <def>The people of a town; especially, the inhabitants of a city, in distinction from country people; townspeople.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Township</ent><br/
<hw>Town"ship</hw> <pr>(?)</pr>, <pos>n.</pos> <sn>1.</sn> <def>The district or territory of a town.</def><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ In the United States, many of the States are divided into townships of five, six, seven, or perhaps ten miles square, and the inhabitants of such townships are invested with certain powers for regulating their own affairs, such as repairing roads and providing for the poor. The township is subordinate to the county.</note><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>In surveys of the public land of the United States, a division of territory six miles square, containing 36 sections.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>In Canada, one of the subdivisions of a county.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Townsman</ent><br/
<hw>Towns"man</hw> <pr>(tounz"m<ait/n)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Townsmen</plw> <pr>(tounz"m<eit/n)</pr>.</plu><br/
[<source>1913 Webster</source>]</p>

<p><sn>1.</sn> <def>An inhabitant of a town; one of the same town with another.</def>  <rj><au>Pope.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A selectman, in New England. See <er>Selectman</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Townspeople</ent><br/
<hw>Towns"peo`ple</hw> <pr>(?)</pr>, <pos>n.</pos> <def>The inhabitants of a town or city, especially in distinction from country people; townsfolk.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Townwards</ent><br/
<ent>Townward</ent><br/
<mhw>{ <hw>Town"ward</hw>, <hw>Town"wards</hw> <pr>(?)</pr>, }</mhw> <pr>(?)</pr>, <pos>adv.</pos> <def>Toward a town.</def>  <rj><au>Longfellow.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Towpath</ent><br/
<hw>Tow"path`</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A path traveled by men or animals in towing boats; -- called also <altname>towing path</altname>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Towrope</ent><br/
<hw>Tow"rope`</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A rope used in towing vessels.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Towser</ent><br/
<hw>Tow"ser</hw> <pr>(tou"z<etil/r)</pr>, <pos>n.</pos> <ety>[See <er>Touse</er> to pull about. ]</ety> <def>A familiar name for a dog.</def> <altsp>[ Written also <asp>Towzer</asp>. ]</altsp><br/
[<source>1913 Webster</source>]</p>

<p><ent>Towy</ent><br/
<hw>Tow"y</hw> <pr>(t<omac/"<ycr/)</pr>, <pos>a.</pos> <def>Composed of, or like, tow.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Toxaemia</ent><br/
||<hw>Tox*<ae/"mi*a</hw> <pr>(t<ocr/ks"<emac/"m<icr/*<adot/)</pr>, <pos>a.</pos> <ety>[NL., fr. Gr. <grk>toxiko`n</grk> poison + <grk>a"i^ma</grk> blood.]</ety> <fld>(Med.)</fld> <def>Blood poisoning. See under <er>Blood</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Toxalbumin</ent><br/
<hw>Tox`al*bu"min</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[See <er>Toxic</er>; <er>Albumin</er>.]</ety> <fld>(Physiol. Chem.)</fld> <def>Any of a class of toxic substances of protein nature; a toxin.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Toxical</ent><br/
<ent>Toxic</ent><br/
<mhw>{ <hw>Tox"ic</hw>, <hw>Tox"ic*al</hw>  }</mhw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>toxicum</ets> poison, originally, a poison in which arrows were dipped, Gr. <grk>toxiko`n</grk> (sc. <?/) poison for smearing arrows with, fr. <grk>toxiko`s</grk> of or for the bow, from <grk>to`xon</grk> bow, arrow.  Cf. <er>Intoxicate</er>.]</ety> <def>Of or pertaining to poison; poisonous; <as>as, <ex>toxic</ex> medicines</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Toxicant</ent><br/
<hw>Tox"i*cant</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A poisonous agent or drug, as opium; an intoxicant.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Toxication</ent><br/
<hw>Tox`i*ca"tion</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>toxicum</ets> a poison: cf. <ets>toxicatus</ets> smeared with poison.]</ety> <def>Poisoning.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Toxicity</ent><br/
<hw>Tox*ic"i*ty</hw> <pr>(?)</pr>, <pos>n.</pos> <def>The quality or state of being toxic or poisonous; poisonousness.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Toxicogenic</ent><br/
<hw>Tox`i*co*gen"ic</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[See <er>Toxic</er>, and <er>-genic</er>.]</ety> <fld>(Physiol. & Med.)</fld> <def>Producing toxic products; <as>as, <ex>toxicogenic</ex> germs or bacteria</as>.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Toxicological</ent><br/
<hw>Tox`i*co*log"ic*al</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[Cf.F. <ets>toxicologique</ets>. ]</ety> <def>Of or pertaining to toxicology.</def> -- <wordforms><wf>Tox`i*co*log"ic*al*ly</wf>, <pos>adv.</pos></wordforms><br/
[<source>1913 Webster</source>]</p>

<p><ent>Toxicologist</ent><br/
<hw>Tox`i*col"o*gist</hw> <pr>(?)</pr>, <pos>n.</pos> <def>One versed in toxicology; the writer of a treatise on poisons.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Toxicology</ent><br/
<hw>Tox`i*col"o*gy</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Gr. <grk>toxiko`n</grk> poison + <ets>-logy</ets>; cf. F. <ets>toxicologie</ets>. See <er>Toxic</er>. ]</ety> <def>The science which treats of poisons, their effects, antidotes, and recognition; also, a discourse or treatise on the science.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Toxicomania</ent><br/
<hw>Tox`i*co*ma"ni*a</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[See. <er>Toxic</er>, and <er>Mania</er>.]</ety> <sn>1.</sn> <fld>(Med.)</fld> <def>Toxiphobia.</def>  <rj><au>A. S. Taylor.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Med.)</fld> <def>An insane desire for intoxicating or poisonous drugs, as alcohol or opium.</def>  <rj><au>B. W. Richardson.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Toxifera</ent><br/
||<hw>Tox*if"e*ra</hw> <pr>(?)</pr>, <pos>n. pl.</pos> <ety>[NL., fr. Gr. <grk>to`xon</grk> a bow + L. <ets>ferre</ets> to bear.]</ety> <fld>(Zool.)</fld> <def>Same as <er>Toxoglossa</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Toxine</ent><br/
<ent>Toxin</ent><br/
<mhw>{ <hw>Tox"in</hw>, <hw>Tox"ine</hw>  }</mhw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Gr. <grk>toxiko`n</grk>. See <er>Toxic</er>. ]</ety> <def>A poisonous product formed by an organism, such as a pathogenic bacterium, a plant or an animal, usually having a high molecular weight, often a protein or a polysaccharide, but occasionally a low-molecular weight agent such as <stype>tetrodotoxin</stype>.</def></p>

<p><ent>Toxiphobia</ent><br/
||<hw>Tox`i*pho"bi*a</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[NL., fr. Gr. <grk>toxiko`n</grk> poison + <grk>fo`bos</grk> fear.]</ety> <fld>(Med.)</fld> <def>An insane or greatly exaggerated dread of poisons.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Toxodon</ent><br/
<hw>Tox"o*don</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Gr. <grk>to`xon</grk> a bow + <?/, <?/, a tooth.]</ety> <fld>(Paleon.)</fld> <def>A gigantic extinct herbivorous mammal from South America, having teeth bent like a bow. It is the type of the order <ord>Toxodonta</ord>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Toxodonta</ent><br/
||<hw>Tox`o*don"ta</hw> <pr>(?)</pr>, <pos>n.pl.</pos> <ety>[NL.]</ety> <fld>(Paleon.)</fld> <def>An extinct order of Mammalia found in the South American Tertiary formation. The incisor teeth were long and curved and provided with a persistent pulp. They are supposed to be related both to the rodents and ungulates. Called also <altname>Toxodontia</altname>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Toxoglossa</ent><br/
||<hw>Tox`o*glos"sa</hw> <pr>(?)</pr>, <pos>n. pl.</pos> <ety>[NL., fr. Gr. <grk>to`xon</grk> a bow + <?/ tongue.]</ety> <fld>(Zool.)</fld> <def>A division of marine gastropod mollusks in which the radula are converted into poison fangs. The cone shells (<gen>Conus</gen>), Pleurotoma, and Terebra, are examples. See <xex>Illust.</xex> of <er>Cone</er>, <pos>n.</pos>, 4, <er>Pleurotoma</er>, and <er>Terebra</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Toxoid</ent><br/
<hw>Tox"oid</hw> <pr>(?)</pr>, <pos>n.</pos>  <ety>[<ets>Tox</ets>in + <ets>-oid</ets>.]</ety> <fld>(Physiol. Chem.)</fld> <def>An altered form of a toxin, possessing little or no toxic power.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Toxophilite</ent><br/
<hw>Tox*oph"i*lite</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Gr. <grk>to`xon</grk> a bow + <grk>filei^n</grk> to love.]</ety> <def>A lover of archery; one devoted to archery.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Toxotes</ent><br/
||<hw>Tox"o*tes</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[NL., fr. Gr. <?/ an archer.]</ety> <fld>(Zool.)</fld> <def>A genus of fishes comprising the archer fishes. See <er>Archer fish</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Toy</ent><br/
<hw>Toy</hw> <pr>(toi)</pr>, <pos>n.</pos> <ety>[D. <ets>tuid</ets> tools, implements, stuff, trash, speel<ets>tuig</ets> playthings, toys; akin to G. <ets>zeug</ets> stuff, materials, MNG. <ets>ziuc</ets>, Icel. <ets>tygi</ets> gear; all ultimately from the root of E. <ets>tug</ets>, v. t.; cf. G. <ets>zeugen</ets> to beget, MHG. <ets>ziugen</ets> to beget, make ready, procure. See <er>Tug</er>, <pos>v. t.</pos>]</ety><br/
[<source>1913 Webster</source>]</p>

<p><sn>1.</sn> <def>A plaything for children; a bawble.</def>  <rj><au>Cowper.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A thing for amusement, but of no real value; an article of trade of little value; a trifle.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>They exchange for knives, glasses, and such <qex>toys</qex>, great abundance of gold and pearl.</q> <rj><qau>Abr. Abbot.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>A wild fancy; an odd conceit; idle sport; folly; trifling opinion.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>To fly about playing their wanton <qex>toys</qex>.</q> <rj><qau>Spenser.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>What if a <qex>toy</qex> take 'em in the heels now, and they all run away.</q> <rj><qau>Beau. & Fl.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Nor light and idle <qex>toys</qex> my lines may vainly swell.</q> <rj><qau>Drayton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>Amorous dalliance; play; sport; pastime.</def>  <rj><au>Milton.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>To dally thus with death is no fit <qex>toy</qex>.</q> <rj><qau>Spenser.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <def>An old story; a silly tale.</def>  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>6.</sn> <ety>[Probably the same word.]</ety> <def>A headdress of linen or woolen, that hangs down over the shoulders, worn by old women of the lower classes; -- called also <altname>toy mutch</altname>.</def> <mark>[Scot.]</mark> <ldquo/Having, moreover, put on her clean <xex>toy</xex>, rokelay, and scarlet plaid.<rdquo/  <rj><au>Sir W. Scott.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Toy</ent><br/
<hw>Toy</hw>, <pos>v. i.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>toyed</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>toying</conjf>.]</vmorph> <def>To dally amorously; to trifle; to play.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>To <qex>toy</qex>, to wanton, dally, smile and jest.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Toy</ent><br/
<hw>Toy</hw>, <pos>v. t.</pos> <def>To treat foolishly.</def> <mark>[Obs.]</mark> <au>E. Dering (1576).</au><br/
[<source>1913 Webster</source>]</p>

<p><ent>Toyear</ent><br/
<hw>To*year</hw> <pr>(?)</pr>, <pos>adv.</pos> <ety>[<ets>To</ets>, prep. + <ets>year</ets>. ]</ety> <def>This year.</def> <mark>[Obs.]</mark>  <rj><au>Chaucer.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Toyer</ent><br/
<hw>Toy"er</hw> <pr>(?)</pr>, <pos>n.</pos> <def>One who toys; one who is full of trifling tricks; a trifler.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Toyful</ent><br/
<hw>Toy"ful</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Full of trifling play.</def> <mark>[Obs.]</mark>  <rj><au>Donne.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Toyhouse</ent><br/
<hw>Toy"house`</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A house for children to play in or to play with; a playhouse.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Toyingly</ent><br/
<hw>Toy"ing*ly</hw> <pr>(?)</pr>, <pos>adv.</pos> <def>In a toying manner.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Toyish</ent><br/
<hw>Toy"ish</hw>, <pos>a.</pos> <sn>1.</sn> <def>Sportive; trifling; wanton.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Resembling a toy.</def><br/
[<source>1913 Webster</source>]</p>

<p>-- <wordforms><wf>Toy"ish*ly</wf>, <pos>adv.</pos> -- <wf>Toy"ish*ness</wf>, <pos>n.</pos></wordforms><br/
[<source>1913 Webster</source>]</p>

<p><-- p. 1524 --><br/
[<source>1913 Webster</source>]</p>

<p><ent>Toyman</ent><br/
<hw>Toy"man</hw> <pr>(toi"m<ait/n)</pr>, <pos>n.</pos> <def>One who deals in toys.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Toyshop</ent><br/
<hw>Toy"shop`</hw> <pr>(-sh<ocr/p`)</pr>, <pos>n.</pos> <def>A shop where toys are sold.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Toysome</ent><br/
<hw>Toy"some</hw> <pr>(toi"s<ucr/m)</pr>, <pos>a.</pos> <def>Disposed to toy; trifling; wanton.</def> <mark>[R.]</mark>  <rj><au>Ford.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Toze</ent><br/
<hw>Toze</hw> <pr>(?)</pr>, <pos>v. t.</pos> <def>To pull violently; to touse.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tozy</ent><br/
<hw>To"zy</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[See <er>Toze</er> ]</ety> <def>Soft, like wool that has been teased.</def> -- <wordforms><wf>To"zi*ness</wf> <pr>(#)</pr>, <pos>n.</pos></wordforms><br/
[<source>1913 Webster</source>]</p>

<p><ent>Trabea</ent><br/
||<hw>Tra"be*a</hw> <pr>(?)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Trabeae</plw> <pr>(#)</pr>.</plu> <ety>[L.]</ety> <fld>(Rom. Antiq.)</fld> <def>A toga of purple, or ornamented with purple horizontal stripes. -- worn by kings, consuls, and augurs.</def>  <rj><au>Dr. W. Smith.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Trabeated</ent><br/
<hw>Tra"be*a`ted</hw> <pr>(?)</pr>, <pos>a.</pos> <fld>(Arch.)</fld> <def>Furnished with an entablature.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Trabeation</ent><br/
<hw>Tra`be*a"tion</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>trabs</ets>, <ets>trabis</ets>, a beam, a timber.]</ety> <fld>(Arch.)</fld> <def>Same as <er>Entablature</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Trabecula</ent><br/
||<hw>Tra*bec"u*la</hw> <pr>(?)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Trabeculae</plw>  <pr>(-l<emac/)</pr>.</plu> <ety>[L., a little beam.]</ety> <fld>(Anat.)</fld> <def>A small bar, rod, bundle of fibers, or septal membrane, in the framework of an organ part.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Trabecular</ent><br/
<hw>Tra*bec"u*lar</hw> <pr>(?)</pr>, <pos>a.</pos> <fld>(Anat.)</fld> <def>Of or pertaining to a trabecula or trabeculae; composed of trabeculae.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Trabeculate</ent><br/
<hw>Tra*bec"u*late</hw> <pr>(?)</pr>, <pos>a.</pos> <fld>(Bot.)</fld> <def>Crossbarred, as the ducts in a banana stem.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Trabu</ent><br/
<hw>Tra"bu</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Zool.)</fld> <def>Same as <er>Trubu</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Trace</ent><br/
<hw>Trace</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F. <ets>trais</ets>. pl. of <ets>trait</ets>. See <er>Trait</er>.]</ety> <sn>1.</sn> <def>One of two straps, chains, or ropes of a harness, extending from the collar or breastplate to a whiffletree attached to a vehicle or thing to be drawn; a tug.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Mech.)</fld> <def>A connecting bar or rod, pivoted at each end to the end of another piece, for  transmitting motion, esp. from one plane to another; specif., such a piece in an organ-stop action to transmit motion from the trundle to the lever actuating the stop slider.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Trace</ent><br/
<hw>Trace</hw>, <pos>n.</pos> <ety>[F. <ets>trace</ets>. See <er>Trace</er>, <pos>v. t.</pos> ]</ety> <sn>1.</sn> <def>A mark left by anything passing; a track; a path; a course; a footprint; a vestige; <as>as, the <ex>trace</ex> of a carriage or sled; the <ex>trace</ex> of a deer; a sinuous <ex>trace</ex></as>.</def>  <rj><au>Milton.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Chem. & Min.)</fld> <def>A very small quantity of an element or compound in a given substance, especially when so small that the amount is not quantitatively determined in an analysis; -- hence, in stating an analysis, often contracted to <xex>tr.</xex></def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>A mark, impression, or visible appearance of anything left when the thing itself no longer exists; remains; token; vestige.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The shady empire shall retain no <qex>trace</qex><br/
Of war or blood, but in the sylvan chase.</q> <rj><qau>Pope.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <fld>(Descriptive Geom. & Persp.)</fld> <def>The intersection of a plane of projection, or an original plane, with a coordinate plane.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <fld>(Fort.)</fld> <def>The ground plan of a work or works.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Syn</b></col>.<cd>-Vestige; mark; token. See <er>Vestige</er>.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Trace</ent><br/
<hw>Trace</hw>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>traced</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>tracing</conjf>.]</vmorph> <ety>[OF. <ets>tracier</ets>, F. <ets>tracer</ets>, from  (assumed) LL. <ets>tractiare</ets>, fr.L. <ets>tractus</ets>, p. p. of <ets>trahere</ets> to draw.  Cf. <er>Abstract</er>, <er>Attract</er>, <er>Contract</er>, <er>Portratt</er>, <er>Tract</er>, <er>Trail</er>, <er>Train</er>, <er>Treat</er>. ]</ety> <sn>1.</sn> <def>To mark out; to draw or delineate with marks; especially, to copy, as a drawing or engraving, by following the lines and marking them on a sheet superimposed, through which they appear; <as>as, to <ex>trace</ex> a figure or an outline; a <ex>traced</ex> drawing</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Some faintly <qex>traced</qex> features or outline of the mother and the child, slowly lading into the twilight of the woods.</q> <rj><qau>Hawthorne.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To follow by some mark that has been left by a person or thing which has preceded; to follow by footsteps, tracks, or tokens.</def>  <rj><au>Cowper.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>You may <qex>trace</qex> the deluge quite round the globe.</q> <rj><qau>T. Burnet.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>I feel thy power . . . to <qex>trace</qex> the ways<br/
Of highest agents.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>Hence, to follow the trace or track of.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>How all the way the prince on footpace <qex>traced</qex>.</q> <rj><qau>Spenser.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>To copy; to imitate.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>That servile path thou nobly dost decline,<br/
Of <qex>tracing</qex> word, and line by line.</q> <rj><qau>Denham.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <def>To walk over; to pass through; to traverse.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>We do <qex>trace</qex>this alley up and down.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Trace</ent><br/
<hw>Trace</hw>, <pos>v. i.</pos> <def>To walk; to go; to travel.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>Not wont on foot with heavy arms to <qex>trace</qex>.</q> <rj><qau>Spenser.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Traceable</ent><br/
<hw>Trace"a*ble</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Capable of being traced.</def> -- <wordforms><wf>Trace"a*ble*ness</wf>, <pos>n.</pos> -- <wf>Trace"a/bly</wf>, <pos>adv.</pos></wordforms><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tracer</ent><br/
<hw>Tra"cer</hw> <pr>(?)</pr>, <pos>n.</pos> <def>One who, or that which, traces.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A person engaged (esp. in the express or railway service) in tracing, or searching out, missing articles, as packages or freight cars.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><sn>3.</sn>  <def>An inquiry sent out (esp. in transportation service) for a missing article, as a letter or an express package.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><sn>4.</sn> <fld>(Mil.)</fld> <def>a type of ammunition that emits light or smoke as it moves toward its target, providing a visible path of the projectile in flight so that the point of impact may be observed; -- called also <altname>tracer ammunition</altname>.</def><br/
[<source>PJC</source>]</p>

<p><sn>5.</sn> <fld>(Mil.)</fld> <def>the chemical substance used in tracer ammunition to cause it to be visible in flight.</def><br/
[<source>PJC</source>]</p>

<p><sn>6.</sn>  <def>a chemical substance with properties, such as radioactivity or fluorescence, which make it easily measurable, used to observe the movements of chemically related substances through a biological, physical, or chemical system; -- in biochemistry, also called <altname>labeled compounds</altname>.</def> <note>Radioactive tracers are used, for example, to measure the retention or distribution of residues of drugs after administration to an animal, to determine the type and rate of metabolism; also, to measure the rate of motion of molecules in electrophoresis or the leakage of small quantities of material from a container.  Small fluorescent tracers may be attached in many cases to macromolecules such as proteins or nucleic acids, allowing the motions of such macromolecules to be easily observed by their acquired fluorescence, without appreciably changing their properties.  In biological and biochemial systems the common radioactive isotopes used in tracers are carbon-14, tritium (hydrogen-3), sulfur-35, phosphorus-32, and iodine-131; other isotopes are also used, including non-radioactive isotopes such as carbon-13.</note><br/
[<source>PJC</source>]</p>

<p><ent>Tracer/y</ent><br/
<hw>Tra"cer/y</hw> <pr>(?)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Traceries</plw> <pr>(<?/)</pr></plu> <fld>(Arch.)</fld> <sn>1.</sn> <def>Ornamental work with rambled lines.</def> <specif>Especially:</specif> -- <sd>(a)</sd> <def>The decorative head of a Gothic window.</def><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ <xex>Window tracery</xex> is of two sorts, <xex>plate tracery</xex> and <xex>bar tracery</xex>. <xex>Plate tracery</xex>, common in Italy, consists of a series of ornamental patterns cut through a flat plate of stone. <xex>Bar tracery</xex> is a decorative pattern formed by the curves and intersections of the molded bars of the mullions.  Window tracery is imitated in many decorative objects, as panels of wood or metal either pierced or in relief. See also <xex>Stump tracery</xex> under <er>Stump</er>, and <xex>Fan tracery</xex> under <er>Fan</er>.</note><br/
[<source>1913 Webster</source>]</p>

<p><sd>(b)</sd> <def>A similar decoration in some styles of vaulting, the ribs of the vault giving off the minor bars of which the tracery is composed.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A tracing of lines; a system of lines produced by, or as if by, tracing, esp. when interweaving or  branching out in ornamental or graceful figures.</def> <ldquo/Knit with curious <xex>tracery</xex>.<rdquo/  <rj><au>Burns.</au></rj><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Trachea</ent><br/
<hw>Tra"che*a</hw> <pr>(?)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Tracheae</plw> <pr>(#)</pr>.</plu> <ety>[NL.,from L. <ets>trachia</ets>, Gr.  <grk>trachei^a</grk>  (sc. <?/ windpipe), from <?/ rough, rugged: cf. F. <ets>trach<eacute/e</ets>.]</ety><br/
[<source>1913 Webster</source>]</p>

<p><sn>1.</sn> <fld>(Anat.)</fld> <def>The windpipe. See <xex>Illust.</xex> of <er>Lung</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Zool.)</fld> <def>One of the respiratory tubes of insects and arachnids.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <fld>(Bot.)</fld> <def>One of the large cells in woody tissue which have spiral, annular, or other markings, and are connected longitudinally so as to form continuous ducts.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tracheal</ent><br/
<hw>Tra"che*al</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[Cf.F. <ets>tracheal</ets>.]</ety> <def>Of or pertaining to the trachea; like a trachea.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Trachearia</ent><br/
||<hw>Tra`che*a"ri*a</hw> <pr>(?)</pr>, <pos>n. pl.</pos> <ety>[NL.]</ety> <fld>(Zool.)</fld> <def>A division of Arachnida including those that breathe only by means of tracheae. It includes the mites, ticks, false scorpions, and harvestmen.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tracheary</ent><br/
<hw>Tra"che*a*ry</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Tracheal; breathing by means of tracheae.</def> -- <def2><pos>n.</pos> <fld>(Zool.)</fld> <def>One of the Trachearia.</def></def2><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tracheata</ent><br/
||<hw>Tra`che*a"ta</hw> <pr>(?)</pr>, <pos>n. pl.</pos> <ety>[NL.]</ety> <fld>(Zool.)</fld> <def>An extensive division of arthropods comprising all those which breathe by tracheae, as distinguished from Crustacea, which breathe by means of branchiae.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tracheate</ent><br/
<hw>Tra"che*ate</hw> <pr>(?)</pr>, <pos>a.</pos> <fld>(Zool.)</fld> <def>Breathing by means of tracheae; of or pertaining to the Tracheata.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tracheate</ent><br/
<hw>Tra"che*ate</hw>, <pos>n.</pos> <fld>(Zool.)</fld> <def>Any arthropod having tracheae; one of the Tracheata.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tracheid</ent><br/
<hw>Tra"che*id</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Bot.)</fld> <def>A wood cell with spiral or other markings and closed throughout, as in pine wood.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tracheitis</ent><br/
||<hw>Tra`che*i"tis</hw> <pr>(tr<amac/`k<esl/*<imac/"t<icr/s)</pr>, <pos>n.</pos> <ety>[NL. See <er>Trachea</er>; <er>-itis</er>.]</ety> <fld>(Med.)</fld> <def>Inflammation of the trachea, or windpipe.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Trachelidan</ent><br/
<hw>Tra*chel"i*dan</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Gr. <?/ neck.]</ety> <fld>(Zool.)</fld> <def>Any one of a tribe of beetles (<fam>Trachelides</fam>) which have the head supported on a pedicel. The oil beetles and the Cantharides are examples.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Trachelipod</ent><br/
<hw>Tra*chel"i*pod</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Gr.<?/ neck + <ets>-pod</ets>:cf.F. <ets>trachelipode</ets>.]</ety> <fld>(Zool.)</fld> <def>One of the Trachelipoda.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Trachelipoda</ent><br/
||<hw>Tra`che*lip"o*da</hw> <pr>(?)</pr>, <pos>n. pl.</pos> <ety>[NL. See <er>Trachelipod</er>.]</ety> <fld>(Zool.)</fld> <def>An extensive artificial group of gastropods comprising all those which have a spiral shell and the foot attached to the base of the neck.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Trachelipodous</ent><br/
<hw>Tra`che*lip"o*dous</hw> <pr>(?)</pr>, <pos>a.</pos> <fld>(Zool.)</fld> <def>Having the foot united with the neck; of or pertaining to the Trachelipoda.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Trachelobranchiate</ent><br/
<hw>Tra`che*lo*bran"chi*ate</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[Gr.<?/ + E. <ets>tranchiate</ets>.]</ety> <fld>(Zool.)</fld> <def>Having the gills situated upon the neck; -- said of certain mollusks.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Trachelorrhaphy</ent><br/
<hw>Tra`che*lor"rha*phy</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Gr. <?/ neck + <?/ to sew.]</ety> <fld>(Med.)</fld> <def>The operation of sewing up a laceration of the neck of the uterus.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Trachenchyma</ent><br/
||<hw>Tra*chen"chy*ma</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[NL.,fr. <ets>trachea</ets> + -<ets>enchyma</ets> as in E. <ets>parenchyma</ets>.]</ety> <fld>(Bot.)</fld> <def>A vegetable tissue consisting of tracheae.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tracheobranchia</ent><br/
||<hw>Tra`che*o*bran"chi*a</hw> <pr>(?)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Tracheobranchlae</plw> <pr>(#)</pr>.</plu> <ety>[NL. See <er>Trachea</er>, and <er>Branchia</er>.]</ety> <fld>(Zool.)</fld> <def>One of the gill-like breathing organs of certain aquatic insect larvae. They contain tracheal tubes somewhat similar to those of other insects.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tracheobronchial</ent><br/
<hw>Tra`che*o*bron"chi*al</hw> <pr>(?)</pr>, <pos>a.</pos> <fld>(Zool.)</fld> <def>Pertaining both to the tracheal and bronchial tubes, or to their junction; -- said of the syrinx of certain birds.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tracheocele</ent><br/
<hw>Tra"che*o*cele</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Gr. <?/ the windpipe + <?/ a tumor: cf.F. <ets>tracheocele</ets>. ]</ety> <fld>(Med.)</fld> <sd>(a)</sd> <def>Goiter.</def> <sd>(b)</sd> <def>A tumor containing air and communicating with the trachea.</def>  <rj><au>Morell Mackenzie.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tracheophonae</ent><br/
||<hw>Tra`che*oph"o*nae</hw> <pr>(<?/)</pr>, <pos>n. pl.</pos> <ety>[NL., from <ets>trachea</ets> + Gr. <grk>fonei^n</grk> to sound.]</ety> <fld>(Zool.)</fld> <def>A group of passerine birds having the syrinx at the lower end of the trachea.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tracheoscopy</ent><br/
<hw>Tra`che*os"co*py</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[<ets>Trachea</ets> + <ets>-scopy</ets>.]</ety> <fld>(Med.)</fld> <def>Examination of the interior of the trachea by means of a mirror.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tracheotomy</ent><br/
<hw>Tra`che*ot"o*my</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[<ets>Trachea</ets> + Gr. <?/ to cut: cf.F. <ets>tracheotomie</ets>.]</ety> <fld>(Surg.)</fld> <def>The operation of making an opening into the windpipe.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Trachinoid</ent><br/
<hw>Tra"chi*noid</hw> <pr>(?)</pr>, <pos>a.</pos> <fld>(Zool.)</fld> <def>Of, pertaining to, or like, <gen>Trachinus</gen>, a genus of fishes which includes the weevers. See <er>Weever</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Trachitis</ent><br/
||<hw>Tra*chi"tis</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[NL.]</ety> <fld>(Med.)</fld> <def>Tracheitis.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Trachoma</ent><br/
||<hw>Tra*cho"ma</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[NL., fr. Gr. <?/ roughness, fr. <?/ rough.]</ety> <fld>(Med.)</fld> <def>a contagious granular conjunctivitis caused by the bacterium <spn>Chlamydia trachomatis</spn>.  It is a chronic inflammation of the conjunctiva</def> -- <wordforms><wf>Tra*chom"a*tous</wf> <pr>(#)</pr>, <pos>a.</pos></wordforms><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Trachycarpous</ent><br/
<hw>Tra`chy*car"pous</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[Gr. <?/ rough + <?/ fruit.]</ety> <fld>(Bot.)</fld> <def>Rough-fruited.</def>  <rj><au>Gray.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Trachymedusae</ent><br/
||<hw>Tra`chy*me*du"sae</hw> <pr>(?)</pr>, <pos>n. pl.</pos> <ety>[NL., fr.Gr. <?/ rough + <ets>medusa</ets>.]</ety> <fld>(Zool.)</fld> <def>A division of acalephs in which the development is direct from the eggs, without a hydroid stage. Some of the species are parasitic on other medusae.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Trachyspermous</ent><br/
<hw>Tra`chy*sper"mous</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[Gr. <?/ rough + <?/ seed.]</ety> <fld>(Bot.)</fld> <def>Rough-seeded.</def>  <rj><au>Gray.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Trachystomata</ent><br/
||<hw>Tra`chy*stom"a*ta</hw> <pr>(?)</pr>, <pos>n. pl.</pos> <ety>[NL., fr. Gr. <?/ rough + <ets>stoma</ets>.]</ety> <fld>(Zool.)</fld> <def>An order of tailed aquatic amphibians, including <gen>Siren</gen> and <gen>Pseudobranchus</gen>. They have anterior legs only, are eel-like in form, and have no teeth except a small patch on the palate. The external gills are persistent through life.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Trachyte</ent><br/
<hw>Tra"chyte</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Gr. <?/ rough, rugged: cg. F. <ets>trachyte</ets>.]</ety> <fld>(Geol.)</fld> <def>An igneous rock, usually light gray in color and breaking with a rough surface. It consists chiefly of orthoclase feldspar with sometimes hornblende and mica.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Trachytic</ent><br/
<hw>Tra*chyt"ic</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[Cf. F. <ets>trachytique</ets>.]</ety> <def>Of, pertaining to, or resembling, trachyte.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Trachytoid</ent><br/
<hw>Trach"y*toid</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[<ets>Trachyte</ets> + <ets>-oid</ets>.]</ety> <fld>(Min.)</fld> <def>Resembling trachyte; -- used to define the structure of certain rocks.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tracing</ent><br/
<hw>Tra"cing</hw> <pr>(?)</pr>, <pos>n.</pos> <sn>1.</sn> <def>The act of one who traces; especially, the act of copying by marking on thin paper, or other transparent substance, the lines of a pattern placed beneath; also, the copy thus producted.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A regular path or track; a course.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><mcol><col><b>Tracing cloth</b></col>, <col><b>Tracing paper</b></col></mcol>, <cd>specially prepared transparent cloth or paper, which enables a drawing or print to be clearly seen through it, and so allows the use of a pen or pencil to produce a facsimile by following the lines of the original placed beneath.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Track</ent><br/
<hw>Track</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[OF. <ets>trac</ets> track of horses, mules, trace of animals; of Teutonic origin; cf.D. <ets>trek</ets> a drawing, <ets>trekken</ets> to draw, travel, march, MHG. <ets>trechen</ets>, pret. <ets>trach</ets>.  Cf. <er>Trick</er>.]</ety> <sn>1.</sn> <def>A mark left by something that has passed along; <as>as, the <ex>track</ex>, or wake, of a ship; the <ex>track</ex> of a meteor; the <ex>track</ex> of a sled or a wheel.</as></def><br/
[<source>1913 Webster</source>]</p>

<p><q>The bright <qex>track</qex> of his fiery car.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A mark or impression left by the foot, either of man or beast; trace; vestige; footprint.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Far from <qex>track</qex> of men.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <fld>(Zool.)</fld> <def>The entire lower surface of the foot; -- said of birds, etc.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>A road; a beaten path.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Behold Torquatus the same <qex>track</qex> pursue.</q> <rj><qau>Dryden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <def>Course; way; <as>as, the <ex>track</ex> of a comet</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>6.</sn> <def>A path or course laid out for a race, for exercise, etc.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>7.</sn> <fld>(Railroad)</fld> <def>The permanent way; the rails.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>8.</sn> <ety>[Perhaps a mistake for <ets>tract</ets>.]</ety> <def>A tract or area, as of land.</def> <mark>[Obs.]</mark> <ldquo/Small <xex>tracks</xex> of ground.<rdquo/  <rj><au>Fuller.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Track scale</b></col>, <cd>a railway scale. See under <er>Railway</er>.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Track</ent><br/
<hw>Track</hw>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>tracked</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>tracking</conjf>.]</vmorph> <def>To follow the tracks or traces of; to pursue by following the marks of the feet; to trace; to trail; <as>as, to <ex>track</ex> a deer in the snow</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>It was often found impossible to <qex>track</qex> the robbers to their retreats among the hills and morasses.</q> <rj><qau>Macaulay.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Naut.)</fld> <def>To draw along continuously, as a vessel, by a line, men or animals on shore being the motive power; to tow.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Trackage</ent><br/
<hw>Track"age</hw> <pr>(?)</pr>, <pos>n.</pos> <sn>1.</sn> <def>The act of tracking, or towing, as a boat; towage.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Railroads)</fld> <def>Lines of track, collectively; <as>as, an extensive <ex>trackage</ex></as>.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Tracker</ent><br/
<hw>Track"er</hw> <pr>(?)</pr>, <pos>n.</pos> <sn>1.</sn> <def>One who, or that which, tracks or pursues, as a man or dog that follows game.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>And of the <qex>trackers</qex> of the deer<br/
Scarce half the lessening pack was near.</q> <rj><qau>Sir W. Scott.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Mus.)</fld> <def>In the organ, a light strip of wood connecting (in path) a key and a pallet, to communicate motion by pulling.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tracklayer</ent><br/
<hw>Track"lay`er</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Railroads)</fld> <def>Any workman engaged in work involved in putting the track in place.</def> <mark>[U. S. & Canada]</mark> -- <wordforms><wf>Track"lay`ing</wf>, <pos>n.</pos></wordforms><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Trackless</ent><br/
<hw>Track"less</hw>, <pos>a.</pos> <def>Having no track; marked by no footsteps; untrodden; <as>as, a <ex>trackless</ex> desert</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>To climb the <qex>trackless</qex> mountain all unseen.</q> <rj><qau>Byron.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p>-- <wordforms><wf>Track"less*ly</wf>, <pos>adv.</pos>-<wf>Track"less*ness</wf>, <pos>n.</pos></wordforms><br/
[<source>1913 Webster</source>]</p>

<p><ent>Trackman</ent><br/
<hw>Track"man</hw> <pr>(?)</pr>, <pos>n.</pos>; <plu>pl. <plw>-men</plw> <pr>(<?/)</pr></plu>. <fld>(Railroads)</fld> <def>One employed on work on the track; specif., a trackwalker.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Trackmaster</ent><br/
<hw>Track"mas`ter</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Railroad)</fld> <def>One who has charge of the track; -- called also <altname>roadmaster</altname>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Track-road</ent><br/
<hw>Track"-road`</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A towing path.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Trackscout</ent><br/
<hw>Track"scout</hw> <pr>(?)</pr>, <pos>n.</pos> <def>See <er>Trackschuyt</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Trackwalker</ent><br/
<hw>Track"walk`er</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Railroads)</fld> <def>A person employed to walk over and inspect a section of tracks.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Trackway</ent><br/
<hw>Track"way`</hw> <pr>(?)</pr>, <pos>n.</pos> <def>Any of two or more narrow paths, of steel, smooth stone, or the like, laid in a public roadway otherwise formed of an inferior pavement, as cobblestones, to provide an easy way for wheels.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Tract</ent><br/
<hw>Tract</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Abbrev.fr. <ets>tractate</ets>.]</ety> <def>A written discourse or dissertation, generally of short extent; a short treatise, especially on practical religion.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The church clergy at that time writ the best collection of <qex>tracts</qex> against popery that ever appeared.</q> <rj><qau>Swift.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Tracts for the Times</b></col>. <cd>See <er>Tractarian</er>.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tract</ent><br/
<hw>Tract</hw>, <pos>n.</pos> <ety>[L. <ets>tractus</ets> a drawing, train, track, course, tract of land, from <ets>trahere tractum</ets>, to draw. Senses 4 and 5 are perhaps due to confusion with <ets>track</ets>. See <er>Trace</er>,<pos>v.</pos>, and cf. <er>Tratt</er>.]</ety> <sn>1.</sn> <def>Something drawn out or extended; expanse.</def> <ldquo/The deep <xex>tract</xex> of hell.<rdquo/  <rj><au>Milton.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A region or quantity of land or water, of indefinite extent; an area; <as>as, an unexplored <ex>tract</ex> of sea</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>A very high mountain joined to the mainland by a narrow <qex>tract</qex> of earth.</q> <rj><qau>Addison.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>Traits; features; lineaments.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>The discovery of a man's self by the <qex>tracts</qex> of his countenance is a great weakness.</q> <rj><qau>Bacon.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>The footprint of a wild beast.</def> <mark>[Obs.]</mark>  <rj><au>Dryden.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <def>Track; trace.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>Efface all <qex>tract</qex> of its traduction.</q> <rj><qau>Sir T. Browne.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>But flies an eagle flight, bold, and forthon,<br/
Leaving no <qex>tract</qex> behind.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>6.</sn> <def>Treatment; exposition.</def> <mark>[Obs.]</mark>  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>7.</sn> <def>Continuity or extension of anything; <as>as, the <ex>tract</ex> of speech</as>.</def> <mark>[Obs.]</mark>  <rj><au>Older.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>8.</sn> <def>Continued or protracted duration; length; extent.</def> <ldquo/Improved by <xex>tract</xex> of time.<rdquo/  <rj><au>Milton.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>9.</sn> <fld>(R. C. Ch.)</fld> <def>Verses of Scripture sung at Mass, instead of the Alleluia, from Septuagesima Sunday till the Saturday befor Easter; -- so called because sung <xex>tractim</xex>, or without a break, by one voice, instead of by many as in the antiphons.</def><br/
[<source>1913 Webster</source>]</p>

<p><syn><b>Syn.</b> -- Region; district; quarter; essay; treatise; dissertation.</syn><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tract</ent><br/
<hw>Tract</hw>, <pos>v. t.</pos> <def>To trace out; to track; also, to draw out; to protact.</def> <mark>[Obs.]</mark>  <rj><au>Spenser.</au>  <au>B. Jonson.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tractability</ent><br/
<hw>Tract`a*bil"i*ty</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>tractabilitas</ets>: cf.F. <ets>tractabilite</ets>.]</ety> <def>The quality or state of being tractable or docile; docility; tractableness.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tractable</ent><br/
<hw>Tract"a*ble</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>tractabilis</ets>, fr, <ets>tractare</ets> to draw violently, to handle, treat. See <er>Treat</er>, <pos>v. t.</pos>]</ety> <sn>1.</sn> <def>Capable of being easily led, taught, or managed; docile; manageable; governable; <as>as, <ex>tractable</ex> children; a <ex>tractable</ex> learner</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>I shall find them <qex>tractable</qex> enough.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Capable of being handled; palpable; practicable; feasible; <as>as, <ex>tractable</ex> measures</as>.</def> <mark>[Obs.]</mark>  <rj><au>Holder.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p>--<wordforms><wf>Tract"a*ble*ness</wf>, <pos>n.</pos> -- <wf>Tract"a/bly</wf>, <pos>adv.</pos></wordforms><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tractarian</ent><br/
<hw>Trac*ta"ri*an</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Ch. of England)</fld> <def>One of the writers of the Oxford tracts, called <ldquo/<xex>Tracts</xex> for the Times,<rdquo/ issued during the period 1833-1841, in which series of papers the sacramental system and authority of the Church, and the value of tradition, were brought into prominence. Also, a member of the High Church party, holding generally the principles of the Tractarian writers; a Puseyite.</def><br/
[<source>1913 Webster</source>]</p>

<p><-- p. 1525 --><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tractarian</ent><br/
<hw>Trac*ta"ri*an</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Of or pertaining to the Tractarians, or their principles.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tractarianism</ent><br/
<hw>Trac*ta"ri*an*ism</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Ch. of England)</fld> <def>The principles of the Tractarians, or of those persons accepting the teachings of the <ldquo/Tracts for the Times.<rdquo/</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tractate</ent><br/
<hw>Tract"ate</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>tractatus</ets> a touching, handling, treatise. See Tractable, and <er>Tract</er> a treatise, <er>Treaty</er>.]</ety> <def>A treatise; a tract; an essay.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Agreeing in substance with Augustin's, from whose fourteenth <qex>Tractate</qex> on St. John the words are translated.</q> <rj><qau>Hare.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tractation</ent><br/
<hw>Trac*ta"tion</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>tractatio</ets>.]</ety> <def>Treatment or handling of a subject; discussion.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>A full <qex>tractation</qex> of the points controverted.</q> <rj><qau>Bp. Hall.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tractator</ent><br/
<hw>Trac*ta"tor</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L., a handler.]</ety> <def>One who writes tracts; specif., a Tractarian.</def> <mark>[R.]</mark>  <rj><au>C. Kingsley.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tractile</ent><br/

<hw>Tract"ile</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>trahere</ets>, <ets>tractum</ets>, to draw.]</ety> <def>Capable of being drawn out in length; ductile.</def>  <rj><au>Bacon.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tractility</ent><br/
<hw>Trac*til"i*ty</hw> <pr>(?)</pr>, <pos>n.</pos> <def>The quality of being tractile; ductility.</def>  <rj><au>Derham.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Traction</ent><br/
<hw>Trac"tion</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>trahere</ets>, <ets>tractum</ets>, to draw: cf. F. <ets>traction</ets>.]</ety> <sn>1.</sn> <def>The act of drawing, or the state of being drawn; <as>as, the <ex>traction</ex> of a muscle</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Specifically, the act of drawing a body along a plane by motive power, as the drawing of a carriage by men or horses, the towing of a boat by a tug.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>Attraction; a drawing toward.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>The adhesive friction of a wheel on a rail, a rope on a pulley, or the like; <as>as, the car is stuck in the snow because it can;t get any <ex>traction</ex></as>.</def>  <rj><au>Knight.</au></rj><br/
[<source>1913 Webster</source> <source>+PJC</source>]</p>

<p><cs><col><b>Angle of traction</b></col> <fld>(Mech.)</fld>, <cd>the angle made with a given plane by the line of direction in which a tractive force acts.</cd> -- <col><b>Traction engine</b></col>, <cd>a locomotive for drawing vehicles on highways or in the fields.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tractional</ent><br/
<hw>Trac"tion*al</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Of or relating to traction.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Traction wheel</ent><br/
<hw>Traction wheel</hw>. <fld>(Mach.)</fld> <sd>(a)</sd> <def>A locomotive driving wheel which acts by friction adhesion to a smooth track.</def> <sd>(b)</sd> <def>A smooth-rimmed friction wheel for giving motion to an endless link belt or the like.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Tractite</ent><br/
<hw>Tract"ite</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A Tractarian.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tractitious</ent><br/
<hw>Trac*ti"tious</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[See <er>Tractate</er>.]</ety> <def>Treating of; handling.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tractive</ent><br/
<hw>Tract"ive</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Serving to draw; pulling; attracting; <as>as, <ex>tractive</ex> power</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tractor</ent><br/
<hw>Tract"or</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[NL., from L. <ets>trahere</ets>, <ets>tractum</ets>, to draw.]</ety> <sn>1.</sn> <def>That which draws, or is used for drawing.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <pluf>pl.</pluf> <fld>(Med.)</fld> <def>Two small, pointed rods of metal, formerly used in the treatment called Perkinism.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>a motor vehicle with a powerful engine and large wheels, used to pull farm equipment such as plows, harvesting machinery, etc., over farm land.  Smaller forms of farm <ex>tractor</ex> are completely open, with only a seat for the driver.</def><br/
[<source>PJC</source>]</p>

<p><sn>4.</sn> <def>a motor vehicle with a driver's cab and a powerful engine, fitted with a coupling at the rear, and designed to pull a large trailer or semi-trailer on the open highway; -- called also <altname>truck tractor</altname>.  The combination of such a tractor with its trailer is called a <partof>tractor-trailer</partof>, and is a common form of truck used for commercial shipping of freight.</def><br/
[<source>PJC</source>]</p>

<p><sn>5.</sn> <fld>(Aviation)</fld> <def>A propeller screw placed in front of the supporting planes of an aeroplane instead of behind them, so that it exerts a pull instead of a push; -- called also <altname>Tractor screw</altname> and <altname>Tractor propeller</altname>.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><sn>6.</sn> <fld>(Aviation)</fld> <def>An airplane having one or more tractor propellers; -- called also <altname>tractor airplane</altname>.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Tractor airplane</ent><br/
<hw>Trac"tor air"plane</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Aviation)</fld> <def>An airplane having one or more tractor propellers; -- called also <altname>tractor</altname>.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Tractoration</ent><br/
<hw>Trac`to*ra"tion</hw> <pr>(?)</pr>, <pos>n.</pos> <def>See <er>Perkinism</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tractor propeller</ent><br/
<ent>Tractor screw</ent><br/
<mhw>{ <hw>Tractor screw</hw> <it>or</it>  <hw>Tractor propeller</hw> }</mhw>. <fld>(Aviation)</fld> <def>A propeller screw placed in front of the supporting planes of an airplane instead of behind them, so that it exerts a pull instead of a push. Hence, <col><b>Tractor monoplane</b></col>, <col><b>Tractor biplane</b></col>, etc.  This is the most common arrangement for propellers on modern airplanes.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Tractor-trailer</ent><br/
<hw>Tract"o*r-trail"er</hw> <pr>(?)</pr>, <pos>n.</pos> <def>a combination of a tractor{4} hooked up to a trailer, forming a common type of <isa>truck{7}</isa> used for carrying freight on highways.</def> <ldquo/Traffic was backed up for miles by a jackknifed <ex>tractor-trailer</ex>.<rdquo/<br/
[<source>PJC</source>]</p>

<p><ent>Tractory</ent><br/
<hw>Tract"o*ry</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>tractorius</ets> of drawing, fr. <ets>trahere</ets>, <ets>tractum</ets>, to draw.]</ety> <fld>(Geom.)</fld> <def>A tractrix.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tractrix</ent><br/
<hw>Tract"rix</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[NL. See <er>Tractor</er>.]</ety> <fld>(Geom.)</fld> <def>A curve such that the part of the tangent between the point of tangency and a given straight line is constant; -- so called because it was conceived as described by the motion of one end of a tangent line as the other end was drawn along the given line.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Trad</ent><br/
<hw>Trad</hw> <pr>(?)</pr>, <mark>obs.</mark> <def><pos>imp.</pos> of <er>Tread</er>.</def>  <rj><au>Chaucer.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Trade</ent><br/
<hw>Trade</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Formerly, a path, OE. <ets>tred</ets> a footmark. See <er>Tread</er>, <pos>n.</pos> & <pos>v.</pos>]</ety> <sn>1.</sn> <def>A track; a trail; a way; a path; also, passage; travel; resort.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>A postern with a blind wicket there was,<br/
A common <qex>trade</qex> to pass through Priam's house.</q> <rj><qau>Surrey.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Hath tracted forth some salvage beastes <qex>trade</qex>.</q> <rj><qau>Spenser.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Or, I'll be buried in the king's highway,<br/
Some way of common <qex>trade</qex>, where subjects' feet<br/
May hourly trample on their sovereign's head.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Course; custom; practice; occupation; employment.</def> <mark>[Obs.]</mark> <ldquo/The right <xex>trade</xex> of religion.<rdquo/  <rj><au>Udall.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>There those five sisters had continual <qex>trade</qex>.</q> <rj><qau>Spenser.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Long did I love this lady,<br/
Long was my travel, long my <qex>trade</qex> to win her.</q> <rj><qau>Massinger.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Thy sin's not accidental but a <qex>trade</qex>.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>Business of any kind; matter of mutual consideration; affair; dealing.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>Have you any further <qex>trade</qex> with us?</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>Specifically: The act or business of exchanging commodities by barter, or by buying and selling for money; commerce; traffic; barter.</def><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ Trade comprehends every species of exchange or dealing, either in the produce of land, in manufactures, in bills, or in money; but it is chiefly used to denote the barter or purchase and sale of goods, wares, and merchandise, either by wholesale or retail. Trade is either <xex>foreign</xex> or <xex>domestic</xex>. <xex>Foreign</xex> trade consists in the exportation and importation of goods, or the exchange of the commodities of different countries. <xex>Domestic</xex>, or <xex>home</xex>, trade is the exchange, or buying and selling, of goods within a country. Trade is also by the <xex>wholesale</xex>, that is, by the package or in large quantities, generally to be sold again, or it is by <xex>retail</xex>, or in small parcels. The <xex>carrying</xex> trade is the business of transporting commodities from one country to another, or between places in the same country, by land or water.</note><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <def>The business which a person has learned, and which he engages in, for procuring subsistence, or for profit; occupation; especially, mechanical employment as distinguished from the liberal arts, the learned professions, and agriculture; <as>as, we speak of the <ex>trade</ex> of a smith, of a carpenter, or mason, but not now of the <ex>trade</ex> of a farmer, or a lawyer, or a physician</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Accursed usury was all his <qex>trade</qex>.</q> <rj><qau>Spenser.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>The homely, slighted, shepherd's <qex>trade</qex>.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>I will instruct thee in my <qex>trade</qex>.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>6.</sn> <def>Instruments of any occupation.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>The house and household goods, his <qex>trade</qex> of war.</q> <rj><qau>Dryden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>7.</sn> <def>A company of men engaged in the same occupation; thus, booksellers and publishers speak of the customs of the <xex>trade</xex>, and are collectively designated as <xex>the trade</xex>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>8.</sn> <pluf>pl.</pluf> <def>The trade winds.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>9.</sn> <def>Refuse or rubbish from a mine.</def> <mark>[Prov. Eng.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><syn><b>Syn.</b> -- Profession; occupation; office; calling; avocation; employment; commerce; dealing; traffic.</syn><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Board of trade</b></col>. <cd>See under <er>Board</er>.</cd> -- <col><b>Trade dollar</b></col>. <cd>See under <er>Dollar</er>.</cd> -- <col><b>Trade price</b></col>, <cd>the price at which goods are sold to members of the same trade, or by wholesale dealers to retailers.</cd> -- <col><b>Trade sale</b></col>, <cd>an auction by and for the trade, especially that of the booksellers.</cd> -- <col><b>Trade wind</b></col>, <cd>a wind in the torrid zone, and often a little beyond at, which blows from the same quarter throughout the year, except when affected by local causes; -- so called because of its usefulness to navigators, and hence to trade.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ The general direction of the trade winds is from N. E. to S. W. on the north side of the equator, and from S. E. to N. W. on the south side of the equator. They are produced by the joint effect of the rotation of the earth and the movement of the air from the polar toward the equatorial regions, to supply the vacancy caused by heating, rarefaction, and consequent ascent of the air in the latter regions. The trade winds are principally limited to two belts in the tropical regions, one on each side of the equator, and separated by a belt which is characterized by calms or variable weather.</note><br/
[<source>1913 Webster</source>]</p>

<p><ent>Trade</ent><br/
<hw>Trade</hw> <pr>(?)</pr>, <pos>v. i.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Traded</conjf>; <pos>p. pr. & vb. n.</pos> <conjf>Trading</conjf>.]</vmorph> <sn>1.</sn> <def>To barter, or to buy and sell; to be engaged in the exchange, purchase, or sale of goods, wares, merchandise, or anything else; to traffic; to bargain; to carry on commerce as a business.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>A free port, where nations . . . resorted with their goods and <qex>traded</qex>.</q> <rj><qau>Arbuthnot.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To buy and sell or exchange property in a single instance.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>To have dealings; to be concerned or associated; -- usually followed by <xex>with</xex>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>How did you dare to <qex>trade</qex> and traffic with Macbeth?</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Trade</ent><br/
<hw>Trade</hw>, <pos>v. t.</pos> <def>To sell or exchange in commerce; to barter.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>They <qex>traded</qex> the persons of men.</q> <rj><qau>Ezek. xxvii. 13.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>To dicker and to swop, to <qex>trade</qex> rifles and watches.</q> <rj><qau>Cooper.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Trade</ent><br/
<hw>Trade</hw>, <mark>obs.</mark> <def><pos>imp.</pos> of <er>Tread</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Traded</ent><br/
<hw>Trad"ed</hw>, <pos>a.</pos> <def>Professional; practiced.</def> <mark>[Obs.]</mark>  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tradeful</ent><br/
<hw>Trade"ful</hw>, <pos>a.</pos> <def>Full of trade; busy in traffic; commercial.</def>  <rj><au>Spenser.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>trade-in</ent><br/
<hw>trade-in</hw> <pos>n.</pos> <sn>1.</sn> <def>an item of property, such as an automobile, that is given in part payment for a new one.</def><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Tradeless</ent><br/
<hw>Trade"less</hw>, <pos>a.</pos> <def>Having no trade or traffic.</def>  <rj><au>Young.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Trade-mark</ent><br/
<hw>Trade"-mark`</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A peculiar distinguishing mark or device affixed by a manufacturer or a merchant to his goods, the exclusive right of using which is recognized by law.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Trade name</ent><br/
<hw>Trade name</hw>. <sn>1.</sn> <sd>(a)</sd> <def>The name by which an article is called among traders, etc.; <as>as, tin spirits is a common <ex>trade name</ex> in the dyeing industry for various solutions of tin salts</as>.</def> <sd>(b)</sd> <def>An invented or arbitrary adopted name given by a manufacturer or merchant to an article to distinguish it as produced or sold by him.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><sn>2.</sn>  <def>The name or style under which a concern or firm does business. This name becomes a part of the good will of a business; it is not protected by the registration acts, but a qualified common-law protection against its misuse exists, analogous to that existing in the case of trade-marks.</def><-- now statutory protection? --><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Trade-off</ent><br/
<ent>Tradeoff</ent><br/
<mhw><hw>Trade"off</hw>, <hw>Trade"-off</hw></mhw> <pr>(?)</pr>, <pos>n.</pos> <sn>1.</sn> <def>the exchange of one thing (object, right, opportunity) for another of approximately equal value, so as to seal a bargain, or effect a compromise.</def><br/
[<source>PJC</source>]</p>

<p><sn>2.</sn> <def>the giving up of one desired objective in order to attain another, when both cannot be achieved at the same time; <as>as, the factory workers viewed the <ex>trade-off</ex> of air quality for jobs as a necessary evil</as>.</def><br/
[<source>PJC</source>]</p>

<p><ent>Trader</ent><br/
<hw>Trad"er</hw> <pr>(?)</pr>, <pos>n.</pos> <sn>1.</sn> <def>One engaged in trade or commerce; one who makes a business of buying and selling or of barter; a merchant; a trafficker; <as>as, a <ex>trader</ex> to the East Indies; a country <ex>trader</ex>.</as></def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A vessel engaged in the coasting or foreign trade.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tradescantia</ent><br/
||<hw>Trad`es*can"ti*a</hw> <pr>(?)</pr>, <pos>prop. n.</pos> <fld>(Bot.)</fld> <def>A genus including spiderwort and Wandering Jew.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tradesfolk</ent><br/
<hw>Trades"folk`</hw> <pr>(?)</pr>, <pos>n.</pos> <def>People employed in trade; tradesmen.</def> <mark>[R.]</mark>  <rj><au>Swift.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tradesman</ent><br/
<hw>Trades"man</hw> <pr>(?)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Tradesmen</plw> <pr>(<?/)</pr>.</plu> <sn>1.</sn> <def>One who trades; a shopkeeper.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A mechanic or artificer; esp., one whose livelihood depends upon the labor of his hands.</def> <mark>[U. S.]</mark>  <rj><au>Burrill.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tradespeople</ent><br/
<hw>Trades"peo`ple</hw> <pr>(?)</pr>, <pos>n.</pos> <def>People engaged in trade; shopkeepers.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Trade union</ent><br/
<ent>trades union</ent><br/
<mhw>{ <hw>trades" un`ion</hw> <pr>(?)</pr>, <it>or</it> <hw>Trade" un`ion</hw>  }</mhw>. <def>An organized combination among workmen for the purpose of maintaining their rights, privileges, and interests with respect to wages, hours of labor, customs, etc.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Trade-unionist</ent><br/
<ent>Trades-unionist</ent><br/
<mhw>{ <hw>Trades"-un`ion*ist</hw>, <hw>Trade"-un`ion*ist</hw>  }</mhw>, <pos>n.</pos> <def>A member of a trades union, or a supporter of trades unions.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tradeswoman</ent><br/
<hw>Trades"wom`an</hw> <pr>(?)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Tradeswomen</plw> <pr>(<?/)</pr>.</plu> <def>A woman who trades, or is skilled in trade.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Trading</ent><br/
<hw>Trad"ing</hw> <pr>(?)</pr>, <pos>a.</pos> <sn>1.</sn> <def>Carrying on trade or commerce; engaged in trade; <as>as, a <ex>trading</ex> company</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Frequented by traders.</def> <mark>[R.]</mark> <ldquo/They on the <xex>trading</xex> flood.<rdquo/  <rj><au>Milton.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>Venal; corrupt; jobbing; <as>as, a <ex>trading</ex> politician</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tradition</ent><br/
<hw>Tra*di"tion</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[OE. <ets>tradicioun</ets>, L. <ets>traditio</ets>, from <ets>tradere</ets> to give up, transmit. See <er>Treason</er>, <er>Traitor</er>.]</ety> <sn>1.</sn> <def>The act of delivering into the hands of another; delivery.</def> <ldquo/A deed takes effect only from the <xex>tradition</xex> or delivery.<rdquo/  <rj><au>Blackstone.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>The unwritten or oral delivery of information, opinions, doctrines, practices, rites, and customs, from father to son, or from ancestors to posterity; the transmission of any knowledge, opinions, or practice, from forefathers to descendants by oral communication, without written memorials.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>Hence, that which is transmitted orally from father to son, or from ancestors to posterity; knowledge or belief transmitted without the aid of written memorials; custom or practice long observed.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Will you mock at an ancient <qex>tradition</qex> begun upon an honorable respect?</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Naught but <qex>tradition</qex> remains of the beautiful village of Grand-Pr<eacute/.</q> <rj><qau>Longfellow.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <fld>(Theol.)</fld> <sd>(a)</sd> <def>An unwritten code of law represented to have been given by God to Moses on Sinai.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Making the word of God of none effect through your <qex>tradition</qex>, which ye have delivered.</q> <rj><qau>Mark vii. 13.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sd>(b)</sd> <def>That body of doctrine and discipline, or any article thereof, supposed to have been put forth by Christ or his apostles, and not committed to writing.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Stand fast, and hold the <qex>traditions</qex> which ye have been taught, whether by word or our epistle.</q> <rj><qau>2 Thess. ii.  15.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Tradition Sunday</b></col> <fld>(Eccl.)</fld>, <cd>Palm Sunday; -- so called because the creed was then taught to candidates for baptism at Easter.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tradition</ent><br/
<hw>Tra*di"tion</hw>, <pos>v. t.</pos> <def>To transmit by way of tradition; to hand down.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>The following story is . . . <qex>traditioned</qex> with very much credit amongst our English Catholics.</q> <rj><qau>Fuller.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Traditional</ent><br/
<hw>Tra*di"tion*al</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[Cf. F. <ets>traditionnel</ets>, LL. <ets>traditionalis</ets>.]</ety> <sn>1.</sn> <def>Of or pertaining to tradition; derived from tradition; communicated from ancestors to descendants by word only; transmitted from age to age without writing; <as>as, <ex>traditional</ex> opinions; <ex>traditional</ex> customs; <ex>traditional</ex> expositions of the Scriptures.</as></def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Observant of tradition; attached to old customs; old-fashioned.</def> <mark>[R.]</mark>  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Traditionalism</ent><br/
<hw>Tra*di"tion*al*ism</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A system of faith founded on tradition; esp., the doctrine that all religious faith is to be based solely upon what is delivered from competent authority, exclusive of rational processes.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Traditionalist</ent><br/
<hw>Tra*di"tion*al*ist</hw> <pr>(?)</pr>, <pos>n.</pos> <def>An advocate of, or believer in, traditionalism; a traditionist.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Traditionally</ent><br/
<hw>Tra*di"tion*al*ly</hw>, <pos>adv.</pos> <def>In a traditional manner.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Traditionarily</ent><br/
<hw>Tra*di"tion*a*ri*ly</hw> <pr>(?)</pr>, <pos>adv.</pos> <def>By tradition.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Traditionary</ent><br/
<hw>Tra*di"tion*a*ry</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Traditional.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The reveries of the Talmud, a collection of Jewish <qex>traditionary</qex> interpolations.</q> <rj><qau>Buckminster.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Traditionary</ent><br/
<hw>Tra*di"tion*a*ry</hw>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Traditionaries</plw> <pr>(<?/)</pr>.</plu> <ety>[Cf. F. <ets>traditionnare</ets>.]</ety> <def>One, among the Jews, who acknowledges the authority of traditions, and explains the Scriptures by them.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Traditionist</ent><br/
<ent>Traditioner</ent><br/
<mhw>{ <hw>Tra*di"tion*er</hw> <pr>(?)</pr>, <hw>Tra*di"tion*ist</hw>, }</mhw> <pos>n.</pos> <ety>[Cf. F. <ets>traditionniste</ets>.]</ety> <def>One who adheres to tradition.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Traditive</ent><br/
<hw>Trad"i*tive</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>tradere</ets>, <ets>traditum</ets>, to transmit, give up: cf. F. <ets>traditif</ets>.]</ety> <def>Transmitted or transmissible from father to son, or from age, by oral communication; traditional.</def> <mark>[R.]</mark>  <rj><au>Jer. Taylor.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Suppose we on things <qex>traditive</qex> divide.</q> <rj><qau>Dryden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Traditor</ent><br/
||<hw>Trad"i*tor</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L., fr. <ets>tradere</ets>, <ets>traditum</ets>. See <er>Traitor</er>.]</ety> <fld>(Eccl. Hist.)</fld> <def>A deliverer; -- a name of infamy given to Christians who delivered the Scriptures, or the goods of the church, to their persecutors to save their lives.</def>  <rj><au>Milner.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Traduce</ent><br/
<hw>Tra*duce"</hw> <pr>(?)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Traduced</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Traducing</conjf> <pr>(?)</pr>.]</vmorph> <ety>[L. <ets>traducere</ets>, <ets>traductum</ets>, to lead across, lead along, exhibit as a spectacle, disgrace, transfer, derive; <ets>trans</ets> across, over + <ets>ducere</ets> to lead: cf. F. <ets>traduire</ets> to transfer, translate, arraign, fr. L. <ets>traducere.</ets> See <er>Duke</er>.]</ety> <sn>1.</sn> <def>To transfer; to transmit; to hand down; <as>as, to <ex>traduce</ex> mental qualities to one's descendants</as>.</def> <mark>[Obs.]</mark>  <rj><au>Glanvill.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To translate from one language to another; <as>as, to <ex>traduce</ex> and compose works</as>.</def> <mark>[Obs.]</mark>  <rj><au>Golden Boke.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>To increase or distribute by propagation.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>From these only the race of perfect animals were propagated and <qex>traduced</qex> over the earth.</q> <rj><qau>Sir M. Hale.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>To draw away; to seduce.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>I can forget the weakness<br/
Of the <qex>traduced</qex> soldiers.</q> <rj><qau>Beau. & Fl.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <def>To represent; to exhibit; to display; to expose; to make an example of.</def> <mark>[Obs.]</mark>  <rj><au>Bacon.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>6.</sn> <def>To expose to contempt or shame; to represent as blamable; to calumniate; to vilify; to defame.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The best stratagem that Satan hath . . . is by <qex>traducing</qex> the form and manner of them [prayers], to bring them into contempt.</q> <rj><qau>Hooker.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>He had the baseness . . . to <qex>traduce</qex> me in libel.</q> <rj><qau>Dryden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><syn><b>Syn.</b> -- To calumniate; vilify; defame; disparage; detract; depreciate; decry; slander.</syn><br/
[<source>1913 Webster</source>]</p>

<p><ent>Traducement</ent><br/
<hw>Tra*duce"ment</hw> <pr>(?)</pr>, <pos>n.</pos> <def>The act of traducing; misrepresentation; ill-founded censure; defamation; calumny.</def> <mark>[R.]</mark>  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Traducent</ent><br/
<hw>Tra*du"cent</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>traducens</ets>, p. pr. of <ets>traducere</ets>. See <er>Traduce</er>.]</ety> <def>Slanderous.</def> <mark>[R.]</mark>  <rj><au>Entick.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Traducer</ent><br/
<hw>Tra*du"cer</hw> <pr>(?)</pr>, <pos>n.</pos> <sn>1.</sn> <def>One who traduces; a slanderer; a calumniator.</def>  <rj><au>Bp. Hall.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>One who derives or deduces.</def> <mark>[Obs.]</mark>  <rj><au>Fuller.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Traducian</ent><br/
<hw>Tra*du"cian</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A believer in traducianism.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Traducianism</ent><br/
<hw>Tra*du"cian*ism</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Theol.)</fld> <def>The doctrine that human souls are produced by the act of generation; -- opposed to <xex>creationism</xex>, and <xex>infusionism</xex>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Traducible</ent><br/
<hw>Tra*du"ci*ble</hw>, <pos>a.</pos> <sn>1.</sn> <def>Capable of being derived or propagated.</def> <mark>[Obs.]</mark>  <rj><au>Sir M. Hale.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Capable of being traduced or calumniated.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Traducingly</ent><br/
<hw>Tra*du"cing*ly</hw>, <pos>adv.</pos> <def>In a traducing manner; by traduction; slanderously.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Traduct</ent><br/
<hw>Tra*duct"</hw> <pr>(?)</pr>, <pos>v. t.</pos> <ety>[L. <ets>traducere</ets>, <ets>traductum</ets>. See Traduce.]</ety> <def>To derive or deduce; also, to transmit; to transfer.</def> <mark>[Obs.]</mark>  <rj><au>Fotherby.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Traduct</ent><br/
<hw>Tra*duct"</hw>, <pos>n.</pos> <def>That which is traducted; that which is transferred; a translation.</def> <mark>[Obs.]</mark>  <rj><au>Howell.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Traduction</ent><br/
<hw>Tra*duc"tion</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>traductio</ets> a transferring: cf. F. <ets>traduction</ets> translation. See <er>Traduce</er>.]</ety> <sn>1.</sn> <def>Transmission from one to another.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>Traditional communication and <qex>traduction</qex> of truths.</q> <rj><qau>Sir M. Hale.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Translation from one language to another.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>Derivation by descent; propagation.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>If by <qex>traduction</qex> came thy mind,<br/
Our wonder is the less to find<br/
A soul so charming from a stock so good.</q> <rj><qau>Dryden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>The act of transferring; conveyance; transportation.</def> <mark>[R.]</mark> <ldquo/The <xex>traduction</xex> of brutes.<rdquo/  <rj><au>Sir M. Hale.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <def>Transition.</def> <mark>[Obs.]</mark>  <rj><au>Bacon.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>6.</sn> <fld>(Logic)</fld> <def>A process of reasoning in which each conclusion applies to just such an object as each of the premises applies to.</def>  <rj><au>Jevons.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Traductive</ent><br/
<hw>Tra*duc"tive</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Capable of being deduced; derivable.</def> <mark>[R.]</mark>  <rj><au>Bp. Warburton.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Traffic</ent><br/
<hw>Traf"fic</hw> <pr>(?)</pr>, <pos>v. i.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Trafficked</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Trafficking</conjf> <pr>(?)</pr>.]</vmorph> <ety>[F. <ets>trafiquer</ets>; cf. It. <ets>trafficare</ets>, Sp. <ets>traficar</ets>, <ets>trafagar</ets>, Pg. <ets>traficar</ets>, <ets>trafegar</ets>, <ets>trafeguear</ets>, LL. <ets>traficare</ets>; of uncertain origin, perhaps fr. L. <ets>trans</ets> across, over + <ets>-ficare</ets> to make (see <er>-fy</er>, and cf. G. <ets><uum/bermachen</ets> to transmit, send over, e. g., money, wares); or cf. Pg. <ets>trasfegar</ets> to pour out from one vessel into another, OPg. also, to traffic, perhaps fr. (assumed) LL. <ets>vicare</ets> to exchange, from L. <ets>vicis</ets> change (cf. <er>Vicar</er>).]</ety> <sn>1.</sn> <def>To pass goods and commodities from one person to another for an equivalent in goods or money; to buy or sell goods; to barter; to trade.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To trade meanly or mercenarily; to bargain.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Traffic</ent><br/
<hw>Traf"fic</hw>, <pos>v. t.</pos> <def>To exchange in traffic; to effect by a bargain or for a consideration.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Traffic</ent><br/
<hw>Traf"fic</hw>, <pos>n.</pos> <ety>[Cf. F. <ets>trafic</ets>, It. <ets>traffico</ets>, Sp. <ets>tr<aacute/fico</ets>, <ets>tr<aacute/fago</ets>, Pg. <ets>tr<aacute/fego</ets>, LL. <ets>traficum</ets>, <ets>trafica</ets>. See <er>Traffic</er>, <pos>v.</pos>]</ety> <sn>1.</sn> <def>Commerce, either by barter or by buying and selling; interchange of goods and commodities; trade.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>A merchant of great <qex>traffic</qex> through the world.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>The <qex>traffic</qex> in honors, places, and pardons.</q> <rj><qau>Macaulay.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ This word, like <xex>trade</xex>, comprehends every species of dealing in the exchange or passing of goods or merchandise from hand to hand for an equivalent, unless the business of relating may be excepted. It signifies appropriately foreign trade, but is not limited to that.</note><br/
[<source>1913 Webster</source>]</p>

<p><-- p. 1526 --><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Commodities of the market.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>You 'll see a draggled damsel<br/
From Billingsgate her fishy <qex>traffic</qex> bear.</q> <rj><qau>Gay.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>The business done upon a railway, steamboat line, etc., with reference to the number of passengers or the amount of freight carried.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Traffic return</b></col>, <cd>a periodical statement of the receipts for goods and passengers, as on a railway line.</cd> -- <col><b>Traffic taker</b></col>, <cd>a computer of the returns of traffic on a railway, steamboat line, etc.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Trafficable</ent><br/
<hw>Traf"fic*a*ble</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Capable of being disposed of in traffic; marketable.</def> <mark>[Obs.]</mark>  <rj><au>Bp. Hall.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Trafficker</ent><br/
<hw>Traf"fick*er</hw> <pr>(?)</pr>, <pos>n.</pos> <def>One who traffics, or carries on commerce; a trader; a merchant.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Trafficless</ent><br/
<hw>Traf"fic*less</hw>, <pos>a.</pos> <def>Destitute of traffic, or trade.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Traffic mile</ent><br/
<hw>Traf"fic mile</hw>. <fld>(Railroad Accounting)</fld> <def>Any unit of the total obtained by adding the passenger miles and ton miles in a railroad's transportation for a given period; -- a term and practice of restricted or erroneous usage.</def></p>

<p><q><qex>Traffic mile</qex> is a term designed to furnish an excuse for the  erroneous practice of adding together two things (ton miles and passenger miles) which, being of different kinds, cannot properly be added.</q>  <rj><qau>Hadley.</qau></rj><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Tragacanth</ent><br/
<hw>Trag"a*canth</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>tragacanthum</ets> tragacanth, <ets>tragacantha</ets> the plant producing tragacanth, Gr. <?/  <?/ a he-goat + <?/ a thorn: cf. F. <ets>tragacanthe</ets>.]</ety> <def>A kind of gum procured from a spiny leguminous shrub (<spn>Astragalus gummifer</spn>) of Western Asia, and other species of Astragalus. It comes in hard whitish or yellowish flakes or filaments, and is nearly insoluble in water, but slowly swells into a mucilaginous mass, which is used as a substitute for gum arabic in medicine and the arts. Called also <altname>gum tragacanth</altname>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tragedian</ent><br/
<hw>Tra*ge"di*an</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Cf. F. <ets>trag<eacute/dien</ets>.]</ety><br/
[<source>1913 Webster</source>]</p>

<p><sn>1.</sn> <def>A writer of tragedy.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Thence what the lofty, grave, <qex>tragedians</qex> taught.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>An actor or player in tragedy.</def>  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tragedienne</ent><br/
||<hw>Tra`g<eacute/`dienne"</hw> <pr>(tr<adot/`zh<asl/`dy<ecr/n"; Eng. tr<adot/*j<emac/"d<icr/*<ecr/n`)</pr>, <pos>n.</pos> <ety>[F.]</ety> <def>A woman who plays in tragedy.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tragedious</ent><br/
<hw>Tra*ge"di*ous</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Like tragedy; tragical.</def> <mark>[Obs.]</mark> <ldquo/<xex>Tragedious</xex> history.<rdquo/  <rj><au>Fabyan.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tragedy</ent><br/
<hw>Trag"e*dy</hw> <pr>(?)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Tragedies</plw> <pr>(#)</pr>.</plu> <ety>[OE. <ets>tragedie</ets>, OF. <ets>tragedie</ets>, F. <ets>trag<eacute/die</ets>, L. <ets>tragoedia</ets>, Gr. <?/, fr. <?/ a tragic poet and singer, originally, a goat singer; <?/ a goat (perhaps akin to <?/ to gnaw, nibble, eat, and E. <ets>trout</ets>) + <?/ to sing; from the oldest tragedies being exhibited when a goat was sacrificed, or because a goat was the prize, or because the actors were clothed in goatskins. See <er>Ode</er>.]</ety><br/
[<source>1913 Webster</source>]</p>

<p><sn>1.</sn> <def>A dramatic poem, composed in elevated style, representing a signal action performed by some person or persons, and having a fatal issue; that species of drama which represents the sad or terrible phases of character and life.</def><br/
[<source>1913 Webster</source>]</p>

<p><q><qex>Tragedy</qex> is to say a certain storie,<br/
As olde bookes maken us memorie,<br/
Of him that stood in great prosperitee<br/
And is yfallen out of high degree<br/
Into misery and endeth wretchedly.</q> <rj><qau>Chaucer.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>All our <qex>tragedies</qex> are of kings and princes.</q> <rj><qau>Jer. Taylor.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q><qex>tragedy</qex> is poetry in its deepest earnest; comedy is poetry in unlimited jest.</q> <rj><qau>Coleridge.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A fatal and mournful event; any event in which human lives are lost by human violence, more especially by unauthorized violence.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tragical</ent><br/
<ent>Tragic</ent><br/
<mhw>{ <hw>Trag"ic</hw> <pr>(?)</pr>, <hw>Trag"ic*al</hw> <pr>(?)</pr>, }</mhw> <pos>a.</pos> <ety>[L. <ets>tragicus</ets>, Gr.<?/: cf. F. <ets>tragique</ets>.]</ety> <sn>1.</sn> <def>Of or pertaining to tragedy; of the nature or character of tragedy; <as>as, a <ex>tragic</ex> poem; a <ex>tragic</ex> play or representation</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Fatal to life; mournful; terrible; calamitous; <as>as, the <ex>tragic</ex> scenes of the French revolution</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>Mournful; expressive of tragedy, the loss of life, or of sorrow.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Why look you still so stern and <qex>tragical</qex> ?</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p>-- <wordforms><wf>Trag"ic*al*ly</wf>, <pos>adv.</pos> -- <wf>Trag"ic*al*ness</wf>, <pos>n.</pos></wordforms><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tragic</ent><br/
<hw>Trag"ic</hw> <pr>(?)</pr>, <pos>n.</pos> <sn>1.</sn> <def>A writer of tragedy.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A tragedy; a tragic drama.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tragi-comedy</ent><br/
<hw>Trag`i-com"e*dy</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Cf. F. <ets>tragicom<eacute/die</ets>, L. <ets>tragicocomoedia</ets>. See <er>Tragic</er>, and <er>Comedy</er>.]</ety> <def>A kind of drama representing some action in which serious and comic scenes are blended; a composition partaking of the nature both of tragedy and comedy.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The noble <qex>tragi-comedy</qex> of  <ldquo/Measure for Measure.<rdquo/</q> <rj><qau>Macaulay.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tragi-comical</ent><br/
<ent>Tragi-comic</ent><br/
<mhw>{ <hw>Trag`i-com"ic</hw> <pr>(?)</pr>, <hw>Trag`i-com"ic*al</hw> <pr>(?)</pr>, }</mhw> <pos>a.</pos> <ety>[Cf. F. <ets>tragi-comique</ets>.]</ety> <def>Of or pertaining to tragi-comedy; partaking of grave and comic scenes.</def> -- <wordforms><wf>Trag`-com"ic*al*ly</wf>, <pos>adv.</pos></wordforms><br/
[<source>1913 Webster</source>]</p>

<p><q>Julian felt toward him that <qex>tragi-comic</qex> sensation which makes us pity the object which excites it not the less that we are somewhat inclined to laugh amid our sympathy.</q> <rj><qau>Sir W. Scott.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tragi-comi-pastoral</ent><br/
<hw>Trag`i-com`i-pas"tor*al</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Partaking of the nature of, or combining, tragedy, comedy, and pastoral poetry.</def> <mark>[R.]</mark>  <rj><au>Gay.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tragopan</ent><br/
<hw>Trag"o*pan</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[NL., fr. L. <ets>tragopan</ets> a fabulous Ethiopian bird, Gr. <?/.]</ety> <fld>(Zool.)</fld> <def>Any one of several species of Asiatic pheasants of the genus <gen>Ceriornis</gen>. They are brilliantly colored with a variety of tints, the back and breast are usually covered with white or buff ocelli, and the head is ornamented with two bright-colored, fleshy wattles. The crimson tragopan, or horned pheasant (<spn>Ceriornis satyra</spn>), of India is one of the best-known species.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tragus</ent><br/
||<hw>Tra"gus</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[NL., fr. Gr. <?/ a part of the inner ear.]</ety> <fld>(Anat.)</fld> <def>The prominence in front of the external opening of the ear. See <xex>Illust.</xex> under <er>Ear</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>T rail</ent><br/
<hw>T" rail`</hw> <pr>(?)</pr>. <def>See under T.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Trail</ent><br/
<hw>Trail</hw> <pr>(tr<amac/l)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Trailed</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Trailing</conjf>.]</vmorph> <ety>[OE. <ets>trailen</ets>, OF. <ets>trailler</ets> to trail a deer, or hunt him upon a cold scent, also, to hunt or pursue him with a limehound, F. <ets>trailler</ets> to trail a fishing line; probably from a derivative of L. <ets>trahere</ets> to draw; cf. L. <ets>traha</ets> a drag, sledge, <ets>tragula</ets> a kind of drag net, a small sledge, Sp. <ets>trailla</ets> a leash, an instrument for leveling the ground, D. <ets>treilen</ets> to draw with a rope, to tow, <ets>treil</ets> a rope for drawing a boat. See <er>Trace</er>, <pos>v. t.</pos>]</ety><br/
[<source>1913 Webster</source>]</p>

<p><sn>1.</sn> <sd>(a)</sd> <def>To hunt by the track; to track.</def> <sd>(b)</sd> <def>to follow behind.</def> <sd>(c)</sd> <def>To pursue.</def>  <rj><au>Halliwell.</au></rj><br/
[<source>1913 Webster</source> <source>+PJC</source>]</p>

<p><sn>2.</sn> <def>To draw or drag, as along the ground.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>And hung his head, and <qex>trailed</qex> his legs along.</q> <rj><qau>Dryden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>They shall not <qex>trail</qex> me through their streets<br/
Like a wild beast.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Long behind he <qex>trails</qex> his pompous robe.</q> <rj><qau>Pope.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <fld>(Mil.)</fld> <def>To carry, as a firearm, with the breech near the ground and the upper part inclined forward, the piece being held by the right hand near the middle.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>To tread down, as grass, by walking through it; to lay flat.</def>  <rj><au>Longfellow.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <def>To take advantage of the ignorance of; to impose upon.</def> <mark>[Prov. Eng.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>I presently perceived she was (what is vernacularly termed) <qex>trailing</qex> Mrs. Dent; that is, playing on her ignorance.</q> <rj><qau>C. Bronte.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Trail</ent><br/
<hw>Trail</hw> <pr>(?)</pr>, <pos>v. i.</pos> <sn>1.</sn> <def>To be drawn out in length; to follow after.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>When his brother saw the red blood <qex>trail</qex>.</q> <rj><qau>Spenser.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To grow to great length, especially when slender and creeping upon the ground, as a plant; to run or climb.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Trail</ent><br/
<hw>Trail</hw>, <pos>n.</pos> <sn>1.</sn> <def>A track left by man or beast; a track followed by the hunter; a scent on the ground by the animal pursued; <as>as, a deer <ex>trail</ex></as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>They traveled in the bed of the brook, leaving no dangerous <qex>trail</qex>.</q> <rj><qau>Cooper.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>How cheerfully on the false <qex>trail</qex> they cry!</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A footpath or road track through a wilderness or wild region; <as>as, an Indian <ex>trail</ex> over the plains</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>Anything drawn out to a length; <as>as, the <ex>trail</ex> of a meteor; a <ex>trail</ex> of smoke.</as></def><br/
[<source>1913 Webster</source>]</p>

<p><q>When lightning shoots in glittering <qex>trails</qex> along.</q> <rj><qau>Rowe.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>Anything drawn behind in long undulations; a train.</def> <ldquo/A radiant <xex>trail</xex> of hair.<rdquo/  <rj><au>Pope.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <def>Anything drawn along, as a vehicle.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><sn>6.</sn> <def>A frame for trailing plants; a trellis.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><sn>7.</sn> <def>The entrails of a fowl, especially of game, as the woodcock, and the like; -- applied also, sometimes, to the entrails of sheep.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The woodcock is a favorite with epicures, and served with its <qex>trail</qex> in, is a delicious dish.</q> <rj><qau>Baird.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>8.</sn> <fld>(Mil.)</fld> <def>That part of the stock of a gun carriage which rests on the ground when the piece is unlimbered. See <xex>Illust.</xex> of <cref>Gun carriage</cref>, under <er>Gun</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>9.</sn> <def>The act of taking advantage of the ignorance of a person; an imposition.</def> <mark>[Prov. Eng.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Trail boards</b></col> <fld>(Shipbuilding)</fld>, <cd>the carved boards on both sides of the cutwater near the figurehead.</cd> -- <col><b>Trail net</b></col>, <cd>a net that is trailed or drawn behind a boat.</cd>  <rj><au>Wright.</au></rj></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Trailer</ent><br/
<hw>Trail"er</hw> <pr>(?)</pr>, <pos>n.</pos> <def>One who, or that which, trails.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Trailer</ent><br/
<hw>Trail"er</hw>, <pos>n.</pos> <sn>1.</sn> <def>a wheeled vehicle without a motor, designed to be drawn by a motor vehicle in front of it;</def> <specif>specifically:</specif> <sd>(a)</sd> <def>such a vehicle used on street railroads. Called also <altname>trail car</altname>.</def>  <sd>(b)</sd> <def>the large wheeled wagon or van pulled by a tractor in a tractor-trailer combination.</def> <sd>(c)</sd> <def>a vehicle equipped as a mobile dwelling unit, pulled by an automobile or other mtor vehicle, and used as a dwelling when parked; -- also called a <altname>mobile home</altname>.</def> <sd>(d)</sd> <def>A wheeled motorless open wagon designed to carry a heavy object, such as a boat <ex>trailer</ex>.</def><br/
[<source>PJC</source>]</p>

<p><sn>2.</sn>  <fld>(Movies)</fld> <def>A short blank segment of movie film attached to the end; -- used for convenient insertion of the film in a projector.</def><br/
[<source>PJC</source>]</p>

<p><sn>3.</sn>  <fld>(Movies)</fld> <def>A short film consisting primarily of one or more short portions of a film, used in promotions or advertisements shortly before initial release of a film.</def><br/
[<source>PJC</source>]</p>

<p><sn>4.</sn> <def>A part of an object which extends some distance beyond the main body of the object; <as>as, the <ex>trailer</ex> of a plant</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>trailer park</b></col>.  <cd>An area equipped to accommodate trailers{2}, often with outlets supplying electrical power and water.  Called also <altname>trailer camp</altname>, <altname>trailer court</altname>.</cd></cs><br/
[<source>PJC</source>]</p>

<p><ent>Trailing</ent><br/
<hw>Trail"ing</hw>, <def><pos>a. & vb. n.</pos> from <er>Trail</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Trailing arbutus</b></col>. <fld>(Bot.)</fld> <cd>See under <er>Arbutus</er>.</cd> -- <col><b>Trailing spring</b></col>, <cd>a spring fixed in the axle box of the trailing wheels of a locomotive engine, and so placed as to assist in deadening any shock which may occur.</cd> <au>Weale.</au> -- <col><b>Trailing wheel</b></col>, <cd>a hind wheel of a locomotive when it is not a driving wheel; also, one of the hind wheels of a carriage.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Trailing edge</ent><br/
<hw>Trail"ing edge</hw>. <fld>(Aeronautics)</fld> <def>A following edge. See <er>Advancing edge</er>, above.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Trail rope</ent><br/
<hw>Trail rope</hw>. <fld>(Aeronautics)</fld> <def>Same as <er>Guide rope</er>, above.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Train</ent><br/
<hw>Train</hw> <pr>(?)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Trained</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Training</conjf>.]</vmorph> <ety>[OF. <ets>trahiner</ets>, <ets>tra<ium/ner</ets>,F. <ets>tra<icir/ner</ets>, LL. <ets>trahinare</ets>, <ets>trainare</ets>, fr. L. <ets>trahere</ets> to draw. See <er>Trail</er>.]</ety><br/
[<source>1913 Webster</source>]</p>

<p><sn>1.</sn> <def>To draw along; to trail; to drag.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>In hollow cube<br/
<qex>Training</qex> his devilish enginery.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To draw by persuasion, artifice, or the like; to attract by stratagem; to entice; to allure.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>If but a dozen French<br/
Were there in arms, they would be as a call<br/
To <qex>train</qex> ten thousand English to their side.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>O, <qex>train</qex> me not, sweet mermaid, with thy note.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>This feast, I'll gage my life,<br/
Is but a plot to <qex>train</qex> you to your ruin.</q> <rj><qau>Ford.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>To teach and form by practice; to educate; to exercise; to discipline; <as>as, to <ex>train</ex> the militia to the manual exercise; to <ex>train</ex> soldiers to the use of arms.</as></def><br/
[<source>1913 Webster</source>]</p>

<p><q>Our <qex>trained</qex> bands, which are the trustiest and most proper strength of a free nation.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>The warrior horse here bred he's taught to <qex>train</qex>.</q> <rj><qau>Dryden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>To break, tame, and accustom to draw, as oxen.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <fld>(Hort.)</fld> <def>To lead or direct, and form to a wall or espalier; to form to a proper shape, by bending, lopping, or pruning; <as>as, to <ex>train</ex> young trees</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>He <qex>trained</qex> the young branches to the right hand or to the left.</q> <rj><qau>Jeffrey.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>6.</sn> <fld>(Mining)</fld> <def>To trace, as a lode or any mineral appearance, to its head.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>To train a gun</b></col> <fld>(Mil. & Naut.)</fld>, <cd>to point it at some object either forward or else abaft the beam, that is, not directly on the side.</cd> <au>Totten.</au> -- <mcol><col><b>To train</b></col>, <it>or</it> <col><b>To train up</b></col></mcol>, <cd>to educate; to teach; to form by instruction or practice; to bring up.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><q><qex>Train up</qex> a child in the way he should go; and when he is old, he will not depart from it.</q> <rj><qau>Prov. xxii. 6.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>The first Christians were, by great hardships, <qex>trained up</qex> for glory.</q> <rj><qau>Tillotson.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Train</ent><br/
<hw>Train</hw>, <pos>v. i.</pos> <sn>1.</sn> <def>To be drilled in military exercises; to do duty in a military company.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To prepare by exercise, diet, instruction, etc., for any physical contest; <as>as, to <ex>train</ex> for a boat race</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Train</ent><br/
<hw>Train</hw>, <pos>n.</pos> <ety>[F. <ets>train</ets>, OF. <ets>tra<ium/n</ets>, <ets>trahin</ets>; cf. (for some of the senses) F. <ets>traine</ets>. See <er>Train</er>, <pos>v.</pos>]</ety> <sn>1.</sn> <def>That which draws along; especially, persuasion, artifice, or enticement; allurement.</def> <mark>[Obs.]</mark> <ldquo/Now to my charms, and to my wily <xex>trains</xex>.<rdquo/  <rj><au>Milton.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Hence, something tied to a lure to entice a hawk; also, a trap for an animal; a snare.</def>  <rj><au>Halliwell.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>With cunning <qex>trains</qex> him to entrap un wares.</q> <rj><qau>Spenser.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>That which is drawn along in the rear of, or after, something; that which is in the hinder part or rear.</def> Specifically : --<br/
[<source>1913 Webster</source>]</p>

<p><sd>(a)</sd> <def>That part of a gown which trails behind the wearer.</def><br/
[<source>1913 Webster</source>]</p>

<p><sd>(b)</sd> <fld>(Mil.)</fld> <def>The after part of a gun carriage; the trail.</def><br/
[<source>1913 Webster</source>]</p>

<p><sd>(c)</sd> <def>The tail of a bird.</def> <ldquo/The <xex>train</xex> steers their flights, and turns their bodies, like the rudder of ship.<rdquo/  <rj><au>Ray.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>A number of followers; a body of attendants; a retinue; a suite.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The king's daughter with a lovely <qex>train</qex>.</q> <rj><qau>Addison.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>My <qex>train</qex> are men of choice and rarest parts.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <def>A consecution or succession of connected things; a series.</def> <ldquo/A <xex>train</xex> of happy sentiments.<rdquo/  <rj><au>I. Watts.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>The <qex>train</qex> of ills our love would draw behind it.</q> <rj><qau>Addison.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Rivers now<br/
Stream and perpetual draw their humid <qex>train</qex>.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Other truths require a <qex>train</qex> of ideas placed in order.</q> <rj><qau>Locke.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>6.</sn> <def>Regular method; process; course; order; <as>as, things now in a <ex>train</ex> for settlement</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>If things were once in this <qex>train</qex>, . . . our duty would take root in our nature.</q> <rj><qau>Swift.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>7.</sn> <def>The number of beats of a watch in any certain time.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>8.</sn> <def>A line of gunpowder laid to lead fire to a charge, mine, or the like.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>9.</sn> <def>A connected line of cars or carriages on a railroad; -- called also <altname>railroad train</altname>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>10.</sn> <def>A heavy, long sleigh used in Canada for the transportation of merchandise, wood, and the like.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>11.</sn> <fld>(Rolling Mill)</fld> <def>A roll train; <as>as, a 12-inch <ex>train</ex></as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>12.</sn>  <fld>(Mil.)</fld> <def>The aggregation of men, animals, and vehicles which accompany an army or one of its subdivisions, and  transport its baggage, ammunition, supplies, and reserve materials of all kinds.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><cs><mcol><col><b>Roll train</b></col>, <it>or</it>  <col><b>Train of rolls</b></col></mcol> <fld>(Rolling Mill)</fld>, <cd>a set of plain or grooved rolls for rolling metal into various forms by a series of consecutive operations.</cd> -- <col><b>Train mile</b></col> <fld>(Railroads)</fld>, <cd>a unit employed in estimating running expenses, etc., being one of the total number of miles run by all the trains of a road, or system of roads, as within a given time, or for a given expenditure; -- called also <altname>mile run</altname>.</cd> -- <col><b>Train of artillery</b></col>, <cd>any number of cannon, mortars, etc., with the attendants and carriages which follow them into the field.</cd> <au>Campbell (Dict. Mil. Sci.).</au> -- <col><b>Train of mechanism</b></col>, <cd>a series of moving pieces, as wheels and pinions, each of which is follower to that which drives it, and driver to that which follows it.</cd> -- <col><b>Train road</b></col>, <cd>a slight railway for small cars, -- used for construction, or in mining.</cd> -- <col><b>Train tackle</b></col> <fld>(Naut.)</fld>, <cd>a tackle for running guns in and out.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><syn><b>Syn.</b> -- Cars.</syn> <usage> -- <er>Train</er>, <er>Cars</er>. At one time <ldquo/train<rdquo/ meaning railroad train was also referred to in the U. S. by the phrase <ldquo/the cars<rdquo/.  In the 1913 dictionary the usage was described thus: <ldquo/<xex>Train</xex> is the word universally used in England with reference to railroad traveling; <as>as, I came in the morning <ex>train</ex></as>. In the United States, the phrase <xex>the cars</xex> has been extensively introduced in the room of <xex>train</xex>; <as>as, <ex>the cars</ex> are late; I came in <ex>the cars</ex></as>. The English expression is obviously more appropriate, and is prevailing more and more among Americans, to the exclusion of <xex>the cars</xex>.<rdquo/</usage><br/
[<source>1913 Webster</source> <source>+PJC</source>]</p>

<p><ent>Trainable</ent><br/
<hw>Train"a*ble</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Capable of being trained or educated; <as>as, boys <ex>trainable</ex> to virtue</as>.</def>  <rj><au>Richardson.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Trainband</ent><br/
<hw>Train"band`</hw> <pr>(?)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Trainbands</plw> <pr>(<?/)</pr>.</plu> <def>A band or company of an organized military force instituted by James I. and dissolved by Charles II.; -- afterwards applied to the London militia.</def> <mark>[Eng.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>He felt that, without some better protection than that of the <qex>trainbands</qex> and Beefeaters, his palace and person would hardly be secure.</q> <rj><qau>Macaulay.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>A <qex>trainband</qex> captain eke was he<br/
Of famous London town.</q> <rj><qau>Cowper.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Trainbearer</ent><br/
<hw>Train"bear`er</hw> <pr>(?)</pr>, <pos>n.</pos> <def>One who holds up a train, as of a robe.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Train dispatcher</ent><br/
<hw>Train dispatcher</hw>. <def>An official who gives the orders on a railroad as to the running of trains and their right of way.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Trainel</ent><br/
<hw>Train"el</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[OF.]</ety> <def>A dragnet.</def> <mark>[Obs.]</mark>  <rj><au>Holland.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Trainer</ent><br/
<hw>Train"er</hw> <pr>(?)</pr>, <pos>n.</pos> <sn>1.</sn> <def>One who trains; an instructor; especially, one who trains or prepares men, horses, etc., for exercises requiring physical agility and strength.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A militiaman when called out for exercise or discipline.</def> <mark>[U. S.]</mark>  <rj><au>Bartlett.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Training</ent><br/
<hw>Train"ing</hw>, <pos>n.</pos> <def>The act of one who trains; the act or process of exercising, disciplining, etc.; education.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Fan training</b></col> <fld>(Hort.)</fld>, <cd>the operation of training fruit trees, grapevines, etc., so that the branches shall radiate from the stem like a fan.</cd> -- <col><b>Horizontal training</b></col> <fld>(Hort.)</fld>, <cd>the operation of training fruit trees, grapevines, etc., so that the branches shall spread out laterally in a horizontal direction.</cd> -- <col><b>Training college</b></col>. <cd>See <cref>Normal school</cref>, under <er>Normal</er>, <pos>a.</pos></cd> -- <col><b>Training day</b></col>, <cd>a day on which a military company assembles for drill or parade.</cd> <mark>[U. S.]</mark> -- <col><b>Training ship</b></col>, <cd>a vessel on board of which boys are trained as sailors.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><syn><b>Syn.</b> -- See <er>Education</er>.</syn><br/
[<source>1913 Webster</source>]</p>

<p><ent>Training wheels</ent><br/
<hw>Train"ing wheels</hw> <pr>(?)</pr>. <def>a pair of small wheels attached to the rear of a bicycle to keep the bicycle upright; -- used to assist those, especially small children, learning how to ride a bicycle.</def><br/
[<source>PJC</source>]</p>

<p><ent>Trainman</ent><br/
<hw>Train"man`</hw> <pr>(?)</pr>. <def>a member of the crew of a railroad train, often an assistant to a conductor.</def><br/
[<source>PJC</source>]</p>

<p><ent>Train oil</ent><br/
<hw>Train" oil`</hw> <pr>(oil`)</pr>. <ety>[D. or LG. <ets>traan</ets> train oil, blubber (cf. Dan. & Sw. <ets>tran</ets>, G. <ets>thran</ets>) + E. <ets>oil</ets>.]</ety> <def>Oil procured from the blubber or fat of whales, by boiling.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Trainy</ent><br/
<hw>Train"y</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Belonging to train oil.</def> <mark>[Obs.]</mark>  <rj><au>Gay.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Traipse</ent><br/
<hw>Traipse</hw> <pr>(?)</pr>, <pos>v. i.</pos> <ety>[Cf. G. <ets>trapsen</ets>, <ets>trappsen</ets>, <ets>trappen</ets>, to tread noisily, to walk stamping. See <er>Trample</er>, <er>Trape</er>.]</ety> <def>To walk or run about in a slatternly, careless, or thoughtless manner.</def> <mark>[Colloq.]</mark>  <rj><au>Pope.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Trays</ent><br/
<ent>Trais</ent><br/
<mhw>{ <hw>Trais</hw> <pr>(?)</pr>, <hw>Trays</hw>  }</mhw>, <pos>n. pl.</pos> <def>Traces.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>Four white bulls in the <qex>trays</qex>.</q> <rj><qau>Chaucer.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Trait</ent><br/
<hw>Trait</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F., fr. L. <ets>tractus</ets>, fr. <ets>trahere</ets> to draw. See <er>Trace</er>, <pos>v.</pos>, and cf. <er>Tract</er> a region, <er>Trace</er> a strap, <er>Tret</er>.]</ety><br/
[<source>1913 Webster</source>]</p>

<p><sn>1.</sn> <def>A stroke; a touch.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>By this single <qex>trait</qex> Homer makes an essential difference between the Iliad and Odyssey.</q> <rj><qau>Broome.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A distinguishing or marked feature; a peculiarity; <as>as, a <ex>trait</ex> of character</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ Formerly pronounced <xex>tr<amac/</xex>, as in French, and still so pronounced to some extent in England.</note><br/
[<source>1913 Webster</source>]</p>

<p><ent>Traiteur</ent><br/
||<hw>Trai`teur"</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F.]</ety> <def>The keeper of an eating house, or restaurant; a restaurateur.</def>  <rj><au>Simmonds.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><-- p. 1527 --><br/
[<source>1913 Webster</source>]</p>

<p><ent>Traitor</ent><br/
<hw>Trai"tor</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[OE. <ets>traitour</ets>, OF. <ets>tra<ium/tor</ets>, <ets>tra<ium/teur</ets>, F. <ets>tre<icir/tre</ets>, L. <ets>traditor</ets>, fr. <ets>tradere</ets>, <ets>traditum</ets>, to deliver, to give up or surrender treacherously, to betray; <ets>trans</ets> across, over + <ets>dare</ets> to give. See <er>Date</er> time, and cf. <er>Betray</er>,<er>Tradition</er>, <er>Traditor</er>, <er>Treason</er>.]</ety> <sn>1.</sn> <def>One who violates his allegiance and betrays his country; one guilty of treason; one who, in breach of trust, delivers his country to an enemy, or yields up any fort or place intrusted to his defense, or surrenders an army or body of troops to the enemy, unless when vanquished; also, one who takes arms and levies war against his country; or one who aids an enemy in conquering his country. See <er>Treason</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>O passing <qex>traitor</qex>, perjured and unjust!</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Hence, one who betrays any confidence or trust; a betrayer.</def> <ldquo/This false <xex>traitor</xex> death.<rdquo/  <rj><au>Chaucer.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Traitor</ent><br/
<hw>Trai"tor</hw>, <pos>a.</pos> <def>Traitorous.</def> <mark>[R.]</mark>  <rj><au>Spenser. Pope.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Traitor</ent><br/
<hw>Trai"tor</hw>, <pos>v. t.</pos> <def>To act the traitor toward; to betray; to deceive.</def> <mark>[Obs.]</mark> <ldquo/ But time, it <xex>traitors</xex> me.<rdquo/  <rj><au>Lithgow.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Traitoress</ent><br/
<hw>Trai"tor*ess</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A traitress.</def> <mark>[Obs.]</mark>  <rj><au>Rom. of R.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Traitorly</ent><br/
<hw>Trai"tor*ly</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Like a traitor; treacherous; traitorous.</def> <mark>[Obs.]</mark> <ldquo/<xex>Traitorly</xex> rascals.<rdquo/  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Traitorous</ent><br/
<hw>Trai"tor*ous</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[Cf. F. <ets>tra<icir/treux</ets>.]</ety> <sn>1.</sn> <def>Guilty of treason; treacherous; perfidious; faithless; <as>as, a <ex>traitorous</ex> officer or subject</as>.</def>  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Consisting in treason; partaking of treason; implying breach of allegiance; <as>as, a <ex>traitorous</ex> scheme</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p>-- <wordforms><wf>Trai"tor*ous*ly</wf>, <pos>adv.</pos> -- <wf>Trai"tor*ous*ness</wf>, <pos>n.</pos></wordforms><br/
[<source>1913 Webster</source>]</p>

<p><ent>Traitory</ent><br/
<hw>Trai"tor*y</hw> <pr>(?)</pr>, <pos>n.</pos> <def>Treachery.</def> <mark>[Obs.]</mark>  <rj><au>Chaucer.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Traitress</ent><br/
<hw>Trai"tress</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F. <ets>tra<icir/tresse</ets>.]</ety> <def>A woman who betrays her country or any trust; a traitoress.</def>  <rj><au>Dryden.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Traject</ent><br/
<hw>Tra*ject"</hw> <pr>(?)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Trajected</conjf>; <pos>p. pr. & vb. n.</pos> <conjf>Trajecting</conjf>.]</vmorph> <ety>[L. <ets>trajectus</ets>, p. p. of <ets>trajicere</ets> to throw across; <ets>trans</ets> across + <ets>jacere</ets> to throw. See <er>Jet</er> a shooting forth.]</ety> <def>To throw or cast through, over, or across; <as>as, to <ex>traject</ex> the sun's light through three or more cross prisms</as>.</def> <mark>[R.]</mark>  <rj><au>Sir I. Newton.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Traject</ent><br/
<hw>Traj"ect</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>trajectus</ets>, fr. <ets>trajicere</ets>: cf. F. <ets>trajet</ets>, OF. <ets>traject</ets>. See <er>Traject</er>, <pos>v. t.</pos>]</ety> <sn>1.</sn> <def>A place for passing across; a passage; a ferry.</def> <mark>[Obs.]</mark>  <rj><au>Cotgrave.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>The act of trajecting; trajection.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>A trajectory.</def> <mark>[R.]</mark>  <rj><au>I. Taylor.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Trajection</ent><br/
<hw>Tra*jec"tion</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>trajectio</ets> a crossing over, transposition.]</ety> <sn>1.</sn> <def>The act of trajecting; a throwing or casting through or across; also, emission.</def>  <rj><au>Boyle.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Transposition.</def> <mark>[R.]</mark>  <rj><au>Knatchbull.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Trajectory</ent><br/
<hw>Tra*ject"o*ry</hw> <pr>(?)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Trajectories</plw> <pr>(#)</pr>.</plu> <ety>[Cf. F. <ets>trajectoire</ets>.]</ety> <def>The curve which a body describes in space, as a planet or comet in its orbit, or stone thrown upward obliquely in the air.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Trajetry</ent><br/
<ent>Trajetour</ent><br/
<ent>Trajet</ent><br/
<mhw>{ <hw>Tra"jet</hw> <pr>(?)</pr>, <hw>Tra"jet*our</hw> <pr>(?)</pr>, <hw>Tra"jet*ry</hw> <pr>(?)</pr> }</mhw>, <pos>n.</pos> <def>See <er>Treget</er>, <er>Tregetour</er>, and <er>Tregetry</er>.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tralation</ent><br/
<hw>Tra*la"tion</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>tralatio</ets>, <ets>translatio</ets>.See <er>Translation</er>.]</ety> <def>The use of a word in a figurative or extended sense; ametaphor; a trope.</def> <mark>[Obs.]</mark>  <rj><au>Bp. Hall.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tralatition</ent><br/
<hw>Tral`a*ti"tion</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[See <er>Tralatitious</er>.]</ety> <def>A change, as in the use of words; a metaphor.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tralatitious</ent><br/
<hw>Tral`a*ti"tious</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>tralatitius</ets>, <ets>translatitius</ets>, <ets>tralaticius</ets>, <ets>translaticius</ets>. See <er>Tralation</er>.]</ety> <sn>1.</sn> <def>Passed along; handed down; transmitted.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Among biblical critics a <qex>tralatitious</qex> interpretation is one received by expositor from expositor.</q> <rj><qau>W. Withington.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Metaphorical; figurative; not literal.</def>  <rj><au>Stackhouse.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tralatitiously</ent><br/
<hw>Tral`a*ti"tious*ly</hw>, <pos>adv.</pos> <def>In a tralatitious manner; metephorically.</def>  <rj><au>Holder.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tralineate</ent><br/
<hw>Tra*lin"e*ate</hw> <pr>(?)</pr>, <pos>v. i.</pos> <ety>[L. <ets>trans</ets> across + <ets>linea</ets> a line: cf. It <ets>tralineare</ets>, <ets>tralignare</ets>.]</ety> <def>To deviate; to stray; to wander.</def> <mark>[Obs.]</mark>  <rj><au>Dryden.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tralucency</ent><br/
<hw>Tra*lu"cen*cy</hw> <pr>(?)</pr>, <pos>n.</pos> <def>Translucency; <as>as, the <ex>tralucency</ex> of a gem</as>.</def> <mark>[Obs.]</mark>  <rj><au>Sir T. Browne.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tralucent</ent><br/
<hw>Tra*lu"cent</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>tralucens</ets>, <ets>translucens</ets>, p. pr. See <er>Translucent</er>.]</ety> <def>Translucent.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>The air's <qex>tralucent</qex> gallery.</q> <rj><qau>Sir. J. Davies.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tram</ent><br/
<hw>Tram</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Prov. E. <ets>tram</ets> a coal wagon, the shaft of a cart or carriage, a beam or bar; probably of Scand, origin; cf. OSw. <ets>tr<aring/m</ets>, <ets>trum</ets>, a beam, OD. <ets>drom</ets>, Prov. & OHG. <ets>tram</ets>.]</ety> <sn>1.</sn> <def>A four-wheeled truck running on rails, and used in a mine, as for carrying coal or ore.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>The shaft of a cart.</def> <mark>[Prov. Eng.]</mark>  <rj><au>De Quincey.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>One of the rails of a tramway.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>A car on a horse railroad.</def> <mark>[Eng.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Tram car</b></col>, <cd>a car made to run on a tramway, especially a street railway car.</cd> -- <col><b>Tram plate</b></col>, <cd>a flat piece of iron laid down as a rail.</cd> -- <col><b>Tram pot</b></col> <fld>(Milling)</fld>, <cd>the step and support for the lower end of the spindle of a millstone.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tram</ent><br/
<hw>Tram</hw>, <pos>n.</pos> <ety>[Sp. <ets>trama</ets> weft, or F. <ets>trame</ets>.]</ety> <def>A silk thread formed of two or more threads twisted together, used especially for the weft, or cross threads, of the best quality of velvets and silk goods.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tram</ent><br/
<hw>Tram</hw>, <pos>n.</pos> <fld>(Mech.)</fld> <def>Same as <er>Trammel</er>, <pos>n.</pos>, 6.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Tram</ent><br/
<hw>Tram</hw> <pr>(?)</pr>, <pos>v. t.</pos>  <vmorph>[<pos>imp. & p. p.</pos> <conjf>Trammed</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Tramming</conjf>.]</vmorph> <def>To convey or transport on a tramway or on a tram car.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Tram</ent><br/
<hw>Tram</hw>, <pos>v. i.</pos> <def>To operate, or conduct the business of, a tramway; to travel by tramway.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Trama</ent><br/
||<hw>Tra"ma</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L., woof.]</ety> <fld>(Bot.)</fld> <def>The loosely woven substance which lines the chambers within the gleba in certain <ord>Gasteromycetes</ord>.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Tramble</ent><br/
<hw>Tram"ble</hw> <pr>(?)</pr>, <pos>v. t.</pos> <fld>(Mining)</fld> <def>To wash, as tin ore, with a shovel in a frame fitted for the purpose.</def>  <rj><au>Smart.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Trammel</ent><br/
<hw>Tram"mel</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F. <ets>tramail</ets>, <ets>tr<eacute/mail</ets>, a net, LL. <ets>tremaculum</ets>, <ets>tremacle</ets>, a kind of net for taking fish; L. <ets>tres</ets> three + <ets>macula</ets> a mesh. See <er>Three</er>, and <er>Mail</er> armor.]</ety> <sn>1.</sn> <def>A kind of net for catching birds, fishes, or other prey.</def>  <rj><au>Carew.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A net for confining a woman's hair.</def>  <rj><au>Spenser.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>A kind of shackle used for regulating the motions of a horse and making him amble.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>Fig.: Whatever impedes activity, progress, or freedom, as a net or shackle.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>[They] disdain the <qex>trammels</qex> of any sordid contract.</q> <rj><qau>Jeffrey.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <def>An iron hook of various forms and sizes, used for handing kettles and other vessels over the fire.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>6.</sn> <fld>(Mech.)</fld> <sd>(a)</sd> <def>An instrument for drawing ellipses, one part of which consists of a cross with two grooves at right angles to each other, the other being a beam carrying two pins (which slide in those grooves), and also the describing pencil.</def> <sd>(b)</sd> <def>A beam compass. See under <er>Beam</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Trammel</ent><br/
<hw>Tram"mel</hw> <pr>(?)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Trammeled</conjf> <pr>(?)</pr> or <conjf>Trammelled</conjf>; <pos>p. pr. & vb. n.</pos> <conjf>Trammeling</conjf>, or <conjf>Trammelling</conjf>.]</vmorph> <sn>1.</sn> <def>To entangle, as in a net; to catch.</def> <mark>[R.]</mark>  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To confine; to hamper; to shackle.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Trammeled</ent><br/
<hw>Tram"meled</hw> <pr>(?)</pr>, <pos>a.</pos> <fld>(Man.)</fld> <def>Having blazes, or white marks, on the fore and hind foot of one side, as if marked by trammels; -- said of a horse.</def> <altsp>[Written also <asp>trammelled</asp>.]</altsp><br/
[<source>1913 Webster</source>]</p>

<p><ent>Trammeler</ent><br/
<hw>Tram"mel*er</hw> <pr>(?)</pr>, <pos>n.</pos> <altsp>[Written also <asp>trammeller</asp>.]</altsp> <sn>1.</sn> <def>One who uses a trammel net.</def>  <rj><au>Nares.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>One who, or that which, trammels or restrains.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Trammel wheel</ent><br/
<hw>Tram"mel wheel`</hw>. <fld>(Mach.)</fld> <def>A circular plate or a cross, with two or more cross grooves intersecting at the center, used on the end of a shaft to transmit motion to another shaft not in line with the first.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Tramming</ent><br/
<hw>Tram"ming</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Silk Manuf.)</fld> <def>The act or process of forming trams. See 2d <er>Tram</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tramontana</ent><br/
||<hw>Tra`mon*ta"na</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[It. See <er>Tramontane</er>.]</ety> <fld>(Meteor.)</fld> <def>A dry, cold, violent, northerly wind of the Adriatic.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Tramontane</ent><br/
<hw>Tra*mon"tane</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[OF. <ets>tramontain</ets>, It. <ets>tramontano</ets>, L. <ets>transmontanus</ets>; <ets>trans</ets> across, beyond + <ets>mons</ets>, <ets>montis</ets>, mountain.]</ety> <def>Lying or being beyond the mountains; coming from the other side of the mountains; hence, foreign; barbarous.</def><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ The Italians sometimes use this epithet for <xex>ultramontane</xex>, and apply it to the countries north of the Alps, as France and Germany, and especially to their ecclesiastics, jurists, painters, etc.; and a north wind is called a <xex>tramontane</xex> wind. The French lawyers call certain Italian canonists <xex>tramontane</xex>, or <xex>ultramontane</xex>, doctors; considering them as favoring too much the court of Rome. See <er>Ultramontane</er>.</note><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tramontane</ent><br/
<hw>Tra*mon"tane</hw>, <pos>n.</pos> <def>One living beyond the mountains; hence, a foreigner; a stranger.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tramp</ent><br/
<hw>Tramp</hw> <pr>(?)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Tramped</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Tramping</conjf>.]</vmorph> <ety>[OE. <ets>trampen</ets>; akin to LG. <ets>trampen</ets>, G. <ets>trampeln</ets>, LG. & D. <ets>trappen</ets>, Dan. <ets>trampe</ets>, Sw. & Icel. <ets>trampa</ets>, Goth. ana<ets>trimpan</ets> to press upon; also to D. <ets>trap</ets> a step, G. <ets>treppe</ets> steps, stairs.  Cf. <er>Trap</er> a kind of rock, <er>Trape</er>, <er>Trip</er>, <pos>v. i.</pos>, <er>Tread</er>.]</ety> <sn>1.</sn> <def>To tread upon forcibly and repeatedly; to trample.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To travel or wander through; <as>as, to <ex>tramp</ex> the country</as>.</def> <mark>[Colloq.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>To cleanse, as clothes, by treading upon them in water.</def> <mark>[Scot.]</mark>  <rj><au>Jamieson.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tramp</ent><br/
<hw>Tramp</hw>, <pos>v. i.</pos> <def>To travel; to wander; to stroll.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tramp</ent><br/
<hw>Tramp</hw>, <pos>n.</pos> <sn>1.</sn> <def>A foot journey or excursion; <as>as, to go on a <ex>tramp</ex>; a long <ex>tramp</ex></as>.</def>  <rj><au>Blackie.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A foot traveler; a tramper; often used in a bad sense for a vagrant or wandering vagabond.</def>  <rj><au>Halliwell.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>The sound of the foot, or of feet, on the earth, as in marching.</def>  <rj><au>Sir W. Scott.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>A tool for trimming hedges.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <def>A plate of iron worn to protect the sole of the foot, or the shoe, when digging with a spade.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tramper</ent><br/
<hw>Tramp"er</hw> <pr>(?)</pr>, <pos>n.</pos> <def>One who tramps; a stroller; a vagrant or vagabond; a tramp.</def>  <rj><au>Dickens.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Trample</ent><br/
<hw>Tram"ple</hw> <pr>(?)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Trampled</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Trampling</conjf> <pr>(?)</pr>.]</vmorph> <ety>[OE. <ets>trampelen</ets>, freq. of <ets>trampen</ets>. See <er>Tramp</er>, <pos>v. t.</pos>]</ety> <sn>1.</sn> <def>To tread under foot; to tread down; to prostrate by treading; <as>as, to <ex>trample</ex> grass or flowers</as>.</def>  <rj><au>Dryden.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Neither cast ye your pearls before swine, lest they <qex>trample</qex> them under their feet.</q> <rj><qau>Matt. vii. 6.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Fig.: To treat with contempt and insult.</def>  <rj><au>Cowper.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Trample</ent><br/
<hw>Tram"ple</hw>, <pos>v. i.</pos> <sn>1.</sn> <def>To tread with force and rapidity; to stamp.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To tread in contempt; -- with <xex>on</xex> or <xex>upon</xex>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Diogenes <qex>trampled</qex> on Plato's pride with greater of his own.</q> <rj><qau>Gov. of Tongue.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Trample</ent><br/
<hw>Tram"ple</hw>, <pos>n.</pos> <def>The act of treading under foot; also, the sound produced by trampling.</def>  <rj><au>Milton.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>The huddling <qex>trample</qex> of a drove of sheep.</q> <rj><qau>Lowell.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Trampler</ent><br/
<hw>Tram"pler</hw> <pr>(?)</pr>, <pos>n.</pos> <def>One who tramples; one who treads down; <as>as, a <ex>trampler</ex> on nature's law</as>.</def>  <rj><au>Cowper.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Trampoose</ent><br/
<hw>Tram*poose"</hw> <pr>(?)</pr>, <pos>v. i.</pos> <ety>[See <er>Tramp</er>, <er>Trample</er>, and <er>Traipse</er>.]</ety> <def>To walk with labor, or heavily; to tramp.</def> <mark>[Law, U. S.]</mark>  <rj><au>Bartlett.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tramrail</ent><br/
<hw>Tram"rail`</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Mach.)</fld> <def>An overhead rail forming a track on which a trolley runs to convey a load, as in a shop.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Tramroad</ent><br/
<hw>Tram"road`</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[<ets>Tram</ets> a coal wagon + <ets>road</ets>.]</ety> <def>A road prepared for easy transit of trams or wagons, by forming the wheel tracks of smooth beams of wood, blocks of stone, or plates of iron.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tramway</ent><br/
<hw>Tram"way`</hw> <pr>(?)</pr>, <pos>n.</pos> <sn>1.</sn> <def>Same as <er>Tramroad</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A street railway  laid in the streets of a town or city, or an interurban railway for local traffic, on which cable cars, or trolley cars, etc., are used, in distinction from an extended railway line for trains drawn by steam or electric locomotives.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Tranation</ent><br/
<hw>Tra*na"tion</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>tranare</ets>, <ets>transnare</ets>, to swim over; <ets>trans</ets> across, over + <ets>nare</ets> to swim.]</ety> <def>The act of swimming over.</def> <mark>[Obs.]</mark>  <rj><au>Bailey.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Trance</ent><br/
<hw>Trance</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F. <ets>transe</ets> fright, in OF. also, trance or swoon, fr. <ets>transir</ets> to chill, benumb, to be chilled, to shiver, OF. also, to die, L. <ets>transire</ets> to pass over, go over, pass away, cease; <ets>trans</ets> across, over + <ets>ire</ets> to go; cf. L. <ets>transitus</ets> a passing over. See <er>Issue</er>, and cf. <er>Transit</er>.]</ety><br/
[<source>1913 Webster</source>]</p>

<p><sn>1.</sn> <def>A tedious journey.</def> <mark>[Prov. Eng.]</mark>  <rj><au>Halliwell.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A state in which the soul seems to have passed out of the body into another state of being, or to be rapt into visions; an ecstasy.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>And he became very hungry, and would have eaten; but while they made ready, he fell into a <qex>trance</qex>.</q> <rj><qau>Acts. x. 10.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>My soul was ravished quite as in a <qex>trance</qex>.</q> <rj><qau>Spenser.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <fld>(Med.)</fld> <def>A condition, often simulating death, in which there is a total suspension of the power of voluntary movement, with abolition of all evidences of mental activity and the reduction to a minimum of all the vital functions so that the patient lies still and apparently unconscious of surrounding objects, while the pulsation of the heart and the breathing, although still present, are almost or altogether imperceptible.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>He fell down in a <qex>trance</qex>.</q> <rj><qau>Chaucer.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Trance</ent><br/
<hw>Trance</hw>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Tranced</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Trancing</conjf> <pr>(?)</pr>.]</vmorph> <sn>1.</sn> <def>To entrance.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>And three I left him <qex>tranced</qex>.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To pass over or across; to traverse.</def> <mark>[Poetic]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q><qex>Trance</qex> the world over.</q> <rj><qau>Beau. & Fl.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>When thickest dark did <qex>trance</qex> the sky.</q> <rj><qau>Tennyson.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Trance</ent><br/
<hw>Trance</hw> <pr>(?)</pr>, <pos>v. i.</pos> <def>To pass; to travel.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tranect</ent><br/
<hw>Tran"ect</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Cf. <er>Traject</er>.]</ety> <def>A ferry.</def> <mark>[Obs.]</mark>  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Trangram</ent><br/
<hw>Tran"gram</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[OE. <ets>trangrain</ets> a strange thing, <ets>trangame</ets> a toy. See <er>Tangram</er>.]</ety> <def>Something intricately contrived; a contrived; a puzzle.</def> <mark>[Cant & Obs.]</mark>  <rj><au>Arbuthnot.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Trannel</ent><br/
<hw>Tran"nel</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Naut.)</fld> <def>A treenail.</def> <mark>[R.]</mark>  <rj><au>Moxon.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tranquil</ent><br/
<hw>Tran"quil</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>tranquillus</ets>; probably fr. <ets>trans</ets> across, over + a word akin to <ets>quietus</ets> quiet: cf. F. <ets>tranquille</ets>. See <er>Quiet</er>.]</ety> <def>Quiet; calm; undisturbed; peaceful; not agitated; <as>as, the atmosphere is <ex>tranquil</ex>; the condition of the country is <ex>tranquil</ex></as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>A style clear, <qex>tranquil</qex>, easy to follow.</q> <rj><qau>De Quincey.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tranquillization</ent><br/
<ent>Tranquilization</ent><br/
<mhw>{ <hw>Tran`quil*i*za"tion</hw>, <hw>Tran`quil*li*za"tion</hw>  }</mhw> <pr>(?)</pr>, <pos>n.</pos> <def>The act of tranquilizing, or the state of being tranquilized.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tranquillize</ent><br/
<ent>Tranquilize</ent><br/
<mhw>{<hw>Tran"quil*ize</hw>, <hw>Tran"quil*lize</hw>  }</mhw> <pr>(?)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Tranquilized</conjf> <pr>(?)</pr> or <conjf>Tranquilliized</conjf>; <pos>p. pr. & vb. n.</pos> <conjf>Tranquilizing</conjf> <pr>(?)</pr> or <conjf>Tranquillizing</conjf>.]</vmorph> <ety>[Cf. F. <ets>tranquilliser</ets>.]</ety> <def>To render tranquil; to allay when agitated; to compose; to make calm and peaceful; <as>as, to <ex>tranquilize</ex> a state disturbed by factions or civil commotions; to <ex>tranquilize</ex> the mind.</as></def><br/
[<source>1913 Webster</source>]</p>

<p><syn><b>Syn.</b> -- To quiet; compose; still; soothe; appease; calm; pacify.</syn><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tranquillizer</ent><br/
<ent>Tranquilizer</ent><br/
<mhw>{ <hw>Tran"quil*i`zer</hw>, <hw>Tran"quil*li`zer</hw>  }</mhw> <pr>(?)</pr>, <pos>n.</pos> <def>One who, or that which, tranquilizes.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tranquillizing</ent><br/
<ent>Tranquilizing</ent><br/
<mhw>{ <hw>Tran"quil*i`zing</hw>, <hw>Tran"quil*li`zing</hw>  }</mhw> <pr>(?)</pr>, <pos>a.</pos> <def>Making tranquil; calming.</def> <ldquo/ The <xex>tranquilizing</xex> power of time.<rdquo/ <au>Wordsworth.</au> -- <wordforms><wf>Tran"quil*i`zing*ly</wf> <it>or</it> <wf>Tran"quil*li`zing*ly</wf>, <pos>adv.</pos></wordforms><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tranquillity</ent><br/
<hw>Tran*quil"li*ty</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F. <ets>tranquillit<eacute/</ets>, L. <ets>tranquillitas</ets>.]</ety> <def>The quality or state of being tranquil; calmness; composure.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tranquilly</ent><br/
<hw>Tran"quil*ly</hw> <pr>(?)</pr>, <pos>adv.</pos> <def>In a tranquil manner; calmly.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tranquilness</ent><br/
<hw>Tran"quil*ness</hw>, <pos>n.</pos> <def>Quality or state of being tranquil.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Trans-</ent><br/
<hw>Trans-</hw> <pr>(?)</pr>. <ety>[L. <ets>trans</ets> across, over.]</ety> <def>A prefix, signifying <xex>over</xex>, <xex>beyond</xex>, <xex>through and through</xex>, <xex>on the other side</xex>, as in <xex>trans</xex>alpine, beyond the Alps; <xex>trans</xex>form, to form through and through, that is, anew, <xex>trans</xex>figure.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Transact</ent><br/
<hw>Trans*act"</hw> <pr>(?)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Transacted</conjf>; <pos>p. pr. & vb. n.</pos> <conjf>Transacting</conjf>.]</vmorph> <ety>[L. <ets>transactus</ets>, p. p. of <ets>transigere</ets>. See <er>Transaction</er>.]</ety> <def>To carry through; to do; perform; to manage; <as>as, to <ex>transact</ex> commercial business; to <ex>transact</ex> business by an agent</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Transact</ent><br/
<hw>Trans*act"</hw>, <pos>v. i.</pos> <def>To conduct matters; to manage affairs.</def> <mark>[R.]</mark>  <rj><au>South.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Transaction</ent><br/
<hw>Trans*ac"tion</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>transactio</ets>, fr. <ets>transigere</ets>, <ets>transactum</ets>, to drive through, carry through, accomplish, transact; <ets>trans</ets> across, over + <ets>agere</ets> to drive; cf. F. <ets>transaction</ets>. See <er>Act</er>, <er>Agent</er>.]</ety> <sn>1.</sn> <def>The doing or performing of any business; management of any affair; performance.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>That which is done; an affair; <as>as, the <ex>transactions</ex> on the exchange</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <fld>(Civil Law)</fld> <def>An adjustment of a dispute between parties by mutual agreement.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Transaction of a society</b></col>, <cd>the published record of what it has done or accomplished.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><syn><b>Syn.</b> -- Proceeding; action; process.</syn> <usage> -- <er>Transaction</er>, <er>Proceeding</er>. A <xex>transaction</xex> is something already done and completed; a <xex>proceeding</xex> is either something which is now going on, or, if ended, is still contemplated with reference to its <xex>progress</xex> or successive stages.</usage><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ <ldquo/ We the word <xex>proceeding</xex> in application to an affray in the street, and the word <xex>transaction</xex> to some commercial negotiation that has been carried on between certain persons. The <xex>proceeding</xex> marks the manner of <xex>proceeding</xex>, as when we speak of the <xex>proceedings</xex> in a court of law. The <xex>transaction</xex> marks the business <xex>transacted</xex>; as, the <xex>transactions</xex> on the Exchange.<rdquo/</note>  <rj><au>Crabb.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Transactor</ent><br/
<hw>Trans*act"or</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L.]</ety> <def>One who transacts, performs, or conducts any business.</def>  <rj><au>Derham.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Transalpine</ent><br/
<hw>Trans*al"pine</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>transalpinus</ets>; <ets>trans</ets> across, beyond + <ets>Alpinus</ets> Alpine, from <ets>Alpes</ets> the Alps: cf. F. <ets>transalpin</ets>.]</ety> <def>Being on the farther side of the Alps in regard to Rome, that is, on the north or west side of the Alps; of or pertaining to the region or the people beyond the Alps; <as>as, <ex>transalpine</ex> Gaul</as>; -- opposed to <contr>cisalpine</contr>.</def> <ldquo/ <xex>Transalpine</xex> garbs.<rdquo/  <rj><au>Beau. & Fl.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Transalpine</ent><br/
<hw>Trans*al"pine</hw>, <pos>n.</pos> <def>A native or inhabitant of a country beyond the Alps, that is, out of Italy.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Transanimate</ent><br/
<hw>Trans*an"i*mate</hw> <pr>(tr<acr/ns*<acr/n"<icr/*m<amac/t)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Transanimated</conjf> <pr>(tr<acr/ns*<acr/n"<icr/*m<amac/`t<ecr/d)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Transanimating</conjf>.]</vmorph> <ety>[<ets>Trans-</ets> + <ets>animate</ets>.]</ety> <def>To animate with a soul conveyed from another body.</def> <mark>[R.]</mark>  <rj><au>Bp. J. King (1608).</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Transanimation</ent><br/
<hw>Trans*an`i*ma"tion</hw> <pr>(tr<acr/ns*<acr/n`<icr/*m<amac/"sh<ucr/n)</pr>, <pos>n.</pos> <ety>[Cf. F. <ets>transanimation</ets>.]</ety> <def>The conveyance of a soul from one body to another.</def> <mark>[R.]</mark>  <rj><au>Fuller.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Transatlantic</ent><br/
<hw>Trans`at*lan"tic</hw> <pr>(tr<acr/ns*<acr/t*l<acr/n"t<icr/k)</pr>, <pos>a.</pos> <ety>[Pref. <ets>trans-</ets> + <ets>Atlantic</ets>: cf. F. <ets>transatlantique</ets>.]</ety> <sn>1.</sn> <def>Lying or being beyond the Atlantic Ocean.</def><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ When used by a person in Europe or Africa, <ex>transatlantic</ex> signifies being in America; when by a person in America, it denotes being or lying in Europe or Africa, especially the former.</note><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Crossing the Atlantic Ocean; <as>as, a <ex>transatlantic</ex> flight</as>.</def><br/
[<source>1913 Webster</source> <source>+PJC</source>]</p>

<p><ent>Transaudient</ent><br/
<hw>Trans*au"di*ent</hw> <pr>(tr<acr/ns*<add/"d<icr/*<eit/nt)</pr>, <pos>a.</pos> <ety>[See <er>Trans-</er>, and <er>Audient</er>.]</ety> <def>Permitting the passage of sound.</def> <mark>[R.]</mark>  <rj><au>Lowell.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Transcalency</ent><br/
<hw>Trans*ca"len*cy</hw> <pr>(tr<acr/ns*k<amac/"l<eit/n*s<ycr/)</pr>, <pos>n.</pos> <def>The quality or state of being transcalent.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Transcalent</ent><br/
<hw>Trans*ca"lent</hw> <pr>(tr<acr/ns*k<amac/"l<eit/nt)</pr>, <pos>a.</pos> <ety>[Pref. <ets>trans-</ets> + L. <ets>calens</ets>, p. pr. of <ets>calere</ets> to grow warm.]</ety> <def>Pervious to, or permitting the passage of, heat.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Transcend</ent><br/
<hw>Tran*scend"</hw> <pr>(tr<acr/n*s<ecr/nd")</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Transcended</conjf>; <pos>p. pr. & vb. n.</pos> <conjf>Transcending</conjf>.]</vmorph> <ety>[L. <ets>transcendere</ets>, <ets>transcensum</ets>; <ets>trans</ets> beyond, over + <ets>scandere</ets> to climb. See <er>Scan</er>.]</ety> <sn>1.</sn> <def>To rise above; to surmount; <as>as, lights in the heavens <ex>transcending</ex> the region of the clouds</as>.</def>  <rj><au>Howell.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To pass over; to go beyond; to exceed.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Such popes as shall <qex>transcend</qex> their limits.</q> <rj><qau>Bacon.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><-- p. 1528 --><br/
[<source>1913 Webster</source>]</p>

<p><sn>8.</sn> <def>To surpass; to outgo; to excel; to exceed.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>How much her worth <qex>transcended</qex> all her kind.</q> <rj><qau>Dryden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Transcend</ent><br/
<hw>Tran*scend"</hw> <pr>(tr<acr/n*s<ecr/nd")</pr>, <pos>v. i.</pos> <sn>1.</sn> <def>To climb; to mount.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To be transcendent; to excel.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Transcendency</ent><br/
<ent>Transcendence</ent><br/
<mhw>{ <hw>Tran*scend"ence</hw> <pr>(-<eit/ns)</pr>, <hw>Tran*scend"en*cy</hw> <pr>(-<eit/n*s<ycr/)</pr>, }</mhw><ety>[Cf. L. <ets>transcendentia</ets>, F. <ets>transcendance</ets>.]</ety> <sn>1.</sn> <def>The quality or state of being transcendent; superior excellence; supereminence.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The Augustinian theology rests upon the <qex>transcendence</qex> of Deity at its controlling principle.</q> <rj><qau>A. V. G. Allen.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Elevation above truth; exaggeration.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q><ldquo/Where <qex>transcendencies</qex> are more allowed.<rdquo/</q> <rj><qau>Bacon.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Transcendent</ent><br/
<hw>Tran*scend"ent</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>transcendens</ets>, <ets>-entis</ets>, p. pr. of <ets>transcendere</ets> to transcend: cf. F. <ets>transcendant</ets>, G. <ets>transcendent</ets>.]</ety> <sn>1.</sn> <def>Very excellent; superior or supreme in excellence; surpassing others; <as>as, <ex>transcendent</ex> worth; <ex>transcendent</ex> valor</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Clothed with <qex>transcendent</qex> brightness.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Kantian Philos.)</fld> <def>Transcending, or reaching beyond, the limits of human knowledge; -- applied to affirmations and speculations concerning what lies beyond the reach of the human intellect.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Transcendent</ent><br/
<hw>Tran*scend"ent</hw>, <pos>n.</pos> <def>That which surpasses or is supereminent; that which is very excellent.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Transcendental</ent><br/
<hw>Tran`scen*den"tal</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[Cf. F. <ets>transcendantal</ets>, G. <ets>transcendental</ets>.]</ety> <sn>1.</sn> <def>Supereminent; surpassing others; <as>as, <ex>transcendental</ex> being or qualities</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Philos.)</fld> <def>In the Kantian system, of or pertaining to that which can be determined <xex>a priori</xex> in regard to the fundamental principles of all human knowledge. What is <xex>transcendental</xex>, therefore, transcends empiricism; but is does not transcend all human knowledge, or become <xex>transcendent</xex>. It simply signifies the <xex>a priori</xex> or necessary conditions of experience which, though affording the conditions of experience, transcend the sphere of that contingent knowledge which is acquired by experience.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>Vaguely and ambitiously extravagant in speculation, imagery, or diction.</def><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ In mathematics, a quantity is said to be <xex>transcendental</xex> relative to another quantity when it is expressed as a transcendental function of the latter; <as>thus, <it>a</it><exp>x</exp>, 10<exp>2x</exp>, log <it>x</it>, sin <it>x</it>, tan <it>x</it>, etc., are <ex>transcendental</ex> relative to <it>x</it></as>.</note><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Transcendental curve</b></col> <fld>(Math.)</fld>, <cd>a curve in which one ordinate is a transcendental function of the other.</cd> -- <col><b>Transcendental equation</b></col> <fld>(Math.)</fld>, <cd>an equation into which a transcendental function of one of the unknown or variable quantities enters.</cd> -- <col><b>Transcendental function</b></col>. <fld>(Math.)</fld> <cd>See under <er>Function</er>.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><syn><b>Syn.</b> -- <er>Transcendental</er>, <er>Empirical</er>.</syn> <usage> These terms, with the corresponding nouns, <xex>transcendentalism</xex> and <xex>empiricism</xex>, are of comparatively recent origin. <xex>Empirical</xex> refers to knowledge which is gained by the experience of actual phenomena, without reference to the principles or laws to which they are to be referred, or by which they are to be explained. <xex>Transcendental</xex> has reference to those beliefs or principles which are not derived from experience, and yet are absolutely necessary to make experience possible or useful. Such, in the better sense of the term, is the <xex>transcendental</xex> philosophy, or <xex>transcendentalism</xex>.  Each of these words is also used in a bad sense, <xex>empiricism</xex> applying to that one-sided view of knowledge which neglects or loses sight of the truths or principles referred to above, and trusts to experience alone; <xex>transcendentalism</xex>, to the opposite extreme, which, in its deprecation of experience, loses sight of the relations which facts and phenomena sustain to principles, and hence to a kind of philosophy, or a use of language, which is vague, obscure, fantastic, or extravagant.</usage><br/
[<source>1913 Webster</source>]</p>

<p><ent>Transcendental</ent><br/
<hw>Tran`scen*den"tal</hw>, <pos>n.</pos> <def>A transcendentalist.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Transcendentalism</ent><br/
<hw>Tran`scen*den"tal*ism</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Cf. F. <ets>transcendantalisme</ets>, G. <ets>transcendentalismus</ets>.]</ety> <sn>1.</sn> <fld>(Kantian Philos.)</fld> <def>The transcending, or going beyond, empiricism, and ascertaining <it>a priori</it> the fundamental principles of human knowledge.</def><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ As Schelling and Hegel claim to have discovered the absolute identity of the objective and subjective in human knowledge, or of things and human conceptions of them, the Kantian distinction between <xex>transcendent</xex> and <xex>transcendental</xex> ideas can have no place in their philosophy; and hence, with them, transcendentalism claims to have a true knowledge of all things, material and immaterial, human and divine, so far as the mind is capable of knowing them. And in this sense the word <xex>transcendentalism</xex> is now most used. It is also sometimes used for that which is vague and illusive in philosophy.</note><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Ambitious and imaginative vagueness in thought, imagery, or diction.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Transcendentalist</ent><br/
<hw>Tran`scen*den"tal*ist</hw>, <pos>n.</pos> <ety>[Cf. F. <ets>transcendantaliste</ets>.]</ety> <def>One who believes in transcendentalism.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Transcendentality</ent><br/
<hw>Tran`scen*den*tal"i*ty</hw> <pr>(?)</pr>, <pos>n.</pos> <def>The quality or state of being transcendental.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Transcendentally</ent><br/
<hw>Tran`scen*den"tal*ly</hw> <pr>(?)</pr>, <pos>adv.</pos> <def>In a transcendental manner.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Transcendently</ent><br/
<hw>Tran*scend"ent*ly</hw> <pr>(?)</pr>, <pos>adv.</pos> <def>In a transcendent manner.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Transcendentness</ent><br/
<hw>Tran*scend"ent*ness</hw>, <pos>n.</pos> <def>Same as <er>Transcendence</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Transcension</ent><br/
<hw>Tran*scen"sion</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[See <er>Transcend</er>.]</ety> <def>The act of transcending, or surpassing; also, passage over.</def> <mark>[Obs.]</mark>  <rj><au>Chapman.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Transcolate</ent><br/
<hw>Trans"co*late</hw> <pr>(?)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Transcolated</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Transcolating</conjf>.]</vmorph> <ety>[Pref. <ets>trans-</ets> + L. <ets>colare</ets>, <ets>colatum</ets>, to filter, to strain.]</ety> <def>To cause to pass through a sieve or colander; to strain, as through a sieve.</def> <mark>[Obs.]</mark>  <rj><au>Harvey.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Transcolation</ent><br/
<hw>Trans`co*la"tion</hw> <pr>(?)</pr>, <pos>n.</pos> <def>Act of transcolating, or state of being transcolated.</def> <mark>[Obs.]</mark>  <rj><au>Bp. Stillingfleet.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Transcontinental</ent><br/
<hw>Trans*con`ti*nen"tal</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[Pref. <ets>trans-</ets> + <ets>continental</ets>.]</ety> <def>Extending or going across a continent; <as>as, a <ex>transcontinental</ex> railroad or journey</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Transcorporate</ent><br/
<hw>Trans*cor"po*rate</hw> <pr>(?)</pr>, <pos>v. i.</pos> <ety>[Pref. <ets>trans-</ets> + <ets>corporate</ets>.]</ety> <def>To transmigrate.</def> <mark>[Obs.]</mark>  <rj><au>Sir T. Browne.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Transcribbler</ent><br/
<hw>Tran*scrib"bler</hw> <pr>(tr<acr/n*skr<icr/b"bl<etil/r)</pr>, <pos>n.</pos> <def>A transcriber; -- used in contempt.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>He [Aristotle] has suffered vastly from the <qex>transcribblers</qex>, as all authors of great brevity necessarily must.</q> <rj><qau>Gray.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Transcribe</ent><br/
<hw>Tran*scribe"</hw> <pr>(tr<acr/n*skr<imac/b")</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Transcribed</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Transcribing</conjf>.]</vmorph> <ety>[L. <ets>transcribere</ets>, <ets>transcriptum</ets>; <ets>trans</ets> across, over + <ets>scribere</ets> to write. See <er>Scribe</er>.]</ety> <def>To write over again, or in the same words; to copy; <as>as, to <ex>transcribe</ex> Livy or Tacitus; to <ex>transcribe</ex> a letter.</as></def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Transcriber</ent><br/
<hw>Tran*scrib"er</hw> <pr>(-<etil/r)</pr>, <pos>n.</pos> <def>One who transcribes, or writes from a copy; a copier; a copyist.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Transcript</ent><br/
<hw>Tran"script</hw> <pr>(tr<acr/n"skr<icr/pt)</pr>, <pos>n.</pos> <ety>[L. <ets>transcriptum</ets>, neut. of <ets>transcriptus</ets>, p. p. of <ets>transcribere</ets>. See <er>Transcribe</er>.]</ety> <sn>1.</sn> <def>That which has been transcribed; a writing or composition consisting of the same words as the original; a written copy.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The decalogue of Moses was but a <qex>transcript</qex>.</q> <rj><qau>South.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A copy of any kind; an imitation.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The Grecian learning was but a <qex>transcript</qex> of the Chaldean and Egyptian.</q> <rj><qau>Glanvill.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><-- 3. <def>A written version of what was said orally; <as>as, a <ex>transcript</ex> of a trial</as>.</def> --><br/
[<source>1913 Webster</source>]</p>

<p><ent>Transcription</ent><br/
<hw>Tran*scrip"tion</hw> <pr>(tr<acr/n*skr<icr/p"sh<ucr/n)</pr>, <pos>n.</pos> <ety>[Cf. F. <ets>transcription</ets>, L. <ets>transcriptio</ets> a transfer.]</ety> <sn>1.</sn> <def>The act or process of transcribing, or copying; <as>as, corruptions creep into books by repeated <ex>transcriptions</ex></as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A copy; a transcript.</def>  <rj><au>Walton.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <fld>(Mus.)</fld> <def>An arrangement of a composition for some other instrument or voice than that for which it was originally written, as the translating of a song, a vocal or instrumental quartet, or even an orchestral work, into a piece for the piano; an adaptation; an arrangement; -- a name applied by modern composers for the piano to a more or less fanciful and ornate reproduction on their own instrument of a song or other piece not originally intended for it; <as>as, Liszt's <ex>transcriptions</ex> of songs by Schubert</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Transcriptive</ent><br/
<hw>Tran*scrip"tive</hw> <pr>(-t<icr/v)</pr>, <pos>a.</pos> <def>Done as from a copy; having the style or appearance of a transcription.</def> <mark>[R.]</mark> -- <wordforms><wf>Tran*scrip"tive*ly</wf>, <pos>adv.</pos> <mark>[R.]</mark></wordforms>  <rj><au>Sir T. Browne.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Transcur</ent><br/
<hw>Trans*cur"</hw> <pr>(?)</pr>, <pos>v. i.</pos> <ety>[L. <ets>transcurrere</ets>, <ets>transcursum</ets>; <ets>trans</ets> across, over + <ets>currere</ets> to run.]</ety> <def>To run or rove to and fro.</def> <mark>[Obs.]</mark>  <rj><au>Bacon.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Transcurrence</ent><br/
<hw>Trans*cur"rence</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>transcurrens</ets>, p. pr. of <ets>transcurrere</ets>.]</ety> <def>A roving hither and thither.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Transcursion</ent><br/
<hw>Trans*cur"sion</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Cf. L. <ets>transcursio</ets> a passing over. See <er>Transcur</er>.]</ety> <def>A rambling or ramble; a passage over bounds; an excursion.</def> <mark>[Obs.]</mark>  <rj><au>Howell.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Transdialect</ent><br/
<hw>Trans*di"a*lect</hw> <pr>(?)</pr>, <pos>v. t.</pos> <ety>[Pref. <ets>trans-</ets> + <ets>dialect</ets>.]</ety> <def>To change or translate from one dialect into another.</def> <mark>[R.]</mark>  <rj><au>Bp. Warburton.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Transduction</ent><br/
<hw>Trans*duc"tion</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>transducere</ets>, <ets>traducere</ets>, <ets>-dictum</ets>, to lead across or over. See <er>Traduce</er>.]</ety> <def>The act of conveying over.</def> <mark>[R.]</mark>  <rj><au>Entick.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Transe</ent><br/
<hw>Transe</hw> <pr>(?)</pr>, <pos>n.</pos> <def>See <er>Trance</er>.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Transelementate</ent><br/
<ent>Transelement</ent><br/
<mhw>{ <hw>Trans*el"e*ment</hw> <pr>(?)</pr>, <hw>Trans*el`e*men"tate</hw> <pr>(?)</pr>, }</mhw> <pos>v. t.</pos> <ety>[Pref. <ets>trans-</ets> element.]</ety> <def>To change or transpose the elements of; to transubstantiate.</def> <mark>[Obs.]</mark>  <rj><au>Jer. Taylor.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Transelementation</ent><br/
<hw>Trans*el`e*men*ta"tion</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Cf. F. <ets>trans<eacute/l<eacute/mentation</ets>.]</ety> <fld>(Eccl.)</fld> <def>Transubstantiation.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Transenne</ent><br/
<hw>Tran"senne</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A transom.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Transept</ent><br/
<hw>Tran"sept</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Pref. <ets>trans-</ets> + L. <ets>septum</ets> an inclosure. See <er>Septum</er>.]</ety> <fld>(Arch.)</fld> <def>The transversal part of a church, which crosses at right angles to the greatest length, and between the nave and choir. In the basilicas, this had often no projection at its two ends. In Gothic churches these project these project greatly, and should be called the <xex>arms</xex> of the transept. It is common, however, to speak of the arms themselves as the <xex>transepts</xex>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Transexion</ent><br/
<hw>Tran*sex"ion</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Pref. <ets>trans-</ets> + L. <ets>sexus</ets> sex.]</ety> <def>Change of sex.</def> <mark>[Obs.]</mark>  <rj><au>Sir T. Browne.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Transfeminate</ent><br/
<hw>Trans*fem"i*nate</hw> <pr>(?)</pr>, <pos>v. t.</pos> <ety>[Pref. <ets>trans-</ets> + L. <ets>femina</ets> woman.]</ety> <def>To change into a woman, as a man.</def> <mark>[Obs. & R.]</mark>  <rj><au>Sir T. Browne.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Transfer</ent><br/
<hw>Trans*fer"</hw> <pr>(?)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Transferred</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Transferring</conjf>.]</vmorph> <ety>[L. <ets>transferre</ets>; <ets>trans</ets> across, over + <ets>ferre</ets> to bear: cf. F. <ets>transf<eacute/rer</ets>. See <er>Bear</er> to carry.]</ety> <sn>1.</sn> <def>To convey from one place or person another; to transport, remove, or cause to pass, to another place or person; <as>as, to <ex>transfer</ex> the laws of one country to another; to <ex>transfer</ex> suspicion</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To make over the possession or control of; to pass; to convey, as a right, from one person to another; to give; <as>as, the title to land is <ex>transferred</ex> by deed</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>To remove from one substance or surface to another; <as>as, to <ex>transfer</ex> drawings or engravings to a lithographic stone</as>.</def>  <rj><au>Tomlinson.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><syn><b>Syn.</b> -- To sell; give; alienate; estrange; sequester.</syn><br/
[<source>1913 Webster</source>]</p>

<p><ent>Transfer</ent><br/
<hw>Trans"fer</hw> <pr>(?)</pr>, <pos>n.</pos> <sn>1.</sn> <def>The act of transferring, or the state of being transferred; the removal or conveyance of a thing from one place or person to another.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Law)</fld> <def>The conveyance of right, title, or property, either real or personal, from one person to another, whether by sale, by gift, or otherwise.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>I shall here only consider it as a <qex>transfer</qex> of property.</q> <rj><qau>Burke.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>That which is transferred.</def> Specifically: --<br/
[<source>1913 Webster</source>]</p>

<p><sd>(a)</sd> <def>A picture, or the like, removed from one body or ground to another, as from wood to canvas, or from one piece of canvas to another.</def>  <rj><au>Fairholt.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sd>(b)</sd> <def>A drawing or writing printed off from one surface on another, as in ceramics and in many decorative arts.</def><br/
[<source>1913 Webster</source>]</p>

<p><sd>(c)</sd> <fld>(Mil.)</fld> <def>A soldier removed from one troop, or body of troops, and placed in another.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <fld>(Med.)</fld> <def>A pathological process by virtue of which a unilateral morbid condition on being abolished on one side of the body makes its appearance in the corresponding region upon the other side.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Transfer day</b></col>, <cd>one of the days fixed by the Bank of England for the transfer, free of charge, of bank stock and government funds. These days are the first five business days in the week before three o'clock.  Transfers may be made on Saturdays on payment of a fee of 2s. 6d.</cd> <au>Bithell.</au> -- <col><b>Transfer office</b></col>, <cd>an office or department where transfers of stocks, etc., are made.</cd> -- <col><b>Transfer paper</b></col>, <cd>a prepared paper used by draughtsmen, engravers, lithographers, etc., for transferring impressions.</cd> -- <col><b>Transfer table</b></col>. <fld>(Railroad)</fld> <cd>Same as <cref>Traverse table</cref>. See under <er>Traverse</er>.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Transferability</ent><br/
<hw>Trans*fer`a*bil"i*ty</hw> <pr>(?)</pr>, <pos>n.</pos> <def>The quality or state of being transferable.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Transferable</ent><br/
<hw>Trans*fer"a*ble</hw> <pr>(?; 277)</pr>, <pos>a.</pos> <ety>[Cf. F. <ets>transf<eacute/rable</ets>.]</ety> <sn>1.</sn> <def>Capable of being transferred or conveyed from one place or person to another.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Negotiable, as a note, bill of exchange, or other evidence of property, that may be conveyed from one person to another by indorsement or other writing; capable of being transferred with no loss of value; <as>as, the stocks of most public companies are <ex>transferable</ex>; some tickets are not <ex>transferable</ex></as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Transferee</ent><br/
<hw>Trans`fer*ee"</hw> <pr>(?)</pr>, <pos>n.</pos> <def>The person to whom a transfer in made.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Transference</ent><br/
<hw>Trans"fer*ence</hw> <pr>(?)</pr>, <pos>n.</pos> <def>The act of transferring; conveyance; passage; transfer.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Transferography</ent><br/
<hw>Trans`fer*og"ra*phy</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[<ets>Transfer</ets> + <ets>-graphy</ets>.]</ety> <def>The act or process of copying inscriptions, or the like, by making transfers.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Transferrence</ent><br/
<hw>Trans*fer"rence</hw> <pr>(?)</pr>, <pos>n.</pos> <def>See <er>Transference</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Transferrer</ent><br/
<hw>Trans*fer"rer</hw> <pr>(?)</pr>, <pos>n.</pos> <def>One who makes a transfer or conveyance.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Transferrible</ent><br/
<hw>Trans*fer"ri*ble</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Capable of being transferred; transferable.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Transfigurate</ent><br/
<hw>Trans*fig"u*rate</hw> <pr>(?)</pr>, <pos>v. t.</pos> <def>To transfigure; to transform.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Transfiguratien</ent><br/
<hw>Trans*fig`u*ra"tien</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>transfiguratio</ets>: cf. <ets>transfiguration</ets>.]</ety> <sn>1.</sn> <def>A change of form or appearance; especially, the supernatural change in the personal appearance of our Savior on the mount.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Eccl.)</fld> <def>A feast held by some branches of the Christian church on the 6th of August, in commemoration of the miraculous change above mentioned.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Transfigure</ent><br/
<hw>Trans*fig"ure</hw> <pr>(?; 135)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Transfigured</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Transfiguring</conjf>.]</vmorph> <ety>[F. <ets>transfigurer</ets>, L. <ets>transfigurare</ets>, <ets>transfiguratum</ets>; <ets>trans</ets> across, over + <ets>figurare</ets> to form, shape. See <er>Figure</er>, <pos>v. t.</pos>]</ety> <sn>1.</sn> <def>To change the outward form or appearance of; to metamorphose; to transform.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Especially, to change to something exalted and glorious; to give an ideal form to.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>[Jesus] was <qex>transfigured</qex> before them; and his face did shine as the sun, and his raiment was white as the light.</q> <rj><qau>Matt. xvii. 2.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Transfix</ent><br/
<hw>Trans*fix"</hw> <pr>(?)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Transfixed</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Transfixing</conjf>.]</vmorph> <ety>[L. <ets>transfixus</ets>, p. p. of <ets>transfigure</ets> to transfix; <ets>trans</ets> across, through + <ets>figere</ets> to fix, fasten. See <er>Fix</er>.]</ety> <def>To pierce through, as with a pointed weapon; to impale; <as>as, to <ex>transfix</ex> one with a dart</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Transfixion</ent><br/
<hw>Trans*fix"ion</hw> <pr>(?)</pr>, <pos>n.</pos> <def>The act of transfixing, or the state of being transfixed, or pierced.</def>  <rj><au>Bp. Hall.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Transfluent</ent><br/
<hw>Trans"flu*ent</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[Pref. <ets>trans-</ets> + <ets>fluent</ets>.]</ety> <sn>1.</sn> <def>Flowing or running across or through; <as>as, a <ex>transfluent</ex> stream</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Her.)</fld> <def>Passing or flowing through a bridge; -- said of water.</def>  <rj><au>Wright.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Transflux</ent><br/
<hw>Trans"flux</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Pref. <ets>trans-</ets> + <ets>flux</ets>.]</ety> <def>A flowing through, across, or beyond.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Transforate</ent><br/
<hw>Trans"fo*rate</hw> <pr>(?)</pr>, <pos>v. t.</pos> <ety>[L. <ets>transforatus</ets>, p. p. of <ets>transforare</ets> to pierce through; <ets>trans</ets> through + <ets>forare</ets> to bore.]</ety> <def>To bore through; to perforate.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Transform</ent><br/
<hw>Trans*form"</hw> <pr>(?)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Transformed</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Transforming</conjf>.]</vmorph> <ety>[L. <ets>transformare</ets>, <ets>transformatum</ets>; <ets>trans</ets> across, over + <ets>formare</ets> to from: cf. F. <ets>transformer</ets>. See <er>Form</er>, <pos>v. t.</pos>]</ety> <sn>1.</sn> <def>To change the form of; to change in shape or appearance; to metamorphose; <as>as, a caterpillar is ultimately <ex>transformed</ex> into a butterfly</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Love may <qex>transform</qex> me to an oyster.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To change into another substance; to transmute; <as>as, the alchemists sought to <ex>transform</ex> lead into gold</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>To change in nature, disposition, heart, character, or the like; to convert.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Be ye <qex>transformed</qex> by the renewing of your mind.</q> <rj><qau>Rom. xii. 2.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <fld>(Math.)</fld> <def>To change, as an algebraic expression or geometrical figure, into another from without altering its value.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Transform</ent><br/
<hw>Trans*form"</hw>, <pos>v. i.</pos> <def>To be changed in form; to be metamorphosed.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>His hair <qex>transforms</qex> to down.</q> <rj><qau>Addison.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Transformable</ent><br/
<hw>Trans*form"a*ble</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Capable of being transformed or changed.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Transformation</ent><br/
<hw>Trans`for*ma"tion</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>transformatio</ets>: cf. <ets>transformation</ets>.]</ety> <def>The act of transforming, or the state of being transformed; change of form or condition.</def> Specifically: --<br/
[<source>1913 Webster</source>]</p>

<p><sd>(a)</sd> <fld>(Biol.)</fld> <def>Any change in an organism which alters its general character and mode of life, as in the development of the germ into the embryo, the egg into the animal, the larva into the insect (metamorphosis), etc.; also, the change which the histological units of a tissue are prone to undergo. See <er>Metamorphosis</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><-- esp. the change from a normal to a cancerous state for a eukaryotic cell --><br/
[<source>1913 Webster</source>]</p>

<p><sd>(b)</sd> <fld>(Physiol.)</fld> <def>Change of one from of material into another, as in assimilation; metabolism; metamorphosis.</def><br/
[<source>1913 Webster</source>]</p>

<p><sd>(c)</sd> <fld>(Alchemy)</fld> <def>The imagined possible or actual change of one metal into another; transmutation.</def><br/
[<source>1913 Webster</source>]</p>

<p><sd>(d)</sd> <fld>(Theol.)</fld> <def>A change in disposition, heart, character, or the like; conversion.</def><br/
[<source>1913 Webster</source>]</p>

<p><sd>(e)</sd> <fld>(Math.)</fld> <def>The change, as of an equation or quantity, into another form without altering the value.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Transformative</ent><br/
<hw>Trans*form"a*tive</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[Cf. F. <ets>transformatif</ets>.]</ety> <def>Having power, or a tendency, to transform.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Transformer</ent><br/
<hw>Trans*form"er</hw> <pr>(?)</pr>, <pos>n.</pos> <sn>1.</sn> <def>One who, or that which, transforms. Specif. <fld>(Elec.)</fld>, an apparatus for producing from a given electrical current another current of different voltage.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fu>(1990's)</fu> <def>a toy consisting of a figure or representation of an object, constructed of interlocking parts in such a way that a series of turning or twisting motions of the parts can transform the object from one type of object to a different type of object, such as from an animal to an automobile.</def><br/
[<source>PJC</source>]</p>

<p><cs>-- <col><b>Multiple transformer</b></col>. <fld>(Elec.)</fld> <sd>(a)</sd> <cd>A transformer connected in multiple or in parallel with the primary circuit.</cd> <sd>(b)</sd> <cd>A transformer with more than one primary or more than one secondary coil.</cd> -- <col><b>Parallel transformer</b></col> <fld>(Elec.)</fld>, <cd>a transformer connected in parallel.</cd></cs><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Transformism</ent><br/
<hw>Trans*form"ism</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F. <ets>transformisme</ets>.]</ety> <fld>(Biol.)</fld> <def>The hypothesis, or doctrine, that living beings have originated by the modification of some other previously existing forms of living matter; -- opposed to <xex>abiogenesis</xex>.</def>  <rj><au>Huxley.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><-- p. 1529 --><br/
[<source>1913 Webster</source>]</p>

<p><ent>Transfreight</ent><br/
<hw>Trans*freight"</hw> <pr>(?)</pr>, <pos>v. i.</pos> <def>To transfrete.</def> <mark>[Obs.]</mark>  <rj><au>Waterhouse.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Transfretation</ent><br/
<hw>Trans`fre*ta"tion</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>transfretatio</ets>. See <er>Transfrete</er>.]</ety> <def>The act of passing over a strait or narrow sea.</def> <mark>[Obs.]</mark>  <rj><au>Sir J. Davies.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Transfrete</ent><br/
<hw>Trans*frete"</hw> <pr>(?)</pr>, <pos>v. i.</pos> <ety>[L. <ets>transfretare</ets>; <ets>trans</ets> across, over + <ets>fretum</ets> a strait: cf. OF. <ets>transfreter</ets>.]</ety> <def>To pass over a strait or narrow sea.</def> <altsp>[Written also <asp>transfreight</asp>.]</altsp> <mark>[Obs.]</mark>  <rj><au>E. Hall.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Transfugitive</ent><br/
<ent>Transfuge</ent><br/
<mhw>{ <hw>Trans"fuge</hw> <pr>(?)</pr>, <hw>Trans*fu"gi*tive</hw> <pr>(?)</pr>, }</mhw> <pos>n.</pos> <ety>[L. <ets>transfuga</ets>; <ets>trans</ets> across, over + <ets>fugere</ets> to flee.]</ety> <def>One who flees from one side to another; hence, a deserter; a turncoat; an apostate.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Transfund</ent><br/
<hw>Trans*fund"</hw> <pr>(?)</pr>, <pos>v. t.</pos> <ety>[L. <ets>transfundere</ets>; <ets>trans</ets> over, across + <ets>fundere</ets> to pour, pour out. See <er>Found</er> to cast, and cf. <er>Transfuse</er>.]</ety> <def>To pour from one vessel into another; to transfuse.</def> <mark>[Obs.]</mark>  <rj><au>Barrow.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Transfuse</ent><br/
<hw>Trans*fuse"</hw> <pr>(?)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Transfused</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Transfusing</conjf>.]</vmorph> <ety>[L. <ets>transfusus</ets>, p. p. of <ets>transfundere</ets>: cf. F. <ets>transfuser</ets>. See <er>Transfund</er>.]</ety> <sn>1.</sn> <def>To pour, as liquid, out of one vessel into another; to transfer by pouring.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Med.)</fld> <def>To transfer, as blood, from the veins or arteries of one man or animal to those of another.</def>
<-- The transfusion may proceded directly between animals, or the blood may be first drawn from the donor and stored, and later infused into the recipient. --><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>To cause to pass from to another; to cause to be instilled or imbibed; <as>as, to <ex>transfuse</ex> a spirit of patriotism into a man; to <ex>transfuse</ex> a love of letters</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Into thee such virtue and grace<br/
Immense I have <qex>transfused</qex>.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Transfusible</ent><br/
<hw>Trans*fu"si*ble</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Capable of being transfused; transferable by transfusion.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Transfusion</ent><br/
<hw>Trans*fu"sion</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>transfusio</ets>: cf. F. <ets>transfusion</ets>.]</ety> <sn>1.</sn> <def>The act of transfusing, or pouring, as liquor, out of one vessel into another.</def>  <rj><au>Howell.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Med.)</fld> <def>The act or operation of transferring the blood of one man or animal into the vascular system of another; also, the introduction of any fluid into the blood vessels, or into a cavity of the body from which it can readily be adsorbed into the vessels; intrafusion; <as>as, the peritoneal <ex>transfusion</ex> of milk</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Transfusive</ent><br/
<hw>Trans*fu"sive</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Tending to transfuse; having power to transfuse.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Transgress</ent><br/
<hw>Trans*gress"</hw> <pr>(?)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Transgressed</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Transgressing</conjf>.]</vmorph> <ety>[Cf. F. <ets>transgresser</ets>. See <er>Transgression</er>.]</ety> <sn>1.</sn> <def>To pass over or beyond; to surpass.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>Surpassing common faith, <qex>transgressing</qex> nature's law.</q> <rj><qau>Dryden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Hence, to overpass, as any prescribed as the <?/imit of duty; to break or violate, as a law, civil or moral.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>For man will hearken to his glozing lies,<br/
And easily <qex>transgress</qex> the sole command.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>To offend against; to vex.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>Why give you peace to this imperate beast<br/
That hath so long <qex>transgressed</qex> you ?</q> <rj><qau>Beau. & Fl.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Transgress</ent><br/
<hw>Trans*gress"</hw>, <pos>v. i.</pos> <def>To offend against the law; to sin.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Who <qex>transgressed</qex> in the thing accursed.</q> <rj><qau>I Chron. ii. 7.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Transgression</ent><br/
<hw>Trans*gres"sion</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>transgressio</ets> a going across, going over, transgression of the law, from <ets>transgredi</ets>, <ets>transgressus</ets>, to step across, go over; <ets>trans</ets> over, across + <ets>gradi</ets> to step, walk: cf. F. <ets>transgression</ets>. See <er>Grade</er>.]</ety> <def>The act of transgressing, or of passing over or beyond any law, civil or moral; the violation of a law or known principle of rectitude; breach of command; fault; offense; crime; sin.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Forgive thy people . . . all their <qex>transgressions</qex> wherein they have transgressed against thee.</q> <rj><qau>I Kings viii. 50.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>What rests, but that the mortal sentence pass<br/
On his <qex>transgression</qex>, death denounced that day ?</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>The <qex>transgression</qex> is in the stealer.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><syn><b>Syn.</b> -- Fault; offense; crime; infringement; misdemeanor; misdeed; affront; sin.</syn><br/
[<source>1913 Webster</source>]</p>

<p><ent>Transgressional</ent><br/
<hw>Trans*gres"sion*al</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Of pertaining to transgression; involving a transgression.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Transgressive</ent><br/
<hw>Trans*gress"ive</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[Cf. L. <ets>transgressivus</ets> passing over into another class. F. <ets>transgressif</ets>.]</ety> <def>Disposed or tending to transgress; faulty; culpable. -</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Transgressively</ent><br/
<hw>Trans*gress"ive*ly</hw>, <pos>adv.</pos><br/
[<source>1913 Webster</source>]</p>

<p><q>Adam, perhaps, . . . from the <qex>transgressive</qex> infirmities of himself, might have erred alone.</q> <rj><qau>Sir T. Browne.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Transgressor</ent><br/
<hw>Trans*gress"or</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L.: cf. F. <ets>transgresseur</ets>.]</ety> <def>One who transgresses; one who breaks a law, or violates a command; one who violates any known rule or principle of rectitude; a sinner.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The way of <qex>transgressors</qex> is hard.</q> <rj><qau>Prov. xiii. 15.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Transhape</ent><br/
<hw>Tran*shape"</hw> <pr>(?)</pr>, <pos>v. t.</pos> <def>To transshape.</def> <mark>[R.]</mark>  <rj><au>J. Webster (1623).</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tranship</ent><br/
<hw>Tran*ship"</hw> <pr>(?)</pr>, <pos>v. t.</pos> <def>Same as <er>Transship</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Transhipment</ent><br/
<hw>Tran*ship"ment</hw> <pr>(?)</pr>, <pos>n.</pos> <def>Same as <er>Transshipment</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Transhuman</ent><br/
<hw>Trans*hu"man</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[Pref. <ets>trans-</ets> + <ets>human</ets>.]</ety> <def>More than human; superhuman.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>Words may not tell of that <qex>transhuman</qex> change.</q> <rj><qau>H. F. Cary.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Transhumanize</ent><br/
<hw>Trans*hu"man*ize</hw> <pr>(?)</pr>, <pos>v. t.</pos> <def>To make more than human; to purity; to elevate above humanity.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>Souls purified by sorrow and self-denial, <qex>transhumanized</qex> to the divine abstraction of pure contemplation.</q> <rj><qau>Lowell.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Transiency</ent><br/
<ent>Transience</ent><br/
<mhw>{ <hw>Tran"sience</hw> <pr>(?)</pr>, <hw>Tran"sien*cy</hw> <pr>(?)</pr>, }</mhw> <pos>n.</pos> <def>The quality of being transient; transientness.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Transient</ent><br/
<hw>Tran"sient</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>transiens</ets>, <ets>-entis</ets>, p. pr. of <ets>transire</ets>, <ets>transitum</ets>, to go or pass over. See <er>Trance</er>.]</ety> <sn>1.</sn> <def>Passing before the sight or perception, or, as it were, moving over or across a space or scene viewed, and then disappearing; hence, of short duration; not permanent; not lasting or durable; not stationary; passing; fleeting; brief; transitory; <as>as, <ex>transient</ex> pleasure</as>.</def> <ldquo/Measured this <xex>transient</xex> world.<rdquo/  <rj><au>Milton.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Hasty; momentary; imperfect; brief; <as>as, a <ex>transient</ex> view of a landscape</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>Staying for a short time; not regular or permanent; <as>as, a <ex>transient</ex> guest; <ex>transient</ex> boarders</as>.</def> <mark>[Colloq. U. S.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><syn><b>Syn.</b> -- <er>Transient</er>, <er>Transitory</er>, <er>Fleeting</er>.</syn> <usage> <xex>Transient</xex> represents a thing as brief at the best; <xex>transitory</xex>, as liable at any moment to pass away. <xex>Fleeting</xex> goes further, and represents it as in the act of taking its flight. Life is <xex>transient</xex>; its joys are <xex>transitory</xex>; its hours are <xex>fleeting</xex>.</usage><br/
[<source>1913 Webster</source>]</p>

<p><q>What is loose love? A <qex>transient</qex> gust.</q> <rj><qau>Pope</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>If [we love] <qex>transitory</qex> things, which soon decay,<br/
Age must be loveliest at the latest day.</q> <rj><qau>Donne.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>O <qex>fleeting</qex> joys<br/
Of Paradise, dear bought with lasting woes.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p>-- <wordforms><wf>Tran"sient*ly</wf> <pr>(#)</pr>, <pos>adv.</pos> -- <wf>Tran"sient*ness</wf>, <pos>n.</pos></wordforms><br/
[<source>1913 Webster</source>]</p>

<p><ent>Transient</ent><br/
<hw>Tran"sient</hw>, <pos>n.</pos> <def>That which remains but for a brief time.</def>  <rj><au>Glanvill.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Transiliency</ent><br/
<ent>Transilience</ent><br/
<mhw>{ <hw>Tran*sil"i*ence</hw> <pr>(?)</pr>, <hw>Tran*sil"i*en*cy</hw> <pr>(?)</pr>, }</mhw> <pos>n.</pos> <ety>[L. <ets>transiliens</ets>, p. pr. of <ets>transilire</ets> to leap across or over; <ets>trans</ets> across, over + <ets>salire</ets> to leap.]</ety> <def>A leap across or from one thing to another.</def> <mark>[R.]</mark> <ldquo/An unadvised <xex>transiliency</xex>.<rdquo/  <rj><au>Glanvill.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Transire</ent><br/
||<hw>Trans*i"re</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>transire</ets> to pass through or across, to pass.]</ety> <fld>(End. Law)</fld> <def>A customhouse clearance for a coasting vessel; a permit.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Transisthmian</ent><br/
<hw>Trans*isth"mi*an</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Extending across an isthmus, as at Suez or Panama.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Transistor</ent><br/
<hw>Trans*ist"or</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[<ets>trans</ets>fer + res<ets>istor</ets>, from its ability to tranfer a current across a resistor.]</ety> <fld>(Electronics)</fld> <def>a component used in electronic devices consisting of three regions of at least two types of a semiconducting material, such as doped silicon, connected to each other and to three electrodes in a conducting path so as to modify the current or voltage in an electronic circuit.</def> <note><ex>Transistors</ex> are used in almost all modern electronic devices, having replaced the vacuum tube in most applications.  Since they are based on the electronic characteristics of solids, they are called <isa>solid-state devices</isa>.  Typically a <ex>transistor</ex> is composed of p, n, and p-type semiconductors in series, or of n, p, and n, with the center region being a thin layer between the two outer regions.  An electronic signal input to the central layer may be substantially amplified by such a device.  In integrated circuits, many thousands of <ex>transistors</ex> may be etched into a single small wafer of silicon.</note><br/
[<source>PJC</source>]</p>

<p><ent>Transit</ent><br/
<hw>Trans"it</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>transitus</ets>, from <ets>transire</ets> to go over: cf. F. <ets>transit</ets>. See <er>Transient</er>.]</ety> <sn>1.</sn> <def>The act of passing; passage through or over.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>In France you are now . . . in the <qex>transit</qex> from one form of government to another.</q> <rj><qau>Burke.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>The act or process of causing to pass; conveyance; <as>as, the <ex>transit</ex> of goods through a country</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>A line or route of passage or conveyance; <as>as, the Nicaragua <ex>transit</ex></as>.</def>  <rj><au>E. G. Squier.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <fld>(Astron.)</fld> <sd>(a)</sd> <def>The passage of a heavenly body over the meridian of a place, or through the field of a telescope.</def> <sd>(b)</sd> <def>The passage of a smaller body across the disk of a larger, as of Venus across the sun's disk, or of a satellite or its shadow across the disk of its primary.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <def>An instrument resembling a theodolite, used by surveyors and engineers; -- called also <altname>transit compass</altname>, and <altname>surveyor's transit</altname>.</def><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ The surveyor's transit differs from the theodolite in having the horizontal axis attached directly to the telescope which is not mounted in <universbold>Y</universbold>'s and can be turned completely over about the axis.</note><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Lower transit</b></col> <fld>(Astron.)</fld>, <cd>the passage of a heavenly body across that part of the meridian which is below the polar axis.</cd> -- <col><b>Surveyor's transit</b></col>. <cd>See <er>Transit</er>, 5, above.</cd> -- <col><b>Transit circle</b></col> <fld>(Astron.)</fld>, <cd>a transit instrument with a graduated circle attached, used for observing the time of transit and the declination at one observation. See <er>Circle</er>, <pos>n.</pos>, 3.</cd> -- <col><b>Transit compass</b></col>. <cd>See <er>Transit</er>, 5, above.</cd> -- <col><b>Transit duty</b></col>, <cd>a duty paid on goods that pass through a country.</cd> -- <col><b>Transit instrument</b></col>. <fld>(Astron.)</fld> <sd>(a)</sd> <cd>A telescope mounted at right angles to a horizontal axis, on which it revolves with its line of collimation in the plane of the meridian, -- used in connection with a clock for observing the time of transit of a heavenly body over the meridian of a place.</cd> <sd>(b)</sd> <fld>(Surv.)</fld> <cd>A surveyor's transit. See <er>Transit</er>, 5, above.</cd> -- <col><b>Transit trade</b></col> <fld>(Com.)</fld>, <cd>the business conected with the passage of goods through a country to their destination.</cd> -- <col><b>Upper transit</b></col> <fld>(Astron.)</fld>, <cd>the passage of a heavenly body across that part of the meridian which is above the polar axis.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Transit</ent><br/
<hw>Trans"it</hw>, <pos>v. t.</pos> <fld>(Astron.)</fld> <def>To pass over the disk of (a heavenly body).</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Transition</ent><br/
<hw>Tran*si"tion</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>transitio</ets>: cf. F. <ets>transition</ets>. See <er>Transient</er>.]</ety> <sn>1.</sn> <def>Passage from one place or state to another; charge; <as>as, the <ex>transition</ex> of the weather from hot to cold</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>There is no death, what seems so is <qex>transition</qex>.</q> <rj><qau>Longfellow.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Mus.)</fld> <def>A direct or indirect passing from one key to another; a modulation.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <fld>(Rhet.)</fld> <def>A passing from one subject to another.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>[He] with <qex>transition</qex> sweet, new speech resumes.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <fld>(Biol.)</fld> <def>Change from one form to another.</def><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ This word is sometimes pronounced <pr><it>tran*sish"un</it></pr>; but according to Walker, Smart, and most other authorities, the customary and preferable pronunciation is <pr><it>tran*sizh"un</it></pr>, although this latter mode violates analogy. Other authorities say <pr><it>tran*zish"un</it></pr>.</note><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Transition rocks</b></col> <fld>(Geol.)</fld>, <cd>a term formerly applied to the lowest uncrystalline stratified rocks (graywacke) supposed to contain no fossils, and so called because thought to have been formed when the earth was passing from an uninhabitable to a habitable state.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Transitional</ent><br/
<hw>Tran*si"tion*al</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Of or pertaining to transition; involving or denoting transition; <as>as, <ex>transitional</ex> changes; <ex>transitional</ex> stage</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Transitionary</ent><br/
<hw>Tran*si"tion*a*ry</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Transitional.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Transition zone</ent><br/
<hw>Tran*si"tion zone</hw>. <fld>(Biogeography)</fld> <def>The zone lying between the Boreal and Sonoran zones of North America. It includes an eastern or humid subdivision and a western arid one of corresponding temperature comprising the northern Great Plains and the lower slopes of the mountains of the western United States and Mexico. Called also <altname>Neutral zone</altname>.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Transitive</ent><br/
<hw>Tran"si*tive</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>transitivus</ets>: cf. F. <ets>transitif</ets>. See <er>Transient</er>.]</ety> <sn>1.</sn> <def>Having the power of making a transit, or passage.</def> <mark>[R.]</mark>  <rj><au>Bacon.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Effected by transference of signification.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>By far the greater part of the <qex>transitive</qex> or derivative applications of words depend on casual and unaccountable caprices of the feelings or the fancy.</q> <rj><qau>Stewart.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <fld>(Gram.)</fld> <def>Passing over to an object; expressing an action which is not limited to the agent or subject, but which requires an object to complete the sense; <as>as, a <ex>transitive</ex> verb, for example, he <ex>holds</ex> the book</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p>-- <wordforms><wf>Tran"si*tive*ly</wf>, <pos>adv.</pos> -- <wf>Tran"si*tive*ness</wf>, <pos>n.</pos></wordforms><br/
[<source>1913 Webster</source>]</p>

<p><ent>Transitorily</ent><br/
<hw>Tran"si*to*ri*ly</hw> <pr>(?)</pr>, <pos>adv.</pos> <def>In a transitory manner; with brief continuance.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Transitoriness</ent><br/
<hw>Tran"si*to*ri*ness</hw>, <pos>n.</pos> <def>The quality or state of being transitory; speedy passage or departure.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Transitory</ent><br/
<hw>Tran"si*to*ry</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>transitorius</ets>: cf. F. <ets>transitoire</ets>. See <er>Transient</er>.]</ety> <def>Continuing only for a short time; not enduring; fleeting; evanescent.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Comfort and succor all those who, in this <qex>transitory</qex> life, are in trouble.</q> <rj><qau>Bk. of Com. Prayer.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>It was not the <qex>transitory</qex> light of a comet, which shines and glows for a wile, and then . . . vanishes into nothing.</q> <rj><qau>South.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Transitory action</b></col> <fld>(Law)</fld>, <cd>an action which may be brought in any county, as actions for debt, and the like; -- opposed to <xex>local action</xex>.</cd>  <rj><au>Blackstone. Bouvier.</au></rj></cs><br/
[<source>1913 Webster</source>]</p>

<p><syn><b>Syn.</b> -- transient; short-lived; brief. See <er>Transient</er>.</syn><br/
[<source>1913 Webster</source>]</p>

<p><ent>Translatable</ent><br/
<hw>Trans*lat"a*ble</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Capable of being translated, or rendered into another language.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Translate</ent><br/
<hw>Trans*late"</hw> <pr>(?)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Translated</conjf>; <pos>p. pr. & vb. n.</pos> <conjf>Translating</conjf>.]</vmorph> <ety>[f. <ets>translatus</ets>, used as p. p. of <ets>transferre</ets> to transfer, but from a different root. See <er>Trans-</er>, and <er>Tolerate</er>, and cf. <er>Translation</er>.]</ety> <sn>1.</sn> <def>To bear, carry, or remove, from one place to another; to transfer; <as>as, to <ex>translate</ex> a tree</as>.</def> <mark>[Archaic]</mark>  <rj><au>Dryden.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>In the chapel of St. Catharine of Sienna, they show her head- the rest of her body being <qex>translated</qex> to Rome.</q> <rj><qau>Evelyn.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To change to another condition, position, place, or office; to transfer; hence, to remove as by death.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>To remove to heaven without a natural death.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>By faith Enoch was <qex>translated</qex>, that he should not see death; and was not found, because God had <qex>translated</qex>him.</q> <rj><qau>Heb. xi. 5.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <fld>(Eccl.)</fld> <def>To remove, as a bishop, from one see to another.</def> <ldquo/Fisher, Bishop of Rochester, when the king would have <xex>translated</xex> him from that poor bishopric to a better, . . . refused.<rdquo/  <rj><au>Camden.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <def>To render into another language; to express the sense of in the words of another language; to interpret; hence, to explain or recapitulate in other words.</def><br/
[<source>1913 Webster</source>]</p>

<p><q><qex>Translating</qex> into his own clear, pure, and flowing language, what he found in books well known to the world, but too bulky or too dry for boys and girls.</q> <rj><qau>Macaulay.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>6.</sn> <def>To change into another form; to transform.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Happy is your grace,<br/
That can <qex>translate</qex>the stubbornness of fortune<br/
Into so quiet and so sweet a style.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>7.</sn> <fld>(Med.)</fld> <def>To cause to remove from one part of the body to another; <as>as, to <ex>translate</ex> a disease</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>8.</sn> <def>To cause to lose senses or recollection; to entrance.</def> <mark>[Obs.]</mark>  <rj><au>J. Fletcher.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Translate</ent><br/
<hw>Trans*late</hw>, <pos>v. i.</pos> <def>To make a translation; to be engaged in translation.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Translation</ent><br/
<hw>Trans*la"tion</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F. <ets>translation</ets>, L. <ets>translatio</ets> a transferring, translation, version. See <er>Translate</er>, and cf. <er>Tralation</er>.]</ety> <sn>1.</sn> <def>The act of translating, removing, or transferring; removal; also, the state of being translated or removed; <as>as, the <ex>translation</ex> of Enoch; the <ex>translation</ex> of a bishop</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>The act of rendering into another language; interpretation; <as>as, the <ex>translation</ex> of idioms is difficult</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>That which is obtained by translating something a version; <as>as, a <ex>translation</ex> of the Scriptures</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <fld>(Rhet.)</fld> <def>A transfer of meaning in a word or phrase, a metaphor; a tralation.</def> <mark>[Obs.]</mark>  <rj><au>B. Jonson.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <fld>(Metaph.)</fld> <def>Transfer of meaning by association; association of ideas.</def>  <rj><au>A. Tucker.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>6.</sn> <fld>(Kinematics)</fld> <def>Motion in which all the points of the moving body have at any instant the same velocity and direction of motion; -- opposed to <xex>rotation</xex>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Translatitious</ent><br/
<hw>Trans`la*ti"tious</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[See <er>Tralatitious</er>.]</ety> <def>Metaphorical; tralatitious; also, foreign; exotic.</def> <mark>[Obs.]</mark>  <rj><au>Evelyn.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Translative</ent><br/
<hw>Trans*lat"ive</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>translativus</ets> that is to be transferred: cf. F. <ets>translatif</ets>.]</ety> <def>tropical; figurative; <as>as, a <ex>translative</ex> sense</as>.</def> <mark>[R.]</mark>  <rj><au>Puttenham.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Translator</ent><br/
<hw>Trans*lat"or</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>translator</ets>: cf. F. <ets>translateur</ets>.]</ety> <sn>1.</sn> <def>One who translates; esp., one who renders into another language; one who expresses the sense of words in one language by equivalent words in another.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Teleg.)</fld> <def>A repeating instrument.</def> <mark>[Eng.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Translatorship</ent><br/
<hw>Trans*lat"or*ship</hw>, <pos>n.</pos> <def>The office or dignity of a translator.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Translatory</ent><br/
<hw>Trans*lat"o*ry</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Serving to translate; transferring.</def> <mark>[R.]</mark>  <rj><au>Arbuthnot.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Translatress</ent><br/
<hw>Trans*lat"ress</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A woman who translates.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Translavation</ent><br/
<hw>Trans`la*va"tion</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Pref. <ets>trans-</ets> + L. <ets>lavatio</ets>, <ets>-onis</ets>, washing.]</ety> <def>A laving or lading from one vessel to another.</def> <mark>[Obs.]</mark>  <rj><au>Holland.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Transliterate</ent><br/
<hw>Trans*lit"er*ate</hw> <pr>(?)</pr>, <pos>v. t.</pos> <ety>[Pref. <ets>trans-</ets> + L. <ets>litera</ets>, <ets>littera</ets> letter.]</ety> <def>To express or represent in the characters of another alphabet; <as>as, to <ex>transliterate</ex> Sanskrit words by means of English letters</as>.</def>  <rj><au>A. J. Ellis.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Transliteration</ent><br/
<hw>Trans*lit`er*a"tion</hw> <pr>(?)</pr>, <pos>n.</pos> <def>The act or product of transliterating, or of expressing words of a language by means of the characters of another alphabet.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Translocation</ent><br/
<hw>Trans`lo*ca"tion</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Pref. <ets>trans-</ets> + <ets>location</ets>.]</ety> <def>removal of things from one place to another; substitution of one thing for another.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>There happened certain <qex>translocations</qex> at the deluge.</q> <rj><qau>Woodward.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Translucency</ent><br/
<ent>Translucence</ent><br/
<mhw>{ <hw>Trans*lu"cence</hw> <pr>(?)</pr>, <hw>Trans*lu"cen*cy</hw> <pr>(?)</pr>, }</mhw> <pos>n.</pos> <def>The quality or state of being translucent; clearness; partial transparency.</def>  <rj><au>Sir T. Browne.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><-- p. 1530 --><br/
[<source>1913 Webster</source>]</p>

<p><ent>Translucent</ent><br/
<hw>Trans*lu"cent</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>translucens</ets>, <ets>-entis</ets>, p. pr. of <ets>translucere</ets> to shine through; <ets>trans</ets> across, through = <ets>lucere</ets> to shine. See <er>Lucid</er>.]</ety> <sn>1.</sn> <def>Transmitting rays of light without permitting objects to be distinctly seen; partially transparent.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Transparent; clear.</def> <mark>[Poetic]</mark> <ldquo/Fountain or fresh current . . . <xex>translucent</xex>, pure.<rdquo/  <rj><au>Milton.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Replenished from the cool, <qex>translucent</qex> springs.</q> <rj><qau>Pope.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><syn><b>Syn.</b> -- <er>Translucent</er>, <er>Transparent</er>.</syn> <usage> A thing is <xex>translucent</xex> when it merely admits the passage of light, without enabling us to distinguish the color and outline of objects through it; it is <xex>transparent</xex> when we can clearly discern objects placed on the other side of it. Glass, water, etc., are <xex>transparent</xex>; ground glass is <xex>translucent</xex>; a <xex>translucent</xex> style.</usage><br/
[<source>1913 Webster</source>]</p>

<p><ent>Translucently</ent><br/
<hw>Trans*lu"cent*ly</hw>, <pos>adv.</pos> <def>In a translucent manner.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Translucid</ent><br/
<hw>Trans*lu"cid</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>translucidus</ets>; <ets>trans</ets> across, through + <ets>lucidus</ets> lucid: cf. F. <ets>translucide</ets>. See <er>Translucent</er>.]</ety> <def>Translucent.</def> <mark>[R.]</mark>  <rj><au>Bacon.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Translunary</ent><br/
<hw>Trans"lu*na*ry</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[Pref. <ets>trans-</ets> + L. <ets>luna</ets> moon.]</ety> <def>Being or lying beyond the moon; hence, ethereal; -- opposed to <xex>sublunary</xex>.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>Marlowe, bathed in the Thespian springs,<br/
Had in him those brave, <qex>translunary</qex> things<br/
That the first poets had.</q> <rj><qau>Drayton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Transmarine</ent><br/
<hw>Trans`ma*rine"</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>transmarinus</ets>; <ets>trans</ets> beyond + <ets>marinus</ets> marine: cf. F. <ets>transmarin</ets>. See <er>Marine</er>.]</ety> <def>Lying or being beyond the sea.</def>  <rj><au>Howell.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Transmeatable</ent><br/
<ent>Transmeable</ent><br/
<mhw>{ <hw>Trans"me*a*ble</hw> <pr>(?)</pr>, <hw>Trans`me*at"a*ble</hw> <pr>(?)</pr>, }</mhw> <pos>a.</pos> <ety>[L. <ets>transmeabilis</ets>.]</ety> <def>Capable of being passed over or traversed; passable.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Transmeate</ent><br/
<hw>Trans"me*ate</hw> <pr>(?)</pr>, <pos>v. t.</pos> <ety>[L. <ets>transmeatus</ets>, p. p. of <ets>transmeare</ets> to pass across; <ets>trans</ets> across, over + <ets>meare</ets> to go.]</ety> <def>To pass over or beyond.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Transmeation</ent><br/
<hw>Trans`me*a"tion</hw> <pr>(?)</pr>, <pos>n.</pos> <def>The act of transmeating; a passing through or beyond.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Transmew</ent><br/
<hw>Trans*mew"</hw> <pr>(?)</pr>, <pos>v. t. & i.</pos> <ety>[F. <ets>transmuer</ets>, L. <ets>transmutare</ets>. See <er>Transmute</er>.]</ety> <def>To transmute; to transform; to metamorphose.</def> <mark>[Archaic]</mark>  <rj><au>Chaucer. Spenser.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>To <qex>transmew</qex> thyself from a holy hermit into a sinful forester.</q> <rj><qau>Sir W. Scott.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Transmigrant</ent><br/
<hw>Trans"mi*grant</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>transmigrans</ets>, p. pr. See <er>Transmigrate</er>.]</ety> <def>Migrating or passing from one place or state to another; passing from one residence to another.</def> -- <def2><pos>n.</pos>  <def>One who transmigrates.</def></def2><br/
[<source>1913 Webster</source>]</p>

<p><ent>Transmigrate</ent><br/
<hw>Trans"mi*grate</hw> <pr>(?)</pr>, <pos>v. i.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Transmigrated</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Transmigrating</conjf>.]</vmorph> <ety>[L. <ets>transmigrare</ets>, <ets>transmigratum</ets>; <ets>trans</ets> across + <ets>migrare</ets> to migrate. See <er>Migrate</er>.]</ety> <sn>1.</sn> <def>To pass from one country or jurisdiction to another for the purpose of residence, as men or families; to migrate.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To pass from one body or condition into another.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Their may <qex>transmigrate</qex> into each other.</q> <rj><qau>Howell.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Transmigration</ent><br/
<hw>Trans`mi*gra"tion</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F. <ets>transmigration</ets>, L. <ets>transmigratio</ets>.]</ety> <sn>1.</sn> <def>The act of passing from one country to another; migration.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>The passing of the soul at death into another mortal body; metempsychosis.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Transmigrator</ent><br/
<hw>Trans"mi*gra`tor</hw> <pr>(?)</pr>, <pos>n.</pos> <def>One who transmigrates.</def>  <rj><au>J. Ellis.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Transmigratory</ent><br/
<hw>Trans*mi"gra*to*ry</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Passing from one body or state to another.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Transmissibility</ent><br/
<hw>Trans*mis`si*bil"i*ty</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Cf. F. <ets>transmissibilit<eacute/</ets>.]</ety> <def>The quality of being transmissible.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Transmissible</ent><br/
<hw>Trans*mis"si*ble</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[Cf. F. <ets>transmissible</ets>.]</ety> <def>Capable of being transmitted from one to another; capable of being passed through any body or substance.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Transmission</ent><br/
<hw>Trans*mis"sion</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>transmissio</ets>; cf. F. <ets>transmission</ets>. See <er>Transmit</er>.]</ety> <sn>1.</sn> <def>The act of transmitting, or the state of being transmitted; <as>as, the <ex>transmission</ex> of letters, writings, papers, news, and the like, from one country to another; the <ex>transmission</ex> of rights, titles, or privileges, from father to son, or from one generation to another</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Law)</fld> <def>The right possessed by an heir or legatee of transmitting to his successor or successors any inheritance, legacy, right, or privilege, to which he is entitled, even if he should die without enjoying or exercising it.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <fld>(Mech.)</fld> <def>The mechanism within a vehicle which transmits rotational power from the engine to the axle of the wheel propelling the vehicle; it includes the gears and gear-changing mechanism as well as the propeller shaft.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>The process or event of sending signals by means of a radio-frequency wave from an electronic transmitter to a receiving device.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Transmission dynamometer</ent><br/
<hw>Trans*mis"sion dy`na*mom"e*ter</hw>. <fld>(Mach.)</fld> <def>A dynamometer in which power is measured, without being absorbed or used up, during transmission.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Transmissionist</ent><br/
<hw>Trans*mis"sion*ist</hw>, <pos>n.</pos> <def>An adherent of a theory, the <col><b>transmission theory</b></col>, that the brain serves to <ldquo/transmit,<rdquo/ rather than to originate, conclusions, and hence that consciousness may exist independently of the brain.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Transmissive</ent><br/
<hw>Trans*mis"sive</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Capable of being transmitted; derived, or handed down, from one to another.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Itself a sun, it with <qex>transmissive</qex> light<br/
Enlivens worlds denied to human sight.</q> <rj><qau>Prior.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Transmit</ent><br/
<hw>Trans*mit"</hw> <pr>(?)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Transmitted</conjf>; <pos>p. pr. & vb. n.</pos> <conjf>Transmitting</conjf>.]</vmorph> <ety>[L. <ets>transmittere</ets>, <ets>transmissum</ets>; <ets>trans</ets> across, over + <ets>mittere</ets> to send: cf. F. <ets>transmettre</ets>. See <er>Missile</er>.]</ety> <sn>1.</sn> <def>To cause to pass over or through; to communicate by sending; to send from one person or place to another; to pass on or down as by inheritance; <as>as, to <ex>transmit</ex> a memorial; to <ex>transmit</ex> dispatches; to <ex>transmit</ex> money, or bills of exchange, from one country to another</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The ancientest fathers must be next removed, as Clement of Alexandria, and that Eusebian book of evangelic preparation, <qex>transmitting</qex> our ears through a hoard of heathenish obscenities to receive the gospel.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>The scepter of that kingdom continued to be <qex>transmitted</qex> in the dynasty of Castile.</q> <rj><qau>Prescott.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To suffer to pass through; <as>as, glass <ex>transmits</ex> light; metals <ex>transmit</ex>, or conduct, electricity</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Transmittal</ent><br/
<hw>Trans*mit"tal</hw> <pr>(?)</pr>, <pos>n.</pos> <def>Transmission.</def>  <rj><au>Swift.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Transmittance</ent><br/
<hw>Trans*mit"tance</hw> <pr>(?)</pr>, <pos>n.</pos> <def>Transmission.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Transmitter</ent><br/
<hw>Trans*mit"ter</hw> <pr>(?)</pr>, <pos>n.</pos> <def>One who, or that which, transmits; specifically, that portion of a telegraphic or telephonic instrument by means of which a message is sent; -- opposed to <contr>receiver</contr>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Transmittible</ent><br/
<hw>Trans*mit"ti*ble</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Capable of being transmitted; transmissible.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Transmogrification</ent><br/
<hw>Trans*mog`ri*fi*ca"tion</hw> <pr>(?)</pr>, <pos>n.</pos> <def>The act of transmogrifying, or the state of being transmogrified; transformation.</def> <mark>[Colloq.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>Clive, who wrote me about the <qex>transmogrification</qex> of our schoolfellow, an attorney's son.</q> <rj><qau>Thackeray.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Transmogrify</ent><br/
<hw>Trans*mog"ri*fy</hw> <pr>(?)</pr>, <pos>v. t.</pos> <ety>[A humorous coinage.]</ety> <def>To change into a different shape; to transform.</def> <mark>[Colloq.]</mark>  <rj><au>Fielding.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Transmove</ent><br/
<hw>Trans*move"</hw> <pr>(?)</pr>, <pos>v. t.</pos> <ety>[Pref. <ets>trans + move</ets>.]</ety> <def>To move or change from one state into another; to transform.</def> <mark>[Obs.]</mark>  <rj><au>Spenser.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Transmutability</ent><br/
<hw>Trans*mu`ta*bil"i*ty</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Cf. F. <ets>transmutabilit<eacute/</ets>.]</ety> <def>The quality of being transmutable.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Transmutable</ent><br/
<hw>Trans*mut"a*ble</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[Cf. F. <ets>transmutable</ets>. See <er>Transmute</er>.]</ety> <def>Capable of being transmuted or changed into a different substance, or into into something of a different form a nature; transformable.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The fluids and solids of an animal body are easily <qex>transmutable</qex> into one another.</q> <rj><qau>Arbuthnot.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p>-- <wordforms><wf>Trans*mut"a*ble*ness</wf>, <pos>n.</pos> -- <wf>Trans*mut"a*bly</wf>, <pos>adv.</pos></wordforms><br/
[<source>1913 Webster</source>]</p>

<p><ent>Transmutation</ent><br/
<hw>Trans`mu*ta"tion</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F. <ets>transmutation</ets>, L. <ets>transmutatio</ets>. See <er>Transmute</er>.]</ety> <sn>1.</sn> <def>The act of transmuting, or the state of being transmuted; <as>as, the <ex>transmutation</ex> of metals</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Geom.)</fld> <def>The change or reduction of one figure or body into another of the same area or solidity, but of a different form, as of a triangle into a square.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <fld>(Biol.)</fld> <def>The change of one species into another, which is assumed to take place in any development theory of life; transformism.</def>  <rj><au>Bacon.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Transmutation of metals</b></col> <fld>(Alchem.)</fld>, <cd>the conversion of base metals into gold or silver, a process often attempted by the alchemists. See <er>Alchemy</er>, and <cref>Philosopher's stone</cref>, under <er>Philosopher</er>.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Transmutationist</ent><br/
<hw>Trans`mu*ta"tion*ist</hw>, <pos>n.</pos> <def>One who believes in the transmutation of metals or of species.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Transmute</ent><br/
<hw>Trans*mute"</hw> <pr>(?)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Transmuted</conjf>; <pos>p. pr. & vb. n.</pos> <conjf>Transmuting</conjf>.]</vmorph> <ety>[L. <ets>transmutare</ets>, <ets>transmutatum</ets>; <ets>trans</ets> across + <ets>mutare</ets> to change. See <er>Mutable</er>, and cf. <er>Transmew</er>.]</ety> <def>To change from one nature, form, or substance, into another; to transform.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The caresses of parents and the blandishments of friends <qex>transmute</qex> us into idols.</q> <rj><qau>Buckminster.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q><qex>Transmuting</qex> sorrow into golden joy<br/
Free from alloy.</q> <rj><qau>H. Smith.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Transmuter</ent><br/
<hw>Trans*mut"er</hw> <pr>(?)</pr>, <pos>n.</pos> <def>One who transmutes.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Transmutual</ent><br/
<hw>Trans*mu"tu*al</hw> <pr>(?; 135)</pr>, <pos>a.</pos> <ety>[Pref. <ets>trans + mutual</ets>.]</ety> <def>Reciprocal; commutual.</def> <mark>[R.]</mark>  <rj><au>Coleridge.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Transnatation</ent><br/
<hw>Trans`na*ta"tion</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>transnatare</ets> to swim over; <ets>trans</ets> across, over + <ets>natare</ets> to swim.]</ety> <def>The act of swimming across, as a river.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Transnature</ent><br/
<hw>Trans*na"ture</hw> <pr>(?; 135)</pr>, <pos>v. t.</pos> <ety>[Pref. <ets>trans-</ets> + <ets>nature</ets>.]</ety> <def>To transfer or transform the nature of.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>We are transelemented, or <qex>transnatured</qex>.</q> <rj><qau>Jewel.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Transom</ent><br/
<hw>Tran"som</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Probably fr. L. <ets>transtrum</ets> a crossbeam, transom, from <ets>trans</ets> across.  Cf. <er>Trestle</er>.]</ety> <sn>1.</sn> <fld>(Arch.)</fld> <def>A horizontal crossbar in a window, over a door, or between a door and a window above it. <xex>Transom</xex> is the horizontal, as <xex>mullion</xex> is the vertical, bar across an opening. See <xex>Illust.</xex> of <er>Mullion</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Naut.)</fld> <def>One of the principal transverse timbers of the stern, bolted to the sternpost and giving shape to the stern structure; -- called also <altname>transsummer</altname>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <fld>(Gun.)</fld> <def>The piece of wood or iron connecting the cheeks of some gun carriages.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <fld>(Surg.)</fld> <def>The vane of a cross-staff.</def>  <rj><au>Chambers.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <fld>(Railroad)</fld> <def>One of the crossbeams connecting the side frames of a truck with each other.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Transom knees</b></col> <fld>(Shipbuilding)</fld>, <cd>knees bolted to the transoms and after timbers.</cd> -- <col><b>Transom window</b></col>. <fld>(Arch.)</fld> <sd>(a)</sd> <cd>A window divided horizontally by a transom or transoms.</cd> <sd>(b)</sd> <cd>A window over a door, with a transom between.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Transpadane</ent><br/
<hw>Trans"pa*dane`</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>transpadanus</ets>; <ets>trans</ets> across + <ets>Padus</ets> the Po.]</ety> <def>Lying or being on the further side of the river Po with reference to Rome, that is, on the north side; -- opposed to <xex>cispadane</xex>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Transpalatine</ent><br/
<hw>Trans*pal"a*tine</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[Pref. <ets>trans-</ets> + <ets>palatine</ets>.]</ety> <fld>(Anat.)</fld> <def>Situated beyond or outside the palatine bone; -- said of a bone in the skull of some reptiles.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Transpare</ent><br/
<hw>Trans*pare"</hw> <pr>(?)</pr>, <pos>v. t. & i.</pos> <ety>[See <er>Transparent</er>.]</ety> <def>To be, or cause to be, transparent; to appear, or cause to appear, or be seen, through something.</def> <mark>[Obs.]</mark>  <rj><au>Stirling.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Transparence</ent><br/
<hw>Trans*par"ence</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Cf. F. <ets>transparence</ets>.]</ety> <def>The quality or state of being transparent; transparency.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Transparency</ent><br/
<hw>Trans*par"en*cy</hw> <pr>(?)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Transparencies</plw> <pr>(#)</pr>.</plu> <ety>[Cf. F. <ets>transparence</ets>.]</ety> <sn>1.</sn> <def>The quality or condition of being transparent; transparence.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>That which is transparent; especially, a picture painted on thin cloth or glass, or impressed on porcelain, or the like, to be viewed by natural or artificial light, which shines through it.</def>  <rj><au>Fairholt.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Transparent</ent><br/
<hw>Trans*par"ent</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[F., from LL. <ets>transparens</ets>, <ets>-entis</ets>, p. pr. of <ets>transparere</ets> to be transparent; L. <ets>trans</ets> across, through + <ets>parere</ets> to appear. See <er>Appear</er>.]</ety> <sn>1.</sn> <def>Having the property of transmitting rays of light, so that bodies can be distinctly seen through; pervious to light; diaphanous; pellucid; <as>as, <ex>transparent</ex> glass; a <ex>transparent</ex> diamond</as>; -- opposed to <ant>opaque</ant>.</def> <ldquo/<xex>Transparent</xex> elemental air.<rdquo/  <rj><au>Milton.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Admitting the passage of light; open; porous; <as>as, a <ex>transparent</ex> veil</as>.</def>  <rj><au>Dryden.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><syn><b>Syn.</b> -- Translucent; pellucid; clear; bright; limpid; lucid; diaphanous. See <er>Translucent</er>.</syn><br/
[<source>1913 Webster</source>]</p>

<p>-- <wordforms><wf>Trans*par"ent*ly</wf>, <pos>adv.</pos> -- <wf>Trans*par"ent*ness</wf>, <pos>n.</pos></wordforms><br/
[<source>1913 Webster</source>]</p>

<p><ent>Transpass</ent><br/
<hw>Trans*pass"</hw> <pr>(?)</pr>, <pos>v. t.</pos> <ety>[Pref. <ets>trans-</ets> + <ets>pass</ets>: cf. LL. <ets>transpassare</ets>.  Cf. <er>Trespass</er>.]</ety> <def>To pass over; <as>as, Alexander <ex>transpassed</ex> the river</as>.</def> <mark>[Obs.]</mark>  <rj><au>J. Gregory.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Transpass</ent><br/
<hw>Trans*pass"</hw>, <pos>v. i.</pos> <def>To pass by; to pass away.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Transpassable</ent><br/
<hw>Trans*pass"a*ble</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Capable of being transpassed, or crossed over.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Transpatronize</ent><br/
<hw>Trans*pat"ron*ize</hw> <pr>(?)</pr>, <pos>v. t.</pos> <ety>[<ets>Trans-</ets> + <ets>patronize</ets>.]</ety> <def>To transfer the patronage of.</def> <mark>[Obs.]</mark>  <rj><au>Warner.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Transpeciate</ent><br/
<hw>Tran*spe"ci*ate</hw> <pr>(?)</pr>, <pos>v. t.</pos> <ety>[Pref. <ets>trans-</ets> + L. <ets>species</ets> form.]</ety> <def>To change from one species to another; to transform.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>Power to <qex>transpeciate</qex> a man into a horse.</q> <rj><qau>Sir T. Browne.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Transpicuous</ent><br/
<hw>Tran*spic"u*ous</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>transpicere</ets> to see or look through + <ets>specere</ets>, <ets>spicere</ets>, to see.  Cf. <er>Conspicuous</er>.]</ety> <def>Transparent; pervious to the sight.</def> <mark>[R.]</mark> <ldquo/The wide, <xex>transpicuous</xex> air.<rdquo/  <rj><au>Milton.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Transpierce</ent><br/
<hw>Trans*pierce"</hw> <pr>(?)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Transpierced</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Transpiercing</conjf> <pr>(?)</pr>.]</vmorph> <ety>[Pref. <ets>trans-</ets> + <ets>pierce</ets>: cf. F. <ets>transpercer</ets>.]</ety> <def>To pierce through; to penetrate; to permeate; to pass through.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The sides <qex>transpierced</qex> return a rattling sound.</q> <rj><qau>Dryden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Transpirable</ent><br/
<hw>Tran*spir"a*ble</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[Cf. F. <ets>transpirable</ets>.]</ety> <def>Capable of being transpired, or of transpiring.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Transpiration</ent><br/
<hw>Tran`spi*ra"tion</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F. <ets>transpiration</ets>.]</ety> <sn>1.</sn> <fld>(Physiol.)</fld> <def>The act or process of transpiring or excreting in the form of vapor; exhalation, as through the skin or other membranes of the body; <as>as, pulmonary <ex>transpiration</ex>, or the excretion of aqueous vapor from the lungs</as>. Perspiration is a form of transpiration.</def>  <rj><au>Cudworth.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(bot.)</fld> <def>The evaporation of water, or exhalation of aqueous vapor, from cells and masses of tissue.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <fld>(Physics)</fld> <def>The passing of gases through fine tubes, porous substances, or the like; <as>as, <ex>transpiration</ex> through membranes</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Transpiratory</ent><br/
<hw>Tran*spir"a*to*ry</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Of or relating to transpiration.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Transpire</ent><br/
<hw>Tran*spire"</hw> <pr>(?)</pr>, <pos>v. i.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Transpired</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Transpiring</conjf>.]</vmorph> <ety>[F. <ets>transpirer</ets>; L. <ets>trans</ets> across, through + <ets>spirare</ets> to breathe. See <er>Spirit</er>.]</ety> <sn>1.</sn> <fld>(Physiol.)</fld> <def>To pass off in the form of vapor or insensible perspiration; to exhale.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Bot.)</fld> <def>To evaporate from living cells.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>To escape from secrecy; to become public; <as>as, the proceedings of the council soon <ex>transpired</ex></as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The story of Paulina's and Maximilian's mutual attachment had <qex>transpired</qex> through many of the travelers.</q> <rj><qau>De Quincey.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>To happen or come to pass; to occur.</def><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ This sense of the word, which is of comparatively recent introduction, is common in the United States, especially in the language of conversation and of newspaper writers, and is used to some extent in England. Its use, however, is censured by critics of both countries.</note>
<-- still common in 1995 --><br/
[<source>1913 Webster</source>]</p>

<p><ent>Transpire</ent><br/
<hw>Tran*spire"</hw>, <pos>v. t.</pos> <sn>1.</sn> <fld>(Physiol.)</fld> <def>To excrete through the skin; to give off in the form of vapor; to exhale; to perspire.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Bot.)</fld> <def>To evaporate (moisture) from living cells.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Transplace</ent><br/
<hw>Trans*place"</hw> <pr>(?)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Transplaced</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Transplacing</conjf> <pr>(?)</pr>.]</vmorph> <ety>[Pref. <ets>trans-</ets> + <ets>place</ets>.]</ety> <def>To remove across some space; to put in an opposite or another place.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>It [an obelisk] was <qex>transplaced</qex> . . . from the left side of the Vatican into a more eminent place.</q> <rj><qau>Bp. Wilkins.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Transplant</ent><br/
<hw>Trans*plant"</hw> <pr>(?)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Transplanted</conjf>; <pos>p. pr. & vb. n.</pos> <conjf>Transplanting</conjf>.]</vmorph> <ety>[F. <ets>transplanter</ets>, L. <ets>transplantare</ets>; <ets>trans</ets> across, over + <ets>plantare</ets> to plant. See <er>Plant</er>.]</ety> <sn>1.</sn> <def>To remove, and plant in another place; <as>as, to <ex>transplant</ex> trees</as>.</def>  <rj><au>Dryden.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To remove, and settle or establish for residence in another place; <as>as, to <ex>transplant</ex> inhabitants</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Being <qex>transplanted</qex> out of his cold, barren diocese of St. David into a warmer climate.</q> <rj><qau>Clarendon.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Transplantation</ent><br/
<hw>Trans`plan*ta"tion</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Cf. F. <ets>transplantation</ets>.]</ety> <sn>1.</sn> <def>The act of transplanting, or the state of being transplanted; also, removal.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The <qex>transplantation</qex> of Ulysses to Sparta.</q> <rj><qau>Broome.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Surg.)</fld> <def>The removal of tissues from a healthy part, and the insertion of them in another place where there is a lesion; <as>as, the <ex>transplantation</ex> of tissues in autoplasty</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><-- <sn>3.</sn> <fld>(Surg.)</fld> <def>The removal of a bodily organ or of tissues from one person, and the insertion of them into another person to replace a damaged organ or tissue; <as>as, the <ex>transplantation</ex> of a heart, kidney, or liver</as>.</def> --><br/
[<source>1913 Webster</source>]</p>

<p><ent>Transplanter</ent><br/
<hw>Trans*plant"er</hw> <pr>(?)</pr>, <pos>n.</pos> <def>One who transplants; also, a machine for transplanting trees.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Transplendency</ent><br/
<hw>Tran*splen"den*cy</hw> <pr>(?)</pr>, <pos>n.</pos> <def>Quality or state of being transplendent.</def> <mark>[R.]</mark>  <rj><au>Dr. H. More.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Transplendent</ent><br/
<hw>Tran*splen"dent</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[<ets>Trans-</ets> + <ets>splendent</ets>.]</ety> <def>Resplendent in the highest degree.</def> <mark>[R.]</mark> -- <wordforms><wf>Tran*splen"dent*ly</wf>, <pos>adv.</pos> <mark>[R.]</mark></wordforms><br/
[<source>1913 Webster</source>]</p>

<p><ent>Transport</ent><br/
<hw>Trans*port"</hw> <pr>(?)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Transported</conjf>; <pos>p. pr. & vb. n.</pos> <conjf>Transporting</conjf>.]</vmorph> <ety>[F. <ets>transporter</ets>, L. <ets>transportare</ets>; <ets>trans</ets> across + <ets>portare</ets> to carry. See <er>Port</er> bearing, demeanor.]</ety> <sn>1.</sn> <def>To carry or bear from one place to another; to remove; to convey; <as>as, to <ex>transport</ex> goods; to <ex>transport</ex> troops.</as></def>  <rj><au>Hakluyt.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To carry, or cause to be carried, into banishment, as a criminal; to banish.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>To carry away with vehement emotion, as joy, sorrow, complacency, anger, etc.; to ravish with pleasure or ecstasy; <as>as, music <ex>transports</ex> the soul</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>[They] laugh as if <qex>transported</qex> with some fit<br/
Of passion.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>We shall then be <qex>transported</qex> with a nobler . . . wonder.</q> <rj><qau>South.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Transport</ent><br/
<hw>Trans"port</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F. See <er>Transport</er>, <pos>v.</pos>]</ety> <sn>1.</sn> <def>Transportation; carriage; conveyance.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The Romans . . . stipulated with the Carthaginians to furnish them with ships for <qex>transport</qex> and war.</q> <rj><qau>Arbuthnot.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A vessel employed for transporting, especially for carrying soldiers, warlike stores, or provisions, from one place to another, or to convey convicts to their destination; -- called also <altname>transport ship</altname>, <altname>transport vessel</altname>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>Vehement emotion; passion; ecstasy; rapture.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>With <qex>transport</qex> views the airy rule his own,<br/
And swells on an imaginary throne.</q> <rj><qau>Pope.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Say not, in <qex>transports</qex> of despair,<br/
That all your hopes are fled.</q> <rj><qau>Doddridge.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>A convict transported, or sentenced to exile.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Transportability</ent><br/
<hw>Trans*port`a*bil"i*ty</hw> <pr>(?)</pr>, <pos>n.</pos> <def>The quality or state of being transportable.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Transportable</ent><br/
<hw>Trans*port"a*ble</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[Cf. F. <ets>transportable</ets>.]</ety> <sn>1.</sn> <def>Capable of being transported.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Incurring, or subject to, the punishment of transportation; <as>as, a <ex>transportable</ex> offense</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Transportal</ent><br/
<hw>Trans*port"al</hw> <pr>(?)</pr>, <pos>n.</pos> <def>Transportation; the act of removing from one locality to another.</def> <ldquo/The <xex>transportal</xex> of seeds in the wool or fur of quadrupeds.<rdquo/  <rj><au>Darwin.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Transportance</ent><br/
<hw>Trans*port"ance</hw> <pr>(?)</pr>, <pos>n.</pos> <def>Transportation.</def> <mark>[Obs.]</mark> <ldquo/Give me swift <xex>transportance</xex>.<rdquo/  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><-- p. 1531 --><br/
[<source>1913 Webster</source>]</p>

<p><ent>Transportant</ent><br/
<hw>Trans*port"ant</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Transporting; <?/avishing; <as>as, <ex>transportant</ex> love</as>.</def> <mark>[Obs.]</mark>  <rj><au>Dr. H. More.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Transportation</ent><br/
<hw>Trans`por*ta"tion</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>transportatio</ets>: cf. F. <ets>transportation</ets>.]</ety> <sn>1.</sn> <def>The act of transporting, or the state of being transported; carriage from one place to another; removal; conveyance.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>To provide a vessel for their <qex>transportation</qex>.</q> <rj><qau>Sir H. Wotton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Transport; ecstasy.</def> <mark>[R.]</mark>  <rj><au>South.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Transported</ent><br/
<hw>Trans*port"ed</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Conveyed from one place to another; figuratively, carried away with passion or pleasure; entranced.</def> -- <wordforms><wf>Trans*port"ed*ly</wf>, <pos>adv.</pos> -- <wf>Trans*port"ed*ness</wf>, <pos>n.</pos></wordforms><br/
[<source>1913 Webster</source>]</p>

<p><ent>Transporter</ent><br/
<hw>Trans*port"er</hw> <pr>(?)</pr>, <pos>n.</pos> <def>One who transports.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Transporting</ent><br/
<hw>Trans*port"ing</hw>, <pos>a.</pos> <def>That transports; fig., ravishing.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Your <qex>transporting</qex> chords ring out.</q> <rj><qau>Keble.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Transportingly</ent><br/
<hw>Trans*port"ing*ly</hw>, <pos>adv.</pos> <def>So as to transport.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Transportment</ent><br/
<hw>Trans*port"ment</hw> <pr>(?)</pr>, <pos>n.</pos> <def>The act of transporting, or the state of being transported; transportation.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Transposable</ent><br/
<hw>Trans*pos"a*ble</hw> <pr>(?)</pr>, <pos>a.</pos> <def>That may transposed; <as>as, a <ex>transposable</ex> phrase</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Transposal</ent><br/
<hw>Trans*pos"al</hw> <pr>(?)</pr>, <pos>n.</pos> <def>The act of transposing, or the state of being transposed; transposition.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Transpose</ent><br/
<hw>Trans*pose"</hw> <pr>(?)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Transposed</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Transposing</conjf>.]</vmorph> <ety>[F. <ets>transposer</ets>; pref. <ets>trans-</ets> (L. <ets>trans</ets> across) + <ets>poser</ets> to put. See <er>Pose</er>.]</ety><br/
[<source>1913 Webster</source>]</p>

<p><sn>1.</sn> <def>To change the place or order of; to substitute one for the other of; to exchange, in respect of position; <as>as, to <ex>transpose</ex> letters, words, or propositions</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To change; to transform; to invert.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>Things base and vile, holding no quantity,<br/
Love can <qex>transpose</qex> to form and dignity.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <fld>(Alg.)</fld> <def>To bring, as any term of an equation, from one side over to the other, without destroying the equation; thus, if <mathex>a + b = c</mathex>, and we make <mathex>a = c - b</mathex>, then <it>b</it> is said to be <xex>transposed</xex>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <fld>(Gram.)</fld> <def>To change the natural order of, as words.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <fld>(Mus.)</fld> <def>To change the key of.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Transposer</ent><br/
<hw>Trans*pos"er</hw> <pr>(?)</pr>, <pos>n.</pos> <def>One who transposes.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Transposition</ent><br/
<hw>Trans`po*si"tion</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F. <ets>transposition</ets>, from L. <ets>transponere</ets>, <ets>transpositum</ets>, to set over, remove, transfer; <ets>trans</ets> across, over + <ets>ponere</ets> to place. See <er>Position</er>.]</ety> <def>The act of transposing, or the state of being transposed.</def> Specifically: --<br/
[<source>1913 Webster</source>]</p>

<p><sd>(a)</sd> <fld>(Alg.)</fld> <def>The bringing of any term of an equation from one side over to the other without destroying the equation.</def><br/
[<source>1913 Webster</source>]</p>

<p><sd>(b)</sd> <fld>(Gram.)</fld> <def>A change of the natural order of words in a sentence; as, the Latin and Greek languages admit <xex>transposition</xex>, without inconvenience, to a much greater extent than the English.</def><br/
[<source>1913 Webster</source>]</p>

<p><sd>(c)</sd> <fld>(Mus.)</fld> <def>A change of a composition into another key.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Transpositional</ent><br/
<hw>Trans`po*si"tion*al</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Of or pertaining to transposition; involving transposition.</def>  <rj><au>Pegge.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Transpositive</ent><br/
<hw>Trans*pos"i*tive</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Made by transposing; consisting in transposition; transposable.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Transprint</ent><br/
<hw>Trans*print"</hw> <pr>(?)</pr>, <pos>v. t.</pos> <ety>[Pref. <ets>trans-</ets> + <ets>print</ets>.]</ety> <def>To transfer to the wrong place in printing; to print out of place.</def> <mark>[R.]</mark>  <rj><au>Coleridge.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Transprose</ent><br/
<hw>Trans*prose"</hw> <pr>(?)</pr>, <pos>v. t.</pos> <ety>[Pref. <ets>trans-</ets> + <ets>prose</ets>.]</ety> <def>To change from prose into verse; to versify; also, to change from verse into prose.</def> <mark>[Obs.]</mark>  <rj><au>Dryden.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Transregionate</ent><br/
<hw>Trans*re"gion*ate</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[Pref. <ets>trans-</ets> + <ets>region</ets>.]</ety> <def>Foreign.</def> <mark>[Obs.]</mark>  <rj><au>Holinshed.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Transshape</ent><br/
<hw>Trans*shape"</hw> <pr>(?)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Transshaped</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Transshaping</conjf>.]</vmorph> <ety>[Pref. <ets>trans-</ets> + <ets>shape</ets>.]</ety> <def>To change into another shape or form; to transform.</def> <altsp>[Written also <asp>transhape</asp>.]</altsp>  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Transship</ent><br/
<hw>Trans*ship"</hw> <pr>(?)</pr>, <pos>v. t.</pos> <ety>[Pref. <ets>trans-</ets> + <ets>ship</ets>.]</ety> <def>To transfer from one ship or conveyance to another.</def> <altsp>[Written also <asp>tranship</asp>.]</altsp><br/
[<source>1913 Webster</source>]</p>

<p><ent>Transshipment</ent><br/
<hw>Trans*ship"ment</hw> <pr>(?)</pr>, <pos>n.</pos> <def>The act of transshipping, or transferring, as goods, from one ship or conveyance to another.</def> <altsp>[Written also <asp>transhipment</asp>.]</altsp><br/
[<source>1913 Webster</source>]</p>

<p><ent>Transsummer</ent><br/
<hw>Trans"sum`mer</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Naut.)</fld> <def>See <er>Transom</er>, 2.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Transubstantiate</ent><br/
<hw>Tran`sub*stan"ti*ate</hw> <pr>(?)</pr>, <pos>v. t.</pos> <ety>[LL. <ets>transubstantiatus</ets>, p. p. of <ets>transubstantiare</ets> to transubstantiate; L. <ets>trans</ets> across, over + <ets>substantia</ets> substance. See <er>Substance</er>.]</ety> <sn>1.</sn> <def>To change into another substance.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>The spider love which <qex>transubstantiates</qex> all,<br/
And can convert manna to gall.</q> <rj><qau>Donne.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(R. C. Theol.)</fld> <def>To change, as the sacramental elements, bread and wine, into the flesh and blood of Christ.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Transubstantiation</ent><br/
<hw>Tran`sub*stan`ti*a"tion</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[LL. <ets>transubstantiatio</ets>: cf. F. <ets>transsubstantiation</ets>.]</ety> <sn>1.</sn> <def>A change into another substance.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(R. C. Theol.)</fld> <def>The doctrine held by Roman Catholics, that the bread and wine in the Mass is converted into the body and blood of Christ; -- distinguished from <contr>consubstantiation</contr>, and <contr>impanation</contr>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Transubstantiator</ent><br/
<hw>Tran`sub*stan"ti*a`tor</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Cf. F. <ets>transsubstantiateur</ets>.]</ety> <def>One who maintains the doctrine of transubstantiation.</def>  <rj><au>Barrow.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Transudation</ent><br/
<hw>Tran`su*da"tion</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Cf. F. <ets>transsudation</ets>.]</ety> <sn>1.</sn> <def>The act or process of transuding.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Physics)</fld> <def>Same as <er>Exosmose</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Transudatory</ent><br/
<hw>Tran*su"da*to*ry</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Of or pertaining to transudation; passing by transudation.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Transude</ent><br/
<hw>Tran*sude"</hw> <pr>(?)</pr>, <pos>v. i.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Transuded</conjf>; <pos>p. pr. & vb. n.</pos> <conjf>Transuding</conjf>.]</vmorph> <ety>[Pref. <ets>trans-</ets> + L. <ets>sudare</ets> to sweat: cf. F. <ets>transsuder</ets>.]</ety> <def>To pass, as perspirable matter does, through the pores or interstices of textures; <as>as, liquor may <ex>transude</ex> through leather or wood</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Transume</ent><br/
<hw>Tran*sume"</hw> <pr>(?)</pr>, <pos>v. t.</pos> <ety>[L. <ets>transumere</ets>, <ets>transsumere</ets>, to take from one to another; <ets>trans</ets> across + <ets>sumere</ets> to take.]</ety> <def>To change; to convert.</def> <mark>[R.]</mark>  <rj><au>Crashaw.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Transumpt</ent><br/
<hw>Tran*sumpt"</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>transumere</ets>, <ets>transumptum</ets>, to take from one to another, in LL., to transcribe. See <er>Transume</er>.]</ety> <def>A copy or exemplification of a record; a transcript.</def> <mark>[Obs.]</mark>  <rj><au>Lord Herbert.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Transumption</ent><br/
<hw>Tran*sump"tion</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>transumptio</ets>.]</ety> <def>Act of taking from one place to another.</def> <mark>[R.]</mark>  <rj><au>South.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Transumptive</ent><br/
<hw>Tran*sump"tive</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>transumptivus</ets>.]</ety> <def>Taking from one to another; metaphorical.</def> <mark>[R.]</mark> <ldquo/A <xex>transumptive</xex> kind of speech.<rdquo/  <rj><au>Drayton.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Fictive, descriptive, digressive, <qex>transumptive</qex>, and withal definitive.</q> <rj><qau>Lowell.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Transvasate</ent><br/
<hw>Trans*va"sate</hw> <pr>(?)</pr>, <pos>v. t.</pos> <ety>[See <er>Transvasation</er>.]</ety> <def>To pour out of one vessel into another.</def> <mark>[Obs.]</mark>  <rj><au>Cudworth.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Transvasation</ent><br/
<hw>Trans`va*sa"tion</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Pref. <ets>trans-</ets> + L. <ets>vas</ets>, <ets>vasis</ets>, vessel.]</ety> <def>The act or process of pouring out of one vessel into another.</def> <mark>[Obs.]</mark>  <rj><au>Holland.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Transvection</ent><br/
<hw>Trans*vec"tion</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>transvectio</ets>, from <ets>transvehere</ets> to carry across; <ets>trans</ets> across + <ets>vehere</ets> to carry.]</ety> <def>The act of conveying or carrying over.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Transverberate</ent><br/
<hw>Trans*ver"ber*ate</hw> <pr>(?)</pr>, <pos>v. t.</pos> <ety>[L. <ets>transverberatus</ets>, p. p. of <ets>transverberare</ets> to strike or pierce through.]</ety> <def>To beat or strike through.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Transversal</ent><br/
<hw>Trans*ver"sal</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[Cf. F. <ets>transversal</ets>. See <er>Transverse</er>.]</ety> <def>Running or lying across; transverse; <as>as, a <ex>transversal</ex> line</as>.</def> -- <wordforms><wf>Trans*ver"sal*ly</wf>, <pos>adv.</pos></wordforms><br/
[<source>1913 Webster</source>]</p>

<p><ent>Transversal</ent><br/
<hw>Trans*ver"sal</hw>, <pos>n.</pos> <ety>[Cf. F. <ets>transversale</ets>.]</ety> <fld>(Geom.)</fld> <def>A straight line which traverses or intersects any system of other lines, as a line intersecting the three sides of a triangle or the sides produced.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Transverse</ent><br/
<hw>Trans*verse"</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>transversus</ets>, p. p.  of <ets>transvertere</ets> to turn on direct across; <ets>trans</ets> across + <ets>vertere</ets> to turn: cf. F. <ets>transverse</ets>. See <er>Verse</er>, and cf. <er>Traverse</er>.]</ety> <def>Lying or being across, or in a crosswise direction; athwart; -- often opposed to <ant>longitudinal</ant>.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Transverse axis</b></col> (of an ellipse or hyperbola) <fld>(Geom.)</fld>, <cd>that axis which passes through the foci.</cd> -- <col><b>Transverse partition</b></col> <fld>(Bot.)</fld>, <cd>a partition, as of a pericarp, at right angles with the valves, as in the siliques of mustard.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Transverse</ent><br/
<hw>Trans"verse</hw> <pr>(?)</pr>, <pos>n.</pos> <sn>1.</sn> <def>Anything that is transverse or athwart.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Geom.)</fld> <def>The longer, or transverse, axis of an ellipse.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Transverse</ent><br/
<hw>Trans*verse"</hw> <pr>(?)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Transversed</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Transversing</conjf>.]</vmorph> <def>To overturn; to change.</def> <mark>[R.]</mark>  <rj><au>C. Leslie.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Transverse</ent><br/
<hw>Trans*verse"</hw>, <pos>v. t.</pos> <ety>[Pref. <ets>trans-</ets> + <ets>verse</ets>, n.  Cf.<er>Transpose</er>.]</ety> <def>To change from prose into verse, or from verse into prose.</def> <mark>[Obs.]</mark>  <rj><au>Duke of Buckingham.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Transversely</ent><br/
<hw>Trans*verse"ly</hw>, <pos>adv.</pos> <def>In a transverse manner.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Transversion</ent><br/
<hw>Trans*ver"sion</hw> <pr>(?)</pr>, <pos>n.</pos> <def>The act of changing from prose into verse, or from verse into prose.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Transvert</ent><br/
<hw>Trans*vert"</hw> <pr>(?)</pr>, <pos>v. t.</pos> <ety>[L. <ets>transvertere</ets>. See <er>Transverse</er>, a.]</ety> <def>To cause to turn across; to transverse.</def> <mark>[Obs.]</mark>  <rj><au>Craft of Lovers (1448).</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Transvertible</ent><br/
<hw>Trans*vert"i*ble</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Capable of being transverted.</def> <mark>[R.]</mark>  <rj><au>Sir T. Browne.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Transvolation</ent><br/
<hw>Trans`vo*la"tion</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>transvolare</ets> to fly over or across; <ets>trans</ets> across + <ets>volare</ets> to fly.]</ety> <def>The act of flying beyond or across.</def>  <rj><au>Jer. Taylor.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Trant</ent><br/
<hw>Trant</hw> <pr>(?)</pr>, <pos>v. i.</pos> <ety>[Cf. OD. <ets>tranten</ets> to walk slowly, LG. & D. <ets>trant</ets> walk, pace.]</ety> <def>To traffic in an itinerary manner; to peddle.</def> <altsp>[Written also <asp>traunt</asp>.]</altsp> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tranter</ent><br/
<hw>Trant"er</hw> <pr>(?)</pr>, <pos>n.</pos> <def>One who trants; a peddler; a carrier.</def> <altsp>[Written also <asp>traunter</asp>.]</altsp> <mark>[Obs. or Prov. Eng.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Trap</ent><br/
<hw>Trap</hw> <pr>(?)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Trapped</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Trapping</conjf>.]</vmorph> <ety>[Akin to OE. <ets>trappe</ets> trappings, and perhaps from an Old French word of the same origin as E. <ets>drab</ets> a kind of cloth.]</ety> <def>To dress with ornaments; to adorn; -- said especially of horses.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Steeds . . . that <qex>trapped</qex> were in steel all glittering.</q> <rj><qau>Chaucer.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>To deck his hearse, and <qex>trap</qex> his tomb-black steed.</q> <rj><qau>Spenser.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>There she found her palfrey <qex>trapped</qex><br/
 In purple blazoned with armorial gold.</q> <rj><qau>Tennyson.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Trap</ent><br/
<hw>Trap</hw>, <pos>n.</pos> <ety>[Sw. <ets>trapp</ets>; akin to <ets>trappa</ets> stairs, Dan. <ets>trappe</ets>, G. <ets>treppe</ets>, D. <ets>trap</ets>; -- so called because the rocks of this class often occur in large, tabular masses, rising above one another, like steps. See <er>Tramp</er>.]</ety> <fld>(Geol.)</fld> <def>An old term rather loosely used to designate various dark-colored, heavy igneous rocks, including especially the feldspathic-augitic rocks, basalt, dolerite, amygdaloid, etc., but including also some kinds of diorite. Called also <altname>trap rock</altname>.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><mcol><col><b>Trap tufa</b></col>, <col><b>Trap tuff</b></col></mcol>, <cd>a kind of fragmental rock made up of fragments and earthy materials from trap rocks.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Trap</ent><br/
<hw>Trap</hw>, <pos>a.</pos> <def>Of or pertaining to trap rock; <as>as, a <ex>trap</ex> dike</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Trap</ent><br/
<hw>Trap</hw>, <pos>n.</pos> <ety>[OE. <ets>trappe</ets>, AS. <ets>treppe</ets>; akin to OD. <ets>trappe</ets>, OHG. <ets>trapo</ets>; probably fr. the root of E. <ets>tramp</ets>, as that which is trod upon: cf. F. <ets>trappe</ets>, which is trod upon: cf. F. <ets>trappe</ets>, which perhaps influenced the English word.]</ety> <sn>1.</sn> <def>A machine or contrivance that shuts suddenly, as with a spring, used for taking game or other animals; <as>as, a <ex>trap</ex> for foxes</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>She would weep if that she saw a mouse<br/
Caught in a <qex>trap</qex>.</q> <rj><qau>Chaucer.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Fig.: A snare; an ambush; a stratagem; any device by which one may be caught unawares.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Let their table be made a snare and a <qex>trap</qex>.</q> <rj><qau>Rom. xi. 9.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>God and your majesty<br/
Protect mine innocence, or I fall into<br/
The <qex>trap</qex> is laid for me!</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>A wooden instrument shaped somewhat like a shoe, used in the game of trapball. It consists of a pivoted arm on one end of which is placed the ball to be thrown into the air by striking the other end. Also, a machine for throwing into the air glass balls, clay pigeons, etc., to be shot at.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>The game of trapball.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <def>A bend, sag, or partitioned chamber, in a drain, soil pipe, sewer, etc., arranged so that the liquid contents form a seal which prevents passage of air or gas, but permits the flow of liquids.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>6.</sn> <def>A place in a water pipe, pump, etc., where air accumulates for want of an outlet.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>7.</sn> <def>A wagon, or other vehicle.</def> <mark>[Colloq.]</mark>  <rj><au>Thackeray.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>8.</sn> <def>A kind of movable stepladder.</def>  <rj><au>Knight.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Trap stairs</b></col>, <cd>a staircase leading to a trapdoor.</cd> -- <col><b>Trap tree</b></col> <fld>(Bot.)</fld> <cd>the jack; -- so called because it furnishes a kind of birdlime. See 1st <er>Jack</er>.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Trap</ent><br/
<hw>Trap</hw> <pr>(?)</pr>, <pos>v. t.</pos> <ety>[AS. <ets>treppan</ets>. See <er>Trap</er> a snare.]</ety><br/
[<source>1913 Webster</source>]</p>

<p><sn>1.</sn> <def>To catch in a trap or traps; <as>as, to <ex>trap</ex> foxes</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Fig.: To insnare; to take by stratagem; to entrap.</def> <ldquo/I <xex>trapped</xex> the foe.<rdquo/  <rj><au>Dryden.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>To provide with a trap; <as>as, to <ex>trap</ex> a drain; to <ex>trap</ex> a sewer pipe</as>. See 4th <er>Trap</er>, 5.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Trap</ent><br/
<hw>Trap</hw>, <pos>v. i.</pos> <def>To set traps for game; to make a business of trapping game; <as>as, to <ex>trap</ex> for beaver</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Trapan</ent><br/
<hw>Tra*pan"</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[OF. <ets>trappan</ets>. See <er>Trap</er>, and cf. <er>Trepan</er> a snare.]</ety> <def>A snare; a stratagem; a trepan. See 3d <er>Trepan</er>.</def>  <rj><au>South.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Trapan</ent><br/
<hw>Tra*pan"</hw>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Trapanned</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Trapanning</conjf>.]</vmorph> <def>To insnare; to catch by stratagem; to entrap; to trepan.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Having some of his people <qex>trapanned</qex> at Baldivia.</q> <rj><qau>Anson.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Trapanner</ent><br/
<hw>Tra*pan"ner</hw> <pr>(?)</pr>, <pos>n.</pos> <def>One who trapans, or insnares.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Trapball</ent><br/
<hw>Trap"ball`</hw> <pr>(?)</pr>, <pos>n.</pos> <def>An old game of ball played with a trap. See 4th <er>Trap</er>, 4.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Trapdoor</ent><br/
<hw>Trap"door`</hw> <pr>(?)</pr>, <pos>n.</pos> <sn>1.</sn> <fld>(Arch.)</fld> <def>A lifting or sliding door covering an opening in a roof or floor.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Mining)</fld> <def>A door in a level for regulating the ventilating current; -- called also <altname>weather door</altname>.</def>  <rj><au>Raymond.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Trapdoor spider</b></col> <fld>(Zool.)</fld>, <cd>any one of several species of large spiders which make a nest consisting of a vertical hole in the earth, lined with a hinged lid, like a trapdoor. Most of the species belong to the genus <gen>Cteniza</gen>, as the California species (<spn>Cteniza Californica</spn>).</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Trape</ent><br/
<hw>Trape</hw> <pr>(?)</pr>, <pos>v. i.</pos> <ety>[See <er>Tramp</er>, and cf. <er>Traipse</er>.]</ety> <def>To walk or run about in an idle or slatternly manner; to traipse.</def> <mark>[Obs. or Colloq.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Trapes</ent><br/
<hw>Trapes</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[See <er>Trape</er>.]</ety> <def>A slattern; an idle, sluttish, or untidy woman.</def> <mark>[Obs. or Colloq.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Trapes</ent><br/
<hw>Trapes</hw>, <pos>v. i.</pos> <def>To go about in an idle or slatternly fashion; to trape; to traipse.</def> <mark>[Colloq.]</mark>  <rj><au>Thackeray.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Trapezate</ent><br/
<hw>Trap"e*zate</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[See <er>Trapezium</er>.]</ety> <def>Having the form of a trapezium; trapeziform.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Trapeze</ent><br/
<hw>Tra*peze"</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Cf. F. <ets>trap<egrave/ze</ets>.]</ety> <sn>1.</sn> <fld>(Geom.)</fld>  <def>A trapezium. See <er>Trapezium</er>, 1.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A swinging horizontal bar, suspended at each end by a rope; -- used by gymnasts.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Trapeziform</ent><br/
<hw>Tra*pe"zi*form</hw> <pr>(?)</pr>, <pos>a.</pos>  <ety>[<ets>Trapezium</ets> + <ets>-form</ets>: cf. F. <ets>trap<eacute/ziforme</ets>.]</ety> <def>Having the form of a trapezium; trapezoid.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Trapezium</ent><br/
<hw>Tra*pe"zi*um</hw> <pr>(?)</pr>, <pos>n.</pos>; <plu><it>pl.</it> E. <plw>Trapeziums</plw> <pr>(#)</pr>, L. <plw>Trapezia</plw> <pr>(#)</pr>.</plu> <ety>[NL., fr. Gr. <?/ a little table, an irregular four-sided figure, dim. of <?/ a table, for <?/; <?/ (see <er>Tetra-</er>) + <?/ foot, akin to <?/ foot; hence, originally, a table with four feet. See <er>Foot</er>.]</ety> <sn>1.</sn> <fld>(Geom.)</fld> <def>A plane figure bounded by four right lines, of which no two are parallel.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Anat.)</fld> <sd>(a)</sd> <def>A bone of the carpus at  the base of the first metacarpal, or thumb.</def> <sd>(b)</sd> <def>A region on the ventral side of the brain, either just back of the pons Varolii, or, as in man, covered by the posterior extension of its transverse fibers.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Trapezohedral</ent><br/
<hw>Trap`e*zo*he"dral</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Of, pertaining to, or resembling, a trapezohedron.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Trapezohedron</ent><br/
<hw>Trap`e*zo*he"dron</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[NL., from <ets>trapezium</ets> + Gr. <?/ seat, base, fr. <?/ to sit.]</ety> <fld>(Crystalloq.)</fld> <sd>(a)</sd> <def>A solid bounded by twenty-four equal and similar trapeziums; a tetragonal trisoctahedron. See the Note under <er>Trisoctahedron</er>.</def> <sd>(b)</sd> <def>A tetartohedral solid of the hexagonal system, bounded by six trapezoidal planes. The faces of this form are common on quartz crystals.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Trapezoid</ent><br/
<hw>Trap"e*zoid</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Gr. <?/ trapezoid-shaped; <?/ table + <?/ shape, likeness: cf. F. <ets>trap<eacute/zo<ium/de</ets>. See <er>Trapezium</er>.]</ety> <sn>1.</sn> <fld>(Geom.)</fld> <def>A plane four-sided figure, having two sides parallel to each other.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Anat.)</fld> <def>A bone of the carpus at the base of the second metacarpal, or index finger.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Trapezoid</ent><br/
<hw>Trap"e*zoid</hw>, <pos>a.</pos> <sn>1.</sn> <def>Having the form of a trapezoid; trapezoidal; <as>as, the  <ex>trapezoid</ex> ligament  which connects the coracoid process and the clavicle</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Anat.)</fld> <def>Of or pertaining to the trapezoid ligament; <as>as, the <ex>trapezoid</ex> line</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Trapezoidal</ent><br/
<hw>Trap`e*zoid"al</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[Cf. F. <ets>trap<eacute/zo<ium/dal</ets>.]</ety><br/
[<source>1913 Webster</source>]</p>

<p><sn>1.</sn> <def>Having the form of a trapezoid; trapezoid.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Min.)</fld> <def>Trapezohedral.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Traphole</ent><br/
<hw>Trap"hole</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Mil.)</fld> <def>See <er>Trou-de-loup</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Trappean</ent><br/
<hw>Trap"pe*an</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[Cf. F. <ets>trapp<eacute/en</ets>. See <er>Trap</er> a kind of rock.]</ety> <fld>(Min.)</fld> <def>Of or pertaining to trap; being of the nature of trap.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Trapper</ent><br/
<hw>Trap"per</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[From <er>Trap</er> to insnare.]</ety> <sn>1.</sn> <def>One who traps animals; one who makes a business of trapping animals for their furs.</def>  <rj><au>W. Irving.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Mining)</fld> <def>A boy who opens and shuts a trapdoor in a gallery or level.</def>  <rj><au>Raymond.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Trappings</ent><br/
<hw>Trap"pings</hw> <pr>(?)</pr>, <pos>n. pl.</pos> <ety>[From <er>Trap</er> to dress with ornaments.]</ety> <sn>1.</sn> <def>That which serves to trap or adorn; ornaments; dress; superficial decorations.</def><br/
[<source>1913 Webster</source>]</p>

<p><q><qex>Trappings</qex> of life, for ornament, not use.</q> <rj><qau>Dryden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>These but the <qex>trappings</qex> and the suits of woe.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Specifically, ornaments to be put on horses.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Caparisons and steeds,<br/
Bases and tinsel <qex>trappings</qex>.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Trappist</ent><br/
<hw>Trap"pist</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F. <ets>trappiste</ets>.]</ety> <fld>(R. C. Ch.)</fld> <def>A monk belonging to a branch of the Cistercian Order, which was established by <person>Armand de Ranc<eacute/</person> in 1660 at the monastery of La Trappe in Normandy.  Extreme austerity characterizes their discipline.  They were introduced permanently into the United States in 1848, and have monasteries in Iowa and Kentucky.</def><br/
[<source>1913 Webster</source>]</p>

<p><-- p. 1532 --><br/
[<source>1913 Webster</source>]</p>

<p><ent>Trappous</ent><br/
<hw>Trap"pous</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[From <er>Trap</er> a kind of rock.]</ety> <fld>(Min.)</fld> <def>Of or performance to trap; resembling trap, or partaking of its form or qualities; trappy.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Trappures</ent><br/
<hw>Trap"pures</hw> <pr>(?)</pr>, <pos>n. pl.</pos> <ety>[See <er>Trap</er> to dress.]</ety> <def>Trappings for a horse.</def> <mark>[Obs.]</mark>  <rj><au>Chaucer.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Trappy</ent><br/
<hw>Trap"py</hw> <pr>(?)</pr>, <pos>a.</pos> <fld>(Min.)</fld> <def>Same as <er>Trappous</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Traps</ent><br/
<hw>Traps</hw> <pr>(?)</pr>, <pos>n. pl.</pos> <ety>[See <er>Trappings</er>, and <er>Trap</er> to dress.]</ety> <def>Small or portable articles for dress, furniture, or use; goods; luggage; things.</def> <mark>[Colloq.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Trap shooting</ent><br/
<hw>Trap shooting</hw>. <fld>(Sport)</fld> <def>Shooting at pigeons liberated, or glass balls or clay pigeons sprung into the air, from a trap.</def> -- <wordforms><wf>Trap shooter</wf>.</wordforms><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Trapstick</ent><br/
<hw>Trap"stick`</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A stick used in playing the game of trapball; hence, fig., a slender leg.</def>  <rj><au>Addison.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Trash</ent><br/
<hw>Trash</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Cf. Icel. <ets>tros</ets> rubbish, leaves, and twigs picked up for fuel, <ets>trassi</ets> a slovenly fellow, Sw. <ets>trasa</ets> a rag, tatter.]</ety> <sn>1.</sn> <def>That which is worthless or useless; rubbish; refuse.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Who steals my purse steals <qex>trash</qex>.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>A haunch of venison would be <qex>trash</qex> to a Brahmin.</q> <rj><qau>Landor.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Especially, loppings and leaves of trees, bruised sugar cane, or the like.</def><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ In the West Indies, the decayed leaves and stems of canes are called <xex>field trash</xex>; the bruised or macerated rind of canes is called <xex>cane trash</xex>; and both are called <xex>trash</xex>.</note>  <rj><au>B. Edwards.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>A worthless person.</def> <mark>[R.]</mark> <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>A collar, leash, or halter used to restrain a dog in pursuing game.</def>  <rj><au>Markham.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Trash ice</b></col>, <cd>crumbled ice mixed with water.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Trash</ent><br/
<hw>Trash</hw>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Trashed</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Trashing</conjf>.]</vmorph> <sn>1.</sn> <def>To free from trash, or worthless matter; hence, to lop; to crop, as to <xex>trash</xex> the rattoons of sugar cane.</def>  <rj><au>B. Edwards.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To treat as trash, or worthless matter; hence, to spurn, humiliate, or crush.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>To hold back by a trash or leash, as a dog in pursuing game; hence, to retard, encumber, or restrain; to clog; to hinder vexatiously.</def> <mark>[R.]</mark>  <rj><au>Beau. & Fl.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><-- 4. To turn into trash, usu. maliciously; to destroy; vandalize. --><br/
[<source>1913 Webster</source>]</p>

<p><ent>Trash</ent><br/
<hw>Trash</hw>, <pos>v. i.</pos> <def>To follow with violence and trampling.</def> <mark>[R.]</mark>  <rj><au>The Puritan (1607).</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Trashily</ent><br/
<hw>Trash"i*ly</hw> <pr>(?)</pr>, <pos>adv.</pos> <def>In a trashy manner.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Trashiness</ent><br/
<hw>Trash"i*ness</hw>, <pos>n.</pos> <def>The quality or state of being trashy.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Trashy</ent><br/
<hw>Trash"y</hw> <pr>(?)</pr>, <pos>a.</pos> <amorph>[<pos>Compar.</pos> <adjf>Trashier</adjf> <pr>(?)</pr>; <pos>superl.</pos> <adjf>Trashiest</adjf>.]</amorph> <def>Like trash; containing much trash; waste; rejected; worthless; useless; <as>as, a <ex>trashy</ex> novel</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Trass</ent><br/
<hw>Trass</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[D. <ets>tras</ets> or Gr. <ets>trass</ets>, probably fr. It. <ets>terrazzo</ets> terrace. See <er>Terrace</er>.]</ety> <fld>(Geol.)</fld> <def>A white to gray volcanic tufa, formed of decomposed trachytic cinders; -- sometimes used as a cement. Hence, a coarse sort of plaster or mortar, durable in water, and used to line cisterns and other reservoirs of water.</def> <altsp>[Formerly written also <asp>tarras</asp>, <asp>tarrace</asp>, <asp>terras</asp>.]</altsp><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ The Dutch <xex>trass</xex> is made by burning and grinding a soft grayish rock found on the lower Rhine.</note><br/
[<source>1913 Webster</source>]</p>

<p><ent>Traulism</ent><br/
<hw>Trau"lism</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Gr. <?/ a lisping, fr. <?/ to lisp, to mispronounce.]</ety> <def>A stammering or stuttering.</def> <mark>[Obs.]</mark>  <rj><au>Dalgarno.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Traumatic</ent><br/
<hw>Trau*mat"ic</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>traumaticus</ets>, Gr. <?/, from <?/, <?/, a wound: cf. F. <ets>traumatique</ets>.]</ety> <fld>(Med.)</fld> <sd>(a)</sd> <def>Of or pertaining to wounds; applied to wounds.</def> <au>Coxe.</au> <sd>(b)</sd> <def>Adapted to the cure of wounds; vulnerary.</def> <au>Wiseman.</au> <sd>(c)</sd> <def>Produced by wounds; <as>as, <ex>traumatic</ex> tetanus</as>.</def> -- <def2><pos>n.</pos>  <def>A traumatic medicine.</def></def2>
<-- also applied figuratively to mental harm, and to experiences causing severe mental upset. --><br/
[<source>1913 Webster</source>]</p>

<p><ent>Traumatism</ent><br/
<hw>Trau"ma*tism</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Med.)</fld> <def>A wound or injury directly produced by causes external to the body; also, violence producing a wound or injury; <as>as, rupture of the stomach caused by <ex>traumatism</ex></as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Traunce</ent><br/
<hw>Traunce</hw> <pr>(?)</pr>, <pos>n. & v.</pos> <def>See <er>Trance</er>.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Traunt</ent><br/
<hw>Traunt</hw> <pr>(?)</pr>, <pos>v. i.</pos> <def>Same as <er>Trant</er>.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Traunter</ent><br/
<hw>Traunt"er</hw> <pr>(?)</pr>, <pos>n.</pos> <def>Same as <er>Tranter</er>.</def> <mark>[Obs. or Prov. Eng.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Travail</ent><br/
<hw>Trav"ail</hw> <pr>(?; 48)</pr>, <pos>n.</pos> <ety>[F. <ets>travail</ets>; cf. Pr. <ets>trabalh</ets>, <ets>trebalh</ets>, toil, torment, torture; probably from LL. <ets>trepalium</ets> a place where criminals are tortured, instrument of torture. But the French word may be akin to L. <ets>trabs</ets> a beam, or have been influenced by a derivative from <ets>trabs</ets> (cf. <er>Trave</er>).  Cf. <er>Travel</er>.]</ety> <sn>1.</sn> <def>Labor with pain; severe toil or exertion.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>As everything of price, so this doth require <qex>travail</qex>.</q> <rj><qau>Hooker.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Parturition; labor; <as>as, an easy <ex>travail</ex></as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Travail</ent><br/
||<hw>Tra`vail"</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Cf. F. <ets>travail</ets>, a frame for confining a horse, or OF. <ets>travail</ets> beam, and E. <ets>trave</ets>, n. Cf. <er>Travail</er>, <pos>v. i.</pos>]</ety> <def>Same as <er>Travois</er>.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Travail</ent><br/
<hw>Trav"ail</hw>, <pos>v. i.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Travailed</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Travailing</conjf>.]</vmorph> <ety>[F.  <ets>travailler</ets>, OF. <ets>traveillier</ets>, <ets>travaillier</ets>, to labor, toil, torment; cf. Pr. <ets>trebalhar</ets> to torment, agitate. See <er>Travail</er>, <pos>n.</pos>]</ety> <sn>1.</sn> <def>To labor with pain; to toil.</def> <mark>[Archaic]</mark> <ldquo/Slothful persons which will not <xex>travail</xex> for their livings.<rdquo/  <rj><au>Latimer.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To suffer the pangs of childbirth; to be in labor.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Travail</ent><br/
<hw>Trav"ail</hw>, <pos>v. t.</pos> <def>To harass; to tire.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>As if all these troubles had not been sufficient to <qex>travail</qex> the realm, a great division fell among the nobility.</q> <rj><qau>Hayward.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Travailous</ent><br/
<hw>Trav"ail*ous</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Causing travail; laborious.</def> <mark>[Obs.]</mark> <au>Wyclif.</au> -- <wordforms><wf>Trav"ail*ous*ly</wf>, <pos>adv.</pos> <mark>[Obs.]</mark></wordforms>  <rj><au>Wyclif.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Trave</ent><br/
<hw>Trave</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Through French, fr. L. <ets>trabs</ets>, <ets>trabis</ets>, a beam; cf. OF. <ets>tref</ets> a beam, also F. <ets>travail</ets> a frame to confine a horse, OE. <ets>trave</ets>, <ets>trevys</ets>, <ets>travise</ets>, It. <ets>travaglio</ets>, F. <ets>trav<eacute/e</ets> the space between two beams.]</ety> <sn>1.</sn> <fld>(Arch.)</fld> <def>A crossbeam; a lay of joists.</def>  <rj><au>Maundrell.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A wooden frame to confine an unruly horse or ox while shoeing.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>She sprung as a colt doth in the <qex>trave</qex>.</q> <rj><qau>Chaucer.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Travel</ent><br/
<hw>Trav"el</hw> <pr>(?)</pr>, <pos>v. i.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Traveled</conjf> <pr>(?)</pr> or <conjf>Travelled</conjf>; <pos>p. pr. & vb. n.</pos> <conjf>Traveling</conjf> or <conjf>Travelling</conjf>.]</vmorph> <ety>[Properly, to labor, and the same word as <ets>travail</ets>.]</ety> <sn>1.</sn> <def>To labor; to travail.</def> <mark>[Obsoles.]</mark>  <rj><au>Hooker.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To go or march on foot; to walk; <as>as, to <ex>travel</ex> over the city, or through the streets</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>To pass by riding, or in any manner, to a distant place, or to many places; to journey; <as>as, a man <ex>travels</ex> for his health; he is <ex>traveling</ex> in California.</as></def><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>To pass; to go; to move.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Time <qex>travels</qex> in divers paces with divers persons.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Travel</ent><br/
<hw>Trav"el</hw> <pr>(?)</pr>, <pos>v. t.</pos> <sn>1.</sn> <def>To journey over; to traverse; <as>as, to <ex>travel</ex> the continent</as>.</def> <ldquo/I <xex>travel</xex> this profound.<rdquo/  <rj><au>Milton.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To force to journey.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>They shall not be <qex>traveled</qex> forth of their own franchises.</q> <rj><qau>Spenser.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Travel</ent><br/
<hw>Trav"el</hw>, <pos>n.</pos> <sn>1.</sn> <def>The act of traveling, or journeying from place to place; a journey.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>With long <qex>travel</qex> I am stiff and weary.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>His <qex>travels</qex> ended at his country seat.</q> <rj><qau>Dryden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <pluf>pl.</pluf> <def>An account, by a traveler, of occurrences and observations during a journey; <as>as, a book of <ex>travels</ex></as>; -- often used as the title of a book; <as>as, <ex>Travels</ex> in Italy</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <fld>(Mach.)</fld> <def>The length of stroke of a reciprocating piece; <as>as, the <ex>travel</ex> of a slide valve</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>Labor; parturition; travail.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Traveled</ent><br/
<hw>Trav"eled</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Having made journeys; having gained knowledge or experience by traveling; hence, knowing; experienced.</def> <altsp>[Written also <asp>travelled</asp>.]</altsp><br/
[<source>1913 Webster</source>]</p>

<p><q>The <qex>traveled</qex> thane, Athenian Aberdeen.</q> <rj><qau>Byron.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Traveler</ent><br/
<hw>Trav"el*er</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Written also <ets>traveler</ets>.]</ety> <sn>1.</sn> <def>One who travels; one who has traveled much.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A commercial agent who travels for the purpose of receiving orders for merchants, making collections, etc.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <fld>(Mach.)</fld> <def>A traveling crane. See under <er>Crane</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <fld>(Spinning)</fld> <def>The metal loop which travels around the ring surrounding the bobbin, in a ring spinner.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <fld>(Naut.)</fld> <def>An iron encircling a rope, bar, spar, or the like, and sliding thereon.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Traveler's joy</b></col> <fld>(Bot.)</fld>, <cd>the <spn>Clematis vitalba</spn>, a climbing plant with white flowers.</cd> -- <col><b>Traveler's tree</b></col>. <fld>(Bot.)</fld> <cd>See <er>Ravenala</er>.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Travel-tainted</ent><br/
<hw>Trav"el-taint`ed</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Harassed; fatigued with travel.</def> <mark>[Obs.]</mark>  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Travers</ent><br/
<hw>Trav"ers</hw> <pr>(?)</pr>, <pos>adv.</pos> <ety>[F. <ets>travers</ets>, breadth, extent from side, <ets><agrave/ travers</ets>, <ets>en travers</ets>, <ets>de travers</ets>, across, athwart. See <er>Traverse</er>, <pos>a.</pos>]</ety> <def>Across; athwart.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>The earl . . . caused . . . high trees to be hewn down, and laid <qex>travers</qex> one over another.</q> <rj><qau>Ld. Berners.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Traversable</ent><br/
<hw>Trav"ers*a*ble</hw> <pr>(?)</pr>, <pos>a.</pos> <sn>1.</sn> <def>Capable of being traversed, or passed over; <as>as, a <ex>traversable</ex> region</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Deniable; specifically <fld>(Law)</fld>, liable to legal objection; <as>as, a <ex>traversable</ex> presentment</as>.</def>  <rj><au>Sir M. Hale.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Traverse</ent><br/
<hw>Trav"erse</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[OF. <ets>travers</ets>, L. <ets>transversus</ets>, p. p. of <ets>transvertere</ets> to turn or direct across. See <er>Transverse</er>, and cf. <er>Travers</er>.]</ety> <def>Lying across; being in a direction across something else; <as>as, paths cut with <ex>traverse</ex> trenches</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Oak . . . being strong in all positions, may be better trusted in cross and <qex>traverse</qex> work.</q> <rj><qau>Sir H. Wotton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>The ridges of the fallow field <qex>traverse</qex>.</q> <rj><qau>Hayward.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Traverse drill</b></col> <fld>(Mach.)</fld>, <cd>a machine tool for drilling slots, in which the work or tool has a lateral motion back and forth; also, a drilling machine in which the spindle holder can be adjusted laterally.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Traverse</ent><br/
<hw>Trav"erse</hw> <pr>(?)</pr>, <pos>adv.</pos> <def>Athwart; across; crosswise.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Traverse</ent><br/
<hw>Trav"erse</hw>, <pos>n.</pos> <ety>[F. <ets>traverse</ets>. See <er>Traverse</er>, <pos>a.</pos>]</ety> <sn>1.</sn> <def>Anything that traverses, or crosses.</def> Specifically: --<br/
[<source>1913 Webster</source>]</p>

<p><sd>(a)</sd> <def>Something that thwarts, crosses, or obstructs; a cross accident; as, he would have succeeded, had it not been for unlucky <xex>traverses</xex> not under his control.</def><br/
[<source>1913 Webster</source>]</p>

<p><sd>(b)</sd> <def>A barrier, sliding door, movable screen, curtain, or the like.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Men drinken and the <qex>travers</qex> draw anon.</q> <rj><qau>Chaucer.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>And the entrance of the king,<br/
The first <qex>traverse</qex> was drawn.</q> <rj><qau>F. Beaumont.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sd>(c)</sd> <fld>(Arch.)</fld> <def>A gallery or loft of communication from side to side of a church or other large building.</def>  <rj><au>Gwilt.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sd>(d)</sd> <fld>(Fort.)</fld> <def>A work thrown up to intercept an enfilade, or reverse fire, along exposed passage, or line of work.</def><br/
[<source>1913 Webster</source>]</p>

<p><sd>(e)</sd> <fld>(Law)</fld> <def>A formal denial of some matter of fact alleged by the opposite party in any stage of the pleadings. The technical words introducing a <xex>traverse</xex> are <xex>absque hoc</xex>, without this; that is, without this which follows.</def><br/
[<source>1913 Webster</source>]</p>

<p><sd>(f)</sd> <fld>(Naut.)</fld> <def>The zigzag course or courses made by a ship in passing from one place to another; a compound course.</def><br/
[<source>1913 Webster</source>]</p>

<p><sd>(g)</sd> <fld>(Geom.)</fld> <def>A line lying across a figure or other lines; a transversal.</def><br/
[<source>1913 Webster</source>]</p>

<p><sd>(h)</sd> <fld>(Surv.)</fld> <def>A line surveyed across a plot of ground.</def><br/
[<source>1913 Webster</source>]</p>

<p><sd>(i)</sd> <fld>(Gun.)</fld> <def>The turning of a gun so as to make it point in any desired direction.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A turning; a trick; a subterfuge.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><cs><mcol><col><b>To work a traverse</b></col> <it>or</it>  <col><b>To solve a traverse</b></col></mcol> <fld>(Naut.)</fld>, <cd>to reduce a series of courses or distances to an equivalent single one; to calculate the resultant of a traverse.</cd> -- <col><b>Traverse board</b></col> <fld>(Naut.)</fld>, <cd>a small board hung in the steerage, having the points of the compass marked on it, and for each point as many holes as there are half hours in a watch. It is used for recording the courses made by the ship in each half hour, by putting a peg in the corresponding hole.</cd> -- <col><b>Traverse jury</b></col> <fld>(Law)</fld>, <cd>a jury that tries cases; a petit jury.</cd> -- <col><b>Traverse sailing</b></col> <fld>(Naut.)</fld>, <cd>a sailing by compound courses; the method or process of finding the resulting course and distance from a series of different shorter courses and distances actually passed over by a ship.</cd> -- <col><b>Traverse table</b></col>. <sd>(a)</sd> <fld>(Naut. & Surv.)</fld> <cd>A table by means of which the difference of latitude and departure corresponding to any given course and distance may be found by inspection. It contains the lengths of the two sides of a right-angled triangle, usually for every quarter of a degree of angle, and for lengths of the hypothenuse, from 1 to 100.</cd> <sd>(b)</sd> <fld>(Railroad)</fld> <cd>A platform with one or more tracks, and arranged to move laterally on wheels, for shifting cars, etc., from one line of track to another.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Traverse</ent><br/
<hw>Trav"erse</hw>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Traversed</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Traversing</conjf>.]</vmorph> <ety>[Cf. F. <ets>traverser</ets>. See <er>Traverse</er>, <pos>a.</pos>]</ety> <sn>1.</sn> <def>To lay in a cross direction; to cross.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The parts should be often <qex>traversed</qex>, or crossed, by the flowing of the folds.</q> <rj><qau>Dryden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To cross by way of opposition; to thwart with obstacles; to obstruct; to bring to naught.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>I can not but . . . admit the force of this reasoning, which I yet hope to <qex>traverse</qex>.</q> <rj><qau>Sir W. Scott.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>To wander over; to cross in traveling; <as>as, to <ex>traverse</ex> the habitable globe</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>What seas you <qex>traversed</qex>, and what fields you fought.</q> <rj><qau>Pope.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>To pass over and view; to survey carefully.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>My purpose is to <qex>traverse</qex> the nature, principles, and properties of this detestable vice -- ingratitude.</q> <rj><qau>South.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <fld>(Gun.)</fld> <def>To turn to the one side or the other, in order to point in any direction; <as>as, to <ex>traverse</ex> a cannon</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>6.</sn> <fld>(Carp.)</fld> <def>To plane in a direction across the grain of the wood; <as>as, to <ex>traverse</ex> a board</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>7.</sn> <fld>(Law)</fld> <def>To deny formally, as what the opposite party has alleged. When the plaintiff or defendant advances new matter, he avers it to be true, and <xex>traverses</xex> what the other party has affirmed. To <xex>traverse</xex> an indictment or an office is to deny it.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>And save the expense of long litigious laws,<br/
Where suits are <qex>traversed</qex>, and so little won<br/
That he who conquers is but last undone.</q> <rj><qau>Dryden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>To traverse a yard</b></col> <fld>(Naut.)</fld>, <cd>to brace it fore and aft.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Traverse</ent><br/
<hw>Trav"erse</hw> <pr>(?)</pr>, <pos>v. i.</pos> <sn>1.</sn> <def>To use the posture or motions of opposition or counteraction, as in fencing.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>To see thee fight, to see thee foin, to see thee <qex>traverse</qex>.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To turn, as on a pivot; to move round; to swivel; <as>as, the needle of a compass <ex>traverses</ex>; if it does not <ex>traverse</ex> well, it is an unsafe guide.</as></def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>To tread or move crosswise, as a horse that throws his croup to one side and his head to the other.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Traverse drill</ent><br/
<hw>Trav"erse drill</hw>. <fld>(Mach.)</fld> <def>A machine tool for drilling slots, in which the work or tool has a lateral motion back and forth; also, a drilling machine in which the spindle holder can be adjusted laterally.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Traverser</ent><br/
<hw>Trav"ers*er</hw> <pr>(?)</pr>, <pos>n.</pos> <sn>1.</sn> <def>One who, or that which, traverses, or moves, as an index on a scale, and the like.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Law)</fld> <def>One who traverses, or denies.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <fld>(Railroad)</fld> <def>A traverse table. See under <er>Traverse</er>, <pos>n.</pos></def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Traversing</ent><br/
<hw>Trav"ers*ing</hw>, <pos>a.</pos> <def>Adjustable laterally; having a lateral motion, or a swinging motion; adapted for giving lateral motion.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Traversing plate</b></col> <fld>(Mil.)</fld>, <cd>one of two thick iron plates at the hinder part of a gun carriage, where the handspike is applied in traversing the piece.</cd> <au>Wilhelm.</au> -- <col><b>Traversing platform</b></col> <fld>(Mil.)</fld>, <cd>a platform for traversing guns.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Travertine</ent><br/
<hw>Trav"er*tine</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F. <ets>travertin</ets>, It. <ets>travertino</ets>, <ets>tiburtino</ets>, L. lapis <ets>Tiburtinus</ets>, fr. <ets>Tibur</ets> an ancient town of Latium, now Tivoli.]</ety> <fld>(Min.)</fld> <def>A white concretionary form of calcium carbonate, usually hard and semicrystalline. It is deposited from the water of springs or streams holding lime in solution. Extensive deposits exist at Tivoli, near Rome.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Travesty</ent><br/
<hw>Trav"es*ty</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[F. <ets>travesti</ets>, p. p. of <ets>travestir</ets> to disguise, to travesty, It. <ets>travestire</ets>, fr. L. <ets>trans</ets> across, over + <ets>vestire</ets> to dress, clothe. See <er>Vest</er>.]</ety> <def>Disguised by dress so as to be ridiculous; travestied; -- applied to a book or shorter composition.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Travesty</ent><br/
<hw>Trav"es*ty</hw>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Travesties</plw> <pr>(<?/)</pr>.</plu> <def>A burlesque translation or imitation of a work.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The second edition is not a recast, but absolutely a <qex>travesty</qex> of the first.</q> <rj><qau>De Quincey.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Travesty</ent><br/
<hw>Trav"es*ty</hw>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Travestied</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Travesting</conjf>.]</vmorph> <def>To translate, imitate, or represent, so as to render ridiculous or ludicrous.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>I see poor Lucan <qex>travestied</qex>, not appareled in his Roman toga, but under the cruel shears of an English tailor.</q> <rj><qau>Bentley.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Travois</ent><br/
||<hw>Tra`vois"</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Cf. <er>Travail</er>.]</ety> <sn>1.</sn> <def>A primitive vehicle, common among the North American Indians, usually two trailing poles serving as shafts and bearing a platform or net for a load.</def></p>

<p><q>On the plains they will have horses dragging <qex>travoises</qex>; dogs with <qex>travoises</qex>, women and children loaded with impediments.</q>  <rj><qau>Julian Ralph.</qau></rj><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><sn>2.</sn>  <def>A logging sled.</def> <mark>[Northern U. S. & Canada]</mark><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Trawl</ent><br/
<hw>Trawl</hw> <pr>(?)</pr>, <pos>v. i.</pos> <ety>[OF. <ets>trauler</ets>, <ets>troller</ets>, F. <ets>tr<ocir/ter</ets>, to drag about, to stroll about; probably of Teutonic origin.  Cf. <er>Troll</er>, <pos>v. t.</pos>]</ety> <def>To take fish, or other marine animals, with a trawl.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Trawl</ent><br/
<hw>Trawl</hw>, <pos>n.</pos> <sn>1.</sn> <def>A fishing line, often extending a mile or more, having many short lines bearing hooks attached to it. It is used for catching cod, halibut, etc.; a boulter.</def> <mark>[U. S. & Canada]</mark><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A large bag net attached to a beam with iron frames at its ends, and dragged at the bottom of the sea, -- used in fishing, and in gathering forms of marine life from the sea bottom.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Trawlboat</ent><br/
<hw>Trawl"boat`</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A boat used in fishing with trawls or trawlnets.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Trawler</ent><br/
<hw>Trawl"er</hw> <pr>(?)</pr>, <pos>n.</pos> <sn>1.</sn> <def>One who, or that which, trawls.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A fishing vessel which trails a net behind it.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Trawlerman</ent><br/
<hw>Trawl"er*man</hw> <pr>(?)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Trawlermen</plw> <pr>(<?/)</pr>.</plu> <def>A fisherman who used unlawful arts and engines to catch fish.</def> <mark>[Obs.]</mark>  <rj><au>Cowell.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Trawlnet</ent><br/
<hw>Trawl"net`</hw> <pr>(?)</pr>, <pos>n.</pos> <def>Same as <er>Trawl</er>, <pos>n.</pos>, 2.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Trawlwarp</ent><br/
<hw>Trawl"warp`</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A rope passing through a block, used in managing or dragging a trawlnet.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tray</ent><br/
<hw>Tray</hw> <pr>(?)</pr>, <pos>v. t.</pos> <ety>[OF. <ets>tra<ium/r</ets>, F. <ets>trahir</ets>, L. <ets>tradere</ets>. See <er>Traitor</er>.]</ety> <def>To betray; to deceive.</def> <mark>[Obs.]</mark>  <rj><au>Chaucer.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tray</ent><br/
<hw>Tray</hw>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Trays</plw> <pr>(#)</pr>.</plu> <ety>[OE. <ets>treye</ets>, AS. <ets>treg</ets>.  Cf. <er>Trough</er>.]</ety> <sn>1.</sn> <def>A small trough or wooden vessel, sometimes scooped out of a block of wood, for various domestic uses, as in making bread, chopping meat, etc.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A flat, broad vessel on which dishes, glasses, etc., are carried; a waiter; a salver.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>A shallow box, generally without a top, often used within a chest, trunk, box, etc., as a removable receptacle for small or light articles.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Trayful</ent><br/
<hw>Tray"ful</hw> <pr>(?)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Trayfuls</plw> <pr>(<?/)</pr>.</plu> <def>As much as a tray will hold; enough to fill a tray.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Trays</ent><br/
<hw>Trays</hw> <pr>(?)</pr>, <pos>n. pl.</pos> <mark>[Obs.]</mark> <def>See <er>Trais</er>.</def>  <rj><au>Chaucer.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tray-trip</ent><br/
<hw>Tray"-trip`</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[From <er>Trey</er> a three.]</ety> <def>An old game played with dice.</def> <mark>[Obs.]</mark>  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Treacher</ent><br/
<hw>Treach"er</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[OE. <ets>trichour</ets>, <ets>trichur</ets>, OF. <ets>tricheor</ets> deceiver, traitor, F. <ets>tricheur</ets> a cheat at play, a trickster. See <er>Treachery</er>.]</ety> <def>A traitor; a cheat.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q><qex>Treacher</qex> and coward both.</q> <rj><qau>Beau. & Fl.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Treacherous</ent><br/
<hw>Treach"er*ous</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[See <er>Treacher</er>.]</ety> <def>Like a traitor; involving treachery; violating allegiance or faith pledged; traitorous to the state or sovereign; perfidious in private life; betraying a trust; faithless.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Loyal father of a <qex>treacherous</qex> son.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>The <qex>treacherous</qex> smile, a mask for secret hate.</q> <rj><qau>Cowper.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><syn><b>Syn.</b> -- Faithless; perfidious; traitorous; false; insidious; plotting.</syn><br/
[<source>1913 Webster</source>]</p>

<p>-- <wordforms><wf>Treach"er*ous*ly</wf>, <pos>adv.</pos> -- <wf>Treach"er*ous*ness</wf>, <pos>n.</pos></wordforms><br/
[<source>1913 Webster</source>]</p>

<p><ent>Treachery</ent><br/
<hw>Treach"er*y</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[OE. <ets>trecher<ium/e</ets>, <ets>trichere</ets>, OF. <ets>trecherie</ets>, <ets>tricherie</ets>, F. <ets>tricherie</ets> trickery, from <ets>tricher</ets> to cheat, to trick, OF. <ets>trichier</ets>, <ets>trechier</ets>; probably of Teutonic origin. See <er>Trickery</er>, <er>Trick</er>.]</ety> <def>Violation of allegiance or of faith and confidence; treasonable or perfidious conduct; perfidy; treason.</def><br/
[<source>1913 Webster</source>]</p>

<p><-- p. 1533 --><br/
[<source>1913 Webster</source>]</p>

<p><q>Be ware, ye lords, of their <qex>treachery</qex>.</q> <rj><qau>Chaucer.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>In the council chamber at Edinburgh, he had contracted a deep taint of <qex>treachery</qex> and corruption.</q> <rj><qau>Macaulay.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Treachour</ent><br/
<ent>Treachetour</ent><br/
<mhw>{ <hw>Treach"e*tour</hw> <pr>(?)</pr>, <hw>Treach"our</hw> <pr>(?)</pr>, }</mhw> <pos>n.</pos> <ety>[See <er>Treacher</er>.]</ety> <def>A traitor.</def> <mark>[Obs.]</mark> <ldquo/<xex>Treachour</xex> full of false despite.<rdquo/  <rj><au>Spenser.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Treacle</ent><br/
<hw>Trea"cle</hw> <pr>(tr<emac/"k'l)</pr>, <pos>n.</pos> <ety>[OE. <ets>triacle</ets> a sovereign remedy, theriac, OF. <ets>triacle</ets>, F. <ets>th<eacute/riaque</ets> (cf. Pr. <ets>triacla</ets>, <ets>tiriaca</ets>, Sp. & It. <ets>triaca</ets>, <ets>teriaca</ets>), L. <ets>theriaca</ets> an antidote against the bite of poisonous animals, Gr. <?/, fr. <?/ of wild or venomous beasts, fr. <grk>qhri`on</grk> a beast, a wild beast, dim. of <grk>qh`r</grk> a beast.  Cf. <er>Theriac</er>.]</ety> <sn>1.</sn> <fld>(Old Med.)</fld> <def>A remedy against poison. See <er>Theriac</er>, 1.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>We kill the viper, and make <qex>treacle</qex> of him.</q> <rj><qau>Jer. Taylor.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A sovereign remedy; a cure.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>Christ which is to every harm <qex>treacle</qex>.</q> <rj><qau>Chaucer.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>Molasses; sometimes, specifically, the molasses which drains from the sugar-refining molds, and which is also called <altname>sugarhouse molasses</altname>.</def><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ In the United States <xex>molasses</xex> is the common name; in England, <xex>treacle</xex>.</note><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>A saccharine fluid, consisting of the inspissated juices or decoctions of certain vegetables, as the sap of the birch, sycamore, and the like.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Treacle mustard</b></col> <fld>(Bot.)</fld>, <cd>a name given to several species of the cruciferous genus <gen>Erysimum</gen>, especially the <spn>Erysimum cheiranthoides</spn>, which was formerly used as an ingredient in Venice treacle, or theriac.</cd> -- <col><b>Treacle water</b></col>, <cd>a compound cordial prepared in different ways from a variety of ingredients, as hartshorn, roots of various plants, flowers, juices of plants, wines, etc., distilled or digested with Venice treacle. It was formerly regarded as a medicine of great virtue.</cd> <au>Nares.</au> -- <col><b>Venice treacle</b></col>. <fld>(Old Med.)</fld> <cd>Same as <er>Theriac</er>, 1.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Treacly</ent><br/
<hw>Trea"cly</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Like, or composed of, treacle.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tread</ent><br/
<hw>Tread</hw> <pr>(?)</pr>, <pos>v. i.</pos> <vmorph>[<pos>imp.</pos> <conjf>Trod</conjf> <pr>(?)</pr>; <pos>p. p.</pos> <conjf>Trodden</conjf> <pr>(?)</pr>, <conjf>Trod</conjf>; <pos>p. pr. & vb. n.</pos> <conjf>Treading</conjf>.]</vmorph> <ety>[OE. <ets>treden</ets>, AS. <ets>tredan</ets>; akin to OFries. <ets>treda</ets>, OS. <ets>tredan</ets>, D. & LG. <ets>treden</ets>, G. <ets>treten</ets>, OHG. <ets>tretan</ets>, Icel. <ets>tro<?/a</ets>, Sw. <ets>tr<aring/da</ets>, <ets>tr<aum/da</ets>, Dan. <ets>tr<ae/de</ets>, Goth. <ets>trudan</ets>, and perhaps ultimately to F. <ets>tramp</ets>; cf. Gr. <?/ a running, Skr. <ets>dram</ets> to run.  Cf. <er>Trade</er>, <er>Tramp</er>, <er>Trot</er>.]</ety> <sn>1.</sn> <def>To set the foot; to step.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Where'er you <qex>tread</qex>, the blushing flowers shall rise.</q> <rj><qau>Pope.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Fools rush in where angels fear to <qex>tread</qex>.</q> <rj><qau>Pope.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>The hard stone<br/
Under our feet, on which we <qex>tread</qex> and go.</q> <rj><qau>Chaucer.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To walk or go; especially, to walk with a stately or a cautious step.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Ye that . . . stately <qex>tread</qex>, or lowly creep.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>To copulate; said of birds, esp. the males.</def>  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><mcol><col><b>To tread on</b></col> <it>or</it> <col><b>To tread upon</b></col></mcol>. <sd>(a)</sd> <cd>To trample; to set the foot on in contempt.</cd> <ldquo/Thou shalt <xex>tread upon</xex> their high places.<rdquo/ <au>Deut. xxxiii. 29.</au> <sd>(b)</sd> <cd>to follow closely.</cd> <ldquo/Year <xex>treads on</xex> year.<rdquo/ <au>Wordsworth.</au> -- <col><b>To tread upon the heels of</b></col>, <cd>to follow close upon.</cd> <ldquo/Dreadful consequences that <xex>tread upon the heels of</xex> those allowances to sin.<rdquo/ <au>Milton.</au></cs><br/
[<source>1913 Webster</source>]</p>

<p><q>One woe doth <qex>tread upon</qex> another's <qex>heel</qex>.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tread</ent><br/
<hw>Tread</hw>, <pos>v. t.</pos> <sn>1.</sn> <def>To step or walk on.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Forbid to <qex>tread</qex> the promised land he saw.</q> <rj><qau>Prior.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Methought she <qex>trod</qex> the ground with greater grace.</q> <rj><qau>Dryden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To beat or press with the feet; <as>as, to <ex>tread</ex> a path; to <ex>tread</ex> land when too light; a well-<ex>trodden</ex> path</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>To go through or accomplish by walking, dancing, or the like.</def> <ldquo/ I am resolved to forsake Malta, <xex>tread</xex> a pilgrimage to fair Jerusalem.<rdquo/  <rj><au>Beau. & Fl.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>They have measured many a mile,<br/
To <qex>tread</qex> a measure with you on this grass.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>To crush under the foot; to trample in contempt or hatred; to subdue.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Through thy name will we <qex>tread</qex> them under that rise up against us.</q> <rj><qau>Ps. xliv. 5.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <def>To copulate with; to feather; to cover; -- said of the male bird.</def>  <rj><au>Chaucer.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>To tread out</b></col>, <cd>to press out with the feet; to press out, as wine or wheat; as, <xex>to tread out</xex> grain with cattle or horses.</cd> -- <col><b>To tread the stage</b></col>, <cd>to act as a stageplayer; to perform a part in a drama.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tread</ent><br/
<hw>Tread</hw>, <pos>n.</pos> <sn>1.</sn> <def>A step or stepping; pressure with the foot; a footstep; <as>as, a nimble <ex>tread</ex></as>; a cautious <xex>tread</xex>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>She is coming, my own, my sweet;<br/
Were it ever so airy a <qex>tread</qex>,<br/
My heart would hear her and beat.</q> <rj><qau>Tennyson.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Manner or style of stepping; action; gait; <as>as, the horse has a good <ex>tread</ex></as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>Way; track; path.</def> <mark>[R.]</mark>  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>The act of copulation in birds.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <fld>(Arch.)</fld> <def>The upper horizontal part of a step, on which the foot is placed.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>6.</sn> <fld>(Fort.)</fld> <def>The top of the banquette, on which soldiers stand to fire over the parapet.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>7.</sn> <fld>(Mach.)</fld> <sd>(a)</sd> <def>The part of a wheel that bears upon the road or rail.</def> <sd>(b)</sd> <def>The part of a rail upon which car wheels bear.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>8.</sn> <fld>(Biol.)</fld> <def>The chalaza of a bird's egg; the treadle.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>9.</sn> <fld>(Far.)</fld> <def>A bruise or abrasion produced on the foot or ankle of a horse that interferes. See <er>Interfere</er>, 3.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Treadboard</ent><br/
<hw>Tread"board`</hw> <pr>(?)</pr>, <pos>n.</pos> <mark>[Arch.]</mark> <def>See <er>Tread</er>, <pos>n.</pos>, 5.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Treader</ent><br/
<hw>Tread"er</hw> <pr>(?)</pr>, <pos>n.</pos> <def>One who treads.</def>  <rj><au>Isa. xvi. 10.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Treadfowl</ent><br/
<hw>Tread"fowl`</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A cock.</def> <mark>[Obs.]</mark>  <rj><au>Chaucer.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Treadle</ent><br/
<hw>Trea"dle</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[OE. <ets>tredyl</ets> a step, AS. <ets>tredel</ets>. See <er>Tread</er>.]</ety> <sn>1.</sn> <def>The part of a foot lathe, or other machine, which is pressed or moved by the foot.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Biol.)</fld> <def>The chalaza of a bird's egg; the tread.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Treadmill</ent><br/
<hw>Tread"mill`</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A mill worked by persons treading upon steps on the periphery of a wide wheel having a horizontal axis. It is used principally as a means of prison discipline. Also, a mill worked by horses, dogs, etc., treading an endless belt.</def><--  or for physical exercise --><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tread-softly</ent><br/
<hw>Tread"-soft`ly</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Bot.)</fld> <def>Spurge nettle. See under <er>Nettle</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Treadwheel</ent><br/
<hw>Tread"wheel`</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A wheel turned by persons or animals, by treading, climbing, or pushing with the feet, upon its periphery or face. See <er>Treadmill</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Treague</ent><br/
<hw>Treague</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[It. <ets>tregua</ets>; of Teutonic origin, and akin to E. <ets>true</ets>.]</ety> <def>A truce.</def> <mark>[Obs.]</mark>  <rj><au>Spenser.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Treason</ent><br/
<hw>Trea"son</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[OE. <ets>tresun</ets>, <ets>treisun</ets>, <ets>traisoun</ets>, OF. <ets>tra<ium/son</ets>, F. <ets>trahison</ets>, L. <ets>traditio</ets> a giving up, a delivering up, fr. <ets>tradere</ets> to give up, betray. See <er>Traitor</er>, and cf. <er>Tradition</er>.]</ety> <sn>1.</sn> <def>The offense of attempting to overthrow the government of the state to which the offender owes allegiance, or of betraying the state into the hands of a foreign power; disloyalty; treachery.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The <qex>treason</qex> of the murthering in the bed.</q> <rj><qau>Chaucer.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ In monarchies, the killing of the sovereign, or an attempt to take his life, is treason. In England, to imagine or compass the death of the king, or of the queen consort, or of the heir apparent to the crown, is <xex>high treason</xex>, as are many other offenses created by statute. In the United States, treason is confined to the actual levying of war against the United States, or to an adhering to their enemies, giving them aid and comfort.</note><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Loosely, the betrayal of any trust or confidence; treachery; perfidy.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>If he be false, she shall his <qex>treason</qex> see.</q> <rj><qau>Chaucer.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Petit treason</b></col>. <cd>See under <er>Petit</er>.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Treasonable</ent><br/
<hw>Trea"son*a*ble</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Pertaining to treason; consisting of treason; involving the crime of treason, or partaking of its guilt.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Most men's heads had been intoxicated with imaginations of plots and <qex>treasonable</qex> practices.</q> <rj><qau>Clarendon.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><syn><b>Syn.</b> -- Treacherous; traitorous; perfidious; insidious.</syn><br/
[<source>1913 Webster</source>]</p>

<p>--<wordforms><wf>Trea"son*a*ble*ness</wf>, <pos>n.</pos> -- <wf>Trea"son*a*bly</wf>, <pos>adv.</pos></wordforms><br/
[<source>1913 Webster</source>]</p>

<p><ent>Treasonous</ent><br/
<hw>Trea"son*ous</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Treasonable.</def>  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>The <qex>treasonous</qex> book of the Court of King James.</q> <rj><qau>Pepys.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Treasure</ent><br/
<hw>Treas"ure</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[OE. <ets>tresor</ets>, <ets>tresour</ets>, F. <ets>tr<eacute/sor</ets>, L. <ets>thesaurus</ets>, Gr. <?/ a stone laid up, treasure, probably from the root of <?/ to put, place.  See <er>Thesis</er>, and cf. <er>Thesaurus</er>.]</ety> <sn>1.</sn> <def>Wealth accumulated; especially, a stock, or store of money in reserve.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>This <qex>treasure</qex> hath fortune unto us given.</q> <rj><qau>Chaucer.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A great quantity of anything collected for future use; abundance; plenty.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>We have <qex>treasures</qex> in the field, of wheat and of barley, and of oil and of honey.</q> <rj><qau>Jer. xli. 8.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>That which is very much valued.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Ye shall be peculiar <qex>treasure</qex> unto me.</q> <rj><qau>Ex. xix. 5.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>From thy wardrobe bring thy chiefest <qex>treasure</qex>.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Treasure city</b></col>, <cd>a city for stores and magazines.</cd>  <rj><au>Ex. i. 11.</au></rj></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Treasure</ent><br/
<hw>Treas"ure</hw>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Treasured</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Treasuring</conjf>.]</vmorph> <def>To collect and deposit, as money or other valuable things, for future use; to lay up; to hoard; usually with <xex>up</xex>; <as>as, to <ex>treasure</ex> up gold</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Treasure-house</ent><br/
<hw>Treas"ure-house`</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A house or building where treasures and stores are kept.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Treasurer</ent><br/
<hw>Treas"ur*er</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[OE. <ets>tresourer</ets>, F. <ets>tr<eacute/sorier</ets>.]</ety> <def>One who has the care of a treasure or treasure or treasury; an officer who receives the public money arising from taxes and duties, or other sources of revenue, takes charge of the same, and disburses it upon orders made by the proper authority; one who has charge of collected funds; <as>as, the <ex>treasurer</ex> of a society or corporation</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Lord high treasurer of England</b></col>, <cd>formerly, the third great officer of the crown. His office is now executed by five persons styled the <xex>lords commissioners of the treasury</xex>, or <xex>treasury lords</xex>.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Treasurership</ent><br/
<hw>Treas"ur*er*ship</hw>, <pos>n.</pos> <def>The office of treasurer.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Treasuress</ent><br/
<hw>Treas"ur*ess</hw>, <pos>n.</pos> <def>A woman who is a treasurer.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Treasure-trove</ent><br/
<hw>Treas"ure-trove`</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[<ets>Treasure</ets> + OF. <ets>trov<eacute/</ets>, F. <ets>trouv<eacute/</ets>, p. p. of OF. <ets>trover</ets> to find, F. <ets>trouver.</ets>  See <er>Trover</er>.]</ety> <fld>(Common Law)</fld> <def>Any money, bullion, or the like, found in the earth, or otherwise hidden, the owner of which is not known. In England such treasure belongs to the crown; whereas similar treasure found in the sea, or upon the surface of the land, belongs to the finder if no owner appears.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Treasury</ent><br/
<hw>Treas"ur*y</hw> <pr>(?)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Treasuries</plw> <pr>(#)</pr>.</plu> <ety>[OE.  <ets>tresorie</ets>, F. <ets>tr<eacute/sorerie</ets>.]</ety> <sn>1.</sn> <def>A place or building in which stores of wealth are deposited; especially, a place where public revenues are deposited and kept, and where money is disbursed to defray the expenses of government; hence, also, the place of deposit and disbursement of any collected funds.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>That department of a government which has charge of the finances.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>A repository of abundance; a storehouse.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>Hence, a book or work containing much valuable knowledge, wisdom, wit, or the like; a thesaurus; <as>as, <ldquo/ Maunder's <ex>Treasury</ex> of Botany</as>.<rdquo/</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <def>A treasure.</def> <mark>[Obs.]</mark>  <rj><au>Marston.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Board of treasury</b></col>, <cd>the board to which is intrusted the management of all matters relating to the sovereign's civil list or other revenues.</cd> <mark>[Eng.]</mark> <au>Brande & C.</au> -- <col><b>Treasury bench</b></col>, <cd>the first row of seats on the right hand of the Speaker in the House of Commons; -- so called because occupied by the first lord of the treasury and chief minister of the crown.</cd> <mark>[Eng.]</mark> -- <col><b>Treasury lord</b></col>. <cd>See <cref>Lord high treasurer of England</cref>, under <er>Treasurer</er>.</cd> <mark>[Eng.]</mark> -- <col><b>Treasury note</b></col> <fld>(U. S. Finance)</fld>, <cd>a circulating note or bill issued by government authority from the Treasury Department, and receivable in payment of dues to the government.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Treasury stock</ent><br/
<hw>Treas"ur*y stock</hw>. <fld>(Finance)</fld> <def>Issued stock of an incorporated company held by the company itself.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Treat</ent><br/
<hw>Treat</hw> <pr>(?)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Treated</conjf>; <pos>p. pr. & vb. n.</pos> <conjf>Treating</conjf>.]</vmorph> <ety>[<er>OE</er>. <ets>treten</ets>, OF. <ets>traitier</ets>, F. <ets>traiter</ets>, from L. <ets>tractare</ets> to draw violently, to handle, manage, treat, v. intens. from <ets>trahere</ets>, <ets>tractum</ets>, to draw. See <er>Trace</er>, <pos>v. t.</pos>, and cf. <er>Entreat</er>, <er>Retreat</er>, <er>Trait</er>.]</ety> <sn>1.</sn> <def>To handle; to manage; to use; to bear one's self toward; <as>as, to <ex>treat</ex> prisoners cruelly; to <ex>treat</ex> children kindly.</as></def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To discourse on; to handle in a particular manner, in writing or speaking; <as>as, to <ex>treat</ex> a subject diffusely</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>To entertain with food or drink, especially the latter, as a compliment, or as an expression of friendship or regard; <as>as, to <ex>treat</ex> the whole company</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>To negotiate; to settle; to make terms for.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>To <qex>treat</qex> the peace, a hundred senators<br/
Shall be commissioned.</q> <rj><qau>Dryden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <fld>(Med.)</fld> <def>To care for medicinally or surgically; to manage in the use of remedies or appliances; <as>as, to <ex>treat</ex> a disease, a wound, or a patient</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>6.</sn> <def>To subject to some action; to apply something to; <as>as, to <ex>treat</ex> a substance with sulphuric acid</as>.</def>  <rj><au>Ure.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>7.</sn> <def>To entreat; to beseech.</def> <mark>[Obs.]</mark>  <rj><au>Ld. Berners.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Treat</ent><br/
<hw>Treat</hw> <pr>(?)</pr>, <pos>v. i.</pos> <sn>1.</sn> <def>To discourse; to handle a subject in writing or speaking; to make discussion; -- usually with <xex>of</xex>; <as>as, Cicero <ex>treats</ex> of old age and of duties</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>And, shortly of this story for to <qex>treat</qex>.</q> <rj><qau>Chaucer.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Now of love they <qex>treat</qex>.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To negotiate; to come to terms of accommodation; -- often followed by <xex>with</xex>; <as>as, envoys were appointed to <ex>treat</ex> with France</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Inform us, will the emperor <qex>treat</qex>!</q> <rj><qau>Swift.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>To give a gratuitous entertainment, esp. of food or drink, as a compliment.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Treat</ent><br/
<hw>Treat</hw>, <pos>n.</pos> <sn>1.</sn> <def>A parley; a conference.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>Bid him battle without further <qex>treat</qex>.</q> <rj><qau>Spenser.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>An entertainment given as an expression of regard.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>That which affords entertainment; a gratification; a satisfaction; <as>as, the concert was a rich <ex>treat</ex></as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Treatable</ent><br/
<hw>Treat"a*ble</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[OE. <ets>tretable</ets>, F. <ets>traitable</ets>, L. <ets>tractabilis</ets>. See <er>Treat</er>, and cf. <er>Tractable</er>.]</ety> <def>Manageable; tractable; hence, moderate; not violent.</def> <mark>[Obs.]</mark> <ldquo/ A <xex>treatable</xex> disposition, a strong memory.<rdquo/  <rj><au>R. Parr.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>A kind of <qex>treatable</qex> dissolution.</q> <rj><qau>Hooker.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>The heats or the colds of seasons are less <qex>treatable</qex> than with us.</q> <rj><qau>Sir W. Temple.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Treatably</ent><br/
<hw>Treat"a*bly</hw>, <pos>adv.</pos> <def>In a treatable manner.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Treater</ent><br/
<hw>Treat"er</hw> <pr>(?)</pr>, <pos>n.</pos> <def>One who treats; one who handles, or discourses on, a subject; also, one who entertains.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Treatise</ent><br/
<hw>Trea"tise</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[OE. <ets>tretis</ets>, OF. <ets>treitis</ets>, <ets>traitis</ets>, well made. See <er>Treat</er>.]</ety> <sn>1.</sn> <def>A written composition on a particular subject, in which its principles are discussed or explained; a tract.</def>  <rj><au>Chaucer.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>He published a <qex>treatise</qex> in which he maintained that a marriage between a member of the Church of England and a dissenter was a nullity.</q> <rj><qau>Macaulay.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ A <xex>treatise</xex> implies more form and method than an <xex>essay</xex>, but may fall short of the fullness and completeness of a systematic exposition.</note><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Story; discourse.</def> <mark>[R.]</mark>  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Treatiser</ent><br/
<hw>Trea"tis*er</hw> <pr>(?)</pr>, <pos>n.</pos> <def>One who writes a treatise.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Treatment</ent><br/
<hw>Treat"ment</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Cf. F. <ets>traitement</ets>. See <er>Treat</er>.]</ety> <sn>1.</sn> <def>The act or manner of treating; management; manipulation; handling; usage; <as>as, unkind <ex>treatment</ex>; medical <ex>treatment</ex></as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Entertainment; treat.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>Accept such <qex>treatment</qex> as a swain affords.</q> <rj><qau>Pope.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Treature</ent><br/
<hw>Trea"ture</hw> <pr>(?)</pr>, <pos>n.</pos> <def>Treatment.</def> <mark>[Obs.]</mark>  <rj><au>Fabyan.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Treaty</ent><br/
<hw>Trea"ty</hw> <pr>(?)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Treaties</plw> <pr>(#)</pr>.</plu> <ety>[OE. <ets>tretee</ets>, F. <ets>trait<eacute/</ets>, LL. <ets>tractatus</ets>; cf. L. <ets>tractatus</ets> a handling, treatment, consultation, tractate. See <er>Treat</er>, and cf. <er>Tractate</er>.]</ety> <sn>1.</sn> <def>The act of treating for the adjustment of differences, as for forming an agreement; negotiation.</def> <ldquo/By sly and wise <xex>treaty</xex>.<rdquo/  <rj><au>Chaucer.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>He cast by <qex>treaty</qex> and by trains<br/
Her to persuade.</q> <rj><qau>Spenser.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>An agreement so made; specifically, an agreement, league, or contract between two or more nations or sovereigns, formally signed by commissioners properly authorized, and solemnly ratified by the several sovereigns, or the supreme power of each state; an agreement between two or more independent states; <as>as, a <ex>treaty</ex> of peace; a <ex>treaty</ex> of alliance.</as></def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>A proposal tending to an agreement.</def> <mark>[Obs.]</mark>  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>A treatise; a tract.</def> <mark>[Obs.]</mark>  <rj><au>Sir T. Browne.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Treble</ent><br/
<hw>Tre"ble</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[OE. <ets>treble</ets> threefold, OF. <ets>treble</ets>, <ets>treible</ets>, L. <ets>triplus</ets>. See <er>Triple</er>.]</ety> <sn>1.</sn> <def>Threefold; triple.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>A lofty tower, and strong on every side<br/
With <qex>treble</qex> walls.</q> <rj><qau>Dryden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Mus.)</fld> <sd>(a)</sd> <def>Acute; sharp; <as>as, a <ex>treble</ex> sound</as>.</def> <au>Bacon.</au> <sd>(b)</sd> <def>Playing or singing the highest part or most acute sounds; playing or singing the treble; <as>as, a <ex>treble</ex> violin or voice</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Treble</ent><br/
<hw>Tre"ble</hw>, <pos>adv.</pos> <def>Trebly; triply.</def> <mark>[Obs.]</mark>  <rj><au>J. Fletcher.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Treble</ent><br/
<hw>Tre"ble</hw>, <pos>n.</pos> <ety>[<ldquo/ It has been said to be a corruption of <ets>triplum</ets> [Lat.], a third part, superadded to the altus and bassus (high and low).<rdquo/ <au>Grove.</au>]</ety> <fld>(Mus.)</fld> <def>The highest of the four principal parts in music; the part usually sung by boys or women; soprano.</def><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ This is sometimes called the <xex>first treble</xex>, to distinguish it from the <xex>second treble</xex>, or <xex>alto</xex>, which is sung by lower female voices.</note><br/
[<source>1913 Webster</source>]</p>

<p><ent>Treble</ent><br/
<hw>Tre"ble</hw>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Trebled</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Trebling</conjf>.]</vmorph> <sn>1.</sn> <def>To make thrice as much; to make threefold.</def> <ldquo/Love <xex>trebled</xex> life.<rdquo/  <rj><au>Tennyson.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To utter in a treble key; to whine.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>He outrageously<br/
(When I accused him) <qex>trebled</qex> his reply.</q> <rj><qau>Chapman.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Treble</ent><br/
<hw>Tre"ble</hw>, <pos>v. i.</pos> <def>To become threefold.</def>  <rj><au>Swift.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Trebleness</ent><br/
<hw>Tre"ble*ness</hw>, <pos>n.</pos> <def>The quality or state of being treble; <as>as, the <ex>trebleness</ex> of tones</as>.</def>  <rj><au>Bacon.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Treblet</ent><br/
<hw>Treb"let</hw> <pr>(?)</pr>, <pos>n.</pos> <def>Same as <er>Triblet</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Trebly</ent><br/
<hw>Tre"bly</hw> <pr>(?)</pr>, <pos>adv.</pos> <def>In a treble manner; with a threefold number or quantity; triply.</def>  <rj><au>Swift.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Trebucket</ent><br/
<ent>Trebuchet</ent><br/
<mhw>{ <hw>Treb"u*chet</hw> <pr>(?)</pr>, <hw>Tre"buck*et</hw> <pr>(?)</pr>, }</mhw> <pos>n.</pos> <ety>[OF. <ets>trebuchet</ets>, <ets>trebukiet</ets>, an engine of war for hurling stones, F. <ets>tr<eacute/buchet</ets> a gin, trap, a kind of balance, fr. OF. <ets>trebuchier</ets>, <ets>trebuquier</ets>, to stumble, trip, F. <ets>tr<eacute/bucher</ets>.]</ety> <sn>1.</sn> <def>A cucking stool; a tumbrel.</def>  <rj><au>Cowell.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A military engine used in the Middle Ages for throwing stones, etc.  It acted by means of a great weight fastened to the short arm of a lever, which, being let fall, raised the end of the long arm with great velocity, hurling stones with much force.
</def><br/
[<source>1913 Webster</source>]</p>

<p><-- p. 1534 --><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>A kind of balance for weighing.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Trecentist</ent><br/
<hw>Tre*cen"tist</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A member of the trecento, or an imitator of its characteristics.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Trecento</ent><br/
||<hw>Tre*cen"to</hw> <pr>(?)</pr>, <pos>n. & a.</pos> <ety>[It., three hundred, abbr. for thirteen hundred.]</ety> <def>The fourteenth century, when applied to Italian art, literature, etc. It marks the period of Dante, Petrarch, and boccaccio in literature, and of Giotto in painting.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Trechometer</ent><br/
<hw>Tre*chom"e*ter</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Gr. <?/ to run + <ets>-meter</ets>.]</ety> <def>An odometer for vehicles.</def>  <rj><au>Knight.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Treckschuyt</ent><br/
||<hw>Treck"schuyt`</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[D. <ets>trekschuit</ets>; <ets>trekken</ets> to draw + <ets>schuit</ets> a boat.  Cf. <er>Trackscout</er>.]</ety> <def>A covered boat for goods and passengers, used on the Dutch and Flemish canals.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Treddle</ent><br/
<hw>Tred"dle</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[See <er>Treadle</er>.]</ety> <sn>1.</sn> <def>See <er>Treadle</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A prostitute; a strumpet.</def> <mark>[Obs.]</mark>  <rj><au>Ford.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <pluf>pl.</pluf> <def>The dung of sheep or hares.</def>  <rj><au>Holland.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tredille</ent><br/
<hw>Tre*dille"</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A game at cards for three.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tree</ent><br/
<hw>Tree</hw> <pr>(tr<emac/)</pr>, <pos>n.</pos> <ety>[OE. <ets>tree</ets>, <ets>tre</ets>, <ets>treo</ets>, AS. <ets>tre<oacute/</ets>, <ets>tre<oacute/w</ets>, tree, wood; akin to OFries. <ets>tr<emac/</ets>, OS. <ets>treo</ets>, <ets>trio</ets>, Icel. <ets>tr<emac/</ets>, Dan. <ets>trae</ets>, Sw. <ets>tr<aum/</ets>, <ets>tr<aum/d</ets>, Goth. <ets>triu</ets>, Russ. <ets>drevo</ets>, W. <ets>derw</ets> an oak, Ir. <ets>darag</ets>, <ets>darog</ets>, Gr. <grk>dry^s</grk> a tree, oak, <grk>do`ry</grk> a beam, spear shaft, spear, Skr. <ets>dru</ets> tree, wood, <ets>d<amac/ru</ets> wood. <root/63, 241.  Cf. <er>Dryad</er>, <er>Germander</er>, <er>Tar</er>, <pos>n.</pos>, <er>Trough</er>.]</ety><br/
[<source>1913 Webster</source>]</p>

<p><sn>1.</sn> <fld>(Bot.)</fld> <def>Any perennial woody plant of considerable size (usually over twenty feet high) and growing with a single trunk.</def><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ The kind of tree referred to, in any particular case, is often indicated by a modifying word; as forest <ex>tree</ex>, fruit <ex>tree</ex>, palm <ex>tree</ex>, apple <ex>tree</ex>, pear <ex>tree</ex>, etc.</note><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Something constructed in the form of, or considered as resembling, a tree, consisting of a stem, or stock, and branches; <as>as, a genealogical <ex>tree</ex></as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>A piece of timber, or something commonly made of timber; -- used in composition, as in axle<ex>tree</ex>, boot<ex>tree</ex>, chess<ex>tree</ex>, cross<ex>tree</ex>, whiffle<ex>tree</ex>, and the like.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>A cross or gallows; as Tyburn <ex>tree</ex>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>[Jesus] whom they slew and hanged on a <qex>tree</qex>.</q> <rj><qau>Acts x. 39.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <def>Wood; timber.</def> <mark>[Obs.]</mark>  <rj><au>Chaucer.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>In a great house ben not only vessels of gold and of silver but also of <qex>tree</qex> and of earth.</q> <rj><qau>Wyclif (2 Tim. ii. 20).</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>6.</sn> <fld>(Chem.)</fld> <def>A mass of crystals, aggregated in arborescent forms, obtained by precipitation of a metal from solution. See <cref>Lead tree</cref>, under <er>Lead</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Tree bear</b></col> <fld>(Zool.)</fld>, <cd>the raccoon.</cd> <mark>[Local, U. S.]</mark> -- <col><b>Tree beetle</b></col> <fld>(Zool.)</fld> <cd>any one of numerous species of beetles which feed on the leaves of trees and shrubs, as the May beetles, the rose beetle, the rose chafer, and the goldsmith beetle.</cd> -- <col><b>Tree bug</b></col> <fld>(Zool.)</fld>, <cd>any one of numerous species of hemipterous insects which live upon, and suck the sap of, trees and shrubs. They belong to <gen>Arma</gen>, <gen>Pentatoma</gen>, <gen>Rhaphigaster</gen>, and allied genera.</cd> -- <col><b>Tree cat</b></col> <fld>(Zool.)</fld>, <cd>the common paradoxure (<spn>Paradoxurus musang</spn>).</cd> -- <col><b>Tree clover</b></col> <fld>(Bot.)</fld>, <cd>a tall kind of melilot (<spn>Melilotus alba</spn>). See <er>Melilot</er>.</cd> -- <col><b>Tree crab</b></col> <fld>(Zool.)</fld>, <cd>the purse crab. See under <er>Purse</er>.</cd> -- <col><b>Tree creeper</b></col> <fld>(Zool.)</fld>, <cd>any one of numerous species of arboreal creepers belonging to <gen>Certhia</gen>, <gen>Climacteris</gen>, and allied genera. See <er>Creeper</er>, 3.</cd> -- <col><b>Tree cricket</b></col> <fld>(Zool.)</fld>, <cd>a nearly white arboreal American cricket (<spn>Ecanthus niv<oe/us</spn>) which is noted for its loud stridulation; -- called also <altname>white cricket</altname>.</cd> -- <col><b>Tree crow</b></col> <fld>(Zool.)</fld>, <cd>any one of several species of Old World crows belonging to <gen>Crypsirhina</gen> and allied genera, intermediate between the true crows and the jays. The tail is long, and the bill is curved and without a tooth.</cd> -- <col><b>Tree dove</b></col> <fld>(Zool.)</fld> <cd>any one of several species of East Indian and Asiatic doves belonging to <gen>Macropygia</gen> and allied genera. They have long and broad tails, are chiefly arboreal in their habits, and feed mainly on fruit.</cd> -- <col><b>Tree duck</b></col> <fld>(Zool.)</fld>, <cd>any one of several species of ducks belonging to <gen>Dendrocygna</gen> and allied genera. These ducks have a long and slender neck and a long hind toe. They are arboreal in their habits, and are found in the tropical parts of America, Africa, Asia, and Australia.</cd> -- <col><b>Tree fern</b></col> <fld>(Bot.)</fld>, <cd>an arborescent fern having a straight trunk, sometimes twenty or twenty-five feet high, or even higher, and bearing a cluster of fronds at the top. Most of the existing species are tropical.</cd> -- <col><b>Tree fish</b></col> <fld>(Zool.)</fld>, <cd>a California market fish (<spn>Sebastichthys serriceps</spn>).</cd> -- <col><b>Tree frog</b></col>. <fld>(Zool.)</fld> <sd>(a)</sd> <cd>Same as <er>Tree toad</er>. <sd>(b)</sd> Any one of numerous species of Old World frogs belonging to <gen>Chiromantis</gen>, <gen>Rhacophorus</gen>, and allied genera of the family <fam>Ranidae</fam>. Their toes are furnished with suckers for adhesion. The flying frog (see under <er>Flying</er>) is an example.</cd> -- <col><b>Tree goose</b></col> <fld>(Zool.)</fld>, <cd>the bernicle goose.</cd> -- <col><b>Tree hopper</b></col> <fld>(Zool.)</fld>, <cd>any one of numerous species of small leaping hemipterous insects which live chiefly on the branches and twigs of trees, and injure them by sucking the sap. Many of them are very odd in shape, the prothorax being often prolonged upward or forward in the form of a spine or crest.</cd> -- <col><b>Tree jobber</b></col> <fld>(Zool.)</fld>, <cd>a woodpecker.</cd> <mark>[Obs.]</mark> -- <col><b>Tree kangaroo</b></col>. <fld>(Zool.)</fld> <cd>See <er>Kangaroo</er>.</cd> -- <col><b>Tree lark</b></col> <fld>(Zool.)</fld>, <cd>the tree pipit.</cd> <mark>[Prov. Eng.]</mark> -- <col><b>Tree lizard</b></col> <fld>(Zool.)</fld>, <cd>any one of a group of Old World arboreal lizards (formerly grouped as the <ord>Dendrosauria</ord>) comprising the chameleons; also applied to various lizards belonging to the families <fam>Agamidae</fam> or <fam>Iguanidae</fam>, especially those of the genus <gen>Urosaurus</gen>, such as the <stype>lined tree lizard</stype> (<spn>Urosaurus ornatus</spn>) of the southwestern U.S.</cd> -- <col><b>Tree lobster</b></col>. <fld>(Zool.)</fld> <cd>Same as <cref>Tree crab</cref>, above.</cd> -- <col><b>Tree louse</b></col> <fld>(Zool.)</fld>, <cd>any aphid; a plant louse.</cd> -- <col><b>Tree moss</b></col>. <fld>(Bot.)</fld> <sd>(a)</sd> <cd>Any moss or lichen growing on trees.</cd> <sd>(b)</sd> <cd>Any species of moss in the form of a miniature tree.</cd> -- <col><b>Tree mouse</b></col> <fld>(Zool.)</fld>, <cd>any one of several species of African mice of the subfamily <fam>Dendromyinae</fam>. They have long claws and habitually live in trees.</cd> -- <col><b>Tree nymph</b></col>, <cd>a wood nymph. See <er>Dryad</er>.</cd> -- <col><b>Tree of a saddle</b></col>, <cd>a saddle frame.</cd> -- <col><b>Tree of heaven</b></col> <fld>(Bot.)</fld>, <cd>an ornamental tree (<spn>Ailantus glandulosus</spn>) having long, handsome pinnate leaves, and greenish flowers of a disagreeable odor.</cd> -- <col><b>Tree of life</b></col> <fld>(Bot.)</fld>, <cd>a tree of the genus Thuja; arbor vitae.</cd> -- <col><b>Tree onion</b></col> <fld>(Bot.)</fld>, <cd>a species of garlic (<spn>Allium proliferum</spn>) which produces bulbs in place of flowers, or among its flowers.</cd> -- <col><b>Tree oyster</b></col> <fld>(Zool.)</fld>, <cd>a small American oyster (<spn>Ostrea folium</spn>) which adheres to the roots of the mangrove tree; -- called also <altname>raccoon oyster</altname>.</cd> -- <col><b>Tree pie</b></col> <fld>(Zool.)</fld>, <cd>any species of Asiatic birds of the genus <gen>Dendrocitta</gen>. The tree pies are allied to the magpie.</cd> -- <col><b>Tree pigeon</b></col> <fld>(Zool.)</fld>, <cd>any one of numerous species of longwinged arboreal pigeons native of Asia, Africa, and Australia, and belonging to <gen>Megaloprepia</gen>, <gen>Carpophaga</gen>, and allied genera.</cd> -- <col><b>Tree pipit</b></col>. <fld>(Zool.)</fld> <cd>See under <er>Pipit</er>.</cd> -- <col><b>Tree porcupine</b></col> <fld>(Zool.)</fld>, <cd>any one of several species of Central and South American arboreal porcupines belonging to the genera <gen>Chaetomys</gen> and <gen>Sphingurus</gen>.  They have an elongated and somewhat prehensile tail, only four toes on the hind feet, and a body covered with short spines mixed with bristles.  One South American species (<spn>Sphingurus villosus</spn>) is called also <stype>couiy</stype>; another (<spn>Sphingurus prehensilis</spn>) is called also <stype>c<oe/ndou</stype>.</cd> -- <col><b>Tree rat</b></col> <fld>(Zool.)</fld>, <cd>any one of several species of large ratlike West Indian rodents belonging to the genera <gen>Capromys</gen> and <gen>Plagiodon</gen>. They are allied to the porcupines.</cd> -- <col><b>Tree serpent</b></col> <fld>(Zool.)</fld>, <cd>a tree snake.</cd> -- <col><b>Tree shrike</b></col> <fld>(Zool.)</fld>, <cd>a bush shrike.</cd> -- <col><b>Tree snake</b></col> <fld>(Zool.)</fld>, <cd>any one of numerous species of snakes of the genus <gen>Dendrophis</gen>. They live chiefly among the branches of trees, and are not venomous.</cd> -- <col><b>Tree sorrel</b></col> <fld>(Bot.)</fld>, <cd>a kind of sorrel (<spn>Rumex Lunaria</spn>) which attains the stature of a small tree, and bears greenish flowers.  It is found in the Canary Islands and Tenerife.</cd> -- <col><b>Tree sparrow</b></col> <fld>(Zool.)</fld> <cd>any one of several species of small arboreal sparrows, especially the American tree sparrow (<spn>Spizella monticola</spn>), and the common European species (<spn>Passer montanus</spn>).</cd> -- <col><b>Tree swallow</b></col> <fld>(Zool.)</fld>, <cd>any one of several species of swallows of the genus <gen>Hylochelidon</gen> which lay their eggs in holes in dead trees. They inhabit Australia and adjacent regions. Called also <altname>martin</altname> in Australia.</cd> -- <col><b>Tree swift</b></col> <fld>(Zool.)</fld>, <cd>any one of several species of swifts of the genus <gen>Dendrochelidon</gen> which inhabit the East Indies and Southern Asia.</cd> -- <col><b>Tree tiger</b></col> <fld>(Zool.)</fld>, <cd>a leopard.</cd> -- <col><b>Tree toad</b></col> <fld>(Zool.)</fld>, <cd>any one of numerous species of amphibians belonging to <gen>Hyla</gen> and allied genera of the family <fam>Hylidae</fam>. They are related to the common frogs and toads, but have the tips of the toes expanded into suckers by means of which they cling to the bark and leaves of trees.  Only one species (<spn>Hyla arborea</spn>) is found in Europe, but numerous species occur in America and Australia.  The common tree toad of the Northern United States (<spn>Hyla versicolor</spn>) is noted for the facility with which it changes its colors.  Called also <altname>tree frog</altname>. See also <cref>Piping frog</cref>, under <er>Piping</er>, and <cref>Cricket frog</cref>, under <er>Cricket</er>.</cd> -- <col><b>Tree warbler</b></col> <fld>(Zool.)</fld>, <cd>any one of several species of arboreal warblers belonging to <gen>Phylloscopus</gen> and allied genera.</cd> -- <col><b>Tree wool</b></col> <fld>(Bot.)</fld>, <cd>a fine fiber obtained from the leaves of pine trees.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tree</ent><br/
<hw>Tree</hw> <pr>(?)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Treed</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Treeing</conjf>.]</vmorph> <sn>1.</sn> <def>To drive to a tree; to cause to ascend a tree; <as>as, a dog <ex>trees</ex> a squirrel</as>.</def>  <rj><au>J. Burroughs.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To place upon a tree; to fit with a tree; to stretch upon a tree; <as>as, to <ex>tree</ex> a boot</as>. See <er>Tree</er>, <pos>n.</pos>, 3.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Treebeard</ent><br/
<hw>Tree"beard`</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Bot.)</fld> <def>A pendulous branching lichen (<spn>Usnea barbata</spn>); -- so called from its resemblance to hair.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tree burial</ent><br/
<hw>Tree burial</hw>. <def>Disposal of the dead by placing the corpse among the branches of a tree or in a hollow trunk, a practice among many primitive peoples.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Tree calf</ent><br/
<hw>Tree calf</hw>. <def>A bright brown polished calfskin binding of  books, stained with a conventional treelike design.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Treeful</ent><br/
<hw>Tree"ful</hw> <pr>(?)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Treefuls</plw> <pr>(<?/)</pr>.</plu> <def>The quantity or number which fills a tree.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Treeless</ent><br/
<hw>Tree"less</hw>, <pos>a.</pos> <def>Destitute of trees.</def>  <rj><au>C. Kingsley.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Treen</ent><br/
<hw>Treen</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[AS. <ets>tre<oacute/wen</ets>.]</ety> <sn>1.</sn> <def>Made of wood; wooden.</def> <mark>[Obs.]</mark> <ldquo/ <xex>Treen</xex> cups.<rdquo/  <rj><au>Camden.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Relating to, or drawn from, trees.</def> <mark>[Obs.]</mark>  <rj><au>Spenser.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q><qex>Treen</qex> liquors, especially that of the date.</q> <rj><qau>Evelyn.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Treen</ent><br/
<hw>Treen</hw>, <mark>obs.</mark> <def><pos>pl.</pos> of <er>Tree</er>.</def> <ldquo/ The shady <xex>treen</xex>.<rdquo/  <rj><au>Fairfax.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Treenail</ent><br/
<hw>Tree"nail`</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Tree + nail.]</ety> <fld>(Shipbuilding)</fld> <def>A long wooden pin used in fastening the planks of a vessel to the timbers or to each other.</def> <altsp>[Written also <asp>trenail</asp>, and <asp>trunnel</asp>.]</altsp><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tref</ent><br/
<hw>Tref</hw> <pr>(tr<ecr/f)</pr>, <pos>a.</pos> <ety>[Yiddish, fr. Heb. <ets>t<ecr/r<emac/ph<amac/h</ets> an animal torn by wild beasts.]</ety> <def>Ceremonially unclean, according to the Jewish law; -- opposed to <ant>kosher</ant>.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Trefle</ent><br/
<hw>Tre"fle</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F. <ets>tr<eacute/fle</ets>. See <er>Trefoil</er>.]</ety> <fld>(Fort.)</fld> <def>A species of time; -- so called from its resemblance in form to a trefoil.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Trefle</ent><br/
||<hw>Tr<eacute/`fl<eacute/"</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[F. <ets>tr<eacute/fl<eacute/</ets>. See <er>Trefoil</er>.]</ety> <fld>(Her.)</fld> <def>Having a three-lobed extremity or extremities, as a cross; also, more rarely, ornamented with trefoils projecting from the edges, as a bearing.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Trefoil</ent><br/
<hw>Tre"foil</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>tres</ets> three + E. <ets>foil</ets> leaf; cf. F. <ets>fr<egrave/fle</ets>, It. <ets>trifoglio</ets>, L. <ets>trifolium</ets>. See <er>Tri-</er>, <er>Foil</er> leaf, and cf. <er>Trifoly</er>.]</ety> <sn>1.</sn> <fld>(Bot.)</fld> <def>Any plant of the genus <gen>Trifolium</gen>, which includes the white clover, red clover, etc.; -- less properly, applied also to the nonesuch, or black medic. See <er>Clover</er>, and <er>Medic</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Arch.)</fld> <def>An ornamental foliation consisting of three divisions, or foils.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <fld>(Her.)</fld> <def>A charge representing the clover leaf.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Trefoiled</ent><br/
<hw>Tre"foiled`</hw> <pr>(?)</pr>, <pos>a.</pos> <fld>(Her.)</fld> <def>Same as <er>Tr<eacute/fl<eacute/</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Treget</ent><br/
<hw>Treg"et</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[See <er>Tregetour</er>.]</ety> <def>Guile; trickery.</def> <mark>[Obs.]</mark>  <rj><au>Rom. of R.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tregetour</ent><br/
<hw>Treg"et*our</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[OE. <ets>tresgeteor</ets>. See <er>Trans-</er>, and <er>Jet</er> a shooting forth.]</ety> <def>A juggler who produces illusions by the use of elaborate machinery.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>Divers appearances<br/
Such as these subtle <qex>tregetours</qex> play.</q> <rj><qau>Chaucer.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tregetry</ent><br/
<hw>Treg"et*ry</hw> <pr>(?)</pr>, <pos>n.</pos> <def>Trickery; also, a trick.</def> <mark>[Obs.]</mark>  <rj><au>Rom. of R.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Trehala</ent><br/
<hw>Tre*ha"la</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Chem.)</fld> <def>An amorphous variety of manna obtained from the nests and cocoons of a Syrian coleopterous insect (<spn>Larinus maculatus</spn>, <spn>Larinus nidificans</spn>, etc.) which feeds on the foliage of a variety of thistle. It is used as an article of food, and is called also <altname>nest sugar</altname>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Trehalose</ent><br/
<hw>Tre"ha*lose`</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Chem.)</fld> <def>Mycose; -- so called because sometimes obtained from <ets>trehala</ets>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Treillage</ent><br/
<hw>Treil"lage</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F. <ets>treillage</ets>. See <er>Trellis</er>.]</ety> <def>Latticework for supporting vines, etc.; an espalier; a trellis.</def>  <rj><au>Spectator.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>I shall plant the roses against my <qex>treillage</qex> to-morrow.</q> <rj><qau>Walpole.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Trek</ent><br/
<hw>Trek</hw> <pr>(?)</pr>, <pos>v. i.</pos>  <vmorph>[<pos>imp. & p. p.</pos> <conjf>Trekked</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Trekking</conjf>.]</vmorph> <altsp>[Written also <asp>treck</asp>.]</altsp> <ety>[D. <ets>trekken</ets>. See <er>Track</er>, <pos>n.</pos>]</ety> <mark>[South Africa]</mark> <sn>1.</sn> <def>To draw or haul a load, as oxen.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><sn>2.</sn>  <def>To travel, esp. by ox wagon; to go from place to place; to migrate.</def> <mark>[Chiefly South Africa]</mark></p>

<p><q>One of the motives which induced the Boers of 1836 to <qex>trek</qex> out of the Colony.</q>  <rj><qau>James Bryce.</qau></rj><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Trek</ent><br/
<hw>Trek</hw>, <pos>n.</pos> <altsp>[Written also <asp>treck</asp>.]</altsp> <ety>[D. Cf. <er>Track</er>, <pos>n.</pos>]</ety> <def>The act of trekking; a drawing or a traveling; a journey; a migration.</def> <mark>[Chiefly South Africa]</mark></p>

<p><q>To the north a <qex>trek</qex> was projected, and some years later was nearly carried out, for the occupation of the Mashonaland.</q>  <rj><qau>James Bryce.</qau></rj></p>

<p><cs><col><b>Great Trek</b></col>, <cd>the great emigration of Boers from Cape Colony which began in 1836, and resulted in the founding of the South African Republic and Orange Free State.</cd></cs><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Trekker</ent><br/
<hw>Trek"ker</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[D.]</ety> <def>One that treks.</def> <altsp>[Written also <asp>trecker</asp>.]</altsp> <mark>[South Africa]</mark>  <rj><au>James Bryce.</au></rj><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Trekometer</ent><br/
<hw>Trek*om"e*ter</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Cf. <er>Trechometer</er>.]</ety> <fld>(Mil.)</fld> <def>A field range finger used in the British service.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Trellis</ent><br/
<hw>Trel"lis</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[OE. <ets>relis</ets>, F. <ets>treillis</ets>, fr. <ets>treille</ets> vine arbor, fr. L. <ets>tricla</ets>, <ets>triclea</ets>, <ets>triclia</ets>, a bower, arbor, summer house.]</ety> <def>A structure or frame of crossbarred work, or latticework, used for various purposes, as for screens or for supporting plants.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Trellised</ent><br/
<hw>Trel"lised</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Having a trellis or trellises.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Cottages <qex>trellised</qex> over with exotic plants.</q> <rj><qau>Jeffrey.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tremando</ent><br/
||<hw>Tre*man"do</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[It.]</ety> <fld>(Mus.)</fld> <def>Trembling; -- used as a direction to perform a passage with a general shaking of the whole chord.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Trematode</ent><br/
<hw>Trem"a*tode</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Zool.)</fld> <def>One of the Trematodea. Also used adjectively.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Trematodea</ent><br/
||<hw>Trem`a*to"de*a</hw> <pr>(?)</pr>, <pos>n. pl.</pos> <ety>[NL., from Gr. <?/ having holes, from <?/, <?/, a hole.]</ety> <fld>(Zool.)</fld> <def>An extensive order of parasitic worms. They are found in the internal cavities of animals belonging to all classes. Many species are found, also, on the gills and skin of fishes. A few species are parasitic on man, and some, of which the fluke is the most important, are injurious parasites of domestic animals. The trematodes usually have a flattened body covered with a chitinous skin, and are furnished with two or more suckers for adhesion. Most of the species are hermaphrodite. Called also <altname>Trematoda</altname>, and <altname>Trematoidea</altname>. See <er>Fluke</er>, <er>Tristoma</er>, and <er>Cercaria</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Trematoid</ent><br/
<hw>Trem"a*toid</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[From Gr. <?/, <?/, a hole + <grk>e'i^dos</grk> form.]</ety> <fld>(Zool.)</fld> <def>Of or pertaining to the Trematodea. See <xex>Illustration</xex> in Appendix.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tremble</ent><br/
<hw>Trem"ble</hw> <pr>(?)</pr>, <pos>v. i.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Trembled</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Trembling</conjf> <pr>(?)</pr>.]</vmorph> <ety>[F. <ets>trembler</ets>, fr. L. <ets>tremulus</ets> trembling, tremulous, fr. <ets>tremere</ets> to shake, tremble; akin to Gr. <?/, Lith. <ets>trimti</ets>.  Cf. <er>Tremulous</er>, <er>Tremor</er>.]</ety> <sn>1.</sn> <def>To shake involuntarily, as with fear, cold, or weakness; to quake; to quiver; to shiver; to shudder; -- said of a person or an animal.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>I <qex>tremble</qex> still with fear.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Frighted Turnus <qex>trembled</qex> as he spoke.</q> <rj><qau>Dryden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To totter; to shake; -- said of a thing.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The Mount of Sinai, whose gray top<br/
Shall <qex>tremble</qex>.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>To quaver or shake, as sound; to be tremulous; as the voice <xex>trembles</xex>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tremble</ent><br/
<hw>Trem"ble</hw>, <pos>n.</pos> <def>An involuntary shaking or quivering.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>I am all of a <qex>tremble</qex> when I think of it.</q> <rj><qau>W. Black.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Trembler</ent><br/
<hw>Trem"bler</hw> <pr>(?)</pr>, <pos>n.</pos> <def>One who trembles.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Any of certain West Indian birds of the genera <gen>Cinclocerthia</gen> and <gen>Rhamphocinclus</gen>, of the family <fam>Mimidae</fam>.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><sn>3.</sn>  <fld>(Elec.)</fld> <def>The vibrating hammer, or spring contact piece of a hammer break, as of the electric ignition apparatus for an internal-combustion engine.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Trembling</ent><br/
<hw>Trem"bling</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Shaking; tottering; quivering.</def> -- <wordforms><wf>Trem"bling*ly</wf>, <pos>adv.</pos></wordforms><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Trembling poplar</b></col> <fld>(Bot.)</fld>, <cd>the aspen.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tremella</ent><br/
||<hw>Tre*mel"la</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[NL., fr. L. <ets>tremere</ets>, to tremble: cf. It. <ets>tremella</ets>, F. <ets>tremelle</ets>.]</ety> <fld>(Bot.)</fld> <def>A genus of gelatinous fungi found in moist grounds.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tremendous</ent><br/
<hw>Tre*men"dous</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>tremendus</ets> that is to be trembled at, fearful, fr. <ets>tremere</ets> to tremble. See <er>Tremble</er>.]</ety> <def>Fitted to excite fear or terror; such as may astonish or terrify by its magnitude, force, or violence; terrible; dreadful; <as>as, a <ex>tremendous</ex> wind; a <ex>tremendous</ex> shower; a <ex>tremendous</ex> shock or fall.</as></def><br/
[<source>1913 Webster</source>]</p>

<p><q>A <qex>tremendous</qex> mischief was a foot.</q> <rj><qau>Motley.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><syn><b>Syn.</b> -- Terrible; dreadful; frightful; terrific; horrible; awful.</syn><br/
[<source>1913 Webster</source>]</p>

<p>-- <wordforms><wf>Tre*men"dous*ly</wf>, <pos>adv.</pos> -- <wf>Tre*men"dous*ness</wf>, <pos>n.</pos></wordforms><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tremex</ent><br/
||<hw>Tre"mex</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[NL.]</ety> <fld>(Zool.)</fld> <def>A genus of large hymenopterous insects allied to the sawflies. The female lays her eggs in holes which she bores in the trunks of trees with her large and long ovipositor, and the larva bores in the wood. See <xex>Illust.</xex> of <er>Horntail</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ The pigeon tremex (<spn>Tremex columba</spn>), a common American species, infests the elm, pear, and other trees.</note><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tremie</ent><br/
<hw>Trem"ie</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F. <ets>tr<eacute/mie</ets> hopper.]</ety> <fld>(Hydraulic Engin.)</fld> <def>An apparatus for depositing and consolidating concrete under water, essentially a tube of wood or sheet metal with a hooperlike top. It is usually handled by a crane.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Tremolando</ent><br/
||<hw>Tre*mo*lan"do</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[It.]</ety> <fld>(Mus.)</fld> <def>Same as <er>Tremando</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tremolite</ent><br/
<hw>Trem"o*lite</hw>, <pos>n.</pos> <ety>[From <ets>Tremola</ets>, a valley in the Alps, where it was discovered.]</ety> <fld>(Min.)</fld> <def>A white variety of amphibole, or hornblende, occurring in long, bladelike crystals, and coarsely fibrous masses.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tremolo</ent><br/
||<hw>Tre"mo*lo</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[It.  Cf. <er>Tremulous</er>.]</ety> <fld>(Mus.)</fld> <sd>(a)</sd> <def>The rapid reiteration of tones without any apparent cessation, so as to produce a tremulous effect.</def> <sd>(b)</sd> <def>A certain contrivance in an organ, which causes the notes to sound with rapid pulses or beats, producing a tremulous effect; -- called also <altname>tremolant</altname>, and <altname>tremulant</altname>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tremor</ent><br/
<hw>Tre"mor</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L., from <ets>tremere</ets> to tremble. See <er>Tremble</er>, <pos>v.</pos>]</ety> <def>A trembling; a shivering or shaking; a quivering or vibratory motion; <as>as, the <ex>tremor</ex> of a person who is weak, infirm, or old</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>He fell into an universal <qex>tremor</qex> of all his joints.</q> <rj><qau>Harvey.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tremulent</ent><br/
<ent>Tremulant</ent><br/
<mhw>{ <hw>Trem"u*lant</hw> <pr>(?)</pr>, <hw>Trem"u*lent</hw> <pr>(?)</pr>, }</mhw> <pos>a.</pos> <def>Tremulous; trembling; shaking.</def> <mark>[R.]</mark> <ldquo/ With <xex>tremulent</xex> white rod.<rdquo/  <rj><au>Carlyle.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tremulous</ent><br/
<hw>Trem"u*lous</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>tremulus</ets>, fr. <ets>tremere</ets> to tremble. See <er>Tremble</er>.]</ety> <sn>1.</sn> <def>Shaking; shivering; quivering; <as>as, a <ex>tremulous</ex> limb; a <ex>tremulous</ex> motion of the hand or the lips; the <ex>tremulous</ex> leaf of the poplar</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Affected with fear or timidity; trembling.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The tender, <qex>tremulous</qex> Christian.</q> <rj><qau>Dr. H. More.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p>-- <wordforms><wf>Trem"u*lous*ly</wf>, <pos>adv.</pos> -- <wf>Trem"u*lous*ness</wf>, <pos>n.</pos></wordforms><br/
[<source>1913 Webster</source>]</p>

<p><-- p. 1535 --><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tren</ent><br/
<hw>Tren</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[See <er>Treen</er> wooden.]</ety> <def>A fish spear.</def> <mark>[Obs.]</mark>  <rj><au>Ainsworth.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Trenail</ent><br/
<hw>Tre"nail`</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Shipbuilding)</fld> <def>Same as <er>Treenail</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Trench</ent><br/
<hw>Trench</hw> <pr>(?)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Trenched</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Trenching</conjf>.]</vmorph> <ety>[OF. <ets>trenchier</ets> to cut, F. <ets>trancher</ets>; akin to Pr. <ets>trencar</ets>, <ets>trenchar</ets>, Sp. <ets>trinchar</ets>, It. <ets>trinciare</ets>; of uncertain origin.]</ety> <sn>1.</sn> <def>To cut; to form or shape by cutting; to make by incision, hewing, or the like.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The wide wound that the boar had <qex>trenched</qex><br/
In his soft flank.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>This weak impress of love is as a figure<br/
<qex>Trenched</qex> in ice, which with an hour's heat<br/
Dissolves to water, and doth lose its form.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Fort.)</fld> <def>To fortify by cutting a ditch, and raising a rampart or breastwork with the earth thrown out of the ditch; to intrench.</def>  <rj><au>Pope.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>No more shall <qex>trenching</qex> war channel her fields.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>To cut furrows or ditches in; <as>as, to <ex>trench</ex> land for the purpose of draining it</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>To dig or cultivate very deeply, usually by digging parallel contiguous trenches in succession, filling each from the next; <as>as, to <ex>trench</ex> a garden for certain crops</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Trench</ent><br/
<hw>Trench</hw>, <pos>v. i.</pos> <sn>1.</sn> <def>To encroach; to intrench.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Does it not seem as if for a creature to challenge to itself a boundless attribute, were to <qex>trench</qex> upon the prerogative of the divine nature?</q> <rj><qau>I. Taylor.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To have direction; to aim or tend.</def> <mark>[R.]</mark>  <rj><au>Bacon.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>To trench at</b></col>, <cd>to make trenches against; to approach by trenches, as a town in besieging it.</cd> <mark>[Obs.]</mark></cs><br/
[<source>1913 Webster</source>]</p>

<p><q>Like powerful armies, <qex>trenching at</qex> a town<br/
By slow and silent, but resistless, sap.</q> <rj><qau>Young.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Trench</ent><br/
<hw>Trench</hw>, <pos>n.</pos> <ety>[OE. <ets>trenche</ets>, F. <ets>tranch<eacute/e</ets>. See <er>Trench</er>, <pos>v. t.</pos>]</ety> <sn>1.</sn> <def>A long, narrow cut in the earth; a ditch; <as>as, a <ex>trench</ex> for draining land</as>.</def>  <rj><au>Mortimer.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>An alley; a narrow path or walk cut through woods, shrubbery, or the like.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>In a <qex>trench</qex>, forth in the park, goeth she.</q> <rj><qau>Chaucer.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <fld>(Fort.)</fld> <def>An excavation made during a siege, for the purpose of covering the troops as they advance toward the besieged place. The term includes the <xex>parallels</xex> and the <xex>approaches</xex>.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>To open the trenches</b></col> <fld>(Mil.)</fld>, <cd>to begin to dig or to form the lines of approach.</cd> -- <col><b>Trench cavalier</b></col> <fld>(Fort.)</fld>, <cd>an elevation constructed (by a besieger) of gabions, fascines, earth, and the like, about half way up the glacis, in order to discover and enfilade the covered way.</cd> -- <mcol><col><b>Trench plow</b></col>, <it>or</it> <col><b>Trench plough</b></col></mcol>, <cd>a kind of plow for opening land to a greater depth than that of common furrows.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Trenchand</ent><br/
<hw>Trench"and</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Trenchant.</def> <mark>[Obs.]</mark>  <rj><au>Spenser.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Trenchant</ent><br/
<hw>Trench"ant</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[OF. <ets>trenchant</ets>, F. <ets>tranchant</ets>, p. pr. See <er>Trench</er>, <pos>v. t.</pos>]</ety> <sn>1.</sn> <def>Fitted to trench or cut; gutting; sharp.</def> <ldquo/ <xex>Trenchant</xex> was the blade.<rdquo/  <rj><au>Chaucer.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Fig.: Keen; biting; severe; <as>as, <ex>trenchant</ex> wit</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Trenchantly</ent><br/
<hw>Trench"ant*ly</hw>, <pos>adv.</pos> <def>In a trenchant, or sharp, manner; sharply; severely.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Trencher</ent><br/
<hw>Trench"er</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[OE. <ets>trencheoir</ets>, F. <ets>tranchoir</ets>, fr. <ets>trancher</ets> to cut, carve. See <er>Trench</er>, <pos>v. t.</pos>]</ety> <sn>1.</sn> <def>One who trenches; esp., one who cuts or digs ditches.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A large wooden plate or platter, as for table use.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>The table; hence, the pleasures of the table; food.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>It could be no ordinary declension of nature that could bring some men, after an ingenuous education, to place their <ldquo/summum bonum<rdquo/ upon their <qex>trenchers</qex>.</q> <rj><qau>South.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Trencher cap</b></col>, <cd>the cap worn by studens at Oxford and Cambridge Universities, having a stiff, flat, square appendage at top. A similar cap used in the United States is called <altname>Oxford cap</altname>, <altname>mortar board</altname>, etc.</cd> -- <col><b>Trencher fly</b></col>, <cd>a person who haunts the tables of others; a parasite.</cd> <mark>[R.]</mark> <au>L'Estrange.</au> -- <col><b>Trencher friend</b></col>, <cd>one who frequents the tables of others; a sponger.</cd> -- <col><b>Trencher mate</b></col>, <cd>a table companion; a parasite; a trencher fly.</cd> <au>Hooker.</au></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Trencher-man</ent><br/
<hw>Trench"er-man</hw> <pr>(?)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Trencher-men</plw> <pr>(<?/)</pr>.</plu><br/
[<source>1913 Webster</source>]</p>

<p><sn>1.</sn> <def>A feeder; a great eater; a gormandizer.</def>  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A cook.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>The skillfulest <qex>trencher-men</qex> of Media.</q> <rj><qau>Sir P. Sidney.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>A table companion; a trencher mate.</def>  <rj><au>Thackeray.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Trenchmore</ent><br/
<hw>Trench"more</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A kind of lively dance of a rude, boisterous character. Also, music in triple time appropriate to the dance.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>All the windows in the town dance new <qex>trenchmore</qex>.</q> <rj><qau>Beau. & Fl.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Trenchmore</ent><br/
<hw>Trench"more</hw> <pr>(?)</pr>, <pos>v. i.</pos> <def>To dance the trenchmore.</def> <mark>[Obs.]</mark>  <rj><au>Marston.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Trench-plough</ent><br/
<ent>Trench-plow</ent><br/
<mhw>{ <hw>Trench"-plow"</hw>, <hw>Trench"-plough`</hw>  }</mhw> <pr>(-plou`)</pr>, <pos>v. t.</pos> <def>To plow with deep furrows, for the purpose of loosening the land to a greater depth than usual.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Trend</ent><br/
<hw>Trend</hw>, <pos>v. i.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Trended</conjf>; <pos>p. pr. & vb. n.</pos> <conjf>Trending</conjf>.]</vmorph> <ety>[OE. <ets>trenden</ets> to roll or turn about; akin to OFries. <ets>trind</ets>, <ets>trund</ets>, round, Dan. & Sw. <ets>trind</ets>, AS. <ets>trendel</ets> a circle, ring, and E. <ets>trendle</ets>, <ets>trundle</ets>.]</ety> <def>To have a particular direction; to run; to stretch; to tend; <as>as, the shore of the sea <ex>trends</ex> to the southwest</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Trend</ent><br/
<hw>Trend</hw>, <pos>v. t.</pos> <def>To cause to turn; to bend.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>Not far beneath i' the valley as she <qex>trends</qex><br/
Her silver stream.</q> <rj><qau>W. Browne.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Trend</ent><br/
<hw>Trend</hw>, <pos>n.</pos> <def>Inclination in a particular direction; tendency; general direction; <as>as, the <ex>trend</ex> of a coast</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Trend of an anchor</b></col>. <fld>(Naut.)</fld> <sd>(a)</sd> <cd>The lower end of the shank of an anchor, being the same distance on the shank from the throat that the arm measures from the throat to the bill.</cd> <au>R. H. Dana, Jr.</au> <sd>(b)</sd> <cd>The angle made by the line of a vessel's keel and the direction of the anchor cable, when she is swinging at anchor.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Trend</ent><br/
<hw>Trend</hw> <pr>(?)</pr>, <pos>v. t.</pos> <ety>[Cf. G. & OD. <ets>trennen</ets> to separate.]</ety> <def>To cleanse, as wool.</def> <mark>[Prov. Eng.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Trend</ent><br/
<hw>Trend</hw>, <pos>n.</pos> <def>Clean wool.</def> <mark>[Prov. Eng.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Trender</ent><br/
<hw>Trend"er</hw> <pr>(?)</pr>, <pos>n.</pos> <def>One whose business is to free wool from its filth.</def> <mark>[Prov. Eng.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Trendle</ent><br/
<hw>Tren"dle</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[AS. <ets>trendel</ets>, <ets>tryndel</ets>, circle, ring. See <er>Trend</er>, <pos>v. i.</pos>, and cf. <er>Trundle</er>.]</ety> <def>A wheel, spindle, or the like; a trundle.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>The shaft the wheel, the wheel, the <qex>trendle</qex> turns.</q> <rj><qau>Sylvester.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Trennel</ent><br/
<hw>Tren"nel</hw> <pr>(?)</pr>, <pos>n.</pos> <def>Corrupt form of <er>Treenail</er>.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Trental</ent><br/
<hw>Tren"tal</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[LL. <ets>trentale</ets>, fr. L. <ets>triginta</ets> thirty; akin to <ets>tres</ets> three: cf. OF. <ets>trentel</ets>. See <er>Three</er>, and cf. <er>Trigintal</er>.]</ety> <sn>1.</sn> <fld>(R. C. Ch.)</fld> <def>An office and mass for the dead on the thirtieth day after death or burial.</def> <ldquo/Their <xex>trentals</xex> and their shrifts.<rdquo/  <rj><au>Spenser.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Hence, a dirge; an elegy.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Trente et quarante</ent><br/
||<hw>Trente" et` qua`rante"</hw> <pr>(?)</pr>. <ety>[F., lit., thirty and forty.]</ety> <def>Same as <altname>Rouge et noir</altname>, under <er>Rouge</er>.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Trenton period</ent><br/
<hw>Tren"ton pe"ri*od</hw> <pr>(?)</pr>. <fld>(Geol.)</fld> <def>A subdivision in the lower Silurian system of America; -- so named from <etsep>Trenton</etsep> Falls, in New York. The rocks are mostly limestones, and the period is divided into the <xex>Trenton</xex>, <xex>Utica</xex>, and <xex>Cincinnati epochs</xex>. See the <xex>Chart</xex> of <er>Geology</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Trepan</ent><br/
<hw>Tre*pan"</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F. <ets>tr<eacute/pan</ets> (cf. Sp. <ets>tr<eacute/pano</ets>, It. <ets>trepano</ets>, <ets>trapano</ets>), LL. <ets>trepanum</ets>, fr. Gr. <?/ a borer, auger, trepan, fr. <?/ to bore, <?/ a hole.  Cf. <er>Trephine</er>.]</ety> <sn>1.</sn> <fld>(Surg.)</fld> <def>A crown-saw or cylindrical saw for perforating the skull, turned, when used, like a bit or gimlet. See <er>Trephine</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Mining)</fld> <def>A kind of broad chisel for sinking shafts.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Trepan</ent><br/
<hw>Tre*pan"</hw>, <pos>v. t. & i.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Trepanned</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Trepanning</conjf>.]</vmorph> <ety>[Cf. F. <ets>tr<eacute/paner</ets>. See <er>Trepan</er> a saw.]</ety> <fld>(Surg.)</fld> <def>To perforate (the skull) with a trepan, so as to remove a portion of the bone, and thus relieve the brain from pressure or irritation; to perform an operation with the trepan.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Trepan</ent><br/
<hw>Tre*pan"</hw>, <pos>n.</pos> <ety>[See <er>Trapan</er>.]</ety> <sn>1.</sn> <def>A snare; a trapan.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Snares and <qex>trepans</qex> that common life lays in its way.</q> <rj><qau>South.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>a deceiver; a cheat.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>He had been from the beginning a spy and a <qex>trepan</qex>.</q> <rj><qau>Macaulay.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Trepan</ent><br/
<hw>Tre*pan"</hw>, <pos>v. t.</pos> <def>To insnare; to trap; to trapan.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Guards even of a dozen men were silently <qex>trepanned</qex> from their stations.</q> <rj><qau>De Quincey.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Trepang</ent><br/
<hw>Tre*pang"</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Malay <ets>tr<imac/pang</ets>.]</ety> <fld>(Zool.)</fld> <def>Any one of several species of large holothurians, some of which are dried and extensively used as food in China; -- called also <altname>b<ecir/che de mer</altname>, <altname>sea cucumber</altname>, and <altname>sea slug</altname>.</def> <altsp>[Written also <asp>tripang</asp>.]</altsp><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ The edible trepangs are mostly large species of <gen>Holothuria</gen>, especially <spn>Holothuria edulis</spn>.  They are taken in vast quantities in the <geog>East Indies</geog>, where they are dried and smoked, and then shipped to <country>China</country>.  They are used as an ingredient in certain kinds of soup.</note><br/
[<source>1913 Webster</source>]</p>

<p><ent>Trepanize</ent><br/
<hw>Trep"an*ize</hw> <pr>(?)</pr>, <pos>v. t.</pos> <def>To trepan.</def> <mark>[Obs.]</mark> <ldquo/By <xex>trepanizing</xex> the skull.<rdquo/  <rj><au>Jer. Taylor.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Trepanner</ent><br/
<hw>Tre*pan"ner</hw> <pr>(?)</pr>, <pos>n.</pos> <def>One who trepans.</def> <ldquo/ Pitiful <xex>trepanners</xex> and impostors.<rdquo/  <rj><au>Gauden.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Trepeget</ent><br/
<hw>Trep"e*get</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Mil.)</fld> <def>A trebuchet.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Trephine</ent><br/
<hw>Tre*phine"</hw> <pr>(?; 277)</pr>, <pos>n.</pos> <ety>[A dim. of 1st <ets>trepan</ets>: cf. F. <ets>tr<eacute/phine</ets>.]</ety> <fld>(Surg.)</fld> <def>An instrument for trepanning, being an improvement on the trepan. It is a circular or cylindrical saw, with a handle like that of a gimlet, and a little sharp perforator called the <xex>center pin</xex>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Trephine</ent><br/
<hw>Tre*phine"</hw>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Trephined</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Trephining</conjf>.]</vmorph> <def>To perforate with a trephine; to trepan.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Trepid</ent><br/
<hw>Trep"id</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>trepidus</ets>.]</ety> <def>Trembling; quaking.</def>  <rj><au>Thackeray.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Trepidation</ent><br/
<hw>Trep`i*da"tion</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F. <ets>tr<eacute/pidation</ets>, L. <ets>trepidatio</ets>, fr. <ets>trepidare</ets> to hurry with alarm, to tremble, from <ets>trepidus</ets> agitated, disturbed, alarmed; cf. <ets>trepit</ets> he turns, Gr. <?/ to turn, E. <ets>torture</ets>.]</ety> <sn>1.</sn> <def>An involuntary trembling, sometimes an effect of paralysis, but usually caused by terror or fear; quaking; quivering.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Hence, a state of terror or alarm; fear; confusion; fright; <as>as, the men were in great <ex>trepidation</ex></as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <fld>(Anc. Astron.)</fld> <def>A libration of the starry sphere in the Ptolemaic system; a motion ascribed to the firmament, to account for certain small changes in the position of the ecliptic and of the stars.</def><br/
[<source>1913 Webster</source>]</p>

<p><syn><b>Syn.</b> -- Tremor; agitation; disturbance; fear.</syn><br/
[<source>1913 Webster</source>]</p>

<p><ent>Trepidity</ent><br/
<hw>Tre*pid"i*ty</hw>, <pos>n.</pos> <def>Trepidation.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tresayle</ent><br/
<hw>Tres"ayle`</hw> <pr>(tr<ecr/s"<amac/l`)</pr>, <pos>n.</pos> <ety>[F. <ets>trisa<ium/eul</ets>, from L. <ets>tris</ets>, <ets>tres</ets>, three + F. <ets>a<ium/eul</ets> grandfather.  Cf. <er>Besaiel</er>, and see <er>Ayle</er>.]</ety> <def>A grandfather's grandfather.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Writ of tresayle</b></col> <fld>(O. Eng. Law)</fld>, <cd>a writ which lay for a man claiming as heir to his grandfather's grandfather, to recover lands of which he had been deprived by an abatement happening on the ancestor's death.</cd>  <rj><au>Mozley & W.</au></rj></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tresor</ent><br/
<hw>Tres"or</hw> <pr>(?)</pr>, <pos>n.</pos> <def>Treasure.</def> <mark>[Obs.]</mark>  <rj><au>Chaucer.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Trespass</ent><br/
<hw>Tres"pass</hw> <pr>(?)</pr>, <pos>v. i.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Trespassed</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Trespassing</conjf>.]</vmorph> <ety>[<er>OF</er>. <ets>trespasser</ets> to go across or over, transgress, F. <ets>tr<eacute/passer</ets> to die; pref. <ets>tres-</ets> (L. <ets>trans</ets> across, over) + <ets>passer</ets> to pass. See <er>Pass</er>, <pos>v. i.</pos>, and cf. <er>Transpass</er>.]</ety> <sn>1.</sn> <def>To pass beyond a limit or boundary; hence, to depart; to go.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>Soon after this, noble Robert de Bruce . . . <qex>trespassed</qex> out of this uncertain world.</q> <rj><qau>Ld. Berners.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Law)</fld> <def>To commit a trespass; esp., to enter unlawfully upon the land of another.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>To go too far; to put any one to inconvenience by demand or importunity; to intrude; <as>as, to <ex>trespass</ex> upon the time or patience of another</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>To commit any offense, or to do any act that injures or annoys another; to violate any rule of rectitude, to the injury of another; hence, in a moral sense, to transgress voluntarily any divine law or command; to violate any known rule of duty; to sin; -- often followed by <xex>against</xex>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>In the time of his distress did he <qex>trespass</qex> yet more against the Lord.</q> <rj><qau>2 Chron. xxviii. 22.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Trespass</ent><br/
<hw>Tres"pass</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[OF. <ets>trespas</ets>, F. <ets>tr<eacute/pas</ets> death. See <er>Trespass</er>, <pos>v.</pos>]</ety> <sn>1.</sn> <def>Any injury or offence done to another.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>I you forgive all wholly this <qex>trespass</qex>.</q> <rj><qau>Chaucer.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>If ye forgive not men their <qex>trespasses</qex>, neither will your Father forgive your <qex>trespasses</qex>.</q> <rj><qau>Matt. vi. 15.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Any voluntary transgression of the moral law; any violation of a known rule of duty; sin.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The fatal <qex>trespass</qex> done by Eve.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>You . . . who were dead in <qex>trespasses</qex> and sins.</q> <rj><qau>Eph. if. 1.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <fld>(Law)</fld> <sd>(a)</sd> <def>An unlawful act committed with force and violence (<xex>vi et armis</xex>) on the person, property, or relative rights of another.</def> <sd>(b)</sd> <def>An action for injuries accompanied with force.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Trespass offering</b></col> <fld>(Jewish Antiq.)</fld>, <cd>an offering in expiation of a trespass.</cd> -- <col><b>Trespass on the case</b></col>. <fld>(Law)</fld> <cd>See <cref>Action on the case</cref>, under <er>Case</er>.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><syn><b>Syn.</b> -- Offense; breach; infringement; transgression; misdemeanor; misdeed.</syn><br/
[<source>1913 Webster</source>]</p>

<p><ent>Trespasser</ent><br/
<hw>Tres"pass*er</hw> <pr>(?)</pr>, <pos>n.</pos> <def>One who commits a trespass</def>; as: <sd>(a)</sd> <fld>(Law)</fld> <def>One who enters upon another's land, or violates his rights.</def> <sd>(b)</sd> <def>A transgressor of the moral law; an offender; a sinner.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tress</ent><br/
<hw>Tress</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[OE. <ets>tresse</ets>, OF. <ets>trece</ets>, F. <ets>tresse</ets>, LL. <ets>tricia</ets>, fr. Gr. <grk>tri`cha</grk> threefold, because a tress is usually formed by interlacing three pieces; akin to <grk>trei^s</grk> three. See <er>Three</er>.]</ety> <sn>1.</sn> <def>A braid, knot, or curl, of hair; a ringlet.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Her yellow hair was braided in a <qex>tress</qex>.</q> <rj><qau>Chaucer.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Fair <qex>tresses</qex> man's imperial race insnare.</q> <rj><qau>Pope.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Fig.: A knot or festoon, as of flowers.</def>  <rj><au>Keats.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tressed</ent><br/
<hw>Tressed</hw> <pr>(?)</pr>, <pos>a.</pos> <sn>1.</sn> <def>Having tresses.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Formed into ringlets or braided; braided; curled.</def>  <rj><au>Spenser. Drayton.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tressel</ent><br/
<hw>Tres"sel</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A trestle.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tressful</ent><br/
<hw>Tress"ful</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Tressy.</def> <mark>[R.]</mark>  <rj><au>Sylvester.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tressure</ent><br/
<hw>Tres"sure</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F. <ets>tresser</ets> to twist, plait. See <er>Tress</er>, <pos>n.</pos>]</ety> <fld>(Her.)</fld> <def>A kind of border similar to the orle, but of only half the breadth of the latter.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tressured</ent><br/
<hw>Tres"sured</hw> <pr>(?)</pr>, <pos>a.</pos> <fld>(Her.)</fld> <def>Provided or bound with a tressure; arranged in the form of a tressure.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The <qex>tressured</qex> fleur-de-lis he claims<br/
To wreathe his shield.</q> <rj><qau>Sir W. Scott.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tressy</ent><br/
<hw>Tress"y</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Abounding in tresses.</def>  <rj><au>J. Baillie.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tres-tine</ent><br/
<hw>Tres"-tine`</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Cf. L. <ets>tris</ets>, <ets>tres</ets>, three, and E. <ets>tyne</ets>, <ets>tine</ets>, a prong.]</ety> <def>The third tine above the base of a stag's antler; the royal antler.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Trestle</ent><br/
<hw>Tres"tle</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[OF. <ets>trestel</ets>, <ets>tresteay</ets>, F. <ets>tr<eacute/teau</ets>; probably from L. <ets>transtillum</ets> a little crossbeam, dim. of <ets>transtrum</ets> a crossbeam.  Cf. <er>Transom</er>.]</ety> <altsp>[Written also <asp>tressel</asp>.]</altsp> <sn>1.</sn> <def>A movable frame or support for anything, as scaffolding, consisting of three or four legs secured to a top piece, and forming a sort of stool or horse, used by carpenters, masons, and other workmen; also, a kind of framework of strong posts or piles, and crossbeams, for supporting a bridge, the track of a railway, or the like.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>The frame of a table.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Trestle board</b></col>, <cd>a board used by architects, draughtsmen, and the like, for drawing designs upon; -- so called because commonly supported by trestles.</cd> -- <col><b>Trestle bridge</b></col>. <cd>See under <er>Bridge</er>, <pos>n.</pos></cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Trestletree</ent><br/
<hw>Tres"tle*tree`</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Naut.)</fld> <def>One of two strong bars of timber, fixed horizontally on the opposite sides of the masthead, to support the crosstrees and the frame of the top; -- generally used in the plural.</def>  <rj><au>Totten.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Trestlework</ent><br/
<hw>Tres"tle*work`</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A viaduct, pier, scaffold, or the like, resting on trestles connected together.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tres-tyne</ent><br/
<hw>Tres"-tyne`</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>tris</ets>, <ets>tres</ets>, three + E. <ets>tyne</ets>.]</ety> <fld>(Zool.)</fld> <def>In the antler of a stag, the third tyne above the base. This tyne appears in the third year. In those deer in which the brow tyne does not divide, the tres-tyne is the second tyne above the base. See <xex>Illust.</xex> under <er>Rucervine</er>, and under <er>Rusine</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tret</ent><br/
<hw>Tret</hw> <pr>(?)</pr>, <mark>obs.</mark> <def><pos>3d pers. sing. pres.</pos> of <er>Tread</er>, for <xex>treadeth</xex>.</def>  <rj><au>Chaucer.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tret</ent><br/
<hw>Tret</hw>, <pos>n.</pos> <ety>[F. <ets>traite</ets> a drawing, trading, journey, tax on wares in transit, anything diminishing the value of coins, fr. OF. <ets>traire</ets> to draw, L. <ets>trahere</ets>. See <er>Trait</er>.]</ety> <fld>(Com.)</fld> <def>An allowance to purchasers, for waste or refuse matter, of four pounds on every 104 pounds of suttle weight, or weight after the tare deducted.</def>  <rj><au>M'Culloch.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tretable</ent><br/
<hw>Tret"a*ble</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[See <er>Treatable</er>.]</ety> <def>Tractable; moderate.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>By nature debonaire and <qex>tretable</qex>.</q> <rj><qau>Chaucer.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Trething</ent><br/
<hw>Treth"ing</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[W. <ets>treth</ets> an allowance, contribution, tribute, or tax, <ets>trethu</ets> to rate or tax.]</ety> <def>A tax; an impost.</def> <mark>[Obs.]</mark>  <rj><au>Johnson.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tretys</ent><br/
<ent>Tretis</ent><br/
<mhw>{ <hw>Tre"tis</hw> <pr>(?)</pr>, <hw>Tre"tys</hw>  }</mhw>, <pos>n.</pos> <ety>[See <er>Treatise</er>.]</ety> <def>A treatise; also, a treaty.</def> <mark>[Obs.]</mark>  <rj><au>Chaucer.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tretys</ent><br/
<ent>Tretis</ent><br/
<mhw>{ <hw>Tre*tis"</hw> <pr>(?)</pr>, <hw>Tre*tys"</hw>  }</mhw>, <pos>a.</pos> <ety>[OF. <ets>traitis</ets>.]</ety> <def>Long and well-proportioned; nicely made; pretty.</def> <mark>[Obs.]</mark> <ldquo/Her nose <xex>tretys</xex>.<rdquo/  <rj><au>Chaucer.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Trevat</ent><br/
<hw>Tre"vat</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A weaver's cutting instrument; for severing the loops of the pile threads of velvet.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Trevet</ent><br/
<hw>Trev"et</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[See <er>Trivet</er>.]</ety> <def>A stool or other thing supported by three legs; a trivet.</def><br/
[<source>1913 Webster</source>]</p>

<p><-- p. 1536 --><br/
[<source>1913 Webster</source>]</p>

<p><ent>Trewe</ent><br/
<ent>Trew</ent><br/
<mhw>{ <hw>Trew</hw> <pr>(?)</pr>, <hw>Trewe</hw>  }</mhw>, <pos>a.</pos> <def>True.</def> <mark>[Obs.]</mark>  <rj><au>Chaucer.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Trews</ent><br/
<hw>Trews</hw> <pr>(?)</pr>, <pos>n. pl.</pos> <def>Trowsers; especially, those of the Scotch Highlanders.</def> <ldquo/He wore the <xex>trews</xex>, or close trowsers, made of tartan.<rdquo/  <rj><au>Sir W. Scott.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Trewth</ent><br/
<hw>Trewth</hw> <pr>(?)</pr>, <pos>n.</pos> <def>Truth.</def> <mark>[Obs.]</mark>  <rj><au>Chaucer.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Trey</ent><br/
<hw>Trey</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[OF. <ets>treis</ets> three, F. <ets>trois</ets>, L. <ets>tres</ets>. See <er>Three</er>, and cf. <er>Tray-trip</er>.]</ety> <def>Three, at cards, dice, or dominoes; a card, die, or domino of three spots or pips.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Seven is my chance and thine is cinq and <qex>trey</qex>.</q> <rj><qau>Chaucer.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tri-</ent><br/
<hw>Tri-</hw> <pr>(?)</pr>. <ety>[Gr. <grk>tri-</grk> or L. <ets>tri-</ets>, sometimes through French; akin to L. <ets>tres</ets> three, and E. <ets>three</ets>. See <er>Three</er>.]</ety><br/
[<source>1913 Webster</source>]</p>

<p><sn>1.</sn> <def>A prefix meaning <xex>three</xex>, <xex>thrice</xex>, <xex>threefold</xex>; as in <ex>tri</ex>colored, <ex>tri</ex>dentate.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Chem.)</fld> <def>A prefix (also used adjectively) denoting <xex>three proportional or combining part</xex>, or <xex>the third degree</xex> of that to the name of which it is prefixed; as in <ex>tri</ex>sulphide, <ex>tri</ex>oxide, <ex>tri</ex>chloride.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Triable</ent><br/
<hw>Tri"a*ble</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[From <er>Try</er>.]</ety> <sn>1.</sn> <def>Fit or possible to be tried; liable to be subjected to trial or test.</def> <ldquo/Experiments <xex>triable</xex>.<rdquo/  <rj><au>Boyle.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Law)</fld> <def>Liable to undergo a judicial examination; properly coming under the cognizance of a court; <as>as, a cause may be <ex>triable</ex> before one court which is not <ex>triable</ex> in another</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Triableness</ent><br/
<hw>Tri"a*ble*ness</hw>, <pos>n.</pos> <def>Quality or state of being triable.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Triacid</ent><br/
<hw>Tri*ac"id</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[Pref. <ets>tri-</ets> + <ets>acid</ets>.]</ety> <fld>(Chem.)</fld> <def>Capable of neutralizing three molecules of a monobasic acid or the equivalent; having three hydrogen atoms which may be acid radicals; -- said of certain bases; thus, glycerin is a <ex>triacid</ex> base.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Triacle</ent><br/
<hw>Tri"a*cle</hw> <pr>(?)</pr>, <pos>n.</pos> <def>See <er>Treacle</er>.</def> <mark>[Obs.]</mark>  <rj><au>Chaucer.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Triacontahedral</ent><br/
<hw>Tri`a*con`ta*he"dral</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[Gr. <?/ thirty + <?/ seat, base.]</ety> <def>Having thirty sides.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Triaconter</ent><br/
<hw>Tri"a*con`ter</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Gr. <?/, fr. <?/ thirty.]</ety> <fld>(Gr. Antiq.)</fld> <def>A vessel with thirty banks of oars, or, as some say, thirty ranks of rowers.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Triad</ent><br/
<hw>Tri"ad</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>trias</ets>, <ets>-adis</ets>, Gr. <?/, <?/, fr.<?/, <?/, three: cf. F. <ets>triade</ets>. See <er>Three</er>, and cf. <er>Trias</er>, <er>Trio</er>.]</ety> <sn>1.</sn> <def>A union of three; three objects treated as one; a ternary; a trinity; <as>as, a <ex>triad</ex> of deities</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Mus.)</fld> <sd>(a)</sd> <def>A chord of three notes.</def> <sd>(b)</sd> <def>The common chord, consisting of a tone with its third and fifth, with or without the octave.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <fld>(Chem.)</fld> <def>An element or radical whose valence is three.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Triads of the Welsh bards</b></col>, <cd>poetical histories, in which the facts recorded are grouped by threes, three things or circumstances of a kind being mentioned together.</cd> -- <col><b>Hindu triad</b></col>. <cd>See <er>Trimurti</er>.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Triadelphous</ent><br/
<hw>Tri`a*del"phous</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[Pref. <ets>tri-</ets> + Gr. <?/ brother.]</ety> <fld>(Bot.)</fld> <def>Having stamens joined by filaments into three bundles. See <xex>Illust.</xex> under <er>Adelphous</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Triadic</ent><br/
<hw>Tri*ad"ic</hw> <pr>(?)</pr>, <pos>a.</pos> <fld>(Chem.)</fld> <def>Having the characteristics of a triad; <as>as, boron is <ex>triadic</ex></as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Triakisoctahedron</ent><br/
<hw>Tri"a*kis*oc`ta*he"dron</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Gr. <?/ thrice + E. <ets>octahedron</ets>.]</ety> <fld>(Crystalloq.)</fld> <def>A trigonal trisoctahedron.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Trial</ent><br/
<hw>Tri"al</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[From <er>Try</er>.]</ety> <sn>1.</sn> <def>The act of trying or testing in any manner.</def> Specifically: --<br/
[<source>1913 Webster</source>]</p>

<p><sd>(a)</sd> <def>Any effort or exertion of strength for the purpose of ascertaining what can be done or effected.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>[I] defy thee to the <qex>trial</qex> of mortal fight.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sd>(b)</sd> <def>The act of testing by experience; proof; test.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Repeated <qex>trials</qex> of the issues and events of actions.</q> <rj><qau>Bp. Wilkins.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sd>(c)</sd> <def>Examination by a test; experiment, as in chemistry, metallurgy, etc.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>The state of being tried or tempted; exposure to suffering that tests strength, patience, faith, or the like; affliction or temptation that exercises and proves the graces or virtues of men.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Others had <qex>trial</qex> of cruel mockings and scourgings.</q> <rj><qau>Heb. xi. 36.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>That which tries or afflicts; that which harasses; that which tries the character or principles; that which tempts to evil; <as>as, his child's conduct was a sore <ex>trial</ex></as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Every station is exposed to some <qex>trials</qex>.</q> <rj><qau>Rogers.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <fld>(Law)</fld> <def>The formal examination of the matter in issue in a cause before a competent tribunal; the mode of determining a question of fact in a court of law; the examination, in legal form, of the facts in issue in a cause pending before a competent tribunal, for the purpose of determining such issue.</def><br/
[<source>1913 Webster</source>]</p>

<p><syn><b>Syn.</b> -- Test; attempt; endeavor; effort; experiment; proof; essay. See <er>Test</er>, and <er>Attempt</er>.</syn><br/
[<source>1913 Webster</source>]</p>

<p><ent>trial and error</ent><br/
<hw>tri"al and er"ror</hw> <def>The method of discovering something desirable by trying many different things until one works; <as>as, the traditional method of finding new drugs is by <ex>trial and error</ex></as>.</def><br/
[<source>PJC</source>]</p>

<p><q>And millions miss for one that hits.</q> <rj><qau>Swift.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Trial balance</ent><br/
<hw>Tri"al bal`ance</hw>. <fld>(Bookkeeping)</fld> <def>The testing of a ledger to discover whether the debits and credits balance, by finding whether the sum of the personal credits increased by the difference between the debit and credit sums in the merchandise and other impersonal accounts equals the sum of personal debits. The equality would not show that the items were all correctly posted.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Triality</ent><br/
<hw>Tri*al"i*ty</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>tres</ets>, <ets>tria</ets>, three.]</ety> <def>Three united; state of being three.</def> <mark>[R.]</mark>  <rj><au>H. Wharton.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Trialogue</ent><br/
<hw>Tri"a*logue</hw>, <pos>n.</pos> <ety>[LL. <ets>trialogus</ets>; <ets>tri-</ets> (see <er>Tri-</er>) + <ets>-logus</ets> as, in L. <ets>dialogus</ets>, E. <ets>dialogue</ets>.]</ety> <def>A discourse or colloquy by three persons.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Triamide</ent><br/
<hw>Tri*am"ide</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[<ets>tri-</ets> + <ets>amine</ets>.]</ety> <fld>(Chem.)</fld> <def>An amide containing three amido groups.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Triamine</ent><br/
<hw>Tri*am"ine</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Pref. <ets>tri-</ets> + <ets>amine</ets>.]</ety> <fld>(Chem.)</fld> <def>An amine containing three amido groups.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Triander</ent><br/
<hw>Tri"an`der</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Bot.)</fld> <def>Any one of the Triandria.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Triandria</ent><br/
||<hw>Tri*an"dri*a</hw> <pr>(?)</pr>, <pos>n. pl.</pos> <ety>[NL. See <er>Tri-</er>, and <er>-androus</er>.]</ety> <fld>(Bot.)</fld> <def>A Linnaean class of plants having three distinct and equal stamens.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Triandrous</ent><br/
<ent>Triandrian</ent><br/
<mhw>{ <hw>Tri*an"dri*an</hw> <pr>(?)</pr>, <hw>Tri*an"drous</hw> <pr>(?)</pr>, }</mhw> <pos>a.</pos> <ety>[Cf. F. <ets>triandre</ets>.]</ety> <fld>(Bot.)</fld> <def>Of or pertaining to the Triandria; having three distinct and equal stamens in the same flower.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Triangle</ent><br/
<hw>Tri"an`gle</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>triangulum</ets>, fr. <ets>triangulus</ets> triangular; <ets>tri-</ets> (see <er>Tri-</er>) + <ets>angulus</ets> angle: cf. F. <ets>triangle</ets>. See <er>Angle</er> a corner.]</ety> <sn>1.</sn> <fld>(Geom.)</fld> <def>A figure bounded by three lines, and containing three angles.</def><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ A triangle is either <xex>plane</xex>, <xex>spherical</xex>, <xex>or curvilinear</xex>, according as its sides are straight lines, or arcs of great circles of a sphere, or any curved lines whatever. A plane triangle is designated as <xex>scalene</xex>, <xex>isosceles</xex>, <xex>or equilateral</xex>, according as it has no two sides equal, two sides equal, or all sides equal; and also as <xex>right-angled</xex>, or <xex>oblique-angled</xex>, according as it has one right angle, or none; and oblique-angled triangle is either <xex>acute-angled</xex>, or <xex>obtuse-angled</xex>, according as all the angles are acute, or one of them obtuse. The terms <xex>scalene</xex>, <xex>isosceles</xex>, <xex>equilateral</xex>, <xex>right-angled</xex>, <xex>acute-angled</xex>, and <xex>obtuse-angled</xex>, are applied to spherical triangles in the same sense as to plane triangles.</note><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Mus.)</fld> <def>An instrument of percussion, usually made of a rod of steel, bent into the form of a triangle, open at one angle, and sounded by being struck with a small metallic rod.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>A draughtsman's square in the form of a right-angled triangle.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <fld>(Mus.)</fld> <def>A kind of frame formed of three poles stuck in the ground and united at the top, to which soldiers were bound when undergoing corporal punishment, -- now disused.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <fld>(Astron.)</fld> <sd>(a)</sd> <def>A small constellation situated between Aries and Andromeda.</def> <sd>(b)</sd> <def>A small constellation near the South Pole, containing three bright stars.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Triangle spider</b></col> <fld>(Zool.)</fld>, <cd>a small American spider (<spn>Hyptiotes Americanus</spn>) of the family <fam>Ciniflonidae</fam>, living among the dead branches of evergreen trees. It constructs a triangular web, or net, usually composed of four radii crossed by a double elastic fiber. The spider holds the thread at the apex of the web and stretches it tight, but lets go and springs the net when an insect comes in contact with it.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Triangled</ent><br/
<hw>Tri"an`gled</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Having three angles; triangular.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Triangular</ent><br/
<hw>Tri*an"gu*lar</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>triangularis</ets>: cf. F. <ets>triangulaire</ets>.]</ety> <sn>1.</sn> <def>Having three angles; having the form of a triangle.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Bot.)</fld> <def>Oblong or elongated, and having three lateral angles; <as>as, a <ex>triangular</ex> seed, leaf, or stem</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Triangular compasses</b></col>, <cd>compasses with three legs for taking off the angular points of a triangle, or any three points at the same time.</cd> -- <col><b>Triangular crab</b></col> <fld>(Zool.)</fld>, <cd>any maioid crab; -- so called because the carapace is usually triangular.</cd> -- <col><b>Triangular numbers</b></col> <fld>(Math.)</fld>, <cd>the series of numbers formed by the successive sums of the terms of an arithmetical progression, of which the first term and the common difference are 1. See <cref>Figurate numbers</cref>, under <er>Figurate</er>.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Triangulares</ent><br/
||<hw>Tri*an`gu*la"res</hw> <pr>(?)</pr>, <pos>n. pl.</pos> <ety>[L.]</ety> <fld>(Zool.)</fld> <def>The triangular, or maioid, crabs. See <xex>Illust.</xex> under <er>Maioid</er>, and <xex>Illust.</xex> of <cref>Spider crab</cref>, under <er>Spider</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Triangularity</ent><br/
<hw>Tri*an`gu*lar"i*ty</hw> <pr>(?)</pr>, <pos>n.</pos> <def>The quality or state of being triangular.</def>  <rj><au>Bolingbroke.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Triangularly</ent><br/
<hw>Tri*an"gu*lar*ly</hw> <pr>(?)</pr>, <pos>adv.</pos> <def>In a triangular manner; in the form of a triangle.</def>  <rj><au>Dampier.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Triangulate</ent><br/
<hw>Tri*an"gu*late</hw> <pr>(?)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Triangulated</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Triangulating</conjf>.]</vmorph> <sn>1.</sn> <def>To divide into triangles; specifically, to survey by means of a series of triangles properly laid down and measured.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To make triangular, or three-cornered.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Triangulation</ent><br/
<hw>Tri*an`gu*la"tion</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Cf. F. <ets>triangulation</ets>.]</ety> <fld>(Surv.)</fld> <def>The series or network of triangles into which the face of a country, or any portion of it, is divided in a trigonometrical survey; the operation of measuring the elements necessary to determine the triangles into which the country to be surveyed is supposed to be divided, and thus to fix the positions and distances of the several points connected by them.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Triarchy</ent><br/
<hw>Tri"ar*chy</hw> <pr>(?)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Triarchies</plw> <pr>(#)</pr>.</plu> <ety>[Gr. <?/; <?/  (see <er>Tri-</er>) + <?/ to rule.]</ety> <def>Government by three persons; a triumvirate; also, a country under three rulers.</def>  <rj><au>Holland.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Triarian</ent><br/
<hw>Tri*a"ri*an</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>triarii</ets>, <ets>pl</ets>., a class of Roman soldiers who formed the third rank from the front, fr. <ets>tres</ets>, <ets>tria</ets>, three.]</ety> <def>Occupying the third post or rank.</def> <mark>[Obs.]</mark>  <rj><au>Cowley.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Triarticulate</ent><br/
<hw>Tri`ar*tic"u*late</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[Pref. <ets>tri-</ets> + <ets> articulate</ets>.]</ety> <fld>(Zool.)</fld> <def>Having three joints.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Trias</ent><br/
<hw>Tri"as</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L., triad. See <er>Triad</er>.]</ety> <fld>(Geol.)</fld> <def>The formation situated between the Permian and Lias, and so named by the Germans, because consisting of three series of strata, which are called in German the <xex>Bunter sandstein</xex>, <xex>Muschelkalk</xex>, and <xex>Keuper</xex>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Triassic</ent><br/
<hw>Tri*as"sic</hw> <pr>(?)</pr>, <pos>a.</pos> <fld>(Geol.)</fld> <def>Of the age of, or pertaining to, the <xex>Trias</xex>.</def> -- <def2><pos>n.</pos>  <def>The Triassic formation.</def></def2><br/
[<source>1913 Webster</source>]</p>

<p><ent>Triatic</ent><br/
<hw>Tri*at"ic</hw> <pr>(?)</pr>, <pos>a.</pos> <fld>(Naut.)</fld> <def>A term used in the phrase <cref>triatic stay</cref>. See under <er>Stay</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Triatomic</ent><br/
<hw>Tri`a*tom"ic</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[Pref. <ets>tri-</ets> + <ets>atomic</ets>.]</ety> <fld>(Chem.)</fld> <sd>(a)</sd> <def>Having three atoms; -- said of certain elements or radicals.</def> <sd>(b)</sd> <def>Having a valence of three; trivalent; sometimes, in a specific sense, having three hydroxyl groups, whether acid or basic; <as>thus, glycerin, glyceric acid, and tartronic acid are each <ex>triatomic</ex></as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tribal</ent><br/
<hw>Trib"al</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Of or pertaining to a tribe or tribes; <as>as, a <ex>tribal</ex> scepter</as>.</def>  <rj><au>Bp. Warburton.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tribalism</ent><br/
<hw>Trib"al*ism</hw> <pr>(?)</pr>, <pos>n.</pos> <def>The state of existing in tribes; also, tribal feeling; tribal prejudice or exclusiveness; tribal peculiarities or characteristics.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tribasic</ent><br/
<hw>Tri*ba"sic</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[Pref. <ets>tri-</ets> + <ets>basic</ets>.]</ety> <fld>(Chem.)</fld> <def>Capable of neutralizing three molecules of a monacid base, or their equivalent; having three hydrogen atoms capable of replacement by basic elements on radicals; -- said of certain acids; thus, citric acid is a <xex>tribasic</xex> acid.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tribble</ent><br/
<hw>Trib"ble</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Paper Manuf.)</fld> <def>A frame on which paper is dried.</def>  <rj><au>Knight.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tribe</ent><br/
<hw>Tribe</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>tribus</ets>, originally, a third part of the Roman people, afterwards, a division of the people, a tribe; of uncertain origin: cf. F. <ets>tribu</ets>.]</ety><br/
[<source>1913 Webster</source>]</p>

<p><sn>1.</sn> <def>A family, race, or series of generations, descending from the same progenitor, and kept distinct, as in the case of the twelve tribes of Israel, descended from the twelve sons of Jacob.</def> <ldquo/The Lion of the <xex>tribe</xex> of Juda.<rdquo/  <rj><au>Rev. v. 5.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>A wealthy Hebrew of my <qex>tribe</qex>.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Bot.)</fld> <def>A number of species or genera having certain structural characteristics in common; <as>as, a <ex>tribe</ex> of plants; a <ex>tribe</ex> of animals</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ By many recent naturalists, <xex>tribe</xex> has been used for a group of animals or plants intermediate between <xex>order</xex> and <xex>genus</xex>.</note><-- in modern taxonomy, the <altname>family</altname> is between order and genus. --><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>A nation of savages or uncivilized people; a body of rude people united under one leader or government; <as>as, the <ex>tribes</ex> of the Six Nations; the Seneca <ex>tribe</ex></as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>A division, class, or distinct portion of a people, from whatever cause that distinction may have originated; <as>as, the city of Athens was divided into ten <ex>tribes</ex></as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <fld>(Stock Breeding)</fld> <def>A family of animals descended from some particular female progenitor, through the female line; <as>as, the Duchess <ex>tribe</ex> of shorthorns</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tribe</ent><br/
<hw>Tribe</hw>, <pos>v. t.</pos> <def>To distribute into tribes or classes.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>Our fowl, fish, and quadruped are well <qex>tribed</qex>.</q> <rj><qau>Abp. Nicolson.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tribolet</ent><br/
<ent>Triblet</ent><br/
<mhw>{ <hw>Trib"let</hw> <pr>(?)</pr>, <hw>Trib"o*let</hw> <pr>(?)</pr> }</mhw>, <pos>n.</pos>} <ety>[F. <ets>triboulet</ets>.]</ety> <sn>1.</sn> <def>A goldsmith's tool used in making rings.</def>  <rj><au>Ainsworth.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A steel cylinder round which metal is drawn in the process of forming tubes.</def>  <rj><au>Tomlinson.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <fld>(Blacksmithing)</fld> <def>A tapering mandrel.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>tribo-</ent><br/
<hw>trib"o-</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Gr. <grk>tri`bein</grk> to rub.]</ety> <def>a combining form relating to friction.</def><br/
[<source>PJC</source>]</p>

<p><ent>triboelectricity</ent><br/
<hw>trib`o*e*lec*tric"i*ty</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[<ets>tribo-</ets> + <ets>electricity</ets>.]</ety> <def>electrical charge developed by rubbing objects together, suc as amber on cat fur.</def><br/
[<source>PJC</source>]</p>

<p><ent>tribology</ent><br/
<hw>tri*bol"o*gy</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[<ets>tribo-</ets> + <ets>-ology</ets>.]</ety> <def>the study of the effects of friction on parts of machinery moving in contact with each other, and of methods, such as lubrication, to counteract negative effects such as wear.</def><br/
[<source>PJC</source>]</p>

<p><ent>Tribometer</ent><br/
<hw>Tri*bom"e*ter</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Gr. <grk>tri`bein</grk> to rub + <ets>-meter</ets>: cf. F. <ets>tribom<egrave/tre</ets>.]</ety> <def>An instrument to ascertain the degree of friction in rubbing surfaces.</def>  <rj><au>Brande & C.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tribrach</ent><br/
<hw>Tri"brach</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>tribrachys</ets>, Gr. <?/ consisting of three short syllables; <?/ (see <er>Tri-</er>) + <?/ short.]</ety> <fld>(Gr. & L. Pros.)</fld> <def>A poetic foot of three short syllables, <as>as, <ex>m<ecir/l<icir/<ucir/s</ex></as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tribracteate</ent><br/
<hw>Tri*brac"te*ate</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[Pref. <ets>tri-</ets> + <ets>bracteate</ets>.]</ety> <fld>(Bot.)</fld> <def>Having three bracts.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tribromphenol</ent><br/
<ent>Tribromophenol</ent><br/
<mhw>{ <hw>Tri*bro`mo*phe"nol</hw> <pr>(?)</pr>, <hw>Tri`brom*phe"nol</hw> <pr>(?)</pr> }</mhw>, <pos>n.</pos> <fld>(Chem.)</fld> <def>A colorless crystalline substance (<chform>C6H3OBr3</chform>) prepared by the reaction of carbolic acid with bromine.  The predominant isomer is 2,4,6-tribromophenol; -- called also <altname>bromol</altname>.</def><br/
[<source>Webster 1913 Suppl.</source> <source>+PJC</source>]</p>

<p><ent>Tribular</ent><br/
<ent>Tribual</ent><br/
<mhw>{ <hw>Trib"u*al</hw> <pr>(?)</pr>, <hw>Trib"u*lar</hw> <pr>(?)</pr>, }</mhw> <pos>a.</pos> <def>Of or relating to a tribe; tribal; <as>as, a <ex>tribual</ex> characteristic; <ex>tribular</ex> worship</as>.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>The <qex>tribual</qex> lispings of the Ephraimites.</q> <rj><qau>Fuller.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tribulation</ent><br/
<hw>Trib`u*la"tion</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[OE. <ets>tribulacium</ets>, F. <ets>tribulation</ets>, L. <ets>tribulatio</ets>, from <ets>tribulare</ets> to press, afflict, fr. <ets>tribulum</ets> a thrashing sledge, akin to <ets>terere</ets>, <ets>tritum</ets>, to rub. See <er>Trite</er>.]</ety> <def>That which occasions distress, trouble, or vexation; severe affliction.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>When <qex>tribulation</qex> or persecution ariseth because of the word, by and by he is offended.</q> <rj><qau>Matt. xiii. 21.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>In the world ye shall have <qex>tribulation</qex>.</q> <rj><qau>John. xvi. 33.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tribunal</ent><br/
<hw>Tri*bu"nal</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>tribunal</ets>, fr. <ets>tribunus</ets> a tribune who administered justice: cf. F. <ets>tribunal</ets>. See <er>Tribune</er>.]</ety> <sn>1.</sn> <def>The seat of a judge; the bench on which a judge and his associates sit for administering justice.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Hence, a court or forum; <as>as, the House of Lords, in England, is the highest <ex>tribunal</ex> in the kingdom</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tribunal</ent><br/
||<hw>Tri`bu*nal"</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Sp.]</ety> <def>In villages of the Philippine Islands, a kind of townhall. At the  tribunal the head men of the village met to transact business, prisoners were confined, and troops and travelers were often quartered.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Tribunary</ent><br/
<hw>Trib"u*na*ry</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Of or pertaining to tribunes; <as>as, <ex>tribunary</ex> powers or authority</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tribunate</ent><br/
<hw>Trib"u*nate</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>tribunatus</ets>: cf. F. <ets>tribunat</ets>.]</ety> <def>The state or office of a tribune; tribuneship.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tribune</ent><br/
<hw>Trib"une</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>tribunus</ets>, properly, the chief of a tribe, fr. <ets>tribus</ets> tribe: cf. F. <ets>tribun</ets>. See <er>Tribe</er>.]</ety> <sn>1.</sn> <fld>(Rom. Antiq.)</fld> <def>An officer or magistrate chosen by the people, to protect them from the oppression of the patricians, or nobles, and to defend their liberties against any attempts that might be made upon them by the senate and consuls.</def><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ The <xex>tribunes</xex> were at first two, but their number was increased ultimately to ten. There were also military tribunes, officers of the army, of whom there were from four to six in each legion. Other officers were also called tribunes; as, <xex>tribunes</xex> of the treasury, etc.</note><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Anciently, a bench or elevated place, from which speeches were delivered; in France, a kind of pulpit in the hall of the legislative assembly, where a member stands while making an address; any place occupied by a public orator.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tribuneship</ent><br/
<hw>Trib"une*ship</hw>, <pos>n.</pos> <def>The office or power of a tribune.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tribunitian</ent><br/
<ent>Tribunitial</ent><br/
<ent>Tribunician</ent><br/
<mhw>{ <hw>Trib`u*ni"cian</hw> <pr>(?)</pr>, <hw>Trib`u*ni"tial</hw> <pr>(?)</pr>, <hw>Trib`u*ni*tian</hw> <pr>(?)</pr>, }</mhw> <pos>a.</pos> <ety>[L. <ets>tribunicius</ets>, <ets>tribunitius</ets>: cf. F. <ets>tribunitien</ets>.]</ety> <def>Of or pertaining to tribunes; befitting a tribune; <as>as, <ex>tribunitial</ex> power or authority</as>.</def>  <rj><au>Dryden.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>A kind of <qex>tribunician</qex> veto, forbidding that which is recognized to be wrong.</q> <rj><qau>Hare.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tribunitious</ent><br/
<hw>Trib`u*ni"tious</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Tribunician; tribunitial.</def> <mark>[Obs.]</mark>  <rj><au>Bacon.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tributarily</ent><br/
<hw>Trib"u*ta*ri*ly</hw> <pr>(?)</pr>, <pos>adv.</pos> <def>In a tributary manner.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tributariness</ent><br/
<hw>Trib"u*ta*ri*ness</hw>, <pos>n.</pos> <def>The quality or state of being tributary.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tributary</ent><br/
<hw>Trib"u*ta*ry</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[OE. <ets>tributaire</ets>, F. <ets>tributaire</ets>, L.  <ets>tributarius</ets>. See <er>Tribute</er>.]</ety> <sn>1.</sn> <def>Paying tribute to another, either from compulsion, as an acknowledgment of submission, or to secure protection, or for the purpose of purchasing peace.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>[Julius] unto Rome made them <qex>tributary</qex>.</q> <rj><qau>Chaucer.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Hence, subject; subordinate; inferior.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>He to grace his <qex>tributary</qex> gods.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>Paid in tribute.</def> <ldquo/<xex>Tributary</xex> tears.<rdquo/  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>Yielding supplies of any kind; serving to form or make up, a greater object of the same kind, as a part, branch, etc.; contributing; <as>as, the Ohio has many <ex>tributary</ex> streams, and is itself <ex>tributary</ex> to the Mississippi</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><-- p. 1537 --><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tributary</ent><br/
<hw>Trib"u*ta*ry</hw> <pr>(?)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Tributaries</plw> <pr>(<?/)</pr>.</plu> <sn>1.</sn> <def>A ruler or state that pays tribute, or a stated sum, to a conquering power, for the purpose of securing peace and protection, or as an acknowledgment of submission, or for the purchase of security.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A stream or river flowing into a larger river or into a lake; an affluent.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tribute</ent><br/
<hw>Trib"ute</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[OE. <ets>tribut</ets>, L. <ets>tributum</ets>, fr. <ets>tribuere</ets>, <ets>tributum</ets>, to bestow, grant, pay, allot, assign, originally, to a tribe, from <ets>tribus</ets> tribe; cf. F. <ets>tribut</ets>. See <er>Tribe</er>, and cf. <er>Attribute</er>, <er>Contribute</er>.]</ety> <sn>1.</sn> <def>An annual or stated sum of money or other valuable thing, paid by one ruler or nation to another, either as an acknowledgment of submission, or as the price of peace and protection, or by virtue of some treaty; <as>as, the Romans made their conquered countries pay <ex>tribute</ex></as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Millions for defense, but not one cent for <qex>tribute</qex>.</q> <rj><qau>C. C. Pinckney.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A personal contribution, as of money, praise, service, etc., made in token of services rendered, or as that which is due or deserved; <as>as, a <ex>tribute</ex> of affection</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Implores the passing <qex>tribute</qex> of a sigh.</q> <rj><qau>Gray.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <fld>(Mining)</fld> <def>A certain proportion of the ore raised, or of its value, given to the miner as his recompense.</def>  <rj><au>Pryce. Tomlinson.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Tribute money</b></col>, <cd>money paid as a tribute or tax.</cd> -- <col><b>Tribute pitch</b></col>. <fld>(Mining)</fld> <cd>See under <er>Tributer</er>.</cd> <mark>[Eng.]</mark></cs><br/
[<source>1913 Webster</source>]</p>

<p><syn><b>Syn.</b> -- See <er>Subsidy</er>.</syn><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tribute</ent><br/
<hw>Trib"ute</hw>, <pos>v. i.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Tributed</conjf>; <pos>p. pr. & vb. n.</pos> <conjf>Tributing</conjf>.]</vmorph> <def>To pay as tribute.</def> <mark>[R.]</mark>  <rj><au>Whitlock (1654).</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tributer</ent><br/
<hw>Trib"u*ter</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Mining)</fld> <def>One who works for a certain portion of the ore, or its value.</def> <mark>[Eng.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ <xex>Tributers</xex> generally work in gangs, and have a limited portion of a lode set them, called a <xex>tribute pitch</xex>, beyond which they are not permitted to work, and for which they receive a certain portion of the ore, or so much per pound, as agreed upon, of the value of what they raise.</note>  <rj><au>Weale.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Trica</ent><br/
||<hw>Tri"ca</hw> <pr>(?)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Tricae</plw> <pr>(#)</pr>.</plu> <ety>[NL.]</ety> <fld>(Bot.)</fld> <def>An apothecium in certain lichens, having a spherical surface marked with spiral or concentric ridges and furrows.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tricarballylic</ent><br/
<hw>Tri*car`bal*lyl"ic</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[Pref. <ets>tri-</ets> + <ets>carb</ets>oxyl + <ets>allyl</ets> + <ets>-ic</ets>.]</ety> <fld>(Chem.)</fld> <def>Of, pertaining to, or designating, a complex tribasic organic acid, <chform>C3H5.(CO2H)3</chform> occurring naturally in unripe beet roots, and produced artificially from glycerin as a white crystalline substance.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tricarbimide</ent><br/
<hw>Tri*car"bi*mide</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Pref. <ets>tri-</ets> + <ets>carbimide</ets>.]</ety> <fld>(Chem.)</fld> <def>See under <er>Cyanuric</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Trice</ent><br/
<hw>Trice</hw> <pr>(?)</pr>, <pos>v. t.</pos> <ety>[OE. <ets>trisen</ets>; of Scand. or Low German origin; cf. Sw. <ets>trissa</ets> a sheave, pulley, <ets>triss</ets> a spritsail brace, Dan. <ets>tridse</ets> a pulley, <ets>tridse</ets> to haul by means of a pulley, to trice, LG. <ets>trisse</ets> a pulley, D. <ets>trijsen</ets> to hoist.]</ety> <altsp>[Written also <asp>trise</asp>.]</altsp> <sn>1.</sn> <def>To pull; to haul; to drag; to pull away.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q> Out of his seat I will him <qex>trice</qex>.</q> <rj><qau>Chaucer.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Naut.)</fld> <def>To haul and tie up by means of a rope.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Trice</ent><br/
<hw>Trice</hw>, <pos>n.</pos> <ety>[Sp. <ets>tris</ets> the noise made by the breaking of glass, an instant, <ets>en un tris</ets> in an instant; probably of imitative origin.]</ety> <def>A very short time; an instant; a moment; -- now used only in the phrase <xex>in a trice</xex>.</def> <ldquo/With a <xex>trice</xex>.<rdquo/ <au>Turbervile.</au> <ldquo/ On a <xex>trice</xex>.<rdquo/  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>A man shall make his fortune in a <qex>trice</qex>.</q> <rj><qau>Young.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tricennarious</ent><br/
<hw>Tri`cen*na"ri*ous</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Of or pertaining to thirty years; tricennial.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tricennial</ent><br/
<hw>Tri*cen"ni*al</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>tricennium</ets> thirty years; <ets>triginta</ets> thirty + <ets>annus</ets> year: cf. L. <ets>tricennalis</ets>.]</ety> <def>Of or pertaining to thirty years; consisting of thirty years; occurring once in every thirty years.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tricentenary</ent><br/
<hw>Tri*cen"te*na*ry</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[Pref. <ets>tri-</ets> + <ets>centenary</ets>.]</ety> <def>Including, or relating to, the interval of three hundred years; tercentenary.</def> -- <def2><pos>n.</pos> <def>A period of three centuries, or three hundred years, also, the three-hundredth anniversary of any event; a tercentenary.</def></def2><br/
[<source>1913 Webster</source>]</p>

<p><ent>Triceps</ent><br/
||<hw>Tri"ceps</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[NL., fr. L. <ets>triceps</ets>, having three beads; <ets>tres</ets>, <ets>tria</ets>, three + <ets>caput</ets> head: cf. F. <ets>triceps</ets>. See <er>Three</er>, and <ets>Chief</ets>.]</ety> <fld>(Anat.)</fld> <def>A muscle having three heads; specif., the great extensor of the forearm, arising by three heads and inserted into the olecranon at the elbow.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Trichiasis</ent><br/
||<hw>Tri*chi"a*sis</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[NL., fr. Gr. <?/, fr. <grk>tri`x</grk>, <grk>tricho`s</grk>, hair.]</ety> <fld>(Med.)</fld> <def>A disease of the eye, in which the eyelashes, being turned in upon the eyeball, produce constant irritation by the motion of the lids.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Trichina</ent><br/
||<hw>Tri*chi"na</hw> <pr>(-n<adot/)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Trichinae</plw> <pr>(#)</pr>.</plu> <ety>[NL., fr. Gr. <?/ hairy, made of hair, fr. <grk>tri`x</grk>, <grk>tricho`s</grk>, hair.]</ety> <fld>(Zool.)</fld> <def>A small, slender nematoid worm (<spn>Trichina spiralis</spn>) which, in the larval state, is parasitic, often in immense numbers, in the voluntary muscles of man, the hog, and many other animals. When insufficiently cooked meat containing the larvae is swallowed by man, they are liberated and rapidly become adult, pair, and the ovoviviparous females produce in a short time large numbers of young which find their way into the muscles, either directly, or indirectly by means of the blood. Their presence in the muscles and the intestines in large numbers produces trichinosis.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Trichiniasis</ent><br/
||<hw>Trich`i*ni"a*sis</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[NL.]</ety> <fld>(Med.)</fld> <def>Trichinosis.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Trichinize</ent><br/
<hw>Trich"i*nize</hw> <pr>(?)</pr>, <pos>v. t.</pos> <def>To render trichinous; to affect with trichinae; -- chiefly used in the past participle; <as>as, <ex>trichinized</ex> pork</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Trichinoscope</ent><br/
<hw>Tri*chi"no*scope</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[<ets>Trichina</ets> + <ets>-scope</ets>.]</ety> <def>An apparatus for the detection of trichinae in the flesh of animals, as of swine.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Trichinosis</ent><br/
||<hw>Trich`i*no"sis</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[NL. See <er>Trichina</er>.]</ety> <fld>(Med.)</fld> <def>The disease produced by the presence of trichinae in the muscles and intestinal track. It is marked by fever, muscular pains, and symptoms resembling those of typhoid fever, and is frequently fatal.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Trichinous</ent><br/
<hw>Trich"i*nous</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Of or pertaining to trichinae or trichinosis; affected with, or containing, trichinae; <as>as, <ex>trichinous</ex> meat</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Trichite</ent><br/
<hw>Trich"ite</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Gr. <?/, <?/, hair.]</ety><br/
[<source>1913 Webster</source>]</p>

<p><sn>1.</sn> <fld>(Min.)</fld> <def>A kind of crystallite resembling a bunch of hairs, common in obsidian. See <xex>Illust.</xex> of <er>Crystallite</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Zool.)</fld> <def>A delicate, hairlike siliceous spicule, found in certain sponges.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Trichite sheaf</b></col> <fld>(Zool.)</fld>, <cd>one of the small sheaflike fascicles of slender setae characteristic of certain sponges. See <xex>Illust.</xex> under <er>Spicule</er>.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Trichiuriform</ent><br/
<hw>Trich`i*u"ri*form</hw> <pr>(?)</pr>, <pos>a.</pos> <fld>(Zool.)</fld> <def>Like or pertaining to the genus Trichiurus or family <fam>Trichiuridae</fam>, comprising the scabbard fishes and hairtails.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Trichiuroid</ent><br/
<hw>Trich`i*u"roid</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[<ets>Trichiurus</ets> + <ets>-oid</ets>.]</ety> <fld>(Zool.)</fld> <def>Of, like, or pertaining to, Trichiurus.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Trichiurus</ent><br/
||<hw>Trich`i*u"rus</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[NL., fr. Gr. <grk>tri`x</grk>, <grk>tricho`s</grk>, a hair + <grk>o'yra`</grk> tail.]</ety> <fld>(Zool.)</fld> <def>A genus of fishes comprising the hairtails. See <er>Hairtail</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Trichloride</ent><br/
<hw>Tri*chlo"ride</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Pref. <ets>tri-</ets> + <ets>chloride</ets>.]</ety> <fld>(Chem.)</fld> <def>A chloride having three atoms of chlorine in the molecule.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Trichobranchia</ent><br/
||<hw>Trich`o*bran"chi*a</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[NL., fr. Gr. <grk>tri`x</grk>, <grk>tricho`s</grk>, hair + <ets>branchia</ets>.]</ety> <fld>(Zool.)</fld> <def>The gill of a crustacean in which the branchial filaments are slender and cylindrical, as in the crawfishes.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Trichocyst</ent><br/
<hw>Trich"o*cyst</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Gr. <grk>tri`x</grk>, <grk>tricho`s</grk>, a hair + <?/ bag.]</ety> <fld>(Zool.)</fld> <def>A lasso cell.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Trichogyne</ent><br/
<hw>Trich"o*gyne</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F., fr. Gr. <grk>tri`x</grk>, <grk>tricho`s</grk>, hair + <?/ woman, female.]</ety> <fld>(Bot.)</fld> <def>The slender, hairlike cell which receives the fertilizing particles, or antherozoids, in red seaweeds.</def> -- <wordforms><wf>Trich`o*gyn"ic</wf> <pr>(#)</pr>, <pos>a.</pos></wordforms><br/
[<source>1913 Webster</source>]</p>

<p><ent>Trichomanes</ent><br/
<hw>Tri*chom"a*nes</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L., a kind of plant, from Gr. <?/.]</ety> <fld>(Bot.)</fld> <def>Any fern of the genus <gen>Trichomanes</gen>. The fronds are very delicate and often translucent, and the sporangia are borne on threadlike receptacles rising from the middle of cup-shaped marginal involucres. Several species are common in conservatories; two are native in the United States.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Trichomatose</ent><br/
<hw>Tri*chom"a*tose`</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[Gr. <?/, <?/, a growth of hair.]</ety> <fld>(Med.)</fld> <def>Affected with a disease which causes agglutination and matting together; -- said of the hair when affected with plica. See <er>Plica</er>, 1.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Trichome</ent><br/
<hw>Trich"ome</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[See <er>Trichomatose</er>.]</ety> <fld>(Bot.)</fld> <def>A hair on the surface of leaf or stem, or any modification of a hair, as a minute scale, or star, or gland. The sporangia of ferns are believed to be of the nature of trichomes.</def> -- <wordforms><wf>Tri*chom"a*tous</wf> <pr>(#)</pr>, <pos>a.</pos></wordforms><br/
[<source>1913 Webster</source>]</p>

<p><ent>Trichophore</ent><br/
<hw>Trich"o*phore</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Gr. <grk>tri`x</grk>, <grk>tricho`s</grk>, hair + <grk>fe`rein</grk> to bear.]</ety> <sn>1.</sn> <fld>(Bot.)</fld> <def>The special cell in red algae which produces or bears a trichogyne. See <xex>Illust.</xex> of <er>Trichogyne</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Zool.)</fld> <def>One of the saclike organs from which the setae of annelids arise.</def><br/
[<source>1913 Webster</source>]</p>

<p>-- <wordforms><wf>Trich`o*phor"ic</wf> <pr>(#)</pr>, <pos>a.</pos></wordforms><br/
[<source>1913 Webster</source>]</p>

<p><ent>Trichopter</ent><br/
<hw>Tri*chop"ter</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Zool.)</fld> <def>One of the Trichoptera.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Trichoptera</ent><br/
||<hw>Tri*chop"te*ra</hw> <pr>(?)</pr>, <pos>n. pl.</pos> <ety>[NL., fr. Gr. <grk>tri`x</grk>, <grk>tricho`s</grk>, a hair + <?/ wing.]</ety> <fld>(Zool.)</fld> <def>A suborder of Neuroptera usually having the wings covered with minute hairs. It comprises the caddice flies, and is considered by some to be a distinct order.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Trichopteran</ent><br/
<hw>Tri*chop"ter*an</hw> <pr>(?)</pr>, <fld>(Zool.)</fld> <def>One of the Trichoptera.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Trichopterous</ent><br/
<hw>Tri*chop"ter*ous</hw> <pr>(?)</pr>, <pos>a.</pos> <fld>(Zool.)</fld> <def>Of, pertaining to, or characterizing, the Trichoptera.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Trichord</ent><br/
<hw>Tri"chord</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Gr. <?/ three stringed; <?/ (see <er>Tri-</er>) + <?/ chord, or string.]</ety> <fld>(Mus.)</fld> <def>An instrument, as a lyre or harp, having three strings.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Trichoscolices</ent><br/
||<hw>Trich`o*scol"i*ces</hw> <pr>(?)</pr>, <pos>n. pl.</pos> <ety>[NL., from Gr. <grk>tri`x</grk>, <grk>tricho`s</grk>, hair + <grk>skw`lhx</grk> a worm.]</ety> <fld>(Zool.)</fld> <def>An extensive group of wormlike animals characterized by being more or less covered with cilia.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Trichotomous</ent><br/
<hw>Tri*chot"o*mous</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[See <er>Trichotomy</er>.]</ety> <def>Divided into three parts, or into threes; three-forked; <as>as, a <ex>trichotomous</ex> stem</as>.</def>  <rj><au>Martyn.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Trichotomy</ent><br/
<hw>Tri*chot"o*my</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Gr. <grk>tri`cha</grk> threefold, in three parts + <grk>te`mnein</grk> to cut or divide: cf. F. <ets>trichotomie</ets>.]</ety> <def>Division into three parts.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Trichroic</ent><br/
<hw>Tri*chro"ic</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Exhibiting trichroism; pleochroic; pleochroism.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Trichroism</ent><br/
<hw>Tri"chro*ism</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Pref. <ets>tri-</ets> + Gr. <?/ color: cf. F. <ets>trichro<ium/sme</ets>.]</ety> <fld>(Min.)</fld> <def>The quality possessed by some crystals of presenting different colors in three different directions.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Trichromatic</ent><br/
<hw>Tri`chro*mat"ic</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[Pref. <ets>tri-</ets> + <ets>chromatic</ets>.]</ety> <fld>(Zool.)</fld> <def>Having or existing in three different phases of color; having three distinct color varieties; -- said of certain birds and insects.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Trichromatism</ent><br/
<hw>Tri*chro"ma*tism</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Zool.)</fld> <def>The quality, state, or phenomenon of being trichromatic.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Trichromic</ent><br/
<hw>Tri*chro"mic</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[Pref. <ets>tri-</ets> + Gr. <?/ color.]</ety> <fld>(Opt.)</fld> <def>Of, pertaining to, or consisting of, three colors or color sensations.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Trichromic</ent><br/
<hw>Tri*chro"mic</hw>, <pos>a.</pos> <ety>[Pref. <ets>tri-</ets> + <ets>chromic</ets>.]</ety> <fld>(Chem.)</fld> <def>Containing three atoms of chromium.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tricipital</ent><br/
<hw>Tri*cip"i*tal</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[See <er>Triceps</er>.]</ety> <fld>(Anat.)</fld> <def>Having three heads, or three origins; <as>as, a <ex>tricipital</ex> muscle</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Trick</ent><br/
<hw>Trick</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[D. <ets>trek</ets> a pull, or drawing, a trick, <ets>trekken</ets> to draw; akin to LG. <ets>trekken</ets>, MHG. <ets>trecken</ets>, <ets>trechen</ets>, Dan. <ets>tr<ae/kke</ets>, and OFries. <ets>trekka</ets>.  Cf. <er>Track</er>, <er>Trachery</er>, <er>Trig</er>, <pos>a.</pos>, <er>Trigger</er>.]</ety> <sn>1.</sn> <def>An artifice or stratagem; a cunning contrivance; a sly procedure, usually with a dishonest intent; <as>as, a <ex>trick</ex> in trade</as>.</def>
<-- the <col><b>tricks of the trade</b></col> mean simply specialized knowledge, in a good or neutral sense. --><br/
[<source>1913 Webster</source>]</p>

<p><q>He comes to me for counsel, and I show him a <qex>trick</qex>.</q> <rj><qau>South.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>I know a trick worth two of that.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A sly, dexterous, or ingenious procedure fitted to puzzle or amuse; <as>as, a bear's <ex>tricks</ex>; a juggler's <ex>tricks</ex></as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>Mischievous or annoying behavior; a prank; <as>as, the <ex>tricks</ex> of boys</as>.</def>  <rj><au>Prior.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>A particular habit or manner; a peculiarity; a trait; <as>as, a <ex>trick</ex> of drumming with the fingers; a <ex>trick</ex> of frowning</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The <qex>trick</qex> of that voice I do well remember.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>He hath a <qex>trick</qex> of C<oe/ur de Lion's face.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <def>A knot, braid, or plait of hair.</def> <mark>[Obs.]</mark>  <rj><au>B. Jonson.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>6.</sn> <fld>(Card Playing)</fld> <def>The whole number of cards played in one round, and consisting of as many cards as there are players.</def><-- in games such as bridge, in which one side takes the trick, to its advntage. --><br/
[<source>1913 Webster</source>]</p>

<p><q>On one nice <qex>trick</qex> depends the general fate.</q> <rj><qau>Pope.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>7.</sn> <fld>(Naut.)</fld> <def>A turn; specifically, the spell of a sailor at the helm, -- usually two hours.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>8.</sn> <def>A toy; a trifle; a plaything.</def> <mark>[Obs.]</mark>  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><syn><b>Syn.</b> -- Stratagem; wile; fraud; cheat; juggle; finesse; sleight; deception; imposture; delusion; imposition.</syn><br/
[<source>1913 Webster</source>]</p>

<p><ent>Trick</ent><br/
<hw>Trick</hw> <pr>(?)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Tricked</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Tricking</conjf>.]</vmorph> <sn>1.</sn> <def>To deceive by cunning or artifice; to impose on; to defraud; to cheat; <as>as, to <ex>trick</ex> another in the sale of a horse</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To dress; to decorate; to set off; to adorn fantastically; -- often followed by <xex>up</xex>, <xex>off</xex>, or <xex>out</xex>.</def> <ldquo/ <xex>Trick</xex> her off in air.<rdquo/  <rj><au>Pope.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>People lavish it profusely in <qex>tricking</qex> up their children in fine clothes, and yet starve their minds.</q> <rj><qau>Locke.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>They are simple, but majestic, records of the feelings of the poet; as little <qex>tricked</qex> out for the public eye as his diary would have been.</q> <rj><qau>Macaulay.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>To draw in outline, as with a pen; to delineate or distinguish without color, as arms, etc., in heraldry.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>They forget that they are in the statutes: . . . there they are <qex>tricked</qex>, they and their pedigrees.</q> <rj><qau>B. Jonson.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tricker</ent><br/
<hw>Trick"er</hw> <pr>(?)</pr>, <pos>n.</pos> <def>One who tricks; a trickster.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tricker</ent><br/
<hw>Trick"er</hw>, <pos>n.</pos> <def>A trigger.</def> <mark>[Obs. or Prov. Eng.]</mark>  <rj><au>Boyle.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Trickery</ent><br/
<hw>Trick"er*y</hw> <pr>(?)</pr>, <pos>n.</pos> <def>The art of dressing up; artifice; stratagem; fraud; imposture.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Trickiness</ent><br/
<hw>Trick"i*ness</hw> <pr>(?)</pr>, <pos>n.</pos> <def>The quality of being tricky.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tricking</ent><br/
<hw>Trick"ing</hw>, <pos>a.</pos> <def>Given to tricks; tricky.</def>  <rj><au>Sir W. Scott.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tricking</ent><br/
<hw>Trick"ing</hw>, <pos>n.</pos> <def>Dress; ornament.</def>  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Trickish</ent><br/
<hw>Trick"ish</hw>, <pos>a.</pos> <def>Given to tricks; artful in making bargains; given to deception and cheating; knavish.</def> -- <wordforms><wf>Trick"ish*ly</wf>, <pos>adv.</pos> -- <wf>Trick"ish*ness</wf>, <pos>n.</pos></wordforms><br/
[<source>1913 Webster</source>]</p>

<p><ent>Trickle</ent><br/
<hw>Tric"kle</hw> <pr>(tr<icr/k"k'l)</pr>, <pos>v. i.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Trickled</conjf> <pr>(tr<icr/k"k'ld)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Trickling</conjf> <pr>(tr<icr/k"kl<icr/ng)</pr>.]</vmorph> <ety>[OE. <ets>triklen</ets>, probably for <ets>striklen</ets>, freq. of <ets>striken</ets> to flow, AS. <ets>str<imac/can</ets>. See <er>Strike</er>, <pos>v. t.</pos>]</ety> <def>To flow in a small, gentle stream; to run in drops.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>His salt tears <qex>trickled</qex> down as rain.</q> <rj><qau>Chaucer.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Fast beside there <qex>trickled</qex> softly down<br/
A gentle stream.</q> <rj><qau>Spenser.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Trickle</ent><br/
<hw>Tric"kle</hw> <pr>(?)</pr>, <pos>n.</pos> <def>The act or state of trickling; also, that which trickles; a small stream; drip.</def></p>

<p><q>Streams that . . . are short and rapid torrents after a storm, but at other times dwindle to feeble <qex>trickles</qex> of mud.</q>  <rj><qau>James Bryce.</qau></rj><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Trickment</ent><br/
<hw>Trick"ment</hw> <pr>(?)</pr>, <pos>n.</pos> <def>Decoration.</def> <mark>[Obs.]</mark> <ldquo/ No <xex>trickments</xex> but my tears.<rdquo/  <rj><au>Beau. & Fl.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tricksiness</ent><br/
<hw>Trick"si*ness</hw> <pr>(?)</pr>, <pos>n.</pos> <def>The quality or state of being tricksy; trickiness.</def>  <rj><au>G. Eliot.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Trickster</ent><br/
<hw>Trick"ster</hw> <pr>(?)</pr>, <pos>n.</pos> <def>One who tricks; a deceiver; a tricker; a cheat.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tricksy</ent><br/
<hw>Trick"sy</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[From <er>Trick</er>.]</ety> <def>Exhibiting artfulness; trickish.</def> <ldquo/My <xex>tricksy</xex> spirit!<rdquo/  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>he <qex>tricksy</qex> policy which in the seventeenth century passed for state wisdom.</q> <rj><qau>Coleridge.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tricktrack</ent><br/
<hw>Trick"track`</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F. <ets>trictrac</ets>.  Cf. <er>Ticktack</er> backgammon.]</ety> <def>An old game resembling backgammon.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tricky</ent><br/
<hw>Trick"y</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Given to tricks; practicing deception; trickish; knavish.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Triclinate</ent><br/
<hw>Tric"li*nate</hw> <pr>(?)</pr>, <pos>a.</pos> <fld>(Min.)</fld> <def>Triclinic.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tricliniary</ent><br/
<hw>Tri*clin"i*a*ry</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>tricliniaris</ets>. See <er>Triclinium</er>.]</ety> <def>Of or pertaining to a triclinium, or to the ancient mode of reclining at table.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Triclinic</ent><br/
<hw>Tri*clin"ic</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[Pref. <ets>tri-</ets> + Gr. <?/ to incline.]</ety> <fld>(Crystallog.)</fld> <def>Having, or characterized by, three unequal axes intersecting at oblique angles. See the Note under <er>crystallization</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Triclinium</ent><br/
||<hw>Tri*clin"i*um</hw> <pr>(?)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Triclinia</plw> <pr>(#)</pr>.</plu> <ety>[L., from Gr. <?/, <?/; <?/ (see <er>Tri-</er>) + <er><?/</er> a couch.]</ety> <fld>(Rom. Antiq.)</fld> <sd>(a)</sd> <def>A couch for reclining at meals, extending round three sides of a table, and usually in three parts.</def> <sd>(b)</sd> <def>A dining room furnished with such a triple couch.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tricoccous</ent><br/
<hw>Tri*coc"cous</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[Gr. <grk>tri`kokkos</grk> with three grains or berries; <?/ (see <er>Tri-</er>) + <grk>ko`kkos</grk> grain, seed.]</ety> <fld>(Bot.)</fld> <def>Having three cocci, or roundish carpels.</def>  <rj><au>Gray.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tricolor</ent><br/
<hw>Tri"col`or</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F. <ets>tricolore</ets>, drapeau <ets>tricolore</ets> a tricolored flag, fr. <ets>tricolore</ets> three-colored; <ets>tri</ets> (see <er>Tri-</er>) + L. <ets>color</ets> color.]</ety> <altsp>[Written also <asp>tricolour</asp>.]</altsp><br/
[<source>1913 Webster</source>]</p>

<p><sn>1.</sn> <def>The national French banner, of three colors, blue, white, and red, adopted at the first revolution.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Hence, any three-colored flag.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tricolored</ent><br/
<hw>Tri"col`ored</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Having three colors.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tricornigerous</ent><br/
<hw>Tri`cor*nig"er*ous</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>tricorniger</ets>; <ets>tricornis</ets> three-horned (see <er>Tri-</er>, and <er>Horn</er>) + <ets>-gerere</ets> to bear.]</ety> <def>Having three horns.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tricorporate</ent><br/
<ent>Tricorporal</ent><br/
<mhw>{ <hw>Tri*cor"po*ral</hw> <pr>(?)</pr>, <hw>Tri*cor"po*rate</hw> <pr>(?)</pr>, }</mhw> <pos>a.</pos> <ety>[L. <ets>tricorpor</ets>; <ets>tri-</ets> (see <er>Tri-</er>) + <ets>corpus</ets>, <ets>-oris</ets>, body.]</ety> <fld>(Her.)</fld> <def>Represented with three bodies conjoined to one head, as a lion.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tricostate</ent><br/
<hw>Tri*cos"tate</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[Pref. <ets>tri-</ets> + <ets>costate</ets>.]</ety> <fld>(Bot.)</fld> <def>Three-ribbed; having three ribs from the base.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tricot</ent><br/
||<hw>Tri`cot"</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F.]</ety> <def>A fabric of woolen, silk, or cotton knitted, or women to resemble knitted work.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tricrotic</ent><br/
<hw>Tri*crot"ic</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[Pref. <ets>tri-</ets> + Gr. <?/ to beat.]</ety> <fld>(Physiol.)</fld> <def>Of or pertaining to tricrotism; characterized by tricrotism.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tricrotism</ent><br/
<hw>Tri"cro*tism</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Physiol.)</fld> <def>That condition of the arterial pulse in which there is a triple beat. The pulse curve obtained in the sphygmographic tracing characteristic of tricrotism shows two secondary crests in addition to the primary.</def><br/
[<source>1913 Webster</source>]</p>

<p><-- p. 1538 --><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tricrotous</ent><br/
<hw>Tri"cro*tous</hw> <pr>(tr<imac/"kr<osl/*t<ucr/s)</pr>, <pos>a.</pos> <fld>(Physiol.)</fld> <def>Tricrotic.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tricurvate</ent><br/
<hw>Tri*cur"vate</hw> <pr>(tr<isl/*k<ucir/r"v<asl/t)</pr>, <pos>a.</pos> <ety>[Pref. <ets>tri-</ets> + <ets>curvate</ets>.]</ety> <fld>(Zool.)</fld> <def>Curved in three directions; <as>as, a <ex>tricurvate</ex> spicule</as> (see <ex>Illust.</ex> of <er>Spicule</er>).</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tricuspid</ent><br/
<hw>Tri*cus"pid</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>tricuspis</ets>, <ets>-idis</ets>; <ets>tri-</ets> (see <er>Tri-</er>) + <ets>cuspis</ets> a point: cf. F. <ets>tricuspide</ets>.]</ety> <sn>1.</sn> <def>Having three cusps, or points; tricuspidate; <as>as, a <ex>tricuspid</ex> molar</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Anat.)</fld> <def>Of or pertaining to the tricuspid valves; <as>as, <ex>tricuspid</ex> obstruction</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Tricuspid valve</b></col> <fld>(Anat.)</fld>, <cd>the valve, consisting of three triangular membranous flaps, at the opening of the right auricle into the right ventricle in the heart of most mammals; -- sometimes called the <altname>tricuspid valves</altname>, each flap being regarded as a valve.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tricuspidate</ent><br/
<hw>Tricus"pid*ate</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Three-pointed; ending in three points; <as>as, a <ex>tricuspidate</ex> leaf</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tricycle</ent><br/
<hw>Tri"cy*cle</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Pref. <ets>tri-</ets> + <ets>cycle</ets> as in<ets>bicycle</ets>.]</ety> <def>A three-wheeled velocipede. See <xex>Illust.</xex> under <er>Velocipede</er>.  Cf. <er>Bicycle</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tridacna</ent><br/
||<hw>Tri*dac"na</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L., pl., a kind of oysters, fr. Gr. <?/ eaten at three bites, <?/ tri- + <?/ to bite.]</ety> <fld>(Zool.)</fld> <def>A genus of very large marine bivalve shells found on the coral reefs of the Indian and Pacific Oceans. One species (<spn>Tridacna gigas</spn>) often weighs four or five hundred pounds, and is sometimes used for baptismal fonts. Called also <altname>paw shell</altname>, and <altname>fountain shell</altname>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tridactyle</ent><br/
<ent>Tridactyl</ent><br/
<mhw>{ <hw>Tri*dac"tyl</hw> <hw>Tri*dac"tyle</hw> }</mhw> <pr>(?)</pr>, <pos>a.</pos> <ety>[Gr. <?/: <?/ (see <er>Tri-</er>) + <?/ digit: cf. F. <ets>tridactyle</ets>.]</ety> <fld>(Biol.)</fld> <def>Having three fingers or toes, or composed of three movable parts attached to a common base.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tridactylous</ent><br/
<hw>Tri*dac"tyl*ous</hw> <pr>(?)</pr>, <pos>a.</pos> <fld>(Biol.)</fld> <def>Tridactyl.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Triddler</ent><br/
<hw>Trid"dler</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Zool.)</fld> <def>The jacksnipe.</def> <mark>[Local, U. S.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tride</ent><br/
<hw>Tride</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[Cf. F. <ets>tride</ets> lively, quick.]</ety> <def>Short and ready; fleet; <as>as, a <ex>tride</ex> pace</as>; -- a term used by sportsmen.</def>  <rj><au>Bailey.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tridecane</ent><br/
<hw>Tri*dec"ane</hw> <pr>(tr<isl/*d<ecr/k"<amac/n)</pr>, <pos>n.</pos> <ety>[Pref. <ets>tri-</ets> + Gr. <?/ ten. So called from the number of carbon atoms in the molecule.]</ety> <fld>(Chem.)</fld> <def>A hydrocarbon, <chform>C13H28</chform>, of the methane series, which is a probable ingredient both of crude petroleum and of kerosene, and is produced artificially as a light colorless liquid.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tridecatoic</ent><br/
<hw>Tri`dec*a*to"ic</hw> <pr>(tr<isl/`d<ecr/k*<adot/*t<omac/"<icr/k)</pr>, <pos>a.</pos> <ety>[Pref. <ets>tri-</ets> + Gr. <grk>de`katos</grk> tenth.]</ety> <fld>(Chem.)</fld> <def>Of, pertaining to, or designating, that acid of the fatty acids heterologous with tridecane. It is a white crystalline substance.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tridecatylene</ent><br/
<hw>Tri`de*cat"y*lene</hw> <pr>(tr<isl/`d<esl/*k<acr/t"<icr/*l<emac/n)</pr>, <pos>n.</pos> <ety>[Pref. <ets>tri-</ets>+ Gr. <grk>de`katos</grk> tenth + E. eth<ets>ylene</ets>.]</ety> <fld>(Chem.)</fld> <def>A hydrocarbon, <chform>C13H26</chform>, of the ethylene series, corresponding to tridecane, and obtained from Burmah petroleum as a light colorless liquid; -- called also <altname>tridecylene</altname>, and <altname>tridecene</altname>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Trident</ent><br/
<hw>Tri"dent</hw> <pr>(tr<imac/"d<eit/nt)</pr>, <pos>n.</pos> <ety>[L. <ets>tridens</ets>, <ets>-entis</ets>; <ets>tri-</ets> (see <er>Tri-</er>) + <ets>dens</ets> tooth: cf. F. <ets>trident</ets>. See <er>Tooth</er>.]</ety> <sn>1.</sn> <fld>(Class Myth.)</fld> <def>A kind of scepter or spear with three prongs, -- the common attribute of Neptune.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Rom. Antiq.)</fld> <def>A three-pronged spear or goad, used for urging horses; also, the weapon used by one class of gladiators.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>A three-pronged fish spear.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <fld>(Geom.)</fld> <def>A curve of third order, having three infinite branches in one direction and a fourth infinite branch in the opposite direction.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Trident bat</b></col> <fld>(Zool.)</fld>, <cd>an Asiatic rhinolophid bat (<spn>Triaenops Persicus</spn>), having the nose membrane in the shape of a trident.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Trident</ent><br/
<hw>Tri"dent</hw>, <pos>a.</pos> <ety>[L. <ets>tridens</ets>.]</ety> <def>Having three teeth or prongs; tridentate.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tridentated</ent><br/
<ent>Tridentate</ent><br/
<mhw>{ <hw>Tri*den"tate</hw> <pr>(?)</pr>, <hw>Tri*den"ta*ted</hw> <pr>(?)</pr>, }</mhw> <pos>a.</pos> <ety>[NL. <ets>tridentatus</ets>. See <er>Trident</er>.]</ety> <def>Having three teeth; three-toothed.</def>  <rj><au>Lee.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tridented</ent><br/
<hw>Tri"dent*ed</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Having three prongs; trident; tridentate; <as>as, a <ex>tridented</ex> mace</as>.</def> <mark>[R.]</mark>  <rj><au>Quarles.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tridentiferous</ent><br/
<hw>Tri`dent*if"er*ous</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>tridentifer</ets>; <ets>tridens</ets> trident + <ets>ferre</ets> to bear.]</ety> <def>Bearing a trident.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tridentine</ent><br/
<hw>Tri*den"tine</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[From L. <ets>Tridentum</ets> Trent.]</ety> <def>Of or pertaining to Trent, or the general church council held in that city.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tridiapason</ent><br/
<hw>Tri*di`a*pa"son</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Pref. <ets>tri-</ets> + <ets>diaposon</ets>.]</ety> <fld>(Anc. Mus.)</fld> <def>A triple octave, or twenty-second.</def>  <rj><au>Busby.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tridimensional</ent><br/
<hw>Tri`di*men"sion*al</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[Pref. <ets>tri-</ets> + <ets>dimensional</ets>.]</ety> <fld>(Chem.)</fld> <def>Having three dimensions; extended in three different directions.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Triding</ent><br/
<hw>Tri"ding</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A riding. See <er>Trithing</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Triduan</ent><br/
<hw>Trid"u*an</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>triduanus</ets>, fr. <ets>triduum</ets> space of three days; <ets>tri-</ets> + <ets>dies</ets> day.]</ety> <def>Lasting three lays; also, happening every third day.</def> <mark>[R.]</mark>  <rj><au>Blount.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tridymite</ent><br/
<hw>Trid"y*mite</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Gr. <?/ threefold. So called in allusion to the supposed threefold nature of the twin crystals.]</ety> <fld>(Min.)</fld> <def>Pure silica, like quartz, but crystallizing in hexagonal tables. It is found in trachyte and similar rocks.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tried</ent><br/
<hw>Tried</hw> <pr>(?)</pr>, <def><pos>imp. & p. p.</pos> of <er>Try.</er></def> <def2>Also <pos>adj.</pos> <def>Proved; tested; faithful; trustworthy; <as>as, a <ex>tried</ex> friend</as>.</def></def2><br/
[<source>1913 Webster</source>]</p>

<p><ent>Triedral</ent><br/
<hw>Tri*e"dral</hw> <pr>(?)</pr>, <pos>a.</pos> <def>See <er>Trihedral</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Triennial</ent><br/
<hw>Tri*en"ni*al</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>triennium</ets> the space of three years; <ets>tri-</ets> (see <er>Tri-</er>) + <ets>annus</ets> year. See <er>Annual</er>.]</ety> <sn>1.</sn> <def>Continuing three years; <as>as, <ex>triennial</ex> parliaments; a <ex>triennial</ex> reign</as>.</def>  <rj><au>Howell.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Happening, coming about, or appearing once in every three years; <as>as, <ex>triennial</ex> elections; a <ex>triennial</ex> catalogue; a <ex>triennial</ex> visitation.</as></def>  <rj><au>T. Warton.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Triennial</ent><br/
<hw>Tri*en"ni*al</hw>, <pos>n.</pos> <def>Something which takes place or appears once in three years.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Triennially</ent><br/
<hw>Tri*en"ni*al*ly</hw>, <pos>adv.</pos> <def>Once in three years.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Triens</ent><br/
||<hw>Tri"ens</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L., from <ets>tres</ets>, <ets>tria</ets>, three.]</ety> <fld>(Rom. Antiq.)</fld> <def>A Roman copper coin, equal to one third of the <it>as.</it> See 3d <er>As</er>, 2.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Trier</ent><br/
<hw>Tri"er</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[From <er>Try</er>.]</ety> <sn>1.</sn> <def>One who tries; one who makes experiments; one who examines anything by a test or standard.</def>  <rj><au>Boyle.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>One who tries judicially.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <fld>(Law)</fld> <def>A person appointed according to law to try challenges of jurors; a trior.</def>  <rj><au>Burrill.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>That which tries or approves; a test.</def>  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Trierarch</ent><br/
<hw>Tri"er*arch</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>trierarchus</ets>, Gr. <?/; <?/ a trireme + <?/ a leader, a chief.]</ety> <fld>(Gr. Antiq.)</fld> <sd>(a)</sd> <def>The commander of a trireme.</def> <sd>(b)</sd> <def>At Athens, one who (singly, or jointly with other citizens) had to fit out a trireme for the public service.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Trierarchy</ent><br/
<hw>Tri"er*arch`y</hw> <pr>(?)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Trierarchises</plw> <pr>(#)</pr>.</plu> <ety>[ Gr. <?/.]</ety> <def>The office duty of a trierarch.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Trieterical</ent><br/
<hw>Tri`e*ter"ic*al</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>trietericus</ets>, Gr. <?/, fr. <?/ (sc. <?/) a triennial festival; <?/ (see <er>Tri-</er>.) + <?/ a year.]</ety> <def>Kept or occurring once in three years; triennial.</def> <mark>[R.]</mark>  <rj><au>J. Gregory.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Trieterics</ent><br/
<hw>Tri`e*ter"ics</hw> <pr>(?)</pr>, <pos>n. pl.</pos> <ety>[L. <ets>trieterica</ets>, pl., fr. Gr. <?/ of a triennial festival.]</ety> <fld>(Class. Antiq.)</fld> <def>Festival games celebrated once in three years.</def> <mark>[R.]</mark>  <rj><au>May.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Triethylamine</ent><br/
<hw>Tri*eth`yl*am"ine</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Pref. <ets>tri-</ets> + <ets>ethylamine</ets>.]</ety> <fld>(Chem.)</fld> <def>A tertiary amine analogous to trimethylamine.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Trifacial</ent><br/
<hw>Tri*fa"cial</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[Pref. <ets>tri-</ets> + <ets>facial</ets>.]</ety> <fld>(Anat.)</fld> <def>See <er>Trigeminal</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Trifallow</ent><br/
<hw>Tri"fal`low</hw> <pr>(?)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Trifallowed</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Trifallowing</conjf>.]</vmorph> <ety>[Pref. <ets>tri-</ets> + <ets>fallow</ets>.  Cf. <er>Thryfallow</er>.]</ety> <def>To plow the third time before sowing, as land.</def>  <rj><au>Mortimer.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Trifarious</ent><br/
<hw>Tri*fa"ri*ous</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>trifarius</ets> of three sorts or ways, threefold; cf. Gr. <?/.  Cf. <er>Bifarious</er>.]</ety> <fld>(Bot.)</fld> <def>Facing three ways; arranged in three vertical ranks, as the leaves of veratrum.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Trifasciated</ent><br/
<hw>Tri*fas"ci*a`ted</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[Pref. <ets>tri-</ets> + <ets>fasciated</ets>.]</ety> <def>Having, or surrounded by, three fasciae, or bands.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Trifid</ent><br/
<hw>Tri"fid</hw>, <pos>a.</pos> <ety>[L. <ets>trifidus</ets>; tri- (see <er>Tri-</er>) + the root of <ets>findere</ets> to split: cf. F. <ets>trifide</ets>.]</ety> <def>Cleft to the middle, or slightly beyond the middle, into three parts; three-cleft.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Trifistulary</ent><br/
<hw>Tri*fis"tu*la*ry</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[Pref. <ets>tri-</ets> + <ets>fistula</ets>, <ets>fistular</ets>.]</ety> <def>Having three pipes.</def>  <rj><au>Sir T. Browne.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Trifle</ent><br/
<hw>Tri"fle</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[OE. <ets>trifle</ets>, <ets>trufle</ets>, OF. <ets>trufle</ets> mockery, raillery, trifle, probably the same word as F. <ets>truffe</ets> truffle, the word being applied to any small or worthless object. See <er>Truffle</er>.]</ety> <sn>1.</sn> <def>A thing of very little value or importance; a paltry, or trivial, affair.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>With such poor <qex>trifles</qex> playing.</q> <rj><qau>Drayton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q><qex>Trifles</qex> light as air<br/
Are to the jealous confirmation strong<br/
As proofs of holy writ.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Small sands the mountain, moments make year,<br/
And <qex>frifles</qex> life.</q> <rj><qau>Young.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A dish composed of sweetmeats, fruits, cake, wine, etc., with syllabub poured over it.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Trifle</ent><br/
<hw>Tri"fle</hw>, <pos>v. i.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Trifled</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Trifling</conjf> <pr>(?)</pr>.]</vmorph> <ety>[OE. <ets>trifelen</ets>, <ets>truflen</ets>. See <er>Trifle</er>, <pos>n.</pos>]</ety> <def>To act or talk without seriousness, gravity, weight, or dignity; to act or talk with levity; to indulge in light or trivial amusements.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>They <qex>trifle</qex>, and they beat the air about nothing which toucheth us.</q> <rj><qau>Hooker.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>To trifle with</b></col>, <cd>to play the fool with; to treat without respect or seriousness; to mock; as, <xex>to trifle with</xex> one's feelings, or with sacred things.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Trifle</ent><br/
<hw>Tri"fle</hw>, <pos>v. t.</pos> <sn>1.</sn> <def>To make of no importance; to treat as a trifle.</def> <mark>[Obs.]</mark>  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To spend in vanity; to fritter away; to waste; <as>as, to <ex>trifle</ex> away money</as>.</def> <ldquo/We <xex>trifle</xex> time.<rdquo/  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Trifler</ent><br/
<hw>Tri"fler</hw> <pr>(?)</pr>, <pos>n.</pos> <def>One who trifles.</def>  <rj><au>Waterland.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Trifling</ent><br/
<hw>Tri"fling</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Being of small value or importance; trivial; paltry; <as>as, a <ex>trifling</ex> debt; a <ex>trifling</ex> affair.</as></def> -- <wordforms><wf>Tri"fling*ly</wf>, <pos>adv.</pos> -- <wf>Tri"fling*ness</wf>, <pos>n.</pos></wordforms><br/
[<source>1913 Webster</source>]</p>

<p><ent>Triflorous</ent><br/
<ent>Trifloral</ent><br/
<mhw>{ <hw>Tri*flo"ral</hw> <pr>(?)</pr>, <hw>Tri*flo"rous</hw> <pr>(?)</pr>, }</mhw> <pos>a.</pos> <ety>[Pref. <ets>tri-</ets> + L. <ets>flos</ets>, <ets>floris</ets>, flower.]</ety> <fld>(Bot.)</fld> <def>Three-flowered; having or bearing three flowers; <as>as, a <ex>triflorous</ex> peduncle</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Trifluctuation</ent><br/
<hw>Tri*fluc`tu*a"tion</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Pref. <ets>tri-</ets> + <ets>fluctuation</ets>.]</ety> <def>A concurrence of three waves.</def> <mark>[Obs.]</mark> <ldquo/A <xex>trifluctuation</xex> of evils.<rdquo/  <rj><au>Sir T. Browne.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Trifoliated</ent><br/
<ent>Trifoliate</ent><br/
<mhw>{ <hw>Tri*fo"li*ate</hw> <pr>(?)</pr>, <hw>Tri*fo"li*a`ted</hw> <pr>(?)</pr>, }</mhw> <pos>a.</pos> <ety>[<ets>Tri-</ets> + <ets>foliate.</ets>   Cf. <er>Trefoil</er>.]</ety> <fld>(Bot.)</fld> <def>Having three leaves or leaflets, as clover. See <xex>Illust.</xex> of <er>Shamrock</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Trifoliolate</ent><br/
<hw>Tri*fo"li*o*late</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[Pref. <ets>tri-</ets> + <ets>foliolate</ets>.]</ety> <fld>(Bot.)</fld> <def>Having three leaflets.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Trifolium</ent><br/
||<hw>Tri*fo"li*um</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L., clover.]</ety> <fld>(Bot.)</fld> <def>A genus of leguminous herbs with densely spiked flowers and usually trifoliate leaves; trefoil. There are many species, all of which are called <xex>clover</xex>. See <er>Clover</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Trifoly</ent><br/
<hw>Tri"fo*ly</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>trifolium</ets>. See <er>Trifoliate</er>, <er>Trefoil</er>.]</ety> <fld>(Bot.)</fld> <def>Sweet trefoil.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>She was crowned with a chaplet of <qex>trifoly</qex>.</q> <rj><qau>B. Jonson.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Triforium</ent><br/
||<hw>Tri*fo"ri*um</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[LL., fr. L. <ets>tri-</ets> (see <er>Tri-</er>) + <ets>foris</ets>, pl. <ets>fores</ets>, a door.]</ety> <fld>(Arch.)</fld> <def>The gallery or open space between the vaulting and the roof of the aisles of a church, often forming a rich arcade in the interior of the church, above the nave arches and below the clearstory windows.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Triform</ent><br/
<hw>Tri"form</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>triformis</ets>; <ets>tri-</ets> (see <er>Tri-</er>) + <ets>forma</ets> form.]</ety> <def>Having a triple form or character.</def> <ldquo/This <xex>triform</xex> antagonism.<rdquo/  <rj><au>I. Taylor.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Goddess <qex>Triform</qex>, I own thy triple spell.</q> <rj><qau>Lowell.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Triformity</ent><br/
<hw>Tri*form"i*ty</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>triformitas</ets>.]</ety> <def>The state of being triform, or of having a threefold shape.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Trifurcated</ent><br/
<ent>Trifurcate</ent><br/
<mhw>{ <hw>Tri*fur"cate</hw> <pr>(?)</pr>, <hw>Tri*fur"ca*ted</hw> <pr>(?)</pr>, }</mhw> <pos>a.</pos> <ety>[L. <ets>trifurcus</ets>; <ets>tri-</ets> (see <er>Tri-</er>) + <ets>furca</ets> fork.]</ety> <def>Having three branches or forks; trichotomous.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Trig</ent><br/
<hw>Trig</hw> <pr>(?)</pr>, <pos>v. t.</pos> <ety>[Cf. Dan. <ets>trykke</ets> to press, Sw. <ets>trycka</ets>.]</ety> <def>To fill; to stuff; to cram.</def> <mark>[Obs.]</mark>  <rj><au>Dr. H. More.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Trig</ent><br/
<hw>Trig</hw>, <pos>a.</pos> <ety>[Formerly written <ets>trick</ets>, akin to <ets>trick</ets> to dress.]</ety> <def>Full; also, trim; neat.</def> <mark>[Prov. Eng. & Scot.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>To sit on a horse square and <qex>trig</qex>.</q> <rj><qau>Brit. Quart. Rev.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Trig</ent><br/
<hw>Trig</hw>, <pos>v. t.</pos> <ety>[See <er>Trigger</er>.]</ety> <def>To stop, as a wheel, by placing something under it; to scotch; to skid.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Trig</ent><br/
<hw>Trig</hw>, <pos>n.</pos> <ety>[See <er>Trigger</er>.]</ety> <def>A stone, block of wood, or anything else, placed under a wheel or barrel to prevent motion; a scotch; a skid.</def> <mark>[Eng.]</mark>  <rj><au>Wright.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Trigamist</ent><br/
<hw>Trig"a*mist</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[See <er>Trigamy</er>.]</ety> <def>One who has been married three times; also, one who has three husbands or three wives at the same time.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Trigamous</ent><br/
<hw>Trig"a*mous</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>trigamus</ets> a thrice-married man, Gr. <?/ thrice married; <?/ (see <er>Tri-</er>) + <?/ marriage: cf. F. <ets>trigame</ets>.]</ety> <fld>(Bot.)</fld> <def>Having three sorts of flowers in the same head, -- male, female, and hermaphrodite, or perfect, flowers.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Trigamy</ent><br/
<hw>Trig"a*my</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>trigamia</ets>,Gr. <?/: cf. F. <ets>trigamie</ets>. See <er>Trigamous</er>.]</ety> <def>The act of marrying, or the state of being married, three times; also, the offense of having three husbands or three wives at the same time.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Trigastric</ent><br/
<hw>Tri*gas"tric</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[Pref. <ets>tri-</ets> + Gr. <?/ belly.]</ety> <fld>(Anat.)</fld> <def>Having three bellies; -- said of a muscle.</def>  <rj><au>Dunglison.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Trigeminal</ent><br/
<hw>Tri*gem"i*nal</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[See <er>Trigeminous</er>.]</ety> <fld>(Anat.)</fld> <def>Of, pertaining to, or designating, the fifth pair of cranial nerves, which divide on each side of the head into three main branches distributed to the orbits, jaws, and parts of the mouth; trifacial.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Trigeminous</ent><br/
<hw>Tri*gem"i*nous</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>trigeminus</ets> born three together; <ets>tri-</ets> (see <er>Tri-</er>) + <ets>geminus</ets> twin.  Cf. <er>Tergeminous</er>.]</ety> <def>Born three together; being one of three born at the same birth; also, threefold.</def>  <rj><au>E. Phillip<?/.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Trigenic</ent><br/
<hw>Tri*gen"ic</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[Pref. <ets>tri-</ets> + <ets>gen-</ets> + <ets>-ic</ets>. So named in reference to its composition, it being supposed to contain the radicals of three molecules of cyanic acid.]</ety> <fld>(Chem.)</fld> <def>Of, pertaining to, or designating, an acid, <chform>C4H7N3O2</chform>, obtained, by the action of the vapor of cyanic acid on cold aldehyde, as a white crystalline substance having a slightly acid taste and faint smell; -- called also <altname>ethidene-biuret</altname> <it>or</it> <altname>ethylidene-biuret</altname>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Triger process</ent><br/
<hw>Tri`ger proc"ess</hw> <pr>(?)</pr>. <ety>[After M. <etsep>Triger</etsep>, French engineer.]</ety> <fld>(Engin. & Mining)</fld> <def>A method of sinking through water-bearing ground, in which the shaft is lined with tubbing and provided with an air lock, work being proceeded with under air pressure.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Trigesimo-secundo</ent><br/
<hw>Tri*ges"i*mo-se*cun"do</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>in trigesimo-secundo</ets> in the thirty-second.]</ety> <def>Having thirty-two leaves to a sheet; <as>as, a <ex>trigesimo-secundo</ex> form, book, leaf, size, etc.</as></def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Trigesimo-secundo</ent><br/
<hw>Tri*ges"i*mo-se*cun"do</hw>, <pos>n.</pos> <def>A book composed of sheets so folded that each one makes thirty-two leaves; hence, indicating, more or less definitely, a size of book; -- usually written <asp>32mo</asp>, or 32<deg/, and called <altname>thirty-twomo</altname>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Trigger</ent><br/
<hw>Trig"ger</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[For older <ets>tricker</ets>, from D. <ets>trekker</ets>, fr. <ets>trekken</ets> to draw, pull. See <er>Trick</er>, <pos>n.</pos>]</ety> <sn>1.</sn> <def>A catch to hold the wheel of a carriage on a declivity.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Mech.)</fld> <def>A piece, as a lever, which is connected with a catch or detent as a means of releasing it; especially <fld>(Firearms)</fld>, the part of a lock which is moved by the finger to release the cock and discharge the piece.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Trigger fish</b></col> <fld>(Zool.)</fld>, <cd>a large plectognath fish (<spn>Balistes Carolinensis</spn> or <spn>Balistes capriscus</spn>) common on the southern coast of the United States, and valued as a food fish in some localities. Its rough skin is used for scouring and polishing in the place of sandpaper. Called also <altname>leather jacket</altname>, and <altname>turbot</altname>.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Trigintal</ent><br/
<hw>Tri*gin`tal</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[LL. <ets>trigintate</ets>, fr. L. <ets>triginta</ets> thirty. See <er>Trental</er>.]</ety> <fld>(R. C. Ch.)</fld> <def>A trental.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Triglyceride</ent><br/
<hw>Tri*glyc"er*ide</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Pref. <ets>tri-</ets> + <ets>glyceride</ets>.]</ety> <fld>(Chem.)</fld> <def>A glyceride formed by the replacement of three hydrogen atoms in glycerin by acid radicals.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Triglyph</ent><br/
<hw>Tri"glyph</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>triglyphus</ets>, Gr. <?/; <?/ (see <er>Tri-</er>) + <?/ to carve: cf. F. <ets>triglyphe</ets>.]</ety> <fld>(Arch.)</fld> <def>An ornament in the frieze of the Doric order, repeated at equal intervals. Each triglyph consists of a rectangular tablet, slightly projecting, and divided nearly to the top by two parallel and perpendicular gutters, or channels, called <xex>glyphs</xex>, into three parts, or spaces, called <xex>femora</xex>. A half channel, or glyph, is also cut upon each of the perpendicular edges of the tablet. See <xex>Illust.</xex> of <er>Entablature</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Triglyphical</ent><br/
<ent>Triglyphic</ent><br/
<mhw>{ <hw>Tri*glyph"ic</hw> <pr>(?)</pr>, <hw>Tri*glyph"ic*al</hw> <pr>(?)</pr>, }</mhw> <pos>a.</pos> <sn>1.</sn> <def>Consisting of, or pertaining to, triglyphs.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Containing three sets of characters or sculptures.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Trigness</ent><br/
<hw>Trig"ness</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[See <er>Trig</er> trim, neat.]</ety> <def>The quality or state of being trig; smartness; neatness.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Their spars had no man-of-war <qex>trigness</qex>.</q> <rj><qau>Kane.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Trigon</ent><br/
<hw>Tri"gon</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>trigonum</ets>, Gr. <?/; <?/ (see <er>Tri-</er>) + <?/ a corner, angle: cf. F. <ets>trigone</ets>.]</ety><br/
[<source>1913 Webster</source>]</p>

<p><sn>1.</sn> <def>A figure having three angles; a triangle.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Astrol.)</fld> <sd>(a)</sd> <def>A division consisting of three signs.</def> <sd>(b)</sd> <def>Trine, an aspect of two planets distant 120 degrees from each other.</def>  <rj><au>Hutton.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <fld>(Gr. & Rom. Antiq.)</fld> <sd>(a)</sd> <def>A kind of triangular lyre or harp.</def> <sd>(b)</sd> <def>A kind of game at ball played by three persons standing at the angular points of a triangle.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Trigonid</ent><br/
<sn>4.</sn> <fld>(Zool.)</fld> <def>The cutting region of the crown of an upper molar, usually the anterior part. That of a lower molar is the <hw>Tri"go*nid</hw> <pr>(<?/)</pr>.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Trigonal</ent><br/
<hw>Trig"o*nal</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Having three angles, or corners; triangular; <as>as, a <ex>trigonal</ex> stem, one having tree prominent longitudinal angles</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><-- p. 1539 --><br/
[<source>1913 Webster</source>]</p>

<p><ent>Trigone</ent><br/
||<hw>Tri`gone"</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F., literally, a trigon.]</ety> <fld>(Anat.)</fld> <def>A smooth triangular area on the inner surface of the bladder, limited by the apertures of the ureters and urethra.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Trigonia</ent><br/
||<hw>Tri*go"ni*a</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[NL. See <er>Trigon</er>. So called in allusion to the triangular shape of some species.]</ety> <fld>(Zool.)</fld> <def>A genus of pearly bivalve shells, numerous extinct species of which are characteristic of the Mesozoic rocks. A few living species exist on the coast of Australia.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Trigonocerous</ent><br/
<hw>Trig`o*noc"er*ous</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[Gr. <?/ triangle + <?/ horn.]</ety> <fld>(Zool.)</fld> <def>Having horns with three angles, like those of some species of goats.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Trigonodont</ent><br/
<hw>Trig"o*no*dont`</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[See <er>Trigon</er>; <er>Odonto</er>.]</ety> <def>See <er>Trituberculy</er>.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Trigonometrical</ent><br/
<ent>Trigonometric</ent><br/
<mhw>{ <hw>Trig`o*no*met"ric</hw> <pr>(?)</pr>, <hw>Trig`o*no*met"ric*al</hw> <pr>(?)</pr>, }</mhw><ety>[Cf. F. <ets>trigonom<eacute/trique</ets>.]</ety> <def>Of or pertaining to trigonometry; performed by the rules of trigonometry.</def><br/
[<source>1913 Webster</source>]</p>

<p>--<wordforms><wf>Trig`o*no*met"ric*al*ly</wf>, <pos>adv.</pos></wordforms><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Trigonometrical curve</b></col>, <cd>a curve one of whose coordinates is a trigonometric function of the other.</cd> -- <col><b>Trigonometrical function</b></col>. <cd>See under <er>Function</er>.</cd> -- <col><b>Trigonometrical lines</b></col>, <cd>lines which are employed in solving the different cases of plane and spherical trigonometry, as sines, tangents, secants, and the like. These lines, or the lengths of them, are <xex>trigonometrical functions</xex> of the arcs and angles to which they belong.</cd> -- <col><b>Trigonometrical survey</b></col>. <cd>See under <er>Survey</er>.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Trigonometry</ent><br/
<hw>Trig`o*nom"e*try</hw> <pr>(?)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>-tries</plw> <pr>(#)</pr>.</plu> <ety>[Gr. <?/ a triangle + <ets>-metry</ets>: cf. F. <ets>trigonom<eacute/trie</ets>. See <er>Trigon</er>.]</ety> <sn>1.</sn> <def>That branch of mathematics which treats of the relations of the sides and angles of triangles, which the methods of deducing from certain given parts other required parts, and also of the general relations which exist between the trigonometrical functions of arcs or angles.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A treatise in this science.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Analytical trigonometry</b></col>, <cd>that branch of trigonometry which treats of the relations and properties of the trigonometrical functions.</cd> -- <mcol><col><b>Plane trigonometry</b></col>, <it>and</it> <col><b>Spherical trigonometry</b></col></mcol>, <cd>those branches of trigonometry in which its principles are applied to plane triangles and spherical triangles respectively.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Trigonous</ent><br/
<hw>Trig"o*nous</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>trigonus</ets>, Gr. <?/. See <er>Trigon</er>.]</ety> <def>Same as <er>Trigonal</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Trigram</ent><br/
<hw>Tri"gram</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Pref. <ets>tri-</ets> + <ets>-gram</ets>.]</ety> <def>Same as <er>Trigraph</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Trigrammatic</ent><br/
<hw>Tri`gram*mat"ic</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[Gr. <?/; <?/ + <?/ a letter.]</ety> <def>Containing three letters or characters, or three sets of letters or characters.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Trigrammic</ent><br/
<hw>Tri*gram"mic</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[Gr. <?/. See <er>Trigrammatic</er>.]</ety> <def>Same as <er>Trigrammatic</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Trigraph</ent><br/
<hw>Tri"graph</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Pref. <ets>tri-</ets> + <ets>-graph</ets>.]</ety> <def>Three letters united in pronunciation so as to have but one sound, or to form but one syllable, as <xex>-ieu</xex> in <xex>adieu</xex>; a triphthong.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Trigyn</ent><br/
<hw>Tri"gyn</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Bot.)</fld> <def>Any one of the Trigynia.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Trigynia</ent><br/
||<hw>Tri*gyn"i*a</hw> <pr>(?)</pr>, <pos>n. pl.</pos> <ety>[NL., Gr. <?/ (see <er>Tri-</er>) + <?/ a woman, a female.]</ety> <fld>(Bot.)</fld> <def>A Linnaean order of plants having three pistils or styles.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Trigynous</ent><br/
<ent>Trigynian</ent><br/
<mhw>{ <hw>Tri*gyn"i*an</hw> <pr>(?)</pr>, <hw>Trig"y*nous</hw> <pr>(?)</pr>, }</mhw> <pos>a.</pos> <fld>(Bot.)</fld> <def>Having three pistils or styles; of or pertaining to the Trigynia.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Trihedral</ent><br/
<hw>Tri*he"dral</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[See <er>Trihedron</er>.]</ety> <fld>(Geom.)</fld> <def>Having three sides or faces; thus, a <xex>trihedral</xex> angle is a solid angle bounded by three plane angles.</def> <altsp>[Written also <asp>triedral</asp>.]</altsp><br/
[<source>1913 Webster</source>]</p>

<p><ent>Trihedron</ent><br/
<hw>Tri*he"dron</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Pref. <ets>tri-</ets> + Gr. <?/ a seat, base.]</ety> <fld>(Geom.)</fld> <def>A figure having three sides.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Trihoral</ent><br/
<hw>Tri*ho"ral</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[Pref. <ets>tri-</ets> + <ets>horal</ets>.]</ety> <def>Occurring once in every three hours.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Trihybrid</ent><br/
<hw>Tri*hy"brid</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Biol.)</fld> <def>A hybrid whose parents differ by three pairs of contrasting Mendelian characters.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Trijugate</ent><br/
<hw>Trij"u*gate</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[See <er>Trijugous</er>.]</ety> <fld>(Bot.)</fld> <def>In three pairs; <as>as, a <ex>trijugate</ex> leaf, or a pinnate leaf with three pairs of leaflets</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Trijugous</ent><br/
<hw>Trij"u*gous</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>trijugus</ets> threefold; <ets>tri-</ets> + <ets>jugum</ets> a yoke.]</ety> <fld>(Bot.)</fld> <def>Same as <er>Trijugate</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Trikosane</ent><br/
<hw>Tri"ko*sane</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Pref. <ets>tri-</ets> + Gr. <?/ twenty.]</ety> <fld>(Chem.)</fld> <def>A hydrocarbon, <chform>C23H48</chform>, of the methane series, resembling paraffin; -- so called because it has <xex>twenty-three</xex> atoms of carbon in the molecule.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Trilateral</ent><br/
<hw>Tri*lat"er*al</hw> <pr>(?)</pr>, <pos>a.</pos><ety>[L. <ets>trilaterus</ets>; <ets>tri-</ets> (see <er>Tri-</er>) + <ets>latus</ets>, <ets>lateris</ets>, side: cf. F. <ets>trilat<eacute/ral</ets>. See <er>Lateral</er>.]</ety> <fld>(Geom.)</fld> <def>Having three sides; being three-sided; <as>as, a <ex>trilateral</ex> triangle</as>.</def> -- <wordforms><wf>Tri*lat"er*al*ly</wf>, <pos>adv.</pos> -- <wf>Tri*lat"er*al*ness</wf>, <pos>n.</pos></wordforms><br/
[<source>1913 Webster</source>]</p>

<p><ent>Trilemma</ent><br/
<hw>Tri*lem"ma</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[NL., fr. Gr. <?/ (see <er>Tri-</er>) + <er><?/</er> any thing received, in logic, an assumption.  Cf. <er>Dilemma</er>.]</ety> <sn>1.</sn> <fld>(Logic)</fld> <def>A syllogism with three conditional propositions, the major premises of which are disjunctively affirmed in the minor. See <er>Dilemma</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A state of things in which it is difficult to determine which one of three courses to pursue.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Trilinear</ent><br/
<hw>Tri*lin"e*ar</hw> <pr>(?)</pr>, <pos>a.</pos> <fld>(Math.)</fld> <def>Of, pertaining to, or included by, three lines; <as>as, <ex>trilinear</ex> coordinates</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Trilingual</ent><br/
<hw>Tri*lin"gual</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>trilinguis</ets>; <ets>tri-</ets> (see <er>Tri-</er>) + <ets>lingua</ets> tongue, language. See <er>Lingual</er>.]</ety> <def>Containing, or consisting of, three languages; expressed in three languages.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The much-noted Rosetta stone . . . bears upon its surface a <qex>trilingual</qex> inscription.</q> <rj><qau>I. Taylor.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Trilinguar</ent><br/
<hw>Tri*lin"guar</hw> <pr>(?)</pr>, <pos>a.</pos> <def>See <er>Trilingual</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Triliteral</ent><br/
<hw>Tri*lit"er*al</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[Pref. <ets>tri-</ets> + <ets>literal</ets>.]</ety> <def>Consisting of three letters; trigrammic; <as>as, a <ex>triliteral</ex> root or word</as>.</def> -- <def2><pos>n.</pos> <def>A triliteral word.</def></def2><br/
[<source>1913 Webster</source>]</p>

<p><ent>Triliteralism</ent><br/
<hw>Tri*lit"er*al*ism</hw> <pr>(?)</pr>, <pos>n.</pos> <def>Same as <er>Triliterality</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Triliteralness</ent><br/
<ent>Triliterality</ent><br/
<mhw>{ <hw>Tri*lit`er*al"i*ty</hw> <pr>(?)</pr>, <hw>Tri*lit"er*al*ness</hw> <pr>(?)</pr>, }</mhw> <pos>n.</pos> <def>The quality of being triliteral; <as>as, the <ex>triliterality</ex> of Hebrew roots</as>.</def>  <rj><au>W. D. Whitney.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Trilith</ent><br/
<hw>Tri"lith</hw> <pr>(?)</pr>, <pos>n.</pos> <def>Same as <er>Trilithon</er>.</def>  <rj><au>Mollett.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Trilithic</ent><br/
<hw>Tri*lith"ic</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Pertaining to a trilith.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Trilithon</ent><br/
<hw>Tril"i*thon</hw> <pr>(tr<icr/l"<icr/*th<ocr/n)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Trilithons</plw> <pr>(#)</pr>.</plu> <ety>[NL., fr. Gr.<?/ of or with three stones; <?/ (see <er>Tri-</er>) + <?/ stone.]</ety> <fld>(Archaeol.)</fld> <def>A monument consisting of three stones; especially, such a monument forming a kind of doorway, as among the ancient Celts.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Trill</ent><br/
<hw>Trill</hw> <pr>(?)</pr>, <pos>v. i.</pos> <ety>[OE. <ets>trillen</ets> to roll, turn round; of Scand. origin; cf. Sw. <ets>trilla</ets> to roll, Dan. <ets>trilde</ets>, Icel. <ets><thorn/yrla</ets> to whirl, and E. <ets>thrill</ets>.  Cf. <er>Thrill</er>.]</ety> <def>To flow in a small stream, or in drops rapidly succeeding each other; to trickle.</def>  <rj><au>Sir W. Scott.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>And now and then an ample tear <qex>trilled</qex> down<br/
Her delicate cheek.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Whispered sounds<br/
Of waters, <qex>trilling</qex> from the riven stone.</q> <rj><qau>Glover.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Trill</ent><br/
<hw>Trill</hw> <pr>(?)</pr>, <pos>v. t.</pos> <ety>[OE. <ets>trillen</ets>; cf. Sw. <ets>trilla</ets> to roll.]</ety> <def>To turn round; to twirl.</def> <mark>[Obs.]</mark>  <rj><au>Gascoigne.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Bid him descend and <qex>trill</qex> another pin.</q> <rj><qau>Chaucer.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Trill</ent><br/
<hw>Trill</hw>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Trilled</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Trilling</conjf>.]</vmorph> <ety>[It. <ets>trillare</ets>; probably of imitative origin.]</ety> <def>To impart the quality of a trill to; to utter as, or with, a trill; <as>as, to <ex>trill</ex> the <it>r</it>; to <ex>trill</ex> a note.</as></def><br/
[<source>1913 Webster</source>]</p>

<p><q>The sober-suited songstress <qex>trills</qex> her lay.</q> <rj><qau>Thomson.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Trill</ent><br/
<hw>Trill</hw>, <pos>v. i.</pos> <def>To utter trills or a trill; to play or sing in tremulous vibrations of sound; to have a trembling sound; to quaver.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>To judge of <qex>trilling</qex> notes and tripping feet.</q> <rj><qau>Dryden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Trill</ent><br/
<hw>Trill</hw>, <pos>n.</pos> <ety>[It. <ets>trillo</ets>, fr. <ets>trillare</ets>. See <er>Trill</er> to shake.]</ety><br/
[<source>1913 Webster</source>]</p>

<p><sn>1.</sn> <def>A sound, of consonantal character, made with a rapid succession of partial or entire intermissions, by the vibration of some one part of the organs in the mouth -- tongue, uvula, epiglottis, or lip -- against another part; <as>as, the <ex>r</ex> is a <ex>trill</ex> in most languages</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>The action of the organs in producing such sounds; <as>as, to give a <ex>trill</ex> to the tongue</as>. d</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <fld>(Mus.)</fld> <def>A shake or quaver of the voice in singing, or of the sound of an instrument, produced by the rapid alternation of two contiguous tones of the scale; <as>as, to give a <ex>trill</ex> on the high C</as>. See <er>Shake</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Trillachan</ent><br/
<hw>Tril"la*chan</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Zool.)</fld> <def>The oyster catcher.</def> <mark>[Prov. Eng.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Trilling</ent><br/
<hw>Tril"ling</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Cf. G. <ets>drilling</ets>.]</ety> <sn>1.</sn> <def>One of tree children born at the same birth.</def>  <rj><au>Wright.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Crystallog.)</fld> <def>A compound crystal, consisting of three individuals.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Trillion</ent><br/
<hw>Tril"lion</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F. <ets>trillion</ets>, formed from the pref. <ets>tri-</ets> in imitation of <ets>million</ets> a million.  Cf. <er>Billion</er>.]</ety> <def>According to the French notation, which is used upon the Continent generally and in the United States, the number expressed by a unit with twelve ciphers annexed; a million millions; according to the English notation, the number produced by involving a million to the third power, or the number represented by a unit with eighteen ciphers annexed. See the Note under <er>Numeration</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Trillium</ent><br/
||<hw>Tril"li*um</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[NL.; cf. L. <ets>trilix</ets> triple-woven, triple.]</ety> <fld>(Bot.)</fld> <def>A genus of liliaceous plants; the three-leaved nightshade; -- so called because all the parts of the plant are in threes.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Trillo</ent><br/
||<hw>Tril"lo</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[It. See <er>Trill</er>.]</ety> <fld>(Mus.)</fld> <def>A trill or shake. See <er>Trill</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Trilobate</ent><br/
<hw>Tri*lo"bate</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[Pref. <ets>tri-</ets> + <ets>lobate</ets>.]</ety> <def>Having three lobes.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Trilobation</ent><br/
<hw>Tri`lo*ba"tion</hw> <pr>(?)</pr>, <pos>n.</pos> <def>The state of being trilobate.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Trilobed</ent><br/
<hw>Tri"lobed</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[Pref. <ets>tri-</ets> + <ets>lobe</ets>.]</ety> <def>Same as <er>Trilobate</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Trilobita</ent><br/
||<hw>Tri`lo*bi"ta</hw> <pr>(?)</pr>, <pos>n. pl.</pos> <ety>[NL.]</ety> <fld>(Paleon.)</fld> <def>An extinct order of arthropods comprising the trilobites.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Trilobite</ent><br/
<hw>Tri"lo*bite</hw> <pr>(tr<imac/"l<osl/*b<imac/t)</pr>, <pos>n.</pos> <ety>[Cf. F. <ets>trilobite</ets>. See <er>Trilobate</er>.]</ety> <fld>(Paleon.)</fld> <def>Any one of numerous species of extinct arthropods belonging to the order Trilobita. Trilobites were very common in the Silurian and Devonian periods, but became extinct at the close of the Paleozoic. So named from the three lobes usually seen on each segment.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Trilobitic</ent><br/
<hw>Tri`lo*bit"ic</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Of, pertaining to or containing, trilobites; <as>as, <ex>trilobitic</ex> rocks</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Trilocular</ent><br/
<hw>Tri*loc"u*lar</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[Pref. <ets>tri-</ets> + <ets>locular</ets>: cf. F. <ets>triloculaire</ets>.]</ety> <def>Having three cells or cavities; <as>as, a <ex>trilocular</ex> capsule; a <ex>trilocular</ex> heart</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Trilogy</ent><br/
<hw>Tril"o*gy</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Gr. <grk>trilogi`a</grk>; pref. <grk>tri-</grk> (see <er>Tri-</er>) + <grk>lo`gos</grk> speech, discourse: cf. F. <ets>trilogie</ets>.]</ety> <def>A series of three dramas which, although each of them is in one sense complete, have a close mutual relation, and form one historical and poetical picture. Shakespeare's <ldquo/ Henry VI.<rdquo/ is an example.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>On the Greek stage, a drama, or acted story, consisted in reality of three dramas, called together a <qex>trilogy</qex>, and performed consecutively in the course of one day.</q> <rj><qau>Coleridge.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Triluminous</ent><br/
<ent>Triluminar</ent><br/
<mhw>{ <hw>Tri*lu"mi*nar</hw> <pr>(?)</pr>, <hw>Tri*lu"mi*nous</hw> <pr>(?)</pr>, }</mhw> <pos>a.</pos> <ety>[Pref. <ets>tri-</ets> + L. <ets>lumen</ets>, <ets>luminis</ets>, light.]</ety> <def>Having three lights</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Trim</ent><br/
<hw>Trim</hw> <pr>(?)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Trimmed</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Trimming</conjf>.]</vmorph> <ety>[OE. <ets>trimen</ets>, <ets>trumen</ets>, AS. <ets>trymian</ets>, <ets>trymman</ets>, to prepare, dispose, make strong, fr. <ets>trum</ets> firm, strong; of uncertain origin.]</ety> <sn>1.</sn> <def>To make trim; to put in due order for any purpose; to make right, neat, or pleasing; to adjust.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The hermit <qex>trimmed</qex> his little fire.</q> <rj><qau>Goldsmith.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To dress; to decorate; to adorn; to invest; to embellish; <as>as, to <ex>trim</ex> a hat</as>.</def>
<-- to <ex>trim</ex> a Christmas tree. --><br/
[<source>1913 Webster</source>]</p>

<p><q>A rotten building newly <qex>trimmed</qex> over.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>I was <qex>trimmed</qex> in Julia's gown.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>To make ready or right by cutting or shortening; to clip or lop; to curtail; <as>as, to <ex>trim</ex> the hair; to <ex>trim</ex> a tree</as>.</def> <ldquo/ And <xex>trimmed</xex> the cheerful lamp.<rdquo/  <rj><au>Byron.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <fld>(Carp.)</fld> <def>To dress, as timber; to make smooth.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <fld>(Naut.)</fld> <sd>(a)</sd> <def>To adjust, as a ship, by arranging the cargo, or disposing the weight of persons or goods, so equally on each side of the center and at each end, that she shall sit well on the water and sail well; <as>as, to <ex>trim</ex> a ship, or a boat</as>.</def> <sd>(b)</sd> <def>To arrange in due order for sailing; <as>as, to <ex>trim</ex> the sails</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>6.</sn> <def>To rebuke; to reprove; also, to beat.</def> <mark>[Colloq.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>To trim in</b></col> <fld>(Carp.)</fld>, <cd>to fit, as a piece of timber, into other work.</cd> -- <col><b>To trim up</b></col>, <cd>to dress; to put in order.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><q>I found her <qex>trimming up</qex> the diadem<br/
On her dead mistress.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Trim</ent><br/
<hw>Trim</hw> <pr>(?)</pr>, <pos>v. i.</pos> <def>To balance; to fluctuate between parties, so as to appear to favor each.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Trim</ent><br/
<hw>Trim</hw>, <pos>n.</pos> <sn>1.</sn> <def>Dress; gear; ornaments.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Seeing him just pass the window in his woodland <qex>trim</qex>.</q> <rj><qau>Sir W. Scott.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Order; disposition; condition; <as>as, to be in good <ex>trim</ex></as>.</def> <ldquo/ The <xex>trim</xex> of an encounter.<rdquo/  <rj><au>Chapman.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>The state of a ship or her cargo, ballast, masts, etc., by which she is well prepared for sailing.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <fld>(Arch)</fld> <def>The lighter woodwork in the interior of a building; especially, that used around openings, generally in the form of a molded architrave, to protect the plastering at those points.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>In ballast trim</b></col> <fld>(Naut.)</fld>, <cd>having only ballast on board.</cd> <au>R. H. Dana, Jr.</au> -- <col><b>Trim of the masts</b></col> <fld>(Naut.)</fld>, <cd>their position in regard to the ship and to each other, as near or distant, far forward or much aft, erect or raking.</cd> -- <col><b>Trim of sails</b></col> <fld>(Naut.)</fld>, <cd>that adjustment, with reference to the wind, witch is best adapted to impel the ship forward.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Trim</ent><br/
<hw>Trim</hw>, <pos>a.</pos> <amorph>[<pos>Compar.</pos> <adjf>Trimmer</adjf> <pr>(?)</pr>; <pos>superl.</pos> <adjf>Trimmest</adjf>.]</amorph> <ety>[See <er>Trim</er>, <pos>v. t.</pos>]</ety> <def>Fitly adjusted; being in good order., or made ready for service or use; firm; compact; snug; neat; fair; <as>as, the ship is <ex>trim</ex>, or <ex>trim</ex> built; everything about the man is <ex>trim</ex>; a person is <ex>trim</ex> when his body is well shaped and firm; his dress is <ex>trim</ex> when it fits closely to his body, and appears tight and snug; a man or a soldier is <ex>trim</ex> when he stands erect.</as></def><br/
[<source>1913 Webster</source>]</p>

<p><q>With comely carriage of her countenance <qex>trim</qex>.</q> <rj><qau>Spenser.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>So deemed I till I viewed their <qex>trim</qex> array<br/
Of boats last night.</q> <rj><qau>Trench.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Trimaculated</ent><br/
<hw>Tri*mac"u*la`ted</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[Pref. <ets>tri-</ets> + <ets>maculated</ets>.]</ety> <def>Marked with three spots, or maculae.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Trimellic</ent><br/
<hw>Tri*mel"lic</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[Pref. <ets>tri-</ets> + L. <ets>mel</ets>, gen. <ets>mellis</ets>, honey.]</ety> <fld>(Chem.)</fld> <def>Of, pertaining to, or designating, a certain tribasic acid (called also <xex>trimellitic acid</xex>) metameric with trimesitic acid.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Trimembral</ent><br/
<hw>Tri*mem"bral</hw>, <pos>a.</pos> <ety>[L. <ets>trimembris</ets> triplemembered. See <er>Tri-</er>, and <er>Member</er>.]</ety> <def>Having, or consisting of, three members.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Trimera</ent><br/
||<hw>Tri"me*ra</hw> <pr>(?)</pr>, <pos>n. pl.</pos> <ety>[NL. See <er>Tri-</er>, and <er>-mere</er>.]</ety> <fld>(Zool.)</fld> <def>A division of Coleoptera including those which have but three joints in the tarsi.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Trimeran</ent><br/
<hw>Tri"mer*an</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Zool.)</fld> <def>One of the Trimera. Also used adjectively.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Trimerous</ent><br/
<hw>Tri"mer*ous</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[See <er>Trimera</er>.]</ety> <fld>(Bot.)</fld> <def>Having the parts in threes.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Trimesitic</ent><br/
<hw>Tri`me*sit"ic</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[Pref. <ets>tri-</ets> + <ets>mesit</ets>ylene + <ets>-ic</ets>.]</ety> <fld>(Chem.)</fld> <def>Of, pertaining to, or designating, a tribasic acid, <chform>C6H3.(CO2)3</chform>, of the aromatic series, obtained, by the oxidation of mesitylene, as a white crystalline substance.</def> <altsp>[Written also <asp>trimesic</asp>.]</altsp><br/
[<source>1913 Webster</source>]</p>

<p><ent>Trimester</ent><br/
<hw>Tri*mes"ter</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>trimestris</ets> of three months; <ets>tri-</ets> (see <er>Tri-</er>) + <ets>mensis</ets> month: cf. F. <ets>trimestre</ets>.]</ety> <def>A term or period of three months.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Trimestral</ent><br/
<hw>Tri*mes"tral</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Trimestrial.</def>  <rj><au>Southey.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Trimestrial</ent><br/
<hw>Tri*mes"tri*al</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Of or pertaining to a trimester, or period of three months; occurring once in every three months; quarterly.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Trimeter</ent><br/
<hw>Trim"e*ter</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>trimetrus</ets>, Gr. <?/; <?/ (see <er>Tri-</er>) + <er><?/</er> <ets>measure</ets>.  See <er>Meter</er> measure.]</ety> <fld>(Pros.)</fld> <def>Consisting of three poetical measures.</def> -- <def2><pos>n.</pos> <def>A poetical division of verse, consisting of three measures.</def></def2>  <rj><au>Lowth.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Trimethyl</ent><br/
<hw>Tri*meth"yl</hw> <pr>(?)</pr>. <fld>(Chem.)</fld> <def>A prefix or combining form (also used adjectively) indicating <xex>the presence of three methyl groups</xex>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Trimethylamine</ent><br/
<hw>Tri*meth`yl*am"ine</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[<ets>Trimethyl-</ets> + <ets>amine</ets>.]</ety> <fld>(Chem.)</fld> <def>A colorless volatile alkaline liquid, <chform>N.(CH3)3</chform>, obtained from herring brine, beet roots, etc., with a characteristic herringlike odor. It is regarded as a substituted ammonia containing three methyl groups.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Trimethylene</ent><br/
<hw>Tri*meth"yl*ene</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Chem.)</fld> <def>A gaseous hydrocarbon, <chform>C3H6</chform>, isomeric with propylene and obtained from it indirectly. It is the base of a series of compounds analogous to the aromatic hydrocarbons.</def><-- usu. called cyclopropane, because the three carbons form a ring:<br/
[<source>1913 Webster</source>]</p>

<p>                   CH2
                  / \
               H2C---CH2
--><br/
[<source>1913 Webster</source>]</p>

<p><ent>Trimetric</ent><br/
<hw>Tri*met"ric</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[Pref. <ets>tri-</ets> + Gr. <?/ measure.]</ety> <fld>(Crystallog.)</fld> <def>Same as <er>Orthorhombic</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Trimetrical</ent><br/
<hw>Tri*met"ric*al</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Same as <er>Trimeter</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Trimly</ent><br/
<hw>Trim"ly</hw> <pr>(?)</pr>, <pos>adv.</pos> <def>In a trim manner; nicely.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Trimmer</ent><br/
<hw>Trim"mer</hw> <pr>(?)</pr>, <pos>n.</pos> <sn>1.</sn> <def>One who trims, arranges, fits, or ornaments.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>One who does not adopt extreme opinions in politics, or the like; one who fluctuates between parties, so as to appear to favor each; a timeserver.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Thus Halifax was a <qex>trimmer</qex> on principle.</q> <rj><qau>Macaulay.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>An instrument with which trimming is done.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <fld>(Arch.)</fld> <def>A beam, into which are framed the ends of headers in floor framing, as when a hole is to be left for stairs, or to avoid bringing joists near chimneys, and the like. See <xex>Illust.</xex> of <er>Header</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <fld>(Coal Storage)</fld> <def>An apparatus used for piling the coal in gradually increasing piles made by building up at the point of the cone or top of the prism.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><-- p. 1540 --><br/
[<source>1913 Webster</source>]</p>

<p><ent>Trimming</ent><br/
<hw>Trim"ming</hw> <pr>(?)</pr>, <def><pos>a.</pos> from <er>Trim</er>, <pos>v.</pos></def><br/
[<source>1913 Webster</source>]</p>

<p><q>The Whigs are, essentially, an inefficient, <qex>trimming</qex>, halfway sort of a party.</q> <rj><qau>Jeffrey.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Trimming joist</b></col> <fld>(Arch.)</fld>, <cd>a joist into which timber trimmers are framed; a header. See <er>Header</er>.</cd>  <rj><au>Knight.</au></rj></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Trimming</ent><br/
<hw>Trim"ming</hw>, <pos>n.</pos> <sn>1.</sn> <def>The act of one who trims.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>That which serves to trim, make right or fitting, adjust, ornament, or the like; especially, the necessary or the ornamental appendages, as of a garment; hence, sometimes, the concomitants of a dish; a relish; -- usually in the plural<-- as, We had a turkey dinner with all the <qex>trimmings</qex>. -->.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>The act of reprimanding or chastisting; <as>as, to give a boy a <ex>trimming</ex></as>.</def> <mark>[Colloq.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Trimmingly</ent><br/
<hw>Trim"ming*ly</hw>, <pos>adv.</pos> <def>In a trimming manner.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Trimness</ent><br/
<hw>Trim"ness</hw>, <pos>n.</pos> <def>The quality or state of being trim; orderliness; compactness; snugness; neatness.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Trimorph</ent><br/
<hw>Tri"morph</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[See <er>Trimorphous</er>.]</ety> <fld>(Crystallog.)</fld> <def>A substance which crystallizes in three distinct forms, or which has three distinct physical states; also, any one of these distinct forms. See <er>Trimorphism</er>, 1.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Trimorphous</ent><br/
<ent>Trimorphic</ent><br/
<mhw>{ <hw>Tri*mor"phic</hw> <pr>(?)</pr>, <hw>Tri*mor"phous</hw> <pr>(?)</pr>, }</mhw> <pos>a.</pos> <ety>[Gr. <?/ three-formed; <?/ (see <er>Tri-</er>) + <grk>morfh`</grk> form.]</ety> <def>Of, pertaining to, or characterized by, trimorphism; -- contrasted with <contr>monomorphic</contr>, <contr>dimorphic</contr>, and <contr>polymorphic</contr>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Trimorphism</ent><br/
<hw>Tri*mor"phism</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[See <er>Trimorphic</er>.]</ety> <sn>1.</sn> <fld>(Crystallog.)</fld> <def>The property of crystallizing in three forms fundamentally distinct, as is the case with titanium dioxide, which crystallizes in the forms of <xex>rutile</xex>, <xex>octahedrite</xex>, and <xex>brookite</xex>. See <er>Pleomorphism</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Biol.)</fld> <def>The coexistence among individuals of the same species of three distinct forms, not connected, as a rule, by intermediate gradations; the condition among individuals of the same species of having three different shapes or proportions of corresponding parts; -- contrasted with <contr>polymorphism</contr>, and <xex>dimorphism</xex>.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Heterogonous trimporphism</b></col> <fld>(Bot.)</fld>, <cd>that condition in which flowers of plants of the same species have three different lengths of stamens, short, medium, and long, the blossoms of one individual plant having short and medium stamens and a long style, those of another having short and long stamens and a style of medium length, and those of a third having medium and long stamens and a short style, the style of each blossom thus being of a length not represented by its stamens.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Trimurti</ent><br/
||<hw>Tri*mur"ti</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Skr. <ets>trim<umac/rti</ets>; <ets>tri</ets> three + <ets>m<umac/rti</ets> body.]</ety> <fld>(Hindu Myth.)</fld> <def>The triad, or trinity, of Hindu gods, consisting of Brahma, the Creator, Vishnu, the Preserver, and Siva, the Destroyer.</def> <altsp>[Spelled also <asp>Trimurtti</asp>.]</altsp><br/
[<source>1913 Webster</source>]</p>

<p><ent>Trimyarian</ent><br/
<hw>Trim`y*a"ri*an</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Pref. <ets>tri</ets> + Gr. <?/, <?/, a muscle.]</ety> <fld>(Zool.)</fld> <def>A lamellibranch which has three muscular scars on each valve.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Trinal</ent><br/
<hw>Tri"nal</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>trinus</ets> triple, <ets>trini</ets> three each, fr. <ets>tres</ets>, <ets>tria</ets>, three. See <er>Three</er>, and cf. <er>Trine</er>, <pos>a. & n.</pos>, <er>Tern</er>, <pos>a.</pos>]</ety> <def>Threefold.</def> <ldquo/<xex>Trinal</xex> unity.<rdquo/  <rj><au>Milton.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>In their <qex>trinal</qex> triplicities on high.</q> <rj><qau>Spenser.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Trindle</ent><br/
<hw>Trin"dle</hw> <pr>(?)</pr>, <pos>v. t. & n.</pos> <def>See <er>Trundle</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Trine</ent><br/
<hw>Trine</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[See <er>Trinal</er>.]</ety> <def>Threefold; triple; <as>as, <ex>trine</ex> dimensions, or length, breadth, and thickness</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Trine</ent><br/
<hw>Trine</hw>, <pos>n.</pos> <ety>[F. <ets>trine</ets>, <ets>trin</ets>. See <er>Trinal</er>.]</ety> <sn>1.</sn> <fld>(Astrol.)</fld> <def>The aspect of planets distant from each other 120 degrees, or one third of the zodiac; trigon.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>In sextile, square, and <qex>trine</qex>.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A triad; trinity.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>A single <qex>trine</qex> of brazen tortoises.</q> <rj><qau>Mrs. Browning.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Eternal One, Almighty <qex>Trine</qex>!</q> <rj><qau>Keble.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Trine</ent><br/
<hw>Trine</hw>, <pos>v. t.</pos> <def>To put in the aspect of a trine.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>By fortune he [Saturn] was now to Venus <qex>trined</qex>.</q> <rj><qau>Dryden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Trinervate</ent><br/
<hw>Tri*nerv"ate</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[NL. <ets>trinervatus</ets>; pref. <ets>tri-</ets> + L.  <ets>nervus</ets> nerve.]</ety> <fld>(Bot.)</fld> <def>Having three ribs or nerves extending unbranched from the base to the apex; -- said of a leaf.</def>  <rj><au>Gray.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Trinerved</ent><br/
<ent>Trinerve</ent><br/
<mhw>{ <hw>Tri"nerve`</hw> <pr>(?)</pr>, <hw>Tri"nerved`</hw> <pr>(?)</pr>, }</mhw> <pos>a.</pos> <ety>[Pref. <ets>tri-</ets> + <ets>nerve</ets>.]</ety> <fld>(Bot.)</fld> <def>Same as <er>Trinervate</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tringa</ent><br/
||<hw>Trin"ga</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[NL.]</ety> <fld>(Zool.)</fld> <def>A genus of limicoline birds including many species of sandpipers. See <er>Dunlin</er>, <er>Knot</er>, and <er>Sandpiper</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tringle</ent><br/
<hw>Trin"gle</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F. <ets>tringle</ets>.]</ety> <def>A curtain rod for a bedstead.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tringoid</ent><br/
<hw>Trin"goid</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[<ets>Tringa</ets> + <ets>-oid</ets>.]</ety> <fld>(Zool.)</fld> <def>Of or pertaining to Tringa, or the Sandpiper family.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Trinitarian</ent><br/
<hw>Trin`i*ta"ri*an</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[Cf. F. <ets>trinitaire</ets>. See <er>Trinity</er>.]</ety> <def>Of or pertaining to the Trinity, the doctrine of the Trinity, or believers in that doctrine.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Trinitarian</ent><br/
<hw>Trin`i*ta"ri*an</hw>, <pos>n.</pos> <sn>1.</sn> <def>One who believes in the doctrine of the Trinity.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Eccl. Hist.)</fld> <def>One of a monastic order founded in Rome in 1198 by St. John of Matha, and an old French hermit, Felix of Valois, for the purpose of redeeming Christian captives from the Moslems.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Trinitarianism</ent><br/
<hw>Trin`i*ta"ri*an*ism</hw> <pr>(?)</pr>, <pos>n.</pos> <def>The doctrine of the Trinity; the doctrine that there are three distinct persons in the Godhead.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Trinitrocellulose</ent><br/
<hw>Tri*ni`tro*cel"lu*lose"</hw> <pr>(?)</pr>, <pos>n.</pos> <def>Gun cotton; -- so called because regarded as containing three nitro groups.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Trinitrophenol</ent><br/
<hw>Tri*ni`tro*phe"nol</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Chem.)</fld> <def>Picric acid.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Trinity</ent><br/
<hw>Trin"i*ty</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[OE. <ets>trinitee</ets>, F. <ets>trinit<eacute/</ets>, L. <ets>trinitas</ets>, fr. <ets>trini</ets> three each. See <er>Trinal</er>.]</ety> <sn>1.</sn> <fld>(Christian Theol.)</fld> <def>The union of three persons (the Father, the Son, and the Holy Ghost) in one Godhead, so that all the three are one God as to substance, but three persons as to individuality.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Any union of three in one; three units treated as one; a triad, as the Hindu trinity, or Trimurti.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>Any symbol of the Trinity employed in Christian art, especially the triangle.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Trinity House</b></col>, <cd>an institution in London for promoting commerce and navigation, by licensing pilots, ordering and erecting beacons, and the like.</cd> -- <col><b>Trinity Sunday</b></col>, <cd>the Sunday next after Whitsunday; -- so called from the feast held on that day in honor of the Holy Trinity.</cd> -- <col><b>Trinity term</b></col>. <fld>(Law)</fld> <cd>See the Note under <er>Term</er>, <pos>n.</pos>, 5.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Triniunity</ent><br/
<hw>Trin`i*u"ni*ty</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[See <er>Trinity</er>, and <er>Unity</er>.]</ety> <def>Triunity; trinity.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>As for terms of trinity, <qex>triniunity</qex>, . . . and the like, they reject them as scholastic notions.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Trink</ent><br/
<hw>Trink</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A kind of fishing net.</def> <mark>[Obs.]</mark>  <rj><au>Crabb.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Trinket</ent><br/
<hw>Trin"ket</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F. <ets>trinquet</ets> foremast, also, a certain sail, <ets>trinquette</ets> a triangular sail, or Sp. <ets>trinquete</ets> triangular.]</ety> <fld>(Naut.)</fld> <def>A three-cornered sail formerly carried on a ship's foremast, probably on a lateen yard.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Sailing always with the sheets of mainsail and <qex>trinket</qex> warily in our hands.</q> <rj><qau>Hakluyt.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Trinket</ent><br/
<hw>Trin"ket</hw>, <pos>n.</pos> <ety>[OE. <ets>trenket</ets> a sort of knife, hence, probably, a toy knife worn as an ornament; probably from an Old French dialectic form of <ets>trenchier</ets> to cut.  Cf. <er>Trench</er>, <pos>v. t.</pos>]</ety> <sn>1.</sn> <def>A knife; a cutting tool.</def>  <rj><au>Tusser.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A small ornament, as a jewel, ring, or the like.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>A thing of little value; a trifle; a toy.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Trinket</ent><br/
<hw>Trin"ket</hw>, <pos>v. i.</pos> <def>To give trinkets; hence, to court favor; to intrigue.</def> <mark>[Obs.]</mark>  <rj><au>South.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Trinketer</ent><br/
<hw>Trin"ket*er</hw> <pr>(?)</pr>, <pos>n.</pos> <def>One who trinkets.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Trinketry</ent><br/
<hw>Trin"ket*ry</hw> <pr>(?)</pr>, <pos>n.</pos> <def>Ornaments of dress; trinkets, collectively.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>No <qex>trinketry</qex> on front, or neck, or breast.</q> <rj><qau>Southey.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Trinkle</ent><br/
<hw>Trin"kle</hw> <pr>(?)</pr>, <pos>v. i.</pos> <def>To act secretly, or in an underhand way; to tamper.</def> <mark>[Obs.]</mark>  <rj><au>Wright.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Trinoctial</ent><br/
<hw>Tri*noc"tial</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>trinoctialis</ets> for three nights; <ets>tri-</ets> (see <er>Tri-</er>) + <ets>nox</ets>, <ets>noctis</ets>, night.]</ety> <def>Lasting during three nights; comprising three nights.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Trinodal</ent><br/
<hw>Tri*nod"al</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>trinodis</ets> three-knotted; <ets>tri-</ets> (see <er>Tri-</er>) + <ets>nodus</ets> knot.]</ety> <sn>1.</sn> <fld>(Bot.)</fld> <def>Having three knots or nodes; having three points from which a leaf may shoot; <as>as, a <ex>trinodal</ex> stem</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Geom.)</fld> <def>Having three nodal points.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Trinomial</ent><br/
<hw>Tri*no"mi*al</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Pref. <ets>tri-</ets> + <ets>-nomial</ets> as in <ets>binomial</ets>: cf. F. <ets>trin<ocir/me</ets>.]</ety> <fld>(Math.)</fld> <def>A quantity consisting of three terms, connected by the sign + or -; <as>as, <mathex>x + y + z</mathex>, or <mathex>ax + 2b - c<exp>2</exp></mathex></as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Trinomial</ent><br/
<hw>Tri*no"mi*al</hw>, <pos>a.</pos> <sn>1.</sn> <fld>(Math.)</fld> <def>Consisting of three terms; of or pertaining to trinomials; <as>as, a <ex>trinomial</ex> root</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Nat. Hist.)</fld> <def>Consisting of, or involving the use of, three terms; <as>as, a <ex>trinomial</ex> systematic name specifying the genus, species, and variety</as>.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Trinominal</ent><br/
<hw>Tri*nom"i*nal</hw> <pr>(?)</pr>, <pos>n. & a.</pos> <ety>[Pref. <ets>tri-</ets> + L. <ets>nomen</ets>, <ets>nominis</ets>, name: cf. L. <ets>trinominis</ets> three-named.]</ety> <fld>(Math.)</fld> <def>Trinomial.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Trinucleus</ent><br/
||<hw>Tri*nu"cle*us</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Pref. <ets>tri-</ets> + <ets>nucleus</ets>.]</ety> <fld>(Paleon.)</fld> <def>A genus of Lower Silurian trilobites in which the glabella and cheeks form three rounded elevations on the head.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Trio</ent><br/
<hw>Tri"o</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[It., fr. L. <ets>tres</ets>, <ets>tria</ets>, three: cf. F. <ets>trio</ets>, from the Italian. See <er>Three</er>.]</ety> <sn>1.</sn> <def>Three, considered collectively; three in company or acting together; a set of three; three united.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The <qex>trio</qex> were well accustomed to act together, and were linked to each other by ties of mutual interest.</q> <rj><qau>Dickens.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Mus.)</fld> <sd>(a)</sd> <def>A composition for three parts or three instruments.</def> <sd>(b)</sd> <def>The secondary, or episodical, movement of a minuet or scherzo, as in a sonata or symphony, or of a march, or of various dance forms; -- not limited to three parts or instruments.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Triobolary</ent><br/
<ent>Triobolar</ent><br/
<mhw>{ <hw>Tri*ob"o*lar</hw> <pr>(?)</pr>, <hw>Tri*ob"o*la*ry</hw> <pr>(?)</pr>, }</mhw> <pos>a.</pos> <ety>[LL. <ets>triobolaris</ets>, fr. L. <ets>triobolus</ets> a piece of three oboli, Gr. <?/. See <ets>Tri-</ets>, and <er>Obolus</er>.]</ety> <def>Of the value of three oboli; hence, mean; worthless.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>It may pass current . . . for a <qex>triobolar</qex> ballad.</q> <rj><qau>Cheyne.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Trioctile</ent><br/
<hw>Tri*oc"tile</hw> <pr>(-<ocr/k"t<icr/l)</pr>, <pos>n.</pos> <ety>[Pref. <ets>tri-</ets> + <ets>octile</ets>.]</ety> <fld>(Astrol.)</fld> <def>An aspect of two planets with regard to the earth when they are three octants, or three eighths of a circle, that is, 135 degrees, distant from each other.</def>  <rj><au>Hutton.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Trioecia</ent><br/
||<hw>Tri*<oe/"ci*a</hw> <pr>(-<emac/"sh<icr/*<adot/)</pr>, <pos>n. pl.</pos> <ety>[NL. See <er>Tri<oe/cious</er>.]</ety> <fld>(Bot.)</fld> <def>The third order of the Linnaean class Polygamia.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Trioecious</ent><br/
<hw>Tri*<oe/"cious</hw> <pr>(-sh<ucr/s)</pr>, <pos>a.</pos> <ety>[Pref. <ets>tri-</ets> + Gr. <?/ house.]</ety> <fld>(Bot.)</fld> <def>Having three sorts of flowers on the same or on different plants, some of the flowers being staminate, others pistillate, and others both staminate and pistillate; belonging to the order Tri<oe/cia.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Triole</ent><br/
<hw>Tri"ole</hw> <pr>(tr<emac/"<omac/l)</pr>, <pos>n.</pos> <ety>[See <er>Triolet</er>.]</ety> <fld>(Mus.)</fld> <def>Same as <er>Triplet</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Triolein</ent><br/
<hw>Tri*o"le*in</hw> <pr>(tr<isl/*<omac/"l<esl/*<icr/n)</pr>, <pos>n.</pos> <ety>[Pref. <ets>tri-</ets> + <ets>olein</ets>.]</ety> <fld>(Physiol. Chem.)</fld> <def>See <er>Olein</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Triolet</ent><br/
<hw>Tri"o*let</hw> <pr>(tr<imac/"<osl/*l<ecr/t)</pr>, <pos>n.</pos> <ety>[F. <ets>triolet</ets>. See <er>Trio</er>.]</ety> <def>A short poem or stanza of eight lines, in which the first line is repeated as the fourth and again as the seventh line, the second being, repeated as the eighth.</def>  <rj><au>Brande & C.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Trional</ent><br/
<hw>Tri"o*nal</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Pref. <ets>tri-</ets> + sulph<ets>onal</ets>; -- so called because it contains three ethyls.]</ety> <fld>(Chem.)</fld> <def>A compound similar to sulphonal, used as a hypnotic in medicine.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Trionychoidea</ent><br/
||<hw>Tri*on`y*choi"de*a</hw> <pr>(?)</pr>, <pos>n. pl.</pos> <ety>[NL. See <er>Trionyx</er>, and <er>-old</er>.]</ety> <fld>(Zool.)</fld> <def>A division of chelonians which comprises Trionyx and allied genera; -- called also <altname>Trionychoides</altname>, and <altname>Trionychina</altname>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Trionyx</ent><br/
||<hw>Tri*on"yx</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[NL., fr. Gr. <?/ (see <er>Tri-</er>) + <?/ a claw.]</ety> <fld>(Zool.)</fld> <def>A genus of fresh-water or river turtles which have the shell imperfectly developed and covered with a soft leathery skin.  They are noted for their agility and rapacity.  Called also <altname>soft tortoise</altname>, <altname>soft-shell tortoise</altname>, and <altname>mud turtle</altname>.</def><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ The common American species (<spn>Trionyx ferox</spn> syn. <spn>Aspidonectus ferox</spn>) becomes over a foot in length and is very voracious.  Similar species are found in Asia and Africa.</note><br/
[<source>1913 Webster</source>]</p>

<p><ent>Trior</ent><br/
<hw>Tri"or</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Law)</fld> <def>Same as <er>Trier</er>, 2 and 3.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Triose</ent><br/
<hw>Tri"ose</hw> <pr>(?)</pr>, <pos>n.</pos>  <ety>[<ets>Tri-</ets> + <ets>-ose</ets>.]</ety> <fld>(Chem.)</fld> <sd>(a)</sd> <def>A sugar derived from a trihydric alcohol</def>. <sd>(b)</sd> <def>A trisaccharide.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Trioxide</ent><br/
<hw>Tri*ox"ide</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Pref. <ets>tri-</ets> + <ets>oxide</ets>.]</ety> <fld>(Chem.)</fld> <def>An oxide containing three atoms of oxygen; <as>as, sulphur <ex>trioxide</ex>, <chform>SO3</chform></as>; -- formerly called <altname>tritoxide</altname>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Trip</ent><br/
<hw>Trip</hw> <pr>(tr<icr/p)</pr>, <pos>v. i.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Tripped</conjf> <pr>(tr<icr/pt)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Tripping</conjf>.]</vmorph> <ety>[OE. <ets>trippen</ets>; akin to D. <ets>trippen</ets>, Dan. <ets>trippe</ets>, and E. <ets>tramp</ets>. See <er>Tramp</er>.]</ety><br/
[<source>1913 Webster</source>]</p>

<p><sn>1.</sn> <def>To move with light, quick steps; to walk or move lightly; to skip; to move the feet nimbly; -- sometimes followed by <xex>it</xex>. See <er>It</er>, 5.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>This horse anon began to <qex>trip</qex> and dance.</q> <rj><qau>Chaucer.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Come, and <qex>trip</qex> it, as you go,<br/
On the light fantastic toe.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>She bounded by, and <qex>tripped</qex> so light<br/
They had not time to take a steady sight.</q> <rj><qau>Dryden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To make a brief journey or pleasure excursion; <as>as, to <ex>trip</ex> to Europe</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>To take a quick step, as when in danger of losing one's balance; hence, to make a false step; to catch the foot; to lose footing; to stumble.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>Fig.: To be guilty of a misstep; to commit an offense against morality, propriety, or rule; to err; to mistake; to fail.</def> <ldquo/Till his tongue <xex>trip</xex>.<rdquo/  <rj><au>Locke.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>A blind will thereupon comes to be led by a blind understanding; there is no remedy, but it must <qex>trip</qex> and stumble.</q> <rj><qau>South.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Virgil is so exact in every word that none can be changed but for a worse; he pretends sometimes to <qex>trip</qex>, but it is to make you think him in danger when most secure.</q> <rj><qau>Dryden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>What? dost thou verily <qex>trip</qex> upon a word?</q> <rj><qau>R. Browning.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Trip</ent><br/
<hw>Trip</hw>, <pos>v. t.</pos> <sn>1.</sn> <def>To cause to stumble, or take a false step; to cause to lose the footing, by striking the feet from under; to cause to fall; to throw off the balance; to supplant; -- often followed by <xex>up</xex>; <as>as, to <ex>trip</ex> up a man in wrestling</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The words of Hobbes's defense <qex>trip</qex> up the heels of his cause.</q> <rj><qau>Abp. Bramhall.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <mark>(Fig.)</mark>: <def>To overthrow by depriving of support; to put an obstacle in the way of; to obstruct; to cause to fail.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>To <qex>trip</qex> the course of law, and blunt the sword.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>To detect in a misstep; to catch; to convict; also called <altname>trip up</altname>.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>These her women can <qex>trip</qex> me if I err.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <fld>(Naut.)</fld> <sd>(a)</sd> <def>To raise (an anchor) from the bottom, by its cable or buoy rope, so that it hangs free.</def> <sd>(b)</sd> <def>To pull (a yard) into a perpendicular position for lowering it.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <fld>(Mach.)</fld> <def>To release, let fall, or set free, as a weight or compressed spring, as by removing a latch or detent; to activate by moving a release mechanism, often unintentionally; <as>as, to <ex>trip</ex> an alarm</as>.</def><br/
[<source>1913 Webster</source> <source>+PJC</source>]</p>

<p><ent>Trip</ent><br/
<hw>Trip</hw>, <pos>n.</pos> <sn>1.</sn> <def>A quick, light step; a lively movement of the feet; a skip.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>His heart bounded as he sometimes could hear the <qex>trip</qex> of a light female step glide to or from the door.</q> <rj><qau>Sir W. Scott.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A brief or rapid journey; an excursion or jaunt.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>I took a <qex>trip</qex> to London on the death of the queen.</q> <rj><qau>Pope.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>A false step; a stumble; a misstep; a loss of footing or balance. Fig.: An error; a failure; a mistake.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Imperfect words, with childish <qex>trips</qex>.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Each seeming <qex>trip</qex>, and each digressive start.</q> <rj><qau>Harte.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>A small piece; a morsel; a bit.</def> <mark>[Obs.]</mark> <ldquo/A <xex>trip</xex> of cheese.<rdquo/  <rj><au>Chaucer.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <def>A stroke, or catch, by which a wrestler causes his antagonist to lose footing.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>And watches with a <qex>trip</qex> his foe to foil.</q> <rj><qau>Dryden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q><qex>It is the sudden trip</qex> in wrestling that fetches a man to the ground.</q> <rj><qau>South.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>6.</sn> <fld>(Naut.)</fld> <def>A single board, or tack, in plying, or beating, to windward.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>7.</sn> <def>A herd or flock, as of sheep, goats, etc.</def> <mark>[Prov. Eng. & Scott.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><sn>8.</sn> <def>A troop of men; a host.</def> <mark>[Obs.]</mark>  <rj><au>Robert of Brunne.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>9.</sn> <fld>(Zool.)</fld> <def>A flock of widgeons.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tripalmitate</ent><br/
<hw>Tri*pal"mi*tate</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Pref. <ets>tri-</ets> + <ets>palmitate</ets>.]</ety> <fld>(Chem.)</fld> <def>A palmitate derived from three molecules of palmitic acid.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tripalmitin</ent><br/
<hw>Tri*pal"mi*tin</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Pref. <ets>tri-</ets> + <ets>palmitin</ets>.]</ety> <fld>(Physiol. Chem.)</fld> <def>See <er>Palmitin</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tripang</ent><br/
<hw>Tri*pang"</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Zool.)</fld> <def>See <er>Trepang</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Triparted</ent><br/
<hw>Tri"part`ed</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[Pref. <ets>tri-</ets> + <ets>parted</ets>.]</ety><br/
[<source>1913 Webster</source>]</p>

<p><sn>1.</sn> <fld>(Her.)</fld> <def>Parted into three piece; having three parts or pieces; -- said of the field or of a bearing; <as>as, a cross <ex>triparted</ex></as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Bot.)</fld> <def>Divided nearly to the base into three segments or lobes.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tripartible</ent><br/
<hw>Tri*part"i*ble</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Divisible into three parts.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tripartient</ent><br/
<hw>Tri*par"tient</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[See <er>Tripartite</er>.]</ety> <fld>(Arith.)</fld> <def>Dividing into three parts; -- said of a number which exactly divides another into three parts.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tripartite</ent><br/
<hw>Trip"ar*tite</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>tripartitus</ets>; <ets>tri-</ets> (see <er>Tri-</er>) + <ets>partitus</ets>, p. p. of <ets>partiri</ets> to part, to divide. See <er>Part</er>, <pos>v. i.</pos>]</ety> <sn>1.</sn> <def>Divided into three parts; triparted; <as>as, a <ex>tripartite</ex> leaf</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Having three corresponding parts or copies; <as>as, to make indentures <ex>tripartite</ex></as>.</def>  <rj><au>A. Smith.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>Made between three parties; <as>as, a <ex>tripartite</ex> treaty</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tripartitely</ent><br/
<hw>Trip`ar*tite*ly</hw>, <pos>adv.</pos> <def>In a tripartite manner.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tripartition</ent><br/
<hw>Trip`ar*ti"tion</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Cf. F. <ets>tripartition</ets>.]</ety> <def>A division by threes, or into three parts; the taking of a third part of any number or quantity.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tripaschal</ent><br/
<hw>Tri*pas"chal</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[Pref. <ets>tri-</ets> + <ets>paschal</ets>.]</ety> <def>Including three passovers.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tripe</ent><br/
<hw>Tripe</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[OE. <ets>tripe</ets>, F. <ets>tripe</ets>; of uncertain origin; cf. Sp. & Pg. <ets>tripa</ets>, It. <ets>trippa</ets>, OD. <ets>tripe</ets>, W. <ets>tripa</ets>, Armor. <ets>stripen</ets>.]</ety> <sn>1.</sn> <def>The large stomach of ruminating animals, when prepared for food.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>How say you to a fat <qex>tripe</qex> finely broiled ?</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>The entrails; hence, humorously or in contempt, the belly; -- generally used in the plural.</def>  <rj><au>Howell.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tripedal</ent><br/
<hw>Trip"e*dal</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>tripedalis</ets>; <ets>tri-</ets> (see <er>Tri-</er>) + <ets>pes</ets>, <ets>pedis</ets>, a foot.]</ety> <def>Having three feet.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tripe-de-roche</ent><br/
||<hw>Tripe`-de-roche"</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F.]</ety> <fld>(Bot.)</fld> <def>Same as Rock tripe, under <er>Rock</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tripel</ent><br/
<hw>Trip"el</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Min.)</fld> <def>Same as <er>Tripoli</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tripeman</ent><br/
<hw>Tripe"man</hw> <pr>(?)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>-men</plw> <pr>(<?/)</pr>.</plu> <def>A man who prepares or sells tripe.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tripennate</ent><br/
<hw>Tri*pen"nate</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[Pref. <ets>tri-</ets> + <ets>pennate</ets>.]</ety> <fld>(Bot.)</fld> <def>Same as <er>Tripinnate</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><-- p. 1541 --><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tripersonal</ent><br/
<hw>Tri*per"son*al</hw> <pr>(tr<isl/*p<etil/r"s<ucr/n*<ait/l)</pr>, <pos>a.</pos> <ety>[Pref. <ets>tri-</ets> + <ets>personal</ets>.]</ety> <def>Consisting of three persons.</def>  <rj><au>Milton.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tripersonalist</ent><br/
<hw>Tri*per"son*al*ist</hw>, <pos>n.</pos> <def>A Trinitarian.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tripersonality</ent><br/
<hw>Tri*per`son*al"i*ty</hw> <pr>(?)</pr>, <pos>n.</pos> <def>The state of existing as three persons in one Godhead; trinity.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tripery</ent><br/
<hw>Trip"er*y</hw> <pr>(tr<imac/p"<etil/r*<ycr/)</pr>, <pos>n.</pos> <ety>[Cf. F. <ets>triperie</ets>.]</ety> <def>A place where tripe is prepared or sold.</def>  <rj><au>London Quart. Rev.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tripestone</ent><br/
<hw>Tripe"stone`</hw> <pr>(tr<imac/p"st<omac/n`)</pr>, <pos>n.</pos> <fld>(Min.)</fld> <def>A variety of anhydrite composed of contorted plates fancied to resemble pieces of tripe.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tripetaloid</ent><br/
<hw>Tri*pet"al*oid</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[Pref. <ets>tri-</ets> + <ets>petaloid</ets>.]</ety> <fld>(Bot.)</fld> <def>Having the form or appearance of three petals; appearing as if furnished with three petals.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tripetalous</ent><br/
<hw>Tri*pet"al*ous</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[Pref. <ets>tri-</ets> + <ets>petalous</ets>: cf. F. <ets>trip<eacute/tale</ets>.]</ety> <fld>(Bot.)</fld> <def>Having three petals, or flower leaves; three-petaled.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Trip hammer</ent><br/
<hw>Trip" ham`mer</hw> <pr>(?)</pr>. <def>A tilt hammer.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Triphane</ent><br/
<hw>Tri"phane</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Gr. <?/ appearing three-fold; <?/ (see <er>Tri-</er>) + <er><?/</er> to appear cf. F. <ets>triphane</ets>.]</ety> <fld>(Min.)</fld> <def>Spodumene.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Triphthong</ent><br/
<hw>Triph"thong</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Pref. <ets>tri-</ets> + <ets>-phthong</ets>, as in <ets>diphthong</ets>: cf. F. <ets>triphthonque</ets>.]</ety> <fld>(Orthoepy)</fld> <def>A combination of three vowel sounds in a single syllable, forming a simple or compound sound; also, a union of three vowel characters, representing together a single sound; a trigraph; <as>as, <ex>eye</ex>, <ex>-ieu</ex> in <ex>adieu</ex>, <ex>-eau</ex> in <ex>beau</ex>, are examples of <ex>triphthongs</ex></as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Triphthongal</ent><br/
<hw>Triph*thon"gal</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Of or pertaining to a triphthong; consisting of three vowel sounds pronounced together in a single syllable.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Triphyline</ent><br/
<hw>Triph"y*line</hw> <pr>(?)</pr>, <pos>n.</pos> <def>Triphylite.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Triphylite</ent><br/
<hw>Triph"y*lite</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Pref. <ets>tri-</ets> + Gr. <?/ a family, class. So called in allusion to its containing three phosphates.]</ety> <fld>(Min.)</fld> <def>A mineral of a grayish-green or bluish color, consisting of the phosphates of iron, manganese, and lithia.</def><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ A salmon-colored or clove-brown variety containing but little iron is known as <xex>lithiophilite</xex>.</note><br/
[<source>1913 Webster</source>]</p>

<p><ent>Triphyllous</ent><br/
<hw>Triph"yl*lous</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[Gr. <?/; <?/ (see <er>Tri-</er>) + <er><?/</er> a leaf: cf. F. <ets>triphylle</ets>.]</ety> <fld>(Bot.)</fld> <def>Having three leaves; three-leaved.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tripinnate</ent><br/
<hw>Tri*pin"nate</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[Pref. <ets>tri-</ets> + <ets>pinnate</ets>.]</ety> <fld>(Bot.)</fld> <def>Having bipinnate leaflets arranged on each side of a rhachis.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tripinnatifid</ent><br/
<hw>Tri`pin*nat"i*fid</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[Pref. <ets>tri-</ets> + <ets>pinnatifid</ets>.]</ety> <fld>(Bot.)</fld> <def>Thrice pinnately cleft; -- said of a pinnatifid leaf when its segments are pinnatifid, and the subdivisions of these also are pinnatifid.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tripitaka</ent><br/
||<hw>Tri*pit"a*ka</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Skr. <ets>tripi<tsdot/aka</ets>.]</ety> <def>The three divisions, or <ldquo/baskets<rdquo/ (<xex>pitakas</xex>), of buddhist scriptures, -- the Vinayapitaka <ety>[Skr. <ets>Vinayapi<tsdot/aka</ets>]</ety> , or Basket of Discipline; Suttapitaka [Pali] , or Basket of Discourses; and Abhidhammapitaka [Pali] , or Basket of Metaphysics.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Triplasian</ent><br/
<hw>Tri*pla"sian</hw>, <pos>a.</pos> <ety>[Gr. <?/ thrice as many.]</ety> <def>Three-fold; triple; treble.</def> <mark>[Obs.]</mark>  <rj><au>Cudworth.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Triple</ent><br/
<hw>Tri"ple</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>triplus</ets>; <ets>tri-</ets> (see <er>Tri-</er>) + <ets>-plus</ets>, as in <ets>duplus</ets> double: cf. F. <ets>triple</ets>. See <er>Double</er>, and cf. <er>Treble</er>.]</ety> <sn>1.</sn> <def>Consisting of three united; multiplied by three; threefold; <as>as, a <ex>triple</ex> knot; a <ex>triple</ex> tie</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>By thy <qex>triple</qex> shape as thou art seen.</q> <rj><qau>Dryden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Three times repeated; treble. See <er>Treble</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>One of three; third.</def> <mark>[Obs.]</mark>  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Triple crown</b></col>, <cd>the crown, or tiara, of the pope. See <er>Tiara</er>, 2.</cd><-- (b) (Sport) The championship in three specific categories of a sport in one year, if won by a single contender; as (Baseball) The championship in batting average, home runs, and runs batted in (RBI's) in one league. --> -- <col><b>Triple-expansion steam engine</b></col>, <cd>a compound steam engine in which the same steam performs work in three cylinders successively.</cd> -- <col><b>Triple measure</b></col> <fld>(Mus.)</fld>, <cd>a measure of tree beats of which first only is accented.</cd> -- <col><b>Triple ratio</b></col> <fld>(Math.)</fld>, <cd>a ratio which is equal to 3.</cd> -- <col><b>Triple salt</b></col> <fld>(Chem.)</fld>, <cd>a salt containing three distinct basic atoms as radicals; thus, microcosmic salt is a <xex>triple salt</xex>.</cd> -- <col><b>Triple star</b></col> <fld>(Astron.)</fld>, <cd>a system of three stars in close proximity.</cd> -- <col><b>Triple time</b></col> <fld>(Mus.)</fld>, <cd>that time in which each measure is divided into three equal parts.</cd> -- <col><b>Triple valve</b></col>, <cd>in an automatic air brake for railroad cars, the valve under each car, by means of which the brake is controlled by a change of pressure in the air pipe leading from the locomotive.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Triple</ent><br/
<hw>Tri"ple</hw>, <pos>v. i.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Tripled</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Tripling</conjf> <pr>(?)</pr>.]</vmorph> <ety>[Cf. F. <ets>tripler</ets>. See <er>Triple</er>, <pos>a.</pos>]</ety> <def>To make threefold, or thrice as much or as many; to treble; <as>as, to <ex>triple</ex> the tax on coffee</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Triple-crowned</ent><br/
<hw>Tri"ple-crowned`</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Having three crowns; wearing the triple crown, as the pope.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Triple-headed</ent><br/
<hw>Tri"ple-head`ed</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Having three heads; three-headed; <as>as, the <ex>triple-headed</ex> dog Cerberus</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Triplet</ent><br/
<hw>Trip"let</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[From <er>Triple</er>.]</ety> <sn>1.</sn> <def>A collection or combination of three of a kind; three united.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Poetry)</fld> <def>Three verses rhyming together.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <fld>(Mus.)</fld> <def>A group of three notes sung or played in the tree of two.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <pluf>pl.</pluf> <def>Three children or offspring born at one birth.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Triple-tail</ent><br/
<hw>Tri"ple-tail`</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Zool.)</fld> <def>An edible fish (<spn>Lobotes Surinamensis</spn>) found in the warmer parts of all the oceans, and common on the southern and middle coasts of the United States. When living it is silvery gray, and becomes brown or blackish when dead. Its dorsal and anal fins are long, and extend back on each side of the tail. It has large silvery scales which are used in the manufacture of fancy work. Called also, locally, <altname>black perch</altname>, <altname>grouper</altname>, and <altname>flasher</altname>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Triplex</ent><br/
<hw>Tri"plex</hw> <pr>(?)</pr>, <pos>a.</pos> <fld>(Mach.)</fld> <def>Having three principal operative parts or motions, so as to produce a three-fold effect.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Triplicate</ent><br/
<hw>Trip"li*cate</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>triplicatus</ets>, p. p. of <ets>triplicare</ets> to triple, treble; <ets>tri-</ets> (see <er>Tri-</er>) + <ets>plicare</ets> to fold. See <er>Ply</er>, <pos>v. t.</pos>]</ety> <def>Made thrice as much; threefold; tripled.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Triplicate ratio</b></col> <fld>(Math.)</fld>, <cd>the ratio of the cubes of two quantities; thus, the <xex>triplicate ratio</xex> of <it>a</it> to <it>b</it> is <mathex>a<exp>3</exp>: b<exp>3</exp></mathex>.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Triplicate</ent><br/
<hw>Trip"li*cate</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A third thing corresponding to two others of the same kind.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Triplicate-ternate</ent><br/
<hw>Trip"li*cate-ter`nate</hw>, <pos>a.</pos> <fld>(Bot.)</fld> <def>Triternate.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Triplication</ent><br/
<hw>Trip"li*ca"tion</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>triplicatio</ets>: cf. F. <ets>triplication</ets>.]</ety> <sn>1.</sn> <def>The act of tripling, or making threefold, or adding three together.</def>  <rj><au>Glanvill.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Civil Law)</fld> <def>Same as <er>Surrejoinder</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Triplicity</ent><br/
<hw>Tri*plic"i*ty</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Cf. F. <ets>triplicit<eacute/</ets>, fr. L. <ets>triplex</ets>, <ets>triplicis</ets>, threefold. See <er>Triplicate</er>, <pos>a.</pos>]</ety> <def>The quality or state of being triple, or threefold; trebleness.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>In their trinal <qex>triplicities</qex> on high.</q> <rj><qau>Spenser.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Triplicostate</ent><br/
<hw>Trip`li*cos"tate</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[<ets>Triple</ets> + <ets>costate</ets>.]</ety> <fld>(Bot.)</fld> <def>Three-ribbed.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Triplite</ent><br/
<hw>Trip"lite</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Cf. F. <ets>triplite</ets>. See <er>Triple</er>.]</ety> <fld>(Min.)</fld> <def>A mineral of a dark brown color, generally with a fibrous, massive structure. It is a fluophosphate of iron and manganese.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Triploblastic</ent><br/
<hw>Trip`lo*blas"tic</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[Gr. <?/ threefold + <ets>-blast + -ic</ets>.]</ety> <fld>(Biol.)</fld> <def>Of, pertaining to, or designating, that condition of the ovum in which there are three primary germinal layers, or in which the blastoderm splits into three layers.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Triploidite</ent><br/
<hw>Trip*loi"dite</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Min.)</fld> <def>A manganese phosphate near triplite, but containing hydroxyl instead of fluorine.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Triply</ent><br/
<hw>Trip"ly</hw> <pr>(?)</pr>, <pos>adv.</pos> <def>In a triple manner.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tripmadam</ent><br/
<hw>Trip"mad`am</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F. <ets>tripe-madame</ets>, <ets>trique-madame</ets>.]</ety> <fld>(Bot.)</fld> <def>Same as <er>Prickmadam</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tripod</ent><br/
<hw>Tri"pod</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>tripus</ets>, <ets>-odis</ets>, Gr. <?/; <?/ (see <er>Tri-</er>) + <?/, <?/, foot. See <er>Foot</er>, and cf. <er>Tripos</er>, <er>Trivet</er>.]</ety> <sn>1.</sn> <def>Any utensil or vessel, as a stool, table, altar, caldron, etc., supported on three feet.</def><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ On such, a stool, in the temple of Apollo at Delphi, the Pythian priestess sat while giving responses to those consulting the Delphic oracle.</note><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A three-legged frame or stand, usually jointed at top, for supporting a theodolite, compass, telescope, camera, or other instrument.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><mcol><col><b>Tripod of life</b></col>, <it>or</it> <col><b>Vital tripod</b></col></mcol> <fld>(Physiol.)</fld>, <cd>the three organs, the heart, lungs, and brain; -- so called because their united action is necessary to the maintenance of life.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tripodian</ent><br/
<hw>Tri*po"di*an</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Mus.)</fld> <def>An ancient stringed instrument; -- so called because, in form, it resembled the Delphic tripod.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tripody</ent><br/
<hw>Trip"o*dy</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Pref. <ets>tri-</ets> + <ets>-pody</ets>, as in <ets>dipody</ets>.]</ety> <fld>(Pros.)</fld> <def>Three metrical feet taken together, or included in one measure.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tripoli</ent><br/
<hw>Trip"o*li</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Min.)</fld> <def>An earthy substance originally brought from Tripoli, used in polishing stones and metals. It consists almost wholly of the siliceous shells of diatoms.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tripoline</ent><br/
<hw>Trip"o*line</hw> <pr>(?)</pr>, <pos>a.</pos> <sn>1.</sn> <def>Of or pertaining to Tripoli or its inhabitants; Tripolitan.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Of or pertaining to tripoli, the mineral.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tripolitan</ent><br/
<hw>Tri*pol"i*tan</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Of or pertaining to Tripoli or its inhabitants; Tripoline.</def> -- <def2><pos>n.</pos>  <def>A native or inhabitant of Tripoli.</def></def2><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tripos</ent><br/
<hw>Tri"pos</hw> <pr>(?)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Triposes</plw> <pr>(#)</pr>.</plu> <ety>[Gr. <?/ a tripod. See <er>Tripod</er>.]</ety> <sn>1.</sn> <def>A tripod.</def> <mark>[Obs.]</mark>  <rj><au>Dryden.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A university examination of questionists, for honors; also, a tripos paper; one who prepares a tripos paper.</def> <mark>[Cambridge University, Eng.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Classical tripos examination</b></col>, <cd>the final university examination for classical honors, optional to all who have taken the mathematical honors.</cd> <au>C. A. Bristed.</au> -- <col><b>Tripos paper</b></col>, <cd>a printed list of the successful candidates for mathematical honors, accompanied by a piece in Latin verse. There are two of these, designed to commemorate the two tripos days. The first contains the names of the wranglers and senior optimes, and the second the names of the junior optimes. The word <xex>tripos</xex> is supposed to refer to the three-legged stool formerly used at the examinations for these honors, though some derive it from the three <xex>brackets</xex> formerly printed on the back of the paper.</cd> <au>C. A. Bristed.</au></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Trippant</ent><br/
<hw>Trip"pant</hw> <pr>(?)</pr>, <pos>a.</pos> <fld>(Her.)</fld> <def>See <er>Tripping</er>, <pos>a.</pos>, 2.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tripper</ent><br/
<hw>Trip"per</hw> <pr>(?)</pr>, <pos>n.</pos> <sn>1.</sn> <def>One who trips or supplants; also, one who walks or trips nimbly; a dancer.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>An excursionist.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Trippet</ent><br/
<hw>Trip"pet</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Mach.)</fld> <def>A cam, wiper, or projecting piece which strikes another piece repeatedly.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tripping</ent><br/
<hw>Trip"ping</hw> <pr>(?)</pr>, <pos>a.</pos> <sn>1.</sn> <def>Quick; nimble; stepping lightly and quickly.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Her.)</fld> <def>Having the right forefoot lifted, the others remaining on the ground, as if he were trotting; trippant; -- said of an animal, as a hart, buck, and the like, used as a bearing.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tripping</ent><br/
<hw>Trip"ping</hw>, <pos>n.</pos> <sn>1.</sn> <def>Act of one who, or that which, trips.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A light dance.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Other <qex>trippings</qex> to be trod of lighter toes.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <fld>(Naut.)</fld> <def>The loosing of an anchor from the ground by means of its cable or buoy rope.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Tripping line</b></col> <fld>(Naut.)</fld>, <cd>a small rope attached to the topgallant or royal yard, used to trip the yard, and in lowering it to the deck; also, a line used in letting go the anchor.</cd>  <rj><au>Luce.</au></rj></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Trippingly</ent><br/
<hw>Trip"ping*ly</hw>, <pos>adv.</pos> <def>In a tripping manner; with a light, nimble, quick step; with agility; nimbly.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Sing, and dance it <qex>trippingly</qex>.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Speak the speech . . . <qex>trippingly</qex> on the tongue.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tripsis</ent><br/
||<hw>Trip"sis</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[NL., fr. Gr. <?/, fr. <?/ to rub.]</ety> <fld>(Med.)</fld> <sd>(a)</sd> <def>Trituration.</def> <mark>[R.]</mark> <sd>(b)</sd> <def>Shampoo.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Triptote</ent><br/
<hw>Trip"tote</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>triptotum</ets>, Gr. <?/ with three cases; <?/ (see <er>Tri-</er>) + <?/ falling, fr. <?/ to fall.]</ety> <fld>(Gram.)</fld> <def>A noun having three cases only.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Triptych</ent><br/
<hw>Trip"tych</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Gr. <?/ consisting of three layers or plates; <?/ (see <er>Tri-</er>) + <?/, <?/, a fold, layer.]</ety> <def>Anything in three parts or leaves.</def> Specifically: -- <sd>(a)</sd> <def>A writing tablet in three parts, two of which fold over on the middle part.</def><br/
[<source>1913 Webster</source>]</p>

<p><sd>(b)</sd> <def>A picture or altarpiece in three compartments.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tripudiary</ent><br/
<hw>Tri*pu"di*a*ry</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>tripudium</ets> a measured stamping, a leaping, a solemn religious dance.]</ety> <def>Of or pertaining to dancing; performed by dancing.</def> <mark>[R.]</mark> <ldquo/ <xex>Tripudiary</xex> augurations.<rdquo/  <rj><au>Sir T. Browne.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tripudiate</ent><br/
<hw>Tri*pu"di*ate</hw> <pr>(?)</pr>, <pos>v. i.</pos> <ety>[L. <ets>tripudiare</ets>, <ets>tripudiatum</ets>.]</ety> <def>To dance.</def> <mark>[R.]</mark>  <rj><au>Cockeram.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tripudiation</ent><br/
<hw>Tri*pu`di*a"tion</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>tripudiatio</ets>.]</ety> <def>The act of dancing.</def> <mark>[R.]</mark>  <rj><au>Bacon. Carlyle.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Triquadrantal</ent><br/
<hw>Tri`quad*ran"tal</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[Pref. <ets>tri-</ets> + <ets>quadrantal</ets>.]</ety> <fld>(Spherical Trig.)</fld> <def>Having three quadrants; thus, a <xex>triquadrantal</xex> triangle is one whose three sides are quadrants, and whose three angles are consequently right angles.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Triquetral</ent><br/
<hw>Tri*que"tral</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Triquetrous.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Triquetrous</ent><br/
<hw>Tri*que"trous</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>triquetrus</ets>.]</ety> <def>Three sided, the sides being plane or concave; having three salient angles or edges; trigonal.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Triquetrum</ent><br/
<hw>Tri*que"trum</hw> <pr>(?)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Triquetra</plw> <pr>(#)</pr>.</plu> <ety>[NL.]</ety> <fld>(Anat.)</fld> <def>One of the bones of the carpus; the cuneiform. See <er>Cuneiform</er> <sd>(b)</sd>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Triradiated</ent><br/
<ent>Triradiate</ent><br/
<mhw>{ <hw>Tri*ra"di*ate</hw> <pr>(?)</pr>, <hw>Tri*ra"di*a`ted</hw> <pr>(?)</pr> }</mhw>, <pos>a.</pos> <ety>[Pref. <ets>tri-</ets> + <ets>radiate</ets>.]</ety> <def>Having three rays.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Trirectangular</ent><br/
<hw>Tri`rec*tan"gu*lar</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[Pref. <ets>tri-</ets> + <ets>rectangular</ets>.]</ety> <fld>(Spherical Trig.)</fld> <def>Having three right angles. See <er>Triquadrantal</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Trireme</ent><br/
<hw>Tri"reme</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>triremis</ets>; <ets>tri-</ets> (see <er>Tri-</er>) + <ets>remus</ets> an oar, akin to E. <ets>row</ets>. See Row to propel with an oar.]</ety> <fld>(Class. Antiq.)</fld> <def>An ancient galley or vessel with tree banks, or tiers, of oars.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Trirhomboidal</ent><br/
<hw>Tri`rhom*boid"al</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[Pref. <ets>tri-</ets> + <ets>rhomboidal</ets>.]</ety> <def>Having three rhombic faces or sides.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>-rid</ent><br/
<ent>Trisaccharide</ent><br/
<mhw><hw>Tri*sac"cha*ride</hw> <pr>(?)</pr>, <pos>n.</pos> Also <hw>-rid</hw></mhw>. <fld>(Chem.)</fld> <def>A complex sugar, as raffinose, yielding by hydrolysis three simple sugar molecules.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Trisacramentarian</ent><br/
<hw>Tri*sac`ra*men*ta"ri*an</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Pref. <ets>tri-</ets> + <ets>sacramentarian</ets>.]</ety> <fld>(Eccl.)</fld> <def>One who recognizes three sacraments, and no more; -- namely, baptism, the Lord's Supper, and penance. See <er>Sacrament</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Trisagion</ent><br/
||<hw>Tris*ag"i*on</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[NL., fr. Gr. <?/ thrice holy; <?/ thrice + <?/ holy.]</ety> <fld>(Eccl.)</fld> <def>An ancient anthem, -- usually known by its Latin name <xex>tersanctus</xex>.See <er>Tersanctus</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Trisect</ent><br/
<hw>Tri*sect"</hw> <pr>(?)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Trisected</conjf>; <pos>p. pr. & vb. n.</pos> <conjf>Trisecting</conjf>.]</vmorph> <ety>[Pref. <ets>tri-</ets> + L. <ets>sectus</ets>, p. p. of <ets>secare</ets> to cut. See <er>Section</er>.]</ety> <sn>1.</sn> <def>To cut or divide into three parts.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Geom.)</fld> <def>To cut or divide into three equal parts.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Trisected</ent><br/
<hw>Tri*sect"ed</hw>, <pos>a.</pos> <fld>(Bot.)</fld> <def>Divided into three parts or segments by incisions extending to the midrib or to the base; -- said of leaves.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Trisection</ent><br/
<hw>Tri*sec"tion</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Cf. F. <ets>trisection</ets>.]</ety> <def>The division of a thing into three parts, Specifically: <fld>(Geom.)</fld> the division of an angle into three equal parts.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Triseralous</ent><br/
<hw>Tri*ser"al*ous</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[Pref. <ets>tri-</ets> + <ets>sepal</ets>.]</ety> <fld>(Bot.)</fld> <def>Having three sepals, or calyx leaves.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Triseriate</ent><br/
<ent>Triserial</ent><br/
<mhw>{ <hw>Tri*se"ri*al</hw> <pr>(?)</pr>, <hw>Tri*se"ri*ate</hw> <pr>(?)</pr>, }</mhw> <pos>a.</pos> <ety>[Pref. <ets>tri-</ets> + <ets>serial</ets>, <ets>seriate</ets>.]</ety> <fld>(Bot.)</fld> <def>Arranged in three vertical or spiral rows.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Triskele</ent><br/
<ent>Triskelion</ent><br/
<mhw>{ <hw>Tris*kel"i*on</hw> <pr>(?)</pr>, <it>or</it> <hw>Tris"kele</hw> <pr>(?)</pr> }</mhw>, <pos>n.</pos> <ety>[Gr. <?/ three-legged. See <er>Tri-</er>; <er>Isosceles</er>.]</ety> <def>A figure composed of three branches, usually curved, radiating from a center, as the figure composed of three human legs, with bent knees, which has long been used as a badge or symbol of <geog>Sicily</geog> and of the <geog>Isle of Man</geog>.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Trismus</ent><br/
||<hw>Tris"mus</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[NL., form Gr. <?/ gnashing of the teeth.]</ety> <fld>(Med.)</fld> <def>The lockjaw.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Trisnitrate</ent><br/
<hw>Tris*ni"trate</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Gr. <?/ thrice + E. <ets>nitrate</ets>.]</ety> <fld>(Chem.)</fld> <def>A nitrate formed from three molecules of nitric acid; also, less properly, applied to certain basic nitrates; <as>as, <ex>trisnitrate</ex> of bismuth</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Trisoctahedron</ent><br/
<hw>Tris*oc`ta*he"dron</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Gr. <?/ thrice + FE. <ets>octahedron</ets>.]</ety> <fld>(Crystallog.)</fld> <def>A solid of the isometric system bounded by twenty-four equal faces, three corresponding to each face of an octahedron.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Tetragonal trisoctahedron</b></col>, <cd>a trisoctahedron each face of which is a quadrilateral; called also <altname>trapezohedron</altname> and <altname>icositetrahedron</altname>.</cd> -- <col><b>Trigonal trisoctahedron</b></col>, <cd>a trisoctahedron each face of which is an isosceles triangle.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Trispaston</ent><br/
<ent>Trispast</ent><br/
<mhw>{ <hw>Tri"spast</hw> <pr>(?)</pr>, <hw>Tri*spas"ton</hw> <pr>(?)</pr>, }</mhw> <pos>n.</pos> <ety>[NL. <ets>trispaston</ets>, fr. Gr. <?/ drawn threefold; <?/ (see <er>Tri-</er>) + <?/ to draw.]</ety> <fld>(Mech.)</fld> <def>A machine with three pulleys which act together for raising great weights.</def>  <rj><au>Brande & C.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Trispermous</ent><br/
<hw>Tri*sper"mous</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[Pref. <ets>tri-</ets> + Gr. <?/ seed.]</ety> <fld>(Bot.)</fld> <def>Containing three seeds; three-seeded; <as>as, a <ex>trispermous</ex> capsule</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Trisplanchnic</ent><br/
<hw>Tri*splanch"nic</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[<ets>Tri-</ets> + <ets>splanchnic</ets>.]</ety> <fld>(Anat.)</fld> <def>Of or pertaining to the three great splanchnic cavities, namely, that of the head, the chest, and the abdomen; -- applied to the sympathetic nervous system.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Trist</ent><br/
<hw>Trist</hw> <pr>(?)</pr>, <pos>v. t. & i.</pos> <vmorph>[<pos>imp.</pos> <conjf>Triste</conjf>.]</vmorph> <def>To trust.</def> <mark>[Obs.]</mark>  <rj><au>Chaucer.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Trist</ent><br/
<hw>Trist</hw>, <pos>n.</pos> <ety>[See <er>Tryst</er>.]</ety> <sn>1.</sn> <def>Trust.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A post, or station, in hunting.</def> <mark>[Obs.]</mark>  <rj><au>Chaucer.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>A secret meeting, or the place of such meeting; a tryst. See <er>Tryst</er>.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>George Douglas caused a <qex>trist</qex> to be set between him and the cardinal and four lords; at the which <qex>trist</qex> he and the cardinal agreed finally.</q> <rj><qau>Letter dated Sept., 1543.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Trist</ent><br/
<hw>Trist</hw>, <pos>a.</pos> <ety>[F. <ets>triste</ets>, L. <ets>tristis</ets>.]</ety> <def>Sad; sorrowful; gloomy.</def> <mark>[Obs.]</mark>  <rj><au>Fairfax.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Triste</ent><br/
<hw>Triste</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A cattle fair.</def> <mark>[Prov. Eng.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tristearate</ent><br/
<hw>Tri*ste"a*rate</hw> <pr>(?)</pr>, <pos>n.</pos> <def>Tristearin.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tristearin</ent><br/
<hw>Tri*ste"a*rin</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Pref. <ets>tri-</ets> + <ets>stearin</ets>.]</ety> <fld>(Physiol. Chem.)</fld> <def>See <er>Stearin</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tristful</ent><br/
<hw>Trist"ful</hw> <pr>(tr<icr/st"f<usdot/l)</pr>, <pos>a.</pos> <def>Sad; sorrowful; gloomy.</def>  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Eyes so <qex>tristful</qex>, eyes so <qex>tristful</qex>,<br/
Heart so full of care and cumber.</q> <rj><qau>Longfellow.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tristfully</ent><br/
<hw>Trist"ful*ly</hw>, <pos>adv.</pos> <def>In a tristful manner; sadly.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tristichous</ent><br/
<hw>Tris"tich*ous</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[Gr. <?/ in three rows; <?/ (see <er>Tri-</er>) + <?/ a row.]</ety> <fld>(Bot.)</fld> <def>Arranged in three vertical rows.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tristigmatose</ent><br/
<ent>Tristigmatic</ent><br/
<mhw>{ <hw>Tri`stig*mat"ic</hw> <pr>(?)</pr>, <hw>Tri*stig"ma*tose`</hw> <pr>(?)</pr>, }</mhw> <pos>a.</pos> <ety>[Pref. <ets>tri-</ets> + <ets>stigma</ets>.]</ety> <fld>(Bot.)</fld> <def>Having, or consisting of, three stigmas.</def>  <rj><au>Gray.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tristitiate</ent><br/
<hw>Tris*ti"ti*ate</hw> <pr>(?)</pr>, <pos>v. t.</pos> <ety>[L. <ets>tristitia</ets> sadness, fr. <ets>tristis</ets> sad.]</ety> <def>To make sad.</def> <mark>[Obs.]</mark>  <rj><au>Feltham.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><-- p. 1542 --><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tristoma</ent><br/
||<hw>Tris"to*ma</hw> <pr>(tr<icr/s"t<osl/*m<adot/)</pr>, <pos>n.</pos> <ety>[NL., from Gr. <grk>tri-</grk> (see <er>Tri-</er>) + <grk>sto`ma</grk> mouth.]</ety> <fld>(Zool.)</fld> <def>Any one of numerous species of trematode worms belonging to <gen>Tristoma</gen> and allied genera having a large posterior sucker and two small anterior ones. They usually have broad, thin, and disklike bodies, and are parasite on the gills and skin of fishes.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tristy</ent><br/
<hw>Trist"y</hw> <pr>(?)</pr>, <pos>a.</pos> <def>See <er>Trist</er>, <pos>a.</pos></def> <mark>[Obs.]</mark>  <rj><au>Ashmole.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Trisulc</ent><br/
<hw>Tri"sulc</hw> <pr>(tr<imac/"s<ucr/lk)</pr>, <pos>n.</pos> <ety>[L. <ets>trisulcus</ets>; <ets>tri-</ets> (see <er>Tri-</er>) + <ets>sulcus</ets> a furrow.]</ety> <def>Something having three forks or prongs, as a trident.</def> <mark>[Obs.]</mark> <ldquo/Jupiter's <xex>trisulc</xex>.<rdquo/  <rj><au>Sir T. Browne.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Trisulcate</ent><br/
<hw>Tri*sul"cate</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[Pref. <ets>tri-</ets> + <ets>sulcate</ets>.]</ety> <def>Having three furrows, forks, or prongs; having three grooves or sulci; three-grooved.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Trisulphide</ent><br/
<hw>Tri*sul"phide</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Pref. <ets>tri-</ets> + <ets>sulphide</ets>.]</ety> <fld>(Chem.)</fld> <def>A sulphide containing three atoms of sulphur.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Trisyllabical</ent><br/
<ent>Trisyllabic</ent><br/
<mhw>{ <hw>Tris`yl*lab"ic</hw> <pr>(?)</pr>, <hw>Tris`yl*lab"ic*al</hw> <pr>(?)</pr>, }</mhw> <pos>a.</pos> <ety>[L. <ets>trisyllabus</ets>, Gr. <?/; <?/ (see <er>Tri-</er>) + <?/ a syllable: cf. F. <ets>trissyllabique</ets>.]</ety> <def>Of or pertaining to a trisyllable; consisting of three syllables; <as>as, <ldquo/syllable<rdquo/ is a <ex>trisyllabic</ex> word</as>.</def> -- <wordforms><wf>Tris`yllab"ic*al*ly</wf>, <pos>adv.</pos></wordforms><br/
[<source>1913 Webster</source>]</p>

<p><ent>Trisyllable</ent><br/
<hw>Tri*syl"la*ble</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Pref. <ets>tri-</ets> + <ets>syllable</ets>.]</ety> <def>A word consisting of three syllables only; <as>as, <ex>a-ven-ger</ex></as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Trite</ent><br/
<hw>Trite</hw> <pr>(tr<imac/t)</pr>, <pos>a.</pos> <ety>[L. <ets>tritus</ets>, p. p. of <ets>terere</ets> to rub, to wear out; probably akin to E. <ets>throw</ets>. See <er>Throw</er>, and cf. <er>Contrite</er>, <er>Detriment</er>, <er>Tribulation</er>, <er>Try</er>.]</ety> <def>Worn out; common; used until so common as to have lost novelty and interest; hackneyed; stale; <as>as, a <ex>trite</ex> remark; a <ex>trite</ex> subject.</as></def> -- <wordforms><wf>Trite"ly</wf>, <pos>adv.</pos> -- <wf>Trite"ness</wf>, <pos>n.</pos></wordforms><br/
[<source>1913 Webster</source>]</p>

<p><ent>Triternate</ent><br/
<hw>Tri*ter"nate</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[Pref. <ets>tri-</ets> + <ets>ternate</ets>.]</ety> <fld>(Bot.)</fld> <def>Three times ternate; -- applied to a leaf whose petiole separates into three branches, each of which divides into three parts which each bear three leafiets.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tritheism</ent><br/
<hw>Tri"the*ism</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Pref. <ets>tri-</ets> + Gr. <?/ God: cf. F. <ets>trith<eacute/isme</ets>.]</ety> <def>The opinion or doctrine that the Father, Son, and Holy Spirit are three distinct Gods.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tritheist</ent><br/
<hw>Tri"the*ist</hw>, <pos>n.</pos> <ety>[Cf. F. <ets>trith<eacute/iste</ets>.]</ety> <def>One who believes in tritheism.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tritheistical</ent><br/
<ent>Tritheistic</ent><br/
<mhw>{ <hw>Tri`the*is"tic</hw> <pr>(?)</pr>, <hw>Tri`the*is"tic*al</hw> <pr>(?)</pr>, }</mhw> <pos>a.</pos> <def>Of or pertaining to tritheism.</def>  <rj><au>Bolingbroke.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tritheite</ent><br/
<hw>Tri"the*ite</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Cf. F. <ets>trith<eacute/ite</ets>.]</ety> <def>A tritheist.</def> <mark>[Obs.]</mark>  <rj><au>E. Phillips.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Trithing</ent><br/
<hw>Tri"thing</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[See Ist <er>Riding</er>.]</ety> <def>One of three ancient divisions of a county in England; -- now called <altname>riding</altname>.</def> <altsp>[Written also <asp>riding</asp>.]</altsp>  <rj><au>Blackstone.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Trithionate</ent><br/
<hw>Tri*thi"on*ate</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Chem.)</fld> <def>A salt of trithionic acid.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Trithionic</ent><br/
<hw>Tri`thi*on"ic</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[Pref. <ets>tri-</ets> + <ets>thionic</ets>.]</ety> <fld>(Chem.)</fld> <def>Of or pertaining to, or designating, a certain thionic acid, <chform>H2S3O6</chform> which is obtained as a colorless, odorless liquid.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tritiated</ent><br/
<hw>Trit"i*at*ed</hw> <pr>(tr<icr/t"<icr/*<amac/t*<ecr/d)</pr>, <pos>a.</pos> <fld>(Chem.)</fld> <def>containing <er>tritium</er>; -- of chemical compounds; <as>as, <ex>tritiated</ex> thymine</as>.</def>  <note>Compounds which are <ex>tritiated</ex> are, as a consequence, radioactive, and may be used as tracers in biochemical or chemical analysis.</note><br/
[<source>PJC</source>]</p>

<p><ent>Tritical</ent><br/
<hw>Trit"ic*al</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Trite.</def> <mark>[Obs.]</mark> <au>T. Warton.</au> -- <wordforms><wf>Trit"ic*al*ly</wf>, <pos>adv.</pos> <mark>[Obs.]</mark> -- <wf>Trit"ic*al*ness</wf>, <pos>n.</pos> <mark>[Obs.]</mark></wordforms><br/
[<source>1913 Webster</source>]</p>

<p><ent>Triticin</ent><br/
<hw>Trit"i*cin</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Chem.)</fld> <def>A carbohydrate isomeric with dextrin, obtained from quitch grass (<spn>Agropyrum repens</spn>, formerly <spn>Triticum repens</spn>) as a white amorphous substance.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Triticum</ent><br/
||<hw>Trit"i*cum</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L., perhaps fr. <ets>tritus</ets>, p. p. of <ets>terere</ets> to grind.]</ety> <fld>(Bot.)</fld> <def>A genus of grasses including the various species of wheat.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>tritium</ent><br/
<hw>trit"i*um</hw> <pr>(tr<icr/t"<icr/*<ucr/m)</pr>, <pos>n.</pos> <fld>(Chem.)</fld> <def>A radioactive isotope of hydrogen having one proton and two neutrons in the nucleus.  It decays spontaneously to Helium-3 by the emission of an electron (beta ray), with a half-life of 12.3 years.  Symbol <sub>1</sub>H<sup>3</sup>.  Atomic weight 3.01605 (C-12 = 12.0000). It is one of the <isa>radioisotopes</isa> commonly used to label chemical compounds for use as tracers in biochemistry and chemistry.  It is also used as one of the fusionable components of a hydrogen bomb.</def><br/
[<source>PJC</source>]</p>

<p><ent>Triton</ent><br/
||<hw>Tri"ton</hw> <pr>(tr<imac/"t<ocr/n)</pr>, <pos>n.</pos> <ety>[L., fr. Gr. <grk>Tri`twn</grk>.]</ety> <fld>(Gr. Myth.)</fld> <def>A fabled sea demigod, the son of Neptune and Amphitrite, and the trumpeter of Neptune.  He is represented by poets and painters as having the upper part of his body like that of a man, and the lower part like that of a fish. He often has a trumpet made of a shell.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Have sight of Proteus rising from the sea,<br/
Or hear old <qex>Triton</qex> blow his wreathed horn.</q> <rj><qau>Wordsworth.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Zool.)</fld> <def>Any one of many species of marine gastropods belonging to <gen>Triton</gen> and allied genera, having a stout spiral shell, often handsomely colored and ornamented with prominent varices. Some of the species are among the largest of all gastropods. Called also <altname>trumpet shell</altname>, and <altname>sea trumpet</altname>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <fld>(Zool.)</fld> <def>Any one of numerous species of aquatic salamanders. The common European species are <spn>Hemisalamandra cristata</spn>, <spn>Molge palmata</spn>, and <spn>Molge alpestris</spn>, a red-bellied species common in Switzerland. The most common species of the United States is <spn>Diemyctylus viridescens</spn>. See <xex>Illust.</xex> under <er>Salamander</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tritone</ent><br/
<hw>Tri"tone`</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Gr. <grk>tri`tonos</grk> of three tones; <grk>tri-</grk> tri- + <grk>to`nos</grk> a tone.]</ety> <fld>(Mus.)</fld> <def>A superfluous or augmented fourth.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tritorium</ent><br/
<hw>Tri*to"ri*um</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[NL.]</ety> <def>Same as <er>Triturium</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tritovum</ent><br/
||<hw>Tri*to"vum</hw> <pr>(?)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Tritova</plw> <pr>(#)</pr>.</plu> <ety>[NL., fr. Gr. <?/ third + L. <ets>ovum</ets> egg.]</ety> <fld>(Zool.)</fld> <def>An embryonic insect which has twice cast its skin previous to hatching from the egg.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tritozooid</ent><br/
<hw>Tri`to*zo"oid</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Gr. <?/ third + <grk>zo^,on</grk> an animal.]</ety> <fld>(Zool.)</fld> <def>A zooid of the third generation in asexual reproduction.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tritubercular</ent><br/
<hw>Tri`tu*ber"cu*lar</hw> <pr>(?)</pr>, <pos>a.</pos> <sd>(a)</sd> <def>Having or designating teeth with three cusps or tubercles; tricuspid.</def> <sd>(b)</sd> <def>Pertaining to trituberculy.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Trituberculy</ent><br/
<hw>Tri`tu*ber"cu*ly</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Pref. <ets>tri-</ets> + L. <ets>tuberculum</ets> tubercle.]</ety> <fld>(Zool.)</fld> <def>A theory of the development of mammalian molar teeth. The primitive stage is that of simple cones, as in reptiles. The simple cone then developed a smaller cone in front and another behind. Next, a <xex>cingulum</xex> was developed, and the three cones became arranged in a triangle, the two smaller cusps having moved to the outer side in upper and to the inner in lower molars. This primitive triangle is called the <xex>trigon</xex> or <xex>trigonid</xex> and this stage the <xex>tritubercular</xex> or <xex>trigonodont</xex>. The trigon being a cutting apparatus, an extension of the posterior part of the crown was developed in lower molars for crushing, and a smaller corresponding part appeared in upper molars. Another large cone then arose, usually from the cingulum. In more complex forms, smaller intermediate cusps appeared.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Triturable</ent><br/
<hw>Trit"u*ra*ble</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[Cf. F. <ets>triturable</ets>.]</ety> <def>Capable of being triturated.</def>  <rj><au>Sir T. Browne.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Triturate</ent><br/
<hw>Trit"u*rate</hw> <pr>(?)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Triturated</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Triturating</conjf>.]</vmorph> <ety>[L. <ets>trituratus</ets>, p. p. of <ets>triturate</ets> to thrash (grain), fr. <ets>terere</ets>, <ets>tritum</ets>, to rub, rub to pieces. See <er>Trite</er>.]</ety> <sn>1.</sn> <def>To rub, grind, bruise, or thrash.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To rub or grind to a very fine or impalpable powder; to pulverize and comminute thoroughly.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Trituration</ent><br/
<hw>Trit`u*ra"tion</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Cf. F. <ets>trituration</ets>, L. <ets>trituratio</ets> a thrashing of grain.]</ety> <def>The act of triturating, or reducing to a fine or impalpable powder by grinding, rubbing, bruising, etc.</def>  <rj><au>Paley.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Triture</ent><br/
<hw>Trit"ure</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>tritura</ets>, from <ets>terere</ets>, <ets>tritum</ets>, to rub, rub to pieces.]</ety> <def>A rubbing or grinding; trituration.</def> <mark>[Obs.]</mark>  <rj><au>Cheyne.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Triturium</ent><br/
<hw>Tri*tu"ri*um</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[NL.; cf. L. <ets>terere</ets>, <ets>tritum</ets> to rub.]</ety> <def>A vessel for separating liquids of different densities.</def> <altsp>[Written also <asp>tritorium</asp>.]</altsp><br/
[<source>1913 Webster</source>]</p>

<p><ent>Trityl</ent><br/
<hw>Tri"tyl</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Gr. <?/ third + <ets>-yl</ets>.]</ety> <fld>(Chem.)</fld> <def>Propyl.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tritylene</ent><br/
<hw>Tri"tyl*ene</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Chem.)</fld> <def>Propylene.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Triumph</ent><br/
<hw>Tri"umph</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>triumphus</ets>, OL. <ets>triumpus</ets>; of uncertain origin; cf. Gr. <?/ a procession in honor of Bacchus: cf. F. <ets>triomphe</ets>.  Cf. <er>Trump</er> at cards.]</ety> <sn>1.</sn> <fld>(Rom. Antiq.)</fld> <def>A magnificent and imposing ceremonial performed in honor of a general who had gained a decisive victory over a foreign enemy.</def><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ The general was allowed to enter the city crowned with a wreath of laurel, bearing a scepter in one hand, and a branch of laurel in the other, riding in a circular chariot, of a peculiar form, drawn by four horses. He was preceded by the senate and magistrates, musicians, the spoils, the captives in fetters, etc., and followed by his army on foot in marching order. The procession advanced in this manner to the Capitoline Hill, where sacrifices were offered, and victorious commander entertained with a public feast.</note><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Hence, any triumphal procession; a pompous exhibition; a stately show or pageant.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>Our daughter,<br/
In honor of whose birth these <qex>triumphs</qex> are,<br/
Sits here, like beauty's child.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>A state of joy or exultation for success.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Great <qex>triumph</qex> and rejoicing was in heaven.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Hercules from Spain<br/
Arrived in <qex>triumph</qex>, from Geryon slain.</q> <rj><qau>Dryden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>Success causing exultation; victory; conquest; <as>as, the <ex>triumph</ex> of knowledge</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <def>A trump card; also, an old game at cards.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Triumph</ent><br/
<hw>Tri"umph</hw>, <pos>v. i.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Triumphed</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Triumphing</conjf>.]</vmorph> <ety>[L. <ets>triumphare</ets>: cf. F. <ets>triompher</ets>. See <er>Triumph</er>, <pos>n.</pos>]</ety> <sn>1.</sn> <def>To celebrate victory with pomp; to rejoice over success; to exult in an advantage gained; to exhibit exultation.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>How long shall the wicked <qex>triumph</qex>?</q> <rj><qau>Ps. xciv. 3.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Sorrow on thee and all the pack of you<br/
That <qex>triumph</qex> thus upon my misery!</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To obtain victory; to be successful; to prevail.</def><br/
[<source>1913 Webster</source>]</p>

<p><q><qex>Triumphing</qex> over death, and chance, and thee, O Time.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>On this occasion, however, genius triumphed.</q> <rj><qau>Macaulay.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>To be prosperous; to flourish.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Where commerce <qex>triumphed</qex> on the favoring gales.</q> <rj><qau>Trumbull.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>To play a trump card.</def> <mark>[Obs.]</mark>  <rj><au>B. Jonson.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Triumph</ent><br/
<hw>Tri"umph</hw>, <pos>v. t.</pos> <def>To obtain a victory over; to prevail over; to conquer. Also, to cause to triumph.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>Two and thirty legions that awe<br/
All nations of the <qex>triumphed</qex> word.</q> <rj><qau>Massinger.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Triumphal</ent><br/
<hw>Tri*um"phal</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>triumphalis</ets>: cf. F. <ets>triomphal</ets>.]</ety> <def>Of or pertaining to triumph; used in a triumph; indicating, or in honor of, a triumph or victory; <as>as, a <ex>triumphal</ex> crown; a <ex>triumphal</ex> arch</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Messiah his <qex>triumphal</qex> chariot turned.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Triumphal</ent><br/
<hw>Tri*um"phal</hw>, <pos>n.</pos> <def>A token of victory.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>Joyless <qex>triumphals</qex> of his hoped success.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Triumphant</ent><br/
<hw>Tri*um"phant</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>triumphans</ets>, p. pr. of <ets>triumphare</ets>: cf. F. <ets>triomphant</ets>. See <er>Triumph</er>, <pos>v. i.</pos>]</ety><br/
[<source>1913 Webster</source>]</p>

<p><sn>1.</sn> <def>Rejoicing for victory; triumphing; exultant.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Successful beyond hope to lead ye forth<br/
<qex>Triumphant</qex> out of this infernal pit.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Celebrating victory; expressive of joy for success; <as>as, a <ex>triumphant</ex> song or ode</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>Graced with conquest; victorious.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Athena, war's <qex>triumphant</qex> maid.</q> <rj><qau>Pope.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>So shall it be in the church <qex>triumphant</qex>.</q> <rj><qau>Perkins.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>Of or pertaining to triumph; triumphal.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>Captives bound to a <qex>triumphant</qex> car.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Church triumphant</b></col>, <cd>the church in heaven, enjoying a state of triumph, her warfare with evil being over; -- distinguished from <contr>church militant</contr>. See under <er>Militant</er>.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Triumphantly</ent><br/
<hw>Tri*um"phant*ly</hw>, <pos>adv.</pos> <def>In a triumphant manner.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Triumpher</ent><br/
<hw>Tri"umph*er</hw> <pr>(?)</pr>, <pos>n.</pos> <sn>1.</sn> <fld>(Rom. Antiq.)</fld> <def>One who was honored with a triumph; a victor.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>One who triumphs or rejoices for victory.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Triumphing</ent><br/
<hw>Tri"umph*ing</hw>, <pos>a.</pos> <def>Having or celebrating a triumph; victorious; triumphant.</def> -- <wordforms><wf>Tri"umph*ing*ly</wf>, <pos>adv.</pos></wordforms><br/
[<source>1913 Webster</source>]</p>

<p><ent>Triumvir</ent><br/
||<hw>Tri*um"vir</hw> <pr>(?)</pr>, <pos>n.</pos>; <plu><it>pl.</it> L. <plw>Triumviri</plw> <pr>(#)</pr>, E. <plw>Triumvirs</plw> <pr>(#)</pr>.</plu> <ety>[L., fr. <ets>res</ets>, gen. <ets>trium</ets>, three + <ets>vir</ets> a man. See <er>Three</er>, and <er>Virile</er>.]</ety> <fld>(Rom. Antiq.)</fld> <def>One of tree men united in public office or authority.</def><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ In later times the triumvirs of Rome were three men who jointly exercised sovereign power. Julius Caesar, Crassus, and Pompey were the first triumvirs; Octavianus (Augustus), Antony, and Lepidus were the second and last.</note><br/
[<source>1913 Webster</source>]</p>

<p><ent>Triumvirate</ent><br/
<hw>Tri*um"vi*rate</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>triumviratus</ets>: cf. F. <ets>triumvirat</ets>.]</ety> <sn>1.</sn> <def>Government by three in coalition or association; the term of such a government.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A coalition or association of three in office or authority; especially, the union of three men who obtained the government of the Roman empire.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Triumviry</ent><br/
<hw>Tri*um"vi*ry</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A triumvirate.</def> <mark>[Obs.]</mark>  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Triune</ent><br/
<hw>Tri"une</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[Pref. <ets>tri-</ets> + L. <ets>unus</ets> one. See <er>One</er>.]</ety> <def>Being three in one; -- an epithet used to express the unity of a trinity of persons in the Godhead.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Triungulus</ent><br/
||<hw>Tri*un"gu*lus</hw> <pr>(?)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Triunguli</plw> <pr>(#)</pr>.</plu> <ety>[NL. See <er>Tri-</er>, and <er>Ungulate</er>.]</ety> <fld>(Zool.)</fld> <def>The active young larva of any oil beetle. It has feet armed with three claws, and is parasitic on bees. See <xex>Illust.</xex> of <cref>Oil beetle</cref>, under <er>Oil</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Triunity</ent><br/
<hw>Tri*u"ni*ty</hw> <pr>(?)</pr>, <pos>n.</pos> <def>The quality or state of being triune; trinity.</def>  <rj><au>Dr. H. More.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Trivalence</ent><br/
<hw>Triv"a*lence</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Chem.)</fld> <def>The quality or state of being trivalent.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Trivalent</ent><br/
<hw>Triv"a*lent</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[Pref. <ets>tri-</ets> + L. <ets>valens</ets>, <ets>-entis</ets>, p. pr. See <er>Valence</er>.]</ety> <fld>(Chem.)</fld> <def>Having a valence of three; capable of being combined with, substituted for, or compared with, three atoms of hydrogen; -- said of triad atoms or radicals; thus, nitrogen is <xex>trivalent</xex> in ammonia.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Trivalve</ent><br/
<hw>Tri"valve</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Pref. <ets>tri-</ets> + <ets>valve</ets>.]</ety> <def>Anything having three valves, especially a shell.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Trivalvular</ent><br/
<hw>Tri*val"vu*lar</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[Pref. <ets>tri-</ets> + <ets>valvular</ets>.]</ety> <def>Having three valves; three-valved.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Trivant</ent><br/
<hw>Triv"ant</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A truant.</def> <mark>[Obs.]</mark>  <rj><au>Burton.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Triverbial</ent><br/
<hw>Tri*ver"bi*al</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[Pref. <ets>tri-</ets> + L. <ets>verbum</ets> a word.]</ety> <fld>(Rom. Antiq.)</fld> <def>Pertaining to, or designating, certain days allowed to the pretor for hearing causes, when be might speak the three characteristic words of his office, <xex>do</xex>, <xex>dico</xex>, <xex>addico</xex>. They were called <xex>dies fasti</xex>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Trivet</ent><br/
<hw>Triv"et</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Probably through French fr. L. <ets>tripes</ets>, <ets>-edis</ets>, three-footed; <ets>tri-</ets> (see <er>Tri-</er>)+ <ets>pes</ets>, <ets>pedis</ets>, foot: cf. F. <ets>tr<eacute/pied</ets>. See <er>Foot</er>, and cf. <er>Tripod</er>.]</ety> <sn>1.</sn> <def>A tree-legged stool, table, or other support; especially, a stand to hold a kettle or similar vessel near the fire; a tripod.</def> <altsp>[Written also <asp>trevet</asp>.]</altsp><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A weaver's knife. See <er>Trevat</er>.</def>  <rj><au>Knight.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Trivet table</b></col>, <cd>a table supported by three legs.</cd>  <rj><au>Dryden.</au></rj></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Trivial</ent><br/
<hw>Triv"i*al</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>trivialis</ets>, properly, that is in, or belongs to, the crossroads or public streets; hence, that may be found everywhere, common, fr. <ets>trivium</ets> a place where three roads meet, a crossroad, the public street; <ets>tri-</ets> (see <er>Tri-</er>) + <ets>via</ets> a way: cf. F. <ets>trivial</ets>. See <er>Voyage</er>.]</ety> <sn>1.</sn> <def>Found anywhere; common.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Ordinary; commonplace; trifling; vulgar.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>As a scholar, meantime, he was <qex>trivial</qex>, and incapable of labor.</q> <rj><qau>De Quincey.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>Of little worth or importance; inconsiderable; trifling; petty; paltry; <as>as, a <ex>trivial</ex> subject or affair</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The <qex>trivial</qex> round, the common task.</q> <rj><qau>Keble.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>Of or pertaining to the trivium.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Trivial name</b></col> <fld>(Nat. Hist.)</fld>, <cd>the specific name.</cd></cs><-- (b) <fld>(Chem.)</fld> The common name, not describing the structure and from which the structure cannot be deduced; -- contrasted with <contr>systematic name</contr>. --><br/
[<source>1913 Webster</source>]</p>

<p><ent>Trivial</ent><br/
<hw>Triv"i*al</hw>, <pos>n.</pos> <def>One of the three liberal arts forming the trivium.</def> <mark>[Obs.]</mark>  <rj><au>Skelton. Wood.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Trivialism</ent><br/
<hw>Triv"i*al*ism</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A trivial matter or method; a triviality.</def>  <rj><au>Carlyle.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Triviality</ent><br/
<hw>Triv`i*al"i*ty</hw> <pr>(?)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Trivialities</plw> <pr>(#)</pr>.</plu> <ety>[Cf. F. <ets>trivialit<eacute/</ets>]</ety> <sn>1.</sn> <def>The quality or state of being trivial; trivialness.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>That which is trivial; a trifle.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The philosophy of our times does not expend itself in furious discussions on mere scholastic <qex>trivialities</qex>.</q> <rj><qau>Lyon Playfair.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Trivially</ent><br/
<hw>Triv"i*al*ly</hw> <pr>(?)</pr>, <pos>adv.</pos> <def>In a trivial manner.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Trivialness</ent><br/
<hw>Triv"i*al*ness</hw>, <pos>n.</pos> <def>Quality or state of being trivial.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Trivium</ent><br/
||<hw>Triv"i*um</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[LL. See <er>Trivial</er>.]</ety> <sn>1.</sn> <def>The three <ldquo/ liberal<rdquo/ arts, grammar, logic, and rhetoric; -- being a triple way, as it were, to eloquence.</def><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ The <xex>trivium</xex> and <xex>quadrivium</xex> together made up the seven liberal arts. See <er>Quadrivium</er>.</note><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Zool.)</fld> <def>The three anterior ambulacra of echinoderms, collectively.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Triweekly</ent><br/
<hw>Tri"week`ly</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[Pref. <ets>tri-</ets> + <ets>weekly</ets>.]</ety> <def>Occurring or appearing three times a week; thriceweekly; <as>as, a <ex>triweekly</ex> newspaper</as>.</def> -- <def2><pos>adv.</pos> <def>Three times a week.</def></def2> -- <def2><pos>n.</pos> <def>A triweekly publication.</def></def2><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ This is a convenient word, but is not legitimately formed. It should mean occurring once in three weeks, as <xex>triennial</xex> means once in three years.  Cf. <er>Biweekly</er>.</note><br/
[<source>1913 Webster</source>]</p>

<p><ent>Troad</ent><br/
<hw>Troad</hw> <pr>(?)</pr>, <pos>n.</pos> <def>See <er>Trode</er>.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Troat</ent><br/
<hw>Troat</hw> <pr>(?)</pr>, <pos>v. i.</pos> <ety>[Either onomatopoetic, or akin to <ets>throat</ets>.]</ety> <def>To cry, as a buck in rutting time.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Troat</ent><br/
<hw>Troat</hw>, <pos>n.</pos> <def>The cry of a buck in rutting time.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Trocar</ent><br/
<hw>Tro"car</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F. <ets>trocart</ets> (or <ets>trois-quarts</ets>, i. e., three quarters); <ets>trois</ets> three (L. <ets>tres</ets>) + <ets>carre</ets> the side of a sword blade; -- so called from its triangular point.]</ety> <fld>(Surg.)</fld> <def>A stylet, usually with a triangular point, used for exploring tissues or for inserting drainage tubes, as in dropsy.</def> <altsp>[Written also <asp>trochar</asp>.]</altsp><br/
[<source>1913 Webster</source>]</p>

<p><ent>Trocha</ent><br/
||<hw>Tro"cha</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Sp., bypath, crossroad.]</ety> <fld>(Mil.)</fld> <def>A line of fortifications, usually rough, constructed to prevent the passage of an enemy across a region.</def> <mark>[Sp. Amer.]</mark><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Trochaic</ent><br/
<hw>Tro*cha"ic</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Pros.)</fld> <def>A trochaic verse or measure.</def>  <rj><au>Dryden.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Trochaical</ent><br/
<ent>Trochaic</ent><br/
<mhw>{ <hw>Tro*cha"ic</hw> <pr>(?)</pr>, <hw>Tro*cha"ic*al</hw> <pr>(?)</pr>, }</mhw> <pos>a.</pos> <ety>[L. <ets>trocha<ium/cus</ets>, Gr. <?/ or <?/. See <er>Trochee</er>.]</ety> <fld>(Pros.)</fld> <def>Of or pertaining to trochees; consisting of trochees; <as>as, <ex>trochaic</ex> measure or verse</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Trochal</ent><br/
<hw>Tro"chal</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[From Gr. <?/ a wheel.]</ety> <fld>(Zool.)</fld> <def>Resembling a wheel.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Trochal disk</b></col> <fld>(Zool.)</fld>, <cd>the cephalic disk of a rotifer. It is usually surrounded by a fringe of cilia.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Trochanter</ent><br/
||<hw>Tro*chan"ter</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[NL., fr. Gr. <?/.]</ety> <sn>1.</sn> <fld>(Anat.)</fld> <def>One of two processes near the head of the femur, the outer being called the <stype>great trochanter</stype>, and the inner the <stype>small trochanter</stype>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Zool.)</fld> <def>The third joint of the leg of an insect, or the second when the trochantine is united with the coxa.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Trochanteric</ent><br/
<hw>Tro`chan*ter"ic</hw> <pr>(?)</pr>, <pos>a.</pos> <fld>(Anat.)</fld> <def>Of or pertaining to one or both of the trochanters.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Trochantine</ent><br/
<hw>Tro*chan"tine</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Zool.)</fld> <def>The second joint of the leg of an insect, -- often united with the coxa.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Trochar</ent><br/
<hw>Tro"char</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Surg.)</fld> <def>See <er>Trocar</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Troche</ent><br/
<hw>Tro"che</hw> <pr>(tr<omac/"k<emac/)</pr>, <pos>n.</pos> <ety>[Gr. <grk>trocho`s</grk> anything round or circular, a wheel, properly, a runner, fr. <grk>tre`chein</grk> to run.  Cf. <er>Trochee</er>.]</ety> <fld>(Pharm.)</fld> <def>A medicinal tablet or lozenge; strictly, one of circular form.</def><br/
[<source>1913 Webster</source>]</p>

<p><-- p. 1543 --><br/
[<source>1913 Webster</source>]</p>

<p><ent>Trochee</ent><br/
<hw>Tro"chee</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>trochaeus</ets>, Gr. <?/ (sc.<?/), from <?/ running, from <?/ to run.  Cf. <er>Troche</er>, <er>Truck</er> a wheel.]</ety> <fld>(Pros.)</fld> <def>A foot of two syllables, the first long and the second short, as in the Latin word <xex>ante</xex>, or the first accented and the second unaccented, as in the English word <xex>motion</xex>; a choreus.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Trochil</ent><br/
<hw>Tro"chil</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Cf. F. <ets>trochile</ets>. See <er>Trochilus</er>.]</ety> <fld>(Zool.)</fld> <def>The crocodile bird.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The crocodile . . . opens his chaps to let the <qex>trochil</qex> in to pick his teeth, which gives it the usual feeding.</q> <rj><qau>Sir T. Herbert.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Trochili</ent><br/
||<hw>Troch"i*li</hw> <pr>(?)</pr>, <pos>n. pl.</pos> <ety>[NL. See <er>Trochilus</er>.]</ety> <fld>(Zool.)</fld> <def>A division of birds comprising the humming birds.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Trochilic</ent><br/
<hw>Tro*chil"ic</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[See <er>Trochilics</er>.]</ety> <def>OF or pertaining to rotary motion; having power to draw out or turn round.</def> <ldquo/By art <xex>trochilic</xex>.<rdquo/  <rj><au>Camden.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Trochilics</ent><br/
<hw>Tro*chil"ics</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Gr. <?/ the sheaf of a pulley, roller of a windlass, from <?/ to run.]</ety> <def>The science of rotary motion, or of wheel work.</def>  <rj><au>Wilkins.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Trochilidist</ent><br/
<hw>Tro*chil"i*dist</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[See <er>Trochilus</er>.]</ety> <def>One who studies, or is versed in, the nature and habits of humming birds, or the <xex>Trochilidae</xex>.</def>  <rj><au>Gould.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Trochilos</ent><br/
||<hw>Troch"i*los</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[NL. See <er>Trochilus</er>.]</ety> <fld>(Zool.)</fld> <def>The crocodile bird, or trochil.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Trochilus</ent><br/
||<hw>Troch"i*lus</hw> <pr>(?)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Trochili</plw> <pr>(#)</pr>.</plu> <ety>[L. <ets>trochilus</ets> a kind of small bird. Gr. <?/, fr. <ets><?/</ets> to run.]</ety> <sn>1.</sn> <fld>(Zool.)</fld> <sd>(a)</sd> <def>A genus of humming birds. It Formerly included all the known species.</def> <sd>(b)</sd> <def>Any one of several species of wrens and kinglets.</def> <mark>[Obs.]</mark> <sd>(c)</sd> <def>The crocodile bird.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Arch.)</fld> <def>An annular molding whose section is concave, like the edge of a pulley; -- called also <altname>scotia</altname>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Troching</ent><br/
<hw>Tro"ching</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[OF. <ets>troche</ets> cluster, group; cf. F. <ets>trochure</ets> a surantler, <ets>troch<eacute/e</ets> branches of a seedling, <ets>trochet</ets> cluster of flowers or fruits.]</ety> <fld>(Zool.)</fld> <def>One of the small branches of a stag's antler.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Trochiscus</ent><br/
||<hw>Tro*chis"cus</hw> <pr>(?)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Trochisci</plw> <pr>(#)</pr>.</plu> <ety>[L., fr. Gr. <?/ a small ball, dim. of <?/ a wheel. See <er>Troche</er>.]</ety> <fld>(Pharm.)</fld> <def>A kind of tablet or lozenge; a troche.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Trochisk</ent><br/
<hw>Tro"chisk</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Cf. F. <ets>trochisque</ets>.]</ety> <def>See <er>Trochiscus</er>.</def> <mark>[Obs.]</mark>  <rj><au>Bacon.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Trochite</ent><br/
<hw>Tro"chite</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Gr. <?/ a wheel.]</ety> <fld>(Paleon.)</fld> <def>A wheel-like joint of the stem of a fossil crinoid.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Trochlea</ent><br/
||<hw>Troch"le*a</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L., a case or sheaf containing one or more pulleys, Gr. <?/. See <er>Trochilics</er>.]</ety> <sn>1.</sn> <fld>(Mach.)</fld> <def>A pulley.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Anat.)</fld> <def>A pulley, or a structure resembling a pulley; <as>as, the <ex>trochlea</ex>, or pulleylike end, of the humerus, which articulates with the ulna; or the <ex>trochlea</ex>, or fibrous ring, in the upper part of the orbit, through which the superior oblique, or trochlear, muscle of the eye passes.</as></def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Trochlear</ent><br/
<hw>Troch"le*ar</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>trochlea</ets> block or pulley.]</ety> <fld>(Anat.)</fld> <def>Shaped like, or resembling, a pulley; pertaining to, or connected with, a trochlea; <as>as, a <ex>trochlear</ex> articular surface; the <ex>trochlear</ex> muscle of the eye.</as></def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Trochlear nerve</b></col>. <cd>See <cref>Pathetic nerve</cref>, under <er>Pathetic</er>.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Trochleary</ent><br/
<hw>Troch"le*a*ry</hw> <pr>(?)</pr>, <pos>a.</pos> <fld>(Anat.)</fld> <def>Pertaining to, or connected with, a trochlea; trochlear; <as>as, the <ex>trochleary</ex>, or trochlear, nerve</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Trochoid</ent><br/
<hw>Tro"choid</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Gr. <?/ a wheel + <ets>-oid</ets>; cf. F. <ets>trocho<ium/de</ets>. See <er>Troche</er>.]</ety> <fld>(Geom.)</fld> <def>The curve described by any point in a wheel rolling on a line; a cycloid; a roulette; in general, the curve described by any point fixedly connected with a moving curve while the moving curve rolls without slipping on a second fixed curve, the curves all being in one plane. Cycloids, epicycloids, hypocycloids, cardioids, etc., are all trochoids.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Trochoid</ent><br/
<hw>Tro"choid</hw>, <pos>a.</pos> <sn>1.</sn> <fld>(Anat.)</fld> <def>Admitting of rotation on an axis; -- sometimes applied to a pivot joint like that between the atlas and axis in the vertebral column.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Zool.)</fld> <def>Top-shaped; having a flat base and conical spire; -- said of certain shells.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <fld>(Zool.)</fld> <def>Of or pertaining to the genus Trochus or family <fam>Trochidae</fam>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Trochoidal</ent><br/
<hw>Tro*choid"al</hw> <pr>(?)</pr>, <pos>a.</pos> <sn>1.</sn> <fld>(Geom.)</fld> <def>Of or pertaining to a trochoid; having the properties of a trochoid.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Anat. &  Zool.)</fld> <def>See <er>Trochoid</er>, <pos>a.</pos></def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Trochometer</ent><br/
<hw>Tro*chom"e*ter</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Gr. <?/ a wheel + <ets>-meter</ets>.]</ety> <def>A contrivance for computing the revolutions of a wheel; an odometer.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Trochosphere</ent><br/
<hw>Troch"o*sphere</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Gr. <?/ a wheel + <ets>sphere</ets>.]</ety> <fld>(Zool.)</fld> <def>A young larval form of many annelids, mollusks, and bryozoans, in which a circle of cilia is developed around the anterior end.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Trochus</ent><br/
||<hw>Tro"chus</hw> <pr>(?)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Trochi</plw> <pr>(#)</pr>.</plu> <ety>[NL., fr. Gr. <?/ a wheel.]</ety> <fld>(Zool.)</fld> <def>Any one of numerous species of marine univalve shells belonging to <gen>Trochus</gen> and many allied genera of the family <fam>Trochidae</fam>. Some of the species are called also <altname>topshells</altname>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Troco</ent><br/
<hw>Tro"co</hw> <pr>(?)</pr>, <pos>n.</pos> <def>An old English game; -- called also <altname>lawn billiards</altname>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Trod</ent><br/
<hw>Trod</hw> <pr>(?)</pr>, <def><pos>imp. & p. p.</pos> of <er>Tread</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Trodden</ent><br/
<hw>Trod"den</hw> <pr>(?)</pr>, <def><pos>p. p.</pos> of <er>Tread</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Trode</ent><br/
<hw>Trode</hw> <pr>(?)</pr>, <mark>archaic</mark> <def><pos>imp.</pos> of <er>Tread</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>On burnished hooves his war-horse <qex>trode</qex>.</q> <rj><qau>Tennyson.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Trode</ent><br/
<hw>Trode</hw>, <pos>n.</pos> <ety>[AS. <ets>trod</ets>, fr. <ets>tredan</ets> to tread. See <er>Tread</er>.]</ety> <def>Tread; footing.</def> <altsp>[Written also <asp>troad</asp>.]</altsp><mark>[Obs.]</mark>  <rj><au>Spenser.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Troglodyte</ent><br/
<hw>Trog"lo*dyte</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>troglodytae</ets>, pl., Gr. <?/ one who creeps into holes; <?/ a hole, cavern (fr. <?/ to gnaw) + <?/ enter: cf. F. <ets>troglodyte</ets>.]</ety> <sn>1.</sn> <fld>(Ethnol.)</fld> <def>One of any savage race that dwells in caves, instead of constructing dwellings; a cave dweller, or cave man. Most of the primitive races of man were troglodytes.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>In the <qex>troglodytes'</qex> country there is a lake, for the hurtful water it beareth called the <ldquo/mad lake.<rdquo/</q> <rj><qau>Holland.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Zool.)</fld> <def>An anthropoid ape, as the chimpanzee.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <fld>(Zool.)</fld> <def>The wren.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Troglodytes</ent><br/
||<hw>Trog`lo*dy"tes</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[NL. See <er>Troglodyte</er>.]</ety> <sn>1.</sn> <fld>(Zool.)</fld> <def>A genus of apes including the chimpanzee.</def> <note>The chimpanzee is now named <spn>Pan troglodytes</spn>.</note>, and its genus is <gen>Pan</gen>.<br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Zool.)</fld> <def>A genus of singing birds including the common wrens.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Troglodytical</ent><br/
<ent>Troglodytic</ent><br/
<mhw>{ <hw>Trog`lo*dyt"ic</hw> <pr>(?)</pr>, <hw>Trog`lo*dyt"ic*al</hw> <pr>(?)</pr>, }</mhw> <pos>a.</pos> <ety>[L. <ets>troglodyticus</ets>, Gr. <?/.]</ety> <def>Of or pertaining to a troglodyte, or dweller in caves.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Trogon</ent><br/
<hw>Tro"gon</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[NL.]</ety> <fld>(Zool.)</fld> <def>Any one of numerous species of beautiful tropical birds belonging to the family <fam>Trogonidae</fam>. They are noted for the brilliant colors and the resplendent luster of their plumage.</def><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ Some of the species have a train of long brilliant feathers lying over the tail and consisting of the upper tail coverts. Unlike other birds having two toes directed forward and two backward, they have the inner toe turned backward. A few species are found in Africa and India, but the greater number, including the most brilliant species, are found in tropical America. See <xex>Illust.</xex> of <er>Quesal</er>.</note>
<-- = quetzal --><br/
[<source>1913 Webster</source>]</p>

<p><ent>Trogonoid</ent><br/
<hw>Tro"gon*oid</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[<ets>Trogon</ets> + <ets>-oid</ets>.]</ety> <fld>(Zool.)</fld> <def>Like or pertaining to the trogons.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Trogue</ent><br/
<hw>Trogue</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Cf. G. <ets>trog</ets> trough, E. <ets>trough</ets>.]</ety> <fld>(Mining)</fld> <def>A wooden trough, forming a drain.</def>  <rj><au>Raymond.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Troic</ent><br/
<hw>Tro"ic</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>Troicus</ets>, Gr. <?/, fr. <?/. See <er>Trojan</er>.]</ety> <def>Pertaining to Troy; Trojan.</def>  <rj><au>Gladstone.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Troilite</ent><br/
<hw>Tro"i*lite</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Named after Dominico <etsep>Troili</etsep>, an Italian of the 18th century.]</ety> <fld>(Min.)</fld> <def>Native iron protosulphide, <chform>FeS</chform>. It is known only in meteoric irons, and is usually in imbedded nodular masses of a bronze color.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Troilus</ent><br/
<hw>Tro"i*lus</hw> <pr>(?)</pr>, <pos>n.</pos>; <plu><it>pl.</it> L. <plw>Troili</plw> <pr>(#)</pr>, E. <plw>Troiluses</plw> <pr>(#)</pr>.</plu> <ety>[NL., fr. L. <ets>Troilus</ets>, Gr. <?/, the son of Priam.]</ety> <fld>(Zool.)</fld> <def>A large, handsome American butterfly (<spn>Euph<oe/ades troilus</spn>, <it>or</it>  <spn>Papilio troilus</spn>). It is black, with yellow marginal spots on the front wings, and blue spots on the rear wings; -- also called <altname>troilus butterfly</altname>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Troilus butterfly</ent><br/
<hw>Tro"i*lus butterfly</hw> <pr>(?)</pr>. <def>A large American butterfly (<spn>Papilio troilus</spn>). It is black, with yellow marginal spots on the front wings, and blue on the rear; -- also called <altname>troilus</altname>.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Trois point</ent><br/
||<hw>Trois` point"</hw> <pr>(?)</pr>. <ety>[F. <ets>trois</ets> three.]</ety> <fld>(Backgammon)</fld> <def>The third point from the outer edge on each player's home table.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Trojan</ent><br/
<hw>Tro"jan</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>Trojanus</ets>, fr. <ets>Troja</ets>, <ets>Troia</ets>, Troy, from <ets>Tros</ets>, Gr. <grk>Trw`s</grk>, <grk>Trwo`s</grk>, Tros, the mythical founder of Troy.]</ety> <sn>1.</sn> <def>Of or pertaining to ancient Troy or its inhabitants.</def> -- <def2><pos>n.</pos> <def>A native or inhabitant of Troy.</def></def2><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>One who shows the pluck, endurance, determined energy, strength, or the like, attributed to the defenders of Troy; -- used chiefly or only in the phrase <col><b>like a Trojan</b></col>; <as>as, he endured the pain <ex>like a Trojan</ex>; he studies <ex>like a Trojan</ex></as>.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><q>Tim jumped like a <xex>Trojan</xex> from the bed.</q>    <qau>Finnegan's Wake (Irish song)</qau><br/
[<source>PJC</source>]</p>

<p><ent>Trojan horse</ent><br/
<hw>Tro"jan horse`</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[from the incident described in Homer's <title>Iliad</title>.]</ety> <sn>1.</sn> <fld>(Classical mythology)</fld> <def>a large hollow wooden horse built by Greek soldiers besieging Troy during the Trojan War, and left as a <ldquo/gift<rdquo/ when they pretended to abandon their seige.  It was taken into the city by the Trojans, and Greek soldiers concealed inside came out and opened the gates to the city, enabling the capture of the city by the Greeks.</def><br/
[<source>RP</source> + <source>PJC</source>]</p>

<p><sn>2.</sn> <def>Hence, any thing or person which appears harmless but is designed to destroy or attack from within.  It may sometimes refer to a group; -- see also <er>fifth column</er>.</def><br/
[<source>RP</source> + <source>PJC</source>]</p>

<p><sn>3.</sn> <fld>(Computers)</fld> <def>A computer program designed to evade the security precautions within a computer system and perform illicit operations, or to do malicious damage, and often designed to look like a different kind of program, such as a game, archiver, or directory lister.  This term is not applied to a program that replicates itself, such as a <contr>virus</contr>.</def><br/
[<source>RP</source> + <source>PJC</source>]</p>

<p><ent>Troll</ent><br/
<hw>Troll</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Icel. <ets>troll</ets>.  Cf. <er>Droll</er>, <er>Trull</er>.]</ety> <fld>(Scand. Myth.)</fld> <def>A supernatural being, often represented as of diminutive size, but sometimes as a giant, and fabled to inhabit caves, hills, and like places; a witch.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Troll flower</b></col>. <fld>(Bot.)</fld> <cd>Same as <er>Globeflower</er> <sd>(a)</sd>.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Troll</ent><br/
<hw>Troll</hw> <pr>(?)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Trolled</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Trolling</conjf>.]</vmorph> <ety>[OE. <ets>trollen</ets> to roll, F. <ets>tr<ocir/ler</ets>, Of. <ets>troller</ets> to drag about, to ramble; probably of Teutonic origin; cf. G. <ets>trollen</ets> to roll, ramble, sich <ets>trollen</ets> to be gone; or perhaps for <ets>trotler</ets>, fr. F. <ets>trotter</ets> to trot (cf. <er>Trot</er>.).  Cf. <er>Trawl</er>.]</ety> <sn>1.</sn> <def>To move circularly or volubly; to roll; to turn.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>To dress and <qex>troll</qex> the tongue, and roll the eye.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To send about; to circulate, as a vessel in drinking.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Then doth she <qex>troll</qex> to the bowl.</q> <rj><qau>Gammer Gurton's Needle.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q><qex>Troll</qex> the brown bowl.</q> <rj><qau>Sir W. Scott.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>To sing the parts of in succession, as of a round, a catch, and the like; also, to sing loudly or freely.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Will you <qex>troll</qex> the catch ?</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>His sonnets charmed the attentive crowd,<br/
By wide-mouthed mortal<qex>trolled</qex> aloud.</q> <rj><qau>Hudibras.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>To angle for with a trolling line, or with a book drawn along the surface of the water; hence, to allure.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <def>To fish in; to seek to catch fish from.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>With patient angle <qex>trolls</qex> the finny deep.</q> <rj><qau>Goldsmith.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Troll</ent><br/
<hw>Troll</hw>, <pos>v. i.</pos> <sn>1.</sn> <def>To roll; to run about; to move around; <as>as, to <ex>troll</ex> in a coach and six</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To move rapidly; to wag.</def>  <rj><au>F. Beaumont.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>To take part in trolling a song.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>To fish with a rod whose line runs on a reel; also, to fish by drawing the hook through the water.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Their young men . . . <qex>trolled</qex> along the brooks that abounded in fish.</q> <rj><qau>Bancroft.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Troll</ent><br/
<hw>Troll</hw>, <pos>n.</pos> <sn>1.</sn> <def>The act of moving round; routine; repetition.</def>  <rj><au>Burke.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A song the parts of which are sung in succession; a catch; a round.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Thence the catch and <qex>troll</qex>, while <ldquo/Laughter, holding both his sides,<rdquo/ sheds tears to song and ballad pathetic on the woes of married life.</q> <rj><qau>Prof. Wilson.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>A trolley.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Troll plate</b></col> <fld>(Mach.)</fld>, <cd>a rotative disk with spiral ribs or grooves, by which several pieces, as the jaws of a chuck, can be brought together or spread radially.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Troller</ent><br/
<hw>Troll"er</hw> <pr>(?)</pr>, <pos>n.</pos> <def>One who trolls.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Trolly</ent><br/
<ent>Trolley</ent><br/
<mhw>{ <hw>Trol"ley</hw>, <hw>Trol"ly</hw>  }</mhw> <pr>(?)</pr>, <pos>n.</pos> <sd>(a)</sd> <def>A form of truck which can be tilted, for carrying railroad materials, or the like.</def> <mark>[Eng.]</mark> <sd>(b)</sd> <def>A narrow cart that is pushed by hand or drawn by an animal.</def> <mark>[Eng.]</mark> <sd>(c)</sd> <fld>(Mach.)</fld> <def>A truck from which the load is suspended in some kinds of cranes.</def> <sd>(d)</sd> <fld>(Electric Railway)</fld> <def>A truck which travels along the fixed conductors, and forms a means of connection between them and a railway car.</def> <sd>(e)</sd> <cd>A trolley car.</cd>
<-- (f) An elongated structure along a roadway containing conducting wire suspended from insulated supports at some height above the street, to provide electrical power for a trolley car. --><br/
[<source>1913 Webster</source>]</p>

<p><ent>Trolley car</ent><br/
<hw>Trol"ley car</hw>. <fld>(Elec.)</fld> <def>A motor car powered by electricity drawn from a trolley, and thus constrained to follow the trolley lines.</def><br/
[<source>Webster 1913 Suppl.</source> <source>+PJC</source>]</p>

<p><col><b>Trolley line</b></col>, <sd>(a)</sd> <cd>A trolley <sd>(f)</sd>.</cd> <sd>(b)</sd> <cd>The path along which a trolley car runs.</cd><br/
[<source>PJC</source>]</p>

<p><ent>Trolley wire</ent><br/
<hw>Trolley wire</hw>. <def>A heavy conducting wire on which the trolley car runs and from which it receives the current.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Trollmydames</ent><br/
<hw>Troll"my*dames`</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F. <ets>trou-madame</ets> pigeon holes.]</ety> <def>The game of nineholes.</def> <altsp>[Written also <asp>trolmydames</asp>.]</altsp> <mark>[Obs.]</mark>  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Trollop</ent><br/
<hw>Trol"lop</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[From <er>Troll</er> to roll, to stroll; but cf. also <er>Trull</er>.]</ety> <def>A stroller; a loiterer; esp., an idle, untidy woman; a slattern; a slut; a whore.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Trollopee</ent><br/
<hw>Trol`lop*ee"</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A kind of loose dress for women.</def> <mark>[Obs.]</mark>  <rj><au>Goldsmith.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Trombone</ent><br/
<hw>Trom"bone</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[It., aug. of <ets>tromba</ets> a trumpet: cf. F. <ets>trombone</ets>. See <er>Trump</er> a trumpet.]</ety> <sn>1.</sn> <fld>(Mus.)</fld> <def>A powerful brass instrument of the trumpet kind, thought by some to be the ancient sackbut, consisting of a tube in three parts, bent twice upon itself and ending in a bell.  The middle part, bent double, slips into the outer parts, as in a telescope, so that by change of the vibrating length any tone within the compass of the instrument (which may be bass or tenor or alto or even, in rare instances, soprano) is commanded.  It is the only member of the family of wind instruments whose scale, both diatonic and chromatic, is complete without the aid of keys or pistons, and which can slide from note to note as smoothly as the human voice or a violin.  Softly blown, it has a rich and mellow sound, which becomes harsh and blatant when the tones are forced; used with discretion, its effect is often solemn and majestic.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Zool.)</fld> <def>The common European bittern.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Trommel</ent><br/
<hw>Trom"mel</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Cf. G. <ets>trommel</ets> a drum.]</ety> <fld>(Mining)</fld> <def>A revolving buddle or sieve for separating, or sizing, ores.</def>  <rj><au>Raymond.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tromp</ent><br/
<hw>Tromp</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F. <ets>trombe</ets>, <ets>trompe</ets>, a waterspout, a water-blowing machine.  Cf. <er>Trump</er> a trumpet.]</ety> <def>A blowing apparatus, in which air, drawn into the upper part of a vertical tube through side holes by a stream of water within, is carried down with the water into a box or chamber below which it is led to a furnace.</def> <altsp>[Written also <asp>trompe</asp>, and <asp>trombe</asp>.]</altsp><br/
[<source>1913 Webster</source>]</p>

<p><ent>Trompe</ent><br/
<ent>Tromp</ent><br/
<mhw>{ <hw>Tromp</hw>, <hw>Trompe</hw>  }</mhw>, <pos>n.</pos> <ety>[See <er>Trump</er> a trumpet.]</ety> <def>A trumpet; a trump.</def> <mark>[Obs.]</mark>  <rj><au>Chaucer.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Trompil</ent><br/
<hw>Tromp"il</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[OF. <ets>trompille</ets>, equiv. to F. <ets>trompette</ets> a trumpet.]</ety> <def>An aperture in a tromp.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tron</ent><br/
<hw>Tron</hw> <pr>(?)</pr>, <pos>n.</pos> <def>See 3d <er>Trone</er>, 2.</def> <mark>[Obs. or Scot.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Trona</ent><br/
<hw>Tro"na</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Of Egyptian or North African origin.]</ety> <fld>(Chem. & Min.)</fld> <def>A native double salt, consisting of a combination of neutral and acid sodium carbonate, <chform>Na2CO3.2HNaCO3.2H2O</chform>, occurring as a white crystalline fibrous deposit from certain soda brine springs and lakes; -- called also <altname>urao</altname>, and by the ancients <altname>nitrum</altname>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tronage</ent><br/
<hw>Tron"age</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[From <er>Trone</er> a steelyard.]</ety> <def>A toll or duty paid for weighing wool; also, the act of weighing wool.</def> <mark>[Obs.]</mark>  <rj><au>Nares.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tronator</ent><br/
<hw>Tro*na"tor</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[LL. See <er>Tronage</er>.]</ety> <def>An officer in London whose duty was to weigh wool.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Trone</ent><br/
<hw>Trone</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A throne.</def> <mark>[Obs.]</mark>  <rj><au>Chaucer.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Trone</ent><br/
<hw>Trone</hw>, <pos>n.</pos> <ety>[Cf. Prov. F. <ets>trogne</ets> a belly.]</ety> <def>A small drain.</def> <mark>[Obs. or Prov. Eng.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Trones</ent><br/
<ent>Trone</ent><br/
<mhw>{ <hw>Trone</hw> <pr>(?)</pr>, <hw>Trones</hw> <pr>(?)</pr>, }</mhw> <pos>n.</pos> <ety>[LL. <ets>trona</ets>, fr. L. <ets>trutina</ets> a balance; cf. Gr. <?/.]</ety> <sn>1.</sn> <def>A steelyard.</def> <mark>[Prov. Eng.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A form of weighing machine for heavy wares, consisting of two horizontal bars crossing each other, beaked at the extremities, and supported by a wooden pillar. It is now mostly disused.</def> <mark>[Scot.]</mark>  <rj><au>Jamieson.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Trone stone</b></col>, <cd>a weight equivalent to nineteen and a half pounds.</cd> <mark>[Scot.]</mark> -- <col><b>Trone weight</b></col>, <cd>a weight formerly used in Scotland, in which a pound varied from 21 to 28 ounces avoirdupois.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Troop</ent><br/
<hw>Troop</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F. <ets>troupe</ets>, OF. <ets>trope</ets>, <ets>trupe</ets>, LL. <ets>troppus</ets>; of uncertain origin; cf. Icel. <ets><thorn/orp</ets> a hamlet, village, G. <ets>dorf</ets> a village, dial. G. <ets>dorf</ets> a meeting. Norw. <ets>torp</ets> a little farm, a crowd, E. <ets>thorp</ets>.  Cf. <er>Troupe</er>.]</ety> <sn>1.</sn> <def>A collection of people; a company; a number; a multitude.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>That which should accompany old age --<br/
As honor, love, obedience, <qex>troops</qex> of friends --<br/
I must not look to have.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Soldiers, collectively; an army; -- now generally used in the plural.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Farewell the plumed <qex>troop</qex>, and the big wars.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>His <qex>troops</qex> moved to victory with the precision of machines.</q> <rj><qau>Macaulay.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <fld>(Mil.)</fld> <def>Specifically, a small body of cavalry, light horse, or dragoons, consisting usually of about sixty men, commanded by a captain; the unit of formation of cavalry, corresponding to the <xex>company</xex> in infantry. Formerly, also, a company of horse artillery; a battery.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>A company of stageplayers; a troupe.</def>  <rj><au>W. Coxe.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <fld>(Mil.)</fld> <def>A particular roll of the drum; a quick march.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>6.</sn> <def>See <er>Boy scout</er>, above.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Troop</ent><br/
<hw>Troop</hw>, <pos>v. i.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Trooped</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Trooping</conjf>.]</vmorph> <sn>1.</sn> <def>To move in numbers; to come or gather in crowds or troops.</def> <ldquo/Armies . . . <xex>troop</xex> to their standard.<rdquo/  <rj><au>Milton.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To march on; to go forward in haste.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Nor do I, as an enemy to peace,<br/
<qex>Troop</qex> in the throngs of military men.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Troop</ent><br/
<hw>Troop</hw>, <pos>v. t.</pos> <cs>-- <mcol><col><b>To troop the colors</b></col> <it>or</it>  <col><b>To troop the colours</b></col></mcol> <fld>(Mil.)</fld>, <cd>in the British army, to perform a ceremony consisting essentially in carrying the colors, accompanied by the band and escort, slowly before the troops drawn up in single file and usually in a hollow square, as in London on the sovereign's birthday.</cd></cs><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Troopbird</ent><br/
<hw>Troop"bird`</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Zool.)</fld> <def>Any troupial.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Trooper</ent><br/
<hw>Troop"er</hw>, <pos>n.</pos> <sn>1.</sn> <def>A soldier in a body of cavalry; a cavalryman; also, the horse of a cavalryman.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A state police officer; -- also called <altname>state trooper</altname>.</def> <mark>[U. S.]</mark><br/
[<source>PJC</source>]</p>

<p><sn>3.</sn> <def>A mounted policeman.</def> <mark>[Australia]</mark> <note>The <stype>black troopers</stype> of Queensland are a regiment of aboriginal police, employed chiefly for dispersing wild aborigines who encroach on sheep runs.</note><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><sn>4.</sn> <def>Trouper.</def><br/
[<source>PJC</source>]</p>

<p><cs><col><b>like a trooper</b></col>, <cd>with energy, endurance, or enthusiasm; <as>as, to work <ex>like a trooper</ex></as>.</cd></cs></p>

<p><ent>Troopfowl</ent><br/
<hw>Troop"fowl`</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Zool.)</fld> <def>The American scaup duck.</def> <mark>[Local, U. S.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Troopial</ent><br/
<hw>Troop"i*al</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Zool.)</fld> <def>Same as <er>Troupial</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Troopmeal</ent><br/
<hw>Troop"meal`</hw> <pr>(?)</pr>, <pos>adv.</pos> <ety>[<ets>Troop</ets> + <ets>-meal</ets> as in <ets>piecemeal</ets>.]</ety> <def>By troops; in crowds.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>So, <qex>troopmeal</qex>, Troy pursued a while, laying on with swords and darts.</q> <rj><qau>Chapman.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><-- p. 1544 --><br/
[<source>1913 Webster</source>]</p>

<p><ent>Troopship</ent><br/
<hw>Troop"ship`</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A vessel built or fitted for the conveyance of troops; a transport.</def> <mark>[Eng.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Troostite</ent><br/
<hw>Troost"ite</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[So named after Dr. Gerard <etsep>Troost</etsep>, of Nashville, Tenn.]</ety> <fld>(Min.)</fld> <def>Willemite.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tropaeolin</ent><br/
<hw>Tro*pae"o*lin</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Chem.)</fld> <def>A name given to any one of a series of orange-red dyestuffs produced artificially from certain complex sulphonic acid derivatives of azo and diazo hydrocarbons of the aromatic series; -- so called because of the general resemblance to the shades of nasturtium (<gen>Tropaeolum</gen>).</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Trope</ent><br/
<hw>Trope</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>tropus</ets>, Gr. <?/, fr. <?/ to turn. See <er>Torture</er>, and cf. <er>Trophy</er>, <er>Tropic</er>, <er>Troubadour</er>, <er>Trover</er>.]</ety> <fld>(Rhet.)</fld> <sd>(a)</sd> <def>The use of a word or expression in a different sense from that which properly belongs to it; the use of a word or expression as changed from the original signification to another, for the sake of giving life or emphasis to an idea; a figure of speech.</def> <sd>(b)</sd> <def>The word or expression so used.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>In his frequent, long, and tedious speeches, it has been said that a <qex>trope</qex> never passed his lips.</q> <rj><qau>Bancroft.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ Tropes are chiefly of four kinds: <xex>metaphor</xex>, <xex>metonymy</xex>, <xex>synecdoche</xex>, and <xex>irony</xex>. Some authors make <xex>figures</xex> the genus, of which <xex>trope</xex> is a species; others make them different things, defining <xex>trope</xex> to be a change of sense, and <xex>figure</xex> to be any ornament, except what becomes so by such change.</note><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tropeine</ent><br/
<hw>Tro*pe"ine</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Chem.)</fld> <def>Any one of a series of artificial ethereal salts derived from the alkaloidal base tropine.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Trophi</ent><br/
||<hw>Tro"phi</hw> <pr>(?)</pr>, <pos>n. pl.</pos> <ety>[NL., fr. Gr. <?/ a feeder, fr. <?/ to feed.]</ety> <fld>(Zool.)</fld> <def>The mouth parts of an insect, collectively, including the labrum, labium, maxillae, mandibles, and lingua, with their appendages.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Trophic</ent><br/
<hw>Troph"ic</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[Gr. <?/ nursing. See <er>Trophi</er>.]</ety> <fld>(Physiol.)</fld> <def>Of or connected with nutrition; nitritional; nourishing; <as>as, the so-called <ex>trophic</ex> nerves, which have a direct influence on nutrition</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Trophied</ent><br/
<hw>Tro"phied</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Adorned with trophies.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The <qex>trophied</qex> arches, storied halls, invade.</q> <rj><qau>Pope.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Trophonian</ent><br/
<hw>Tro*pho"ni*an</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>Trophonianus</ets>, fr. <ets>Trophonius</ets>, Gr. <?/, a Grecian architect, fabled to have been the builder of the first temple of Apollo at Delphi. He was worshiped after death, and had a celebrated oracle in a cave in B<oe/otia.]</ety> <def>Of or pertaining to Trophonius, his architecture, or his cave and oracle.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Trophosome</ent><br/
<hw>Troph"o*some</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Gr. <?/ a feeder + <ets>-some</ets> body.]</ety> <fld>(Zool.)</fld> <def>The nutritive zooids of a hydroid, collectively, as distinguished from the gonosome, or reproductive zooids.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Trophosperm</ent><br/
<hw>Troph"o*sperm</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Gr. <?/ a feeder + <?/ seed: cf. F. <ets>trophosperme</ets>. See <er>Trophi</er>.]</ety> <fld>(Bot.)</fld> <def>The placenta.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Trophy</ent><br/
<hw>Tro"phy</hw> <pr>(?)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Trophies</plw> <pr>(#)</pr>.</plu> <ety>[F. <ets>troph<eacute/e</ets> (cf. It. & Sp. <ets>trofeo</ets>), L. <ets>tropaeum</ets>, <ets>trophaeum</ets>, Gr. <?/, strictly, a monument of the enemy's defeat, fr.<?/ a turn, especially, a turning about of the enemy, a putting to flight or routing him, fr. <?/ to turn. See <er>Trope</er>.]</ety> <sn>1.</sn> <fld>(Gr. & Rom. Antiq.)</fld> <def>A sign or memorial of a victory raised on the field of battle, or, in case of a naval victory, on the nearest land. Sometimes trophies were erected in the chief city of the conquered people.</def><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ A trophy consisted originally of some of the armor, weapons, etc., of the defeated enemy fixed to the trunk of a tree or to a post erected on an elevated site, with an inscription, and a dedication to a divinity. The Romans often erected their trophies in the Capitol.</note><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>The representation of such a memorial, as on a medal; esp. <fld>(Arch.)</fld>, an ornament representing a group of arms and military weapons, offensive and defensive.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>Anything taken from an enemy and preserved as a memorial of victory, as arms, flags, standards, etc.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Around the posts hung helmets, darts, and spears,<br/
And captive chariots, axes, shields, and bars,<br/
And broken beaks of ships, the <qex>trophies</qex> of their wars.</q> <rj><qau>Dryden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>Any evidence or memorial of victory or conquest; <as>as, every redeemed soul is a <ex>trophy</ex> of grace</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><-- 5.  An object memorializing a victory in a sporting contest.<br/
[<source>1913 Webster</source>]</p>

<p><note> Some trophies(5) are unique, temporary possession of the same object passing to the new victors of some periodic contest in subsequent occurrences.  Others are objects of little inherent worth, given by the authority sponsoring the contest to the victor.  A trophy is sometimes shaped like a cup, and in such cases may be called a <stype>cup</stype>, as the <stype>America's Cup</stype> (in Yacht racing).</note> --><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Trophy money</b></col>, <cd>a duty paid formerly in England, annually, by housekeepers, toward providing harness, drums, colors, and the like, for the militia.</cd></cs>
<-- Trophy room, a room in which trophies are kept. --><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tropic</ent><br/

<hw>Trop"ic</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[<ets>Atrop</ets>ine + <ets>-ic</ets>.]</ety> <fld>(Chem.)</fld> <def>Of, pertaining to, or designating, an acid obtained from atropine and certain other alkaloids, as a white crystalline substance slightly soluble in water.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tropic</ent><br/
<hw>Trop"ic</hw>, <pos>n.</pos> <ety>[F. <ets>tropique</ets>, L. <ets>tropicus</ets> of or belonging to a turn, <it>i. e.</it>, of the sun, Gr. <?/ of the solstice, <?/ (sc. <?/) the tropic or solstice, fr. <?/ to turn. See <er>Trope</er>.]</ety> <sn>1.</sn> <fld>(Astron.)</fld> <def>One of the two small circles of the celestial sphere, situated on each side of the equator, at a distance of 23<deg/ 28<min/, and parallel to it, which the sun just reaches at its greatest declination north or south, and from which it turns again toward the equator, the northern circle being called the <stype>Tropic of Cancer</stype>, and the southern the <stype>Tropic of Capricorn</stype>, from the names of the two signs at which they touch the ecliptic.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Geog.)</fld> <sd>(a)</sd> <def>One of the two parallels of terrestrial latitude corresponding to the celestial tropics, and called by the same names.</def> <sd>(b)</sd> <pluf>pl.</pluf> <def>The region lying between these parallels of latitude, or near them on either side.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The brilliant flowers of the <qex>tropics</qex> bloom from the windows of the greenhouse and the saloon.</q> <rj><qau>Bancroft.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tropic</ent><br/
<hw>Trop"ic</hw>, <pos>a.</pos> <def>Of or pertaining to the tropics; tropical.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Tropic bird</b></col> <fld>(Zool.)</fld>, <cd>any one of three species of oceanic belonging to the genus <gen>Phaethon</gen>, found chiefly in tropical seas. They are mostly white, and have two central tail feathers very long and slender. The yellow-billed tropic bird. <spn>Phaethon flavirostris</spn> (called also <altname>boatswain</altname>), is found on the Atlantic coast of America, and is common at the Bermudas, where it breeds.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tropical</ent><br/
<hw>Trop"ic*al</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[Cf. L. <ets>tropicus</ets> of turning, Gr. <?/. See <er>Tropic</er>, <pos>n.</pos>]</ety> <sn>1.</sn> <def>Of or pertaining to the tropics; characteristic of, or incident to, the tropics; being within the tropics; <as>as, <ex>tropical</ex> climate; <ex>tropical</ex> latitudes; <ex>tropical</ex> heat; <ex>tropical</ex> diseases.</as></def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <ety>[From <er>Trope</er>.]</ety> <def>Rhetorically changed from its exact original sense; being of the nature of a trope; figurative; metaphorical.</def>  <rj><au>Jer. Taylor.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>The foundation of all parables is some analogy or similitude between the <qex>tropical</qex> or allusive part of the parable and the thing intended by it.</q> <rj><qau>South.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Tropic month</b></col>. <cd>See <cref>Lunar month</cref>, under <er>Month</er>.</cd> -- <col><b>Tropic year</b></col>, <cd>the solar year; the period occupied by the sun in passing from one tropic or one equinox to the same again, having a mean length of 365 days, 5 hours, 48 minutes, 46.0 seconds, which is 20 minutes, 23.3 seconds shorter than the sidereal year, on account of the precession of the equinoxes.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tropically</ent><br/
<hw>Trop"ic*al*ly</hw>, <pos>adv.</pos> <def>In a tropical manner; figuratively; metaphorically.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tropidine</ent><br/
<hw>Trop"i*dine</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[See <er>Tropine</er>.]</ety> <fld>(Chem.)</fld> <def>An alkaloid, <chform>C8H13N</chform>, obtained by the chemical dehydration of tropine, as an oily liquid having a coninelike odor.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tropilidene</ent><br/
<hw>Tro*pil"i*dene</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[See <er>Tropine</er>.]</ety> <fld>(Chem.)</fld> <def>A liquid hydrocarbon obtained by the dry distillation of tropine with quicklime. It is regarded as being homologous with dipropargyl.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tropine</ent><br/
<hw>Tro"pine</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[From <er>Atropine</er>.]</ety> <fld>(Chem.)</fld> <def>A white crystalline alkaloid, <chform>C8H15NO</chform>, produced by decomposing atropine.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tropism</ent><br/
<hw>Tro"pism</hw> <pr>(tr<omac/"p<icr/z'm)</pr>, <pos>n.</pos> <ety>[Gr. <grk>troph`</grk> a turning, <grk>tre`pein</grk> to turn + <ets>-ism</ets>.]</ety> <fld>(Physiol.)</fld> <def>Modification of the direction of growth, caused by some external influence, such as light; -- sometimes used for motion of an organism toward or away from an external stimulus, more properly called <contr>taxis</contr>.</def><br/
[<source>Webster 1913 Suppl.</source> <source>+PJC</source>]</p>

<p><ent>Tropist</ent><br/
<hw>Trop"ist</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Cf. F. <ets>tropiste</ets>. See <er>Trope</er>.]</ety> <def>One who deals in tropes;</def> <specif>specifically,</specif> <def>one who avoids the literal sense of the language of Scripture by explaining it as mere tropes and figures of speech.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tropological</ent><br/
<ent>Tropologic</ent><br/
<mhw>{ <hw>Trop`o*log"ic</hw> <pr>(?)</pr>, <hw>Trop`o*log"ic*al</hw> <pr>(?)</pr>, }</mhw> <pos>a.</pos> <ety>[Gr. <?/: cf. F. <ets>tropologique</ets>. See <er>Tropology</er>.]</ety> <def>Characterized by tropes; varied by tropes; tropical.</def> <au>Burton.</au> -- <wordforms><wf>Trop`o*log"ic*al*ly</wf>, <pos>adv.</pos></wordforms><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tropologize</ent><br/
<hw>Tro*pol"o*gize</hw> <pr>(?)</pr>, <pos>v. t.</pos> <def>To use in a tropological sense, as a word; to make a trope of.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>If . . . Minerva be <qex>tropologized</qex> into prudence.</q> <rj><qau>Cudworth.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tropology</ent><br/
<hw>Tro*pol"o*gy</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Gr. <?/; <?/ a trope + <?/ discourse: cf. F. <ets>tropologie</ets>.]</ety> <def>A rhetorical mode of speech, including tropes, or changes from the original import of the word.</def>  <rj><au>Sir T. Browne.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Troppo</ent><br/
||<hw>Trop"po</hw> <pr>(?)</pr>, <pos>adv.</pos> <ety>[It.]</ety> <fld>(Mus.)</fld> <def>Too much; <as>as, allegro ma non <ex>troppo</ex>, brisk but not too much so</as>.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Trossers</ent><br/
<hw>Tros"sers</hw> <pr>(?)</pr>, <pos>n. pl.</pos> <def>Trousers.</def> <mark>[Obs.]</mark>  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Trot</ent><br/
<hw>Trot</hw> <pr>(?)</pr>, <pos>v. i.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Trotted</conjf>; <pos>p. pr. & vb. n.</pos> <conjf>Trotting</conjf>.]</vmorph> <ety>[OE. <ets>trotten</ets>, OF. <ets>troter</ets>, F. <ets>trotter</ets>; probably of Teutonic origin, and akin to E. <ets>tread</ets>; cf. OHG. <ets>trott<?/n</ets> to tread. See <er>Tread</er>.]</ety> <sn>1.</sn> <def>To proceed by a certain gait peculiar to quadrupeds; to ride or drive at a trot. See <er>Trot</er>, <pos>n.</pos></def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Fig.: To run; to jog; to hurry.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>He that rises late must <qex>trot</qex> all day, and will scarcely overtake his business at night.</q> <rj><qau>Franklin.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Trot</ent><br/
<hw>Trot</hw>, <pos>v. t.</pos> <def>To cause to move, as a horse or other animal, in the pace called a trot; to cause to run without galloping or cantering.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>To trot out</b></col>, <cd>to lead or bring out, as a horse, to show his paces; hence, to bring forward, as for exhibition.</cd> <mark>[Slang.]</mark></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Trot</ent><br/
<hw>Trot</hw>, <pos>n.</pos> <ety>[F. See <er>Trot</er>, <pos>v. i.</pos>]</ety> <sn>1.</sn> <def>The pace of a horse or other quadruped, more rapid than a walk, but of various degrees of swiftness, in which one fore foot and the hind foot of the opposite side are lifted at the same time.</def> <ldquo/The limbs move diagonally in pairs in the <xex>trot</xex>.<rdquo/  <rj><au>Stillman (The Horse in Motion).</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Fig.: A jogging pace, as of a person hurrying.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>One who trots; a child; a woman.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>An old <qex>trot</qex> with ne'er a tooth.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Troth</ent><br/
<hw>Troth</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[A variant of <ets>truth</ets>. See <er>Truth</er>.]</ety> <sn>1.</sn> <def>Belief; faith; fidelity.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Bid her alight<br/
And her<qex>troth</qex> plight.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Truth; verity; veracity; <as>as, by my <ex>troth</ex></as>.</def>  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>In <qex>troth</qex>, thou art able to instruct gray hairs.</q> <rj><qau>Addison.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>Betrothal.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Trothless</ent><br/
<hw>Troth"less</hw>, <pos>a.</pos> <def>Faitless; false; treacherous.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Thrall to the faithless waves and <qex>trothless</qex> sky.</q> <rj><qau>Fairfax.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Trothplight</ent><br/
<hw>Troth"plight`</hw> <pr>(?)</pr>, <pos>v. t.</pos> <def>To betroth.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Trothplight</ent><br/
<hw>Troth"plight`</hw>, <pos>a.</pos> <def>Betrothed; espoused; affianced.</def> <mark>[Obs.]</mark>  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Trothplight</ent><br/
<hw>Troth"plight`</hw>, <pos>n.</pos> <def>The act of betrothing, or plighting faith; betrothing.</def> <mark>[Obs.]</mark>  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Trothplighted</ent><br/
<hw>Troth"plight`ed</hw>, <pos>a.</pos> <def>Having fidelity pledged.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Trotter</ent><br/
<hw>Trot"ter</hw> <pr>(?)</pr>, <pos>n.</pos> <sn>1.</sn> <def>One that trots; especially, a horse trained to be driven in trotting matches.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>The foot of an animal, especially that of a sheep; also, humorously, the human foot.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Trottoir</ent><br/
<hw>Trot"toir</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F., from <ets>trotter</ets> to trot. See <er>Trot</er>.]</ety> <def>Footpath; pavement; sidewalk.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Headless bodies trailed along the <qex>trottoirs</qex>.</q> <rj><qau>Froude.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Troubadour</ent><br/
<hw>Trou"ba*dour`</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F. <ets>troubadour</ets>, fr. Pr. <ets>trobador</ets>, (assumed) LL. <ets>tropator</ets> a singer, <ets>tropare</ets> to sing, fr. <ets>tropus</ets> a kind of singing, a melody, song, L. <ets>tropus</ets> a trope, a song, Gr. <?/ a turn, way, manner, particular mode in music, a trope. See <er>Trope</er>, and cf. <er>Trouv<?/re</er>.]</ety> <def>One of a school of poets who flourished from the eleventh to the thirteenth century, principally in Provence, in the south of France, and also in the north of Italy. They invented, and especially cultivated, a kind of lyrical poetry characterized by intricacy of meter and rhyme, and usually of a romantic, amatory strain.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Troublable</ent><br/
<hw>Trou"bla*ble</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Causing trouble; troublesome.</def> <mark>[Obs.]</mark> <ldquo/<xex>troublable</xex> ire.<rdquo/  <rj><au>Chaucer.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Trouble</ent><br/
<hw>Trou"ble</hw> <pr>(?)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Troubled</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Troubling</conjf>.]</vmorph> <ety>[F. <ets>troubler</ets>, OF. <ets>trobler</ets>, <ets>trubler</ets>, <ets>tourbler</ets>,fr. (assumed) LL. <ets>turbulare</ets>, L. <ets>turbare</ets> to disorderly group, a little crowd; both from <ets>turba</ets> a disorder, tumult, crowd; akin to Gr. <?/, and perhaps to E. <ets>thorp</ets>; cf. Skr. <ets>tvar</ets>, <ets>tur</ets>,o hasten.  Cf. <er>Turbid</er>.]</ety> <sn>1.</sn> <def>To put into confused motion; to disturb; to agitate.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>An angel went down at a certain season into the pool, and <qex>troubled</qex> the water.</q> <rj><qau>John v. 4.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>God looking forth will <qex>trouble</qex> all his host.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To disturb; to perplex; to afflict; to distress; to grieve; to fret; to annoy; to vex.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Now is my soul <qex>troubled</qex>.</q> <rj><qau>John xii. 27.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Take the boy to you; he so <qex>troubles</qex> me<br/
'T is past enduring.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Never <qex>trouble</qex> yourself about those faults which age will cure.</q> <rj><qau>Locke.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>To give occasion for labor to; -- used in polite phraseology; <as>as, I will not <ex>trouble</ex> you to deliver the letter</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><syn><b>Syn.</b> -- To disturb; perplex; afflict; distress; grieve; harass; annoy; tease; vex; molest.</syn><br/
[<source>1913 Webster</source>]</p>

<p><ent>Trouble</ent><br/
<hw>Trou"ble</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Troubled; dark; gloomy.</def> <mark>[Obs.]</mark> <ldquo/With full <xex>trouble</xex> cheer.<rdquo/  <rj><au>Chaucer.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Trouble</ent><br/
<hw>Trou"ble</hw>, <pos>n.</pos> <ety>[F. <ets>trouble</ets>, OF. <ets>troble</ets>, <ets>truble</ets>. See <er>Trouble</er>, <pos>v. t.</pos>]</ety> <sn>1.</sn> <def>The state of being troubled; disturbance; agitation; uneasiness; vexation; calamity.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Lest the fiend . . . some new <qex>trouble raise</qex>.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Foul whisperings are abroad; unnatural deeds<br/
Do breed unnatural <qex>troubles</qex>.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>That which gives disturbance, annoyance, or vexation; that which afflicts.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <fld>(Mining)</fld> <def>A fault or interruption in a stratum.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>To get into trouble</b></col>, <cd>to get into difficulty or danger.</cd> <mark>[Colloq.]</mark> -- <col><b>To take the trouble</b></col>, <cd>to be at the pains; to exert one's self; to give one's self inconvenience.</cd><br/
[<source>1913 Webster</source>]</p>

<p><q>She never <qex>took the trouble</qex> to close them.</q> <rj><qau>Bryant.</qau></rj>
</cs><br/
[<source>1913 Webster</source>]</p>

<p><syn><b>Syn.</b> -- Affliction; disturbance; perplexity; annoyance; molestation; vexation; inconvenience; calamity; misfortune; adversity; embarrassment; anxiety; sorrow; misery.</syn><br/
[<source>1913 Webster</source>]</p>

<p><ent>Troubler</ent><br/
<hw>Trou"bler</hw> <pr>(?)</pr>, <pos>n.</pos> <def>One who troubles or disturbs; one who afflicts or molests; a disturber; <as>as, a <ex>troubler</ex> of the peace</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The rich <qex>troublers</qex> of the world's repose.</q> <rj><qau>Waller.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Troublesome</ent><br/
<hw>Trou"ble*some</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Giving trouble or anxiety; vexatious; burdensome; wearisome.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>This <qex>troublesome</qex> world.</q> <rj><qau>Book of Common Prayer.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>These <qex>troublesome</qex> disguises that we wear.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>My mother will never be <qex>troublesome</qex> to me.</q> <rj><qau>Pope.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><syn><b>Syn.</b> -- Uneasy; vexatious; perplexing; harassing; annoying; disgusting; irksome; afflictive; burdensome; tiresome; wearisome; importunate.</syn><br/
[<source>1913 Webster</source>]</p>

<p>-- <wordforms><wf>Trou"ble*some*ly</wf>, <pos>adv.</pos> -- <wf>Trou"ble*some*ness</wf>, <pos>n.</pos></wordforms><br/
[<source>1913 Webster</source>]</p>

<p><ent>Troublous</ent><br/
<hw>Trou"blous</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Full of trouble; causing trouble.</def> <ldquo/In doubtful time of <xex>troublous</xex> need.<rdquo/  <rj><au>Byron.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>A tall ship tossed in <qex>troublous</qex> seas.</q> <rj><qau>Spenser.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Trou-de-loup</ent><br/
||<hw>Trou"-de-loup"</hw> <pr>(?)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Trous-de-loup</plw> <pr>(<?/)</pr>.</plu> <ety>[F. <ets>trou</ets> hole + <ets>de</ets> of + <ets>loup</ets> wolf.]</ety> <fld>(Mil.)</fld> <def>A pit in the form of an inverted cone or pyramid, constructed as an obstacle to the approach of an enemy, and having a pointed stake in the middle. The pits are called also <altname>trapholes</altname>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Trough</ent><br/
<hw>Trough</hw> <pr>(tr<ocr/f)</pr>, <pos>n.</pos> <ety>[OE. <ets>trough</ets>, <ets>trogh</ets>, AS. <ets>trog</ets>, <ets>troh</ets>; akin to D., G., & Icel. <ets>trog</ets>, Sw. <ets>tr<aring/g</ets>, Dan. <ets>trug</ets>; probably originally meaning, made of wood, and akin to E. <ets>tree</ets>. <root/63 & 241. See <er>Tree</er>, and cf. <er>Trug</er>.]</ety> <sn>1.</sn> <def>A long, hollow vessel, generally for holding water or other liquid, especially one formed by excavating a log longitudinally on one side; a long tray; also, a wooden channel for conveying water, as to a mill wheel.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Any channel, receptacle, or depression, of a long and narrow shape; <as>as, <ex>trough</ex> between two ridges, etc</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <fld>(Meteor.)</fld> <def>The transverse section of a cyclonic area where the barometric pressure, neither rising nor falling, has reached its lowest point.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><cs><col><b>Trough gutter</b></col> <fld>(Arch.)</fld>, <cd>a rectangular or V-shaped gutter, usually hung below the eaves of a house.</cd> -- <col><b>Trough of the sea</b></col>, <cd>the depression between two waves.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Trough-shell</ent><br/
<hw>Trough"-shell`</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Zool.)</fld> <def>Any bivalve shell of the genus Mactra. See <er>Mactra</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Troul</ent><br/
<hw>Troul</hw> <pr>(?)</pr>, <pos>v. t. & i.</pos> <def>See <er>Troll</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Trounce</ent><br/
<hw>Trounce</hw> <pr>(?)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Trounced</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Trouncing</conjf> <pr>(?)</pr>.]</vmorph> <ety>[F. <ets>tronce</ets>, <ets>tronche</ets>, a stump, piece of wood. See <er>Truncheon</er>.]</ety> <def>To punish or beat severely; to whip smartly; to flog; to castigate.</def> <mark>[Colloq.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Troupe</ent><br/
||<hw>Troupe</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F., troop. See <er>Troop</er>.]</ety> <def>A company or troop, especially the company pf performers in a play or an opera.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Troupial</ent><br/
<hw>Troup"i*al</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F. <ets>troupiale</ets>.]</ety> <fld>(Zool.)</fld> <def>Any one of numerous species of bright-colored American birds belonging to <gen>Icterus</gen> and allied genera, especially <spn>Icterus icterus</spn>, a native of the West Indies and South America. Many of the species are called <stype>orioles</stype> in America.</def> <altsp>[Written also <asp>troopial</asp>.]</altsp><br/
[<source>1913 Webster</source>]</p>

<p><ent>Trouse</ent><br/
<hw>Trouse</hw> <pr>(?)</pr>, <pos>n.</pos> <def>Trousers.</def> <mark>[Obs.]</mark>  <rj><au>Spenser.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Trousering</ent><br/
<hw>Trou"ser*ing</hw> <pr>(?)</pr>, <pos>n.</pos> <def>Cloth or material for making trousers.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Trousers</ent><br/
<hw>Trou"sers</hw> <pr>(?)</pr>, <pos>n. pl.</pos> <ety>[OF. <ets>trousses</ets> breeches worn by pages, from <ets>trousse</ets>, <ets>trosse</ets>, a bundle, a truss. See <er>Truss</er>, and cf. <er>Trossers</er>, <er>Trouse</er>.]</ety> <def>A garment worn by men and boys, extending from the waist to the knee or to the ankle, and covering each leg separately.</def>
<--; also, <altname>pants</altname>; used attrib. in the singular, as a <xex>trouser</xex> leg; see <er>pant</er>  --><br/
[<source>1913 Webster</source>]</p>

<p><ent>Trousse</ent><br/
||<hw>Trousse</hw> <pr>(tr<oomac/s)</pr>, <pos>n.</pos> <ety>[F. See <er>Truss</er>; cf. <er>Trousseau</er>.]</ety> <def>A case for small implements; <as>as, a surgeon's <ex>trousse</ex></as>.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Trousseau</ent><br/
||<hw>Trous`seau"</hw> <pr>(tr<oomac/`s<omac/")</pr>, <pos>n.</pos> <ety>[F., fr. OF. <ets>trossel</ets>, dim. of <ets>trousse</ets> a bundle, truss. See <er>Truss</er>.]</ety> <def>The collective lighter equipments or outfit of a bride, including clothes, jewelry, and the like; especially, that which is provided for her by her family.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Trout</ent><br/
<hw>Trout</hw> <pr>(trout)</pr>, <pos>n.</pos> <ety>[AS. <ets>truht</ets>, L. <ets>tructa</ets>, <ets>tructus</ets>; akin to Gr. <grk>trw`kths</grk> a sea fish with sharp teeth, fr. <grk>trw`gein</grk> to gnaw.]</ety> <sn>1.</sn> <fld>(Zool.)</fld> <def>Any one of numerous species of fishes belonging to <gen>Salmo</gen>, <gen>Salvelinus</gen>, and allied genera of the family <fam>Salmonidae</fam>. They are highly esteemed as game fishes and for the quality of their flesh. All the species breed in fresh water, but after spawning many of them descend to the sea if they have an opportunity.</def><br/
[<source>1913 Webster</source>]</p>

<p><-- p. 1545 --><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ The most important European species are the river, or brown, trout (<spn>Salmo fario</spn>), the salmon trout, and the sewen. The most important American species are the brook, speckled, or red-spotted, trout (<spn>Salvelinus fontinalis</spn>) of the Northern United States and Canada; the red-spotted trout, or Dolly Varden (see <er>Malma</er>); the lake trout (see <er>Namaycush</er>); the black-spotted, mountain, or silver, trout (<spn>Salmo purpuratus</spn>); the golden, or rainbow, trout (see under <er>Rainbow</er>); the blueback trout (see <er>Oquassa</er>); and the salmon trout (see under <er>Salmon</er>.) The European trout has been introduced into America.</note><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Zool.)</fld> <def>Any one of several species of marine fishes more or less resembling a trout in appearance or habits, but not belonging to the same family, especially the California rock trouts, the common squeteague, and the southern, or spotted, squeteague; -- called also <altname>salt-water trout</altname>, <altname>sea trout</altname>, <altname>shad trout</altname>, and <altname>gray trout</altname>. See <er>Squeteague</er>, and <cref>Rock trout</cref> under <er>Rock</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Trout perch</b></col> <fld>(Zool.)</fld>, <cd>a small fresh-water American fish (<spn>Percopsis guttatus</spn>), allied to the trout, but resembling a perch in its scales and mouth.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Troutbird</ent><br/
<hw>Trout"bird`</hw> <pr>(trout"b<etil/rd`)</pr>, <pos>n.</pos> <fld>(Zool.)</fld> <def>The American golden plover.</def> <mark>[Local, U. S.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Trout-colored</ent><br/
<hw>Trout"-col`ored</hw> <pr>(-k<ucr/l`<etil/rd)</pr>, <pos>a.</pos> <def>White, with spots of black, bay, or sorrel; <as>as, a <ex>trout-colored</ex> horse</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Troutlet</ent><br/
<hw>Trout"let</hw> <pr>(-l<ecr/t)</pr>, <pos>n.</pos> <def>A little trout; a troutling.</def>  <rj><au>Hood.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Troutling</ent><br/
<hw>Trout"ling</hw> <pr>(-l<icr/ng)</pr>, <pos>n.</pos> <def>A little trout; a troutlet.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Trouveur</ent><br/
<ent>Trouvere</ent><br/
<mhw>{ ||<hw>Trou`v<egrave/re"</hw> <pr>(?)</pr>, ||<hw>Trou`veur"</hw> <pr>(?)</pr>, }</mhw> <pos>n.</pos> <ety>[F. <ets>trouveur</ets>, <ets>trouv<egrave/re</ets>. See <er>Troubadour</er>.]</ety> <def>One of a school of poets who flourished in Northern France from the eleventh to the fourteenth century.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Trover</ent><br/
<hw>Tro"ver</hw> <pr>(tr<omac/"v<etil/r)</pr>, <pos>n.</pos> <ety>[OF. <ets>trover</ets>, <ets>truver</ets>, to find, F. <ets>trouver</ets>; probably originally, to invent or compose (melodies), fr. (assumed) LL. <ets>tropare</ets>. See <er>Troubadour</er>, <er>Trope</er>, and cf. <er>Contrive</er>, <er>Reirieve</er>, <er>Trouveur</er>.]</ety> <fld>(Law)</fld> <sd>(a)</sd> <def>The gaining possession of any goods, whether by finding or by other means.</def> <sd>(b)</sd> <def>An action to recover damages against one who found goods, and would not deliver them to the owner on demand; an action which lies in any case to recover the value of goods wrongfully converted by another to his own use. In this case the <xex>finding</xex>, though alleged, is an immaterial fact; the injury lies in the <xex>conversion</xex>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Trow</ent><br/
<hw>Trow</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A boat with an open well amidships. It is used in spearing fish.</def>  <rj><au>Knight.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Trow</ent><br/
<hw>Trow</hw> <pr>(?)</pr>, <pos>v. i. & t.</pos> <ety>[OE. <ets>trowen</ets>, AS. <ets>tre<oacute/wan</ets> to trust, believe, fr. <ets>tre<oacute/w</ets> trust, <ets>tre<oacute/we</ets> true, faithful. See <er>True</er>.]</ety> <def>To believe; to trust; to think or suppose.</def> <mark>[Archaic]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>So that ye <qex>trow</qex> in Christ, and you baptize.</q> <rj><qau>Chaucer.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>A better priest, I <qex>trow</qex>, there nowhere none is.</q> <rj><qau>Chaucer.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>It never yet was worn, I <qex>trow</qex>.</q> <rj><qau>Tennyson.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ <xex>I trow</xex>, or <xex>trow</xex> alone, was formerly sometimes added to questions to express contemptuous or indignant surprise.</note><br/
[<source>1913 Webster</source>]</p>

<p><q>What tempest, I <qex>trow</qex>, threw this whale . . . ashore?</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>What is the matter, <qex>trow</qex>?</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Trowel</ent><br/
<hw>Trow"el</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[OE. <ets>truel</ets>, OF. <ets>truele</ets>, F. <ets>truelle</ets>, LL. <ets>truella</ets>, L. <ets>trulla</ets>, dim. of <ets>trua</ets> a ladle; probably akin to Gr. <?/ a stirrer, ladle, G. <ets>quirl</ets> a stirrer, MHG. <ets>twirel</ets>, OHG. <ets>dwiril</ets>, Icel. <ets><thorn/vara</ets>, AS. <ets><thorn/wiril</ets>.  Cf. <er>Twirl</er>.]</ety> <sn>1.</sn> <def>A mason's tool, used in spreading and dressing mortar, and breaking bricks to shape them.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A gardener's tool, somewhat like a scoop, used in taking up plants, stirring the earth, etc.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <fld>(Founding)</fld> <def>A tool used for smoothing a mold.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Trowel bayonet</b></col>. <cd>See <cref>Spade bayonet</cref>, under <er>Spade</er>.</cd> -- <col><b>Fish trowel</b></col>. <cd>See <cref>Fish slice</cref>, under <er>Fish</er>.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Troweled</ent><br/
<hw>Trow"eled</hw> <pr>(?)</pr>. <def>Formed with a trowel; smoothed with a trowel; <as>as, <ex>troweled</ex> stucco, that is, stucco laid on and ready for the reception of paint</as>.</def> <altsp>[Written also <asp>trowelled</asp>.]</altsp><br/
[<source>1913 Webster</source>]</p>

<p><ent>Trowelful</ent><br/
<hw>Trow"el*ful</hw> <pr>(?)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Trowelfuls</plw> <pr>(<?/)</pr>.</plu> <def>As much as a trowel will hold; enough to fill a trowel.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Trowl</ent><br/
<hw>Trowl</hw> <pr>(?)</pr>, <pos>n.</pos> <def>See <er>Troll</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Trowsed</ent><br/
<hw>Trowsed</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Wearing trousers.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Trowsers</ent><br/
<hw>Trow"sers</hw> <pr>(?)</pr>, <pos>n. pl.</pos> <def>Same as <er>Trousers</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Troy</ent><br/
<hw>Troy</hw> <pr>(?)</pr>, <pos>n.</pos> <def>Troy weight.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Troy weight</b></col>, <cd>the weight which gold and silver, jewels, and the like, are weighed. It was so named from <etsep>Troyes</etsep>, in France, where it was first adopted in Europe. The troy ounce is supposed to have been brought from Cairo during the crusades. In this weight the pound is divided into 12 ounces, the ounce into 20 pennyweights, and the pennyweight into 24 grains; hence, the troy ounce contains 480 grains, and the troy pound contains 5760 grains. The avoirdupois pound contains 7000 troy grains; so that 175 pounds troy equal 144 pounds avoirdupois, or 1 pound troy = 0.82286 of a pound avoirdupois, and 1 ounce troy = 1<frac17x175/ or 1.09714 ounce avoirdupois. Troy weight when divided, the pound into 12 ounces, the ounce into 8 drams, the dram into 3 scruples, and the scruple into 20 grains, is called <xex>apothecaries' weight</xex>, used in weighing medicines, etc. In the standard weights of the United States, the troy ounce is divided decimally down to the <frac1x10000/ part.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Troyounce</ent><br/
<hw>Troy"ounce</hw> <pr>(?)</pr>, <pos>n.</pos> <def>See <cref>Troy ounce</cref>, under <cref>Troy weight</cref>, above, and under <er>Ounce</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Truage</ent><br/
<hw>Tru"age</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Cf. OF. <ets>truage</ets> a tax. See <er>True</er>.]</ety> <sn>1.</sn> <def>A pledge of truth or peace made on payment of a tax.</def> <mark>[Obs.]</mark>  <rj><au>Ld. Berners.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A tax or impost; tribute.</def> <mark>[Obs.]</mark>  <rj><au>R. of Gloucester.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Truancy</ent><br/
<hw>Tru"an*cy</hw> <pr>(?)</pr>, <pos>n.</pos> <def>The act of playing truant, or the state of being truant; <as>as, addicted to <ex>truancy</ex></as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Truand</ent><br/
<hw>Tru"and</hw> <pr>(-<ait/nd)</pr>, <pos>n. & a.</pos> <def>See <er>Truant</er>.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Truant</ent><br/
<hw>Tru"ant</hw>, <pos>n.</pos> <ety>[F. <ets>truand</ets>, OF. <ets>truant</ets>, a vagrant, beggar; of Celtic origin; cf. W. <ets>tru</ets>, <ets>truan</ets>, wretched, miserable, <ets>truan</ets> a wretch, Ir. <ets>trogha</ets> miserable, Gael. <ets>truaghan</ets> a poor, distressed, or wretched creature, <ets>truagh</ets> wretched.]</ety> <def>One who stays away from business or any duty; especially, one who stays out of school without leave; an idler; a loiterer; a shirk.</def>  <rj><au>Dryden.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>I have a <qex>truant</qex> been to chivalry.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>To play truant</b></col>, <cd>to stray away; to loiter; especially, to stay out of school without leave.</cd>  <rj><au>Sir T. Browne</au></rj></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Truant</ent><br/
<hw>Tru"ant</hw>, <pos>a.</pos> <def>Wandering from business or duty; loitering; idle, and shirking duty; <as>as, a <ex>truant</ex> boy</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>While <qex>truant</qex> Jove, in infant pride,<br/
Played barefoot on Olympus' side.</q> <rj><qau>Trumbull.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Truant</ent><br/
<hw>Tru"ant</hw>, <pos>v. i.</pos> <ety>[Cf. F. <ets>truander</ets>.]</ety> <def>To idle away time; to loiter, or wander; to play the truant.</def>  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>By this means they lost their time and <qex>truanted</qex> on the fundamental grounds of saving knowledge.</q> <rj><qau>Lowell.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Truant</ent><br/
<hw>Tru"ant</hw>, <pos>v. t.</pos> <def>To idle away; to waste.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>I dare not be the author<br/
Of <qex>truanting</qex> the time.</q> <rj><qau>Ford.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Truantly</ent><br/
<hw>Tru"ant*ly</hw>, <pos>adv.</pos> <def>Like a truant; in idleness.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Truantship</ent><br/
<hw>Tru"ant*ship</hw>, <pos>n.</pos> <def>The conduct of a truant; neglect of employment; idleness; truancy.</def>  <rj><au>Ascham.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Trub</ent><br/
<hw>Trub</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Cf. <er>Truffle</er>.]</ety> <def>A truffle.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Trubtall</ent><br/
<hw>Trub"tall`</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Prov. E. <ets>trub</ets> slut; cf. Sw. <ets>trubbig</ets> stumpy.]</ety> <def>A short, squat woman.</def> <mark>[Obs.]</mark>  <rj><au>Ainsworth.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Trubu</ent><br/
<hw>Tru*bu"</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Zool.)</fld> <def>An East India herring (<spn>Clupea toli</spn>) which is extensively caught for the sake of its roe and for its flesh.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Truce</ent><br/
<hw>Truce</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[OE. <ets>trewes</ets>, <ets>triwes</ets>, <ets>treowes</ets>, pl. of <ets>trewe</ets> a truce, properly, pledge of fidelity, truth, AS. <ets>tre<oacute/w</ets> fidelity, faith, troth. See <er>True</er>.]</ety> <sn>1.</sn> <fld>(Mil.)</fld> <def>A suspension of arms by agreement of the commanders of opposing forces; a temporary cessation of hostilities, for negotiation or other purpose; an armistice.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Hence, intermission of action, pain, or contest; temporary cessation; short quiet.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Where he may likeliest find<br/
<qex>Truce</qex> to his restless thoughts.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Flag of truce</b></col> <fld>(Mil.)</fld>, <cd>a white flag carried or exhibited by one of the hostile parties, during the flying of which hostilities are suspended.</cd> -- <col><b>Truce of God</b></col>, <cd>a suspension of arms promulgated by the church, which occasionally took place in the Middle Ages, putting a stop to private hostilities at or within certain periods.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Trucebreaker</ent><br/
<hw>Truce"break`er</hw> <pr>(?)</pr>, <pos>n.</pos> <def>One who violates a truce, covenant, or engagement.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Truceless</ent><br/
<hw>Truce"less</hw>, <pos>a.</pos> <def>Without a truce; unforbearing.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Two minds in one, and each a <qex>truceless</qex> guest.</q> <rj><qau>H. Brooke.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Truchman</ent><br/
<hw>Truch"man</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Cf. F. <ets>trucheman</ets>. See <er>Dragoman</er>.]</ety> <def>An interpreter. See <er>Dragoman</er>.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>And after, by the tongue,<br/
Her <qex>truchman</qex>, she reports the mind's each throw.</q> <rj><qau>B. Jonson.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Trucidation</ent><br/
<hw>Tru`ci*da"tion</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>trucidatio</ets>, fr. <ets>trucidare</ets> to slaughter.]</ety> <def>The act of killing.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Truck</ent><br/
<hw>Truck</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>trochus</ets> an iron hoop, Gr. <?/ a wheel, fr. <?/ to run. See <er>Trochee</er>, and cf. <er>Truckle</er>, <pos>v. i.</pos>]</ety> <sn>1.</sn> <def>A small wheel, as of a vehicle; specifically <fld>(Ord.)</fld>, a small strong wheel, as of wood or iron, for a gun carriage.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A low, wheeled vehicle or barrow for carrying goods, stone, and other heavy articles.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Goods were conveyed about the town almost exclusively in <qex>trucks</qex> drawn by dogs.</q> <rj><qau>Macaulay.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <fld>(Railroad Mach.)</fld> <def>A swiveling carriage, consisting of a frame with one or more pairs of wheels and the necessary boxes, springs, etc., to carry and guide one end of a locomotive or a car; -- sometimes called <xex>bogie</xex> in England. Trucks usually have four or six wheels.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <fld>(Naut.)</fld> <sd>(a)</sd> <def>A small wooden cap at the summit of a flagstaff or a masthead, having holes in it for reeving halyards through.</def> <sd>(b)</sd> <def>A small piece of wood, usually cylindrical or disk-shaped, used for various purposes.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <def>A freight car.</def> <mark>[Eng.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><sn>6.</sn> <def>A frame on low wheels or rollers; -- used for various purposes, as for a movable support for heavy bodies.</def><-- = MW10 2nd Truck 3(c) --><br/
[<source>1913 Webster</source>]</p>

<p><sn>7.</sn> <def>a motorized vehicle larger than an automobile with a compartment in front for the driver, behind which is a separate compartment for freight;</def> <specif>esp.</specif> <sd>(a)</sd> <def>such a vehicle with an inflexible body.</def> <sd>(b)</sd> <def>A vehicle with a short body and a support for attaching a trailer; -- also called a <er>tractor{4}</er>.</def> <sd>(c)</sd> <def>the combination of tractor and trailer, also called a <stype>tractor-trailer</stype> (a form of articulated vehicle); it is a common form of <ex>truck</ex>, and is used primarily for hauling freight on a highway.</def> <sd>(d)</sd> <def>a tractor with more than one trailer attached in a series.  In Australia, often referred to as a <altname>road train</altname>.</def><br/
[<source>PJC</source>]</p>

<p><-- Truck, n. 1. barter.  2. commodidites for barter or for small trade. 3. association, interaction, or connection, as in "I'll have no truck with the likes of them."  4. payment of wages in goods, rather than cash. 5. vegetable grown for market, as in truck farm. 6. small articles of little value.
  [All from MW10.  No comparable definitions in W1913!] --><br/
[<source>1913 Webster</source>]</p>

<p><ent>Truck</ent><br/
<hw>Truck</hw>, <pos>v. t.</pos> <def>To transport on a truck or trucks.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Truck</ent><br/
<hw>Truck</hw>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Trucked</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>trucking</conjf>.]</vmorph> <ety>[OE. <ets>trukken</ets>,F. <ets>troquer</ets>; akin to Sp. & Pg. <ets>trocar</ets>; of uncertain origin.]</ety> <def>To exchange; to give in exchange; to barter; <as>as, to <ex>truck</ex> knives for gold dust</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>We will begin by supposing the international trade to be in form, what it always is in reality, an actual <qex>trucking</qex> of one commodity against another.</q> <rj><qau>J. S. Mill.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Truck</ent><br/
<hw>Truck</hw>, <pos>v. i.</pos> <def>To exchange commodities; to barter; to trade; to deal.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>A master of a ship, who deceived them under color of <qex>trucking</qex> with them.</q> <rj><qau>Palfrey.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Despotism itself is obliged to <qex>truck</qex> and huckster.</q> <rj><qau>Burke.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>To <qex>truck</qex> and higgle for a private good.</q> <rj><qau>Emerson.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Truck</ent><br/
<hw>Truck</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Cf. F. <ets>troc</ets>.]</ety> <sn>1.</sn> <def>Exchange of commodities; barter.</def>  <rj><au>Hakluyt.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Commodities appropriate for barter, or for small trade; small commodities; esp., in the United States, garden vegetables raised for the market.</def> <mark>[Colloq.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>The practice of paying wages in goods instead of money; -- called also <altname>truck system</altname>.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Garden truck</b></col>, <cd>vegetables raised for market. <mark>[Colloq.]</mark> <mark>[U. S.]</mark> -- <col><b>Truck farming</b></col>, raising vegetables for market: market gardening.</cd> <mark>[Colloq. U. S.]</mark></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Truckage</ent><br/
<hw>Truck"age</hw> <pr>(?)</pr>, <pos>n.</pos> <def>The practice of bartering goods; exchange; barter; truck.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The <qex>truckage</qex> of perishing coin.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Truckage</ent><br/
<hw>Truck"age</hw>, <pos>n.</pos> <def>Money paid for the conveyance of goods on a truck; freight.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Trucker</ent><br/
<hw>Truck"er</hw> <pr>(?)</pr>, <pos>n.</pos> <def>One who trucks; a trafficker.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>No man having ever yet driven a saving bargain with this great <qex>trucker</qex> for souls.</q> <rj><qau>South.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Trucking</ent><br/
<hw>Truck"ing</hw>, <pos>n.</pos> <def>The business of conveying goods on trucks.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Truckle</ent><br/
<hw>Truc"kle</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Dim. of <ets>truck</ets> a wheel; or from the kindred L. <ets>trochlea</ets> a block, sheaf containing one or more pulleys.  See <er>Truck</er> a wheel.]</ety> <def>A small wheel or caster.</def>  <rj><au>Hudibras.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Truckle</ent><br/
<hw>Truc"kle</hw>, <pos>v. i.</pos> <ety>[From <ets>truckle</ets> in <ets>truckle-bed</ets>, in allusion to the fact that the truckle-bed on which the pupil slept was rolled under the large bed of the master.]</ety> <def>To yield or bend obsequiously to the will of another; to submit; to creep.</def> <ldquo/Small, <xex>trucking</xex> states.<rdquo/  <rj><au>Burke.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Religion itself is forced to <qex>truckle</qex> to worldly poliey.</q> <rj><qau>Norris.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Truckle</ent><br/
<hw>Truc"kle</hw>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Truckled</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Truckling</conjf> <pr>(?)</pr>.]</vmorph> <def>To roll or move upon truckles, or casters; to trundle.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Truckle-bed</ent><br/
<hw>Truc"kle-bed`</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A low bed on wheels, that may be pushed under another bed; a trundle-bed.</def> <ldquo/His standing bed and <xex>truckle-bed</xex>.<rdquo/  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Truckler</ent><br/
<hw>Truc"kler</hw> <pr>(?)</pr>, <pos>n.</pos> <def>One who truckles, or yields servilely to the will of another.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Truckman</ent><br/
<hw>Truck"man</hw> <pr>(?)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Truckmen</plw> <pr>(#)</pr>.</plu> <sn>1.</sn> <ety>[From <er>Truck</er> barter.]</ety> <def>One who does business in the way of barter or exchange.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <ety>[From <er>Truck</er> a carriage.]</ety> <def>One who drives a truck, or whose business is the conveyance of goods on trucks.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Truculency</ent><br/
<ent>Truculence</ent><br/
<mhw>{ <hw>Tru"cu*lence</hw> <pr>(?)</pr>, <hw>Tru"cu*len*cy</hw> <pr>(?)</pr>, }</mhw> <pos>n.</pos> <ety>[L. <ets>truculentia</ets>.]</ety> <def>The quality or state of being truculent; savageness of manners; ferociousness.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Truculent</ent><br/
<hw>Tru"cu*lent</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>truculentus</ets>, fr. <ets>trux</ets>, gen. <ets>trucis</ets>, wild, fierce: cf. F. <ets>truculent</ets>.]</ety> <sn>1.</sn> <def>Fierce; savage; ferocious; barbarous; <as>as, the <ex>truculent</ex> inhabitants of Scythia</as>.</def>  <rj><au>Ray.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Cruel; destructive; ruthless.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>More or less <qex>truculent</qex> plagues.</q> <rj><qau>Harvey.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Truculently</ent><br/
<hw>Tru"cu*lent*ly</hw>, <pos>adv.</pos> <def>In a truculent manner.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Trudge</ent><br/
<hw>Trudge</hw> <pr>(?)</pr>, <pos>v. i.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Trudged</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Trudging</conjf>.]</vmorph> <ety>[Perhaps of Scand. origin, and originally meaning, to walk on snowshoes; cf. dial. Sw. <ets>truga</ets>, <ets>trudja</ets>, a snowshoe, Norw. <ets>truga</ets>, Icel. <ets><thorn/r<umac/ga</ets>.]</ety> <def>To walk or march with labor; to jog along; to move wearily.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>And <qex>trudged</qex> to Rome upon my naked feet.</q> <rj><qau>Dryden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Trudgeman</ent><br/
<hw>Trudge"man</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A truchman.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Trudgen stroke</ent><br/
<hw>Trudg"en stroke</hw> <pr>(?)</pr>. <fld>(Swimming)</fld> <def>A racing stroke in which a double over-arm motion is used; -- so called from its use by an amateur named <person>Trudgen</person>, but often erroneously written <asp>trudgeon</asp>.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>True</ent><br/
<hw>True</hw> <pr>(tr<udd/)</pr>, <pos>a.</pos> <amorph>[<pos>Compar.</pos> <adjf>Truer</adjf> <pr>(tr<udd/"<etil/r)</pr>; <pos>superl.</pos> <adjf>Truest</adjf>.]</amorph> <ety>[OE. <ets>trewe</ets>, AS. <ets>tre<oacute/we</ets> faithful, true, from <ets>tre<oacute/w</ets> fidelity, faith, troth; akin to OFries. <ets>triuwe</ets>, adj., <ets>treuwa</ets>, n., OS. <ets>triuwi</ets>, adj., <ets>trewa</ets>, n., D. <ets>trouw</ets>, adj. & n., G. <ets>treu</ets>, adj., <ets>treue</ets>, n., OHG. <ets>gitriuwi</ets>, adj., <ets>triuwa</ets>, <pos>n.</pos>, Icel. <ets>tryggr</ets>, adj., Dan. <ets>tro</ets>, adj. & n., Sw. <ets>trogen</ets>, adj., <ets>tro</ets>, n., Goth. <ets>triggws</ets>, adj., <ets>triggwa</ets>, n., <ets>trauan</ets> to trust, OPruss <ets>druwis</ets> faith.  Cf. <er>Trow</er>, <er>Trust</er>, <er>Truth</er>.]</ety><br/
[<source>1913 Webster</source>]</p>

<p><sn>1.</sn> <def>Conformable to fact; in accordance with the actual state of things; correct; not false, erroneous, inaccurate, or the like; <as>as, a <ex>true</ex> relation or narration; a <ex>true</ex> history; a declaration is <ex>true</ex> when it states the facts</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Right to precision; conformable to a rule or pattern; exact; accurate; <as>as, a <ex>true</ex> copy; a <ex>true</ex> likeness of the original</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Making his eye, foot, and hand keep <qex>true</qex> time.</q> <rj><qau>Sir W. Scott.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>Steady in adhering to friends, to promises, to a prince, or the like; unwavering; faithful; loyal; not false, fickle, or perfidious; <as>as, a <ex>true</ex> friend; a wife <ex>true</ex> to her husband; an officer <ex>true</ex> to his charge</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Thy so <qex>true</qex>,<br/
So faithful, love unequaled.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Dare to be <qex>true</qex>: nothing can need a lie.</q> <rj><qau>Herbert.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>Actual; not counterfeit, adulterated, or pretended; genuine; pure; real; <as>as, <ex>true</ex> balsam; <ex>true</ex> love of country; a <ex>true</ex> Christian</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The <qex>true</qex> light which lighteth every man that cometh into the world.</q> <rj><qau>John i. 9.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q><qex>True</qex> ease in writing comes from art, not chance.</q> <rj><qau>Pope.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <fld>(Biol.)</fld> <def>Genuine; real; not deviating from the essential characters of a class; <as>as, a lizard is a <ex>true</ex> reptile; a whale is a <ex>true</ex>, but not a <xex>typical</xex>, mammal</as>.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><note><hand/ <xex>True</xex> is sometimes used elliptically for <xex>It is true</xex>.</note><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Out of true</b></col>, <cd>varying from correct mechanical form, alignment, adjustment, etc.; -- said of a wall that is not perpendicular, of a wheel whose circumference is not in the same plane, and the like.</cd> <mark>[Colloq.]</mark> -- <col><b>A true bill</b></col> <fld>(Law)</fld>, <cd>a bill of indictment which is returned by the grand jury so indorsed, signifying that the charges to be true.</cd> -- <col><b>True time</b></col>. <cd>See under <er>Time</er>.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>True</ent><br/
<hw>True</hw>, <pos>adv.</pos> <def>In accordance with truth; truly.</def>  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>True-blue</ent><br/
<hw>True"-blue`</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Of inflexible honesty and fidelity; -- a term derived from the <xex>true</xex>, or <xex>Coventry</xex>, <xex>blue</xex>, formerly celebrated for its unchanging color. See <cref>True blue</cref>, under <er>Blue</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>True-blue</ent><br/
<hw>True"-blue`</hw>, <pos>n.</pos> <def>A person of inflexible integrity or fidelity.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>True-born</ent><br/
<hw>True"-born`</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Of genuine birth; having a right by birth to any title; <as>as, a <ex>true-born</ex> Englishman</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>True-bred</ent><br/
<hw>True"-bred`</hw> <pr>(?)</pr>, <pos>a.</pos> <sn>1.</sn> <def>Of a genuine or right breed; <as>as, a <ex>true-bred</ex> beast</as>.</def>  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Being of real breeding or education; <as>as, a <ex>true-bred</ex> gentleman</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>True-hearted</ent><br/
<hw>True"-heart`ed</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Of a faithful heart; honest; sincere; not faithless or deceitful; as, a <xex>truhearted</xex> friend.</def> -- <wordforms><wf>True"-heart`ed*ness</wf>, <pos>n.</pos></wordforms><br/
[<source>1913 Webster</source>]</p>

<p><-- p. 1546 --><br/
[<source>1913 Webster</source>]</p>

<p><ent>Truelove</ent><br/
<hw>True"love`</hw> <pr>(?)</pr>, <pos>n.</pos> <sn>1.</sn> <def>One really beloved.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Bot.)</fld> <def>A plant. See <er>Paris</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>An unexplained word occurring in Chaucer, meaning, perhaps, an aromatic sweetmeat for sweetening the breath.</def>  <rj><au>T. R. Lounsbury.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Under his tongue a <qex>truelove</qex> he bore.</q> <rj><qau>Chaucer.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Truelove knot</b></col>, <cd>a complicated, involved knot that does not readily untie; the emblem of interwoven affection or engagement; -- called also <altname>true-lover's knot</altname>.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Trueness</ent><br/
<hw>True"ness</hw>, <pos>n.</pos> <def>The quality of being true; reality; genuineness; faithfulness; sincerity; exactness; truth.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>True-penny</ent><br/
<hw>True"-pen`ny</hw> <pr>(?)</pr>, <pos>n.</pos> <def>An honest fellow.</def>  <rj><au>Shak. Bacon.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Truffle</ent><br/
<hw>Truf"fle</hw> <pr>(?; 277)</pr>, <pos>n.</pos> <ety>[OF. <ets>trufle</ets>, F. <ets>truffe</ets>; akin to Sp. <ets>trufa</ets>, <ets>tartufo</ets>; of uncertain origin; perhaps from L. <ets>tuber</ets> a tumor, knob, truffle.  Cf. <er>Tuber</er>, <er>Trifle</er>.]</ety> <def>Any one of several kinds of roundish, subterranean fungi, usually of a blackish color. The French truffle (<spn>Tuber melanosporum</spn>) and the English truffle (<spn>Tuber aestivum</spn>) are much esteemed as articles of food.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Truffle worm</b></col> <fld>(Zool.)</fld>, <cd>the larva of a fly of the genus <gen>Leiodes</gen>, injurious to truffles.</cd></cs>
<-- <col><b>Truffle pig</b></col>, <cd>a pig used for finding truffles.</cd>  <note>When trained, certain pigs have a peculiar ability to smell truffles which lie underground, making them useful for searching out hidden truffles.</note> --><br/
[<source>1913 Webster</source>]</p>

<p><ent>Truffled</ent><br/
<hw>Truf"fled</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Provided or cooked with truffles; stuffed with truffles; <as>as, a <ex>truffled</ex> turkey</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Trug</ent><br/
<hw>Trug</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Cf. <er>Trough</er>.]</ety> <sn>1.</sn> <def>A trough, or tray.</def> Specifically: <sd>(a)</sd> <def>A hod for mortar.</def> <sd>(b)</sd> <def>An old measure of wheat equal to two thirds of a bushel.</def>  <rj><au>Bailey.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A concubine; a harlot.</def> <mark>[Obs.]</mark>  <rj><au>Taylor (1630).</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Trugging-house</ent><br/
<hw>Trug"ging-house`</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[See <er>Trug</er>, <pos>n.</pos>, 2.]</ety> <def>A brothel.</def> <mark>[Obs.]</mark>  <rj><au>Robert Greene.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Truism</ent><br/
<hw>Tru"ism</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[From <er>True</er>.]</ety> <def>An undoubted or self-evident truth; a statement which is pliantly true; a proposition needing no proof or argument; -- opposed to <xex>falsism</xex>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Trifling <qex>truisms</qex> clothed in great, swelling words.</q> <rj><qau>J. P. Smith.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Truismatic</ent><br/
<hw>Tru`is*mat"ic</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Of or pertaining to truisms; consisting of truisms.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Truite</ent><br/
||<hw>Trui`t<eacute/"</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[F., fr. <ets>truite</ets> trout.]</ety> <def>Having a delicately crackled surface; -- applied to porcelian, etc.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Trull</ent><br/
<hw>Trull</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[G. <ets>trolle</ets>, <ets>trulle</ets>; cf. OD. <ets>drol</ets> a jester, Dan. <ets>trold</ets> an elf, imp, Sw. <ets>troll</ets> a goblin, Icel. <ets>troll</ets>, <ets>tr<oum/ll</ets>, a giant, fiend, demon.  Cf. <er>Droll</er>, <er>Troll</er>.]</ety> <sn>1.</sn> <def>A drab; a strumpet; a harlot; a trollop.</def>  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A girl; a wench; a lass.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Trullization</ent><br/
<hw>Trul`li*za"tion</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>trullissatio</ets>, from <ets>trullissare</ets> to trowel, to plaster, fr. <ets>trulla</ets> a trowel.]</ety> <def>The act of laying on coats of plaster with a trowel.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Truly</ent><br/
<hw>Tru"ly</hw> <pr>(?)</pr>, <pos>adv.</pos> <ety>[From <er>True</er>.]</ety> <sn>1.</sn> <def>In a true manner; according to truth; in agreement with fact; <as>as, to state things <ex>truly</ex>; the facts are <ex>truly</ex> represented</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>I can not <qex>truly</qex> say how I came here.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Exactly; justly; precisely; accurately; <as>as, to estimate <ex>truly</ex> the weight of evidence</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>Sincerely; honestly; really; faithfully; <as>as, to be <ex>truly</ex> attached to a lover</as>; the citizens are <xex>truly</xex> loyal to their prince or their country.</def>  <rj><au>Burke.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>Conformably to law; legally; legitimately.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>His innocent babe [is] <qex>truly</qex> begotten.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <def>In fact; in deed; in reality; in truth.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Beauty is excelled by manly grace<br/
And wisdom, which alone is <qex>truly</qex> fair.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Trump</ent><br/
<hw>Trump</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[OE. <ets>trumpe</ets>, <ets>trompe</ets>, F. <ets>trompe</ets>; probably fr. L. <ets>triumphare</ets> to triumph, to exult, hence, probably, to make a joyous sound or noise. See <er>Triumph</er>, <pos>v. i.</pos> & <pos>n.</pos>, and cf. <er>Trombone</er>, <er>Tromp</er>, <er>Trump</er> at cards, <er>Trumpery</er>, <er>Trumpet</er>, <er>Trunk</er> a proboscis.]</ety> <def>A wind instrument of music; a trumpet, or sound of a trumpet; -- used chiefly in Scripture and poetry.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>We shall all be changed, in a moment, in the twinkling of an eye, at the last <qex>trump</qex>.</q> <rj><qau>1 Cor. xv. 51, 52.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>The wakeful <qex>trump</qex> of doom.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Trump</ent><br/
<hw>Trump</hw>, <pos>v. i.</pos> <ety>[Cf. OF. <ets>tromper</ets>. See <er>Trump</er> a trumpet.]</ety> <def>To blow a trumpet.</def> <mark>[Obs.]</mark>  <rj><au>Wyclif (Matt. vi. 2).</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Trump</ent><br/
<hw>Trump</hw>, <pos>n.</pos> <ety>[A corruption of <ets>triumph</ets>, F. <ets>triomphe</ets>. See <er>Triumph</er>, and cf. <er>Trump</er> a trumpet.]</ety> <sn>1.</sn> <def>A winning card; one of a particular suit (usually determined by chance for each deal) any card of which takes any card of the other suits.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>An old game with cards, nearly the same as whist; -- called also <altname>ruff</altname>.</def>  <rj><au>Decker.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>A good fellow; an excellent person.</def> <mark>[Slang]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>Alfred is a <qex>trump</qex>, I think you say.</q> <rj><qau>Thackeray.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><mcol><col><b>To put to one's trumps</b></col>, <it>or</it> <col><b>To put on one's trumps</b></col></mcol>, <cd>to force to the last expedient, or to the utmost exertion.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><q>But when kings come so low as to fawn upon philosophy, which before they neither valued nor understood, it is a sign that fails not, they are then <qex>put to their</qex> last <qex>trump</qex>.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q><qex>Put</qex> the housekeeper <qex>to her trumps</qex> to accommodate them.</q> <rj><qau>W. Irving.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Trump</ent><br/
<hw>Trump</hw>, <pos>v. i.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Trumped</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Trumping</conjf>.]</vmorph> <def>To play a trump card when one of another suit has been led.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Trump</ent><br/
<hw>Trump</hw>, <pos>v. t.</pos> <def>To play a trump card upon; to take with a trump card; <as>as, she <ex>trumped</ex> the first trick</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Trump</ent><br/
<hw>Trump</hw>, <pos>v. t.</pos> <ety>[F. <ets>tromper</ets> to deceive, in OF., to blow a trumpet, <ets>se tromper de</ets> to mock. See <er>Trump</er> a trumpet.]</ety> <sn>1.</sn> <def>To trick, or impose on; to deceive.</def> <mark>[Obs.]</mark> <ldquo/To trick or <xex>trump</xex> mankind.<rdquo/  <rj><au>B. Jonson.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To impose unfairly; to palm off.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Authors have been <qex>trumped</qex> upon us.</q> <rj><qau>C. Leslie.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>To trump up</b></col>, <cd>to devise; to collect with unfairness; to fabricate; as, <xex>to trump up</xex> a charge.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Trumpery</ent><br/
<hw>Trump"er*y</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F. <ets>tromperie</ets> deceit, fr. <ets>tromper</ets> to deceive. See <er>Trump</er> to trick.]</ety> <sn>1.</sn> <def>Deceit; fraud.</def> <mark>[Obs.]</mark>  <rj><au>Grenewey.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Something serving to deceive by false show or pretense; falsehood; deceit; worthless but showy matter; hence, things worn out and of no value; rubbish.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The <qex>trumpery</qex> in my house, go bring it hither,<br/
for state to catch these thieves.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Upon the coming of Christ, very much, though not all, of this idolatrous <qex>trumpery</qex> and superstition was driven out of the world.</q> <rj><qau>South.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Trumpery</ent><br/
<hw>Trump"er*y</hw>, <pos>a.</pos> <def>Worthless or deceptive in character.</def> <ldquo/A <xex>trumpery</xex> little ring.<rdquo/  <rj><au>Thackeray.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Trumpet</ent><br/
<hw>Trump"et</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F. <ets>trompette</ets>, dim. of <ets>trompe</ets>. See <er>Trump</er> a trumpet.]</ety> <sn>1.</sn> <fld>(Mus.)</fld> <def>A wind instrument of great antiquity, much used in war and military exercises, and of great value in the orchestra. In consists of a long metallic tube, curved (once or twice) into a convenient shape, and ending in a bell. Its scale in the lower octaves is limited to the first natural harmonics; but there are modern trumpets capable, by means of valves or pistons, of producing every tone within their compass, although at the expense of the true ringing quality of tone.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The <qex>trumpet's</qex> loud clangor<br/
Excites us to arms.</q> <rj><qau>Dryden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Mil.)</fld> <def>A trumpeter.</def>  <rj><au>Clarendon.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>One who praises, or propagates praise, or is the instrument of propagating it.</def>  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>That great politician was pleased to have the greatest wit of those times . . . to be the <qex>trumpet</qex> of his praises.</q> <rj><qau>Dryden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <fld>(Mach)</fld> <def>A funnel, or short, fiaring pipe, used as a guide or conductor, as for yarn in a knitting machine.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Ear trumpet</b></col>. <cd>See under <er>Ear</er>.</cd> -- <col><b>Sea trumpet</b></col> <fld>(Bot.)</fld>, <cd>a great seaweed (<spn>Ecklonia buccinalis</spn>) of the Southern Ocean. It has a long, hollow stem, enlarging upwards, which may be made into a kind of trumpet, and is used for many purposes.</cd> -- <col><b>Speaking trumpet</b></col>, <cd>an instrument for conveying articulate sounds with increased force.</cd> -- <col><b>Trumpet animalcule</b></col> <fld>(Zool.)</fld>, <cd>any infusorian belonging to Stentor and allied genera, in which the body is trumpet-shaped. See <er>Stentor</er>.</cd> -- <col><b>Trumpet ash</b></col> <fld>(Bot.)</fld>, <cd>the trumpet creeper.</cd> <mark>[Eng.]</mark> -- <col><b>Trumpet conch</b></col> <fld>(Zool.)</fld>, <cd>a trumpet shell, or triton.</cd> -- <col><b>Trumpet creeper</b></col> <fld>(Bot.)</fld>, <cd>an American climbing plant (<spn>Tecoma radicans</spn>) bearing clusters of large red trumpet-shaped flowers; -- called also <altname>trumpet flower</altname>, and in England <altname>trumpet ash</altname>.</cd> -- <col><b>Trumpet fish</b></col>. <fld>(Zool.)</fld> <sd>(a)</sd> <cd>The bellows fish.</cd> <sd>(b)</sd> <cd>The fistularia.</cd> -- <col><b>Trumpet flower</b></col>. <fld>(Bot.)</fld> <sd>(a)</sd> <cd>The trumpet creeper; also, its blossom.</cd> <sd>(b)</sd> <cd>The trumpet honeysuckle.</cd> <sd>(c)</sd> <cd>A West Indian name for several plants with trumpet-shaped flowers.</cd> -- <col><b>Trumpet fly</b></col> <fld>(Zool.)</fld>, <cd>a botfly.</cd> -- <col><b>Trumpet honeysuckle</b></col> <fld>(Bot.)</fld>, <cd>a twining plant (<spn>Lonicera sempervirens</spn>) with red and yellow trumpet-shaped flowers; -- called also <altname>trumpet flower</altname>.</cd> -- <col><b>Trumpet leaf</b></col> <fld>(Bot.)</fld>, <cd>a name of several plants of the genus <gen>Sarracenia</gen>.</cd> -- <col><b>Trumpet major</b></col> <fld>(Mil.)</fld>, <cd>the chief trumpeter of a band or regiment.</cd> -- <col><b>Trumpet marine</b></col> <fld>(Mus.)</fld>, <cd>a monochord, having a thick string, sounded with a bow, and stopped with the thumb so as to produce the harmonic tones; -- said to be the oldest bowed instrument known, and in form the archetype of all others. It probably owes its name to <ldquo/its external resemblance to the large speaking trumpet used on board Italian vessels, which is of the same length and tapering shape.<rdquo/</cd> <au>Grove.</au> -- <col><b>Trumpet shell</b></col> <fld>(Zool.)</fld>, <cd>any species of large marine univalve shells belonging to Triton and allied genera. See <er>Triton</er>, 2.</cd> -- <col><b>Trumpet tree</b></col>. <fld>(Bot.)</fld> <cd>See <er>Trumpetwood</er>.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Trumpet</ent><br/
<hw>Trump"et</hw>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Trumpeted</conjf>; <pos>p. pr. & vb. n.</pos> <conjf>Trumpeting</conjf>.]</vmorph> <ety>[Cf. F. <ets>trompeter</ets>.]</ety> <def>To publish by, or as by, sound of trumpet; to noise abroad; to proclaim; <as>as, to <ex>trumpet</ex> good tidings</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>They did nothing but publish and <qex>trumpet</qex> all the reproaches they could devise against the Irish.</q> <rj><qau>Bacon.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Trumpet</ent><br/
<hw>Trump"et</hw>, <pos>v. i.</pos> <def>To sound loudly, or with a tone like a trumpet; to utter a trumplike cry.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Trumpeter</ent><br/
<hw>Trump"et*er</hw> <pr>(?)</pr>, <pos>n.</pos> <sn>1.</sn> <def>One who sounds a trumpet.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>One who proclaims, publishes, or denounces.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>These men are good <qex>trumpeters</qex>.</q> <rj><qau>Bacon.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <fld>(Zool.)</fld> <sd>(a)</sd> <def>Any one of several species of long-legged South American birds of the genus <gen>Psophia</gen>, especially <spn>Psophia crepitans</spn>, which is abundant, and often domesticated and kept with other poultry by the natives. They are allied to the cranes. So called from their loud cry.  Called also <altname>agami</altname>, and <altname>yakamik</altname>.</def> <sd>(b)</sd> <def>A variety of the domestic pigeon.</def> <sd>(c)</sd> <def>An American swan (<spn>Olor buccinator</spn>) which has a very loud note.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <fld>(Zool.)</fld> <def>A large edible fish (<spn>Latris hecateia</spn>) of the family <fam>Cirrhitidae</fam>, native of Tasmania and New Zealand. It sometimes weighs as much as fifty or sixty pounds, and is highly esteemed as a food fish.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Trumpeting</ent><br/
<hw>Trump"et*ing</hw>, <pos>n.</pos> <fld>(Mining)</fld> <def>A channel cut behind the brick lining of a shaft.</def>  <rj><au>Raymond.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Trumpets</ent><br/
<hw>Trump"ets</hw> <pr>(?)</pr>, <pos>n. pl.</pos> <fld>(Bot.)</fld> <def>A plant (<spn>Sarracenia flava</spn>) with long, hollow leaves.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Trumpet-shaped</ent><br/
<hw>Trump"et-shaped`</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Tubular with one end dilated, as the flower of the trumpet creeper.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Trumpet-tongued</ent><br/
<hw>Trump"et-tongued`</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Having a powerful, far-reaching voice or speech.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Trumpetweed</ent><br/
<hw>Trump"et*weed`</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Bot.)</fld> <sd>(a)</sd> <def>An herbaceous composite plant (<spn>Eupatorium purpureum</spn>), often having hollow stems, and bearing purplish flowers in small corymbed heads.</def> <sd>(b)</sd> <def>The sea trumpet.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Trumpetwood</ent><br/
<hw>Trump"et*wood`</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Bot.)</fld> <def>A tropical American tree (<spn>Cecropia peltata</spn>) of the Breadfruit family, having hollow stems, which are used for wind instruments; -- called also <altname>snakewood</altname>, and <altname>trumpet tree</altname>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Trumpie</ent><br/
<hw>Trum"pie</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Zool.)</fld> <def>The Richardson's skua (<spn>Stercorarius parasiticus</spn>).</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Trumplike</ent><br/
<hw>Trump"like`</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Resembling a trumpet, esp. in sound; <as>as, a <ex>trumplike</ex> voice</as>.</def>  <rj><au>Chapman.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Truncal</ent><br/
<hw>Trun"cal</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Of or pertaining to the trunk, or body.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Truncate</ent><br/
<hw>Trun"cate</hw> <pr>(?)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Truncated</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Truncating</conjf>.]</vmorph> <ety>[L. <ets>truncatus</ets>, p. p. of <ets>truncare</ets> to cut off, mutilate, fr. <ets>truncus</ets> maimed, mutilated, cut short. See <er>Trunk</er>.]</ety> <def>To cut off; to lop; to maim.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Truncate</ent><br/
<hw>Trun"cate</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>truncatus</ets>, p. p. ]</ety> <def>Appearing as if cut off at the tip; <as>as, a <ex>truncate</ex> leaf or feather</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Truncated</ent><br/
<hw>Trun"ca*ted</hw> <pr>(?)</pr>, <pos>a.</pos> <sn>1.</sn> <def>Cut off; cut short; maimed.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Min.)</fld> <def>Replaced, or cut off, by a plane, especially when equally inclined to the adjoining faces; <as>as, a <ex>truncated</ex> edge</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <fld>(Zool.)</fld> <def>Lacking the apex; -- said of certain spiral shells in which the apex naturally drops off.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><mcol><col><b>Truncated cone</b></col> <it>or</it> <col><b>Truncated pyramid</b></col></mcol> <fld>(Geom.)</fld>, <cd>a cone or pyramid whose vertex is cut off by a plane, the plane being usually parallel to the base.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Truncation</ent><br/
<hw>Trun*ca"tion</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>truncatio</ets>.]</ety> <sn>1.</sn> <def>The act of truncating, lopping, or cutting off.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>The state of being truncated.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <fld>(Min.)</fld> <def>The replacement of an edge or solid angle by a plane, especially when the plane is equally inclined to the adjoining faces.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Trunch</ent><br/
<hw>Trunch</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[See <er>Truncheon</er>.]</ety> <def>A stake; a small post.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Truncheon</ent><br/
<hw>Trun"cheon</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[OE. <ets>tronchoun</ets> the shaft of a broken spear, broken piece, OF. <ets>tronchon</ets>, <ets>tron<?/on</ets>, F. <ets>tron<?/on</ets>, fr. OF. & F. <ets>tronce</ets>, <ets>tronche</ets>, a piece of wood; cf. OF. <ets>trons</ets>, <ets>tros</ets>, <ets>trois</ets>; all perhaps from L. <ets>thyrsus</ets> a stalk, stem, staff. See <er>Thyrsus</er>, and cf. <er>Trounce</er>.]</ety> <sn>1.</sn> <def>A short staff, a club; a cudgel; a shaft of a spear.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>With his <qex>truncheon</qex> he so rudely struck.</q> <rj><qau>Spenser.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A baton, or military staff of command.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The marshal's <qex>truncheon</qex> nor the judges robe.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>A stout stem, as of a tree, with the branches lopped off, to produce rapid growth.</def>  <rj><au>Gardner.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Truncheon</ent><br/
<hw>Trun"cheon</hw>, <pos>v. t.</pos> <def>To beat with a truncheon.</def>  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Truncheoned</ent><br/
<hw>Trun"cheoned</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Having a truncheon.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Truncheoneer</ent><br/
<hw>Trun`cheon*eer"</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A person armed with a truncheon.</def> <altsp>[Written also <asp>truncheoner</asp>.]</altsp><br/
[<source>1913 Webster</source>]</p>

<p><ent>Truncus</ent><br/
||<hw>Trun"cus</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L.]</ety> <fld>(Zool.)</fld> <def>The thorax of an insect. See <er>Trunk</er>, <pos>n.</pos>, 5.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Trundle</ent><br/
<hw>Trun"dle</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[AS. <ets>tryndel</ets> a little shield. See <er>Trend</er>, <pos>v. i.</pos>]</ety> <sn>1.</sn> <def>A round body; a little wheel.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A lind of low-wheeled cart; a truck.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>A motion as of something moving upon little wheels or rollers; a rolling motion.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <fld>(Mach.)</fld> <sd>(a)</sd> <def>A lantern wheel. See under <er>Lantern</er>.</def> <sd>(b)</sd> <def>One of the bars of a lantern wheel.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Trundle</ent><br/
<hw>Trun"dle</hw>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Trundled</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Trundling</conjf> <pr>(?)</pr>.]</vmorph> <sn>1.</sn> <def>To roll (a thing) on little wheels; <as>as, to <ex>trundle</ex> a bed or a gun carriage</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To cause to roll or revolve; to roll along; <as>as, to <ex>trundle</ex> a hoop or a ball</as>.</def>  <rj><au>R. A. Proctor.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Trundle</ent><br/
<hw>Trun"dle</hw>, <pos>v. i.</pos> <sn>1.</sn> <def>To go or move on small wheels; <as>as, a bed <ex>trundles</ex> under another</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To roll, or go by revolving, as a hoop.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Trundle-bed</ent><br/
<hw>Trun"dle-bed`</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A low bed that is moved on trundles, or little wheels, so that it can be pushed under a higher bed; a truckle-bed; also, sometimes, a simiral bed without wheels.</def>  <rj><au>Chapman.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Trundlehead</ent><br/
<hw>Trun"dle*head`</hw> <pr>(?)</pr>, <pos>n.</pos> <sn>1.</sn> <fld>(Gearing)</fld> <def>One of the disks forming the ends of a lantern wheel or pinion.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>The drumhead of a capstan; especially, the drumhead of the lower of two capstans on the sane axis.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Trundletail</ent><br/
<hw>Trun"dle*tail`</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A round or curled-up tail; also, a dog with such a tail.</def>  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Trunk</ent><br/
<hw>Trunk</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F. <ets>tronc</ets>, L. <ets>truncus</ets>, fr. <ets>truncus</ets> maimed, mutilated; perhaps akin to <ets>torquere</ets> to twist wrench, and E. <ets>torture</ets>. <ets>Trunk</ets> in the sense of proboscis is fr. F. <ets>trompe</ets> (the same word as <ets>trompe</ets> a trumpet), but has been confused in English with <ets>trunk</ets> the stem of a tree (see <er>Trump</er> a trumpet).  Cf. <er>Truncate</er>.]</ety> <sn>1.</sn> <def>The stem, or body, of a tree, apart from its limbs and roots; the main stem, without the branches; stock; stalk.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>About the mossy <qex>trunk</qex> I wound me soon,<br/
For, high from ground, the branches would require<br/
Thy utmost reach.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>The body of an animal, apart from the head and limbs.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>The main body of anything; <as>as, the <ex>trunk</ex> of a vein or of an artery, as distinct from the branches</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <fld>(Arch)</fld> <def>That part of a pilaster which is between the base and the capital, corresponding to the shaft of a column.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <fld>(Zool.)</fld> <def>That segment of the body of an insect which is between the head and abdomen, and bears the wings and legs; the thorax; the truncus.</def><br/
[<source>1913 Webster</source>]</p>

<p><-- p. 1547 --><br/
[<source>1913 Webster</source>]</p>

<p><sn>6.</sn> <fld>(Zool.)</fld> <sd>(a)</sd> <def>The proboscis of an elephant.</def> <sd>(b)</sd> <def>The proboscis of an insect.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>7.</sn> <def>A long tube through which pellets of clay, p<?/as, etc., are driven by the force of the breath.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>He shot sugarplums them out of a <qex>trunk</qex>.</q> <rj><qau>Howell.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>8.</sn> <def>A box or chest usually covered with leather, metal, or cloth, or sometimes made of leather, hide, or metal, for containing clothes or other goods; especially, one used to convey the effects of a traveler.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Locked up in chests and <qex>trunks</qex>.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>9.</sn> <fld>(Mining)</fld> <def>A flume or sluice in which ores are separated from the slimes in which they are contained.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>10.</sn> <fld>(Steam Engine)</fld> <def>A large pipe forming the piston rod of a steam engine, of sufficient diameter to allow one end of the connecting rod to be attached to the crank, and the other end to pass within the pipe directly to the piston, thus making the engine more compact.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>11.</sn> <def>A long, large box, pipe, or conductor, made of plank or metal plates, for various uses, as for conveying air to a mine or to a furnace, water to a mill, grain to an elevator, etc.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Trunk engine</b></col>, <cd>a marine engine, the piston rod of which is a trunk. See <er>Trunk</er>, 10.</cd> -- <col><b>Trunk hose</b></col>, <cd>large breeches formerly worn, reaching to the knees.</cd> -- <col><b>Trunk line</b></col>, <cd>the main line of a railway, canal, or route of conveyance.</cd> -- <col><b>Trunk turtle</b></col> <fld>(Zool.)</fld>, <cd>the leatherback.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Trunk</ent><br/
<hw>Trunk</hw> <pr>(?)</pr>, <pos>v. t.</pos> <ety>[Cf. F. <ets>tronquer</ets>. See <er>Truncate</er>.]</ety> <sn>1.</sn> <def>To lop off; to curtail; to truncate; to maim.</def> <mark>[Obs.]</mark> <ldquo/Out of the <xex>trunked</xex> stock.<rdquo/  <rj><au>Spenser.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Mining)</fld> <def>To extract (ores) from the slimes in which they are contained, by means of a trunk. See <er>Trunk</er>, <pos>n.</pos>, 9.</def>  <rj><au>Weale.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Trunkback</ent><br/
<hw>Trunk"back`</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Zool.)</fld> <def>The leatherback.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Trunked</ent><br/
<hw>Trunked</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Having (such) a trunk.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Thickset with strong and well-<qex>trunked</qex> trees.</q> <rj><qau>Howell.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Trunk engine</ent><br/
<hw>Trunk engine</hw>. <def>An engine having a trunk piston, as most internal combustion engines.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Trunkfish</ent><br/
<hw>Trunk"fish`</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Zool.)</fld> <def>Any one of several species of plectognath fishes, belonging to the genus <gen>Ostracion</gen>, or the family <fam>Ostraciontidae</fam>, having an angular body covered with a rigid integument consisting of bony scales. Some of the species are called also <stype>coffer fish</stype>, and <stype>boxfish</stype>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Trunkful</ent><br/
<hw>Trunk"ful</hw> <pr>(?)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Trunkfuls</plw> <pr>(<?/)</pr>.</plu> <def>As much as a trunk will hold; enough to fill a trunk.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Trunk piston</ent><br/
<hw>Trunk piston</hw>. <def>In a single-acting engine, an elongated hollow piston, open at the end, in which the end of the connecting rod is pivoted. The piston rod, crosshead and stuffing box are thus dispensed with.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Trunk steamer</ent><br/
<hw>Trunk steamer</hw>. <def>A freight steamer having a high hatch coaming extending almost continuously fore and aft, but not of whaleback form at the sides.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Trunkwork</ent><br/
<hw>Trunk"work`</hw> <pr>(?)</pr>, <pos>n.</pos> <def>Work or devices suitable to be concealed; a secret stratagem.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Trunnel</ent><br/
<hw>Trun"nel</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A trundle.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Trunnel</ent><br/
<hw>Trun"nel</hw>, <pos>n.</pos> <fld>(Shipbuilding)</fld> <def>See <er>Treenail</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Trunnion</ent><br/
<hw>Trun"nion</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[OF. <ets>trognon</ets> the stock, stump, or truck of a tree, F. <ets>trognon</ets> a core, stalk, fr. <ets>tron</ets> a trunk, stem.  Cf. <er>Trunk</er>.]</ety> <sn>1.</sn> <fld>(Gun.)</fld> <def>A cylindrical projection on each side of a piece, whether gun, mortar, or howitzer, serving to support it on the cheeks of the carriage. See <xex>Illust.</xex> of <er>Cannon</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Steam Engine)</fld> <def>A gudgeon on each side of an oscillating steam cylinder, to support it. It is usually tubular, to convey steam.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Trunnion plate</b></col> <fld>(Gun.)</fld>, <cd>a plate in the carriage of a gun, mortar, or howitzer, which covers the upper part of the cheek, and forms a bearing under the trunnion.</cd> -- <col><b>Trunnion ring</b></col> <fld>(Gun.)</fld>, <cd>a ring on a cannon next before the trunnions.</cd> <mark>[R.]</mark></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Trunnioned</ent><br/
<hw>Trun"nioned</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Provided with trunnions; <as>as, the <ex>trunnioned</ex> cylinder of an oscillating steam engine</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Trusion</ent><br/
<hw>Tru"sion</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>trudere</ets>, <ets>trusum</ets>, to thrust, shove: cf. F. <ets>trusion</ets>.]</ety> <def>The act of pushing or thrusting.</def> <mark>[R.]</mark>  <rj><au>Bentley.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Truss</ent><br/
<hw>Truss</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[OE. <ets>trusse</ets>, F. <ets>trousse</ets>, OF. also <ets>tourse</ets>; perhaps fr. L. <ets>tryrsus</ets> stalk, stem.  Cf. <er>Thyrsus</er>, <er>Torso</er>, <er>Trousers</er>, <er>Trousseau</er>.]</ety> <sn>1.</sn> <def>A bundle; a package; <as>as, a <ex>truss</ex> of grass</as>.</def>  <rj><au>Fabyan.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Bearing a <qex>truss</qex> of trifles at his back.</q> <rj><qau>Spenser.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ A <xex>truss</xex> of hay in England is 56 lbs. of old and 60 lbs. of new hay; a <xex>truss</xex> of straw is 36 lbs.</note><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A padded jacket or dress worn under armor, to protect the body from the effects of friction; also, a part of a woman's dress; a stomacher.</def> <mark>[Obs.]</mark>  <rj><au>Nares.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Puts off his palmer's weed unto his <qex>truss</qex>, which bore<br/
The stains of ancient arms.</q> <rj><qau>Drayton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <fld>(Surg.)</fld> <def>A bandage or apparatus used in cases of hernia, to keep up the reduced parts and hinder further protrusion, and for other purposes.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <fld>(Bot.)</fld> <def>A tuft of flowers formed at the top of the main stalk, or stem, of certain plants.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <fld>(Naut.)</fld> <def>The rope or iron used to keep the center of a yard to the mast.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>6.</sn> <fld>(Arch. & Engin.)</fld> <def>An assemblage of members of wood or metal, supported at two points, and arranged to transmit pressure vertically to those points, with the least possible strain across the length of any member. Architectural trusses when left visible, as in open timber roofs, often contain members not needed for construction, or are built with greater massiveness than is requisite, or are composed in unscientific ways in accordance with the exigencies of style.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Truss rod</b></col>, <cd>a rod which forms the tension member of a trussed beam, or a tie rod in a truss.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Truss</ent><br/
<hw>Truss</hw>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Trussed</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Trussing</conjf>.]</vmorph> <ety>[F. <ets>trousser</ets>. See <er>Truss</er>, <pos>n.</pos>]</ety> <sn>1.</sn> <def>To bind or pack close; to tie up tightly; to make into a truss.</def>  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>It [his hood] was <qex>trussed</qex> up in his wallet.</q> <rj><qau>Chaucer.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To take fast hold of; to seize and hold firmly; to pounce upon.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>Who <qex>trussing</qex> me as eagle doth his prey.</q> <rj><qau>Spenser.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>To strengthen or stiffen, as a beam or girder, by means of a brace or braces.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>To skewer; to make fast, as the wings of a fowl to the body in cooking it.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <def>To execute by hanging; to hang; -- usually with <xex>up</xex>.</def> <mark>[Slang.]</mark>  <rj><au>Sir W. Scott.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><mcol><col><b>To truss a person</b></col> <it>or</it> <col><b>To truss one's self</b></col></mcol>, <cd>to adjust and fasten the clothing of; especially, to draw tight and tie the laces of garments.</cd> <mark>[Obs.]</mark> <ldquo/Enter Honeysuckle, in his nightcap, <xex>trussing</xex> himself.<rdquo/ <au>J. Webster (1607).</au> -- <col><b>To truss up</b></col>, <cd>to strain; to make close or tight.</cd> -- <col><b>Trussed beam</b></col>, <cd>a beam which is stiffened by a system of braces constituting a truss of which the beam is a chord.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Trussing</ent><br/
<hw>Truss"ing</hw> <pr>(?)</pr>, <pos>n.</pos> <sn>1.</sn> <fld>(Arch. & Engin.)</fld> <def>The timbers, etc., which form a truss, taken collectively.</def>  <rj><au>Weale.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Arch. & Engin.)</fld> <def>The art of stiffening or bracing a set of timbers, or the like, by putting in struts, ties, etc., till it has something of the character of a truss.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>The act of a hawk, or other bird of prey, in seizing its quarry, and soaring with it into air.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Trust</ent><br/
<hw>Trust</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[OE. <ets>trust</ets>, <ets>trost</ets>, Icel. <ets>traust</ets> confidence, security; akin to Dan. & Sw. <ets>tr<oum/st</ets> comfort, consolation, G. <ets>trost</ets>, Goth. <ets>trausti</ets> a convention, covenant, and E. <ets>true</ets>. See <er>True</er>, and cf. <er>Tryst</er>.]</ety> <sn>1.</sn> <def>Assured resting of the mind on the integrity, veracity, justice, friendship, or other sound principle, of another person; confidence; reliance; reliance.</def> <ldquo/O ever-failing <xex>trust</xex> in mortal strength!<rdquo/  <rj><au>Milton.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Most take things upon <qex>trust</qex>.</q> <rj><qau>Locke.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Credit given; especially, delivery of property or merchandise in reliance upon future payment; exchange without immediate receipt of an equivalent; <as>as, to sell or buy goods on <ex>trust</ex></as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>Assured anticipation; dependence upon something future or contingent, as if present or actual; hope; belief.</def> <ldquo/Such <xex>trust</xex> have we through Christ.<rdquo/ <au>2 Cor. iii. 4.</au><br/
[<source>1913 Webster</source>]</p>

<p><q>His <qex>trust</qex> was with the Eternal to be deemed<br/
Equal in strength.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>That which is committed or intrusted to one; something received in confidence; charge; deposit.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <def>The condition or obligation of one to whom anything is confided; responsible charge or office.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>[I] serve him truly that will put me in <qex>trust</qex>.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Reward them well, if they observe their <qex>trust</qex>.</q> <rj><qau>Denham.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>6.</sn> <def>That upon which confidence is reposed; ground of reliance; hope.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>O Lord God, thou art my <qex>trust</qex> from my youth.</q> <rj><qau>Ps. lxxi. 5.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>7.</sn> <fld>(Law)</fld> <def>An estate devised or granted in confidence that the devisee or grantee shall convey it, or dispose of the profits, at the will, or for the benefit, of another; an estate held for the use of another; a confidence respecting property reposed in one person, who is termed the <xex>trustee</xex>, for the benefit of another, who is called the <xex>cestui que trust</xex>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>8.</sn> <def>An equitable right or interest in property distinct from the legal ownership thereof; a use (as it existed before the Statute of Uses); also, a property interest held by one person for the benefit of another. Trusts are <xex>active</xex>, or <xex>special</xex>, <xex>express</xex>, <xex>implied</xex>, <xex>constructive</xex>, etc. In a <col><b>passive trust</b></col> the trustee simply has title to the trust property, while its control and management are in the beneficiary.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><sn>9.</sn>  <def>A business organization or combination consisting of a number of firms or corporations operating, and often united, under an agreement creating a trust (in sense 1), esp. one formed mainly for the purpose of regulating the  supply and price of commodities, etc.; often, opprobriously, a combination formed for the purpose of controlling or monopolizing a trade, industry, or business, by doing acts in restraint or trade; <as>as, a sugar <ex>trust</ex></as>. A trust may take the form of a corporation or of a body of persons or corporations acting together by mutual arrangement, as under a contract or a so-called gentlemen's agreement. When it consists of corporations it may be effected by putting a majority of their stock either in the hands of a board of trustees (whence the name <xex>trust</xex> for the combination) or by transferring a majority to a holding company. The advantages of a trust are partly due to the economies made possible in carrying on a large business, as well as the doing away with competition. In the United States severe statutes against trusts have been passed by the Federal government and in many States, with elaborate statutory definitions.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><syn><b>Syn.</b> -- Confidence; belief; faith; hope; expectation.</syn><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Trust deed</b></col> <fld>(Law)</fld>, <cd>a deed conveying property to a trustee, for some specific use.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Trust</ent><br/
<hw>Trust</hw>, <pos>a.</pos> <def>Held in trust; <as>as, <ex>trust</ex> property; <ex>trust</ex>money</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Trust</ent><br/
<hw>Trust</hw>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Trusted</conjf>; <pos>p. pr. & vb. n.</pos> <conjf>Trusting</conjf>.]</vmorph> <ety>[OE. <ets>trusten</ets>, <ets>trosten</ets>. See <er>Trust</er>, <pos>n.</pos>]</ety> <sn>1.</sn> <def>To place confidence in; to rely on, to confide, or repose faith, in; <as>as, we can not <ex>trust</ex> those who have deceived us</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>I will never <qex>trust</qex> his word after.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>He that <qex>trusts</qex> every one without reserve will at last be deceived.</q> <rj><qau>Johnson.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To give credence to; to believe; to credit.</def><br/
[<source>1913 Webster</source>]</p>

<p><q><qex>Trust</qex> me, you look well.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>To hope confidently; to believe; -- usually with a phrase or infinitive clause as the object.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>I <qex>trust</qex> to come unto you, and speak face to face.</q> <rj><qau>2 John 12.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>We <qex>trust</qex>we have a good conscience.</q> <rj><qau>Heb. xiii. 18.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>to show confidence in a person by intrusting (him) with something.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Whom, with your power and fortune, sir, you <qex>trust</qex>,<br/
Now to suspect is vain.</q> <rj><qau>Dryden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <def>To commit, as to one's care; to intrust.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Merchants were not willing to <qex>trust</qex> precious cargoes to any custody but that of a man-of-war.</q> <rj><qau>Macaulay.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>6.</sn> <def>To give credit to; to sell to upon credit, or in confidence of future payment; <as>as, merchants and manufacturers <ex>trust</ex> their customers annually with goods</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>7.</sn> <def>To risk; to venture confidently.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>[Beguiled] by thee<br/
to <qex>trust</qex> thee from my side.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Trust</ent><br/
<hw>Trust</hw>, <pos>v. i.</pos> <sn>1.</sn> <def>To have trust; to be credulous; to be won to confidence; to confide.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>More to know could not be more to <qex>trust</qex>.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To be confident, as of something future; to hope.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>I will <qex>trust</qex> and not be afraid.</q> <rj><qau>Isa. xii. 2.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>To sell or deliver anything in reliance upon a promise of payment; to give credit.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>It is happier sometimes to be cheated than not to <qex>trust</qex>.</q> <rj><qau>Johnson.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><mcol><col><b>To trust in</b></col>, <col><b>To trust on</b></col></mcol>, <cd>to place confidence in,; to rely on; to depend.</cd> <ldquo/<xex>Trust in</xex> the Lord, and do good.<rdquo/ <au>Ps. xxxvii. 3.</au> <ldquo/A priest . . . <xex>on</xex> whom we <xex>trust</xex>.<rdquo/ <au>Chaucer.</au><br/
[<source>1913 Webster</source>]</p>

<p><q>Her widening streets <qex>on</qex> new foundations <qex>trust</qex>.</q> <rj><qau>Dryden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p>-- <mcol><col><b>To trust to</b></col> <it>or</it> <col><b>To trust unto</b></col></mcol>, <cd>to depend on; to have confidence in; to rely on; <as>as, to <ex>trust to</ex> luck</as>.</cd><br/
[<source>1913 Webster</source>]</p>

<p><q>They <qex>trusted unto</qex> the liers in wait.</q> <rj><qau>Judges xx. 36.</qau></rj>
</cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Trust company</ent><br/
<hw>Trust company</hw>. <def>Any corporation formed for the purpose of acting as trustee.  Such companies usually do more or less of a banking business.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Trustee</ent><br/
<hw>Trus*tee"</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Law)</fld> <def>A person to whom property is legally committed in trust, to be applied either for the benefit of specified individuals, or for public uses; one who is intrusted with property for the benefit of another; also, a person in whose hands the effects of another are attached in a trustee process.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Trustee process</b></col> <fld>(Law)</fld>, <cd>a process by which a creditor may attach his debtor's goods, effects, and credits, in the hands of a third person; -- called, in some States, the <altname>process of foreign attachment</altname>, <altname>garnishment</altname>, or <altname>factorizing process</altname>.</cd> <mark>[U. S.]</mark></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Trustee</ent><br/
<hw>Trus*tee"</hw> <pr>(?)</pr>, <pos>v. t.</pos> <sn>1.</sn> <def>To commit (property) to the care of a trustee; <as>as, to <ex>trustee</ex> an estate</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Law)</fld> <def>To attach (a debtor's wages, credits, or property in the hands of a third person) in the interest of the creditor.</def> <mark>[U. S.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Trustee process</ent><br/
<hw>Trus*tee" proc"ess</hw>. <fld>(Law)</fld> <def>The process of attachment by garnishment.</def> <mark>[U. S.]</mark><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Trusteeship</ent><br/
<hw>Trus*tee"ship</hw>, <pos>n.</pos> <def>The office or duty of a trustee.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Trustee stock</ent><br/
<hw>Trustee stock</hw>. <fld>(Finance)</fld> <def>High-grade stock in which trust funds may be legally invested.</def> <mark>[Colloq.]</mark><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Truster</ent><br/
<hw>Trust"er</hw> <pr>(?)</pr>, <pos>n.</pos> <sn>1.</sn> <def>One who trusts, or credits.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Scots Law)</fld> <def>One who makes a trust; -- the correlative of <xex>trustee</xex>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Trustful</ent><br/
<hw>Trust"ful</hw> <pr>(?)</pr>, <pos>a.</pos> <sn>1.</sn> <def>Full of trust; trusting.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Worthy of trust; faithful; trusty; trustworthy.</def><br/
[<source>1913 Webster</source>]</p>

<p>-- <wordforms><wf>Trust"ful*ly</wf>,<pos>adv.</pos> -- <wf>Trust"ful*ness</wf>, <pos>n.</pos></wordforms><br/
[<source>1913 Webster</source>]</p>

<p><ent>Trustily</ent><br/
<hw>Trust"i*ly</hw> <pr>(?)</pr>, <pos>adv.</pos> <def>In a trusty manner.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Trustiness</ent><br/
<hw>Trust"i*ness</hw>, <pos>n.</pos> <def>The quality or state of being trusty.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Trusting</ent><br/
<hw>Trust"ing</hw>, <pos>a.</pos> <def>Having or exercising trust; confiding; unsuspecting; trustful.</def> -- <wordforms><wf>Trust"ing*ly</wf>, <pos>adv.</pos></wordforms><br/
[<source>1913 Webster</source>]</p>

<p><ent>Trustless</ent><br/
<hw>Trust"less</hw>, <pos>a.</pos> <def>That may not be trusted; not worthy of trust; unfaithful.</def> -- <wordforms><wf>Trust"less*ness</wf>, <pos>n.</pos></wordforms><br/
[<source>1913 Webster</source>]</p>

<p><ent>Trustworthy</ent><br/
<hw>Trust"wor`thy</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Worthy of trust or confidence; trusty.</def> -- <wordforms><wf>Trust"wor`thi*ness</wf> <pr>(#)</pr>, <pos>n.</pos></wordforms><br/
[<source>1913 Webster</source>]</p>

<p><ent>Trusty</ent><br/
<hw>Trust"y</hw> <pr>(?)</pr>, <pos>a.</pos> <amorph>[<pos>Compar.</pos> <adjf>Trustier</adjf> <pr>(?)</pr>; <pos>superl.</pos> <adjf>Trustiest</adjf>.]</amorph> <sn>1.</sn> <def>Admitting of being safely trusted; justly deserving confidence; fit to be confided in; trustworthy; reliable.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Your <qex>trusty</qex> and most valiant servitor.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Hence, not liable to fail; strong; firm.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>His <qex>trusty</qex> sword he called to his aid.</q> <rj><qau>Spenser.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>Involving trust; <as>as, a <ex>trusty</ex> business</as>.</def> <mark>[R.]</mark>  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Truth</ent><br/
<hw>Truth</hw> <pr>(?)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Truths</plw> <pr>(#)</pr>.</plu> <ety>[OE. <ets>treuthe</ets>, <ets>trouthe</ets>, <ets>treowpe</ets>, AS. <ets>tre<oacute/w<?/</ets>. See <er>True</er>; cf. <er>Troth</er>, <er>Betroth</er>.]</ety> <sn>1.</sn> <def>The quality or being true; as: -- <sd>(a)</sd> Conformity to fact or reality; exact accordance with that which is, or has been; or shall be.</def><br/
[<source>1913 Webster</source>]</p>

<p><sd>(b)</sd> <def>Conformity to rule; exactness; close correspondence with an example, mood, object of imitation, or the like.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Plows, to go true, depend much on the <qex>truth</qex> of the ironwork.</q> <rj><qau>Mortimer.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sd>(c)</sd> <def>Fidelity; constancy; steadfastness; faithfulness.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Alas! they had been friends in youth,<br/
But whispering tongues can poison <qex>truth</qex>.</q> <rj><qau>Coleridge.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sd>(d)</sd> <def>The practice of speaking what is true; freedom from falsehood; veracity.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>If this will not suffice, it must appear<br/
That malice bears down <qex>truth</qex>.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>That which is true or certain concerning any matter or subject, or generally on all subjects; real state of things; fact; verity; reality.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Speak ye every man the <xex>truth</xex> to his neighbor.</q> <rj><qau>Zech. viii. 16.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>I long to know the <qex>truth</qex> here of at large.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>The <qex>truth</qex> depends on, or is only arrived at by, a legitimate deduction from all the facts which are truly material.</q> <rj><qau>Coleridge.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>A true thing; a verified fact; a true statement or proposition; an established principle, fixed law, or the like; <as>as, the great <ex>truths</ex> of morals</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Even so our boasting . . . is found a <qex>truth</qex>.</q> <rj><qau>2 Cor. vii. 14.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>Righteousness; true religion.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Grace and <qex>truth</qex> came by Jesus Christ.</q> <rj><qau>John i. 17.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Sanctify them through thy <qex>truth</qex>; thy word is <qex>truth</qex>.</q> <rj><qau>John xvii. 17.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>In truth</b></col>, <cd>in reality; in fact.</cd> -- <col><b>Of a truth</b></col>, <cd>in reality; certainly.</cd> -- <col><b>To do truth</b></col>, <cd>to practice what God commands.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><q>He that <qex>doeth truth</qex> cometh to the light.</q> <rj><qau>John iii. 21.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Truth</ent><br/
<hw>Truth</hw>, <pos>v. t.</pos> <def>To assert as true; to declare.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>Had they [the ancients] dreamt this, they would have <qex>truthed</qex> it heaven.</q> <rj><qau>Ford.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Truthful</ent><br/
<hw>Truth"ful</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Full of truth; veracious; reliable.</def> -- <wordforms><wf>Truth"ful*ly</wf>, <pos>adv.</pos> -- <wf>Truth"ful*ness</wf>, <pos>n.</pos></wordforms><br/
[<source>1913 Webster</source>]</p>

<p><ent>Truthless</ent><br/
<hw>Truth"less</hw>, <pos>a.</pos> <def>Devoid of truth; dishonest; dishonest; spurious; faithless.</def> -- <wordforms><wf>Truth"less*ness</wf>, <pos>n.</pos></wordforms><br/
[<source>1913 Webster</source>]</p>

<p><ent>Truth-lover</ent><br/
<hw>Truth"-lov`er</hw> <pr>(?)</pr>, <pos>n.</pos> <def>One who loves the truth.</def><br/
[<source>1913 Webster</source>]</p>

<p><q><qex>Truth-lover</qex> was our English Duke.</q> <rj><qau>Tennyson.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Truthness</ent><br/
<hw>Truth"ness</hw>, <pos>n.</pos> <def>Truth.</def> <mark>[Obs. & R.]</mark>  <rj><au>Marston.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Truth-teller</ent><br/
<hw>Truth"-tell`er</hw> <pr>(?)</pr>, <pos>n.</pos> <def>One who tells the truth.</def><br/
[<source>1913 Webster</source>]</p>

<p><q><qex>Truth-teller</qex> was our England's Alfred named.</q> <rj><qau>Tennyson.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Truthy</ent><br/
<hw>Truth"y</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Truthful; likely; probable.</def> <mark>[R.]</mark> <ldquo/A more <xex>truthy</xex> import.<rdquo/  <rj><au>W. G. Palgrave.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Trutination</ent><br/
<hw>Tru`ti*na"tion</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>trutinari</ets> to weigh, from <ets>trutina</ets> a balance. See <er>Trone</er> a steelyard.]</ety> <def>The act of weighing.</def> <mark>[Obs.]</mark>  <rj><au>Sir T. Browne.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Truttaceous</ent><br/
<hw>Trut*ta"ceous</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[LL. <ets>trutta</ets> a trout, L. <ets>tructa</ets>. See <er>Trout</er>.]</ety> <fld>(Zool.)</fld> <def>Of, pertaining to, or resembling, a trout; <as>as, fish of the <ex>truttaceous</ex> kind</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Try</ent><br/
<hw>Try</hw> <pr>(?)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>tried</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Trying</conjf>.]</vmorph> <ety>[OE. <ets>trien</ets> to select, pick out, F. <ets>trier</ets> to cull, to out, LL. <ets>tritare</ets> to triturate (hence the sense of, to thresh, to separate the grain from the straw, to select), L. <ets>terere</ets>, <ets>tritum</ets>, to rub, bruise, grind, thresh. See <er>Trite</er>.]</ety> <sn>1.</sn> <def>To divide or separate, as one sort from another; to winnow; to sift; to pick out; -- frequently followed by <xex>out</xex>; <as>as, to <ex>try</ex> out the wild corn from the good</as>.</def> <mark>[Obs.]</mark>  <rj><au>Sir T. Elyot.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To purify or refine, as metals; to melt out, and procure in a pure state, as oil, tallow, lard, etc.</def>  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>The words of the Lord are pure words: as silver <qex>tried</qex> in a furnace of earth, purified seven times.</q> <rj><qau>Ps. xii. 6.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>For thou, O God, hast proved us: thou hast <qex>tried</qex> us, as silver is <qex>tried</qex>.</q> <rj><qau>Ps. lxvi. 10.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>To prove by experiment; to apply a test to, for the purpose of determining the quality; to examine; to prove; to test; <as>as, to <ex>try</ex> weights or measures by a standard; to <ex>try</ex> a man's opinions</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Let the end <qex>try</qex> the man.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>To subject to severe trial; to put to the test; to cause suffering or trouble to.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Thus far to <qex>try</qex> thee, Adam, I was pleased.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>These are the times that try men's souls.</q>  <qau>Thomas Paine (1776)</qau><br/
[<source>PJC</source>]</p>

<p><-- p. 1548 --></p>

<p><sn>5.</sn> <def>To experiment with; to test by use; <as>as, to <ex>try</ex> a remedy for disease; to <ex>try</ex> a horse</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Come, <qex>try</qex> upon yourselves what you have seen me.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>To ease her cares the force of sleep she <qex>tries</qex>.</q> <rj><qau>Swift.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>6.</sn> <def>To strain; to subject to excessive tests; <as>as, the light <ex>tries</ex> his eyes</as>; repeated disappointments <xex>try</xex> one's patience.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>7.</sn> <fld>(Law)</fld> <def>To examine or investigate judicially; to examine by witnesses or other judicial evidence and the principles of law; <as>as, to <ex>try</ex> a cause, or a criminal</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>8.</sn> <def>To settle; to decide; to determine; specifically, to decide by an appeal to arms; <as>as, to <ex>try</ex> rival claims by a duel; to <ex>try</ex> conclusions</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Left I the court, to see this quarrel <qex>tried</qex>.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>9.</sn> <def>To experience; to have or gain knowledge of by experience.</def>  <rj><au>Milton.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Or <qex>try</qex> the Libyan heat or Scythian cold.</q> <rj><qau>Dryden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>10.</sn> <def>To essay; to attempt; to endeavor.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Let us <qex>try</qex> . . . to found a path.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>To try on</b></col>. <sd>(a)</sd> <cd>To put on, as a garment, to ascertain whether it fits the person.</cd> <sd>(b)</sd> <cd>To attempt; to undertake.</cd> <mark>[Slang]</mark> <au>Dickens.</au></cs><br/
[<source>1913 Webster</source>]</p>

<p><syn><b>Syn.</b> -- To attempt; endeavor; strive; aim; examine.</syn> <usage> -- <er>Try</er>, <er>Attempt</er>. To <xex>try</xex> is the generic, to <xex>attempt</xex> is the specific, term. When we <xex>try</xex>, we are usually uncertain as to success; when we <xex>attempt</xex>, we have always some definite object in view which we seek to accomplish. We may be indifferent as to the result of a <xex>trial</xex>, but we rarely <xex>attempt</xex> anything without a desire to succeed.</usage><br/
[<source>1913 Webster</source>]</p>

<p><q>He first deceased: she for a little <qex>tried</qex><br/
To live without him; liked it not, and died.</q> <rj><qau>Sir H. Wotton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Alack, I am afraid they have a waked,<br/
And 't is not done. The <qex>attempt</qex>, and not the deed,<br/
Confounds us.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Try</ent><br/
<hw>Try</hw> <pr>(?)</pr>, <pos>v. i.</pos> <sn>1.</sn> <def>To exert strength; to endeavor; to make an effort or an attempt; <as>as, you must <ex>try</ex> hard if you wish to learn</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To do; to fare; <as>as, how do you <ex>try</ex></as>!</def> <mark>[Prov. Eng.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Try</ent><br/
<hw>Try</hw>, <pos>n.</pos> <sn>1.</sn> <def>A screen, or sieve, for grain.</def> <mark>[Obs. or Prov. Eng.]</mark>  <rj><au>Holland.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Act of trying; attempt; experiment; trial.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>This breaking of his has been but a <qex>try</qex> for his friends.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>In Rugby and Northern Union football, a score (counting three points) made by grounding the ball on or behind the opponent's goal line; -- so called because it entitles the side making it to a place kick for a goal (counting two points more if successful).</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Try</ent><br/
<hw>Try</hw>, <pos>a.</pos> <ety>[Cf. <er>Try</er>, <pos>v. t.</pos>]</ety> <def>Refined; select; excellent; choice.</def> <mark>[Obs.]</mark> <ldquo/Sugar that is <xex>try</xex>.<rdquo/  <rj><au>Chaucer.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Try cock</ent><br/
<hw>Try cock</hw>. <def>A cock for withdrawing a small quantity of liquid, as for testing.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Trygon</ent><br/
||<hw>Try"gon</hw> <pr>(tr<imac/"g<ocr/n)</pr>, <pos>n.</pos> <ety>[L., a sting ray, from Gr.<?/ a kind of fish with a prickle in the tail.]</ety> <fld>(Zool.)</fld> <def>Any one of several species of large sting rays belonging to <gen>Trygon</gen> and allied genera.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Trying</ent><br/
<hw>Try"ing</hw>, <pos>a.</pos> <def>Adapted to try, or put to severe trial; severe; afflictive; <as>as, a <ex>trying</ex> occasion or position</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tryout</ent><br/
<hw>Try"out</hw> <pr>(?)</pr>, <pos>n.</pos> <sn>1.</sn> <def>a test of the performance capability of a person, to ascertain fitness for a particular task; in sports, a test by which the fitness of a player or contestant to remain in a certain class is determined.</def><br/
[<source>Webster 1913 Suppl.</source> <source>+PJC</source>]</p>

<p><sn>2.</sn> <fld>(Theater)</fld> <def>one or more performances of a play prior to the official opening, held at a location outside of the city where it is to be formally presented, conducted for the purpose of determining audience response or ascertaining weaknesses needing correction.</def><br/
[<source>PJC</source>]</p>

<p><ent>Try out</ent><br/
<hw>Try out</hw> <pr>(?)</pr>, <pos>v. i.</pos> <def>to compete for a postition, as on a sports team or in a theatrical presentation.</def><br/
[<source>PJC</source>]</p>

<p><ent>Try out</ent><br/
<hw>Try out</hw> <pr>(?)</pr>, <pos>v. t.</pos> <def>to use (something not previously used) to determine its fitness for a particular purpose; also, to attempt (a deed) to determine if it will accomplish a particular purpose.</def><br/
[<source>PJC</source>]</p>

<p><ent>Trypsin</ent><br/
<hw>Tryp"sin</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Gr. <?/ a rubbing, fr. <?/ to rub, grind. So called because it causes protein to break up or to fall apart.]</ety> <fld>(physiol.)</fld> <def>A proteolytic enzyme present in the pancreatic juice. Unlike the pepsin of the gastric juice, it acts in a neutral or alkaline fluid, and not only converts the albuminous matter of the food into soluble peptones, but also, in part, into leucin and tyrosin.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Trypsinogen</ent><br/
<hw>Tryp*sin"o*gen</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[<ets>Trypsin</ets> + <ets>-gen</ets>.]</ety> <fld>(Physiol.)</fld> <def>The antecedent of trypsin, a substance which is contained in the cells of the pancreas and gives rise to the trypsin.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tryptic</ent><br/
<hw>Tryp"tic</hw> <pr>(?)</pr>, <pos>a.</pos> <fld>(Physiol.)</fld> <def>Relating to trypsin or to its action; produced by trypsin; <as>as, <ex>trypsin</ex> digestion</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tryptone</ent><br/
<hw>Tryp"tone</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Physiol. Chem.)</fld> <def>The peptone formed by pancreatic digestion; -- so called because it is formed through the agency of the ferment <xex>trypsin</xex>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Trysail</ent><br/
<hw>Try"sail</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Naut.)</fld> <def>A fore-and-aft sail, bent to a gaff, and hoisted on a lower mast or on a small mast, called the <xex>trysail mast</xex>, close abaft a lower mast; -- used chiefly as a storm sail. Called also <altname>spencer</altname>.</def>  <rj><au>Totten.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Try-square</ent><br/
<hw>Try"-square`</hw> <pr>(?)</pr>, <pos>n.</pos> <def>An instrument used by carpenters, joiners, etc., for laying off right angles off right angles, and testing whether work is square.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tryst</ent><br/
<hw>Tryst</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[OE. <ets>trist</ets>, <ets>tryst</ets>, a variant of <ets>trust</ets>; cf. Icel. <ets>treysta</ets> to make trusty, fr. <ets>traust</ets> confidence, security. See <er>Trust</er>, <pos>n.</pos>]</ety> <sn>1.</sn> <def>Trust.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>An appointment to meet; also, an appointed place or time of meeting; <as>as, to keep <ex>tryst</ex>; to break <ex>tryst</ex></as>.</def> <mark>[Scot. or Poetic]</mark><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>To bide tryst</b></col>, <cd>to wait, at the appointed time, for one with whom a tryst or engagement is made; to keep an engagement or appointment.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><q>The tenderest-hearted maid<br/
That ever <qex>bided tryst</qex> at village stile.</q> <rj><qau>Tennyson.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tryst</ent><br/
<hw>Tryst</hw>, <pos>v. t.</pos> <ety>[OE. <ets>tristen</ets>, <ets>trysten</ets>. See <er>Tryst</er>, <pos>n.</pos>]</ety><br/
[<source>1913 Webster</source>]</p>

<p><sn>1.</sn> <def>To trust.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To agree with to meet at a certain place; to make an appointment with.</def> <mark>[Scot.]</mark>  <rj><au>Burns.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tryst</ent><br/
<hw>Tryst</hw>, <pos>v. i.</pos> <def>To mutually agree to meet at a certain place.</def> <mark>[Scot.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tryster</ent><br/
<hw>Tryst"er</hw> <pr>(?)</pr>, <pos>n.</pos> <def>One who makes an appointment, or tryst; one who meets with another.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Trysting</ent><br/
<hw>Tryst"ing</hw>, <pos>n.</pos> <def>An appointment; a tryst.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Trysting day</b></col>, <cd>an arranged day of meeting or assembling, as of soldiers, friends, and the like.</cd><br/
[<source>1913 Webster</source>]</p>

<p><q>And named a <qex>trysting day</qex>,<br/
And bade his messengers ride forth<br/
East and west and south and north,<br/
To summon his array.</q> <rj><qau>Macaulay.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p>-- <col><b>Trysting place</b></col>, <cd>a place designated for the assembling of soldiers, the meeting of parties for an interview, or the like; a rendezvous.</cd> <au>Byron.</au></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tsar</ent><br/
<hw>Tsar</hw> <pr>(?)</pr>, <pos>n.</pos> <def>The title of the emperor of Russia. See <er>Czar</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tsaritsa</ent><br/
<ent>Tsarina</ent><br/
<mhw>{ <hw>Tsa*ri"na</hw> <pr>(?)</pr>, <hw>Tsa*rit"sa</hw> <pr>(?)</pr>, }</mhw> <pos>n.</pos> <ety>[Russ. <ets>tsaritsa</ets>.  Cf. <er>Czarina</er>.]</ety> <def>The title of the empress of Russia. See <er>Czarina</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tschakmeck</ent><br/
||<hw>Tschak*meck"</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Zool.)</fld> <def>The chameck.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tschego</ent><br/
||<hw>Tsche"go</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[From a native name.]</ety> <fld>(Zool.)</fld> <def>A West African anthropoid ape allied to the gorilla and chimpanzee, and by some considered only a variety of the chimpanzee. It is noted for building large, umbrella-shaped nests in trees. Called also <altname>tscheigo</altname>, <altname>tschiego</altname>, <altname>nschego</altname>, <altname>nscheigo</altname>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tsebe</ent><br/
<hw>Tse"be</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Zool.)</fld> <def>The springbok.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tsetse</ent><br/
<hw>Tset"se</hw>, <pos>n.</pos> <fld>(Zool.)</fld> <def>A venomous two-winged African fly (<spn>Glossina morsitans</spn>) whose bite is very poisonous, and even fatal, to horses and cattle, but harmless to men. It renders extensive districts in which it abounds uninhabitable during certain seasons of the year.</def> <altsp>[Written also <asp>tzetze</asp>, and <asp>tsetze</asp>.]</altsp>
<-- carrier of sleeping sickness, in areas where that disease is endemic. --><br/
[<source>1913 Webster</source>]</p>

<p><ent>T square</ent><br/
<hw>T" square`</hw> <pr>(?)</pr>. <def>See under <er>T</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tsung-li Yamen</ent><br/
||<hw>Tsung"-li Ya"men</hw> <pr>(?)</pr>. <altsp>[Written also <asp>Tsung-li-Yamen</asp> or <asp>Tsungli Yamen</asp>.]</altsp> <ety>[Chin.]</ety> <def>The board or department of foreign affairs in the Chinese government. See <er>Yamen</er>.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Tsung tu</ent><br/
||<hw>Tsung" tu`</hw> <pr>(?)</pr>. <def>A viceroy or governor-general, the highest provincial official in China, with civil and military authority over one or more provinces.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Tuatara</ent><br/
<hw>Tu`a*ta"ra</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Maori <ets>tuat<agrave/ra</ets>; <ets>tua</ets> on the farther side (the back) + <ets>tara</ets> spine.]</ety> <fld>(Zool.)</fld> <def>A large iguanalike reptile (<spn>Sphenodon punctatum</spn>) formerly common in New Zealand, but by 1900 confined to certain islets near the coast. It reaches a length of two and a half feet, is dark olive-green with small white or yellowish specks on the sides, and has yellow spines along the back, except on the neck.  It is the only surviving member of the order <ord>Rhyncocephala</ord>.  Also called <altname>tuatera</altname> and <altname>hatteria</altname>.</def><br/
[<source>Webster 1913 Suppl.</source> <source>+PJC</source>]</p>

<p><ent>Tuatera</ent><br/
<hw>Tu`a*te"ra</hw> <pr>(?)</pr>, <pos>n.</pos> <def>Same as <er>tuatara</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tub</ent><br/
<hw>Tub</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[OE. <ets>tubbe</ets>; of Dutch or Low German origin; cf. LG. <ets>tubbe</ets>, D. <ets>tobbe</ets>.]</ety> <sn>1.</sn> <def>An open wooden vessel formed with staves, bottom, and hoops; a kind of short cask, half barrel, or firkin, usually with but one head, -- used for various purposes.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>The amount which a tub contains, as a measure of quantity; <as>as, a <ex>tub</ex> of butter; a <ex>tub</ex> of camphor, which is about 1 cwt., etc</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>Any structure shaped like a tub: as, a certain old form of pulpit; a short, broad boat, etc., -- often used jocosely or opprobriously.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>All being took up and busied, some in pulpits and some in <qex>tubs</qex>, in the grand work of preaching and holding forth.</q> <rj><qau>South.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>A sweating in a tub; a tub fast.</def> <mark>[Obs.]</mark>  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <def>A small cask; <as>as, a <ex>tub</ex> of gin</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>6.</sn> <def>A box or bucket in which coal or ore is sent up a shaft; -- so called by miners.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Tub fast</b></col>, <cd>an old mode of treatment for the venereal disease, by sweating in a close place, or tub, and fasting.</cd> <mark>[Obs.]</mark> <au>Shak.</au> -- <col><b>Tub wheel</b></col>, <cd>a horizontal water wheel, usually in the form of a short cylinder, to the circumference of which spiral vanes or floats, placed radially, are attached, turned by the impact of one or more streams of water, conducted so as to strike against the floats in the direction of a tangent to the cylinder.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tub</ent><br/
<hw>Tub</hw>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Tubbed</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Tubbing</conjf>.]</vmorph> <def>To plant or set in a tub; <as>as, to <ex>tub</ex> a plant</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tub</ent><br/
<hw>Tub</hw>, <pos>v. i.</pos> <def>To make use of a bathing tub; to lie or be in a bath; to bathe.</def> <mark>[Colloq.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>Don't we all <qex>tub</qex> in England ?</q> <rj><qau>London Spectator.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tuba</ent><br/
<hw>Tu"ba</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L., trumpet.]</ety> <fld>(Mus.)</fld> <sd>(a)</sd> <def>An ancient trumpet.</def> <sd>(b)</sd> <def>A sax-tuba. See <er>Sax-tuba</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tubal</ent><br/
<hw>Tub"al</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Of or pertaining to a tube; specifically, of or pertaining to one of the Fallopian tubes; <as>as, <ex>tubal</ex> pregnancy</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tubbing</ent><br/
<hw>Tub"bing</hw> <pr>(?)</pr>, <pos>n.</pos> <sn>1.</sn> <def>The forming of a tub; also, collectively, materials for tubs.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A lining of timber or metal around the shaft of a mine; especially, a series of cast-iron cylinders bolted together, used to enable those who sink a shaft to penetrate quicksand, water, etc., with safety.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tubby</ent><br/
<hw>Tub"by</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Resembling a tub; specifically sounding dull and without resonance, like a tub; wanting elasticity or freedom of sound; <as>as, a <ex>tubby</ex> violin</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tube</ent><br/
<hw>Tube</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>tubus</ets>; akin to <ets>tuba</ets> a trumpet: cf F. <ets>tube</ets>.]</ety> <sn>1.</sn> <def>A hollow cylinder, of any material, used for the conveyance of fluids, and for various other purposes; a pipe.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A telescope.</def> <ldquo/Glazed optic <xex>tube</xex>.<rdquo/  <rj><au>Milton.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>A vessel in animal bodies or plants, which conveys a fluid or other substance.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <fld>(Bot.)</fld> <def>The narrow, hollow part of a gamopetalous corolla.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <fld>(Gun.)</fld> <def>A priming tube, or friction primer. See under <er>Priming</er>, and <er>Friction</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>6.</sn> <fld>(Steam Boilers)</fld> <def>A small pipe forming part of the boiler, containing water and surrounded by flame or hot gases, or else surrounded by water and forming a flue for the gases to pass through.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>7.</sn> <fld>(Zool.)</fld> <sd>(a)</sd> <def>A more or less cylindrical, and often spiral, case secreted or constructed by many annelids, crustaceans, insects, and other animals, for protection or concealment. See <xex>Illust.</xex> of <er>Tubeworm</er>.</def> <sd>(b)</sd> <def>One of the siphons of a bivalve mollusk.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>8.</sn> <fld>(Elec. Railways)</fld> <def>A tunnel for a tube railway; also <mark>(Colloq.)</mark>, a tube railway; a subway.</def> <mark>[Chiefly Eng.]</mark> <note>In the New York area, the subways running under the Hudson River are sometimes referred to as the <ex>tube</ex>.</note><br/
[<source>Webster 1913 Suppl.</source> <source>+PJC</source>]</p>

<p><cs><col><b>Capillary tube</b></col>, <cd>a tube of very fine bore. See <er>Capillary</er>.</cd> -- <col><b>Fire tube</b></col> <fld>(Steam Boilers)</fld>, <cd>a tube which forms a flue.</cd> -- <col><b>Tube coral</b></col>. <fld>(Zool.)</fld> <cd>Same as <er>Tubipore</er>.</cd> -- <col><b>Tube foot</b></col> <fld>(Zool.)</fld>, <cd>one of the ambulacral suckers of an echinoderm.</cd> -- <mcol><col><b>Tube plate</b></col>, <it>or</it>  <col><b>Tube sheet</b></col></mcol> <fld>(Steam Boilers)</fld>, <cd>a flue plate. See under <er>Flue</er>.</cd> -- <col><b>Tube pouch</b></col> <fld>(Mil.)</fld>, <cd>a pouch containing priming tubes.</cd> -- <col><b>Tube spinner</b></col> <fld>(Zool.)</fld>, <cd>any one of various species of spiders that construct tubelike webs. They belong to <gen>Tegenaria</gen>, <gen>Agelena</gen>, and allied genera.</cd> -- <col><b>Water tube</b></col> <fld>(Steam Boilers)</fld>, <cd>a tube containing water and surrounded by flame or hot gases.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tube</ent><br/
<hw>Tube</hw>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Tubed</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Tubing</conjf>.]</vmorph> <def>To furnish with a tube; <as>as, to <ex>tube</ex> a well</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tubeform</ent><br/
<hw>Tube"form</hw> <pr>(?)</pr>, <pos>a.</pos> <def>In the form of a tube; tubular; tubiform.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tube-nosed</ent><br/
<hw>Tube"-nosed`</hw> <pr>(?)</pr>, <pos>a.</pos> <fld>(Zool.)</fld> <sd>(a)</sd> <def>Having the nostrils prolonged in the form of horny tubes along the sides of the beak; -- said of certain sea birds.</def> <sd>(b)</sd> <def>Belonging to the Tubinares.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tuber</ent><br/
<hw>Tu"ber</hw> <pr>(?)</pr>, <pos>n.</pos><ety>[L., a hump. knob; probably akin to <ets>tumere</ets> to swell.  Cf. <er>Tumid</er>.]</ety> <sn>1.</sn> <fld>(Bot.)</fld> <sd>(a)</sd> <def>A fleshy, rounded stem or root, usually containing starchy matter, as the potato or arrowroot; a thickened root-stock. See <xex>Illust.</xex> of <er>Tuberous</er>.</def> <sd>(b)</sd> <def>A genus of fungi. See <er>Truffle</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Anat.)</fld> <def>A tuberosity; a tubercle.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tube railway</ent><br/
<hw>Tube rail"way</hw>, <pos>n.</pos> <def>an electrically powered railroad with tracks running through a tunnel underground; a subway.</def><br/
[<source>PJC</source>]</p>

<p><ent>Tubercle</ent><br/
<hw>Tu"ber*cle</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>tuberculum</ets>, dim. of <ets>tuber</ets>: cf. F. <ets>tubercule</ets>, OF. also <ets>tubercle</ets>. See <er>Tuber</er>.]</ety><br/
[<source>1913 Webster</source>]</p>

<p><sn>1.</sn> <def>A small knoblike prominence or excrescence, whether natural or morbid; <as>as, a <ex>tubercle</ex> on a plant; a <ex>tubercle</ex> on a bone; the <ex>tubercles</ex> appearing on the body in leprosy.</as></def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Med.)</fld> <def>A small mass or aggregation of morbid matter; especially, the deposit which accompanies scrofula or phthisis. This is composed of a hard, grayish, or yellowish, translucent or opaque matter, which gradually softens, and excites suppuration in its vicinity. It is most frequently found in the lungs, causing consumption.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Tubercle bacillus</b></col> <fld>(Med.)</fld>, <cd>a minute vegetable organism (<spn>Mycobacterium tuberculosis</spn>, formerly <spn>Bacillus tuberculosis</spn>, and also called <altname>Koch's bacillus</altname>) discovered by Koch, a German physician, in the sputum of consumptive patients and in tuberculous tissue. It is the causative agent of tuberculosis.</cd></cs><br/
[<source>1913 Webster</source> <source>+PJC</source>]</p>

<p><ent>Tubercled</ent><br/
<hw>Tu"ber*cled</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Having tubercles; affected with, tubercles; tuberculate; <as>as, a <ex>tubercled</ex> lung or stalk</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tubercular</ent><br/
<hw>Tu*ber"cu*lar</hw> <pr>(?)</pr>, <pos>a.</pos> <sn>1.</sn> <def>Having tubercles; affected with tubercles; tubercled; tuberculate.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Like a tubercle; <as>as, a <ex>tubercular</ex> excrescence</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <fld>(Med.)</fld> <def>Characterized by the development of tubercles; <as>as, <ex>tubercular</ex> diathesis</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tubercularize</ent><br/
<hw>Tu*ber"cu*lar*ize</hw> <pr>(?)</pr>, <pos>v. t.</pos>  <vmorph>[<pos>imp. & p. p.</pos> <conjf>-ized</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>i-zing</conjf> <pr>(?)</pr>.]</vmorph> <ety>[<ets>Tubercular</ets> + <ets>-ize</ets>.]</ety> <fld>(Med.)</fld> <def>To infect with tuberculosis.</def> -- <wordforms><wf>Tu*ber`cu*lar*i*za"tion</wf> <pr>(#)</pr>, <pos>n.</pos></wordforms><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Tuberculated</ent><br/
<ent>Tuberculate</ent><br/
<mhw>{ <hw>Tu*ber"cu*late</hw> <pr>(?)</pr>, <hw>Tu*ber"cu*la`ted</hw> <pr>(?)</pr>, }</mhw> <pos>a.</pos> <ety>[NL. <ets>tuberculatus</ets>: cf. F. <ets>tubercul<eacute/</ets>.]</ety> <def>Tubercled; tubercular.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tuberculin</ent><br/
<hw>Tu*ber"cu*lin</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[See <er>Tubercle</er>.]</ety> <def>A fluid containing the products formed by the growth of the tubercle bacillus in a suitable culture medium.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tuberculin test</ent><br/
<hw>Tu*ber"cu*lin test</hw> <pr>(?)</pr>. <def>The hypodermic injection of tuberculin, used as a test for past or present infection with tuberculosis.  It is used both for humans and cattle.</def> <note>In humans, a previous or acive infection with tuberculosis will cause a distinct reaction to the injection of tuberculin.  The test is thus used to screen populations to discover those who may harbor an active tuberculosis infection, and more definitive tests are performed on those showing a positive reaction, to determine if the infection is active.  In cattle there is little or no effect of the injection of tuberculin on healthy animals, but there is a marked rise in temperature in tuberculous animals.</note><br/
[<source>Webster 1913 Suppl.</source> <source>+PJC</source>]</p>

<p><ent>Tuberculization</ent><br/
<hw>Tu*ber`cu*li*za"tion</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Med.)</fld> <def>The development of tubercles; the condition of one who is affected with tubercles.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tuberculocidal</ent><br/
<hw>Tu*ber"cu*lo*cid`al</hw> <pr>(?)</pr>, <pos>a.</pos>  <ety>[<ets>Tuberculum</ets> + root of L. <ets>caedere</ets> to kill.]</ety> <def>able to kill <spn>Mycobacterum tuberculosis</spn>.</def><br/
[<source>PJC</source>]</p>

<p><ent>Tuberculocidin</ent><br/
<hw>Tu*ber"cu*lo*ci`din</hw> <pr>(?)</pr>, <pos>n.</pos>  <ety>[<ets>Tuberculum</ets> + root of L. <ets>caedere</ets> to kill.]</ety> <fld>(Physiol. Chem.)</fld> <def>A special substance contained in tuberculin, supposed to be the active agent of the latter freed from various impurities.</def> <-- modern equivalent?? Not in Stedman --><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Tuberculoid</ent><br/
<hw>Tu*ber"cu*loid</hw> <pr>(?)</pr>, <pos>a.</pos>  <ety>[<ets>Tuberculum</ets> + <ets>-oid</ets>.]</ety> <fld>(Med.)</fld> <def>Resembling a tubercle.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Tuberculous</ent><br/
<ent>Tuberculose</ent><br/
<mhw>{ <hw>Tu*ber"cu*lose`</hw> <pr>(?)</pr>, <hw>Tu*ber"cu*lous</hw> <pr>(?)</pr>, }</mhw> <pos>a.</pos> <def>Having tubercles; affected with, or characterized by, tubercles; tubercular.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tuberculosed</ent><br/
<hw>Tu*ber"cu*losed`</hw> <pr>(?)</pr>, <pos>a.</pos> <fld>(Med.)</fld> <def>Affected with tuberculosis.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Tuberculosis</ent><br/
||<hw>Tu*ber`cu*lo"sis</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[NL. See <er>Tubercle</er>.]</ety> <fld>(Med.)</fld> <def>A constitutional disease caused by infection with <spn>Mycobacterium tuberculosis</spn> (also called the <er>Tubercle bacillus</er>), characterized by the production of tubercles in the internal organs, and especially in the lungs, where it constitutes the most common variety of pulmonary phthisis (consumption).  The Mycobacteria are slow-growing and without cell walls, and are thus not affected by the beta-lactam antibiotics; treatment is difficult, usually requiring simultaneous administration of multiple antibiotics to effect a cure.  Prior to availability of antibiotic treatment, the cure required extensive rest, for which special sanatoriums were constructed.</def><br/
[<source>1913 Webster</source> <source>+PJC</source>]</p>

<p><ent>Tuberculous</ent><br/
<hw>Tu*ber"cu*lous</hw> <pr>(?)</pr>, <pos>a.</pos> <fld>(Med.)</fld> <def>Pertaining to, or affected with, a tuberculosis.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Tuberculum</ent><br/
||<hw>Tu*ber"cu*lum</hw> <pr>(?)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Tubercula</plw> <pr>(#)</pr>.</plu> <ety>[L., dim. of <ets>tuber</ets> a swelling.]</ety> <fld>(Zool.)</fld> <def>A tubercle.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tuberiferous</ent><br/
<hw>Tu`ber*if"er*ous</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[<ets>Tuber</ets> + <ets>-ferous</ets>.]</ety> <def>Producing or bearing tubers.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tuberose</ent><br/
<hw>Tube"rose`</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Cf. G. <ets>tuberose</ets>, F. <ets>tub<eacute/reuse</ets>, NL. <ets>Polianthes tuberosa</ets>. See <er>Tuberous</er>.]</ety> <fld>(Bot.)</fld> <def>A plant (<spn>Polianthes tuberosa</spn>) with a tuberous root and a liliaceous flower. It is much cultivated for its beautiful and fragrant white blossoms.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tuberose</ent><br/
<hw>Tu"ber*ose`</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Tuberous.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tuberosity</ent><br/
<hw>Tu`ber*os"i*ty</hw> <pr>(?)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Tuberosities</plw> <pr>(#)</pr>.</plu> <ety>[Cf. F. <ets>tub<eacute/rosit<eacute/</ets>.]</ety> <sn>1.</sn> <def>The state of being tuberous.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>An obtuse or knoblike prominence; a protuberance.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tuberous</ent><br/
<hw>Tu"ber*ous</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>tuberosus</ets>: cf. F. <ets>tub<eacute/reux</ets>. See <er>Tuber</er>, and cf. also <er>Tuberose</er>.]</ety><br/
[<source>1913 Webster</source>]</p>

<p><sn>1.</sn> <def>Covered with knobby or wartlike prominences; knobbed.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Bot.)</fld> <def>Consisting of, or bearing, tubers; resembling a tuber.</def><br/
[<source>1913 Webster</source>]</p>

<p>-- <wordforms><wf>Tu"ber*ous*ness</wf>, <pos>n.</pos></wordforms><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tube-shell</ent><br/
<hw>Tube"-shell`</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Zool.)</fld> <def>Any bivalve mollusk which secretes a shelly tube around its siphon, as the watering-shell.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tubeworm</ent><br/
<hw>Tube"worm`</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Zool.)</fld> <def>Any annelid which constructs a tube; one of the Tubicolae.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tubfish</ent><br/
<hw>Tub"fish`</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Zool.)</fld> <def>The sapphirine gurnard (<spn>Trigla hirundo</spn>). See <xex>Illust.</xex> under <er>Gurnard</er>.</def> <mark>[Prov. Eng.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tubful</ent><br/
<hw>Tub"ful</hw> <pr>(?)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Tubfuls</plw> <pr>(<?/)</pr>.</plu> <def>As much as a tub will hold; enough to fill a tub.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tubicinate</ent><br/
<hw>Tu*bic"i*nate</hw> <pr>(?)</pr>, <pos>v. i.</pos> <ety>[L. <ets>tubicen</ets> trumpeter.]</ety> <def>To blow a trumpet.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tubicolae</ent><br/
||<hw>Tu*bic"o*lae</hw> <pr>(?)</pr>, <pos>n. pl.</pos> <ety>[L.  <ets>tubus</ets> a tube + <ets>colere</ets> to inhabit.]</ety> <fld>(Zool.)</fld> <def>A division of annelids including those which construct, and habitually live in, tubes. The head or anterior segments usually bear gills and cirri. Called also <altname>Sedentaria</altname>, and <altname>Capitibranchiata</altname>. See <er>Serpula</er>, and <er>Sabella</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tubicolar</ent><br/
<hw>Tu*bic"o*lar</hw> <pr>(?)</pr>, <pos>a.</pos> <fld>(Zool.)</fld> <def>Tubicolous.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tubicole</ent><br/
<hw>Tu"bi*cole</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Zool.)</fld> <def>One of the Tubicolae.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tubicolous</ent><br/
<hw>Tu*bic"o*lous</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[See <er>Tubicolae</er>.]</ety> <fld>(Zool.)</fld> <def>Inhabiting a tube; <as>as, <ex>tubicolous</ex> worms</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tubicorn</ent><br/
<hw>Tu"bi*corn</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>tubus</ets> tube + <ets>cornu</ets> horn: cf. F. <ets>tubicorne</ets>.]</ety> <fld>(Zool.)</fld> <def>Any ruminant having horns composed of a bony axis covered with a horny sheath; a hollow-horned ruminant.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tubicornous</ent><br/
<hw>Tu"bi*corn`ous</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Having hollow horns.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tubiform</ent><br/
<hw>Tu"bi*form</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Having the form of a tube; tubeform.</def> <ldquo/<xex>Tubiform</xex> cells.<rdquo/  <rj><au>Carpenter.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tubinares</ent><br/
<hw>Tu`bi*na"res</hw> <pr>(?)</pr>, <pos>n. pl.</pos> <ety>[NL., fr. L. <ets>tubus</ets> tube + <ets>nares</ets> the nostrils.]</ety> <fld>(Zool.)</fld> <def>A tribe of sea birds comprising the petrels, shearwaters, albatrosses, hagdons, and allied birds having tubular horny nostrils.</def><br/
[<source>1913 Webster</source>]</p>

<p><-- p. 1549 --><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tubing</ent><br/
<hw>Tub"ing</hw> <pr>(?)</pr>, <pos>n.</pos> <sn>1.</sn> <def>The act of making tubes.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A series of tubes; tubes, collectively; a length or piece of a tube; material for tubes; <as>as, leather <ex>tubing</ex></as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tubipora</ent><br/
||<hw>Tu*bip"o*ra</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[NL., from L. <ets>tubus</ets> tube + <ets>porus</ets> passage, pore.]</ety> <fld>(Zool.)</fld> <def>A genus of halcyonoids in which the skeleton, or coral (called <xex>organ-pipe coral</xex>), consists of a mass of parallel cylindrical tubes united at intervals by transverse plates. These corals are usually red or purple and form large masses. They are natives of the tropical parts of the Indian and Pacific Oceans.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tubipore</ent><br/
<hw>Tu"bi*pore</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Cf. F. <ets>tubipore</ets>.]</ety> <fld>(Zool.)</fld> <def>Any species of the genus Tubipora.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tubiporite</ent><br/
<hw>Tu"bi*po*rite</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Paleon.)</fld> <def>Any fossil coral of the genus <gen>Syringopora</gen> consisting of a cluster of upright tubes united together by small transverse tubules.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tubivalve</ent><br/
<hw>Tu"bi*valve</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[See <er>Tube</er>, <er>Valve</er>.]</ety> <fld>(Zool.)</fld> <def>A shell or tube formed by an annelid, as a serpula.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tubman</ent><br/
<hw>Tub"man</hw> <pr>(?)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Tubmen</plw> <pr>(<?/)</pr>.</plu> <fld>(Eng. Law)</fld> <def>One of the two most experienced barristers in the Court of Exchequer.  Cf. <er>Postman</er>, 2.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tubular</ent><br/
<hw>Tu"bu*lar</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>tubulus</ets>, dim. of <ets>tubus</ets> a tube, or pipe. See <er>Tube</er>.]</ety> <def>Having the form of a tube, or pipe; consisting of a pipe; fistular; <as>as, a <ex>tubular</ex> snout; a <ex>tubular</ex> calyx. Also, containing, or provided with, tubes.</as></def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Tubular boiler</b></col>. <cd>See under <er>Boiler</er>.</cd> -- <col><b>Tubular breathing</b></col> <fld>(Med.)</fld>, <cd>a variety of respiratory sound, heard on auscultation over the lungs in certain cases of disease, resembling that produced by the air passing through the trachea.</cd> -- <col><b>Tubular bridge</b></col>, <cd>a bridge in the form of a hollow trunk or tube, made of iron plates riveted together, as the Victoria bridge over the St. Lawrence, at Montreal, Canada, and the Britannia bridge over the Menai Straits.</cd> -- <col><b>Tubular girder</b></col>, <cd>a plate girder having two or more vertical webs with a space between them.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tubularia</ent><br/
||<hw>Tu`bu*la"ri*a</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[NL.]</ety> <fld>(Zool.)</fld> <def>A genus of hydroids having large, naked, flowerlike hydranths at the summits of long, slender, usually simple, stems. The gonophores are small, and form clusters at the bases of the outer tentacles.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tubulariae</ent><br/
||<hw>Tu`bu*la"ri*ae</hw> <pr>(?)</pr>, <pos>n. pl.</pos> <ety>[NL.]</ety> <def>See <er>Tubularida</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tubularian</ent><br/
<hw>Tu`bu*la"ri*an</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Zool.)</fld> <def>Any hydroid belonging to the suborder Tubularida.</def><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ These hydroids usually form tufts of delicate tubes, and both gonophores and hydranths are naked. The gonophores of many of the species become free jellyfishes; those of other species remain permanently attached as medusoid buds or sporosacs. See <xex>Illust.</xex> under <er>Gonosome</er>, and <er>Cymnoblastea</er>.</note><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tubularian</ent><br/
<hw>Tu`bu*la"ri*an</hw>, <pos>a.</pos> <fld>(Zool.)</fld> <def>Of or pertaining to the tubularians.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tubularida</ent><br/
||<hw>Tu"bu*lar`i*da</hw> <pr>(?)</pr>, <pos>n. pl.</pos> <ety>[NL.]</ety> <fld>(Zool.)</fld> <def>An extensive division of Hydroidea; the tubularians; -- called also <altname>Athecata</altname>, <altname>Gymnoblastea</altname>, and <altname>Tubulariae</altname>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tubulate</ent><br/
<hw>Tu"bu*late</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>tubulatus</ets>. See <er>Tubular</er>.]</ety> <def>Tubular; tubulated; tubulous.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tubulated</ent><br/
<hw>Tu"bu*la`ted</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Made in the form of a small tube; provided with a tube, or elongated opening.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><mcol><col><b>Tubulated bottle</b></col> <it>or</it> <col><b>Tubulated retort</b></col></mcol> <fld>(Chem.)</fld>, <cd>a bottle or retort having a stoppered opening for the introduction or removal of materials.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tubulation</ent><br/
<hw>Tu`bu*la"tion</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Chem.)</fld> <def>The act of shaping or making a tube, or of providing with a tube; also, a tube or tubulure; <as>as, the <ex>tubulation</ex> of a retort</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tubulature</ent><br/
<hw>Tu"bu*la`ture</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Chem.)</fld> <def>A tubulure.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tubule</ent><br/
<hw>Tu"bule</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F. <ets>tubule</ets>, or L. <ets>tubulus</ets>, dim. of <ets>tubus</ets> a tube, a pipe.]</ety> <sn>1.</sn> <def>A small pipe or fistular body; a little tube.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Anat.)</fld> <def>A minute tube lined with glandular epithelium; <as>as, the uriniferous <ex>tubules</ex> of the kidney</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tubulibranchian</ent><br/
<hw>Tu`bu*li*bran"chi*an</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Zool.)</fld> <def>One of the Tubulibranchiata.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tubulibranchiata</ent><br/
||<hw>Tu`bu*li*bran`chi*a"ta</hw> <pr>(?)</pr>, <pos>n. pl.</pos> <ety>[NL., from L. <ets>tubulus</ets> a little tube + <ets>branchia</ets> a gill.]</ety> <fld>(Zool.)</fld> <def>A group of gastropod mollusks having a tubular shell. Vermetus is an example.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tubulicole</ent><br/
<hw>Tu"bu*li*cole`</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>tubulus</ets> little tube + <ets>colere</ets> to inhabit.]</ety> <fld>(Zool.)</fld> <def>Any hydroid which has tubular chitinous stems.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tubulidentate</ent><br/
<hw>Tu`bu*li*den"tate</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[<ets>Tubule</ets> + <ets>dentate</ets>.]</ety> <fld>(Zool.)</fld> <def>Having teeth traversed by canals; -- said of certain edentates.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tubuliform</ent><br/
<hw>Tu"bu*li*form`</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[Cf. F. <ets>tubuliforme</ets>.]</ety> <def>Having the form of a small tube.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tubulipore</ent><br/
<hw>Tu"bu*li*pore"</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Zool.)</fld> <def>Any one of numerous species of Bryozoa belonging to <gen>Tubulipora</gen> and allied genera, having tubular calcareous calicles.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tubulous</ent><br/
<ent>Tubulose</ent><br/
<mhw>{ <hw>Tu"bu*lose`</hw> <pr>(?)</pr>, <hw>Tu"bu*lous</hw> <pr>(?)</pr>, }</mhw> <pos>a.</pos> <ety>[Cf. F. <ets>tubuleux</ets>. See <er>Tubule</er>.]</ety> <sn>1.</sn> <def>Resembling, or in the form of, a tube; longitudinally hollow; specifically <fld>(Bot.)</fld>, having a hollow cylindrical corolla, often expanded or toothed at the border; <as>as, a <ex>tubulose</ex> flower</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Containing, or consisting of, small tubes; specifically <fld>(Bot.)</fld>, composed wholly of tubulous florets; <as>as, a <ex>tubulous</ex> compound flower</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Tubulous boiler</b></col>, <cd>a steam boiler composed chiefly of tubes containing water and surrounded by flame and hot gases; -- sometimes distinguished from <xex>tubular boiler</xex>.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tubulure</ent><br/
<hw>Tu"bu*lure</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Cf. F. <ets>tubulure</ets>.]</ety> <fld>(Chem.)</fld> <def>A short tubular opening at the top of a retort, or at the top or side of a bottle; a tubulation.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tucan</ent><br/
<hw>Tu*can"</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Zool.)</fld> <def>The Mexican pocket gopher (<spn>Geomys Mexicanus</spn>). It resembles the common pocket gopher of the Western United States, but is larger. Called also <altname>tugan</altname>, and <altname>tuza</altname>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tucet</ent><br/
<hw>Tu"cet</hw> <pr>(?)</pr>, <pos>n.</pos> <def>See <er>Tucket</er>, a steak.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tuch</ent><br/
<hw>Tuch</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[See <er>Touchstone</er>.]</ety> <def>A dark-colored kind of marble; touchstone.</def> <mark>[Obs.]</mark>  <rj><au>Sir J. Harrington.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tuck</ent><br/
<hw>Tuck</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F. <ets>estoc</ets>; cf. It. <ets>stocco</ets>; both of German origin, and akin to E. <ets>stock</ets>. See <er>Stock</er>.]</ety> <def>A long, narrow sword; a rapier.</def> <mark>[Obs.]</mark>  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>He wore large hose, and a <qex>tuck</qex>, as it was then called, or rapier, of tremendous length.</q> <rj><qau>Sir W. Scot.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tuck</ent><br/
<hw>Tuck</hw>, <pos>n.</pos> <ety>[Cf. <er>Tocsin</er>.]</ety> <def>The beat of a drum.</def>  <rj><au>Scot.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tuck</ent><br/
<hw>Tuck</hw>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Tucked</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Tucking</conjf>.]</vmorph> <ety>[OE. <ets>tukken</ets>, LG. <ets>tukken</ets> to pull up, tuck up, entice; akin to OD. <ets>tocken</ets> to entice, G. <ets>zucken</ets> to draw with a short and quick motion, and E. <ets>tug</ets>. See <er>Tug</er>.]</ety> <sn>1.</sn> <def>To draw up; to shorten; to fold under; to press into a narrower compass; <as>as, to <ex>tuck</ex> the bedclothes in; to <ex>tuck</ex> up one's sleeves.</as></def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To make a tuck or tucks in; <as>as, to <ex>tuck</ex> a dress</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>To inclose; to put within; to press into a close place; <as>as, to <ex>tuck</ex> a child into a bed; to <ex>tuck</ex> a book under one's arm, or into a pocket.</as></def><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <ety>[Perhaps originally, to strike, beat: cf. F. <ets>toquer</ets> to touch.  Cf. <er>Tocsin</er>.]</ety> <def>To full, as cloth.</def> <mark>[Prov. Eng.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tuck</ent><br/
<hw>Tuck</hw>, <pos>v. i.</pos> <def>To contract; to draw together.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tuck</ent><br/
<hw>Tuck</hw>, <pos>n.</pos> <sn>1.</sn> <def>A horizontal sewed fold, such as is made in a garment, to shorten it; a plait.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A small net used for taking fish from a larger one; -- called also <altname>tuck-net</altname>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>A pull; a lugging.</def> <mark>[Obs.]</mark> <see>See <er>Tug</er>.</see>  <rj><au>Life of A. Wood.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <fld>(Naut.)</fld> <def>The part of a vessel where the ends of the bottom planks meet under the stern.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <def>Food; pastry; sweetmeats.</def> <mark>[Slang]</mark>  <rj><au>T. Hughes.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tuckahoe</ent><br/
<hw>Tuck"a*hoe</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[North American Indian, bread.]</ety> <fld>(Bot.)</fld> <def>A curious vegetable production of the Southern Atlantic United States, growing under ground like a truffle and often attaining immense size. The real nature is unknown. Called also <altname>Indian bread</altname>, and <altname>Indian loaf</altname>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tucker</ent><br/
<hw>Tuck"er</hw> <pr>(?)</pr>, <pos>n.</pos> <sn>1.</sn> <def>One who, or that which, tucks; specifically, an instrument with which tuck are made.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A narrow piece of linen or the like, folded across the breast, or attached to the gown at the neck, forming a part of a woman's dress in the 17th century and later.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <ety>[See <er>Tuck</er>, <pos>v. t.</pos>, 4.]</ety> <def>A fuller.</def> <mark>[Prov. Eng.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <ety>[Cf. <er>Tuck</er>, <pos>n.</pos>, 5.]</ety> <def>Daily food; meals; also, food in general.</def> <mark>[Slang or Colloq.]</mark></p>

<p><q>Tobacco, matches, and <qex>tucker</qex>, the latter comprising almost anything within the province of food.</q>  <rj><qau>C. L. Money.</qau></rj><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Tucker</ent><br/
<hw>Tuck"er</hw>, <pos>v. t.</pos> <def>To tire; to weary; -- usually with <ptcl>out</ptcl>.</def> <mark>[Colloq. U. S.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tucket</ent><br/
<hw>Tuck"et</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[It <ets>toccata</ets> a prelude, fr. <ets>toccare</ets> to touch. See <er>Toccata</er>, <er>Touch</er>.]</ety> <def>A slight flourish on a trumpet; a fanfare.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Tucket sonance</b></col>, <cd>the sound of the tucket.</cd> <mark>[Obs.]</mark></cs><br/
[<source>1913 Webster</source>]</p>

<p><q>Let the trumpets sound<br/
The <qex>tucket sonance</qex> and the note to mount.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tucket</ent><br/
<hw>Tuck"et</hw>, <pos>n.</pos> <ety>[Cf. It. <ets>tocchetto</ets> a ragout of fish, meat, fr. <ets>tocco</ets> a bit, morsel, LL. <ets>tucetum</ets>, <ets>tuccetum</ets>, a thick gravy.]</ety> <def>A steak; a collop.</def> <mark>[Obs.]</mark>  <rj><au>Jer. Taylor.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tuck-net</ent><br/
<hw>Tuck"-net`</hw> <pr>(?)</pr>, <pos>n.</pos> <def>See <er>Tuck</er>, <pos>n.</pos>, 2.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tuck pointing</ent><br/
<hw>Tuck pointing</hw>. <fld>(Masonry)</fld> <def>The finishing of joints along the center lines with a narrow ridge of putty or fine lime mortar.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Tucum</ent><br/
||<hw>Tu"cum</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[So called by the Indians of Brazil.]</ety> <def>A fine, strong fiber obtained from the young leaves of a Brazilian palm (<spn>Astrocaryum vulgare</spn>), used for cordage, bowstrings, etc.; also, the plant yielding this fiber. Called also <altname>tecum</altname>, and <altname>tecum fiber</altname>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tucuma</ent><br/
||<hw>Tu*cu"ma</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Bot.)</fld> <def>A Brazilian palm (<spn>Astrocaryum Tucuma</spn>) which furnishes an edible fruit.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tudor</ent><br/
<hw>Tu"dor</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Of or pertaining to a royal line of England, descended from Owen <ets>Tudor</ets> of Wales, who married the widowed queen of Henry V. The first reigning Tudor was Henry VII.; the last, Elizabeth.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Tudor style</b></col> <fld>(Arch.)</fld>, <cd>the latest development of Gothic architecture in England, under the Tudors, characterized by flat four-centered arches, shallow moldings, and a profusion of paneling on the walls.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tue</ent><br/
<hw>Tu"e</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Zool.)</fld> <def>The parson bird.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tuefall</ent><br/
<hw>Tue"fall</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Arch.)</fld> <def>See <er>To-fall</er>.</def> <mark>[Eng.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tue-iron</ent><br/
<hw>Tue"-i`ron</hw> <pr>(?)</pr>, <pos>n.</pos> <def>See <er>Tuy<egrave/re</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tue-irons</ent><br/
<hw>Tue"-i`rons</hw>, <pos>n. pl.</pos> <def>A pair of blacksmith's tongs.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tuesday</ent><br/
<hw>Tues"day</hw> <pr>(t<umac/z"d<asl/; 48)</pr>, <pos>n.</pos> <ety>[OE. <ets>Tewesday</ets>, AS. <ets>Tiwes d<ae/g</ets> the day of <ets>Tiw</ets> the god of war; akin to OHG. <ets>Zio</ets>, Icel. <ets>T<ymac/r</ets>, L. <ets>Ju</ets>piter, Gr. <grk>Zey`s</grk>;, cf. OHG. <ets>Ziostac</ets> Tuesday, G. <ets>Dienstag</ets>, Icel. <ets>T<ymac/sdagr</ets>. <root/244.  See <er>Deity</er>, <er>Day</er>, and cf. <er>Jovial</er>.]</ety> <def>The third day of the week, following Monday and preceding Wednesday.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tuet</ent><br/
<hw>Tu"et</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Zool.)</fld> <def>The lapwing.</def> <mark>[Prov. Eng.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tufa</ent><br/
<hw>Tu"fa</hw> <pr>(?)</pr>, <ety>[It. <ets>fufo</ets> soft, sandy stone, L. <ets>tofus</ets>, <ets>tophus</ets>.  Cf. <er>Tofus</er>, <er>Toph</er>, and <er>Tophin</er>.]</ety> <fld>(Min.)</fld> <sd>(a)</sd> <def>A soft or porous stone formed by depositions from water, usually calcareous; -- called also <altname>calcareous tufa</altname>.</def> <sd>(b)</sd> <def>A friable volcanic rock or conglomerate, formed of consolidated cinders, or scoria.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tufaceous</ent><br/
<hw>Tu*fa"ceous</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[Cf. It. <ets>tufaceo</ets>, L. <ets>tofaceus</ets>, <ets>tofacius</ets>. See <er>Tufa</er>.]</ety> <fld>(Min.)</fld> <def>Pertaining to tufa; consisting of, or resembling, tufa.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tuff</ent><br/
<hw>Tuff</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Min.)</fld> <def>Same as <er>Tufa</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tuffoon</ent><br/
<hw>Tuf*foon"</hw> <pr>(?)</pr>, <pos>n.</pos> <def>See <er>Typhoon</er>.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tuft</ent><br/
<hw>Tuft</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Prov. E. <ets>tuff</ets>, F. <ets>touffe</ets>; of German origin; cf. G. <ets>zopf</ets> a weft of hair, pigtail, top of a tree. See <er>Top</er> summit.]</ety> <sn>1.</sn> <def>A collection of small, flexible, or soft things in a knot or bunch; a waving or bending and spreading cluster; <as>as, a <ex>tuft</ex> of flowers or feathers</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A cluster; a clump; <as>as, a <ex>tuft</ex> of plants</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Under a <qex>tuft</qex> of shade.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Green lake, and cedar <qex>fuft</qex>, and spicy glade.</q> <rj><qau>Keble.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>A nobleman, or person of quality, especially in the English universities; -- so called from the tuft, or gold tassel, on the cap worn by them.</def> <mark>[Cant, Eng.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>Several young <qex>tufts</qex>, and others of the faster men.</q> <rj><qau>T. Hughes.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tuft</ent><br/
<hw>Tuft</hw>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Tufted</conjf>; <pos>p. pr. & vb. n.</pos> <conjf>Tufting</conjf>.]</vmorph> <sn>1.</sn> <def>To separate into tufts.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To adorn with tufts or with a tuft.</def>  <rj><au>Thomson.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tuft</ent><br/
<hw>Tuft</hw>, <pos>v. i.</pos> <def>To grow in, or form, a tuft or tufts.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tuf-taffeta</ent><br/
<hw>Tuf-taf"fe*ta</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A silk fabric formerly in use, having a nap or pile.</def> <altsp>[Written also <asp>tuft-taffeta</asp>.]</altsp><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tufted</ent><br/
<hw>Tuft"ed</hw> <pr>(?)</pr>, <pos>a.</pos> <sn>1.</sn> <def>Adorned with a tuft; <as>as, the <ex>tufted</ex> duck</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Growing in tufts or clusters; tufty.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The <qex>tufted</qex> crowtoe, and pale jessamine.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q><qex>Tufted</qex> trees and springing corn.</q> <rj><qau>Pope.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Tufted duck</b></col> <fld>(Zool.)</fld>, <cd>the ring-necked duck.</cd> <mark>[Local, U. S.]</mark></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tufthunter</ent><br/
<hw>Tuft"hunt`er</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A hanger-on to noblemen, or persons of quality, especially in English universities; a toady. See 1st <er>Tuft</er>, 3.</def> <mark>[Cant, Eng.]</mark>  <rj><au>Halliwell.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tufthunting</ent><br/
<hw>Tuft"hunt`ing</hw>, <pos>n.</pos> <def>The practice of seeking after, and hanging on, noblemen, or persons of quality, especially in English universities.</def> <mark>[Cant, Eng.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tufty</ent><br/
<hw>Tuft"y</hw> <pr>(?)</pr>, <pos>a.</pos> <sn>1.</sn> <def>Abounding with tufts.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Both in the <qex>tufty</qex> frith and in the mossy fell.</q> <rj><qau>Drayton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Growing in tufts or clusters.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Where <qex>tufty</qex> daisies nod at every gale.</q> <rj><qau>W. Browne.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tug</ent><br/
<hw>Tug</hw> <pr>(?)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Tugged</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Tugging</conjf>.]</vmorph> <ety>[OE. <ets>toggen</ets>; akin to OD. <ets>tocken</ets> to entice, G. <ets>zucken</ets> to jerk, draw, Icel. <ets>toga</ets> to draw, AS. <ets>t<eacute/on</ets>, p. p. <ets>togen</ets>, to draw, G. <ets>ziehen</ets>, OHG. <ets>ziohan</ets>, Goth. <ets>tiuhan</ets>, L. <ets>ducere</ets> to lead, draw.  Cf. <er>Duke</er>, <er>Team</er>, <er>Tie</er>, <pos>v. t.</pos>, <er>Touch</er>, <er>Tow</er>, <pos>v. t.</pos>, <er>Tuck</er> to press in, <er>Toy</er> a plaything.]</ety> <sn>1.</sn> <def>To pull or draw with great effort; to draw along with continued exertion; to haul along; to tow; <as>as, to <ex>tug</ex> a loaded cart; to <ex>tug</ex> a ship into port.</as></def><br/
[<source>1913 Webster</source>]</p>

<p><q>There sweat, there strain, <qex>tug</qex> the laborious oar.</q> <rj><qau>Roscommon.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To pull; to pluck.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>To ease the pain,<br/
His <qex>tugged</qex> cars suffered with a strain.</q> <rj><qau>Hudibras.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tug</ent><br/
<hw>Tug</hw>, <pos>v. i.</pos> <sn>1.</sn> <def>To pull with great effort; to strain in labor; <as>as, to <ex>tug</ex> at the oar; to <ex>tug</ex> against the stream</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>He <qex>tugged</qex>, he shook, till down they came.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To labor; to strive; to struggle.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>England now is left<br/
To <qex>tug</qex> and scamble and to part by the teeth<br/
The unowed interest of proud-swelling state.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tug</ent><br/
<hw>Tug</hw>, <pos>n.</pos> <sn>1.</sn> <def>A pull with the utmost effort, as in the athletic contest called <xex>tug of war</xex>; a supreme effort.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>At the <qex>tug</qex> he falls,<br/
Vast ruins come along, rent from the smoking walls.</q> <rj><qau>Dryden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A sort of vehicle, used for conveying timber and heavy articles.</def> <mark>[Prov. Eng.]</mark>  <rj><au>Halliwell.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <fld>(Naut.)</fld> <def>A small, powerful steamboat used to tow vessels; -- called also <altname>steam tug</altname>, <altname>tugboat</altname>, and <altname>towboat</altname>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>A trace, or drawing strap, of a harness.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <fld>(Mining.)</fld> <def>An iron hook of a hoisting tub, to which a tackle is affixed.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Tug iron</b></col>, <cd>an iron hook or button to which a tug or trace may be attached, as on the shaft of a wagon.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tugan</ent><br/
<hw>Tu*gan"</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Zool.)</fld> <def>Same as <er>Tucan</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tugboat</ent><br/
<hw>Tug"boat`</hw> <pr>(?)</pr>, <pos>n.</pos> <def>See <er>Tug</er>, <pos>n.</pos>, 3.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tugger</ent><br/
<hw>Tug"ger</hw> <pr>(?)</pr>, <pos>n.</pos> <def>One who tugs.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tuggingly</ent><br/
<hw>Tug"ging*ly</hw> <pr>(?)</pr>, <pos>adv.</pos> <def>In a tugging manner; with laborious pulling.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tuille</ent><br/
<hw>Tuille</hw> <pr>(tw<icr/l)</pr>, <pos>n.</pos> <ety>[Cf. F. <ets>tuile</ets> a tile.]</ety> <def>In plate armor, a suspended plate in from of the thigh. See <xex>Illust.</xex> of <er>Tasses</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tuition</ent><br/
<hw>Tu*i"tion</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>tuitio</ets> protection, guarding, from <ets>tueri</ets>, p. p. <ets>tuitus</ets>, to see, watch, protect: cf. F. <ets>tuition</ets>.  Cf. <er>Tutor</er>.]</ety> <sn>1.</sn> <def>Superintending care over a young person; the particular watch and care of a tutor or guardian over his pupil or ward; guardianship.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Especially, the act, art, or business of teaching; instruction; <as>as, children are sent to school for <ex>tuition</ex>; his <ex>tuition</ex> was thorough.</as></def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>The money paid for instruction; the price or payment for instruction; <as>as, <ex>tuition</ex> must be paid in full before graduation</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tuitionary</ent><br/
<hw>Tu*i"tion*a*ry</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Of or pertaining to tuition.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tuko-tuko</ent><br/
<hw>Tu"ko-tu*ko</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[From the native name .]</ety> <fld>(Zool.)</fld> <def>A burrowing South American rodent (<spn>Ctenomys Braziliensis</spn>). It has small eyes and ears and a short tail. It resembles the pocket gopher in size, form, and habits, but is more nearly allied to the porcupines.</def> <altsp>[Written also <asp>tucu-tuco</asp>.]</altsp><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tula metal</ent><br/
<hw>Tu"la met`al</hw> <pr>(?)</pr>. <def>An alloy of silver, copper, and lead made at <ets>Tula</ets> in Russia.</def> <altsp>[Written also <asp>toola metal</asp>.]</altsp><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tule</ent><br/
<hw>Tu"le</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Mex.]</ety> <fld>(Bot.)</fld> <def>A large bulrush (<spn>Scirpus lacustris</spn>, and <spn>Scirpus Tatora</spn>) growing abundantly on overflowed land in California and elsewhere.</def><br/
[<source>1913 Webster</source>]</p>

<p><-- p. 1550 pr=PI --><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tulip</ent><br/
<hw>Tu"lip</hw> <pr>(t<umac/"l<icr/p)</pr>, <pos>n.</pos> <ety>[F. <ets>tulipe</ets>, OF. also <ets>tulipan</ets>, It. <ets>tulipano</ets>, <ets>tulipa</ets>, from Turk. <ets>tulbend</ets>, <ets>dulbend</ets>, literally, a turban, Per. <ets>dulband</ets>; -- so called from the resemblance of the form of this flower to a turban. See <er>Turban</er>.]</ety> <fld>(Bot.)</fld> <def>Any plant of the liliaceous genus <gen>Tulipa</gen>.  Many varieties are cultivated for their beautiful, often variegated flowers.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Tulip tree</b></col>. <sd>(a)</sd> <cd>A large American tree (<spn>Liriodendron tulipifera</spn>) of the magnolia family (<fam>Magnoliaceae</fam>) bearing tuliplike flowers. See <er>Liriodendron</er>.</cd> <sd>(b)</sd> <cd>A West Indian malvaceous tree (<spn>Paritium tiliaceum</spn> syn. <spn>Hibiscus tiliaceum</spn>).</cd></cs><br/
[<source>1913 Webster</source> + <source>PJC</source>]</p>

<p><ent>Tulip-eared</ent><br/
<hw>Tu"lip-eared`</hw> <pr>(?)</pr>, <pos>a.</pos> <fld>(Zool.)</fld> <def>Having erect, pointed ears; prick-eared; -- said of certain dogs.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tulipist</ent><br/
<hw>Tu"lip*ist</hw>, <pos>n.</pos> <def>A person who is especially devoted to the cultivation of tulips.</def>  <rj><au>Sir T. Browne.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tulipomania</ent><br/
<hw>Tu`lip*o*ma"ni*a</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[<ets>Tulip</ets> + <ets>mania</ets>.]</ety> <def>A violent passion for the acquisition or cultivation of tulips; -- a word said by Beckman to have been coined by Menage.</def><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ In Holland, in the first half of the 17th century, the cultivation of tulips became a mania.  It began about the year 1634, and, like a violent epidemic, seized upon all classes of the community, leading to disasters and misery such as the records of commerce or of bankruptcies can scarcely parallel.  In 1636, tulip marts had been established in Amsterdam, Rotterdam, Haarlem, Leyden, and various other towns, where tulip bulbs were sold and resold in the same manner as stocks are on the Stock Exchange of London.</note>  <rj><au>Baird.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tulipomaniac</ent><br/
<hw>Tu`lip*o*ma"ni*ac</hw> <pr>(?)</pr>, <pos>n.</pos> <def>One who is affected with tulipomania.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tulip-shell</ent><br/
<hw>Tu"lip-shell`</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Zool.)</fld> <def>A large, handsomely colored, marine univalve shell (<spn>Fasciolaria tulipa</spn>) native of the Southern United States.  The name is sometimes applied also to other species of <gen>Fasciolaria</gen>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tulipwood</ent><br/
<hw>Tu"lip*wood`</hw> <pr>(?)</pr>, <pos>n.</pos> <def>The beautiful rose-colored striped wood of a Brazilian tree (<spn>Physocalymna floribunda</spn>), much used by cabinetmakers for inlaying.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Queensland tulipwood</b></col>, <cd>the variegated wood of an Australian sapindaceous tree (<spn>Harpullia pendula</spn>).</cd>  <rj><au>J. Smith (Dict. Econ. Plants).</au></rj></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tull</ent><br/
<hw>Tull</hw> <pr>(?)</pr>, <pos>v. t.</pos> <ety>[OE. <ets>tullen</ets>. See <er>Tole</er>.]</ety> <def>To allure; to tole.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>With empty hands men may no hawkes <qex>tull</qex>.</q> <rj><qau>Chaucer.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tulle</ent><br/
<hw>Tulle</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F.; -- so called from the town of <ets>Tulle</ets>, in France.]</ety> <def>A kind of silk lace or light netting, used for veils, etc.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tullian</ent><br/
<hw>Tul"li*an</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>Tullianus</ets>, from <etsep>Tullius</etsep>, the name of a Roman gens.]</ety> <def>Belonging to, or in the style of, <persfn>Tully</persfn> (<person>Marcus Tullius Cicero</person>).</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tullibee</ent><br/
<hw>Tul"li*bee</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Zool.)</fld> <def>A whitefish (<spn>Coregonus tullibee</spn>) found in the Great Lakes of North America; -- called also <altname>mongrel whitefish</altname>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tumble</ent><br/
<hw>Tum"ble</hw> <pr>(t<ucr/m"b'l)</pr>, <pos>v. i.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Tumbled</conjf> <pr>(t<ucr/m"b'ld)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Tumbling</conjf> <pr>(t<ucr/m"bl<icr/ng)</pr>.]</vmorph> <ety>[OE. <ets>tumblen</ets>, AS. <ets>tumbian</ets> to turn heels over head, to dance violently; akin to D. <ets>tuimelen</ets> to fall, Sw. <ets>tumla</ets>, Dan. <ets>tumle</ets>, Icel. <ets>tumba</ets>; and cf. G.  <ets>taumeln</ets> to reel, to stagger.]</ety> <sn>1.</sn> <def>To roll over, or to and fro; to throw one's self about; <as>as, a person in pain <ex>tumbles</ex> and tosses</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To roll down; to fall suddenly and violently; to be precipitated; <as>as, to <ex>tumble</ex> from a scaffold</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>He who <qex>tumbles</qex> from a tower surely has a greater blow than he who slides from a molehill.</q> <rj><qau>South.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>To play tricks by various movements and contortions of the body; to perform the feats of an acrobat.</def>  <rj><au>Rowe.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>To tumble home</b></col> <fld>(Naut.)</fld>, <cd>to incline inward, as the sides of a vessel, above the bends or extreme breadth; -- used esp. in the phrase <xex>tumbling home</xex>.  Cf. <er>Wall-sided</er>.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tumble</ent><br/
<hw>Tum"ble</hw>, <pos>v. t.</pos> <sn>1.</sn> <def>To turn over; to turn or throw about, as for examination or search; to roll or move in a rough, coarse, or unceremonious manner; to throw down or headlong; to precipitate; -- sometimes with <xex>over</xex>, <xex>about</xex>, etc.; <as>as, to <ex>tumble</ex> books or papers</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To disturb; to rumple; <as>as, to <ex>tumble</ex> a bed</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tumble</ent><br/
<hw>Tum"ble</hw>, <pos>n.</pos> <def>Act of tumbling, or rolling over; a fall.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tumblebug</ent><br/
<hw>Tum"ble*bug`</hw> <pr>(?)</pr>, <pos>n.</pos> <def>See <er>Tumbledung</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tumble-down</ent><br/
<hw>Tum"ble-down`</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Ready to fall; dilapidated; ruinous; <as>as, a <ex>tumble-down</ex> house</as>.</def> <mark>[Colloq.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tumbledung</ent><br/
<hw>Tum"ble*dung`</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Zool.)</fld> <def>Any one of numerous species of scaraboid beetles belonging to <gen>Scarabaeus</gen>, <gen>Copris</gen>, <gen>Phanaeus</gen>, and allied genera. The female lays her eggs in a globular mass of dung which she rolls by means of her hind legs to a burrow excavated in the earth in which she buries it.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tumbler</ent><br/
<hw>Tum"bler</hw> <pr>(?)</pr>, <pos>n.</pos> <sn>1.</sn> <def>One who tumbles; one who plays tricks by various motions of the body; an acrobat.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A movable obstruction in a lock, consisting of a lever, latch, wheel, slide, or the like, which must be adjusted to a particular position by a key or other means before the bolt can be thrown in locking or unlocking.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <fld>(Firearms)</fld> <def>A piece attached to, or forming part of, the hammer of a gunlock, upon which the mainspring acts and in which are the notches for the sear point to enter.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>A drinking glass, without a foot or stem; -- so called because originally it had a pointed or convex base, and could not be set down with any liquor in it, thus compelling the drinker to finish his measure.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <fld>(Zool.)</fld> <def>A variety of the domestic pigeon remarkable for its habit of tumbling, or turning somersaults, during its flight.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>6.</sn> <fld>(Zool.)</fld> <def>A breed of dogs that tumble when pursuing game. They were formerly used in hunting rabbits.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>7.</sn> <def>A kind of cart; a tumbrel.</def> <mark>[Prov. Eng. & Scot.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tumblerful</ent><br/
<hw>Tum"bler*ful</hw> <pr>(?)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Tumblerfuls</plw> <pr>(<?/)</pr>.</plu> <def>As much as a tumbler will hold; enough to fill a tumbler.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tumbleweed</ent><br/
<hw>Tum"ble*weed`</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Bot.)</fld> <def>Any plant which habitually breaks away from its roots in the autumn, and is driven by the wind, as a light, rolling mass, over the fields and prairies; such as witch grass, wild indigo, <spn>Amarantus albus</spn>, etc.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tumbling</ent><br/
<hw>Tum"bling</hw> <pr>(?)</pr>, <def><pos>a. & vb. n.</pos> from <er>Tumble</er>, <pos>v.</pos></def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Tumbling barrel</b></col>. <cd>Same as <er>Rumble</er>, <pos>n.</pos>, 4.</cd> -- <col><b>Tumbling bay</b></col>, <cd>an overfall, or weir, in a canal.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tumbril</ent><br/
<ent>Tumbrel</ent><br/
<mhw>{ <hw>Tum"brel</hw> <pr>(?)</pr>, <hw>Tum"bril</hw> <pr>(?)</pr>, }</mhw> <pos>n.</pos> <ety>[OF. <ets>tomberel</ets>, F. <ets>tombereau</ets>, fr. <ets>tomber</ets> to fall, to tumble; of Teutonic origin.  Cf. <er>Tumble</er>.]</ety> <sn>1.</sn> <def>A cucking stool for the punishment of scolds.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A rough cart.</def>  <rj><au>Tusser.</au>  <au>Tatler.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <fld>(Mil.)</fld> <def>A cart or carriage with two wheels, which accompanies troops or artillery, to convey the tools of pioneers, cartridges, and the like.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>A kind of basket or cage of osiers, willows, or the like, to hold hay and other food for sheep.</def> <mark>[Eng.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tumefacient</ent><br/
<hw>Tu`me*fa"cient</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>tumefaciens</ets>, <ets>-entis</ets>, p. pr. of <ets>tumefacere</ets> to tumefy; <ets>tumere</ets> to swell + <ets>facere</ets> to make.]</ety> <def>Producing swelling; tumefying.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Tumefaction</ent><br/
<hw>Tu`me*fac"tion</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Cf. F. <ets>tum<eacute/faction</ets>.]</ety> <def>The act or process of tumefying, swelling, or rising into a tumor; a tumor; a swelling.</def>  <rj><au>Arbuthnot.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tumefy</ent><br/
<hw>Tu"me*fy</hw> <pr>(t<umac/"m<esl/*f<imac/)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Tumefied</conjf> <pr>(t<umac/"m<esl/*f<imac/d)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Tumefying</conjf>.]</vmorph> <ety>[F. <ets>tum<eacute/fier</ets>, fr. L. <ets>tumere</ets> to swell + <ets>-ficare</ets> (in comp.) to make; cf. L. <ets>tumefacere</ets> to tumefy. See <er>Tumid</er>, and <er>-fy</er>.]</ety> <def>To swell; to cause to swell, or puff up.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>To swell, <qex>tumefy</qex>, stiffen, not the diction only, but the tenor of the thought.</q> <rj><qau>De Quincey.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tumefy</ent><br/
<hw>Tu"me*fy</hw>, <pos>v. i.</pos> <def>To rise in a tumor; to swell.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tumescence</ent><br/
<hw>Tu*mes"cence</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>tumescens</ets>, <ets>-entis</ets>, p.pr. of <ets>tumescere</ets> to swell up, v. incho. fr. <ets>tumere</ets> to swell.]</ety> <def>The act of becoming tumid; the state of being swollen; intumescence; -- applied especially to the state of swelling of the vascular tissue in the male and female sex organs when they have been stimulated to readiness for sexual intercourse.</def><br/
[<source>Webster 1913 Suppl.</source> <source>+PJC</source>]</p>

<p><ent>Tumescent</ent><br/
<hw>Tu*mes"cent</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Slightly tumid; swollen, as certain moss capsules.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Tumid</ent><br/
<hw>Tu"mid</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>tumidus</ets>, fr.  <ets>tumere</ets> to swell; cf. Skr. <ets>tumra</ets> strong, fat.  Cf. <er>Thumb</er>.]</ety> <sn>1.</sn> <def>Swelled, enlarged, or distended; <as>as, a <ex>tumid</ex> leg; <ex>tumid</ex> flesh.</as></def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Rising above the level; protuberant.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>So high as heaved the <qex>tumid</qex> hills.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>Swelling in sound or sense; pompous; puffy; inflated; bombastic; falsely sublime; turgid; <as>as, a <ex>tumid</ex> expression; a <ex>tumid</ex> style</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p>-- <wordforms><wf>Tu"mid*ly</wf>, <pos>adv.</pos> -- <wf>Tu"mid*ness</wf>, <pos>n.</pos></wordforms><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tumidity</ent><br/
<hw>Tu*mid"i*ty</hw> <pr>(?)</pr>, <pos>n.</pos> <def>The quality or state of being tumid.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tummals</ent><br/
<hw>Tum"mals</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Mining)</fld> <def>A great quantity or heap.</def>  <rj><au>Weale.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tumor</ent><br/
<hw>Tu"mor</hw> <pr>(t<umac/"m<etil/r)</pr>, <pos>n.</pos> <ety>[L., fr. <ets>tumere</ets> to swell: cf. F. <ets>tume<uacute/r</ets>. See <er>Tumid</er>.]</ety> <sn>1.</sn> <fld>(Med.)</fld> <def>A morbid swelling, prominence, or growth, on any part of the body; especially, a growth produced by deposition of new tissue; a neoplasm.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Affected pomp; bombast; swelling words or expressions; false magnificence or sublimity.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>Better, however, to be a flippant, than, by a revolting form of <qex>tumor</qex> and perplexity, to lead men into habits of intellect such as result from the modern vice of English style.</q> <rj><qau>De Quincey.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Encysted tumor</b></col>, <cd>a tumor which is inclosed in a membrane called a <xex>cyst</xex>, connected with the surrounding parts by the neighboring cellular substance.</cd> -- <col><b>Fatty tumor</b></col>. <cd>See under <er>Fatty</er>.</cd> -- <mcol><col><b>Innocent tumor</b></col>, <it>or</it> <col><b>Benign tumor</b></col></mcol>, <cd>one which does not of itself threaten life, and does not usually tend to recur after extirpation; a tumor which has not metastesized.</cd> -- <col><b>Malignant tumor</b></col>, <cd>a tumor which tends continually to spread, to become generalized in different parts of the body, and to recur after extirpation, and which, if left to itself, causes death.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tumored</ent><br/
<hw>Tu"mored</hw> <pr>(t<umac/"m<etil/rd)</pr>, <pos>a.</pos> <def>Distended; swelled.</def> <mark>[R.]</mark> <ldquo/His <xex>tumored</xex> breast.<rdquo/  <rj><au>R. Junius.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tumorous</ent><br/
<hw>Tu"mor*ous</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>tumorosus</ets> inflated.]</ety>  <sn>1.</sn> <def>Swelling; protuberant.</def> <mark>[R.]</mark>  <rj><au>Sir H. Wotton.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Inflated; bombastic.</def> <mark>[R.]</mark>  <rj><au>B. Jonson.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tump</ent><br/
<hw>Tump</hw> <pr>(t<ucr/mp)</pr>, <pos>n.</pos> <ety>[W. <ets>twmp</ets>, <ets>twm</ets>, a round mass or heap, a hillock.]</ety> <def>A little hillock; a knoll.</def>  <rj><au>Ainsworth.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tump</ent><br/
<hw>Tump</hw>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Tumped</conjf> <pr>(t<ucr/mt; 215)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Tumping</conjf>.]</vmorph> <sn>1.</sn> <def>To form a mass of earth or a hillock about; <as>as, to <ex>tump</ex> teasel</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To draw or drag, as a deer or other animal after it has been killed.</def> <mark>[Local, U. S.]</mark>  <rj><au>Bartlett.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tumpline</ent><br/
<hw>Tump"line`</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A strap placed across a man's forehead to assist him in carrying a pack on his back.</def> <mark>[Local, U. S.]</mark>  <rj><au>Bartlett.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tum-tum</ent><br/
<hw>Tum"-tum`</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A dish made in the West Indies by beating boiled plantain quite soft in a wooden mortar.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tumular</ent><br/
<hw>Tu"mu*lar</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>tumulus</ets> a mound: cf. F. <ets>tumulaire</ets>. See <er>Tumulus</er>.]</ety> <def>Consisting in a heap; formed or being in a heap or hillock.</def>  <rj><au>Pinkerton.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tumulate</ent><br/
<hw>Tu"mu*late</hw> <pr>(?)</pr>, <pos>v. t.</pos> <ety>[L. <ets>tumulatus</ets>, p. p. of <ets>tumulare</ets> to tumulate. See <er>Tumulus</er>.]</ety> <def>To cover, as a corpse, with a mound or tomb; to bury.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tumulate</ent><br/
<hw>Tu"mu*late</hw>, <pos>v. i.</pos> <def>To swell.</def> <mark>[Obs.]</mark>  <rj><au>Wilkins.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tumulose</ent><br/
<hw>Tu"mu*lose`</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Tumulous.</def> <mark>[R.]</mark>  <rj><au>Bailey.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tumulosity</ent><br/
<hw>Tu`mu*los"i*ty</hw> <pr>(?)</pr>, <pos>n.</pos> <def>The quality or state of being tumulous; hilliness.</def> <mark>[R.]</mark>  <rj><au>Bailey.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tumulous</ent><br/
<hw>Tu"mu*lous</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>tumulosus</ets>, fr.  <ets>tumulus</ets> a mound.]</ety> <def>Full of small hills or mounds; hilly; tumulose.</def> <mark>[R.]</mark>  <rj><au>Bailey.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tumult</ent><br/
<hw>Tu"mult</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>tumultus</ets>; probably akin to Skr. <ets>tumula</ets> noise, noisy, and perhaps to L. <ets>tumere</ets> to swell, E. <ets>tumid</ets>: cf. F. <ets>tumulte</ets>.]</ety> <sn>1.</sn> <def>The commotion or agitation of a multitude, usually accompanied with great noise, uproar, and confusion of voices; hurly-burly; noisy confusion.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>What meaneth the noise of this <qex>tumult</qex> ?</q> <rj><qau>1 Sam. iv. 14.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Till in loud <qex>tumult</qex> all the Greeks arose.</q> <rj><qau>Pope.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Violent commotion or agitation, with confusion of sounds; <as>as, the <ex>tumult</ex> of the elements</as>.</def>  <rj><au>Addison.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>Irregular or confused motion; agitation; high excitement; <as>as, the <ex>tumult</ex> of the spirits or passions</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><syn><b>Syn.</b> -- Uproar; ferment; disturbance; turbulence; disorder; confusion; noise; bluster; hubbub; bustle; stir; brawl; riot.</syn><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tumult</ent><br/
<hw>Tu"mult</hw> <pr>(?)</pr>, <pos>v. i.</pos> <def>To make a tumult; to be in great commotion.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>Importuning and <qex>tumulting</qex> even to the fear of a revolt.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tumulter</ent><br/
<hw>Tu"mult*er</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A maker of tumults.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>He severely punished the <qex>tumulters</qex>.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tumultuarily</ent><br/
<hw>Tu*mul"tu*a*ri*ly</hw> <pr>(t<usl/*m<ucr/l"t<usl/*<asl/*r<icr/*l<ycr/)</pr>, <pos>adv.</pos> <def>In a tumultuary manner.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tumultuariness</ent><br/
<hw>Tu*mul"tu*a*ri*ness</hw>, <pos>n.</pos> <def>The quality or state of being tumultuary.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tumultuary</ent><br/
<hw>Tu*mul"tu*a*ry</hw> <pr>(t<usl/*m<ucr/l"t<usl/*<asl/*r<ycr/; 135)</pr>, <pos>a.</pos> <ety>[L. <ets>tumultuarius</ets>:  cf. F. <ets>tumultuaire</ets>.]</ety> <sn>1.</sn> <def>Attended by, or producing, a tumult; disorderly; promiscuous; confused; tumultuous.</def> <ldquo/A <xex>tumultuary</xex> conflict.<rdquo/  <rj><au>Eikon Basilike.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>A <qex>tumultuary</qex> attack of the Celtic peasantry.</q> <rj><qau>Macaulay.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Sudden flight or <qex>tumultuary</qex> skirmish.</q> <rj><qau>De Quincey.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Restless; agitated; unquiet.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Men who live without religion live always in a <qex>tumultuary</qex> and restless state.</q> <rj><qau>Atterbury.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tumultuate</ent><br/
<hw>Tu*mul"tu*ate</hw> <pr>(t<usl/*m<ucr/l"t<usl/*<amac/t)</pr>, <pos>v. i.</pos> <ety>[L. <ets>tumultuatus</ets>, p. p. of <ets>tumultuari</ets> to make a tumult.]</ety> <def>To make a tumult.</def> <mark>[Obs.]</mark> <ldquo/He will murmur and <xex>tumultuate</xex>.<rdquo/  <rj><au>South.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tumultuation</ent><br/
<hw>Tu*mul`tu*a"tion</hw> <pr>(t<usl/*m<ucr/l`t<usl/*<amac/"sh<ucr/n)</pr>, <pos>n.</pos> <ety>[L. <ets>tumultuatio</ets>.]</ety> <def>Irregular or disorderly movement; commotion; <as>as, the <ex>tumultuation</ex> of the parts of a fluid</as>.</def> <mark>[Obs.]</mark>  <rj><au>Boyle.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tumultuous</ent><br/
<hw>Tu*mul"tu*ous</hw> <pr>(t<usl/*m<ucr/l"t<usl/*<ucr/s)</pr>, <pos>a.</pos> <ety>[L. <ets>tumultuosus</ets>: cf. F.  <ets>tumultueux</ets>.]</ety> <sn>1.</sn> <def>Full of tumult; characterized by tumult; disorderly; turbulent.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The flight became wild and <qex>tumultuous</qex>.</q> <rj><qau>Macaulay.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Conducted with disorder; noisy; confused; boisterous; disorderly; <as>as, a <ex>tumultuous</ex> assembly or meeting</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>Agitated, as with conflicting passions; disturbed.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>His dire attempt, which, nigh the birth<br/
Now rolling, boils in his <qex>tumultuous</qex> breast.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>Turbulent; violent; <as>as, a <ex>tumultuous</ex> speech</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><syn><b>Syn.</b> -- Disorderly; irregular; noisy; confused; turbulent; violent; agitated; disturbed; boisterous; lawless; riotous; seditious.</syn><br/
[<source>1913 Webster</source>]</p>

<p>-- <wordforms><wf>Tu*mul"tu*ous*ly</wf>, <pos>adv.</pos> -- <wf>Tu*mul"tu*ous*ness</wf>, <pos>n.</pos></wordforms><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tumulus</ent><br/
||<hw>Tu"mu*lus</hw> <pr>(?)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Tumuli</plw> <pr>(#)</pr>.</plu> <ety>[L., a mound, a sepulchral mound, probably from <ets>tumere</ets> to swell.  Cf. <er>Tumid</er>.]</ety> <def>An artificial hillock, especially one raised over a grave, particularly over the graves of persons buried in ancient times; a barrow.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tun</ent><br/
<hw>Tun</hw> <pr>(t<ucr/n)</pr>, <pos>n.</pos> <ety>[AS. <ets>tunne</ets>. See <er>Ton</er> a weight.]</ety> <sn>1.</sn> <def>A large cask; an oblong vessel bulging in the middle, like a pipe or puncheon, and girt with hoops; a wine cask.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Brewing)</fld> <def>A fermenting vat.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>A certain measure for liquids, as for wine, equal to two pipes, four hogsheads, or 252 gallons.  In different countries, the tun differs in quantity.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <fld>(Com.)</fld> <def>A weight of 2,240 pounds. See <er>Ton</er>.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <def>An indefinite large quantity.</def>  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>A <qex>tun</qex> of man in thy large bulk is writ.</q> <rj><qau>Dryden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>6.</sn> <def>A drunkard; -- so called humorously, or in contempt.</def> <rj><qau>Dryden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>7.</sn> <fld>(Zool.)</fld> <def>Any shell belonging to <gen>Dolium</gen> and allied genera; -- called also <altname>tun-shell</altname>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tun</ent><br/
<hw>Tun</hw>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Tunned</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Tunning</conjf>.]</vmorph> <def>To put into tuns, or casks.</def>  <rj><au>Boyle.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tuna</ent><br/
<hw>Tu"na</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Bot.)</fld> <def>The <spn>Opuntia Tuna</spn>. See <cref>Prickly pear</cref>, under <er>Prickly</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tuna</ent><br/
<hw>Tu"na</hw>, <pos>n.</pos> <ety>[Cf. <er>Tunny</er>.]</ety> <sn>1.</sn> <fld>(Zool.)</fld> <def>Any one of several species of large oceanic fishes belonging to the mackerel family <fam>Scombridae</fam>, especially the <stype>bluefin tuna</stype> (<spn>Thunnus thynnus</spn>, formerly <spn>Orcynus thynnus</spn> <it>or</it>  <spn>Albacora thynnus</spn>), called also the <altname>common tunny</altname> or <altname>great tunny</altname>, a native of the Mediterranean Sea and of temperate parts of the Atlantic Ocean.  It sometimes weighs a thousand pounds or more, and is caught commercially in large quantity for use as food; -- also called, especially in Britain, <altname>tunny</altname>.  It is also one of the favorite fishes used by the Japanese in preparing sushi.  On the American coast, especially in New England, it is sometimes called the <altname>horse mackerel</altname>.  Another well-known species is the <stype>yellowfin tuna</stype> (<spn>Thunnus albacares</spn>) of warm seas. the See <xex>Illust.</xex> of <cref>Horse mackerel</cref>, under <er>Horse</er>.</def> <note>The little tunny (<spn>Gymnosarda alletterata</spn>) of the Mediterranean and North Atlantic, and the long-finned tunny, or albacore (<spn>Thunnus alalunga</spn>) (see <er>Albacore</er>), are related species of smaller size.</note><br/
[<source>1913 Webster</source> <source>+PJC</source>]</p>

<p><sn>2.</sn> <def>The bonito, 2.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>the meat of the tuna, used as food; -- also called <altname>tuna fish</altname>.</def><br/
[<source>PJC</source>]</p>

<p><ent>Tunable</ent><br/
<hw>Tun"a*ble</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Capable of being tuned, or made harmonious; hence, harmonious; musical; tuneful.</def> -- <wordforms><wf>Tun"a*ble*ness</wf>, <pos>n.</pos> -- <wf>Tun"a*bly</wf>, <pos>adv.</pos></wordforms><br/
[<source>1913 Webster</source>]</p>

<p><q>And <qex>tunable</qex> as sylvan pipe or song.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tun-bellied</ent><br/
<hw>Tun"-bel`lied</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Having a large, protuberant belly, or one shaped like a tun; pot-bellied.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tun-dish</ent><br/
<hw>Tun"-dish`</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A tunnel.</def> <mark>[Obs.]</mark>  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tundra</ent><br/
||<hw>Tun"dra</hw> <pr>(t<oomac/n"dr<adot/)</pr>, <pos>n.</pos> <ety>[Russ.]</ety> <def>One of the level or undulating treeless plains characteristic of northern arctic regions in both hemispheres; the term is most commony associated with the arctic plains of Siberia.  The tundras mark the limit of arborescent vegetation; they consist of black mucky soil with a permanently frozen subsoil, but support a dense growth of mosses and lichens, and dwarf herbs and shrubs, often showy-flowered.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Tune</ent><br/
<hw>Tune</hw> <pr>(t<umac/n)</pr>, <pos>n.</pos> <ety>[A variant of <ets>tone</ets>.]</ety> <sn>1.</sn> <def>A sound; a note; a tone.</def> <ldquo/The <xex>tune</xex> of your voices.<rdquo/  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Mus.)</fld> <sd>(a)</sd> <def>A rhythmical, melodious, symmetrical series of tones for one voice or instrument, or for any number of voices or instruments in unison, or two or more such series forming parts in harmony; a melody; an air; <as>as, a merry <ex>tune</ex>; a mournful <ex>tune</ex>; a slow <ex>tune</ex>; a psalm <ex>tune</ex>. See <er>Air</er>.</as></def> <sd>(b)</sd> <def>The state of giving the proper sound or sounds; just intonation; harmonious accordance; pitch of the voice or an instrument; adjustment of the parts of an instrument so as to harmonize with itself or with others; <as>as, the piano, or the organ, is not in <ex>tune</ex></as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Like sweet bells jangled, out of <qex>tune</qex> and harsh.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>Order; harmony; concord; fit disposition, temper, or humor; right mood.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>A child will learn three times as much when he is in <qex>tune</qex>, as when he . . . is dragged unwillingly to [his task].</q> <rj><qau>Locke.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tune</ent><br/
<hw>Tune</hw>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Tuned</conjf> <pr>(t<umac/nd)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Tuning</conjf>.]</vmorph> <sn>1.</sn> <def>To put into a state adapted to produce the proper sounds; to harmonize, to cause to be in tune; to correct the tone of; <as>as, to <ex>tune</ex> a piano or a violin</as>.</def> <ldquo/ <xex>Tune</xex> your harps.<rdquo/  <rj><au>Dryden.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><-- p. 1551 pr=PI --><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To give tone to; to attune; to adapt in style of music; to make harmonious.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>For now to sorrow must I <qex>tune</qex> my song.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>To sing with melody or harmony.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Fountains, and ye, that warble, as ye flow,<br/
Melodious murmurs, warbling <qex>tune</qex> his praise.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>To put into a proper state or disposition.</def>  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tune</ent><br/
<hw>Tune</hw> <pr>(t<umac/n)</pr>, <pos>v. i.</pos> <sn>1.</sn> <def>To form one sound to another; to form accordant musical sounds.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Whilst <qex>tuning</qex> to the water's fall,<br/
The small birds sang to her.</q> <rj><qau>Drayton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To utter inarticulate harmony with the voice; to sing without pronouncing words; to hum.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tuneful</ent><br/
<hw>Tune"ful</hw> <pr>(-f<usdot/l)</pr>, <pos>a.</pos> <def>Harmonious; melodious; musical; <as>as, <ex>tuneful</ex> notes</as>.</def> <ldquo/ <xex>Tuneful</xex> birds.<rdquo/ <au>Milton.</au> -- <wordforms><wf>Tune"ful*ly</wf>, <pos>adv.</pos> -- <wf>Tune"ful*ness</wf>, <pos>n.</pos></wordforms><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tuneless</ent><br/
<hw>Tune"less</hw> <pr>(t<umac/n"l<ecr/s)</pr>, <pos>a.</pos> <sn>1.</sn> <def>Without tune; inharmonious; unmusical.</def> <ldquo/ Thy <xex>tuneless</xex> serenade.<rdquo/  <rj><au>Cowley.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>How often have I led thy sportive choir,<br/
With <qex>tuneless</qex> pipe, beside the murmuring Loire!</q> <rj><qau>Goldsmith.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Not employed in making music; <as>as, <ex>tuneless</ex> harps</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>Not expressed in music or poetry; unsung.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tuner</ent><br/
<hw>Tun"er</hw> <pr>(t<umac/n"<etil/r)</pr>, <pos>n.</pos> <def>One who tunes; especially, one whose occupation is to tune musical instruments.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tun-great</ent><br/
<hw>Tun"-great`</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Having the circumference of a tun.</def> <mark>[Obs.]</mark>  <rj><au>Chaucer.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tungstate</ent><br/
<hw>Tung"state</hw> <pr>(t<ucr/ng"st<asl/t)</pr>, <pos>n.</pos> <ety>[Cf. F. <ets>tungstate</ets>.]</ety> <fld>(Chem.)</fld> <def>A salt of tungstic acid; a wolframate.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tungsten</ent><br/
<hw>Tung"sten</hw> <pr>(t<ucr/ng"st<ecr/n)</pr>, <pos>n.</pos> <ety>[Sw. <ets>tungsten</ets> (cf. Dan. <ets>tungsteen</ets>, G. <ets>tungstein</ets>); <ets>tung</ets> heavy (akin to Dan. <ets>tung</ets>, Icel. <ets><thorn/ungr</ets>) + <ets>sten</ets> stone. See <er>Stone</er>.]</ety> <sn>1.</sn> <fld>(Chem.)</fld> <def>A rare element of the chromium group found in certain minerals, as wolfram and scheelite, and isolated as a heavy steel-gray metal which is very hard and infusible. It has both acid and basic properties. When alloyed in small quantities with steel, it greatly increases its hardness. Symbol W (<xex>Wolframium</xex>). Atomic weight, 183.6. Specific gravity, 18.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Min.)</fld> <def>Scheelite, or calcium tungstate.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><cs><mcol><col><b>Tungsten ocher</b></col>, <it>or</it> <col><b>Tungstic ocher</b></col></mcol> <fld>(Min.)</fld>, <cd>tungstite.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tungstenic</ent><br/
<hw>Tung*sten"ic</hw> <pr>(t<ucr/ng*st<ecr/n"<icr/k)</pr>, <pos>a.</pos> <def>Of or pertaining to tungsten; containing tungsten; <as>as, <ex>tungstenic</ex> ores</as>.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tungsten lamp</ent><br/
<hw>Tung"sten lamp</hw>. <def>An electric glow lamp having filaments of metallic tungsten, and contained in a glass bulb which is evacuated or has an inert gas, to avoid oxidation of the tungsten; a common form of <isa>light bulb</isa>. Such lamps, owing to the refractory nature of the metal, may be maintained at a very high temperature and require an expenditure of only about 1.25 watts per candle power, depending on the total wattage and the design of the bulb.  By mid-20th century <ex>tungsten lamps</ex> became the most common type of incandescent (as contrasted with fluorescent) lamp; thus the phrase <col><b>incandescent lamp</b></col> or <col><b>incandescent light</b></col> typically refers to a <ex>tungsten lamp</ex>.</def><br/
[<source>Webster 1913 Suppl.</source> <source>+PJC</source>]</p>

<p><ent>Tungsten steel</ent><br/
<hw>Tungsten steel</hw>. <fld>(Metal.)</fld> <def>A steel containing a small amount of tungsten, noted for tenacity and hardness, even under a considerable degree of heat.  Magnets made of it are said to be highly permanent.  It often contains manganese.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Tungstic</ent><br/
<hw>Tung"stic</hw> <pr>(t<ucr/ng"st<icr/k)</pr>, <pos>a.</pos> <fld>(Chem.)</fld> <def>Of or pertaining to tungsten; derived from, or resembling, tungsten; wolframic; <as>as, <ex>tungstic</ex> oxide</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Tungstic acid</b></col>, <cd>an acid of tungsten, <chform>H2WO4</chform>, analogous to sulphuric and chromic acids.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tungstite</ent><br/
<hw>Tung"stite</hw> <pr>(t<ucr/ng"st<imac/t)</pr>, <pos>n.</pos> <fld>(Min.)</fld> <def>The oxide of tungsten, a yellow mineral occurring in a pulverulent form. It is often associated with wolfram.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tunguses</ent><br/
<hw>Tun*gus"es</hw> <pr>(?)</pr>, <pos>n. pl.</pos> <fld>(Ethnol.)</fld> <def>A group of roving Turanian tribes occupying Eastern Siberia and the <river>Amur</river> valley. They resemble the Mongols.</def> <altsp>[Written also <asp>Tungooses</asp>.]</altsp><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tungusic</ent><br/
<hw>Tun*gus"ic</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Of or pertaining to the Tunguses; <as>as, the <ex>Tungusic</ex> dialects</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tunhoof</ent><br/
<hw>Tun"hoof`</hw> <pr>(t<ucr/n"h<oomac/f`)</pr>, <pos>n.</pos> <ety>[Cf. <er>Alehoof</er>.]</ety> <fld>(Bot.)</fld> <def>Ground ivy; alehoof.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tunic</ent><br/
<hw>Tu"nic</hw> <pr>(t<umac/"n<icr/k)</pr>, <pos>n.</pos> <ety>[L. <ets>tunica</ets>: cf. F. <ets>tunique</ets>.]</ety><br/
[<source>1913 Webster</source>]</p>

<p><sn>1.</sn> <fld>(Rom. Antiq.)</fld> <def>An under-garment worn by the ancient Romans of both sexes.  It was made with or without sleeves, reached to or below the knees, and was confined at the waist by a girdle.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Any similar garment worn by ancient or Oriental peoples; also, a common name for various styles of loose-fitting under-garments and over-garments worn in modern times by Europeans and others.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <fld>(R. C. Ch.)</fld> <def>Same as <er>Tunicle</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <fld>(Anat.)</fld> <def>A membrane, or layer of tissue, especially when enveloping an organ or part, as the eye.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <fld>(Bot.)</fld> <def>A natural covering; an integument; <as>as, the <ex>tunic</ex> of a seed</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>6.</sn> <fld>(Zool.)</fld> <def>See <er>Mantle</er>, <pos>n.</pos>, 3 <sd>(a)</sd>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tunicary</ent><br/
<hw>Tu"ni*ca*ry</hw> <pr>(?)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Tunicaries</plw> <pr>(#)</pr>.</plu> <ety>[L. <ets>tunica</ets> a tunic.]</ety> <fld>(Zool.)</fld> <def>One of the Tunicata.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tunicata</ent><br/
||<hw>Tu`ni*ca"ta</hw> <pr>(?)</pr>, <pos>pr. n. pl.</pos> <ety>[NL. See <er>Tunicate</er>.]</ety> <fld>(Zool.)</fld> <def>A grand division of the animal kingdom, intermediate, in some respects, between the invertebrates and vertebrates, and in modern classifications considered a subphylum of the vertebrates; called also <subphylum>urochordata</subphylum>.  They were formerly classed with acephalous mollusks.  The body is usually covered with a firm external tunic, consisting in part of cellulose, and having two openings, one for the entrance and one for the exit of water.  The pharynx is usually dilated in the form of a sac, pierced by several series of ciliated slits, and serves as a gill.</def><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ Most of the species when mature are firmly attached to foreign substances, but have free-swimming larvae which are furnished with an elongated tail and somewhat resemble a tadpole.  In this state the larva has a urochord and certain other structures resembling some embryonic vertebrates. See <er>Ascidian</er>, <er>Doliolum</er>, <er>Salpa</er>, <er>Urochord</er>, and <xex>Illust.</xex> of <cref>Social ascidian</cref>, under <er>Social</er>.</note><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tunicated</ent><br/
<ent>Tunicate</ent><br/
<mhw>{ <hw>Tu"ni*cate</hw> <pr>(?)</pr>, <hw>Tu"ni*ca`ted</hw> <pr>(?)</pr>, }</mhw> <pos>a.</pos> <ety>[L. <ets>tunicatus</ets>, p. p. of <ets>tunicare</ets> to clothe with a tunic, fr. <ets>tunica</ets> a tunic.]</ety> <sn>1.</sn> <fld>(Bot.)</fld> <def>Covered with a tunic; covered or coated with layers; <as>as, a <ex>tunicated</ex> bulb</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Zool.)</fld> <sd>(a)</sd> <def>Having a tunic, or mantle; of or pertaining to the Tunicata.</def> <sd>(b)</sd> <def>Having each joint buried in the preceding funnel-shaped one, as in certain antennae of insects.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tunicate</ent><br/
<hw>Tu"ni*cate</hw> <pr>(?)</pr>, <pos>pr. n.</pos> <fld>(Zool.)</fld> <def>One of the <subphylum>Tunicata</subphylum>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tunicin</ent><br/
<hw>Tu"ni*cin</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Physiol. Chem.)</fld> <def>Animal cellulose; a substance present in the mantle, or tunic, of the Tunicates, which resembles, or is identical with, the cellulose of the vegetable kingdom.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tunicle</ent><br/
<hw>Tu"ni*cle</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>tunicula</ets> a little tunic, coat, or membrane, dim. of <ets>tunica</ets> a tunic: cf. OF. <ets>tunicle</ets>.]</ety> <sn>1.</sn> <def>A slight natural covering; an integument.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The <qex>tunicles</qex> that make the ball or apple of the eye.</q> <rj><qau>Holland.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(R. C. Ch.)</fld> <def>A short, close-fitting vestment worn by bishops under the dalmatic, and by subdeacons.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tuning</ent><br/
<hw>Tun"ing</hw> <pr>(t<umac/n"<icr/ng)</pr>, <def><pos>a. & n.</pos> from <er>Tune</er>, <pos>v.</pos></def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Tuning fork</b></col> <fld>(Mus.)</fld>, <cd>a steel instrument consisting of two prongs and a handle, which, being struck, gives a certain fixed tone. It is used for tuning instruments, or for ascertaining the pitch of tunes.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tunk</ent><br/
<hw>Tunk</hw> <pr>(t<ucr/<nsm/k)</pr>, <pos>n.</pos> <def>A sharp blow; a thump.</def> <mark>[Prov. Eng. <or/ Colloq. U. S.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tunker</ent><br/
<hw>Tun"ker</hw> <pr>(t<ucr/<nsm/"k<etil/r)</pr>, <pos>n.</pos> <fld>(Eccl.)</fld> <def>Same as <er>Dunker</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tunnage</ent><br/
<hw>Tun"nage</hw> <pr>(t<ucr/n"n<asl/j; 48)</pr>, <pos>n.</pos> <ety>[From <er>Tun</er>; cf. <er>Tonnage</er>.]</ety> <def>See <er>Tonnage</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tunnel</ent><br/
<hw>Tun"nel</hw> <pr>(t<ucr/n"n<ecr/l)</pr>, <pos>n.</pos> <ety>[F. <ets>tonnelle</ets> a semicircular, wagon-headed vault, a tunnel net, an arbor, OF. also <ets>tonnel</ets>; dim. of <ets>tonne</ets> a tun; -- so named from its resemblance to a tun in shape. See <er>Ton</er>.]</ety> <sn>1.</sn> <def>A vessel with a broad mouth at one end, and a pipe or tube at the other, for conveying liquor, fluids, etc., into casks, bottles, or other vessels; a funnel.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>The opening of a chimney for the passage of smoke; a flue; a funnel.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>And one great chimney, whose long <qex>tunnel</qex> thence<br/
The smoke forth threw.</q> <rj><qau>Spenser.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>An artificial passage or archway for conducting canals, roads, or railroads under elevated ground, for the formation of roads under rivers or canals, and the construction of sewers, drains, and the like.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <fld>(Mining)</fld> <def>A level passage driven across the measures, or at right angles to veins which it is desired to reach; -- distinguished from the <contr>drift</contr>, or <contr>gangway</contr>, which is led along the vein when reached by the tunnel.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Tunnel head</b></col> <fld>(Metal.)</fld>, <cd>the top of a smelting furnace where the materials are put in.</cd> -- <col><b>Tunnel kiln</b></col>, <cd>a limekiln in which coal is burned, as distinguished from a <contr>flame kiln</contr>, in which wood or peat is used.</cd> -- <col><b>Tunnel net</b></col>, <cd>a net with a wide mouth at one end and narrow at the other.</cd> -- <mcol><col><b>Tunnel pit</b></col>, <col><b>Tunnel shaft</b></col></mcol>, <cd>a pit or shaft sunk from the top of the ground to the level of a tunnel, for drawing up the earth and stones, for ventilation, lighting, and the like.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tunnel</ent><br/
<hw>Tun"nel</hw>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Tunneled</conjf> <pr>(t<ucr/n"n<ecr/ld)</pr> or <conjf>Tunnelled</conjf>; <pos>p. pr. & vb. n.</pos> <conjf>Tunneling</conjf> or <conjf>Tunnelling</conjf>.]</vmorph><br/
[<source>1913 Webster</source>]</p>

<p><sn>1.</sn> <def>To form into a tunnel, or funnel, or to form like a tunnel; <as>as, to <ex>tunnel</ex> fibrous plants into nests</as>.</def>  <rj><au>Derham.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To catch in a tunnel net.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>To make an opening, or a passageway, through or under; <as>as, to <ex>tunnel</ex> a mountain; to <ex>tunnel</ex> a river.</as></def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tunnel</ent><br/
<hw>Tun"nel</hw>, <pos>v. i.</pos> <def>To make a tunnel; <as>as, to <ex>tunnel</ex> under a river</as>.</def><br/
[<source>PJC</source>]</p>

<p><ent>Tunnel stern</ent><br/
<hw>Tun"nel stern</hw>. <def>A design of motor-boat stern, for  use in shallow waters, in which the propeller is housed in a tunnel and does not extend below the greatest draft.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Tunny</ent><br/
<hw>Tun"ny</hw> <pr>(t<ucr/n"n<ycr/)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Tunnies</plw> <pr>(#)</pr>.</plu> <ety>[L. <ets>thunnus</ets>, <ets>thynnus</ets>, Gr. <grk>qy`nnos</grk>, <grk>qy^nos</grk>: cf. It. <ets>tonno</ets>, F. & Pr. <ets>thon</ets>.]</ety> <fld>(Zool.)</fld> <def>The chiefly British equivalent of <altname>tuna</altname>; any one of several species of large oceanic fishes belonging to the Mackerel family, especially the common or great tunny (<spn>Thunnus thynnus</spn> syn. <spn>Albacora thynnus</spn>, formerly <spn>Orcynus thynnus</spn>) native of the <geog>Mediterranean Sea</geog> and the <geog>Atlantic Ocean</geog>.  It sometimes weighs a thousand pounds or more, and is extensively caught in the Mediterranean.  On the American coast it is called <altname>horse mackerel</altname>. See <xex>Illust.</xex> of <cref>Horse mackerel</cref>, under <er>Horse</er>.</def> <altsp>[Written also <asp>thynny</asp>.]</altsp><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ The little tunny (<spn>Gymnosarda alletterata</spn>) of the Mediterranean and North Atlantic, and the long-finned tunny, or albicore (<spn>Thunnus alalunga</spn>, see <er>Albacore</er>), are related species of smaller size.</note><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tup</ent><br/
<hw>Tup</hw> <pr>(t<ucr/p)</pr>, <pos>v. t. & i.</pos> <ety>[Probably akin to <ets>top</ets> summit, head.]</ety> <sn>1.</sn> <def>To butt, as a ram does.</def> <mark>[Prov. Eng.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To cover; -- said of a ram.</def>  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tup</ent><br/
<hw>Tup</hw>, <pos>n.</pos> <fld>(Zool.)</fld> <def>A ram.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tupai</ent><br/
<hw>Tu*pai"</hw> <pr>(t<oocr/*p<imac/")</pr>, <pos>n.</pos> <fld>(Zool.)</fld> <def>Any one of the tupaiids.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tupaiid</ent><br/
<hw>Tu*pai"id</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Zool.)</fld> <def>Any one of several species of East Indian and Asiatic insectivores of the family <fam>Tupaiidae</fam>, somewhat resembling squirrels in size and arboreal habits.  The nose is long and pointed.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tupelo</ent><br/
<hw>Tu"pe*lo</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[<ets>Tupelo</ets>, or <ets>tupebo</ets>, the native American Indian name.]</ety> <fld>(Bot.)</fld> <def>A North American tree (<spn>Nyssa multiflora</spn>) of the Dogwood family, having brilliant, glossy foliage and acid red berries.  The wood is crossgrained and very difficult to split. Called also <altname>black gum</altname>, <altname>sour gum</altname>, and <altname>pepperidge</altname>.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><mcol><col><b>Largo tupelo</b></col>, <it>or</it> <col><b>Tupelo gum</b></col></mcol> <fld>(Bot.)</fld>, <cd>an American tree (<spn>Nyssa uniflora</spn>) with softer wood than the tupelo.</cd> -- <col><b>Sour tupelo</b></col> <fld>(Bot.)</fld>, <cd>the Ogeechee lime.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tupi</ent><br/
<hw>Tu"pi</hw> <pr>(?)</pr>, <pos>pr. n.</pos> <def>An Indian of the tribe from which the Tupian stock takes its name, dwelling, at the advent of the Portuguese, about the mouth of the Amazon.  Also, their language, which is the basis of the Indian trade language of the Amazon.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Tupian</ent><br/
<hw>Tu"pi*an</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Designating, or pertaining to, a linguistic stock of South American Indians comprising the most important Brazilian tribes.  Agriculture, pottery, and stone working were practiced by them at the time of the conquest.  The Tupi and the Guarani were originally the most powerful of the stock, which is hence also called <altname>Tupi-Guaranian</altname>.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Tupman</ent><br/
<hw>Tup"man</hw> <pr>(?)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Tupmen</plw> <pr>(<?/)</pr>.</plu> <def>A man who breeds, or deals in tups.</def> <mark>[Prov. Eng.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>tuppence</ent><br/
<hw>tuppence</hw> <pos>n.</pos> <sn>1.</sn> <ety>[contraction from <ets>two pence</ets>.]</ety> <def>A former U.K. silver <isa>coin</isa>; a U.K. bronze decimal coin worth two pennies.</def><br/
<syn><b>Syn. --</b> twopence</syn><br/
[<source>WordNet 1.5</source>]</p>

<p><ent>Tupper ware</ent><br/
<ent>Tupperware</ent><br/
<mhw><hw>Tupperware</hw>, <hw>Tup"per ware</hw></mhw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Trade name.]</ety> <sn>1.</sn> <def>the collective trade name for a type of plastic food-storage container with a translucent bottom and a close-fitting lid designed to seal the container tightly.  It is in common use in the late 20th century and early 21st century in the U. S.  It comes in a variety of shapes and sizes, suitable for storing a variety of items.  Also used attributively.</def> <illu>She saved the leftovers in a couple of <ex>Tupperware</ex> containers.</illu><br/
[<source>PJC</source>]</p>

<p><ent>Tupperware party</ent><br/
<hw>Tupperware party</hw> <pos>n.</pos> <def>a social gathering at which the host (or more typically hostess) entertains the guests, and provides them with an opportunity to order Tupperware.  This was used as an effective sales strategy by the Tupperware manufacturer, and provided income to the host(ess) from commissions on sales.</def><br/
[<source>PJC</source>]</p>

<p><ent>Tuque</ent><br/
<hw>Tuque</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Canadian F. See <er>Toque</er>.]</ety> <def>A kind of warm cap winter wear, made from a knit bag with closed tapered ends by pushing one end within the other, thus making a conical cap of double thickness.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><q>Picturesque fellow with <qex>tuques</qex>, red sashes, and fur coats.</q>  <rj><qau>F. Remington.</qau></rj><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Tur</ent><br/
||<hw>Tur</hw> <pr>(t<oomac/r)</pr>, <pos>n.</pos> <ety>[Pol.]</ety> <fld>(Zool.)</fld> <def>The <er>urus</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Turacin</ent><br/
<hw>Tu"ra*cin</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Physiol.)</fld> <fld>(Chem.)</fld> <def>A red or crimson pigment obtained from certain feathers of several species of <prodby>turacou</prodby>; whence the name. It contains nearly six per cent of copper.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>turacou</ent><br/
<ent>turaco</ent><br/
<mhw><hw>turaco</hw>, <hw>tu*ra"cou</hw></mhw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Cf. F. <ets>touraco</ets>.]</ety> <fld>(Zool.)</fld> <def>Any one of several species of large brightly crested birds of the genus <gen>Turacus</gen>, native of Africa.  They eat plantain and are remarkable for the peculiar green and red pigments found in their feathers.</def> <altsp>[Written also <asp>touraco</asp>, <asp>touracou</asp>, and <asp>turakoo</asp>.]</altsp><br/
[<source>1913 Webster</source> + <source>WordNet 1.5</source>]</p>

<p><ent>Turacoverdin</ent><br/
<hw>Tu*ra`co*ver"din</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[See <er>Turacou</er>, and <er>Verdant</er>.]</ety> <fld>(Physiol.)</fld> <fld>(Chem.)</fld> <def>A green pigment found in the feathers of the <prodby>turacou</prodby>. See <er>Turacin</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Turanian</ent><br/
<hw>Tu*ra"ni*an</hw> <pr>(?)</pr>, <pos>pr. a.</pos> <ety>[From <ets>Tur</ets>, the name, in Persian legendary history, of one of the three brothers from whom sprang the races of mankind.]</ety> <def>Of, pertaining to, or designating, an extensive family of languages of simple structure and low grade (called also <stype>Altaic</stype>, <stype>Ural-Altaic</stype>, and <stype>Scythian</stype>), spoken in the northern parts of Europe and Asia and in Central Asia; of pertaining to, or designating, the people who speak these languages.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Turanian</ent><br/
<hw>Tu*ra"ni*an</hw> <pr>(?)</pr>, <pos>pr. n.</pos> <def>One of the Turanians.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Turanians</ent><br/
<hw>Tu*ra"ni*ans</hw> <pr>(?)</pr>, <pos>n. pl.</pos> <fld>(Ethnol.)</fld> <sd>(a)</sd> <def>An extensive division of mankind including the Mongols and allied races of Asia, together with the Malays and Polynesians.</def> <sd>(b)</sd> <def>A group of races or tribes inhabiting Asia and closely related to the Mongols.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Turatt</ent><br/
<hw>Tu"ratt</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Zool.)</fld> <def>The hare kangaroo.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Turban</ent><br/
<hw>Tur"ban</hw> <pr>(t<ucir/r"b<ait/n)</pr>, <pos>n.</pos> <ety>[OE. <ets>turband</ets>, <ets>turbant</ets>, <ets>tolibant</ets>, F. <ets>turban</ets>, It. <ets>turbante</ets>, Turk. <ets>tulbend</ets>, <ets>dulbend</ets>, fr. Per. <ets>dulband</ets>.  Cf. <er>Tulip</er>.]</ety> <sn>1.</sn> <def>A headdress worn by men in the Levant and by many Muslims of the male sex, consisting of a cap, and a sash, scarf, or shawl, usually of cotton or linen, wound about the cap, and sometimes hanging down the neck.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A kind of headdress worn by women.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <fld>(Zool.)</fld> <def>The whole set of whorls of a spiral shell.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Turband</ent><br/
<hw>Tur"band</hw> <pr>(t<ucir/r"b<ait/nd)</pr>, <pos>n.</pos> <def>A turban.</def>  <rj><au>Balfour (Cyc. of Ind.).</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Turbaned</ent><br/
<hw>Tur"baned</hw> <pr>(t<ucir/r"b<ait/nd)</pr>, <pos>a.</pos> <def>Wearing a turban.</def> <ldquo/ A malignant and a <xex>turbaned</xex> Turk.<rdquo/  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Turban-shell</ent><br/
<hw>Tur"ban-shell`</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Zool.)</fld> <def>A sea urchin when deprived of its spines; -- popularly so called from a fancied resemblance to a turban.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Turbant</ent><br/
<hw>Tur"bant</hw> <pr>(t<ucir/r"b<ait/nt)</pr>, <pos>n.</pos> <def>A turban.</def> <mark>[Obs.]</mark>  <rj><au>Milton.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>I see the Turk nodding with his <qex>turbant</qex>.</q> <rj><qau>Howell.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Turban-top</ent><br/
<hw>Tur"ban-top`</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Bot.)</fld> <def>A kind of fungus with an irregularly wrinkled, somewhat globular pileus (<spn>Helvella esculenta</spn> syn. <spn>Gyromitra esculenta</spn>.).</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Turbary</ent><br/
<hw>Tur"ba*ry</hw> <pr>(?)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Turbaries</plw> <pr>(#)</pr>.</plu> <ety>[LL. <ets>turbaria</ets> a place for digging peat, from <ets>turba</ets> peat. See <er>Turf</er>.]</ety> <fld>(Eng. Law)</fld> <def>A right of digging turf on another man's land; also, the ground where turf is dug.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Turbellaria</ent><br/
||<hw>Tur`bel*la"ri*a</hw> <pr>(?)</pr>, <pos>n. pl.</pos> <ety>[NL., dim. fr. L. <ets>turbo</ets> a whirling.]</ety> <fld>(Zool.)</fld> <def>An extensive group of worms which have the body covered externally with vibrating cilia. It includes the Rhabdoc<oe/la and Dendroc<oe/la. Formerly, the nemerteans were also included in this group.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Turbellarian</ent><br/
<hw>Tur`bel*la"ri*an</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Zool.)</fld> <def>One of the Turbellaria. Also used adjectively.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Turbeth</ent><br/
<hw>Tur"beth</hw> <pr>(t<ucir/r"b<ecr/th)</pr>, <pos>n.</pos> <def>See <er>Turpeth</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Turbid</ent><br/
<hw>Tur"bid</hw> <pr>(t<ucir/r"b<icr/d)</pr>, <pos>a.</pos> <ety>[L. <ets>turbidus</ets>, from <ets>turba</ets> tumult, disturbance, akin to <ets>turbare</ets> to disturb. See <er>Trouble</er>, and cf. <er>Disturb</er>, <er>Perturb</er>.]</ety> <sn>1.</sn> <def>Not clear; having suspended matter that scatters light passing through; having the lees or sediment disturbed; roiled; muddy; thick; -- used of liquids of any kind; <as>as, <ex>turbid</ex> water; <ex>turbid</ex> wine.</as></def><br/
[<source>1913 Webster</source>]</p>

<p><q>On that strong, <qex>turbid</qex> water, a small boat,<br/
Guided by one weak hand, was seen to float.</q> <rj><qau>Whittier.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Disturbed; confused; disordered.</def> <ldquo/ Such <xex>turbid</xex> intervals that use to attend close prisoners.<rdquo/  <rj><au>Howell.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Turbidity</ent><br/
<hw>Tur*bid"i*ty</hw> <pr>(?)</pr>, <pos>n.</pos> <def>Turbidness.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Turbidly</ent><br/
<hw>Tur"bid*ly</hw> <pr>(?)</pr>, <pos>adv.</pos> <sn>1.</sn> <def>In a turbid manner; with muddiness or confusion.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Proudly; haughtily.</def> <mark>[A Latinism. R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>One of great merit <qex>turbidly</qex> resents them.</q> <rj><qau>Young.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Turbidness</ent><br/
<hw>Tur"bid*ness</hw>, <pos>n.</pos> <def>The quality or state of being turbid; muddiness; foulness.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Turbillion</ent><br/
<hw>Tur*bil"lion</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F. <ets>tourbillon</ets>, from L. <ets>turbo</ets> a whirl.]</ety> <def>A whirl; a vortex.</def>  <rj><au>Spectator.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Turbinaceous</ent><br/
<hw>Tur`bi*na"ceous</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[See <er>Turbary</er>.]</ety> <def>Of or pertaining to peat, or turf; of the nature of peat, or turf; peaty; turfy.</def>  <rj><au>Sir. W. Scott.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Turbinal</ent><br/
<hw>Tur"bi*nal</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>turbo</ets>, <ets>turben</ets>, <ets>-inis</ets>, a top, whirl.]</ety> <fld>(Anat.)</fld> <def>Rolled in a spiral; scroll-like; turbinate; -- applied to the thin, plicated, bony or cartilaginous plates which support the olfactory and mucous membranes of the nasal chambers.</def><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ There are usually several of these plates in each nasal chamber.  The upper ones, connected directly with the ethmoid bone, are called <stype>ethmoturbinals</stype>, and the lower, connected with the maxillae, <stype>maxillo-turbinals</stype>. Incurved portions of the wall of the nasal chamber are sometimes called <contr>pseudoturbinals</contr>, to distinguish them from the true turbinals which are free outgrowths into the chambers.</note><br/
[<source>1913 Webster</source>]</p>

<p><ent>Turbinal</ent><br/
<hw>Tur"bi*nal</hw>, <pos>n.</pos> <fld>(Anat.)</fld> <def>A turbinal bone or cartilage.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Turbinate</ent><br/
<hw>Tur"bi*nate</hw> <pr>(?)</pr>, <pos>v. i.</pos> <def>To revolve or spin like a top; to whirl.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Turbinated</ent><br/
<ent>Turbinate</ent><br/
<mhw>{ <hw>Tur"bi*nate</hw> <pr>(?)</pr>, <hw>Tur"bi*na`ted</hw> <pr>(?)</pr>, }</mhw> <pos>a.</pos> <ety>[L. <ets>turbinatus</ets>, fr. <ets>turbo</ets>, <ets>turben</ets>, <ets>turbinis</ets>, a whirl, top.]</ety> <sn>1.</sn> <def>Whirling in the manner of a top.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>A spiral and <qex>turbinated</qex> motion of the whole.</q> <rj><qau>Bentley.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Bot.)</fld> <def>Shaped like a top, or inverted cone; narrow at the base, and broad at the apex; <as>as, a <ex>turbinated</ex> ovary, pericarp, or root</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <fld>(Anat.)</fld> <def>Turbinal.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <fld>(Zool.)</fld> <def>Spiral with the whorls decreasing rapidly from a large base to a pointed apex; -- said of certain shells.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Turbination</ent><br/
<hw>Tur`bi*na"tion</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Cf. L. <ets>tirbinatio</ets> a pointing in the form of a cone.  See <er>Turbinate</er>.]</ety> <def>The act of spinning or whirling, as a top.</def><br/
[<source>1913 Webster</source>]</p>

<p><-- p. 1552 pr=PI --><br/
[<source>1913 Webster</source>]</p>

<p><ent>Turbine</ent><br/
<hw>Tur"bine</hw> <pr>(t<ucir/r"b<icr/n)</pr>, <pos>n.</pos> <ety>[L. <ets>turbo</ets>, <ets>-inis</ets>, that which spins or whirls round, whirl.]</ety> <sn>1.</sn> <def>A water wheel, commonly horizontal, variously constructed, but usually having a series of curved floats or buckets, against which the water acts by its impulse or reaction in flowing either outward from a central chamber, inward from an external casing, or from above downward, etc.; -- also called <altname>turbine wheel</altname>.</def><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ In some turbines, the water is supplied to the wheel from below, instead of above. Turbines in which the water flows in a direction parallel to the axis are called <xex>parallel-flow</xex> turbines.</note><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A type of rotary engine with a set of rotating vanes, diagonally inclined and often curved, attached to a central spindle, and obtaining its motive force from the passage of a fluid, as water, steam, combusted gases, or air, over the vanes.  Water turbines are frequently used for generating power at hydroelectric power stations, and steam turbines are used for generating power from coal- or oil-fired electric power stations.  Turbines are also found in jet engines, and in some automobile engines.</def>  <note>In the 1913 dictionary, the turbine was further decribed thus: <ldquo/There are practically only two distinct kinds, and they are typified in the de Laval and the Parsons and Curtis turbines. The <col><b>de Laval turbine</b></col> is an impulse turbine, in which steam impinges upon revolving blades from a flared nozzle. The flare of the nozzle causes expansion of the steam, and hence changes its pressure energy into kinetic energy. An enormous velocity (30,000 revolutions per minute in the 5 H. P. size) is requisite for high efficiency, and the machine has therefore to be geared down to be of practical use. Some recent development of this type include turbines formed of several de Laval elements compounded as in the ordinary expansion engine. The <ecol><b>Parsons turbine</b></ecol> is an impulse-and-reaction turbine, usually of the axial type. The steam is constrained to pass successively through alternate rows of fixed and moving blades, being expanded down to a condenser pressure of about 1 lb. per square inch absolute. The <ecol><b>Curtis turbine</b></ecol> is somewhat simpler than the Parsons, and consists of elements each of which has at least two rows of moving blades and one row of stationary. The bucket velocity is lowered by fractional velocity reduction. Both the Parsons and Curtis turbines are suitable for driving dynamos and steamships directly. In efficiency, lightness, and bulk for a given power, they compare favorably with reciprocating engines.<rdquo/</note><br/
[<source>Webster 1913 Suppl.</source> <source>+PJC</source>]</p>

<p><ent>Turbinella</ent><br/
||<hw>Tur`bi*nel"la</hw> <pr>(?)</pr>, <pos>pr. n.</pos> <ety>[NL., dim. fr. L. <ets>turbo</ets>, <ets>-inis</ets>, a top.]</ety> <fld>(Zool.)</fld> <def>A genus of large marine gastropods having a thick heavy shell with conspicuous folds on the columella.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Turbinite</ent><br/
<hw>Tur"bi*nite</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[NL. <ets>Turbo</ets>, the generic name, fr. L. <ets>turbo</ets> a whirl, top: cf. F. <ets>turbinite</ets>.]</ety> <fld>(Paleon.)</fld> <def>A petrified shell resembling a member of the genus <gen>Turbo</gen>.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Turbinoid</ent><br/
<hw>Tur"bi*noid</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[See <er>Turbo</er>, and <er>-oid</er>.]</ety> <fld>(Zool.)</fld> <def>Like or pertaining to <gen>Turbo</gen> or the family <fam>Turbinidae</fam>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Turbit</ent><br/
<hw>Tur"bit</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Cf. <er>Turbot</er>.]</ety><br/
[<source>1913 Webster</source>]</p>

<p><sn>1.</sn> <fld>(Zool.)</fld> <def>The turbot.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Zool.)</fld> <def>A variety of the domestic pigeon, remarkable for its short beak.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Turbite</ent><br/
<hw>Tur"bite</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Paleon.)</fld> <def>A fossil turbo.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Turbith</ent><br/
<hw>Tur"bith</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F., fr. Per. See <er>Turpeth</er>.]</ety> <def>See <er>Turpeth</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>turbo</ent><br/
<hw>tur"bo</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>turbo</ets>, <ets>-inis</ets>, a top. See <er>Turbine</er>.]</ety> <fld>(Zool.)</fld> <def>Any one of numerous marine gastropods of the genus <gen>Turbo</gen> or family <fam>Turbinidae</fam>, usually having a turbinate shell, pearly on the inside, and a calcareous operculum.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>turbo-</ent><br/
<hw>tur"bo-</hw> <pr>(?)</pr>, <pos>n.</pos> <def>Sontaining or using a turbine; -- a combining form; <as>as, <ex>turbo</ex>jet</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Turbogenerator</ent><br/
<hw>Tur`bo*gen"er*a`tor</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[See <er>Turbine</er>, and <er>Generator</er>.]</ety> <def>An electric generator or dynamo which is combined on one frame with a turbomotor, by which it is driven.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Turbot</ent><br/
<hw>Tur"bot</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F.; -- probably so named from its shape, and from L. <ets>turbo</ets> a top, a whirl.]</ety> <fld>(Zool.)</fld> <sd>(a)</sd> <def>A large European flounder (<spn>Rhombus maximus</spn>) highly esteemed as a food fish. It often weighs from thirty to forty pounds.  Its color on the upper side is brownish with small roundish tubercles scattered over the surface.  The lower, or blind, side is white.  Called also <altname>bannock fluke</altname>.</def> <sd>(b)</sd> <def>Any one of numerous species of flounders more or less related to the true turbots, as the American plaice, or summer flounder (see <er>Flounder</er>), the halibut, and the diamond flounder (<spn>Hypsopsetta guttulata</spn>) of California.</def> <sd>(c)</sd> <def>The filefish; -- so called in Bermuda.</def> <sd>(d)</sd> <def>The trigger fish.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Spotted turbot</b></col>. <cd>See <er>Windowpane</er>.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Turbulence</ent><br/
<hw>Tur"bu*lence</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>turbulentia</ets>: cf. F. <ets>turbulence</ets>.]</ety> <def>The quality or state of being turbulent; a disturbed state; tumult; disorder; agitation.</def>  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>The years of . . . warfare and <qex>turbulence</qex> which ensued.</q> <rj><qau>Southey.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><syn><b>Syn.</b> -- Agitation; commotion; tumult; tumultuousness; termagance; unruliness; insubordination; rioting.</syn><br/
[<source>1913 Webster</source>]</p>

<p><ent>Turbulency</ent><br/
<hw>Tur"bu*len*cy</hw> <pr>(?)</pr>, <pos>n.</pos> <def>Turbulence.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>What a tale of terror now its <qex>turbulency</qex> tells!</q> <rj><qau>Poe.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Turbulent</ent><br/
<hw>Tur"bu*lent</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>turbulentus</ets>, fr. <ets>turba</ets> disorder, tumult: cf. F. <ets>turbulent</ets>. See <er>Turbid</er>.]</ety> <sn>1.</sn> <def>Disturbed; agitated; tumultuous; roused to violent commotion; <as>as, the <ex>turbulent</ex> ocean</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Calm region once,<br/
And full of peace, now tossed and <qex>turbulent</qex>.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Disposed to insubordination and disorder; restless; unquiet; refractory; <as>as, <ex>turbulent</ex> spirits</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Sagacious, bold, and <qex>turbulent</qex> of wit.</q> <rj><qau>Dryden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>Producing commotion; disturbing; exciting.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Whose heads that <qex>turbulent</qex> liquor fills with fumes.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><syn><b>Syn.</b> -- Disturbed; agitated; tumultuous; riotous; seditious; insubordinate; refractory; unquiet.</syn><br/
[<source>1913 Webster</source>]</p>

<p><ent>Turbulently</ent><br/
<hw>Tur"bu*lent*ly</hw>, <pos>adv.</pos> <def>In a turbulent manner.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Turcism</ent><br/
<hw>Tur"cism</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A mode of speech peculiar to the Turks; a Turkish idiom or expression; also, in general, a Turkish mode or custom.  Same as <er>Turkism</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Turcoman</ent><br/
<hw>Tur"co*man</hw> <pr>(?)</pr>, <pos>pr. n.</pos>; <plu><it>pl.</it> <plw>Turcomans</plw> <pr>(<?/)</pr>.</plu>  <sn>1.</sn> <def>A member of a tribe of Turanians inhabiting a region east of the Caspian Sea.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A Turcoman carpet.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><mcol><col><b>Turcoman carpet</b></col> <it>or</it> <col><b>Turcoman rug</b></col></mcol>, <cd>a kind of carpet or rug supposed to be made by the Turcomans.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Turdiformes</ent><br/
||<hw>Tur`di*for"mes</hw> <pr>(?)</pr>, <pos>n. pl.</pos> <ety>[NL., fr. L. <ets>turdus</ets> a thrush + <ets>forma</ets> form.]</ety> <fld>(Zool.)</fld> <def>A division of singing birds including the thrushes and allied kinds.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Turdus</ent><br/
||<hw>Tur"dus</hw> <pr>(?)</pr>, <pos>pr. n.</pos> <ety>[L., a thrush.]</ety> <fld>(Zool.)</fld> <def>A genus of singing birds including the true thrushes.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tureen</ent><br/
<hw>Tu*reen"</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F. <ets>terrine</ets>, L. <ets>terra</ets> earth. See <er>Terrace</er>.]</ety> <def>A large, deep vessel for holding soup, or other liquid food, at the table.</def> <altsp>[Written also <asp>terreen</asp>.]</altsp><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tureenful</ent><br/
<hw>Tu*reen"ful</hw> <pr>(?)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Tureenfuls</plw> <pr>(<?/)</pr>.</plu> <def>As much as a tureen can hold; enough to fill a tureen.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Turf</ent><br/
<hw>Turf</hw> <pr>(t<ucir/rf)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Turfs</plw> <pr>(t<ucir/rfs)</pr>, <mark>Obs.</mark> <plw>Turves</plw> <pr>(t<ucir/rvz)</pr>.</plu> <ety>[AS. <ets>turf</ets>; akin to D. <ets>turf</ets> peat, G. <ets>torf</ets>, OHG. <ets>zurba</ets> turf, Sw. & Icel. <ets>torf</ets> turf, peat, Dan. <ets>t<oum/rv</ets>, Skr. <ets>darbha</ets> a kind of grass, a tuft of grass. <root/242.]</ety> <sn>1.</sn> <def>That upper stratum of earth and vegetable mold which is filled with the roots of grass and other small plants, so as to adhere and form a kind of mat; sward; sod.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>At his head a grass-green <qex>turf</qex>.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>The Greek historian sets her in the field on a high heap of <qex>turves</qex>.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Peat, especially when prepared for fuel. See <er>Peat</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>Race course; horse racing; -- preceded by <xex>the</xex>.</def> <ldquo/We . . . claim the honors of the <xex>turf</xex>.<rdquo/  <rj><au>Cowper.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ <xex>Turf</xex> is often used adjectively, or to form compounds which are generally self-explaining; as, <xex>turf</xex> ashes, <xex>turf</xex> cutter or <xex>turf</xex>-cutter, <xex>turf</xex> pit or <xex>turf</xex>-pit, <xex>turf</xex>-built, <xex>turf</xex>-clad, <xex>turf</xex>-covered, etc.</note><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Turf ant</b></col> <fld>(Zool.)</fld>, <cd>a small European ant (<spn>Formica flava</spn>) which makes small ant-hills on heaths and commons.</cd> -- <col><b>Turf drain</b></col>, <cd>a drain made with turf or peat.</cd> -- <col><b>Turf hedge</b></col>, <cd>a hedge or fence formed with turf and plants of different kinds.</cd> -- <col><b>Turf house</b></col>, <cd>a house or shed formed of turf, common in the northern parts of Europe.</cd> -- <col><b>Turf moss</b></col> <cd>a tract of turfy, mossy, or boggy land.</cd> -- <col><b>Turf spade</b></col>, <cd>a spade for cutting and digging turf, longer and narrower than the common spade.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Turf</ent><br/
<hw>Turf</hw>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Turfed</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Turfing</conjf>.]</vmorph> <def>To cover with turf or sod; <as>as, to <ex>turf</ex> a bank, or the border of a terrace</as>.</def>  <rj><au>A. Tucker.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Turfen</ent><br/
<hw>Turf"en</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Made of turf; covered with turf.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Turfiness</ent><br/
<hw>Turf"i*ness</hw> <pr>(?)</pr>, <pos>n.</pos> <def>Quality or state of being turfy.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Turfing</ent><br/
<hw>Turf"ing</hw>, <pos>n.</pos> <def>The act or process of providing or covering with turf.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><mcol><col><b>Turfing iron</b></col>, <col><b>Turfing spade</b></col></mcol>, <cd>an implement for cutting, and paring off, turf.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Turfite</ent><br/
<hw>Turf"ite</hw> <pr>(t<ucir/rf"<imac/t)</pr>, <pos>n.</pos> <def>A votary of the turf, or race course; hence, sometimes, a blackleg.</def> <mark>[Colloq.]</mark>  <rj><au>Thackeray.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Turfless</ent><br/
<hw>Turf"less</hw> <pr>(t<ucir/rf"l<ecr/s)</pr>, <pos>a.</pos> <def>Destitute of turf.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Turfman</ent><br/
<hw>Turf"man</hw> <pr>(t<ucir/rf"m<ait/n)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Turfmen</plw> <pr>(t<ucir/rf"m<eit/n)</pr>.</plu> <def>A turfite; a votary of the turf, or race course.</def> <mark>[Colloq.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Turfy</ent><br/
<hw>Turf"y</hw> <pr>(t<ucir/rf"<ycr/)</pr>, <pos>a.</pos> <amorph>[<pos>Compar.</pos> <adjf>Turfier</adjf> <pr>(?)</pr>; <pos>superl.</pos> <adjf>Turfiest</adjf>.]</amorph> <sn>1.</sn> <def>Abounding with turf; made of, or covered with, turf.</def> <ldquo/The <xex>turfy</xex> mountains.<rdquo/  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Having the nature or appearance of turf.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>Of or pertaining to the turf, or horse racing.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Turgent</ent><br/
<hw>Tur"gent</hw> <pr>(t<ucir/r"j<eit/nt)</pr>, <pos>a.</pos> <ety>[L. <ets>turgens</ets>, <ets>-entis</ets>, p. pr. of <ets>turgere</ets> to swell.]</ety> <sn>1.</sn> <def>Rising into a tumor, or a puffy state; swelling; tumid; <as>as, <ex>turgent</ex> humors</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Inflated; bombastic; turgid; pompous.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Recompensed with <qex>turgent</qex> titles.</q> <rj><qau>Burton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Turgesce</ent><br/
<hw>Tur*gesce"</hw> <pr>(?)</pr>, <pos>v. i.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Turgesced</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Turgescing</conjf> <pr>(?)</pr>.]</vmorph> <ety>[L. <ets>turgescere</ets>, v. incho. fr. <ets>turgere</ets> to swell.]</ety> <def>To become turgid; to swell or be inflated.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Turgescency</ent><br/
<ent>Turgescence</ent><br/
<mhw>{ <hw>Tur*ges"cence</hw> <pr>(?)</pr>, <hw>Tur*ges"cen*cy</hw> <pr>(?)</pr>, }</mhw> <pos>n.</pos> <ety>[Cf. F. <ets>turgescence</ets>. See <er>Turgescent</er>.]</ety> <sn>1.</sn> <def>The act of swelling, or the state of being swollen, or turgescent.</def>  <rj><au>Sir T. Browne.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Empty magnificence or pompousness; inflation; bombast; turgidity.</def>  <rj><au>Johnson.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Turgescent</ent><br/
<hw>Tur*ges"cent</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>turgescens</ets>, <ets>-entis</ets>, p. pr. of <ets>turgescere</ets>: cf. F. <ets>turgescent</ets>. See <er>Turgesce</er>.]</ety> <def>Becoming turgid or inflated; swelling; growing big.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Turgid</ent><br/
<hw>Tur"gid</hw> <pr>(t<ucir/r"j<icr/d)</pr>, <pos>a.</pos> <ety>[L. <ets>turgidus</ets>, from <ets>turgere</ets> to swell.]</ety> <sn>1.</sn> <def>Distended beyond the natural state by some internal agent or expansive force; swelled; swollen; bloated; inflated; tumid; -- especially applied to an enlarged part of the body; <as>as, a <ex>turgid</ex> limb; <ex>turgid</ex> fruit</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>A bladder . . . held near the fire grew <qex>turgid</qex>.</q> <rj><qau>Boyle.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Swelling in style or language; vainly ostentatious; bombastic; pompous; <as>as, a <ex>turgid</ex> style of speaking</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p>-- <wordforms><wf>Tur"gid*ly</wf> <pr>(#)</pr>, <pos>adv.</pos> -- <wf>Tur"gid*ness</wf>, <pos>n.</pos></wordforms><br/
[<source>1913 Webster</source>]</p>

<p><ent>Turgidity</ent><br/
<hw>Tur*gid"i*ty</hw> <pr>(?)</pr>, <pos>n.</pos> <def>The quality or state of being turgid.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Turgidous</ent><br/
<hw>Tur"gid*ous</hw> <pr>(t<ucir/r"j<icr/d*<ucr/s)</pr>, <pos>a.</pos> <def>Turgid.</def> <mark>[Obs.]</mark>  <rj><au>B. Jonson.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Turio</ent><br/
||<hw>Tu"ri*o</hw> <pr>(?)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Turiones</plw> <pr>(#)</pr>.</plu> <ety>[L.]</ety> <fld>(Bot.)</fld> <def>A shoot or sprout from the ground.</def>  <rj><au>Gray.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Turiole</ent><br/
<hw>Tu"ri*ole</hw> <pr>(?)</pr>, <pos>n.</pos> <def>The golden oriole.</def> <mark>[Prov. Eng.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Turion</ent><br/
<hw>Tu"ri*on</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Bot.)</fld> <def>Same as <er>Turio</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Turioniferous</ent><br/
<hw>Tu`ri*o*nif"er*ous</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>turio</ets> a sprout + <ets>-ferous</ets>.]</ety> <def>Producing shoots, as asparagus.</def>  <rj><au>Barton.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Turk</ent><br/
<hw>Turk</hw> <pr>(t<ucir/rk)</pr>, <pos>n.</pos> <ety>[Per. <ets>Turk</ets>; probably of Tartar origin: cf. F. <ets>Turc</ets>.]</ety> <sn>1.</sn> <def>A member of any of numerous Tartar tribes of Central Asia, etc.; esp., one of the dominant race in Turkey.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A native or inhabitant of Turkey.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>A Muslim; esp., one living in Turkey.</def> <mark>[Archaic]</mark> <br/
[<source>1913 Webster</source>]</p>

<p><q>It is no good reason for a man's religion that he was born and brought up in it; for then a <qex>Turk</qex> would have as much reason to be a <qex>Turk</qex> as a Christian to be a Christian.</q> <rj><qau>Chillingworth.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <fld>(Zool.)</fld> <def>The plum weevil. See <er>Curculio</er>, and <cref>Plum weevil</cref>, under <er>Plum</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Turk's cap</b></col>. <fld>(Bot.)</fld> <sd>(a)</sd> <cd>Turk's-cap lily. See under <er>Lily</er>.</cd> <sd>(b)</sd> <cd>A tulip.</cd> <sd>(c)</sd> <cd>A plant of the genus <gen>Melocactus</gen>; Turk's head. See <cref>Melon cactus</cref>, under <er>Melon</er>.</cd> -- <col><b>Turk's head</b></col>. <sd>(a)</sd> <fld>(Naut.)</fld> <cd>A knot of turbanlike form worked on a rope with a piece of small line.</cd> <au>R. H. Dana, Jr.</au> <sd>(b)</sd> <fld>(Bot.)</fld> <cd>See <cref>Turk's cap</cref> <sd>(c)</sd> above.</cd> -- <col><b>Turk's turban</b></col> <fld>(Bot.)</fld>, <cd>a plant of the genus <gen>Ranunculus</gen>; crowfoot.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Turkeis</ent><br/
<hw>Tur"keis</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[Cf. <er>Turquoise</er>.]</ety> <def>Turkish.</def> <mark>[Obs.]</mark>  <rj><au>Chaucer.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Turkey</ent><br/
<hw>Tur"key</hw> <pr>(?)</pr>, <pos>pr. n.</pos> <ety>[Cf. 2d <er>Turkey</er>.]</ety> <def>A country in the southeast of Europe and southwest of Asia.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Turkey carpet</b></col>, <cd>a superior kind of carpet made in Asia Minor and adjoining countries, having a deep pile and composed of pure wool with a weft of different material.  It is distinguishable by its coloring and patterns from similar carpets made in India and elsewhere.</cd> -- <col><b>Turkey oak</b></col>. <fld>(Bot.)</fld> <cd>See <er>Cerris</er>.</cd> -- <col><b>Turkey red</b></col>. <sd>(a)</sd> <cd>A brilliant red imparted by madder to cottons, calicoes, etc., the fiber of which has been prepared previously with oil or other fatty matter.</cd> <sd>(b)</sd> <cd>Cloth dyed with this red.</cd> -- <col><b>Turkey sponge</b></col>. <fld>(Zool.)</fld> <cd>See <cref>Toilet sponge</cref>, under <er>Sponge</er>.</cd> -- <col><b>Turkey stone</b></col>, <cd>a kind of oilstone from Turkey; novaculite; -- called also <altname>Turkey oilstone</altname>.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Turkey</ent><br/
<hw>Tur"key</hw> <pr>(t<ucir/r"k<emac/)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Turkeys</plw> <pr>(#)</pr>.</plu> <ety>[So called because it was formerly erroneously believed that it came originally from <ets>Turkey</ets>: cf. F. <ets>Turquie</ets> Turkey. See <er>Turk</er>.]</ety> <fld>(Zool.)</fld> <def>Any large American gallinaceous bird belonging to the genus <gen>Meleagris</gen>, especially the North American wild turkey (<spn>Meleagris gallopavo</spn>), and the domestic turkey, which was probably derived from the Mexican wild turkey, but had been domesticated by the Indians long before the discovery of America.</def><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ The Mexican wild turkey is now considered a variety of the northern species (var. <varn>Mexicana</varn>).  Its tail feathers and coverts are tipped with white instead of brownish chestnut, and its flesh is white. The Central American, or ocellated, turkey (<spn>Meleagris ocellata</spn>) is more elegantly colored than the common species. See under <er>Ocellated</er>.  The Australian, or native, turkey is a bustard (<spn>Choriotis australis</spn>).  See under <er>Native</er>.</note><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Turkey beard</b></col> <fld>(Bot.)</fld>, <cd>a name of certain American perennial liliaceous herbs of the genus <gen>Xerophyllum</gen>. They have a dense tuft of hard, narrowly linear radical leaves, and a long raceme of small whitish flowers. Also called <altname>turkey's beard</altname>.</cd> -- <col><b>Turkey berry</b></col> <fld>(Bot.)</fld>, <cd>a West Indian name for the fruit of certain kinds of nightshade (<spn>Solanum mammosum</spn>, and <spn>Solanum torvum</spn>).</cd> -- <col><b>Turkey bird</b></col> <fld>(Zool.)</fld>, <cd>the wryneck.  So called because it erects and ruffles the feathers of its neck when disturbed.</cd> <mark>[Prov. Eng.]</mark> -- <col><b>Turkey buzzard</b></col> <fld>(Zool.)</fld>, <cd>a black or nearly black buzzard (<spn>Cathartes aura</spn>), abundant in the Southern United States.  It is so called because its naked and warty head and neck resemble those of a turkey.  It is noted for its high and graceful flight. Called also <altname>turkey vulture</altname>.</cd> -- <col><b>Turkey cock</b></col> <fld>(Zool.)</fld>, <cd>a male turkey.</cd> -- <col><b>Turkey hen</b></col> <fld>(Zool.)</fld>, <cd>a female turkey.</cd> -- <col><b>Turkey pout</b></col> <fld>(Zool.)</fld>, <cd>a young turkey.</cd> <mark>[R.]</mark> -- <col><b>Turkey vulture</b></col> <fld>(Zool.)</fld>, <cd>the turkey buzzard.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Turkeys</ent><br/
<hw>Tur"keys</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Turkish.</def> <mark>[Obs.]</mark>  <rj><au>Chaucer.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Turkey-trot</ent><br/
<hw>Tur"key-trot`</hw> <pr>(?)</pr>, <pos>n.</pos> <def>An eccentric ragtime dance, danced with the feet well apart and with a characteristic rise on the ball of the foot, followed by a drop upon the heel. The original form, owing to the positions assumed by the dancers, is offensively suggestive.  Similar dances are the <col><b>bunny hug</b></col> and <col><b>grizzly bear</b></col>, so called in allusion to the movements and the positions assumed by the partners in dancing.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Turkic</ent><br/
<hw>Turk"ic</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Turkish.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Turkis</ent><br/
<hw>Tur"kis</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Min.)</fld> <def>Turquois.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Turkish</ent><br/
<hw>Turk"ish</hw> <pr>(?)</pr>, <pos>pr. a.</pos> <def>Of or pertaining to Turkey or the Turks.</def> -- <def2><pos>n.</pos> <def>The language spoken by Turks, esp. that of the people of Turkey.</def></def2> -- <wordforms><wf>Turk"ish*ly</wf>, <pos>adv.</pos> -- <wf>Turk"ish*ness</wf>, <pos>n.</pos></wordforms><br/
[<source>1913 Webster</source>]</p>

<p><ent>Turkism</ent><br/
<hw>Turk"ism</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A Turkish idiom or expression; also, in general, a Turkish mode or custom.  Same as <er>Turcism</er>.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Turkle</ent><br/
<hw>Tur"kle</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A turtle.</def> <mark>[Obs. or Illiterate]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Turko</ent><br/
<hw>Turk"o</hw> <pr>(?)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Turkos</plw> <pr>(#)</pr>.</plu> <ety>[F. <ets>turco</ets>.]</ety> <def>One of a body of native Algerian tirailleurs in the French army, dressed as a Turk.</def> <altsp>[Written also <asp>Turco</asp>.]</altsp><br/
[<source>1913 Webster</source>]</p>

<p><ent>Turko-Iranian</ent><br/
<hw>Tur`ko-I*ra"ni*an</hw> <pr>(?)</pr>, <pos>pr. a.</pos> <fld>(Ethnol.)</fld> <def>Designating, or pert. to, a mixed racial type including the Afghans, and characterized chiefly by stature above mean, fair complexion, dark, or sometimes gray, eyes, brachycephaly, and very long, prominent, and moderately narrow nose.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Turko-Iranian</ent><br/
<hw>Tur`ko-I*ra"ni*an</hw>, <pos>pr. n.</pos> <def>A member of any race of the Turko-Iranian type.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Turkois</ent><br/
<hw>Tur*kois"</hw> <pr>(?)</pr>, <pos>n. & a.</pos> <def>Turquoise.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Turkoman</ent><br/
<hw>Tur"ko*man</hw> <pr>(?)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Turkomans</plw> <pr>(<?/)</pr>.</plu> <def>Same as <er>Turcoman</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Turk's-head</ent><br/
<hw>Turk's"-head`</hw>, <pos>n.</pos> <sn>1.</sn> <fld>(Naut.)</fld> <def>A knot of turbanlike form  worked on a rope with a piece of small line.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><sn>2.</sn>  <sd>(a)</sd> <def>The melon cactus.</def> <mark>[West Indies]</mark> <sd>(b)</sd> <def>Any of several species of <gen>Echinocactus</gen>.</def> <mark>[California]</mark><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><sn>3.</sn>  <def>A long-handled, round-headed broom for sweeping ceilings, etc.</def> <mark>[Colloq. or Dial.]</mark><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Turlupin</ent><br/
<hw>Tur"lu*pin</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F.]</ety> <fld>(Fr. Eccl. Hist.)</fld> <def>One of the precursors of the Reformation; -- a nickname corresponding to <xex>Lollard</xex>, etc.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Turm</ent><br/
<hw>Turm</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>turma</ets>.]</ety> <def>A troop; a company.</def> <mark>[Obs. or Poetic]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>Legions and cohorts, <qex>turms</qex> of horse and wings.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Turmaline</ent><br/
<hw>Tur"ma*line</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Min.)</fld> <def>See <er>Tourmaline</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Turmeric</ent><br/
<hw>Tur"mer*ic</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F. <ets>terre-m<eacute/rite</ets>, NL. <ets>terramerita</ets>, <ets>turmerica</ets>; apparently meaning, excellent earth, but perhaps a corruption of Ar. <ets>kurkum</ets>.  Cf. <er>Curcuma</er>.]</ety>  <sn>1.</sn> <fld>(Bot.)</fld> <def>An East Indian plant of the genus <gen>Curcuma</gen>, of the Ginger family.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>The root or rootstock of the <spn>Curcuma longa</spn>. It is externally grayish, but internally of a deep, lively yellow or saffron color, and has a slight aromatic smell, and a bitterish, slightly acrid taste. It is used for a dye, a medicine, a condiment, and a chemical test.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Turmeric</ent><br/
<hw>Tur"mer*ic</hw>, <pos>a.</pos> <fld>(Chem.)</fld> <def>Of or pertaining to turmeric; resembling, or obtained from, turmeric; specif., designating an acid obtained by the oxidation of turmerol.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Turmeric paper</b></col> <fld>(Chem.)</fld>, <cd>paper impregnated with turmeric and used as a test for alkaline substances, by which it is changed from yellow to brown.</cd> -- <col><b>Turmeric root</b></col>. <fld>(Bot.)</fld> <sd>(a)</sd> <cd>Bloodroot.</cd> <sd>(b)</sd> <cd>Orangeroot.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Turmerol</ent><br/
<hw>Tur"mer*ol</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[<ets>Turmeric</ets> + L. <ets>ol</ets>eum oil.]</ety> <fld>(Chem.)</fld> <def>Turmeric oil, a brownish yellow, oily substance extracted from turmeric by ligroin.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Turmoil</ent><br/
<hw>Tur"moil</hw> <pr>(t<ucir/r"moil)</pr> <it>sometimes</it> <pr>(t<ucir/r*moil")</pr>, <pos>n.</pos> <ety>[Of uncertain origin; perhaps fr. OF. <ets>tremouille</ets> the hopper of a mill, <ets>trembler</ets> to tremble (cf. E. <ets>tremble</ets>); influenced by E. <ets>turn</ets> and <ets>moil</ets>.]</ety> <def>Harassing labor; trouble; molestation by tumult; disturbance; worrying confusion.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>And there I'll rest, as after much <qex>turmoil</qex>,<br/
A blessed soul doth in Elysium.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><-- p. 1553 pr=PI --><br/
[<source>1913 Webster</source>]</p>

<p><ent>Turmoil</ent><br/
<hw>Tur*moil"</hw> <pr>(t<ucir/r*moil")</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Turmoiled</conjf> <pr>(t<ucir/r*moil"d)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Turmoiling</conjf>.]</vmorph> <def>To harass with commotion; to disquiet; to worry.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>It is her fatal misfortune . . . to be miserably tossed and <qex>turmoiled</qex> with these storms of affliction.</q> <rj><qau>Spenser.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Turmoil</ent><br/
<hw>Tur*moil"</hw>, <pos>v. i.</pos> <def>To be disquieted or confused; to be in commotion.</def> <mark>[Obs.]</mark>  <rj><au>Milton.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Turn</ent><br/
<hw>Turn</hw> <pr>(t<ucir/rn)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Turned</conjf> <pr>(t<ucir/rnd)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Turning</conjf>.]</vmorph> <ety>[OE. <ets>turnen</ets>, <ets>tournen</ets>, OF. <ets>tourner</ets>, <ets>torner</ets>, <ets>turner</ets>, F. <ets>tourner</ets>, LL. <ets>tornare</ets>, fr. L. <ets>tornare</ets> to turn in a lathe, to round off, fr. <ets>tornus</ets> a lathe, Gr. <grk>to`rnos</grk> a turner's chisel, a carpenter's tool for drawing circles; probably akin to E. <ets>throw</ets>. See <er>Throw</er>, and cf. <er>Attorney</er>, <er>Return</er>, <er>Tornado</er>, <er>Tour</er>, <er>Tournament</er>.]</ety> <sn>1.</sn> <def>To cause to move upon a center, or as if upon a center; to give circular motion to; to cause to revolve; to cause to move round, either partially, wholly, or repeatedly; to make to change position so as to present other sides in given directions; to make to face otherwise; <as>as, to <ex>turn</ex> a wheel or a spindle; to <ex>turn</ex> the body or the head</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q><qex>Turn</qex> the adamantine spindle round.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>The monarch <qex>turns</qex> him to his royal guest.</q> <rj><qau>Pope.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To cause to present a different side uppermost or outmost; to make the upper side the lower, or the inside to be the outside of; to reverse the position of; <as>as, to <ex>turn</ex> a box or a board; to <ex>turn</ex> a coat</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>To give another direction, tendency, or inclination to; to direct otherwise; to deflect; to incline differently; -- used both literally and figuratively; <as>as, to <ex>turn</ex> the eyes to the heavens; to <ex>turn</ex> a horse from the road, or a ship from her course; to <ex>turn</ex> the attention to or from something</as>.</def> <ldquo/Expert when to advance, or stand, or, <xex>turn</xex> the sway of battle.<rdquo/  <rj><au>Milton.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Thrice I deluded her, and <qex>turned</qex> to sport<br/
Her importunity.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>My thoughts are <qex>turned</qex> on peace.</q> <rj><qau>Addison.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>To change from a given use or office; to divert, as to another purpose or end; to transfer; to use or employ; to apply; to devote.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Therefore he slew him, and <qex>turned</qex> the kingdom unto David.</q> <rj><qau>1 Chron. x. 14.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>God will make these evils the occasion of a greater good, by <qex>turning</qex> them to advantage in this world.</q> <rj><qau>Tillotson.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>When the passage is open, land will be <qex>turned</qex> most to cattle; when shut, to sheep.</q> <rj><qau>Sir W. Temple.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <def>To change the form, quality, aspect, or effect of; to alter; to metamorphose; to convert; to transform; -- often with <xex>to or into</xex> before the word denoting the effect or product of the change; <as>as, to <ex>turn</ex> a worm into a winged insect; to <ex>turn</ex> green to blue; to <ex>turn</ex> prose into verse; to <ex>turn</ex> a Whig to a Tory, or a Hindu to a Christian; to <ex>turn</ex> good to evil, and the like</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The Lord thy God will <qex>turn</qex> thy captivity, and have compassion upon thee.</q> <rj><qau>Deut. xxx. 3.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>And David said, O Lord, I pray thee, <qex>turn</qex> the counsel of Ahithophel into foolishness.</q> <rj><qau>2 Sam. xv. 31.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Impatience <qex>turns</qex> an ague into a fever.</q> <rj><qau>Jer. Taylor.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>6.</sn> <def>To form in a lathe; to shape or fashion (anything) by applying a cutting tool to it while revolving; <as>as, to <ex>turn</ex> the legs of stools or tables; to <ex>turn</ex> ivory or metal</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>I had rather hear a brazen canstick <qex>turned</qex>.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>7.</sn> <def>Hence, to give form to; to shape; to mold; to put in proper condition; to adapt.</def> <ldquo/The poet's pen <xex>turns</xex> them to shapes.<rdquo/  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>His limbs how <qex>turned</qex>, how broad his shoulders spread !</q> <rj><qau>Pope.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>He was perfectly well <qex>turned</qex> for trade.</q> <rj><qau>Addison.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>8.</sn> <specif>Specifically:</specif> -- <sd>(a)</sd> <def>To translate; to construe; <as>as, to <ex>turn</ex> the Iliad</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Who <qex>turns</qex> a Persian tale for half a crown.</q> <rj><qau>Pope.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sd>(b)</sd> <def>To make acid or sour; to ferment; to curdle, etc.: <as>as, to <ex>turn</ex> cider or wine; electricity <ex>turns</ex> milk quickly</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sd>(c)</sd> <def>To sicken; to nauseate; <as>as, an emetic <ex>turns</ex> one's stomach</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>9.</sn> <def>To make a turn about or around (something); to go or pass around by turning; <as>as, to <ex>turn</ex> a corner</as>.</def></p>

<p><q>The ranges are not high or steep, and one can <qex>turn</qex> a kopje instead of cutting or tunneling through it.</q>  <rj><qau>James Bryce.</qau></rj></p>

<p><cs><col><b>To be turned of</b></col>, <cd>to be advanced beyond; <as>as, <ex>to be turned of</ex> sixty-six</as>.</cd> -- <col><b>To turn a cold shoulder to</b></col>, <cd>to treat with neglect or indifference.</cd> -- <col><b>To turn a corner</b></col>, <sd>(a)</sd> <cd>to go round a corner.</cd>  <sd>(b)</sd> <mark>[Fig.]</mark> <cd>To advance beyond a difficult stage in a project, or in life.</cd>  -- <col><b>To turn adrift</b></col>, <cd>to cast off, to cease to care for.</cd> -- <col><b>To turn a flange</b></col> <fld>(Mech.)</fld>, <cd>to form a flange on, as around a metal sheet or boiler plate, by stretching, bending, and hammering, or rolling the metal.</cd> -- <col><b>To turn against</b></col>. <sd>(a)</sd> <cd>To direct against; <as>as, <ex>to turn</ex> one's arguments <ex>against</ex> himself</as>.</cd> <sd>(b)</sd> <cd>To make unfavorable or hostile to; <as>as, <ex>to turn</ex> one's friends <ex>against</ex> him</as>.</cd> -- <mcol><col><b>To turn a hostile army</b></col>, <col><b>To turn the enemy's flank</b></col>, or the like</mcol> <fld>(Mil.)</fld>, <cd>to pass round it, and take a position behind it or upon its side.</cd> -- <mcol><col><b>To turn a penny</b></col>, <it>or</it> <col><b>To turn an honest penny</b></col></mcol>, <cd>to make a small profit by trade, or the like.</cd> -- <col><b>To turn around one's finger</b></col>, <cd>to have complete control of the will and actions of; to be able to influence at pleasure.</cd> -- <col><b>To turn aside</b></col>, <cd>to avert.</cd> -- <col><b>To turn away</b></col>. <sd>(a)</sd> <cd>To dismiss from service; to discard; <as>as, <ex>to turn away</ex> a servant</as>.</cd> <sd>(b)</sd> <cd>To avert; <as>as, <ex>to turn away</ex> wrath or evil</as>.</cd> -- <col><b>To turn back</b></col>. <sd>(a)</sd> <cd>To give back; to return.</cd><br/
[<source>1913 Webster</source>]</p>

<p><q>We <qex>turn</qex> not <qex>back</qex> the silks upon the merchants,<br/
When we have soiled them.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sd>(b)</sd> <cd>To cause to return or retrace one's steps; hence, to drive away; to repel.</cd> <au>Shak.</au> -- <col><b>To turn down</b></col>. <sd>(a)</sd> <cd>To fold or double down.</cd> <sd>(b)</sd> <cd>To turn over so as to conceal the face of; as, <xex>to turn down</xex> cards.</cd> <sd>(c)</sd> <cd>To lower, or reduce in size, by turning a valve, stopcock, or the like; <as>as, <ex>turn down</ex> the lights</as>.</cd> -- <col><b>To turn in</b></col>. <sd>(a)</sd> <cd>To fold or double under; <as>as, <ex>to turn in</ex> the edge of cloth</as>.</cd> <sd>(b)</sd> <cd>To direct inwards; <as>as, <ex>to turn</ex> the toes <ex>in</ex> when walking</as>.</cd> <sd>(c)</sd> <cd>To contribute; to deliver up; as, he <xex>turned in</xex> a large amount.</cd> <mark>[Colloq.]</mark> -- <col><b>To turn in the mind</b></col>, <cd>to revolve, ponder, or meditate upon; -- with <xex>about</xex>, <xex>over</xex>, etc.</cd>  <ldquo/ <xex>Turn</xex> these ideas about <xex>in</xex> your <xex>mind</xex>.<rdquo/ <au>I. Watts.</au> -- <col><b>To turn off</b></col>. <sd>(a)</sd> <cd>To dismiss contemptuously; as, <xex>to turn off</xex> a sycophant or a parasite.</cd> <sd>(b)</sd> <cd>To give over; to reduce.</cd> <sd>(c)</sd> <cd>To divert; to deflect; as, <xex>to turn off</xex> the thoughts from serious subjects; <xex>to turn off</xex> a joke.</cd> <sd>(d)</sd> <cd>To accomplish; to perform, as work.</cd> <sd>(e)</sd> <fld>(Mech.)</fld> <cd>To remove, as a surface, by the process of turning; to reduce in size by turning.</cd> <sd>(f)</sd> <cd>To shut off, as a fluid, by means of a valve, stopcock, or other device; to stop the passage of; as, <xex>to turn off</xex> the water or the gas.</cd><-- (g) (colloq.) To dampen the enthusiasm of. --> -- <col><b>To turn on</b></col>, <cd>to cause to flow by turning a valve, stopcock, or the like; to give passage to; <as>as, <ex>to turn on</ex> steam</as>.</cd><-- (b) (Colloq.) To make enthusiastic; to arouse sexually. --> -- <col><b>To turn one's coat</b></col>, <cd>to change one's uniform or colors; to go over to the opposite party.</cd> -- <mcol><col><b>To turn one's goods</b></col> <it>or</it> <col><b>To turn one's money</b></col>, and the like</mcol>, <cd>to exchange in the course of trade; to keep in lively exchange or circulation; to gain or increase in trade.</cd> -- <col><b>To turn one's hand to</b></col>, <cd>to adapt or apply one's self to; to engage in.</cd> -- <col><b>To turn out</b></col>. <sd>(a)</sd> <cd>To drive out; to expel; <as>as, <ex>to turn</ex> a family <ex>out</ex> of doors; <ex>to turn</ex> a man <ex>out</ex> of office</as>.</cd><br/
[<source>1913 Webster</source>]</p>

<p><q>I'll <qex>turn</qex> you <qex>out</qex> of my kingdom.</q> <rj><qau> Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sd>(b)</sd> <cd>to put to pasture, as cattle or horses.</cd> <sd>(c)</sd> <cd>To produce, as the result of labor, or any process of manufacture; to furnish in a completed state.</cd> <sd>(d)</sd> <cd>To reverse, as a pocket, bag, etc., so as to bring the inside to the outside; hence, to produce.</cd> <sd>(e)</sd> <cd>To cause to cease, or to put out, by turning a stopcock, valve, or the like; as, <xex>to turn out</xex> the lights.</cd> -- <col><b>To turn over</b></col>. <sd>(a)</sd> <cd>To change or reverse the position of; to overset; to overturn; to cause to roll over.</cd> <sd>(b)</sd> <cd>To transfer; <as>as, <ex>to turn over</ex> business to another hand</as>.</cd> <sd>(c)</sd> <cd>To read or examine, as a book, while, turning the leaves.</cd> <ldquo/We <xex>turned o'er</xex> many books together.<rdquo/ <au>Shak.</au> <sd>(d)</sd> <cd>To handle in business; to do business to the amount of; as, he <xex>turns over</xex> millions a year.</cd> <mark>[Colloq.]</mark> -- <col><b>To turn over a new leaf</b></col>. <cd>See under <er>Leaf</er>.</cd> -- <col><b>To turn tail</b></col>, <cd>to run away; to retreat ignominiously.</cd> -- <col><b>To turn the back</b></col>, <cd>to flee; to retreat.</cd> -- <col><b>To turn the back on</b></col> <it>or</it> <col><b>To turn the back upon</b></col>, <cd>to treat with contempt; to reject or refuse unceremoniously.</cd> -- <col><b>To turn the corner</b></col>, <cd>to pass the critical stage; to get by the worst point; hence, to begin to improve, or to succeed.</cd> -- <mcol><col><b>To turn the die</b></col> <it>or</it> <col><b>To turn the dice</b></col></mcol>, <cd>to change fortune.</cd> -- <mcol><col><b>To turn the edge of</b></col> <it>or</it> <col><b>To turn the point of</b></col></mcol>, <cd>to bend over the edge or point of so as to make dull; to blunt.</cd> -- <mcol><col><b>To turn the head of</b></col> <it>or</it> <col><b>To turn the brain of</b></col></mcol>, <cd>to make giddy, wild, insane, or the like; to infatuate; to overthrow the reason or judgment of; <as>as, a little success <ex>turned his head</ex></as>.</cd> -- <mcol><col><b>To turn the scale</b></col> <it>or</it> <col><b>To turn the balance</b></col></mcol>, <cd>to change the preponderance; to decide or determine something doubtful; to tip the balance.</cd> -- <col><b>To turn the stomach of</b></col>, <cd>to nauseate; to sicken.</cd> -- <col><b>To turn the tables</b></col>, <cd>to reverse the chances or conditions of success or superiority; to give the advantage to the person or side previously at a disadvantage.</cd> -- <col><b>To turn tippet</b></col>, <cd>to make a change.</cd> <mark>[Obs.]</mark> <au>B. Jonson.</au> -- <mcol><col><b>To turn to profit</b></col>, <col><b>To turn to advantage</b></col>, etc.</mcol>, <cd>to make profitable or advantageous.</cd> -- <col><b>To turn turtle</b></col>, <cd>to capsize bottom upward; -- said of a vessel.</cd> <mark>[Naut. slang]</mark> -- <col><b>To turn under</b></col> <fld>(Agric.)</fld>, <cd>to put, as soil, manure, etc., underneath from the surface by plowing, digging, or the like.</cd> -- <col><b>To turn up</b></col>. <sd>(a)</sd> <cd>To turn so as to bring the bottom side on top; as, <xex>to turn up</xex> the trump.</cd> <sd>(b)</sd> <cd>To bring from beneath to the surface, as in plowing, digging, etc.</cd> <sd>(c)</sd> <cd>To give an upward curve to; to tilt; <as>as, <ex>to turn up</ex> the nose</as>.</cd> -- <col><b>To turn upon</b></col>, <cd>to retort; to throw back; <as>as, <ex>to turn</ex> the arguments of an opponent <ex>upon</ex> himself</as>.</cd> -- <col><b>To turn upside down</b></col>, <cd>to confuse by putting things awry; to throw into disorder.</cd><br/
[<source>1913 Webster</source>]</p>

<p><q>This house is <qex>turned upside down</qex> since Robin Ostler died.</q> <rj><qau>Shak.</qau></rj>
</cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Turn</ent><br/
<hw>Turn</hw> <pr>(t<ucir/rn)</pr>, <pos>v. i.</pos> <sn>1.</sn> <def>To move round; to have a circular motion; to revolve entirely, repeatedly, or partially; to change position, so as to face differently; to whirl or wheel round; <as>as, a wheel <ex>turns</ex> on its axis; a spindle <ex>turns</ex> on a pivot; a man <ex>turns</ex> on his heel.</as></def><br/
[<source>1913 Webster</source>]</p>

<p><q>The gate . . . on golden hinges <qex>turning</qex>.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Hence, to revolve as if upon a point of support; to hinge; to depend; <as>as, the decision <ex>turns</ex> on a single fact</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Conditions of peace certainly <qex>turn</qex> upon events of war.</q> <rj><qau>Swift.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>To result or terminate; to come about; to eventuate; to issue.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>If we repent seriously, submit contentedly, and serve him faithfully, afflictions shall <qex>turn</qex> to our advantage.</q> <rj><qau>Wake.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>To be deflected; to take a different direction or tendency; to be directed otherwise; to be differently applied; to be transferred; <as>as, to <ex>turn</ex> from the road</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q><qex>Turn</qex> from thy fierce wrath.</q> <rj><qau>Ex. xxxii. 12.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q><qex>Turn</qex> ye, <qex>turn</qex> ye from your evil ways.</q> <rj><qau>Ezek. xxxiii. 11.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>The understanding <qex>turns</qex> inward on itself, and reflects on its own operations.</q> <rj><qau>Locke.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <def>To be changed, altered, or transformed; to become transmuted; also, to become by a change or changes; to grow; <as>as, wood <ex>turns</ex> to stone; water <ex>turns</ex> to ice; one color <ex>turns</ex> to another; to <ex>turn</ex> Muslim</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>I hope you have no intent to <qex>turn</qex> husband.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Cygnets from gray <qex>turn</qex> white.</q> <rj><qau>Bacon.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>6.</sn> <def>To undergo the process of turning on a lathe; <as>as, ivory <ex>turns</ex> well</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>7.</sn> Specifically: --<br/
[<source>1913 Webster</source>]</p>

<p><sd>(a)</sd> <def>To become acid; to sour; -- said of milk, ale, etc.</def><br/
[<source>1913 Webster</source>]</p>

<p><sd>(b)</sd> <def>To become giddy; -- said of the head or brain.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>I'll look no more;<br/
Lest my brain <qex>turn</qex>.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sd>(c)</sd> <def>To be nauseated; -- said of the stomach.</def><br/
[<source>1913 Webster</source>]</p>

<p><sd>(d)</sd> <def>To become inclined in the other direction; -- said of scales.</def><br/
[<source>1913 Webster</source>]</p>

<p><sd>(e)</sd> <def>To change from ebb to flow, or from flow to ebb; -- said of the tide.</def><br/
[<source>1913 Webster</source>]</p>

<p><sd>(f)</sd> <fld>(Obstetrics)</fld> <def>To bring down the feet of a child in the womb, in order to facilitate delivery.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>8.</sn> <fld>(Print.)</fld> <def>To invert a type of the same thickness, as temporary substitute for any sort which is exhausted.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>To turn about</b></col>, <cd>to face to another quarter; to turn around.</cd> -- <col><b>To turn again</b></col>, <cd>to come back after going; to return.</cd> <au>Shak.</au> -- <col><b>To turn against</b></col>, <cd>to become unfriendly or hostile to.</cd> -- <mcol><col><b>To turn aside</b></col> <it>or</it> <col><b>To turn away</b></col></mcol>. <sd>(a)</sd> <cd>To turn from the direct course; to withdraw from a company; to deviate.</cd> <sd>(b)</sd> <cd>To depart; to remove.</cd> <sd>(c)</sd> <cd>To avert one's face.</cd> -- <col><b>To turn back</b></col>, <cd>to turn so as to go in an opposite direction; to retrace one's steps.</cd> -- <col><b>To turn in</b></col>. <sd>(a)</sd> <cd>To bend inward.</cd> <sd>(b)</sd> <cd>To enter for lodgings or entertainment.</cd> <sd>(c)</sd> <cd>To go to bed.</cd> <mark>[Colloq.]</mark> -- <col><b>To turn into</b></col>, <cd>to enter by making a turn; <as>as, <ex>to turn into</ex> a side street</as>.</cd> -- <col><b>To turn off</b></col>, <cd>to be diverted; to deviate from a course; <as>as, the road <ex>turns off</ex> to the left</as>.</cd> -- <mcol><col><b>To turn on</b></col> <it>or</it> <col><b>To turn upon</b></col></mcol>. <sd>(a)</sd> <cd>To turn against; to confront in hostility or anger.</cd> <sd>(b)</sd> <cd>To reply to or retort.</cd> <sd>(c)</sd> <cd>To depend on; <as>as, the result <ex>turns on</ex> one condition</as>.</cd> -- <col><b>To turn out</b></col>. <sd>(a)</sd> <cd>To move from its place, as a bone.</cd> <sd>(b)</sd> <cd>To bend or point outward; <as>as, his toes <ex>turn out</ex></as>.</cd> <sd>(c)</sd> <cd>To rise from bed.</cd> <mark>[Colloq.]</mark> <sd>(d)</sd> <cd>To come abroad; to appear; <as>as, not many <ex>turned out</ex> to the fire</as>.</cd> <sd>(e)</sd> <cd>To prove in the result; to issue; to result; <as>as, the crops <ex>turned out</ex> poorly</as>.</cd> -- <col><b>To turn over</b></col>, <cd>to turn from side to side; to roll; to tumble.</cd> -- <col><b>To turn round</b></col>. <sd>(a)</sd> <cd>To change position so as to face in another direction.</cd> <sd>(b)</sd> <cd>To change one's opinion; to change from one view or party to another.</cd> -- <col><b>To turn to</b></col>, <cd>to apply one's self to; to have recourse to; to refer to.</cd> <ldquo/Helvicus's tables may be <xex>turned to</xex> on all occasions.<rdquo/ <au>Locke.</au> -- <mcol><col><b>To turn to account</b></col>, <col><b>profit</b></col>, <col><b>advantage</b></col>, or the like</mcol>, <cd>to be made profitable or advantageous; to become worth the while.</cd> -- <col><b>To turn under</b></col>, <cd>to bend, or be folded, downward or under.</cd> -- <col><b>To turn up</b></col>. <sd>(a)</sd> <cd>To bend, or be doubled, upward.</cd> <sd>(b)</sd> <cd>To appear; to come to light; to transpire; to occur; to happen.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Turn</ent><br/
<hw>Turn</hw> <pr>(t<ucir/rn)</pr>, <pos>n.</pos> <sn>1.</sn> <def>The act of turning; movement or motion about, or as if about, a center or axis; revolution; <as>as, the <ex>turn</ex> of a wheel</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Change of direction, course, or tendency; different order, position, or aspect of affairs; alteration; vicissitude; <as>as, the <ex>turn</ex> of the tide</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>At length his complaint took a favorable <qex>turn</qex>.</q> <rj><qau>Macaulay.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>The <qex>turns</qex> and varieties of all passions.</q> <rj><qau>Hooker.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Too well the <qex>turns</qex> of mortal chance I know.</q> <rj><qau>Pope.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>One of the successive portions of a course, or of a series of occurrences, reckoning from change to change; hence, a winding; a bend; a meander.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>And all its [the river's] thousand <qex>turns</qex> disclose.<br/
Some fresher beauty varying round.</q> <rj><qau>Byron.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>A circuitous walk, or a walk to and fro, ending where it began; a short walk; a stroll.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Come, you and I must walk a <qex>turn</qex> together.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>I will take a <qex>turn</qex> in your garden.</q> <rj><qau>Dryden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <def>Successive course; opportunity enjoyed by alternation with another or with others, or in due order; due chance; alternate or incidental occasion; appropriate time.</def> <ldquo/Nobleness and bounty . . . had their <xex>turns</xex> in his [the king's] nature.<rdquo/<br/
[<source>1913 Webster</source>]</p>

<p><q>His <qex>turn</qex> will come to laugh at you again.</q> <rj><qau>Denham.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Every one has a fair <qex>turn</qex> to be as great as he pleases.</q> <rj><qau>Collier.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>6.</sn> <def>Incidental or opportune deed or office; occasional act of kindness or malice; <as>as, to do one an ill <ex>turn</ex></as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Had I not done a friendes <qex>turn</qex> to thee?</q> <rj><qau>Chaucer.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>thanks are half lost when good <qex>turns</qex> are delayed.</q> <rj><qau>Fairfax.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>7.</sn> <def>Convenience; occasion; purpose; exigence; <as>as, this will not serve his <ex>turn</ex></as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>I have enough to serve mine own <qex>turn</qex>.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>8.</sn> <def>Form; cast; shape; manner; fashion; -- used in a literal or figurative sense; hence, form of expression; mode of signifying; <as>as, the <ex>turn</ex> of thought; a man of a sprightly <ex>turn</ex> in conversation</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The <qex>turn</qex> of both his expressions and thoughts is unharmonious.</q> <rj><qau>Dryden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>The Roman poets, in their description of a beautiful man, often mention the <qex>turn</qex> of his neck and arms.</q> <rj><qau>Addison.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>9.</sn> <def>A change of condition; especially, a sudden or recurring symptom of illness, as a nervous shock, or fainting spell; <as>as, a bad <ex>turn</ex></as>.</def> <mark>[Colloq.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><sn>10.</sn> <def>A fall off the ladder at the gallows; a hanging; -- so called from the practice of causing the criminal to stand on a ladder which was turned over, so throwing him off, when the signal was given.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><sn>11.</sn> <def>A round of a rope or cord in order to secure it, as about a pin or a cleat.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>12.</sn> <fld>(Mining)</fld> <def>A pit sunk in some part of a drift.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>13.</sn> <fld>(Eng. Law)</fld> <def>A court of record, held by the sheriff twice a year in every hundred within his county.</def>  <rj><au>Blount.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>14.</sn> <pluf>pl.</pluf> <fld>(Med.)</fld> <def>Monthly courses; menses.</def> <mark>[Colloq.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><sn>15.</sn> <fld>(Mus.)</fld> <def>An embellishment or grace (marked thus, <?/), commonly consisting of the principal note, or that on which the turn is made, with the note above, and the semitone below, the note above being sounded first, the principal note next, and the semitone below last, the three being performed quickly, as a triplet preceding the marked note. The turn may be inverted so as to begin with the lower note, in which case the sign is either placed on end thus <?/, or drawn thus <?/.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>By turns</b></col>. <sd>(a)</sd> <cd>One after another; alternately; in succession.</cd> <sd>(b)</sd> <cd>At intervals.</cd> <ldquo/[They] feel <xex>by turns</xex> the bitter change.<rdquo/ <au>Milton.</au> -- <col><b>In turn</b></col>, <cd>in due order of succession.</cd> -- <col><b>To a turn</b></col>, <cd>exactly; perfectly; <as>as, done <ex>to a turn</ex></as>; -- a phrase alluding to the practice of cooking on a revolving spit.</cd> -- <col><b>To take turns</b></col>, <cd>to alternate; to succeed one another in due order.</cd> -- <col><b>Turn and turn about</b></col>, <cd>by equal alternating periods of service or duty; by turns.</cd> -- <col><b>Turn bench</b></col>, <cd>a simple portable lathe, used on a bench by clock makers and watchmakers.</cd> -- <col><b>Turn buckle</b></col>. <cd>See <er>Turnbuckle</er>, in Vocabulary.</cd> -- <col><b>Turn cap</b></col>, <cd>a sort of chimney cap which turns round with the wind so as to present its opening to the leeward.</cd> <au>G. Francis.</au> -- <col><b>Turn of life</b></col> <fld>(Med.)</fld>, <cd>change of life. See under <er>Change</er>.</cd> -- <col><b>Turn screw</b></col>, <cd>a screw driver.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Turnbroach</ent><br/
<hw>Turn"broach`</hw> <pr>(t<ucir/rn"br<omac/ch`)</pr>, <pos>n.</pos> <def>A turnspit.</def> <mark>[Obs.]</mark> <ldquo/ One that was her <xex>turnbroach</xex>.<rdquo/  <rj><au>Beau. & Fl.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Turn-buckle</ent><br/
<hw>Turn"-buc`kle</hw> <pr>(t<ucir/rn"b<ucr/k`k'l)</pr>, <pos>n.</pos> <fld>(Mech.)</fld> <sd>(a)</sd> <def>A loop or sleeve with a screw thread at one end and a swivel at the other, -- used for tightening a rod, stay, etc.</def> <sd>(b)</sd> <def>A gravitating catch, as for fastening a shutter, the end of a chain, or a hasp.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Turnbull's blue</ent><br/
<hw>Turn"bull's blue`</hw> <pr>(t<ucir/rn"b<usdot/lz bl<umac/`)</pr>. <fld>(Chem.)</fld> <def>The double cyanide of ferrous and ferric iron, a dark blue amorphous substance having a coppery luster, used in dyeing, calico printing, etc.  Cf. <cref>Prussian blue</cref>, under <er>Prussian</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Turncoat</ent><br/
<hw>Turn"coat`</hw> <pr>(t<ucir/rn"k<omac/t`)</pr>, <pos>n.</pos> <def>One who forsakes his party or his principles; a renegade; an apostate; a defector to the enemy.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>He is a <qex>turncoat</qex>, he was not true to his profession.</q> <rj><qau>Bunyan.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Turndown</ent><br/
<hw>Turn"down`</hw> <pr>(t<ucir/rn"doun`)</pr>, <pos>a.</pos> <sn>1.</sn> <def>Capable of being turned down;</def> specif. <fld>(Elec.)</fld>, <def>Designating, or pertaining to, an incandescent lamp with a small additional filament which can be made incandescent when only a small amount of light is required.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><sn>2.</sn>  <def>Made to wear with the upper part turned down; <as>as, a <ex>turndown</ex> collar</as>.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Turndown</ent><br/
<hw>Turn"down`</hw> <pr>(t<ucir/rn"doun`)</pr>, <pos>n.</pos> <sn>1.</sn> <def>An act of refusing or of being refused; <as>as, to get a <ex>turndown</ex> in an application for a job, a grant, etc.</as></def><br/
[<source>PJC</source>]</p>

<p><sn>2.</sn> <def>A decline in activity, especially business; a downturn; <as>as, they expect a <ex>turndown</ex> in housing construction in the fall</as>.</def><br/
[<source>PJC</source>]</p>

<p><ent>Turnep</ent><br/
<hw>Tur"nep</hw> <pr>(t<ucir/r"n<ecr/p)</pr>, <pos>n.</pos> <fld>(Bot.)</fld> <def>See <er>Turnip</er>.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Turner</ent><br/
<hw>Turn"er</hw> <pr>(t<ucir/rn"<etil/r)</pr>, <pos>n.</pos> <sn>1.</sn> <def>One who turns; especially, one whose occupation is to form articles with a lathe.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Zool.)</fld> <def>A variety of pigeon; a tumbler.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Turner</ent><br/
||<hw>Tur"ner</hw> <pr>(t<oomac/r"n<etil/r)</pr>, <pos>n.</pos> <ety>[G.]</ety> <def>A person who practices athletic or gymnastic exercises.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Turnerite</ent><br/
<hw>Tur"ner*ite</hw> <pr>(t<ucir/r"n<etil/r*<imac/t)</pr>, <pos>n.</pos> <ety>[So called from the English chemist and mineralogist, <person>C. H. <etsep>Turner</etsep></person>.]</ety> <fld>(Min.)</fld> <def>A variety of monazite.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Turnery</ent><br/
<hw>Turn"er*y</hw> <pr>(t<ucir/rn"<etil/r*<ycr/)</pr>, <pos>n.</pos> <ety>[Cf. F. <ets>tournerie</ets>.]</ety> <sn>1.</sn> <def>The art of fashioning solid bodies into cylindrical or other forms by means of a lathe.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Things or forms made by a turner, or in the lathe.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Chairs of wood, the seats triangular, the backs, arms, and legs loaded with <qex>turnery</qex>.</q> <rj><qau>Walpole.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><-- p. 1554 pr=PI --><br/
[<source>1913 Webster</source>]</p>

<p><ent>Turney</ent><br/
<hw>Tur"ney</hw> <pr>(t<ucir/r"n<ycr/)</pr>, <pos>n. & v.</pos> <def>Tourney.</def> <mark>[Obs.]</mark> <au>Chaucer.</au> <ldquo/In open <xex>turney</xex>.<rdquo/  <rj><au>Spenser. Milton.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Turnhalle</ent><br/
||<hw>Turn"hal`le</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[G., from <ets>turnen</ets> to exercise gymnastics + <ets>halle</ets> hall.]</ety> <def>A building used as a school of gymnastics.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Turnicimorphae</ent><br/
||<hw>Tur`ni*ci*mor"phae</hw> <pr>(?)</pr>, <pos>pr. n. pl.</pos> <ety>[NL. See <er>Turnix</er>, and <er>-morphous</er>.]</ety> <fld>(Zool.)</fld> <def>A division of birds including <gen>Turnix</gen> and allied genera, resembling quails in appearance but differing from them anatomically.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Turning</ent><br/
<hw>Turn"ing</hw> <pr>(?)</pr>, <pos>n.</pos> <sn>1.</sn> <def>The act of one who, or that which, turns; also, a winding; a bending course; a flexure; a meander.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Through paths and <qex>turnings</qex> often trod by day.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>The place of a turn; an angle or corner, as of a road.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>It is preached at every <qex>turning</qex>.</q> <rj><qau>Coleridge.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>Deviation from the way or proper course.</def>  <rj><au>Harmar.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>Turnery, or the shaping of solid substances into various forms by means of a lathe and cutting tools.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <pluf>pl.</pluf> <def>The pieces, or chips, detached in the process of turning from the material turned; --  usually used in the plural.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>6.</sn> <fld>(Mil.)</fld> <def>A maneuver by which an enemy or a position is turned.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Turning and boring mill</b></col>, <cd>a kind of lathe having a vertical spindle and horizontal face plate, for turning and boring large work.</cd> -- <col><b>Turning bridge</b></col>. <cd>See the Note under <er>Drawbridge</er>.</cd> -- <col><b>Turning engine</b></col>, <cd>an engine lathe.</cd> -- <col><b>Turning lathe</b></col>, <cd>a lathe used by turners to shape their work.</cd> -- <col><b>Turning pair</b></col>. <cd>See the Note under <er>Pair</er>, <pos>n.</pos></cd> -- <col><b>Turning point</b></col>, <cd>the point upon which a question turns, and which decides a case.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Turningness</ent><br/
<hw>Turn"ing*ness</hw>, <pos>n.</pos> <def>The quality of turning; instability; tergiversation.</def> <mark>[Obs.]</mark>  <rj><au>Sir P. Sidney.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Turnip</ent><br/
<hw>Tur"nip</hw> <pr>(t<ucir/r"n<icr/p)</pr>, <pos>n.</pos> <ety>[OE. <ets>turnep</ets>; probably fr. <ets>turn</ets>, or F. <ets>tour</ets> a turn, turning lathe + OE. <ets>nepe</ets> a turnip, AS. <ets>n<aemac/pe</ets>, L. <ets>napus</ets>.  Cf. <er>Turn</er>, <pos>v. t.</pos>, <er>Navew</er>.]</ety> <fld>(Bot.)</fld> <def>The edible, fleshy, roundish, or somewhat conical, root of a cruciferous plant (<spn>Brassica campestris</spn>, <it>var.</it> <varn>Napus</varn>); also, the plant itself.</def> <altsp>[Formerly written also <asp>turnep</asp>.]</altsp><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Swedish turnip</b></col> <fld>(Bot.)</fld>, <cd>a kind of turnip. See <er>Ruta-baga</er>.</cd> -- <col><b>Turnip flea</b></col> <fld>(Zool.)</fld>, <cd>a small flea-beetle (<spn>Haltica, striolata</spn> syn. <spn>Phyllotreta striolata</spn>), which feeds upon the turnip, and often seriously injures it.  It is black with a stripe of yellow on each elytron.  The name is also applied to several other small insects which are injurious to turnips. See <xex>Illust.</xex> under <er>Flea-beetle</er>.</cd> -- <col><b>Turnip fly</b></col>. <fld>(Zool.)</fld> <sd>(a)</sd> <cd>The turnip flea.</cd> <sd>(b)</sd> <cd>A two-winged fly (<spn>Anthomyia radicum</spn>) whose larvae live in the turnip root.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Turnip-shell</ent><br/
<hw>Tur"nip-shell"</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Zool.)</fld> <def>Any one of several large, thick, spiral marine shells belonging to <gen>Rapa</gen> and allied genera, somewhat turnip-shaped.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Turnix</ent><br/
<hw>Tur"nix</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[NL., fr. L. <ets>coturnix</ets> a quail.]</ety> <fld>(Zool.)</fld> <def>Any one of numerous species of birds belonging to <gen>Turnix</gen> or <gen>Hemipodius</gen> and allied genera of the family <fam>Turnicidae</fam>. These birds resemble quails and partridges in general appearance and in some of their habits, but differ in important anatomical characteristics.  The hind toe is usually lacking.  They are found in Asia, Africa, Southern Europe, the East Indian Islands, and esp. in Australia and adjacent islands, where they are called <altname>quails</altname> (see <er>Quail</er>, <pos>n.</pos>, 3.). See <er>Turnicimorphae</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Turnkey</ent><br/
<hw>Turn"key`</hw> <pr>(t<ucir/rn"k<emac/`)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Turnkeys</plw> <pr>(t<ucir/r"k<emac/z`)</pr>.</plu>  <sn>1.</sn> <def>A person who has charge of the keys of a prison, for opening and fastening the doors; a warder.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Dentistry)</fld> <def>An instrument with a hinged claw, -- used for extracting teeth with a twist.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>turnkey</ent><br/
<hw>turn"key`</hw> <pr>(t<ucir/rn"k<emac/`)</pr>, <pos>a.</pos> <def>Of or pertaining to a building, complex device, system, or industrial installation which is sold by a contractor only after it is ready for immediate occupation or use; fully functional and ready for use;  -- used of complex systems of a type which often require preparation or installation by the user before being capable of functioning as intended; <as>as, a <ex>turnkey</ex> ethylene production plant;  a <ex>turnkey</ex> apartment building</as>.</def><br/
[<source>PJC</source>]</p>

<p><ent>Turn-out</ent><br/
<hw>Turn"-out`</hw> <pr>(t<ucir/rn"out`)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Turn-outs</plw> <pr>(t<ucir/rn"outs`)</pr>.</plu> <sn>1.</sn> <def>The act of coming forth; a leaving of houses, shops, etc.; esp., a quitting of employment for the purpose of forcing increase of wages; a strike; -- opposed to <xex>lockout</xex>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A short side track on a railroad, which may be occupied by one train while another is passing on a main track; a shunt; a siding; a switch.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>That which is prominently brought forward or exhibited; hence, an equipage; <as>as, a man with a showy carriage and horses is said to have a fine <ex>turn-out</ex></as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>The aggregate number of persons who have come out, as from their houses, for a special purpose; the number in attendance at a gathering; <as>as, a light <ex>turnout</ex> for the election</as>.</def><br/
[<source>1913 Webster</source> <source>+PJC</source>]</p>

<p><sn>5.</sn> <def>Net quantity of produce yielded.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>6.</sn> <def>A space alongside a highway where vehicles may stop, esp. for emergency purposes, or to admire the view.</def> <br/
[<source>PJC</source>]</p>

<p><ent>Turnover</ent><br/
<hw>Turn"o`ver</hw> <pr>(t<ucir/rn"<omac/`v<etil/r)</pr>, <pos>n.</pos> <sn>1.</sn> <def>The act or result of turning over; an upset; <as>as, a bad <ex>turnover</ex> in a carriage</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A semicircular pie or tart made by turning one half of a circular crust over the other, inclosing the fruit or other materials; <as>as, an apple <ex>turnover</ex></as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>An apprentice, in any trade, who is handed over from one master to another to complete his time.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Turnover</ent><br/
<hw>Turn"o`ver</hw>, <pos>a.</pos> <def>Admitting of being turned over; made to be turned over; <as>as, a <ex>turnover</ex> collar, etc</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Turnpike</ent><br/
<hw>Turn"pike`</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[<ets>Turn</ets> + <ets>pike</ets>.]</ety> <sn>1.</sn> <def>A frame consisting of two bars crossing each other at right angles and turning on a post or pin, to hinder the passage of beasts, but admitting a person to pass between the arms; a turnstile. See <er>Turnstile</er>, 1.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>I move upon my axle like a <qex>turnpike</qex>.</q> <rj><qau>B. Jonson.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A gate or bar set across a road to stop carriages, animals, and sometimes people, till toll is paid for keeping the road in repair; a tollgate.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>A turnpike road.</def>  <rj><au>De Foe.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>A winding stairway.</def> <mark>[Scot.]</mark>  <rj><au>Sir W. Scott.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <fld>(Mil.)</fld> <def>A beam filled with spikes to obstruct passage; a cheval-de-frise.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Turnpike man</b></col>, <cd>a man who collects tolls at a turnpike.</cd> -- <col><b>Turnpike road</b></col>, <cd>a road on which turnpikes, or tollgates, are established by law, in order to collect from the users tolls to defray the cost of building, repairing, etc.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Turnpike</ent><br/
<hw>Turn"pike`</hw> <pr>(?)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Turnpiked</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Turnpiking</conjf>.]</vmorph> <def>To form, as a road, in the manner of a turnpike road; to throw into a rounded form, as the path of a road.</def>  <rj><au>Knowles.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Turnplate</ent><br/
<hw>Turn"plate`</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A turntable.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Turn-sick</ent><br/
<hw>Turn"-sick`</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Giddy.</def> <mark>[Obs.]</mark>  <rj><au>Bacon.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Turn-sick</ent><br/
<hw>Turn"-sick`</hw>, <pos>n.</pos> <fld>(Far.)</fld> <def>A disease with which sheep are sometimes affected; gid; sturdy. See <er>Gid</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Turnsole</ent><br/
<hw>Turn"sole`</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F. <ets>tournesol</ets>, It. <ets>tornasole</ets>; <ets>tornare</ets> to turn (LL. <ets>tornare</ets>) + <ets>sole</ets> the sun, L. <ets>sol</ets>. See <er>Turn</er>, <er>Solar</er>, <pos>a.</pos>, and cf. <er>Heliotrope</er>.]</ety> <altsp>[Written also <asp>turnsol</asp>.]</altsp> <sn>1.</sn> <fld>(Bot.)</fld> <sd>(a)</sd> <def>A plant of the genus <gen>Heliotropium</gen>; heliotrope; -- so named because its flowers are supposed to turn toward the sun.</def> <sd>(b)</sd> <def>The sunflower.</def> <sd>(c)</sd> <def>A kind of spurge (<spn>Euphorbia Helioscopia</spn>).</def> <sd>(d)</sd> <def>The euphorbiaceous plant <spn>Chrozophora tinctoria</spn>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Chem.)</fld> <sd>(a)</sd> <def>Litmus.</def> <mark>[Obs.]</mark> <sd>(b)</sd> <def>A purple dye obtained from the plant turnsole. See def. 1 <sd>(d)</sd>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Turnspit</ent><br/
<hw>Turn"spit`</hw> <pr>(?)</pr>, <pos>n.</pos> <sn>1.</sn> <def>One who turns a spit; hence, a person engaged in some menial office.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>His lordship is his majesty's <qex>turnspit</qex>.</q> <rj><qau>Burke.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Zool.)</fld> <def>A small breed of dogs having a long body and short crooked legs. These dogs were formerly much used for turning a spit on which meat was roasting.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Turnstile</ent><br/
<hw>Turn"stile`</hw> <pr>(?)</pr>, <pos>n.</pos> <sn>1.</sn> <def>A revolving frame in a footpath, preventing the passage of horses or cattle, but admitting that of persons; a turnpike. See <er>Turnpike</er>, <pos>n.</pos>, 1.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A similar arrangement for registering the number of persons passing through a gateway, doorway, or the like.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Turnstone</ent><br/
<hw>Turn"stone`</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Zool.)</fld> <def>Any species of limicoline birds of the genera <gen>Strepsilas</gen> and <gen>Arenaria</gen>, allied to the plovers, especially the common American and European species (<spn>Strepsilas interpres</spn>). They are so called from their habit of turning up small stones in search of mollusks and other aquatic animals. Called also <altname>brant bird</altname>, <altname>sand runner</altname>, <altname>sea quail</altname>, <altname>sea lark</altname>, <altname>sparkback</altname>, and <altname>skirlcrake</altname>.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Black turnstone</b></col>, <cd>the California turnstone (<spn>Arenaria melanocephala</spn>). The adult in summer is mostly black, except some white streaks on the chest and forehead, and two white loral spots.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Turntable</ent><br/
<hw>Turn"ta`ble</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A large revolving platform, for turning railroad cars, locomotives, etc., in a different direction; -- called also <altname>turnplate</altname>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Turnus</ent><br/
||<hw>Tur"nus</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[NL., fr. L. <ets>Turnus</ets>, the king of the Rutuli, mentioned in the Aeneid.]</ety> <fld>(Zool.)</fld> <def>A common, large, handsome, American swallowtail butterfly, now regarded as one of the forms of <spn>Papilio glaucus</spn> syn. <spn>Jasoniades glaucus</spn>.  The wings are yellow, margined and barred with black, and with an orange-red spot near the posterior angle of the hind wings.  Called also <altname>tiger swallowtail</altname>. See <xex>Illust.</xex> under <er>Swallowtail</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Turnverein</ent><br/
||<hw>Turn"ve*rein`</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[G., from <ets>turnen</ets> to exercise + <ets>verein</ets> a union.]</ety> <def>A company or association of gymnasts and athletes.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Turnwrest</ent><br/
<hw>Turn"wrest`</hw> <pr>(t<ucir/rn"r<ecr/st`)</pr>, <pos>n.</pos> <sd>(a)</sd> <def>Designating a cumbersome style of plow used in England, esp. in Kent.</def> <sd>(b)</sd> <def>Designating a kind of hillside plow.</def> <mark>[Eng.]</mark>  <rj><au>Knight.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Turonian</ent><br/
<hw>Tu*ro"ni*an</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Geol.)</fld> <def>One of the subdivisions into which the Upper Cretaceous formation of Europe is divided.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Turpentine</ent><br/
<hw>Tur"pen*tine</hw> <pr>(t<ucir/r"p<ecr/n*t<imac/n)</pr>, <pos>n.</pos> <ety>[F. <ets>t<eacute/r<eacute/benthine</ets>, OF. also <ets>turbentine</ets>; cf. Pr. <ets>terebentina</ets>, <ets>terbentina</ets>, It. <ets>terebentina</ets>, <ets>trementina</ets>; fr. L. <ets>terebinthinus</ets> of the turpentine tree, from <ets>terebinthus</ets> the turpentine tree. Gr. <grk>tere`binqos</grk>, <grk>te`rminqos</grk>. See <er>Terebinth</er>.]</ety> <def>A semifluid or fluid oleoresin, primarily the exudation of the terebinth, or turpentine, tree (<spn>Pistacia Terebinthus</spn>), a native of the Mediterranean region. It is also obtained from many coniferous trees, especially species of pine, larch, and fir.</def><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ There are many varieties of <xex>turpentine</xex>. Chian turpentine is produced in small quantities by the turpentine tree (<spn>Pistacia Terebinthus</spn>). Venice, Swiss, or larch turpentine, is obtained from <spn>Larix Europaea</spn>. It is a clear, colorless balsam, having a tendency to solidify. Canada turpentine, or Canada balsam, is the purest of all the pine turpentines (see under <er>Balsam</er>). The Carpathian and Hungarian varieties are derived from <spn>Pinus Cembra</spn> and <spn>Pinus Mugho</spn>. Carolina turpentine, the most abundant kind, comes from the long-leaved pine (<spn>Pinus palustris</spn>). Strasburg turpentine is from the silver fir (<spn>Abies pectinata</spn>).</note><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Oil of turpentine</b></col> <fld>(Chem.)</fld>, <cd>a colorless oily hydrocarbon, <chform>C10H16</chform>, of a pleasant aromatic odor, obtained by the distillation of crude turpentine. It is used in making varnishes, in medicine, etc. It is the type of the terpenes and is related to cymene. Called also <altname>terebenthene</altname>, <altname>terpene</altname>, etc.</cd> -- <col><b>Turpentine moth</b></col> <fld>(Zool.)</fld>, <cd>any one of several species of small tortricid moths whose larvae eat the tender shoots of pine and fir trees, causing an exudation of pitch or resin.</cd> -- <col><b>Turpentine tree</b></col> <fld>(Bot.)</fld>, <cd>the terebinth tree, the original source of turpentine. See <er>Turpentine</er>, above.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Turpentine State</ent><br/
<hw>Tur"pen*tine State</hw>. <def>North Carolina; -- a nickname alluding to its extensive production of turpentine.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Turpeth</ent><br/
<hw>Tur"peth</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[NL. <ets>turpethum</ets>, fr. Per. <ets>tirbid</ets> a cathartic, <ets>turbad</ets> a purgative root.  Cf. <er>Turbith</er>.]</ety> <altsp>[Written also <asp>turbeth</asp>, and <asp>turbith</asp>.]</altsp> <sn>1.</sn> <fld>(Bot.)</fld> <def>The root of <spn>Ipom<oe/a Turpethum</spn>, a plant of Ceylon, Malabar, and Australia, formerly used in medicine as a purgative; -- sometimes called <altname>vegetable turpeth</altname>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Chem.)</fld> <def>A heavy yellow powder, <chform>Hg3O2SO4</chform>, which consists of a basic mercuric sulphate; -- called also <altname>turpeth mineral</altname>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Turpin</ent><br/
<hw>Tur"pin</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Zool.)</fld> <def>A land tortoise.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Turpitude</ent><br/
<hw>Tur"pi*tude</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>turpitudo</ets>, from <ets>turpis</ets> foul, base.]</ety> <def>Inherent baseness or vileness of principle, words, or actions; shameful wickedness; depravity.</def>  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Turquois</ent><br/
<ent>Turquoise</ent><br/
<mhw>{ <hw>Tur*quoise"</hw>, <hw>Tur*quois"</hw>  }</mhw> <pr>(?; 277)</pr>, <pos>n.</pos> <ety>[F. <ets>turquoise</ets>; cf. Pr. & Sp. <ets>turquesa</ets>, It. <ets>turchese</ets>, <ets>turchina</ets>, LL. <ets>turchesius</ets>, <ets>turchina</ets>; -- so called because first brought from <ets>Turkey</ets>. See 1st <er>Turkey</er>.]</ety> <fld>(Min.)</fld> <def>A hydrous phosphate of alumina containing a little copper; calaite. It has a blue, or bluish green, color, and usually occurs in reniform masses with a botryoidal surface.</def> <altsp>[Formerly written also <asp>turcois</asp>, and <asp>turkois</asp>.]</altsp><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ Turquoise is susceptible of a high polish, and when of a bright blue color is much esteemed as a gem. The finest specimens come from Persia. It is also found in New Mexico and Arizona, and is regarded as identical with the <xex>chalchihuitl</xex> of the Mexicans.</note><br/
[<source>1913 Webster</source>]</p>

<p><ent>Turquoise</ent><br/
<hw>Tur*quoise"</hw>, <pos>a.</pos> <def>Having a fine light blue color, like that of choice mineral turquoise.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Turrel</ent><br/
<hw>Tur"rel</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Cf. OF. <ets>touroul</ets> a little wooden instrument to fasten doors or windows.]</ety> <def>A certain tool used by coopers.</def>  <rj><au>Sherwood.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Turret</ent><br/
<hw>Tur"ret</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[OE. <ets>touret</ets>, OF. <ets>tourette</ets>, dim. of <ets>tour</ets> a tower, L. <ets>turris</ets>. See <er>Tower</er>.]</ety><br/
[<source>1913 Webster</source>]</p>

<p><sn>1.</sn> <fld>(Arch.)</fld> <def>A little tower, frequently a merely ornamental structure at one of the angles of a larger structure.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Anc. Mil.)</fld> <def>A movable building, of a square form, consisting of ten or even twenty stories and sometimes one hundred and twenty cubits high, usually moved on wheels, and employed in approaching a fortified place, for carrying soldiers, engines, ladders, casting bridges, and other necessaries.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <fld>(Mil.)</fld> <def>A revolving tower constructed of thick iron plates, within which cannon are mounted. Turrets are used on vessels of war and on land.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <fld>(Railroads)</fld> <def>The elevated central portion of the roof of a passenger car. Its sides are pierced for light and ventilation.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Turret clock</b></col>, <cd>a large clock adapted for an elevated position, as in the tower of a church.</cd> -- <col><b>Turret head</b></col> <fld>(Mach.)</fld>, <cd>a vertical cylindrical revolving tool holder for bringing different tools into action successively in a machine, as in a lathe.</cd> -- <col><b>Turret lathe</b></col>, <cd>a turning lathe having a turret head.</cd> -- <col><b>Turret ship</b></col>, <cd>an ironclad war vessel, with low sides, on which heavy guns are mounted within one or more iron turrets, which may be rotated, so that the guns may be made to bear in any required direction.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Turret deck</ent><br/
<hw>Tur"ret deck</hw>. <def>A narrow superstructure running from stem to stern on the upper deck of a steam cargo vessel having a rounded gunwale and sides curved inward convexly.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Turreted</ent><br/
<hw>Tur"ret*ed</hw>, <pos>a.</pos> <sn>1.</sn> <def>Furnished with a turret or turrets; specifically <fld>(Zool.)</fld>, having the whorls somewhat flattened on the upper side and often ornamented by spines or tubercles; -- said of certain spiral shells.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Formed like a tower; <as>as, a <ex>turreted</ex> lamp</as>.</def>  <rj><au>Bacon.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Turrethead</ent><br/
<hw>Tur"ret*head</hw>. <def>an attachment fitted to a lathe or other machinery which holds a variety of tools which can act on the object being worked, and which are interchangeable by a pivoting motion, thus allowing efficient performance of multiple operations; -- also called a <altname>turret</altname>.</def><br/
[<source>PJC</source>]</p>

<p><ent>Turret lathe</ent><br/
<hw>Tur"ret lathe</hw>. <def>a lathe fitted with a turrethead.</def><br/
[<source>PJC</source>]</p>

<p><ent>Turret steamer</ent><br/
<hw>Tur"ret steam`er</hw>. <def>A whaleback steamer with a hatch coaming, usually about seven feet high, extending almost continuously fore and aft.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Turribant</ent><br/
<hw>Tur"ri*bant</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[See <er>Turban</er>.]</ety> <def>A turban.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>With hundred turrets like a <qex>turribant</qex>.</q> <rj><qau>Spenser.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Turrical</ent><br/
<hw>Tur"ric*al</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Of or pertaining to a turret, or tower; resembling a tower.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Turriculated</ent><br/
<ent>Turriculate</ent><br/
<mhw>{ <hw>Tur*ric"u*late</hw> <pr>(?)</pr>, <hw>Tur*ric"u*la`ted</hw> <pr>(?)</pr>, }</mhw> <pos>a.</pos> <ety>[L. <ets>turricula</ets> small tower, turret.]</ety> <def>Furnished with, or formed like, a small turret or turrets; somewhat turreted.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Turrilite</ent><br/
<hw>Tur"ri*lite</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>turris</ets> tower + Gr. <?/ stone: cf. F. <ets>turrilite</ets>.]</ety> <fld>(Paleon.)</fld> <def>Any fossil ammonite of the genus <gen>Turrilites</gen>. The shell forms an open spiral with the later whorls separate.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Turritella</ent><br/
<hw>Tur`ri*tel"la</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[NL., dim fr. L. <ets>turris</ets> tower.]</ety> <fld>(Zool.)</fld> <def>Any spiral marine gastropod belonging to <gen>Turritella</gen> and allied genera. These mollusks have an elongated, turreted shell, composed of many whorls. They have a rounded aperture, and a horny multispiral operculum.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Turritelloid</ent><br/
<hw>Tur`ri*tel"loid</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[<ets>Turritella</ets> + <ets>-oid</ets>.]</ety> <fld>(Zool.)</fld> <def>Of, pertaining to, or resembling, the turritellas.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Turtle</ent><br/
<hw>Tur"tle</hw> <pr>(t<ucir/r"t'l)</pr>, <pos>n.</pos> <ety>[AS. <ets>turtle</ets>, L. <ets>turtur</ets>; probably of imitative origin.  Cf. <er>Turtle</er> the sea tortoise.]</ety> <fld>(Zool.)</fld> <def>The turtledove.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Turtle</ent><br/
<hw>Tur"tle</hw>, <pos>n.</pos> <ety>[Probably the same word as the word preceding, and substituted (probably by sailors) for the Spanish or Portuguese name; cf. Sp. <ets>tortuga</ets> tortoise, turtle, Pg. <ets>tartaruga</ets>, also F. <ets>tortue</ets>, and E. <ets>tortoise</ets>.]</ety><br/
[<source>1913 Webster</source>]</p>

<p><-- p. 1555 pr=PI --><br/
[<source>1913 Webster</source>]</p>

<p><sn>1.</sn> <fld>(Zool.)</fld> <def>Any one of the numerous species of Testudinata, especially a sea turtle, or chelonian.</def><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ In the United States the land and fresh-water tortoises are also called turtles.</note><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Printing)</fld> <def>The curved plate in which the form is held in a type-revolving cylinder press.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><mcol><col><b>Alligator turtle</b></col>, <col><b>Box turtle</b></col></mcol>, <cd>etc. See under <er>Alligator</er>, <er>Box</er>, etc.</cd> -- <col><b>green turtle</b></col> <fld>(Zool.)</fld>, <cd>a marine turtle of the genus <gen>Chelonia</gen>, having usually a smooth greenish or olive-colored shell. It is highly valued for the delicacy of its flesh, which is used especially for turtle soup. Two distinct species or varieties are known; one of which (<spn>Chelonia Midas</spn>) inhabits the warm part of the Atlantic Ocean, and sometimes weighs eight hundred pounds or more; the other (<spn>Chelonia virgata</spn>) inhabits the Pacific Ocean. Both species are similar in habits and feed principally on seaweed and other marine plants, especially the turtle grass.</cd> -- <col><b>Turtle cowrie</b></col> <fld>(Zool.)</fld>, <cd>a large, handsome cowrie (<spn>Cypraea testudinaria</spn>); the turtle-shell; so called because of its fancied resemblance to a tortoise in color and form.</cd> -- <col><b>Turtle grass</b></col> <fld>(Bot.)</fld>, <cd>a marine plant (<spn>Thalassia testudinum</spn>) with grasslike leaves, common about the West Indies.</cd> -- <col><b>Turtle shell</b></col>, <cd>tortoise shell. See under <er>Tortoise</er>.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Turtleback</ent><br/
<hw>Tur"tle*back`</hw> <pr>(t<ucir/r"t'l-b<acr/k`)</pr>, <pos>n.</pos> <sn>1.</sn> <fld>(Archaeol.)</fld> <def>A rude stone celt of a form suggesting the back of a turtle.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><sn>2.</sn>  <fld>(Naut.)</fld> <def>A convex deck at the bow or stern of a vessel, so made to shed the seas quickly.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Turtledove</ent><br/
<hw>Tur"tle*dove`</hw> <pr>(t<ucir/r"t'l-d<ucr/v`)</pr>, <pos>n.</pos> <ety>[See 1st <er>Turtle</er>.]</ety><br/
[<source>1913 Webster</source>]</p>

<p><sn>1.</sn> <fld>(Zool.)</fld> <def>Any one of numerous species of pigeons belonging to <gen>Turtur</gen> and allied genera, native of various parts of the Old World; especially, the common European species (<spn>Turtur vulgaris</spn>), which is noted for its plaintive note, affectionate disposition, and devotion to its mate.</def><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ The South African turtledove (<spn>Turtur albiventris</spn>), and the ashy turtledove of India (<spn>Turtur rubicolus</spn>), are similar to the European species in their habits.</note><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Zool.)</fld> <def>Any one of several species of pigeons more or less resembling the true turtledoves, as the American mourning dove (see under <er>Dove</er>), and the Australian turtledove (<spn>Stictopelia cuneata</spn>).</def><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ The turtledove of the Scriptures is probably <xex>Turtur risorius</xex>, a species which is still plentiful in Egypt and other Eastern countries. It is closely allied to the European turtledove.</note><br/
[<source>1913 Webster</source>]</p>

<p><ent>Turtle-footed</ent><br/
<hw>Tur"tle-foot`ed</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Slow-footed.</def> <mark>[R.]</mark> <ldquo/<xex>Turtle-footed</xex> Peace.<rdquo/  <rj><au>Ford.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Turtlehead</ent><br/
<hw>Tur"tle*head`</hw> <pr>(t<ucir/r"t'l-h<ecr/d`)</pr>, <pos>n.</pos> <fld>(Bot.)</fld> <def>An American perennial herb (<spn>Chelone glabra</spn>) having white flowers shaped like the head of a turtle. Called also <altname>snakehead</altname>, <altname>shell flower</altname>, and <altname>balmony</altname>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Turtle peg</ent><br/
<hw>Tur"tle peg</hw>. <def>A sharp steel spear attached to a cord, used in taking sea turtles.</def> -- <wordforms><wf>Turtle pegging</wf>.</wordforms><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Turtler</ent><br/
<hw>Tur"tler</hw> <pr>(t<ucir/r"tl<etil/r)</pr>, <pos>n.</pos> <def>One who catches turtles or tortoises.</def> <ldquo/The Jamaica <xex>turtlers</xex>.<rdquo/  <rj><au>Dampier.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Turtle-shell</ent><br/
<hw>Tur"tle-shell`</hw> <pr>(t<ucir/r"t'l-sh<ecr/l`)</pr>, <pos>n.</pos> <def>The turtle cowrie.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Turtling</ent><br/
<hw>Tur"tling</hw> <pr>(?)</pr>, <pos>n.</pos> <def>The act, practice, or art of catching turtles.</def>  <rj><au>Marryat.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Turves</ent><br/
<hw>Turves</hw> <pr>(t<ucir/rvz)</pr>, <mark>obs.</mark> <def><pos>pl.</pos> of <er>Turf</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tuscan</ent><br/
<hw>Tus"can</hw> <pr>(t<ucr/s"k<ait/n)</pr>, <pos>a.</pos> <ety>[L. <ets>Tuscanus</ets>, <ets>Tuscus</ets>.]</ety> <def>Of or pertaining to Tuscany in Italy; -- specifically designating one of the five orders of architecture recognized and described by the Italian writers of the 16th century, or characteristic of the order. The original of this order was not used by the Greeks, but by the Romans under the Empire. See <er>Order</er>, and <xex>Illust.</xex> of <er>Capital</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tuscan</ent><br/
<hw>Tus"can</hw>, <pos>n.</pos> <def>A native or inhabitant of Tuscany.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tuscaroras</ent><br/
<hw>Tus`ca*ro"ras</hw> <pr>(?)</pr>, <pos>n. pl.</pos>; <sing>sing. <singw>Tuscarora</singw> <pr>(<?/)</pr></sing>. <fld>(Ethnol.)</fld> <def>A tribe of North American Indians formerly living on the Neuse and Tar rivers in North Carolina. They were conquered in 1713, after which the remnant of the tribe joined the Five Nations, thus forming the Six Nations. See <cref>Six Nations</cref>, under <er>Six</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tousche</ent><br/
<ent>Tushe</ent><br/
<ent>Tusche</ent><br/
<mhw>{ ||<hw>Tusch"e</hw> <pr>(?)</pr>, <pos>n.</pos> Also <hw>Tushe</hw>, <hw>Tousche</hw>, etc. }</mhw><ety>[G. <ets>tusche</ets>, fr. F. <ets>toucher</ets> to touch.]</ety> <def>A lithographic drawing or painting material of the same nature as lithographic ink. It is also used as a resistant in the biting-in process.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Tuscor</ent><br/
<hw>Tus"cor</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[See <er>Tusk</er>.]</ety> <def>A tush of a horse.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tush</ent><br/
<hw>Tush</hw> <pr>(t<ucr/sh)</pr>, <pos>interj.</pos> <def>An exclamation indicating check, rebuke, or contempt; <as>as, <ex>tush</ex>, <ex>tush</ex>! do not speak of it</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q><qex>Tush</qex>, say they, how should God perceive it?</q> <rj><qau>Bk. of Com. Prayer (Ps. lxxiii. 11).</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tush</ent><br/
<hw>Tush</hw>, <pos>n.</pos> <ety>[OE. <ets>tusch</ets>, AS. <ets>tusc</ets>; akin to OFries. <ets>tusk</ets>, <ets>tusch</ets>, and probably to AS. <ets>t<omac/<edh/</ets> tooth. See <er>Tooth</er>, and cf. <er>Tusk</er>.]</ety> <def>A long, pointed tooth; a tusk; -- applied especially to certain teeth of horses.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>tush</ent><br/
<hw>tush</hw> <pr>(t<oocr/sh)</pr>, <pos>n.</pos> <ety>[Shortened form of <ets>tushie</ets>.]</ety> <def>The buttocks; -- a euphemism.</def><br/
[<source>PJC</source>]</p>

<p><ent>tushie</ent><br/
<hw>tush"ie</hw> <pr>(t<oocr/sh"<esl/)</pr>, <pos>n.</pos> <ety>[Fr. Yiddish <ets>tokhes</ets>. Ca. <fu>1960-65</fu>.]</ety> <def>The buttocks; -- a euphemism.</def><br/
[<source>PJC</source>]</p>

<p><ent>Tusk</ent><br/
<hw>Tusk</hw> <pr>(t<ucr/sk)</pr>, <pos>n.</pos> <fld>(Zool.)</fld> <def>Same as <er>Torsk</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tusk</ent><br/
<hw>Tusk</hw>, <pos>n.</pos> <ety>[OE. <ets>tusk</ets>, the same word as <ets>tusch</ets>, AS. <ets>tusc</ets>. See <er>Tush</er> a tooth.]</ety> <sn>1.</sn> <fld>(Zool.)</fld> <def>One of the elongated incisor or canine teeth of the wild boar, elephant, etc.; hence, any long, protruding tooth.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Zool.)</fld> <def>A toothshell, or Dentalium; -- called also <altname>tusk-shell</altname>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <fld>(Carp.)</fld> <def>A projecting member like a tenon, and serving the same or a similar purpose, but composed of several steps, or offsets. Thus, in the illustration, <it>a</it> is the <xex>tusk</xex>, and each of the several parts, or offsets, is called a <xex>tooth</xex>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tusk</ent><br/
<hw>Tusk</hw> <pr>(t<ucr/sk)</pr>, <pos>v. i.</pos> <def>To bare or gnash the teeth.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tusked</ent><br/
<hw>Tusked</hw> <pr>(t<ucr/sk)</pr>, <pos>a.</pos> <def>Furnished with tusks.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The <qex>tusked</qex> boar out of the wood.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tusker</ent><br/
<hw>Tusk"er</hw> <pr>(t<ucr/sk"<etil/r)</pr>, <pos>n.</pos> <sn>1.</sn> <fld>(Zool.)</fld> <def>An elephant having large tusks.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Zool.)</fld> <def>A large wild boar.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Tusk-shell</ent><br/
<hw>Tusk"-shell`</hw> <pr>(t<ucr/sk"-sh<ecr/l`)</pr>, <pos>n.</pos> <fld>(Zool.)</fld> <def>See 2d <er>Tusk</er>, <pos>n.</pos>, 2.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tusky</ent><br/
<hw>Tusk"y</hw> <pr>(t<ucr/sk"<ycr/)</pr>, <pos>a.</pos> <def>Having tusks.</def> <ldquo/The scar indented by the <xex>tusky</xex> boar.<rdquo/  <rj><au>Dryden.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tussac grass</ent><br/
<hw>Tus"sac grass`</hw> <pr>(?)</pr>. <def>Tussock grass.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tusseh</ent><br/
<ent>Tussah</ent><br/
<mhw>{ <hw>Tus"sah</hw> <hw>Tus"seh</hw> }</mhw> <pr>(t<ucr/s"s<adot/)</pr>, <pos>n.</pos> <altsp>[Also <asp>tussa</asp>, <asp>tussar</asp>, <asp>tusser</asp>, <asp>tussur</asp>, etc.]</altsp> <ety>[Prob. fr. Hind. <ets>tasar</ets> a shuttle, Skr. <ets>tasara</ets>, <ets>trasara</ets>.]</ety> <def>An undomesticated East Indian silkworn (<spn>Antheraea mylitta</spn>), that feeds on the leaves of the oak and other plants.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Tussah silk</ent><br/
<hw>Tus"sah silk`</hw> <pr>(?)</pr>. <ety>[Probably fr. Hind. <ets>tasar</ets> a shuttle, Skr. <ets>tasara</ets>, <ets>trasara</ets>.]</ety> <sd>(a)</sd> <def>A silk cloth made from the cocoons of a caterpillar other than the common silkworm, much used in Bengal and China.</def> <sd>(b)</sd> <def>The silk fiber itself.</def> <altsp>[Written also <asp>tusseh silk</asp>.]</altsp><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tussal</ent><br/
<hw>Tus"sal</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>tussis</ets> cough.]</ety> <fld>(Med.)</fld> <def>Pertaining to, or manifested by, cough.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Tussicular</ent><br/
<hw>Tus*sic"u*lar</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>tussicularis</ets>, fr. <ets>tussicula</ets>, dim. of <ets>tussis</ets> a cough.]</ety> <def>Of or pertaining to a cough.</def>  <rj><au>Dunglison.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tussis</ent><br/
||<hw>Tus"sis</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L.]</ety> <fld>(Med.)</fld> <def>A cough.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Tussive</ent><br/
<hw>Tus"sive</hw> <pr>(?)</pr>, <pos>a.</pos> <fld>(Med.)</fld> <def>Pertaining to a cough; caused by coughing.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Tussle</ent><br/
<hw>Tus"sle</hw> <pr>(?)</pr>, <pos>v. i. & t.</pos> <ety>[See <er>Tousle</er>.]</ety> <def>To struggle, as in sport; to scuffle; to struggle with.</def> <mark>[Colloq.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tussle</ent><br/
<hw>Tus"sle</hw>, <pos>n.</pos> <def>A struggle; a scuffle.</def> <mark>[Colloq.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tussock</ent><br/
<hw>Tus"sock</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[From <er>Tuz</er>.]</ety> <altsp>[Written also <asp>tussuck</asp>.]</altsp> <sn>1.</sn> <def>A tuft, as of grass, twigs, hair, or the like; especially, a dense tuft or bunch of grass or sedge.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Such laying of the hair in <qex>tussocks</qex> and tufts.</q> <rj><qau>Latimer.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Bot.)</fld> <def>Same as <cref>Tussock grass</cref>, below.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <fld>(Zool.)</fld> <def>A caterpillar of any one of numerous species of bombycid moths. The body of these caterpillars is covered with hairs which form long tufts or brushes. Some species are very injurious to shade and fruit trees. Called also <altname>tussock caterpillar</altname>. See <er>Orgyia</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Tussock grass</b></col>. <fld>(Bot.)</fld> <sd>(a)</sd> <cd>A tall, strong grass of the genus <gen>Dactylis</gen> (<spn>Dactylis caespitosa</spn>), valuable for fodder, introduced into Scotland from the Falkland Islands.</cd> <sd>(b)</sd> <cd>A tufted grass (<spn>Aira caespitosa</spn>).</cd> <sd>(c)</sd> <cd>Any kind of sedge (<gen>Carex</gen>) which forms dense tufts in a wet meadow or boggy place.</cd> -- <col><b>Tussock moth</b></col> <fld>(Zool.)</fld>, <cd>the imago of any tussock caterpillar. They belong to <gen>Orgyia</gen>, <gen>Halecidota</gen>, and allied genera.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tussocky</ent><br/
<hw>Tus"sock*y</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Having the form of tussocks; full of, or covered with, tussocks, or tufts.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tussuck</ent><br/
<hw>Tus"suck</hw> <pr>(?)</pr>, <pos>n.</pos> <def>See <er>Tussock</er>.</def>  <rj><au>Grew.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tut</ent><br/
<hw>Tut</hw> <pr>(t<ucr/t)</pr>, <pos>interj.</pos> <def>Be still; hush; -- an exclamation used for checking or rebuking.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tut</ent><br/
<hw>Tut</hw>, <pos>n.</pos> <ety>[Cf. Sw. <ets>tut</ets> a point, pipe, tube, Dan. <ets>tut</ets> a cornet.]</ety> <sn>1.</sn> <def>An imperial ensign consisting of a golden globe with a cross on it.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A hassock.</def> <mark>[Obs. or Prov. Eng.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tutelage</ent><br/
<hw>Tu"te*lage</hw> <pr>(t<umac/"t<esl/*l<asl/j; 48)</pr>, <pos>n.</pos> <ety>[L. <ets>tutela</ets> protection, fr. <ets>tutus</ets> safe, fr. <ets>tueri</ets> to watch, defend.  Cf. <er>Tuition</er>.]</ety><br/
[<source>1913 Webster</source>]</p>

<p><sn>1.</sn> <def>The act of guarding or protecting; guardianship; protection; <as>as, the king's right of seigniory and <ex>tutelage</ex></as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The childhood of the European nations was passed under the <qex>tutelage</qex> of the clergy.</q> <rj><qau>Macaulay.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>The state of being under a guardian; care or protection enjoyed.</def>  <rj><au>V. Knox.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tutelary</ent><br/
<ent>Tutelar</ent><br/
<mhw>{ <hw>Tu"te*lar</hw> <pr>(?)</pr>, <hw>Tu"te*la*ry</hw> <pr>(?)</pr>, }</mhw> <pos>a.</pos> <ety>[L. <ets>tutelaris</ets>: cf. F. <ets>tut<eacute/laire</ets>. See <er>Tutelage</er>.]</ety> <def>Having the guardianship or charge of protecting a person or a thing; guardian; protecting; <as>as, <ex>tutelary</ex> goddesses</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>This, of all advantages, is the greatest . . . the most <qex>tutelary</qex> of morals.</q> <rj><qau>Landor.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tutele</ent><br/
<hw>Tu"tele</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>tutela</ets>: cf. F. <ets>tutelle</ets>. See <er>Tutelage</er>.]</ety> <def>Tutelage.</def> <mark>[Obs.]</mark>  <rj><au>Howell.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tutenag</ent><br/
<hw>Tu"te*nag</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F. <ets>toutenague</ets>; cf. Pg. <ets>tutenaga</ets>, <ets>tutanaga</ets>. See <er>Tutty</er>.]</ety> <fld>(Metal.)</fld> <sd>(a)</sd> <def>Crude zinc.</def> <mark>[India]</mark> <sd>(b)</sd> <def>Packfong.</def> <altsp>[Written also <asp>tutenague</asp>.]</altsp><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tut-mouthed</ent><br/
<hw>Tut"-mouthed`</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Having a projecting under jaw; prognathous.</def> <mark>[Obs.]</mark>  <rj><au>Holland.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tut-nose</ent><br/
<hw>Tut"-nose`</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A snub nose.</def> <mark>[Prov. Eng.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tutor</ent><br/
<hw>Tu"tor</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[OE. <ets>tutour</ets>, L. <ets>tutor</ets>, fr. <ets>tueri</ets> to watch, defend: cf. F. <ets>tuteur</ets>.  Cf. <er>Tuition</er>.]</ety> <def>One who guards, protects, watches over, or has the care of, some person or thing.</def> Specifically: --<br/
[<source>1913 Webster</source>]</p>

<p><sd>(a)</sd> <def>A treasurer; a keeper.</def> <ldquo/<xex>Tutour</xex> of your treasure.<rdquo/  <rj><au>Piers Plowman.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sd>(b)</sd> <fld>(Civ. Law)</fld> <def>One who has the charge of a child or pupil and his estate; a guardian.</def><br/
[<source>1913 Webster</source>]</p>

<p><sd>(c)</sd> <def>A private or public teacher.</def><br/
[<source>1913 Webster</source>]</p>

<p><sd>(d)</sd> <fld>(Eng. Universities)</fld> <def>An officer or member of some hall, who instructs students, and is responsible for their discipline.</def><br/
[<source>1913 Webster</source>]</p>

<p><sd>(e)</sd> <fld>(Am. Colleges)</fld> <def>An instructor of a lower rank than a professor.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tutor</ent><br/
<hw>Tu"tor</hw>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Tutored</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Tutoring</conjf>.]</vmorph> <sn>1.</sn> <def>To have the guardianship or care of; to teach; to instruct.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Their sons are well <qex>tutored</qex> by you.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To play the tutor toward; to treat with authority or severity.</def>  <rj><au>Addison.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tutorage</ent><br/
<hw>Tu"tor*age</hw> <pr>(?; 48)</pr>, <pos>n.</pos> <def>The office or occupation of a tutor; tutorship; guardianship.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tutoress</ent><br/
<hw>Tu"tor*ess</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A woman who performs the duties of a tutor; an instructress.</def>  <rj><au>E. Moore.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tutorial</ent><br/
<hw>Tu*to"ri*al</hw> <pr>(t<usl/*t<omac/"r<icr/*<ait/l)</pr>, <pos>a.</pos> <ety>[L. <ets>tutorius</ets>.]</ety> <def>Of or pertaining to a tutor; belonging to, or exercised by, a tutor.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>tutorial</ent><br/
<hw>tu*to"ri*al</hw> <pr>(t<usl/*t<omac/r"<esl/*<ait/l)</pr>, <pos>n.</pos> <sn>1.</sn> <def>A class{6} or short series of classes in which one or more instructors provide intensive instruction on some subject to a small group.  Such short courses of instruction may be held at an institution of learning, or in any other place where a small group may desire a brief but thorough introduction to a topic.</def><br/
[<source>PJC</source>]</p>

<p><sn>2.</sn> <specif>Hence:</specif> <fld>(Computers)</fld>  <def>A set of instructions on how to use a particular computer program, built into the software package itself, displayed on the computer screen, and organized in a stepwise manner so as to familiarize a  new user with all of or the most important features of the program.</def><br/
[<source>PJC</source>]</p>

<p><ent>Tutorism</ent><br/
<hw>Tu"tor*ism</hw> <pr>(?)</pr>, <pos>n.</pos> <def>Tutorship.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tutorize</ent><br/
<hw>Tu"tor*ize</hw> <pr>(?)</pr>, <pos>v. t.</pos> <def>To teach; to instruct.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>I . . . shall <qex>tutorize</qex> him some day.</q> <rj><qau>J. H. Newman.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tutorship</ent><br/
<hw>Tu"tor*ship</hw>, <pos>n.</pos> <def>The office, duty, or care of a tutor; guardianship; tutelage.</def>  <rj><au>Hooker.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tutory</ent><br/
<hw>Tu"tor*y</hw> <pr>(?)</pr>, <pos>n.</pos> <def>Tutorage.</def> <mark>[Obs.]</mark>  <rj><au>Holinshed.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tutress</ent><br/
<hw>Tu"tress</hw> <pr>(?)</pr>, <pos>n.</pos> <def>Tutoress.</def> <mark>[Obs.]</mark>  <rj><au>Selden.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tutrix</ent><br/
<hw>Tu"trix</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. See <er>Tutor</er>.]</ety> <def>A female guardian; a tutoress.</def> <mark>[R.]</mark>  <rj><au>Smollett.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tutsan</ent><br/
<hw>Tut"san</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F. <ets>toutesaine</ets>; <ets>tout</ets>, <ets>toute</ets>, all (L. <ets>totus</ets>) + <ets>sain</ets>, <ets>saine</ets>, sound, healthy, L. <ets>sanus</ets>.]</ety> <fld>(Bot.)</fld> <def>A plant of the genus <gen>Hypericum</gen> (<spn>Hypericum Androsoemum</spn>), from which a healing ointment is prepared in Spain; -- called also <altname>parkleaves</altname>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tutti</ent><br/
||<hw>Tut"ti</hw> <pr>(?)</pr>, <pos>n. pl.</pos> <ety>[It., fr. L. <ets>totus</ets>, pl. <ets>toti</ets>, all.]</ety> <fld>(Mus.)</fld> <def>All; -- a direction for all the singers or players to perform together.</def>  <rj><au>Moore (Encyc. of Music).</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tutti-frutti</ent><br/
<hw>Tut"ti-frut`ti</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[It., lit., all fruits.]</ety> <def>A confection of different kinds of preserved fruits.</def> -- <def2><pos>a.</pos>  <def>Flavored with, or containing, various fruits.</def></def2><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Tutty</ent><br/
<hw>Tut"ty</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F. <ets>tutie</ets>; cf. Sp. <ets>tutia</ets>, <ets>atutia</ets>, LL. <ets>tutia</ets>; all from Per. <ets>t<umac/tiy<amac/</ets>.]</ety> <fld>(Chem.)</fld> <def>A yellow or brown amorphous substance obtained as a sublimation product in the flues of smelting furnaces of zinc, and consisting of a crude zinc oxide.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tut-work</ent><br/
<hw>Tut"-work`</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Mining)</fld> <def>Work done by the piece, as in nonmetaliferous rock, the amount done being usually reckoned by the fathom.</def>  <rj><au>Tomlinson.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tut-workman</ent><br/
<hw>Tut"-work`man</hw>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Tut-workmen</plw> <pr>(<?/)</pr>.</plu> <fld>(Mining)</fld> <def>One who does tut-work.</def>  <rj><au>Tomlinson.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tuum</ent><br/
||<hw>Tu"um</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L.]</ety> <def>Lit., thine; that which is thine; -- used in <xex>meum and tuum</xex>. See 2d <er>Meum</er>.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Tu-whoo</ent><br/
<ent>Tu-whit</ent><br/
<mhw>{ <hw>Tu-whit"</hw> <pr>(?)</pr>, <hw>Tu-whoo"</hw> <pr>(?)</pr>, }</mhw> <pos>n. & interj.</pos> <def>Words imitative of the notes of the owl.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Thy <qex>tu-whits</qex> are lulled, I wot,<br/
Thy <qex>tu-whoos</qex> of yesternight.</q> <rj><qau>Tennyson.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>tux</ent><br/
<hw>tux</hw> <def>Shortened form of <er>tuxedo</er>.</def> <mark>[informal]</mark><br/
[<source>PJC</source>]</p>

<p><ent>Tuxedo coat</ent><br/
<ent>Tuxedo</ent><br/
<mhw> <hw>Tux*e"do</hw>, <hw>Tux*e"do coat`</hw></mhw> <pr>(?)</pr>, <pos>n.</pos> <sn>1.</sn> <def>A kind of black jacket for semiformal evening dress made without tails, usually of black or dark blue color and having satin or grosgrain facing on the lapels; -- so named after a fashionable country club at <etsep>Tuxedo</etsep> Park, New York.</def> <mark>[U. S.]</mark> <au>[RHUD]</au><br/
[<source>Webster 1913 Suppl.</source> <source>+PJC</source>]</p>

<p><sn>2.</sn> <def>The complete semiformal evening suit, including the tuxedo jacket, matching trousers, and black bow tie, and usually including a cummerbund; -- the style of shirt worn with this suit varies, and the outfit may include a dickey.</def> <au>[RHUD]</au><br/
[<source>PJC</source>]</p>

<p><ent>Tuyere</ent><br/
||<hw>Tu`y<egrave/re"</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F.; akin to <ets>tuyau</ets> a pipe; of Teutonic origin.  Cf. <er>Tweer</er>, <er>Tewel</er>.]</ety> <def>A nozzle, mouthpiece, or fixture through which the blast is delivered to the interior of a blast furnace, or to the fire of a forge.</def> <altsp>[Corruptly written also <asp>tweer</asp>, and <asp>twier</asp>.]</altsp><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Tuy<egrave/re arch</b></col>, <cd>the embrasure, in the wall of a blast furnace through which the tuy<egrave/re enters.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tuz</ent><br/
<hw>Tuz</hw> <pr>(t<ucr/z)</pr>, <pos>n.</pos> <ety>[Cf. W. <ets>tusw</ets> a wisp, a bunch, <ets>tus</ets> that binds or wraps, <ets>tusiaw</ets> to bind round, to wrap.  Cf. <er>Tussock</er>.]</ety> <def>A lock or tuft of hair.</def> <mark>[Obs.]</mark>  <rj><au>Dryden.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tuza</ent><br/
<hw>Tu"za</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Zool.)</fld> <def>The tucan.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Twaddell's hydrometer</ent><br/
<ent>Twaddell</ent><br/
<mhw>{ <hw>Twad"dell</hw> <pr>(?)</pr>, <pos>n.</pos>, <hw>Twad"dell's hy*drom"e*ter</hw> <pr>(?)</pr> }</mhw>. <ety>[After one <etsep>Twaddell</etsep>, its inventor.]</ety> <def>A form of hydrometer for liquids heavier than water, graduated with an arbitrary scale such that the readings when multiplied by .005 and added to unity give the specific gravity.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Twaddle</ent><br/
<hw>Twad"dle</hw> <pr>(?)</pr>, <pos>v. i. & t.</pos> <ety>[See <er>Twattle</er>.]</ety> <def>To talk in a weak and silly manner, like one whose faculties are decayed; to prate; to prattle.</def>  <rj><au>Stanyhurst.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Twaddle</ent><br/
<hw>Twad"dle</hw>, <pos>n.</pos> <def>Silly talk; gabble; fustian.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>I have put in this chapter on fighting . . . because of the cant and <qex>twaddle</qex> that's talked of boxing and fighting with fists now-a-days.</q> <rj><qau>T. Hughes.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Twaddler</ent><br/
<hw>Twad"dler</hw> <pr>(?)</pr>, <pos>n.</pos> <def>One who prates in a weak and silly manner, like one whose faculties are decayed.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Twaddling</ent><br/
<hw>Twad"dling</hw> <pr>(?)</pr>, <def><pos>a. & n.</pos> from <er>Twaddle</er>, <pos>v.</pos></def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Twaddy</ent><br/
<hw>Twad"dy</hw> <pr>(?)</pr>, <pos>n.</pos> <def>Idle trifling; twaddle.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Twagger</ent><br/
<hw>Twag"ger</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A lamb.</def> <mark>[Prov. Eng.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Twain</ent><br/
<hw>Twain</hw> <pr>(tw<amac/n)</pr>, <pos>a. & n.</pos> <ety>[OE. <ets>twein</ets>, <ets>tweien</ets>, <ets>tweyne</ets>, AS. <ets>tw<emac/gen</ets>, masc. See <er>Two</er>.]</ety> <def>Two; -- nearly obsolete in common discourse, but used in poetry and burlesque.</def> <ldquo/Children <xex>twain</xex>.<rdquo/  <rj><au>Chaucer.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>And whosoever shall compel thee to go a mile, go with him <qex>twain</qex>.</q> <rj><qau>Matt. v. 41.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>In twain</b></col>, <cd>in halves; into two parts; asunder.</cd><br/
[<source>1913 Webster</source>]</p>

<p><q>When old winter split the rocks in <qex>twain</qex>.</q> <rj><qau>Dryden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p>-- <col><b>Twain cloud</b></col>. <fld>(Meteor.)</fld> <cd>Same as <er>Cumulo-stratus</er>.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Twaite</ent><br/
<hw>Twaite</hw> <pr>(tw<amac/t)</pr>, <pos>n.</pos> <ety>[Prov. E.]</ety> <fld>(Zool.)</fld> <def>A European shad; -- called also <altname>twaite shad</altname>. See <er>Shad</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Twaite</ent><br/
<hw>Twaite</hw>, <pos>n.</pos> <fld>(O. Eng. Law)</fld> <def>A piece of cleared ground. See <er>Thwaite</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Twang</ent><br/
<hw>Twang</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A tang. See <er>Tang</er> a state.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Twang</ent><br/
<hw>Twang</hw> <pr>(?)</pr>, <pos>v. i.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Twanged</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Twanging</conjf>.]</vmorph> <ety>[Of imitative origin; cf. <er>Tang</er> a sharp sound, <er>Tinkle</er>.]</ety> <def>To sound with a quick, harsh noise; to make the sound of a tense string pulled and suddenly let go; <as>as, the bowstring <ex>twanged</ex></as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Twang</ent><br/
<hw>Twang</hw>, <pos>v. t.</pos> <def>To make to sound, as by pulling a tense string and letting it go suddenly.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Sounds the tough horn, and <qex>twangs</qex> the quivering string.</q> <rj><qau>Pope.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Twang</ent><br/
<hw>Twang</hw>, <pos>n.</pos> <sn>1.</sn> <def>A harsh, quick sound, like that made by a stretched string when pulled and suddenly let go; <as>as, the <ex>twang</ex> of a bowstring</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>An affected modulation of the voice; a kind of nasal sound.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>He has such a <qex>twang</qex> in his discourse.</q> <rj><qau>Arbuthnot.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Twangle</ent><br/
<hw>Twan"gle</hw> <pr>(?)</pr>, <pos>v. i. & t.</pos> <def>To twang.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>While the <qex>twangling</qex> violin<br/
Struck up with Soldier-laddie.</q> <rj><qau>Tennyson.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Twank</ent><br/
<hw>Twank</hw> <pr>(?)</pr>, <pos>v. t.</pos> <def>To cause to make a sharp twanging sound; to twang, or twangle.</def>  <rj><au>Addison.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Twankay</ent><br/
<hw>Twan"kay</hw> <pr>(?)</pr>, <pos>n.</pos> <def>See Note under <er>Tea</er>, <pos>n.</pos>, 1.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>'T was</ent><br/
<hw>'T was</hw> <pr>(?)</pr>. <def>A contraction of <xex>it was</xex>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Twattle</ent><br/
<hw>Twat"tle</hw> <pr>(?)</pr>, <pos>v. i.</pos> <ety>[Cf. <er>Tattle</er>, <er>Twaddle</er>.]</ety> <def>To prate; to talk much and idly; to gabble; to chatter; to twaddle; <as>as, a <ex>twattling</ex> gossip</as>.</def>  <rj><au>L'Estrange.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Twattle</ent><br/
<hw>Twat"tle</hw>, <pos>v. t.</pos> <def>To make much of, as a domestic animal; to pet.</def> <mark>[Prov. Eng.]</mark>  <rj><au>Grose.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Twattle</ent><br/
<hw>Twat"tle</hw>, <pos>n.</pos> <def>Act of prating; idle talk; twaddle.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Twattler</ent><br/
<hw>Twat"tler</hw> <pr>(?)</pr>, <pos>n.</pos> <def>One who twattles; a twaddler.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tway</ent><br/
<hw>Tway</hw> <pr>(?)</pr>, <pos>a. & n.</pos> <ety>[OE. <ets>twei</ets>. See <er>Twain</er>.]</ety> <def>Two; twain.</def> <mark>[Obs.]</mark>  <rj><au>Spenser.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Twayblade</ent><br/
<hw>Tway"blade`</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Bot.)</fld> <def>Any one of several orchidaceous plants which have only two leaves, as the species of <gen>Listera</gen> and of <gen>Liparis</gen>.</def> <altsp>[Written also <asp>twyblade</asp>.]</altsp><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tweag</ent><br/
<hw>Tweag</hw> <pr>(?)</pr>, <pos>v. t.</pos> <def>To tweak.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tweague</ent><br/
<ent>Tweag</ent><br/
<mhw>{ <hw>Tweag</hw>, <hw>Tweague</hw>, }</mhw> <pos>n.</pos> <def>A pinching condition; perplexity; trouble; distress.</def> <mark>[Obs. or Prov. Eng.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>This put the old fellow in a rare <qex>tweague</qex>.</q> <rj><qau>Arbuthnot.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tweak</ent><br/
<hw>Tweak</hw> <pr>(?)</pr>, <pos>v. t.</pos> <ety>[OE. <ets>twikken</ets>, originally the same word as <ets>twicchen</ets>; cf. LG. <ets>twikken</ets>. See <er>Twitch</er>.]</ety> <def>To pinch and pull with a sudden jerk and twist; to twitch; <as>as, to <ex>tweak</ex> the nose</as>.</def>  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tweak</ent><br/
<hw>Tweak</hw>, <pos>n.</pos> <sn>1.</sn> <def>A sharp pinch or jerk; a twist or twitch; <as>as, a <ex>tweak</ex> of the nose</as>.</def>  <rj><au>Swift.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Trouble; distress; tweag.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>A prostitute.</def> <mark>[Obs.]</mark>  <rj><au>Brathwait.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tweed</ent><br/
<hw>Tweed</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Probably a corruption of <ets>twills</ets>. See <er>Twill</er>.]</ety> <def>A soft and flexible fabric for men's wear, made wholly of wool except in some inferior kinds, the wool being dyed, usually in two colors, before weaving.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tweedle</ent><br/
<hw>Twee"dle</hw> <pr>(?)</pr>, <pos>v. t.</pos> <ety>[Cf. <er>Twiddle</er>.]</ety> <altsp>[Written also <asp>twidle</asp>.]</altsp> <sn>1.</sn> <def>To handle lightly; -- said with reference to awkward fiddling; hence, to influence as if by fiddling; to coax; to allure.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>A fiddler brought in with him a body of lusty young fellows, whom he had <qex>tweedled</qex> into the service.</q> <rj><qau>Addison.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To twist.</def> <mark>[Prov. Eng.]</mark>  <rj><au>Halliwell.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tweedledum and Tweedledee</ent><br/
<hw>Twee"dle*dum` and Twee"dle*dee`</hw> <pr>(?)</pr>. <def>Two things practically alike; -- a phrase coined by <person>John Byrom</person> (1692-1793) in his satire <ldquo/On the Feuds between Handel and Bononcini.<rdquo/</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Tweel</ent><br/
<hw>Tweel</hw> <pr>(?)</pr>, <pos>n. & v.</pos> <def>See <er>Twill</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tweer</ent><br/
<hw>Tweer</hw> <pr>(?)</pr>, <pos>n.</pos> <def>Same as <er>Tuy<egrave/re</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><-- p. 1556 pr=PI --><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tweeze</ent><br/
<ent>Tweese</ent><br/
<mhw>{ <hw>Tweese</hw>, <hw>Tweeze</hw>  }</mhw> <pr>(tw<emac/z)</pr>, <pos>n.</pos> <ety>[OE. <ets>tweeze</ets>, <ets>tweese</ets>, fr. F. <ets><eacute/tuis</ets>, pl. of <ets><eacute/tui</ets> a case, sheath, box; probably of Teutonic origin; cf. MNG. <ets>stuche</ets> a wide sleeve in which articles could be carried, OHG. <ets>st<umac/hha</ets>, G. <ets>stauche</ets> a short and narrow muff. Cf <er>Etui</er>, <er>Tweezers</er>.]</ety> <def>A surgeon's case of instruments.</def>  <rj><au>Howell.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tweezers</ent><br/
<hw>Twee"zers</hw> <pr>(tw<emac/"z<etil/rz)</pr>, <pos>n. pl.</pos> <ety>[See <er>Tweese</er>.]</ety> <def>Small pinchers used to pluck out hairs, and for other purposes.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Twelfth</ent><br/
<hw>Twelfth</hw> <pr>(tw<ecr/lfth)</pr>, <pos>a.</pos> <ety>[For <ets>twelft</ets>, OE. <ets>twelfte</ets>, AS. <ets>twelfta</ets>. See <er>Twelve</er>.]</ety> <sn>1.</sn> <def>Next in order after the eleventh; coming after eleven others; -- the ordinal of <xex>twelve</xex>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Constituting, or being one of, twelve equal parts into which anything is divided.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Twelfth</ent><br/
<hw>Twelfth</hw>, <pos>n.</pos> <sn>1.</sn> <def>The quotient of a unit divided by twelve; one of twelve equal parts of one whole.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>The next in order after the eleventh.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <fld>(Mus.)</fld> <def>An interval comprising an octave and a fifth.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Twelfth-cake</ent><br/
<hw>Twelfth"-cake`</hw> <pr>(-k<amac/k`)</pr>, <pos>n.</pos> <def>An ornamented cake distributed among friends or visitors on the festival of Twelfth-night.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Twelfth-day</ent><br/
<hw>Twelfth"-day`</hw> <pr>(?)</pr>, <pos>n.</pos> <def>See <er>Twelfthtide</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Twelfth-night</ent><br/
<hw>Twelfth"-night`</hw> <pr>(?)</pr>, <pos>n.</pos> <def>The evening of Epiphany, or the twelfth day after Christmas, observed as a festival by various churches.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Twelfth-second</ent><br/
<hw>Twelfth"-sec`ond</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Physics)</fld> <def>A unit for the measurement of small intervals of time, such that 10<exp>12</exp> (ten trillion) of these units make one second.</def><-- now picosecond.  But-- should be thousand trillion, no???  --><br/
[<source>1913 Webster</source>]</p>

<p><ent>Twelfthtide</ent><br/
<hw>Twelfth"tide`</hw> <pr>(?)</pr>, <pos>n.</pos> <def>The twelfth day after Christmas; Epiphany; -- called also <altname>Twelfth-day</altname>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Twelve</ent><br/
<hw>Twelve</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[OE. <ets>twelve</ets>, <ets>twelf</ets>, AS. <ets>twelf</ets>; akin to OFries. <ets>twelf</ets>, <ets>twelef</ets>, <ets>twilif</ets>, OS. <ets>twelif</ets>, D. <ets>twaalf</ets>, G. <ets>zw<oum/lf</ets>, OHG. <ets>zwelif</ets>, Icel. <ets>t<omac/lf</ets>, Sw. <ets>tolf</ets>, Dan. <ets>tolv</ets>, Goth. <ets>twalif</ets>, from the root of E. <ets>two</ets> + the same element as in the second part of E. <ets>eleven</ets>. See <er>Two</er>, and <er>Eleven</er>.]</ety> <def>One more that eleven; two and ten; twice six; a dozen.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Twelve-men's morris</b></col>. <cd>See the Note under <er>Morris</er>.</cd> -- <col><b>Twelve Tables</b></col>. <fld>(Rom. Antiq.)</fld> <cd>See under <er>Table</er>.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Twelve</ent><br/
<hw>Twelve</hw>, <pos>n.</pos> <sn>1.</sn> <def>The number next following eleven; the sum of ten and two, or of twice six; twelve units or objects; a dozen.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A symbol representing twelve units, as 12, or xii.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>The Twelve</b></col> <fld>(Script.)</fld>, <cd>the twelve apostles.</cd>  <rj><au>Matt. xxvi. 20.</au></rj></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Twelvemo</ent><br/
<hw>Twelve"mo</hw> <pr>(?)</pr>, <pos>a. & n.</pos> <def>See <er>Duodecimo</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Twelvemonth</ent><br/
<hw>Twelve"month</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A year which consists of twelve calendar months.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>I shall laugh at this a <qex>twelvemonth</qex> hence.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Twelvepence</ent><br/
<hw>Twelve"pence</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A shilling sterling, being about twenty-four cents.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Twelvepenny</ent><br/
<hw>Twelve"pen*ny</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Sold for a shilling; worth or costing a shilling.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Twelvescore</ent><br/
<hw>Twelve"score`</hw> <pr>(?)</pr>, <pos>n. & a.</pos> <def>Twelve times twenty; two hundred and forty.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Twentieth</ent><br/
<hw>Twen"ti*eth</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[From <er>Twenty</er>; cf. AS. <ets>twentigo<edh/a</ets>. See <er>Twenty</er>.]</ety> <sn>1.</sn> <def>Next in order after the nineteenth; tenth after the tenth; coming after nineteen others; -- the ordinal of <xex>twenty</xex>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Constituting, or being, one of twenty equal parts into which anything is divided.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Twentieth</ent><br/
<hw>Twen"ti*eth</hw>, <pos>n.</pos> <sn>1.</sn> <def>The next in order after the nineteenth; one coming after nineteen others.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>The quotient of a unit divided by twenty; one of twenty equal parts of one whole.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Twenty</ent><br/
<hw>Twen"ty</hw> <pr>(tw<ecr/n"t<ycr/)</pr>, <pos>a.</pos> <ety>[OE. <ets>twenty</ets>, AS. <ets>tw<emac/ntig</ets>, <ets>twentig</ets>; akin to OFries. <ets>twintich</ets>, OS. <ets>tw<emac/ntig</ets>, D. & LG. <ets>twintig</ets>, OHG. <ets>zweinzug</ets>, G. <ets>zwanzig</ets>, Goth. <ets>twai tigjis</ets>; that is, two tens. See <er>Twain</er>, <er>Two</er>, and <er>Ten</er>.]</ety> <sn>1.</sn> <def>One more than nineteen; twice ten; <as>as, <ex>twenty</ex> men</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>An indefinite number more or less that twenty.</def>  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Maximilian, upon <qex>twenty</qex> respects, could not have been the man.</q> <rj><qau>Bacon.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Twenty</ent><br/
<hw>Twen"ty</hw>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Twenties</plw> <pr>(<?/)</pr>.</plu> <sn>1.</sn> <def>The number next following nineteen; the sum of twelve and eight, or twice ten; twenty units or objects; a score.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A symbol representing twenty units, as 20, or xx.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Twentyfold</ent><br/
<hw>Twen"ty*fold`</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Twenty times as many.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Twenty-fourmo</ent><br/
<hw>Twen`ty-four"mo</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Having twenty-four leaves to a sheet; <as>as, a <ex>twenty-fourmo</ex> form, book, leaf, size, etc</as>.</def> -- <def2><pos>n.</pos>  <def>A book composed of sheets, each of which is folded into twenty-four leaves; hence, indicating more or less definitely a size of book whose sheets are so folded; -- usually written 24mo, or 24<deg/.</def></def2><br/
[<source>1913 Webster</source>]</p>

<p><ent>twenty-four-seven</ent><br/
<hw>twenty-four-seven</hw> <pos>adj. & adv.</pos> <pr>(tw<ecr/n"t<emac/f<ocir/r-s<ecr/v"<eit/n)</pr> <ety>[From <ets>24</ets> hours per day, <ets>7</ets> days per week.]</ety>  <def>Without interruption; non-stop; continuous; <as>as, the computer manufacturer provides <ex>24/7</ex> technical help</as>.</def><br/
<syn><b>Syn. --</b> 24/7, 7/24</syn><br/
[<source>PJC</source>]</p>

<p><ent>Twey</ent><br/
<hw>Twey</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[See <er>Two</er>.]</ety> <def>Two.</def> <mark>[Obs.]</mark>  <rj><au>Chaucer.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tweyfold</ent><br/
<hw>Twey"fold`</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Twofold.</def> <mark>[Obs.]</mark>  <rj><au>Chaucer.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Twibil</ent><br/
<hw>Twi"bil</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[AS. <ets>twibill</ets>; <ets>twi-</ets> (in comp.) two + <ets>bill</ets>, <ets>bil</ets>, an ax, hoe, bill. See <er>Twice</er>, and <er>Bill</er> a cutting instrument.]</ety> <sn>1.</sn> <def>A kind of mattock, or ax; esp., a tool like a pickax, but having, instead of the points, flat terminations, one of which is parallel to the handle, the other perpendicular to it.</def> <mark>[Prov. Eng.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A tool for making mortises.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>A reaping hook.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Twibilled</ent><br/
<hw>Twi"billed</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Armed or provided with a twibil or twibils.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Twice</ent><br/
<hw>Twice</hw> <pr>(tw<imac/s)</pr>, <pos>adv.</pos> <ety>[OE. <ets>twies</ets> (where the <it>s</it> is the adverbial ending; see <er>-wards</er>), <ets>twie</ets>, AS. <ets>twiges</ets>, <ets>twiwa</ets>; akin to <ets>twi-</ets> (in comp.) two, G. <ets>zwie-</ets>, OHG. <ets>zwi-</ets>, Icel. <ets>tv<imac/-</ets>, L. <ets>bi-</ets>, Gr. <grk>di-</grk>, Skr. <ets>dvi-</ets>, and E. <ets>two</ets>. See <er>Two</er>.]</ety><br/
[<source>1913 Webster</source>]</p>

<p><sn>1.</sn> <def>Two times; once and again.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>He <qex>twice</qex> essayed to cast his son in gold.</q> <rj><qau>Dryden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Doubly; in twofold quantity or degree; <as>as, <ex>twice</ex> the sum; he is <ex>twice</ex> as fortunate as his neighbor</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/<xex>Twice</xex> is used in the formation of compounds, mostly self-explaining; <as>as, <ex>twice</ex>-born, <ex>twice</ex>-conquered, <ex>twice</ex>-planted, <ex>twice</ex>-told, and the like</as>.</note><br/
[<source>1913 Webster</source>]</p>

<p><ent>Twiddle</ent><br/
<hw>Twid"dle</hw> <pr>(?)</pr>, <pos>v. t.</pos> <ety>[Probably of imitative origin.  Cf. <er>Tweedle</er>.]</ety> <def>To touch lightly, or play with; to tweedle; to twirl; <as>as, to <ex>twiddle</ex> one's thumbs; to <ex>twiddle</ex> a watch key.</as></def> <altsp>[Written also <ex>twidle</ex>.]</altsp>  <rj><au>Thackeray.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Twiddle</ent><br/
<hw>Twid"dle</hw>, <pos>v. i.</pos> <def>To play with anything; hence, to be busy about trifles.</def>  <rj><au>Halliwell.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Twiddle</ent><br/
<hw>Twid"dle</hw> <pr>(?)</pr>, <pos>n.</pos> <sn>1.</sn> <def>A slight twist with the fingers.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A pimple.</def> <mark>[Prov. Eng.]</mark>  <rj><au>Halliwell.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Twifallow</ent><br/
<hw>Twi"fal`low</hw> <pr>(?)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Twifallowed</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Twifallowing</conjf>.]</vmorph> <ety>[AS. <ets>twi-</ets> (see <er>Twice</er>) two + <ets>fallow</ets>.]</ety> <def>To plow, or fallow, a second time (land that has been once fallowed).</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Twifold</ent><br/
<hw>Twi"fold`</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[AS. <ets>twifeald</ets>. See <er>Twice</er>, and cf. <er>Twofold</er>.]</ety> <def>Twofold; double.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Twig</ent><br/
<hw>Twig</hw> <pr>(tw<icr/g)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Twigged</conjf> <pr>(tw<icr/gd)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Twigging</conjf>.]</vmorph> <ety>[Cf. <er>Tweak</er>.]</ety> <def>To twitch; to pull; to tweak.</def> <mark>[Obs. or Scot.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Twig</ent><br/
<hw>Twig</hw>, <pos>v. t.</pos> <ety>[Gael. <ets>tuig</ets>, or Ir. <ets>tuigim</ets> I understand.]</ety><br/
[<source>1913 Webster</source>]</p>

<p><sn>1.</sn> <def>To understand the meaning of; to comprehend; as, do you <xex>twig</xex> me?</def> <mark>[Colloq.]</mark>  <rj><au>Marryat.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To observe slyly; also, to perceive; to discover.</def> <ldquo/Now <xex>twig</xex> him; now mind him.<rdquo/  <rj><au>Foote.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>As if he were looking right into your eyes and <qex>twigged</qex> something there which you had half a mind to conceal.</q> <rj><qau>Hawthorne.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Twig</ent><br/
<hw>Twig</hw>, <pos>n.</pos> <ety>[AS. <ets>twig</ets>; akin to D. <ets>twijg</ets>, OHG. <ets>zw<imac/g</ets>, <ets>zw<imac/</ets>, G. <ets>zweig</ets>, and probably to E. <ets>two</ets>.]</ety> <def>A small shoot or branch of a tree or other plant, of no definite length or size.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The Britons had boats made of willow <qex>twigs</qex>, covered on the outside with hides.</q> <rj><qau>Sir W. Raleigh.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Twig borer</b></col> <fld>(Zool.)</fld>, <cd>any one of several species of small beetles which bore into twigs of shrubs and trees, as the apple-tree <ex>twig borer</ex> (<spn>Amphicerus bicaudatus</spn>).</cd> -- <col><b>Twig girdler</b></col>. <fld>(Zool.)</fld> <cd>See <er>Girdler</er>, 3.</cd> -- <col><b>Twig rush</b></col> <fld>(Bot.)</fld>, <cd>any rushlike plant of the genus <gen>Cladium</gen> having hard, and sometimes prickly-edged, leaves or stalks. See <cref>Saw grass</cref>, under <er>Saw</er>.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Twig</ent><br/
<hw>Twig</hw>, <pos>v. t.</pos> <def>To beat with twigs.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Twiggen</ent><br/
<hw>Twig"gen</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Made of twigs; wicker.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Twigger</ent><br/
<hw>Twig"ger</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A fornicator.</def> <mark>[Eng.]</mark>  <rj><au>Halliwell.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Twiggy</ent><br/
<hw>Twig"gy</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Of or pertaining to a twig or twigs; like a twig or twigs; full of twigs; abounding with shoots.</def> <ldquo/ <xex>Twiggy</xex> trees.<rdquo/  <rj><au>Evelyn.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Twight</ent><br/
<hw>Twight</hw> <pr>(?)</pr>, <pos>v. t.</pos> <def>To twit.</def> <mark>[Obs.]</mark>  <rj><au>Spenser.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Twight</ent><br/
<hw>Twight</hw>, <mark>obs.</mark> <def><pos>p. p.</pos> of <er>Twitch</er>.</def>  <rj><au>Chaucer.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Twighte</ent><br/
<hw>Twight"e</hw> <pr>(?)</pr>, <mark>obs.</mark> <def><pos>imp.</pos> of <er>Twitch</er>.</def>  <rj><au>Chaucer.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Twigless</ent><br/
<hw>Twig"less</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Having no twigs.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Twigsome</ent><br/
<hw>Twig"some</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Full of, or abounding in, twigs; twiggy.</def> <mark>[R.]</mark> <ldquo/ <xex>Twigsome</xex> trees.<rdquo/  <rj><au>Dickens.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Twilight</ent><br/
<hw>Twi"light`</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[OE. <ets>twilight</ets>, AS. <ets>twi-</ets> (see <er>Twice</er>) + <ets>le<oacute/ht</ets> light; hence the sense of doubtful or half light; cf. LG. <ets>twelecht</ets>, G. <ets>zwielicht</ets>. See <er>Light</er>.]</ety><br/
[<source>1913 Webster</source>]</p>

<p><sn>1.</sn> <def>The light perceived before the rising, and after the setting, of the sun, or when the sun is less than 18<deg/ below the horizon, occasioned by the illumination of the earth's atmosphere by the direct rays of the sun and their reflection on the earth.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>faint light; a dubious or uncertain medium through which anything is viewed.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>As when the sun . . . from behind the moon,<br/
In dim eclipse, disastrous <qex>twilight</qex> sheds.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>The <qex>twilight</qex> of probability.</q> <rj><qau>Locke.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Twilight</ent><br/
<hw>Twi"light`</hw>, <pos>a.</pos> <sn>1.</sn> <def>Seen or done by twilight.</def>  <rj><au>Milton.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Imperfectly illuminated; shaded; obscure.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>O'er the <qex>twilight</qex> groves and dusky caves.</q> <rj><qau>Pope.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Twill</ent><br/
<hw>Twill</hw> <pr>(tw<icr/l)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Twilled</conjf> <pr>(tw<icr/l)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Twilling</conjf>.]</vmorph> <ety>[Scotch <ets>tweel</ets>; probably from LG. <ets>twillen</ets> to make double, from <ets>twi-</ets> two; akin to AS. <ets>twi-</ets>, E. <ets>twi-</ets> in <ets>twilight</ets>. See <er>Twice</er>, and cf. <er>Tweed</er>, <er>Tweel</er>.]</ety> <def>To weave, as cloth, so as to produce the appearance of diagonal lines or ribs on the surface.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Twill</ent><br/
<hw>Twill</hw>, <pos>n.</pos> <ety>[Scotch <ets>tweel</ets>. See <er>Twill</er>, <pos>v. t.</pos>]</ety> <sn>1.</sn> <def>An appearance of diagonal lines or ribs produced in textile fabrics by causing the weft threads to pass over one and under two, or over one and under three or more, warp threads, instead of over one and under the next in regular succession, as in plain weaving.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A fabric woven with a twill.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <ety>[Perhaps fr. <ets>quill</ets>.]</ety> <def>A quill, or spool, for yarn.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Twilly</ent><br/
<hw>Twil"ly</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Cf. <er>Willy</er>.]</ety> <def>A machine for cleansing or loosening wool by the action of a revolving cylinder covered with long iron spikes or teeth; a willy or willying machine; -- called also <altname>twilly devil</altname>, and <altname>devil</altname>. See <er>Devil</er>, <pos>n.</pos>, 6, and <er>Willy</er>.</def>  <rj><au>Tomlinson.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Twilt</ent><br/
<hw>Twilt</hw> <pr>(tw<icr/lt)</pr>, <pos>n.</pos> <ety>[See <er>Quilt</er>.]</ety> <def>A quilt.</def> <mark>[Prov. Eng.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Twin</ent><br/
<hw>Twin</hw> <pr>(tw<icr/n)</pr>, <pos>a.</pos> <ety>[OE. <ets>twin</ets> double, AS. <ets>getwinne</ets> two and two, pl., twins; akin to D. <ets>tweeling</ets> a twin, G. <ets>zwilling</ets>, OHG. <ets>zwiniling</ets>, Icel. <ets>tvennr</ets>, <ets>tvinnr</ets>, two and two, twin, and to AS. <ets>twi-</ets> two. See <er>Twice</er>, <er>Two</er>.]</ety> <sn>1.</sn> <def>Being one of two born at a birth; <as>as, a <ex>twin</ex> brother or sister</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Being one of a pair much resembling one another; standing in the relation of a twin to something else; -- often followed by <xex>to</xex> or <xex>with</xex>.</def>  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <fld>(Bot.)</fld> <def>Double; consisting of two similar and corresponding parts.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <fld>(Crystallog.)</fld> <def>Composed of parts united according to some definite law of twinning. See <er>Twin</er>, <pos>n.</pos>, 4.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><mcol><col><b>Twin boat</b></col>, <it>or</it> <col><b>Twin ship</b></col></mcol> <fld>(Naut.)</fld>, <cd>a vessel whose deck and upper works rest on two parallel hulls.</cd> -- <col><b>Twin crystal</b></col>. <cd>See <er>Twin</er>, <pos>n.</pos>, 4.</cd> -- <col><b>Twin flower</b></col> <fld>(Bot.)</fld>, <cd>a delicate evergreen plant (<spn>Linnaea borealis</spn>) of northern climates, which has pretty, fragrant, pendulous flowers borne in pairs on a slender stalk.</cd> -- <col><b>Twin-screw steamer</b></col>, <cd>a steam vessel propelled by two screws, one on either side of the plane of the keel.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Twin</ent><br/
<hw>Twin</hw>, <pos>n.</pos> <sn>1.</sn> <def>One of two produced at a birth, especially by an animal that ordinarily brings forth but one at a birth; -- used chiefly in the plural, and applied to the young of beasts as well as to human young.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <pluf>pl.</pluf> <fld>(Astron.)</fld> <def>A sign and constellation of the zodiac; Gemini. See <er>Gemini</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>A person or thing that closely resembles another.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <fld>(Crystallog.)</fld> <def>A compound crystal composed of two or more crystals, or parts of crystals, in reversed position with reference to each other.</def><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ The relative position of the parts of a twin may be explained by supposing one part to be revolved 180<deg/ about a certain axis (called the <xex>twinning axis</xex>), this axis being normal to a plane (called the <xex>twinning plane</xex>) which is usually one of the fundamental planes of the crystal. This revolution brings the two parts into parallel position, or vice versa. A <xex>contact twin</xex> is one in which the parts are united by a plane surface, called the <xex>composition face</xex>, which is usually the same as the twinning plane. A <xex>penetration twin</xex> is one in which the parts interpenetrate each other, often very irregularly. Twins are also called, according to form, <xex>cruciform</xex>, <xex>geniculated</xex>, etc.</note><br/
[<source>1913 Webster</source>]</p>

<p><ent>Twin</ent><br/
<hw>Twin</hw> <pr>(?)</pr>, <pos>v. i.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Twinned</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Twinning</conjf>.]</vmorph> <sn>1.</sn> <def>To bring forth twins.</def>  <rj><au>Tusser.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To be born at the same birth.</def>  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Twin</ent><br/
<hw>Twin</hw>, <pos>v. t.</pos> <sn>1.</sn> <def>To cause to be twins, or like twins in any way.</def>  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Still we moved<br/
Together, <qex>twinned</qex>, as horse's ear and eye.</q> <rj><qau>Tennyson.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To separate into two parts; to part; to divide; hence, to remove; also, to strip; to rob.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>The life out of her body for to <qex>twin</qex>.</q> <rj><qau>Chaucer.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Twin</ent><br/
<hw>Twin</hw>, <pos>v. i.</pos> <def>To depart from a place or thing.</def> <mark>[Obs.]</mark> <ldquo/Ere that we farther <xex>twin</xex>.<rdquo/  <rj><au>Chaucer.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Twinborn</ent><br/
<hw>Twin"born`</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Born at the same birth.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Twine</ent><br/
<hw>Twine</hw> <pr>(tw<imac/n)</pr>, <pos>n.</pos> <ety>[AS. <ets>tw<imac/n</ets>, properly, a twisted or double thread; akin to D. <ets>twijn</ets>, Icel. <ets>tvinni</ets>; from <ets>twi-</ets>. See <er>Twice</er>, and cf. <er>Twin</er>.]</ety> <sn>1.</sn> <def>A twist; a convolution.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Typhon huge, ending in snaky <qex>twine</qex>.</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A strong thread composed of two or three smaller threads or strands twisted together, and used for various purposes, as for binding small parcels, making nets, and the like; a small cord or string.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>The act of twining or winding round.</def>  <rj><au>J. Philips.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Twine reeler</b></col>, <cd>a kind of machine for twisting twine; a kind of mule, or spinning machine.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Twine</ent><br/
<hw>Twine</hw>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Twined</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Twining</conjf>.]</vmorph> <ety>[OE. <ets>twinen</ets>, fr. AS. <ets>tw<imac/n</ets> a twisted thread; akin to D. <ets>twijnen</ets> to twine, Icel. & Sw. <ets>tvinna</ets>, Dan. <ets>tvinde</ets>. See <er>Twine</er>, <pos>n.</pos>]</ety> <sn>1.</sn> <def>To twist together; to form by twisting or winding of threads; to wreathe; <as>as, fine <ex>twined</ex> linen</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To wind, as one thread around another, or as any flexible substance around another body.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Let me <qex>twine</qex><br/
Mine arms about that body.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>To wind about; to embrace; to entwine.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Let wreaths of triumph now my temples <qex>twine</qex>.</q> <rj><qau>Pope.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>To change the direction of.</def> <mark>[Obs.]</mark>  <rj><au>Fairfax.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <def>To mingle; to mix.</def> <mark>[Obs.]</mark>  <rj><au>Crashaw.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Twine</ent><br/
<hw>Twine</hw>, <pos>v. i.</pos> <sn>1.</sn> <def>To mutually twist together; to become mutually involved.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To wind; to bend; to make turns; to meander.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>As rivers, though they bend and <qex>twine</qex>,<br/
Still to the sea their course incline.</q> <rj><qau>Swift.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>To turn round; to revolve.</def> <mark>[Obs.]</mark>  <rj><au>Chapman.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>To ascend in spiral lines about a support; to climb spirally; <as>as, many plants <ex>twine</ex></as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Twiner</ent><br/
<hw>Twin"er</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Bot.)</fld> <def>Any plant which twines about a support.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Twinge</ent><br/
<hw>Twinge</hw> <pr>(tw<icr/nj)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Twinged</conjf> <pr>(tw<icr/njd)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Twinging</conjf>.]</vmorph> <ety>[OE. <ets>twengen</ets>, AS. <ets>twengan</ets>; akin to OE. <ets>twingen</ets> to pain, afflict, OFries. <ets>thwinga</ets>, <ets>twinga</ets>, <ets>dwinga</ets>, to constrain, D. <ets>dwingen</ets>, OS. <ets>thwingan</ets>, G. <ets>zwingen</ets>, OHG. <ets>dwingan</ets>, <ets>thwingan</ets>, to press, oppress, overcome, Icel. <ets><thorn/vinga</ets>, Sw. <ets>tvinga</ets> to subdue, constrain, Dan. <ets>tvinge</ets>, and AS. <ets><thorn/<uum/n</ets> to press, OHG. <ets>d<umac/hen</ets>, and probably to E. <ets>thong</ets>. Perhaps influenced by <ets>twitch</ets>.  Cf. <er>Thong</er>.]</ety> <sn>1.</sn> <def>To pull with a twitch; to pinch; to tweak.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>When a man is past his sense,<br/
There's no way to reduce him thence,<br/
But <qex>twinging</qex> him by the ears or nose,<br/
Or laying on of heavy blows.</q> <rj><qau>Hudibras.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To affect with a sharp, sudden pain; to torment with pinching or sharp pains.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The gnat . . . <qex>twinged</qex> him [the lion] till he made him tear<br/
himself, and so mastered him.</q> <rj><qau>L'Estrange.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Twinge</ent><br/
<hw>Twinge</hw>, <pos>v. i.</pos> <def>To have a sudden, sharp, local pain, like a twitch; to suffer a keen, darting, or shooting pain; <as>as, the side <ex>twinges</ex></as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Twinge</ent><br/
<hw>Twinge</hw>, <pos>n.</pos> <sn>1.</sn> <def>A pinch; a tweak; a twitch.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>A master that gives you . . . <qex>twinges</qex> by the ears.</q> <rj><qau>L' Estrange.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A sudden sharp pain; a darting local pain of momentary continuance; <as>as, a <ex>twinge</ex> in the arm or side</as>.</def> <ldquo/ A <xex>twinge</xex> for my own sin.<rdquo/  <rj><au>Dryden.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Twining</ent><br/
<hw>Twin"ing</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Winding around something; twisting; embracing; climbing by winding about a support; <as>as, the hop is a <ex>twining</ex> plant</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Twining</ent><br/
<hw>Twin"ing</hw>, <pos>a.</pos> <def>The act of one who, or that which, twines; <fld>(Bot.)</fld> the act of climbing spirally.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Twink</ent><br/
<hw>Twink</hw> <pr>(tw<icr/<nsm/k)</pr>, <pos>v. i.</pos> <ety>[OE. <ets>twinken</ets>. See <er>Twinkle</er>.]</ety> <def>To twinkle.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Twink</ent><br/
<hw>Twink</hw> <pr>(tw<icr/<nsm/k)</pr>, <pos>n.</pos> <sn>1.</sn> <def>A wink; a twinkling.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Zool.)</fld> <def>The chaffinch.</def> <mark>[Prov. Eng.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Twinkle</ent><br/
<hw>Twin"kle</hw> <pr>(tw<icr/<nsm/"k'l)</pr>, <pos>v. i.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Twinkled</conjf> <pr>(tw<icr/<nsm/"k'ld)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Twinkling</conjf> <pr>(tw<icr/<nsm/"kl<icr/ng)</pr>.]</vmorph> <ety>[OE. <ets>twinklen</ets>, AS. <ets>twinclian</ets>; akin to OE. <ets>twinken</ets> to blink, wink, G. <ets>zwinken</ets>, <ets>zwinkern</ets>, and perhaps to E. <ets>twitch</ets>.]</ety> <sn>1.</sn> <def>To open and shut the eye rapidly; to blink; to wink.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The owl fell a moping and <qex>twinkling</qex>.</q> <rj><qau>L' Estrange.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><-- p. 1557 pr=PI --><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To shine with an intermitted or a broken, quavering light; to flash at intervals; to sparkle; to scintillate.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>These stars do not <qex>twinkle</qex> when viewed through telescopes that have large apertures.</q> <rj><qau>Sir I. Newton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>The western sky <qex>twinkled</qex> with stars.</q> <rj><qau>Sir W. Scott.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Twinkle</ent><br/
<hw>Twin"kle</hw> <pr>(?)</pr>, <pos>n.</pos> <sn>1.</sn> <def>A closing or opening, or a quick motion, of the eye; a wink or sparkle of the eye.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Suddenly, with <qex>twinkle</qex> of her eye,<br/
The damsel broke his misintended dart.</q> <rj><qau>Spenser.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A brief flash or gleam, esp. when rapidly repeated.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>The time of a wink; a twinkling.</def>  <rj><au>Dryden.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Twinkler</ent><br/
<hw>Twin"kler</hw> <pr>(?)</pr>, <pos>n.</pos> <def>One who, or that which, twinkles, or winks; a winker; an eye.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Twinkling</ent><br/
<hw>Twin"kling</hw> <pr>(?)</pr>, <pos>n.</pos> <sn>1.</sn> <def>The act of one who, or of that which, twinkles; a quick movement of the eye; a wink; a twinkle.</def>  <rj><au>Holland.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A shining with intermitted light; a scintillation; a sparkling; <as>as, the <ex>twinkling</ex> of the stars</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>The time of a wink; a moment; an instant.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>In a moment, in the <qex>twinkling</qex> of an eye, at the last trump, . . . the dead shall be raised incorruptible.</q> <rj><qau>1 Cor. xv. 52.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Twinleaf</ent><br/
<hw>Twin"leaf`</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Bot.)</fld> <def>See <er>Jeffersonia</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Twinlike</ent><br/
<hw>Twin"like`</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Closely resembling; being a counterpart.</def> -- <wordforms><wf>Twin"like`ness</wf>, <pos>n.</pos></wordforms><br/
[<source>1913 Webster</source>]</p>

<p><ent>Twinling</ent><br/
<hw>Twin"ling</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[<ets>Twin</ets> + 1st <ets>-ling</ets>.]</ety> <def>A young or little twin, especially a twin lamb.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Twinned</ent><br/
<hw>Twinned</hw> <pr>(?)</pr>, <pos>a.</pos> <fld>(Crystallog.)</fld> <def>Composed of parts united according to a law of twinning. See <er>Twin</er>, <pos>n.</pos>, 4.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Twinner</ent><br/
<hw>Twin"ner</hw> <pr>(?)</pr>, <pos>n.</pos> <def>One who gives birth to twins; a breeder of twins.</def>  <rj><au>Tusser.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Twinning</ent><br/
<hw>Twin"ning</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Crystallog.)</fld> <def>The assemblage of two or more crystals, or parts of crystals, in reversed position with reference to each other in accordance with some definite law; also, rarely, in artificial twinning (accomplished for example by pressure), the process by which this reversal is brought about.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Polysynthetic twinning</b></col>, <cd>repeated twinning of crystal lamellae, as that of the triclinic feldspars.</cd> -- <col><b>Repeated twinning</b></col>, <cd>twinning of more than two crystals, or parts of crystals.</cd> -- <mcol><col><b>Twinning axis</b></col>, <col><b>Twinning plane</b></col></mcol>. <cd>See the Note under <er>Twin</er>, <pos>n.</pos></cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Twinter</ent><br/
<hw>Twin"ter</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[AS. <ets>twi-</ets> two + <ets>winter</ets> winter. See <er>Twice</er>, and <er>Winter</er>.]</ety> <def>A domestic animal two winters old.</def> <mark>[Prov. Eng.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Twire</ent><br/
<hw>Twire</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Cf. D. <ets>tweern</ets>, G. <ets>zwirn</ets>, and E. <ets>twine</ets>.]</ety> <def>A twisted filament; a thread.</def> <mark>[Obs.]</mark>  <rj><au>Locke.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Twire</ent><br/
<hw>Twire</hw>, <pos>v. i.</pos> <ety>[Cf. MHG. <ets>zwieren</ets>, E. <ets>thwart</ets>, and <ets>queer</ets>.]</ety> <sn>1.</sn> <def>To peep; to glance obliquely; to leer.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>Which maids will <qex>twire</qex> at 'tween their fingers.</q> <rj><qau>B. Jonson.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>I saw the wench that <qex>twired</qex> and twinkled at thee.</q> <rj><qau>Beau. & Fl.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To twinkle; to glance; to gleam.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>When sparkling stars <qex>twire</qex> not.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Twire</ent><br/
<hw>Twire</hw>, <pos>v. i.</pos> <def>To sing, or twitter.</def> <mark>[Obs.]</mark>  <rj><au>Chaucer.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Twire-pipe</ent><br/
<hw>Twire"-pipe`</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A vagabond musician.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>You are an ass, a <qex>twire-pipe</qex>.</q> <rj><qau>Beau. & Fl.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>You looked like <qex>Twire-pipe</qex>, the taborer.</q> <rj><qau>Chapman.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Twirl</ent><br/
<hw>Twirl</hw> <pr>(tw<etil/rl)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Twirled</conjf> <pr>(tw<etil/rld)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Twirling</conjf>.]</vmorph> <ety>[Cf. AS. <ets><thorn/wiril</ets> a churn staff, a stirrer, flail, <ets><thorn/weran</ets>, <ets><amac/<thorn/weran</ets>, to agitate, twirl, G. <ets>zwirlen</ets>, <ets>quirlen</ets>, <ets>querlen</ets> to twirl, to turn round or about, <ets>quirl</ets> a twirling stick, OHG. <ets>dweran</ets> to twirl, stir.  Cf. <er>Trowel</er>.]</ety>  <def>To move or turn round rapidly; to whirl round; to move and turn rapidly with the fingers.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>See ruddy maids,<br/
Some taught with dexterous hand to <qex>twirl</qex> the wheel.</q> <rj><qau>Dodsley.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>No more beneath soft eve's consenting star<br/
Fandango <qex>twirls</qex> his jocund castanet.</q> <rj><qau>Byron.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Twirl</ent><br/
<hw>Twirl</hw>, <pos>v. i.</pos> <def>To revolve with velocity; to be whirled round rapidly.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Twirl</ent><br/
<hw>Twirl</hw>, <pos>n.</pos> <sn>1.</sn> <def>The act of twirling; a rapid circular motion; a whirl or whirling; quick rotation.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A twist; a convolution.</def>  <rj><au>Woodward.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Twist</ent><br/
<hw>Twist</hw> <pr>(tw<icr/st)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Twisted</conjf>; <pos>p. pr. & vb. n.</pos> <conjf>Twisting</conjf>.]</vmorph> <ety>[OE. <ets>twisten</ets>, AS. <ets>twist</ets> a rope, as made of two (twisted) strands, fr. <ets>twi-</ets> two; akin to D. <ets>twist</ets> a quarrel, dissension, G. <ets>zwist</ets>, Dan. & Sw. <ets>tvist</ets>, Icel. <ets>tvistr</ets> the deuce in cards, <ets>tvistr</ets> distressed. See <er>Twice</er>, <er>Two</er>.]</ety> <sn>1.</sn> <def>To contort; to writhe; to complicate; to crook spirally; to convolve.</def><br/
[<source>1913 Webster</source>]</p>

<p><q><qex>Twist</qex> it into a serpentine form.</q> <rj><qau>Pope.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Hence, to turn from the true form or meaning; to pervert; <as>as, to <ex>twist</ex> a passage cited from an author</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>To distort, as a solid body, by turning one part relatively to another about an axis passing through both; to subject to torsion; <as>as, to <ex>twist</ex> a shaft</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>To wreathe; to wind; to encircle; to unite by intertexture of parts.</def> <ldquo/Longing to <xex>twist</xex> bays with that ivy.<rdquo/  <rj><au>Waller.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>There are pillars of smoke <qex>twisted</qex> about with wreaths of flame.</q> <rj><qau>T. Burnet.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <def>To wind into; to insinuate; -- used reflexively; <as>as, avarice <ex>twists</ex> itself into all human concerns</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>6.</sn> <def>To unite by winding one thread, strand, or other flexible substance, round another; to form by convolution, or winding separate things round each other; <as>as, to <ex>twist</ex> yarn or thread</as>.</def>  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>7.</sn> <def>Hence, to form as if by winding one part around another; to wreathe; to make up.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Was it not to this end<br/
That thou began'st to <qex>twist</qex> so fine a story?</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>8.</sn> <def>To form into a thread from many fine filaments; <as>as, to <ex>twist</ex> wool or cotton</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Twist</ent><br/
<hw>Twist</hw>, <pos>v. i.</pos> <sn>1.</sn> <def>To be contorted; to writhe; to be distorted by torsion; to be united by winding round each other; to be or become twisted; <as>as, some strands will <ex>twist</ex> more easily than others</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To follow a helical or spiral course; to be in the form of a helix.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Twist</ent><br/
<hw>Twist</hw>, <pos>n.</pos> <sn>1.</sn> <def>The act of twisting; a contortion; a flexure; a convolution; a bending.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Not the least turn or <qex>twist</qex> in the fibers of any one animal which does not render them more proper for that particular animal's way of life than any other cast or texture.</q> <rj><qau>Addison.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>The form given in twisting.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>[He] shrunk at first sight of it; he found fault with the length, the thickness, and the <qex>twist</qex>.</q> <rj><qau>Arbuthnot.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>That which is formed by twisting, convoluting, or uniting parts.</def> Specifically: --<br/
[<source>1913 Webster</source>]</p>

<p><sd>(a)</sd> <def>A cord, thread, or anything flexible, formed by winding strands or separate things round each other.</def><br/
[<source>1913 Webster</source>]</p>

<p><sd>(b)</sd> <def>A kind of closely twisted, strong sewing silk, used by tailors, saddlers, and the like.</def><br/
[<source>1913 Webster</source>]</p>

<p><sd>(c)</sd> <def>A kind of cotton yarn, of several varieties.</def><br/
[<source>1913 Webster</source>]</p>

<p><sd>(d)</sd> <def>A roll of twisted dough, baked.</def><br/
[<source>1913 Webster</source>]</p>

<p><sd>(e)</sd> <def>A little twisted roll of tobacco.</def><br/
[<source>1913 Webster</source>]</p>

<p><sd>(f)</sd> <fld>(Weaving)</fld> <def>One of the threads of a warp, -- usually more tightly twisted than the filling.</def><br/
[<source>1913 Webster</source>]</p>

<p><sd>(g)</sd> <fld>(Firearms)</fld> <def>A material for gun barrels, consisting of iron and steel twisted and welded together; <as>as, Damascus <ex>twist</ex></as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sd>(h)</sd> <fld>(Firearms & Ord.)</fld> <def>The spiral course of the rifling of a gun barrel or a cannon.</def><br/
[<source>1913 Webster</source>]</p>

<p><sd>(i)</sd> <def>A beverage made of brandy and gin.</def> <mark>[Slang]</mark><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <ety>[OE.; -- so called as being a <ets>two</ets>-forked branch. See <er>Twist</er>, <pos>v. t.</pos>]</ety> <def>A twig.</def> <mark>[Obs.]</mark>  <rj><au>Chaucer. Fairfax.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <def>Act of imparting a turning or twisting motion, as to a pitched ball; also, the motion thus imparted; <as>as, the <ex>twist</ex> of a billiard ball</as>.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><sn>6.</sn>  <def>A strong individual tendency, or bent; a marked inclination; a bias; -- often implying a peculiar or unusual tendency; <as>as, a <ex>twist</ex> toward fanaticism</as>.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><cs><mcol><col><b>Gain twist</b></col>, <it>or</it> <col><b>Gaining twist</b></col></mcol> <fld>(Firearms)</fld>, <cd>twist of which the pitch is less, and the inclination greater, at the muzzle than at the breech.</cd> -- <col><b>Twist drill</b></col>, <cd>a drill the body of which is twisted like that of an auger. See <xex>Illust.</xex> of <er>Drill</er>.</cd> -- <col><b>Uniform twist</b></col> <fld>(Firearms)</fld>, <cd>a twist of which the spiral course has an equal pitch throughout.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Twiste</ent><br/
<hw>Twist"e</hw> <pr>(?)</pr>, <mark>obs.</mark> <def><pos>imp.</pos> of <er>Twist</er>.</def>  <rj><au>Chaucer.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Twisted</ent><br/
<hw>Twist"ed</hw>, <pos>a.</pos> <def>Contorted; crooked spirally; subjected to torsion; hence, perverted.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Twisted curve</b></col> <fld>(Geom.)</fld>, <cd>a curve of double curvature. See <cref>Plane curve</cref>, under <er>Curve</er>.</cd> -- <col><b>Twisted surface</b></col> <fld>(Geom.)</fld>, <cd>a surface described by a straight line moving according to any law whatever, yet so that the consecutive positions of the line shall not be in one plane; a warped surface.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Twister</ent><br/
<hw>Twist"er</hw> <pr>(?)</pr>, <pos>n.</pos> <sn>1.</sn> <def>One who twists; specifically, the person whose occupation is to twist or join the threads of one warp to those of another, in weaving.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>The instrument used in twisting, or making twists.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>He, twirling his <qex>twister</qex>, makes a twist of the twine.</q> <rj><qau>Wallis.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <fld>(Carp.)</fld> <def>A girder.</def>  <rj><au>Craig.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <fld>(Man.)</fld> <def>The inner part of the thigh, the proper place to rest upon when on horseback.</def>  <rj><au>Craig.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><-- 5. (Colloq.) A tornado. --><br/
[<source>1913 Webster</source>]</p>

<p><ent>Twistical</ent><br/
<hw>Twist"ic*al</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Crooked; tortuous; hence, perverse; unfair; dishonest.</def> <mark>[Slang, U. S.]</mark>  <rj><au>Bartlett.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Twisting</ent><br/
<hw>Twist"ing</hw>, <def><pos>a. & n.</pos> from <er>Twist</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Twisting pair</b></col>. <fld>(Kinematics)</fld> <cd>See under <er>Pair</er>, <pos>n.</pos>, 7.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Twit</ent><br/
<hw>Twit</hw> <pr>(?)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Twitted</conjf>; <pos>p. pr. & vb. n.</pos> <conjf>Twitting</conjf>.]</vmorph> <ety>[OE. <ets>atwiten</ets>, AS. <ets><ae/tw<imac/tan</ets> to reproach, blame; <ets>aet</ets> at + <ets>w<imac/tan</ets> to reproach, blame; originally, to observe, see, hence, to observe what is wrong (cf. the meanings of E. <ets>animadvert</ets>; akin to G. ver<ets>weisen</ets> to censure, OHG. fir<ets>w<imac/zan</ets>, Goth. fra<ets>weitan</ets> to avenge, L. <ets>videre</ets> to see. See <er>Vision</er>, <er>Wit</er>.]</ety> <def>To vex by bringing to notice, or reminding of, a fault, defect, misfortune, or the like; to revile; to reproach; to upbraid; to taunt; <as>as, he <ex>twitted</ex> his friend of falsehood</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>This these scoffers <qex>twitted</qex> the Christians with.</q> <rj><qau>Tillotson.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Aesop minds men of their errors, without <qex>twitting</qex> them for what is amiss.</q> <rj><qau>L'Estrange.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Twitch</ent><br/
<hw>Twitch</hw> <pr>(?)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Twitched</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Twitching</conjf>.]</vmorph> <ety>[OE. <ets>twicchen</ets>, fr. (doubtful) AS. <ets>twiccian</ets>; akin to AS. angel<ets>twicca</ets> a worm used for bait, literally, a hook twitcher, LG. <ets>twikken</ets> to tweak, G. <ets>zwicken</ets>.  Cf. <er>Tweak</er>.]</ety> <def>To pull with a sudden jerk; to pluck with a short, quick motion; to snatch; <as>as, to <ex>twitch</ex> one by the sleeve; to <ex>twitch</ex> a thing out of another's hand; to <ex>twitch</ex> off clusters of grapes</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Thrice they <qex>twitched</qex> the diamond in her ear.</q> <rj><qau>Pope.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Twitch</ent><br/
<hw>Twitch</hw>, <pos>n.</pos> <sn>1.</sn> <def>The act of twitching; a pull with a jerk; a short, sudden, quick pull; <as>as, a <ex>twitch</ex> by the sleeve</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A short, spastic contraction of the fibers or muscles; a simple muscular contraction; <as>as, convulsive <ex>twitches</ex>; a <ex>twitch</ex> in the side</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <fld>(Far.)</fld> <def>A stick with a hole in one end through which passes a loop, which can be drawn tightly over the upper lip or an ear of a horse. By twisting the stick the compression is made sufficiently painful to keep the animal quiet during a slight surgical operation.</def>  <rj><au>J. H. Walsh.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Twitcher</ent><br/
<hw>Twitch"er</hw> <pr>(?)</pr>, <pos>n.</pos> <def>One who, or that which, twitches.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Twitch grass</ent><br/
<hw>Twitch" grass`</hw> <pr>(?)</pr>. <fld>(Bot.)</fld> <def>See <er>Quitch grass</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Twite</ent><br/
<hw>Twite</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Zool.)</fld> <sd>(a)</sd> <def>The European tree sparrow.</def> <sd>(b)</sd> <def>The mountain linnet (<spn>Linota flavirostris</spn>).</def> <mark>[Prov. Eng.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Twitlark</ent><br/
<hw>Twit"lark`</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Zool.)</fld> <def>The meadow pipit.</def> <mark>[Prov. Eng.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Twitter</ent><br/
<hw>Twit"ter</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[From <er>Twit</er>.]</ety> <def>One who twits, or reproaches; an upbraider.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Twitter</ent><br/
<hw>Twit"ter</hw>, <pos>v. i.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Twittered</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Twittering</conjf>.]</vmorph> <ety>[OE. <ets>twiteren</ets>; of imitative origin; cf. G. <ets>zwitschern</ets>, OHG. <ets>zwizzir<omac/n</ets>, D. <ets>kwetteren</ets>, Sw. <ets>qvittra</ets>, Dan. <ets>quiddre</ets>.  Cf. <er>Titter</er>.]</ety> <sn>1.</sn> <def>To make a succession of small, tremulous, intermitted noises.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The swallow <qex>twittering</qex> from the straw-built shed.</q> <rj><qau>Gray.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To make the sound of a half-suppressed laugh; to titter; to giggle.</def>  <rj><au>J. Fletcher.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <ety>[Perhaps influenced by <ets>twitch</ets>.]</ety> <def>To have a slight trembling of the nerves; to be excited or agitated.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Twitter</ent><br/
<hw>Twit"ter</hw>, <pos>v. t.</pos> <def>To utter with a twitter.</def>  <rj><au>Cowper.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Twitter</ent><br/
<hw>Twit"ter</hw>, <pos>n.</pos> <sn>1.</sn> <def>The act of twittering; a small, tremulous, intermitted noise, as that made by a swallow.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A half-suppressed laugh; a fit of laughter partially restrained; a titter; a giggle.</def>  <rj><au>Hudibras.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>A slight trembling or agitation of the nerves.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Twittering</ent><br/
<hw>Twit"ter*ing</hw>, <pos>n.</pos> <sn>1.</sn> <def>The act of one who, or that which, twitters.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A slight nervous excitement or agitation, such as is caused by desire, expectation, or suspense.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>A widow, who had a <qex>twittering</qex> towards a second husband, took a gossiping companion to manage the job.</q> <rj><qau>L'Estrange.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Twittingly</ent><br/
<hw>Twit"ting*ly</hw> <pr>(?)</pr>, <pos>adv.</pos> <def>In a twitting manner; with upbraiding.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Twittle-twattle</ent><br/
<hw>Twit"tle-twat`tle</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[See <er>Twattle</er>.]</ety> <def>Tattle; gabble.</def>  <rj><au>L'Estrange.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>'Twixt</ent><br/
<hw>'Twixt</hw> <pr>(?)</pr>. <def>An abbreviation of <er>Betwixt</er>, used in poetry, or in colloquial language.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>'Twixt-brain</ent><br/
<hw>'Twixt"-brain`</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Anat.)</fld> <def>The thalamencephalon.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Two</ent><br/
<hw>Two</hw> <pr>(t<oomac/)</pr>, <pos>a.</pos> <ety>[OE. <ets>two</ets>, <ets>twa</ets>, properly fem. & neut., <ets>twei</ets>, <ets>twein</ets>, <ets>tweien</ets>, properly masc. (whence E. <ets>twain</ets>), AS. <ets>tw<amac/</ets>, fem. & neut., <ets>tw<emac/gen</ets>, masc., <ets>t<umac/</ets>, neut.; akin to OFries. <ets>tw<emac/ne</ets>, masc., <ets>tw<amac/</ets>, fem. & neut., OS. <ets>tw<emac/ne</ets>, masc., <ets>tw<amac/</ets>, fem., <ets>tw<emac/</ets>, neut., D. <ets>twee</ets>, OHG. <ets>zw<emac/ne</ets>, <ets>zw<omac/</ets>, <ets>zwei</ets>, G. <ets>zwei</ets>, Icel. <ets>tveir</ets>, <ets>tv<ae/r</ets>, <ets>tvau</ets>, Sw. <ets>tv<aring/</ets>, Dan. <ets>to</ets>, Goth. <ets>twai</ets>, <ets>tw<omac/s</ets>, <ets>twa</ets>; Lith. <ets>du</ets>, Russ. <ets>dva</ets>, Ir. & Gael. <ets>da</ets>, W. <ets>dau</ets>, <ets>dwy</ets>, L. <ets>duo</ets>, Gr. <grk>dy`o</grk>, Skr. <ets>dva</ets>. <root/300.   Cf. <er>Balance</er>, <er>Barouche</er>, <er>Between</er>, <er>Bi-</er>, <er>Combine</er>, <er>Deuce</er> two in cards, <er>Double</er>, <er>Doubt</er>, <er>Dozen</er>, <er>Dual</er>, <er>Duet</er>, <er>Dyad</er>, <er>Twain</er>, <er>Twelve</er>, <er>Twenty</er>, <er>Twice</er>, <er>Twilight</er>, <er>Twig</er>, <er>Twine</er>, <pos>n.</pos>, <er>Twist</er>.]</ety> <def>One and one; twice one.</def> <ldquo/<xex>Two</xex> great lights.<rdquo/ <au>Gen. i. 16.</au> <ldquo/<xex>Two</xex> black clouds.<rdquo/  <rj><au>Milton.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ <xex>Two</xex> is often joined with other words, forming compounds signifying <xex>divided into</xex>, <xex>consisting of</xex>, or <xex>having</xex>, <xex>two parts</xex>, <xex>divisions</xex>, <xex>organs</xex>, or the like; as <xex>two</xex>-bladed, <xex>two</xex>-celled, <xex>two</xex>-eared, <xex>two</xex>-flowered, <xex>two</xex>hand, <xex>two</xex>-headed, <xex>two</xex>-horse, <xex>two</xex>-leafed or <xex>two</xex>-leaved, <xex>two</xex>-legged, <xex>two</xex>-lobed, <xex>two</xex>-masted, <xex>two</xex>-named, <xex>two</xex>-part, <xex>two</xex>-petaled, <xex>two</xex>-pronged, <xex>two</xex>-seeded, <xex>two</xex>-sided, <xex>two</xex>-story, <xex>two</xex>-stringed, <xex>two</xex>-toothed, <xex>two</xex>-valved, <xex>two</xex>-winged, and the like.</note><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>One or two</b></col>, <cd>a phrase often used indefinitely for a small number.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Two</ent><br/
<hw>Two</hw> <pr>(?)</pr>, <pos>n.</pos> <sn>1.</sn> <def>The sum of one and one; the number next greater than one, and next less than three; two units or objects.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A symbol representing two units, as 2, II., or ii.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>In two</b></col>, <cd>asunder; into two parts; in halves; in twain; as, cut <xex>in two</xex>.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Two-capsuled</ent><br/
<hw>Two"-cap`suled</hw> <pr>(?)</pr>, <pos>a.</pos> <fld>(Bot.)</fld> <def>Having two distinct capsules; bicapsular.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Two-cleft</ent><br/
<hw>Two"-cleft`</hw> <pr>(?)</pr>, <pos>a.</pos> <fld>(Bot.)</fld> <def>Divided about half way from the border to the base into two segments; bifid.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Two-cycle</ent><br/
<hw>Two"-cy`cle</hw>, <pos>n.</pos> <fld>(Thermodynamics)</fld> <def>A two-stroke cycle for an internal-combustion engine.</def> -- <wordforms><wf>Two"-cy`cle</wf>, <pos>a.</pos></wordforms><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Two-decker</ent><br/
<hw>Two"-deck`er</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A vessel of war carrying guns on two decks.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Two-edged</ent><br/
<hw>Two"-edged`</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Having two edges, or edges on both sides; <as>as, a <ex>two-edged</ex> sword</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Twofold</ent><br/
<hw>Two"fold`</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[Cf. <er>Twifold</er>.]</ety> <def>Double; duplicate; multiplied by two; <as>as, a <ex>twofold</ex> nature; a <ex>twofold</ex> sense; a twofold argument</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Twofold</ent><br/
<hw>Two"fold`</hw>, <pos>adv.</pos> <def>In a double degree; doubly.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Two-foot</ent><br/
<hw>Two"-foot`</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Measuring two feet; two feet long, thick, or wide; <as>as, a <ex>two-foot</ex> rule</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Two-forked</ent><br/
<hw>Two"-forked`</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Divided into two parts, somewhat after the manner of a fork; dichotomous.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Two-hand</ent><br/
<hw>Two"-hand`</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Employing two hands; <as>as, the <ex>two-hand</ex> alphabet</as>. See <er>Dactylology</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Two-handed</ent><br/
<hw>Two"-hand`ed</hw> <pr>(?)</pr>, <pos>a.</pos> <sn>1.</sn> <def>Having two hands; -- often used as an epithet equivalent to <xex>large</xex>, <xex>stout</xex>, <xex>strong</xex>, or <xex>powerful</xex>.</def> <ldquo/<xex>Two-handed</xex> sway.<rdquo/  <rj><au>Milton.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Used with both hands; <as>as, a <ex>two-handed</ex> sword</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>That <qex>two-handed</qex> engine [the sword].</q> <rj><qau>Milton.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>Using either hand equally well; ambidextrous.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Two-lipped</ent><br/
<hw>Two"-lipped`</hw> <pr>(?)</pr>, <pos>a.</pos> <sn>1.</sn> <def>Having two lips.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Bot.)</fld> <def>Divided in such a manner as to resemble the two lips when the mouth is more or less open; bilabiate.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Two-name</ent><br/
<hw>Two"-name`</hw>, <pos>a.</pos> <fld>(Banking)</fld> <def>Having or bearing two names; <as>as, <col><b>two-name paper</b></col>, that is, negotiable paper on which at least two persons are severally liable as separate makers, or, usually, one as maker and one as indorser</as>.</def> <mark>[Colloq.]</mark><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Two-parted</ent><br/
<hw>Two"-part`ed</hw> <pr>(-p<aum/rt`<ecr/d)</pr>, <pos>a.</pos> <fld>(Bot.)</fld> <def>Divided from the border to the base into two distinct parts; bipartite.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Twopence</ent><br/
<hw>Two"pence</hw> <pr>(?; 277)</pr>, <pos>n.</pos> <def>A small coin, and money of account, in England, equivalent to two pennies, -- minted to a fixed annual amount, for almsgiving by the sovereign on Maundy Thursday.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Twopenny</ent><br/
<hw>Two"pen*ny</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Of the value of twopence.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Two-phaser</ent><br/
<ent>Two-phase</ent><br/
<mhw><hw>Two"-phase`</hw> <pr>(?)</pr>, <pos>a.</pos>, <hw>Two"-phas`er</hw> <pr>(<?/)</pr></mhw>, <pos>n.</pos> <fld>(Elec.)</fld> <def>Same as <er>Diphase</er>, <er>Diphaser</er>.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Two-ply</ent><br/
<hw>Two"-ply`</hw> <pr>(?)</pr>, <pos>a.</pos> <sn>1.</sn> <def>Consisting of two thicknesses, as cloth; double.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Woven double, as cloth or carpeting, by incorporating two sets of warp thread and two of weft.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Two-port</ent><br/
<hw>Two"-port`</hw>, <pos>a.</pos> <def>Having two ports; specif.: Designating a type of two-cycle internal-combustion engine in which the admission of the mixture to the crank case is through a suction valve.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Two-ranked</ent><br/
<hw>Two"-ranked`</hw> <pr>(?)</pr>, <pos>a.</pos> <fld>(Bot.)</fld> <def>Alternately disposed on exactly opposite sides of the stem so as to from two ranks; distichous.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Two-sided</ent><br/
<hw>Two"-sid`ed</hw> <pr>(?)</pr>, <pos>a.</pos> <sn>1.</sn> <def>Having two sides only; hence, double-faced; hypocritical.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Biol.)</fld> <def>Symmetrical.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Two-speed</ent><br/
<hw>Two"-speed`</hw>, <pos>a.</pos> <def>Adapted for producing or for receiving either of two speeds; -- said of a power-transmitting device.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Two-step</ent><br/
<hw>Two"-step`</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A kind of round dance in march or polka time; also, a piece of music for this dance.</def> <mark>[U. S.]</mark><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Two-throw</ent><br/
<hw>Two"-throw`</hw> <pr>(?)</pr>, <pos>a.</pos> <fld>(Mach.)</fld> <sd>(a)</sd> <def>Capable of being thrown or cranked in two directions, usually opposite to one another; <as>as, a <ex>two-throw</ex> crank; a <ex>two-throw</ex> switch</as>.</def> <sd>(b)</sd> <def>Having two crank set near together and opposite to one another; <as>as, a <ex>two-throw</ex> crank shaft</as>.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Two-tongued</ent><br/
<hw>Two"-tongued`</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Double-tongued; deceitful.</def>  <rj><au>Sandys.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Two-to-one</ent><br/
<hw>Two"-to-one"</hw>, <pos>a.</pos> <fld>(Mach.)</fld> <def>Designating, or pert. to, a gear for reducing or increasing a velocity ratio two to one.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Two-way</ent><br/
<hw>Two"-way`</hw> <pr>(?)</pr>, <pos>a.</pos> <fld>(Pipe Fitting)</fld> <def>Serving to connect at will one pipe or channel with either of two others; <as>as, a <ex>two-way</ex> cock</as>.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Twyblade</ent><br/
<hw>Twy"blade`</hw> <pr>(?)</pr>, <pos>n.</pos> <def>See <er>Twayblade</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Ty-all</ent><br/
<hw>Ty"-all`</hw> <pr>(?)</pr>, <pos>n.</pos> <def>Something serving to tie or secure.</def> <mark>[Obs.]</mark>  <rj><au>Latimer.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tyburn ticket</ent><br/
<hw>Ty"burn tick`et</hw> <pr>(?)</pr>. <ety>[So called in allusion to <ets>Tyburn</ets>, formerly a place of execution in England.]</ety> <fld>(O. Eng. Law)</fld> <def>A certificate given to one who prosecutes a felon to conviction, exempting him from certain parish and ward offices.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tychism</ent><br/
<hw>Ty"chism</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Gr. <grk>ty`chh</grk> fortune, chance + <ets>-ism</ets>.]</ety> <def>Any theory which conceives chance as an objective reality; esp., a theory of evolution which considers that variation may be purely fortuitous.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Tychonic</ent><br/
<hw>Ty*chon"ic</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Of or pertaining to <person>Tycho Brahe</person>, or his system of astronomy.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tycoon</ent><br/
<hw>Ty*coon"</hw> <pr>(t<isl/`k<oomac/n")</pr>, <pos>n.</pos> <ety>[Chinese <ets>tai-kun</ets> great prince.]</ety> <def>The title by which the shogun, or former commander in chief of the Japanese army, was known to foreigners.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tydy</ent><br/
<hw>Ty"dy</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Zool.)</fld> <def>Same as <er>Tidy</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tye</ent><br/
<hw>Tye</hw> <pr>(?)</pr>, <pos>n.</pos> <sn>1.</sn> <def>A knot; a tie.</def> <mark>[R.]</mark> <see>See <er>Tie</er>.</see><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Naut.)</fld> <def>A chain or rope, one end of which passes through the mast, and is made fast to the center of a yard; the other end is attached to a tackle, by means of which the yard is hoisted or lowered.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <fld>(Mining)</fld> <def>A trough for washing ores.</def>  <rj><au>Knight.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tye</ent><br/
<hw>Tye</hw>, <pos>v. t.</pos> <def>See <er>Tie</er>, the proper orthography.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tyer</ent><br/
<hw>Ty"er</hw> <pr>(?)</pr>, <pos>n.</pos> <def>One who ties, or unites.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tyfoon</ent><br/
<hw>Ty*foon"</hw> <pr>(?)</pr>, <pos>n.</pos> <def>See <er>Typhoon</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tyger</ent><br/
<hw>Ty"ger</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Zool.)</fld> <def>A tiger.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tying</ent><br/
<hw>Ty"ing</hw> <pr>(?)</pr>, <def><pos>p. pr.</pos> of <er>Tie</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tying</ent><br/
<hw>Ty"ing</hw>, <pos>n.</pos> <fld>(Mining)</fld> <def>The act or process of washing ores in a buddle.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tyke</ent><br/
<hw>Tyke</hw> <pr>(?)</pr>, <pos>n.</pos> <sn>1.</sn> <def>See 2d <er>Tike</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>a small child.</def><br/
[<source>PJC</source>]</p>

<p><ent>Tylarus</ent><br/
||<hw>Ty"la*rus</hw> <pr>(?)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Tylari</plw> <pr>(#)</pr>.</plu> <ety>[NL., fr. Gr. <grk>ty`lh</grk> a cushion.]</ety> <fld>(Zool.)</fld> <def>One of the pads on the under surface of the toes of birds.</def><br/
[<source>1913 Webster</source>]</p>

<p><-- p. 1558 pr=PI --><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tyler</ent><br/
<hw>Ty"ler</hw> <pr>(t<imac/"l<etil/r)</pr>, <pos>n.</pos> <def>See 2d <er>Tiler</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tylopoda</ent><br/
||<hw>Ty*lop"o*da</hw> <pr>(t<imac/*l<ocr/p"<osl/*d<adot/)</pr>, <pos>n. pl.</pos> <ety>[NL., from <grk>ty`lh</grk> a cushion + <ets>-poda</ets>.]</ety> <fld>(Zool.)</fld> <def>A tribe of ungulates comprising the camels.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tylosis</ent><br/
||<hw>Ty*lo"sis</hw> <pr>(?)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Tyloses</plw> <pr>(#)</pr>.</plu> <ety>[NL., fr. Gr. <grk>ty`los</grk> a lump, knot.]</ety> <fld>(Bot.)</fld> <def>An intrusion of one vegetable cell into the cavity of another, sometimes forming there an irregular mass of cells.</def>  <rj><au>Goodale.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tymbal</ent><br/
<hw>Tym"bal</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F. <ets>timbale</ets>, Sp. <ets>timbal</ets>; cf. <ets>atabal</ets>; probably of Arabic origin.  Cf. <er>Atabal</er>, <er>Timbrel</er>.]</ety> <def>A kind of kettledrum.</def> <altsp>[Written also <asp>timbal</asp>.]</altsp><br/
[<source>1913 Webster</source>]</p>

<p><q>A <qex>tymbal's</qex> sound were better than my voice.</q> <rj><qau>Prior.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tymp</ent><br/
<hw>Tymp</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Cf. <er>Tympan</er>.]</ety> <fld>(Blast Furnace)</fld> <def>A hollow water-cooled iron casting in the upper part of the archway in which the dam stands.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tympan</ent><br/
<hw>Tym"pan</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[F., fr. L. <ets>tympanum</ets> a kettledrum, a panel of a door. See <er>Tympanum</er>, and cf. <er>Tymp</er>.]</ety> <sn>1.</sn> <def>A drum.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Arch.)</fld> <def>A panel; a tympanum.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <fld>(Print.)</fld> <def>A frame covered with parchment or cloth, on which the blank sheets are put, in order to be laid on the form to be impressed.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Tympan sheet</b></col> <fld>(Print.)</fld>, <cd>a sheet of paper of the same size as that to be printed, pasted on the tympan, and serving as a guide in laying the sheets evenly for printing.</cd>  <rj><au>W. Savage.</au></rj></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tympanal</ent><br/
<hw>Tym"pa*nal</hw> <pr>(?)</pr>, <pos>n.</pos> <def>Tympanic.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tympanic</ent><br/
<hw>Tym*pan"ic</hw> <pr>(?; 277)</pr>, <pos>a.</pos> <ety>[See <er>Tympanum</er>.]</ety> <sn>1.</sn> <def>Like a tympanum or drum; acting like a drumhead; <as>as, a <ex>tympanic</ex> membrane</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Anat.)</fld> <def>Of or pertaining to the tympanum.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Tympanic bone</b></col> <fld>(Anat.)</fld>, <cd>a bone of the skull which incloses a part of the tympanum and supports the tympanic membrane.</cd> -- <col><b>Tympanic membrane</b></col>. <fld>(Anat.)</fld> <cd>See the Note under <er>Ear</er>.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tympanic</ent><br/
<hw>Tym*pan"ic</hw>, <pos>n.</pos> <fld>(Anat.)</fld> <def>The tympanic bone.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tympanist</ent><br/
<hw>Tym"pa*nist</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>tympaniste</ets>, Gr. <grk>tympanisth`s</grk>.]</ety> <def>One who beats a drum.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tympanites</ent><br/
||<hw>Tym`pa*ni"tes</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L., fr. Gr. <grk>tympani`ths</grk>, fr. <grk>ty`mpanon</grk> a kettledrum.]</ety> <fld>(Med.)</fld> <def>A flatulent distention of the belly; tympany.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tympanitic</ent><br/
<hw>Tym`pa*nit"ic</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>tympaniticus</ets> one afflicted with tympanites.]</ety> <fld>(Med.)</fld> <def>Of, pertaining to, or affected with, tympanites.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tympanitis</ent><br/
<hw>Tym`pa*ni"tis</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[NL. See <er>Tympanum</er>, <er>and</er> <er>-itis</er>.]</ety> <fld>(Med.)</fld> <def>Inflammation of the lining membrane of the middle ear.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tympanize</ent><br/
<hw>Tym"pa*nize</hw> <pr>(?)</pr>, <pos>v. i.</pos> <ety>[L. <ets>tympanizare</ets> to beat a drum, Gr. <grk>tympani`zein</grk>.]</ety> <def>To drum.</def> <mark>[R.]</mark>  <rj><au>Coles.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tympanize</ent><br/
<hw>Tym"pa*nize</hw>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Tympanized</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Tympanizing</conjf> <pr>(?)</pr>.]</vmorph> <def>To stretch, as a skin over the head of a drum; to make into a drum or drumhead, or cause to act or sound like a drum.</def> <mark>[Obs.]</mark> <ldquo/<xex>Tympanized</xex>, as other saints of God were.<rdquo/  <rj><au>Oley.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tympano</ent><br/
||<hw>Tym"pa*no</hw> <pr>(?)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Tympani</plw> <pr>(#)</pr>.</plu> <ety>[It. <ets>timpano</ets>. See <er>Tympanum</er>.]</ety> <fld>(Mus.)</fld> <def>A kettledrum; -- chiefly used in the plural to denote the kettledrums of an orchestra. See <er>Kettledrum</er>.</def> <altsp>[Written also <asp>timpano</asp>.]</altsp><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tympano-</ent><br/
<hw>Tym"pa*no-</hw> <pr>(?)</pr>. <def>A combining form used in anatomy to indicate <xex>connection with</xex>, or <xex>relation to</xex>, <xex>the tympanum</xex>; as in <xex>tympano</xex>hyal, <xex>tympano</xex>-Eustachian.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tympanohyal</ent><br/
<hw>Tym`pa*no*hy"al</hw> <pr>(?)</pr>, <pos>a.</pos> <fld>(Anat.)</fld> <def>Of or pertaining to the tympanum and the hyoidean arch.</def> -- <def2><pos>n.</pos> <def>The proximal segment in the hyoidean arch, becoming a part of the styloid process of the temporal bone in adult man.</def></def2><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tympanum</ent><br/
<hw>Tym"pa*num</hw> <pr>(?)</pr>, <pos>n.</pos>; <plu><it>pl.</it> E. <plw>Tympanums</plw> <pr>(#)</pr>, L. <plw>Tympana</plw> <pr>(#)</pr>.</plu> <ety>[L., a kettledrum, a drum or wheel in machines, the triangular area in a pediment, the panel of a door, Gr. <grk>ty`mpanon</grk>, <grk>ty`panon</grk>, fr. <?/ to strike, beat. See <er>Type</er>, and cf. <er>Timbrel</er>.]</ety> <sn>1.</sn> <fld>(Anat.)</fld> <sd>(a)</sd> <def>The ear drum, or middle ear. Sometimes applied incorrectly to the tympanic membrane. See <er>Ear</er>.</def> <sd>(b)</sd> <def>A chamber in the anterior part of the syrinx of birds.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Zool.)</fld> <def>One of the naked, inflatable air sacs on the neck of the prairie chicken and other species of grouse.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <fld>(Arch.)</fld> <sd>(a)</sd> <def>The recessed face of a pediment within the frame made by the upper and lower cornices, being usually a triangular space or table.</def> <sd>(b)</sd> <def>The space within an arch, and above a lintel or a subordinate arch, spanning the opening below the arch.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <fld>(Mech.)</fld> <def>A drum-shaped wheel with spirally curved partitions by which water is raised to the axis when the wheel revolves with the lower part of the circumference submerged, -- used for raising water, as for irrigation.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tympany</ent><br/
<hw>Tym"pa*ny</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Gr. <?/, fr. <?/ a kettledrum. See <er>Tympanites</er>.]</ety> <sn>1.</sn> <fld>(Med.)</fld> <def>A flatulent distention of the belly; tympanites.</def>  <rj><au>Fuller.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Hence, inflation; conceit; bombast; turgidness.</def> <ldquo/Thine's a <xex>tympany</xex> of sense.<rdquo/  <rj><au>Dryden.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>A plethoric a tautologic <qex>tympany</qex> of sentence.</q> <rj><qau>De Quincey.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tynd</ent><br/
<hw>Tynd</hw> <pr>(?)</pr>, <pos>v. t.</pos> <ety>[See <er>Tine</er> to shut in.]</ety> <def>To shut; to close.</def> <mark>[Obs.]</mark>  <rj><au>Wyclif.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tyne</ent><br/
<hw>Tyne</hw> <pr>(?)</pr>, <pos>v. t.</pos> <ety>[Icel. <ets>t<ymac/na</ets>.]</ety> <def>To lose.</def> <mark>[Obs. or Scot.]</mark> <ldquo/His bliss gan he <xex>tyne</xex>.<rdquo/ <au>Piers Plowman.</au> <au>Sir W. Scott.</au><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tyne</ent><br/
<hw>Tyne</hw>, <pos>v. i.</pos> <def>To become lost; to perish.</def> <mark>[Obs.]</mark>  <rj><au>Spenser.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tyne</ent><br/
<hw>Tyne</hw>, <pos>n.</pos> <ety>[See <er>Tine</er> a prong.]</ety> <fld>(Zool.)</fld> <def>A prong or point of an antler.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tyne</ent><br/
<hw>Tyne</hw>, <pos>n.</pos> <ety>[See <er>Teen</er>, <pos>n.</pos>]</ety> <def>Anxiety; tine.</def> <mark>[Obs.]</mark> <ldquo/With labor and long <xex>tyne</xex>.<rdquo/  <rj><au>Spenser.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tyny</ent><br/
<hw>Ty"ny</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[See <er>Tiny</er>.]</ety> <def>Small; tiny.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Typal</ent><br/
<hw>Typ"al</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Relating to a type or types; belonging to types; serving as a type; typical.</def>  <rj><au>Owen.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>-type</ent><br/
<hw>-type</hw> <pr>(?)</pr>. <ety>[See <er>Type</er>, <pos>n.</pos>]</ety> <def>A combining form signifying <sig>impressed form</sig>; <sig>stamp</sig>; <sig>print</sig>; <sig>type</sig>; <sig>typical form</sig>; <sig>representative</sig>; as in stereo<xex>type</xex> photo<xex>type</xex>, ferro<xex>type</xex>, mono<xex>type</xex>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Type</ent><br/
<hw>Type</hw> <pr>(t<imac/p)</pr>, <pos>n.</pos> <ety>[F. <ets>type</ets>; cf. It. <ets>tipo</ets>, from L. <ets>typus</ets> a figure, image, a form, type, character, Gr. <grk>ty`pos</grk> the mark of a blow, impression, form of character, model, from the root of <grk>ty`ptein</grk> to beat, strike; cf. Skr. <ets>tup</ets> to hurt.]</ety>  <sn>1.</sn> <def>The mark or impression of something; stamp; impressed sign; emblem.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The faith they have in tennis, and tall stockings,<br/
Short blistered breeches, and those <qex>types</qex> of travel.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Form or character impressed; style; semblance.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Thy father bears the <qex>type</qex> of king of Naples.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>A figure or representation of something to come; a token; a sign; a symbol; -- correlative to <xex>antitype</xex>.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>A <qex>type</qex> is no longer a <qex>type</qex> when the thing typified comes to be actually exhibited.</q> <rj><qau>South.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>4.</sn> <def>That which possesses or exemplifies characteristic qualities; the representative.</def> Specifically: <sd>(a)</sd> <fld>(Biol.)</fld> <def>A general form or structure common to a number of individuals; hence, the ideal representation of a species, genus, or other group, combining the essential characteristics; an animal or plant possessing or exemplifying the essential characteristics of a species, genus, or other group.  Also, a group or division of animals having a certain typical or characteristic structure of body maintained within the group.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Since the time of Cuvier and Baer . . . the whole animal kingdom has been universally held to be divisible into a small number of main divisions or <qex>types</qex>.</q> <rj><qau>Haeckel.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sd>(b)</sd> <fld>(Fine Arts)</fld> <def>The original object, or class of objects, scene, face, or conception, which becomes the subject of a copy; esp., the design on the face of a medal or a coin.</def><br/
[<source>1913 Webster</source>]</p>

<p><sd>(c)</sd> <fld>(Chem.)</fld> <def>A simple compound, used as a model or pattern to which other compounds are conveniently regarded as being related, and from which they may be actually or theoretically derived.</def><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ The fundamental types used to express the simplest and most essential chemical relations are hydrochloric acid, <chform>HCl</chform>; water, <chform>H2O</chform>; ammonia, <chform>NH3</chform>; and methane, <chform>CH4</chform>.</note><br/
[<source>1913 Webster</source>]</p>

<p><sn>5.</sn> <fld>(Typog.)</fld> <sd>(a)</sd> <def>A raised letter, figure, accent, or other character, cast in metal or cut in wood, used in printing.</def> <sd>(b)</sd> <def>Such letters or characters, in general, or the whole quantity of them used in printing, spoken of collectively; any number or mass of such letters or characters, however disposed.</def><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ Type are mostly made by casting type metal in a mold, though some of the larger sizes are made from maple, mahogany, or boxwood. In the cut, <it>a</it> is the <part>body</part>; <it>b</it>, the <part>face</part>, or part from which the impression is taken; <it>c</it>, the <part>shoulder</part>, or top of the body; <it>d</it>, the <part>nick</part> (sometimes two or more are made), designed to assist the compositor in distinguishing the bottom of the face from t`e top; <it>e</it>, the <part>groove</part> made in the process of finishing, -- each type as cast having attached to the bottom of the body a jet, or small piece of metal (formed by the surplus metal poured into the mold), which, when broken off, leaves a roughness that requires to be removed. The fine lines at the top and bottom of a letter are technically called <xex>ceriphs</xex>, and when part of the face projects over the body, as in the letter <xex>f</xex>, the projection is called a <xex>kern</xex>.<br/
[<source>1913 Webster</source>]</p>

<p>   The type which compose an ordinary book font consist of Roman <sc>CAPITALS</sc>, <sc>small capitals</sc>, and lower-case letters, and <it>Italic CAPITALS</it> and <it>lower-case</it> letters, with accompanying figures, points, and reference marks, -- in all about two hundred characters. Including the various modern styles of fancy type, some three or four hundred varieties of face are made. Besides the ordinary Roman and <xex>Italic</xex>, some of the most important of the varieties are --<br/
[<source>1913 Webster</source>]</p>

<p><table>
<row><oldenglishtype>Old English.</oldenglishtype>  <blacklettertype>Black Letter.</blacklettertype></row>
<row><oldstyletype>Old Style.</oldstyletype>  <frenchelzevirtype>French Elzevir.</frenchelzevirtype></row>
<row><boldfacetype>Boldface.</boldfacetype>  <antiquetype>Antique.</antiquetype></row>
<row><clarendontype>Clarendon.</clarendontype>  <gothictype>Gothic.</gothictype></row>
<row><typewritertype>Typewriter.</typewritertype>  <scripttype>Script.</scripttype></row>
</table><br/
[<source>1913 Webster</source>]</p>

<p>  The smallest body in common use is diamond; then follow in order of size, pearl, agate, nonpareil, minion, brevier, bourgeois (or two-line diamond), long primer (or two-line pearl), small pica (or two-line agate), pica (or two-line nonpareil), English (or two-line minion), Columbian (or two-line brevier), great primer (or two-line bourgeois), paragon (or two-line long primer), double small pica (or two-line small pica), double pica (or two-line pica), double English (or two-line English), double great primer (or two-line great primer), double paragon (or two-line paragon), canon (or two-line double pica). Above this, the sizes are called five-line pica, six-line pica, seven-line pica, and so on, being made mostly of wood. The following alphabets show the different sizes up to great primer.<br/
[<source>1913 Webster</source>]</p>

<p><table>
<row>Brilliant . . abcdefghijklmnopqrstuvwxyz</row>
<-- brilliant l = 1.0 mm -->
<row>Diamond . .  abcdefghijklmnopqrstuvwxyz</row>
<--  l =  1.1 mm -->
<row>Pearl . . . abcdefghijklmnopqrstuvwxyz</row>
<--  l =  mm -->
<row>Agate . . . abcdefghijklmnopqrstuvwxyz</row>
<--  l =  mm -->
<row>Nonpareil . . . abcdefghijklmnopqrstuvwxyz</row>
<--  l =  mm -->
<row>Minion . . . abcdefghijklmnopqrstuvwxyz</row>
<--  l =  mm -->
<row>Brevier . . . abcdefghijklmnopqrstuvwxyz</row>
<--  l =  mm -->
<row>Bourgeois . . abcdefghijklmnopqrstuvwxyz</row>
<--  l =  mm -->
<row>Long primer . . . abcdefghijklmnopqrstuvwxyz</row>
<--  l =  mm -->
<row>Small pica . . abcdefghijklmnopqrstuvwxyz</row>
<-- small pica l = 2.5 mm -->
<row>Pica . . . . . abcdefghijklmnopqrstuvwxyz</row>
<--  l = 2.9 mm -->
<row>English . . . abcdefghijklmnopqrstuvwxyz</row>
<--  l = 3.3 mm -->
<row>Columbian . . . abcdefghijklmnopqrstuvwxyz</row>
<--  l = 3.6 mm -->
<row>Great primer . . . abcdefghijklmnopqrstuvwxyz</row>
<-- Great primer l = 4.0 mm -->
</table><br/
[<source>1913 Webster</source>]</p>

<p>   The foregoing account is conformed to the designations made use of by American type founders, but is substantially correct for England. <xex>Agate</xex>, however, is called <xex>ruby</xex>, in England, where, also, a size intermediate between nonpareil and minion is employed, called <xex>emerald</xex>.</note><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Point system of type bodies</b></col> <fld>(Type Founding)</fld>, <cd>a system adopted by the type founders of the United States by which the various sizes of type have been so modified and changed that each size bears an exact proportional relation to every other size. The system is a modification of a French system, and is based on the pica body. This pica body is divided into twelfths, which are termed <ldquo/points,<rdquo/ and every type body consist of a given number of these points. Many of the type founders indicate the new sizes of type by the number of points, and the old names are gradually being done away with. By the point system type founders cast type of a uniform size and height, whereas formerly fonts of pica or other type made by different founders would often vary slightly so that they could not be used together. There are no type in actual use corresponding to the smaller theoretical sizes of the point system. In some cases, as in that of <xex>ruby</xex>, the term used designates a different size from that heretofore so called.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><-- Here is inserted a two-column table, in which the sizes of the bodies of the various types are represented by a 2.5 cm long bar of the corresponding height; the two columns are separated by a thin vertical line. -->
<-- pica (12 pt.) = 4.3 mm tall -->
<table>
1  American  9  Bourgeois
<point1><bar/</point1> <point9><bar/</point9>
1<frac12/  German  
<point1.5><bar/</point1.5>
2  Saxon   10  Long Primer
<point2><bar/</point2> <point10><bar/</point10>
2<frac12/  Norse
<point2.5><bar/</point2.5>
3  Brilliant  11 Small Pica
<point3><bar/</point3> <point11><bar/</point11>
3<frac12/  Ruby   12  Pica
<point3.5><bar/</point3.5> <point12><bar/</point12>
4  Excelsior
<point4><bar/</point4>
4<frac12/  Diamond  14  English
<point4.5><bar/</point4.5> <point14><bar/</point14>
5  Pearl   16  Columbian
<point5><bar/</point5> <point16><bar/</point16>
5<frac12/  Agate
<point5.5><bar/</point5.5>
6  Nonpareil   18  Great Primer
<point6><bar/</point6> <point18><bar/</point18>
7  Minion
<point7><bar/</point7>
8  Brevier   20  Paragon
<point8><bar/</point8> <point20><bar/</point20>
<caption>Diagram of the "points" by which sizes of Type are graduated in the "Point System".</caption>
</table><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Type founder</b></col>, <cd>one who casts or manufacture type.</cd> -- <mcol><col><b>Type foundry</b></col>, <col><b>Type foundery</b></col></mcol>, <cd>a place for the manufacture of type.</cd> -- <col><b>Type metal</b></col>, <cd>an alloy used in making type, stereotype plates, etc., and in backing up electrotype plates. It consists essentially of lead and antimony, often with a little tin, nickel, or copper.</cd> -- <col><b>Type wheel</b></col>, <cd>a wheel having raised letters or characters on its periphery, and used in typewriters, printing telegraphs, etc.</cd> -- <col><b>Unity of type</b></col> <fld>(Biol.)</fld>, <cd>that fundamental agreement in structure which is seen in organic beings of the same class, and is quite independent of their habits of life.</cd>  <rj><au>Darwin.</au></rj></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Type</ent><br/
<hw>Type</hw> <pr>(?)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Typed</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Typing</conjf>.]</vmorph> <sn>1.</sn> <def>To represent by a type, model, or symbol beforehand; to prefigure.</def> <mark>[R.]</mark>  <rj><au>White (Johnson).</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To furnish an expression or copy of; to represent; to typify.</def> <mark>[R.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>Let us <qex>type</qex> them now in our own lives.</q> <rj><qau>Tennyson.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Typesetter</ent><br/
<hw>Type"set`ter</hw> <pr>(?)</pr>, <pos>n.</pos> <def>One who, or that which, sets type; a compositor; a machine for setting type.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Typesetting</ent><br/
<hw>Type"set`ting</hw>, <pos>n.</pos> <def>The act or art of setting type.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Typewrite</ent><br/
<hw>Type"write`</hw> <pr>(?)</pr>, <pos>v. t. & i.</pos> <def>To write with a typewriter.</def> <mark>[Recent]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Typewriter</ent><br/
<hw>Type"writ`er</hw> <pr>(?)</pr>, <pos>n.</pos> <sn>1.</sn> <def>An instrument for writing by means of type, a typewheel, or the like, in which the operator makes use of a sort of keyboard, in order to obtain printed impressions of the characters upon paper.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>One who uses such an instrument.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Typewriting</ent><br/
<hw>Type"writ`ing</hw>, <pos>n.</pos> <def>The act or art of using a typewriter; also, a print made with a typewriter.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Typhlitis</ent><br/
||<hw>Typh*li"tis</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[NL., fr. Gr. <?/ blind, closed (as applied to the caecum) + <ets>-itis</ets>.]</ety> <fld>(Med.)</fld> <def>Inflammation of the caecum.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Typhlosole</ent><br/
<hw>Typh"lo*sole</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Gr. <?/ blind + <?/ channel.]</ety> <fld>(Zool.)</fld> <def>A fold of the wall which projects into the cavity of the intestine in bivalve mollusks, certain annelids, starfishes, and some other animals.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Typhoean</ent><br/
<hw>Ty*pho"e*an</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>Typhoius</ets>, from <ets>Typhoeus</ets>, Gr. <?/, <?/.]</ety> <def>Of or pertaining to Typhoeus <pr>(t<isl/*f<omac/"<umac/s)</pr>, the fabled giant of Greek mythology, having a hundred heads; resembling Typhoeus.</def><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ Sometimes incorrectly written and pronounced <it>Ty-ph<oe/<bprime/an</it> <pr>(<?/)</pr>, or <it>Ty-phe<bprime/an.</it></note><br/
[<source>1913 Webster</source>]</p>

<p><ent>Typhoid</ent><br/
<hw>Ty"phoid</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[<ets>Typhus</ets> + <ets>-oid</ets>: cf. F. <ets>typho<ium/de</ets>, Gr. <?/. See <er>Typhus</er>.]</ety> <fld>(Med.)</fld> <def>Of or pertaining to typhus; resembling typhus; of a low grade like typhus; <as>as, <ex>typhoid</ex> symptoms</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Typhoid fever</b></col>, <cd>a disease formerly confounded with typhus, but essentially different from the latter. It is characterized by fever, lasting usually three or more weeks, diarrhaea with evacuations resembling pea soup in appearance, and prostration and muscular debility, gradually increasing and often becoming profound at the acme of the disease. Its local lesions are a scanty eruption of spots, resembling flea bites, on the belly, enlargement of the spleen, and ulceration of the intestines over the areas occupied by Peyer's glands. The virus, or contagion, of this fever is supposed to be a microscopic vegetable organism, or bacterium. Called also <altname>enteric fever</altname>. See <er>Peyer's glands</er>.</cd> <-- caused by the gram-negative bacillus, Salmonella typhosa (syn. Salmonella typhi), also called typhoid bacillus.  The organism is often ingested by drinking contaminated water. --> -- <col><b>Typhoid state</b></col>, <cd>a condition common to many diseases, characterized by profound prostration and other symptoms resembling those of typhus.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><-- p. 1559 pr=PI --><br/
[<source>1913 Webster</source>]</p>

<p><ent>Typhomalarial</ent><br/
<hw>Ty`pho*ma*la"ri*al</hw> <pr>(t<imac/`f<osl/*m<adot/*l<amac/*r<icr/*<ait/l)</pr>, <pos>a.</pos> <fld>(Med.)</fld> <def>Pertaining to typhoid fever and malaria; <as>as, <ex>typhomalarial</ex> fever, a form of fever having symptoms both of malarial and typhoid fever</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Typhomania</ent><br/
<hw>Ty`pho*ma"ni*a</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[NL. See <er>Typhus</er>, and <er>Mania</er>.]</ety> <fld>(Med.)</fld> <def>A low delirium common in typhus fever.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Typhon</ent><br/
<hw>Ty"phon</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Gr. <?/, and <?/. See <er>Typhoon</er>.]</ety> <fld>(Class. Mythol.)</fld> <sn>1.</sn> <def>According to Hesiod, the son of Typhoeus, and father of the winds, but later identified with him.</def>
<-- father of Cerberus, the Chimera, and the Sphinx --><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ By modern writers, <xex>Typhon</xex> is identified with the Egyptian Set, who represents physical evil.  <rj><au>Encyc. Brit.</au></rj>
</note><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>A violent whirlwind; a typhoon.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>The circling <qex>typhon</qex> whirled from point to point.</q> <rj><qau>Thomson.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Typhoon</ent><br/
<hw>Ty*phoon"</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Earlier <ets>tuffoon</ets>, <ets>tuffon</ets>, Pg. <ets>tuf<atil/o</ets>, Ar. <ets>tuf<amac/n</ets> a violent storm; probably fr. Gr. <grk>tyfw^n</grk>, <grk>tyfw^s</grk>, a violent whirlwind, that rushes upward from the earth, whirling clouds of dust (cf. <er>Typhus</er>); or perhaps from Chin. <ets>t'ai-fung</ets> a cyclonic wind.]</ety> <def>A violent whirlwind; specifically, a violent whirlwind occurring in the Chinese seas.</def>
<-- 2. a tropical cyclone of the Chinese seas. (2 senses confounded in W1913) --><br/
[<source>1913 Webster</source>]</p>

<p><ent>Typhos</ent><br/
||<hw>Ty"phos</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[NL.]</ety> <fld>(Med.)</fld> <def>Typhus.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Typhotoxin</ent><br/
<hw>Ty`pho*tox"in</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[<ets>Typh</ets>oid + <ets>-toxic</ets>.]</ety> <fld>(Physiol. Chem.)</fld> <def>A basic substance, <chform>C7H17NO2</chform>, formed from the growth of the typhoid bacillus on meat pulp. It induces in small animals lethargic conditions with liquid dejecta.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Typhous</ent><br/
<hw>Ty"phous</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[Cf. F. <ets>typheux</ets>.]</ety> <fld>(Med.)</fld> <def>Of or pertaining to typhus; of the nature of typhus.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Typhus</ent><br/
<hw>Ty"phus</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[NL., fr. Gr. <?/ smoke, cloud, stupor arising from fever; akin to <?/ to smoke, Skr. <ets>dh<?/pa</ets> smoke.]</ety> <fld>(Med.)</fld> <def>A contagious continued fever lasting from two to three weeks, attended with great prostration and cerebral disorder, and marked by a copious eruption of red spots upon the body. Also called <altname>jail fever</altname>, <altname>famine fever</altname>, <altname>putrid fever</altname>, <altname>spottled fever</altname>, etc. See <cref>Jail fever</cref>, under <er>Jail</er>.</def>
<-- caused by various species of Rickettsia --><br/
[<source>1913 Webster</source>]</p>

<p><ent>Typic</ent><br/
<hw>Typ"ic</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Typical.</def> <ldquo/<xex>Typic</xex> shades.<rdquo/  <rj><au>Boyle.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Typical</ent><br/
<hw>Typ"ic*al</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>typicus</ets>, Gr. <?/, from <?/ type: cf. F. <ets>typique</ets>. See <er>Type</er>.]</ety> <sn>1.</sn> <def>Of the nature of a type; representing something by a form, model, or resemblance; emblematic; prefigurative.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The Levitical priesthood was only <qex>typical</qex> of the Christian.</q> <rj><qau>Atterbury.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Nat. Hist.)</fld> <def>Combining or exhibiting the essential characteristics of a group; <as>as, a <ex>typical</ex> genus</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p>-- <wordforms><wf>Typ"ic*al*ly</wf>, <pos>adv.</pos> -- <wf>Typ"ic*al*ness</wf>, <pos>n.</pos></wordforms><br/
[<source>1913 Webster</source>]</p>

<p><ent>Typification</ent><br/
<hw>Typ`i*fi*ca"tion</hw> <pr>(?)</pr>, <pos>n.</pos> <def>The act of typifying, or representing by a figure.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Typifier</ent><br/
<hw>Typ"i*fi`er</hw> <pr>(?)</pr>, <pos>n.</pos> <def>One who, or that which, typifies.</def>  <rj><au>Bp. Warburton.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Typify</ent><br/
<hw>Typ"i*fy</hw> <pr>(?)</pr>, <pos>v. t.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Typified</conjf> <pr>(?)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Typifying</conjf>.]</vmorph> <ety>[<ets>Type</ets> + <ets>-fy</ets>.]</ety> <sn>1.</sn> <def>To represent by an image, form, model, or resemblance.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>Our Savior was <qex>typified</qex>, indeed, by the goat that was slain, and the scapegoat in the wilderness.</q> <rj><qau>Sir T. Browne.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>To embody the essential or salient characteristics of; to be the type of; <as>as, the genus <gen>Rosa</gen> <ex>typifies</ex> the family <fam>Rosaceae</fam>, which in turn <ex>typifies</ex> the series <ord>Rosales</ord></as>.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Typist</ent><br/
<hw>Typ"ist</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A person who operates a typewriting machine; a typewriter.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Typo</ent><br/
<hw>Ty"po</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[An abbreviation of <ets>typographer</ets>.]</ety> <def>A compositor.</def> <mark>[Colloq.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Typo</ent><br/
<hw>Ty"po</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[An abbreviation of <ets>typo</ets>graphical error.]</ety> <def>a typographical error; an error in typing, printing, etc.</def> <mark>[Colloq.]</mark><br/
[<source>PJC</source>]</p>

<p><ent>Typocosmy</ent><br/
<hw>Typ"o*cos`my</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Gr. <grk>ty`pos</grk> type + <grk>ko`smos</grk> the world.]</ety> <def>A representation of the world.</def> <mark>[R.]</mark> <rj><qau>Camden.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Typograph</ent><br/
<hw>Ty"po*graph</hw> <pr>(?)</pr>, <pos>n.</pos>  <ety>[<ets>Type</ets> + <ets>-graph</ets>.]</ety> <def>A machine for setting type or for casting lines of type and setting them.</def><br/
[<source>Webster 1913 Suppl.</source>]</p>

<p><ent>Typographer</ent><br/
<hw>Ty*pog"ra*pher</hw> <pr>(?; 277)</pr>, <pos>n.</pos> <ety>[Cf. F. <ets>typographe</ets>. See <er>Typography</er>.]</ety> <def>A printer.</def>  <rj><au>T. Warton.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Typographical</ent><br/
<ent>Typographic</ent><br/
<mhw>{ <hw>Ty`po*graph"ic</hw> <pr>(?; 277)</pr>, <hw>Ty`po*graph"ic*al</hw> <pr>(?; 277)</pr>, }</mhw> <pos>a.</pos> <ety>[Cf. F. <ets>typographique</ets>.]</ety> <sn>1.</sn> <def>Of or pertaining to the act or art of representing by types or symbols; emblematic; figurative; typical.</def> <mark>[Obs.]</mark>  <rj><au>Johnson.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Of or pertaining to typography or printing; <as>as, the <ex>typographic</ex> art</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p>-- <wordforms><wf>Ty`po*graph"ic*al*ly</wf>, <pos>adv.</pos></wordforms><br/
[<source>1913 Webster</source>]</p>

<p><ent>Typography</ent><br/
<hw>Ty*pog"ra*phy</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[<ets>Type</ets> + <ets>-graphy</ets>: cf. F. <ets>typographie</ets>.]</ety> <sn>1.</sn> <def>The act or art of expressing by means of types or symbols; emblematical or hieroglyphic representation.</def> <mark>[Obs.]</mark>  <rj><au>Sir T. Browne.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>The art of printing with types; the use of types to produce impressions on paper, vellum, etc.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Typolite</ent><br/
<hw>Typ"o*lite</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Gr. <?/ image, type + <ets>-lite</ets>: cf. F. <ets>typolithe</ets>.]</ety> <fld>(Min.)</fld> <def>A stone or fossil which has on it impressions or figures of plants and animals.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Typolithography</ent><br/
<hw>Ty`po*li*thog"ra*phy</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Gr. <?/ type + <ets>lithography</ets>.]</ety> <def>A branch of lithography in which impressions from printers' types are transferred to stone for reproduction.</def> -- <wordforms><wf>Ty`po*lith`o*graph"ic</wf> <pr>(#)</pr>, <pos>a.</pos></wordforms></p>

<p><ent>Typology</ent><br/
<hw>Ty*pol"o*gy</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[<ets>Type</ets> + <ets>-logy</ets>.]</ety> <sn>1.</sn> <fld>(Theol.)</fld> <def>A discourse or treatise on types.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <fld>(Theol.)</fld> <def>The doctrine of types.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Typothetae</ent><br/
||<hw>Ty*poth"e*tae</hw> <pr>(?)</pr>, <pos>n. pl.</pos> <ety>[NL., fr. E. <ets>type</ets> + Gr. <?/ to put, set.]</ety> <def>Printers; -- used in the name of an association of the master printers of the United States and Canada, called The United <xex>Typothetae</xex> of America.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tyran</ent><br/
<hw>Ty"ran</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[See <er>Tyrant</er>.]</ety> <def>A tyrant.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><q>Lordly love is such a <qex>tyran</qex> fell.</q> <rj><qau>Spenser.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tyranness</ent><br/
<hw>Ty"ran*ness</hw> <pr>(?)</pr>, <pos>n.</pos> <def>A female tyrant.</def> <mark>[Obs.]</mark> <ldquo/That proud <xex>tyranness</xex>.<rdquo/  <rj><au>Spenser.</au>  <au>Akenside.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tyrannical</ent><br/
<ent>Tyrannic</ent><br/
<mhw>{ <hw>Ty*ran"nic</hw> <pr>(?; 277)</pr>, <hw>Ty*ran"nic*al</hw> <pr>(?)</pr>, }</mhw> <pos>a.</pos> <ety>[L. <ets>tyrannicus</ets>, Gr. <?/: cf. F. <ets>tyrannique</ets>. See <er>Tyrant</er>.]</ety> <def>Of or pertaining to a tyrant; suiting a tyrant; unjustly severe in government; absolute; imperious; despotic; cruel; arbitrary; <as>as, a <ex>tyrannical</ex> prince; a <ex>tyrannical</ex> master; <ex>tyrannical</ex> government.</as></def> <ldquo/A power <xex>tyrannical</xex>.<rdquo/  <rj><au>Shak.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Our sects a more <qex>tyrannic</qex> power assume.</q> <rj><qau>Roscommon.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>The oppressor ruled <qex>tyrannic</qex> where he durst.</q> <rj><qau>Pope.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p>-- <wordforms><wf>Ty*ran"nic*al*ly</wf>, <pos>adv.</pos> -- <wf>Ty*ran"nic*al*ness</wf>, .</wordforms><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tyrannicidal</ent><br/
<hw>Ty*ran"ni*ci`dal</hw> <pr>(?)</pr>, <pos>a.</pos> <def>Of or pertaining to tyrannicide, or the murder of a tyrant.</def>  <rj><au>Booth.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tyrannicide</ent><br/
<hw>Ty*ran"ni*ci`de</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>tyrannicidium</ets> the killing of a tyrant, <ets>tyrannicida</ets> the killer of a tyrant; <ets>tyrannus</ets> a tyrant + <ets>caedere</ets> to kill: cf. F. <ets>tyrannicide</ets>.]</ety> <sn>1.</sn> <def>The act of killing a tyrant.</def>  <rj><au>Hume.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>One who kills a tyrant.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tyrannish</ent><br/
<hw>Tyr"an*nish</hw> <pr>(t<icr/r"<ait/n*n<icr/sh)</pr>, <pos>a.</pos> <def>Like a tyrant; tyrannical.</def> <mark>[Obs.]</mark> <ldquo/The proud <xex>tyrannish</xex> Roman.<rdquo/  <rj><au>Gower.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tyrannize</ent><br/
<hw>Tyr"an*nize</hw> <pr>(t<icr/r"<ait/n*n<imac/z)</pr>, <pos>v. i.</pos> <vmorph>[<pos>imp. & p. p.</pos> <conjf>Tyrannized</conjf> <pr>(t<icr/r"<ait/n*n<imac/zd)</pr>; <pos>p. pr. & vb. n.</pos> <conjf>Tyrannizing</conjf> <pr>(t<icr/r"<ait/n*n<imac/`z<icr/ng)</pr>.]</vmorph> <ety>[Cf. F. <ets>tyranniser</ets>, Gr. <grk>tyranni`zein</grk> to take the part of tyrants.]</ety> <def>To act the tyrant; to exercise arbitrary power; to rule with unjust and oppressive severity; to exercise power over others not permitted by law or required by justice, or with a severity not necessary to the ends of justice and government; <as>as, a prince will often <ex>tyrannize</ex> over his subjects; masters sometimes <ex>tyrannize</ex> over their servants or apprentices</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tyrannize</ent><br/
<hw>Tyr"an*nize</hw>, <pos>v. t.</pos> <def>To subject to arbitrary, oppressive, or tyrannical treatment; to oppress.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tyrannous</ent><br/
<hw>Tyr"an*nous</hw> <pr>(t<icr/r"<ait/n*n<ucr/s)</pr>, <pos>a.</pos> <def>Tyrannical; arbitrary; unjustly severe; despotic.</def> <au>Sir P. Sidney.</au> -- <wordforms><wf>Tyr"an*nous*ly</wf>, <pos>adv.</pos></wordforms><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tyranny</ent><br/
<hw>Tyr"an*ny</hw> <pr>(t<icr/r"<ait/n*n<ycr/)</pr>, <pos>n.</pos> <ety>[OE. <ets>tirannye</ets>, OF. <ets>tirannie</ets>, F. <ets>tyrannie</ets>; cf. It. <ets>tirannia</ets>; Gr. <grk>tyranni`a</grk>, <grk>tyranni`s</grk>, L. <ets>tyrannis</ets>. See <er>Tyrant</er>.]</ety> <sn>1.</sn> <def>The government or authority of a tyrant; a country governed by an absolute ruler; hence, arbitrary or despotic exercise of power; exercise of power over subjects and others with a rigor not authorized by law or justice, or not requisite for the purposes of government.</def><br/
[<source>1913 Webster</source>]</p>

<p><q><ldquo/Sir,<rdquo/ would he [Seneca] say, <ldquo/an emperor mote need<br/
Be virtuous and hate <qex>tyranny</qex>.<rdquo/</q> <rj><qau>Chaucer.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Cruel government or discipline; <as>as, the <ex>tyranny</ex> of a schoolmaster</as>.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <def>Severity; rigor; inclemency.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The <qex>tyranny</qex> of the open night's too rough<br/
For nature to endure.</q> <rj><qau>Shak.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tyrant</ent><br/
<hw>Tyr"ant</hw> <pr>(t<imac/"r<ait/nt)</pr>, <pos>n.</pos> <ety>[OE. <ets>tirant</ets>, <ets>tiraunt</ets>, <ets>tyraunt</ets>, OF. <ets>tiran</ets>, <ets>tirant</ets> (probably from confusion with the p. pr. of verbs), F. <ets>tyran</ets>, L. <ets>tyrannus</ets>, Gr. <grk>ty`rannos</grk>, originally, an absolute sovereign, but afterwards, a severe or cruel ruler.]</ety> <sn>1.</sn> <def>An absolute ruler; a sovereign unrestrained by law or constitution; a usurper of sovereignty.</def><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ Free governments [in Greece] having superseded the old hereditary sovereignties (<grk>basilei^ai</grk>), all who obtained absolute power in a state were called <grk>ty`rannoi</grk>, <xex>tyrants</xex>, or rather <xex>despots</xex>; -- for the term rather regards the irregular way in which the power was <xex>gained</xex>, whether force or fraud, than the way in which it was <xex>exercised</xex>, being applied to the mild Pisistratus, but not to the despotic kings of Persia. However, the word soon came to imply reproach, and was then used like our <xex>tyrant</xex>.</note>  <rj><au>Liddell & Scott.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Specifically, a monarch, or other ruler or master, who uses power to oppress his subjects; a person who exercises unlawful authority, or lawful authority in an unlawful manner; one who by taxation, injustice, or cruel punishment, or the demand of unreasonable services, imposes burdens and hardships on those under his control, which law and humanity do not authorize, or which the purposes of government do not require; a cruel master; an oppressor.</def> <ldquo/This false <xex>tyrant</xex>, this Nero.<rdquo/  <rj><au>Chaucer.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><q>Love, to a yielding heart, is a king, but to a resisting, is a <qex>tyrant</qex>.</q> <rj><qau>Sir P. Sidney.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><sn>3.</sn> <fld>(Zool.)</fld> <def>Any one of numerous species of American clamatorial birds belonging to the family <fam>Tyrannidae</fam>; -- called also <altname>tyrant bird</altname>.</def><br/
[<source>1913 Webster</source>]</p>

<p><note><hand/ These birds are noted for their irritability and pugnacity, and for the courage with which they attack rapacious birds far exceeding them in size and strength. They are mostly plain-colored birds, but often have a bright-colored crown patch. A few species, as the scissorstail, are handsomely colored. The kingbird and pewee are familiar examples.</note><br/
[<source>1913 Webster</source>]</p>

<p><cs><col><b>Tyrant flycatcher</b></col> <fld>(Zool.)</fld>, <cd>any one of numerous species of tyrants which have a flattened bill, toothed at the tip, and resemble the true flycatchers in habits. The Acadian flycatcher (<spn>Empidonax Acadicus</spn>) and the vermilion flycatcher (<spn>Pyrocephalus rubineus</spn>) are examples.</cd> -- <col><b>Tyrant shrike</b></col> <fld>(Zool.)</fld>, <cd>any one of numerous species of American tyrants of the genus <gen>Tyrannus</gen> having a strong toothed bill and resembling the shrikes in habits.  The kingbird is an example.</cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tyrant</ent><br/
<hw>Ty"rant</hw> <pr>(?)</pr>, <pos>v. i.</pos> <def>To act like a tyrant; to play the tyrant; be to tyrannical.</def> <mark>[Obs.]</mark>  <rj><au>Fuller.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tyre</ent><br/
<hw>Tyre</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Tamil <ets>tayir</ets>.]</ety> <def>Curdled milk.</def> <mark>[India]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>tyre</ent><br/
<hw>tyre</hw> <pr>(?)</pr>, <pos>n.</pos> <def>Same as 2nd <er>tire</er>, <pos>n.</pos>, sense 5.</def> <mark>[British spelling]</mark> <br/
[<source>PJC</source>]</p>

<p><ent>Tyre</ent><br/
<hw>Tyre</hw>, <pos>n. & v.</pos> <def>Attire. See 2d and 3d <er>Tire</er>.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tyre</ent><br/
<hw>Tyre</hw>, <pos>v. i.</pos> <def>To prey upon. See 4th <er>Tire</er>.</def> <mark>[Obs.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tyrian</ent><br/
<hw>Tyr"i*an</hw> <pr>(?)</pr>, <pos>a.</pos> <ety>[L. <ets>Tyrius</ets>, from <ets>Tyrus</ets> Tyre, Gr. <?/.]</ety> <sn>1.</sn> <def>Of or pertaining to Tyre or its people.</def><br/
[<source>1913 Webster</source>]</p>

<p><sn>2.</sn> <def>Being of the color called Tyrian purple.</def><br/
[<source>1913 Webster</source>]</p>

<p><q>The bright-eyed perch with fins of <qex>Tyrian</qex> dye.</q> <rj><qau>Pope.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><cs><mcol><col><b>Tyrian purple</b></col>, <it>or</it> <col><b>Tyrian dye</b></col></mcol>, <cd>a celebrated purple dye prepared in ancient <geog>Tyre</geog> from several mollusks, especially Ianthina, Murex, and Purpura. See the Note under <er>Purple</er>, <pos>n.</pos>, 1, and <cref>Purple of mollusca</cref>, under <er>Purple</er>, <pos>n.</pos></cd></cs><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tyrian</ent><br/
<hw>Tyr"i*an</hw>, <pos>n.</pos> <ety>[L. <ets>Tyrius</ets>.]</ety> <def>A native of Tyre.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tyro</ent><br/
<hw>Ty"ro</hw> <pr>(?)</pr>, <pos>n.</pos>; <plu><it>pl.</it> <plw>Tyros</plw> <pr>(#)</pr>.</plu> <ety>[L. <ets>tiro</ets> a newlylevied soldier, a beginner.]</ety> <def>A beginner in learning; one who is in the rudiments of any branch of study; a person imperfectly acquainted with a subject; a novice.</def> <altsp>[Written also <asp>tiro</asp>.]</altsp><br/
[<source>1913 Webster</source>]</p>

<p><q>The management of <qex>tyros</qex> of eighteen<br/
Is difficult.</q> <rj><qau>Cowper.</qau></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tyrociny</ent><br/
<hw>Ty"ro*cin*y</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[L. <ets>tirocinium</ets> first service or trial, fr. <ets>tiro</ets>. See <er>Tyro</er>.]</ety> <def>The state of being a tyro, or beginner; apprenticeship.</def> <mark>[Obs.]</mark>  <rj><au>Blount.</au></rj><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tyrolite</ent><br/
<hw>Tyr"o*lite</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[From <ets>Tyrol</ets>, where it occurs.]</ety> <fld>(Min.)</fld> <def>A translucent mineral of a green color and pearly or vitreous luster. It is a hydrous arseniate of copper.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tyronism</ent><br/
<hw>Ty"ro*nism</hw> <pr>(?)</pr>, <pos>n.</pos> <def>The state of being a tyro, or beginner.</def> <altsp>[Written also <asp>tironism</asp>.]</altsp><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tyrosin</ent><br/
<hw>Tyr"o*sin</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Gr. <?/ cheese.]</ety> <fld>(Physiol. Chem.)</fld> <def>A white crystalline nitrogenous substance<--amino acid--> present in small amount in the pancreas and spleen, and formed in large quantity from the decomposition of proteid matter by various means, -- as by pancreatic digestion, by putrefaction as of cheese, by the action of boiling acids, etc. Chemically, it consists of oxyphenol and amidopropionic acid, and by decomposition yields oxybenzoic acid, or some other benzol derivative.</def> <altsp>[Written also <asp>tyrosine</asp>.]</altsp><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tyrotoxicon</ent><br/
<hw>Tyr`o*tox"i*con</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[NL., fr. Gr. <?/ cheese + <?/. See <er>Toxic</er>.]</ety> <fld>(Physiol. Chem.)</fld> <def>A ptomaine discovered by Vaughan in putrid cheese and other dairy products, and producing symptoms similar to cholera infantum. Chemically, it appears to be related to, or identical with, diazobenzol.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tyrotoxine</ent><br/
<hw>Tyr`o*tox"ine</hw> <pr>(?)</pr>, <pos>n.</pos> <def>Same as <er>Tyrotoxicon</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tysonite</ent><br/
<hw>Ty"son*ite</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[After S. T. <ets>Tyson</ets>.]</ety> <fld>(Min.)</fld> <def>A fluoride of the cerium metals occurring in hexagonal crystals of a pale yellow color.   Cf. <er>Fluocerite</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tystie</ent><br/
<hw>Ty"stie</hw> <pr>(?)</pr>, <pos>n.</pos> <ety>[Cf. Icel. <ets><thorn/eisti</ets>, Norw. <ets>teiste</ets>.]</ety> <fld>(Zool.)</fld> <def>The black guillemot.</def> <mark>[Prov. Eng.]</mark><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tythe</ent><br/
<hw>Tythe</hw> <pr>(?)</pr>, <pos>n.</pos> <def>See <er>Tithe</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tything</ent><br/
<hw>Tyth"ing</hw> <pr>(?)</pr>, <pos>n.</pos> <def>See <er>Tithing</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tzar</ent><br/
<hw>Tzar</hw> <pr>(?)</pr>, <pos>n.</pos> <def>The emperor of Russia.  See <er>Czar</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tzaritza</ent><br/
<ent>Tzarina</ent><br/
<mhw>{ <hw>Tza*ri"na</hw> <pr>(?)</pr>, <hw>Tza*rit"za</hw> <pr>(?)</pr>, }</mhw> <pos>n.</pos> <def>The empress of Russia.  See <er>Czarina</er>.</def><br/
[<source>1913 Webster</source>]</p>

<p><ent>Tzetze</ent><br/
<hw>Tzet"ze</hw> <pr>(?)</pr>, <pos>n.</pos> <fld>(Zool.)</fld> <def>Same as <er>Tsetse</er>.</def><br/
[<source>1913 Webster</source>]</p>

